//*****************************************************************************
//*
//*
//*		Bz2File.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf BZIP-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *Type=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *Type=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractMem(int iNum,char *pMem,int)		// Datei in Speicher extrahieren
// 		int		ExtractAll(         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(   int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *Datei,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *Datei)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int		Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *Datei);					// Datei suchen
// 		time_t	Time (char  *Datei);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen

#include	<stdio.h>
#include	<string.h>
#include	<sys/stat.h>
#include	"Bz2File.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		wcslen
#define 	strzcmp		wcscmp
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#define 	strzncat	wcsncat
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		strlen
#define 	strzcmp		strcmp
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#define 	strzncat	strncat
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif




#define PROG(iMode,pEntry)								\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))			\
	{													\
	if(Progress(iMode,pEntry))throw CAF_ERROR_BREAK;	\
	}
#define PROGR(iMode,pEntry,iRet)						\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))			\
	{													\
	if(Progress(iMode,pEntry))return iRet;				\
	}
#define PROGN(iMode,pEntry)								\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))			\
	{													\
	Progress(iMode,pEntry);								\
	}



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CBz2File::CBz2File()
{

	m_iDoneMode		= 0;
	m_cIsCpio		= 0;

}

//*****************************************************************************
//*
//*		Denstruktor
//*
//*****************************************************************************
CBz2File::~CBz2File()
{

	if(m_iOpenMode!=CAF_CLOSED)								// Ist die TAR-Datei noch offen
		{
		Close();
		}

	if(m_cTarName[0])										// Lsche temorre TAR-Datei
		{
		DeleteFile(m_cTarName);
		m_cTarName[0]=0;
		}

	if(m_hSubFile!=INVALID_HANDLE_VALUE)
		{
		CloseFile(m_hSubFile);
		m_hSubFile = INVALID_HANDLE_VALUE;
		}
}




//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine BZIP2-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CBz2File::ArchOpen(int iLevel)
{
aint64		iByte;
aint64		iSize;
int			iRet;
int			iLen;
int			iMin;
int			iMax;



	SetDoneMode(-1);										// Done-Modus in Tar-Datei

	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;
	m_cIsCpio	= 0;


	iByte		= m_iFileSize+16;


	Clear();
	InitSubFile();


	if(iByte<0x400000)										// Bis 4MB im Speicher ffnen
		{
		iMin	= (int)iByte;
		iMax	= 0x1000000;								// 10MB maximal
		}
	else{
		iMin	= 0;
		iMax	= 0;
		}


	SetMemSub(0x10000,0x30000,m_cTarName,Z(".tar"));		// Dynamischen Speicher erstellen
		

	if(m_iSubMode==CAF_CREATE)								// Neues Archiv erzeugen 
		{
		m_iOpenMode = CAF_CREATE;
		return CTarFile::ArchOpen(iLevel);
		}


	m_iOpenMode = CAF_WRITE;


//******************** Archiv dekomprimieren **********************************


	try	{
		if(m_iSubSize<16)throw CAF_ERROR_FORM;

		m_iOpenMode=CAF_WRITE;
		SetInput(&m_sBzip,CAF_BASE);						// Die BZ2-Datei als Eingabe benutzen

		m_sBzip.SetOut(this,WriteToTar);					// Ausgabefunktion fr TAR-Datei
		m_iFileSize		= 0;
		m_iTarOffset	= 0;

		PROG(CAF_PROG_DOEXTRACT,-1);
		m_sBzip.Reset();
	
		m_uTarSize	= (int)m_iSubSize*4;
		m_iDoneLen	= m_iSubSize;
		m_iDoneMode = 1;
		SetDoneMode(2);

		try	{												// Endkomprmire die TAR-Datei
			m_cThrowWr = 0;
			m_sBzip.Decompress();
			m_cThrowWr = 1;
			}
		catch(char*)
			{
			m_cThrowWr = 1;
			throw CAF_ERROR_DECOMP;
			}

		iSize = m_sBzip.Value64(CBZ_LEN);
		
		   iLen = (unsigned)iSize%512;						// Aligment prfen
		if(iLen>0)
			{
			char cBuffer[512];
			iLen = 512-iLen;
			memset(cBuffer,0,iLen);
			MainWrite(cBuffer,iLen);
			}

		if(m_iTarOffset==TAR_DUMMY_HEADER)					// TAR-Header einfgen bei einzelner Datei
			{
			WriteDummy(iSize);
			}

		m_iFileSize	= MainSize();
		m_iDoneSave = m_iDoneLen+m_iDoneNow;
		m_iDoneLen	= m_iDoneLen*6-m_iDoneSave;
		m_iDoneNow	= 0;	
		m_iDoneMax	= 0;
		m_iDoneMode	= 2;
		SetDoneMode(-1);


		if(m_iSubMode==CAF_READ)							// Lesemodus fr TAR-Datei
			{
			if(DoChange(CAF_READ,0)!=CAF_ERROR_OK)
				{
				return m_iErrorCode;
				}
			}
		else{
			MainSeek(0);
			}

		iRet = CTarFile::ArchOpen(iLevel);					// Die Dateien aus der TAR-Datei einlesen

		if(iRet!=CAF_ERROR_OK)								// Gab es Fehler beim ffnen
			{
			if(m_hFile!=INVALID_HANDLE_VALUE)
				{
				CloseFile(m_hFile);
				m_hFile = INVALID_HANDLE_VALUE;
				}

			DeleteFile(m_cTarName);

			return iRet;
			}
															// Skalierung fr komprimierte Gre	
		SetScale(m_sBzip.Value64(CBC_LEN),m_sBzip.Value64(CBC_SIZE));
		}
	catch(int iError)										// Gab es Fehler
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		Break();
		}


	m_iDoneNow	= Done(&m_iDoneMax);						// Aktuellen Verlauf in TAR-Datei bernehmen
	m_iDoneMode	= 0;
	SetDoneMode(-1);										
	
	

return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das BZ2-Archiv und Speichert alle Daten
//	Ergibt 0 wenn ok, oder einen Fehlercode ungleich 0:
//			CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CBz2File::Close()
{
int			iWrite;
int			iRet;




	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;


	if(m_iOpenMode==CAF_READ || !m_iChanged)
			iWrite=0;
	else	iWrite=1;


	if(!iWrite)												// BZ2-Datei nur schlieen
		{
		if(m_hSubFile!=INVALID_HANDLE_VALUE)
			{
			CloseFile(m_hSubFile);
			m_hSubFile=INVALID_HANDLE_VALUE;
			}
		}
	else{													// Alles in der TAR-Datei speichern
		m_iDoneLen	= AllLen   ();
		m_iDoneSave	= FlushCalc();
		m_iDoneMode	= 3;

		   m_iErrorCode =Flush();
		if(m_iErrorCode!=CAF_ERROR_OK)						
			{
			Break();
			return m_iErrorCode;
			}

		m_iDoneMode	= 4;

		   m_iErrorCode = CloseFlush();						// Die TAR-Daten in die BZ2-Datei schreiben
		if(m_iErrorCode!=CAF_ERROR_OK)
			{
			Break();
			return m_iErrorCode;
			}

		m_iDoneNow	= Done(&m_iDoneMax);					// Aktuellen Verlauf in TAR-Datei bernehmen
		m_iDoneMode	= 0;
		SetDoneMode(-1);										
		}


	   iRet =CTarFile::Close();
	if(iRet!=CAF_ERROR_OK)									// Alles in der TAR-Datei speichern
		{
		m_iErrorCode = iRet;
		}


	if(m_cTarName[0])										// Lsche temorre TAR-Datei
		{
		DeleteFile(m_cTarName);
		m_cTarName[0]=0;
		}

	ReleaseData();



return m_iErrorCode;
}

//*****************************************************************************
//*
//*		CloseFlush
//*
//*****************************************************************************
//	Beim Schlieen der BZ2-Datei die Daten aus der TAR-Datei in die BZ2-Datei komprimieren.
int CBz2File::CloseFlush()
{
DWORD		aSuffix[2];


	try	{
		MainSeek(0);										// Lesezeiger in TAR-Datei an den Anfang
		ZeroFile(1);										// Die BZ2-Datei auf 0 krzen

		try	{
			PROG(CAF_PROG_COMPRESS ,-1);
			SetInput (&m_sBzip,CAF_MAIN,-1,-1);				// Die TAR-Datei als Eingabe benutzen
			SetOutput(&m_sBzip,CAF_BASE      );				// Die BZ2-Datei als Ausgabe benutzen
			m_sBzip.Compress(m_iLevel);						// Die TAR-Datei in BZ2-Datei komprimieren
			}
		catch(char*)
			{
			throw CAF_ERROR_WRITE;
			}

		aSuffix[0] =		m_sBzip.GetCrcInp();			// CRC   der TAR-Daten 
		aSuffix[1] = (DWORD)m_sBzip.GetLength();			// Gre der TAR-Daten

		BaseWrite(aSuffix,8);								// Die Suffix-Bytes speichern 


															// Skalierung fr komprimierte Gre	
		SetScale(m_sBzip.Value64(CBC_LEN),m_sBzip.Value64(CBC_SIZE));

		PROG(CAF_PROG_CLOSE,-1);

		m_iErrorCode = CAF_ERROR_OK;
		}
	catch(int iError)
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		return m_iErrorCode;
		}


	if(m_hSubFile!=INVALID_HANDLE_VALUE)					// Die BZ2-Datei schlieen
		{
		CloseFile(m_hSubFile);
		m_hSubFile=INVALID_HANDLE_VALUE;
		}



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush,Change ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt die Position des aktuellen Verlaufs.
aint64	CBz2File::Done(aint64 *pMax)
{
aint64		iNow;
aint64		iMax;
aint64		iOut;
aint64		iAll;


	switch(m_iDoneMode)
		{
	case 1:													// Entpacke das BZ2-Archiv in die TAR-Datei

			iNow	= m_sBzip.Value64(CBZ_INP);				// Bytes aus BZ2-Datei gelesen
			iOut	= CTarFile::Done();						// Bytes in  TAR-Datei entpackt

			if(iOut>m_iDoneLen*4)						
				{
				iOut = m_iDoneLen*4;
				}
			
			if(pMax)*pMax=m_iDoneLen*6;

			return iNow+iOut;


	case 2:	iNow	= m_sBzip.Value64(CBZ_OUT);				// ffne TAR-Datei nach dem Entpacken

			if(m_uTarSize>0)
				{
				iNow *= m_iDoneLen;							// Den TAR-Verlauf skallieren
				iNow /= m_uTarSize;
				}

			if(pMax)*pMax=m_iDoneSave+m_iDoneLen;
			
			return m_iDoneSave+iNow;

	case 3:													// Speichere alles im TAR-Archiv 

			iNow	= CTarFile::Done(&iMax);				// ffne TAR-Datei nach dem Entpacken
			iAll	= m_iDoneSave;
			iAll   += m_iDoneLen;
			iAll   += m_iDoneLen/4;

			if(iNow>m_iDoneSave)							// Den TAR-Verlauf prfen
				{
				iNow = m_iDoneSave;
				}
	
			if(pMax)*pMax=iAll;
			
			return iNow;


	case 4:													// Komprimiere TAR-Archiv in die BZ2-Datei

			iNow	= m_sBzip.Value64(CBZ_INP);				// Bytes aus TAR-Datei geladen
			iOut	= m_sBzip.Value64(CBZ_OUT);				// Bytes in  BZ2-Datei komprimiert

			if(iNow>m_iDoneLen)
				{
				iNow = m_iDoneLen;
				}

			if(iOut>m_iDoneLen/4)
				{
				iOut = m_iDoneLen/4;
				}

			iAll	= m_iDoneSave;
			iAll   += m_iDoneLen;
			iAll   += m_iDoneLen/4;

			if(pMax)*pMax=iAll;
			
			return m_iDoneSave+iNow+iOut;
		}


return CTarFile::Done(pMax);
}


//*****************************************************************************
//*
//*		Change
//*
//*****************************************************************************
//	Schreiberlaubnis aktivieren oder deaktivieren.
//	iMode		: Ist der neue Modus (CAF_WRITE oder CAF_READ)
//	Ergibt CAF_ERROR_OK wenn erfolgreich, oder eine Fehlernummer ungleich 0.
int CBz2File::Change(int iMode)
{
int		iError;


	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;



	if(m_iSubMode==iMode)									// Keine nderung	
		{
		return CAF_ERROR_OK;
		}


	if(iMode==CAF_READ && m_iChanged)						// TAR-Date in BZ2-Datei speichern
		{
		m_iDoneLen	= AllLen   ();
		m_iDoneSave	= FlushCalc();
		m_iDoneMode	= 3;

		   m_iErrorCode =Flush();							// Alles in der TAR-Datei speichern
		if(m_iErrorCode!=CAF_ERROR_OK)					
			{
			return m_iErrorCode;
			}

		m_iDoneMode	= 4;

		   m_iErrorCode = CloseFlush();						// Die BZ2-Datei anlegen
		if(m_iErrorCode!=CAF_ERROR_OK)
			{
			return m_iErrorCode;
			}

		m_iDoneNow	= Done(&m_iDoneMax);					// Aktuellen Verlauf in TAR-Datei bernehmen
		m_iDoneMode	= 0;
		SetDoneMode(-1);										
		}



	   iError = CTarFile::Change(iMode);					// Die TAR-Datei umstellen
	if(iError)											
		{
		Close();
		m_iErrorCode = iError;
		return         iError;
		}

	   iError = DoChangeSub(iMode,0);						// Die Unterdatei umstellen		
	if(iError!=CAF_ERROR_OK)							
		{
		return iError;
		}



return iError;
}

