//*****************************************************************************
//*
//*
//*		CabFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf BZIP-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *Type=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *Type=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractMem(int iNum,char *pMem,int)		// Datei in Speicher extrahieren
// 		int		ExtractAll(         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(   int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *Datei,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *Datei)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int		Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *Datei);					// Datei suchen
// 		time_t	Time (char  *Datei);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen

#include	<stdio.h>
#include	<string.h>
#include	<sys/stat.h>
#include	"CabFile.h"
#include	"Codecs/LzxClass.h"
#include	"Codecs/LzxDecompress.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		wcslen
#define 	strzcmp		wcscmp
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#define 	strzncat	wcsncat
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		strlen
#define 	strzcmp		strcmp
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#define 	strzncat	strncat
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif



#define 	LF_OLDNORMAL   	 0				// Normal disk file, Unix compatible
#define 	LF_NORMAL      	'0'        		// Normal disk file
#define 	LF_LINK        	'1'        		// Link to previously dumped file
#define 	LF_SYMLINK     	'2'        		// Symbolic link
#define 	LF_CHR         	'3'        		// Character special file
#define 	LF_BLK         	'4'        		// Block special file
#define 	LF_DIR         	'5'        		// Directory
#define 	LF_FIFO        	'6'        		// FIFO special file
#define		LF_CONTIG	   	'7'        		// Contiguous file

#define 	TAR_RECORDSIZE 	512
#define 	TAR_NAMSIZE    	100
#define 	TAR_TUNMLEN    	32
#define 	TAR_TGNMLEN		32

#define		CBF_FOLDER_MAX	16				// Maximale Anzahl an Foldern im Archiv


typedef		unsigned int	uint32_x;
typedef		unsigned short	uint16_x;
typedef		unsigned char 	uint8_x;

#pragma pack(push,1)


typedef struct
	{
	uint32_x		ulSignature;			// file signature
	uint32_x		ulRes1;					// reserved
	uint32_x		ulCabinetSize;			// size of cabinet file
	uint32_x		ulRes2;					// reserved
	uint32_x		ulFilesOffset;			// offset of the first file entry
	uint32_x		ulRes3;					// reserved
	uint8_x			ucVersionMinor;			// file format version, minor
	uint8_x			ucVersionMajor;			// file format version, major
	uint16_x		usFolders;				// number of folder entries
	uint16_x		usFileCount; 			// number of file entries
	uint16_x		usFlags;				// option flags
	uint16_x		usSetId;				// multiple cabs related
	uint16_x		usCabinet;				// multiple cabs related
	}CabHeader;

typedef struct
	{
	uint16_x		usSizeHeader; 			// size of reserved header area
	uint8_x			ucSizeFolder; 			// size of reserved folder area
	uint8_x			ucSizeBlock;			// size of reserved block area
	}CabHeaderOpt;

typedef struct
	{
	uint32_x		ulStartOffset;			// offset of the first data block
	uint16_x		usDataBlocks;			// number of data blocks
	uint16_x		usCompressType;			// compression type
	}CabFolder;

typedef struct
	{
	uint32_x		ulFileSize; 			// uncompressed size
	uint32_x		ulFolderOffset;			// uncompressed offset of file in folder
	uint16_x		usFolderIndex;			// folder index
	uint16_x		usDate;					// date stamp
	uint16_x		usTime;					// time stamp
	uint16_x		usAttribute;			// attribute flags
	}CabFile;

typedef struct
	{
	uint32_x		ulCheckSum;				// data block checksum
	uint16_x		usSizeCompressed; 		// number of compressed bytes
	uint16_x		usSizeUncompressed;		// number of uncompressed bytes
	}CabBlock;

union  TarData
	{
	unsigned char	ucBuffer[TAR_RECORDSIZE];
	struct
		{
		char		cName    [TAR_NAMSIZE];
		char    	cMode    [8];
		char    	cUid     [8];
		char    	cGid     [8];
		char    	cSize    [12];
		char    	cTime    [12];
		char    	cChkSum  [8];
		char    	cLinkFlag;
		char    	cLinkName[TAR_NAMSIZE];
		char    	cMagic   [8];
		char    	cUname   [TAR_TUNMLEN];
		char    	cGname   [TAR_TGNMLEN];
		char    	cDevMajor[8];
		char    	cDevMinor[8];
		};
	};

#pragma pack(pop)




#define PROG(iMode,pEntry)									\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))				\
	{														\
	if(Progress(iMode,pEntry))throw CAF_ERROR_BREAK;		\
	}
#define PROGR(iMode,pEntry,iRet)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))				\
	{														\
	if(Progress(iMode,pEntry))return iRet;					\
	}
#define PROGN(iMode,pEntry)									\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))				\
	{														\
	Progress(iMode,pEntry);									\
	}
#define PROGT(iMode,iEntry)									\
	if(pThis->m_pProgress && (pThis->m_iProgOn&(1<<iMode)))	\
	{														\
	if(pThis->Progress(iMode,iEntry))throw CAF_ERROR_BREAK;	\
	}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CCabFile::CCabFile()
{

	m_iDoneMode		= 0;
	m_cIsCpio		= 0;

}

//*****************************************************************************
//*
//*		Denstruktor
//*
//*****************************************************************************
CCabFile::~CCabFile()
{

	if(m_iOpenMode!=CAF_CLOSED)								// Ist die TAR-Datei noch offen
		{
		Close();
		}

	if(m_cTarName[0])										// Lsche temorre TAR-Datei
		{
		DeleteFile(m_cTarName);
		m_cTarName[0]=0;
		}

	if(m_hSubFile!=INVALID_HANDLE_VALUE)
		{
		CloseFile(m_hSubFile);
		m_hSubFile = INVALID_HANDLE_VALUE;
		}
}


//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CCabFile::CmpPos(const void *pPos1,const void *pPos2)
{
CabEntry	*pPtr1;
CabEntry	*pPtr2;


	pPtr1 = (CabEntry*)pPos1;
	pPtr2 = (CabEntry*)pPos2;

	if(pPtr1->uFolder!=pPtr2->uFolder)
		{
		return pPtr1->uFolder-pPtr2->uFolder;
		}

	if(pPtr1->uOffset<pPtr2->uOffset)return -1;
	if(pPtr1->uOffset>pPtr2->uOffset)return  1;
	if(pPtr1->uSize  <pPtr2->uSize  )return -1;
	if(pPtr1->uSize  >pPtr2->uSize  )return  1;


return 0;
}

//*****************************************************************************
//*
//*		DosToUtc
//*
//*****************************************************************************
//	Eine DOS-Zeit in eine UTC-Zeit umwandeln.
time_t CCabFile::DosToUtc(unsigned uDate,unsigned uTime)
{
struct  tm sTime;


	sTime.tm_year  = ((uDate>> 9)     )+80;
	sTime.tm_mon   = ((uDate>> 4)&0x0F)-1;
	sTime.tm_mday  = ((uDate    )&0x1F);
	sTime.tm_hour  = ((uTime>>11)&0x1F);
	sTime.tm_min   = ((uTime>> 5)&0x3F);
	sTime.tm_sec   = ((uTime<< 1)&0x3F);
	sTime.tm_isdst = 0;
	sTime.tm_wday  = 0;
	sTime.tm_yday  = 0;
				   

return mktime(&sTime);
}

//*****************************************************************************
//*
//*		UtcToDos
//*
//*****************************************************************************
//	Eine UTC-Zeit in eine DOS-Zeit umwandeln.
void CCabFile::UtcToDos(time_t iUtcTime,unsigned short &uDate,unsigned short &uTime)
{
time_t iEven;
struct tm *pData;


	iEven = (iUtcTime + 1)&(~1);					// Gerade Anzahl an Sekunden
	pData = localtime(&iEven);						// Unix-Zeit von MSDOS

	if(pData->tm_year<80)
		{
		uDate = (1<<5)|1;							// 00:00:00 1.1.1980
		uTime = 0;
		return;
		}

	uDate = (((unsigned)pData->tm_year -   80) <<  9) |
	        (((unsigned)pData->tm_mon  +    1) <<  5) |
		    (((unsigned)pData->tm_mday	 	 )      );

	uTime =  ((unsigned)pData->tm_hour         << 11) |
		     ((unsigned)pData->tm_min          <<  5) |
		     ((unsigned)pData->tm_sec          >>  1);
}


//*****************************************************************************
//*
//*		ReadString
//*
//*****************************************************************************
//	Liet einen String aus dem CAB-Arichiv
//	pBuffer		: Puffer in dem die Daten gespeichert werden
//	iSize		: Ist die Anzahl an Bytes die maximal gelesen werden sollen
//	Ergibt die Lnge der gelesenen Bytes
int CCabFile::ReadString(void *pBuffer,int iSize)
{
char   *pString;
int		iLen;

	
	if(iSize<=0)return 0;

													
	if(BaseRead(pBuffer,iSize)!=iSize)					
		{
		return 0;
		}


	pString = (char*)pBuffer;

	for(iLen=0;iLen<iSize;iLen++)
		{
		if(pString[iLen])continue;
		break;
		}


	if(iLen>=iSize)
		{
		BaseSeek(-iSize,SEEK_CUR);
		return 0;
		}

	iLen++;

	BaseSeek(iLen-iSize,SEEK_CUR);



return iLen;
}


//*****************************************************************************
//*
//*		CabReadBuf
//*
//*****************************************************************************
//	Liet Daten aus dem CAB-File mit Puffererung
//	pBuffer		: Puffer in dem die Daten gespeichert werden
//	iSize		: Ist die Anzahl an Bytes die gelesen werden sollen
//	Ergibt die Anzahl der gelesenen Bytes
int __cdecl CCabFile::CabReadBuf(void *pParam,unsigned char *pMem,int iLen)
{
CCabFile   *pThis=(CCabFile*)pParam;
CabBlock	sBlock;
int			iDoto;
int			iRest;
int			iOut;



	iDoto = iLen;
	iOut  = 0;

	while(iDoto>0)
		{
		   iRest = pThis->m_uBufferSize-pThis->m_uBufferRd;
		if(iRest)
			{
			if(iRest > iDoto)
				{
				iRest = iDoto;
				}

			memcpy(pMem,pThis->m_pBuffer+pThis->m_uBufferRd,iRest);
			pThis->m_uBufferRd += iRest;
			pMem			   += iRest;
			iOut			   += iRest;
			iDoto			   -= iRest;
			continue;
			}

		pThis->m_uBufferNum++;

		if(pThis->m_uBufferNum>(pThis->m_uBufferType&0xFFFF))
			{
			if(pThis->m_uBufferBreak)break;
			throw CAF_ERROR_FORM;
			}

		if(BaseRead(pThis,&sBlock,sizeof(sBlock))!=sizeof(sBlock))
			{
			throw CAF_ERROR_FORM;
			}

		pThis->m_iDoneNow   += sizeof(sBlock);

		pThis->BaseSeek(pThis->m_uBufferSeek,SEEK_CUR);

		if(sBlock.usSizeCompressed>32768+6144)
			{
			throw CAF_ERROR_FORM;
			}

		if(sBlock.usSizeUncompressed>32768)
			{
			throw CAF_ERROR_FORM;
			}

		if(BaseRead(pThis,pThis->m_pBuffer,sBlock.usSizeCompressed)!=sBlock.usSizeCompressed)
			{
			throw CAF_ERROR_FORM;
			}

		pThis->m_iDoneNow   += sBlock.usSizeCompressed;
		pThis->m_uBufferSize = sBlock.usSizeCompressed;
		pThis->m_uBufferRd   = 0;


		if((pThis->m_uBufferType&0x000F0000)==0x00020000)		// Quantum hack
			{
			if(pThis->m_uBufferSize<0x10000)
				{
				pThis->m_pBuffer[pThis->m_uBufferSize] = '\xFF';
				pThis->m_uBufferSize++;
				}
			}

		if(pThis->m_uBufferNum>=(pThis->m_uBufferType&0xFFFF))
			{
			if((pThis->m_uBufferType&0x000F0000)==0x00030000)  // LZX hack
				{
				lzx_decompress_setout((lzx_stream*)pThis->m_uBufferStream,(pThis->m_uBufferNum-1)*32768+sBlock.usSizeUncompressed);
				pThis->m_uBufferBreak=1;
				}
			}
		}


return iOut;
}

//*****************************************************************************
//*
//*		ReadFromTar
//*
//*****************************************************************************
int __cdecl CCabFile::ReadFromTar(void *pParam,unsigned char *pMem,int iLen)
{
CCabFile   *pThis=(CCabFile*)pParam;
char		cTemp[1024+4];
CabBlock	sBlock;
TarData		sData;
unsigned	uRet;
int			iNum;
int			i;



	if(pThis->m_uTarOut<0)return 0;


	if(pThis->m_uBufferRd>=0x8000)								// Puffergrenze erreichet
		{
		pThis->m_pEncoder->Flush();

		sBlock.ulCheckSum			= 0;
		sBlock.usSizeCompressed		= pThis->m_uBufferSize;
		sBlock.usSizeUncompressed	= pThis->m_uBufferRd;

		BaseWrite(pThis,&sBlock,sizeof(sBlock));
		BaseWrite(pThis,pThis->m_pBuffer,sBlock.usSizeCompressed);

		pThis->m_iDoneNow   += pThis->m_uBufferRd;
		pThis->m_uBufferNum += 1;
		pThis->m_uBufferSize = 0;
		pThis->m_uBufferRd	 = 0;
		}


	uRet = 0;


	while(iLen>0)												// Date komprimieren der Daten
		{
		if(pThis->m_uTarOut==0)									// Einen neuen Header einlesen
			{
			PROGT(CAF_PROG_COMPRESS,pThis->m_uTarFile);

			if(MainRead(pThis,&sData,512)!=512)
				{
				pThis->m_uTarOut = -1;
				break;
				}

			pThis->m_uTarOut   = 512;
			pThis->m_uTarFile += 1;
			pThis->m_uTarPos  += 1;

			if(memcmp(sData.cMagic,"ustar  ",8))				// Passt der Header
				{
				for(i=0;i<TAR_RECORDSIZE;i++)
					{
					if(sData.ucBuffer[i])throw CAF_ERROR_FREAD;
					}

				pThis->m_uTarOut = -1;
				break;
				}


			pThis->m_uTarLen  = OctToInt(sData.cSize);
			pThis->m_uTarOff +=    pThis->m_uTarLen+512;
			pThis->m_uTarOff += (0-pThis->m_uTarLen)&0x1FF;


			if(sData.cLinkFlag!=LF_NORMAL)						// Ist die Datei eine normale Datei
				{
				pThis->m_uTarLen +=  0x1FF;
				pThis->m_uTarLen &= ~0x1FF;

				MainSeek(pThis,pThis->m_uTarOff,SEEK_SET);
				continue;
				}

			if(sData.cLinkFlag=='L')							// Langer Dateiname
			if(pThis->m_uTarLen<=1024)
			if(memcmp(sData.cName,"././@LongLink\0\0",16)==0)
				{
				iNum = (pThis->m_uTarLen+511)&~0x1FF;

				if(MainRead(pThis,cTemp,iNum)!=iNum)
					{
					throw CAF_ERROR_FREAD;
					break;
					}

				cTemp[iNum] = 0;

				if(MainRead(pThis,&sData,512)!=512)
					{
					throw CAF_ERROR_FREAD;
					break;
					}

				pThis->m_uTarOut   = 512;
				pThis->m_uTarFile += 1;
				pThis->m_uTarOff  += 512+iNum;

				if(memcmp(sData.cMagic,"ustar  ",8))			// Passt der Header
					{
					throw CAF_ERROR_FREAD;
					}

				pThis->m_uTarLen  = OctToInt(sData.cSize);
				pThis->m_uTarOff +=    pThis->m_uTarLen;
				pThis->m_uTarOff += (0-pThis->m_uTarLen)&0x1FF;
				}
			}


		iNum = pThis->m_uTarLen;

		if(iNum>iLen)iNum=iLen;
		if(pThis->m_uBufferRd+iNum>0x8000)
			{
			iNum = 0x8000-pThis->m_uBufferRd;
			}


		if(MainRead(pThis,pMem,iNum)!=iNum)
			{
			throw CAF_ERROR_FREAD;
			}


		pThis->m_uBufferRd  += iNum;
		pThis->m_uTarLen	-= iNum;
		pThis->m_uTarOut	+= iNum;
		iLen				-= iNum;
		pMem				+= iNum;
		uRet				+= iNum;


		if(pThis->m_uTarLen<=0)								// Dateiende in TAR erreicht
			{
			MainSeek(pThis,pThis->m_uTarOff,SEEK_SET);
			pThis->m_uTarOut = 0;
			}

		if(pThis->m_uBufferRd>=0x8000)						// Puffergrenze erreichet
			{
			if(uRet>0)break;

			pThis->m_pEncoder->Flush();

			sBlock.ulCheckSum			= 0;
			sBlock.usSizeCompressed		= pThis->m_uBufferSize;
			sBlock.usSizeUncompressed	= pThis->m_uBufferRd;

			BaseWrite(pThis,&sBlock,sizeof(sBlock));
			BaseWrite(pThis,pThis->m_pBuffer,sBlock.usSizeCompressed);

			pThis->m_uBufferNum += 1;
			pThis->m_uBufferSize = 0;
			pThis->m_uBufferRd	 = 0;
			}
		}



return uRet;
}


//*****************************************************************************
//*
//*		CabWrite
//*
//*****************************************************************************
//	Schreibt Daten in eine CAB-Datei
int _cdecl 	CCabFile::CabWrite(void *pParam,const char *pMem,int iLen)
{
CCabFile   *pThis=(CCabFile*)pParam;


	if(iLen<=0)return 0;

	if(pThis->m_uBufferSize+(unsigned)iLen>0xC000)
		{
		iLen = 0xC000-pThis->m_uBufferSize;
		}


	memcpy(pThis->m_pBuffer+pThis->m_uBufferSize,pMem,iLen);
	pThis->m_uBufferSize += iLen;



return iLen;
}


//*****************************************************************************
//*
//*		WriteToTar
//*
//*****************************************************************************
//	Schreibt Daten in die temporre TAR-Datei.
//	Wenn ntig wird ein Header eingefgt
int	__cdecl CCabFile::WriteToTar(void *pParam,unsigned char *pMem,int iLen)
{
CCabFile   *pThis=(CCabFile*)pParam;
CabEntry   *pFile;
char		cBuffer[512];
TarItem		sEntry;
int			iNum;
int			iRet;


	pFile	= pThis->m_pFiles;
	pFile  += pThis->m_uTarPos;
	iRet    = 0;


	while(iLen>0 && pThis->m_uTarPos<pThis->m_uFileCount)
		{
		if(pThis->m_uTarOut==0)								// Beginnt eine neue Datei
			{
			sEntry.iAttrib	 	= (pFile->cReadOnly)? 0100444:0100664;
			sEntry.iLen		 	=  pFile->uSize;
			sEntry.iTime	 	=  pFile->iTime;
			sEntry.pName	 	=  pFile->pName;
			sEntry.iNameLen	 	=  (int)strzlen(sEntry.pName);
			sEntry.cType		=  '0';

			pThis->MakeTarHeader(cBuffer,&sEntry);
			pThis->MainWrite    (cBuffer,512);
			pThis->m_iDoneNow  += 512;
			pThis->m_uTarOut	= 512;
			}


		   iNum = pThis->m_uTarOff-pFile->uOffset;
		if(iNum<0)											// Datei-Offset wurde noch nicht erreicht
			{
			iNum = -iNum;

			if(iNum>=iLen)									// Zuwenige Bytes
				{
				pThis->m_uTarOff+= iLen;
				return iLen;
				}

			iLen -= iNum;
			iRet += iNum;
			pMem += iNum;
			iNum  = 0;
			}


		if(iNum+iLen<(int)pFile->uSize)						// Dateiende wird nicht erreicht
			{
			pThis->MainWrite(pMem,iLen);
			pThis->m_iDoneNow  += iLen;
			pThis->m_uTarOut   += iLen;
			pThis->m_uTarOff   += iLen;

			return iRet+iLen;
			}


		iNum = pFile->uSize-iNum;							// Dateiende wird erreicht

		pThis->MainWrite(pMem,iNum);
		pThis->m_iDoneNow  += iNum;
		pThis->m_uTarOut   += iNum;
		pThis->m_uTarOff   += iNum;
		iLen               -= iNum;
		iRet               += iNum;
		pMem               += iNum;

		iNum  = (512-(pThis->m_uTarOut&0x1FF))&0x1FF;

		memset(cBuffer,0,iNum);								// Auf 512 Byte-Grenze auffllen

		pThis->MainWrite(cBuffer,iNum);
		pThis->m_iDoneNow  += iNum;
		pThis->m_uTarPos   += 1;
		pThis->m_uTarOut	= 0;
		pFile++;
		}




return iLen+iRet;
}

//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine BZIP2-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CCabFile::ArchOpen(int iLevel)
{
unsigned	aFolterType[CBF_FOLDER_MAX];
unsigned	aFolterSize[CBF_FOLDER_MAX];
unsigned	aFolterFile[CBF_FOLDER_MAX];
unsigned	aFolterOff [CBF_FOLDER_MAX];
unsigned	uFolterCount;				
unsigned	uBufferSeek;
unsigned	uLastPos;	
unsigned 	uFolder;
unsigned	uOffset;
unsigned	uSize;
aint64		iByte;
int			iLen;
int			iMin;
int			iMax;
int			iPos;
int			iNum;



	SetDoneMode(-1);										// Done-Modus in Tar-Datei

	m_iDoneMode		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_cIsCpio		= 0;

	uFolterCount	= 0;	
	uBufferSeek		= 0;
	uLastPos		= 0;
	m_pFiles		= 0;
	m_pBuffer		= 0;
	m_uTarPos		= 0;
	m_uFileCount	= 0;
	iByte			= m_iFileSize+16;


	Clear();
	InitSubFile();


	if(iByte<0x400000)										// Bis 4MB im Speicher ffnen
		{
		iMin	= (int)iByte;
		iMax	= 0x1000000;								// 10MB maximal
		}
	else{
		iMin	= 0;
		iMax	= 0;
		}


	SetMemSub(0x10000,0x30000,m_cTarName,Z(".tar"));		// Dynamischen Speicher erstellen
		

	if(m_iSubMode==CAF_CREATE)								// Neues Archiv erzeugen 
		{
		m_iOpenMode = CAF_CREATE;
		return CTarFile::ArchOpen(iLevel);
		}


	m_iOpenMode = CAF_WRITE;


//******************** Dateinamen einlesen ************************************
char			cText[256];
CabHeaderOpt	sOptional;
CabHeader		sHeader;
CabFolder		sFolder;
CabFile			sFile;
zchar		   *pText;
unsigned		uFile;
unsigned		uChar;


	try	{
		memset(&sOptional,0,sizeof(sOptional));

		if(BaseRead(&sHeader,sizeof(sHeader))!=sizeof(sHeader))throw CAF_ERROR_FORM;

		m_iDoneMax	= m_iSubSize;
		m_iDoneNow	= 0;
		m_iDoneMode = 1;

		if(sHeader.ulSignature   !=0x4643534D)throw CAF_ERROR_FORM;
		if(sHeader.ucVersionMajor!=1         )throw CAF_ERROR_FORM;


		if(sHeader.usFlags&0x0004)							// Optionalen Header laden
			{
			if(BaseRead(&sOptional,sizeof(sOptional))!=sizeof(sOptional))throw CAF_ERROR_FORM;
			if(sOptional.usSizeHeader)BaseSeek(sOptional.usSizeHeader,SEEK_CUR);
			uBufferSeek = sOptional.ucSizeBlock;
			}

		if(sHeader.usFlags&0x0001)							// Vorheriges CAB-Archiv
			{
			if(!ReadString(cText,sizeof(cText)))throw CAF_ERROR_FORM;
			if(!ReadString(cText,sizeof(cText)))throw CAF_ERROR_FORM;
			}

		if(sHeader.usFlags&0x0002) 							// Nchstes CAB-Archiv
			{
			if(!ReadString(cText,sizeof(cText)))throw CAF_ERROR_FORM;
			if(!ReadString(cText,sizeof(cText)))throw CAF_ERROR_FORM;
			}

		for(uFolder=0;uFolder<sHeader.usFolders;uFolder++)	// Verzeichnisse laden
			{
			if(BaseRead(&sFolder,sizeof(sFolder))!=sizeof(sFolder))throw CAF_ERROR_FORM;
			if((sFolder.usCompressType&0x000F)>3)throw CAF_ERROR_FORM;

			if(sOptional.ucSizeFolder)
				{
				BaseSeek(sOptional.ucSizeFolder,SEEK_CUR);
				}

			if(uFolterCount<CBF_FOLDER_MAX)					// Verzeichnissedaten speichern
				{
				aFolterOff [uFolterCount] = sFolder.ulStartOffset;
				aFolterType[uFolterCount] = sFolder.usDataBlocks|(sFolder.usCompressType<<16);
				uFolterCount++;
				}
			}


			m_pFiles = new CabEntry[sHeader.usFileCount];
		if(!m_pFiles)
			{
			throw CAF_ERROR_MEM;
			}


		m_uFileCount = sHeader.usFileCount;


		for(uFile=0;uFile<m_uFileCount;uFile++)				// Dateinamen laden
			{
			if(BaseRead (&sFile,sizeof(sFile))!=sizeof(sFile))throw CAF_ERROR_FORM;
			if(ReadString(cText,sizeof(cText))            ==0)throw CAF_ERROR_FORM;


			if(sFile.usFolderIndex<0xFFFD)
			if(sFile.usFolderIndex>=sHeader.usFolders)
				{
				throw CAF_ERROR_FORM;
				}

			if(sFile.usFolderIndex>=uFolterCount)			// Verzeichnisse prfen
				{
				continue;
				}

			for(iLen=0;cText[iLen];iLen++);

			iLen++;

				pText = new zchar[iLen];
			if(!pText)throw CAF_ERROR_MEM;


			if(sFile.usAttribute&0x80)						// UTF8 Dateiname
				{
				for(iPos=0,iNum=0;iPos<iLen;iNum++)
					{
					iPos += utf8tochar(cText+iPos,uChar);
					#ifndef UNICODE
					if(uChar>255)uChar=255;
					#endif
					pText[iNum] = (zchar)uChar;
					}
				}
			else{											// Normaler Dateiname
				for(iPos=0;iPos<iLen;iPos++)
					{
					pText[iPos] = (unsigned char)cText[iPos];
					}
				}

			m_pFiles[uFile].iTime		= DosToUtc(sFile.usDate,sFile.usTime);
			m_pFiles[uFile].cReadOnly	= (char)(sFile.usAttribute&0x01);
			m_pFiles[uFile].uOffset		= sFile.ulFolderOffset;
			m_pFiles[uFile].uFolder		= sFile.usFolderIndex;
			m_pFiles[uFile].uSize		= sFile.ulFileSize;
			m_pFiles[uFile].pName		= pText;
			uLastPos					= sFile.ulFolderOffset;
			uLastPos				   += sFile.ulFileSize;
			m_iDoneMax				   +=(sFile.ulFileSize+1535)&~0x1FF; 	
			}
		

//******************** Die Dateidaten dekomprimieren **************************
mszip_stream   *pZipStream=0;
qtm_stream     *pQtmStream=0;
lzx_stream     *pLzxStream=0;
zchar			cTemp[1024];
unsigned		uPos;
int				iLen;
int				iRet;


		m_uBufferSeek = 0;			

		qsort(m_pFiles,m_uFileCount,sizeof(CabEntry),CmpPos);
		memset(aFolterSize,0,sizeof(aFolterSize));
		
		uFolder = 0;
		
		for(uPos=1;uPos<m_uFileCount;uPos++)				// Gren der einzelnen Verzeichniss berechen
			{
			if(uFolder==m_pFiles[uPos].uFolder)continue;

			uSize	= m_pFiles[uPos-1].uOffset;				
			uSize  += m_pFiles[uPos-1].uSize;

			aFolterFile[uFolder] = uPos;
			aFolterSize[uFolder] = uSize;
			uFolder = m_pFiles[uPos].uFolder;
			}

		if(m_uFileCount)
			{
			uSize	= m_pFiles[uPos-1].uOffset;				
			uSize  += m_pFiles[uPos-1].uSize;

			aFolterFile[uFolder] = uPos;
			aFolterSize[uFolder] = uSize;
			}


		PROG(CAF_PROG_DOEXTRACT,-1);


		m_pBuffer	= new unsigned char[0x10000];
		m_uTarOut	= 0;

		if(!m_pBuffer)throw CAF_ERROR_MEM;


		for(m_uTarPos=0;m_uTarPos<m_uFileCount;)			// Alle Dateien entkomprimieren
			{
			uFolder			= m_pFiles[m_uTarPos].uFolder;
			m_uTarOff		= 0;

			BaseSeek(aFolterOff[uFolder]);

			m_uBufferRd		= 0;
			m_uBufferNum	= 0;
			m_uBufferSize	= 0;
			m_uBufferBreak	= 0;
			m_uBufferType	= aFolterType[uFolder];
			uSize			= aFolterSize[uFolder];


			switch((m_uBufferType>>16)&0x0F)
				{
			case 0x0000:									// STORE

				for(uOffset=0;uOffset<uSize;uOffset+=iLen)
					{
					unsigned char	aBuffer[0x1000];

					   iLen = uSize-uOffset;
					if(iLen>sizeof(aBuffer))iLen=sizeof(aBuffer);
					if(BaseRead(aBuffer,iLen)!=iLen)throw CAF_ERROR_READ;
					m_iDoneNow += iLen;
					WriteToTar(this,aBuffer,iLen);
					}

				break;


			case 0x0001:									// MSZIP

					pZipStream = (mszip_stream*)mszip_decompress_init(this,this,4096,1,CabReadBuf,WriteToTar);
				if(!pZipStream)throw CAF_ERROR_MEM;

				iRet=mszip_decompress(pZipStream,uSize);
				mszip_decompress_free(pZipStream);
				pZipStream=0;
				break;


			case 0x0002:									// QUANTUM

					pQtmStream = (qtm_stream*)qtm_decompress_init(this,this,(m_uBufferType>>24)&0x1F,4096,CabReadBuf,WriteToTar);
				if(!pQtmStream)throw CAF_ERROR_MEM;

				iRet=qtm_decompress(pQtmStream,uSize);
				qtm_decompress_free(pQtmStream);
				pQtmStream=0;
				break;


			case 0x0003:									// LZX

					pLzxStream = (lzx_stream*)lzx_decompress_init(this,this,(m_uBufferType>>24)&0x1F,0,4096,0,CabReadBuf,WriteToTar);
				if(!pLzxStream)throw CAF_ERROR_MEM;

				m_uBufferStream=pLzxStream;
				iRet=lzx_decompress(pLzxStream,uSize);
				lzx_decompress_free(pLzxStream);
				pLzxStream=0;
				break;

			default:

				throw CAF_ERROR_FORM;
				}

			switch(iRet)									// Fehlermedungen auswerten
				{
			case MSP_SUCCESS:	break;						// no error
			case MSP_EIO:		throw CAF_ERROR_READ;		// general I/O error
			case MSP_EIO_RD:	throw CAF_ERROR_READ;		// general I/O error
			case MSP_EIO_WR:	throw CAF_ERROR_WRITE;		// general I/O error
			case MSP_EFORMAT:	throw CAF_ERROR_FORM;		// bad format or broken file
			case MSP_EMEM: 		throw CAF_ERROR_MEM;		// memory allocation error
			default:			throw CAF_ERROR_DECOMP;
				}

															// Wurde genug endkomprimiert
			if(m_uTarOff<uSize || m_uTarPos<aFolterFile[uFolder])			
				{
				throw CAF_ERROR_DECOMP;
				}
			}


		memset   (cTemp+512,0,512);							// Dummy-Header am Ende in TAR-Archiv speichern
		MainWrite(cTemp+512,  512);


		if(m_iDoneMax>m_iDoneNow)
			{
			m_uTarSize = (unsigned)(m_iDoneMax-m_iDoneNow);
			}
		else{
			m_uTarSize = 0;
			}

		m_iDoneMode	= 2;



		if(m_iSubMode==CAF_READ)							// Lesemodus fr TAR-Datei
			{
			if(DoChange(CAF_READ,0)!=CAF_ERROR_OK)
				{
				return m_iErrorCode;
				}
			}
		else{
			MainSeek(0);
			}

		iRet = CTarFile::ArchOpen(iLevel);					// Die Dateien aus der TAR-Datei einlesen

		if(iRet!=CAF_ERROR_OK)								// Gab es Fehler beim ffnen
			{
			if(m_hFile!=INVALID_HANDLE_VALUE)
				{
				CloseFile(m_hFile);
				m_hFile = INVALID_HANDLE_VALUE;
				}

			DeleteFile(m_cTarName);

			return iRet;
			}
															
		SetScale(MainSize(),iByte);							// Skalierung fr komprimierte Gre	
		}
	catch(int iError)										// Gab es Fehler
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		Break();
		}


	if(m_pBuffer)delete m_pBuffer;
	if(m_pFiles )delete m_pFiles;


	m_iDoneNow	= Done(&m_iDoneMax);						// Aktuellen Verlauf in TAR-Datei bernehmen
	m_iDoneMode	= 0;
	SetDoneMode(-1);										
	
	

return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das CAB-Archiv und Speichert alle Daten
//	Ergibt 0 wenn ok, oder einen Fehlercode ungleich 0:
//			CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CCabFile::Close()
{
int			iWrite;
int			iRet;




	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;


	if(m_iOpenMode==CAF_READ || !m_iChanged)
			iWrite=0;
	else	iWrite=1;


	if(!iWrite)												// CAB-Datei nur schlieen
		{
		if(m_hSubFile!=INVALID_HANDLE_VALUE)
			{
			CloseFile(m_hSubFile);
			m_hSubFile=INVALID_HANDLE_VALUE;
			}
		}
	else{													// Alles in der TAR-Datei speichern
		m_iDoneNow  = 0;
		m_iDoneMax	= AllLen   ();
		m_iDoneSave	= FlushCalc();
		m_iDoneMax += m_iDoneSave;
		m_iDoneMode	= 3;

		   m_iErrorCode =Flush();
		if(m_iErrorCode!=CAF_ERROR_OK)						
			{
			Break();
			return m_iErrorCode;
			}

		m_iDoneMode	= 4;

		   m_iErrorCode = CloseFlush();						// Die TAR-Daten in die CAB-Datei schreiben
		if(m_iErrorCode!=CAF_ERROR_OK)
			{
			Break();
			return m_iErrorCode;
			}

		m_iDoneNow	= Done(&m_iDoneMax);					// Aktuellen Verlauf in TAR-Datei bernehmen
		m_iDoneMode	= 0;
		SetDoneMode(-1);										
		}


	   iRet =CTarFile::Close();
	if(iRet!=CAF_ERROR_OK)									// Alles in der TAR-Datei speichern
		{
		m_iErrorCode = iRet;
		}


	if(m_cTarName[0])										// Lsche temorre TAR-Datei
		{
		DeleteFile(m_cTarName);
		m_cTarName[0]=0;
		}

	ReleaseData();



return m_iErrorCode;
}

//*****************************************************************************
//*
//*		CloseFlush
//*
//*****************************************************************************
//	Beim Schlieen der CAB-Datei die Daten aus der TAR-Datei in die CAB-Datei komprimieren.
int CCabFile::CloseFlush()
{
TarData		sData;
char	   *pName;
char		cTemp[1024+2];
CabHeader	sHeader;
CabFolder	sFolter;
CabBlock	sBlock;
CabFile		sFile;
unsigned	uChkSum;
int			iFullSize;
int			iCount;
int			iSize;
int			iMode;
int			iOff;
int			iLen;
int			iAdd;
int			iPos;
int			iNum;
int			i;




	try	{
		unsigned char	aBuffer[0xC000];
		CLzxClass		sCompress;
		time_t			iTime;

		memset(&sHeader,0,sizeof(sHeader));

		sHeader.ulSignature		= 0x4643534D;				// file signature
		sHeader.ulFilesOffset	= sizeof(sHeader)+sizeof(sFolter);// offset of the first file entry
		sHeader.ucVersionMinor	= 3;						// file format version, minor
		sHeader.ucVersionMajor	= 1;						// file format version, major
		sHeader.usFolders		= 1;						// number of folder entries
		sHeader.usSetId			= 12345;					// multiple cabs related



		BaseSeek(sizeof(sHeader)+sizeof(sFolter));
		EndOfFile(1);
		MainSeek (0);


		//*********************************************************************

		iFullSize	= 0;
		iCount		= 0;
		iPos		= 0;

		for(i=0;;i++)										// Dateien in der TAR-Datei zhlen
			{
			PROG(CAF_PROG_COMPRESS,i);

			if(MainRead(&sData,512)!=512)
				{
				break;
				}

			if(memcmp(sData.cMagic,"ustar  ",8))			// Passt der Header
				{
				for(i=0;i<TAR_RECORDSIZE;i++)
					{
					if(sData.ucBuffer[i])throw CAF_ERROR_FREAD;
					}

				break;
				}


			uChkSum = OctToInt(sData.cChkSum);
			memset(sData.cChkSum,' ',sizeof(sData.cChkSum));

			for(i=0;i<TAR_RECORDSIZE;i++)					// Cheksumme prfen
				{
				uChkSum -= sData.ucBuffer[i];
				}

			if(uChkSum!=0)
				{
				throw CAF_ERROR_CRC;
				}


			iLen  = OctToInt(sData.cSize);

			if(iLen<0 || iLen>0x40000000)					// Die Dateilnge Prfen
				{
				throw CAF_ERROR_FREAD;
				}

			iOff  = iPos+TAR_RECORDSIZE;
			iAdd  = iLen+TAR_RECORDSIZE-1;
			iAdd -= iAdd%TAR_RECORDSIZE;
			iPos += iAdd+TAR_RECORDSIZE;


			MainSeek(iPos);
			pName = sData.cName;

			if(sData.cLinkFlag=='L' && iLen<=1024)			// Langer Dateiname
			if(memcmp(sData.cName,"././@LongLink\0\0",16)==0)
				{
				iNum = (iLen+511)&~0x1FF;

				if(MainRead(cTemp,iNum)!=iNum)
					{
					throw CAF_ERROR_FREAD;
					}

				cTemp[iNum] = 0;
				pName = cTemp;

				if(MainRead(&sData,512)!=512)
					{
					throw CAF_ERROR_FREAD;
					}

				if(memcmp(sData.cMagic,"ustar  ",8))		// Passt der Header
					{
					throw CAF_ERROR_FREAD;
					}

				iLen  = OctToInt(sData.cSize);

				if(iLen<0 || iLen>0x40000000)				// Die Dateilnge Prfen
					{
					throw CAF_ERROR_FREAD;
					}

				iOff  = iPos+TAR_RECORDSIZE;
				iAdd  = iLen+TAR_RECORDSIZE-1;
				iAdd -= iAdd%TAR_RECORDSIZE;
				iPos += iAdd+TAR_RECORDSIZE;
				}

			if(sData.cLinkFlag!=LF_NORMAL)					// Ist die Datei eine normale Datei
				{
				continue;
				}

			while(*pName=='/')pName++;

			for(iSize=0;pName[iSize];iSize++)
				{
				if(pName[iSize]=='/')pName[iSize]='\\';
				}

			iSize  += 1;
			iTime	= OctToInt(sData.cTime);				// Ist die nderungszeit
			iMode   = OctToInt(sData.cMode);				// Attribute abfragen

			sFile.ulFileSize	 = iLen; 					// uncompressed size
			sFile.ulFolderOffset = iFullSize;				// uncompressed offset of file in folder
			sFile.usFolderIndex	 = 0;						// folder index
			sFile.usAttribute	 = 0x80;					// attribute flags

			if(!(iMode&0200))sFile.usAttribute |= 0x01;		// Schreibgeschtzt

			UtcToDos(iTime,sFile.usDate,sFile.usTime);


			BaseWrite(&sFile,sizeof(sFile));
			BaseWrite(pName,iSize);

			iFullSize += iLen;
			iCount    += 1;
			}


		sHeader.usFileCount		= iCount;
		sFolter.ulStartOffset	= (unsigned)BaseTell();
		sFolter.usCompressType	= 0x0F03;
		
		MainSeek(0);


		//*********************************************************************

		m_uTarFile		= 0;
		m_uTarPos		= 0;
		m_uTarOff		= 0;
		m_uTarOut		= 0;
		m_uBufferRd		= 0;
		m_uBufferNum	= 0;
		m_uBufferSize	= 0;
		m_pBuffer		= aBuffer;
		m_pEncoder		= &sCompress;


		sCompress.SetIn (this,ReadFromTar);
		sCompress.SetOut(this,CabWrite);


		try	{
			sCompress.Compress(1);
			}
		catch(int iError)
			{
			throw iError;
			}
		catch(...)
			{
			throw CAF_ERROR_COMP;
			}


		if(m_uBufferSize>0)
			{
			sBlock.ulCheckSum			= 0;
			sBlock.usSizeCompressed		= m_uBufferSize;
			sBlock.usSizeUncompressed	= m_uBufferRd;


			BaseWrite(&sBlock,sizeof(sBlock));
			BaseWrite(m_pBuffer,sBlock.usSizeCompressed);

			m_uBufferNum++;
			}

		//*********************************************************************

															
		sHeader.ulCabinetSize = (unsigned)BaseTell();		// Dateigre holen
		sFolter.usDataBlocks  = m_uBufferNum;


		BaseSeek(0);
		BaseWrite(&sHeader,sizeof(sHeader));
		BaseWrite(&sFolter,sizeof(sFolter));


		PROG(CAF_PROG_CLOSE,-1);

		m_iErrorCode = CAF_ERROR_OK;
		}
	catch(int iError)
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		return m_iErrorCode;
		}


return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush,Change ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt die Position des aktuellen Verlaufs.
aint64	CCabFile::Done(aint64 *pMax)
{
aint64		iNow;
aint64		iMax;



	switch(m_iDoneMode)
		{
	case 1:													// Entpacke das CAB-Archiv in die TAR-Datei

			iNow = m_iDoneNow;

			if(iNow>m_iDoneMax)
				{
				iNow = m_iDoneMax;
				}

			if(pMax)*pMax=m_iDoneMax;

			return iNow;

	case 2:													// ffne TAR-Datei nach dem Entpacken
	
			iNow	= CTarFile::Done(&iMax);		

			if(m_uTarSize>0 && iMax>0)
				{
				iNow *= m_uTarSize;							// Den TAR-Verlauf skallieren
				iNow /= iMax;
				}

			   iNow += m_iDoneNow;
			if(iNow>m_iDoneMax)
				{
				iNow = m_iDoneMax;
				}

			if(pMax)*pMax=m_iDoneMax;
			
			return m_iDoneNow+iNow;

	case 3:													// Speichere alles im TAR-Archiv 

			iNow	= CTarFile::Done(&iMax);				

			if(iNow>m_iDoneSave)							// Den TAR-Verlauf prfen
				{
				iNow = m_iDoneSave;
				}
	
			if(pMax)*pMax=m_iDoneMax;
			
			return iNow;


	case 4:													// Komprimiere TAR-Archiv in die CAB-Datei

			iNow	= m_iDoneNow;
			iNow   += m_iDoneSave;

			if(iNow>m_iDoneMax)
				{
				iNow = m_iDoneMax;
				}
			
			if(pMax)*pMax=m_iDoneMax;
			
			return iNow;
		}


return CTarFile::Done(pMax);
}


//*****************************************************************************
//*
//*		Change
//*
//*****************************************************************************
//	Schreiberlaubnis aktivieren oder deaktivieren.
//	iMode		: Ist der neue Modus (CAF_WRITE oder CAF_READ)
//	Ergibt CAF_ERROR_OK wenn erfolgreich, oder eine Fehlernummer ungleich 0.
int CCabFile::Change(int iMode)
{
int		iError;


	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;



	if(m_iSubMode==iMode)									// Keine nderung	
		{
		return CAF_ERROR_OK;
		}


	if(iMode==CAF_READ && m_iChanged)						// TAR-Date in CAB-Datei speichern
		{
		m_iDoneNow  = 0;
		m_iDoneMax	= AllLen   ();
		m_iDoneSave	= FlushCalc();
		m_iDoneMax += m_iDoneSave;
		m_iDoneMode	= 3;

		   m_iErrorCode =Flush();							// Alles in der TAR-Datei speichern
		if(m_iErrorCode!=CAF_ERROR_OK)					
			{
			return m_iErrorCode;
			}

		m_iDoneMode	= 4;

		   m_iErrorCode = CloseFlush();						// Die CAB-Datei anlegen
		if(m_iErrorCode!=CAF_ERROR_OK)
			{
			return m_iErrorCode;
			}

		m_iDoneNow	= Done(&m_iDoneMax);					// Aktuellen Verlauf in TAR-Datei bernehmen
		m_iDoneMode	= 0;
		SetDoneMode(-1);										
		}



	   iError = CTarFile::Change(iMode);					// Die TAR-Datei umstellen
	if(iError)											
		{
		Close();
		m_iErrorCode = iError;
		return         iError;
		}

	   iError = DoChangeSub(iMode,0);						// Die Unterdatei umstellen		
	if(iError!=CAF_ERROR_OK)							
		{
		return iError;
		}



return iError;
}


















