//*****************************************************************************
//*
//*
//*		CabFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf CAB-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *Type=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *Type=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractMem(int iNum,char *pMem,int)		// Datei in Speicher extrahieren
// 		int		ExtractAll(         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(   int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *Datei,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *Datei)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int		Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *Datei);					// Datei suchen
// 		time_t	Time (char  *Datei);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef 	__CABFILE_H__
#define 	__CABFILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	"TarFile.h"

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif


class		CLzxClass;

//*****************************************************************************
//*
//*		Klasse CCabFile
//*
//*****************************************************************************
class CCabFile : public CTarFile
{
protected:

	typedef	struct
		{
		zchar		   *pName;					// Name der Datei
		char			cReadOnly;				// Ist die Datei schreibgeschtzt
		unsigned		uFolder;				// Ist die Folder
		unsigned 		uOffset;				// Offset im entpackten Stream
		unsigned 		uSize;					// Ist die Gre der Datei in Byte
		time_t			iTime;					// Dateizeit
		}CabEntry;


	zchar				m_cTarName[MAX_PATH];	// Name der temporren TAR-Datei
	aint64				m_iDoneSave;			// Wieviele Byte mssen ins TAR-Achive geschrieben werden
	int					m_iDoneMode;			// Verlaufmodus 0=normal 1=Mit Offset 2=Open mit Huffman 3=Close mit Huffman
	unsigned			m_uTarSize;				// Gre der TAR-Daten zum entkomprimieren
	unsigned			m_uTarFile;				// Ist die aktuelle Datei beim Komprimieren
	unsigned			m_uTarLen;				// Ist die aktuelle Dateilnge beim Komprimieren
	unsigned 			m_uTarOff;				// Aktuelles Offset frs TAR-Entpacken (Datei)
	unsigned 			m_uTarOut;				// Aktuelles Offset frs TAR-Entpacken (TAR)
	unsigned 			m_uTarPos;				// Aktuelle  Datei  frs TAR-Entpacken

	CLzxClass		   *m_pEncoder;				// Klasse zum Komprimieren
	CabEntry		   *m_pFiles;				// Liste mit Dateien im CAB-Archiv
	unsigned 			m_uFileCount;			// Anzahl der Dateien im Archiv

	unsigned char	   *m_pBuffer;				// Zeiger auf Lesepuffer
	unsigned			m_uBufferRd;			// Position im Lesepuffer
	unsigned			m_uBufferNum;			// Nummer des aktuellen Puffers
	unsigned			m_uBufferSize;			// Gre des Lesepuffers
	unsigned			m_uBufferType;			// Aktueller Puffertyp
	unsigned			m_uBufferSeek;			// Zusatzspeicher fr Puffer
	unsigned			m_uBufferBreak;			// Wurde das Pufferende erreicht
	void   			   *m_uBufferStream;		// Ist ein Zeiger auf den LZX-Stream



//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

						CCabFile();
	virtual			   ~CCabFile();

	virtual	int			Change     (int   iMode=CAF_WRITE);	// Schreiberlaubnis de/aktivieren
	virtual	int			Close      ();						// Datei schlieen
	virtual	aint64		Done	   (aint64 *pMax=0);		// Verlaufsabfrage

protected:

	static  int	__cdecl CabWrite   (void *pParam,const    char *pMem,int iLen);
	static  int	__cdecl WriteToTar (void *pParam,unsigned char *pMem,int iLen);
	static  int	__cdecl ReadFromTar(void *pParam,unsigned char *pMem,int iLen);
	static  int __cdecl CabReadBuf (void *pParam,unsigned char *pMem,int iLen);
	static  void		UtcToDos   (time_t iUtcTime,unsigned short &uDate,unsigned short &uTime);
	static  time_t		DosToUtc   (unsigned uDate,unsigned uTime);
	static  int __cdecl	CmpPos     (const void *pPos1,const void *pPos2);
			int			ReadString (void *pBuffer,int iSize);
	virtual	int			ArchOpen   (int iLevel=9);
	virtual	int			CloseFlush ();




};

#endif



