//*****************************************************************************
//*
//*
//*		ChmFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
//
//	Klasse fr den Zugriff auf CHM-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int		Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen
//
#include	<stdio.h>
#include	<string.h>
#include	"ChmFile.h"
#include	"BitFunctions.h"
#include	"Codecs/Huffman.h"
#include	"Codecs/LzxClass.h"
#include	"Codecs/BzipClass.h"

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		wcslen
#define 	strzncpy	wcsncpy
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		strlen
#define 	strzncpy	strncpy
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif


#define PROG(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))throw CAF_ERROR_BREAK;	\
	}
#define PROGR(iMode,iNum,Ret)						\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))return Ret;				\
	}
#define PROGN(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	Progress(iMode,iNum);							\
	}


static const char *pSect0="::DataSpace/Storage/MSCompressed/Transform/{0A9007C6-4076-11D3-8789-0000F8105754}/InstanceData/ResetTable";
static const char *pSect1="::DataSpace/Storage/MSCompressed/Transform/{7FC28940-9D31-11D0-9B27-00A0C91E9C7C}/InstanceData/ResetTable";
static const char *pSect2="::DataSpace/Storage/MSCompressed/ControlData";
static const char *pSect3="::DataSpace/Storage/MSCompressed/Content";


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CChmFile::CChmFile()
{

	m_pFiles		=  0;
	m_pBlocks		=  0;
	m_iFilesNum		=  0;
	m_iDoneNow		=  0;
	m_iDoneMax		=  0;
	m_iDoneMode		=  0;
	m_cHasCompress	=  0;
	m_iDecompDone	=  0;
	m_pDecompMem	=  0;
	m_uDecompSize	=  0;
	m_uDecompAdd	=  0;
	m_iFileSize		=  0;
	m_uDecompMode	=  4;
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CChmFile::~CChmFile()
{

	if(m_iOpenMode!=CAF_CLOSED)Close();
	Clear();
}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Datenfelder und setzt alle Variabeln zurck
void CChmFile::Clear()
{

	if(m_pFiles)
		{
		while(m_iFilesNum>0)
			{
			m_iFilesNum--;
			delete m_pFiles[m_iFilesNum].pName;
			}

		delete m_pFiles;

		m_pFiles=NULL;
		}

	if(m_pBlocks)
		{
		delete m_pBlocks;
		m_pBlocks   = 0;
		}

	m_sFind.Set(0);

	m_iModeFlags	= 0;
	m_iFilesNum		= 0;
	m_iFilesMax		= 0;
	m_iDecompDone	= 0;
	m_uDecompSize	= 0;
	m_uDecompAdd	= 0;
	m_uDecompMode	= 4;

	m_uContentSec	= 0;
	m_uContentOff	= 0;
	m_uContentLen	= 0;

	m_uControlSec	= 0;
	m_uControlOff	= 0;
	m_uControlLen	= 0;

	m_uResetSec		= 0;
	m_uResetOff		= 0;
	m_uResetLen		= 0;


	if(m_pDecompMem)
		{
		delete m_pDecompMem;
		m_pDecompMem = NULL;
		}


}

//*****************************************************************************
//*
//*		CmpEx
//*
//*****************************************************************************
int __cdecl CChmFile::CmpEx(const void *pPos1,const void *pPos2)
{
ChmItem	*pPtr1,*pPtr2;


	pPtr1 = (ChmItem*)pPos1;
	pPtr2 = (ChmItem*)pPos2;


return strvicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpName
//*
//*****************************************************************************
int __cdecl CChmFile::CmpName(const void *pPos1,const void *pPos2)
{
ChmItem	*pPtr1,*pPtr2;


	pPtr1 = (ChmItem*)pPos1;
	pPtr2 = (ChmItem*)pPos2;


return strzicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpLen
//*
//*****************************************************************************
int __cdecl CChmFile::CmpLen(const void *pPos1,const void *pPos2)
{
ChmItem	*pPtr1,*pPtr2;


	pPtr1 = (ChmItem*)pPos1;
	pPtr2 = (ChmItem*)pPos2;


return pPtr1->uLenFull-pPtr2->uLenFull;
}

//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CChmFile::CmpPos(const void *pPos1,const void *pPos2)
{
ChmItem	*pPtr1,*pPtr2;


	pPtr1 = (ChmItem*)pPos1;
	pPtr2 = (ChmItem*)pPos2;


return (int)(pPtr1-pPtr2);
}

//*****************************************************************************
//*
//*		Len
//*
//*****************************************************************************
//	Ergibt die Lnge der Datei mit der Indexnummer iNum
//	Ist iMode==0 wird die originale    Lnge zurckgegeben
//	Ist iMode!=0 wird die komprimierte Lnge zurckgegeben
aint64 CChmFile::Len(int iNum,int iMode)
{
ChmItem	*pEntry;

	if(iNum<0 || iNum>=m_iFilesNum)return 0;
	pEntry=m_pFiles+iNum;

return (iMode)? pEntry->uLenComp:pEntry->uLenFull;
}

//*****************************************************************************
//*
//*		SetFail
//*
//*****************************************************************************
//	Setzt das Defekt-Flag und ergibt den alten Wert.
int CChmFile::SetFail(int iNum,int iFail)
{
int		iOld;

	if(iNum<0 || iNum>=m_iFilesNum)return -1;

	iOld = m_pFiles[iNum].cError;
	m_pFiles[iNum].cError = iFail;

return iOld;
}

//*****************************************************************************
//*
//*		IsFail
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit iNum beschdigt ist sonst 0
int CChmFile::IsFail(int iNum)
{

	if(iNum<0 || iNum>=m_iFilesNum)return -1;

return m_pFiles[iNum].cError;
}

//*****************************************************************************
//*
//*		IsDir
//*
//*****************************************************************************
// Ergibt die 1 wenn die Datei mit iNum ein Verzeichnis ist sonst 0
int CChmFile::IsDir(int iNum)
{

	if(iNum<0 || iNum>=m_iFilesNum)return 0;

return (m_pFiles[iNum].cDirectory)? 1:0;
}

//*****************************************************************************
//*
//*		IsOk
//*
//*****************************************************************************
// Ergibt:	2	Wenn die Datei vollstntig Entzipt wurde
//          1	Wenn die Datei schon vorhanden war
//			0   Bei Fehlern
int CChmFile::IsOk(int iNum)
{

	if(iNum<0 || iNum>=m_iFilesNum)return 0;

return m_pFiles[iNum].cDecomp;
}

//*****************************************************************************
//*
//*		Stats
//*
//*****************************************************************************
//	Die Zustandbits abfragen
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//	iDel		: Welche Bits sollen gelscht werden
//	Ergibt einen Wert mit veschiedenen Bits:
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//					CAF_STAT_DELETE		0x0004	Die Datei wurde aus dem Verzeichnis gelscht
//					CAF_STAT_WRITE		0x0008	Die Datei muss ins Archiv geschrieben werden
//					CAF_STAT_DIR		0x0010	Die Datei ist ein Verzeichniss
//					CAF_STAT_READONLY	0x0020	Die Datei ist schreibgeschtzt
//					CAF_STAT_SAVED		0x0040	Die Datei ist im Archiv gespeichert
int CChmFile::Stats(int iNum)
{
ChmItem	   *pEntry;
int			iBits;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return 0;
		}

			pEntry=m_pFiles+iNum;
	iBits = pEntry->cDecomp&3;

					      iBits|=CAF_STAT_SAVED;
	if(pEntry->cDirectory)iBits|=CAF_STAT_DIR;
	

return iBits;	
}


//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
// Ergibt die Zeit der letzten nderung einer Datei
// bzw.	CAF_ERROR_MISS wenn iNum/Datei nicht gefunden wurde.
time_t CChmFile::Time(int iNum)
{

	if(iNum<0 || iNum>=m_iFilesNum)return CAF_ERROR_MISS;

return m_iTime;
}

time_t CChmFile::Time(int iNum,int Mode)
{
	if(iNum<0 || iNum>=m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}

	switch(Mode)
		{
	case CAF_TIME_W:
	case CAF_TIME_C:
	case CAF_TIME_A: return m_iTime;
		}


return CAF_ERROR_MISS;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CChmFile::Name(int iNum)
{

	if((unsigned)iNum<(unsigned)m_iFilesNum)				// Prfe den Dateiindex
		{
		return m_pFiles[iNum].pName;
		}

	if(iNum==-1)											// Name der Archiv-Datei
		{
		return m_cFilePath;
		}


return NULL;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei samt Lnge.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CChmFile::Name(int iNum,int &iLen)
{

	if((unsigned)iNum<(unsigned)m_iFilesNum)				// Prfe den Dateiindex	
		{
		iLen = m_pFiles[iNum].uNameLen;
		return m_pFiles[iNum].pName;
		}

	if(iNum==-1)											// Name der Archiv-Datei
		{
		iLen = strzlen(m_cFilePath);					
		return m_cFilePath;
		}

	iLen = 0;


return NULL;
}

//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Liste in m_pFiles
//  iMode:	CAF_FLAG_SORT			0x0400	Nach Namen  sortieren
//			CAF_FLAG_SORTLEN	    0x0500	Nach Lngen sortieren
//			CAF_FLAG_SORTPOS	    0x0600	Nach Offset sortieren (intern)
//			CAF_FLAG_SORTEX			0x0700	Nach Namen mit Zahlen Bercksichtigung
int	CChmFile::Sort(int iMode)
{

		iMode&=0x0700;
	if(!iMode)iMode=m_iModeFlags&0x0700;
	iMode |= 0x0400;

	PROGR(CAF_PROG_SORT,-1,0);


	if(!m_pFiles)return 0;

	if(iMode==m_iSortMode)									// Ist die Liste bereits sortiert
		{
		return m_iFilesNum;
		}

	if(m_iFilesNum)
		{
		switch(iMode)
			{
		case    CAF_FLAG_SORTEX:	qsort(m_pFiles,m_iFilesNum,sizeof(ChmItem),CmpEx  );break;
		case    CAF_FLAG_SORTLEN:	qsort(m_pFiles,m_iFilesNum,sizeof(ChmItem),CmpLen );break;
		case    CAF_FLAG_SORTPOS:	qsort(m_pFiles,m_iFilesNum,sizeof(ChmItem),CmpPos );break;
		default:					qsort(m_pFiles,m_iFilesNum,sizeof(ChmItem),CmpName);break;
			}
		}

	m_iModeFlags&=~0x0300;
	m_iModeFlags|= 0x0300&iMode;
	m_iSortMode  = iMode;


return m_iFilesNum;
}

//*****************************************************************************
//*
//*		LoadDecompData
//*
//*****************************************************************************
void CChmFile::LoadDecompData()
{
ChmItem	   *pEntry;
unsigned	uOff;
unsigned	uMax;
int			iPos;
int			i;



	if(m_uCommpressed)
		{
		for(iPos=0;iPos<m_iFilesNum;iPos++)					// Komprimierte Lnge berechnen
			{
			pEntry = m_pFiles+iPos;

			if(pEntry->uItemSec==0)continue;
			if(pEntry->uLenFull==0)continue;

			uOff = pEntry->uOffset;
			uMax = pEntry->uLenFull+uOff;

			for(i=0;i<m_iFilesNum;i++)
				{
				if(i==iPos)continue;

				pEntry = m_pFiles+i;

				if(pEntry->uOffset<=uOff)continue;
				if(pEntry->uOffset>=uMax)continue;

				uMax = pEntry->uOffset;
				}

			pEntry->uLenComp = uMax-uOff;
			}


		if(m_uResetLen==0)throw CAF_ERROR_FORM;				// Reset-Tabele laden


		MainSeek(m_uBaseOffest+m_uResetOff,SEEK_SET);
		MainRead(&m_sTable,40);

		if(m_sTable.uBlockLen<1024 || m_sTable.uBlockLen>0x80000 || m_sTable.uVersion>3)
			{
			throw CAF_ERROR_FORM;
			}

			m_pBlocks = new aint64[m_sTable.uBlockCount+1];
		if(!m_pBlocks)throw CAF_ERROR_MEM;
		MainRead(m_pBlocks,8*m_sTable.uBlockCount);
		m_pBlocks[m_sTable.uBlockCount] = m_pBlocks[0]+m_sTable.uCompressedLen;


		if(!m_uControlOff)throw CAF_ERROR_FORM;				// Control-Daten laden

		MainSeek(m_uBaseOffest+m_uControlOff,SEEK_SET);
		MainRead(&m_sControl,28);

		if(memcmp(m_sControl.aSignature,"LZXC",4) || m_uControlSec!=0)
			{
			throw CAF_ERROR_FORM;
			}

		if(m_sControl.uVersion>=2)
			{
			m_sControl.uResetInterval;
			m_sControl.uWindowSize;
			}

		if(m_sControl.uWindowSize==0 || m_sControl.uWindowSize>16)
			{
			throw CAF_ERROR_FORM;
			}
		}
}

//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine CHM-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CChmFile::ArchOpen(int iLevel)
{
char				cBuffer[1024];
ChmItem			   *pOldList;
ChmItem			   *pEntry;
char			   *pPath;
unsigned			uChar;
unsigned 			uDirSize;
unsigned 			uDirChunks;
unsigned 			uQuickLen;
unsigned 			uNumItems;
unsigned 			uNameLen;
unsigned 			uStartOff;
unsigned 			uItemSec;
unsigned 			uItemOff;
unsigned 			uItemLen;
unsigned 			uEndOff;
unsigned 			uChunk;
unsigned			uOffset;
unsigned			uPos;
unsigned			uNum;
aint64				iSec2Off;
aint64				iContOff;
int					iByte;
int					iDone;
int					iSize;
int					iAdd;
int					i;




	Clear();

	m_iDoneMode		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= (int)m_iFileSize;
	m_iErrorCode	= CAF_ERROR_OK;

try	{

	MainRead(cBuffer,96);

	if(cBuffer[0]=='M' && cBuffer[1]=='Z' && cBuffer[2]==0 && cBuffer[3]==0)
		{
		OpenHxs(cBuffer);

		return CAF_ERROR_OK;
		}

	if(memcmp(cBuffer   ,"ITSF",4     ))throw CAF_ERROR_FORM;
	if(memcmp(cBuffer+ 4,"\x3\0\0\0",4))throw CAF_ERROR_FORM;
	if(memcmp(cBuffer+12,"\x1\0\0\0",4))
	if(memcmp(cBuffer+12,"\x0\0\0\0",4))throw CAF_ERROR_FORM;


	iSec2Off	= *(aint64*)(cBuffer+72);
	iContOff	= *(aint64*)(cBuffer+88);
	m_iDoneMax	= *(  int  *)(cBuffer+80)+84;
	m_iDoneNow	= 0;
	m_iDoneMode = 0;

	MainSeek((int)iSec2Off);
	MainRead(cBuffer,84);

	if(memcmp(cBuffer,"ITSP\x1\0\0\0\x54\0\0\0",12))throw CAF_ERROR_FORM;


	uDirSize = *(unsigned*)(cBuffer+16);

	if(uDirSize<32)
		{
		throw CAF_ERROR_FORM;
		}

	uDirChunks		= *(unsigned*)(cBuffer+44);
	m_uCommpressed	= 1;


	for(uChunk=0;uChunk<uDirChunks;uChunk++)				// Alle Chunks laden
		{
		uStartOff = (unsigned)MainTell();
		MainRead(cBuffer,4);


		if(memcmp(cBuffer,"PMGL",4))
			{
			MainSeek(uDirSize-4,SEEK_CUR);
			continue;
			}

		MainRead(cBuffer,16);
		uQuickLen = *(unsigned*)cBuffer;

		if(uQuickLen>uDirSize || uQuickLen<2)
			{
			throw CAF_ERROR_FORM;
			}


		uEndOff		= uStartOff+uDirSize-uQuickLen;
		iDone		= 0;
		iByte		= 0;

		for(uNumItems=0;;uNumItems++)
			{
			iSize	= iByte-iDone;

			if(iSize<512)									// Puffer nach fllen
				{
				memmove(cBuffer,cBuffer+iDone,iSize);
				uOffset  = (unsigned)MainTell();
				iByte	 = iSize;
				iDone	 = 0;

				   iSize = sizeof(cBuffer)-iByte;
				if(iSize>(int)(uEndOff-uOffset))
					{
					iSize = (int)(uEndOff-uOffset);
					}

				if(iSize>0)
					{
					MainRead(cBuffer+iByte,iSize);
					iByte   += iSize;
					uOffset += iSize;
					}

				if(uOffset>=uEndOff && iByte<=0)
					{
					if(uOffset==uEndOff)break;
					throw CAF_ERROR_FORM;
					}
				}


			uNameLen = (unsigned)GetEncInt(cBuffer+iDone,iAdd); iDone+=iAdd;

			if(uNameLen==0 || uNameLen>400)
				{
				throw CAF_ERROR_FORM;
				}

			pPath		= cBuffer+iDone;
			iDone      += uNameLen;

			uItemSec	= (unsigned)GetEncInt(cBuffer+iDone,iAdd); iDone+=iAdd;
			uItemOff 	= (unsigned)GetEncInt(cBuffer+iDone,iAdd); iDone+=iAdd;
			uItemLen	= (unsigned)GetEncInt(cBuffer+iDone,iAdd); iDone+=iAdd;

			if(!memcmp(pPath,"::DataSpace/",12))			// Speziele Bereiche
				{
				pPath[uNameLen]=0;
				uNameLen++;

				if(!memcmp(pPath,pSect1,uNameLen))
					{
					m_uResetSec	= uItemSec;
					m_uResetOff	= uItemOff;
					m_uResetLen	= uItemLen;
					continue;
					}

				if(!memcmp(pPath,pSect2,uNameLen))
					{
					m_uControlSec = uItemSec;
					m_uControlOff = uItemOff;
					m_uControlLen = uItemLen;
					if(uItemSec==1)m_uCommpressed=0;
					continue;
					}

				if(!memcmp(pPath,pSect3,uNameLen))
					{
					m_uContentSec = uItemSec;
					m_uContentOff = uItemOff;
					m_uContentLen = uItemLen;
					continue;
					}
				}
			else{
				if(m_iFilesNum>=m_iFilesMax)				// Eintrags Liste erweitern
					{
					i		= m_iFilesMax+m_iFilesMax/2+256;
					pEntry	= new ChmItem[i];
					if(!pEntry)throw CAF_ERROR_MEM;

					pOldList = m_pFiles;
					memcpy(pEntry,m_pFiles,m_iFilesNum*sizeof(ChmItem));
					m_pFiles = pEntry;
					delete pOldList;
					m_iFilesMax = i;
					}

				pEntry = m_pFiles+m_iFilesNum;
				memset(pEntry,0,sizeof(ChmItem));
				pEntry->pName = new zchar[uNameLen+1];
				if(!pEntry->pName)throw CAF_ERROR_MEM;

				uPos = (uNameLen>0 && *pPath=='/')? 1:0;

				for(uNum=0;uPos<uNameLen;)					// Namen kopieren
					{
					uPos +=utf8tochar(pPath+uPos,uChar);
					pEntry->pName[uNum] = (zchar)uChar;
					uNum++;
					}


				if(!uNum)continue;

				pEntry->pName[uNum] = 0;
				pEntry->uNameLen	= uNum;
				pEntry->uItemSec	= uItemSec;
				pEntry->uOffset 	= uItemOff;
				pEntry->uLenFull	= uItemLen;
				pEntry->uLenComp	= uItemLen;
				pEntry->cDirectory	= (pEntry->pName[uNum-1]=='/')? 1:0;
				m_iFilesNum	   += 1;
				}
			}

		MainSeek(uQuickLen-2,SEEK_CUR);
		MainRead(cBuffer,2);

		if(*(WORD*)cBuffer!=uNumItems)
			{
			throw CAF_ERROR_FORM;
			}
		}


	m_uBaseOffest	= (unsigned)iContOff;
	m_uDecompSize	= (unsigned)uOffset;
	m_uDecompAdd	= (unsigned)m_sTable.uCompressedLen;

	LoadDecompData();

	m_iDoneNow		= m_iDoneMax;
	}
catch(int iError)											// Bekannter Fehler
	{
	m_iErrorCode=iError;
	PROGN(CAF_PROG_ERROR,-1);
	Break();
	return iError;
	}

	if(m_iModeFlags&CAF_FLAG_SORT)Sort();					// Sortieren ?

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow	 = m_iDoneMax;



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einer Datei in der Dateiliste
//	pName	: Ist der  Dateiname nach dem gesucht wird
//	Ergibt die Indexnummer der Datei, bzw. -1 wenn die Datei nicht gefunden wurde.
int CChmFile::Find(const zchar *pName)
{
ChmItem	*pEntry;
unsigned uLen;
int		 iPos;


	if(*pName=='/' )pName++;
	if(*pName=='\\')pName++;


	uLen = (int)strzlen(pName);

	for(iPos=0;iPos<m_iFilesNum;iPos++)								// Eintrag in Kette
		{
		   pEntry=m_pFiles+iPos;
		if(pEntry->uNameLen!=uLen)continue;

		if(!CmpFileNameN(pName,pEntry->pName))
			{
			return iPos;
			}
		}


return -1;
}


//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
// Bearbeitung abbrechen ohne Daten zu speichern
void CChmFile::Break()
{

	ReleaseData();
	m_iOpenMode=CAF_CLOSED;
	Clear();

}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das ChmFile und Speichert alle Daten
//	Ergibt;	CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CChmFile::Close()
{


	m_iErrorCode=CAF_ERROR_OK;

	if(m_iOpenMode==CAF_CLOSED)
		{
		m_iOpenMode =CAF_CLOSED;
		m_iErrorCode=CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,-1);
		return     CAF_ERROR_MISS;
		}

	ReleaseData();
	m_iOpenMode=CAF_CLOSED;



return CAF_ERROR_OK;
}


//*****************************************************************************
//*
//*		ExtractProc
//*
//*****************************************************************************
//	Extrahiert eine Datei aus dem Archiv ber eine Ausgabefunktion
//	iNum	: Ist die Indexnummer der Datei die geprft werden soll
//	pParam	: Ist ein Parameter der an die Ausgabefunktionbergeben wird
//	pProc	: Ist der Zeiger auf die Ausgabefunktion
//	pInfo	: Hier werden Zeiten und Attribute der Datei gespeichert.
//			  Bei NULL werden diese Werte ignoriert.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_OK		= Wenn alles korrekt war
//				CAF_ERROR_MISS		= Wenn die Datei nicht gefunden wurde
//				CAF_ERROR_PATH		= Pfad ungltig ist.
//				CAF_ERROR_FCREATE	= Wenn Datei nicht erzeugt werden konnte
//				CAF_ERROR_CRC		= Crc32 Fehler
//				CAF_ERROR_DECOMP	= Fehler beim Entkomprimieren
//				CAF_ERROR_IGNORE	= Die Datei wurde bersprungen
//				... siehe Fehler Codes
int	CChmFile::ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo)
{
ChmItem		   *pEntry;
unsigned		uPos;
unsigned		uLen;
unsigned		uRet;


	if(m_iDoneMode<=1)										// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}

	m_iErrorCode = CAF_ERROR_OK;

	try	{
		if(m_iOpenMode==CAF_CLOSED              )throw CAF_ERROR_MISS;
		if((unsigned)iNum>=(unsigned)m_iFilesNum)throw CAF_ERROR_MISS;

			pEntry=m_pFiles+iNum;
		if(!pEntry)throw CAF_ERROR_MISS;

		if(m_iDoneMode==1)									// Verlaufsanzeige einstellen
			{
			m_iDoneNow   = 0;
			m_iDoneMax   = pEntry->uLenFull;
			}

		m_pErrorFile = pEntry->pName;

		if(pEntry->cError)
			{
			throw CAF_ERROR_DECOMP;
			}

		if(!pEntry->uItemSec)								// Unkomprimierte Daten
			{
			char	cBuffer[4096];

			MainSeek(pEntry->uOffset+m_uBaseOffest);

			for(uPos=0;uPos<pEntry->uLenFull;uPos+=uLen)	// Komprimierte Daten speichern
				{
				   uLen = 4096;
				if(uLen+uPos>pEntry->uLenFull)uLen=pEntry->uLenFull-uPos;

				if(MainRead(cBuffer,uLen)!=(int)uLen)
					{
					throw CAF_ERROR_FREAD;
					}

				   uRet = pProc(pParam,cBuffer,uLen);
				if(uRet!=uLen)
					{
					throw CAF_ERROR_FWRITE;
					}

				m_iDoneNow += uLen;
				}
			}
		else{
			if(!m_iDecompDone && pEntry->uItemSec)			// Alle Dateien entkomprimieren
				{
				m_iDoneMax += m_uDecompAdd;
				if(!Decompess())throw CAF_ERROR_FORM;
				}

			for(uPos=0;uPos<pEntry->uLenFull;uPos+=uLen)	// Daten speichern
				{
				   uLen = 4096;
				if(uLen+uPos>pEntry->uLenFull)uLen=pEntry->uLenFull-uPos;


				   uRet = pProc(pParam,m_pDecompMem+pEntry->uOffset+uPos,uLen);
				if(uRet!=uLen)
					{
					throw CAF_ERROR_FWRITE;
					}

				m_iDoneNow += uLen;
				}
			}
		
		if(pProc(pParam,NULL,0)!=0)							// Ende anzeigen
			{
			throw CAF_ERROR_FWRITE;
			}

		if(pInfo)
			{
			pInfo->dwAttr		=(pEntry->cDirectory)? SYS_DIR:SYS_READ|SYS_WRITE;
			pInfo->dwSize		= pEntry->uLenFull;
			pInfo->dwTimeWrite	= m_iTime;
			pInfo->dwTimeAccess	= 0;
			pInfo->dwTimeCreate	= 0;
			pInfo->cName[0]		= 0;
			}
		}
	catch(int iError)										// Fehler abfagen
		{
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}

	if(m_iDoneMode<=1)										// Die Verlaufsanzeige zurcksetzen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}



return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Count
//*
//*****************************************************************************
//	Ergibt die Anzahl der Eintrge in der Dateiliste
int CChmFile::Count()
{
	return m_iFilesNum;
}

//*****************************************************************************
//*
//*		Decompess
//*
//*****************************************************************************
//	Alle Dateien in eine Speicher endkomprimiern
//	Ergibt 1 wenn ok bzw. 0 bei einem Fehler
int CChmFile::Decompess()
{
CLzxClass		sDecomp;
unsigned		uOffset;
unsigned		uReset;
unsigned		uBit;
unsigned		uLen;
int				iOut;



	if(m_iOpenMode!=CAF_READ)return 0;
	if(m_iFilesNum   <=0    )return 0;
	if(m_uCommpressed==0	)return 0;
	if(m_iDecompDone        )return 1;


		m_pDecompMem = new char[(int)m_sTable.uUncompressedLen+16];
	if(!m_pDecompMem)throw CAF_ERROR_MEM;

	uBit	= bit_find_1(&m_sControl.uWindowSize,32)+1;
	uReset	= m_sControl.uResetInterval;

	if(!uReset)uReset=0x40000000;
	if(!m_sTable.uBlockCount)return 0;

	m_uDecompSize = 0;

	uOffset  = (unsigned)m_pBlocks[0]+m_uBaseOffest+m_uContentOff;
	uLen	 = (unsigned)m_sTable.uUncompressedLen;

	MainSeek(uOffset);

	m_cThrowRd = 0;

	sDecomp.SetIn (this,MainRead);
	sDecomp.SetOut(uLen,m_pDecompMem);

	try	{
		iOut = (int)sDecomp.Decompress(uLen,uBit,0,uReset);
		}
	catch(...)
		{
		m_cThrowRd = 1;
		return 0;
		}

	m_uDecompSize	= iOut;
	m_iDecompDone	= 1;
	m_cThrowRd		= 1;


return 1;
}

//*****************************************************************************
//*
//*		SetDoneMode
//*
//*****************************************************************************
//	Den Modus fr die Verlaufsanzeige setzen.
//	iMode	: Ist der neue Modus
//			    0 = Zurck zu Normal (Ende)
//				1 = Begin von Extract (einzelene Datei)
//				2 = Begin von ExtractAll
//	Ergibt den vorherigen Modus.
int CChmFile::SetDoneMode(int iMode)
{
int		iRet;

	
	iRet = m_iDoneMode;
	
	switch(iMode)
		{
	case 0:													// Zurck zu Normal (Ende)

		if(m_iDoneMode==1 || m_iDoneMode==2)
		if(m_iErrorCode==CAF_ERROR_OK)
			{
			m_iDoneNow  = m_iDoneMax;
			}

		m_iDoneMode	 = 0;
		break;

	case 1:													// Begin von Extract (einzelene Datei)

		if(iRet==2)return iRet;
		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 0;
		break;

	case 2:													// Begin von ExtractAll

		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 1;
		break;
		}


return iRet;
}





//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CChmFile::Done(aint64 *pMax)
{
aint64	iRet;


	switch(m_iDoneMode)
		{
	case 0:
	case 1:	iRet = m_iDoneNow;		break;
	default:iRet = 0;				break;
		}


	if(iRet>m_iDoneMax)iRet=m_iDoneMax;
	if(pMax)*pMax=m_iDoneMax;


return iRet;
}

//*****************************************************************************
//*
//*		GetUserPtr
//*
//*****************************************************************************
//	User-Pointer abfragen
void *CChmFile::GetUserPtr(int iNum)
{

	if(iNum<0 || iNum>=m_iFilesNum)return 0;

return m_pFiles[iNum].pUserPtr;
}

//*****************************************************************************
//*
//*		SetUserPtr
//*
//*****************************************************************************
//	User-Pointer einstellen
void CChmFile::SetUserPtr(int iNum,void *pData)
{

	if(iNum<0 || iNum>=m_iFilesNum)return;

	m_pFiles[iNum].pUserPtr = pData;

}

//*****************************************************************************
//*
//*		GetEncInt
//*
//*****************************************************************************
aint64 CChmFile::GetEncInt(const void *pBuffer,int &iLen)
{
unsigned char  *pData=(unsigned char*)pBuffer;
aint64			iValue;
unsigned		uByte;
int				iPos;


	iValue = 0;

	for(iPos=0;iPos<10;iPos++)
		{
		uByte   =  pData[iPos];
		iValue |= (uByte&0x7F);

		if(uByte<0x80)
			{
			iLen = iPos+1;
			return iValue;
			}

		iValue <<= 7;
		}

	throw CAF_ERROR_FORM;

}

//*****************************************************************************
//*
//*		ReadEncInt
//*
//*****************************************************************************
aint64 CChmFile::ReadEncInt()
{
unsigned char	aData[8];
aint64			iValue;
int				iLen;


	MainRead(aData,8);
	iValue = GetEncInt(aData,iLen);
	MainSeek(iLen-8,SEEK_CUR);

return iValue;
}

//*****************************************************************************
//*
//*		OpenHxs
//*
//*****************************************************************************
//	HtmlHelp2 Format ffnen.
//	pBuffer		: Zeiger auf einen 1024 Byte Puffer mit 96 geladen Bytes.
void CChmFile::OpenHxs(char *pBuffer)
{
ChmItem	   *pEntry;
ChmItem	   *pOldList;
unsigned   *pSections;
unsigned	uContent;
unsigned	uNameLen;
unsigned	uItemSec;
unsigned	uItemOff;
unsigned	uItemLen;
unsigned	uOffset;
unsigned	uChunks;
unsigned	uFormat;
unsigned	uItems;
unsigned	uDirs;
unsigned	uSize;
unsigned	uChar;
unsigned	uPos;
unsigned	uNum;
unsigned	uLen;
unsigned	uVal;



	MainRead(pBuffer+96,1024-96);
	uSize = 1024;

	for(uOffset=0;uOffset<0x8000;uOffset+=1016)				// Suche ITOLITLS Header
		{
		for(uPos=0;uPos<uSize;uPos++)
			{
			if(pBuffer[uPos]!='I')continue;
			if(!memcmp(pBuffer+uPos,"ITOLITLS",8))goto Found;
			}

		memcpy(pBuffer,pBuffer+1016,8);
		MainRead(pBuffer+8,1016);
		}

	throw CAF_ERROR_FORM;


Found:

	uOffset += uPos;

	MainSeek(uOffset+8);

	if(MainRead(pBuffer,276)!=276)
		{
		throw CAF_ERROR_FORM;
		}

	if(memcmp(pBuffer,"\x01\x00\x00\x00\x28\x00\x00\x00\x05\x00\x00\x00",12))
		{
		throw CAF_ERROR_FORM;
		}

	if(memcmp(pBuffer+264,"CAOL\x02\x00\x00\x00",8))
		{
		throw CAF_ERROR_FORM;
		}

	pSections =  (unsigned*)(pBuffer+ 32);
	uDirs	  = *(unsigned*)(pBuffer+176);
	uLen	  = *(unsigned*)(pBuffer+272);

	if(uLen>=44)
		{
		if(MainRead(pBuffer+276,32)!=32)
			{
			throw CAF_ERROR_FORM;
			}

		if(uLen==44)
			{
			uContent	= 0;
			uFormat		= 1;
			}
		else if(uLen==80)
			{
			uFormat		= 0;
			MainRead(pBuffer+276,48);

			if(memcmp(pBuffer+276+4,"ITSF\x04\x00\x00\x00\x20\x00\x00\x00",12))
				{
				throw CAF_ERROR_FORM;
				}

			uVal = *(unsigned*)(pBuffer+276+16);

			if(uVal>1)										// 0=HxW 1=HxS
				{
				throw CAF_ERROR_FORM;
				}

			uContent = *(unsigned*)(pBuffer+276+20);
			}
		else{
			throw CAF_ERROR_FORM;
			}
		}


	MainSeek(uOffset+pSections[4*1]);
	MainRead(pBuffer,32);

	if(memcmp(pBuffer,"IFCM\x01\x00\x00\x00",8))
		{
		throw CAF_ERROR_FORM;
		}

	uSize	= *(unsigned*)(pBuffer+8);
	uChunks	= *(unsigned*)(pBuffer+24);

	if(uSize<64)
		{
		throw CAF_ERROR_FORM;
		}


	for(uNum=0;uNum<uChunks;uNum++)
		{
		uPos = (unsigned)MainTell();

		MainRead(pBuffer,4);

		if(memcmp(pBuffer,"AOLL",4))
			{
			MainSeek(uSize-4,SEEK_CUR);
			continue;
			}

		MainRead(&uLen,4);

		if(uLen>uSize || uLen<2)
			{
			throw CAF_ERROR_FORM;
			}

		MainSeek(40,SEEK_CUR);
		uItems = 0;

		for(;;)												// Alle Eintge einlesen
			{
			unsigned	uMyPos;
			unsigned	uLimit;

			uMyPos = (unsigned)MainTell()-uPos;
			uLimit = uSize-uLen;

			if(uMyPos>uLimit)
				{
				throw CAF_ERROR_FORM;
				}

			if(uMyPos==uLimit)
				{
				break;
				}

			if(m_iFilesNum>=m_iFilesMax)					// Eintrags Liste erweitern
				{
				uVal	= m_iFilesMax+m_iFilesMax/2+256;
				pEntry	= new ChmItem[uVal];
				if(!pEntry)throw CAF_ERROR_MEM;

				pOldList = m_pFiles;
				memcpy(pEntry,m_pFiles,m_iFilesNum*sizeof(ChmItem));
				m_pFiles = pEntry;
				delete pOldList;
				m_iFilesMax = uVal;
				}


			if(uFormat)
				{
				uNameLen = 0;
				MainRead(&uNameLen,2);

				if(!uNameLen || uNameLen>512)
					{
					throw CAF_ERROR_FORM;
					}

				MainRead(pBuffer,uNameLen*2+1);

				uItemLen = (unsigned)ReadEncInt();
				MainRead(pBuffer,uNameLen);

				// REM AZ hier fehlt was
				throw CAF_ERROR_NOTSUPP;
				}

			else{
				uNameLen = (unsigned)ReadEncInt();

				if(!uNameLen || uNameLen>512)
					{
					throw CAF_ERROR_FORM;
					}

				MainRead(pBuffer,uNameLen);
				pBuffer[uNameLen]=0;

				uItemSec = (unsigned)ReadEncInt();
				uItemOff = (unsigned)ReadEncInt();
				uItemLen = (unsigned)ReadEncInt();

				if(!memcmp(pBuffer,"::DataSpace/",12))		// Speziele Bereiche
					{
					uNameLen++;
					uItems++;

					if(!memcmp(pBuffer,pSect0,uNameLen) ||
					   !memcmp(pBuffer,pSect1,uNameLen))
						{
						m_uResetSec	= uItemSec;
						m_uResetOff	= uItemOff;
						m_uResetLen	= uItemLen;
						continue;
						}

					if(!memcmp(pBuffer,pSect2,uNameLen))
						{
						m_uControlSec	= uItemSec;
						m_uControlOff	= uItemOff;
						m_uControlLen	= uItemLen;
						if(uItemSec==1)m_uCommpressed=0;
						continue;
						}

					if(!memcmp(pBuffer,pSect3,uNameLen))
						{
						m_uContentSec = uItemSec;
						m_uContentOff = uItemOff;
						m_uContentLen = uItemLen;
						continue;
						}

					continue;
					}

				pEntry   = m_pFiles+m_iFilesNum;
				memset(pEntry,0,sizeof(ChmItem));
				pEntry->pName = new zchar[uNameLen+1];
				if(!pEntry->pName)throw CAF_ERROR_MEM;

				uVal = (uNameLen>0 && *pBuffer=='/')? 1:0;

				for(uNum=0;uVal<uNameLen;)					// Namen kopieren
					{
					uVal +=utf8tochar(pBuffer+uVal,uChar);
					pEntry->pName[uNum] = (zchar)uChar;
					uNum++;
					}

				if(!uNum)continue;

				pEntry->pName[uNum] = 0;
				pEntry->uNameLen	= uNum;
				pEntry->uItemSec	= uItemSec;
				pEntry->uOffset 	= uItemOff;
				pEntry->uLenFull	= uItemLen;
				pEntry->uLenComp	= uItemLen;
				pEntry->cDirectory	= (pEntry->pName[uNum-1]=='/')? 1:0;
				m_iFilesNum	   += 1;
				}

			uItems++;
			}

		MainSeek(uLen-2,SEEK_CUR);							// Quick-Referenz berspringen

		uVal = 0;
		MainRead(&uVal,2);

		if(uItems+1<uVal || uItems>uVal || uItems>uDirs)
			{
			throw CAF_ERROR_FORM;
			}


		uDirs -= uItems;
		}


	m_uBaseOffest	= uOffset+uContent;
	m_uCommpressed	= 1;

	LoadDecompData();


}
