//*****************************************************************************
//*
//*
//*		ChmFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
//
//	Klasse fr den Zugriff auf CHM-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int		Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef		__CHMFILE_H__
#define     __CHMFILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	"FileFinder.h"
#include	"FileFunctions.h"
#include	"ArchiveFile.h"
#include	"Codecs/Huffman.h"
#include	"Filter/Crc32.h"


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif



//*****************************************************************************
//*
//*		Klasse CChmFile
//*
//*****************************************************************************
class CChmFile : public CArchiveFile
{
protected:

	typedef struct 
	{
    unsigned		uVersion;
    unsigned		uBlockCount;
    unsigned		uUnknown;
    unsigned		uTableOffset;
    auint64			uUncompressedLen;
    auint64			uCompressedLen;
    auint64			uBlockLen;
	}ChmResetTable;

	typedef struct ChmControlData
	{
    unsigned		uSize; 
    char			aSignature[4];
    unsigned		uVersion; 
    unsigned		uResetInterval;
    unsigned		uWindowSize;
    unsigned		uWindowsPerReset;
    unsigned		uUnknown;
	}ChmControlData;
	
	typedef struct 
	{
	void		   *pUserPtr;					// User-Pointer
	zchar		   *pName;       				// File name in zip file
	unsigned		uNameLen;					// Dateinamen-Lnge
	unsigned		uLenComp;					// Dateilnge komrimiert
	unsigned		uLenFull;					// Dateilnge unkomrimiert
	unsigned		uOffset;					// Offset in Datei der komprimierten Daten
	char 			cError;						// Ist Block fehlerhaft
	char 			cDirectory;					// Ist die Datei ein Verzeichnis
	char 			cDecomp;					// Ist Block richtig endkomprimiert
	unsigned		uItemSec;					// Seite im Datenstrom
	}ChmItem;


	ChmControlData	m_sControl;					// Control Daten
	ChmResetTable	m_sTable;					// Reset Tabele
	CFileFinder		m_sFind;					// Klasse fr Dateisuche
	time_t			m_iTime;					// Zeit der letzten nderung

	ChmItem        *m_pFiles;					// Alfabethische Liste
	aint64		   *m_pBlocks;					// Liste mit Block-Offsets	
	int				m_iFilesMax;				// Listen Eintrge reservierte Eintrge
	int				m_iFilesNum;				// Listen Eintrge ohne gelschte
	int				m_iDecompDone;				// Wurde bereits alles entpackt
	char		   *m_pDecompMem;				// Speicher mit allen entpackten Dateien
	unsigned		m_uDecompAdd;				// Ist die unkomprimierte Gre der Daten
	unsigned		m_uDecompSize;				// Entpackegre aller Dateien
	unsigned		m_uDecompMode;				// 0=none 1=gzip 2=bzip
	unsigned		m_uDecompOffset;			// Position der Komprimierten Daten in der Datei
	unsigned		m_uCommpressed;				// Ist die CHM Datei komprimiert

	int				m_iDoneNow;					// Aktueller Verlauf
	int				m_iDoneMax;					// Maximaler Verlauf
	int				m_iDoneMode;				// 0=List von m_iDoneNow 1=Extraxt 2=ExtractAll 3=Flush

	unsigned		m_uBaseOffest;				// Offset der Dateiobjekte

	unsigned		m_uContentSec;				// Sektion der komprimierten Daten
	unsigned		m_uContentOff;				// Offset  der komprimierten Daten
	unsigned		m_uContentLen;				// Lnge   der komprimierten Daten

	unsigned 		m_uControlSec;				// Sektion der Control-Daten
	unsigned 		m_uControlOff;				// Offset  der Control-Daten
	unsigned 		m_uControlLen;				// Lnge   der Control-Daten

	unsigned 		m_uResetSec;				// Sektion der Reset-Tabelle
	unsigned 		m_uResetOff;				// Offset  der Reset-Tabelle
	unsigned 		m_uResetLen;				// Lnge   der Reset-Tabelle


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

					CChmFile();
	virtual		   ~CChmFile();

	virtual	int		ExtractProc	(int iNum,void *pParam,OutProc pProc,FileData *pInfo);
	virtual	int		Close		();						// Datei schliesen
	virtual	int	    Count  		();						// Anzahl der Dateien
	virtual	void	Break  		();						// Bearbeitung abbrechen
	virtual	int		Sort   		(int  iMode=0);			// Sortierte Kette
	virtual	zchar  *Name   		(int  iNum);			// Hole Dateiname
	virtual	zchar  *Name   		(int  iNum,int &iLen);	// Hole Dateiname samt Lnge
	virtual	int     SetFail		(int  iNum,int iFail=1);// Defekt-Flag setzen
	virtual	int     Stats		(int  iNum);			// Zustansbits abfragen
	virtual	int     IsFail 		(int  iNum);			// Ist die Datei defekt
	virtual	int     IsDir  		(int  iNum);			// Prfe ob Verzeichnis
	virtual	int     IsOk   		(int  iNum);			// Richtig endkomprimiert
	virtual	aint64	Len    		(int  iNum,int iMode=0);// Hole Dateilnge
	virtual	time_t	Time   		(int  iNum);			// Dateizeit abfragen
	virtual	time_t	Time   		(int  iNum,int iMode);	// Dateizeit abfragen
	virtual	int		Find   		(const zchar  *pName);	// Datei suchen
	virtual	aint64	Done   		(aint64 *pMax);			// Verlaufsabfrage
	virtual	void   *GetUserPtr	(int iNum            );	// Datenwert auslesen
	virtual	void    SetUserPtr	(int iNum,void *pData);	// Datenwert einstellen


protected:

	virtual	int		ArchOpen(int iLevel=9);				// Eine CHM-Datei ffnen
	virtual	int		SetDoneMode(int iMode);				// Den Done-Modus setzen
	void			Clear();							// Speicher lschen
	int				Decompess();						// Alle Dateien in eine Speicher endkomprimiern
	void			OpenHxs(char *pBuffer);				// HtmlHelp2 Format ffnen
	void			LoadDecompData();	

static int  __cdecl CmpName    (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int  __cdecl CmpEx      (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int  __cdecl CmpLen     (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int  __cdecl CmpPos     (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion

static aint64		GetEncInt  (const void *pBuffer,int &iLen);
	   aint64		ReadEncInt ();
};


#endif



