//*****************************************************************************
//*
//*
//*		BzipClass.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//	Klasse zum Komprimieren von Daten mit dem BZIP Algorithmus.
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned GetFlags(int)						: Flags setzen/abfragen
// 	unsigned GetLevel(int)						: Level setzen/abfragen
// 	unsigned GetError()							: Fehler Code holen
// 	unsigned GetMode ()							: Modus setzen/abfragen
//
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iMode =-2);			: Starte Dekomprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//

#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif

#include	<stdio.h>
#include	<string.h>
#include	"BzipLib.h"
#include	"BzipClass.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


#define 	CBZ_BUFFER_SIZE		0x2000
#define 	CBZ_FILE_BUF		0x2000

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CBzipClass::CBzipClass()
{

	m_pTempBuffer	= NULL;							// Temporrer interne Puffer
	m_pStreamData	= NULL;							// bz_stream daten fr Algorithmus

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CBzipClass::~CBzipClass()
{
	Free();
}

//*****************************************************************************
//*
//*		Free
//*
//*****************************************************************************
//	Alle Puffer freigeben.
void CBzipClass::Free()
{

	if(m_pTempBuffer)
		{
		delete m_pTempBuffer;
		m_pTempBuffer = NULL;
		}

	if(m_pStreamData)
		{
		delete m_pStreamData;
		m_pStreamData = NULL;
		}

	Clear(2);

}

//*****************************************************************************
//*
//*		Store
//*
//*****************************************************************************
//	Daten eins zu eins laden oder speichern
iocint64 CBzipClass::Store()
{
char	cTemp[4];
char   *pBuffer;
int		iCount;
int		iBytes;


	if(m_uOutMode==IOC_MEM)							// Direkt in Speicher einlesen
		{
		for(;;)
			{
			if(OutputEof())
				{
				   iCount=Read(cTemp,sizeof(cTemp));
				if(iCount>0)						// Sind noch Daten im Stream
					{
					m_iErrorCode = CBZ_ERROR_OVERFLOW;
					throw "Fehler Speicherberlauf des Ausgabepuffers";
					}
				}
			else{
				iCount = ReadOut();					// Daten in den Ausgabepuffer
				}

			if(iCount==0)							// Stream-Ende erreicht
				{
				break;
				}

			if(iCount<0)							// Lesefehler entdeckt ?
				{
				m_iErrorCode = CBZ_ERROR_READ;
				throw "Fehler beim Lesen";
				}
			}

		m_uDecompresLen = m_uOutOffset;
		m_uCompresLen	= m_uOutOffset;
		}
	else{											// In Stream speichern
		pBuffer = GetBuffer();
		iBytes	= GetBufLen();

		for(;;)										// Mit normaler Ausgabe
			{
			   iCount = Read(pBuffer,iBytes);
			if(iCount==0)
				{
				break;								// Stream-Ende erreicht
				}

			if(iCount<0)							// Fehler beim Lesen ?
				{
				m_iErrorCode = CBZ_ERROR_READ;
				throw "Fehler beim Lesen";
				}

			if(Write(pBuffer,iCount)!=iCount)		// Daten in Stream speichern
				{
				m_iErrorCode = CBZ_ERROR_WRITE;
				throw "Fehler beim Schreiben in die Datei";
				}

			m_uCompresLen   += iCount;
			m_uDecompresLen += iCount;
			}
		}


return m_uDecompresLen;
}

//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
//	Startet die Endkomprimierung eines Streams
//	iMode		: Bestimmt die Endkomprimierungsart
//					-2			= aktuellen Modus verwenden
//					CBZ_STORE	= Daten eins zu eins laden
//					CBZ_DEFLATE	= Daten entkomprimieren
//	Ergibt die Anzahl der Entkomprimierten Bytes
iocint64 CBzipClass::Decompress(int iMode)
{
bz_stream  *pStream;
char	   *pOutput;
int			iCount;
int			iRead;
int			iRet;


	IOC_ASSERT(m_uOutMode<CBZ_NONE && m_uInpMode<CBZ_NONE);


	if(iMode==-2)									// Aktuellen Modus verwenden
		{
		iMode = m_iMode;
		}

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CBZ_ERROR_OK;

	Reset();


	if(iMode==CBZ_STORE)
		{
		return Store();								// Daten eins zu eins laden
		}


//******************** Daten entkomprimieren **********************************


	if(iMode!=CBZ_DEFLATE)							// Prfe Modus
		{
		m_iErrorCode = CBZ_ERROR_MODE;
		throw "Falscher Modus beim Entkomprimieren !";
		}

	if(!m_pStreamData)
		{
			m_pStreamData = new char[sizeof(bz_stream)];
		if(!m_pStreamData)
			{
			m_iErrorCode = CBZ_ERROR_MEMORY;
			throw "Zu wenig Speicher";
			}
		}

	pStream = (bz_stream*)m_pStreamData;			// Stream erzeugen
	pStream->bzalloc	= NULL;
	pStream->bzfree		= NULL;
	pStream->opaque		= NULL;
	pStream->avail_in	= 0;

	   iRet = Bz2_BzDecompressInit(pStream,0,0);
	if(iRet!= BZ_OK)
		{
		m_iErrorCode = CBZ_ERROR_BADBZIP;
		throw "Fehler bei Initialisierung";
		};


	if(m_pTempBuffer==0)							// Gibt es den temporren Puffer
		{
			m_pTempBuffer = new char[CBZ_BUFFER_SIZE+16];
		if(!m_pTempBuffer)
			{
			m_iErrorCode = CBZ_ERROR_MEMORY;
			throw "Zu wenig Speicher";
			}
		}


	iRet  = BZ_OK;
	iRead = 1;


	for(;;)
		{
		if(!pStream->avail_in)						// Wieder Bytes einlesen
			{
			   iCount=Read(m_pTempBuffer,CBZ_BUFFER_SIZE);
			if(iCount<0)
				{
				Bz2_BzDecompressEnd(pStream);
				m_iErrorCode = CBZ_ERROR_READ;
				throw "Fehler beim Lesen";
				}

			m_uCompresLen += iCount;

			if(iCount==0)							// Wurde alles eingelesen
				{
				if(iRet==BZ_STREAM_END)break;
				if(iRet==BZ_OK && iRead==0 && pStream->avail_in==0 && pStream->avail_out>0)
					{
					m_iErrorCode = CBZ_ERROR_BADBZIP;
					throw "Ungltiges BZIP Format";
					}

				if(iRet==BZ_OK && iRead==0)break;
				}

			pStream->avail_in	= iCount;
			pStream->next_in	= m_pTempBuffer;
			}
		else{										// Es sind noch Daten im Puffer
			iCount				= pStream->avail_in;

			if(iCount==8 && iRet==BZ_STREAM_END)	// CRC ignorieren
				{
				break;
				}
			}


		if(m_uOutMode==IOC_MEM)						// Direkt in Speicher endpacken
			{
			pStream->avail_out	= GetOutLen();
			pStream->next_out	= GetOutMem();
			}
		else{										// Puffer fr's Entkomprimieren holen
			pStream->avail_out	= GetBufLen();
			pStream->next_out	= GetBuffer();
			}

		pOutput = pStream->next_out;

		   iRet = Bz2_BzDecompress(pStream);
		if(iRet!= BZ_OK && iRet!=BZ_STREAM_END)
			{
			Bz2_BzDecompressEnd(pStream);
			m_iErrorCode = CBZ_ERROR_BADBZIP;
			throw "Ungltiges BZIP Format";
			}


		if(m_uOutMode==IOC_MEM)						// Direkt in Speicher entpacken
			{
			iRead			= pStream->next_out-pOutput;
			m_uDecompresLen = TakeOut(iRead);

			if(OutputEof() && iRet!=BZ_STREAM_END)
				{
				Bz2_BzDecompressEnd(pStream);
				m_iErrorCode = CBZ_ERROR_OVERFLOW;
				throw "Fehler Speicherberlauf des Ausgabepuffers";
				}
			}
		else{
			   iRead = pStream->next_out-pOutput;
			if(iRead>0 && Write(pOutput,iRead)!=iRead)
				{
				m_iErrorCode = CBZ_ERROR_WRITE;
				throw "Fehler beim Schreiben in die Datei";
				}

			m_uDecompresLen += iRead;
			}
		}

	Bz2_BzDecompressEnd(pStream);



return m_uDecompresLen;
}


//*****************************************************************************
//*
//*		Compress
//*
//*****************************************************************************
//	Startet die Komprimierung eines Streams
//	iLevel		: Bestimmt den Kompimierungslevel (0..9)
//					-1	= aktuellen Level verwenden
//					 9	= maximale Kompimierung
//					 0	= minimale Kompimierung (keine STORE)
//	Ergibt die Anzahl der Entkomprimierten Bytes
iocint64 CBzipClass::Compress(int iLevel)
{
bz_stream  *pStream;
char	   *pOutput;
int			iAction;
int			iCount;
int			iRead;
int			iRet;



	IOC_ASSERT(m_uOutMode<CBZ_NONE && m_uInpMode<CBZ_NONE);


	if(iLevel     != -1)m_iModeLevel=iLevel;		// Kompimierungslevel
	if(m_iModeLevel<  0)m_iModeLevel=0;
	if(m_iModeLevel>  9)m_iModeLevel=9;

		iLevel=m_iModeLevel;
	if(!iLevel)iLevel=1;

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CBZ_ERROR_OK;
	m_iModeFlags   &=~CBZ_NOCOMPESS;

	Reset();


//******************** Daten entkomprimieren **********************************


	if(!m_pStreamData)
		{
		m_pStreamData = new char[sizeof(bz_stream)];
		}

	pStream = (bz_stream*)m_pStreamData;			// Stream erzeugen
	pStream->bzalloc	= NULL;
	pStream->bzfree		= NULL;
	pStream->opaque		= NULL;
	pStream->avail_in	= 0;

	   iRet = Bz2_BzCompressInit(pStream,iLevel,0,0);
	if(iRet!= BZ_OK)
		{
		m_iErrorCode = CBZ_ERROR_BADBZIP;
		throw "Fehler bei Initialisierung";
		};


	if(m_pTempBuffer==0)							// Gibt es den temporren Puffer
		{
			m_pTempBuffer = new char[CBZ_BUFFER_SIZE+4];
		if(!m_pTempBuffer)throw "Zu wenig Speicher";
		}

	iAction =  BZ_RUN;
	iRet    =  BZ_OK;
	iRead	= -1;

	for(;;)											// Komprimierschleife
		{
		if(iAction==BZ_RUN)
			{
			if(pStream->avail_in<=0)
				{
				   iCount=Read(m_pTempBuffer,CBZ_BUFFER_SIZE);
				if(iCount<0)
					{
					Bz2_BzCompressEnd(pStream);
					m_iErrorCode = CBZ_ERROR_READ;
					throw "Fehler beim Lesen";
					}

				if(iCount==0)						// Alles bertragen
					{
					   iAction=BZ_FINISH;
					if(iRet  ==BZ_STREAM_END    )break;
					if(iRet  ==BZ_OK && iRead==0)break;
					}

				m_uDecompresLen	   += iCount;
				pStream->avail_in	= iCount;
				pStream->next_in	= m_pTempBuffer;
				}
			else{
				iCount				= pStream->avail_in;
				}
			}


		if(m_uOutMode==IOC_MEM)						// Direkt in Speicher packen
			{
			pStream->avail_out	= GetOutLen();
			pStream->next_out	= GetOutMem();
			}
		else{										// Puffer fr's Komprimieren holen
			pStream->avail_out	= GetBufLen();
			pStream->next_out	= GetBuffer();
			}

		pOutput = pStream->next_out;

		   iRet=Bz2_BzCompress(pStream,iAction);
		if(iRet<BZ_OK)
			{
			Bz2_BzCompressEnd(pStream);
			m_iErrorCode = CBZ_ERROR_BADBZIP;
			throw "Fehler beim Komprmieren";
			};


		if(m_uOutMode==IOC_MEM)						// Direkt in Speicher packen
			{
			iRead		  = pStream->next_out-pOutput;
			m_uCompresLen = TakeOut(iRead);

			if(OutputEof() && iRet!=BZ_STREAM_END)
				{
				Bz2_BzCompressEnd(pStream);
				m_iErrorCode = CBZ_ERROR_OVERFLOW;
				throw "Fehler Speicherberlauf des Ausgabepuffers";
				}
			}
		else{
			   iRead = pStream->next_out-pOutput;
			if(iRead>0 && Write(pOutput,iRead)!=iRead)
				{
				m_iErrorCode = CBZ_ERROR_WRITE;
				throw "Fehler beim Schreiben in die Datei";
				}

			m_uCompresLen += iRead;
			}

		if(iRet==BZ_STREAM_END)break;
		}


	Bz2_BzCompressEnd(pStream);						// Stream anhalten

	if(m_uCompresLen>m_uDecompresLen)				// Ziel grer Quelle
		{
		m_iModeFlags |= CBZ_NOCOMPESS;
		}



return m_uCompresLen;
}


