//*****************************************************************************
//*
//*
//*		BzipClass.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//	Klasse zum Komprimieren von Daten mit dem BZIP Algorithmus.
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned GetFlags(int)						: Flags setzen/abfragen
// 	unsigned GetLevel(int)						: Level setzen/abfragen
// 	unsigned GetError()							: Fehler Code holen
// 	unsigned GetMode ()							: Modus setzen/abfragen
//
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iMode =-2);			: Starte Dekomprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//

#ifndef		__CBZIPCLASS_H__
#define		__CBZIPCLASS_H__

#if _MSC_VER >= 1000
#pragma once
#endif

#include	 "IoCallbacks.h"



//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define CBZ_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CBZ_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CBZ_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CBZ_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CBZ_STORE			CBC_STORE			// Nur Gleich abspeichern

#define CBZ_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CBZ_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CBZ_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CBZ_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CBZ_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CBZ_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CBZ_ERROR_BADBZIP  	CBC_ERROR_BADFORMAT	// Ungltige Bzip2-Format
#define CBZ_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zuwenig Speicher

#define CBZ_LEVEL			CBC_LEVEL			// Level ausgeben
#define CBZ_FLAG			CBC_FLAG			// Flag Ausgeben
#define CBZ_MODE			CBC_MODE			// Modus ausgeben
#define CBZ_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CBZ_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CBZ_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CBZ_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CBZ_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CBZ_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CBZ_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes	
#define CBZ_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CBZ_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes	
#define CBZ_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)

#define CBZ_NONE			IOC_NONE			// Ein/Ausgabe noch nicht definiert	
#define CBZ_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CBZ_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CBZ_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CBZ_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher


//*****************************************************************************
//*
//*		Klasse CBzipClass
//*
//*****************************************************************************
class CBzipClass : public CBaseCompressor
{
protected:

	char       *m_pTempBuffer;				// Temporrer interne Puffer
	char       *m_pStreamData;				// bz_stream Daten fr Algorithmus


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************

public:

					 CBzipClass();
	virtual	        ~CBzipClass();

	virtual	void	 Free      ();
	virtual	iocint64 Compress  (int iLevel=-1);
	virtual	iocint64 Decompress(int iMode =-2);

protected:

	iocint64 Store();

};



#endif
