//*****************************************************************************
//*
//*
//*		CopyClass.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//	Klasse zum Komprimieren von Daten mit dem LZX Algorithmus
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
//
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iOutLen,int iMode);	: Starte Dekomprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<stdio.h>
#include	<string.h>
#include	<malloc.h>
#include	"CopyClass.h"
#include	"../Filter/Crc32.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	CPY_FILE_BUF				0x1000

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CCopyClass::CCopyClass()
{

	m_iDataBytes	= -1;

	Clear(1);
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CCopyClass::~CCopyClass()
{

	Clear(0);

}

//*****************************************************************************
//*
//*		Free
//*
//*****************************************************************************
//	Alle Puffer lschen
void CCopyClass::Free()
{
	Clear(2);
}


//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//  0 :	Alles rcksetzen
//  1 :	Nur Variabeln rcksetzen , Puffer nicht lschen
//  2 :	Nur Puffer lschen
void CCopyClass::Clear(int iMode)
{

	CIoCallbacks::Clear(iMode);

	if(iMode==2)return;

	m_uDecompresLen	= 0;								// Anzahl der Entkomprimierten Bytes
	m_uCompresLen	= 0;								// Anzahl der Komprimierten Bytes
	m_iErrorCode	= 0;								// Aktuelle Fehlernummer (siehe CPY_ERROR_???)

}


//*****************************************************************************
//*
//*		Compress
//*
//*****************************************************************************
//	Startet die Komprimierung eines Streams
//	iLevel		: Bestimmt den Kompimierungslevel (1..
//					-1	= aktuellen Level verwenden
//					 7	= maximale Framegre
//					 0	= minimale Framegre
//	Ergibt die Anzahl der komprimierten Bytes
iocint64 CCopyClass::Compress(int iLevel)
{
unsigned char		aBuffer[CPY_FILE_BUF];
iocint64			iOffset;
iocint64			iCount;
int 				iByte;



	IOC_ASSERT(m_uOutMode<CPY_NONE && m_uInpMode<CPY_NONE);


	if(iLevel     != -1)m_iModeLevel=iLevel;			// Kompimierungslevel
	if(m_iModeLevel<  1)m_iModeLevel=1;
	if(m_iModeLevel>  7)m_iModeLevel=7;

	Clear(2);
	Reset ();
	InitRd();
	InitWr();

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CPY_ERROR_OK;

	for(iOffset=0;;iOffset+=iByte)
		{
		if(m_iDataBytes<0)								// Unbekanntes Ende
			{
			iCount = sizeof(aBuffer);
			}
		else{										
			   iCount = m_iDataBytes-iOffset;
			if(iCount<=0)break;

			if(iCount>sizeof(aBuffer))					// Ende erreicht
				{
				iCount = sizeof(aBuffer);
				}
			}

		iByte = GetData(aBuffer,(int)iCount);			// Daten lesen
		
		if(iByte<0)
			{
			m_iErrorCode = CPY_ERROR_READ;
			throw "Fehler beim Lesen";
			}
		
		if(iByte==0)
			{
			break;
			}
			
		if(PutData(aBuffer,iByte))						// Daten schreiben
			{
			m_iErrorCode = CPY_ERROR_WRITE;
			throw "Fehler beim Schreiben";
			}
		}


	if(Flush())
		{
		m_iErrorCode = CPY_ERROR_WRITE;
		throw "Fehler beim Schreiben";
		}

	m_uDecompresLen	= GetPosInp();
	m_uCompresLen	= GetPosOut();



return m_uCompresLen;
}


//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
//	Startet die Endkomprimierung eines Streams
//	iMode		: Bestimmt die Endkomprimierungsart
//					-2			= aktuellen Modus verwenden
//					CBZ_STORE	= Daten eins zu eins laden
//					CBZ_DEFLATE	= Daten entkomprimieren
//	Ergibt die Anzahl der Entkomprimierten Bytes
iocint64 CCopyClass::Decompress(int iMode)
{
unsigned char		aBuffer[CPY_FILE_BUF];
iocint64			iOffset;
iocint64			iCount;
int 				iByte;



	IOC_ASSERT(m_uOutMode<CPY_NONE && m_uInpMode<CPY_NONE);


	Clear(2);
	Reset ();
	InitRd();
	InitWr();


	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CPY_ERROR_OK;


	for(iOffset=0;;iOffset+=iByte)
		{
		if(m_iDataBytes<0)								// Unbekanntes Ende
			{
			iCount = sizeof(aBuffer);
			}
		else{										
			   iCount = m_iDataBytes-iOffset;
			if(iCount<=0)break;

			if(iCount>sizeof(aBuffer))					// Ende erreicht
				{
				iCount = sizeof(aBuffer);
				}
			}

		iByte = GetData(aBuffer,(int)iCount);			// Daten lesen
		
		if(iByte<0)
			{
			m_iErrorCode = CPY_ERROR_READ;
			throw "Fehler beim Lesen";
			}
		
		if(iByte==0)
			{
			break;
			}
			
		if(PutData(aBuffer,iByte))						// Daten schreiben
			{
			m_iErrorCode = CPY_ERROR_WRITE;
			throw "Fehler beim Schreiben";
			}
		}


	if(Flush())
		{
		m_iErrorCode = CPY_ERROR_WRITE;
		throw "Fehler beim Schreiben";
		}

	m_uDecompresLen	= GetPosOut();
	m_uCompresLen	= GetPosInp();



return m_uDecompresLen;
}

