//*****************************************************************************
//*
//*
//*		CopyClass.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2013
//
//	Klasse zum Komprimieren von Daten mit dem COPY-Algorithmus
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
//
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iOutLen,int iMode);	: Starte Dekomprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//

#ifndef		__COPYCLASS_H__
#define		__COPYCLASS_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#include	 "IoCallbacks.h"



//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define CPY_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CPY_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CPY_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CPY_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CPY_STORE			CBC_STORE			// Nur Gleich abspeichern

#define CPY_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CPY_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CPY_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CPY_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CPY_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CPY_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CPY_ERROR_BADFORMAT	CBC_ERROR_BADFORMAT	// Ungltige Bzip2-Format
#define CPY_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zuwenig Speicher

#define CPY_LEVEL			CBC_LEVEL			// Level ausgeben
#define CPY_FLAG			CBC_FLAG			// Flag Ausgeben
#define CPY_MODE			CBC_MODE			// Modus ausgeben
#define CPY_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CPY_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CPY_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CPY_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CPY_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CPY_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CPY_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes
#define CPY_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CPY_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes
#define CPY_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)

#define CPY_NONE			IOC_NONE			// Ein/Ausgabe noch nicht definiert
#define CPY_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CPY_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CPY_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CPY_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher


//*****************************************************************************
//*
//*		Klasse CCopyClass
//*
//*****************************************************************************
class CCopyClass : public CBaseCompressor
{
public:

						CCopyClass();
	virtual			   ~CCopyClass();

	virtual	void		Free ();
	virtual	iocint64	Compress  (int iLevel=-1);
	virtual	iocint64	Decompress(int iMode =-2);
	void				SetDataSize(iocint64 iSize);

protected:

	void			Clear(int iMode=0);
	iocint64		m_iDataBytes;				// Ausgabelnge (-1=Ende unbekannt)

};

//*****************************************************************************
//*
//*		Inline Funktioen
//*
//*****************************************************************************
inline void CCopyClass::SetDataSize(iocint64 iSize){m_iDataBytes=iSize;}


#endif 	