//*****************************************************************************
//*
//*
//*		Huffman.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Klasse zum Komprimieren von Daten
//
// 	unsigned Value(int)											: Wert  abfargen
// 	unsigned GetFlags(int)										: Flags setzen/abfragen
// 	unsigned GetLevel(int)										: Level setzen/abfragen
// 	unsigned GetError()											: Fehler Code holen
// 	unsigned GetMode ()											: Modus setzen/abfragen
//
//	int		 Decompress(int Mode=-2);							: Starte Dekomprimierung
// 	int		 Compress()											: Starte   Komprimierung
//
// 	void	 SetOut(HNADLE,int,unsigned)						: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)						: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,char*)								: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,char*)								: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe

#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif

#include	"Huffman.h"
#include	"../ZipLib/zlib.h"
#ifdef		CHF_HAS_DEFALTE64
#include	"../Delate64/infback9.h"
#endif


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CHuffman::CHuffman()
{

}

//*****************************************************************************
//*
//*		Destrucktor
//*
//*****************************************************************************
CHuffman::~CHuffman()
{
	Free();
}

//*****************************************************************************
//*
//*		Free
//*
//*****************************************************************************
//	Alle Puffer freigeben.
void CHuffman::Free()
{
	Clear(2);
}

//*****************************************************************************
//*
//*		Deflate64Get
//*
//*****************************************************************************
//	Eingabefunktion fr Deflate64
//	pParam		: Zeiger auf den This-Pointer
//	pPtrBuf		: Hier wird der Zeiger auf den Eingangspuffer gespeichert
//	Ergibt die Zahl der gelesenen Bytes.
unsigned CHuffman::Deflate64Get(void *pParam, unsigned char **pBuffPtr)
{
CHuffman 	   *pThis=(CHuffman*)pParam;
unsigned char  *pBuffer;
int				iBytes;


	if(pThis->m_uInpMode==CHF_MEM)					// Direkt aus dem Speicher holen
		{
		if(pBuffPtr)*pBuffPtr=(unsigned char*)pThis->GetInpMem();
		iBytes = pThis->GetInpLen();
		pThis->TakeInp(iBytes);
		return iBytes;
		}


	   pBuffer = (unsigned char*)pThis->GetBuffer();
    if(pBuffPtr)*pBuffPtr = pBuffer;


	   iBytes = pThis->GetBufLen();
       iBytes = pThis->Read(pBuffer,iBytes);		// Den Lesepuffer fllen
	if(iBytes<0)
        {
		pThis->m_iLastIo = 0;
		return 0;
		}


return iBytes;
}

//*****************************************************************************
//*
//*		Deflate64Put
//*
//*****************************************************************************
//	Ausgabefunktion fr Deflate64
//	pParam		: Zeiger auf den This-Pointer
//	pBuffer		: Ist der Ausgabepuffer
//	uSize		: Anzahl der zu schreibenden Bytes
//	Ergibt 1 bei einem Fehler bzw. 0 wenn ok.
int CHuffman::Deflate64Put(void *pParam,unsigned char *pBuffer,unsigned uSize)
{
CHuffman   *pThis=(CHuffman*)pParam;
unsigned 	uCount;


	uCount = pThis->Write(pBuffer,uSize);

	if(uCount==uSize)
		{
		return 0;
		}

	pThis->m_iLastIo = 1;


return 1;
}

//*****************************************************************************
//*
//*		Store
//*
//*****************************************************************************
//	Daten eins zu eins speichern oder laden.
//	Ergibt die Lnge dieser Daten in Byte.
iocint64 CHuffman::Store()
{
char   *pOutput;
char	cTemp[4];
int		iBytes;
int		iCount;


	m_iModeFlags |= CHF_FAST;


	if(m_uOutMode==IOC_MEM)								// Direkt in Speicher einlesen
		{
		for(;;)
			{
			if(OutputEof())
				{
				   iCount=Read(cTemp,sizeof(cTemp));
				if(iCount>0)							// Sind noch Daten im Stream
					{
					m_iErrorCode = CHF_ERROR_OVERFLOW;
					throw "Fehler Speicherberlauf des Ausgabepuffers";
					}
				}
			else{
				iCount = ReadOut();						// Daten in den Ausgabepuffer
				}

			if(iCount==0)								// Stream-Ende erreicht
				{
				break;
				}

			if(iCount<0)								// Lesefehler entdeckt ?
				{
				m_iErrorCode = CHF_ERROR_READ;
				throw "Fehler beim Lesen";
				}
			}

		m_uDecompresLen = m_uOutOffset;
		m_uCompresLen	= m_uOutOffset;
		}
	else{												// In Stream ausgeben
		pOutput = GetBuffer();
		iBytes  = GetBufLen();

		for(;;)
			{
			   iCount = Read(pOutput,iBytes);
			if(iCount==0)
				{
				break;									// Stream-Ende erreicht
				}

			if(iCount<0)								// Fehler beim Lesen ?
				{
				m_iErrorCode = CHF_ERROR_READ;
				throw "Fehler beim Lesen";
				}

			if(Write(pOutput,iCount)!=iCount)			// Daten in Stream speichern
				{
				m_iErrorCode = CHF_ERROR_WRITE;
				throw "Fehler beim Schreiben in die Datei";
				}

			m_uDecompresLen += iCount;
			m_uCompresLen   += iCount;
			}
		}


return m_uCompresLen;
}

//*****************************************************************************
//*
//*		Deflate64
//*
//*****************************************************************************
//	Daten im Deflate64-Format entpacken.
//	Ergibt die Lnge dieser entpackten Daten in Byte.
#ifdef CHF_HAS_DEFALTE64
iocint64 CHuffman::Deflate64()
{
z_stream		sStream;
unsigned char  *pOutput;
unsigned char  *pInput;
int				iBytes;
int				iRet;



	m_iLastIo	= -1;
	pOutput		= new unsigned char[0x10008];
	iRet		= inflateBack9Init(&sStream,pOutput);

	if(iRet!=Z_OK)
		{
		if(pOutput)delete pOutput;
		m_iErrorCode = CHF_ERROR_MEMORY;
		throw "Zu wenig Speicher";
		}

	if(m_uInpMode==CHF_MEM)							// Direkt aus dem Speicher lesen
		{
		pInput = (unsigned char*)GetInpMem();
		iBytes =                 GetInpLen();
		TakeInp(iBytes);
		}
	else{
		pInput = (unsigned char*)GetBuffer();
		}

	sStream.avail_in = 0;
	sStream.next_in  = pInput;
													// Daten entpacken
	iRet = inflateBack9(&sStream,Deflate64Get,this,Deflate64Put,this);

	inflateBack9End(&sStream);
	if(pOutput)delete pOutput;


	switch(iRet)									// Fehlercode auswerten
		{
	case Z_STREAM_END:		break;
	case Z_MEM_ERROR:		m_iErrorCode = CHF_ERROR_MEMORY;
							throw "Zu wenig Speicher";

	case Z_BUF_ERROR:	if(m_iLastIo> 0)
							{
							m_iErrorCode = CHF_ERROR_WRITE;
							throw "Fehler beim Schreiben in Datei !";
							}

						if(m_iLastIo==0)
							{
							m_iErrorCode = CHF_ERROR_READ;
							throw "Fehler beim Lesen aus Datei !";
							}

	default:				m_iErrorCode = CHF_ERROR_BADHUF;
							throw "Ungldige Huffmann-Tabelle !";
		}

	m_uDecompresLen	= GetPosOut();
	m_uCompresLen	= GetPosInp();


return m_uDecompresLen;
}
#endif


//*****************************************************************************
//*
//*		Commpress
//*
//*****************************************************************************
//	Beginnt mit der Kompression und ergibt die komprimierte Lnge.
//	Fehler  werden mit throw(char *Text) ausgegeben.
//	Fehlernummern knnen mit GetError() geholt werden.
//	Ein- und Ausgaben mssen zuvor mit SetIn/SetOut eingestellt werden.
//	iLevel		: Ist der Komprimierungslevel
//						-1		= Aktuellen Level benutzen
//						 0		= Store (eins zu eins)
//						 1..9	= Komprimiren (9=maximal)
iocint64 CHuffman::Compress(int iLevel)
{
unsigned char	aInput[0x4010];
unsigned char  *pOutput;
z_stream		sStream;
int				iFlush;
int				iBytes;
int				iErr;
int				iRet;



	if(iLevel    !=-1)m_iModeLevel=iLevel;				// Kompimierungs Level
	if(m_iModeLevel<0)m_iModeLevel=0;
	if(m_iModeLevel>9)m_iModeLevel=9;

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CHF_ERROR_OK;
	m_iMode			= (m_iModeLevel)? CHF_DEFLATE:CHF_STORE;
	m_iModeFlags	= 0;

	Reset();


	if(m_iModeLevel==CHF_STORE)							// Daten eins zu eins speichern
		{
		return Store();
		}


//******************** Mit Komprimierung **************************************


    sStream.zalloc	= (alloc_func)0;
    sStream.zfree	= (free_func )0;
    sStream.opaque	= 0;

	if(m_uOutMode==CHF_MEM)								// Direkt in Speicher packen
		{
		pOutput				= (unsigned char*)GetOutMem();
		sStream.avail_out	=                 GetOutLen();
		sStream.next_out	= (unsigned char*)pOutput;
		}
	else{												// Puffer fr's Entkomprimieren holen
		pOutput				= (unsigned char*)GetBuffer();
		sStream.avail_out	=                 GetBufLen();
		sStream.next_out	= (unsigned char*)pOutput;
		}


       iErr = deflateInit2(&sStream,m_iModeLevel,m_iMode,-15,Z_DEFLATED,0);
    if(iErr!=Z_OK)
		{
		deflateEnd(&sStream);
		m_iErrorCode = CHF_ERROR_MODE;
		throw "Initialisierungsfehler";
		}


	iFlush	= Z_NO_FLUSH;
	iBytes	= 0;
	iRet	= 0;


	while(iFlush!=Z_FINISH)								// Schleife frs Komprimieren
		{
		if(m_uInpMode==CHF_MEM)
			{
			sStream.next_in	 = (unsigned char*)GetInpMem();
			sStream.avail_in = GetInpLen();
			TakeInp(sStream.avail_in);
			}
		else{
			sStream.next_in  = (unsigned char*)aInput;
			sStream.avail_in = Read(aInput,0x4000);
			}


		m_uDecompresLen += sStream.avail_in;
		iBytes			 = sStream.avail_in;


		if(iBytes==0)									// Ende der Eingabe erreicht
			{
			iFlush = Z_FINISH;
			}
		else if(iBytes<0)								// Einlesefehler bei Eingabe
			{
			deflateEnd(&sStream);
			m_iErrorCode = CHF_ERROR_READ;
			throw "Fehler beim Lesen aus Datei !";
			}


		do	{
			if(sStream.avail_out==0 || (iFlush!=Z_NO_FLUSH && (iFlush!=Z_FINISH || iRet==Z_STREAM_END)))
				{
				   iBytes = (int)(sStream.next_out-pOutput);
				if(iBytes>0)
					{
					if(m_uOutMode==CHF_MEM)				// Daten von Speicherausgabe bernehmen
						{
						TakeOut(iBytes);
						}								// Daten in Stream ausgeben
					else if(Write(pOutput,iBytes)!=iBytes)
						{
						deflateEnd(&sStream);
						m_iErrorCode = CHF_ERROR_WRITE;
						throw "Fehler beim Schreiben in Datei !";
						}
					}

				m_uCompresLen += iBytes;

				if(sStream.avail_out==0)				// Ausgabepuffer nachladen
					{
					if(m_uOutMode==CHF_MEM)
						{
						if(OutputEof())
							{
							deflateEnd(&sStream);
							m_iErrorCode = CHF_ERROR_OVERFLOW;
							throw "Fehler Speicherberlauf des Ausgabepuffers";
							}

						pOutput				= (unsigned char*)GetOutMem();
						sStream.avail_out	=                 GetOutLen();
						sStream.next_out	= (unsigned char*)pOutput;
						}
					else{
						pOutput				= (unsigned char*)GetBuffer();
						sStream.avail_out	=                 GetBufLen();
						sStream.next_out	= (unsigned char*)pOutput;
						}
					}
				}


			iBytes	= sStream.avail_out;
			iRet	= deflate(&sStream,iFlush);			// Komprimiere die Daten
			iBytes -= sStream.avail_out;


			if(iRet==Z_STREAM_ERROR)					// Gab es einen Fehler
				{
				deflateEnd(&sStream);
				m_iErrorCode = CHF_ERROR_BADHUF;
				throw "Fehler beim Komprimieren";
				}

			}while(iBytes);
	    }



	deflateEnd(&sStream);


	if(m_uCompresLen>m_uDecompresLen)					// Ziel grer Quelle
		{
		m_iModeFlags |= CHF_NOCOMPESS;
		}

	if(m_uOutMode!=CHF_MEM)								// Das Ende signalisieren
		{
		Write(NULL,0);
		}


return (int)m_uCompresLen;
}

//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
//	Dekomprimiert einen Huffamn Stream
//	iMode		: Ist der Dekomprimier-Modus
//						-2		= Aktuellen Modus benutzen
//						 0		= Store-Modus
//						 8		= Deflate-Modus
//						 9		= Deflate64-Modus
//	Ergibt die Lnge der komprimierten Daten
iocint64 CHuffman::Decompress(int iMode)
{
unsigned char	aInput[0x4010];
unsigned char  *pInput;
unsigned char  *pOutput;
z_stream		sStream;
int				iFlush;
int				iBytes;
int				iDone;
int				iErr;
int				iRet;



	if(iMode==-2)iMode=m_iMode;

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;

	Reset();

	if(iMode==CHF_STORE)								// Daten eins zu eins laden
		{
		return Store();
		}


	#if CHF_HAS_DEFALTE64

	if(iMode==CHF_DEFLATE64)							// Daten sind Defalte64
		{
		return Deflate64();
		}

	#endif

	if(iMode!=CHF_DEFLATE)								// Prfe Modus
		{
		m_iErrorCode = CHF_ERROR_MODE;
		throw "Falscher Modus beim Entkomprimieren !";
		}


//******************** Mit Komprimierung **************************************


    sStream.zalloc		= (alloc_func)0;
    sStream.zfree		= (free_func )0;
    sStream.opaque		= 0;
	sStream.avail_in	= 0;


	if(m_uOutMode==CHF_MEM)								// Direkt in Speicher entpacken
		{
		sStream.next_out  = (unsigned char*)GetOutMem();
		sStream.avail_out =					GetOutLen();
		}
	else{												// In Stream speichern
		sStream.next_out  = (unsigned char*)GetBuffer();
		sStream.avail_out =                 GetBufLen();
		}


       iErr = inflateInit2(&sStream,-MAX_WBITS);
    if(iErr!=Z_OK)
		{
		inflateEnd(&sStream);
		m_iErrorCode = CHF_ERROR_MODE;
		throw "Initialisierungsfehler";
		}

	pOutput	= sStream.next_out;
	iFlush	= Z_SYNC_FLUSH;
	iBytes	= 0;
	iDone	= 0;
	iRet	= 0;


	for(;;)
		{
        if(sStream.avail_in==0 && !iDone)				// Daten nachladen
			{
			if(m_uInpMode==CHF_MEM)						// Daten direkt aus Speicher holen
				{
				pInput = (unsigned char*)GetInpMem();
				iBytes =                 GetInpLen();
				TakeInp(iBytes);
				}
			else{										// Daten aus den Stream einlesen
				pInput = aInput;
				iBytes = Read(aInput,0x4000);
				}

			if(iBytes==0)								// Alles eingelesen
				{
				iDone = 1;
				}
			else if(iBytes<0)							// Fehler beim Einlesen
				{
				inflateEnd(&sStream);
				m_iErrorCode = CHF_ERROR_READ;
				throw "Fehler beim Lesen aus Datei !";
				}

			sStream.next_in   = pInput;
			sStream.avail_in  = iBytes;
			m_uCompresLen    += iBytes;
			}

		if(sStream.avail_out==0) 						// Ausgabepuffer nachladen
			{
			if(m_uOutMode==CHF_MEM)
				{
				if(OutputEof())
					{
					inflateEnd(&sStream);
					m_iErrorCode = CHF_ERROR_OVERFLOW;
					throw "Fehler Speicherberlauf des Ausgabepuffers";
					}

				sStream.next_out  = (unsigned char*)GetOutMem();
				sStream.avail_out =					GetOutLen();
				}
			else{
				sStream.next_out  = (unsigned char*)GetBuffer();
				sStream.avail_out =                 GetBufLen();
				}
			}


		pOutput = sStream.next_out;
		iBytes	= sStream.avail_out;
		iErr    = inflate(&sStream,iFlush);				// Daten dekomprimieren
		iBytes -= sStream.avail_out;


		if(iBytes>0)									// Daten ausgeben
			{
 			if(m_uOutMode==CHF_MEM)
				{
				TakeOut(iBytes);						// Daten von Speicherausgabe bernehmen
				}
			else{
				if(Write(pOutput,iBytes)!=iBytes)		// Daten in Stream ausgeben
					{
					inflateEnd(&sStream);
					m_iErrorCode = CHF_ERROR_WRITE;
					throw "Fehler beim Schreiben in Datei !";
					}
				}

			m_uDecompresLen += iBytes;
			}

		if(iErr==Z_STREAM_END)							// Ende der Huffman-Daten
			{
            break;
			}

		if(iErr!=Z_OK)
			{
			inflateEnd(&sStream);
			m_iErrorCode = CHF_ERROR_BADHUF;
			throw "Ungldige Huffmann-Tabelle !";
			}
		}


	inflateEnd(&sStream);


	if(m_uOutMode!=CHF_MEM)								// Das Ende signalisieren
		{
		Write(NULL,0);
		}



return m_uDecompresLen;
}



