//*****************************************************************************
//*
//*
//*		Huffman.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Klasse zum Komprimieren von Daten
//
// 	unsigned Value(int)											: Wert  abfargen
// 	unsigned GetFlags(int)										: Flags setzen/abfragen
// 	unsigned GetLevel(int)										: Level setzen/abfragen
// 	unsigned GetError()											: Fehler Code holen
// 	unsigned GetMode ()											: Modus setzen/abfragen
//
//	int		 Decompress(int Mode=-2);							: Starte Dekomprimierung
// 	int		 Compress()											: Starte   Komprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)						: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)						: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)								: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)								: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe


#ifndef	__HUFFMAN_H__
#define	__HUFFMAN_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#include	"IoCallbacks.h"


//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define CHF_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CHF_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CHF_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CHF_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CHF_STORE			CBC_STORE			// Nur Gleich abspeichern
#define CHF_DEFLATE64		9					// Deflate64 Format	

#define CHF_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CHF_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CHF_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CHF_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CHF_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CHF_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CHF_ERROR_BADHUF  	CBC_ERROR_BADFORMAT	// Ungltige Huffman-Format
#define CHF_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zu wenig SPeicher
#define CHF_ERROR_PARAM  	CBC_ERROR_PARAM		// Falscher Parameter

#define CHF_LEVEL			CBC_LEVEL			// Level ausgeben
#define CHF_FLAG			CBC_FLAG			// Flag Ausgeben
#define CHF_MODE			CBC_MODE			// Modus ausgeben
#define CHF_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CHF_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CHF_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CHF_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CHF_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CHF_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CHF_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes	
#define CHF_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CHF_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes	
#define CHF_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
												
#define CHF_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CHF_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CHF_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CHF_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher

//*****************************************************************************
//*
//*		Klasse CHuffman
//*
//*****************************************************************************
class CHuffman : public CBaseCompressor
{
public:

					 CHuffman();
	virtual		    ~CHuffman();

	virtual	void	 Free      ();
	virtual	iocint64 Compress  (int iLevel=-1);
	virtual	iocint64 Decompress(int iMode =-2);

protected:

	int				 m_iLastIo;					// 0=Lesen 1=Schreiben
	iocint64		 Store();
	iocint64		 Deflate64();

	static	unsigned Deflate64Get(void *pParam,unsigned char **pBuffPtr);
	static	int		 Deflate64Put(void *pParam,unsigned char  *pBuffer,unsigned uSize);

};


#endif
