//*****************************************************************************
//*
//*
//*		Huffman1.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Klasse zum Komprimieren von Daten
//
// 	unsigned Value(int)					: Wert  abfargen
// 	unsigned Flags(int)					: Flags setzen/abfragen
// 	unsigned Level(int)					: Level setzen/abfragen
// 	unsigned Error()					: Fehler Code holen
// 	unsigned Mode ()					: Modus setzen/abfragen
//
//	int		 Decompress(int Mode=-2);	: Starte Dekomprimierung
// 	int		 Compress()					: Starte   Komprimierung
//
// 	void	 SetOut(HNADLE,int,unsigned): Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned): Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,char*)		: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,char*)		: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*Funk)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*Funk)(void*,char*,unsigned))	: Externe Eingabe


#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "Huffman.h"
#include "../Filter/Crc32.h"



#define STORED            0		// compression methods
#define SHRUNK            1		// unshrink(pG)
#define REDUCED1          2		// unreduce(pG)
#define REDUCED2          3				 //
#define REDUCED3          4				 //
#define REDUCED4          5				 //
#define IMPLODED          6		//:int explode(pG)
#define TOKENIZED         7
#define DEFLATED          8		// Inflate(pG)
#define ENHDEFLATED       9
#define DCLIMPLODED      10
#define NUM_METHODS      11		// index of last method + 1



//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************

											// Tables fordeflate from PKZIP's appnote.txt.
const unsigned CHuffman::border[] =			// Order of the bit length code lengths
		{
        16, 17, 18, 0, 8, 7, 9, 6, 10, 5,
		11, 4, 12, 3, 13, 2, 14, 1, 15
		};
const unsigned CHuffman::cplens[] =			// Copy lengths forliteral codes 257..285
		{
        3, 4, 5, 6, 7,  8,  9, 10, 11, 13, 15, 17,
		19, 23, 27, 31,35, 43, 51, 59, 67, 83, 99,
		115, 131, 163, 195, 227, 258, 0, 0
		};									// note: see note #13 above about the 258 in this list.
const unsigned CHuffman::cplext[] =			// Extra bits forliteral codes 257..285
		{
        0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,
		2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4,
		5, 5, 5, 5, 0, 99, 99
		};									// 99==invalid
const unsigned CHuffman::cpdist[] =
		{									// Copy offsets fordistance codes 0..29
        1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49,
		65, 97, 129, 193,  257, 385, 513, 769,
		1025, 1537, 2049, 3073, 4097, 6145,
        8193, 12289, 16385, 24577
		};
const unsigned CHuffman::cpdext[] =			// Extra bits fordistance codes
		{
        0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4,
		5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10,
		11, 11, 12, 12, 13, 13
		};
const unsigned CHuffman::near mask_bits[] =
		{
		0x0000,	0x0001, 0x0003, 0x0007, 0x000F, 0x001F, 0x003F, 0x007F, 0x00FF,
		0x01FF, 0x03FF, 0x07FF, 0x0FFF, 0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF
		};


#define NEXTBYTE     GetByte()
#define NEEDBITS(n) {while(k<(n)){b|=((unsigned)GetByte())<<k;k+=8;}}
#define DUMPBITS(n) {b>>=(n);k-=(n);}

#define BMAX			 16		// maximum bit length of any code (16 forexplode)
#define N_MAX			288     // maximum number of codes in any set



static const int lbits = 9;    // bits in base literal/length lookup table
static const int dbits = 6;    // bits in base distance lookup table


//*****************************************************************************
//*
//*		GetByte
//*
//*****************************************************************************
unsigned CHuffman::GetByte()
{
int		anz;


	if(in_mode==CHF_MEM)						// Speicher Eingabe ?
		{
		if(in_offset>=in_size)
			{
			error_nr=CHF_ERROR_EOF;
			throw	"Speicher-Ende erreicht beim Lesen !";
			}
					compres_len++;
		return in_buf[in_offset++];
		}

	if(get_offset>=get_size)					// Puffer fllen
		{
		anz=read_buf(isubmit,(char*)get_buf,CHF_GET_BUF);
		if(anz<=0)
			{
			error_nr=CHF_ERROR_EOF;
			throw	"Dateiende erreicht beim Lesen!";
			}
		get_offset=0;
		get_size=anz;
		}

	compres_len++;


return get_buf[get_offset++];
}
//*****************************************************************************
//*
//*		FlushInflate
//*
//*****************************************************************************
void CHuffman::FlushInflate(unsigned Anz)
{
	if(out_mode!=CHF_MEM)
		{
		if(write_buf(osubmit,(char *) window,Anz)!=(int)Anz)
			{
			error_nr=CHF_ERROR_WRITE;
			throw	"Fehler beim Schreiben in Datei !";
			}
		decompres_len+=Anz;
		}
	else{
		if(Anz>out_size-out_offset)
			{
			memcpy(out_buf+out_offset,window,out_size-out_offset);
			decompres_len=out_size;
			   out_offset=out_size;
			error_nr=CHF_ERROR_OVERFLOW;
			throw	"Fehler berlauf des Ausgabepuffers !";
			}
		memcpy(out_buf+out_offset,window,Anz);
		decompres_len+=Anz;
		   out_offset+=Anz;
		}

}

//*****************************************************************************
//*
//*		InflateCodes
//*
//*****************************************************************************
// huft *tl, *td;		literal/length and distance decoder tables
// int bl, bd;          number of bits decoded by tl[] and td[]
// Inflate (decompress) the codes in a deflated (compressed) block.
// Return an error code or zero ifit all goes ok.
int CHuffman::InflateCodes(huft *tl,huft *td,int  bl,int  bd)
{
register unsigned e;				// table entry flag/number of extra bits
unsigned n, d;						// length and index forcopy
unsigned w;							// current window position
		 huft *t;					// pointer to table entry
unsigned ml, md;					// masks forbl and bd bits
register unsigned b;				// bit buffer
register unsigned k;				// number of bits in bit buffer


									// make local copies of globals
	b = bb;							// initialize bit buffer
	k = bk;
	w = wp;							// initialize window position


									// Inflate the coded data
	ml = mask_bits[bl];				// precompute masks forspeed
	md = mask_bits[bd];
	while (1)						// do until end of block
		{
		NEEDBITS((unsigned)bl)
		if((e = (t = tl + ((unsigned)b & ml))->e) > 16)
		do	{
			if(e == 99)return 1;
			DUMPBITS(t->b)
			e -= 16;
			NEEDBITS(e)
			}
		while ((e = (t = t->v.t + ((unsigned)b & mask_bits[e]))->e) > 16);

		DUMPBITS(t->b)
		if(e == 16)					// then it's a literal
			{
			window[w++] = (unsigned char)t->v.n;
			if(w == WSIZE)
				{
      			FlushInflate(w);					//  out window
				w = 0;
				}
			}
		else{						// it's an EOB or a length
			if(e == 15)break;		// exit ifend of block
			NEEDBITS(e)				// get length of block to copy
			n = t->v.n + ((unsigned)b & mask_bits[e]);
			DUMPBITS(e);
			NEEDBITS((unsigned)bd)	// decode distance of block to copy
			if((e = (t = td + ((unsigned)b & md))->e) > 16)
			do	{
				if(e == 99)return 1;
				DUMPBITS(t->b)
				e -= 16;
				NEEDBITS(e)
				}
			while ((e = (t = t->v.t + ((unsigned)b & mask_bits[e]))->e) > 16);
			DUMPBITS(t->b)
			NEEDBITS(e)
			d = w - t->v.n - ((unsigned)b & mask_bits[e]);
			DUMPBITS(e)


			do	{					// do the copy
				n -= (e = (e = WSIZE - ((d &= WSIZE-1) > w ? d : w)) > n ? n : e);
				if(w - d >= e)		// (this test assumes unsigned comparison)
					{
					memcpy(window + w, window + d, e);
					w += e;
					d += e;
					}
				else do				// do it slowly to avoid memcpy() overlap
					{
					window[w++] = window[d++];
					}
				while (--e);
				if(w == WSIZE)
					{
					FlushInflate(w);						//  out window
					w = 0;
					}
				}
			while (n);
			}
		}


	// restore the globals from the locals
	wp = w;                       // restore global window pointer
	bb = b;                       // restore global bit buffer
	bk = k;

return 0;
}
//*****************************************************************************
//*
//*		InflateStored
//*
//*****************************************************************************
// "decompress" an inflated type 0 (stored) block.
int CHuffman::InflateStored()
{
unsigned n;						// number of bytes in block
unsigned w;						// current window position
register unsigned b;		   	// bit buffer
register unsigned k;			// number of bits in bit buffer



								// make local copies of globals
	b = bb;						// initialize bit buffer
	k = bk;
	w = wp;						// initialize window position

	n = k & 7;					// go to byte boundary /
	DUMPBITS(n);

	NEEDBITS(16)				// get the length and its complement
	n = ((unsigned)b & 0xFFFF);
	DUMPBITS(16)
	NEEDBITS(16)
	if(n != (unsigned)((~b) & 0xFFFF))
		return 1;              // error in compressed data
	DUMPBITS(16)


	while (n--)					// read and output the compressed data
		{
		NEEDBITS(8)
		window[w++] = (unsigned char)b;
		if(w == WSIZE)
			{
			FlushInflate(w);					//  out window
			w = 0;
			}
		DUMPBITS(8)
		}


								// restore the globals from the locals
	wp = w;						// restore global window pointer
	bb = b;						// restore global bit buffer
	bk = k;

return 0;
}

//*****************************************************************************
//*
//*		InflateFixed
//*
//*****************************************************************************
//    decompress an inflated type 1 (fixed Huffman codes) block.  We should
//    either replace this with a custom decoder, or at least precompute the
//    Huffman tables.
int CHuffman::InflateFixed()
{


	if(fixed_tl == 0)
		{
		int i;								// temporary variable
		unsigned l[288];					// length list forHuftBuild

											// literal table
		for(i = 0; i < 144; i++)l[i] = 8;
		for(     ; i < 256; i++)l[i] = 9;
		for(     ; i < 280; i++)l[i] = 7;
		for(     ; i < 288; i++)l[i] = 8;	// make a complete, but wrong code set

		fixed_bl = 7;
		i = HuftBuild(l, 288, 257, cplens, cplext,&fixed_tl, &fixed_bl);
		if(i)
			{
			fixed_tl = 0;
			return i;
			}

											// distance table
		for(i = 0; i < 30; i++)l[i] = 5;	// make an incomplete code set

		fixed_bd = 5;
		i = HuftBuild(l, 30, 0, cpdist, cpdext,&fixed_td, &fixed_bd);
		if(i > 1)
			{
			if(fixed_tl)HuftFree(fixed_tl);
			   fixed_tl=0;
			return i;
			}
		}

											// decompress until an end-of-block code
return InflateCodes(fixed_tl, fixed_td,fixed_bl, fixed_bd) != 0;
}




//*****************************************************************************
//*
//*		InflateDynamic
//*
//*****************************************************************************
//	decompress an inflated type 2 (dynamic Huffman codes) block.
int CHuffman::InflateDynamic()
{
int      i;           						// temporary variables
unsigned j;
unsigned l;           						// last length
unsigned m;           						// mask forbit lengths table
unsigned n;           						// number of lengths to get
huft *tl=0;    								// literal/length code table
huft *td=0;    								// distance code table
int bl;               						// lookup bits fortl
int bd;               						// lookup bits fortd
unsigned nb;          						// number of bit length codes
unsigned nl;          						// number of literal/length codes
unsigned nd;          						// number of distance codes
unsigned ll[288+32];						// literal/length and distance code lengths
register unsigned b;						// bit buffer
register unsigned k;						// number of bits in bit buffer



	b = bb;									// make local bit buffer
	k = bk;

	try	{

		NEEDBITS(5)								// read in table lengths
		nl = 257 + ((unsigned)b & 0x1f);		// number of literal/length codes
		DUMPBITS(5)
		NEEDBITS(5)
		nd = 1 + ((unsigned)b & 0x1f);			// number of distance codes
		DUMPBITS(5)
		NEEDBITS(4)
		nb = 4 + ((unsigned)b & 0xf);			// number of bit length codes
		DUMPBITS(4)

		if(nl > 288 || nd > 32)return 1;		// bad lengths




		for(j = 0; j < nb; j++)					// read in bit-length-code lengths
			{
			NEEDBITS(3)
			ll[border[j]] = (unsigned)b & 7;
			DUMPBITS(3)
			}
		for(; j < 19; j++)ll[border[j]] = 0;



		// build decoding table fortrees--single level, 7 bit lookup
		bl = 7;
		i = HuftBuild(ll, 19, 19, 0, 0, &tl, &bl);
		if(bl == 0)i = 1;                        // no bit lengths

		if(i)
			{
			if(i == 1)HuftFree(tl);
			return i;							// incomplete code set
			}


		// read in literal and distance code lengths
		n = nl + nd;
		m = mask_bits[bl];
		i = l = 0;
		while ((unsigned)i < n)
			{
			NEEDBITS((unsigned)bl)
			j = (td = tl + ((unsigned)b & m))->b;
			DUMPBITS(j)
			j = td->v.n;
			if(j < 16)ll[i++] = l = j;  // length of code in bits (0..15)
			else if(j == 16)			// repeat last length 3 to 6 times
				{
				NEEDBITS(2)
				j = 3 + ((unsigned)b & 3);
				DUMPBITS(2)
				if((unsigned)i + j > n)return 1;
				while (j--)ll[i++] = l;
				}
			else if(j == 17)			// 3 to 10 zero length codes
				{
				NEEDBITS(3)
				j = 3 + ((unsigned)b & 7);
				DUMPBITS(3)
				if((unsigned)i + j > n)return 1;
				while (j--)ll[i++] = 0;
				l = 0;
				}
			else{						// j == 18: 11 to 138 zero length codes
				NEEDBITS(7)
				j = 11 + ((unsigned)b & 0x7f);
				DUMPBITS(7)
				if((unsigned)i + j > n)return 1;
				while (j--)ll[i++] = 0;
				l = 0;
				}
			}


		HuftFree(tl);					// free decoding table fortrees



		bb = b;							// restore the global bit buffer
		bk = k;


										// build the decoding tables forliteral/length and distance codes
		bl = lbits;
		i = HuftBuild(ll, nl, 257, cplens, cplext, &tl, &bl);
		if(bl == 0)i = 1;               // no literals or lengths

		if(i)
			{
			if(i == 1)HuftFree(tl);
			return i;                   // incomplete code set
			}

		bd = dbits;
		i = HuftBuild(ll + nl, nd, 0, cpdist, cpdext, &td, &bd);

		if(bd == 0 && nl > 257)			// lengths but no distances
			{
			HuftFree(tl);
			return 1;
			}
		if(i == 1)i = 0;
		if(i)
			{
			HuftFree(tl);
			return i;
			}


		// decompress until an end-of-block code
		if(InflateCodes( tl, td, bl, bd))return 1;

		// free the decoding tables, return
		}
	catch(char *Text)
		{
		if(tl)HuftFree(tl);
		if(td)HuftFree(td);
		throw Text;
		}

	if(tl)HuftFree(tl);
	if(td)HuftFree(td);


return 0;
}




//*****************************************************************************
//*
//*		InflateBlock
//*
//*****************************************************************************
//	int *e        last block flag
//	decompress an inflated block
int CHuffman::InflateBlock(int *e)
{
int						x;
unsigned				t;		// block type
register unsigned		b;  	// bit buffer
register unsigned		k;		// number of bits in bit buffer



	b = bb;						// make local bit buffer
	k = bk;



	NEEDBITS(1)					// read in last block bit
	*e = (int)b & 1;
	DUMPBITS(1)

	NEEDBITS(2)					// read in block type
	t = (unsigned)b & 3;
	DUMPBITS(2)

	bb = b;						// restore the global bit buffer
	bk = k;


	switch(t)					// Inflate that block type
		{
	case 0:	x=InflateStored ();break;
	case 1:	x=InflateFixed  ();break;
	case 2:	x=InflateDynamic();break;
	default:x=1;
		}

	if(x)
		{
		error_nr=CHF_ERROR_BADHUF;
		throw "Ungldige Huffmann-Tabelle !";
		return 2;				// bad block type
		}

return 0;
}







//*****************************************************************************
//*
//*		InflateFree
//*
//*****************************************************************************
int CHuffman::InflateFree()
{

	if(fixed_tl)
		{
		if(fixed_td)HuftFree(fixed_td);
		if(fixed_tl)HuftFree(fixed_tl);
		   fixed_td=0;
		   fixed_tl=0;
		}

return 0;
}


//*****************************************************************************
//*
//*		HuftBuild
//*
//*****************************************************************************
// const unsigned *b;			 code lengths in bits (all assumed <= BMAX)
// unsigned n;					 number of codes (assumed <= N_MAX)
// unsigned s;					 number of simple-valued codes (0..s-1)
// const unsigned short *d;      list of base values fornon-simple codes
// const unsigned short *e;      list of extra bits fornon-simple codes
// struct huft **t;				 result: starting table
// int *m;						 maximum lookup bits, returns actual
int CHuffman::HuftBuild(const unsigned *b,
						unsigned  n,unsigned  s,
						const unsigned * d,
						const unsigned * e,
						huft **t,int *m)
{
unsigned a;                   // counter forcodes of length k
unsigned c[BMAX+1];           // bit length count table
unsigned el;                  // length of EOB code (value 256)
unsigned f;                   // i repeats in table every f entries
int g;                        // maximum code length
int h;                        // table level
register unsigned i;          // counter, current code
register unsigned j;          // counter
register int k;               // number of bits in current code
int lx[BMAX+1];               // memory forl[-1..BMAX-1]
int *l = lx+1;                // stack of bits per table
register unsigned *p;         // pointer into c[], b[], or v[]
register huft *q;			  // points to current table
huft r;						  // table entry forstructure assignment
huft *u[BMAX];				  // table stack
unsigned v[N_MAX];            // values in order of bit length
register int w;               // bits before this table == (l * h)
unsigned x[BMAX+1];           // bit offsets, then code stack
unsigned *xp;                 // pointer into x
int y;                        // number of dummy codes added
unsigned z;                   // number of entries in current table


	// Generate counts foreach bit length
	el = n > 256 ? b[256] : BMAX; // set length of EOB code, ifany
	memset(c,0,sizeof(c));
	p = (unsigned *)b;  i = n;
	do	{
		c[*p]++; p++;             // assume all entries <= BMAX
		}
	while (--i);

	if(c[0] == n)				  // null input--all zero length codes
		{
		*t = 0;
		*m = 0;
		return 0;
		}


	// Find minimum and maximum length, bound *m by those
	for(j = 1; j <= BMAX; j++)
		if(c[j])break;

	k = j;                        // minimum code length
	if((unsigned)*m < j)*m = j;

	for(i = BMAX; i; i--)
		if(c[i])break;

	g = i;                        // maximum code length
	if((unsigned)*m > i)*m = i;



	// Adjust last length count to fill out codes, ifneeded
	for(y = 1 << j; j < i; j++, y <<= 1)
		if((y -= c[j]) < 0)return 2;     // bad input: more codes than bits

	if((y -= c[i]) < 0)return 2;


	c[i] += y;


	  // Generate starting offsets into the value table foreach length
	x[1] = j = 0;
	p = c + 1;  xp = x + 2;
	while (--i)                  // note that i == g from above
		{
		*xp++ = (j += *p++);
		}


	memset((char *)v,0,sizeof(v));
	p = (unsigned *)b;  i = 0;

	do	{
		if((j = *p++) != 0)
		v[x[j]++] = i;
		}
	while (++i < n);


	n = x[g];                     // set n to length of v


	// Generate the Huffman codes and foreach, make the table entries
	x[0] = i = 0;                 // first Huffman code is zero
	p = v;                        // grab values in bit order
	h = -1;                       // no tables yet--level -1
	w = l[-1] = 0;                // no bits decoded yet
	u[0] = 0;					  // just to keep compilers happy
	q    = 0;					  // ditto
	z    = 0;                     // ditto

	// go through the bit lengths (k already is bits in shortest code)
	for(; k <= g; k++)
		{
		a = c[k];
		while (a--)
			{
			while (k > w + l[h])
				{
				w += l[h++];            // add bits already decoded

				// compute minimum size table less than or equal to *m bits
				z = (z = g - w) > (unsigned)*m ? *m : z;        // upper limit
				if((f = 1 << (j = k - w)) > a + 1)				// try a k-w bit table
					{                   // too few codes fork-w bit table
					f -= a + 1;         // deduct codes from patterns left
					xp = c + k;
					while (++j < z)     // try smaller tables up to z bits
						{				// enough codes to use up j bits
						if((f <<= 1) <= *++xp)break;
						f -= *xp;       // else deduct codes from patterns
						}
					}
										// make EOB code end at table
				if((unsigned)w + j > el && (unsigned)w < el)j = el - w;

				z = 1 << j;             // table entries forj-bit table
				l[h] = j;               // set table size in stack

				// allocate and link in new table
				q = new huft[z+1];
				*t = q + 1;				// link to list forHuftFree()
				*(t = &(q->v.t)) = 0;
				u[h] = ++q;				// table starts after link

				// connect to last table, ifthere is one
				if(h)
					{
					x[h] = i;			// save pattern forbacking up
					r.b = (unsigned char)l[h-1];    // bits to dump before this table
					r.e = (unsigned char)(16 + j);  // bits in this table
					r.v.t = q;          // pointer to this table
					j = (i & ((1 << w) - 1)) >> (w - l[h-1]);
					u[h-1][j] = r;		// connect to last table
					}
				}


			r.b = (unsigned char)(k - w);
			if(p >= v + n)r.e = 99;
			else if(*p < s)
				{
				r.e = (unsigned char)(*p < 256 ? 16 : 15);  // 256 is end-of-block code
				r.v.n = (unsigned short)*p++;               // simple code is just the value
				}
			else{
				r.e = (unsigned char)e[*p - s];				// non-simple--look up in lists
				r.v.n = d[*p++ - s];
				}

			// fill code-like entries with r
			f = 1 << (k - w);
			for(j = i >> w; j < z; j += f) q[j] = r;

			// backwards increment the k-bit code i
			for(j = 1 << (k - 1); i & j; j >>= 1)i ^= j;
			i ^= j;

			// backup over finished tables
			while ((i & ((1 << w) - 1)) != x[h])w -= l[--h];            // don't need to update q
			}
		}


	// return actual size of base table
	*m = l[0];


	// Return true (1) ifwe were given an incomplete table
return y != 0 && g != 1;
}




//*****************************************************************************
//*
//*		HuftFree
//*
//*****************************************************************************
//  Free the malloc'ed tables built by HuftBuild(), which makes a linked
//  list of the tables it made, with the links in a dummy first entry of
//  each table.
int CHuffman::HuftFree(huft *t)
{
register huft *p, *q;

	p = t;

	while(p)
		{
		q = (--p)->v.t;
		delete p;
		p = q;
		}

return 0;
}




//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
//	Ergibt die Dekomprimierte Lnge
int CHuffman::Decompress(int Mode)
{
	int anz;
	int e=0;					// last block flag
	int r=0;					// result code


	if(Mode==-2)Mode=mode;

	decompres_len=0;
	  compres_len=0;
	     byte_pos=0;

//*****************************************************************************

	if(Mode==CHF_STORE)						// Nicht kompimiert
		{
		if(window == 0)
			{
			window= new unsigned char [WSIZE*2];
			window_size =              WSIZE*2;
			}
		for(;;)
			{
			anz=read_buf(isubmit,(char*)window,window_size);
			if(anz<0)
				{
				error_nr=CHF_ERROR_READ;
				throw "Fehler beim Lesen";
				}
			if(anz==0)break;
			if(write_buf)
				{
				if(write_buf(osubmit,(char*)window,anz)!=anz)
					{
					error_nr=CHF_ERROR_WRITE;
					throw "Fehler beim Schreiben in die Datei";
					}
				}
			else{
				BlockCopy((char*)window,anz,0);
				}
			decompres_len+=anz;
			}

		Clear(2);

		if(ocrc_on && !write_buf)				// CRC fr Speicherdekomprmierung
			{
			ocrc=Crc32(ocrc,out_buf,(unsigned)decompres_len);
			}

		return (int)decompres_len;
		}

//*****************************************************************************

	if(Mode!=CHF_DEFLATE)						// Nicht kompimieren
		{
		error_nr=CHF_ERROR_MODE;
		throw "Falscher Modus beim Entkomprimieren !";
		}


	if(window == 0)
		{
		window= new unsigned char [WSIZE*2];
		window_size =              WSIZE*2;
		}

	if(in_mode !=CHF_MEM)						// Speicher Eingabe ?
	if(get_buf == 0)
		{
		get_buf= new unsigned char [CHF_GET_BUF];
		get_size=0;
		}

	fixed_tl = 0;
	fixed_td = 0;
	wp		 = 0;
	bk		 = 0;
	bb		 = 0;




	do	{
		if((r=InflateBlock(&e))!=0)
			{
			return (int)decompres_len;
			}
		}
	while(!e);



	FlushInflate(wp);							//  out window


	if(ocrc_on && !write_buf)					// CRC fr Speicherdekomprmierung
		{
		ocrc=Crc32(ocrc,out_buf,(unsigned)decompres_len);
		}


return (int)decompres_len;
}

