//*****************************************************************************
//*
//*
//*		IoCallbacks.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2015
//
//	Basisklasse zum Lesen und Schreiben von Daten mit CRC32 Funktion uns Lese-Scheribpuffern:
//
//		void	 Reset   ()							: Lese/Schreibpositionen zurcksetzen
//		bool	 MemoryEof()						: Ist der Ausgabespeicher voll (mur bei IOC_MEM erlaubt)
//		char    *GetOutMem()						: Zeiger auf Ausgabespeicher holen
//		int		 GetOutLen()						: Restliche Gre vom Ausgabespeicher holen
//		char    *GetInpMem()						: Zeiger auf Eingabespeicher holen
//		int		 GetInpLen()						: Restliche Gre vom Eingabespeicher holen
//		char    *GetBuffer()						: Zeiger auf Puffer holen
//		int		 GetBufLen()						: Gre  vom Puffer holen
//
//		unsigned GetCrcOut()						: CRC fr Ausgabe abfragen
//		unsigned GetCrcInp()						: CRC fr Eingabe abfragen
//		int64    GetPosOut()						: Aktuelle Schreibposition in Ausgabe abfragen
//		int64    GetPosInp()						: Aktuelle Leseposition    in Eingabe abfragen
//
//		int		 ReadOut(              int iSize)	: Daten lesen
//		int		 Read   (void *pBuffer,int iSize)	: Daten lesen
//		int		 Write  (void *pBuffer,int iSize)	: Daten schreiben
//		int		 TakeOut(              int iSize)	: Daten direkt in Speicher bernehmen
//
// 		void	 SetOut (HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 		void	 SetIn  (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 		void	 SetOut (FILE *,int,unsigned)		: Datei -Ausgabe bestimmen
// 		void	 SetIn  (FILE *,int,unsigned)		: Datei -Eingabe bestimmen
// 		void	 SetOut (unsigned,void*)			: Memory-Ausgabe bestimmen
// 		void	 SetIn  (unsigned,void*)			: Memory-Eingabe bestimmen
// 		void	 SetOut (void*  ,int(*cb)(v*,c*,u)) : Externe Ausgabe ber Funktion
// 		void	 SetIn  (void*  ,int(*cb)(v*,c*,u)) : Externe Eingabe ber Funktion
//		void	 SetOutFilter(v*,int(*cb)(v*,c*,u)) : Filter bei Ausgabe einfgen
//		void	 SetInFilter (v*,int(*cb)(v*,c*,u)) : Filter bei Eingabe einfgen
//		void	 SetOutCrc   (v*,void(cb)(v*,c*,u)) : Eine Funktion fr die CRC Berechung einstellen
//		void	 SetInCrc    (v*,void(cb)(v*,c*,u)) : Eine Funktion fr die CRC Berechung einstellen
//
//	Lese- und Schreibpuffer Funktionen:
//
//		int		 InitWr	(int iSize)					: Schreibpuffer initialisieren
//		int		 InitRd	(int iSize)					: Lesepuffer initialisieren
//		int		 Fill	(int iMin )					: Lesepuffer nachladen
//		int		 Flush	();							: Schreibpuffer wegschreiben
//
//		int		 GetByte (         char  & cValue)	: Ein Byte  aus dem Lesepuffer holen
//		int		 GetByte (unsigned char  &ucValue)	: Ein Byte  aus dem Lesepuffer holen
//		int		 GetWord (         short & sValue)	: Ein Word  aus dem Lesepuffer holen
//		int		 GetWord (unsigned short &usValue)	: Ein Word  aus dem Lesepuffer holen
//		int		 GetDWord(         int   & iValue)	: Ein DWord aus dem Lesepuffer holen
//		int		 GetDWord(unsigned int   &uiValue)	: Ein DWord aus dem Lesepuffer holen
//		int		 GetData (void *pData,int iLen)		: Daten     aus dem Lesepuffer holen
//
//		int		 PutByte (int iValue)				: Ein Byte  in den Schreibpuffer geben
//		int		 PutWord (int iValue)				: Ein Word  in den Schreibpuffer geben
//		int		 PutDWord(int iValue)				: Ein DWord in den Schreibpuffer geben
//		int		 PutData (const void *p,int iLen)	: Daten     in den Schreibpuffer geben
//
//	Basisklasse zum Paken und Entpaken:
//
//		iocint64 Compress  (int)					: Daten komprimieren
//		iocint64 Decompress(int)					: Daten entkomprimieren
//		void	 Free      ()   					: Alle Puffer freigeben
// 		unsigned Value     (int)					: Werte abfargen (32Bit)
// 		int64    Value64   (int)					: Werte abfargen (64Bit)
//		int64    GetSize   ()						: Gre der komprimierten Daten (m_uCompresLen)
//		int64    GetLength ()						: Gre der entpakten     Daten (m_uDecompresLen)
//		int		 GetError  ()						: Fehlercode abfragen
//		int		 GetLevel  ()						: Kompressions-Level abfragen
//		int		 GetFlags  ()						: Kompressions-Flags abfragen
//		int		 GetMode   ()						: Kompressions-Modus abfragen
//
//	Folgende Variablen knnen von der abgeleiteten Klassen benutzt werden:
//
//		m_uDecompresLen								: Anzahl der Entkomprimierten Bytes
//		m_uCompresLen								: Anzahl der Komprimierten Bytes
//		m_iModeFlags								: Modus Flags (CBZ_SLOW,CBZ_FAST)
//		m_iModeLevel								: Kompression 1-9
//		m_iErrorCode								: Aktuelle Fehlernummer (siehe CBZ_ERROR_???)
//

#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif

#include	 <stdio.h>
#include	 <string.h>
#include	 "IoCallbacks.h"

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


unsigned	 CIoCallbacks::uMemPosDummy;			// Dummy-Variable fr Speicherpositionen	

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CIoCallbacks::CIoCallbacks()
{
	
	IOC_ASSSET(m_iInpState=0);						// Aktueller Zustand (0=keines 1=SetIn  2=SetCrc 3=SetFilter)
	
	m_uOutPos		= 0;							// Aktuelle gespeicherte Bytes
	m_uInpPos		= 0;							// Aktuelle geladene Bytes
	m_pFileBuffer	= 0;

	Clear(1);
}

//*****************************************************************************
//*
//*		Destrucktor
//*
//*****************************************************************************
CIoCallbacks::~CIoCallbacks()
{
	Clear(0);
}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//  0 :	Alles rcksetzen
//  1 :	Nur Variabeln rcksetzen, Puffer nicht lschen
//  2 :	Nur Puffer lschen
void CIoCallbacks::Clear(int iMode)
{

	if(iMode==0 || iMode==2)						// Puffer freiegeben ?
		{
		if(m_pFileBuffer)
			{
			delete m_pFileBuffer;
			m_pFileBuffer = NULL;
			}

		if(m_pBufferWr && !m_bBufMemWr)
			{
			delete m_pBufferWr;
			m_pBufferWr = NULL;
			m_iBufLenWr = 0;
			m_iBufOffWr = 0;
			}

		if(m_pBufferRd && !m_bBufMemRd)
			{
			delete m_pBufferRd;
			m_pBufferRd = NULL;
			m_iBufMaxRd = 0;
			m_iBufLenRd = 0;
			m_iBufOffRd = 0;
			}
		}

	if(iMode==2)return;

	m_iOutMax		= 0;							// Maximale Bytezahl fr das Ausgeben in eine Datei
	m_pOutProcCrc	= 0;							// Funktion fr die CRC-Berechnung
	m_uOutCrc		= 0;							// Aktueller CRC Wert fr die Ausgabe
	m_uOutSize		= 0;							// Gre des Ausgabepuffers in Byte
	m_uOutOffset	= 0;							// Aktuelle Schreib-Offset im Ausgabepuffer
	m_pOutBuffer	= 0;							// Zeiger auf den Ausgabepuffer
	m_hOutFile		= 0;							// Datei-Handle fr Ausgabe
	m_pOutProc		= 0;							// Ausgabefunktion
	m_uOutMode		= IOC_NONE;						// Aktuelle Ausgabemodus IOC_FILE,IOC_MEM,IOC_EXT

	m_iInpMax		= 0;							// Maximale Bytezahl fr das Einlesen aus einer Datei
	m_pInpProcCrc	= 0;							// Funktion fr die CRC-Berechnung
	m_uInpCrc		= 0;							// Aktueller CRC Wert fr die Eingabe
	m_uInpSize		= 0;							// Gre des Eingabepuffers in Byte
	m_uInpOffset	= 0;							// Aktuelle Schreib-Offset im Eingabepuffer
	m_pInpBuffer	= 0;							// Zeiger auf den Eingabepuffer
	m_hInpFile		= 0;							// Datei-Handle fr Eingabe
	m_pInpProc 		= 0;							// Eingabefunktion
	m_uInpMode		= IOC_NONE;						// Aktuelle Eingabemodus IOC_FILE,IOC_MEM,IOC_EXT

	m_pBufferWr		= 0;							// Zeiger auf Schreibpuffer
	m_iBufOffWr		= 0;							// Position im Schreibpuffer
	m_iBufLenWr		= 0;							// Gre des Schreibpuffer
	m_bBufMemWr		= 0;							// Ist Schreibpuffer der Ausgabepuffers
	m_iBufOutWr		= 0;							// Anzahl der geflashten Bytes (nur IOC_MEM)

	m_pBufferRd		= 0;							// Zeiger auf Lesepuffer
	m_iBufOffRd		= 0;							// Position im Lesepuffer
	m_iBufLenRd		= 0;							// Gre des Lesepuffer (Fllung)
	m_iBufMaxRd		= 0;							// Gre des Lesepuffer (Allokiert)
	m_bBufMemRd		= 0;							// Ist Lesepuffer der Eingabepuffers

}

//*****************************************************************************
//*
//*		GetBuffer
//*
//*****************************************************************************
//	Holt den Zeiger auf den IO-Puffer
char *CIoCallbacks::GetBuffer()
{

	if(!m_pFileBuffer)
		{
		m_pFileBuffer = new char[IOC_BUFFER_SIZE+16];
		}


return  m_pFileBuffer;
}

//*****************************************************************************
//*
//*		ReadMem
//*
//*****************************************************************************
//	Callback zum Lesen aus einem Speicher.
int __cdecl CIoCallbacks::ReadMem(void *pParam,char *pMem,int iLen)
{
register CIoCallbacks	*pThis=(CIoCallbacks*)pParam;


	if(iLen<=0)return 0;

	if(pThis->m_uInpOffset>=pThis->m_uInpSize)
		{
		return 0;
		}

	if((unsigned)iLen+pThis->m_uInpOffset>pThis->m_uInpSize)
		{
		iLen = pThis->m_uInpSize-pThis->m_uInpOffset;
		}

	memcpy(pMem,pThis->m_pInpBuffer+pThis->m_uInpOffset,iLen);

	if(pThis->m_pInpProcCrc)
		{
		pThis->m_pInpProcCrc(pThis->m_pInpParamCrc,pMem,iLen);
		}

	pThis->m_uInpOffset    += iLen;
	pThis->m_pInpMemPos[0] += iLen;


return iLen;
}

//*****************************************************************************
//*
//*		ReadFile
//*
//*****************************************************************************
//	Callback zum Lesen aus HANDLE.
int __cdecl CIoCallbacks::ReadFile(void *pParam,char *pMem,int iLen)
{
CIoCallbacks	*pThis=(CIoCallbacks*)pParam;


	if(iLen<=0)return 0;

	if(pThis->m_iInpMax>=0)								// Maximale Bytezahl ?
		{
		if(pThis->m_iInpMax <iLen)iLen=pThis->m_iInpMax;
		   pThis->m_iInpMax-=iLen;
		}

	if(!::ReadFile(pThis->m_hInpFile,pMem,iLen,(unsigned long*)&iLen,0))
		{
		return -1;
		}

	if(pThis->m_pInpProcCrc)
		{
		pThis->m_pInpProcCrc(pThis->m_pInpParamCrc,pMem,iLen);
		}


return iLen;
}

//*****************************************************************************
//*
//*		ReadFrom
//*
//*****************************************************************************
//	Callback zum Lesen aus *FILE.
int __cdecl CIoCallbacks::ReadFrom(void *pParam,char *pMem,int iLen)
{
CIoCallbacks   *pThis=(CIoCallbacks*)pParam;
int				iCount;


	if(iLen<=0)return 0;

	if(pThis->m_iInpMax>=0)								// Maximale Bytezahl ?
		{
		if(pThis->m_iInpMax <iLen)iLen=pThis->m_iInpMax;
		   pThis->m_iInpMax-=iLen;
		}

	   iCount=(int)fread(pMem,1,iLen,(FILE*)pThis->m_hInpFile);
	if(iCount<0)return -1;

	if(pThis->m_pInpProcCrc)
		{
		pThis->m_pInpProcCrc(pThis->m_pInpParamCrc,pMem,iCount);
		}


return iCount;
}

//*****************************************************************************
//*
//*		ReadProc
//*
//*****************************************************************************
//	Callback zum Lesen mit einer Funktion.
int __cdecl CIoCallbacks::ReadProc(void *pParam,char *pMem,int iLen)
{
CIoCallbacks   *pThis=(CIoCallbacks*)pParam;
int				iCount;


	if(iLen<=0)return 0;

	   iCount=pThis->m_pInpProcSub(pThis->m_pInpParamSub,pMem,iLen);
	if(iCount<0)return -1;

	if(pThis->m_pInpProcCrc)
		{
		pThis->m_pInpProcCrc(pThis->m_pInpParamCrc,pMem,iCount);
		}


return iCount;
}

//*****************************************************************************
//*
//*		ReadFilter
//*
//*****************************************************************************
//	Callback zum Lesen mit einem Filter.
int __cdecl CIoCallbacks::ReadFilter(void *pParam,char *pMem,int iLen)
{
CIoCallbacks   *pThis=(CIoCallbacks*)pParam;
int				iCount;


	if(iLen<=0)return 0;

	   iCount=pThis->m_pInpProcFilSub(pThis->m_pInpParamFilterS,pMem,iLen);
	if(iCount<0)return -1;

	iCount = pThis->m_pInpProcFilter(pThis->m_pInpParamFilter,pMem,iCount);


return iCount;
}

//*****************************************************************************
//*
//*		WriteFile
//*
//*****************************************************************************
//	Callback zum Schreiben in HANDLE.
int __cdecl CIoCallbacks::WriteFile(void *pParam,const char *pMem,int iLen)
{
CIoCallbacks	*pThis=(CIoCallbacks*)pParam;


	if(iLen<=0)return 0;

	if(pThis->m_iOutMax>=0)								// Maximale Bytezahl ?
		{
		if(pThis->m_iOutMax <iLen)iLen=pThis->m_iOutMax;
		   pThis->m_iOutMax-=iLen;
		}

	if(!::WriteFile(pThis->m_hOutFile,pMem,iLen,(unsigned long*)&iLen,0))
		{
		return -1;
		}

	if(pThis->m_pOutProcCrc)
		{
		pThis->m_pOutProcCrc(pThis->m_pOutParamCrc,pMem,iLen);
		}


return iLen;
}

//*****************************************************************************
//*
//*		WriteMem
//*
//*****************************************************************************
//	Callback zum Schreiben in eine Speicher. 
int __cdecl CIoCallbacks::WriteMem(void *pParam,const char *pMem,int iLen)
{
CIoCallbacks	*pThis=(CIoCallbacks*)pParam;

	
	if(iLen<=0)return 0;

	if(pThis->m_uOutOffset+(unsigned)iLen>pThis->m_uOutSize)
		{
		iLen = pThis->m_uOutSize-pThis->m_uOutOffset;
		}

	memcpy(pThis->m_pOutBuffer+pThis->m_uOutOffset,pMem,iLen);

	if(pThis->m_pOutProcCrc)
		{
		pThis->m_pOutProcCrc(pThis->m_pOutParamCrc,pMem,iLen);
		}

	pThis->m_uOutOffset    += iLen;
	pThis->m_pOutMemPos[0] += iLen;

														// Speicher-End-Punkt prfen
	if(pThis->m_pOutMemEnd && *pThis->m_pOutMemEnd<*pThis->m_pOutMemPos)
		{
		*pThis->m_pOutMemEnd = *pThis->m_pOutMemPos;
		}


return iLen;
}

//*****************************************************************************
//*
//*		WriteFrom
//*
//*****************************************************************************
//	Callback zum Schreiben in *FILE.
int __cdecl CIoCallbacks::WriteFrom(void *pParam,const char *pMem,int iLen)
{
CIoCallbacks   *pThis=(CIoCallbacks*)pParam;
int				iCount;


	if(iLen<=0)return 0;

	if(pThis->m_iOutMax>=0)								// Maximale Bytezahl ?
		{
		if(pThis->m_iOutMax <iLen)iLen=pThis->m_iOutMax;
		   pThis->m_iOutMax-=iLen;
		}

	   iCount=(int)fwrite(pMem,1,iLen,(FILE*)pThis->m_hOutFile);
	if(iCount<0)return -1;

	if(pThis->m_pOutProcCrc)
		{
		pThis->m_pOutProcCrc(pThis->m_pOutParamCrc,pMem,iCount);
		}


return iCount;
}

//*****************************************************************************
//*
//*		WriteProc
//*
//*****************************************************************************
//	Callback zum Schreiben via Funktion.
int __cdecl CIoCallbacks::WriteProc(void *pParam,const char *pMem,int iLen)
{
CIoCallbacks   *pThis=(CIoCallbacks*)pParam;
int				iCount;


	if(iLen<=0)return 0;

	   iCount=pThis->m_pOutProcSub(pThis->m_pOutParamSub,pMem,iLen);
	if(iCount<0)return -1;

	if(pThis->m_pOutProcCrc)
		{
		pThis->m_pOutProcCrc(pThis->m_pOutParamCrc,pMem,iCount);
		}


return iCount;
}

//*****************************************************************************
//*
//*		WriteFilter
//*
//*****************************************************************************
//	Callback zum Schreiben mit einem Filter.
int __cdecl CIoCallbacks::WriteFilter(void *pParam,const char *pMem,int iLen)
{
CIoCallbacks   *pThis=(CIoCallbacks*)pParam;
int				iCount;
int				iBytes;


	if(iLen<=0)return 0;

	   iBytes = pThis->m_pOutProcFilter(pThis->m_pOutParamFilter,(char*)pMem,iLen);
	if(iBytes<0)return -1;


	   iCount=pThis->m_pOutProcFilSub(pThis->m_pOutParamFilterS,pMem,iBytes);
	if(iCount<0)return -1;
	if(iCount!=iBytes)
		{
		iLen--;
		}


return iLen;
}

//*****************************************************************************
//*
//*		WriteFilterBuf
//*
//*****************************************************************************
//	Callback zum Schreiben mit einem Filter via Buffer.
int __cdecl CIoCallbacks::WriteFilterBuf(void *pParam,const char *pMem,int iLen)
{
CIoCallbacks   *pThis=(CIoCallbacks*)pParam;
char			aBuffer[4096+16];
int				iCount;
int				iBytes;
int				iAdd;
unsigned		uSub;
unsigned		uPos;
unsigned		uNum;


	if(iLen<=0)return 0;

	iAdd = 0;
	uSub = 0;

	for(uPos=0;uPos<(unsigned)iLen;uPos+=uNum,pMem+=uNum)
		{
		   uNum	= iLen-uPos;
		if(uNum>4096)uNum=4096;

		memcpy(aBuffer,pMem,uNum);

		   iBytes = pThis->m_pOutProcFilter(pThis->m_pOutParamFilter,aBuffer,uNum);
		if(iBytes<0)return -1;

		   iCount=pThis->m_pOutProcFilSub(pThis->m_pOutParamFilterS,aBuffer,iBytes);
		if(iCount<0)return -1;
		if(iCount!=iBytes)
			{
			uSub++;
			}

		iAdd += iCount;
		}



return iLen-uSub;
}

//*****************************************************************************
//*
//*		SetOut
//*
//*****************************************************************************
//	Stellt den Ausgabemodus auf Dateiausgabe
//	hFile	: Ist das Dateihandle aus dem geschriebn wird
//	iMax	: Bestimmt die maximale Anzahl an zuschreibenden Bytes
//			 	Ist iMax>=0 so werden maximal iMax-Byte geschrieben
//				Ist iMax< 0 gibt es keine Beschrnkung
//	uCrc	: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetOut(HANDLE hFile,int iMax,unsigned uCrc)
{


	IOC_ASSSET(m_iOutState=1);

	m_hOutFile		= hFile;
	m_pOutParam		= this;
	m_pOutProc		= WriteFile;
	m_uOutMode		= IOC_HANDLE;
	m_iOutMax		= iMax;
	m_uOutSize		= 0;
	m_uOutOffset	= 0;
	m_pOutBuffer    = 0;
	m_pOutProcCrc	= 0;

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pOutProcCrc	= (CrcProc)Crc32Ex;
		m_pOutParamCrc	= &m_uOutCrc;
		m_uOutCrc		= ~(iocint64)uCrc;
		}

}

//*****************************************************************************
//*
//*		SetIn
//*
//*****************************************************************************
//	Stellt den Eingabemodus auf Dateieingabe
//	hFile	: Ist das Dateihandle aus dem gelensen wird
//	iMax	: Bestimmt die maximale Anzahl an zulesenden Bytes
//			 	Ist iMax>=0 so werden maximal iMax-Byte gelesen
//				Ist iMax< 0 gibt es keine Beschrnkung
//	uCrc	: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetIn(HANDLE hFile,int iMax,unsigned uCrc)
{

	IOC_ASSSET(m_iInpState=1);

	m_hInpFile		= hFile;
	m_pInpParam		= this;
	m_pInpProc		= ReadFile;
	m_uInpMode		= IOC_HANDLE;
	m_iInpMax		= iMax;
	m_uInpSize		= 0;
	m_uInpOffset	= 0;
	m_pInpBuffer	= 0;
	m_pInpProcCrc	= 0;

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pInpProcCrc	= (CrcProc)Crc32Ex;
		m_pInpParamCrc	= &m_uInpCrc;
		m_uInpCrc		= ~(iocint64)uCrc;
		}

}

//*****************************************************************************
//*
//*		SetOut
//*
//*****************************************************************************
//	Stellt den Ausgabemodus auf Dateiausgabe
//	hFile	: Ist das Dateihandle aus dem geschriebn wird
//	iMax	: Bestimmt die maximale Anzahl an zuschreibenden Bytes
//			 	Ist iMax>=0 so werden maximal iMax-Byte geschrieben
//				Ist iMax< 0 gibt es keine Beschrnkung
//	uCrc	: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetOut(FILE *pFile,int iMax,unsigned uCrc)
{

	IOC_ASSSET(m_iOutState=1);

	m_hOutFile		= (HANDLE)pFile;
	m_pOutParam		= this;
	m_pOutProc		= WriteFrom;
	m_uOutMode		= IOC_FILE;
	m_iOutMax		= iMax;
	m_pOutBuffer    = 0;
	m_uOutOffset	= 0;
	m_uOutSize		= 0;
	m_pOutProcCrc	= 0;

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pOutProcCrc	= (CrcProc)Crc32Ex;
		m_pOutParamCrc	= &m_uOutCrc;
		m_uOutCrc		= ~(iocint64)uCrc;
		}

}

//*****************************************************************************
//*
//*		SetIn
//*
//*****************************************************************************
//	Stellt den Eingabemodus auf Dateieingabe
//	pFile	: Ist das Dateihandle aus dem gelensen wird
//	iMax	: Bestimmt die maximale Anzahl an zulesenden Bytes
//			 	Ist iMax>=0 so werden maximal iMax-Byte gelesen
//				Ist iMax< 0 gibt es keine Beschrnkung
//	uCrc	: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetIn(FILE *pFile,int iMax,unsigned uCrc)
{

	IOC_ASSSET(m_iInpState=1);
	
	m_hInpFile		= (HANDLE)pFile;
	m_pInpParam 	= this;
	m_pInpProc		= ReadFrom;
	m_uInpMode		= IOC_FILE;
	m_iInpMax		= iMax;
	m_uInpSize		= 0;
	m_pInpBuffer	= 0;
	m_uInpOffset	= 0;
	m_pInpProcCrc	= 0;

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pInpProcCrc	= (CrcProc)Crc32Ex;
		m_pInpParamCrc	= &m_uInpCrc;
		m_uInpCrc		= ~(iocint64)uCrc;
		}

}

//*****************************************************************************
//*
//*		SetOut
//*
//*****************************************************************************
//	Stellt den Ausgabemodus auf Speicherausgabe
//	uLen		: Maximale Gre des Ausgabespeichers in Byte
//	pMem		: Zeiger auf den Ausgabespeicher
//	uCrc		: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetOut(unsigned uLen,void *pMem,unsigned uCrc,void *pPos)
{

	IOC_ASSSET(m_iOutState=1);

	m_hOutFile		= 0;
	m_pOutProc		= WriteMem;
	m_pOutParam		= this;
	m_pOutBuffer	= (char*)pMem;
	m_uOutMode		= IOC_MEM;
	m_uOutSize		= uLen;
	m_uOutOffset	= 0;
	m_pOutProcCrc	= 0;
	m_pOutMemEnd	= 0;
	m_pOutMemPos	= (pPos)? (unsigned*)pPos:&uMemPosDummy;

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pOutProcCrc	= (CrcProc)Crc32Ex;
		m_pOutParamCrc	= &m_uOutCrc;
		m_uOutCrc		= ~(iocint64)uCrc;
		}

}

//*****************************************************************************
//*
//*		SetIn
//*
//*****************************************************************************
//	Stellt den Eingabemodus auf Speichereingabe
//	uLen		: Gre des Eingabespeichers in Byte
//	pMem		: Zeiger auf den Ausgabespeicher
//	CrcOn		: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetIn(unsigned uLen,const void *pMem,unsigned uCrc,void *pPos)
{

	IOC_ASSSET(m_iInpState=1);

	m_hInpFile		= 0;
	m_pInpProc		= ReadMem;
	m_pInpParam		= this;
	m_pInpBuffer	= (char*)pMem;
	m_uInpMode		= IOC_MEM;
	m_uInpSize		= uLen;
	m_uInpOffset	= 0;
	m_pInpProcCrc	= 0;
	m_pInpMemPos	= (pPos)? (unsigned*)pPos:&uMemPosDummy;

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pInpProcCrc	= (CrcProc)Crc32Ex;
		m_pInpParamCrc	= &m_uInpCrc;
		m_uInpCrc		= ~(iocint64)uCrc;
		}

}

//*****************************************************************************
//*
//*		SetOut
//*
//*****************************************************************************
//	Stellt den Ausgabemodus auf Fiunktionsausgabe
//	pParam		: Ist der Parameter der immer an die Funktion bergeben wird
//	pProc		: Ist die Funktion die fr die Ausgabe aufgerufen wird
//	CrcOn		: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetOut(void *pParam,int (__cdecl*pProc)(void *pParam,const char *pMem,int iLen),unsigned uCrc)
{

	IOC_ASSSET(m_iOutState=1);

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pOutProcCrc	= (CrcProc)Crc32Ex;
		m_pOutParamCrc	= &m_uOutCrc;
		m_uOutCrc		= ~(iocint64)uCrc;
		m_pOutParamSub	= pParam;						// Wenn eine CRC berechnet wird,
		m_pOutProcSub	= pProc;						// dann eine zweistufigen Aufruf
		m_pOutParam		= this;
		m_pOutProc		= WriteProc;
		}
	else{												// CRC32 Berechung deaktivieren
		m_pOutProcCrc	= NULL;
		m_pOutParam		= pParam;
		m_pOutProc		= pProc;
		}

	m_uOutMode			= IOC_EXT;
	m_pOutBuffer		= 0;
	m_uOutOffset		= 0;
	m_uOutSize			= 0;
	m_hOutFile			= 0;

}

//*****************************************************************************
//*
//*		SetIn
//*
//*****************************************************************************
//	Stellt den Eingabemodus auf Fiunktionseingabe
//	pParam		: Ist der Parameter der immer an die Funktion bergeben wird
//	pProc		: Ist die Funktion die fr die Eingabe aufgerufen wird
//	CrcOn		: Start CRC-Summe, bei IOC_NO_CRC (0x12345678) erfolgt keine Berechnung
void CIoCallbacks::SetIn(void *pParam,int (__cdecl*pProc)(void *pParam,char *pMem,int iLen),unsigned uCrc)
{

	
	IOC_ASSSET(m_iInpState=1);
	

	if(uCrc!=IOC_NO_CRC)								// CRC32 Berechung aktivieren
		{
		m_pInpProcCrc	= (CrcProc)Crc32Ex;
		m_pInpParamCrc	= &m_uInpCrc;
		m_uInpCrc		= ~(iocint64)uCrc;
		m_pInpParamSub	= pParam;						// Wenn eine CRC berechnet wird,
		m_pInpProcSub	= pProc;						// dann einen zweistufigen Aufruf
		m_pInpParam		= this;
		m_pInpProc		= ReadProc;
		}
	else{												// CRC32 Berechung deaktivieren
		m_pInpProcCrc	= NULL;
		m_pInpParam		= pParam;
		m_pInpProc		= pProc;
		}

	m_hInpFile			= 0;
	m_pInpBuffer		= 0;
	m_uInpOffset		= 0;
	m_uInpSize			= 0;
	m_uInpMode			= IOC_EXT;

}

//*****************************************************************************
//*
//*		SetOutCrc
//*
//*****************************************************************************
//	Eine Funktion fr die CRC Berechung bei der Ausgabe einstellen.
//	Diese Funktion muss immer nach SetOut(...) aufgerufen werden.
//	aufgerufen werden, damit der Filter eingehngt werden kann.
//	pParam		: Ist der Parameter fr die CRC-Brechnung
//	pProc		: Ist die Funktion fr die CRC-Brechnung (NULL=aus)
//					void _cdecl Proc(void *pParam,char *pMem,int iLen)
void CIoCallbacks::SetOutCrc(void *pParam,void *pProc)
{

	IOC_ASSERT(m_iOutState<2);
	IOC_ASSSET(m_iOutState=2);

	if(m_uOutMode=IOC_EXT)								// Ausgabe ber Callback
		{
		if(m_pOutProcCrc==0 && pProc!=0)
			{
			m_pOutParamSub	= m_pOutParam;
			m_pOutProcSub	= m_pOutProc;
			m_pOutParam		= this;
			m_pOutProc		= WriteProc;
			}

		if(m_pOutProcCrc!=0 && pProc==0)
			{
			m_pOutParam		= m_pOutParamSub;
			m_pOutProc		= m_pOutProcSub;
			}
		}


	m_pOutProcCrc	= (CrcProc)pProc;					// Neue CRC-Berechung aktivieren
	m_pInpParamCrc	=  pParam;


}


//*****************************************************************************
//*
//*		SetInpCrc
//*
//*****************************************************************************
//	Eine Funktion fr die CRC Berechung bei der Einbe einstellen
//	Diese Funktion muss immer nach SetIn(...) aufgerufen werden.
//	pParam		: Ist der Parameter fr die CRC-Brechnung
//	pProc		: Ist die Funktion fr die CRC-Brechnung (NULL=aus)
//					void _cdecl Proc(void *pParam,char *pMem,int iLen)
void CIoCallbacks::SetInCrc(void *pParam,void *pProc)
{

	IOC_ASSERT(m_iInpState<2);
	IOC_ASSSET(m_iInpState=2);

	if(m_uInpMode=IOC_EXT)								// Einbe ber Callback
		{
		if(m_pInpProcCrc==0 && pProc!=0)
			{
			m_pInpParamSub	= m_pInpParam;
			m_pInpProcSub	= m_pInpProc;
			m_pInpParam		= this;
			m_pInpProc		= ReadProc;
			}

		if(m_pInpProcCrc!=0 && pProc==0)
			{
			m_pInpParam		= m_pInpParamSub;
			m_pInpProc		= m_pInpProcSub;
			}
		}


	m_pInpProcCrc	= (CrcProc)pProc;					// Neue CRC-Berechung aktivieren
	m_pInpParamCrc	=  pParam;


}

//*****************************************************************************
//*
//*		SetOutFilter
//*
//*****************************************************************************
//	Fgt einen Filter beim Lesen. Diese Funktion muss immer nach SetOut(...) und 
//	SetOutCrc(...) aufgerufen werden, damit der Filter eingehngt werden kann.
//	pParam		: Parameter fr die Callback-Funktion
//	pProc		: Zeiger auf die Callback-Funktion
//	iBuffered	: Soll die Ausgabe ber einen eigenen Puffer erfolgen
//					Die Funktion muss als Rckgabewert die Anzahl der in pMem
//					gespeicherten Bytes angeben, oder -1 bei einem Fehler.
void CIoCallbacks::SetOutFilter(void *pParam,int (__cdecl*pProc)(void *pParam,char *pMem,int iLen),int iBuffered)
{

	IOC_ASSERT(m_iOutState<3);
	IOC_ASSSET(m_iOutState=3);

	m_pOutParamFilter	= pParam;
	m_pOutProcFilter	= pProc;

	m_pOutParamFilterS	= m_pOutParam;
	m_pOutProcFilSub	= m_pOutProc;

	m_pOutProc			= (iBuffered)? WriteFilterBuf:WriteFilter;
	m_pOutParam			= this;

}

//*****************************************************************************
//*
//*		SetInFilter
//*
//*****************************************************************************
//	Fgt einen Filter beim Lesen. Diese Funktion muss immer nach SetIn(...) und
//	SetInCrc(...) aufgerufen werden, damit der Filter eingehngt werden kann.
//	pParam		: Parameter fr die Callback-Funktion
//	pProc		: Zeiger auf die Callback-Funktion
//					Die Funktion muss als Rckgabewert die Anzahl der in pMem
//					gespeicherten Bytes angeben, oder -1 bei einem Fehler.
//					Mit dem Rckgabewert 0 wird das Ende der Eingaben signalisiert.
void CIoCallbacks::SetInFilter(void *pParam,int (__cdecl*pProc)(void *pParam,char *pMem,int iLen))
{

	IOC_ASSERT(m_iInpState<3);
	IOC_ASSSET(m_iInpState=3);

	m_pInpParamFilter	= pParam;
	m_pInpProcFilter	= pProc;

	m_pInpParamFilterS	= m_pInpParam;
	m_pInpProcFilSub	= m_pInpProc;

	m_pInpProc			= ReadFilter;
	m_pInpParam			= this;

}

//*****************************************************************************
//*
//*		ReadOut
//*
//*****************************************************************************
//	Daten vom Eingang direkt in den in Ausgabebuffer schreiben.
//	Das ist nur erlaubt wenn eine Speicherausgabe aktiviert ist.
//	iSize	: Ist die Anzahl an Bytes die ausgegeben werden sollen.
//	Ergibt die Anzahl an geschrieben Bytes, bzw. -1 bei einem Fehler.
int	CIoCallbacks::ReadOut(int iSize)
{
int		iCount;


	IOC_ASSERT(m_uOutMode==IOC_MEM);


	if(m_uOutOffset>=m_uOutSize)						// Ist der Speicher voll
		{
		return -1;
		}
														// Daten einlesen
	   iCount=Read(m_pOutBuffer+m_uOutOffset,m_uOutSize-m_uOutOffset);
	if(iCount<=0)
		{
		return iCount;
		}

	if(m_pOutProcCrc)									// CRC fr Speicherdekomprmierung
		{
		m_pOutProcCrc(m_pOutParamCrc,m_pOutBuffer+m_uOutOffset,iCount);
		}

	m_pOutMemPos[0]	+= iCount;
	m_uOutOffset    += iCount;
	m_uOutPos	    += iCount;
														
	if(m_pOutMemEnd && *m_pOutMemEnd<*m_pOutMemPos)		// Speicher-End-Punkt prfen
		{
		*m_pOutMemEnd = *m_pOutMemPos;
		}



return iCount;
}

//*****************************************************************************
//*
//*		SetOutEnd
//*
//*****************************************************************************
//	Memory-Ausgabe End-Punkt-Prfung aktiviren. Wenn die Schreibposition
//	(*m_pOutMemPos) grer als der End-Punkt ist, so wird der End-Punkt
//	nach gezogen. Damit lsst sich wahlfreies Schreiben realisieren.
//	pEnd		: Zeiger auf den End-Punkt (int oder uint Wert)
void CIoCallbacks::SetOutEnd(void *pEnd)
{
	
	IOC_ASSSET(m_iOutState>=1 && m_uOutMode==IOC_MEM); 

	m_pOutMemEnd = (unsigned*)pEnd;

}


//*****************************************************************************
//*
//*		InitWr
//*
//*****************************************************************************
//	Schreibpuffer initialisieren
//	iSize		: Ist die Gre fr den Schreibpuffer
//	Ergibt 1 wenn ok bzw. 0 bei einem Fehler
int	CIoCallbacks::InitWr(int iSize)
{


	IOC_ASSERT(m_uOutMode!=IOC_NONE && iSize>0);

	if(m_uOutMode==IOC_MEM)								// Direkt in den Speicher schreiben
		{
		if(m_pBufferWr && !m_bBufMemWr)
			{
			delete m_pBufferWr;
			}

		m_pBufferWr = m_pOutBuffer;
		m_iBufOffWr = m_uOutOffset;
		m_iBufOutWr	= m_uOutOffset;
		m_iBufLenWr = m_uOutSize;
		m_bBufMemWr = true;
		return 1;
		}

	if(m_bBufMemWr)										// Zeiger auf Eingansspeicher
		{
		m_iBufLenWr = 0;
		m_pBufferWr = 0;
		m_bBufMemWr = 0;
		}

	m_iBufOffWr = 0;
	m_iBufOutWr	= 0;

	if(iSize<=m_iBufLenWr)								// Gibt es den Puffer bereits
		{
		return 1;
		}

	if(m_pBufferWr)										// Alten Puffer lschen
		{
		delete m_pBufferWr;
		m_pBufferWr = 0;
		m_iBufLenWr = 0;
		}

		m_pBufferWr = new char[iSize+16];
	if(!m_pBufferWr)									// Fehler beim Allokieren
		{
		return 0;
		}

	m_iBufLenWr = iSize;


return 1;
}

//*****************************************************************************
//*
//*		InitRd
//*
//*****************************************************************************
//	Lesepuffer initialisieren
//	iSize		: Ist die Gre fr den Schreibpuffer
//	Ergibt 1 wenn ok bzw. 0 bei einem Fehler
int	CIoCallbacks::InitRd(int iSize)
{

	IOC_ASSERT(m_uInpMode!=IOC_NONE && iSize>0);


	if(m_uInpMode==IOC_MEM)								// Direkt aus dem Speicher lesen
		{
		if(m_pBufferRd && !m_bBufMemRd)
			{
			delete m_pBufferRd;
			}

		m_pBufferRd = m_pInpBuffer;
		m_iBufOffRd = m_uInpOffset;
		m_iBufMaxRd = m_uInpSize;
		m_iBufLenRd = m_uInpSize;
		m_bBufEofRd = false;
		m_bBufMemRd = true;

		TakeInp(m_uInpSize);

		return 1;
		}

	if(m_bBufMemRd)										// Zeiger auf Eingansspeicher
		{
		m_iBufLenRd = 0;
		m_pBufferRd = 0;
		m_bBufMemRd = 0;
		}

	m_iBufLenRd = 0;
	m_iBufOffRd = 0;
	m_bBufEofRd = false;
	m_bBufMemRd = false;

	if(iSize<=m_iBufLenRd)								// Gibt es den Puffer bereits
		{
		return 1;
		}

	if(m_pBufferRd)										// Alten Puffer lschen
		{
		delete m_pBufferRd;
		m_pBufferRd = 0;
		m_iBufMaxRd = 0;
		}

		m_pBufferRd = new char[iSize+16];
	if(!m_pBufferRd)									// Fehler beim Allokieren
		{
		return 0;
		}

	m_iBufMaxRd = iSize;


return 1;
}


//*****************************************************************************
//*
//*		PutData
//*
//*****************************************************************************
//	Gibt Daten in den Schreibpuffer.
//	pData		: Zeiger auf die auszugebenden Daten
//	iBytes		: Anzahl der Daten
//	Ergibt 0 wenn ok, bzw. 1 bei einem Fehler
int CIoCallbacks::PutData(const void *pData,int iBytes)
{
char   *pPtr=(char*)pData;
int		iRest;


	IOC_ASSERT(m_pBufferWr!=0);


	while(m_iBufOffWr+iBytes>=m_iBufLenWr)				// Passt nicht alles in den Puffer
		{
		iRest        = m_iBufLenWr-m_iBufOffWr;
		memcpy(m_pBufferWr+m_iBufOffWr,pPtr,iRest);
		m_iBufOffWr += iBytes;
		pPtr	    += iRest;
		iBytes      -= iRest;

		if(Flush())										// Puffer leeren und in den Stream ausgeben
			{
			return 1;
			}

		if(iBytes<=0)
			{
			return 0;
			}

		if(m_iBufOffWr>=m_iBufLenWr)					// Ist kein Platz mehr im Puffer
			{
			return 1;
			}
		}

	memcpy(m_pBufferWr+m_iBufOffWr,pPtr,iBytes);		// Daten in den Puffer kopieren
	m_iBufOffWr += iBytes;



return 0;
}


//*****************************************************************************
//*
//*		PutData
//*
//*****************************************************************************
//	Gibt Daten in den Schreibpuffer ber eine statische Funktion
//	pParam		: Ist der This-Zeiger
//	pData		: Zeiger auf die auszugebenden Daten
//	iBytes		: Anzahl der Daten
//	Ergibt die Anzahl der gespeicherten Dateien
int __cdecl CIoCallbacks::PutCall(void *pParam,const void *pData,int iLen)
{

	if(((CIoCallbacks*)pParam)->PutData(pData,iLen))
		{
		return -1;
		}

return iLen;
}

//*****************************************************************************
//*
//*		GetData
//*
//*****************************************************************************
//	Daten aus dem Lesepuffer holen.
//	pData		: Zeiger auf die auszugebenden Daten
//	iBytes		: Anzahl der Daten
//	Ergibt die geladenen Bytes, bzw. 0 bei einem Fehler.
int CIoCallbacks::GetData(void *pData,int iBytes)
{
char   *pPtr =(char*)pData;
char   *pFrom=pPtr;
int		iRest;



	IOC_ASSERT(m_pBufferRd!=0);

	iRest = m_iBufLenRd-m_iBufOffRd;

	while(iRest<iBytes)									// Sind genug Daten im Puffer
		{
		memcpy(pPtr,m_pBufferRd+m_iBufOffRd,iRest);

		m_iBufOffRd	+= iRest;
		pPtr		+= iRest;
		iBytes		-= iRest;

		if(m_uInpMode==IOC_MEM)							// Direkt aus dem Eingabespeicher
			{
			IOC_ASSERT(m_bBufMemRd && m_pBufferRd==m_pInpBuffer);
			m_bBufEofRd = true;
			return (int)(pPtr-pFrom);
			}

		   iRest = Fill();
		if(iRest<=0)									// Alles aur Stream eingelesen
			{
			return (int)(pPtr-pFrom);
			}
		}												// Puffer nachladen aus Stream


	memcpy(pPtr,m_pBufferRd+m_iBufOffRd,iBytes);		// Daten in den Puffer kopieren
	m_iBufOffRd += iBytes;
	pPtr		+= iBytes;


return (int)(pPtr-pFrom);
}

//*****************************************************************************
//*
//*		GetCall
//*
//*****************************************************************************
//	Daten aus dem Lesepuffer holen mit einer Static-Funktion.
//	pParam		: Ist der This-Zeiger
//	pData		: Zeiger auf die auszugebenden Daten
//	iBytes		: Anzahl der Daten
//	Ergibt die geladenen Bytes, bzw. 0 bei einem Fehler.
int	__cdecl CIoCallbacks::GetCall(void *pParam,void *pData,int iLen)
{
return ((CIoCallbacks*)pParam)->GetData(pData,iLen);
}


//*****************************************************************************
//*
//*		Flush
//*
//*****************************************************************************
//	Den Schreibpuffer in den Stream leeren
//	Ergibt 0 wenn ok, bzw. 1 bei einem Fehler
int CIoCallbacks::Flush()
{
int		iBytes;


	IOC_ASSERT(m_pBufferWr!=0);


	if(m_iBufOffWr<=0)									// Keine Daten im Puffer
		{
		return 0;
		}


	if(m_uOutMode==IOC_MEM)								// Daten stehen schon im Ausgabespeicher
		{
		IOC_ASSERT(m_bBufMemWr && m_pBufferWr==m_pOutBuffer);

		iBytes		= m_iBufOffWr-m_iBufOutWr;
		m_iBufOutWr	= TakeOut(iBytes);
		}
	else{
		IOC_ASSERT(!m_bBufMemWr);

		   iBytes = m_iBufOffWr;
		if(iBytes<=0)return 0;

		if(Write(m_pBufferWr,iBytes)!=iBytes)			// Daten in den Stream ausgeben
			{
			return 1;
			}

		m_iBufOffWr = 0;
		}



return 0;
}

//*****************************************************************************
//*
//*		Fill
//*
//*****************************************************************************
//	Den Lesepuffer nachladen mit Daten aus dem Stream.
//	iMin	: Nur nachladen wenn im Lesepuffer weniger als iMin Steht
//	Ergibt 0 wenn bei einem Fehler oder die Bytes im Puffer
int CIoCallbacks::Fill(int iMin)
{
int		iBytes;
int		iRest;


	IOC_ASSERT(m_pBufferRd!=0 && iMin>0);

	iRest = m_iBufLenRd-m_iBufOffRd;

	if(m_uInpMode==IOC_MEM)								// Daten stehen schon im Eingabespeicher
		{
		IOC_ASSERT(m_bBufMemRd && m_pBufferRd==m_pInpBuffer);
		if(iRest<=0)m_bBufEofRd = true;
		else m_uInpPos+=iRest;
		return iRest;
		}


	IOC_ASSERT(!m_bBufMemRd);							// Daten aus Stream holen


	if(iRest>=iMin)										// Sind noch genug Byte im Puffer
		{
		return iRest;
		}

	memmove(m_pBufferRd,m_pBufferRd+m_iBufOffRd,iRest);

	m_iBufOffRd = 0;
	m_iBufLenRd = iRest;
	iBytes		= m_iBufMaxRd-iRest;
	iBytes		= Read(m_pBufferRd+iRest,iBytes);

	if(iBytes<=0)										// Gab es einen Lesefehler
		{
		if(iBytes<0)return 0;
		m_bBufEofRd = true;
		return 0;
		}

	m_iBufLenRd += iBytes;


return m_iBufLenRd-m_iBufOffRd;
}






//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CBaseCompressor::CBaseCompressor()
{

	m_uDecompresLen	= 0;							// Anzahl der Entkomprimierten Bytes
	m_uCompresLen	= 0;							// Anzahl der Komprimierten Bytes
	m_iErrorCode	= 0;							// Aktuelle Fehlernummer (siehe CBC_ERROR_???)
	m_iModeFlags	= 0;							// Modus Flags (CBC_SLOW,CBC_FAST)
	m_iModeLevel	= 9;							// Kompressions-Level 1-9
	m_iMode			= CBC_DEFLATE;					// Kompressionsmodus

}


//*****************************************************************************
//*
//*		Compress
//*
//*****************************************************************************
// Daten komprimieren, virtuelle Dummy-Funtion falls nicht abgeleited.
iocint64 CBaseCompressor::Compress(int iLevel)
{

	m_iErrorCode = CBC_ERROR_NOTSUPP;
	throw "Die Funktion wird nicht untersttzt";


return 0;
}

//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
// Daten entkomprimieren, virtuelle Dummy-Funtion falls nicht abgeleited.
iocint64 CBaseCompressor::Decompress(int iLevel)
{

	m_iErrorCode = CBC_ERROR_NOTSUPP;
	throw "Die Funktion wird nicht untersttzt";


return 0;
}


//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//  0 :	Alles rcksetzen
//  1 :	Nur Variabeln rcksetzen , Puffer nicht lschen
//  2 :	Nur Puffer lschen
void CBaseCompressor::Clear(int iMode)
{

	CIoCallbacks::Clear(iMode);

	if(iMode==2)return;

	m_uDecompresLen	= 0;							// Anzahl der Entkomprimierten Bytes
	m_uCompresLen	= 0;							// Anzahl der Komprimierten Bytes
	m_iErrorCode	= 0;							// Aktuelle Fehlernummer (siehe CBC_ERROR_???)
	m_iModeFlags	= 0;							// Modus Flags (CBC_SLOW,CBC_FAST)
	m_iModeLevel	= 9;							// Kompressions-Level 1-9
	m_iMode			= CBC_DEFLATE;					// Kompressionsmodus

}

//*****************************************************************************
//*
//*		Value64
//*
//*****************************************************************************
//	Eine 64-Bit-Wert abfragen.
//	iMode	: Auswahl des Wertes
//	Ergibt den Wert.
iocint64 CBaseCompressor::Value64(int iMode)const
{

	switch(iMode)
		{
	case CBC_LEVEL:	return m_iModeLevel;
	case CBC_FLAG:	return m_iModeFlags;
	case CBC_MODE:	return m_iModeLevel;
	case CBC_OCRC:	return~m_uOutCrc;
	case CBC_ICRC:	return~m_uInpCrc;
	case CBC_SIZE:	return m_uCompresLen;
	case CBC_LEN:	return m_uDecompresLen;
	case CBC_OUT:	return m_uOutPos;
	case CBC_INP:	return m_uInpPos;
		}


return 0;
}

//*****************************************************************************
//*
//*		Value
//*
//*****************************************************************************
//	Eine 32-Bit-Wert abfragen.
//	iMode	: Auswahl des Wertes
//	Ergibt den Wert.
unsigned CBaseCompressor::Value(int iMode)const
{

	switch(iMode)
		{
	case CBC_LEVEL:	return (unsigned) (m_iModeLevel);
	case CBC_FLAG:	return (unsigned) (m_iModeFlags);
	case CBC_MODE:	return (unsigned) (m_iModeLevel);
	case CBC_OCRC:	return (unsigned)~(m_uOutCrc);
	case CBC_OCRCH:	return (unsigned)~(m_uOutCrc>>32);
	case CBC_ICRC:	return (unsigned)~(m_uInpCrc);
	case CBC_ICRCH:	return (unsigned)~(m_uInpCrc>>32);
	case CBC_SIZE:	return (unsigned) (m_uCompresLen);
	case CBC_SIZEH:	return (unsigned) (m_uCompresLen>>32);
	case CBC_LEN:	return (unsigned) (m_uDecompresLen);
	case CBC_LENH:	return (unsigned) (m_uDecompresLen>>32);
	case CBC_OUT:	return (unsigned) (m_uOutPos);
	case CBC_OUTH:	return (unsigned) (m_uOutPos>>32);
	case CBC_INP:	return (unsigned) (m_uInpPos);
	case CBC_INPH:	return (unsigned) (m_uInpPos>>32);
		}


return 0;
}


