//*****************************************************************************
//*
//*
//*		IoCallbacks.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2015
//
//	Basisklasse zum Lesen und Schreiben von Daten mit CRC32 Funktion uns Lese-Scheribpuffern:
//
//		void	 Reset   ()							: Lese/Schreibpositionen zurcksetzen
//		bool	 MemoryEof()						: Ist der Ausgabespeicher voll (mur bei IOC_MEM erlaubt)
//		char    *GetOutMem()						: Zeiger auf Ausgabespeicher holen
//		int		 GetOutLen()						: Restliche Gre vom Ausgabespeicher holen
//		char    *GetInpMem()						: Zeiger auf Eingabespeicher holen
//		int		 GetInpLen()						: Restliche Gre vom Eingabespeicher holen
//		char    *GetBuffer()						: Zeiger auf Puffer holen
//		int		 GetBufLen()						: Gre  vom Puffer holen
//
//		unsigned GetCrcOut()						: CRC fr Ausgabe abfragen
//		unsigned GetCrcInp()						: CRC fr Eingabe abfragen
//		int64    GetPosOut()						: Aktuelle Schreibposition in Ausgabe abfragen
//		int64    GetPosInp()						: Aktuelle Leseposition    in Eingabe abfragen
//
//		int		 ReadOut(              int iSize)	: Daten lesen
//		int		 Read   (void *pBuffer,int iSize)	: Daten lesen
//		int		 Write  (void *pBuffer,int iSize)	: Daten schreiben
//		int		 TakeOut(              int iSize)	: Daten direkt in Speicher bernehmen
//
// 		void	 SetOut (HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 		void	 SetIn  (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 		void	 SetOut (FILE *,int,unsigned)		: Datei -Ausgabe bestimmen
// 		void	 SetIn  (FILE *,int,unsigned)		: Datei -Eingabe bestimmen
// 		void	 SetOut (unsigned,void*)			: Memory-Ausgabe bestimmen
// 		void	 SetIn  (unsigned,void*)			: Memory-Eingabe bestimmen
// 		void	 SetOut (void*  ,int(*cb)(v*,c*,u)) : Externe Ausgabe ber Funktion
// 		void	 SetIn  (void*  ,int(*cb)(v*,c*,u)) : Externe Eingabe ber Funktion
//		void	 SetOutFilter(v*,int(*cb)(v*,c*,u)) : Filter bei Ausgabe einfgen
//		void	 SetInFilter (v*,int(*cb)(v*,c*,u)) : Filter bei Eingabe einfgen
//		void	 SetOutCrc   (v*,void(cb)(v*,c*,u)) : Eine Funktion fr die CRC Berechung einstellen
//		void	 SetInCrc    (v*,void(cb)(v*,c*,u)) : Eine Funktion fr die CRC Berechung einstellen
// 		void	 SetOutEnd   (void*)				:Memory-Ausgabe End-Punkt-Prfung
//
//	Lese- und Schreibpuffer Funktionen:
//
//		int		 InitWr	(int iSize)					: Schreibpuffer initialisieren
//		int		 InitRd	(int iSize)					: Lesepuffer initialisieren
//		int		 Fill	(int iMin )					: Lesepuffer nachladen
//		int		 Flush	()							: Schreibpuffer wegschreiben
//
//		int		 GetByte (         char  & cValue)	: Ein Byte  aus dem Lesepuffer holen
//		int		 GetByte (unsigned char  &ucValue)	: Ein Byte  aus dem Lesepuffer holen
//		int		 GetWord (         short & sValue)	: Ein Word  aus dem Lesepuffer holen
//		int		 GetWord (unsigned short &usValue)	: Ein Word  aus dem Lesepuffer holen
//		int		 GetDWord(         int   & iValue)	: Ein DWord aus dem Lesepuffer holen
//		int		 GetDWord(unsigned int   &uiValue)	: Ein DWord aus dem Lesepuffer holen
//		int		 GetData (void *pData,int iLen)		: Daten     aus dem Lesepuffer holen
//
//		int		 PutByte (int iValue)				: Ein Byte  in den Schreibpuffer geben
//		int		 PutWord (int iValue)				: Ein Word  in den Schreibpuffer geben
//		int		 PutDWord(int iValue)				: Ein DWord in den Schreibpuffer geben
//		int		 PutData (const void *p,int iLen)	: Daten     in den Schreibpuffer geben
//
//	Basisklasse zum Paken und Entpaken:
//
//		iocint64 Compress  (int)					: Daten komprimieren
//		iocint64 Decompress(int)					: Daten entkomprimieren
//		void	 Free      ()   					: Alle Puffer freigeben
// 		unsigned Value     (int)					: Werte abfargen (32Bit)
// 		int64    Value64   (int)					: Werte abfargen (64Bit)
//		int64    GetSize   ()						: Gre der komprimierten Daten (m_uCompresLen)
//		int64    GetLength ()						: Gre der entpakten     Daten (m_uDecompresLen)
//		int		 GetError  ()						: Fehlercode abfragen
//		int		 GetLevel  ()						: Kompressions-Level abfragen
//		int		 GetFlags  ()						: Kompressions-Flags abfragen
//		int		 GetMode   ()						: Kompressions-Modus abfragen
//
//	Folgende Variablen knnen von der abgeleiteten Klassen benutzt werden:
//
//		m_uDecompresLen								: Anzahl der Entkomprimierten Bytes
//		m_uCompresLen								: Anzahl der Komprimierten Bytes
//		m_iModeFlags								: Modus Flags (CBZ_SLOW,CBZ_FAST)
//		m_iModeLevel								: Kompression 1-9
//		m_iErrorCode								: Aktuelle Fehlernummer (siehe CBZ_ERROR_???)
//
#ifndef		__IOCALLBACK_H__
#define		__IOCALLBACK_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
	#define 	iocint64		__int64
	#define 	IOC_ASSERT(a)	ASSERT(a)
	#ifdef	   _DEBUG
	#define 	IOC_ASSSET(a)	a
	#else
	#define 	IOC_ASSSET(a)
	#endif 	
#else
	#ifdef		WIN32
	#include	<Windows.h>
	#define 	iocint64		__int64
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#define 	iocint64		long long
	#endif
	#define 	IOC_ASSERT(a)
	#define 	IOC_ASSSET(a)
#endif

#ifndef 	_MSC_VER
#ifndef 	 _cdecl
#define 	 _cdecl
#endif
#ifndef 	__cdecl
#define 	__cdecl
#endif
#endif

#ifndef		iocint64
#define 	iocint64			__int64
#endif

#include	<stdio.h>
#include	"../Filter/Crc32.h"


//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define IOC_BUFFER_SIZE		0x4000			// Gre des IO-Puffers
#define IOC_NO_CRC			0x12345678		// CRC32 Berechung deaktivieren

#define IOC_MEM				0				// Ein/Ausgabe ber Speicher
#define IOC_EXT				1				// Ein/Ausgabe ber externe Funktion
#define IOC_FILE			2				// Ein/Ausgabe ber Dateien	mit *FILE
#define IOC_HANDLE			3				// Ein/Ausgabe ber Dateien mit HANDLE
#define IOC_NONE			4				// Ein/Ausgabe noch nicht definiert

#define CBC_LEVEL			0				// Level ausgeben
#define CBC_FLAG			1				// Flag  ausgeben
#define CBC_MODE			2				// Modus ausgeben
#define CBC_OCRC			3				// Out CRC ausgeben
#define CBC_OCRCH			13				// Out CRC ausgeben	(obere 32 Bit)
#define CBC_ICRC			4				// In  CRC ausgeben
#define CBC_ICRCH			14				// In  CRC ausgeben	(obere 32 Bit)
#define CBC_SIZE			5				// Komprimierte Lnge ausgeben
#define CBC_SIZEH			9				// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CBC_LEN				6				// Dekomprimierte Lnge ausgeben
#define CBC_LENH			10				// Dekomprimierte Lnge ausgeben  (obere 32 Bit)
#define CBC_INP				7				// Aktuelle geladene/gespeicherte Bytes
#define CBC_INPH			11				// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CBC_OUT				8				// Aktuelle geladene/gespeicherte Bytes
#define CBC_OUTH			12				// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)

#define CBC_SLOW			0x02			// Kompressions-Flag bei Level <=2
#define CBC_FAST			0x04			// Kompressions-Flag bei Level >=8
#define CBC_NOCOMPESS	    0x10			// Fehler beim Komprimieren (Ziel grer Quelle)

#define CBC_DEFLATE			8   			// Kompressions-Modus (komprimieren)
#define CBC_STORE			0				// Kompressions-Modus (1zu1 speichern)

#define CBC_ERROR_OK		0				// Kein Fehler
#define CBC_ERROR_READ		1				// Fehler beim Lesen
#define CBC_ERROR_WRITE		2				// Fehler beim Schreiben
#define CBC_ERROR_OVERFLOW	3				// Speicherberlauf
#define CBC_ERROR_EOF     	4				// Vorzeitiges Eingabeende
#define CBC_ERROR_MODE    	5				// Falscher Modus
#define CBC_ERROR_BADFORMAT	6				// Ungltiges Format
#define CBC_ERROR_MEMORY  	7				// Zuwenig Speicher
#define CBC_ERROR_PARAM  	8				// Falsche Parameter
#define CBC_ERROR_IO		9				// IO-Fehler
#define CBC_ERROR_NOTSUPP 	10				// Die Funktion wird nicht untersttzt


//*****************************************************************************
//*
//*		Klasse CIoCallbacks
//*
//*****************************************************************************
class CIoCallbacks
{
protected:

	typedef unsigned char	  uchar;
	typedef unsigned iocint64 uint64;

	typedef void (__cdecl *CrcProc)(void *,const char  *,int);
	typedef void (__cdecl *CrcCall)(void *,const void  *,int);
	typedef void (__cdecl *CrcFunc)(void *,const uchar *,int);
	typedef int  (__cdecl *OutProc)(void *,const char  *,int);
	typedef int  (__cdecl *OutCall)(void *,const void  *,int);
	typedef int  (__cdecl *OutFunc)(void *,const uchar *,int);
	typedef int  (__cdecl *InpProc)(void *,      char  *,int);
	typedef int  (__cdecl *InpCall)(void *,      void  *,int);
	typedef int  (__cdecl *InpFunc)(void *,      uchar *,int);
	typedef int  (__cdecl *MemProc)(void *,      char  *,int);
	typedef int  (__cdecl *MemCall)(void *,      void  *,int);
	typedef int  (__cdecl *MemFunc)(void *,      uchar *,int);

	char	   *m_pFileBuffer;				// Temporrer Dateipuffer

	int			m_iOutMax;					// Maximale Bytezahl fr das Ausgeben in eine Datei (-1=keine Prfung)
	uint64		m_uOutPos;					// Aktuelle gespeicherte Bytes
	uint64		m_uOutCrc;					// Aktueller CRC Wert fr die Ausgabe
	unsigned	m_uOutSize;					// Gre des Ausgabepuffers in Byte
	unsigned	m_uOutMode;					// Aktuelle Ausgabemodus CBZ_FILE,CBZ_MEM,CBZ_EXT
	unsigned	m_uOutOffset;				// Aktuelle Schreib-Offset im Ausgabepuffer
	unsigned   *m_pOutMemPos;				// Zeiger auf Position fr Speicherausgaben
	unsigned   *m_pOutMemEnd;				// Zeiger auf End-Position fr Speicherausgaben
	char       *m_pOutBuffer;				// Zeiger auf den Ausgabepuffer
	void	   *m_pOutParam;				// Parameter fr Schreibefunktion
	void	   *m_pOutParamCrc;				// Parameter fr die CRC Funktion
	void	   *m_pOutParamSub;				// Parameter fr Schreiben ber einer Funktion (zweistufig)
	void	   *m_hOutFile;					// Datei-Handle fr Ausgabe
	void	   *m_pOutParamFilter;			// Parameter fr Filter Schreib-Callback
	void	   *m_pOutParamFilterS;			// Parameter fr Filteraufruf Schreib-Callback

	int			m_iInpMax;					// Maximale Bytezahl fr das Einlesen aus einer Datei (-1=keine Prfung)
	uint64		m_uInpPos;					// Aktuelle geladene Bytes
	uint64		m_uInpCrc;					// Aktueller CRC Wert fr die Eingabe
	unsigned	m_uInpSize;					// Gre des Eingabepuffers in Byte
	unsigned	m_uInpMode;					// Aktuelle Eingabemodus CBZ_FILE,CBZ_MEM,CBZ_EXT
	unsigned	m_uInpOffset;				// Aktuelle Schreib-Offset im Eingabepuffer
	unsigned   *m_pInpMemPos;				// Zeiger auf Position fr Speichereingaben
	char       *m_pInpBuffer;				// Zeiger auf den Eingabepuffer
	void	   *m_pInpParam;				// Parameter fr Einlesefunktion
	void	   *m_pInpParamCrc;				// Parameter fr die CRC Funktion
	void	   *m_pInpParamSub;				// Parameter fr Einlesen ber einer Funktion (zweistufig)
	void	   *m_hInpFile;					// Datei-Handle fr Eingabe
	void	   *m_pInpParamFilter;			// Parameter fr Filter Lese-Callback
	void	   *m_pInpParamFilterS;			// Parameter fr Filteraufruf Lese-Callback

	char	   *m_pBufferWr;				// Zeiger auf Schreibpuffer
	int			m_iBufOffWr;				// Position im Schreibpuffer
	int 		m_iBufOutWr;				// Anzahl der geflashten Bytes (nur IOC_MEM)
	int			m_iBufLenWr;				// Gre des Schreibpuffer
	bool		m_bBufMemWr;				// Ist Schreibpuffer der Ausgabepuffers

	char	   *m_pBufferRd;				// Zeiger auf Lesepuffer
	int			m_iBufOffRd;				// Position im Lesepuffer
	int			m_iBufLenRd;				// Gre des Lesepuffer (Fllung)
	int			m_iBufMaxRd;				// Gre des Lesepuffer (Allokiert)
	bool		m_bBufMemRd;				// Ist Lesepuffer der Eingabepuffers
	bool		m_bBufEofRd;				// Ende beim Einlesen erreicht


	MemProc		m_pInpProcFilter;			// Callbacks fr Filter
	MemProc		m_pOutProcFilter;			//		*
	InpProc		m_pInpProcFilSub;			// Unteraufrufe fr Filter
	OutProc		m_pOutProcFilSub;			//		*
	InpProc		m_pInpProcSub;				// Unteraufrufe fr Funktionen
	OutProc		m_pOutProcSub;				//		*
	CrcProc		m_pInpProcCrc;				// Unteraufrufe fr Funktionen
	CrcProc		m_pOutProcCrc;				//		*
	InpProc		m_pInpProc;					// Hauptaufrufe fr alle
	OutProc		m_pOutProc;					//		*


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************

public:

			 CIoCallbacks();
	virtual ~CIoCallbacks();

	void	 Reset    ();								// Schreib- und Leseposition wieder auf 0 setzen
	char    *GetBuffer();								// Zeiger auf den Datenpuffers holen (nur wenn keine Speicherausgabe)
	int		 GetBufLen()const;							// Gre des Datenpuffers abfragen   (nur wenn keine Speicherausgabe)
	char    *GetOutMem()const;							// Zeiger auf den Puffer fr die Speicherausgabe
	int		 GetOutLen()const;							// Restliche Gre bei Speicherausgabe abfragen
	char    *GetInpMem()const;							// Zeiger auf den Puffer fr die Speicherausgabe
	int		 GetInpLen()const;							// Restliche Gre bei Speicherausgabe abfragen
	unsigned GetCrcOut()const;							// Aktueller CRC-Wert fr die Ausgabe
	unsigned GetCrcInp()const;							// Aktueller CRC-Wert fr die Eingabe
	iocint64 GetPosOut()const;							// Aktuelle Schreibposition
	iocint64 GetPosInp()const;							// Aktuelle Leseposition
	bool	 OutputEof()const;							// Ist das Ende im Puffer fr die Speicherausgabe erreicht
	bool	 InputEof ()const;							// Ist das Ende im Puffer fr die Speicherausgabe erreicht

	int		 ReadOut	 (int   iSize =0x20000000);		// Daten vom Eingabe direkt in den die Speicherausgabe schreiben
	int		 Read		 (void *pBuffer,int iSize);		// Daten von Eingabe in einen Puffer einlesen
	int		 Write       (void *pBuffer,int iSize);		// Daten in die Ausgabe aus einen Puffer schreiben
	int		 TakeOut     (              int iSize);		// Daten die in die Speicherausgabe bertragen wurden, mit CRC bernehmen
	int		 TakeInp     (              int iSize);		// Daten von der    Speichereingabe bertragen wurden, mit CRC bernehmen

														// Die Ein-/Ausgabe definieren
	void	 SetOut      (HANDLE hFile,int iMax=-1      ,unsigned uCrc=IOC_NO_CRC);
	void	 SetIn       (HANDLE hFile,int iMax=-1      ,unsigned uCrc=IOC_NO_CRC);
	void	 SetOut      (FILE  *pFile,int iMax=-1      ,unsigned uCrc=IOC_NO_CRC);
	void	 SetIn       (FILE  *pFile,int iMax=-1      ,unsigned uCrc=IOC_NO_CRC);
	void	 SetOut      (unsigned uLen,      void *pMem,unsigned uCrc=IOC_NO_CRC,void *pPos=0);
	void	 SetIn       (unsigned uLen,const void *pMem,unsigned uCrc=IOC_NO_CRC,void *pPos=0);
	void	 SetOut      (void *pParam,OutProc pProc    ,unsigned uCrc=IOC_NO_CRC);
	void	 SetOut      (void *pParam,OutCall pProc    ,unsigned uCrc=IOC_NO_CRC);
	void	 SetOut      (void *pParam,OutFunc pProc    ,unsigned uCrc=IOC_NO_CRC);
	void	 SetOut      (void *pParam,InpCall pProc    ,unsigned uCrc=IOC_NO_CRC);
	void	 SetIn       (void *pParam,InpProc pProc    ,unsigned uCrc=IOC_NO_CRC);
	void	 SetIn       (void *pParam,InpCall pProc    ,unsigned uCrc=IOC_NO_CRC);
	void	 SetIn       (void *pParam,InpFunc pProc    ,unsigned uCrc=IOC_NO_CRC);
	void	 SetOutFilter(void *pParam,MemProc pProc    ,int iBuffered=0);
	void	 SetOutFilter(void *pParam,MemCall pProc    ,int iBuffered=0);
	void	 SetOutFilter(void *pParam,MemFunc pProc    ,int iBuffered=0);
	void	 SetInFilter (void *pParam,MemProc pProc);
	void	 SetInFilter (void *pParam,MemCall pProc);
	void	 SetInFilter (void *pParam,MemFunc pProc);
	void	 SetOutCrcEx (iocint64 iCrc,void *pProc);	// Eine Funktion fr die CRC Berechung einstellen mit CRC Wert
	void	 SetInCrcEx  (iocint64 iCrc,void *pProc);	// Eine Funktion fr die CRC Berechung einstellen mit CRC Wert
	void	 SetOutCrc   (void *pParam ,void *pProc);	// Eine Funktion fr die CRC Berechung einstellen
	void	 SetInCrc    (void *pParam ,void *pProc);	// Eine Funktion fr die CRC Berechung einstellen
	void	 SetOutEnd   (void *pEnd);					// Memory-Ausgabe End-Punkt-Prfung aktiviren

	int		 InitWr		 (int iSize=IOC_BUFFER_SIZE);	// Schreibpuffer initialisieren
	int		 InitRd		 (int iSize=IOC_BUFFER_SIZE);	// Lesepuffer initialisieren
	int		 Fill		 (int iMin =IOC_BUFFER_SIZE/16);// Lesepuffer nachladen
	int		 Flush		 ();							// Schreibpuffer wegschreiben
	bool	 Eof		 ()const;						// Lesepuffer hat Stream-Ende ereicht

	int		 GetByte	 (         char     & cValue);	// Ein Byte  aus dem Lesepuffer holen
	int		 GetByte	 (unsigned char     &ucValue);	// Ein Byte  aus dem Lesepuffer holen
	int		 GetWord	 (         short    & sValue);	// Ein Word  aus dem Lesepuffer holen
	int		 GetWord	 (unsigned short    &usValue);	// Ein Word  aus dem Lesepuffer holen
	int		 GetDWord	 (         int      & iValue);	// Ein DWord aus dem Lesepuffer holen
	int		 GetDWord	 (unsigned int      &uiValue);	// Ein DWord aus dem Lesepuffer holen
	int		 GetQWord	 (         iocint64 & iValue);	// Ein QWord aus dem Lesepuffer holen
	int		 GetQWord	 (unsigned iocint64 &uiValue);	// Ein QWord aus dem Lesepuffer holen
	int		 GetData	 (void *pData,int iLen);		// Daten     aus dem Lesepuffer holen

	int		 PutByte	 (int      iValue);				// Ein Byte  in den Schreibpuffer geben
	int		 PutWord	 (int      iValue);				// Ein Word  in den Schreibpuffer geben
	int		 PutDWord	 (int      iValue);				// Ein DWord in den Schreibpuffer geben
	int		 PutQWord	 (iocint64 iValue);				// Ein QWord in den Schreibpuffer geben
	int		 PutData	 (const void *pData,int iLen);	// Daten     in den Schreibpuffer geben

	void    *ThisPtr	 ();							// Hole This-Zeiger fr GetCall/PutCall
	static int __cdecl GetCall(void *pPrm,void *p,int iLen);	// Daten aus dem Lesepuffer holen via Funktion
	static int __cdecl PutCall(void *p,const void *d,int i);	// Daten in den Schreibpuffer geben via Funktion

protected:

	void	 Clear		 (int iMode=0);					// Puffer lschen und Variablen rcksetzen
	static	unsigned	 uMemPosDummy;					// Dummy-Variable fr Speicherpositionen	

	static	int	__cdecl  ReadMem      (void *pParam,      char *pMem,int iLen);
	static	int	__cdecl WriteMem      (void *pParam,const char *pMem,int iLen);
	static	int	__cdecl  ReadFile     (void *pParam,      char *pMem,int iLen);
	static	int	__cdecl WriteFile     (void *pParam,const char *pMem,int iLen);
	static	int	__cdecl  ReadProc     (void *pParam,      char *pMem,int iLen);
	static	int	__cdecl WriteProc     (void *pParam,const char *pMem,int iLen);
	static	int	__cdecl  ReadFrom     (void *pParam,      char *pMem,int iLen);
	static	int	__cdecl WriteFrom     (void *pParam,const char *pMem,int iLen);
	static	int __cdecl  ReadFilter   (void *pParam,      char *pMem,int iLen);
	static	int	__cdecl WriteFilter   (void *pParam,const char *pMem,int iLen);
	static	int __cdecl WriteFilterBuf(void *pParam,const char *pMem,int iLen);

private:

	#ifdef _DEBUG
	int		m_iInpState;								// Aktueller Zustand (0=keines 1=SetIn  2=SetCrc 3=SetFilter)
	int		m_iOutState;								// Aktueller Zustand (0=keines 1=SetOut 2=SetCrc 3=SetFilter)
	#endif 	

};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

inline void	*CIoCallbacks::ThisPtr()
			{
			return this;
			}

inline void	CIoCallbacks::Reset()
			{
			m_uInpPos = 0;
			m_uOutPos = 0;
			}

inline int	CIoCallbacks::Read(void *pBuffer,int iSize)
			{
			int	iRet;
			iRet = m_pInpProc(m_pInpParam,(char*)pBuffer,iSize);
			if(iRet>0)m_uInpPos += iRet;
			return iRet;
			}

inline int	CIoCallbacks::Write(void *pBuffer,int iSize)
			{
			int	iRet;
			iRet = m_pOutProc(m_pOutParam,(char*)pBuffer,iSize);
			if(iRet>0)m_uOutPos += iRet;
			return iRet;
			}

inline int	CIoCallbacks::TakeOut(int iSize)
			{
			int iMax=m_uOutSize-m_uOutOffset;
			IOC_ASSERT(m_uOutMode==IOC_MEM);
			IOC_ASSERT(m_uOutOffset<=m_uOutSize);
			if(iSize<=0)return m_uOutOffset;
			if(iSize>iMax)iSize=iMax;
			if(m_pOutProcCrc)m_pOutProcCrc(m_pOutParamCrc,m_pOutBuffer+m_uOutOffset,iSize);
			m_pOutMemPos[0]	+= iSize;
			m_uOutOffset    += iSize;
			m_uOutPos       += iSize;
			if(m_pOutMemEnd && *m_pOutMemEnd<*m_pOutMemPos)*m_pOutMemEnd=*m_pOutMemPos;
			return m_uOutOffset;
			}

inline int	CIoCallbacks::TakeInp(int iSize)
			{
			int iMax=m_uInpSize-m_uInpOffset;
			IOC_ASSERT(m_uInpMode==IOC_MEM);
			IOC_ASSERT(m_uInpOffset<=m_uInpSize);
			if(iSize<=0)return m_uInpOffset;
			if(iSize>iMax)iSize=iMax;
			if(m_pInpProcCrc)m_pInpProcCrc(m_pInpParamCrc,m_pInpBuffer+m_uInpOffset,iSize);
			m_pInpMemPos[0]	+= iSize;
			m_uInpOffset    += iSize;
			m_uInpPos       += iSize;
			return m_uInpOffset;
			}

inline int	CIoCallbacks::PutByte(int iValue)
			{
			IOC_ASSERT(m_pBufferWr!=0 && m_iBufOffWr<=m_iBufLenWr);
			if(m_iBufOffWr+1>m_iBufLenWr && Flush())return 1;
			m_pBufferWr[m_iBufOffWr] = iValue;
			m_iBufOffWr++;
			return 0;
			}

inline int	CIoCallbacks::PutWord(int iValue)
			{
			IOC_ASSERT(m_pBufferWr!=0 && m_iBufOffWr<=m_iBufLenWr);
			if(m_iBufOffWr+2>m_iBufLenWr && Flush())return 1;
			*(short*)(m_pBufferWr+m_iBufOffWr) = iValue;
			m_iBufOffWr+=2;
			return 0;
			}

inline int	CIoCallbacks::PutDWord(int iValue)
			{
			IOC_ASSERT(m_pBufferWr!=0 && m_iBufOffWr<=m_iBufLenWr);
			if(m_iBufOffWr+4>m_iBufLenWr && Flush())return 1;
			*(int*)(m_pBufferWr+m_iBufOffWr) = iValue;
			m_iBufOffWr+=4;
			return 0;
			}

inline int	CIoCallbacks::PutQWord(iocint64 iValue)
			{
			IOC_ASSERT(m_pBufferWr!=0 && m_iBufOffWr<=m_iBufLenWr);
			if(m_iBufOffWr+8>m_iBufLenWr && Flush())return 1;
			*(iocint64*)(m_pBufferWr+m_iBufOffWr) = iValue;
			m_iBufOffWr+=8;
			return 0;
			}

inline int	CIoCallbacks::GetByte(unsigned char &ucValue){return GetByte(*(char*)&ucValue);}
inline int	CIoCallbacks::GetByte(char &cValue)
			{
			IOC_ASSERT(m_pBufferRd!=0 && m_iBufOffRd<=m_iBufLenRd);
			if(m_iBufOffRd+1>m_iBufLenRd && Fill(1)<1)return 1;
			cValue=m_pBufferRd[m_iBufOffRd];
			m_iBufOffRd++;
			return 0;
			}

inline int	CIoCallbacks::GetWord(unsigned short &usValue){return GetWord(*(short*)&usValue);}
inline int	CIoCallbacks::GetWord(short &sValue)
			{
			IOC_ASSERT(m_pBufferRd!=0 && m_iBufOffRd<=m_iBufLenRd);
			if(m_iBufOffRd+2>m_iBufLenRd && Fill(1)<2)return 1;
			sValue=*(short*)(m_pBufferRd+m_iBufOffRd);
			m_iBufOffRd+=2;
			return 0;
			}

inline int	CIoCallbacks::GetDWord(unsigned int &uiValue){return GetDWord(*(int*)&uiValue);}
inline int	CIoCallbacks::GetDWord(int &iValue)
			{
			IOC_ASSERT(m_pBufferRd!=0 && m_iBufOffRd<=m_iBufLenRd);
			if(m_iBufOffRd+4>m_iBufLenRd && Fill(1)<4)return 1;
			iValue=*(int*)(m_pBufferRd+m_iBufOffRd);
			m_iBufOffRd+=4;
			return 0;
			}
			
inline int	CIoCallbacks::GetQWord(unsigned iocint64 &uiValue){return GetQWord(*(iocint64*)&uiValue);}
inline int	CIoCallbacks::GetQWord(iocint64 &iValue)
			{
			IOC_ASSERT(m_pBufferRd!=0 && m_iBufOffRd<=m_iBufLenRd);
			if(m_iBufOffRd+8>m_iBufLenRd && Fill(1)<8)return 1;
			iValue=*(iocint64*)(m_pBufferRd+m_iBufOffRd);
			m_iBufOffRd+=8;
			return 0;
			}


inline iocint64	CIoCallbacks::GetPosOut   ()const{return  m_uOutPos;}
inline iocint64	CIoCallbacks::GetPosInp   ()const{return  m_uInpPos;}
inline int		CIoCallbacks::GetBufLen   ()const{return  IOC_BUFFER_SIZE;}
inline unsigned CIoCallbacks::GetCrcOut   ()const{return ~(unsigned)m_uOutCrc;}
inline unsigned CIoCallbacks::GetCrcInp   ()const{return ~(unsigned)m_uInpCrc;}
inline char    *CIoCallbacks::GetOutMem   ()const{IOC_ASSERT(m_uOutMode==IOC_MEM);return  m_pOutBuffer +m_uOutOffset;}
inline int		CIoCallbacks::GetOutLen   ()const{IOC_ASSERT(m_uOutMode==IOC_MEM);return  m_uOutSize   -m_uOutOffset;}
inline char    *CIoCallbacks::GetInpMem   ()const{IOC_ASSERT(m_uInpMode==IOC_MEM);return  m_pInpBuffer +m_uInpOffset;}
inline int		CIoCallbacks::GetInpLen   ()const{IOC_ASSERT(m_uInpMode==IOC_MEM);return  m_uInpSize   -m_uInpOffset;}
inline bool		CIoCallbacks::OutputEof   ()const{IOC_ASSERT(m_uOutMode==IOC_MEM);return  m_uOutOffset>=m_uOutSize  ;}
inline bool		CIoCallbacks::InputEof    ()const{IOC_ASSERT(m_uInpMode==IOC_MEM);return  m_uInpOffset>=m_uInpSize  ;}
inline bool		CIoCallbacks::Eof	      ()const{IOC_ASSERT(m_pBufferRd !=   0 );return  m_bBufEofRd;}
inline void	    CIoCallbacks::SetOutCrcEx (iocint64 iCrc,void *pProc){m_uOutCrc=iCrc;SetOutCrc(&m_uOutCrc,pProc);}
inline void	    CIoCallbacks::SetInCrcEx  (iocint64 iCrc,void *pProc){m_uInpCrc=iCrc;SetInCrc (&m_uInpCrc,pProc);}
inline void		CIoCallbacks::SetIn       (void *pParam,InpCall pProc,unsigned uCrc){SetIn       (pParam,(InpProc)pProc,uCrc     );}
inline void		CIoCallbacks::SetIn       (void *pParam,InpFunc pProc,unsigned uCrc){SetIn       (pParam,(InpProc)pProc,uCrc     );}
inline void		CIoCallbacks::SetOut      (void *pParam,OutCall pProc,unsigned uCrc){SetOut      (pParam,(OutProc)pProc,uCrc     );}
inline void		CIoCallbacks::SetOut      (void *pParam,OutFunc pProc,unsigned uCrc){SetOut      (pParam,(OutProc)pProc,uCrc     );}
inline void		CIoCallbacks::SetOut      (void *pParam,InpCall pProc,unsigned uCrc){SetOut      (pParam,(OutProc)pProc,uCrc     );}
inline void		CIoCallbacks::SetInFilter (void *pParam,MemCall pProc              ){SetInFilter (pParam,(MemProc)pProc          );}
inline void		CIoCallbacks::SetInFilter (void *pParam,MemFunc pProc              ){SetInFilter (pParam,(MemProc)pProc          );}
inline void		CIoCallbacks::SetOutFilter(void *pParam,MemCall pProc,int iBuffered){SetOutFilter(pParam,(MemProc)pProc,iBuffered);}
inline void		CIoCallbacks::SetOutFilter(void *pParam,MemFunc pProc,int iBuffered){SetOutFilter(pParam,(MemProc)pProc,iBuffered);}


//******************************************************************************
//*
//*		Klasse CBaseCompressor
//*
//******************************************************************************
class CBaseCompressor : public CIoCallbacks
{
public:

	CBaseCompressor();									// Konstruktor

	virtual	iocint64	Compress  (int iLevel=-1)=0;	// Daten komprimieren
	virtual	iocint64	Decompress(int iMode =-2)=0;	// Daten entkomprimieren
	virtual	void		Free      ()=0;					// Alle Puffer freigeben

	char	   *GetBuffer();							// Zeiger auf den Datenpuffers holen (nur wenn keine Speicherausgabe)
	int			GetMode  ()const;						// Aktuellen Kompressions-Modus abfragen
	int		 	GetError ()const;						// Aktuellen Fehlercode abfragen
	int		 	GetFlags ()const;						// Aktuellen Kompressions-Flags abfragen
	int		 	GetLevel ()const;						// Aktuellen Kompressions-Level abfragen
	iocint64	GetSize  ()const;						// Anzahl der Komprimierten Bytes abfragen
	iocint64	GetLength()const;						// Anzahl der Entkomprimierten Bytes abfragen
	int			SetMode  (int iMode);					// Modus einstellen
	int		 	SetFlags (int iFlags);					// Kompressions-Flags ndern
	int		 	SetLevel (int iLevel);					// Kompressions-Level ndern
	void		SetSize  (iocint64 iValue);				// Anzahl der komprimierten Bytes ndern
	void		SetLength(iocint64 iValue);				// Anzahl der entkomprimierten Bytes ndern

	unsigned	Value    (int iMode)const;				// Diverse Werte abfragen (32-Bit)
	iocint64	Value64  (int iMode)const;				// Diverse Werte abfragen (64-Bit)

protected:

	void		Clear(int iMode=0);						// Puffer lschen und Variablen rcksetzen

	uint64		m_uDecompresLen;						// Anzahl der entkomprimierten Bytes
	uint64		m_uCompresLen;							// Anzahl der komprimierten Bytes
	int			m_iModeFlags;							// Modus Flags (CBC_SLOW,CBC_FAST,CBC_NOCOMPESS)
	int			m_iModeLevel;							// Kompressions-Level  (1-9)
	int			m_iErrorCode;							// Aktuelle Fehlernummer (siehe CBC_ERROR_???)
	int			m_iMode;								// Modus (CBC_DEFLATE,CBC_STORE)

};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

inline int		CBaseCompressor::SetFlags    (int iFlags){if(iFlags!=-1)m_iModeFlags=iFlags;return m_iModeFlags;}
inline int		CBaseCompressor::SetLevel    (int iLevel){if(iLevel!=-1)m_iModeLevel=iLevel;return m_iModeLevel;}
inline int		CBaseCompressor::SetMode     (int iMode ){              m_iMode     =iMode ;return m_iMode     ;}
inline int		CBaseCompressor::GetMode     ()const{return  m_iMode;}
inline int		CBaseCompressor::GetError    ()const{return  m_iErrorCode;}
inline int		CBaseCompressor::GetLevel    ()const{return  m_iModeLevel;}
inline int		CBaseCompressor::GetFlags    ()const{return  m_iModeFlags;}
inline iocint64	CBaseCompressor::GetSize     ()const{return  m_uCompresLen;}
inline iocint64	CBaseCompressor::GetLength   ()const{return  m_uDecompresLen;}
inline void		CBaseCompressor::SetSize     (iocint64 iValue){m_uCompresLen  =iValue;}
inline void		CBaseCompressor::SetLength   (iocint64 iValue){m_uDecompresLen=iValue;}
inline char    *CBaseCompressor::GetBuffer   (){char *pBuffer=CIoCallbacks::GetBuffer();if(!pBuffer){m_iErrorCode=CBC_ERROR_MEMORY;throw "Zu wenig Speicher";}return pBuffer;}



#endif
