//*****************************************************************************
//*
//*
//*		LzmaClass.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//	Klasse zum Komprimieren von Daten mit dem LZMA Algorithmus von 7Zip.
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
//
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iMode =-2);			: Starte Dekomprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//

#ifndef		__CLZMACLASS_H__
#define		__CLZMACLASS_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#include	"IoCallbacks.h"


//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************


#define CLA_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CLA_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CLA_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CLA_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CLA_STORE			CBC_STORE			// Nur Gleich abspeichern

#define CLA_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CLA_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CLA_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CLA_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CLA_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CLA_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CLA_ERROR_BADBLZMA 	CBC_ERROR_BADFORMAT	// Ungltige Bzip2-Format
#define CLA_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zuwenig Speicher
#define CLA_ERROR_PARAM		CBC_ERROR_PARAM  	// Falsche Parameter

#define CLA_LEVEL			CBC_LEVEL			// Level ausgeben
#define CLA_FLAG			CBC_FLAG			// Flag Ausgeben
#define CLA_MODE			CBC_MODE			// Modus ausgeben
#define CLA_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CLA_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CLA_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CLA_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CLA_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CLA_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CLA_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes	
#define CLA_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CLA_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes	
#define CLA_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
												
#define CLA_NONE			IOC_NONE			// Ein/Ausgabe noch nicht definiert	
#define CLA_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CLA_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CLA_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CLA_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher



//*****************************************************************************
//*
//*		Klasse CLzmaClass
//*
//*****************************************************************************
class CLzmaClass : public CBaseCompressor
{
protected:

	char       *m_pStreamData;				// LZMA-Stream Daten fr Algorithmus

public:

					 CLzmaClass();
	virtual			~CLzmaClass();

	virtual	void	 Free ();
	virtual	iocint64 Compress  (int iLevel=-1);
	virtual	iocint64 Decompress(int iMode =-2);
	void			 SetDataSize(iocint64 iSize);
	int 			 SetProbs(const void *pData,unsigned uSize=5);

protected:

	void			Clear(int iMode=0);
	
	unsigned char 	m_aProbs[5];			// Speicher fr Properties
	iocint64		m_iDataBytes;			// Ausgabelnge (-1=Ende unbekannt)

	static int		CompressRead (void *pParam,      void *pData,size_t *pSize);
	static size_t	CompressWrite(void *pParam,const void *pData,size_t  uSize);

};


//*****************************************************************************
//*
//*		Inline Funktioen
//*
//*****************************************************************************
inline void CLzmaClass::SetDataSize(iocint64 iSize){m_iDataBytes=iSize;}

#endif
