//*****************************************************************************
//*
//*
//*		LzwClass.h
//*		
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2013
//
//	Klasse zum Komprimieren von Daten mit dem LWZ Algorithmus
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
// 												
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iOutLen,int iMode);	: Starte Dekomprimierung
// 												
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//

#ifndef		__LWZCLASS_H__
#define		__LWZCLASS_H__

#if _MSC_VER >= 1000
#pragma once
#endif 


#include	 "IoCallbacks.h"



//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define CLW_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CLW_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CLW_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CLW_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CLW_STORE			CBC_STORE			// Nur Gleich abspeichern

#define CLW_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CLW_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CLW_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CLW_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CLW_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CLW_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CLW_ERROR_BADFORMAT	CBC_ERROR_BADFORMAT	// Ungltige Bzip2-Format
#define CLW_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zuwenig Speicher

#define CLW_LEVEL			CBC_LEVEL			// Level ausgeben
#define CLW_FLAG			CBC_FLAG			// Flag Ausgeben
#define CLW_MODE			CBC_MODE			// Modus ausgeben
#define CLW_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CLW_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CLW_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CLW_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CLW_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CLW_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CLW_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes	
#define CLW_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CLW_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes	
#define CLW_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
												
#define CLW_NONE			IOC_NONE			// Ein/Ausgabe noch nicht definiert	
#define CLW_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CLW_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CLW_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CLW_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher


//*****************************************************************************
//*
//*		Klasse CLzwClass
//*
//*****************************************************************************
class CLzwClass : public CBaseCompressor
{
protected:

	typedef unsigned iocint64 unsigned64;
	typedef int				  FLAG;
	typedef unsigned short	  CODE;
	typedef unsigned char 	  BYTE;
	typedef unsigned int	  HASH;
	typedef unsigned int	  INTCODE;


	unsigned 				m_uMaxIn;					// LZW-Daten
	unsigned 				m_uNextIn;					//		*
	int						m_iTempSize;				// Grer des temorrer Puffers
	char				   *m_pTempPtr;					// Temorrer Puffer
	char		 		   *m_pTokens;					//		*	
	char		 		   *m_pSfxPtr;					//		*
	CODE		 		   *m_pPfxPtr;					//		*
	CODE		 		   *m_pHtPtr;					//		*
	INTCODE					m_uOldMaxCode;				//		*
	HASH					m_uOldHashSize;				//		*
	int						m_iOldBits;					//		*
	int						m_iMaxBits; 				//		*
	int						m_iMaxToklen;				//		*
	int						m_iBlockCompres;			//		*
	int						m_iRestBits;				//		*
	int						m_iBits;					//		*
	HASH					m_uHashSize;				//		*
	INTCODE					m_uMaxCode;					//		*
	INTCODE					m_uNextFree;				//		*
	INTCODE					m_uHighCode;				//		*
	INTCODE					m_uPrefxCode;				//		*
	unsigned char 			m_ucBitMemory;				//		*
	

	static const BYTE		m_aRmask[];
	static const BYTE		m_aMagicHeader[];
	static const INTCODE	m_aMcValues[];
	static const HASH		m_aHsValues[];


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************

public:

						CLzwClass();					
	virtual			   ~CLzwClass();
	
	virtual	iocint64	Compress  (int iLevel=-1);
	virtual	iocint64	Decompress(int iMode =-2);
	virtual	void		Free ();

protected:

	void		Clear(int iMode=0);
	void	 	PutCode(INTCODE uCode,int iBits);
	int		 	NextCode(INTCODE *pCodePtr);
	int		 	AllocTables(INTCODE uMaxCode,HASH uHashSize);
	int		 	GetChar();
	
void InitTables(unsigned hash);
};

#endif 

