//*****************************************************************************
//*
//*
//*		LzxClass.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//	Klasse zum Komprimieren von Daten mit dem LZX Algorithmus (fr *cap *.msi)
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
//
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iOutLen,int iMode);	: Starte Dekomprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<stdio.h>
#include	<string.h>
#include	"LzxClass.h"
#include	"LzxCompress.h"
#include	"LzxDecompress.h"
#include	"../Filter/Crc32.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	CLZ_BUFFER_SIZE		0x2000
#define 	CLZ_FILE_BUF		0x2000

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CLzxClass::CLzxClass()
{

	m_pStream		= NULL;							// Zeiger auf den aktellen LZX Stream

	Clear(1);

	m_iModeLevel	= 0;

}

//*****************************************************************************
//*
//*		Destrucktor
//*
//*****************************************************************************
CLzxClass::~CLzxClass()
{

	Clear(0);
}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//  0 :	Alles rcksetzen
//  1 :	Nur Variabeln rcksetzen , Puffer nicht lschen
//  2 :	Nur Puffer lschen
void CLzxClass::Clear(int iMode)
{

	CBaseCompressor::Clear(iMode);

	if(m_pStream)									// Den letzten Stream lschen
		{
		lzx_decompress_free((lzx_stream*)m_pStream);
		m_pStream	= NULL;
		}

	if(iMode==2)return;

	m_iOutLength	= 0;
	m_iReset		= 0;
	m_iNext			= 0;

}

//*****************************************************************************
//*
//*		Free
//*
//*****************************************************************************
void CLzxClass::Free()
{
	Clear(2);
}

//*****************************************************************************
//*
//*		SetParam
//*
//*****************************************************************************
//	Setzt die Endkomprimierungsparameter eines Streams
//	iOutLength	: Ist die Anzahl an Bytes die endkomprimiert werden sollen
//	iMode		: Bestimmt die Endkomprimierungsart
//					-2			= aktuellen Modus verwenden
//					 1...7		= 1<<n ist die Blockgre
//	iNext		: Soll der nchste Block dekodiert werden.
//	iReset		: Nach wievielen Blcken erfolgt ein Restet. (0=nicht ndern)
void CLzxClass::SetParam(int iOutLength,int iNext,int iReset)
{

	m_iOutLength	= iOutLength;
	m_iReset		= iReset;
	m_iNext			= iNext;

}

//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
//	Startet die Endkomprimierung eines Streams
//	iOutLength	: Ist die Anzahl an Bytes die endkomprimiert werden sollen
//	iMode		: Bestimmt die Endkomprimierungsart
//					-2			= aktuellen Modus verwenden
//					 1...7		= 1<<n ist die Blockgre
//	iNext		: Soll der nchste Block dekodiert werden.
//	iReset		: Nach wievielen Blcken erfolgt ein Restet. (0=nicht ndern)
//	Ergibt die Anzahl der Entkomprimierten Bytes
iocint64 CLzxClass::Decompress(int iOutLength,int iMode,int iNext,int iReset)
{

	m_iOutLength	= iOutLength;
	m_iReset		= iReset;
	m_iNext			= iNext;

return Decompress(iMode);
}

iocint64 CLzxClass::Decompress(int iMode)
{
lzx_stream *pStream;
int			iRet;


	IOC_ASSERT(m_uOutMode<CLZ_NONE && m_uInpMode<CLZ_NONE);

	if(iMode==-2)
		{
		iMode = m_iMode;
		}

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;

	Reset();

	m_iErrorCode	= CLZ_ERROR_OK;


	   iMode += 14;
	if(iMode<15)iMode=15;
	if(iMode>21)iMode=21;

	InitWr();


	if(!m_iNext)										// Den letzten Stream lschen
		{
		if(m_pStream)
			{
			lzx_decompress_free((lzx_stream*)m_pStream);
			m_pStream	= NULL;
			}

			pStream = lzx_decompress_init(m_pInpParam,m_pOutParam,iMode,m_iReset,4096,0,(lzx_ioproc_t)m_pInpProc,(lzx_ioproc_t)m_pOutProc);
		if(!pStream)
			{
			m_iErrorCode = CLZ_ERROR_MEMORY;
			throw "Zuwenig Speicher";
			}

		m_pStream = pStream;
		}
	else{
		if(!m_pStream)
			{
			throw "Stream nicht initialisiert";
			}

		pStream   = (lzx_stream*)m_pStream;
		}



	iRet = lzx_decompress(pStream,m_iOutLength);

	m_uDecompresLen	= m_uOutOffset;
	m_uCompresLen	= m_uInpOffset;


	switch(iRet)
		{
	case MSP_ENULLARG: 	m_iErrorCode = CLZ_ERROR_PARAM;
						throw "Falsche Parameter";

	case MSP_EFORMAT:	m_iErrorCode = CLZ_ERROR_BADBZIP;
						throw "Fehlerhaftes LWZ Stream";

	case MSP_EMEM: 		m_iErrorCode = CLZ_ERROR_MEMORY;
						throw "Zuwenig Speicher";

	case MSP_EIO_RD:	m_iErrorCode = CLZ_ERROR_READ;
						throw "Fehler beim Lesen";

	case MSP_EIO_WR:	m_iErrorCode = CLZ_ERROR_WRITE;
						throw "Fehler beim Schreiben";

	case MSP_EIO:		m_iErrorCode = CLZ_ERROR_IO;
						throw "Fehler bei IO-Operation";
		}





return m_uDecompresLen;
}


//*****************************************************************************
//*
//*		Compress
//*
//*****************************************************************************
//	Startet die Komprimierung eines Streams
//	iLevel		: Bestimmt den Kompimierungslevel (1..
//					-1	= aktuellen Level verwenden
//					 7	= maximale Framegre
//					 0	= minimale Framegre
//	Ergibt die Anzahl der komprimierten Bytes
iocint64 CLzxClass::Compress(int iLevel)
{
lzx_data   *pStream;
lzx_results sResult;
int			iRet;


	IOC_ASSERT(m_uOutMode<CLZ_NONE && m_uInpMode<CLZ_NONE);


	if(m_pStream)										// Den letzten Stream lschen
		{
		lzx_decompress_free((lzx_stream*)m_pStream);
		m_pStream	= NULL;
		}


	if(iLevel     != -1)m_iModeLevel=iLevel;			// Kompimierungslevel
	if(m_iModeLevel<  1)m_iModeLevel=1;
	if(m_iModeLevel>  7)m_iModeLevel=7;

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CLZ_ERROR_OK;
	iLevel			= m_iModeLevel+14;

	InitRd();
	InitWr();
	Reset ();

		pStream = lzx_compress_init(iLevel,(lzx_get_bytes_t)GetCall,ThisPtr(),(lzx_put_bytes_t)PutCall,ThisPtr());
	if(!pStream)
		{
		m_iErrorCode = CLZ_ERROR_MEMORY;
		throw "Zuwenig Speicher";
		}

	do	{
		iRet = lzx_compress(pStream,1<<iLevel,1);
		}while(!iRet && !pStream->at_eof);


	lzx_compress_finish(pStream,&sResult);

	Flush();

	m_uDecompresLen	= m_uInpOffset;
	m_uCompresLen	= m_uOutOffset;




return m_uCompresLen;
}



