//*****************************************************************************
//*
//*
//*		LzxClass.h
//*		
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//	Klasse zum Komprimieren von Daten mit dem LZX Algorithmus (fr *cap *.msi)
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
// 												
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iOutLen,int iMode);	: Starte Dekomprimierung
// 												
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//

#ifndef		__LZXCLASS_H__
#define		__LZXCLASS_H__

#if _MSC_VER >= 1000
#pragma once
#endif 

#include	"IoCallbacks.h"

//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define CLZ_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CLZ_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CLZ_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CLZ_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CLZ_STORE			CBC_STORE			// Nur Gleich abspeichern

#define CLZ_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CLZ_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CLZ_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CLZ_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CLZ_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CLZ_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CLZ_ERROR_BADBZIP  	CBC_ERROR_BADFORMAT	// Ungltige Bzip2-Format
#define CLZ_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zuwenig Speicher
#define CLZ_ERROR_PARAM  	CBC_ERROR_PARAM		// Falsche Parameter
#define CLZ_ERROR_IO		CBC_ERROR_IO		// IO-Fehler	

#define CLZ_LEVEL			CBC_LEVEL			// Level ausgeben
#define CLZ_FLAG			CBC_FLAG			// Flag Ausgeben
#define CLZ_MODE			CBC_MODE			// Modus ausgeben
#define CLZ_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CLZ_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CLZ_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CLZ_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CLZ_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CLZ_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CLZ_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes	
#define CLZ_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CLZ_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes	
#define CLZ_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
												
#define CLZ_NONE			IOC_NONE			// Ein/Ausgabe noch nicht definiert													
#define CLZ_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CLZ_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CLZ_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CLZ_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher


//*****************************************************************************
//*
//*		Klasse CLzxClass
//*
//*****************************************************************************
class CLzxClass : public CBaseCompressor 
{
public:

						CLzxClass();					
	virtual			   ~CLzxClass();
	
	virtual	void		Free    ();
	virtual	iocint64	Compress  (int iLevel=-1);
	virtual	iocint64	Decompress(int iMode =-2);
	        iocint64	Decompress(int iOutLength,int iMode,int iNext=0,int iReset=0);
			void 		SetParam  (int iOutLength,          int iNext=0,int iReset=0);

protected:

	void				Clear(int iMode=0);

	void			   *m_pStream;					// Zeiger auf den aktellen LZX Stream
	int					m_iOutLength;
	int					m_iReset;
	int					m_iNext;


};


#endif 
