//*****************************************************************************
//*
//*
//*		LzxCompress.c
//*
//*
//*****************************************************************************
//
//  File lzx_layer.c, part of lzxcomp library
//  Copyright (C) 2002 Matthew T. Russotto
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation; version 2.1 only
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>											// for memset on Linux
#include	<math.h>

#include	"LzxCompress.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define		MIN_MATCH					2
#define		MAX_MATCH					257
#define		NUM_CHARS					256

// number of position slots given window_size-5
// as corrected by Caie
static	short			num_position_slots[] = {30, 32, 34, 36, 38, 42, 50};
static	unsigned long	position_base[51];
static	unsigned char	extra_bits[52];

struct ih_elem_str
	{
	int					freq;
	short				sym;
	short				pathlength;
	struct ih_elem_str *parent;
	struct ih_elem_str *right;
	struct ih_elem_str *left;
	};

struct h_elem_str
	{
	int					freq;
	short				sym;
	short				pathlength;
	struct ih_elem_str *parent;
	unsigned short		code;
	};


typedef struct ih_elem_str	ih_elem;
typedef struct h_elem_str	h_elem;


//*****************************************************************************
//*
//*		lxz_init_sub
//*
//*****************************************************************************
static void lxz_init_sub(	lxz_info		   *lzi,
							int					wsize,
							int					max_dist,
							int					max_match,
							int					min_match,
							int					frame_size,
							get_chars_t			get_chars,
							output_match_t		output_match,
							output_literal_t	output_literal,
							void			   *user_data)
{
	// the reason for the separate max_dist value is LZX can't reach the
	// first three characters in its nominal window.  But using a smaller
	// window results in inefficiency when dealing with reset intervals
	// which are the length of the nominal window

	lzi->wsize = wsize;
	if(max_match > wsize)
		{
		lzi->max_match = wsize;
		}
	else{
		lzi->max_match = max_match;
		}

	   lzi->min_match = min_match;
	if(lzi->min_match<3)lzi->min_match = 3;

	lzi->max_dist		= max_dist;
	lzi->block_buf_size = wsize + lzi->max_dist;
	lzi->block_buf		= new unsigned char[lzi->block_buf_size];
	lzi->block_bufe 	= lzi->block_buf  + lzi->block_buf_size;
	lzi->eofcount		= 0;
	lzi->cur_loc		= 0;
	lzi->block_loc		= 0;
	lzi->chars_in_buf	= 0;
	lzi->analysis_valid = 0;
	lzi->get_chars		= get_chars;
	lzi->output_match	= output_match;
	lzi->output_literal = output_literal;
	lzi->frame_size 	= frame_size;
	lzi->user_data		= user_data;
	lzi->lentab 		= new int           [lzi->block_buf_size];
	lzi->prevtab		= new unsigned char*[lzi->block_buf_size];

}

//*****************************************************************************
//*
//*		lxz_release
//*
//*****************************************************************************
static void lxz_release(lxz_info *lzi)
{
	delete lzi->block_buf;
	delete lzi->prevtab;
	delete lzi->lentab;
}

//*****************************************************************************
//*
//*		lxz_reset
//*
//*****************************************************************************
static void lxz_reset(lxz_info *lzi)
{
int		residual;


	residual = lzi->chars_in_buf - lzi->block_loc;

	memmove(lzi->block_buf, lzi->block_buf + lzi->block_loc, residual);

	lzi->chars_in_buf	= residual;
	lzi->block_loc		= 0;
	lzi->analysis_valid = 0;

}

//*****************************************************************************
//*
//*		lxz_left_to_process
//*
//*****************************************************************************
static int lxz_left_to_process(lxz_info *lzi)
{

return lzi->chars_in_buf - lzi->block_loc;
}

//*****************************************************************************
//*
//*		fill_blockbuf
//*
//*****************************************************************************
static void fill_blockbuf(lxz_info *lzi, int maxchars)
{
int				toread;
int				nread;
unsigned char  *readhere;



	if(lzi->eofcount) return;

	maxchars   -= lxz_left_to_process(lzi);
	toread		= lzi->block_buf_size - lzi->chars_in_buf;

	if(toread > maxchars) toread = maxchars;

	readhere			= lzi->block_buf + lzi->chars_in_buf;
	nread				= lzi->get_chars(lzi, toread, readhere);
	lzi->chars_in_buf  += nread;

	if(nread != toread)
		{
		lzi->eofcount++;
		}
}

//*****************************************************************************
//*
//*		lxz_analyze_block
//*
//*****************************************************************************
static void lxz_analyze_block(lxz_info *lzi)
{
int			   *lentab, *lenp;
unsigned char **prevtab, **prevp;
unsigned char  *bbp, *bbe;
unsigned char  *chartab[256];
unsigned char  *cursor;
int				prevlen;
int				ch;
int				maxlen;
long			wasinc;
int				max_dist = lzi->max_dist;






	memset(chartab, 0, sizeof(chartab));
	prevtab = prevp = lzi->prevtab;
	lentab	= lenp = lzi->lentab;
	memset(prevtab, 0, sizeof(*prevtab) * lzi->chars_in_buf);
	memset(lentab, 0, sizeof(*prevtab) * lzi->chars_in_buf);

	bbp = lzi->block_buf;
	bbe = bbp + lzi->chars_in_buf;

	while(bbp < bbe)
		{
		if(chartab[ch = *bbp])
			{
			*prevp	= chartab[ch];
			*lenp	= 1;
			}

		chartab[ch] = bbp;
		bbp++;
		prevp++;
		lenp++;
		}

	wasinc = 1;

	for(maxlen = 1; wasinc && (maxlen < lzi->max_match); maxlen++)
		{

		bbp 	= bbe - maxlen - 1;
		lenp	= lentab + lzi->chars_in_buf - maxlen - 1;
		prevp	= prevtab + lzi->chars_in_buf - maxlen - 1;
		wasinc	= 0;

		while(bbp > lzi->block_buf)
			{
			if(*lenp == maxlen)
				{

				ch		=  bbp[maxlen];
				cursor	= *prevp;

				while(cursor && ((bbp - cursor) <= max_dist))
					{
					prevlen = *(cursor - lzi->block_buf + lentab);
					if(cursor[maxlen] == ch)
						{
						*prevp = cursor;
						(*lenp)++;
						wasinc++;
						break;
						}

					if(prevlen != maxlen) break;
					cursor = *(cursor - lzi->block_buf + prevtab);
					}
				}

			bbp--;
			prevp--;
			lenp--;
			}
		}

	lzi->analysis_valid = 1;

}

//*****************************************************************************
//*
//*		lxz_stop_compressing
//*
//*****************************************************************************
static void lxz_stop_compressing(lxz_info *lzi)
{
	lzi->stop = 1;
}

//*****************************************************************************
//*
//*		lxz_do_compress
//*
//*****************************************************************************
static int lxz_do_compress(lxz_info *lzi, int nchars)
{
unsigned char  *bbp,     *bbe;
int			   *lentab,  *lenp;
unsigned char **prevtab,**prevp;
int				len;
int				holdback;
short			trimmed;



	lzi->stop = 0;

	while((lxz_left_to_process(lzi) || !lzi->eofcount) && !lzi->stop && nchars>0)
		{
		#if 1
		if(!lzi->analysis_valid ||
		  (!lzi->eofcount &&
		 ((lzi->chars_in_buf- lzi->block_loc) < nchars)))
			{
			int residual = lzi->chars_in_buf - lzi->block_loc;
			int bytes_to_move = lzi->max_dist + residual;
			if(bytes_to_move > lzi->chars_in_buf)
				{
				bytes_to_move = lzi->chars_in_buf;
				}

			memmove(lzi->block_buf, lzi->block_buf+ lzi->chars_in_buf- bytes_to_move,bytes_to_move);

			lzi->block_loc		= bytes_to_move - residual;
			lzi->chars_in_buf	= bytes_to_move;
			fill_blockbuf(lzi, nchars);
			lxz_analyze_block(lzi);
			}
		#else

		if(!lzi->analysis_valid || lzi->block_loc-lzi->chars_in_buf==0)
			{
			lzi->block_loc		= 0;
			lzi->chars_in_buf	= 0;
			fill_blockbuf(lzi, nchars);
			lxz_analyze_block(lzi);
			}
		#endif

		prevtab 	= prevp = lzi->prevtab + lzi->block_loc;
		lentab		= lenp = lzi->lentab + lzi->block_loc;
		bbp 		= lzi->block_buf + lzi->block_loc;
		holdback	= lzi->max_match;

		if(lzi->eofcount) holdback = 0;
		if(lzi->chars_in_buf < (nchars + lzi->block_loc))
			{
			bbe = lzi->block_buf + lzi->chars_in_buf - holdback;
			}
		else{
			bbe = bbp + nchars;
			}

		while((bbp < bbe) && (!lzi->stop))
			{
			trimmed =  0;
			len 	= *lenp;
			if(lzi->frame_size && (len > (lzi->frame_size - lzi->cur_loc % lzi->frame_size)))
				{
				trimmed =  1;
				len 	= (lzi->frame_size - lzi->cur_loc % lzi->frame_size);
				}

			if(len > nchars)
				{
				trimmed = 1;
				len 	= nchars;
				}

			if(len >= lzi->min_match)
				{
				#ifdef		LAZY
				if((bbp < bbe -1) && !trimmed && ((lenp[1] > (len + 1)) /* || ((lenp[1] == len) && (prevp[1] > prevp[0])) */))
					{
					len = 1;									// this is the lazy eval case
					}
				else
				#endif
				if(lzi->output_match(lzi, (int)(*prevp - lzi->block_buf) - lzi->block_loc,len) < 0)
					{
					len = 1;									// match rejected
					}
				}
			else{
				len = 1;
				}

			if(len < lzi->min_match)
				{
				lzi->output_literal(lzi, *bbp);
				}

			bbp 		   += len;
			prevp		   += len;
			lenp		   += len;
			lzi->cur_loc   += len;
			lzi->block_loc += len;
			nchars         -= len;

			}
		}

return 0;
}

//*****************************************************************************
//*
//*		cmp_leaves
//*
//*****************************************************************************
static int cmp_leaves(const void *in_a, const void *in_b)
{
const h_elem	*a = (h_elem*)in_a;
const h_elem	*b = (h_elem*)in_b;



	if(!a->freq && b->freq)
		{
		return 1;
		}

	if(a->freq && !b->freq)
		{
		return -1;
		}

	if(a->freq == b->freq)
		{
		return a->sym - b->sym;
		}


return a->freq - b->freq;
}

//*****************************************************************************
//*
//*		cmp_pathlengths
//*
//*****************************************************************************
static int cmp_pathlengths(const void *in_a, const void *in_b)
{
const h_elem	*a = (h_elem*)in_a;
const h_elem	*b = (h_elem*)in_b;


	if(a->pathlength == b->pathlength)
		{
		return b->sym - a->sym;
		}


return b->pathlength - a->pathlength;
}

//*****************************************************************************
//*
//*		build_huffman_tree
//*
//*****************************************************************************
//  standard huffman building algorithm
static void build_huffman_tree(int nelem, int max_code_length, int *freq, huff_entry*tree)
{
h_elem		   *leaves;
ih_elem		   *inodes;
ih_elem		   *next_inode;
ih_elem		   *cur_inode;
h_elem		   *cur_leaf;
ih_elem		   *f1, *f2;
unsigned short	cur_code;
short			codes_too_long;
int				leaves_left;
int				pathlength;
int				nleaves;
int				i;



	leaves			= new h_elem[nelem];
	codes_too_long	= 0;


	for(i = 0; i < nelem; i++)
		{
		leaves[i].freq			= freq[i];
		leaves[i].sym			= i;
		leaves[i].pathlength	= 0;
		}

	qsort(leaves, nelem, sizeof(h_elem), cmp_leaves);

	for(leaves_left = 0; leaves_left < nelem; leaves_left++)
		{
		if(!leaves[leaves_left].freq) break;
		}

	nleaves = leaves_left;

	if(nleaves >= 2)
		{
		inodes = new ih_elem[nelem-1];

		do	{
			if(codes_too_long)
				{
				for(leaves_left = 0; leaves_left < nelem; leaves_left++)
					{
					if(!leaves[leaves_left].freq) break;
					if(leaves[leaves_left].freq != 1)
						{
						leaves[leaves_left].freq  >>= 1;
						codes_too_long				= 0;
						}
					}
				}

			cur_leaf	= leaves;
			next_inode	= cur_inode = inodes;

			do	{
				f1 = f2 = NULL;
				if(leaves_left && (cur_inode==next_inode || cur_leaf->freq<=cur_inode->freq))
					{
					f1 = (ih_elem *)cur_leaf++;
					leaves_left--;
					}
				else if(cur_inode!=next_inode)
					{
					f1 = cur_inode++;
					}

				if(leaves_left && (cur_inode==next_inode || cur_leaf->freq<=cur_inode->freq))
					{
					f2 = (ih_elem *)cur_leaf++;
					leaves_left--;
					}
				else if(cur_inode!=next_inode)
					{
					f2 = cur_inode++;
					}

				if(f1 && f2)
					{
					next_inode->freq	=  f1->freq + f2->freq;
					next_inode->sym 	= -1;
					next_inode->left	=  f1;
					next_inode->right	=  f2;
					next_inode->parent	=  NULL;
					f1->parent			=  next_inode;
					f2->parent			=  next_inode;

					if(f1->pathlength > f2->pathlength)
						{
						next_inode->pathlength = f1->pathlength + 1;
						}
					else{
						next_inode->pathlength = f2->pathlength + 1;
						}

					if(next_inode->pathlength > max_code_length)
						{
						codes_too_long = 1;
						break;
						}

					next_inode++;
					}
				}while(f1 && f2);
			}while(codes_too_long);



		cur_inode				=  next_inode - 1;				// now traverse tree depth-first
		pathlength				=  0;
		cur_inode->pathlength	= -1;

		do	{
			if(cur_inode->sym == -1)							// precondition: at unmarked node
				{
				cur_inode				=  cur_inode->left;		// left node of unmarked node is unmarked
				cur_inode->pathlength	= -1;
				pathlength++;
				}
			else{
				cur_inode->pathlength = pathlength;				// mark node

																// time to come up.	 Keep coming up until an unmarked node is reached
																// or the tree is exhausted
				do	{
					cur_inode = cur_inode->parent;
					pathlength--;
					}while(cur_inode && (cur_inode->pathlength != -1));

				if(cur_inode)
					{
																// found unmarked node; mark it and go right
					cur_inode->pathlength	=  pathlength;
					cur_inode				=  cur_inode->right;
					cur_inode->pathlength	= -1;
					pathlength++;
																// would be complex if cur_inode could be null here.	It can't
					}
				}
			}while(cur_inode);

		delete inodes;

		qsort(leaves, nelem, sizeof(h_elem), cmp_pathlengths);	// the pathlengths are already in order, so this sorts by symbol


		// Microsoft's second condition on its canonical huffman codes is:
		//
		// For each level, starting at the deepest level of the tree and then
		// moving upwards, leaf nodes must start as far left as possible. An
		// alternative way of stating this constraint is that if any tree node
		// has children then all tree nodes to the left of it with the same path
		// length must also have children.
		//
		// These 'alternatives' are not equivalent.  The latter alternative gives
		// the common canonical code where the longest code is all zeros.  The former
		// gives an opposite code where the longest code is all ones.  Microsoft uses the
		// former alternative.


		pathlength	= leaves[nleaves-1].pathlength;
		cur_code	= 0;

		for(i=nleaves-1;i>=0;i--)
			{
			while(leaves[i].pathlength > pathlength)
				{
				cur_code <<= 1;
				pathlength++;
				}

			leaves[i].code = cur_code;
			cur_code++;
			}

		}
	else if(nleaves == 1)
		{
		// 0 symbols is OK (not according to doc, but according to Caie)
		// but if only one symbol is present, two symbols are required

		nleaves 			   = 2;
		leaves[0].pathlength   = leaves[1].pathlength = 1;

		if(leaves[1].sym > leaves[0].sym)
			{
			leaves[1].code = 1;
			leaves[0].code = 0;
			}
		else{
			leaves[0].code = 1;
			leaves[1].code = 0;
			}
		}

	memset(tree, 0, nelem * sizeof(huff_entry));

	for(i = 0; i < nleaves; i++)
		{
		tree[leaves[i].sym].codelength	= leaves[i].pathlength;
		tree[leaves[i].sym].code		= leaves[i].code;
		}

	delete leaves;

}

//*****************************************************************************
//*
//*		lzx_init_static
//*
//*****************************************************************************
//  from Stuart Caie's code -- I'm hoping this code is too small to encumber
//  this file.  If not, you could rip it out and hard-code the tables
static void lzx_init_static(void)
{
int			i, j;



	if(extra_bits[49])return;

	for(i=0, j=0; i <= 50; i += 2)
		{
		extra_bits[i] = extra_bits[i+1] = j;					// 0,0,0,0,1,1,2,2,3,3...
		if(i!=0 && j<17) j++;									// 0,0,1,2,3,4...15,16,17,17,17,17...
		}

	for(i=0, j=0; i <= 50; i++)
		{
		position_base[i]  = j;									// 0,1,2,3,4,6,8,12,16,24,32,...
		j				 += 1 << extra_bits[i]; 				// 1,1,1,1,2,2,4,4,8,8,16,16,32,32,...
		}
}


//*****************************************************************************
//*
//*		lzx_get_chars
//*
//*****************************************************************************
static int _cdecl lzx_get_chars(void *lzi_ptr,int n,unsigned char *buf)
{
lxz_info   *lzi=(lxz_info*)lzi_ptr;
int			chars_read;
int			chars_pad;
int			num;

	// force lz compression to stop after every block

	lzx_data *lzud = (lzx_data *)lzi->user_data;


	#ifdef		OLDFRAMING

	if(lzud->subdivide < 0) return 0;

	if(n > lzud->left_in_frame)
		{
		n = lzud->left_in_frame;
		}

	if(n > lzud->left_in_block)
		{
		n = lzud->left_in_block;
		}
	#endif

	for(chars_read=0;;)
		{
		   num = lzud->get_bytes(lzud->in_arg,buf+chars_read,n-chars_read);
		if(num<=0)
			{
			if(num==0)break;

			lzud->lzi->stop = 1;
			lzud->rd_error  = 1;
			lzud->at_eof    = 1;
			chars_read      = 0;
			break;
			}

		   chars_read+=num;
		if(chars_read>=n)break;
		}


	#ifdef		OLDFRAMING
	lzud->left_in_frame -= chars_read;
	lzud->left_in_block -= chars_read;
	#else
	   lzud->left_in_frame -= chars_read % LZX_FRAME_SIZE_C;
	if(lzud->left_in_frame < 0)
		{
		lzud->left_in_frame += LZX_FRAME_SIZE_C;
		}
	#endif


	if(chars_read<n)
		{
		lzud->at_eof = 1;

		if(lzud->left_in_frame)
			{
			   chars_pad = n - chars_read;
			if(chars_pad > lzud->left_in_frame) chars_pad = lzud->left_in_frame;

			// never emit a full frame of padding.	This prevents silliness when lzx_compress is called when at EOF but EOF not yet detected

			if(chars_pad == LZX_FRAME_SIZE_C) chars_pad = 0;

			#ifdef		OLDFRAMING
			if(chars_pad > lzud->left_in_block) chars_pad = lzud->left_in_block;
			#endif

			memset(buf + chars_read, 0, chars_pad);
			lzud->left_in_frame -= chars_pad;

			#ifdef		OLDFRAMING
			lzud->left_in_block -= chars_pad;
			#endif

			chars_read += chars_pad;
			}
		}


return chars_read;
}

//*****************************************************************************
//*
//*		find_match_at
//*
//*****************************************************************************
static int find_match_at(lxz_info *lzi, int loc, int match_len, int *match_locp)
{
unsigned char	   *matchb;
unsigned char	   *nmatchb;
unsigned char	   *c1, *c2;
int					j;



	if(-*match_locp == loc) return -1;
	if(loc < match_len    ) return -1;

	matchb	= lzi->block_buf + lzi->block_loc + *match_locp;
	nmatchb = lzi->block_buf + lzi->block_loc - loc;
	c1		= matchb;
	c2		= nmatchb;


	for(j = 0; j < match_len; j++)
		{
		if(*c1++ != *c2++) break;
		}

	if(j == match_len)
		{
		*match_locp = -loc;
		return 0;
		}


return -1;
}
//*****************************************************************************
//*
//*		check_entropy
//*
//*****************************************************************************
static void check_entropy(lzx_data *lzud, int main_index)
{
	// entropy = -      sum_alphabet P(x)   * log2 P(x)
	// entropy = -      sum_alphabet f(x)/N * log2(f(x)/N)
	// entropy = - 1/N  sum_alphabet f(x)   *(log2 f(x)  - log2 N)
	// entropy = - 1/N (sum_alphabet f(x)   * log2 f(x)) - sum_alphabet f(x) log2 N
	// entropy = - 1/N (sum_alphabet f(x)   * log2 f(x)) - log2 N sum_alphabet f(x)
	// entropy = - 1/N (sum_alphabet f(x)   * log2 f(x)) - N * log2 N

	// entropy = - 1/N ((sum_alphabet f(x) * log2 f(x) ) - N * log2 N)
	// entropy = - 1/N ((sum_alphabet f(x) * ln f(x) * 1/ln 2) - N * ln N * 1/ln 2)
	// entropy =   1/(N ln 2) (N * ln N - (sum_alphabet f(x) * ln f(x)))
	// entropy =   1/(N ln 2) (N * ln N + (sum_alphabet -f(x) * ln f(x)))

	// entropy =   1/(N ln 2) ( sum_alphabet ln N * f(x) + (sum_alphabet -f(x) * ln f(x)))
	// entropy =   1/(N ln 2) ( sum_alphabet ln N * f(x) +	(-f(x) * ln f(x)))
	// entropy = - 1/(N ln 2) ( sum_alphabet-ln N * f(x) +  (f(x) * ln f(x)))
	// entropy = - 1/(N ln 2) ( sum_alphabet f(x)(- ln N  + ln f(x)))
	// entropy = - 1/(N ln 2) ( sum_alphabet f(x)(ln f(x)/N))
	// entropy = - 1/N	      ( sum_alphabet (1/(ln 2))f(x)(ln f(x)/N))
	// entropy = - 1/N	      ( sum_alphabet f(x)(log2 f(x)/N))
	// entropy = -            ( sum_alphabet f(x)/N(log2 f(x)/N))
	// entropy = -            ( sum_alphabet P(x)(log2 P(x)))


	double	freq;
	double	n_ln_n;
	double	rn_ln2;
	double	cur_ratio;
	int		n;


	if(lzud->main_freq_table[main_index] != 1)			// delete old entropy accumulation
		{
		freq				= (double)lzud->main_freq_table[main_index]-1;
		lzud->main_entropy +=  freq * log(freq);
		}


														// add new entropy accumulation

	freq				= (double)lzud->main_freq_table[main_index];
	lzud->main_entropy -= freq * log(freq);
	n					= (int)(lzud->block_codesp - lzud->block_codes);

	if(((n&0x0FFF)==0) && (lzud->left_in_block >= 0x1000))
		{
		n_ln_n		= (double)n * log((double)n);
		rn_ln2		=  1.4426950408889634073599246810019 / (double)n;
		cur_ratio	= (n * rn_ln2 *(n_ln_n + lzud->main_entropy) + 24 + 3 * 80 + NUM_CHARS + (lzud->main_tree_size-NUM_CHARS)*3 + LZX_NUM_SECONDARY_LEN ) / (double)n;

		if(cur_ratio > lzud->last_ratio)
			{
			lzud->subdivide = -1;
			lxz_stop_compressing(lzud->lzi);
			}

		lzud->last_ratio = cur_ratio;
		}
}

//*****************************************************************************
//*
//*		lzx_output_match
//*
//*****************************************************************************
static int _cdecl lzx_output_match(void *lzi_ptr, int match_pos, int match_len)
{
lxz_info       *lzi  = (lxz_info*)lzi_ptr;
lzx_data	   *lzud = (lzx_data*)lzi->user_data;
unsigned		formatted_offset;
unsigned		position_footer;
unsigned char	length_footer;
unsigned char	length_header;
unsigned short	len_pos_header;
int				position_slot;
short			btdt;



	position_footer = 0;
	btdt			= 0;

testforr:

	if(match_pos == -lzud->R0)
		{
		match_pos			= 0;
		formatted_offset	= 0;
		position_slot		= 0;
		}
	else if(match_pos == -lzud->R1)
		{
		lzud->R1			=  lzud->R0;
		lzud->R0			= -match_pos;
		match_pos			=  1;
		formatted_offset	=  1;
		position_slot		=  1;
		}
	else if(match_pos == -lzud->R2)
		{
		lzud->R2			=  lzud->R0;
		lzud->R0			= -match_pos;
		match_pos			=  2;
		formatted_offset	=  2;
		position_slot		=  2;
		}
	else{
		if(!btdt)
			{
			btdt = 1;
			if(find_match_at(lzi, lzud->R0, match_len, &match_pos) == 0)
				{
				goto testforr;
				}

			if(find_match_at(lzi, lzud->R1, match_len, &match_pos) == 0)
				{
				goto testforr;
				}

			if(find_match_at(lzi, lzud->R2, match_len, &match_pos) == 0)
				{
				goto testforr;
				}
			}

		formatted_offset = -match_pos + 2;

		if((match_len < 3) ||
		  ((formatted_offset >=    64) && (match_len < 4)) ||
		  ((formatted_offset >=  2048) && (match_len < 5)) ||
		  ((formatted_offset >= 65536) && (match_len < 6)))
			{
			// reject matches where extra_bits will likely be bigger than just outputting
			// literals.  The numbers are basically derived through guessing and trial and error

			return -1;											// reject the match
			}

		lzud->R2	=  lzud->R1;
		lzud->R1	=  lzud->R0;
		lzud->R0	= -match_pos;

		//	calculate position base using binary search of table; if log2 can be
		//	done in hardware, approximation might work;
		//	trunc(log2(formatted_offset*formatted_offset)) gets either the proper
		//	position slot or the next one, except for slots 0, 1, and 39-49
		//
		//	Slots 0-1 are handled by the R0-R1 procedures
		//
		//	Slots 36-49 (formatted_offset >= 262144) can be found by
		//	(formatted_offset/131072) + 34 ==
		//	(formatted_offset >>  17) + 34;

		if(formatted_offset >= 262144)
			{
			position_slot = (formatted_offset >> 17) + 34;
			}
		else{
			int left, right, mid;

			left			=  3;
			right			=  lzud->num_position_slots - 1;
			position_slot	= -1;
			while(left <= right)
				{
				mid = (left + right)/2;

				if(position_base[mid  ] <= formatted_offset &&
				   position_base[mid+1] >  formatted_offset)
					{
					position_slot = mid;
					break;
					}

				if(formatted_offset > position_base[mid])		// too low
					{
					left = mid + 1;
					}
				else{											// too high
					right = mid;
					}
				}

			// FIXME precalc extra_mask table
			}

		position_footer = ((1UL << extra_bits[position_slot]) - 1) & formatted_offset;
		}

	// match length    =  8 bits
	// position_slot   =  6 bits
	// position_footer = 17 bits
	// total           = 31 bits
	// plus one to say whether it's a literal or not

	*lzud->block_codesp++ = 0x80000000 |						// bit  31 in intelligent bit ordering
							(position_slot   << 25) |			// bits 30-25
							(position_footer <<  8) |			// bits  8-24
							(match_len - MIN_MATCH);			// bits  0-7

	if(match_len < (LZX_NUM_PRIMARY_LEN + MIN_MATCH))
		{
		length_header = match_len - MIN_MATCH;
		// length_footer = 255;									// not necessary
		}
	else{
		length_header = LZX_NUM_PRIMARY_LEN;
		length_footer = match_len - (LZX_NUM_PRIMARY_LEN + MIN_MATCH);
		lzud->length_freq_table[length_footer]++;
		}

	len_pos_header = (position_slot << 3) | length_header;
	lzud->main_freq_table[len_pos_header + NUM_CHARS]++;

	if(extra_bits[position_slot] >= 3)
		{
		lzud->aligned_freq_table[position_footer & 7]++;
		}

	#ifndef		OLDFRAMING
	lzud->left_in_block -= match_len;
	#endif

	if(lzud->subdivide)
		{
		check_entropy(lzud, len_pos_header + NUM_CHARS);
		}


return 0; // accept the match
}

//*****************************************************************************
//*
//*		lzx_output_literal
//*
//*****************************************************************************
static void _cdecl lzx_output_literal(void  *lzi_ptr, unsigned char ch)
{
lxz_info   *lzi  = (lxz_info*)lzi_ptr;
lzx_data   *lzud = (lzx_data*)lzi->user_data;



	#ifndef		OLDFRAMING
	lzud->left_in_block--;
	#endif

	*lzud->block_codesp++ = ch;


	   lzud->main_freq_table[ch]++;
	if(lzud->subdivide)
		{
		check_entropy(lzud, ch);
		}
}

//*****************************************************************************
//*
//*		lzx_write_bits
//*
//*****************************************************************************
static void lzx_write_bits(lzx_data *lzxd, int nbits, unsigned bits)
{
int				out_num;
int				cur_bits;
int				shift_bits;
int				rshift_bits;
unsigned short	mask_bits;




	cur_bits = lzxd->bits_in_buf;

	while(cur_bits+nbits >= 16)
		{
		 shift_bits	= 16    -   cur_bits;
		rshift_bits = nbits - shift_bits;

		if(shift_bits == 16)
			{
			lzxd->bit_buf = (bits>>rshift_bits) & 0xFFFF;
			}
		else{
			mask_bits		= (1U << shift_bits) - 1;
			lzxd->bit_buf <<=  shift_bits;
			lzxd->bit_buf  |= (bits>>rshift_bits) & mask_bits;
			}

		#ifdef		LZX_BIG_ENDIAN
		lzxd->bit_buf = ((lzxd->bit_buf & 0xFF)<<8) | (lzxd->bit_buf >> 8);
		#endif

		   out_num = lzxd->put_bytes(lzxd->out_arg,&lzxd->bit_buf,sizeof(lzxd->bit_buf));
		if(out_num!=sizeof(lzxd->bit_buf))
			{
			if(out_num<0)
				{
				lzxd->lzi->stop = 1;
				lzxd->wr_error  = 1;
				lzxd->at_eof    = 1;
				out_num		    = 0;
				}
			}

		lzxd->len_compressed_output += out_num;
		lzxd->bit_buf		         = 0;
		cur_bits			         = 0;
		nbits				        -= shift_bits;
		}

	shift_bits		   =  nbits;
	 mask_bits		   = (1U << shift_bits) - 1;
	lzxd->bit_buf    <<=  shift_bits;
	lzxd->bit_buf     |=  bits & mask_bits;
	  cur_bits	      +=  nbits;
	lzxd->bits_in_buf  =  cur_bits;

}

//*****************************************************************************
//*
//*		lzx_align_output
//*
//*****************************************************************************
static void lzx_align_output(lzx_data *lzxd)
{
	if(lzxd->bits_in_buf)
		{
		lzx_write_bits(lzxd, 16 - lzxd->bits_in_buf, 0);
		}
}

//*****************************************************************************
//*
//*		lzx_write_compressed_literals
//*
//*****************************************************************************
static void lzx_write_compressed_literals(lzx_data *lzxd, int block_type)
{
unsigned	   *cursor = lzxd->block_codes;
unsigned	   *endp = lzxd->block_codesp;
unsigned short	position_slot;
unsigned		position_footer;
unsigned		match_len_m2;									// match length minus 2, which is MIN_MATCH
unsigned		verbatim_bits;
unsigned		block_code;
unsigned short	length_header;
unsigned short	length_footer;
unsigned short	len_pos_header;
huff_entry	   *huffe;
int				frame_count;


	frame_count = (lzxd->len_uncompressed_input % LZX_FRAME_SIZE_C);
	lzxd->len_uncompressed_input -= frame_count;				// will be added back in later


	while(cursor < endp)
		{
		   block_code = *cursor++;
		if(block_code & 0x80000000)
			{
			// 0x80000000 |                   bit  31 in intelligent bit ordering
			// (position_slot << 25) |        bits 30-25
			// (position_footer << 8) |       bits 8-24
			// (match_len - MIN_MATCH);       bits 0-7


			match_len_m2	=  block_code & 0xFF;				// 8 bits
			position_footer = (block_code >> 8)& 0x0001FFFF;	// 17 bits
			position_slot	= (block_code >> 25) & 0x3F;		// 6 bits

			if(match_len_m2 < LZX_NUM_PRIMARY_LEN)
				{
				length_header = match_len_m2;
				length_footer = 255;							// personal encoding for NULL
				}
			else{
				length_header = LZX_NUM_PRIMARY_LEN;
				length_footer = match_len_m2 - LZX_NUM_PRIMARY_LEN;
				}

			len_pos_header	= (position_slot << 3) | length_header;
			huffe			=  &lzxd->main_tree[len_pos_header+NUM_CHARS];
			lzx_write_bits(lzxd,huffe->codelength,huffe->code);

			if(length_footer != 255)
				{
				huffe = &lzxd->length_tree[length_footer];
				lzx_write_bits(lzxd, huffe->codelength, huffe->code);
				}

			if((block_type == LZX_ALIGNED_OFFSET_BLOCK) && (extra_bits[position_slot] >= 3))
				{
				// aligned offset block and code
				verbatim_bits = position_footer >> 3;
				lzx_write_bits(lzxd, extra_bits[position_slot] - 3, verbatim_bits);
				huffe = &lzxd->aligned_tree[position_footer&7];
				lzx_write_bits(lzxd, huffe->codelength, huffe->code);
				}
			else{
				verbatim_bits = position_footer;
				lzx_write_bits(lzxd, extra_bits[position_slot], verbatim_bits);
				}

			frame_count += match_len_m2 + 2;
			}
		else{
			huffe = &lzxd->main_tree[block_code];				// literal
			lzx_write_bits(lzxd, huffe->codelength, huffe->code);
			frame_count++;
			}

		if(frame_count == LZX_FRAME_SIZE_C)
			{
			lzxd->len_uncompressed_input += frame_count;
			lzx_align_output(lzxd);
			frame_count = 0;
			}
		}

	lzxd->len_uncompressed_input += frame_count;
}

//*****************************************************************************
//*
//*		lzx_write_compressed_tree
//*
//*****************************************************************************
static int lzx_write_compressed_tree(lzx_data *lzxd,huff_entry *tree, unsigned char *prevlengths,int treesize)
{
unsigned char  *runp;
unsigned char  *runs;
unsigned char  *codes;
unsigned char  *codep;
unsigned char  *codee;
huff_entry		pretree[20];
int				freqs[LZX_PRETREE_SIZE];
int				cur_run;
int				cur_code;
int				last_len;
int				excess;
int				i;



	codep	= codes = new unsigned char [treesize];
	runp	= runs  = new unsigned char [treesize];

	memset(freqs,0,sizeof(freqs));

	cur_run 	= 1;
	last_len	= tree[0].codelength;

	for(i = 1; i <= treesize; i++)
		{
		if(i==treesize || tree[i].codelength!=last_len)
			{
			if(last_len == 0)
				{
				while(cur_run >= 20)
					{
					   excess =	cur_run - 20;
					if(excess > 31) excess = 31;

					*codep++ = 18;
					*runp++  = excess;
					cur_run -= excess + 20;

					freqs[18]++;
					}

				while(cur_run >= 4)
					{
					   excess =	cur_run - 4;
					if(excess > 15) excess = 15;

					*codep++ = 17;
					*runp++  = excess;
					cur_run -= excess + 4;

					freqs[17]++;
					}

				while(cur_run > 0)
					{
					*codep = prevlengths[i - cur_run];
					freqs[*codep++]++;
					*runp++ = 0;								// not necessary
					cur_run--;
					}
				}
			else{
				while(cur_run >= 4)
					{
					if(cur_run == 4)
							excess = 0;
					else	excess = 1;

					*codep++ = 19;
					*runp++ = excess;
					freqs[19]++;

					/* right, MS lies again.	Code is NOT prev_len + len (mod 17), it's prev_len - len (mod 17)*/
					*codep = prevlengths[i-cur_run] - last_len;

					if(*codep > 16) *codep += 17;

					freqs[*codep++]++;
					*runp++ = 0;								// not necessary
					cur_run -= excess+4;
					}

				while(cur_run > 0)
					{
					*codep = prevlengths[i-cur_run] - last_len;
					if(*codep > 16) *codep += 17;
					*runp++ = 0;								// not necessary
					cur_run--;
					freqs[*codep++]++;
					}
				}

			if(i != treesize)
				{
				last_len = tree[i].codelength;
				}

			cur_run = 0;
			}

		cur_run++;
		}

	codee = codep;


	build_huffman_tree(LZX_PRETREE_SIZE, 16, freqs, pretree);	// now create the huffman table and write out the pretree

	for(i=0;i<LZX_PRETREE_SIZE;i++)
		{
		lzx_write_bits(lzxd, 4, pretree[i].codelength);
		}


	codep	= codes;
	runp	= runs;
	cur_run = 0;


	while(codep < codee)
		{
		cur_code = *codep++;
		lzx_write_bits(lzxd, pretree[cur_code].codelength, pretree[cur_code].code);
		if(cur_code == 17)
			{
			cur_run += *runp + 4;
			lzx_write_bits(lzxd, 4, *runp);
			}
		else if(cur_code == 18)
			{
			cur_run += *runp + 20;
			lzx_write_bits(lzxd, 5, *runp);
			}
		else if(cur_code == 19)
			{
			cur_run += *runp + 4;
			lzx_write_bits(lzxd, 1, *runp);
			cur_code = *codep++;
			lzx_write_bits(lzxd, pretree[cur_code].codelength, pretree[cur_code].code);
			runp++;
			}
		else{
			cur_run++;
			}

		runp++;
		}

	delete codes;
	delete runs;



return 0;
}

//*****************************************************************************
//*
//*		lzx_compress_reset
//*
//*****************************************************************************
void lzx_compress_reset(lzx_data *lzxd)
{

	lzxd->need_1bit_header	= 1;
	lzxd->R0				= lzxd->R1 = lzxd->R2 = 1;

	memset(lzxd->prev_main_treelengths  , 0, lzxd->main_tree_size);
	memset(lzxd->prev_length_treelengths, 0, LZX_NUM_SECONDARY_LEN);

	lxz_reset(lzxd->lzi);
}

//*****************************************************************************
//*
//*		lzx_compress
//*
//*****************************************************************************
int lzx_compress(lzx_data *lzxd, int block_size, int subdivide)
{
int			i;
int			block_type;
unsigned	written_sofar;
long		uncomp_bits;
long		uncomp_length;
long		comp_bits_ovh;
long		comp_bits;



	written_sofar = 0;

	if(lzxd->block_size!=block_size || lzxd->block_codes==NULL)
		{
		if(lzxd->block_codes!=NULL)delete lzxd->block_codes;
		lzxd->block_size	= block_size;
		lzxd->block_codes	= new unsigned[block_size];
		}

	lzxd->subdivide		= (subdivide)? 1:0;
	lzxd->left_in_block = block_size;
	lzxd->left_in_frame = LZX_FRAME_SIZE_C;
	lzxd->main_entropy	= 0.0;
	lzxd->last_ratio	= 9999999.0;
	lzxd->block_codesp	= lzxd->block_codes;

	memset(lzxd->aligned_freq_table, 0, LZX_ALIGNED_SIZE      * sizeof(int));
	memset(lzxd->length_freq_table , 0, LZX_NUM_SECONDARY_LEN * sizeof(int));
	memset(lzxd->main_freq_table   , 0, lzxd->main_tree_size  * sizeof(int));

	do	{
		lxz_do_compress(lzxd->lzi, lzxd->left_in_block);

		if(lzxd->left_in_frame == 0)
			{
			lzxd->left_in_frame = LZX_FRAME_SIZE_C;
			}

		if(lzxd->subdivide<0 || !lzxd->left_in_block || (!lxz_left_to_process(lzxd->lzi) && lzxd->at_eof))
			{
																	// now one block is LZ-analyzed time to write it out

			uncomp_length = lzxd->block_size-lzxd->left_in_block-written_sofar;

			if(uncomp_length == 0)									// uncomp_length will sometimes be 0 when input length is an exact multiple of frame size
				{
				continue;
				}

			if(lzxd->subdivide < 0)
				{
				lzxd->subdivide = 1;
				}

			if(lzxd->need_1bit_header)
				{													// one bit Intel preprocessing header
				lzx_write_bits(lzxd, 1, 0);							// always 0 because this implementation doesn't do Intel preprocessing
				lzxd->need_1bit_header = 0;
				}


			uncomp_bits = comp_bits = 0;							// handle extra bits
			build_huffman_tree(LZX_ALIGNED_SIZE, 7, lzxd->aligned_freq_table, lzxd->aligned_tree);


			for(i = 0; i < LZX_ALIGNED_SIZE; i++)
				{
				uncomp_bits    += lzxd->aligned_freq_table[i]* 3;
				comp_bits	   += lzxd->aligned_freq_table[i]* lzxd->aligned_tree[i].codelength;
				}

			   comp_bits_ovh = comp_bits + LZX_ALIGNED_SIZE * 3;
			if(comp_bits_ovh < uncomp_bits)
				{
				block_type = LZX_ALIGNED_OFFSET_BLOCK;
				}
			else{
				block_type = LZX_VERBATIM_BLOCK;
				}

			lzx_write_bits(lzxd,  3, block_type);					// block type
			lzx_write_bits(lzxd, 24, uncomp_length);				// uncompressed length

			written_sofar = lzxd->block_size - lzxd->left_in_block;


			if(block_type == LZX_ALIGNED_OFFSET_BLOCK)				// now write out the aligned offset trees if present
				{
				for(i = 0; i < LZX_ALIGNED_SIZE; i++)
					{
					lzx_write_bits(lzxd, 3, lzxd->aligned_tree[i].codelength);
					}
				}

																	// end extra bits
			build_huffman_tree(lzxd->main_tree_size, LZX_MAX_CODE_LENGTH,lzxd->main_freq_table, lzxd->main_tree);
			build_huffman_tree(LZX_NUM_SECONDARY_LEN, 16,lzxd->length_freq_table, lzxd->length_tree);



																	// now write the pre-tree and tree for main 1
			lzx_write_compressed_tree(lzxd, lzxd->main_tree, lzxd->prev_main_treelengths, NUM_CHARS);

																	// now write the pre-tree and tree for main 2
			lzx_write_compressed_tree(lzxd, lzxd->main_tree+ NUM_CHARS,lzxd->prev_main_treelengths+ NUM_CHARS,lzxd->main_tree_size- NUM_CHARS);

																	// now write the pre tree and tree for length
			lzx_write_compressed_tree(lzxd, lzxd->length_tree, lzxd->prev_length_treelengths,LZX_NUM_SECONDARY_LEN);

																	// now write literals
			lzx_write_compressed_literals(lzxd, block_type);


			for(i = 0; i < lzxd->main_tree_size; i++) 				// copy treelengths somewhere safe to do delta compression
				{
				lzxd->prev_main_treelengths[i] = (unsigned char)lzxd->main_tree[i].codelength;
				}

			for(i = 0; i < LZX_NUM_SECONDARY_LEN; i++)
				{
				lzxd->prev_length_treelengths[i] = (unsigned char)lzxd->length_tree[i].codelength;
				}

			lzxd->main_entropy	= 0.0;
			lzxd->last_ratio	= 9999999.0;
			lzxd->block_codesp	= lzxd->block_codes;

			memset(lzxd->length_freq_table , 0, LZX_NUM_SECONDARY_LEN * sizeof(int));
			memset(lzxd->main_freq_table   , 0, lzxd->main_tree_size  * sizeof(int));
			memset(lzxd->aligned_freq_table, 0, LZX_ALIGNED_SIZE      * sizeof(int));
			}

		}while(lzxd->left_in_block && (lxz_left_to_process(lzxd->lzi) || !lzxd->at_eof));



	if(lzxd->wr_error)return -1;
	if(lzxd->rd_error)return -2;



return 0;
}

//*****************************************************************************
//*
//*		lzx_compress_init
//*
//*****************************************************************************
lzx_data *lzx_compress_init(int					wsize_code,
							lzx_get_bytes_t		get_bytes,
							void			   *get_bytes_arg,
							lzx_put_bytes_t		put_bytes,
							void		       *put_bytes_arg)
{
int				wsize;
lzx_data	   *lzxd;



	if((wsize_code < 15) || (wsize_code > 21))
		{
		return NULL;
		}

	lzx_init_static();

	   lzxd = new lzx_data;
	if(lzxd == 0)
		{
		return NULL;
		}


	wsize							= 1 << (wsize_code);
	lzxd->in_arg					= get_bytes_arg;
	lzxd->out_arg					= put_bytes_arg;
	lzxd->get_bytes 				= get_bytes;
	lzxd->put_bytes 				= put_bytes;
	lzxd->at_eof					= 0;
	lzxd->wr_error					= 0;
	lzxd->rd_error					= 0;
	lzxd->bits_in_buf				= 0;
	lzxd->block_codes				= NULL;
	lzxd->num_position_slots		= num_position_slots[wsize_code-15];
	lzxd->main_tree_size			= (NUM_CHARS + 8 * lzxd->num_position_slots);
	lzxd->main_freq_table			= new int          [lzxd->main_tree_size];
	lzxd->main_tree 				= new huff_entry   [lzxd->main_tree_size];
	lzxd->prev_main_treelengths 	= new unsigned char[lzxd->main_tree_size];
	lzxd->lzi						= new lxz_info;
	lzxd->len_compressed_output 	= 0;
	lzxd->len_uncompressed_input	= 0;

	lxz_init_sub(lzxd->lzi, wsize, wsize-3, MAX_MATCH, MIN_MATCH, LZX_FRAME_SIZE_C, lzx_get_chars, lzx_output_match, lzx_output_literal,lzxd);
	lzx_compress_reset(lzxd);



return lzxd;
}

//*****************************************************************************
//*
//*		lzx_compress_finish
//*
//*****************************************************************************
int lzx_compress_finish(lzx_data *lzxd,lzx_results *lzxr)
{

	//  lzx_align_output(lzxd);  Not needed as long as frame padding is in place
	if(lzxr)
		{
		lzxr->len_compressed_output 	= lzxd->len_compressed_output;
		lzxr->len_uncompressed_input	= lzxd->len_uncompressed_input;
		}

	lxz_release(lzxd->lzi);

	delete lzxd->prev_main_treelengths;
	delete lzxd->main_freq_table;
	delete lzxd->main_tree;
	delete lzxd->lzi;
	delete lzxd;


return 0;
}

