//*****************************************************************************
//*
//*
//*		LzxCompress.h
//*
//*
//*****************************************************************************
//
//  File lzx_compress.h, part of lzxcomp library
//  Copyright (C) 2002 Matthew T. Russotto
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation; version 2.1 only
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#ifndef 	__LZX_COMPRESS_H__
#define 	__LZX_COMPRESS_H__



// the names of these constants are specific to this library 
#define		LZX_MAX_CODE_LENGTH 		16
#define		LZX_NUM_PRIMARY_LEN 		7
#define		LZX_NUM_SECONDARY_LEN		249
#define		LZX_FRAME_SIZE_C			32768
#define		LZX_PRETREE_SIZE			20
#define		LZX_ALIGNED_BITS			3
#define		LZX_ALIGNED_SIZE			8
#define		LZX_VERBATIM_BLOCK			1
#define		LZX_ALIGNED_OFFSET_BLOCK	2



typedef int  (_cdecl *lzx_get_bytes_t )(void *arg, void *buf, int n);
typedef int  (_cdecl *lzx_put_bytes_t )(void *arg, void *buf, int n);

typedef struct huff_entry 
	{
	short				codelength;
	unsigned short		code;
	}huff_entry; 


typedef int  (_cdecl *get_chars_t     )(void *lzi, int n, unsigned char  *buf);
typedef int  (_cdecl *output_match_t  )(void *lzi, int match_pos, int match_len);
typedef void (_cdecl *output_literal_t)(void *lzi, unsigned char ch);


typedef struct 
	{
	int					wsize;										// window size in bytes 
	int					max_match;									// size of longest match in bytes 
	int					min_match;
	unsigned char	   *block_buf;
	unsigned char	   *block_bufe;
	int					block_buf_size;
	int					chars_in_buf;
	int					cur_loc;									// location within stream 
	int					block_loc;
	int					frame_size;
	int					max_dist;
	unsigned char	  **prevtab;
	int				   *lentab;
	short				eofcount;
	short				stop;
	short				analysis_valid;
	get_chars_t			get_chars;
	output_match_t		output_match;
	output_literal_t	output_literal;
	void			   *user_data;
	}lxz_info;


typedef struct 
	{
	void			   *in_arg;
	void			   *out_arg;
	void			   *mark_frame_arg;
	lzx_get_bytes_t		get_bytes;
	lzx_put_bytes_t		put_bytes;
	lxz_info	       *lzi;
	int					at_eof;
	int					wr_error;
	int					rd_error;
	int					left_in_frame;								// a 'frame' is an 0x8000 byte thing.	 Called that because otherwise I'd confuse myself overloading 'block' 
	int 				left_in_block;
	int 				R0, R1, R2;
	int 				num_position_slots;
	int 				block_size;									// this is the LZX block size 
	int				   *main_freq_table;
	int					length_freq_table[LZX_NUM_SECONDARY_LEN];
	int					aligned_freq_table[LZX_ALIGNED_SIZE];
	unsigned		   *block_codes;
	unsigned		   *block_codesp;
	huff_entry		   *main_tree;
	huff_entry			length_tree[LZX_NUM_SECONDARY_LEN];
	huff_entry			aligned_tree[LZX_ALIGNED_SIZE];
	int					main_tree_size;
	unsigned short 		bit_buf;
	int					bits_in_buf;
	double				main_entropy;
	double				last_ratio;
	unsigned char 	   *prev_main_treelengths;
	unsigned char		prev_length_treelengths[LZX_NUM_SECONDARY_LEN];
	unsigned			len_uncompressed_input;
	unsigned			len_compressed_output;
	short				need_1bit_header;
	short				subdivide;									// 0 = don't subdivide, 1 = allowed, -1 = requested 
	}lzx_data;

typedef struct
	{
	long len_compressed_output;										// add more here? Error codes, # blocks, # frames, etc? 
	long len_uncompressed_input;
	}lzx_results; 

void		lzx_compress_reset (lzx_data *lzxd);
int			lzx_compress	   (lzx_data *lzxd, int block_size, int subdivide);
int			lzx_compress_finish(lzx_data *lzxd, lzx_results *lzxr);
lzx_data   *lzx_compress_init  (int					wsize_code,
								lzx_get_bytes_t		get_bytes, 
								void			   *get_bytes_arg,
								lzx_put_bytes_t		put_bytes,  
								void			   *put_bytes_arg);




#endif 	