//*****************************************************************************
//*
//*
//*		PpmdClass.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//	Klasse zum Komprimieren von Daten mit dem PPMD Algorithmus (WinZip/7Zip)
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
//
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iOutLen,int iMode);	: Starte Dekomprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<stdio.h>
#include	<string.h>
#include	<malloc.h>
#include	"PpmClass.h"
#include	"../Filter/Crc32.h"



#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

//	GetByte
//	PutData

typedef		int					PPM_BOOL;
typedef 	unsigned char		PPM_BYTE;
typedef 	unsigned short		PPM_WORD;
typedef 	unsigned int		PPM_DWORD;
typedef 	CPpmClass		    PPM_STREAM;


#define		PPM_USE_PREFETCHING									// for puzzling mainly			

#ifndef 	FALSE
#define		FALSE				0
#endif 	
#ifndef 	TRUE
#define		TRUE				1
#endif

#ifdef		_MSC_VER
#define		PPM_FASTCALL		__fastcall
#define		PPM_STDCALL			__stdcall
#else
#define		PPM_FASTCALL					
#define		PPM_STDCALL					
#endif 

#ifdef		__GNUC__
#define		PPM_PACK_ATTR		__attribute__((packed))
#else 
#define		PPM_PACK_ATTR					
#endif


enum MR_METHOD 
	{
	MRM_RESTART = 0, 
	MRM_CUT_OFF = 1, 
	MRM_FREEZE  = 2, 
	};

#define 	PPM_LOCK()			if(!hPpmSem)hPpmSem=CreateSemaphore(0,1,0x40000000,NULL);WaitForSingleObject(hPpmSem,INFINITE)
#define 	PPM_UNLOCK()		ReleaseSemaphore(hPpmSem,1,NULL)		

#define		PPM_E_GETC(fp)		fp->GetByteEx( )
#define		PPM_E_PUTC(c,fp)	fp->PutByteEx(c)
#define		PPM_D_GETC(fp)		fp->GetByteEx( )


PPM_BOOL	PPM_STDCALL StartSubAllocator(PPM_DWORD uSubAllocatorSize);
void		PPM_STDCALL StopSubAllocator();								// it can be called once		
PPM_DWORD	PPM_STDCALL GetUsedMemory(); 								// for information only			
void		PPM_STDCALL EncodeFile(PPM_STREAM* EncodedFile,PPM_STREAM* DecodedFile,int iMaxOrder,MR_METHOD uMrMethod);
void		PPM_STDCALL DecodeFile(PPM_STREAM* DecodedFile,PPM_STREAM* EncodedFile,int iMaxOrder,MR_METHOD uMrMethod);


#define		GET_MEAN(SUMM,SHIFT,ROUND)	((SUMM+(1<<(SHIFT-ROUND)))>>(SHIFT))
#define		P_CALL(p,f)					(PrefetchData(p->pSuccessor),p->pSuccessor=p->pSuccessor->f(iOrder+1))


#define 	TOP			(1<<24)
#define 	BOT			(1<<15) 
#define 	MAX_FREQ	124 
#define 	MAX_O		16
#define 	UNIT_SIZE	12
#define 	O_BOUND		9 
#define 	UP_FREQ		5 
#define 	INT_BITS	7 
#define 	PERIOD_BITS	7 
#define 	INTERVAL	(1<<INT_BITS) 
#define 	BIN_SCALE	(1<<TOT_BITS) 
#define 	N_INDEXES	(N1+N2+N3+N4)
#define 	TOT_BITS	(INT_BITS+PERIOD_BITS)
#define 	N4			((128+3-1*N1-2*N2-3*N3)/4)
#define 	N3			4 
#define 	N2			4 
#define 	N1			4 


static struct SUBRANGE 
	{
	PPM_DWORD			uLow; 
	PPM_DWORD			uHigh; 
	PPM_DWORD			uScale; 
	}sRange; 


static	HANDLE			hPpmSem			= 0;
const	PPM_DWORD		uVariant		= 'I';
const	PPM_DWORD		uPpmSignature	= 0x84ACAF8F;
static	PPM_BYTE		aUnitsToIndex[128]; 					
static	PPM_BYTE		aIndexToUnits[N_INDEXES];
static	PPM_DWORD		uSubAllocatorSize=0;
static	PPM_DWORD		uGlueCount;
static	PPM_BYTE	   *pUnitsStart;
static	PPM_BYTE	   *pHeapStart;
static	PPM_BYTE	   *pLoUnit;
static	PPM_BYTE	   *pHiUnit;
static	PPM_BYTE	   *pText;
static	PPM_DWORD		uRange;
static	PPM_DWORD		uCode;
static	PPM_DWORD		uLow;
struct	PPM_CONTEXT; 
	  	
static const PPM_WORD	aInitBinEsc[8]=
		{
		0x3CDD,0x1F3F,0x59BF,0x48F3,
		0x64A1,0x5ABC,0x6632,0x6051,
		};

static const PPM_BYTE aExpEscape[16]=
		{
		25,14, 9, 7, 5, 5, 4, 4, 
		 4, 3, 3, 3, 2, 2, 2, 2,
		};
	


#pragma	pack(1) 					

typedef	struct  
	{
	PPM_BYTE		ucSymbol;
	PPM_BYTE		ucFreq;									// 3. contexts example
	PPM_CONTEXT    *pSuccessor; 							// maximal order
	}PPM_PACK_ATTR PPM_STATE;								// ABCD	context


struct BLK_NODE 
	{
	PPM_DWORD		uStamp;
	BLK_NODE	   *pNext;

	inline PPM_BOOL Avail() const 
		{
		return (pNext!=NULL); 
		}

	inline void Link(BLK_NODE* pNode) 
		{
		pNode->pNext = pNext; 
		pNext		 = pNode; 
		}

	inline void Unlink()			  
		{
		pNext = pNext->pNext; 
		}

	BLK_NODE *Remove()			  
		{
		BLK_NODE   *pNode;
		
		pNode = pNext;					
		Unlink();
		uStamp--;							
		
		return pNode;
		}

	void Insert(void *pMem,int iNu);

	}aBlkList[N_INDEXES]; 


struct MEM_BLK: public BLK_NODE 
	{
	PPM_DWORD	uNu; 
	}PPM_PACK_ATTR; 


struct SEE2_CONTEXT 
	{
	PPM_WORD			usSumm;
	PPM_BYTE			ucShift;
	PPM_BYTE			ucCount;

	void Init(PPM_DWORD uInitVal) 
		{
		usSumm  = uInitVal<<(PERIOD_BITS-4); 
		ucShift	= PERIOD_BITS-4;
		ucCount = 7; 
		}

	PPM_DWORD GetMean() 
		{
		PPM_DWORD	uRetVal;		
		
		uRetVal	= usSumm>>ucShift;		
		usSumm -= uRetVal;
		
		return uRetVal+(uRetVal==0);
		}

	void Update() 
		{
		if(ucShift<PERIOD_BITS && --ucCount == 0) 
			{
			usSumm += usSumm;					
			ucCount = 3<<ucShift++;
			}
		}
	}PPM_PACK_ATTR;


struct PPM_CONTEXT 
	{
	PPM_BYTE		ucNumStats;	
	PPM_BYTE		ucFlags;								// 1. ucNumStats & ucNumMasked contain
	PPM_WORD		usSummFreq;								// number of symbols minus 1
	PPM_STATE	   *pStats;	
	PPM_CONTEXT    *pSuffix;								// BCD	suffix

	inline void		EncodeBinSymbol(int iSymbol);			// BCDE	 successor
	inline void		EncodeSymbol1  (int iSymbol);			// other orders:
	inline void		EncodeSymbol2  (int iSymbol);			// BCD	 context
	inline void		DecodeBinSymbol();						// CD	 suffix
	inline void		DecodeSymbol1();						// BCDE	 successor
	inline void		DecodeSymbol2();
	inline void		Update1(PPM_STATE* pState);
	inline void		Update2(PPM_STATE* pState);
	PPM_CONTEXT    *CutOff(int iOrder);
	PPM_CONTEXT    *RemoveBinConts(int iOrder);
	PPM_STATE	   &OneState()const {return (PPM_STATE&)usSummFreq;}
	void			Refresh(int uOldNu,PPM_BOOL bScale);
	void			Rescale();
	void			MakeSuffix();
	SEE2_CONTEXT   *MakeEscFreq2();
	}PPM_PACK_ATTR;

#pragma	pack()						


static PPM_STATE   *pFoundState; 							// found pNext state transition
static PPM_CONTEXT *pMaxContext;
static SEE2_CONTEXT	aDummySeeToCont;
static SEE2_CONTEXT	aSeeToCont[24][32];
static PPM_WORD		aBinSumm  [25][64];						// binary SEE-contexts
static PPM_BYTE		aNsToBsIndex[256];
static PPM_BYTE		aQTable     [260];
static PPM_BYTE		aCharMask   [256];
static PPM_BYTE		ucPrevSuccess;
static PPM_BYTE		ucNumMasked;
static PPM_BYTE		ucEscCount;
static MR_METHOD	uMrMethod;
static int			iOrderFall;
static int			iRunLength;
static int			iMaxOrder;
static int			iInitEsc;
static int			iInitRl;


//*****************************************************************************
//*
//*		RcInitEncoder
//*
//*****************************************************************************
static inline void RcInitEncoder() 
{
	uLow	= 0; 
	uRange	= 0xFFFFFFFF; 
}

//*****************************************************************************
//*
//*		RcEncNormalize
//*
//*****************************************************************************
static inline void RcEncNormalize(PPM_STREAM *pStream)	
{																					

	while((uLow^(uLow+uRange))<TOP || uRange<BOT && ((uRange=-(int)uLow&(BOT-1)),1))
		{                             
		PPM_E_PUTC(uLow>>24,pStream);
		uRange <<= 8;				
		uLow   <<= 8;                
		}                           
		
}

//*****************************************************************************
//*
//*		RcEncodeSymbol
//*
//*****************************************************************************
static inline void RcEncodeSymbol()
{
	uRange /= sRange.uScale;
	uLow   += sRange.uLow *uRange;	
	uRange *= sRange.uHigh-sRange.uLow;
}

//*****************************************************************************
//*
//*		RcFlushEncoder
//*
//*****************************************************************************
static inline void RcFlushEncoder(PPM_STREAM *pStream)
{
int 	i;

	
	for(i=0;i<4;i++) 
		{
		PPM_E_PUTC(uLow>>24,pStream);		
		uLow <<= 8;
		}
}

//*****************************************************************************
//*
//*		RcInitDecoder
//*
//*****************************************************************************
static inline void RcInitDecoder(PPM_STREAM *pStream)
{
int		i;
	
	uLow   = 0;
	uCode  = 0;								
	uRange = 0xFFFFFFFF;

	for(i=0;i<4;i++)
		{
		uCode <<= 8;
		uCode  |= PPM_D_GETC(pStream);
		}
}

//*****************************************************************************
//*
//*		RcDecNormalize
//*
//*****************************************************************************
static inline void RcDecNormalize(PPM_STREAM *pStream)	
{
	
	while((uLow^(uLow+uRange))<TOP || uRange<BOT && ((uRange=-(int)uLow&(BOT-1)),1)) 
		{
		uCode    = (uCode<<8)|PPM_D_GETC(pStream);            
		uRange <<= 8;                        
		uLow   <<= 8;
		}           

}

//*****************************************************************************
//*
//*		RcGetCurrentCount
//*
//*****************************************************************************
static inline PPM_DWORD RcGetCurrentCount() 
{

	uRange /= sRange.uScale;

return (uCode-uLow)/uRange; 
}

//*****************************************************************************
//*
//*		RcRemoveSubrange
//*
//*****************************************************************************
static inline void RcRemoveSubrange()
{

	uLow   += uRange     *sRange.uLow;					
	uRange *= sRange.uHigh-sRange.uLow;

}

//*****************************************************************************
//*
//*		RcBinStart
//*
//*****************************************************************************
static inline PPM_DWORD RcBinStart(PPM_DWORD uSumm,PPM_DWORD uShift) 
{

	uRange >>= uShift; 

return uSumm*uRange; 
}

//*****************************************************************************
//*
//*		RcBinDecode
//*
//*****************************************************************************
static inline PPM_DWORD RcBinDecode (PPM_DWORD uTemp) 
{

return (uCode-uLow >= uTemp); 
}

//*****************************************************************************
//*
//*		RcBinCorrect0
//*
//*****************************************************************************
static inline void RcBinCorrect0(PPM_DWORD uTemp) 
{
	uRange = uTemp; 
}

//*****************************************************************************
//*
//*		RcBinCorrect1
//*
//*****************************************************************************
static inline void RcBinCorrect1(PPM_DWORD uTemp,PPM_DWORD uScale) 
{
	uLow   += uTemp;   
	uRange *= uScale; 
}


//*****************************************************************************
//*
//*		PrefetchData
//*
//*****************************************************************************
static inline void PrefetchData(void *pAddr)
{
	#ifdef	PPM_USE_PREFETCHING
	PPM_BYTE ucPrefetchByte = *(volatile PPM_BYTE*)pAddr;
	#endif
}

//*****************************************************************************
//*
//*		Insert
//*
//*****************************************************************************
inline void BLK_NODE::Insert(void *pPtr,int iNu) 
{
MEM_BLK	   *pMem=(MEM_BLK*)pPtr;				


	Link(pMem);

	pMem->uStamp = 0xFFFFFFFF;							
	pMem->uNu	 = iNu;
	uStamp	    += 1;

}

//*****************************************************************************
//*
//*		U2B
//*
//*****************************************************************************
static inline PPM_DWORD U2B(PPM_DWORD uNu) 
{
return	uNu*8+uNu*sizeof(void*); 
}

//*****************************************************************************
//*
//*		SplitBlock
//*
//*****************************************************************************
static inline void SplitBlock(BLK_NODE *pNode,PPM_DWORD uOldIndex,PPM_DWORD uNewIndex)
{
PPM_BYTE	   *pData;
PPM_DWORD		uDiff;
PPM_DWORD		uPos;
PPM_DWORD		uIdx;


	uDiff	= aIndexToUnits[uOldIndex]-aIndexToUnits[uNewIndex];
	pData	= ((PPM_BYTE*)pNode)+U2B(aIndexToUnits[uNewIndex]);

	if(aIndexToUnits[uIdx=aUnitsToIndex[uDiff-1]]!=uDiff) 
		{
		uPos    = aIndexToUnits[--uIdx];				
		aBlkList[uIdx].Insert(pData,uPos);
		pData  += U2B(uPos);						
		uDiff  -= uPos;
		}

	aBlkList[aUnitsToIndex[uDiff-1]].Insert(pData,uDiff);


}

//*****************************************************************************
//*
//*		GetUsedMemory
//*
//*****************************************************************************
static PPM_DWORD PPM_STDCALL GetUsedMemory()
{
PPM_DWORD		uIdx;
PPM_DWORD		uRet;


	uRet = uSubAllocatorSize-(pHiUnit-pLoUnit)-(pUnitsStart-pText);

	for(uIdx=0;uIdx<N_INDEXES;uIdx++)
		{
		uRet -= UNIT_SIZE*aIndexToUnits[uIdx]*aBlkList[uIdx].uStamp;
		}


return uRet;
}

//*****************************************************************************
//*
//*		StopSubAllocator
//*
//*****************************************************************************
static void PPM_STDCALL StopSubAllocator() 
{

	if(uSubAllocatorSize) 
		{
		uSubAllocatorSize=0;					
		delete[] pHeapStart;
		}

}

//*****************************************************************************
//*
//*		StartSubAllocator
//*
//*****************************************************************************
static PPM_BOOL PPM_STDCALL StartSubAllocator(PPM_DWORD uSize)
{

	if(uSubAllocatorSize==uSize)				
		{
		return TRUE;
		}

	StopSubAllocator();

	pHeapStart = new PPM_BYTE[uSize];

	if(pHeapStart==NULL)	
		{
		return FALSE;
		}

	uSubAllocatorSize = uSize;						

	
return TRUE;
}

//*****************************************************************************
//*
//*		InitSubAllocator
//*
//*****************************************************************************
static inline void InitSubAllocator()
{
PPM_DWORD	uDiff;

	
	memset(aBlkList,0,sizeof(aBlkList));

	pText		= pHeapStart;
	pHiUnit		= pHeapStart+uSubAllocatorSize;
	uDiff		= UNIT_SIZE*(uSubAllocatorSize/8/UNIT_SIZE*7);
	pUnitsStart	= pHiUnit-uDiff;			
	pLoUnit		= pUnitsStart;
	uGlueCount	= 0;

}

//*****************************************************************************
//*
//*		GlueFreeBlocks
//*
//*****************************************************************************
static void GlueFreeBlocks()
{
MEM_BLK		sBlock;
MEM_BLK	   *pPtr0;
MEM_BLK	   *pPtr1;
MEM_BLK	   *pMem;
PPM_DWORD	uSize;
PPM_DWORD	uPos;
PPM_DWORD	uNum;




	if(pLoUnit!=pHiUnit)
		{
		*pLoUnit=0;
		}

	for(uPos=0,(pPtr0=&sBlock)->pNext=NULL;uPos<N_INDEXES;uPos++)
		{
		while(aBlkList[uPos].Avail()) 
			{
				pMem=(MEM_BLK*) aBlkList[uPos].Remove();
			if(!pMem->uNu)
				{
				continue;
				}
			
			while((pPtr1=pMem+pMem->uNu)->uStamp==0xFFFFFFFF) 
				{
				pMem->uNu += pPtr1->uNu;		
				pPtr1->uNu=0;
				}

			pPtr0->Link(pMem);				
			pPtr0 = pMem;
			}
		}

	while(sBlock.Avail()) 
		{
		pMem	= (MEM_BLK*) sBlock.Remove();			
		uSize	= pMem->uNu;

		if(!uSize)continue;

		for(;uSize>128;uSize-=128,pMem+=128)
			{
			aBlkList[N_INDEXES-1].Insert(pMem,128);
			}

		if(aIndexToUnits[uPos=aUnitsToIndex[uSize-1]] != uSize) 
			{
			uNum	= uSize-aIndexToUnits[--uPos];			
			aBlkList[uNum-1].Insert(pMem+(uSize-uNum),uNum);
			}

		aBlkList[uPos].Insert(pMem,aIndexToUnits[uPos]);
		}

	uGlueCount = 1<<13;

}

//*****************************************************************************
//*
//*		AllocUnitsRare
//*
//*****************************************************************************
static BLK_NODE *PPM_STDCALL AllocUnitsRare(PPM_DWORD uIndex)
{
BLK_NODE   *pRetVal;
PPM_DWORD	uDiff;
PPM_DWORD	uPos;


	uPos = uIndex;

	if(!uGlueCount) 
		{
		GlueFreeBlocks();
		if(aBlkList[uPos].Avail())				
		return aBlkList[uPos].Remove();
		}

	do	{ 
		uPos++;

		if(uPos==N_INDEXES) 
			{
			uGlueCount--;					
			uPos	= U2B(aIndexToUnits[uIndex]);
			uDiff	= pUnitsStart-pText;

			if(uDiff<=uPos)
				{
				return NULL;
				}
			
			pUnitsStart -= uPos;

			return (BLK_NODE*)pUnitsStart;
			}
		
		}while(!aBlkList[uPos].Avail());
	
	
	pRetVal = aBlkList[uPos].Remove();			
	SplitBlock(pRetVal,uPos,uIndex);



return pRetVal;
}

//*****************************************************************************
//*
//*		AllocUnits
//*
//*****************************************************************************
inline BLK_NODE *AllocUnits(PPM_DWORD uCount)
{
BLK_NODE   *pRetVal;
PPM_DWORD	uIdx;


	uIdx = aUnitsToIndex[uCount-1];

	if(aBlkList[uIdx].Avail())				
		{
		return aBlkList[uIdx].Remove();
		}

	pRetVal  = (BLK_NODE*)pLoUnit;					
	pLoUnit += U2B(aIndexToUnits[uIdx]);
	
	if(pLoUnit<=pHiUnit)					
		{
		return pRetVal;
		}

	pLoUnit -= U2B(aIndexToUnits[uIdx]);		
	

return AllocUnitsRare(uIdx);
}

//*****************************************************************************
//*
//*		AllocContext
//*
//*****************************************************************************
inline void *AllocContext()
{

	if(pHiUnit!=pLoUnit)					
		{
		pHiUnit -= UNIT_SIZE;
		return pHiUnit;
		}
	
	if(aBlkList->Avail())				
		{
		return aBlkList->Remove();
		}
	

return AllocUnitsRare(0);
}

//*****************************************************************************
//*
//*		UnitsCpy
//*
//*****************************************************************************
inline void UnitsCpy(BLK_NODE *pTarget,BLK_NODE *pSource,PPM_DWORD uCount)
{
PPM_DWORD	   *pPtr1;
PPM_DWORD	   *pPtr2;


	pPtr1 = (PPM_DWORD*)pTarget;
	pPtr2 = (PPM_DWORD*)pSource;

	for(;uCount>0;uCount--)
		{
		pPtr1[0]	= pPtr2[0];						
		pPtr1[1]	= pPtr2[1];
		pPtr1[2]	= pPtr2[2];

		if(sizeof(void*)==8)
			{
			pPtr1[3]= pPtr2[3];
			pPtr1  += 4;							
			pPtr2  += 4;
			}
		else{
			pPtr1  += 3;							
			pPtr2  += 3;
			}
		}

}

//*****************************************************************************
//*
//*		ExpandUnits
//*
//*****************************************************************************
inline void *ExpandUnits(PPM_STATE *pOldPtr,PPM_DWORD uOldCnt)
{
PPM_DWORD	uPos0;
PPM_DWORD	uPos1;
BLK_NODE   *pPtr;


	uPos0 = aUnitsToIndex[uOldCnt-1  ];
	uPos1 = aUnitsToIndex[uOldCnt-1+1];


	if(uPos0==uPos1)							
		{
		return pOldPtr;
		}

	pPtr = AllocUnits(uOldCnt+1);
	
	if(pPtr) 
		{
		UnitsCpy(pPtr,(BLK_NODE*)pOldPtr,uOldCnt);			
		aBlkList[uPos0].Insert(pOldPtr,uOldCnt);
		}


return pPtr;
}

//*****************************************************************************
//*
//*		ShrinkUnits
//*
//*****************************************************************************
inline BLK_NODE *ShrinkUnits(BLK_NODE *pOldPtr,PPM_DWORD uOldCnt,PPM_DWORD uNewCnt)
{
PPM_DWORD	uPos0;
PPM_DWORD	uPos1;
BLK_NODE   *pPtr;


	uPos0 = aUnitsToIndex[uOldCnt-1];
	uPos1 = aUnitsToIndex[uNewCnt-1];


	if(uPos0==uPos1)							
		{
		return pOldPtr;
		}

	if(aBlkList[uPos1].Avail()) 
		{
		pPtr = aBlkList[uPos1].Remove();		
		UnitsCpy(pPtr,(BLK_NODE*)pOldPtr,uNewCnt);
		aBlkList[uPos0].Insert(pOldPtr,aIndexToUnits[uPos0]);
		return pPtr;
		}

	SplitBlock(pOldPtr,uPos0,uPos1);			


return pOldPtr;
}

//*****************************************************************************
//*
//*		FreeUnits
//*
//*****************************************************************************
inline void FreeUnits(void *pPtr,PPM_DWORD uNu) 
{
PPM_DWORD		uIdx;


	uIdx = aUnitsToIndex[uNu-1];
	aBlkList[uIdx].Insert(pPtr,aIndexToUnits[uIdx]);

}

//*****************************************************************************
//*
//*		SpecialFreeUnit
//*
//*****************************************************************************
inline void SpecialFreeUnit(void *pPtr)
{
	
	if((PPM_BYTE*)pPtr!=pUnitsStart)			
		{
		aBlkList->Insert(pPtr,1);
		}
	else{ 
		*(PPM_DWORD*) pPtr = 0xFFFFFFFF;				
		pUnitsStart += UNIT_SIZE; 
		}
}

//*****************************************************************************
//*
//*		MoveUnitsUp
//*
//*****************************************************************************
inline void *MoveUnitsUp(PPM_STATE *pOldPtr,PPM_DWORD uCount)
{
BLK_NODE   *pPtr;
PPM_DWORD	uIdx;


	uIdx = aUnitsToIndex[uCount-1];

	if((PPM_BYTE*)pOldPtr>pUnitsStart+16*1024 || (BLK_NODE*)pOldPtr>aBlkList[uIdx].pNext)
		{
		return pOldPtr;
		}

	pPtr = aBlkList[uIdx].Remove();
	UnitsCpy(pPtr,(BLK_NODE*)pOldPtr,uCount);				
	uCount  = aIndexToUnits[uIdx];
	
	if((PPM_BYTE*)pOldPtr!=pUnitsStart)		
			aBlkList[uIdx].Insert(pOldPtr,uCount);
	else	pUnitsStart += U2B(uCount);


return pPtr;
}

//*****************************************************************************
//*
//*		ExpandTextArea
//*
//*****************************************************************************
static inline void ExpandTextArea()
{
PPM_DWORD	aCount[N_INDEXES];					
PPM_DWORD	uIdx;
MEM_BLK	   *pMem;
BLK_NODE   *pNode;



	memset(aCount,0,sizeof(aCount));


	while((pNode=(BLK_NODE*)pUnitsStart)->uStamp == 0xFFFFFFFF) 
		{
		pMem = (MEM_BLK*)pNode;			

		pUnitsStart  = (PPM_BYTE*)(pMem+pMem->uNu);
		aCount[aUnitsToIndex[pMem->uNu-1]]++;		
		pMem->uStamp = 0;
		}

	for(uIdx=0;uIdx<N_INDEXES;uIdx++)
		{
		for(pNode=aBlkList+uIdx;aCount[uIdx]!=0;pNode=pNode->pNext)
			{
			while(!pNode->pNext->uStamp) 
				{
				pNode->Unlink();				
				aBlkList[uIdx].uStamp--;
				aCount[uIdx]--;
				if(!aCount[uIdx])break;
				}
			}
		}
}

//*****************************************************************************
//*
//*		Swap
//*
//*****************************************************************************
inline void Swap(PPM_STATE &s1,PPM_STATE &s2)
{
PPM_WORD	 	t1=(PPM_WORD&) s1;						
PPM_CONTEXT*	t2=s1.pSuccessor;


	(PPM_WORD&) s1 = (PPM_WORD&) s2;				
	s1.pSuccessor=s2.pSuccessor;
	(PPM_WORD&) s2 = t1;						
	s2.pSuccessor=t2;
}

//*****************************************************************************
//*
//*		StateCpy
//*
//*****************************************************************************
inline void StateCpy(PPM_STATE &s1,const PPM_STATE &s2)
{
	(PPM_WORD&) s1=(PPM_WORD&) s2;					
	s1.pSuccessor=s2.pSuccessor;
}

//*****************************************************************************
//*
//*		PPMD_STARTUP
//*
//*****************************************************************************
static void PpmdStartUp() 		
{
static	int		iDone=0;		
PPM_DWORD		uStep;
PPM_DWORD		i,k,m;


	if(iDone)return;

	iDone = 1;

	for(i=0,k=1;i<N1;i++,k+=1)	
		{
		aIndexToUnits[i]=k;
		}

	for(k++;i<N1+N2;i++,k+=2)	
		{
		aIndexToUnits[i]=k;
		}

	for(k++;i<N1+N2+N3;i++,k+=3)	
		{
		aIndexToUnits[i]=k;
		}

	for(k++;i<N1+N2+N3+N4;i++,k+=4)	
		{
		aIndexToUnits[i]=k;
		}

	for(k=i=0;k<128;k++) 
		{
		i += (aIndexToUnits[i] < k+1);			
		aUnitsToIndex[k]=i;
		}

	aNsToBsIndex[0] = 2*0;						
	aNsToBsIndex[1] = 2*1;
	
	memset(aNsToBsIndex+2,2*2,9);				
	memset(aNsToBsIndex+11,2*3,256-11);
	
	for(i=0;i<UP_FREQ;i++)				
		{
		aQTable[i]=i;
		}

	for(m=i=UP_FREQ,k=uStep=1;i<260;i++) 
		{
		aQTable[i] = m;
		
		if(!--k) 
			{
			uStep++;
			k = uStep;			
			m++; 
			}
		}

	(PPM_DWORD&)aDummySeeToCont = uPpmSignature;
}

//*****************************************************************************
//*
//*		StartModelRare
//*
//*****************************************************************************
static void PPM_STDCALL StartModelRare(int iOrder,MR_METHOD uMethod)
{
PPM_CONTEXT	   *pPtr;
PPM_DWORD		i,k,m;


	memset(aCharMask,0,sizeof(aCharMask));	
	
	ucEscCount		= 1;

	if(iOrder<2)											// we are in solid mode
		{			    
		iOrderFall = iMaxOrder;

		for(pPtr=pMaxContext;pPtr->pSuffix!=NULL;pPtr=pPtr->pSuffix)
			{
			iOrderFall--;
			}

		return;
		}

	iMaxOrder	= iOrder;
	iOrderFall	= iOrder;			
	uMrMethod	= uMethod;
	
	InitSubAllocator();

	iInitRl		= -((iOrder<12)? iOrder:12)-1;
	iRunLength	= iInitRl;
	
	for(i=m=0;m<25;m++) 
		{
		while(aQTable[i] == m)
			{
			i++;
			}

		for(k=0;k<8;k++)
			{
			aBinSumm[m][k]=BIN_SCALE-aInitBinEsc[k]/(i+1);
			}

		for(k=8;k<64;k+=8)
			{
			memcpy(aBinSumm[m]+k,aBinSumm[m],8*sizeof(PPM_WORD));
			}
		}

	for(i=m=0;m<24;m++) 
		{
		while(aQTable[i+3] == m+3)			
			{
			i++;
			}

		aSeeToCont[m][0].Init(2*i+5);

		for(k=1;k<32;k++)				
			{
			aSeeToCont[m][k]=aSeeToCont[m][0];
			}
		}

	pMaxContext				= (PPM_CONTEXT*)AllocContext();
	pMaxContext->pSuffix	=  NULL;

	
	pMaxContext->usSummFreq	= (pMaxContext->ucNumStats=255)+2;
	pMaxContext->pStats		= (PPM_STATE*)AllocUnits(256/2);

	for(ucPrevSuccess=i=0;i<256;i++) 
		{
		pMaxContext->pStats[i].ucSymbol 	= i;	
		pMaxContext->pStats[i].ucFreq		= 1;
		pMaxContext->pStats[i].pSuccessor	= NULL;
		}
}

//*****************************************************************************
//*
//*		MakeSuffix
//*
//*****************************************************************************
void PPM_CONTEXT::MakeSuffix()
{
PPM_STATE	   *p1;
PPM_STATE	   *p;


	if(!ucNumStats) 
		{
		if(!(p=&OneState())->pSuccessor)	
			{
			return;
			}

		if(!pSuffix->ucNumStats)			
			{
			p1=&(pSuffix->OneState());
			}
		else{
			for(p1=pSuffix->pStats;p1->ucSymbol!=p->ucSymbol;p1++);
			}

		p->pSuccessor->pSuffix = p1->pSuccessor; 
		p->pSuccessor->MakeSuffix();
		}
	else{ 
		for(p=pStats;p<=pStats+ucNumStats;p++) 
			{
			if(!p->pSuccessor)
				{
				continue;
				}

			if(!pSuffix)
				{
				p->pSuccessor->pSuffix=this;
				}
			else{ 
				for(p1=pSuffix->pStats;p1->ucSymbol!=p->ucSymbol;p1++);
				p->pSuccessor->pSuffix=p1->pSuccessor;
				}

			p->pSuccessor->MakeSuffix();
			}
		}
}


//*****************************************************************************
//*
//*		Refresh
//*
//*****************************************************************************
void PPM_CONTEXT::Refresh(int uOldNu,PPM_BOOL bScale)
{
PPM_STATE	   *pState;
int				iEscFreq;
int				i;


	i			= ucNumStats;
	pStats		= (PPM_STATE*)ShrinkUnits((BLK_NODE*)pStats,uOldNu,(i+2)>>1);
	pState		= pStats;

	ucFlags		= (ucFlags & (0x10+0x04*bScale))+0x08*(pState->ucSymbol >= 0x40);
	iEscFreq 	=  usSummFreq-pState->ucFreq;
	usSummFreq	= (pState->ucFreq=(pState->ucFreq+bScale) >> bScale);

	do	{ 
		pState         += 1;
		iEscFreq       -= (pState->ucFreq);
		pState->ucFreq	= (pState->ucFreq+bScale)>>bScale;
		usSummFreq     += (pState->ucFreq);
		ucFlags	       |= (pState->ucSymbol >= 0x40)*0x08;
		}while(--i);

	iEscFreq	= (iEscFreq+bScale)>>bScale;
	usSummFreq +=  iEscFreq;

}


//*****************************************************************************
//*
//*		CutOff
//*
//*****************************************************************************
PPM_CONTEXT *PPM_CONTEXT::CutOff(int iOrder)
{
PPM_STATE  *pState;
int			iTemp;
int			iPos;


	if(!ucNumStats) 
		{
		pState = &OneState();

		if((PPM_BYTE*)pState->pSuccessor >= pUnitsStart) 
			{
			if(iOrder<iMaxOrder)			
				{
				P_CALL(pState,CutOff);
				}
			else{
				pState->pSuccessor=NULL;
				}

			if(!pState->pSuccessor && iOrder>O_BOUND)
				{
				goto REMOVE;
				}

			return this;
			}
		else{ 
REMOVE:		SpecialFreeUnit(this);			
			return NULL;
			}
		}

	PrefetchData(pStats);
	pStats	= (PPM_STATE*)MoveUnitsUp(pStats,iTemp=(ucNumStats+2) >> 1);
	iPos	= ucNumStats;

	for(pState=pStats+iPos;pState>=pStats;pState--)
		{
		if((PPM_BYTE*)pState->pSuccessor<pUnitsStart) 
			{
			pState->pSuccessor = NULL;			
			Swap(*pState,pStats[iPos--]);
			}
		else if(iOrder < iMaxOrder)	
			{
			P_CALL(pState,CutOff);
			}
		else{
			pState->pSuccessor = NULL;
			}
		}

	if(iPos!=ucNumStats && iOrder) 
		{
		ucNumStats	= iPos;							
		pState		= pStats;

		if(iPos<0) 
			{
			FreeUnits(pState,iTemp);		
			goto REMOVE; 
			}
		else if(iPos==0) 
			{
			ucFlags=(ucFlags&0x10)+0x08*(pState->ucSymbol >= 0x40);
			StateCpy(OneState(),*pState);		
			FreeUnits(pState,iTemp);
			OneState().ucFreq = (OneState().ucFreq+11)>>3;
			}
		else{								
			Refresh(iTemp,usSummFreq > 16*iPos);
			}
		}

return this;
}

//*****************************************************************************
//*
//*		RemoveBinConts
//*
//*****************************************************************************
PPM_CONTEXT *PPM_CONTEXT::RemoveBinConts(int iOrder)
{
PPM_STATE	   *pState;


	if(!ucNumStats) 
		{
		pState=&OneState();

		if((PPM_BYTE*)pState->pSuccessor>=pUnitsStart && iOrder<iMaxOrder)
			{
			P_CALL(pState,RemoveBinConts);
			}
		else{
			pState->pSuccessor=NULL;
			}

		if(!pState->pSuccessor && (pSuffix->ucNumStats==0 || pSuffix->ucFlags==0xFF)) 
			{
			FreeUnits(this,1);				
			return NULL;
			}
		else{								
			return this; 
			}
		}

	PrefetchData(pStats);

	for(pState=pStats+ucNumStats;pState>=pStats;pState--)
		{
		if((PPM_BYTE*)pState->pSuccessor >= pUnitsStart && iOrder<iMaxOrder)
			{
			P_CALL(pState,RemoveBinConts);
			}
		else{
			pState->pSuccessor=NULL;
			}
		}


return this;
}

//*****************************************************************************
//*
//*		RestoreModelRare
//*
//*****************************************************************************
static void RestoreModelRare(PPM_CONTEXT *pc1,PPM_CONTEXT *MinContext,PPM_CONTEXT *pSuccessorF)
{
PPM_CONTEXT	 *pContext;
PPM_STATE	 *pState;


	for(pContext=pMaxContext,pText=pHeapStart;pContext!=pc1;pContext=pContext->pSuffix)
		{
		if(--(pContext->ucNumStats) == 0) 
			{
			pContext->ucFlags	= (pContext->ucFlags & 0x10)+0x08*(pContext->pStats->ucSymbol >= 0x40);
			pState				=  pContext->pStats;				
			
			StateCpy(pContext->OneState(),*pState);
			SpecialFreeUnit(pState);
			pContext->OneState().ucFreq=(pContext->OneState().ucFreq+11) >> 3;
			}
		else{
			pContext->Refresh((pContext->ucNumStats+3) >> 1,FALSE);
			}
		}

	for(;pContext!=MinContext;pContext=pContext->pSuffix)
		{
		if(!pContext->ucNumStats)
			{
			pContext->OneState().ucFreq -= pContext->OneState().ucFreq >> 1;
			}
		else if((pContext->usSummFreq += 4) > 128+4*pContext->ucNumStats)
			{
			pContext->Refresh((pContext->ucNumStats+2) >> 1,TRUE);
			}
		}

	if(uMrMethod > MRM_FREEZE) 
		{
		pMaxContext	= pSuccessorF;				
		uGlueCount += !(aBlkList[1].uStamp&1);
		} 
	else if(uMrMethod == MRM_FREEZE)
		{
		while(pMaxContext->pSuffix)		
			{
			pMaxContext = pMaxContext->pSuffix;
			}

		pMaxContext->RemoveBinConts(0);		
		uMrMethod	= MR_METHOD(uMrMethod+1);
		iOrderFall	= iMaxOrder;
		uGlueCount	= 0;						
		} 
	else if(uMrMethod==MRM_RESTART || GetUsedMemory()<(uSubAllocatorSize>>1))
		{
		StartModelRare(iMaxOrder,uMrMethod);
		ucEscCount		= 0;							
		}
	else{ 
		while(pMaxContext->pSuffix)
			{
			pMaxContext = pMaxContext->pSuffix;
			}

		do	{ 
			pMaxContext->CutOff(0);			
			ExpandTextArea();
			}while(GetUsedMemory()>3*(uSubAllocatorSize >> 2));
		
		iOrderFall	= iMaxOrder;
		uGlueCount	= 0;						
		}

}

//*****************************************************************************
//*
//*		CreateSuccessors
//*
//*****************************************************************************
static PPM_CONTEXT *PPM_FASTCALL CreateSuccessors(PPM_BOOL bSkip,PPM_STATE *pState,PPM_CONTEXT *pContext)
{
PPM_CONTEXT			sData; 
PPM_CONTEXT		   *pUpBranch;
PPM_STATE		   *aList[MAX_O];
PPM_STATE		  **pNext;
PPM_BYTE			uTemp;
PPM_BYTE			uSym;
PPM_DWORD			uVal;
PPM_DWORD			uNum;


	uSym		= pFoundState->ucSymbol;
	pUpBranch	= pFoundState->pSuccessor;
	pNext		= aList;


	if(!bSkip) 
		{
		*pNext++ = pFoundState;
		if(!pContext->pSuffix)goto NO_LOOP;
		}

	if(pState) 
		{
		pContext = pContext->pSuffix;				
		goto LOOP_ENTRY; 
		}

	do	{ 
		pContext = pContext->pSuffix;

		if(pContext->ucNumStats) 
			{
			pState = pContext->pStats;

			if(pState->ucSymbol!=uSym)
			do	{ 
				uTemp = pState[1].ucSymbol;	
				pState++; 
				}while(uTemp!=uSym);

			uTemp 		          = (pState->ucFreq < MAX_FREQ-9);
			pState->ucFreq       +=  uTemp;					
			pContext->usSummFreq +=  uTemp;
			}
		else{ 
			pState		    = &(pContext->OneState());
			pState->ucFreq +=  !pContext->pSuffix->ucNumStats & (pState->ucFreq < 24);
			}

LOOP_ENTRY:

		if(pState->pSuccessor != pUpBranch) 
			{
			pContext=pState->pSuccessor;				
			break;
			}

		*pNext++ = pState;
		}while(pContext->pSuffix);

NO_LOOP:

	if(pNext==aList)return pContext;
	
	sData.ucNumStats = 0;							
	sData.ucFlags	 = 0x10*(uSym>=0x40);
	sData.OneState().ucSymbol   = uSym        = *(PPM_BYTE*)pUpBranch;
	sData.OneState().pSuccessor =(PPM_CONTEXT*)(((PPM_BYTE*)pUpBranch)+1);
	sData.ucFlags   |= 0x08*(uSym>=0x40);
	
	if(pContext->ucNumStats) 
		{
		   pState = pContext->pStats;
		if(pState->ucSymbol!=uSym)
		do	{ 
			uTemp=pState[1].ucSymbol;		
			pState++; 
			}while(uTemp != uSym);

		uNum=pContext->usSummFreq-pContext->ucNumStats-(uVal=pState->ucFreq-1);
		sData.OneState().ucFreq=1+((2*uVal <= uNum)?(5*uVal > uNum):((uVal+2*uNum-3)/uNum));
		}
	else{
		sData.OneState().ucFreq=pContext->OneState().ucFreq;
		}

	do	{ 
		PPM_CONTEXT *pPtr;
		
			pPtr = (PPM_CONTEXT*)AllocContext();
		if(!pPtr)
			{
			return NULL;
			}

		*((PPM_DWORD*)pPtr)	  = *((PPM_DWORD*)&sData);

		pNext			     -= 1;
		pPtr->pStats		  = sData.pStats;
		pPtr->pSuffix		  = pContext;						
		pContext			  = pPtr;
		pNext[0]->pSuccessor  = pPtr;
		
		}while(pNext!=aList);

return pContext;
}

//*****************************************************************************
//*
//*		ReduceOrder
//*
//*****************************************************************************
static PPM_CONTEXT *PPM_FASTCALL ReduceOrder(PPM_STATE *pPtr,PPM_CONTEXT *pContInp)
{
PPM_STATE	   **pNext;
PPM_STATE		*pState;
PPM_STATE		*aList[MAX_O];
PPM_CONTEXT		*pUpBranch;
PPM_CONTEXT		*pContext;
PPM_BYTE		 uTemp;
PPM_BYTE		 uSym;

	
	pUpBranch	= (PPM_CONTEXT*) pText;
	pNext		= aList;
	pContext	= pContInp;
	uSym		= pFoundState->ucSymbol;
	*pNext++	= pFoundState;					
	
	pFoundState->pSuccessor	= pUpBranch;
	iOrderFall++;
	
	if(pPtr) 
		{
		pContInp = pContInp->pSuffix;				
		goto LOOP_ENTRY; 
		}

	for(;;) 
		{
		if(!pContInp->pSuffix) 
			{
			if(uMrMethod > MRM_FREEZE) 
				{
FROZEN:			do	{ 
					(*--pNext)->pSuccessor = pContInp; 
					}while(pNext != aList);

				pText		= pHeapStart+1;			
				iOrderFall	= 1;
				}

			return pContInp;
			}

		   pContInp=pContInp->pSuffix;
		if(pContInp->ucNumStats) 
			{
			if((pPtr=pContInp->pStats)->ucSymbol != uSym)
			do	{ 
				uTemp=pPtr[1].ucSymbol;	
				pPtr++; 
				}while(uTemp != uSym);

			uTemp 				  = 2*(pPtr->ucFreq < MAX_FREQ-9);
			pPtr->ucFreq         += uTemp;					
			pContInp->usSummFreq += uTemp;
			}
		else{ 
			pPtr          = &(pContInp->OneState());		
			pPtr->ucFreq +=  (pPtr->ucFreq < 32); 
			}

LOOP_ENTRY:

		if(pPtr->pSuccessor)break;
		*pNext++ = pPtr;							
		pPtr->pSuccessor = pUpBranch;
		iOrderFall++;
		}

	if(uMrMethod > MRM_FREEZE) 
		{
		pContInp = pPtr->pSuccessor;					
		goto FROZEN;
		} 
	else if(pPtr->pSuccessor <= pUpBranch)
		{
		pState			 = pFoundState;						
		pFoundState		 = pPtr;
		pPtr->pSuccessor = CreateSuccessors(FALSE,NULL,pContInp);
		pFoundState		 = pState;
		}

	if(iOrderFall==1  && pContext==pMaxContext) 
		{
		pFoundState->pSuccessor = pPtr->pSuccessor; 
		pText--;
		}


return pPtr->pSuccessor;
}

//*****************************************************************************
//*
//*		Rescale
//*
//*****************************************************************************
void PPM_CONTEXT::Rescale()
{
PPM_STATE		sTemp;
PPM_STATE	   *pPtr;
PPM_STATE	   *pState;
PPM_DWORD		uEscFreq;
PPM_DWORD		uOldNu;
PPM_DWORD		uAdder;
PPM_DWORD		uPos;


	uPos = ucNumStats;

	for(pPtr=pFoundState;pPtr!=pStats;pPtr--)		
		{
		Swap(pPtr[0],pPtr[-1]);
		}

	pPtr->ucFreq    +=  4;							
	usSummFreq      +=  4;
	uEscFreq 		 =  usSummFreq-pPtr->ucFreq;
	uAdder			 = (iOrderFall!=0 || uMrMethod>MRM_FREEZE);
	pPtr->ucFreq	 = (pPtr->ucFreq+uAdder) >> 1;
	usSummFreq		 =  pPtr->ucFreq;
	
	do	{ 
		pPtr        += 1;
		uEscFreq    -=  pPtr->ucFreq;
		pPtr->ucFreq = (pPtr->ucFreq+uAdder)>>1;
		usSummFreq  +=  pPtr->ucFreq;
		
		if(pPtr[0].ucFreq>pPtr[-1].ucFreq) 
			{
			StateCpy(sTemp,*(pState=pPtr));

			do	{
				StateCpy(pState[0],pState[-1]); 
				}while(sTemp.ucFreq > (--pState)[-1].ucFreq);
			
			StateCpy(*pState,sTemp);
			}
		
		}while(--uPos);
	
	if(pPtr->ucFreq == 0) 
		{
		do	{ 
			uPos++; 
			}while((--pPtr)->ucFreq == 0);
		
		uEscFreq += uPos;						
		uOldNu	  =(ucNumStats+2) >> 1;

		if((ucNumStats -= uPos) == 0) 
			{
			StateCpy(sTemp,*pStats);
			sTemp.ucFreq=(2*sTemp.ucFreq+uEscFreq-1)/uEscFreq;
			
			if(sTemp.ucFreq > MAX_FREQ/3)
				{
				sTemp.ucFreq = MAX_FREQ/3;
				}

			FreeUnits(pStats,uOldNu);			
			StateCpy(OneState(),sTemp);

			ucFlags		= (ucFlags & 0x10)+0x08*(sTemp.ucSymbol >= 0x40);
			pFoundState	=  &OneState();			
			
			return;
			}

		pStats	  = (PPM_STATE*)ShrinkUnits((BLK_NODE*)pStats,uOldNu,(ucNumStats+2)>>1);
		ucFlags  &= ~0x08;						
		uPos  	  =  ucNumStats;
		pPtr      =  pStats;
		ucFlags  |=  0x08*(pPtr->ucSymbol>=0x40);
		
		do	{ 
			ucFlags |= 0x08*((++pPtr)->ucSymbol >= 0x40); 
			}while(--uPos);
		}

	usSummFreq   += (uEscFreq -= (uEscFreq >> 1));
	ucFlags	     |=  0x04;							
	pFoundState   =  pStats;

}

//*****************************************************************************
//*
//*		UpdateModel
//*
//*****************************************************************************
static inline void UpdateModel(PPM_CONTEXT *pMinContext)
{
PPM_STATE		   *pState;
PPM_CONTEXT	       *pPtr;
PPM_CONTEXT	       *pContext;
PPM_CONTEXT	       *pSuccessor;
PPM_CONTEXT	       *pSuccessorF;
PPM_BYTE			ucSymbolF;
PPM_BYTE			ucFlag;
PPM_BYTE			ucSym;
PPM_DWORD			uFreq;
PPM_DWORD			uNs1;
PPM_DWORD			uNs;
PPM_DWORD			uCf;
PPM_DWORD			uSf;
PPM_DWORD			uS0;


	pState		= NULL;
	pPtr		= pMaxContext;
	uFreq		= pFoundState->ucFreq;
	ucSymbolF	= pFoundState->ucSymbol;
	pSuccessorF	= pFoundState->pSuccessor;		
	pContext	= pMinContext->pSuffix;

	if(uFreq<MAX_FREQ/4 && pContext) 
		{
		if(pContext->ucNumStats) 
			{
			pState = pContext->pStats;

			if(pState->ucSymbol!=ucSymbolF) 
				{
				do	{ 
					ucSym = pState[1].ucSymbol;		
					pState++; 
					}while(ucSym!=ucSymbolF);
				
				if(pState[0].ucFreq >= pState[-1].ucFreq) 
					{
					Swap(pState[0],pState[-1]);		
					pState--;
					}
				}

			uCf					  = 2*(pState->ucFreq < MAX_FREQ-9);
			pState->ucFreq       += uCf;					
			pContext->usSummFreq += uCf;
			}
		else{ 
			pState          = &(pContext->OneState());		
			pState->ucFreq +=  (pState->ucFreq < 32); 
			}
		}

	if(!iOrderFall && pSuccessorF) 
		{
			pFoundState->pSuccessor=CreateSuccessors(TRUE,pState,pMinContext);
		if(!pFoundState->pSuccessor)goto RESTART_MODEL;
		
		pMaxContext = pFoundState->pSuccessor;	
		
		return;
		}

	*pText++   = ucSymbolF;						
	pSuccessor = (PPM_CONTEXT*) pText;
	
	if(pText >= pUnitsStart)
		{
		goto RESTART_MODEL;
		}

	if(pSuccessorF) 
		{
		if((PPM_BYTE*)pSuccessorF < pUnitsStart)
			{
			pSuccessorF = CreateSuccessors(FALSE,pState,pMinContext);
			}
		}
	else{
		pSuccessorF = ReduceOrder(pState,pMinContext);
		}

	if(!pSuccessorF)						
		{
		goto RESTART_MODEL;
		}

	if(!--iOrderFall) 
		{
		pSuccessor  =  pSuccessorF;				
		pText      -= (pMaxContext!=pMinContext);
		} 
	else if(uMrMethod > MRM_FREEZE)
		{
		pSuccessor	= pSuccessorF;				
		pText		= pHeapStart;
		iOrderFall  = 0;
		}

	uNs = pMinContext->ucNumStats;
	uS0 = pMinContext->usSummFreq-uNs-uFreq;
	
	for(ucFlag=0x08*(ucSymbolF>=0x40);pPtr!=pMinContext;pPtr=pPtr->pSuffix) 
		{
		uNs1 = pPtr->ucNumStats;

		if(uNs1!=0) 
			{
			if((uNs1&1) != 0) 
				{
					pState=(PPM_STATE*)ExpandUnits(pPtr->pStats,(uNs1+1)>>1);
				if(!pState)goto RESTART_MODEL;
				
				pPtr->pStats = pState;
				}

			pPtr->usSummFreq += (3*uNs1+1 < uNs);
			}
		else{ 
			pState = (PPM_STATE*)AllocUnits(1);
			
			if(!pState)goto RESTART_MODEL;
			
			StateCpy(*pState,pPtr->OneState());	
			pPtr->pStats=pState;
			
			if(pState->ucFreq < MAX_FREQ/4-1)		
					pState->ucFreq += pState->ucFreq;
			else	pState->ucFreq	= MAX_FREQ-4;

			pPtr->usSummFreq = pState->ucFreq+iInitEsc+(uNs>2);
			}

		uCf = 2*uFreq*(pPtr->usSummFreq+6);		
		uSf = uS0+pPtr->usSummFreq;

		if(uCf < 6*uSf) 
			{
			uCf				   = 1+(uCf > uSf)+(uCf >= 4*uSf);
			pPtr->usSummFreq  += 4;
			}
		else{ 
			uCf				   = 4+(uCf > 9*uSf)+(uCf > 12*uSf)+(uCf > 15*uSf);
			pPtr->usSummFreq  += uCf;
			}

		pState				= pPtr->pStats+(++pPtr->ucNumStats);		
		pState->pSuccessor	= pSuccessor;
		pState->ucSymbol	= ucSymbolF;				
		pState->ucFreq		= uCf;
		pPtr->ucFlags	   |= ucFlag;
		}

	pMaxContext = pSuccessorF;					
	return;

RESTART_MODEL:

	RestoreModelRare(pPtr,pMinContext,pSuccessorF);

}


//*****************************************************************************
//*
//*		EncodeBinSymbol
//*
//*****************************************************************************
inline void PPM_CONTEXT::EncodeBinSymbol(int iSymbol)
{
PPM_BYTE	uIndex;
PPM_DWORD	uTemp;


	uIndex = aNsToBsIndex[pSuffix->ucNumStats]+ucPrevSuccess+ucFlags;

	PPM_STATE  &sState=OneState();
	PPM_WORD   &usSumm=aBinSumm[aQTable[sState.ucFreq-1]][uIndex+((iRunLength >> 26) & 0x20)];
	
	uTemp = RcBinStart(usSumm,TOT_BITS);

	if(sState.ucSymbol==iSymbol) 
		{
		pFoundState		= &sState;						
		sState.ucFreq  += (sState.ucFreq < 196);
		RcBinCorrect0(uTemp);					
		usSumm         += (PPM_WORD)(INTERVAL-GET_MEAN(usSumm,PERIOD_BITS,2));
		ucPrevSuccess	= 1;						
		iRunLength     += 1;
		}
	else{ 
		RcBinCorrect1(uTemp,BIN_SCALE-usSumm);	
		
		usSumm         -= (PPM_WORD)GET_MEAN(usSumm,PERIOD_BITS,2);
		iInitEsc 		= aExpEscape[usSumm >> 10];		
		aCharMask[sState.ucSymbol]=ucEscCount;
		ucNumMasked		= 0;
		ucPrevSuccess	= 0;			
		pFoundState		= 0;
		}
}

//*****************************************************************************
//*
//*		DecodeBinSymbol
//*
//*****************************************************************************
inline void PPM_CONTEXT::DecodeBinSymbol()
{
PPM_BYTE		uIndex;
PPM_DWORD		uTemp;

	
	uIndex = aNsToBsIndex[pSuffix->ucNumStats]+ucPrevSuccess+ucFlags;

	PPM_STATE &sState = OneState();
	PPM_WORD  &usSumm = aBinSumm[aQTable[sState.ucFreq-1]][uIndex+((iRunLength>>26) & 0x20)];
	
	uTemp	= RcBinStart(usSumm,TOT_BITS);
	
	if(!RcBinDecode(uTemp)) 
		{
		pFoundState		= &sState;						
		sState.ucFreq  += (sState.ucFreq < 196);
		
		RcBinCorrect0(uTemp);					

		usSumm		   += (PPM_WORD)(INTERVAL-GET_MEAN(usSumm,PERIOD_BITS,2));
		ucPrevSuccess	= 1;						
		iRunLength	   += 1;
		}
	else{ 
		RcBinCorrect1(uTemp,BIN_SCALE-usSumm);	

		usSumm         -= (PPM_WORD)GET_MEAN(usSumm,PERIOD_BITS,2);
		iInitEsc 		= aExpEscape[usSumm>>10];		
		aCharMask[sState.ucSymbol] = ucEscCount;
		ucNumMasked		= 0;
		ucPrevSuccess	= 0;			
		pFoundState		= 0;
		}
}

//*****************************************************************************
//*
//*		Update1
//*
//*****************************************************************************
inline void PPM_CONTEXT::Update1(PPM_STATE *pState)
{
	
	
	pFoundState		= pState;
	pState->ucFreq += 4;				
	usSummFreq     += 4;

	if(pState[0].ucFreq>pState[-1].ucFreq) 
		{
		Swap(pState[0],pState[-1]);					
		pFoundState = --pState;
		
		if(pState->ucFreq>MAX_FREQ)
			{
			Rescale();
			}
		}
}

//*****************************************************************************
//*
//*		EncodeSymbol1
//*
//*****************************************************************************
inline void PPM_CONTEXT::EncodeSymbol1(int iSymbol)
{
PPM_DWORD		uLoCnt;
PPM_DWORD		uPos;
PPM_STATE	   *pPtr;


	uPos			= pStats->ucSymbol;
	sRange.uScale	= usSummFreq;
	pPtr			= pStats;


	if((int)uPos==iSymbol) 
		{
		sRange.uHigh	= pPtr->ucFreq;
		ucPrevSuccess	= sRange.uHigh*2>=sRange.uScale;
		pFoundState		= pPtr;
		pPtr->ucFreq   += 4;			
		usSummFreq     += 4;
		iRunLength     += ucPrevSuccess;
		
		if(pPtr->ucFreq>MAX_FREQ)
			{
			Rescale();
			}

		sRange.uLow	    = 0;						
		return;
		}

	uLoCnt			= pPtr->ucFreq;
	uPos			= ucNumStats;								
	ucPrevSuccess	= 0;

	while((++pPtr)->ucSymbol != iSymbol) 
		{
		uLoCnt += pPtr->ucFreq;
		
		if(--uPos == 0) 
			{
			if(pSuffix)					
				{
				PrefetchData(pSuffix);
				}

			sRange.uLow	= uLoCnt;				
			aCharMask[pPtr->ucSymbol]=ucEscCount;
			
			uPos		= ucNumStats;			
			ucNumMasked	= ucNumStats;			
			pFoundState	= NULL;

			do	{ 
				aCharMask[(--pPtr)->ucSymbol]=ucEscCount; 
				}while(--uPos);
			
			sRange.uHigh = sRange.uScale;			
			return;
			}
		}

	sRange.uLow		= uLoCnt;
	sRange.uHigh	= uLoCnt+pPtr->ucFreq;	
	
	Update1(pPtr);

}

//*****************************************************************************
//*
//*		DecodeSymbol1
//*
//*****************************************************************************
inline void PPM_CONTEXT::DecodeSymbol1()
{
PPM_DWORD		uHiCnt;
PPM_DWORD		uPos;
PPM_DWORD		uCnt;
PPM_STATE	   *pPtr;							


	uHiCnt			= pStats->ucFreq;
	sRange.uScale	= usSummFreq;
	pPtr			= pStats;							
	uCnt			= RcGetCurrentCount();

	if(uCnt<uHiCnt) 
		{
		pFoundState		= pPtr;
		sRange.uHigh	= uHiCnt;
		ucPrevSuccess	= 2*uHiCnt >= sRange.uScale;
		pPtr->ucFreq	= (uHiCnt += 4);	
		usSummFreq     += 4;
		iRunLength     += ucPrevSuccess;

		if(uHiCnt>MAX_FREQ)				
			{
			Rescale();
			}

		sRange.uLow	   =0;						
		
		return;
		}

	uPos = ucNumStats;								
	ucPrevSuccess = 0;

	while((uHiCnt += (++pPtr)->ucFreq) <= uCnt)
		{
		if(--uPos == 0) 
			{
			if(pSuffix)
				{
				PrefetchData(pSuffix);
				}

			sRange.uLow					= uHiCnt;				
			aCharMask[pPtr->ucSymbol]	= ucEscCount;
			uPos						= ucNumStats;
			ucNumMasked					= ucNumStats;			
			pFoundState					= NULL;

			do	{ 
				aCharMask[(--pPtr)->ucSymbol]=ucEscCount; 
				}while(--uPos);
			
			sRange.uHigh	= sRange.uScale;			
			return;
			}
		}

	sRange.uHigh	= uHiCnt;
	sRange.uLow		= uHiCnt-pPtr->ucFreq;	

	Update1(pPtr);

}

//*****************************************************************************
//*
//*		Update2
//*
//*****************************************************************************
inline void PPM_CONTEXT::Update2(PPM_STATE *pState)
{
	
	pFoundState		= pState;
	pState->ucFreq += 4;				
	usSummFreq	   += 4;
	
	if(pState->ucFreq>MAX_FREQ)					
		{
		Rescale();
		}

	ucEscCount++;								
	iRunLength = iInitRl;
}

//*****************************************************************************
//*
//*		MakeEscFreq2
//*
//*****************************************************************************
SEE2_CONTEXT *PPM_CONTEXT::MakeEscFreq2()
{
SEE2_CONTEXT   *pSee2c;
PPM_BYTE	   *pPtr;					
PPM_DWORD		uVal;
PPM_DWORD		uTemp;


	pPtr	= (PPM_BYTE*)pStats;					
	uTemp	= ucNumStats*2;

	PrefetchData(pPtr);						
	PrefetchData(pPtr+uTemp);
	PrefetchData(pPtr+=2*uTemp);				
	PrefetchData(pPtr+uTemp);
	
	
	if(ucNumStats != 0xFF) 
		{
		uTemp			= pSuffix->ucNumStats;
		pSee2c			= aSeeToCont[aQTable[ucNumStats+2]-3]+(usSummFreq > 11*(ucNumStats+1));
		uVal			= ((PPM_DWORD)(2*ucNumStats) < uTemp+ucNumMasked)? 2:0;
		pSee2c	       += uVal+ucFlags;
		sRange.uScale	= pSee2c->GetMean();
		}
	else{ 
		pSee2c			= &aDummySeeToCont;				
		sRange.uScale	= 1;
		}


return pSee2c;
}

//*****************************************************************************
//*
//*		EncodeSymbol2
//*
//*****************************************************************************
inline void PPM_CONTEXT::EncodeSymbol2(int iSymbol)
{
PPM_STATE	   *pPtr;
PPM_STATE	   *pNext;
SEE2_CONTEXT   *pSee2c;
PPM_DWORD		uLoCnt;
PPM_DWORD		uSym;
PPM_DWORD		uPos;


	uLoCnt	= 0;
	pSee2c	= MakeEscFreq2();
	uPos	= ucNumStats-ucNumMasked;
	pPtr	= pStats-1;


	do	{ 
		do	{ 
			uSym=pPtr[1].ucSymbol;	
			pPtr++; 
			}while(aCharMask[uSym]==ucEscCount);
		
		aCharMask[uSym] = ucEscCount;
		
		if(uSym == (PPM_DWORD)iSymbol)					
			{
			goto SYMBOL_FOUND;
			}

		uLoCnt += pPtr->ucFreq;
		}while(--uPos);
	
	sRange.uLow		=  uLoCnt;
	sRange.uScale  +=  uLoCnt;
	sRange.uHigh	=  sRange.uScale;
	pSee2c->usSumm +=  (PPM_WORD)sRange.uScale;			
	ucNumMasked		=  ucNumStats;

	return;

SYMBOL_FOUND:

	sRange.uLow		= uLoCnt;						
	uLoCnt		   += pPtr->ucFreq;
	sRange.uHigh	= uLoCnt;
	
	for(pNext=pPtr;--uPos;) 
		{
		do	{ 
			uSym = pNext[1].ucSymbol;	
			pNext++; 
			}while(aCharMask[uSym]==ucEscCount);
		
		uLoCnt += pNext->ucFreq;
		}

	sRange.uScale += uLoCnt;
	pSee2c->Update();						
	Update2(pPtr);

}

//*****************************************************************************
//*
//*		DecodeSymbol2
//*
//*****************************************************************************
inline void PPM_CONTEXT::DecodeSymbol2()
{
PPM_STATE	  **pNext;
PPM_STATE	   *pPtr;
PPM_STATE	   *aList[256];
SEE2_CONTEXT   *pSee2c;
PPM_DWORD		uHiCnt;
PPM_DWORD		uSym;
PPM_DWORD		uCnt;
PPM_DWORD		uPos;

	
	uHiCnt	= 0;
	pSee2c	= MakeEscFreq2();
	uPos	= ucNumStats-ucNumMasked;
	pNext	= aList;
	pPtr	= pStats-1;

	do	{ 
		do	{ 
			uSym = pPtr[1].ucSymbol;	
			pPtr++; 
			}while(aCharMask[uSym]==ucEscCount);
		
		uHiCnt  += pPtr->ucFreq;					
		*pNext++ = pPtr;

		}while(--uPos);
	
	sRange.uScale  += uHiCnt;					
	uCnt			= RcGetCurrentCount();
	pPtr			= *(pNext=aList);
	
	if(uCnt<uHiCnt) 
		{
		uHiCnt = 0;
		
		while((uHiCnt+=pPtr->ucFreq) <= uCnt) 
			{
			pPtr = *++pNext;
			}

		sRange.uHigh	= uHiCnt;
		sRange.uLow		= uHiCnt-pPtr->ucFreq;
		pSee2c->Update();					
		Update2(pPtr);
		}
	else{ 
		sRange.uLow		= uHiCnt;					
		sRange.uHigh	= sRange.uScale;
		uPos			= ucNumStats-ucNumMasked;				
		ucNumMasked		= ucNumStats;

		do	{ 
			aCharMask[(*pNext)->ucSymbol] = ucEscCount; 
			pNext++; 
			}while(--uPos);
		
		pSee2c->usSumm += (PPM_WORD)sRange.uScale;
		}

}

//*****************************************************************************
//*
//*		ClearMask
//*
//*****************************************************************************
inline void ClearMask(PPM_STREAM *pEncodedFile,PPM_STREAM *pDecodedFile)
{

	ucEscCount = 1;								
	
	memset(aCharMask,0,sizeof(aCharMask));

}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CPpmClass::CPpmClass()
{
	m_uProbSize	      = 0;
	*(short*)m_aProbs = 0;
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CPpmClass::~CPpmClass()
{


}

//*****************************************************************************
//*
//*		Free
//*
//*****************************************************************************
//	Alle Puffer lschen
void CPpmClass::Free()
{

}

//*****************************************************************************
//*
//*		GetByteEx
//*
//*****************************************************************************
inline int CPpmClass::GetByteEx()
{
unsigned char	ucData;


	if(!GetByte(ucData))return ucData;
	
	m_iErrorCode = CPP_ERROR_READ;
	throw 0;

}

//*****************************************************************************
//*
//*		PutByteEx
//*
//*****************************************************************************
inline void CPpmClass::PutByteEx(unsigned char ucData)
{

	if(!PutByte(ucData))return;

	m_iErrorCode = CPP_ERROR_WRITE;
	throw 0;

}

//*****************************************************************************
//*
//*		SetProbs
//*
//*****************************************************************************
//	Die Propedaten einstellen
//	pData	: Zeiger auf die Daten
//					Bit  0.. 3 : MaxModelOrder - 1		(2..16, 8=default)
//					Bit  4..11 : SubAllocatorSize - 1	(in MB)
//					Bit 12..15 : ModelRestorationMethod	(0=RESTART CUT_OFF=1 FREEZE=2) 
//	uSize	: Gre der Daten
//	Ergibt die kopierten Bytes bzw. 0 bei einem Fehler
int CPpmClass::SetProbs(const void *pData,unsigned uSize)
{

	if(uSize!=2 && uSize!=5)return 0;

	m_uProbSize = uSize;
	memcpy(m_aProbs,pData,uSize);


return uSize;
}


//*****************************************************************************
//*
//*		Compress
//*
//*****************************************************************************
//	Startet die Komprimierung eines Streams
//	iLevel		: Bestimmt den Kompimierungslevel (1..7)
//					-1	= aktuellen Level verwenden
//					 7	= maximale Framegre
//					 0	= minimale Framegre
//	Ergibt die Anzahl der komprimierten Bytes
iocint64 CPpmClass::Compress(int iLevel)
{
PPM_CONTEXT    *pMinContext;
int				iSubAlloc;
int				iOrder;
int				iProps;	
int				iValue;
PPM_BYTE		ucByte;
PPM_BYTE		ucRead;



	IOC_ASSERT(m_uOutMode<CPP_NONE && m_uInpMode<CPP_NONE);


	if(iLevel     != -1)m_iModeLevel=iLevel;			// Kompimierungslevel
	if(m_iModeLevel<  1)m_iModeLevel=1;
	if(m_iModeLevel>  7)m_iModeLevel=7;

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CPP_ERROR_OK;
	iSubAlloc		= m_iModeLevel+7;
	iOrder			= 16-m_iModeLevel;
	
	Reset();
	InitRd(0x2000);
	InitWr(0x2000);
	
	PPM_LOCK();
	
	PpmdStartUp(); 

	if(!StartSubAllocator(iSubAlloc<<20)) 
		{
		m_iErrorCode = CPP_ERROR_MEMORY;
		throw "Zu wenig SPeicher";
		}


	if(m_iMode&0x01)									// ZIP Header schreiben
		{
		iProps	= (iSubAlloc-1)<<4;	
		iProps |= (iOrder-1);

		if(PutWord(iProps))
			{
			m_iErrorCode = CPP_ERROR_WRITE;
			throw "Fehler beim Speichern";
			}

		m_uCompresLen += 2;
		}


	try	{
		RcInitEncoder();						
		StartModelRare(iOrder,MRM_RESTART);

		
		for(;;) 
			{
			pMinContext	= pMaxContext;
			ucByte		= pMinContext->ucNumStats;

			if(GetByte(ucRead))						// EIn Byte einlesen
				{
				if(!Eof())
					{
					m_iErrorCode = CPP_ERROR_READ;
					throw 0;
					}

				iValue = -1;
				}
			else{
				iValue = ucRead;
				}


			if(ucByte) 
				{
				pMinContext->EncodeSymbol1(iValue);	
				RcEncodeSymbol();
				}
			else{
				pMinContext->EncodeBinSymbol(iValue);
				}

			while(!pFoundState) 
				{
				RcEncNormalize(this);

				do	{ 
					iOrderFall++;				
					pMinContext=pMinContext->pSuffix;
					if(!pMinContext)goto STOP_ENCODING;
					}while(pMinContext->ucNumStats == ucNumMasked);
				
				pMinContext->EncodeSymbol2(iValue);	
				RcEncodeSymbol();
				}

			if(!iOrderFall && (PPM_BYTE*)pFoundState->pSuccessor>=pUnitsStart)
				{
				PrefetchData(pMaxContext=pFoundState->pSuccessor);
				}
			else{ 
				UpdateModel (pMinContext);		
				PrefetchData(pMaxContext);

				if(ucEscCount==0)
					{
					ClearMask(this,this);
					}
				}

			RcEncNormalize(this);
			}

	STOP_ENCODING:

		RcFlushEncoder(this);			
		}
	catch(...)
		{
		PPM_UNLOCK();

		m_uDecompresLen	= GetPosInp();
		m_uCompresLen	= GetPosOut();

		if(m_iErrorCode==CPP_ERROR_WRITE   )throw "Fehler beim Speichern";
		if(m_iErrorCode==CPP_ERROR_READ    )throw "Fehler beim Lesen";
		   m_iErrorCode =CPP_ERROR_OVERFLOW;throw "Interner Fehler";
		}

	
	PPM_UNLOCK();

	if(Flush())										// Puffer leeren und in den Stream ausgeben
		{
		m_iErrorCode = CPP_ERROR_WRITE;
		throw "Fehler beim Speichern";
		}


	m_uDecompresLen	= GetPosInp();
	m_uCompresLen	= GetPosOut();



return m_uCompresLen;
}


//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
//	Startet die Endkomprimierung eines Streams
//	iMode		: Bestimmt die Endkomprimierungsart
//					Bit0  = Zip-Header Laden (2 Byte)
//					-2	  = Default Modus verwenden
//	Ergibt die Anzahl der Entkomprimierten Bytes
iocint64 CPpmClass::Decompress(int iMode)
{
PPM_CONTEXT	   *pMinContext;
int				iMyMethod;
int				iSubAlloc;
int				iDicSize;
int				iOrder;
short 			sProps;	
PPM_BYTE		ucByte;



	IOC_ASSERT(m_uOutMode<CPP_NONE && m_uInpMode<CPP_NONE);

	if(iMode==-2)
		{
		iMode = m_iMode;
		}


	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;
	m_iErrorCode	= CPP_ERROR_OK;

	Reset();	  
	InitRd(0x2000*8);
	InitWr(0x2000*8);

	if(iMode&0x01)									// Zip-Header Laden 
		{
		if(GetWord(sProps))
			{
			m_iErrorCode = CPP_ERROR_READ;
			throw "Fehler beim Lesen";
			}

		iOrder		= ((sProps    )&0x0F)+1;
		iSubAlloc	= ((sProps>> 4)&0xFF)+1;
		iMyMethod	= ((sProps>>12)&0x03);
		iDicSize	=   iSubAlloc<<20;
		}
	else if(m_uProbSize==5)							// 7ZIP Properties
		{
		iOrder		= m_aProbs[0];
		iDicSize	= *(PPM_DWORD*)(m_aProbs+1);
		iMyMethod	= MRM_RESTART; 
		}
	else{
		sProps		= *(short*)m_aProbs; 
		iOrder		= ((sProps    )&0x0F)+1;
		iSubAlloc	= ((sProps>> 4)&0xFF)+1;
		iMyMethod	= ((sProps>>12)&0x03);
		iDicSize	=   iSubAlloc<<20;
		}


	if(iOrder<2 || iOrder>MAX_O || iMyMethod>2)
		{
		m_iErrorCode = CPP_ERROR_BADFORMAT;
		throw "Formatfehler";
		}

	PPM_LOCK();
	
	PpmdStartUp(); 

	if(!StartSubAllocator(iDicSize))				// Speicher fr Alokator reservieren
		{
		m_iErrorCode = CPP_ERROR_MEMORY;
		throw "Zu wenig SPeicher";
		}

	if(iMode&0x02)									// Bei 7Zip muss erstes Byte 0 sein
		{
		ucByte = PPM_D_GETC(this);
		if(ucByte)throw CPP_ERROR_BADFORMAT;
		}

	try	{
		RcInitDecoder(this);				
		StartModelRare(iOrder,(MR_METHOD)iMyMethod);

		pMinContext = pMaxContext;
		ucByte		= pMinContext->ucNumStats;

		for(;;) 
			{
			if(ucByte) 
				{
				pMinContext->DecodeSymbol1();	
				RcRemoveSubrange();
				}
			else{
				pMinContext->DecodeBinSymbol();
				}

			while(!pFoundState) 
				{
				RcDecNormalize(this);
				
				do	{ 
					iOrderFall++;				
					pMinContext=pMinContext->pSuffix;
					if(!pMinContext)goto Done;
					}while(pMinContext->ucNumStats==ucNumMasked);
				
				pMinContext->DecodeSymbol2();	
				RcRemoveSubrange();
				}


			if(PutByte(pFoundState->ucSymbol))		// Dekotiertes Byte ausgeben	
				{
				m_iErrorCode = CPP_ERROR_WRITE;
				throw 0;
				}


			if(!iOrderFall && (PPM_BYTE*)pFoundState->pSuccessor>=pUnitsStart)
				{
				PrefetchData(pMaxContext=pFoundState->pSuccessor);
				}
			else{ 
				UpdateModel (pMinContext);		
				PrefetchData(pMaxContext);

				if(ucEscCount==0)
					{
					ClearMask(this,this);
					}
				}
			
			pMinContext	= pMaxContext;
			ucByte		= pMaxContext->ucNumStats;
			RcDecNormalize(this);
			}
		}
	catch(...)
		{
		PPM_UNLOCK();

		m_uDecompresLen	= GetPosInp();
		m_uCompresLen	= GetPosOut();

		if(m_iErrorCode==CPP_ERROR_WRITE   )throw "Fehler beim Speichern";
		if(m_iErrorCode==CPP_ERROR_READ    )throw "Fehler beim Lesen";
		   m_iErrorCode =CPP_ERROR_OVERFLOW;throw "Interner Fehler";
		}


Done:

	PPM_UNLOCK();

	if(Flush())										// Puffer leeren und in den Stream ausgeben
		{
		m_iErrorCode = CPP_ERROR_WRITE;
		throw "Fehler beim Speichern";
		}

	m_uDecompresLen	= GetPosOut();
	m_uCompresLen	= GetPosInp();



return m_uDecompresLen;
}

