//*****************************************************************************
//*
//*
//*		PpmClass.h
//*		
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2015
//
//	Klasse zum Komprimieren von Daten mit dem PPMD Algorithmus (WinZip/7Zip)
//
// 	unsigned Value(int)							: Wert  abfargen
// 	unsigned Flags(int)							: Flags setzen/abfragen
// 	unsigned Level(int)							: Level setzen/abfragen
// 	unsigned Error()							: Fehler Code holen
// 	unsigned Mode ()							: Modus setzen/abfragen
// 												
// 	int		 Compress  (int iLevel=-1);			: Starte   Komprimierung
//	int		 Decompress(int iOutLen,int iMode);	: Starte Dekomprimierung
// 												
// 	void	 SetOut(HANDLE,int,unsigned)		: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)		: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)				: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)				: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe
//

#ifndef		__PPMDCLASSEX_H__
#define		__PPMDCLASSEX_H__

#if _MSC_VER >= 1000
#pragma once
#endif 


#include	 "IoCallbacks.h"



//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define CPP_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CPP_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CPP_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CPP_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CPP_STORE			CBC_STORE			// Nur Gleich abspeichern

#define CPP_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CPP_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CPP_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CPP_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CPP_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CPP_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CPP_ERROR_BADFORMAT	CBC_ERROR_BADFORMAT	// Ungltige Bzip2-Format
#define CPP_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zuwenig Speicher

#define CPP_LEVEL			CBC_LEVEL			// Level ausgeben
#define CPP_FLAG			CBC_FLAG			// Flag Ausgeben
#define CPP_MODE			CBC_MODE			// Modus ausgeben
#define CPP_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CPP_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CPP_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CPP_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CPP_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CPP_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CPP_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes	
#define CPP_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CPP_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes	
#define CPP_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
												
#define CPP_NONE			IOC_NONE			// Ein/Ausgabe noch nicht definiert	
#define CPP_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CPP_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CPP_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CPP_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher

struct		See2Data;
class		ModelPpmd;

//*****************************************************************************
//*
//*		Klasse CPpmClass
//*
//*****************************************************************************
class CPpmClass : public CBaseCompressor
{
protected:




//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************

public:

						CPpmClass();					
	virtual			   ~CPpmClass();
	
	virtual	void		Free ();
	virtual	iocint64	Compress  (int iLevel=-1);
	virtual	iocint64	Decompress(int iMode =-2);
	int 				SetProbs(const void *pData,unsigned uSize=5);

	int					GetByteEx();
	void				PutByteEx(unsigned char ucData);

protected:

	void				Clear(int iMode=0);

	unsigned			m_uProbSize;			// Gre der Properties (2 oder 5)
	unsigned char 		m_aProbs[5];			// Speicher fr Properties
	iocint64			m_iDataBytes;			// Ausgabelnge (-1=Ende unbekannt)
	

};

#endif 

