//*****************************************************************************
//*
//*
//*		XzClass.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2013
//
//	Klasse zum Komprimieren von Daten XZ-Format (LZMA2 Algorithmus fr ZIP,XZ)
//
// 	unsigned Value(int)											: Wert  abfargen
// 	unsigned Flags(int)											: Flags setzen/abfragen
// 	unsigned Level(int)											: Level setzen/abfragen
// 	unsigned Error()											: Fehler Code holen
// 	unsigned Mode ()											: Modus setzen/abfragen
//
//	int		 Decompress(int Mode=-2);							: Starte Dekomprimierung
// 	int		 Compress()											: Starte   Komprimierung
//
// 	void	 SetOut(HNADLE,int,unsigned)						: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)						: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,char*)								: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,char*)								: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe

#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif

#include	<stdio.h>
#include	<string.h>
#include	"XzClass.h"
#include	"Lzma/Xz.h"
#include	"Lzma/XzEnc.h"
#include	"../Filter/Crc32.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CXzClass::CXzClass()
{

	m_uCompresLen	= 0;
	m_uDecompresLen	= 0;

	Clear(1);

}

//*****************************************************************************
//*
//*		Destrucktor
//*
//*****************************************************************************
CXzClass::~CXzClass()
{
	Clear(2);
}

//*****************************************************************************
//*
//*		Free
//*
//*****************************************************************************
void CXzClass::Free()
{

	Clear(2);

}

//*****************************************************************************
//*
//*		ThrowError
//*
//*****************************************************************************
void CXzClass::ThrowError(int iError)
{

	switch(iError)
		{
	case SZ_OK:					return;
	case SZ_ERROR_DATA:			m_iErrorCode = CXZ_ERROR_OVERFLOW;	throw "Datenfehler";
	case SZ_ERROR_MEM:			m_iErrorCode = CXZ_ERROR_MEMORY;	throw "Zuwenig Speicher";
	case SZ_ERROR_CRC:			m_iErrorCode = CXZ_ERROR_BADXZ;		throw "Flasche CRC-Summe";
	case SZ_ERROR_UNSUPPORTED:	m_iErrorCode = CXZ_ERROR_OVERFLOW;	throw "Funktion wird nicht untersttzt";
	case SZ_ERROR_PARAM:		m_iErrorCode = CXZ_ERROR_OVERFLOW;	throw "Falscher Parameter";
	case SZ_ERROR_INPUT_EOF:	m_iErrorCode = CXZ_ERROR_EOF;		throw "Ende der Eingabe erreicht";
	case SZ_ERROR_OUTPUT_EOF:	m_iErrorCode = CXZ_ERROR_WRITE;		throw "Ende der Ausgabe erreicht";
	case SZ_ERROR_READ:			m_iErrorCode = CXZ_ERROR_READ;		throw "Lesefehler";
	case SZ_ERROR_WRITE:		m_iErrorCode = CXZ_ERROR_WRITE;		throw "Schreibfehler";
	case SZ_ERROR_NO_ARCHIVE: 	m_iErrorCode = CXZ_ERROR_BADXZ;		throw "Falsches Archivformat";
	default:					m_iErrorCode = CXZ_ERROR_BADXZ;		throw "Unbekannter XZ Fehler";
		}

}

//*****************************************************************************
//*
//*		XzFree
//*
//*****************************************************************************
void __cdecl CXzClass::XzFree(void *pPtr,void *pMem)
{
	delete pMem;
}

//*****************************************************************************
//*
//*		XzAlloc
//*
//*****************************************************************************
void *__cdecl CXzClass::XzAlloc(void *pPtr,size_t uSize)
{
return new char[uSize];
}

//*****************************************************************************
//*
//*		XzRead
//*
//*****************************************************************************
//	Lesefunktion fr das Komprimieren
int __cdecl CXzClass::XzRead(void *pPtr,void *pMem,size_t *pSize)
{
ISeqInStream   *pInp =(ISeqInStream *)pPtr;
CXzClass	   *pThis=(CXzClass*)pInp->ptr;
int				iSize;
int				iRet;


	iSize = (int)*pSize;
	iRet  = pThis->Read(pMem,iSize);

	if(iRet>=0)
		{
	   *pSize = iRet;
		return SZ_OK;
		}


return (pThis->GetPosInp()>0)? SZ_ERROR_INPUT_EOF:SZ_ERROR_READ;
}

//*****************************************************************************
//*
//*		XzWrite
//*
//*****************************************************************************
//	Schreibfunktion fr das Komprimieren
size_t __cdecl CXzClass::XzWrite(void *pPtr,const void *pMem,size_t uSize)
{
ISeqOutStream  *pOut =(ISeqOutStream*)pPtr;
CXzClass	   *pThis=(CXzClass*)pOut->ptr;
int				iRet;


	iRet  =  pThis->Write((void*)pMem,uSize);

	if(iRet>0)return iRet;


return 0;
}

//*****************************************************************************
//*
//*		XzProgress
//*
//*****************************************************************************
//	Verlaufsfunktion fr das Komprimieren
int __cdecl CXzClass::XzProgress(void *pPtr,unsigned iocint64 uInpSize,unsigned iocint64 uOutSize)
{
ICompressProgress  *pOut =(ICompressProgress*)pPtr;
CXzClass	       *pThis=(CXzClass*)pOut->ptr;


	pThis->m_uCompresLen   = uOutSize;
	pThis->m_uDecompresLen = uInpSize;


return SZ_OK;
}


//*****************************************************************************
//*
//*		Commpress
//*
//*****************************************************************************
//	Beginnt mit der Kompression und ergibt die komprimierte Lnge
//	Fehler  werden mit throw(char *Text) ausgegeben
//	Fehkler Nummern knnen ber Error() geholt werden
//	Ein und Ausgaben mssen zuvor mit SetIn/SetOut eingestellt werden
iocint64 CXzClass::Compress(int iLevel)
{
ICompressProgress	sProgress;
ISeqOutStream		sOutStream;
ISeqInStream		sInpStream;
CLzma2EncProps      sLzma2Props;
int 				iRet;



	IOC_ASSERT(m_uOutMode<CXZ_NONE && m_uInpMode<CXZ_NONE);

	if(iLevel!=-1)m_iModeLevel=iLevel;						// Kompimierungs Level
	if(m_iModeLevel<0)m_iModeLevel=0;
	if(m_iModeLevel>9)m_iModeLevel=9;

	Reset();

	m_iModeFlags		= 0;
	m_iErrorCode		= 0;
	m_uCompresLen		= 0;
	m_uDecompresLen		= 0;
	m_iModeFlags	   &=~CXZ_NOCOMPESS;
	m_iMode				= CXZ_DEFLATE;


	sInpStream.Read		= XzRead;
	sInpStream.ptr		= this;
	sOutStream.Write	= XzWrite;
	sOutStream.ptr		= this;
	sProgress .Progress	= XzProgress;
	sProgress .ptr		= this;

	Lzma2EncProps_Init(&sLzma2Props);

	iRet = Xz_Encode(&sOutStream,&sInpStream,&sLzma2Props,False,&sProgress);

	if(iRet!=SZ_OK)ThrowError(iRet);

	m_uDecompresLen = GetPosInp();
	m_uCompresLen	= GetPosOut();


return m_uCompresLen;
}

//*****************************************************************************
//*
//*		Decompress
//*
//*****************************************************************************
iocint64 CXzClass::Decompress(int iMode)
{
unsigned		uInpBufSize=1<<15;
unsigned		uOutBufSize=1<<21;
ISzAlloc		sAlloc;
ECoderStatus	iStatus;
CXzStream		sStream;
CXzUnpacker		sUnpack;
unsigned char  *pOutput;
unsigned char  *pInput;
unsigned		uFinished;
unsigned		uInpSize;
unsigned		uInpPos;
unsigned		uOutPos;
SizeT			uInpLen;
SizeT			uOutLen;
int				iStop;
int				iRet;
int				iRes;



	IOC_ASSERT(m_uOutMode<CXZ_NONE && m_uInpMode<CXZ_NONE);

	if(iMode==-2)iMode=m_iMode;

	Reset();

	m_uDecompresLen	= 0;
	m_uCompresLen	= 0;


	if(m_uInpMode!=CXZ_MEM)
		{
			pInput	= new unsigned char[uInpBufSize];
		if(!pInput)
			{
			ThrowError(SZ_ERROR_MEM);
			}

		uInpSize = 0;
		}

	if(m_uOutMode!=CXZ_MEM)
		{
			pOutput	= new unsigned char[uOutBufSize];
		if(!pOutput)
			{
			if(m_uInpMode!=CXZ_MEM)delete pInput;
			ThrowError(SZ_ERROR_MEM);
			}
		}
	else{
		pOutput		= (unsigned char*)GetOutMem();
		uOutBufSize	=				  GetOutLen();
		}


	sAlloc.Alloc	= XzAlloc;
	sAlloc.Free		= XzFree;


	Xz_Construct(&sStream);

	   iRet = XzUnpacker_Create(&sUnpack,&sAlloc);
	if(iRet!=SZ_OK)
		{
		ThrowError(iRet);
		}

	iStatus	 = CODER_STATUS_NOT_SPECIFIED;
	uOutLen  = 0;
	uOutPos	 = 0;
	uInpLen  = 0;
	uInpPos  = 0;
	uInpSize = 0;
	iStop	 = 0;


	for(;;)
		{
		if(uInpPos==uInpSize)								// Ist der Einganspuffer leer
			{
			if(m_uInpMode==CXZ_MEM)
				{
				pInput		   = (unsigned char*)GetInpMem();
				uInpLen		   =				 GetInpLen();

				if(uInpLen>1<<15)uInpLen=1<<15;

				m_uCompresLen += (unsigned)uInpLen;
				uInpBufSize	   = (unsigned)uInpLen;
				uInpSize	   = (unsigned)uInpLen;
				uInpPos		   = (unsigned)0;

				TakeInp((unsigned)uInpLen);
				}
			else{
				uInpPos  = 0;
				uInpSize = 0;

				for(;;)
					{
					   iRet = Read(pInput+uInpPos,uInpBufSize-uInpPos);
					if(iRet<=0)break;

					uInpSize      += iRet;
					m_uCompresLen += iRet;

					if(uInpSize*2>=uInpBufSize)break;		// Ist der Puffer voll genug
					}

				if(iRet<0)
					{
					iRet = SZ_ERROR_READ;
					break;
					}

				}
			}

		uInpLen  = uInpSize-uInpPos;
		uOutLen  = uOutBufSize-uOutPos;
		iRes     = XzUnpacker_Code(&sUnpack,pOutput+uOutPos,&uOutLen,pInput+uInpPos,&uInpLen,(uInpSize==0)? CODER_FINISH_END:CODER_FINISH_ANY,&iStatus);
		uInpPos += (unsigned)uInpLen;

		if(!uInpLen && !uOutLen)							// Manchmal bleibt der Algorithmus hngen
			{
			iStop++;
			
			if(iStop>16)
				{
				if(m_iOutMax>0 && m_uDecompresLen>(unsigned)m_iOutMax)
					{
					break;
					}
				
				if(uInpPos+64>uInpSize && uInpPos+4<uInpSize && pInput[uInpPos]==0)
					{
					break;
					}
				
				iRes = SZ_ERROR_FAIL;
				break;
				}
			}
		else{
		 	iStop = 0;
			}

		if(iRes!=SZ_OK)
			{
			break;
			}

		if(m_uOutMode==CXZ_MEM)								// Dekomprimierte Bytes speichern
			{
			TakeOut ((unsigned)uOutLen);
			uOutPos+=(unsigned)uOutLen;
			}
		else{
			   iRet =Write(pOutput+uOutPos,(unsigned)uOutLen);
			if(iRet^uOutLen)
				{
				if(iRet>0)m_uDecompresLen+=iRet;
				iRet = SZ_ERROR_WRITE;
				break;
				}

			uOutPos = 0;
			}


		m_uDecompresLen += uOutLen;

			uFinished = (uInpLen==0 && uOutLen==0) && iRes==SZ_OK;
		if(!uFinished || iStatus==CODER_STATUS_NOT_FINISHED)// Ende erreicht
			{
			continue;
			}

		if(iStatus==CODER_STATUS_NEEDS_MORE_INPUT)
			{
			if(!XzUnpacker_IsStreamWasFinished(&sUnpack))
				{
				iRes = SZ_ERROR_DATA;
				}

			break;
			}
		else if(iStatus==CODER_STATUS_FINISHED_WITH_MARK)
			{
			break;
			}

		iRes = SZ_ERROR_FAIL;
		break;
		}


	if(m_uInpMode!=CXZ_MEM)delete pInput;
	if(m_uOutMode!=CXZ_MEM)delete pOutput;

	XzUnpacker_Free(&sUnpack);


	if(iRes!=SZ_OK)										// Gab es Fehler
		{
		ThrowError(iRes);
		}



return m_uDecompresLen;
}

