//*****************************************************************************
//*
//*
//*		XzClass.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2013
//
//	Klasse zum Komprimieren von Daten im XZ-Format (LZMA2 Algorithmus fr ZIP,XZ)
//
// 	unsigned Value(int)										: Wert  abfargen
// 	unsigned Flags(int)										: Flags setzen/abfragen
// 	unsigned Level(int)										: Level setzen/abfragen
// 	unsigned Error()										: Fehler Code holen
// 	unsigned Mode ()										: Modus setzen/abfragen
//
//	int		 Decompress(int Mode=-2);						: Starte Dekomprimierung
// 	int		 Compress()										: Starte   Komprimierung
//
// 	void	 SetOut(HANDLE,int,unsigned)					: Datei -Ausgabe bestimmen
// 	void	 SetIn (HANDLE,int,unsigned)					: Datei -Eingabe bestimmen
// 	void	 SetOut(unsigned,void*)							: Memory-Ausgabe bestimmen
// 	void	 SetIn (unsigned,void*)							: Memory-Eingabe bestimmen
// 	void	 SetOut(void*,int(*pProc)(void*,char*,unsigned))	: Externe Ausgabe
// 	void	 SetIn (void*,int(*pProc)(void*,char*,unsigned))	: Externe Eingabe


#ifndef	__XZCLASS_H__
#define	__XZCLASS_H__

#if _MSC_VER >= 1000
#pragma once
#endif

#include	 "IoCallbacks.h"


//*****************************************************************************
//*
//*		Definitionen
//*
//*****************************************************************************

#define CXZ_SLOW			CBC_SLOW			// Kompress Flag bei Level <=2
#define CXZ_FAST			CBC_FAST			// Kompress Flag bei Level >=8
#define CXZ_NOCOMPESS	    CBC_NOCOMPESS		// Fehler beim Komprimieren (Ziel grer Quelle)
#define CXZ_DEFLATE			CBC_DEFLATE   		// Komprimieren
#define CXZ_STORE			CBC_STORE			// Nur Gleich abspeichern

#define CXZ_ERROR_OK		CBC_ERROR_OK		// Kein Fehler
#define CXZ_ERROR_READ		CBC_ERROR_READ		// Fehler beim Lesen
#define CXZ_ERROR_WRITE		CBC_ERROR_WRITE		// Fehler beim Schreiben
#define CXZ_ERROR_OVERFLOW	CBC_ERROR_OVERFLOW	// Speicher berlauf
#define CXZ_ERROR_EOF     	CBC_ERROR_EOF		// Vorzeitiges Eingabeende
#define CXZ_ERROR_MODE    	CBC_ERROR_MODE		// Falscher Modus
#define CXZ_ERROR_BADXZ  	CBC_ERROR_BADFORMAT	// Ungltige Bzip2-Format
#define CXZ_ERROR_MEMORY  	CBC_ERROR_MEMORY	// Zuwenig Speicher

#define CXZ_LEVEL			CBC_LEVEL			// Level ausgeben
#define CXZ_FLAG			CBC_FLAG			// Flag Ausgeben
#define CXZ_MODE			CBC_MODE			// Modus ausgeben
#define CXZ_OCRC			CBC_OCRC			// Out CRC ausgeben
#define CXZ_ICRC			CBC_ICRC			// In  CRC ausgeben
#define CXZ_SIZE			CBC_SIZE			// Komprimierte Lnge ausgeben
#define CXZ_SIZEH			CBC_SIZEH			// Komprimierte Lnge ausgeben (obere 32 Bit)
#define CXZ_LEN				CBC_LEN				// Dekmprimierte Lnge ausgeben
#define CXZ_LENH			CBC_LENH			// Dekmprimierte Lnge ausgeben  (obere 32 Bit)
#define CXZ_INP				CBC_INP				// Aktuelle geladene/gespeicherte Bytes
#define CXZ_INPH			CBC_INPH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)
#define CXZ_OUT				CBC_OUT				// Aktuelle geladene/gespeicherte Bytes
#define CXZ_OUTH			CBC_OUTH			// Aktuelle geladene/gespeicherte Bytes	(obere 32 Bit)

#define CXZ_NONE			IOC_NONE			// Ein/Ausgabe noch nicht definiert
#define CXZ_HANDLE			IOC_HANDLE			// Ein/Ausgabe ber Dateien mit HANDLE
#define CXZ_FILE			IOC_FILE			// Ein/Ausgabe ber Dateien	mit *FILE
#define CXZ_EXT				IOC_EXT				// Ein/Ausgabe ber externe Funktiorn
#define CXZ_MEM				IOC_MEM				// Ein/Ausgabe ber Speicher


//*****************************************************************************
//*
//*		Klasse CXzClass
//*
//*****************************************************************************
class CXzClass : public CBaseCompressor
{
public:

					 CXzClass();
	virtual		    ~CXzClass();

	virtual iocint64 Compress  (int  iLevel=-1);
	virtual iocint64 Decompress(int  iMode =-2);
	void			 ThrowError(int  iError);
	void			 Free();

protected:

	static	int		 XzProgress(void *pPtr,unsigned iocint64 uInpSize,unsigned iocint64 uOutSize);
	static	int		 XzRead    (void *pPtr,      void *pMem,size_t *pSize);
	static	size_t   XzWrite   (void *pPtr,const void *pMem,size_t  uSize);
	static	void	 XzFree    (void *pPtr,void       *pMem);
	static	void    *XzAlloc   (void *pPtr,size_t      uSize);

};


//*****************************************************************************
//*
//*		Inline Funktioen
//*
//*****************************************************************************

#endif
