//*****************************************************************************
//*
//*
//*		ExeFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
//
//	Klasse fr den Zugriff auf Exe-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef		__EXEFILE_H__
#define     __EXEFILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	"DataField.h"
#include	"FileFinder.h"
#include	"FileFunctions.h"
#include	"ArchiveFile.h"
#include	"Filter/Crc32.h"
#include	"Codecs/Huffman.h"


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif


//*****************************************************************************
//*
//*		Klasse CExeFile
//*
//*****************************************************************************
class CExeFile : public CArchiveFile
{
protected:

	typedef struct
	{
    unsigned char   Name[8];
    union 
		{
    unsigned		PhysicalAddress;
    unsigned		VirtualSize;
		}			Misc;
    unsigned		VirtualAddress;
    unsigned		SizeOfRawData;
    unsigned		PointerToRawData;
    unsigned		PointerToRelocations;
    unsigned		PointerToLinenumbers;
    unsigned short	NumberOfRelocations;
    unsigned short	NumberOfLinenumbers;
    unsigned		Characteristics;
	}SectionHeader;

	typedef struct 
	{
   	unsigned  		biSize;
	int  			biWidth;
   	int  			biHeight;
   	unsigned short	biPlanes;
   	unsigned short	biBitCount;
   	unsigned  		biCompression;
   	unsigned  		biSizeImage;
   	int  			biXPelsPerMeter;
   	int  			biYPelsPerMeter;
   	unsigned  		biClrUsed;
   	unsigned  		biClrImportant;
	}BitmapInfo;

	typedef struct
	{
    unsigned char	bWidth;          			// Width, in pixels, of the image
    unsigned char	bHeight;         			// Height, in pixels, of the image
    unsigned char	bColorCount;     			// Number of colors in image (0 if >=8bpp)
    unsigned char	bReserved;       			// Reserved ( must be 0)
    unsigned short	wPlanes;         			// Color Planes
    unsigned short	wBitCount;       			// Bits per pixel
    unsigned int 	dwBytesInRes;    			// How many bytes in this resource?
    unsigned int  	dwImageOffset;   			// Where in the file is this image?
	}IconEntry;						 			

	typedef struct
	{
    unsigned short	idReserved;					// Reserved (must be 0)
    unsigned short	idType;       	 			// Resource Type (1 for icons 2 for cursors)
    unsigned short	idCount;      	 			// How many images?
	}ItemDir;

	typedef struct
	{
    unsigned char	bWidth;          			// Width, in pixels, of the image
    unsigned char	bHeight;         			// Height, in pixels, of the image
    unsigned char	bColorCount;     			// Number of colors in image (0 ==> 1bpp)
    unsigned char	bReserved;       			// Reserved ( must be 0)
    unsigned short	wHotSpotX;         			// Hotspot X position
    unsigned short	wHotSpotY;       			// Hotspot Y position
    unsigned int 	dwBytesInRes;    			// How many bytes in this resource?
    unsigned int  	dwImageOffset;   			// Where in the file is this image?
	}CursorEntry;						 			

	typedef struct ExeItem
	{
	void		   *pUserPtr;					// User-Pointer
	zchar		   *pName;       				// File name in Datei
	time_t			iTime;						// Zeit der letzten nderung
	unsigned	 	uNameLen;					// Dateinamen-Lnge
	unsigned short 	uType;						// Typ der Resource
	unsigned short 	uLang;						// Codepage Nummer
	unsigned short	uId;						// ID der Resource
	char			cError;						// Fehler-Flag
	char			cDecomp;					// Wurde die Datei entpackt
	int 			iLen;						// Lnge der Datei 
	int				aList[5];					// Liste mit Offsets ([0]=Typ [1]=Len [2]=...)
	int			   *pList;						// Zeiger auf die Liste
	}ExeItem;

	unsigned 		m_uExeSize;					// ExeFile Lnge in Byte
	unsigned		m_uFileTime;				// nderungszeit des Archivs
	int				m_iOpenMode;				// ffnungs Modus

	ExeItem       **m_pFiles;					// Alfabethische Liste
	int				m_iFilesNum;				// Anzahl der Listeneintrge	
	int				m_iFilesMax;				// Gesamt Listengre in Eintrgen
	
	aint64			m_iDoneNow;					// Aktueller Verlauf
	aint64			m_iDoneMax;					// Maximaler Verlauf
	int 			m_iDoneMode;				// 0=direkt von m_iDoneNow 1=Extraxt 2=ExtractAll 3=Flush

	CDataPointer	m_sDataList;				// Daten fr spezielle Eintrge
	


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

					CExeFile();
	virtual		   ~CExeFile();

	virtual	int		ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo=0);

	virtual	int		Close		();						// Datei schliesen
	virtual	int	    Count  		();						// Anzahl der Dateien
	virtual	void	Break  	    ();						// Bearbeitung abbrechen
	virtual	int		Sort   		(int  iMode=0);			// Sortierte Kette
	virtual	zchar  *Name		(int  iNum,int &iLen);	// Hole Dateiname mit Lnge
	virtual	zchar  *Name   		(int  iNum);			// Hole Dateiname
	virtual	int		Stats		(int  iNum);			// Zustansbits abfragen
	virtual	int		SetFail		(int  iNum,int iFail);	// Fehlerbit setzen
	virtual	int     IsFail 		(int  iNum);			// Ist die Datei defekt
	virtual	int     IsDir  		(int  iNum);			// Prfe ob Verzeichnis
	virtual	int     IsOk   		(int  iNum);			// Richtig endkomprimiert
	virtual	aint64	Len    		(int  iNum,int iMode=0);// Hole Dateilnge
	virtual	time_t	Time   		(int  iNum);			// Dateizeit abfragen
	virtual	time_t	Time   		(int  iNum,int iMode);	// Dateizeit abfragen
	virtual	int		Find   		(const zchar  *pName);	// Datei suchen
	virtual	aint64	Done   		(aint64 *pMax);			// Verlaufsabfrage
	virtual	void   *GetUserPtr  (int  iNum            );// Datenwert auslesen
	virtual	void    SetUserPtr  (int  iNum,void *pData);// Datenwert einstellen

protected:

	void			Clear       ();						// Speicher lschen
	virtual	int		ArchOpen	(int iLevel=9);			// Die Archiv Datei ffnen
	virtual	int		SetDoneMode (int  iMode);			// Modus fr Done setzen
	time_t			TimeConvertE(unsigned char *pTime);
	time_t			TimeConvert (unsigned char *pTime);

	void			ExeDel		(int  iNum);			// Listen Eintrag lschen
	int				ExeNew		();						// Listen Eintrag erzeugen
							  	
	void			ScanPe    	(unsigned char *pMem,int iOffset);
	int				CheckImage	(ExeItem *pEntry ,zchar   *pName,unsigned &uSize,unsigned char *pData);
	int				AddBitmap 	(unsigned uOffset,unsigned uSize,unsigned char *pData);
	int				AddString 	(unsigned uOffset,unsigned uSize,unsigned char *pData,ExeItem *pEntry);
	int				AddCursor 	(unsigned uOffset,unsigned uSize,unsigned char *pData,ExeItem *pEntry);
	int				AddIcon   	(unsigned uOffset,unsigned uSize,unsigned char *pData,ExeItem *pEntry);
							  	
static unsigned		RvaToOffset	(unsigned uRva,unsigned uCount,SectionHeader *pSections);
static int __cdecl 	CmpName 	(const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpLen  	(const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpPos  	(const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpEx   	(const void *pPtr1,const void *pPtr2);// Vergleichsfunktion

};

#endif



