//*****************************************************************************
//*
//*
//*		FatFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
//
//	Klasse fr den Zugriff auf Fat Files:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int uLen)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen
//
#include	<stdio.h>
#include	<stddef.h>
#include	<string.h>
#include	<malloc.h>
#include	<sys/stat.h>
#include	"FatFile.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	GET_SECTOR_COUNT	CTRL_SECTOR_COUNT
#define 	_USE_MKFS	1
#define		_VOLUMES	4
#define 	FS_SEC32	1
#define 	FS_STATIC	static
#define 	FS_API		extern "C"
#include	"../FileSystem/Fat/ff.c"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		wcslen
#define 	strzcmp		wcscmp
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#define 	strzncat	wcsncat
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		strlen
#define 	strzcmp		strcmp
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#define 	strzncat	strncat
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif


#define 	ENTRIES(a)	 (sizeof(a)/sizeof((a)[0]))	

#pragma pack(1)

typedef struct
	{
	BYTE	bBootIndicator;       		// 0x80 for bootable, 0 otherwise
    BYTE	bStartHead;           		// 0 based
    BYTE	bStartSector;         		// 1 based, bits 0..5, bits 6,7 are bits 8,9 of StartTrack
    BYTE	bStartTrack;          		// 0 based, bits 0..7
    BYTE	bOSType;              		// FAT-12: 1, FAT-16: 4, 6, 14, FAT-32: 11, 12
    BYTE	bEndHead;             		// see StartHead
    BYTE	bEndSector;           		// see StartSector
    BYTE	bEndTrack;					// see StartTrack
	DWORD	dwRelativeSector;			// offset to first sector of partition data
    DWORD	dwSectors;					// number of sectors in partition
	}PartitionRecord;

typedef struct
	{
	BYTE			bBootCode[440];
	DWORD			dwSerialNumber;
	WORD			wFree;
	PartitionRecord sTable[4];
	WORD			wSignature;			// 0xAA55
	}MasterBootRecord;

typedef struct
	{
	BYTE	bNearJump[3];				// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;			// relative to partition start
	BYTE    bNumberOfFATs;
	WORD    wDirEntries;				// zero on FAT-32
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;
	WORD    wSectorsPerFAT;				// zero on FAT-32
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;	// OS specific, unreliable
	DWORD	dwTotalSectors;
	BYTE    bPhysicalDiskNumber;		// 0x80, 0x81, etc.
	BYTE    bCurrentHead;				// not used
	BYTE    bSignature;					// needed by NT
	DWORD   dwSerialNumber;
	char    bLabel[11];					// not used
	char    cSystemID[8];
	BYTE	bBootCode[448];
	WORD	wSignature;					// 0xAA55
	}BootRecord16;

typedef struct
	{
	BYTE	bNearJump[3];				// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;			// relative to partition start
	BYTE    bNumberOfFATs;
	WORD    wDirEntries;				// zero on FAT-32
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;
	WORD    wSectorsPerFAT;				// zero on FAT-32
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;	// OS specific, unreliable
	DWORD	dwTotalSectors;
	DWORD	dwSectorsPerFAT;
	WORD    wFlags;						// if (Flags & 0x0080) { SingleFAT; ActiveFat = (Flags & 0x000F); }
	WORD    wVersion;
	DWORD	dwRootDirCluster;
	WORD    wFSInfoSector;				// relative to start of partition, usually 1, 0xFFFF for none.
	WORD    wBackupBootSector;			// usually 6
	BYTE    bReserved[12];				// should be all zero
	BYTE    bPhysicalDiskNumber;		// 0x80, 0x81, etc.
	BYTE    bCurrentHead;				// not used
	BYTE    bSignature;					// needed by NT
	DWORD   dwSerialNumber;
	char    bLabel[11];					// not used
	char    cSystemID[8];
	BYTE	bBootCode[420];
	WORD	wSignature;					// 0xAA55
	}BootRecord32;

#pragma pack()


#define PROG(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))throw CAF_ERROR_BREAK;	\
	}
#define PROGR(iMode,iNum,iRet)						\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))return iRet;			\
	}
#define PROGN(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	Progress(iMode,iNum);							\
	}

typedef struct
	{
	FATDATA		sFatData;
	FATFS		aDrives  [_VOLUMES];
	unsigned	aStartSec[_VOLUMES];
	unsigned	aCountSec[_VOLUMES];
	unsigned	uDriveCount;
	CFatFile   *pThis;
	}FatEntries;

unsigned char CFatFile::m_aMbrDefault[512]=
	{
	0xFA,0x33,0xC0,0x8E,0xD0,0xBC,0x00,0x7C,0x8B,0xF4,0x50,0x07,0x50,0x1F,0xFB,0xFC,
	0xBF,0x00,0x06,0xB9,0x00,0x01,0xF3,0xA5,0xEA,0x1D,0x06,0x00,0x00,0xBE,0x77,0x07,
	0xE8,0x36,0x00,0xE8,0x98,0x00,0xBE,0xBE,0x07,0xB3,0x04,0xF6,0x04,0x78,0x75,0x21,
	0xF6,0x04,0x80,0x75,0x09,0x83,0xC6,0x10,0xFE,0xCB,0x75,0xEF,0xCD,0x18,0x8B,0x14,
	0x8B,0x4C,0x02,0x8B,0xEE,0x83,0xC6,0x10,0xFE,0xCB,0x74,0x20,0x80,0x3C,0x00,0x74,
	0xF4,0xBE,0x07,0x07,0xE8,0x02,0x00,0xEB,0xFE,0x60,0xAC,0x3C,0x00,0x74,0x0B,0x56,
	0xBB,0x07,0x00,0xB4,0x0E,0xCD,0x10,0x5E,0xEB,0xF0,0x61,0xC3,0xBF,0x05,0x00,0xB4,
	0x41,0xBB,0xAA,0x55,0xCD,0x13,0x72,0x1B,0x81,0xFB,0x55,0xAA,0x75,0x15,0xBE,0xF7,
	0x06,0x8B,0x46,0x08,0x89,0x44,0x08,0x8B,0x46,0x0A,0x89,0x44,0x0A,0xB4,0x42,0xCD,
	0x13,0x73,0x18,0xBB,0x00,0x7C,0xB8,0x01,0x02,0x57,0xCD,0x13,0x5F,0x73,0x0C,0x33,
	0xC0,0xCD,0x13,0x4F,0x75,0xC9,0xBE,0x21,0x07,0xEB,0xA9,0xBE,0x42,0x07,0xBF,0xFE,
	0x7D,0x81,0x3D,0x55,0xAA,0x75,0x9D,0x8B,0xF5,0xEA,0x00,0x7C,0x00,0x00,0x60,0xBF,
	0x05,0x00,0xBB,0x00,0x60,0xB8,0x08,0x02,0xB9,0x02,0x00,0xBA,0x80,0x00,0x57,0xCD,
	0x13,0x5F,0x73,0x0D,0x33,0xC0,0xCD,0x13,0x4F,0x75,0xE7,0xBE,0x5D,0x07,0xE9,0x73,
	0xFF,0xBF,0x00,0x60,0x81,0x3D,0x55,0xAA,0x75,0x0B,0xBE,0x8D,0x07,0xE8,0x69,0xFF,
	0x9A,0x02,0x00,0x00,0x06,0x61,0xC3,0x10,0x00,0x01,0x00,0x00,0x7C,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x49,0x6E,0x76,0x61,0x6C,0x69,0x64,0x20,0x70,
	0x61,0x72,0x74,0x69,0x74,0x69,0x6F,0x6E,0x20,0x74,0x61,0x62,0x6C,0x65,0x0A,0x0D,
	0x00,0x45,0x72,0x72,0x6F,0x72,0x20,0x6C,0x6F,0x61,0x64,0x69,0x6E,0x67,0x20,0x6F,
	0x70,0x65,0x72,0x61,0x74,0x69,0x6E,0x67,0x20,0x73,0x79,0x73,0x74,0x65,0x6D,0x0A,
	0x0D,0x00,0x4D,0x69,0x73,0x73,0x69,0x6E,0x67,0x20,0x6F,0x70,0x65,0x72,0x61,0x74,
	0x69,0x6E,0x67,0x20,0x73,0x79,0x73,0x74,0x65,0x6D,0x0A,0x0D,0x00,0x45,0x72,0x72,
	0x6F,0x72,0x20,0x6C,0x6F,0x61,0x64,0x69,0x6E,0x67,0x20,0x65,0x78,0x74,0x65,0x6E,
	0x73,0x69,0x6F,0x6E,0x0A,0x0D,0x00,0x53,0x74,0x61,0x72,0x74,0x20,0x41,0x5A,0x2D,
	0x42,0x6F,0x6F,0x74,0x4C,0x6F,0x61,0x64,0x65,0x72,0x0A,0x0D,0x00,0x53,0x74,0x61,
	0x72,0x74,0x20,0x41,0x5A,0x2D,0x45,0x78,0x74,0x65,0x6E,0x73,0x69,0x6F,0x6E,0x0A,
	0x0D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0xAA,
	};


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFatFile::CFatFile()
{

	m_pMyFat		=  0;
	m_pFiles		=  0;
	m_iFilesMax		=  0;
	m_iFilesNum		=  0;
	m_pErrorText	=  0;
	m_iDoneNow		=  0;
	m_iDoneMax		=  0;
	m_iDoneMode		=  0;
	m_iReadPos		=  0;
	m_iOpenMode		=  0;
	m_uFatDrives	=  0;
	m_uFatDrvOk		=  0;
	m_uNameOff		=  0;

	DefaultOptions();

	Clear();

}
	    
//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFatFile::~CFatFile()
{
	if(m_pMyFat)
		{
		delete m_pMyFat;
		m_pMyFat = 0;
		}
	
	if(m_iOpenMode!=CAF_CLOSED)Close();
	
	Clear();

}
			 
//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Datenfelder und setzt alle Variabeln zurck
void CFatFile::Clear()
{
int	iPos;



	for(iPos=m_iFilesNum-1;iPos>=0;iPos--)
		{
		FatDel(iPos);
		}

	m_iFilesNum		= 0;
	m_uNameOff		= 0;
	m_uFatDrives	= 0;
	m_uFatDrvOk		= 0;
	m_iCreateCnt	= 0;

}


//*****************************************************************************
//*
//*		CmpEx
//*
//*****************************************************************************
int __cdecl CFatFile::CmpEx(const void *pPos1,const void *pPos2)
{
FatItem	*pPtr1,*pPtr2;

	pPtr1 = *(FatItem**)pPos1;
	pPtr2 = *(FatItem**)pPos2;

return strvicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpName
//*
//*****************************************************************************
int __cdecl CFatFile::CmpName(const void *pPos1,const void *pPos2)
{
FatItem	*pPtr1,*pPtr2;

	pPtr1 = *(FatItem**)pPos1;
	pPtr2 = *(FatItem**)pPos2;

return strzicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpLen
//*
//*****************************************************************************
int __cdecl CFatFile::CmpLen(const void *pPos1,const void *pPos2)
{
FatItem	*pPtr1,*pPtr2;


	pPtr1 = *(FatItem**)pPos1;
	pPtr2 = *(FatItem**)pPos2;

	if(pPtr1->iLen==pPtr2->iLen)return 0;

return (pPtr1->iLen<pPtr2->iLen)? -1:0;
}

//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CFatFile::CmpPos(const void *pPos1,const void * pPos2)
{
FatItem	*pPtr1,*pPtr2;

	pPtr1 = *(FatItem**)pPos1;
	pPtr2 = *(FatItem**)pPos2;

	if(pPtr1->uOff==pPtr2->uOff)return 0;

return (pPtr1->uOff<pPtr2->uOff)? -1:1;
}

//*****************************************************************************
//*
//*		CheckSize
//*
//*****************************************************************************
//	Ergibt 1 wenn die Gre verletzt wurde.
inline int CFatFile::CheckSize(unsigned uSector,unsigned aint64 uSize)
{
unsigned 	uNum;


	uNum = (unsigned)((uSize+2047)/512);

	if(uSector     >m_uFatSectors)return 1;
	if(uSector+uNum>m_uFatSectors)return 1;
	if(uSector+uNum<uSector		 )return 1;


return 0;
}

//*****************************************************************************
//*
//*		Len
//*
//*****************************************************************************
//	Ergibt die Lnge der Datei mit der Indexnummer iNum
//	Ist iMode==0 wird die originale    Lnge zurckgegeben
//	Ist iMode!=0 wird die komprimierte Lnge zurckgegeben
aint64 CFatFile::Len(int iNum,int iMode)
{
FatItem	*pEntry;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;
	pEntry = m_pFiles[iNum];

return pEntry->iLen;
}

//*****************************************************************************
//*
//*		Stats
//*
//*****************************************************************************
//	Die Zustandbits abfragen
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//	iDel		: Welche Bits sollen gelscht werden
//	Ergibt einen Wert mit veschiedenen Bits:
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//					CAF_STAT_DELETE		0x0004	Die Datei wurde aus dem Verzeichnis gelscht
//					CAF_STAT_WRITE		0x0008	Die Datei muss ins Archiv geschrieben werden
//					CAF_STAT_DIR		0x0010	Die Datei ist ein Verzeichniss
//					CAF_STAT_READONLY	0x0020	Die Datei ist schreibgeschtzt
//					CAF_STAT_SAVED		0x0040	Die Datei ist im Archiv gespeichert
int CFatFile::Stats(int iNum)
{
FatItem	   *pEntry;
int			iBits;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return 0;
		}

			pEntry=m_pFiles[iNum];
	iBits = pEntry->cDecomp&3;

						 iBits|=CAF_STAT_SAVED;
	if(pEntry->cDecomp  )iBits|=CAF_STAT_DECOMP;
	if(pEntry->cRdOnly  )iBits|=CAF_STAT_READONLY;
	if(pEntry->cDel	    )iBits|=CAF_STAT_DELETE;
	if(pEntry->cDir		)iBits|=CAF_STAT_DIR;
	

return iBits;	
}

//*****************************************************************************
//*
//*		SetStats
//*
//*****************************************************************************
//	Den Zustand einer Arichiv Datei verndern.
//	iNum	: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet	: Welche Bits sollen gesetzt werden
//				CAF_STAT_NONE	0x0000	Die Datei wurde noch nicht entpackt
//				CAF_STAT_EXIST	0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//				CAF_STAT_DECOMP	0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//	Ergibt den alten Wert oder -1 bei einem Fehler.
int CFatFile::SetStats(int iNum,int iSet)
{
FatItem	   *pEntry;
int			iOld;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return -1;
		}

	pEntry			= m_pFiles[iNum];
	iOld			= pEntry->cDecomp;
	pEntry->cDecomp = iSet;
	

return iOld;	
}


//*****************************************************************************
//*
//*		SetFail
//*
//*****************************************************************************
//	Setzt das Defekt-Flag und ergibt den alten Wert.
int CFatFile::SetFail(int iNum,int iFail)
{
int		iOld;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

	iOld = m_pFiles[iNum]->cError;
	m_pFiles[iNum]->cError = iFail;

return iOld;
}

//*****************************************************************************
//*
//*		IsFail
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit iNum beschdigt ist sonst 0
int CFatFile::IsFail(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

return m_pFiles[iNum]->cError;
}

//*****************************************************************************
//*
//*		IsDir
//*
//*****************************************************************************
// Ergibt die 1 wenn die Datei mit iNum ein Verzeichnis ist sonst 0
int CFatFile::IsDir(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->cDir;
}

//*****************************************************************************
//*
//*		IsOk
//*
//*****************************************************************************
// Ergibt:	2	Wenn die Datei vollstntig Entzipt wurde
//          1	Wenn die Datei schon vorhanden war
//			0   Bei Fehlern
int CFatFile::IsOk(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->cDecomp;
}

//*****************************************************************************
//*
//*		IsReadOnly
//*
//*****************************************************************************
//	Ergibt 1 wenn die Datei schreibgeschtzt ist.
int CFatFile::IsReadOnly(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return (m_pFiles[iNum]->cRdOnly)? 1:0;
}

//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
// Ergibt die Zeit der letzten nderung einer Datei
// bzw.	CAF_ERROR_MISS wenn iNum/Datei nicht gefunden wurde.
time_t CFatFile::Time(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return CAF_ERROR_MISS;

return m_pFiles[iNum]->iTime;
}

time_t CFatFile::Time(int iNum,int Mode)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}


return m_pFiles[iNum]->iTime;
}


//*****************************************************************************
//*
//*		GetUserPtr
//*
//*****************************************************************************
//	User-Pointer abfragen
void *CFatFile::GetUserPtr(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->pUserPtr;
}

//*****************************************************************************
//*
//*		SetUserPtr
//*
//*****************************************************************************
//	User-Pointer einstellen
void CFatFile::SetUserPtr(int iNum,void *pData)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;

	m_pFiles[iNum]->pUserPtr = pData;

}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Hole einen Zeiger auf den Dateinamen.
//	iNum		: Ist die Indexnummer des Namens (-1=Archiv-Datei)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt einen Zeiger auf den Namen oder NULL bei einem Fehler.
zchar *CFatFile::Name(int iNum)
{


	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex			
		{
		return m_pFiles[iNum]->pName+m_uNameOff;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum)								// Datei die gerade eingefgt wird
		{
		return m_pInsertFile+m_uNameOff;
		}


return NULL;
}

//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Hole Dateiname samt Lnge.
//	iNum		: Ist die Indexnummer des Namens (-1=ZIP-File)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt einen Zeiger auf den Namen oder NULL bei einem Fehler.
zchar *CFatFile::Name(int iNum,int &iLen)
{


	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex	
		{
		iLen = m_pFiles[iNum]->uNameLen-m_uNameOff;
		return m_pFiles[iNum]->pName   +m_uNameOff;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		iLen = strzlen(m_cFilePath);
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum && m_pInsertFile)				// Datei die gerade eingefgt wird
		{
		iLen = strzlen(m_pInsertFile)-m_uNameOff;
		return         m_pInsertFile +m_uNameOff;
		}
	
	iLen = 0;


return NULL;
}

//*****************************************************************************
//*
//*		FatCreate
//*
//*****************************************************************************
//	FatFile erzeugen
//	Ergibt 0 wenn ok oder eine Fehlernummer
int	CFatFile::FatCreate()
{
FatEntries *pFat=(FatEntries*)m_pMyFat;
unsigned	aOffsets[4];
BYTE		aBuffer[512];
BYTE	   *pByte;
int			iSize;
int			iSeg;
int			iCyl;
int			iNum;
int			iOff;
int			i;



	for(i=0,iSize=0,iNum=0;i<4;i++)						// Gesamt Gre ermitteln
		{
		if(m_aOptions[i]<= 0)continue;
		if(m_aOptions[i]<256)return CAF_ERROR_PARMS;

		iSize += m_aOptions[i];
		iNum  += 1;
		}

	if(iNum>1 || m_aOptions[4])							// Gre fr MBR addieren
		{
		if(m_aOptions[4]>1)
				iOff = m_aOptions[4];
		else	iOff = m_aOptions[5];
		}
	else{
		iOff = 0;
		}


	if(iNum==0 || iSize<256 || m_aOptions[5]<8 || m_aOptions[6]<8 || m_aOptions[5]>63 || m_aOptions[6]>255)
		{
		return CAF_ERROR_PARMS;
		}

	
	m_uFatSectors = iSize;

	FatSeekSec(iSize);
	EndOfFile();
	MainSeek(0);


	if(m_aOptions[9]>=0)								// Die Datei vorab fllen
		{
		memset(aBuffer,m_aOptions[9],512);

		for(i=0;i<512;i++)
			{
			MainWrite(aBuffer,512);
			}
		
		MainSeek(0);
		}


	iSeg = m_aOptions[5]*m_aOptions[6];


	if(iNum>1 || m_aOptions[4])							// MBR erzeugen
		{
		memcpy(aBuffer,m_aMbrDefault,sizeof(m_aMbrDefault));

		pByte = aBuffer+0x1BE;

		for(i=0;i<4;i++,pByte+=16)						// Alle Partitionen eintragen
			{
			aOffsets[i] = iOff;
			memset(pByte,0,16);

			if(!m_aOptions[i])continue;


			pByte[0] = (m_aOptions[7]==i+1)? 0x80:0x00;
			iCyl	 = iOff/iSeg;

			*(DWORD*)(pByte+ 8) = iOff;					// Anfang der Partition (LBA)
			*(DWORD*)(pByte+12) = m_aOptions[i];		// Gre  der Partition

			if(iCyl)									// Start der Partition
				{
				pByte[1] = 0x00;
				pByte[2] = 0xC1;
				pByte[3] = 0xFF;
				}
			else{
				pByte[1] = (BYTE)((iOff/m_aOptions[5])%m_aOptions[6]);
				pByte[2] = (BYTE)((iOff%m_aOptions[5]+1) | ((iCyl>>2)&0xC0));
				pByte[3] = (BYTE)(iCyl);
				}

			iOff += m_aOptions[i]-1;
			iCyl  = iOff/iSeg;

			if(iCyl>1023)								// Ende der Partition
				{
				pByte[5] = 0xFE;
				pByte[6] = 0xFF;
				pByte[7] = 0xFF;
				}
			else{
				pByte[5] = (BYTE)((iOff/m_aOptions[5])%m_aOptions[6]);
				pByte[6] = (BYTE)((iOff%m_aOptions[5]+1) | ((iCyl>>2)&0xC0));
				pByte[7] = (BYTE)(iCyl);
				}


			iOff++;

							 	    pByte[4] = 0x01;	// FAT12
			if(m_aOptions[i]>= 4086)pByte[4] = 0x0E;	// FAT16
			if(m_aOptions[i]>=65526)pByte[4] = 0x0C;	// FAT32
			}

		MainWrite(aBuffer,512);

		memset(aBuffer,0,512);

		for(i=1;i<m_aOptions[5];i++)					// Leeraum bis zum Bootsektor
			{
			MainWrite(aBuffer,512);
			}

		for(i=0;i<4;i++,pByte+=16)						// Bootsektoren leeren
			{
			if(!m_aOptions[i])continue;
			FatSeekSec(aOffsets[i]);
			MainWrite(aBuffer,512);
			}

		m_iCreateCnt = iNum;
		}
	else{												// Diskette ohne MBR
		FatSeekSec(0);
		m_iCreateCnt = 0;
		memset(aBuffer,0,512);
		MainWrite(aBuffer,512);							// Bootsektor leeren
		}


	if(m_aOptions[9]<0)									// Letzen Sektor speichern
		{
		FatSeekSec(iSize-1);
		MainWrite(aBuffer,512);
		}



return CAF_ERROR_OK;
}



//*****************************************************************************
//*
//*		CreateStringA
//*
//*****************************************************************************
//	Erzeugt einen ASCII bzw. Unicode Text aus dem ANSI-Text
int	CFatFile::CreateStringA(zchar *pBuffer,const unsigned char *pName,int iSize)
{
int			iPos;


	for(iPos=0;iPos<iSize;iPos++)
		{
		pBuffer[iPos] = pName[iPos];
		}


	pBuffer[iSize] = 0;


return iSize;
}

//*****************************************************************************
//*
//*		CreateStringW
//*
//*****************************************************************************
//	Erzeugt einen ASCII bzw. Unicode Text aus dem UNICODE-Text
int	CFatFile::CreateStringW(zchar *pBuffer,const unsigned char *pName,int iSize)
{
unsigned	uChar;
int			iPos;


	iSize /= 2;

	for(iPos=0;iPos<iSize;iPos++)
		{
		uChar	= pName[iPos*2+1];
		uChar  += pName[iPos*2  ]*256;

		#ifndef UNICODE
		if(uChar>255)uChar=255;
		#endif

		pBuffer[iPos] = uChar;
		}


	pBuffer[iSize] = 0;


return iSize;
}

//*****************************************************************************
//*
//*		FatSeekSec
//*
//*****************************************************************************
//	Setzt File-Pointer im FAT-Archiv mittels einer Sektornummer
//	iSector		: Ist die Nummer des Sektors
//	Ergibt aktuellen File-Pointer
inline aint64 CFatFile::FatSeekSec(aint64 iSector,int iMode)
{
return MainSeek(iSector*(aint64)512,iMode);
}

//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Liste in m_pFiles
//  iMode:	CAF_FLAG_SORT			0x0400	Nach Namen  sortieren
//			CAF_FLAG_SORTLEN	    0x0500	Nach Lngen sortieren
//			CAF_FLAG_SORTPOS	    0x0600	Nach Offset sortieren (intern)
//			CAF_FLAG_SORTEX			0x0700	Nach Namen mit Zahlen Bercksichtigung
int	CFatFile::Sort(int iMode)
{

		iMode&=0x0700;
	if(!iMode)iMode=m_iModeFlags&0x0700;
	iMode |= 0x0400;

	PROGR(CAF_PROG_SORT,-1,0);


	if(!m_pFiles)return 0;

	if(iMode==m_iSortMode)									// Ist die Liste bereits sortiert
		{
		return m_iFilesNum;
		}

	if(m_iFilesNum)
		{
		switch(iMode)
			{
		case    CAF_FLAG_SORTEX:	qsort(m_pFiles,m_iFilesNum,sizeof(FatItem*),CmpEx  );break;
		case    CAF_FLAG_SORTLEN:	qsort(m_pFiles,m_iFilesNum,sizeof(FatItem*),CmpLen );break;
		case    CAF_FLAG_SORTPOS:	qsort(m_pFiles,m_iFilesNum,sizeof(FatItem*),CmpPos );break;
		default:					qsort(m_pFiles,m_iFilesNum,sizeof(FatItem*),CmpName);break;
			}
		}

	m_iModeFlags&=~0x0300;
	m_iModeFlags|= 0x0300&iMode;
	m_iSortMode  = iMode;


return m_iFilesNum;
}

//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet ein FAT-Archiv und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//				CAF_ERROR_OK		Alles ok
//				CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//				CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//				CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//				CAF_ERROR_READ		Bei Lese-Fehlern
//				CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//				CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//				CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CFatFile::ArchOpen(int iLevel)
{
DIR			sDir;
FILINFO		sInfo;
zchar		cName[256];
zchar		cPath[256];
FatEntries *pFat;
FatItem	   *pEntry=0;
DWORD		dwDrive;
int			iScan;
int			iPos;
int			iRet;
int			iNum;
int			iAdd;



	Clear();

	m_iErrorCode	= CAF_ERROR_OK;
	m_iSortMode		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_iDoneMode		= 0;
	m_uFatSectors	= 0;


	if(m_iFileSize==0)										// Leere Datei
		{
		if(m_iOpenMode!=CAF_CREATE)
			{
			return CAF_ERROR_FORM;
			}
		}



	if(!m_pMyFat)											// FAT-Instanz anlegen	
		{
		m_pMyFat	= new FatEntries;
		}


	pFat			= (FatEntries*)m_pMyFat;
	memset(m_pMyFat,0,sizeof(FatEntries));
	pFat->pThis		= this;

	pFat->sFatData.disk_init_proc	= DiskInit;				// Callbacks einstellen
	pFat->sFatData.disk_status_proc	= DiskStatus;
	pFat->sFatData.disk_ioctl_proc	= DiskIoctl;
	pFat->sFatData.disk_write_proc	= DiskWrite;
	pFat->sFatData.disk_read_proc	= DiskRead;


	if(m_iOpenMode==CAF_CREATE)
		{	
		FatCreate();
		}
	else{
		m_uFatSectors = (unsigned)(m_iFileSize/512);
		}



//*****************************************************************************


try	{
	if(m_uFatSectors<32)
		{
		throw CAF_ERROR_FORM;
		}

	
	if(m_iOpenMode==CAF_CREATE && m_iCreateCnt==0)			// Wurde eine einzelne Diskette ohne MBR erzeugt
		{
		ScanFAT(0,m_uFatSectors);
		}
	else{
		ScanMBR();
		}

	if(!m_uFatDrvOk)										// Keine FAT Partitionen gefunden
		{
		throw CAF_ERROR_FORM;
		}


	if(m_iOpenMode==CAF_CREATE)								// Die Partitionen formatieren
		{
		for(dwDrive=0;dwDrive<m_uFatDrives;dwDrive++)
		{
			if(!m_aOptions[dwDrive])continue;

			if(dwDrive<4 && f_mkfs(&pFat->sFatData,(unsigned char)dwDrive,1,m_aOptions[dwDrive]*512)!=FR_OK)
				{
				throw CAF_ERROR_CREATE;
				}
			}
		}


	m_uNameOff	= (m_uFatDrives>1)? 0:2;
	iScan		= 0;
	cPath[0]	= 0;


	for(dwDrive=0;dwDrive<m_uFatDrives && iScan>=0;)
		{
		for(;;)												// Suche das nchste Verzeichnis
			{
			if(iScan>=m_iFilesNum)
				{
				if(dwDrive>=m_uFatDrives)					// Drive-Verzeichnis einfgen
					{
					iScan = -1;
					break;
					}

				if(m_uFatDrives<=1)
					{
					cPath[0] ='0';
					cPath[1] =':';
					cPath[2] = 0 ;
					dwDrive  = 1;
					iAdd     = 2;
					break;
					}

				iPos				= FatNew();
				pEntry				= m_pFiles[iPos];
				pEntry->pName		= new zchar[2];
				pEntry->uNameLen	= 1;
				pEntry->uOff		= iPos;
				pEntry->iTime		= TimeConvert(sInfo.fdate,sInfo.ftime);
				pEntry->iLen		= 0;
				pEntry->cDir		= AM_DIR;
				pEntry->cError		= 0;
				pEntry->cDecomp	= 1;
				pEntry->pName[0]	= (zchar)('0'+dwDrive);
				pEntry->pName[1]	= 0;

				dwDrive++;
				}

				pEntry	= m_pFiles[iScan];
			if(!pEntry->cDir)
				{
				iScan++;
				continue;
				}

			iAdd = pEntry->uNameLen+1;

			if(iAdd>250)									// Ist der Name zu lang
				{
				iScan++;
				continue;
				}

			memcpy(cPath+2,pEntry->pName+1,(iAdd+1)*sizeof(zchar));
			cPath[0] = pEntry->pName[0];
			cPath[1] =':';
			if(iAdd<=2)cPath[2] = 0;
			iScan++;
			break;
			}


		if(iScan<0)break;


		if(f_opendir(&pFat->sFatData,&sDir,cPath)!=FR_OK)
			{
			continue;
			}


		cPath[1] = '/';


		for(;;)												// Verzeichnis
			{
			sInfo.lname  = cName;
			sInfo.lfsize = 256;

			   iRet = f_readdir(&pFat->sFatData,&sDir,&sInfo);
			if(iRet || !sInfo.fname[0])break;

			   iNum = (int)strzlen(cName);
			if(iNum==0)
				{
				for(;sInfo.fname[iNum];iNum++)
					{
					cName[iNum] = sInfo.fname[iNum];
					}

				cName[iNum] = 0;
				}

			if(iAdd+iNum>255)continue;

			iPos				= FatNew();					// Listen Eintrag erzeugen
			pEntry				= m_pFiles[iPos];
			pEntry->pName		= new zchar[iNum+iAdd+1];
			pEntry->uNameLen	= iAdd+iNum;
			pEntry->uOff		= iPos;
			pEntry->iTime		= 0;
			pEntry->iLen		= sInfo.fsize;
			pEntry->cDir		= sInfo.fattrib&AM_DIR;
			pEntry->cRdOnly		= sInfo.fattrib&AM_RDO;
			pEntry->cError		= 0;
			pEntry->cDecomp	= 1;

			memcpy(pEntry->pName+1   ,cPath+2,(iAdd-2)*sizeof(zchar));
			memcpy(pEntry->pName+iAdd,cName  ,(iNum+1)*sizeof(zchar));
			pEntry->pName[0     ] = cPath[0];
			pEntry->pName[iAdd-1] = '/';


			if(m_iFilesNum>100000)break;
			}


		if(m_iFilesNum>100000)break;
		}
	}
catch(int iError)											// Bekannter Fehler
	{
	m_iErrorCode=iError;
	PROGN(CAF_PROG_ERROR,-1);
	Break();
	return iError;
	}

	if(m_iModeFlags&CAF_FLAG_SORT)							// Sortieren ?
		{
		Sort();				
		}

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow	 = m_iDoneMax;



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einer Datei in der Dateiliste
//	pName	: Ist der  Dateiname nach dem gesucht wird
//	Ergibt die Indexnummer der Datei, bzw. -1 wenn die Datei nicht gefunden wurde.
int CFatFile::Find(const zchar *pName)
{
zchar	 cTemp[256];
FatItem	*pEntry;
unsigned uLen;
int		 iPos;



	if(m_uFatDrives<=1 && (pName[0]<'0' || pName[0]>'9' || (pName[1]!='\\' && pName[1]!='/')))
		{
		cTemp[0] = '0';
		cTemp[1] = '/';
		uLen	 = strlcpy(cTemp+2,pName,ENTRIES(cTemp)-2)+2;
		pName 	 = cTemp;
		}
	else{
		if(*pName=='/' )pName++;
		if(*pName=='\\')pName++;

		uLen		= (int)strzlen(pName);
		}


	for(iPos=0;iPos<m_iFilesNum;iPos++)						// Eintrag in Kette
		{
		   pEntry=m_pFiles[iPos];
		if(pEntry->uNameLen==uLen)
			{
			if(!CmpFileNameN(pName,pEntry->pName))
				{
				m_iFindPos = iPos;
				return iPos;
				}
			}
		}


return -1;
}


//*****************************************************************************
//*
//*		FatDel
//*
//*****************************************************************************
//	Listeneintrag lschen
void CFatFile::FatDel(int iNum)
{
FatItem	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;

	pEntry=m_pFiles[iNum];

	if(!pEntry)return;

	m_iFilesNum--;
	m_pFiles[iNum]=0;

	if(iNum<m_iFilesNum)
		{
		memmove(m_pFiles+iNum,m_pFiles+iNum+1,(m_iFilesNum-iNum)*sizeof(m_pFiles[0]));
		}

	if(pEntry->pName)delete pEntry->pName;

	delete pEntry;

	if(m_iFilesNum<=0)
		{
		delete  m_pFiles;
		m_iFilesNum	 = 0;
		m_iFilesMax	 = 0;
		}
}

//*****************************************************************************
//*
//*		FatNew
//*
//*****************************************************************************
//	Listeneintrag erzeugen
#define	ZNEW_ADD	256
int	CFatFile::FatNew()
{
FatItem	**pNew;



	if(!m_pFiles || !m_iFilesMax)							// Neue Kette
		{
		m_pFiles = new FatItem*[ZNEW_ADD];
		memset(m_pFiles,0,sizeof(FatItem*[ZNEW_ADD]));
		m_iFilesMax=ZNEW_ADD;
		}

	if(m_iFilesNum >= m_iFilesMax)							// Kette vergrern
		{
		pNew = new FatItem*[m_iFilesMax+ZNEW_ADD];
		memset(pNew+m_iFilesMax,0,sizeof(FatItem*[ZNEW_ADD]));
		memcpy(pNew,m_pFiles     ,sizeof(FatItem*)*m_iFilesMax);
		delete m_pFiles;

		m_pFiles	= pNew;
		m_iFilesMax+= ZNEW_ADD;
		}


		   m_pFiles[m_iFilesNum] = new    FatItem;
	memset(m_pFiles[m_iFilesNum],0,sizeof(FatItem));
		   m_iFilesNum++;


return m_iFilesNum-1;
}

//*****************************************************************************
//*
//*		Del
//*
//*****************************************************************************
//	Datei aus FatFile lschen
//	Ergibt:		CAF_ERROR_MISS   Wenn Datei nicht gefunden wurde
//				CAF_ERROR_OK     Wenn Datei gelscht wurde
//				CAF_ERROR_WRITE  Wenn Datei keine Schreib erlaubnis hat
int CFatFile::Del(int iNum)
{
zchar			cName[MAX_PATH+256];
FatEntries	   *pFat=(FatEntries*)m_pMyFat;
FatItem	       *pEntry;
int				iRet;



	PROGR(CAF_PROG_ZDEL,iNum,CAF_ERROR_MISS);

	if(m_iOpenMode!=CAF_WRITE)								// Keine Schreiberlaubnis
		{
		m_iErrorCode=CAF_ERROR_WRITE;
		PROGN(CAF_PROG_ERROR,iNum);
		return m_iErrorCode;
		}

	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Falscher Index
		{
		m_iErrorCode=CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,iNum);
		return m_iErrorCode;
		}

	m_iSortMode = 0;
	pEntry		= m_pFiles[iNum];

	if(m_uFatDrives>1)
		{
		memcpy(cName,pEntry->pName,(pEntry->uNameLen+1)*sizeof(zchar));
		cName[1] = ':';
		}
	else{
		memcpy(cName+2,pEntry->pName,(pEntry->uNameLen+1)*sizeof(zchar));
		cName[0] = '0';
		cName[1] = ':';
		}


	   iRet = f_unlink(&pFat->sFatData,cName);				// Datei lschen
	if(iRet!=FR_OK)
		{
		if(m_uFatDrives!=1 || cName[3]!='/' || f_unlink(&pFat->sFatData,cName+4)!=FR_OK)
			{
			m_iErrorCode = CAF_ERROR_FWRITE;
			return CAF_ERROR_FWRITE;
			}
		}


	FatDel(iNum);
	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow   = m_iDoneMax;



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
//	Bearbeitung abbrechen ohne Daten zu speichern
void CFatFile::Break()
{

	ReleaseData();
	m_iOpenMode=CAF_CLOSED;
	Clear();

}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das FatFile und Speichert alle Daten
//	Ergibt;	CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CFatFile::Close()
{
FatEntries *pFat=(FatEntries*)m_pMyFat;



	m_iErrorCode = CAF_ERROR_OK;

	if(m_iOpenMode==CAF_CLOSED)
		{
		m_iOpenMode  = CAF_CLOSED;
		m_iErrorCode = CAF_ERROR_MISS;
		
		PROGN(CAF_PROG_ERROR,-1);

		return     CAF_ERROR_MISS;
		}


	ReleaseData();
	m_iOpenMode  = CAF_CLOSED;
	m_uFatDrives = 0;


	if(pFat)
		{
		pFat->uDriveCount = 0;
		}

	Clear();
	DefaultOptions();



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		DefaultOptions
//*
//*****************************************************************************
// Standart-Optionen einstellen
void CFatFile::DefaultOptions()
{

	m_aOptions[0]	= 1440*2;					// Gre in Sektoren
	m_aOptions[1]	= 0;						//		*
	m_aOptions[2]	= 0;						//		*
	m_aOptions[3]	= 0;						//		*
	m_aOptions[4]	= 0;						// immer MBR erzeugen
	m_aOptions[5]	= 63;						// Sektoren pro Kopf
	m_aOptions[6]	= 255;						// Kpfe je Zylinder
	m_aOptions[7]	= 0;						// Boot Partition (0=keine)
	m_aOptions[8]	= 0;						// Standartlaufwerk beim Einfgen
	m_aOptions[9]	=-1;						// Die Date mit Bytes fllen

}

//*****************************************************************************
//*
//*		TimeConvert	   DOSTIME --> SYSTEMTIME
//*
//*****************************************************************************
time_t CFatFile::TimeConvert(WORD wDate,WORD wTime)
{
struct tm	sTm;
time_t		iTime;


	sTm.tm_sec  = (wTime<< 1)&0x3F;
	sTm.tm_min  = (wTime>> 5)&0x3F;
	sTm.tm_hour = (wTime>>11);
	sTm.tm_mday = (wDate    )&0x1F;
	sTm.tm_mon  = (wDate>> 5)&0x0F - 1;
	sTm.tm_year = (wDate>> 9)+1980 - 1900;

	iTime = mktime(&sTm);
	if(iTime<0)iTime=0;


return iTime;
}

//*****************************************************************************
//*
//*		TimeConvert	   SYSTEMTIME --> DOSTIME
//*
//*****************************************************************************
//	Eine UTC-Zeit in eine DOS-Zeit umwandeln.
void CFatFile::TimeConvert(time_t iUtcTime,unsigned short &uDate,unsigned short &uTime)
{
time_t	   iEven;
struct tm *pData;


	iEven = (iUtcTime + 1)&(~1);							// Gerade Anzahl an Sekunden
	pData = localtime(&iEven);								// Unix-Zeit von MSDOS

	if(pData->tm_year<80)
		{
		uDate = (1<<5)|1;									// 00:00:00 1.1.1980
		uTime = 0;
		return;
		}

	uDate = (((unsigned)pData->tm_year -   80) <<  9) |
	        (((unsigned)pData->tm_mon  +    1) <<  5) |
		    (((unsigned)pData->tm_mday	 	 )      );

	uTime =  ((unsigned)pData->tm_hour         << 11) |
		     ((unsigned)pData->tm_min          <<  5) |
		     ((unsigned)pData->tm_sec          >>  1);
}

//*****************************************************************************
//*
//*		Flush
//*
//*****************************************************************************
//	Speichert alle Dateien ins FatFile ohne CEN und END-Blcke
//	Ergibt;	CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Bei schreibe Problemen
//			... siehe Error Codes
int CFatFile::Flush()
{


	m_iErrorCode = CAF_ERROR_OK;


return m_iErrorCode;
}


//*****************************************************************************
//*
//*		ExtractProc
//*
//*****************************************************************************
//	Extrahiert eine Datei aus dem Archiv ber eine Ausgabefunktion
//	iNum	: Ist die Indexnummer der Datei die geprft werden soll
//	pParam	: Ist ein Parameter der an die Ausgabefunktionbergeben wird
//	pProc	: Ist der Zeiger auf die Ausgabefunktion
//	pInfo	: Hier werden Zeiten und Attribute der Datei gespeichert.
//			  Bei NULL werden diese Werte ignoriert.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_OK		= Wenn alles korrekt war
//				CAF_ERROR_MISS		= Wenn die Datei nicht gefunden wurde
//				CAF_ERROR_PATH		= Pfad ungltig ist.
//				CAF_ERROR_FCREATE	= Wenn Datei nicht erzeugt werden konnte
//				CAF_ERROR_CRC		= Crc32 Fehler
//				CAF_ERROR_DECOMP	= Fehler beim Entkomprimieren
//				CAF_ERROR_IGNORE	= Die Datei wurde bersprungen
//				... siehe Fehler Codes
int	CFatFile::ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo)
{
FatEntries	   *pFat=(FatEntries*)m_pMyFat;
char			aBuffer[512];
zchar			cName[MAX_PATH+256];
FIL				sFile;
FatItem		   *pEntry;
aint64			iSize;
int				iAttr;
int				iCnt;
int				iRet;


	if(m_iDoneMode<=1)										// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}


	m_iErrorCode = CAF_ERROR_OK;


	try	{
		if( m_iOpenMode==CAF_CLOSED             ){m_iErrorCode=CAF_ERROR_MISS;goto Exit;}
		if((unsigned)iNum>=(unsigned)m_iFilesNum){m_iErrorCode=CAF_ERROR_MISS;goto Exit;}

			pEntry=m_pFiles[iNum];
		if(!pEntry)throw CAF_ERROR_MISS;

		if(m_iDoneMode==1)									// Verlaufsanzeige starten
			{
			m_iDoneNow   = 0;
			m_iDoneMax   = pEntry->iLen;
			}

		m_pErrorFile = pEntry->pName;

		if(pEntry->cError)throw CAF_ERROR_READ;
		if(pEntry->cDir  )goto Stop;


		if(m_uFatDrives>1)									// Nur eine Partion (kein "0:...")
			{
			memcpy(cName,pEntry->pName,(pEntry->uNameLen+1)*sizeof(zchar));
			cName[1] = ':';
			}
		else{
			memcpy(cName+2,pEntry->pName,(pEntry->uNameLen+1)*sizeof(zchar));
			cName[0] = '0';
			cName[1] = ':';
			}												
															// Die Datei ffnen
		if(f_open(&pFat->sFatData,&sFile,cName,FA_READ|FA_OPEN_EXISTING)!=FR_OK)
			{
			if(m_uFatDrives!=1 || cName[3]!='/' || f_open(&pFat->sFatData,&sFile,cName+4,FA_READ|FA_OPEN_EXISTING)!=FR_OK)
				{
				m_iErrorCode = CAF_ERROR_FOPEN;
				PROGN(CAF_PROG_ERROR,iNum);
				goto Exit;
				}
			}
															// Die Datei speichern
		for(m_iReadPos=0;m_iReadPos<pEntry->iLen;m_iReadPos+=iSize)
			{
			   iSize = pEntry->iLen-m_iReadPos;
			if(iSize>512)iSize=512;

			iRet = f_read(&pFat->sFatData,&sFile,aBuffer,(int)iSize,(UINT*)&iCnt);

			if(iCnt!=iSize || iRet!=FR_OK)
				{
				f_close(&pFat->sFatData,&sFile);
				m_iErrorCode = CAF_ERROR_READ;
				PROGN(CAF_PROG_ERROR,iNum);
				goto Exit;
				}

			if(pProc(pParam,aBuffer,(int)iSize)!=iSize)
				{
				f_close(&pFat->sFatData,&sFile);
				m_iErrorCode = CAF_ERROR_FWRITE;
				PROGN(CAF_PROG_ERROR,iNum);
				goto Exit;
				}

			m_iDoneNow += iSize;
			}

		f_close(&pFat->sFatData,&sFile);

Stop:

		if(pInfo)											// Dateidaten speichern
			{
			iAttr				= (pEntry->cRdOnly)? SYS_READ:SYS_WRITE|SYS_READ;
			pInfo->dwAttr		= (pEntry->cDir   )? SYS_DIR:iAttr;
			pInfo->dwSize		= (unsigned)pEntry->iLen;
			pInfo->dwTimeWrite	= 0;
			pInfo->dwTimeCreate	= 0;
			pInfo->dwTimeAccess	= 0;
			pInfo->cName[0]		= 0;
			}

		if(pProc(pParam,NULL,0)!=0)							// Ende anzeigen
			{
			m_iErrorCode = CAF_ERROR_FWRITE;
			PROGN(CAF_PROG_ERROR,iNum);
			goto Exit;
			}
		}
	catch(int iError)
		{
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}

Exit:

	if(m_iDoneMode<=1)										// Die Verlaufsanzeige zurcksetzen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}



return m_iErrorCode;
}

//*****************************************************************************
//*
//*		AddPath
//*
//*****************************************************************************
//	Fgt der FAT-Datei einen Pfad hinzu. Der Pfad muss dabei nicht real existieren
//	ber die Flags() kann mann die Speicherungs Art festlegen
//	Falgs:	CAF_FLAG_FULLPATH	0x0880	 Ganze Namen mit  Laufwerken
//	       	CAF_FLAG_FULLNAME	0x0800	 Ganze Namen ohne Laufwerken
//	pFileName	: ist der Dateiname fr den pPathName
//	Ergibt CAF_ERROR_OK wenn alles ok war bzw. einen Fehlercode
int CFatFile::AddPath(const zchar *pName)
{
FatEntries	   *pFat=(FatEntries*)m_pMyFat;
zchar			cName[512];
zchar		   *pTemp;	
FatItem		   *pEntry;
unsigned		uDrive;
time_t			iTime;
int				iPos;
int				iNum;
int				iLen;
int				iRet;


	m_iDoneNow		= 0;
	m_iDoneMax		= 0;

	if(m_iOpenMode<CAF_WRITE)								// Keine Schreiberlaubnis
		{
		m_iErrorCode=CAF_ERROR_MODE;
		PROGR(CAF_PROG_ERROR,-1,-1);
		return CAF_ERROR_MODE;
		}


	iTime		= time(0);
	m_iDoneMode = 1;
	m_iDoneNow  = 0;
	m_iDoneMax  = 0;


	if(pName[0]>='A' && pName[1]==':')						// "C:\\" entferenen
		{
		pName += 2;
		}

	if(*pName=='/' || *pName=='\\')							// Slash am Begin berspringen
		{
		pName += 1;
		}



	if(pName[0]>='0' && pName[0]<(zchar)m_uFatDrives+'0' && // Ist eine Laufwerksangabe im Namen
	  (pName[1]=='/' || pName[1]=='\\' || pName[1]==':'))
		{
		uDrive = pName[0]-'0';
		}
	else{
		uDrive = (m_uFatDrives)? (unsigned)m_aOptions[8]%m_uFatDrives:0;
		}

	if(pName[0]=='0'+(zchar)uDrive)							// Laufwerksangabe entfernen
	if(pName[1]=='/' || pName[1]=='\\' || pName[1]==':')
		{
		if(pName[1]==':')
				pName += 2;
		else	pName += 1;

		if(*pName=='/' || *pName=='\\')
			{
			pName += 1;
			}
		}

	iLen	 = (int)strzlen(pName);

	if(iLen>256)											// Dateiname zulang
		{
		m_iErrorCode = CAF_ERROR_PARMS;
		m_iDoneNow   = m_iDoneMax;
		return CAF_ERROR_PARMS;
		}

	cName[0] ='0'+uDrive;
	cName[1] =':';
	cName[2] ='/';

	memcpy(cName+3,pName,(iLen+1)*sizeof(zchar));
	CharToPath(cName+3);


	if(m_iModeFlags&CAF_FLAG_NOFIND)						// Nicht prfen ob die Datei im Archiv ist
		{
		iNum = -1;

		if((unsigned)m_iFindPos<(unsigned)m_iFilesNum)		// Letzte Position versuchen
			{
			pTemp = Name(m_iFindPos);
			if(!strzcmp(pTemp,cName))iNum=m_iFindPos;
			}
		}
	else{
		iNum = Find(cName);
		}


	if(iNum>=0 && (m_iModeFlags&CAF_FLAG_NOEXIST))
		{
		m_iErrorCode = CAF_ERROR_EXIST;
		return CAF_ERROR_OK;
		}


	if(iNum<0)
		{
		iNum = FatNew();
		}


	pEntry	= m_pFiles[iNum];


	for(iPos=3;cName[iPos];iPos++)							// Pfad erzeugen
		{
		if(cName[iPos]!='/')continue;
		cName[iPos] = 0;
		iRet=f_mkdir(&pFat->sFatData,cName);
		cName[iPos] = '/';
		}

       iRet =f_mkdir(&pFat->sFatData,cName);
	if(iRet!=FR_OK)
		{
		FatDel(iNum);
		m_iErrorCode = CAF_ERROR_FCREATE;
		m_iDoneNow   = m_iDoneMax;
		PROGN(CAF_PROG_ERROR,-1);
		return CAF_ERROR_FCREATE;
		}


	m_iErrorCode		= CAF_ERROR_OK;
	m_iSortMode			= 0;
	pEntry->iTime		= iTime;
	pEntry->iLen		= 0;
	pEntry->cDir		= 1;
	pEntry->uNameLen	= iLen+2;							// Namen kopieren
	pEntry->pName		= new zchar[iLen+3];

	memcpy(pEntry->pName+1,cName+2,(iLen+2)*sizeof(zchar));
	pEntry->pName[0] = cName[0];

	PROGN(CAF_PROG_WRITE,iNum);

	if(m_iModeFlags&CAF_FLAG_SORT)Sort();					// Sortieren ?



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		AddProc
//*
//*****************************************************************************
//	Fgt der FAT-Datei eine Datei hinzu, die Daten wird ber eine Funktion geladen
//	ber die Flags() kann mann die Speicherungs Art festlegen
//	Falgs:	CAF_FLAG_FULLPATH	0x0880	 Ganze Namen mit  Laufwerken
//	       	CAF_FLAG_FULLNAME	0x0800	 Ganze Namen ohne Laufwerken
//	pFileName	: Ist der Dateiname fr die Datei
//	pParam		: Ist der Zeiger der an die Funktion bergeben wird
//	pProc		: Ist der Zeiger auf die Funktion
//	Ergibt CAF_ERROR_OK wenn alles ok war bzw. einen Fehlercode
int CFatFile::AddProc(const zchar *pName,void *pParam,InpProc pProc,FileData *pInfo)
{
FatEntries	   *pFat=(FatEntries*)m_pMyFat;
zchar			cName[4096];
zchar		   *pTemp;	
FIL				sFile;
FatItem		   *pEntry;
unsigned		uDrive;
UINT			uBytes;
time_t			iTime;
int				iPos;
int				iNew;
int				iCnt;
int				iSet;
int				iNum;
int				iLen;
int				iRet;


	m_iDoneNow		= 0;
	m_iDoneMax		= 0;

	if(m_iOpenMode<CAF_WRITE)								// Keine Schreiberlaubnis
		{
		m_iErrorCode=CAF_ERROR_MODE;
		PROGR(CAF_PROG_ERROR,-1,-1);
		return CAF_ERROR_MODE;
		}


	m_iDoneMode = 1;
	m_iDoneNow  = 0;
	m_iDoneMax  = 0;
	

	if(pName[0]>='A' && pName[1]==':')						// "C:\\" entferenen
		{
		pName += 2;
		}

	if(*pName=='/' || *pName=='\\')							// Slash am Begin berspringen
		{
		pName += 1;
		}


	if(pName[0]>='0' && pName[0]<(zchar)m_uFatDrives+'0' && // Ist eine Laufwerksangabe im Namen
	  (pName[1]=='/' || pName[1]=='\\' || pName[1]==':'))
		{
		uDrive = pName[0]-'0';
		}
	else{
		uDrive = (m_uFatDrives)? (unsigned)m_aOptions[8]%m_uFatDrives:0;
		}

	if(pName[0]=='0'+(zchar)uDrive)							// Laufwerksangabe entfernen
	if(pName[1]=='/' || pName[1]=='\\' || pName[1]==':')
		{
		if(pName[1]==':')
				pName += 2;
		else	pName += 1;

		if(*pName=='/' || *pName=='\\')
			{
			pName += 1;
			}
		}

	iLen = (int)strzlen(pName);

	if(iLen>256)											// Dateiname zulang
		{
		m_iErrorCode = CAF_ERROR_PARMS;
		m_iDoneNow   = m_iDoneMax;
		return CAF_ERROR_PARMS;
		}

	cName[0] ='0'+uDrive;
	cName[1] =':';
	cName[2] ='/';

	memcpy(cName+3,pName,(iLen+1)*sizeof(zchar));
	CharToPath(cName+3);


	if(m_iModeFlags&CAF_FLAG_NOFIND)						// Nicht prfen ob die Datei im Archiv ist
		{
		iNum = -1;

		if((unsigned)m_iFindPos<(unsigned)m_iFilesNum)		// Letzte Position versuchen
			{
			pTemp = Name(m_iFindPos);
			if(!strzcmp(pTemp,cName))iNum=m_iFindPos;
			}
		}
	else{
		iNum = Find(cName);
		}



	if(pInfo)												// Gibt es Infos zur neuen Datei
		{
		if(pInfo->dwTimeWrite!=0  && pInfo->dwTimeWrite !=0xFFFFFFFF)
			{
			iTime = pInfo->dwTimeWrite;
			iSet  = 1;
			}
		else{
			iTime = time(0);		
			iSet  = 0;
			}
		}
	else{
		iTime	= time(0);									// Aktuelle Zeit benutzen
		iSet	= 0;		
		}


	if(iNum>=0)												// Ist die Datei schon vorhanden
		{
		iNew		 = 0;
		pEntry		 = m_pFiles[iNum];
		m_pErrorFile = pEntry->pName;

		if(m_iModeFlags&CAF_FLAG_NOEXIST)					// Keine Dateien im Archiv ersetzen
			{
			m_iErrorCode = CAF_ERROR_EXIST;
			m_iDoneNow   = m_iDoneMax;
			return CAF_ERROR_EXIST;
			}
		}													// Neuen Eintrag erzeugen
	else{
		iNew	= 1;
		iNum	= FatNew();
		pEntry	= m_pFiles[iNum];
		}

															// FAT-Datei ffnen
	   iRet = f_open(&pFat->sFatData,&sFile,cName,FA_WRITE|FA_CREATE_ALWAYS);
	if(iRet==FR_NO_PATH)
		{
		for(iPos=3;cName[iPos];iPos++)						// Pfad erzeugen
			{
			if(cName[iPos]!='/')continue;
			cName[iPos] = 0;
			f_mkdir(&pFat->sFatData,cName);
			cName[iPos] = '/';
			}

		iRet = f_open(&pFat->sFatData,&sFile,cName,FA_WRITE|FA_CREATE_ALWAYS);
		}

	if(iRet!=FR_OK)
		{
		if(iNew)FatDel(iNum);
		m_iErrorCode = CAF_ERROR_FCREATE;
		m_iDoneNow   = m_iDoneMax;
		return CAF_ERROR_FCREATE;
		}


	if(pEntry->pName)										// Alten Namen lschen
		{
		delete pEntry->pName;
		pEntry->pName = 0;
		}


	m_iSortMode			= 0;
	pEntry->iTime		= iTime;
	pEntry->iLen		= 0;
	pEntry->uNameLen	= iLen+2;							// Namen kopieren
	pEntry->pName		= new zchar[iLen+3];

	memcpy(pEntry->pName+1,cName+2,(iLen+2)*sizeof(zchar));
	pEntry->pName[0] = cName[0];


	if(m_pProgress && (m_iProgOn&(1<<CAF_PROG_QUERY)))
		{
		if(Progress(CAF_PROG_QUERY,iNum))
			{
			f_close(&pFat->sFatData,&sFile);
			FatDel(iNum);
			m_iErrorCode = CAF_ERROR_BREAK;
			m_iDoneNow   = m_iDoneMax;
			return CAF_ERROR_BREAK;
			}
		}

	for(;;)													// Datei kopieren
		{
		   iCnt = pProc(pParam,cName,sizeof(cName));
		if(iCnt==0)break;
		if(iCnt <0)
			{
			FatDel(iNum);
			f_close(&pFat->sFatData,&sFile);
			m_iErrorCode = CAF_ERROR_FREAD;
			return CAF_ERROR_FREAD;
			}


		iRet = f_write(&pFat->sFatData,&sFile,cName,iCnt,&uBytes);

		if(iRet!=FR_OK || iCnt!=(int)uBytes)
			{
			f_close(&pFat->sFatData,&sFile);
			FatDel(iNum);
			m_iErrorCode = (iRet==FR_OK)? CAF_ERROR_DISKFULL:CAF_ERROR_FWRITE;;
			m_iDoneNow   = m_iDoneMax;
			return m_iErrorCode;
			}

		pEntry->iLen += iCnt;
		m_iDoneNow   += iCnt;
		}


	f_close(&pFat->sFatData,&sFile);

	if(iSet)												// Zeit einstellen
		{
		FILINFO	sData;

		TimeConvert(iTime,sData.fdate,sData.ftime);
		f_utime(&pFat->sFatData,pEntry->pName,&sData);	
		}

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow   = m_iDoneMax;
	PROGR(CAF_PROG_ZADD,iNum,1);



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Count
//*
//*****************************************************************************
//	Ergibt die Anzahl der Eintrge in der Dateiliste
int CFatFile::Count()
{
	return m_iFilesNum;
}


//*****************************************************************************
//*
//*		SetDoneMode
//*
//*****************************************************************************
//	Den Modus fr die Verlaufsanzeige setzen.
//	iMode	: Ist der neue Modus
//			    0 = Zurck zu Normal (Ende)
//				1 = Begin von Extract (einzelene Datei)
//				2 = Begin von ExtractAll
//	Ergibt den vorherigen Modus.
int CFatFile::SetDoneMode(int iMode)
{
int		iRet;

	
	iRet = m_iDoneMode;
	
	switch(iMode)
		{
	case 0:													// Zurck zu Normal (Ende)

		if(m_iDoneMode==1 || m_iDoneMode==2)
		if(m_iErrorCode==CAF_ERROR_OK)
			{
			m_iDoneNow  = m_iDoneMax;
			}

		m_iDoneMode	 = 0;
		break;

	case 1:													// Begin von Extract (einzelene Datei)

		if(iRet==2)return iRet;
		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 0;
		break;

	case 2:													// Begin von ExtractAll

		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 1;
		break;
		}


return iRet;
}



//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CFatFile::Done(aint64 *pMax)
{
aint64	iRet;


	switch(m_iDoneMode)
		{
	case 0:	iRet = m_iDoneNow;				break;			// Aus
	case 1:	iRet =            m_iReadPos;	break;			// Extract-Single
	case 2:	iRet = m_iDoneNow+m_iReadPos;	break;			// Extract-All
	default:iRet = 0;						break;
		}

	if(iRet>m_iDoneMax)iRet=m_iDoneMax;
	if(pMax)*pMax=m_iDoneMax;




return iRet;
}

//*****************************************************************************
//*
//*		ScanMBR
//*
//*****************************************************************************
BOOL CFatFile::ScanMBR()
{
BootRecord32	   *pBoot32;
BootRecord16	   *pBoot16;
MasterBootRecord	sMbr;
DWORD				dwPos;
DWORD				dwMax;
DWORD				dwOff;
DWORD				dwNext[32];
DWORD				dwTemp[32];
int					iAdd;
int					iNum;
int					i;
int					j;




	MainSeek(0);


	if(MainRead(&sMbr,512)!=512)
		{
		return FALSE;
		}

	iNum	  = 1;
	iAdd	  = 0;
	dwNext[0] = 0;
	dwTemp[0] = 0;
	pBoot16   = (BootRecord16*)&sMbr;						// Prfe ob Boot-Sektor statt MBR
	pBoot32   = (BootRecord32*)&sMbr;

	if(pBoot16->wSignature==0xAA55)
		{
		if(!memcmp(pBoot16->cSystemID,"FAT",3))
			{
			return ScanFAT(0,m_uFatSectors);
			}

		if(!memcmp(pBoot32->cSystemID,"FAT",3))
			{
			return ScanFAT(0,m_uFatSectors);
			}
		}


//******************** Untersuche MBR *****************************************


	dwPos = 0;

	for(j=0;j<iNum && j<32;j++)
		{
		dwPos=dwNext[j];

		FatSeekSec(dwPos);

		if(MainRead(&sMbr,512)!=512)
			{
			return FALSE;
			}

		if(sMbr.wSignature!=0xAA55)
			{
			if(dwTemp[j]==0        )continue;
			if(dwTemp[j]==dwNext[j])continue;
			dwNext[j] = dwTemp[j];
			dwTemp[j] = 0;
			j--;
			continue;
			}

		for(i=0;i<4;i++)									// Scanne Tabelleneintrge
			{
			if(!sMbr.sTable[i].dwRelativeSector)continue;
			if(!sMbr.sTable[i].dwSectors       )continue;


			if(sMbr.sTable[i].bOSType==0x01 ||				// FAT Partitionen
			   sMbr.sTable[i].bOSType==0x04 ||
			   sMbr.sTable[i].bOSType==0x06 ||
			   sMbr.sTable[i].bOSType==0x0B ||
			   sMbr.sTable[i].bOSType==0x0C ||
			   sMbr.sTable[i].bOSType==0x0E ||
			   sMbr.sTable[i].bOSType==0x11 ||
			   sMbr.sTable[i].bOSType==0x14 ||
			   sMbr.sTable[i].bOSType==0x16 ||
			   sMbr.sTable[i].bOSType==0x1B ||
			   sMbr.sTable[i].bOSType==0x1C ||
			   sMbr.sTable[i].bOSType==0x1E)
				{
				   dwOff = sMbr.sTable[i].dwRelativeSector+dwPos;
				if(dwOff>=m_uFatSectors)continue;

				   dwMax = sMbr.sTable[i].dwSectors;
				if(dwMax+dwOff>m_uFatSectors)
					{
					dwMax = m_uFatSectors-dwOff;
					}

				if(ScanFAT(dwOff,dwMax))iAdd++;
				continue;
				}

			if(iNum<32)										// Erweiterte Partitionen
			if(sMbr.sTable[i].bOSType==0x05 || sMbr.sTable[i].bOSType==0x0F)
				{
				dwNext[iNum]=dwPos+sMbr.sTable[i].dwRelativeSector;
				dwTemp[iNum]=0;

				if(i>0)										// Alternative Position (BUG in Win ???)
					{
					dwTemp[iNum]  = dwPos;
					dwTemp[iNum] += sMbr.sTable[i-1].dwSectors;
					dwTemp[iNum] += sMbr.sTable[i-1].dwRelativeSector;
					}

				iNum++;
				}
			}
		}

return (iAdd>0);
}

//*****************************************************************************
//*
//*		ScanFAT
//*
//*****************************************************************************
BOOL CFatFile::ScanFAT(unsigned uStartSec,unsigned uMaxSec)
{
FatEntries	*pFat=(FatEntries*)m_pMyFat;



	if(uStartSec>=m_uFatSectors)
		{
		return FALSE;
		}

	if(uStartSec+uMaxSec>m_uFatSectors)
		{
		return FALSE;
		}

	if(pFat->uDriveCount>=_VOLUMES)
		{
		return FALSE;
		}


	pFat->aStartSec[pFat->uDriveCount]	= uStartSec;
	pFat->aCountSec[pFat->uDriveCount]	= uMaxSec;
	pFat->uDriveCount++;
	m_uFatDrives++;

	if(f_mount(&pFat->sFatData,pFat->uDriveCount-1,pFat->aDrives+pFat->uDriveCount-1)!=FR_OK)
		{
		return FALSE;
		}


	m_uFatDrvOk++;


return TRUE;
}


//*****************************************************************************
//*
//*		DiskInit
//*
//*****************************************************************************
BYTE CFatFile::DiskInit(void *pPtr,DWORD uDrive)
{
FatEntries	*pFat=(FatEntries*)pPtr;


	if(uDrive>=pFat->uDriveCount)
		{
		return STA_NOINIT;
		}


return (pFat->pThis->m_iOpenMode<CAF_WRITE)? STA_PROTECT:0;
}

//*****************************************************************************
//*
//*		DiskStatus
//*
//*****************************************************************************
BYTE CFatFile::DiskStatus(void *pPtr,DWORD uDrive)
{
FatEntries	*pFat=(FatEntries*)pPtr;


	if(uDrive>=pFat->uDriveCount)
		{
		return STA_NOINIT;
		}

return (pFat->pThis->m_iOpenMode<CAF_WRITE)? STA_PROTECT:0;
}

//*****************************************************************************
//*
//*		DiskRead
//*
//*****************************************************************************
BYTE CFatFile::DiskRead(void *pPtr,DWORD uDrive,BYTE *pData,DWORD uSecOffset,DWORD uSecCount)
{
FatEntries *pFat=(FatEntries*)pPtr;
int			iBytes;


	if(uDrive>=pFat->uDriveCount)
		{
		return STA_NODISK;
		}


	uSecOffset += pFat->aStartSec[uDrive];
	pFat->pThis->FatSeekSec(uSecOffset);
	iBytes = uSecCount*512;


	if(pFat->pThis->MainRead(pData,iBytes)!=iBytes)
		{
		return RES_ERROR;
		}


return RES_OK;
}

//*****************************************************************************
//*
//*		DiskWrite
//*
//*****************************************************************************
BYTE CFatFile::DiskWrite(void *pPtr,DWORD uDrive,const BYTE *pData,DWORD uSecOffset,DWORD uSecCount)
{
FatEntries *pFat=(FatEntries*)pPtr;
int			iBytes;


	if(uDrive>=pFat->uDriveCount)
		{
		return STA_NODISK;
		}


	uSecOffset += pFat->aStartSec[uDrive];
	pFat->pThis->FatSeekSec(uSecOffset);
	iBytes = uSecCount*512;


	if(pFat->pThis->MainWrite(pData,iBytes)!=iBytes)
		{
		return RES_ERROR;
		}


return RES_OK;
}

//*****************************************************************************
//*
//*		DiskIoctl
//*
//*****************************************************************************
BYTE CFatFile::DiskIoctl(void *pPtr,DWORD uDrive,BYTE ucCommand,void *pData)
{
FatEntries *pFat=(FatEntries*)pPtr;



	if(uDrive>=pFat->uDriveCount)
		{
		return RES_ERROR;
		}


	switch(ucCommand)
		{
	case CTRL_SECTOR_COUNT:	*(DWORD*)pData = (DWORD)pFat->aCountSec[uDrive];
							return RES_OK;
	case CTRL_SECTOR_CNT64:	*(QWORD*)pData = pFat->aCountSec[uDrive];
							return RES_OK;
	case CTRL_SECTOR_SIZE:	*(WORD *)pData = 512;
							return RES_OK;
	case CTRL_BLOCK_SIZE:	*(DWORD*)pData = 1;
							return RES_OK;
	case CTRL_POWER	:		return RES_OK;
	case CTRL_EJECT	:		return RES_OK;
	case CTRL_LOCK	:		return RES_OK;
	case CTRL_SYNC  :		return RES_OK;
		}


return RES_PARERR;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Eine Option einstellen
//	iNum		: Ist die Art der Option
//					0..3 = Gre der Partionen in Sektoren (0=nicht anlegen)
//					4    = immer MBR erzeugen (0=nein 1=ja 2..=Offset)
//					5    = Sektoren je Kopf
//					6    = Kpfe je Zylinder
//					7    = Bootpartition (0=keine)
//					8	 = Standartlaufwerk beim Einfgen
//					9	 = Die Datei fllen (-1=nein 0=mit 0x00 ... 0xFF)
//	iValue		: Ist der neue Wert der Option
//	Ergibt den alten Wert bzw. -1 bei einem Fehler
int CFatFile::SetOption(int iNum,int iValue)
{
int		iOld;


	if((unsigned)iNum>=sizeof(m_aOptions)/sizeof(m_aOptions[0]))
		{
		return -1;
		}

	iOld				= m_aOptions[iNum];
	m_aOptions[iNum]	= iValue;


return iOld;
}

//*****************************************************************************
//*
//*		Change
//*
//*****************************************************************************
//	Schreiberlaubnis de/aktivieren
int CFatFile::Change(int iMode)
{

															// Prfe den Modus
	if(iMode<CAF_READ || iMode>CAF_WRITE || m_iOpenMode<=CAF_CLOSED)				
		{
		return CAF_ERROR_MODE;
		}
	
	if(m_iOpenMode==iMode)									// Gleicher Modus
		{
		return CAF_ERROR_OK;
		}

	m_iDoneMode	 = 0;
	m_iDoneNow	 = 0;
	m_iDoneMax	 = 0;


return DoChange(iMode);
}
