//*****************************************************************************
//*
//*
//*		FatFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
//
//	Klasse fr den Zugriff auf Fat Files:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef		__FATFILE_H__
#define     __FATFILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	"FileFinder.h"
#include	"FileFunctions.h"
#include	"ArchiveFile.h"
#include	"Filter/Crc32.h"
#include	"Codecs/Huffman.h"


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif


//*****************************************************************************
//*
//*		Klasse CFatFile
//*
//*****************************************************************************
class CFatFile : public CArchiveFile
{
protected:

	typedef struct 
	{
	void		   *pUserPtr;					// User-Pointer
	zchar		   *pName;       				// File name in zip file
	unsigned		uNameLen;					// Lnge des Namens in Zeichen
	unsigned		uOff;						// Logische Position
	time_t			iTime;						// Zeit der letzten nderung
	aint64			iLen;						// Lnge der Datei 
	char			cDir;						// Ist der Eintrag ein Verzeichnis
	char			cDel;						// Wurde der Eintrag gelscht
	char			cError;						// Ist die Position ok
	char			cRdOnly;					// Ist die Datei schreibgeschtzt
	char			cDecomp;					// Zustand der Entkomprimierung
	}FatItem;

	void		   *m_pMyFat;					// Zeiger af FAT Daten
	unsigned 		m_uNameOff;					// Offset im den Dateinamen
	unsigned 		m_uFatDrvOk;				// Anzahl der gemounteten Laufwerke (ok)
	unsigned 		m_uFatDrives;				// Anzahl der gemounteten Laufwerke
	unsigned 		m_uFatSectors;				// FatFile Lnge in Sektoren

	FatItem       **m_pFiles;					// Alfabethische Liste
	int				m_iFilesNum;				// Anzahl der Listeneintrge
	int				m_iFilesMax;				// Gesamt Listengre in Eintrgen
	int				m_iCreateCnt;				// Anzahl der erzeugten Partitionen 	

	int				m_iFindPos;					// Gefunden Position
	aint64			m_iReadPos;					// Aktuelle Leseposition

	aint64			m_iDoneNow;					// Aktueller Verlauf
	aint64			m_iDoneMax;					// Maximaler Verlauf
	int 			m_iDoneMode;				// 0=direkt von m_iDoneNow 1=ExtractAll 2=Extraxt 3=Flush
	int				m_aOptions[10];				// 0..3=Gre in Sec 4=immer MBR 5=Sektoren 6=Kpfe 7=Boot 8=default 9=fill
	static BYTE		m_aMbrDefault[512];			// MBR Daten
	


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

					CFatFile();
	virtual		   ~CFatFile();

	virtual	int     AddPath    (const zchar  *pName);	// Ein Verzeichniss einfgen
	virtual	int		AddProc	   (const zchar  *pName,void *Param ,InpProc pProc,FileData *pInfo=0);
	virtual	int		ExtractProc(int iNum           ,void *pParam,OutProc pProc,FileData *pInfo=0);
	virtual	int		Change	   (int iMode=CAF_WRITE);	// Schreiberlaubnis aktivieren

	virtual	int		Flush	   ();						// Datei speichern
	virtual	int		Close  	   ();						// Datei schliesen
	virtual	int	    Count  	   ();						// Anzahl der Dateien
	virtual	void	Break  	   ();						// Bearbeitung abbrechen
	virtual	int		Sort   	   (int iMode=0);			// Sortierte Kette
	virtual	zchar  *Name   	   (int iNum);				// Hole Dateiname
	virtual	zchar  *Name	   (int iNum,int &iLen);	// Hole Dateiname
	virtual	int		Stats	   (int iNum);				// Zustansbits abfragen
	virtual	int     SetFail	   (int iNum,int iFail=1);	// Defekt-Flag setzen
	virtual	int     IsOk   	   (int iNum);				// Richtig endkomprimiert
	virtual	int     IsDir  	   (int iNum);				// Prfe ob Verzeichnis
	virtual	int     IsFail 	   (int iNum);				// Ist die Datei defekt
	virtual	int		IsReadOnly (int iNum);				// Ist eine Datei schreibgeschtzt
	virtual	aint64	Len    	   (int iNum,int iMode=0);	// Hole Dateilnge
	virtual	int		Del    	   (int iNum);				// Datei lschen
	virtual	time_t	Time   	   (int iNum);				// Dateizeit abfragen
	virtual	time_t	Time   	   (int iNum,int iMode);	// Dateizeit abfragen
	virtual	int		Find   	   (const zchar  *pName);	// Datei suchen
	virtual	aint64	Done   	   (aint64 *pMax);			// Verlaufsabfrage
	virtual	void   *GetUserPtr (int iNum            );	// Datenwert auslesen
	virtual	void    SetUserPtr (int iNum,void *pData);	// Datenwert einstellen
	int				SetOption  (int iNum,int  iValue);	// Eine Option einstellen

protected:


	virtual	int		SetStats   (int iNum,int iSet);		// Die Zustand einer Archivdatei setzen
	virtual	int     SetDoneMode(int iMode );			// Modus fr Done setzen
	virtual	int		ArchOpen   (int iLevel=9);			// ffnet die FAT-Datei

	void			Clear	   ();						// Speicher lschen
	int				CreateStringA(zchar *pBuffer,const unsigned char *pName,int iSize);
	int				CreateStringW(zchar *pBuffer,const unsigned char *pName,int iSize);
	int				CheckSize   (unsigned uSector,unsigned aint64 uSize);
	static time_t	TimeConvert (WORD wDate,WORD wTime);
	static void		TimeConvert (time_t iUtcTime,unsigned short &uDate,unsigned short &uTime);

	void			DefaultOptions();					// Standart-Optionen einstellen

	int				FatCreate ();						// Ein neues FAT-Image erzeugen
	int				FatNew    ();						// Listen Eintrag erzeugen
	void			FatDel    (int  iNum);				// Listen Eintrag lschen
	aint64			FatSeekSec(aint64		iSect,int iMode=SEEK_SET);	// FatFile Offset setzen mittels Sektor
	
	BOOL			ScanFAT	  (unsigned uStartSec,unsigned uMaxSec);
	BOOL			ScanMBR   ();


static BYTE _cdecl  DiskInit  (void *pPtr,DWORD uDrive);
static BYTE _cdecl  DiskStatus(void *pPtr,DWORD uDrive);
static BYTE _cdecl  DiskRead  (void *pPtr,DWORD uDrive,      BYTE *pData,DWORD uSecOffset,DWORD uSecCount);
static BYTE _cdecl  DiskWrite (void *pPtr,DWORD uDrive,const BYTE* pData,DWORD uSecOffset,DWORD uSecCount);
static BYTE _cdecl  DiskIoctl (void *pPtr,DWORD uDrive,      BYTE  ucCommand,void *pData);

static int __cdecl 	CmpName   (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpLen    (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpPos    (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpEx     (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion

};

#endif



