//*****************************************************************************
//*
//*
//*		Sha1.c
//*
//*
//*****************************************************************************
/*
 ---------------------------------------------------------------------------
 Copyright (c) 2002, Dr Brian Gladman, Worcester, UK.   All rights reserved.

 LICENSE TERMS

 The free distribution and use of this software in both source and binary
 form is allowed (with or without changes) provided that:

   1. distributions of this source code include the above copyright
      notice, this list of conditions and the following disclaimer;

   2. distributions in binary form include the above copyright
      notice, this list of conditions and the following disclaimer
      in the documentation and/or other associated materials;

   3. the copyright holder's name is not used to endorse products
      built using this software without specific written permission.

 ALTERNATIVELY, provided that this notice is retained in full, this product
 may be distributed under the terms of the GNU General Public License (GPL),
 in which case the provisions of the GPL apply INSTEAD OF those given above.

 DISCLAIMER

 This software is provided 'as is' with no explicit or implied warranties
 in respect of its properties, including, but not limited to, correctness
 and/or fitness for purpose.
 ---------------------------------------------------------------------------
 Issue Date: 01/08/2005

 This is a byte oriented version of SHA1 that operates on arrays of bytes
 stored in memory.
*/

#include	"Sha1.h"
#include	<stdlib.h>
#include	<string.h>


#define 	IS_LITTLE_ENDIAN		1


//*****************************************************************************
//*
//*		Bit Funktionen
//*
//*****************************************************************************

#if defined(_MSC_VER) && (_MSC_VER > 1000 )

#pragma intrinsic(memcpy)

#if defined(_WIN64) || !defined(_M_IX86)

	#include <intrin.h>
	#define		rotl32(x,n)				_rotl((x), (n))
	#define		rotr32(x,n)				_rotr((x), (n))
	#define 	bswap_32(x)				_byteswap_ulong(x);

#else

	#pragma warning( disable :4035 )

	static __inline unsigned int rotl32(unsigned int uBits,int iCount)
	{
	_asm	{
			mov		eax,uBits
			mov		ecx,iCount
			rol		eax,cl
			}
	}

	static __inline unsigned int rotr32(unsigned int uBits,int iCount)
	{
	_asm	{
			mov		eax,uBits
			mov		ecx,iCount
			ror		eax,cl
			}
	}

	static inline unsigned int bswap_dw(unsigned int uValue)
	{

	_asm	{
			mov		eax,uValue
			bswap	eax;				;// Tausche Bytes in EAX
			}

	}

	#define bswap_32(x)			x=bswap_dw(x)


#endif

#else

	#define		rotlFixed(x,n)			(((x)<<(n)) | ((x)>>(32 -(n))))
	#define		rotrFixed(x,n)			(((x)>>(n)) | ((x)<<(32 -(n))))

	#ifndef		bswap_32
	#define		bswap_32(x)				((rotr32((x),24)&0x00ff00ff)|(rotr32((x),8)&0xff00ff00))
	#endif 	

#endif




#if		IS_LITTLE_ENDIAN
#define SWAP_BYTES
#else
#undef  SWAP_BYTES
#endif

#ifdef	SWAP_BYTES
#define bsw_32(p,n)						{ int _i = (n); while(_i--) ((unsigned*)p)[_i] = bswap_32(((unsigned*)p)[_i]); }
#else
#define bsw_32(p,n)
#endif




#define SHA1_MASK   (SHA1_BLOCK_SIZE - 1)


#define ch(x,y,z)       ( (z)^((x)  & ((y) ^  (z))))
#define maj(x,y,z)      (((x)& (y)) | ((z) & ((x)^(y))))
#define parity(x,y,z)   ( (x)^ (y)  ^  (z))


/* Compile 64 bytes of aHash data into SHA1 context. Note    */
/* that this routine assumes that the byte order in the     */
/* pData->aBuffer[] at this point is in such an order that low   */
/* address bytes in the ORIGINAL byte stream will go in     */
/* this buffer to the high end of 32-bit words on BOTH big  */
/* and little endian systems                                */

#ifdef ARRAY
#define q(v,n)  v[n]
#else
#define q(v,n)  v##n
#endif

#define one_cycle(v,a,b,c,d,e,f,k,h)	q(v,e) += rotr32(q(v,a),27) + f(q(v,b),q(v,c),q(v,d)) + k + h;  \
										q(v,b)  = rotr32(q(v,b), 2)

#define five_cycle(v,f,k,i)							\
			one_cycle(v, 0,1,2,3,4, f,k,hf(i  ));   \
			one_cycle(v, 4,0,1,2,3, f,k,hf(i+1));   \
			one_cycle(v, 3,4,0,1,2, f,k,hf(i+2));   \
			one_cycle(v, 2,3,4,0,1, f,k,hf(i+3));   \
			one_cycle(v, 1,2,3,4,0, f,k,hf(i+4))

//*****************************************************************************
//*
//*		Sha1Compile
//*
//*****************************************************************************
extern "C" void Sha1Compile(Sha1Data *pData)
{   
unsigned    *w = pData->aBuffer;

	#ifdef ARRAY
    
	unsigned    v[5];
    memcpy(v,pData->aHash,5*sizeof(unsigned));
	
	#else
    
	unsigned    v0, v1, v2, v3, v4;
    v0 = pData->aHash[0]; v1 = pData->aHash[1];
    v2 = pData->aHash[2]; v3 = pData->aHash[3];
    v4 = pData->aHash[4];

	#endif

	#define hf(i)   w[i]

    five_cycle(v, ch, 0x5a827999,  0);
    five_cycle(v, ch, 0x5a827999,  5);
    five_cycle(v, ch, 0x5a827999, 10);
    one_cycle (v,0,1,2,3,4, ch, 0x5a827999, hf(15));			

	#undef  hf
	#define hf(i)   (w[(i) & 15] = rotl32(						\
		             w[((i) + 13) & 15] ^ w[((i) + 8) & 15]		\
			       ^ w[((i) +  2) & 15] ^ w[(i)  & 15], 1))

    one_cycle(v,4,0,1,2,3, ch, 0x5a827999, hf(16));
    one_cycle(v,3,4,0,1,2, ch, 0x5a827999, hf(17));
    one_cycle(v,2,3,4,0,1, ch, 0x5a827999, hf(18));
    one_cycle(v,1,2,3,4,0, ch, 0x5a827999, hf(19));

    five_cycle(v, parity, 0x6ed9eba1,  20);
    five_cycle(v, parity, 0x6ed9eba1,  25);
    five_cycle(v, parity, 0x6ed9eba1,  30);
    five_cycle(v, parity, 0x6ed9eba1,  35);

    five_cycle(v, maj   , 0x8f1bbcdc,  40);
    five_cycle(v, maj   , 0x8f1bbcdc,  45);
    five_cycle(v, maj   , 0x8f1bbcdc,  50);
    five_cycle(v, maj   , 0x8f1bbcdc,  55);

    five_cycle(v, parity, 0xca62c1d6,  60);
    five_cycle(v, parity, 0xca62c1d6,  65);
    five_cycle(v, parity, 0xca62c1d6,  70);
    five_cycle(v, parity, 0xca62c1d6,  75);

	#ifdef ARRAY
    pData->aHash[0] += v[0]; 
	pData->aHash[1] += v[1];
    pData->aHash[2] += v[2]; 
	pData->aHash[3] += v[3];
    pData->aHash[4] += v[4];
	#else
    pData->aHash[0] += v0; 
	pData->aHash[1] += v1;
    pData->aHash[2] += v2; 
	pData->aHash[3] += v3;
    pData->aHash[4] += v4;
	#endif
}

//*****************************************************************************
//*
//*		Sha1Init
//*
//*****************************************************************************
extern "C" void Sha1Init(Sha1Data *pData)
{

    pData->aCount[0] = 0;
	pData->aCount[1] = 0;
    pData->aHash [0] = 0x67452301;
    pData->aHash [1] = 0xefcdab89;
    pData->aHash [2] = 0x98badcfe;
    pData->aHash [3] = 0x10325476;
    pData->aHash [4] = 0xc3d2e1f0;

}


//*****************************************************************************
//*
//*		Sha1Hash
//*
//*****************************************************************************
//	 SHA1 aHash data in an array of bytes into aHash buffer and 
//	 call the hash_compile function as required.             
extern "C" void Sha1Hash(Sha1Data *pData,const void *pInput, int iSize)
{   
const unsigned char *pPtr;
unsigned			 uPos;
int					 iSpace;

	
	uPos	= (unsigned)(pData->aCount[0] & SHA1_MASK);
	iSpace	= SHA1_BLOCK_SIZE - uPos;
    pPtr	= (unsigned char*)pInput;

	pData->aCount[0] += iSize;

    if(pData->aCount[0]<(unsigned)iSize)
		{
        (pData->aCount[1])++;
		}

    while(iSize>=iSpace)							// tranfer whole blocks if possible  
		{
        memcpy(((unsigned char*)pData->aBuffer)+uPos,pPtr,iSpace);
        
		pPtr    += iSpace; 
		iSize   -= iSpace; 
		uPos     = 0;
		iSpace   = SHA1_BLOCK_SIZE; 

        bsw_32(pData->aBuffer,SHA1_BLOCK_SIZE>>2);
        
		Sha1Compile(pData);
	    }

    memcpy(((unsigned char*)pData->aBuffer)+uPos,pPtr,iSize);

}

//*****************************************************************************
//*
//*		Sha1Exit
//*
//*****************************************************************************
//	SHA1 final padding and digest calculation  
extern "C" void Sha1Exit(Sha1Data *pData,void *pOutput)
{   
unsigned char  *pPtr=(unsigned char*)pOutput;
unsigned		i;


	i = (unsigned)(pData->aCount[0]&SHA1_MASK);

    // put bytes in the buffer in an order in which references to   
    // 32-bit words will put bytes with lower addresses into the    
    // top of 32 bit words on BOTH big and little endian machines   
    
	bsw_32(pData->aBuffer,(i+3)>>2);

    // we now need to mask valid bytes and add the padding which is 
    // a single 1 bit and as many zero bits as necessary. Note that 
    // we can always add the first padding byte here because the    
    // buffer always has at least one empty slot                    

    pData->aBuffer[i >> 2] &= 0xffffff80 << 8 * (~i & 3);
    pData->aBuffer[i >> 2] |= 0x00000080 << 8 * (~i & 3);

    // we need 9 or more empty positions, one for the padding byte  
    // (above) and eight for the length aCount. If there is not     
    // enough iSpace, pad and empty the buffer                       
    
	if(i>SHA1_BLOCK_SIZE-9)
		{
        if(i<60)pData->aBuffer[15]=0;
        Sha1Compile(pData);
        i = 0;
		}
    else{								
        i = (i>>2)+1;						// compute a word index for the empty buffer positions  
		}

    while(i<14)								// and zero pad all but last two positions
		{
        pData->aBuffer[i] = 0;
		i++;
		}

    // the following 32-bit length fields are assembled in the 
    // wrong byte order on little endian machines but this is  
    // corrected later since they are only ever used as 32-bit 
    // word values.                                            
    
	pData->aBuffer[14] = (pData->aCount[1] << 3) | (pData->aCount[0] >> 29);
    pData->aBuffer[15] = (pData->aCount[0] << 3);
    Sha1Compile(pData);

    // extract the aHash value as bytes in case the aHash buffer is 
    // misaligned for 32-bit words                                  
    
	for(i=0;i<SHA1_DIGEST_SIZE;i++)
		{
        pPtr[i] = (unsigned char)(pData->aHash[i>>2] >> (8 * (~i & 3)));
		}
}

//*****************************************************************************
//*
//*		Sha1
//*
//*****************************************************************************
extern "C" void Sha1(void *pOutput,const void *pInput,int iSize)
{   
Sha1Data    sContext;

    Sha1Init(&sContext); 
	Sha1Hash(&sContext,pInput,iSize); 
	Sha1Exit(&sContext,pOutput);

}

