//*****************************************************************************
//*
//*		Sha1.h
//*
//*****************************************************************************
//	
//	Copyright (c) 2002, Dr Brian Gladman, Worcester, UK.   All rights reserved.
//	
//	LICENSE TERMS
//	
//	The free distribution and use of this software in both source and binary
//	form is allowed (with or without changes) provided that:
//	
//	  1. distributions of this source code include the above copyright
//	     notice, this list of conditions and the following disclaimer;
//	
//	  2. distributions in binary form include the above copyright
//	     notice, this list of conditions and the following disclaimer
//	     in the documentation and/or other associated materials;
//	
//	  3. the copyright holder's name is not used to endorse products
//	     built using this software without specific written permission.
//	
//	ALTERNATIVELY, provided that this notice is retained in full, this product
//	may be distributed under the terms of the GNU General Public License (GPL),
//	in which case the provisions of the GPL apply INSTEAD OF those given above.
//	
//	DISCLAIMER
//	
//	This software is provided 'as is' with no explicit or implied warranties
//	in respect of its properties, including, but not limited to, correctness
//	and/or fitness for purpose.
//
#ifndef		__SHA1_H__
#define		__SHA1_H__



#define		SHA1_BLOCK_SIZE  64
#define		SHA1_DIGEST_SIZE 20


#if defined(__cplusplus)
extern "C"{
#endif



typedef struct										// type to hold the SHA256 context  
	{   
	unsigned	aCount [ 2];
    unsigned	aHash  [ 5];
    unsigned	aBuffer[16];
	}Sha1Data;


// Note that these prototypes are the same for both bit and 
// byte oriented implementations. However the length fields 
// are in bytes or bits as appropriate for the version used 
// and bit sequences are input as arrays of bytes in which  
// bit sequences run from the most to the least significant 
// end of each byte                                         

void Sha1Compile(Sha1Data *pData);
void Sha1Init   (Sha1Data *pData);
void Sha1Hash   (Sha1Data *pData,const void *pInput,int iSize);
void Sha1Exit   (Sha1Data *pData,      void *pOutput);
void Sha1       (void   *pOutput,const void *pInput,int iSize);


#if defined(__cplusplus)
}
#endif


#endif
