//*****************************************************************************
//*
//*
//*		Sha256.c
//*
//*
//*****************************************************************************
#include	"Sha256.h"
#define 	SHA256_UNROLL2



//*****************************************************************************
//*
//*		Bits rollen
//*
//*****************************************************************************

#ifdef		_MSC_VER					

#if defined(_WIN64) || !defined(_M_IX86)

	#include <intrin.h>	
	#define		rotlFixed(x,n)				_rotl((x), (n))
	#define		rotrFixed(x,n)				_rotr((x), (n))

#else

	#pragma warning( disable :4035 )

	static __inline unsigned int rotlFixed(unsigned int uBits,int iCount)
	{
	_asm	{
			mov		eax,uBits
			mov		ecx,iCount
			rol		eax,cl
			}
	}

	static __inline unsigned int rotrFixed(unsigned int uBits,int iCount)
	{
	_asm	{
			mov		eax,uBits
			mov		ecx,iCount
			ror		eax,cl
			}
	}

#endif

#else

	#define		rotlFixed(x,n)			(((x)<<(n)) | ((x)>>(32 -(n))))
	#define		rotrFixed(x,n)			(((x)>>(n)) | ((x)<<(32 -(n))))

#endif

//*****************************************************************************
//*
//*		Makros
//*
//*****************************************************************************

#define		S0(x)						(rotrFixed(x, 2) ^ rotrFixed(x,13) ^ rotrFixed(x,22))
#define		S1(x)						(rotrFixed(x, 6) ^ rotrFixed(x,11) ^ rotrFixed(x,25))
#define		s0(x)						(rotrFixed(x, 7) ^ rotrFixed(x,18) ^ (x >> 3))
#define		s1(x)						(rotrFixed(x,17) ^ rotrFixed(x,19) ^ (x >>10))

#define		blk0(i) 					(aArray[i   ]  = pData[i])
#define		blk2(i) 					(aArray[i&15] += s1(aArray[(i-2)&15]) + aArray[(i-7)&15] + s0(aArray[(i-15)&15]))

#define		Ch(x,y,z)					(z^(x&(y^z)))
#define		Maj(x,y,z)					((x&y)|(z&(x|y)))

#define		a(i)						aTemp[(0-(i))&7]
#define		b(i)						aTemp[(1-(i))&7]
#define		c(i)						aTemp[(2-(i))&7]
#define		d(i)						aTemp[(3-(i))&7]
#define		e(i)						aTemp[(4-(i))&7]
#define		f(i)						aTemp[(5-(i))&7]
#define		g(i)						aTemp[(6-(i))&7]
#define		h(i)						aTemp[(7-(i))&7]


#ifdef		SHA256_UNROLL2 			

#define		R(a,b,c,d,e,f,g,h,i,j)		h += S1(e) + Ch(e,f,g) + aCrypt[i+j] + ((j)? blk2(i):blk0(i));	\
										d += h; h += S0(a) + Maj(a, b, c)

#define		RX_8(i,j) 					R(a,b,c,d,e,f,g,h, i  ,j);  \
										R(h,a,b,c,d,e,f,g, i+1,j); 	\
										R(g,h,a,b,c,d,e,f, i+2,j); 	\
										R(f,g,h,a,b,c,d,e, i+3,j); 	\
										R(e,f,g,h,a,b,c,d, i+4,j); 	\
										R(d,e,f,g,h,a,b,c, i+5,j); 	\
										R(c,d,e,f,g,h,a,b, i+6,j); 	\
										R(b,c,d,e,f,g,h,a, i+7,j)

#else

#define		R(i,j)						h(i) += S1(e(i)) + Ch(e(i),f(i),g(i)) + aCrypt[i+j] + ((j)? blk2(i):blk0(i));	\
										d(i) += h(i); h(i) += S0(a(i)) + Maj(a(i), b(i), c(i))

#ifdef		SHA256_UNROLL				

#define		RX_8(i,j) 					R(i+0,j); R(i+1,j); R(i+2,j); R(i+3,j); R(i+4,j); R(i+5,j); R(i+6,j); R(i+7,j);

#endif

#endif

const Sha32uint aCrypt[64] = 
			{
			0x428A2F98, 0x71374491, 0xB5C0FBCF, 0xE9B5DBA5,
			0x3956C25B, 0x59F111F1, 0x923F82A4, 0xAB1C5ED5,
			0xD807AA98, 0x12835B01, 0x243185BE, 0x550C7DC3,
			0x72BE5D74, 0x80DEB1FE, 0x9BDC06A7, 0xC19BF174,
			0xE49B69C1, 0xEFBE4786, 0x0FC19DC6, 0x240CA1CC,
			0x2DE92C6F, 0x4A7484AA, 0x5CB0A9DC, 0x76F988DA,
			0x983E5152, 0xA831C66D, 0xB00327C8, 0xBF597FC7,
			0xC6E00BF3, 0xD5A79147, 0x06CA6351, 0x14292967,
			0x27B70A85, 0x2E1B2138, 0x4D2C6DFC, 0x53380D13,
			0x650A7354, 0x766A0ABB, 0x81C2C92E, 0x92722C85,
			0xA2BFE8A1, 0xA81A664B, 0xC24B8B70, 0xC76C51A3,
			0xD192E819, 0xD6990624, 0xF40E3585, 0x106AA070,
			0x19A4C116, 0x1E376C08, 0x2748774C, 0x34B0BCB5,
			0x391C0CB3, 0x4ED8AA4A, 0x5B9CCA4F, 0x682E6FF3,
			0x748F82EE, 0x78A5636F, 0x84C87814, 0x8CC70208,
			0x90BEFFFA, 0xA4506CEB, 0xBEF9A3F7, 0xC67178F2
			};


//*****************************************************************************
//*
//*		Sha256Transform
//*
//*****************************************************************************
static void Sha256Transform(Sha32uint *pState,const Sha32uint *pData)
{
Sha32uint 	aArray[16];
unsigned	uPos;


	#ifdef SHA256_UNROLL2 			
	
	Sha32uint a,b,c,d,e,f,g,h;
	
	a = pState[0];
	b = pState[1];
	c = pState[2];
	d = pState[3];
	e = pState[4];
	f = pState[5];
	g = pState[6];
	h = pState[7];
	
	#else
	
	Sha32uint aTemp[8];
	
	for(uPos=0;uPos<8;uPos++)
		{
		aTemp[uPos] = pState[uPos];
		}
	
	#endif

	for(uPos=0;uPos<64;uPos+=16)
		{
		#if defined(SHA256_UNROLL) || defined(SHA256_UNROLL2)

		RX_8(0,uPos); 
		RX_8(8,uPos);
		
		#else
		
		unsigned uOff;
		
		for(uOff=0;uOff<16;uOff++) 
			{ 
  			R(uOff,uPos); 
			}
		
		#endif
		}

	#ifdef SHA256_UNROLL2 		
	
	pState[0] += a;
	pState[1] += b;
	pState[2] += c;
	pState[3] += d;
	pState[4] += e;
	pState[5] += f;
	pState[6] += g;
	pState[7] += h;
	
	#else
	
	for(uPos=0;uPos<8;uPos++)
		{
		pState[uPos] += aTemp[uPos];
		}
	
	#endif

	
}

//*****************************************************************************
//*
//*		Sha256WriteByteBlock
//*
//*****************************************************************************
static void Sha256_WriteByteBlock(Sha256Data *pData)
{
Sha32uint	aArray[16];
unsigned	uPos;


	for(uPos=0;uPos<16;uPos++)
		{
		aArray[uPos] =	((Sha32uint)(pData->aBuffer[uPos*4  ])<<24) +
						((Sha32uint)(pData->aBuffer[uPos*4+1])<<16) +
						((Sha32uint)(pData->aBuffer[uPos*4+2])<< 8) +
						((Sha32uint)(pData->aBuffer[uPos*4+3]));
		}

	Sha256Transform(pData->aState,aArray);
}

//*****************************************************************************
//*
//*		Sha256
//*
//*****************************************************************************
//	Eine SHA256 Prftsumme aus einem Speicherbereich berechnen
//	pData	: Zeiger auf die SHA256 Daten
//	pMem	: Ist der Zeiger auf den Speicherbereich
//	iSize	: Ist die Gre des Speicherbereiches in Byte
extern "C" void Sha256(Sha256Data *pData,const void *pMem,int iSize)
{
unsigned char  *pPtr=(unsigned char*)pMem;
unsigned 		uPos;

	
	
	uPos = (unsigned)(pData->uCount&0x3F);

	while(iSize>0)
		{
		pData->aBuffer[uPos] = *pPtr;
		pData->uCount++;
		iSize--;
		uPos++;
		pPtr++;
		
		if(uPos==64)
			{
			uPos = 0;
			Sha256_WriteByteBlock(pData);
			}
		}

}

//*****************************************************************************
//*
//*		Sha256Init
//*
//*****************************************************************************
//	SHA256 Prftsumme Daten initialisieren
//	pData	: Zeiger auf die SHA256 Daten
extern "C" void Sha256Init(Sha256Data *pData)
{

	pData->aState[0] = 0x6A09E667;
	pData->aState[1] = 0xBB67AE85;
	pData->aState[2] = 0x3C6EF372;
	pData->aState[3] = 0xA54FF53A;
	pData->aState[4] = 0x510E527F;
	pData->aState[5] = 0x9B05688C;
	pData->aState[6] = 0x1F83D9AB;
	pData->aState[7] = 0x5BE0CD19;
	pData->uCount	 = 0;

}

//*****************************************************************************
//*
//*		Sha256Exit
//*
//*****************************************************************************
//	Das Ergebnis einer SHA256 Prftsumme holen
//	pData		: Zeiger auf die SHA256 Daten
//	pDigestPtr	: Hier wird das Ergebnis gespeichert (SHA256_DIGEST_SIZE=32)
extern "C" void Sha256Exit(Sha256Data *pData,void *pDigestPtr)
{
unsigned char  *pDigest=(unsigned char*)pDigestPtr; 
Sha64uint		uInBits;
unsigned 		uPos;
unsigned		uOff;


	uPos    = (unsigned)pData->uCount&0x3F;
	uInBits = pData->uCount<<3;
	pData->aBuffer[uPos] = 0x80;
	uPos++;
	
	while(uPos!=64-8)
		{
		   uPos &= 0x3F;
		if(uPos == 0)
			{
			Sha256_WriteByteBlock(pData);
			}

		pData->aBuffer[uPos] = 0;
		uPos++;
		}

	for(uOff=0;uOff<8;uOff++)
		{
		pData->aBuffer[uPos] = (unsigned char)(uInBits >>56);
		uPos     += 1;
		uInBits	<<= 8;
		}

	Sha256_WriteByteBlock(pData);

	for(uOff=0;uOff<8;uOff++)
		{
		*pDigest++ = (unsigned char)(pData->aState[uOff]>>24);
		*pDigest++ = (unsigned char)(pData->aState[uOff]>>16);
		*pDigest++ = (unsigned char)(pData->aState[uOff]>> 8);
		*pDigest++ = (unsigned char)(pData->aState[uOff]);
		}

	Sha256Init(pData);
}
