//*****************************************************************************
//*
//*
//*		IsoFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
//
//	Klasse fr den Zugriff auf Iso Files:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int uLen)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen
//
#include	<stdio.h>
#include	<stddef.h>
#include	<string.h>
#include	<malloc.h>
#include	<sys/stat.h>
#include	"IsoFile.h"

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		(int)wcslen
#define 	strzcmp		wcscmp
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#define 	strzncat	wcsncat
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		(int)strlen
#define 	strzcmp		strcmp
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#define 	strzncat	strncat
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif


#define		A_RONLY    	0x01
#define 	A_HIDDEN   	0x02
#define 	A_SYSTEM   	0x04
#define 	A_LABEL    	0x08
#define 	A_DIR      	0x10
#define 	A_ARCHIVE  	0x20

#define		ISO_NEW_ADD	256

#pragma pack(1)

typedef unsigned aint64 QWORD;

typedef struct
	{
	BYTE	bLength;					// Lnge der Sructur
	BYTE	bExtendedSectors;			// Anzahl der erweiterten Sektoren am Beginn einer Datei
	DWORD	dwLsbSector;				// Startsektor der Datei
	DWORD	dwMsbSector;				// Startsektor der Datei (Motorola Format)
	DWORD	dwLsbFileLength;			// Lnge der Datei
	DWORD	dwMsbFileLength;			// Lnge der Datei
	BYTE	bYear;						// Jahr (+1900)
	BYTE	bMonth;						// Monat (1=Jan)
	BYTE	bDay;						// 1-31
	BYTE	bHour;						// Stunde
	BYTE	bMinute;					// Minute
	BYTE	bSecound;					// Sekunde
	BYTE	bTimezone;					// Zeitzone in 15min Schritten
	BYTE	bFlags;						//
	BYTE	bInterleaveSize;			// Bit: 0=Versteckt 1=Verzeichnis 7=nicht Ende
	BYTE	bInterleaveGAP;				//
	WORD	wLsbVolumeSequenz;			//
	WORD	wMsbVolumeSequenz;			//
	BYTE	bNameLength;				// Lnge des Dateinamens inklusive 0 am Ende
	char	cName[1];					// Dateiname (Rootverzeichnis="")
	}DirectoryRecord;


typedef struct
	{
	BYTE	bVDType;					// 1=primary volume descriptor 2=Secondary 0xFF=Terminator
	char	cVSStdId[5];				// Must be "CD001".
	BYTE	bVSStdVersion;           	// Must be 1.
	BYTE	bVolumeFlags;            	// 0 in primary volume descriptor.
	char	cSystemIdentifier[32];   	// What system this CD-ROM is meant for.
	char	cVolumeIdentifier[32];   	// The volume name.
	char	cReserved2[8];           	// Must be 0's.
	DWORD	dwLsbVolumeSpaceSize;     	// Volume size, least-significant-byte order.
	DWORD	dwMsbVolumeSpaceSize;     	// Volume size, most-significant-byte order.
	char	cEscapeSequences[32];    	// 0's in primary volume descriptor
	WORD	wLsbVolumeSetSize;       	// Number of volumes in volume set (must be 1).
	WORD	wMsbVolumeSetSize;
	WORD	wLsbVolumeSetSequenceNumber;// Which volume in volume set (not used).
	WORD	wMsbVolumeSetSequenceNumber;
	WORD	wLsbLogicalBlockSize;    	// We'll assume 2048 for block size.
	WORD	wMsbLogicalBlockSize;
	DWORD	dwLsbPathTableSize;       	// How many bytes in path table.
	DWORD	dwMsbPathTableSize;
	DWORD	dwLsbPathTable1;          	// Mandatory occurrence.
	DWORD	dwLsbPathTable2;          	// Optional occurrence.
	DWORD	dwMsbPathTable1;          	// Mandatory occurrence.
	DWORD	dwMsbPathTable2;          	// Optional occurrence.
	DirectoryRecord	sRootDirectory; 	// Duplicate root directory entry.
	char	cVolumeSetIdentifier[128];	// Various copyright and  control fields follow.
	char	cPublisherIdentifier[128];
	char	cDataPreparerIdentifier[128];
	char	cApplicationIdentifier[128];
	char	cCopyrightFileIdentifier[37];
	char	cAbstractFileIdentifier[37];
	char	cBibliographicFileIdentifier[37];
	char	cVolumeCreation[17];
	char	cVolumeModification[17];
	char	cVolumeExpiration[17];
	char	cVolumeEffective[17];
	BYTE	bFileStructureStandardVersion;
	char	cReserved4;               	// Must be 0.
	char	cApplicationUse[512];
	char	cFutureStandardization[653];
	}VolumeDescriptor;

typedef struct
	{
	WORD	wTagIdentifier;				// Art des Tags
	WORD	wDescVersion;				// Version des Tags
	BYTE	bChecksum;					// Checksumme des Tags
	BYTE	bReserved;					// Unbenutzt
	WORD	wSerialNumber;				// eriennummer
	WORD	wDescCrc;					// Checksumme fr Descriptor
	WORD	wDescLen;					// Descriptor Lnge
	DWORD	dwLocation;					// Logischer Sektor vom Inhalt
	}EcmaTag;

typedef struct
	{
	DWORD	dwDirectEntries;			// Prior Recorded Number of Direct Entries
	WORD	wStrategyType;				// Strategy Type
	WORD	wStrategyParam;				// Strategy Parameter
	WORD	wMaxNumberOfEntries;		// Maximum Number of Entries
	BYTE	bReserved;					// Reserved
	BYTE	bFileType;					// File Type
	BYTE	aParentIcbLoc[6];			// Parent ICB Location
	WORD	wFlags;						// Flags
	}EcmaIcb;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=2)
	DWORD	dwMainVolumeDescLen;		// Main Volume Descriptor
	DWORD	dwMainVolumeDescLoc;		// Main Volume Descriptor
	DWORD	dwReserveVolumeDescLen;		// Reserve Volume Descriptor
	DWORD	dwReserveVolumeDescLoc;		// Reserve Volume Descriptor
	BYTE	aReserved[480];				// Reserved
	}EcmaAnchorVolumeDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=5)
	DWORD	dwSequenceNumber;			// Volume Descriptor Sequence Number
	WORD	wPartitionFlags;			// Partition Flags
	WORD	wPartitionNumber;			// Partition Number
	BYTE	aPartitionContents[32];		// Partition Contents
	BYTE	aPartitionContentsUse[128];	// Partition Contents Use
	DWORD	dwAccessType;				// Access Type
	DWORD	dwPartitionLoc;				// Partition Starting Location
	DWORD	dwPartitionLen;				// Partition Length
	BYTE	aImplementationId[32];		// Implementation Identifier
	BYTE	aImplementationUse[128];	// Implementation Use
	BYTE	aReserved[356];				// Reserved
	}EcmaPartitionDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=256)
	BYTE	aRecordingTime[12];			// Recording Date and Time
	WORD	wInterchangeLevel;			// Interchange Level
	WORD	wMaxInterchangeLevel;		// Max.Interchange Level
	DWORD	dwCharacterSetList;			// Character Set List
	DWORD	dwMaxCharacterSetList;		// Max. Character Set List
	DWORD	dwFileSetNumber;			// File Set Number
	DWORD	dwFileSetDescNumber;		// File Set Descriptor Number
	BYTE	aVolumeIdentCharSet[64];	// Logical Volume Identifier Character Set
	BYTE	aVolumeIdentifier[128];		// Logical Volume Identifier
	BYTE	aFileSetCharSet[64];		// File Set Character Set
	BYTE	aFileSetIdentifier[32];		// File Set Identifier
	BYTE	aCopyrightIdentifier[32];	// Copyright File Identifier
	BYTE	aAbstractIdentifier[32];	// Root Directory ICB
	BYTE	aRootDirectoryIcb[16];		// Root Directory ICB
	BYTE	aDomainIdentifier[32];		// Domain Identifier
	BYTE	aNextExtentIcb[16];			// Next Extent
	BYTE	aSystemStreamIcb[16];		// System Stream Directory ICB
	BYTE	bReserved[32];				// Reserved
	}EcmaFileSetDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=257)
	WORD	wFileVersionNumber;			// File Version Number
	BYTE	bFileCharacteristics;		// File Characteristics
	BYTE	bLengthOfFileIdentifier;	// Length of File Identifier L_FI
	BYTE	aFileIcb[16];				// File ICB
	WORD	wImplementationUseLen;		// Length of Implementation Use L_IU
	BYTE	aData[1];					// ImplementationUse[L_IU]+FileIdentifier[L_FI]
	}EcmaFileIdentifierDesc;

typedef struct
	{
	EcmaTag	sDescTag;					// Descriptor Tag (Tag=261)
	EcmaIcb	sIcbTag;					// ICB Tag
	DWORD	dwUid;						// Uid
	DWORD	dwGid;						// Gid
	DWORD	dwPermissions;				// Permissions
	WORD	wFileLinkCount;				// File Link Count
	BYTE	bRecordFormat;				// Record Format
	BYTE	bRecordAttributes;			// Record Display Attributes
	DWORD	dwRecordLength;				// Record Length
	QWORD	qwInformationLength;		// Information Length
	QWORD	qwLogicalBlocksRecorded;	// Logical Blocks Recorded
	BYTE	aAccessTime[12];			// Access Date and Time
	BYTE	aModificationTime[12];		// Modification Date and Time
	BYTE	aAttributeTime[12];			// Attribute Date and Time
	DWORD	dwCheckpoint;				// Checkpoint
	BYTE	aExtendedAttributeIcb[16];	// Extended Attribute ICB
	BYTE	aImplementationId[32];		// Implementation Identifier
	QWORD	qwUniqueId;					// Unique Id
	DWORD	dwLengthOfExAttributes;		// Length of Extended Attributes L_EA
	DWORD	dwLengthOfAllocation;		// Length of Allocation Descriptors	L_AD
	BYTE	aData[1];					// ExtendedAttributes[L_EA]+AllocationDescriptors[L_AD]
	}EcmaFileEntry;

typedef struct
	{
	DWORD	dwEntry;					// Nummer des Elternverzeichnisses
	DWORD	dwSector;					// Startsektor des Verzeichnisses
	DWORD	dwSize;						// Gre in Byte
	}UdfDirEntry;

#pragma pack()


#define PROG(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))throw CAF_ERROR_BREAK;	\
	}
#define PROGR(iMode,iNum,iRet)						\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))return iRet;			\
	}
#define PROGN(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	Progress(iMode,iNum);							\
	}




//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CIsoFile::CIsoFile()
{
	m_pFiles		=  0;
	m_iFilesMax		=  0;
	m_iFilesNum		=  0;
	m_iDoneNow		=  0;
	m_iDoneMax		=  0;
	m_iDoneMode		=  0;
	m_iReadPos		=  0;
	m_uSectorShift	=  0;
	m_uSectorSize	=  0;
	m_uSectorOff	=  0;
	m_uSectorEnd	=  0;
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CIsoFile::~CIsoFile()
{
	if(m_iOpenMode!=CAF_CLOSED)Close();
	Clear();
}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Datenfelder und setzt alle Variabeln zurck
void CIsoFile::Clear()
{
int	i;


	for(i=m_iFilesNum-1;i>=0;i--)IsoDel(i);

	m_iFilesNum		= 0;
	m_uSectorShift	= 0;
	m_uSectorSize	= 0;
	m_iModeFlags	= 0;

}

//*****************************************************************************
//*
//*		EcmaTagCrc
//*
//*****************************************************************************
unsigned CIsoFile::EcmaTagCrc(void *pTagPtr)
{
unsigned char  *pTag=(unsigned char*)pTagPtr;
unsigned char 	bCrc;
int				iPos;


	bCrc = pTag[0];

	for(iPos=1;iPos< 4;iPos++)bCrc+=pTag[iPos];
	for(iPos=5;iPos<16;iPos++)bCrc+=pTag[iPos];


return bCrc;
}

//*****************************************************************************
//*
//*		EcmaName
//*
//*****************************************************************************
//	Erstellt einen ECMA String
unsigned CIsoFile::EcmaName(const unsigned char *pQuelle,int iMax,zchar *pOut)
{
static char		cText[512+4];
int				iPos;


	if(iMax>512)iMax=512;

	if(pQuelle[0]==16)
		{
		iMax -= 1;
		iMax /= 2;

		for(iPos=0;iPos<iMax;iPos++)
			{
			#ifdef UNICODE

			pOut[iPos]  = pQuelle[2+iPos*2+1]*256;
			pOut[iPos] += pQuelle[2+iPos*2  ];

			#else

			pOut[iPos]  = (pQuelle[2+iPos*2+1])? 255:pQuelle[2+iPos*2];

			#endif

			if(!pOut[iPos])break;
			}

		pOut[iPos]=0;

		return iPos;
		}

	if(iMax>256)iMax=256;

	if(pQuelle[0]==8)
		{
		if(pQuelle[1])
			{
			iMax--;
			pQuelle++;
			}
		else{
			iMax-=2;
			pQuelle+=2;
			}

		for(iPos=0;iPos<iMax;iPos++)
			{
			pOut[iPos]  = pQuelle[iPos];

			if(!pOut[iPos])break;
			}

		pOut[iPos]=0;

		return iPos;
		}


	pOut[0] = 0;

return 0;
}


//*****************************************************************************
//*
//*		CmpEx
//*
//*****************************************************************************
int __cdecl CIsoFile::CmpEx(const void *pPos1,const void *pPos2)
{
IsoItem	*pPtr1,*pPtr2;

	pPtr1=*(IsoItem**)pPos1;
	pPtr2=*(IsoItem**)pPos2;

return strvicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpName
//*
//*****************************************************************************
int __cdecl CIsoFile::CmpName(const void *pPos1,const void *pPos2)
{
IsoItem	*pPtr1,*pPtr2;

	pPtr1=*(IsoItem**)pPos1;
	pPtr2=*(IsoItem**)pPos2;

return strzicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpLen
//*
//*****************************************************************************
int __cdecl CIsoFile::CmpLen(const void *pPos1,const void *pPos2)
{
IsoItem	*pPtr1,*pPtr2;


	pPtr1=*(IsoItem**)pPos1;
	pPtr2=*(IsoItem**)pPos2;

	if(pPtr1->iLen==pPtr2->iLen)return 0;

return (pPtr1->iLen<pPtr2->iLen)? -1:0;
}

//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CIsoFile::CmpPos(const void *pPos1,const void * pPos2)
{
IsoItem	*pPtr1,*pPtr2;

	pPtr1=*(IsoItem**)pPos1;
	pPtr2=*(IsoItem**)pPos2;

	if(pPtr1->uOff==pPtr2->uOff)return 0;

return (pPtr1->uOff<pPtr2->uOff)? -1:1;
}

//*****************************************************************************
//*
//*		CheckSize
//*
//*****************************************************************************
//	Ergibt 1 wenn die Gre verletzt wurde.
inline int CIsoFile::CheckSize(unsigned uSector,unsigned aint64 uSize)
{
unsigned 	uNum;


	uNum = (unsigned)((uSize+2047)/2048);

	if(uSector     >m_uIsoSectors)return 1;
	if(uSector+uNum>m_uIsoSectors)return 1;
	if(uSector+uNum<uSector		 )return 1;


return 0;
}

//*****************************************************************************
//*
//*		Len
//*
//*****************************************************************************
//	Ergibt die Lnge der Datei mit der Indexnummer iNum
//	Ist iMode==0 wird die originale    Lnge zurckgegeben
//	Ist iMode!=0 wird die komprimierte Lnge zurckgegeben
aint64 CIsoFile::Len(int iNum,int iMode)
{
IsoItem	*pEntry;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;
	pEntry=m_pFiles[iNum];

return pEntry->iLen;
}

//*****************************************************************************
//*
//*		SetFail
//*
//*****************************************************************************
//	Setzt das Defekt-Flag und ergibt den alten Wert.
int CIsoFile::SetFail(int iNum,int iFail)
{
int		iOld;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

	iOld = m_pFiles[iNum]->cError;
	m_pFiles[iNum]->cError = iFail;

return iOld;
}

//*****************************************************************************
//*
//*		IsFail
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit iNum beschdigt ist sonst 0
int CIsoFile::IsFail(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

return m_pFiles[iNum]->cError;
}

//*****************************************************************************
//*
//*		IsDir
//*
//*****************************************************************************
// Ergibt die 1 wenn die Datei mit iNum ein Verzeichnis ist sonst 0
int CIsoFile::IsDir(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->cDir;
}

//*****************************************************************************
//*
//*		IsOk
//*
//*****************************************************************************
// Ergibt:	2	Wenn die Datei vollstntig Entzipt wurde
//          1	Wenn die Datei schon vorhanden war
//			0   Bei Fehlern
int CIsoFile::IsOk(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->cDecomp;
}

//*****************************************************************************
//*
//*		Stats
//*
//*****************************************************************************
//	Die Zustandbits abfragen
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//	iDel		: Welche Bits sollen gelscht werden
//	Ergibt einen Wert mit veschiedenen Bits:
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//					CAF_STAT_DELETE		0x0004	Die Datei wurde aus dem Verzeichnis gelscht
//					CAF_STAT_WRITE		0x0008	Die Datei muss ins Archiv geschrieben werden
//					CAF_STAT_DIR		0x0010	Die Datei ist ein Verzeichniss
//					CAF_STAT_READONLY	0x0020	Die Datei ist schreibgeschtzt
//					CAF_STAT_SAVED		0x0040	Die Datei ist im Archiv gespeichert
int CIsoFile::Stats(int iNum)
{
IsoItem	   *pEntry;
int			iBits;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return 0;
		}

			pEntry=m_pFiles[iNum];
	iBits = pEntry->cDecomp&3;
	iBits|= CAF_STAT_SAVED;

	if(pEntry->cDir)iBits|= CAF_STAT_DIR;
	

return iBits;	
}

//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
// Ergibt die Zeit der letzten nderung einer Datei
// bzw.	CAF_ERROR_MISS wenn iNum/Datei nicht gefunden wurde.
time_t CIsoFile::Time(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return CAF_ERROR_MISS;

return m_pFiles[iNum]->iTime;
}

time_t CIsoFile::Time(int iNum,int Mode)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}


return m_pFiles[iNum]->iTime;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Ergibt den Dateinamen auf iNum bzw. 0 bei einem Fehler
zchar *CIsoFile::Name(int iNum)
{

	if(iNum==-1)return m_cFilePath;
	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;


return m_pFiles[iNum]->pName;
}

//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Hole Dateiname mit Lnge
//	iNum		: Ist die Indexnummer des Namens
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt einen Zeiger auf den Namen oder NULL beieinem Fehler.
zchar *CIsoFile::Name(int iNum,int &iLen)
{


	if(iNum==-1)
		{
		iLen = (m_cFilePath)? strzlen(m_cFilePath):0;
		return  m_cFilePath;
		}

	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		iLen = 0;
		return 0;
		}

	iLen = m_pFiles[iNum]->uNameLen;


return m_pFiles[iNum]->pName;
}



//*****************************************************************************
//*
//*		IsoRead
//*
//*****************************************************************************
// Liest aus IsoFile
// Ergibt -1 bei einem Fehler ansonsten die geladenen Bytes
// Bei einem Fehler kommt es zur Exeption CAF_ERROR_READ
int	CIsoFile::IsoRead(void *pMem,int iSize)
{
DWORD	dwOffset;
aint64	iPos64;
int		iDelta;
int		iNum;
int		iLen;
int		iMax;



	if(iSize<=0)return 0;

	if(m_uSectorSize)										// Spezeiller Modus
		{
		iNum		= 0;
		iPos64		= MainTell();

		if(iPos64<(aint64)m_uSectorShift)
			{
			MainSeek(m_uSectorShift,SEEK_CUR);
			iPos64 = m_uSectorShift;
			}

		dwOffset	= (unsigned)((iPos64-(aint64)m_uSectorShift)%(aint64)m_uSectorSize);

		while(iNum<iSize)
			{
			if(dwOffset<m_uSectorOff)						// Vor Start-Offset
				{
				iDelta = m_uSectorOff-dwOffset;
				MainSeek(iDelta,SEEK_CUR);
				dwOffset = m_uSectorOff;
				}
			else if(dwOffset>=m_uSectorEnd)					// Nach Start-Offset
				{
				iDelta = m_uSectorSize-dwOffset+m_uSectorOff;
				MainSeek(iDelta,SEEK_CUR);
				dwOffset = m_uSectorOff;
				}

			iLen	= 2048-dwOffset+m_uSectorOff;
			iMax	= iSize-iNum;

			if(iLen>iMax)iLen=iMax;

			if(MainRead(pMem,iLen)!=iLen)					// Einen Sektor laden
				{
				return -1;
				}

			dwOffset		 += iLen;
			iNum			 += iLen;
			*((char**)&pMem) += iLen;
			}

		return iSize;
		}


	if(m_uSectorShift)										// Offset am Begin
		{
		iPos64 = MainSeek(0,SEEK_CUR);

		if(iPos64<(aint64)m_uSectorShift)
			{
			MainSeek(m_uSectorShift);
			}
		}

	if(MainRead(pMem,iSize)!=iSize)							// Daten laden
		{
		return -1;
		}


return iSize;
}


//*****************************************************************************
//*
//*		CreateStringA
//*
//*****************************************************************************
//	Erzeugt einen ASCII bzw. Unicode Text aus dem ANSI-Text
int	CIsoFile::CreateStringA(zchar *pBuffer,const unsigned char *pName,int iSize)
{
int			iPos;



	for(iPos=0;iPos<iSize;iPos++)
		{
		pBuffer[iPos] = pName[iPos];
		}


	pBuffer[iSize] = 0;


return iSize;
}

//*****************************************************************************
//*
//*		CreateStringW
//*
//*****************************************************************************
//	Erzeugt einen ASCII bzw. Unicode Text aus dem UNICODE-Text
int	CIsoFile::CreateStringW(zchar *pBuffer,const unsigned char *pName,int iSize)
{
unsigned	uChar;
int			iPos;


	iSize /= 2;

	for(iPos=0;iPos<iSize;iPos++)
		{
		uChar	= pName[iPos*2+1];
		uChar  += pName[iPos*2  ]*256;

		#ifndef UNICODE
		if(uChar>255)uChar=255;
		#endif

		pBuffer[iPos] = uChar;
		}


	pBuffer[iSize] = 0;


return iSize;
}

//*****************************************************************************
//*
//*		IsoSeekSec
//*
//*****************************************************************************
//	Setzt File-Pointer in IsoFile mittels einer Sektornummer
//	Ergibt aktuellen File-Pointer
inline aint64	CIsoFile::IsoSeekSec(int iSect,int iMode)
{
return IsoSeek(((aint64)iSect)*2048,iMode);
}

//*****************************************************************************
//*
//*		IsoSeek
//*
//*****************************************************************************
//	Setzt File-Pointer in IsoFile
//	Ergibt aktuellen File-Pointer
aint64	CIsoFile::IsoSeek(aint64 iPos64,int iMode)
{
aint64	iNow64;
int		iOff;



	if(m_uSectorSize)									// Spezieller Modus
		{
		switch(iMode)
				{
		case SEEK_END:	iNow64  = MainSeek(0,SEEK_END);
						iNow64 -= m_uSectorShift;
						break;

		case SEEK_CUR:	iNow64  = MainSeek(0,SEEK_CUR);
						iNow64 -= m_uSectorShift;
						break;

		default:		iNow64  = 0;
						break;
				}

		if(iNow64<0)iNow64=0;

		   iOff = (int)(iNow64%(aint64)m_uSectorSize);
		   iOff-= m_uSectorOff;
		if(iOff<   0)iOff=0;
		if(iOff>2048)iOff=2048;

		iPos64 += iOff;
		iPos64 += (iNow64/(aint64)m_uSectorSize)*(aint64)2048;

		iNow64  = iPos64;
		iNow64 /= 2048;
		iNow64 *= m_uSectorSize;
		iNow64 += m_uSectorOff;
		iNow64 += iPos64%(aint64)2048;
		iNow64 += m_uSectorShift;

		MainSeek(iNow64);

		return iPos64;
		}


//******************** Linearer Modus *****************************************


	if(iMode==SEEK_SET)
		{
		iPos64 += m_uSectorShift;
		}

	iPos64  = MainSeek(iPos64,iMode);
	iPos64 -= m_uSectorShift;

	if(iPos64<0)
		{
		MainSeek(m_uSectorShift);
		iPos64 = 0;
		}


return iPos64;
}


//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Liste in m_pFiles
//  iMode:	CAF_FLAG_SORT			0x0400	Nach Namen  sortieren
//			CAF_FLAG_SORTLEN	    0x0500	Nach Lngen sortieren
//			CAF_FLAG_SORTPOS	    0x0600	Nach Offset sortieren (intern)
//			CAF_FLAG_SORTEX			0x0700	Nach Namen mit Zahlen Bercksichtigung
int	CIsoFile::Sort(int iMode)
{

		iMode&=0x0700;
	if(!iMode)iMode=m_iModeFlags&0x0700;
	iMode |= 0x0400;

	PROGR(CAF_PROG_SORT,-1,0);


	if(!m_pFiles)return 0;

	if(iMode==m_iSortMode)								// Ist die Liste bereits sortiert
		{
		return m_iFilesNum;
		}

	if(m_iFilesNum)
		{
		switch(iMode)
			{
		case    CAF_FLAG_SORTEX:	qsort(m_pFiles,m_iFilesNum,sizeof(IsoItem*),CmpEx  );break;
		case    CAF_FLAG_SORTLEN:	qsort(m_pFiles,m_iFilesNum,sizeof(IsoItem*),CmpLen );break;
		case    CAF_FLAG_SORTPOS:	qsort(m_pFiles,m_iFilesNum,sizeof(IsoItem*),CmpPos );break;
		default:					qsort(m_pFiles,m_iFilesNum,sizeof(IsoItem*),CmpName);break;
			}
		}

	m_iModeFlags&=~0x0300;
	m_iModeFlags|= 0x0300&iMode;
	m_iSortMode  = iMode;


return m_iFilesNum;
}

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
//	ffnet eine BZIP2-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CIsoFile::ArchOpen(int iLevel)
{
EcmaAnchorVolumeDesc   *pAnch;
VolumeDescriptor	   *pDesk;
IsoItem				   *pEntry=0;
BYTE					aMem[2048*3+4];
int						iFound;
int						iSeek;
int						iPos;
int						iOff;




	Clear();

	m_iSortMode		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_iDoneMode		= 0;
	m_iErrorCode	= CAF_ERROR_OK;

try	{

	if(m_uIsoSectors<32)
		{
		throw CAF_ERROR_FORM;
		}


	iFound	= 0;
	pDesk	= (VolumeDescriptor*)aMem;


	if(IsoRead(aMem,2048)==2048)						// Suche CUE Format
		{
		if(memcmp(aMem,"\x00\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x00",12)==0)
		if(IsoRead(aMem,2048)==2048)
			{
			for(iOff=0;iOff<512;iOff++)
				{
				if(memcmp(aMem+iOff,"\x00\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x00",12))continue;
				m_uSectorSize  = 2048+iOff;
				m_uSectorEnd   = 2048+16;
				m_uSectorOff   = 16;
				m_uSectorShift = 0;
				break;
				}
			}
		}

	IsoSeek(-256,SEEK_END);

	if(IsoRead(aMem,256)==256)							// Suche NRG Format
		{
		if(!memcmp(aMem+256-12,"NER5",4))
			{
			for(iOff=0;iOff<256;iOff++)
				{
				if(memcmp(aMem+iOff,"DAOX",4))continue;
				m_uSectorShift  = aMem[iOff+60]<<24;
				m_uSectorShift |= aMem[iOff+61]<<16;
				m_uSectorShift |= aMem[iOff+62]<<8;
				m_uSectorShift |= aMem[iOff+63];
				break;
				}
			}

		if(!memcmp(aMem+256-8,"NERO",4))
			{
			for(iOff=0;iOff<256;iOff++)
				{
				if(memcmp(aMem+iOff,"DAOI",4))continue;
				m_uSectorShift  = aMem[iOff+56]<<24;
				m_uSectorShift |= aMem[iOff+57]<<16;
				m_uSectorShift |= aMem[iOff+58]<<8;
				m_uSectorShift |= aMem[iOff+59];
				break;
				}
			}
		}


	IsoSeekSec(16);
	iSeek = -1;


	for(iPos=16;iPos<32;iPos++)							// Suche ISO Format
		{
		if(IsoRead(aMem,2048)!=2048)break;
		if(memcmp(pDesk->cVSStdId,"CD001",5))
			{
			if(memcmp(pDesk->cVSStdId,"BEA",3)==0)iSeek=iPos;
			continue;
			}

		if(pDesk->bVDType==0xFF){iPos++;break;}

		if(pDesk->bVDType==1 && iFound==0)
			{
			memcpy(aMem+2048,aMem,2048);
			iFound=1;
			continue;
			}

		if(pDesk->bVDType==2 && iFound<=1)
			{
			memcpy(aMem+2048,aMem,2048);
			iFound=2;
			continue;
			}
		}

	if(iSeek>=0)
		{
		IsoSeekSec(iSeek);
		iPos = iSeek;
		}

	if(IsoRead(aMem,2048)==2048)						// Suche UDF Format
	if(memcmp(pDesk->cVSStdId,"BEA",3)==0)
		{
		for(iPos++;iPos<32;iPos++)
			{
			if(IsoRead(aMem,2048)!=2048)break;
			if(memcmp(pDesk->cVSStdId,"TEA",3)==0)break;
			if(memcmp(pDesk->cVSStdId,"NSR",3)!=0)continue;

			IsoSeekSec(256);
			if(IsoRead(aMem,2048)!=2048)break;

			pAnch = (EcmaAnchorVolumeDesc*)aMem;

			if(pAnch->sDescTag.wTagIdentifier!=2)break;
			if(pAnch->sDescTag.bChecksum!=EcmaTagCrc(pAnch))break;

			memcpy(pDesk+1,aMem,2048);
			iFound = 3;
			break;
			}
		}



	if(!iFound)											// Kein Format gefunden
		{
		throw CAF_ERROR_FORM;
		}


	switch(iFound)
		{
	case 1:	ScanByte(pDesk+1,aMem);	break;
	case 2:	ScanWide(pDesk+1,aMem);	break;
	case 3:	ScanUdf (pDesk+1,aMem);	break;
		}
	}
catch(int iError)										// Bekannter Fehler
	{
	m_iErrorCode=iError;
	PROGN(CAF_PROG_ERROR,-1);
	Break();
	return iError;
	}

	if(m_iModeFlags&CAF_FLAG_SORT)Sort();				// Sortieren ?

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow	 = m_iDoneMax;



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einer Datei in der Dateiliste
//	pName	: Ist der  Dateiname nach dem gesucht wird
//	Ergibt die Indexnummer der Datei, bzw. -1 wenn die Datei nicht gefunden wurde.
int CIsoFile::Find(const zchar *pName)
{
IsoItem	*pEntry;
unsigned uLen;
int		 iPos;


	if(*pName=='/' )pName++;
	if(*pName=='\\')pName++;


	uLen = (int)strzlen(pName);

	for(iPos=0;iPos<m_iFilesNum;iPos++)								// Eintrag in Kette
		{
		   pEntry=m_pFiles[iPos];
		if(pEntry->uNameLen!=uLen)continue;

		if(!CmpFileNameN(pName,pEntry->pName))
			{
			return iPos;
			}
		}


return -1;
}

//*****************************************************************************
//*
//*		IsoDel
//*
//*****************************************************************************
//	Listeneintrag lschen
void CIsoFile::IsoDel(int iNum)
{
IsoItem	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;

	pEntry=m_pFiles[iNum];

	if(!pEntry)return;

	m_iFilesNum--;
	m_pFiles[iNum]=0;

	if(iNum<m_iFilesNum)
		{
		memmove(m_pFiles+iNum,m_pFiles+iNum+1,(m_iFilesNum-iNum)*sizeof(m_pFiles[0]));
		}

	if(pEntry->pName)delete pEntry->pName;

	delete pEntry;

	if(m_iFilesNum<=0)
		{
		delete  m_pFiles;
		m_iFilesNum	 = 0;
		m_iFilesMax	 = 0;
		}
}

//*****************************************************************************
//*
//*		IsoNew
//*
//*****************************************************************************
//	Listeneintrag erzeugen
int	CIsoFile::IsoNew()
{
IsoItem	**pNew;



	if(!m_pFiles || !m_iFilesMax)						// Neue Kette
		{
		m_pFiles = new IsoItem*[ISO_NEW_ADD];
		memset(m_pFiles,0,sizeof(IsoItem*[ISO_NEW_ADD]));
		m_iFilesMax=ISO_NEW_ADD;
		}

	if(m_iFilesNum >= m_iFilesMax)						// Kette vergrern
		{
		pNew = new IsoItem*[m_iFilesMax+ISO_NEW_ADD];
		memset(pNew+m_iFilesMax,0,sizeof(IsoItem*[ISO_NEW_ADD]));
		memcpy(pNew,m_pFiles     ,sizeof(IsoItem*)*m_iFilesMax);
		delete m_pFiles;

		m_pFiles	= pNew;
		m_iFilesMax+= ISO_NEW_ADD;
		}


		   m_pFiles[m_iFilesNum] = new    IsoItem;
	memset(m_pFiles[m_iFilesNum],0,sizeof(IsoItem));
		   m_iFilesNum++;


return m_iFilesNum-1;
}

//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
//	Bearbeitung abbrechen ohne Daten zu speichern
void CIsoFile::Break()
{

	ReleaseData();
	m_iOpenMode=CAF_CLOSED;
	Clear();

}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das IsoFile und Speichert alle Daten
//	Ergibt;	CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CIsoFile::Close()
{


	m_iErrorCode=CAF_ERROR_OK;

	if(m_iOpenMode==CAF_CLOSED)
		{
		m_iOpenMode =CAF_CLOSED;
		m_iErrorCode=CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,-1);
		return     CAF_ERROR_MISS;
		}


	ReleaseData();
	m_iOpenMode=CAF_CLOSED;


return CAF_ERROR_OK;
}


//*****************************************************************************
//*
//*		ExtractProc
//*
//*****************************************************************************
//	Extrahiert eine Datei aus dem Archiv ber eine Ausgabefunktion
//	iNum	: Ist die Indexnummer der Datei die geprft werden soll
//	pParam	: Ist ein Parameter der an die Ausgabefunktionbergeben wird
//	pProc	: Ist der Zeiger auf die Ausgabefunktion
//	pInfo	: Hier werden Zeiten und Attribute der Datei gespeichert.
//			  Bei NULL werden diese Werte ignoriert.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_OK		= Wenn alles korrekt war
//				CAF_ERROR_MISS		= Wenn die Datei nicht gefunden wurde
//				CAF_ERROR_PATH		= Pfad ungltig ist.
//				CAF_ERROR_FCREATE	= Wenn Datei nicht erzeugt werden konnte
//				CAF_ERROR_CRC		= Crc32 Fehler
//				CAF_ERROR_DECOMP	= Fehler beim Entkomprimieren
//				CAF_ERROR_IGNORE	= Die Datei wurde bersprungen
//				... siehe Fehler Codes
int	CIsoFile::ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo)
{
char		aBuffer[2048];
IsoItem	   *pEntry;
aint64		iSize;
int			i;


	if(m_iDoneMode<=1)										// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}

	m_iErrorCode = CAF_ERROR_OK;

	try	{

		if( m_iOpenMode==CAF_CLOSED             )throw CAF_ERROR_MISS;
		if((unsigned)iNum>=(unsigned)m_iFilesNum)throw CAF_ERROR_MISS;

			pEntry=m_pFiles[iNum];
		if(!pEntry)throw CAF_ERROR_MISS;

		if(m_iDoneMode==1)									// Verlaufsanzeige starten
			{
			m_iDoneNow   = 0;
			m_iDoneMax   = pEntry->iLen;
			}


		m_pErrorFile = pEntry->pName;


		if(pEntry->cError)throw CAF_ERROR_READ;


		IsoSeekSec(pEntry->uOff);
		if(pEntry->uOffAdd)IsoSeek(pEntry->uOffAdd,SEEK_CUR);


			i = pEntry->uIntUsed;
		if(!i)i=0x7FFFFFFF;
															// Die Datei speichern
		for(m_iReadPos=0;m_iReadPos<pEntry->iLen;m_iReadPos+=iSize)
			{
			   iSize = pEntry->iLen-m_iReadPos;
			if(iSize>2048)iSize=2048;

			if(IsoRead(aBuffer,(int)iSize)!=iSize)
				{
				throw CAF_ERROR_READ;
				}

			if(pProc(pParam,aBuffer,(int)iSize)!=iSize)
				{
				throw CAF_ERROR_FWRITE;
				}

			i--;
			if(i)continue;
			IsoSeekSec(pEntry->uIntSize-pEntry->uIntUsed,SEEK_CUR);
			i = pEntry->uIntUsed;
			}
		
		if(pProc(pParam,NULL,0)!=0)							// Ende anzeigen
			{
			throw CAF_ERROR_FWRITE;
			}

		if(pInfo)
			{
			pInfo->dwAttr		=(pEntry->cDir)? SYS_DIR:SYS_READ|SYS_WRITE;
			pInfo->dwSize		= pEntry->uIntSize;
			pInfo->dwTimeWrite	= pEntry->iTime;
			pInfo->dwTimeAccess	= 0;
			pInfo->dwTimeCreate	= 0;
			pInfo->cName[0]		= 0;
			}
		}
	catch(int iError)
		{
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}

	if(m_iDoneMode<=1)										// Die Verlaufsanzeige zurcksetzen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}



return m_iErrorCode;
}


//*****************************************************************************
//*
//*		TimeConvert
//*
//*****************************************************************************
//	pTime[0] = Year;			Jahr (+1900)
//	pTime[1] = Month;			Monat (1=Jan)
//	pTime[2] = Day;				Day (1-31)
//	pTime[3] = Hour;			Stunde
//	pTime[4] = Minute;			Minute
//	pTime[5] = Secound;			Sekunde
//	pTime[6] = Timezone;		Zeitzone in 15min Schritten
time_t CIsoFile::TimeConvert(unsigned char *pTime)
{
time_t		iTime;
struct tm	sTime;


	sTime.tm_year	= pTime[0];
	sTime.tm_mon	= pTime[1]-1;
	sTime.tm_mday	= pTime[2];
	sTime.tm_hour	= pTime[3];
	sTime.tm_min	= pTime[4];
	sTime.tm_sec	= pTime[5];
	sTime.tm_isdst	= 0;

	iTime  = mktime(&sTime);
	iTime += ((char)pTime[6])*15*60;

	if(iTime<0)iTime=0;


return iTime;
}

//*****************************************************************************
//*
//*		TimeConvertE
//*
//*****************************************************************************
//	pTime[0] = Year;			Jahr (+1900)
//	pTime[1] = Month;			Monat (1=Jan)
//	pTime[2] = Day;				Day (1-31)
//	pTime[3] = Hour;			Stunde
//	pTime[4] = Minute;			Minute
//	pTime[5] = Secound;			Sekunde
//	pTime[6] = Timezone;		Zeitzone in 15min Schritten
time_t CIsoFile::TimeConvertE(unsigned char *pTime)
{
time_t		iTime;
struct tm	sTime;
int			iOff;


	sTime.tm_year	= (*(WORD*)(pTime+2))-1900;
	sTime.tm_mon	= pTime[3]-1;
	sTime.tm_mday	= pTime[4];
	sTime.tm_hour	= pTime[5];
	sTime.tm_min	= pTime[6];
	sTime.tm_sec	= pTime[7];
	sTime.tm_isdst	= 0;

	iTime  = mktime(&sTime);

	if(iTime<0)return 0;

	iOff   = *(WORD*)(pTime);
	iOff  &= 0x0FFF;

	if(iOff&0x0800)iOff|=~0x0FFF;

	if(iOff>=-1440 && iOff<=1440)
		{
		iTime += iOff*60;
		if(iTime<0)return 0;
		}



return iTime;
}

//*****************************************************************************
//*
//*		Count
//*
//*****************************************************************************
//	Ergibt die Anzahl der Eintrge in der Dateiliste
int CIsoFile::Count()
{
	return m_iFilesNum;
}


//*****************************************************************************
//*
//*		SetDoneMode
//*
//*****************************************************************************
//	Den Modus fr die Verlaufsanzeige setzen.
//	iMode	: Ist der neue Modus
//			    0 = Zurck zu Normal (Ende)
//				1 = Begin von Extract (einzelene Datei)
//				2 = Begin von ExtractAll
//	Ergibt den vorherigen Modus.
int CIsoFile::SetDoneMode(int iMode)
{
int		iRet;

	
	iRet = m_iDoneMode;
	
	switch(iMode)
		{
	case 0:													// Zurck zu Normal (Ende)

		if(m_iDoneMode==1 || m_iDoneMode==2)
		if(m_iErrorCode==CAF_ERROR_OK)
			{
			m_iDoneNow  = m_iDoneMax;
			}

		m_iDoneMode	 = 0;
		break;

	case 1:													// Begin von Extract (einzelene Datei)

		if(iRet==2)return iRet;
		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 0;
		break;

	case 2:													// Begin von ExtractAll

		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 1;
		break;
		}


return iRet;
}




//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CIsoFile::Done(aint64 *pMax)
{
aint64	iRet;


	switch(m_iDoneMode)
		{
	case 0:	iRet = m_iDoneNow;				break;			// Aus
	case 1:	iRet =            m_iReadPos;	break;			// Extract-Single
	case 2:	iRet = m_iDoneNow+m_iReadPos;	break;			// Extract-All
	default:iRet = 0;						break;
		}

	if(iRet>m_iDoneMax)iRet=m_iDoneMax;
	if(pMax)*pMax=m_iDoneMax;




return iRet;
}


//*****************************************************************************
//*
//*		ScanByte
//*
//*****************************************************************************
void CIsoFile::ScanByte(void *pDeskPtr,unsigned char *pBuffer)
{
VolumeDescriptor   *pDesk=(VolumeDescriptor*)pDeskPtr;
DirectoryRecord	   *pList;
zchar			   *pNameP;
IsoItem			   *pEntry;
IsoItem			   *pParent;
unsigned			uParent;
unsigned			uStart;
unsigned			uSize;
unsigned			uRead;
unsigned			uRoot;
unsigned			uLenP;
unsigned			uPos;
unsigned			uNum;
unsigned			uOff;
unsigned			uLen;
int					iIdx;
int					iCnt;
int					iNew;
int					iMax;



	if(pDesk->wLsbLogicalBlockSize!=2048)
		{
		throw CAF_ERROR_NOTSUPP;
		}


	uPos	= pDesk->dwLsbPathTable1;
	uSize	= pDesk->dwLsbPathTableSize;

	if(CheckSize(uPos,uSize))throw CAF_ERROR_FORM;


	uNum  = 512;
	uOff  = 512;
	uRoot = 0xFFFFFFFF;


	IsoSeekSec(uPos);


	for(uRead=0;uRead<uSize;uRead+=uLen,uOff+=uLen)				// Verzeichnisse einlesen
		{
		if(uOff+512>uNum)
			{
			uLen  = uNum-uOff;

			memcpy(pBuffer+512-uLen,pBuffer+uOff,uLen);

			uNum  = 512;
			uOff  = 512-uLen;
			iCnt  = IsoRead(pBuffer+uNum,2048);

			if(iCnt<0)throw CAF_ERROR_READ;

			uNum += iCnt;
			}


		uLen	= pBuffer[uOff]+pBuffer[uOff+1]+8;
		uStart	= *(DWORD*)(pBuffer+uOff+2);
		uParent	= *( WORD*)(pBuffer+uOff+6)-1;

		if(uLen&1)uLen++;

		if(!pBuffer[uOff+8] && pBuffer[uOff]==1 && uParent==0)	// Root Verzeichneis
			{
			uRoot = uStart;
			continue;
			}

		if(uParent>(unsigned)m_iFilesNum || pBuffer[uOff]<=0)
			{
			throw CAF_ERROR_FORM;
			}

		iIdx				= IsoNew();
		pEntry				= m_pFiles[iIdx];
		pEntry->uParent		= uParent;
		pEntry->uOff		= uStart;
		pEntry->cDir		= 1;

		if(uParent)												// Padanhngen
			{
			pParent			= m_pFiles[uParent-1];
			pEntry->pName	= new zchar[pBuffer[uOff]+2+pParent->uNameLen];
			pEntry->uNameLen= pParent->uNameLen+1;
			pEntry->pName[pParent->uNameLen] = '/';
			memcpy(pEntry->pName,pParent->pName,pParent->uNameLen*sizeof(zchar));
			}
		else{													// Im Root-Verzeichnis
			pEntry->pName	= new zchar[pBuffer[uOff]+1];
			pEntry->uNameLen= 0;
			}

		pEntry->uNameLen   += CreateStringA(pEntry->pName+pEntry->uNameLen,pBuffer+uOff+8,pBuffer[uOff]);

		PROG(CAF_PROG_FIND,iIdx);
		}



	iMax		= m_iFilesNum;
	uStart		= uRoot;
	uParent		= 0;
	pNameP		= 0;
	uLenP		= 0;
	iIdx		=-1;


	for(m_iDoneMax=iMax+1;;m_iDoneNow++)					// Dateien in Verzeichnissen suchen
		{
		if(CheckSize(uStart,2048))throw CAF_ERROR_FORM;

		uNum = 512;
		uOff = 512;

		IsoSeekSec(uStart);

		for(uRead=0;;uRead+=uLen,uOff+=uLen)				// Verzeichnisse einlesen
			{
			if(uOff+512>uNum)
				{
				uLen  = uNum-uOff;

				memcpy(pBuffer+512-uLen,pBuffer+uOff,uLen);

				uNum  = 512;
				uOff  = 512-uLen;
				iCnt  = IsoRead(pBuffer+uNum,2048);

				if(iCnt<0)throw CAF_ERROR_READ;

				uNum += iCnt;
				}

			   uLen	= pBuffer[uOff];
			if(uLen==0)break;
			if(uLen<34)throw CAF_ERROR_READ;
			   uLen += pBuffer[uOff+1];
			if(uLen&1)uLen++;

			pList	= (DirectoryRecord*)(pBuffer+uOff);

			if(pList->bNameLength<=1 && pList->cName[0]<=1)
				{
				if(pList->cName[0]==0 && iIdx>=0)
					{
					pParent->iTime = TimeConvert(&pList->bYear);
					}

				continue;
				}

			if(pList->bFlags&0x02)continue;
															// Verzeichnisse auslassen

			iNew				= IsoNew();
			pEntry				= m_pFiles[iNew];
			pEntry->uParent		= uParent;
			pEntry->iLen		= pList->dwLsbFileLength;
			pEntry->uOff		= pList->dwLsbSector;
			pEntry->uIntUsed	= pList->bInterleaveGAP;
			pEntry->uIntSize	= pList->bInterleaveSize;
			pEntry->iTime		= TimeConvert(&pList->bYear);


			pEntry->cError = CheckSize(pEntry->uOff,pEntry->iLen);



			if(uLenP)										// Pfad anhngen
				{
				pEntry->pName	= new zchar[pBuffer[uOff]+2+uLenP];
				pEntry->uNameLen= uLenP+1;
				pEntry->pName[uLenP] = '/';
				memcpy(pEntry->pName,pNameP,uLenP*sizeof(zchar));
				}
			else{											// Im Root-Verzeichnis
				pEntry->pName	= new zchar[pBuffer[uOff]+1];
				pEntry->uNameLen= 0;
				}

			pEntry->uNameLen   += CreateStringA(pEntry->pName+pEntry->uNameLen,pBuffer+uOff+33,pList->bNameLength);

			if(pEntry->uNameLen>3                     && pEntry->pName[pEntry->uNameLen-2]==';')
			if(pEntry->pName[pEntry->uNameLen-1]<='9' && pEntry->pName[pEntry->uNameLen-1]>='0')
				{
				pEntry->uNameLen -= 2;
				pEntry->pName[pEntry->uNameLen]=0;
				}

			if(pEntry->uNameLen>4                     && pEntry->pName[pEntry->uNameLen-3]==';')
			if(pEntry->pName[pEntry->uNameLen-1]<='9' && pEntry->pName[pEntry->uNameLen-1]>='0')
			if(pEntry->pName[pEntry->uNameLen-2]<='9' && pEntry->pName[pEntry->uNameLen-2]>='0')
				{
				pEntry->uNameLen -= 3;
				pEntry->pName[pEntry->uNameLen]=0;
				}

			PROG(CAF_PROG_FIND,iNew);
			}


		iIdx++;

		if(iIdx>=iMax)break;

		pParent = m_pFiles[iIdx];
		uStart 	= pParent->uOff;
		pNameP	= pParent->pName;
		uLenP	= pParent->uNameLen;
		}

}

//*****************************************************************************
//*
//*		ScanWide
//*
//*****************************************************************************
void CIsoFile::ScanWide(void *pDeskPtr,unsigned char *pBuffer)
{
VolumeDescriptor   *pDesk=(VolumeDescriptor*)pDeskPtr;
DirectoryRecord	   *pList;
zchar			   *pNameP;
IsoItem			   *pEntry;
IsoItem			   *pParent;
unsigned			uParent;
unsigned			uStart;
unsigned			uSize;
unsigned			uRead;
unsigned			uRoot;
unsigned			uLenP;
unsigned			uPos;
unsigned			uNum;
unsigned			uOff;
unsigned			uLen;
int					iIdx;
int					iCnt;
int					iNew;
int					iMax;



	if(pDesk->wLsbLogicalBlockSize!=2048)
		{
		throw CAF_ERROR_NOTSUPP;
		}


	uPos	= pDesk->dwLsbPathTable1;
	uSize	= pDesk->dwLsbPathTableSize;

	if(CheckSize(uPos,uSize))throw CAF_ERROR_FORM;


	uNum  = 512;
	uOff  = 512;
	uRoot = 0xFFFFFFFF;


	IsoSeekSec(uPos);


	for(uRead=0;uRead<uSize;uRead+=uLen,uOff+=uLen)				// Verzeichnisse einlesen
		{
		if(uOff+512>uNum)
			{
			uLen  = uNum-uOff;

			memcpy(pBuffer+512-uLen,pBuffer+uOff,uLen);

			uNum  = 512;
			uOff  = 512-uLen;
			iCnt  = IsoRead(pBuffer+uNum,2048);

			if(iCnt<0)throw CAF_ERROR_READ;

			uNum += iCnt;
			}


		uLen	= pBuffer[uOff]+pBuffer[uOff+1]+8;
		uStart	= *(DWORD*)(pBuffer+uOff+2);
		uParent	= *( WORD*)(pBuffer+uOff+6)-1;

		if(uLen&1)uLen++;

		if(!pBuffer[uOff+8] && pBuffer[uOff]==1 && uParent==0)	// Root Verzeichneis
			{
			uRoot = uStart;
			continue;
			}

		if(uParent>(unsigned)m_iFilesNum || pBuffer[uOff]<=0)
			{
			throw CAF_ERROR_FORM;
			}

		iIdx				= IsoNew();
		pEntry				= m_pFiles[iIdx];
		pEntry->uParent		= uParent;
		pEntry->uOff		= uStart;
		pEntry->cDir		= 1;

		if(uParent)												// Padanhngen
			{
			pParent			= m_pFiles[uParent-1];
			pEntry->pName	= new zchar[pBuffer[uOff]/2+2+pParent->uNameLen];
			pEntry->uNameLen= pParent->uNameLen+1;
			pEntry->pName[pParent->uNameLen] = '/';
			memcpy(pEntry->pName,pParent->pName,pParent->uNameLen*sizeof(zchar));
			}
		else{													// Im Root-Verzeichnis
			pEntry->pName	= new zchar[pBuffer[uOff]/2+1];
			pEntry->uNameLen= 0;
			}

		pEntry->uNameLen   += CreateStringW(pEntry->pName+pEntry->uNameLen,pBuffer+uOff+8,pBuffer[uOff]);

		PROG(CAF_PROG_FIND,iIdx);
		}



	iMax		= m_iFilesNum;
	uStart		= uRoot;
	uParent		= 0;
	pNameP		= 0;
	uLenP		= 0;
	iIdx		=-1;


	for(m_iDoneMax=iMax+1;;m_iDoneNow++)					// Dateien in Verzeichnissen suchen
		{
		if(CheckSize(uStart,2048))throw CAF_ERROR_FORM;

		uNum = 512;
		uOff = 512;

		IsoSeekSec(uStart);

		for(uRead=0;;uRead+=uLen,uOff+=uLen)				// Verzeichnisse einlesen
			{
			if(uOff+512>uNum)
				{
				uLen  = uNum-uOff;

				memcpy(pBuffer+512-uLen,pBuffer+uOff,uLen);

				uNum  = 512;
				uOff  = 512-uLen;
				iCnt  = IsoRead(pBuffer+uNum,2048);

				if(iCnt<0)throw CAF_ERROR_READ;

				uNum += iCnt;
				}

			   uLen	= pBuffer[uOff];
			if(uLen==0)break;
			if(uLen<34)throw CAF_ERROR_READ;
			   uLen += pBuffer[uOff+1];
			if(uLen&1)uLen++;

			pList	= (DirectoryRecord*)(pBuffer+uOff);

			if(pList->bNameLength<=1 && pList->cName[0]<=1)
				{
				if(pList->cName[0]==0 && iIdx>=0)
					{
					pParent->iTime = TimeConvertE(&pList->bYear);
					}

				continue;
				}

			if(pList->bFlags&0x02)continue;
															// Verzeichnisse auslassen

			iNew				= IsoNew();
			pEntry				= m_pFiles[iNew];
			pEntry->uParent		= uParent;
			pEntry->iLen		= pList->dwLsbFileLength;
			pEntry->uOff		= pList->dwLsbSector;
			pEntry->uIntUsed	= pList->bInterleaveGAP;
			pEntry->uIntSize	= pList->bInterleaveSize;
			pEntry->iTime		= TimeConvert(&pList->bYear);


			pEntry->cError = CheckSize(pEntry->uOff,pEntry->iLen);


			if(uLenP)										// Pfad anhngen
				{
				pEntry->pName	= new zchar[pBuffer[uOff]/2+2+uLenP];
				pEntry->uNameLen= uLenP+1;
				pEntry->pName[uLenP] = '/';
				memcpy(pEntry->pName,pNameP,uLenP*sizeof(zchar));
				}
			else{											// Im Root-Verzeichnis
				pEntry->pName	= new zchar[pBuffer[uOff]/2+1];
				pEntry->uNameLen= 0;
				}

			pEntry->uNameLen   += CreateStringW(pEntry->pName+pEntry->uNameLen,pBuffer+uOff+33,pList->bNameLength);

			PROG(CAF_PROG_FIND,iNew);
			}


		iIdx++;

		if(iIdx>=iMax)break;

		pParent = m_pFiles[iIdx];
		uStart 	= pParent->uOff;
		pNameP	= pParent->pName;
		uLenP	= pParent->uNameLen;
		}

}


//*****************************************************************************
//*
//*		ScanUdf
//*
//*****************************************************************************
void CIsoFile::ScanUdf(void *pDeskPtr,unsigned char *pBuffer)
{
EcmaAnchorVolumeDesc   *pAnch=(EcmaAnchorVolumeDesc*)pDeskPtr;
EcmaFileIdentifierDesc *pIden;
EcmaPartitionDesc      *pPart;
EcmaFileSetDesc		   *pFset;
EcmaFileEntry		   *pFile;
DWORD				   *pDwPtr;
zchar				   *pNameP;
IsoItem				   *pEntry;
IsoItem				   *pParent;
UdfDirEntry		 	   *pDirNow;
UdfDirEntry		 	   *pDirNew;
UdfDirEntry		 	   *pDirPtr;
zchar					cName[256];
unsigned				uDirMax;
unsigned				uDirCnt;
unsigned				uParent;
unsigned				uRead;
unsigned				uLenP;
unsigned				uLenN;
unsigned				uPos;
unsigned				uEnd;
unsigned				uPar;
unsigned				uNum;
unsigned				uOff;
unsigned				uLen;
unsigned				uMax;
int						iCnt;
int						iNew;




	pPart	= (EcmaPartitionDesc*)pBuffer;
	uPos	=  pAnch->dwMainVolumeDescLoc;
	uEnd	= (pAnch->dwMainVolumeDescLen+2047)/2048;
	uEnd   +=  uPos;


	IsoSeekSec(uPos);

	for(;;)												// Suche die Partition
		{
		if(uPos>=uEnd)throw CAF_ERROR_FORM;
		if(IsoRead(pBuffer,2048)!=2048)throw CAF_ERROR_FORM;
		if(pPart->sDescTag.bChecksum!=EcmaTagCrc(pPart))throw CAF_ERROR_FORM;
		if(pPart->sDescTag.wTagIdentifier==8)throw CAF_ERROR_FORM;
		if(pPart->sDescTag.wTagIdentifier==5)break;
		}


	uPar	= pPart->dwPartitionLoc;
	uEnd	= pPart->dwPartitionLen+uPar;

	if(uPar>=m_uIsoSectors)throw CAF_ERROR_FORM;		// Gre prfen
	if(uEnd> m_uIsoSectors)throw CAF_ERROR_FORM;
	if(uEnd<=uPar         )throw CAF_ERROR_FORM;

	pFset	= (EcmaFileSetDesc*)pBuffer;

	IsoSeekSec(uPar);

	for(uPos=0;;uPos++)									// Suche die Root-Verzeichnis
		{
		if(uPos>=16)throw CAF_ERROR_FORM;
		if(IsoRead(pBuffer,2048)!=2048)throw CAF_ERROR_FORM;
		if(pPart->sDescTag.bChecksum!=EcmaTagCrc(pPart))throw CAF_ERROR_FORM;
		if(pPart->sDescTag.wTagIdentifier==8)throw CAF_ERROR_FORM;
		if(pPart->sDescTag.wTagIdentifier==256)break;
		}


	pDirPtr	= new UdfDirEntry[256];
	uDirMax = 256;
	uDirCnt	= 1;

	if(!pDirPtr)throw CAF_ERROR_MEM;

	pDwPtr				= (DWORD*)(pFset->aRootDirectoryIcb);
	pDirPtr->dwSize		= pDwPtr[0];
	pDirPtr->dwSector	= pDwPtr[1];
	pDirPtr->dwEntry	= 0;


//******************** Verzeichnisse einlesen *********************************




	while(uDirCnt>0)										// Dateien in Verzeichnissen suchen
		{
		uDirCnt	   -= 1;
		pDirNow		= pDirPtr+uDirCnt;

		if(CheckSize(pDirNow->dwSector+uPar,pDirNow->dwSize))
			{
			delete pDirPtr;
			throw  CAF_ERROR_FORM;
			}

		uNum	= 2048;
		uOff	= 2048;

		if(pDirNow->dwEntry)
			{
			uParent = pDirNow->dwEntry;
			pParent	= m_pFiles[uParent-1];
			uLenP   = pParent->uNameLen;
			pNameP  = pParent->pName;
			}
		else{
			uParent = 0;
			pParent	= 0;
			uLenP   = 0;
			pNameP	= Z("");
			}


		IsoSeekSec(pDirNow->dwSector+uPar);
		uMax = pDirNow->dwSize;


		for(uRead=0;;uRead+=uLen,uOff+=uLen)				// Verzeichnisse einlesen
			{
			if(uOff+2048>uNum)
				{
				uLen  = uNum-uOff;

				memcpy(pBuffer+2048-uLen,pBuffer+uOff,uLen);

				uNum  = 2048;
				uOff  = 2048-uLen;
				iCnt  = IsoRead(pBuffer+uNum,2048);

				if(iCnt<0)
					{
					delete pDirPtr;
					throw  CAF_ERROR_READ;
					}

				uNum += iCnt;
				}


			if(uRead==0)									// Beginn des Verzeichnisse
				{
				pFile = (EcmaFileEntry*)(pBuffer+uOff);

				if(pFile->sDescTag.bChecksum!=EcmaTagCrc(pFile) ||
				   pFile->dwLengthOfExAttributes>2000 ||
				   pFile->sDescTag.wTagIdentifier!=261)
					{
					delete pDirPtr;
					throw  CAF_ERROR_FORM;
					}

				if(pFile->sIcbTag.bFileType!=4)break;

				uMax   = pFile->sDescTag.wDescLen+16;
				uLen   = offsetof(EcmaFileEntry,aData);
				uLen  += pFile->dwLengthOfExAttributes;

				if(uParent)
					{
					pEntry		  = m_pFiles[uParent-1];
					pEntry->iTime = TimeConvertE(pFile->aModificationTime);
					}
															// Weiter zu Stream
				if(pFile->dwLengthOfAllocation  != 8)
				if(pFile->dwLengthOfAllocation  !=16)continue;
				if(pFile->qwLogicalBlocksRecorded==0)continue;

				uMax   = (unsigned)pFile->qwInformationLength;
				uMax  += 2048;
				uNum   = 2048*2;
				uOff   = 2048;
				uLen   = 2048;

				pDwPtr = (DWORD*)(pFile->aData+pFile->dwLengthOfExAttributes);
				IsoSeekSec(pDwPtr[1]+uPar);

				continue;
				}


			if(uRead>=uMax)break;							// Ende des Verzeichnisses

			pIden = (EcmaFileIdentifierDesc*)(pBuffer+uOff);

			if(pIden->sDescTag.wTagIdentifier==0)break;


			if(pIden->sDescTag.bChecksum!=EcmaTagCrc(pIden) ||
			   pIden->sDescTag.wDescLen>2022)
				{
				delete pDirPtr;
				throw  CAF_ERROR_FORM;
				}


			uLen = pIden->sDescTag.wDescLen+16;

			if(pIden->sDescTag.wTagIdentifier!=257)continue;

			if(pIden->bFileCharacteristics&0xFD)continue;	// Diverse Dateien ignorieren


				uLenN = EcmaName(pIden->aData+pIden->wImplementationUseLen,pIden->bLengthOfFileIdentifier,cName);
			if(!uLenN)continue;


			if(pIden->bFileCharacteristics&0x02)			// Ein Verzeichniseintrag
				{
				if(uDirCnt>=uDirMax)
					{
					pDirNew  = new UdfDirEntry[uDirMax*2];
					uDirMax  = 256;

					if(!pDirNew)
						{
						delete pDirPtr;
						throw  CAF_ERROR_MEM;
						}

					memcpy(pDirNew,pDirPtr,sizeof(UdfDirEntry)*uDirCnt);
					delete pDirPtr;
					pDirPtr = pDirNew;
					}

				iNew				= IsoNew();
				pDwPtr				= (DWORD*)(pIden->aFileIcb);
				pDirNew				= pDirPtr+uDirCnt;
				pDirNew->dwSize		= pDwPtr[0];
				pDirNew->dwSector	= pDwPtr[1];
				pDirNew->dwEntry	= iNew+1;

				pEntry				= m_pFiles[pDirNew->dwEntry-1];
				pEntry->uParent		= uParent;
				pEntry->cDir		= 1;

				if(uLenP)									// Pfad anhngen
					{
					pEntry->pName	= new zchar[uLenP+uLenN+2];
					pEntry->uNameLen= uLenP+1;
					pEntry->pName[uLenP] = '/';
					memcpy(pEntry->pName,pNameP,uLenP*sizeof(zchar));
					}
				else{										// Im Root-Verzeichnis
					pEntry->pName	= new zchar[uLenN+1];
					pEntry->uNameLen= 0;
					}

				memcpy(pEntry->pName+pEntry->uNameLen,cName,(uLenN+1)*sizeof(zchar));
				pEntry->uNameLen   += uLenN;
				uDirCnt++;

				PROG(CAF_PROG_FIND,iNew);

				continue;
				}


			pDwPtr				= (DWORD*)(pIden->aFileIcb);// Datei einfgen
			iNew				= IsoNew();
			pEntry				= m_pFiles[iNew];
			pEntry->uOff		= pDwPtr[1]+uPar;
			pEntry->uParent		= uParent;

			if(uLenP)										// Pfad anhngen
				{
				pEntry->pName	= new zchar[uLenP+uLenN+2];
				pEntry->uNameLen= uLenP+1;
				pEntry->pName[uLenP] = '/';
				memcpy(pEntry->pName,pNameP,uLenP*sizeof(zchar));
				}
			else{											// Im Root-Verzeichnis
				pEntry->pName	= new zchar[uLenN+1];
				pEntry->uNameLen= 0;
				}

			memcpy(pEntry->pName+pEntry->uNameLen,cName,(uLenN+1)*sizeof(zchar));
			pEntry->uNameLen   += uLenN;

			PROG(CAF_PROG_FIND,iNew);
			}
		}


	delete 	pDirPtr;


//******************** Datei anpassen *****************************************


	pFile  = (EcmaFileEntry*)pBuffer;

	for(uPos=m_iFilesNum;uPos>0;)
		{
		uPos--;
		pEntry = m_pFiles[uPos];

		if(pEntry->cDir)continue;

		IsoSeekSec(pEntry->uOff);

		if(IsoRead(pBuffer,2048)!=2048 ||
		   pFile->sDescTag.bChecksum!=EcmaTagCrc(pFile) ||
		   pFile->sDescTag.wTagIdentifier != 261)
			{
			pEntry->cError = 1;
			pEntry->iLen   = 0;
			continue;
			}

		pEntry->iTime	 = TimeConvertE(pFile->aModificationTime);
		pEntry->iLen	 = pFile->qwInformationLength;

		switch(pFile->sIcbTag.wFlags&0x07)
			{
		case 0:								// FLAG_AD_SHORT

			if(pFile->dwLengthOfAllocation>=8)
			if(pFile->qwLogicalBlocksRecorded!=0)
				{
				pDwPtr		 = (DWORD*)(pFile->aData+pFile->dwLengthOfExAttributes);
				pEntry->uOff = pDwPtr[1]+uPar;
				continue;
				}

			break;

		case 1:								// FLAG_AD_LONG

			if(pFile->dwLengthOfAllocation>=16)
			if(pFile->qwLogicalBlocksRecorded!=0)
				{
				pDwPtr		 = (DWORD*)(pFile->aData+pFile->dwLengthOfExAttributes);
				pEntry->uOff = pDwPtr[1]+uPar;
				continue;
				}

			break;

		case 2:								// FLAG_AD_EXTENDED

			if(pFile->dwLengthOfAllocation>=16)
			if(pFile->qwLogicalBlocksRecorded!=0)
				{
				pDwPtr		 = (DWORD*)(pFile->aData+pFile->dwLengthOfExAttributes);
				pEntry->uOff = pDwPtr[3]+uPar;
				continue;
				}

			break;
			}

		pEntry->uOffAdd  = offsetof(EcmaFileEntry,aData);
		pEntry->uOffAdd += (WORD)pFile->dwLengthOfExAttributes;
		}

}

//*****************************************************************************
//*
//*		GetUserPtr
//*
//*****************************************************************************
//	User-Pointer abfragen
void *CIsoFile::GetUserPtr(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->pUserPtr;
}

//*****************************************************************************
//*
//*		SetUserPtr
//*
//*****************************************************************************
//	User-Pointer einstellen
void CIsoFile::SetUserPtr(int iNum,void *pData)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;

	m_pFiles[iNum]->pUserPtr = pData;

}

