//*****************************************************************************
//*
//*
//*		IsoFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
//
//	Klasse fr den Zugriff auf Iso Files:
//
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		char   *Name (int iNum,int &iLen);				// Hole Dateiname mit Lnge
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef		__ISOFILE_H__
#define     __ISOFILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	"FileFinder.h"
#include	"FileFunctions.h"
#include	"ArchiveFile.h"
#include	"Filter/Crc32.h"
#include	"Codecs/Huffman.h"


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif



//*****************************************************************************
//*
//*		Klasse CIsoFile
//*
//*****************************************************************************
class CIsoFile : public CArchiveFile
{
protected:

	typedef struct IsoItem
	{
	void		   *pUserPtr;					// User-Pointer
	zchar		   *pName;       				// File name in zip file
	unsigned 		uOff;						// Startsektor der Datei
	unsigned short	uOffAdd;					// Offest im Sektor
	unsigned char	uIntUsed;					// Gre des benutzten Interleaveblocks 
	unsigned char	uIntSize;					// Gre des Interleaveblocks (0=aus)
	unsigned		uParent;					// Ist das Elternverzeichnis
	unsigned	 	uNameLen;					// Dateinamen-Lnge
	time_t			iTime;						// Zeit der letzten nderung
	aint64			iLen;						// Lnge der Datei 
	char			cDir;						// Ist der Eintrag ein Verzeichnis
	char			cError;						// Ist die Position ok
	char			cDecomp;					// Zustand der Entkomprimierung
	}IsoItem;

	unsigned 		m_uIsoSectors;				// IsoFile Lnge in Sektoren
	unsigned		m_uSectorSize;				// Gre eines Sektors (0=2048)
	unsigned		m_uSectorShift;				// Offset zum ersten Sektor
	unsigned		m_uSectorOff;				// Offset der Daten im Sektor
	unsigned		m_uSectorEnd;				// Ende der Daten im Sektor
	int				m_iOpenMode;				// ffnungs Modus

	IsoItem       **m_pFiles;					// Alfabethische Liste
	int				m_iFilesMax;				// Gesamt Listengre in Eintrgen
	int				m_iFilesNum;				// Anzahl der Listeneintrge
	
	aint64			m_iReadPos;					// Aktuelle Leseposition
	aint64			m_iDoneNow;					// Aktueller Verlauf
	aint64			m_iDoneMax;					// Maximaler Verlauf
	int 			m_iDoneMode;				// 0=direkt von m_iDoneNow 1=Extraxt 2=ExtractAll 3=Flush

	


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

					CIsoFile();
	virtual		   ~CIsoFile();


	virtual	int		ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo);
	virtual	int		Close  		();						// Datei schliesen
	virtual	int	    Count  		();						// Anzahl der Dateien
	virtual	void	Break  		();						// Bearbeitung abbrechen
	virtual	zchar  *Name   		(int  iNum);			// Hole Dateiname
	virtual	zchar  *Name   		(int  iNum,int &iLen);	// Hole Dateiname mit Lnge
	virtual	int		Sort   		(int  iMode=0);			// Sortierte Kette
	virtual	int     Stats		(int  iNum);			// Zustansbits abfragen
	virtual	int     SetFail		(int  iNum,int iFail=1);// Defekt-Flag setzen
	virtual	int     IsFail 		(int  iNum            );// Ist die Datei defekt
	virtual	int     IsDir  		(int  iNum);			// Prfe ob Verzeichnis
	virtual	int     IsOk   		(int  iNum);			// Richtig endkomprimiert
	virtual	aint64	Len    		(int  iNum,int iMode=0);// Hole Dateilnge
	virtual	time_t	Time   		(int  iNum);			// Dateizeit abfragen
	virtual	time_t	Time   		(int  iNum,int iMode);	// Dateizeit abfragen
	virtual	int		Find   		(const zchar  *pName);	// Datei suchen
	virtual	aint64	Done   		(aint64 *pMax);			// Verlaufsabfrage
	virtual	void   *GetUserPtr 	(int iNum            );	// Datenwert auslesen
	virtual	void    SetUserPtr 	(int iNum,void *pData);	// Datenwert einstellen



protected:

	virtual	int		ArchOpen	(int iLevel=9);			// Die ISO-Datei ffnen	
	virtual	int		SetDoneMode	(int iMode);			// Den Done-Modus setzen
	void			Clear		();						// Speicher lschen
	time_t			TimeConvertE (unsigned char *pTime);
	time_t			TimeConvert  (unsigned char *pTime);
	int				CreateStringA(zchar *pBuffer,const unsigned char *pName,int iSize);
	int				CreateStringW(zchar *pBuffer,const unsigned char *pName,int iSize);
	int				CheckSize    (unsigned uSector,unsigned aint64 uSize);

	int				IsoNew    ();						// Listen Eintrag erzeugen
	void			IsoDel    (int  iNum);				// Listen Eintrag lschen
	int				IsoOpen   (const zchar *pName,int iMode=0);

	int				IsoRead   (void        *pMem ,int          iMode);	// IsoFile lesen
	aint64			IsoSeek   (aint64      lPos ,int iMode=SEEK_SET);	// IsoFile Offset setzen
	aint64			IsoSeekSec(int			iSect,int iMode=SEEK_SET);	// IsoFile Offset setzen mittels Sektor

	void			ScanByte  (void *pDeskPtr,unsigned char *pBuffer);
	void			ScanWide  (void *pDeskPtr,unsigned char *pBuffer);
	void			ScanUdf   (void *pDeskPtr,unsigned char *pBuffer);

static unsigned		EcmaTagCrc(void *pTagPtr);
static unsigned		EcmaName(const unsigned char *pQuelle,int iMax,zchar *pOut);
static int __cdecl 	CmpName (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpLen  (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpPos  (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpEx   (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion

};

#endif



