//*****************************************************************************
//*
//*
//*		Xz.c
//*
//*
//*****************************************************************************
#include	"../Filter/Sha256.h"
#include	"../Filter/Crc32.h"
#include	"../Filter/Crc64.h"
#include	"CpuArch.h"
#include	"Xz.h"

#include	"../Filter/Crc32.h"


Byte		XZ_SIG       [XZ_SIG_SIZE       ] = { 0xFD, '7', 'z', 'X', 'Z', 0 };
Byte		XZ_FOOTER_SIG[XZ_FOOTER_SIG_SIZE] = { 'Y', 'Z' };

//*****************************************************************************
//*
//*		Xz_WriteVarInt
//*
//*****************************************************************************
unsigned Xz_WriteVarInt(Byte *buf,UInt64 v)
{
unsigned	i = 0;


	do	{
		buf[i] = (Byte)((v & 0x7F) | 0x80);
		i     +=  1;
		v	 >>=  7;
		}while(v!=0);
	
	buf[i-1] &= 0x7F;


return i;
}

//*****************************************************************************
//*
//*		Xz_Construct
//*
//*****************************************************************************
void Xz_Construct(CXzStream *p)
{
	p->numBlocks	= p->numBlocksAllocated = 0;
	p->blocks		= 0;
	p->flags		= 0;
}

//*****************************************************************************
//*
//*		Xz_Free
//*
//*****************************************************************************
void Xz_Free(CXzStream *p,ISzAlloc *alloc)
{
	alloc->Free(alloc, p->blocks);
	p->numBlocks	= p->numBlocksAllocated = 0;
	p->blocks		= 0;
}

//*****************************************************************************
//*
//*		XzFlags_GetCheckSize
//*
//*****************************************************************************
unsigned XzFlags_GetCheckSize(CXzStreamFlags f)
{
int			t = XzFlags_GetCheckType(f);

return (t==0)? 0 : (4<<((t-1)/3));
}

//*****************************************************************************
//*
//*		XzCheck_Init
//*
//*****************************************************************************
void XzCheck_Init(CXzCheck *p,int mode)
{
	
	p->mode = mode;
	
	switch(mode)
		{
	case XZ_CHECK_CRC32:  p->crc   = CRC32_INIT_VAL; break;
	case XZ_CHECK_CRC64:  p->crc64 = CRC64_INIT_VAL; break;
	case XZ_CHECK_SHA256: Sha256Init(&p->sha);		 break;	
		}

}

//*****************************************************************************
//*
//*		XzCheck_Update
//*
//*****************************************************************************
void XzCheck_Update(CXzCheck *p,const void *data,size_t size)
{

	switch(p->mode)
		{
	case XZ_CHECK_CRC32: p->crc   = Crc32 ( p->crc  ,data,(int)size); break;
	case XZ_CHECK_CRC64: p->crc64 = Crc64 ( p->crc64,data,(int)size); break;
	case XZ_CHECK_SHA256:           Sha256(&p->sha  ,data,(int)size); break;
		}

}

//*****************************************************************************
//*
//*		XzCheck_Final
//*
//*****************************************************************************
int XzCheck_Final(CXzCheck *p,Byte *digest)
{
UInt64	v ;
int		i;


	switch(p->mode)
		{
	case XZ_CHECK_CRC32:

		SetUi32(digest,p->crc^CRC32_INIT_VAL);
		break;

	case XZ_CHECK_CRC64:

		v = p->crc64^CRC64_INIT_VAL;

		for(i=0;i<8;i++,v>>=8)
			{
			digest[i] = (Byte)(v & 0xFF);
			}

		break;

	case XZ_CHECK_SHA256:

		Sha256Exit(&p->sha, digest);
		break;

	default:

		return 0;
		}


return 1;
}
