//*****************************************************************************
//*
//*
//*		rar_errhnd.h
//*
//*
//*****************************************************************************
#ifndef		_RAR_ERRHANDLER_			
#define		_RAR_ERRHANDLER_			

enum RAR_EXIT // RAR exit code.
	{
	RARX_SUCCESS	 =	 0,				// CAF_ERROR_OK,		// REM AZ die Fehlernummern wurde auf die von Zip-Dateien gemappt
	RARX_WARNING	 =	 5,				// CAF_ERROR_LOGIC,
	RARX_FATAL		 =	 3,				// CAF_ERROR_FORM,
	RARX_CRC		 =	21,				// CAF_ERROR_CRC,
	RARX_LOCK		 =	14,				// CAF_ERROR_WRITE,
	RARX_WRITE		 =	14,				// CAF_ERROR_WRITE,
	RARX_OPEN		 =	18,				// CAF_ERROR_OPEN,
	RARX_USERERROR	 =	16,				// CAF_ERROR_PARAM,
	RARX_MEMORY		 =	 4,				// CAF_ERROR_MEM,
	RARX_CREATE		 =	15,				// CAF_ERROR_CREATE,
	RARX_NOFILES	 =	16,				// CAF_ERROR_PARAM,
	RARX_BADPWD		 =	32,				// CAF_ERROR_PASSWD
	RARX_USERBREAK   =  23,				// CAF_ERROR_BREAK,
	};

class ErrorHandler
{
private:
	RAR_EXIT ExitCode;
	uint ErrCount;
	bool EnableBreak;
	bool Silent;
	bool DisableShutdown;										// Shutdown is not suitable after last error.
public:
	ErrorHandler();
	void Clean();
	void MemoryError();
	void OpenError(const wchar *FileName);
	void CloseError(const wchar *FileName);
	void ReadError(const wchar *FileName);
	bool AskRepeatRead(const wchar *FileName);
	void WriteError(const wchar *ArcName,const wchar *FileName);
	void WriteErrorFAT(const wchar *FileName);
	bool AskRepeatWrite(const wchar *FileName,bool DiskFull);
	void SeekError(const wchar *FileName);
	void GeneralErrMsg(const wchar *fmt,...);
	void MemoryErrorMsg();
	void OpenErrorMsg(const wchar *FileName);
	void OpenErrorMsg(const wchar *ArcName,const wchar *FileName);
	void CreateErrorMsg(const wchar *FileName);
	void CreateErrorMsg(const wchar *ArcName,const wchar *FileName);
	void ReadErrorMsg(const wchar *FileName);
	void ReadErrorMsg(const wchar *ArcName,const wchar *FileName);
	void WriteErrorMsg(const wchar *ArcName,const wchar *FileName);
	void ArcBrokenMsg(const wchar *ArcName);
	void ChecksumFailedMsg(const wchar *ArcName,const wchar *FileName);
	void UnknownMethodMsg(const wchar *ArcName,const wchar *FileName);
	void Exit(RAR_EXIT ExitCode);
	void SetErrorCode(RAR_EXIT Code);
	RAR_EXIT GetErrorCode() 
		{
		return ExitCode;
		}

	uint GetErrorCount() 
		{
		return ErrCount;
		}

	void SetSignalHandlers(bool Enable);
	void Throw(RAR_EXIT Code);
	void SetSilent(bool Mode) 
		{
		Silent=Mode;
		}

	bool GetSysErrMsg(wchar *Msg,size_t Size);
	void SysErrMsg();
	int GetSystemErrorCode();
	void SetSystemErrorCode(int Code);
	void SetDisableShutdown() 
		{
		DisableShutdown=true;
		}

	bool IsShutdownEnabled() 
		{
		return !DisableShutdown;
		}

	bool UserBreak; 											// Ctrl+Break is pressed.
	bool MainExit; // main() is completed.
};


#endif
