//*****************************************************************************
//*
//*
//*		rar_sha256.cpp
//*
//*
//*****************************************************************************
#include	"rar_main.h"
#include	"rar_sha256.h"

static const uint32 K[64] =
	{
	0x428A2F98, 0x71374491, 0xB5C0FBCF, 0xE9B5DBA5,
	0x3956C25B, 0x59F111F1, 0x923F82A4, 0xAB1C5ED5,
	0xD807AA98, 0x12835B01, 0x243185BE, 0x550C7DC3,
	0x72BE5D74, 0x80DEB1FE, 0x9BDC06A7, 0xC19BF174,
	0xE49B69C1, 0xEFBE4786, 0x0FC19DC6, 0x240CA1CC,
	0x2DE92C6F, 0x4A7484AA, 0x5CB0A9DC, 0x76F988DA,
	0x983E5152, 0xA831C66D, 0xB00327C8, 0xBF597FC7,
	0xC6E00BF3, 0xD5A79147, 0x06CA6351, 0x14292967,
	0x27B70A85, 0x2E1B2138, 0x4D2C6DFC, 0x53380D13,
	0x650A7354, 0x766A0ABB, 0x81C2C92E, 0x92722C85,
	0xA2BFE8A1, 0xA81A664B, 0xC24B8B70, 0xC76C51A3,
	0xD192E819, 0xD6990624, 0xF40E3585, 0x106AA070,
	0x19A4C116, 0x1E376C08, 0x2748774C, 0x34B0BCB5,
	0x391C0CB3, 0x4ED8AA4A, 0x5B9CCA4F, 0x682E6FF3,
	0x748F82EE, 0x78A5636F, 0x84C87814, 0x8CC70208,
	0x90BEFFFA, 0xA4506CEB, 0xBEF9A3F7, 0xC67178F2
	};

// SHA-256 functions. We could optimize Ch and Maj a little,
// but with no visible speed benefit.
#define		Ch(x,y,z)					((x & y) ^ (~x & z))
#define		Maj(x,y,z)					((x & y) ^ (x & z) ^ (y & z))

// Sigma functions.
#define		Sg0(x)						(rotr32(x, 2) ^ rotr32(x,13) ^ rotr32(x, 22))
#define		Sg1(x)						(rotr32(x, 6) ^ rotr32(x,11) ^ rotr32(x, 25))
#define		sg0(x)						(rotr32(x, 7) ^ rotr32(x,18) ^ (x >> 3))
#define		sg1(x)						(rotr32(x,17) ^ rotr32(x,19) ^ (x >> 10))

//*****************************************************************************
//*
//*		sha256_init
//*
//*****************************************************************************
void sha256_init(sha256_context *ctx)
{
	ctx->H[0]	= 0x6A09E667;									// Set the initial hash value.
	ctx->H[1]	= 0xBB67AE85;
	ctx->H[2]	= 0x3C6EF372;
	ctx->H[3]	= 0xA54FF53A;
	ctx->H[4]	= 0x510E527F;
	ctx->H[5]	= 0x9B05688C;
	ctx->H[6]	= 0x1F83D9AB;
	ctx->H[7]	= 0x5BE0CD19;
	ctx->Count	= 0;											// Processed data counter.
}


//*****************************************************************************
//*
//*		sha256_transform
//*
//*****************************************************************************
static void sha256_transform(sha256_context *ctx)
{
uint32		W[64]; // Words of message schedule.
uint32		v[8];												// FIPS a, b, c, d, e, f, g, h working variables.

	for(uint I=0;I<16;I++)										// Prepare message schedule.
		{
		W[I] = RawGetBE4(ctx->Buffer + I * 4);
		}

	for(uint I=16;I<64;I++)
		{
		W[I] = sg1(W[I-2]) + W[I-7] + sg0(W[I-15]) + W[I-16];
		}

	uint32 *H=ctx->H;
	v[0] = H[0]; v[1]=H[1]; v[2]=H[2]; v[3]=H[3];
	v[4] = H[4]; v[5]=H[5]; v[6]=H[6]; v[7]=H[7];

	for(uint I=0;I<64;I++)
		{
		uint	T1 = v[7] + Sg1(v[4]) + Ch(v[4], v[5], v[6]) + K[I] + W[I];

		// It is possible to eliminate variable copying if we unroll loop
		// and rename variables every time. But my test did not show any speed
		// gain on i7 for such full or partial unrolling.
		v[7] = v[6];
		v[6] = v[5];
		v[5] = v[4];
		v[4] = v[3] + T1;

		// It works a little faster when moved here from beginning of loop.
		uint T2 = Sg0(v[0]) + Maj(v[0], v[1], v[2]);

		v[3] = v[2];
		v[2] = v[1];
		v[1] = v[0];
		v[0] = T1 + T2;
		}

	H[0]+=v[0]; H[1]+=v[1]; H[2]+=v[2]; H[3]+=v[3];
	H[4]+=v[4]; H[5]+=v[5]; H[6]+=v[6]; H[7]+=v[7];
}


//*****************************************************************************
//*
//*		sha256_process
//*
//*****************************************************************************
void sha256_process(sha256_context *ctx,const void *Data,size_t Size)
{
const byte	   *Src=(const byte *)Data;
size_t			BufPos = (uint)ctx->Count & 0x3F;


	ctx->Count+=Size;
	while(Size > 0)
		{
		size_t	BufSpace=sizeof(ctx->Buffer)-BufPos;
		size_t	CopySize=Size>BufSpace ? BufSpace:Size;

		memcpy(ctx->Buffer+BufPos,Src,CopySize);

		Src    += CopySize;
		BufPos += CopySize;
		Size   -= CopySize;
		if(BufPos == 64)
			{
			BufPos = 0;
			sha256_transform(ctx);
			}
		}
}


//*****************************************************************************
//*
//*		sha256_done
//*
//*****************************************************************************
void sha256_done(sha256_context *ctx,byte *Digest)
{
uint64		BitLength = ctx->Count * 8;
uint		BufPos = (uint)ctx->Count & 0x3F;


	ctx->Buffer[BufPos++] = 0x80;								// Padding the message with "1" bit.

	if(BufPos!=56) // We need 56 bytes block followed by 8 byte length.
		{
		if(BufPos>56)
			{
			while(BufPos<64)
				{
				ctx->Buffer[BufPos++] = 0;
				}

			BufPos=0;
			}

		if(BufPos==0)
			{
			sha256_transform(ctx);
			}

		memset(ctx->Buffer+BufPos,0,56-BufPos);
		}

	RawPutBE4((uint32)(BitLength>>32), ctx->Buffer + 56);
	RawPutBE4((uint32)(BitLength), ctx->Buffer + 60);

	sha256_transform(ctx);

	RawPutBE4(ctx->H[0], Digest +	 0);
	RawPutBE4(ctx->H[1], Digest +	 4);
	RawPutBE4(ctx->H[2], Digest +	 8);
	RawPutBE4(ctx->H[3], Digest + 12);
	RawPutBE4(ctx->H[4], Digest + 16);
	RawPutBE4(ctx->H[5], Digest + 20);
	RawPutBE4(ctx->H[6], Digest + 24);
	RawPutBE4(ctx->H[7], Digest + 28);

	sha256_init(ctx);
}
