//*****************************************************************************
//*
//*
//*		RarFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//
//	Klasse fr den Zugriff auf RAR-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen
//
#include	<stdio.h>
#include	<string.h>
#include	"RarFile.h"
#include	"Rar/rar_api.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		(int)wcslen
#define 	strzncpy	wcsncpy
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		(int)strlen
#define 	strzncpy	strncpy
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif


#define PROG(iMode,iNum)									\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))				\
	{														\
	if(Progress(iMode,iNum))throw CAF_ERROR_BREAK;			\
	}														
#define PROGR(iMode,iNum,iRet)								\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))				\
	{														\
	if(Progress(iMode,iNum))return iRet;					\
	}														
#define PROGN(iMode,iNum)									\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))				\
	{														\
	Progress(iMode,iNum);									\
	}
#define PROGT(iMode,iNum,iRet)								\
	if(pThis->m_pProgress && (pThis->m_iProgOn&(1<<iMode)))	\
	{														\
	if(pThis->Progress(iMode,iNum))return iRet;				\
	}														


static		HANDLE			hRarSem;
#define 	RAR_INIT()		if(!hRarSem)CreateSemaphore(0,1,0x70000000,0)
#define 	RAR_LOCK()		WaitForSingleObject(hRarSem,0xFFFFFFFF)
#define 	RAR_UNLOCK()	ReleaseSemaphore(hRarSem,1,0)

extern "C"
{
extern	void    *		  rar_param;
extern	void    *(_cdecl *rar_open )(void *stream,const char *filename,const char *mode);
extern	int 	 (_cdecl *rar_read )(void *stream,void *buf,unsigned len);
extern	int 	 (_cdecl *rar_seek )(void *stream,int offset,int fromwhere);
extern	int 	 (_cdecl *rar_tell )(void *stream);
extern	int 	 (_cdecl *rar_close)(void *stream);
}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CRarFile::CRarFile()
{

	RAR_INIT();

	m_pFiles			=  0;
	m_pFilesSort		=  0;
	m_iFileCount		=  0;
	m_pProc				=  0;
	m_iSolidMode		=  0;
	m_iDoneNow			=  0;
	m_iDoneMax			=  0;
	m_iDoneMode			=  0;
	m_iPassFlag			=  0;
	m_cHasCompress		=  0;
	m_pDecompMem		=  0;
	m_uDecompSize		=  0;
	m_uDecompPos		=  0;
	m_uDecompNow		=  0;
	m_iDecompNum		=  0;
	m_cDecompFile[0]	=  0;
	m_hDecompFile		=  INVALID_HANDLE_VALUE;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CRarFile::~CRarFile()
{

	if(m_iOpenMode!=CAF_CLOSED)Close();
	Clear();


}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Datenfelder und setzt alle Variabeln zurck
void CRarFile::Clear()
{

	if(m_pFiles)
		{
		delete m_pFiles;
		m_iFileCount = 0;
		m_pFiles	 = NULL;
		}

	if(m_pFilesSort)
		{
		delete m_pFilesSort;
		m_pFilesSort = 0;
		}

	m_iModeFlags	= 0;						// Flags
	m_iFileCount	= 0;
	m_uDecompSize	= 0;


}

//*****************************************************************************
//*
//*		CmpEx
//*
//*****************************************************************************
int __cdecl CRarFile::CmpEx(const RarList *pPtr1,const RarList *pPtr2)
{
return strvicmp(pPtr1->cFileName,pPtr2->cFileName);
}

//*****************************************************************************
//*
//*		RarCmpName
//*
//*****************************************************************************
int __cdecl CRarFile::RarCmpName(const RarList *pPtr1,const RarList *pPtr2)
{
return strzicmp(pPtr1->cFileName,pPtr2->cFileName);
}

//*****************************************************************************
//*
//*		CmpLen
//*
//*****************************************************************************
int __cdecl CRarFile::CmpLen(const RarList *pPtr1,const RarList *pPtr2)
{

	if(pPtr1->uLenFull<pPtr2->uLenFull)return -1;
	if(pPtr1->uLenFull>pPtr2->uLenFull)return  1;

return 0;
}

//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CRarFile::CmpPos(const RarList *pPtr1,const RarList *pPtr2)
{
return (int)(pPtr1-pPtr2);
}

//*****************************************************************************
//*
//*		Len
//*
//*****************************************************************************
//	Ergibt die Lnge der Datei mit der Indexnummer iNum
//	Ist iMode==0 wird die originale    Lnge zurckgegeben
//	Ist iMode!=0 wird die komprimierte Lnge zurckgegeben
aint64 CRarFile::Len(int iNum,int iMode)
{
RarList	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFileCount)return 0;
	iNum	 =m_pFilesSort[iNum];
	pEntry	= m_pFiles+iNum;

return (iMode)? pEntry->uLenComp:pEntry->uLenFull;
}

//*****************************************************************************
//*
//*		SetFail
//*
//*****************************************************************************
//	Setzt das Defekt-Flag und ergibt den alten Wert.
int CRarFile::SetFail(int iNum,int iFail)
{
int		iOld;

	if((unsigned)iNum>=(unsigned)m_iFileCount)return -1;

	iOld = m_pFiles[iNum].cError;
	m_pFiles[iNum].cError = iFail;

return iOld;
}

//*****************************************************************************
//*
//*		IsFail
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit iNum beschdigt ist sonst 0
int CRarFile::IsFail(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFileCount)return -1;

return m_pFiles[iNum].cError;
}

//*****************************************************************************
//*
//*		IsDir
//*
//*****************************************************************************
// Ergibt die 1 wenn die Datei mit iNum ein Verzeichnis ist sonst 0
int CRarFile::IsDir(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFileCount)return 0;
	iNum=m_pFilesSort[iNum];

return (m_pFiles[iNum].cDirectory)? 1:0;
}

//*****************************************************************************
//*
//*		IsOk
//*
//*****************************************************************************
// Ergibt:	2	Wenn die Datei vollstntig Entzipt wurde
//          1	Wenn die Datei schon vorhanden war
//			0   Bei Fehlern
int CRarFile::IsOk(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFileCount)return 0;
	iNum=m_pFilesSort[iNum];

return m_pFiles[iNum].cDecompOk;
}

//*****************************************************************************
//*
//*		Stats
//*
//*****************************************************************************
//	Die Zustandbits abfragen
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//	iDel		: Welche Bits sollen gelscht werden
//	Ergibt einen Wert mit veschiedenen Bits:
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//					CAF_STAT_DELETE		0x0004	Die Datei wurde aus dem Verzeichnis gelscht
//					CAF_STAT_WRITE		0x0008	Die Datei muss ins Archiv geschrieben werden
//					CAF_STAT_DIR		0x0010	Die Datei ist ein Verzeichniss
//					CAF_STAT_READONLY	0x0020	Die Datei ist schreibgeschtzt
//					CAF_STAT_SAVED		0x0040	Die Datei ist im Archiv gespeichert
int CRarFile::Stats(int iNum)
{
RarList	   *pEntry;
int			iBits;


	if((unsigned)iNum>=(unsigned)m_iFileCount)return 0;
	iNum=m_pFilesSort[iNum];

			pEntry=m_pFiles+iNum;
	iBits = pEntry->cDecompOk&3;

						  iBits|=CAF_STAT_SAVED;
	if(pEntry->cDirectory)iBits|=CAF_STAT_DIR;
	

return iBits;	
}

//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
// Ergibt die Zeit der letzten nderung einer Datei
// bzw.	CAF_ERROR_MISS wenn iNum/Datei nicht gefunden wurde.
time_t CRarFile::Time(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFileCount)return CAF_ERROR_MISS;
	iNum=m_pFilesSort[iNum];

return m_pFiles[iNum].iTime;
}

time_t CRarFile::Time(int iNum,int Mode)
{
	if((unsigned)iNum>=(unsigned)m_iFileCount)
		{
		return CAF_ERROR_MISS;
		}

	iNum=m_pFilesSort[iNum];

	switch(Mode)
		{
	case CAF_TIME_W:
	case CAF_TIME_C:
	case CAF_TIME_A: return m_pFiles[iNum].iTime;
		}


return CAF_ERROR_MISS;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CRarFile::Name(int iNum)
{

	if((unsigned)iNum<(unsigned)m_iFileCount)
		{
		iNum = m_pFilesSort[iNum];
		return m_pFiles[iNum].cFileName;
		}

	if(iNum==-1)
		{
		return m_cFilePath;
		}


return NULL;
}

//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei samt Lnge.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CRarFile::Name(int iNum,int &iLen)
{


	if((unsigned)iNum<(unsigned)m_iFileCount)
		{
		iNum = m_pFilesSort[iNum];
		iLen = m_pFiles[iNum].uNameLen;


		return m_pFiles[iNum].cFileName;
		}
	
	if(iNum==-1)
		{
		iLen = strzlen(m_cFilePath);
		return m_cFilePath;
		}

	iLen = 0;


return NULL;
}


//*****************************************************************************
//*
//*		RarClose
//*
//*****************************************************************************
//	Schliet RarFile
//	Ergibt -1 bei einem Fehler ansonsten 0
int	CRarFile::RarClose()
{
int		iErr;


	RAR_LOCK();

	iErr   = RarCloseArchive(m_hRar);
	m_hRar = 0;

	RAR_UNLOCK();

	if(iErr!=ERAR_SUCCESS)
		{
		return -1;
		}



return 0;
}

//*****************************************************************************
//*
//*		RarCallback
//*
//*****************************************************************************
//	Callback aus der RAR-Datei
int __cdecl CRarFile::RarCallback(unsigned uMsg,RAR_LPARAM lUserData,RAR_LPARAM lParam1,RAR_LPARAM lParam2)
{
CRarFile   *pThis=(CRarFile*)lUserData;
char	   *pPassWd;
int			iPos;


	if(uMsg!=UCM_NEEDPASSWORD)return 0;

	pPassWd = (char*)lParam1;

	if(!pThis->m_pPassword)								// Password via Callback anfordern
		{
		PROGT(CAF_PROG_PASSWD,-1,0);
		}

	if(!pThis->m_pPassword)
		{
		pThis->m_iPassFlag = 1;
		return -1;
		}

	for(iPos=0;iPos<lParam2;iPos++)						// Kopiere das Password
		{
			pPassWd[iPos]=pThis->m_pPassword[iPos];
		if(!pPassWd[iPos])break;
		}


return iPos;
}

//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Liste in m_pFiles
//  iMode:	CAF_FLAG_SORT			0x0400	Nach Namen  sortieren
//			CAF_FLAG_SORTLEN	    0x0500	Nach Lngen sortieren
//			CAF_FLAG_SORTPOS	    0x0600	Nach Offset sortieren (intern)
//			CAF_FLAG_SORTEX			0x0700	Nach Namen mit Zahlen Bercksichtigung
int	CRarFile::Sort(int iMode)
{
int		(__cdecl *pCmp)(const RarList*,const RarList *);
int		iStart,iEnd,iTemp,iLevel;
int		aMenU[30];
int		aMenO[30];
int		u,o,m,c;



		iMode&=0x0700;
	if(!iMode)iMode=m_iModeFlags&0x0700;
	iMode |= 0x0400;

	PROGR(CAF_PROG_SORT,-1,0);


	if(!m_pFiles)return 0;

	if(iMode==m_iSortMode)								// Ist die Liste bereits sortiert
		{
		return m_iFileCount;
		}

	if(m_iFileCount>0)
		{
		switch(iMode)
			{
		case    CAF_FLAG_SORTLEN:	pCmp=CmpLen ;break;
		case    CAF_FLAG_SORTPOS:	pCmp=CmpPos ;break;
		case	CAF_FLAG_SORTEX:	pCmp=CmpEx	;break;
		default:					pCmp=RarCmpName;break;
			}

		iLevel = 0;
		iStart = 0;
		iEnd   = m_iFileCount-1;

		for(;;)
			{
			m = (iStart + iEnd)>>1;								// Mitte berechnen
			u =  iStart;
			o =  iEnd + 1;

			iTemp           = m_pFilesSort[m];
			m_pFilesSort[m] = m_pFilesSort[u];
			m_pFilesSort[u] = iTemp;

			for(;;)
				{
				do  {
					u++;
					if(u>iEnd)break;
					c=pCmp(m_pFiles+m_pFilesSort[u],m_pFiles+m_pFilesSort[iStart]);
					}
				while(c<=0);

				do  {
					o--;
					if(o<=iStart)break;
					c=pCmp(m_pFiles+m_pFilesSort[o],m_pFiles+m_pFilesSort[iStart]);
					}
				while(c>=0);

				if(o<u)break;

				iTemp           = m_pFilesSort[o];
				m_pFilesSort[o] = m_pFilesSort[u];
				m_pFilesSort[u] = iTemp;
				}

			iTemp                = m_pFilesSort[iStart];
			m_pFilesSort[iStart] = m_pFilesSort[o     ];
			m_pFilesSort[o     ] = iTemp;


			if(o-1-iStart >= iEnd-u)
				{
				if(iStart+1 < o)
					{
					aMenU[iLevel] = iStart;
					aMenO[iLevel] = o-1;
					iLevel++;
					}

				if(u<iEnd)
					{
					iStart = u;
					continue;
					}
				}
			else{
				if(u<iEnd)
					{
					aMenU[iLevel] = u;
					aMenO[iLevel] = iEnd;
					iLevel++;
					}

				if(iStart+1 < o)
					{
					iEnd = o-1;
					continue;
					}
				}
															// Eine Ebene absteigen
			iLevel--;

			if (iLevel >= 0)								// Noch Ebenen vorhanden
				{
				iStart = aMenU[iLevel];
				iEnd   = aMenO[iLevel];
				continue;
				}

			break;
			}
		}




	m_iModeFlags&=~0x0300;
	m_iModeFlags|= 0x0300&iMode;
	m_iSortMode  = iMode;


return m_iFileCount;
}

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
//	ffnet eine RAR-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CRarFile::ArchOpen(int iLevel)
{
zchar	   *pPath;
RarList	   *pDummy;
RarList	   *pEntry;
auint64		uOffset;
int			iFiles;
int			iMax;
int			iNum;
int			iErr;
int			i;


	Clear();

	m_iSortMode		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_iDoneMode		= 0;
	m_iErrorCode	= CAF_ERROR_OK;


try	{
	if(m_hFile==INVALID_HANDLE_VALUE)						// Aus Speicher ffnen	
		{
		m_sRarOpen.pArcName 	= Z("");
		m_sRarOpen.uOpenMode	= RAR_OM_EXTRACT;
		m_sRarOpen.iMemorySize	= m_iMemLen;
		m_sRarOpen.pMemoryPtr	= m_pMemPtr;
		m_sRarOpen.uOpenFromMem	= 1;
		m_sRarOpen.uOpenResult	= 0;
		}
	else{													// Die Datei direct ffnen
		m_sRarOpen.pArcName 	= m_cFilePath;
		m_sRarOpen.uOpenMode	= RAR_OM_EXTRACT;
		m_sRarOpen.uOpenResult	= 0;
		m_sRarOpen.iMemorySize	= 0;
		m_sRarOpen.pMemoryPtr	= 0;
		m_sRarOpen.uOpenFromMem	= 0;
		}

	RAR_LOCK();

	   m_hRar=RarOpenArchive(&m_sRarOpen);
	if(m_hRar==NULL)
		{
		m_iErrorCode = m_sRarOpen.uOpenResult;
		RAR_UNLOCK();
		return -1;
		}

	m_iSolidMode = RarIsSolidArchive(m_hRar);

	RAR_UNLOCK();

	m_iDoneMax	= m_iFileSize;


//******************** Die Datei Liste laden **********************************


		m_pFiles=new RarList[128];
	if(!m_pFiles)throw CAF_ERROR_MEM;

	iMax	= 128;
	uOffset	= 0;

	RAR_LOCK();


	m_iPassFlag  = 0;
	RarSetCallback(m_hRar,RarCallback,(RAR_LPARAM)this);


	for(iFiles=0;;iFiles++)									// Alle Dateien aus der Liste holen
		{
		if(iFiles>=iMax)
			{
			iNum	= iMax+iMax/2;
			pEntry	= new RarList[iNum];

			if(!pEntry)
				{
				delete m_pFiles;
				m_iFileCount = 0;
				m_pFiles	 = 0;
				throw CAF_ERROR_MEM;
				}

			memcpy(pEntry,m_pFiles,sizeof(RarList)*iFiles);
			pDummy	 = m_pFiles;
			m_pFiles = pEntry;
			iMax     = iNum;
			delete pDummy;
			}

		pEntry  = m_pFiles+iFiles;
Next:
		   iErr = RarReadHeader(m_hRar,pEntry);
		if(iErr==ERAR_END_ARCHIVE)break;
		if(iErr!=ERAR_SUCCESS)throw iErr;

		   iErr = RarProcessEntry(m_hRar,RAR_SKIP,NULL,NULL);
		if(iErr!=ERAR_SUCCESS)throw iErr;

		if(pEntry->uFlags&RHDF_SPLITBEFORE)					// Hat die Datei weitere Teile
			{
			goto Next;
			}

		pPath=pEntry->cFileName;
		pPath[sizeof(pEntry->cFileName)/sizeof(zchar)-2]=0;

		for(;pPath[0];pPath++)								// Backslashes ausgleichen
			{
			if(pPath[0]!='\\')continue;
			pPath[0]='/';
			}

		if(pEntry->cDirectory==0)
			{
			pEntry->uOffset = uOffset;
			uOffset	       += pEntry->uLenFull;
			}
		else{
			if(pPath[-1]!='/')								// Slash bei Pfaden anhngen
				{
				pPath[0] = '/';
				pPath[1] =  0 ;
				pEntry->uNameLen++;
				}
			}

		PROG(CAF_PROG_FIND,iFiles);

		m_iDoneNow = pEntry->iFileHdrPos;
		}


	m_pFilesSort	= new unsigned[iFiles];					// Das Sortier-Array erstellen

	if(!m_pFilesSort)
		{
		delete m_pFiles;
		m_iFileCount = 0;
		m_pFiles	 = 0;
		throw CAF_ERROR_MEM;
		}

	for(i=0;i<iFiles;i++)m_pFilesSort[i]=i;

	if(m_pDecompMem)
		{
		delete m_pDecompMem;
		m_pDecompMem = 0;
		}


	if(m_iSolidMode)										// Beim SOLID-Modus muss die Datei neu geffnet werden
		{
		RarCloseArchive(m_hRar);
		m_hRar = 0;

		   m_hRar=RarOpenArchive(&m_sRarOpen);
		if(m_hRar==NULL)
			{
			m_iErrorCode = m_sRarOpen.uOpenResult;
			RAR_UNLOCK();
			throw m_iErrorCode;
			}
		}

	m_iDoneNow		= m_iFileSize;
	m_iFileCount	= iFiles;
	m_uDecompSize	= uOffset;
	m_uDecompPos	= 0;
	m_iDecompNum	= 0;
	m_uDecompNow	= 0;
	}													
catch(int iError)											// Bekannter Fehler
	{
	if(iError==CAF_ERROR_BREAK && m_iPassFlag)
		{
		iError = CAF_ERROR_PASSWD;
		}

	m_iErrorCode=iError;
	PROGN(CAF_PROG_ERROR,-1);
	Break();
	RAR_UNLOCK();

	return iError;
	}


	RAR_UNLOCK();


	if(m_iModeFlags&CAF_FLAG_SORT)Sort();					// Sortieren ?

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow	 = m_iDoneMax;




return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einer Datei in der Dateiliste
//	pName	: Ist der  Dateiname nach dem gesucht wird
//	Ergibt die Indexnummer der Datei, bzw. -1 wenn die Datei nicht gefunden wurde.
int CRarFile::Find(const zchar *pName)
{
RarList	*pEntry;
unsigned uLen;
int		 i;


	if(*pName=='/' )pName++;
	if(*pName=='\\')pName++;


	m_iFindPos = CAF_ERROR_MISS;
	uLen       = (int)strzlen(pName);

	for(i=0;i<m_iFileCount;i++)								// Eintrag in Kette
		{
		   pEntry=m_pFiles+m_pFilesSort[i];
		if(pEntry->uNameLen!=uLen)continue;

		if(!CmpFileNameN(pName,pEntry->cFileName))
			{
			m_iFindPos=i;
			break;
			}
		}


return m_iFindPos;
}

//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
// Bearbeitung abbrechen ohne Daten zu speichern
void CRarFile::Break()
{

	RarClose();
	m_iOpenMode=CAF_CLOSED;
	Clear();

}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das RarFile und Speichert alle Daten
//	Ergibt;	CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CRarFile::Close()
{


	m_iErrorCode=CAF_ERROR_OK;

	m_cFilePath[0] = 0;

	if(m_iOpenMode==CAF_CLOSED)
		{
		m_iOpenMode =CAF_CLOSED;
		m_iErrorCode=CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,-1);
		return     CAF_ERROR_MISS;
		}

	RarClose();
	m_iOpenMode=CAF_CLOSED;

	ReleaseData();



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		ExtractProc
//*
//*****************************************************************************
//	Extrahiert eine Datei aus dem Archiv ber eine Ausgabefunktion
//	iNum	: Ist die Indexnummer der Datei die geprft werden soll
//	pParam	: Ist ein Parameter der an die Ausgabefunktionbergeben wird
//	pProc	: Ist der Zeiger auf die Ausgabefunktion
//	pInfo	: Hier werden Zeiten und Attribute der Datei gespeichert.
//			  Bei NULL werden diese Werte ignoriert.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_OK		= Wenn alles korrekt war
//				CAF_ERROR_MISS		= Wenn die Datei nicht gefunden wurde
//				CAF_ERROR_PATH		= Pfad ungltig ist.
//				CAF_ERROR_FCREATE	= Wenn Datei nicht erzeugt werden konnte
//				CAF_ERROR_CRC		= Crc32 Fehler
//				CAF_ERROR_DECOMP	= Fehler beim Entkomprimieren
//				CAF_ERROR_IGNORE	= Die Datei wurde bersprungen
//				... siehe Fehler Codes
int	CRarFile::ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo)
{
char			cBuffer[4096];
DWORD			dwBytes;
RarList			sHeader;
RarList		   *pEntry;
unsigned		uPos;
unsigned		uLen;
unsigned		uRet;
LONG			lOff;
int				iErr;



	if(m_iDoneMode<=1)										// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}



	m_iErrorCode = CAF_ERROR_OK;
	m_iPassFlag  = 0;

	try	{
		if( m_iOpenMode==CAF_CLOSED )throw CAF_ERROR_MISS;
		if((unsigned)iNum>=(unsigned)m_iFileCount)throw CAF_ERROR_MISS;

			pEntry=m_pFiles+m_pFilesSort[iNum];
		if(!pEntry)throw CAF_ERROR_MISS;
		if( pEntry->uFlags&4)								// Wird ein Passwort bentigt
			{
			if(m_pPassword)RarSetPasswordEx(m_hRar,m_pPassword);
			else	throw CAF_ERROR_PASSWD;
			}

		if(m_iDoneMode==1)									// Verlaufsanzeige einstellen
			{
			m_iDoneNow  = 0;
			m_iDoneMax  = pEntry->uLenFull;
			}

		m_pErrorText = pEntry->cFileName;

		if(m_iSolidMode)
			{
			m_iDoneMax += pEntry->uLenFull;
			m_iDoneMax += pEntry->uOffset-m_iDoneNow;
			}

		if(!Decompess(iNum))								// Dekomrimieren beim Solid-Modus
			{
			throw CAF_ERROR_FORM;
			}

		if(pEntry->cError)
			{
			throw CAF_ERROR_DECOMP;
			}

//******************** Daten extrahieren **************************************

		if(!m_iSolidMode)
			{
			RAR_LOCK();

			   iErr = RarSeekToFile(m_hRar,pEntry);
			if(iErr)
				{
				RAR_UNLOCK();
				throw CAF_ERROR_LOGIC;
				}

			   iErr = RarReadHeader(m_hRar,&sHeader);
			if(iErr)
				{
				RAR_UNLOCK();
				pEntry->cError = 1;
				throw iErr;
				}

			if(pEntry->uLenFull!=sHeader.uLenFull){pEntry->cError=1;RAR_UNLOCK();throw CAF_ERROR_LOGIC;}
			if(pEntry->uLenComp!=sHeader.uLenComp){pEntry->cError=1;RAR_UNLOCK();throw CAF_ERROR_LOGIC;}
			if(pEntry->uFileCRC!=sHeader.uFileCRC){pEntry->cError=1;RAR_UNLOCK();throw CAF_ERROR_LOGIC;}

			m_pProc  = pProc;
			m_pParam = pParam;
			iErr     = RarProcessProc(m_hRar,CountProc,this);

			RAR_UNLOCK();

			if(iErr)
				{
				pEntry->cError = 1;
				throw iErr;
				}

			if(pProc(pParam,NULL,0)!=0)						// Ende anzeigen
				{
				PROGN(CAF_PROG_ERROR,iNum);
				throw CAF_ERROR_FWRITE;
				}

			if(pInfo)										// Dateidaten speichern
				{
				pInfo->dwAttr		=(pEntry->cDirectory)? SYS_DIR|SYS_READ:SYS_READ;
				pInfo->dwAttr	   |=(pEntry->uLenFull>>(32-SYS_HIGHSHIFT))&SYS_HIGHMASK;
				pInfo->dwSize		=(unsigned)(pEntry->uLenFull  );
				pInfo->dwTimeWrite	=(unsigned)(pEntry->iTime!=-1)? pEntry->iTime:0;
				pInfo->dwTimeCreate	= 0;
				pInfo->dwTimeAccess	= 0;
				pInfo->cName[0]		= 0;
				}

			goto Exit;
			}


//******************** Entkomprimieren einer SOLID-Datei **********************

		if(m_pDecompMem)									// Daten speichern in Speicher
			{
			for(uPos=0;uPos<pEntry->uLenFull;uPos+=uLen)
				{
				uLen = 4096;

				if((auint64)uLen+uPos>pEntry->uLenFull)
					{
					uLen = (unsigned)(pEntry->uLenFull-uPos);
					}

				   uRet = pProc(pParam,m_pDecompMem+pEntry->uOffset+uPos,uLen);
				if(uRet!=uLen)
					{
					throw CAF_ERROR_FWRITE;
					}

				m_iDoneNow += uLen;
				}
			}
		else{												// Datenspeichern in Datei
			lOff = (LONG)(pEntry->uOffset>>32);

			if(SetFilePointer(m_hDecompFile,(DWORD)pEntry->uOffset,&lOff,FILE_BEGIN)==0xFFFFFFFF && GetLastError())
				{
				throw CAF_ERROR_FREAD;
				}

			for(uPos=0;uPos<pEntry->uLenFull;uPos+=uLen)
				{
				uLen = 4096;

				if((auint64)uLen+uPos>pEntry->uLenFull)
					{
					uLen = (unsigned)(pEntry->uLenFull-uPos);
					}

				if(!ReadFile(m_hDecompFile,cBuffer,uLen,&dwBytes,NULL) || uLen!=dwBytes)
					{
					throw CAF_ERROR_FREAD;
					}

				   uRet = pProc(pParam,cBuffer,uLen);
				if(uRet!=uLen)
					{
					throw CAF_ERROR_FWRITE;
					}

				m_iDoneNow += uLen;
				}
			}


		for(uPos=0;uPos<pEntry->uLenFull;uPos+=uLen)		// Daten speichern
			{
			uLen = 4096;

			if((auint64)uLen+uPos>pEntry->uLenFull)
				{
				uLen = (unsigned)(pEntry->uLenFull-uPos);
				}

			   uRet = pProc(pParam,m_pDecompMem+pEntry->uOffset+uPos,uLen);
			if(uRet!=uLen)
				{
				throw CAF_ERROR_FWRITE;
				}

			m_iDoneNow += uLen;
			}
				 

		if(pProc(pParam,NULL,0)!=0)							// Ende anzeigen
			{
			PROGN(CAF_PROG_ERROR,iNum);
			throw CAF_ERROR_FWRITE;
			}


		if(pInfo)											// Dateidaten speichern
			{
			pInfo->dwAttr		=(pEntry->cDirectory)? SYS_DIR|SYS_READ:SYS_READ;
			pInfo->dwAttr	   |=(pEntry->uLenFull>>(32-SYS_HIGHSHIFT))&SYS_HIGHMASK;
			pInfo->dwSize		=(unsigned)(pEntry->uLenFull  );
			pInfo->dwTimeWrite	=(unsigned)(pEntry->iTime!=-1)? pEntry->iTime:0;
			pInfo->dwTimeCreate	= 0;
			pInfo->dwTimeAccess	= 0;
			pInfo->cName[0]		= 0;
			}
		}
	catch(int iError)
		{
		if(iError==CAF_ERROR_BREAK && m_iPassFlag)
			{
			iError = CAF_ERROR_PASSWD;
			}

		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}


Exit:

	if(m_iDoneMode<=1)										// Die Verlaufsanzeige zurcksetzen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}



return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Test
//*
//*****************************************************************************
//	Teste eine Komprimierte Datei
//	Ergibt:	CAF_ERROR_OK		: Wenn alles korrekt war
//			CAF_ERROR_MISS		: Wenn die Datei nicht gefunden wurde
//			CAF_ERROR_CRC		: Crc32 Fehler
//			CAF_ERROR_DECOMP	: Fehler beim Entkomprimieren
//			CAF_ERROR_EOF		: Bei vorzeitigem Leseende
//		... siehe Fehler Codes
int	CRarFile::Test(int iNum)
{
RarList		sHeader;
RarList    *pEntry;
int			iErr;


	m_iErrorCode = CAF_ERROR_OK;
	m_iPassFlag  = 0;

	try	{
		PROG(CAF_PROG_TEST,iNum);

		if(m_iOpenMode==CAF_CLOSED    )throw CAF_ERROR_MISS;
		if((unsigned)iNum>=(unsigned)m_iFileCount)throw CAF_ERROR_MISS;


			pEntry=m_pFiles+m_pFilesSort[iNum];;
		if(!pEntry	        )throw  CAF_ERROR_MISS;
		if(!pEntry->uLenFull)return CAF_ERROR_OK;
		if( pEntry->uFlags&4)								// Wird ein Passwort bentigt
			{
			if(m_pPassword)RarSetPasswordEx(m_hRar,m_pPassword);
			else	throw CAF_ERROR_PASSWD;
			}


		m_iDoneMode  = 1;									// Verlaufsanzeige einstellen
		m_iDoneNow   = 0;
		m_iDoneMax   = pEntry->uLenFull;
		m_pErrorFile = pEntry->cFileName;


//******************** Daten extrahieren **************************************


		if(!m_iSolidMode)
			{
			RAR_LOCK();

			   iErr = RarSeekToFile(m_hRar,pEntry);
			if(iErr)
				{
				RAR_UNLOCK();
				throw CAF_ERROR_LOGIC;
				}

			   iErr = RarReadHeader(m_hRar,&sHeader);
			if(iErr)
				{
				RAR_UNLOCK();
				pEntry->cError = 1;
				throw iErr;
				}

			if(pEntry->uLenFull!=sHeader.uLenFull){pEntry->cError=1;RAR_UNLOCK();throw CAF_ERROR_LOGIC;}
			if(pEntry->uLenComp!=sHeader.uLenComp){pEntry->cError=1;RAR_UNLOCK();throw CAF_ERROR_LOGIC;}
			if(pEntry->uFileCRC!=sHeader.uFileCRC){pEntry->cError=1;RAR_UNLOCK();throw CAF_ERROR_LOGIC;}

			RarSetProcessDataProc(m_hRar,CountWrite,this);
			iErr     = RarProcessTestEx(m_hRar);

			RAR_UNLOCK();

			if(iErr)
				{
				pEntry->cError = 1;
				throw iErr;
				}

			return m_iErrorCode;
			}

		if(!Decompess(iNum))								// Datei entkomprimieren
			{
			throw CAF_ERROR_FORM;
			}

		if(pEntry->cError==2)
			{
			throw CAF_ERROR_CRC;
			}

		if(pEntry->cError)
			{
			throw CAF_ERROR_DECOMP;
			}
		}
	catch(int iError)
		{
		if(iError==CAF_ERROR_BREAK && m_iPassFlag)
			{
			iError = CAF_ERROR_PASSWD;
			}

		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}


	m_iDoneMode = 0;
	m_iDoneNow  = m_iDoneMax;



return m_iErrorCode;
}


//*****************************************************************************
//*
//*		Count
//*
//*****************************************************************************
//	Ergibt die Anzahl der Eintrge in der Dateiliste
int CRarFile::Count()
{
	return m_iFileCount;
}


//*****************************************************************************
//*
//*		CreateTempFile
//*
//*****************************************************************************
//	Erzeugt eine temorre TAR-Datei
//	pBuffer		: Zeiger auf den Puffer fr den Dateinamen
//	iSize		: Gre des Puffers in Zeichen
//	Ergibt das Handle auf die Datei
HANDLE CRarFile::CreateTempFile(zchar *pBuffer,int iSize)
{
HANDLE	hTemp;
zchar	cPath[MAX_PATH];


	if(m_pTempPath)
		{
		GetTempFileName(m_pTempPath,Z("rar"),0,pBuffer);
		}
	else{
		GetTempPath(iSize-32,cPath);
		GetTempFileName(cPath,Z("rar"),0,pBuffer);
		}


	   hTemp=CreateFile(pBuffer,GENERIC_READ|GENERIC_WRITE,0,0,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
	if(hTemp==INVALID_HANDLE_VALUE)
		{
		throw CAF_ERROR_CREATE;
		}

return hTemp;
}

//*****************************************************************************
//*
//*		CountProc
//*
//*****************************************************************************
int __cdecl CRarFile::CountProc(void *pParam,char *pData,unsigned uLen)
{
CRarFile	   *pThis=(CRarFile*)pParam;
unsigned		uPos;
unsigned		uNum;
unsigned		uRet;


	for(uPos=0;uPos<uLen;uPos+=uNum)
		{
		   uNum = uLen-uPos;
		if(uNum>4096)uNum=4096;

		   uRet = pThis->m_pProc(pThis->m_pParam,pData+uPos,uNum);
		if(uRet!=uNum)
			{
			if((int)uRet>0)
				{
				pThis->m_iDoneNow += uRet;
				uPos              += uRet;
				}

			return uRet;
			}

		pThis->m_iDoneNow += uRet;
		}


return uPos;
}

//*****************************************************************************
//*
//*		CountWrite
//*
//*****************************************************************************
int __cdecl CRarFile::CountWrite(void *pParam,char *pData,unsigned uLen)
{
CRarFile	   *pThis=(CRarFile*)pParam;


	pThis->m_iDoneNow   += uLen;


return uLen;
}

//*****************************************************************************
//*
//*		WriteTempMem
//*
//*****************************************************************************
//	Speichert Daten im temporren Dekompressspeicher
//	pParam		: Zeiger auf die CRarFile Klasse
//	pData		: Zeiger auf die zu speichernten Daten
//	uLen		: Lnge der Daten in Byte
//	Ergibt die Anzahl der gespeicherten Bytes
int __cdecl CRarFile::WriteTempMem(void *pParam,char *pData,unsigned uLen)
{
CRarFile	   *pThis=(CRarFile*)pParam;


	if(uLen>0x7FFFFFFFF)return 0;

	if(pThis->m_uDecompPos>=pThis->m_uDecompSize)
		{
		pThis->m_uDecompPos = pThis->m_uDecompSize+1;
		throw CAF_ERROR_DECOMP;
		}

	if(pThis->m_uDecompPos+uLen>pThis->m_uDecompSize)
		{
		uLen = (unsigned)(pThis->m_uDecompSize-pThis->m_uDecompPos);
		memcpy(pThis->m_pDecompMem+pThis->m_uDecompPos,pData,uLen);
		pThis->m_uDecompPos  = pThis->m_uDecompSize+1;
		pThis->m_iDoneNow   += uLen;
		throw CAF_ERROR_DECOMP;
		}


	memcpy(pThis->m_pDecompMem+pThis->m_uDecompPos,pData,uLen);
	pThis->m_uDecompPos += uLen;
	pThis->m_iDoneNow   += uLen;


return uLen;
}

//*****************************************************************************
//*
//*		WriteTempFile
//*
//*****************************************************************************
//	Speichert Daten in einer temporren Dekompressdatei
//	pParam		: Zeiger auf die CRarFile Klasse
//	pData		: Zeiger auf die zu speichernten Daten
//	uLen		: Lnge der Daten in Byte
//	Ergibt die Anzahl der gespeicherten Bytes
int __cdecl CRarFile::WriteTempFile(void *pParam,char *pData,unsigned uLen)
{
CRarFile	   *pThis=(CRarFile*)pParam;
DWORD			dwBytes;


	if(uLen>0x7FFFFFFFF)return 0;

	if(pThis->m_uDecompPos>=pThis->m_uDecompSize)
		{
		pThis->m_uDecompPos = pThis->m_uDecompSize+1;
		throw CAF_ERROR_DECOMP;
		}

	if(pThis->m_uDecompPos+uLen>pThis->m_uDecompSize)
		{
		uLen = (unsigned)(pThis->m_uDecompSize-pThis->m_uDecompPos);

		if(!WriteFile(pThis->m_hDecompFile,pData,uLen,&dwBytes,NULL))
			{
			throw CAF_ERROR_FWRITE;
			}

		pThis->m_uDecompPos  = pThis->m_uDecompSize+1;
		pThis->m_iDoneNow   += dwBytes;

		throw CAF_ERROR_DECOMP;
		}


	if(!WriteFile(pThis->m_hDecompFile,pData,uLen,&dwBytes,NULL))
		{
		throw CAF_ERROR_FWRITE;
		}

	pThis->m_uDecompPos += dwBytes;
	pThis->m_iDoneNow   += dwBytes;


return dwBytes;
}

//*****************************************************************************
//*
//*		Decompess
//*
//*****************************************************************************
//	Alle Dateien in einen Speicher bzw. eine Datei entkomprimiern.
//	iNum		: Ist die Datei bis einschlislich der entkomprimiert wird
//	Ergibt 1 wenn ok bzw. 0 bei einem Fehler
int CRarFile::Decompess(int iNum)
{
RarList		   *pEntry;
RarList			sHeader;
LONG			lOff;
int				iErr;


	if(m_iOpenMode!=CAF_READ)return 0;
	if(m_iFileCount<=0      )return 0;
	if(m_iSolidMode==0     	)return 1;


	if(iNum>=m_iFileCount)throw CAF_ERROR_MISS;

	RAR_LOCK();											// Die Dateien auflisten

	iNum = m_pFilesSort[iNum];


	if(m_uDecompSize<0x800000)							// Teporrer Speicher
		{
		if(!m_pDecompMem)
			{
				m_pDecompMem = new char[(unsigned)m_uDecompSize+16];
			if(!m_pDecompMem)
				{
				RAR_UNLOCK();
				throw CAF_ERROR_MEM;
				}
			}

		while(m_iDecompNum<=iNum)
			{
			pEntry = m_pFiles+m_iDecompNum;

			for(;;)										// Nchsten Datei-Header einlesen
				{
				   iErr = RarReadHeader(m_hRar,&sHeader);
				if(iErr)
					{
					RAR_UNLOCK();
					pEntry->cError = 1;
					throw iErr;
					}

				if(!(sHeader.uFlags&RHDF_SPLITBEFORE))	// Hat die Datei weitere Teile
					{
					break;
					}
				}

			pEntry = m_pFiles+m_iDecompNum;

			if(pEntry->uLenFull!=sHeader.uLenFull){pEntry->cError=1;throw CAF_ERROR_LOGIC;}
			if(pEntry->uLenComp!=sHeader.uLenComp){pEntry->cError=1;throw CAF_ERROR_LOGIC;}
			if(pEntry->uFileCRC!=sHeader.uFileCRC){pEntry->cError=1;throw CAF_ERROR_LOGIC;}
			if(pEntry->cDirectory){m_iDecompNum++;continue;}

			m_uDecompPos = pEntry->uOffset;

			   iErr = RarProcessProc(m_hRar,WriteTempMem,this);
			if(iErr)
				{
				RAR_UNLOCK();
				pEntry->cError = 1;
				throw iErr;
				}

			if(m_uDecompPos!=pEntry->uOffset+pEntry->uLenFull)
				{
				RAR_UNLOCK();
				pEntry->cError = 1;
				throw CAF_ERROR_DECOMP;
				}

			if(m_uDecompNow<m_uDecompPos)
				{
				m_uDecompNow=m_uDecompPos;
				}

			m_iDecompNum += 1;
			}
		}
	else{												// Temporre Datei
		if(m_hDecompFile==INVALID_HANDLE_VALUE)
			{
			m_hDecompFile=CreateTempFile(m_cDecompFile,sizeof(m_cDecompFile));
			}

		while(m_iDecompNum<=iNum)
			{
			pEntry = m_pFiles+m_iDecompNum;

			   iErr = RarReadHeader(m_hRar,&sHeader);
			if(iErr)
				{
				RAR_UNLOCK();
				pEntry->cError = 1;
				throw iErr;
				}

			if(pEntry->uLenFull!=sHeader.uLenFull){pEntry->cError=1;throw CAF_ERROR_LOGIC;}
			if(pEntry->uLenComp!=sHeader.uLenComp){pEntry->cError=1;throw CAF_ERROR_LOGIC;}
			if(pEntry->uFileCRC!=sHeader.uFileCRC){pEntry->cError=1;throw CAF_ERROR_LOGIC;}
			if(pEntry->cDirectory){m_iDecompNum++;continue;}

			lOff = (LONG)(pEntry->uOffset>>32);

			if(SetFilePointer(m_hDecompFile,(DWORD)pEntry->uOffset,&lOff,FILE_BEGIN)==0xFFFFFFFF && GetLastError())
				{
				RAR_UNLOCK();
				throw CAF_ERROR_FWRITE;
				}

			m_uDecompPos = pEntry->uOffset;

			   iErr = RarProcessProc(m_hRar,WriteTempFile,this);
			if(iErr)
				{
				RAR_UNLOCK();
				pEntry->cError = 1;
				throw iErr;
				}

			if(SetFilePointer(m_hDecompFile,0,0,FILE_CURRENT)!=pEntry->uOffset+pEntry->uLenFull)
				{
				RAR_UNLOCK();
				pEntry->cError = 1;
				throw CAF_ERROR_DECOMP;
				}

			if(m_uDecompNow<m_uDecompPos)
				{
				m_uDecompNow=m_uDecompPos;
				}

			m_iDecompNum += 1;
			}
		}


	RAR_UNLOCK();





return 1;
}

//*****************************************************************************
//*
//*		SetDoneMode
//*
//*****************************************************************************
//	Den Modus fr die Verlaufsanzeige setzen.
//	iMode	: Ist der neue Modus
//			    0 = Zurck zu Normal (Ende)
//				1 = Begin von Extract (einzelene Datei)
//				2 = Begin von ExtractAll
//	Ergibt den vorherigen Modus.
int CRarFile::SetDoneMode(int iMode)
{
int		iRet;

	
	iRet = m_iDoneMode;
	
	switch(iMode)
		{
	case 0:													// Zurck zu Normal (Ende)

		if(m_iDoneMode==1 || m_iDoneMode==2)
		if(m_iErrorCode==CAF_ERROR_OK)
			{
			m_iDoneNow  = m_iDoneMax;
			}

		m_iDoneMode	 = 0;
		break;

	case 1:													// Begin von Extract (einzelene Datei)

		if(iRet==2)return iRet;
		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 0;
		break;

	case 2:													// Begin von ExtractAll

		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 1;
		break;
		}


return iRet;
}


//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CRarFile::Done(aint64 *pMax)
{
aint64	iRet;


	switch(m_iDoneMode)
		{
	case 0:
	case 1:
	case 2:	iRet = m_iDoneNow;		break;
	default:iRet = 0;				break;
		}


	if(iRet>m_iDoneMax)iRet=m_iDoneMax;
	if(pMax)*pMax=m_iDoneMax;


return iRet;
}


//*****************************************************************************
//*
//*		GetUserPtr
//*
//*****************************************************************************
//	User-Pointer abfragen
void *CRarFile::GetUserPtr(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFileCount)return 0;

return m_pFiles[iNum].pUserPtr;
}

//*****************************************************************************
//*
//*		SetUserPtr
//*
//*****************************************************************************
//	User-Pointer einstellen
void CRarFile::SetUserPtr(int iNum,void *pData)
{

	if((unsigned)iNum>=(unsigned)m_iFileCount)return;

	m_pFiles[iNum].pUserPtr = pData;

}

