//*****************************************************************************
//*
//*
//*		RpmFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//
//	Klasse fr den Zugriff auf RPM-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen
//
#include	<stdio.h>
#include	<string.h>
#include	"RpmFile.h"
#include	"BitFunctions.h"
#include	"Codecs/Huffman.h"
#include	"Codecs/XzClass.h"
#include	"Codecs/BzipClass.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		wcslen
#define 	strzncpy	wcsncpy
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		strlen
#define 	strzncpy	strncpy
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif


#define PROG(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))throw CAF_ERROR_BREAK;	\
	}
#define PROGR(iMode,iNum,Ret)						\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))return Ret;				\
	}
#define PROGN(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	Progress(iMode,iNum);							\
	}


typedef	struct
	{
	unsigned char		ucMagic[4];					// Immer 0xED,0xAB,0xEE,0xDB
	unsigned char		ucMajorVersion;				// Erste  Versionsnummer
	unsigned char		ucMinorVersion;				// Zweite Versionsnummer
	unsigned char		ucType[2];					// Typ der gespeicherten Daten
	unsigned char		ucPlatform[2];				// ID fr System (0=keine 1=x86)
	char				cName[66];					// Name des Paketes
	unsigned char		ucOperatingSytem;			// OS fr Daten
	unsigned char		ucSignature;				// Versin der Signatur (1...5)
	unsigned char		ucReserved[18];				// Unbenutzt
	}RpmHeader;

typedef	struct
	{
	unsigned char		ucMagic     [3];			// Immer 0x8E,0xAD,0xE8
	unsigned char		ucVersion;					// Vertsionsnummer (ist 1)
	unsigned char		ucReserved  [4];			// Unbenutzt
	unsigned char		ucIndexCount[4];			// Anzahl der folgenden Indizes im Big-Endian-Format
	unsigned char		ucStoreSize [4];			// Gre der gespeicherten Daten im Big-Endian-Format (ab letztdem RpmIndexHeader)
	}RpmSubHeader;

typedef	struct
	{
	unsigned char		ucTag   [4];				// Context der Daten
	unsigned char		ucType  [4];				// Typ der Daten (0=none 1=char 2=int8 3=int16 4=int32 5=int64 6=string 7=binary)
	unsigned char		ucOffset[4];				// Offset im Datenbereich (ab letztdem RpmIndexHeader)
	unsigned char		ucCount [4];				// Anzahl der Eintrge (nicht in Byte sontern in Eintrgen)
	}RpmIndexHeader;

typedef struct
	{
	unsigned char		ucIdenifier1;				// Immer auf 0x1F
	unsigned char		ucIdenifier2;				// Immer auf 0x8B
	unsigned char		ucCompressMode;				// Komprimierungsmodus (0..8, 8=max)
	unsigned char		ucFlags;					// Falgs (siehe TGZ_FLAG_??=)
	unsigned char		ucTime[4];					// Unixzeit der Datei (litle endian)
	unsigned char		ucExtraFlags;				// Erweiterte Flags
	unsigned char		ucOpSystem;					// Bit1=beste Bit3=schnellste Kompression
	}GzipHeader;

#define 	TGZ_FLAG_FTEXT			0x01			// Datei ist ein ASCII Text
#define 	TGZ_FLAG_FHCRC			0x02			// Eine CRC16 Summe ist vorhanden
#define 	TGZ_FLAG_FEXTRA			0x04			// Extrafelter sind vorhanden
#define 	TGZ_FLAG_FNAME			0x08			// Ein Dateiname ist angehngt
#define 	TGZ_FLAG_FCOMMENT		0x10			// Ein Commentar ist angehngt

#define 	RPM_HEADER_SIZE			96
#define 	RPM_SUBHEADER_SIZE		16
#define 	RPM_INDEXHEADER_SIZE	16


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CRpmFile::CRpmFile()
{

	m_pFiles		=  0;
	m_iFilesNum		=  0;
	m_iDoneNow		=  0;
	m_iDoneMax		=  0;
	m_iDoneMode		=  0;
	m_iDecompDone	=  0;
	m_pDecompMem	=  0;
	m_uDecompSize	=  0;
	m_uDecompMode	=  5;
	m_cHasCompress	=  0;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CRpmFile::~CRpmFile()
{

	if(m_iOpenMode!=CAF_CLOSED)Close();

	Clear();
}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Datenfelder und setzt alle Variabeln zurck
void CRpmFile::Clear()
{

	if(m_pFiles)
		{
		while(m_iFilesNum>0)
			{
			m_iFilesNum--;
			delete m_pFiles[m_iFilesNum].pName;
			}

		delete m_pFiles;

		m_pFiles=NULL;
		}

	m_iFilesNum		= 0;
	m_iDecompDone	= 0;
	m_uDecompSize	= 0;
	m_uDecompMode	= 5;

	if(m_pDecompMem)
		{
		delete m_pDecompMem;
		m_pDecompMem = NULL;
		}


}

//*****************************************************************************
//*
//*		CmpEx
//*
//*****************************************************************************
int __cdecl CRpmFile::CmpEx(const void *pPos1,const void *pPos2)
{
RpmItem	*pPtr1,*pPtr2;


	pPtr1 = (RpmItem*)pPos1;
	pPtr2 = (RpmItem*)pPos2;


return strvicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpName
//*
//*****************************************************************************
int __cdecl CRpmFile::CmpName(const void *pPos1,const void *pPos2)
{
RpmItem	*pPtr1,*pPtr2;


	pPtr1 = (RpmItem*)pPos1;
	pPtr2 = (RpmItem*)pPos2;


return strzicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpLen
//*
//*****************************************************************************
int __cdecl CRpmFile::CmpLen(const void *pPos1,const void *pPos2)
{
RpmItem	*pPtr1,*pPtr2;


	pPtr1 = (RpmItem*)pPos1;
	pPtr2 = (RpmItem*)pPos2;


return pPtr1->uLenFull-pPtr2->uLenFull;
}

//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CRpmFile::CmpPos(const void *pPos1,const void *pPos2)
{
RpmItem	*pPtr1,*pPtr2;


	pPtr1 = (RpmItem*)pPos1;
	pPtr2 = (RpmItem*)pPos2;


return (int)(pPtr1-pPtr2);
}

//*****************************************************************************
//*
//*		Len
//*
//*****************************************************************************
//	Ergibt die Lnge der Datei mit der Indexnummer iNum
//	Ist iMode==0 wird die originale    Lnge zurckgegeben
//	Ist iMode!=0 wird die komprimierte Lnge zurckgegeben
aint64 CRpmFile::Len(int iNum,int iMode)
{
RpmItem	*pEntry;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;
	pEntry = m_pFiles+iNum;

return (iMode)? pEntry->uLenComp:pEntry->uLenFull;
}

//*****************************************************************************
//*
//*		SetFail
//*
//*****************************************************************************
//	Setzt das Defekt-Flag und ergibt den alten Wert.
int CRpmFile::SetFail(int iNum,int iFail)
{
int		iOld;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

	iOld = m_pFiles[iNum].cError;
	m_pFiles[iNum].cError = iFail;

return iOld;
}

//*****************************************************************************
//*
//*		IsFail
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit iNum beschdigt ist sonst 0
int CRpmFile::IsFail(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

return m_pFiles[iNum].cError;
}

//*****************************************************************************
//*
//*		IsDir
//*
//*****************************************************************************
// Ergibt die 1 wenn die Datei mit iNum ein Verzeichnis ist sonst 0
int CRpmFile::IsDir(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return (m_pFiles[iNum].cDirectory)? 1:0;
}

//*****************************************************************************
//*
//*		IsOk
//*
//*****************************************************************************
// Ergibt:	2	Wenn die Datei vollstntig Entzipt wurde
//          1	Wenn die Datei schon vorhanden war
//			0   Bei Fehlern
int CRpmFile::IsOk(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum].cDecompOk;
}

//*****************************************************************************
//*
//*		IsReadOnly
//*
//*****************************************************************************
//	Ergibt 1 wenn die Datei schreibgeschtzt ist.
int CRpmFile::IsReadOnly(int iNum)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return (m_pFiles[iNum].iAttrib&0200)? 0:1;
}

//*****************************************************************************
//*
//*		Stats
//*
//*****************************************************************************
//	Die Zustandbits abfragen
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//	iDel		: Welche Bits sollen gelscht werden
//	Ergibt einen Wert mit veschiedenen Bits:
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//					CAF_STAT_DELETE		0x0004	Die Datei wurde aus dem Verzeichnis gelscht
//					CAF_STAT_WRITE		0x0008	Die Datei muss ins Archiv geschrieben werden
//					CAF_STAT_DIR		0x0010	Die Datei ist ein Verzeichniss
//					CAF_STAT_READONLY	0x0020	Die Datei ist schreibgeschtzt
//					CAF_STAT_SAVED		0x0040	Die Datei ist im Archiv gespeichert
int CRpmFile::Stats(int iNum)
{
RpmItem	   *pEntry;
int			iBits;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

			pEntry=m_pFiles+iNum;
	iBits = pEntry->cDecompOk&3;

						    iBits|=CAF_STAT_SAVED;
	if(pEntry->iAttrib&0200)iBits|=CAF_STAT_READONLY;
	if(pEntry->cDirectory  )iBits|=CAF_STAT_DIR;
	

return iBits;	
}

//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
// Ergibt die Zeit der letzten nderung einer Datei
// bzw.	CAF_ERROR_MISS wenn iNum/Datei nicht gefunden wurde.
time_t CRpmFile::Time(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return CAF_ERROR_MISS;

return m_pFiles[iNum].iTime;
}

time_t CRpmFile::Time(int iNum,int Mode)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}

	switch(Mode)
		{
	case CAF_TIME_W:
	case CAF_TIME_C:
	case CAF_TIME_A: return m_pFiles[iNum].iTime;
		}


return CAF_ERROR_MISS;
}


//*****************************************************************************
//*
//*		Attribute
//*
//*****************************************************************************
//	UNIX-Dateiattribute auslesen
//	iNum		: Ist die Nummer der Datei
//	Ergibt die Attribute bzw. -1 bei einem Fehler
//					0x00000001	= Execute Andere
//					0x00000002	= Write   Andere
//					0x00000004	= Read    Andere
//					0x00000080	= Execute Gruppe
//					0x00000100	= Write   Gruppe
//					0x00000200	= Read    Gruppe
//					0x00000400	= Execute Besitzer
//					0x00000800	= Write   Besitzer
//					0x00001000	= Read    Besitzer
//					0x00004000	= Verzeichnis
//					0x80000000	= Die Attribute sind nachgebiltet
int CRpmFile::Attribute(int iNum)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum].iAttrib;
}

//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CRpmFile::Name(int iNum)
{

	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex
		{
		return m_pFiles[iNum].pName;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum)								// Datei die gerade eingefgt wird
		{
		return m_pInsertFile;
		}


return NULL;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei samt Lnge.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CRpmFile::Name(int iNum,int &iLen)
{


	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex	
		{
		iLen = m_pFiles[iNum].uNameLen;
		return m_pFiles[iNum].pName;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		iLen = strzlen(m_cFilePath);					
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum)								// Datei die gerade eingefgt wird
		{
		iLen = (m_pInsertFile)? strzlen(m_pInsertFile):0;
		return  m_pInsertFile;
		}

	iLen = 0;


return NULL;
}


//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Liste in m_pFiles
//  iMode:	CAF_FLAG_SORT			0x0400	Nach Namen  sortieren
//			CAF_FLAG_SORTLEN	    0x0500	Nach Lngen sortieren
//			CAF_FLAG_SORTPOS	    0x0600	Nach Offset sortieren (intern)
//			CAF_FLAG_SORTEX			0x0700	Nach Namen mit Zahlen Bercksichtigung
int	CRpmFile::Sort(int iMode)
{

		iMode&=0x0700;
	if(!iMode)iMode=m_iModeFlags&0x0700;
	iMode |= 0x0400;

	PROGR(CAF_PROG_SORT,-1,0);


	if(!m_pFiles)return 0;

	if(iMode==m_iSortMode)								// Ist die Liste bereits sortiert
		{
		return m_iFilesNum;
		}

	if(m_iFilesNum)
		{
		switch(iMode)
			{
		case    CAF_FLAG_SORTEX:	qsort(m_pFiles,m_iFilesNum,sizeof(RpmItem),CmpEx  );break;
		case    CAF_FLAG_SORTLEN:	qsort(m_pFiles,m_iFilesNum,sizeof(RpmItem),CmpLen );break;
		case    CAF_FLAG_SORTPOS:	qsort(m_pFiles,m_iFilesNum,sizeof(RpmItem),CmpPos );break;
		default:					qsort(m_pFiles,m_iFilesNum,sizeof(RpmItem),CmpName);break;
			}
		}

	m_iModeFlags&=~0x0300;
	m_iModeFlags|= 0x0300&iMode;
	m_iSortMode  = iMode;


return m_iFilesNum;
}

//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine RPM-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CRpmFile::ArchOpen(int iLevel)
{
RpmHeader			sHeader;
RpmSubHeader		sItem;
RpmIndexHeader		sIndex;
RpmItem			   *pEntry;
char			   *pData=0;
char			   *pFile;
char			   *pDir;
unsigned			uChar;
unsigned			uIndex;
unsigned			uIndexNow;
unsigned			uIndexCount;
unsigned			uStoreSize;
unsigned			uDirNamesNum;
unsigned			uDirNamesOff;
unsigned			uDirIndexNum;
unsigned			uDirIndexOff;
unsigned			uFileModeNum;
unsigned			uFileModeOff;
unsigned			uFileSizeNum;
unsigned			uFileSizeOff;
unsigned			uFileFlagsNum;
unsigned			uFileFlagsOff;
unsigned			uFileTimesNum;
unsigned			uFileTimesOff;
unsigned			uFileNamesOff;
unsigned			uFileNamesNum;
unsigned			uFileNamesTxt;
unsigned			uFileNamesCnt;
unsigned			uDecompTypeOff;
unsigned			uDecompModeOff;
unsigned			uOffset;
unsigned			uCount;
unsigned			uType;
unsigned			uTag;
unsigned			uAdd;
int					iFilePos;
int					iLenN;
int					iLenP;
int					iPos;
int					i;


	Clear();

	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_iDoneMode		= 0;
	m_iErrorCode	= CAF_ERROR_OK;


try	{
	m_iDoneMax	= m_iFileSize;


	iFilePos = MainRead(&sHeader,RPM_HEADER_SIZE);

	if(sHeader.ucMagic[0]!=0xED || sHeader.ucMagic[1]!=0xAB || sHeader.ucMagic[2]!=0xEE || sHeader.ucMagic[3]!=0xDB)
		{
		throw CAF_ERROR_FORM;
		}


	iFilePos += MainRead(&sItem,RPM_SUBHEADER_SIZE);

	if(sItem.ucMagic[0]!=0x8E || sItem.ucMagic[1]!=0xAD || sItem.ucMagic[2]!=0xE8 || sItem.ucVersion!=1)
		{
		throw CAF_ERROR_FORM;
		}


	uIndexCount = btol_dw(*(int*)sItem.ucIndexCount);
	uStoreSize	= btol_dw(*(int*)sItem.ucStoreSize);
	uAdd		= (uIndexCount*RPM_INDEXHEADER_SIZE+uStoreSize+7)&~7;

	MainSeek(uAdd,SEEK_CUR);

	iFilePos += uAdd;
	iFilePos += MainRead(&sItem,RPM_SUBHEADER_SIZE);

	if(sItem.ucMagic[0]!=0x8E || sItem.ucMagic[1]!=0xAD || sItem.ucMagic[2]!=0xE8 || sItem.ucVersion!=1)
		{
		throw CAF_ERROR_FORM;
		}

	uIndexCount = btol_dw(*(int*)sItem.ucIndexCount);
	uStoreSize	= btol_dw(*(int*)sItem.ucStoreSize);

	uAdd			= 0;
	uDirNamesNum	= 0;
	uDirNamesOff	= 0;
	uDirIndexNum	= 0;
	uDirIndexOff	= 0;
	uFileModeNum	= 0;
	uFileModeOff	= 0;
	uFileSizeNum	= 0;
	uFileSizeOff	= 0;
	uFileFlagsNum	= 0;
	uFileFlagsOff	= 0;
	uFileTimesNum	= 0;
	uFileTimesOff	= 0;
	uFileNamesNum	= 0;
	uFileNamesOff	= 0;
	uFileNamesTxt	= 0;
	uFileNamesCnt	= 0;
	uDecompTypeOff	= 0;
	uDecompModeOff	= 0;
	m_uDecompMode	= 5;

	for(;uIndexCount>0;uIndexCount--)
		{
		iFilePos += MainRead(&sIndex,RPM_INDEXHEADER_SIZE);

		uTag    = btol_dw(*(int*)sIndex.ucTag   );
		uType   = btol_dw(*(int*)sIndex.ucType  );
		uCount  = btol_dw(*(int*)sIndex.ucCount );
		uOffset	= btol_dw(*(int*)sIndex.ucOffset);


		switch(uTag)
			{
		case 1118:	if(uType!=8)break;
					uDirNamesOff	= uOffset;
					uDirNamesNum	= uCount;
					break;

		case 1116:	if(uType!=4)break;
					uDirIndexOff	= uOffset;
					uDirIndexNum	= uCount;
					break;

		case 1028:	if(uType!=4)break;
					uFileSizeOff	= uOffset;
					uFileSizeNum	= uCount;
					break;

		case 1030:	if(uType!=3)break;
					uFileModeOff	= uOffset;
					uFileModeNum	= uCount;
					break;

		case 1037:	if(uType!=4)break;
					uFileFlagsOff	= uOffset;
					uFileFlagsNum	= uCount;
					break;

		case 1034:	if(uType!=4)break;
					uFileTimesOff	= uOffset;
					uFileTimesNum	= uCount;
					break;

		case 1117:	if(uType!=8)break;
					uFileNamesOff	= uOffset;
					uFileNamesNum	= uCount;
					break;

		case 1125:	if(uType!=6)break;
					uDecompTypeOff	= uOffset;
					m_uDecompMode   = 4;
					break;

		case 1124:	if(uType!=6)break;
					uDecompModeOff	= uOffset;
					break;

		case 1027:	if(uType!=8)break;
					uFileNamesTxt	= uOffset;
					uFileNamesCnt	= uCount;
					break;
			}
		}


	if(uFileNamesNum==0 && uFileNamesCnt>0)				// Altes Format ohne Verzeichnisse
		{
		uFileNamesNum = uFileNamesCnt;
		uFileNamesOff = uFileNamesTxt;

		if(m_uDecompMode>=5)
			{
			m_uDecompMode  = 1;
			uDecompModeOff = uStoreSize;
			uDecompTypeOff = uStoreSize+4;
			}

		uDirIndexOff = uStoreSize+4;
		uDirNamesOff = uStoreSize+4;
		uDirIndexNum = uFileNamesNum;
		uAdd		 = 4+uFileNamesNum*4;
		uStoreSize  += uAdd;
		}


	if(m_uDecompMode>=5			   )throw CAF_ERROR_FORM;
	if(uDirIndexNum	==0            )throw CAF_ERROR_FORM;
	if(uFileModeNum ==0            )throw CAF_ERROR_FORM;
	if(uFileSizeNum	==0            )throw CAF_ERROR_FORM;
	if(uFileFlagsNum==0            )throw CAF_ERROR_FORM;
	if(uFileTimesNum==0            )throw CAF_ERROR_FORM;
	if(uFileNamesNum==0            )throw CAF_ERROR_FORM;
	if(uDirNamesOff >=uStoreSize   )throw CAF_ERROR_FORM;
	if(uDirIndexOff	>=uStoreSize   )throw CAF_ERROR_FORM;
	if(uFileSizeOff	>=uStoreSize   )throw CAF_ERROR_FORM;
	if(uFileFlagsOff>=uStoreSize   )throw CAF_ERROR_FORM;
	if(uFileTimesOff>=uStoreSize   )throw CAF_ERROR_FORM;
	if(uFileNamesOff>=uStoreSize   )throw CAF_ERROR_FORM;
	if(uFileSizeNum	!=uFileFlagsNum)throw CAF_ERROR_FORM;
	if(uFileFlagsNum!=uFileTimesNum)throw CAF_ERROR_FORM;
	if(uFileTimesNum!=uFileNamesNum)throw CAF_ERROR_FORM;
	if(uDecompModeOff>=uStoreSize  )throw CAF_ERROR_FORM;
	if(uDecompTypeOff>=uStoreSize  )throw CAF_ERROR_FORM;

		pData = new char[uStoreSize];
	if(!pData)throw CAF_ERROR_MEM;

	iFilePos +=	MainRead(pData,uStoreSize-uAdd);

	if(uAdd)											// Verzeichnisse anhngen
		{
		memcpy(pData+uStoreSize-uAdd  ,"cpio",4);
		memset(pData+uStoreSize-uAdd+4,0,uAdd-4);
		}

	pFile	  = pData+uFileNamesOff;
	pDir	  = pData+uDirNamesOff;
	iLenP	  = (int)strlen(pDir);
	uIndexNow = 0;


	if(iLenP>0 && pDir[iLenP-1]=='/'){iLenP--;}
	if(iLenP>0 && pDir[      0]=='/'){iLenP--;pDir++;}

	if(!stricmp(pData+uDecompTypeOff,"none" ))m_uDecompMode=0;
	if(!stricmp(pData+uDecompTypeOff,"gzip" ))m_uDecompMode=1;
	if(!stricmp(pData+uDecompTypeOff,"bzip2"))m_uDecompMode=2;
	if(!stricmp(pData+uDecompTypeOff,"xz"   ))m_uDecompMode=3;
	if( stricmp(pData+uDecompModeOff,"cpio" ))throw CAF_ERROR_FORM;

	if(m_uDecompMode>=4)throw CAF_ERROR_FORM;

		m_pFiles = new RpmItem[uFileNamesNum];
	if(!m_pFiles)throw CAF_ERROR_MEM;

	pEntry = m_pFiles;


	for(m_iFilesNum=0;m_iFilesNum<(int)uFileTimesNum;m_iFilesNum++)
		{												// Alle Dateien aus der Liste holen
		iLenN	= (int)strlen(pFile);
		uIndex  = btol_dw(((int*)(pData+uDirIndexOff))[m_iFilesNum]);

		if(uIndex>=uDirIndexNum)throw CAF_ERROR_FORM;
		if(uIndex!=uIndexNow)							// Anderer Pfad
			{
			pDir		= pData+uDirNamesOff;
			uIndexNow	= uIndex;

			while(uIndex>0)								// Pfadnamen suchen
				{
				uIndex--;
				pDir+=strlen(pDir)+1;
				}

			   iLenP = (int)strlen(pDir);
			if(iLenP>0 && pDir[iLenP-1]=='/'){iLenP--;}
			if(iLenP>0 && pDir[      0]=='/'){iLenP--;pDir++;}
			}


														// Dateinamen umwandeln
			pEntry->pName = new zchar[iLenN+iLenP+2];
		if(!pEntry->pName)throw CAF_ERROR_MEM;

		iPos = 0;

		for(i=0;i<iLenP;)
			{
			i += utf8tochar(pDir+i,uChar);
			#ifndef UNICODE
			if(uChar>255)uChar=255;
			#endif
			pEntry->pName[iPos]=uChar;
			iPos++;
			}

		if(iLenP>0)
			{
			pEntry->pName[iPos]='/';
			iPos++;
			}

		for(i=0;i<iLenN;)
			{
			i += utf8tochar(pFile+i,uChar);
			#ifndef UNICODE
			if(uChar>255)uChar=255;
			#endif
			pEntry->pName[iPos]=uChar;
			iPos++;
			}


		uOffset += 110;
		uOffset += (iPos+3)&~3;

		pEntry->pName[iPos]	=  0;
		pEntry->uNameLen	=  iPos;
		pEntry->uOffset		=  0xFFFFFFFF;
		pEntry->iAttrib		=  btol_w (((unsigned short*)(pData+uFileModeOff ))[m_iFilesNum]);
		pEntry->uLenFull	=  btol_dw(((unsigned int  *)(pData+uFileSizeOff ))[m_iFilesNum]);
		pEntry->iTime		=  btol_dw(((unsigned int  *)(pData+uFileTimesOff))[m_iFilesNum]);
		pEntry->cDirectory  = (pEntry->iAttrib>>12)==4;
		pEntry->uLenComp	=  pEntry->uLenFull;
		pEntry->cDecompOk	=  0;
		pEntry->cError		=  1;

		if(pEntry->cDirectory)pEntry->uLenFull=0;

		uOffset += (pEntry->uLenFull+3)&~3;
		pFile   += iLenN+1;
		pEntry++;
		}

	delete pData;

	m_uDecompSize	= uOffset;
	m_uDecompOffset	= iFilePos;
	m_iDoneNow		= m_iDoneMax;
	}
catch(int iError)										// Bekannter Fehler
	{
	if(pData)delete pData;
	m_iErrorCode=iError;
	PROGN(CAF_PROG_ERROR,-1);
	Break();
	return iError;
	}

	if(m_iModeFlags&CAF_FLAG_SORT)Sort();				// Sortieren ?

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow	 = m_iDoneMax;



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einer Datei in der Dateiliste
//	pName	: Ist der  Dateiname nach dem gesucht wird
//	Ergibt die Indexnummer der Datei, bzw. -1 wenn die Datei nicht gefunden wurde.
int CRpmFile::Find(const zchar *pName)
{
RpmItem	*pEntry;
unsigned uLen;
int		 iPos;


	if(*pName=='/' )pName++;
	if(*pName=='\\')pName++;


	uLen = (int)strzlen(pName);

	for(iPos=0;iPos<m_iFilesNum;iPos++)							// Eintrag in Kette
		{
		   pEntry=m_pFiles+iPos;
		if(pEntry->uNameLen!=uLen)continue;

		if(!CmpFileNameN(pName,pEntry->pName))
			{
			return iPos;
			}
		}


return -1;
}

//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
// Bearbeitung abbrechen ohne Daten zu speichern
void CRpmFile::Break()
{

	ReleaseData();
	m_iOpenMode=CAF_CLOSED;
	Clear();

}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das RpmFile und Speichert alle Daten
//	Ergibt;	CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CRpmFile::Close()
{


	m_iErrorCode = CAF_ERROR_OK;

	m_cFilePath[0] = 0;

	if(m_iOpenMode==CAF_CLOSED)
		{
		m_iOpenMode =CAF_CLOSED;
		m_iErrorCode=CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,-1);
		return     CAF_ERROR_MISS;
		}

	m_iOpenMode = CAF_CLOSED;

	ReleaseData();



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		ExtractProc
//*
//*****************************************************************************
//	Extrahiert eine Datei aus dem Archiv ber eine Ausgabefunktion
//	iNum	: Ist die Indexnummer der Datei die geprft werden soll
//	pParam	: Ist ein Parameter der an die Ausgabefunktionbergeben wird
//	pProc	: Ist der Zeiger auf die Ausgabefunktion
//	pInfo	: Hier werden Zeiten und Attribute der Datei gespeichert.
//			  Bei NULL werden diese Werte ignoriert.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_OK		= Wenn alles korrekt war
//				CAF_ERROR_MISS		= Wenn die Datei nicht gefunden wurde
//				CAF_ERROR_PATH		= Pfad ungltig ist.
//				CAF_ERROR_FCREATE	= Wenn Datei nicht erzeugt werden konnte
//				CAF_ERROR_CRC		= Crc32 Fehler
//				CAF_ERROR_DECOMP	= Fehler beim Entkomprimieren
//				CAF_ERROR_IGNORE	= Die Datei wurde bersprungen
//				... siehe Fehler Codes
int	CRpmFile::ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo)
{
unsigned		uPos;
unsigned		uLen;
unsigned		uRet;
RpmItem		   *pEntry;


	if(m_iDoneMode<=1)										// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}

	m_iErrorCode = CAF_ERROR_OK;
	m_cThrowRd   = 0;

	try	{
		if( m_iOpenMode==CAF_CLOSED             )throw CAF_ERROR_MISS;
		if((unsigned)iNum>=(unsigned)m_iFilesNum)throw CAF_ERROR_MISS;

			pEntry=m_pFiles+iNum;
		if(!pEntry)throw CAF_ERROR_MISS;

		if(m_iDoneMode==1)									// Verlaufsanzeige einstellen
			{
			m_iDoneNow   = 0;
			m_iDoneMax   = pEntry->uLenFull;
			}

		m_pErrorFile = pEntry->pName;

		if(!m_iDecompDone)									// Alle Dateien entkomprimieren
			{
			m_iDoneMax += m_uDecompSize;
			if(!Decompess())throw CAF_ERROR_FORM;
			}

		if(pEntry->cError)
			{
			throw CAF_ERROR_DECOMP;
			}

		for(uPos=0;uPos<pEntry->uLenFull;uPos+=uLen)		// Daten speichern
			{
			   uLen = 4096;
			if(uLen+uPos>pEntry->uLenFull)uLen=pEntry->uLenFull-uPos;


			   uRet = pProc(pParam,m_pDecompMem+pEntry->uOffset+uPos,uLen);
			if(uRet!=uLen)
				{
				throw CAF_ERROR_FWRITE;
				}

			m_iDoneNow += uLen;
			}


		if(pProc(pParam,NULL,0)!=0)							// Ende anzeigen
			{
			PROGN(CAF_PROG_ERROR,iNum);
			throw CAF_ERROR_FWRITE;
			}

		if(pInfo)											// Dateidaten speichern
			{
			pInfo->dwAttr		=(pEntry->cDirectory)? SYS_DIR|SYS_READ:SYS_READ;
			pInfo->dwAttr	   |=(pEntry->uLenFull>>(32-SYS_HIGHSHIFT))&SYS_HIGHMASK;
			pInfo->dwSize		=(unsigned)(pEntry->uLenFull  );
			pInfo->dwTimeWrite	=(unsigned)(pEntry->iTime!=-1)? pEntry->iTime:0;
			pInfo->dwTimeCreate	= 0;
			pInfo->dwTimeAccess	= 0;
			pInfo->cName[0]		= 0;
			}
		}
	catch(int iError)
		{
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}

	m_cThrowRd   = 1;
	
	if(m_iDoneMode<=1)										// Verlaufsanzeige abpassen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}



return m_iErrorCode;
}


//*****************************************************************************
//*
//*		Count
//*
//*****************************************************************************
//	Ergibt die Anzahl der Eintrge in der Dateiliste
int CRpmFile::Count()
{
	return m_iFilesNum;
}



//*****************************************************************************
//*
//*		Decompess
//*
//*****************************************************************************
//	Alle Dateien in eine Speicher endkomprimiern
//	Ergibt 1 wenn ok bzw. 0 bei einem Fehler
int CRpmFile::Decompess()
{
unsigned char	ucTemp[1025];
zchar			cName [1024];
RpmItem		   *pEntry;
GzipHeader		sHead;
CBzipClass  	sBzip;
CXzClass		sLzma;
CHuffman    	sHuff;
unsigned		uChar;
unsigned		uSize;
unsigned		uTime;
unsigned		uMode;
unsigned		uMask;
unsigned		uOff;
unsigned		uPos;
unsigned		uLen;
unsigned		uNum;
int				iFile;
int				iErr;
int				iAdd;
int				i;


	if(m_iOpenMode!=CAF_READ)return 0;
	if(m_iFilesNum<=0       )return 0;
	if(m_uDecompMode>3      )return 0;
	if(m_iDecompDone        )return 1;



		m_pDecompMem = new char[m_uDecompSize+16];
	if(!m_pDecompMem)throw CAF_ERROR_MEM;

	uSize			= (unsigned)m_iFileSize-m_uDecompOffset;
	m_iDoneNow     += m_uDecompOffset;
	m_iDecompDone   = 1;

	MainSeek(m_uDecompOffset,SEEK_SET);


	switch(m_uDecompMode)
		{
	case 0:													// none
			if(uSize>m_uDecompSize)
				{
				uSize = m_uDecompSize;
				}

			for(uPos=0;uPos<uSize;uPos+=uLen)
				{
				   uLen = 4096;
				if(uLen+uPos>uSize)uLen=uSize-uPos;
				uNum=MainRead(m_pDecompMem+uPos,uLen);
				if(uNum!=uLen)break;
				}

			break;

	case 1:													// gzip

			MainRead(&sHead,sizeof(sHead));
			iAdd = 0;

			if(sHead.ucIdenifier1!=0x1F || sHead.ucIdenifier2!=0x8B)
				{
				throw CAF_ERROR_FORM;
				}

			if(sHead.ucFlags&TGZ_FLAG_FEXTRA)				// Extrafelder berspringen
				{
				MainRead(ucTemp,2);

				iAdd += ucTemp[iAdd];
				iAdd += ucTemp[iAdd]*256;

				MainSeek(iAdd,SEEK_CUR);
				}

			if(sHead.ucFlags&TGZ_FLAG_FNAME)				// Dateinamen berspringen
				{
				ucTemp[sizeof(ucTemp)-1]=0;
				iAdd  = MainRead(ucTemp,sizeof(ucTemp)-1);
				iAdd -= (int)strlen((char*)ucTemp)+1;

				MainSeek(-iAdd,SEEK_CUR);
				}

			if(sHead.ucFlags&TGZ_FLAG_FCOMMENT)				// Kommentrar berspringen
				{
				ucTemp[sizeof(ucTemp)-1]=0;
				iAdd  = MainRead(ucTemp,sizeof(ucTemp)-1);
				iAdd -= (int)strlen((char*)ucTemp)+1;

				MainSeek(-iAdd,SEEK_CUR);
				}

			if(sHead.ucFlags&TGZ_FLAG_FHCRC)				// CRC16-Summe berspringen
				{
				MainSeek(2,SEEK_CUR);
				}



			try	{
				sHuff.SetIn(this,MainRead);
				sHuff.SetOut(m_uDecompSize,m_pDecompMem);
				sHuff.Decompress();
				}
			catch(char *pText)
				{
				pText=0;
				iErr =sHuff.GetError();

				if(iErr==CHF_ERROR_READ )throw CAF_ERROR_FREAD;
				if(iErr==CHF_ERROR_WRITE)throw CAF_ERROR_WRITE;
				 						 throw CAF_ERROR_LOGIC;
				}

			uPos = sHuff.Value(CHF_LEN);
			break;

	case 2:													// bzip2
			try	{
				sBzip.SetIn(this,MainRead);
				sBzip.SetOut(m_uDecompSize,m_pDecompMem);
				sBzip.Decompress();
				}
			catch(char *pText)
				{
				pText=0;
				iErr =sBzip.GetError();

				if(iErr==CHF_ERROR_READ )throw CAF_ERROR_FREAD;
				if(iErr==CHF_ERROR_WRITE)throw CAF_ERROR_WRITE;
				 						 throw CAF_ERROR_LOGIC;
				}

			uPos = sBzip.Value(CHF_LEN);
			break;

	case 3:													// XZ-Format
			try	{
				sLzma.SetIn(this,MainRead);
				sLzma.SetOut(m_uDecompSize,m_pDecompMem);
				sLzma.Decompress();
				}
			catch(char *pText)
				{
				pText=0;
				iErr =sLzma.GetError();

				if(iErr==CXZ_ERROR_READ )throw CAF_ERROR_FREAD;
				if(iErr==CXZ_ERROR_WRITE)throw CAF_ERROR_WRITE;
				 						 throw CAF_ERROR_LOGIC;
				}

			uPos = sLzma.Value(CHF_LEN);
			break;

		}

	m_uDecompSize	= uPos;
	uOff			= 0;
	iFile			= 0;

	if(m_iFilesNum>0)
	while(uOff+6<m_uDecompSize)								// Lschen der Fehlerflags
		{
		if(!memcmp(m_pDecompMem+uOff,"070701",6) ||
		   !memcmp(m_pDecompMem+uOff,"070702",6))
			{
			if(uOff+110>=m_uDecompSize)break;

			m_pDecompMem[uOff+94+8]=0;uNum =strtol(m_pDecompMem+uOff+94,NULL,16);
			m_pDecompMem[uOff+54+8]=0;uSize=strtol(m_pDecompMem+uOff+54,NULL,16);
			m_pDecompMem[uOff+46+8]=0;uTime=strtol(m_pDecompMem+uOff+46,NULL,16);
			m_pDecompMem[uOff+14+8]=0;uMode=strtol(m_pDecompMem+uOff+14,NULL,16);

			uOff  += 110;
			uMask  = 3;
			}
		else if(!memcmp(m_pDecompMem+uOff,"070707",6))
			{
			if(uOff+76>=m_uDecompSize)break;

			m_pDecompMem[uOff+   8]=m_pDecompMem[uOff+65+8];
			m_pDecompMem[uOff+65+8]=0;uSize=strtol(m_pDecompMem+uOff+65,NULL,8);
			m_pDecompMem[uOff+59+8]=0;uNum =strtol(m_pDecompMem+uOff+59,NULL,8);
			m_pDecompMem[uOff+48+8]=0;uTime=strtol(m_pDecompMem+uOff+48,NULL,8);
			m_pDecompMem[uOff+18+6]=0;uMode=strtol(m_pDecompMem+uOff+18,NULL,8);
			m_pDecompMem[uOff+65+8]=m_pDecompMem[uOff+8];

			uOff  += 76;
			uNum  += 1;
			uMask  = 0;
			}
		else if(*(unsigned short*)(m_pDecompMem+uOff)==0x01C7)
			{
			if(uOff+26>=m_uDecompSize)break;

			uMode  = *(unsigned short*)(m_pDecompMem+uOff+ 6);
			uTime  = *(unsigned short*)(m_pDecompMem+uOff+18);
			uTime += *(unsigned short*)(m_pDecompMem+uOff+16)*0x10000;
			uNum   = *(unsigned short*)(m_pDecompMem+uOff+20);
			uSize  = *(unsigned short*)(m_pDecompMem+uOff+24);
			uSize += *(unsigned short*)(m_pDecompMem+uOff+22)*0x10000;

			uOff  += 26;
			uMask  = 1;
			}
		else if(*(unsigned short*)(m_pDecompMem+uOff)==0xC701)
			{
			if(uOff+26>=m_uDecompSize)break;

			uMode  = btol_w(*(unsigned short*)(m_pDecompMem+uOff+ 6));
			uTime  = btol_w(*(unsigned short*)(m_pDecompMem+uOff+18));
			uTime += btol_w(*(unsigned short*)(m_pDecompMem+uOff+16))*0x10000;
			uNum   = btol_w(*(unsigned short*)(m_pDecompMem+uOff+20));
			uSize  = btol_w(*(unsigned short*)(m_pDecompMem+uOff+24));
			uSize += btol_w(*(unsigned short*)(m_pDecompMem+uOff+22))*0x10000;

			uOff  += 26;
			uMask  = 1;
			}
		else{
			break;
			}


		if(uOff+uNum>m_uDecompSize)							// Prfe die Namenslnge
			{
			break;
			}


		if(m_pDecompMem[uOff]=='.' && m_pDecompMem[uOff+1]=='/')
			{
			uPos=2;
			}
		else if(m_pDecompMem[uOff]=='/')
			{
			uPos=1;
			}
		else{
			uPos=0;
			}

		for(i=0;uPos<uNum;)									// UTF-Namen umwandeln
			{
			uPos += utf8tochar(m_pDecompMem+uOff+uPos,uChar);
			#ifndef UNICODE
			if(uChar>255)uChar=255;
			#endif
			cName[i]=uChar;
			i++;
			if(i>=sizeof(cName)/sizeof(cName[0]))break;
			}

		uOff +=  uMask+uNum;
		uOff &= ~uMask;

		if(uOff+uSize>m_uDecompSize)						// Prfe Dateigre
			{
			break;
			}

		pEntry = m_pFiles+iFile;

		if(strzicmp(cName,pEntry->pName))					// Nach dem Namen suchen
			{
			   i = Find(cName);
			if(i<0)
				{
				uOff += uSize+3;
				uOff &= ~3;
				continue;
				}

			iFile  = i;
			pEntry = m_pFiles+iFile;
			}

		pEntry->cError		=  0;							// Eintrag aktualisieren
		pEntry->uOffset		=  uOff;
		pEntry->uLenComp	=  uSize;
		pEntry->cDirectory	=((uMode>>12)&0x0F)==4;
		pEntry->iTime		=  uTime;

		if(iFile+1<m_iFilesNum)
			{
			iFile++;
			}

		uOff +=  uMask+uSize;
		uOff &= ~uMask;
		}


return 1;
}

//*****************************************************************************
//*
//*		SetDoneMode
//*
//*****************************************************************************
//	Den Modus fr die Verlaufsanzeige setzen.
//	iMode	: Ist der neue Modus
//			    0 = Zurck zu Normal (Ende)
//				1 = Begin von Extract (einzelene Datei)
//				2 = Begin von ExtractAll
//	Ergibt den vorherigen Modus.
int CRpmFile::SetDoneMode(int iMode)
{
int		iRet;

	
	iRet = m_iDoneMode;
	
	switch(iMode)
		{
	case 0:													// Zurck zu Normal (Ende)

		if(m_iDoneMode==1 || m_iDoneMode==2)
		if(m_iErrorCode==CAF_ERROR_OK)
			{
			m_iDoneNow  = m_iDoneMax;
			}

		m_iDoneMode	 = 0;
		break;

	case 1:													// Begin von Extract (einzelene Datei)

		if(iRet==2)return iRet;
		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 0;
		break;

	case 2:													// Begin von ExtractAll

		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 1;
		break;
		}


return iRet;
}

//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CRpmFile::Done(aint64 *pMax)
{
aint64	iRet;


	switch(m_iDoneMode)
		{
	case 0:	iRet = m_iDoneNow;		break;
	case 1:	iRet = m_iDoneNow;		break;
	default:iRet = 0;				break;
		}


	if(iRet>m_iDoneMax)iRet=m_iDoneMax;
	if(pMax)*pMax=m_iDoneMax;


return iRet;
}

//*****************************************************************************
//*
//*		GetUserPtr
//*
//*****************************************************************************
//	User-Pointer abfragen
void *CRpmFile::GetUserPtr(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum].pUserPtr;
}

//*****************************************************************************
//*
//*		SetUserPtr
//*
//*****************************************************************************
//	User-Pointer einstellen
void CRpmFile::SetUserPtr(int iNum,void *pData)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;

	m_pFiles[iNum].pUserPtr = pData;

}

