//*****************************************************************************
//*
//*
//*		RpmFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//
//	Klasse fr den Zugriff auf RPM-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef		__RPMFILE_H__
#define     __RPMFILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	"FileFinder.h"
#include	"FileFunctions.h"
#include	"ArchiveFile.h"
#include	"Codecs/Huffman.h"
#include	"Filter/Crc32.h"


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif




//*****************************************************************************
//*
//*		Klasse CRpmFile
//*
//*****************************************************************************
class CRpmFile : public CArchiveFile
{
protected:

	typedef struct RpmItem
	{
	void		   *pUserPtr;					// User-Pointer
	zchar		   *pName;       				// File name in zip file
	unsigned		uNameLen;					// Dateinamen-Lnge
	unsigned		uLenComp;					// Dateilnge komrimiert
	unsigned		uLenFull;					// Dateilnge unkomrimiert
	unsigned		uOffset;					// Offset in Datei der komprimierten Daten
	char 			cError;						// Ist Block fehlerhaft
	char 			cDirectory;					// Ist die Datei ein Verzeichnis
	char 			cDecompMode;				// Komprimiermedothe 0=none 1=gzip 2=bzip
	char 			cDecompOk;					// Ist Block richtig endkomprimiert
	int				iAttrib;					// Datei Attribute
	time_t			iTime;						// Zeit der letzten nderung
	}RpmItem;

	RpmItem        *m_pFiles;					// Alfabethische Liste
	int				m_iFilesNum;				// Listen Eintrge ohne gelschte
	int				m_iFindPos;					// Gefunden Position
	int				m_iDecompDone;				// Wurde bereits alles entpackt
	char		   *m_pDecompMem;				// Speicher mit allen entpackten Dateien
	unsigned		m_uDecompSize;				// Entpackegre aller Dateien
	unsigned		m_uDecompMode;				// 0=none 1=gzip 2=bzip
	unsigned		m_uDecompOffset;			// Position der Komprimierten Daten in der Datei

	aint64			m_iDoneNow;					// Aktueller Verlauf
	aint64			m_iDoneMax;					// Maximaler Verlauf
	int				m_iDoneMode;				// 0=List von m_iDoneNow 1=Extraxt 2=ExtractAll 

//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

					CRpmFile();
	virtual		   ~CRpmFile();

	virtual	int		ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pFind=0);

	virtual	int		Close      ();						// Datei schliesen
	virtual	int	    Count      ();						// Anzahl der Dateien
	virtual	void	Break      ();						// Bearbeitung abbrechen
	virtual	zchar  *Name       (int  iNum);				// Hole Dateiname
	virtual	zchar  *Name       (int  iNum,int &iLen);	// Hole Dateiname samt Lnge
	virtual	int		Stats	   (int  iNum);				// Datei-Flags abfragen
	virtual	int		Sort       (int  iMode=0);			// Sortierte Kette
	virtual	int     SetFail    (int  iNum,int iFail=1);	// Defekt-Flag setzen
	virtual int		Attribute  (int    iNum);			// Datei Attribute auslesen
	virtual	int     IsReadOnly (int    iNum);			// Ist die Datei schreibgeschtzt
	virtual	int     IsFail     (int  iNum);				// Ist die Datei defekt
	virtual	int     IsDir      (int  iNum);				// Prfe ob Verzeichnis
	virtual	int     IsOk       (int  iNum);				// Richtig endkomprimiert
	virtual	aint64	Len        (int  iNum,int iMode=0);	// Hole Dateilnge
	virtual	time_t	Time       (int  iNum);				// Dateizeit abfragen
	virtual	time_t	Time       (int  iNum,int iMode);	// Dateizeit abfragen
	virtual	int		Find       (const zchar  *pName);	// Datei suchen
	virtual	aint64	Done       (aint64 *pMax);			// Verlaufsabfrage
	virtual	void   *GetUserPtr (int iNum            );	// Datenwert auslesen
	virtual	void    SetUserPtr (int iNum,void *pData);	// Datenwert einstellen

protected:

	virtual	int		SetDoneMode(int iMode);				// Verlaufsanzeige einstellen
	virtual	int		ArchOpen   (int iLevel=9);			// Das Archiv ffnen
	int				Decompess  ();						// Alle Dateien in eine Speicher endkomprimiern
	void			Clear      ();						// Speicher lschen

static int  __cdecl CmpName    (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int  __cdecl CmpEx      (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int  __cdecl CmpLen     (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int  __cdecl CmpPos     (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
};


#endif



