//*****************************************************************************
//*
//*
//*		TarFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf TAR-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen

#include	<stdio.h>
#include	<stddef.h>
#include	<string.h>
#include	<sys/stat.h>

#include	"TarFile.h"
#include	"BitFunctions.h"
#include	"FileFunctions.h"

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		(int)wcslen
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		(int)strlen
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif

#define 	ENTRIES(a)	 (sizeof(a)/sizeof((a)[0]))
#define		TAR_ADD		 256

#define 	LF_OLDNORMAL  0					// Normal disk file, Unix compatible
#define 	LF_NORMAL    '0'        		// Normal disk file
#define 	LF_LINK      '1'        		// Link to previously dumped file
#define 	LF_SYMLINK   '2'        		// Symbolic link
#define 	LF_CHR       '3'        		// Character special file
#define 	LF_BLK       '4'        		// Block special file
#define 	LF_DIR       '5'        		// Directory
#define 	LF_FIFO      '6'        		// FIFO special file
#define		LF_CONTIG    '7'        		// Contiguous file



union  TarData
	{
	unsigned char	ucBuffer [TAR_RECORDSIZE];
	unsigned short 	usBuffer [16];

	struct
		{
		char		cName    [TAR_NAMSIZE];
		char    	cMode    [8];
		char    	cUid     [8];
		char    	cGid     [8];
		char    	cSize    [12];
		char    	cTimeM   [12];
		char    	cChkSum  [8];
		char    	cLinkFlag;
		char    	cLinkName[TAR_NAMSIZE];
		char    	cMagic   [8];
		char    	cUname   [TAR_TUNMLEN];
		char    	cGname   [TAR_TGNMLEN];
		char    	cDevMajor[8];
		char    	cDevMinor[8];
		char		cPrefix  [131];
		char		cTimeA   [12];
		char		cTime    [12];
		};

	struct
		{
		char		cCpioMagic	 [6];	// "070707
		char		cCpioDev  	 [6];
		char		cCpioIno  	 [6];
		char		cCpioMode 	 [6];
		char		cCpioUid  	 [6];
		char		cCpioGid  	 [6];
		char		cCpioNlink	 [6];
		char		cCpioRdev 	 [6];
		char		cCpioTimeM	 [11];
		char		cCpioNamesize[6];
		char		cCpioFilesize[11];
		char		cCpioFileName[1];
		};

	struct
		{
		char		cCpNwMagic   [6];	// "070701" for "new" portable format "070702" for CRC format
		char		cCpNwIno     [8];
		char		cCpNwMode    [8];
		char		cCpNwUid     [8];
		char		cCpNwGid     [8];
		char		cCpNwNlink   [8];
		char		cCpNwTimeM   [8];
		char		cCpNwFilesize[8];
		char		cCpNwDevMaj  [8];
		char		cCpNwDevMin  [8];
		char		cCpNwRdevMaj [8];
		char		cCpNwRdevMin [8];
		char		cCpNwNamesize[8];	// count includes terminating NUL in pathname
		char		cCpNwChksum  [8];
		char		cCpNwFileName[1];
		};

	struct
		{
		WORD		usMagic;			// 0x71C7 little endian or 0xC771 big endian
		WORD		usDev;
		WORD		usIno;
		WORD		usMode;
		WORD		usUid;
		WORD		usGid;
		WORD		usNlink;
		WORD		usRdev;
		WORD		usTimesM[2];
		WORD		usNamesize;
		WORD		usFilesizes[2];
		char		cCpBnFileName[1];
		};
	};



#define PROG(iMode,pEntry)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,pEntry))throw CAF_ERROR_BREAK;\
	}
#define PROGR(iMode,pEntry,iRet)					\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,pEntry))return iRet;			\
	}
#define PROGN(iMode,pEntry)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	Progress(iMode,pEntry);							\
	}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CTarFile::CTarFile()
{

	m_pFiles		= 0;
	m_iFilesNum		= 0;
	m_iFilesMax		= 0;
	m_iFileCount	= 0;
	m_iDoneMode		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_iChanged		= 0;
	m_iCreated		= 0;
	m_iCreateFormat	= 1;
	m_iFileFormat	= 0;
	m_cHasAddFile	= 1;
	m_cIsCpio		= 0;
	m_uScale		= 0;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CTarFile::~CTarFile()
{

	if(m_pErrorText){delete m_pErrorText;m_pErrorText=0;}
	if(m_iOpenMode!=CAF_CLOSED)Close();

	if(!m_pFiles)
		{
		while(m_iFilesNum>0)
			{
			m_iFilesNum--;
			delete m_pFiles[m_iFilesNum];
			}

		delete m_pFiles;

		m_pFiles	= 0;
		m_iFilesMax = 0;
		}

	Clear();

}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Datenfelder und setzt alle Variabeln zurck
void CTarFile::Clear()
{
int	iPos;


	for(iPos=m_iFilesNum-1;iPos>=0;iPos--)
		{
		TarDel(iPos);
		}


	m_sFind.Set(0);

}

//*****************************************************************************
//*
//*		CalcCheckSum
//*
//*****************************************************************************
// Cheksumme berechnen
unsigned CTarFile::CalcCheckSum(unsigned char *pBuffer)
{
unsigned	uSum;
unsigned 	uPos;


	uSum = ' '*sizeof(((TarData*)0)->cChkSum);

	for(uPos=0;uPos<offsetof(TarData,cChkSum);uPos++)
		{
		uSum += pBuffer[uPos];
		}

	for(uPos+=sizeof(((TarData*)0)->cChkSum);uPos<TAR_RECORDSIZE;uPos++)
		{
		uSum += pBuffer[uPos];
		}


return uSum;
}

//*****************************************************************************
//*
//*		CmpEx
//*
//*****************************************************************************
int __cdecl CTarFile::CmpEx(const void *pPos1,const void *pPos2)
{
TarItem	*pPtr1,*pPtr2;


	pPtr1=*(TarItem**)pPos1;
	pPtr2=*(TarItem**)pPos2;

	if(pPtr1->cDelete)return (pPtr2->cDelete)? 0:1;
	if(pPtr2->cDelete)return -1;


return strvicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpName
//*
//*****************************************************************************
int __cdecl CTarFile::CmpName(const void *pPos1,const void *pPos2)
{
TarItem	*pPtr1,*pPtr2;


	pPtr1=*(TarItem**)pPos1;
	pPtr2=*(TarItem**)pPos2;

	if(pPtr1->cDelete)return (pPtr2->cDelete)? 0:1;
	if(pPtr2->cDelete)return -1;


return strzicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpLen
//*
//*****************************************************************************
int __cdecl CTarFile::CmpLen(const void *pPos1,const void * pPos2)
{
TarItem	*pPtr1,*pPtr2;


	pPtr1=*(TarItem**)pPos1;
	pPtr2=*(TarItem**)pPos2;

	if(pPtr1->cDelete)return (pPtr2->cDelete)? 0:1;
	if(pPtr2->cDelete)return -1;

	if(pPtr1->iLen==pPtr2->iLen)return 0;


return (pPtr1->iLen<pPtr2->iLen)? -1:1;
}

//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CTarFile::CmpPos(const void *pPos1,const void *pPos2)
{
TarItem	*pPtr1,*pPtr2;


	pPtr1=*(TarItem**)pPos1;
	pPtr2=*(TarItem**)pPos2;

	if(pPtr1->cWrite)return (pPtr2->cWrite)? 0:1;
	if(pPtr2->cWrite)return -1;

	if(pPtr1->iOff==pPtr2->iOff)return 0;


return (pPtr1->iOff<pPtr2->iOff)? -1:1;
}

//*****************************************************************************
//*
//*		Len
//*
//*****************************************************************************
//	Ergibt die Lnge einer einzelnen Datei im Archiv.
//	iNum	: Ist die Nmmer der Datei in der Liste
//	iMode	: Welche Lnge soll zurck gegeben werden
//				0 = wird die originale    Lnge zurckgegeben
//				1 = wird die komprimierte Lnge zurckgegeben
//	Ergibt die Lnge
aint64 CTarFile::Len(int iNum,int iMode)
{
TarItem	   *pEntry;
aint64		iLen;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

	pEntry	= m_pFiles[iNum];
	iLen	= pEntry->iLen;

	if(iMode && m_uScale)								// Die Komprimierte Gre berechnen	
		{
		iLen *= m_uScale;
		iLen /= 1024;
		}

	

return iLen;
}

//*****************************************************************************
//*
//*		SetFail
//*
//*****************************************************************************
//	Setzt das Defekt-Flag und ergibt den alten Wert.
int CTarFile::SetFail(int iNum,int iFail)
{
int		iOld;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

	iOld = m_pFiles[iNum]->cError;
	m_pFiles[iNum]->cError = iFail;

return iOld;
}

//*****************************************************************************
//*
//*		IsFail
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit iNum beschdigt ist sonst 0
int CTarFile::IsFail(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

return m_pFiles[iNum]->cError;
}

//*****************************************************************************
//*
//*		IsDir
//*
//*****************************************************************************
// Ergibt die 1 wenn die Datei mit iNum ein Verzeichnis ist sonst 0
int CTarFile::IsDir(int iNum)
{


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;


return (m_pFiles[iNum]->cType==LF_DIR)? 1:0;
}

//*****************************************************************************
//*
//*		IsOk
//*
//*****************************************************************************
//	Ergibt:	2	Wenn die Datei vollstndig entpackt wurde
//          1	Wenn die Datei schon vorhanden war
//			0   Bei Fehlern
int CTarFile::IsOk(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;


return m_pFiles[iNum]->cDecomp;
}

//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
//	Die nderungzeit einer einzelnen Datei abfragen.
//	iNum		: Ist die Nummer der Datei
//	iMode		: Bestimmt welche Zeit abgefragt werden soll
//					CAF_TIME_W	0	= Zeit der letzten nderung
//					CAF_TIME_C	1	= Zeit der Dateierstellung 
//					CAF_TIME_A	2	= Zeit des letzten Zugriffs 
//	Ergibt die Zeit der letzten nderung einer Datei bzw. CAF_ERROR_MISS (-1)
//	wenn die Datei nicht existiert.
time_t CTarFile::Time(int iNum)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}

	return m_pFiles[iNum]->iTime;
}

time_t CTarFile::Time(int iNum,int iMode)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}

	switch(iMode)
		{
	case CAF_TIME_W:									// Zeit der letzten nderung
	case CAF_TIME_C:									// Zeit der Dateierstellung
	case CAF_TIME_A: return m_pFiles[iNum]->iTime;		// Zeit des letzten Zugriffs
		}

	return CAF_ERROR_MISS;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Hole einen Zeiger auf den Dateinamen.
//	iNum		: Ist die Indexnummer des Namens (-1=Archiv-Datei)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt einen Zeiger auf den Namen oder NULL bei einem Fehler.
zchar *CTarFile::Name(int iNum)
{


	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex			
		{
		return m_pFiles[iNum]->pName;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum)								// Datei die gerade eingefgt wird
		{
		return m_pInsertFile;
		}


return NULL;
}

//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Hole Dateiname samt Lnge.
//	iNum		: Ist die Indexnummer des Namens (-1=ZIP-File)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt einen Zeiger auf den Namen oder NULL bei einem Fehler.
zchar *CTarFile::Name(int iNum,int &iLen)
{


	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex	
		{
		iLen = m_pFiles[iNum]->iNameLen;
		return m_pFiles[iNum]->pName;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		iLen = strzlen(m_cFilePath);
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum)								// Datei die gerade eingefgt wird
		{
		iLen = (m_pInsertFile)? strzlen(m_pInsertFile):0;
		return  m_pInsertFile;
		}
	
	iLen = 0;


return NULL;
}


//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Liste der Dateien im Archiv.
//  iMode	: Ist der Sortiermodus (0=Standartmodus) 	
//				CAF_FLAG_SORT	 0x0400		= Nach Namen  sortieren
//				CAF_FLAG_SORTLEN 0x0500		= Nach Lngen sortieren
//				CAF_FLAG_SORTPOS 0x0600		= Nach Offset sortieren (intern)
//				CAF_FLAG_SORTEX	 0x0700		= Nach Namen mit Zahlen Bercksichtigung
//	Ergibt die Anzahl der Dateien im Archiv.
int	CTarFile::Sort(int iMode)
{


		iMode&=0x0700;
	if(!iMode)											// Standardmodus benutzen
		{
		iMode = m_iModeFlags&0x0700;
		}

	iMode |= 0x0400;

	PROGR(CAF_PROG_SORT,-1,0);


	if(!m_pFiles)return 0;

	if(iMode==m_iSortMode)								// Ist die Liste bereits sortiert
		{
		return m_iFilesNum;
		}

	if(m_iFilesNum)
		{
		switch(iMode)
			{
		case CAF_FLAG_SORTEX:	qsort(m_pFiles, m_iFilesNum, sizeof(TarItem*),CmpEx  );break;
		case CAF_FLAG_SORTLEN:	qsort(m_pFiles, m_iFilesNum, sizeof(TarItem*),CmpLen );break;
		case CAF_FLAG_SORTPOS:	qsort(m_pFiles, m_iFilesNum, sizeof(TarItem*),CmpPos );break;
		default:				qsort(m_pFiles, m_iFilesNum, sizeof(TarItem*),CmpName);break;
			}
		}

	m_iModeFlags &= ~0x0300;
	m_iModeFlags |=  0x0300&iMode;
	m_iSortMode   =  iMode;


return m_iFilesNum;
}

//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine TAR-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Archiv
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CTarFile::ArchOpen(int iLevel)
{
TarData		sData;
TarItem    *pEntry;
char	   *pName;
char		cName[1024];
unsigned	uChkSum;
unsigned	uHdrSum;
int			iFound;
int			iSize;
int			iTime;
int			iMode;
aint64		iByte;
aint64		iData;
aint64		iAdd;
aint64		iPos;
aint64		iOff;
aint64		iLen;
int			iNum;
int 		iMax;
int			i;


	Clear();

	iNum			= 0;
	m_uScale		= 0;
	m_iFileEnd		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_iDoneMode		= 0;
	m_iChanged		= 0;
	m_iFileFormat	= 0;
	m_iErrorCode	= CAF_ERROR_OK;

														// Liegt die Datei im Speicher
	if(m_hFile==INVALID_HANDLE_VALUE && m_iOpenMode!=CAF_CREATE)
		{
		m_iFileSize = m_iMemEnd;
		}


	if(m_iFileSize<=0)									// Leere Datei
		{
		if(m_iOpenMode!=CAF_CREATE)
			{
			return CAF_ERROR_FORM;
			}

		m_iChanged	= 1;
		m_iOpenMode = CAF_WRITE;

		return CAF_ERROR_OK;
		}


//******************** Dateiliste laden ***************************************


	iByte			= MainSize();
	m_iDoneMax		= iByte;
	m_iDoneNow		= 0;
	iFound			= 0;
	cName[0]		= 0;

try	{
	for(iPos=0;iPos<iByte;)								// Dateien einlesen
		{
		iOff = iPos;


		if(MainSeek(iPos)!=iPos)
			{
			throw CAF_ERROR_FORM;
			}

		if(m_iFileFormat>=2 && iPos+512>iByte)			// Zuwenige Bytes in der Datei
			{
			iLen = iByte-iPos;
			memset(sData.ucBuffer+iLen,0,512-(int)iLen);
			}
		else{
			iLen = sizeof(sData);
			}

		MainRead(&sData,(int)iLen);

		
		if(sData.cName[0]==0)							// Letzter Block in der Datei
			{
			if(m_iFileFormat>=2)
				{
				throw CAF_ERROR_FORM;
				}

			break;
			}


		uChkSum = OctToInt(sData.cChkSum);
		uHdrSum = CalcCheckSum(sData.ucBuffer);


		if(uChkSum!=uHdrSum)							// Ist es kein TAR
			{
			for(i=0;i<sizeof(sData.cChkSum);i++)
				{
				uHdrSum += sData.ucBuffer[offsetof(TarData,cChkSum)+i];
				}

			if(uHdrSum==' '*sizeof(sData.cChkSum) && iPos+4096>=iByte)
				{
				break;									// Null-Block am Ende erreicht
				}

			if(!memcmp(sData.cCpNwMagic,"070701",6) ||	// CPIO new ascii
			   !memcmp(sData.cCpNwMagic,"070702",6))
				{
				if(!m_iFileFormat)
					{
					m_iFileFormat = 2;
					}
				else if(m_iFileFormat!=2)
					{
					throw CAF_ERROR_FORM;
					}

				sData.cCpNwNamesize[sizeof(sData.cCpNwNamesize)]=0;iSize=(unsigned)strtol(sData.cCpNwNamesize,NULL,16);
				sData.cCpNwFilesize[sizeof(sData.cCpNwFilesize)]=0;iLen =(unsigned)strtol(sData.cCpNwFilesize,NULL,16);
				sData.cCpNwTimeM   [sizeof(sData.cCpNwTimeM   )]=0;iTime=(unsigned)strtol(sData.cCpNwTimeM   ,NULL,16);
				sData.cCpNwMode    [sizeof(sData.cCpNwMode    )]=0;iMode=(unsigned)strtol(sData.cCpNwMode    ,NULL,16);

				pName  = sData.cCpNwFileName;
				iPos  += 110;
				iSize -= 1;
				i	   = 3;
				}
			else if(!memcmp(sData.ucBuffer,"070707",6))	// CPIO old ascii
				{
				if(!m_iFileFormat)
					{
					m_iFileFormat = 3;
					}
				else if(m_iFileFormat!=3)
					{
					throw CAF_ERROR_FORM;
					}

				sData.ucBuffer     [0                          ]=sData.cCpioNamesize[sizeof(sData.cCpioNamesize)];
				sData.cCpioNamesize[sizeof(sData.cCpioNamesize)]=0;iSize=(unsigned)strtol(sData.cCpioNamesize,NULL,8);
				sData.cCpioFilesize[sizeof(sData.cCpioFilesize)]=0;iLen =(unsigned)strtol(sData.cCpioFilesize,NULL,8);
				sData.cCpioTimeM   [sizeof(sData.cCpioTimeM   )]=0;iTime=(unsigned)strtol(sData.cCpioTimeM   ,NULL,8);
				sData.cCpioMode    [sizeof(sData.cCpioMode    )]=0;iMode=(unsigned)strtol(sData.cCpioMode    ,NULL,8);
				sData.cCpioNamesize[sizeof(sData.cCpioNamesize)]=sData.ucBuffer[0];

				pName = sData.cCpioFileName;
				iPos += 76;
				i     = 0;
				}
			else if(sData.usMagic==0x71C7)				// CPIO binary little endian
				{
				if(!m_iFileFormat)
					{
					m_iFileFormat = 4;
					}
				else if(m_iFileFormat!=4)
					{
					throw CAF_ERROR_FORM;
					}

				iMode  = sData.usMode;
				iSize  = sData.usNamesize;
				iTime  = sData.usTimesM   [1]+sData.usTimesM   [0]*0x10000;
				iLen   = sData.usFilesizes[1]+sData.usFilesizes[0]*0x10000;

				pName  = sData.cCpBnFileName;
				iPos  += 26;
				iSize -= 1;
				i      = 1;
				}
			else if(sData.usMagic==0xC771)				// CPIO binary little endian
				{
				if(!m_iFileFormat)
					{
					m_iFileFormat = 5;
					}
				else if(m_iFileFormat!=5)
					{
					throw CAF_ERROR_FORM;
					}

				iMode  = btol_w(sData.usMode);
				iSize  = btol_w(sData.usNamesize);
				iTime  = btol_w(sData.usTimesM   [1])+btol_w(sData.usTimesM   [0])*0x10000;
				iLen   = btol_w(sData.usFilesizes[1])+btol_w(sData.usFilesizes[0])*0x10000;

				pName  = sData.cCpBnFileName;
				iPos  += 26;
				iSize -= 1;
				i      = 1;
				}
			else{
				throw CAF_ERROR_FORM;
				}

			
			if(!iLen && !memcmp(pName,"TRAILER!!!",11))	// Ende erreicht
				{
				break;
				}


			   iPos += iSize+1;
			if(iPos&i)iPos+=(-iPos&i)&i;

			   iData = iPos;

			   iPos += iLen;
			if(iPos&i)iPos+=(-iPos&i)&i;

			sData.cLinkFlag = LF_NORMAL;

			if(iMode&040000)
				{
				sData.cLinkFlag = LF_DIR;
				}
			}
		else{											// TAR-Header gefunden
			if(!m_iFileFormat)
				{
				m_iFileFormat = 1;
				}
			else if(m_iFileFormat!=1)
				{
				throw CAF_ERROR_FORM;
				}

			iLen  = OctToInt(sData.cSize);

			if(iLen<0 || iLen>0x7F000000)
				{
				throw CAF_ERROR_FORM;
				}

			iPos += sizeof(sData);
			iAdd  = iLen+TAR_RECORDSIZE-1;
			iAdd -= iAdd%TAR_RECORDSIZE;
			pName = sData.cName;

			m_iFileFormat = 1;
			sData.cName[sizeof(sData.cName)]=0;

			if(sData.cLinkFlag=='L' && iAdd<=1024)		// Langer Dateiname
			if(memcmp(pName,"././@LongLink\0\0",16)==0)
				{
				MainRead(cName,(int)iAdd);
				iPos       += iAdd;

				iLen		= MainRead(&sData,sizeof(sData));
				uHdrSum     = CalcCheckSum(sData.ucBuffer);
				uChkSum     = OctToInt    (sData.cChkSum);

				if(!sData.cName[0] || uChkSum!=uHdrSum)	// Letzter Block in der Datei
					{
					throw CAF_ERROR_FORM;
					}

				iLen		= OctToInt(sData.cSize);
				iPos       += sizeof(sData);
				iData		= iPos;
				pName		= cName;
				}


			iData	= iPos;
			iTime	= OctToInt(sData.cTimeM);
			iMode	= OctToInt(sData.cMode);
			iAdd    = iLen+TAR_RECORDSIZE-1;
			iAdd   -= iAdd%TAR_RECORDSIZE;
			iPos   += iAdd;
														// Dateidaten und Fllbytes berspringen
			if(iPos>iByte || MainSeek(iPos,SEEK_SET)!=iPos)
				{
				throw CAF_ERROR_FORM;
				}

			if(sData.cLinkFlag<'0' || sData.cLinkFlag>'9')
				{
				sData.cLinkFlag = '0';
				}

			iSize   = (int)strlen(pName);

			if(iSize<=2 && iLen==0)
			if(sData.cLinkFlag==LF_DIR && sData.cName[0]=='.')
			if(sData.cName[1]==0       || sData.cName[1]=='/')
				{
				continue;
				}
			}


		//************ Eintrag einfgen ***************************************


		iNum		= TarNew();							// Neuer Block
		pEntry		= m_pFiles[iNum];

		if(pName[0]=='.' && pName[1]=='/')
			{
			pName += 2;
			iSize -= 2;
			}

		if(iSize>0 && pName[iSize-1]=='/')
			{
			iSize -= 1;
			}

		if(iSize<1)										// Lnge vom Dateinamen Prfen
			{
			throw CAF_ERROR_FORM;
			}


		pEntry->pName = new zchar[iSize+1];

		for(iMax=0,i=0;i<iSize;iMax++)					// Von UTF8 auf ANSI
			{
			unsigned	uUnicode;
			i += utf8tochar(pName+i,uUnicode);
			#ifndef UNICODE
			if(uUnicode>255)uUnicode=255;
			#endif
			pEntry->pName[iMax] = (zchar)uUnicode;
			}

		pEntry->pName[iMax] = 0;
		pEntry->iNameLen    = iMax;

														// Slash am Ende entfernen
		if(pEntry->iNameLen>0 && pEntry->pName[pEntry->iNameLen-1]=='/')
			{
			pEntry->iNameLen--;
			pEntry->pName[pEntry->iNameLen]=0;
			}

		pEntry->cSaved	= 1;							// Ist die Datei in TAR vorhanden
		pEntry->iOff	= iOff;							// Ist der Offset im TAR-Archiv
		pEntry->iEnd	= iPos;							// Ist der End-Offset in der Datei
		pEntry->iLen	= iLen;							// Ist die Gre der Datei
		pEntry->iLenHdr = (int)(iData-iOff);			// Ist die Lnge des Headers
		pEntry->cType	= sData.cLinkFlag;				// Ist der Eintrag Typ (LF_???)
		pEntry->iTime	= iTime;						// Ist die nderungszeit
		pEntry->iAttrib	= iMode;						// Sind die Unix-Atribute
		m_iDoneNow	   += iPos-iOff;

		PROG(CAF_PROG_FIND,iNum);
		}

//*****************************************************************************
	}
catch(int iError)										// Bekannter Fehler
	{
	m_iErrorCode = iError;
	PROGN(CAF_PROG_ERROR,-1);
	CTarFile::Break();
	return iError;
	}

	if(m_iModeFlags&CAF_FLAG_SORT)						// Sortieren ?
		{
		Sort();
		}

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow	 = m_iDoneMax;
	m_iFileEnd	 = iOff;




return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einer Datei in der Dateiliste
//	pName	: Ist der  Dateiname nach dem gesucht wird
//	Ergibt die Indexnummer der Datei, bzw. -1 wenn die Datei nicht gefunden wurde.
int CTarFile::Find(const zchar *pName)
{
TarItem	   *pEntry;
int			iLen;
int			iPos;


	if(*pName=='/' )pName++;
	if(*pName=='\\')pName++;


	iLen = strzlen(pName);

	for(iPos=0;iPos<m_iFilesNum;iPos++)								// Eintrag in Kette suchen
		{
		   pEntry = m_pFiles[iPos];
		if(pEntry->cDelete       )continue;
		if(pEntry->iNameLen!=iLen)continue;

		if(!CmpFileNameN(pName,pEntry->pName))
			{
			return iPos;
			}
		}


return -1;
}


//*****************************************************************************
//*
//*		MakeTarHeader
//*
//*****************************************************************************
//	Erzeugt einen TAR-Header
//	pTarData	: Teiger auf den Puffer fr den Header
//	pEntry		: Daten der Datei
//	Ergibt 0 wenn ok bzw. die bentigte Textlnge fr den Pfad
int CTarFile::MakeTarHeader(void *pTarData,TarItem *pEntry)
{
TarData	   *pPtr=(TarData*)pTarData;
char		cTemp[8];
unsigned 	uChar;
unsigned	uChkSum;
int			iTextPos;
int			iLen;
int			iRet;
int			i;


	memset(pPtr->ucBuffer,0,sizeof(TarData));

	iTextPos = 0;										// Dateinamen umkopieren
	iRet	 = 0;

	for(i=0;i<pEntry->iNameLen;i++)						// Von ANSI auf UTF8
		{
		#ifdef UNICODE
		uChar = pEntry->pName[i];
		#else
		uChar = ((unsigned char*)(pEntry->pName))[i];
		#endif

		   iTextPos+=chartoutf8(pPtr->cName+iTextPos,uChar);
		if(iTextPos>=sizeof(pPtr->cName))
			{
			iLen = iTextPos-sizeof(pPtr->cName);

			for(;i<pEntry->iNameLen;i++)
				{
				#ifdef UNICODE
				uChar = pEntry->pName[i];
				#else
				uChar = ((unsigned char*)(pEntry->pName))[i];
				#endif

				iTextPos+=chartoutf8(cTemp,uChar);
				}

			iRet = iTextPos;
			}
		}

	if(pEntry->cType==LF_DIR)							// Slash anhngen
		{
		if(iTextPos>=sizeof(pPtr->cName))
			{
			iTextPos++;
			iRet = iTextPos;
			}
		else{
			pPtr->cName[iTextPos] = '/';
			}
		}


	memcpy (pPtr->cUid  ,"0000000" ,8);
	memcpy (pPtr->cGid  ,"0000000" ,8);
	sprintf(pPtr->cSize ,"%i%010o" ,(unsigned)(pEntry->iLen>>30)&7,(unsigned)pEntry->iLen&0x3FFFFFFF);
	sprintf(pPtr->cTimeM,"%011o"   ,pEntry->iTime);
	sprintf(pPtr->cMode ,"%07o"    ,pEntry->iAttrib);
	memcpy (pPtr->cMagic,"ustar  " ,8);
	memcpy (pPtr->cUname,"guest"   ,8);
	memcpy (pPtr->cGname,"guest"   ,8);
		    pPtr->cLinkFlag = pEntry->cType;

	uChkSum = CalcCheckSum(pPtr->ucBuffer);

	sprintf(pPtr->cChkSum,"%07o",uChkSum);


return iRet;
}

//*****************************************************************************
//*
//*		WriteLongName
//*
//*****************************************************************************
//	Einen langen Dateinamen speichern
//	iLen		: Ist die Lnge des Dateinamens
//	pEntry		: Zeiger auf die Dateidaten
//	Ergibt die geschriebenen Bytes
int CTarFile::WriteLongName(TarItem *pEntry,int iLen)
{
char		cBuffer[1024+16];
TarItem		sTemp;
TarData		sData;
unsigned 	uChar;
int			iPos;
int			iOff;


	if(iLen>1023)return 1;


	sTemp.iNameLen = 13;
	sTemp.pName    = Z("././@LongLink");
	sTemp.cType	   = 'L';
	sTemp.iLen	   = iLen+1;
	sTemp.iTime    = 0;
	sTemp.iAttrib  = 0;

	MakeTarHeader(&sData,&sTemp);						// Dummy-Header erzeugen

	MainWrite(&sData,sizeof(sData));


	for(iOff=0,iPos=0;iOff<pEntry->iNameLen;iOff++)		// Von ANSI auf UTF8
		{
		if(iPos>=1022)
			{
			throw CAF_ERROR_BIG;
			}

		#ifdef UNICODE
		uChar = pEntry->pName[iOff];
		#else
		uChar = ((unsigned char*)(pEntry->pName))[iOff];
		#endif

		iPos += chartoutf8(cBuffer+iPos,uChar);
		}

	if(pEntry->cType==LF_DIR)							// Slash anhngen
		{
		cBuffer[iPos] = '/';
		iPos++;
		}

	cBuffer[iPos] = 0;
	iPos += 1;
	iOff  = (iPos+511)&~0x1FF;

	memset(cBuffer+iPos,0,iOff-iPos);

	MainWrite(cBuffer,iOff);							// Speichere langen Dateinamen



return sizeof(sData)+iOff;
}

//*****************************************************************************
//*
//*		MakeCpio1Header
//*
//*****************************************************************************
//	Erzeugt einen neuen CPIO-ASCII-Header
//	pTarData	: Teiger auf den Puffer fr den Header
//	pEntry		: Daten der Datei
//	Ergibt 0 wenn ok bzw. die bentigte Textlnge fr den Pfad
int	CTarFile::MakeCpio1Header(void *pTarData,TarItem *pEntry)
{
TarData	   *pPtr =(TarData*)pTarData;
char	   *pName=pPtr->cCpNwFileName;
unsigned 	uChar;
int			iPos;
int			iRet;
int			i;


	iPos = 0;											// Dateinamen umkopieren
	iRet = 0;

	for(i=0;i<pEntry->iNameLen;i++)						// Von ANSI auf UTF8
		{
		#ifdef UNICODE
		uChar = pEntry->pName[i];
		#else
		uChar = ((unsigned char*)(pEntry->pName))[i];
		#endif

		iPos += chartoutf8(pName+iPos,uChar);

		if(iPos>=348)break;
		}

	if(pEntry->cType==LF_DIR)							// Slash anhngen
		{
		pName[iPos] = '/';
		iPos++;
		}

	*(int*)(pName+iPos) = 0;
	iPos++;


	memcpy (pPtr->cCpNwMagic  	,"070701"   ,6);
	memcpy (pPtr->cCpNwIno	  	,"00000001" ,8);
	sprintf(pPtr->cCpNwMode	  	,"%08X"		,pEntry->iAttrib);
	memcpy (pPtr->cCpNwUid    	,"00000001" ,8);
	memcpy (pPtr->cCpNwGid    	,"00000001" ,8);
	memcpy (pPtr->cCpNwNlink	,"00000001" ,8);
	sprintf(pPtr->cCpNwTimeM	,"%08X"		,pEntry->iTime);
	sprintf(pPtr->cCpNwFilesize	,"%08X"		,(unsigned)pEntry->iLen);
	memcpy (pPtr->cCpNwDevMaj  	,"00000008" ,8);
	memcpy (pPtr->cCpNwDevMin  	,"00000001" ,8);
	memcpy (pPtr->cCpNwRdevMaj 	,"00000000" ,8);
	memcpy (pPtr->cCpNwRdevMin 	,"00000000" ,8);
	sprintf(pPtr->cCpNwNamesize	,"%08X"		,iPos);
	memcpy (pPtr->cCpNwChksum 	,"00000000" ,8);

	if(pEntry->cType=='L')
		{
		pPtr->cCpNwIno   [7] = '0';
		pPtr->cCpNwUid   [7] = '0';
		pPtr->cCpNwGid   [7] = '0';
		pPtr->cCpNwDevMaj[7] = '0';
		pPtr->cCpNwDevMin[7] = '0';
		}

	iPos += (int)(pName-((char*)pPtr));

	if(iPos&3)
		{
		iPos+= 4-(iPos&3);
		}


return iPos;
}

//*****************************************************************************
//*
//*		MakeCpio2Header
//*
//*****************************************************************************
//	Erzeugt einen neuen CPIO-ASCII-Header (alt)
//	pTarData	: Teiger auf den Puffer fr den Header
//	pEntry		: Daten der Datei
//	Ergibt 0 wenn ok bzw. die bentigte Textlnge fr den Pfad
int	CTarFile::MakeCpio2Header(void *pTarData,TarItem *pEntry)
{
TarData	   *pPtr =(TarData*)pTarData;
char	   *pName=pPtr->cCpioFileName;
unsigned 	uChar;
int			iPos;
int			iRet;
int			i;


	iPos = 0;											// Dateinamen umkopieren
	iRet = 0;

	for(i=0;i<pEntry->iNameLen;i++)						// Von ANSI auf UTF8
		{
		#ifdef UNICODE
		uChar = pEntry->pName[i];
		#else
		uChar = ((unsigned char*)(pEntry->pName))[i];
		#endif

		iPos += chartoutf8(pName+iPos,uChar);

		if(iPos>=348)break;
		}

	if(pEntry->cType==LF_DIR)							// Slash anhngen
		{
		pName[iPos] = '/';
		iPos++;
		}

	*(int*)(pName+iPos) = 0;

	memcpy (pPtr->cCpioMagic  	,"070707"   ,6);
	memcpy (pPtr->cCpioDev	  	,"000001"   ,6);
	memcpy (pPtr->cCpioIno	  	,"000001"   ,6);
	sprintf(pPtr->cCpioMode	  	,"%06o"		,pEntry->iAttrib);
	memcpy (pPtr->cCpioUid    	,"000001"   ,6);
	memcpy (pPtr->cCpioGid    	,"000001"   ,6);
	memcpy (pPtr->cCpioNlink	,"000001"   ,6);
	memcpy (pPtr->cCpioRdev		,"000000"   ,6);
	sprintf(pPtr->cCpioTimeM	,"%011o"	,pEntry->iTime);
	sprintf(pPtr->cCpioNamesize	,"%06o"		,iPos);
	sprintf(pPtr->cCpioFilesize	,"%i%010o"  ,(unsigned)(pEntry->iLen>>30)&7,(unsigned)pEntry->iLen&0x3FFFFFFF);

	iPos++;

	if(pEntry->cType=='L')
		{
		pPtr->cCpioDev[5] = '0';
		pPtr->cCpioIno[5] = '0';
		pPtr->cCpioUid[5] = '0';
		pPtr->cCpioGid[5] = '0';
		}


	iPos += (int)(pName-((char*)pPtr));


return iPos;
}

//*****************************************************************************
//*
//*		MakeCpio3Header
//*
//*****************************************************************************
//	Erzeugt einen neuen CPIO-BINARY-Header (littel endian)
//	pTarData	: Teiger auf den Puffer fr den Header
//	pEntry		: Daten der Datei
//	Ergibt 0 wenn ok bzw. die bentigte Textlnge fr den Pfad
int	CTarFile::MakeCpio3Header(void *pTarData,TarItem *pEntry)
{
TarData	   *pPtr =(TarData*)pTarData;
char	   *pName=pPtr->cCpBnFileName;
unsigned 	uChar;
int			iPos;
int			iRet;
int			i;


	iPos = 0;											// Dateinamen umkopieren
	iRet = 0;

	for(i=0;i<pEntry->iNameLen;i++)						// Von ANSI auf UTF8
		{
		#ifdef UNICODE
		uChar = pEntry->pName[i];
		#else
		uChar = ((unsigned char*)(pEntry->pName))[i];
		#endif

		iPos += chartoutf8(pName+iPos,uChar);

		if(iPos>=348)break;
		}

	if(pEntry->cType==LF_DIR)							// Slash anhngen
		{
		pName[iPos] = '/';
		iPos++;
		}

	*(int*)(pName+iPos) = 0;
	iPos++;

	pPtr->usMagic			= 0x71C7;					// 0x71C7 little endian
	pPtr->usDev				= 0x0001;
	pPtr->usIno				= 0x0001;
	pPtr->usMode			= (WORD)pEntry->iAttrib;
	pPtr->usUid				= 0x0001;
	pPtr->usGid				= 0x0001;
	pPtr->usNlink			= 0x0001;
	pPtr->usRdev			= 0x0000;
	pPtr->usTimesM[0]		= (WORD)(pEntry->iTime>>16);
	pPtr->usTimesM[1]		= (WORD)(pEntry->iTime);
	pPtr->usNamesize		= (WORD)(iPos);
	pPtr->usFilesizes[0]	= (WORD)(pEntry->iLen>>16);
	pPtr->usFilesizes[1]	= (WORD)(pEntry->iLen);

	if(pEntry->cType=='L')
		{
		pPtr->usDev			= 0x0000;
		pPtr->usIno			= 0x0000;
		pPtr->usUid			= 0x0000;
		pPtr->usGid			= 0x0000;
		}


	iPos += (int)(pName-((char*)pPtr));

	if(iPos&1)iPos++;


return iPos;
}


//*****************************************************************************
//*
//*		WriteToTar
//*
//*****************************************************************************
//	Schreibt Daten ins temporre TAR-Archiv. Wenn ntig wird ein Dummy-Header 
//	eingefgt. Diese Funktion wird von gepacken Formaten bentigt wenn sie
//	eine eins zu eins Kopie in ein TAR-Archiv entkomprimieren.
int	_cdecl CTarFile::WriteToTar(void *pParam,const char *pMem,int iLen)
{
CTarFile   *pThis=(CTarFile*)pParam;
unsigned	uCheck;
int			iNum=0;
int			iRet=0;
int			iPos;



	if(pThis->m_iTarOffset>=512 || pThis->m_cIsCpio)	// Wurde der Header bereits geschrieben
		{
		   iLen = pThis->MainWrite(pMem,iLen);
		if(iLen<0)return iLen;

		pThis->m_iDoneNow += iLen;
		pThis->m_iDoneMax += iLen;
		
		return iLen;
		}


	   iNum = 512-pThis->m_iTarOffset;
	if(iNum>iLen)iNum=iLen;

	
														// Die ersten 512 Byte puffern
	memcpy(pThis->m_aTarBuffer+pThis->m_iTarOffset,pMem,iNum);

	pThis->m_iTarOffset += iNum;
	pMem				+= iNum;
	iLen				-= iNum;



	if(pThis->m_iTarOffset>=6)							 // Ist es ein CPIO-Format
		{
		if(!memcmp(pThis->m_aTarBuffer,"070701",6) ||
		   !memcmp(pThis->m_aTarBuffer,"070702",6) ||
		   !memcmp(pThis->m_aTarBuffer,"070707",6))
			{
			pThis->m_cIsCpio = 1;

			   iRet = pThis->MainWrite(pThis->m_aTarBuffer,pThis->m_iTarOffset);
			if(iRet<0)return -1;
			   iLen = pThis->MainWrite(pMem,iLen);
			if(iLen<0)return -1;
			
			return iNum+iLen;
			}
		}


	if(pThis->m_iTarOffset==512)						// Wurde der ganze TAR-Header eingelesen
		{
		uCheck = 8*' ';

		for(iPos=0;iPos<148;iPos++)
			{
			uCheck += ((unsigned char*)pThis->m_aTarBuffer)[iPos];
			}

		for(iPos+=8;iPos<512;iPos++)
			{
			uCheck += ((unsigned char*)pThis->m_aTarBuffer)[iPos];
			}
													
		if(uCheck^strtol(pThis->m_aTarBuffer+148,0,8))	// Prfe die Checksumme	
			{
			iRet = pThis->MainWrite(pThis->m_aTarBuffer,512);
			pThis->m_iTarOffset = TAR_DUMMY_HEADER;		// Platz fr Dummy-Header anlegen
			}

		   iRet = pThis->MainWrite(pThis->m_aTarBuffer,512);
		if(iRet!=512)return -1;
		}



	   iLen = pThis->MainWrite(pMem,iLen);					// Den Rest speichern
	if(iLen<0)return iLen;

	iLen += iNum;
	
	pThis->m_iDoneNow += iLen;
	pThis->m_iDoneMax += iLen;



return	iLen;
}

//*****************************************************************************
//*
//*		WriteDummy
//*
//*****************************************************************************
//	Schreibt einen Dummy-Header an den Anfang einer temporren TAR-Datei. 
//	Als Dateiname wird der aktuelle Name der ZIP-Datei benutzt.	Diese Funktion 
//	wird von gepacken Formaten bentigt wenn sie eins zu eins in eine  
//	TAR-Datei entkomprimieren.
//	iSize	: Ist die Gre der entkomprimierten TAR-Datei
//	Ergibt 1 wen ok oder 0 bei einm
int CTarFile::WriteDummy(aint64 iSize)
{
char		cBuffer[512];
zchar		cName  [100];
TarItem		sEntry;
int			iPoint;
int			iOff;
int			iPos;
int			iSet;
int			iLen;




	iLen   = (int)strzlen(m_cFilePath);
	iOff   = iLen;
	iPoint = 0;

	for(iPos=iLen;iPos>0;iPos--)					// Suche Pfadende
		{
		if(m_cFilePath[iPos]=='.')
			{
			if(!iPoint)iOff=iPos;
				iPoint++;
			}

		if(m_cFilePath[iPos-1]=='/' )break;
		if(m_cFilePath[iPos-1]=='\\')break;
		}

	#ifdef UNICODE

	for(iSet=0;iPos<=iOff;iPos++)					// Dateinamen kopieren (UNICODE)
		{
		iSet += chartoutf16(cName+iSet,m_cFilePath[iPos]);
		if(iSet>90)break;
		}

	#else

	for(iSet=0;iPos<=iOff;iPos++)					// Dateinamen kopieren
		{
		cName[iSet] = m_cFilePath[iPos];
		iSet++;
		if(iSet>90)break;
		}

	#endif

	if(!iPoint)
		{
		memcpy(cName+iSet,"dat",4);
		iSet+=3;
		}
	else{
		iSet--;
		cName[iSet]=0;
		}


	sEntry.cType	= '0';							// Virtuellen TAR-Header erzeugen
	sEntry.pName	= cName;
	sEntry.iNameLen	= iSet;
	sEntry.iLen		= iSize;
	sEntry.iTime	= time(0);
	sEntry.iAttrib	= 0100664;

	MakeTarHeader(cBuffer,&sEntry);
	MainSeek	 (0,SEEK_SET);
	
	if(MainWrite(cBuffer,512)!=512)
		{
		m_iErrorCode = CAF_ERROR_FWRITE;
		return		   CAF_ERROR_FWRITE;
		}

	

return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		WriteHeader
//*
//*****************************************************************************
//	Speichert den Header einer einzelnen Datei
//	iSize		: Ist die Gre der Datei in Byte
//	Ergibt die Fllbytes
int CTarFile::WriteHeader(TarItem *pEntry,aint64 iSize)
{
TarData		sData;
int			iRet;
int			iAdd;


	pEntry->iLen = iSize;
	pEntry->iOff = MainTell();

	if(m_iFileFormat==0)
		{
		m_iFileFormat = m_iCreateFormat;
		}


	switch(m_iFileFormat)
		{
	default:
	case 1:												// TAR Header

			   iRet = MakeTarHeader(&sData,pEntry);
			if(iRet)									// Ist der Dateiname zu lang ?
				{
				WriteLongName(pEntry,iRet);
				}


			MainWrite(&sData,sizeof(sData));
			iAdd  = (-(int)iSize)&0x1FF;

			break;

	case 2:												// CPIO new ascii

			iRet = MakeCpio1Header(&sData,pEntry);

			MainWrite(&sData,iRet);
			iAdd  = (-(int)iSize&3)&3;

			break;

	case 3:												// CPIO new ascii

			iRet = MakeCpio2Header(&sData,pEntry);

			MainWrite(&sData,iRet);
			iAdd  = 0;

			break;

	case 4:												// CPIO binary little endian

			iRet = MakeCpio3Header(&sData,pEntry);

			MainWrite(&sData,iRet);
			iAdd  = ((int)iSize&1);

			break;

	case 5:												// CPIO binary big endian

			iRet = MakeCpio3Header(&sData,pEntry);

			for(iAdd=0;iAdd<13;iAdd++)
				{
				sData.usBuffer[iAdd] = btol_w(sData.usBuffer[iAdd]);
				}

			MainWrite(&sData,iRet);
			iAdd  = ((int)iSize&1);

			break;

		}

	pEntry->iLenHdr  = (int)(MainTell()-pEntry->iOff);
	pEntry->iEnd	 = pEntry->iOff+iSize+iAdd;




return iAdd;
}

//*****************************************************************************
//*
//*		Write
//*
//*****************************************************************************
//	Speichert eine einzelne Datei in der TAR-Datei.
//	iNum	: Ist die Nummer der Datei
//	ber throw werden die Fehler-Codes zurck gegeben
int CTarFile::Write(int iNum)
{
char		cBuffer[0x1000];
zchar		cName[256];
TarItem	   *pEntry;
DWORD		dwAttr;
DWORD		dwBytes;
HANDLE		hFile;
unsigned 	uSize;
unsigned 	uAdd;
aint64	 	iLen;
aint64	 	iPos;
aint64	 	iDif;





	pEntry = m_pFiles[iNum];
	PROG(CAF_PROG_WRITE,iNum);




//******************** Daten speichern ****************************************


	m_pErrorFile = pEntry->pName;

	if(pEntry->cType==LF_DIR)
		{
		pEntry->iAttrib	= 0040755;
		pEntry->iLen	= 0;
		iLen			= 0;
		hFile			= INVALID_HANDLE_VALUE;
		}
	else{												
		iPos = strlcpy(cName,pEntry->pPath,ENTRIES(cName));

														// Datei zum Lesen ffnen
		   hFile =CreateFile(cName,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,0);
		if(hFile==INVALID_HANDLE_VALUE)throw CAF_ERROR_FOPEN;

		   iLen = GetFileSize(hFile);
		if(iLen<0)
			{
			CloseFile(hFile);
			throw CAF_ERROR_WRITE;
			}

		   dwAttr=GetFileAttributes(cName);
		if(dwAttr&FILE_ATTRIBUTE_READONLY)
				pEntry->iAttrib = 0100444;
		else	pEntry->iAttrib = 0100664;

		pEntry->iLen = iLen;
		}


	try	{
		uAdd = WriteHeader(pEntry,(int)iLen);			// Den Datei-Header speichern


		if(pEntry->cType==LF_NORMAL)					// Kopiere die Datei
			{
			for(iPos=0;iPos<iLen;iPos+=uSize)
				{
				   iDif = iLen-iPos;
				if(iDif>sizeof(cBuffer))
						uSize = sizeof(cBuffer);
				else	uSize = (unsigned)iDif;
					

				if(!ReadFile(hFile,cBuffer,uSize,&dwBytes,NULL))
					{
					throw CAF_ERROR_FREAD;
					}

				if(dwBytes!=(unsigned)uSize)
					{
					throw CAF_ERROR_FREAD;
					}

				MainWrite(cBuffer,uSize);

				m_iDoneNow += uSize;
				}

			if(uAdd)									// Fllbytes speichern
				{
				memset   (cBuffer,0,uAdd);
				MainWrite(cBuffer  ,uAdd);
				}

			CloseFile(hFile);
			}

		pEntry->iEnd	= MainTell();
		pEntry->cSaved	= 1;
		pEntry->cWrite	= 0;
		pEntry->cDelete	= 0;
		m_iChanged		= 1;

		PROG(CAF_PROG_COMPRESS,iNum);
		}
	catch(int iError)
		{
		if(hFile!=INVALID_HANDLE_VALUE)CloseFile(hFile);
		m_iErrorCode = iError;
		throw iError;
		}



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		TarDel
//*
//*****************************************************************************
//	Einen einzelnen Datei-Listeneintrag lschen.
//	iNum	: Ist die Numer der Datei
void CTarFile::TarDel(int iNum)
{
TarItem	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)			// Prfe die Dateinummer	
		{
		return;
		}

	pEntry = m_pFiles[iNum];

	if(!pEntry)return;

	m_iFilesNum--;
	m_pFiles[iNum]=0;

	if(iNum<m_iFilesNum)
		{
		memmove(m_pFiles+iNum,m_pFiles+iNum+1,(m_iFilesNum-iNum)*sizeof(m_pFiles[0]));
		}

	if(!pEntry->cDelete)m_iFileCount--;
	if( pEntry->pName  )delete pEntry->pName;
 					    delete pEntry;

	if(m_iFilesNum<=0)
		{
		delete  m_pFiles;
		m_iFilesNum		= 0;
		m_iFilesMax		= 0;
		m_iFileCount	= 0;
		}

	m_iChanged = 1;

}

//*****************************************************************************
//*
//*		TarSetDel
//*
//*****************************************************************************
//	Listeneintrag lschen
void CTarFile::TarSetDel(int iNum)
{
TarItem	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)			// Prfe die Dateinummer	
		{
		return;			
		}

		pEntry=m_pFiles[iNum];
	if(!pEntry->cDelete)
		{
		m_iChanged		= 1;
		m_iFileCount   -= 1;
		pEntry->cDelete = 1;
		}

	if(pEntry->cWrite)									// Ist es eine Datei die Eingefgt wurde
		{
		TarDel(iNum);
		return;
		}

	if(iNum>=m_iFileCount)return;
	if(m_iModeFlags&CAF_FLAG_SORT)						// Sortierte Liste
		{
		Sort();
		return;
		}

	m_pFiles[iNum]         = m_pFiles[m_iFileCount];	// Ans Ende der Liste verschieben
	m_pFiles[m_iFileCount] = pEntry;
	
	

}

//*****************************************************************************
//*
//*		TarNew
//*
//*****************************************************************************
//	Listeneintrag erzeugen
int	CTarFile::TarNew()
{
TarItem	**pNew;



	if(!m_pFiles || !m_iFilesMax)						// Neue Kette
		{
		m_pFiles           = new TarItem*[TAR_ADD];
		memset(m_pFiles,0,sizeof(TarItem*[TAR_ADD]));
		m_iFilesMax = TAR_ADD;
		}


	if(m_iFilesNum >= m_iFilesMax)						// Kette vergrern
		{
		pNew = new TarItem*[m_iFilesMax+TAR_ADD];
		memset(pNew+m_iFilesMax,0,sizeof(TarItem*[TAR_ADD]));
		memcpy(pNew,m_pFiles     ,sizeof(TarItem*)*m_iFilesMax);
		delete m_pFiles;
		m_pFiles     = pNew;
		m_iFilesMax += TAR_ADD;
		}


		   m_pFiles[m_iFilesNum]=new TarItem;
	memset(m_pFiles[m_iFilesNum],0,sizeof(TarItem));
		   m_iFileCount++;
		   m_iFilesNum++;


return m_iFilesNum-1;
}

//*****************************************************************************
//*
//*		Del
//*
//*****************************************************************************
//	Lscht eine Datei aus dem Archiv.
//	iNum	: Ist die Indexnummer der zu lschenden Datei.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_MISS   Wenn Datei nicht gefunden wurde
//				CAF_ERROR_OK     Wenn Datei gelscht wurde
//				CAF_ERROR_WRITE  Wenn Datei keine Schreib erlaubnis hat
int CTarFile::Del(int iNum)
{


	PROGR(CAF_PROG_ZDEL,iNum,CAF_ERROR_MISS);

	if(m_iOpenMode!=CAF_WRITE)							// Keine Schreiberlaubnis
		{
		m_iErrorCode=CAF_ERROR_WRITE;
		PROGN(CAF_PROG_ERROR,iNum);
		return m_iErrorCode;
		}

	if((unsigned)iNum>=(unsigned)m_iFileCount)					// Falscher Index
		{
		m_iErrorCode=CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,iNum);
		return m_iErrorCode;
		}

	TarSetDel(iNum);

	m_iSortMode  = 0;
	m_iErrorCode = CAF_ERROR_OK;


return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
// Bearbeitung abbrechen ohne Daten zu speichern.
void CTarFile::Break()
{
	ReleaseData();
	m_iOpenMode=CAF_CLOSED;
	Clear();
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das TAR-Archiv und Speichert alle Daten
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//			CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CTarFile::Close()
{
int		iRet;




	m_iErrorCode = CAF_ERROR_OK;

	
	if(m_iOpenMode==CAF_CLOSED)
		{
		m_iCreated   = 0;
		m_iOpenMode  = CAF_CLOSED;
		m_iErrorCode = CAF_ERROR_MISS;

		PROGN(CAF_PROG_ERROR,-1);

		return CAF_ERROR_MISS;
		}

	if(m_iOpenMode==CAF_READ)
		{
		CTarFile::Break();
		
		m_iCreated	 = 0;
		m_iOpenMode  = CAF_CLOSED;

		return CAF_ERROR_OK;
		}

	if(m_iOpenMode!=CAF_WRITE)
		{
		CTarFile::Break();

		m_iCreated	 = 0;
		m_iOpenMode  = CAF_CLOSED;
		m_iErrorCode = CAF_ERROR_LOGIC;

		PROGN(CAF_PROG_ERROR,-1);

		return CAF_ERROR_LOGIC;
		}


//***************** Alte Dateien lschen neue speichern ***********************


	   iRet=Flush();
	if(iRet!=CAF_ERROR_OK)
	if(iRet!=CAF_ERROR_BREAK)
		{
		CTarFile::Break();
		m_iErrorCode = iRet;
		m_iCreated   = 0;
		return iRet;
		}


//******************** Neue speichern *****************************************

	m_iCreated = 0;

	CTarFile::Break();



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		FlushCalc
//*
//*****************************************************************************
//	Gre der zu nderneden Dateien 	
aint64	CTarFile::FlushCalc()
{
TarItem	   *pEntry;
aint64		iFull;
aint64		iSize;
aint64		iMin;
int			iPos;
int			iNum;


	iSize = 0;	
												 

	if(m_iFilesNum>m_iFileCount)							// Wurden Dateien gelscht
		{
		iFull	= 0;
		iNum	= 0;
		iMin	= 1;
		iMin  <<= 56;

		for(iPos=0;iPos<m_iFilesNum;iPos++)				
			{
			   pEntry=m_pFiles[iPos];
			if(pEntry->cDelete)
				{
				if(iMin>pEntry->iOff)iMin=pEntry->iOff;
				iSize += 64;
				iNum  += 1;
				continue;
				}

			if(pEntry->cWrite)								// Datei wird neu eingefgt
				{
				iSize += pEntry->iLen;
				}
			else{
				iFull += pEntry->iEnd;
				iFull -= pEntry->iOff;
				}
			}

		if(iFull>iMin && iNum)								// Ab erster gelschten Datei wird verschoben
			{
			iSize += iFull-iMin;
			}
		}
	else{
		for(iPos=0;iPos<m_iFilesNum;iPos++)					// Es werden nur Dateien eingefgt
			{
			   pEntry=m_pFiles[iPos];
			if(pEntry->cDelete)continue;

			if(pEntry->cWrite)
				{
				iSize += pEntry->iLen;
				}
			}
		}


return iSize;
}

//*****************************************************************************
//*
//*		Flush
//*
//*****************************************************************************
//	Speichert alle Dateien in die TAR-Datei ohne END-Blcke
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//			CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Bei schreibe Problemen
//			... siehe Error Codes
int CTarFile::Flush()
{
char	   *pMem;
TarItem	   *pEntry;
HANDLE   	hMapObj;
aint64	 	iStart;
aint64	 	iCount;
aint64	 	iPosR;
aint64	 	iPosW;
aint64	 	iSize;
aint64	 	iLen;
aint64	 	iOff;
int		 	iAdd;
int		 	iNum;
int			i;



	m_iDoneMode	 = 0;
	m_iDoneNow	 = 0;
	m_iDoneMax	 = 0;

	if(m_iOpenMode!=CAF_WRITE)
		{
		m_iErrorCode = CAF_ERROR_MISS;

		return CAF_ERROR_MISS;
		}

	if(m_iChanged!=1)								// Gab es nderungen
		{
		return CAF_ERROR_OK;
		}

	m_iDoneMax	= FlushCalc();
	iPosW		= 0;



//***************** Alte Dateien lschen **************************************

	
	iLen = MainSize();

	try	{											
		PROG(CAF_PROG_FLUSH,-1);

		if(m_iFilesNum>m_iFileCount)
			{
			Sort(CAF_FLAG_SORTPOS);					// Nach Offsets sortieren

			if(iLen<0x10000000 || m_hFile==INVALID_HANDLE_VALUE)
				{									// Via Memory-Map oder Speicher
				if(m_hFile==INVALID_HANDLE_VALUE)
					{
					hMapObj	= 0;
					pMem	= m_pMemPtr;
					}
				else{
						hMapObj=CreateFileMapping(m_hFile,0,PAGE_READWRITE,0,(int)iLen,0);
					if(!hMapObj)throw CAF_ERROR_WRITE;
						pMem=(char*)MapViewOfFile(hMapObj,FILE_MAP_WRITE,0,0,(int)iLen);
					}

				if(!pMem)							// Prfe den Speicherzeiger
					{
					CloseHandle(hMapObj);
					throw CAF_ERROR_WRITE;
					}

				for(i=0;i<m_iFilesNum;i++)			// Dateien lschen
					{
					   pEntry=m_pFiles[i];
					if(pEntry->cWrite )continue;
					if(pEntry->cDelete)continue;

					PROG(CAF_PROG_WRITE,i);

					iPosR   = pEntry->iOff;
					iCount  = pEntry->iEnd-iPosR;


					if(iPosR!=iPosW)
						{
						if(iPosW+iCount>iLen)
							{
							UnmapViewOfFile(pMem);	// Mapping aufheben
							CloseHandle(hMapObj);	// Objekt schlieen
							throw CAF_ERROR_LOGIC;
							}

						for(iOff=0;iOff<iCount;iOff+=iSize)
							{
							   iSize = iCount-iOff;
							if(iSize>4096)iSize=4096;

							memmove(pMem+iPosW+iOff,pMem+iPosR+iOff,(int)iSize);

							m_iDoneNow += iSize;
							}

						pEntry->iOff = iPosW;
						pEntry->iEnd = iPosW+iCount;
						}

					m_iDoneNow  += iCount;
					iPosW       += iCount;
					}

				if(hMapObj)
					{
					UnmapViewOfFile(pMem);			// Mapping aufheben
					CloseHandle(hMapObj);			// Objekt schlieen
					}

				m_iChanged    = 1;
				m_iFileEnd    = iPosW;
				m_iFileSize	  = iPosW;
				}
			else{									// Mit Umkopieren
				unsigned char	ucBuffer[0x4000];


				for(i=0;i<m_iFilesNum;i++)			// Dateien lschen
					{
					   pEntry=m_pFiles[i];
					if(pEntry->cWrite )continue;
					if(pEntry->cDelete)continue;

					PROG(CAF_PROG_WRITE,i);

					iPosR   = pEntry->iOff;
					iCount  = pEntry->iEnd-iPosR;


					if(iPosR!=iPosW)
						{
						if(iPosW+iCount>iLen)
							{
							throw CAF_ERROR_LOGIC;
							}

						for(iOff=0;iOff<iCount;iOff+=iSize)
							{
							   iSize = iCount-iOff;
							if(iSize>sizeof(ucBuffer))
								{
								iSize=sizeof(ucBuffer);
								}

							MainSeek (iPosR+iOff,SEEK_SET);
							MainRead (ucBuffer,(int)iSize);
							MainSeek (iPosW+iOff,SEEK_SET);
							MainWrite(ucBuffer,(int)iSize);

							m_iDoneNow += iSize;
							}

						pEntry->iOff = iPosW;
						pEntry->iEnd = iPosW+iCount;
						}

					m_iDoneNow  += iCount;
					iPosW       += iCount;
					}

				m_iChanged    = 1;
				m_iFileEnd    = iPosW;
				m_iFileSize	  = iPosW;
				}
			}


		MainSeek (m_iFileSize);
		EndOfFile();
		}
	catch(int iError)								// Bekannter Fehler
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		return iError;
		}


//***************** Neue Dateien speichern ************************************


	try	{
		iPosW = MainSeek(m_iFileEnd);

		for(i=0;i<m_iFilesNum;i++)					// Dateien speichern
			{
			pEntry = m_pFiles[i];
			iStart = m_iDoneNow;

			if(pEntry->cWrite && !pEntry->cDelete)
				{
				m_iChanged = 1;

				try	{
					Write(i);						// Komprimiere Datei
					}
				catch(int iError)
					{
					m_iErrorCode = iError;

					if(iError==CAF_ERROR_BREAK)break;
					if(iError!=CAF_ERROR_FREAD)
					if(iError!=CAF_ERROR_FOPEN)throw iError;

					PROGN(CAF_PROG_ERROR,i);
					pEntry->cDelete=1;
					m_iFileCount--;
					}

				m_iFileSize		= MainTell();
				m_iFileEnd		= m_iFileSize;
				m_iDoneNow		= iStart+pEntry->iLen;
				}
			}
		}
	catch(int iError)								// Bekannter Fehler
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		CTarFile::Break();
		return iError;
		}


//***************** Glscht markierte Blcke entfernen ************************


	for(i=m_iFilesNum-1;i>=0;i--)
		{
		   pEntry=m_pFiles[i];
		if(pEntry->cDelete)TarDel(i);
		}

	if(m_iModeFlags&CAF_FLAG_SORT)Sort();			// Liste sortieren
	m_iDoneNow = m_iDoneMax;



//******************** Letzen Header speichern ********************************


	try	{
		char		cBuffer[1024+16];
		TarItem		sTemp;

		PROG(CAF_PROG_CLOSE,-1);

		if(m_iChanged==1)
			{
			sTemp.iNameLen = 10;
			sTemp.pName    = Z("TRAILER!!!");
			sTemp.cType	   = 'L';
			sTemp.iLen	   = 0;
			sTemp.iTime    = 0;
			sTemp.iAttrib  = 0;

			switch(m_iFileFormat)
				{
			case 0:	m_iFileFormat = 1;

			case 1:	memset(cBuffer,0,sizeof(cBuffer));
					iNum = sizeof(cBuffer);
					break;

			case 2:	iNum = MakeCpio1Header(cBuffer,&sTemp);
					iAdd = (int)(-m_iFileEnd-iNum)&0x1FF;
					memset(cBuffer+iNum,0,iAdd);
					iNum += iAdd;
					break;

			case 3:	iNum = MakeCpio2Header(cBuffer,&sTemp);
					iAdd = (int)(-m_iFileEnd-iNum)&0x1FF;
					memset(cBuffer+iNum,0,iAdd);
					iNum += iAdd;
					break;

			case 4:	iNum = MakeCpio3Header(cBuffer,&sTemp);
					iAdd = (int)(-m_iFileEnd-iNum)&0x1FF;
					memset(cBuffer+iNum,0,iAdd);
					iNum += iAdd;
					break;

			case 5:	iNum = MakeCpio3Header(cBuffer,&sTemp);

					for(iAdd=0;iAdd<13;iAdd++)
						{
						short *pWords=(short*)cBuffer;
						pWords[iAdd] = btol_w(pWords[iAdd]);
						}

					iAdd = (int)(-m_iFileEnd-iNum)&0x1FF;
					memset(cBuffer+iNum,0,iAdd);
					iNum += iAdd;
					break;

				};

			MainSeek (m_iFileEnd);
			MainWrite(cBuffer,iNum);
			EndOfFile();

			m_iChanged = 2;
			}
		}
	catch(int iError)									// Bekannter Fehler
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		m_iCreated = 0;
		return iError;
		}


	m_iErrorCode = CAF_ERROR_OK;


return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		ExtractProc
//*
//*****************************************************************************
//	Extrahiert eine Datei aus dem Archiv ber eine Ausgabefunktion
//	iNum	: Ist die Indexnummer der Datei die geprft werden soll
//	pParam	: Ist ein Parameter der an die Ausgabefunktionbergeben wird
//	pProc	: Ist der Zeiger auf die Ausgabefunktion
//	pInfo	: Hier werden Zeiten und Attribute der Datei gespeichert.
//			  Bei NULL werden diese Werte ignoriert.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_OK		= Wenn alles korrekt war
//				CAF_ERROR_MISS		= Wenn die Datei nicht gefunden wurde
//				CAF_ERROR_PATH		= Pfad ungltig ist.
//				CAF_ERROR_FCREATE	= Wenn Datei nicht erzeugt werden konnte
//				CAF_ERROR_CRC		= Crc32 Fehler
//				CAF_ERROR_DECOMP	= Fehler beim Entkomprimieren
//				CAF_ERROR_IGNORE	= Die Datei wurde bersprungen
//				... siehe Fehler Codes
int	CTarFile::ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo)
{
char			cBuffer[0x4000];
TarItem		   *pEntry;
aint64			iSize;
aint64			iLen;
aint64			iPos;
int				iMax;
int				iVal;



	if(m_iDoneMode<=1)										// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}

	m_iErrorCode	= CAF_ERROR_OK;
	m_pErrorFile	= NULL;
	pEntry			= NULL;


	try	{
		if(pProc==(OutProc)NoneWrite)						// Kommt der Aufruf von Test(...)
				iVal=CAF_PROG_TEST;
		else	iVal=CAF_PROG_READ;

		PROG(iVal,iNum);

		if((unsigned)iNum>=(unsigned)m_iFilesNum)			// Den Index Prfen
			{
			m_iErrorCode = CAF_ERROR_MISS;
			PROGN(CAF_PROG_ERROR,-1);
			goto Exit;
			}

			pEntry=m_pFiles[iNum];
		if(!pEntry || !pEntry->cSaved)						// Ist die Datei nicht im Arschiv
			{
			m_iErrorCode = CAF_ERROR_MISS;
			PROGN(CAF_PROG_ERROR,-1);
			goto Exit;
			}

		m_pErrorFile = pEntry->pName;
		m_iDoneMax   = pEntry->iLen;

		MainSeek(pEntry->iOff+pEntry->iLenHdr);

		iLen = pEntry->iLen;

		if(m_pMemPtr)										// Daten aus Speicher holen 		
			{
			for(iPos=0;iPos<iLen;iPos+=iSize)
				{
				   iSize = iLen-iPos;
				if(iSize>0x4000)
					{
					iSize = 0x4000;
					}

				iMax  = (m_iMemEnd>=0)? m_iMemEnd:m_iMemLen;
				iMax -=  m_iMemPos;

				if(iMax<iSize)								// Prfe Offset im Speicher
					{
					m_iErrorCode = CAF_ERROR_FREAD;
					PROGN(CAF_PROG_ERROR,iNum);
					goto Exit;
					}

				if(pProc(pParam,m_pMemPtr+m_iMemPos,(int)iLen)!=iLen)
					{
					m_iErrorCode = CAF_ERROR_FWRITE;
					PROGN(CAF_PROG_ERROR,iNum);
					goto Exit;
					}

				m_iMemPos  += (int)iSize;
				m_iDoneNow +=      iSize;
				}
			}
		else{												// Daten aus Datei holen
			for(iPos=0;iPos<iLen;iPos+=iSize)
				{
				   iSize = iLen-iPos;
				if(iSize>sizeof(cBuffer))
					{
					iSize=sizeof(cBuffer);
					}

				   iVal = MainRead(cBuffer,(int)iSize);
				if(iVal<=0)
					{
					m_iErrorCode = CAF_ERROR_FREAD;
					PROGN(CAF_PROG_ERROR,iNum);
					goto Exit;
					}

				if(pProc(pParam,cBuffer,iVal)!=iVal)
					{
					m_iErrorCode = CAF_ERROR_FWRITE;
					PROGN(CAF_PROG_ERROR,iNum);
					goto Exit;
					}

				m_iDoneNow += iSize;			  
				}
			}

		
		if(pProc(pParam,NULL,0)!=0)							// Ende anzeigen
			{
			m_iErrorCode = CAF_ERROR_FWRITE;
			PROGN(CAF_PROG_ERROR,iNum);
			goto Exit;
			}

		if(pInfo)											// Dateidaten speichern
			{
			pInfo->dwAttr		=(pEntry->iAttrib&0200)? SYS_WRITE|SYS_READ:SYS_READ;
			pInfo->dwAttr	   |=(pEntry->iLen>>(32-SYS_HIGHSHIFT))&SYS_HIGHMASK;
			pInfo->dwSize		=(unsigned)pEntry->iLen;
			pInfo->dwTimeWrite	= pEntry->iTime;
			pInfo->dwTimeCreate	= 0;
			pInfo->dwTimeAccess	= 0;
			pInfo->cName[0]		= 0;
			}

		PROG(CAF_PROG_EXTRACT,iNum);
		}
	catch(int iError)
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}

Exit:

	if(m_iDoneMode<=1)										// Die Verlaufsanzeige zurcksetzen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}


return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Stats
//*
//*****************************************************************************
//	Die Zustandbits abfragen
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//	iDel		: Welche Bits sollen gelscht werden
//	Ergibt einen Wert mit veschiedenen Bits:
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//					CAF_STAT_DELETE		0x0004	Die Datei wurde aus dem Verzeichnis gelscht
//					CAF_STAT_WRITE		0x0008	Die Datei muss ins Archiv geschrieben werden
//					CAF_STAT_DIR		0x0010	Die Datei ist ein Verzeichniss
//					CAF_STAT_READONLY	0x0020	Die Datei ist schreibgeschtzt
//					CAF_STAT_SAVED		0x0040	Die Datei ist im Archiv gespeichert
int CTarFile::Stats(int iNum)
{
TarItem	   *pEntry;
int			iBits;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return 0;
		}

			pEntry=m_pFiles[iNum];
	iBits = pEntry->cDecomp&3;

	if(  pEntry->cDecomp      )iBits|=CAF_STAT_DECOMP;
	if(  pEntry->cDelete      )iBits|=CAF_STAT_DELETE;
	if(  pEntry->cWrite       )iBits|=CAF_STAT_WRITE;
	if(  pEntry->cSaved       )iBits|=CAF_STAT_SAVED;
	if(  pEntry->cType==LF_DIR)iBits|=CAF_STAT_DIR;
	if(!(pEntry->iAttrib&0200))iBits|=CAF_STAT_READONLY;
	

return iBits;	
}

//*****************************************************************************
//*
//*		SetStats
//*
//*****************************************************************************
//	Den Zustand einer Arichive Datei verndern.
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//					CAF_STAT_NONE		0x0000	Die Datei wurde noch nicht entpackt
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//	Ergibt den alten Wert oder -1 bei einem Fehler.
int CTarFile::SetStats(int iNum,int iSet)
{
TarItem	   *pEntry;
int			iOld;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return -1;
		}

	pEntry			= m_pFiles[iNum];
	iOld			= pEntry->cDecomp;
	pEntry->cDecomp = iSet;
	

return iOld;	
}

//*****************************************************************************
//*
//*		SetDoneMode
//*
//*****************************************************************************
//	Modus fr die Verlaufsanzeige setzen.
//	iMode		: Ist der neue Modus
//				    0 = Zurck zu Normal (Ende)
//					1 = Begin von Extract (einzelene Datei)
//					2 = Begin von ExtractAll
//	Ergibt den vorherigen Modus.
int CTarFile::SetDoneMode(int iMode)
{
int		iRet;

	
	iRet = m_iDoneMode;
	
	switch(iMode)
		{
	case 0:												// Zurck zu Normal (Ende)

		if(m_iDoneMode==1 || m_iDoneMode==2)
		if(m_iErrorCode==CAF_ERROR_OK)
			{
			m_iDoneNow  = m_iDoneMax;
			}

		m_iDoneMode	 = 0;
		break;

	case 1:												// Begin von Extract (einzelene Datei)

		if(iRet==2)return iRet;
		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 0;
		break;

	case 2:												// Begin von ExtractAll

		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 1;
		break;
		}


return iRet;
}



//*****************************************************************************
//*
//*		AddPath
//*
//*****************************************************************************
//	Fgt dem TAR-Archiv einen Pfad hinzu. Der Pfad muss dabei nicht real 
//	existieren. ber die Flags kann mann die Speicherungsart festlegen.
//			CAF_FLAG_FULLPATH	0x0880	 Ganze Namen mit  Laufwerken
//	       	CAF_FLAG_FULLNAME	0x0800	 Ganze Namen ohne Laufwerken
//	pFileName	: ist der Dateiname fr den Pfad
//	Ergibt CAF_ERROR_OK wenn alles ok war bzw. einen Fehlercode
int CTarFile::AddPath(const zchar *pFileName)
{
TarItem		   *pEntry;
zchar		   *pName;
time_t			iTime;
aint64			iOff;
int				iPos;
int				iNum;
int				i;


	m_iErrorCode=CAF_ERROR_OK;

	try	{
		if(m_iOpenMode!=CAF_WRITE)throw CAF_ERROR_WRITE;

		pName=(zchar*)pFileName;

		if(!(m_iModeFlags&0x0080))						// Keine Laufwerksbuschstaben speichern
			{
			iPos=strcwlen(pName,0x00700000);
			if(pName[iPos]==':')pName+=iPos+1;
			}

		if(*pName=='\\')pName++;
		if(*pName=='/' )pName++;


		if(m_iModeFlags&CAF_FLAG_NOFIND)				// Nicht prfen ob die Datei im Archiv ist
			{
			iNum = -1;
			}
		else{
			   iNum=Find(pName);
			if(iNum>=0 && (m_iModeFlags&CAF_FLAG_NOEXIST))
				{
				return CAF_ERROR_OK;
				}

			if(iNum>=0)
				{
				TarSetDel(iNum);
				}
			}


		iTime	= time(0);								// Neue Liste erzeugen
		iNum	= TarNew();
		iOff	= MainSeek(m_iFileEnd,SEEK_SET);
		pEntry	= m_pFiles[iNum];


		i = strzlen(pName);								// Eintrag erzeugen
		pEntry->cDelete		=  1;
		pEntry->cWrite		=  1;
		pEntry->cSaved		=  0;
		pEntry->iNameLen    =  i;
		pEntry->iLen		=  0;
		pEntry->iOff		=  iOff;
		pEntry->iTime		=  iTime;
		pEntry->cType		=  LF_DIR;
		pEntry->pName		=  new zchar[i+1];

		memcpy(pEntry->pName,pName,(i+1)*sizeof(zchar));
		CharToPath(pEntry->pName);

		PROGN(CAF_PROG_WRITE,iNum);

		Write(iNum);

		m_iFileSize			=  pEntry->iEnd;
		m_iFileEnd			=  pEntry->iEnd;
		m_iChanged			=  1;
		m_iSortMode 		=  0;

		if(m_iModeFlags&CAF_FLAG_SORT)					// Sortieren ?
			{
			Sort();
			}
		}
	catch(int iError)
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		return iError;
		}


return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		AddProc
//*
//*****************************************************************************
//	Fgt dem TAR-Archiv eine Datei hinzu, die Daten wird ber ein Callback 
//	geladen. ber die Flags() kann mann die Speicherungsart der Namens 
//	im Archiv festlegen:
//					CAF_FLAG_FULLPATH	0x0880	 Ganze Namen mit  Laufwerken
//	       			CAF_FLAG_FULLNAME	0x0800	 Ganze Namen ohne Laufwerken
//	pFileName	: Ist der Dateiname fr die Datei
//	pParam		: Ist der Zeiger der an die Funktion bergeben wird.
//	pProc		: Ist der Zeiger auf die Callback-Funktion.
//					Mit  0 signalisiert der Callback das Dateiende 
//					Mit -1 signalisiert der Callback einen Fehler
//	pInfo		: Enthlt Zeiten, Attribute und Gre der Datei.
//				  Bei NULL werden diese Werte ignoriert.
//				  Mit "*...<Path>..." kann eine Datei direkt bergeben werden
//	Ergibt CAF_ERROR_OK wenn alles ok war oder einen Fehlercode.
int CTarFile::AddProc(const zchar *pFileName,void *pParam,InpProc pProc,FileData *pInfo)
{
char			cBuffer[0x4000];
TarItem		   *pEntry;
zchar		   *pName;
unsigned 		uAttr;
time_t			iTime;
aint64			iSize;
aint64			iOff;
aint64			iPos;
int				iNum;
int				iIdx;
int				iAdd;
int				iLen;


	if(pInfo && (pInfo->dwAttr&SYS_DIR))				// Ein Verzeichniss einfgen
		{
		return AddPath(pFileName);
		}

	m_iErrorCode=CAF_ERROR_OK;

	try	{
		if(m_iOpenMode!=CAF_WRITE)throw CAF_ERROR_WRITE;

		pName=(zchar*)pFileName;

		if(!(m_iModeFlags&0x0080))						// Keine Laufwerksbuschstaben speichern
			{
			iPos=strcwlen(pName,0x00700000);
			if(pName[iPos]==':')pName+=iPos+1;
			}

		if(*pName=='\\')pName++;
		if(*pName=='/' )pName++;

		if(m_iModeFlags&CAF_FLAG_NOFIND)				// Nicht prfen ob die Datei im Archiv ist
			{
			iNum = -1;
			}
		else{
			   iNum=Find(pName);
			if(iNum>=0 && (m_iModeFlags&CAF_FLAG_NOEXIST))
				{
				return CAF_ERROR_OK;
				}
			
			if(iNum>=0)
				{
				TarSetDel(iNum);
				}
			}

		if(pInfo)										// Gibt es Infos zur neuen Datei
			{
			if(pInfo->dwAttr&SYS_WRITE)
					uAttr = 0100664;
			else	uAttr = 0100444;

			if(pInfo->dwTimeWrite!=0 && pInfo->dwTimeWrite!=0xFFFFFFFF)
					iTime = pInfo->dwTimeWrite;
			else	iTime = time(0);		

			iSize	= (pInfo->dwAttr&SYS_HIGHMASK)>>SYS_HIGHSHIFT;
			iSize <<= 32;
			iSize  += pInfo->dwSize;

			if(pProc==CAF_ADD_FILE)						// Die Datei erst mit Flash() speichern
					iAdd = strzlen(pInfo->cName)+1;			
			else	iAdd = 0;
			}
		else{
			iTime	= time(0);							// Aktuelle Zeit benutzen
			uAttr	= 0100664;
			iSize	= 0;
			iAdd	= 0;
			}

		
		iLen				=  strzlen(pName);			// Eintrag erzeugen
		iNum				=  TarNew();
		pEntry				=  m_pFiles[iNum];
		pEntry->iNameLen    =  iLen;
		pEntry->iLen		=  iSize;
		pEntry->iOff		=  m_iFileEnd;
		pEntry->iTime		=  iTime;
		pEntry->iAttrib		=  uAttr;
		pEntry->cType		=  LF_NORMAL;
		pEntry->pName		=  new zchar[iLen+1+iAdd];

		memcpy(pEntry->pName,pName,(iLen+1)*sizeof(zchar));
		CharToPath(pEntry->pName);

		if(iAdd)										// Die Datei erst mit Flash() speichern
			{
			pEntry->cWrite  = 1;
			pEntry->pPath	= pEntry->pName+iLen+1;
			memcpy(pEntry->pPath,pInfo->cName,iAdd*sizeof(zchar));

			return CAF_ERROR_OK;
			}

		PROGN(CAF_PROG_WRITE,iNum);

		pEntry->cSaved		=  1;

		iOff = MainSeek(m_iFileEnd,SEEK_SET);
		iAdd = WriteHeader(pEntry,0);


		for(iIdx=0;;iIdx+=iLen)							// Daten speichern
			{
			   iLen = pProc(pParam,cBuffer,sizeof(cBuffer));
			if(iLen <0)throw CAF_ERROR_READ;
			if(iLen==0)break;

			MainWrite(cBuffer,iLen);
			}


		iPos = MainTell();

		if(MainSeek(iOff,SEEK_SET)!=iOff)				// Zurck zum Header
			{
			throw CAF_ERROR_LOGIC;
			}


		iAdd = WriteHeader(pEntry,iIdx);


		if(MainSeek(iPos,SEEK_SET)!=iPos)				// Zurck zum Ende
			{
			throw CAF_ERROR_LOGIC;
			}

		if(iAdd>0)										// Fllbytes speichern
			{
			memset  (cBuffer,0,iAdd);
			MainWrite(cBuffer  ,iAdd);
			}

		pEntry->iEnd	=  MainTell();
		m_iFileSize		=  pEntry->iEnd;
		m_iFileEnd		=  pEntry->iEnd;
		m_iChanged		=  1;
		m_iSortMode		=  0;

		if(m_iModeFlags&CAF_FLAG_SORT)					// Sortieren ?
			{
			Sort();
			}
		}
	catch(int iError)
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		return iError;
		}


return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		OctToInt
//*
//*****************************************************************************
//	Wantelt eine Oktal-Sting in eine Zahl um
unsigned CTarFile::OctToInt(const char *pText)
{
unsigned 	uChar;
unsigned 	uVal;
int			iPos;


	uVal = 0;

	for(iPos=0;iPos<12;iPos++)
		{
		if(pText[iPos]!=' ')break;
		}

	for(;iPos<12;iPos++)
		{
		uChar  = pText[iPos];
		uChar -= '0';

		if(uChar>7)break;

		uVal<<=3;
		uVal += uChar;
		}

return uVal;
}


//*****************************************************************************
//*
//*		Count
//*
//*****************************************************************************
//	Ergibt die Anzahl der Eintrge in der Dateiliste
int CTarFile::Count()
{
	return m_iFilesNum;
}

//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CTarFile::Done(aint64 *pMax)
{
aint64	iRet;


	iRet = m_iDoneNow;

	if(iRet>m_iDoneMax)iRet=m_iDoneMax;
	if(pMax)*pMax=m_iDoneMax;


return iRet;
}

//*****************************************************************************
//*
//*		Change
//*
//*****************************************************************************
//	Schreiberlaubnis de/aktivieren
int CTarFile::Change(int iMode)
{

														// Prfe den Modus
	if(iMode<CAF_READ || iMode>CAF_WRITE || m_iOpenMode<=CAF_CLOSED)				
		{
		return CAF_ERROR_MODE;
		}
	
	if(m_iOpenMode==iMode)								// Gleicher Modus
		{
		return CAF_ERROR_OK;
		}

	if(m_iChanged==1 && iMode==CAF_READ)				// Gibt es nderungen
		{
		Flush();
		}
	
	m_iDoneMode	 = 0;
	m_iDoneNow	 = 0;
	m_iDoneMax	 = 0;


return DoChange(iMode);
}

//*****************************************************************************
//*
//*		GetUserPtr
//*
//*****************************************************************************
//	User-Pointer einer einzelnen Datei abfragen.
void *CTarFile::GetUserPtr(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)			// Dateinummer prfen		
		{
		return 0;
		}

return m_pFiles[iNum]->pUserPtr;
}

//*****************************************************************************
//*
//*		SetUserPtr
//*
//*****************************************************************************
//	User-Pointer einer einzelnen Datei einstellen.
void CTarFile::SetUserPtr(int iNum,void *pData)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)			// Dateinummer prfen		
		{
		return;
		}

	m_pFiles[iNum]->pUserPtr = pData;

}

//*****************************************************************************
//*
//*		IsDel
//*
//*****************************************************************************
//	Ergibt 1 wenn die Datei gelscht wurde.
int CTarFile::IsDel(int iNum)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->cDelete;
}


//*****************************************************************************
//*
//*		IsReadOnly
//*
//*****************************************************************************
//	Ergibt 1 wenn die Datei schreibgeschtzt ist.
int CTarFile::IsReadOnly(int iNum)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return (m_pFiles[iNum]->iAttrib&0200)? 0:1;
}

//*****************************************************************************
//*
//*		Attribute
//*
//*****************************************************************************
//	UNIX-Dateiattribute auslesen
//	iNum		: Ist die Nummer der Datei
//	Ergibt die Attribute bzw. -1 bei einem Fehler
//					0x00000001	= Execute Andere
//					0x00000002	= Write   Andere
//					0x00000004	= Read    Andere
//					0x00000080	= Execute Gruppe
//					0x00000100	= Write   Gruppe
//					0x00000200	= Read    Gruppe
//					0x00000400	= Execute Besitzer
//					0x00000800	= Write   Besitzer
//					0x00001000	= Read    Besitzer
//					0x00004000	= Verzeichnis
//					0x80000000	= Die Attribute sind nachgebiltet
int CTarFile::Attribute(int iNum)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->iAttrib;
}


//*****************************************************************************
//*
//*		SetScale
//*
//*****************************************************************************
//	Skalierung fr die komprimierte Gre einstellen.
//	iByte	: Ist die entkomprimierte Gre 
//	iSize	: Ist die komprimierte Gre 
void CTarFile::SetScale(aint64 iByte,aint64 iSize)
{

	if(iByte>256)										
		{			
		iSize	   *= 1024;
		iSize	   /= iByte;
		m_uScale	= (unsigned)iSize;

		if(m_uScale<4)m_uScale=4;
		}
	else{
		m_uScale	= 0;
		}

}