//*****************************************************************************
//*
//*
//*		TarFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf TAR-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef 	__TARFILE_H__X
#define 	__TARFILE_H__X

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	"ArchiveFile.h"
#include	"FileFinder.h"
#include	"FileFunctions.h"

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif



//*****************************************************************************
//*
//*		Klasse CTarFile
//*
//*****************************************************************************
class CTarFile : public CArchiveFile
{
protected:

	#define		TAR_DUMMY_HEADER	1000		// Einen Dummy-Header einfgen
	#define 	TAR_RECORDSIZE		512			// Gre eines TAR-Headers
	#define 	TAR_NAMSIZE    		100			// Maximale Lnge des Dateinamens im Header
	#define 	TAR_TUNMLEN    		32			// Maximale Lnge des User-Namens im Header
	#define 	TAR_TGNMLEN    		32			// Maximale Lnge des Gruppennamens im Header

	typedef struct
		{
		void   *pUserPtr;						// User-Pointer
		zchar  *pName;							// Dateinames des Eintrages
		zchar  *pPath;							// Dateipfad der einzufgenden Datei
		char	cType;							// Ist der Eintrag Typ (LF_???)
		char	cWrite;							// Soll der Eintrag geschrieben werden
		char 	cSaved;							// Ist Block in Datei
		char	cError;							// Ist die Datei defekt
		char	cDelete;						// Soll der Eintrag gelscht werden
		char	cDecomp;						// 0=Fehler  1=war vor handen  2=entpackt
		int		iAttrib;						// Sind die Unix Attribute
		int		iNameLen;						// Ist die Lnge des Namens in Zeichen
		int 	iLenHdr;						// Ist die Lnge des Headers
		time_t	iTime;							// Ist die nderungszeit
		aint64	iLen;							// Ist die Gre der Datei
		aint64	iOff;							// Ist der Offset in der TAR-Datei (vom Header)
		aint64	iEnd;							// Ist der Offset des Endes (mit Fllbytes)
		}TarItem;

	CFileFinder	m_sFind;						// Dateien suchen

	TarItem   **m_pFiles;						// Alfabethische Liste
	int			m_iFilesNum;					// Anzahl der Listeneintrge
	int			m_iFilesMax;					// Gesamt Listengre in Eintrgen
	int			m_iFileCount;					// Listen Eintrge ohne gelschte
	int			m_iFileFormat;					// Ist das Format der Datei (0=unbekannt 1=TAR 2..5=CPIO)
	int			m_iCreateFormat;				// Zum Erstellen des Archives (0= 1=TAR 2..5=CPIO)
	int			m_iCreated;						// Wurde die Datei neu erstellt
	aint64		m_iFileEnd;						// Position des letzten Headers
	unsigned	m_uScale;						// Skalierung fr Komprimierte Gre (1/1024)

	int			m_iChanged;						// Wurde die TAR-Datei verndert
	int			m_iChangeFlag;					// Wurde die TAR-Datei verndert (bleibt nach close)
	int			m_iDoneMode;					// Verlaufmodus 0=normal 1=ExtractAll
	aint64		m_iDoneNow;						// Aktueller Verlauf
	aint64		m_iDoneMax;						// Maximaler Verlauf

	char 		m_cIsCpio;						// CPIO-Flag fr Dummy-Header
	int			m_iTarOffset;					// Offset fr Dummy-Header
	char 		m_aTarBuffer[512];				// Puffer fr Dummy-Header


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

					CTarFile();
	virtual		   ~CTarFile();
	
	virtual	int     AddPath(const zchar  *pFileName);			// Ein Verzeichniss einfgen
	virtual	int		AddProc(const zchar  *pFileName,void *pParam,InpProc pProc,FileData *pInfo=0);
	virtual	int		ExtractProc(int iNum           ,void *pParam,OutProc pProc,FileData *pInfo=0);
	virtual	int		Change     (int iMode=CAF_WRITE);			// Schreiberlaubnis aktivieren

	virtual	int		Flush      ();								// Datei speichern
	virtual	int		Close      ();								// Datei schliesen
	virtual	int	    Count      ();								// Anzahl der Dateien
	virtual	void	Break      ();								// Bearbeitung abbrechen
	virtual	int		Stats	   (int    iNum);					// Die Zustandbits abfragen
	virtual	zchar  *Name       (int    iNum);					// Hole Dateiname
	virtual	zchar  *Name       (int    iNum,int &iLen);			// Hole Dateiname mit Lnge
	virtual	int     SetFail    (int    iNum,int iFail=1);		// Defekt-Flag setzen
	virtual int		Attribute  (int    iNum);					// Datei Attribute auslesen
	virtual	int     IsReadOnly (int    iNum);					// Ist die Datei schreibgeschtzt
	virtual	int     IsFail     (int    iNum);					// Ist die Datei defekt
	virtual	int     IsDel      (int	   iNum);					// Wurde die Datei gelscht
	virtual	int     IsDir      (int    iNum);					// Prfe ob Verzeichnis
	virtual	int     IsOk       (int    iNum);					// Richtig endkomprimiert
	virtual	aint64	Len        (int    iNum,int iMode=0);		// Hole Dateilnge
	virtual	int		Del        (int    iNum);					// Datei lschen
	virtual	time_t	Time       (int    iNum);					// Dateizeit abfragen
	virtual	time_t	Time       (int    iNum,int iMode);			// Dateizeit abfragen
	virtual	int		Sort       (int    iMode=0);				// Sortierte Kette
	virtual	int		Find       (const zchar  *pFileName);		// Datei suchen
	virtual	aint64	Done       (aint64 *pMax=0);				// Verlaufsabfrage
	virtual	void   *GetUserPtr (int iNum            );			// Datenwert auslesen
	virtual	void    SetUserPtr (int iNum,void *pData);			// Datenwert einstellen

protected:

	void			SetScale   (aint64 iByte,aint64 iSize);		// Skalierung fr komprimierte Gre
	virtual	int     SetDoneMode(int iMode);						// Modus fr Done setzen
	virtual	int		SetStats   (int   iNum,int iSet);			// Die Zustand einer Archivdatei setzen
	virtual	int		ArchOpen   (int iLevel);					// ffnet die TAR-Datei
	aint64			FlushCalc  ();								// Gre der zu nderneden Dateien 	
	void			Clear	   ();								// Speicher lschen
	int				Write	   (int iNum);						// Einen Eintrag speichern
	int 			WriteDummy (aint64 iSize);					// Dummy-Header an den Anfang schreiben
	int				WriteHeader  (TarItem *pEntry,aint64 iLen);	// TAR-Header speichern	
	int				WriteLongName(TarItem *pEntry,int    iLen);	// Einen Langen Dateinamen speichern

	int				TarNew     ();								// Listen Eintrag erzeugen
	void			TarDel     (int    iNum);					// Listen Eintrag lschen
	void			TarSetDel  (int    iNum);					// Listen Eintrag lschen

static unsigned 	OctToInt    (const    char *pText);
static unsigned		CalcCheckSum(unsigned char *pBuffer);
static int			MakeTarHeader  (void *pData,TarItem *pEntry);
static int			MakeCpio1Header(void *pData,TarItem *pEntry);
static int			MakeCpio2Header(void *pData,TarItem *pEntry);
static int			MakeCpio3Header(void *pData,TarItem *pEntry);
static int	_cdecl	WriteToTar(void *pPtr,const char *p,int i);	// Daten in die temporre TAR-Datei schreiben
static int __cdecl 	CmpName (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpLen  (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpPos  (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpEx   (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
};

#endif



