//*****************************************************************************
//*
//*
//*		TgzFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf TGZ-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *Type=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *Type=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractMem(int iNum,char *pMem,int)		// Datei in Speicher extrahieren
// 		int		ExtractAll(         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(   int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *Datei,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *Datei)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int		Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *Datei);					// Datei suchen
// 		time_t	Time (char  *Datei);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen

#include	<time.h>
#include	<stdio.h>
#include	<string.h>
#include	<sys/stat.h>
#include	"TgzFile.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		wcslen
#define 	strzcmp		wcscmp
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#define 	strzncat	wcsncat
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		strlen
#define 	strzcmp		strcmp
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#define 	strzncat	strncat
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif




#define PROG(iMode,pEntry)								\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))			\
	{													\
	if(Progress(iMode,pEntry))throw CAF_ERROR_BREAK;	\
	}
#define PROGR(iMode,pEntry,iRet)						\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))			\
	{													\
	if(Progress(iMode,pEntry))return iRet;				\
	}
#define PROGN(iMode,pEntry)								\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))			\
	{													\
	Progress(iMode,pEntry);								\
	}

#define 	TGZ_VERSION			"AZ CTgzFile V2.01 (C) 2007"

#define 	TGZ_FLAG_FTEXT		0x01				// Datei ist ein ASCII Text
#define 	TGZ_FLAG_FHCRC		0x02				// Eine CRC16 Summe ist vorhanden
#define 	TGZ_FLAG_FEXTRA		0x04				// Extrafelter sind vorhanden
#define 	TGZ_FLAG_FNAME		0x08				// Ein Dateiname ist angehngt
#define 	TGZ_FLAG_FCOMMENT	0x10				// Ein Commentar ist angehngt


typedef struct
	{
	unsigned char	ucIdenifier1;					// Immer auf 0x1F
	unsigned char	ucIdenifier2;					// Immer auf 0x8B
	unsigned char	ucCompressMode;					// Komprimierungsmodus (0..8, 8=max)
	unsigned char	ucFlags;						// Falgs (siehe TGZ_FLAG_??=)
	unsigned char	ucTime[4];						// Unixzeit der Datei (litle endian)
	unsigned char	ucExtraFlags;					// Erweiterte Flags
	unsigned char	ucOpSystem;						// Bit1=beste Bit3=schnellste Kompression
	}TgzHeader;



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CTgzFile::CTgzFile()
{

	m_iDoneMode		= 0;
	m_iTgzStart		= 0;
	m_cIsCpio		= 0;

}

//*****************************************************************************
//*
//*		Denstruktor
//*
//*****************************************************************************
CTgzFile::~CTgzFile()
{

	if(m_iOpenMode!=CAF_CLOSED)								// Ist die TAR-Datei noch offen
		{
		Close();
		}

	if(m_cTarName[0])										// Lsche temorre TAR-Datei
		{
		DeleteFile(m_cTarName);
		m_cTarName[0]=0;
		}

	if(m_hSubFile!=INVALID_HANDLE_VALUE)
		{
		CloseFile(m_hSubFile);
		m_hSubFile = INVALID_HANDLE_VALUE;
		}
}




//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine BZIP2-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//					CAF_ERROR_OK		Alles ok
//					CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//					CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//					CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//					CAF_ERROR_READ		Bei Lese-Fehlern
//					CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//					CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//					CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CTgzFile::ArchOpen(int iLevel)
{
unsigned char	aBuffer[0x1000];
unsigned		aSuffix[2];
unsigned		uOutLen;
unsigned		uOutCrc;
TgzHeader	  *pHeader;
int				iBytes;
aint64			iByte;
aint64			iSize;
int				iLen;
int				iRet;
int				iOff;
int				iMin;
int				iMax;



	SetDoneMode(-1);										// Done-Modus in Tar-Datei

	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;
	m_iTgzStart = 0;
	m_cIsCpio	= 0;

	Clear();
	InitSubFile();

	   iByte = m_iFileSize+16;
	if(iByte<0x400000)										// Bis 4MB im Speicher ffnen
		{
		iMin	= (int)iByte;
		iMax	= 0x1000000;								// 10MB maximal
		}
	else{
		iMin	= 0;
		iMax	= 0;
		}

	SetMemSub(0x10000,0x30000,m_cTarName,Z(".tar"));		// Dynamischen Speicher erstellen

		

	if(m_iSubMode==CAF_CREATE)								// Neues Archiv erzeugen 
		{
		m_iOpenMode = CAF_CREATE;
		return CTarFile::ArchOpen(iLevel);
		}


	m_iOpenMode = CAF_WRITE;


//******************** Header einlesen ****************************************


	m_cThrowRd	= 0;
	iBytes		= (int)BaseRead(aBuffer,sizeof(aBuffer));
	pHeader		= (TgzHeader*)aBuffer;
	m_cThrowRd	= 1;

	


	try	{
		if(iBytes<16                  )throw CAF_ERROR_FORM;
		if(pHeader->ucIdenifier1!=0x1F)throw CAF_ERROR_FORM;
		if(pHeader->ucIdenifier2!=0x8B)throw CAF_ERROR_FORM;
		if(pHeader->ucCompressMode>8  )throw CAF_ERROR_FORM;

		iOff	= 10;
		iLevel	= pHeader->ucCompressMode;

		if(pHeader->ucFlags&TGZ_FLAG_FEXTRA)				// Extrafelder berspringen
			{
			iOff +=  aBuffer[iOff];
			iOff +=  aBuffer[iOff]*256;
			iOff +=  2;

			if(iOff<iBytes)throw CAF_ERROR_FORM;
			}

		if(pHeader->ucFlags&TGZ_FLAG_FNAME)					// Dateinamen berspringen
			{
			for(;iOff<iBytes;iOff++)
				{
				if(aBuffer[iOff]==0)break;
				}

			if(iOff>=iBytes)throw CAF_ERROR_FORM;
			   iOff++;
			}

		if(pHeader->ucFlags&TGZ_FLAG_FCOMMENT)				// Kommentrar berspringen
			{
			for(;iOff<iBytes;iOff++)
				{
				if(aBuffer[iOff]==0)break;
				}

			if(iOff>=iBytes)throw CAF_ERROR_FORM;
			   iOff++;
			}

		if(pHeader->ucFlags&TGZ_FLAG_FHCRC)					// CRC16-Summe berspringen
			{
			   iOff += 2;
			if(iOff>=iBytes)throw CAF_ERROR_FORM;
			}

		if(iOff+4>m_iSubSize)								// Prfe die Gre
			{
			throw CAF_ERROR_FORM;
			}

		m_iTgzStart  = iOff;

		BaseSeek(-8,SEEK_END);
		BaseRead(aSuffix,8);



//******************** Archiv dekomprimieren **********************************


		if(m_iSubSize<16)throw CAF_ERROR_FORM;
		
		iLen = (int)(m_iSubSize-m_iTgzStart-8);

		m_iOpenMode=CAF_WRITE;
		BaseSeek(m_iTgzStart);
		SetInput(&m_sHuffman,CAF_BASE,iLen);				// Die TGZ-Datei als Eingabe benutzen

		m_sHuffman.SetOut(this,WriteToTar,0);				// Ausgabefunktion fr TAR-Datei
		m_iFileSize		= 0;
		m_iTarOffset	= 0;
		PROG(CAF_PROG_DOEXTRACT,-1);
		m_sHuffman.Reset();
	
		m_uTarSize  = aSuffix[1];
		m_iDoneLen	= m_iSubSize;
		m_iDoneMode = 1;
		SetDoneMode(2);

		try	{												// Endkomprmire die TAR-Datei
			m_cThrowWr = 0;
			m_sHuffman.Decompress();
			m_cThrowWr = 1;
			}
		catch(char*)
			{
			m_cThrowWr = 1;
			throw CAF_ERROR_DECOMP;
			}

		iSize = m_sHuffman.Value64(CHF_LEN);
		
		   iLen = (unsigned)iSize%512;						// Aligment prfen
		if(iLen>0)
			{
			char cBuffer[512];
			iLen = 512-iLen;
			memset(cBuffer,0,iLen);
			MainWrite(cBuffer,iLen);
			}

		if(m_iTarOffset==TAR_DUMMY_HEADER)					// TAR-Header einfgen bei einzelner Datei
			{
			WriteDummy(iSize);
			}
		
		m_iFileSize	= MainSize();
		m_iDoneSave = m_iDoneLen+m_iDoneNow;
		m_iDoneLen	= m_iDoneLen*6-m_iDoneSave;
		m_iDoneNow	= 0;	
		m_iDoneMax	= 0;
		m_iDoneMode	= 2;
		SetDoneMode(-1);


		uOutLen = m_sHuffman.Value(CHF_OUT );
		uOutCrc = m_sHuffman.Value(CHF_OCRC);

		if(aSuffix[0]==0)aSuffix[0]=uOutCrc;
		if(aSuffix[1]==0)aSuffix[1]=uOutLen;


		if(uOutLen!=aSuffix[1] || uOutCrc!=aSuffix[0])		// Prfe CRC-Checksumme
			{
			throw m_iErrorCode = CAF_ERROR_CRC;
			}


		if(m_iSubMode==CAF_READ)							// Lesemodus fr TAR-Datei
			{
			if(DoChange(CAF_READ,0)!=CAF_ERROR_OK)
				{
				return m_iErrorCode;
				}
			}
		else{
			MainSeek(0);
			}

		iRet = CTarFile::ArchOpen(iLevel);					// Die Dateien aus der TAR-Datei einlesen

		if(iRet!=CAF_ERROR_OK)								// Gab es Fehler beim ffnen
			{
			if(m_hFile!=INVALID_HANDLE_VALUE)
				{
				CloseFile(m_hFile);
				m_hFile = INVALID_HANDLE_VALUE;
				}

			DeleteFile(m_cTarName);

			return iRet;
			}

															// Skalierung fr komprimierte Gre	
		SetScale(m_sHuffman.Value64(CBC_LEN),m_sHuffman.Value64(CBC_SIZE));

		}
	catch(int iError)										// Gab es Fehler
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		Break();
		}


	m_iDoneNow	= Done(&m_iDoneMax);						// Aktuellen Verlauf in TAR-Datei bernehmen
	m_iDoneMode	= 0;
	SetDoneMode(-1);										
	
	

return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das TGZ-Archive und Speichert alle Daten
//	Ergibt 0 wenn ok, oder einen Fehlercode ungleich 0:
//			CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CTgzFile::Close()
{
int			iWrite;
int			iRet;




	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;


	if(m_iOpenMode==CAF_READ || !m_iChanged)
			iWrite=0;
	else	iWrite=1;


	if(!iWrite)												// TGZ-Datei nur schlieen
		{
		if(m_hSubFile!=INVALID_HANDLE_VALUE)
			{
			CloseFile(m_hSubFile);
			m_hSubFile=INVALID_HANDLE_VALUE;
			}
		}
	else{													// Alles in der TAR-Datei speichern
		m_iDoneLen	= AllLen   ();
		m_iDoneSave	= FlushCalc();
		m_iDoneMode	= 3;

		   m_iErrorCode =Flush();
		if(m_iErrorCode!=CAF_ERROR_OK)						
			{
			Break();
			return m_iErrorCode;
			}

		m_iDoneMode	= 4;

		   m_iErrorCode = CloseFlush();						// Die TAR-Daten in die TGZ-Datei schreiben	
		if(m_iErrorCode!=CAF_ERROR_OK)
			{
			Break();
			return m_iErrorCode;
			}

		m_iDoneNow	= Done(&m_iDoneMax);					// Aktuellen Verlauf in TAR-Datei bernehmen
		m_iDoneMode	= 0;
		SetDoneMode(-1);										
		}


	   iRet =CTarFile::Close();
	if(iRet!=CAF_ERROR_OK)									// Alles in der TAR-Datei speichern
		{
		m_iErrorCode = iRet;
		}


	if(m_cTarName[0])										// Lsche temorre TAR-Datei
		{
		DeleteFile(m_cTarName);
		m_cTarName[0]=0;
		}

	ReleaseData();



return m_iErrorCode;
}

//*****************************************************************************
//*
//*		CloseFlush
//*
//*****************************************************************************
//	Beim Schlieen der TGZ-Datei die Daten aus der TAR-Datei in die TGZ-Datei komprimieren.
int CTgzFile::CloseFlush()
{
char		cName[256];
DWORD		aSuffix[2];
TgzHeader	sHeader;
int			iLevel;
time_t		iTime;
int 		iSize;
int			iPos;


	try	{
		MainSeek (0);										// Lesezeiger in TAR-Datei an den Anfang

		try	{
			if(m_iTgzStart==0)								// Einen neuen Header erzeugen
				{
				ZeroFile(1);

				if(m_iLevel>0 && m_iLevel<=8)
						iLevel=m_iLevel;
				else	iLevel=8;

				iTime = time(0);
									 
				PROG(CAF_PROG_WRITE,-1);

				sHeader.ucIdenifier1	= 0x1F;
				sHeader.ucIdenifier2	= 0x8B;
				sHeader.ucCompressMode	= iLevel;
				sHeader.ucFlags			= TGZ_FLAG_FNAME;
				sHeader.ucFlags		   |= TGZ_FLAG_FCOMMENT;
				sHeader.ucTime[0]		= (unsigned char)(iTime    );
				sHeader.ucTime[1]		= (unsigned char)(iTime>> 8);
				sHeader.ucTime[2]		= (unsigned char)(iTime>>16);
				sHeader.ucTime[3]		= (unsigned char)(iTime>>24);
				sHeader.ucExtraFlags	= 0x02;
				sHeader.ucOpSystem		= 0x03;

				if(BaseWrite(&sHeader,10)!=10)
					{
					throw CAF_ERROR_WRITE;
					}

				#ifdef UNICODE								// Internen Dateinamenerzeugen

				for(iSize=0,iPos=0;iPos<sizeof(cName)-8;iSize++)
					{
					iPos += chartoutf8(cName+iPos,m_pFileName[iSize]);
					}

				#else

				iPos=(int)strlen(m_pFileName);
				memcpy(cName,m_pFileName,iPos);

				#endif

				for(iSize=iPos;iPos>0;iPos--)				// Dateisuffix auf ".tar" ndern
					{
					if(cName[iPos]=='.')break;	
					}

				if(iPos==0)iPos=iSize;

				memcpy(cName+iPos,".tar",5);
				iPos+=5;

				if(BaseWrite(cName,iPos)!=iPos)
					{
					throw CAF_ERROR_WRITE;
					}

				iPos=(int)strlen(TGZ_VERSION)+1;

				if(BaseWrite(TGZ_VERSION,iPos)!=iPos)
					{
					throw CAF_ERROR_WRITE;
					}

				m_iTgzStart = BaseTell();
				}
			else{
				if(m_iLevel>0 && m_iLevel<=8)
						iLevel=m_iLevel;
				else	iLevel=8;


				PROG(CAF_PROG_WRITE,-1);

				sHeader.ucIdenifier1	= 0x1F;
				sHeader.ucIdenifier2	= 0x8B;
				sHeader.ucCompressMode	= iLevel;

				BaseSeek(0,SEEK_SET);

				if(BaseWrite(&sHeader,3)!=3)
					{
					throw CAF_ERROR_WRITE;
					}
				
				BaseSeek(m_iTgzStart,SEEK_SET);
				EndOfFile(1);
				}


			PROG(CAF_PROG_COMPRESS,-1);

			SetInput (&m_sHuffman,CAF_MAIN,-1,0);			// Die TAR-Datei als Eingabe benutzen
			SetOutput(&m_sHuffman,CAF_BASE     );			// Die TGZ-Datei als Ausgabe benutzen
			m_sHuffman.Compress(m_iLevel);					// Die TAR-Datei in TGZ-Datei komprimieren
			}
		catch(char*)
			{
			throw CAF_ERROR_WRITE;
			}

		aSuffix[0] =		m_sHuffman.GetCrcInp();			// CRC   der TAR-Daten 
		aSuffix[1] = (DWORD)m_sHuffman.GetLength();			// Gre der TAR-Daten

		BaseWrite(aSuffix,8);								// Die Suffix-Bytes speichern 

															// Skalierung fr komprimierte Gre	
		SetScale(m_sHuffman.Value64(CBC_LEN),m_sHuffman.Value64(CBC_SIZE));

		PROG(CAF_PROG_CLOSE,-1);

		m_iErrorCode = CAF_ERROR_OK;
		}
	catch(int iError)
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		return m_iErrorCode;		    
		}


	if(m_hSubFile!=INVALID_HANDLE_VALUE)					// Die TGZ-Datei schlieen
		{
		CloseFile(m_hSubFile);
		m_hSubFile=INVALID_HANDLE_VALUE;
		}



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush,Change ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt die Position des aktuellen Verlaufs.
aint64	CTgzFile::Done(aint64 *pMax)
{
aint64		iNow;
aint64		iMax;
aint64		iOut;
aint64		iAll;


	switch(m_iDoneMode)
		{
	case 1:													// Entpacke das TGZ-Archiv in die TAR-Datei

			iNow	= m_sHuffman.Value64(CHF_INP);			// Bytes aus TGZ-Datei gelesen
			iOut	= CTarFile::Done();						// Bytes in  TAR-Datei entpackt

			if(iOut>m_iDoneLen*4)						
				{
				iOut = m_iDoneLen*4;
				}
			
			if(pMax)*pMax=m_iDoneLen*6;

			return iNow+iOut;


	case 2:	iNow	= m_sHuffman.Value64(CHF_OUT);			// ffne TAR-Datei nach dem Entpacken

			if(m_uTarSize>0)
				{
				iNow *= m_iDoneLen;							// Den TAR-Verlauf skallieren
				iNow /= m_uTarSize;
				}

			if(pMax)*pMax=m_iDoneSave+m_iDoneLen;
			
			return m_iDoneSave+iNow;

	case 3:													// Speichere alles im TAR-Archive 

			iNow	= CTarFile::Done(&iMax);				
			iAll	= m_iDoneSave;
			iAll   += m_iDoneLen;
			iAll   += m_iDoneLen/4;

			if(iNow>m_iDoneSave)							// Den TAR-Verlauf prfen
				{
				iNow = m_iDoneSave;
				}
	
			if(pMax)*pMax=iAll;
			
			return iNow;


	case 4:													// Komprimiere TAR-Archiv in die TGZ-Datei

			iNow	= m_sHuffman.Value64(CHF_INP);			// Bytes aus TAR-Datei geladen
			iOut	= m_sHuffman.Value64(CHF_OUT);			// Bytes in  TGZ-Datei komprimiert

			if(iNow>m_iDoneLen)
				{
				iNow = m_iDoneLen;
				}

			if(iOut>m_iDoneLen/4)
				{
				iOut = m_iDoneLen/4;
				}

			iAll	= m_iDoneSave;
			iAll   += m_iDoneLen;
			iAll   += m_iDoneLen/4;

			if(pMax)*pMax=iAll;
			
			return m_iDoneSave+iNow+iOut;
		}


return CTarFile::Done(pMax);
}


//*****************************************************************************
//*
//*		Change
//*
//*****************************************************************************
//	Schreiberlaubnis aktivieren oder deaktivieren.
//	iMode		: Ist der neue Modus (CAF_WRITE oder CAF_READ)
//	Ergibt CAF_ERROR_OK wenn erfolgreich, oder eine Fehlernummer ungleich 0.
int CTgzFile::Change(int iMode)
{
int		iError;


	m_iDoneMode	= 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;



	if(m_iSubMode==iMode)									// Keine nderung	
		{
		return CAF_ERROR_OK;
		}


	if(iMode==CAF_READ && m_iChanged)						// TAR-Date in TGZ-Datei speichern
		{
		m_iDoneLen	= AllLen   ();
		m_iDoneSave	= FlushCalc();
		m_iDoneMode	= 3;

		   m_iErrorCode =Flush();							// Alles in der TAR-Datei speichern
		if(m_iErrorCode!=CAF_ERROR_OK)					
			{
			return m_iErrorCode;
			}

		m_iDoneMode	= 4;

		   m_iErrorCode = CloseFlush();						// Die TGZ-Datei anlegen
		if(m_iErrorCode!=CAF_ERROR_OK)
			{
			return m_iErrorCode;
			}

		m_iDoneNow	= Done(&m_iDoneMax);					// Aktuellen Verlauf in TAR-Datei bernehmen
		m_iDoneMode	= 0;
		SetDoneMode(-1);										
		}



	   iError = CTarFile::Change(iMode);					// Die TAR-Datei umstellen
	if(iError)											
		{
		Close();
		m_iErrorCode = iError;
		return         iError;
		}

	   iError = DoChangeSub(iMode,0);						// Die Unterdatei umstellen		
	if(iError!=CAF_ERROR_OK)							
		{
		return iError;
		}



return iError;
}

