//*****************************************************************************
//*
//*
//*		7zFile.c
//*
//*
//*****************************************************************************
/* 7zFile.c -- File IO
2008-11-22 : Igor Pavlov : Public domain */

#include	"7zFile.h"

#ifndef		USE_WINDOWS_FILE						

#include	<errno.h>

#endif

#ifdef		USE_WINDOWS_FILE						

/*
   ReadFile and WriteFile functions in Windows have BUG:
   If you Read or Write 64MB or more (probably min_failure_size = 64MB - 32KB + 1)
   from/to Network file, it returns ERROR_NO_SYSTEM_RESOURCES
   (Insufficient system resources exist to complete the requested service).
   Probably in some version of Windows there are problems with other sizes:
   for 32 MB (maybe also for 16 MB).
   And message can be "Network connection was lost"
*/

#define		kChunkSizeMax							(1 << 22)

#endif

//*****************************************************************************
//*
//*		File_Construct
//*
//*****************************************************************************
void File_Construct(CSzFile *p,void *param,FileSeekType	seek,FileReadType read,FileWriteType write)
{
	p->param	= param;
	p->seek		= seek;
	p->read		= read;
	p->write	= write;
	p->written	= 0;
}

WRes OutFile_Open(CSzFile *p, const char *name) { return 0;/*File_Open(p, name, 1);*/ }

//*****************************************************************************
//*
//*		File_Read
//*
//*****************************************************************************
WRes File_Read(CSzFile *p,void *data,size_t *size)
{
size_t		originalSize = *size;


	if(originalSize == 0)
		{
		return 0;
		}

	*size = p->read(p->param,data,(unsigned)originalSize);

	if(*size == originalSize)
		{
		return 0;
		}


return -1;
}

//*****************************************************************************
//*
//*		File_Write
//*
//*****************************************************************************
WRes File_Write(CSzFile *p,const void *data,size_t *size)
{
size_t		originalSize = *size;


	if(originalSize == 0)
		{
		return 0;
		}


	*size = p->write(p->param,data,(unsigned)originalSize);
	
	p->written += (int)*size;

	if(*size == originalSize)
		{
		return 0;
		}

return -1;
}

//*****************************************************************************
//*
//*		File_Seek
//*
//*****************************************************************************
WRes File_Seek(CSzFile *p,Int64 *pos,ESzSeek origin)
{

	*pos = p->seek(p->param,(int)*pos,origin);

	
return 0;
}

//*****************************************************************************
//*
//*		File_GetLength
//*
//*****************************************************************************
WRes File_GetLength(CSzFile *p,UInt64 *length)
{
int		pos;
int		len;
	

	pos = p->seek(p->param,0  ,SEEK_CUR);
	len = p->seek(p->param,0  ,SEEK_END);
		  p->seek(p->param,pos,SEEK_SET);


return len;
}


//*****************************************************************************
//*
//*		FileSeqInStream_Read
//*
//*****************************************************************************
//   FileSeqInStream           
static SRes FileSeqInStream_Read(void *pp,void *buf,size_t *size)
{
CFileSeqInStream   *p = (CFileSeqInStream *)pp;

return File_Read(&p->file, buf, size) == 0 ? SZ_OK : SZ_ERROR_READ;
}

//*****************************************************************************
//*
//*		FileSeqInStream_CreateVTable
//*
//*****************************************************************************
void FileSeqInStream_CreateVTable(CFileSeqInStream *p)
{
	p->s.Read = FileSeqInStream_Read;
}


//*****************************************************************************
//*
//*		FileInStream_Read
//*
//*****************************************************************************
//            FileInStream           
static SRes FileInStream_Read(void *pp,void *buf,size_t *size)
{
CFileInStream  *p = (CFileInStream *)pp;

return (File_Read(&p->file, buf, size) == 0) ? SZ_OK : SZ_ERROR_READ;
}

//*****************************************************************************
//*
//*		FileInStream_Seek
//*
//*****************************************************************************
static SRes FileInStream_Seek(void *pp,Int64 *pos,ESzSeek origin)
{
CFileInStream  *p = (CFileInStream *)pp;

return File_Seek(&p->file, pos, origin);
}

//*****************************************************************************
//*
//*		FileInStream_CreateVTable
//*
//*****************************************************************************
void FileInStream_CreateVTable(CFileInStream *p)
{
	p->s.Read = FileInStream_Read;
	p->s.Seek = FileInStream_Seek;
}


//*****************************************************************************
//*
//*		FileOutStream_Write
//*
//*****************************************************************************
//            FileOutStream           
static size_t FileOutStream_Write(void *pp,const void *data,size_t size)
{
CFileOutStream	   *p = (CFileOutStream *)pp;


	File_Write(&p->file, data, &size);

return size;
}

//*****************************************************************************
//*
//*		FileOutStream_CreateVTable
//*
//*****************************************************************************
void FileOutStream_CreateVTable(CFileOutStream *p)
{
	p->s.Write = FileOutStream_Write;
}
