//*****************************************************************************
//*
//*
//*		Aes7zip.c
//*
//*
//*****************************************************************************
//
//	UIN32	aes_ptr[0..3]	= init vector
//			aes_ptr[4...]	= aes data	
//
#include "Aes7zip.h"


#define	GET_U32(p)		(*(UINT_AES*)(p))	
#define	SET_U32(p,v)	(*(UINT_AES*)(p))=(v)	

#define FT(i,x)			Sbox[GB ## x(m[(i + x) & 3])]
#define FT4(d,i)		d[i] = UI32(FT(i,0), FT(i,1), FT(i,2), FT(i,3)) ^ aes_ptr[i];
						
#define FD(i,x)			InvS[GB ## x(m[(i - x) & 3])]
#define FD4(d,i)		d[i] = UI32(FD(i,0), FD(i,1), FD(i,2), FD(i,3)) ^ aes_ptr[i];

#define	HT(i,x,s)		(T + (x << 8))[GB ## x(s[(i + x) & 3])]
#define HD(i,x,s)		(D + (x << 8))[GB ## x(s[(i - x) & 3])]

#define	HT4(m,i,s,p) 	m[i] =										\
						HT(i,0,s) ^ 								\
						HT(i,1,s) ^ 								\
						HT(i,2,s) ^ 								\
						HT(i,3,s) ^ aes_ptr[p + i]

#define HT16(m,s,p) 												\
						HT4(m,2,s,p); 								\
						HT4(m,0,s,p); 								\
						HT4(m,3,s,p); 								\
						HT4(m,1,s,p);
						
#define HD4(m,i,s,p)	m[i] =										\
						HD(i,0,s) ^									\
						HD(i,1,s) ^									\
						HD(i,2,s) ^									\
						HD(i,3,s) ^ aes_ptr[p + i];

#define HD16(m,s,p)													\
						HD4(m,0,s,p);								\
						HD4(m,2,s,p);								\
						HD4(m,3,s,p);								\
						HD4(m,1,s,p);			
						

#define XTIME(x)		((UCHAR_AES)(((x)<<1)^((((x)&0x80)!=0)? 0x1B:0x00)))

#define UI32(a,b,c,d)	((UINT_AES)(a) | ((UINT_AES)(b)<<8) | ((UINT_AES)(c)<<16) | ((UINT_AES)(d)<<24))

#define	GB0(x) 			((UCHAR_AES)((x)    ))
#define GB1(x) 			((UCHAR_AES)((x)>> 8))
#define GB2(x) 			((UCHAR_AES)((x)>>16))
#define GB3(x) 			((UCHAR_AES)((x)>>24))




static const UCHAR_AES Sbox[256] = 
	{
	0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5, 0x30, 0x01, 0x67, 0x2b, 0xfe, 0xd7, 0xab, 0x76,
	0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59, 0x47, 0xf0, 0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0,
	0xb7, 0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc, 0x34, 0xa5, 0xe5, 0xf1, 0x71, 0xd8, 0x31, 0x15,
	0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05, 0x9a, 0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75,
	0x09, 0x83, 0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0, 0x52, 0x3b, 0xd6, 0xb3, 0x29, 0xe3, 0x2f, 0x84,
	0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b, 0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf,
	0xd0, 0xef, 0xaa, 0xfb, 0x43, 0x4d, 0x33, 0x85, 0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c, 0x9f, 0xa8,
	0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5, 0xbc, 0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2,
	0xcd, 0x0c, 0x13, 0xec, 0x5f, 0x97, 0x44, 0x17, 0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19, 0x73,
	0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88, 0x46, 0xee, 0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb,
	0xe0, 0x32, 0x3a, 0x0a, 0x49, 0x06, 0x24, 0x5c, 0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79,
	0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9, 0x6c, 0x56, 0xf4, 0xea, 0x65, 0x7a, 0xae, 0x08,
	0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6, 0xb4, 0xc6, 0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a,
	0x70, 0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e, 0x61, 0x35, 0x57, 0xb9, 0x86, 0xc1, 0x1d, 0x9e,
	0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e, 0x94, 0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf,
	0x8c, 0xa1, 0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68, 0x41, 0x99, 0x2d, 0x0f, 0xb0, 0x54, 0xbb, 0x16,
	};

static UCHAR_AES	Rcon[11] = { 0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x1b, 0x36 };
static UCHAR_AES	InvS[256];
static UINT_AES		D[256*4];
static UINT_AES		T[256*4];


void	AesCbc_Encode(UINT_AES *ivAes, UCHAR_AES *data, unsigned numBlocks);
void	AesCbc_Decode(UINT_AES *ivAes, UCHAR_AES *data, unsigned numBlocks);
void	AesCtr_Code  (UINT_AES *ivAes, UCHAR_AES *data, unsigned numBlocks);



//*****************************************************************************
//*
//*		AesGenTables
//*
//*****************************************************************************
void AesGenTables(void)
{
static int	init=0;
unsigned	pos;


	if(init)return;

	for(pos=0;pos<256;pos++)
		{
		InvS[Sbox[pos]] = (UCHAR_AES)pos;
		}

	for(pos=0;pos<256;pos++)
		{
		UINT_AES a1		= Sbox[pos];
		UINT_AES a2 	= XTIME(a1);
		UINT_AES a3 	= a2^a1;

		T[      pos] 	= UI32(a2,a1,a1,a3);
		T[0x100+pos] 	= UI32(a3,a2,a1,a1);
		T[0x200+pos] 	= UI32(a1,a3,a2,a1);
		T[0x300+pos] 	= UI32(a1,a1,a3,a2);
		}

	for(pos=0;pos<256;pos++)
		{
		UINT_AES a1 	= InvS[pos];
		UINT_AES a2 	= XTIME(a1);
		UINT_AES a4 	= XTIME(a2);
		UINT_AES a8 	= XTIME(a4);
		UINT_AES a9 	= a8^a1;
		UINT_AES aB 	= a8^a2^a1;
		UINT_AES aD 	= a8^a4^a1;
		UINT_AES aE 	= a8^a4^a2;

		D[      pos] 	= UI32(aE,a9,aD,aB);
		D[0x100+pos] 	= UI32(aB,aE,a9,aD);
		D[0x200+pos] 	= UI32(aD,aB,aE,a9);
		D[0x300+pos] 	= UI32(a9,aD,aB,aE);
		}

	init = 1;

}


//*****************************************************************************
//*
//*		Aes_SetKey_Enc
//*
//*****************************************************************************
void Aes_SetKey_Enc(UINT_AES *aes_ptr,const UCHAR_AES *key,unsigned keySize)
{
UINT_AES	data;
unsigned	size;
unsigned	temp;
unsigned	pos;

  
	size		= keySize + 28;
	keySize	   /= 4;
	aes_ptr[0]	= ((UINT_AES)keySize / 2) + 3;
	aes_ptr	   += 4;

	for(pos=0;pos<keySize;pos++,key+=4)
		{	
		aes_ptr[pos] = GET_U32(key);
		}

	for(;pos<size;pos++)
		{
		data	= aes_ptr[pos-1];
		temp	= pos%keySize;
    
		if(temp==0)
			{
			data = UI32(Sbox[GB1(data)] ^ Rcon[pos / keySize], Sbox[GB2(data)], Sbox[GB3(data)], Sbox[GB0(data)]);
			}
		else if(keySize>6 && temp==4)
			{
			data = UI32(Sbox[GB0(data)], Sbox[GB1(data)], Sbox[GB2(data)], Sbox[GB3(data)]);
			}
		
		aes_ptr[pos] = aes_ptr[pos - keySize] ^ data;
		}

}

//*****************************************************************************
//*
//*		Aes_SetKey_Dec
//*
//*****************************************************************************
void Aes_SetKey_Dec(UINT_AES *aes_ptr,const UCHAR_AES *key,unsigned keySize)
{
UINT_AES	data;
unsigned	pos;
unsigned	num;
  
  
	Aes_SetKey_Enc(aes_ptr,key,keySize);

	num		 = keySize+20;
	aes_ptr += 8;
  
	for(pos=0;pos<num;pos++)
		{
		data = aes_ptr[pos];
		aes_ptr[pos] =	D[        Sbox[GB0(data)]] ^
						D[0x100 + Sbox[GB1(data)]] ^
						D[0x200 + Sbox[GB2(data)]] ^
						D[0x300 + Sbox[GB3(data)]];
		}

}

//*****************************************************************************
//*
//*		Aes_Encode
//*
//*****************************************************************************
//	Aes_Encode and Aes_Decode functions work with little-endian words.
//	src and dest are pointers to 4 UINT_AES words.
//	arc and dest can point to same block
static __inline void Aes_Encode(const UINT_AES *aes_ptr,UINT_AES *dest,const UINT_AES *src)
{
UINT_AES	rounds;
UINT_AES	s[4];
UINT_AES	m[4];

	
	rounds	 = aes_ptr[0];
	aes_ptr	+= 4;

	s[0]	 = src[0] ^ aes_ptr[0];
	s[1] 	 = src[1] ^ aes_ptr[1];
	s[2] 	 = src[2] ^ aes_ptr[2];
	s[3] 	 = src[3] ^ aes_ptr[3];

	aes_ptr += 4;
	
	for(;;)
		{
		HT16(m,s,0);
	
		rounds--;

		if(rounds==0)
			{
			break;
			}

		HT16(s,m,4);
		aes_ptr += 8;
		}
	
	aes_ptr += 4;
	
	FT4(dest,0); 
	FT4(dest,1); 
	FT4(dest,2); 
	FT4(dest,3);

}

//*****************************************************************************
//*
//*		Aes_Decode
//*
//*****************************************************************************
static __inline void Aes_Decode(const UINT_AES *aes_ptr,UINT_AES *dest,const UINT_AES *src)
{
UINT_AES	rounds;
UINT_AES	s[4];
UINT_AES	m[4];
  
  
	rounds	 = aes_ptr[0];
	aes_ptr += rounds*8+4;

	s[0]	 = src[0] ^ aes_ptr[0];
	s[1]	 = src[1] ^ aes_ptr[1];
	s[2]	 = src[2] ^ aes_ptr[2];
	s[3]	 = src[3] ^ aes_ptr[3];

	for(;;)
		{
		aes_ptr -= 8;
		HD16(m,s,4);
		rounds--;
		 
		if(rounds==0)
			{
			break;
			}

		HD16(s,m,0);
		}
  
	FD4(dest,0); 
	FD4(dest,1); 
	FD4(dest,2); 
	FD4(dest,3);

}

//*****************************************************************************
//*
//*		AesCbc_Encode
//*
//*****************************************************************************
void AesCbc_Encode(UINT_AES *aes_ptr, UCHAR_AES *data,unsigned numBlocks)
{

	for(;numBlocks!=0;numBlocks--,data+=AES_BLOCK_SIZE)
		{
		aes_ptr[0] ^= GET_U32(data   );
		aes_ptr[1] ^= GET_U32(data+ 4);
		aes_ptr[2] ^= GET_U32(data+ 8);
		aes_ptr[3] ^= GET_U32(data+12);

		Aes_Encode(aes_ptr+4,aes_ptr,aes_ptr);

		SET_U32(data   ,aes_ptr[0]);
		SET_U32(data+ 4,aes_ptr[1]);
		SET_U32(data+ 8,aes_ptr[2]);
		SET_U32(data+12,aes_ptr[3]);
		}
}


//*****************************************************************************
//*
//*		AesCbc_Decode
//*
//*****************************************************************************
void AesCbc_Decode(UINT_AES *aes_ptr,UCHAR_AES *data,unsigned numBlocks)
{
UINT_AES	out[4];
UINT_AES	in [4];
  

	for(;numBlocks!=0;numBlocks--,data+=AES_BLOCK_SIZE)
		{
		in[0] = GET_U32(data   );
		in[1] = GET_U32(data+ 4);
		in[2] = GET_U32(data+ 8);
		in[3] = GET_U32(data+12);

		Aes_Decode(aes_ptr+4,out,in);

		SET_U32(data   ,aes_ptr[0]^out[0]);
		SET_U32(data+ 4,aes_ptr[1]^out[1]);
		SET_U32(data+ 8,aes_ptr[2]^out[2]);
		SET_U32(data+12,aes_ptr[3]^out[3]);

		aes_ptr[0] = in[0];
		aes_ptr[1] = in[1];
		aes_ptr[2] = in[2];
		aes_ptr[3] = in[3];
		}
}

//*****************************************************************************
//*
//*		AesCtr_Code
//*
//*****************************************************************************
void AesCtr_Code(UINT_AES *aes_ptr,UCHAR_AES *data,unsigned numBlocks)
{
UCHAR_AES	buf[16];
UINT_AES	temp[4];
unsigned 	pos;  
	
	
	for(;numBlocks!=0;numBlocks--)
		{
		aes_ptr[0]++;

	    if(aes_ptr[0]==0)
			{
			aes_ptr[1]++;
			}

		Aes_Encode(aes_ptr+4,temp,aes_ptr);

		SET_U32(buf   ,temp[0]);
		SET_U32(buf+ 4,temp[1]);
		SET_U32(buf+ 8,temp[2]);
		SET_U32(buf+12,temp[3]);
		
		for(pos=0;pos<16;pos++)
			{
			data[0] ^= buf[pos];
			data++;
			}
		}
}

//*****************************************************************************
//*
//*		AesCbc_Init
//*
//*****************************************************************************
//	set the init vector on the firrst four 
void AesCbc_Init(UINT_AES *aes_ptr,const UCHAR_AES *initvec)
{

	aes_ptr[0] = GET_U32(initvec    );
	aes_ptr[1] = GET_U32(initvec+1*4);
	aes_ptr[2] = GET_U32(initvec+2*4);
	aes_ptr[3] = GET_U32(initvec+3*4);

}
