//*****************************************************************************
//*
//*
//*		Bcj27zip.c
//*
//*
//*****************************************************************************
//
//	Converter for x86 uCode (BCJ2)
//
#include	"Bcj27zip.h"


#define		BJC2_TOP_BITS 				24
#define		BJC2_TOP_VALUE				(1<<BJC2_TOP_BITS)
#define		BJC2_MODE_BITS				11
#define		BJC2_MODE_VALUE				(1<<BJC2_MODE_BITS)
#define		BJC2_MOVE_BITS				5

#define		BCJ2_STREAM_MAIN			0
#define		BCJ2_STREAM_CALL			1
#define		BCJ2_STREAM_JUMP			2
#define		BCJ2_STREAM_RC				3

#define		IS_32BIT_STREAM(s)			((s)==BCJ2_STREAM_CALL || (s)==BCJ2_STREAM_JUMP)



//*****************************************************************************
//*
//*		Bcj2Convert
//*
//*****************************************************************************
int Bcj2Convert(const unsigned char *pBuffer0,unsigned uSize0,
				const unsigned char *pBuffer1,unsigned uSize1,
				const unsigned char *pBuffer2,unsigned uSize2,
				const unsigned char *pBuffer3,unsigned uSize3,
				      unsigned char *pOutput ,unsigned uMaxOut)
{
const unsigned char    *pEnd;
const unsigned char    *pBase;
const unsigned char    *pFrom;
const unsigned char    *pStop;
const unsigned char    *pStop0;
const unsigned char    *pStop1;
const unsigned char    *pStop2;
const unsigned char    *pStop3;
unsigned char 			ucByte;
unsigned char			ucPrev;
unsigned char			ucLast;
unsigned				aProbs[256+2];
unsigned				uRange;
unsigned				uBound;
unsigned				uIndex;
unsigned				uDelta;
unsigned				uCode;
unsigned				uTemp;
unsigned				uNum;
unsigned				uPos;
unsigned				uVal;


	uCode		= 0;
	ucLast		= 0;
	uDelta		= 0;
	pStop0		= pBuffer0+uSize0;
	pStop1		= pBuffer1+uSize1;
	pStop2		= pBuffer2+uSize2;
	pStop3		= pBuffer3+uSize3;
	pEnd		= pOutput+uMaxOut;
	pBase		= pOutput;
		

	for(uPos=0;uPos<sizeof(aProbs)/sizeof(aProbs[0]);uPos++)
		{
		aProbs[uPos] = BJC2_MODE_VALUE>>1;
		}



	for(uRange=0;uRange<5;uRange++)
		{
		if(uRange==1 && uCode!=0)
			{
			return -1;
			}
  
		if(pBuffer3>=pStop3)
			{
			return 0;
			}

		uCode = (uCode<<8)|*pBuffer3;
		
		pBuffer3++;
		}
    
	if(uCode==0xFFFFFFFF)
		{
		return -1;
		}

	uRange = 0xFFFFFFFF;



	for(;;)
		{
		if(uRange<BJC2_TOP_VALUE)
			{
			if(pBuffer3>=pStop3)
				{
				break;
				}

			uRange  <<= 8;
			uCode	  = (uCode<<8) | *pBuffer3;
			
			pBuffer3++;
			}

		pFrom	= pBuffer0;
		uNum	= pStop0-pFrom;
        
        if(uNum==0)
			{
			break;
			}
        
		if(uNum>(unsigned)(pEnd-pOutput))
			{
			   uNum = pEnd-pOutput;
			if(uNum==0)
				{
				break;
				}
			}
       
        pStop = pFrom+uNum;

        if(ucLast==0x0F && (pFrom[0]&0xF0)==0x80)
			{
			ucByte	 = *pFrom;
			*pOutput = ucByte;
			}
        else while(pFrom<pStop)
			{
			ucByte	 = *pFrom;
			*pOutput = ucByte;
          
			if(ucByte!=0x0F)
				{
				if((ucByte&0xFE)==0xE8)
					{
					break;
					}
            
				pOutput++;
				pFrom++;
				continue;
				}
			
			pOutput++;
			pFrom++;
			
			if(pFrom>=pStop)
				{
				ucByte	 = *pFrom;
				break;
				}

			if((*pFrom&0xF0)==0x80)
				{
				ucByte	 = *pFrom;
				*pOutput = ucByte;
				break;
				}
			}
        
        uNum = pFrom-pBuffer0;
        
        if(pFrom>=pStop)
			{
			break;
			}
        
		ucPrev		= (!uNum)? ucLast:pFrom[-1];
		pBuffer0	= pFrom+1;
		uNum       += 1;
		pOutput	   += 1;
		uDelta     += uNum;
     

		if(ucByte==0xE8)
			{
			uIndex = 2+ucPrev;
			}
		else{
			uIndex = (ucByte==0xE9)? 1:0;
			}

		uTemp  = aProbs[uIndex];
		uBound = (uRange>>BJC2_MODE_BITS)*uTemp; 
		
		if(uCode<uBound)
			{
			uRange			= uBound; 
			aProbs[uIndex]	= uTemp + ((BJC2_MODE_VALUE-uTemp)>>BJC2_MOVE_BITS);
			ucLast			= ucByte;
			continue;
			}
		else{          
			uRange		   -= uBound; 
			uCode		   -= uBound; 
			aProbs[uIndex]	= uTemp - (uTemp>>BJC2_MOVE_BITS);
			}


		if(ucByte==0xE8)
			{
			pBuffer1   += 4;
			pStop		= pStop1;
			pFrom		= pBuffer1;
			if(pFrom>pStop)break;
			}
		else{
			pBuffer2   += 4;
			pStop		= pStop2;
			pFrom		= pBuffer2;
			if(pFrom>pStop)break;
			}

		uVal	= pFrom[-1];
		uVal   |= pFrom[-2]<<8;
		uVal   |= pFrom[-3]<<16;
		uVal   |= pFrom[-4]<<24;
      		   
		uDelta     += 4;
		uVal	   -= uDelta;
		pOutput[0]	= (unsigned char)(uVal);      
		pOutput[1]	= (unsigned char)(uVal>>8);
		pOutput[2]	= (unsigned char)(uVal>>16);
		pOutput[3]	= (unsigned char)(uVal>>24);
		ucLast		= (unsigned char)(uVal>>24);
		pOutput	   += 4;
		}


return pOutput-pBase;
}
