//*****************************************************************************
//*
//*
//*		ZipFile.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//
//	Klasse fr den Zugriff auf Zip Files:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int uLen)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen
//
#include	<stdio.h>
#include	<stddef.h>
#include	<string.h>
#include	<malloc.h>
#include	<sys/stat.h>

#include	"ZipFile.h"
#include	"FileFinder.h"
#include	"FileFunctions.h"
#include	"Codecs/Huffman.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		ZIP_USE_ALL
#ifndef		ZIP_USE_AES
#define 	ZIP_USE_AES
#endif
#ifndef		ZIP_USE_BZIP2
#define 	ZIP_USE_BZIP2
#endif
#ifndef		ZIP_USE_LZMA
#define 	ZIP_USE_LZMA
#endif
#ifndef		ZIP_USE_LZMA2
#define 	ZIP_USE_LZMA2
#endif
#ifndef		ZIP_USE_PPMD
#define 	ZIP_USE_PPMD
#endif
#endif

#ifdef		ZIP_USE_BZIP2
#include	"Codecs/BzipClass.h"
#endif

#ifdef		ZIP_USE_LZMA
#include	"Codecs/LzmaClass.h"
#endif

#ifdef		ZIP_USE_LZMA2
#include	"Codecs/XzClass.h"
#endif

#ifdef		ZIP_USE_PPMD
#include	"Codecs/PpmClass.h"
#endif

#ifdef		ZIP_USE_AES
#include	"ZipAes/ZipCoder.h"
#endif


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

#define 	ENTRIES(a)	(sizeof(a)/sizeof((a)[0]))
#define 	CATCHALL(a)	try{a;}catch(...){};

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		(int)wcslen
#define 	strzcmp		wcscmp
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#define 	strzncat	wcsncat
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		(int)strlen
#define 	strzcmp		strcmp
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#define 	strzncat	strncat
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif

											// Flags fr ZipItem->uFlags
#define FLGENC  		0x0001          	// Die Datei ist verschlsselt
#define FLGSLW  		0x0002          	// Langsame Komprimierung
#define FLGFST  		0x0004          	// Schnelle Komprimierung
#define FLGEXT  		0x0008          	// Extended Block angehngen
#define FLGUDF  		0x0800          	// UDF-8 Dateinamen

											// Offsets of values in headers
#define LOCVER  		0               	// version needed entry extract
#define LOCFLG  		2               	// encrypt, deflate flags
#define LOCHOW  		4               	// compression mode
#define LOCTIM  		6               	// last modified file time, DOS format
#define LOCDAT  		8               	// last modified file date, DOS format
#define LOCCRC  		10              	// uncompressed crc-32 for file
#define LOCSIZ  		14              	// compressed size in zip file
#define LOCLEN  		18              	// uncompressed size
#define LOCNAM  		22              	// length of filename
#define LOCEXT  		24              	// length of extra field

#define EXTCRC  		0               	// uncompressed crc-32 for file
#define EXTSIZ  		4               	// compressed size in zip file
#define EXTLEN  		8               	// uncompressed size

#define CENVEM  		0               	// version made by
#define CENVER  		2               	// version needed entry extract
#define CENFLG  		4               	// encrypt, deflate flags
#define CENHOW  		6               	// compression mode
#define CENTIM  		8               	// last modified file time, DOS format
#define CENDAT  		10              	// last modified file date, DOS format
#define CENCRC  		12              	// uncompressed crc-32 for file
#define CENSIZ  		16              	// compressed size in zip file
#define CENLEN  		20              	// uncompressed size
#define CENNAM  		24              	// length of filename
#define CENEXT  		26              	// length of extra field
#define CENCOM  		28              	// file comment length
#define CENDSK  		30              	// disk number start
#define CENATT  		32              	// internal file attributes
#define CENATX  		34              	// external file attributes
#define CENOFF  		38              	// relative offset of header

#define ENDDSK  		0               	// number of this disk
#define ENDBEG  		2               	// number of the starting disk
#define ENDSUB  		4               	// entries on this disk
#define ENDTOT  		6               	// total number of entries
#define ENDSIZ  		8               	// size of entire central directory
#define ENDOFF  		12              	// offset of central on starting disk
#define ENDCOM  		16              	// length of zip file comment

#define LOCHEAD 		26
#define CENHEAD 		42					// Lnge des CEN-Headers
#define CNDHEAD64		56					// Lnge des 64-Bit CEN-Headers am Ende
#define ENDHEAD 		18					// Lnge des Headers am Ende
#define ENDHEAD64		16					// Lnge des 64-Bit Headers am Ende

#define UNKNOWN 		(-1)				// Dateitypen
#define BINARY  		0
#define ASCII   		1

#define EF_SIZE64		0x0001   			// 64-Bit Gren
#define EF_NTFS			0x000A   			// NTFS Extra Field ID
#define EF_IZUNIX    	0x5855   			// UNIX Extra Field ID ("UX")
#define EF_IZUNIX2   	0x7855   			// Info-ZIP'size new Unix ("Ux")
#define EF_TIME      	0x5455   			// universal timestamp ("UT")
#define EF_AESCRYPT		0x9901				// AES Extra Field ID

#define EB_HEADSIZE     4     				// length of a extra field block header
#define EB_ID           0     				// offset of block ID in header
#define EB_LEN          2     				// offset of data length field in header

#define EB_UX_MINLEN    8     				// minimal "UX" field contains iTimeA, iTimeM
#define EB_UX_ATIME     0     				// offset of iTimeA in "UX" extra field data
#define EB_UX_MTIME     4     				// offset of iTimeM in "UX" extra field data

#define EB_UX_FULLSIZE  12    				// full "UX" field (iTimeA, iTimeM, uid, gid)
#define EB_UX_UID       8     				// byte offset of UID in "UX" field data
#define EB_UX_GID       10    				// byte offset of GID in "UX" field data

#define EB_UT_MINLEN    1     				// minimal UT field contains Flags byte
#define EB_UT_FL_MTIME  (1 << 0)			// iTimeM present
#define EB_UT_FL_ATIME  (1 << 1)      		// iTimeA present
#define EB_UT_FL_CTIME  (1 << 2)      		// iTimeC present
#define EB_64_SIZE		(1 << 8)			// 64-Bits-FullSize present
#define EB_64_COMP		(1 << 9)			// 64-Bits-CompSize present
#define EB_64_POS		(1 << 10)			// 64-Bits-Offset   present

#define EB_UX2_MINLEN   4					// minimal Ux field contains UID/GID
#define EB_UX2_UID      0     				// byte offset of UID in "Ux" field data
#define EB_UX2_GID      2     				// byte offset of GID in "Ux" field data
#define EB_UX2_VALID    (1<<8)				// UID/GID present

#define	CRC32_INIT		0

#define Z_MAJORVER   	2					// Versions Nummer
#define Z_MINORVER   	3
#define Z_PATCHLEVEL 	0
#define Z_BETALEVEL		""

#define COMMENT 		Z("AZ CZipFile V5.01 (C) 2019")

#define DW_MAJORVER 	Z_MAJORVER
#define DW_MINORVER 	Z_MINORVER
#define DW_PATCHLEVEL	Z_PATCHLEVEL

#define OS_CODE			0xB00

#define A_NORMAL		0x00	
#define A_RONLY    		0x01
#define A_HIDDEN   		0x02
#define A_SYSTEM   		0x04
#define A_LABEL    		0x08
#define A_DIR      		0x10
#define A_ARCHIVE  		0x20

#define VERSION			(OS_CODE + Z_MAJORVER*10 + Z_MINORVER)
#define FHOW			(O_RDONLY|O_BINARY)
#define FBAD			-1


#define	PUSHB(uMem)									\
	{												\
	*(char *)(cPush+cPushPos)=(char)(uMem);			\
				    cPushPos	+=1;				\
	}
#define	PUSHW(uMem)									\
	{												\
	*(short*)(cPush+cPushPos)=(short)(uMem);		\
				    cPushPos	+=2;				\
	}
#define	PUSHD(uMem)									\
	{												\
	*(int  *)(cPush+cPushPos)=(int)(uMem);			\
				    cPushPos	+=4;				\
	}
#define	PUSHQ(uMem)									\
	{												\
	*(aint64*)(cPush+cPushPos)=(aint64)(uMem);		\
				    cPushPos	+=8;				\
	}
#define	PUSHS(pMem)									\
	{												\
	*(int  *)(cPush+cPushPos)=*(int*)(pMem);		\
				    cPushPos	+=4;				\
	}

#define PROG(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))throw CAF_ERROR_BREAK;	\
	}
#define PROGR(iMode,iNum,Ret)						\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,iNum))return Ret;				\
	}
#define PROGN(iMode,iNum)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	Progress(iMode,iNum);							\
	}



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CZipFile::CZipFile()
{

	m_iDecoder		= -1;
	m_pDecoder		=  0;
	m_pComment		=  0;
	m_pCrypt		=  0;
	m_pFiles		=  0;
	m_iFilesMax		=  0;
	m_iFilesNum		=  0;
	m_iFileCount	=  0;
	m_iDoneNow		=  0;
	m_iDoneMax		=  0;
	m_iDoneMode		=  0;	 
	m_cHasStore		=  1;
	m_cHasAddFile	=  1;

	SelectDecoder(0,0);

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CZipFile::~CZipFile()
{

	if(m_pCrypt	   ){delete m_pCrypt    ;m_pCrypt    =0;}
	if(m_pErrorText){delete m_pErrorText;m_pErrorText=0;}
	if(m_iOpenMode!=CAF_CLOSED)Close();

	if(m_pDecoder)
		{
		m_pDecoder->Free();
		delete m_pDecoder;
		m_pDecoder = 0;
		}

	Clear();

}

//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Datenfelder und setzt alle Variabeln zurck.
void CZipFile::Clear()
{
int	iPos;


	for(iPos=m_iFilesNum-1;iPos>=0;iPos--)
		{
		ZipDel(iPos);
		}

	if(m_pComment)
		{
		delete m_pComment;
		m_pComment = 0;
		}
	

	m_iCompressMode	= CAF_BEST;
	m_iFilesNum		= 0;
	m_iEndPos		= 0;						// Position des END-Blockes
	m_iCenPos		= 0;						// Position des ersten CEN-Blockes
	m_iChanged		= 0;						// Wurde Datei Verndert

}

//*****************************************************************************
//*
//*		CmpEx
//*
//*****************************************************************************
int __cdecl CZipFile::CmpEx(const void *pPos1,const void *pPos2)
{
ZipItem	*pPtr1,*pPtr2;

	
	pPtr1=*(ZipItem**)pPos1;
	pPtr2=*(ZipItem**)pPos2;

	if(pPtr1->cDelete)return (pPtr2->cDelete)? 0:1;
	if(pPtr2->cDelete)return -1;


return strvicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpName
//*
//*****************************************************************************
int __cdecl CZipFile::CmpName(const void *pPos1,const void *pPos2)
{
ZipItem	*pPtr1,*pPtr2;


	pPtr1=*(ZipItem**)pPos1;
	pPtr2=*(ZipItem**)pPos2;

	if(pPtr1->cDelete)return (pPtr2->cDelete)? 0:1;
	if(pPtr2->cDelete)return -1;


return strzicmp(pPtr1->pName,pPtr2->pName);
}

//*****************************************************************************
//*
//*		CmpLen
//*
//*****************************************************************************
int __cdecl CZipFile::CmpLen(const void *pPos1,const void *pPos2)
{
ZipItem	*pPtr1,*pPtr2;

	
	pPtr1=*(ZipItem**)pPos1;
	pPtr2=*(ZipItem**)pPos2;

	if(pPtr1->cDelete)return (pPtr2->cDelete)? 0:1;
	if(pPtr2->cDelete)return -1;

	if(pPtr1->uLenFull==pPtr2->uLenFull)return 0;


return (pPtr1->uLenFull<pPtr2->uLenFull)? -1:0;
}

//*****************************************************************************
//*
//*		CmpPos
//*
//*****************************************************************************
int __cdecl CZipFile::CmpPos(const void *pPos1,const void * pPos2)
{
ZipItem	*pPtr1,*pPtr2;
								  

	pPtr1=*(ZipItem**)pPos1;
	pPtr2=*(ZipItem**)pPos2;

	if(pPtr1->cWrite)return (pPtr2->cWrite)? 0:1;
	if(pPtr2->cWrite)return -1;

	if(pPtr1->uOff==pPtr2->uOff)return 0;


return (pPtr1->uOff<pPtr2->uOff)? -1:1;
}

//*****************************************************************************
//*
//*		Count
//*
//*****************************************************************************
//	Ergibt die Anzahl der Dateien, samt den gelschten Dateien.
int CZipFile::Count()
{
	return m_iFilesNum;
}

//*****************************************************************************
//*
//*		Len
//*
//*****************************************************************************
//	Ergibt die Lnge einer einzelnen Datei im Archiv.
//	iNum	: Ist die Nmmer der Datei in der Liste
//	iMode	: Welche Lnge soll zurck gegeben werden
//				0 = wird die originale    Lnge zurckgegeben
//				1 = wird die komprimierte Lnge zurckgegeben
//	Ergibt die Lnge
aint64 CZipFile::Len(int iNum,int iMode)
{
ZipItem	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;
	pEntry = m_pFiles[iNum];


return (iMode)? pEntry->uLenComp:pEntry->uLenFull;
}

//*****************************************************************************
//*
//*		SetFail
//*
//*****************************************************************************
//	Setzt das Defekt-Flag und ergibt den alten Wert.
int CZipFile::SetFail(int iNum,int iFail)
{
int		iOld;

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

	iOld = m_pFiles[iNum]->cError;
	m_pFiles[iNum]->cError = iFail;

return iOld;
}

//*****************************************************************************
//*
//*		IsFail
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit iNum beschdigt ist sonst 0
int CZipFile::IsFail(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return -1;

return m_pFiles[iNum]->cError;
}

//*****************************************************************************
//*
//*		IsReadOnly
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei mit nur Leserechte hat ist sonst 0
int CZipFile::IsReadOnly(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return (m_pFiles[iNum]->uAtx&A_RONLY)? 1:0;
}

//*****************************************************************************
//*
//*		IsDir
//*
//*****************************************************************************
//	Ergibt die 1 wenn die Datei ein Verzeichnis ist sonst 0
int CZipFile::IsDir(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return (m_pFiles[iNum]->uAtx&A_DIR)? 1:0;
}

//*****************************************************************************
//*
//*		IsDel
//*
//*****************************************************************************
//	Ergibt 1 wenn die Datei gelscht wurde.
int CZipFile::IsDel(int iNum)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->cDelete;
}


//*****************************************************************************
//*
//*		IsOk
//*
//*****************************************************************************
// Ergibt:	2	Wenn die Datei vollstndig entpackt wurde
//          1	Wenn die Datei schon vorhanden war
//			0   Bei Fehlern
int CZipFile::IsOk(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->cDecomp;
}

//*****************************************************************************
//*
//*		Stats
//*
//*****************************************************************************
//	Die Zustandbits abfragen
//	iNum		: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet		: Welche Bits sollen gesetzt werden
//	iDel		: Welche Bits sollen gelscht werden
//	Ergibt einen Wert mit veschiedenen Bits:
//					CAF_STAT_EXIST		0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//					CAF_STAT_DECOMP		0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//					CAF_STAT_DELETE		0x0004	Die Datei wurde aus dem Verzeichnis gelscht
//					CAF_STAT_WRITE		0x0008	Die Datei muss ins Archiv geschrieben werden
//					CAF_STAT_DIR		0x0010	Die Datei ist ein Verzeichniss
//					CAF_STAT_READONLY	0x0020	Die Datei ist schreibgeschtzt
//					CAF_STAT_SAVED		0x0040	Die Datei ist im Archiv gespeichert
int CZipFile::Stats(int iNum)
{
ZipItem	   *pEntry;
int			iBits;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return 0;
		}

			pEntry=m_pFiles[iNum];
	iBits = pEntry->cDecomp&3;

	if(pEntry->cDelete     )iBits|=CAF_STAT_DELETE;
	if(pEntry->cWrite      )iBits|=CAF_STAT_WRITE;
	if(pEntry->cSaved      )iBits|=CAF_STAT_SAVED;
	if(pEntry->uAtx&A_DIR  )iBits|=CAF_STAT_DIR;
	if(pEntry->uAtx&A_RONLY)iBits|=CAF_STAT_READONLY;
	

return iBits;	
}

//*****************************************************************************
//*
//*		SetStats
//*
//*****************************************************************************
//	Den Zustand einer Arichiv Datei verndern.
//	iNum	: Ist der Name der Datei samt Pfad die geprft werden soll.
//	iSet	: Welche Bits sollen gesetzt werden
//				CAF_STAT_NONE	0x0000	Die Datei wurde noch nicht entpackt
//				CAF_STAT_EXIST	0x0001	Wenn die Datei beim Entpacken schon vorhanden war
//				CAF_STAT_DECOMP	0x0002	Wenn die Datei vollstndig neu entpackt wurde (immer ohne CAF_STAT_FILEOK)
//	Ergibt den alten Wert oder -1 bei einem Fehler.
int CZipFile::SetStats(int iNum,int iSet)
{
ZipItem	   *pEntry;
int			iOld;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)				// Prfe den Index
		{
		return -1;
		}

	pEntry			= m_pFiles[iNum];
	iOld			= pEntry->cDecomp;
	pEntry->cDecomp = iSet;
	

return iOld;	
}

//*****************************************************************************
//*
//*		SetDoneMode
//*
//*****************************************************************************
//	Den Modus fr die Verlaufsanzeige setzen.
//	iMode	: Ist der neue Modus
//			    0 = Zurck zu Normal (Ende)
//				1 = Begin von Extract (einzelene Datei)
//				2 = Begin von ExtractAll
//	Ergibt den vorherigen Modus.
int CZipFile::SetDoneMode(int iMode)
{
int		iRet;

	
	iRet = m_iDoneMode;
	
	switch(iMode)
		{
	case 0:												// Zurck zu Normal (Ende)

		if(m_iDoneMode==1 || m_iDoneMode==2)
		if(m_iErrorCode==CAF_ERROR_OK)
			{
			m_iDoneNow  = m_iDoneMax;
			}

		m_iDoneMode	 = 0;
		break;

	case 1:												// Begin von Extract (einzelene Datei)

		if(iRet==2)return iRet;
		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 0;
		break;

	case 2:												// Begin von ExtractAll

		m_iDoneNow		= 0;
		m_iDoneMax		= 0;
		m_iDoneMode		= 1;
		break;
		}


return iRet;
}


//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
//	Die nderungzeit einer einzelnen Datei abfragen.
//	iNum		: Ist die Nummer der Datei
//	iMode		: Bestimmt welche Zeit abgefragt werden soll
//					CAF_TIME_W	0	= Zeit der letzten nderung
//					CAF_TIME_C	1	= Zeit der Dateierstellung 
//					CAF_TIME_A	2	= Zeit des letzten Zugriffs 
//	Ergibt die Zeit der letzten nderung einer Datei bzw. CAF_ERROR_MISS (-1)
//	wenn die Datei nicht existiert.
time_t CZipFile::Time(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}

return m_pFiles[iNum]->iTimeM;
}

time_t CZipFile::Time(int iNum,int Mode)
{
	if((unsigned)iNum>=(unsigned)m_iFilesNum)
		{
		return CAF_ERROR_MISS;
		}

	switch(Mode)
		{
	case CAF_TIME_W: return m_pFiles[iNum]->iTimeM;		// Zeit der letzten nderung
	case CAF_TIME_C: return m_pFiles[iNum]->iTimeC;		// Zeit der Dateierstellung
	case CAF_TIME_A: return m_pFiles[iNum]->iTimeA;		// Zeit des letzten Zugriffs
		}


return CAF_ERROR_MISS;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CZipFile::Name(int iNum)
{

	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex
		{
		return m_pFiles[iNum]->pName;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum)								// Datei die gerade eingefgt wird
		{
		return m_pInsertFile;
		}


return NULL;
}


//*****************************************************************************
//*
//*		Name
//*
//*****************************************************************************
//	Holt den Dateinamen einer Archivdatei samt Lnge.
//	iNum		: Ist die Indexnummer der Datei (-1=Archiv-Datei)
//	iLen		: Hier wird die Lnge des Namens gespeichert
//	Ergibt eine Zeiger auf den Dateinamen der Datei bzw. NULL bei einem Fehler.
zchar *CZipFile::Name(int iNum,int &iLen)
{


	if((unsigned)iNum<(unsigned)m_iFilesNum)			// Prfe den Dateiindex	
		{
		iLen = m_pFiles[iNum]->uNameLen;
		return m_pFiles[iNum]->pName;
		}

	if(iNum==-1)										// Name der Archiv-Datei
		{
		iLen = strzlen(m_cFilePath);					
		return m_cFilePath;
		}

	if(iNum==m_iFilesNum)								// Datei die gerade eingefgt wird
		{
		iLen = (m_pInsertFile)? strzlen(m_pInsertFile):0;
		return  m_pInsertFile;
		}

	iLen = 0;


return NULL;
}


//*****************************************************************************
//*
//*		GetFileMode
//*
//*****************************************************************************
//  Wandelt die Attribute aus SysFindStruck in ZIP-Attribute um.
int CZipFile::GetFileMode(unsigned uAttr)
{
int		uRet=0;


	if(!(uAttr&SYS_WRITE ))uRet|=A_RONLY;
	if(  uAttr&SYS_ARCHIVE)uRet|=A_ARCHIVE;
	if(  uAttr&SYS_SYSTEM )uRet|=A_SYSTEM;
	if(  uAttr&SYS_HIDDEN )uRet|=A_HIDDEN;
	if(  uAttr&SYS_DIR    )uRet|=A_DIR;


return uRet;
}



//*****************************************************************************
//*
//*		CreateString
//*
//*****************************************************************************
//	Erzeugt einen ASCII bzw. Unicode Text aus dem einem Text.
int	CZipFile::CreateString(const zchar *pName,char *pBuffer,int iSize,unsigned short *pFlags)
{
static const unsigned char  aAnisToAscii[128] =
	{
	0x00,0x00,0x00,0x9F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xAD,0xBD,0x9C,0xCF,0xBE,0xDD,0xF5,0xF9,0xB8,0xA6,0xAE,0xAA,0xF0,0xA9,0xEE,
	0xF8,0xF1,0xFD,0xFC,0xEF,0xE6,0xF4,0xFA,0xF7,0xFB,0xA7,0xAF,0xAC,0xAB,0xF3,0xA8,
	0xB7,0xB5,0xB6,0xC7,0x8E,0x8F,0x92,0x80,0xD4,0x90,0xD2,0xD3,0xDE,0xD6,0xD7,0xD8,
	0xD1,0xA5,0xE3,0xE0,0xE2,0xE5,0x99,0x9E,0x9D,0xEB,0xE9,0xEA,0x9A,0xED,0xE8,0xE1,
	0x85,0xA0,0x83,0xC6,0x84,0x86,0x91,0x87,0x8A,0x82,0x88,0x89,0x8D,0xA1,0x8C,0x8B,
	0xD0,0xA4,0x95,0xA2,0x93,0xE4,0x94,0xF6,0x9B,0x97,0xA3,0x96,0x81,0xEC,0xE7,0x98,
	};

unsigned 	uChar;
unsigned 	uLast;
int			iType;
int			iPos;
int			iLen;


	iType = 0;
	uLast = 0;

	for(iPos=0,iLen=0;iPos<iSize;iPos++)				// Text in ASCII umwandeln
		{
		#ifdef UNICODE
		uChar = pName[iPos];
		#else
		uChar = ((unsigned char*)pName)[iPos];
		#endif

		if(uChar>=256)									// ASCII unmglich
			{
			iType=1;
			break;
			}

		if(uChar>=128)
			{
				uChar = aAnisToAscii[uChar-128];
			if(!uChar || uLast>=0xC0){iType=1;break;}
			}

		pBuffer[iLen] = uChar;
		uLast         = uChar;
		iLen++;
		}

	if(!iType)											// Ist es ASCII
		{
		if(pFlags)*pFlags&=~FLGUDF;
		return iLen;
		}

	for(iPos=0,iLen=0;iPos<iSize;iPos++)				// In UTF8 umwandeln
		{
		#ifdef UNICODE
		uChar = pName[iPos];
		#else
		uChar = ((unsigned char*)pName)[iPos];
		#endif

		iLen += chartoutf8(pBuffer+iLen,uChar);
		}

	if(pFlags)*pFlags|=FLGUDF;



return iLen;
}


//*****************************************************************************
//*
//*		ReadString
//*
//*****************************************************************************
//	Liest aus dem Zip-Archiv einen Text und wandelt ihn in ANSI um.
//	pMem	: Textpuffer fr den geladenen Text
//	iSize	: Gre des Textpuffer in Zeichen
//	uFlags	: Diverse Flags (FLGUDF=ASCII zu ANSI)
//	Ergibt -1 bei einem Fehler ansonsten die geladenen Bytes
//	Bei einem Fehler kommt es zur Exeption CAF_ERROR_READ
int	CZipFile::ReadString(zchar *pMem,int iSize,unsigned uFlags)
{
char		cBuffer[260];
char	   *pPtr;
unsigned	uChar;
unsigned	uMask;
int			iCount;
int			iType;
int			iPos;
int			iLen;
int			iOff;




	if(iSize>=sizeof(cBuffer))								// Speicher fr Text
		   pPtr = (char*)alloca(iSize+1);
	else   pPtr = cBuffer;


	iCount		= MainRead(pPtr,iSize);
	iType		= 0;
	uMask		= 0;
	pPtr[iCount]= 0;


	for(iPos=0,iOff=0;iPos<iCount;iOff++)					// Ist es ein Unicode Text
		{
		iLen        = utf8tochar(pPtr+iPos,uChar);
		iPos       += iLen;
		iType      |= iLen;
		uMask      |= uChar;
		pMem[iOff]  = uChar;
		}

	pMem[iOff] = 0;

	if(!(uFlags&FLGUDF) && iType<=1 && (uMask&0x80))		// Von ASCII nach ANSI
		{
		OemToChar(pPtr,pMem);
		}



return iOff;
}

//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Liste der Dateien im Archiv.
//  iMode	: Ist der Sortiermodus (0=Standartmodus) 	
//				CAF_FLAG_SORT	 0x0400		= Nach Namen  sortieren
//				CAF_FLAG_SORTLEN 0x0500		= Nach Lngen sortieren
//				CAF_FLAG_SORTPOS 0x0600		= Nach Offset sortieren (intern)
//				CAF_FLAG_SORTEX	 0x0700		= Nach Namen mit Zahlen Bercksichtigung
//	Ergibt die Anzahl der Dateien im Archiv.
int	CZipFile::Sort(int iMode)
{

		iMode&=0x0700;
	if(!iMode)											// Standardmodus benutzen
		{
		iMode = m_iModeFlags&0x0700;
		}
	
	iMode |= 0x0400;

	PROGR(CAF_PROG_SORT,-1,0);


	if(!m_pFiles)return 0;

	if(iMode==m_iSortMode)								// Ist die Liste bereits sortiert
		{
		return m_iFilesNum;
		}

	if(m_iFilesNum)
		{
		switch(iMode)
			{
		case    CAF_FLAG_SORTEX:	qsort(m_pFiles,m_iFilesNum,sizeof(ZipItem*),CmpEx  );break;
		case    CAF_FLAG_SORTLEN:	qsort(m_pFiles,m_iFilesNum,sizeof(ZipItem*),CmpLen );break;
		case    CAF_FLAG_SORTPOS:	qsort(m_pFiles,m_iFilesNum,sizeof(ZipItem*),CmpPos );break;
		default:					qsort(m_pFiles,m_iFilesNum,sizeof(ZipItem*),CmpName);break;
			}
		}

	m_iModeFlags &= ~0x0300;
	m_iModeFlags |=  0x0300&iMode;
	m_iSortMode   =  iMode;


return m_iFilesNum;
}

//*****************************************************************************
//*
//*		UtcToDosTime
//*
//*****************************************************************************
//	Eine UTC-Zeit (Sekunden seit 1.1.1970) in eine Dos-Zeit umwandeln.
unsigned CZipFile::UtcToDosTime(time_t iUtcTime)
{
struct tm  *pData;
time_t		iEven;


	iEven = (iUtcTime+1)&(~1);							// Gerade Anzahl an Sekunden
	pData = localtime(&iEven);							// Unix-Zeit von MSDOS

	if(!pData || pData->tm_year<80)						// 00:00:00 1.1.1980
		{
		return (1<<21)|(1<<16);
		}

return (((unsigned)pData->tm_year -   80) << 25) |
	   (((unsigned)pData->tm_mon  +    1) << 21) |
		((unsigned)pData->tm_mday		  << 16) |
        ((unsigned)pData->tm_hour         << 11) |
		((unsigned)pData->tm_min          <<  5) |
		((unsigned)pData->tm_sec          >>  1);
}

//*****************************************************************************
//*
//*		DosToUtcTime
//*
//*****************************************************************************
//	Eine Dos-Zeit in eine UTC-Zeit (Sekunden seit 1.1.1970) umwandeln.
time_t CZipFile::DosToUtcTime(unsigned uDosTime)
{
struct  tm sTime;


	sTime.tm_year  = ((uDosTime>>25)     )+80;
	sTime.tm_mon   = ((uDosTime>>21)&0x0F)-1;
	sTime.tm_mday  = ((uDosTime>>16)&0x1F);
	sTime.tm_hour  = ((uDosTime>>11)&0x1F);
	sTime.tm_min   = ((uDosTime>> 5)&0x3F);
	sTime.tm_sec   = ((uDosTime<< 1)&0x3F);
	sTime.tm_isdst = 0;
	sTime.tm_wday  = 0;
	sTime.tm_yday  = 0;


return mktime(&sTime);
}


//*****************************************************************************
//*
//*		PutLOC
//*
//*****************************************************************************
//	LOC-Block speichern
//	"PK\03\04"				Kennung
//	LOCVER		0           version needed pEntry extract
//	LOCFLG		2           encrypt, deflate flags
//	LOCHOW		4           compression m_iCompressMode
//	LOCTIM		6           last modified file time, DOS format
//	LOCDAT		8           last modified file date, DOS format
//	LOCCRC  	10          uncompressed crc-32 for file
//	LOCSIZ  	14          compressed size in zip file
//	LOCLEN  	18          uncompressed size
//	LOCNAM  	22          length of filename
//	LOCEXT  	24          length of extra field
int CZipFile::PutLOC(ZipItem  *pEntry)
{
char			cBuffer[1024];
char			cPush[64];
char			cPushPos=0;
unsigned		uLenComp;
unsigned		uLenFull;
int				iLen;


	uLenComp = (pEntry->uLenComp>=0xFFFFFFFF)?  0xFFFFFFFF:(unsigned)pEntry->uLenComp;
	uLenFull = (pEntry->uLenFull>=0xFFFFFFFF)?  0xFFFFFFFF:(unsigned)pEntry->uLenFull;
	iLen     = CreateString(pEntry->pName,cBuffer,pEntry->uNameLen,&pEntry->uFlags);

	SetExHeader(pEntry);

	PUSHS("PK\03\04"       );
	PUSHW(pEntry->uVerExrt );
	PUSHW(pEntry->uFlags   );
	PUSHW(pEntry->uCompMode);
	PUSHD(pEntry->uDosTime );
	PUSHD(pEntry->uCrcSum  );
	PUSHD(        uLenComp );
	PUSHD(        uLenFull );
	PUSHW(iLen             );
	PUSHW(pEntry->uExt     );

	MainWrite(cPush,cPushPos);
	MainWrite(cBuffer,iLen  );
	MainWrite(pEntry->pExtra,pEntry->uExt);



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		PutCEN
//*
//*****************************************************************************
//	CEN-Block speichern
//	"PK\01\02"				Kennung
//  CENVEM		0           version made by
//  CENVER  	2           version needed pEntry extract
//  CENFLG  	4           encrypt, deflate flags
//  CENHOW  	6           compression m_iCompressMode
//  CENTIM  	8           last modified file time, DOS format
//  CENDAT  	10          last modified file date, DOS format
//  CENCRC  	12          uncompressed crc-32 for file
//  CENSIZ  	16          compressed size in zip file
//  CENLEN  	20          uncompressed size
//  CENNAM  	24          length of filename
//  CENEXT  	26          length of extra field
//  CENCOM  	28          file m_pComment length
//  CENDSK  	30          disk number start
//  CENATT  	32          internal file attributes
//  CENATX  	34          external file attributes
//  CENOFF  	38          relative offset of LOC-header
int CZipFile::PutCEN(ZipItem  *pEntry)
{
char		cBuffer[1024];
char		cPush[64];
char		cPushPos=0;
unsigned	uLenComp;
unsigned	uLenFull;
unsigned	uOffset;
int			iLen;



	uOffset  = (pEntry->uOff    >=0xFFFFFFFF)?  0xFFFFFFFF:(unsigned)pEntry->uOff;
	uLenComp = (pEntry->uLenComp>=0xFFFFFFFF)?  0xFFFFFFFF:(unsigned)pEntry->uLenComp;
	uLenFull = (pEntry->uLenFull>=0xFFFFFFFF)?  0xFFFFFFFF:(unsigned)pEntry->uLenFull;
	iLen     = CreateString(pEntry->pName,cBuffer,pEntry->uNameLen,&pEntry->uFlags);

	SetExHeader(pEntry);

	PUSHS("PK\01\02");
	PUSHW(pEntry->uVerCreate);
	PUSHW(pEntry->uVerExrt  );
	PUSHW(pEntry->uFlags    );
	PUSHW(pEntry->uCompMode );
	PUSHD(pEntry->uDosTime  );
	PUSHD(pEntry->uCrcSum   );
	PUSHD(        uLenComp  );
	PUSHD(        uLenFull  );
	PUSHW(iLen              );
	PUSHW(pEntry->uCenLen   );
	PUSHW(pEntry->uComLen   );
	PUSHW(pEntry->uDisk     );
	PUSHW(pEntry->uAtt      );
	PUSHD(pEntry->uAtx      );
	PUSHD(        uOffset   );

	MainWrite(cPush,cPushPos);
	MainWrite(cBuffer,iLen  );
	MainWrite(pEntry->pCentral,pEntry->uCenLen);



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		PutEND
//*
//*****************************************************************************
//	Einene END-Block speichern und die CEN-Blcke.
//	"PK\05\06"				Kennung
//  ENDDSK		0           number of this disk
//  ENDBEG  	2           number of the starting disk
//  ENDSUB  	4           entries on this disk
//  ENDTOT  	6           total number of entries
//  ENDSIZ  	8           size of entire central directory
//  ENDOFF  	12          offset of central on starting disk
//  ENDCOM  	16          length of zip file m_pComment
int CZipFile::PutEND()
{
char		cBuffer[2048];
char		cPush[128];
char		cPushPos=0;
aint64		iDirSize;
unsigned	uDirPos;
int			iComLen;
int			iCount;
ZipItem    *pEntry;
zchar      *pText;
int			iLen;
int			iPos;



	m_iCenPos	= MainTell();
	iCount		= 0;

	for(iPos=0;iPos<m_iFilesNum;iPos++)					// Das Verzeichnis speichern
		{
			pEntry=m_pFiles[iPos];
		if( pEntry->cDelete)continue;
		if( pEntry->cError )continue;
		if(!pEntry->cSaved )continue;
		
		PutCEN(pEntry);
		iCount++;
		}

	m_iEndPos	= MainTell();
	uDirPos		= (int)m_iCenPos;
	iDirSize	= m_iEndPos-m_iCenPos;

		pText=m_pComment;
	if(!pText)pText=COMMENT;

	iComLen = strzlen(pText);

	if(iComLen>sizeof(cBuffer)/4)
		{
		iComLen = sizeof(cBuffer)/4;
		}

	iLen = CreateString(pText,cBuffer,iComLen);

	if(m_iEndPos>0xFFFFF000)							// Im 64-Bit-Format speichern
		{
		PUSHS("PK\06\06");
		PUSHQ(44		)
		PUSHW(45		);
		PUSHW(45		);
		PUSHD(0			);
		PUSHD(0			);
		PUSHQ(iCount	);
		PUSHQ(iCount	);
		PUSHQ(iDirSize	);
		PUSHQ(m_iCenPos	);
		PUSHS("PK\06\07");
		PUSHD(0			);
		PUSHQ(m_iEndPos	);
		PUSHD(0			);

		uDirPos	= 0xFFFFFFFF;
		}

	if(iCount>0xFFFF)
		{
		iCount = 0xFFFF;
		}

	PUSHS("PK\05\06");									// END-Header speichern
	PUSHW(0			);
	PUSHW(0			);
	PUSHW(iCount	);
	PUSHW(iCount	);
	PUSHD(iDirSize	);
	PUSHD(uDirPos	);
	PUSHW(iLen   	);

	MainWrite(cPush,cPushPos);
	MainWrite(cBuffer,iLen  );


	m_iFileSize = MainTell();							// Dateilnge

	EndOfFile();



return CAF_ERROR_OK;
}


//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine ZIP-Datei und liest alle bereits gespeicherten Dateien ein.
//	Zuvor wurde eine Datei bzw. ein Speicherbereich in CArchiveFile::Open geffnet.
//	iLevel	: Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//				CAF_ERROR_OK		Alles ok
//				CAF_ERROR_NONE		Wenn die Datei nicht gefunden werden konnte
//				CAF_ERROR_OPEN		Wenn die Datei nicht geffnet werden konnte
//				CAF_ERROR_CREATE	Wenn die Datei nicht erzeugt  werden konnte
//				CAF_ERROR_READ		Bei Lese-Fehlern
//				CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//				CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//				CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
int CZipFile::ArchOpen(int iLevel)
{
char		cMem[4096+4];
ZipItem    *pEntry=0;
unsigned	uHeader2;
unsigned	uHeader1;
unsigned	uHeader;
int			iDiskNum;
int			iDiskAnf;
int			iDiskLst;
int			iCommLen;
int			iListNum;
aint64		iListPos;
aint64		iListLen;
int			iFound;
int			iFlags;
int 		iCount;
aint64		iLast;
aint64		iPos;
int			iAdd;
int			iNum;
int			iOff;



	Clear();

	m_iErrorCode	= CAF_ERROR_OK;
	m_iSortMode		= 0;
	m_iDoneNow		= 0;
	m_iDoneMax		= 0;
	m_iDoneMode		= 0;


	if(m_iFileSize==0)									// Leere Datei
		{
		if(m_iOpenMode!=CAF_CREATE)
			{
			return CAF_ERROR_FORM;
			}

		m_iChanged	= 1;
		m_iCenPos	= 0;

		return CAF_ERROR_OK;
		}


//******************** Suche END Block ****************************************

try	{
	*(int*)cMem   = 0;
		   iFound = 0;


	iPos     =      m_iFileSize&~0x0FFF;
	iCount   = (int)m_iFileSize& 0x0FFF;
	uHeader1 = GetL4("PK\05\06");
	uHeader2 = GetL4("PK\06\07");

	if(iCount>0)										// In den obersten Bytes suchen
		{
		MainSeek(iPos,SEEK_SET);
		MainRead(cMem,iCount  );

		cMem[iCount  ] = cMem[0];
		cMem[iCount+1] = cMem[0];
		cMem[iCount+2] = cMem[0];

		for(iOff=iCount-1;iOff>=0;iOff--)				// Suche Ende "PK\05\06" oder "PK\06\07"
			{
			   uHeader = GetL4(cMem+iOff);
			if(uHeader==uHeader1)break;
			if(uHeader==uHeader2)break;
			}

		if(iOff>=0)
			{
			iFound	= 1;
			iPos   += iOff;
			}
		}

	if(!iFound)iPos-=0x1000;
	if(!iFound)
		{
		while(iPos>=0) 									// In den restlichen Blcken suchen
			{
			cMem[0x1000]=cMem[0];
			cMem[0x1001]=cMem[1];
			cMem[0x1002]=cMem[2];

			MainSeek(iPos);
			MainRead(cMem,4096);

			for(iOff=4095;iOff>=0;iOff--)				// Suche Ende "PK\05\06" oder "PK\06\07"
				{
				   uHeader = GetL4(cMem+iOff);
				if(uHeader==uHeader1)break;
				if(uHeader==uHeader2)break;
				}

			if(iOff<0)
				{
				iPos-=4096;
				continue;
				}

			iFound	= 1;
			iPos   += iOff;
			
			break;
			}
		}


	if(!iFound)											// Kein END-Block
		{
		throw CAF_ERROR_FORM;
		}


//********************** END-Header lesen *************************************


	MainSeek(iPos+4);
	m_iDoneMax	= iPos;
	m_iEndPos	= iPos;
	iListNum	= 0;
	iListLen	= 0;
	iDiskLst	= 0;

	if(uHeader==uHeader1)								// 32-Bit Version
		{
		if(MainRead(cMem,ENDHEAD)==-1)
			{
			return m_iErrorCode=CAF_ERROR_READ;
			}

		iListNum	= GetL2(cMem+ENDTOT);
		iListLen    = GetL4(cMem+ENDSIZ);				// Offset zu Listeneintrge
		iDiskNum	= GetL2(cMem+ENDDSK);				// Diskettenanzahl
		iDiskAnf	= GetL2(cMem+ENDBEG);				// Startdiskette
		iDiskLst	= GetL2(cMem+ENDSUB);				// Listeneintrge auf Diskette
		iCommLen	= GetL2(cMem+ENDCOM);				// Lnge des Kommentars
		iListPos    = iPos-iListLen;

		if(iCommLen)									// Kommentar laden
			{
			if(m_pComment)delete m_pComment;
			   m_pComment=new zchar[iCommLen+1];
			   m_pComment[iCommLen]=0;

			ReadString(m_pComment,iCommLen);
			}

		if(GetL4(cMem+ENDOFF)==0xFFFFFFFF)				// Ist es eine 64-Bit Version
			{
			iPos -= 20;
			MainSeek(iPos);

			if(MainRead(cMem,4)==-1)
				{
				return m_iErrorCode=CAF_ERROR_READ;
				}

			   uHeader = GetL4(cMem);
			if(uHeader!=uHeader2)
				{
				throw CAF_ERROR_FORM;					// Kein END64-Block
				}
			}
		}

	if(uHeader==uHeader2)								// 64-Bit Version
		{
		if(MainRead(cMem,ENDHEAD64)==-1)
			{
			return m_iErrorCode=CAF_ERROR_READ;
			}

		iDiskNum	= GetL4(cMem+12);					// Diskettenanzahl
		iDiskAnf	= GetL4(cMem+ 0);					// Startdiskette
		iListPos    = GetL8(cMem+ 4);					// Offset zu Listeneintrge

		MainSeek(iListPos);

		if(MainRead(cMem,CNDHEAD64)==-1)
			{
			return m_iErrorCode=CAF_ERROR_READ;
			}

		   uHeader = GetL4(cMem);
		if(uHeader!=GetL4("PK\06\06"))
			{
			throw CAF_ERROR_FORM;						// Kein CEN64-Block
			}

		iListPos = GetL8(cMem+48);						// Offset zu Listeneintrge
		}

	if(iDiskNum>1 || iDiskAnf>0 || iDiskLst!=iListNum)
		{
		throw CAF_ERROR_MULTI;							// Zip ber mehrere Disketten
		}


//************************** CEN Blcke laden *********************************


	if(iListPos<0 || iListPos>m_iFileSize)				// Falsche LOC Position
		{
		throw CAF_ERROR_FORM;
		}


	m_iCenPos = iListPos;								// Position des ersten CEN-Blockes
	MainSeek(iListPos);
	MainRead(cMem,4);
	iPos     = iListPos+4;
	uHeader1 = GetL4("PK\01\02");
	uHeader2 = GetL4("PK\06\06");

	for(;;)
		{
		iLast   = iPos-3;
		uHeader = GetL4(cMem);

		if(uHeader!=uHeader1)
			{
			if(iPos>=m_iEndPos)break;

			if(m_iModeFlags&CAF_FLAG_NOCHECK)			// Auch kappute Header suchen
				{
				goto ScanNext;
				}

			break;
			}

		MainRead(cMem,CENHEAD);

		iNum   = ZipNew();								// Neuer Block
		iPos  += CENHEAD;
		pEntry = m_pFiles[iNum];
		memcpy(&(pEntry->uVerCreate),cMem+CENVEM,CENNAM-CENVEM);

		pEntry->uVerCreate = GetL2(cMem+CENVEM);
		pEntry->uVerExrt   = GetL2(cMem+CENVER);
		pEntry->uFlags     = GetL2(cMem+CENFLG);
		pEntry->uCompMode  = GetL2(cMem+CENHOW);
		pEntry->uDosTime   = GetL4(cMem+CENTIM);
		pEntry->uCrcSum    = GetL4(cMem+CENCRC);
		pEntry->uLenComp   = GetL4(cMem+CENSIZ);
		pEntry->uLenFull   = GetL4(cMem+CENLEN);
		pEntry->uNameLen   = GetL2(cMem+CENNAM);
		pEntry->uCenLen    = GetL2(cMem+CENEXT);
		pEntry->uComLen    = GetL2(cMem+CENCOM);
		pEntry->uDisk      = GetL2(cMem+CENDSK);
		pEntry->uAtt       = GetL2(cMem+CENATT);
		pEntry->uAtx       = GetL4(cMem+CENATX);
		pEntry->uOff       = GetL4(cMem+CENOFF);
		pEntry->cDosFlag   = (pEntry->uVerCreate&0xFF00) == 0;
		pEntry->iTimeM     = DosToUtcTime(pEntry->uDosTime);
		pEntry->cAesMode   = 0;
		pEntry->cAesDecom  = 0;

		if(!pEntry->uNameLen)							// Kein Dateiname
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)
				{
				pEntry->pName    = new  zchar[1];
				pEntry->pName[0] = 0;
				goto ScanNext;
				}

			throw CAF_ERROR_FORM;
			}
		else if(iPos+pEntry->uNameLen>m_iEndPos)
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)
				{
				pEntry->pName    = new  zchar[1];
				pEntry->pName[0] = 0;
				goto ScanNext;
				}

			throw CAF_ERROR_FORM;
			}
		else{
			pEntry->pName = new  zchar[pEntry->uNameLen+1];
			pEntry->pName             [pEntry->uNameLen]=0;
			
			iOff	= ReadString(pEntry->pName,pEntry->uNameLen,pEntry->uFlags);
			iPos   += pEntry->uNameLen;

			if(iOff>0 && pEntry->pName[iOff-1]=='/')	// Slash am Ende entfernen
				{
				iOff--;
				pEntry->pName[iOff]=0;

				if(pEntry->uLenFull==0)
					{
					pEntry->uAtx |= A_DIR;
					}
				}

			pEntry->uNameLen = iOff;
			}

		if(iPos+pEntry->uCenLen>m_iEndPos)				// Passt die Lnge der Extra Bytes
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)goto ScanNext;
			throw CAF_ERROR_FORM;
			}
		else if(pEntry->uCenLen)						// Extra Bytes laden
			{
			pEntry->pCentral = new uchar[pEntry->uCenLen+1];
			pEntry->pCentral            [pEntry->uCenLen]=0;
			MainRead(pEntry->pCentral  , pEntry->uCenLen);
			iPos                      += pEntry->uCenLen;
			}

		if(iPos+pEntry->uComLen>m_iEndPos)				// Passt die Lnge des Kommentar
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)goto ScanNext;
			throw CAF_ERROR_FORM;
			}
		else if(pEntry->uComLen)						// Kommentar laden
			{
			pEntry->pComment = new zchar [pEntry->uComLen+1];
			pEntry->pComment             [pEntry->uComLen]=0;
      		
			iOff	= ReadString(pEntry->pComment,pEntry->uComLen);
			iPos   += pEntry->uComLen;
			
			pEntry->uComLen = iOff;
			}

		ScanExHeader(pEntry,(unsigned char*)pEntry->pCentral,pEntry->uCenLen);

		PROG(CAF_PROG_FIND,iNum);
		MainRead(cMem,4);								// Nchste Kennug lesen
		iPos += 4;
		continue;

ScanNext:

		MainSeek(iLast);
		iNum = (int)(m_iEndPos-iLast);
		if(iNum<CENHEAD+4)break;

		if(iNum>4096)iNum=4096;
		MainRead(cMem,iNum);
		iNum -= 3;

		for(iPos=0;iPos<iNum;iPos++)					// Suche weitere Dateien
			{
			if(GetL4(cMem+iPos)==GetL4("PK\01\02"))break;
			}

		if(iPos>=iNum)
			{
			iLast += iNum;
			goto ScanNext;
			}

		memcpy(cMem,"PK\01\02",4);
		iPos = iLast+iPos+4;
		MainSeek(iPos);
		}

//************************** LOC Blcke laden *********************************

	if(m_iFilesNum>0)
			iAdd = (int)(m_iDoneMax/m_iFilesNum);
	else	iAdd = 1;

	for(iNum=0;iNum<m_iFilesNum;iNum++)
		{
		pEntry = m_pFiles[iNum];

		if((aint64)pEntry->uOff+LOCHEAD+4>=m_iCenPos)	// Prfe den Offset
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)
				{
				pEntry->cError    = 1;
				pEntry->uExt	  = 0;
				goto Next;
				}

			throw CAF_ERROR_FORM;
			}

		MainSeek(pEntry->uOff);							// Zu LOC-Block
		MainRead(cMem,4);

		if(GetL4(cMem)!=GetL4("PK\03\04"))				// Kein LOC-Block
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)
				{
				pEntry->cError    = 1;
				pEntry->uExt	  = 0;
				goto Next;
				}

			throw CAF_ERROR_FORM;
			}

		MainRead(cMem,LOCHEAD);

		iFlags				= GetL2(cMem+LOCFLG);
		iOff				= GetL2(cMem+LOCNAM);
		pEntry->uExt		= GetL2(cMem+LOCEXT);

		   iOff = ReadString((zchar*)(cMem+LOCHEAD),iOff,pEntry->uFlags);
		if(iOff>0 && ((zchar*)(cMem+LOCHEAD))[iOff-1]=='/')
			{
			iOff--;
			}

		if(iOff!=pEntry->uNameLen)						// Sind die Namen gleich ?
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)
				{
				pEntry->cError=1;
				goto Next;
				}

			throw CAF_ERROR_FORM;
			}

		if(memicmp(cMem+LOCHEAD,pEntry->pName,iOff*sizeof(zchar)))
			{
			if(m_iModeFlags&CAF_FLAG_NOCHECK)
				{
				pEntry->cError=1;
				goto Next;
				}

			throw CAF_ERROR_FORM;
			}

		if(pEntry->uExt)
			{
			pEntry->pExtra=new uchar[pEntry->uExt+1];
			pEntry->pExtra          [pEntry->uExt]=0;
			MainRead(pEntry->pExtra ,pEntry->uExt);
			ScanExHeader(pEntry,(unsigned char*)pEntry->pExtra,pEntry->uExt);
			}

		if(iFlags&FLGEXT)								// Extended Header vorhanden
			{
			char		cTemp[16];
			auint64		uLen;

				uLen=GetL4(cMem+LOCSIZ);
			if(!uLen || uLen==0xFFFFFFFF)
				{
				uLen = pEntry->uLenComp;
				}

			iPos = pEntry->uOff+4+LOCHEAD+iOff+pEntry->uExt+uLen;

			MainSeek(iPos);
			MainRead(cTemp,16);

			if(GetL4(cTemp)!=GetL4("PK\x07\x08"))		// Kein EXT-Block
				{
				if(m_iModeFlags&CAF_FLAG_NOCHECK)
					{
					pEntry->cError=1;
					goto Next;
					}

				throw CAF_ERROR_FORM;
				}

			memcpy(cMem+LOCCRC,cTemp+4,12);
			}

		pEntry->uFlags  &= 0x1FFF;
		cMem[LOCFLG+1]  &= 0x001F;

Next:
		pEntry->cSaved=1;
		m_iDoneNow += iAdd;
		}

//*****************************************************************************
	}
catch(int iError)										// Bekannter Fehler
	{
	m_iErrorCode=iError;
	PROGN(CAF_PROG_ERROR,-1);
	Break();
	return iError;
	}

	if(m_iModeFlags&CAF_FLAG_SORT)						// Sortieren ?
		{
		Sort();
		}

	m_iErrorCode = CAF_ERROR_OK;
	m_iDoneNow	 = m_iDoneMax;



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einer Datei in der Dateiliste
//	pName	: Ist der  Dateiname nach dem gesucht wird
//	Ergibt die Indexnummer der Datei, bzw. -1 wenn die Datei nicht gefunden wurde.
int CZipFile::Find(const zchar *pName)
{
ZipItem	*pEntry;
int		 iPos;
int		 iLen;


	if(*pName=='/' )pName++;
	if(*pName=='\\')pName++;


	iLen = strzlen(pName);

	for(iPos=0;iPos<m_iFilesNum;iPos++)					// Eintrag in Kette
		{
		   pEntry=m_pFiles[iPos];
		if(pEntry->cDelete       )continue;
		if(pEntry->uNameLen!=iLen)continue;

		if(!CmpFileNameN(pName,pEntry->pName))
			{
			return iPos;
			}
		}


return -1;
}

//*****************************************************************************
//*
//*		Write
//*
//*****************************************************************************
//	Speichert eine einzelnen Datei am Ende im ZIP-Archiv, samt LOC-Header 
//	und die allenen Extra-Feldern.
//	Dateidaten. ber throw werden die Fehler-Codes zurck gegeben
//	iNum		: Ist die Indexnummer der Datei
int CZipFile::Write(int iNum)
{
ZipItem	   *pEntry;
HANDLE		hFile;
unsigned	uAttr;
auint64		uOffD;
auint64		uOffE;
int			iIsDir;
int			iLevel;
int			iMode;
int			iErr;
aint64		iLen;
aint64		iPos;




	pEntry = m_pFiles[iNum];
	PROG(CAF_PROG_WRITE,iNum);

	
	if(pEntry->uAtx&A_DIR)								// Ist ese ein Verzeichniss
		{
		iIsDir	= 1;
		iLevel	= 0;
		iLen	= 0;
		uAttr	= A_DIR;
		iMode	= CAF_STORE;
		}												// Es ist eine Datei
	else{	
		iIsDir	= 0;
		iLevel	= m_iLevel;
		iMode	= m_iCompressMode;
		uAttr  	= 0;

		if(ArchiveType(pEntry->pPath))					// Gezippte Dateien nicht komprimieren
			{
			iMode = CAF_STORE;
			}

		   hFile=CreateFile(pEntry->pPath,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,0);
		if(hFile==INVALID_HANDLE_VALUE)
			{
			throw CAF_ERROR_FOPEN;
			}

		iLen = GetFileSize(hFile);

		if(iLen ==0	       ) iMode = CAF_STORE;
		if(iMode==CAF_BEST ){iMode = CAF_DEFLATE;iLevel=9;}
		if(iMode==CAF_STORE){                    iLevel=0;}
		}



//******************** Daten speichern ****************************************


	iPos				=  MainTell();
	m_pErrorFile		=  pEntry->pName;

	pEntry->uAtt	 	=  BINARY;
	pEntry->uFlags  	=  0;
	pEntry->uVerCreate  =  VERSION;
	pEntry->uVerExrt  	= (iMode==CAF_STORE)? 10:20;
	pEntry->uCompMode  	=  iMode;
	pEntry->uLenComp  	= (iMode==CAF_STORE && iLen>=0)? iLen:0;
	pEntry->uLenFull  	= (iLen>0)? iLen:0;
	pEntry->uAtx  		=  uAttr | (pEntry->uAtx & 0x0000FF00);
	pEntry->uCrcSum  	=  CRC32_INIT;
	pEntry->uOff  		=  iPos;
	pEntry->uLenComp	=  0;
	pEntry->cDosFlag	=  0;
	pEntry->uDisk  		=  0;


	PutLOC(pEntry);
	uOffD=MainTell();

	if(!iIsDir)											// Komprimiere Datei
		{
		try	{
			SetOutput(m_pDecoder,CAF_MAIN);
			m_pDecoder->SetIn(hFile,-1,CRC32_INIT);
			m_pDecoder->Compress(iLevel);
			}
		catch(char *pText)								// Fehler beim Komprimieren
			{
			CloseFile(hFile);
			
			   iErr = m_pDecoder->GetError(); pText=0;
			if(iErr==CHF_ERROR_READ )throw CAF_ERROR_FREAD;
			if(iErr==CHF_ERROR_WRITE)throw CAF_ERROR_WRITE;
									 throw CAF_ERROR_LOGIC;
			}


		if(m_pDecoder->GetFlags()&CHF_NOCOMPESS)		// Wenn die komprimierten Daten grer sind
			{
			if(MainSeek(uOffD,SEEK_SET)!=(aint64)uOffD)throw CAF_ERROR_FREAD;
			if(SetFilePointer(hFile,0,0,FILE_BEGIN)!=0)throw CAF_ERROR_FREAD;

			try	{
				SetOutput(m_pDecoder,CAF_MAIN);
				m_pDecoder->SetIn(hFile,-1,CRC32_INIT);
				m_pDecoder->Compress(0);
				}
			catch(char *pText)							// Fehler beim Komprimieren
				{
				CloseFile(hFile);

				MainSeek(iPos);
				CATCHALL(EndOfFile());
				
				   iErr = m_pDecoder->GetError(); pText=0;
				if(iErr==CHF_ERROR_READ )throw CAF_ERROR_FREAD;
				if(iErr==CHF_ERROR_WRITE)throw CAF_ERROR_WRITE;
										 throw CAF_ERROR_LOGIC;
				}
			
			iMode = CAF_STORE;
			}
		
		CloseFile(hFile);							
														// Neue werte fr LOC-Header
		iMode			  	= (m_pDecoder->GetMode  ()==CHF_STORE)? CAF_STORE:CAF_DEFLATE;
		pEntry->uCrcSum   	=  m_pDecoder->GetCrcInp();
		pEntry->uLenComp  	=  m_pDecoder->GetSize  ();
		pEntry->uLenFull  	=  iLen;
		pEntry->uCompMode 	=  iMode;
		pEntry->uVerExrt  	= (iMode==CAF_STORE)? 10:20;// Need PKUNZIP 2.0 except for store
		uOffE				=  MainTell();

		MainSeek(pEntry->uOff);
		PutLOC  (pEntry);
		MainSeek(uOffE );
		}


	m_iCenPos			=  MainTell();
	pEntry->cSaved	  	=  1;
	pEntry->cDelete	  	=  0;
	pEntry->cWrite	  	=  0;

	PROG(CAF_PROG_COMPRESS,iNum);



return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		ZipDel
//*
//*****************************************************************************
//	Einen Listeneintrag lschen
//	iNum		: Ist die Indexnummer der Datei
void CZipFile::ZipDel(int iNum)
{
ZipItem	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;

	pEntry=m_pFiles[iNum];

	if(!pEntry)return;

	m_iFilesNum--;
	m_pFiles[iNum]=0;

	if(iNum<m_iFilesNum)
		{
		memmove(m_pFiles+iNum,m_pFiles+iNum+1,(m_iFilesNum-iNum)*sizeof(m_pFiles[0]));
		}

	if(!pEntry->cDelete )m_iFileCount--;
	if( pEntry->pName   )delete pEntry->pName;
	if( pEntry->pExtra  )delete pEntry->pExtra;
	if( pEntry->pCentral)delete pEntry->pCentral;
	if( pEntry->pComment)delete pEntry->pComment;
 					     delete pEntry;

	if(m_iFilesNum<=0)
		{
		delete  m_pFiles;
		m_iFilesNum	 = 0;
		m_iFilesMax	 = 0;
		m_iFileCount = 0;
		}
}

//*****************************************************************************
//*
//*		ZipSetDel
//*
//*****************************************************************************
//	Listeneintrag lschen
//	iNum		: Ist die Indexnummer der Datei
void CZipFile::ZipSetDel(int iNum)
{
ZipItem	*pEntry;


	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;	// Nummer im Bereich

		pEntry=m_pFiles[iNum];
	if(!pEntry->cDelete)
		{
		m_iChanged		= 1;
		m_iFileCount   -= 1;
		pEntry->cDelete	= 1;
		}

	if(pEntry->cWrite)									// Gleich Block lschen
		{
		ZipDel(iNum);
		return;
		}

	if(iNum>=m_iFileCount)return;
	if(m_iModeFlags&CAF_FLAG_SORT)						// Sortierte Liste
		{
		Sort();
		return;
		}

	m_pFiles[iNum        ] = m_pFiles[m_iFileCount];
	m_pFiles[m_iFileCount] = pEntry;

}

//*****************************************************************************
//*
//*		ZipNew
//*
//*****************************************************************************
//	Listeneintrag erzeugen
#define	ZNEW_ADD	256
int	CZipFile::ZipNew()
{
ZipItem	**pNew;



	if(!m_pFiles || !m_iFilesMax)						// Neue Kette
		{
		m_pFiles = new ZipItem*[ZNEW_ADD];
		memset(m_pFiles,0,sizeof(ZipItem*[ZNEW_ADD]));
		m_iFilesMax=ZNEW_ADD;
		}

	if(m_iFilesNum >= m_iFilesMax)						// Kette vergrern
		{
		pNew = new ZipItem*[m_iFilesMax+ZNEW_ADD];
		memset(pNew+m_iFilesMax,0,sizeof(ZipItem*[ZNEW_ADD]));
		memcpy(pNew,m_pFiles     ,sizeof(ZipItem*)*m_iFilesMax);
		delete m_pFiles;

		m_pFiles	= pNew;
		m_iFilesMax+= ZNEW_ADD;
		}


		   m_pFiles[m_iFilesNum] = new    ZipItem;
	memset(m_pFiles[m_iFilesNum],0,sizeof(ZipItem));
		   m_iFilesNum++;
		   m_iFileCount++;


return m_iFilesNum-1;
}

//*****************************************************************************
//*
//*		Del
//*
//*****************************************************************************
//	Eine einzelne Datei aus dem Zip-Archiv lschen.
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//				CAF_ERROR_MISS   Wenn Datei nicht gefunden wurde
//				CAF_ERROR_WRITE  Wenn Datei keine Schreib erlaubnis hat
//				CAF_ERROR_OK     Wenn Datei gelscht wurde
int CZipFile::Del(int iNum)
{


	PROGR(CAF_PROG_ZDEL,iNum,CAF_ERROR_MISS);

	if(m_iOpenMode<CAF_WRITE)							// Keine Schreiberlaubnis
		{
		m_iErrorCode=CAF_ERROR_WRITE;
		PROGN(CAF_PROG_ERROR,iNum);
		return m_iErrorCode;
		}

	if(iNum<0 || iNum>=m_iFileCount)					// Falscher Index
		{
		m_iErrorCode=CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,iNum);
		return m_iErrorCode;
		}

	ZipSetDel(iNum);

	m_iSortMode 	=  0;
	m_iErrorCode	=  CAF_ERROR_OK;


return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
//	Die Bearbeitung abbrechen ohne Daten zu speichern
void CZipFile::Break()
{

	ReleaseData();
	m_iOpenMode=CAF_CLOSED;
	Clear();

}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das Zip-Archiv und speichert alle Daten.
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//			CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CZipFile::Close()
{
int		 iRet;


	
	m_iErrorCode = CAF_ERROR_OK;
	

	if(m_iOpenMode==CAF_CLOSED)
		{
		m_iOpenMode  = CAF_CLOSED;
		m_iErrorCode = CAF_ERROR_MISS;
		PROGN(CAF_PROG_ERROR,-1);
		return CAF_ERROR_MISS;
		}

	if(m_iOpenMode==CAF_READ)
		{
		ReleaseData();
		m_iOpenMode = CAF_CLOSED;
		return CAF_ERROR_OK;
		}

	if(m_iOpenMode<CAF_WRITE)
		{
		ReleaseData();
		m_iOpenMode  = CAF_CLOSED;
		m_iErrorCode = CAF_ERROR_LOGIC;
		PROGN(CAF_PROG_ERROR,-1);
		return CAF_ERROR_LOGIC;
		}


//***************** Alte Dateien lschen neue speichern ***********************


	   iRet=Flush();
	if(iRet!=CAF_ERROR_OK)
	if(iRet!=CAF_ERROR_BREAK)
		{
		Break();
		m_iErrorCode = iRet;
		return iRet;
		}


//***************** CEN und END-Block speichern *******************************


	try	{
		PROG(CAF_PROG_CLOSE,-1);
		if(m_iChanged)
			{
			MainSeek(m_iCenPos);
			PutEND();
			m_iChanged=0;
			}
		}
	catch(int iError)									// Bekannter Fehler
		{
		Break();
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,-1);
		return iError;
		}

	Break();


return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		Flush
//*
//*****************************************************************************
//	Speichert alle Dateien ins Zip-Archiv ohne CEN und END-Blcke
//	Ergibt 0 wenn alles ok war, bzw. einen anderen Fehlercode:
//			CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Bei schreibe Problemen
//			... siehe Error Codes
int CZipFile::Flush()
{
char       *pMem;
ZipItem    *pNext;
ZipItem    *pEntry;
HANDLE		hMapObj;
aint64		iPosR;
aint64		iPosW;
aint64		iSize;
aint64		iFull;
aint64		iMin;
aint64		iEnd;
aint64		iLen;
aint64		iOff;
aint64		iNum;
int			iIdx;
int			iDel;



	if(m_iOpenMode<CAF_WRITE)							// Modus des Archivs prfen
		{
		m_iErrorCode = CAF_ERROR_MISS;
		return CAF_ERROR_MISS;
		}


	m_iDoneNow  = 0;
	m_iDoneMax  = 0;
	m_iDoneMode = 0;
	iDel		= 0;


	if(m_iFilesNum>m_iFileCount)						// Wurden Dateien gelscht
		{
		iFull	= 0;
		iNum	= 0;
		iMin	= 1;
		iMin  <<= 56;

		for(iIdx=0;iIdx<m_iFilesNum;iIdx++)				
			{
			   pEntry=m_pFiles[iIdx];
			if(pEntry->cDelete)
				{
				if(iMin>(aint64)pEntry->uOff)iMin=pEntry->uOff;
				m_iDoneMax += 1024;
				iNum       += 1;
				iDel		= 1;
				continue;
				}

			if(pEntry->cWrite)							// Datei wird neu eingefgt
				{
				m_iDoneMax += pEntry->uLenFull;
				}
			else{
				iFull += pEntry->uLenComp;
				iFull += pEntry->uNameLen;
				iFull += 64;
				}
			}

		if(iFull>iMin && iNum)							// Ab erster gelschten Datei wird verschoben
			{
			m_iDoneMax += iFull-iMin;
			}
		}
	else{
		for(iIdx=0;iIdx<m_iFilesNum;iIdx++)				// Es werden nur Dateien eingefgt
			{
			   pEntry=m_pFiles[iIdx];
			if(pEntry->cDelete)
				{
				iDel = 1;
				continue;
				}

			if(pEntry->cWrite)
				{
				m_iDoneMax += pEntry->uLenFull;
				}
			}
		}



//******************** Alte Dateien lschen ***********************************


	iLen = MainSize();

	try	{
		PROG(CAF_PROG_FLUSH,-1);

		if(m_iFilesNum>m_iFileCount || iDel)
			{
			Sort(CAF_FLAG_SORTPOS);						// Nach Offsets sortieren

														// Via Memory-Map oder Speicher
			if(iLen<0x10000000 || m_hFile==INVALID_HANDLE_VALUE)
				{
				if(m_hFile==INVALID_HANDLE_VALUE)
					{
					hMapObj	= 0;
					pMem	= m_pMemPtr;
					}
				else{
						hMapObj=CreateFileMapping(m_hFile,0,PAGE_READWRITE,0,(int)iLen,0);
					if(!hMapObj)throw CAF_ERROR_WRITE;
						pMem=(char*)MapViewOfFile(hMapObj,FILE_MAP_WRITE,0,0,(int)iLen);
					}

				if(!pMem)								// Prfe den Speicherzeiger
					{
					CloseHandle(hMapObj);
					throw CAF_ERROR_WRITE;
					}

				iPosW = 0;

				for(iIdx=0;iIdx<m_iFilesNum;iIdx++)		// Dateien lschen
					{
					pEntry = m_pFiles[iIdx];

					if(pEntry->cWrite )continue;
					if(pEntry->cDelete)continue;

					PROG(CAF_PROG_WRITE,iIdx);

					iPosR  = pEntry->uOff;
					iSize  = 4+LOCHEAD;
					iSize += pEntry->uExt;
					iSize += pEntry->uLenComp;
					iSize += pEntry->uNameLen;

					if(pEntry->uFlags&FLGEXT)			// Extended Block vorhanden
						{
						iSize += 16;
						}

					iEnd   = m_iCenPos;

					if(iIdx+1<m_iFilesNum)				// Abstand zur nchsten Datei
						{
						pNext = m_pFiles[iIdx+1];

						if(!pNext->cWrite && !pNext->cDelete)
							{
							iEnd = pNext->uOff;
							}
						}
														// Endposition prfen
					if((iEnd && iPosR+iSize>iEnd) || iSize<=0)
						{
						if(hMapObj)						// Mapping aufheben
							{
							UnmapViewOfFile(pMem);		
							CloseHandle(hMapObj);		
							}

						throw CAF_ERROR_LOGIC;
						}

					if(iPosR!=iPosW)					// Muss die Datei verschoben werden
						{
						if(iPosW+iSize>iLen)
							{
							if(hMapObj)					// Mapping aufheben
								{
								UnmapViewOfFile(pMem);	
								CloseHandle(hMapObj);	
								}

							throw CAF_ERROR_LOGIC;
							}

						memmove(pMem+iPosW,pMem+iPosR,(int)iSize);
						}

					pEntry->uOff  = iPosW;
					iPosW        += iSize;
					m_iDoneNow	 += pEntry->uLenFull;
					}

				if(hMapObj)								// Mapping aufheben
					{
					UnmapViewOfFile(pMem);				
					CloseHandle(hMapObj);				
					}

				m_iCenPos  = iPosW;
				m_iChanged = 1;
				}
			else{
				unsigned char	aBuffer[0x4000];

				iPosW=0;

				for(iIdx=0;iIdx<m_iFilesNum;iIdx++)		// Dateien lschen
					{
					pEntry=m_pFiles[iIdx];

					if(pEntry->cWrite )continue;
					if(pEntry->cDelete)continue;

					PROG(CAF_PROG_WRITE,iIdx);

					iPosR  = pEntry->uOff;
					iPosR  = pEntry->uOff;
					iSize  = 4+LOCHEAD;
					iSize += pEntry->uExt;
					iSize += pEntry->uLenComp;
					iSize += pEntry->uNameLen;

					if(pEntry->uFlags&FLGEXT)			// Extended Block vorhanden
						{
						iSize += 16;
						}

					iEnd  = m_iCenPos;

					if(iIdx+1<m_iFilesNum)				// Abstand zur nchsten Datei
						{
							pNext = m_pFiles[iIdx+1];
						if(!pNext->cWrite && !pNext->cDelete)
							{
							iEnd = pNext->uOff;
							}
						}
														// Endposition prfen
					if((iEnd && iPosR+iSize>iEnd) || iSize<=0)
						{
						throw CAF_ERROR_LOGIC;
						}

					if(iPosR!=iPosW)					// Muss die Datei verschoben werden
						{
						if(iPosW+iSize>iLen)
							{
							throw CAF_ERROR_LOGIC;
							}							
														// Datei verschieben
						for(iOff=0;iOff<iSize;iOff+=iNum)
							{
							   iNum = iSize-iOff;
							if(iNum>sizeof(aBuffer))
								{
								iNum = sizeof(aBuffer);
								}

							MainSeek (iPosR+iOff,SEEK_SET);
							MainRead (aBuffer  ,(int)iNum);
							MainSeek (iPosW+iOff,SEEK_SET);
							MainWrite(aBuffer  ,(int)iNum);
							}
						}

					pEntry->uOff  = iPosW;
					iPosW        += iSize;
					m_iDoneNow	 += pEntry->uLenFull;
					}

				MainSeek(iPosW);
				EndOfFile();

				m_iCenPos  = iPosW;
				m_iChanged = 1;
				}
			}
		}
	catch(int iError)									// Bekannter Fehler
		{
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,-1);
		return iError;
		}


//******************** Neue Dateien speichern *********************************


	try	{
		MainSeek(m_iCenPos);

		m_pDecoder->Reset();
		m_iDoneMode = 3;

		for(iIdx=0;iIdx<m_iFilesNum;iIdx++)				// Dateien speichern
			{
			   pEntry = m_pFiles[iIdx];
			if(pEntry->cWrite && !pEntry->cDelete)
				{
				m_iChanged = 1;
				MainSeek(m_iCenPos);

				try	{
					Write(iIdx);						// Komprimiere Datei
					m_iDoneNow += pEntry->uLenFull;
					}
				catch(int iError)
					{
					m_iErrorCode = iError;

					if(iError==CAF_ERROR_BREAK)break;
					if(iError!=CAF_ERROR_FREAD)
					if(iError!=CAF_ERROR_FOPEN)throw iError;

					PROGN(CAF_PROG_ERROR,iIdx);

					pEntry->cDelete = 1;
					m_iFileCount   -= 1;
					}
				}
			}

		m_iDoneMode  = 0;
		}
	catch(int iError)									// Bekannter Fehler
		{
		m_iErrorCode = iError;
		m_iDoneMode  = 0;
		PROGN(CAF_PROG_ERROR,-1);
		Break();
		return iError;
		}


//******************** Blcke lschen *****************************************


	for(iIdx=m_iFilesNum-1;iIdx>=0;iIdx--)
		{
		   pEntry = m_pFiles[iIdx];
		if(pEntry->cDelete)ZipDel(iIdx);
		}

	if(m_iModeFlags&CAF_FLAG_SORT)						// Liste sortieren
		{
		Sort();			
		}


return m_iErrorCode=CAF_ERROR_OK;
}



//*****************************************************************************
//*
//*		ExtractProc
//*
//*****************************************************************************
//	Extrahiert eine Datei aus dem Archiv ber eine Ausgabefunktion
//	iNum	: Ist die Indexnummer der Datei die geprft werden soll
//	pParam	: Ist ein Parameter der an die Ausgabefunktionbergeben wird
//	pProc	: Ist der Zeiger auf die Ausgabefunktion
//	pInfo	: Hier werden Zeiten und Attribute der Datei gespeichert.
//			  Bei NULL werden diese Werte ignoriert.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//				CAF_ERROR_OK		= Wenn alles korrekt war
//				CAF_ERROR_MISS		= Wenn die Datei nicht gefunden wurde
//				CAF_ERROR_PATH		= Pfad ungltig ist.
//				CAF_ERROR_FCREATE	= Wenn Datei nicht erzeugt werden konnte
//				CAF_ERROR_CRC		= Crc32 Fehler
//				CAF_ERROR_DECOMP	= Fehler beim Entkomprimieren
//				CAF_ERROR_IGNORE	= Die Datei wurde bersprungen
//				... siehe Fehler Codes
int	CZipFile::ExtractProc(int iNum,void *pParam,OutProc pProc,FileData *pInfo)
{
ZipItem	   *pEntry;
int			iMode;
int			iErr;


	if(m_iDoneMode<=1)									// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}


	m_iErrorCode = CAF_ERROR_OK;

	try	{
		if((unsigned)iNum>=(unsigned)m_iFilesNum)		// Prfe den Dateiindex
			{
			m_iErrorCode = CAF_ERROR_MISS;
			PROGN(CAF_PROG_ERROR,-1);
			goto Exit; 
			}

			pEntry = m_pFiles[iNum];
		if(!pEntry || !pEntry->cSaved)					// Prfe den Index
			{
			m_iErrorCode = CAF_ERROR_MISS;
			PROGN(CAF_PROG_ERROR,-1);
			goto Exit; 
			}

		iMode = pEntry->uCompMode;

		#ifdef ZIP_USE_AES
		if(iMode==99)									// Ist Datei AES verschlsselt
			{
			iMode = pEntry->cAesDecom;
			}
		#endif

		switch(SelectDecoder(iMode,pEntry))				// Decoder auswhlen
			{
		case 1:	m_iErrorCode=CAF_ERROR_NOTSUPP;	PROGN(CAF_PROG_ERROR,iNum); goto Exit; 
		case 2:	m_iErrorCode=CAF_ERROR_MEM;		PROGN(CAF_PROG_ERROR,iNum); goto Exit; 
			}


		m_pDecoder->Reset();

		m_iDoneMode  = 1;								// Verlaufsanzeige einstellen
		m_iDoneNow   = 0;
		m_iDoneMax   = pEntry->uLenFull;
		m_pErrorFile = pEntry->pName;

		MainSeek(pEntry->uOff+4+LOCHEAD+pEntry->uNameLen+pEntry->uExt);

		try	{
			SetInput(m_pDecoder,CAF_MAIN,pEntry->uLenComp);
			if(pEntry->uFlags&FLGENC)SetEncrypter(pEntry);
			m_pDecoder->SetOut(pParam,pProc,CRC32_INIT);
			m_pDecoder->Decompress();
			}
		catch(int iEncError)							// Fehler beim Entschlsseln
			{
			m_iErrorCode = iEncError;
			PROGN(CAF_PROG_ERROR,iNum);
			goto Exit; 
			}
		catch(char *pText)
			{
			        iErr = m_pDecoder->GetError(); pText = 0;
			     if(iErr==CHF_ERROR_EOF	 )m_iErrorCode=CAF_ERROR_EOF;
			else if(iErr==CHF_ERROR_READ )m_iErrorCode=CAF_ERROR_READ;
			else if(iErr==CHF_ERROR_WRITE)m_iErrorCode=CAF_ERROR_WRITE;
			else 						  m_iErrorCode=CAF_ERROR_DECOMP;

			PROGN(CAF_PROG_ERROR,iNum);
			goto Exit; 
			}

		if(pEntry->uCrcSum!=m_pDecoder->GetCrcOut())	// CRC-Summe prfen
			{
			#ifdef ZIP_USE_AES
			if(pEntry->uCrcSum || !pEntry->cAesMode)
			#endif
				{
				m_iErrorCode=CAF_ERROR_DECOMP;
				PROGN(CAF_PROG_ERROR,iNum);
				goto Exit;
				}
			}

		if(pProc(pParam,NULL,0)!=0)						// Ende anzeigen
			{
			m_iErrorCode = CAF_ERROR_FWRITE;
			PROGN(CAF_PROG_ERROR,iNum);
			goto Exit;
			}

		if(pInfo)										// Dateidaten speichern
			{
			pInfo->dwAttr		=(pEntry->uAtx&A_RONLY)?SYS_WRITE|SYS_READ:SYS_READ;
			pInfo->dwAttr	   |=(pEntry->uLenFull>>(32-SYS_HIGHSHIFT))&SYS_HIGHMASK;
			pInfo->dwSize		=(unsigned)(pEntry->uLenFull  );
			pInfo->dwTimeWrite	=(unsigned)(pEntry->iTimeM!=-1)? pEntry->iTimeM:0;
			pInfo->dwTimeCreate	=(unsigned)(pEntry->iTimeC!=-1)? pEntry->iTimeC:0;
			pInfo->dwTimeAccess	=(unsigned)(pEntry->iTimeA!=-1)? pEntry->iTimeA:0;
			pInfo->cName[0]		= 0;
			}

		PROG(CAF_PROG_EXTRACT,iNum);
		}
	catch(int iError)									// Gab es Fehler
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}

Exit:

	if(m_iDoneMode<=1)									// Die Verlaufsanzeige zurcksetzen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}



return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Test
//*
//*****************************************************************************
//	Teste eine komprimierte Datei.
//	Ergibt 0 wenn ok, oder eine Fehlernummer ungleich 0:
//			CAF_ERROR_OK		: Wenn alles korrekt war
//			CAF_ERROR_MISS		: Wenn die Datei nicht gefunden wurde
//			CAF_ERROR_CRC		: Crc32 Fehler
//			CAF_ERROR_DECOMP	: Fehler beim Entkomprimieren
//			CAF_ERROR_EOF		: Bei vorzeitigem Leseende
//		... siehe Fehler Codes
int	CZipFile::Test(int iNum)
{
ZipItem    *pEntry=0;
int			iMode;
int			iErr;


	if(m_iDoneMode<=1)									// Verlaufsanzeige starten
		{
		m_iDoneMode	= 1;
		m_iDoneNow	= 0;
		m_iDoneMax	= 0;
		}

	m_iErrorCode = CAF_ERROR_OK;


	try	{
		PROG(CAF_PROG_TEST,iNum);

		if((unsigned)iNum>=(unsigned)m_iFilesNum)		// Prfe den Dateiindex
			{
			m_iErrorCode = CAF_ERROR_MISS;
			PROGN(CAF_PROG_ERROR,-1);
			goto Exit;
			}

			pEntry = m_pFiles[iNum];
		if(!pEntry || !pEntry->cSaved)
			{
			m_iErrorCode = CAF_ERROR_MISS;
			PROGN(CAF_PROG_ERROR,-1);
			goto Exit;
			}

		if(!pEntry->uLenFull)
			{
			return CAF_ERROR_OK;
			}

		iMode = pEntry->uCompMode;

		#ifdef ZIP_USE_AES
		if(iMode==99)									// Ist Datei AES verschlsselt
			{
			iMode = pEntry->cAesDecom;
			}
		#endif

		switch(SelectDecoder(iMode,pEntry))				// Decoder auswhlen
			{
		case 1:	m_iErrorCode=CAF_ERROR_NOTSUPP;	PROGN(CAF_PROG_ERROR,iNum);	goto Exit;
		case 2:	m_iErrorCode=CAF_ERROR_MEM;		PROGN(CAF_PROG_ERROR,iNum);	goto Exit;
			}

		m_pDecoder->Reset();

		m_iDoneMode  =  2;								// Verlaufsanzeige einstellen
		m_iDoneNow   =  0;
		m_iDoneMax   =  pEntry->uLenFull;
		m_pErrorFile =  pEntry->pName;

		MainSeek(pEntry->uOff+4+LOCHEAD+pEntry->uNameLen+pEntry->uExt);

		try	{
			SetInput(m_pDecoder,CAF_MAIN,pEntry->uLenComp);
			if(pEntry->uFlags&FLGENC)SetEncrypter(pEntry);
			m_pDecoder->SetOut(this,NoneWrite,CRC32_INIT);
			m_pDecoder->Decompress();
			}
		catch(int iEncError)							// Fehler beim Entschlsseln
			{
			pEntry->cError=1;
			throw iEncError;
			}
		catch(char *pText)
			{
			pEntry->cError = 1;
			
			        iErr = m_pDecoder->GetError(); pText = 0;
			     if(iErr==CHF_ERROR_EOF )m_iErrorCode=CAF_ERROR_EOF;
			else if(iErr==CHF_ERROR_READ)m_iErrorCode=CAF_ERROR_FREAD;
			else						 m_iErrorCode=CAF_ERROR_DECOMP;
			
			PROGN(CAF_PROG_ERROR,-1);

			goto Exit;
			}

		if(pEntry->uCrcSum!=m_pDecoder->GetCrcOut())	// CRC-Summe prfen
			{
			#ifdef ZIP_USE_AES
			if(pEntry->uCrcSum || !pEntry->cAesMode)
			#endif
				{
				pEntry->cError=1;
				m_iErrorCode = CAF_ERROR_CRC;
				PROGN(CAF_PROG_ERROR,-1);
				goto Exit;
				}
			}
		}
	catch(int iError)
		{
		if(pEntry)pEntry->cError=1;
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,iNum);
		}

Exit:

	if(m_iDoneMode<=1)									// Die Verlaufsanzeige zurcksetzen
		{
		m_iDoneMode = 0;
		m_iDoneNow  = m_iDoneMax;
		}



return m_iErrorCode;
}


//*****************************************************************************
//*
//*		AddPath
//*
//*****************************************************************************
//	Fgt dem Zip-Archiv einen Pfad hinzu. Der Pfad muss dabei nicht real existieren.
//	Datei ist der Dateiname fr den Pfad
//	ber die Flags() kann mann die Speicherungsart festlegen
//	Falgs:	CAF_FLAG_FULLPATH	0x0880	 Ganze Namen mit  Laufwerken
//	       	CAF_FLAG_FULLNAME	0x0800	 Ganze Namen ohne Laufwerken
//	Ergibt wenn die Anzahl der angehngten Dateien bzw. -1 bei einem Fehler.
int CZipFile::AddPath(const zchar *pName)
{
ZipItem		   *pEntry;
zchar		   *pTemp;
time_t			iTime;
int				iPos;
int				iNum;
int				iLen;



	if(!pName || !*pName)								// Prf den Pfadnamen
		{
		m_iErrorCode=CAF_ERROR_NAME;
		PROGR(CAF_PROG_ERROR,-1,-1);
		return CAF_ERROR_NAME;
		}

	m_iErrorCode = CAF_ERROR_OK;

	try	{
		if(m_iOpenMode<CAF_WRITE)throw CAF_ERROR_WRITE;

		pTemp=(zchar*)pName;

		if(!(m_iModeFlags&0x0080))						// Keine Laufwerksbuschstaben speichern
			{
			iPos=strcwlen(pTemp,0x00700000);
			if(pTemp[iPos]==':')pTemp+=iPos+1;
			}

		if(*pTemp=='\\')pTemp++;
		if(*pTemp=='/' )pTemp++;


		if(m_iModeFlags&CAF_FLAG_NOFIND)				// Nicht prfen ob die Datei im Archiv ist
			{
			iNum = -1;
			}
		else{
			   iNum=Find(pTemp);
			if(iNum>=0) 
				{
				if(m_iModeFlags&CAF_FLAG_NOEXIST)		// Keine Dateien im Archiv ersetzen
					{
					return CAF_ERROR_OK;
					}
				
				ZipSetDel(iNum);
				}
			}


		iTime				=  time(0);
		iNum				=  ZipNew();				// Neuen Eintrag erzeugen
		iLen				=  strzlen(pTemp);				

		pEntry				=  m_pFiles[iNum];
		pEntry->uAtt		=  BINARY;
		pEntry->uAtx		=  A_DIR;
		pEntry->uVerExrt	=  10;
		pEntry->uVerCreate  =  VERSION;
		pEntry->uCompMode   =  CAF_STORE;
		pEntry->uFlags		=  FLGEXT;
		pEntry->iTimeM		=  iTime;
		pEntry->iTimeA		=  iTime;
		pEntry->iTimeC		=  iTime;
		pEntry->uDosTime	=  UtcToDosTime(iTime);
		pEntry->uNameLen    =  iLen;
		pEntry->pName		=  new zchar[iLen+1];
		pEntry->cWrite 		=  1;
														// Den Name kopieren	
		memcpy    (pEntry->pName,pTemp,(iLen+1)*sizeof(zchar));
		CharToPath(pEntry->pName);
		
		m_iSortMode 	  	=  0;
		m_iChanged			=  1;

		if(m_iModeFlags&CAF_FLAG_SORT)					// Sortieren ?
			{
			Sort();
			}

		PROGR(CAF_PROG_ZADD,iNum,1);
		}
	catch(int iError)
		{
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,-1);
		return iError;
		}


return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		AddProc
//*
//*****************************************************************************
//	Fgt dem TAR-Datei eine Datei hinzu, die Daten wird ber ein Callback 
//	geladen. ber die Flags() kann mann die Speicherungsart der Namens 
//	im Archiv festlegen:
//					CAF_FLAG_FULLPATH	0x0880	 Ganze Namen mit  Laufwerken
//	       			CAF_FLAG_FULLNAME	0x0800	 Ganze Namen ohne Laufwerken
//	Achtung es kann passieren das sich Dateien komprimiert grer sind,
//	dann kommet der Fehler CAF_ERROR_COMP. Dann muss der Modus vorher 
//	mit Level(CAF_STORE) umgestellt werden.
//	pName		: Ist der Dateiname fr die Datei
//	pParam		: Ist der Zeiger der an die Funktion bergeben wird.
//	pProc		: Ist der Zeiger auf die Callback-Funktion.
//					Mit  0 signalisiert der Callback das Dateiende 
//					Mit -1 signalisiert der Callback einen Fehler
//	pInfo		: Enthlt Zeiten und Attribute der Datei.
//				  Bei NULL werden diese Werte ignoriert.
//				  Mit SYS_VOLUME wird die Komprimierung abgeschalten.
//	Ergibt CAF_ERROR_OK wenn alles ok war oder einen Fehlercode.
int CZipFile::AddProc(const zchar *pName,void *pParam,InpProc pProc,FileData *pInfo)
{
time_t			iTimeW;
time_t			iTimeA;
time_t			iTimeC;
ZipItem		   *pEntry;
zchar		   *pTemp;
unsigned 		uLevel;
unsigned		uAttr;
auint64			uOffD;
auint64			uOffE;
aint64			iPosZ;
aint64			iSize;
int				iMode;
int				iNum;
int				iPos;
int				iLen;
int				iAdd;


	if(pInfo && (pInfo->dwAttr&SYS_DIR))				// Ein Verzeichniss einfgen
		{
		return AddPath(pName);
		}

	m_iErrorCode = CAF_ERROR_OK;

	try	{
		if(m_iOpenMode<CAF_WRITE)throw CAF_ERROR_WRITE;

		pTemp	= (zchar*)pName;
		iMode   = m_iCompressMode;
		uLevel  = m_iLevel;
								 
		if(!(m_iModeFlags&0x0080))						// Keine Laufwerksbuschstaben speichern
			{
			iPos=strcwlen(pTemp,0x00700000);
			if(pTemp[iPos]==':')pTemp+=iPos+1;
			}

		if(*pTemp=='\\')pTemp++;
		if(*pTemp=='/' )pTemp++;


		if(m_iModeFlags&CAF_FLAG_NOFIND)				// Nicht prfen ob die Datei im Archiv ist
			{
			iNum = -1;
			}
		else{
			   iNum=Find(pTemp);
			if(iNum>=0)
				{
				if(m_iModeFlags&CAF_FLAG_NOEXIST)		// Keine Dateien im Archiv ersetzen
					{
					return CAF_ERROR_OK;
					}
				
				ZipSetDel(iNum);
				}
			}


		if(ArchiveType(pTemp))							// Gezippte Dateien nicht komprimieren
			{
			iMode = CAF_STORE;
			}


		if(pInfo)										// Gibt es Infos zur neuen Datei
			{
			if(pInfo->dwTimeWrite!=0  && pInfo->dwTimeWrite !=0xFFFFFFFF)
					iTimeW = pInfo->dwTimeWrite;
			else	iTimeW = time(0);		

			if(pInfo->dwTimeCreate!=0 && pInfo->dwTimeCreate!=0xFFFFFFFF)
					iTimeC = pInfo->dwTimeCreate;
			else	iTimeC = iTimeW;

			if(pInfo->dwTimeAccess!=0 && pInfo->dwTimeAccess!=0xFFFFFFFF)
					iTimeA = pInfo->dwTimeAccess;
			else	iTimeA = iTimeW;

			uAttr = GetFileMode(pInfo->dwAttr);

			if(pInfo->dwAttr&SYS_VOLUME)				// Datei nicht komprimieren
				{
				iMode = CAF_STORE;
				}
			
			iSize	= (pInfo->dwAttr&SYS_HIGHMASK)>>SYS_HIGHSHIFT;
			iSize <<= 32;
			iSize  += pInfo->dwSize;

			if(pProc==CAF_ADD_FILE)						// Die Datei erst mit Flash() speichern
					iAdd = strzlen(pInfo->cName)+1;			
			else	iAdd = 0;
			}
		else{
			iTimeW	= time(0);							// Aktuelle Zeit benutzen
			iTimeC	= iTimeW;
			iTimeA	= iTimeW;
			uAttr	= A_NORMAL;
			iSize	= 1;
			iAdd    = 0;
			}
		

		iNum				=  ZipNew();				// Neuen Eintrag erzeugen
		iLen				=  strzlen(pTemp);			
		pEntry				=  m_pFiles[iNum];
		pEntry->uAtt		=  BINARY;
		pEntry->uAtx		=  uAttr;
		pEntry->uDosTime	=  UtcToDosTime(iTimeW);
		pEntry->iTimeM		=  iTimeW;
		pEntry->iTimeA  	=  iTimeA;
		pEntry->iTimeC  	=  iTimeC;
		pEntry->uLenFull    =  iSize;
		pEntry->uNameLen    =  iLen;
		pEntry->pName   	=  new zchar[iLen+1+iAdd];
														// Name kopieren
		memcpy    (pEntry->pName,pTemp,(iLen+1)*sizeof(zchar));
		CharToPath(pEntry->pName);

		PROGN(CAF_PROG_WRITE,iNum);

		if(iAdd)										// Die Datei erst mit Flash() speichern
			{
			pEntry->cWrite  = 1;
			pEntry->pPath	= pEntry->pName+iLen+1;
			memcpy(pEntry->pPath,pInfo->cName,iAdd*sizeof(zchar));

			return CAF_ERROR_OK;
			}


//******************** Daten speichern ****************************************


		iPosZ = MainSeek(m_iCenPos);

		if(iSize==0	       ){iMode=CAF_STORE;           }
		if(iMode==CAF_BEST ){iMode=CAF_DEFLATE;uLevel=9;}
		if(iMode==CAF_STORE){                  uLevel=0;}

		pEntry->uVerCreate  =  VERSION;
		pEntry->uVerExrt   	= (iMode==CAF_STORE)? 10:20;
		pEntry->uFlags   	=  FLGEXT;
		pEntry->uLenComp   	= (iMode==CAF_STORE && iSize>=0)? iSize:0;
		pEntry->uLenFull   	= (iSize>0)? iSize:0;
		pEntry->uAtx   		=  uAttr|(pEntry->uAtx&0x0000FF00);
		pEntry->uOff   		=  iPosZ;
		pEntry->cWrite 		=  1;						
		pEntry->cDelete   	=  1;						// Zum Lschen markieren falls eine Exeption kommt	

		m_iChanged=1;
		PutLOC(pEntry);
		uOffD=MainTell();

		try	{											// Die Daten komprimieren
			SetOutput(m_pDecoder,CAF_MAIN);
			m_pDecoder->SetIn(pParam,pProc,CRC32_INIT);
			m_pDecoder->Compress(uLevel);
			}
		catch(char *pText)								// Fehler beim Komprimieren
			{
			int	iErrNum=m_pDecoder->GetError();pText=0;
			if( iErrNum==CHF_ERROR_READ )throw CAF_ERROR_READ;
			if( iErrNum==CHF_ERROR_WRITE)throw CAF_ERROR_WRITE;
										 throw CAF_ERROR_LOGIC;
			}		 

		if(m_pDecoder->GetFlags()&CHF_NOCOMPESS)		// Wenn die komprimierten Daten grer sind
			{
			throw CAF_ERROR_COMP;
			}

		iMode              = (m_pDecoder->GetMode()==CHF_STORE)? CAF_STORE:CAF_DEFLATE;
		uOffE              =  MainTell();
		pEntry->uCrcSum    =  m_pDecoder->GetCrcInp();
		pEntry->uLenFull   =  m_pDecoder->GetLength();
		pEntry->uLenComp   =  m_pDecoder->GetSize  ();
		pEntry->uCompMode  =  iMode;
		pEntry->uVerExrt   = (iMode==CAF_STORE)? 10:20;	// Need PKUNZIP 2.0 except for store
		pEntry->uFlags    &= ~FLGEXT;					// Kein Extended Header
		pEntry->cWrite     =  0;
		pEntry->cDelete    =  0;
		pEntry->cSaved     =  1;


		MainSeek(pEntry->uOff);
		PutLOC  (pEntry);
		MainSeek(uOffE);

		m_iCenPos          = MainTell();
		m_iSortMode 	   = 0;

		if(m_iModeFlags&CAF_FLAG_SORT)					// Sortieren ?
			{
			Sort();
			}
		
		PROGR(CAF_PROG_ZADD,iNum,1);
		}
	catch(int iError)
		{
		m_iErrorCode=iError;
		PROGN(CAF_PROG_ERROR,-1);
		return iError;
		}

	


return CAF_ERROR_OK;
}

//*****************************************************************************
//*
//*		ScanExHeader
//*
//*****************************************************************************
// 	Diese Funktion sucht nach UNIX Zeitangaben im Extrafeld	und 64-Bit-Gren.
// 	pExtra	  : Puffer auf Extrafeld
// 	uExtraLen : Gesamtlnge des Extrafeldes
//	Setzt folgende Flags wenn eine Zeitangabe gefunden wurde
//		EB_UT_FL_MTIME	Zeit der letzten nderung
// 		EB_UT_FL_ATIME	Zeit des letzten Zugriffs
// 		EB_UT_FL_CTIME	Zeit der Dateierstellung
//		EB_64_SIZE		64-Bit Gren
int CZipFile::ScanExHeader(ZipItem *pEntry,unsigned char *pExtra,unsigned uExtraLen)
{
int			iHaveNewType = FALSE;
int			iFlags=0;
FILETIME	sFtTime;
unsigned	uEbLen;
unsigned	uEbId;
unsigned	uType;
unsigned	uPos;
unsigned	uLen;
unsigned	uOff;




	if(uExtraLen==0 || pExtra==NULL)return 0;

	while(uExtraLen>=EB_HEADSIZE)
		{
		uEbId  = pExtra[EB_ID ]+pExtra[EB_ID +1]*256;
		uEbLen = pExtra[EB_LEN]+pExtra[EB_LEN+1]*256;

		if(uEbLen+EB_HEADSIZE>uExtraLen)break;


		switch(uEbId)
			{
		case EF_SIZE64:									// 64-Bit-Gren

			uOff = 0;

			if(uEbLen>=uOff+8 && pEntry->uLenFull==0xFFFFFFFF)
				{
				pEntry->uLenFull = GetL8(pExtra+EB_HEADSIZE+uOff);
				iFlags |= EB_64_SIZE;
				uOff   += 8;
				}

			if(uEbLen>=uOff+8 && pEntry->uLenComp==0xFFFFFFFF)
				{
				pEntry->uLenComp = GetL8(pExtra+EB_HEADSIZE+uOff);
				iFlags |= EB_64_COMP;
				uOff   += 8;
				}

			if(uEbLen>=uOff+8 && pEntry->uOff==0xFFFFFFFF)
				{
				pEntry->uOff     = GetL8(pExtra+EB_HEADSIZE+uOff);
				iFlags |= EB_64_POS;
				uOff   += 8;
				}

			break;

		case EF_NTFS:									// NTFS-Atttribute

			for(uOff=8;;uOff+=uLen)
				{
				uType = pExtra[uOff+0]+pExtra[uOff+1]*256;
				uLen  = pExtra[uOff+2]+pExtra[uOff+3]*256;

				if(uOff+uLen>uEbLen)break;
				if(uType!=1)continue;

				*(aint64*)&sFtTime = GetL8(pExtra+uOff+ 4);	FileTimeToUtcTime(&sFtTime,&pEntry->iTimeM);
				*(aint64*)&sFtTime = GetL8(pExtra+uOff+12);	FileTimeToUtcTime(&sFtTime,&pEntry->iTimeA);
				*(aint64*)&sFtTime = GetL8(pExtra+uOff+20);	FileTimeToUtcTime(&sFtTime,&pEntry->iTimeC);

				iFlags |= EB_UT_FL_MTIME|EB_UT_FL_ATIME|EB_UT_FL_CTIME;
				}

			break;

		case EF_TIME:

			iFlags      &= ~0x00ff;
			iHaveNewType =  TRUE;

			if(uEbLen>=EB_UT_MINLEN)
				{
				uPos    = 1;
				iFlags |= pExtra[EB_HEADSIZE];

				if(iFlags&EB_UT_FL_MTIME)
					{
					if(uPos+4<=uEbLen) 					// Zeit der letzten nderung
						{
						pEntry->iTimeM = GetL4(pExtra+EB_HEADSIZE+uPos);
						uPos+=4;
						}
					else{
						iFlags &= ~EB_UT_FL_MTIME;
						}
					}

				if(iFlags&EB_UT_FL_ATIME)				// Zeit des letzten Zugriffes
					{
					if(uPos+4 <= uEbLen)
						{
						pEntry->iTimeA = GetL4(pExtra+EB_HEADSIZE+uPos);
						uPos += 4;
						}
					else{
						iFlags &= ~EB_UT_FL_ATIME;
						}
					}

				if(iFlags&EB_UT_FL_CTIME)				// Zeit der Dateierzeugung
					{
					if(uPos+4 <= uEbLen)
						{
						pEntry->iTimeC = GetL4(pExtra+EB_HEADSIZE+uPos);
						uPos += 4;
						}
					else{
						iFlags &= ~EB_UT_FL_CTIME;
						}
					}
				}

			break;

		case EF_IZUNIX2:

			if(!iHaveNewType)
				{
				iFlags      &= ~0x00ff;					// Ignore any previous IZUNIX field
				iHaveNewType = TRUE;
				}

			break;

		case EF_IZUNIX:

			if(uEbLen >= EB_UX_MINLEN)
				{
				if(iHaveNewType)break;					// Ignore IZUNIX extra field block

				pEntry->iTimeA = GetL4(pExtra+EB_HEADSIZE+EB_UX_ATIME);
				pEntry->iTimeM = GetL4(pExtra+EB_HEADSIZE+EB_UX_MTIME);

				iFlags |=  EB_UT_FL_MTIME|EB_UT_FL_ATIME;
				}

			break;

		case EF_AESCRYPT:

			if(uExtraLen<7)break;
			if(pExtra[4]<1     || pExtra[4]>2     || pExtra[5]!=0)break;
			if(pExtra[6]!=0x41 || pExtra[7]!=0x45 || pExtra[10]  )break;
			if(pExtra[8]<1     || pExtra[4]>3                    )break;

			pEntry->cAesDecom = pExtra[9];				// Koprimierungsmethode bei AES
			pEntry->cAesMode  =(char)(pExtra[8]|(pExtra[4]<<2));
			break;
			}


		pExtra    += uEbLen+EB_HEADSIZE;				// Nchster Block
		uExtraLen -= uEbLen+EB_HEADSIZE;
		}


return iFlags;
}


//*****************************************************************************
//*
//*		SetExHeader
//*
//*****************************************************************************
//	Speicher die UNIX-Zeitangaben und 64-Bit-Gren im Extrafeld.
//	Ergibt 1 wenn ein neues Extrafeld hinzugefgt wurde.
int CZipFile::SetExHeader(ZipItem *pEntry)
{
unsigned char  *pBuffer;
unsigned char  *pStart;
FILETIME		sTime;
int				iSize;
int				iMode;
int				iOff;



	iMode = 0;

	if(pEntry->uLenFull>=0xFFFFFFFF)iMode++;
	if(pEntry->uOff    >=0xFFFFFFFF)iMode++;


	if(pEntry->pExtra || pEntry->pCentral)				// Gibt es die Header schon
		{
		if(!iMode)return 0;

		if(pEntry->pExtra  ){delete pEntry->pExtra  ;pEntry->pExtra  =0;}
		if(pEntry->pCentral){delete pEntry->pCentral;pEntry->pCentral=0;}
		}


	iSize = EB_HEADSIZE+1+4*3 + EB_HEADSIZE+8*4;


	if(iMode)											// 64-Bit Gren anhngen
		{
		if(pEntry->uLenComp>=0xFFFFFFFF)iMode++;

		iOff       =  EB_HEADSIZE;
		iSize     +=  EB_HEADSIZE+iMode*8;
		pStart	   =  new unsigned char[iSize];
		pBuffer    =  pStart;
		pBuffer[0] =  EF_SIZE64     &0xFF;
		pBuffer[1] = (EF_SIZE64>>8) &0xFF;
		pBuffer[2] =  iMode*8;
		pBuffer[3] =  0;

		if(pEntry->uLenFull>=0xFFFFFFFF){SetL8(pBuffer+iOff,pEntry->uLenFull);iOff+=8;}
		if(pEntry->uLenComp>=0xFFFFFFFF){SetL8(pBuffer+iOff,pEntry->uLenComp);iOff+=8;}
		if(pEntry->uOff    >=0xFFFFFFFF){SetL8(pBuffer+iOff,pEntry->uOff    );iOff+=8;}

		pBuffer          += EB_HEADSIZE+iMode*8;
		pEntry->uVerExrt  = 45;

		if(pEntry->uOff>=0xFFFFFFFF)iMode--;
		}
	else{
		pStart	= new unsigned char[iSize];
		pBuffer = pStart;

		if(pEntry->uVerExrt>20 && pEntry->uCompMode<=8)
			{
			pEntry->uVerExrt = 20;
			}
		}

	pBuffer[0]  =  EF_TIME     &0xFF;					// UNIX-Zeiten speichern
	pBuffer[1]  = (EF_TIME>>8) &0xFF;
	pBuffer[2]  =  1+4*3;
	pBuffer[3]  =  0;
	pBuffer[4]  =  EB_UT_FL_ATIME|EB_UT_FL_CTIME|EB_UT_FL_MTIME;
	pBuffer    +=  EB_HEADSIZE+1;

	*(int*)(pBuffer+4*0) = (int)pEntry->iTimeM;
 	*(int*)(pBuffer+4*1) = (int)pEntry->iTimeA;
 	*(int*)(pBuffer+4*2) = (int)pEntry->iTimeC;

	pBuffer    +=  4*3;									// NTFS-Zeiten speichern
	pBuffer[ 0] =  EF_NTFS     &0xFF;
	pBuffer[ 1] = (EF_NTFS>>8) &0xFF;
	pBuffer[ 2] =  8*4;
	pBuffer[ 3] =  0;
	pBuffer[ 4] =  0;
	pBuffer[ 5] =  0;
	pBuffer[ 6] =  0;
	pBuffer[ 7] =  0;
	pBuffer[ 8] =  1;
	pBuffer[ 9] =  0;
	pBuffer[10] =  24;
	pBuffer[11] =  0;

	UtcTimeToFileTime(pEntry->iTimeM,&sTime);	SetL8(pBuffer+12+8*0,*(aint64*)&sTime);
	UtcTimeToFileTime(pEntry->iTimeA,&sTime);	SetL8(pBuffer+12+8*1,*(aint64*)&sTime);
	UtcTimeToFileTime(pEntry->iTimeC,&sTime);	SetL8(pBuffer+12+8*2,*(aint64*)&sTime);

	pEntry->pCentral  = pStart;
	pEntry->uCenLen   = iSize;
													
	if(iMode)											// 64-Bit-Gre fr LOC-Header
		{
		pEntry->pExtra    = new uchar[EB_HEADSIZE+iMode*8];
		pEntry->uExt      =           EB_HEADSIZE+iMode*8;
		memcpy(pEntry->pExtra,pStart ,EB_HEADSIZE+iMode*8);
		pEntry->pExtra[2] = iMode*8;
		}


return 1;
}


//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CZipFile::Done(aint64 *pMax)
{
aint64	iRet;


	switch(m_iDoneMode)
		{
	case 0:	iRet = m_iDoneNow;							break;
	case 1:	iRet =            m_pDecoder->GetPosOut();	break;	// Einzelne Datei Extrahieren
	case 2:	iRet = m_iDoneNow+m_pDecoder->GetPosOut();	break;	// Alles extrahieren
	case 3:	iRet = m_iDoneNow+m_pDecoder->GetPosOut();	break;	// Flush+Komprimieren
	default:iRet = 0;									break;
		}

	if(iRet>m_iDoneMax)iRet=m_iDoneMax;
	if(pMax)*pMax=m_iDoneMax;


return iRet;
}

//*****************************************************************************
//*
//*		Change
//*
//*****************************************************************************
//	Schreiberlaubnis de/aktivieren
int CZipFile::Change(int iMode)
{

														// Prfe den Modus
	if(iMode<CAF_READ || iMode>CAF_WRITE || m_iOpenMode<=CAF_CLOSED)				
		{
		return CAF_ERROR_MODE;
		}
	
	if(m_iOpenMode==iMode)								// Gleicher Modus
		{
		return CAF_ERROR_OK;
		}

	if(m_iChanged==1 && iMode==CAF_READ)				// Gibt es nderungen
		{
		Flush ();
		PutEND();
		m_iChanged = 0;
		}
	
	m_iDoneMode	 = 0;
	m_iDoneNow	 = 0;
	m_iDoneMax	 = 0;


return DoChange(iMode);
}

//*****************************************************************************
//*
//*		GetUserPtr
//*
//*****************************************************************************
//	User-Pointer abfragen
void *CZipFile::GetUserPtr(int iNum)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return 0;

return m_pFiles[iNum]->pUserPtr;
}

//*****************************************************************************
//*
//*		SetUserPtr
//*
//*****************************************************************************
//	User-Pointer einstellen
void CZipFile::SetUserPtr(int iNum,void *pData)
{

	if((unsigned)iNum>=(unsigned)m_iFilesNum)return;

	m_pFiles[iNum]->pUserPtr = pData;

}

//*****************************************************************************
//*
//*		DecryptByte
//*
//*****************************************************************************
//	Funktion fr alte Verschlsselung
inline unsigned char CZipFile::DecryptByte()
{
unsigned	uTemp;

    uTemp = (m_aKeys[2]&0xFFFF)|2;

return (uTemp*(uTemp^1))>>8;
}


//*****************************************************************************
//*
//*		UpdateKeys
//*
//*****************************************************************************
//	Funktion fr alte Verschlsselung
inline unsigned char CZipFile::UpdateKeys(unsigned char ucByte)
{
unsigned  char	ucTemp;


    m_aKeys[0]  = aCrc32Table[(m_aKeys[0]^ucByte)&0xFF] ^ (m_aKeys[0]>>8);
    m_aKeys[1] += m_aKeys[0]&0xFF;
    m_aKeys[1]  = m_aKeys[1]*134775813 + 1;
    ucTemp		= m_aKeys[1]>>24;
    m_aKeys[2]  = aCrc32Table[(m_aKeys[2]^ucTemp)&0xFF] ^ (m_aKeys[2]>>8);


return ucByte;
}

//*****************************************************************************
//*
//*		EncryptOld
//*
//*****************************************************************************
//	Filterfunktion fr alte Verschlsselung
int __cdecl CZipFile::EncryptOld(void *pParam,char *pMem,int iLen)
{
unsigned char 	*pData=(unsigned char*)pMem;
CZipFile		*pThis=(CZipFile*)pParam;
int				 iPos;



	for(iPos=0;iPos<iLen;iPos++)
		{
		pData[iPos] ^= pThis->DecryptByte();
		pThis->UpdateKeys(pData[iPos]);
		}


return iLen;
}

//*****************************************************************************
//*
//*		EncryptAes
//*
//*****************************************************************************
//	Filterfunktion fr AES Verschlsselung
#ifdef	ZIP_USE_AES
int __cdecl CZipFile::EncryptAes(void *pParam,char *pMem,int iSize)
{
unsigned char 	*pData=(unsigned char*)pMem;
ZipCrypt		*pCrypt=(ZipCrypt*)pParam;
unsigned char	 aBuffer[10];
aint64			 iMax;
int				 iLen;
int				 iNum;
int				 iOff;


	iMax = pCrypt->input_max-pCrypt->input_now;
	iLen = iSize;


	if(iLen<=iMax)										// Ende nicht erreicht
		{
		ZipDecrypt(pCrypt,(unsigned char*)pMem,iSize);
		pCrypt->input_now += iSize;
		return iSize;
		}

	if(iMax>0)											// Noch Bytes zum decodieren
		{
		iNum = (int)iMax;

		if(iNum>0)
			{
			ZipDecrypt(pCrypt,(unsigned char*)pMem,iNum);
			pCrypt->input_now	+= iNum;
			pMem				+= iNum;
			iLen				-= iNum;
			}

		iMax = 0;
		}
	else{
		iNum = 0;
		}

	iOff = -(int)iMax;

	if(iLen>0)											// Suffix fom Stream kopieren
		{
		if(iOff+iLen>11)
			{
			iLen = 11-iOff;
			if(iLen<0)iLen=0;
			}

		memcpy(pCrypt->suffix+iOff,pMem,iLen);
		pCrypt->input_now	+= iLen;
		iSize				-= iLen;

		if(pCrypt->input_now>=pCrypt->input_max+11)		// Alles eingelesen
			{
			ZipDecrypt(pCrypt,pCrypt->suffix,1);
			pMem[iSize-iNum]  = pCrypt->suffix[0];
			iSize++;

			if(ZipCryptExit(pCrypt,aBuffer)!=10)
				{
				return -1;
				}

			if(memcmp(pCrypt->suffix+1,aBuffer,10))
				{
				return -1;
				}
			}
		}


return iSize;
}
#endif

//*****************************************************************************
//*
//*		SetEncrypter
//*
//*****************************************************************************
//	Bei einer verschlsselte Datei die Entkodierung starten
void CZipFile::SetEncrypter(ZipItem *pEntry)
{
unsigned char	aData[20];
unsigned		uVal1;
unsigned		uVal2;
int				iPos;


	#ifdef ZIP_USE_AES

	if(pEntry->uCompMode==99)								// Ist die Datei AES verschlsselt
		{
		ZipCrypt   *pCrypt=(ZipCrypt*)m_pCrypt;
		unsigned	uHdrLen;

		if(!pCrypt)
			{
			pCrypt		= new ZipCrypt;
			m_pCrypt	= pCrypt;
			}

		uHdrLen = (pEntry->cAesMode&3)*4+4+2;

		if(m_pDecoder->Read(aData,uHdrLen)!=(int)uHdrLen)	// AES-Salt-Wert und Passwortprfer einlesen
			{
			throw CAF_ERROR_READ;
			}

		if(!m_pPassword)
			{
			throw CAF_ERROR_PASSWD;
			}

		if(pEntry->uLenComp<uHdrLen+10)						// Prfe die Lnge der komprimierten Daten
			{
			throw CAF_ERROR_FORM;
			}

		if(ZipCryptInit(pCrypt,pEntry->cAesMode&3,(unsigned char*)m_pPassword,(int)strlen(m_pPassword),aData,aData+uHdrLen)!=GOOD_RETURN)
			{
			throw CAF_ERROR_FORM;
			}

		if(memcmp(aData+uHdrLen-2,aData+uHdrLen,2))			// Passwort prfen
			{
			throw CAF_ERROR_PASSWD;
			}

		pCrypt->input_max = pEntry->uLenComp-11;			// maximum input length minus one
		pCrypt->input_now = uHdrLen;						// current input length

		m_pDecoder->SetInFilter(pCrypt,EncryptAes);			// Filter einstellen

		return;
		}
	else

	#endif
		{													// Alte Zip-Verschlsselung
		if(m_pDecoder->Read(aData,12)!=12)
			{
			throw CAF_ERROR_READ;
			}

		if(!m_pPassword)
			{
			throw CAF_ERROR_PASSWD;
			}

		m_aKeys[0] = 305419896L;
		m_aKeys[1] = 591751049L;
		m_aKeys[2] = 878082192L;

		for(iPos=0;m_pPassword[iPos];iPos++)
			{
			UpdateKeys(m_pPassword[iPos]);
			}

		for(iPos=0;iPos<12;iPos++)							// Passwort prfen
			{
			aData[iPos] ^= DecryptByte();
			UpdateKeys(aData[iPos]);
			}

		uVal1	= aData[10]|(aData[11]<<8);
		uVal2	= (pEntry->uFlags&8)? pEntry->uDosTime:pEntry->uCrcSum>>16;
		uVal2   &= 0xFFFF;

		if(uVal1!=uVal2)
			{
			/*
			if(pEntry->uVerCreate>=10)						// Vor ZIP-Version 10
				{
				throw CAF_ERROR_PASSWD;
				}
			*/
			uVal1	= aData[11];
			uVal2	= (pEntry->uFlags&8)? pEntry->uDosTime>>9:pEntry->uCrcSum>>24;
			uVal2  &= 0xFF;

			if(uVal1!=uVal2)
				{
				throw CAF_ERROR_PASSWD;
				}
			}

		m_pDecoder->SetInFilter(this,EncryptOld);			// Filter einstellen

		return;
		}


	throw CAF_ERROR_PASSWD;									// Unbekannte Methode

}

//*****************************************************************************
//*
//*		SelectDecoder
//*
//*****************************************************************************
//	Whlt den Modus fr den Entpacker
//	uMode		: Ist die Komprimirungs Methode
//	pEntry		: Ist die Datei die entpackt wird
//	bWrite		: Wird komprimiert
//	Ergibt 0 wenn der neue Entpacker existiert bzw einen Fehlercode grer 0
//				0 = ok Entpacker wurde erstellt
//				1 = unbekannter Modus
//				2 = zu wenig Speicher
int CZipFile::SelectDecoder(unsigned uMode,ZipItem *pEntry,bool bWrite)
{
CBaseCompressor	   *pOldClass;
CBaseCompressor	   *pNewClass;
int					iBaseMode;


	iBaseMode = uMode;

	#if CHF_HAS_DEFALTE64
	if(uMode<=9)uMode = 0;
	#else
	if(uMode<=8)uMode = 0;
	#endif


	if(uMode==(unsigned)m_iDecoder)							// Keine nderung
		{
		if(iBaseMode>0 && iBaseMode<=8)
			{
			m_pDecoder->SetMode(CHF_DEFLATE);
			}
		else if(iBaseMode==0)
			{
			m_pDecoder->SetMode(CHF_STORE);
			}
		else if(iBaseMode==9)
			{
			if(bWrite)return 1;
			m_pDecoder->SetMode(CHF_DEFLATE64);
			}
		#ifdef ZIP_USE_LZMA
		else if(iBaseMode==14)
			{
			((CLzmaClass*)m_pDecoder)->SetDataSize(pEntry->uLenFull);
			}
		#endif

		return 0;
		}


	pOldClass = m_pDecoder;


	switch(uMode)											// Neuen Entpacker erstellen
		{
	case 0:	{
			pNewClass = new CHuffman;

				 if(iBaseMode==0)iBaseMode=CHF_STORE;
			else if(iBaseMode<=8)iBaseMode=CHF_DEFLATE;
			else if(iBaseMode==0)iBaseMode=CHF_DEFLATE64;
			}
			break;

	#ifdef ZIP_USE_BZIP2
	case 12:{												// BZIP2 Modus
			pNewClass = new CBzipClass;
			iBaseMode = CBZ_DEFLATE;
			}
			break;
	#endif

	#ifdef ZIP_USE_LZMA
	case 14:{												// LZMA Modus
			CLzmaClass	*pClass;
			pClass    = new CLzmaClass;
			pClass->SetDataSize(pEntry->uLenFull);
			pNewClass = pClass;
			iBaseMode = 3;
			}
			break;
	#endif

	#ifdef ZIP_USE_LZMA2
	case 95:{												// LZMA2 Modus
			pNewClass = new CXzClass;
			iBaseMode = CXZ_DEFLATE;
			}
			break;
	#endif

	#ifdef ZIP_USE_PPMD
	case 98:{												// PPMD Modus
			pNewClass = new CPpmClass;
			iBaseMode = 1;
			}
			break;
	#endif

	default:
			return 1;
		}


	if(!pNewClass)											// Speicher prfen
		{
		return 2;
		}


	pNewClass->SetMode(iBaseMode);
	pNewClass->Reset();

	m_pDecoder = pNewClass;

	if(pOldClass)
		{
		pOldClass->Free();
		delete pOldClass;
		}


	m_pDecoder->SetMode(iBaseMode);
	m_iDecoder = uMode;



return 0;
}

