//*****************************************************************************
//*
//*
//*		ZipFile.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//
//	Klasse fr den Zugriff auf Zip Files:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *pType=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *pType=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractProc(int iNum,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractProc(char *  ,void *, ...)		// Datei in Funktion extrahieren
//		int		ExtractMem (int iNum,char *pMem,int)	// Datei in Speicher extrahieren
//		int		ExtractMem (char *  ,char *pMem,int)	// Datei in Speicher extrahieren
// 		int		ExtractAll (         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(    int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *pName ,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *pName)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		char   *Name (int iNum,int &iLen);				// Hole Dateiname mit Lnge
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *pName);					// Datei suchen
// 		time_t	Time (char  *pName);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen


#ifndef		__ZIPFILE_H__
#define     __ZIPFILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	"ArchiveFile.h"
#include	"FileFinder.h"
#include	"Filter/Crc32.h"
#include	"Codecs/IoCallbacks.h"


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif




//*****************************************************************************
//*
//*		Klasse CZipFile
//*
//*****************************************************************************
class CZipFile : public CArchiveFile
{
protected:

	typedef unsigned char    uchar;

	typedef struct ZipItem
	{
	unsigned char  *pExtra;      				// Extra field (set only ifext != 0)
	unsigned char  *pCentral;     				// Extra in central (set only ifcext != 0)
	void		   *pUserPtr;					// User-Pointer
	zchar		   *pComment;    				// Kommentar (set only ifcom != 0)
	zchar		   *pPath;						// Dateipfad der einzufgenden Datei (real)
	zchar		   *pName;       				// Dateiname im ZIP-Archiv
	auint64			uLenComp;					// Dateilnge komrimiert
	auint64			uLenFull;					// Dateilnge unkomrimiert
	auint64			uOff;						// Offset des Datei-Headers im Zip-Archiv
	time_t			iTimeM;						// Zeit der letzten nderung
	time_t			iTimeC;						// Zeit der Dateierstellung
	time_t			iTimeA;						// Zeit des letzten Zugriffs
	unsigned short	uVerExrt;					// Version zum Extrahieren
	unsigned short	uVerCreate;					// Version beim Erzeugen
	unsigned short	uCompMode;					// Koprimierungsmethode
	unsigned short	uNameLen;					// Dateinamen-Lnge
	unsigned short	uComLen;					// Kommentarlnge
	unsigned short	uCenLen;					// Anzahl der Extra Bytes in CEN Block
	unsigned short	uFlags;						// Flags
	unsigned short	uExt;						// Anzahl der Extra Bytes in LOC Block
	unsigned short  uDisk;						// Disketten Nummer vom Dateianfang
	unsigned short  uAtt;						// Interne Attribute
	unsigned 		uAtx;						// Externe Attribute
	unsigned		uCrcSum;					// CRC32 Code
	unsigned 		uDosTime;					// Zeit und Datum (Dos-Format)
	char 			cDosFlag;					// Ist Datei eine DOS-Datei
	char 			cDecomp;					// Ist Block richtig endkomprimiert
	char 			cDelete;					// Ist Block zu lschen
	char 			cWrite;						// Ist Block neu zuschreiben
	char 			cSaved;						// Ist Block in Datei
	char 			cError;						// Ist Block fehlerhaft
	char			cAesMode;					// AES-Verschlsselung (0=keine 1=AES1 2=AES2)
	char			cAesDecom;					// Koprimierungsmethode bei AES
	}ZipItem;

	typedef struct								// Zeit Structur
	{
	time_t 			iTimeA;						// Letzter Zugriff
	time_t 			iTimeM;						// Letzte nderung
	time_t 			iTimeC;						// Erzeugung
	}ZipTimes;

	
	CBaseCompressor*m_pDecoder;					// Klasse fr De/Komprimieren
	int				m_iDecoder;					// Ist die Nummer des En-/Decoders
	int				m_iChanged;					// Wurde die ZipDatei verndert
	aint64			m_iEndPos;					// Position des END-Blockes
	aint64			m_iCenPos;					// Position des ersten CEN-Blockes

	ZipItem       **m_pFiles;					// Alfabethische Liste
	int				m_iFilesNum;				// Anzahl der Listeneintrge
	int				m_iFilesMax;				// Gesamt Listengre in Eintrgen
	int				m_iFileCount;				// Listen Eintrge ohne gelschte

	void 		   *m_pCrypt;					// Zeiger auf Encoder
	zchar		   *m_pComment;					// Kommentar vom Archiv 

	aint64			m_iDoneNow;					// Aktueller Verlauf
	aint64			m_iDoneMax;					// Maximaler Verlauf
	int				m_iDoneMode;				// 0=direkt von m_iDoneNow 1=Extraxt 2=ExtractAll 3=Flush

	unsigned		m_aKeys[3];					// Key fr alte Verschlsselung


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

					CZipFile();
	virtual		   ~CZipFile();

	virtual	int     AddPath    (const zchar   *pName);	// Ein Verzeichniss einfgen
	virtual	int		AddProc    (const zchar   *pName,void *pParam,InpProc pProc,FileData *pInfo=0);
	virtual	int		ExtractProc(int iNum            ,void *pParam,OutProc pProc,FileData *pInfo=0);
	virtual	int		Change     (int iMode=CAF_WRITE);	// Schreiberlaubnis aktivieren
	virtual	int		Test       (int iNum);				// Eine Datei prfen

	virtual	int		Flush      ();						// Datei speichern
	virtual	int		Close      ();						// Datei schliesen
	virtual	int		Count	   ();						// Anzahl der Dateien
	virtual	void	Break      ();						// Bearbeitung abbrechen
	virtual	int		Stats	   (int  iNum);				// Zustansbits abfragen
	virtual	zchar  *Name       (int  iNum);				// Hole Dateiname
	virtual	zchar  *Name       (int  iNum,int &iLen);	// Hole Dateiname mit Lnge
	virtual	int		Sort       (int  iMode=0);			// Sortierte Kette
	virtual	int     SetFail    (int	 iNum,int iFail=1);	// Defekt-Flag setzen
	virtual	int     IsReadOnly (int  iNum);				// Ist die Datei schreibgeschtzt
	virtual	int     IsFail     (int  iNum);				// Ist die Datei defekt
	virtual	int     IsDel      (int  iNum);				// Wurde die Datei gelscht
	virtual	int     IsDir      (int  iNum);				// Prfe ob Verzeichnis
	virtual	int     IsOk       (int  iNum);				// Richtig endkomprimiert
	virtual	aint64	Len        (int  iNum,int iMode=0);	// Hole Dateilnge
	virtual	int		Del        (int  iNum);				// Datei lschen
	virtual	time_t	Time       (int  iNum);				// Dateizeit abfragen
	virtual	time_t	Time       (int  iNum,int iMode );	// Dateizeit abfragen
	virtual	int		Find       (const zchar  *pName );	// Datei suchen
	virtual	aint64	Done       (aint64 *pMax        );	// Verlaufsabfrage
	virtual	void   *GetUserPtr (int iNum            );	// Datenwert auslesen
	virtual	void    SetUserPtr (int iNum,void *pData);	// Datenwert einstellen

protected:

	virtual	int		SetStats      (int iNum,int iSet);	// Die Zustand einer Archivdatei setzen
	virtual	int     SetDoneMode   (int iMode );			// Modus fr Done setzen
	virtual	int		ArchOpen      (int iLevel);			// ffnet die ZIP-Datei

	void			Clear         ();					// Speicher lschen
	time_t			DosToUtcTime  (unsigned uDosTime );	// Zeit konvertieren
	unsigned		UtcToDosTime  (time_t   iUnixTime);	// Zeit konvertieren
	int				GetFileMode   (unsigned     uAttr);	// Dateiattribute umwandeln
	int				Write         (int          iNum );
	int				ScanExHeader  (ZipItem     *pEntry,unsigned char *pExtra,unsigned uExtraLen);
	int				SetExHeader   (ZipItem     *pEntry);
	int				CreateString  (const zchar *pName,char *pBuffer,int iSize,unsigned short *pFlags=0);
	void			SetEncrypter  (ZipItem     *pEntry);
	int 			SelectDecoder (unsigned     uMode,ZipItem *pEntry,bool bWrite=false);
	unsigned char	UpdateKeys    (unsigned char ucByte);
	unsigned char	DecryptByte   ();
								  
	int				PutCEN		  (ZipItem *pEntry);	// CEN-Block speichern
	int				PutLOC        (ZipItem *pEntry);	// LOC-Block speichern
	int				PutEND        ();					// END-Block speichern
								  
	int				ZipNew        ();					// Listen Eintrag erzeugen
	void			ZipDel        (int  iNum);			// Listen Eintrag lschen
	void			ZipSetDel     (int  iNum);			// Listen Eintrag lschen
	int				ReadString    (zchar *pMem,int iSize,unsigned uFlags=0);// ZipFile lesen fr Texte

	unsigned short 	GetL2         (void *pMem);  
	unsigned int   	GetL4         (void *pMem);
	auint64	   		GetL8         (void *pMem);
	void           	SetL2         (void *pMem,short  sVal);
	void           	SetL4         (void *pMem,int    iVal);
	void           	SetL8         (void *pMem,aint64 iVal);

static int __cdecl 	CmpName       (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpLen        (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpPos        (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl 	CmpEx         (const void *pPtr1,const void *pPtr2);// Vergleichsfunktion
static int __cdecl  EncryptOld    (void *pParam,      char *pMem,int iLen);
static int __cdecl  EncryptAes    (void *pParam,      char *pMem,int iLen);
};
//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline unsigned short   CZipFile::GetL2(void *pMem){return *(unsigned short  *)pMem;}
inline unsigned int     CZipFile::GetL4(void *pMem){return *(unsigned int    *)pMem;}
inline unsigned aint64	CZipFile::GetL8(void *pMem){return *(unsigned aint64 *)pMem;}
inline void             CZipFile::SetL2(void *pMem,short    sVal){*(short  *)pMem=sVal;}
inline void             CZipFile::SetL4(void *pMem,int      iVal){*(int    *)pMem=iVal;}
inline void             CZipFile::SetL8(void *pMem,aint64   iVal){*(aint64 *)pMem=iVal;}



#endif




#ifdef _DEBUG
#endif // _DEBUG