//*****************************************************************************
//*
//*
//*		Zp7File.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf Z7-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *Type=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *Type=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractMem(int iNum,char *pMem,int)		// Datei in Speicher extrahieren
// 		int		ExtractAll(         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(   int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *Datei,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *Datei)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *Datei);					// Datei suchen
// 		time_t	Time (char  *Datei);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen

#include	<stdio.h>
#include	<string.h>
#include	<stddef.h>
#include	"Zp7File.h"
#include	"BitFunctions.h"
#include	"Zip7/Aes7zip.h"
#include	"Zip7/Bcj7zip.h"
#include	"Zip7/Bcj27zip.h"
#include	"Filter/Sha256.h"
#include	"Codecs/PpmClass.h"
#include	"Codecs/BzipClass.h"
#include	"Codecs/CopyClass.h"
#include	"Codecs/LzmaClass.h"
#include	"Codecs/Lzma2Class.h"


#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	atoiz	   _wtoi
#define 	itoaz	   _itow
#define 	strzlen		wcslen
#define 	strzcmp		wcscmp
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#define 	strzncat	wcsncat
#define 	strzupr		wcsupr
#define 	strzicmp	wcsicmp
#define 	strzcspn	wcscspn
#define 	szprintf	swprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	atoiz	    atoi
#define 	itoaz		itoa
#define 	strzlen		strlen
#define 	strzcmp		strcmp
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#define 	strzncat	strncat
#define 	strzupr		strupr
#define 	strzicmp	stricmp
#define 	strzcspn	strcspn
#define 	szprintf	sprintf
#endif

#define 	ZIP7_ID_END						0
#define 	ZIP7_ID_HEADER					1
#define 	ZIP7_ID_ARCHIVEPROPERTIES		2
#define 	ZIP7_ID_ADDITIONALSTREAMSINFO	3
#define 	ZIP7_ID_MAINSTREAMSINFO			4
#define 	ZIP7_ID_FILESINFO				5
#define 	ZIP7_ID_PACKINFO				6
#define 	ZIP7_ID_UNPACKINFO				7
#define 	ZIP7_ID_SUBSTREAMSINFO			8
#define 	ZIP7_ID_SIZE					9
#define 	ZIP7_ID_CRC						10
#define 	ZIP7_ID_FOLDER					11
#define 	ZIP7_ID_CODERSUNPACKSIZE		12
#define 	ZIP7_ID_NUMUNPACKSTREAM			13
#define 	ZIP7_ID_EMPTYSTREAM				14
#define 	ZIP7_ID_EMPTYFILE				15
#define 	ZIP7_ID_ANTI					16
#define 	ZIP7_ID_NAME					17
#define 	ZIP7_ID_CTIME					18
#define 	ZIP7_ID_ATIME					19
#define 	ZIP7_ID_MTIME					20
#define 	ZIP7_ID_WINATTRIBUTES			21
#define 	ZIP7_ID_COMMENT					22
#define 	ZIP7_ID_ENCODEDHEADER			23
#define 	ZIP7_ID_STARTPOS				24
#define 	ZIP7_ID_DUMMY					25

#define 	MAX_HEADER_SIZE					0x10000000
#define 	FILTER_BUFFER_SIZE				0x20000
#define		DELTA_STATE_SIZE				256


union  TarData
	{
	unsigned char	ucBuffer[TAR_RECORDSIZE];
	struct
		{
		char		cName    [TAR_NAMSIZE];
		char    	cMode    [8];
		char    	cUid     [8];
		char    	cGid     [8];
		char    	cSize    [12];
		char    	cTimeM   [12];
		char    	cChkSum  [8];
		char    	cLinkFlag;
		char    	cLinkName[TAR_NAMSIZE];
		char    	cMagic   [8];
		char    	cUname   [TAR_TUNMLEN];
		char    	cGname   [TAR_TGNMLEN];
		char    	cDevMajor[8];
		char    	cDevMinor[8];
		char		cPrefix  [131];
		char		cTimeA   [12];
		char		cTime    [12];
		};
	};



#define PROG(iMode,pEntry)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,pEntry))throw CAF_ERROR_BREAK;\
	}
#define PROGR(iMode,pEntry,iRet)					\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	if(Progress(iMode,pEntry))return iRet;			\
	}
#define PROGN(iMode,pEntry)							\
	if(m_pProgress && (m_iProgOn&(1<<iMode)))		\
	{												\
	Progress(iMode,pEntry);							\
	}

#define 	VerctorRead(p,i)	bit_read(p,(i)^7)

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CZp7File::CZp7File()
{

	m_pZp7StreamSize	= 0;
	m_pZp7StreamCrc		= 0;
	m_pFilterBuffer		= 0;
	m_pUnpackAlloc		= 0;
	m_pPackAlloc		= 0;
	m_pFilterBuffer		= 0;
	m_pZp7Files			= 0;
	m_pZp7Vector		= 0;
	m_uZp7FileCount		= 0;
	m_uZp7EmptyStream	= 0;
	m_uZp7VecSize		= 0;
	m_iDoneMode			= 0;
	m_iDoneNow			= 0;
	m_iDoneMax			= 0;
	m_pZp7Next			= 0;
	m_cHasCompress		= 0;
	m_pSpecialMem		= 0;
	m_uSpecialSize		= 0;

}

//*****************************************************************************
//*
//*		Denstruktor
//*
//*****************************************************************************
CZp7File::~CZp7File()
{


	if(m_pZp7Files)
		{
		while(m_uZp7FileCount>0)
			{
			m_uZp7FileCount--;
			delete m_pZp7Files[m_uZp7FileCount];
			}

		delete m_pZp7Files;
		}

	if(m_pZp7Next      )delete m_pZp7Next;
	if(m_pZp7Vector    )delete m_pZp7Vector;
	if(m_pFilterBuffer )delete m_pFilterBuffer;
	if(m_pSpecialMem   )delete m_pSpecialMem;
	if(m_pPackAlloc    )delete m_pPackAlloc;
	if(m_pUnpackAlloc  )delete m_pUnpackAlloc;
	if(m_pFilterBuffer )delete m_pFilterBuffer;
	if(m_pZp7StreamSize)delete m_pZp7StreamSize;



}

//*****************************************************************************
//*
//*		IsMainCoder
//*
//*****************************************************************************
//	Prft ob es ein Haup-Codec ist
int _cdecl CZp7File::IsMainCoder(uint64_zip7 uCodec)
{
static const uint64_zip7	aMainCodecs[] =
		{
		0x00030401,											// PPMD
		0x00000021,											// LZMA2
		0x00030101,											// LZMA
		0x00040202,											// BZIP2
		0x00000000,											// COPY
		};

int  iPos;


	for(iPos=0;iPos<sizeof(aMainCodecs)/sizeof(aMainCodecs[0]);iPos++)
		{
		if(aMainCodecs[iPos]==uCodec)return iPos;
		}



return -1;
}

//*****************************************************************************
//*
//*		Zip7ReadBigEndian64
//*
//*****************************************************************************
//	Liet eine Big-Endian-Zahl mit variabler Lnge ein
//	iSize		: Ist die Lnge der Zahl
//	Ergibt die Zahl
CZp7File::uint64_zip7 CZp7File::Zip7ReadBigEndian64(int iSize)
{
uint64_zip7		uValue;
unsigned char	ucByte;



	uValue = 0;

	while(iSize>0)
		{
		GetByte(ucByte);

		uValue <<= 8;
		uValue  |= ucByte;
		iSize   -= 1;
		}


return uValue;
}

//*****************************************************************************
//*
//*		Zip7ReadNumber64
//*
//*****************************************************************************
//	Liet eine Encoded-64-Bit-Zahl mit variabler Lnge ein
//	Ergibt die Zahl
CZp7File::uint64_zip7 CZp7File::Zip7ReadNumber64()
{
uint64_zip7		uTemp;
uint64_zip7		uValue;
unsigned char	ucFirst;
unsigned char	ucMask;
unsigned char	ucByte;
int				iPos;


	GetByte(ucFirst);

	uValue = 0;
	ucMask = 0x80;

	for(iPos=0;iPos<64;iPos+=8)
		{
		if(ucFirst&ucMask)
			{
			GetByte(ucByte);

			uTemp	 = ucByte;
			uValue  |= uTemp<<iPos;
			ucMask >>=1;
			continue;
			}

		ucMask	= ucMask-1;
		uTemp	= ucFirst&ucMask;
		uValue |= uTemp<<iPos;

		break;
		}

return uValue;
}


//*****************************************************************************
//*
//*		Zip7AllocVector
//*
//*****************************************************************************
//	Allokiert einen Speicher fr ein Bit-Feld.
inline CZp7File::uint8_zip7 *CZp7File::Zip7AllocVector(unsigned uCount)
{
unsigned	uMax;


	if(uCount<=m_uZp7VecSize)
		{
		return m_pZp7Vector;
		}

	if(m_pZp7Vector)
		{
		delete m_pZp7Vector;
		m_uZp7VecSize = 0;
		m_pZp7Vector  = 0;
		}

	uMax			= (uCount+0xFF)&~0xFF;
	m_pZp7Vector	= new uint8_zip7[uMax/8];

	if(!m_pZp7Vector)
		{
		m_uZp7VecSize = 0;
		throw CAF_ERROR_MEM;
		}

	m_uZp7VecSize = uMax;


return m_pZp7Vector;
}

//*****************************************************************************
//*
//*		Zip7ReadVector
//*
//*****************************************************************************
void CZp7File::Zip7ReadVector(unsigned uSize,uint8_zip7 *pData,unsigned uMax)
{
unsigned char 	ucByte;
unsigned		uNum;


	GetByte(ucByte);

	uNum	= (uSize+7)/8;

	if(ucByte)												// Alle Eintrge sind auf 1
		{
		if(pData)
			{
			if(uNum>uMax)uNum=uMax;
			memset(pData,0xFF,uNum);
			}
		return;
		}

	if(pData)
		{
		if(uMax>uNum)uMax=uNum;
		GetData(pData,uMax);
		uNum -= uMax;
		}

	while(uNum)
		{
		GetByte(ucByte);
		uNum--;
		}

}

//*****************************************************************************
//*
//*		Zip7ReadBoolVector
//*
//*****************************************************************************
void CZp7File::Zip7ReadBoolVector(unsigned uSize,uint8_zip7 *pData,unsigned uMax)
{
unsigned char 	ucByte;
unsigned		uNum;


	uNum	= (uSize+7)/8;

	if(pData)
		{
		if(uMax>uNum)uMax=uNum;
		GetData(pData,uMax);
		uNum -= uMax;
		}

	while(uNum)
		{
		GetByte(ucByte);
		uNum--;
		}

}

//*****************************************************************************
//*
//*		Zip7WaitId
//*
//*****************************************************************************
//	berspringt alle ID bis zu einer bestimmten
void CZp7File::Zip7WaitId(unsigned uStop)
{
uint64_zip7		uSize;
uint32_zip7		uId;


	for(;;)
		{
		   uId = Zip7ReadNumber32();
		if(uId==uStop)break;

		uSize = Zip7ReadNumber64();
	    Zip7SkipData(uSize);
		}


}

//*****************************************************************************
//*
//*		Zip7ReadPackInfo
//*
//*****************************************************************************
int CZp7File::Zip7ReadPackInfo(Zip7PackInfo **pInfoPtr)
{
unsigned char	aVector[(ZIP7_MAX_ALLOC+7)/8];
unsigned		uBytes;
Zip7PackInfo   *pInfo;
uint32_zip7		uCount;
uint64_zip7		uOffset;
uint64_zip7		uSize;
uint32_zip7		uPos;
uint32_zip7		uCrc;
uint32_zip7		uId;



	uOffset	= Zip7ReadNumber64();
	uCount	= Zip7ReadNumber32();

	if(uCount==0 || uCount>ZIP7_MAX_ALLOC)				// Prfe die Werte
		{
		throw CAF_ERROR_FORM;
		}

	uBytes = sizeof(Zip7PackInfo)+sizeof(Zip7PackItem)*(uCount-1);

		pInfo = (Zip7PackInfo*)new unsigned char[uBytes];
	if(!pInfo)
		{
		throw CAF_ERROR_MEM;
		}

	if(m_pPackAlloc)									// Alten Speicher freigeben
		{
		delete m_pPackAlloc;
		m_pPackAlloc = 0;
		}

	memset(pInfo,0,uBytes);
   *pInfoPtr			= pInfo;
	m_pPackAlloc		= pInfo;
	pInfo->uStreamCount	= uCount;  
	pInfo->uDataOffset	= uOffset;


	Zip7WaitId(ZIP7_ID_SIZE);


	for(uPos=0;uPos<uCount;uPos++)						// Stream-Gren laden
		{
		uSize = Zip7ReadNumber64();
		pInfo->aEntries[uPos].uPackSize = uSize;
		}

	for(;;)				   
		{
		uId = Zip7ReadNumber32();

		if(uId==ZIP7_ID_END)
			{
			break;
			}

		if(uId==ZIP7_ID_CRC)							// CRC's laden
			{
			Zip7ReadVector(uCount,aVector,sizeof(aVector));

			for(uPos=0;uPos<uCount;uPos++)
				{
				if(!VerctorRead(aVector,uPos))continue;
				GetDWord(uCrc);
				pInfo->aEntries[uPos].uStreamCrc = uCrc;
				}

			continue;
			}

		uSize = Zip7ReadNumber64();
	    Zip7SkipData(uSize);
		}




return pInfo->uStreamCount;
}

//*****************************************************************************
//*
//*		Zip7ReadFolder
//*
//*****************************************************************************
int CZp7File::Zip7ReadFolder(Zip7Folder *pFolder)
{
unsigned char	ucByte;
unsigned		uCoder;
unsigned		uOff;
unsigned		uPos;
unsigned		uInp;
unsigned		uOut;
unsigned		uNum;



	memset(pFolder,0,sizeof(Zip7Folder));

	pFolder->uCoderCount = Zip7ReadNumber32();

	if(pFolder->uCoderCount>ZIP7_MAX_CODERS)
		{
		throw CAF_ERROR_NOTSUPP;
		}

	if(pFolder->uCoderCount==0)
		{
		throw CAF_ERROR_FORM;
		}

	for(uCoder=0;uCoder<pFolder->uCoderCount;uCoder++)
		{
		GetByte(ucByte);

		if((ucByte&0xC0)!=0 || (ucByte&0x0F)>8)
			{
			throw CAF_ERROR_FORM;
			}

		pFolder->aCoders[uCoder] = Zip7ReadBigEndian64(ucByte&0x0F);

		if(ucByte&0x10)
			{
			uInp = Zip7ReadNumber32();
			uOut = Zip7ReadNumber32();
			}
		else{
			uInp = 1;
			uOut = 1;
			}

		pFolder->uInpStreams += uInp;
		pFolder->uOutStreams += uOut;

		if(ucByte&0x20)
			{
			uNum = Zip7ReadNumber32();

			if(uNum>ZIP7_MAX_PROPERTY)
				{
				throw CAF_ERROR_FORM;
				}

			pFolder->aPropSizes[uCoder] = uNum;
			uOff = uCoder*ZIP7_MAX_PROPERTY;
			GetData(pFolder->aProperties+uOff,uNum);
			}

		while(ucByte&0x80)
			{
			GetByte(ucByte);
			Zip7SkipData(ucByte&0x0F);

			if(ucByte&0x10)
				{
				uInp = Zip7ReadNumber32();
				uOut = Zip7ReadNumber32();
				}

			if(ucByte&0x20)
				{
				uNum = Zip7ReadNumber32();
				Zip7SkipData(uNum);
				}
			}
		}


	if(pFolder->uOutStreams==0)
		{
		throw CAF_ERROR_FORM;
		}

	if(pFolder->uInpStreams==0)
		{
		throw CAF_ERROR_FORM;
		}
	
													
	if(pFolder->uCoderCount!=1 || pFolder->uInpStreams!=1)
		{
		pFolder->uBindPairs = pFolder->uCoderCount-1;

		if(pFolder->uInpStreams<pFolder->uBindPairs)
			{
			throw CAF_ERROR_FORM;
			}

		if(pFolder->uInpStreams>ZIP7_MAX_STREAMS)
			{
			throw CAF_ERROR_NOTSUPP;
			}

		if(pFolder->uBindPairs>0)
			{
			for(uPos=0;uPos<pFolder->uBindPairs;uPos++)
				{
				uInp = Zip7ReadNumber32();
				uOut = Zip7ReadNumber32();
				}
			}

		pFolder->uPackStreams = pFolder->uInpStreams-pFolder->uBindPairs;

		if(pFolder->uPackStreams>1)
			{
			for(uPos=0;uPos<pFolder->uPackStreams;uPos++)
				{
				uNum = Zip7ReadNumber32();
				}
			}

		if(pFolder->uOutStreams>pFolder->uCoderCount)
			{
			throw CAF_ERROR_FORM;
			}
		}


	if(pFolder->uOutStreams>ZIP7_MAX_STREAMS)
		{
		throw CAF_ERROR_FORM;
		}

	if(pFolder->uCoderCount>ZIP7_MAX_CODERS)
		{
		throw CAF_ERROR_FORM;
		}



return pFolder->uCoderCount;
}

//*****************************************************************************
//*
//*		Zip7ReadUnpackInfo
//*
//*****************************************************************************
int CZp7File::Zip7ReadUnpackInfo(Zip7UnpackInfo **pInfoPtr)
{
unsigned char	aVector[(ZIP7_MAX_ALLOC+7)/8];
unsigned		uStream;
unsigned		uBytes;
unsigned char	ucByte;
Zip7UnpackInfo *pInfo;
uint32_zip7		uCount;
uint64_zip7		uSize;
uint32_zip7		uPos;
uint32_zip7		uCrc;
uint32_zip7		uId;



	Zip7WaitId(ZIP7_ID_FOLDER);

	uCount	= Zip7ReadNumber32();						// Anzahl der Folder holen
	GetByte(ucByte);

	if(uCount==0 || uCount>ZIP7_MAX_ALLOC || ucByte)	// Prfe die Werte
		{
		throw CAF_ERROR_FORM;
		}

	uBytes = sizeof(Zip7UnpackInfo)+sizeof(Zip7Folder)*(uCount-1);

		pInfo = (Zip7UnpackInfo*)new unsigned char[uBytes+64];
	if(!pInfo)
		{
		throw CAF_ERROR_MEM;
		}

	if(m_pUnpackAlloc)									// Alten Speicher freigeben
		{
		delete m_pUnpackAlloc;
		m_pUnpackAlloc = 0;
		}

	memset(pInfo,0,uBytes);
   *pInfoPtr			= pInfo;
	m_pUnpackAlloc		= pInfo;
	pInfo->uFolderCount = uCount;

														
	if(uCount>ZIP7_MAX_FOLDERS)							// Prfe die Anzahl der Eintrge
		{
		throw CAF_ERROR_NOTSUPP;
		}


	for(uPos=0;uPos<uCount;uPos++)						// Alle Folder laden
		{
		Zip7ReadFolder(pInfo->aFolders+uPos);
		}



	Zip7WaitId(ZIP7_ID_CODERSUNPACKSIZE);


	for(uPos=0;uPos<uCount;uPos++)						// Die Gren der Streams laden
		{
		for(uStream=0;uStream<pInfo->aFolders[uPos].uOutStreams;uStream++)
			{
			uSize = Zip7ReadNumber64();
			pInfo->aFolders[uPos].aUnpackSize[uStream] = uSize;
			}
		}


	for(;;)												// CRC's der Folder laden
		{
		uId = Zip7ReadNumber32();

		if(uId==ZIP7_ID_END)
			{
			break;
			}

		if(uId==ZIP7_ID_CRC)
			{
			Zip7ReadVector(uCount,aVector,sizeof(aVector));

			for(uPos=0;uPos<uCount;uPos++)
				{
				if(!VerctorRead(aVector,uPos))continue;
				GetDWord(uCrc);
				pInfo->aFolders[uPos].uCrcValue = uCrc;
				}

			continue;
			}

		uSize = Zip7ReadNumber64();
	    Zip7SkipData(uSize);
		}




return uCount;
}


//*****************************************************************************
//*
//*		Zip7ReadSubStreamsInfo
//*
//*****************************************************************************
//	Einlesen der Einzellngen von Dateien
int CZp7File::Zip7ReadSubStreamsInfo(Zip7UnpackInfo *pInfo)
{
unsigned char	aVector[(ZIP7_MAX_ALLOC+7)/8];
unsigned char  *pVector;
unsigned		uStream;
unsigned		uCount;
uint64_zip7		uSize;
uint64_zip7		uAll;
uint32_zip7		uPos;
uint32_zip7		uOff;
uint32_zip7		uIdx;
uint32_zip7		uCrc;
uint32_zip7		uNum;
uint32_zip7		uId;



	for(uPos=0;uPos<pInfo->uFolderCount;uPos++)				// Standart auf einen Stream
		{
		pInfo->aFolders[uPos].uUnpackStreams = 1;
		}


	for(;;)
		{
		uId = Zip7ReadNumber32();

		if(uId==ZIP7_ID_END )break;
		if(uId==ZIP7_ID_CRC )break;
		if(uId==ZIP7_ID_SIZE)break;

		if(uId==ZIP7_ID_NUMUNPACKSTREAM)					// Stream-Counter laden
			{
			for(uPos=0;uPos<pInfo->uFolderCount;uPos++)
				{
				uNum = Zip7ReadNumber32();
				pInfo->aFolders[uPos].uUnpackStreams = uNum;
				}

			continue;
			}

		uSize = Zip7ReadNumber64();
	    Zip7SkipData(uSize);
		}


	for(uPos=0,uCount=0;uPos<pInfo->uFolderCount;uPos++)	// Streams zhlen
		{
		uCount += pInfo->aFolders[uPos].uUnpackStreams;
		}


	if(uCount)												// Speicher allokieren
		{
		pInfo->uSubStreams = uCount;

		if(m_pZp7StreamSize)
			{
			delete m_pZp7StreamSize;
			m_pZp7StreamSize = 0;
			}

			m_pZp7StreamSize = new uint64_zip7[uCount+uCount/2+1];
		if(!m_pZp7StreamSize)
			{
			throw CAF_ERROR_MEM;
			}

			m_pZp7StreamCrc = (uint32_zip7*)(m_pZp7StreamSize+uCount);
		if(!m_pZp7StreamCrc)throw CAF_ERROR_MEM;

		memset(m_pZp7StreamCrc ,0,sizeof(uint32_zip7)*(uCount+1));
		memset(m_pZp7StreamSize,0,sizeof(uint64_zip7)*(uCount+1));
		}



	uOff = 0;

	if(uId==ZIP7_ID_SIZE)									// Stream-Lngen fr die Dateien laden
		{
		for(uPos=0;uPos<pInfo->uFolderCount;uPos++)
			{
				uNum = pInfo->aFolders[uPos].uUnpackStreams;
			if(!uNum)continue;

			uAll = 0;

			if(uOff+uNum>uCount)
				{
				throw CAF_ERROR_FORM;
				}

			for(uStream=1;uStream<uNum;uStream++)
				{
				uSize = Zip7ReadNumber64();
				m_pZp7StreamSize[uOff] = uSize;
				uAll+=uSize;
				uOff++;
				}

			uIdx = pInfo->aFolders[uPos].uCoderCount-1;
			m_pZp7StreamSize[uOff]  = pInfo->aFolders[uPos].aUnpackSize[uIdx];
			m_pZp7StreamSize[uOff] -= uAll;
			uOff++;
			}

		uId = Zip7ReadNumber32();
		}

															// Keine Gre fr Folder
	if(uOff==0 && uCount==pInfo->uFolderCount && pInfo->aFolders[0].uCoderCount>0)
		{
		for(uPos=0;uPos<pInfo->uFolderCount;uPos++)
			{
			uIdx                   = pInfo->aFolders[uPos].uCoderCount-1;
			m_pZp7StreamSize[uPos] = pInfo->aFolders[uPos].aUnpackSize[uIdx];
			}
		}


	if(uId==ZIP7_ID_CRC)									// CRC's fr die einzelnen Dateien
		{
		if(uCount>sizeof(aVector)*8)
				pVector = Zip7AllocVector(uCount);
		else	pVector = aVector;

		Zip7ReadVector(uCount,pVector);

		for(uPos=0,uOff=0;uPos<pInfo->uFolderCount;uPos++)
			{
				uNum = pInfo->aFolders[uPos].uUnpackStreams;
			if(!uNum)continue;

			for(uStream=0;uStream<uNum;uStream++,uOff++)
				{
				if(!VerctorRead(pVector,uOff))continue;
				GetDWord(uCrc);

				if(uOff>=uCount)break;
				m_pZp7StreamCrc[uOff] = uCrc;
				}
			}

		if(uOff!=uCount)
			{
			throw CAF_ERROR_FORM;
			}

		uId = Zip7ReadNumber32();
		}



	if(uId!=ZIP7_ID_END)									// Vor auf's Ende
		{
		Zip7WaitId(ZIP7_ID_END);
		}



return uCount;
}

//*****************************************************************************
//*
//*		Zip7ReadFileInfo
//*
//*****************************************************************************
int CZp7File::Zip7ReadFileInfo(Zip7UnpackInfo *pInfo)
{
uint8_zip7     *pVector;
Zip7File	   *pFile;
uint64_zip7	   *pQword;
unsigned char  *pEmpty;
unsigned char	ucByte;
uint32_zip7		uBytes;
uint64_zip7		uSize;
uint16_zip7		uChar;
uint32_zip7		uPos;
uint32_zip7		uNum;
uint32_zip7		uOff;
uint32_zip7		uId;



	pEmpty				= 0;
	m_uZp7EmptyStream	= 0;
	m_uZp7FileCount		= Zip7ReadNumber32();

	if(m_uZp7FileCount<=0 || m_uZp7FileCount>0x400000)
		{
		throw CAF_ERROR_FORM;
		}

	if(m_pZp7Files)
		{
		delete m_pZp7Files;
		m_pZp7Files = 0;
		}


		m_pZp7Files	= new Zip7File *[m_uZp7FileCount];
	if(!m_pZp7Files)
		{
		throw CAF_ERROR_MEM;
		}

	pVector = Zip7AllocVector(m_uZp7FileCount);



	for(uPos=0;uPos<m_uZp7FileCount;uPos++)
		{
		pFile = new Zip7File;
		m_pZp7Files[uPos] = pFile;
		memset(pFile,0,sizeof(Zip7File));
		}


	for(;;)
		{
		   uId = Zip7ReadNumber32();
		if(uId==ZIP7_ID_END)break;

		uSize = Zip7ReadNumber64();

		switch(uId)
			{
		case ZIP7_ID_NAME:									// Dateinamen einlesen

			GetByte(ucByte);

			if(ucByte)
				{
				throw CAF_ERROR_NOTSUPP;
				}

			if(!(uSize&1) || (uSize>>32))
				{
				throw CAF_ERROR_FORM;
				}

			uNum	= 0;
			uPos	= 0;
			pFile	= m_pZp7Files[0];
			uBytes	= ((unsigned)uSize-1)/2;

			for(uOff=0;uOff<uBytes;uOff++)
				{
				GetWord(uChar);

				pFile->cName[uNum] = uChar;
				uNum++;

				if(uChar)continue;

				uPos += 1;
				uNum  = 0;
				pFile = m_pZp7Files[uPos];

				if(uPos<m_uZp7FileCount)continue;
				if(uOff+1!=uBytes)
					{
					throw CAF_ERROR_FORM;
					}

				break;
				}

			break;

		case ZIP7_ID_CTIME:									// Zeiten einlesen
		case ZIP7_ID_ATIME:
		case ZIP7_ID_MTIME:

				                      uOff=offsetof(Zip7File,uTimeWrite);
				if(uId==ZIP7_ID_CTIME)uOff=offsetof(Zip7File,uTimeCreate);
				if(uId==ZIP7_ID_ATIME)uOff=offsetof(Zip7File,uTimeAccess);

				Zip7ReadVector(m_uZp7FileCount,pVector);

				GetByte(ucByte);

				if(ucByte)
					{
					throw CAF_ERROR_NOTSUPP;
					}

				for(uPos=0;uPos<m_uZp7FileCount;uPos++)
					{
					if(!VerctorRead(pVector,uPos))continue;
					pQword = (uint64_zip7*)(((char*)m_pZp7Files[uPos])+uOff);
					GetQWord(*pQword);
					}

				break;

		case ZIP7_ID_WINATTRIBUTES:							// Dateiattribute holen

				Zip7ReadVector(m_uZp7FileCount,pVector);

				GetByte(ucByte);

				if(ucByte)
					{
					throw CAF_ERROR_NOTSUPP;
					}

				for(uPos=0;uPos<m_uZp7FileCount;uPos++)
					{
					if(!VerctorRead(pVector,uPos))continue;
					GetDWord(m_pZp7Files[uPos]->uAttributes);
					}

				break;

		case ZIP7_ID_EMPTYSTREAM:							// Welche Datei hat Daten im Stream

				Zip7ReadBoolVector(m_uZp7FileCount,pVector);

				if(m_uZp7FileCount&7)
					{
					pVector[m_uZp7FileCount/8] &= 0xFF00>>(m_uZp7FileCount&7);
					}

				m_uZp7EmptyStream = bit_count_1(pVector,(m_uZp7FileCount+7)&~7);
				uNum	= (m_uZp7FileCount+7)/8;
				pEmpty	= new unsigned char[uNum];
				memcpy(pEmpty,pVector,uNum);

				break;

		case ZIP7_ID_EMPTYFILE:

				Zip7ReadBoolVector(m_uZp7EmptyStream,pVector);
				break;

		default:
				Zip7SkipData(uSize);
				break;
			}
		}


	for(uPos=0,uOff=0;uPos<m_uZp7FileCount;uPos++)			// CRC und Dateilngen zuordenen
		{
		pFile = m_pZp7Files[uPos];

		if(!pEmpty || !VerctorRead(pEmpty,uPos))
			{
			pFile->uCrcValue	= m_pZp7StreamCrc [uOff];
			pFile->uFileSize	= m_pZp7StreamSize[uOff];
			uOff++;
			}
		}

															// Nur eine Datei im Stream
	if(m_uZp7FileCount==1 && pFile->uFileSize==0 && pInfo->uFolderCount==1)
		{
		uPos             = pInfo->aFolders[0].uCoderCount-1;
		pFile->uFileSize = pInfo->aFolders[0].aUnpackSize[uPos];
		}

	if(pEmpty)delete pEmpty;



return m_uZp7FileCount;
}

//*****************************************************************************
//*
//*		MakeHeader
//*
//*****************************************************************************
//	Speichert einen TAR-Header
//	pPtr		: Zeiger auf den Puffer fr den Header
//	iMode		: Zugriffsflags fr die Datei
//	pName		: Ist der Dateiname
//	iSize		: Ist die Dateigre
//	iTime		: Ist die Dateizeit
//	Ergibt 0 wenn ok bzw. die bentigte Textlnge fr den Pfad
int CZp7File::MakeHeader(void *pTarData,int iMode,const char *pName,aint64 iSize,unsigned uTime)
{
TarData	   *pPtr=(TarData*)pTarData;
unsigned	uChkSum;
int			iNameLen;
int			iRet;
int			i;


	memset(pPtr->ucBuffer,0,sizeof(TarData));

	iNameLen = (int)strlen(pName);
	iRet	 = 0;

	if(iNameLen>=sizeof(pPtr->cName))
		{
		iRet	 = iNameLen;
		iNameLen = sizeof(pPtr->cName);
		}


	memcpy(pPtr->cName,pName,iNameLen);


	if(iMode=='5')										// Slash anhngen
		{
		if(iNameLen>=sizeof(pPtr->cName))
			{
			iRet = (iRet)? iRet+1:iNameLen+1;
			}
		else{
			pPtr->cName[iNameLen] = '/';
			}
		}


	memcpy (pPtr->cUid  ,"0000000" ,8);
	memcpy (pPtr->cGid  ,"0000000" ,8);
	sprintf(pPtr->cSize ,"%07I64o" ,iSize);
	sprintf(pPtr->cTimeM,"%07o"    ,uTime);
	sprintf(pPtr->cMode ,"%07o"    ,iMode);
	memcpy (pPtr->cMagic,"ustar  " ,8);
	memcpy (pPtr->cUname,"guest"   ,8);
	memcpy (pPtr->cGname,"guest"   ,8);
		    pPtr->cLinkFlag = iMode;
	memset (pPtr->cChkSum,' ',sizeof(pPtr->cChkSum));


	for(uChkSum=0,i=0;i<TAR_RECORDSIZE;i++)
		{
		uChkSum += pPtr->ucBuffer[i];
		}

	sprintf(pPtr->cChkSum,"%07o",uChkSum);


return iRet;
}


//*****************************************************************************
//*
//*		WriteToTar
//*
//*****************************************************************************
//	Schreibt Daten in die temporre TAR-Datei.
//	Wenn ntig wird ein Header eingefgt
int	_cdecl CZp7File::WriteToTar(void *pParam,const char *pMem,int iSize)
{
CZp7File   *pThis=(CZp7File*)pParam;
Zip7File   *pFile;
char		cBuffer[512];
char		cDummy [512];
char		cName  [512];
DWORD		dwBytes;
unsigned	uTime;
int			iLen;
int			iAll;
int			iRet;


	iAll = 0;

Next:

	while(pThis->m_uWritePos==0)						// Header fr Dateien schreiben
		{
		if(pThis->m_uWriteRest)
			{
			memset(cBuffer,0,pThis->m_uWriteRest);

			if(pThis->MainWrite(pThis,cBuffer,pThis->m_uWriteRest)^pThis->m_uWriteRest)
				{
				return -1;
				}

			pThis->m_iDoneNow  += pThis->m_uWriteRest;
			pThis->m_uWriteRest = 0;
			}

		if(pThis->m_uZp7FilePos)						// Prfe die CRC
			{
			   pFile = pThis->m_pZp7Files[pThis->m_uZp7FilePos-1];
			if(pFile->uCrcValue && pFile->uCrcValue!=~pThis->m_uWriteCrc)
				{
				return -1;
				}
			}

		if(pThis->m_uZp7FilePos>=pThis->m_uZp7FileCount)// Wurden alle Dateien geschrieben
			{
			return (iSize)? -1:iAll;
			}


		pFile = pThis->m_pZp7Files[pThis->m_uZp7FilePos];
		iLen  = strtoutf8(cName,pFile->cName,sizeof(cName));

		FileTimeToUtcTime((FILETIME*)&pFile->uTimeWrite,&uTime);

		if(pFile->uAttributes&FILE_ATTRIBUTE_DIRECTORY)
			{
			iRet				= pThis->MakeHeader(cBuffer,'5',cName,0,uTime);
			}
		else{
			iRet				= pThis->MakeHeader(cBuffer,'0',cName,pFile->uFileSize,uTime);
			pThis->m_uWriteRest = (-(int)pFile->uFileSize)&0x01FF;
			pThis->m_uWritePos	= pFile->uFileSize;
			}

		if(iRet)										// Ist der Dateiname zu lang ?
			{
			pThis->MakeHeader(cDummy,'L',"././@LongLink",iRet,0);
			memset(cName+iLen,0,512-iLen);

			if(pThis->MainWrite(pThis,cDummy,512)!=512)
				{
				return -1;
				}

			if(pThis->MainWrite(pThis,cName,512)!=512)
				{
				return -1;
				}
			}

		if(pThis->MainWrite(pThis,cBuffer,512)!=512)
			{
			return -1;
			}

		pThis->m_uWriteCrc = 0xFFFFFFFF;
		pThis->m_uZp7FilePos++;
		}


	if(iSize>=pThis->m_uWritePos)						// Wird ber die Dateigrenze geschrieben
		{
		dwBytes = (unsigned)pThis->m_uWritePos;

		if(pThis->MainWrite(pThis,pMem,dwBytes)^dwBytes)
			{
			return -1;
			}

		pThis->m_uWriteCrc = Crc32(pThis->m_uWriteCrc,pMem,dwBytes);
		pThis->m_uWritePos = 0;
		pThis->m_iDoneNow += dwBytes;

		iAll  += dwBytes;
		pMem  += dwBytes;
		iSize -= dwBytes;

		goto Next;
		}


	dwBytes = 0;

	pThis->MainWrite(pThis,pMem,iSize);

	pThis->m_uWriteCrc  = Crc32(pThis->m_uWriteCrc,pMem,iSize);
	pThis->m_uWritePos -= iSize;
	pThis->m_iDoneNow  += iSize;
	iAll               += iSize;



return iAll;
}

//*****************************************************************************
//*
//*		Zip7Decode
//*
//*****************************************************************************
CZp7File::uint64_zip7 CZp7File::Zip7Decode(Zip7PackInfo *pPack,Zip7UnpackInfo *pUnpack,uint64_zip7 uSize,unsigned char *pMem)
{
CBaseCompressor	   *pDecoder;
CLzma2Class		   *pLzm2Dec;
CLzmaClass		   *pLzmaDec;
CBzipClass		   *pBzipDec;
CCopyClass		   *pCopyDec;
uint64_zip7			aSeeks[ZIP7_MAX_STREAMS+1];
uint64_zip7			aReads[ZIP7_MAX_STREAMS];
uint64_zip7			aSizes[ZIP7_MAX_STREAMS];
unsigned			aPlace[ZIP7_MAX_STREAMS];
unsigned			uSpecial;
unsigned			uStream;
unsigned			uFilter;
unsigned			uCrypt;
unsigned			uCoder;
unsigned			uIndex;
uint64_zip7			uOffset;
uint64_zip7			uCodec;
uint64_zip7			uByte;
uint64_zip7			uRead;
uint64_zip7			uDone;
uint64_zip7			uAll;
unsigned			uSlot;
unsigned			uPos;
unsigned			uOff;
unsigned			uNum;
unsigned			uCrc;



	if(pUnpack->uFolderCount==0 || pPack->uStreamCount<pUnpack->uFolderCount)
		{
		throw CAF_ERROR_FORM;
		}


	uAll	= 0;
	uIndex	= 0;
	uOffset	= pPack->uDataOffset+sizeof(Zip7Header);

	for(uPos=0;uPos<pUnpack->uFolderCount;uPos++)
		{
		uCoder		= pUnpack->aFolders[uPos].uCoderCount;

		if(uCoder>ZIP7_MAX_STREAMS)
			{
			throw CAF_ERROR_NOTSUPP;
			}


		uCrypt	 	= 0;
		uFilter	 	= 0;
		uStream	 	= 0;
		uSpecial 	= 0;

		//*********************************************************************
													
		uCodec		= pUnpack->aFolders[uPos].aCoders[0];

		switch(uCodec)										// Nach einer Verschlsselung am Anfang suchen
			{
		case 0x06F10701:									// AES Verschlsselung	
			
			
			uRead	= pPack->aEntries[uPos].uPackSize;

			if(uRead&0x0F)
				{
				throw CAF_ERROR_FORM;
				}

			if(!InitAes(pUnpack->aFolders[uPos].aProperties,pUnpack->aFolders[uPos].aPropSizes[0]))
				{
				throw CAF_ERROR_PASSWD;
				}

			m_uAesMax	= pUnpack->aFolders[uPos].aUnpackSize[0];
			m_uAesRest	= 0;
			uCrypt		= 1;
			
			break;
			}

		if(uCrypt)
			{
			   uStream++;
			if(uStream>=uCoder)throw CAF_ERROR_FORM;

			uOffset += pPack->aEntries[uIndex].uPackSize;
			uIndex++;
			}

		
		//*********************************************************************

		
		aSeeks[0] = uOffset;

		uCodec = pUnpack->aFolders[uPos].aCoders[uCoder-1];

		switch(uCodec)										// Nach einem Filter beim letzen Codec suchen
			{
		case 0x03030103:									// BCJ Filter

			if(!m_pFilterBuffer)m_pFilterBuffer = new unsigned char[FILTER_BUFFER_SIZE+256];
			if(!m_pFilterBuffer)throw CAF_ERROR_MEM;
			
			m_uFilterSize		= pUnpack->aFolders[uPos].aUnpackSize[uCoder-1];
			m_uFilterOffset		= 0;
			m_uFilterPrevMask	= 0;
			m_uFilterBufferPos	= 0;
			uFilter				= 1;
			break;

		case 0x0303011B:									// BCJ2 Filter (mit vier Streams)

			if(uCoder<4)
				{
				throw CAF_ERROR_FORM;
				}
															// Gren im Spezial-Speicher
			m_uFilterSize	= pUnpack->aFolders[uPos].aUnpackSize[uCoder-1];
			aSizes[0]		= pUnpack->aFolders[uPos].aUnpackSize[uCoder-4];
			aSizes[1]		= pUnpack->aFolders[uPos].aUnpackSize[uCoder-3];
			aSizes[2]		= pUnpack->aFolders[uPos].aUnpackSize[uCoder-2];
			aSizes[3]		= pPack->aEntries[uIndex+1].uPackSize;
			aPlace[0]		= (unsigned)m_uFilterSize;
						
			for(uNum=0;uNum<3;uNum++)
				{
				uCodec	= pUnpack->aFolders[uPos].aCoders[uCoder-uNum-2];
				if(IsMainCoder(uCodec)<0)throw CAF_ERROR_FORM;
				aPlace[uNum+1] = aPlace[uNum]+(unsigned)aSizes[uNum];
				}

			
			uNum = (unsigned)(aPlace[3]+aSizes[3]);

			if(uNum>m_uSpecialSize)							// Spezial-Speicher anlegen	
				{
				delete m_pSpecialMem;
				m_pSpecialMem  = 0;
				m_uSpecialSize = 0;
				}

			if(!m_pSpecialMem)
				{
				if(m_uFilterSize>0x10000000)throw CAF_ERROR_NOTSUPP;
				m_pSpecialMem  = new unsigned char[uNum+4];
				if(!m_pSpecialMem)throw CAF_ERROR_MEM;
				m_uSpecialSize = uNum;
				}

			if(pPack->uStreamCount<uIndex+4)
				{
				throw CAF_ERROR_FORM;
				}

			aSeeks[2]	= uOffset;							// Lese Offsets der Streams in der ZIP-Datei
			aSeeks[3]	= aSeeks[2]+pPack->aEntries[uIndex  ].uPackSize;
			aSeeks[1]	= aSeeks[3]+pPack->aEntries[uIndex+1].uPackSize;
			aSeeks[0]	= aSeeks[1]+pPack->aEntries[uIndex+2].uPackSize;

															// Bytes der Streams in der ZIP-Datei
			aReads[0]	= pPack->aEntries[uIndex+3].uPackSize;
			aReads[1]	= pPack->aEntries[uIndex+2].uPackSize;
			aReads[2]	= pPack->aEntries[uIndex  ].uPackSize;
			aReads[3]	= pPack->aEntries[uIndex+1].uPackSize;

			pUnpack->aFolders[uPos].aUnpackSize[uCoder-1] = aSizes[3];
			pUnpack->aFolders[uPos].aCoders    [uCoder-1] = 0x00000000;

			uSpecial    = 1;

			break;

		case 0x00000003:									// Delta Filter

			if(!m_pFilterBuffer)m_pFilterBuffer = new unsigned char[FILTER_BUFFER_SIZE+256];
			if(!m_pFilterBuffer)throw CAF_ERROR_MEM;
			
			uOff				= ZIP7_MAX_PROPERTY;
			m_uFilterPrevMask	= pUnpack->aFolders[uPos].aProperties[uOff]+1;
			uFilter				= 2;
			memset(m_pFilterBuffer,0,m_uFilterPrevMask);
			break;			  
			}

		if(uFilter)
			{
			   uCoder--;
			if(uCoder==0)throw CAF_ERROR_FORM;
			}

		if(!uSpecial)										// Offsets und Gren der Streams holen
			{
			for(uSlot=0;uSlot<uCoder;uSlot++)				
				{
				aReads[uSlot  ]	= pPack->aEntries[uIndex+uSlot].uPackSize;
				aSeeks[uSlot+1]	= aSeeks[uSlot]+aReads[uSlot];
				}
			}		


		//*********************************************************************


															// Die Streams entkomprimieren
		for(uSlot=0;uStream<uCoder;uStream++,uIndex++,uSlot++)				
			{
			if(uIndex>pPack->uStreamCount)
				{
				throw CAF_ERROR_FORM;
				}

			uByte	= pUnpack->aFolders[uPos].aUnpackSize[uStream];
			uRead	= aReads[uSlot];


			if(uByte==0 || uRead==0)
				{
				throw 1;
				}


			switch(pUnpack->aFolders[uPos].aCoders[uStream])
				{
			/*case 0x00030401:								// PPMD

					if(pUnpack->aFolders[uPos].aPropSizes[uStream]!=5)
						{
						throw CAF_ERROR_FORM;
						}

					uOff = uStream*ZIP7_MAX_PROPERTY;

					pPpmdDec = new CPpmClass;
					pPpmdDec->SetProbs(pUnpack->aFolders[uPos].aProperties+uOff,5);
					pPpmdDec->SetMode(0x02);
					pDecoder = pPpmdDec;
					break;*/

			case 0x00000021:								// LZMA2

					if(pUnpack->aFolders[uPos].aPropSizes[uStream]!=1)
						{
						throw CAF_ERROR_FORM;
						}

					uOff = uStream*ZIP7_MAX_PROPERTY;

					pLzm2Dec = new CLzma2Class;
					pLzm2Dec->SetProbs(pUnpack->aFolders[uPos].aProperties+uOff,1);
					pLzm2Dec->SetDataSize(uByte);
					pLzm2Dec->SetMode(0);
					pDecoder = pLzm2Dec;
					break;

			case 0x00030101:								// LZMA

					if(pUnpack->aFolders[uPos].aPropSizes[uStream]!=5)
						{
						throw CAF_ERROR_FORM;
						}

					uOff = uStream*ZIP7_MAX_PROPERTY;

					pLzmaDec = new CLzmaClass;
					pLzmaDec->SetProbs(pUnpack->aFolders[uPos].aProperties+uOff);
					pLzmaDec->SetDataSize(uByte);
					pLzmaDec->SetMode(0);
					pDecoder = pLzmaDec;
					break;

			case 0x00040202:								// BZIP2

					if(pUnpack->aFolders[uPos].aPropSizes[uStream]!=0)
						{
						throw CAF_ERROR_FORM;
						}

					pBzipDec = new CBzipClass;
					pBzipDec->SetMode(CBZ_DEFLATE);
					pDecoder = pBzipDec;
					break;

			case 0x00000000:								// COPY

					if(uByte!=uRead)
						{
						throw CAF_ERROR_FORM;
						}

					if(pUnpack->aFolders[uPos].aPropSizes[uStream]!=0)
						{
						throw CAF_ERROR_FORM;
						}

					pCopyDec = new CCopyClass;
					pCopyDec->SetMode(CPY_STORE);
					pCopyDec->SetDataSize(uByte);
					pDecoder = pCopyDec;
					break;

			default:

					throw CAF_ERROR_NOTSUPP;

				}

		
			//*****************************************************************

		
			if(uSize && uRead>uSize)throw CAF_ERROR_FORM;	// Zum Stream in der *.7z Datei
			BaseSeek(aSeeks[uSlot]);
			
			uCrc = pPack->aEntries[uPos].uStreamCrc;
			SetInput(pDecoder,1,uRead,(uCrc)? uCrc:IOC_NO_CRC);
			

			switch(uCrypt)
				{
			case 1:											// AES Verschlsselung

				pDecoder->SetInFilter(this,AesFilter);
				break;
				}


			uCrc = pUnpack->aFolders[uPos].aUnpackCrc[uStream];

			
			if(uSpecial)									// Spezial-Ausgabe	
				{
				pDecoder->SetOut((unsigned)uByte,m_pSpecialMem+aPlace[uSlot],(uCrc)? uCrc:IOC_NO_CRC);
				}
			else if(pMem)									// Ausgabe in Speicher
				{
				pDecoder->SetOut((unsigned)uByte,pMem,(uCrc)? uCrc:IOC_NO_CRC);
				}
			else{											// Ausgabe in Datei
				pDecoder->SetOut(this,WriteToTar,uCrc);
				}


			switch(uFilter)
				{
			case 1:											// BCJ Filter

				pDecoder->SetOut(this,BcjFilter,uCrc);
				break;

			case 2:											// Delta Filter

				pDecoder->SetOut(this,DeltaFilter,uCrc);
				break;			  
				}


			try	{
				m_cThrowRd = 0;
				uDone = pDecoder->Decompress();				// Stream entpacken
				}
			catch(...)
				{
				delete pDecoder;
				throw CAF_ERROR_FORM;
				}

			delete pDecoder;
			m_cThrowRd = 1;

															// Prfe CRC und Lnge
			if(uDone!=uByte || (uCrc && uCrc!=pDecoder->GetCrcOut()))
				{
				throw CAF_ERROR_FORM;
				}

			if(pMem)
				{
				pMem  += uDone;
				uSize -= uDone;
				}

			uAll	+= uDone;
			uOffset	+= uRead;
			}

		
		//*********************************************************************

		
		if(uSpecial)										// Die Spezail-Augabe
			{
			switch(uSpecial)
				{
			case 1:											// BCJ2 Filter
			
				uNum = Bcj2Convert(	m_pSpecialMem+aPlace[2],(unsigned)aSizes[2],
									m_pSpecialMem+aPlace[1],(unsigned)aSizes[1],
									m_pSpecialMem+aPlace[0],(unsigned)aSizes[0],
									m_pSpecialMem+aPlace[3],(unsigned)aSizes[3],
									m_pSpecialMem		   ,(unsigned)m_uFilterSize);

				if(uNum!=m_uFilterSize)
					{
					throw CAF_ERROR_FORM;
					}	

				WriteToTar(this,(char*)m_pSpecialMem,uNum);
				
				break;
				}
			}

		if(uFilter)
			{
		// REM AZ Test ob ein 7ZIp mit vier Folter geht
		// 	uIndex++;  
			}
		}


return uAll;
}

//*****************************************************************************
//*
//*		Zip7CryptInit
//*
//*****************************************************************************
int CZp7File::InitAes(const unsigned char *pProperties,int iPropSize)
{
unsigned char	aPassword[92];
unsigned char	aSaltData[16];
unsigned char	aIvData  [16];
unsigned char	aKeyData [32];
unsigned char	aTempData[8];
unsigned		uNumCycles;
unsigned		uSaltSize;
unsigned		uIvSize;
unsigned		uRounds;
unsigned		uRound;
unsigned		uPwLen;
unsigned		uPos;
Sha256Data		sSha;



	if(!m_pPassword || *m_pPassword==0)				// Existiert ein Passwort
		{
		return 0;
		}

	for(uPwLen=0;uPwLen<sizeof(aPassword);uPwLen+=2)	// Password auf WCHAR erweitern
		{
		if(!m_pPassword[uPwLen/2])break;
		aPassword[uPwLen  ] = m_pPassword[uPwLen/2];
		aPassword[uPwLen+1] = 0;
		}

	memset(aIvData,0,sizeof(aIvData));

	if(iPropSize<2)return 0;

	uNumCycles	= (pProperties[0]   )&0x3F;
	uIvSize		= (pProperties[0]>>6)&1;
	uSaltSize	= (pProperties[0]>>7);

	if(uSaltSize==0 && uIvSize==0)
		{
		return 0;
		}

	uIvSize	   += (pProperties[1]   )&0x0F;
	uSaltSize  += (pProperties[1]>>4);

	if(uIvSize+uSaltSize+2>(unsigned)iPropSize)
		{
		return 0;
		}

	memcpy(aSaltData,pProperties+2,uSaltSize);
	memcpy(aIvData  ,pProperties+2+uSaltSize,uIvSize);

	if(uNumCycles>24)
		{
		return 0;										// Nicht implementiert
		}


	Sha256Init(&sSha);									// CalculateDigest() in 7Zip

	uRounds = 1<<uNumCycles;
	memset(aTempData,0,8);

	for(uRound=0;uRound<uRounds;uRound++)
		{
		Sha256(&sSha,aSaltData,uSaltSize);
		Sha256(&sSha,aPassword,uPwLen);
		Sha256(&sSha,aTempData,8);

		for(uPos=0;uPos<8;uPos++)
			{
			   aTempData[uPos]++;
			if(aTempData[uPos])
				{
				break;
				}
			}
    	}

	Sha256Exit(&sSha,aKeyData);


	AesGenTables();
	Aes_SetKey_Dec(m_sAes+4,aKeyData,32);
	AesCbc_Init   (m_sAes,aIvData);


return 1;
}

//*****************************************************************************
//*
//*		DeltaFilter
//*
//*****************************************************************************
int _cdecl 	CZp7File::DeltaFilter(void *pParam,const char *pMem,int iLen)
{
CZp7File       *pThis=(CZp7File*)pParam;
unsigned char	aBuffer[DELTA_STATE_SIZE];
unsigned char	aOutput[1024];
unsigned char	ucByte;
unsigned		uDelta;
unsigned		uOff;
int				iPos;
int				iNum;
int				iRet;
  

	uDelta = pThis->m_uFilterPrevMask;
	memcpy(aBuffer,pThis->m_pFilterBuffer,uDelta);


	for(iPos=0,iNum=0;iPos<iLen;)
		{
		for(uOff=0;uOff<uDelta && iPos<iLen;iPos++,uOff++)
			{
			ucByte			= aBuffer[uOff]+pMem[iPos];
			aBuffer[uOff]	= ucByte;
			aOutput[iNum]   = ucByte;
			
			iNum++;

			if(iNum<sizeof(aOutput))continue;

			   iRet	= WriteToTar(pParam,(char*)aOutput,iNum);
			if(iRet<iNum)return -1;

			iNum = 0;
			}
		}

	if(iNum>0)
		{
		   iRet	= WriteToTar(pParam,(char*)aOutput,iNum);
		if(iRet<iNum)return -1;
		}
  
	if(uOff==uDelta)uOff=0;
		
	memcpy(pThis->m_pFilterBuffer            ,aBuffer+uOff,uDelta-uOff);
	memcpy(pThis->m_pFilterBuffer+uDelta-uOff,aBuffer,uOff);



return iLen;
}

//*****************************************************************************
//*
//*		BcjFilter
//*
//*****************************************************************************
int _cdecl 	CZp7File::BcjFilter(void *pParam,const char *pMem,int iLen)
{
CZp7File       *pThis=(CZp7File*)pParam;
unsigned char  *pData;
unsigned 		uDone;
unsigned 		uOff;
unsigned		uNum;
int				iRet;
int				iNum;


	uDone	= 0;
	pData	= pThis->m_pFilterBuffer;
	uOff	= pThis->m_uFilterOffset;

	while(iLen>0)
		{
		if(uOff+iLen > FILTER_BUFFER_SIZE)				// Passt alles in den BCJ-Puffer
				uNum = FILTER_BUFFER_SIZE-uOff;
		else	uNum = iLen;

		memcpy(pData+uOff,pMem,uNum);					// BCJ-Puffer

		pMem  += uNum;
		uDone += uNum;
		uOff  += uNum;
		iLen  -= uNum;

		pThis->m_uFilterSize -= uNum;

		if(pThis->m_uFilterSize>0 && uOff<0x10000)		// Nur den Puffer fllen
			{
			break;
			}

		iNum  = BcjConvert(pData,uOff,pThis->m_uFilterBufferPos,&pThis->m_uFilterPrevMask,0);
		uOff -= iNum;

		if(pThis->m_uFilterSize==0 && uOff<=4)			// Die letzten 4 Byte direkt aus Puffer holen
			{
			iNum += uOff;
			uOff  = 0;
			}

		pThis->m_uFilterBufferPos += iNum;

		   iRet	= WriteToTar(pParam,(char*)pData,iNum);
		if(iRet<iNum)return -1;

		memcpy(pData,pMem-uOff,uOff);					// Den Rest im Puffer lassen
		}

	pThis->m_uFilterOffset = uOff;


return uDone;
}

//*****************************************************************************
//*
//*		AesFilter
//*
//*****************************************************************************
int _cdecl 	CZp7File::AesFilter(void *pParam,char *pMem,int iLen)
{
CZp7File   *pThis=(CZp7File*)pParam;
unsigned	uByte;
unsigned	uRet;


	if(pThis->m_uAesRest)								// Sind noch Bytes im Puffer
		{
		   uByte = 16-pThis->m_uAesRest;
		if(uByte>(unsigned)iLen)uByte=iLen;

		memcpy(pThis->m_aAesTemp+pThis->m_uAesRest,pMem,uByte);

		pThis->m_uAesRest += uByte;

		if(pThis->m_uAesRest<16)return 0;

		AesCbc_Decode(pThis->m_sAes,pThis->m_aAesTemp,1);

		pMem += uByte;
		iLen -= uByte;
		uRet  = 16;
		}
	else{
		uRet  = 0;
		}

														// Entschlsseln
	AesCbc_Decode(pThis->m_sAes,(unsigned char*)pMem,iLen/16);
	uRet             += iLen&~0x0F;
	pThis->m_uAesRest = iLen& 0x0F;

	if(iLen&0x0F)										// Sind Bytes brig geblieben
		{
		pMem += iLen&~0x0F;
		memcpy(pThis->m_aAesTemp,pMem,pThis->m_uAesRest);
		}


	if(pThis->m_uAesMax<uRet)							// Maximales Ausgabelnge erreicht
		{
		uRet = (unsigned)pThis->m_uAesMax;
		}

	pThis->m_uAesMax -= uRet;


return uRet;
}

//*****************************************************************************
//*
//*		ArchOpen
//*
//*****************************************************************************
//	ffnet eine TAR-Datei und liest alle bereits gespeicherten Dateien ein.
//	pFileName:	Ist der Dateiname des TGZ-Archivs ("res:\\" fr Resourcen)
//	iMode:		CAF_READ	1		Zp7-Datei nur Lesen
//				CAF_CREATE	3		Zp7-Datei neu erstellen Altes lschen
//				CAF_WRITE	2		Zp7-Datei auch schreiben bzw. neuerstellen
//									wenn nicht vorhanden
//	iLevel:		Komprimierungs Levl (0-9) 9=maximal  (unbenutzt)
//	Ergibt:		CAF_ERROR_NONE		Wenn pFileName nicht gefunden werden konnte
//				CAF_ERROR_OPEN		Wenn pFileName nicht geffnet werden konnte
//				CAF_ERROR_CREATE		Wenn pFileName nicht erzeugt  werden konnte
//				CAF_ERROR_READ		Bei Lese-Fehlern
//				CAF_ERROR_FORM		Bei Fehlern im TAR-Datei
//				CAF_ERROR_MULTI		Wenn Zib ber mehere Disketten geht
//				CAF_ERROR_LOGIC		Bei internen unbekannten Fehlern
//	Ergibt CAF_ERROR_OK wenn alles ok war, bzw. einen anderen Fehlercode
int CZp7File::ArchOpen(int iLevel)
{
Zip7UnpackInfo *pUnpackInfo;
Zip7PackInfo   *pPackInfo;
Zip7Header		sHeader;
uint64_zip7		uCodec;
uint64_zip7		uSize;
unsigned		uFlag;
unsigned		uCrc;
unsigned		uId;
unsigned		uPos;
unsigned		uLevel;
unsigned		uCoder;
unsigned		uStream;
int				iByte;
int				iRet;
int				iMin;
int				iMax;



	SetDoneMode(-1);										// Done-Modus in Tar-Datei

	m_iDoneMode	= 0;
	m_iDoneBase = 0;
	m_iDoneNow	= 0;
	m_iDoneMax	= 0;
	m_cIsCpio	= 0;


	if(m_iZp7Size<0x400000)
			iByte = (int)m_iFileSize+16;
	else	iByte = 0x400000;	

	Clear();
	InitSubFile();

	if(iByte<0x400000)										// Bis 4MB im Speicher ffnen
		{
		iMin	= (int)iByte;
		iMax	= 0x1000000;								// 10MB maximal
		}
	else{
		iMin	= 0;
		iMax	= 0;
		}


	SetMemSub(0x10000,0x30000,m_cTarName,Z(".tar"));		// Dynamischen Speicher erstellen
		

	m_iErrorCode			= CAF_ERROR_OK;
	m_iZp7Size				= m_iSubSize;
	m_uZp7FileCount			= 0;
	pUnpackInfo				= 0;
	pPackInfo				= 0;


//******************** Archiv dekomprimieren **********************************


	m_iDoneMode = 1;


	try	{
		if(m_iZp7Size<sizeof(Zip7Header))
			{
			throw CAF_ERROR_FORM;
			}

		m_iReadMax = sizeof(sHeader);

		BaseRead(&sHeader,sizeof(sHeader));
		
		if(memcmp(sHeader.aSignature,"7z\xBC\xAF\x27\x1C\x00\x03",7))
			{
			throw CAF_ERROR_FORM;
			}

		   uCrc	= ~Crc32(0xFFFFFFFF,sHeader.aSignature+12,20);
		if(uCrc!=sHeader.uStartHeaderCrc)
			{
			throw CAF_ERROR_FORM;
			}

		if(sHeader.uNextHeaderSize  >=MAX_HEADER_SIZE ||
		   sHeader.uNextHeaderOffset>=0x8000000000    ||
		   sHeader.uNextHeaderOffset+sHeader.uNextHeaderSize>(uint64_zip7)m_iZp7Size)
			{
			throw CAF_ERROR_FORM;
			}

		if(m_pZp7Next)
			{
			delete m_pZp7Next;
			m_pZp7Next = 0;
			}

			m_pZp7Next = new unsigned char[(int)sHeader.uNextHeaderSize+16];
		if(!m_pZp7Next)
			{
			throw CAF_ERROR_MEM;
			}

		m_iReadMax		= sHeader.uNextHeaderSize;
		m_uZp7NextSize	= (int)sHeader.uNextHeaderSize;

															// 7ZIP-Header einlesen
		BaseSeek(sHeader.uNextHeaderOffset+sizeof(Zip7Header));
		BaseRead(m_pZp7Next,m_uZp7NextSize);


		   uCrc	 = ~Crc32(0xFFFFFFFF,m_pZp7Next,(int)sHeader.uNextHeaderSize);
		if(uCrc!=sHeader.uNextHeaderCrc)
			{
			throw CAF_ERROR_FORM;
			}

		uFlag			= 0;
		uLevel			= 0;
		m_uZp7NextPos	= 0;								// Zeiger auf Position in Folder-Daten

		for(;;)												// Zip-Next-Header dekodieren
			{
			uId	= Zip7ReadNumber32();

			switch(uId)
				{
			case ZIP7_ID_HEADER:

					uLevel++;
					break;

			case ZIP7_ID_ENCODEDHEADER:

					if(uFlag>=1)throw 1;
					uFlag = 1;
					uLevel++;
					break;

			case ZIP7_ID_MAINSTREAMSINFO:

					if(uFlag>=2)throw 1;
					uFlag = 2;
					uLevel++;
					break;

			case ZIP7_ID_PACKINFO:

					Zip7ReadPackInfo(&pPackInfo);
					break;

			case ZIP7_ID_UNPACKINFO:

					Zip7ReadUnpackInfo(&pUnpackInfo);
					break;

			case ZIP7_ID_SUBSTREAMSINFO:

					if(!pUnpackInfo)throw CAF_ERROR_FORM;
					Zip7ReadSubStreamsInfo(pUnpackInfo);
					break;

			case ZIP7_ID_FILESINFO:

					if(!pUnpackInfo)throw CAF_ERROR_FORM;
					Zip7ReadFileInfo(pUnpackInfo);
					break;

			case ZIP7_ID_END:

  					   uLevel--;
					if(uLevel)break;

					if(uFlag==1)							// Header mit Dateien entpacken
						{
						if(pUnpackInfo==0)
							{
							throw CAF_ERROR_FORM;
							}

						if(pUnpackInfo->uFolderCount!=1)
							{
							throw CAF_ERROR_FORM;
							}

						if(pUnpackInfo->aFolders[0].aCoders==0)
							{
							throw CAF_ERROR_FORM;
							}

						uPos  = pUnpackInfo->aFolders[0].uCoderCount-1;
						uSize = pUnpackInfo->aFolders[0].aUnpackSize[uPos];

						if(uSize==0 || uSize>MAX_HEADER_SIZE || pPackInfo==0)
							{
							throw CAF_ERROR_FORM;
							}

						delete m_pZp7Next;

							m_pZp7Next = new unsigned char[(int)uSize+16];
						if(!m_pZp7Next)
							{
							throw CAF_ERROR_MEM;
							}

						uSize = Zip7Decode(pPackInfo,pUnpackInfo,uSize,m_pZp7Next);

						m_uZp7NextSize	= (int)uSize;
						m_uZp7NextPos	= 0;
						pUnpackInfo		= 0;
						pPackInfo		= 0;

						continue;
						}

					if(m_uZp7FileCount==0 || pUnpackInfo==0 || pPackInfo==0 || uFlag!=2)
						{
						throw CAF_ERROR_FORM;
						}

					m_uZp7FilePos = 0;
					m_uWriteRest  = 0;
					m_uWritePos   = 0;

															// Gesamtgre berechnen
					for(uPos=0;uPos<pUnpackInfo->uFolderCount;uPos++)
						{
						uCoder = pUnpackInfo->aFolders[uPos].uCoderCount;
						

						if(uCoder==0 || uCoder>ZIP7_MAX_CODERS)
							{
							throw CAF_ERROR_FORM;
							}

						uStream = 0;
															// Filter am Ende ignorieren
						if(IsMainCoder(pUnpackInfo->aFolders[uPos].aCoders[uCoder-1])<0)
							{
							uCoder--;
							}
															// Verschlsselung ams Anfang ignorieren
						if(IsMainCoder(pUnpackInfo->aFolders[uPos].aCoders[0])<0)
							{
							uStream++;
							}

						for(;uStream<uCoder;uStream++)
							{
							uCodec = pUnpackInfo->aFolders[uPos].aCoders[uStream];
							if(IsMainCoder(pUnpackInfo->aFolders[uPos].aCoders[0])<0)
								{
								throw CAF_ERROR_NOTSUPP;
								}

							m_iDoneMax += pUnpackInfo->aFolders[uPos].aUnpackSize[uStream];
							}
						}

					m_iDoneMax  += m_iDoneMax/8+0x10000;

					uSize = Zip7Decode(pPackInfo,pUnpackInfo);

					goto Exit;

			default:

					throw CAF_ERROR_FORM;
				}
			}

	Exit:

		if(m_uWriteRest)
			{
			char	cBuffer[512];

			memset(cBuffer,0,512);

			MainWrite(cBuffer,m_uWriteRest);
			MainWrite(cBuffer,512);
			}


		   m_iDoneBase	 = m_iDoneMax-m_iDoneNow;
		if(m_iDoneBase<0)m_iDoneBase=0;

		m_iDoneMode  = 2;
		m_iOpenMode	 = CAF_WRITE;

		EndOfFile();
		MainSeek(0);

		iRet = CTarFile::ArchOpen(iLevel);

		if(iRet!=CAF_ERROR_OK)								// Gab es Fehler beim ffnen
			{									
			CloseFile(m_hFile);
			m_hFile = INVALID_HANDLE_VALUE;
			return iRet;
			}

		if(DoChange(CAF_READ,0)!=CAF_ERROR_OK)
			{
			return m_iErrorCode;
			}
		}
	catch(int iError)
		{
		m_iErrorCode = iError;
		PROGN(CAF_PROG_ERROR,-1);
		Break();
		}


	m_iDoneNow	= Done(&m_iDoneMax);						// Aktuellen Verlauf in TAR-Datei bernehmen
	m_iDoneMode	= 0;
	SetDoneMode(-1);										



return m_iErrorCode;
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet das TGZ-Archive und Speichert alle Daten
//	Ergibt;	CAF_ERROR_LOGIC		Bei einem Internen Fehler
//			CAF_ERROR_OK		Wenn alles stimmte
//			CAF_ERROR_WRITE		Problemen beim Schreiben
//			... siehe Error Codes
int CZp7File::Close()
{
int			iState;
int			iRet;



	if(m_pZp7Files)											// Die Dateidaten lschen 
		{
		while(m_uZp7FileCount>0)
			{
			m_uZp7FileCount--;
			delete m_pZp7Files[m_uZp7FileCount];
			}

		delete m_pZp7Files;

		m_pZp7Files = 0;
		}


	CTarFile::m_iDoneNow	= 0;
	CTarFile::m_iDoneMax	= 0;
	m_iDoneMode				= 0;
	m_iDoneNow				= 0;
	m_iDoneMax				= 0;
	iState					= m_iOpenMode;
	iRet					= CTarFile::Close();

	if(m_cTarName[0])										// Lsche temorre TAR-Datei
		{
		DeleteFile(m_cTarName);
		m_cTarName[0]=0;
		}

	m_iOpenMode  = CAF_CLOSED;
	m_iErrorCode = iRet;
	m_iZp7Size   = 0;


	ReleaseData();


return m_iErrorCode;
}

//*****************************************************************************
//*
//*		Done
//*
//*****************************************************************************
//	Fragt den aktuellen Verlauf bei Open,Extract,Close,Flush ab
//	pMax	: Wenn ungleich 0 wird hier das Maximum gespeichert
//	Ergibt den aktuellen Verlauf
aint64	CZp7File::Done(aint64 *pMax)
{
aint64	iNow;
aint64	iMax;


	switch(m_iDoneMode)
		{
	case 1:	   iNow = m_iDoneNow;							// 7-ZIP entpacken
			if(iNow>m_iDoneMax)
				{
				iNow = m_iDoneMax;
				}
	
			if(pMax)*pMax=m_iDoneMax;
			
			return iNow;

	case 2:	iNow  = CTarFile::Done(&iMax);					// TAR-Datei laden
			
			if(m_iDoneBase<=0 || iMax<=0)
				{
				iNow  = m_iDoneNow;
				}
			else{
				iNow *= m_iDoneBase;
				iNow /=	iMax;
				iNow += m_iDoneNow;
				}

			if(iNow>m_iDoneMax)
				{
				iNow = m_iDoneMax;
				}

			if(pMax)*pMax=m_iDoneMax;
			return iNow;
		}


return CTarFile::Done(pMax);
}


