//*****************************************************************************
//*
//*
//*		Zp7File.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Klasse fr den Zugriff auf 7Z-Dateien:
//
// 		int		Del   (char*)							// Datei lschen
// 		int		Del   (int)								// Datei lschen
// 		int		Open  (char*,int=0,int=0)				// Datei ffnen
//		int 	OpenRes(const char *,char *Type=0)		// Resource ffnen
//		int		OpenRes(unsigned    ,char *Type=0)		// Resource ffnen
//
// 		int		Close ();								// Datei schliesen
// 		int		Progress(void *,int (*Funkt)(...))		// Protokoll Funktion
// 		int		ProgressMode(int =0 ,int =0)			// Protokoll Funktion Modus
// 		int		ProgressAdd (int =31,int =0)			// Protokoll Nachrichten Ein
// 		int		ProgressDel (int =0 ,int =0)			// Protokoll Nachrichten Aus
//
//		char   *ErrorText()								// Ergibt Fehler-Text
//		int     Error()									// Ergibt Fehler-Code
// 		int		Add   (char*,char*=0,char*=0;char*)		// Dateien anhngen
//		int		AddMem(char*,char* ,int Len)			// Speicher Datei
//		int		ExtractMem(int iNum,char *pMem,int)		// Datei in Speicher extrahieren
// 		int		ExtractAll(         char *pPathName=0)	// Alle Dateien extrahieren
//		int		Extract(   int iNum,char *pPathName=0)	// Datei extrahieren
//		int		Extract(char *Datei,char *pPathName=0)	// Datei extrahieren
// 		int		Test   (char *Datei)					// Datei prfen
//  	int		Test   (int   iNum   )					// Datei prfen
//
// 		void	Break();								// Bearbeitung abbrechen
// 		void	Sort (int iMode=0);						// Sortierte Kette
// 		char   *Name (int iNum);						// Hole Dateiname
// 		int64	Len  (int iNum,int iMode=0);			// Hole Dateilnge
// 		int		Level(int iLevel=-2);					// Setze Komprimierungslevel
// 		int		Flags(int iFlags=-2);					// Setze iFlags
// 		int		Mode (int iMode =-2);					// Setze Komprimierungsmodus
// 		int		Find (char  *Datei);					// Datei suchen
// 		time_t	Time (char  *Datei);					// Datei Zeit holen
// 		time_t	Time (int    iNum);						// Datei Zeit holen

#ifndef 	__ZP7FILE_H__
#define 	__ZP7FILE_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if	    _MSC_VER > 1299
	#ifndef		WINVER 
	#define		WINVER 	0x0501 
	#endif 	
	#endif 	
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<windows.h>
	#endif
	#ifdef		LINUX
	#include	<unistd.h>
	#include	"stdfunk.h"
	#endif
#endif

#ifndef 	 _MSC_VER
#define 	 _cdecl
#define 	__cdecl
#endif

#include	"TarFile.h"
#include	"Codecs/BzipClass.h"

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#else
#define 	Z(a)		a
#define 	zchar		char
#endif


#define ZIP7_MAX_FOLDERS	65536						// Maximale Folder-Anzahl
#define ZIP7_MAX_ALLOC		8048						// Maximale Anzahl an Folder frs Unpacken/Packen
#define	ZIP7_MAX_PROPERTY	12							// Maximale Gre der Property-Daten
#define ZIP7_MAX_STREAMS	8							// Maximale Anzahl an Streams	
#define ZIP7_MAX_CODERS		4							// Maximale Codier-Tiefe 
						



//*****************************************************************************
//*
//*		Klasse CZp7File
//*
//*****************************************************************************
class CZp7File : public CTarFile
{
protected:

	typedef	unsigned aint64		uint64_zip7;
	typedef	unsigned int		uint32_zip7;
	typedef	unsigned short 		uint16_zip7;
	typedef	unsigned char	 	uint8_zip7;
	typedef	unsigned char	   *puint8_zip7;

	#pragma pack(push,1)

	typedef struct
		{
		uint8_zip7		aSignature[6];					// Signature "7z\xBC\xAF\x27\x1C"
		uint8_zip7		ucMajorVersion;					// Hauptversion (0)
		uint8_zip7		ucMinorVersion;					// Hauptversion (3)
		uint32_zip7		uStartHeaderCrc;				// CRC32 vom nchsten Header (von 12...31)
		uint64_zip7		uNextHeaderOffset;				// Offset des nchsten Headers (relativ zum Ende von diesen Header)
		uint64_zip7		uNextHeaderSize;				// Gre des nchsten Headers
		uint32_zip7		uNextHeaderCrc;					// CRC32 vom nchsten Header
		}Zip7Header;

	#pragma pack(pop)

	typedef struct
		{
		uint64_zip7		uPackSize;
		uint32_zip7		uStreamCrc;
		}Zip7PackItem;

	typedef struct
		{
		uint64_zip7		uDataOffset;
		uint32_zip7		uStreamCount;
		Zip7PackItem	aEntries[1];
		}Zip7PackInfo;

	typedef struct
		{
		uint32_zip7		uCoderCount;
		uint32_zip7		uInpStreams;
		uint32_zip7		uOutStreams;
		uint32_zip7		uPackStreams;
		uint32_zip7		uBindPairs;
		uint32_zip7		uCrcValue;
		uint8_zip7		aProperties[ZIP7_MAX_CODERS*ZIP7_MAX_PROPERTY];
		uint32_zip7		aPropSizes [ZIP7_MAX_CODERS ];
		uint64_zip7		aCoders    [ZIP7_MAX_CODERS ];
		uint64_zip7		aUnpackSize[ZIP7_MAX_STREAMS];
		uint32_zip7		aUnpackCrc [ZIP7_MAX_STREAMS];
		uint32_zip7		uUnpackStreams;
		}Zip7Folder;

	typedef struct
		{
		uint32_zip7		uFolderCount;
		uint32_zip7		uSubStreams;
		uint32_zip7	   	uTempVectorSize;
		Zip7Folder		aFolders[1];
		}Zip7UnpackInfo;

	typedef struct
		{
		uint64_zip7		uFileSize;
		uint64_zip7		uTimeWrite;
		uint64_zip7		uTimeCreate;
		uint64_zip7		uTimeAccess;
		uint32_zip7		uAttributes;
		uint32_zip7		uCrcValue;
		wchar_t			cName[256];
		}Zip7File;

	unsigned char 	   *m_pZp7Next;				// Zeiger auf Zip-7 Folder-Daten
	unsigned			m_uZp7NextPos;			// Zeiger auf Position in Folder-Daten
	unsigned			m_uZp7NextSize;			// Zeiger auf Gre der Folder-Daten
	aint64				m_iZp7Done;				// Gelesen aus Z7-Datei
	aint64				m_iZp7Size;				// Gre det Zip-7 Datei
	unsigned			m_uZp7FileCount;		// Anzahl der gefunden Dateien
	unsigned			m_uZp7EmptyStream;		// Anzahl der leeren Streams
	unsigned			m_uZp7FilePos;			// Nchste zu schreibende Datei
	uint8_zip7		    m_uZp7VecSize;			// Bit-Array Gre Vektoren
	uint8_zip7		   *m_pZp7Vector;			// Bit-Array fr Vectoren
	uint64_zip7		   *m_pZp7StreamSize;		// Temporrer Speicher fr Dateigre
	uint32_zip7		   *m_pZp7StreamCrc;		// Temporrer Speicher fr Datei-CRC
	Zip7File		  **m_pZp7Files;			// Zeiger auf 7ZIP-Dateiliste
	Zip7UnpackInfo	   *m_pUnpackAlloc;			// Zeiger der freigeben werden muss	
	Zip7PackInfo	   *m_pPackAlloc;			// Zeiger der freigeben werden muss	
	void			   *m_pDecoder;				// Aktueller Decoder
	unsigned 			m_uWriteRest;			// Rest der Datei zu schreiben
	unsigned 			m_uWriteCrc;			// Ist die CRC von der aktuellen Datei
	uint64_zip7			m_uWritePos;			// Aktuelle Schreibposition
	uint64_zip7			m_uAesMax;				// Maximale Ausgabelnge fr AES
	zchar				m_cTarName[MAX_PATH];	// Name der temporren TAR-Datei
	unsigned 			m_sAes[72];				// Daten fr AES256 Verschlsselung
	unsigned char		m_aAesTemp[16];			// Block-Puffer fr AES
	unsigned			m_uAesRest;				// Bytes im Block
	unsigned			m_uFilterBufferPos;		// Position im BCJ-Filter
	unsigned			m_uFilterPrevMask;		// Maske fr BCJ-Filter
	unsigned 			m_uFilterOffset;		// Offset in der Ausgabe
	uint64_zip7			m_uFilterSize;			// Gre der gesamten Ausgabe
	unsigned char 	   *m_pFilterBuffer;		// Zeiger auf den Ausgabepuffer
	unsigned char 	   *m_pSpecialMem;			// Ausgabespeicher fr Spezialfilter 
	unsigned			m_uSpecialSize;			// Gre der gesamten Ausgabe

	aint64				m_iDoneNow;				// Aktueller Verlauf
	aint64				m_iDoneMax;				// Maximaler Verlauf
	aint64				m_iDoneBase;			// Maximaler Verlauf in 7-Zip
	int 				m_iDoneMode;			// Verlaufmodus 0=normal 1=Mit Offset 2=Open mit Huffman 3=Close mit Decoder
	


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
public:

						CZp7File();
	virtual		       ~CZp7File();

	virtual	int			Close  ();				// Datei schlieen
	virtual	aint64		Done   (aint64 *pMax);	// Verlaufsabfrage


protected:

	virtual	int			ArchOpen	(int iLevel=9);
	int					MakeHeader	(void *pTarData,int iMode,const char *pName,aint64 iSize,unsigned uTime);
	int					InitAes     (const unsigned char *pProperties,int iPropSize);
	static int _cdecl 	WriteToTar	(void *pParam,const char *pMem,int iLen);
	static int _cdecl 	DeltaFilter	(void *pParam,const char *pMem,int iLen);
	static int _cdecl 	BcjFilter   (void *pParam,const char *pMem,int iLen);
	static int _cdecl 	AesFilter   (void *pParam,      char *pMem,int iLen);
	static int _cdecl 	IsMainCoder (uint64_zip7 uCodec);



//******************** Funktionen fr Next-Header *****************************


	inline void GetByte(unsigned char &ucValue)
							{
							if(m_uZp7NextPos>=m_uZp7NextSize)throw CAF_ERROR_FORM;
							ucValue = m_pZp7Next[m_uZp7NextPos];
							m_uZp7NextPos++;
							}

	inline void GetWord(unsigned short &usValue)
							{
							if(m_uZp7NextPos+2>m_uZp7NextSize)throw CAF_ERROR_FORM;
							usValue = *(short*)(m_pZp7Next+m_uZp7NextPos);
							m_uZp7NextPos+=2;
							}

	inline void GetDWord(unsigned &uValue)
							{
							if(m_uZp7NextPos+4>m_uZp7NextSize)throw CAF_ERROR_FORM;
							uValue = *(unsigned*)(m_pZp7Next+m_uZp7NextPos);
							m_uZp7NextPos+=4;
							}

	inline void GetQWord(unsigned aint64 &uValue)
							{
							if(m_uZp7NextPos+8>m_uZp7NextSize)throw CAF_ERROR_FORM;
							uValue = *(aint64*)(m_pZp7Next+m_uZp7NextPos);
							m_uZp7NextPos+=8;
							}

	inline void GetData(void *pData,unsigned uLen)
							{
							if(m_uZp7NextPos+uLen>m_uZp7NextSize)throw CAF_ERROR_FORM;
							memcpy(pData,m_pZp7Next+m_uZp7NextPos,uLen);
							m_uZp7NextPos+=uLen;
							}

	inline uint32_zip7 Zip7ReadNumber32()
							{
							uint64_zip7		uValue;
							uValue = Zip7ReadNumber64();
							if(uValue>>32)throw CAF_ERROR_FORM;
							return (uint32_zip7)uValue;
							}

	inline void Zip7SkipData(uint64_zip7 uSize)
							{
							   m_uZp7NextPos += (int)uSize;
							if(m_uZp7NextPos>m_uZp7NextSize)throw CAF_ERROR_FORM;
							}
	
	uint64_zip7 		Zip7ReadNumber64      ();
	uint64_zip7 		Zip7ReadBigEndian64   (int iSize);
	inline uint8_zip7  *Zip7AllocVector       (unsigned uCount);
	uint64_zip7			Zip7Decode			  (Zip7PackInfo *pPack,Zip7UnpackInfo *pUnpack,uint64_zip7 uSize=0,unsigned char *pMem=0);
	void 				Zip7ReadVector        (unsigned uSize,uint8_zip7 *pData=0,unsigned uMax=0xFFFFFFFF);
	void 				Zip7ReadBoolVector    (unsigned uSize,uint8_zip7 *pData=0,unsigned uMax=0xFFFFFFFF);
	void 				Zip7WaitId		      (unsigned uStop);
	int 				Zip7ReadFolder		  (Zip7Folder      *pFolder);
	int 				Zip7ReadPackInfo      (Zip7PackInfo   **pInfo);
	int 				Zip7ReadUnpackInfo    (Zip7UnpackInfo **pInfo);
	int 				Zip7ReadSubStreamsInfo(Zip7UnpackInfo  *pInfo);
	int 				Zip7ReadFileInfo      (Zip7UnpackInfo  *pInfo);

};

#endif



