//*****************************************************************************
//*
//*
//*		AsnDecoder.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2018
//
#include	<stdio.h>


#define 	ASN_MAX_LEVELS		20


typedef struct
	{
	unsigned char	ucTagByte;							// Ist das TAG Byte
	unsigned		uOffset;							// Offset im Datensatz
	unsigned		uLength;							// Lnge des Tags in Byte
	unsigned		uStop;								// Ende des Seggmentes
	}AsnTag;

//*****************************************************************************
//*
//*		AsnDecoder
//*
//*****************************************************************************
//	Decodiert binre ASN Daten.
//	pData		: Zeiger auf die Daten
//	iSize		: Gre der Daten in Byte
//	Ergibt die Anzahl der dekodierten Tags bzw. eine Fehlernummer kleiner 0		
//					-1 = Falscher ASN Datensatz
//					-2 = Datensatz zu klein	
int AsnDecoder(const unsigned char *pData,int iSize)
{
AsnTag			aTags[ASN_MAX_LEVELS];
unsigned char	ucTagByte;
unsigned		uOffset;
unsigned		uLength;
unsigned		uCount;
unsigned		uBytes;
unsigned 		uLevel;
unsigned		uStop;
unsigned		uTagId;



	if(iSize<0 || !pData)return -1;						// Prfe Gre vom Datensatz


	uStop	= iSize;
	uOffset = 0;
	uCount	= 0;
	uLevel	= 0;

	aTags[0].uStop	= uStop;


	for(;;)												// Hauptschleife
		{
		if(uOffset>=uStop)								// Eine Ebene hher
			{
			if(uLevel==0)break;
			
			uLevel--;

			uStop	 = aTags[uLevel].uStop;
			uOffset	 = aTags[uLevel].uOffset;
			uOffset += aTags[uLevel].uLength;

			continue;
			}


		ucTagByte	= pData[uOffset];
		uTagId		= ucTagByte&0x1F;

		uOffset++;


		if(uTagId==0x1F)								// Eine Kette von 7-Bit-Ocets
			{
			for(uBytes=0;;uBytes++)
				{
				if(uOffset+uBytes>=uStop)
					{
					return -2;
					}

				if(!(pData[uOffset+uBytes]&0x80))		// Ende erreicht
					{
					uBytes++;
					break;
					}
				}	
			
			// REM AZ Bytes zu Kette zusammen fgen
			}
		else{
			if(uOffset>=uStop)							// Prfe Offset	
				{
				return -2;
				}
			
			uLength  = pData[uOffset];
			uOffset += 1;

			if(uLength&0x80)							// Lnge folgt mit N Bytes
				{
				uBytes = uLength&0x7F;

				if(uOffset+uBytes>uStop || uBytes>4)
					{
					return 2;
					}

				for(uLength=0;uBytes>0;uBytes--)		// Lnge einlesen
					{
					uLength <<= 8;
					uLength  |= pData[uOffset];
					uOffset  += 1;
					}
				}
			
			if(uOffset+uLength>uStop)					// Passen alle Daten in den Datensatz
				{
				return -2;
				}
			}

			
		aTags[uLevel].ucTagByte	= ucTagByte;
		aTags[uLevel].uOffset	= uOffset;
		aTags[uLevel].uLength	= uLength;

		uCount++;

printf("%.*s off=%04u len=%04u tag=%02u\n",uLevel*2,"                                            ",uOffset,uLength,uTagId);

		//if(uTagId==16 || uTagId==16 || uTagId==8)		// Neue Ebene
		if((ucTagByte&0x20) && uLength>0)
			{
			uLevel++;
			uStop	= uOffset+uLength;
			aTags[uLevel].uStop	= uStop;
			continue;
			}


		uOffset += uLength;
		}







		


return uCount;
}
