//******************************************************************************
//*
//*     BitFunctionsIarThumb.s
//*
//******************************************************************************
//
//  BitFunctions for the IAR comiler in the tump mode
//

    SECTION .text:CODE:NOROOT(4)
    CODE16
    THUMB

    PUBLIC bit_find_0_thumb2
    PUBLIC bit_find_1_thumb2
    PUBLIC bit_find_0o_thumb2
    PUBLIC bit_find_1o_thumb2
    PUBLIC bit_find_0x_thumb2
    PUBLIC bit_find_1x_thumb2
    PUBLIC bit_find_0xo_thumb2
    PUBLIC bit_find_1xo_thumb2


//*****************************************************************************
//*
//*		bit_find_0
//*
//*****************************************************************************
//	Sucht das erste gelschte Bit in einem Speicherbereich.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_0_thumb2:

    MOV		r2,r1						// r2(pos) = uCount
    ANDS 	r3,r0,#3					// r3      = pMem&3;
    BEQ		next_a

    PUSH	{r4}						// Register retten
    MOV	    r4,#-1						// r4(mask)= 0xFFFFFFFF
    LSL		r3,r3,#3					// r3 	  *= 8
    LSL		r4,r4,r3					// mask  <<= (pMem&3)*8
    ADD		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    BICS	r3,r4,r3					// DWORD  &= ~DWORD&mask
    POP		{r4}						// Register wieder herstellen
    BNE     find_a					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_a						// if(pos<=0)...

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
next_a:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    MVNS    r3,r3						// DWORD = ~DWORD
    BNE     find_a					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT		next_a						// if(pos<=0)...
stop_a:
    MOV		r0,r1						// return uCount;
    BLX		lr							//		*
find_a:
    RBIT	r3,r3						// Bitreihenfolge umkehren
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_a						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    BLX		lr							//		*

	ALIGNROM 4

//*****************************************************************************
//*
//*		bit_find_0
//*
//*****************************************************************************
//	Sucht das nchste gelschte Bit in einem Speicherbereich ab einem Offset.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	[r2] uOffset	: Ist die Bitposition ab der gesucht wird
//	[r0] Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_0o_thumb2:

    PUSH	{r4}						// Register retten
    CMP		r2,r1						// if(uOffset>=uCount)...
    BHS		stop_b						// 			*
    UBFX    r3,r2,#0,#3					// r3      = BitOffset
    ADD		r0,r0,r2,LSR #3				// pMem   += uOffset/8
    MOV     r4,#-1						// r4(mask)= 0xFFFFFFFF
    BFC		r2,#0,#3					// uOffset&= ~3
    LSL		r4,r4,r3					// r4(mask)= (1<<uOffset&7)-1

    UBFX 	r3,r0,#0,#2					// r3      =  pMem&3;
    LSL		r3,r3,#3					// r3 	  *=  8
    LSL		r4,r4,r3					// mask  <<= (pMem&3)*8
    SUB		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3
    SUB		r2,r1,r2					// r2(pos) = uCount-r2

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
    // R4 = Maske fr estes DWORD

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    BICS	r3,r4,r3					// DWORD  &= ~DWORD&mask
    BNE     find_b				    	// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_b						// if(pos<=0)...
next_b:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    MVNS    r3,r3						// DWORD = ~DWORD
    BNE     find_b					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT     next_b						// if(pos<=0)...
stop_b:
    MOV		r0,r1						// return uCount;
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*
find_b:
    RBIT	r3,r3						// Bitreihenfolge umkehren
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_b						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*

	ALIGNROM 4

//*****************************************************************************
//*
//*		bit_find_1
//*
//*****************************************************************************
//	Sucht das erste gesetzte Bit in einem Speicherbereich.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_1_thumb2:

    MOV		r2,r1						// r2(pos) = uCount
    ANDS 	r3,r0,#3					// r3      = pMem&3;
    BEQ		next_c

    PUSH	{r4}						// Register retten
    MOV	    r4,#-1						// r4(mask)= 0xFFFFFFFF
    LSL		r3,r3,#3					// r3 	  *= 8
    LSL		r4,r4,r3					// mask  <<= (pMem&3)*8
    ADD		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    ANDS	r3,r3,r4					// DWORD  &= mask
    POP		{r4}						// Register wieder herstellen
    BNE     find_c					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_c						// if(pos<=0)...

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
next_c:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    CBNZ	r3,find_c					// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT		next_c						// if(pos<=0)...
stop_c:
    MOV		r0,r1						// return uCount;
    BLX		lr							//		*
find_c:
    RBIT	r3,r3						// Bitreihenfolge umkehren
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_c						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    BLX		lr							//		*

	ALIGNROM 4

//*****************************************************************************
//*
//*		bit_find_1
//*
//*****************************************************************************
//	Sucht das nchste gesetzte Bit in einem Speicherbereich ab einem Offset.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	[r2] uOffset	: Ist die Bitposition ab der gesucht wird
//	[r0] Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_1o_thumb2:

    PUSH	{r4}						// Register retten
    CMP		r2,r1						// if(uOffset>=uCount)...
    BHS		stop_d						// 			*
    UBFX 	r3,r2,#0,#3					// r3      = BitOffset
    ADD		r0,r0,r2,LSR #3				// pMem   += uOffset/8
    MOV	    r4,#-1						// r4(mask)= 0xFFFFFFFF
    BFC		r2,#0,#3					// uOffset&= ~3
    LSL		r4,r4,r3					// r4(mask)= (1<<uOffset&7)-1

    UBFX 	r3,r0,#0,#2					// r3      =  pMem&3;
    LSL		r3,r3,#3					// r3 	  *=  8
    LSL		r4,r4,r3					// mask  <<= (pMem&3)*8
    SUB		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3
    SUB		r2,r1,r2					// r2(pos) = uCount-r2

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
    // R4 = Maske fr estes DWORD

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    ANDS	r3,r3,r4					// DWORD  &= mask
    BNE     find_d	    				// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_d						// if(pos<=0)...
next_d:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__==0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    CBNZ	r3,find_d					// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT		next_d						// if(pos<=0)...
stop_d:
    MOV		r0,r1						// return uCount;
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*
find_d:
    RBIT	r3,r3						// Bitreihenfolge umkehren
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_d						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*

	ALIGNROM 4




//*****************************************************************************
//*
//*		bit_find_0x
//*
//*****************************************************************************
//	Sucht das erste gelschte Bit in einem Speicherbereich.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_0x_thumb2:

    MOV		r2,r1						// r2(pos) = uCount
    ANDS 	r3,r0,#3					// r3      = pMem&3;
    BEQ		next_e

    PUSH	{r4}						// Register retten
    MOV	    r4,#-1						// r4(mask)= 0xFFFFFFFF
    LSL		r3,r3,#3					// r3 	  *= 8
    LSR		r4,r4,r3					// mask  >>= (pMem&3)*8
    ADD		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    BICS	r3,r4,r3					// DWORD  &= ~DWORD&mask
    POP		{r4}						// Register wieder herstellen
    BNE     find_e					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_e						// if(pos<=0)...

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
next_e:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    MVNS    r3,r3						// DWORD = ~DWORD
    BNE     find_e					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT		next_e						// if(pos<=0)...
stop_e:
    MOV		r0,r1						// return uCount;
    BLX		lr							//		*
find_e:
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_e						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    BLX		lr							//		*

	ALIGNROM 4

//*****************************************************************************
//*
//*		bit_find_0x
//*
//*****************************************************************************
//	Sucht das nchste gelschte Bit in einem Speicherbereich ab einem Offset.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	[r2] uOffset	: Ist die Bitposition ab der gesucht wird
//	[r0] Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_0xo_thumb2:

    PUSH	{r4}						// Register retten
    CMP		r2,r1						// if(uOffset>=uCount)...
    BHS		stop_f						// 			*
    UBFX    r3,r2,#0,#3					// r3      = BitOffset
    ADD		r0,r0,r2,LSR #3				// pMem   += uOffset/8
    MOV     r4,#-1						// r4(mask)= 0xFFFFFFFF
    BFC		r2,#0,#3					// uOffset&= ~3
    LSR		r4,r4,r3					// r4(mask)= 0xFFFFFFFF>>(uOffset&7)

    UBFX 	r3,r0,#0,#2					// r3      =  pMem&3;
    LSL		r3,r3,#3					// r3 	  *=  8
    LSR		r4,r4,r3					// mask  >>= (pMem&3)*8
    SUB		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3
    SUB		r2,r1,r2					// r2(pos) = uCount-r2

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
    // R4 = Maske fr estes DWORD

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    BICS	r3,r4,r3					// DWORD  &= ~DWORD&mask
    BNE     find_f				    	// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_f						// if(pos<=0)...
next_f:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    MVNS    r3,r3						// DWORD = ~DWORD
    BNE     find_f					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT     next_f						// if(pos<=0)...
stop_f:
    MOV		r0,r1						// return uCount;
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*
find_f:
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_f						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*

	ALIGNROM 4

//*****************************************************************************
//*
//*		bit_find_1x
//*
//*****************************************************************************
//	Sucht das erste gesetzte Bit in einem Speicherbereich.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_1x_thumb2:

    MOV		r2,r1						// r2(pos) = uCount
    ANDS 	r3,r0,#3					// r3      = pMem&3;
    BEQ		next_g

    PUSH	{r4}						// Register retten
    MOV	    r4,#-1						// r4(mask)= 0xFFFFFFFF
    LSL		r3,r3,#3					// r3 	  *= 8
    LSR		r4,r4,r3					// mask  >>= (pMem&3)*8
    ADD		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    ANDS	r3,r3,r4					// DWORD  &= mask
    POP		{r4}						// Register wieder herstellen
    BNE     find_g					    // if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_g						// if(pos<=0)...

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
next_g:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    CBNZ	r3,find_g					// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT		next_g						// if(pos<=0)...
stop_g:
    MOV		r0,r1						// return uCount;
    BLX		lr							//		*
find_g:
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_g						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    BLX		lr							//		*

	ALIGNROM 4

//*****************************************************************************
//*
//*		bit_find_1x
//*
//*****************************************************************************
//	Sucht das nchste gesetzte Bit in einem Speicherbereich ab einem Offset.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	[r0] pMem		: Ist der Zeiger auf den Speicherbereich.
//	[r1] uCount		: Ist die Gre der Speicherbereiches in Bits.
//	[r2] uOffset	: Ist die Bitposition ab der gesucht wird
//	[r0] Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.
bit_find_1xo_thumb2:

    PUSH	{r4}						// Register retten
    CMP		r2,r1						// if(uOffset>=uCount)...
    BHS		stop_h						// 			*
    UBFX 	r3,r2,#0,#3					// r3      = BitOffset
    ADD		r0,r0,r2,LSR #3				// pMem   += uOffset/8
    MOV	    r4,#-1						// r4(mask)= 0xFFFFFFFF
    BFC		r2,#0,#3					// uOffset&= ~3
    LSR		r4,r4,r3					// r4(mask)= 0xFFFFFFFF>>(uOffset&7)

    UBFX 	r3,r0,#0,#2					// r3      =  pMem&3;
    LSL		r3,r3,#3					// r3 	  *=  8
    LSR		r4,r4,r3					// mask  >>= (pMem&3)*8
    SUB		r2,r2,r3					// r2(pos)-= (pMem&3)*8
    BFC		r0,#0,#2					// pMem   &= ~3
    SUB		r2,r1,r2					// r2(pos) = uCount-r2

    // R0 = pMem auf Offset und Align 4 abgerundet
    // R1 = uCount (Gesamtanzahl der Bits)
    // R2 = pos (Bits von R0 bis zum Ende)
    // R4 = Maske fr estes DWORD

    LDR		r3,[r0],#4					// Lade erste DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    ANDS	r3,r3,r4					// DWORD  &= mask
    BNE     find_h	    				// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BLE		stop_h						// if(pos<=0)...
next_h:
    LDR		r3,[r0],#4					// Lade nchtes DWORD
	#if __LITTLE_ENDIAN__!=0
	REV		r3,r3						// Big zu Little-Endian
	#endif
    CBNZ	r3,find_h					// if(DWORD==0)...
    SUBS	r2,r2,#32					// pos    -= 32
    BGT		next_h						// if(pos<=0)...
stop_h:
    MOV		r0,r1						// return uCount;
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*
find_h:
    CLZ		r3,r3						// r3      = erstes high Bit
    SUBS	r2,r2,r3                    // pos    -= erstes high Bit
    BLT    	stop_h						// if(ret<uCount)...
    SUB		r0,r1,r2					// ret     = uCount-pos
    POP		{r4}						// Register wieder herstellen
    BLX		lr							//		*

	ALIGNROM 4
	END



