//*****************************************************************************
//*
//*
//*		BitFunctions.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Funktionen:
//
//   int  bit_set				: Bit in Speicherbereich setzen
//   int  bit_clear				: Bit in Speicherbereich lschen
//   int  bit_change			: Bit in Speicherbereich invertiren
//   int  bit_read				: Bit in Speicherbereich lesen
//   int  bit_write				: Bit in Speicherbereich schreiben
//   int  bit_rol				: Bits durch rollen	links herum
//   int  bit_ror				: Bits durch rollen rechts herum
//   int  bit_find_0			: Erstes  gelschtes Bit suchen
//   int  bit_find_1			: Erstes  gesetzes   Bit suchen
//   int  bit_swap				: Tauscht Bits von unten nach oben aus
//   int  bit_swap_qw			: Tauscht Bits von unten nach oben aus (64 Bit)
//   int  bit_shift				: Bits nach frei   verschieben
//   int  bit_shift_l			: Bits nach links  verschieben mit berlaufprfung
//   int  bit_shift_r			: Bits nach rechts verschieben mit berlaufprfung
//   int  bit_shift_a			: Bits nach rechts verschieben mit berlaufprfung (arimetisch)
//   int  btol_qw				: Ergibt Big-Endial zu Little-Endian fr QWORD
//   int  btol_dw				: Ergibt Big-Endial zu Little-Endian fr DWORD
//   int  btol_w				: Ergibt Big-Endial zu Little-Endian fr WORD
//   int  btol					: Ergibt Big-Endial zu Little-Endian
//
//	Makros:
//
//		[Cpu+Compiler]		[Optinale Makros]
//
//		BIT_MSC_X64			BIT_MSC_INSTR
//		BIT_MSC_X86			BIT_MSC_INSTR oder BIT_MSC_VC60
//		BIT_MSC_ARM64
//		BIT_MSC_ARM			BIT_MSC_THUMB
//
//		BIT_GNU_X64
//		BIT_GNU_X86
//		BIT_GNU_ARM64
//		BIT_GNU_ARM			BIT_GNU_THUMB
//
//		BIT_IAR_ARM			BIT_IAR_THUMB
//
//		BIT_KEIL_ARM		BIT_KEIL_THUMB
//
//		BIT_USE_IRQ			<DISABLE_IRQ statt LDREX auf ARM CPU's verwenden>
//
#ifndef __BIT_FUNCTIONS_H__
#define __BIT_FUNCTIONS_H__

#if      _MSC_VER >= 1000
#pragma	  warning( push )
#pragma	  warning( disable :4302)
#pragma	  warning( disable :4311)
#pragma	  warning( disable :4312)
#pragma	  once
#endif

#ifdef	  GLOBAL_CONFIG
#define	  BITFUNCTIONS_CPP
#include  <GlobalConfig.h>
#undef	  BITFUNCTIONS_CPP
#endif

#include  <string.h>
#include  <stdlib.h>


#ifndef	  BIT_USE_LITTLE_ENDIAN
#define	  BIT_USE_LITTLE_ENDIAN		1
#endif

#ifndef	  BIT_USE_BIG_ENDIAN
#define	  BIT_USE_BIG_ENDIAN		0
#endif



//******************** No inline Assembler ************************************
#if defined(NO_BIT_ASM)

	#ifdef _MSC_VER

		#ifndef	  ibt64
		#define   ibt64			__int64
		#endif

		#ifndef	  BIT_INLINE
		#define   BIT_INLINE	__inline
		#endif

		#ifndef	  BIT_STATIC
		#define   BIT_STATIC	static
		#endif

	#elif defined(__GNUC__)

		#ifndef	 _cdecl
		#define	 _cdecl
		#endif

		#ifndef	__cdecl
		#define	__cdecl
		#endif

		#ifndef	  ibt64
		#define   ibt64			long long
		#endif

		#ifndef	  BIT_INLINE
		#define   BIT_INLINE	inline
		#endif

		#ifndef	  BIT_STATIC
		#define   BIT_STATIC	static
		#endif

	#elif defined(__CC_ARM)

		#ifndef	 _cdecl
		#define	 _cdecl
		#endif
		#ifndef	__cdecl
		#define	__cdecl
		#endif

		#ifndef	  ibt64
		#define	  ibt64			long long
		#endif

		#ifndef	  BIT_INLINE
		#define	  BIT_INLINE	__inline
		#endif

		#ifndef	  BIT_STATIC
		#define	  BIT_STATIC	static
		#endif

	#elif defined(__ICCARM__)

		#ifndef	 _cdecl
		#define	 _cdecl
		#endif
		#ifndef	__cdecl
		#define	__cdecl
		#endif

		#ifndef	  ibt64
		#define	  ibt64			long long
		#endif

		#ifndef	  BIT_INLINE
		#define	  BIT_INLINE	inline
		#endif

		#ifndef	  BIT_STATIC
		#define	  BIT_STATIC	static
		#endif

	#else

		#error    unknown compiler

	#endif

//******************** Microsoft Compiler *************************************
#elif _MSC_VER

	#ifndef	ibt64
	#define ibt64		__int64
	#endif

	#ifndef	BIT_INLINE
	#define BIT_INLINE	__inline
	#endif

	#ifndef	BIT_STATIC
	#define BIT_STATIC	static
	#endif

	#if defined(_M_X64) || defined(_M_AMD64)

		#define		BIT_MSC_INSTR
		#define		BIT_MSC_X64
		#include	<intrin.h>

	#elif defined(_M_IX86) && (_MSC_VER>=1300)

		#define		BIT_MSC_INSTR
		#define		BIT_MSC_X86
		#include	<intrin.h>

	#elif defined(_M_IX86) && (_MSC_VER<1300)

		#define		BIT_MSC_VC60
		#define		BIT_MSC_X86
		#pragma		warning(push)
		#pragma		warning(disable:4035)

	#elif defined(_M_ARM64)

		#define		BIT_MSC_ARM64
		#include	<immintrin.h>
		#include	<intrin.h>

	#elif defined(_M_ARM)

		#define		BIT_MSC_ARM
		#include	<intrin.h>

	#elif defined(_M_ARMT)

		#define		BIT_MSC_ARM
		#define		BIT_MSC_THUMB
		#include	<intrin.h>

	#else

		#error	unkonwn cpu type

	#endif

//******************** GNU GCC Compiler ***************************************
#elif defined(__GNUC__)

	#ifndef	 _cdecl
	#define	 _cdecl
	#endif

	#ifndef	__cdecl
	#define	__cdecl
	#endif

	#ifndef	  ibt64
	#define   ibt64			long long
	#endif

	#ifndef	  BIT_INLINE
	#define   BIT_INLINE	inline
	#endif

	#ifndef	  BIT_STATIC
	#define   BIT_STATIC	static
	#endif

	#if	(__GNUC__<4) || (__GNUC__==4 && __GNUC_MINOR__<1)

		#define	  BIT_ADDR(x)	"=m" (*(volatile long *) (x))

	#else

		#define	  BIT_ADDR(x)	"+m" (*(volatile long *) (x))

	#endif

	#if (__GNUC__>4) || (__GNUC__==4 && __GNUC_MINOR__>=3)

		#define		BIT_GNU_4_3

	#endif

	#if defined(__x86_64__) || defined(__amd64__)

		#define		BIT_GNU_X64

	#elif defined(__i386__) || defined(__i486__) || defined(__i586__)

		#define		BIT_GNU_X86

	#elif defined(__arm__) || defined(__aarch64__)

		#ifdef    __thumb__
		#define		BIT_GNU_THUMB
		#endif

		#ifdef	  __aarch64__)
		#define		BIT_GNU_ARM64
		#else
		#define		BIT_GNU_ARM
		#endif

		#ifdef	  __BIG_ENDIAN__
		#undef		BIT_USE_LITTLE_ENDIAN
		#undef		BIT_USE_BIG_ENDIAN
		#define		BIT_USE_LITTLE_ENDIAN	0
		#define		BIT_USE_BIG_ENDIAN  	1
		#endif
		
		#define		__ASM	asm
		#include	<inttypes.h>

		#if defined(__TARGET_CPU_CORTEX_M0) || defined(__TARGET_CPU_CORTEX_M3) || defined(__TARGET_CPU_CORTEX_M4) || defined(__TARGET_CPU_CORTEX_M7)

			__attribute__((always_inline)) static inline  uint32_t bit_get_xpsr(void)
				{
				uint32_t uRet;
				__ASM volatile ("mrs %0,XPSR":"=r" (uRet):) ;
				return uRet;
				}

			#define	B_READ_IRQ()		((bit_get_xpsr()&0xFF)

		#else

			__attribute__((always_inline)) static inline  uint32_t bit_get_cpsr(void)
				{
				uint32_t uRet;
				__ASM volatile ("mrs %0,CPSR":"=r" (uRet):) ;
				return uRet;
				}

			#define	B_READ_IRQ()		((bit_get_cpsr()>>7)&1)

		#endif

		__attribute__((always_inline)) static inline  void bit_enable_irq(void)
			{
			__ASM volatile ("cpsie i" : : : "memory");
			}

		__attribute__((always_inline)) static inline  void bit_disable_irq(void)
			{
			__ASM volatile ("cpsid i" : : : "memory");
			}

		__attribute__((always_inline)) static inline  uint32_t bit_strexb(uint8_t uValue, volatile uint8_t *pAddr)
			{
			uint32_t uResult;
			__ASM volatile ("strexb %0, %2, %1" : "=&r" (uResult), "=Q" (*pAddr) : "r" ((uint32_t)uValue));
			return(uResult);
			}

		__attribute__((always_inline)) static inline  uint8_t bit_ldrexb(volatile uint8_t *pAddr)
			{
			uint32_t uResult;
			__ASM volatile ("ldrexb %0, %1" : "=r" (uResult) : "Q" (*pAddr));
			return ((uint8_t) uResult);
			}

		__attribute__((always_inline)) static inline  uint32_t bit_strexh(uint16_t uValue, volatile uint16_t *pAddr)
			{
			uint32_t uResult;
			__ASM volatile ("strexh %0, %2, %1" : "=&r" (uResult), "=Q" (*pAddr) : "r" ((uint32_t)uValue));
			return(uResult);
			}

		__attribute__((always_inline)) static inline  uint16_t bit_ldrexh(volatile uint16_t *pAddr)
			{
			uint32_t uResult;
			__ASM volatile ("ldrexh %0, %1" : "=r" (uResult) : "Q" (*pAddr));
			return ((uint16_t) uResult);
			}

		__attribute__((always_inline)) static inline  uint32_t bit_strex(uint32_t uValue, volatile uint32_t *pAddr)
			{
			uint32_t uResult;
			__ASM volatile ("strex %0, %2, %1" : "=&r" (uResult), "=Q" (*pAddr) : "r" ((uint32_t)uValue));
			return(uResult);
			}

		__attribute__((always_inline)) static inline  uint32_t bit_ldrex(volatile uint32_t *pAddr)
			{
			uint32_t uResult;
			__ASM volatile ("ldrex %0, %1" : "=r" (uResult) : "Q" (*pAddr));
			return ((uint32_t) uResult);
			}

		#define		B_DISABLE_IRQ()		bit_disable_irq()
		#define		B_ENABLE_IRQ()		bit_enable_irq()
		#define 	B_LDREXB			bit_ldrexb
		#define 	B_LDREXH			bit_ldrexh
		#define 	B_LDREX 			bit_ldrex
		#define 	B_STREXB			bit_strexb
		#define 	B_STREXH			bit_strexh
		#define 	B_STREX 			bit_strex

	#else

		#error	unknown cpu type

	#endif

//******************** Keil/Realview Compiler *********************************
#elif defined(__CC_ARM)

	#ifndef	   _cdecl
	#define	   _cdecl
	#endif
	#ifndef	  __cdecl
	#define	  __cdecl
	#endif

	#ifndef		ibt64
	#define		ibt64		long long
	#endif

	#ifndef		BIT_INLINE
	#define		BIT_INLINE	__inline
	#endif

	#ifndef		BIT_STATIC
	#define		BIT_STATIC	static
	#endif

    #ifdef    __thumb__
    #define 	BIT_KEIL_THUMB
    #endif

	#define		BIT_KEIL_ARM

	#ifdef	  __BIG_ENDIAN
	#undef		BIT_USE_LITTLE_ENDIAN
	#undef		BIT_USE_BIG_ENDIAN
	#define		BIT_USE_LITTLE_ENDIAN	0
	#define		BIT_USE_BIG_ENDIAN  	1
	#endif

	#if defined(__TARGET_CPU_CORTEX_M0) || defined(__TARGET_CPU_CORTEX_M3) || defined(__TARGET_CPU_CORTEX_M4) || defined(__TARGET_CPU_CORTEX_M7)

		static __inline unsigned int bit_get_xpsr(void)
			{
			register int	uRet __asm("xpsr");
			return uRet;
			}

		#define	B_READ_IRQ()	((bit_get_xpsr()&0xFF)

	#else

		static __inline unsigned int bit_get_cpsr(void)
			{
			register int	uRet __asm("cpsr");
			return uRet;
			}

		#define	B_READ_IRQ()	((bit_get_cpsr()>>7)&1)

	#endif

	#define		B_DISABLE_IRQ()		__disable_irq()
	#define		B_ENABLE_IRQ()		__enable_irq()
	#define 	B_STREXB(value,ptr) __strex(value,(unsigned char *)(ptr))
	#define 	B_STREXH(value,ptr) __strex(value,(unsigned short*)(ptr))
	#define 	B_STREX(value,ptr)	__strex(value,(unsigned int  *)(ptr))
	#define 	B_LDREXB(ptr)       ((unsigned char )__ldrex((unsigned char *)(ptr)))
	#define 	B_LDREXH(ptr)       ((unsigned short)__ldrex((unsigned short*)(ptr)))
	#define 	B_LDREX(ptr)        ((unsigned int  )__ldrex((unsigned int  *)(ptr)))

//******************** IAR Compiler *******************************************
#elif defined(__ICCARM__)

	#ifndef	   _cdecl
	#define	   _cdecl
	#endif
	#ifndef	  __cdecl
	#define	  __cdecl
	#endif

	#ifndef		ibt64
	#define		ibt64		long long
	#endif

	#ifndef		BIT_INLINE
	#define		BIT_INLINE	inline
	#endif

	#ifndef		BIT_STATIC
	#define		BIT_STATIC	static
	#endif

	#if			__CPU_MODE__==1
	#define 	BIT_IAR_THUMB
	#endif

	#ifdef	  __LITTLE_ENDIAN__
	#if		  __LITTLE_ENDIAN__ == 0
	#undef		BIT_USE_LITTLE_ENDIAN
	#undef		BIT_USE_BIG_ENDIAN
	#define		BIT_USE_LITTLE_ENDIAN	0
	#define		BIT_USE_BIG_ENDIAN  	1
	#endif
	#endif

	#include	<intrinsics.h>

	#if defined(__TARGET_CPU_CORTEX_M0) || defined(__TARGET_CPU_CORTEX_M3) || defined(__TARGET_CPU_CORTEX_M4) || defined(__TARGET_CPU_CORTEX_M7)

		#define	B_READ_IRQ()		((__get_XPSR()&0xFF)

	#else

		#define	B_READ_IRQ()		((__get_CPSR()>>7)&1)

	#endif

	#define		B_DISABLE_IRQ()		__disable_irq()
	#define		B_ENABLE_IRQ()		__enable_irq()
	#define 	B_LDREXB			__LDREXB
	#define 	B_LDREXH			__LDREXH
	#define 	B_LDREX 			__LDREX
	#define 	B_STREXB			__STREXB
	#define 	B_STREXH			__STREXH
	#define 	B_STREX 			__STREX

	#define		BIT_IAR_ARM

	// __CORE__		defines the cpu type
	// __CPU_MODE__	defines the thumb mode (0=none 1=thumb 2=arm32)

//*****************************************************************************
#else

	#error	unknown compiler

#endif



//*****************************************************************************
//*
//*		Allgemeine Definitionen
//*
//*****************************************************************************

void 			_cdecl bit_set     (void *pMem,int iOffset,int iCount);
void 			_cdecl bit_clear   (void *pMem,int iOffset,int iCount);
void 			_cdecl bit_change  (void *pMem,int iOffset,int iCount);
int				_cdecl bit_find_0  (const void *pMem,unsigned uSize,unsigned uOffset);
int				_cdecl bit_find_0x (const void *pMem,unsigned uSize,unsigned uOffset);
int				_cdecl bit_find_1  (const void *pMem,unsigned uSize,unsigned uOffset);
int				_cdecl bit_find_1x (const void *pMem,unsigned uSize,unsigned uOffset);
int             _cdecl bit_count_1 (const void *pMem,unsigned uSize,unsigned uOffset=0);
int             _cdecl bit_count_1x(const void *pMem,unsigned uSize,unsigned uOffset=0);
unsigned ibt64	_cdecl bit_swap_qw (unsigned ibt64 uValue,int iCount);


#if defined(BIT_MSC_X64) || defined(BIT_MSC_ARM64) || defined(BIT_MSC_ARM)

	typedef int (_cdecl *bit_change_proc)(void *pMem,int iOffset);

#endif



//*****************************************************************************
//*
//*		bit_set
//*
//*****************************************************************************
//	Setzt ein Bit in einem Speicherbereich auf 1. Die Funktion ist atomar
//	pMem		: Zeiger auf den Speicherbereich
//	iOffset		: Ist die Nummer des Bits das gesetzt werden soll.
//	Ergibt einen Wert ungleich 0 wenn das alte Bit gesetzt war sonst 0

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_set(void *pMem,int iOffset);

#elif defined(NO_BIT_ASM)

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_set(void *pMem,int iOffset);

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE  int bit_set(void *pMem,int iOffset)
	{
	int iRet;

	asm volatile (
		"lock; bts  %2,%1  \n"
		"sbb        %0,%0  \n"
		:"=r" (iRet), BIT_ADDR(pMem)
		:"Ir" (iOffset)
		:"memory", "cc"
		);

	return  iRet;
	}

#elif defined(BIT_MSC_X64)

	BIT_STATIC BIT_INLINE  int bit_set(void *pMem,int iOffset)
	{
	return _interlockedbittestandset64((__int64*)pMem,iOffset);
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE  int bit_set(void *pMem,int iOffset)
	{
	return _interlockedbittestandset((long*)pMem,iOffset);
	}

#elif defined(BIT_MSC_ARM64)

	BIT_STATIC BIT_INLINE  int bit_set(void *pMem,int iOffset)
	{
	return _interlockedbittestandset64_HLEAcquire((long*)pMem,iOffset);
	}

#elif defined(BIT_MSC_ARM)

	BIT_STATIC BIT_INLINE  int bit_set(void *pMem,int iOffset)
	{
	return _interlockedbittestandset_acq((long*)pMem,iOffset);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE  int bit_set(void *pMem,int iOffset)
	{
	_asm{
		mov	 	eax,pMem
		mov  	ebx,iOffset
		lock 	bts [eax],ebx
		sbb  	eax,eax
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM) || defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE  int bit_set(void *pMem,int iOffset)
	{
	unsigned char  ucMask;
	unsigned char  ucBits;
	unsigned char *pPtr;


	pPtr   = (unsigned char*)pMem+iOffset/8;
	ucMask = 1<<(iOffset&7);

	#ifdef BIT_USE_IRQ

	{
	int iIrq;

	   iIrq=B_READ_IRQ();
	if(iIrq)B_DISABLE_IRQ();

	ucBits = *pPtr;
	*pPtr  = ucBits|ucMask;

	if(iIrq)B_ENABLE_IRQ();
	}

	#else

	for(;;)
		{
		ucBits          = B_LDREXB(pPtr);
		if(!B_STREXB(ucBits|ucMask,pPtr))
			{
			break;
			}
		}

	#endif

	return ucBits&ucMask;
	}

#endif

//*****************************************************************************
//*
//*		bit_clear
//*
//*****************************************************************************
//	Lscht ein Bit in einem Speicherbereich. Die Funktion ist atomar
//	pMem		: Zeiger auf den Speicherbereich
//	iOffset		: Ist die Nummer des Bits das gelscht werden soll.
//	Ergibt einen Wert ungleich 0 wenn das alte Bit gesetzt war sonst 0

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_clear(void *pMem,int iOffset);

#elif defined(NO_BIT_ASM)

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_clear(void *pMem,int iOffset);

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE  int bit_clear(void *pMem,int iOffset)
	{
	int iRet;

	asm volatile (
		"lock; btr  %2,%1  \n"
		"sbb        %0,%0  \n"
		:"=r" (iRet), BIT_ADDR(pMem)
		:"Ir" (iOffset)
		:"memory", "cc"
		);

	return  iRet;
	}

#elif defined(BIT_MSC_X64)

	BIT_STATIC BIT_INLINE  int bit_clear(void *pMem,int iOffset)
	{
	return _interlockedbittestandreset64((__int64*)pMem,iOffset);
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE  int bit_clear(void *pMem,int iOffset)
	{
	return _interlockedbittestandreset((long*)pMem,iOffset);
	}

#elif defined(BIT_MSC_ARM64)

	BIT_STATIC BIT_INLINE  int bit_clear(void *pMem,int iOffset)
	{
	return _interlockedbittestandreset64_HLEAcquire((__int64*)pMem,iOffset);
	}

#elif defined(BIT_MSC_ARM)

	BIT_STATIC BIT_INLINE  int bit_clear(void *pMem,int iOffset)
	{
	return _interlockedbittestandreset_acq((long*)pMem,iOffset);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE  int bit_clear(void *pMem,int iOffset)
	{
	_asm{
		mov	 	eax,pMem
		mov  	ebx,iOffset
		lock 	btr [eax],ebx
		sbb  	eax,eax
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM) || defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE  int bit_clear(void *pMem,int iOffset)
	{
	unsigned char  ucMask;
	unsigned char  ucBits;
	unsigned char *pPtr;


	pPtr   = (unsigned char*)pMem+iOffset/8;
	ucMask = 1<<(iOffset&7);

	#ifdef BIT_USE_IRQ

	{
	int iIrq;

	   iIrq=B_READ_IRQ();
	if(iIrq)B_DISABLE_IRQ();

	ucBits = *pPtr;
	*pPtr  = ucBits&~ucMask;

	if(iIrq)B_ENABLE_IRQ();
	}

	#else

	for(;;)
		{
		ucBits           = B_LDREXB(pPtr);
		if(!B_STREXB(ucBits&~ucMask,pPtr))
			{
			break;
			}
		}

	#endif

	return ucBits&ucMask;
	}

#endif

//*****************************************************************************
//*
//*		bit_change
//*
//*****************************************************************************
//	Invertiert ein Bit in einem Speicherbereich. Die Funktion ist atomar
//	pMem		: Zeiger auf den Speicherbereich
//	iOffset		: Ist die Nummer des Bits das ivertiert werden soll.
//	Ergibt einen Wert ungleich 0 wenn das alte Bit gesetzt war sonst 0

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_change(void *pMem,int iOffset);

#elif defined(NO_BIT_ASM)

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_change(void *pMem,int iOffset);

#elif defined(BIT_MSC_X64)

	extern	unsigned char	aBitChangeX64[];

	BIT_STATIC BIT_INLINE int bit_change(void *pMem,int iOffset)
		{
		return ((bit_change_proc)&aBitChangeX64)(pMem,iOffset);
		}

#elif defined(BIT_MSC_ARM64)

	extern int _cdecl bit_change(void * pMem,int iOffset);

#elif defined(BIT_MSC_THUMB)

	extern	unsigned short 	aBitChangeThumb[];

	BIT_STATIC BIT_INLINE  int bit_change(void *pMem,int iOffset)
		{
		return ((bit_change_proc)&aBitChangeThumb)(pMem,iOffset);
		}

#elif defined(BIT_MSC_ARM)

	extern	unsigned int 	aBitChangeArm[];

	BIT_STATIC BIT_INLINE  int bit_change(void *pMem,int iOffset)
		{
		return ((bit_change_proc)&aBitChangeArm)(pMem,iOffset);
		}
#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE  int bit_change(void *pMem,int iOffset)
	{
	int iRet;

	asm volatile (
		"lock; btc  %2,%1  \n"
		"sbb        %0,%0  \n"
		:"=r" (iRet), BIT_ADDR(pMem)
		:"Ir" (iOffset)
		:"memory", "cc"
		);

	return  iRet;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE  int bit_change(void *pMem,int iOffset)
	{
	_asm{
		mov	 	eax,pMem
		mov  	ebx,iOffset
		lock 	btc [eax],ebx
		sbb  	eax,eax
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM) || defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE  int bit_change(void *pMem,int iOffset)
	{
	unsigned char  ucMask;
	unsigned char  ucBits;
	unsigned char *pPtr;


	pPtr   = (unsigned char*)pMem+iOffset/8;
	ucMask = 1<<(iOffset&7);

	#ifdef BIT_USE_IRQ

	{
	int iIrq;

	   iIrq=B_READ_IRQ();
	if(iIrq)B_DISABLE_IRQ();

	ucBits = *pPtr;
	*pPtr  = ucBits^ucMask;

	if(iIrq)B_ENABLE_IRQ();
	}

	#else

	for(;;)
		{
		ucBits          = B_LDREXB(pPtr);
		if(!B_STREXB(ucBits^ucMask,pPtr))
			{
			break;
			}
		}

	#endif

	return ucBits&ucMask;
	}

#endif

//*****************************************************************************
//*
//*		bit_read
//*
//*****************************************************************************
//	Liet ein Bit in einem Speicherbereich aus. Die Funktion ist atomar
//	pMem		: Zeiger auf den Speicherbereich
//	iOffset		: Ist die Nummer des Bits das gelesen werden soll.
//	Ergibt einen Wert ungleich 0 wenn das Bit gesetzt war sonst 0

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_read(const void *pMem,int iOffset);

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE int bit_read(const void *pMem,int iOffset)
	{
	int iRet;

	asm volatile (
		"bt		%2,%1  \n"
		"sbb    %0,%0  \n"
		:"=r" (iRet), BIT_ADDR(pMem)
		:"Ir" (iOffset)
		:"memory", "cc"
		);

	return  iRet;
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE int bit_read(const void *pMem,int iOffset)
	{
	int iRet;

    asm volatile (
        "movl      %1   ,%%rax  \n"
        "bt        %2,  (%%rax) \n"
        "sbb       %%eax,%%eax  \n"
        : "=a" (iRet)
        : "m"  (pMem) , "c" (iOffset)
        );

	return  iRet;
	}

#elif defined(BIT_MSC_X64) || defined(BIT_MSC_ARM64)

	BIT_STATIC BIT_INLINE int bit_read(const void *pMem,int iOffset)
	{
	return _bittest64((__int64*)pMem,iOffset);
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE int bit_read(const void *pMem,int iOffset)
	{
	return _bittest((long*)pMem,iOffset);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE int bit_read(const void *pMem,int iOffset)
	{
	_asm{
		mov		eax,pMem
		mov		ebx,iOffset
		bt		[eax],ebx
		sbb		eax,eax
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_read(const void *pMem,int iOffset)
	{
	return (((unsigned char*)pMem)[iOffset>>3]>>(iOffset&7))&1;
	}

#endif

//*****************************************************************************
//*
//*		bit_write
//*
//*****************************************************************************
//	Schreibt ein Bit in einen Speicherbereich.
//	pMem		: Zeiger auf den Speicherbereich
//	iOffset		: Ist die Nummer des Bits das geschrieben werden soll.
//	iBit		: Wenn ungleich 0 dann wird das Bit gesetzt.
//	Ergibt einen Wert ungleich 0 wenn das alte Bit gesetzt war sonst 0

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_write(void *pMem,int iOffset,int iBit);

#else

	BIT_STATIC BIT_INLINE int bit_write(void *pMem,int iOffset,int iBit)
	{
	if(iBit)return bit_set  (pMem,iOffset);
			return bit_clear(pMem,iOffset);
	}

#endif

//*****************************************************************************
//*
//*		bit_swap
//*
//*****************************************************************************
//	Tauscht die Bits in einer Zahl. Dabei wird das niederwetigste Bit an die
//	hchste Position verschoben und umgekehrt.
//	uValue		: Ist die Zahl in der die Bits getauscht werden sollen.
//	iCount		: Bestimmt wieviele Bits ab dem niederwetigsten getauscht werden.
//	Ergibt die vernderte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned _cdecl bit_swap(unsigned uValue,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned bit_swap(unsigned uValue,int iCount)
	{
	unsigned	uRet=0;

	for(;iCount>0;iCount--)
		{
		uRet   <<= 1;
		if(uValue&1)uRet|=1;
		uValue >>= 1;
		}

	return uRet;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned bit_swap(unsigned uValue,int iCount)
	{
	unsigned	uRet=0;

    asm volatile(
        "xorl   %%eax,%%eax     \n"
		"jecxz  2f              \n"
        "1:                     \n"
        "shr    $1,%%edx        \n"
		"rcl    $1,%%eax        \n"
        "dec    %%ecx           \n"
        "jnz    1b              \n"
        "2:                     \n"
        :"=a" (uRet)
        :"d"  (uValue), "c" (iCount)
		:"cc"
        );

	return  uRet;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE unsigned bit_swap(unsigned uValue,int iCount)
	{
	_asm{
		mov	 	edx,uValue
		mov  	ecx,iCount
		xor  	eax,eax
		jecxz	Exit
	Schl:
		shr	 	edx,1
		rcl  	eax,1
		dec	 	ecx
		jnz	 	Schl
	Exit:
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned bit_swap(unsigned uValue,int iCount)
	{
	unsigned 	uRet;

    asm volatile( "RBIT %0,%1"
                  : "=r"(uRet)
                  : "r" (uValue));

	return uRet>>(32-iCount);
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned bit_swap(unsigned uValue,int iCount)
	{
	unsigned 	uRet;

	if(iCount<=0)return 0;

    __asm
		{
		RBIT 	uRet,uValue
		}

	return uRet>>(32-iCount);
	}

#else

	BIT_STATIC BIT_INLINE unsigned bit_swap(unsigned uValue,int iCount)
	{
	unsigned	uRet=0;

	for(;iCount>0;iCount--)
		{
		uRet   <<= 1;
		if(uValue&1)uRet|=1;
		uValue >>= 1;
		}

	return uRet;
	}

#endif

//*****************************************************************************
//*
//*		bit_rol_qw
//*
//*****************************************************************************
//	Rollt einen 64-Bitwert Bits um eine Anzahl an Bits nach links oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach links  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned ibt64 _cdecl bit_rol_qw(unsigned ibt64 uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_rol_qw(unsigned ibt64 uBits,int iCount)
	{
	unsigned ibt64	uValue;

	iCount &= 0x3F;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (64-iCount);

	return uValue;
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_rol_qw(unsigned ibt64 uBits,int iCount)
	{
	ibt64  iRet;

    asm volatile(
        "rol   %%cl,%%rax      \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
        :"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_rol_qw(unsigned ibt64 uBits,int iCount)
	{
	return _rotl64(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_rol_qw(unsigned ibt64 uBits,int iCount)
	{
	_asm{
		mov	  	esi,dword ptr uBits+0
		mov	  	edi,dword ptr uBits+4
		mov   	ecx,iCount
		and   	ecx,63
		test  	ecx,32
		jz    	No64
		and   	ecx,31
		xchg  	esi,edi
	No64:
		jecxz 	NoRol
		shld  	edx,esi,cl
		shld  	eax,edi,cl
		neg   	cl
		shl   	edx,cl
		shl   	eax,cl
		shrd  	edx,edi,cl
		shrd  	eax,esi,cl
		jmp   	RolExit
	NoRol:
		mov   	eax,esi
		mov   	edx,edi
	RolExit:
		}
	}

#else

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_rol_qw(unsigned ibt64 uBits,int iCount)
	{
	unsigned ibt64	uValue;

	iCount &= 0x3F;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (64-iCount);

	return uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_rol_dw
//*
//*****************************************************************************
//	Rollt einen 32-Bitwert Bits um eine Anzahl an Bits nach links oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach links  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned int _cdecl bit_rol_dw(unsigned int uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned int bit_rol_dw(unsigned int uBits,int iCount)
	{
	unsigned 	uValue;

	iCount &= 0x1F;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (32-iCount);

	return uValue;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned int bit_rol_dw(unsigned int uBits,int iCount)
	{
	int  iRet;

    asm volatile(
        "rol  %%cl,%%eax   \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
        :"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned int bit_rol_dw(unsigned int uBits,int iCount)
	{
	return _rotl(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned int bit_rol_dw(unsigned int uBits,int iCount)
	{
	_asm{
		mov		eax,uBits
		mov		ecx,iCount
		rol		eax,cl
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned int bit_rol_dw(unsigned int uBits,int iCount)
	{
	int  iRet;

    iCount = -iCount;

	asm volatile( "ROR %0,%1,%2"
                  : "=r"(iRet)
                  : "r" (uBits),"r" (iCount)
                  : "cc" );

	return iRet;
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned int bit_rol_dw(unsigned int uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;
	iCount = -iCount;

    __asm
		{
		ROR 	iRet,iVal,iCount
		}

	return iRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned int bit_rol_dw(unsigned int uBits,int iCount)
	{
	unsigned 	uValue;

	iCount &= 0x1F;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (32-iCount);

	return uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_rol_w
//*
//*****************************************************************************
//	Rollt einen 16-Bitwert Bits um eine Anzahl an Bits nach links oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach links  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned short _cdecl bit_rol_w(unsigned short uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned short bit_rol_w(unsigned short uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x0F;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (16-iCount);

	return (unsigned short)uValue;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned short bit_rol_w(unsigned short uBits,int iCount)
	{
	int  iRet;

    asm volatile(
        "rol    %%cl,%%ax    \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
        :"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned short bit_rol_w(unsigned short uBits,int iCount)
	{
	return _rotl16(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned short bit_rol_w(unsigned short uBits,int iCount)
	{
	_asm{
		mov		ax ,uBits
		mov		ecx,iCount
		rol		ax ,cl
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned short bit_rol_w(unsigned short uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;
	iCount = -iCount;

    asm volatile( "ORR %1,%1,%1,ROR #16 \n"
				  "ROR %0,%1,%2         \n"
                  : "=r"(iRet)
                  : "r" (iVal),"r" (iCount)
                  : "cc" );

	return iRet;
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned short bit_rol_w(unsigned short uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;
	iCount = -iCount;

    __asm
		{
		ORR 	iVal,iVal,iVal,ROR #16
		ROR 	iRet,iVal,iCount
		}

	return iRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned short bit_rol_w(unsigned short uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x0F;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (16-iCount);

	return (unsigned short)uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_rol_b
//*
//*****************************************************************************
//	Rollt einen 8-Bitwert Bits um eine Anzahl an Bits nach links oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach links  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned char _cdecl bit_rol_b(unsigned char uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned char bit_rol_b(unsigned char uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x07;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (8-iCount);

	return (unsigned char)uValue;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned char bit_rol_b(unsigned char uBits,int iCount)
	{
	int  iRet;

    asm volatile(
        "rol  %%cl,%%al   \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
        :"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned char bit_rol_b(unsigned char uBits,int iCount)
	{
	return _rotl8(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned char bit_rol_b(unsigned char uBits,int iCount)
	{
	_asm{
		mov		al ,uBits
		mov		ecx,iCount
		rol		al ,cl
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned char bit_rol_b(unsigned char uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;
	iCount = -iCount;

    asm volatile( "ORR %1,%1,%1,ROR #8  \n"
				  "ORR %1,%1,%1,ROR #16 \n"
				  "ROR %0,%1,%2         \n"
                  : "=r"(iRet)
                  : "r" (iVal),"r" (iCount)
                  : "cc" );

	return iRet;
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned char bit_rol_b(unsigned char uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;
	iCount = -iCount;

    __asm
		{
		ORR 	iVal,iVal,iVal,ROR #8
		ORR 	iVal,iVal,iVal,ROR #16
		ROR 	iRet,iVal,iCount
		}

	return iRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned char bit_rol_b(unsigned char uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x07;
	uValue	= uBits <<  iCount;
	uValue |= uBits >> (8-iCount);

	return (unsigned char)uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_ror_qw
//*
//*****************************************************************************
//	Rollt einen 64-Bitwert Bits um eine Anzahl an Bits nach rechts oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach rechts  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned ibt64 _cdecl bit_ror_qw(unsigned ibt64 uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_ror_qw(unsigned ibt64 uBits,int iCount)
	{
	unsigned ibt64	uValue;

	iCount &= 0x3F;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (64-iCount);

	return uValue;
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_ror_qw(unsigned ibt64 uBits,int iCount)
	{
	ibt64  iRet;

    asm volatile(
        "ror    %%cl,%%rax   \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
        :"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_ror_qw(unsigned ibt64 uBits,int iCount)
	{
	return _rotr64(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_ror_qw(unsigned ibt64 uBits,int iCount)
	{
	_asm{
		mov	  	esi,dword ptr uBits+0
		mov	  	edi,dword ptr uBits+4
		mov   	ecx,iCount
		and   	ecx,63
		test  	ecx,32
		jz    	No64
		and   	ecx,31
		xchg  	esi,edi
	No64:
		jecxz 	NoRol
		shrd  	edx,esi,cl
		shrd  	eax,edi,cl
		neg   	cl
		shr   	edx,cl
		shr   	eax,cl
		shld  	edx,edi,cl
		shld  	eax,esi,cl
		jmp   	RolExit
	NoRol:
		mov   	eax,esi
		mov   	edx,edi
	RolExit:
		}
	}

#else

	BIT_STATIC BIT_INLINE unsigned ibt64 bit_ror_qw(unsigned ibt64 uBits,int iCount)
	{
	unsigned ibt64	uValue;

	iCount &= 0x3F;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (64-iCount);

	return uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_ror_dw
//*
//*****************************************************************************
//	Rollt einen 32-Bitwert Bits um eine Anzahl an Bits nach rechts oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach rechts  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned int _cdecl bit_ror_dw(unsigned int uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned int bit_ror_dw(unsigned int uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x1F;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (32-iCount);

	return uValue;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned int bit_ror_dw(unsigned int uBits,int iCount)
	{
	int  iRet;

    asm volatile(
        "ror  %%cl,%%eax    \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
		:"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned int bit_ror_dw(unsigned int uBits,int iCount)
	{
	return _rotr(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned int bit_ror_dw(unsigned int uBits,int iCount)
	{
	_asm{
		mov		eax,uBits
		mov		ecx,iCount
		ror		eax,cl
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned int bit_ror_dw(unsigned int uBits,int iCount)
	{
	int  iRet;

    asm volatile( "ROR %0,%1,%2"
                  : "=r"(iRet)
                  : "r" (uBits),"r" (iCount)
                  : "cc" );

	return iRet;
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned int bit_ror_dw(unsigned int uBits,int iCount)
	{
	int		iRet;

    __asm
		{
		ROR 	iRet,uBits,iCount
		}

	return iRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned int bit_ror_dw(unsigned int uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x1F;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (32-iCount);

	return uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_ror_w
//*
//*****************************************************************************
//	Rollt einen 16-Bitwert Bits um eine Anzahl an Bits nach rechts oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach rechts  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned short _cdecl bit_ror_w(unsigned short uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned short bit_ror_w(unsigned short uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x0F;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (16-iCount);

	return (unsigned short)uValue;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned short bit_ror_w(unsigned short uBits,int iCount)
	{
	int  iRet;

    asm volatile(
        "ror  %%cl,%%ax  \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
        :"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned short bit_ror_w(unsigned short uBits,int iCount)
	{
	return _rotr16(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned short bit_ror_w(unsigned short uBits,int iCount)
	{
	_asm{
		mov		ax ,uBits
		mov		ecx,iCount
		ror		ax ,cl
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned short bit_ror_w(unsigned short uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;

    asm volatile( "ORR %1,%1,%1,ROR #16 \n"
				  "ROR %0,%1,%2         \n"
                  : "=r"(iRet)
                  : "r" (iVal),"r" (iCount)
                  : "cc" );

	return iRet;
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned short bit_ror_w(unsigned short uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;

    __asm
		{
		ORR 	iVal,iVal,iVal,ROR #16
		ROR 	iRet,iVal,iCount
		}

	return iRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned short bit_ror_w(unsigned short uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x0F;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (16-iCount);

	return (unsigned short)uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_ror_b
//*
//*****************************************************************************
//	Rollt einen 8-Bitwert Bits um eine Anzahl an Bits nach rechts oder rechts.
//	uBits		: Ist der Wert der gerollt werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert gerollt werden soll.
//					>0 es wird nach rechts  gerollt
//					<0 es wird nach rechts gerollt
//	Ergibt die gerollte Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned char _cdecl bit_ror_b(unsigned char uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned char bit_ror_b(unsigned char uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x07;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (8-iCount);

	return (unsigned char)uValue;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned char bit_ror_b(unsigned char uBits,int iCount)
	{
	int  iRet;

    asm volatile(
        "ror  %%cl,%%al  \n"
        :"=a" (iRet)
        :"a"  (uBits), "c" (iCount)
        :"cc"
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR)

	BIT_STATIC BIT_INLINE unsigned char bit_ror_b(unsigned char uBits,int iCount)
	{
	return _rotr8(uBits,iCount);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned char bit_ror_b(unsigned char uBits,int iCount)
	{
	_asm{
		mov		al ,uBits
		mov		ecx,iCount
		ror		al ,cl
		}
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64) || defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned char bit_ror_b(unsigned char uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;

    asm volatile( "ORR %1,%1,%1,ROR #8  \n"
				  "ORR %1,%1,%1,ROR #16 \n"
				  "ROR %0,%1,%2         \n"
                  : "=r"(iRet)
                  : "r" (iVal),"r" (iCount)
                  : "cc" );

	return iRet;
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned char bit_ror_b(unsigned char uBits,int iCount)
	{
	int		iRet;
	int		iVal;

    iVal   =  uBits;

    __asm
		{
		ORR 	iVal,iVal,iVal,ROR #8
		ORR 	iVal,iVal,iVal,ROR #16
		ROR 	iRet,iVal,iCount
		}

	return iRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned char bit_ror_b(unsigned char uBits,int iCount)
	{
	unsigned  	uValue;

	iCount &= 0x07;
	uValue	= uBits >>  iCount;
	uValue |= uBits << (8-iCount);

	return (unsigned char)uValue;
	}

#endif

//*****************************************************************************
//*
//*		bit_shift
//*
//*****************************************************************************
//	Verschibt die Bits eines Wertes mit berlaufprfung. (nicht arithmetisch)
//	uBits		: Ist der Wert verschoben werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert verschoben werden soll.
//					>0 es wird nach links  verschoben
//					<0 es wird nach rechts verschoben
//	Ergibt die verschobene Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_shift(unsigned int uBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int bit_shift(unsigned int uBits,int iCount)
	{
	if(iCount<0)
		{
		if(iCount<=-32)return 0;
		return uBits>>-iCount;
		}

	if(iCount>=32)return 0;

	return uBits<<iCount;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int bit_shift(unsigned int uBits,int iCount)
	{
	_asm{
		mov		ecx,iCount
		test 	ecx,80000000h
		jz	 	left

		neg  	ecx
		test 	ecx,NOT 1Fh
		jnz	 	null
		mov	 	eax,uBits
		shr	 	eax,cl
		jmp  	ende
	left:
		test 	ecx,NOT 1Fh
		jz	 	shiftl
	null:
		xor  	eax,eax
		jmp  	ende
	shiftl:
		mov	 	eax,uBits
		shl	 	eax,cl
	ende:
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_shift(unsigned int uBits,int iCount)
	{
	if(iCount<0)
		{
		if(iCount<=-32)return 0;
		return uBits>>-iCount;
		}

	if(iCount>=32)return 0;

	return uBits<<iCount;
	}

#endif

//*****************************************************************************
//*
//*		bit_shift_l
//*
//*****************************************************************************
//	Verschibt die Bits eines Wertes mit berlaufprfung nach links.
//	iBits		: Ist der Wert verschoben werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert verschoben werden soll.
//	Ergibt die verschobene Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_shift_l(int iBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int bit_shift_l(int iBits,int iCount)
	{
	if(iCount&~0x1F)return 0;
	return iBits<<iCount;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int bit_shift_l(int iBits,int iCount)
	{
	_asm{
		mov		ecx,iCount
		test 	ecx,NOT 1Fh
		jz	 	shift
		xor  	eax,eax
		jmp  	ende
	shift:
		mov	 	eax,iBits
		shl	 	eax,cl
	ende:
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_shift_l(int iBits,int iCount)
	{
	if(iCount&~0x1F)return 0;
	return iBits<<iCount;
	}

#endif

//*****************************************************************************
//*
//*		bit_shift_r
//*
//*****************************************************************************
//	Verschibt die Bits eines Wertes mit berlaufprfung nach rechts. (nicht arithmetisch)
//	iBits		: Ist der Wert verschoben werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert verschoben werden soll.
//	Ergibt die verschobene Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_shift_r(int iBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int bit_shift_r(int iBits,int iCount)
	{
	if(iCount&~0x1F)return 0;
	unsigned uBits=iBits;
	return   uBits>>iCount;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int bit_shift_r(int iBits,int iCount)
	{
	_asm{
		mov  	ecx,iCount
		test 	ecx,NOT 1Fh
		jz	 	shift
		xor  	eax,eax
		jmp  	ende
	shift:
		mov	 	eax,iBits
		shr	 	eax,cl
	ende:
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_shift_r(int iBits,int iCount)
	{
	if(iCount&~0x1F)return 0;
	unsigned uBits=iBits;
	return   uBits>>iCount;
	}

#endif

//*****************************************************************************
//*
//*		bit_shift_a
//*
//*****************************************************************************
//	Verschibt die Bits eines Wertes mit berlaufprfung arithmetisch nach rechts.
//	iBits		: Ist der Wert verschoben werden soll.
//	iCount		: Ist die Anzahl der Bits um die der Wert verschoben werden soll.
//					>0 es wird nach links  verschoben
//					<0 es wird nach rechts verschoben
//	Ergibt die verschobene Zahl

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_shift_a(int iBits,int iCount);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int bit_shift_a(int iBits,int iCount)
	{
	if(iCount&~0x1F)iCount=31;
	return   iBits>>iCount;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int bit_shift_a(int iBits,int iCount)
	{
	_asm{
		mov  	ecx,iCount
		mov	 	eax,iBits
		test 	ecx,NOT 1Fh
		jz	 	shift
		mov  	cl ,31
	shift:
		sar	 	eax,cl
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_shift_a(int iBits,int iCount)
	{
	if(iCount&~0x1F)iCount=31;
	return   iBits>>iCount;
	}

#endif

//*****************************************************************************
//*
//*		bit_find_1
//*
//*****************************************************************************
//	Sucht das erste gesetzte Bit in einem Speicherbereich.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_1(const void *pMem,unsigned uSize);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem, unsigned uCount)
	{
	return bit_find_1(pMem,uCount,0);
	}

#elif defined(BIT_KEIL_ARM)

	extern _cdecl int bit_find_1(const void *pMem,unsigned uCount);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_1_thumb2(const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem,unsigned uCount)
	{
	return bit_find_1_thumb2(pMem,uCount);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_1_arm(const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem,unsigned uCount)
	{
	return bit_find_1_arm(pMem,uCount);
	}

#elif defined(BIT_GNU_X86)

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem, unsigned uCount)
	{
	int iRet;

    asm volatile(
        "xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addl   $4   ,%%esi     \n"
        "addl   $32  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
        "bsfl   (%%esi),%%ecx   \n"
        "jz     1b              \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"ecx"
        );

	return  iRet;
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem, unsigned uCount)
	{
	int iRet;

    asm volatile(
        "xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addq   $8   ,%%rsi     \n"
        "addl   $64  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
        "bsfq   (%%rsi),%%rcx   \n"
        "jz     1b              \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"ecx"
        );

    return iRet;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem, unsigned uCount)
	{
	_asm{
		mov		esi,pMem			;// ESI = Adresse
		xor		eax,eax				;// EAX = Position
		mov		edi,uCount			;// EDI = Anzahl
		jmp		start
	loop1:
		add		esi,4				;// Die nchsten 32-Bit
		add		eax,32				;// 		*
	start:
		cmp		eax,edi				;// 		*
		jnb     ende1				;// berlauf ?
		bsf	    ecx,[esi]			;// Suche Bit
		jz	    loop1				;// 		*

		add     eax,ecx				;// Ergebnis berechen
      	cmp		eax,edi				;// berlauf ?
		jbe     ende2				;// 		*
	ende1:
		mov		eax,edi				;// 		*
	ende2:
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem, unsigned uCount)
	{
	return bit_find_1(pMem,uCount,0);
	}

#endif

//*****************************************************************************
//*
//*		bit_find_1
//*
//*****************************************************************************
//	Sucht das nchste gesetzte Bit in einem Speicherbereich ab einem Offset.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	uOffset		: Ist die Bitposition ab der gesucht wird
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_1(const void *pMem,unsigned uSize,unsigned uOffset);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_1o_thumb2(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_1o_thumb2(pMem,uCount,uOffset);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_1o_arm(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_1(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_1o_arm(pMem,uCount,uOffset);
	}

#endif

//*****************************************************************************
//*
//*		bit_find_1x
//*
//*****************************************************************************
//	Sucht das erste gesetzte Bit in einem Speicherbereich.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_1x(const void *pMem,unsigned uSize);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem, unsigned uCount)
	{
	return bit_find_1x(pMem,uCount,0);
	}

#elif defined(BIT_KEIL_ARM)

	extern _cdecl int bit_find_1x(const void *pMem,unsigned uCount);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_1x_thumb2(const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem,unsigned uCount)
	{
	return bit_find_1x_thumb2(pMem,uCount);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_1x_arm(const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem,unsigned uCount)
	{
	return bit_find_1x_arm(pMem,uCount);
	}

#elif defined(BIT_GNU_X86)

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem, unsigned uCount)
	{
	int iRet;

    asm volatile(
        "xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addl   $4   ,%%esi     \n"
        "addl   $32  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
		"movl   (%%esi),%%ecx   \n"
		"bswap  %%ecx           \n"
        "bsrl   %%ecx,%%ecx     \n"
        "jz     1b              \n"
		"xorl   $31  ,%%ecx     \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"ecx"
        );

	return  iRet;
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem, unsigned uCount)
	{
	int iRet;

    asm volatile(
        "xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addq   $8   ,%%rsi     \n"
        "addl   $64  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
		"movq   (%%rsi),%%rcx   \n"
		"bswap  %%rcx           \n"
        "bsrq   %%rcx,%%rcx     \n"
        "jz     1b              \n"
		"xorl   $63  ,%%ecx     \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"ecx"
        );

    return iRet;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem, unsigned uCount)
	{
	_asm{
		mov		esi,pMem			;// ESI = Adresse
		xor		eax,eax				;// EAX = Position
		mov		edi,uCount			;// EDI = Anzahl
		jmp		start
	loop1:
		add		esi,4				;// Die nchsten 32-Bit
		add		eax,32				;// 		*
	start:
		cmp		eax,edi				;// 		*
		jnb     ende1				;// berlauf ?
		mov	    ecx,[esi]			;// Lade 32-Bit
		bswap   ecx					;// Drehe Byte Reihenfolge
		bsr	    ecx,ecx  			;// Suche Bit
		jz	    loop1				;// 		*

		xor     ecx,1Fh				;// Bitposition korrigiren
		add     eax,ecx				;// Ergebnis berechen
      	cmp		eax,edi				;// berlauf ?
		jbe     ende2				;// 		*
	ende1:
		mov		eax,edi				;// 		*
	ende2:
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem, unsigned uCount)
	{
	return bit_find_1x(pMem,uCount,0);
	}

#endif

//*****************************************************************************
//*
//*		bit_find_1x
//*
//*****************************************************************************
//	Sucht das nchste gesetzte Bit in einem Speicherbereich ab einem Offset.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	uOffset		: Ist die Bitposition ab der gesucht wird
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_1x(const void *pMem,unsigned uSize,unsigned uOffset);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_1xo_thumb2(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_1xo_thumb2(pMem,uCount,uOffset);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_1xo_arm(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_1x(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_1xo_arm(pMem,uCount,uOffset);
	}

#endif

//*****************************************************************************
//*
//*		bit_find_0
//*
//*****************************************************************************
//	Sucht das erste gelschte Bit in einem Speicherbereich.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_0(const void *pMem,unsigned uSize);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0(const void *pMem,unsigned uCount)
	{
	return bit_find_0(pMem,uCount,0);
	}

#elif defined(BIT_KEIL_ARM)

	extern _cdecl int bit_find_0(const void *pMem,unsigned uCount);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_0_thumb2 (const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_0(const void *pMem,unsigned uCount)
	{
	return bit_find_0_thumb2(pMem,uCount);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_0_arm(const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_0(const void *pMem,unsigned uCount)
	{
	return bit_find_0_arm(pMem,uCount);
	}

#elif defined(BIT_GNU_X86)

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0(const void *pMem,unsigned uCount)
	{
	int iRet;

    asm volatile(
        "pushl  %%ebx	        \n"
		"xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addl   $4   ,%%esi     \n"
        "addl   $32  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
        "movl   (%%esi),%%ebx   \n"
        "notl   %%ebx           \n"
        "bsfl   %%ebx,%%ecx     \n"
        "jz     1b              \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
		"popl   %%ebx	        \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"ecx"
        );

	return  iRet;
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0(const void *pMem,unsigned uCount)
	{
	int iRet;

    asm volatile(
		"pushl  %%rbx	        \n"
        "xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addq   $8   ,%%rsi     \n"
        "addl   $64  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
        "movq   (%%rsi),%%rbx   \n"
        "notq   %%rbx           \n"
        "bsfl   %%rbx,%%rcx     \n"
        "jz     1b              \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
		"popl   %%rbx	        \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"rcx"
        );

    return iRet;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0(const void *pMem,unsigned uCount)
	{
	_asm{
		mov		esi,pMem			;// ESI = Adresse
		xor		eax,eax				;// EAX = Position
		mov		edi,uCount			;// EDI = Anzahl
		jmp		start
	loop1:
		add		esi,4				;// Die nchsten 32-Bit
		add		eax,32				;// 		*
	start:
		cmp		eax,edi				;// 		*
		jnb     ende1				;// berlauf ?

		mov     ebx,[esi]			;// 32-Bit laden
		not     ebx					;// Invertieren
		bsf	    ecx,ebx				;// Suche Bit
		jz	    loop1				;// 		*

		add     eax,ecx				;// Ergebnis berechen
      	cmp		eax,edi				;// berlauf ?
		jbe     ende2				;// 		*
	ende1:
		mov		eax,edi				;// 		*
	ende2:
		}
	}

#else

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0(const void *pMem,unsigned uCount)
	{
	return bit_find_0(pMem,uCount,0);
	}

#endif

//*****************************************************************************
//*
//*		bit_find_0
//*
//*****************************************************************************
//	Sucht das nchste gelschte Bit in einem Speicherbereich ab einem Offset.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	uOffset		: Ist die Bitposition ab der gesucht wird
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_0(const void *pMem,unsigned uSize,unsigned uOffset);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_0o_thumb2(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_0(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_0o_thumb2(pMem,uCount,uOffset);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_0o_arm(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_0(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_0o_arm(pMem,uCount,uOffset);
	}

#endif

//*****************************************************************************
//*
//*		bit_find_0x
//*
//*****************************************************************************
//	Sucht das erste gelschte Bit in einem Speicherbereich.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_0x(const void *pMem,unsigned uSize);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0x(const void *pMem,unsigned uCount)
	{
	return bit_find_0x(pMem,uCount,0);
	}

#elif defined(BIT_KEIL_ARM)

	extern _cdecl int bit_find_0x(const void *pMem,unsigned uCount);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_0x_thumb2 (const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_0x(const void *pMem,unsigned uCount)
	{
	return bit_find_0x_thumb2(pMem,uCount);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_0x_arm(const void *pMem,unsigned uCount);

	BIT_STATIC BIT_INLINE int bit_find_0x(const void *pMem,unsigned uCount)
	{
	return bit_find_0x_arm(pMem,uCount);
	}

#elif defined(BIT_GNU_X86)

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0x(const void *pMem,unsigned uCount)
	{
	int iRet;

    asm volatile(
        "pushl  %%ebx	        \n"
        "xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addl   $4   ,%%esi     \n"
        "addl   $32  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
        "movl   (%%esi),%%ebx   \n"
        "notl   %%ebx           \n"
		"bswap  %%ebx           \n"
        "bsrl   %%ebx,%%ecx     \n"
        "jz     1b              \n"
		"xorl   $31  ,%%ecx     \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
        "popl   %%ebx	        \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"ecx"
        );

	return  iRet;
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE int _cdecl bit_find_0x(const void *pMem,unsigned uCount)
	{
	int iRet;

    asm volatile(
        "pushl  %%rbx	        \n"
        "xorl   %%eax,%%eax     \n"
        "jmp    2f              \n"
        "1:                     \n"
        "addq   $8   ,%%rsi     \n"
        "addl   $64  ,%%eax     \n"
        "2:                     \n"
        "cmpl   %%edi,%%eax     \n"
        "jnb    3f              \n"
        "movq   (%%rsi),%%rbx   \n"
        "notq   %%rbx           \n"
		"bswap  %%rbx           \n"
        "bsrq   %%rbx,%%rcx     \n"
        "jz     1b              \n"
		"xorl   $63  ,%%ecx     \n"
        "add    %%ecx,%%eax     \n"
        "cmpl   %%edi,%%eax     \n"
        "jbe    4f              \n"
        "3:                     \n"
        "movl   %%edi,%%eax     \n"
        "4:                     \n"
        "popl  %%rbx	        \n"
        :"=a" (iRet)
        :"S" (pMem), "D" (uCount)
        :"rcx"
        );

    return iRet;
	}

#elif defined(BIT_MSC_X86)

	BIT_STATIC BIT_INLINE int bit_find_0x(const void *pMem,unsigned uCount)
	{
	_asm{
		mov		esi,pMem			;// ESI = Adresse
		xor		eax,eax				;// EAX = Position
		mov		edi,uCount			;// EDI = Anzahl
		jmp		start
	loop1:
		add		esi,4				;// Die nchsten 32-Bit
		add		eax,32				;// 		*
	start:
		cmp		eax,edi				;// 		*
		jnb     ende1				;// berlauf ?

		mov     ebx,[esi]			;// 32-Bit laden
		not     ebx					;// Invertieren
		bswap   ebx					;// Drehe Byte-Reihenfolge
		bsr	    ecx,ebx				;// Suche Bit
		jz	    loop1				;// 		*

		xor     ecx,1Fh				;// Bitposition korrigiren
		add     eax,ecx				;// Ergebnis berechen
      	cmp		eax,edi				;// berlauf ?
		jbe     ende2				;// 		*
	ende1:
		mov		eax,edi				;// 		*
	ende2:
		}
	}

#else

	BIT_STATIC BIT_INLINE int bit_find_0x(const void *pMem,unsigned uCount)
	{
	return bit_find_0x(pMem,uCount,0);
	}

#endif

//*****************************************************************************
//*
//*		bit_find_0x
//*
//*****************************************************************************
//	Sucht das nchste gelschte Bit in einem Speicherbereich ab einem Offset.
//	Die Bits werden von 0x80 in Richtung 0x01 gesucht.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	uCount		: Ist die Gre der Speicherbereiches in Bits.
//	uOffset		: Ist die Bitposition ab der gesucht wird
//	Ergibt die Position an der das Bit gefunden wurde, bzw. die Gre des Bereiches.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_find_0x(const void *pMem,unsigned uSize,unsigned uOffset);

#elif defined(BIT_IAR_THUMB)

	// this functions are defined in "./Assembler/BitFunctionsIarThumb.s"
	extern "C" int bit_find_0xo_thumb2(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_0x(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_0xo_thumb2(pMem,uCount,uOffset);
	}

#elif defined(BIT_IAR_ARM)

	// this functions are defined in "./Assembler/BitFunctionsIar.s"
	extern "C" int bit_find_0xo_arm(const void *pMem,unsigned uCount,unsigned uOffset);

	BIT_STATIC BIT_INLINE int bit_find_0x(const void *pMem,unsigned uCount,unsigned uOffset)
	{
	return bit_find_0xo_arm(pMem,uCount,uOffset);
	}

#endif

//*****************************************************************************
//*
//*		btol_w
//*
//*****************************************************************************

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned short _cdecl btol_w(unsigned short uValue);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	return (unsigned short)((uValue>>8)|(uValue<<8));
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	unsigned 	uRet;

    asm volatile( "REVSH %0,%1"
                  : "=r"(uRet)
                  : "r" (uValue));

	return uRet;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	int iRet;

	asm volatile (
        "xchg   %%al,%%ah  \n"
        : "=a" (iRet)
        : "a"  (uValue)
        );

	return iRet;
	}

#elif defined(BIT_GNU_4_3)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	return __builtin_bswap16(uValue);
	}

#elif defined(BIT_MSC_INSTR) || defined(BIT_MSC_ARM) || defined(BIT_MSC_ARM64)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	return _byteswap_ushort(uValue);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	_asm{
		mov		ax,uValue
		xchg	al,ah;				;// Tausche AL mit AH
		}
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	unsigned 	uRet;

    __asm
		{
		REVSH 	uRet,uValue
		}

	return uRet;
	}

#elif defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	unsigned 	uRet;

    asm volatile( "REVSH %0,%1"
                  : "=r"(uRet)
                  : "r" (uValue));

	return uRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned short btol_w(unsigned short uValue)
	{
	return (unsigned short)((uValue>>8)|(uValue<<8));
	}

#endif

//*****************************************************************************
//*
//*		btol_dw
//*
//*****************************************************************************
#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned int _cdecl btol_dw(unsigned int uValue);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	unsigned int  uVal;

	uVal  =            btol_w((unsigned short)(uValue >>16));
	uVal |= ((unsigned)btol_w((unsigned short)(uValue)))<<16;

	return uVal;
	}

#elif defined(BIT_GNU_ARM) || defined(BIT_GNU_ARM64)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	unsigned 	uRet;

    asm volatile( "REV %0,%1"
                  : "=r"(uRet)
                  : "r" (uValue));

	return uRet;
	}

#elif defined(BIT_GNU_X86) || defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
    int iRet;

	asm volatile (
        "bswap     %%eax  \n"
        : "=a" (iRet)
        : "a"  (uValue)
        );

	return iRet;
	}

#elif defined(BIT_GNU_4_3)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	return __builtin_bswap32(uValue);
	}

#elif defined(BIT_MSC_INSTR) || defined(BIT_MSC_ARM) || defined(BIT_MSC_ARM64)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	return _byteswap_ulong(uValue);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	_asm{
		mov		eax,uValue
		bswap	eax;				;// Tausche Bytes in EAX
		}
	}

#elif defined(BIT_KEIL_ARM)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	unsigned 	uRet;

    __asm
		{
		REV 	uRet,uValue
		}

	return uRet;
	}

#elif defined(BIT_IAR_ARM)

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	unsigned 	uRet;

    asm volatile( "REV %0,%1"
                  : "=r"(uRet)
                  : "r" (uValue));

	return uRet;
	}

#else

	BIT_STATIC BIT_INLINE unsigned int btol_dw(unsigned int uValue)
	{
	unsigned int  uVal;

	uVal  =            btol_w((unsigned short)(uValue >>16));
	uVal |= ((unsigned)btol_w((unsigned short)(uValue)))<<16;

	return uVal;
	}

#endif

//*****************************************************************************
//*
//*		btol_qw
//*
//*****************************************************************************
#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	unsigned ibt64 _cdecl btol_qw(unsigned ibt64 uValue);

#elif defined(NO_BIT_ASM)

	BIT_STATIC BIT_INLINE unsigned ibt64 btol_qw(unsigned ibt64 uValue)
	{
	unsigned ibt64  uVal;

	uVal  =         btol_dw((unsigned int)(uValue >>32));
	uVal |= ((ibt64)btol_dw((unsigned int)(uValue)))<<32;

	return uVal;
	}

#elif defined(BIT_GNU_4_3)

	BIT_STATIC BIT_INLINE unsigned ibt64 btol_qw(unsigned ibt64 uValue)
	{
	return __builtin_bswap64(uValue);
	}

#elif defined(BIT_GNU_X64)

	BIT_STATIC BIT_INLINE unsigned ibt64 btol_qw(unsigned ibt64 uValue)
	{
    ibt64 iRet;

	asm volatile (
        "bswap     %%rax  \n"
        : "=a" (iRet)
        : "a"  (uValue)
        );

	return iRet;
	}

#elif defined(BIT_MSC_INSTR) || defined(BIT_MSC_ARM) || defined(BIT_MSC_ARM64)

	BIT_STATIC BIT_INLINE unsigned ibt64 btol_qw(unsigned ibt64 uValue)
	{
	return _byteswap_uint64(uValue);
	}

#elif defined(BIT_MSC_VC60)

	BIT_STATIC BIT_INLINE unsigned ibt64 btol_qw(unsigned ibt64 uValue)
	{
	_asm{
		mov		edx,dword ptr uValue+0
		mov		eax,dword ptr uValue+4
		bswap	edx;
		bswap	eax;
		}
	}

#else

	BIT_STATIC BIT_INLINE unsigned ibt64 btol_qw(unsigned ibt64 uValue)
	{
	unsigned ibt64  uVal;

	uVal  =         btol_dw((unsigned int)(uValue >>32));
	uVal |= ((ibt64)btol_dw((unsigned int)(uValue)))<<32;

	return uVal;
	}

#endif

//*****************************************************************************
//*
//*		bit_count_0
//*
//*****************************************************************************
//	Zhlt die Low-Bits in einem Speicherbereich
//	pMem	: Zeiger auf das Bit-Array
//	uSize	: Gre des Bit-Arrays in Bits
//	uOffset	: Bit ab dem die Low-Bits gesucht werden
//	Ergibt die Anzahl der gefunden Low-Bits.

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	int _cdecl bit_count_0 (const void *pMem,unsigned uSize,unsigned uOffset=0);
	int _cdecl bit_count_0x(const void *pMem,unsigned uSize,unsigned uOffset=0);

#else

	BIT_STATIC BIT_INLINE int bit_count_0(const void *pMem,unsigned uSize,unsigned uOffset=0)
	{
	if(uOffset>=uSize)return 0;

	return uSize-uOffset-bit_count_1(pMem,uSize,uOffset);
	}

	BIT_STATIC BIT_INLINE int bit_count_0x(const void *pMem,unsigned uSize,unsigned uOffset=0)
	{
	if(uOffset>=uSize)return 0;

	return uSize-uOffset-bit_count_1x(pMem,uSize,uOffset);
	}

#endif

//*****************************************************************************
//*
//*		btol
//*
//*****************************************************************************
//	Big-Endial zu Little-Endian konvertieren

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	void _cdecl	btol(		  ibt64 &iValue);
	void _cdecl	btol(unsigned ibt64 &uValue);
	void _cdecl	btol(		  short &iValue);
	void _cdecl	btol(unsigned short &uValue);
	void _cdecl	btol(		  long  &iValue);
	void _cdecl	btol(unsigned long  &uValue);
	void _cdecl	btol(		  int   &iValue);
	void _cdecl	btol(unsigned int   &uValue);

#else

	BIT_STATIC BIT_INLINE void	btol(		  ibt64 &iValue){iValue=btol_qw(iValue);}
	BIT_STATIC BIT_INLINE void	btol(unsigned ibt64 &uValue){uValue=btol_qw(uValue);}
	BIT_STATIC BIT_INLINE void	btol(		  short &iValue){iValue=btol_w (iValue);}
	BIT_STATIC BIT_INLINE void	btol(unsigned short &uValue){uValue=btol_w (uValue);}
	BIT_STATIC BIT_INLINE void	btol(		  int   &iValue){iValue=btol_dw(iValue);}
	BIT_STATIC BIT_INLINE void	btol(unsigned int   &uValue){uValue=btol_dw(uValue);}

	#if defined(BIT_GNU_X64) || defined(BIT_GNU_ARM64)
	BIT_STATIC BIT_INLINE void	btol(		  long  &iValue){iValue=btol_qw(iValue);}
	BIT_STATIC BIT_INLINE void	btol(unsigned long  &uValue){uValue=btol_qw(uValue);}
	#else
	BIT_STATIC BIT_INLINE void	btol(		  long  &iValue){iValue=btol_dw(iValue);}
	BIT_STATIC BIT_INLINE void	btol(unsigned long  &uValue){uValue=btol_dw(uValue);}
	#endif

#endif


//*****************************************************************************
//*
//*		bit_rol
//*
//*****************************************************************************
//	Bits nach links rollen

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	void _cdecl	bit_rol(		 ibt64 &iValue,int iCount);
	void _cdecl	bit_rol(unsigned ibt64 &uValue,int iCount);
	void _cdecl	bit_rol(		 short &iValue,int iCount);
	void _cdecl	bit_rol(unsigned short &uValue,int iCount);
	void _cdecl	bit_rol(		 long  &iValue,int iCount);
	void _cdecl	bit_rol(unsigned long  &uValue,int iCount);
	void _cdecl	bit_rol(		 int   &iValue,int iCount);
	void _cdecl	bit_rol(unsigned int   &uValue,int iCount);

#else

	BIT_STATIC BIT_INLINE void	bit_rol(		 ibt64 &iValue,int iCount){iValue=bit_rol_qw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_rol(unsigned ibt64 &uValue,int iCount){uValue=bit_rol_qw(uValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_rol(		 short &iValue,int iCount){iValue=bit_rol_w (iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_rol(unsigned short &uValue,int iCount){uValue=bit_rol_w (uValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_rol(		 int   &iValue,int iCount){iValue=bit_rol_dw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_rol(unsigned int   &uValue,int iCount){uValue=bit_rol_dw(uValue,iCount);}

	#if defined(BIT_GNU_X64) || defined(BIT_GNU_ARM64)
	BIT_STATIC BIT_INLINE void	bit_rol(		 long  &iValue,int iCount){iValue=bit_rol_qw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_rol(unsigned long  &uValue,int iCount){uValue=bit_rol_qw(uValue,iCount);}
	#else
	BIT_STATIC BIT_INLINE void	bit_rol(		 long  &iValue,int iCount){iValue=bit_rol_dw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_rol(unsigned long  &uValue,int iCount){uValue=bit_rol_dw(uValue,iCount);}
	#endif

#endif

//*****************************************************************************
//*
//*		bit_ror
//*
//*****************************************************************************
//	Bits nach rechts rollen

#ifdef	_DEBUG

	// function defined in "BitFunctions.cpp"

	void _cdecl	bit_ror(		 ibt64 &iValue,int iCount);
	void _cdecl	bit_ror(unsigned ibt64 &uValue,int iCount);
	void _cdecl	bit_ror(		 short &iValue,int iCount);
	void _cdecl	bit_ror(unsigned short &uValue,int iCount);
	void _cdecl	bit_ror(		 long  &iValue,int iCount);
	void _cdecl	bit_ror(unsigned long  &uValue,int iCount);
	void _cdecl	bit_ror(		 int   &iValue,int iCount);
	void _cdecl	bit_ror(unsigned int   &uValue,int iCount);

#else

	BIT_STATIC BIT_INLINE void	bit_ror(		 ibt64 &iValue,int iCount){iValue=bit_ror_qw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_ror(unsigned ibt64 &uValue,int iCount){uValue=bit_ror_qw(uValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_ror(		 short &iValue,int iCount){iValue=bit_ror_w (iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_ror(unsigned short &uValue,int iCount){uValue=bit_ror_w (uValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_ror(		 int   &iValue,int iCount){iValue=bit_ror_dw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_ror(unsigned int   &uValue,int iCount){uValue=bit_ror_dw(uValue,iCount);}

	#if defined(BIT_GNU_X64) || defined(BIT_GNU_ARM64)
	BIT_STATIC BIT_INLINE void	bit_ror(		 long  &iValue,int iCount){iValue=bit_ror_qw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_ror(unsigned long  &uValue,int iCount){uValue=bit_ror_qw(uValue,iCount);}
	#else
	BIT_STATIC BIT_INLINE void	bit_ror(		 long  &iValue,int iCount){iValue=bit_ror_dw(iValue,iCount);}
	BIT_STATIC BIT_INLINE void	bit_ror(unsigned long  &uValue,int iCount){uValue=bit_ror_dw(uValue,iCount);}
	#endif

#endif


#ifdef	BIT_MSC_VC60
#pragma warning( pop )
#endif

#if     _MSC_VER >= 1000
#pragma warning( pop )
#endif


#endif
