//*****************************************************************************
//*
//*
//*		CompareData.cpp
//*
//*
//*****************************************************************************
//
//	Das ist eine Klasse zum speichern von Suchergebnissen
//
#include	"StringFunctions.h"
#include	"MemFunctions.h"
#include	"CompareData.h"
#include	"TextBuffer.h"
#include	<stddef.h>

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


typedef	struct
	{
	int		iFile;
	int		iPosition;
	int		iLength;
	}CompareEntry;


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CCompareData::CCompareData()
{

	m_sData = new char[2];
	m_sData[0]=':';
	m_sData[1]=0;
	m_iDataLength=0;
	m_iRegionStart=0;
	m_iRegionEnd=0;
	m_iRegionAdd=0;

	m_sPositions.Len(sizeof(CompareEntry));
	m_sFiles.DataLength(sizeof(int));

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CCompareData::~CCompareData()
{

	delete m_sData;

}


//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle Daten
void CCompareData::Clear()
{

	delete m_sData;
	m_sData = new char[2];
	m_sData[0]=':';
	m_sData[1]=0;
	m_iDataLength=0;
	m_iRegionStart=0;
	m_iRegionEnd=0;
	m_iRegionAdd=0;
	m_sFiles.Del();
	m_sPositions.Del();

}

//*****************************************************************************
//*
//*		InsertFile
//*
//*****************************************************************************
//	Fgt eine Datei in die Fileliste ein
//	pFileName	: Ist der Name der Datei
//	Ergibt die Nummer der Datei in der Liste
int	CCompareData::InsertFile(const char *pFileName)
{
int iNr;


	   iNr=m_sFiles.Find(pFileName,0,-1,1);
	if(iNr>=0)return iNr;

	iNr=m_sFiles.Count();
	m_sFiles.Add(pFileName);

return iNr;
}

//*****************************************************************************
//*
//*		InsertData
//*
//*****************************************************************************
//	Fgt eine Position ein von gefunden Daten
//	iPos	: Ist der Offset in der Datei
//	iLen	: Ist die Lnge der Daten in Byte (0=standard Lnge)
//	uFile	: Ist die Dateinummer
//	Ergibt 1 wenn der Eintrag eingefgt wurde
int	CCompareData::InsertData(int iPos,int iLen,unsigned uFile)
{
CompareEntry	sData;


	if(uFile>=(unsigned)m_sFiles.Count())return 0;
	if(m_sPositions.Count()>=1000)return 0;

	sData.iFile		= uFile;
	sData.iPosition	= iPos;
	sData.iLength	= iLen;

	m_sPositions.Add(&sData);


return m_sPositions.Count();
}


//*****************************************************************************
//*
//*		SetData
//*
//*****************************************************************************
//	Einstellen der Suchdaten
//	pData	: Ist ein Zeiger auf die Daten
//	iLen	: Ist die Lnge der Daten in Byte (Maximal 256 Byte)
//	pDump	: Ist der String mit dem HexDump ":00 11 22 ..."
void CCompareData::SetData(void *pData,int iLen)
{
char *pDump;

	if(iLen<0)iLen=0;
	if(iLen>256)iLen=256;

	pDump = new char[iLen*3+2];

	memtodump(pDump,pData,iLen,HEX_DPOINT|HEX_SPACE|HEX_NULL);
	delete m_sData;
	m_sData=pDump;
	m_iDataLength=iLen;

}

void CCompareData::SetData(const char *pDump)
{
char cMem[256];
int  iLen;

	iLen=dumptomem(cMem,sizeof(cMem),pDump);
	SetData(cMem,iLen);

}



//*****************************************************************************
//*
//*		Save
//*
//*****************************************************************************
//	Speichert alle Daten
//	Ergibt 1 wenn alles gespeichert wurde
int	CCompareData::Save(const char *pFileName)
{
CFileBuffer	sFile;
int			i;


	if(!sFile.Open(pFileName,CFB_CREATE))return 0;


	sFile.Write("[DATA]");
	sFile.WriteLF();
	sFile.Write("\t\"");
	sFile.Write(m_sData);
	sFile.Write("\"");
	sFile.WriteLF();
	sFile.WriteLF();

	sFile.Write("[FILES]");
	sFile.WriteLF();
	for(i=0;i<m_sFiles.Count();i++)
		{
		sFile.Write("\t\"");
		sFile.Write(m_sFiles[i]);
		sFile.Write("\"");
		sFile.WriteLF();
		}
	sFile.WriteLF();

	sFile.Write("[REGION]");
	sFile.WriteLF();
	sFile.WriteFormat("\t%i %i %i",m_iRegionStart,m_iRegionEnd,m_iRegionAdd);
	sFile.WriteLF();
	sFile.WriteLF();

	sFile.Write("[POSITIONS]");
	sFile.WriteLF();

	for(i=0;i<m_sPositions.Count();i++)
		{
		CompareEntry *pData;
		pData=(CompareEntry*)m_sPositions[i];
		sFile.WriteFormat("\t%i 0x%08X",pData->iFile,pData->iPosition);
		if(pData->iLength)sFile.WriteFormat(" %i",pData->iLength);
		sFile.WriteLF();
		}

	sFile.WriteLF();
	sFile.Close();


return 1;
}


//*****************************************************************************
//*
//*		Load
//*
//*****************************************************************************
//	Ld Daten aus einer Datei
//	pFileName	: Dateiname mit den Daten
//	iMode		: Lade Modus	CCD_NEW		0 = Alles neu laden
//								CCD_AND		1 = Neue Daten mit den alten Und verknfen
//								CCD_OR		2 = Neue Daten mit den alten Oder verknfen
//								CCD_SINGLE	3 = Einzelne Dateien ohne Dateivergleich Und verknfpen
//	Ergibt 1 wenn die Daten geladen wurden
int	CCompareData::Load(const char *pFileName,int iMode)
{
CDataField		sPositions(sizeof(CompareEntry));
CStringChain	sFiles(sizeof(int));
CTextBuffer		sFile;
CompareEntry	sData,*pEntry;
char			cBuffer[1024],*pMem;
int				i,iCount,iPos,iAdd=0;


	if(iMode==CCD_SINGLE && m_sFiles.Count()!=1)return 0;
	if(iMode==CCD_NEW)Clear();

	if(!sFile.Open(pFileName))return 0;

//*****************************************************************************

	while(sFile.Text(iAdd,&pMem))
		{
		iAdd=sFile.TextLineLen();
		iPos=sFile.TextString(cBuffer,sizeof(cBuffer),'[',']');

		if(!cBuffer[0])continue;

		if(!stricmp(cBuffer,"DATA"))
			{
			iAdd=sFile.TextLineLen();
			while(sFile.Text(iAdd,&pMem))
				{
				iAdd=sFile.TextLineLen();
				iPos=sFile.TextString(cBuffer,sizeof(cBuffer));
				if(pMem[iPos]=='['){iAdd=0;break;}
				if(cBuffer[0]!=':')continue;
				if(iMode==CCD_NEW)SetData(cBuffer);
				break;
				}
			continue;
			}

		if(!stricmp(cBuffer,"FILES"))
			{
			iAdd=sFile.TextLineLen();
			while(sFile.Text(iAdd,&pMem))
				{
				iAdd=sFile.TextLineLen();
				iPos=sFile.TextString(cBuffer,sizeof(cBuffer));
				if(pMem[iPos]=='['){iAdd=0;break;}
				if(cBuffer[0]==0)continue;
				if(iMode==CCD_NEW)m_sFiles.Add(cBuffer);
				else                sFiles.Add(cBuffer);
				}
			continue;
			}

		if(!stricmp(cBuffer,"REGION"))
			{
			iAdd=sFile.TextLineLen();
			while(sFile.Text(iAdd,&pMem))
				{
				iAdd=sFile.TextLineLen();
				iPos=sFile.TextValue(cBuffer,sizeof(cBuffer));
				if(pMem[iPos]=='['){iAdd=0;break;}
				if(cBuffer[0]==0)continue;
				if(iMode!=CCD_NEW)break;
				m_iRegionStart=strtoi(cBuffer);
				sFile.Text(iPos,&pMem);
				iPos=sFile.TextValue(cBuffer,sizeof(cBuffer));
				m_iRegionEnd=strtoi(cBuffer);
				sFile.Text(iPos,&pMem);
				iPos=sFile.TextValue(cBuffer,sizeof(cBuffer));
				m_iRegionAdd=strtoi(cBuffer);
				iAdd=sFile.TextLineLen();
				break;
				}
			continue;
			}

		if(!stricmp(cBuffer,"POSITIONS"))
			{
			iAdd=sFile.TextLineLen();
			while(sFile.Text(iAdd,&pMem))
				{
				iAdd=sFile.TextLineLen();
				iPos=sFile.TextValue(cBuffer,sizeof(cBuffer));
				if(pMem[iPos]=='['){iAdd=0;break;}
				if(cBuffer[0]==0)continue;

				sData.iFile=strtoi(cBuffer);
				sFile.Text(iPos,&pMem);
				iPos=sFile.TextWord(cBuffer,sizeof(cBuffer));
				sData.iPosition=strtoi(cBuffer);
				sFile.Text(iPos,&pMem);
				iPos=sFile.TextValue(cBuffer,sizeof(cBuffer));
				sData.iLength=strtoi(cBuffer);
				iAdd=sFile.TextLineLen();

				if(iMode==CCD_NEW)m_sPositions.Add(&sData);
				else	            sPositions.Add(&sData);
				}
			continue;
			}
		}

//*****************************************************************************

	if(iMode==CCD_NEW)return 1;

	if(iMode==CCD_AND)
		{
		for(i=0,iAdd=0;i<m_sFiles.Count();i++)			// Suche doppelte Dateien
			{
			iPos=sFiles.Find(m_sFiles[i],0,-1,1);
			if(iPos>=0){m_sFiles.DataSet(i,iAdd);iAdd++;}
			else		m_sFiles.DataSet(i,-1);
			}

		for(i=0;i<sFiles.Count();i++)
			{
			iPos=m_sFiles.Find(m_sFiles[i],0,-1,1);
			if(iPos>=0)sFiles.DataSet(i,iPos);
			else	   sFiles.DataSet(i,-1);
			}

		for(i=0;i<m_sPositions.Count();i++)				// File Nummer umschreiben
			{
			pEntry=(CompareEntry*)m_sPositions[i];
			pEntry->iFile=m_sFiles.DataGet(pEntry->iFile);
			}

		for(i=0;i<sPositions.Count();i++)
			{
			pEntry=(CompareEntry*)sPositions[i];
			pEntry->iFile=sFiles.DataGet(pEntry->iFile);
			}

		for(i=0;i<m_sPositions.Count();i++)				// Lsche nicht doppelte Positionen
			{
			pEntry=(CompareEntry*)m_sPositions[i];
			if(pEntry->iFile<0)
				{
				m_sPositions.Del(i);
				i--;
				continue;
				}

			if(m_sFiles.DataGet(pEntry->iFile)==-1)
				{
				m_sPositions.Del(i);
				i--;
				continue;
				}
			   iPos=sPositions.Find(pEntry,0,-1,sizeof(int)*2);
			if(iPos<0)
				{
				m_sPositions.Del(i);
				i--;
				continue;
				}
			}

		for(i=0;i<m_sFiles.Count();i++)					// Lsche Dateien
			{
			if(m_sFiles.DataGet(i)!=-1)continue;
			m_sFiles.Del(i);
			i--;
			}

		return 1;
		}

	if(iMode==CCD_SINGLE)
		{
		for(i=0;i<m_sPositions.Count();i++)
			{
			pEntry=(CompareEntry*)m_sPositions[i];
			iPos=sPositions.Find(pEntry->iPosition,0,-1,offsetof(CompareEntry,iPosition));
			if(iPos<0)
				{
				m_sPositions.Del(i);
				i--;
				continue;
				}
			}

		return 1;
		}


	if(iMode==CCD_OR)
		{
		iCount=m_sFiles.Count();
		for(i=0,iAdd=iCount;i<sFiles.Count();i++)
			{
			iPos=m_sFiles.Find(sFiles[i],0,-1,1);
			if(iPos>=0){sFiles.DataSet(i,iPos);continue;}
		    m_sFiles.Add(sFiles[i]);
			  sFiles.DataSet(i,iAdd);
			iAdd++;
			}

		for(i=0;i<sPositions.Count();i++)
			{
			pEntry=(CompareEntry*)sPositions[i];
			iPos=sFiles.DataGet(pEntry->iFile);
			pEntry->iFile=iPos;

			if(iPos>=iCount)
				{
				m_sPositions.Add(pEntry);
				continue;
				}

			   iPos=sPositions.Find(pEntry,0,-1,sizeof(int)*2);
			if(iPos>=0)continue;

			m_sPositions.Add(pEntry);
			}

		return 1;
		}


return 0;
}

//*****************************************************************************
//*
//*		DeleteFile
//*
//*****************************************************************************
//	Lscht eine Datei sammt Positionen
//	iNr		: Ist die Nummer der zulschenden Dateien
//	Ergibt 1 wenn die Datei gelscht wurde.
int	 CCompareData::DelFile(int iNr)
{
int i,*iPtr;

	if(iNr<0 || iNr>=m_sFiles.Count())return 0;

	m_sFiles.Del(iNr);

	for(i=0;i<m_sPositions.Count();i++)
		{
		iPtr=(int*)m_sPositions[i];
		if(iPtr[0]<iNr)continue;
		if(iPtr[0]>iNr){iPtr[0]--;continue;}
		m_sPositions.Del(iNr);
		i--;
		}


return 1;
}

//*****************************************************************************
//*
//*		DeleteData
//*
//*****************************************************************************
//	Lscht eine Position
//	iNr		: Ist die Nummer der zulschenden Position
//	Ergibt 1 wenn die Position gelscht wurde.
int	 CCompareData::DelData(int iNr)
{
int i,iCount,iFile,*iPtr;


	if(iNr<0 || iNr>=m_sPositions.Count())return 0;

	iFile=((int*)m_sPositions[iNr])[0];

	for(i=0,iCount=0;i<m_sPositions.Count();i++)
		{
		iPtr=(int*)m_sPositions[i];
		if(iPtr[0]!=iFile)continue;
		iCount++;
		if(iCount>1)break;
		}

	m_sPositions.Del(iNr);
	if(iCount==1)DelFile(iFile);

return 1;
}
