//*****************************************************************************
//*
//*
//*		AsmDecoder
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//
//	Mit diesen Funktionen knnen binre	Programmdaten disassembliert werden.
//	Die Funktionen sind Prozessor unabhngig, und knnen mittels *.acf
//	Dateien an alle mglichen Befehlsstze angepasst werden.
//							   
//	Copyright Anton Zechner (C) 2003
//
//	Beispiel:	AsmEntryInfo	info;
//				unsigned     	offs;
//				unsigned     	page=0;
//				unsigned    	addr=0x00400000;
//				unsigned char	data[]={0x12,0x34,0x12,0x67,0xAA,0x34,0x98,0x12};
//
//				AsmInit("I386.acf",0,0);
//				AsmSetFormat(8,6);
//
//				for(offs=0;offs<sizeof(data);)
//					{
//					offs+=AsmDecode(data+offs,&info,addr+offs,0,page);
//					printf("\n%s",info.line_text);
//					page=info.page;
//					}

#include	<stdio.h>
#include	<malloc.h>
#include	"AZ_Lib.h"
#include	"FileBuffer.h"
#include	"AsmDecoder.h"
#include	"BitFunctions.h"
#include	"StringFunctions.h"

#define 	LINE_BRT   1024				// Maximale Breite einer Zeile
#define 	LINE_MAX	  2				// Maximale Anzahl der Zeilen
#define 	MASKEN_CNT	  4				// Maximale Anzahl der Maskenseizen
#define 	MASKEN_MAX 1024				// Maximale Anzahl der Masken
#define 	MARKEN_MAX  512				// Maximale Anzahl der Sprungmarken
#define 	RTEXT_MAX	512				// Maximale Anzahl der Registernamen
#define 	BFIELD_MAX	  4				// Maximale Anzahl der Bitfelder
#define 	ALIAS_MAX    64				// Maximale Anzahl der Alias Namen
#define 	LTEXT_MAX	 16				// Maximale Anzahl der Layout   Texte
#define 	STEXT_MAX	 16				// Maximale Anzahl der Segment  Texte
#define 	ENTRY_MAX	  8				// Maximale Anzahl der Eintrge
#define 	TEXT_MAX	  8				// Maximale Anzahl an  Ausgabetexten
#define 	BEF_MAX		512				// Maximale Anzahl der Befehle
#define 	M_MAX		128				// Maximale Anzahl der Speicher

#ifndef    _MSC_VER
#define    _cdecl
#define   __cdecl
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


typedef	struct AsmCommand
		{
		void	  (*proc)(AsmCommand*);	// Prozetur
		ASM_DWORD 	m1;					// M1 oder Zahl1
		ASM_DWORD	m2;					// M2 oder Zahl2
		ASM_DWORD	and;				// AND Wert
		ASM_DWORD	or;					// OR/XOR Wert oder Marke
		ASM_DWORD	shift;				// +/-/>>/<< Wert oder Vergleichstyp
		ASM_DWORD	type;				// Typ Nummer
		ASM_DWORD	flags;				// Flags was ist angegeben
		char	   *text;				// Text
		char	   *text2;				// Text2
		#ifdef     _DEBUG
		int			zeile;				// Zeilen nummer in der *.ACF Datei
		#endif
		}AsmCommand;

#ifdef _DEBUG
		char  		   *dsub;			// Debug-Subprogramm
		char  		   *dzeile;			// Debug-Zeile
		CStringChain	dtext;			// Debug-Texte
#endif


typedef	struct							// Standard Befehlsliste
		{
		char	    *name;
		void	   (*proc)(AsmCommand*);
		unsigned 	 type;
		}Befehl;

typedef	struct							// M Speicher Bereiche
		{
		ASM_DWORD  *ptr;				// Zeiger auf Speicher
		ASM_DWORD	f_and;				// AND Zahl fr AsmEntry.flags
		ASM_DWORD	f_or;				// OR  Zahl fr AsmEntry.flags
		ASM_DWORD	l_and;				// AND Zahl fr AsmEntry.layout
		ASM_DWORD	l_or;				// OR  Zahl fr AsmEntry.layout
		ASM_DWORD	c_and;				// AND Zahl fr AsmEntry.clips
		ASM_DWORD	c_or;				// OR  Zahl fr AsmEntry.clips
		ASM_DWORD	offset;				// Offset in AsmEntry
		}Speicher;

typedef	struct							// Standard Befehlsliste
		{
		ASM_DWORD      bits;			// Vergleichs Bits
		ASM_DWORD	   maske;			// Und-Maske
		ASM_BYTE       add_c;			// Code  Addition
		ASM_BYTE       add_d;			// Daten Addition
		char		  *name;			// Befehls Namen
		unsigned char *prog;			// Sub-Program Code
		}Maske;

typedef	struct							// Textkonstanten
		{
		char   **ptr;					// Vergleichs Bits
		char    *text;					// Befehls Namen
		}TextConst;

static unsigned	 AsmGetFlags(char *Mem,int &Add);
static int		 AsmLoadMask(CTextBuffer &file,int &zeile);
static void		 AsmTextInsert(char *Text,int Select,int Pos);
static void		 AsmSET64(AsmCommand *Bef);
static void		 AsmSET  (AsmCommand *Bef);
static void		 AsmGET  (AsmCommand *Bef);
static void		 AsmIF   (AsmCommand *Bef);
static void		 AsmNEXT (AsmCommand *Bef);
static void		 AsmXCHG (AsmCommand *Bef);
static void		 AsmGOTO (AsmCommand *Bef);
static void		 AsmTEXT (AsmCommand *Bef);
static void		 AsmPRINT(AsmCommand *Bef);
static void		 AsmBIT  (AsmCommand *Bef);
static void		 AsmRLIST(AsmCommand *Bef);

//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************

static char		 line    [LINE_MAX*LINE_BRT*+4];		// Textspeicher
static int  	 line_len[LINE_MAX+1];					// Zeilen Lngen
static int  	 line_pos	 = 0;						// Aktuell Ausgabeposition in der Linie
static int		 line_nr	 = 0;						// Aktuell Liniennummer

static ASM_DWORD match		 = 0;						// Match Bitfeld fr MaskenSort
static ASM_DWORD pre_bits	 = 0;						// Bestimmt wo die LTEXTE liegen sollen
static ASM_DWORD cut_sign    = 0;						// Spezielle Trennzeichen
static ASM_DWORD code_type   = 0;						// Befehls Type
static ASM_DWORD bit_dir	 = 0;						// Bit-Hol-Richtung
static ASM_DWORD seg0on      = 0;						// Standard Segment ausgeben
static ASM_DWORD seg_pos	 = 0;						// Position der Segment-Angabe
static ASM_DWORD flags_take	 = 0;						// Mit genommene Bits in flags
static ASM_DWORD matchset	 = 0;						// Bei welcen Bit soll gesucht werden
static ASM_DWORD no_end      = 0;						// Nchsten Befehl holen
static ASM_DWORD mono_lt     = 0;						// LTEXTE auch ohne aktiviertem Eintarg
static ASM_DWORD pre_plus	 = 0xFF00;					// Vor welche LTEXT darf ein + gesetzt werden
static ASM_DWORD textsel     = 0;						// Auswahltext
static ASM_DWORD byte_order	 = 0;						// Byteorder fr Maskenvergleich
static ASM_DWORD code_addr[2]={0,0};					// bergabe Paramerter
static ASM_DWORD gparam[2];								// Standard Paramerter
static ASM_DWORD param[4];								// bergabe Paramerter
static ASM_DWORD m_mem[8];								// Speicher Zellen
static ASM_DWORD t_brt[8]={8,16,32,64,0,0x80,0x88,0x90};// Standard Ausgabe-Breiten

static char     *alias [ALIAS_MAX*2];					// Segment  Texte
static char     *stext [STEXT_MAX];						// Segment  Texte
static char     *ltext [LTEXT_MAX];						// Layout   Texte
static char     *rtext [RTEXT_MAX];						// Registernamen
static char		 fname [MAX_PATH ];						// Name der *.acf Datei
static char	 	 bfield[BFIELD_MAX][32];				// Bitfelder

static TextConst ktext[]=								// Textkonstanten
			{
			&ltext[ 0]		, 0   ,
			&ltext[ 1]		, "\xFF\x01\x01" "%"   "\x00\x02",
			&ltext[ 2]		, "\xFF\x02\x01" "++"  "\x00\x03",
			&ltext[ 3]		, "\xFF\x02\x01" "--"  "\x00\x03",
			&ltext[ 4]		, "\xFF\x01\x01" "#"   "\x00\x02",
			&ltext[ 5]		, "\xFF\x01\x01" "@"   "\x00\x02",
			&ltext[ 6]		, "\xFF\x02\x01" "++"  "\x00\x03",
			&ltext[ 7]		, "\xFF\x02\x01" "--"  "\x00\x03",
			&ltext[ 8]		, "\xFF\x01\x01" "*"   "\x00\x02",
			&ltext[ 9]		, "\xFF\x01\x01" "%"   "\x00\x02",
			&ltext[10]		, "\xFF\x02\x01" "++"  "\x00\x03",
			&ltext[11]		, "\xFF\x02\x01" "--"  "\x00\x03",
			&ltext[12]		, "\xFF\x01\x01" "#"   "\x00\x02",
			&ltext[13]		, "\xFF\x01\x01" "@"   "\x00\x02",
			&ltext[14]		, "\xFF\x02\x01" "++"  "\x00\x03",
			&ltext[15]		, "\xFF\x02\x01" "--"  "\x00\x03",
			&stext[ 0]		, 0   ,
			&stext[ 1]		, "\xFF\x01\x01" ":"   "\x00\x02",
			&stext[ 2]		, "\xFF\x01\x01" ","   "\x00\x02",
			&stext[ 3]		, "\xFF\x03\x01" " - " "\x00\x04",
			&stext[ 4]		, "\xFF\x03\x01" " + " "\x00\x04",
			&stext[ 5]		, "\xFF\x03\x01" " | " "\x00\x04",
			&stext[ 6]		, "\xFF\x03\x01" " * " "\x00\x04",
			&stext[ 7]		, "\xFF\x03\x01" " ^ " "\x00\x04",
			&stext[ 8]		, "\xFF\x03\x01" " & " "\x00\x04",
			&stext[ 9]		, "\xFF\x04\x01" " & !""\x00\x05",
			&stext[10]		, "\xFF\x02\x01" " !"  "\x00\x03",
			&stext[11]		, "\xFF\x03\x01" " = " "\x00\x04",
			&stext[12]		, "\xFF\x03\x01" " / " "\x00\x04",
			&stext[13]		, "\xFF\x03\x01" " % " "\x00\x04",
			&stext[14]		, "\xFF\x02\x01" ",{"  "\x00\x03",
			&stext[15]		, "\xFF\x01\x01" "}"   "\x00\x02",
			0,0};

static int		  entry_nr =0;			 				// Aktueller Eintrag
static int		  entry_pos[ENTRY_MAX+1];				// Linien Positionen
static AsmEntry	 *entry_ptr[ENTRY_MAX+1];				// Zeiger fr Eintrge
static AsmEntry	  entry    [ENTRY_MAX+1];				// Eintge
static AsmEntry   gentry;				 				// Globaler Eintrag
static AsmCommand befehl[BEF_MAX];		 				// Befehl Speicher
static int		  befehl_anzahl= 0;		 				// Anzahl der Befehle
static int		  befehl_nr    = 0;		 				// Aktueller Befehl
static int		  alias_anzahl = 0;		 				// Anzahl der Alias Namen

static unsigned char *code_mem;			 				// Zeiger auf Code
static			int   code_bit;			 				// Nummer der Bits im Code
static			int   data_bit;			 				// Nummer der Bits in Daten

#define 	BEF_CONST		0			 				// Spezialbefehle
#define 	BEF_MASK		1
#define 	BEF_STEXT		2
#define 	BEF_LTEXT		3
#define 	BEF_RTEXT		4
#define 	BEF_BITFIELD	5
#define 	BEF_SETMATCH	6
#define 	BEF_ALIAS		7
#define 	BEF_MACRO		8
#define 	BEF_INFO		9
#define 	BEF_GOTO		10

#define 	BEF_TYPE_TEXT2	0x80000000

static	Befehl  syntax[]=								// Befehlsnamen und Prozeturen
		{
		"CONST"			,0,0,
		"MASK"			,0,0,
		"STEXT"			,0,0,
		"LTEXT"			,0,0,
		"RTEXT"			,0,0,
		"BFIELD"		,0,0,
		"SETMATCH"		,0,0,
		"ALIAS"			,0,BEF_TYPE_TEXT2,
		"MACRO"			,0,BEF_TYPE_TEXT2,
		"INFO"			,0,BEF_TYPE_TEXT2,
		"GOTO"			,AsmGOTO	,0,
		"GET"			,AsmGET		,0,
		"SET"			,AsmSET		,0,
		"AND"			,AsmSET		,1,
		"OR"			,AsmSET		,2,
		"XOR"			,AsmSET		,3,
		"ADD"			,AsmSET		,4,
		"SUB"			,AsmSET		,5,
		"MUL"			,AsmSET		,6,
		"DIV"			,AsmSET		,7,
		"MOD"			,AsmSET		,8,
		"SHIFT"			,AsmSET		,9,
		"ROL"			,AsmSET		,10,
		"ROR"			,AsmSET		,11,
		"NEXT"			,AsmNEXT	,0,
		"XCHG"			,AsmXCHG	,0,
		"IF"			,AsmIF		,0,
		"TEXT"			,AsmTEXT	,0,
		"PRINT"			,AsmPRINT	,0,
		"BIT"			,AsmBIT		,0,
		"SET64"			,AsmSET64	,0,
		"AND64"			,AsmSET64	,1,
		"OR64"			,AsmSET64	,2,
		"XOR64"			,AsmSET64	,3,
		"ADD64"			,AsmSET64	,4,
		"SUB64"			,AsmSET64	,5,
		"MUL64"			,AsmSET64	,6,
		"DIV64"			,AsmSET64	,7,
		"MOD64"			,AsmSET64	,8,
		"SHIFT64"		,AsmSET64	,9,
		"ROL64"			,AsmSET64	,10,
		"ROR64"			,AsmSET64	,11,
		"ADD64S"		,AsmSET64	,12,
		"RLIST"			,AsmRLIST	,0,
		"END"			,0,0
		};


static Speicher m[M_MAX]=
	{
	// ADR					AND        OR         TYPE
	&gparam[0]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gparam[1]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&param [0]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&param [1]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&seg0on				,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&seg_pos			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.flags 		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.layout 		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.clips 		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.type 		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.regs [0]	,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.regs [1]	,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.regs [2]	,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.regs [3]	,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.value0		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.value0_h	,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.value1		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&gentry.value1_h	,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[0]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[1]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[2]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[3]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[4]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[5]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[6]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&m_mem[7]			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	0        			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.flags     -&gentry.flags,
	0        			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.layout    -&gentry.flags,
	0        			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.clips     -&gentry.flags,
	0        			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.type      -&gentry.flags,
	0        			,~0x00000100,0x00000100,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.regs   [0]-&gentry.flags,
	0        			,~0x00000200,0x00000200,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.regs   [1]-&gentry.flags,
	0        			,~0x00000400,0x00000400,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.regs   [2]-&gentry.flags,
	0        			,~0x00000800,0x00000800,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.regs   [3]-&gentry.flags,
	0        			,~0x00001000,0x00001000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.value0    -&gentry.flags,
	0        			,~0x00001000,0x00001000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.value0_h  -&gentry.flags,
	0        			,~0x00002000,0x00002000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.value1    -&gentry.flags,
	0        			,~0x00002000,0x00002000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.value1_h  -&gentry.flags,
	0        			,~0x00004000,0x00004000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.regs   [2]-&gentry.flags,
	0        			,~0x00008000,0x00008000,~0x00000000,0x00000000,~0x00000000,0x00000000,&gentry.regs   [3]-&gentry.flags,
	&textsel			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&no_end				,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&code_addr[0]		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&code_addr[1]		,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&bit_dir			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&code_type			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&pre_bits			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&cut_sign			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&byte_order			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&flags_take			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&mono_lt			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	&pre_plus			,~0x00000000,0x00000000,~0x00000000,0x00000000,~0x00000000,0x00000000,0x00000000,
	};

static char	*m_name[]=									// Namen fr Speicher
	{
	"g_param0",		"g_param1",
	"param0",		"param1",
	"seg_on",		"segment_pos",
	"g_flags",		"g_layout",
	"g_clips",		"g_type",
	"g_reg0",		"g_reg1",
	"g_reg2",		"g_reg3",
	"g_value0",		"g_value1",
	"g_value0h",	"g_value1h",
	"m0",	"m1",	"m2",	"m3",
	"m4",	"m5",	"m6",	"m7",
	"flags",		"layout",
	"clips",		"type",
	"reg0",			"reg1",
	"reg2",			"reg3",
	"value0",		"value0_h",
	"value1",		"value1_h",
	"reg2n",		"reg3n",
	"select",		"no_end",
	"code_addres",	"code_addres_h",
	"bit_direction","code_type",
	"pre_bits",		"cut_sign",
	"byte_order",	"flags_take",
	"mono_lt",		"pre_plus",
	0
	};

static int				m_anzahl      = 0;				// Anzahl der Speicher
static int				masken_seiten = 0;				// Anzahl der geladenen Masken
static Maske			masken       [MASKEN_MAX];		// Masken
static unsigned char	masken_bits  [MASKEN_CNT];		// Byte Raster fr Masken
static int	 			masken_start [MASKEN_CNT*257];	// Byte Raster fr Masken
static int   			masken_offset[MASKEN_CNT+1];	// Wo beginne die Maskenseiten
static int   			masken_anzahl[MASKEN_CNT];		// Anzahl der Masken
static int	 			masken_match [MASKEN_CNT];		// Standard Rasterposition
static int   			sub_bef['z'-'A'+1];				// Marken fr Einsprnge
static int   			sub_def['z'-'A'+1];				// Sind Einsprnge definiert
static CStringChain		macros;							// Makrospeicherspeicher
static CStringChain		gtext;							// Textspeicher
static CStringChain		ptext;							// Textspeicher
static CStringChain		infos(sizeof(int));				// Kommandonamen
static CStringChain		info_txt;						// Kommandoinfos	

static int   			xchg_nr[16];					// Eintrge tauschen
static int   			xchg_anz=0;						// Eintrge Anzahl
static int   			addr_anz=0;						// Ausgabe Breite fr Addresen
static int   			data_anz=0;						// Ausgabe Breite fr Code-Bytes
static int   			bef_anz =7;						// Minimale Breite einer Befehlsangabe
static int   			btext_sub[TEXT_MAX];			// Befehls Text-Sub Nummern
static char* 			btext_ptr[TEXT_MAX];			// Befehls Texte
static int   			btext_anz=0;					// Befehls Text Anzahl
static int   			ntext_sub[TEXT_MAX];			// Nach Text-Sub Nummern
static char* 			ntext_ptr[TEXT_MAX];			// Nach Texte
static int   			ntext_anz=0;					// Nach Text Anzahl
static int   			ltext_sub[TEXT_MAX];			// Line Texte Sub Nummern
static char* 			ltext_ptr[TEXT_MAX];			// Line Texte
static int   			ltext_anz=0;					// Line Text Anzahl
static int   			vtext_sub[TEXT_MAX];			// Vor  Texte Sub Nummern
static char* 			vtext_ptr[TEXT_MAX];			// Vor  Texte
static int   			vtext_anz=0;					// Vor  Text Anzahl

static char	 			hext_chars[]="0123456789ABCDEF";


#define 	BEF_END		(sizeof(syntax)/sizeof(syntax[0]))

//*****************************************************************************
//*
//*		Flags fr befehl[].flag
//*
//*****************************************************************************

#define 	B_ZAHL			0x000000FF					// Bit Zahl
#define 	B_AKT_WBRT		0x00000100					// 'A' Aktuelle Wortbreite
#define 	B_AKT_BBRT		0x00000200					// 'B' Aktuelle Befehlsbreite
#define 	B_NO_BITADD		0x00000400					// 'N' Bitadresse nicht erhhen
#define 	B_ADR_0			0x00000800					// 'S' Startadrese = 0
#define 	B_ADR_DATA		0x00001000					// 'D' Startadrese = Datenadresse
#define 	B_L_TO_B		0x00002000					// 'G' Littel to Big Indian
#define 	B_MISCOUNT		0x00004000					// 'M' M2 ist Bit Zahl
#define 	B_UNSIGNED		0x00008000					// 'U' Auf Unsigned erweitern
#define 	B_FLAGS			0x0000FFFF					// Programm Befehls Flags

#define 	B_AND     		0x80000000					// &... wurde angegeben
#define 	B_OR     		0x40000000					// |... wurde angegeben
#define 	B_XOR     		0x20000000					// ^... wurde angegeben
#define 	B_SHIFT    		0x10000000					// <<   wurde angegeben
#define 	B_M1      		0x08000000					// M1   wurde angegeben
#define 	B_M2      		0x04000000					// M2   wurde angegeben
#define 	B_ZAHL1   		0x02000000					// Zahl wurde angegeben
#define 	B_ZAHL2   		0x01000000					// Zahl wurde angegeben
#define 	B_WERT    		0x00800000					// Wert wurde angegeben
#define 	B_TEXT     		0x00400000					// Text wurde angegeben
#define 	B_MARKE    		0x00200000					// Goto wurde angegeben
#define 	B_CMP      		0x00100000					// ==   wurde angegeben
#define 	B_LAND     		0x00080000					// Layout AND wurde angegeben
#define 	B_LOR      		0x00040000					// Layout OR  wurde angegeben
#define 	B_CAND     		0x00020000					// Clips  AND wurde angegeben
#define 	B_COR      		0x00010000					// Clips  OR  wurde angegeben

//*****************************************************************************
//*
//*		AsmError
//*
//*****************************************************************************
//	Fehlerausgabe fr den AsmDecoder
//	pFormat		: Zeiger auf das printf Format
//	iVal1		: Erster Parameter
//	iVal2		: Zweiter Parameter
static void AsmError(const char *pFormat,int iVal1=0,int iVal2=0)
{
	printf(pFormat,iVal1,iVal2);
}


//*****************************************************************************
//*
//*		AsmError
//*
//*****************************************************************************
//	Fehlerausgabe fr den AsmDecoder
//	pFormat		: Zeiger auf das printf Format
//	pParam1		: Erster Parameter
//	iVal2		: Zweiter Parameter
static void AsmError(const char *pFormat,void *pParam1,int iVal2=0)
{
	printf(pFormat,pParam1,iVal2);
}

//*****************************************************************************
//*
//*		AsmBigEndian
//*
//*****************************************************************************
//	Ist der BigEndian Modus fr Masken eingeschalten
int AsmBigEndian()
{
return byte_order;
}

//*****************************************************************************
//*
//*		AsmExit
//*
//*****************************************************************************
//	Alle speicher freigeben
void AsmExit()
{

	gtext.Del();										// Textspeicher leeren
	infos.Del();										// Kommandonamen
	info_txt.Del();										// Kommandoinfos	

}

//*****************************************************************************
//*
//*		AsmReInit
//*
//*****************************************************************************
//	Ld die Datei Filename die den ASM-Befehlscode beschreibt
//	Param1 und Param2 sind die bergabe-Paramerter fr die Init-Prozetur
//	Ergibt die Anzahl der geladenen Befehle
int AsmReInit()
{
return AsmInit(fname,param[2],param[3]);
}

//*****************************************************************************
//*
//*		AsmInit
//*
//*****************************************************************************
//	Ld die Datei Filename die den ASM-Befehlscode beschreibt
//	Param1 und Param2 sind die bergabe-Paramerter fr die Init-Prozetur
//	Ergibt die Anzahl der geladenen Befehle
int AsmInit(const char *Filename,unsigned Param1,unsigned Param2)
{
char		   *text,*bptr,*mem;
char 			puffer[1024];
char			par1[32],par2[32];
unsigned		val,l_and=0,l_or=0,c_and=0,c_or=0;
CTextBuffer		file;
CStringChain	m_namen;
CStringChain	marken_namen(sizeof(int));
int				marken[MARKEN_MAX];
int				marken_anzahl;
int				x,y,z,add,anz;
int				zeile,bef,nr;
int				mask,bits,ok;


	gtext .Del();								// Textspeicher leeren
	macros.Del();								// Mackrospeicher leeren
	macros.DataLength(sizeof(int    ));			//			*
	memset( sub_bef,0,sizeof(sub_bef));			// Marken fr Einstrnge
	memset( sub_def,0,sizeof(sub_def));			// Sind Einsprnge definiert
	memset( marken ,0,sizeof(marken ));			// Sprung Marken lschen
	memset(&gentry ,0,sizeof(gentry ));			// Globale Daten lschen
	memset( rtext  ,0,sizeof(rtext  ));			// Register Texte lschen
	memset( m_mem  ,0,sizeof(m_mem  ));			// Speicher leeren
	memset( alias  ,0,sizeof(alias  ));			// Aliasnamen lschen
	marken_anzahl='z'-'A'+1;					// Sprung Marken Anzahl
	befehl_anzahl=nr= 1;						// Anzahl der Befehle
	masken_seiten   = 0;						// Masken zurcksetzen
	alias_anzahl	= 0;						// Anzahl der Alias Namen
	befehl[0].proc  = 0;						// END Befehl
	gparam[0]		= 0;						// Parameter lschen
	gparam[1]		= 0;						//			*
	byte_order		= 0;						// Byteorder fr Maskenvergleich
	mono_lt			= 0;						// LTEXTE auch ohne aktiviertem Eintarg
	flags_take		= 0x88FF0000;				// Mit genommene Bits in flags
	pre_plus		= 0xFF00;					// Vor welche LTEXT darf ein + gesetzt werden
	pre_bits 		= 0xF1F1;					// Bestimmt wo die LTEXTE liegen sollen
	textsel  		= 0;						// Auswahltext
	befehl_nr		= 0;						// Anktueller Befehl
	code_type		= 0;						// Befehls Type
	xchg_anz 		= 0;						// Keine Eintge tauschen
	cut_sign 		= 0;						// Spezielle Trennzeichen
	line_nr  		= 0;						// Aktuell Liniennummer
	bit_dir	 		= 0;						// Bit-Hol-Richtung
	seg_pos	 		= 0;						// Position der Segment-Angabe
	mask	 		= 0;						// Maskenmodus beim Laden


	for(m_anzahl=0;m_name[m_anzahl];m_anzahl++)	// Speicher rcksetzen
		{
		m_namen.Add(m_name[m_anzahl]);
		}

	for(x=0;ktext[x].ptr;x++)					// Textkonstanten laden
		{
		*ktext[x].ptr=ktext[x].text;
		}

	memset(masken        ,0,sizeof(masken       ));
	memset(masken_start  ,0,sizeof(masken_start ));
	memset(masken_anzahl ,0,sizeof(masken_anzahl));
	memset(masken_offset ,0,sizeof(masken_offset));
	memset(masken_match  ,0,sizeof(masken_match ));
	memset(masken_bits,0xFF,sizeof(masken_bits  ));

	if(!file.Open(Filename))					// Datei ffnen
		{
		AsmError("\nKann Datei nicht ffnen.");
		return 0;
		}

   strncpy(fname,Filename,sizeof(fname)-1);

//******************** Syntax scannen *****************************************
add=0;zeile=1;ok=1;

	itoa(Param1,par1,10);
	itoa(Param2,par2,10);


	while(ok && file.Text(add,&mem,1024))
		{
		if(add=file.TextEmpty())				// Leerzeichen entfernen
			{
			file.Text(add,&mem,1024);
			}

		if(*mem==';')							// REM-Marke gefunden
			{
			file.TextLine();
			zeile++;
			add=0;
			continue;
			}

		if(*mem=='\n'){add=1;zeile++;continue;}	// Zeilenende
		if(*mem=='\r'){add=1;        continue;}

		if(mem[0]=='@')							// Makro gefunden
			{
			file.Text(1,&mem,1024);
			file.TextWord(puffer,sizeof(puffer),&add);

			   y = macros.FindFast(add,puffer);
			if(y<0)
				{
				AsmError("Unbekanntes Makro in Zeile %i",zeile);
				z=0;
				}
			else{
				z=macros.DataGet(y);
				}

			if(z==0)							// Zeile berspringen
				{
					file.Text(add,&mem,1024);
				add=file.TextEmpty();
					file.Text(add,&mem,1024);

				if(mem[0]==':' && mem[1]>='A' && mem[1]<='z' && !strwchar(mem[1],0x0F))
					{
					x =mem[1]-'A';
					sub_def[x]=1;
					}

				file.TextLine();
				zeile++;
				add=0;
				continue;
				}

			continue;
			}

		if(mem[0]==':')							// Marke gefunden
			{
			file.Text(  1,&mem,1024);
			file.TextWord(puffer,sizeof(puffer),&anz);

			if(anz==1 && puffer[0]>='A' && puffer[0]<='z')
				{
				x=puffer[0]-'A';
				sub_def[x] = 2;
				marken [x] = nr;
				}
			else{
				x=marken_namen.Find(puffer);
				if(x!=-1)marken[x+'z'-'A'+1]=nr;
				else{
					x=marken_namen.Add(anz,puffer,CSC_END,&nr);
					marken[marken_anzahl]=nr;
					marken_anzahl++;
					if(marken_anzahl>=MARKEN_MAX)
						{
						AsmError("Zuviele Marken wurden angegeben in Zeile %i",zeile);
						ok=0;
						break;
						}
					}
				}
			add=anz;
			continue;
			}

			anz=file.TextWordLen();				// Befehl suchen
		if(!anz)
			{
			AsmError("Erwarte Befehl in Zeile %i",zeile);
 			file.TextLine();
			zeile++;add=0;
			continue;
			}

												// Welcher Befehl
		for(bef=0;bef<BEF_END;bef++)
			{
			if((int)strlen (syntax[bef].name)  !=anz )continue;
			if(    !strncmp(mem,syntax[bef].name,anz))break;
			}

		if(bef>=BEF_END)
			{
			AsmError("Unbekannter Befehl in Zeile %i",zeile);
			zeile++;file.TextLine();
			continue;
			}

		if(mask && bef>=BEF_MASK)				// Nur mehr Masken erlaubt
			{
			AsmError("Nur mehr MASK-Befehler erbaubt in Zeile %i",zeile);
			zeile++;file.TextLine();
			continue;
			}


		if(bef!=BEF_GOTO)file.Text(anz,&mem,1024);

		befehl[nr].m1   = 0;
		befehl[nr].m2   = 0;
		befehl[nr].and  =~0;
		befehl[nr].or   = 0;
		befehl[nr].shift= 0;
		befehl[nr].flags= 8;
		befehl[nr].text = 0;
		befehl[nr].text2= 0;
		#ifdef	_DEBUG
		befehl[nr].zeile= zeile;
		#endif
		bits=0x08;add=0;

		for(x=0;x<10;x++)
			{
			file.Text(add,&mem,1024);
			//******************************** Leerzeichen entfernen
			if(add=file.TextEmpty())
				{
				file.Text(add,&mem,1024);
				}
			//******************************** Zeilenende
			if(mem[0]=='\0'){add=0;break;}
			if(mem[0]=='\n'){add=0;break;}
			if(mem[0]=='\r'){add=0;break;}
			//******************************** Text einlesen
			if(mem[0]=='\"')
				{
				add=file.TextString(puffer+3,255,&anz);

				puffer[0] = '\xFF';
				puffer[1] = anz;

				for(z=1,y=0;y<anz;y++)
					{
					if(puffer[y+3]!=':')continue;
					puffer[3+anz+z]=y+1;
					z++;
					}

				puffer[2]       = z;
				puffer[3+anz+z] = y+1;
				anz            += 3+z+1;

				if(bits&B_TEXT)
					{
					if(syntax[bef].type&BEF_TYPE_TEXT2 && !befehl[nr].text2)
						{
						y=gtext.Add(anz,puffer);
						befehl[nr].text2=gtext[y];
						continue;
						}
					AsmError("Mehrfache Textangabe");
					continue;
					}
				else{
					y=gtext.Add(anz,puffer);
					befehl[nr].text=gtext[y];
					}

				bits|=B_TEXT;
				continue;
				}
			//******************************** Zahlen einlesen
			y=(int)mem[0]&0xFF;
			z=(strcspn((char*)&y,"&|^+-*/$!"))? 0:1;
			if(mem[0]=='>' && mem[1]=='>')z=2;
			if(mem[0]=='<' && mem[1]=='<')z=2;
			if(z)
				{
				for(;z<32;z++)if(mem[z]!=' ' && mem[z]!='\t')break;
				if(mem[z]=='~')z++;
				y=strtoi(mem+z,0x8000,&add);add+=z;
				if(mem[z-1]=='~')y=~y;
				switch(mem[0])
						{
				case '&':befehl[nr].and=y;	 				bits|= B_AND;	break;
				case '|':befehl[nr].or =y;	 bits&=~B_XOR;	bits|= B_OR;	break;
				case '^':befehl[nr].or =y;	 bits&=~B_OR;	bits|= B_XOR;	break;
				case '+':befehl[nr].shift= y;bits&=~B_WERT; bits|= B_WERT;	break;
				case '-':befehl[nr].shift=-y;bits&=~B_WERT; bits|= B_WERT;	break;
				case '<':befehl[nr].shift= y;bits&=~B_SHIFT;bits|= B_SHIFT;	break;
				case '>':befehl[nr].shift=-y;bits&=~B_SHIFT;bits|= B_SHIFT;	break;
				case '*':l_and           = y;				bits|= B_LAND;	break;
				case '/':l_or            = y;				bits|= B_LOR;	break;
				case '$':c_and           = y;				bits|= B_CAND;	break;
				case '!':c_or            = y;				bits|= B_COR;	break;
						}
				continue;
				}
			//******************************** Zahlen einlesen
			if(mem[0]=='~' || (mem[0]>='0' && mem[0]<='9'))
				{
				z=(mem[0]=='~')? 1:0;
				y=strtoi(mem+z,0x8000,&add);
				if(z){y=-y;add++;}
				if(bits&(B_M1|B_ZAHL1))
					{
					befehl[nr].m2=y;
					bits&=~B_M2;
					bits|= B_ZAHL2;
					}
				else{
					befehl[nr].m1=y;
					bits&=~B_M1;
					bits|= B_ZAHL1;
					}
				continue;
				}
			//******************************** Flags einlesen
			if(mem[0]=='#')
				{
				bits&=~B_FLAGS;
				bits|=AsmGetFlags(mem,add);
				continue;
				}
			//******************************** Vergleiche einlesen
			add=0;

			if((mem[0]=='!')&&(mem[1]=='=')){y=0;add=2;}
			if((mem[0]=='=')&&(mem[1]=='=')){y=1;add=2;}
			if((mem[0]=='>')&&(mem[1]=='=')){y=2;add=2;}
			if((mem[0]=='<')&&(mem[1]=='=')){y=3;add=2;}
			if((mem[0]=='>')			   ){y=4;add=1;}
			if((mem[0]=='<')			   ){y=5;add=1;}
			if(add)
				{
				befehl[nr].shift=y;
				bits&=~(B_SHIFT|B_WERT);
				bits|=  B_CMP;
				continue;
				}
			//******************************** GOTO einlesen
			if(!strnicmp(mem,"GOTO",4) && (mem[4]==' ' || mem[4]=='\t'))
				{
					file.Text(5,&mem,1024);
				add=file.TextEmpty();
					file.Text(add,&mem,1024);
					file.TextWord(puffer,sizeof(puffer),&anz);

				if(anz==1 && puffer[0]>='A' && puffer[0]<='z')
					{
					befehl[nr].or=puffer[0]-'A';
					}
				else{
					   y=marken_namen.Find(puffer);
					if(y!=-1)befehl[nr].or=y+'z'-'A'+1;
					else{
						y=marken_namen.Add(anz,puffer,CSC_END,&nr);
						befehl[nr].or=y+'z'-'A'+1;
						marken_anzahl++;
 						if(marken_anzahl>=MARKEN_MAX)
							{
							AsmError("Zuviele Marken wurden angegeben in Zeile %i",zeile);
							ok=0;
							break;
							}
						}

					}
				bits&=~(B_OR|B_XOR);
				bits|=  B_MARKE;
				add=anz;
				continue;
				}
			//******************************** Gleich Zeichen
			if(mem[0]=='='){add=1;continue;}
			if(mem[0]==','){add=1;continue;}
			//******************************** REM Marke
			if(mem[0]==';')
				{
				add=file.TextLineLen(1);
					file.Text(add);
				add=0;
				break;
				}
			//******************************** M1/M2
			file.TextWord(puffer,sizeof(puffer),&add);
			y=m_namen.Find(puffer);
			if(y!=-1)
				{
				if(bits&(B_M1|B_ZAHL1))
					{
					befehl[nr].m2=y;
					bits&=~B_ZAHL2;
					bits|= B_M2;
					}
				else{
					befehl[nr].m1=y;
					bits&=~B_ZAHL1;
					bits|= B_M1;
					}
				continue;
				}

			AsmError("Unbekannter Speicherbereich in Zeile %i",zeile);
			add=1;
			break;
			}
		//**************** MASK  Befehl ***************************************
		if(bef==BEF_MASK)
			{
			if(masken_seiten>=MARKEN_MAX)
				{
				AsmError("Zuviele Maskenseiten wurden angegeben in Zeile %i",zeile);
				break;
				}

			file.TextLine();						// Zeilenende
			zeile++;add=0;
			AsmLoadMask(file,zeile);
			mask=1;
			continue;
			}
		//**************** CONST Befehl ***************************************
		if(bef==BEF_CONST)
			{
			file.TextLine();						// Zeilenende
			zeile++;
			add=0;

			if(~bits&B_TEXT)continue;				// Kein Text

				text = befehl[nr].text;
			if(*text=='\xFF')text+=3;

			if(m_namen.Find(text)!=-1)
				{
				AsmError("Konstante (%s) ist bereits definiert in Zeile %i",text,zeile-1);
				continue;
				}

			z=befehl[nr].m1;

			m[m_anzahl].f_and =0xFFFFFFFF;			// Keine Bits setzen
			m[m_anzahl].f_or  =0x00000000;
			m[m_anzahl].l_and =0xFFFFFFFF;
			m[m_anzahl].l_or  =0x00000000;
			m[m_anzahl].c_and =0xFFFFFFFF;
			m[m_anzahl].c_or  =0x00000000;
			m[m_anzahl].offset=0;

			if( bits&B_M1)							// Alias Speicher
				{
				m[m_anzahl]=m[z];
				}
			else{									// Neuer Speicher
				if(~bits&B_ZAHL1)z=0;
				m[m_anzahl].ptr=&m[m_anzahl].offset;
				m[m_anzahl].offset=z;
				}

			if(bits&B_AND )m[m_anzahl].f_and=befehl[nr].and;
			if(bits&B_OR  )m[m_anzahl].f_or =befehl[nr].or;
			if(bits&B_LAND)m[m_anzahl].l_and=l_and;
			if(bits&B_LOR )m[m_anzahl].l_or =l_or;
			if(bits&B_CAND)m[m_anzahl].c_and=c_and;
			if(bits&B_COR )m[m_anzahl].c_or =c_or;

			m_namen.Add(text);						// Text einfgen
			m_anzahl++;

			if(m_anzahl>=M_MAX)
				{
				AsmError("Zu viele Konstanten wurden definiert in Zeile %i",zeile);
				ok=0;
				}

			continue;
			}
		//**************** SETMATCH Befehl ************************************
		if(bef==BEF_SETMATCH)
			{
			file.TextLine();						// Zeilenende
			zeile++;
			add	= 0;
			y	= befehl[nr].m1;
			z	= befehl[nr].m2;

			if(~bits&B_ZAHL2)z=0;
			if(~bits&B_ZAHL1)continue;				// Keine Zahl
			if(z >MASKEN_CNT)continue;


			   masken_match[z] = y&0x1F;
			   masken_bits [z] = ((y>>8)&0xFF);
			if(masken_bits [z]==0)
				{
				masken_bits[z]=0xFF;
				}

			continue;
			}
		//**************** LTEXT Befehl ***************************************
		if(bef==BEF_LTEXT)
			{
			file.TextLine();						// Zeilenende
			zeile++;add=0;

			if(~bits&B_TEXT )continue;				// Kein  Text
			if(~bits&B_ZAHL1)continue;				// Keine Zahl

			   z=befehl[nr].m1;
			if(z<0 || z>LTEXT_MAX)continue;

			ltext[z]=befehl[nr].text;
			continue;
			}
		//**************** STEXT Befehl ***************************************
		if(bef==BEF_STEXT)
			{
			file.TextLine();						// Zeilenende
			zeile++;add=0;

			if(~bits&B_TEXT )continue;				// Kein Text
			if(~bits&B_ZAHL1)continue;				// Keine Zahl
			   z=befehl[nr].m1;
			if(z<0 || z>STEXT_MAX)continue;

			stext[z]=befehl[nr].text;
			continue;
			}
		//**************** RTEXT Befehl ***************************************
		if(bef==BEF_RTEXT)
			{
			file.TextLine();						// Zeilenende
			zeile++;add=0;

			if(~bits&B_TEXT )continue;				// Kein  Text
			if(~bits&B_ZAHL1)continue;				// Keine Zahl

			   z=befehl[nr].m1;
			if(z<0 || z>RTEXT_MAX)continue;

			rtext[z]=befehl[nr].text;
			continue;
			}
		//**************** ALIAS Befehl ***************************************
		if(bef==BEF_ALIAS)
			{
			file.TextLine();						// Zeilenende
			zeile++;add=0;

			if(!befehl[nr].text )continue;			// Kein  Text
			if(!befehl[nr].text2)continue;			// Keine Zahl

			if(alias_anzahl>=ALIAS_MAX)continue;

			text = befehl[nr].text;
			bptr = befehl[nr].text2;

			if(*text=='\xFF')text+=3;
			if(*bptr=='\xFF')bptr+=3;

			alias[alias_anzahl*2  ] = text;
			alias[alias_anzahl*2+1] = bptr;
			alias_anzahl++;

			continue;
			}
		//**************** INFO Befehl ****************************************
		if(bef==BEF_INFO)
			{
			file.TextLine();						// Zeilenende
			zeile++;add=0;

			if(!befehl[nr].text )continue;			// Kein  Text
			if(!befehl[nr].text2)continue;			// Keine Zahl

			text = befehl[nr].text;
			bptr = befehl[nr].text2;

			if(*text=='\xFF')text+=3;
			if(*bptr=='\xFF')bptr+=3;

			val = info_txt.Add(bptr);
			infos.Add(text,CSC_ABCG,&val);

			continue;
			}
		//**************** MACRO Befehl ***************************************
		if(bef==BEF_MACRO)
			{
			file.TextLine();						// Zeilenende
			zeile++;add=0;

			if(!befehl[nr].text )continue;			// Kein  Text
			if(!befehl[nr].text2)continue;			// Keine Zahl

			   y = macros.FindFast(befehl[nr].text);
			if(y>=0)
				{
				AsmError("Doppelte Makrodefinition in Zeile %i",zeile);
				continue;
				}

			strreplace(puffer,befehl[nr].text2+3,"param0",par1,sizeof(macros));
			strreplace(puffer                   ,"param1",par2,sizeof(macros));
			z =strcalc(puffer);

			macros.Add(befehl[nr].text+3,CSC_ABC,&z);

			continue;
			}
		//**************** BITFIELD Befehl ************************************
		if(bef==BEF_BITFIELD)
			{
			file.TextLine();						// Zeilenende
			zeile++;add=0;

			if(~bits&B_TEXT )continue;				// Kein  Text
			if(~bits&B_ZAHL1)continue;				// Keine Zahl

			   z=befehl[nr].m1;
			if(z<0 || z>BFIELD_MAX)continue;


			text = befehl[nr].text;
			bptr = bfield[z];

			if(*text=='\xFF')text+=3;

			for(z=0;z<64;text++)					// Hex-Dump einlesen
				{
				   y=*text;
				if(y==0)break;
				if(y==' ' || y=='\t')continue;
				if(y>='a' && y<='f' )y-=32;
				if(y< '0' || y> 'F' )break;
				if(y> '9' && y< 'A' )break;
				if(y> '9' )  y-='A'-10;
				else	     y-='0';
				if(z&1)
					{
					bptr[z>>1]&=0xF0;
					bptr[z>>1]|=y;
					}
				else{
					bptr[z>>1]&=0x0F;
					bptr[z>>1]|=y<<4;
					}
				z++;
				}

			continue;
			}
		//**************** Nchster Befehl ************************************

		file.TextLine();						// Zeilenende
		add=0;

		   befehl[nr].flags= bits;				// Daten einfgen
		   befehl[nr].proc = syntax[bef].proc;
		   befehl[nr].type = syntax[bef].type;
		   befehl_anzahl++;
		nr=befehl_anzahl;

		if(befehl_anzahl>=BEF_MAX-1)
			{
			printf("Uebelauf im Befehlsspeicher! Zeile %i",zeile);
			ok=0;
			break;
			}

		zeile++;
		}

//******************** Marken umrechnen ***************************************

	befehl[befehl_anzahl].proc  = 0;			// END Befehl am Ende anhngen
	befehl[befehl_anzahl].flags = 0;
		   befehl_anzahl++;

	for(x=0;x<'z'-'A'+1;x++)					// Sub-Marken kopieren
		{
		sub_bef[x]=marken[x];
		}

	for(x=0;x<befehl_anzahl;x++)
		{
		if(~befehl[x].flags&B_MARKE)continue;	//  Keine Marke vorhanden
		  y=befehl[x].or;

		if(y>=marken_anzahl || y<0)
			{
			befehl[x].or=0;
			continue;
			}

		befehl[x].or=marken[y];
		}

//******************** Init Program *******************************************

	#ifdef _DEBUG
	dzeile	= 0;								// Debug-Zeile
	dtext.Del( );								// Debug-Texte
	file.Seek(0);
	add		= 0;
	dsub	= "Init-Zeile";
	while(file.Text(add,&mem))
		{
		add=file.TextLineLen();
		dtext.Add(add,mem);
		}
	#endif

	   y=marken_namen.Find("Init");				// Suche Init-Marke
	if(y==-1)return befehl_anzahl;

	befehl_nr=marken_namen.DataGet(y);
	param[0]=Param1;
	param[1]=Param2;
	param[2]=Param1;
	param[3]=Param2;

	while(befehl[befehl_nr].proc)
		{
		#ifdef _DEBUG
		dzeile=dtext[befehl[befehl_nr].zeile-1];
		#endif
		befehl[befehl_nr].proc(&befehl[befehl_nr]);
		}

	if(m_mem[0])t_brt[6]=m_mem[0]&0xFC;			// Ausgabeformate 6/7 ndern
	if(m_mem[1])t_brt[7]=m_mem[1]&0xFC;

	m_anzahl = 0;

	for(x=0;x<masken_seiten;x++)
		{
		if(masken_offset[x]+masken_anzahl[x]<=m_anzahl)continue;
		m_anzahl = masken_offset[x]+masken_anzahl[x];
		}

	for(nr=0;nr<m_anzahl;nr++)					// Startoffsets berechnen
		{
		val	=	masken[nr].maske;

		if(masken[nr].bits==0x67)
			{
			val+=0;
			}

		if(byte_order)
			{
			val=btol_dw(val);
			}

		x	=   0;

		while((val&0xFF)==0xFF)
			{
			x    += 8;
			val >>= 8;
			}

		masken[nr].add_c = x;
		x		=   0;

		while(val)
			{
			x    += 8;
			val >>= 8;
			}

		masken[nr].add_d = x;
		}



return befehl_anzahl;
}

//*****************************************************************************
//*
//*		AsmGetRegName
//*
//*****************************************************************************
//	Ergibt einen Registernamen
char *AsmGetRegName(int Nr)
{
char	*name;


	if((unsigned)Nr>=RTEXT_MAX)return 0;

	name = rtext[Nr];

	if(name && *name=='\xFF')name+=3;


return name;
}

//*****************************************************************************
//*
//*		AsmGetFlags
//*
//*****************************************************************************
//	Holt die Falgs aus Mem
//	In Add wird die Anzahl der abgearbeiteten Bytes gestellt
static unsigned AsmGetFlags(char *Mem,int &Add)
{
unsigned	flags=8;
int			pos,z,anz;


	for(pos=1,z=0;pos<16 && !z;pos++)
	switch(Mem[pos])
		{
	case 'A':	flags|=B_AKT_WBRT;		break;
	case 'B':	flags|=B_AKT_BBRT;		break;
	case 'N':	flags|=B_NO_BITADD;		break;
	case 'S':	flags|=B_ADR_0;			break;
	case 'D':	flags|=B_ADR_DATA;		break;
	case 'G':	flags|=B_L_TO_B;		break;
	case 'M':	flags|=B_MISCOUNT;		break;
	case 'U':	flags|=B_UNSIGNED;		break;
	default:	if(Mem[pos]<'0' || Mem[pos]>'9')		// Ende ?
						{z=1;break;}
				flags&=~B_ZAHL;
				flags|=strtoi(Mem+pos,0,&anz)&B_ZAHL;
				pos+=anz-1;
		}

	Add=pos;

return flags;
}

//*****************************************************************************
//*
//*		AsmMaskSort
//*
//*****************************************************************************
static int _cdecl AsmMaskSort(const void *Mask1,const void *Mask2)
{
unsigned	 c1,c2;
Maske		*m1,*m2;
int			 b1, b2;
int			 x;


	m1 = (Maske*)Mask1;
	m2 = (Maske*)Mask2;
	c1 = m1->maske;
	c2 = m2->maske;

	if((m1->maske&match)==match)				// Volle Bit Zahl bei 1
		{
		if((m2->maske&match)==match)
			{
			if((m2->bits&match)!=(m1->bits&match))
				{
				return ((m1->bits&match)<(m2->bits&match))? -1:1;
				}
			}
		else{
			return 1;
			}
		}
	else{
		if((m2->maske&match)==match)return -1;
		}

	b1=b2=0;

	for(x=0;x<32;x++)							// Zhle Masken Bits
		{
		b1+=c1&1;c1>>=1;
		b2+=c2&1;c2>>=1;
		if(!c1 && !c2)break;
		}

return b2-b1;									// Nach Bits sortiert
}

//*****************************************************************************
//*
//*		AsmLoadMask
//*
//*****************************************************************************
//	Ldt Masken der Form
//	0F:10???111:4?FF	"Name"			abc r:1:2
//	0F34				"Name:Name1"	abc r:1:2
//	':'		Seht fr den Wechsel von Hex auf Bin-Mode (nur alle 8 Bit erlaubt)
//	'?'		Seht fr ein Beliebiges Bit
//	"Name"	Ist der Name des Befehls fr Namesketten ':' einfgen
//	abc     Subbrogramm Code
//	r:1:2   Subbrogramm Code mit 2 bergabeparametern (zwischen 0-47)
//	In matchset wird die maximale Breite berprft
static int AsmLoadMask(CTextBuffer &file,int &zeile)
{
unsigned char  *ptr;
unsigned		wert,mask;
int     		x,y,z,w,bit,add,anz;
int				shift,ok,shift_min;
int				mask_anzahl,order,off;
char		   *mem,puffer[256+3+33];



	if(masken_seiten>=MASKEN_CNT)return 0;


	y=0;ok=1;add=0;
	shift_min=32;

	mask_anzahl = masken_offset[masken_seiten];
	matchset    = masken_match [masken_seiten];

	while(ok && file.Text(add,&mem,1024))
		{
		if(add=file.TextEmpty())				// Leerzeichen entfernen
			{
			file.Text(add,&mem,1024);
			}

		if(*mem==';')							// REM-Marke gefunden
			{
			file.TextLine();
			zeile++;
			add=0;
			continue;
			}

		if(*mem=='M')							// Nchste Maskenseite
			{
			break;
			}

		if(*mem=='\n'){add=1;zeile++;continue;}	// Zeilenende
		if(*mem=='\r'){add=1;        continue;}

		x     = 0;
		bit   = 4;
		wert  = 0;
		mask  = 0;
		shift = 0;
		order = 0;

		if(mem[0]=='>')							// Bigendian 32-Bit-Maske
			{
			order= 1;
			x    = 1;
			}

		for(;x<256 && shift<32 ;x++)
			{
			if( mem[x]=='_' )continue;
			if(!mem[x]      )break;				// Normales Masken Ende
			if( mem[x]==' ' )break;
			if( mem[x]=='\t')break;
			if( mem[x]=='\'')break;
			if( mem[x]=='\"')break;
			if( mem[x]==':' )					// Bin Oder Hex
				{
				if(shift&7){shift=256;break;}
				if(bit== 1)bit=4;
				else	   bit=1;
				continue;
				}
			if(bit==1)							// Einzel Bits Bit
				{
				switch(mem[x])
					{
				case '1':	wert |=1<<(shift^7);
				case '0':	mask |=1<<(shift^7);
				case '?':	shift++;			break;
				default:	shift=256;			break;
					}
				continue;
				}
			if(mem[x]>='0' && mem[x]<='9')		// Hex Zahl 0-9
				{
				y=mem[x]-'0';
				wert |= y  <<(shift^4);
				mask |=0x0F<<(shift^4);
				shift+=4;
				continue;
				}
			if(mem[x]>='A' && mem[x]<='F')		// Hex Zahl A-F
				{
				y=mem[x]-'A'+10;
				wert |= y  <<(shift^4);
				mask |=0x0F<<(shift^4);
				shift+=4;
				continue;
				}
			if(mem[x]>='a' && mem[x]<='f')		// Hex Zahl a-f
				{
				y=mem[x]-'a'+10;
				wert |= y  <<(shift^4);
				mask |=0x0F<<(shift^4);
				shift+=4;
				continue;
				}
			if(mem[x]=='?')						// Hex Maske
				{
				shift+=4;
				continue;
				}
			shift=256;							// Fehler
			AsmError("Unerlaubtes Zeichen in Maske in Zeile %i",zeile);
			break;
			}

		if(order)								// Bigendian 32-Bit-Maske
			{
			wert  = btol_dw(wert);
			mask  = btol_dw(mask);
			}

//******************** Befehlsnamen *******************************************

		if(shift>256 || (shift&7) || shift<8)	// Falscher Shift
			{
			file.TextLine();
			zeile++;
			continue;
			}

			   file.Text(x,&mem,1024);
		if(add=file.TextEmpty())				// Leerzeichen entfernen
			{
			file.Text(add,&mem,1024);
			}

		if(*mem=='"')
			{
			add=file.TextString(puffer+3,255,&anz);
				file.Text(add,&mem,1024);
			}
		else{
			anz=0;
			}

		if(anz>0)								// Wurde ein Text gefunden
			{
			puffer[0] = '\xFF';
			puffer[1] = anz;

			for(x=1,y=0;y<anz;y++)
				{
				if(puffer[y+3]!=':')continue;
				puffer[3+anz+x]=y+1;
				x++;
				}

			puffer[2]       = x;
			puffer[3+anz+x] = y+1;

			y=gtext.Add(3+anz+x+1,puffer);
			masken[mask_anzahl].name=gtext[y];
			}
		else{
			masken[mask_anzahl].name=0;
			}

//******************** Sub-Befehle ********************************************


		for(x=0,z=0;z<(sizeof(puffer)-16);x++)	// Sub Befehle
			{
			if(mem[x]== 0  )break;
			if(mem[x]=='\r')break;
			if(mem[x]=='\n')break;
			if(mem[x]==';' )break;
			if(mem[x]==' ' )continue;
			if(mem[x]=='\t')continue;
			if(mem[x]< 'A' )continue;
			if(mem[x]> 'z' )continue;

			w=z;
			puffer[z]=mem[x];
			z++;

			if(mem[x+1]==':')					// Parameter 1
				{
				y=strtoi(mem+x+2,0,&add);
				y&=0xFF;if(y>47)y=47;
				x+=1+add;
				puffer[z]=y;
				z++;

				if(mem[x+1]==':')				// Parameter 2
					{
					y=strtoi(mem+x+2,0,&add);
					y&=0xFF;if(y>47)y=47;
					x+=1+add;
					puffer[z]=y;
					z++;
					}
				}

			y = puffer[w]-'A';

			if(sub_def[y]<2)					// Existiert die Sprungmarke
				{
				if(sub_def[y])AsmError("Unbekanntes Mikroprogram %c in Zeile %i",mem[w],zeile);
				z=w;
				}
			}

		puffer[z]=-1;z++;						// Sub-Befehls Ende
		puffer[z]= 0;z++;


		if(mask_anzahl<MASKEN_MAX)
			{
			y   = gtext.Add(z,puffer);			// Sub-Programm
			ptr = (unsigned char*)gtext[y];
			masken[mask_anzahl].prog  = ptr;
			masken[mask_anzahl].bits  = wert;	// Vergleichs Bits
			masken[mask_anzahl].maske = mask;	// Und-Maske
			mask_anzahl++;
			}

		file.TextLine();
		zeile++;
		if(shift_min>shift)shift_min=shift;
		add=0;
		}

	if(matchset>(unsigned)(shift_min-8))		// Maximaler Raster
		{
		masken_match [masken_seiten] = y;
		matchset=y;
		}

	add     = masken_offset[masken_seiten];
	match   = masken_bits  [masken_seiten];
	match <<= matchset;

	qsort(masken+add,mask_anzahl-add,sizeof(masken[0]),AsmMaskSort);



	for(x=add;x<mask_anzahl;x++)				// Suche Masken ohne Treffer
		{
		if(x>add)
			{
			if(masken[x].bits ==masken[x-1].bits )
			if(masken[x].maske==masken[x-1].maske)
				{
				printf("Doppelter Maskeneinterag (mask=0x%08X/bits=0x%08X)",masken[x].bits);
				}
			}
		if((masken[x].maske&match)==match)break;
		}


	off = masken_seiten*257;
	masken_start[off] = x;
	off++;

	for(y=0;y<256;y++)							// Suche Masken mit Treffer
		{
		while(x<mask_anzahl)
			{
			if(x>add)
				{
				if(masken[x].bits ==masken[x-1].bits )
				if(masken[x].maske==masken[x-1].maske)
					{
					printf("Doppelter Maskeneinterag (mask=0x%08X/bits=0x%08X)",masken[x].bits);
					}
				}
			   z  = masken[x].bits;
			   z &= match;
			   z>>= matchset;
			if(z!=y)break;

			x++;
			}

		masken_start[y+off]=x;
		}

												// Speichert die Daten der Maskenseite
	masken_anzahl[masken_seiten  ]  = mask_anzahl;
	masken_anzahl[masken_seiten  ] -= masken_offset[masken_seiten];
	masken_offset[masken_seiten+1]  = mask_anzahl;
	masken_seiten++;



return mask_anzahl;
}

//*****************************************************************************
//*
//*		AsmWriteNumber
//*
//*****************************************************************************
//	Gibt eine Zahl in der Zeile aus
//	Mode	: Wenn ein Bit aus 8-31 gesetzt ist werden (Mode&0x0F) Zeichen ausgegeben.
//			  Wenn die Bits 8-31 Null sind dann ist (Mode&0x07)	der Modus
//				0 =  2 Zeichen ausgeben
//				1 =  4 Zeichen ausgeben
//				2 =  8 Zeichen ausgeben
//				3 =	16 Zeichen ausgeben
//				4 = variable Lnge bis zu '0' Zeichen
//				5 = variable Lnge bis zu '0' Zeichen mit Vorzeichen
//				6 =  2 Zeichen mit Vorzeichen ausgeben
//				7 =  4 Zeichen mit Vorzeichen ausgeben
//	Nr		: Bits  0-31 der Zahl
//	NrHigh	: Bits 32-63 der Zahl
static void AsmWriteNumber(int Mode,unsigned Nr,unsigned NrHigh)
{
int			shift;
unsigned 	val;


	if(Mode&0xFFFFFF00)								// Modus auswerten
		{
		if(Mode&0x1F)
			{
			shift=((Mode&0x1F)<<2);
			}
		else{										// Variable Lnge
			if(NrHigh)
				{
				val = NrHigh>>4;
				for(shift=4;shift<32;shift+=4,val>>=4)if(!val)break;
				shift+=32;
				}
			else{
				val = Nr>>4;
				for(shift=4;shift<32;shift+=4,val>>=4)if(!val)break;
				}
			}
		}
	else{
		shift=t_brt[Mode&7];

		if(shift&0x80)								// Vorzeichen ausgeben
			{
			if(line_pos>=LINE_BRT)return;
			if(Nr&0x80000000)
				{line[line_pos]='-';line_pos++;Nr=-(int)Nr;}
			else{line[line_pos]='+';line_pos++;}
			}

			shift&=0x7F;							// Variable Lnge
		if( shift==0)
			{
			val = Nr>>4;
			for(shift=4;shift<32;shift+=4,val>>=4)if(!val)break;

			if(shift<32 && ((Nr>>(shift-4))>=0x0A))
				{
				shift+=4;
				}
			}
		}

	if(shift>32)									// Bits 63-32 ausgeben
		{
		for(shift-=32+4;shift>=0;shift-=4)
			{
			line[line_pos]=hext_chars[(NrHigh>>shift)&0x0F];

			   line_pos++;
			if(line_pos>=LINE_BRT)break;
			}

		shift=32;
		}

	for(shift-=4;shift>=0;shift-=4)					// Bits 31-0 ausgeben
		{
		line[line_pos]=hext_chars[(Nr>>shift)&0x0F];

		   line_pos++;
		if(line_pos>=LINE_BRT)break;
		}

}

static void AsmWriteNumber(int Mode,unsigned Nr)
{
int			shift;
unsigned 	val;


	if(Mode&0xFFFFFF00)								// Modus auswerten
			shift=((Mode&0x0F)<<2);
	else	shift=t_brt[Mode&7];

	if(shift&0x80)									// Vorzeichen ausgeben
		{
		if(line_pos>=LINE_BRT)return;
		if(Nr&0x80000000)
			{line[line_pos]='-';line_pos++;Nr=(~Nr)+1;}
		else{line[line_pos]='+';line_pos++;}
		}

	   shift&=0x7F;
	if(shift==0)									// Variable Lnge
		{
		val = Nr;
		for(;shift<28;shift+=4,val>>=4)if(!val)break;
		if(shift<28 && ((Nr>>shift)>=0x0A))
			{
			shift+=4;
			}
		}

	for(shift-=4;shift>=0;shift-=4)					// Bits 31-0 ausgeben
		{
		line[line_pos]=hext_chars[(Nr>>shift)&0x0F];

		   line_pos++;
		if(line_pos>=LINE_BRT)break;
		}

}

//*****************************************************************************
//*
//*		AsmWriteText
//*
//*****************************************************************************
//	Gibt einen Text in der Zeile aus
//	Text	: Ist der Zeiger auf den Text
//			  Wenn dieser mit 0xFF anfngt dann der Text ein spezial Format.
//					Text[0  ] = 0xFF
//					Text[1  ] = Lnge (=n)
//					Text[2  ] = Subanzahl
//					Text[3+n] = 0x00
//					Text[4+n] = Offset der Subtexte
static void AsmWriteText(char *Text)
{
char   *ptr;
char	sign;
int		len;
int		pos;


	if(!Text)return;

	   sign = *Text;
	if(sign=='\x00')return;
	if(sign=='\xFF')							// Spezialtext mit Lngenangabe
		{
		len = (unsigned char)Text[1];

		if(line_pos+len>=LINE_BRT)
			{
			len = LINE_BRT-line_pos;
			}

		Text += 3;
		ptr   = line+line_pos;

		for(pos=0;pos<len;pos+=4)
			{
			*(int*)(ptr+pos) = *(int*)(Text+pos);
			}

		line_pos+=len;
		return;
		}


	do	{
		if(line_pos>=LINE_BRT)break;

		line[line_pos]=sign;
	 	line_pos++;
		Text++;
		sign = *Text;
		}while(sign);
}

//*****************************************************************************
//*
//*		AsmWriteChar
//*
//*****************************************************************************
//	Gibt ein Zeichen in der Zeile aus
static inline void AsmWriteChar(char cChar)
{

	if(line_pos<LINE_BRT)
		{
		line[line_pos]=cChar;
	 	line_pos++;
		}
}

//*****************************************************************************
//*
//*		AsmWriteSub
//*
//*****************************************************************************
//	Speichert einen Untertext in der Zeile. z,B.: "abc:123:xxxxx"
//	Text	: Ist der Zeiger auf den Text
//			  Wenn dieser mit 0xFF anfngt dann der Text ein spezial Format.
//					Text[0  ] = 0xFF
//					Text[1  ] = Lnge (=n)
//					Text[2  ] = Subanzahl
//					Text[3+n] = 0x00
//					Text[4+n] = Offset der Subtexte
//	Sub		: Ist die Nummer des Untertextes
static void AsmWriteSub(char *Text,int Sub)
{
unsigned char  *offsets;
char		   *ptr;
char			sign;
int				count;
int				len;
int				pos;


	if(!Text)return;

	   sign = *Text;
	if(sign=='\x00')return;
	if(sign=='\xFF')							// Spezialtext mit Lngenangabe
		{
		len     = (unsigned char ) Text[1];
		count   = (unsigned char ) Text[2];
		offsets = (unsigned char*)(Text+3+len);

		if(Sub<0)Sub=0;
		if(count<=Sub)
			{
			return;
			}

		Text += 3;
		Text += offsets[Sub  ];
		len   =	offsets[Sub+1]-offsets[Sub]-1;
		ptr   = line+line_pos;

		for(pos=0;pos<len;pos+=4)
			{
			*(int*)(ptr+pos) = *(int*)(Text+pos);
			}

		line_pos+=len;
		return;
		}


	while(Sub>0 && *Text)
		{
		if(*Text==':')Sub--;
			Text++;
		}

	while(*Text)
		{
		if(*Text==':')break;
		line[line_pos]=*Text;
	 		 line_pos++;Text++;
		if(line_pos>=LINE_BRT)break;
		}
}

//*****************************************************************************
//*
//*		AsmWriteSpace
//*
//*****************************************************************************
static inline void AsmWriteSpace(int Count)
{

	while(Count>0)
		{
		line[line_pos]=' ';
	 		 line_pos++;
		if(line_pos>=LINE_BRT)break;

		Count--;
		}
}


//*****************************************************************************
//*
//*		AsmTextInsert
//*
//*****************************************************************************
static void AsmTextInsert(char *Text,int Select,int Pos)
{
	if(!Text || !*Text)return;

	switch(Pos)
			{
	case 0:	if(vtext_anz>=TEXT_MAX)return;
			   vtext_sub[vtext_anz]=Select;
			   vtext_ptr[vtext_anz]=Text;
						 vtext_anz++;
			return;
	case 1:	if(btext_anz>=TEXT_MAX)return;
			   btext_sub[btext_anz]=Select;
			   btext_ptr[btext_anz]=Text;
						 btext_anz++;
			return;
	case 2:	if(ntext_anz>=TEXT_MAX)return;
			   ntext_sub[ntext_anz]=Select;
			   ntext_ptr[ntext_anz]=Text;
						 ntext_anz++;
			return;
	case 3:	if(ltext_anz>=TEXT_MAX)return;
			   ltext_sub[ltext_anz]=line_nr;
			   ltext_ptr[ltext_anz]=Text;
						 ltext_anz++;
			ltext_anz=0;
			return;
			}
}
//*****************************************************************************
//*
//*		AsmGetTerm
//*
//*****************************************************************************
//	Ergibt den Wert eines Terms
static unsigned AsmGetTerm(AsmCommand *Bef,int Nr)
{
ASM_DWORD	flags,wert,*ptr;


	flags=Bef->flags;
	wert =0;

	if(Nr==0)
		{
		if(flags&B_M1)
			{
				ptr=m[Bef->m1].ptr;
			if(!ptr)
				{
				ptr =&entry[entry_nr].flags;
				ptr+=m[Bef->m1].offset;
				}
			wert=*ptr;
			}
		else{
			if(flags&B_ZAHL1)wert=Bef->m1;
			}

		return wert;
		}

	if(flags&B_M2)
		{
			ptr=m[Bef->m2].ptr;
		if(!ptr)
			{
			ptr=&entry[entry_nr].flags;
			ptr+=m[Bef->m2].offset;
			}
		wert=*ptr;
		}
	else{
		if(flags&B_ZAHL2)wert=Bef->m2;
		}


	if(flags&(B_AND|B_WERT|B_XOR|B_OR|B_SHIFT))
		{
		if(flags&B_AND )wert&=Bef->and;
		if(flags&B_WERT)wert+=Bef->shift;
		if(flags&B_XOR )wert^=Bef->or;
		if(flags&B_OR  )wert|=Bef->or;
		if(flags&B_SHIFT)
			{
			if(Bef->shift&0x80000000)
					wert>>=-(int)Bef->shift;
			else	wert<<= (int)Bef->shift;
			}
		}

return wert;
}

//*****************************************************************************
//*
//*		AsmSetTerm
//*
//*****************************************************************************
//	Ergibt den Wert eines Terms
static void AsmSetTerm(AsmCommand *Bef,unsigned Wert,unsigned Type)
{
ASM_DWORD		val;
ASM_DWORD	   *ptr;
Speicher       *m_ptr;
AsmEntry       *e_ptr;


	if(~Bef->flags&B_M1)return;							// Kein Speicher

	e_ptr=&entry[entry_nr];
	m_ptr=&m[Bef->m1];
	  ptr= m_ptr->ptr;

	if(!ptr)
		{
		ptr=&entry[entry_nr].flags;						// Zeiger auf Entry-Element
		ptr+=m_ptr->offset;
		}

	e_ptr->flags  &= m_ptr->f_and;						// Flags setzen
	e_ptr->flags  |= m_ptr->f_or;
	e_ptr->layout &= m_ptr->l_and;
	e_ptr->layout |= m_ptr->l_or;
	e_ptr->clips  &= m_ptr->c_and;
	e_ptr->clips  |= m_ptr->c_or;


	switch(Type)										// Neuen Wert berechnen
			{
	case 0:	*ptr  = Wert;		return;
	case 1:	*ptr &= Wert;		return;
	case 2:	*ptr |= Wert;		return;
	case 3:	*ptr ^= Wert;		return;
	case 4:	*ptr += Wert;		return;
	case 5:	*ptr -= Wert;		return;
	case 6:	*ptr *= Wert;		return;

	case 7:	if(!Wert)			return;
			*ptr /= Wert;		return;
	case 8:	if(!Wert)			return;
			*ptr %= Wert;		return;
	case 9:	if(Wert&0x80000000)
					*ptr>>=-(int)Wert;
			else	*ptr<<= (int)Wert;
			return;
	case 10: val =  *ptr;
			*ptr = (val<<Wert)|(val>>(32-Wert));
			return;
	case 11: val =  *ptr;
			*ptr = (val>>Wert)|(val<<(32-Wert));
			return;
			}

}

//*****************************************************************************
//*
//*		AsmSetTerm64
//*
//*****************************************************************************
//	Ergibt den Wert eines Terms
static void AsmSetTerm64(AsmCommand *Bef,ASM_QWORD Wert,unsigned Type)
{
ASM_QWORD   val;
ASM_DWORD  *ptr;
ASM_QWORD  *qptr;
Speicher   *m_ptr;
AsmEntry   *e_ptr;

	if(~Bef->flags&B_M1)return;							// Kein Speicher

	e_ptr=&entry[entry_nr];
	m_ptr=&m[Bef->m1];
	  ptr= m_ptr->ptr;

	if(!ptr)
		{
		ptr=&entry[entry_nr].flags;						// Zeiger auf Entry-Element
		ptr+=m_ptr->offset;
		}

	e_ptr->flags  &= m_ptr->f_and;						// Flags Setzen
	e_ptr->flags  |= m_ptr->f_or;
	e_ptr->layout &= m_ptr->l_and;
	e_ptr->layout |= m_ptr->l_or;
	e_ptr->clips  &= m_ptr->c_and;
	e_ptr->clips  |= m_ptr->c_or;


	qptr = (ASM_QWORD*)ptr;

	switch(Type)										// Neuen Wert berechnen
			{
	case 0:	*qptr  = Wert;		return;
	case 1:	*qptr &= Wert;		return;
	case 2:	*qptr |= Wert;		return;
	case 3:	*qptr ^= Wert;		return;
	case 4:	*qptr += Wert;		return;
	case 5:	*qptr -= Wert;		return;
	case 6:	*qptr *= Wert;		return;
	case 7:	if(!Wert)			return;
			*qptr /= Wert;		return;
	case 8:	if(!Wert)			return;
			*qptr %= Wert;		return;
	case 9:	if(Wert&0x80000000)
					*qptr>>=-(int)Wert;
			else	*qptr<<= (int)Wert;
			return;
	case 10: val  = *ptr;
			*qptr = (val<<Wert)|(val>>(64-Wert));
			return;
	case 11: val  = *ptr;
			*qptr = (val>>Wert)|(val<<(64-Wert));
			return;
	case 12:if(Wert&0x80000000)Wert|=0xFFFFFFFF<<32;
			*qptr += Wert;
			return;
			}

}

//*****************************************************************************
//*
//*		AsmBIT
//*
//*****************************************************************************
//	Ein Bit aus einem Bitfeld auslesen
static void AsmBIT(AsmCommand *Bef)
{
unsigned	wert,nr;
char       *bits;


	nr    = (Bef->flags&B_ZAHL)%BFIELD_MAX;
	bits  = bfield[nr];

	wert=AsmGetTerm(Bef,1)&0xFF;
		 AsmSetTerm(Bef,(bits[wert>>3]>>(wert&7))&1,Bef->type);

	befehl_nr++;
}

//*****************************************************************************
//*
//*		AsmSET
//*
//*****************************************************************************
//	Rechen Operation mit Termen
void AsmSET(AsmCommand *Bef)
{
unsigned	wert;


	wert=AsmGetTerm(Bef,1);
		 AsmSetTerm(Bef,wert,Bef->type);

	befehl_nr++;

}

//*****************************************************************************
//*
//*		AsmSET64
//*
//*****************************************************************************
//	Rechen Operation mit Termen
void AsmSET64(AsmCommand *Bef)
{
ASM_QWORD	wert;

	wert=(int)AsmGetTerm  (Bef,1);
			  AsmSetTerm64(Bef,wert,Bef->type);

	befehl_nr++;
}

//*****************************************************************************
//*
//*		AsmIF
//*
//*****************************************************************************
//	Bedingter Sprung
static void AsmIF(AsmCommand *Bef)
{
unsigned	flags,wert1,wert2,type;



	befehl_nr++;

		flags=Bef->flags;
	if(~flags&B_MARKE)return;

	type =(flags&B_CMP)? Bef->shift:0;
	wert1=AsmGetTerm(Bef,0);
	wert2=AsmGetTerm(Bef,1);

	switch(type)
			{
	case 0:	if(wert1==wert2)return;	break;
	case 1:	if(wert1!=wert2)return;	break;
	case 2:	if(wert1< wert2)return;	break;
	case 3:	if(wert1> wert2)return;	break;
	case 4:	if(wert1<=wert2)return;	break;
	case 5:	if(wert1>=wert2)return;	break;
			}

	befehl_nr=Bef->or;
}

//*****************************************************************************
//*
//*		AsmGET
//*
//*****************************************************************************
//	Daten aus den Befehls Bits holen
static void AsmGET(AsmCommand *Bef)
{
ASM_DWORD		flags,wert,anz,pos,nr,x,y,*ptr;


	befehl_nr++;


	wert  = 0;
	flags = Bef->flags;
	anz	  = flags&B_ZAHL;

	if(flags&B_ZAHL2)wert=Bef->m2;						// Wert laden
	if(flags&B_M2)
		{
			ptr=m[Bef->m2].ptr;
		if(!ptr)
			{
			ptr=&entry[entry_nr].flags;
			ptr+=m[Bef->m2].offset;
			}
		wert=*ptr;
		}

	if(flags&B_MISCOUNT)								// M2 ist Anzahl
		{
		anz=wert;
		wert=0;
		}

	if(flags&B_AKT_WBRT)								// Standard Addressbreite
		{
		anz = entry[entry_nr].flags>>18;
		anz = 8<<(anz&3);
		}

	if(flags&B_AKT_BBRT)								// Standard Befehlsbreite
		{
		anz = entry[entry_nr].flags>>16;
		anz = 8<<(anz&3);
		}

	if(anz>16*8)										// Zu viele Bits ?
		{
		anz=16*8;
		}

	pos=code_bit;


	if(flags&B_ADR_0   ){pos=0;flags&=~B_ADR_DATA;}		// 'S' Startadrese = 0
	if(flags&B_ADR_DATA){pos=data_bit;		      }		// 'D' Startadrese Daten

	pos+=wert;wert=0;nr=0;

	switch(bit_dir)										// Wert einlesen
			{
	case 1:	while(nr<anz)								// Von Bit 7 nach 0
				{
				x=code_mem[pos>>3];
				y=8-(pos&7);
				if(y>anz-nr)x>>=y-anz+nr;
				wert|=x<<nr;
				nr+=y;pos+=y;
				}
			break;
	case 0:	while(nr<anz)								// Von Bit 0 nach 7
				{
				x=code_mem[pos>>3];
				y=8-(pos&7);
				x>>=(pos&7);
				wert|=x<<nr;
				nr+=y;pos+=y;
				}
			}

	if(anz<=0)wert=0;
	wert&=0xFFFFFFFF>>(32-anz);

	if(flags&B_L_TO_B)									// Byte Order
		{
		if(anz>8 && anz<=16)wert=((wert&0x00FF)<<8) | ((wert&0xFF00)>>8);
		if(anz>16)
			{
			wert= ((wert&0x000000FF)<<24) |
				  ((wert&0x0000FF00)<< 8) |
  				  ((wert&0x00FF0000)>> 8) |
				  ((wert&0xFF000000)>>24);
			}
		}

	if(flags&B_UNSIGNED && anz<32)						// 'U' auf Unsigned
		{
		if(wert&(1<<(anz-1)))wert|=0xFFFFFFFF<<anz;
		}


	if(!(flags&(B_NO_BITADD|B_ADR_0)))					// Addresse erhhen
		{
		if(flags&B_ADR_DATA )							// 'D' Startadrese Daten
				data_bit+=anz;
		else	code_bit+=anz;
		}

	if(flags&(B_WERT|B_AND|B_XOR|B_OR|B_SHIFT))
		{
		if(flags&B_WERT )wert+=Bef->shift;
		if(flags&B_AND  )wert&=Bef->and;				// Rechen Operationen
		if(flags&B_XOR  )wert^=Bef->or;
		if(flags&B_OR   )wert|=Bef->or;
		if(flags&B_SHIFT)
			{
			if(Bef->shift<0){wert>>=Bef->shift;}
			else			{wert<<=Bef->shift;}
			}
		}

	if(flags&B_M1)AsmSetTerm(Bef,wert,0);				// Wert speichern


}

//*****************************************************************************
//*
//*		AsmNEXT
//*
//*****************************************************************************
//	Auf den nchsten Eintrag springen
static void AsmNEXT(AsmCommand *Bef)
{

	   befehl_nr++;
	if(entry_nr>=ENTRY_MAX)return;
	   entry_nr++;

	memcpy(entry+entry_nr,&gentry,sizeof(gentry));
	entry[entry_nr].flags=entry[entry_nr-1].flags&flags_take;
}

//*****************************************************************************
//*
//*		AsmGOTO
//*
//*****************************************************************************
//	Sprung zu anderen Befehl
void AsmGOTO(AsmCommand *Bef)
{
						  befehl_nr++;
	if(Bef->flags&B_MARKE)befehl_nr=Bef->or;
}

//*****************************************************************************
//*
//*		AsmXCHG
//*
//*****************************************************************************
//	Zwei Eintrger vertauschen
static void AsmXCHG(AsmCommand *Bef)
{
ASM_DWORD	wert1=0,wert2=1,flags,*ptr;


	   befehl_nr++;
	if(xchg_anz>=(sizeof(xchg_nr)/sizeof(xchg_nr[0])))return;

	flags=Bef->flags;

	if(flags&B_ZAHL1)wert1=Bef->m1;					// Wert1 laden
	if(flags&B_M1)
		{
			ptr=m[Bef->m1].ptr;
		if(!ptr)
			{
			ptr=&entry[entry_nr].flags;
			ptr+=m[Bef->m1].offset;
			}
		wert1=*ptr;
		}

	if(flags&B_ZAHL2)wert2=Bef->m2;					// Wert2 laden
	if(flags&B_M2)
		{
			ptr=m[Bef->m2].ptr;
		if(!ptr)
			{
			ptr=&entry[entry_nr].flags;
			ptr+=m[Bef->m2].offset;
			}
		wert2=*ptr;
		}

	xchg_nr[xchg_anz  ]=wert1;						// Eintrge tauschen
	xchg_nr[xchg_anz+1]=wert2;
	xchg_anz+=2;
}

//*****************************************************************************
//*
//*		AsmTEXT
//*
//*****************************************************************************
//	Texte ausgeben
static void AsmTEXT(AsmCommand *Bef)
{
unsigned	flags;
int			pos=0;


		befehl_nr++;
		flags=Bef->flags;
	if((flags&B_TEXT)==0 )return;
	if( flags&B_AKT_BBRT )pos=1;
	if( flags&B_NO_BITADD)pos=2;
	if( flags&B_AKT_WBRT )pos=3;

	AsmTextInsert(Bef->text,-1,pos);

}

//*****************************************************************************
//*
//*		AsmPRINT
//*
//*****************************************************************************
//	Zahlen oder Formatierten Text ausgeben
static void AsmPRINT(AsmCommand *Bef)
{
unsigned	wert,brt,flags;
int			len,pos,nr;
char	   *ptr;



	befehl_nr++;

	wert  = AsmGetTerm(Bef,0);
	brt   = AsmGetTerm(Bef,1);
	flags = Bef->flags;


	if(Bef->flags)									// printf Text ?
		{
		ptr=Bef->text;
		if(*ptr=='\xFF')ptr+=3;
		len=sprintf(line,ptr,wert,brt);
		}
	else{											// Nur Zahl ausgeben
		line_pos=0;
		AsmWriteNumber(brt|0x8000,wert);
		len=line_pos;
		line_pos=0;
		}


	nr=ptext.Add(len,line);

						 pos=0;
	if(flags&B_AKT_BBRT )pos=1;
	if(flags&B_NO_BITADD)pos=2;
	if(flags&B_AKT_WBRT )pos=3;

	AsmTextInsert(ptext[nr],-1,pos);

}

//*****************************************************************************
//*
//*		AsmRLIST
//*
//*****************************************************************************
//	Erzeugt eine Liste aus Registern
static void AsmRLIST(AsmCommand *Bef)
{
unsigned	wert,brt,flags;
int			pos,bit,nr;
char	   *ptr;



	befehl_nr++;

	wert  = AsmGetTerm(Bef,1);
	brt   = AsmGetTerm(Bef,0);
	flags = Bef->flags;


	pos     = 1;
	line[0] = '{';


	for(bit=0;bit<32;bit++,wert>>=1)
		{
		if(!(wert&1))continue;

		ptr=rtext[bit+brt];
		if(!ptr)ptr="??";
		if(*ptr=='\xFF')ptr+=3;
		pos+=sprintf(line+pos,"%s",ptr);


		if(!(wert&2) || !(wert&4))
			{
			if(wert&~1)
				{
				line[pos]=',';
				pos++;
				}

			continue;
			}

		for(;bit<32;bit++,wert>>=1)
			{
			if(!(wert&2))break;
			}


		ptr=rtext[bit+brt];
		if(!ptr)ptr="??";
		if(*ptr=='\xFF')ptr+=3;
		pos+=sprintf(line+pos,"-%s,",ptr);
		}

	line[pos] = '}'; pos++;
	line[pos] = 0;

 	nr=ptext.Add(pos,line);

						 pos=0;
	if(flags&B_AKT_BBRT )pos=1;
	if(flags&B_NO_BITADD)pos=2;
	if(flags&B_AKT_WBRT )pos=3;

	AsmTextInsert(ptext[nr],-1,pos);

}

//*****************************************************************************
//*
//*		AsmSetFormat
//*
//*****************************************************************************
//	Addr_anz : Breite der Addressangabe
//	Data_anz : Anzahl der Daten Bytes fr die Ausgabe
//	Bef_anz	 : Minimale Breite eines Befehls
void AsmSetFormat(int  Addr_anz,int  Data_anz,int  Bef_anz)
{

	if(Addr_anz< 0)Addr_anz= 0;
	if(Data_anz< 0)Data_anz= 0;
	if(Addr_anz> 8)Addr_anz= 8;
	if(Data_anz>16)Data_anz=16;
	if(Bef_anz < 0)Bef_anz = 0;
	if(Bef_anz >16)Bef_anz =16;


	addr_anz=Addr_anz;								// Ausgabe Breite fr Addresen
	data_anz=Data_anz;								// Ausgabe Breite fr Code-Bytes
	bef_anz =Bef_anz;

}

//*****************************************************************************
//*
//*		AsmGetFormat
//*
//*****************************************************************************
//	Addr_anz : Breite der Addressangabe
//	Data_anz : Anzahl der Daten Bytes fr die Ausgabe
//	Bef_anz	 : Minimale Breite eines Befehls
void AsmGetFormat(int *Addr_anz,int *Data_anz,int *Bef_anz)
{
	if(Addr_anz)*Addr_anz=addr_anz;
	if(Data_anz)*Data_anz=data_anz;
	if(Bef_anz )*Bef_anz =bef_anz;
}

//*****************************************************************************
//*
//*		AsmLineCmp
//*
//*****************************************************************************
//	Vergleicht zwei Codezeilen, dabei werden HEX-Zahlen nicht ignoriert.
//	Code1 muss dabei nicht ganz in Code2 enthalten sein
//	Ergibt 1 wenn sie gleich sind
static int AsmLineCmp(const char *Code1,const char *Code2)
{
int		len;
int		num;
int		off;


	while(*Code1)
		{
		if(*Code1==' ' ){Code1++;continue;}
		if(*Code1=='\t'){Code1++;continue;}
		if(*Code2==' ' ){Code2++;continue;}
		if(*Code2=='\t'){Code2++;continue;}

		if(strwchar(*Code1,0x01))						// Zahl
			{
			if(!strwchar(*Code2,0x01))
				{
				return 0;
				}

			Code1 += strwlen(Code1,0x0F);
			Code2 += strwlen(Code2,0x0F);
			continue;
			}

		if(*Code1=='[')									// "byte ptr [" zu "["
			{
			off = 0;

			if((*Code2&~0x20)=='B' && !memicmp(Code2+1,"YTE" ,3))off=4;
			if((*Code2&~0x20)=='W' && !memicmp(Code2+1,"ORD" ,3))off=4;
			if((*Code2&~0x20)=='D' && !memicmp(Code2+1,"WORD",4))off=5;

			if(off)
				{
				off += strwlen(Code2+off,0xE0);

				if(!memicmp(Code2+off,"ptr",3))
					{
					off += 3;
					off += strwlen(Code2+off,0xE0);
					if(Code2[off]=='[')Code2+=off;
					}
				}
			}

		if(*Code2=='[')									// "[" zu "byte ptr ["
			{
			off = 0;

			if((*Code1&~0x20)=='B' && !memicmp(Code1+1,"YTE" ,3))off=4;
			if((*Code1&~0x20)=='W' && !memicmp(Code1+1,"ORD" ,3))off=4;
			if((*Code1&~0x20)=='D' && !memicmp(Code1+1,"WORD",4))off=5;

			if(off)
				{
				off += strwlen(Code1+off,0xE0);

				if(!memicmp(Code1+off,"ptr",3))
					{
					off += 3;
					off += strwlen(Code1+off,0xE0);
					if(Code1[off]=='[')Code1+=off;
					}
				}
			}

		if(strwchar(*Code1,0x0F))						// Text
			{
			len = strwlen(Code1,0x0F);

			if(memicmp(Code1,Code2,len) || strwchar(Code2[len],0x0F))
				{
				return 0;
				}

			Code1 += len;
			Code2 += len;
			continue;
			}

		if(*Code1=='*')									// Multiplikator
			{
			if(*Code2!='*')
				{
				return 0;
				}

			Code1 += 1;
			Code2 += 1;

			if(strtoi(Code1,16,&len)!=strtoi(Code2,16,&num))
				{
				return 0;
				}

			if(!strwchar(*Code2,0x01))
				{
				return 0;
				}

			Code1 += len;
			Code2 += num;
			continue;
			}

		if(*Code1!=*Code2)								// Diverse Zeichen
			{
			return 0;
			}

		Code1 += 1;
		Code2 += 1;
		}


return 1;
}

//*****************************************************************************
//*
//*		AsmEncode
//*
//*****************************************************************************
//	Speichert in Masken alle Bitfolgen die den Befehl Codename erzeugen knnen
//	Bitfolgen werden in Masken gespeichert (4 Byte Bits/4 Byte Maske)
//	Max ist die maximale gre von Masken in 16 Byte Abschnitten
//	Mit Page wird die Maskenseite gewhlt. (-1 steht fr alle Seiten)
//	Ergibt die Anzahl der gespeicherten Bitfolgen
int	AsmEncode(const char *Codename,unsigned *Masken,int Max,int Page)
{
char			text[92];
unsigned		ibit[16];
unsigned		data[4];
AsmEntryInfo	info;
char		   *name;
char		   *hits;
int				clen;
int				mask;
int				last;
int				size;
int				end;
int				len;
int				max;
int				anz;
int				bit;
int				off;
int				pos;
int				num;
int				add;
int				ok;
int				x;



	Codename += strwlen(Codename,0xA0);			// Leerzeichen auslassen
	clen      = strwlen(Codename);				// Wortlnge
	anz       = 0;


	for(x=0;x<alias_anzahl;x++)					// Aliasnamen suchen
		{
		if(strnicmp(Codename,alias[x<<1],clen))continue;
		if(alias[x<<1][clen])continue;
		strlcpy(text,alias[(x<<1)+1],sizeof(text));
		strlcat(text,Codename+clen  ,sizeof(text));
		Codename = text;
		break;
		}

	if(Page<0)									// Alle Maskenseiten durchsuchen
		{
		max  = MASKEN_CNT;
		Page = 0;
		}
	else{
		   max = Page+1;
		if(max>MASKEN_CNT)max=MASKEN_CNT;
		}


	for(;Page<max;Page++)						// Maskenseiten durchlaufen
		{
		if(Max<=0)break;

		x   = masken_offset[Page];
		end	= masken_anzahl[Page]+x;

		for(;x<end;x++)							// Alle Masken duchsuchen
			{
			if(anz>=Max)break;

				name=masken[x].name;
			if(!name)continue;

			if(*name=='\xFF')name+=3;

			while(*name)
				{
				for(len=0;;len++)				// Befehlslnge holen
					{
					if(name[len]== 0 )break;
					if(name[len]==':')break;
					}
												// Sind Befehlsnamen gleich
				if(clen!=len || strnicmp(Codename,name,len))
					{
						name+=len;
					if(*name==':')name++;
					continue;
					}

				if(anz>=Max)break;

				Masken[anz*4  ] = masken[x].bits;
				Masken[anz*4+1] = masken[x].maske;
				Masken[anz*4+2] = Page;
				Masken[anz*4+3] = 0;
				anz++;

				break;
				}
			}
		}


	clen += strwlen(Codename+clen,0xA0);

	if(Codename[clen] && anz>0)					// Muss weiter dekodiert werden
		{
		add  = 0;
		hits = (char*)alloca(Max);
		memset(hits,0,Max);

		for(pos=0;pos<anz;pos++)
			{
			bit = bit_find_0(Masken+pos*4+1,32);

			if(!Masken[pos*4+3])				// Nach fixen Lngen suchen
				{
				add = 0;

				if(anz+3<=Max && bit<=24)
					{
						num = bit/8;
					if(!num)num=1;

					memmove(Masken+(pos+5-num)*4,Masken+(pos+1)*4,(anz-pos-1)*4*sizeof(unsigned));

					mask			 = Masken[pos*4+1];
					Masken[pos*4+1] |= bit_shift_l(0xFFFFFFFF,num*8)|mask;
					Masken[pos*4+3]  = num;
					add				 = 4-num;

					for(off=1,num++;num<=4;off++,num++)
						{
						Masken[(pos+off)*4  ] = Masken[pos*4  ];
						Masken[(pos+off)*4+1] = bit_shift_l(0xFFFFFFFF,num*8)|mask;
						Masken[(pos+off)*4+2] = Masken[pos*4+2];
						Masken[(pos+off)*4+3] = num;
						anz++;
						}
					}
				}

			   bit = bit_count_1(Masken+pos*4+1,32);
			if(bit&7)
				{
				end = 16-(bit&7);
				if(end>16)end=16;
				}
			else{
				end = 16;
				}

			mask = Masken[pos*4+1];

			for(bit=1,num=0;bit;bit<<=1)		// Suche die zu nderenden Bits
				{
				if(mask&bit)continue;

				ibit[num] = bit;
				mask     |= bit;
				num++;

				if(num>=end)break;
				}

			for(x=0,len=num-1;x<len;x++,len--)	// Bitliste berechnen fr bessere Optimierung
				{
				bit		  = ibit[x  ];
				ibit[x  ] = ibit[len];
				ibit[len] = bit;
				}

			memset(data,0,sizeof(data));
			data[0] = Masken[pos*4  ];
			size	= Masken[pos*4+3];
			num		= (1<<num)-1;

			for(ok=0;num>=0;num--)				// Suche passendes Muster
				{
				len = AsmDecode(data,info,0,Masken[pos*4+2]);

				if(size==0 || size==len || (size>=4 && len>=4))
				if(AsmLineCmp(Codename,info.line_text+info.line_cpos))
					{
					if(!ok)last=data[0];
					mask &= ~(last^data[0]);
					if(ok<len)ok=len;
					}

				x = bit_find_1(&num,15);

				if(len<4)						// berspringe bereits geprfte codes
					{
					bit = 0xFFFFFFFF<<(len*8);

					while((ibit[x]&bit) && num>0)
						{
						data[0] ^= ibit[x];
						num--;
						x = bit_find_1(&num,15);
						}
					}

				data[0] ^= ibit[x];
				}

			if(ok)								// Gab es bereinstimmungen
				{
				hits[pos] = 1;
				if(ok<4)mask &= (1<<(ok*8))-1;
				Masken[pos*4  ] = last&mask;
				Masken[pos*4+1] = mask;
				Masken[pos*4+3] = ok;
				}
			else if(size<4 || add>0)
				{
				memmove(Masken+pos*4,Masken+pos*4+4,(anz-pos-1)*4*sizeof(unsigned));
				anz--;
				add--;
				pos--;
				}
			else{
				Masken[pos*4+3] = 0;
				}
			}
		}



return anz;
}

//*****************************************************************************
//*
//*		AsmGetInfo
//*
//*****************************************************************************
//	Infos zu einem Kommando abfragen
//	Command		: Ist der Name des Kommandos
//	Ergibt einen Zeiger auf die Infos oder 0 wenn nicht gefunden wurde
char *AsmGetInfo(const char *Command)
{
int		pos;
int		num;
	
	   pos = infos.FindFast(Command,0,-1,0x01);
	if(pos<0)return 0;
	
	num = infos.DataGet(pos);


return info_txt[num];
}

//*****************************************************************************
//*
//*		AsmGetLine
//*
//*****************************************************************************
//	Eine weitere Zeile holen fr mehr Zeilen Befehle
//	Nr		: Nummer der Zeile
//	Info	: Speicher fr Zeilen Daten
void AsmGetLine(int Nr,AsmEntryInfo &Info)
{
int x,y;

	if(Nr>=line_nr)
		{
		memset(&Info,0,sizeof(Info));		// Zeilparameter lschen
		Info.line_number=Nr;				// Linen Nummer
		Info.line_count	=line_nr;			// Anzahl der Zeilen
		Info.line_text	="";				// Zeilen Text
		return;
		}


	for(x=0,y=0;x<entry_nr;x++)				// Ersten Eintrag suchen
		{
		if(y>=Nr)break;
		if(entry_ptr[x]->flags&ASM_EF_NEWLINE)y++;
		}

	for(y=0;y<4;y++,x++)					// Eintrge kopieren
		{
		if(x>=entry_nr)break;
		Info .entry[y]=entry_ptr[x];
		if(   entry_ptr[x]->flags&ASM_EF_NEWLINE){y++;break;}
		}

				 Info.entry_count=y;
	for(;y<4;y++)Info.entry[y]=0;


	Info.line_text=line;					// Zeilentext suchen

	for(x=0;x<Nr;x++)
		{
		Info.line_text+=line_len[x]+1;
		}

	Info.line_number = Nr;					// Linen Nummer
	Info.line_count	 = line_nr;				// Anzahl der Zeilen
	Info.line_length = line_len[Nr];		// Zeile lnge in Text Zeichen
	Info.line_text	 = line;				// Zeilen Text
	Info.type		 = code_type;			// Befehls Type

}

//*****************************************************************************
//*
//*		AsmDecode
//*
//*****************************************************************************
//	Decodiert den ASM-Code der in Mem steht
//	Info	: Dort werden die ergebnise gespeichert
//	Address : Ist die Anzeige Addresse (Bit  0-63)
//	Page	: Maskenseite die Benutzt werden soll. Damit knnen verschiedene
//			  Instruction-Sets benutzt weren. (z.B. fr ARM normal/thumb)
//				Bit 31 = Keinen Zeilentext in AsmDecode generieren (ASM_NO_TEXT)
//	Ergibt die Zahl der abgearbeiteten Bytes
int AsmDecode(const void *Mem,AsmEntryInfo &Info,ASM_QWORD Address,unsigned Page)
{
AsmEntry        *xchg_e,*e;
unsigned char   *sub_prog,*start_mem,*code_mem1;
unsigned		 code,code_old,flags,layout,clips,bits,l1,l2,mset,mbit;
int				 x,y,z,start,ende,match,code_len,code_len1,vnr,rnr,pre;
int				 line_num,line_last,e_nr,tab,ok,ppos,off,no_text,num;




	no_text			= Page&ASM_NO_TEXT;
	code_mem  		= (unsigned char *)Mem;		// Zeiger auf Code
	start_mem 		= code_mem;

	code_type 		= 0;						// Befehls Type = 0
	line_nr   		= 0;						// Liniennummer
	line_num  		= 0;						// Position in Linien Texten
	line_last 		= 0;						// Beginn der letzten Linie
	code_len  		= 0;						// Befehls Lnge
	befehl_nr 		= 0;						// Aktueller Befehl
	entry_nr  		= 0;						// Aktueller Eintrag
	xchg_anz  		= 0;						// Keine Eintrge tauschen
	vtext_anz 		= 0;						// Text-Speicher lchen
	btext_anz 		= 0;						//			*
	ntext_anz 		= 0;						//			*
	ltext_anz 		= 0;						//			*
	ptext.Del();								// PRINT-Texte lschen
	code_addr[0]	= ASM_LO_DWORD(Address);	// Code-Adresse
	code_addr[1]	= ASM_HI_DWORD(Address);	//			*

	Page&=0xFF;
	if(Page>=(unsigned)masken_seiten)Page=0;

	mset = masken_match[Page];
	mbit = masken_bits [Page];
	off	 = Page*257;
	e_nr = 0;

	memcpy(&entry[0],&gentry,sizeof(gentry));	// Globalen Eintrag kopieren
			entry[0].flags&=0xFFFF0000;

	memset(&Info,0,sizeof(Info));

//******************** Befehls-Schleife ***************************************
do	{
	code_mem1 = code_mem;
	code	  = *(unsigned*)code_mem;
	if(byte_order)code=btol_dw(code);
	match	  = (code>>mset)&mbit;
	start	  = masken_start[off+match  ];
	ende 	  = masken_start[off+match+1];
	sub_prog  = 0;								// Kein Sub-Programm

	for(;start<ende;start++)					// Suche Rastertreffer
		{
		if(masken[start].bits==(code&masken[start].maske))
			{
			sub_prog=masken[start].prog;
			break;
			}
		}

	if(!sub_prog)								// Keine passende Maske gefunden
		{
		start = off;
		ende  = masken_start[off];

		for(start=0;start<ende;start++)			// Suche allgemeinen Treffer
			{
			if(masken[start].bits==(code&masken[start].maske))
				{
				sub_prog=masken[start].prog;
				break;
				}
			}
		}

//******************** Sub-Program ausfhren **********************************

	x		 = masken[start].add_c;
	code_mem+= x/8;
	x		 = masken[start].add_d;
	data_bit = x;								// Nummer der Bits in Daten
	code_bit = 0;								// Nummer der Bits im Code
	cut_sign = 0;								// Spezielle Trennzeichen
	textsel  = 0;								// Kein Auswahltext
	no_end   = 0;								// Keine weiteren Befehle


	if(sub_prog)
	while(*sub_prog!=0xFF)
		{
		#ifdef _DEBUG
		char  prog=*sub_prog;
		dsub=(char*)sub_prog;
		#endif

		befehl_nr = sub_bef[*sub_prog-'A'];sub_prog++;
		param[0]  = gparam[0];					// Standard Paramerter
		param[1]  = gparam[1];					// Standard Paramerter

		if(*sub_prog<'A'){param[0]=*sub_prog;sub_prog++;}
		if(*sub_prog<'A'){param[1]=*sub_prog;sub_prog++;}

		while(befehl[befehl_nr].proc)
			{
			#ifdef _DEBUG
			dzeile=dtext[befehl[befehl_nr].zeile-1];
			#endif
			befehl[befehl_nr].proc(&befehl[befehl_nr]);
			}
		}

	if(sub_prog)								// Befehlsname in Textliste
		{
		AsmTextInsert(masken[start].name,textsel,1);
		}

	if(code_bit<data_bit)						// Abgearbeitete Bytes
			code_len     = (data_bit+7)>>3;
	else	code_len     = (code_bit+7)>>3;
			code_mem    += code_len;			// Auf nchsten Befehl
			code_len     = code_mem-code_mem1;
			code_len1    = code_mem-start_mem;
			code_old     = code_addr[0];
			code_addr[0]+= code_len;
			if(code_old>code_addr[0])code_addr[1]++;

	for(;e_nr<entry_nr;e_nr++)					// Adresslngen addieren
		{
		   e=&entry[e_nr];
		if(e->flags&ASM_EF_ADDLEN )
			{
			code_old=e->value1;
			e->value1+=code_len;
			if(code_old>e->value1)e->value1_h++;
			}
		if(e->flags&ASM_EF_ADDLEN1)
			{
			code_old=e->value1;
			e->value1+=code_len1;
			if(code_old>e->value1)e->value1_h++;
			}
		}


	}while(no_end);

	   code_len=code_len1;
	if(code_len<=0)code_len=1;

//******************** Eintrge tauschen **************************************

	for(x=0;x<=ENTRY_MAX;x++)					// Zeiger auf Eintrge
		{
		entry_ptr[x]=(x<entry_nr)? &entry[x]:0;
		}

	xchg_anz<<=1;								// Eintrge tauschen

	for(x=0;x<xchg_anz;x+=2)
		{
		y=xchg_nr[x  ];
		z=xchg_nr[x+1];
		if(y>=entry_nr)continue;
		if(z>=entry_nr)continue;

		l1=entry_ptr[y]->flags&ASM_EF_NEWLINE;	// Line-Flags sichern
		l2=entry_ptr[z]->flags&ASM_EF_NEWLINE;

		xchg_e=entry_ptr[y];
			   entry_ptr[y]=entry_ptr[z];
			   entry_ptr[z]=xchg_e;

		entry_ptr[y]->flags&=~ASM_EF_NEWLINE;	// Line-Flags setzen
		entry_ptr[y]->flags|=l1;
		entry_ptr[z]->flags&=~ASM_EF_NEWLINE;
		entry_ptr[z]->flags|=l2;
		}

//******************** Text Ausgabe *******************************************


	line_pos=0;

	#ifdef _DEBUG
	if(no_text)goto NoText;
	#endif

	if(addr_anz)								// Adresse ausgeben
		{
		AsmWriteNumber(addr_anz|0x8000,ASM_LO_DWORD(Address),ASM_HI_DWORD(Address));
		AsmWriteSpace (2);
		}

	for(x=0;x<data_anz && x<code_len;x++)		// Daten-Bytes Ausgeben
		{
		AsmWriteNumber(0x8002,start_mem[x]);
		AsmWriteSpace (1);
		}

	            AsmWriteSpace(3*(data_anz-x));
	if(data_anz)AsmWriteSpace(1);


	tab = line_pos;

	for(x=0;x<vtext_anz;x++)					// Vor Texte ausgeben
		{
		y=vtext_sub[x];
		if(y<=-1)AsmWriteText (vtext_ptr[x]  );
		else	 AsmWriteSub  (vtext_ptr[x],y);
				 AsmWriteSpace(1);
		}
	for(x=0;x<btext_anz;x++)					// Befehls Texte ausgeben
		{
		y=btext_sub[x];
		if(y<=-1)AsmWriteText (btext_ptr[x]  );
		else	 AsmWriteSub  (btext_ptr[x],y);
				 AsmWriteSpace(1);
		}

	AsmWriteSpace((tab+bef_anz>line_pos)? tab+bef_anz-line_pos:1);
	ppos=line_pos;


//******************** Eintrge ausgeben **************************************

	vnr=0;
	rnr=0;

	for(x=0;x<entry_nr;x++)
		{
		e=entry_ptr[x];
		flags=e->flags;

		if(flags&ASM_EF_NEWLINE)				// Neue Linie beginnen
			{
			if(line_nr>=LINE_MAX)break;
			   line_len[line_nr]=line_pos-line_last;
			   line[line_pos]=0;line_pos++;
			   line_last=line_pos;

			AsmWriteSpace(tab);
			z=line_pos;

			for(;line_num<ltext_anz;line_num++)	// Linien Texte
				{
				if(ltext_sub[line_num]>line_nr)break;
				AsmWriteText(ltext_ptr[line_num]);
				}

			AsmWriteSpace((z+bef_anz>line_pos)? z+bef_anz-line_pos:1);
			line_nr++;
			}

		if(x && !cut_sign)						// Trennzeichen einfgen
			{
			AsmWriteChar(',');
			}

		ok     = 0;
		bits   = flags>>8;
		clips  = e->clips;
		layout = e->layout;

		for(y=0;y<8;y++,clips>>=4,layout>>=4,bits>>=1)
			{
			if(y==(int)seg_pos)
			if(flags&ASM_EF_SEGON)				// Segment ausgeben
			if(flags&ASM_EF_SEG || seg0on)
				{
				AsmWriteText(stext[0]);
				AsmWriteText(rtext[(flags>>20)&0x0F]);
				AsmWriteText(stext[1]);
				}

			if(clips&0x04){AsmWriteChar('[');ok=0;}
			if(clips&0x01){AsmWriteChar('(');     }
			if(bits&1)
				{
				num = 1<<(layout&0x0F);
				if(!(num&pre_plus) && ok)AsmWriteChar('+');

				ok  = 1;
				num = layout&0x0F;
				pre = (pre_bits>>num)&1;

				if(pre)AsmWriteText(ltext[num]);

				switch(y)
					{
				case  4:if(vnr<4)Info.line_vpos[vnr++]=line_pos;AsmWriteNumber((flags>>24)&7,e->value0,e->value0_h);break;
				case  5:if(vnr<4)Info.line_vpos[vnr++]=line_pos;AsmWriteNumber((flags>>28)&7,e->value1,e->value1_h);break;
				default:if(rnr<8)Info.line_rpos[rnr++]=line_pos;AsmWriteText  (rtext[e->regs[y&3]]);				break;
					}

				if(!pre)AsmWriteText(ltext[num]);
				}
			else if(mono_lt)
				{
				   num = layout&0x0F;
				if(num)
					{
					AsmWriteText(ltext[num]);
					ok=0;
					}
				}

			if(clips&0x02)AsmWriteChar(')');
			if(clips&0x08)AsmWriteChar(']');
			}

		if(cut_sign)							// Spezielle Trennzeichen einfgen
			{
			AsmWriteText(stext[(flags>>20)&0x0F]);
			}
		}

	for(x=0;x<ntext_anz;x++)					// Abschlusstexte ausgeben
		{
		y=ntext_sub[x];
		if(y<=-1)AsmWriteText (ntext_ptr[x]  );
		else	 AsmWriteSub  (ntext_ptr[x],y);
				 AsmWriteSpace(1);
		}

#ifdef _DEBUG
NoText:
#endif

	line    [line_pos]=0;						// Textende
	line_len[line_nr ]=line_pos-line_last;
	line_nr++;

//******************** Info ausgeben ******************************************

	for(x=0;x<4;x++)							// Eintrge
		{
		if(x>=entry_nr)break;
		 Info.entry[x]=entry_ptr[x];
		if(   entry_ptr[x]->flags&ASM_EF_NEWLINE){x++;break;}
		}
				 Info.entry_count=x;
	for(;x<4;x++)Info.entry[x]=0;

	Info.line_number = 0;						// Linen Nummer
	Info.line_count	 = line_nr;					// Anzahl der Zeilen
	Info.line_length = line_len[0];				// Zeile lnge in Text Zeichen
	Info.line_text	 = line;					// Zeilen Text
	Info.line_cpos	 = tab;						// Position des Code nach HEX-Dump
	Info.line_ppos   = ppos;					// Position der Parameter
	Info.type        = code_type;				// Befehls Type

	ptext.Del();


return code_len;
}



