//*****************************************************************************
//*
//*
//*		AsmDecoder
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Vorgehensweise:
//
//	*) Laden der Konfiguration mit AsmInit(Datei,Param0,Param1)
//
// 		Datei		: Konfigurations Datei (*.ACF)
//		Param0		: 1. bergabeparameter an die :Init Prozetur
// 		Param1		: 2. bergabeparameter an die :Init Prozetur
//
// 	*) Einstellen des Ausgabeformats mit AsmSetFormat(Addr_anz,Data_anz)
//
//		Addr_anz	: Breite der Code-Addresse in Zeichen
//		Data_anz	: Anzahl der Code-Bytes die vor den Befehlen
//
// 	*) Auslesen der Befehle mit AsmDecode(Mem,Info,Adresse,AdresseH,Page)
//
//		Mem			: Zeiger auf Code Bytes
//		Info		: Infostruktur frs Ergebnis
//		Addresse	: Addresse des ersten Code-Bytes
//		AddresseH	: Addresse des ersten Code-Bytes Bits 32-63
//		Page		: Ist die Maskenseite fr die Dekodierung
//
// 	*) Weitere Funktionen
//
//		void AsmGetLine  (int  Nr, AsmEntryInfo &Info)
//		void AsmGetFormat(int *Addr_anz,int *Data_anz)
//		void AsmExit()
//
// Die Konfigurations Datei
//
//		Es wird zwischen Gro- und Kleinschreibung unterschieden
//		Es gibt 2 Abschnitte in der Sub-Datei
//			1.)	Sub-Befehls Programme
//			2.) Maskenseiten mit Sub-Programm
//
//		Mikroprogramme fangen mit einer ein Zeichen langen
//		Sprungmarke ('A'-'Z' oder 'a'-'z') z.B. :M
//		Lngere Sprungmarken knnen nur innerhalb der
//		Mikroprogramme verwendet werden.
//
//		Die Maskeneintrge bestimmen welchen Bitfolgen werden 
//		welchem Befehlsnamen und Mikroprogramm zugeordnet.
//		Die Definition von Maskeneintrgen erfolgt so:
//			-) Blcke zu 8-Bit hex (5F?4) oder binr (:11??1100)
//			-) Byte-Order, der erste Block ist das
//			   erste Byte im Speicher. 
//			-) Fngt der Maskeneintrag mit '>' an wird die Byte-Orter
//			   gedreht. Die Maske muss dann 32-Bit lang sein.
//			-) Befehls Namen ("MOV" oder "ADD:SUB"). Mit der "select"
//			   Variabel kann bei Auswahlnamen (mit ':' getrennt)
//			   der Befehlsname ausgewhlt werden.
//			-) Mikroprogramme die zur Maske gehren
//			   Mit	:?:? knnen den Mikobefehlen bis zu zwei
//			   Parameter bergeben werden die nicht grer
//			   als 47 sein drfen.
//			   Achtung !!! Nach der letzten Zahl immer ein Leerzeichen einfgen.
//
//
/******************* Globale Variabeln fr Mikroprogramme **********************

	g_param0	Zum Einstellen der Standartbergabeparameter
	g_param1	fr Mikroprogramme, falls keine Parameter angegeben sind.

	param0		bergabeprameter fr Mikroprogramme
	param1		z.B.	wird r:7 angegeben
				so wird in 	param0 7 bergeben
				und     in	param1 g_param1 bergeben.

	seg_on		Wird seg_on auf 1 gesetzt so wird das Standard-
				segment (flags 20-23 == 0) ausgegeben.
				Bei seg_on==0 wird es nicht angezeigt.

	g_flags		Stellt die Werte im Stantarteintrag ein
	g_layout	Bei jedem neuen Befehlbeginn oder Eintragswechsel
	g_clips		(mit NEXT) werden in den neuen Eintrag die 
	g_type		Stantartwerte geladen (auer flags Bits bei NEXT).
	g_reg0   -3
	g_value0 -1

	m0 m1		Globale Speicher zur freien Verwendung.
	m2 m3
	m4 m5
	m6 m7

	flags		Aktuelle flags im aktuellen Eintrag
				Die Bits 8-15 bestimmen welche Register (reg?)
				oder Werte (value?) augegeben werden sollen.
				Bei jedem Eintragswechsel mit NEXT werden
				die Bits 0-15 24-26 28-30 auf 0 gesetzt und
				Die anderen vom letzten Eintrag bernommen.
				(siehe flags Beschreibung weiter unten)

	layout		Mit layout knnen die Texte vor oder nach
				einem Register (reg?) oder Werte (value?)
				augegeben werden.
				(siehe layout Beschreibung weiter unten)

	clips		Mit clips knnen die Klammern  vor oder nach
				einem Register (reg?) oder Werte (value?)
				augegeben werden.
				(siehe clips Beschreibung weiter unten)

	type		Zur freien Verfhgung hier kann mam z.B.
				die Art eines Befehls speichern.
				(entry[?].type Sprungbefehl oder Returnbefehl ...)

	reg0 - 3	Mit einem Schreibzugriff kann man Register ausgeben.
				Diese Register werden bei der Ausgabe vor die
				Werte (value?) gesetzt.
				(entry[?].reg[?])

	reg2n - 3n	Mit einem Schreibzugriff kann man Register ausgeben.
				Diese Register werden bei der Ausgabe nach
				Werte (value?) gesetzt.
				(entry[?].reg[?])

	value0 - 1	Mit einem Schreibzugriff kann man Werte ausgeben.
				Diese Werte werden bei der Ausgabe zwischen
				reg? und reg?n gesetzt.
				(entry[?].value[?])

	select		Mit select kann man bei Auswahlbefehlsnamen
				den Befehl whlen. (z.B. bei "ADD:SUB") 

	cut_sign	Wenn dieser Wert auf 1 gesetzt wird dann werden
				statt "," die Trennzeichen STEXT[n] nach jedem 
				Eintrag gespeichert. 
	
	code_addres	In code_addres ist die aktuelle Anzeige-Adresse
				des Befehls gespeichert.

	code_addres_h	In code_addres ist die aktuelle Anzeige-Adresse
				des Befehls gespeichert (Bits 32-64).

	no_end		Setzt man no_end auf 1 so wird der nchste Befehl 
				geladen. Fr Befehlsvorstze wie z.B. ES oder REP

	pre_bits	Mit diesen Bits wird bestimmt ob die LTEXT'e vor oder nach
				einen Eintrag eingefgt werden.

	bit_direction	Gibt an wie bei GET die Bits in einem 8 Bit
				Block eingelesen werden.
				0 :    Von Bit 0 bis 7
				1 :    Von Bit 7 bis 0

	segment_pos	Gibt an wo die Segmentangabe ausgegeben
				werden soll.
				0 :		Vor reg0
				1 :		Vor reg1
				2 :		Vor reg2
				3 :		Vor reg3
				4 :		Vor value0
				5 :		Vor value1
				6 :		Vor reg0n
				7 :		Vor reg1n
				8 :		Keine Ausgabe

********************** entry[?].flags *****************************************

	Bits		Funktion

	0-3		: Art des Eintrages AsmEntry;
				0   = Konstante	(mov	ax, 1234h )
				1   = Adresse	(jump	    1234h )
				2   = Referenz	(mov	ax,[1234h])
				3   = Register	(mov	ax, ax    )
				4...= frei

	4		: Segmentnamen vor dem Eintrag schreiben.
	5		: Neue Linie beginnen fr doppel Befehle. (z.B. Texas mpi3 || addi3)
	6		: Befehls Lnge zu value1 addieren ohne Flagbefehle.
	7		: Befehls Lnge zu value1 addieren mit  Flagbefehle.
	8-15	: Positionen sind definiert und werden ausgegeben.
				 8 = regs[0]	(Ausgabe: rtext[regs[0]] )
				 9 = regs[1]
				10 = regs[2]
				11 = regs[3]
				12 = value0	    (Zahlenausgabe )
				13 = value1
				14 = regs[2]	(Ausgabe: rtext[regs[2]] )
				15 = regs[3]
			  Hhere Bit-Positionen werden spter ausgegeben.
	16-24,27,31	: Diese Bits werden nach jedem entry Wechsel mitgenommen.
	16-17	: Aktuelle Befelsbreite 
				0 = Byte  Befehl  (mov  al,[bx + 0012])
				1 = Word  Befehl  (mov  ax,[bx + 0012])
				2 = DWord Befehl  (mov eax,[bx + 0012])
				3 = QWord Befehl   ???
	18-19	: Aktuelle Addressbreitebreite
				0 = Byte  Befehl  (mov  ax,[ bx + 00      ])
				1 = Word  Befehl  (mov  ax,[ bx + 0012    ])
				2 = DWord Befehl  (mov  ax,[ebx + 00001234])
				3 = QWord Befehl   ???
	20-23	: Segmentnummer oder Spezialtrennzeichen (siehe cut_sign)
			  Die Bits whlen einen der ersten 16 Registernamen aus RTEXT
			  bzw. die Trennzeichen aus STEXT
				0	= Segment wird nur angezeigt wenn seg0on != 0
				1...  Segment wird immer angezeigt
			  Die Ausgabe Position wird mit seg_pos bestimmt.
	24-26	: Ausgabelnge von value0 / value0_h
				0 =  2 Zeichen ausgeben
				1 =  4 Zeichen ausgeben
				2 =  8 Zeichen ausgeben
				3 =	16 Zeichen ausgeben
				4 = variable Lnge bis zu '0' Zeichen
				5 = variable Lnge bis zu '0' Zeichen mit Vorzeichen
				6 =  2 Zeichen mit Vorzeichen ausgeben
				7 =  4 Zeichen mit Vorzeichen ausgeben
	27		: Prozessor Modusbits (Zur freien Verwendung)
	28-30	: Ausgabe Lnge von value1 / value1_h wie bei Bit 24-26
	31		: Prozessor Modusbits (Zur freien Verwendung)


//******************** entry[?].layout ****************************************

	Bits		Funktion

	0-3		: Fr regs[0] , falls Bit 8 in flags gesetzt ist
				0   : + oder Space vor ltext[0]	vor  Reg "" bzw. "*" fr 8
				1   : + oder Space und ltext[1] nach Reg "%"
				2   : + oder Space und ltext[2] nach Reg "++"
				3   : + oder Space und ltext[3] nach Reg "--"
				4   : + oder Space und ltext[4] vor  Reg "#"
				5   : + oder Space und ltext[5] vor  Reg "@"
				6   : + oder Space und ltext[6] vor  Reg "++"
				7   : + oder Space und ltext[7] vor  Reg "--"
				8-15: wie oben nur ohne + und Space
	4-7		: Fr regs[1]	Ausgabe wie oben
	8-11	: Fr regs[2]	Ausgabe wie oben
	12-15	: Fr regs[3]	Ausgabe wie oben
	16-19	: Fr value0	Ausgabe wie oben
 	20-23	: Fr value1	Ausgabe wie oben
	24-27	: Fr regs[2]	Ausgabe wie oben
	28-31	: Fr regs[3]	Ausgabe wie oben

//******************** entry[?].clips *****************************************

	Bits		Funktion

	0		: Runde   Klammer auf
	1		: Runde   Klammer zu
	2		: Eckige  Klammer auf
	3		: Eckige  Klammer zu
	4-31	: Wie oben nur fr regs[1],regs[2], ...

//****************** Wie sieht die Ausgabe aus ********************************

Ausgabe Layout fr ein Register

	c = in clips  definiert
	f = in flags  definiert
	l = in layout definiert
	g = global    definiert

	stext0 "SegReg" stext1 [+(text? "reg[0]" text?)] ...
	   g	  f		  g	   clc  g l            g lcc

	Anmerkung "SegReg" kann mit der Variabel seg_pos verschoben werden.

//******************** Befehle fr Mikroprogramme ******************************

	GET  M1 = [Startbit/M2] ZahlenTerm #ABNSDGMU  0-32

	 	Holt einen Wert aus der Bit-Schlange (Standard 8-Bit)
		und speichert ihn in M1
		Startbit/M2 gibt an wo begonnen werden soll (0=aktuell)
		M1 = ZahlenTerm(Wert)

		#A	aktuelle Wortbreite   nehmen (flags 18-19)
		#B	aktuelle Befehlbreite nehmen (flags 16-17)
		#N	Bitadresse nicht erhhen
		#S  Startadresse = 0            (bei Maskenanfang)
		#D  Startadresse = Daten-Anfang (nach Maske)
		#G  Litle zu Big Indian tauschen
		#M  M2 ist die Bit Anzahl
		#U  Wert erweitern auf Unsigned
		#0-32	Bit-Anzahl die geholt wird

		Anmerkung die Richtung wie die Bits geholt werden kann
		mit der Variabel "bit_direction" eingestellt werden.

	SET  M1 = [Zahl/M2] ZahlenTerm

		Speichert einen Wert in M1
		Zahl/M2 ist die Quelle
		M1 = ZahlenTerm(Wert)

	BIT  M1 = [Zahl/M2] #Nr

		Speichert ein Bit in M1
		Zahl/M2 ist die Bitnummer im Bitfeld (0-255)
		#Nr ist die Nummer des Bitfeldes

	AND	  Rechenbefehle (Syntax wie SET)
	OR
	XOR
	ADD
	SUB
	MUL
	DIV
	MOD
	SHIFT

	SET64	Rechenbefehle mit 64-Bit (Syntax wie SET)
	AND64	Es wird mit 32-Bit gerechnet und auf einen
	OR64	64-Bit wert zugewisen
	XOR64	(z.B. value0/value0_h m0/m1 ... )
	ADD64
	SUB64
	MUL64
	DIV64
	MOD64
	SHIFT64

	NEXT							Nchsten Asm-Eintrag bestimmen
	XCHG [M1/Zahl1]	[M2/Zahl2]		Asm-Eintrge tauschen

	IF	M1/Zahl1 != M2/Zahl2  &... GOTO Marke
				 ==
				 >=
				 <=
				 >
				 <

		Fhrt einen bedingten Sprung aus

	CONST	"Name" M/Zahl &... |...  *... /....  $... !....

							Neuen M Eintrag erzeugen
			&... |...		Maske fr entry.flags  (AND/OR)
			*... /...		Maske fr entry.layout (AND/OR)
			$... !...		Maske fr entry.clip   (AND/OR)
							Bei einen Schreibzugriff auf ein
							mit SET AND OR... werden diese
							Bits im aktuellen entry Eintrag gesetzt
							Mit CONST kann man neue	Eintrge erschafffen.
							Ist M angegeben so wird ein bestehender
							Eintrag abgeleitet und kann
							mit neuen Flags versehen werden
							Ist Zahl angegeben wird ein neuer
							Speicher angegeben der mit Zahl
							initialisiert wird

	TEXT	"Text"	#BNA	Text ausgeben (kein Flag = am Anfang augeben)
							#B : Den Text als Befehlsnamen ausgeben
							#N : Den Text am Ende anhngen
							#A : In die nchste Linie geben


	PRINT	[M1/Zahl] [M2/Brt] #BNA
							Zahl ausgeben
							Brt=Anzahl der Zeichen
							#B : Den Text als Befehlsnamen ausgeben
							#N : Den Text am Ende anhngen
							#A : In die Nchste Linie geben


	PRINT	"Format" [M1/Zahl] [M2/Zah2] #BNA
							Formatirten Text aus geben wie
							printf("Format",M1,M2);
							#B : Den Text als Befehlsnamen ausgeben
							#N : Den Text am Ende anhngen
							#A : In die Nchste Linie geben

	BFIELD	"Dump" Zahl		Neues Bitfeld erzeugen, Zahl ist die Nummer des Feldes
							In Dump steht ein 32-Byte Hex-Dump z.B. "7F 22 44 5555 .... "

	ALIAS	"Text" "Text2"	Einen anderen  Text fr einen Befehl festlegen
	LTEXT	"Text" Zahl		Neuen Layout   Text speichern (fr entry[?].layout)
	RTEXT	"Text" Zahl		Neuen Register Text speichern (fr entry[?].regs)
	STEXT	"Text" Zahl		Neuen Segment  Text speichern (Segment Anzeige)
	SETMATCH Zahl [Seite]	Shift fr den Match-Raster einstellen (in Bits 0..5 von Zahl)
							z.B. SETMATCH 0 dann wird das erste Byte
							fr den Masken Raster benutzt. In den Bits 8..15 von Zahl
							wird die Maske fr den Match gespeichert.
	GOTO	"Marke"			Zu anderer Marke springen
	END						Ende des Sub-Befehls
	INFO					Infotext zu einen Befehl 
	MASK					Begin der Befehlsmaske

	:Marke					Marken von 'A' bis 'z' sind Einsprung
							Punkte fr die Mikroprogramme
							Andere Marken sind normale Sprung Ziele

	:Init					Marke fr das Init-Programm
							Das in AsmInit() aufgerufen wird.
							ber m0 und m1 knnen die Ausgabeformate
								von value[?] verndert werden
								(siehe flags Bits 24-26)
								0x80 = Vorzeichen ein
								0x7F = ist die Zahl der Ausgabebits

	ZahlenTerm:

		[M/Zahl] [&...] [|... / ^...] [<<... / >>... / +.... / -....]
		... steht fr eine Zahl
		Jede Zahl kann mit '~' negiert werden
		Der Term wird so ausgerechnet:

		Wert= (((Zahl&...)+...)|...)
		Wert= (((Zahl&...)    )^...)<<...
		Wert= (((Zahl&...)-...)^...)
		Wert= (((Zahl&...)    )|...)>>...

		OR-Werte knnen nicht mit XOR-Werten angegeben werden.
		Es ist auch nur einer von  + - >> << erlaubt.
		Wird for die Zahl ein ~ gesetzte so wird die Zahl invertiert

//******************** Masken *************************************************

	Maskenformat:

		0F:10???111:4?FF	"Name"			abc r:1:2
		0F34				"Name:Name1"	abc r:1:2
		:1111????			"Name"			abc
		66									ab
		>44332211			"Name"			ab

		':'		Steht fr den Wechsel von Hex auf Bin-Mode (nur alle 8 Bit erlaubt)
		'?'		Steht fr ein beliebiges Bit
		'_'     Platzhalterzeichen
		'>'     Bigendian Maske mit 32 Bit
		"Name"	Ist der Name des Befehls fr Namesketten ':' einfgen
		abc     Sub-Programm Code
		r:32:2  Sub-Programm Code mit 2 bergabeparametern (zwischen 0-47)
				Die bergabeparameter werden in param0 und param1 gespeichert

******************************************************************************/
#ifndef __ASM_DECODER_H__
#define __ASM_DECODER_H__


#if _MSC_VER >= 1000
#pragma once
#endif

#define 	ASM_NO_TEXT		0x80000000		// Keinen Zeilentext in AsmDecode generieren (fr Page)
#define 	ASM_EF_TYPE		0x0000000F		// Maske fr entry[?].flags (Art des Eintrages)
#define 	ASM_EF_SEGON	0x00000010		// Segmentnamen vor dem Eintrag schreiben
#define 	ASM_EF_NEWLINE	0x00000020		// Neue Linie beginnen fr doppel Befehle (z.B. Texas mpi3 || addi3)
#define 	ASM_EF_ADDLEN	0x00000040		// Befehls Lnge zu value1 addieren ohne Flagbefehle
#define 	ASM_EF_ADDLEN1	0x00000080		// Befehls Lnge zu value1 addieren mit  Flagbefehle
#define 	ASM_EF_REG0		0x00000100		// entry[?].regs[0]  wurde ausgegeben
#define 	ASM_EF_REG1		0x00000200		// entry[?].regs[1]  wurde ausgegeben
#define 	ASM_EF_REG2		0x00000400		// entry[?].regs[2]  wurde ausgegeben
#define 	ASM_EF_REG3		0x00000800		// entry[?].regs[3]  wurde ausgegeben
#define 	ASM_EF_VALUE0	0x00001000		// entry[?].value0 wurde ausgegeben
#define 	ASM_EF_VALUE1	0x00002000		// entry[?].value1 wurde ausgegeben
#define 	ASM_EF_REG2N	0x00004000		// entry[?].regs[2]  wurde am Ende ausgegeben
#define 	ASM_EF_REG3N	0x00008000		// entry[?].regs[3]  wurde am Ende ausgegeben
#define 	ASM_EF_CODEBRT 	0x00030000		// Aktuelle Befels Breite (0=Byte 1=Wort 2=DWord 3=QWord)
#define 	ASM_EF_ADDRBRT	0x000C0000		// Aktuelle Zeiger Breite (0=Byte 1=Wort 2=DWord 3=QWord)
#define 	ASM_EF_SEG		0x00F00000		// Segment Nummer (die ersten 16 Registernamen)
#define 	ASM_EF_USER		0x88000000		// fr frei Benutzung
#define 	ASM_EF_VLEN0	0x07000000		// Ausgabe Lnge von value0
#define 	ASM_EF_VLEN1	0x70000000		// Ausgabe Lnge von value1

#define		ASM_DO_QWORD(a,b)				(((ULONGLONG)(b)<<32)|((DWORD)(a)))
#define 	ASM_HI_DWORD(a)					((DWORD)((ULONGLONG)(a)>>32))
#define 	ASM_LO_DWORD(a)					((DWORD)(a))

typedef	unsigned int 		ASM_DWORD;
typedef	unsigned short 		ASM_WORD;
typedef	unsigned char  		ASM_BYTE;

#ifdef 	_MSC_VER
typedef	unsigned __int64	ASM_QWORD;
#else
typedef	unsigned long long  ASM_QWORD;
#endif



typedef	struct
		{
		ASM_DWORD 	flags;			// Was wurde definiert ?
		ASM_DWORD	layout;			// Was soll angezeigt werden
		ASM_DWORD	clips;			// Welche Klammern setzen
		ASM_DWORD	type;			// Befelstyp (frei definierbar)
		ASM_DWORD	regs[4];		// Registernummern

	    union
	  	  {
	    struct
		    {
		ASM_DWORD 	value0;			// Konstanten
		ASM_DWORD	value0_h;		// Konstanten
		    };	
		ASM_BYTE	value0_b[8];	// Konstanten als Array
		ASM_WORD	value0_w[4];	// Konstanten als Array
		ASM_DWORD	value0_d[2];	// Konstanten als Array
		ASM_QWORD	value0_q;		// Konstanten als 64-Bit
		  }; 

	    union
	  	  {
	    struct
		    {
		ASM_DWORD 	value1;			// Konstanten
		ASM_DWORD	value1_h;		// Konstanten
		    };	
		ASM_BYTE	value1_b[8];	// Konstanten als Array
		ASM_WORD	value1_w[4];	// Konstanten als Array
		ASM_DWORD	value1_d[2];	// Konstanten als Array
		ASM_QWORD	value1_q;		// Konstanten als 64-Bit
		  }; 

		}AsmEntry;


typedef	struct
		{
		AsmEntry   *entry[4];		// Eintrge in der Zeile
		int			entry_count;	// Anzahl der Eintrge in der Zeile
		int			page;			// Maskenseite fr den nchsten Befehl
		int			type;			// Befehls Type
		int			line_number;	// Nummer der Zeile
		short 		line_count;		// Anzahl der Zeilen
		short 		line_length;	// Zeilelnge in Textzeichen
		short		line_cpos;		// Position des Code nach HEX-Dump
		short 		line_ppos;		// Position der Parameter
		short 		line_vpos[4];	// Position der Values
		short 		line_rpos[8];	// Position der Register
		char	   *line_text;		// Zeilentext
		}AsmEntryInfo;



void	AsmExit();
int		AsmReInit    ();
int		AsmInit      (const char *Filename,unsigned  Param1=0,unsigned Param2=0);
int		AsmEncode    (const char *Codename,unsigned *Masken,int Max,int Page=-1);
int		AsmDecode    (const void *Mem,AsmEntryInfo &Info,ASM_QWORD Address,unsigned Page=0);
void	AsmGetFormat (int  *Addr_anz,int *Data_anz,int *Bef_anz=0);
void	AsmSetFormat (int   Addr_anz,int  Data_anz,int  Bef_anz=7);
void	AsmGetLine   (int   Nr, AsmEntryInfo &Info);
char   *AsmGetInfo   (const char *Command);
char   *AsmGetRegName(int   Nr);
int		AsmBigEndian ();

#endif
