//*****************************************************************************
//*
//*
//*		CBackgroundCtrl
//*
//*
//*****************************************************************************
//
//	Zeichnet ein Hintergrundbild in ein Fenster (*.gif,*.bmp,*.jpg ...)
//
//	Funktionen:
//
//		BOOL  Init(CWnd *pWnd ,LPCTSTR  pFilename=0,int iMode=1)	// Fenster initialisieren
//		BOOL  Init(HWND  hHwnd,LPCTSTR  pFilename=0,int iMode=1)	// Fenster initialisieren
//		BOOL  Init(CWnd *pWnd ,UINT     uResId     ,int iMode=1)	// Fenster initialisieren
//		BOOL  Init(HWND  hHwnd,UINT     uResId     ,int iMode=1)	// Fenster initialisieren
//		BOOL  Remove()												// Grafik entfernen
//		BOOL  Save(LPCTSTR pFilename)								// Grafik speichern
//		BOOL  Open(LPCTSTR pFilename)								// Grafik laden
//		BOOL  Open(UINT    uResId )									// Grafik aus Resource laden
//		BOOL  GetSize(int &iSizeX,int &iSizeY)						// Grafikgre holen (1/100mm)
//		CSize GetSize()												// Grafikgre holen (1/100mm)
//		void  SetDefaultBrush(HBRUSH hBrush)						// Hintergrundfarbe fr leeres Bild
//		void  SetDefaultBrush(int    iBrush)						// Hintergrundfarbe fr leeres Bild ber GetStockObjekt(Brush)
//		void  SetViewRect    (LPRECT pRect )						// Anzeige Position
//		char* GetFileName();										// Dateinamen abfragen
//
//	Anmerkung:
//
//		CMfcTestModule _Module;										// Einfgen in Code
//
#ifndef __BACKGROUNDCTRL_H__
#define	__BACKGROUNDCTRL_H__


#if	_MSC_VER > 1000
#pragma once
#endif


#include <atlbase.h>
extern CMfcTestModule _Module;
#include <atlwin.h>

#ifndef	WM_SUBDRAW
#define WM_SUBDRAW		0x7F0F
#endif

#ifndef	ON_WM_SUBDRAW
#define	ON_WM_SUBDRAW()	{WM_SUBDRAW,0,0,0,AfxSig_lwl,(AFX_PMSG)(AFX_PMSGW)(void (AFX_MSG_CALL CWnd::*)(WPARAM,LPARAM))&OnSubDraw},
#endif


//*****************************************************************************
//*
//*		Klasse	CBackgroundCtrl
//*
//*****************************************************************************
class CBackgroundCtrl:public CWindowImpl<CBackgroundCtrl>
{
public:

	typedef enum HandlerTypeEnum
		{
		ClientPaint     = WM_PAINT,
		BackGroundPaint = WM_ERASEBKGND
		}HandlerTypeEnum;


	inline BOOL		Draw(HDC   hHdc ,LPRECT  pRect);
	inline BOOL		Init(CWnd *pWnd ,LPCTSTR pFilename=0,int iMode=1);
	inline BOOL		Init(HWND  hHwnd,LPCTSTR pFilename=0,int iMode=1);
	inline BOOL		Init(CWnd *pWnd ,UINT    uResId     ,int iMode=1);
	inline BOOL		Init(HWND  hHwnd,UINT    uResId     ,int iMode=1);
	inline BOOL	    GetSize(int &iSizeX,int &iSizeY);
	inline CSize    GetSize();
	inline void 	SetDefaultBrush(HBRUSH hBrush);
	inline void 	SetDefaultBrush(int    iBrush);
	inline void		SetViewRect    (LPRECT pRect );

	virtual ~CBackgroundCtrl(){};
	         CBackgroundCtrl()
			{
			m_bClipChilds	  = TRUE;
			m_pRect           = NULL;
			m_nMessageHandler = ClientPaint;
			m_hDefBrush       = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
			};

	inline BOOL Remove()
		{
		BOOL	bOk;

		bOk			= (m_pPicture)? TRUE:FALSE;
		m_pPicture  = NULL;
		Invalidate();

		return bOk;
		}


	inline LPCSTR GetFileName();

	BEGIN_MSG_MAP(CBackgroundCtrl)
		MESSAGE_HANDLER(WM_PAINT,	  OnPaint)
		MESSAGE_HANDLER(WM_ERASEBKGND,OnEraseBkGnd)
	END_MSG_MAP()


//*****************************************************************************
//*
//*		OpenRes
//*
//*****************************************************************************
//	Ld ein Bild mit der Id aus der Resource "PICTURE" oder in RT_HTML,RT_BITMAP
// 	Ergibt TRUE wenn das Bild geladen wurde

protected:

virtual BOOL OpenRes(LPCSTR pResString)
	{
	BOOL	bResult;
	LPVOID	pPicData;
	LPVOID	pResData;
	HRESULT hResult;
	HGLOBAL hGlobal;
	HGLOBAL hRscData;
	HRSRC	hResource;
	DWORD	uSize;
	DWORD	uAdd;


	bResult = FALSE;
	uAdd    = 0;

	Close();

	               hResource= FindResource(NULL,pResString,"PICTURE" );
	if(!hResource) hResource= FindResource(NULL,pResString, RT_HTML  );
	if(!hResource){hResource= FindResource(NULL,pResString, RT_BITMAP);uAdd=14;}
	if(!hResource){Invalidate();return FALSE;}

	hRscData = LoadResource  (NULL,hResource);
	uSize    = SizeofResource(NULL,hResource);

	if(!hRscData){Invalidate();return FALSE;}
	if( hGlobal  = GlobalAlloc(GMEM_MOVEABLE,uSize+uAdd))
	if( pPicData = GlobalLock (hGlobal))
		{
		hResult  = NULL;
		pResData = LockResource(hRscData);
		memcpy((char*)pPicData+uAdd,pResData,uSize);
		GlobalUnlock(hGlobal);

		if(uAdd)									// Bitmap Header vorsetzen
			{
			BITMAPINFOHEADER   *pBmInfo;
			BITMAPFILEHEADER	sBitmap;

			pBmInfo = (BITMAPINFOHEADER*)((char*)pPicData+uAdd);

			sBitmap.bfType		 = 'B'+'M'*256;
			sBitmap.bfSize		 = uSize+uAdd;
			sBitmap.bfReserved1  = 0;
			sBitmap.bfReserved2  = 0;
			sBitmap.bfOffBits	 = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER) ;

			if(pBmInfo->biBitCount<=8)
				{
				sBitmap.bfOffBits += sizeof(RGBQUAD)*(1<<pBmInfo->biBitCount);
				}

			memcpy(pPicData,&sBitmap,uAdd);
			}

		CComPtr<IStream> sStream;

		if(SUCCEEDED(CreateStreamOnHGlobal(hGlobal,TRUE,&sStream)))
		if(SUCCEEDED(hResult=OleLoadPicture(sStream,0,FALSE,IID_IPicture,(void**)&m_pPicture)))
			{
			bResult = TRUE;
			}
		}

	DeleteObject(hRscData);
	Invalidate();

	return bResult;
	}

public:

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
//	Ld das Bild pFilename ein Bild in den Speicher
// 	Ergibt TRUE wenn das Bild geladen wurde
virtual BOOL Open(LPCTSTR pFilename)
	{
	OFSTRUCT	sOfData;
	HGLOBAL		hGlobal;
	LPVOID		pPicData;
	BOOL		bResult;
	BOOL		bRead;
	HANDLE		hFile;

	bResult = FALSE;
	hFile   = NULL;

	Close();

	if(pFilename[0]=='R'  || pFilename[0]=='s')
	if(pFilename[1]=='E'  || pFilename[1]=='e')
	if(pFilename[2]=='S'  || pFilename[2]=='s')
	if(pFilename[3]==':')
	if(pFilename[4]=='\\' || pFilename[4]=='/')
	if(pFilename[5]=='\\' || pFilename[5]=='/')
		{
		   bResult=OpenRes(pFilename+6);
		if(bResult){m_sFilename=pFilename;}
		return bResult;
		}


	if(pFilename)
		{
		   hFile  = (HANDLE)OpenFile(pFilename,&sOfData,OF_READ|OF_SHARE_COMPAT);
		if(hFile != (HANDLE)HFILE_ERROR )
			{
			DWORD   uFileSize;
			DWORD   uHighWord;
			DWORD	uSizeLow;
			DWORD	uBytes;
			HRESULT	hResult;

			hResult   = NULL;
			uHighWord = NULL;
			uSizeLow  = GetFileSize(hFile,&uHighWord);
			uFileSize = uSizeLow;
			hGlobal   = GlobalAlloc(GMEM_MOVEABLE,uFileSize);

			if(hGlobal)
				{
				   pPicData = GlobalLock(hGlobal);
				if(pPicData)
					{
					uBytes = 0;
					bRead  = ReadFile(hFile,pPicData,uFileSize,&uBytes,NULL);
					GlobalUnlock(hGlobal);

					if(bRead)
						{
						CComPtr<IStream> sStream;
						_ASSERTE( uFileSize == uBytes );
						if(SUCCEEDED(CreateStreamOnHGlobal(hGlobal,TRUE,&sStream)))
						if(SUCCEEDED(hResult=OleLoadPicture(sStream,0,FALSE,IID_IPicture,(void**)&m_pPicture)))
							{
							bResult = TRUE;
							}
						}
					}
				}

			CloseHandle(hFile);
			}
		}

	if(bResult)
		{
		m_sFilename = pFilename;
		}

	Invalidate();

	return bResult;
	}

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
//	Ld ein Bild mit der Id aus der Resource "PICTURE" oder in RT_HTML,RT_BITMAP
// 	Ergibt TRUE wenn das Bild geladen wurde
inline BOOL Open(UINT uId)
	{
	BOOL bResult;

	   bResult=OpenRes(MAKEINTRESOURCE(uId));
	if(bResult)m_sFilename.Format("res:\\\\#%u",uId);

	return bResult;
	}

//*****************************************************************************
//*
//*		Save
//*
//*****************************************************************************
//	Speichert das Bild als Datei pFilename
// 	Ergibt TRUE wenn das Bild gespeichert wurde
virtual BOOL Save(LPCTSTR pFilename)
	{
	LONG		uFileSize;
	LPVOID		pPicData;
	HGLOBAL		hGlobal;
	OFSTRUCT	sOfData;
	HRESULT		hResult;
	BOOL		bResult;
	BOOL		bWrite;
	DWORD		uBytes;
	HANDLE		hFile;

	hFile   = NULL;
	bResult = FALSE;

	if(pFilename && m_pPicture)
		{
		uFileSize = 1;
		hResult   = NULL;
		hGlobal   = GlobalAlloc(GMEM_MOVEABLE,uFileSize);

		if(hGlobal)
			{
			CComPtr<IStream> sStream;

			uBytes = 0;

			if(SUCCEEDED(CreateStreamOnHGlobal (hGlobal,TRUE ,&sStream  )))
			if(SUCCEEDED(m_pPicture->SaveAsFile(sStream,FALSE,&uFileSize)))
				{
				pPicData = GlobalLock(hGlobal);

				if(pPicData)
					{
					   hFile =(HANDLE)OpenFile(pFilename,&sOfData,OF_CREATE|OF_SHARE_EXCLUSIVE|OF_WRITE);
					if(hFile!=(HANDLE)HFILE_ERROR)
						{
						bWrite = WriteFile(hFile,pPicData,uFileSize,&uBytes,NULL);
						CloseHandle(hFile);
						if(bWrite && uBytes==(DWORD)uFileSize)bResult=TRUE;
						}
					}
				}
			}
		}

	return bResult;
	}

//*****************************************************************************
//*
//*		protected Funktionen
//*
//*****************************************************************************

protected:

	inline BOOL IsHandlerMessage(UINT uMsg)
		{
		return (m_nMessageHandler==(HandlerTypeEnum)uMsg);
		}

	void PutPicture(IPicture *pPicture,HDC hDC,RECT sRect)
		{
		OLE_YSIZE_HIMETRIC nHeight=0;
		OLE_XSIZE_HIMETRIC nWidth =0;

		pPicture->get_Width (&nWidth );
		pPicture->get_Height(&nHeight);
		pPicture->Render(hDC,sRect.left,sRect.top,sRect.right-sRect.left,sRect.bottom-sRect.top,0,nHeight,nWidth,-nHeight,NULL);
		};

	LRESULT OnEraseBkGnd(UINT uMsg,WPARAM wParam,LPARAM lParam,BOOL &bHandled)
		{
		HWND	hWndChild;
		HGDIOBJ	hOldBr;
		HGDIOBJ	hOldPn;
		RECT	sRect;
		RECT 	sTemp;
		RECT 	sPos;
		HDC  	hDC;

		if(!IsHandlerMessage(uMsg))				// Wird ber diese Message gezeichnet
			{
			bHandled = FALSE;
			return FALSE;
			}

		GetClientRect(&sRect);
		BeginPaint(NULL);
		hDC = GetDC();

		if(m_bClipChilds)						// Kind-Fenster nicht bemalen
			{
			hWndChild = GetWindow(GW_CHILD);

			while(hWndChild && ::IsWindow(hWndChild))
				{
				if(::IsWindowVisible(hWndChild))
					{
					::GetWindowRect(hWndChild,&sPos);
					ScreenToClient (&sPos);
					ExcludeClipRect(hDC,sPos.left,sPos.top,sPos.right,sPos.bottom);
					}

				hWndChild = ::GetWindow(hWndChild,GW_HWNDNEXT);
				}
			}

		if(m_pPicture)
			{
			if(!m_pRect)						// bers ganze Fenster zeichnen
				{
				PutPicture(m_pPicture,hDC,sRect);
				}
			else{								// Auf spezielle Position zeichnen
				hOldBr = SelectObject(hDC,m_hDefBrush);
				hOldPn = SelectObject(hDC,GetStockObject(NULL_PEN));
				sTemp  = m_sRect;
												// Rahmen um Bild zeichnen
				if(sTemp.top   <            0)sTemp.top   = 0;
				if(sTemp.left  <            0)sTemp.left  = 0;
				if(sTemp.right >sRect.right  )sTemp.right = sRect.right;
				if(sTemp.bottom>sRect.bottom )sTemp.bottom= sRect.bottom;
				if(sTemp.top   >            0)Rectangle(hDC,0          ,0           ,sRect.right+1,sTemp.top   +1);
				if(sTemp.bottom<=sRect.bottom)Rectangle(hDC,0          ,sTemp.bottom,sRect.right+1,sRect.bottom+1);
				if(sTemp.left  >            0)Rectangle(hDC,0          ,sTemp.top   ,sTemp.left +1,sTemp.bottom+1);
				if(sTemp.right <=sRect.right )Rectangle(hDC,sTemp.right,sTemp.top   ,sRect.right+1,sTemp.bottom+1);

				SelectObject(hDC,hOldBr);
				SelectObject(hDC,hOldPn);
				PutPicture(m_pPicture,hDC,m_sRect);
				}
			}
		else{									// Kein Bild vorhanden
			hOldBr = SelectObject(hDC,m_hDefBrush);
			hOldPn = SelectObject(hDC,GetStockObject(NULL_PEN));
			Rectangle(hDC,sRect.left,sRect.top,sRect.right+1,sRect.bottom+1);
			SelectObject(hDC,hOldBr);
			SelectObject(hDC,hOldPn);
			}

		SendMessage(m_hWnd,WM_SUBDRAW,(WPARAM)hDC,(LPARAM)&sRect);
		ReleaseDC(hDC);
		EndPaint(NULL);

		return 1;
		}

	LRESULT OnPaint(UINT uMsg,WPARAM wParam,LPARAM lParam,BOOL &bHandled)
		{
		HGDIOBJ	hOldBr;
		HGDIOBJ	hOldPn;
		RECT	sRect;
		RECT	sTemp;
		HDC		hDC;

		if(!IsHandlerMessage(uMsg))				// Wird ber diese Message gezeichnet
			{
			bHandled = FALSE;
			return NULL;
			}

		GetClientRect(&sRect);
		BeginPaint(NULL);
		hDC = GetDC();

		if(m_pPicture)
			{									// bers ganze Fenster zeichnen
			if(!m_pRect)
				{
				PutPicture(m_pPicture,hDC,sRect);
				}
			else{								// Auf spezielle Position zeichnen
				hOldBr = SelectObject(hDC,m_hDefBrush);
				hOldPn = SelectObject(hDC,GetStockObject(NULL_PEN));
				sTemp  = m_sRect;
												// Rahmen um Bild zeichnen
				if(sTemp.top   <            0)sTemp.top   =0;
				if(sTemp.left  <            0)sTemp.left  =0;
				if(sTemp.right > sRect.right )sTemp.right =sRect.right;
				if(sTemp.bottom> sRect.bottom)sTemp.bottom=sRect.bottom;
				if(sTemp.top   >            0)Rectangle(hDC,0          ,0           ,sRect.right+1,sTemp.top   +1);
				if(sTemp.bottom<=sRect.bottom)Rectangle(hDC,0          ,sTemp.bottom,sRect.right+1,sRect.bottom+1);
				if(sTemp.left  >            0)Rectangle(hDC,0          ,sTemp.top   ,sTemp.left +1,sTemp.bottom+1);
				if(sTemp.right <=sRect.right )Rectangle(hDC,sTemp.right,sTemp.top   ,sRect.right+1,sTemp.bottom+1);

				SelectObject(hDC,hOldBr);
				SelectObject(hDC,hOldPn);
				PutPicture(m_pPicture,hDC,m_sRect);
				}
			}
		else{									// Kein Bild vorhanden
			hOldBr = SelectObject(hDC,m_hDefBrush);
			hOldPn = SelectObject(hDC,GetStockObject(NULL_PEN));
			Rectangle(hDC,sRect.left,sRect.top,sRect.right+1,sRect.bottom+1);
			SelectObject(hDC,hOldBr);
			SelectObject(hDC,hOldPn);
			}

		SendMessage(m_hWnd,WM_SUBDRAW,(WPARAM)hDC,(LPARAM)&sRect);
		ReleaseDC(hDC);
		EndPaint(NULL);

		return 1;
		};

	void Close()
		{
		m_pPicture  = NULL;
		m_sFilename = _T("");
		}


	CComPtr<IPicture>	m_pPicture;
	HandlerTypeEnum		m_nMessageHandler;
	HBRUSH				m_hDefBrush;
	BOOL				m_bClipChilds;
	RECT				m_sRect;
	LPRECT				m_pRect;
	CString				m_sFilename;

};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

inline BOOL CBackgroundCtrl::Init(CWnd *pWnd ,LPCTSTR pFilename,int iMode){return Init(pWnd->m_hWnd,pFilename,iMode);}
inline BOOL CBackgroundCtrl::Init(HWND  hHwnd,LPCTSTR pFilename,int iMode)
	{
	m_nMessageHandler = (iMode&1)? BackGroundPaint:ClientPaint;
	m_bClipChilds     = (iMode&2)? FALSE:TRUE;
	if(!SubclassWindow(hHwnd))return FALSE;
	if(!pFilename)return TRUE;
	return Open(pFilename);
	}

inline BOOL CBackgroundCtrl::Init(CWnd *pWnd ,UINT uResId,int iMode){return Init(pWnd->m_hWnd,uResId,iMode);}
inline BOOL CBackgroundCtrl::Init(HWND  hHwnd,UINT uResId,int iMode)
	{
	m_nMessageHandler = (iMode&1)? BackGroundPaint:ClientPaint;
	m_bClipChilds     = (iMode&2)? FALSE:TRUE;
	if(!SubclassWindow(hHwnd))return FALSE;
	return Open(uResId);
	}

inline BOOL CBackgroundCtrl::Draw(HDC hDc,LPRECT pRect)
	{
	if(!m_pPicture)return FALSE;
	PutPicture(m_pPicture,hDc,*pRect);
	SendMessage(GetParent(),WM_SUBDRAW,(WPARAM)hDc,(LPARAM)pRect);
	return TRUE;
	};

inline BOOL	CBackgroundCtrl::GetSize(int &iSizeX,int &iSizeY)
	{
	OLE_XSIZE_HIMETRIC nWidth  = 0;
	OLE_YSIZE_HIMETRIC nHeight = 0;
	if(!m_pPicture){iSizeX=0;iSizeY=0;return FALSE;}
	m_pPicture->get_Width (&nWidth );
	m_pPicture->get_Height(&nHeight);
	iSizeY = nHeight;
	iSizeX = nWidth;
	return TRUE;
	}

inline CSize CBackgroundCtrl::GetSize()
	{
	OLE_XSIZE_HIMETRIC nWidth  = 0;
	OLE_YSIZE_HIMETRIC nHeight = 0;
	if(!m_pPicture)return CSize(0,0);
	m_pPicture->get_Width (&nWidth );
	m_pPicture->get_Height(&nHeight);
	return CSize(nWidth,nHeight);
	}

inline void CBackgroundCtrl::SetViewRect(LPRECT pRect)
	{
	bool	bChange=0;

	if(pRect)
		{
		if(!m_pRect || memcmp(&m_sRect,pRect,sizeof(m_sRect)))
			{
			m_pRect = &m_sRect;
			m_sRect = *pRect;
			bChange = 1;
			}
		}
	else{
		if(m_pRect)
			{
			m_pRect = NULL;
			bChange = 1;
			}
		}

	if(bChange && ::IsWindow(m_hWnd))
		{
		Invalidate();
		}
	}

inline LPCSTR CBackgroundCtrl::GetFileName(){return m_sFilename;}
inline void   CBackgroundCtrl::SetDefaultBrush(HBRUSH hBrush){m_hDefBrush=hBrush;}
inline void   CBackgroundCtrl::SetDefaultBrush(int    iBrush)
	{
	HBRUSH	hBrush;

	hBrush=(HBRUSH)GetStockObject(iBrush);

	if(GetObjectType(hBrush)==OBJ_BRUSH)
		{
		m_hDefBrush = hBrush;
		}
	}

#endif
