//*****************************************************************************
//*
//*
//*		BitmapWnd.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
#ifndef	  __BITMAPWND_H__
#define   __BITMAPWND_H__

#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<AfxWin.h>
#include	<AfxMt.h>
#include	"Image/BitmapImage.h"

#define 	ID_BITMAPWND_DEF	0x9718
#define		CBW_SCROLL_X        0x00000001			// X-Scrollbar anzeigen
#define		CBW_SCROLL_Y        0x00000002			// Y-Scrollbar anzeigen
#define		CBW_SHOW_SCR_X      0x00000004			// X-Scrollbar immer anzeigen
#define		CBW_SHOW_SCR_Y      0x00000008			// Y-Scrollbar immer anzeigen
#define		CBW_CENTER	        0x00000010			// Soll das Bild zentriert werden
#define		CBW_FIXED_SIZE	    0x00000020			// Das Bild hat eine fixe Gre
#define 	CBW_ZERO_SCROLL		0x00000040			// Wenn Fenster grer als Bild immer auf 0/0 scrollen

//*****************************************************************************
//*
//*		Klasse CBitmapWnd
//*
//*****************************************************************************
class CBitmapWnd : public CWnd
{
public:

	typedef			void (_cdecl *DrawCB )(void *pParam,CDC *pDc,PAINTSTRUCT *pPaint);
	typedef			void (_cdecl *MouseCB)(void *pParam,UINT uType,UINT nFlags,CPoint sPoint);

	CBitmapWnd();
   ~CBitmapWnd();

	BitmapImage	   *GetImage();
	CDC			   *GetMemDC();

	BOOL			Create  (                DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			Create  (                DWORD dwStyle,unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);
	BOOL			Create  (                              const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			Create  (                              unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,DWORD dwStyle,unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,              const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,              unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);

	BOOL			IsRgb();
	int 			Lock();
	int 			Unlock();

	HCURSOR			SetCursor (HCURSOR  hCursor);
	void			SetBkColor(COLORREF uBkColor);
	void			SetMinSize(CSize sMin);
	void			SetMinSize(int iMinX,int iMinY);
	void			SetMaxSize(CSize sMax);
	void			SetMaxSize(int iMaxX,int iMaxY);
	BOOL			SetScrollPos(CPoint sScroll     ,BOOL bRedraw=TRUE);
	BOOL			SetScrollPos(int iPosX,int iPosY,BOOL bRedraw=TRUE);
	void			SetDrawCallback (DrawCB  pProc,void *pParam=0);
	void			SetMouseCallback(MouseCB pProc,void *pParam=0);
	void			GetScrollPos(int &iScrollX,int &iScrollY);
	void			GetScrollPos(CPoint &sScroll);
	void			GetWindowSize(CSize &sSize);
	void			GetWindowSize(int &iSizeX,int &iSizeY);
	void			InvalidateRect (LPCRECT pRect,BOOL bErase=TRUE);
	void			InvalidateFrame(LPCRECT pRect,BOOL bErase=TRUE);
	void			InvalidateRect (int iPosX,int iPosY,int iSizeX,int iSizeY,BOOL bErase=TRUE);
	void			InvalidateFrame(int iPosX,int iPosY,int iSizeX,int iSizeY,BOOL bErase=TRUE);
	void 			PointToOffset  (int &iPosX,int &iPosY);
	void 			OffsetToPoint  (int &iPosX,int &iPosY);
	void 			PointToOffset  (POINT  &sPoint);
	void 			OffsetToPoint  (POINT  &sPoint);
	void 			PointToOffset  (LPPOINT pPoint);
	void 			OffsetToPoint  (LPPOINT pPoint);
	void 			ScreenToClient (LPPOINT pPoint);
	void 			ClientToScreen (LPPOINT pPoint);
	BOOL			PtInClientRect (LPPOINT pPoint);
	BOOL			PtInClientRect (POINT   sPoint);
	BOOL			PtInClientRect (UINT uPosX,UINT uPosY);
	BOOL			PtInImageRect  (POINT   sPoint);
	BOOL			PtInImageRect  (LPPOINT pPoint);
	BOOL			PtInImageRect  (UINT uPosX,UINT uPosY);
	BOOL			ChangeSize(int iSizeX,int iSizeY);
	BOOL			ChangeSize(CSize sSize);
	BOOL			EnsureVisible(int iPosX,int iPosY,int iSizeX=0,int iSizeY=0);
	BOOL			EnsureVisible(CPoint &sPoint);
	BOOL			EnsureVisible(CRect  &sRect);


//*****************************************************************************
//*
//*		Variablen
//*
//*****************************************************************************
protected:

	void		   *m_pMouseParam;
	MouseCB			m_pMouseCB;
	void		   *m_pDrawParam;
	DrawCB			m_pDrawCB;
	BitmapImage	   *m_pImage;
	CWnd		   *m_pMainWnd;
	CDC			   *m_pMainDc;
	CDC				m_sMemDC;
	BOOL			m_iShowScrollX;
	BOOL			m_iShowScrollY;
	BOOL			m_bHasScrollX;
	BOOL			m_bHasScrollY;
	BOOL			m_bZeroScroll;
	BOOL			m_bFixedSize;
	BOOL			m_bDoCenter;
	HBITMAP			m_hBitmap;
	HCURSOR			m_hCursor;
	CSemaphore		m_sLockSem;
	COLORREF		m_uBkColor;
	int				m_iWndSizeX;
	int				m_iWndSizeY;
	int				m_iMinSizeX;
	int				m_iMinSizeY;
	int				m_iMaxSizeX;
	int				m_iMaxSizeY;
	int				m_iScrollX;
	int				m_iScrollY;
	int				m_iLockCnt;
	int				m_iBits;
	static TCHAR	m_cClassName[];

	//{{AFX_MSG(CBitmapWnd)
	afx_msg BOOL OnSetCursor(CWnd *pWnd,UINT nHitTest,UINT uMsg);
	afx_msg void OnHScroll(UINT uCode,UINT uPos,CScrollBar *pScrollBar);
	afx_msg void OnVScroll(UINT uCode,UINT uPos,CScrollBar *pScrollBar);
	afx_msg void OnSize(UINT nType,int iSizeX,int iSizeY);
	afx_msg BOOL OnMouseWheel   (UINT nFlags,short zDelta,CPoint sPoint);
	afx_msg void OnMouseMove    (UINT nFlags,CPoint sPoint);
	afx_msg void OnLButtonDblClk(UINT nFlags,CPoint sPoint);
	afx_msg void OnMButtonDblClk(UINT nFlags,CPoint sPoint);
	afx_msg void OnRButtonDblClk(UINT nFlags,CPoint sPoint);
	afx_msg void OnLButtonDown  (UINT nFlags,CPoint sPoint);
	afx_msg void OnMButtonDown  (UINT nFlags,CPoint sPoint);
	afx_msg void OnRButtonDown  (UINT nFlags,CPoint sPoint);
	afx_msg void OnLButtonUp    (UINT nFlags,CPoint sPoint);
	afx_msg void OnMButtonUp    (UINT nFlags,CPoint sPoint);
	afx_msg void OnRButtonUp    (UINT nFlags,CPoint sPoint);
	afx_msg void OnDestroy();
	afx_msg void OnPaint();
	//}}AFX_MSG

	afx_msg void OnDoSizing();

	void	CreateImage(int iSizeX,int iSizeY);
	BOOL	UpdateScroll(int iPosX,int iPosY);
	void	UpdateScrollInfo();
	


	DECLARE_MESSAGE_MAP()


};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	BitmapImage	   *CBitmapWnd::GetImage(){return  m_pImage;}
inline	CDC			   *CBitmapWnd::GetMemDC(){return &m_sMemDC;}
inline	void			CBitmapWnd::GetWindowSize(CSize &sSize){sSize.cx=m_iWndSizeX;sSize.cy=m_iWndSizeY;}
inline	void			CBitmapWnd::GetWindowSize(int &iSizeX,int &iSizeY){iSizeX=m_iWndSizeX;iSizeY=m_iWndSizeY;}
inline	BOOL			CBitmapWnd::ChangeSize(CSize sSize){return ChangeSize(sSize.cx,sSize.cy);}
inline	void			CBitmapWnd::SetMinSize(CSize sMin              ){m_iMinSizeX=sMin.cx;m_iMinSizeY=sMin.cy;};
inline	void			CBitmapWnd::SetMinSize(int iMinX,int iMinY     ){m_iMinSizeX=iMinX  ;m_iMinSizeY=iMinY  ;};
inline	void			CBitmapWnd::SetMaxSize(CSize sMax              ){m_iMaxSizeX=sMax.cx;m_iMaxSizeY=sMax.cy;};
inline	void			CBitmapWnd::SetMaxSize(int iMaxX,int iMaxY     ){m_iMaxSizeX=iMaxX  ;m_iMaxSizeY=iMaxY  ;};
inline	void 			CBitmapWnd::PointToOffset(int &iPosX,int &iPosY){iPosX    -=m_iScrollX;iPosY    -=m_iScrollY;}
inline	void 			CBitmapWnd::PointToOffset(POINT   &sPoint      ){sPoint.x -=m_iScrollX;sPoint.y -=m_iScrollY;}
inline	void 			CBitmapWnd::PointToOffset(LPPOINT  pPoint      ){pPoint->x-=m_iScrollX;pPoint->y-=m_iScrollY;}
inline	void 			CBitmapWnd::OffsetToPoint(int &iPosX,int &iPosY){iPosX    +=m_iScrollX;iPosY    +=m_iScrollY;}
inline	void 			CBitmapWnd::OffsetToPoint(POINT  &sPoint       ){sPoint.x +=m_iScrollX;sPoint.y +=m_iScrollY;}
inline	void 			CBitmapWnd::OffsetToPoint(LPPOINT pPoint       ){pPoint->x+=m_iScrollX;pPoint->y+=m_iScrollY;}
inline	BOOL			CBitmapWnd::SetScrollPos(CPoint sScroll,BOOL bRedraw){return SetScrollPos(sScroll.x,sScroll.y,bRedraw);}
inline	BOOL			CBitmapWnd::CreateEx(DWORD dwExStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID){return CreateEx(dwExStyle,WS_CHILD|WS_VISIBLE,sRect,pParentWnd,nID);}
inline	BOOL			CBitmapWnd::CreateEx(DWORD dwExStyle,unsigned	   uId,CDialog *pDlg   ,UINT nID){return CreateEx(dwExStyle,WS_CHILD|WS_VISIBLE,uId  ,pDlg      ,nID);}
inline	BOOL			CBitmapWnd::Create  (DWORD dwStyle  ,const RECT &sRect,CWnd *pParentWnd,UINT nID){return CreateEx(0        ,dwStyle            ,sRect,pParentWnd,nID);}
inline	BOOL			CBitmapWnd::Create  (DWORD dwStyle  ,unsigned	   uId,CDialog *pDlg   ,UINT nID){return CreateEx(0        ,dwStyle            ,uId  ,pDlg      ,nID);}
inline	BOOL			CBitmapWnd::Create  (                const RECT &sRect,CWnd *pParentWnd,UINT nID){return CreateEx(0        ,WS_CHILD|WS_VISIBLE,sRect,pParentWnd,nID);}
inline	BOOL			CBitmapWnd::Create  (                unsigned	   uId,CDialog *pDlg   ,UINT nID){return CreateEx(0        ,WS_CHILD|WS_VISIBLE,uId  ,pDlg      ,nID);}
inline	BOOL			CBitmapWnd::IsRgb(){return m_iBits>8;}
inline	BOOL			CBitmapWnd::PtInClientRect(LPPOINT pPoint){unsigned	uPosX;unsigned uPosY;uPosX=pPoint->x-m_iScrollX;if(uPosX>=(unsigned)m_iWndSizeX)return FALSE;uPosY=pPoint->y-m_iScrollY;if(uPosY>=(unsigned)m_iWndSizeY)return FALSE;return TRUE;}
inline	BOOL			CBitmapWnd::PtInClientRect(  POINT sPoint){unsigned	uPosX;unsigned uPosY;uPosX=sPoint .x-m_iScrollX;if(uPosX>=(unsigned)m_iWndSizeX)return FALSE;uPosY=sPoint .y-m_iScrollY;if(uPosY>=(unsigned)m_iWndSizeY)return FALSE;return TRUE;}
inline	BOOL			CBitmapWnd::PtInClientRect(UINT uPosX,UINT uPosY){uPosX-=m_iScrollX;if(uPosX>=(unsigned)m_iWndSizeX)return FALSE;uPosY-=m_iScrollY;if(uPosY>=(unsigned)m_iWndSizeY)return FALSE;return TRUE;}
inline	BOOL			CBitmapWnd::PtInImageRect (LPPOINT pPoint){if((unsigned)pPoint->x>=m_pImage->Width ())return FALSE;if((unsigned)pPoint->y>=m_pImage->Height())return FALSE;return TRUE;}
inline	BOOL			CBitmapWnd::PtInImageRect (  POINT sPoint){if((unsigned)sPoint .x>=m_pImage->Width ())return FALSE;if((unsigned)sPoint .y>=m_pImage->Height())return FALSE;return TRUE;}
inline	BOOL			CBitmapWnd::PtInImageRect (UINT uPosX,UINT uPosY){if(uPosX>=m_pImage->Width ())return FALSE;if(uPosY>=m_pImage->Height())return FALSE;return TRUE;}
inline	void			CBitmapWnd::ScreenToClient(LPPOINT pPoint){pPoint->x+=m_iScrollX;pPoint->y+=m_iScrollY;CWnd::ScreenToClient(pPoint);}
inline	void 			CBitmapWnd::ClientToScreen(LPPOINT pPoint){pPoint->x-=m_iScrollX;pPoint->y-=m_iScrollY;CWnd::ClientToScreen(pPoint);}
inline	void			CBitmapWnd::GetScrollPos  (int &iScrollX,int &iScrollY){iScrollX=m_iScrollX;iScrollY=m_iScrollY;}
inline	void			CBitmapWnd::GetScrollPos  (CPoint &sScroll){sScroll.x=m_iScrollX;sScroll.y=m_iScrollY;}
inline	void			CBitmapWnd::InvalidateFrame(LPCRECT pRect, BOOL bErase){CWnd::InvalidateRect(pRect,bErase);}
inline	BOOL			CBitmapWnd::EnsureVisible(CRect  &sRect ){return EnsureVisible(sRect.top,sRect.left,sRect.Width(),sRect.Height());}
inline	BOOL			CBitmapWnd::EnsureVisible(CPoint &sPoint){return EnsureVisible(sPoint.x,sPoint.y,0,0);}

#endif
