//*****************************************************************************
//*
//*
//*		ColorComboWnd.cpp
//*
//*
//*****************************************************************************
//
//	Ist einen Win-API-ComboBox-Fenster das Farben und dicke Schrift zu lst.
//
//	Fenster Styles:
//	
//		CBS_SIMPLE          	0x0000	Auswahlliste ist immer sichtbar
//		CBS_DROPDOWN        	0x0002	ComboBox mit Eingabefeld
//		CBS_DROPDOWNLIST    	0x0003	ComboBox ohne Eingabefeld
//		CBS_OWNERDRAWFIXED		0x0010	--- nicht untersttzt ---
//		CBS_OWNERDRAWVARIABLE   0x0020	--- nicht untersttzt ---
//		CBS_AUTOHSCROLL        	0x0040	--- nicht untersttzt ---
//		CBS_OEMCONVERT         	0x0080	--- nicht untersttzt ---
//		CBS_SORT		      	0x0100	Die Eintrge in der Listbox werden sortiert
//		CBS_HASSTRINGS      	0x0200	--- nicht untersttzt ---
//		CBS_WANTRETURN			0x0200	Die Tasten RET/TAB/ESC via Notify melden
//		CBS_NOINTEGRALHEIGHT   	0x0400	Die Listbox kann partielle Elemente anzeigen
//		CBS_DISABLENOSCROLL    	0x0800	Keien Y-Scrollbar in der Listbox
//		CBS_UPPERCASE       	0x1000	--- nicht untersttzt ---
//		CBS_LOWERCASE          	0x2000	--- nicht untersttzt ---
//		CBS_ALLOWDROP			0x4000	Dateien knnen auf das Control fallengelassen werden
//		CBS_NOGRAYDISABLE		0x8000	Fenster nicht grau machen wenn es gesperrt ist
//		CBS_ONLYLISTPAINT     WS_GROUP	Nur die Listbox farbig zeichnen
//	
//	Beispiel:
//
//		ColorComboProc(hWnd,WM_ATTACH,0,0);					// Eingngen
//		SendMessage   (hWnd,CB_SETIMAGELIST    ,0,(LPARAM)GetIconList());
//		SendMessage   (hWnd,CB_SETIMAGE        ,-1,GetIconNr("Test.cpp",SHGFI_USEFILEATTRIBUTES));
//		SendMessage   (hWnd,CB_SETCOLOR        ,MAKELONG(16,-1),(LPARAM)"\x42\x42\x42\x42\0\3\3\3\3\3\0\x20\x20\x20\x20\x20");
//		SendMessage   (hWnd,CB_SETDROPPEDHEIGHT,-15,0);
//		SendMessage   (hWnd,CB_SETTABSTOP      ,60,0);
//
//		ColorEditProc(hWnd,WM_DETACH,0,0);					// Aushngen
//	
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#ifdef		UNICODE
#ifndef	   _UNICODE
#define    _UNICODE
#endif
#endif

#include	<windows.h>
#include	<commctrl.h>
#include	<assert.h>
#include	<string.h>
#include	<stdlib.h>
#include	<stddef.h>
#include	<stdio.h>
#include	<tchar.h>

#include	"WinExtension.h"
#include	"ColorEditWnd.h"
#include	"ColorComboWnd.h"
#include	"StringFunctions.h"

#ifdef	   _MSC_VER
#pragma		warning( disable :4996 )
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	TEXT_MAX			1024
#define 	ID_SELDRAW			0x4321
#define 	ASSERT				assert
#define 	ENTRIES(a)			(sizeof(a)/sizeof((a)[0]))

#ifndef 	CB_GETCOMBOBOXINFO
#define		CB_GETCOMBOBOXINFO	0x0164
#endif 	

typedef struct  
	{
	DWORD		cbSize;									// Gre der Struktur
	RECT 		rcItem;									// Rechteck mit dem Text
	RECT 		rcButton;								// Rechteck mit der Taste
	DWORD		stateButton;							// Zustand der Taste
	HWND 		hwndCombo;								// ComboHox Fenster-Handle
	HWND 		hwndItem;								// Edit     Fenster-Handle
	HWND 		hwndList;								// Listbox  Fenster-Handle
	}ComboInfo;

typedef struct  
	{
	INT			iIcon;									// Nummer des Icons
	WORD		wSizeMax;								// Maximale Lnge der Farbdaten 
	WORD		wSizeLen;								// Aktuelle Lnge der Farbdaten 
	BYTE		aColor[28];								// Farbdaten fr den Text
	}ComboItem;

typedef struct
	{
	WNDPROC		pOldProc;								// Vorherige Fensterfunktion
	WNDPROC		pOldList;								// Vorherige Fensterfunktion (Listbox)
	LPVOID		pCbParam;								// User-Parameter fr Callback
   ColorComboCb	pCallback;								// Callback fr Farben
	LPCOLORREF	pColTable;								// Farbtabele fr ExtTextColor() (0=default)
	RECT		sTextRect;								// Rechteck fr den Text
	RECT		sDropRect;								// Rechteck fr die Taste
	HIMAGELIST	hImgages;								// Handle der Image-Liste
	LPBYTE		pColors;								// Farben der Buchstaben	
	HWND		hEditWnd;								// Handle vom Edit-Fenster
	HWND		hListWnd;								// Handle vom Listbox-Fenster
	HWND		hComboWnd;								// Handle vom Combo-Fenster
	HRGN		hHrgnBack;								// Handle fr die Hintergrungregion
	HFONT		aFont[2];								// Fonts fr das Zeichnen ([1]=bold)
	LPARAM		lUserId;								// User ID fr Callback
	UINT		uStyle;									// Style des Fensters
	DWORD		uOption;								// Bits mit den Farboptionen fr ExtTextColor()
	WORD		wTabStop;								// Ist der Tabstop in Pixel
	short		sWindowX;								// Ist die Fenstergre
	short		sWindowY;								//			*
	short		sDropY;									// Dropdown-Hhe (0=aus <0=Zeile >0=Pixel)
	bool		bIsHot;									// Ist die Maus ber dem Button
	bool		bEnable;								// Ist das Fenster freigegeben
	bool		bHasInfo;								// Hat das Fenster CB_GETCOMBOBOXINFO
	bool		bTimerOn;								// Ist das Scroll-Timer aktiv
	bool		bHasFocus;								// Hat das Fenster den Fokus
	bool		bLockDraw;								// Ist das Zeichnen gesperrt
	BYTE		bFontHeigh;								// Hhe der Schrift
	BYTE		bIconX;									// Breite der Icons
	BYTE		bIconY;									// Hhe der Icons
	WORD		wLastKey;								// Letzte weitergeleitete Taste
	INT			iSelLast;								// Letzter ausgewhlter Listeneintrag
	INT			iEntryCnt;								// Anzahl der Eintrge
	INT			iEntryMax;								// Maximale Anzahl der Eintrge
	ComboItem **pEntries;								// Liste mit Farbdaten und Icons
	}ColorComboData;


static	WNDPROC				pOldComboProc	= 0;		// Fensterfunktion von Edit
static	ATOM				wColorComboAtom	= 0;		// ID-Fr die Fenster-Extra-Daten
static	CRITICAL_SECTION	sComboLock;					// Semaphore fr den Zugriff

#define COMBO_LOCK(d)		EnterCriticalSection(&sComboLock);
#define COMBO_UNLOCK(d)		LeaveCriticalSection(&sComboLock);



//*****************************************************************************
//*
//*		ColorComboEnum
//*
//*****************************************************************************
//	Sucht nach allen Kinderfenstern der ComboBox.
static BOOL CALLBACK ColorComboEnum(HWND hWnd,LPARAM lParam)
{
ColorComboData *pData=(ColorComboData*)lParam;	
char			cName[64];


	
	GetClassNameA(hWnd,cName,sizeof(cName));

	if(!stricmp(cName,"EDIT"))
		{
		pData->hEditWnd = hWnd;
		}

	if(!stricmp(cName,"LISTBOX"))
		{
		pData->hListWnd = hWnd;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		ColorComboUpdateRegion
//*
//*****************************************************************************
//	Erzeugt eine Region nur fr den Hintergrund.
static void ColorComboUpdateRegion(ColorComboData *pData)
{
HRGN	hRect;
HRGN	hText;
HRGN	hIcon;


	hRect	= CreateRectRgn(0,0,pData->sWindowX,pData->sWindowY);
	hText	= CreateRectRgnIndirect(&pData->sTextRect);
	
	CombineRgn(hRect,hRect,hText,RGN_DIFF);

	if(pData->bIconX)
		{
		hIcon = CreateRectRgn(0,0,pData->bIconX+2,pData->sWindowY);
		CombineRgn(hRect,hRect,hText,RGN_OR);
		DeleteObject(hIcon);
		}

	if(pData->hHrgnBack)
		{
		DeleteObject(pData->hHrgnBack);
		}

	pData->hHrgnBack = hRect;
	DeleteObject(hText);
	
}

//*****************************************************************************
//*
//*		ColorComboCalcRect
//*
//*****************************************************************************
//	Die Gren der einzelnen Rechtecke in der ComboBox berechnen.
static void ColorComboCalcRect(ColorComboData *pData)
{
int		iSize;

 
	if((pData->uOption&0x0F)==CBS_SIMPLE)
		{
		pData->sTextRect.left	= 1;
		pData->sTextRect.top	= 1;
		pData->sTextRect.right	= pData->sWindowX-1;
		pData->sTextRect.bottom	= pData->sWindowY-1;
		pData->sDropRect.left	= 0;
		pData->sDropRect.top	= 0;
		pData->sDropRect.right	= 0;
		pData->sDropRect.bottom	= 0;
		}
	else{
		iSize					= GetSystemMetrics(SM_CYHSCROLL);
		pData->sTextRect.left	= 3;
		pData->sTextRect.top	= 3;
		pData->sTextRect.right	= pData->sWindowX-1-iSize-4;
		pData->sTextRect.bottom	= pData->sWindowY-3;
		pData->sDropRect.left	= pData->sWindowX-iSize;
		pData->sDropRect.top	= 0;
		pData->sDropRect.right	= pData->sWindowX;
		pData->sDropRect.bottom	= pData->sWindowY;
		}


}

//*****************************************************************************
//*
//*		ColorComboInsert
//*
//*****************************************************************************
//	Einfgen eines neuen Eintrages.
//	pData		: Zeiger auf die Daten der ComboBox
//	iPos		: Ist die Einfgeposition (-1 um am Ende anhngen)
//	iSize		: Ist die Textgre in Zeichen
//	pColors		: Zeiger auf die Farbdaten oder NULL.
//	Ergibt die Einfgeposition
static int ColorComboInsert(ColorComboData *pData,int iPos,int iSize,const LPBYTE pColors=NULL)
{
ComboItem **pList;
ComboItem  *pItem;
BYTE		uByte;
BYTE		uFlag;
unsigned	uMax;
int			iOff;

	
	if(iPos<0)											// Die Position prfen
		{
		if(iPos!=-1)return -1;
		iPos = pData->iEntryCnt;
		}
	else if(iPos>pData->iEntryCnt)
		{
		iPos = pData->iEntryCnt;
		}


	COMBO_LOCK(pData);
	
	if(iPos>=pData->iEntryMax)							// Die Liste vergrern
		{
		uMax	= pData->iEntryMax+pData->iEntryMax/2+16;
		pList	= new ComboItem*[uMax];

		memcpy(pList,pData->pEntries,pData->iEntryMax*sizeof(ComboItem*));
		pData->iEntryMax = uMax;

		delete pData->pEntries;
		pData->pEntries = pList;
		}


	pList = pData->pEntries;


	if(iPos<pData->iEntryCnt)							// Oberen Teil der Liste verschieben
		{
		memmove(pList+iPos+1,pList+iPos,(pData->iEntryCnt-iPos)*sizeof(ComboItem*));
		}

	
	if(iSize>TEXT_MAX)iSize=TEXT_MAX;
	if(iSize<0       )iSize=0;

	uMax			= (iSize+15)&~0x0F;					// Neuen Eintrag erstellen
	pItem			= (ComboItem*)new char[uMax+offsetof(ComboItem,aColor)];
	pList[iPos]		= pItem;
	pItem->iIcon	= -1;
	pItem->wSizeMax	= (WORD)uMax;
	pData->iEntryCnt++;

	if(pColors)											// Farben kopieren
		{						  
		for(iOff=0,uFlag=0;iOff<iSize;iOff++)
			{
			uByte	= pColors[iOff];
			uFlag  |= uByte;
			pItem->aColor[iOff] = uByte;
			}
		
		pItem->wSizeLen	= iSize;
		iSize = 0;
		}
	else{
		pItem->wSizeLen	= 0;
		}

	memset(pItem->aColor+iSize,0,uMax-iSize);			// Rest der Farben auf 0 setzen


	COMBO_UNLOCK(pData);

	if(!pData->aFont[0])
		{
		pData->aFont[0]		= GetDefaultFont();
		pData->bFontHeigh	= GetFontHeight(NULL,pData->aFont[0]);
		}

	if((uFlag&0x40) && !pData->aFont[1])
		{
		pData->aFont[1]		= GetBoldFont(pData->aFont[0]);
		}


return iPos;
}

//*****************************************************************************
//*
//*		ColorComboDelete
//*
//*****************************************************************************
//	Von Eintrgen aus der Combobox
//	pData		: Zeiger auf die Daten der ComboBox
//	iPos		: Ist die Einfgeposition (-1 um am Ende anhngen)
//	iCount		: Ist die Anzahl der  zu lschenden Eintrge
//	Ergibt die Anzahl der gelschten Eintrge.
static int ColorComboDelete(ColorComboData *pData,int iPos,int iCount)
{
ComboItem **pList;
int			iOff;
int			iEnd;


	if(iPos>=pData->iEntryCnt || iPos<0 || iCount<=0)
		{
		return 0;
		}

	COMBO_LOCK(pData);

	if(iPos+iCount>pData->iEntryCnt)					// Bereich prfen
		{
		iCount = pData->iEntryCnt-iPos;
		}

	pList = pData->pEntries;
	iEnd  = iPos+iCount;

	for(iOff=iEnd;iOff>iPos;)							// Eintrge freigeben
		{
		iOff--;
		delete pList[iOff];
		}

	if(iEnd<pData->iEntryCnt)							// Oberen Teil der Liste verschieben
		{
		memmove(pList+iPos,pList+iEnd,(pData->iEntryCnt-iEnd)*sizeof(ComboItem*));
		}

	pData->iEntryCnt -= iCount;
	
	COMBO_UNLOCK(pData);


return iCount;
}

//*****************************************************************************
//*
//*		ColorComboListPaint
//*
//*****************************************************************************
//	Zeichnet das List-Fenster
static void ColorComboListPaint(ColorComboData *pData,HWND hWnd,WPARAM wParam,LPARAM lParam)
{
TCHAR			cText[TEXT_MAX];
LPBYTE			pColor;
COLORREF		uColor;
POINT			sPoint;
PAINTSTRUCT		sPaint;
HRGN			hRrgn;
ComboItem	   *pItem;
RECT			sRect;
int				iSize;
int				iBase;
int				iPosY;
int				iPosX;
int				iTop;
int				iCnt;
int				iEnd;
int				iMax;
int				iSel;
int				iLen;
int				iOut;
HDC				hDc;



	GetCursorPos  (&sPoint);
	ScreenToClient(hWnd,&sPoint);

	
	GetClientRect(hWnd,&sRect);

	iSize	= CallWindowProc(pData->pOldList,hWnd,LB_GETITEMHEIGHT,0,0);
	iTop	= CallWindowProc(pData->pOldList,hWnd,LB_GETTOPINDEX  ,0,0);
	iCnt	= CallWindowProc(pData->pOldList,hWnd,LB_GETCOUNT     ,0,0);
	iSel	= CallWindowProc(pData->pOldList,hWnd,LB_ITEMFROMPOINT,0,MAKELPARAM(sPoint.x,sPoint.y));
	iEnd	= iTop+iCnt;
	iMax	= sRect.bottom;
	iBase	= sRect.left;
	

	if(lParam)
		{
		hDc	= (HDC)lParam;
		}
	else{
		hDc	= BeginPaint(hWnd,&sPaint);
		}


	if(sPaint.rcPaint.left<sPaint.rcPaint.right)		// In welchembereich darf gezeichnet werden
		{
		hRrgn = CreateRectRgn(sPaint.rcPaint.left,sPaint.rcPaint.top,sPaint.rcPaint.right,sPaint.rcPaint.bottom);
		}
	else{
		hRrgn = CreateRectRgn(sRect.left,sRect.top,sRect.right,sRect.bottom);
		}

	while(sRect.top<iMax)
		{
		if(iTop>=iEnd)break;
														// Nur Auswahlnderung zeichnen	
		if(wParam==0x12345678 && iTop!=iSel && iTop!=pData->iSelLast)
			{
			sRect.top += iSize;	
			iTop	  += 1;
			continue;				
			}
		else if((wParam&0xFFFF0000)==0x23450000 && (WORD)wParam!=iTop)
			{
			sRect.top += iSize;	
			iTop	  += 1;
			continue;				
			}
														// Kann ein Text ausgeben werden
		sRect.bottom = sRect.top+iSize;
		pItem		 = pData->pEntries[iTop];			

		ASSERT(pItem->wSizeMax>=iLen);

		   iLen = (int)CallWindowProc(pData->pOldList,hWnd,LB_GETTEXT,iTop,(LPARAM)cText);
		if(iLen<0)iLen=0;

		if(pData->pCallback && !pItem->wSizeLen)		// Callback fr Farben
			{
			pData->pCallback(pData->pCbParam,cText,iLen,pItem->aColor,&pItem->iIcon,pData->lUserId,iTop);
			pItem->wSizeLen	= (WORD)iLen;
			}
		
		if(iSel==iTop)
			{
			uColor	= (GetSysColor(COLOR_HIGHLIGHT)>>1)&0x007F7F7F;
			uColor += (GetSysColor(COLOR_WINDOW   )>>1)&0x007F7F7F;
			}
		else{
			uColor  =  GetSysColor(COLOR_WINDOW);
			}

		SetBkColor(hDc,uColor);
														
		iPosX	= sRect.left+3;	if(pData->bIconX)iPosX+=pData->bIconX+2;
		iPosY	=(sRect.bottom+sRect.top-pData->bFontHeigh)/2;
		pColor	=(pItem->wSizeLen)? pItem->aColor:NULL;

		if(iPosY<sRect.top)iPosY=sRect.top;
														// Text ausgeben
		iOut	= strchar(cText,'\t',iLen);

		if(iOut>=iLen || !pData->wTabStop)
			{
			ExtTextColor(hDc,iPosX,iPosY,pData->uOption,&sRect,cText,iLen,pColor,pData->pColTable,pData->aFont);
			}
		else{											// Mit Tabulatoren ausgeben
			COLORREF	uOldC;					
			SIZE		sSize;
			int			iBase;
			int			iFrom;
			int			iTab;
			int			iOff;

			iEnd	= sRect.right;
			iFrom	= sRect.left;
			iBase	= iPosX;

			for(iOff=0;iOff<iLen;)
				{
				if(iOff+iOut<iLen && GetTextExtentEx(hDc,cText+iOff,iOut,iEnd,NULL,NULL,&sSize,pColor,pData->aFont))
						sRect.right = iPosX+sSize.cx;
				else	sRect.right = iEnd;

				   iPosX = ExtTextColor(hDc,iPosX,iPosY,pData->uOption,&sRect,cText+iOff,iOut,(pColor)? pColor+iOff:0,pData->pColTable,pData->aFont);
				if(iPosX>=iEnd){iOff=iLen;break;}

				iOff   += iOut;

				if(iOff>=iLen)break;
				
				iTab	= iPosX-iBase;
				iTab   += pData->wTabStop;
				iTab    = pData->wTabStop-iTab%pData->wTabStop;

				if(pColor && (pData->uOption&0x70000000))
					{
					uOldC = SelectBkColorEx(hDc,pData->uOption,pColor[iOff],pData->pColTable);
					}

				DrawRectangle(hDc,iPosX,sRect.top,iTab,sRect.bottom-sRect.top);

				if(pColor && (pData->uOption&0x70000000))
					{
					uOldC = SetBkColor(hDc,uOldC);
					}

				
				iPosX  += iTab;
				iOff   += 1;	
				iOut	= strchar(cText+iOff,'\t',iLen-iOff);

				sRect.left = iPosX;	
				}
			
			sRect.right = iEnd;
			sRect.left	= iFrom;
			}


		if(pData->bIconX && pItem->iIcon>=0)			// Ein Icon zeichnen	
			{											
			   iPosX =  sRect.left+1;
			   iPosY = (sRect.bottom+sRect.top-pData->bIconY+2)/2;
			if(iPosY<sRect.top)iPosY=sRect.top;	

			ImageList_Draw(pData->hImgages,pItem->iIcon,hDc,iPosX,iPosY,ILD_TRANSPARENT);
			}

		sRect.top   += iSize;	
		sRect.left	 = iBase;
		iTop++;
		}

	if(!lParam)EndPaint(hWnd,&sPaint);
	if(hRrgn  )DeleteObject(hRrgn);

	pData->iSelLast = iSel;

}

//*****************************************************************************
//*
//*		ColorComboList
//*
//*****************************************************************************
//	Ist die Fensterfunktion fr das ListBox Fenster
static LRESULT CALLBACK ColorComboList(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
ColorComboData *pData;
LPWINDOWPOS		pPos;
SCROLLINFO		sInfo;
static DWORD	uLast;
UINT			uDiff;
UINT			uTick;
RECT			sRect;
RECT			sArea;
int				iSize;
int				iTop;
int				iVal;
int				iSel;
int				iMax;
int				iNum;
HDC				hDc;

	
	pData = (ColorComboData*)GetWindowLongPtr(hWnd,GWL_USERDATA);
	
	switch(uMsg)
		{
	case WM_SHOWWINDOW:									// Das Fenster wird sichtbar

		if(!wParam || !pData->sDropY)break;

		GetWindowRect(pData->hComboWnd,&sArea);
		ColorComboList(hWnd,WM_SIZE,0,MAKELONG(sArea.right-sArea.left,sArea.bottom-sArea.top));
		
		break;


	case WM_SIZE:										// Die Fenstergre hat sich verndert

		SetTimer(pData->hListWnd,ID_SELDRAW,50,NULL);

		if(!pData->sDropY)break;

		iSize = CallWindowProc(pData->pOldList,hWnd,LB_GETITEMHEIGHT,0,0);if(iSize<10)iSize=10;
		iMax  = iSize*pData->iEntryCnt;
		iNum  = pData->sDropY;

		if(iNum<   0)iNum=iSize*-iNum;
		if(iNum>iMax)iNum=iMax;

		iSel = GetSystemMetrics(SM_CYSCREEN);
		iMax = (iSel*2)/3;
		iVal = lParam>>16;
		
		if(iNum>iMax   )iNum=iMax;
		if(iVal==iNum  )return 0;
		if(iVal>=iNum-2)
		if(iVal<=iNum+2)return 0;

		uTick	= GetTickCount();
		uDiff	= uTick-uLast;

		if(uDiff<100)return 0;

		uLast	= uTick;

		GetWindowRect(pData->hComboWnd,&sArea);
		GetWindowRect(hWnd,&sRect);
														// Listbox unter oder ober Combo-Fenster
		if(sRect.top>sArea.top || sArea.bottom+iNum+10<iSel)
			{
			sArea.top	= sArea.bottom;
			}
		else{
			sArea.top  -= iNum+2;
			}

		SetWindowPos(hWnd,NULL,sArea.left,sArea.top,sRect.right-sRect.left,iNum+2,SWP_NOZORDER);

		return 0;


	case WM_WINDOWPOSCHANGED:							// Einstellungen haben sich verndert

		   pPos=(LPWINDOWPOS)lParam;
		if(pPos->flags&SWP_SHOWWINDOW)
			{
			if(pData->sDropY)ColorComboList(hWnd,WM_SHOWWINDOW,TRUE,SW_PARENTOPENING);
			PostMessage(hWnd,WM_PAINT,0x12344321,0);
			}

		break;

	case WM_SETCURSOR:									// Cursor-Position verndert
	case WM_NCMOUSEMOVE:
	case WM_MOUSEMOVE:

		iSel = CallWindowProc(pData->pOldList,hWnd,LB_ITEMFROMPOINT,0,lParam);

		if(pData->iSelLast==iSel)return 0;

			hDc = GetDCEx(hWnd,0,DCX_CACHE);
		if(!hDc)return 0;

		ColorComboListPaint(pData,hWnd,0x12345678,(LPARAM)hDc);
		ReleaseDC(hWnd,hDc);

		return 0;
	

	case WM_LBUTTONDOWN:								// Linke Maustaste gedrckt

		iVal = CallWindowProc(pData->pOldList,hWnd,LB_ITEMFROMPOINT,0,lParam);
		if(HIWORD(iVal))break;

		CallWindowProc(pData->pOldList,hWnd,LB_SETSEL,iVal,0);
		
		CallWindowProc(pData->pOldList,hWnd,WM_SETREDRAW,0,0);
		CallWindowProc(pData->pOldList,hWnd,uMsg,wParam,lParam);
		CallWindowProc(pData->pOldList,hWnd,WM_SETREDRAW,1,0);

		return 0;

	
	case WM_CHAR:										// Zeicheneingabe
	case WM_KEYUP:

		return 0;


	case WM_MOUSEWHEEL:									// Mausrad bettigt

		iVal  = (short)HIWORD(wParam);		
		iVal /= 40;

		if(!iVal)return 0;

		sInfo.cbSize	= sizeof(sInfo);
		sInfo.fMask		= SIF_ALL;

		GetScrollInfo(hWnd,SB_VERT,&sInfo);

		sInfo.nPos	   -= iVal;	

		goto Done;


	case WM_KEYDOWN:									// Taste gedrckt

		switch(wParam)
			{
		case VK_HOME:	wParam	= SB_TOP;		break;
		case VK_END:	wParam	= SB_BOTTOM;	break;
		case VK_UP:		wParam	= SB_LINEUP;	break;
		case VK_DOWN:	wParam	= SB_LINEDOWN;	break;
		case VK_PRIOR:	wParam	= SB_PAGEUP;	break;
		case VK_NEXT:	wParam	= SB_PAGEDOWN;	break;
		default:return 0;		
			}
		

	case WM_VSCROLL:									// Es wird gescrollt

		sInfo.cbSize	= sizeof(sInfo);
		sInfo.fMask		= SIF_ALL;

		GetScrollInfo(hWnd,SB_VERT,&sInfo);
		switch(LOWORD(wParam))
			{		
		case SB_TOP:			sInfo.nPos  = 0;				break;
		case SB_BOTTOM:			sInfo.nPos  = sInfo.nMax;		break;
		case SB_LINEDOWN:		sInfo.nPos += 1;				break;
		case SB_LINEUP:			sInfo.nPos -= 1;				break;
		case SB_PAGEDOWN:		sInfo.nPos += sInfo.nPage;		break;
		case SB_PAGEUP:			sInfo.nPos -= sInfo.nPage;		break;
		case SB_THUMBPOSITION:	sInfo.nPos	= HIWORD(wParam);	break;
		case SB_THUMBTRACK: 	sInfo.nPos	= HIWORD(wParam);	break;
		default:	return 0;
			}

	Done:

		if(sInfo.nPos>sInfo.nMax)sInfo.nPos=sInfo.nMax;
		if(sInfo.nPos<sInfo.nMin)sInfo.nPos=sInfo.nMin;
		
		iTop = CallWindowProc(pData->pOldList,hWnd,LB_GETTOPINDEX,0,0);

		if(sInfo.nPos==iTop)return 0;

		CallWindowProc(pData->pOldList,hWnd,WM_SETREDRAW,0,0);
		CallWindowProc(pData->pOldList,hWnd,LB_SETTOPINDEX,sInfo.nPos,0);
		SetScrollPos(hWnd,SB_VERT,sInfo.nPos,TRUE);

		   hDc = GetDCEx(hWnd,0,DCX_CACHE);
		if(hDc)
			{
			ColorComboListPaint(pData,hWnd,0,(LPARAM)hDc);
			ReleaseDC(hWnd,hDc);
			}

		CallWindowProc(pData->pOldList,hWnd,WM_SETREDRAW,1,0);

		return 0;



		   hDc = GetDCEx(hWnd,0,DCX_CACHE);
		if(hDc)
			{
			iSel = CallWindowProc(pData->pOldList,hWnd,LB_GETTOPINDEX,0,0);
			ColorComboListPaint(pData,hWnd,iSel|0x23450000,(LPARAM)hDc);
			ReleaseDC(hWnd,hDc);
			}

		return 0;


	case WM_TIMER:										// Auswahl neuzeichen

		if(wParam!=ID_SELDRAW)break;

		if(IsWindowVisible(hWnd)==0)
			{
			KillTimer(hWnd,ID_SELDRAW);
			return 0;
			}

		if(GetAsyncKeyState(VK_LBUTTON)&0x8000)			// Erst zeichen wenn Maustatste nicht gedrckt ist	
			{
			return 0;
			}

		KillTimer(hWnd,ID_SELDRAW);

		   hDc = GetDCEx(hWnd,0,DCX_CACHE);
		if(hDc)
			{
			iSel = CallWindowProc(pData->pOldList,hWnd,LB_GETTOPINDEX,0,0);
			ColorComboListPaint(pData,hWnd,iSel|0x23450000,(LPARAM)hDc);
			ReleaseDC(hWnd,hDc);
			}

		return 0;


	case WM_PAINT:										// Das Fenster zeichnen


		if(wParam==0x12344321)
			{
			lParam = (LPARAM)GetDCEx(hWnd,0,DCX_CACHE);
			}

		ColorComboListPaint(pData,hWnd,wParam,lParam);

		if(wParam==0x12344321 && lParam)
			{
			ReleaseDC(hWnd,(HDC)lParam);
			}

		return 0;
		}



return CallWindowProc(pData->pOldList,hWnd,uMsg,wParam,lParam);
}

//*****************************************************************************
//*
//*		ColorComboProc
//*
//*****************************************************************************
//	Ist die Fensterfunktion fr das ComboBox Fenster
LRESULT CALLBACK ColorComboProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
LPNMHDR			pNotify;
POINT			sPoint;
PAINTSTRUCT		sPaint;
LPBYTE			pColor;
LPCTSTR			pText;
ColorComboData *pData;	
ComboItem	   *pItem;
WNDPROC			pProc;
LPRECT			pRect;
HRGN			hRrgn;
RECT			sRect;
BYTE			bDiff;
BYTE			bByte;
UINT			uSize;
UINT			uFlag;
UINT			uMode;
UINT			uBits;
INT				iPosX;
INT				iPosY;
INT				iMax;
INT				iOff;
INT				iSel;
INT				iEnd;
INT				iOut;
HDC				hDc;



		pData = (ColorComboData*)GetPropW(hWnd,(LPCWSTR)wColorComboAtom);
	if(!pData && uMsg!=WM_NCDESTROY && uMsg!=WM_DETACH)		// Keine Daten vorhanden
		{
		ComboInfo	sInfo;

			pData = new ColorComboData;
		if(!pData)return 0;

		if(!wColorComboAtom)								// Globale Initalisierung
			{
			WNDCLASS	sClass;
			InitializeCriticalSection(&sComboLock);
			GetClassInfo(NULL,_T("COMBOBOX"),&sClass);
			pOldComboProc   = sClass.lpfnWndProc;
			wColorComboAtom = GlobalAddAtomW(L"CeData");
			}

		ASSERT(IsWindow(hWnd));
		memset(pData,0,sizeof(ColorComboData));

		#ifdef _DEBUG
		char 	cClass[32];
		GetClassNameA(hWnd,cClass,sizeof(cClass));
		ASSERT(!strcmp(cClass,"ComboBox"));
		#endif 	

		   pData->pOldProc	= (WNDPROC)GetWindowLongPtr(hWnd,GWL_WNDPROC);
		if(pData->pOldProc==ColorComboProc)
			{
			pData->pOldProc = pOldComboProc;
			}
		
		if(uMsg==WM_ATTACH)									// Eine andere alte Fensterfunktion bernehmen
			{
			if(lParam)
					pData->pOldProc = (WNDPROC)lParam;
			else	SetWindowLongPtr(hWnd,GWL_WNDPROC,(LPARAM)ColorComboProc);	
			}

		GetClientRect(hWnd,&sRect);
		
		pData->uOption		= ETO_OPAQUE|ETO_FONTCNT_2;
		pData->uStyle		= GetWindowLong(hWnd,GWL_STYLE);
		pData->aFont[0]		= (HFONT)CallWindowProc(pData->pOldProc,hWnd,WM_GETFONT,0,0);
		pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);
		pData->bEnable		= IsWindowEnabled(hWnd)!=0;
		pData->bHasFocus	= GetFocus()==hWnd;
		pData->sWindowX		= (short)sRect.right;
		pData->sWindowY		= (short)sRect.bottom;
		pData->hComboWnd	= hWnd;
		pData->wTabStop		= 32;
		pData->lUserId		= -1;

		sInfo.cbSize		= sizeof(sInfo);
															// Infos abfragen
		if(CallWindowProc(pData->pOldProc,hWnd,CB_GETCOMBOBOXINFO,0,(LPARAM)&sInfo))
			{
			pData->bHasInfo	= TRUE;
			pData->hEditWnd	=(sInfo.hwndItem!=hWnd)? sInfo.hwndItem:NULL;
			pData->hListWnd	= sInfo.hwndList;
			pData->sTextRect= sInfo.rcItem;
			pData->sDropRect= sInfo.rcButton;
			InflateRect(&pData->sDropRect,1,1);
			}
		else{
			EnumChildWindows(hWnd,ColorComboEnum,(LPARAM)pData);
			}

		ASSERT(pData->hEditWnd!=NULL || (pData->uStyle&0x0F)==CBS_DROPDOWNLIST);

		iMax = CallWindowProc(pData->pOldProc,hWnd,CB_GETCOUNT,0,0);
		
		for(iOff=0;iOff<iMax;iOff++)						// Farbspeiche initialisieren
			{
			iEnd = CallWindowProc(pData->pOldProc,hWnd,CB_GETLBTEXTLEN,iOff,0);
			ColorComboInsert(pData,iOff,iEnd);
			ASSERT(iEnd<TEXT_MAX);
			}


		ASSERT((pData->uStyle&CBS_ALLOWDROP        )==0 || (GetWindowLong(hWnd,GWL_EXSTYLE)&WS_EX_ACCEPTFILES));
		ASSERT((pData->uStyle&CBS_DROPDOWNLIST     )!=0);
		ASSERT((pData->uStyle&CBS_OWNERDRAWFIXED   )==0);
		ASSERT((pData->uStyle&CBS_OWNERDRAWVARIABLE)==0);

		ColorComboUpdateRegion(pData);
		SetPropW(hWnd,(LPCWSTR)wColorComboAtom,pData);
		
		if(pData->hEditWnd)									// Ins Edit-Fenster einhngen
			{
			ColorEditProc(pData->hEditWnd,WM_ATTACH,0,0);
			}

		if(pData->hListWnd)									// Ins Listbox-Fenster einhngen
			{
			pData->pOldList = (WNDPROC)GetWindowLongPtr(pData->hListWnd,GWL_WNDPROC);
			SetWindowLongPtr(pData->hListWnd,GWL_USERDATA,(LPARAM)pData);
			SetWindowLongPtr(pData->hListWnd,GWL_WNDPROC ,(LPARAM)ColorComboList);
			}
		} 


	if(uMsg>=WM_KEYFIRST && uMsg<=WM_KEYLAST)				//	TAB/RET/ESC an Eltern-Fenster weiterleiten
	if(pData && pData->uStyle&CBS_WANTRETURN)						
		{
		if(uMsg!=WM_KEYDOWN && pData->wLastKey==wParam)
			{
			return 0;
			}

		if(wParam==VK_TAB || wParam==VK_RETURN || wParam==VK_ESCAPE)
			{
			NMKEY		sHeader;
			LRESULT		lRet;

			sHeader.hdr.hwndFrom	= hWnd;
			sHeader.hdr.code		= NM_KEYDOWN;
			sHeader.hdr.idFrom		= GetWindowLong(hWnd,GWL_ID);
			sHeader.nVKey			= (UINT)wParam;
			sHeader.uFlags			= (UINT)lParam;

			   lRet = SendMessage(GetParent(hWnd),WM_NOTIFY,sHeader.hdr.idFrom,(LPARAM)&sHeader);
			if(lRet)
				{
				pData->wLastKey = (WORD)wParam;
				return 0;
				}

			pData->wLastKey = 0;
			}
		}

	
	switch(uMsg)
		{													// Fenster anhngen 
	case WM_ATTACH:
		
		ASSERT(IsWindow(hWnd));
		
		return TRUE;


	case WM_DETACH:											// Fenster abhngen 

		if(!lParam)
			{
			SetWindowLongPtr(hWnd,GWL_WNDPROC,(LPARAM)pData->pOldProc);
			}

		RemovePropW(hWnd,(LPCWSTR)wColorComboAtom);

		if(pData->hListWnd)
			{
			SetWindowLongPtr(pData->hListWnd,GWL_WNDPROC,(LPARAM)pData->pOldList);
			}

		if(pData->hEditWnd)
			{
			ColorEditProc(pData->hEditWnd,WM_DETACH,0,0);
			}

		pProc = pData->pOldProc;

		ColorComboDelete(pData,0,pData->iEntryCnt);

		if(pData->hHrgnBack)
			{
			DeleteObject(pData->hHrgnBack);
			pData->hHrgnBack = NULL;
			}
		
		if(pData->pColTable)
			{
			delete pData->pColTable;
			}

		if(pData->pEntries)
			{
			delete pData->pEntries;
			}
		
		delete pData;
		
		return (LPARAM)pProc;


	case WM_NCDESTROY:										// Wird das Fenster zerstrt

		pProc = pData->pOldProc;
		ColorComboProc(hWnd,WM_DETACH,0,0);
		
		return CallWindowProc(pProc,hWnd,uMsg,wParam,lParam);


	case WM_NOTIFY:

		pNotify = (LPNMHDR)lParam;
															// Key-Down-Notify weiterleiten
		if(pNotify->code==NM_KEYDOWN && pNotify->hwndFrom==pData->hEditWnd)
			{
			LPNMKEY		pRecv=(LPNMKEY)lParam;
			NMKEY		sHeader;

			sHeader.hdr.hwndFrom	= hWnd;
			sHeader.hdr.code		= NM_KEYDOWN;
			sHeader.hdr.idFrom		= GetWindowLong(hWnd,GWL_ID);
			sHeader.nVKey			= pRecv->nVKey;
			sHeader.uFlags			= pRecv->uFlags;

			return SendMessage(GetParent(hWnd),WM_NOTIFY,sHeader.hdr.idFrom,(LPARAM)&sHeader);
			}	
		
		break;


	case WM_STYLECHANGED:									// Hat sich der Style verndert

		if(wParam==GWL_STYLE)
			{
			uBits = GetWindowLong(hWnd,GWL_STYLE);		  
			
			if(uBits&0x03)RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);

			pData->uStyle = uBits;

			ASSERT((uBits&CBS_ALLOWDROP        )==0 || (GetWindowLong(hWnd,GWL_EXSTYLE)&WS_EX_ACCEPTFILES));
			ASSERT((uBits&CBS_OWNERDRAWFIXED   )==0);
			ASSERT((uBits&CBS_OWNERDRAWVARIABLE)==0);
			}

		break;


	case WM_SETFONT:										// Neue Schriftart

		if(wParam==0)
			{
			wParam = (WPARAM)GetDefaultFont();
			}
		
		if(wParam && (HFONT)wParam!=pData->aFont[0])
			{
			pData->aFont[0]		= (HFONT)wParam;	
			pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);

			if(pData->aFont[1])
				{
				pData->aFont[1] = GetBoldFont(pData->aFont[0]);
				}
			}

		break;


	case WM_ERASEBKGND:										// Hintergrund lschen

		return 0;

	
	case WM_SETFOCUS:										// Fokus setzen

		if(!pData->bHasFocus)
			{
			pData->bHasFocus = true;
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}

		break;

	case WM_KILLFOCUS:										// Fokus geht verloren

		if(pData->bHasFocus)
			{
			pData->bHasFocus = false;
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}
		
		break;


	case WM_SIZE:											// Die Fensterge hat sich verndert

		pData->sWindowX	= LOWORD(lParam);  
		pData->sWindowY	= HIWORD(lParam);  

		if(pData->bHasInfo)
			{
			ComboInfo	sInfo;

			if(CallWindowProc(pData->pOldProc,hWnd,CB_GETCOMBOBOXINFO,0,(LPARAM)&sInfo))
				{				  
				pData->hEditWnd	=(sInfo.hwndItem!=hWnd)? sInfo.hwndItem:NULL;
				pData->hListWnd	= sInfo.hwndList;
				pData->sTextRect= sInfo.rcItem;
				pData->sDropRect= sInfo.rcButton;
				InflateRect(&pData->sDropRect,1,1);
				}
			}
		else{
			ColorComboCalcRect(pData);
			}

		ColorComboUpdateRegion(pData);

		break;


	case WM_TIMER:											// Timer Signal empfangen
	
		break;


	case WM_MOUSEWHEEL:										// Mausrad bettigt

		if(pData->hListWnd && IsWindowVisible(pData->hListWnd))
			{
			ColorComboList(pData->hListWnd,uMsg,wParam,lParam);
			return TRUE;
			}
		
		iSel = (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETCURSEL,0,0);
		iOff = (short)HIWORD(wParam);    
		
		if(iSel+1<pData->iEntryCnt && iOff<0)
			{
			CallWindowProc(pData->pOldProc,hWnd,WM_SETREDRAW,0,0);
			CallWindowProc(pData->pOldProc,hWnd,CB_SETCURSEL,iSel+1,0);
			CallWindowProc(pData->pOldProc,hWnd,WM_SETREDRAW,1,0);
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			return TRUE;
			}

		if(iSel>0 && iOff>0)
			{
			CallWindowProc(pData->pOldProc,hWnd,WM_SETREDRAW,0,0);
			CallWindowProc(pData->pOldProc,hWnd,CB_SETCURSEL,iSel-1,0);
			CallWindowProc(pData->pOldProc,hWnd,WM_SETREDRAW,1,0);
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			return TRUE;
			}

		return TRUE;


	case WM_MOUSELEAVE:										// Die Maus geht aus dem Fenster

		if(pData->bIsHot)
			{
			pData->bIsHot = false;
			InvalidateRect(hWnd,&pData->sDropRect,FALSE);
			}

		break;

	case WM_NCMOUSEMOVE:
	case WM_MOUSEMOVE:										// Die Maus bewegt sich

		sPoint.x = GET_X_LPARAM(lParam);
		sPoint.y = GET_Y_LPARAM(lParam);

		if(pData->bEnable && PtInRect(&pData->sDropRect,sPoint) && !CallWindowProc(pData->pOldProc,hWnd,CB_GETDROPPEDSTATE,0,0))
			{
			if(!pData->bIsHot)
				{
				TRACKMOUSEEVENT sEvent;

				pData->bIsHot = true;
				InvalidateRect(hWnd,&pData->sDropRect,FALSE);

				sEvent.cbSize		= sizeof(TRACKMOUSEEVENT);
				sEvent.dwFlags		= TME_LEAVE;
				sEvent.hwndTrack	= hWnd;

				_TrackMouseEvent(&sEvent);
				}
			else{
				return 0;
				}
			}
		else{
			if(pData->bIsHot)
				{
				pData->bIsHot = false;
				InvalidateRect(hWnd,&pData->sDropRect,FALSE);
				}
			}

		
		return 1;


	case WM_ENABLE:											// Wurde das Fenster freigegeben

		pData->bEnable		= wParam!=0;

		break;


	case WM_CTLCOLOREDIT:									// Farbgebung weiterleiten an ComboBox
		
		return SendMessage(GetParent(hWnd),uMsg,wParam,(LPARAM)hWnd);
		

	case WM_DROPFILES:										// Datei an Edit-Fenster weiter leiten

		if(!(pData->uStyle&CBS_ALLOWDROP))break;
		
		if(pData->hEditWnd)
			{
			SendMessage(pData->hEditWnd,EM_DROPFILES,wParam,lParam);
			}

		return 0;


	case WM_GETTEXT:										// Textauslesen weiterleiten
				
		   iOff = (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETCURSEL,0,0);
		if(iOff>=0)
			{
			LPTSTR	pMem =(LPTSTR)lParam;
			INT		iSize=(int)wParam;
			TCHAR	cText[TEXT_MAX];

			   iMax  = (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETLBTEXTLEN,iOff,0);
			if(iMax<iSize && iSize>0)
				{
				return CallWindowProc(pData->pOldProc,hWnd,CB_GETLBTEXT,iOff,lParam);
				}

			if(iMax>=TEXT_MAX || iMax<=0)					// Prfe Puffergre
				{
				if(iSize>0)pMem[0]=0;
				return 0;
				}

			   iMax  = (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETLBTEXT,iOff,(LPARAM)cText);
			if(iMax<0)iMax=0;
			
			memcpy(pMem,cText,(iMax+1)*sizeof(TCHAR));

			pMem[iSize-1] = 0;
			return wParam-1;
			}

		if(!pData->hEditWnd)								// Kein Edit-Fenster
			{
			LPTSTR	pMem =(LPTSTR)lParam;
			INT		iSize=(int)wParam;

			if(iSize>0)pMem[0]=0;
			
			return 0;
			}

		return SendMessage(pData->hEditWnd,uMsg,wParam,lParam);


	case WM_GETTEXTLENGTH:									// Textlnge zu Edit-Fenster weiterleiten
		
		   iOff = (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETCURSEL,0,0);
		if(iOff>=0)
			{
			return (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETLBTEXTLEN,iOff,0);
			}

		if(!pData->hEditWnd)
			{
			return 0;
			}

		return SendMessage(pData->hEditWnd,uMsg,wParam,lParam);
		

	case WM_SETREDRAW:										// Das Zeichnen sperren

		pData->bLockDraw = !wParam;
		break;


	case WM_NCPAINT:			

		return 0;


	case WM_PAINT:											// Zeichnen des Fensters 

		if(pData->bLockDraw)								
			{
			return 0;
			}
		
		if(pData->uStyle&CBS_ONLYLISTPAINT)
			{
			break;
			}

		if(!pData->aFont[0])
			{
			pData->aFont[0]		= GetDefaultFont();
			pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);
			}

		GetClientRect(hWnd,&sRect);
		pData->sWindowX = (short)sRect.right;

		hDc		= BeginPaint(hWnd,&sPaint);

		if(sPaint.rcPaint.left<sPaint.rcPaint.right)		// In welchembereich darf gezeichnet werden
			{
			hRrgn	= CreateRectRgn(sPaint.rcPaint.left,sPaint.rcPaint.top,sPaint.rcPaint.right,sPaint.rcPaint.bottom);
			}
		else{
			hRrgn	= CreateRectRgn(sRect.left,sRect.top,sRect.right,sRect.bottom);
			}
		

															// Fenster grau machen
		if(!pData->bEnable && !(pData->uStyle&CBS_NOGRAYDISABLE))
			{
			SetBkColor  (hDc,RGB(192,192,192));
			SendMessage(GetParent(hWnd),WM_CTLCOLORSTATIC,(WPARAM)hDc,(LPARAM)hWnd);
			uMode = (CP_BORDER<<16)|MPI_DISABLED;
			iOut  = ILD_TRANSPARENT|ILD_BLEND50;
			}
		else{
			SetBkColor  (hDc,GetSysColor(COLOR_WINDOW));
			SendMessage(GetParent(hWnd),WM_CTLCOLOREDIT,(WPARAM)hDc,(LPARAM)hWnd);
			iOut  = ILD_TRANSPARENT;

			if(!pData->bHasFocus)
					uMode = (CP_BORDER<<16)|MPI_NORMAL;
			else	uMode = (CP_BORDER<<16)|MPI_HOT;
			}
		

		SelectObject(hDc,pData->hHrgnBack);
		DrawThemeBack(hDc,1,&sRect,uMode,0);
		SelectObject(hDc,hRrgn);

		iSel = (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETCURSEL,0,0);

		ASSERT(pData->iEntryCnt==(int)CallWindowProc(pData->pOldProc,hWnd,CB_GETCOUNT,0,0));
		
															// Kann ein Text ausgeben werden
		if(iSel>=0 && iSel<pData->iEntryCnt && (pData->uStyle&0x0F)==CBS_DROPDOWNLIST)
			{
			TCHAR	cText[TEXT_MAX];
			int		iLen;

			pItem	= pData->pEntries[iSel];			

			   iLen = (int)CallWindowProc(pData->pOldProc,hWnd,CB_GETLBTEXT,iSel,(LPARAM)cText);
			if(iLen<0)iLen=0;

			ASSERT(iLen<TEXT_MAX);
			ASSERT(iLen<=pItem->wSizeMax);

			if(pData->pCallback && !pItem->wSizeLen)		// Callback fr Farben
				{
				pData->pCallback(pData->pCbParam,cText,iLen,pItem->aColor,&pItem->iIcon,pData->lUserId,iSel);
				pItem->wSizeLen	= (WORD)iLen;
				}

															
			if(pData->bIconX>=0)							// Ein Icon zeichnen	
				{											
				iEnd	=  sRect.left+pData->bIconX+2;
				iPosX	=  sRect.left+2;
				iPosY	= (sRect.bottom-sRect.top-pData->bIconY+1)/2;if(iPosY<0)iPosY=0;	

				if(iOut!=ILD_TRANSPARENT)
					{
					DrawRectangle(hDc,iPosX,iPosY,pData->bIconX,pData->bIconY);
					}
				
				if(pItem->iIcon>=0)
					{		
					ImageList_Draw(pData->hImgages,pItem->iIcon,hDc,iPosX,iPosY,iOut);
					}

				pRect		= &sRect;
				sRect		=  pData->sTextRect;
				sRect.left	=  iEnd;
				}
			else{
				sRect		=  pData->sTextRect;
				}

			iPosX	= sRect.left+3;
			iPosY	=(sRect.bottom+sRect.top-pData->bFontHeigh+2)/2;
			pColor	=(pItem->wSizeLen)? pItem->aColor:NULL;
			
			if(iPosY<sRect.top)iPosY=sRect.top;
															// Text ausgeben
			iOut	= strchar(cText,'\t',iLen);

			if(iOut>=iLen || !pData->wTabStop)
				{
				ExtTextColor(hDc,iPosX,iPosY,pData->uOption,&sRect,cText,iLen,pColor,pData->pColTable,pData->aFont);
				}
			else{											// Mit Tabulatoren ausgeben
				COLORREF	uOldC;					
				SIZE		sSize;
				int			iBase;
				int			iTab;

				iEnd	= sRect.right;
				iBase	= iPosX;

				for(iOff=0;iOff<iLen;)
					{
					if(iOff+iOut<iLen && GetTextExtentEx(hDc,cText+iOff,iOut,iEnd,NULL,NULL,&sSize,pColor,pData->aFont))
							sRect.right = iPosX+sSize.cx;
					else	sRect.right = iEnd;

					   iPosX = ExtTextColor(hDc,iPosX,iPosY,pData->uOption,&sRect,cText+iOff,iOut,(pColor)? pColor+iOff:0,pData->pColTable,pData->aFont);
					if(iPosX>=iEnd){iOff=iLen;break;}

					iOff   += iOut;

					if(iOff>=iLen)break;
					
					iTab	= iPosX-iBase;
					iTab   += pData->wTabStop;
					iTab    = pData->wTabStop-iTab%pData->wTabStop;

					if(pColor && (pData->uOption&0x70000000))
						{
						uOldC = SelectBkColorEx(hDc,pData->uOption,pColor[iOff],pData->pColTable);
						}

					DrawRectangle(hDc,iPosX,sRect.top,iTab,pData->bFontHeigh);

					if(pColor && (pData->uOption&0x70000000))
						{
						uOldC = SetBkColor(hDc,uOldC);
						}

					
					iPosX  += iTab;
					iOff   += 1;	
					iOut	= strchar(cText+iOff,'\t',iLen-iOff);

					sRect.left = iPosX;	
					}
				}
			}
		else if((pData->uStyle&0x0F)==CBS_DROPDOWNLIST)
			{												// Kein Text
			DrawRectanglePtr(hDc,&pData->sTextRect);
			}			 


		if(pData->sDropRect.right>pData->sDropRect.left)	// Ist der DropDownButton sichtbar
			{
			if(pData->bEnable==0)
				{
				uMode = CBXS_DISABLED;
				}
			else if(CallWindowProc(pData->pOldProc,hWnd,CB_GETDROPPEDSTATE,0,0))
				{
				uMode = CBXS_PRESSED;
				}
			else if(pData->bIsHot)
				{
				uMode = CBXS_HOT;
				}
			else{
				uMode = CBXS_NORMAL;
				}

			DrawCbButton(hDc,&pData->sDropRect,uMode,0);
			}


		EndPaint(hWnd,&sPaint);
		if(hRrgn)DeleteObject(hRrgn);

		return 0;	
			

	case CB_RESETCONTENT:									// Alle Eintrge lschen

		ColorComboDelete(pData,0,pData->iEntryCnt);
		break;


	case CB_DELETESTRING:									// Einen Text lschen

		ColorComboDelete(pData,(int)wParam,1);
		break;


	case CB_ADDSTRING:										// Einen Text am Ende einfgen

		iOff  = CallWindowProc(pData->pOldProc,hWnd,CB_ADDSTRING,wParam,lParam);
		pText = (LPCTSTR)lParam;

		ColorComboInsert(pData,iOff,_tcslen(pText));

		return iOff;	

	
	case CB_INSERTSTRING:									// Einen Text einfgen

		iOff  = CallWindowProc(pData->pOldProc,hWnd,CB_INSERTSTRING,wParam,lParam);
		pText = (LPCTSTR)lParam;

		ColorComboInsert(pData,(int)wParam,_tcslen(pText));

		break;	

			 
	case CB_SETCOLOR:										// Die Farben zu den Buchstaben einstellen

		iOff = (short)HIWORD(wParam);
		
		if(iOff<0)
			{
			if(iOff!=-1 || !pData->hEditWnd)return FALSE;
			return SendMessage(pData->hEditWnd,EM_SETCOLOR,wParam&0xFFFF,lParam);
			}
		
		if(iOff>=pData->iEntryCnt)
			{
			return FALSE;
			}

		COMBO_LOCK(pData);
		
		pItem	= pData->pEntries[iOff];
		pColor	= (LPBYTE)lParam;				
		uSize	= LOWORD(wParam);
		uFlag	= 0;

		if(uSize>pItem->wSizeMax)
			{
			uSize = pItem->wSizeMax;
			}

		for(iPosX=uSize-1;iPosX>=0;iPosX--)					// Gibt es nderungen
			{
			bByte = pColor[iPosX]&0x7F;
			bDiff = bByte^pItem->aColor[iPosX];

			if(!bDiff)continue;

			uFlag				|= bDiff;
			pItem->aColor[iPosX] = bByte;
			}
		
		pItem->wSizeLen = uSize;

		COMBO_UNLOCK(pData)
		
		if(uFlag)											// Hat sich was verndert
			{
			if(uFlag&0x40)									// Schrift verndert
				{
				if(!pData->aFont[0])
					{
					pData->aFont[0]		= GetDefaultFont();
					pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);
					}

				if(!pData->aFont[1])
					{
					pData->aFont[1]		= GetBoldFont(pData->aFont[0]);
					}
				}
			
			if(iOff==(int)CallWindowProc(pData->pOldProc,hWnd,CB_GETCURSEL,0,0))
				{
				RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
				}
			}
		
		return TRUE;
			

	case CB_SETCOLORMODE:									// Die Farboptionen einstellen

		   uBits = wParam|ETO_OPAQUE|ETO_FONTCNT_2;
		if(uBits!=pData->uOption)
			{
			pData->uOption = uBits;
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}

		if(!pData->hEditWnd)
			{
			SendMessage(pData->hEditWnd,EM_SETCOLORMODE,wParam,0);
			}

		return TRUE;


	case CB_SETCOLORITEM:									// Einen Farbeintrag einstellen

		if(!pData->pColTable)
			{
			pData->pColTable = new COLORREF[32];
			memset(pData->pColTable,0,sizeof(COLORREF)*32);
			}

		iMax = HIWORD(wParam);
		iOff = LOWORD(wParam);
					
		if(iMax)											// Liste mit Farben
			{
			if(iOff     >=32)return 0;
			if(iOff+iMax> 32)iMax=32-iOff;
			
			if(memcmp(pData->pColTable+iOff,(void*)lParam,sizeof(COLORREF)*iMax))
				{
				memcpy(pData->pColTable+iOff,(void*)lParam,sizeof(COLORREF)*iMax);
				RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
				}
			}
		else{												// Einzelne Farbe
			if(iOff>=32)return 0;
			if(pData->pColTable[iOff]!=(COLORREF)lParam)
				{
				pData->pColTable[iOff] = lParam;
				RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
				}
			
			iMax = 1;
			}
		
		if(!pData->hEditWnd)
			{
			SendMessage(pData->hEditWnd,EM_SETCOLORITEM,wParam,lParam);
			}
		
		return iMax;


	case CB_SETIMAGELIST:									// Die Iconliste einstellen

		if(!lParam)											// Liste lschen
			{
			pData->bIconX = 0;
			pData->bIconY = 0;
			iOff		  = pData->bFontHeigh;
			}
		else{												
			IMAGEINFO sInfo;
			
			ImageList_GetImageInfo((HIMAGELIST)lParam,0,&sInfo);

			pData->bIconX = (BYTE)(sInfo.rcImage.right -sInfo.rcImage.left);
			pData->bIconY = (BYTE)(sInfo.rcImage.bottom-sInfo.rcImage.top );
			iOff		  = (pData->bIconY+2>pData->bFontHeigh)? pData->bIconY+2:pData->bFontHeigh;
			}									

		pData->hImgages = (HIMAGELIST)lParam;
		RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
		ColorComboUpdateRegion(pData);
		
		if(pData->hEditWnd)
			{
			SendMessage(pData->hEditWnd,EM_SETIMAGELIST,0,lParam);
			}

		if(iOff && pData->hListWnd)
			{
			CallWindowProc(pData->pOldProc,hWnd,CB_SETITEMHEIGHT  ,0,iOff);
			}

		return TRUE;


	case CB_SETIMAGE:										// Das Icon einstellen

		iOff = (int)wParam;
		
		if(iOff<0)
			{
			if(iOff!=-1 || !pData->hEditWnd)return FALSE;
			SendMessage(pData->hEditWnd,EM_SETIMAGE,lParam,0);
			return TRUE;
			}
		
		if(iOff>=pData->iEntryCnt)
			{
			return FALSE;
			}

		COMBO_LOCK(pData);
		
		pItem = pData->pEntries[iOff];

		if(pItem->iIcon!=iOff)
			{
			   pItem->iIcon = (int)lParam;
			if(pData->bIconX && iOff==(int)CallWindowProc(pData->pOldProc,hWnd,CB_GETCURSEL,0,0))
				{
				RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
				}
			}

		COMBO_UNLOCK(pData);

		return TRUE;

	
	case CB_SETCALLBACK:									// Callback fr Farben

		pData->pCallback = (ColorComboCb)lParam;	
		pData->pCbParam  = (LPVOID)wParam;

		if(pData->hEditWnd)
			{
			SendMessage(pData->hEditWnd,EM_SETCALLBACK,wParam,lParam);
			}

		return TRUE;


	case CB_GETTEXTPTR:										// Einen Pointer direkt auf den Text holen

		if(!pData->hEditWnd)return (LPARAM)_T("");
		
		return SendMessage(pData->hEditWnd,EM_GETTEXTPTR,0,0);


	case CB_GETIMAGE:										// Die Icon-Nummer holen

		iOff = (int)wParam;
		
		if(iOff<0)
			{
			if(iOff!=-1 || !pData->hEditWnd)return -1;
			return SendMessage(pData->hEditWnd,EM_GETIMAGE,0,0);
			}
		
		if(iOff>=pData->iEntryCnt)
			{
			return -1;
			}

		pItem = pData->pEntries[iOff];

		return pItem->iIcon;


	case CB_GETCOLORPTR:									// Einen Pointer direkt auf die Farben holen
		
		iOff = (int)wParam;
		
		if(iOff<0)
			{
			if(iOff!=-1 || !pData->hEditWnd)return 0;
			return SendMessage(pData->hEditWnd,EM_GETCOLORPTR,0,0);
			}
		
		if(iOff>=pData->iEntryCnt)
			{
			return 0;
			}

		pItem = pData->pEntries[iOff];

		return (LPARAM)pItem->aColor;


	case CB_SYNCHWINDOW:									// Die Daten mit dem unterliegenden Fenster synchronisieren

		if(!pData->hEditWnd)return FALSE;
		return SendMessage(pData->hEditWnd,EM_SYNCHWINDOW,0,0);


	case CB_SETDROPPEDHEIGHT:								// Einstellen der Dropdown-Hhe

		pData->sDropY = (short)wParam;

		return TRUE;


	case CB_SETEDITSTYLE:									// Einstellen des Syles des Edit-Fensters

		uFlag  = GetWindowLong(pData->hEditWnd,GWL_STYLE);
		uBits  = ~((UINT)lParam)&uFlag;
		uBits |=  ((UINT)lParam)&((UINT)wParam);
		
		SetWindowLong(pData->hEditWnd,GWL_STYLE,uBits);

		return uBits;

	case CB_SETTABSTOP:									// Einstellen des Tab-Stops in Pixel

		   uSize = pData->wTabStop;
		if(uSize!=(WORD)wParam)
			{
			pData->wTabStop = (WORD)wParam;
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}

		return uSize;
		}



 
return CallWindowProc(pData->pOldProc,hWnd,uMsg,wParam,lParam);
}

