//*****************************************************************************
//*
//*
//*		ColorEditWnd.cpp
//*
//*
//*****************************************************************************
//
//	Ist einen Win-API-Edit-Fenster das Farben und dicke Schrift zu lst.
//
//	Fenster Styles:
//	
//		ES_LEFT             0x0000	Text links ausrichten	
//		ES_CENTER           0x0001	Text mittig ausrichten	
//		ES_RIGHT            0x0002	Text rects ausrichten	
//		ES_MULTILINE        0x0004	--- nicht untersttzt ---
//		ES_UPPERCASE        0x0008	--- nicht untersttzt ---
//		ES_LOWERCASE        0x0010	--- nicht untersttzt ---
//		ES_PASSWORD         0x0020	--- nicht untersttzt ---
//		ES_AUTOVSCROLL      0x0040	--- nicht untersttzt ---
//		ES_AUTOHSCROLL      0x0080	--- nicht untersttzt ---
//		ES_NOHIDESEL        0x0100	Textauswahl auch ohne Fokus anzeigen
//		ES_OEMCONVERT       0x0400	--- nicht untersttzt ---
//		ES_READONLY         0x0800	Es ist keine Texteingabe mglich
//		ES_WANTRETURN       0x1000	Die Tasten RET/TAB/ESC nia Notify melden
//		ES_NUMBER           0x2000	Nur die Eingabe von Ziffern ist mglich
//		ES_ALLOWDROP		0x4000	Dateien knnen auf das Control fallengelassen werden
//		ES_NOGRAYDISABLE	0x8000	Fenster nicht grau machen wenn es gesperrt ist
//	
//	Beispiel: 
//
//		ColorEditProc(hWnd,WM_ATTACH,0,0);					// Eingngen
//		SendMessage  (hWnd,EM_SETIMAGELIST,0,(LPARAM)GetIconList());
//		SendMessage  (hWnd,EM_SETIMAGE    ,GetIconNr("Test.cpp",SHGFI_USEFILEATTRIBUTES));
//		SendMessage  (hWnd,EM_SETCOLOR    ,16,(LPARAM)"\x42\x42\x42\x42\0\3\3\3\3\3\0\x20\x20\x20\x20\x20");
//				     
//		ColorEditProc(hWnd,WM_DETACH,0,0);					// Aushngen
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#ifdef		UNICODE
#ifndef	   _UNICODE
#define    _UNICODE
#endif
#endif

#include	<windows.h>
#include	<commctrl.h>
#include	<assert.h>
#include	<string.h>
#include	<stdlib.h>
#include	<stddef.h>
#include	<stdio.h>
#include	<tchar.h>

#include	"WinExtension.h"
#include	"ColorEditWnd.h"
#include	"StringFunctions.h"

#ifdef	   _MSC_VER
#pragma		warning( disable :4996 )
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	UNDO_MAX		32
#define 	TEXT_MAX		1024
#define 	ID_SCROLL		0x2345
#define		ES_COMBO		0x0200
#define 	ASSERT			assert
#define 	ENTRIES(a)		(sizeof(a)/sizeof((a)[0]))
#define		CF_TEXT_EX		((sizeof(TCHAR)>1)? CF_UNICODETEXT:CF_TEXT)

typedef struct
	{
	WORD		wFlags;									// Bit15=Cursor links Bit14=WM_CHAR
	WORD		wCursor;								// Wo stand der Cursor nach dem Lschen
	WORD		wSizeDel;								// Wie lange ist der Lschtext
	WORD		wSizeIns;								// Wie lange ist der Einfgetext
	TCHAR		cText[1];								// Zeiger auf Textspeicher
	}UndoData;

typedef struct
	{
	WNDPROC		pOldProc;								// Vorherige Fensterfunktion
	LPVOID		pCbParam;								// User-Parameter fr Callback
	ColorEditCb	pCallback;								// Callback fr Farben
	LPCOLORREF	pColTable;								// Farbtabele fr ExtTextColor() (0=default)
	HIMAGELIST	hImgages;								// Handle der Image-Liste
	LPBYTE		pColors;								// Farben der Buchstaben	
	HFONT		aFont[2];								// Fonts fr das Zeichnen ([1]=bold)
	LPTSTR		pText;									// Zeiger auf den Text
	LPARAM		lUserId;								// User ID fr Callback
	UINT		uSize;									// Anzahl der Buchstaben
	UINT		uStyle;									// Style des Fensters
	UINT		uMaxSize;								// MAximale Anzahl der Buchstaben
	DWORD		uOption;								// Bits mit den Farboptionen fr ExtTextColor()
	DWORD		uSelNow;								// Startposition der Auswahl
	DWORD		uSelEnd;								// Startposition der Auswahl
	DWORD		uScroll;								// Scroll-Position (Zeichen)
	short		sStart;									// Start-Pixel vom Text
	short		sScrOff;								// Pixel-Offset von Scroll-Position
	short		sTextSize;								// Textbreite in Pixel
	short		sCaretPosX;								// Pixel-Offset von Blink-Cursor
	short		sCaretPosY;								//			*
	short		sWindowX;								// Ist die Fenstergre
	short		sWindowY;								//			*
	short		sMarks[2];								// Pixel-Offset der Markierung
	bool		bLbDown;								// Ist die linke Maustaste gedrckt
	bool		bEnable;								// Ist das Fenster freigegeben
	bool		bTimerOn;								// Ist das Scroll-Timer aktiv
	bool		bChanged;								// Ist der Text im unteren Fenster ungleich
	bool		bLockDraw;								// Ist das Zeichnen gesperrt
	bool		bHasFocus;								// Hat das Fenster den Fokus
	BYTE		bFontHeigh;								// Hhe der Schrift
	BYTE		bShowCaret;								// Ist der Cursor sichtbar
	BYTE		bIconX;									// Breite der Icons
	BYTE		bIconY;									// Hhe der Icons
	INT			iIcon;									// Ist die Icon-Nummer (-1 keines)
	WORD		wLastKey;								// Letzte abgefangene Taste
	WORD		wUndoMax;								// Wievile Felder in den Undo-Daten sind belegt
	WORD		wUndoEnd;								// Position des ersten Undo-Feldes
	WORD		wUndoPos;								// Ist die aktuelle Undo Position
	UndoData  **pUndo;									// Zeiger auf die Undo-Daten	
	}ColorEditData;


static	WNDPROC				pOldEditProc	= 0;		// Fensterfunktion von Edit
static	ATOM				wColorEditAtom	= 0;		// ID-Fr die Fenster-Extra-Daten
static	CRITICAL_SECTION	sEditLock;					// Semaphore fr den Zugriff

extern	LPTSTR	 WINAPI		GetClipboardText(int *pLen);
extern	int		 WINAPI		SetClipboardText(LPCTSTR pText,int iSize);

#define EDIT_LOCK(d)		EnterCriticalSection(&sEditLock);
#define EDIT_UNLOCK(d)		LeaveCriticalSection(&sEditLock);




//*****************************************************************************
//*
//*		ColorEditGetIndex
//*
//*****************************************************************************
//	Textindex zu einer Koordinate ermitteln
//	pData	: Zeiger auf die Fensterdaten
//	hWnd	: Ist das Fenster-Handle
//	iPosX	: X-Koordinate im Fenster
//	pPos	: Hier wird der Pixel-Offset gespeichert
//	Ergibt den Buchstaben-Index zur Koordinate.
static int ColorEditGetIndex(ColorEditData *pData,HWND hWnd,int iPosX,LPINT pPos=0)
{
INT		aArray[TEXT_MAX+1];
RECT	sRect;
UINT	uStep;
UINT	uSize;
INT		iDiff;
INT		iMax;
HDC		hDc;



	GetClientRect(hWnd,&sRect);

	hDc		  = GetWindowDC(hWnd);
	iPosX    += pData->sScrOff;
	iPosX    -= pData->sStart;
	iMax	  = iPosX+24;	

	if(iMax<32)iMax=32;

	GetTextExtentEx(hDc,pData->pText,pData->uSize,iMax,(LPINT)&uSize,aArray+1,NULL,pData->pColors,pData->aFont);

	ReleaseDC(hWnd,hDc);

	aArray[0] = 0;

	for(uStep=1;uStep<uSize;uStep++)
		{
		if(aArray[uStep]>=iPosX)break;
		}

	iDiff = aArray[uStep]-aArray[uStep-1]+1;
	iDiff = aArray[uStep]-iDiff/2;
	
	if(iPosX<=iDiff)uStep--;
	if(pPos)*pPos=aArray[uStep];


return uStep;
}


//*****************************************************************************
//*
//*		ColorEditCheckScroll
//*
//*****************************************************************************
//	Prft die Scroll-Position
//	pData		: Zeiger auf die Fensterdaten
//	hWnd		: Ist das Fenster-Handle
//	uMode		: Diverse Flags fr die Bearbeitung
//					Bit0 = Nur die Offsets berechnen
//					Bit1 = Kein Wordverschub
//	Ergibt TRUE wenn sich die Scroll-Position verndert hat
static BOOL ColorEditCheckScroll(ColorEditData *pData,HWND hWnd,UINT uMode=0)
{
INT		aArray[TEXT_MAX+1];
LPCTSTR	pText;
RECT	sRect;
BOOL	bFlag;
UINT	uStep;
UINT	uNext;
INT		iSize;
INT		iDiff;
INT		iOff;
HDC		hDc;


	GetClientRect(hWnd,&sRect);

	   iSize = sRect.right-sRect.left-pData->sStart;
	if(iSize<=0)
		{
		return 0;
		}


	if(pData->uScroll>pData->uSize)pData->uScroll=pData->uSize;
	if(pData->uSelNow>pData->uSize)pData->uSelNow=pData->uSize;
	if(pData->uSelEnd>pData->uSize)pData->uSelEnd=pData->uSize;
	
	pText	= pData->pText;
	hDc		= GetWindowDC(hWnd);

	aArray[0] = 0;
	GetTextExtentEx(hDc,pText,pData->uSize,0x40000000,NULL,aArray+1,NULL,pData->pColors,pData->aFont);

	ReleaseDC(hWnd,hDc);


	bFlag = FALSE;		

	if(uMode&1)goto Done;


	if(aArray[pData->uSize]+5<iSize)						// Ist Text krzer als Fenster	
		{
		if(pData->uScroll)
			{
			pData->uScroll = 0;
			bFlag = TRUE;		
			}
		
		goto Done;
		}


	iOff   = aArray[pData->uSelNow]-iSize;
	if(!(uMode&2))iOff+=10;

														
	if(aArray[pData->uScroll]<iOff)							// Cursor ist rechts auen
		{
		uStep = pData->uScroll;

		while(uStep<pData->uSelNow)
			{
			uStep++;
			if(aArray[uStep]>=iOff)break;
			}

		uNext	= pData->uSelNow;
		iOff	= aArray[uNext];

		while(uNext<pData->uSize)							// Ganzes Wort vor scrollen
			{
			if(!strwchar(pText[uNext],0x3000000F))break;

			   iDiff = aArray[uNext+1]-iOff;
			if(iDiff*3>iSize)break;

			uNext++;
			uStep++;
			}
		

		if(pData->uScroll!=uStep)							// Hat sich die Position gendert
			{
			pData->uScroll	= uStep;
			bFlag			= TRUE;
			}
		}
	
	
	iOff = aArray[pData->uSelNow];
	if(!(uMode&2))iOff-=10;

	if(aArray[pData->uScroll]>iOff)							// Cursor ist links auen
		{
		uStep = pData->uScroll;

		while(uStep>0)
			{
			uStep--;
			if(aArray[uStep]<=iOff)break;
			}

		iOff = aArray[uStep];
		
		if(!(uMode&2))										// Ganzes Wort zurck scrollen
			{
			while(uStep>0)										
				{
				if(!strwchar(pText[uStep-1],0x3000000F))break;

				   iDiff = iOff-aArray[uStep-1];
				if(iDiff*3>iSize)break;

				uStep--;
				}
			}
		
		if(pData->uScroll!=uStep)							// Hat sich die Position gendert
			{
			pData->uScroll	= uStep;
			bFlag			= TRUE;
			}
		}

Done:

	pData->sTextSize = aArray[pData->uSize  ];				// Textbreite in Pixel
	pData->sScrOff	 = aArray[pData->uScroll];				// Pixel-Offset von Scroll-Position
	pData->sMarks[0] = aArray[pData->uSelNow];				// Pixel-Offset der Markierung
	pData->sMarks[1] = aArray[pData->uSelEnd];

	if(bFlag)SetScrollPos(hWnd,SB_HORZ,pData->uScroll,FALSE);



return bFlag;
}

//*****************************************************************************
//*
//*		ColorEditInvalidRect
//*
//*****************************************************************************
//	Einen Teil des Fensters neuzeichnen
//	pData		: Zeiger auf die Fensterdaten
//	hWnd		: Ist das Fenster-Handle
//	iFrom		: Ist das Startzeichen in X-Richtung
//	iStop		: Ist das Stopzeichen  in X-Richtung
//	Eribt TRUE wenn etwas neu zu zeichnen war.	
static BOOL ColorEditInvalidRect(ColorEditData *pData,HWND hWnd,int iFrom=0,int iStop=0xFFFF)
{
LPCTSTR	pText;
RECT	sRect;
SIZE	sSize;
INT		iRest;
INT		iBase;
INT		iEnd;
INT		iMax;
HDC		hDc;



	if(iFrom<(int)pData->uScroll)							// Start Position vor Scroll-Position
		{
		iFrom = pData->uScroll;
		}

	if(iFrom>(int)pData->uSize)								// Position nach dem Textende
		{
		return FALSE;
		}

	if(iFrom>=iStop)										// Ist der Bereich gltig	
		{
		return FALSE;
		}

	pText	= pData->pText;
	hDc		= GetWindowDC(hWnd);

	GetClientRect(hWnd,&sRect);
	sRect.left += pData->bIconX;

	iRest	= pData->uScroll-iFrom;							// Linken Offset berechnen
	GetTextExtentEx(hDc,pText+pData->uScroll,iRest,0x40000000,NULL,NULL,&sSize,pData->pColors+pData->uScroll,pData->aFont);
	iBase	= sSize.cx-2;

	if(iStop<(int)pData->uSize)
		{
		iRest	= pData->uScroll-iStop;						// Rechten Offset berechnen
		GetTextExtentEx(hDc,pText+pData->uScroll,iRest,0x40000000,NULL,NULL,&sSize,pData->pColors+pData->uScroll,pData->aFont);
		iEnd	= sSize.cx+2;
		}
	else{
		iEnd	= 0xFFFF;
		}

	ReleaseDC(hWnd,hDc);

															// Den Text ausrichten
	if(pData->uScroll==0 && (pData->uStyle&(ES_RIGHT|ES_CENTER)))	
		{
		   iMax = sRect.right-sRect.left-pData->sTextSize;
		if(iMax>4)			
			{
			if(pData->uStyle&ES_RIGHT)
				{
				iBase += iMax-1;
				iEnd  += iMax-1;
				}
			else if(pData->uStyle&ES_CENTER)
				{
				iBase += iMax/2;
				iEnd  += iMax/2;
				}
			}
		}

	if(iBase>sRect.left)
		{
		if(iBase>=sRect.right)return TRUE;
		sRect.left = iBase;
		}

	if(iEnd>sRect.right)
		{
		sRect.right = iEnd;
		}


	InvalidateRect(hWnd,&sRect,FALSE);	



return TRUE;
}


//*****************************************************************************
//*
//*		ColorEditRemoveUndo
//*
//*****************************************************************************
//	Lscht UNDO-Speicher
//	pData	: Zeiger auf die Fensterdaten
//	uStart	: Erster zu lschender Index
//	uStop	: Bis zu diesem Index wird glscht (ohne uStop)	
static void ColorEditRemoveUndo(ColorEditData *pData,UINT uStart,UINT uStop)
{

	if(!pData->pUndo)return;

	uStart %= UNDO_MAX;
	uStop  %= UNDO_MAX;

	while(uStart!=uStop)
		{
		delete pData->pUndo[uStart];
		pData->pUndo[uStart] = NULL;

		uStart += 1;
		uStart %= UNDO_MAX;
		}

}

//*****************************************************************************
//*
//*		ColorEditInput
//*
//*****************************************************************************
//	ndert den Text in einem 
//	pData	: Zeiger auf die Fensterdaten
//	hWnd	: Ist das Fenster-Handle
//	iMode	: Was soll gelscht werden (0=Select 1=Entf 2=Back) Bit8=kein-Undo
//	pInsert	: Ist der Text der eingefgt werden soll
//	iSize	: Ist die Lnge des einzufgenden Textes
//	Ergibt 1 wenn etwas vendert wurde, ansonsten 0.
static int ColorEditInput(ColorEditData *pData,HWND hWnd,int iMode,LPCTSTR pInsert,int iSize)
{
TCHAR		cText[TEXT_MAX];
HWND		hParent;
UINT		uMyId;
UINT		uNext;
UndoData	sUndo;
UndoData  **pList; 	
UndoData   *pItem; 	
int			iFlag;
int			iByte;
int			iNone;
int			iLen;
int			iNow;
int			iEnd;
int			iMin;


	if((pData->uStyle&ES_READONLY) || !pData->bEnable)		// Schreibschutz vorhanden
		{
		return 0;
		}

	
	EDIT_LOCK(pData)

	iNone	= iMode&0x100;
	iMode  &= 0xFF;
		

	if(pData->uSelNow!=pData->uSelEnd)						// Ist etwas ausgewhlt
		{
		iMode = 0;
		}
	else if((iMode&2) && pData->uSelNow==0)					// Cursor am linken Rand
		{
		iMode = 0;
		}
	else if((iMode&1) && pData->uSelNow>=pData->uSize)		// Cursor am Rechten Rand
		{
		iMode = 0;
		}


	iFlag			= 0;
	sUndo.wFlags	= 0;
	sUndo.wSizeDel  = 0;		
	sUndo.wSizeIns  = 0;		
	
	switch(iMode)
		{
	default: if(pData->uSelNow<pData->uSelEnd)
				{
				iNow = pData->uSelNow;
				iEnd = pData->uSelEnd;
				sUndo.wFlags = 0x8000;
				}
			else{
				iEnd = pData->uSelNow;
				iNow = pData->uSelEnd;
				}

			 break;

	case 1: iNow = pData->uSelNow;
			iEnd = pData->uSelNow+1;
			break;

	case 2: iNow = pData->uSelNow-1;
			iEnd = pData->uSelNow;
			break;
		}

	
	


	if(iNow<iEnd)											// Etwas lschen	
		{
		iLen	= pData->uSize-iEnd;
		iByte	= iEnd-iNow;

		memmove(cText              ,pData->pText  +iNow,(iByte )*sizeof(TCHAR));
		memmove(pData->pText  +iNow,pData->pText  +iEnd,(iLen+1)*sizeof(TCHAR));
		memmove(pData->pColors+iNow,pData->pColors+iEnd, iLen);
	
		sUndo.wSizeDel = (WORD)iByte;		
		sUndo.wCursor  = (WORD)iNow;	
		pData->uSize  -= iByte;
		pData->uSelNow = iNow;
		pData->uSelEnd = iNow;

		iFlag = 1;
		}

	iMin = iNow;
	
	if(iSize>0)												// Etwas einfgnen
		{
		if(iSize>=TEXT_MAX || pData->uSize+iSize>TEXT_MAX)
			{
			MessageBeep(MB_ICONEXCLAMATION);
			goto Done;
			}


		iLen = pData->uSize-iNow;
		
		memmove(pData->pText  +iNow+iSize,pData->pText  +iNow,(iLen+1)*sizeof(TCHAR));
		memmove(pData->pColors+iNow+iSize,pData->pColors+iNow, iLen);
		
		memcpy (pData->pText  +iNow,pInsert,iSize*sizeof(TCHAR));
		memset (pData->pColors+iNow,0      ,iSize);

		sUndo.wSizeIns = (WORD)iSize;		
		sUndo.wCursor  = (WORD)iNow;	
		iNow          += iSize;
		pData->uSize  += iSize;
		pData->uSelNow = iNow;
		pData->uSelEnd = iNow;

		iFlag = 1;
		}


Done:

	if(iFlag)												// Muss etwas neu gezeichnet werden
		{
		pData->bChanged = true;

		if(iNone==0)										// Undo-Eintrag erstellen
			{
			if(pData->pUndo)
				{
				pList			= pData->pUndo;
				}
			else{
				pList			= new UndoData*[UNDO_MAX];
				pData->pUndo	= pList;
				memset(pList,0,sizeof(UndoData*)*UNDO_MAX);
				}

															// Nur Buchstaben hizugefgt
			if(sUndo.wSizeIns==1 && strwchar(*pInsert,0x3000000F))
				{
				uNext  = pData->wUndoPos;
				uNext += UNDO_MAX-1;
				uNext %= UNDO_MAX;
				pItem  = pList[uNext];

				if(pItem && (pItem->wFlags&0x4000) && pItem->wSizeIns<32 && sUndo.wSizeDel==0)
					{
					pItem->cText[pItem->wSizeDel+pItem->wSizeIns] = *pInsert;
					pItem->wSizeIns++;

					ColorEditRemoveUndo(pData,pData->wUndoPos,pData->wUndoMax);
					pData->wUndoMax = pData->wUndoPos;

					goto Jump;
					}

				sUndo.wFlags |= 0x4000;
				}


			uNext	= pData->wUndoPos+1;
			uNext  %= UNDO_MAX;


			if(uNext==pData->wUndoEnd)						// Endmarke nachziehen
				{
				delete pList[uNext];
				pList[uNext] = NULL;
				pData->wUndoEnd += 1;
				pData->wUndoEnd %= UNDO_MAX;
				}
										
			if((sUndo.wFlags&0x4000) && sUndo.wSizeIns<32)	// Bereich fr Buchstaben reservieren
					iLen = sUndo.wSizeDel+32;
			else	iLen = sUndo.wSizeDel+sUndo.wSizeIns;
			
			iLen   *= sizeof(TCHAR);
			iLen   += offsetof(UndoData,cText);
			pItem	= (UndoData*)new BYTE[iLen];

			memcpy(pItem,&sUndo,offsetof(UndoData,cText));
			memcpy(pItem->cText,cText,sUndo.wSizeDel*sizeof(TCHAR));
			memcpy(pItem->cText+sUndo.wSizeDel,pInsert,sUndo.wSizeIns*sizeof(TCHAR));
			
			ColorEditRemoveUndo(pData,pData->wUndoPos,pData->wUndoMax);

			pList[pData->wUndoPos]	= pItem;
			pData->wUndoMax			= uNext;
			pData->wUndoPos			= uNext;
Jump:;		}

		
		if(pData->pCallback)								// Callback fr Farben
			{
			pData->pCallback(pData->pCbParam,pData->pText,pData->uSize,pData->pColors,&pData->iIcon,pData->lUserId,-1);
			}

		hParent = GetParent(hWnd);
		uMyId	= GetWindowLong(hWnd,GWL_ID);

		SendMessage(hParent,WM_COMMAND,MAKELONG(uMyId,EN_CHANGE   ),(LPARAM)hWnd);
		SendMessage(hParent,WM_COMMAND,MAKELONG(uMyId,EN_SELCHANGE),(LPARAM)hWnd);

		if(ColorEditCheckScroll(pData,hWnd))
			{
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}
		else{
			ColorEditInvalidRect(pData,hWnd,iMin);
			}
		}


	EDIT_UNLOCK(pData)


return iFlag;
}



//*****************************************************************************
//*
//*		ColorEditProc
//*
//*****************************************************************************
//	Ist die Fensterfunktion fr das Edit-Fenster.
LRESULT CALLBACK ColorEditProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
POINT			sPoint;
PAINTSTRUCT		sPaint;
ColorEditData  *pData;	
UndoData	   *pUndo;
WNDPROC			pProc;
LPTSTR			pText;
LPBYTE			pByte;
TCHAR			cSign;
TCHAR			cLast;
HDROP			hDrop;
HRGN			hRrgn;
SIZE			sSize;
RECT			sRect;
UINT			uSize;
UINT			uBits;
UINT			uFlag;
BYTE			bDiff;
BYTE			bItem;
INT				iPosX;
INT				iPosY;
INT				iPosA;
INT				iPosB;
INT				iOffA;
INT				iOffB;
INT				iOldA;
INT				iOldB;
INT				iStep;
INT				iBits;
INT				iMax;
INT				iOff;
INT				iEnd;
HDC				hDc;



		pData = (ColorEditData*)GetPropW(hWnd,(LPCWSTR)wColorEditAtom);
	if(!pData && uMsg!=WM_NCDESTROY && uMsg!=WM_DETACH)		// Keine Daten vorhanden
		{
			pData = new ColorEditData;
		if(!pData)return 0;

		if(!wColorEditAtom)									// Globale Initalisierung
			{
			WNDCLASS	sClass;
			InitializeCriticalSection(&sEditLock);
			GetClassInfo(NULL,_T("EDIT"),&sClass);
			pOldEditProc   = sClass.lpfnWndProc;
			wColorEditAtom = GlobalAddAtomW(L"CeData");
			}

		ASSERT(IsWindow(hWnd));
		memset(pData,0,sizeof(ColorEditData));

		#ifdef _DEBUG
		char 	cClass[32];
		GetClassNameA(hWnd,cClass,sizeof(cClass));
		ASSERT(!strcmp(cClass,"Edit"));
		#endif 	

		   pData->pOldProc	= (WNDPROC)GetWindowLongPtr(hWnd,GWL_WNDPROC);
		if(pData->pOldProc==ColorEditProc)
			{
			pData->pOldProc = pOldEditProc;
			}
		
		if(uMsg==WM_ATTACH)									// Eine andere alte Fensterfunktion bernehmen
			{
			if(lParam)
					pData->pOldProc = (WNDPROC)lParam;
			else	SetWindowLongPtr(hWnd,GWL_WNDPROC,(LPARAM)ColorEditProc);	
			}

		GetClientRect(hWnd,&sRect);
		
		pData->uOption		= ETO_OPAQUE|ETO_FONTCNT_2;
		pData->uStyle		= GetWindowLong(hWnd,GWL_STYLE);
		pData->aFont[0]		= (HFONT)CallWindowProc(pData->pOldProc,hWnd,WM_GETFONT     ,0,0);
		pData->uMaxSize		= (UINT )CallWindowProc(pData->pOldProc,hWnd,EM_GETLIMITTEXT,0,0);
		pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);
		pData->bEnable		= IsWindowEnabled(hWnd)!=0;
		pData->bHasFocus	= GetFocus()==hWnd;
		pData->sWindowX		= (short)sRect.right;
		pData->sWindowY		= (short)sRect.bottom;
		pData->lUserId		= -1;
		pData->iIcon		= -1;

		if(pData->uMaxSize==0)pData->uMaxSize=256;
		pData->pColors		= new BYTE[pData->uMaxSize*(1+sizeof(TCHAR))+8];
		pData->pText		= (LPTSTR)(pData->pColors+((pData->uMaxSize+2)&~1));
		pData->uSize		= CallWindowProc(pData->pOldProc,hWnd,WM_GETTEXT,pData->uMaxSize,(LPARAM)pData->pText);
							  CallWindowProc(pData->pOldProc,hWnd,EM_GETSEL,(LPARAM)&pData->uSelNow,(LPARAM)&pData->uSelEnd);
		

		ASSERT((pData->uStyle&ES_ALLOWDROP)==0 || (GetWindowLong(hWnd,GWL_EXSTYLE)&WS_EX_ACCEPTFILES));
		ASSERT((pData->uStyle&ES_MULTILINE)==0);

		if(pData->bHasFocus)CreateCaret(hWnd,NULL,1,pData->bFontHeigh-1);
		memset(pData->pColors,0,pData->uMaxSize);
		ColorEditCheckScroll(pData,hWnd);

		SetPropW(hWnd,(LPCWSTR)wColorEditAtom,pData);
		} 


	if(uMsg>=WM_KEYFIRST && uMsg<=WM_KEYLAST)				//	TAB/RET/ESC an Eltern-Fenster weiterleiten
	if(pData && pData->uStyle&ES_WANTRETURN)							
		{
		if(uMsg!=WM_KEYDOWN && pData->wLastKey==wParam)
			{
			return 0;
			}

		if(wParam==VK_TAB || wParam==VK_RETURN || wParam==VK_ESCAPE)
			{
			NMKEY		sHeader;
			LRESULT		lRet;

			sHeader.hdr.hwndFrom	= hWnd;
			sHeader.hdr.code		= NM_KEYDOWN;
			sHeader.hdr.idFrom		= GetWindowLong(hWnd,GWL_ID);
			sHeader.nVKey			= (UINT)wParam;
			sHeader.uFlags			= (UINT)lParam;

			   lRet = SendMessage(GetParent(hWnd),WM_NOTIFY,sHeader.hdr.idFrom,(LPARAM)&sHeader);
			if(lRet)
				{
				pData->wLastKey = (WORD)wParam;
				return 0;
				}

			pData->wLastKey = 0;
			}
		}

	
	switch(uMsg)
		{													// Fenster anhngen 
	case WM_ATTACH:
		
		ASSERT(IsWindow(hWnd));
		
		return TRUE;		


	case WM_DETACH:											// Fenster abhngen 

		if(!lParam)
			{
			SetWindowLongPtr(hWnd,GWL_WNDPROC,(LPARAM)pData->pOldProc);
			}

		RemovePropW(hWnd,(LPCWSTR)wColorEditAtom);
		
		pProc = pData->pOldProc;

		if(pData->pColTable)
			{
			delete pData->pColTable;
			}
		
		if(pData->pUndo)
			{
			ColorEditRemoveUndo(pData,pData->wUndoEnd,pData->wUndoMax);
			delete pData->pUndo;
			}

		delete pData->pColors;
		delete pData;
		
		return (LPARAM)pProc;


	case WM_NCDESTROY:										// Wird das Fenster zerstrt

		pProc = pData->pOldProc;
		ColorEditProc(hWnd,WM_DETACH,0,0);
		
		return CallWindowProc(pProc,hWnd,uMsg,wParam,lParam);


	case WM_STYLECHANGED:									// Hat sich der Style verndert

		if(wParam==GWL_STYLE)
			{
			uBits = GetWindowLong(hWnd,GWL_STYLE);		  
			
			if(uBits&0x03)RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);

			pData->uStyle = uBits;

			ASSERT((uBits&ES_ALLOWDROP)==0 || (GetWindowLong(hWnd,GWL_EXSTYLE)&WS_EX_ACCEPTFILES));
			ASSERT((pData->uStyle&ES_MULTILINE)==0);
			}

		break;


	case WM_SETFONT:										// Neue Schriftart

		if(wParam==0)
			{
			wParam = (WPARAM)GetDefaultFont();
			}
		
		if(wParam && (HFONT)wParam!=pData->aFont[0])
			{
			pData->aFont[0]		= (HFONT)wParam;	
			pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);

			if(pData->aFont[1])
				{
				pData->aFont[1]	= GetBoldFont(pData->aFont[0]);
				}
			
			if(pData->bHasFocus)DestroyCaret();
			CreateCaret(hWnd,NULL,1,pData->bFontHeigh-1);
			}

		break;


	case WM_SETTEXT:										// Text abfragen

		if(!lParam)return FALSE;

		EDIT_LOCK(pData)

		iMax	= pData->uMaxSize;
		pText	= (LPTSTR)lParam;
		cLast	= 0;

		for(iOff=0;iOff<iMax;iOff++)						// Neuen Text kopieren
			{
			   cSign = pText[iOff];
			if(cSign==0   )break;
			if(cSign=='\t')cSign=' ';						// Tausche Tabs zu Space

			cLast |= pData->pText[iOff]^cSign;			
			pData->pText[iOff] = cSign;
			}

		if(!cLast && pData->uSize==(UINT)iOff)				// Wurde nichts verndert
			{
			return TRUE;
			}

		pData->uSize		= iOff;
		pData->bChanged	 	= true;
		pData->pText[iOff]	= 0;

		EDIT_UNLOCK(pData)

		if(pData->uSelNow>(UINT)iOff)
			{
			pData->uSelNow=iOff;
			SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(GetWindowLong(hWnd,GWL_ID),EN_SELCHANGE),(LPARAM)hWnd);
			}
		
		if(pData->uSelEnd>(UINT)iOff)
			{
			pData->uSelEnd=iOff;
			SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(GetWindowLong(hWnd,GWL_ID),EN_SELCHANGE),(LPARAM)hWnd);
			}

		if(pData->pCallback)								// Callback fr Farben
			{
			pData->pCallback(pData->pCbParam,pData->pText,pData->uSize,pData->pColors,&pData->iIcon,pData->lUserId,-1);
			}

		ColorEditCheckScroll(pData,hWnd);
		RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);

		return TRUE;


	case WM_GETTEXT:										// Text abfragen

		   iMax = (int)wParam;
		if(iMax<=0 || !lParam)return 0;

		iMax--;

		if((UINT)iMax>pData->uSize)iMax=pData->uSize;
		memcpy((void*)lParam,pData->pText,iMax*sizeof(TCHAR));
		((LPTSTR)lParam)[iMax] = 0;

		return iMax;

	
	case WM_GETTEXTLENGTH:									// Textlnge abfragen

		return pData->uSize;
	

	case WM_ERASEBKGND:										// Hintergrund lschen

		return 0;

	
	case WM_PASTE:											// Text einfgen

		   pText = GetClipboardText(&iEnd);
		if(pText)
			{
			ColorEditInput(pData,hWnd,0,pText,iEnd);
			delete pText;
			}
		
		return 0;

	
	case WM_CLEAR:											// Auswahl lschen

		ColorEditInput(pData,hWnd,0,NULL,0);

		return 0;


	case WM_COPY:											// Text kopieren
	case WM_CUT:

		if(pData->uSelEnd==pData->uSelNow)
			{
			SetClipboardText(pData->pText,pData->uSize);
			}
		else{
			if(pData->uSelEnd>pData->uSelNow)
				{
				SetClipboardText(pData->pText+pData->uSelNow,pData->uSelEnd-pData->uSelNow);
				}
			else{
				SetClipboardText(pData->pText+pData->uSelEnd,pData->uSelNow-pData->uSelEnd);
				}
			
			if(uMsg==WM_CUT)								// Auswahl lschen
				{
				ColorEditInput(pData,hWnd,0,NULL,0);
				}
			}
		
		return 0;


	case WM_UNDO:											// Rckgnig machen
	case EM_UNDO:

		if(pData->wUndoMax==0 || (pData->uStyle&ES_READONLY) || !pData->bEnable)
			{
			return FALSE;
			}

		if(pData->wUndoEnd==pData->wUndoPos) 
			{
			return FALSE;
			}
			
		iOff  = pData->wUndoPos;
		iOff += UNDO_MAX-1;
		iOff %= UNDO_MAX;

			pUndo = pData->pUndo[iOff];
		if(!pUndo)return FALSE;

		pData->wUndoPos	= (WORD)iOff;
		pData->uSelEnd  = pUndo->wCursor;
		pData->uSelNow  = pUndo->wCursor+pUndo->wSizeIns;

		ColorEditInput(pData,hWnd,0x100,pUndo->cText,pUndo->wSizeDel);

		return TRUE;


	case EM_REDO:											// Wiederherstellen

		if(pData->wUndoMax==0 || (pData->uStyle&ES_READONLY) || !pData->bEnable)
			{
			return FALSE;
			}

		if(pData->wUndoMax==pData->wUndoPos) 
			{
			return FALSE;
			}
			
		iOff  = pData->wUndoPos+1;
		iOff %= UNDO_MAX;

			pUndo = pData->pUndo[pData->wUndoPos];
		if(!pUndo)return FALSE;

		pData->wUndoPos	= (WORD)iOff;
		pData->uSelEnd  = pUndo->wCursor;
		pData->uSelNow  = pUndo->wCursor+pUndo->wSizeDel;

		ColorEditInput(pData,hWnd,0x100,pUndo->cText+pUndo->wSizeDel,pUndo->wSizeIns);

		return TRUE;

	
	case EM_CANUNDO:										// Is Undo mglich

		
		if(pData->wUndoMax==0 || (pData->uStyle&ES_READONLY) || !pData->bEnable)
			{
			return FALSE;
			}

		if(pData->wUndoEnd==pData->wUndoPos) 
			{
			return FALSE;
			}
			
		return TRUE;

	
	case WM_SETFOCUS:										// Fokus setzen

		if(!pData->bHasFocus)
			{
			CreateCaret(hWnd,NULL,1,pData->bFontHeigh-1);
			pData->bHasFocus = true;
			
			if(pData->uSelNow!=pData->uSelEnd)
				{
				ColorEditInvalidRect(pData,hWnd);
				}
			}

		SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(GetWindowLong(hWnd,GWL_ID),EN_SETFOCUS),(LPARAM)hWnd);
		
		if(!pData->bShowCaret && pData->sCaretPosX>=0 && pData->sCaretPosX<=pData->sWindowX)
			{
			pData->bShowCaret = true;
			SetCaretPos(pData->sCaretPosX,pData->sCaretPosY);
			ShowCaret(hWnd);
			}
			
		return 0;


	case WM_KILLFOCUS:										// Fokus geht verloren

		if(pData->bShowCaret)
			{
			pData->bShowCaret = false;
			HideCaret(hWnd);
			}
		
		if(pData->bHasFocus)
			{
			pData->bHasFocus = false;
			DestroyCaret();		

			if(pData->uSelNow!=pData->uSelEnd)
				{
				ColorEditInvalidRect(pData,hWnd);
				}
			}
		
		SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(GetWindowLong(hWnd,GWL_ID),EN_KILLFOCUS),(LPARAM)hWnd);

		return 0;		


	case WM_SIZE:											// Die Fensterge hat sich verndert

		pData->sWindowX	= LOWORD(lParam);  
		pData->sWindowY	= HIWORD(lParam);  

		break;


	case WM_RBUTTONUP:										// Rechte Maustaste fr Context-Men

		RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
		break;


	case WM_RBUTTONDOWN:									// Rechte Maustaste fr Context-Men

		if(pData->bChanged)
			{
			pData->bChanged = false;
			CallWindowProc(pData->pOldProc,hWnd,WM_SETTEXT,0,(LPARAM)pData->pText);
			}

		if(pData->uSelNow!=pData->uSelEnd)
			{
			CallWindowProc(pData->pOldProc,hWnd,EM_SETSEL,pData->uSelNow,pData->uSelEnd);
			}
		else{
//			CallWindowProc(pData->pOldProc,hWnd,EM_SETSEL,pData->uSize,0);
			}

		break;


	case WM_LBUTTONDBLCLK:									// Linke Maustaste Doppelclick

		if(!pData->bHasFocus)
			{
			SetFocus(hWnd);
			}
		
		iMax  = pData->uSize;
		pText = pData->pText;
		iOffA = ColorEditGetIndex(pData,hWnd,GET_X_LPARAM(lParam));
		iOffB = iOffA;

		while(iOffA>0    && strwchar(pText[iOffA-1],0x3000000F))iOffA--;
		while(iOffB<iMax && strwchar(pText[iOffB  ],0x3000000F))iOffB++;

		if((iOffB^pData->uSelNow) || (iOffA^pData->uSelEnd))	
			{
			iOldA = pData->uSelNow;
			iOldB = pData->uSelEnd;
			pData->uSelNow	= iOffB;
			pData->uSelEnd	= iOffA;

			goto UpdateMark;
			}

		return 0;

	
	case WM_LBUTTONDOWN:									// Linke Maustaste gedrckt

		if(!pData->bHasFocus)
			{
			SetFocus(hWnd);
			}
		
		pData->bLbDown = true;
		SetCapture(hWnd);

		iPosX = ColorEditGetIndex(pData,hWnd,GET_X_LPARAM(lParam),&iOff);

		if((iPosX^pData->uSelNow) || (iPosX^pData->uSelEnd))	
			{
			iOffA = pData->uSelNow;
			iOffB = pData->uSelEnd;
															// Nur Cursor verschieben
			if(pData->uSelNow==pData->uSelEnd && iOffA==iOffB)				
				{
				pData->uSelNow	  = iPosX;
				pData->uSelEnd	  = iPosX;
				pData->sMarks[0]  = iOff;
				pData->sMarks[1]  = iOff;
				pData->sCaretPosX = pData->sStart+iOff-pData->sScrOff;

				HideCaret(hWnd);
				SetCaretPos(pData->sCaretPosX,pData->sCaretPosY);
				ShowCaret(hWnd);
				pData->bShowCaret = true;
				}
			else{
				iOldA			  = iPosX;
				iOldB			  = iPosX;
				pData->uSelNow	  = iPosX;
				pData->uSelEnd	  = iPosX;
				}
			
			goto UpdateMark;
			}

		return 0;


	case WM_KEYDOWN:										// Tastendruck

		if(wParam==VK_CONTROL)return 0;
		if(wParam==VK_SHIFT  )return 0;

		iPosA = GetAsyncKeyState(VK_CONTROL)&0x8000;
		iPosB = GetAsyncKeyState(VK_SHIFT  )&0x8000;
		iOffA = pData->uSelNow;
		iOffB = pData->uSelEnd;
		iMax  = pData->uSize;


		switch((int)wParam)
			{
		case VK_HOME:	iOffA = 0;							// An den Anfang
						break;

		case VK_END:	iOffA = iMax;						// Ans Ende der Liste
						break;
		
		case VK_LEFT:	if(iOffA!=iOffB && !iPosB)			// Cursor nach Links
							{
							if(iOffB<iOffA)iOffA=iOffB;
							break;
							}
						
						if(iOffA>0)iOffA--;
						if(iPosA==0)break;

						pText = pData->pText;

						if(strwchar(pText[iOffA],0x3000000F))
							{
							while(iOffA>0 &&  strwchar(pText[iOffA-1],0x3000000F))iOffA--;
							}
						else if(strwchar(pText[iOffA],0x000000E0))
							{
							while(iOffA>0 &&  strwchar(pText[iOffA-1],0x000000E0))iOffA--;
							}
						else{
							while(iOffA>0 && !strwchar(pText[iOffA-1],0x300000EF))iOffA--;
							}

						break;

		case VK_RIGHT:	if(iOffA!=iOffB && !iPosB)			// Cursor nach Rechts
							{
							if(iOffB>iOffA)iOffA=iOffB;
							break;
							}
						
						if(iOffA<iMax)iOffA++;				
						if(iPosA==0)break;

						pText = pData->pText;

						if(iOffA>0 && strwchar(pText[iOffA-1],0x3000000F))
							{
							while(iOffA<iMax &&  strwchar(pText[iOffA],0x3000000F))iOffA++;
							}
						else if(iOffA<=0 || strwchar(pText[iOffA-1],0x000000E0))
							{
							while(iOffA<iMax &&  strwchar(pText[iOffA],0x000000E0))iOffA++;
							}
						else{
							while(iOffA<iMax && !strwchar(pText[iOffA],0x300000EF))iOffA++;
							}

						break;

		case VK_CLEAR:										// Ein Wort auswhlen
						pText = pData->pText;

						while(iOffB>0    && strwchar(pText[iOffB-1],0x3000000F))iOffB--;
						while(iOffA<iMax && strwchar(pText[iOffA  ],0x3000000F))iOffA++;

						if(iOffA==iOffB)return 0;
						iPosB = 1;
						break;

		case VK_INSERT:										// Text einfgen/kopieren
						     if(iPosA)ColorEditProc(hWnd,WM_COPY ,0,0);
						else if(iPosB)ColorEditProc(hWnd,WM_PASTE,0,0);
						
						return 0;
						
		case 'A':											// Ganzen Text auswhlen
						if(iPosA==0)return 0;
						if(iOffA==0 && iOffB==(int)pData->uSize)return 0;
						if(iOffB==0 && iOffA==(int)pData->uSize)return 0;

						iOffA = pData->uSize;
						iOffB = 0;
						iPosA = 0;
						iPosB = 1;

						break;
		
		case 'Z':											// Rckgnig
						if(iPosA)ColorEditProc(hWnd,WM_UNDO,0,0);

						return 0;

		case 'Y':											// Wiederherstellen
						if(iPosA)ColorEditProc(hWnd,EM_REDO,0,0);

						return 0;

		case 'X':											// Text ausschneiden	
						if(iPosA!=0)ColorEditProc(hWnd,WM_CUT,0,0);
						
						return 0;

		case 'C':											// Text kopieren
						if(iPosA)ColorEditProc(hWnd,WM_COPY,0,0);

						return 0;

		case VK_DELETE:										// Markierten Text ausschneiden/lschen
						     if(iPosB!=0)ColorEditProc (hWnd ,WM_CUT,0,0);
						else if(iPosA==0)ColorEditInput(pData,hWnd,1,NULL,0);
						
						return 0;

		default:											// Taste wird nicht verarbeitet
						return 0;
			}

		
		if(!iPosB)iOffB=iOffA;

		if((iOffA^pData->uSelNow) || (iOffB^pData->uSelEnd))	
			{
			iOldA	= pData->uSelNow;
			iOldB	= pData->uSelEnd;
			pData->uSelNow	= iOffA;
			pData->uSelEnd	= iOffB;

			if(pData->uSelNow==pData->uSelEnd && iOffA==iOffB)				
				{											// Nur Cursor verschieben
				if(ColorEditCheckScroll(pData,hWnd))
					{
					RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
					}
				else{
					pData->sCaretPosX = pData->sStart+pData->sMarks[0]-pData->sScrOff;
					HideCaret(hWnd);
					SetCaretPos(pData->sCaretPosX,pData->sCaretPosY);
					ShowCaret(hWnd);
					pData->bShowCaret = true;

					if(iOldA!=iOldB)
						{
						if(iOldA>iOldB)
							{
							iOff	= iOldA;
							iOldA	= iOldB;
							iOldB	= iOff;
							}
						
						ColorEditInvalidRect(pData,hWnd,iOldA,iOldB);
						}
					}
				}
			else{
				UpdateMark: 
				
				if(ColorEditCheckScroll(pData,hWnd))
					{	
					RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
					}
				else{
					if(iOldA>iOldB)
						{
						iOff	= iOldA;
						iOldA	= iOldB;
						iOldB	= iOff;
						}
					else if(iOldA==iOldB)
						{
						iOldA	= iOffA;
						iOldB	= iOffB;
						}

					if(iOldA>iOffA)iOldA=iOffA;
					if(iOldA>iOffB)iOldA=iOffB;
					if(iOldB<iOffA)iOldB=iOffA;
					if(iOldB<iOffB)iOldB=iOffB;

					ColorEditInvalidRect(pData,hWnd,iOldA,iOldB);
					}
				}
			
			SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(GetWindowLong(hWnd,GWL_ID),EN_SELCHANGE),(LPARAM)hWnd);
			}

		return 0;

	
	case WM_KEYUP:											// Taste loslassen

		return 0;

	
	case WM_CHAR:											// Zeicheneingabe

		if(wParam==VK_BACK)
			{
			ColorEditInput(pData,hWnd,2,NULL,0);
			}
		else if(wParam>=' ')
			{
			if((pData->uStyle&ES_NUMBER) && (wParam<'0' || wParam>'9'))
				{
				return 0;
				}

			ColorEditInput(pData,hWnd,0,(LPCTSTR)&wParam,1);
			}
		
		return 0;
			

	case WM_LBUTTONUP:										// Linke Maustaste losgelasen

		if(pData->bLbDown)
			{
			pData->bTimerOn	= false;
			pData->bLbDown  = false;
			ReleaseCapture();
			}

		return 0;


	case WM_MBUTTONUP:										// Mit mittler Maustaste Wort whlen

		if(!pData->bLbDown && !(wParam&(MK_CONTROL|MK_SHIFT)))
			{
			ColorEditProc(hWnd,WM_LBUTTONDBLCLK,wParam,lParam);
			}

		return 0;

	case WM_TIMER:											// Timer Signal empfangen

		if(wParam!=ID_SCROLL)break;
		if(pData->bTimerOn==false)
			{
			KillTimer(hWnd,ID_SCROLL);
			return 0;
			}

		GetCursorPos(&sPoint);
		ScreenToClient(hWnd,&sPoint);
		lParam = MAKELONG(sPoint.x,sPoint.y);


	case WM_MOUSEMOVE:										// Linke Maustaste losgelasen

		if(!pData->bLbDown)return 0;

		iPosA = GET_X_LPARAM(lParam);
															// Sind wir auserhalb des Fensters
		if(iPosA<=pData->sStart || iPosA>=pData->sWindowX)			
			{
			if(!pData->bTimerOn)
				{
				pData->bTimerOn = true;
				SetTimer(hWnd,ID_SCROLL,200,NULL);
				}
			else if(uMsg!=WM_TIMER)
				{
				return 0;
				}

			if(iPosA<=pData->sStart)						// Scroll Geschwindigkeit fr den Timer
				{
				iPosA -= pData->sStart;
				iPosA -= 8;
				iPosA /= 4;

				if(iPosA<-30)iPosA=-30;
				iPosA += pData->sStart;
				}
			else{
				iPosA -= pData->sWindowX;
				iPosA += 8;
				iPosA /= 4;

				if(iPosA>30)iPosA=30;
				iPosA += pData->sWindowX;
				}

			}
		else{
			pData->bTimerOn = false;
			}

		iPosX = ColorEditGetIndex(pData,hWnd,iPosA);

		if(iPosX^pData->uSelNow)	
			{
			iOldA = pData->uSelNow;
			pData->uSelNow	= iPosX;

			if(ColorEditCheckScroll(pData,hWnd,2))
				{
				RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
				}
			else{			
				if(iOldA>iPosX)
					{
					iOff	= iOldA;
					iOldA	= iPosX;
					iPosX	= iOff;
					}

				ColorEditInvalidRect(pData,hWnd,iOldA,iPosX);
				}

			SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(GetWindowLong(hWnd,GWL_ID),EN_SELCHANGE),(LPARAM)hWnd);
			}

		return 0;


	case WM_ENABLE:											// Wurde das Fenster freigegeben

		pData->bEnable = wParam!=0;

		break;


	case WM_DROPFILES:										// Datei an Edit-Fenster weiter leiten

		if(!(pData->uStyle&ES_ALLOWDROP))break;

	case EM_DROPFILES:
	
		hDrop = (HDROP)wParam;
		
		if(DragQueryFile(hDrop,-1,0,0)>=1)
			{
			TCHAR	cFile[MAX_PATH];
			HWND	hParent;	
			UINT	uMyId;
			INT		iNum;

			hParent	= GetParent(hWnd);
			uMyId	= GetWindowLong(hWnd,GWL_ID);
			iNum	= DragQueryFile(hDrop,0,cFile,MAX_PATH);
			
			if(iNum>0)
				{
				pData->uSelNow = 0;
				pData->uSelEnd = pData->uSize;
				ColorEditInput(pData,hWnd,0,cFile,iNum);

				PostMessage(hParent,WM_COMMAND,MAKELONG(uMyId,EN_CHANGE   ),(LPARAM)hWnd);
				PostMessage(hParent,WM_COMMAND,MAKELONG(uMyId,EN_DROPFILES),(LPARAM)hWnd);
				}
			}

		DragFinish(hDrop);

		return 0;

	case WM_SETREDRAW:

		pData->bLockDraw = !wParam;
		break;


	case WM_PAINT:											// Zeichnen des Fensters 

		if(pData->bLockDraw)								
			{
			return 0;
			}
		
		if(!pData->aFont[0])
			{
			pData->aFont[0]		= GetDefaultFont();
			pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);
			ColorEditCheckScroll(pData,hWnd);
			}

		GetClientRect(hWnd,&sRect);
		pData->sWindowX = (short)sRect.right;

		uBits	= DT_SINGLELINE|DT_NOPREFIX;

		if(pData->bShowCaret)
			{
			HideCaret(hWnd);
			}

		hDc		= BeginPaint(hWnd,&sPaint);

		if(sPaint.rcPaint.left<sPaint.rcPaint.right)
			{
			hRrgn	= CreateRectRgn(sPaint.rcPaint.left,sPaint.rcPaint.top,sPaint.rcPaint.right,sPaint.rcPaint.bottom);
			SelectObject(hDc,hRrgn);
			}
		else{
			hRrgn	= NULL;
			}

															// Fenster grau machen
		if((!pData->bEnable || (pData->uStyle&ES_READONLY)) && !(pData->uStyle&ES_NOGRAYDISABLE))
			{
			SetBkColor(hDc,GetSysColor(COLOR_MENU));
			SendMessage(GetParent(hWnd),WM_CTLCOLORSTATIC,(WPARAM)hDc,(LPARAM)hWnd);
			iBits = ILD_TRANSPARENT|ILD_BLEND50;
			}
		else{
			SetBkColor(hDc,GetSysColor(COLOR_WINDOW));
			SendMessage(GetParent(hWnd),WM_CTLCOLOREDIT  ,(WPARAM)hDc,(LPARAM)hWnd);
			iBits = ILD_TRANSPARENT;
			}
		
		
		if(pData->bIconX)									// Ein Icon zeichnen
			{
			iEnd		= sRect.right;		
			sRect.right	= sRect.left+pData->bIconX+2;

			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sRect,NULL,0,NULL);

			if(pData->iIcon>=0)
				{		
				iPosY	= (sRect.bottom+sRect.top-pData->bIconY)/2;if(iPosY<sRect.top)iPosY=sRect.top;	
				ImageList_Draw(pData->hImgages,pData->iIcon,hDc,sRect.left+1,iPosY,iBits);
				}

			sRect.left	= sRect.right;
			sRect.right	= iEnd;
			}

		iOff	= pData->uScroll;
		uSize	= pData->uSize;
		iPosX	= sRect.left+1;
		iPosY	= (sRect.bottom-sRect.top-pData->bFontHeigh+2)/2;if(iPosY<0)iPosY=0;
		
		if(iOff==0 && (pData->uStyle&(ES_RIGHT|ES_CENTER)))	// Den Text anderst ausrichten
			{
			   iMax = sRect.right-sRect.left-pData->sTextSize;
			if(iMax>4)			
				{
				if(pData->uStyle&ES_RIGHT)
					{
					iPosX += iMax-1;
					}
				else if(pData->uStyle&ES_CENTER)
					{
					iPosX += iMax/2;
					}
				}
			}

		pData->sStart	  = iPosX;
		pData->sCaretPosX = iPosX+pData->sMarks[0]-pData->sScrOff;
		pData->sCaretPosY = iPosY;

															// Gibt es eine Markierung im Text
		if(pData->uSelNow!=pData->uSelEnd && (pData->bHasFocus || ((pData->uStyle&ES_NOHIDESEL) && !(pData->uStyle&ES_COMBO))))					
			{
			iPosA	= iPosX+pData->sMarks[0]-pData->sScrOff;
			iPosB	= iPosX+pData->sMarks[1]-pData->sScrOff;
			iOffA	= pData->uSelNow;
			iOffB	= pData->uSelEnd;
			
			if(iPosA>iPosB)
				{
				iEnd  = iPosA;
				iPosA = iPosB;
				iPosB = iEnd;
				iEnd  = iOffA;
				iOffA = iOffB;
				iOffB = iEnd;
				}

			iEnd = sRect.right;
															// Den linken Teil zeichnen
			if(iOffA>iOff)
				{
				sRect.right = iPosA;
				iPosX		= ExtTextColor(hDc,iPosX,iPosY,pData->uOption,&sRect,pData->pText+iOff,iOffA-iOff,pData->pColors+iOff,pData->pColTable,pData->aFont);
				sRect.left  = iPosA;
				iOff		= iOffA;
				}

			if(iOffB>iOff)									// Den maekierten Teil zeichnen
				{
				BYTE		aColors[TEXT_MAX];
				COLORREF	aColDat[32];

				aColDat[31]	= GetSysColor((pData->bHasFocus)? COLOR_HIGHLIGHT:COLOR_BTNSHADOW);
				aColDat[ 1]	= GetSysColor(COLOR_HIGHLIGHTTEXT);

				memset(aColors,0x11,iOffB-iOff);

				for(iStep=iOff;iStep<iOffB;iStep++)			// Schrift kopieren	
					{
					aColors[iStep-iOff] |= pData->pColors[iStep]&0xC0;
					}

				sRect.right = iPosB;
				iPosX		= ExtTextColor(hDc,iPosX,iPosY,ETO_OPAQUE|ETO_BKCOLOR_1|ETO_FONTCNT_2,&sRect,pData->pText+iOff,iOffB-iOff,aColors,aColDat,pData->aFont);
				sRect.left  = iPosB;
				iOff		= iOffB;
				}

			if(sRect.left<iEnd)								// Den rechten Teil zeichnen
				{
				sRect.right = iEnd;
				ExtTextColor(hDc,iPosX,iPosY,pData->uOption,&sRect,pData->pText+iOff,uSize-iOff,pData->pColors+iOff,pData->pColTable,pData->aFont);
				}
			}
		else{
			ExtTextColor(hDc,iPosX,iPosY,pData->uOption,&sRect,pData->pText+iOff,uSize-iOff,pData->pColors+iOff,pData->pColTable,pData->aFont);
			}
		
		EndPaint(hWnd,&sPaint);
		if(hRrgn)DeleteObject(hRrgn);


		if(pData->bHasFocus && pData->sCaretPosX>=0 && pData->sCaretPosX<=pData->sWindowX)
			{
			pData->bShowCaret = true;
			SetCaretPos(pData->sCaretPosX,pData->sCaretPosY);
			ShowCaret(hWnd);
			}

		return 0;	
			

	case EM_GETLIMITTEXT:									// Maximale Textgre abfragen

		return pData->uMaxSize;


	case EM_SETLIMITTEXT:									// Maximale Textgre ndern

		   uSize = wParam;
		if(uSize>TEXT_MAX)
			{
			uSize = TEXT_MAX;
			ASSERT(0);
			}

		if(pData->uMaxSize==uSize)
			{
			return 0;
			}

		if(pData->uSize>uSize)
			{
			pData->uSize = uSize;
			if(pData->uSelNow>uSize)pData->uSelNow=uSize;
			if(pData->uSelEnd>uSize)pData->uSelEnd=uSize;

			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}

		pByte			= new BYTE[uSize*(1+sizeof(TCHAR))+8];
		pText			= (LPTSTR)(pData->pColors+((pData->uMaxSize+2)&~1));

		memcpy(pByte,pData->pColors,pData->uSize);
		memcpy(pText,pData->pText, (pData->uSize+1)*sizeof(TCHAR));

		delete pData->pColors;

		pData->bChanged = false;
		pData->pColors	= pByte;
		pData->pText	= pText;

		ColorEditCheckScroll(pData,hWnd);
		RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
		
		break;


	case EM_GETSEL:											// Cursor und Auswahl abfragen

		return MAKELONG(pData->uSelEnd,pData->uSelNow);


	case EM_SETSEL:											// Cursor und Auswahl setzen

		uSize = (unsigned)wParam;
		uBits = (unsigned)lParam;

		if(uSize==0xFFFFFFFF)uSize=pData->uSize;
		if(uBits==0xFFFFFFFF)uBits=pData->uSize;

		if(uSize>pData->uSize)uSize=(uSize&0x80000000)? 0:pData->uSize;
		if(uBits>pData->uSize)uBits=(uBits&0x80000000)? 0:pData->uSize;
		
		if(pData->uSelNow==uSize && pData->uSelEnd==uBits)
			{
			return 0;
			}

		pData->uSelNow = uSize; 
		pData->uSelEnd = uBits;

		ColorEditCheckScroll(pData,hWnd);
		RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
		SendMessage(GetParent(hWnd),WM_COMMAND,MAKELONG(GetWindowLong(hWnd,GWL_ID),EN_SELCHANGE),(LPARAM)hWnd);

		return 0;


	case EM_SETCOLOR:										// Die Farben zu den Buchstaben einstellen

		if(!lParam || wParam>=0x8000000)return FALSE;

		EDIT_LOCK(pData)

		   uSize = (UINT)wParam;
		if(uSize>pData->uMaxSize)
			{
			uSize = pData->uMaxSize;
			}
		
		pByte = (LPBYTE)lParam;				
		uFlag = 0;

		for(iPosX=uSize-1;iPosX>=0;iPosX--)					// Gibt es nderungen
			{
			bItem = pByte[iPosX]&0x7F;
			bDiff = bItem^pData->pColors[iPosX];

			if(!bDiff)continue;

			uFlag				 |= bDiff;
			pData->pColors[iPosX] = bItem;
			}

		EDIT_UNLOCK(pData)
		
		if(uFlag)											// Hat sich was verndert
			{
			if(uFlag&0x40)									// Schrift verndert
				{
				if(!pData->aFont[0])
					{
					pData->aFont[0]		= GetDefaultFont();
					pData->bFontHeigh	= GetFontHeight(hWnd,pData->aFont[0]);
					}

				if(!pData->aFont[1])
					{
					pData->aFont[1]		= GetBoldFont(pData->aFont[0]);
					}

				ColorEditCheckScroll(pData,hWnd,TRUE);
				}
			
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}

		return TRUE;
	

	case EM_SETCOLORMODE:									// Die Farboptionen einstellen

		   uBits = wParam|ETO_OPAQUE|ETO_FONTCNT_2;
		if(uBits!=pData->uOption)
			{
			pData->uOption = uBits;
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}

		return TRUE;


	case EM_SETCOLORITEM:									// Einen Farbeintrag einstellen

		if(!pData->pColTable)
			{
			pData->pColTable = new COLORREF[32];
			memset(pData->pColTable,0,sizeof(COLORREF)*32);
			}

		iMax = HIWORD(wParam);
		iOff = LOWORD(wParam);
					
		if(iMax)											// Liste mit Farben
			{
			if(iOff>=32)return 0;
			if(iOff+iMax>32)iMax=32-iOff;
			
			if(memcmp(pData->pColTable+iOff,(void*)lParam,sizeof(COLORREF)*iMax))
				{
				memcpy(pData->pColTable+iOff,(void*)lParam,sizeof(COLORREF)*iMax);
				RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
				}
			}
		else{												// Einzelne Farbe
			if(iOff>=32)return 0;
			if(pData->pColTable[iOff]!=(COLORREF)lParam)
				{
				pData->pColTable[iOff] = lParam;
				RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
				}
			
			iMax = 1;
			}
		
		return iMax;


	case EM_SETIMAGELIST:									// Die Iconliste einstellen

		if(!lParam)											// Liste lschen
			{
			pData->bIconX = 0;
			pData->bIconY = 0;
			}
		else{												
			IMAGEINFO sInfo;
			
			ImageList_GetImageInfo((HIMAGELIST)lParam,0,&sInfo);

			pData->bIconX = (BYTE)(sInfo.rcImage.right -sInfo.rcImage.left);
			pData->bIconY = (BYTE)(sInfo.rcImage.bottom-sInfo.rcImage.top );
			}									

		pData->hImgages = (HIMAGELIST)lParam;
		RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);

		return TRUE;


	case EM_SETIMAGE:										// Das Icon einstellen

		if(pData->iIcon!=(int)wParam)
			{
			pData->iIcon = (int)wParam;
			if(pData->bIconX)RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}

		return TRUE;

	
	case EM_SETCALLBACK:									// Callback fr Farben

		pData->pCallback = (ColorEditCb)lParam;	
		pData->pCbParam  = (LPVOID)wParam;

		return TRUE;


	case EM_SCROLL:											// Horizonal scrollen
	  
		if(HIWORD(wParam)!=0x1234)break;

		if(LOWORD(wParam))									// Pixel
			{
			hDc		= GetDC(hWnd);
			iPosX	= (int)lParam;
			
			if(iPosX<0)iPosX=0;

			iStep	= pData->sScrOff;
			uFlag	= GetTextExtentEx(hDc,pData->pText,pData->uSize,iPosX+2,&iMax,NULL,&sSize,pData->pColors,pData->aFont);

			ReleaseDC(hWnd,hDc);
			}
		else{												// Buchstaben
			uFlag	= 1;
			iMax	= (int)lParam;
			}

		if(uFlag && pData->uScroll^iMax)					// Hat sich die Scroll-Position verndert
			{
			if(pData->bShowCaret)
				{
				pData->bShowCaret = 0;
				HideCaret(hWnd);
				}
			
			pData->uScroll = iMax;
			ColorEditCheckScroll(pData,hWnd,1);

			iStep  -= pData->sScrOff;
			pData->sCaretPosX += iStep;

			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);
			}
		
		return TRUE;


	case EM_GETTEXTPTR:										// Einen Pointer direkt auf den Text holen

		return (LPARAM)pData->pText;


	case EM_GETIMAGE:										// Die Icon-Nummer holen

		return pData->iIcon;


	case EM_GETCOLORPTR:									// Einen Pointer direkt auf die Farben holen
		
		return (LPARAM)pData->pColors;


	case EM_SYNCHWINDOW:									// Die Daten mit dem unterliegenden Fenster synchronisieren

		if(pData->bChanged)
			{
			pData->bChanged = false;
			CallWindowProc(pData->pOldProc,hWnd,WM_SETTEXT,0,(LPARAM)pData->pText);
			}

		CallWindowProc(pData->pOldProc,hWnd,EM_SETSEL,pData->uSelNow,pData->uSelEnd);

		return 0;
		}


return CallWindowProc(pData->pOldProc,hWnd,uMsg,wParam,lParam);
}

