//*****************************************************************************
//*
//*
//*		DataWnd.h: Schnittstelle fr die Klasse CDataWnd
//*
//*
//*****************************************************************************
//
//	Wird im Daten-Fester die TAB-Taste getdrckt so wird eine Nachricht 
//	WM_DATAWNDTAB abgeschickt
//	Diese kann mit ON_WM_DATAWNDTAB(id,memberFxn)  abgefangen werden
//			id			: ID von CDataWnd
//			memberFxn	: Aufzurufende Funktion 
//	Die Standard Id ist IDW_DATAWND
//	Die Funktion fr ON_WM_DATAWNDTAB hat die Form Funktion(int Id,long Offset)
//	Anmerkungen die DataWnd Id's sollten nicht grer als 0x3FF sein.
//
//
//	Funktionen: 
// 
//		void	SetMem   (char*,int,int,int,int,int)	: Speicher einstellen
// 		int		SetSel   (int  iNum =0)					: Aktives Fenster einstellen
// 		int		SetCount (int  iCount  )				: Anzahl der Scanner einstellen
// 		BOOL	SetStyle (...)							: Anzeige einstellen
// 		BOOL	SetStyle (DWORD ,BOOL)					: Anzeige einstellen
// 		void	ReadData (BOOL bRedraw=true,int iNum=-1): Fensterdaten einlesen
// 		int		MaxLines (int   iLines=4096         )	: Maximale Zeilen Anzahl
//		void    LoadDlg  (int   iMode,char *pRegKey  )	: Modul mit Dialog laden
// 		BOOL	LoadModul(char *pFile,char *Name=0  )	: Modul laden
// 		int 	LoadList (char *pRegKey="DataWindow")	: ModulListe laden	
// 		int		AddToList(char *pFile  ,char *pRegKey)	: Datei zur	ModulListe
//		int		DelFromList(char *pFile,char *pRegKey)	: Modul aus Liste lschen
//		int		PropertyDlg()							: Eigenschaften einstellen
//		void	DeleteModul()							: Module lschen
//		BOOL	Create (int iLabels,DWORD dwStyle ,		: Fenster erzeugern
//					   const RECT& sRect,CWnd*,UINT)
// 		
// 		int		SetWidth(int      iWidth      )			: Laschenbreiten auf einmal setzen
// 		int		SetWidth(int,int  iWidth ,BOOL)			: Einzelne Laschenbreite setzen
// 		int		SetWidth(int,int *pWidths,BOOL)			: Laschenbreiten ber Feld setzen 
// 		int		GetWidth(int iLabel)					: Laschenbreite lesen 
// 		char   *GetText (int iLabel)					: Alle  Laschennamen lesen
// 		int	 	SetText (char      **,BOOL)				: Alle  Laschennamen setzen mit Kette 
// 		int	 	SetText (char       *,BOOL)				: Alle  Laschennamen setzen 
// 		int	 	SetText (CString     ,BOOL)				: Alle  Laschennamen setzen 
// 		int	 	SetText (int,char  **,BOOL)				: Alle  Laschennamen setzen mit Feld
// 		int	 	SetText (int,char   *,BOOL)  			: Einen Laschennamen setzen 
// 		int	 	SetText (int,CString ,BOOL)  			: Einen Laschennamen setzen 
// 
#ifndef	AFX_DATAWND_H__CD0BF773_BE04_11D3_9041_56915ADDED5C__INCLUDED_
#define AFX_DATAWND_H__CD0BF773_BE04_11D3_9041_56915ADDED5C__INCLUDED_

#include	<Afx.h>
#include	<AfxPriv.h>

#if _MSC_VER >= 1000
#pragma once
#endif 

#include	"FastListCtrl.h"
#include	"DataScanner.h"
#include	"LabelCtrl.h"

//*****************************************************************************
//*
//*		Konstanten
//*
//*****************************************************************************

#define 	IDW_DATAWND			0xABAA					// Databar Standard ID
#define 	CDW_LABELS			0x0001					// Laschen anzeigen
#define 	CDW_MODULLIST		0x0002					// ModulListe Loaden
#define 	CDW_NOMODULDLG		0x0004					// Keine Laden der Module ber einen Dlg
#define 	CDW_NOPROPERTYS 	0x0008					// Keine Eigenschafts Men
#define 	CDW_ADDRES_ON		0x0010					// Adress Anzeige
#define 	CDW_OFFSET_ON		0x0020					// Offset Anzeige
#define 	CDW_LENGTH_ON		0x0040					// Lngen Anzeige
#define 	CDW_FIXEDFONT		0x0080					// Fixe Schriftzeichen Breite
#define 	CDW_LISTBORDER		0x0100					// Rahmen um List Fenster
#define 	CDW_LISTLINES 		0x0200					// Linien im List Fenster
#define 	CDW_LOADCONFIG		0x0400					// Standard Konfiguration laden
#define 	CDW_BLOCKNAMES		0x0800					// Labels auf Blocknamen setzen
#define 	CDW_ALTERNATE		0x1000					// Abwechslelnde Farben
#define		CDW_MAX				16						// Maximal Anzahl der Fenster		

#define		WM_DATAWNDTAB	0x4800			  
#define		ON_WM_DATAWNDTAB(id,memberFxn)								\
		{   WM_DATAWNDTAB|(id&0x03FF), 0 ,0,0, AfxSig_vwii,				\
		(AFX_PMSG)(AFX_PMSGW)(void (AFX_MSG_CALL CWnd::*)(int,long))&memberFxn },

//*****************************************************************************
//*
//*		Klasse
//*
//*****************************************************************************
class CDataWnd : public CWnd
{
public:

	CDataWnd();
	virtual ~CDataWnd();
	
	virtual	BOOL Create(int     iLabels,DWORD dwStyle ,const RECT &sRect,CWnd *pParentWnd=0,UINT nID=IDW_DATAWND);
	int		GetSel	   (              );
	int		SetSel     (int     iNum  );
	int		SetCount   (int     iCount);
	void	SetMem     (char   *pMem  ,int iMax   ,int iOff= 0,int iLen=-1,LONGLONG iAdr=0,int iNum=-1);
	BOOL	SetStyle   (DWORD   uFlags,BOOL bRedraw=1);
	BOOL	SetStyle   (int     iOff,int iLen,int iAdr,int iFix=-1,int iLin=-1,int iAlt=-1,BOOL bRedraw=1);
	BOOL	LoadModul  (LPCTSTR pFile ,LPCTSTR pName=0);
	void    LoadDlg    (int     iMode ,LPCTSTR pRegKey=_T("DataWindow"));
	int 	LoadList   (               LPCTSTR pRegKey=_T("DataWindow"));
	int		AddToList  (LPCTSTR pFile ,LPCTSTR pRegKey=_T("DataWindow"));
	int		DelFromList(LPCTSTR pFile ,LPCTSTR pRegKey=_T("DataWindow"));
	int 	LoadConfig (               LPCTSTR pRegKey=_T("DataWindow"));
	int 	SaveConfig (               LPCTSTR pRegKey=_T("DataWindow"));
	void	ReadData   (BOOL	bRedraw=true,int iNum=-1);
	int		Copy	   (int		iMode=0);
	int		MaxLines   (int		iLines=4096);
	int		PropertyDlg(); 
	void	DeleteModul();

	int		SetFlags   (int		iFlags);
	int		SetWidth   (int		iWidth);
	int		SetWidth   (int   	iLabel, int         iWidth ,BOOL bRedraw=TRUE);
	int		SetWidth   (int   	iCount, const int  *pWidths,BOOL bRedraw=TRUE);
	int		GetWidth   (int   	iLabel);
	LPCTSTR GetText    (int   	iLabel);
	int		SetText    (      	        LPCTSTR     *pChain,BOOL bRedraw=TRUE);
	int		SetText    (      	        LPCTSTR      pText ,BOOL bRedraw=TRUE);
	int		SetText    (int   	iCount ,LPCTSTR     *pTexte,BOOL bRedraw=TRUE);
	int		SetText    (int   	iLabel ,LPCTSTR      pText ,BOOL bRedraw=TRUE);
	
protected:

	void	Expand(int iItem,int iMode=0);

	//{{AFX_MSG(CDataWnd)
	afx_msg LRESULT OnCopy(WPARAM wParam,LPARAM lParam);
	afx_msg void OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags);
	afx_msg void OnSize( UINT nType, int iSizeX, int iSizeY);
	afx_msg void OnListview(NMHDR *pNmHdr, LRESULT *pResult);
	afx_msg void OnClick   (NMHDR *pNmHdr, LRESULT *pResult);
	afx_msg void OnLabel   (NMHDR *pNmHdr, LRESULT *pResult);
	afx_msg void OnLoadModul();
	afx_msg void OnLoadListe();
	afx_msg void OnAddToList();
	afx_msg void OnCopyMouse();
	afx_msg void OnLastView();
	afx_msg void OnNextView();
	afx_msg void OnDelModul();
	afx_msg void OnDestroy();
	afx_msg void OnAdress();
	afx_msg void OnLines();
	afx_msg void OnLength();
	afx_msg void OnOffset();
	afx_msg void OnFixFont();
	afx_msg void OnReadData();
	afx_msg void OnAlternate();
	afx_msg void OnPropertyDlg();
	afx_msg void OnDelFromList();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
	
	static	LRESULT	CALLBACK ListProc(WNDPROC pProc,HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam,CWnd *pWnd);
	static	int		CALLBACK LineCallback(LPVOID pParam,LPARAM lItem,UINT uStart,UINT uEnd,FL_ENTRY *pEntries);
	
//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************

protected:
	
	CLabelCtrl		m_cLabelCtrl;						// Laschen Fenster
	CFastListCtrl	m_aListWnd[CDW_MAX];				// Listen Fenster
	CDataScanner	m_sScanner;							// Scanner
	CPoint			m_sContextPt;						// Letzter Context-Men-Punkt		
														
	int				m_iSizeX;							// Aktuelle Fenster Breiten
	int				m_iSizeY;							//		*
	int				m_iFlags;							// Flags 
	int				m_iCount;							// Anzahl der Scanner
	int				m_iScanSel;							// Aktueller Scanner 
														
	LONGLONG		m_aMemAddr [CDW_MAX];				// Speicher Startadresse
	char		   *m_aMemPtr  [CDW_MAX];				// Speicher Zeiger
	int				m_aMemOff  [CDW_MAX];				// Speicher Offset
	int				m_aMemLen  [CDW_MAX];				// Speicher Lnge
	int				m_aMemMax  [CDW_MAX];				// Speicher maximal Lnge
	
	BOOL			m_bLockPaint;						// Aussetzen des Zeichnens
	BOOL 			m_bOffsetOn;						// Offset Anzeige ein/aus
	BOOL			m_bLengthOn;						// Lngen Anzeige ein/aus
	BOOL			m_bAddressOn;						// Adress Anzeige ein/aus
	BOOL			m_bFixFontOn;						// Fixer Font
	BOOL			m_bLinesOn;							// Linien im Dadenfenster
	BOOL			m_bAltLineOn;						// Abwechselnde Farben	
	int				m_iSaveCfg;							// Konfiguration speichern
	int				m_iMaxLines;						// Maximale Linien Anzahl
	int				m_aBlockNum[CDW_MAX];				// Aktuelle Block-Nummer
	int				m_aModulNum[CDW_MAX];				// Aktuelle Modul-Nummer
														
private:												
														
static	const int	m_aAreaSizes[5];					// Abschnitts Breiten
static	LPTSTR		m_pClassName;						// Klassenname
static	BOOL		m_bInitDone;						// Gab es eine Initialisierung
static	BYTE		m_uHeighVar;						// Variabler Font
static	BYTE		m_uHeighFix;						// Hhe vom fixen Font
static	HFONT		m_hTextVar;							// Hhe vom variablen Font
static	HFONT		m_hTextFix;							// Fixer Font
static	CPen		m_sPen,m_sPen1;						// Graue Linie
static	int			m_iWndCount;						// Anzahl der Fenster
static	int			m_iScrollX;							// Breite von Scrollleisten
static	int			m_iScrollY;							// Hhe   von Scrollleisten
		int 		m_iInit;							// Sind Fenster initialisiert
														
};


//*****************************************************************************
//*
//*		Klasse CDataBarDlg
//*
//*****************************************************************************
class CDataBarDlg : public CDialog
{
public:

	CDataBarDlg(CWnd *pParent = NULL);					// Standardkonstruktor
			
	//{{AFX_DATA(CDataBarDlg)
	BOOL			m_bOffsetOn;
	BOOL			m_bLengthOn;
	BOOL			m_bAddressOn;
	BOOL			m_bFixFontOn;
	BOOL			m_bAltLineOn;
	BOOL			m_bLinesOn;
	int				m_iMaxLines;
	int				m_aMemLen;
	//}}AFX_DATA

	CDataScanner   *m_pScanner;							// Daten Scanner
	int				m_aMemOff;							// Speicher Offset
	int				m_aBlockNum;						// Aktuelle Block-Nummer
	int				m_aModulNum;						// Aktuelle Modul-Nummer
														
	//{{AFX_VIRTUAL(CDataBarDlg)
public:
	virtual INT_PTR DoModal();
protected:
	virtual void DoDataExchange(CDataExchange *pDX);    // DDX/DDV-Untersttzung
	//}}AFX_VIRTUAL

protected:

	void		LoadBlocks(int Modul);

	CEdit	   *m_pEdit;
	CComboBox  *m_pCombo1;
	CComboBox  *m_pCombo2;
	static WORD	m_wDlgData[];

	//{{AFX_MSG(CDataBarDlg)
	afx_msg void OnSelchangeCombo2();
	afx_msg void OnSelchangeCombo1();
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

//*****************************************************************************
//*
//*		Klasse CDataBarErrorDlg
//*
//*****************************************************************************
class CDataBarErrorDlg : public CDialog
{
public:

	CDataBarErrorDlg(CWnd *pParent = NULL);   			// Standardkonstruktor


	LPTSTR	 m_pText;
	CString	 m_sTitle;									// Titel des Dialogs
	
	//{{AFX_VIRTUAL(CDataBarErrorDlg)
	//}}AFX_VIRTUAL

protected:

	//{{AFX_MSG(CDataBarErrorDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG

	static WORD		m_wDlgData[];
};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

inline	int		CDataWnd::SetWidth(int B                    ){return m_cLabelCtrl.SetWidth(B    );}
inline	int		CDataWnd::SetWidth(int L,int  B      ,BOOL R){return m_cLabelCtrl.SetWidth(L,B,R);}
inline	int		CDataWnd::SetWidth(int A,const int *B,BOOL R){return m_cLabelCtrl.SetWidth(A,B,R);}
inline	int		CDataWnd::GetWidth(int L                    ){return m_cLabelCtrl.GetWidth(L    );}
inline	LPCTSTR CDataWnd::GetText (int L                    ){return m_cLabelCtrl.GetText (L    );}
inline	int		CDataWnd::SetText (       LPCTSTR  *K,BOOL R){return m_cLabelCtrl.SetText (  K,R);}
inline	int		CDataWnd::SetText (       LPCTSTR   T,BOOL R){return m_cLabelCtrl.SetText (  T,R);}
inline	int		CDataWnd::SetText (int A ,LPCTSTR  *T,BOOL R){return m_cLabelCtrl.SetText (A,T,R);}
inline	int		CDataWnd::SetText (int L ,LPCTSTR   T,BOOL R){return m_cLabelCtrl.SetText (L,T,R);}
inline	int		CDataWnd::SetFlags(int iFlags               ){return m_sScanner.SetFlags(iFlags );}
inline	int		CDataWnd::GetSel  (                         ){return m_iScanSel                  ;}

#endif 


