//*****************************************************************************
//*
//*
//*		FastListCtrl.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//	This code creates a Tree-List-Control for MFC
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<Afx.h>
#include	<tchar.h>
#include	"FastListCtrl.h"



IMPLEMENT_DYNAMIC(CFastListCtrl, CWnd)



//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
//	Creates a new Tree-List-Control
//	dwStyle		: Style of the Window
//					LVS_CHECKBOXES
//					LVS_DISABLEDRAGDROP
//					LVS_EDITLABELS
//					LVS_FULLROWSELECT
//					LVS_HASBUTTONS
//					LVS_HASLINES
//					LVS_INFOTIP
//					LVS_LINESATROOT
//					LVS_NONEVENHEIGHT
//					LVS_NOSCROLL
//					LVS_NOTOOLTIPS
//					LVS_RTLREADING
//					LVS_SHOWSELALWAYS
//					LVS_SINGLEEXPAND
//					LVS_TRACKSELECT
//	sRect		: Rectangle wher the window should be shown
//	pParentWnd	: Pointer to parent window
//	nId			: Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CFastListCtrl::Create(DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nId)
{

	FastListRegister(AfxGetInstanceHandle());


return CWnd::Create(_T("FastList"),NULL,dwStyle|WS_HSCROLL|WS_VSCROLL,sRect,pParentWnd,nId);
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Creates a new Tree-List-Control
//	dwExStyle	: Is the exdended style of the window (WS_EX_????)
//	dwStyle     : Style of the Window
//                	LVS_CHECKBOXES
//                	LVS_DISABLEDRAGDROP
//                	LVS_EDITLABELS
//                	LVS_FULLROWSELECT
//                	LVS_HASBUTTONS
//                	LVS_HASLINES
//                	LVS_INFOTIP
//                	LVS_LINESATROOT
//                	LVS_NONEVENHEIGHT
//                	LVS_NOSCROLL
//                	LVS_NOTOOLTIPS
//                	LVS_RTLREADING
//                	LVS_SHOWSELALWAYS
//                	LVS_SINGLEEXPAND
//                	LVS_TRACKSELECT
//	sRect		: Rectangle wher the window should be shown
//	pParentWnd  : Pointer to parent window
//	nId         : Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CFastListCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nId)
{

    FastListRegister(AfxGetInstanceHandle());

return CWnd::CreateEx(dwExStyle,_T("FastList"),NULL,dwStyle|WS_CHILD|WS_HSCROLL|WS_VSCROLL,sRect,pParentWnd,nId);
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Creates a new Tree-List-Control over a dialog item
//	dwExStyle	: Is the exdended style of the window (WS_EX_????)
//	dwStyle     : Style of the Window
//                	LVS_CHECKBOXES
//                	LVS_DISABLEDRAGDROP
//                	LVS_EDITLABELS
//                	LVS_FULLROWSELECT
//                	LVS_HASBUTTONS
//                	LVS_HASLINES
//                	LVS_INFOTIP
//                	LVS_LINESATROOT
//                	LVS_NONEVENHEIGHT
//                	LVS_NOSCROLL
//                	LVS_NOTOOLTIPS
//                	LVS_RTLREADING
//                	LVS_SHOWSELALWAYS
//                	LVS_SINGLEEXPAND
//                	LVS_TRACKSELECT
//	uItem       : Is the ID of the dialog item
//	pDlg		: Pointer to parent dialog
//	nId			: Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CFastListCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,UINT uItem,CDialog *pDlg,UINT nId)
{
CPoint	sPoint(0,0);
CRect	sRect;
CWnd   *pWnd;


	pWnd=pDlg->GetDlgItem(uItem);
	ASSERT(pWnd!=0);


	pWnd->GetWindowRect (&sRect);
	pDlg->ClientToScreen(&sPoint);

	sRect.left	  -= sPoint.x;
	sRect.right   -= sPoint.x;
	sRect.top	  -= sPoint.y;
	sRect.bottom  -= sPoint.y;

	if(!CFastListCtrl::CreateEx(dwExStyle,dwStyle|WS_CHILD|WS_VISIBLE|WS_HSCROLL|WS_VSCROLL,sRect,pDlg,nId))
		{
		return FALSE;
		}

	SetWindowPos(pWnd,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOREDRAW);


return TRUE;
}

//*****************************************************************************
//*
//*		InsertColumn
//*
//*****************************************************************************
//	Inserts a column in the header of the Tree-List-Control
//	nCol		: Position where the column should be inserted (use -1 for end)
//	pColumnText	: Text for the column
//	nFormat		: Ist the format for the text
//					LVCFMT_CENTER
//					LVCFMT_IMAGE
//					LVCFMT_RIGHT
//					LVCFMT_LEFT
//					LVCFMT_MARK
//	nWidth		: Is the width for the column in pixels.
//				  A negative value means is a autoexpanting column.
//				  The value of abs(nWidth) is the weight of the collumn.
//	nMin		: Ist the minimal size for the column.
//				  A negative value means abs(nMin)  the minimal size
//				  is only for dynamic size changing.
//				  Use 0x8000 for no minimal size
//	Returns the position of the column or -1 if an error occurs
int CFastListCtrl::InsertColumn(int nCol,LPCTSTR pColumnText,int nFormat,int nWidth,int nMin)
{
LV_COLUMN	sItem;


	ASSERT(::IsWindow(m_hWnd));

	sItem.mask = 0;

	if(pColumnText)
		{
		sItem.mask		|=  LVCF_TEXT;
		sItem.pszText	 =  (LPTSTR)pColumnText;
		sItem.cchTextMax =  256;
		}

	if(nFormat>=0)
		{
		sItem.mask		|=  LVCF_FMT|LVCF_MARK;
		sItem.fmt		 =  nFormat;
		}

	if(nMin!=0x8000)
		{
		sItem.mask		|=  LVCF_MIN;
		sItem.iOrder	 =  nMin;
		}

	if(nWidth>=0)
		{
		sItem.mask		|=  LVCF_WIDTH;
		sItem.cx		 =  nWidth;
		}
	else{
		sItem.mask		|=  LVCF_VWIDTH;
		sItem.cx		 = -nWidth;
		}



return InsertColumn(nCol,&sItem);
}

//*****************************************************************************
//*
//*		SetItem
//*
//*****************************************************************************
//	Changes some properties of an item
//	iItem			: is the handle of the item
//	nCol			: is the column for the text (0=tree column)
//	nMask			: defies which propaties should be set (LVIF_????)
//	pText			: is the new text of the item (add LVIF_TEXT to nMask)
//	nImage			: is the new image for the item (add LVIF_IMAGE to nMask)
//	nState			: is the new state for the item (add LVIF_STATE to nMask)
//	nStateMask		: is a mask for the states
//	lParam			: is the new LPARAM parameter for the item (add LVIF_PARAM to nMask)
//	Returns TRUE if the text was changed or FALSE if an error occurs
BOOL CFastListCtrl::SetItem(INT iItem,int nCol,UINT nMask,LPCTSTR pText,int nImage,UINT nState,UINT nStateMask,LPARAM lParam)
{
LV_ITEM		sItem;


	ASSERT(::IsWindow(m_hWnd));

	sItem.mask				= nMask;
    sItem.iItem				= iItem;
	sItem.iSubItem			= nCol;
    sItem.state				= nState;
    sItem.stateMask			= nStateMask;
    sItem.pszText			= (LPTSTR)pText;
    sItem.cchTextMax		=(pText)? 256:0;
    sItem.iImage			= nImage;
	sItem.lParam			= lParam;


return SetItem(&sItem);
}


//*****************************************************************************
//*
//*		FindItem
//*
//*****************************************************************************
//	Seaches for an Item in the control with several properties
//	iItem		: Is the item handle where the seach begins
//	iFlags		: This flag define the seach options
//					LVIF_NEXT	= begin at the next item after iItem to seach
//					LVIF_PARAM	= compare the the lParam  entry at the seach
//					LVIF_TEXT	= compare the the pText   entry at the seach
//					LVIF_STATE	= compare the the uState  entry at the seach
//					LVIF_CASE	= don't differ between lower and upper case
//	nCol		: Selects the column for the text comparison
//	pText		: Is the text option for the seach
//	lParam		: Is the param option for the seach
//	uState		: Are the state bits for the seach
//	uStateMask	: Only bits which are set in this mask will be comared
//	Returns the handle of the item which was found, or NULL if no item was found
INT CFastListCtrl::FindItem(INT iItem,int iFlags,int nCol,LPCTSTR pText,LPARAM lParam,UINT uState,UINT uStateMask)
{
LV_FIND		sFind;


	sFind.uFlags		= iFlags;
	sFind.uColumn		= nCol;
	sFind.pText			= pText;
	sFind.lParam		= lParam;
	sFind.uState		= uState;
	sFind.uStateMask	= uStateMask;


return FindItem(iItem,&sFind);
}


//*****************************************************************************
//*
//*		InsertItem
//*
//*****************************************************************************
//	Call this function to insert a new item in a tree list control.
//	uMask			: Integer specifying which attributes to set. See the LVITEM structure in the Platform SDK.
//	iItem			: Zero-based index of the item to which this structure refers.
//	pText			: Address of a string containing the items text.
//	nState			: Specifies values for the items states. SeeTree View Control Item States in the Platform SDK for a list of appropriate states.
//	nStateMask		: Specifies which states are to be set. See the LVITEM structure in the Platform SDK.
//	iImage			: Index of the items image in the tree view controls image list.
//	lParam			: A 32-bit application-specific value associated with the item.
//	Returns the index of the new item if successful or -1 otherwise.
INT CFastListCtrl::InsertItem(UINT uMask,INT iItem,LPCTSTR pText,UINT uState,UINT uStateMask,INT iImage,LPARAM lParam)
{
LV_ITEM		sInsert;


	ASSERT(::IsWindow(m_hWnd));

	sInsert.mask			= uMask;
	sInsert.iItem			= iItem;
	sInsert.iSubItem		= 0;
	sInsert.state			= uState;
	sInsert.stateMask		= uStateMask;
	sInsert.pszText			= (LPTSTR)pText;
	sInsert.cchTextMax		= 0;
	sInsert.iImage			= iImage;
	sInsert.lParam			= lParam;


return (INT)::SendMessage(m_hWnd,LVM_INSERTITEM,0,(LPARAM)&sInsert);
}

//*****************************************************************************
//*
//*		InsertItem
//*
//*****************************************************************************
//	Call this function to insert a new item in a tree list control.
//	uMask			: Integer specifying which attributes to set. See the LVITEM structure in the Platform SDK.
//	iItem			: Zero-based index of the item to which this structure refers.
//	lParam			: A 32-bit application-specific value associated with the item.
//	uFlags			: Sind erweiterte Flags fr das Einfgen
//						LVIF_FAST		= fast insert mode, update list some times later
//						LVIF_ENDSCROLL	= scroll to the end of the list
//						LVIF_SHOWSCROLL = the item should be visible
//	Returns the index of the new item if successful or -1 otherwise.
INT CFastListCtrl::InsertItem(INT iItem,LPARAM lParam,UINT uFlags)
{
LV_ITEM		sInsert;


	ASSERT(::IsWindow(m_hWnd));

	sInsert.mask			= LVIF_PARAM|uFlags;
	sInsert.iItem			= iItem;
	sInsert.iSubItem		= 0;
	sInsert.state			= 0;
	sInsert.stateMask		= 0;
	sInsert.pszText			= _T("");
	sInsert.cchTextMax		= 0;
	sInsert.iImage			= 0;
	sInsert.lParam			= lParam;


return (INT)::SendMessage(m_hWnd,LVM_INSERTITEM,0,(LPARAM)&sInsert);
}


//*****************************************************************************
//*
//*		SetColumnText
//*
//*****************************************************************************
//	Sets the text of an item in the column header
//	nCol	: is the number of the column
//	pText	: is the new text
//	Returns TRUE if successful
BOOL CFastListCtrl::SetColumnText(int nCol,LPCTSTR pText)
{
LV_COLUMN	sData;


	sData.pszText		= (LPTSTR)pText;
	sData.cchTextMax	= (int)_tcslen(pText);
	sData.mask			= LVCF_TEXT;


return	SetColumn(nCol,&sData);
}

//*****************************************************************************
//*
//*		SetColumnImage
//*
//*****************************************************************************
//	Sets the text of an item in the column header
//	nCol	: is the number of the column
//	iImage	: is the new image number
//	iPlace	: is the pace for the image (0=left of text 1=right of text)
//	Returns TRUE if successful
BOOL CFastListCtrl::SetColumnImage(int nCol,int iImage,int iPlace)
{
CHeaderCtrl	   *pHeader;
HDITEM			sItem;


		pHeader	= GetHeaderCtrl();
	if(!pHeader)return FALSE;

	sItem.mask	= HDI_FORMAT;

	pHeader->GetItem(nCol,&sItem);

	if(iImage<-1)
			sItem.fmt&=~HDF_IMAGE;
	else	sItem.fmt|= HDF_IMAGE;

	if(iPlace&1)
			sItem.fmt|= HDF_BITMAP_ON_RIGHT;
	else	sItem.fmt&=~HDF_BITMAP_ON_RIGHT;

	sItem.mask	|= HDI_IMAGE;
	sItem.iImage = iImage;


return pHeader->SetItem(nCol,&sItem);
}

//*****************************************************************************
//*
//*		SetColumnBitmap
//*
//*****************************************************************************
//	Sets the bitmap in the column header.
//	nCol	: is the number of the column
//	hBmp	: is the bitmap handle
//	iPlace	: is the pace for the bitmap (0=left of text 1=right of text)
//	Returns TRUE if successful
BOOL CFastListCtrl::SetColumnBitmap(int nCol,HBITMAP hBmp,int iPlace)
{
CHeaderCtrl	   *pHeader;
HDITEM			sItem;


		pHeader	= GetHeaderCtrl();
	if(!pHeader)return FALSE;

	sItem.mask  = HDI_FORMAT;

	pHeader->GetItem(nCol,&sItem);

	if(hBmp==0)
			sItem.fmt&=~HDF_BITMAP;
	else	sItem.fmt|= HDF_BITMAP;

	if(iPlace&1)
			sItem.fmt|= HDF_BITMAP_ON_RIGHT;
	else	sItem.fmt&=~HDF_BITMAP_ON_RIGHT;

	sItem.mask |= HDI_BITMAP;
	sItem.hbm	= hBmp;


return pHeader->SetItem(nCol,&sItem);
}

//*****************************************************************************
//*
//*		SetColumnMark
//*
//*****************************************************************************
//	Sets the mark state of a column. A marked column will be drawn darker.
//	nCol	: is the number of the column
//	bOn		: is the mark state
//	Returns TRUE if successful
BOOL CFastListCtrl::SetColumnMark(int nCol,BOOL bOn)
{
LV_COLUMN	sData;


	sData.fmt	= (bOn)? LVCFMT_MARK:0;
	sData.mask	= LVCF_MARK;


return	SetColumn(nCol,&sData);
}

//*****************************************************************************
//*
//*		FixColumnSize
//*
//*****************************************************************************
//	Fixes the size of a a column. The size of a fixed column can't be changed.
//	nCol	: is the number of the column
//	bOn		: is the fixed state
//	iWidth	: is the new width for the column (-1 means don't change width)
//			  LVCF_LASTSIZE restore size befor fixing
//	iRecalc	: Sollen die variablen Spaltenbreiten neu berechnet
//				-1 = nein
//				 0 = nur die nderung
//				 1 = alle neu setzen
//	Returns TRUE if successful
BOOL CFastListCtrl::FixColumnSize(int nCol,BOOL bOn,int iWidth,int iRecalc)
{
LV_COLUMN	sData;


	sData.fmt	= (bOn)? LVCFMT_FIXED:0;
	sData.mask	= LVCF_FIXED;

	if(iWidth>=0)										// set also the column width
		{
		sData.mask |= LVCF_WIDTH;
		sData.cx    = iWidth;
		}

	if(iRecalc>=0)sData.mask|=LVCF_RECALC;
	if(iRecalc> 0)sData.fmt |=LVCFMT_RECALC;


return	SetColumn(nCol,&sData);
}

//*****************************************************************************
//*
//*		CreateDragImage
//*
//*****************************************************************************
CImageList *CFastListCtrl::CreateDragImage(INT iItem,int nCol)
{
CImageList *pImageList;
HIMAGELIST	hImageList;


	ASSERT(::IsWindow(m_hWnd));

		hImageList = FastList_CreateDragImageEx(m_hWnd,iItem,nCol);
	if(!hImageList)
		{
		return NULL;
		}

	pImageList = new CImageList;
	VERIFY(pImageList->Attach(hImageList));

return pImageList;
}

