//*****************************************************************************
//*
//*
//*		FileListCtrl.cpp: Implementierungsdatei
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//	Anmerkung:		 AfxOleInit() vorher aufrufen, damit Drags funktionieren !
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<AfxWin.h>
#include	<AfxOle.h>
#include	<AfxCmn.h>
#include	"WinExtension.h"
#include	"FileListCtrl.h"
#include	"FileFunctions.h"


#ifdef		_DEBUG
#define new  DEBUG_NEW
#undef		 THIS_FILE
static char  THIS_FILE[] = __FILE__;
#endif

#define		 CFT_NAME_BASE	4
#define 	 ID_STATUS		0x5533
#define 	 ID_START		0x5534
#define 	 ID_SHOWCOL		0x5560

#if         _MFC_VER >= 0x0700
typedef		 LRESULT (AFX_MSG_CALL CWnd::*AfxMsgProc)(WPARAM, LPARAM);
#define 	 AFX_MSG_PROC	(AfxMsgProc)
#else
#define 	 AFX_MSG_PROC
#endif



BEGIN_MESSAGE_MAP(CFileListCtrl, CFastListCtrl)
	//{{AFX_MSG_MAP(CFileListCtrl)
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
	ON_WM_SETFOCUS()
	ON_WM_DROPFILES()
	ON_NOTIFY_REFLECT(LVN_RESELECT,   		 OnResSelect)
	ON_NOTIFY_REFLECT(LVN_BEGINDRAG,  		 OnBeginDrag)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK,		 OnColumnClick)
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED,		 OnItemChanged)
	ON_NOTIFY_REFLECT(LVN_COLUMNRCLICK,		 OnColumnRClick)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_CUT,   AFX_MSG_PROC 		 OnCut)
	ON_MESSAGE(WM_COPY,  AFX_MSG_PROC 		 OnCopy)
	ON_MESSAGE(WM_CLEAR, AFX_MSG_PROC 		 OnClear)
	ON_MESSAGE(WM_PASTE, AFX_MSG_PROC		 OnPaste)
	ON_COMMAND(ID_START,            		 OnStart)
	ON_COMMAND_RANGE(ID_SHOWCOL,ID_SHOWCOL+7,OnShowCol)
END_MESSAGE_MAP()

char CFileListCtrl::m_aSortTypes[8] = 						// Spaltenart zu Sortierart
			{
			CFL_SORT_NAME,
			CFL_SORT_PATH,
			CFL_SORT_SIZE,
			CFF_SORT_TYPE,
			CFL_SORT_SUFFIX,
			CFL_SORT_TIME,
			CFL_SORT_CREATE,
			CFL_SORT_ATTRIBUTE,
			};

LPCTSTR	CFileListCtrl::m_aNames[] =
			{
			_T("%i Dateien gewhlt" ),
			_T("Eine Datei gewhlt" ),
			_T("Keine Datei gewhlt"),
			_T("Abbrechen"),
			_T("Dateiname"),
			_T("Pfad"     ),
			_T("Gre"    ),
			_T("Dateityp" ),
			_T("Typ"      ),
			_T("Zeit"     ),
			_T("Erstellt" ),
			_T("Attribute"),
			};



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFileListCtrl::CFileListCtrl()
{

	m_iTimeMode		 = STR_DATE_ISO|STR_LONG|STR_TIME;
	m_iSizeMode		 = VTS_ADDSUFFIX|VTS_ADDSPACE|VTS_BIN10|0x8320;
	m_iSortMode		 = CFL_SORT_NAME|CFL_SORT_NODIR|CFL_SORT_VALUE;
	m_iSortDir		 = 0;										// Sortierrichtung
	m_iFlags		 = 0;										// Fenster-Style
	m_iColumnCount   = 0;										// Spaltenanzahl
	m_iColumnMode	 = 0;										// Spaltenbist fr Anzeige
	m_iColumnMark	 =-1;										// Welche Spalte ist markiert
	m_iColumnMarkOn	 = 1;										// Darf eine Spalte markiert werden
	m_iSelectedItem	 =-1;										// Erste gewhlte Dateien
	m_iSelectedSize	 = 0;										// Gre der gewhlten Dateien
	m_iSelectedCount = 0;										// Anzahl der gewhlten Dateien
	m_iStatusBarOn	 = 0;										// Statusbar Timer ein

	m_sFiles.Set(_T(""),CFF_FIND_ALL);

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFileListCtrl::~CFileListCtrl()
{
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Erzeugt das Fenster
//	dwExStyle	: Erweiterte Flags fr das Fenster
//	dwStyle		: Sind die Flags fr das Aussehen des Fensters
//					CFL_PATH			Pfad anzeigen
//					CFL_SIZE			Dateigre anzeigen
//					CFL_TYPE			Dateityp   anzeigen
//					CFL_TIME			Dateigre anzeigen
//					CFL_DEFAULT			Alle Spalten anzeigen
//					CFL_ADDDIR			Auch in Unterverzeichnisse einfgen
//					CFL_CANDELETE		Die Dateien knnen mit Del entfernt werden
//					CFL_ALLOWDROP		Hinzufgen von Datei per Drag & Drop
//					CFL_ALLOWDRAG		Dateien per Drag weiter geben
//					CFL_ALLOWPASTE		Einfgen per Tastatur erlauben
//					CFL_SINGLESEL		Nur eine Datei ist whlbar
//					CFL_TABEXIT			Bei TAB den Fokus ans Elternfenster geben
//					CFL_SORT			Sortieren erlaubt
//					CFL_STATUSBAR		Statusbar Nachrichten abschicken
//	sRect		: Ist die Position des Fensters im Elternfenster
//	pParent		: Zeiger auf das Elternfenster
//	uId			: Ist die ID fr das Fenster
//	Ergibt TRUE wenn das Fenster erzeugt werden konnte.
BOOL CFileListCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId)
{
LV_DATAFIELD	sField;
HIMAGELIST		hImages;
SHFILEINFO		sShInfo;
static int		iWidth;
int				iMode;
int				iBits;
int				iBit;
int				iCol;


	#ifdef	_DEBUG
	if(dwStyle&CFL_ALLOWDRAG)
		{
		_AFX_THREAD_STATE *pState = AfxGetThreadState();
		ASSERT(pState->m_bNeedTerm);							// Wurde AfxOleInit() aufgerufen ?
		}
	#endif



	iMode = dwStyle&(0xFFFF0000&~WS_HSCROLL&~WS_VSCROLL&~WS_POPUP&~WS_GROUP);

	if(dwStyle&CFL_SINGLESEL)
		{
		iMode |= LVS_SINGLESEL;
		}

	if(!CFastListCtrl::CreateEx(dwExStyle,WS_CHILD|LVS_SHAREIMAGELISTS|LVS_EDITLABELS|iMode,sRect,pParent,uId))
		{
		return FALSE;
		}

	hImages = (HIMAGELIST)SHGetFileInfo(_T(""),0,&sShInfo,sizeof(sShInfo),SHGFI_PIDL|SHGFI_SMALLICON|SHGFI_SYSICONINDEX);

	SetImageList(hImages,LVSIL_SMALL);

 	if(iWidth==0)												// Schriftbreite holen
		{
		iWidth = (GetDefaultSize()*256)/7;
		}

						 
	InsertColumn(0,m_aNames[0+CFT_NAME_BASE],LVCFMT_LEFT , -1,(-150*iWidth)/256);
	InsertColumn(1,m_aNames[1+CFT_NAME_BASE],LVCFMT_LEFT , -2);
	InsertColumn(2,m_aNames[2+CFT_NAME_BASE],LVCFMT_RIGHT,( 80*iWidth)/256);
	InsertColumn(3,m_aNames[3+CFT_NAME_BASE],LVCFMT_LEFT ,(145*iWidth)/256);
	InsertColumn(4,m_aNames[4+CFT_NAME_BASE],LVCFMT_LEFT ,( 45*iWidth)/256);
	InsertColumn(5,m_aNames[5+CFT_NAME_BASE],LVCFMT_LEFT ,(100*iWidth)/256);
	InsertColumn(6,m_aNames[6+CFT_NAME_BASE],LVCFMT_LEFT ,(100*iWidth)/256);
	InsertColumn(7,m_aNames[7+CFT_NAME_BASE],LVCFMT_LEFT ,( 60*iWidth)/256);

	for(iCol=1,iBit=1;iCol<8;iCol++,iBit<<=1)					// Unbenutzte Spalten ausblenden
		{
		if(dwStyle&iBit)continue;
		FixColumnSize(iCol,TRUE,0,2);
		}

	
	iBits = LVS_EX_NODELNOTIFY|LVS_EX_HEADERDRAGDROP;

	if(dwStyle&CFL_ALLOWDROP  )DragAcceptFiles(TRUE);
	if(dwStyle&CFL_NORIGHTSEL )iBits|=LVS_EX_NORIGHTSELECT;
	if(dwStyle&CFL_FULLROWMARK)iBits|=LVS_EX_FULLROWSELECT;
	

	SetExtendedStyle(iBits);

		 			  m_sFiles.SetUserDataSize(sizeof(void*)*2);
    sField.pField	= m_sFiles.GetFieldPointer(&sField.pCount);
    sField.iOffset	= m_sFiles.InfoOffset();
	sField.iSortMem = 0;
    sField.pLockCb	= 0;

	SetSetDataField(&sField);									// Callbacks einstellen

	m_sFiles.SetCallback(sField.pLockCb,sField.pParam);
					  
	SetCloseCallback(CloseCallback);
	SetLineCallback (LineCallback);
	SetCloseCbParam (this);
	SetLineCbParam  (this);

	m_pLockParam	= sField.pParam;
	m_pLockProc		= sField.pLockCb;
	m_iFlags		= dwStyle&(0xFFFF|WS_HSCROLL|WS_VSCROLL);
	m_iStatusBarOn	= 0;
	m_iSelectedSize	= 0;
	m_iSelectedCount= 0;
	m_iSelectedItem =-1;

	if(m_sFiles.Count())										// Einfgen der vorhandenen Liste
		{
		m_pLockProc(m_pLockParam,0,m_sFiles.Count(),1|LVIF_LOCKING);
		m_pLockProc(m_pLockParam,0,m_sFiles.Count(),1);
		}

	if(m_iColumnMarkOn)											// Eine Spalte markieren
		{
		switch(m_iSortMode&0xFF)
			{
		case CFL_SORT_NAME:		m_iColumnMark = 0;	break;
		case CFL_SORT_PATH:		m_iColumnMark = 1;	break;
		case CFL_SORT_SIZE:		m_iColumnMark = 2;	break;
		case CFL_SORT_TYPE:		m_iColumnMark = 3;	break;
		case CFL_SORT_TIME:		m_iColumnMark = 5;	break;
		case CFL_SORT_SUFFIX:	m_iColumnMark = 4;	break;
		case CFL_SORT_CREATE:	m_iColumnMark = 6;	break;
		case CFL_SORT_ATTRIBUTE:m_iColumnMark = 7;	break;
			}

		SetColumnMark(m_iColumnMark,TRUE);
		}


	if(m_iFlags&CFL_SORT)										// Sortieren
		{
		iBits = m_iSortMode&~(CFF_SORT_INVERT|0xFF);
		if(m_iSortDir)iBits|=CFF_SORT_INVERT;
		m_sFiles.Sort(m_iSortMode,iBits);
		}



	if(dwStyle&CFL_STATUSBAR)
		{
		UpdateStatusBar();
		}


return TRUE;
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Erzeugt das Fenster	in einem Dialog, ber einen Rechteck.
//	dwExStyle	: Erweiterte Flags fr das Fenster
//	dwStyle		: Sind die Flags fr das Aussehen des Fensters
//					CFL_PATH			Pfad anzeigen
//					CFL_SIZE			Dateigre anzeigen
//					CFL_TYPE			Dateityp   anzeigen
//					CFL_TIME			Dateigre anzeigen
//					CFL_DEFAULT			Alle Spalten anzeigen
//					CFL_ADDDIR			Auch in Unterverzeichnisse einfgen
//					CFL_CANDELETE		Die Dateien knnen mit Del entfernt werden
//					CFL_ALLOWDROP		Hinzufgen von Datei per Drag & Drop
//					CFL_ALLOWDRAG		Dateien per Drag weiter geben
//					CFL_ALLOWPASTE		Einfgen per Tastatur erlauben
//					CFL_SINGLESEL		Nur eine Datei ist whlbar
//					CFL_TABEXIT			Bei TAB den Fokus ans Elternfenster geben
//					CFL_SORT			Sortieren erlaubt
//					CFL_STATUSBAR		Statusbar Nachrichten abschicken
//	uItem       : Ist die ID des Rechtecks im Dialog.
//	pDlg		: Zeiger auf den Eltern-Dialog.
//	uId			: Ist die ID fr das Fenster.
//	Ergibt TRUE wenn das Fenster erzeugt werden konnte.
BOOL CFileListCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,UINT uItem,CDialog *pDlg,UINT nId)
{
CPoint	sPoint(0,0);
CRect	sRect;
CWnd   *pWnd;


	pWnd=pDlg->GetDlgItem(uItem);
	ASSERT(pWnd!=0);


	pWnd->GetWindowRect (&sRect);
	pDlg->ClientToScreen(&sPoint);

	sRect.left	  -= sPoint.x;
	sRect.right   -= sPoint.x;
	sRect.top	  -= sPoint.y;
	sRect.bottom  -= sPoint.y;


	if(!CFileListCtrl::CreateEx(dwExStyle,dwStyle|WS_CHILD|WS_VISIBLE|WS_HSCROLL|WS_VSCROLL,sRect,pDlg,nId))
		{
		return FALSE;
		}

	SetWindowPos(pWnd,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOREDRAW);


return TRUE;
}

//*****************************************************************************
//*
//*		ClearSelection
//*
//*****************************************************************************
//	Lscht die aktuelle Auswahl im Fenster
void CFileListCtrl::ClearSelection()
{

	SetItemStates(0,0x7FFFFFFF,0,LVIS_SELECTED);

	m_iSelectedCount	= 0;
	m_iSelectedSize		= 0;

}

//*****************************************************************************
//*
//*		Scroll
//*
//*****************************************************************************
//	Scrollt das Fenster einige Zeilen in Y-Richtung
//	iRows		: Ist die Anzahl der zu scrollenden Zeilen
//	Ergibt TRUE wenn gescrollt werden konnte
BOOL CFileListCtrl::Scroll(int iRows)
{
int		iTop;


	if(iRows==0)return TRUE;

	iTop = GetTopIndex();


return SetTopIndex(iTop+iRows);
}

//*****************************************************************************
//*
//*		Add
//*
//*****************************************************************************
//	Fgt Dateien in das Fenster ein
//	pPath		: Ist der Suchpfad, es knnen mehrere Filter angehngt werden
//	iMode		:	CFL_SUBDIR		: Auch in Unterverzeichnissen suchen
//					CFL_ADDDIR		: Auch Unterverzeichnisse einfgen
//					CFL_ONLYDIR		: Nur Verzeichnisse anzeigen
//					CFL_SHOWPATHUP	: Das ".." Verzeichnis einfgen
//	pPlus		: Plusfilter fr Dateien
//	pMinus		: Minusfilter fr Dateien
//	iProtect	: Bestimmt welche Dateien eingefgt werden sollen
//					0=alle einfgen  1=nur schreibgeschtzte 2=nicht schreibgeschtzte
//	iTimeMode	: Bestimmt den Zeitmodus fr das einfgen der Dateien
//					0=egal
//					1=erstellt vor
//					2=gendert vor
//					3=erstellt vor
//					4=gendert nach
//					5=erstellt von/bis
//					6=gendert von/bis
//					7=erstellt vor/nach
//					8=gendert vor/nach
//	iTimeValue	: Zeitwert wenn TimeMode nicht 0 ist
//	iTimeUntil	: Biszeit wenn TimeMode nicht 5 oder 6 ist
//	Ergibt die Anzahl der hinzugefgten Dateien.
int CFileListCtrl::Add(LPCTSTR pPath,int iMode,LPCTSTR pPlus,LPCTSTR pMinus,int iProtect,int iTimeMode,time_t iTimeValue,time_t iTimeUntil)
{
int		iFlags;
int		iBits;
int		iNum;


	if(m_iFlags&CFL_SORT)									// Sortieren
		{
		iBits = m_iSortMode&~(CFF_SORT_INVERT|0xFF);
		if(m_iSortDir)iBits|=CFF_SORT_INVERT;
		m_sFiles.Sort(m_iSortMode,iBits);
		iFlags = CFF_FIND_ALL|CFF_FIND_SORT;
		}
	else{
		iFlags = CFF_FIND_ALL;
		}

	if(  iMode &CFL_NETSEACH)iFlags|=CFF_FIND_NET;
	if(  iMode &CFL_ONLYDIR )iFlags|=CFF_FIND_DIR;
	if(  iMode &CFL_ADDUP   )iFlags|=CFF_FIND_ADDUP;
	if(  iMode &CFL_SUBDIR  )iFlags|=CFF_FIND_SUBDIR;
	if(  iMode &CFL_DIRCMP  )iFlags|=CFF_FIND_DIRCMP;
	if(m_iFlags&CFL_ADDDIR  )iFlags|=CFF_FIND_ADDDIR;
	if(m_sFiles.Count() >= 0)iFlags|=CFF_FIND_ADD;

	m_sFiles.TimeFilter(iTimeMode,iTimeValue,iTimeUntil);
	m_sFiles.ProtectFilter(iProtect);
	iNum = m_sFiles.Read(pPath,iFlags,pPlus,pMinus);		// Dateien suchen

	m_sFiles.ProtectFilter(0);
	m_sFiles.TimeFilter(0);



return iNum;
}


//*****************************************************************************
//*
//*		OnKeyDown
//*
//*****************************************************************************
void CFileListCtrl::OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags)
{
CWnd   *pParent;
int		iShift;
int		iCtrl;



	if(nChar==VK_DELETE)									// Auswahl lschen oder ausschneiden
		{
		iShift = GetAsyncKeyState(VK_SHIFT)&0x8000;

		if(!iShift && (m_iFlags&CFL_CANDELETE))DeleteSelItems();
		if( iShift && (m_iFlags&CFL_CANDELETE))OnCut();
		return;
		}

	if(nChar==VK_INSERT)									// Aus Zwischenablage einfgen
		{
		iShift = GetAsyncKeyState(VK_SHIFT  )&0x8000;
		iCtrl  = GetAsyncKeyState(VK_CONTROL)&0x8000;

		if( iCtrl && !iShift){OnCopy ();return;}
		if(!iCtrl &&  iShift){OnPaste();return;}
		}

	if(nChar==VK_TAB && (m_iFlags&CFL_TABEXIT))				// Zu anderem Fenster springen
		{
		   pParent=GetParent();
		if(pParent)pParent->SetFocus();
		return;
		}

	if(nChar=='A' && (GetAsyncKeyState(VK_CONTROL)&0x8000))	// Alles auswhlen
		{
		SetItemStates(0,m_sFiles.Count(),LVIS_SELECTED,LVIS_SELECTED);
		return;
		}


CFastListCtrl::OnKeyDown(nChar,nRepCnt,nFlags);
}

//*****************************************************************************
//*
//*		DeleteItem
//*
//*****************************************************************************
//	Lscht eine Datei aus der Liste
//	iItem	: Ist die Nummer der zu lschenden Datei
BOOL CFileListCtrl::DeleteItem(int iItem)
{


	if(!m_sFiles.Del(iItem))
		{
		return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		DeleteSelItems
//*
//*****************************************************************************
//	Die ausgewhlten Eintrge lschen. Es werden dabei nur die Eintrge
//	aus der Liste gelscht und nicht die realen Dateien.
BOOL CFileListCtrl::DeleteSelItems()
{
int		iItem;
int		iPos;


	   iItem  = GetFirstSelected();
	if(iItem<0)return FALSE;

	iPos = iItem;

	while(iPos>=0)
		{
		SetItemState(iPos,0,LVNI_SELECTED);
		if(m_sFiles.Del(iPos))iPos--;
		iPos = GetNextSelected(iPos);
		}


return TRUE;
}


//*****************************************************************************
//*
//*		SelectAllItems
//*
//*****************************************************************************
//	Alle Eintrge auswhlen
BOOL CFileListCtrl::SelectAllItems()
{


	if(m_iFlags&CFL_SINGLESEL)								// Nur Einzelauswahl erlaubt
		{
		return FALSE;
		}

	SetItemStates(0,m_sFiles.Count(),LVIS_SELECTED,LVIS_SELECTED);
	m_iSelectedItem = 0;


return TRUE;
}

//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die Eintrge.
//	iMode	: Bestimmt nach welchem Kriterium sortiert werden soll.
//				CFL_SORT_NAME		Nach Dateinamen     sortieren
//				CFL_SORT_PATH		Nach Dateipfad      sortieren
//				CFL_SORT_SIZE		Nach Dateigre     sortieren
//				CFL_SORT_TYPE		Nach Dateitype      sortieren
//				CFL_SORT_TIME		Nach Dateizeit		sortieren
//				CFL_SORT_SUFFIX		Nach Dateisuffix	sortieren
//				CFL_SORT_CREATE		Nach Erstllungszeit sortieren
//				CFL_SORT_ACCCESS	Nach Zugiffszeit    sortieren
//				CFL_SORT_NODIR		Verzeichnisse nicht an den Anfang
//				CFL_SORT_VALUE		Bercksichtigt Zahlen in den Texten
//	iDir	: Ist die Sortierrichtung.
//				==0				Normal   sortieren
//				!=0				Verkerht sortieren
void CFileListCtrl::Sort(int iMode,int iDir)
{
int		iBits;
int		iItem;

															
															
	if(IsWindow(m_hWnd))
		{
		iItem = sizeof(m_aSortTypes)/sizeof(m_aSortTypes[0])-1;
		
		for(;iItem>0;iItem--)
			{
			if((char)iMode==m_aSortTypes[iItem])break;
			}
				
		if(m_iColumnMark!=iItem && m_iColumnMarkOn)			// Neue Sortierreihenfolge
			{
			SetColumnMark(m_iColumnMark,FALSE);
			SetColumnMark(iItem        ,TRUE );
			m_iColumnMark = iItem;
			}
		}

	m_iSortMode = iMode;
	m_iSortDir  = iDir;

	if(m_sFiles.Count())
		{
		iBits	= iMode&~(CFF_SORT_INVERT|0xFF);
		iMode  &= 0x00FF;

		if(iDir)iBits|=CFF_SORT_INVERT;

		m_sFiles.Sort(iMode,iBits);
		}

}

//*****************************************************************************
//*
//*		SetTimeMode
//*
//*****************************************************************************
//	Stellt den Anzeigemodus fr Zeiten ein.
//	iMode	: Ist der neue Modus (siehe strtimeset(...))
//				STR_DATE_ISO|STR_LONG|STR_TIME			"2014-11-30 12:59"
//				STR_DATE_ISO|STR_LONG|STR_TIME_SEC		"2014-11-30 12:59:59"
//				STR_DATE_ABC|STR_LONG|STR_TIME			"30.Nov.2014 12:59"
void CFileListCtrl::SetTimeMode(int iMode,BOOL bReDraw)
{

	if(iMode==m_iTimeMode)return;

	m_iTimeMode = iMode;
	RedrawWindow();

}

//*****************************************************************************
//*
//*		SetSizeMode
//*
//*****************************************************************************
//	Stellt den Anzeigemodus fr Dateigren ein.
//	iMode	: Ist der neue Modus (siehe vtostr(...))
//				VTS_ADDSUFFIX|VTS_ADDSPACE|VTS_BIN10|0x8320			  "123.45 kB"
//				VTS_ADDSUFFIX|VTS_ADDSPACE|VTS_BIN10|0x8F20			  "12345 Byte"
//				VTS_ADDSUFFIX|VTS_ADDSPACE|VTS_BIN10|0x8F20|(' '<<24) "12 345 Byte"
void CFileListCtrl::SetSizeMode(int iMode,BOOL bReDraw)
{

	if(iMode==m_iSizeMode)return;

	m_iSizeMode = iMode;
	if(bReDraw)RedrawWindow();

}

//*****************************************************************************
//*
//*		OnDropFiles
//*
//*****************************************************************************
//	Werden Dateien auf dem Fenster fallen gelassen.
void CFileListCtrl::OnDropFiles(HDROP hDropInfo)
{

	if(m_iFlags&CFL_ALLOWDROP)
		{
		Paste(hDropInfo);
		}

}

//*****************************************************************************
//*
//*		CreateSelection
//*
//*****************************************************************************
//	Erzeugt ein HGLOBAL Objekt mit den ausgewhlten Dateien.
//	Ergibt das Objekt handle oder 0 bei einem Fehler
HGLOBAL	CFileListCtrl::CreateSelection()
{
HGLOBAL			hGlobal;
LPCTSTR			pName;
LPTSTR			pMem;
int				iPos;
int				iLen;
int				iNum;


	iLen = 24;
	iPos = -1;

	for(;;) 												// Speichergre berechnen
		{
		   iPos=GetNextItem(iPos,LVIS_SELECTED);
		if(iPos<0)break;

		iLen += m_sFiles.NameFullLen(iPos)+1;
		}

	if(iLen<=24)return 0;

	hGlobal	 = GlobalAlloc(GMEM_ZEROINIT,iLen*sizeof(TCHAR)+4);
	pMem	 = (LPTSTR)GlobalLock(hGlobal);
	pMem[0]	 = 20;
	#ifdef UNICODE
	pMem[8]  = 1;
	#endif
	pMem	+= 20/sizeof(TCHAR);
	iPos	 = -1;

	for(;;) 												// Dateinamen anhngen
		{
		   iPos=GetNextItem(iPos,LVIS_SELECTED);
		if(iPos<0)break;

		pName = m_sFiles.Path(iPos,0,&iNum);
		memcpy(pMem,pName,iNum*sizeof(TCHAR));
		pMem += iNum;
		pName = m_sFiles.Name(iPos,0,&iNum);
		iNum += 1;
		memcpy(pMem,pName,iNum*sizeof(TCHAR));
		pMem += iNum;
		}

	GlobalUnlock(hGlobal);


return hGlobal;
}

//*****************************************************************************
//*
//*		OnBeginDrag
//*
//*****************************************************************************
void CFileListCtrl::OnBeginDrag(NMHDR *pNmHdr,LRESULT *pResult)
{
NMLISTVIEW	   *pNotify=(NMLISTVIEW*)pNmHdr;
CFL_DRAGSTATUS	sNotify;
COleDataSource	sDrag;
STGMEDIUM		sData;



	if(!(m_iFlags&CFL_ALLOWDRAG))							// Drag erlaubt ?
		{
		*pResult = 0;
		return;
		}

		sData.hGlobal = CreateSelection();
	if(!sData.hGlobal)
		{
		*pResult = 0;
		return;
		}

	sNotify.sNmHdr.code		= CFL_DRAGACTION;
	sNotify.sNmHdr.idFrom	= GetWindowLong(m_hWnd,GWL_ID);
	sNotify.sNmHdr.hwndFrom	= m_hWnd;
	sNotify.iItem			= pNotify->iItem;
	sNotify.iMode			= 0;
															// Eltern-Fenster benachrichtigen
	if(GetParent()->SendMessage(WM_NOTIFY,sNotify.sNmHdr.idFrom,(LPARAM)&sNotify))
		{
		*pResult = 0;
		return;
		}

	sData.pUnkForRelease	= 0;
	sData.tymed				= TYMED_HGLOBAL;

	if(m_iFlags&CFL_ALLOWDROP)DragAcceptFiles(FALSE);

	sDrag.CacheGlobalData(CF_HDROP,sData.hGlobal);
	sDrag.DoDragDrop();
	sDrag.Empty();

	if(m_iFlags&CFL_ALLOWDROP)DragAcceptFiles(TRUE);

	sNotify.iMode			= 1;
															// Eltern-Fenster benachrichtigen
	GetParent()->SendMessage(WM_NOTIFY,sNotify.sNmHdr.idFrom,(LPARAM)&sNotify);

	*pResult = 0;

}

//*****************************************************************************
//*
//*		OnColumnClick
//*
//*****************************************************************************
void CFileListCtrl::OnColumnClick(NMHDR *pNmHdr, LRESULT *pResult)
{
NM_LISTVIEW    *pNotify=(NM_LISTVIEW*)pNmHdr;
int				iMode;
int				iItem;


	if(!(m_iFlags&CFL_SORT))								// Ist das Sortieren gesperrt
		{
	   *pResult=0;
		return;
		}

	iItem	= pNotify->iSubItem;
	iMode	= m_aSortTypes[iItem&7];
	iMode  |= m_iSortMode&0xFF00;

	if(m_iSortMode==iMode)									// Neue Sortierreihenfolge
		{
		m_iSortDir = !m_iSortDir;
		}

	Sort(iMode,m_iSortDir);									// Liste neu sortieren

	*pResult = 0;

}

//*****************************************************************************
//*
//*		OnColumnRClick
//*
//*****************************************************************************
void CFileListCtrl::OnColumnRClick(NMHDR *pNmHdr, LRESULT *pResult)
{
LV_HDRCLICK    *pNotify=(LV_HDRCLICK*)pNmHdr;
int				aOrder[8];
CPoint			sPoint;
CMenu			sMenu;
int 			iPos;
int 			iCol;
int 			iBit;


	if(!(m_iFlags&CFL_SETCOLUMN))							// Ist das Einblenden gesperrt
		{
	   	*pResult=0;
		return;
		}

	GetColumnOrderArray(8,aOrder);

	sMenu.CreatePopupMenu();

	for(iPos=0;iPos<7;iPos++)								// Namen in Men einfgen
		{
		iCol = aOrder[iPos+1];
		iBit = (m_iFlags&(1<<iCol))? MF_CHECKED:0;
		sMenu.AppendMenu(iBit,ID_SHOWCOL+iCol,m_aNames[iCol+CFT_NAME_BASE]);
		}

	sPoint = pNotify->sPoint;
	ClientToScreen(&sPoint);

	sMenu.AppendMenu(MF_SEPARATOR);
	sMenu.AppendMenu(0,ID_SHOWCOL,m_aNames[3]);
	sMenu.TrackPopupMenu(TPM_LEFTALIGN,sPoint.x,sPoint.y,this,0);

	*pResult=1;

}

//******************************************************************************
//*
//*		OnShowCol
//*
//******************************************************************************
void CFileListCtrl::OnShowCol(UINT nId)
{
int		iRecalc;
int 	iOn;


	nId -= ID_SHOWCOL+1;
	if(nId>=7)return;

	m_iFlags ^= (1<<nId);
	iOn		  = (1<<nId)&m_iFlags;

	if(nId<=0)iRecalc=1;
	else      iRecalc=0;

	FixColumnSize(nId+1,!iOn,(iOn)? LVCF_LASTSIZE:0,iRecalc);


}

//*****************************************************************************
//*
//*		OnSetFocus
//*
//*****************************************************************************
void CFileListCtrl::OnSetFocus(CWnd* pOldWnd)
{

	CFastListCtrl::OnSetFocus(pOldWnd);

	UpdateStatusBar();

}

//*****************************************************************************
//*
//*		Save
//*
//*****************************************************************************
//	Speichert die Liste in einer Datei
//	pFilename	: ist der Name der Datei
//	Ergibt TRUE wenn die Liste gespeichert wurde
BOOL CFileListCtrl::Save(LPCTSTR pFilename)
{
FILE	*pFile;
LPTSTR   pPath;
int		 i;



		pFile=_tfopen(pFilename,_T("wb"));
	if(!pFile)return FALSE;

	for(i=0;i<m_sFiles.Count();i++)
		{
		pPath = m_sFiles.Name(i,1);
		_ftprintf(pFile,_T("\"%s\"\r\n"),pPath);
		}

	fclose(pFile);


return TRUE;
}

//*****************************************************************************
//*
//*		Load
//*
//*****************************************************************************
//	Ld aus einer Datei eine Dateiliste.
//	pFilename	: Ist der Pfad der Datei mit der Dateiliste.
//	bNew		: Die Dateien vorher in der Liste lschen.
BOOL CFileListCtrl::Load(LPCTSTR pFilename,BOOL bNew)
{
TCHAR		cBuffer[MAX_PATH+16];
FILE	   *pFile;
LPTSTR		pPath;
int			iVal;
int			iLen;



		pFile=_tfopen(pFilename,_T("rb"));
	if(!pFile)return FALSE;

	if(bNew)												// Alte Dateien lschen ?
		{
		m_sFiles.Set(_T(""),CFF_FIND_ALL);
		}


	iVal = (m_iFlags&CFL_ADDDIR)? 0x03:0x07;


	while(_fgetts(cBuffer,sizeof(cBuffer)/sizeof(TCHAR),pFile))
		{
		pPath = cBuffer+strchar(cBuffer,'"');

		if(*pPath!='"')continue;

		pPath++;
		iLen = strchar(pPath,'"');

		if(pPath[iLen]!='"')continue;

		pPath[iLen] = 0;

		m_sFiles.Insert(pPath,-1,iVal);
		}

	fclose(pFile);




return TRUE;
}

//*****************************************************************************
//*
//*		OnItemChanged
//*
//*****************************************************************************
void CFileListCtrl::OnItemChanged(NMHDR *pNmHdr,LRESULT *pResult)
{
NM_LISTVIEW	   *pNotify=(NM_LISTVIEW*)pNmHdr;
int				iChange;


	if(!(m_iFlags&CFL_STATUSBAR))
		{
		return;
		}

	iChange  = pNotify->uNewState;
	iChange ^= pNotify->uOldState;

	if(iChange&LVIS_SELECTED)
		{
		if(pNotify->uNewState&LVIS_SELECTED)
			{
			if(m_iSelectedCount<0)
				{
				UpdateSelection();
				}
			else{
				m_iSelectedSize  += m_sFiles.Len64(pNotify->iItem);
				m_iSelectedCount += 1;
				}

			if(m_iSelectedCount==1)
				{
				m_iSelectedItem = pNotify->iItem;
				}
			else if(m_iSelectedItem>pNotify->iItem)
				{
				m_iSelectedItem = pNotify->iItem;
				}
			}
		else{
			if(m_iSelectedCount<=0)
				{
				UpdateSelection();
				}
			else{
				m_iSelectedSize  -= m_sFiles.Len64(pNotify->iItem);
				m_iSelectedCount -= 1;
				}
			}

		if(m_iSelectedCount<=0)
			{
			m_iSelectedItem = -1;
			m_iSelectedSize =  0;
			}

		UpdateStatusBar();
		}

	*pResult = 0;

}

//*****************************************************************************
//*
//*		StatusbarNotify
//*
//*****************************************************************************
//	Sendet eine Notify-Nachricht fr die Statuszeilenanzeige
int CFileListCtrl::StatusbarNotify()
{
TCHAR		cSize[32];
TCHAR		cText[32];
CFL_STATUS	sNotify;


	if(!(m_iFlags&CFL_STATUSBAR))
		{
		return 0;
		}

	if(!IsWindow(m_hWnd))
		{
		return 0;
		}

	sNotify.iMode			= GetSelectedText(cText,cSize);
	sNotify.sNmHdr.code		= CFL_STATUSBARSET;
	sNotify.sNmHdr.idFrom	= GetWindowLong(m_hWnd,GWL_ID);
	sNotify.sNmHdr.hwndFrom	= m_hWnd;
	sNotify.pCountText		= cText;
	sNotify.pSizeText		= cSize;
	sNotify.iCount			= m_iSelectedCount;
	sNotify.iSize64			= m_iSelectedSize;
	sNotify.iItem			= m_iSelectedItem;

	GetParent()->SendMessage(WM_NOTIFY,sNotify.sNmHdr.idFrom,(LPARAM)&sNotify);


return 1;
}

//*****************************************************************************
//*
//*		GetSelectedText
//*
//*****************************************************************************
//	Erzeugt Texte mit den gewhlten Dateien bzw. der gewhlten Gre.
//	pText		: Hier werden die gewhlten Dateien gespeichert.
//				  Der Zeiger kann NULL seine oder muss 32 Byte gro sein.
//	pSize		: Hier wird die gewhlte Gre gespeichert.
//				  Der Zeiger kann NULL seine oder muss 32 Byte gro sein.
//	Ergibt 2 wenn 2 oder mehr Dateien gewhlt wurden, 1 bei einer oder 0 bei keiner.
UINT CFileListCtrl::GetSelectedText(TCHAR *pText,TCHAR *pSize)
{
TCHAR	cTemp[32];
int		iMode;


	if(!pText)pText=cTemp;

	if(m_iSelectedCount>=2)									// Anzahl der ausgewhlten Dateien
		{
		_stprintf(pText,m_aNames[0],m_iSelectedCount);
		iMode = 2;
		}
	else if(m_iSelectedCount==1)
		{
		_stprintf(pText,m_aNames[1]);
		iMode = 1;
		}
	else{
		_stprintf(pText,m_aNames[2]);
		iMode = 0;
		}

	if(pSize)												// Gre als Text 
		{
		vtostr(m_iSelectedSize,pSize,VTS_ADDSUFFIX|VTS_ADDSPACE|VTS_BIN10|0x8320,VTS_BYTESUFFIX);
		}


return iMode;
} 

//*****************************************************************************
//*
//*		OnTimer
//*
//*****************************************************************************
void CFileListCtrl::OnTimer(UINT_PTR nIdEvent)
{

	if(nIdEvent==ID_STATUS)
		{
		if(m_iStatusBarOn==1)
			{
			StatusbarNotify();
			}

		m_iStatusBarOn++;

		if(m_iStatusBarOn>5)
			{
			m_iStatusBarOn=0;
			KillTimer(ID_STATUS);
			}

		return;
		}

	CFastListCtrl::OnTimer(nIdEvent);

}


//*****************************************************************************
//*
//*		Paste
//*
//*****************************************************************************
//	Fgr Dateien aus einer HDROP Liste ein.
//	hDrop		: Ist das Handle der HDROP Liste.
//	Ergibt die Zahl der eingefgten Dateien.
int CFileListCtrl::Paste(HDROP hDrop)
{
TCHAR		cFile[MAX_PATH];
int			iCount;
int			iItem;
int			iDone;
int			iMode;
int			iFlag;
int			iLen;



	iFlag   = (m_iFlags&CFL_ADDDIR )? 0x03:0x07;
	iMode	= (m_iFlags&CFL_SORTADD)? -2:-1;
	iCount	= DragQueryFile(hDrop,-1,0,0);
	iDone	= 0;

 	for(iItem=0;iItem<iCount;iItem++)						// Die Liste durchlaufen
		{
		   iLen = DragQueryFile(hDrop,iItem,cFile,MAX_PATH);
		if(iLen<=0)continue;

  		if(m_sFiles.Insert(cFile,iMode,iFlag)>=0)
			{
			iDone++;
			}
		}

	DragFinish(hDrop);



return TRUE;
}


//*****************************************************************************
//*
//*		OnPaste
//*
//*****************************************************************************
//	Dateien aus der Zwischenablage einfgen.
//	Ergibt TRUE wenn erfolgreich.
BOOL CFileListCtrl::OnPaste()
{
HDROP	hDrop;
int		iDone;


	if(!IsClipboardFormatAvailable(CF_HDROP))				// Ist eine Dateiliste in der Zwischenablage
		{
		return FALSE;
		}

	if(!(m_iFlags&CFL_ALLOWPASTE))							// Ist das Einfgen erlaubt
		{
		return FALSE;
		}

	if(!OpenClipboard())									// Zwischenablage ffnen
		{
		return FALSE;
		}


		hDrop=(HDROP)GetClipboardData(CF_HDROP);			// Handle auf Dateiliste holen
	if(!hDrop)
		{
		CloseClipboard();
		return FALSE;
		}

	iDone = Paste(hDrop);									// Auswahl einfgen

	CloseClipboard();

	if(iDone && GetFocusItem()<0)
		{
		SetFocusItem(0);
		}



return TRUE;
}

//*****************************************************************************
//*
//*		OnCopy
//*
//*****************************************************************************
//	Kopiert die Dateien in die Zwischenablage
BOOL CFileListCtrl::OnCopy()
{
HGLOBAL			hGlobal;
BOOL			bOk;



		hGlobal = CreateSelection();
	if(!hGlobal)											// Selektierte Dateien holen
		{
		return FALSE;
		}

	bOk = FALSE;

	if(OpenClipboard())										// In Clipboard speichern
		{
		EmptyClipboard();

		if(SetClipboardData(CF_HDROP,hGlobal))
			{
			bOk = TRUE;
			}

		CloseClipboard();
		}

	if(!bOk)
		{
		GlobalFree(hGlobal);
		}


return bOk;
}

//*****************************************************************************
//*
//*		OnClear
//*
//*****************************************************************************
//	Die ausgewhlten Dateien aus der Liste lschen.
//	Ergibt TRUE wenn erfolgreich.
BOOL CFileListCtrl::OnClear()
{

	if(m_iFlags&CFL_CANDELETE)								// Ist das Lschen erlaubt
		{
		DeleteSelItems();
		return TRUE;
		}


return FALSE;
}


//*****************************************************************************
//*
//*		OnCut
//*
//*****************************************************************************
//	Die ausgewhlten Dateien aus der Liste ausschneiden.
//	Ergibt TRUE wenn erfolgreich.
BOOL CFileListCtrl::OnCut()
{

	if(m_iFlags&CFL_CANDELETE)
		{													// Ist das Lschen erlaubt
		OnCopy();
		DeleteSelItems();
		return TRUE;
		}


return FALSE;
}


//*****************************************************************************
//*
//*		OnResSelect
//*
//*****************************************************************************
//	Die Auswahl muss neu berechnet werden
void CFileListCtrl::OnResSelect(NMHDR *pNmHdr, LRESULT *pResult)
{
LV_RESELECT    *pNotify;


	if(!(m_iFlags&CFL_STATUSBAR))
		{
		return;
		}

	pNotify = (LV_RESELECT*)pNmHdr;

	if(pNotify->iSelected==0)
		{
		m_iSelectedCount	= 0;
		m_iSelectedSize		= 0;
		}
	else{
		UpdateSelection();
		}

	UpdateStatusBar();

	*pResult = 0;

}

//*****************************************************************************
//*
//*		OnStart
//*
//*****************************************************************************
//	Starten des Timer fr die Stausbar via PostMessage(...)
void CFileListCtrl::OnStart()
{
	SetTimer(ID_STATUS,250,0);
}


//*****************************************************************************
//*
//*		SetColumnOrder
//*
//*****************************************************************************
//	Stellt die Reihenfolge der Spalten ein
//	iOrder		: Ist die Reihenfolge der Spalten, jeweils 4 Bits bestimmen eine
//				  die Position einer Spalte
//					Bit  0...3	= Namen
//					Bit  4...7	= Pfad
//					Bit  8..11	= Gre
//					Bit 12..15	= Dateityp
//					Bit 16..19	= Suffix
//					Bit 20..23	= nderungszeit
//					Bit 24..27	= Erstellungszeit
//					Bit 28..31	= Attribute
void CFileListCtrl::SetColumnOrder(UINT uOrder)
{
char	aExist[16];
int		aOrder[8];
int		iCol;
int		iNum;
int		iPos;
int		iMax;
int		iMin;


	memset(aOrder,-1,sizeof(aOrder));


	for(iCol=0;iCol<7;iCol++,uOrder>>=4)					// Positionen auslesen
		{
		iPos = uOrder&0x0F;
		aOrder[iCol] = iPos;
		}

	iMin = 256;
	iMax = 0;

	for(iCol=0;iCol<8;iCol++)
		{
		for(iPos=0;iPos<iCol;iPos++)
			{
			if(aOrder[iCol]!=aOrder[iPos])					// Suche doppelte Positionen
				{
				continue;
				}

			aOrder[iCol]++;
			iPos = -1;
			}

		if(iMin>aOrder[iCol])iMin=aOrder[iCol];
		if(iMax<aOrder[iCol])iMax=aOrder[iCol];
		}


	if(iMin>0)												// Auf untere Grenze
		{
		for(iCol=0;iCol<8;iCol++)
			{
			aOrder[iCol] -= iMin;
			}

		iMax -= iMin;
		}

	if(iMax>6)												// Zu hohe Werte anpassen
		{
		memset(aExist,0,sizeof(aExist));

		for(iCol=0;iCol<8;iCol++)
			{
			aExist[aOrder[iCol]] = 1;
			}

		iNum = 0;

		for(iMin=1;iMin<=iMax;iMin++)
			{
			if(aExist[iMin]==0)iNum++;

			for(iCol=0;iCol<8;iCol++)
				{
				if(aOrder[iCol]!=iMin)continue;
				aOrder[iCol] -= iNum;
				}
			}
		}


	SetColumnOrderArray(8,aOrder);

}

//*****************************************************************************
//*
//*		GetColumnOrder
//*
//*****************************************************************************
//	Holt die Reihenfolge der Spalten
//	Ergibt einen Wert in dem die Reihenfolge der Spalten, jeweils von 4 Bits
//	bestimmen werden:
//			Bit  0...3	= Namen
//			Bit  4...7	= Pfad
//			Bit  8..11	= Gre
//			Bit 12..15	= Dateityp
//			Bit 16..19	= Suffix
//			Bit 20..23	= nderungszeit
//			Bit 24..27	= Erstellungszeit
//			Bit 28..31	= Attribute
UINT CFileListCtrl::GetColumnOrder()
{
int		aOrder[8];
int		iCol;
int		iRet;


	GetColumnOrderArray(8,aOrder);

	iRet = aOrder[7];

	for(iCol=6;iCol>=0;iCol--)
		{
		iRet <<= 4;
		iRet  |= aOrder[iCol];
		}


return iRet;
}

//*****************************************************************************
//*
//*		UpdateStatusBar
//*
//*****************************************************************************
//	Startet die neu ausgabe der Satusbaranzeige
void CFileListCtrl::UpdateStatusBar()
{

	if(!(m_iFlags&CFL_STATUSBAR))
		{
		return;
		}

	if(m_iStatusBarOn)
		{
		m_iStatusBarOn = 1;
		return;
		}

	if(PostMessage(WM_COMMAND,ID_START))
		{
		m_iStatusBarOn = 1;
		}

}

//*****************************************************************************
//*
//*		UpdateSelection
//*
//*****************************************************************************
//	Berechnet die Gre der ausgewhlten Dateien neu.
//	Ergibt die Anzahl der ausgewhlten Dateien.
int CFileListCtrl::UpdateSelection()
{
int		iState;
int		iCount;
int		iItem;


	iCount = GetSelectedCount();


	if(iCount==0 || !m_pLockProc)							// Keine Auswahl
		{
		m_iSelectedCount	= 0;
		m_iSelectedSize		= 0;
		return 0;
		}


	if(iCount==1)											// Nur eine einzelne Datei
		{
		iItem				= GetNextItem(-1,LVNI_CARET);
		m_iSelectedSize		= m_sFiles.Len64(iItem);
		m_iSelectedCount	= 1;

		if(m_iSelectedSize>=0)
			{
			return 1;
			}
		}

															// Status auf User-Werte kopieren
	m_pLockProc(m_pLockParam,0,m_sFiles.Count(),5|LVIF_LOCKING);

	m_iSelectedCount	= 0;
	m_iSelectedSize		= 0;

	for(iItem=m_sFiles.Count();iItem>=0;)					// Auswahl aufsummieren
		{
		iItem--;

		   iState = m_sFiles.GetData(iItem);
		if(iState&LVIS_SELECTED)
			{
			m_iSelectedSize  += m_sFiles.Len64(iItem);
			m_iSelectedCount += 1;
			}
		}

	m_pLockProc(m_pLockParam,0,m_sFiles.Count(),5);


return m_iSelectedCount;
}

//*****************************************************************************
//*
//*		CloseCallback
//*
//*****************************************************************************
//	Das FastList-Contro wird geschlossen
int CALLBACK CFileListCtrl::CloseCallback(LPVOID pParam,HWND hWnd)
{
CFileListCtrl  *pThis=(CFileListCtrl*)pParam;

	pThis->m_sFiles.SetCallback(NULL);
	pThis->m_pLockProc	= NULL;

return 0;
}

//*****************************************************************************
//*
//*		LineCallback
//*
//*****************************************************************************
//	Callback fr den Text der einzelnen Zeilen und Spalten
int CALLBACK CFileListCtrl::LineCallback(LPVOID pParam,LPARAM lParam,UINT uStart,UINT uEnd,FL_ENTRY *pEntries)
{
CFileListCtrl  *pThis=(CFileListCtrl*)pParam;
FfFindStruct   *pInfo=(FfFindStruct *)lParam;
__int64			iSize;
time_t			iTime;
int				iAttr;



	while(uStart<uEnd)
		{
		switch(uStart)
			{
		case 0:												// Dateinamen
				pEntries->iImage = pThis->m_sFiles.GetIcon(pInfo,SHGFI_SMALLICON|SHGFI_USEFILEATTRIBUTES);
				pEntries->pText  = pThis->m_sFiles.Name(pInfo,&pEntries->iSize);
				break;

		case 1:												// Dateipfad
				pEntries->pText  = pThis->m_sFiles.Path(pInfo,0,&pEntries->iSize);
				break;

		case 2:												// Dateigre
				iSize			 = pThis->m_sFiles.Len64(pInfo);
				pEntries->iSize  = vtostr(iSize,pEntries->pText,pThis->m_iSizeMode,VTS_BYTESUFFIX);
				pEntries->pText[pEntries->iSize] = ' '; pEntries->iSize++;
				pEntries->pText[pEntries->iSize] = 0;
				break;

		case 3:												// Dateityp
				pEntries->pText	 = pThis->m_sFiles.GetType(pInfo);
				pEntries->iSize	 = (pEntries->pText)? (int)_tcslen(pEntries->pText):0;
				break;

		case 4:												// Datei-Suffix
				pEntries->pText  = pThis->m_sFiles.Suffix(pInfo,&pEntries->iSize);
				break;

		case 5:												// nderungszeit
				iTime			 = pThis->m_sFiles.TimeWrite(pInfo);
				pEntries->iSize  = strtimeset(pEntries->pText,iTime,pThis->m_iTimeMode);
				break;

		case 6:												// Erstellungszeit
				iTime		 	 = pThis->m_sFiles.TimeCreate(pInfo);
				pEntries->iSize  = strtimeset(pEntries->pText,iTime,pThis->m_iTimeMode);
				break;

		case 7:												// Dateiattribute
				iAttr		 	 = pInfo->dwAttr;
				pEntries->pText[0] = (iAttr&SYS_VOLUME )? 'V':'_';
				pEntries->pText[1] = (iAttr&SYS_SYSTEM )? 'S':'_';
				pEntries->pText[2] = (iAttr&SYS_ARCHIVE)? 'A':'_';
				pEntries->pText[3] = (iAttr&SYS_HIDDEN )? 'H':'_';
				pEntries->pText[4] = (iAttr&SYS_DIR    )? 'D':'_';
				pEntries->pText[5] = (iAttr&SYS_WRITE  )? 'W':'_';
				pEntries->pText[6] = (iAttr&SYS_READ   )? 'R':'_';
				pEntries->iSize  = 7;
				break;
				}

		pEntries++;
		uStart++;
		}


return 1;
}

//*****************************************************************************
//*
//*		GetStyle
//*
//*****************************************************************************
//	Holt die aktuellen Style-Bits
DWORD CFileListCtrl::GetStyle()
{
DWORD	dwTemp;
DWORD	dwRet;


	dwTemp	=  0xFFFF0000&~WS_HSCROLL&~WS_VSCROLL;				// Nur Windos-Styles ohne WS_?SCROLL
	dwRet	=  FastList_GetStyle(m_hWnd);
	dwRet  &=  dwTemp;
	dwRet  |= ~dwTemp&m_iFlags;


return dwRet;
}

//*****************************************************************************
//*
//*		SetStyle
//*
//*****************************************************************************
//	Setzt den Fenster-Style
//	dwStyle		: Sind die neuen Style-Bits:
//					CFL_PATH		0x0001		Pfad anzeigen
//					CFL_SIZE		0x0002		Dateigre anzeigen
//					CFL_TYPE		0x0004		Dateityp anzeigen
//					CFL_SUFFIX		0x0008		Datei-Suffix anzeigen
//					CFL_TIME		0x0010		nderungszeit anzeigen
//					CFL_CREATE		0x0020		Erstellungszeit anzeigen
//					CFL_ATTRIBUTE	0x0040		Dateiattribute
//					CFL_TABEXIT		0x0080		Bei TAB den Fokus ans Elternfenster geben
//					CFL_STATUSBAR	0x0100		Statusbar Nachrichten abschicken
//					CFL_SETCOLUMN	0x0200		Die Spalten mit rechten Mausklick einblenden
//					CFL_SINGLESEL	0x0400		Nur eine Datei ist whlbar
//					CFL_ALLOWPASTE	0x0800		Einfgen per Tastatur erlauben
//					CFL_ALLOWDRAG	0x1000		Dateien per Drag weiter geben
//					CFL_ALLOWDROP	0x2000		Hinzufgen von Dateien per Drag & Drop
//					CFL_CANDELETE	0x4000		Die Dateien knnen mit DEL-Taste entfernt werden
//					CFL_ADDDIR		0x8000		Auch in Unterverzeichnisse einfgen	(Fenster wird nicht aktualisiert)
//					CFL_NORIGHTSEL	WS_POPUP	Keine Auswahl mit der rechneten Maustaste
//					CFL_SORTADD		WS_HSCROLL	Eingefgte Dateien sortieren
//					CFL_SORT		WS_VSCROLL	Sortieren erlaubt
//	dwMask		: Ist die Maske der Bits die gesetzt werden sollen.
//	Ergibt die neuen Style-Bits
DWORD CFileListCtrl::SetStyle(DWORD dwStyle,DWORD dwMask)
{
DWORD	dwTemp;
DWORD	dwRet;
int		iMode;
int		iMask;
int		iCol;
int		iBit;




	dwTemp	= 0xFFFF0000&~WS_HSCROLL&~WS_VSCROLL;				// Nur Windos-Styles ohne WS_?SCROLL

	if(dwMask&(dwTemp|CFL_SINGLESEL))							// ndert sich der Style des Listenfensters
		{
		iMask = dwMask &dwTemp;
		iMode = dwStyle&iMask;

		if(dwMask&CFL_SINGLESEL)
			{
			iMask |= LVS_SINGLESEL;

			if(dwStyle&CFL_SINGLESEL)
				{
				iMode |= LVS_SINGLESEL;
				}
			}

		dwRet  = FastList_SetStyleEx(m_hWnd,iMode,iMask);
		dwRet &= dwTemp;
		}
	else{
		dwRet  = FastList_GetStyle(m_hWnd)&dwTemp;
		}


	if((m_iFlags^dwStyle)&dwMask&0x7F)							// Werden Spalten gendert
		{
		for(iCol=1,iBit=1;iCol<8;iCol++,iBit<<=1)
			{
			if((dwStyle^m_iFlags)&dwMask&iBit)
				{
				OnShowCol(ID_SHOWCOL+iCol);
				}
			}
		}


	if((dwStyle^m_iFlags)&dwMask&CFL_ALLOWDROP)					// ndert sich Drag&Drop
		{
		DragAcceptFiles((CFL_ALLOWDROP&dwStyle)? TRUE:FALSE);
		}

	if((dwStyle^m_iFlags)&dwMask&CFL_NORIGHTSEL)				// ndert sich die Rechtsauswahl
		{
		SetExtendedStyle((dwStyle&CFL_NORIGHTSEL)? LVS_EX_NORIGHTSELECT:0,LVS_EX_NORIGHTSELECT);
		}


	m_iFlags &=  ~dwMask| dwTemp;
	m_iFlags |=   dwMask&~dwTemp&dwStyle;
	dwRet	 |= m_iFlags&~dwTemp;


return dwRet;
}

