//*****************************************************************************
//*
//*
//*		FileListCtrl.h : Header-Datei
//*
//*
//*****************************************************************************
//
//	Funktionen:
//
//	BOOL	Create	(DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId);
//	int		Add		(const char *pPath,int iMode=0,const char *pPlus=0,const char *pMinus=0);
//
//	BOOL	Scroll			(int iRows)				Eintrge Scrollen
//	BOOL	RedrawAllItems	()						Alles neuzeichen
//	BOOL	RedrawItem(int iItem)					Einen Eintag neuzeichen
//	BOOL	SelectItem(int iItem,BOOL bOn=TRUE)		Eintag auswhlen
//	BOOL	SelectAllItems	()						Alle Eintge auswhlen
//	void	ClearSelection	()						Auswahl aufheben
//	void	Sort(int iMode=CFL_SORT_NAME,int iDir)	Sortieren
//	void	Break()									Add abbrechen
//		    
//	BOOL	DeleteAllItems	()						Alle Eintrge lschen
//	BOOL	DeleteSelItems	()						Ausgewhlte Eintrge lschen
//	BOOL	DeleteItem		(int iItem)				Einen Eintrag lschen
//
//	CDat.. *GetFiles		()						Zeiger auf CFileFinder	Klasse
//	char   *GetPathName		(int iItem)				Pfad abfragen
//	char   *GetFileName		(int iItem,int uFull=0)	Namen abfragen
//	int		GetFileLength	(int iItem)				Dateilnge abfragen
//	int64	GetFileLength64 (int iItem)				Dateilnge abfragen (64 Bit)
//	time_t	GetFileTime		(int iItem)				Dateierstellungszeit abfragen
//
//	BOOL	Save(const char *pFilename)				Dateiliste laden
//	BOOL	Load(const char *pFilename,BOOL bNew=1)	Dateiliste speichern
//
//	BOOL	OnCut()									Clipboard Funktionen
//	BOOL	OnCopy()
//	BOOL	OnClear()
//	BOOL	OnPaste()
//
#ifndef		__FILELISTCTRL_H__
#define     __FILELISTCTRL_H__

#if _MSC_VER > 1000
#pragma once
#endif

#include	"FileFinder.h"
#include	"FastListCtrl.h"

#define 	IDW_FILELIST		0xABCF			// Standard uId fr Fenster

// Konstanten fr Add
#define 	CFL_SUBDIR			0x0001			// Auch in Unterverzeichnissen suchen
#define 	CFL_ONLYDIR			0x0002			// Nur Verzeichnisse anzeigen
#define		CFL_ADDUP			0x0004			// Das ".." Verzeichnis einfgen
#define 	CFL_NETSEACH		0x0008			// Virtuelle Netztwerkspfade suchen
#define 	CFL_DIRCMP			0x0010			// Auch Verzeichnisnamen prfen

// Konstanten fr Sort
#define 	CFL_SORT_NAME		CFF_SORT_NAME 	// Nach Dateinamen     sortieren
#define 	CFL_SORT_PATH		CFF_SORT_PATH	// Nach Dateipfad      sortieren
#define 	CFL_SORT_SIZE		CFF_SORT_LEN	// Nach Dateigre     sortieren
#define 	CFL_SORT_TYPE		CFF_SORT_TYPE	// Nach Dateitype      sortieren
#define 	CFL_SORT_TIME		CFF_SORT_WRITE	// Nach Dateizeit      sortieren
#define 	CFL_SORT_SUFFIX		CFF_SORT_SUFFIX	// Nach Dateisuffix    sortieren
#define 	CFL_SORT_CREATE		CFF_SORT_CREATE	// Nach Erstllungszeit sortieren
#define 	CFL_SORT_ACCCESS	CFF_SORT_ACCCESS// Nach Zugiffszeit    sortieren
#define 	CFL_SORT_NODIR		CFF_SORT_NODIR	// Verzeichnisse nicht an den Anfang
#define 	CFL_SORT_VALUE		CFF_SORT_VALUE	// Bercksichtigt Zahlen in den Texten

// Konstanten fr Create
#define 	CFL_DEFAULT			0x0017			// Alle Spalten auer Suffix anzeigen
#define 	CFL_PATH			0x0001			// Pfad anzeigen
#define 	CFL_SIZE			0x0002			// Dateigre anzeigen
#define 	CFL_TYPE			0x0004			// Dateityp anzeigen
#define 	CFL_SUFFIX			0x0008			// Datei-Suffix anzeigen
#define 	CFL_TIME			0x0010			// nderungszeit anzeigen
#define 	CFL_CREATE			0x0020			// Erstellungszeit anzeigen
#define 	CFL_ATTRIBUTE		0x0040			// Dateiattribute
#define 	CFL_TABEXIT			0x0080			// Bei TAB den Fokus ans Elternfenster geben
#define		CFL_STATUSBAR		0x0100			// Statusbar Nachrichten abschicken
#define		CFL_SETCOLUMN		0x0200			// Die Spalten mit rechten Mausklick einblenden
#define 	CFL_SINGLESEL		0x0400			// Nur eine Datei ist whlbar
#define 	CFL_ALLOWPASTE		0x0800			// Einfgen per Tastatur erlauben
#define 	CFL_ALLOWDRAG		0x1000			// Dateien per Drag weiter geben
#define 	CFL_ALLOWDROP		0x2000			// Hinzufgen von Dateien per Drag & Drop
#define 	CFL_CANDELETE		0x4000			// Die Dateien knnen mit DEL-Taste entfernt werden
#define 	CFL_ADDDIR			0x8000			// Auch in Unterverzeichnisse einfgen
#define 	CFL_FULLROWMARK		WS_GROUP		// Die ganze Zeile markiert zeichnen
#define 	CFL_NORIGHTSEL		WS_POPUP		// Keine Auswahl mit der rechneten Maustaste
#define 	CFL_SORTADD			WS_HSCROLL		// Eingefgte Dateien sortieren
#define 	CFL_SORT			WS_VSCROLL		// Sortieren erlaubt

// Konstanten fr Notify-Nachrichten
#define		CFL_STATUSBARSET	0x1206			// StatusBar via Notify anpassen
#define		CFL_DRAGACTION		0x1207			// Begin und Ende von Darg-Aktione

typedef struct
	{
	NMHDR	sNmHdr;								// Notify Daten
	int		iMode;								// 0=Keines 1=Eines 2=Mehrere
	int		iItem;								// Erstes gewhlter Eintrag
	union	{
	struct	{
	int		iSize;								// Byte Gre
	int		iSizeHigh;							// Byte Gre obere 4 Byte
			};
	__int64	iSize64;							// Byte Gre 64-Bit-Format
			};
	int		iCount;								// Dateianzahl
	TCHAR  *pCountText;							// Dateianzahl als Text
	TCHAR  *pSizeText;							// Geamtgre  als Text
	}CFL_STATUS,*LPCFL_STATUS;

typedef struct
	{
	NMHDR		sNmHdr;							// Notify Daten
	int			iItem;							// Erster Ausgewhlter Eintrag
	int			iMode;							// 0=Real 1=Virtuel
	}CFL_DRAGSTATUS,*LPCFL_DRAGSTATUS;


//*****************************************************************************
//*
//*		KLASSE	CFileListCtrl
//*
//*****************************************************************************
class CFileListCtrl : public CFastListCtrl
{
public:		  
									 
	CFileListCtrl();

	BOOL	Create	      	(                DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId=IDW_FILELIST);
	BOOL	CreateEx	  	(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId=IDW_FILELIST);
	BOOL	Create        	(                DWORD dwStyle,UINT        uItem,CDialog *pDlg,UINT nId=IDW_FILELIST);
	BOOL	CreateEx      	(DWORD dwExStyle,DWORD dwStyle,UINT        uItem,CDialog *pDlg,UINT nId=IDW_FILELIST);

	int		Add		      	(LPCTSTR pPath,int iMode=0,LPCTSTR pPlus=0,LPCTSTR pMinus=0,int iProtect=0,int iTimeMode=0,time_t iTimeValue=0,time_t iTimeUntil=0);
	int		Insert		    (LPCTSTR pPath,int iPos=-1,int iExist=0);

	BOOL	RedrawAllItems	();
	BOOL	RedrawItem    	(int iItem);
	BOOL	Scroll        	(int iRows);
	BOOL	SelectItem    	(int iItem,BOOL bOn=TRUE);
	UINT	GetSelectedText	(TCHAR *pText,TCHAR *pSize=NULL);
	BOOL	SelectAllItems	();
	HGLOBAL	CreateSelection ();
	void	ClearSelection	();
	void 	UpdateStatusBar	();
	int		UpdateSelection	();

	BOOL	DeleteAllItems	();
	BOOL	DeleteSelItems	();
	BOOL	DeleteItem    	(int iItem);
	DWORD	GetStyle		();
	DWORD	SetStyle		(DWORD dwStyle,DWORD dwMask=0xFFFF|CFL_SORTADD|CFL_SORT);

	LPCTSTR GetPathName   	(int iItem,int iMode=0,int *pLen=0);
	LPCTSTR GetFileName   	(int iItem,int iMode=0,int *pLen=0);
	LPCTSTR GetPathSuffix  	(int iItem            ,int *pLen=0);
	int	    GetFileLength 	(int iItem);
	__int64	GetFileLength64	(int iItem);
	time_t  GetFileTime   	(int iItem);
	int		GetFileAttribute(int iItem);
	void 	SetGridLines	(BOOL bOn);
	void 	SetAlternated 	(BOOL bOn);
	void 	SetFullRowSelect(BOOL bOn);
	void	Sort          	(int iMode=CFL_SORT_NAME,int iDir=0);
	void	SetTimeMode		(int iMode=STR_DATE_ISO|STR_LONG|STR_TIME             ,BOOL bReDraw=TRUE);
	void	SetSizeMode		(int iMode=VTS_ADDSUFFIX|VTS_ADDSPACE|VTS_BIN10|0x8320,BOOL bReDraw=TRUE);
	void	SetColumnOrder  (UINT uOrder);
	UINT	GetColumnOrder  ();
	void	Break         	();
	int		Count	  		();
	iff64	GetSizeAll		();
	iff64	GetSelectedSize	();
	int 	GetSelectedCount();
	int 	GetSortMode     (int *pDir);

	int     IsNet			(int iItem);						// Ergibt ob Eintrag ein Netzzweig ist
    int     IsDir      		(int iItem);						// Ergibt ob Datei Verzeichnis
	int     IsDrive    		(int iItem);						// Ergibt ob Datei ein Laufwerk ist
	int		IsReadOnly 		(int iItem);						// Ergibt ob Datei schreibgeschtzt ist

	int 	Paste			(HDROP   hDrop);
	BOOL	Save			(LPCTSTR pFilename);
	BOOL	Load			(LPCTSTR pFilename,BOOL bNew=1);
	BOOL	OnCut			();
	BOOL	OnCopy			();
	BOOL	OnClear			();
	BOOL	OnPaste			();
	CFileFinder	*GetFiles 	();
	FfFindStruct*GetFileInfo(int iItem);

public:

	//{{AFX_VIRTUAL(CFileListCtrl)
	//}}AFX_VIRTUAL

public:
	virtual ~CFileListCtrl();

protected:
	//{{AFX_MSG(CFileListCtrl)
	afx_msg void OnDropFiles   (HDROP  hDropInfo);
	afx_msg void OnSetFocus    (CWnd  *pOldWnd  );
	afx_msg void OnKeyDown     (UINT   nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnBeginDrag   (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnColumnClick (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnColumnRClick(NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnItemChanged (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnResSelect   (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnTimer       (UINT_PTR nIdEvent);
	afx_msg void OnShowCol	   (UINT nID);
	afx_msg void OnStart	   ();
	//}}AFX_MSG

	int StatusbarNotify();

	DECLARE_MESSAGE_MAP()

	static	int CALLBACK	LineCallback (LPVOID pParam,LPARAM lItem,UINT uStart,UINT uEnd,FL_ENTRY *pEntries);
	static	int CALLBACK	CloseCallback(LPVOID pParam,HWND hWnd);


//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************
protected:

	int				m_iFlags;						// Fensterstyle
	int				m_iColumnMode;					// Spaltenbits fr Anzeige
	int				m_iColumnMark;					// Welche Spalte ist markiert
	int				m_iColumnMarkOn;				// Darf eine Spalte markiert werden
	int				m_iColumnCount;					// Spaltenanzahl
	int 			m_iSelectedItem;				// Erste gewhlte Dateien
	int				m_iSelectedCount;				// Anzahl der gewhlten Dateien
	__int64			m_iSelectedSize;				// Gre der gewhlten Dateien
	int				m_iSortDir;						// Sortier Richtung
	int				m_iSortMode;					// Sortier Modus
	int				m_iStatusBarOn;					// Statusbar Timer ein
	int				m_iTimeMode;					// Anzeigemodus fr Zeiten
	int				m_iSizeMode;					// Anzeigemodus	fr Dateigre
	static char		m_aSortTypes[8];				// Spaltenart zu Sortierart
	static LPCTSTR	m_aNames[];						// Namen der Spalten und ander
	CFileFinder		m_sFiles;						// Dateiliste
	PFNFLLOCKCB		m_pLockProc;					// Callback fr Fenster
	LPVOID			m_pLockParam;					// Parameter fr Fenster-Callback

};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline BOOL			CFileListCtrl::Create		   (DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId){return CreateEx(0,dwStyle,sRect,pParent,uId);}
inline LPCTSTR		CFileListCtrl::GetFileName     (int iItem,int iMode,int *pLen){return  m_sFiles.Name(iItem,iMode,pLen);}
inline LPCTSTR		CFileListCtrl::GetPathName     (int iItem,int iMode,int *pLen){return  m_sFiles.Path      (iItem,0,pLen);}
inline int			CFileListCtrl::Insert		   (LPCTSTR pPath,int iPos,int iExist){return m_sFiles.Insert(pPath,iPos,iExist);}
inline void			CFileListCtrl::SetFullRowSelect(BOOL bOn            ){SetExtendedStyle((bOn)? LVS_EX_FULLROWSELECT :0,LVS_EX_FULLROWSELECT );}
inline void			CFileListCtrl::SetAlternated   (BOOL bOn            ){SetExtendedStyle((bOn)? LVS_EX_ALTERNATECOLOR:0,LVS_EX_ALTERNATECOLOR);}
inline void			CFileListCtrl::SetGridLines    (BOOL bOn            ){SetExtendedStyle((bOn)? LVS_EX_GRIDLINES     :0,LVS_EX_GRIDLINES     );}
inline BOOL			CFileListCtrl::DeleteAllItems  (                    ){return  m_sFiles.DelAll();}
inline BOOL         CFileListCtrl::RedrawAllItems  (                    ){return  RedrawItems(0,0x0FFFFFFF);}
inline BOOL         CFileListCtrl::RedrawItem      (int iItem           ){return  RedrawItems        (iItem,iItem);}
inline BOOL         CFileListCtrl::SelectItem      (int iItem,BOOL bOn  ){return  SetItemState       (iItem,(bOn)? LVIS_SELECTED:0,LVIS_SELECTED);}
inline LPCTSTR		CFileListCtrl::GetPathSuffix   (int iItem,int *pLen ){return  m_sFiles.Suffix    (iItem,  pLen);}
inline FfFindStruct*CFileListCtrl::GetFileInfo	   (int iItem           ){return  m_sFiles.Info      (iItem);}
inline int	        CFileListCtrl::GetFileLength   (int iItem           ){return  m_sFiles.Len       (iItem);}
inline __int64      CFileListCtrl::GetFileLength64 (int iItem           ){return  m_sFiles.Len64     (iItem);}
inline time_t       CFileListCtrl::GetFileTime     (int iItem           ){return  m_sFiles.Time      (iItem);}
inline int			CFileListCtrl::GetFileAttribute(int iItem			){return  m_sFiles.Attribute (iItem);}
inline int			CFileListCtrl::IsNet		   (int iItem			){return  m_sFiles.IsNet     (iItem);}
inline int			CFileListCtrl::IsDir		   (int iItem			){return  m_sFiles.IsDir     (iItem);}
inline int			CFileListCtrl::IsDrive 		   (int iItem			){return  m_sFiles.IsDrive   (iItem);}
inline int			CFileListCtrl::IsReadOnly	   (int iItem			){return  m_sFiles.IsReadOnly(iItem);}
inline CFileFinder *CFileListCtrl::GetFiles        (                    ){return &m_sFiles        ;}
inline void	        CFileListCtrl::Break           (                    ){        m_sFiles.Break();}
inline int			CFileListCtrl::Count           (                    ){return  m_sFiles.Count();}
inline iff64		CFileListCtrl::GetSizeAll	   (                    ){return  m_sFiles.GetSizeAll();}
inline iff64		CFileListCtrl::GetSelectedSize (                    ){return  m_iSelectedSize;}
inline int 			CFileListCtrl::GetSelectedCount(                    ){return  m_iSelectedCount;}
inline int 			CFileListCtrl::GetSortMode     (int *pDir=0         ){if(pDir)*pDir=m_iSortDir;return  m_iSortMode;}

//{{AFX_INSERT_LOCATION}}


#endif
