//*****************************************************************************
//*
//*
//*		FolderTreeCtrl.cpp: Implementierungsdatei
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//	Anmerkung:		 AfxOleInit() vorher aufrufen, damit Drags funktionieren !
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<AfxWin.h>
#include	<AfxOle.h>
#include	<AfxCmn.h>
#include	"FileFinder.h"
#include	"FileFunctions.h"
#include	"FolderTreeCtrl.h"

#ifdef		_DEBUG
#define new  DEBUG_NEW
#undef		 THIS_FILE
static char  THIS_FILE[] = __FILE__;
#endif

#if         _MFC_VER >= 0x0700
typedef		 LRESULT (AFX_MSG_CALL CWnd::*AfxMsgProc)(WPARAM, LPARAM);
#define 	 AFX_MSG_PROC		(AfxMsgProc)
#else
#define 	 AFX_MSG_PROC
#endif

#ifndef	    _WIN64
#ifndef		 SetWindowLongPtr
#define		 SetWindowLongPtr	SetWindowLong
#endif
#ifndef		 GetWindowLongPtr
#define		 GetWindowLongPtr	GetWindowLong
#endif
#endif

#define 	 FLAG_NET			0x00010000					// Es ist ein virtuelles Netzlaufwerk
#define 	 FLAG_DRIVE			0x00020000					// Es ist ein Laufwerk
#define 	 FLAG_NOSCAN		0x00040000					// Nicht automatisch scannen
#define 	 FLAG_PARTIAL		0x00080000					// Nur erster Pfad gesucht
#define 	 FLAG_DONE			0x00100000					// Alles wurde eingelesen
#define 	 FLAG_LINK			0x00200000					// Angehngeter Pfad ist ein Link
#define 	 FLAG_SET			0x00400000					// Merker fr genderte Eintrge

#define 	 TYPE_SCAN_NONE		0x00000000					// Action wurde gelscht
#define 	 TYPE_SCAN_EXPAND	0x00000001					// Action via OnExpanded
#define 	 TYPE_SCAN_NETWORK	0x00000002					// Action via ScanNetwork
#define 	 TYPE_SCAN_NETWORK	0x00000002					// Action via ScanNetwork
#define 	 TYPE_SCAN_SELECT	0x80000000					// Aus einem Select-Vorgang
#define 	 TYPE_SCAN_BASE		0x40000000					// Ein Basis-Laufwerk scannen
#define 	 TYPE_SCAN_ONLY		0x20000000					// Nur das erste Verzeichnis finden
#define 	 TYPE_SCAN_UPDATE	0x10000000					// Das Verzeichnis sannen
#define 	 TYPE_SCAN_MASK		0xF0000000					// Maske fr Select-Typen

#define 	 MAX_ACTIONS		50							// Maximale Eintrge die upgetatet werden
#define 	 WM_SELECTENTRY		0x0000194F					// Eintrag wurde ausgewlt
#define 	 ID_KEYTICKER		0x00001950					// Ticker fr Tastatur eingaben
#define 	 ID_UPDATETICKER	0x00001951					// Ticker um Ordner zu aktualisieren
#define 	 ID_STOPTICKER		0x00001952					// Ticker um das Einlesen zu beenden
#define 	 ID_STARTUPDATE		0x00001953					// Updates Starten
#define 	 ID_SCANNETWORK		0x00001954					// Die Netzlaufwerke einscannen
#define 	 ID_WAITTICKER		0x00001955					// Ticker fr den Wait-Cursor

#define 	 LOCK()				EnterCriticalSection(&m_sSynch)
#define 	 UNLOCK()			LeaveCriticalSection(&m_sSynch)


BEGIN_MESSAGE_MAP(CFolderTreeCtrl, CTreeListCtrl)
	//{{AFX_MSG_MAP(CFolderTreeCtrl)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_KEYDOWN()
	ON_WM_SETFOCUS()
	ON_WM_DROPFILES()
	ON_NOTIFY_REFLECT(TVN_ITEMDBLCLICK, OnItemDblClick)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED,	OnSelChanged)
	ON_NOTIFY_REFLECT(TVN_BEGINDRAG,	OnBeginDrag)
	ON_NOTIFY_REFLECT(TVN_ITEMEXPANDED,	OnExpanded)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_SCANNETWORK,			OnScanNetwork)	
	ON_COMMAND(ID_STARTUPDATE,			OnStartUpdate)
	ON_MESSAGE(WM_SELECTENTRY,			OnSelectEntry)
	ON_MESSAGE(WM_COPY,AFX_MSG_PROC     OnCopy)
END_MESSAGE_MAP()


HCURSOR		CFolderTreeCtrl::m_hCursorArrow	= NULL;			// Handle vom Pfeil-Cursor
HCURSOR		CFolderTreeCtrl::m_hCursorWait	= NULL;			// Handle vom Warte-Cursor

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFolderTreeCtrl::CFolderTreeCtrl()
{

	m_iFlags			= 0;								// Fenster-Style
	m_iNetFlags			= CFF_FIND_NET;						// Such-Flags fr Laufwerke
	m_hOpenTreeItem		= 0;								// Offener Ordner fr Treefeld
	m_hCurrentItem		= 0;								// Ist der aktuell gewhlte Eintrag
	m_cCurrentPath[0]	= 0;								// Speicher fr aktuellen Pfad
	m_iCurrentNumber	= 0;								// Aktuelle Aktionsnummer
	m_iCurrentAction	= 0;								// Aktionsnummer die ausgefhrt wird
	m_iCurrentMode		= 0;								// 0=Netzwerkszweig 1=Realer Pfad 2=Laufwerk
	m_uUpdateTicker		= 0;								// Ticks bis zum nchsten Update
	m_bNetScanned		= 0;								// Wurde das Netzwerk gescannt
	m_uStopTicker		= 0;								// Ticks bis zum Lschen von Suchen
	m_uKeyTicker		= 0;								// Ticks bei Tastatureingaben
	m_lWaitCount		= 0;								// Wievile wichtige Auftrge warten
	m_iThreadRunning    = 0;								// Luft der Aktions-Thread

	InitializeCriticalSection(&m_sSynch);

	m_sActionList.DataLength(offsetof(ActionItem,cText));
	m_sNetPaths  .DataLength(sizeof(HTREEITEM));

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFolderTreeCtrl::~CFolderTreeCtrl()
{

	if(m_iThreadRunning)StopThread();

	DeleteCriticalSection(&m_sSynch);

}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Erzeugt das Fenster
//	dwExStyle	: Erweiterte Flags fr das Fenster
//	dwStyle		: Sind die Flags fr das Aussehen des Fensters
//					CFT_TABEXIT			Bei TAB den Fokus ans Elternfenster geben
//					CFT_ALLOWDRAG		Dateien per Drag weiter geben
//					CFT_ALLOWDROP		Hinzufgen von Dateien per Drag & Drop
//					CFT_NORIGHTSEL		Keine Auswahl mit der rechneten Maustaste
//					CFT_NONETDIRS		Keine Netzlaufwerke anzeigen
//	sRect		: Ist die Position des Fensters im Elternfenster
//	pParent		: Zeiger auf das Elternfenster
//	uId			: Ist die ID fr das Fenster
//	Ergibt TRUE wenn das Fenster erzeugt werden konnte.
BOOL CFolderTreeCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId)
{
DWORD			dwThread;
HANDLE			hThread;
HIMAGELIST		hImages;
SHFILEINFO		sShInfo;
int				iBits;
int				iMode;



	#ifdef	_DEBUG
	if(dwStyle&CFT_ALLOWDRAG)
		{
		_AFX_THREAD_STATE *pState = AfxGetThreadState();
		ASSERT(pState->m_bNeedTerm);						// Wurde AfxOleInit() aufgerufen ?
		}
	#endif


	iMode = dwStyle&0xFFFF0000;

	if(!CTreeListCtrlEx::CreateEx(dwExStyle,WS_CHILD|TVS_HASBUTTONS|TVS_HASLINES|TVS_LINESATROOT|iMode,sRect,pParent,uId))
		{
		return FALSE;
		}

	hImages = (HIMAGELIST)SHGetFileInfo(_T(""),0,&sShInfo,sizeof(sShInfo),SHGFI_PIDL|SHGFI_SMALLICON|SHGFI_SYSICONINDEX);

	iBits = (dwStyle&CFT_NORIGHTSEL)? TVS_EX_NORIGHTSELECT|TVS_EX_SHAREIMAGELISTS:TVS_EX_SHAREIMAGELISTS;

	SetExtendedStyle(iBits);
	SetImageList(hImages,TVSIL_NORMAL);

	InsertColumn(0,_T("Ordner"),TVCFMT_LEFT,-1);
	SetOption(TVOP_MAXFIRSTCOLUMN,2);

	if(dwStyle&CFT_ALLOWDROP )DragAcceptFiles(TRUE);
	if(dwStyle&CFT_NONETDIRS )m_iNetFlags=0;

	m_iFlags				= dwStyle&0xFFFF;
	m_hNetworkItem			= TVI_ROOT;

	/*
	sInsert.hParent			= TVI_ROOT;
	sInsert.hInsertAfter 	= TVI_LAST;
	sInsert.item.mask		= TVIF_STATE|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_CHILDREN;
	sInsert.item.stateMask	= 0xFFFF0000;
	sInsert.item.cchTextMax	= 256;
	sInsert.item.cChildren	= 1;

	m_sFolders.Set(_T(":"),CFF_FIND_DIR|m_iNetFlags);

	SetRedraw(FALSE);


	for(;;)													// Laufwerke einlesen
		{
			pName=m_sFolders.Next();
		if(!pName)break;

		iIcon	=  m_sFolders.GetSmallIcon();

		if(m_sFolders.IsNet())
				iFlag=FLAG_NET|FLAG_NOSCAN;
		else	iFlag=IsFixedDrive(pName)? FLAG_DRIVE:FLAG_DRIVE|FLAG_NOSCAN;

		sInsert.item.state		 	= iFlag;
		sInsert.item.pszText	 	= pName;
		sInsert.item.iImage		 	= iIcon;
		sInsert.item.iSelectedImage	= iIcon;

		hItem	= InsertItem(&sInsert);

		if(m_sFolders.IsNet())								// Hier Netztlaufwerke einfgen
			{
			m_hNetworkItem = hItem;
			iNet	= m_sNetPaths.Add(2,_T("\\\\"),CSC_ABCG,&hItem);
			pName	= m_sNetPaths.Get(iNet);
			SetItemDataEx(hItem,(LPARAM)pName);
			}
		}
	
	SetRedraw(TRUE);

	*/

	if(!m_hCursorArrow)m_hCursorArrow=LoadCursor(NULL,IDC_ARROW);
	if(!m_hCursorWait )m_hCursorWait =LoadCursor(NULL,IDC_WAIT );
	
	m_uWaitTicker    = 0;
	m_lWaitCount     = 0;		
	m_iStopThread	 = 0;
	m_iThreadRunning = 1;

	hThread = CreateThread(NULL,0xFF00,ActionThread,this,0,&dwThread);

	CloseHandle(hThread);


return TRUE;
}

//*****************************************************************************
//*
//*		SortDrives
//*
//*****************************************************************************
int CALLBACK CFolderTreeCtrl::SortDrives(HWND hWnd,HTREEITEM hItem,LPCTSTR pTextItem,LPCTSTR pTextInsert,LPARAM lParamItem,LPARAM lParamInsert)
{
unsigned	uDrive1;
unsigned	uDrive2;
unsigned	uSign;
int			iDiff;


	   uSign = pTextItem[0];
	if(uSign>='a')uSign-=0x20;

	if(uSign>='A' && uSign<='Z' && pTextItem[1]==':' && pTextItem[2]==0)
			uDrive1 = uSign-'A';
	else	uDrive1 = 0x80;

	   uSign = pTextInsert[0];
	if(uSign>='a')uSign-=0x20;

	if(uSign>='A' && uSign<='Z' && pTextInsert[1]==':' && pTextInsert[2]==0)
			uDrive2 = uSign-'A';
	else	uDrive2 = 0x80;

	   iDiff = uDrive2-uDrive1;
	if(iDiff)return iDiff;


return _tcsicmp(pTextInsert,pTextItem);
}

//*****************************************************************************
//*
//*		InsertDrives
//*
//*****************************************************************************
//	Fgt alle Laufwerke ein
void CFolderTreeCtrl::InsertDrives()
{
TVINSERTSTRUCT	sInsert;
HTREEITEM		hFocus;
HTREEITEM		hItem;
LPTSTR			pName;
int				iFlag;
int				iIcon;
int				iNet;
int				iVal;


	sInsert.hParent			= TVI_ROOT;
	sInsert.hInsertAfter 	= TVI_SORTEX;
	sInsert.item.stateMask	= 0xFFFF0000;
	sInsert.item.cchTextMax	= 256;
	sInsert.item.cChildren	= 1;

	m_sFolders.Set(_T(":"),CFF_FIND_DIR|m_iNetFlags);


	for(;;)													// Laufwerke einlesen
		{
			pName=m_sFolders.Next();
		if(!pName)break;

		iIcon	=  m_sFolders.GetSmallIcon();

		if(m_sFolders.IsNet())
				iFlag=FLAG_NET|FLAG_NOSCAN;
		else	iFlag=IsFixedDrive(pName)? FLAG_DRIVE:FLAG_DRIVE|FLAG_NOSCAN;


		   hFocus = GetFocusItemEx();
		if(hFocus)
			{
			   iVal = IsItemVisibleEx(hFocus);
			if(iVal<1)hFocus = NULL;
			}

		LOCK();
		
			hItem = FindItemEx(TVI_ROOT,TVIF_CHILDS|TVIF_TEXT,0,pName);
		if(!hItem)
			{
			sInsert.item.mask			= TVIF_STATE|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_CHILDREN;
			sInsert.item.hItem			= (HTREEITEM)SortDrives;
			sInsert.item.state		 	= iFlag;
			sInsert.item.pszText	 	= pName;
			sInsert.item.iImage		 	= iIcon;
			sInsert.item.iSelectedImage	= iIcon;

			hItem	= InsertItemEx(&sInsert);
			}
		else{
			sInsert.item.mask			= TVIF_IMAGE|TVIF_SELECTEDIMAGE;
			sInsert.item.hItem			= hItem;
			sInsert.item.iImage		 	= iIcon;
			sInsert.item.iSelectedImage	= iIcon;

			SetItemEx(&sInsert.item);

			hFocus = NULL;
			}

		UNLOCK();


		if(hFocus && IsItemVisibleEx(hFocus)<4)
			{			
			PostMessage(TVM_ENSUREVISIBLE,(iVal==4)? 0:0xFFFFFFFF,(LPARAM)hFocus);
			}

		if(m_sFolders.IsNet())								// Hier Netztlaufwerke einfgen
			{
			m_hNetworkItem = hItem;
			iNet	= m_sNetPaths.Add(2,_T("\\\\"),CSC_ABCG,&hItem);
			pName	= m_sNetPaths.Get(iNet);
			SetItemDataEx(hItem,(LPARAM)pName);
			}
		}

	

}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Erzeugt das Fenster	in einem Dialog, ber einen Rechteck.
//	dwExStyle	: Erweiterte Flags fr das Fenster
//	dwStyle		: Sind die Flags fr das Aussehen des Fensters
//					CFT_ALLOWDROP		Hinzufgen von Datei per Drag & Drop
//					CFT_ALLOWDRAG		Dateien per Drag weiter geben
//					CFT_ALLOWPASTE		Einfgen per Tastatur erlauben
//					CFT_SINGLESEL		Nur eine Datei ist whlbar
//					CFT_SORT			Sortieren erlaubt
//	uItem       : Ist die ID des Rechtecks im Dialog.
//	pDlg		: Zeiger auf den Eltern-Dialog.
//	uId			: Ist die ID fr das Fenster.
//	Ergibt TRUE wenn das Fenster erzeugt werden konnte.
BOOL CFolderTreeCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,UINT uItem,CDialog *pDlg,UINT nId)
{
CPoint	sPoint(0,0);
CRect	sRect;
CWnd   *pWnd;


	pWnd=pDlg->GetDlgItem(uItem);
	ASSERT(pWnd!=0);


	pWnd->GetWindowRect (&sRect);
	pDlg->ClientToScreen(&sPoint);

	sRect.left	  -= sPoint.x;
	sRect.right   -= sPoint.x;
	sRect.top	  -= sPoint.y;
	sRect.bottom  -= sPoint.y;


	if(!CFolderTreeCtrl::CreateEx(dwExStyle,dwStyle|WS_CHILD|WS_VISIBLE|WS_HSCROLL|WS_VSCROLL,sRect,pDlg,nId))
		{
		return FALSE;
		}

	SetWindowPos(pWnd,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOREDRAW);


return TRUE;
}

//*****************************************************************************
//*
//*		StopThread
//*
//*****************************************************************************
//	Warte bis alle Threads gestopt sind
void CFolderTreeCtrl::StopThread()
{
	
	m_iStopThread = 1;
	m_sNextAction.Unlock();

	while(m_iThreadRunning)
		{
		m_sNextAction.Unlock();
		Sleep(50);
		}

}

//*****************************************************************************
//*
//*		OnDestroy
//*
//*****************************************************************************
void CFolderTreeCtrl::OnDestroy()
{

	StopThread();

	CTreeListCtrl::OnDestroy();

}

//*****************************************************************************
//*
//*		OnKeyDown
//*
//*****************************************************************************
void CFolderTreeCtrl::OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags)
{
CWnd   *pParent;


	if(nChar==VK_TAB && (m_iFlags&CFT_TABEXIT))				// Zu anderem Fenster springen
		{
		   pParent=GetParent();
		if(pParent)pParent->SetFocus();
		return;
		}



CTreeListCtrl::OnKeyDown(nChar,nRepCnt,nFlags);
}

//*****************************************************************************
//*
//*		OnDropFiles
//*
//*****************************************************************************
//	Werden Dateien auf dem Fenster fallen gelassen.
void CFolderTreeCtrl::OnDropFiles(HDROP hDropInfo)
{

	if(m_iFlags&CFT_ALLOWDROP)
		{
		}

}

//*****************************************************************************
//*
//*		OnItemDblClick
//*
//*****************************************************************************
//	Im Baumfenster wurde ein dobel klick gemacht
void CFolderTreeCtrl::OnItemDblClick(NMHDR *pNmHdr,LRESULT *pResult)
{
NMTREEVIEW *pNotify = (NMTREEVIEW*)pNmHdr;
TCHAR		cBuffer[MAX_PATH];
CFT_FOLDER	sNotify;
HTREEITEM   hItem; 
int			iLen;


    hItem = pNotify->itemNew.hItem;

	if(hItem)
		{
		m_uKeyTicker = 0;

		   iLen = GetPath(hItem,cBuffer,MAX_PATH);
		if(iLen>0)
			{
			SelectEntry(hItem,cBuffer,iLen,0);
			}

	    *pResult = 1;

		LOCK();

		sNotify.sNmHdr.code		= CFT_ITEMDBLCLICK;
		sNotify.sNmHdr.hwndFrom	= m_hWnd;
		sNotify.sNmHdr.idFrom	= GetWindowLong(m_hWnd,GWL_ID);
		sNotify.iMode			= m_iCurrentMode;
		sNotify.pPath			= m_cCurrentPath;
		sNotify.hItem			= hItem;
																// Eltern-Fenster benachrichtigen
		GetParent()->SendMessage(WM_NOTIFY,sNotify.sNmHdr.idFrom,(LPARAM)&sNotify);

		UNLOCK();
		}

} 

//*****************************************************************************
//*
//*		OnSelChanged
//*
//*****************************************************************************
void CFolderTreeCtrl::OnSelChanged(NMHDR *pNmHdr,LRESULT *pResult)
{
NM_TREEVIEW	   *pNotify=(NM_TREEVIEW*)pNmHdr;
TCHAR			cBuffer[MAX_PATH];
int				iLen;


	if(!(pNotify->itemNew.state&TVIS_SELECTED))				// Nicht ausgewhlt
		{
		return;
		}

	if(pNotify->itemNew.hItem==pNotify->itemOld.hItem)		// Keine neue Auswahl
	if(pNotify->itemOld.state&TVIS_SELECTED)				// Nicht ausgewhlt
		{
		return;
		}

	if(pNotify->itemNew.hItem==m_hCurrentItem)				// Keine nderung
		{
		m_uKeyTicker = 0;
		return;
		}

	if(pNotify->action==TVC_BYKEYBOARD)						// Wahl ber Tastatur
		{
		if(!m_uKeyTicker)
			{
			SetTimer(ID_KEYTICKER,200,NULL);
			}

		m_uKeyTicker = 5;

		return;
		}


	m_uKeyTicker = 0;

	   iLen = GetPath(pNotify->itemNew.hItem,cBuffer,MAX_PATH);
	if(iLen>0)
		{
		SelectEntry(pNotify->itemNew.hItem,cBuffer,iLen,0);
		}


}

//*****************************************************************************
//*
//*		OnBeginDrag
//*
//*****************************************************************************
void CFolderTreeCtrl::OnBeginDrag(NMHDR *pNmHdr,LRESULT *pResult)
{
NM_TREEVIEW	   *pNotify=(NM_TREEVIEW*)pNmHdr;
HGLOBAL			hGlobal;
CFT_FOLDER		sNotify;
COleDataSource	sDrag;
STGMEDIUM		sData;
LPTSTR			pMem;
int				iLen;


	if(!(m_iFlags&CFT_ALLOWDRAG))							// Drag erlaubt ?
		{
		*pResult = 0;
		return;
		}

	if(pNotify->itemNew.state&FLAG_NET)
		{
		*pResult = 0;
		return;
		}

	LOCK();

	sNotify.sNmHdr.code		= CFT_DRAGACTION;
	sNotify.sNmHdr.hwndFrom	= m_hWnd;
	sNotify.sNmHdr.idFrom	= GetWindowLong(m_hWnd,GWL_ID);
	sNotify.iMode			= 0;
	sNotify.hItem			= pNotify->itemNew.hItem;
	sNotify.pPath			= m_cCurrentPath;
															// Eltern-Fenster benachrichtigen
	if(GetParent()->SendMessage(WM_NOTIFY,sNotify.sNmHdr.idFrom,(LPARAM)&sNotify))
		{
		UNLOCK();
		return;
		}

	UNLOCK();

															// HGLOBAL mit Pfad erzeugen
	hGlobal	 = GlobalAlloc(GMEM_ZEROINIT,MAX_PATH*sizeof(TCHAR)+24);
	pMem	 = (LPTSTR)GlobalLock(hGlobal);
	pMem[0]	 = 20;
	#ifdef UNICODE
	pMem[8]  = 1;
	#endif
	pMem	+= 20/sizeof(TCHAR);
	iLen	 = GetPath(pNotify->itemNew.hItem,pMem,MAX_PATH);
	iLen    += 24;

	GlobalUnlock(hGlobal);

	sData.pUnkForRelease	= 0;
	sData.hGlobal			= hGlobal;
	sData.tymed				= TYMED_HGLOBAL;

	if(m_iFlags&CFT_ALLOWDROP)DragAcceptFiles(FALSE);

	sDrag.CacheGlobalData(CF_HDROP,sData.hGlobal);
	sDrag.DoDragDrop();
	sDrag.Empty();

	if(m_iFlags&CFT_ALLOWDROP)DragAcceptFiles(TRUE);

	LOCK();

	sNotify.iMode			= 1;
															// Eltern-Fenster benachrichtigen
	GetParent()->SendMessage(WM_NOTIFY,sNotify.sNmHdr.idFrom,(LPARAM)&sNotify);

	UNLOCK();

	*pResult = 0;

}

//*****************************************************************************
//*
//*		OnExpanded
//*
//*****************************************************************************
//	Ein Eintrag wurde aufgeklappt
void CFolderTreeCtrl::OnExpanded(NMHDR *pNmHdr,LRESULT *pResult)
{
NM_TREEVIEW	   *pNotify=(NM_TREEVIEW*)pNmHdr;
ActionItem		sAction;


	if(pNotify->itemNew.state&FLAG_DONE)
		{
		*pResult = 0;
		return;
		}

	InterlockedIncrement(&m_lWaitCount);

	if(!m_uWaitTicker)
		{
		m_uWaitTicker = 2;
		SetTimer(ID_WAITTICKER,300,NULL);
		}

	sAction.hWait	= 0;
	sAction.hItem	= pNotify->itemNew.hItem;
	sAction.iType	= TYPE_SCAN_EXPAND;
	sAction.bFlag	= 1;

	LOCK();
	m_sActionList.Add(0,0,CSC_END,&sAction);
	m_sNextAction.Unlock();
	UNLOCK();

}

//*****************************************************************************
//*
//*		OnSetFocus
//*
//*****************************************************************************
void CFolderTreeCtrl::OnSetFocus(CWnd* pOldWnd)
{

	CTreeListCtrl::OnSetFocus(pOldWnd);

	//UpdateStatusBar();

}


//*****************************************************************************
//*
//*		OnTimer
//*
//*****************************************************************************
void CFolderTreeCtrl::OnTimer(UINT_PTR nIdEvent)
{
TCHAR			cBuffer[MAX_PATH];
ActionItem	   *pAction;
HTREEITEM		hItem;
int				iBits;
int				iPos;
int				iLen;



	if(nIdEvent==ID_KEYTICKER)							// Eingabe via Tastatur
		{
		if(m_uKeyTicker==0)
			{
			KillTimer(ID_KEYTICKER);
			return;
			}

			m_uKeyTicker--;
		if(!m_uKeyTicker)
			{
			hItem = GetFocusItemEx();

			   iLen = GetPath(hItem,cBuffer,MAX_PATH);
			if(iLen>0)
				{
				SelectEntry(hItem,cBuffer,iLen,0);
				}
			}

		return;
		}

	if(nIdEvent==ID_WAITTICKER)							// Wait-Cursor einblenden
		{
		POINT	sPoint;		

		if(m_lWaitCount==0)								// Alle Jobs abgearbeitet	
			{
			m_uWaitTicker = 0;
			KillTimer(ID_WAITTICKER);

			if(GetCursorPos(&sPoint) && ::WindowFromPoint(sPoint)==m_hWnd)
				{
				SetCursor(m_hCursorArrow);
				}

			return;
			}

		if(m_uWaitTicker%2==0)							// Cursor auf Wait setzen
			{
			if(GetCursorPos(&sPoint) && ::WindowFromPoint(sPoint)==m_hWnd)
				{
				SetCursor(m_hCursorWait);
				}
			}
		
		m_uWaitTicker++;
		}

	if(nIdEvent==ID_UPDATETICKER)						// Weitere Ordner einlesen
		{
		if(m_uUpdateTicker==0)
			{
			KillTimer(ID_UPDATETICKER);
			return;
			}

			m_uUpdateTicker--;
		if(!m_uUpdateTicker)
			{
			ScanUpdate();
			}

		return;
		}

	if(nIdEvent==ID_STOPTICKER)							// Eine Suche anhalten
		{
		if(m_uStopTicker==0)
			{
			KillTimer(ID_STOPTICKER);
			return;
			}

			m_uStopTicker--;
		if(!m_uStopTicker)
			{
			LOCK();

			for(iPos=m_sActionList.Count()-1;iPos>=0;iPos--)
				{
				pAction = (ActionItem*)m_sActionList.DataNone(iPos);
				iBits	= pAction->iType;

				if(!(iBits&TYPE_SCAN_MASK))				// Kein Select-Typ
					{
					continue;
					}

				iBits  ^=  m_iCurrentNumber;
				iBits  &= ~TYPE_SCAN_MASK;

				if(iBits)continue;

				pAction->iType = TYPE_SCAN_NONE;
				}

			if(m_iCurrentAction&TYPE_SCAN_MASK)			// Aktuelle Suche abbrechen
				{
				iBits	=  m_iCurrentAction;
				iBits  &= ~TYPE_SCAN_MASK;

				if(iBits)
					{
					m_sFolders.Break();
					}
				}

			UNLOCK();
			}

		return;
		}


	CTreeListCtrl::OnTimer(nIdEvent);

}

//*****************************************************************************
//*
//*		OnStartUpdate
//*
//*****************************************************************************
void CFolderTreeCtrl::OnStartUpdate()
{

	if(!m_uUpdateTicker)
		{
		SetTimer(ID_UPDATETICKER,200,NULL);
		}

	m_uUpdateTicker = 4;

}

//*****************************************************************************
//*
//*		ActionThread
//*
//*****************************************************************************
DWORD WINAPI CFolderTreeCtrl::ActionThread(LPVOID pParam){return ((CFolderTreeCtrl*)pParam)->ActionThread();}
DWORD		 CFolderTreeCtrl::ActionThread()
{
TCHAR			cBuffer[MAX_PATH];
ActionItem	   *pEntry;
ActionItem	   *pAction;
TV_ITEM			sChange;
HTREEITEM		hChild;
int				iState;
int				iDone;
int				iMode;
int				iPos;
int				iLen;


	InsertDrives();

	for(;;)
		{
		m_sNextAction.Lock();									// Warte auf nchsten Auftrag
		if(m_iStopThread)break;

		LOCK();
		pAction = (ActionItem*)m_sActionList.Data(0);

		if(!pAction)
			{
			UNLOCK();
			continue;
			}

		if(pAction->iType==TYPE_SCAN_NONE)						// Wurde die Aktion gelscht
			{
			if(pAction->bFlag)InterlockedDecrement(&m_lWaitCount);
			m_sActionList.Del(0);
			UNLOCK();
			continue;
			}

		m_iCurrentAction = pAction->iType;

		UNLOCK();


		if(pAction->cText[0])									// Einen Pfad aktualisieren
			{
			strlcpy(cBuffer,pAction->cText,MAX_PATH);
			pAction->hItem = FindPath(cBuffer);
			iState = (cBuffer[0]=='\\' || cBuffer[0]=='/')? FLAG_NET:0;
			iLen   = (int)_tcslen(cBuffer);
			}
		else{													// Einen Eintrg aktualisieren
			iLen   = GetPath(pAction->hItem,cBuffer,MAX_PATH,0,&iState);
			}

		if(iLen==2 && cBuffer[1]==':')
			{
			cBuffer[2] = '\\';
			cBuffer[3] = 0;
			iLen++;
			}

		if(pAction->hItem)										// Einlesen
			{
			m_sFolders.DelAll();

			if(iState&FLAG_NET)
				{
				m_sFolders.Sort(CFF_SORT_NAME,CFF_SORT_VALUE);
				m_sFolders.Read(cBuffer,CFF_FIND_DIR|CFF_FIND_SORT|CFF_FIND_NET);
				}
			else{
				if(m_iCurrentAction&TYPE_SCAN_ONLY)
						iMode=CFF_FIND_DIR|CFF_FIND_FIRST;
				else	iMode=CFF_FIND_DIR;
				m_sFolders.Read(cBuffer,iMode);
				}

			   iDone = m_sFolders.DoneFlag();
			if(iDone==0)										// Alles eingelesen
				{
				SynchPath(pAction->hItem,&m_sFolders,pAction->iType!=TYPE_SCAN_EXPAND);
				}
			else if(iDone==1)									// Nur bis zum ersten
				{
				hChild = GetChildItemEx(pAction->hItem);
				iState = GetItemStateEx(pAction->hItem,0xFFFFFFFF);

				if(!m_sFolders.Count())							// Keine Ordner gefunden
					{
					if(hChild)DeleteChildItemsEx(pAction->hItem);
					sChange.hItem		= pAction->hItem;
					sChange.mask		= TVIF_STATE|TVIF_CHILDREN;
					sChange.state		= FLAG_DONE|TVIS_EXPANDED|TVIS_EXPANDPARTIAL;
					sChange.stateMask	= FLAG_DONE|FLAG_PARTIAL;
					sChange.cChildren	= 0;
					SetItemEx(&sChange);
					}											// Ein Ordner wurde gefunden
				else if(!(iState&FLAG_DONE) && !(iState&TVIS_EXPANDED))
					{
					sChange.hItem		= pAction->hItem;
					sChange.mask		= TVIF_STATE|TVIF_CHILDREN;
					sChange.state		= FLAG_PARTIAL;
					sChange.stateMask	= FLAG_PARTIAL|FLAG_DONE;
					sChange.cChildren	= 1;
					SetItemEx(&sChange);
					}
				}
			}

		LOCK();

		m_iCurrentAction = 0;

		if(pAction->hWait)
			{
			ReleaseSemaphore(pAction->hWait,1,NULL);
			}

		for(iPos=0;iPos<m_sActionList.Count();iPos++)
			{
			   pEntry = (ActionItem*)m_sActionList.DataNone(iPos);
			if(pEntry!=pAction)continue;
			if(pEntry->bFlag)InterlockedDecrement(&m_lWaitCount);

			m_sActionList.Del(iPos);

			break;
			}

		UNLOCK();

		if(m_sActionList.Count()==0)
			{
			PostMessage(WM_COMMAND,ID_STARTUPDATE);
			}
		}


	m_iThreadRunning = 0;


return 0;
}

//*****************************************************************************
//*
//*		IsFixedDrive
//*
//*****************************************************************************
//	Erzeugt aus einem Netz-Pfad einen realen Pfad.
//	pPath		: Aktueller Pfad aus Shell.
//	pBuffer		: Speicher fr realen Pfad.
//	iMax		: Gre des Speichers in Zeichen.
//	pIsNet		: Hier wird gespeichert um wechem Pfad es sich handelt.
//					0 = Realer Pfad
//					1 = Virtueller Pfad
//					2 = Basis Netzpfad	("\\")
//	Ergibt die Lnge des Pfades in Zeichen.
BOOL CFolderTreeCtrl::IsFixedDrive(LPCTSTR pPath)
{
TCHAR	cPath[4];
int		iType;


	if(pPath[0]=='\0')return FALSE;
	if(pPath[0]=='\\')return FALSE;
	if(pPath[0]=='/' )return FALSE;
	if(pPath[1]!=':' )return FALSE;

	cPath[0] = pPath[0];
	cPath[1] = pPath[1];
	cPath[2] = 0;

	iType = GetDriveType(cPath);

	if(iType==DRIVE_FIXED || iType==DRIVE_RAMDISK)
		{
		return TRUE;
		}


return FALSE;
}

//*****************************************************************************
//*
//*		GetFullNetPath
//*
//*****************************************************************************
//	Erzeugt aus einem Netz-Pfad einen realen Pfad.
//	pPath		: Aktueller Pfad aus Shell.
//	pBuffer		: Speicher fr realen Pfad.
//	iMax		: Gre des Speichers in Zeichen.
//	pIsNet		: Hier wird gespeichert um wechem Pfad es sich handelt.
//					0 = Realer Pfad
//					1 = Virtueller Pfad
//					2 = Basis Netzpfad	("\\")
//	Ergibt die Lnge des Pfades in Zeichen.
int CFolderTreeCtrl::GetFullNetPath(LPCTSTR pPath,int iSize,TCHAR *pBuffer,int *pIsNet)
{
CFileFinder		sFind;
CString			sSubPath;
TCHAR		    cStr [MAX_PATH];
TCHAR		    cName[MAX_PATH];
LPCTSTR			pName;
int				iCount;
int				iStop;
int				iNet;
int				iPos;
int				iLen;




	if((pPath[0]!='\\' && pPath[0]!='/') || (pPath[1]!='\\' && pPath[1]!='/'))
		{
		if(pIsNet)*pIsNet=0;
		return GetFullPathName(pPath,iSize,pBuffer,0);
		}

	if(pIsNet)*pIsNet=2;

	sSubPath = _T("\\\\");
	pName	 = pPath;
	iStop	 = 0;
	iPos	 = 2;

	while(pName[iPos] && !iStop)							// Netzpfad durchlaufen
		{
			iCount=strcwlen(pName+iPos,0x00300000);
		if(!iCount)
			{
			if(pName[iPos+iCount]=='\\'){iPos++;continue;}
			if(pName[iPos+iCount]=='/' ){iPos++;continue;}
			break;
			}

		memcpy(cName,pName+iPos,iCount*sizeof(TCHAR));
		cName[iCount]=0;

		memcpy(cStr,pName,iPos*sizeof(TCHAR));

		if(iPos>2 && cStr[iPos-1]!='\\' && cStr[iPos-1]!='/')
				 {cStr[iPos]='\\';cStr[iPos+1]=0;}
 		else 	  cStr[iPos]=0;

	 	if(sFind.Read(cStr,CFF_FIND_DIR|m_iNetFlags,cName)==1)
		if(sFind.IsNet(0)==0)								// Realer Pfad ?
			{
			sSubPath	= sFind.Path((int)0);
			iLen		= sSubPath.GetLength();
			sSubPath   += pName+iPos;

			if(pIsNet)
				{
				for(iLen=iPos;pName[iLen];iLen++)
					{
					if(pName[iLen]=='\\'){iLen++;break;}
					if(pName[iLen]=='/' ){iLen++;break;}
					}

				*pIsNet=(pName[iLen])? 0:1;
				}

			pName	= sSubPath;
			iStop	= 1;
			iNet	= 0;

			break;
			}

		if(sFind.IsArchive(0))								// Ist es ein Link
			{
 			   pName = sFind.Link(0L,&iLen);
			if(pName && *pName)
				{
				if(iLen>0 && pName[iLen-1]=='\\')iLen--;

				strlcpy(sSubPath.GetBuffer(iLen),pName,iLen+1,iLen);
				sSubPath.ReleaseBuffer(iLen);

				if(pIsNet)*pIsNet=1;

				pName	= sSubPath;
				iStop	= 1;
				iNet	= 0;

				break;
				}
			}

		iPos+=iCount;
		}

	if(iStop)
		{
		iLen = sSubPath.GetLength();

		if(iLen>iSize-1)iLen=iSize-1;
		if(iLen<0)return 0;

		memcpy(pBuffer,(LPCTSTR)sSubPath,iLen*sizeof(TCHAR));
		pBuffer[iLen]=0;
		}
	else{
		if(pBuffer!=pPath)
			{
			memmove(pBuffer,pPath,iPos);
			pBuffer[iPos]=0;
			}

		iLen = iPos;
		}



return iLen;
}

//*****************************************************************************
//*
//*		OnScanNetwork
//*
//*****************************************************************************
void CFolderTreeCtrl::OnScanNetwork()
{

	m_bNetScanned = TRUE;
	ScanNetwork(m_hNetworkItem);

}

//*****************************************************************************
//*
//*		ScanNetwork
//*
//*****************************************************************************
BOOL CFolderTreeCtrl::ScanNetwork(HTREEITEM hRoot,UINT uTimeout)
{
ActionItem	   *pAction;
ActionItem		sAction;
HTREEITEM		hItem;
DWORD			uStart;
DWORD			uDiff;
DWORD			uWait;
BOOL			bOk;



	uStart			= GetTickCount();
	sAction.hItem	= hRoot;
	sAction.hWait	= CreateSemaphore(NULL,0,16,0);
	sAction.iType	= TYPE_SCAN_NETWORK;
	sAction.bFlag	= 0;

	LOCK();
	m_sActionList.Add(0,0,0,&sAction);
	pAction = (ActionItem*)m_sActionList.Data(0);
	m_sNextAction.Unlock();
	UNLOCK();

	if(WaitForSingleObject(sAction.hWait,uTimeout+111111)!=0)
		{
		LOCK();

		if(m_sActionList.Data(0)==pAction)
			{
			pAction->hWait = 0;
			}

		UNLOCK();
		}

	CloseHandle(sAction.hWait);

		hItem = FindItemEx(hRoot,TVIF_CHILDS|TVIF_STATE,0,NULL,0,FLAG_NET,FLAG_NET|FLAG_DONE);
	if(!hItem)return FALSE;

	bOk = TRUE;

	while(hItem)												// Ebene einlesen
		{
		if(uTimeout!=INFINITE)
			{
			   uDiff=GetTickCount()-uStart;
			if(uDiff>uTimeout)
				{
				uWait = (uTimeout>500)? 50:10;
				}
			else{
				uWait = uTimeout-uDiff;
				}
			}

		if(!ScanNetwork(hItem))bOk=FALSE;

		hItem = FindItemEx(hItem,TVIF_NEXT|TVIF_STATE,0,NULL,0,FLAG_NET,FLAG_NET|FLAG_DONE);
		}


return bOk;
}

//*****************************************************************************
//*
//*		FindPath
//*
//*****************************************************************************
//	Sucht einen Eintrag im Fenster mit einem Pfad
//	pPath		: Ist der Pfad der gesucht wird
//	pLast		: Wenn ungleich 0 dann wird hier gespeichert wo
//				  das letzte Untersegment gefunden wurde, und zurck gegeben
//	iMax		: Ist die maximale Lnge des Pfades
//	hItem		: Ist der Starteintrag
//	iOff		: Ist der Start-Offsetim Pfad
//	Ergibt das Handle des Eintrages oder NULL wenn der Pfad nicht gefunden wurde.
HTREEITEM CFolderTreeCtrl::FindPath(LPCTSTR pPath,int *pLast,int iMax,HTREEITEM hItem,int iOff)
{
TCHAR			cBuffer[MAX_PATH];
HTREEITEM		hNext;
LPCTSTR			pName;
int				iLen;


																// Ist es eine Netzwerkspfad
	if(hItem==TVI_ROOT && (pPath[0]=='\\' || pPath[0]=='/') && (pPath[1]=='\\' || pPath[1]=='/'))
	do	{
		iOff += 2;
		iLen  = strcwlen(pPath+iOff,0x00300000,iMax-iOff);
		strlcpy(cBuffer ,pPath+iOff,MAX_PATH  ,iLen);

		   hNext = FindItemEx(m_hNetworkItem,TVIF_TEXT|TVIF_CHILDS|TVIF_CASE,0,cBuffer,0,0,0);
		if(hNext)
			{
			hItem = hNext;
			iOff += iLen;

			if(iOff<iMax && pPath[iOff])
				{
				iOff++;
				}

			break;
			}
																// Zweite Suche in Root
		if(m_hNetworkItem!=TVI_ROOT)
			{
			   pName = GetItemText(m_hNetworkItem);
			if(pName && !_tcsicmp(cBuffer,pName))
				{
				hItem = hNext;
				iOff += iLen;

				if(iOff<iMax && pPath[iOff])
					{
					iOff++;
					}

				break;
				}
			}

		hItem = m_hNetworkItem;
		}while(0);



	if(iOff<iMax && iOff>=2)									// Backslash berspringen
	if(pPath[iOff]=='\\' || pPath[iOff]=='/')
		{
		iOff++;
		}


	while(iOff<iMax && pPath[iOff])								// Suche Unterordner
		{
		iLen = strcwlen(pPath+iOff,0x00300000,iMax-iOff);
		strlcpy(cBuffer,pPath+iOff,MAX_PATH  ,iLen);

			hNext = FindItemEx(hItem,TVIF_TEXT|TVIF_CHILDS|TVIF_CASE,0,cBuffer,0,0,0);
		if(!hNext)
			{
			if(!pLast)hItem=0;
			break;
			}

		hItem = hNext;
		iOff += iLen;

		if(iOff<iMax && pPath[iOff])
			{
			iOff++;
			}
		}


	if(pLast)*pLast=iOff;


return hItem;
}

//*****************************************************************************
//*
//*		GetIcon
//*
//*****************************************************************************
//	Die Icon-Nommer eines Pfades abfragen
//	pPath		: Ist der Pfad
//	iOpenIcon	: Das Open-Icon holen
//	Ergibt die Icon-Nummer oder -1 bei einem Fehler
int	CFolderTreeCtrl::GetIcon(LPCTSTR pPath,int iOpenIcon)
{
SHFILEINFO		sShInfo;
unsigned		uFlags;


	uFlags = (iOpenIcon)? SHGFI_OPENICON|SHGFI_SYSICONINDEX|SHGFI_USEFILEATTRIBUTES:SHGFI_SYSICONINDEX|SHGFI_USEFILEATTRIBUTES;

	if(SHGetFileInfo(pPath,FILE_ATTRIBUTE_DIRECTORY,&sShInfo,sizeof(sShInfo),uFlags))
		{
		return sShInfo.iIcon;
		}


return -1;
}

//*****************************************************************************
//*
//*		ExpandChain
//*
//*****************************************************************************
//	Expantiert alle Eintrge einer Kette
//	hItem		: Erste Eintrag in der Kette
//	iMode		: Bit0=immer ganz aufklappen Bit1=bei Elternelement beginnenen
void CFolderTreeCtrl::ExpandChain(HTREEITEM hItem,int iMode)
{
unsigned	uState;



	if(iMode&2)
		{
		hItem  = GetParentItemEx(hItem);
		iMode &= ~2;
		}

	while(hItem)
		{
		uState = GetItemStateEx(hItem,0xFFFFFFFF);

		if((iMode || (uState&FLAG_NET)) && (uState&FLAG_DONE))
			{
			ExpandEx(hItem,TVE_EXPAND|TVE_NONOTIFY|TVE_EXPANDFORCE);
			}
		else{
			ExpandEx(hItem,TVE_EXPAND|TVE_NONOTIFY|TVE_EXPANDFORCE|TVE_EXPANDPARTIAL);
			}

		hItem  = GetParentItemEx(hItem);
		}

}

//*****************************************************************************
//*
//*		AddFolder
//*
//*****************************************************************************
//	Fgt einen Folder rekursiv ein
//	hRoot		: Ist der Eintrag ab dem eingefgt wird.
//	pPath		: Ist der Einfgepfad
//	iOff		: Ist der Start-Offset im Pfad
//	Ergibt das Handle des eingefgten Eintrages, oder NULL bei einem Fehler.
HTREEITEM CFolderTreeCtrl::AddFolder(HTREEITEM hRoot,LPTSTR pPath,int iOff)
{
TVINSERTSTRUCT	sInsert;
HTREEITEM		hFocus;
DWORD			uLen;
DWORD			uEnd;
TCHAR			cOld;
INT				iVal;


	sInsert.hInsertAfter 	= TVI_SORTEX;
	sInsert.item.hItem		= (HTREEITEM)SortProc;
	sInsert.item.mask		= TVIF_STATE|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_CHILDREN;
	sInsert.item.state		= IsFixedDrive(pPath)? FLAG_PARTIAL|TVIS_EXPANDPARTIAL|TVIS_EXPANDED:FLAG_PARTIAL|FLAG_NOSCAN|TVIS_EXPANDPARTIAL|TVIS_EXPANDED;
	sInsert.item.stateMask	= 0xFFFF0000|TVIS_EXPANDPARTIAL|TVIS_EXPANDED;
	sInsert.item.cchTextMax	= 256;
	sInsert.item.cChildren	= 1;


	while(hRoot)											// Pfad rekursiv durchlaufen
		{
		if(pPath[iOff]=='\\')iOff++;
		if(pPath[iOff]== 0  )break;

		uLen = strcwlen(pPath+iOff,0x00300000);
		uEnd = iOff+uLen;
		cOld = pPath[uEnd];

		pPath[iOff+uLen] = 0;

		if(!cOld || !pPath[uEnd+1])							// Ende des Pfades erreicht
			{
			sInsert.item.stateMask &= ~(TVIS_EXPANDPARTIAL|TVIS_EXPANDED);
			}

		sInsert.hParent				= hRoot;
		sInsert.item.iImage		 	= GetIcon(pPath);
		sInsert.item.iSelectedImage	= sInsert.item.iImage;
		sInsert.item.pszText	 	= pPath+iOff;

		   hFocus = GetFocusItemEx();
		if(hFocus)
			{
			   iVal = IsItemVisibleEx(hFocus);
			if(iVal<1)hFocus = NULL;
			}

		hRoot = InsertItemEx(&sInsert);						// Eintrag einfgen

		if(hFocus && IsItemVisibleEx(hFocus)<4)
			{			
			PostMessage(TVM_ENSUREVISIBLE,(iVal==4)? 0:0xFFFFFFFF,(LPARAM)hFocus);
			}

		pPath[uEnd]	= cOld;
		iOff		= uEnd;
		}


return hRoot;
}

//*****************************************************************************
//*
//*		AddDrive
//*
//*****************************************************************************
//	Ein neues Laufwerk einfgen
//	cLetter		: Ist die Laufwerksnummer ('A'...'Z')
//	Ergibt das Handle des neuen EIntrags, oder NULL bei einem Fehler.
HTREEITEM CFolderTreeCtrl::AddDrive(TCHAR cLetter)
{
TCHAR			cName[4];
TVINSERTSTRUCT	sInsert;
HTREEITEM		hItem;
DWORD			uBits;
DWORD			uMask;



	cName[0] = cLetter&~0x20;

	if(cName[0]<'A' || cName[0]>'Z')						// Laufwerksbuchstaben prfen
		{
		return NULL;
		}

	uBits	= GetLogicalDrives();
	uMask	= 1<<(cName[0]-'A');

	if(!(uBits&uMask))										// Prfe ob es das Laufwerk gibt
		{
		return FALSE;
		}

	
	cName[1] = ':';
	cName[2] = 0;

	LOCK();
	
	   hItem = FindItemEx(TVI_ROOT,TVIF_CHILDS|TVIF_TEXT,0,cName);
	if(hItem)												// Existiert das Laufwerk bereits
		{
		UNLOCK();

		return hItem;
		}
	

	sInsert.hParent				= TVI_ROOT;
	sInsert.hInsertAfter 		= TVI_SORTEX;
	sInsert.item.hItem			= (HTREEITEM)SortDrives;
	sInsert.item.mask			= TVIF_STATE|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_CHILDREN;
	sInsert.item.state		 	= IsFixedDrive(cName)? FLAG_DRIVE|TVIS_EXPANDPARTIAL|TVIS_EXPANDED:FLAG_DRIVE|FLAG_NOSCAN|TVIS_EXPANDPARTIAL|TVIS_EXPANDED;
	sInsert.item.stateMask		= 0xFFFF0000|TVIS_EXPANDPARTIAL|TVIS_EXPANDED;
	sInsert.item.iImage		 	= GetIcon(cName);
	sInsert.item.iSelectedImage	= sInsert.item.iImage;
	sInsert.item.pszText	 	= cName;
	sInsert.item.cchTextMax		= 256;
	sInsert.item.cChildren		= 1;

	hItem = InsertItemEx(&sInsert);


	UNLOCK();
	


return hItem;
}

//*****************************************************************************
//*
//*		AddNetwork
//*
//*****************************************************************************
//	Ein neues Laufwerk einfgen
//	hRoot		: Elterneintrag frs einfgen
//	pPath		: Gesamter reale Netzwerkspfad
//	iOff		: Offset des Unternamens zum Einfgen
//	iNet		: Ist die Nummer des Netzpfades (-1=keiner -2=einfgen -3=realer Pfad)
//	Ergibt das Handle des neuen EIntrags, oder NULL bei einem Fehler.
HTREEITEM CFolderTreeCtrl::AddNetwork(HTREEITEM hRoot,LPTSTR pPath,int iOff,int iNet)
{
TVINSERTSTRUCT	sInsert;
int				iLen;


	sInsert.hParent				= hRoot;
	sInsert.hInsertAfter 		= TVI_LAST;
	sInsert.item.mask			= TVIF_STATE|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_CHILDREN;
	sInsert.item.state		 	= FLAG_NET|FLAG_NOSCAN|TVIS_EXPANDPARTIAL|TVIS_EXPANDED;
	sInsert.item.stateMask		= 0xFFFF0000;
	sInsert.item.iImage		 	= GetIcon(pPath);
	sInsert.item.iSelectedImage	= sInsert.item.iImage;
	sInsert.item.pszText	 	= pPath+iOff;
	sInsert.item.cchTextMax		= 256;
	sInsert.item.cChildren		= 1;

	if(iNet<=-3)
		{
		sInsert.item.state	   &= ~FLAG_NET;
		}

	if(iNet==-2)
		{
		iLen = (int)_tcslen(pPath+iOff)+iOff;

		LOCK();    iNet=m_sNetPaths.FindFast(iLen,pPath,0,-1,1);
		if(iNet<=0)iNet=m_sNetPaths.Add(iLen,pPath,CSC_ABCG);
		pPath	       =m_sNetPaths.GetNone(iNet);
		UNLOCK();
		}

	if(iNet>=0)
		{
		sInsert.item.state	   &= ~FLAG_NET;
		sInsert.item.mask	   |= TVIF_PARAM;
		sInsert.item.lParam		= (LPARAM)m_sNetPaths[iNet];
		}


return InsertItemEx(&sInsert);
}

//*****************************************************************************
//*
//*		SelectEntry
//*
//*****************************************************************************
//	Whlt einen Eintrag aus und setzt den aktuellen Pfad
//	hItem		: Ist der Eintrag der gewhlt werden soll
//	pPath		: Ist der Pfad des Eintrages
//	iLen		: Ist die Pfadlnge (-1=holen)
//	iScan		: Soll der neue Pfad eingelesen werden :
//						0     = nein
//						1     = wenn nicht aktuell
//						2	  = immer
//					   Bit7   = Auswahl muss sichtbar sein
//	Ergibt 0 bei einem Fehler 1 wenn gewhlt bzw. bei neu gewhlten Eintrag.
UINT CFolderTreeCtrl::SelectEntry(HTREEITEM hItem,LPCTSTR pPath,int iLen,int iScan)
{
UINT	uVisible;
UINT	uState;



	if(!hItem)return 0;

	if(iLen<=0)iLen=(int)_tcslen(pPath);

	uState		= GetItemStateEx(hItem,0xFFFF0000);
	uVisible	= iScan&0x80;
	iScan	   &= 0x7F;

	LOCK();

	memcpy(m_cCurrentPath,pPath,iLen*sizeof(TCHAR));		// Neuen aktuellen Pfad speichern
	m_cCurrentPath[iLen] = 0;

	if(uState&FLAG_DRIVE)
		{
		m_iCurrentMode = 2;
		}
	else if(uState&FLAG_NET)
		{
		m_iCurrentMode = 0;
		}
	else{
		m_iCurrentMode = 1;
		}

	UNLOCK();


	if(hItem==m_hCurrentItem)
		{
		if(hItem!=GetFocusItem())
			{
			SetFocusItem(hItem);
			}

		if(uVisible)
			{
			PostMessage(TVM_ENSUREVISIBLE,0,(LPARAM)hItem);
			}

		return 1;
		}


	m_hCurrentItem = hItem;

	PostMessage(WM_SELECTENTRY,iScan,(LPARAM)hItem);

	if(uVisible)
		{
		PostMessage(TVM_ENSUREVISIBLE,0,(LPARAM)hItem);
		}



return 2;
}


//*****************************************************************************
//*
//*		OnSelectEntry
//*
//*****************************************************************************
LRESULT CFolderTreeCtrl::OnSelectEntry(WPARAM wParam,LPARAM lParam)
{
HTREEITEM	hItem=(HTREEITEM)lParam;
CFT_FOLDER	sNotify;
ActionItem	sAction;
UINT		uState;
int			iTemp;
int			iIcon;



	if(!SelectItem(hItem))
		{
		return 0;
		}

	if(m_hOpenTreeItem)										// Alten offenen Ordner
		{
		SetItemImage(m_hOpenTreeItem,m_iLastOpenIcon,m_iLastOpenIcon);
		m_hOpenTreeItem=0;
		}

	uState = GetItemState(hItem,0xFFFFFFFF);

	if(!(uState&(FLAG_NET|FLAG_DRIVE)))						// Open-Icon setzen
		{
		if(GetItemImage(hItem,iIcon,iTemp))
			{
			m_hOpenTreeItem = hItem;
			m_iLastOpenIcon = iIcon;
			iIcon = GetIcon(m_cCurrentPath,1);
			SetItemImage(hItem,iIcon,iIcon);
			}
		}


	LOCK();

	sNotify.sNmHdr.code		= CFT_FOLDERSELECT;
	sNotify.sNmHdr.hwndFrom	= m_hWnd;
	sNotify.sNmHdr.idFrom	= GetWindowLong(m_hWnd,GWL_ID);
	sNotify.iMode			= m_iCurrentMode;
	sNotify.pPath			= m_cCurrentPath;
	sNotify.hItem			= hItem;
															// Eltern-Fenster benachrichtigen
	if(GetParent()->SendMessage(WM_NOTIFY,sNotify.sNmHdr.idFrom,(LPARAM)&sNotify))
		{
		wParam = 0;
		}

	UNLOCK();

	if(wParam==1 && !(uState&FLAG_DONE))					// Einlesen wenn nicht aktuell
		{
		wParam = 2;
		}


	EnsureVisible(hItem);

	if(wParam==0)return 0;


	LOCK();

	m_iCurrentNumber++;

	if(m_sActionList.Count())								// Die aktuelle Suche abbrechen
		{
		if(m_uStopTicker==0)
			{
			SetTimer(ID_STOPTICKER,200,NULL);
			}

		m_uStopTicker = 5;
		}

	if(wParam==2)											// Einlesen starten
		{
		sAction.iType		= m_iCurrentNumber|TYPE_SCAN_SELECT;
		sAction.hItem		= hItem;
		sAction.hWait		= 0;
		sAction.bFlag		= 0;
		m_sActionList.Add(0,0,0,&sAction);
		m_sNextAction.Unlock();
		}

	for(;;)
		{													// Einlesen der Eltern
			hItem = GetParentItem(hItem);
		if(!hItem)break;

		   uState = GetItemStateEx(hItem,0xFFFFFFFF);
		if(uState&FLAG_NET)break;

		if(!(uState&FLAG_NOSCAN))
			{
			sAction.iType		= m_iCurrentNumber|TYPE_SCAN_BASE;
			sAction.hItem		= hItem;
			sAction.hWait		= 0;
			sAction.bFlag		= 0;
			m_sActionList.Add(0,0,1,&sAction);
			m_sNextAction.Unlock();
			}

		if(GetItemDataEx(hItem))break;
		}

	UNLOCK();


return 0;
}

//*****************************************************************************
//*
//*		SortProc
//*
//*****************************************************************************
int CALLBACK CFolderTreeCtrl::SortProc(HWND hWnd,HTREEITEM hItem,LPCTSTR pTextItem,LPCTSTR pTextInsert,LPARAM lParamItem,LPARAM lParamInsert)
{
return strvicmp(pTextInsert,pTextItem);
}

//*****************************************************************************
//*
//*		SortProcEx
//*
//*****************************************************************************
int CALLBACK CFolderTreeCtrl::SortProcEx(HWND hWnd,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParam)
{
TV_ITEM		sItem1;
TV_ITEM		sItem2;


	sItem1.mask		= TVIF_TEXT|TVIF_TEXTPTR;
	sItem1.hItem	= hItem1;

	TreeList_GetItem(hWnd,&sItem1);

	sItem2.mask		= TVIF_TEXT|TVIF_TEXTPTR;
	sItem2.hItem	= hItem2;

	TreeList_GetItem(hWnd,&sItem2);


return strvicmp(sItem1.pszText,sItem2.pszText);
}


//*****************************************************************************
//*
//*		SynchPath
//*
//*****************************************************************************
//	Synchronisiert einen Ordner mit einer gefunden Dateiliste
//	pPath		: Ist der Pfad der gesucht wird
//	bVisible	: Prfen ob das Fokus-Item sichtbar ist
//	Ergibt TRUE wenn alles aktualisiert wurde
BOOL CFolderTreeCtrl::SynchPath(HTREEITEM hRoot,CFileFinder *pFind,BOOL bVisible)
{
TVINSERTSTRUCT	sInsert;
HTREEITEM		hFocus;
HTREEITEM		hItem;
HTREEITEM		hNext;
LPTSTR			pName;
LPTSTR			pPath;
int				iState;
int				iFlags;
int				iItems;
int				iCount;
int				iIcon;
int				iNum;
int				iLen;
int				iVal;



	pFind->Reset();
	iCount	= 0;
	iItems	= 0;
	iLen	= 0;

	hFocus					= NULL;
	pName					= pFind->Next();
	iFlags					= pFind->Attribute(0);
	iState					= GetItemStateEx(hRoot,0xFFFF0000);
	sInsert.hParent			= hRoot;
	sInsert.hInsertAfter 	= (iFlags&SYS_NET)? TVI_LAST:TVI_SORTEX;
	sInsert.item.hItem		= (HTREEITEM)SortProc;
	sInsert.item.mask		= TVIF_STATE|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE|TVIF_CHILDREN|TVIF_PARAM;
	sInsert.item.state		= (iState&FLAG_NOSCAN)|FLAG_SET;
	sInsert.item.stateMask	= 0xFFFF0000;
	sInsert.item.cchTextMax	= 256;
	sInsert.item.cChildren	= 1;
	sInsert.item.lParam		= 0;

															// Merker lschen
	SetMultiItemsEx(hRoot,0,TVIF_CHILDS|TVIF_STATE,0,0,0,0,FLAG_SET);


	for(;pName;pName=pFind->Next())							// Alle Dateien der Suche durchlaufen
		{
		if(pFind->IsDir()==0)
			{
			continue;
			}

		if(pName[0]=='.' && pName[1]=='.' && pName[2]==0)
			{
			continue;
			}

		iItems++;
															// Ist der Eintrag bereits vorhanden
		   hItem = FindItemEx(hRoot,TVIF_TEXT|TVIF_CHILDS|TVIF_CASE,0,pName,0,0,0);
		if(hItem)
			{
			SetItemStateEx(hItem,FLAG_SET,FLAG_SET);
			continue;
			}

		iIcon = pFind->GetIcon();
		iNum  = -1;

 		if(m_sFolders.IsNet() && m_sFolders.IsArchive())	// Netzwerks-Link gefunden
			{
			pPath	= pFind->Link(-1,&iLen);

			LOCK();    iNum=m_sNetPaths.FindFast(iLen,pPath,0,-1,1);
			if(iNum<=0)iNum=m_sNetPaths.Add(iLen,pPath,CSC_ABCG);
			pPath	       =m_sNetPaths.GetNone(iNum);
			UNLOCK();

			sInsert.item.state |=  FLAG_NOSCAN|FLAG_LINK;
			sInsert.item.state &= ~FLAG_NET;
			sInsert.item.lParam	= (LPARAM)pPath;

			if(IsFixedDrive(pPath))
				{
				sInsert.item.state &= ~FLAG_NOSCAN;
				}
			}
 		else if(m_sFolders.IsNet())							// Virtuelles Netzwerk
			{
			sInsert.item.state |=  FLAG_NET|FLAG_NOSCAN;
			sInsert.item.state &= ~FLAG_LINK;
			sInsert.item.lParam	= 0;
			}
		else if(iState&FLAG_NET)							// Erster realer Pfad in Ebenen
			{
			pPath	= pFind->Name(-1,1,&iLen);

			LOCK();    iNum=m_sNetPaths.FindFast(iLen,pPath,0,-1,1);
			if(iNum<=0)iNum=m_sNetPaths.Add(iLen,pPath,CSC_ABCG);
			pPath	       =m_sNetPaths.GetNone(iNum);
			UNLOCK();

			sInsert.item.lParam	=  (LPARAM)pPath;
			sInsert.item.state |=   FLAG_NOSCAN;
			sInsert.item.state &= ~(FLAG_NET|FLAG_LINK);

			if(IsFixedDrive(pPath))
				{
				sInsert.item.state &= ~FLAG_NOSCAN;
				}
			}
		else{												// Normales Verzeichnis
			sInsert.item.state &= ~(FLAG_NET|FLAG_LINK);
			sInsert.item.lParam	= 0;
			}

		sInsert.item.pszText	 	=  pName;
		sInsert.item.iImage		 	=  iIcon;
		sInsert.item.iSelectedImage	=  iIcon;

		   hFocus = GetFocusItemEx();
		if(hFocus)
			{
			   iVal = IsItemVisibleEx(hFocus);
			if(iVal<1)hFocus = NULL;
			}

		hItem = InsertItemEx(&sInsert);						// Neuer Eintrag

		if(bVisible && hFocus && IsItemVisibleEx(hFocus)<4)
			{			
			PostMessage(TVM_ENSUREVISIBLE,(iVal==4)? 0:0xFFFFFFFF,(LPARAM)hFocus);
			}

		if(iNum>=0)
			{
			m_sNetPaths.DataSetPtr(iNum,hItem);
			}

		iCount++;
		}


	if(pFind->DoneFlag()>=1)								// Wurde alles eingelesen
		{
		return FALSE;
		}

	if(hRoot!=TVI_ROOT)
		{
		hItem = FindItemEx(hRoot,TVIF_CHILDS|TVIF_STATE,0,0,0,0,FLAG_SET);

		while(hItem)										// Lsche Eintrge die nicht aktualisiert wurden
			{
			hNext = FindItemEx(hItem,TVIF_NEXT|TVIF_STATE,0,0,0,0,FLAG_SET);

			   hFocus = GetFocusItemEx();
			if(hFocus && IsItemVisibleEx(hFocus)<1)
				{
				hFocus = NULL;
				}

			DeleteItemEx(hItem);

			if(bVisible && hFocus && IsItemVisibleEx(hFocus)<4)
				{			
				PostMessage(TVM_ENSUREVISIBLE,(iVal==4)? 0:0xFFFFFFFF,(LPARAM)hFocus);
				}

			hItem = hNext;
			}
		}


	sInsert.item.hItem		= hRoot;
	sInsert.item.mask		= TVIF_STATE;
	sInsert.item.state		= FLAG_DONE;
	sInsert.item.stateMask	= FLAG_DONE;
	sInsert.item.cChildren	= 0;

	if(iItems==0)
		{
		sInsert.item.mask  |= TVIF_CHILDREN;
		}


	SetItemEx(&sInsert.item);								// ROOT-Eintrag aktualisieren


	if(iCount>0 && m_sActionList.Count()==0)
		{
		PostMessage(WM_COMMAND,ID_STARTUPDATE);
		}


return TRUE;
}

//*****************************************************************************
//*
//*		GetPath
//*
//*****************************************************************************
//	Holt den aktuellen Pfad.
//	pBuffer		: Ist der Puffer in dem der Pfad gespeichert wird
//	iSize		: Ist die Gre des Puffers
//	pItem		: Hier wird der aktuelle Eintrag gespeichert
//	Ergibt die gespeicherte Lnge.
int	CFolderTreeCtrl::GetPath(LPTSTR pBuffer,int iSize,HTREEITEM *pItem)
{
int		iLen;

	LOCK();
	if(pItem)*pItem=m_hCurrentItem;
	iLen = strlcpy(pBuffer,m_cCurrentPath,iSize);
	UNLOCK();

return iLen;
}

//*****************************************************************************
//*
//*		IsRealPath
//*
//*****************************************************************************
int CFolderTreeCtrl::IsRealPath(HTREEITEM hItem)
{
UINT	uState;


	uState = GetItemStateEx(hItem,0xFFFF0000);


return (uState&FLAG_NET)? 0:1;
}

//*****************************************************************************
//*
//*		GetPath
//*
//*****************************************************************************
//	Holt den Pfad von einem Eintrag
//	hItem		: Handle vom Eintrag
//	pBuffer		: Zeiger auf den Textpuffer in dem der Pfad gespeichert wird
//	iSize		: Ist die Gre des Puffers
//	iMode		: Welcher Pfad soll geholt werden
//					0 = realer Pfad
//					1 = gesamter Pfad fr Suche
//					2 = gesamter Pfad
//	pState		: Hier wird der Status des ersten Eintrages gespeichert
//	Ergibt die Lnge desgeholten Pfades bzw. 0 bei einem Fehler.
int CFolderTreeCtrl::GetPath(HTREEITEM hItem,LPTSTR pBuffer,int iSize,int iMode,int *pState)
{
LPCTSTR		aText [32];
UINT		aSize [32];
UINT		aState[32];
LPARAM		aParam[32];
LPCTSTR		pPath;
HTREEITEM	hNext;
TV_ITEM		sItem;
int			iNum;
int			iLen;


	if(iSize<0)return 0;

	sItem.mask		= TVIF_STATE|TVIF_PARAM|TVIF_TEXT|TVIF_TEXTPTR;
	sItem.stateMask	= 0xFFFF0000;
	sItem.hItem		= hItem;

	aState[0]		= 0;
	iNum			= 0;

	while(sItem.hItem)										// Eintrge rekursif durchlaufen
		{
		if(!GetItemEx(&sItem))break;

		aText [iNum] = sItem.pszText;
		aSize [iNum] = sItem.cchTextMax-1;
		aState[iNum] = sItem.state;
		aParam[iNum] = sItem.lParam;
		hNext        = GetParentItemEx(sItem.hItem);

		if(pState)
			{
			pState[0]= sItem.state;
			pState	 = 0;
			}

		if(iMode==0 && sItem.lParam)						// Pfadende erreicht
			{
			pPath		= (LPCTSTR)sItem.lParam;
			aSize[iNum] = (UINT   )_tcslen (pPath);
			aText[iNum] = (LPCTSTR)pPath;
			iNum++;
			break;
			}
		else if(iMode==1 && sItem.hItem==m_hNetworkItem)
			{
			aSize[iNum] = 2;
			aText[iNum] = _T("\\\\");
			}

		sItem.hItem = hNext;
		iNum++;
		}

	iSize -= 1;
	iLen   = 0;




	while(iNum>0)											// Pfad zusammen bauen
		{
		iNum--;

		if(iLen && pBuffer[iLen-1]!='\\')
			{
			pBuffer[iLen] = '\\';
			iLen++;
			}

		if(iLen+(int)aSize[iNum]>iSize)						// Gre prfen
			{
			iLen = 0;
			break;
			}

		memcpy(pBuffer+iLen,aText[iNum],aSize[iNum]*sizeof(TCHAR));
		iLen += aSize[iNum];
		}


	if(pState)
		{
		pState[0] = 0;
		}


	pBuffer[iLen] = 0;


return iLen;
}

//*****************************************************************************
//*
//*		SetPath
//*
//*****************************************************************************
//	Setzt den aktuellen Pfad fr das Fenster.
//	pPath		: Ist der Pfad der eingestellt werden soll.
//				  Bei 0 wird der aktuelle Pfad genommen.
//	pSubPath	: Ist der Pfad der an pPath angehngt wird.
//				  Bei 0 so wird nichts angehngt.
//	iScan		: Soll der neue Pfad eingelesen werden :
//						0     = nein
//						1     = wenn nicht aktuell
//						2	  = immer
//					   Bit7   = Auswahl muss sichtbar sein
// Ergibt TRUE wenn der Pfad bernommen wurde, bzw. FALSE bei einem Fehler
// (z.B. wenn der Pfad nicht gefunden wurde)
BOOL CFolderTreeCtrl::SetPath(LPCTSTR pPath,LPCTSTR pSubPath,int iScan)
{
TCHAR			cBuffer[MAX_PATH+16];
TCHAR			cName  [MAX_PATH+16];
LPCTSTR			pName;
HTREEITEM		hItem;
HTREEITEM		hRoot;
HTREEITEM		hNext;
DWORD			uAttr;
TCHAR			cOld;
int				iLen;
int				iEnd;
int				iPos;
int				iNum;
int				iIdx;



	if(!pPath)												// Alten Pfad nehmen ?
		{
		LOCK();
		pPath    = (m_cCurrentPath[0])? m_cCurrentPath:_T("C:\\");
		strlcpy(cName,m_cCurrentPath,sizeof(m_cCurrentPath));
		pPath	 = cName;
		UNLOCK();
		}

	if(pSubPath)											// Untersverzeichnis anhngen
		{
		iLen	= strlcpy(cName     ,pPath   ,MAX_PATH);
		iLen	= AddBackSlash(cName,iLen);
				  strlcpy(cName+iLen,pSubPath,MAX_PATH-iLen);
		pPath	= cName;
		}


	iLen	= GetFullPathName(pPath,sizeof(cBuffer),cBuffer,0);
	pPath	= cBuffer;


	if(iLen>2 && (cBuffer[iLen-1]=='\\'))					// Backslash entfernen
		{
		iLen--;
		cBuffer[iLen]=0;
		}

															// Langen Dateinamen erzeugen ("C:\TEST~3" --> "D:\Test Daten ..:")
	if(iLen>0 && cBuffer[1]==':' && strchar(cBuffer,'~')<iLen)
		{
		iLen = MakeLongPath(cName,cBuffer,MAX_PATH);
		memcpy(cBuffer,cName,(iLen+1)*sizeof(TCHAR));
		}


	   hItem = FindPath(cBuffer,&iEnd);
	if(hItem && iEnd>=iLen)									// Existiert der Pfad bereis
		{
		SelectEntry(hItem,cBuffer,iLen,iScan);
		return TRUE;
		}


//******************** Lokaler Laufwerkspfad **********************************


	if(iLen>0 && cBuffer[1]==':')							// Ist es ein Laufwerk
		{
		   uAttr = GetFileAttributes(cBuffer);				// Realer Pfad ?
		if(uAttr==0xFFFFFFFF || !(uAttr&FILE_ATTRIBUTE_DIRECTORY))
			{
			return FALSE;
			}

		if(iEnd<2)											// Existiert das Laufwerk noch nicht ?
			{
				hRoot = AddDrive(cBuffer[0]);
			if(!hRoot)
				{
				return FALSE;
				}

			hItem = hRoot;
			iEnd  = 2;
			}


		hItem = AddFolder(hItem,cBuffer,iEnd);				// Den Pfad einfgen

		return SelectEntry(hItem,cBuffer,iLen,iScan);
		}



//******************** Netzwerkspfad ******************************************


	if(pPath[0]!='\\' || pPath[1]!='\\')					// Netzpfad korrigieren
		{
		return FALSE;
		}


	if(!m_iNetFlags)										// Sind Netzlaufwerke erlaubt
		{
		return FALSE;
		}


	if(!m_bNetScanned)										// Sind die Netzlaufwerke eingelesen ?
		{
		m_bNetScanned = TRUE;
		PostMessage(ID_SCANNETWORK);
		}

	   iPos = m_sNetPaths.FindFast(iLen,cBuffer,0,-1,0x0D);
	if(iPos>0)												// Ist ein realer Netzwekspfad vorhanden
		{
		pName	= m_sNetPaths.Get(iPos-1,iEnd);

		if(!_tcsnicmp(pName,cBuffer,iEnd))
			{
			   uAttr = GetFileAttributes(cBuffer);			// Realer Pfad ?
			if(uAttr==0xFFFFFFFF || !(uAttr&FILE_ATTRIBUTE_DIRECTORY))
				{
				return FALSE;
				}

			hItem	= (HTREEITEM)m_sNetPaths.DataGetPtr(iPos-1);
			hRoot	= FindPath(cBuffer,&iEnd,iLen,hItem,iEnd);

			ExpandChain(hRoot,2);

			hItem	= AddFolder(hRoot,cBuffer,iEnd);		// Den Pfad einfgen

			return SelectEntry(hItem,cBuffer,iLen,iScan);
			}
		}


//******************** Unbekannter Netzwerkspfad ******************************


	   uAttr = GetFileAttributes(cBuffer);					// Realer Pfad ?
	if(uAttr==0xFFFFFFFF || !(uAttr&FILE_ATTRIBUTE_DIRECTORY))
		{
		return FALSE;
		}

	iPos	= 2;
	hItem	= m_hNetworkItem;


	for(iIdx=-1;;iIdx--)									// Netzwerks Pfad einfgen
		{
		   iNum = strcwlen(cBuffer+iPos,0x00300000);
		if(iNum==0)
			{
			if(iIdx>=-2)return FALSE;
			break;
			}

		iEnd		  = iPos+iNum;
		cOld		  = cBuffer[iEnd];

		if(cOld==0 && iIdx==-1)
			{
			return FALSE;
			}

		cBuffer[iEnd] = 0;
		hNext		  = FindItemEx(hItem,cBuffer+iPos);
		hItem		  = (hNext)? hNext:AddNetwork(hItem,cBuffer,iPos,iIdx);

		if(hItem==0)return FALSE;
		if(cOld ==0)break;

		cBuffer[iEnd] = cOld;
		iPos		  = iEnd+1;
		}


	ExpandChain(hItem,2);



return SelectEntry(hItem,cBuffer,iEnd,iScan);
}


//*****************************************************************************
//*
//*		ScanUpdate
//*
//*****************************************************************************
//	Sucht nach den nchsten Aktionen
void CFolderTreeCtrl::ScanUpdate()
{
ActionItem		sAction;
unsigned		uState;
HTREEITEM		hItem;
HTREEITEM		hRoot;
int				iPos;



	if(m_sActionList.Count()>MAX_ACTIONS)return;


	sAction.hWait	= 0;
	hRoot  = m_hCurrentItem;

	while(hRoot)											// Haben Unterordner von aktuellen Eintrag weitere Eintrge
		{
		   uState=GetItemStateEx(hRoot,0xFFFFFFFF);
		if(uState&FLAG_NET)break;

		if((uState&FLAG_DONE) && !(uState&FLAG_NOSCAN) && (uState&TVIS_EXPANDED))
			{
			hItem = FindItemEx(hRoot,TVIF_CHILDS|TVIF_STATE,0,0,0,0,FLAG_DONE|FLAG_PARTIAL|FLAG_NET|FLAG_NOSCAN|TVIS_EXPANDED);
			while(hItem)
				{
				LOCK();

				   iPos  = m_sActionList.Find((int)hItem);
				if(iPos<0)
					{
					sAction.bFlag	= 0;
					sAction.hItem	= hItem;
					sAction.iType	= m_iCurrentNumber|TYPE_SCAN_ONLY;
					m_sActionList.Add(0,0,CSC_END,&sAction);
					m_sNextAction.Unlock();
					}

				UNLOCK();

				if(m_sActionList.Count()>MAX_ACTIONS)return;

				hItem = FindItemEx(hItem,TVIF_STATE,0,0,0,0,FLAG_DONE|FLAG_PARTIAL|FLAG_NET|FLAG_NOSCAN|TVIS_EXPANDED);
				}
			}

		hRoot = GetParentItemEx(hRoot);
		}


	hItem = FindItemEx(0,TVIF_RECURSIVE|TVIF_STATE|TVIF_VISIBLE,0,0,0,TVIS_EXPANDED,FLAG_DONE|FLAG_PARTIAL|FLAG_NET|FLAG_NOSCAN|TVIS_EXPANDED);

	while(hItem)											// Suche Eintrge die noch nicht eingelesen sind
		{
		LOCK();

		   iPos  = m_sActionList.Find((int)hItem);
		if(iPos<0)
			{
			sAction.bFlag	= 0;
			sAction.hItem	= hItem;
			sAction.iType	= m_iCurrentNumber|TYPE_SCAN_UPDATE;
			m_sActionList.Add(0,0,CSC_END,&sAction);
			m_sNextAction.Unlock();
			}

		UNLOCK();

		if(m_sActionList.Count()>MAX_ACTIONS)return;

		hItem = FindItemEx(hItem,TVIF_RECURSIVE|TVIF_STATE|TVIF_VISIBLE,0,0,0,TVIS_EXPANDED,FLAG_DONE|FLAG_PARTIAL|FLAG_NET|FLAG_NOSCAN|TVIS_EXPANDED);
		}


	hItem = FindItemEx(0,TVIF_RECURSIVE|TVIF_STATE|TVIF_VISIBLE,0,0,0,0,FLAG_DONE|FLAG_PARTIAL|FLAG_NET|FLAG_NOSCAN|TVIS_EXPANDED);

	while(hItem)											// Suche Untereintrge die noch nicht eingelesen sind
		{
		LOCK();

		   iPos  = m_sActionList.Find((int)hItem);
		if(iPos<0)
			{
			sAction.bFlag	= 0;
			sAction.hItem	= hItem;
			sAction.iType	= m_iCurrentNumber|TYPE_SCAN_ONLY;
			m_sActionList.Add(0,0,CSC_END,&sAction);
			m_sNextAction.Unlock();
			}

		UNLOCK();

		if(m_sActionList.Count()>MAX_ACTIONS)return;

		hItem = FindItemEx(hItem,TVIF_RECURSIVE|TVIF_STATE|TVIF_VISIBLE,0,0,0,0,FLAG_DONE|FLAG_PARTIAL|FLAG_NET|FLAG_NOSCAN|TVIS_EXPANDED);
		}

}

//*****************************************************************************
//*
//*		OnCopy
//*
//*****************************************************************************
BOOL CFolderTreeCtrl::OnCopy()
{
HGLOBAL			hGlobal;
UINT			uState;
LPTSTR			pMem;
UINT			uLen;
BOOL			bOk;



	if(m_hCurrentItem==NULL)								// Kein Eintrag gewhlt
		{
		return FALSE;
		}

	   uState=GetItemStateEx(m_hCurrentItem,0xFFFFFFFF);	// Kein realer Pfad
	if(uState&FLAG_NET)
		{
		return FALSE;
		}


															// HGLOBAL mit Pfad erzeugen
	hGlobal	 = GlobalAlloc(GMEM_ZEROINIT,MAX_PATH*sizeof(TCHAR)+24);
	pMem	 = (LPTSTR)GlobalLock(hGlobal);
	pMem[0]	 = 20;
	#ifdef UNICODE
	pMem[8]  = 1;
	#endif
	pMem	+= 20/sizeof(TCHAR);
	uLen	 = GetPath(m_hCurrentItem,pMem,MAX_PATH);
	uLen    += 24;

	GlobalUnlock(hGlobal);

	bOk = FALSE;

	if(OpenClipboard())										// In Clipboard speichern
		{
		EmptyClipboard();

		if(SetClipboardData(CF_HDROP,hGlobal))
			{
			bOk = TRUE;
			}

		CloseClipboard();
		}

	if(!bOk)
		{
		GlobalFree(hGlobal);
		}


return bOk;
}

//*****************************************************************************
//*
//*		SortFolders
//*
//*****************************************************************************
//	Sortiert die Kindereintrge eines Eintrags
int CFolderTreeCtrl::SortFolders(HTREEITEM hItem)
{
TVSORTEX	sSort;


    sSort.lpfnCompare	= SortProcEx;
    sSort.hParent		= hItem;
    sSort.lParam		= 0;;


return SortChildrenEX(&sSort);
}
