//*****************************************************************************
//*
//*		
//*		FolderTreeCtrl.h : Header-Datei
//*
//*
//*****************************************************************************
//
//	Funktionen:
//
//	BOOL	Create	(DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId);
//	int		Add		(const char *pPath,int iMode=0,const char *pPlus=0,const char *pMinus=0);
//
//	BOOL	Scroll			(int iRows)				Eintrge Scrollen
//	BOOL	RedrawAllItems	()						Alles neuzeichen
//	BOOL	RedrawItem(int iItem)					Einen Eintag neuzeichen
//	BOOL	SelectItem(int iItem,BOOL bOn=TRUE)		Eintag auswhlen
//	BOOL	SelectAllItems	()						Alle Eintge auswhlen
//	void	ClearSelection	()						Auswahl aufheben
//	void	Sort(int iMode=CFT_SORT_NAME,int iDir)	Sortieren
//	void	Break()									Add abbrechen
//				  
//	BOOL	DeleteAllItems	()						Alle Eintrge lschen
//	BOOL	DeleteSelItems	()						Ausgewhlte Eintrge lschen
//	BOOL	DeleteItem		(int iItem)				Einen Eintrag lschen
//
//	CDat.. *GetFiles		()						Zeiger auf CFileFinder	Klasse
//	char   *GetPathName		(int iItem)				Pfad abfragen
//	char   *GetFileName		(int iItem,int uFull=0)	Namen abfragen
//	int		GetFileLength	(int iItem)				Dateilnge abfragen
//	int64	GetFileLength64 (int iItem)				Dateilnge abfragen (64 Bit)
//	time_t	GetFileTime		(int iItem)				Dateierstellungszeit abfragen
//
//	BOOL	Save(const char *pFilename)				Dateiliste laden
//	BOOL	Load(const char *pFilename,BOOL bNew=1)	Dateiliste speichern
//
//	BOOL	OnCut()									Clipboard Funktionen		
//	BOOL	OnCopy()								
//	BOOL	OnClear()
//	BOOL	OnPaste() 
	 

#ifndef		__FOLDERTREECTRL_H__
#define     __FOLDERTREECTRL_H__

#if _MSC_VER > 1000
#pragma once
#endif

#include	"SemaphoreEx.h"
#include	"TreeListCtrlEx.h" 

#define 	IDW_FOLDERTREE		0xAB7F			// Standard Id fr Fenster

// Konstanten fr Create
#define 	CFT_TABEXIT			0x0080			// Bei TAB den Fokus ans Elternfenster geben
#define 	CFT_ALLOWDRAG		0x1000			// Dateien per Drag weiter geben
#define 	CFT_ALLOWDROP		0x2000			// Hinzufgen von Dateien per Drag & Drop
#define 	CFT_NORIGHTSEL		0x4000			// Keine Auswahl mit der rechneten Maustaste
#define		CFT_NONETDIRS		0x8000			// Keine Netzlaufwerke anzeigen

// Konstanten fr Notify-Nachrichten
#define		CFT_FOLDERSELECT	0x1204			// StatusBar via Notify anpassen
#define		CFT_DRAGACTION		0x1205			// Begin und Ende von Darg-Aktione
#define		CFT_ITEMDBLCLICK	0x1206			// Doppleclick auf einen Pfad


typedef struct
	{
	NMHDR		sNmHdr;							// Notify Daten
	HTREEITEM	hItem;							// Ausgewhlter Eintrag
	int			iMode;							// 0=Netzwerkszweig 1=Realer Pfad 2=Laufwerk
	LPCTSTR		pPath;							// Pfad des neunen Ordners
	}CFT_FOLDER,*LPCFT_FOLDER;



//*****************************************************************************
//*
//*		KLASSE	CFolderTreeCtrl
//*
//*****************************************************************************
class CFolderTreeCtrl : public CTreeListCtrlEx
{
protected:

	typedef struct
		{
		HTREEITEM	hItem;
		HANDLE		hWait;
		INT			iType;
		BYTE		bFlag;
		TCHAR		cText[1];
		}ActionItem;

public:

	CFolderTreeCtrl();

	BOOL		Create	   		(                DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId=IDW_FOLDERTREE);
	BOOL		CreateEx  		(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId=IDW_FOLDERTREE);
	BOOL		Create     		(                DWORD dwStyle,UINT        uItem,CDialog *pDlg,UINT nId=IDW_FOLDERTREE);
	BOOL		CreateEx   		(DWORD dwExStyle,DWORD dwStyle,UINT        uItem,CDialog *pDlg,UINT nId=IDW_FOLDERTREE);
	BOOL		SetPath			(LPCTSTR pPath  ,LPCTSTR pSubPath=0,int iScan=1);
	BOOL		SynchPath		(HTREEITEM hItem,CFileFinder *pFind,BOOL bVisible=TRUE);
	int 		GetPath			(HTREEITEM hItem,LPTSTR  pBuffer,int iSize,int iMode=0,int *pState=0);
	int 		IsRealPath		(HTREEITEM hItem);
	int			SortFolders		(HTREEITEM hItem);
	void 		SetGridLines	(BOOL bOn);
	void 		SetAlternated 	(BOOL bOn);
	void 		SetFullRowSelect(BOOL bOn);
	int			GetPath			(LPTSTR pBuffer,int iSize,HTREEITEM *pItem=0);
	LPCTSTR		GetPath			();
	int			GetPathMode		();
	BOOL		OnCopy			();
	void		StopThread		();


public:

	//{{AFX_VIRTUAL(CFolderTreeCtrl)
	//}}AFX_VIRTUAL

public:
	virtual ~CFolderTreeCtrl();

protected:

	//{{AFX_MSG(CFolderTreeCtrl)
	afx_msg void OnSetFocus		(CWnd  *pOldWnd);
	afx_msg void OnTimer       	(UINT_PTR nIdEvent);
	afx_msg void OnKeyDown     	(UINT   nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnItemDblClick	(NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnSelChanged	(NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnBeginDrag  	(NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnExpanded     (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void OnDropFiles   	(HDROP  hDropInfo);
	afx_msg void OnScanNetwork	();
	afx_msg void OnDestroy		();
	//}}AFX_MSG

	afx_msg LRESULT OnSelectEntry (WPARAM wParam,LPARAM lParam);
	afx_msg void	OnStartUpdate ();

	DECLARE_MESSAGE_MAP()

	void		ScanUpdate		();
	void		InsertDrives	();
	void		ExpandChain		(HTREEITEM hItem,int iMode=0);
	int			GetIcon			(LPCTSTR   pPath,int iOpenIcon=0);
	int			GetFullNetPath	(LPCTSTR   pPath,int iSize,TCHAR *pBuffer,int *pIsNet=0);
	BOOL		ScanNetwork		(HTREEITEM hRoot,UINT uTimeout=5000);
	UINT		SelectEntry		(HTREEITEM hItem,LPCTSTR pPath,int iLen=0,int iScan=1);
	HTREEITEM	FindPath		(LPCTSTR   pPath,int *pLast=0,int iMax=1024,HTREEITEM hItem=TVI_ROOT,int iOff=0);
	HTREEITEM	AddNetwork		(HTREEITEM hRoot,LPTSTR pPath,int iOff,int iNet=-1);
	HTREEITEM	AddFolder		(HTREEITEM hRoot,LPTSTR pPath,int iOff);
	HTREEITEM	AddDrive		(TCHAR	   cLetter);
	BOOL		IsFixedDrive	(LPCTSTR   pPath);

	static int CALLBACK SortDrives  (HWND hWnd,HTREEITEM hItem ,LPCTSTR pTextItem,LPCTSTR pTextInsert,LPARAM lParamItem,LPARAM lParamInsert);
	static int CALLBACK SortProc    (HWND hWnd,HTREEITEM hItem ,LPCTSTR pTextItem,LPCTSTR pTextInsert,LPARAM lParamItem,LPARAM lParamInsert);
	static int CALLBACK SortProcEx  (HWND hWnd,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParam);
	static DWORD WINAPI	ActionThread(LPVOID pParam);
		   DWORD		ActionThread();
		
//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************
protected:
	
	int					m_iFlags;						// Fensterstyle
	int					m_iNetFlags;					// Such-Flags fr Laufwerke
	BOOL				m_bNetScanned;					// Wurde das Netzwerk gescannt
	UINT				m_uKeyTicker;					// Ticks bei Tastatureingaben
	UINT				m_uStopTicker;					// Ticks bis zum Lschen von Suchen
	UINT				m_uWaitTicker;					// Ticks bis zum Einblenden des Wait-Cursors
	UINT				m_uUpdateTicker;				// Ticks bis zum nchsten Update
	TCHAR				m_cCurrentPath[MAX_PATH];		// Speicher fr aktuellen Pfad
	int 				m_iCurrentNumber;				// Aktuelle Aktionsnummer
	int					m_iCurrentAction;				// Aktionsnummer die ausgefhrt wird
	int					m_iCurrentMode;					// 0=Netzwerkszweig 1=Realer Pfad 2=Laufwerk
	HTREEITEM			m_hCurrentItem;					// Ist der aktuell gewhlte Eintrag 
	HTREEITEM			m_hOpenTreeItem;				// Offener Ordner fr Treefeld
	HTREEITEM			m_hNetworkItem;					// Eintrag in den Netzpfade eingefgt werden
	int					m_iLastOpenIcon;				// Letzter Zustand des ffnen-Icons
	CSemaphoreNull		m_sNextAction;					// Semahore fr den nchsten Auftrag
	CStringChain		m_sActionList;					// Liste mit den Aktionen
	CStringChain		m_sNetPaths;					// Liste Netzwerkspfaden
	CFileFinder			m_sFolders;						// Dateiscanner
	CRITICAL_SECTION 	m_sSynch;						// Synchronisierungs Objekt	
	LONG				m_lWaitCount;					// Wievile wichtige Auftrge warten
	int volatile		m_iStopThread;					// Den Aktions-Thread beenden
	int volatile		m_iThreadRunning;				// Luft der Aktions-Thread
	static HCURSOR		m_hCursorArrow;					// Handle vom Pfeil-Cursor
	static HCURSOR		m_hCursorWait;					// Handle vom Warte-Cursor

};

					  
//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	BOOL		CFolderTreeCtrl::Create			 (DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId){return CreateEx(0,dwStyle,sRect,pParent,uId);}
inline	void		CFolderTreeCtrl::SetFullRowSelect(BOOL bOn            ){SetExtendedStyle((bOn)? TVS_EX_FULLROWITEMS|TVS_EX_FULLROWMARK:0,TVS_EX_FULLROWITEMS|TVS_EX_FULLROWMARK );}
inline	void		CFolderTreeCtrl::SetAlternated   (BOOL bOn            ){SetExtendedStyle((bOn)? TVS_EX_ALTERNATECOLOR:0,TVS_EX_ALTERNATECOLOR);}
inline	void		CFolderTreeCtrl::SetGridLines    (BOOL bOn            ){SetExtendedStyle((bOn)? TVS_EX_ITEMLINES     :0,TVS_EX_ITEMLINES     );}
inline	LPCTSTR		CFolderTreeCtrl::GetPath		 (                    ){return m_cCurrentPath;}
inline	int			CFolderTreeCtrl::GetPathMode	 (                    ){return m_iCurrentMode;}

//{{AFX_INSERT_LOCATION}}



#endif
