//*****************************************************************************
//*
//*
//*		InfoWindow.cpp
//*
//*
//*****************************************************************************
//
//	InfoWindow ist ein Grafikfenster mit dem man schnell und einfach
//	grafische Informationen anzeigen kann:
//
//	Funktion:
//
//		BitmapImage	  *pImage;
//
//		pImage = InfoWindowCreate(320,200,"MeinText");		// Fenster erstellen
//
//		for(;;)
//			{
//			pImage->Box(10,10,30,30,RGB(0,255,0));
//			pImage->Print(10,40,"Hallo");
//			...
//			InfoWindowUpdate(pImage);
//			}
//
//		InfoWindowDelete(pImage);							// Fenster entfernen
//
#include	<AZ_Lib.h>
#include	<AZ_Image.h>
#include	"InfoWindow.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifndef	   _WIN64
#ifndef		SetWindowLongPtr
#define		SetWindowLongPtr		SetWindowLong
#endif
#ifndef		GetWindowLongPtr
#define		GetWindowLongPtr		GetWindowLong
#endif
#else
#ifndef		GWL_USERDATA
#define 	GWL_USERDATA			GWPL_USERDATA
#endif
#ifndef		GWL_WNDPROC
#define 	GWL_WNDPROC				GWLP_WNDPROC
#endif
#endif

#define 	WM_REDRAW				0x1234


unsigned char aFontCourier8[]=
	{
	#include "Image/Fonts/CourierNew8.h"
	};

unsigned char aFontCourier10[]=
	{
	#include "Image/Fonts/CourierNew10.h"
	};

unsigned char aFontCourier12[]=
	{
	#include "Image/Fonts/CourierNew12.h"
	};

unsigned char aFontCourier14[]=
	{
	#include "Image/Fonts/CourierNew14.h"
	};

unsigned char aFontArial6[]=
	{
	#include "Image/Fonts/Arial6.h"
	};

unsigned char aFontArial8[]=
	{
	#include "Image/Fonts/Arial8.h"
	};

unsigned char aFontArial10[]=
	{
	#include "Image/Fonts/Arial10.h"
	};

unsigned char aFontArial12[]=
	{
	#include "Image/Fonts/Arial12.h"
	};

unsigned char aFontArial14[]=
	{
	#include "Image/Fonts/Arial14.h"
	};

unsigned char aFontMicro6[]=
	{
	#include "Image/Fonts/Micro6.h"
	};

unsigned char aFontMicroVar6[]=
	{
	#include "Image/Fonts/MicroVar6.h"
	};

typedef struct
	{
	BitmapImage		sImage;							// Bild auf dem gezeichnet wird
	char			cName[128];						// Fenstername
	HANDLE			hThread;						// Fenster Thread
	HWND volatile	hWnd;							// Fenster Handle
	int				iAddX;
	int				iAddY;
	}InfoWindow;



//*****************************************************************************
//*
//*		InfoWindowProc
//*
//*****************************************************************************
static BOOL CALLBACK InfoWindowProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
static UINT		uNumber=0;
InfoWindow     *pInfoWnd;
PAINTSTRUCT		sPaint;
POINT			sPoint;
RECT			sRect;
RECT		   *pRect;
HDC				hDc;
INT				iSize;



	switch(uMsg)
		{
	case WM_INITDIALOG:

		pInfoWnd = (InfoWindow*)lParam;
		GetWindowRect(hWnd,&sRect);
		sPoint.x	= sRect.left;
		sPoint.y	= sRect.top;
		ScreenToClient(hWnd,&sPoint);
		pInfoWnd->iAddX	= -sPoint.x*2;
		pInfoWnd->iAddY	= -sPoint.x-sPoint.y;
		SetWindowText(hWnd,pInfoWnd->cName);
		SetWindowLongPtr(hWnd,GWL_USERDATA,lParam);
		SetWindowPos(hWnd,NULL,4+uNumber*16,4+uNumber*16,pInfoWnd->sImage.Width()-sPoint.x*2,pInfoWnd->sImage.Height()-sPoint.x-sPoint.y,SWP_NOZORDER);
		ShowWindow(hWnd,SW_SHOW);
		uNumber += 1;
		uNumber %= 30;
		break;

	case WM_REDRAW:

			pInfoWnd = (InfoWindow*)GetWindowLongPtr(hWnd,GWL_USERDATA);
		if(!pInfoWnd)break;

		sRect.top		= 0;
		sRect.left		= 0;
		sRect.right		= pInfoWnd->sImage.Width();
		sRect.bottom	= pInfoWnd->sImage.Height();

		RedrawWindow(pInfoWnd->hWnd,&sRect,NULL,RDW_INVALIDATE);

		return TRUE;

	case WM_SIZING:

			pInfoWnd = (InfoWindow*)GetWindowLongPtr(hWnd,GWL_USERDATA);
		if(!pInfoWnd)break;

		pRect = (LPRECT)lParam;
		iSize = pInfoWnd->sImage.Width()+pInfoWnd->iAddX;

		if(pRect->right-pRect->left>iSize)
			{
			switch(wParam)
				{
			case WMSZ_BOTTOM:
			case WMSZ_BOTTOMLEFT:
			case WMSZ_LEFT:
			case WMSZ_TOPLEFT:		pRect->left = pRect->right-iSize;
									break;

			case WMSZ_BOTTOMRIGHT:
			case WMSZ_RIGHT:
			case WMSZ_TOP:
			case WMSZ_TOPRIGHT:		pRect->right = pRect->left+iSize;
									break;
				}
			}

		iSize = pInfoWnd->sImage.Height()+pInfoWnd->iAddY;

		if(pRect->bottom-pRect->top>iSize)
			{
			switch(wParam)
				{
			case WMSZ_BOTTOM:
			case WMSZ_BOTTOMLEFT:
			case WMSZ_BOTTOMRIGHT:
			case WMSZ_LEFT:			pRect->bottom = pRect->top+iSize;
									break;



			case WMSZ_RIGHT:
			case WMSZ_TOP:
			case WMSZ_TOPLEFT:
			case WMSZ_TOPRIGHT:		pRect->top = pRect->bottom-iSize;
									break;
				}
			}

		return TRUE;

	case WM_PAINT:

			pInfoWnd = (InfoWindow*)GetWindowLongPtr(hWnd,GWL_USERDATA);
		if(!pInfoWnd)break;

        hDc = BeginPaint(hWnd,&sPaint);
        pInfoWnd->sImage.SetDIB(hDc,0,0);
        EndPaint(hWnd,&sPaint);

        return TRUE;
		}



return FALSE;
}

//*****************************************************************************
//*
//*		InfoWindowThread
//*
//*****************************************************************************
//	Fenster-Thread
static DWORD WINAPI InfoWindowThread(LPVOID pParam)
{
InfoWindow     *pInfoWnd=(InfoWindow*)pParam;
MSG				sMsg;
static WORD		aDlgData[]=
					{
					0x0040,0x80CE,0x0000,0x0000,0x0000,0x0000,0x0000,0x00BA,0x005A,0x0000,0x0000,0x0044,0x0069,0x0061,0x006C,0x006F,
					0x0067,0x0000,0x0008,0x004D,0x0053,0x0020,0x0053,0x0061,0x006E,0x0073,0x0020,0x0053,0x0065,0x0072,0x0069,0x0066,
					0x0000,
					};


	pInfoWnd->hWnd = CreateDialogIndirectParam(GetModuleHandle(NULL),(LPCDLGTEMPLATE)aDlgData,NULL,InfoWindowProc,(LPARAM)pInfoWnd);


	while(GetMessage(&sMsg,NULL,0,0))
		{
		TranslateMessage(&sMsg);
		DispatchMessage (&sMsg);

		if(sMsg.hwnd==pInfoWnd->hWnd && sMsg.message==WM_COMMAND && LOWORD(sMsg.wParam)==IDCANCEL)
			{
			break;
			}
		}

	pInfoWnd->hWnd = 0;


return 0;
}



//*****************************************************************************
//*
//*		InfoWindowCreate
//*
//*****************************************************************************
//	Erzeugt ein neues InfoFenster
//	iSizeX/Y	: Ist die Gre des Fentsters in Pixel
//	pName		: Ist der Name des Fensters
//	iBits		: Bits pro Pixel
//	Eribt einen Zeiger auf das Fenster, bzw. 0 bei einem Fehler
BitmapImage	*InfoWindowCreate(int iSizeX,int iSizeY,const char *pName,int iBits)
{
InfoWindow     *pInfoWnd;
DWORD			dwThread;


	if(iSizeX<=0)return 0;
	if(iSizeY<=0)return 0;


	pInfoWnd			= new InfoWindow;			// Neues Bild erzeugen
	pInfoWnd->sImage.SetSize(iBits,iSizeX,iSizeY,RGB(255,255,255));
	pInfoWnd->sImage.SetPixelFunction(TEXT_NORMAL,0,0);
	pInfoWnd->sImage.SetPixelFunction(TEXT_NONE  ,0,1);
	pInfoWnd->sImage.SetFont(aFontArial8);
	strncpy(pInfoWnd->cName,pName,sizeof(pInfoWnd->cName)-1);
	pInfoWnd->cName[sizeof(pInfoWnd->cName)-1]=0;
	pInfoWnd->hThread	= CreateThread(NULL,0x1000,InfoWindowThread,pInfoWnd,0,&dwThread);




return &pInfoWnd->sImage;
}

//*****************************************************************************
//*
//*		InfoWindowDelete
//*
//*****************************************************************************
//	Lscht ein Infofenster
//	pWindow			: Zeiger auf die Fensterdaten
//	Ergibt 1 wenn das Fenster gelscht wurde, bzw. 0 bei einem Fehler
int InfoWindowDelete(BitmapImage *pWindow)
{
InfoWindow     *pInfoWnd=(InfoWindow*)pWindow;


	if(!pInfoWnd)return 0;

	if(pInfoWnd->hWnd)								// Fenster schlieen
		{
		PostMessage(pInfoWnd->hWnd,WM_COMMAND,IDCANCEL,0);
		Sleep(0);
		while(pInfoWnd->hWnd)Sleep(20);
		}

	CloseHandle(pInfoWnd->hThread);
	pInfoWnd->hThread	= 0;
	pInfoWnd->hWnd		= 0;
	pInfoWnd->sImage.Clear();

	delete pInfoWnd;


return 1;
}

//*****************************************************************************
//*
//*		InfoWindowUpdate
//*
//*****************************************************************************
//	Zeichnet das Infofenster neu
//	pWindow			: Zeiger auf die Fensterdaten
void InfoWindowUpdate(BitmapImage *pWindow)
{
InfoWindow     *pInfoWnd=(InfoWindow*)pWindow;


	if(!pInfoWnd)return;

	PostMessage(pInfoWnd->hWnd,WM_REDRAW,0,0);

}
