//*****************************************************************************
//*
//*		
//*		MonitorFunctions.cpp
//*
//*
//*****************************************************************************
#include	"MonitorFunctions.h"

#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
#define 	ASSERT_MONITOR(n)	ASSERT(n)
#else 	
#define 	ASSERT_MONITOR(n)	
#endif 	


#ifndef		SM_CMONITORS
#define 	SM_CMONITORS		80
#endif

#if	   _MSC_VER <= 1299

typedef void     *HMONITOR_;
typedef BOOL     (CALLBACK* MONITORENUMPROC_)(HMONITOR_,HDC,LPRECT,LPARAM);
extern "C"       WINUSERAPI BOOL WINAPI EnumDisplayMonitors(HDC,LPCRECT,MONITORENUMPROC_,LPARAM);

#else

typedef HMONITOR			HMONITOR_;
typedef MONITORENUMPROC		MONITORENUMPROC_;

#endif


		RECT	aMonitorRect[16];						// Alle Monitorpositionen
	 	UINT	uMonitorCount;							// Anzahl der Monitore
static 	UINT	uMonitorPos;							// Monitorposition fr Enum


//*****************************************************************************
//*
//*		MonitorOfPoint
//*
//*****************************************************************************
//	Holt die Monitornummer zu einem Punkt
//	sPoint		: Ist der Punkt zu dem ein Monitor gesucht wird
//	Ergibt die Monitornummer bzw. -1 bei einem Fehler
int MONAPI MonitorOfPoint(POINT sPoint)
{
unsigned	uPos;


	ASSERT_MONITOR(uMonitorCount>0);


	for(uPos=0;uPos<uMonitorCount;uPos++)				// Aktuellen Monitor suchen
		{
		if(PtInRect(aMonitorRect+uPos,sPoint))
			{
			return uPos;
			}
		}


return -1;
}

//*****************************************************************************
//*
//*		MonitorOfCoord
//*
//*****************************************************************************
//	Holt die Monitornummer zu einem Punkt
//	iPosX/iPosY	: Ist der Punkt zu dem ein Monitor gesucht wird
//	Ergibt die Monitornummer bzw. -1 bei einem Fehler
int MONAPI MonitorOfCoord(int iPosX,int iPosY)
{
POINT	sPoint;


	sPoint.x	= iPosX;
	sPoint.y	= iPosY;


return MonitorOfPoint(sPoint);
}

//*****************************************************************************
//*
//*		MonitorOfRect
//*
//*****************************************************************************
//	Holt die Monitornummer zum Mittelpunkt eines CREATESTRUCT-Rechteckes
//	pStruct		: Zeiger auf das CREATESTRUCT-Rechteck
//	Ergibt die Monitornummer bzw. -1 bei einem Fehler
int MONAPI MonitorOfRect(CREATESTRUCT *pStruct)
{
POINT	sPoint;

	
	ASSERT_MONITOR(pStruct);
	ASSERT_MONITOR(uMonitorCount>0);

	sPoint.x	= pStruct->x;
	sPoint.y	= pStruct->y;

	if(pStruct->cx>0)sPoint.x+=pStruct->cx/2;
	if(pStruct->cy>0)sPoint.y+=pStruct->cy/2;


return MonitorOfPoint(sPoint);
}

//*****************************************************************************
//*
//*		MonitorOfRect
//*
//*****************************************************************************
//	Holt die Monitornummer zum Mittelpunkt eines Rechteckes
//	pRect		: Zeiger auf das Rechteck
//	Ergibt die Monitornummer bzw. -1 bei einem Fehler
int MONAPI MonitorOfRect(RECT *pRect)
{
POINT	sPoint;

	ASSERT_MONITOR(pRect);
	ASSERT_MONITOR(uMonitorCount>0);

	sPoint.x	=  (pRect->left+pRect->right )/2;
	sPoint.y	=  (pRect->top +pRect->bottom)/2;


return MonitorOfPoint(sPoint);
}

//*****************************************************************************
//*
//*		MonitorOfWindow
//*
//*****************************************************************************
//	Holt die Monitornummer zum Mittelpunkt eines Rechteckes
//	hWnd		: Handle des Fensters
//	Ergibt die Monitornummer bzw. -1 bei einem Fehler
int MONAPI MonitorOfWindow(HWND hWnd)
{
RECT	sRect;

	
	ASSERT_MONITOR(IsWindow(hWnd));
	ASSERT_MONITOR(uMonitorCount>0);


	if(!GetWindowRect(hWnd,&sRect))
		{
		return -1;
		}


return MonitorOfRect(&sRect);
}


//*****************************************************************************
//*
//*		MonitorEnumProc
//*
//*****************************************************************************
static BOOL CALLBACK MonitorEnumProc(HMONITOR_ hMonitor,HDC hHdcMonitor,LPRECT pRect,LPARAM lData)
{

	if(uMonitorPos>=16)return FALSE;

	aMonitorRect[uMonitorPos] = *pRect;
	uMonitorPos++;


return TRUE;
}

//*****************************************************************************
//*
//*		MonitorFrameCheck
//*
//*****************************************************************************
//	Prft on ein Frame in einem Monitor vernnftig sichtbar ist
//	uMonitor		: Ist die Monitornummer
//	iPosX/iPosY		: Die Linke ober Ecke der Frames
//	iSizeX/iSizeY	: Breite und Hhe des Frames
//	iAdd			: Ist die Breite des Sicherheitsframes
//	Ergibt TRUE wenn der Frame in Ortnung ist.
int MONAPI MonitorFrameCheck(unsigned uMonitor,int iPosX,int iPosY,int iSizeX,int iSizeY,int iAdd)
{
RECT		sRect;


	ASSERT_MONITOR(uMonitorCount>0);

	if(uMonitor>=uMonitorCount)
		{
		sRect.left	 = 0;
		sRect.top	 = 0;
		sRect.right  = GetSystemMetrics(SM_CXSCREEN);
		sRect.bottom = GetSystemMetrics(SM_CYSCREEN);
		}
	else{
		sRect = aMonitorRect[uMonitor];
		}

	 if(iSizeX<iAdd || iSizeY<iAdd  )return FALSE;
	 if(iPosX+iSizeX<sRect.left+iAdd)return FALSE;
	 if(iPosY+iSizeY<sRect.top +iAdd)return FALSE;
	 if(iPosX+iAdd  >sRect.right    )return FALSE;
	 if(iPosY+iAdd  >sRect.bottom   )return FALSE;


return TRUE;
}

//*****************************************************************************
//*
//*		MonitorInit
//*
//*****************************************************************************
//	Holt alle Monitorpositionen
//	bForce		: Forciert das berschreiben der alten Positionen
//	Ergibt die Anzahl der gefunden Monitore
int MONAPI MonitorInit(BOOL bForce)
{


	if(!uMonitorCount || bForce)
		{
		uMonitorCount	= GetSystemMetrics(SM_CMONITORS);
		uMonitorPos		= 0;


		if(uMonitorCount<=1)							// Monitor-Positionen holen
			{
			aMonitorRect[0].top		= 0;
			aMonitorRect[0].left	= 0;
			aMonitorRect[0].right	= GetSystemMetrics(SM_CXSCREEN);
			aMonitorRect[0].bottom	= GetSystemMetrics(SM_CYSCREEN);
			uMonitorCount = 1;

			// REM AZ nur fr Test
			/*
			#ifdef _DEBUG
			uMonitorCount		  = 2;
			aMonitorRect[1]       = aMonitorRect[0];
			aMonitorRect[1].left  = aMonitorRect[1].right/2;
			aMonitorRect[0].right = aMonitorRect[1].right/2-1;
			#endif
			//*/
			}
		else{
			EnumDisplayMonitors(NULL,NULL,MonitorEnumProc,NULL);

			while(uMonitorPos<uMonitorCount)
				{
				aMonitorRect[uMonitorPos].top	 = -1;
				aMonitorRect[uMonitorPos].left	 = -1;
				aMonitorRect[uMonitorPos].right	 = -1;
				aMonitorRect[uMonitorPos].bottom = -1;

				uMonitorPos++;
				}
			}
		}


return uMonitorCount;
}

//*****************************************************************************
//*
//*		MonitorMove
//*
//*****************************************************************************
//	Verschiebt ein Fenster in einen Monitor
//	hWnd		: Handle des Fensters
//	uMonitor	: Nummer des Monitors
//	iMode		: Verschibe Modus
//					0 = Relativ zum aktuellen Monitor
//					1 = In die Mitte des Monitors
//	Ergibt TRUE wenn das Fenster verschoben wurde
BOOL MONAPI MonitorMove(HWND hWnd,unsigned uMonitor,int iMode)
{
HWND		hParent;
POINT		sPoint;
RECT		sRect;
int			iAddX;
int			iAddY;
int			iMon;


	ASSERT_MONITOR(IsWindow(hWnd));
	ASSERT_MONITOR(uMonitorCount>0);

	if(uMonitor>=uMonitorCount)
		{
		return FALSE;
		}

	if(uMonitorCount<=1)return FALSE;					// Nur ein Monitor vorhanden

	switch(iMode)
		{
	case 0:												// Relativ verschieben
			iMon	= MonitorOfWindow(hWnd);

			if(iMon<0 || iMon==(int)uMonitor)return FALSE;

			iAddX    = aMonitorRect[uMonitor].left;
			iAddY    = aMonitorRect[uMonitor].top;
			iAddX   -= aMonitorRect[iMon    ].left;
			iAddY   -= aMonitorRect[iMon    ].top;

			if(!GetWindowRect(hWnd,&sRect))return FALSE;
				  
				hParent=GetParent(hWnd);
			if(!hParent)return FALSE;

			sPoint.x = 0;
			sPoint.y = 0;

			ClientToScreen(hParent,&sPoint);

			iAddX   -= sPoint.x;
			iAddY   -= sPoint.y;

			SetWindowPos(hWnd,0,sRect.left+iAddX,sRect.top+iAddY,0,0,SWP_NOOWNERZORDER|SWP_NOSIZE|SWP_NOZORDER);

			break;

	case 1:												// In die Mitte vom Monitor
			if(uMonitor>=uMonitorCount)return FALSE;

			if(!GetWindowRect(hWnd,&sRect))return FALSE;

			sPoint.x = (aMonitorRect[uMonitor].left+aMonitorRect[uMonitor].right )/2;
			sPoint.y = (aMonitorRect[uMonitor].top +aMonitorRect[uMonitor].bottom)/2;

			iAddX	 = sPoint.x-(sRect.right -sRect.left)/2;
			iAddY	 = sPoint.y-(sRect.bottom-sRect.top )/2;

			SetWindowPos(hWnd,0,iAddX,iAddY,0,0,SWP_NOOWNERZORDER|SWP_NOSIZE|SWP_NOZORDER);

			break;

	default:											// Falscher Modus
			return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		MonitorMove
//*
//*****************************************************************************
//	Verschiebt ein Rechteck in einen Monitor
//	pRect		: Zeiger auf ein Rechteck mit den aktuellen Koordinaten
//	uMonitor	: Nummer des Monitors
//	iMode		: Verschiebe-Modus
//					0 = Relativ zum aktuellen Monitor
//					1 = In die Mitte des Monitors
//	Ergibt TRUE wenn das Fenster verschoben wurde
BOOL MONAPI MonitorMove(CREATESTRUCT *pStruct,unsigned uMonitor,int iMode)
{
RECT		sRect;

	
	ASSERT_MONITOR(pStruct);
	ASSERT_MONITOR(uMonitorCount>0);

	sRect.left		= (pStruct->x==CW_USEDEFAULT)? 20:pStruct->x;
	sRect.top		= (pStruct->y==CW_USEDEFAULT)? 20:pStruct->y;
	sRect.right		= sRect.left;
	sRect.bottom	= sRect.top;

	if(pStruct->cx==CW_USEDEFAULT)sRect.right +=aMonitorRect[0].right -aMonitorRect[0].left-100;
	if(pStruct->cy==CW_USEDEFAULT)sRect.bottom+=aMonitorRect[0].bottom-aMonitorRect[0].top -100;
	if(pStruct->cx>0)sRect.right +=pStruct->cx;
	if(pStruct->cy>0)sRect.bottom+=pStruct->cy;

	if(!MonitorMove(&sRect,uMonitor,iMode))return FALSE;

	pStruct->x		= sRect.left;
	pStruct->y		= sRect.top;

	if(pStruct->cx>0)pStruct->cx=sRect.right -sRect.left;
	if(pStruct->cy>0)pStruct->cy=sRect.bottom-sRect.right;


return TRUE;
}

//*****************************************************************************
//*
//*		MonitorMove
//*
//*****************************************************************************
//	Verschiebt ein Rechteck in einen Monitor
//	pRect		: Zeiger auf ein Rechteck mit den aktuellen Koordinaten
//	uMonitor	: Nummer des Monitors
//	iMode		: Verschiebe-Modus
//					0 = Relativ zum aktuellen Monitor
//					1 = In die Mitte des Monitors
//	Ergibt TRUE wenn das Fenster verschoben wurde
BOOL MONAPI MonitorMove(LPRECT pRect,unsigned uMonitor,int iMode)
{
POINT		sPoint;
int			iAddX;
int			iAddY;
int			iMon;


	ASSERT_MONITOR(pRect);
	ASSERT_MONITOR(uMonitorCount>0);

	if(uMonitor>=uMonitorCount)
		{												// Prfe die Monitornummer
		return FALSE;
		}

	if(uMonitorCount<=1)return FALSE;					// Nur ein Monitor vorhanden

	switch(iMode)
		{
	case 0:												// Relativ verschieben
			iMon	= MonitorOfRect(pRect);

			if(iMon<0 || iMon==(int)uMonitor)return FALSE;

			iAddX   = aMonitorRect[uMonitor].left;
			iAddY   = aMonitorRect[uMonitor].top;
			iAddX  -= aMonitorRect[iMon    ].left;
			iAddY  -= aMonitorRect[iMon    ].top;

			pRect->left		+= iAddX;
			pRect->right	+= iAddX;
			pRect->top		+= iAddY;
			pRect->bottom	+= iAddY;

			break;

	case 1:												// In die Mitte vom Monitor

			if(uMonitor>=uMonitorCount)return FALSE;

			sPoint.x = (aMonitorRect[uMonitor].left+aMonitorRect[uMonitor].right )/2;
			sPoint.y = (aMonitorRect[uMonitor].top +aMonitorRect[uMonitor].bottom)/2;
			iAddX	 =  pRect->right -pRect->left;
			iAddY	 =  pRect->bottom-pRect->top;

			if(iAddX<0)iAddX=0;
			if(iAddY<0)iAddY=0;

			pRect->left		+= sPoint.x   -iAddX/2;
			pRect->right	+= pRect->left+iAddX;
			pRect->top		+= sPoint.y   -iAddY/2;
			pRect->bottom	+= pRect->top +iAddY;

			break;

	default:											// Falscher Modus
			return FALSE;
		}


return TRUE;
}
