//*****************************************************************************
//*
//*
//*		PathCtrl.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2015
//
#ifndef	  __PATHCTRL_H__
#define   __PATHCTRL_H__

#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#include	<AfxWin.h>
#include	<AfxCmn.h>
#include	<AfxMt.h>

#include	"FileFinder.h"
#include	"StringChain.h"

#define 	ID_PATHCTRL_DEF		0x3471					// Standart-ID fr das Fenster
#define 	CPC_MAX_FOLDERS		32						// Maximale Anzahl an Verzeichnissen im Pfad
#define 	CPC_ALLOWDRAG		0x0001					// Dateien per Drag weiter geben
#define 	CPC_ALLOWDROP		0x0002					// Hinzufgen von Pfaden per Drag & Drop
#define 	CPC_ADDWITHENTER	0x0004					// Pfade mit Enter in die Liste einfgen
#define 	CPC_HASHISTORY		0x0008					// Hat das Fenster eine Histrorie
#define 	CPC_PATHMUSTEXIST	CBS_OWNERDRAWFIXED		// Der Pfad muss existieren
#define 	CPC_NOBREADCRUMB	CBS_HASSTRINGS			// Keine Brotkrmel-Navigation
#define 	CPC_TABEXIT			CBS_AUTOHSCROLL			// Fokus zum Eltern-Fenster bei TAB
#define 	CPC_NONETDIRS		CBS_SORT				// Keine Netzlaufwerke


// Konstanten fr Notify-Nachrichten
#define		CPC_FOLDERSELECT	0x1208					// StatusBar via Notify anpassen
#define		CPC_DRAGACTION		0x1209					// Begin und Ende von Darg-Aktione

// Konstanten fr SetPath
#define		CPC_ICONOK			0x00008000				// Das Icon ist gesetzt (nur intern)
#define		CPC_PATHNOK			0x00004000				// Der Pfad exitiert nicht (nur intern)
#define		CPC_NETPATH			0x497A0000				// Der Pfad ist ein Netzpfad
#define		CPC_FULLPATH		0x497B0000				// Der Pfad ist ein voller pfad



typedef struct
	{
	NMHDR		sNmHdr;									// Notify Daten
	LPVOID		hDummy;									// Unbenutzt
	int			iMode;									// 0=Netzwerkszweig 1=Realer Pfad 2=Laufwerk 3=Neuer Pfad
	LPCTSTR		pPath;									// Pfad des neunen Ordners
	}CPC_FOLDER,*LPCPC_FOLDER;

//*****************************************************************************
//*
//*		Klasse CPathCtrl
//*
//*****************************************************************************
class CPathCtrl : public CComboBox
{
public:

	CPathCtrl();
   ~CPathCtrl();

	static void GlobalInit();
	static int  GetRawHeight();

	BOOL Create  (                              const RECT &sRect ,CWnd *pParentWnd,UINT nID=ID_PATHCTRL_DEF);
	BOOL Create  (                DWORD dwStyle,const RECT &sRect ,CWnd *pParentWnd,UINT nID=ID_PATHCTRL_DEF);
	BOOL Create  (                              unsigned    uId,CDialog *pDialog   ,UINT nID=ID_PATHCTRL_DEF);
	BOOL Create  (                DWORD dwStyle,unsigned    uId,CDialog *pDialog   ,UINT nID=ID_PATHCTRL_DEF);
	BOOL CreateEx(DWORD dwExStyle,              const RECT &sRect ,CWnd *pParentWnd,UINT nID=ID_PATHCTRL_DEF);
	BOOL CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect ,CWnd *pParentWnd,UINT nID=ID_PATHCTRL_DEF);
	BOOL CreateEx(DWORD dwExStyle,              unsigned    uId,CDialog *pDialog   ,UINT nID=ID_PATHCTRL_DEF);
	BOOL CreateEx(DWORD dwExStyle,DWORD dwStyle,unsigned    uId,CDialog *pDialog   ,UINT nID=ID_PATHCTRL_DEF);

	BOOL SubclassDlgItem(DWORD dwStyle,UINT uId,CDialog *pDlg);
	BOOL SubclassWindow (DWORD dwStyle,CWnd *pWnd);
	BOOL SubclassWindow (DWORD dwStyle,HWND  hWnd);
	HWND UnsubclassWindow();

	int	 SetPath(LPCTSTR pPath,int iSize=-1);	
	BOOL StopEdit      (BOOL bCancel=TRUE);
	BOOL SetBreadCrumb (BOOL bOn);
	BOOL HistoryJump   (BOOL bUp);
	BOOL SetAllowDrop  (BOOL bOn);
	BOOL GetTextChanged();
	BOOL GetBreadCrumb ();
	BOOL GetAllowDrop  ();
	BOOL GetEditMode   ();


//*****************************************************************************
//*
//*		Variablen
//*
//*****************************************************************************
protected:

	typedef struct
		{
		CPathCtrl	   *pThis;								// Zeiger auf eigene Instanz
		CSemaphore	    sSem;								// Warte Semaphore 
		CFileFinder	    sFind;								// Zeiger auf die Dateisuchklasse
		UINT volatile	uInitOk;							// Ist die Instanz ok (0=none 1=init 2=exit 3=stopped)
		UINT volatile	uCommand;							// Command fr die Suche
		UINT volatile	uCurrent;							// Aktuelle Suche
		UINT volatile	uDoneCmd;							// Fertige Suche
		}ThreadData;

	typedef struct
		{
		INT				iIcon;								// Ist die Icon-Nummer
		WORD			uTextSize;							// Breite des Textes in Pixel
		BYTE			uType;								// Art des Verzeichnisses (0=Normal 1=Folder 2=Nicht erstellt)
		BYTE			uBits;								// Select und Focused Bits
		}DirData;

	static CCriticalSection	m_sLock;						// Globale Sperre
	static HFONT		m_hFontText;						// Font fr den Text
	static HFONT		m_hFontChar;						// Font fr den Spezialzeichen
	static HMODULE		m_hUxThemeDll;						// Handle von UxTheme.dll
	static HIMAGELIST	m_hImageList;						// Handle fr Icon-Liste
	static INT			m_iPathImage;						// Standart Icon fr Pfade
	static INT			m_iImagesSizeX;						// Gre der Icons
	static INT			m_iImagesSizeY;						//		*
	static INT			m_iItemHeight;						// Ist die Hhe eines Listeneintrages
	static INT			m_iItemStart;						// Ist die Icon-Breite in der Liste
	static INT			m_iFontHeight;						// Ist die Schriftgre
	static INT			m_iRawHeight;						// Ist die Zeilengre
	static UINT			m_uKeyPos;							// Aktuelle Position im Textspeicher
	static DWORD		m_uKeyLast;							// Tick der letzten Tasteneingabe
	static TCHAR		m_cKeyData[16];						// Textspeicher
	
	TCHAR				m_cPath      [MAX_PATH];			// Aktueller Pfad
	TCHAR				m_cEditPath  [MAX_PATH];			// Letzter eingegbener Pfad
	CStringChain		m_aPathDirs	 [CPC_MAX_FOLDERS+1];	// Liste mit Verzeichnissen
	WORD				m_aPathSize  [CPC_MAX_FOLDERS+1];	// Textlnge der Unterpfade
	WORD				m_aPathWidth [CPC_MAX_FOLDERS+1];	// Textbreite in Pixel
	WORD				m_aPathHeight[CPC_MAX_FOLDERS+1];	// Listenhhe in Pixel
	WORD				m_aPathStart [CPC_MAX_FOLDERS*2+2];	// Lnge der Unterpfade in Pixel
	BYTE				m_aPathScroll[CPC_MAX_FOLDERS+1];	// Hat die Liste eine Scrollbar
	INT					m_aPathOffset[CPC_MAX_FOLDERS+1];	// Textlnge der Unterpfade 
	INT					m_iBaseOffset;						// Index des ersten Unterpfade
	INT					m_iBasePos;							//		*	
	INT					m_iBaseEnd;							// Letztes sichtbares Segmen
	INT					m_iSegSel;							// Ausgewhlter Abschnitt
	INT					m_iKeySel;							// Ausgewhlter Abschnitt via Tastatur
	INT					m_iFolders;							// Anzahl der Folder
	INT					m_iFolderIcon;						// Icon des Folders
	INT					m_iPathType;						// 0=Netzwerkszweig 1=Realer Pfad 2=Laufwerk 3=Neuer Pfad
	INT					m_iIconNext;						// Das Icon fpr SetPath(...)
	HANDLE				m_hTheme;							// Thema fr's Zeichen
	HWND				m_hEdit;							// Handle vom Edit-Fenster
	INT					m_iButtonEnd;						// Position der Taste
	INT					m_iButtonStart;						//		*
	INT					m_iButtonSize;						// Breite der Taste
	INT					m_iButtonMode;						// Zustand der Taste
	INT					m_iWndSizeX;						// Fensterabmessungen
	INT					m_iWndSizeY;						//		*
	INT					m_iListLast;						// Welche Liste war gewhlt
	INT					m_iListBits;						// Welche Liste ist aktiv
	INT					m_iListSel;							// Welche Liste ist gewhlt
	INT					m_iListCol;							// Spalte der aktiven Liste
	INT					m_iListMax;							// Maximale Lnge der Liste
	INT	volatile		m_iTurn;							// Aktueller Umlauf
	DWORD				m_uLastDrives;						// Zuletzt eingefgte Laufwerke
	DWORD				m_uLockClick;						// Sperre fr Mausklicks
	WNDPROC				m_pOldEdit;							// Alte Edit-Fenster Funktion
	WNDPROC				m_pOldList;							// Alte List-Fenster Funktion
	bool				m_bTickerOn;						// Ist der Ticker ein
	bool				m_bEditOn;							// Edit-Fenster sichtbar	
	bool				m_bListOn;							// Ist die Verzeichnissliste ein
	bool				m_bTabExit;							// Fokus zum Eltern-Fenster bei TAB
	bool				m_bIconClick;						// Gab es einen Klick aufs Icon
	bool				m_bMustExist;						// Muss der Pfad existieren
	bool				m_bAllowDrag;						// Dateien per Drag weiter geben
	bool				m_bAllowDrop;						// Hinzufgen von Pfaden per Drag & Drop
	bool				m_bBreadCrumb;						// Brotkrmel-Navigation erlaubt
	bool				m_bAddWihtRet;						// Pfade mit Enter in die Liste einfgen 
	bool				m_bHasHistory;						// Hat das Fenster eine Histrorie
	bool				m_bNoNetPaths;						// Keine Netzwerkspfade
	bool				m_bTextChanged;						// Hat sich der Text im Edit-Fenster verndert
	BYTE				m_bEditCheck;						// Die Position des Edit-Fensters prfen
	CPoint				m_sClickPoint;						// Letzte Klick in Fenster
	CListCtrl			m_cDirList;							// Fenster fr Verzeichnissliste
	CStringChain		m_sHistory;							// Pfade der Historie
	int					m_iHistoryPos;						// Position in der Historie
	int					m_iHistoryMax;						// Maximale Anzahl an Historieeintrgen
	int					m_iMenuSegment;						// Gewhltes Segment vom Men
	ThreadData		   *m_pThreadData;						// Zeiger auf Daten vom Thread

	//{{AFX_MSG(CPathCtrl)
public:
	virtual void DrawItem   (LPDRAWITEMSTRUCT    pDrawItem);
	virtual void MeasureItem(LPMEASUREITEMSTRUCT pMeasureItem);
protected:
	afx_msg LRESULT OnMouseLeave(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnSetCurSel (WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnSelString	(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnSetText	(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnGetText	(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnGetTextLen(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnSetSytle  (WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnThreadMsg (WPARAM wParam,LPARAM lParam);
	afx_msg void OnGetListItem  (NMHDR *pNotify,LRESULT *pResult);
	afx_msg void OnListItemClick(NMHDR *pNotify,LRESULT *pResult);
	afx_msg void OnSize(UINT nType,int iSizeX,int iSizeY);
	afx_msg void OnStyleChanging(int nStyleType,LPSTYLESTRUCT pStyle);
	afx_msg void OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags);
	afx_msg void OnLButtonDblClk(UINT nFlags,CPoint sPoint);
	afx_msg void OnRButtonDown(UINT nFlags,CPoint sPoint);
	afx_msg void OnLButtonDown(UINT nFlags,CPoint sPoint);
	afx_msg void OnLButtonUp(UINT nFlags,CPoint sPoint);
	afx_msg void OnMouseMove(UINT nFlags,CPoint sPoint);
	afx_msg void OnDropFiles(HDROP hDropInfo);
	afx_msg void OnTimer(UINT_PTR nIdEvent);
	afx_msg void OnEditNoFocus();
	afx_msg void OnEditChange();
	afx_msg void OnEditFocus();
	afx_msg void OnCopyName();
	afx_msg void OnCopyPath();
	afx_msg void OnDestroy();
	afx_msg void OnCloseUp();
	afx_msg void OnPaint();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP();

	void LocalInit    (DWORD dwStyle);
	void UpdateDrives (unsigned uDrives);
	int  SelectFolder (int iSegment,int iItem);
	int  SelectSegment(int iSegment);
	int  StartSearch  (int iSegment);
	int  ShowList     (int iSegment);
	void OnListKeyDown(int iVirtKey);
	void OnCharInput  (int iChar);
	void OnMouseCheck ();
	void OnExitEdit   ();
	void OnTakeEdit   ();
	BOOL PostInit     ();
	void CalcBase     ();
	void EditSel      ();
	void EditPos      ();
	BOOL BeginDrag    ();
	
	static LRESULT WINAPI	ListProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);
	static LRESULT WINAPI	EditProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);
	static INT	   WINAPI	GetSmallIcon(LPCTSTR pPath,DWORD &uAttr);	
	static DWORD   WINAPI	SearchThread(LPVOID  pParam);


};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
BOOL inline	CPathCtrl::Create  (                const RECT &sRect ,CWnd *pParentWnd,UINT nID){return CreateEx(0        ,WS_CHILD|WS_VISIBLE,sRect ,pParentWnd,nID);}
BOOL inline	CPathCtrl::Create  (DWORD dwStyle  ,const RECT &sRect ,CWnd *pParentWnd,UINT nID){return CreateEx(0        ,dwStyle            ,sRect ,pParentWnd,nID);}
BOOL inline	CPathCtrl::Create  (                unsigned    uId,CDialog *pDialog   ,UINT nID){return CreateEx(0        ,WS_CHILD|WS_VISIBLE,uId   ,pDialog   ,nID);}
BOOL inline	CPathCtrl::Create  (DWORD dwStyle  ,unsigned    uId,CDialog *pDialog   ,UINT nID){return CreateEx(0        ,dwStyle            ,uId   ,pDialog   ,nID);}
BOOL inline	CPathCtrl::CreateEx(DWORD dwExStyle,const RECT &sRect ,CWnd *pParentWnd,UINT nID){return CreateEx(dwExStyle,WS_CHILD|WS_VISIBLE,sRect ,pParentWnd,nID);}
BOOL inline	CPathCtrl::CreateEx(DWORD dwExStyle,unsigned    uId,CDialog *pDialog   ,UINT nID){return CreateEx(dwExStyle,WS_CHILD|WS_VISIBLE,uId   ,pDialog   ,nID);}
BOOL inline	CPathCtrl::SubclassDlgItem(DWORD dwStyle,UINT uId,CDialog *pDlg){ return SubclassWindow(dwStyle,pDlg->GetDlgItem(uId));}
BOOL inline	CPathCtrl::SubclassWindow (DWORD dwStyle,CWnd *pWnd){ASSERT(pWnd);return SubclassWindow(dwStyle,pWnd->m_hWnd);}
BOOL inline	CPathCtrl::GetTextChanged(){return m_bTextChanged;};
BOOL inline	CPathCtrl::GetBreadCrumb (){return m_bBreadCrumb;};
BOOL inline	CPathCtrl::GetAllowDrop  (){return m_bAllowDrop;};
BOOL inline	CPathCtrl::GetEditMode   (){return m_bEditOn;};


#endif
