//*****************************************************************************
//*
//*
//*		CPictureCtrl
//*
//*
//*****************************************************************************
//
//	CPictureCtrl ist eine Fenster-Klasse in der man ein Bildladen kann
//
//	Funktionen:
//
//		BOOL  Create(				 		         const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF )
//		BOOL  Create(				   DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF )
//		BOOL  Create(LPCTSTR pFilename,              const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF )
//		BOOL  Create(LPCTSTR pFilename,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF )
//		BOOL  Create(UINT    uResId   ,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF )
//		BOOL  Remove()										// Grafik entfernen
//		BOOL  Save  (LPCTSTR pFilename)						// Grafik speichern
//		BOOL  Open  (LPCTSTR pFilename)						// Grafik laden
//		BOOL  Open  (UINT uResId )							// Grafik aus Resource laden
//		BOOL  GetSize(int &iSizeX,int &iSizeY)				// Grafikgre holen (1/100mm)
//		CSize GetSize()										// Grafikgre holen (1/100mm)
//		void  SetDefaultBrush(HBRUSH hBrush)				// Hintergrundfarbe fr leeres Bild
//		void  SetDefaultBrush(int    iBrush)				// Hintergrundfarbe fr leeres Bild ber GetStockObjekt(Brush)
//		void  SetViewRect    (LPRECT pRect )				// Anzeige Position
//		char* GetFileName()									// Dateinamen abfragen
//
//	Anmerkung:
//
//		CMfcTestModule _Module;								// Einfgen in Code
//

#ifndef __PICTURECTRL_H__
#define __PICTURECTRL_H__

#if		_MSC_VER > 1000
#pragma once
#endif


#include "BackgroundCtrl.h"


//*****************************************************************************
//*
//*		Klasse	CPictureCtrl
//*
//*****************************************************************************
class CPictureCtrl : public CStatic
{
public:

	CPictureCtrl(){};

	inline BOOL 	Create(					 		        const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF);
	inline BOOL 	Create(					  DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF);
	inline BOOL 	Create( LPCTSTR pFilename,              const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF);
	inline BOOL 	Create( LPCTSTR pFilename,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF);
	inline BOOL 	Create( UINT	uResId   ,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=0xFFFF);
	inline BOOL		Open  ( UINT	uResId    );
	inline BOOL		Open  ( LPCTSTR pFilename );
	inline BOOL		Save  ( LPCTSTR pFilename );
	inline BOOL	    GetSize(int &iSizeX,int &iSizeY);
	inline BOOL		Remove();
	inline CSize    GetSize();
	inline void 	SetDefaultBrush(HBRUSH hBrush);
	inline void 	SetDefaultBrush(int    iBrush);
	inline void		SetViewRect    (LPRECT pRect );
	inline LPCSTR	GetFileName();

protected:

	CBackgroundCtrl	m_sBackgndWnd;

};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

inline BOOL CPictureCtrl::Create(const RECT &sRect,CWnd *pParentWnd,UINT nID)
	{
	return Create((LPCTSTR)0,WS_CHILD|WS_VISIBLE,sRect,pParentWnd,nID);
	}
inline BOOL CPictureCtrl::Create(LPCTSTR pFilename,const RECT &sRect,CWnd *pParentWnd,UINT nID)
	{
	return Create(pFilename,WS_CHILD|WS_VISIBLE,sRect,pParentWnd,nID);
	}
inline BOOL CPictureCtrl::Create(DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID)
	{
	return Create((LPCTSTR)0,dwStyle,sRect,pParentWnd,nID);
	}
inline BOOL CPictureCtrl::Create(LPCTSTR pFilename,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID)
	{
	BOOL bOk;
	bOk=CStatic::Create(_T(""),dwStyle|SS_USERITEM,sRect,pParentWnd,nID);
	if(!bOk)return FALSE;
	return m_sBackgndWnd.Init(this,pFilename,0);
	}
inline BOOL CPictureCtrl::Create(UINT uResId,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID)
	{
	BOOL bOk;
	bOk=CStatic::Create(_T(""),dwStyle|SS_USERITEM,sRect,pParentWnd,nID);
	if(!bOk)return FALSE;
	return m_sBackgndWnd.Init(this,uResId,0);
	}
inline LPCSTR CPictureCtrl::GetFileName(                   ){return m_sBackgndWnd.GetFileName()          ;}
inline BOOL   CPictureCtrl::Remove(                        ){return m_sBackgndWnd.Remove()               ;}
inline BOOL   CPictureCtrl::Save(LPCTSTR pFilename         ){return m_sBackgndWnd.Save(pFilename)        ;}
inline BOOL   CPictureCtrl::Open(LPCTSTR pFilename         ){return m_sBackgndWnd.Open(pFilename)        ;}
inline BOOL	  CPictureCtrl::Open(UINT	 uResId            ){return m_sBackgndWnd.Open(uResId)           ;}
inline BOOL	  CPictureCtrl::GetSize(int &iSizeX,int &iSizeY){return m_sBackgndWnd.GetSize(iSizeX,iSizeY) ;}
inline CSize  CPictureCtrl::GetSize(                       ){return m_sBackgndWnd.GetSize()              ;}
inline void   CPictureCtrl::SetDefaultBrush(HBRUSH hBrush  ){       m_sBackgndWnd.SetDefaultBrush(hBrush);}
inline void   CPictureCtrl::SetDefaultBrush(int    iBrush  ){       m_sBackgndWnd.SetDefaultBrush(iBrush);}
inline void	  CPictureCtrl::SetViewRect    (LPRECT pRect   ){       m_sBackgndWnd.SetViewRect    (pRect );}

#endif
