//*****************************************************************************
//*
//*
//*		RecentFileListUrl.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2010
//
#include	"RecentFileListUrl.h"
#include	"StringFunctions.h"
#include	"FileFunctions.h"
#include	"AfxUtility.h"


#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif


//*****************************************************************************
//*
//*		Add
//*
//*****************************************************************************
// 	URL in die Liste einfgen
// 	pPathName		: Ist der Zeiger auf die URL (z.B.: "file://D:/test.txt")
void CRecentFileListUrl::Add(LPCTSTR pPathName)
{
TCHAR		cPath[MAX_PATH];
int			iPos;


	ASSERT(m_arrNames != NULL);
	ASSERT(pPathName  != NULL);
	ASSERT(AfxIsValidString(pPathName));


	strlcpy(cPath,pPathName,MAX_PATH);
	PathConvert (cPath,'/');
	DelBackSlash(cPath);

	for(iPos=0;iPos<m_nSize-1;iPos++)							// update the MRU list, if an existing MRU string matches file name
		{
		if(!strpicmp(m_arrNames[iPos],cPath))
			{
			break;												// iPos will point to matching entry
			}
		}

	for(;iPos>0;iPos--)											// move MRU strings before this one down
		{
		ASSERT(iPos>0);
		ASSERT(iPos<m_nSize);

		m_arrNames[iPos] = m_arrNames[iPos-1];
		}

	m_arrNames[0] = cPath;										// place this one at the beginning
}



//*****************************************************************************
//*
//*		Add
//*
//*****************************************************************************
// 	URL in die Liste einfgen
// 	pPathName		: Ist der Zeiger auf die URL (z.B.: "file://D:/test.txt")
// 	pProtocol		: Protokoll das vorgesetzt wird wenn nichts vorhanden ist.
void CRecentFileListUrl::Add(LPCTSTR pPathName,LPCTSTR pProtocol)
{
TCHAR		cPath[MAX_PATH];
int			iPos;
int			iOff;


	ASSERT(pPathName !=NULL);
	ASSERT(m_arrNames!=NULL);
	ASSERT(AfxIsValidString(pPathName));

	iOff = strcwlen(pPathName,0x00700000);


	if(pPathName[iOff]!=':' || !strwchar(pPathName[iOff+1],0x00300000)	|| !strwchar(pPathName[iOff+2],0x00300000))
		{
		iOff		= strlcpy(cPath,pProtocol,16);
		cPath[iOff] = ':'; iOff++;
		cPath[iOff] = '/'; iOff++;
		cPath[iOff] = '/'; iOff++;
		strlcpy(cPath+iOff,pProtocol,MAX_PATH-iOff);
		}
	else{
		strlcpy(cPath,pPathName,MAX_PATH);
		}


	PathConvert (cPath,'/');
	DelBackSlash(cPath);


	for(iPos=0;iPos<m_nSize-1;iPos++)							// update the MRU list, if an existing MRU string matches file name
		{
		if(strpicmp(m_arrNames[iPos],cPath))
			{
			break;												// iPos will point to matching entry
			}
		}

	for(;iPos>0;iPos--)											// move MRU strings before this one down
		{
		ASSERT(iPos>0);
		ASSERT(iPos<m_nSize);

		m_arrNames[iPos] = m_arrNames[iPos-1];
		}

	m_arrNames[0] = cPath;										// place this one at the beginning
}


//*****************************************************************************
//*
//*		UpdateMenu
//*
//*****************************************************************************
void CRecentFileListUrl::UpdateMenu(CCmdUI *pCmdUI)
{
TCHAR		cPath[MAX_PATH];
CMenu 	   *pMenu;
LPCTSTR		pName;
int			iPos;
int			iLen;



	ASSERT(m_arrNames!=NULL);

	pMenu = pCmdUI->m_pMenu;

	if(m_strOriginal.IsEmpty() && pMenu != NULL)
		{
		pMenu->GetMenuString(pCmdUI->m_nID, m_strOriginal, MF_BYCOMMAND);
		}

	if(m_arrNames[0].IsEmpty())
		{
		if(!m_strOriginal.IsEmpty())							// no MRU files
			{
			pCmdUI->SetText(m_strOriginal);
			}

		pCmdUI->Enable(FALSE);
		
		return;
		}

	if(pCmdUI->m_pMenu==NULL)
		{
		return;
		}

	for(iPos=0;iPos<m_nSize;iPos++)
		{
		pCmdUI->m_pMenu->DeleteMenu(pCmdUI->m_nID + iPos,MF_BYCOMMAND);
		}

	for(iPos=0;iPos<m_nSize;iPos++)
		{
		pName	= m_arrNames[iPos];
		if(!*pName)break;
		iLen    = wsprintf(cPath,_T("&%d "),(iPos+1+m_nStart)%10);
		iLen    = CommpressFileName(pName,cPath+iLen,MAX_PATH-iLen,60);
		pCmdUI->m_pMenu->InsertMenu(pCmdUI->m_nIndex++,MF_STRING|MF_BYPOSITION,pCmdUI->m_nID++,cPath);
		}

	
	pCmdUI->m_nIndex        -= 1; 								// point to last menu added
	pCmdUI->m_nIndexMax      = pCmdUI->m_pMenu->GetMenuItemCount();
	pCmdUI->m_bEnableChanged = TRUE;							// all the added items are enabled


}
