//*****************************************************************************
//*
//*
//*		ScrollList.cpp
//*
//*
//*****************************************************************************
//
//	Erzeugt eine ListBox mit Scrollbar, die per Maus vergrerbar ist.
//
#include	"ScrollList.h"
#include	"WinExtension.h"

#include	<tchar.h>

#define 	ID_SUBSCROLL	0x00001248
#define 	ID_SUBLIST		0x00001249

#if !defined WINVER || WINVER<0x0500
#define 	HMONITOR__	void*

typedef struct
	{
    DWORD   cbSize;
    RECT    rcMonitor;
    RECT    rcWork;
    DWORD   dwFlags;
	}MONITORINFO__;

extern "C" WINUSERAPI BOOL       WINAPI GetMonitorInfoA(HMONITOR__ hMonitor,MONITORINFO__ *pInfo);
extern "C" WINUSERAPI HMONITOR__ WINAPI MonitorFromRect(LPCRECT    pRect   ,DWORD          dwFlags);

#else
#define 	HMONITOR__		HMONITOR
#define 	MONITORINFO__	MONITORINFO
#endif


typedef struct
	{
	WNDPROC	pProc;									// Fensterfunktion vom Static-WND
	HWND	hList;									// Handle der Listbox
	HWND	hScoll;									// Handle der Scrollbar
	WORD	wWndId;									// Fenster ID
	INT		iTopPos;								// Erstes sichtbares Element
	INT		iHeight;								// Hhe des Fensters
	INT		iWidth;									// Breite des Fensters
	INT		iStep;									// Gre der Greifflche
	INT		iPage;									// Element pro Seite
	INT		iSizeX;									// Gre beim anfassen
	INT		iSizeY;									//			*
	INT		iGrabX;									// Wo wurde das Fenster angefasst
	INT		iGrabY;									//			*
	bool 	bGranOn;								// Vergrern aktiviert
	}ScrollListData;


//*****************************************************************************
//*
//*		GetDisplayRect
//*
//*****************************************************************************
//	Holt die Gre des Monitors bei einem bestimmten Rechteck
static void GetDisplayRect(LPCRECT pInRect,LPRECT pOutRect)
{
HMONITOR__		hMonitor;
MONITORINFO__	sInfo;


	   hMonitor	= MonitorFromRect(pInRect,0);
	if(hMonitor	&& GetMonitorInfoA(hMonitor,&sInfo))
		{
		*pOutRect			= sInfo.rcMonitor;
		}
	else{
		pOutRect->top		= 0;
		pOutRect->left		= 0;
		pOutRect->right		= GetSystemMetrics(SM_CXSCREEN);
		pOutRect->bottom	= GetSystemMetrics(SM_CYSCREEN);
		}

}

//*****************************************************************************
//*
//*		ScrollUpdate
//*
//*****************************************************************************
static void ScrollUpdate(ScrollListData *pData)
{
SCROLLINFO	sInfo;
int			iSizeY;


	iSizeY			= SendMessage(pData->hList,LB_GETITEMHEIGHT,0,0); if(iSizeY<=4)iSizeY=4;
	pData->iPage	= pData->iHeight/iSizeY;
	sInfo.cbSize	= sizeof(sInfo);
	sInfo.fMask		= SIF_PAGE|SIF_RANGE|SIF_POS;
	sInfo.nPage		= pData->iPage;
	sInfo.nPos		= SendMessage(pData->hList,LB_GETTOPINDEX,0,0);
	sInfo.nMax		= SendMessage(pData->hList,LB_GETCOUNT   ,0,0);
	sInfo.nMin		= 0;

	SetScrollInfo(pData->hScoll,SB_CTL,&sInfo,TRUE);

}

//*****************************************************************************
//*
//*		SubListProc
//*
//*****************************************************************************
static LRESULT CALLBACK SubListProc(HWND hWnd,UINT uMessage,WPARAM wParam,LPARAM lParam)
{
WNDPROC		pProc;
INT			iItem;


	pProc = (WNDPROC)GetWindowLongPtr(hWnd,GWL_USERDATA);

	if(uMessage==WM_MOUSEWHEEL)
		{
		return SendMessage(GetParent(hWnd),uMessage,wParam,lParam);
		}

	if(uMessage==WM_MOUSEMOVE)
		{
		   iItem = CallWindowProc(pProc,hWnd,LB_ITEMFROMPOINT,0,lParam);
		if(iItem>=0)
			{
			CallWindowProc(pProc,hWnd,LB_SETCURSEL,iItem,0);
			}
		}


return CallWindowProc(pProc,hWnd,uMessage,wParam,lParam);
}

//*****************************************************************************
//*
//*		ScrollListProc
//*
//*****************************************************************************
static LRESULT CALLBACK ScrollListProc(HWND hWnd,UINT uMessage,WPARAM wParam,LPARAM lParam)
{
ScrollListData *pData;
PAINTSTRUCT		sPaint;
LRESULT			lReturn;
int				iSizeX;
int				iSizeY;
UINT			uPlace;
UINT			uMode;
SCROLLINFO		sInfo;
RECT			sRect;
BOOL			bFlag;
HDC				hDc;



		pData = (ScrollListData*)GetWindowLongPtr(hWnd,GWL_USERDATA);
	if(!pData)return 0;

	switch(uMessage)
		{
	case WM_DESTROY:

		if(pData->bGranOn)
			{
			pData->bGranOn = false;
			ReleaseCapture();
			}

		SetWindowLongPtr(hWnd,GWL_WNDPROC ,(LPARAM)pData->pProc);
		SetWindowLongPtr(hWnd,GWL_USERDATA,0);
		delete pData;

	case WM_PAINT:

		hDc = (HDC)wParam;

		if(hDc)
			{
			bFlag = 0;
			}
		else{
			bFlag = 1;
			hDc = BeginPaint(hWnd,&sPaint);
			}

		sRect.left		= pData->iWidth -pData->iStep;
		sRect.top		= pData->iHeight-pData->iStep;
		sRect.right		= pData->iWidth;
		sRect.bottom	= pData->iHeight;

		DrawFrameControl(hDc,&sRect,DFC_SCROLL,DFCS_SCROLLSIZEGRIP|DFCS_MONO);

		if(bFlag)
			{
			EndPaint(hWnd,&sPaint);
			}

		return 0;

	case WM_SIZE:

		pData->iWidth	= LOWORD(lParam);
		pData->iHeight	= HIWORD(lParam);

		SetWindowPos(pData->hList ,NULL,0,0,pData->iWidth-pData->iStep+1,pData->iHeight,SWP_NOZORDER|SWP_NOMOVE);
		SetWindowPos(pData->hScoll,NULL,pData->iWidth-pData->iStep+1,0,pData->iStep,pData->iHeight-pData->iStep+1,SWP_NOZORDER);
		ScrollUpdate(pData);

		break;

	case WM_LBUTTONUP:

		if(pData->bGranOn)
			{
			pData->bGranOn = false;
			ReleaseCapture();
			RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE|RDW_UPDATENOW);
			}

		return 0;

	case WM_LBUTTONDOWN:

		pData->iGrabX	= GET_X_LPARAM(lParam);
		pData->iGrabY	= GET_Y_LPARAM(lParam);
		pData->iSizeX	= pData->iWidth;
		pData->iSizeY	= pData->iHeight;

		if(pData->iGrabX>pData->iWidth -pData->iStep)
		if(pData->iGrabY>pData->iHeight-pData->iStep)
			{
			pData->bGranOn = true;
			SetCapture(hWnd);
			}

		return 0;

	case WM_MOUSEWHEEL:

			iSizeY	= ((short) HIWORD(wParam))/120;
		if(!iSizeY)return 0;

		sInfo.nPos = pData->iTopPos-iSizeY;

		SetScrollPos(pData->hScoll,SB_CTL,sInfo.nPos,TRUE);
		SendMessage(pData->hList,LB_SETTOPINDEX,sInfo.nPos,0);
		pData->iTopPos = SendMessage(pData->hList,LB_GETTOPINDEX,0,0);

		break;

	case WM_MOUSEMOVE:

		if(!pData->bGranOn)return 0;
		if(!(wParam&MK_LBUTTON))
			{
			pData->bGranOn = false;
			ReleaseCapture();
			return 0;
			}

		iSizeX	= pData->iSizeX-pData->iGrabX+GET_X_LPARAM(lParam);
		iSizeY	= pData->iSizeY-pData->iGrabY+GET_Y_LPARAM(lParam);

		if(iSizeX<pData->iStep*2)iSizeX=pData->iStep*2;
		if(iSizeY<pData->iStep*2)iSizeY=pData->iStep*2;

		SetWindowPos(hWnd,NULL,0,0,iSizeX,iSizeY,SWP_NOZORDER|SWP_NOMOVE);
		RedrawWindow(hWnd,NULL,NULL,RDW_INVALIDATE);

		return 0;

	case WM_VSCROLL:

		if(lParam)
			{
			sInfo.cbSize	= sizeof(sInfo);
			sInfo.fMask		= SIF_RANGE|SIF_PAGE|SIF_POS;
			GetScrollInfo(pData->hScoll,SB_CTL,&sInfo);
			uMode			= LOWORD(wParam);
			uPlace			= HIWORD(wParam);

			switch(uMode)
				{
			case SB_THUMBPOSITION:
			case SB_THUMBTRACK:

				sInfo.nPos = uPlace;
				break;

			case SB_LINEDOWN:

				sInfo.nPos += 1;
				break;

			case SB_LINEUP:

				sInfo.nPos -= 1;
				break;

			case SB_PAGEDOWN:

				sInfo.nPos += sInfo.nPage;
				break;

			case SB_PAGEUP:

				sInfo.nPos -= sInfo.nPage;
				break;

			case SB_TOP:

				sInfo.nPos = 0;
				break;
				}

			SetScrollPos(pData->hScoll,SB_CTL,sInfo.nPos,TRUE);
			SendMessage(pData->hList,LB_SETTOPINDEX,sInfo.nPos,0);
			pData->iTopPos = SendMessage(pData->hList,LB_GETTOPINDEX,0,0);

			return 0;
			}
		break;

	case WM_ACTIVATE:

		return 0;

	case WM_COMMAND:

		if(LOWORD(wParam)==ID_SUBLIST)
			{
			if(HIWORD(wParam)==LBN_SELCHANGE)
				{
				uPlace = SendMessage(pData->hList,LB_GETTOPINDEX,0,0);

				if(uPlace^pData->iTopPos)
					{
					pData->iTopPos = uPlace;
					SetScrollPos(pData->hScoll,SB_CTL,pData->iTopPos,TRUE);
					}
				}

			wParam &= ~0xFFFF;
			wParam |= pData->wWndId;

			return SendMessage(GetParent(hWnd),uMessage,wParam,lParam);
			}

		break;
		}


	if(uMessage>=LB_ADDBUFFER && uMessage<LB_SETCOUNT)
		{
		bFlag = FALSE;

		switch(uMessage)
			{
		case LB_ADDBUFFER:
		case LB_INSERTBUFFER:	uMessage+=2;
		case LB_ADDSTRING:
		case LB_INSERTSTRING:
		case LB_DELETESTRING:
		case LB_RESETCONTENT:
		case LB_SETCURSEL:

			bFlag = TRUE;
			break;
			}

		lReturn = SendMessage(pData->hList,uMessage,wParam&~0x7FFF0000,lParam);

		if(bFlag)
			{
			if(lParam && (wParam&0x7FFF0000)==LB_FREEDATA)
				{
				delete (char*)lParam;
				}

			ScrollUpdate(pData);
			}

		return lReturn;
		}



return CallWindowProc(pData->pProc,hWnd,uMessage,wParam,lParam);
}

//*****************************************************************************
//*
//*		CreateScrollListEx
//*
//*****************************************************************************
//	Erzeugt eine ListBox mit Scrollbar, die per Maus vergrerbar ist.
//	dwExStyle		: Erweiterter Style fr das Fenster
//	dwStyle			: Style fr das Fenster	(WS_CHILD darf nicht gesetzt sein)
//	pRect			: Position und Gre des Fensters (globale Koordinaten, oder NULL)
//	hParent			: Handel des Elternfensters
//	uId				: Ist die ID des Fensters
//	Ergibt das Fenster-Handle oder 0 bei einem Fehler
HWND WINAPI CreateScrollListEx(DWORD dwExStyle,DWORD dwStyle,LPCRECT pRect,HWND hParent,UINT uId)
{
ScrollListData *pData;
RECT			sRect;
HWND			hList;
HWND			hMain;
HWND			hScoll;
INT				iHeight;
INT				iWidth;
INT				iStep;



		pData = new ScrollListData;
	if(!pData)return NULL;

	if(!pRect)											// Keine Gre
		{
		sRect.top		= 0;
		sRect.left		= 0;
		sRect.right		= 0;
		sRect.bottom	= 0;
		pRect			= &sRect;
		}
	else{
		GetDisplayRect(pRect,&sRect);
														// Y-Gre prfen
		if(sRect.top>pRect->top || sRect.bottom<pRect->bottom)
			{
			sRect.left	= pRect->left;
			sRect.right	= pRect->right;

			if(sRect.top   <pRect->top)sRect.top   =pRect->top;
			if(sRect.bottom>pRect->top)sRect.bottom=pRect->bottom;
			}
		}

	iWidth	= pRect->right-pRect->left;
	iHeight	= pRect->bottom-pRect->top;
														// Popup Fenster erstellen
		hMain = CreateWindowEx(dwExStyle,_T("STATIC"),_T(""),dwStyle|WS_POPUP|WS_CLIPCHILDREN|SS_NOTIFY|SS_BLACKRECT,pRect->left,pRect->top,iWidth,iHeight,hParent,NULL,NULL,NULL);
	if(!hMain)return NULL;


	GetClientRect(hMain,&sRect);
	iWidth	= sRect.right -sRect.left;
	iHeight	= sRect.bottom-sRect.top;
	iStep	= GetSystemMetrics(SM_CXVSCROLL);

														// Listbox erstellen
		hList = CreateWindowEx(0,_T("LISTBOX"),_T(""),WS_VISIBLE|WS_CHILD|LBS_SORT|LBS_NOTIFY|LBS_NOINTEGRALHEIGHT,0,0,iWidth-iStep+1,iHeight,hMain,(HMENU)ID_SUBLIST,NULL,NULL);
	if(!hList)
		{
		DestroyWindow(hMain);
		return NULL;
		}
														// Scrollbar erstellen
		hScoll = CreateWindowEx(0,_T("SCROLLBAR"),_T(""),WS_VISIBLE|WS_CHILD|SBS_VERT,iWidth-iStep+1,0,iStep,iHeight-iStep+1,hMain,(HMENU)ID_SUBSCROLL,NULL,NULL);
	if(!hScoll)
		{
		DestroyWindow(hList);
		DestroyWindow(hMain);
		return NULL;
		}


	SendMessage(hList,WM_SETFONT,(WPARAM)GetDefaultFont(),0);


	pData->pProc	= (WNDPROC)GetWindowLongPtr(hMain,GWL_WNDPROC);	// Fensterfunktion vom Static-WND
	pData->hList	= hList;								// Handle der Listbox
	pData->hScoll	= hScoll;								// Handle der Scrollbar
	pData->iHeight	= iHeight;								// Hhe des Fensters
	pData->iWidth	= iWidth;								// Breite des Fensters
	pData->iStep	= iStep;								// Gre der Greifflche
	pData->bGranOn	= false;								// Vergrern aktiviert
	pData->wWndId	= uId;									// Fenster ID
	pData->iPage	= 1;									// Elemente pro Seite
	pData->iTopPos	= 0;									// Erstes sichtbares Element

	SetClassLongPtr (hMain,GCLP_HBRBACKGROUND,0);
	SetClassLongPtr (hMain,GCLP_HCURSOR,(LPARAM)LoadCursor(NULL,IDC_SIZENWSE));
	SetWindowLongPtr(hMain,GWL_USERDATA,(LPARAM)pData);
	SetWindowLongPtr(hMain,GWL_WNDPROC ,(LPARAM)ScrollListProc);
	SetWindowLongPtr(hList,GWL_USERDATA,GetWindowLongPtr(hList,GWL_WNDPROC));
	SetWindowLongPtr(hList,GWL_WNDPROC ,(LPARAM)SubListProc);

	ScrollUpdate(pData);


return hMain;
}
