//*****************************************************************************
//*
//*
//*		ToolBarNew.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2017
//
//	This code creates a Toolbar for MFC.
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#ifdef	   _MSC_VER
#pragma		warning( disable :4996 )
#endif

#include	<Afx.h>
#include	<tchar.h>
#include	<malloc.h>
#include	<AfxPriv.h>
#include	"ToolBarNew.h"
#include	"WinExtension.h"


#ifdef		_DEBUG
#define new  DEBUG_NEW
#undef		 THIS_FILE
static char  THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CToolBarNew,CToolBar)

BEGIN_MESSAGE_MAP(CToolBarNew,CToolBar)
	//{{AFX_MSG_MAP(CToolBarNew)
	ON_WM_TIMER()
	ON_WM_CONTEXTMENU()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


#define		ID_UPDATE			0x197F
#define 	TOOLBAR_TO_EX(p)	((CToolBarDataEx*)(((LPBYTE)(p))-offsetof(CToolBarDataEx,wVersion)))

BOOL		  CToolBarNew::m_bInitDone=0;
const LPCTSTR CToolBarNew::m_aDefaultMenuNames[CNB_MENU_SIZE] =
	{
	_T("Kleine Icons"),							// Standart Texte fr's Context-Men
	_T("Mittler Icons"),
	_T("Groe Icons"),
	_T("Flache Symbolleiste"),
	_T("ausblenden"),
	};



//*****************************************************************************
//*
//*		Klasse CToolCmdNewUI
//*
//*****************************************************************************
class CToolCmdNewUI : public CCmdUI
{
public:

	virtual void Enable  (BOOL    bOn);
	virtual void SetCheck(int     iCheck);
	virtual void SetText (LPCTSTR pText);
};


//*****************************************************************************
//*
//*		Enable
//*
//*****************************************************************************
void CToolCmdNewUI::Enable(BOOL bOn)
{
CToolBarNew *pToolBar;


	m_bEnableChanged	= TRUE;
	pToolBar			= (CToolBarNew*)m_pOther;

	ASSERT(pToolBar!=NULL);
	ASSERT(m_nIndex<m_nIndexMax);
	ASSERT_KINDOF(CToolBarNew,pToolBar);


	if(bOn)pToolBar->SetButtonStyle(m_nIndex,0,TBBS_DISABLED);
	else   pToolBar->SetButtonStyle(m_nIndex,TBBS_DISABLED,TBBS_DISABLED|TBBS_PRESSED);


}

//*****************************************************************************
//*
//*		SetCheck
//*
//*****************************************************************************
void CToolCmdNewUI::SetCheck(int iCheck)
{
CToolBarNew *pToolBar;
UINT		uNewStyle;


	pToolBar			= (CToolBarNew*)m_pOther;

	ASSERT(pToolBar!=NULL);
	ASSERT(m_nIndex<m_nIndexMax);
	ASSERT(iCheck>=0 && iCheck<=2);
	ASSERT_KINDOF(CToolBarNew, pToolBar);

	uNewStyle = pToolBar->GetButtonStyle(m_nIndex) & ~(TBBS_CHECKED|TBBS_INDETERMINATE);

	if(iCheck==1)
		{
		uNewStyle |= TBBS_CHECKED;
		}
	else if(iCheck==2)
		{
		uNewStyle |= TBBS_INDETERMINATE;
		}

	ASSERT(!(uNewStyle&TBBS_SEPARATOR));

	pToolBar->SetButtonStyle(m_nIndex,uNewStyle|TBBS_CHECKBOX);

}

//*****************************************************************************
//*
//*		SetText
//*
//*****************************************************************************
void CToolCmdNewUI::SetText(LPCTSTR pText)
{

}

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CToolBarNew::CToolBarNew()
{
int		iPos;

	m_uExtraStyle	= 0;					// Erweiterte Styles
	m_uIconSize		= 0;					// Gre der Icons fr Men (0=klein 1=mittel 2=gro)
	m_iMinSize		= 16;					// Minimale Breite vom ungedocken Fenster
	m_iWidth		= 0;					// Breite eines Standart-Icons


	for(iPos=0;iPos<CNB_MENU_SIZE;iPos++)
		{
		m_aMenuTxt[iPos] = NULL;
		m_aMenuIds[iPos] = ID_TOOLMENU_ID+iPos;
		}

}

//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
BOOL CToolBarNew::Create(CWnd* pParentWnd,DWORD dwStyle,UINT nID)
{

	return CreateEx(pParentWnd,0,dwStyle,CRect(m_cxLeftBorder,m_cyTopBorder,m_cxRightBorder,m_cyBottomBorder),nID);

}


//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Erzeugt eine neue Toolbar
//	pParent		: Ist das Eltern Fenster
//	dwCtrlStyle : Fr das Toolbar- Fenster
//					TBSTYLE_PACKED		0x0020	Vertikal stapeln ist erlaut
//					TBSTYLE_FLAT		0x0800	Flache Tasten
//					TBSTYLE_TOOLTIPS    0x0100	Tooltips anzeigen
//					CBES_MENU_SMALL		Contextmen fr kleine Gre vorhanden
//					CBES_MENU_MIDDLE	Contextmen fr mittlere Gre vorhanden
//					CBES_MENU_BIG		Contextmen fr groe Gre vorhanden
//					CBES_MENU_FLAT		Contextmen fr flache Toolbar vorhanden
//	dwStyle		: Windows Fenster-Stile
//					WS_VISIBLE			Toolbar ist sichtbar
//					CBRS_TOP			Toolbar an der Oberseite andocken
//					CBRS_BOTTOM			Toolbar an der Unterseite andocken
//					CBRS_LEFT			Toolbar an der linken Seite andocken
//					CBRS_RIGHT			Toolbar an der rechten Seite andocken
//					CBRS_GRIPPER		Einen Hategriff links anzeigen
//					CBRS_TOOLTIPS		Tooltibs in der Toobar anzeigen
//					CBRS_FLYBY 			Die Statusbar zeigt Information ber die Tasten an
//					CBRS_SIZE_DYNAMIC	Toolbar kann die Gre ndern (muss gesetzt sein)
//	sBorders	: Ist die Gre der Rahmen
//	nId         : Ist die ID des Fensters
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CToolBarNew::CreateEx(CWnd *pParent,DWORD dwCtrlStyle,DWORD dwStyle,CRect sBorders,UINT nID)
{
HINSTANCE		hInst;
CRect			sRect;
CWnd		   *pWnd;



	ASSERT_VALID(pParent);
	ASSERT(!((dwStyle&CBRS_SIZE_FIXED) && (dwStyle&CBRS_SIZE_DYNAMIC)));

	m_uExtraStyle	= 0xFFFF0000&dwCtrlStyle;
	dwCtrlStyle	   &= 0x0000FFFF;
	hInst			= ::GetModuleHandleA("COMCTL32.DLL");

	ASSERT(hInst != NULL);



	if(!m_bInitDone)											// Die Toolbar muss einmal initialisiert werden
		{
		CToolBar	sBar;
		m_bInitDone = FALSE;
		if(sBar.Create(this))sBar.DestroyWindow();
		}


	m_iMinSize = GetSystemMetrics(SM_CYICON);
	m_iMinSize = 28+((m_iMinSize-32)*64)/32;

	if(m_iMinSize<16)m_iMinSize=16;
	if(m_iMinSize>48)m_iMinSize=48;

	ToolbarExRegister(AfxGetInstanceHandle());


	SetBorders(sBorders);

	m_dwStyle = (dwStyle&CBRS_ALL);								// save the style

	if(nID==AFX_IDW_TOOLBAR)
		{
		m_dwStyle |= CBRS_HIDE_INPLACE;
		}

	dwStyle &= ~CBRS_ALL;
	dwStyle |= CCS_NOPARENTALIGN|CCS_NOMOVEY|CCS_NODIVIDER|CCS_NORESIZE;
	dwStyle |= dwCtrlStyle;


	sRect.SetRectEmpty();

																// create the HWND
	if(!CWnd::Create(_T("ToolbarExWindow32"),NULL,dwStyle|WS_CHILD|WS_HSCROLL|WS_VSCROLL,sRect,pParent,nID))
		{
		return FALSE;
		}

	   pWnd = AfxGetMainWnd();
	if(pWnd)ToolBarEx_SetParent(m_hWnd,1,pWnd->m_hWnd);
	ToolBarEx_SetParent(m_hWnd,2,TB_SENDCB);

	SetSizes(m_sizeButton, m_sizeImage);


return TRUE;
}


//*****************************************************************************
//*
//*		CalcFixedLayout
//*
//*****************************************************************************
//	Berechnet die Gre einer angeddockten Toolbar.
//	bStretch	: Die Breite auf die ganze Lnge strecken.
//	bHorz		: Horizontal ausgereichet
//	Ergibt die Gre des Fensters.
CSize CToolBarNew::CalcFixedLayout(BOOL bStretch,BOOL bHorz)
{
CRect		sRect;
CSize		sSize;
LRESULT		lRet;



	sRect.SetRectEmpty();
	CalcInsideRect(sRect,bHorz);

	lRet		= ToolBarEx_CalcFrame(m_hWnd,(bHorz)? 17000:0,0x00);
	sSize.cx	= (lRet&0xFFFF)-sRect.Width ();
	sSize.cy	= (lRet >> 16 )-sRect.Height();



return sSize;
}

//*****************************************************************************
//*
//*		CalcDynamicLayout
//*
//*****************************************************************************
//	Berechnet die Gre der Toolbar, sie kann muss aber nicht geddockt sein.
//	nLength		: Ist die Fenster Breite die im ungedocken Modus per
//				  Maus ausgewhlt wird. Mit LM_LENGTHY in nMode wird
//				  hier die Fenster Hhe bergeben.
//	nMode		: Diverse Flags fr die Berechnung
//					LM_STRETCH  0x01  Auf ganze Breite oder Hhe stecken
//					LM_HORZ     0x02  Horizonatl oder Vertical ausgreichtet
//					LM_MRUWIDTH 0x04  Benutze die letzte Breite m_nMRUWidth
//					LM_HORZDOCK	0x08  Toolbar ist horizontal angedockt
//					LM_VERTDOCK	0x10  Toolbar ist vertikal angedockt
//					LM_LENGTHY  0x20  nLength ist die Y-Ausrichtung (nur float)
//					LM_COMMIT   0x40  Neue Breite setzen;
//	Ergibt die Gre des Fensters.
CSize CToolBarNew::CalcDynamicLayout(int nLength,DWORD nMode)
{
BOOL		bHorz;
CRect		sRect;
CSize		sSize;
LRESULT		lRet;


	if(nMode&(LM_HORZDOCK|LM_VERTDOCK))						// Toolbar angedockt
		{
		bHorz		= nMode&LM_HORZDOCK;

		sRect.SetRectEmpty();
		CalcInsideRect(sRect,bHorz);

		lRet		= ToolBarEx_CalcFrame(m_hWnd,(bHorz)? 17000:0,(nMode&LM_COMMIT)? 0x01:0x00);
		sSize.cx	= (lRet&0xFFFF)-sRect.Width ();
		sSize.cy	= (lRet >> 16 )-sRect.Height();
		}
	else if(nMode&LM_LENGTHY)
		{													// Vertical vergrern
		sRect.SetRectEmpty();
		CalcInsideRect(sRect,TRUE);

		nLength	   += sRect.Height();
		lRet		= ToolBarEx_CalcFrame(m_hWnd,nLength,(nMode&LM_COMMIT)? 0x03:0x02);
		sSize.cx	= (lRet&0xFFFF)-sRect.Width ();
		sSize.cy	= (lRet >> 16 )-sRect.Height();

		if(sSize.cx<m_iMinSize)sSize.cx=m_iMinSize;

		if(nMode&LM_COMMIT)m_nMRUWidth=sSize.cx;
		}
	else{													// Horizontal vergrern
		sRect.SetRectEmpty();
		CalcInsideRect(sRect,TRUE);

		if(nMode&LM_MRUWIDTH)nLength=m_nMRUWidth;

		nLength	   += sRect.Width();
		lRet		= ToolBarEx_CalcFrame(m_hWnd,nLength,(nMode&LM_COMMIT)? 0x01:0x00);
		sSize.cx	= (lRet&0xFFFF)-sRect.Width ();
		sSize.cy	= (lRet >> 16 )-sRect.Height();

		if(sSize.cx<m_iMinSize)sSize.cx=m_iMinSize;

		if(nMode&LM_COMMIT)m_nMRUWidth=sSize.cx;
		}


return sSize;
}


//*****************************************************************************
//*
//*		SetBitmap
//*
//*****************************************************************************
//	Setzt das ein Bild als neue Icons
//	hBitmap		: Ist das Handle des Bildes mit den Icons
//	iWidth		: Ist die Breite eines einzlenen Icons
//					0			= aktuelle Breite benutzen
//					TB_VARSIZE	= fr Bitmap mit variablen Gren
//	bDelete		: Das Bitmap am Ende lschen
//	pItemList	: Liste mit den ID's wenn diese noch nicht vorhanden sind
//	iCount		: Anzahl der ID's in der Liste
//	uScale		: Neue Skalierung fr das Bitmap (0=alte beibehalten)
//	Ergibt TRUE wenn erfolgreich
BOOL CToolBarNew::SetBitmap(HBITMAP hBitmap,int iWidth,BOOL bDelete,UINT *pItemList,int iCount,UINT uScale)
{
LPCTSTR			aData[2];
TBREPLACEBITMAP	sReplace;
TBBITMAPINFO	sCurrent;
BITMAP			sInfo;
CToolBarDataEx *pData;
BYTE			bType;
INT				iPos;
INT				iMax;
BOOL			bOk;



	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	if(!hBitmap)return FALSE;


	if(iWidth<=0)								// Aktuelle Breite benutzen
		{
			iWidth = m_iWidth;
		if(!iWidth)
			{
			if(bDelete)DeleteObject(hBitmap);
			return FALSE;
			}
		}


	if(iWidth==TB_VARSIZE)						// Anzahl der Bilder bestimmen
		{
		sReplace.nButtons = TB_VARSIZE;
		iMax = ToolBarEx_CalcPicCount(m_hWnd,TB_ISHBITMAP,(LPARAM)hBitmap);

		if(iMax<=0)
			{
			if(bDelete)DeleteObject(hBitmap);
			return FALSE;
			}
		}
	else{										// Bilder haben fixe Breite
		GetObject(hBitmap,sizeof(sInfo),&sInfo);
		iMax = sInfo.bmWidth/iWidth;
		ASSERT(iMax>0 && sInfo.bmWidth%iWidth==0);
		sReplace.nButtons	= iMax;
		}


	if(!pItemList || !iCount)					// ID's aus dem aktuellen Fenster holen
		{
		if(ToolBarEx_GetBitmapInfo(m_hWnd,0,1,&sCurrent))
			{
			sReplace.hInstNew	= NULL;
			sReplace.nIDNew		= (UINT_PTR)hBitmap;
			sReplace.hInstOld	= sCurrent.hInstance;
			sReplace.nIDOld		= sCurrent.uBitmapId;

			if(ToolBarEx_ReplaceBitmap(m_hWnd,uScale,&sReplace))
				{
				UpdateLayout();
				return TRUE;
				}

			if(!uScale)uScale=sCurrent.wScale;
			}

		pItemList		= NULL;
		iCount			= CNB_CURRENTLIST;
		}


	if(!uScale)									// Sie Skalierung bernehmen
		{
		if(ToolBarEx_GetBitmapInfo(m_hWnd,0,1,&sCurrent))
				uScale  = sCurrent.wScale;
		else	uScale  = 256;
		}


	pData				= (CToolBarDataEx*)new BYTE[sizeof(CToolBarDataEx)+iCount*sizeof(WORD)];
	pData->uBitmapId	= (DWORD_PTR)hBitmap;
	pData->hInstance	= NULL;
	pData->wHeight		= (WORD)sInfo.bmHeight;
	pData->wFlags		= (bDelete)? 0x04:0x00;
	pData->wWidth		= iWidth;
	pData->wScale		= uScale;
	pData->wVersion		= 0x0001;
	pData->wItemCount	= 0;

	for(iPos=0;iPos<iCount;iPos++)				// Kopiere die Liste mit ID's
		{
		if(pItemList[iPos])
			{
			iMax--;
			if(iMax<0)break;					// Zuwenige Icon's im Bild
			}

		pData->aItems[iPos] = pItemList[iPos];
		pData->wItemCount  += 1;
		}

	aData[0]	= (LPCTSTR)pData;
	aData[1]	= NULL;
	bType		= 1;

		bOk = LoadToolBar(aData,FALSE,pItemList,iCount,&bType);
	if(!bOk)
		{
		if(bDelete)DeleteObject(hBitmap);
		}

	delete pData;


return bOk;
}


//*****************************************************************************
//*
//*		SetBitmap
//*
//*****************************************************************************
//	Setzt das ein Bild als neue Icons
//	uBitmapId	: Die ID der BITMAP-Resource
//	iWidth		: Ist die Breite eines einzlenen Icons
//					0			= aktuelle Breite benutzen
//					TB_VARSIZE	= fr Bitmap mit variablen Gren
//	hModule		: Instanz-Handle der Resource (0=Standart Resourcen)
//	pItemList	: Liste mit den ID's wenn diese noch nicht vorhanden sind
//	iCount		: Anzahl der ID's in der Liste
//	uScale		: Neue Skalierung fr das Bitmap (0=alte beibehalten)
//	Ergibt TRUE wenn erfolgreich
BOOL CToolBarNew::SetBitmap(DWORD_PTR uBitmapId,int iWidth,HMODULE hModule,UINT *pItemList,int iCount,UINT uScale)
{
LPCTSTR			aData[2];
TBREPLACEBITMAP	sReplace;
TBBITMAPINFO	sCurrent;
LPBYTE			pPixels;
LPBITMAPINFO	pInfo;
CToolBarDataEx *pData;
BYTE			bType;
INT				iPos;
INT				iMax;
BOOL			bOk;


	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	if(!uBitmapId)return FALSE;


	if(iWidth<=0)								// Aktuelle Breite benutzen
		{
			iWidth = m_iWidth;
		if(!iWidth)
			{
			return FALSE;
			}
		}

	if(!hModule)
		{
		hModule = AfxGetResourceHandle();
		}
												// Resource holen
		pInfo = GetDibOfResource(NULL,hModule,(LPCTSTR)uBitmapId,(LPCTSTR)RT_BITMAP,0,&pPixels);
	if(!pInfo)
		{
		return FALSE;
		}


	if(iWidth==TB_VARSIZE)						// Anzahl der Bilder bestimmen
		{
		sReplace.nButtons = TB_VARSIZE;
		iMax = ToolBarEx_CalcPicCount(m_hWnd,pInfo,pPixels);

		if(iMax<=0)
			{
			return FALSE;
			}
		}
	else{
		iMax = pInfo->bmiHeader.biWidth/iWidth;
		ASSERT(iMax>0 && pInfo->bmiHeader.biWidth%iWidth==0);
		sReplace.nButtons	= iMax;
		}


	if(!pItemList || !iCount)					// ID's aus dem aktuellen Fenster holen
		{
		if(ToolBarEx_GetBitmapInfo(m_hWnd,0,1,&sCurrent))
			{
			sReplace.hInstNew	= hModule;
			sReplace.nIDNew		= uBitmapId;
			sReplace.hInstOld	= sCurrent.hInstance;
			sReplace.nIDOld		= sCurrent.uBitmapId;

			if(ToolBarEx_ReplaceBitmap(m_hWnd,uScale,&sReplace))
				{
				UpdateLayout();
				return TRUE;
				}

			if(!uScale)uScale=sCurrent.wScale;
			}

		pItemList		= NULL;
		iCount			= CNB_CURRENTLIST;
		}


												// Sie Skalierung bernehmen
	if(!uScale)
		{
		if(ToolBarEx_GetBitmapInfo(m_hWnd,0,1,&sCurrent))
				uScale  = sCurrent.wScale;
		else	uScale  = 256;
		}


	pData				= (CToolBarDataEx*)new BYTE[sizeof(CToolBarDataEx)+iCount*sizeof(WORD)];
	pData->uBitmapId	= uBitmapId;
	pData->hInstance	= hModule;
	pData->wHeight		= (WORD)pInfo->bmiHeader.biHeight;
	pData->wWidth		= iWidth;
	pData->wScale		= uScale;
	pData->wFlags		= 0x0002;
	pData->wVersion		= 0x0001;
	pData->wItemCount	= 0;

	for(iPos=0;iPos<iCount;iPos++)				// Kopiere die Liste mit ID's
		{
		if(pItemList[iPos])
			{
			iMax--;
			if(iMax<0)break;					// Zuwenige Icon's im Bild
			}

		pData->aItems[iPos] = pItemList[iPos];
		pData->wItemCount  += 1;
		}

	aData[0]	= (LPCTSTR)pData;
	aData[1]	= NULL;
	bType		= 1;

	bOk = LoadToolBar(aData,FALSE,pItemList,iCount,&bType);
	delete pData;



return bOk;
}

//*****************************************************************************
//*
//*		LoadBitmap
//*
//*****************************************************************************
//	Setzt das ein Bild als neue Icons geladen aus einer Resource
//	pResourceName	: Ist der Name der Resource
//	iWidth			: Ist die Breite eines einzlenen Icons (0=aktuelle Breite)
//	Ergibt TRUE wenn erfolgreich
BOOL CToolBarNew::LoadBitmap(LPCTSTR pResourceName,int iWidth)
{
BOOL		bOk;


	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	bOk = SetBitmap((DWORD_PTR)pResourceName,iWidth);


return bOk;
}

//*****************************************************************************
//*
//*		LoadToolBar
//*
//*****************************************************************************
//	Ld eine Toolbar aus den Recourcen
//	pResources	: Zeiger auf die Recourenamen. (Mit 0 abgeschlossen)
//	bCopyItems	: Sollen die alten Eintrge bernommmen werden (Gren+Id's)
//	pItemList	: Zeiger auf die ID's der Tasten.
//				  Wenn die ID's 0 sind werden sie aus der ersten Recource genommen.
//				  Wenn der Zeiger ungleich Null ist werden die Tasten aus allen
//				  Recourcen zusammen gesucht.
//				  Ein NULL-Eintrag steht fr einen Separator.
//	iListCount	: Ist die Anzahl der ID-Eintrge in pItemList. Bei
//				  CNB_CURRENTLIST wird die Liste aus der Toolbar geholt.
//	pTypes		: Ist eine Liste welche Arten von Recourenamen bergeben werden.
//				  Wenn pTypes Null ist dann sind alle Eintrge ToolBar-Ressourcen.
//					0 =  ToolBar-Ressource
//					1 =  CToolBarDataEx
//	uScale		: Skalierung fr die Bitmaps der Toolbars, falls keine eigen
//				  Bitmap-Skalierung angegeben ist. (256=1:1)
//	Ergibt 1 wenn die Toolbar geladen wurde.
BOOL CToolBarNew::LoadToolBar(LPCTSTR *pResources,BOOL bCopyItems,UINT *pItemList,int iListCount,LPBYTE pTypes,UINT uScale)
{
CWnd		   *pWnd;
CToolBarDataEx *pData;
HGLOBAL			aGlobal[CNB_MAX_RSRC];
WORD			aCount [CNB_MAX_RSRC];
LPWORD			aItems [CNB_MAX_RSRC];
TBADDBITMAPEX	aAddBitmap;
LPCTSTR			pResName;
HGLOBAL			hGlobal;
TBBUTTONEX		sButton;
LPWORD			pItems;
UINT			uCount;
UINT			uPlace;
WORD			wItem;
CRect			sRect;
HRSRC			hRsrc;
HINSTANCE		hInst;
LPVOID			pPtr;
int				iRes;
int				iRet;
int				iPos;
int				iPic;
int				iOff;
BOOL			bOk;



	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));
	ASSERT(pResources[0]!=NULL);


	if(iListCount==CNB_CURRENTLIST)				// Direkt aus Toolbar
		{
		iListCount	= ButtonCount();
		pItemList	= (UINT*)alloca(iListCount*sizeof(UINT));
		iListCount	= ToolBarEx_GetIdList(m_hWnd,iListCount,pItemList);

		if(iListCount<0)
			{
			return FALSE;
			}
		}


	ToolBarEx_DelButtons(m_hWnd,0,0xFFFF);		// Alle Tasten lschen
	ToolBarEx_DelBitmaps(m_hWnd);				// Alle Bilder lschen


	bOk		= TRUE;
	hInst	= AfxGetResourceHandle();

	for(iRes=0;iRes<CNB_MAX_RSRC;iRes++)		// Resourcen ausuwerten
		{
			pResName = pResources[iRes];
		if(!pResName)break;

		if(!pTypes || !pTypes[iRes])			// ToolBar Ressource
			{
			   hRsrc  = FindResource(hInst,pResName,RT_TOOLBAR);
			if(hRsrc  ==NULL)return FALSE;
			   hGlobal= LoadResource(hInst,hRsrc);
			if(hGlobal==NULL)return FALSE;

				pPtr = LockResource(hGlobal);
			if(!pPtr)
				{
				FreeResource(hGlobal);
				return FALSE;
				}

			pData				= TOOLBAR_TO_EX(pPtr);
			uCount				= pData->wItemCount;
			pItems				= pData->aItems;
			aAddBitmap.wFlags	= 0x0003;
			aAddBitmap.nID		= (DWORD_PTR)pResName;
			aAddBitmap.wScale	= uScale;
			aAddBitmap.hInst	= hInst;
			}
		else{									// Direkter CToolBarDataEx Zeiger
			hGlobal				= 0;
			pData				= (CToolBarDataEx*)pResName;
			uCount				= pData->wItemCount;
			pItems				= pData->aItems;
			aAddBitmap.wFlags	= pData->wFlags|0x0002;
			aAddBitmap.wScale	= pData->wScale;
			aAddBitmap.nID		= pData->uBitmapId;
			aAddBitmap.hInst	= pData->hInstance;

			if(!aAddBitmap.wScale)
				{
				aAddBitmap.wScale = uScale;
				}
			}

		ASSERT(pData->wVersion==1);

		aAddBitmap.wWidth	= pData->wWidth;
		aAddBitmap.wFlags  |= 0x0010;

		iRet = ToolBarEx_AddBitmap(m_hWnd,TB_EXTENDED,&aAddBitmap);

		ASSERT(iRet>=0);

		aGlobal[iRes] = hGlobal;
		aItems [iRes] = pItems;
		aCount [iRes] = uCount;

		if(iRet<0)
			{
			bOk = FALSE;
			iRes++;
			break;
			}
												// Bei erster Bar die Breite initalisieren
		if((iRes==0 || !m_iWidth) && pData->wWidth!=TB_VARSIZE)
			{
			m_iWidth = pData->wWidth;
			}
		}

	if(bOk && pItemList && iListCount>0)		// Eintrge aus der Liste holen
		{
		memset(&sButton,0, sizeof(sButton));
		sButton.fsState = TBSTATE_ENABLED;
		sButton.iString = -1;

		for(iPos=0;iPos<iListCount;iPos++)
			{
				wItem = (WORD)pItemList[iPos];
			if(!wItem)							// Separator
				{
				sButton.fsStyle		= TBSTYLE_SEP;
				sButton.iBitmap		= -1;
				sButton.idCommand	=  0;
				}
			else{								// Taste mit Bild
				sButton.fsStyle		= TBSTYLE_BUTTON;
				sButton.idCommand	= wItem;
				sButton.iBitmap		= 0;

				for(iOff=0;iOff<iRes;iOff++)	// Suche Bild zur ID
					{
					uCount	= aCount[iOff];
					pItems	= aItems[iOff];

					for(uPlace=0;uPlace<uCount;uPlace++)
						{
						if(pItems[uPlace]==0    )continue;
						if(pItems[uPlace]==wItem)break;
						sButton.iBitmap++;
						}
					}
				}

			iRet = ToolBarEx_AddButtonsEx(m_hWnd,TB_ENDINDEX,1,&sButton);

			ASSERT(iRet!=0);
			}
		}
	else{										// Standart Liste der Resouren benutzen
		memset(&sButton,0,sizeof(sButton));
		sButton.fsState = TBSTATE_ENABLED;
		sButton.iString = -1;
		iPic			= 0;

		if(iRes>0)								// Die erste Toolbar einfgen
			{
			uCount	= aCount[0];
			pItems	= aItems[0];

			for(uPlace=0;uPlace<uCount;uPlace++)
				{
				wItem = pItems[uPlace];

				if(!wItem)						// Separator
					{
					sButton.fsStyle		= TBSTYLE_SEP;
					sButton.iBitmap		= -1;
					sButton.idCommand	=  0;
					}
				else{							// Taste mit Bild
					sButton.fsStyle		= TBSTYLE_BUTTON;
					sButton.idCommand	= wItem;
					sButton.iBitmap		= iPic;
					iPic			   += 1;
					}

				iRet = ToolBarEx_AddButtonsEx(m_hWnd,TB_ENDINDEX,1,&sButton);
				}

			ASSERT(iRet!=0);
			}
		}



	while(iRes>0)								// Resourcen freigeben
		{
		iRes--;

		if(aGlobal[iRes])
			{
			UnlockResource(aGlobal[iRes]);
			FreeResource  (aGlobal[iRes]);
			}
		}


	   pWnd = GetParent();
	if(pWnd->GetStyle()&WS_POPUP)				// Ist die Toolbar in einem Dialog
		{
		pWnd->GetClientRect(&sRect);
		CalcSize(TRUE,sRect.right);
		}
	else{
		GetClientRect(sRect);

		if(sRect.Width()<2 || sRect.Height()<2)	// Hat das Fenster noch keine Gre
			{
			CalcSize(TRUE);
			}
		}



return bOk;
}


//*****************************************************************************
//*
//*		GetControl
//*
//*****************************************************************************
//	Holt das Fenster-Handle eines Controls
//	uNumber		: Ist der Index des Controls
//	Ergibt einen Zeiger auf das Fenster oder NULL wenn nichts gefunden wurde.
CWnd *CToolBarNew::GetControl(unsigned uNumber) const
{
HWND	hWnd;


	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	if(uNumber>=128)return NULL;

	hWnd = (HWND)ToolBarEx_GetParam(m_hWnd,TB_USE_FLAG(BTFL_CONTROL,uNumber));

	if(!hWnd)return NULL;


return CWnd::FromHandle(hWnd);
}

//*****************************************************************************
//*
//*		GetControlId
//*
//*****************************************************************************
//	Sucht ein Control mit eine bestimmten ID.
//	uId			: Ist die ID des Contols
//	uPos		: Ist die Position des Controls bei gleichen ID's
//	Ergibt einen Zeiger auf das Contol, bzw. NULL wenn nichts gefunden wurde.
CWnd *CToolBarNew::GetControlId(unsigned uId,unsigned uPos) const
{
unsigned	uNumber;
HWND		hWnd;


	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	for(uNumber=0;uNumber<128;uNumber++)
		{
			hWnd = (HWND)ToolBarEx_GetParam(m_hWnd,TB_USE_FLAG(BTFL_CONTROL,uNumber));
		if(!hWnd)break;

		if(GetWindowLong(hWnd,GWL_ID)^uId)continue;

		if(!uPos)
			{
			return CWnd::FromHandle(hWnd);
			}

		uPos--;
		}


return NULL;
}

//*****************************************************************************
//*
//*		AddControl
//*
//*****************************************************************************
//	Fgt ein Control in die Toolbar ein.
//	Das Control muss als Elternfenster die Toolbar haben.
//	pWnd		: Ist das Control-Fenster.
//	uId			: Ist die ID in der Toolbar wo das Control eingefgt wird,
//				  oder es ist ein spezieller Toolbar Index:
//						TB_USE_ID(i)		i=Id
//						TB_USE_INDEX(i)		i=Index
//						TB_USE_NID(i,n)		i=Id       n=Position
//						TB_USE_FLAG(f,n)	f=BTFL_??? n=Position
//	iNumber		: Nummer der ID (-1 nchste freie). Dieser Wert wird
//				  bei speziellen Toolbar Inizes ignoriert.
//	bSetFont	: Den Font bei EDIT und COMBOBOX einstellen.
//	Ergibt die Control-Nummer oder -1 bei einem Fehler.
int CToolBarNew::AddControl(CWnd *pWnd,unsigned uId,int iNumber,BOOL bSetFont)
{
TCHAR	cName[32];
UINT	uIndex;
UINT	uLen;
BYTE	uPos;



	ASSERT_VALID(this);
	ASSERT(pWnd!=0);
	ASSERT(IsWindow(m_hWnd));
	ASSERT(pWnd->GetParent() && m_hWnd==pWnd->GetParent()->m_hWnd);


	if(TB_SPECIAL(uId))									// Spezieler Index
		{
		uIndex = ToolBarEx_CommandToIndex(m_hWnd,uId);
		}
	else if(iNumber>=0)									// Absoluter Index
		{
		uIndex = ToolBarEx_CommandToIndex(m_hWnd,TB_USE_NID(uId,iNumber));
		}
	else for(uPos=0;;uPos++)							// Suche die nchste freie ID
		{
		   uIndex = ToolBarEx_CommandToIndex(m_hWnd,TB_USE_NID(uId,uPos));
		if(uIndex==0xFFFFFFFF)
			{
			return -1;
			}
														// Control schon benutzt
		if(!ToolBarEx_GetParam(m_hWnd,TB_USE_NID(uId,uPos)))
			{
			break;
			}
		}

	if(uIndex>=0xFFFF)									// Prfe den Index
		{
		return -1;
		}

	if(bSetFont)										// Font einstellen
		{
		   uLen = GetClassName(pWnd->m_hWnd,cName,32);
		if(uLen==8 && !memicmp(cName,_T("ComboBox"),8*sizeof(TCHAR)))
			{
			::SendMessage(pWnd->m_hWnd,WM_SETFONT,(WPARAM)GetStockObject(ANSI_VAR_FONT),0);
			}
		else if(uLen==4 && !memicmp(cName,_T("Edit"),4*sizeof(TCHAR)))
			{
			::SendMessage(pWnd->m_hWnd,WM_SETFONT,(WPARAM)GetStockObject(ANSI_VAR_FONT),0);
			}
		}
														// Das Control einfgen
	ToolBarEx_SetFlags(m_hWnd,uIndex,BTFL_CONTROL,BTFL_CONTROL);
	ToolBarEx_SetParam(m_hWnd,uIndex,pWnd->m_hWnd);



return ToolBarEx_CountItems(m_hWnd,0,uIndex,BTFL_CONTROL,BTFL_CONTROL);
}

//*****************************************************************************
//*
//*		SetColors
//*
//*****************************************************************************
//	Stellt die Tauschfarben fr bestimmte ID's ein
//	iCount			: Anzahl der ID's mit Tauschfarben
//	pIds			: Liste mit den ID's der Tasten
//	pColors			: Sind die neuen Farben
//	uXchgColor		: Ist die Farbe in der Resource die getauscht werden soll
//					  (0xFFFFFFFF Farbe nicht ndern)
void CToolBarNew::SetColors(int iCount,const LPWORD pIds,COLORREF *pColors,COLORREF uXchgColor)
{
int		iPos;


	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	if(uXchgColor!=0xFFFFFFFF)
		{
		ToolBarEx_SetOption(m_hWnd,TBOP_CANGECOLOOR1,uXchgColor);
		}

	for(iPos=0;iPos<iCount;iPos++)
		{
		ASSERT((ToolBarEx_GetFlags(m_hWnd,TB_USE_ID(pIds[iPos]))!=0xFFFFFFFF));
		ASSERT((ToolBarEx_GetFlags(m_hWnd,TB_USE_ID(pIds[iPos]))&(BTFL_COLOR_DUAL|BTFL_COLOR_CHANGE))==BTFL_COLOR_CHANGE);
		ToolBarEx_SetParam(m_hWnd,TB_USE_ID(pIds[iPos]),pColors[iPos]);
		}

}

//*****************************************************************************
//*
//*		SetColors
//*
//*****************************************************************************
//	Stellt die Tauschfarben fr bestimmte Reihe von ID's ein.
//	iCount			: Anzahl der ID's mit Tauschfarben
//	pIds			: Ist die erste ID der Tasten.
//	pColors			: Sind die neuen Farben
//	uXchgColor		: Ist die Farbe in der Resource die getauscht werden soll
//					  (0xFFFFFFFF Farbe nicht ndern)
void CToolBarNew::SetColors(int iCount,int iId ,COLORREF *pColors,COLORREF uXchgColor)
{
int		iPos;


	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	if(uXchgColor!=0xFFFFFFFF)
		{
		ToolBarEx_SetOption(m_hWnd,TBOP_CANGECOLOOR1,uXchgColor);
		}

	for(iPos=0;iPos<iCount;iPos++,iId++)
		{
		ASSERT((ToolBarEx_GetFlags(m_hWnd,TB_USE_ID(iId))!=0xFFFFFFFF));
		ASSERT((ToolBarEx_GetFlags(m_hWnd,TB_USE_ID(iId))&(BTFL_COLOR_DUAL|BTFL_COLOR_CHANGE))==BTFL_COLOR_CHANGE);
		ToolBarEx_SetParam(m_hWnd,TB_USE_ID(iId),pColors[iPos]);
		}

}

//*****************************************************************************
//*
//*		SetColors
//*
//*****************************************************************************
//	Stellt die Tauschfarben fr bestimmte ID's ein
//	uItem			: Ist der Index oder die ID der Taste
//						TB_USE_ID(i)		i=Id
//						TB_USE_INDEX(i)		i=Index
//						TB_USE_NID(i,n)		i=Id       n=Position
//						TB_USE_FLAG(f,n)	f=BTFL_??? n=Position
//	uColor1			: Ist die erste Farbe
//	uColor2			: Ist die zweite Farbe (0xFFFFFFFF nur eine Farbe)
void CToolBarNew::SetColor(unsigned uItem,COLORREF uColor1,COLORREF uColor2)
{

	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	if(uColor2==0xFFFFFFFF)
		{
		ASSERT((ToolBarEx_GetFlags(m_hWnd,uItem)!=0xFFFFFFFF));
		ASSERT((ToolBarEx_GetFlags(m_hWnd,uItem)&(BTFL_COLOR_DUAL|BTFL_COLOR_CHANGE))==(BTFL_COLOR_DUAL|BTFL_COLOR_CHANGE));
		ToolBarEx_SetParam(m_hWnd,uItem,MAKELONG(TBC_RGB_TO_C16(uColor1),TBC_RGB_TO_C16(uColor2)));
		}
	else{
		ASSERT((ToolBarEx_GetFlags(m_hWnd,uItem)!=0xFFFFFFFF));
		ASSERT((ToolBarEx_GetFlags(m_hWnd,uItem)&(BTFL_COLOR_DUAL|BTFL_COLOR_CHANGE))==BTFL_COLOR_CHANGE);
		ToolBarEx_SetParam(m_hWnd,uItem,uColor1);
		}


}


//*****************************************************************************
//*
//*		SetColorXchg
//*
//*****************************************************************************
//	Stellt die Tauschfarben fr alle Tasten ein
//	uColor1			: Ist die erste Tauschfarbe	 (0xFFFFFFFF=nicht setzen)
//	uColor2			: Ist die zweite Tauschfarbe (0xFFFFFFFF=nicht setzen)
void CToolBarNew::SetColorXchg(COLORREF uColor1,COLORREF uColor2)
{

	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	if(uColor1!=0xFFFFFFFF)
		{
		ToolBarEx_SetOption(m_hWnd,TBOP_CANGECOLOOR1,uColor1);
		}

	if(uColor1!=0xFFFFFFFF)
		{
		ToolBarEx_SetOption(m_hWnd,TBOP_CANGECOLOOR2,uColor2);
		}

}


//*****************************************************************************
//*
//*		SetButtonInfo
//*
//*****************************************************************************
//	Stellt diverse Tasten-Attribute ein.
//	iIndex		: Ist der Index der Taste, Es knne auch folgende Makros benutzt werden:
//					TB_USE_ID(i)		Taste ber ID suchen
//					TB_USE_INDEX(i)		Taste ber Index suchen (optional)
//					TB_USE_NID(i,n)		Taste ber n'te ID suchen
//					TB_USE_FLAG(f,n)	Taste ber n'tes Flags suchen (BTFL_???)
//	uCommand	: Ist die neue ID fr die Taste (0xFFFFFFFF=nicht ndern)
//	uStyle		: Ist der neue Style/State (0xFFFFFFFF=nicht ndern)
//	iImage		: Ist die neue Bilde-ID	   (0xFFFFFFFE/-2=nicht ndern)
void CToolBarNew::SetButtonInfo(int iIndex,UINT uCommand,UINT uStyle,int iImage)
{

	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	if(uCommand!=0xFFFFFFFF)ToolBarEx_SetCommandId   (m_hWnd,iIndex,uCommand);
	if(uStyle  !=0xFFFFFFFF)ToolBarEx_SetStyleStateEx(m_hWnd,iIndex,uStyle  );
	if(iImage  >=        -1)ToolBarEx_SetBitmapId    (m_hWnd,iIndex,iImage  );


}

//*****************************************************************************
//*
//*		SetButtonInfo
//*
//*****************************************************************************
//	Stellt diverse Tasten-Attribute ein.
//	iIndex		: Ist der Index der Taste, Es knne auch folgende Makros benutzt werden:
//					TB_USE_ID(i)		Taste ber ID suchen
//					TB_USE_INDEX(i)		Taste ber Index suchen (optional)
//					TB_USE_NID(i,n)		Taste ber n'te ID suchen
//					TB_USE_FLAG(f,n)	Taste ber n'tes Flags suchen (BTFL_???)
//	uCommand	: Ist die neue ID fr die Taste (0xFFFFFFFF=nicht ndern)
//	uStyle		: Ist der neue Style/State (0xFFFFFFFF=nicht ndern)
//	iImage		: Ist die neue Bilde-ID	   (0xFFFFFFFE/-2=nicht ndern)
//	uFlags		: Sind die neuen Flags     (BTFL_???)
//	uMask		: Ist die Maske fr die Flags
void CToolBarNew::SetButtonInfo(int iIndex,UINT uCommand,UINT uStyle,int iImage,UINT uFlags,UINT uMask)
{

	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	if(uCommand!=0xFFFFFFFF)ToolBarEx_SetCommandId   (m_hWnd,iIndex,uCommand);
	if(uStyle  !=0xFFFFFFFF)ToolBarEx_SetStyleStateEx(m_hWnd,iIndex,uStyle^TBBS_DISABLED);
	if(iImage  >=        -1)ToolBarEx_SetBitmapId    (m_hWnd,iIndex,iImage  );
	if(uMask               )ToolBarEx_SetFlags       (m_hWnd,iIndex,uFlags,uMask);


}

//*****************************************************************************
//*
//*		GetButtonInfo
//*
//*****************************************************************************
//	Holt diverse Tasten-Attribute.
//	iIndex		: Ist der Index der Taste, Es knne auch folgende Makros benutzt werden:
//					TB_USE_ID(i)		Taste ber ID suchen
//					TB_USE_INDEX(i)		Taste ber Index suchen (optional)
//					TB_USE_NID(i,n)		Taste ber n'te ID suchen
//					TB_USE_FLAG(f,n)	Taste ber n'tes Flags suchen (BTFL_???)
//	uCommand	: Ist die ID fr die Taste
//	uStyle		: Ist der Style/State  (BTNS_???/TBBS_???)
//	iImage		: Ist die Bilde-ID	   (-1 keines)
void CToolBarNew::GetButtonInfo(int iIndex,UINT &uCommand,UINT &uStyle,int &iImage) const
{
TBBUTTONINFOEX	sButton;


	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	sButton.cbSize	= sizeof(sButton);
	sButton.dwMask	= TBIF_COMMAND|TBIF_STYLE|TBIF_STATE|TBIF_IMAGE;

	ToolBarEx_GetButtonInfo(m_hWnd,iIndex,&sButton);

	uCommand		= sButton.idCommand;
	uStyle			= MAKELONG(sButton.fsStyle,sButton.fsState)^TBBS_DISABLED;
	iImage			= sButton.iImage;

}

//*****************************************************************************
//*
//*		GetButtonInfo
//*
//*****************************************************************************
//	Holt diverse Tasten-Attribute.
//	iIndex		: Ist der Index der Taste, Es knne auch folgende Makros benutzt werden:
//					TB_USE_ID(i)		Taste ber ID suchen
//					TB_USE_INDEX(i)		Taste ber Index suchen (optional)
//					TB_USE_NID(i,n)		Taste ber n'te ID suchen
//					TB_USE_FLAG(f,n)	Taste ber n'tes Flags suchen (BTFL_???)
//	uCommand	: Ist die ID fr die Taste
//	uStyle		: Ist der Style/State  (BTNS_???/TBBS_???)
//	iImage		: Ist die Bilde-ID	   (-1 keines)
//	uFlags		: Sind die Flags       (BTFL_???)
void CToolBarNew::GetButtonInfo(int iIndex,UINT &uCommand,UINT &uStyle,int &iImage,UINT uFlags) const
{
TBBUTTONINFOEX	sButton;


	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));

	sButton.cbSize	= sizeof(sButton);
	sButton.dwMask	= TBIF_COMMAND|TBIF_STYLE|TBIF_STATE|TBIF_IMAGE|TBIF_FLAGS;

	ToolBarEx_GetButtonInfo(m_hWnd,iIndex,&sButton);

	uCommand		= sButton.idCommand;
	uStyle			= MAKELONG(sButton.fsStyle,sButton.fsState)^TBBS_DISABLED;
	iImage			= sButton.iImage;
	uFlags			= sButton.wFlags;

}

//*****************************************************************************
//*
//*		SetStyle
//*
//*****************************************************************************
//	ndert den Style der Toolbar.
//	dwStyle		: Ist der neue Style
//					TBSTYLE_PACKED		0x0020	Vertikal stapeln ist erlaut
//					TBSTYLE_FLAT		0x0800	Flache Tasten
//					TBSTYLE_TOOLTIPS    0x0100	Tooltips anzeigen
//	Ergibt die alten Style-Bits.
DWORD CToolBarNew::SetStyle(DWORD dwStyle)
{
DWORD	dwOld;

	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	dwOld = ToolBarEx_SetStyle(m_hWnd,dwStyle);

	UpdateLayout();

return dwOld;
}

//*****************************************************************************
//*
//*		SetStyle
//*
//*****************************************************************************
//	ndert den Style der Toolbar mittel Maske
//	dwStyle		: Ist der neue Style
//					TBSTYLE_PACKED		0x0020	Vertikal stapeln ist erlaut
//					TBSTYLE_FLAT		0x0800	Flache Tasten
//					TBSTYLE_TOOLTIPS    0x0100	Tooltips anzeigen
//	bRedraw		: Soll die Toolbar neu gezeichnet werden
//	Ergibt die alten Style-Bits.
DWORD CToolBarNew::SetStyle(DWORD dwStyle,DWORD dwMask,BOOL bRedraw)
{
DWORD	dwOld;

	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	dwOld = ToolBarEx_SetStyleEx(m_hWnd,dwStyle,dwMask);

	if(((dwOld^dwStyle)&dwMask&(TBSTYLE_PACKED|TBSTYLE_FLAT|WS_VISIBLE)) && bRedraw)
		{
		UpdateLayout();
		}


return dwOld;
}


//*****************************************************************************
//*
//*		SetScaling
//*
//*****************************************************************************
//	Stellt die Skalirung fr die gesammte Toolbar ein
//	iScale		: Ist die neue Skalierung (256=1:1)
//	bRedraw		: Die Toolbar neu zeichnen
//	Ergibt die vorherige Skalierung
int CToolBarNew::SetScaling(int iScale,BOOL bRedraw)
{
int		iOld;


	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	if(!iScale)iScale=256;

	iOld = ToolBarEx_SetScaling(m_hWnd,iScale);

	if(iOld!=iScale && bRedraw)
		{
		m_bDelayedButtonLayout = TRUE;
		UpdateLayout();
		}


return iOld;
}

//*****************************************************************************
//*
//*		SetScaling
//*
//*****************************************************************************
//	Stellt die Skalirung fr die gesammte Toolbar ein
//	iScale		: Ist die neue Skalierung (256=1:1)
//	bRedraw		: Die Toolbar neu zeichnen
//	Ergibt die vorherige Skalierung
int CToolBarNew::SetBitmapScaling(int iBitmap,int iScale,BOOL bRedraw)
{
int		iOld;


	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	if(!iScale)iScale=256;

	iOld = ToolBarEx_SetBitmapScaling(m_hWnd,iBitmap,iScale);

	if(iOld!=iScale && bRedraw)
		{
		m_bDelayedButtonLayout = TRUE;
		UpdateLayout();
		}


return iOld;
}

//*****************************************************************************
//*
//*		SetButtons
//*
//*****************************************************************************
//	Fgt eine Liste von Tasten ein. Die alten Tasten werden gelscht.
//	pIdArray	: Liste mit den ID's fr die Tasten (0=Separator)
//				  Die Tasten Bilder werden von 0 bis N nummeriert.
//				  Wenn dieser Zeiger NULL ist weden nur Tasten ohne
//				  Bild und mit der ID 0 eingefgt.
//	iCount		: Ist die Anzahl der Tasten.
//	Ergibt TRUE wenn erflogreich.
BOOL CToolBarNew::SetButtons(const UINT *pIdArray,int iCount)
{
TBBUTTONEX	sButton;
int			iIndex;
int			iImage;
int			iRet;


	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));
	ASSERT(pIdArray==NULL || AfxIsValidAddress(pIdArray,sizeof(UINT)*iCount,FALSE));


	ToolBarEx_DelButtons(m_hWnd,0,0xFFFF);				// Lsche vorhandene Tasten

	memset(&sButton,0,sizeof(sButton));

	sButton.iString = -1;


	if(pIdArray)										// add new buttons to the common control
		{
		iImage = 0;

		for(iIndex=0;iIndex<iCount;iIndex++)
			{
			sButton.fsState	  = TBSTATE_ENABLED;
			sButton.idCommand = *pIdArray;

			pIdArray++;

			if(!sButton.idCommand)						// Separator einfgen
				{
				sButton.fsStyle = TBSTYLE_SEP;
				sButton.iBitmap = -1;
				}
			else{										// Eine Taste mit Bild
				sButton.fsStyle = TBSTYLE_BUTTON;
				sButton.iBitmap = iImage;
				iImage++;
			}

				iRet = ToolBarEx_AddButtonsEx(m_hWnd,iIndex,1,&sButton);
			if(!iRet)
				{
				return FALSE;
				}
			}
		}
	else{												// Tasten ohne Bilder
		sButton.fsState = TBSTATE_ENABLED;

		for(iIndex=0;iIndex<iCount;iIndex++)
			{
			ASSERT(sButton.fsStyle==TBSTYLE_BUTTON);

				iRet = ToolBarEx_AddButtonsEx(m_hWnd,iIndex,1,&sButton);
			if(!iRet)
				{
				return FALSE;
				}
			}
		}

	m_bDelayedButtonLayout = TRUE;



return TRUE;
}

//*****************************************************************************
//*
//*		SetGroup
//*
//*****************************************************************************
//	Bestimmt eine Tastengruppe. Bei der ersten Taste wird der Stype BTNS_GROUP
//	und BTNS_GROUPSTART gesetzt. Bei allen weiteren nur BTNS_GROUP.
//	iIndex		: Ist der Index der ersten Taste der Gruppe.
//				  Es knne auch folgende Makros benutzt werden:
//					TB_USE_ID(i)		Taste ber ID suchen
//					TB_USE_INDEX(i)		Taste ber Index suchen (optional)
//					TB_USE_NID(i,n)		Taste ber n'te ID suchen
//					TB_USE_FLAG(f,n)	Taste ber n'tes Flags suchen (BTFL_???)
//	iCount		: Ist die Anzahl der Tasten in der Gruppe
BOOL CToolBarNew::SetGroup(int iIndex,int iCount)
{
int		iDone;


	ASSERT_VALID(this);
	ASSERT(IsWindow(m_hWnd));


	if(iCount<=1)return FALSE;

	if(TB_SPECIAL(iIndex))									// Spezial Index
		{
		iIndex = ToolBarEx_CommandToIndex(m_hWnd,iIndex);
		}


	iDone	= ToolBarEx_ChangeStyle(m_hWnd,BTNS_GROUP|BTNS_GROUPSTART,BTNS_GROUP|BTNS_GROUPSTART,iIndex  ,1       );
	iDone  += ToolBarEx_ChangeStyle(m_hWnd,BTNS_GROUP                ,BTNS_GROUP                ,iIndex+1,iCount-1);


return (iDone==iCount);
}

//*****************************************************************************
//*
//*		OnUpdateCmdUI
//*
//*****************************************************************************
void CToolBarNew::OnUpdateCmdUI(CFrameWnd *pTarget,BOOL bDisableIfNoHndler)
{
CToolCmdNewUI	sHandler;
TBBUTTONINFOEX	sButton;
BOOL			bResult;



	sButton.cbSize		 = sizeof(sButton);
	sButton.dwMask		 = TBIF_COMMAND|TBIF_STYLE;
	sHandler.m_nIndexMax = ButtonCount();
	sHandler.m_pOther	 = this;
	sHandler.m_nIndex	 = 0;

															// Prfen ob alle Tasten aktiv sind
	for(sHandler.m_nIndex=0;sHandler.m_nIndex<sHandler.m_nIndexMax;sHandler.m_nIndex++)
		{
		ToolBarEx_GetButtonInfo(m_hWnd,sHandler.m_nIndex,&sButton);

		sHandler.m_nID	= sButton.idCommand;

		if(sButton.fsStyle&TBSTYLE_SEP)continue;			// Ignoriere Separatoren

															// Nachrichten Reflektion
		if(CWnd::OnCmdMsg(0,MAKELONG((int)CN_UPDATE_COMMAND_UI,WM_COMMAND+WM_REFLECT_BASE),&sHandler,NULL))
			{
			continue;
			}
															// Eigene Handler aufrufen
		if(CWnd::OnCmdMsg(sHandler.m_nID,CN_UPDATE_COMMAND_UI,&sHandler,NULL))
			{
			continue;
			}

		if(sButton.idCommand==0 || LOWORD(sButton.idCommand)==0xFFFF)
			{
			continue;										// Falsche ID
			}

		sHandler.m_bEnableChanged = FALSE;

			bResult = pTarget->OnCmdMsg(sButton.idCommand,CN_UPDATE_COMMAND_UI,&sHandler,NULL);
		if(!bResult)
			{
			ASSERT(!sHandler.m_bEnableChanged);				// Nicht geroutet
			}

		sHandler.DoUpdate(pTarget,bDisableIfNoHndler);
		}


	UpdateDialogControls(pTarget,bDisableIfNoHndler);		// Update Controls

}


//*****************************************************************************
//*
//*		OnWndMsg
//*
//*****************************************************************************
//	WM_COMMAND und WM_NOTIFY Nachrichten der Controls weiterleiten.
BOOL CToolBarNew::OnWndMsg(UINT uMsg,WPARAM wParam,LPARAM lParam,LRESULT *pResult)
{
CFrameWnd  *pFrame;
CView	   *pView;
int			iIndex;
unsigned	uIdx;
LRESULT		lRet;


	if(uMsg==WM_COMMAND)
		{
		if(LOWORD(wParam)==ID_TOOLMENU_HIDE)			// Toolbar ausblenden
			{
			   pFrame = (CFrameWnd*)AfxGetMainWnd();
			if(pFrame)
				{
				pFrame->ShowControlBar(this,FALSE,FALSE);
				return TRUE;
				}
			}

		   iIndex = ToolBarEx_FindControl(m_hWnd,lParam);
		if(iIndex>=0)
			{
			if(HIWORD(wParam)==CBN_SELCHANGE)			// Auswahl in ComboBox verndert
				{
				uIdx = (int)::SendMessage((HWND)lParam,CB_GETCURSEL,0,0);
				lRet = (int)::SendMessage((HWND)lParam,CB_GETITEMDATA,uIdx,0);

				if(lRet<=0)lRet=GetWindowLong((HWND)lParam,GWL_ID)+uIdx;
				}

			   pFrame	= (CFrameWnd*)AfxGetMainWnd();
			if(pFrame)
				{
				           pFrame->PostMessage(WM_COMMAND,wParam,lParam);
				if(lRet>=0)pFrame->PostMessage(WM_COMMAND,lRet  ,lParam);

				   pView = pFrame->GetActiveView();
				if(pView)
					{
							   pView->PostMessage(WM_COMMAND,wParam,lParam);
					if(lRet>=0)pView->PostMessage(WM_COMMAND,lRet  ,lParam);
					}
				}

			return TRUE;
			}
		}

	if(uMsg==WM_NOTIFY)
		{
		   iIndex = ToolBarEx_FindControl(m_hWnd,lParam);
		if(iIndex>=0)
			{
			*pResult = 0;

			   pFrame = (CFrameWnd*)AfxGetMainWnd();
			if(pFrame)
				{
				   lRet = pFrame->SendMessage(WM_NOTIFY,wParam,lParam);
				if(lRet)*pResult=lRet;

				   pView = pFrame->GetActiveView();
				if(pView)
					{
					   lRet = pView->SendMessage(WM_NOTIFY,wParam,lParam);
					if(lRet)*pResult=lRet;
					}
				}

			return TRUE;
			}
		}


return CToolBar::OnWndMsg(uMsg,wParam,lParam,pResult);
}

//*****************************************************************************
//*
//*		OnTimer
//*
//*****************************************************************************
void CToolBarNew::OnTimer(UINT_PTR nIdEvent)
{

	if(nIdEvent==ID_UPDATE)
		{
		::SendMessage(m_hWnd,TB_TIMER,0,0);
		return;
		}


	CToolBar::OnTimer(nIdEvent);

}

//*****************************************************************************
//*
//*		OnContextMenu
//*
//*****************************************************************************
void CToolBarNew::OnContextMenu(CWnd *pWnd,CPoint sPoint)
{
TCHAR		aTemp[64];
CWnd	   *pFrame;
CString		sNames;
LPCTSTR		pText;
LPCTSTR		pList;
UINT		uFlags;
CMenu		sMenu;
CPoint		sPos;
unsigned	uNum;
unsigned	uLen;



	if(!pWnd)return;


	if(!(m_uExtraStyle&(((1<<CNB_MENU_SIZE)-1)<<16)))	// Ist ein Menpunkt aktiviert
		{
		CToolBar::OnContextMenu(pWnd,sPoint);
		return;
		}

	if(sNames.LoadString(AFX_TB_NAMES))					// Text aus den Resourcen laden
			pList = sNames;
	else	pList = 0;


		pFrame  = AfxGetMainWnd();
	if(!pFrame)return;

	sPos	= sPoint;
	ScreenToClient(&sPos);


	sMenu.CreatePopupMenu();

	for(uNum=0;uNum<CNB_MENU_SIZE;uNum++)
		{
		pText = m_aDefaultMenuNames[uNum];

		if(pList)
			{											// Texte aus den Resourcen nehmen
			if(*pList=='\n')
				{
				pList++;
				}
			else if(*pList!=0)
				{
				for(uLen=0;uLen<63;uLen++)
					{
					if(pList[uLen]==0   )break;
					if(pList[uLen]=='\n')break;
					}

				memcpy(aTemp,pList,uLen*sizeof(TCHAR));
				aTemp[uLen] = 0;
				pText = aTemp;

				if(pList[uLen])pList++;
				}
			}

		if(!((m_uExtraStyle>>(16+uNum))&1))
			{
			continue;
			}

		if(m_aMenuTxt[uNum])							// Ist ein Text definiert
			{
			pText = m_aMenuTxt[uNum];
			}

		if(uNum<3)										// Haken im Men setzen
			{
			uFlags	= (uNum==m_uIconSize)? MF_CHECKED:0;
			}
		else if(uNum==3)
			{
			uFlags	= GetStyle();
			uFlags	= (uFlags&TBSTYLE_FLAT)? MF_CHECKED:0;
			if(m_uExtraStyle&0x00070000)sMenu.AppendMenu(MF_SEPARATOR);
			}
		else{
			uFlags  = 0;
			if(m_uExtraStyle&0x000F0000)sMenu.AppendMenu(MF_SEPARATOR);
			}

		sMenu.AppendMenu(MF_STRING|uFlags,m_aMenuIds[uNum],pText);
		}


	sMenu.TrackPopupMenu(0,sPoint.x,sPoint.y,pFrame);
	sMenu.DestroyMenu();



}

//*****************************************************************************
//*
//*		UpdateLayout
//*
//*****************************************************************************
//	Die ganze Toolbar wird upgedatet und neu berechnet
void CToolBarNew::UpdateLayout()
{
CFrameWnd  *pFrame;


	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));

	m_bDelayedButtonLayout = TRUE;

	if(!::IsWindowVisible(m_hWnd))
		{
		ToolBarEx_ReCaclLayout(m_hWnd,0x03,TRUE);
		return;
		}

		pFrame = (CFrameWnd*)AfxGetMainWnd();
	if(!pFrame)return;

	ToolBarEx_ReCaclLayout(m_hWnd,0x03,FALSE);
	pFrame->ShowControlBar(this ,FALSE,FALSE);
	pFrame->ShowControlBar(this ,TRUE ,FALSE);

}

//*****************************************************************************
//*
//*		CalcSize
//*
//*****************************************************************************
//	Berechnet die Gre vom horizontalen Toolbar-Fenster.
//	bSetSize		: Die Fenstegre bernehmen
//	Ergibt die Fenstegre (LOWORD=X HIWORD=Y)
CSize CToolBarNew::CalcSize(BOOL bSetSize,int iMinX)
{
CRect		sRect;
CSize		sSize;
LRESULT		lRet;


	sRect.SetRectEmpty();

	CalcInsideRect(sRect,TRUE);

	lRet		= ToolBarEx_CalcFrame(m_hWnd,17000,0x00);
	sSize.cx	= (lRet&0xFFFF)-sRect.Width ();
	sSize.cy	= (lRet >> 16 )-sRect.Height();

	if(sSize.cx<iMinX)sSize.cx=iMinX;

	if(bSetSize)SetWindowPos(NULL,0,0,sSize.cx,sSize.cy,SWP_NOZORDER|SWP_NOMOVE|SWP_NOACTIVATE);


return sSize;
}

//*****************************************************************************
//*
//*		TagInsert
//*
//*****************************************************************************
//	Fgt eine TAG (Text) ber einen Eintrag ein.
//	pText		: Ist der Text fr den Eintrag
//	uIndex		: Ist der Index vom Eintrag bder den der Text sein soll
//					TB_USE_ID(i)		Index aus Command-ID holen
//					TB_USE_INDEX(i)		Index aus LOWORD holen
//					TB_USE_NID(i,n)		Die n'te Command-ID benutzen
//					TB_USE_FLAG(f,n)	Den Eintrag mit dem n'ten Flag benutzen
//	iPosX/Y		: Ist der Pixel-Offest zum Eintrag
//	bFlags		: Diverse Flags fr das TAG.
//					TBTF_LEFT		0x00	TAG links  ausurichten
//					TBTF_CENTER		0x01	TAG mittig ausurichten
//					TBTF_RIGHT		0x02	TAG rechts ausurichten
//					TBTF_TOP		0x00	TAG oben   ausurichten
//					TBTF_VCENTER	0x04	TAG mittig ausurichten
//					TBTF_BOTTOM		0x08	TAG unten  ausurichten
//					TBTF_DELEXIST	0x20	TAG ein existierntes TAG lschen
//					TBTF_USEID		0x40	TAG hat statt Index eine ID
//	bId			: Ist die lokale ID vom TAG
//	Ergibt TRUE wenn der TAG eingefgt wurde
BOOL CToolBarNew::TagInsert(LPCTSTR pText,UINT uIndex,int iPosX,int iPosY,BYTE bFlags,BYTE bId)
{
TBTAGITEM	sItem;


	sItem.uIndex	= uIndex;									// Index oder ID der Taste fr das TAG
	sItem.sDeltaX	= (SHORT)iPosX;								// Offset zum Icon
	sItem.sDeltaY	= (SHORT)iPosY;								//			*
	sItem.ucFlags	= bFlags;									// Diverse Flags (TBTF_???)
	sItem.ucItemId	= bId;										// ID fr's lchen und einfgen


return TagInsert(&sItem,pText);
}

