//*****************************************************************************
//*
//*
//*		TreeListCtrlEx.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//	Anmerkung:		 AfxOleInit() vorher aufrufen, damit Drags funktionieren !
//
#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif

#include	"TreeListCtrlEx.h"

#ifdef		_DEBUG
#define new  DEBUG_NEW
#undef		 THIS_FILE
static char  THIS_FILE[] = __FILE__;
#endif

#ifndef	    _WIN64
#ifndef		 SetWindowLongPtr
#define		 SetWindowLongPtr	SetWindowLong
#endif
#ifndef		 GetWindowLongPtr
#define		 GetWindowLongPtr	GetWindowLong
#endif
#endif


WNDPROC		 CTreeListCtrlEx::m_pWndProc;				// Fenster Funktion

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Creates a new Tree-List-Control
//	dwExStyle	: Is the exdended style of the window (WS_EX_????)
//	dwStyle     : Style of the Window
//                	TVS_CHECKBOXES
//                	TVS_DISABLEDRAGDROP
//                	TVS_EDITLABELS
//                	TVS_FULLROWSELECT
//                	TVS_HASBUTTONS
//                	TVS_HASLINES
//                	TVS_INFOTIP
//                	TVS_LINESATROOT
//                	TVS_NONEVENHEIGHT
//                	TVS_NOSCROLL
//                	TVS_NOTOOLTIPS
//                	TVS_RTLREADING
//                	TVS_SHOWSELALWAYS
//                	TVS_SINGLEEXPAND
//                	TVS_TRACKSELECT
//	sRect		: Rectangle where the window should be shown
//	pParentWnd  : Pointer to parent window
//	nId         : Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CTreeListCtrlEx::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nId)
{
BOOL	bOk;

		bOk	= CTreeListCtrl::CreateEx(dwExStyle,dwStyle,sRect,pParentWnd,nId);
	if(!bOk)return FALSE;

	if(!m_pWndProc)m_pWndProc=(WNDPROC)GetOption(TVOP_WINDOWPROC);
	

return TRUE;
}


//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Creates a new Tree-List-Control over a dialog item
//	dwExStyle	: Is the exdended style of the window (WS_EX_????)
//	dwStyle     : Style of the Window
//                	TVS_CHECKBOXES
//                	TVS_DISABLEDRAGDROP
//                	TVS_EDITLABELS
//                	TVS_FULLROWSELECT
//                	TVS_HASBUTTONS
//                	TVS_HASLINES
//                	TVS_INFOTIP
//                	TVS_LINESATROOT
//                	TVS_NONEVENHEIGHT
//                	TVS_NOSCROLL
//                	TVS_NOTOOLTIPS
//                	TVS_RTLREADING
//                	TVS_SHOWSELALWAYS
//                	TVS_SINGLEEXPAND
//                	TVS_TRACKSELECT
//	uItem       : Is the ID of the dialog item
//	pDlg		: Pointer to parent dialog
//	nId			: Is the ID of the window
//	Returns TRUE if the window was created or FALSE if an error occurs
BOOL CTreeListCtrlEx::CreateEx(DWORD dwExStyle,DWORD dwStyle,UINT uItem,CDialog *pDlg,UINT nId)
{
BOOL	bOk;

		bOk	= CTreeListCtrl::CreateEx(dwExStyle,dwStyle,uItem,pDlg,nId);
	if(!bOk)return FALSE;

	if(!m_pWndProc)m_pWndProc=(WNDPROC)GetOption(TVOP_WINDOWPROC);

return TRUE;
}

//*****************************************************************************
//*
//*		InsertItemEx
//*
//*****************************************************************************
//	Call this function to insert a new item in a tree list control.
//	nMask			: Integer specifying which attributes to set. See theTVITEM structure in the Platform SDK.
//	pText			: Address of a string containing the items text.
//	nImage			: Index of the items image in the tree view controls image list.
//	nSelImage		: Index of the items selected image in the tree view controls image list.
//	nState			: Specifies values for the items states. SeeTree View Control Item States in the Platform SDK for a list of appropriate states.
//	nStateMask		: Specifies which states are to be set. See theTVITEM structure in the Platform SDK.
//	lParam			: A 32-bit application-specific value associated with the item.
//	hParent			: Handle of the inserted items parent.
//	hInsertAfter	: Handle of the item after which the new item is to be inserted.
//	Returns the handle of the new item if successful, otherwise NULL.
HTREEITEM CTreeListCtrlEx::InsertItemEx(UINT nMask, LPCTSTR pText,int nImage,int nSelImage,UINT nState,UINT nStateMask,LPARAM lParam,HTREEITEM hParent,HTREEITEM hInsertAfter)
{
TVINSERTSTRUCT	sInsert;


	ASSERT(::IsWindow(m_hWnd));

	sInsert.hParent				= hParent;
	sInsert.hInsertAfter		= hInsertAfter;
	sInsert.item.mask			= nMask|TVIF_REMOTE;
	sInsert.item.pszText		= (LPTSTR)pText;
	sInsert.item.iImage			= nImage;
	sInsert.item.iSelectedImage = nSelImage;
	sInsert.item.state			= nState;
	sInsert.item.stateMask		= nStateMask;
	sInsert.item.lParam			= lParam;


return (HTREEITEM)m_pWndProc(m_hWnd,TVM_INSERTITEM,0,(LPARAM)&sInsert);
}

//*****************************************************************************
//*
//*		InsertItemEx
//*
//*****************************************************************************
//	Call this function to insert a new item in a tree list control, with a sort fonction.
//	nMask			: Integer specifying which attributes to set. See theTVITEM structure in the Platform SDK.
//	pText			: Address of a string containing the items text.
//	nImage			: Index of the items image in the tree view controls image list.
//	nSelImage		: Index of the items selected image in the tree view controls image list.
//	nState			: Specifies values for the items states. SeeTree View Control Item States in the Platform SDK for a list of appropriate states.
//	nStateMask		: Specifies which states are to be set. See theTVITEM structure in the Platform SDK.
//	lParam			: A 32-bit application-specific value associated with the item.
//	pCmpProc		: Is the sort function which is used to find the insert position
//	hParent			: Handle of the inserted items parent.
//	Returns the handle of the new item if successful, otherwise NULL.
HTREEITEM CTreeListCtrlEx::InsertItemEx(UINT nMask, LPCTSTR pText,int nImage,int nSelImage,UINT nState,UINT nStateMask,LPARAM lParam,PFNTVSORTEX pCmpProc,HTREEITEM hParent)
{
TVINSERTSTRUCT	sInsert;


	ASSERT(::IsWindow(m_hWnd));

	sInsert.hParent				= hParent;
	sInsert.hInsertAfter		= TVI_SORTEX;
	sInsert.item.hItem			= (HTREEITEM)pCmpProc;
	sInsert.item.pszText		= (LPTSTR)pText;
	sInsert.item.mask			= nMask|TVIF_REMOTE;
	sInsert.item.iImage			= nImage;
	sInsert.item.iSelectedImage = nSelImage;
	sInsert.item.state			= nState;
	sInsert.item.stateMask		= nStateMask;
	sInsert.item.lParam			= lParam;


return (HTREEITEM)m_pWndProc(m_hWnd,TVM_INSERTITEM,0,(LPARAM)&sInsert);
}

//*****************************************************************************
//*
//*		InsertItemEx
//*
//*****************************************************************************
//	Call this function to insert a new item in a tree list control, with a sort fonction.
//	nMask			: Integer specifying which attributes to set. See theTVITEM structure in the Platform SDK.
//	pText			: Address of a string containing the items text.
//	nImage			: Index of the items image in the tree view controls image list.
//	nSelImage		: Index of the items selected image in the tree view controls image list.
//	nState			: Specifies values for the items states. SeeTree View Control Item States in the Platform SDK for a list of appropriate states.
//	nStateMask		: Specifies which states are to be set. See theTVITEM structure in the Platform SDK.
//	lParam			: A 32-bit application-specific value associated with the item.
//	pCmpProc		: Is the sort function which is used to find the insert position
//	hParent			: Handle of the inserted items parent.
//	Returns the handle of the new item if successful, otherwise NULL.
HTREEITEM CTreeListCtrlEx::InsertItemEx(UINT nMask, LPCTSTR pText,int nImage,int nSelImage,UINT nState,UINT nStateMask,LPARAM lParam,PFNTVTEXTCMP pCmpProc,HTREEITEM hParent)
{
TVINSERTSTRUCT	sInsert;


	ASSERT(::IsWindow(m_hWnd));

	sInsert.hParent				= hParent;
	sInsert.hInsertAfter		= TVI_SORTEX;
	sInsert.item.hItem			= (HTREEITEM)InsertCmp;
	sInsert.item.pszText		= (LPTSTR)pText;
	sInsert.item.mask			= nMask;
	sInsert.item.iImage			= nImage;
	sInsert.item.iSelectedImage = nSelImage;
	sInsert.item.state			= nState;
	sInsert.item.stateMask		= nStateMask;
	sInsert.item.lParam			= (LPARAM)pCmpProc;

	if(sInsert.item.mask&TVIF_PARAM)
		{
		sInsert.item.mask      &= ~TVIF_PARAM;
		sInsert.item.mask      |=  TVIF_SETPARAM;

		TVIS_SET_LPARAM(&sInsert,lParam);
		}


return (HTREEITEM)::SendMessage(m_hWnd,TVM_INSERTITEM,0,(LPARAM)&sInsert);
}


//*****************************************************************************
//*
//*		InsertItemEx
//*
//*****************************************************************************
//	Call this function to insert a new item in a tree list control, with a sort fonction.
//	nMask			: Integer specifying which attributes to set. See theTVITEM structure in the Platform SDK.
//	pText			: Address of a string containing the items text.
//	nImage			: Index of the items image in the tree view controls image list.
//	nSelImage		: Index of the items selected image in the tree view controls image list.
//	nState			: Specifies values for the items states. SeeTree View Control Item States in the Platform SDK for a list of appropriate states.
//	nStateMask		: Specifies which states are to be set. See theTVITEM structure in the Platform SDK.
//	lParam			: A 32-bit application-specific value which is set on the item.
//	pCmpProc		: Is the sort function which is used to find the insert position
//	hParent			: Handle of the inserted items parent.
//	pCbParam		: Is the parameter fpr the callback funktion instead of lParam
//	Returns the handle of the new item if successful, otherwise NULL.
HTREEITEM CTreeListCtrlEx::InsertItemEx(UINT nMask,LPCTSTR pText,int nImage,int nSelImage,UINT nState,UINT nStateMask,LPARAM lParam,PFNTVSORTEX pCmpProc,HTREEITEM hParent,LPVOID pCbParam)
{
TVINSERTSTRUCT	sInsert;


	ASSERT(::IsWindow(m_hWnd));

	sInsert.hParent				= hParent;
	sInsert.hInsertAfter		= TVI_SORTEX;
	sInsert.item.hItem			= (HTREEITEM)pCmpProc;
	sInsert.item.pszText		= (LPTSTR)pText;
	sInsert.item.mask			= nMask|TVIF_SETPARAM;
	sInsert.item.iImage			= nImage;
	sInsert.item.iSelectedImage = nSelImage;
	sInsert.item.state			= nState;
	sInsert.item.stateMask		= nStateMask;
	sInsert.item.lParam			= (LPARAM)pCbParam;

	if(sInsert.item.mask&TVIF_PARAM)
		{
		sInsert.item.mask      &= ~TVIF_PARAM;
		sInsert.item.mask      |=  TVIF_SETPARAM;

		TVIS_SET_LPARAM(&sInsert,lParam);
		}


return (HTREEITEM)m_pWndProc(m_hWnd,TVM_INSERTITEM,0,(LPARAM)&sInsert);
}


//*****************************************************************************
//*
//*		FindItemEx
//*
//*****************************************************************************
//	Seaches for an Item in the control with several properties
//	hItem		: Is the item handle where the seach begins
//	iFlags		: This flag define the seach options
//					TVIF_CHILDS	= begin at the first child of hItem to seach
//					TVIF_NEXT	= begin at the next item after hItem to seach
//					TVIF_PARAM	= compare the the lParam  entry at the seach
//					TVIF_TEXT	= compare the the pText   entry at the seach
//					TVIF_STATE	= compare the the uState  entry at the seach
//					TVIF_CASE	= don't differ between lower and upper case
//	nCol		: Selects the column for the text comparison
//	pText		: Is the text option for the seach
//	lParam		: Is the param option for the seach
//	uState		: Are the state bits for the seach
//	uStateMask	: Only bits which are set in this mask will be comared
//	Returns the handle of the item which was found, or NULL if no item was found
HTREEITEM CTreeListCtrlEx::FindItemEx(HTREEITEM hItem,int iFlags,int nCol,LPCTSTR pText,LPARAM lParam,UINT uState,UINT uStateMask)
{
TV_FIND		sFind;


	sFind.uFlags		= iFlags;
	sFind.uColumn		= nCol;
	sFind.pText			= pText;
	sFind.lParam		= lParam;
	sFind.uState		= uState;
	sFind.uStateMask	= uStateMask;


return FindItemEx(hItem,&sFind);
}

//*****************************************************************************
//*
//*		SetMultiItemsEx
//*
//*****************************************************************************
//	Changes some properties of several items item
//	hItem			: is the handle of the item
//	nCol			: is the column for the text (0=tree column)
//	nMask			: defies which properties should be set and which items:
//						TVIF_PARENTS      = change all parents items
//						TVIF_OWNITEM      = change own item
//						TVIF_SIBLINGS     = change all sibling items
//						TVIF_CHILDS	      = change all child items
//						TVIF_SAVESTATE	  = save states from nStateMask>>16
//						TVIF_RESTORESTATE = restore states from state high word
//						TVIF_...	      = see at TreeList documentation
//	pText			: is the new text of the item (add TVIF_TEXT to nMask)
//	nImage			: is the new image for the item (add TVIF_IMAGE to nMask)
//	nSelectedImage	: is the new image for the item (add TVIF_SELECTEDIMAGE to nMask)
//	nState			: is the new state for the item (add TVIF_STATE to nMask)
//	nStateMask		: is a mask for the states
//	lParam			: is the new LPARAM parameter for the item (add TVIF_PARAM to nMask)
//	Returns the count of changed items, or -1 if an error occurs.
BOOL CTreeListCtrlEx::SetMultiItemsEx(HTREEITEM hItem,int nCol,UINT nMask,LPCTSTR pText,int nImage,int nSelectedImage,UINT nState, UINT nStateMask,LPARAM lParam)
{
TV_ITEM		sItem;


	ASSERT(::IsWindow(m_hWnd));

	sItem.mask				= nMask|TVIF_SUBITEM|TVIF_REMOTE;
    sItem.hItem				= hItem;
    sItem.state				= nState;
    sItem.stateMask			= nStateMask;
    sItem.pszText			=(LPTSTR)pText;
    sItem.cchTextMax		=(pText)? 256:0;
    sItem.iImage			= nImage;
    sItem.iSelectedImage	= nSelectedImage;
    sItem.cChildren			= nCol;
	sItem.lParam			= lParam;


return SetMultiItemsEx(&sItem);
}


