//*****************************************************************************
//*
//*
//*		CoordFunctions.cpp
//*
//*
//*****************************************************************************
#include	<math.h>
#include	<time.h>
#include	<stddef.h>

#include	"StringFunctions.h"
#include	"CoordFunctions.h"
#include	"FileFunctions.h"
#include	"MemFunctions.h"
#include	"ZipFile.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define		PI				3.1415926535897932384626433832795
#define		EARTH_RADIUS	6371000.8				// Mittlerer Erdradius
#define		DELTA_CNT		15						// Gre des Zitternfilters

static	const char	*pGpxTail   = "</trkseg></trk></gpx>";
static	const char	*pGpxHeader =
				"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>\n"
				"<gpx xmlns=\"http://www.topografix.com/GPX/1/1\" xmlns:gpxx=\"http://www.garmin.com/xmlschemas/WaypointExtension/v1\" xmlns:gpxtrx=\"http://www.garmin.com/xmlschemas/GpxExtensions/v3\" xmlns:gpxtpx=\"http://www.garmin.com/xmlschemas/TrackPointExtension/v1\" creator=\"Oregon 550\" version=\"1.1\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd http://www.garmin.com/xmlschemas/WaypointExtension/v1 http://www8.garmin.com/xmlschemas/WaypointExtensionv1.xsd http://www.garmin.com/xmlschemas/TrackPointExtension/v1 http://www.garmin.com/xmlschemas/TrackPointExtensionv1.xsd\"><metadata><link href=\"http://www.garmin.com\">\n"
				"<text>Garmin International</text>\n"
				"</link>\n"
				"<time>%sT%sZ</time>\n"
				"</metadata>\n"
				"<trk>\n\t<name>%s</name>\n"
				"<extensions>"
				"\t<gpxtrx:TrackExtension>"
				"\t<gpxtrx:DisplayColor>DarkBlue</gpxtrx:DisplayColor>"
				"\t</gpxtrx:TrackExtension>"
				"</extensions>";

const char	*pKmlTail   = "\t\t\t</coordinates>\n\t\t</LineString>\n\t<styleUrl>#roadStyle</styleUrl>\n\t</Placemark>\n</Document>\n</kml>\n";
const char	*pKmlHeader =
				"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
				"<kml xmlns=\"http://www.opengis.net/kml/2.2\" xmlns:gx=\"http://www.google.com/kml/ext/2.2\" xmlns:kml=\"http://www.opengis.net/kml/2.2\" xmlns:atom=\"http://www.w3.org/2005/Atom\">\n"
				"<Document>\n"
				"	<name>%s</name>\n"
				"	<Style id=\"roadStyle\">\n"
				"		<LineStyle>\n"
				"			<colorMode>random</colorMode>\n"
				"			<width>5</width>\n"
				"		</LineStyle>\n"
				"	</Style>\n"
				"	<Style id=\"sh_ylw-pushpin\">\n"
				"		<IconStyle>\n"
				"			<scale>1.3</scale>\n"
				"			<Icon>\n"
				"				<href>http://maps.google.com/mapfiles/kml/pushpin/ylw-pushpin.png</href>\n"
				"			</Icon>\n"
				"			<hotSpot x=\"20\" y=\"2\" xunits=\"pixels\" yunits=\"pixels\"/>\n"
				"		</IconStyle>\n"
				"      <LineStyle>\n"
				"        <colorMode>random</colorMode>\n"
				"        <width>5</width>\n"
				"      </LineStyle>\n"
				"	</Style>\n"
				"	<StyleMap id=\"msn_ylw-pushpin\">\n"
				"		<Pair>\n"
				"			<key>normal</key>\n"
				"			<styleUrl>#sn_ylw-pushpin</styleUrl>\n"
				"		</Pair>\n"
				"		<Pair>\n"
				"			<key>highlight</key>\n"
				"			<styleUrl>#sh_ylw-pushpin</styleUrl>\n"
				"		</Pair>\n"
				"	</StyleMap>\n"
				"	<Style id=\"sn_ylw-pushpin\">\n"
				"		<IconStyle>\n"
				"			<scale>1.1</scale>\n"
				"			<Icon>\n"
				"				<href>http://maps.google.com/mapfiles/kml/pushpin/ylw-pushpin.png</href>\n"
				"			</Icon>\n"
				"			<hotSpot x=\"20\" y=\"2\" xunits=\"pixels\" yunits=\"pixels\"/>\n"
				"		</IconStyle>\n"
				"	</Style>\n"
				"	<Placemark>\n"
				"		<name>%s</name>\n"
				"		<styleUrl>#msn_ylw-pushpin</styleUrl>\n"
				"		<LineString>\n"
				"			<tessellate>1</tessellate>\n"
				"			<coordinates>\n";


//*****************************************************************************
//*
//*		DegreeToRadiant
//*
//*****************************************************************************
//	Grade in Radianten umrechen
static inline double DegreeToRadiant(double dValue)
{
return dValue*0.017453292519943295769236907684886;
}


//*****************************************************************************
//*
//*		RadiantToDegree
//*
//*****************************************************************************
//	Radianten in Grade umrechen
static inline double RadiantToDegree(double dValue)
{
return dValue*57.295779513082320876798154814105;
}

//*****************************************************************************
//*
//*		DistanceOfPoints
//*
//*****************************************************************************
//	Berechnet den Abstand zweier Punkte auf der Erde. (6317 km mittlerer Radius)
//	dLongitude1		: Lngengrad  des ersten  Ortes in Radianten (X-Winkel)
//	dLatitude1		: Breitengrad des ersten  Ortes in Radianten (Y-Winkel)
//	dLongitude2		: Lngengrad  des zweiten Ortes in Radianten (X-Winkel)
//	dLatitude2		: Breitengrad des zweiten Ortes in Radianten (Y-Winkel)
//	Ergibt den Abstand der Punkte in Metern
double DistanceOfPoints(double dLongitude1,double dLatitude1,double dLongitude2,double dLatitude2)
{
double		dVal1;
double		dVal2;
double		dVal3;


	dVal1	= sin(dLatitude1)*sin(dLatitude2);
	dVal2	= cos(dLatitude1)*cos(dLatitude2)*cos(dLongitude2-dLongitude1);
	dVal3	= acos(dVal1+dVal2);


return dVal3*EARTH_RADIUS;
}

//*****************************************************************************
//*
//*		DistanceOfPointsEx
//*
//*****************************************************************************
//	Berechnet den Abstand zweier Punkte auf der Erde mittels WGS84-Ellipsoid.
//	dLongitude1		: Lngengrad  des ersten  Ortes in Radianten (X-Winkel)
//	dLatitude1		: Breitengrad des ersten  Ortes in Radianten (Y-Winkel)
//	dLongitude2		: Lngengrad  des zweiten Ortes in Radianten (X-Winkel)
//	dLatitude2		: Breitengrad des zweiten Ortes in Radianten (Y-Winkel)
//	Ergibt den Abstand der Punkte in Metern
double DistanceOfPointsEx(double dLongitude1,double dLatitude1,double dLongitude2,double dLatitude2)
{
#define 	FACT_F	(1.0/298.527223563)					// Abplattung der Erde
#define 	FACT_A	(6378137.0)							// Erdradius am quator
double		dVal1;
double		dVal2;
double		dVal3;
double		dVal4;
double		dVal5;
double		dVal6;
double		dValF;
double		dValG;
double		dValL;
double		dValS;
double		dValC;
double		dValW;
double		dValD;
double		dValR;


	dVal1	= FACT_F;
	dVal2	= FACT_A;

	dValF	= (dLatitude1 +dLatitude2 )*0.5;
	dValG	= (dLatitude1 -dLatitude2 )*0.5;
	dValL	= (dLongitude1-dLongitude2)*0.5;

	dVal1	= sin(dValL);	dVal1 *= dVal1;
	dVal2	= cos(dValL);	dVal2 *= dVal2;
	dVal3	= sin(dValG);	dVal3 *= dVal3;
	dVal4	= cos(dValG);	dVal4 *= dVal4;
	dVal5	= sin(dValF);	dVal5 *= dVal5;
	dVal6	= cos(dValF);	dVal6 *= dVal6;

	dValS	= dVal3*dVal2 + dVal6*dVal1;
	dValC	= dVal4*dVal2 + dVal5*dVal1;
	dValW	= atan(sqrt(dValS/dValC));
	dValD	= 2.0*dValW*FACT_A;

	if(dValD<0.00000000001)return 0;

	dValR	= sqrt(dValS*dValC)/dValW;
	dVal1	= (3.00*dValR-1.00)/dValC*0.5;
	dVal2	= (3.00*dValR+1.00)/dValS*0.5;



return dValD*(1.0 + FACT_F*dVal1*dVal5*dVal4 - FACT_F*dVal2*dVal6*dVal3);
}

//*****************************************************************************
//*
//*		CoordDelta
//*
//*****************************************************************************
//	Berechnet die Koordinate eines Verschubpunktes. (nahe den Polen ungenau)
//	dPosX/Y		: Ist die Ortskoortinate eines Punktes in Grad.
//	dDeltaX/Y	: Ist der der Verschub in Metern
//	dCoordX/Y	: Hier wird die Koordinate des Verschubpunktes gespeichert.
void CoordDelta(double dPosX,double dPosY,double dDeltaX,double dDeltaY,double &dCoordX,double &dCoordY)
{
#define 	RADIUS_R	(6378137.0)						// Erdradius am quator
#define 	RADIUS_Q	(6356752.0)						// Erdradius am Poll
double		dAngelY;
double		dAngelX;
double		dAxisZ;
double		dAxisX;
double		dTempZ;
double		dTempY;
double		dTempX;
double		dValZ;
double		dValY;
double		dValX;
double		dSinY;
double		dSinX;
double		dCosY;
double		dCosX;
double		dSubX;


	dAngelX	= dDeltaX*(2.0/(RADIUS_R));					// Drechung aus Verschub berechnen
	dAngelY = dDeltaY*(2.0/(RADIUS_R+RADIUS_Q));

	dValX	= DegreeToRadiant(dPosX);
	dValY	= DegreeToRadiant(dPosY);
	dValY  += dAngelY;

	dSinY	= sin(dValY);								// 3D Koordinate in XZ Ebene berechnen
	dCosY	= cos(dValY);
	dValX	= dCosY;
	dValZ	= dSinY;

	dAxisX	=-dSinY;
	dAxisZ	= dCosY;

	dSinX	= sin(dAngelX);								// Drehachse berechnen
	dCosX	= cos(dAngelX);
	dSubX	= 1.0-dCosX;

	dTempX	= (dSubX*dAxisX*dAxisX+dCosX)*dValX;		// 3D Koordinate um Drehachse rotieren
	dTempX += (dSubX*dAxisX*dAxisZ      )*dValZ;
	dTempY	= (dSinX       *dAxisZ      )*dValX;
	dTempY -= (dSinX*dAxisX             )*dValZ;
	dTempZ	= (dSubX*dAxisX*dAxisZ      )*dValX;
	dTempZ += (dSubX*dAxisZ*dAxisZ+dCosX)*dValZ;

	dCoordX	= RadiantToDegree(atan2(dTempY,dTempX))+dPosX;
	dCoordY	= RadiantToDegree(asin (dTempZ));


}


//*****************************************************************************
//*
//*		CoordDeltaEx
//*
//*****************************************************************************
//	Berechnet die Koordinate eines Verschubpunktes mittels WGS84-Ellipsoid.
//	dPosX/Y		: Ist die Ortskoortinate eines Punktes in Grad.
//	dDeltaX/Y	: Ist der der Verschub in Metern
//	dCoordX/Y	: Hier wird die Koordinate des Verschubpunktes gespeichert.
void CoordDeltaEx(double dPosX,double dPosY,double dDeltaX,double dDeltaY,double &dCoordX,double &dCoordY)
{
#define 	FACT_R	(6378137.0)							// Erdradius am quator
#define 	FACT_Q	(6356752.0)							// Erdradius am Poll
double		dRadius;
double		dAngelY;
double		dAngelX;
double		dValue;
double		dAxisZ;
double		dAxisX;
double		dTempZ;
double		dTempY;
double		dTempX;
double		dValZ;
double		dValY;
double		dValX;
double		dSinY;
double		dSinX;
double		dCosY;
double		dCosX;
double		dSubX;


	dAngelY = dDeltaY*(180.0/((FACT_R+FACT_Q)*PI));
	dValue	= fmod(dPosY+dAngelY,90.0)*(1.0/90.0);

	if(dValue<0.0)dValue+=1.0;

	dRadius	= FACT_R*(1.0-dValue) + FACT_Q*dValue;		// Radiaus fr Lngengrad

	dAngelX	= dDeltaX*(1.0/FACT_R );					// Drechung aus Verschub berechnen
	dAngelY = dDeltaY/dRadius;

	dValX	= DegreeToRadiant(dPosX);
	dValY	= DegreeToRadiant(dPosY);
	dValY  += dAngelY;

	dSinY	= sin(dValY);								// 3D Koordinate in XZ Ebene berechnen
	dCosY	= cos(dValY);
	dValX	= dCosY*FACT_R;
	dValZ	= dSinY*FACT_Q;

	dAxisX	=-dSinY;
	dAxisZ	= dCosY;

	dSinX	= sin(dAngelX);								// Drehachse berechnen
	dCosX	= cos(dAngelX);
	dSubX	= 1.0-dCosX;

	dTempX	= (dSubX*dAxisX*dAxisX+dCosX)*dValX;		// 3D Koordinate um Drehachse rotieren
	dTempX += (dSubX*dAxisX*dAxisZ      )*dValZ;
	dTempY	= (dSinX       *dAxisZ      )*dValX;
	dTempY -= (dSinX*dAxisX             )*dValZ;
	dTempZ	= (dSubX*dAxisX*dAxisZ      )*dValX;
	dTempZ += (dSubX*dAxisZ*dAxisZ+dCosX)*dValZ;

	dCoordX	= RadiantToDegree(atan2(dTempY,dTempX))+dPosX;
	dCoordY	= RadiantToDegree(asin (dTempZ*(2.0/(FACT_R+FACT_Q))));


}


//*****************************************************************************
//*
//*		CoordToMercator
//*
//*****************************************************************************
//	Wandelt eine Ortskoordinate in eine X/Y-Position der Mercator-Projektion um.
//	dLatitude		: Breitengrad des Ortes in Radianten (Y-Winkel)
//	Ergibt die Y-Position in der Mercator-Projektion.
//						+/-1.4835298641951802 bei 85 in Radianten
double CoordToMercator(double dLatitude)
{
double		dVal;


	dVal	= tan(dLatitude);


return log(dVal+sqrt(dVal*dVal+1.0));					// = asinh(dVal)
}

//*****************************************************************************
//*
//*		MercatorToCoord
//*
//*****************************************************************************
//	Wandelt eine Y-Position der Mercator-Projektion in den Breitengrad um.
//	dPosY			: Ist die Y-Position der Mercator-Projektion
//						+/-1.4835298641951802 bei 85 in Radianten
//	Ergibt den Breitengrad  des Ortes in Radianten (Y-Winkel)
double MercatorToCoord(double dPosY)
{
double		dVal;


	dVal = sinh(dPosY);


return atan(dVal);
}


//*****************************************************************************
//*
//*		CoordToPixel
//*
//*****************************************************************************
//	Brechnet die Pixelposition in einem referenzierten Bild aus X/Y-Koordinaten.
//	Das Bild muss eine Mercator-Projektion sein.
//	dPosX		: Ist der Lngengrad  (-180 bis +180)
//	dPosY		: Ist der Breitengrad ( -90 bis  +90)
//	dPixelX/Y	: Hier wird die Pixelposition im Bild gespeichert
//					0/0 = links oben	+N/+M = rechts unten
//	pRefValues	: Sind die Referenzwerte des linken oberen Punktes im Bild
//					[0]	= Grad     pro Pixel in X-Richtung
//					[1]	= Mercator pro Pixel in Y-Richtung
//					[2]	= X-Koordinate des linken oberen Punktes in Grad
//					[3]	= Y-Koordinate des linken oberen Punktes in Grad
//					[4]	= Y-Mercator-Wert des linken oberen Punktes
void CoordToPixel(double dPosX,double dPosY,double &dPixelX,double &dPixelY,const double *pRefValues)
{
double	dValY;


	dPixelX	= (dPosX-pRefValues[2])/pRefValues[0];
	dValY	= CoordToMercator(DegreeToRadiant(dPosY));
	dPixelY	= (pRefValues[4]-dValY)/pRefValues[1];

}

//*****************************************************************************
//*
//*		PixelToCoord
//*
//*****************************************************************************
//	Brechnet die X/Y-Koordinaten in einem referenzierten Bild aus der Pixelposition.
//	Das Bild muss eine Mercator-Projektion sein.
//	dPosX		: Hier wird der Lngengrad  gespeichert (-180 bis +180)
//	dPosY		: Hier wird der Breitengrad gespeichert ( -90 bis  +90)
//	dPixelX/Y	: Ist die Pixelposition im Bild gespeichert
//					0/0 = links oben	+N/+M = rechts unten
//	pRefValues	: Sind die Referenzwerte des linken oberen Punktes im Bild
//					[0]	= Grad     pro Pixel in X-Richtung
//					[1]	= Mercator pro Pixel in Y-Richtung
//					[2]	= X-Koordinate des linken oberen Punktes in Grad
//					[3]	= Y-Koordinate des linken oberen Punktes in Grad
//					[4]	= Y-Mercator-Wert des linken oberen Punktes
void PixelToCoord(double &dPosX,double &dPosY,double dPixelX,double dPixelY,const double *pRefValues)
{
double	dValX;
double	dValY;


	dValX	= (dPixelX*pRefValues[0])+pRefValues[2];
	dValY	= CoordToMercator(DegreeToRadiant(pRefValues[3]));
	dValY  -= (dPixelY*pRefValues[1]);
	dPosY	= RadiantToDegree(MercatorToCoord(dValY));

	while(dValX<-180.0)dValX+=360.0;					// X-Koordinate anpassen
	while(dValX> 180.0)dValX-=360.0;

	dPosX	= dValX;

}

//*****************************************************************************
//*
//*		DecodeMercatorRef
//*
//*****************************************************************************
//	Wantelt einen Text in die Mercator-Referenzwerte eines Bildes um.
//	pComment	: Ist der Text mit den Referenzwerten.
//	pRefValues	: Sind die Referenzwerte des linken oberen Punktes im Bild
//					[0]	= Grad     pro Pixel in X-Richtung
//					[1]	= Mercator pro Pixel in Y-Richtung
//					[2]	= X-Koordinate des linken oberen Punktes in Grad
//					[3]	= Y-Koordinate des linken oberen Punktes in Grad
//					[4]	= Y-Mercator-Wert des linken oberen Punktes
//	Ergibt 1 wenn die Referenzwerte dekodiert wurden.
int DecodeMercatorRef(const char *pComment,double *pRefValues)
{
int		iOff;
int		iPos;


	if(!pComment || memcmp(pComment,"GMREF:",6))
		{
		memset(pRefValues,0,sizeof(double)*5);
		return 0;
		}


	for(iPos=0,iOff=6;iPos<4;iPos++)
		{
		iOff            += strwlen (pComment+iOff,0xA0);
		pRefValues[iPos] = atof    (pComment+iOff);
		iOff            += strcwlen(pComment+iOff,0xA0);
		}

	pRefValues[4] = CoordToMercator(DegreeToRadiant(pRefValues[3]));


return 1;
}


//*****************************************************************************
//*
//*		EncodeMercatorRef
//*
//*****************************************************************************
//	Die Mercator-Referenzwerte eines Bildes in einen Text um.
//	pComment	: Ist ein Zeiger auf den Textpuffer (Gre 96 Byte).
//	dCoordX/Y	: Ist die Koordinate des linken oberen Punktes in Grad
//	dResX		: Grad     pro Pixel in X-Richtung
//	dResY		: Mercator pro Pixel in Y-Richtung
//	Ergibt die Lnge des erzeugten Textes.
int EncodeMercatorRef(char *pComment,double dCoordX,double dCoordY,double dResX,double dResY)
{
int		iLen;


	while(dCoordX<-180.0)dCoordX+=360.0;				// X-Koordinate anpassen
	while(dCoordX> 180.0)dCoordX-=360.0;

	iLen = sprintf(pComment,"GMREF: %2.15f %2.15f %4.9f %3.9f",dResX,dResY,dCoordX,dCoordY);


return iLen;
}

//*****************************************************************************
//*
//*		EncodeMercatorRef
//*
//*****************************************************************************
//	Die Mercator-Referenzwerte eines Bildes in einen Text um.
//	pComment	: Ist ein Zeiger auf den Textpuffer (Gre 96 Byte).
//	dCoordX/Y	: Ist die Koordinate des linken oberen Punktes in Grad
//	iZoom		: Ist der Open-Steet-Map Zoom-Level
//	Ergibt die Lnge des erzeugten Textes.
int EncodeMercatorRef(char *pComment,double dCoordX,double dCoordY,int iZoom)
{
double	dResX;
double	dResY;


	dResX	= 360.0/256.0;
	dResX  /= 1<<iZoom;

	dResY	= PI/128.0;
	dResY  /= 1<<iZoom;


return EncodeMercatorRef(pComment,dCoordX,dCoordY,dResX,dResY);
}

//*****************************************************************************
//*
//*		EncodeMercatorRef
//*
//*****************************************************************************
//	Die Mercator-Referenzwerte eines Bildes in einen Text um.
//	pComment	: Ist ein Zeiger auf den Textpuffer (Gre 96 Byte).
//	dCoordX/Y	: Ist die Koordinate des linken oberen Punktes in Grad
//	iZoom		: Ist der Open-Steet-Map Zoom-Level
//	iPosX/Y		: Ist die Koordinate des Punktes im Bild
//	Ergibt die Lnge des erzeugten Textes.
int EncodeMercatorRef(char *pComment,double dCoordX,double dCoordY,int iZoom,int iPosX,int iPosY)
{
double	dResX;
double	dResY;


	dResX	= 360.0/256.0;
	dResX  /= 1<<iZoom;

	dResY	= PI/128.0;
	dResY  /= 1<<iZoom;


return EncodeMercatorRef(pComment,dCoordX,dCoordY,dResX,dResY,iPosX,iPosY);
}

//*****************************************************************************
//*
//*		EncodeMercatorRef
//*
//*****************************************************************************
//	Die Mercator-Referenzwerte eines Bildes in einen Text um.
//	pComment	: Ist ein Zeiger auf den Textpuffer (Gre 96 Byte).
//	pRefValues	: Sind die Referenzwerte eines Punktes im Bild
//					[0]	= Grad     pro Pixel in X-Richtung
//					[1]	= Mercator pro Pixel in Y-Richtung
//					[2]	= X-Koordinate des linken oberen Punktes in Grad
//					[3]	= Y-Koordinate des linken oberen Punktes in Grad
//					[4]	= Y-Mercator-Wert des linken oberen Punktes
//	iPosX/Y		: Ist die Koordinate des Punktes im Bild
//	Ergibt die Lnge des erzeugten Textes.
int EncodeMercatorRef(char *pComment,const double *pRefValues,int iPosX,int iPosY)
{
double		dCoordX;
double		dCoordY;


	PixelToCoord(dCoordX,dCoordY,-iPosX,-iPosY,pRefValues);


return EncodeMercatorRef(pComment,dCoordX,dCoordY,pRefValues[0],pRefValues[1]);
}

//*****************************************************************************
//*
//*		EncodeMercatorRef
//*
//*****************************************************************************
//	Die Mercator-Referenzwerte eines Bildes in einen Text um.
//	pComment	: Ist ein Zeiger auf den Textpuffer (Gre 96 Byte).
//	dCoordX/Y	: Ist die Koordinate eines Punktes in Grad
//	dResX		: Grad     pro Pixel in X-Richtung
//	dResY		: Mercator pro Pixel in Y-Richtung
//	iPosX/Y		: Ist die Koordinate des Punktes im Bild
//	Ergibt die Lnge des erzeugten Textes.
int EncodeMercatorRef(char *pComment,double dCoordX,double dCoordY,double dResX,double dResY,int iPosX,int iPosY)
{
double	aRefValues[5];


	aRefValues[0]	= dResX;
	aRefValues[1]	= dResY;
	aRefValues[2]	= dCoordX;
	aRefValues[3]	= dCoordY;
	aRefValues[4]	= CoordToMercator(DegreeToRadiant(aRefValues[3]));


return EncodeMercatorRef(pComment,aRefValues,iPosX,iPosY);
}


//*****************************************************************************
//*
//*		ReadGpxFile
//*
//*****************************************************************************
//	Ld die Koordinaten aus einer *.gpx Datei
//	pFilename	: Name der Datei mit den Koordinaten
//	pRoutePtr	: Hier werden die Koordinaten gespeichert
//				  Dieser Pointer muss mit delete wieder freigegeben werden.
//	pBuffer		: Hier wird der Name der Route gespeichert (maximal 128 Byte)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Die Datei kann nicht geffnet werden
//					-2 Die Datei hat ein falscher Format
//					-3 Zu wenig Speicher
//					-4 Es wurden keine Koordinaten gefunden
int	ReadGpxFile(const char *pFilename,RouteData **pRoutePtr,char *pBuffer)
{
RouteData  *pRoute;
FILE	   *pFile;
char	   *pTag;
char	   *pData;
char	   *pName;
time_t		iTime;
int			iSize;
int			iFrom;
int			iPos;
int			iNum;
int			iMax;
int			iLen;
int			iOff;
int			iLon;
int			iLat;
int			iTim;



	*pRoutePtr = 0;
	if(pBuffer)*pBuffer=0;


		iSize = LengthFile(pFilename);
	if( iSize<=0)return -1;

		pFile = fopen(pFilename,"rb");
	if(!pFile)return -1;


		pData = new char[iSize+1];
	if(!pData)return -3;

	fread(pData,iSize,1,pFile);
	fclose(pFile);
	pData[iSize] = 0;

	   iPos = strifind(pData,"<gpx");
	if(iPos<0)
		{
		delete pData;
		return -2;
		}

	iFrom = iPos;

	if(pBuffer)
		{
		   iPos=strifind(pData,"<name>");				// Namen der Route suchen
		if(iPos>0)
			{
			iPos += 6;
			iPos += strwlen(pData+iPos,0xE0);

			if(!memcmp(pData+iPos,"<![CDATA[",9))iPos+=9;
			iLen  = strwlen(pData+iPos,0x064000FF,127);
			strlcpy(pBuffer,pData+iPos,128,iLen);
			}
		}


	           pTag="<trkpt";iPos = strifind(pData+iFrom,pTag);pName="</trkpt";
	if(iPos<0){pTag="<rtept";iPos = strifind(pData+iFrom,pTag);pName="</rtept";}
	if(iPos<0){pTag="<wpt"  ;iPos = strifind(pData+iFrom,pTag);pName="</wpt"  ;}
	if(iPos<0)
		{
		delete pData;
		return -2;
		}


	iPos = iFrom;
	iLen = strlen(pName)-2;

	for(iMax=0;;)											// Zhle die Eintrge
		{
		if(pData[iPos]==0)break;
		if(pData[iPos]!='<'){iPos++;continue;}

		iPos++;

		if(strnicmp(pData+iPos,pName+2,iLen))continue;
		if(strwchar(pData[iPos+iLen],0x1F  ))continue;

		iPos += iLen;
		iOff  = strifind(pData+iPos,pName);
		iLon  = strlifind(pData+iPos,"lon=",iOff);
		iLat  = strlifind(pData+iPos,"lat=",iOff);

		if(iLon<iOff && iLat<iOff)
			{
			iMax++;
			}

		iPos += iOff;
		}

	if(iMax<=1)
		{
		delete pData;
		return -4;
		}

		pRoute	= new RouteData[iMax];
	if(!pRoute)
		{
		delete pData;
		return -3;
		}

	memset(pRoute,0,sizeof(RouteData)*iMax);
	iPos = 0;

	for(iNum=0;iNum<iMax;)									// Eintrge einlesen
		{
		if(pData[iPos]==0)break;
		if(pData[iPos]!='<'){iPos++;continue;}

		iPos++;

		if(strnicmp(pData+iPos,pName+2,iLen))continue;
		if(strwchar(pData[iPos+iLen],0x1F  ))continue;

		iPos += iLen;
		iOff  = strifind (pData+iPos,pName);
		iLon  = strlifind(pData+iPos,"lon=",iOff);
		iLat  = strlifind(pData+iPos,"lat=",iOff);

		if(iLon<iOff && iLat<iOff)
			{
			iLon+=4;
			iLat+=4;
			iLon+=strwlen(pData+iPos+iLon,0x01A0);
			iLat+=strwlen(pData+iPos+iLat,0x01A0);
			pRoute[iNum].dCoordX = strtof(pData+iPos+iLon);
			pRoute[iNum].dCoordY = strtof(pData+iPos+iLat);

			   iTim = strlifind(pData+iPos,"<time>",iOff);
			if(iTim<iOff)
				{
				iTime = strtoutc(pData+iPos+iTim+6,STR_DATE|STR_TIME|STR_GLOBAL);

				if(iTime>0)
					{
					pRoute[iNum].dGlobalTime   = (double)iTime;
					pRoute[iNum].dGlobalTime  /= 86400.0;
					pRoute[iNum].dGlobalTime  += 25569.0;
					}
				}

			iNum++;
			}

		iPos += iOff;
		}



	delete pData;
	*pRoutePtr = pRoute;



return iNum;
}

//*****************************************************************************
//*
//*		ReadKmlFile
//*
//*****************************************************************************
//	Ld die Koordinaten aus einer *.kml Datei
//	pFilename	: Name der Datei mit den Koordinaten
//	pRoutePtr	: Hier werden die Koordinaten gespeichert
//				  Dieser Pointer muss mit delete wieder freigegeben werden.
//	pBuffer		: Hier wird der Name der Route gespeichert (maximal 128 Byte)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Die Datei kann nicht geffnet werden
//					-2 Die Datei hat ein falscher Format
//					-3 Zu wenig Speicher
//					-4 Es wurden keine Koordinaten gefunden
int	ReadKmlFile(const char *pFilename,RouteData **pRoutePtr,char *pBuffer)
{
RouteData  *pRoute;
FILE	   *pFile;
char	   *pData;
time_t		iTime;
int			iStart;
int			iSize;
int			iFrom;
int			iPos;
int			iNum;
int			iLen;
int			iMax;
int			iOff;
int			iAdd;


	*pRoutePtr = 0;
	if(pBuffer)*pBuffer=0;


		iSize = LengthFile(pFilename);
	if( iSize<=0)return -1;

		pFile = fopen(pFilename,"rb");
	if(!pFile)return -1;


		pData = new char[iSize+1];
	if(!pData)return -3;

	fread(pData,iSize,1,pFile);
	fclose(pFile);
	pData[iSize] = 0;

	          iPos = strifind(pData,"MultiGeometry");
	if(iPos<0)iPos = strifind(pData,"GeometryCollection");
	if(iPos<0)iPos = strifind(pData,"LineString");
	if(iPos<0)
		{
		delete pData;
		return -2;
		}

	iFrom = iPos;
	iOff  = 0;

	          iPos=strifind(pData,"<Placemark");		// Namen der Route suchen
	if(iPos>0)iOff=strifind(pData+iPos,"<name>");
	if(iPos>0 && iOff>0 && pBuffer)
		{
		iPos += iOff+6;
		iPos += strwlen(pData+iPos,0xE0);

		if(!memcmp(pData+iPos,"<![CDATA[",9))
			{
			iPos += 9;
			iLen  = iSize-iPos;
			if(iLen>100)iLen=100;
			iLen  = strlfind(pData+iPos,"]]",iLen);
			}
		else{
			iLen  = strwlen(pData+iPos,0x324000FF,127);
			}

		strfromutf8(pBuffer,pData+iPos,128,iLen);
		}


	   iStart = strifind(pData,"<TimeStamp");
	if(iStart>0)										// Route ber Zeitstempel laden
		{
		iPos = iStart;

		for(iMax=0;;iMax++)								// Zhle Zeitstempel
			{
			   iOff = strifind(pData+iPos,"<TimeStamp");
			if(iOff<0)break;

			iPos += iOff;

			   iLen = strifind(pData+iPos,"</TimeStamp");
			if(iLen<0)break;

			   iOff = strifind(pData+iPos,"</Placemark");
			if(iOff<=iLen)
				{
				iPos += iLen;
				iMax -= 1;
				continue;
				}

			iLen = iOff;

			   iOff = strlifind(pData+iPos,"<when",iLen);
			if(iOff<0)
				{
				iPos += iLen;
				iMax -= 1;
				continue;
				}

			   iOff = strlifind(pData+iPos,"<coordinates",iLen);
			if(iOff<0)
				{
				iPos += iLen;
				iMax -= 1;
				continue;
				}

			iPos += iLen;
			}

		if(iMax<16)goto NoStamps;

			pRoute	= new RouteData[iMax];
		if(!pRoute)
			{
			delete pData;
			return -3;
			}

		memset(pRoute,0,sizeof(RouteData)*iMax);
		iPos   = iStart;

		for(iNum=0;iNum<iMax;iNum++)					// Zeitstempel einlesen
			{
			   iOff = strifind(pData+iPos,"<TimeStamp");
			if(iOff<0)break;

			iPos += iOff;

			   iLen = strifind(pData+iPos,"</TimeStamp");
			if(iLen<0)break;

			   iOff = strifind(pData+iPos,"</Placemark");
			if(iOff<=iLen)
				{
				iPos += iLen;
				iMax -= 1;
				continue;
				}

			iLen = iOff;

			   iOff = strlifind(pData+iPos,"<when",iLen);
			if(iOff<0)
				{
				iPos += iLen;
				iMax -= 1;
				continue;
				}

			iOff += strlchar(pData+iPos+iOff,'>',iLen-iOff);
			iTime = strtoutc(pData+iPos+iOff,STR_DATE|STR_TIME|STR_GLOBAL);

			if(iTime>0)
				{
				pRoute[iNum].dGlobalTime   = (double)iTime;
				pRoute[iNum].dGlobalTime  /= 86400.0;
				pRoute[iNum].dGlobalTime  += 25569.0;
				}


			   iOff = strlifind(pData+iPos,"<coordinates",iLen);
			if(iOff<0)
				{
				iPos += iLen;
				iMax -= 1;
				continue;
				}

			iOff += strlchar(pData+iPos+iOff,'>',iLen-iOff);

			pRoute[iNum].dCoordX = strtof(pData+iPos+iOff,&iAdd);
			iOff += iAdd;
			iOff += strwlen(pData+iPos+iOff,0xE0);
			if(pData[iPos+iOff]==',')iOff++;
			pRoute[iNum].dCoordY = strtof(pData+iPos+iOff);

			iPos += iLen;
			}

		delete pData;
		*pRoutePtr = pRoute;

		return iMax;
		}

//*****************************************************************************
NoStamps:

	   iPos = strifind(pData+iFrom,"coordinates");
	if(iPos<0)
		{
		delete pData;
		return -2;
		}


	iPos  += 11;
	iFrom += iPos;

	for(iPos=iFrom;iPos<iSize;iPos++)						// Vor auf erste Komma
		{
		if(pData[iPos]>='0')
		if(pData[iPos]<='9')break;
		if(pData[iPos]=='.')break;
		}

	for(;iPos<iSize;iPos++)
		{
		if(pData[iPos]<'0')break;
		if(pData[iPos]>'9')break;
		}

	if(pData[iPos]==',')									// Sind die Zahlen mit ','
		{
		for(iNum=0;iPos<iSize;iPos++)
			{
			if(pData[iPos]==' ')
				{
				iNum=0;
				continue;
				}

			if(pData[iPos]=='.')
				{
				iNum++;
				iNum&=1;
				continue;
				}

			if(pData[iPos]!=',')continue;

			iNum++;
			if(!(iNum&1))continue;

			pData[iPos] = '.';
			}
		}


	for(iPos=iFrom;iPos<iSize;iPos++)
		{
		if(pData[iPos]==',' )pData[iPos]=' ';
		if(pData[iPos]=='<' )pData[iPos]=' ';
		if(pData[iPos]=='>' )pData[iPos]=' ';
		if(pData[iPos]=='\r')pData[iPos]=' ';
		if(pData[iPos]=='\n')pData[iPos]=' ';
		}


	iPos = iFrom;

	for(iMax=0;;iMax++)										// Zhle die Eintrge
		{
		iPos += strwlen(pData+iPos,0xE0);

		if(pData[iPos]!='+')
		if(pData[iPos]!='-')
		if(pData[iPos]!='.')
		if(pData[iPos]< '0' || pData[iPos]> '9')
			{
			iNum  = memycmp("/coordinates\\_/LineString\\_",pData+iPos,iSize-iPos,0x05);
			memset(pData+iPos,' ',iNum);
			iPos += iNum;

			/*
			iNum  = strifind(pData+iPos,"coordinates");

			if(iNum>0 && !strwchar(pData[iPos+iNum+21],0x1F))
				{
				iNum += 12;
				iNum += strlchar(pData+iPos+iNum,'>');
				memset(pData+iPos,' ',iNum);
				iPos += iNum;
				}
			*/
			while(iPos<iSize)								// Vor auf den nchsten Bereich
				{
				iNum  = strifind(pData+iPos,"LineString");
				if(iNum<0)iNum=iSize-iPos;
				memset(pData+iPos,' ',iNum);
				iPos += iNum;

				if(!pData[iPos])break;
				iNum  = memycmp("LineString\\_coordinates\\_",pData+iPos,iSize-iPos,0x05);
				if(iNum==0)
					{
					iNum  = memycmp("LineString\\_*altitudeMode*/altitudeMode\\_coordinates\\_",pData+iPos,iSize-iPos,0x05);
					}

				if(iNum)
					{
					memset(pData+iPos,' ',iNum);
					iPos += iNum;
					break;
					}

				pData[iPos]=' ';
				iPos++;
				}

			if(iPos>=iSize)break;
			iMax--;
			continue;
			}

		iPos += strcwlen(pData+iPos,0xE0);
		}

	if(iMax<=1)
		{
		delete pData;
		return -4;
		}

	iMax  /= 3;

		pRoute	= new RouteData[iMax];
	if(!pRoute)
		{
		delete pData;
		return -3;
		}

	memset(pRoute,0,sizeof(RouteData)*iMax);
	iPos   = iFrom;

	for(iNum=0;iNum<iMax;iNum++)
		{
		iPos += strwlen (pData+iPos,0xE0);
		pRoute[iNum].dCoordX = strtof(pData+iPos);
		iPos += strcwlen(pData+iPos,0xE0);

		iPos += strwlen (pData+iPos,0xE0);
		pRoute[iNum].dCoordY = strtof(pData+iPos);
		iPos += strcwlen(pData+iPos,0xE0);

		iPos += strwlen (pData+iPos,0xE0);
		iPos += strcwlen(pData+iPos,0xE0);
		}


	delete pData;
	*pRoutePtr = pRoute;



return iNum;
}

//*****************************************************************************
//*
//*		ReadCsvFile
//*
//*****************************************************************************
//	Ld die Koordinaten aus einer *.csv Datei
//	pFilename	: Name der Datei mit den Koordinaten
//	pRoutePtr	: Hier werden die Koordinaten gespeichert
//				  Dieser Pointer muss mit delete wieder freigegeben werden.
//	pBuffer		: Hier wird der Name der Route gespeichert (maximal 128 Byte)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Die Datei kann nicht geffnet werden
//					-2 Die Datei hat ein falscher Format
//					-3 Zu wenig Speicher
//					-4 Es wurden keine Koordinaten gefunden
int	ReadCsvFile(const char *pFilename,RouteData **pRoutePtr,char *pBuffer)
{
char		cBuffer[128];
RouteData  *pRoute;
FILE	   *pFile;
char	   *pData;
int			iSize;
int			iPos;
int			iNum;
int			iMax;
int			iLen;
int			iOff;




	*pRoutePtr = 0;
	if(pBuffer)*pBuffer=0;


		iSize = LengthFile(pFilename);
	if( iSize<=0)return -1;

		pFile = fopen(pFilename,"rb");
	if(!pFile)return -1;


		pData = new char[iSize+1];
	if(!pData)return -3;

	fread(pData,iSize,1,pFile);
	fclose(pFile);
	pData[iSize] = 0;


	for(iPos=0,iMax=0;iPos<iSize;iPos+=iLen)				// Zhle die Eintrge
		{
		iLen = strlchar(pData+iPos,'\n');

		if(pData[iPos]>='0' && pData[iPos]<='9')
			{
			iMax++;
			continue;
			}

		if(pData[iPos]=='+' || pData[iPos]=='-' || pData[iPos]=='.')
			{
			iMax++;
			continue;
			}
		}

	if(iMax<=1)
		{
		delete pData;
		return -4;
		}

		pRoute	= new RouteData[iMax];
	if(!pRoute)
		{
		delete pData;
		return -3;
		}

	memset(pRoute,0,sizeof(RouteData)*iMax);


	for(iPos=0,iNum=0;iPos<iSize;iPos+=iLen)				// Eintrge laden
		{
		iLen = strlchar(pData+iPos,'\n');


		if(iNum==0)
			{
			iOff  = iPos;
			iOff += strfromcsv(pData+iOff,cBuffer,sizeof(cBuffer));
			iOff += strfromcsv(pData+iOff,cBuffer,sizeof(cBuffer));
			iOff += strfromcsv(pData+iOff,cBuffer,sizeof(cBuffer));
			iOff += strfromcsv(pData+iOff,cBuffer,sizeof(cBuffer));
			if(pBuffer)strlcpy(pBuffer,cBuffer,128);
			}


		if(pData[iPos]< '0' && pData[iPos]>'9')
		if(pData[iPos]!='+' && pData[iPos]!='-' && pData[iPos]!='.')
			{
			continue;
			}

		iOff  = iPos;
		iOff += strfromcsv(pData+iOff,cBuffer,sizeof(cBuffer));if(!cBuffer[0])continue;
		pRoute[iNum].dCoordX	= strtof(cBuffer,0,0x07);
		iOff += strfromcsv(pData+iOff,cBuffer,sizeof(cBuffer));
		pRoute[iNum].dCoordY	= strtof(cBuffer,0,0x07);
		iOff += strfromcsv(pData+iOff,cBuffer,sizeof(cBuffer));
		pRoute[iNum].dTime		= strtof(cBuffer,0,0x07);

		if(pRoute[iNum].dTime!=0.0)
			{
			pRoute[iNum].dGlobalTime  /= 86400.0;
			pRoute[iNum].dGlobalTime  += 25569.0;
			}

		iNum++;
		if(iNum>=iMax)break;
		}



	delete pData;
	*pRoutePtr = pRoute;



return iNum;
}


//*****************************************************************************
//*
//*		ReadKmzFile
//*
//*****************************************************************************
//	Ld die Koordinaten aus einer *.kmz Datei (komprimierte kml Datei)
//	pFilename	: Name der Datei mit den Koordinaten
//	pRoutePtr	: Hier werden die Koordinaten gespeichert
//				  Dieser Pointer muss mit delete wieder freigegeben werden.
//	pBuffer		: Hier wird der Name der Route gespeichert (maximal 128 Byte)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Die Datei kann nicht geffnet werden
//					-2 Die Datei hat ein falscher Format
//					-3 Zu wenig Speicher
//					-4 Es wurden keine Koordinaten gefunden
int	ReadKmzFile(const char *pFilename,RouteData **pRoutePtr,char *pBuffer)
{
char		cTemp[256];
CZipFile	sZip;
int			iLen;
int			iRet;


	*pRoutePtr = 0;
	if(pBuffer)*pBuffer=0;


	   iLen = LengthFile(pFilename);
	if(iLen<=0)return -1;

	if(sZip.Open(pFilename)!=CAF_ERROR_OK)
		{
		return -1;
		}


	if(sZip.Count()<1)
		{
		return -2;
		}

	iLen = GetTempPath(sizeof(cTemp)-16,cTemp);
	if(iLen)iLen=AddBackSlash(cTemp,iLen);
	strcpy(cTemp+iLen,"kml_path.tmp");


	if(sZip.Extract(0,cTemp)!=CAF_ERROR_OK)
		{
		return -2;
		}

	iRet = ReadKmlFile(cTemp,pRoutePtr,pBuffer);


	RemoveFile(cTemp);


return iRet;
}


//*****************************************************************************
//*
//*		ReadTrkFile
//*
//*****************************************************************************
//	Ld die Koordinaten aus einer *.trk Datei
//	pFilename	: Name der Datei mit den Koordinaten
//	pRoutePtr	: Hier werden die Koordinaten gespeichert
//				  Dieser Pointer muss mit delete wieder freigegeben werden.
//	pBuffer		: Hier wird der Name der Route gespeichert (maximal 128 Byte)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Die Datei kann nicht geffnet werden
//					-2 Die Datei hat ein falscher Format
//					-3 Zu wenig Speicher
//					-4 Es wurden keine Koordinaten gefunden
int	ReadTrkFile(const char *pFilename,RouteData **pRoutePtr,char *pBuffer)
{
RouteData  *pRoute;
FILE	   *pFile;
char	   *pData;
time_t		iTime;
int			iSize;
int			iPos;
int			iOff;
int			iNum;
int			iMax;
int			iLen;




	*pRoutePtr = 0;
	if(pBuffer)*pBuffer=0;

		iSize = LengthFile(pFilename);
	if( iSize<=0)return -1;

		pFile = fopen(pFilename,"rb");
	if(!pFile)return -1;


		pData = new char[iSize+48];
	if(!pData)return -3;

	fread(pData,iSize,1,pFile);
	fclose(pFile);
	pData[iSize] = 0;


	if(!memcmp(pData,"FUGTRK",6))								// Binres Format
		{
		iLen = *(int*)(pData+8);

		if(iLen<16 || iLen>256)
			{
			delete pData;
			return -2;
			}

		iMax   = (iSize-iLen)/48;

			pRoute	= new RouteData[iMax];
		if(!pRoute)
			{
			delete pData;
			return -3;
			}

		memset(pRoute,0,sizeof(RouteData)*iMax);
		iPos   = iLen;

		for(iNum=0;iNum<iMax;iNum++)
			{
			pRoute[iNum].dCoordY = *(double*)(pData+iPos+24);
			pRoute[iNum].dCoordX = *(double*)(pData+iPos+32);
			iPos += 48;
			}

		*pRoutePtr = pRoute;
		delete pData;
		return iNum;
		}


//******************** ASCII Version ******************************************


	   iPos = strifind(pData,"LATITUDE");
	if(iPos<0)
		{
		delete pData;
		return -2;
		}

	   iPos = strifind(pData,"LONGITUDE");
	if(iPos<0)
		{
		delete pData;
		return -2;
		}



	if(pBuffer)												// Suche Tracknamen
		{
		iMax = iPos;

		for(iPos=0;iPos<iMax;iPos+=iLen)
			{
			iLen	= strcwlen(pData+iPos     ,0x40);
			iLen   += strwlen (pData+iPos+iLen,0x40);

			if(iLen<=0)iLen=1;


			if(pData[iPos]!='H' || !strwchar(pData[iPos+1],0xA0))
				{
				continue;
				}

			iOff  = iPos+1;
			iOff += strwlen (pData+iOff,0xA0);

			if(pData[iOff]!='T' || pData[iOff+1]!='N' || !strwchar(pData[iOff+2],0xA0))
				{
				continue;
				}

			for(iNum=0;iNum<6;iNum++)						// 6 Spalten berspringen
				{
				iOff += strcwlen(pData+iOff,0xA0);
				iOff += strwlen (pData+iOff,0xA0);
				}

			iLen = strcwlen(pData+iOff,0x40);
			strlcpy(pBuffer,pData+iPos,128,iLen);
			}
		}

	iPos = 0;

	for(iMax=0;;)											// Zhle die Eintrge
		{
		if(pData[iPos]==0)break;
		if(pData[iPos]!='\n' && pData[iPos]!='\r')
			{
			iPos++;
			continue;
			}

		iPos++;

		if(pData[iPos]!='T')continue;

		iMax++;
		}

	if(iMax<=1)
		{
		delete pData;
		return -4;
		}

		pRoute	= new RouteData[iMax];
	if(!pRoute)
		{
		delete pData;
		return -3;
		}

	memset(pRoute,0,sizeof(RouteData)*iMax);
	iPos   = 0;

	for(iNum=0;iNum<iMax;)									// Eintrge einlesen
		{
		if(pData[iPos]==0)break;
		if(pData[iPos]!='\n' && pData[iPos]!='\r')
			{
			iPos++;
			continue;
			}

		iPos++;

		if(pData[iPos]!='T')continue;

		iPos++;

		pRoute[iNum].dCoordX = strtof  (pData+iPos,&iLen);	iPos+=iLen;
		pRoute[iNum].dCoordY = strtof  (pData+iPos,&iLen);	iPos+=iLen;

		iTime = strtoutc(pData+iPos,STR_DATE|STR_TIME|STR_GLOBAL);

		if(iTime>0)
			{
			pRoute[iNum].dGlobalTime   = (double)iTime;
			pRoute[iNum].dGlobalTime  /= 86400.0;
			pRoute[iNum].dGlobalTime  += 25569.0;
			}

		iNum++;
		}


	delete pData;
	*pRoutePtr = pRoute;



return iNum;
}

//*****************************************************************************
//*
//*		ReadTkFile
//*
//*****************************************************************************
//	Ld die Koordinaten aus einer *.tk Datei
//	pFilename	: Name der Datei mit den Koordinaten
//	pRoutePtr	: Hier werden die Koordinaten gespeichert
//				  Dieser Pointer muss mit delete wieder freigegeben werden.
//	pBuffer		: Hier wird der Name der Route gespeichert (maximal 128 Byte)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Die Datei kann nicht geffnet werden
//					-2 Die Datei hat ein falscher Format
//					-3 Zu wenig Speicher
//					-4 Es wurden keine Koordinaten gefunden
int	ReadTkFile(const char *pFilename,RouteData **pRoutePtr,char *pBuffer)
{
RouteData  *pRoute;
FILE	   *pFile;
char	   *pData;
int			iSize;
int			iPos;
int			iNum;
int			iMax;
int			iLen;



	*pRoutePtr = 0;
	if(pBuffer)*pBuffer=0;


		iSize = LengthFile(pFilename);
	if( iSize<=0)return -1;

		pFile = fopen(pFilename,"rb");
	if(!pFile)return -1;


		pData = new char[iSize+1];
	if(!pData)return -3;

	fread(pData,iSize,1,pFile);
	fclose(pFile);
	pData[iSize] = 0;

	iPos = strwlen(pData,0xE0);

	if(strflen(pData+iPos)<3)
		{
		delete pData;
		return -2;
		}

	iPos = 0;

	for(iMax=0;;)											// Zhle die Eintrge
		{
		if(pData[iPos]==0)break;
		if(pData[iPos]=='\n' || pData[iPos]=='\r')
			{
			iPos++;
			continue;
			}

		iPos++;

		if(strflen(pData+iPos)<3)
			{
			continue;
			}

		iMax++;
		}

	if(iMax<=1)
		{
		delete pData;
		return -4;
		}


		pRoute	= new RouteData[iMax];
	if(!pRoute)
		{
		delete pData;
		return -3;
		}

	memset(pRoute,0,sizeof(RouteData)*iMax);
	iPos = 0;

	for(iNum=0;iNum<iMax;)									// Eintrge einlesen
		{
		if(pData[iPos]==0)break;
		if(pData[iPos]=='\n' || pData[iPos]=='\r')
			{
			iPos++;
			continue;
			}

		iPos++;

		if(strflen(pData+iPos)<3)
			{
			continue;
			}

		pRoute[iNum].dCoordY = strtof(pData+iPos,&iLen);

		iPos += iLen;
		iPos += strwlen(pData+iPos,0xA0);
		if(pData[iPos]==',')iPos++;

		pRoute[iNum].dCoordX = strtof(pData+iPos,&iLen);
		iPos += iLen;

		iNum++;
		}


	delete pData;
	*pRoutePtr = pRoute;



return iNum;
}


//*****************************************************************************
//*
//*		ReadTrackFile
//*
//*****************************************************************************
//	Ld die Koordinaten aus einer *.gpx *.klm *.trk *.tk Datei
//	pFilename	: Name der Datei mit den Koordinaten
//	pRoutePtr	: Hier werden die Koordinaten gespeichert
//				  Dieser Pointer muss mit delete wieder freigegeben werden.
//	pBuffer		: Hier wird der Name der Route gespeichert (maximal 128 Byte)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Die Datei kann nicht geffnet werden
//					-2 Die Datei hat ein falscher Format
//					-3 Zu wenig Speicher
//					-4 Es wurden keine Koordinaten gefunden
int	ReadTrackFile(const char *pFilename,RouteData **pRoutePtr,char *pBuffer)
{
char	cData[256];
int		iErr;


	if(!LoadMemory(pFilename,cData,sizeof(cData)))
		{
		return -1;
		}


	if(!memcmp(cData,"FUGTRK",6))							// Binres TK Format
		{
		return ReadTrkFile(pFilename,pRoutePtr,pBuffer);
		}

	if(!memcmp(cData,"PK",2))								// KLM als ZIP-Format
		{
		return ReadKmzFile(pFilename,pRoutePtr,pBuffer);
		}


	if(strifind(cData,"<gpx")>0)							// GPX-Format
		{
		return ReadGpxFile(pFilename,pRoutePtr,pBuffer);
		}

	if(strifind(cData,"<kml")>0)							// KML-Format
		{
		return ReadKmlFile(pFilename,pRoutePtr,pBuffer);
		}

	if(strncmp(cData,";;;",3)==0)							// CSV-Format
		{
		return ReadCsvFile(pFilename,pRoutePtr,pBuffer);
		}

	if(memycmp("\\z.\\z,\\.\\z.\\z",cData,strlen(cData),0x04))// TRK-Format
		{
		return ReadTkFile(pFilename,pRoutePtr,pBuffer);
		}



	if(CmpFileType(pFilename,"gpx"))
		{
		return ReadGpxFile(pFilename,pRoutePtr,pBuffer);
		}
	else if(CmpFileType(pFilename,"trk"))
		{
		return ReadTrkFile(pFilename,pRoutePtr,pBuffer);
		}
	else if(CmpFileType(pFilename,"tk"))
		{
		return ReadTkFile(pFilename,pRoutePtr,pBuffer);
		}
	else if(CmpFileType(pFilename,"kmz"))
		{
		return ReadKmzFile(pFilename,pRoutePtr,pBuffer);
		}
	else if(CmpFileType(pFilename,"csv"))
		{
		return ReadCsvFile(pFilename,pRoutePtr,pBuffer);
		}

	   iErr = ReadGpxFile(pFilename,pRoutePtr,pBuffer);
	if(iErr>=0)return iErr;


return ReadKmlFile(pFilename,pRoutePtr,pBuffer);
}

int	ReadTrackFile(const wchar_t *pFilename,RouteData **pRoutePtr,wchar_t *pBuffer)
{
wchar_t		cShort[MAX_PATH];
char		cPath [MAX_PATH];
char		cName [128];
int			iRet;


	MakeShortPath(cShort,pFilename,MAX_PATH);
	strawcpy(cPath,cShort,sizeof(cPath));

	iRet = ReadTrackFile(cPath,pRoutePtr,cName);

	if(pBuffer)strawcpy(pBuffer,cName,128);


return iRet;
}

//*****************************************************************************
//*
//*		SaveGpx
//*
//*****************************************************************************
//	Speichert eine Route als GPX-Datei
//	pFile		: Ist das Datei Handle zum Speichen
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	pName		: Ist der Name der Route (NULL ist auch erlaubt)
//	Ergibt 0 wenn die Datei gespeichert wurde bzw. eine Fehlernummer kleiner 0
//					-1 Die Datei konnte nicht erstellt werden
//					-2 Keine Routendaten vorhanden
static int SaveGpx(FILE *pFile,const RouteData *pRoute,int iCount,const char *pName)
{
char		cDate[32];
char		cTime[32];
time_t		iTime;
int			i;


	if(pRoute[0].dGlobalTime>1.0)
		{
		iTime = (time_t)(pRoute[0].dGlobalTime*86400.0);
		}
	else{
		iTime = time(0);
		}

	strtimeset(cDate,iTime,STR_DATE_ISO|STR_LONG);
	strtimeset(cTime,iTime,STR_TIME|STR_SEC);
	strcut(cDate);

	if(!pName)pName="";

	fprintf(pFile,pGpxHeader,cDate,cTime,pName);
	fprintf(pFile,"<trkseg>\n");

	for(i=0;i<iCount;i++)										// Weg ber Trackpunkte
		{
		fprintf(pFile,"\t<trkpt lat=\"%7.10f\" lon=\"%7.10f\">",pRoute[i].dCoordY,pRoute[i].dCoordX);
		fprintf(pFile,"<ele>%4.2f</ele>",pRoute[i].dCoordZ);

		if(pRoute[i].dGlobalTime>1.0)
			{
			iTime = (time_t)(pRoute[i].dGlobalTime*86400.0);
			strtimeset(cDate,iTime,STR_DATE_ISO|STR_LONG|STR_GLOBAL);
			strtimeset(cTime,iTime,STR_TIME|STR_SEC|STR_GLOBAL);
			strcut(cDate);
			fprintf(pFile,"<time>%sT%sZ</time>",cDate,cTime);
			}

		fprintf(pFile,"</trkpt>\n");
		}

	fprintf(pFile,"%s",pGpxTail);
	fclose (pFile);


return 0;
}

//*****************************************************************************
//*
//*		SaveGpx
//*
//*****************************************************************************
//	Speichert eine Route als GPX-Datei
//	pFilename	: Ist der Name der Datei
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	pName		: Ist der Name der Route (NULL ist auch erlaubt)
//	Ergibt 0 wenn die Datei gespeichert wurde bzw. eine Fehlernummer kleiner 0
//					-1 Die Datei konnte nicht erstellt werden
//					-2 Keine Routendaten vorhanden
int SaveGpx(const char *pFilename,const RouteData *pRoute,int iCount,const char *pName)
{
char	cTemp[128];	
FILE   *pFile;


	if(iCount<=1)
		{
		return -2;
		}

	if(!pFilename || !*pFilename)
		{
		return -1;
		}

		pFile = fopen(pFilename,"wb");
	if(!pFile)
		{
		return -1;
		}

	if(!pName || !*pName)
		{
		GetFileBody(cTemp,pFilename,sizeof(cTemp));
		pName = cTemp;
		}


return SaveGpx(pFile,pRoute,iCount,pName);
}

int SaveGpx(const wchar_t *pFilename,const RouteData *pRoute,int iCount,const wchar_t *pName)
{
wchar_t cTemp[128];
char	cName[128];
char   *pTemp;
FILE   *pFile;


	if(iCount<=1)
		{
		return -2;
		}

	if(!pFilename || !*pFilename)
		{
		return -1;
		}

		pFile = _wfopen(pFilename,L"wb");
	if(!pFile)
		{
		return -1;
		}

	if(!pName || !*pName)
		{
		GetFileBody(cTemp,pFilename,sizeof(cTemp));
		pName = cTemp;
		}

	if(pName)
		{
		strawcpy(cName,pName,128);
		pTemp = cName;
		}
	else{
		pTemp = 0;
		}



return SaveGpx(pFile,pRoute,iCount,pTemp);
}

//*****************************************************************************
//*
//*		SaveKml
//*
//*****************************************************************************
//	Speichert eine Route als KML-Datei
//	pFile		: Ist das Datei Handle zum Speichen
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	pRoutePtr	: Hier wird die neue Route gespeichert
//	dStep		: Ist der Abstand der Abschnitte bzw. die gesamte Anzahl der
//				  Abschnitte wenn der Wert kleiner 0 ist.
//	uMode		: Bestimmt welcher Wert benutzt wird (0=Distanz 1=Zeit)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Falschermodus
//					-2 Keine Routendaten vorhanden
//					-3 Es ist kein Abschnittsbereich vorhanden
//					-4 Der neue Bereich ist zu gro oder zu klein
//					-5 Zuwenig Speicher
static int SaveKml(FILE *pFile,const RouteData *pRoute,int iCount,const char *pName,const char *pFilename)
{
char		cDate[32];
char		cTime[32];
time_t		iTime;
int			i;



	if(pRoute[0].dGlobalTime>1.0)
		{
		iTime = (time_t)(pRoute[0].dGlobalTime*86400.0);
		}
	else{
		iTime = time(0);
		}

	strtimeset(cDate,iTime,STR_DATE_ISO|STR_LONG);
	strtimeset(cTime,iTime,STR_TIME|STR_SEC);
	strcut(cDate);
	if(!pName)pName="";

	fprintf(pFile,pKmlHeader,pFilename,pName);

	for(i=0;i<iCount;i++)
		{
		fprintf(pFile,"\t\t\t\t%7.10f,%7.10f,0\n",pRoute[i].dCoordX,pRoute[i].dCoordY);
		}

	fprintf(pFile,"%s",pKmlTail);
	fclose (pFile);



return 0;
}


//*****************************************************************************
//*
//*		SaveKml
//*
//*****************************************************************************
//	Speichert eine Route als KML-Datei
//	pFilename	: Ist der Name der Datei
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	pName		: Ist der Name der Route (NULL ist auch erlaubt)
//	Ergibt 0 wenn die Datei gespeichert wurde bzw. eine Fehlernummer kleiner 0
//					-1 Die Datei konnte nicht erstellt werden
//					-2 Keine Routendaten vorhanden
int SaveKml(const char *pFilename,const RouteData *pRoute,int iCount,const char *pName)
{
char	cTemp[128];
FILE   *pFile;


	if(iCount<=1)
		{
		return -2;
		}

	if(!pFilename || !*pFilename)
		{
		return -1;
		}

		pFile = fopen(pFilename,"wb");
	if(!pFile)
		{
		return -1;
		}

	if(!pName || !*pName)
		{
		GetFileBody(cTemp,pFilename,sizeof(cTemp));
		pName = cTemp;
		}


return SaveKml(pFile,pRoute,iCount,pName,GetFileName(pFilename));
}

int SaveKml(const wchar_t *pFilename,const RouteData *pRoute,int iCount,const wchar_t *pName)
{
wchar_t cTemp[128];
char	cName[128];
char	cBuff[128];
char   *pTemp;
FILE   *pFile;


	if(iCount<=1)
		{
		return -2;
		}

	if(!pFilename || !*pFilename)
		{
		return -1;
		}

		pFile = _wfopen(pFilename,L"wb");
	if(!pFile)
		{
		return -1;
		}

	if(!pName || !*pName)
		{
		GetFileBody(cTemp,pFilename,sizeof(cTemp));
		pName = cTemp;
		}

	if(pName)
		{
		strawcpy(cName,pName,128);
		pTemp = cName;
		}
	else{
		pTemp = 0;
		}

	strawcpy(cBuff,GetFileName(pFilename),128);


return SaveKml(pFile,pRoute,iCount,pTemp,cBuff);
}


//*****************************************************************************
//*
//*		SaveCsv
//*
//*****************************************************************************
//	Speichert eine Route als CSV-Datei
//	pFile		: Ist das Datei Handle zum Speichen
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	pRoutePtr	: Hier wird die neue Route gespeichert
//	dStep		: Ist der Abstand der Abschnitte bzw. die gesamte Anzahl der
//				  Abschnitte wenn der Wert kleiner 0 ist.
//	uMode		: Bestimmt welcher Wert benutzt wird (0=Distanz 1=Zeit)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Falschermodus
//					-2 Keine Routendaten vorhanden
//					-3 Es ist kein Abschnittsbereich vorhanden
//					-4 Der neue Bereich ist zu gro oder zu klein
//					-5 Zuwenig Speicher
static int SaveCsv(FILE *pFile,const RouteData *pRoute,int iCount,const char *pName)
{
char		cName[128];
char		cDate[64];
double		dValue;
time_t		iTime;
int			i;



	if(pRoute[0].dGlobalTime>1.0)
		{
		iTime = (time_t)(pRoute[0].dGlobalTime*86400.0);
		}
	else{
		iTime = time(0);
		}

	strtimeset(cDate,iTime,STR_ALL_ISO);
	if(!pName)pName="";

	strtocsv(pName,cName,sizeof(cName));
	fprintf(pFile,";;;%s%s;\n",cName,cDate);

	for(i=0;i<iCount;i++)
		{
		   dValue = pRoute[i].dTime;
		if(dValue>1.0)
			{
			dValue  += 25569.0;
			dValue  /= 86400.0;
			sprintf(cDate,"%7.10f;",dValue);
			}
		else{
			cDate[0] = 0;
			}

		fprintf(pFile,"%7.10f;%7.10f;%s\n",pRoute[i].dCoordX,pRoute[i].dCoordY,cDate);
		}

	fclose(pFile);



return 0;
}

//*****************************************************************************
//*
//*		SaveCsv
//*
//*****************************************************************************
//	Speichert eine Route als CSV-Datei
//	pFilename	: Ist der Name der Datei
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	pName		: Ist der Name der Route (NULL ist auch erlaubt)
//	Ergibt 0 wenn die Datei gespeichert wurde bzw. eine Fehlernummer kleiner 0
//					-1 Die Datei konnte nicht erstellt werden
//					-2 Keine Routendaten vorhanden
int SaveCsv(const char *pFilename,const RouteData *pRoute,int iCount,const char *pName)
{
char	cTemp[128];
FILE   *pFile;


	if(iCount<=1)
		{
		return -2;
		}

	if(!pFilename || !*pFilename)
		{
		return -1;
		}

		pFile = fopen(pFilename,"wb");
	if(!pFile)
		{
		return -1;
		}

	if(!pName || !*pName)
		{
		GetFileBody(cTemp,pFilename,sizeof(cTemp));
		pName = cTemp;
		}

return SaveCsv(pFile,pRoute,iCount,pName);
}

int SaveCsv(const wchar_t *pFilename,const RouteData *pRoute,int iCount,const wchar_t *pName)
{
wchar_t cTemp[128];
char	cName[128];
char   *pTemp;
FILE   *pFile;


	if(iCount<=1)
		{
		return -2;
		}

	if(!pFilename || !*pFilename)
		{
		return -1;
		}

		pFile = _wfopen(pFilename,L"wb");
	if(!pFile)
		{
		return -1;
		}

	if(!pName || !*pName)
		{
		GetFileBody(cTemp,pFilename,sizeof(cTemp));
		pName = cTemp;
		}

	if(pName)
		{
		strawcpy(cName,pName,128);
		pTemp = cName;
		}
	else{
		pTemp = 0;
		}



return SaveCsv(pFile,pRoute,iCount,pTemp);
}


//*****************************************************************************
//*
//*		CalcDistance
//*
//*****************************************************************************
//	Berechnet die Wege und Geschwindigkeiten einer Strecke mit Sttzpunkten
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	iFlags		: Diverse Flags fr die Verarbeitung
//					Bit0 = Die Geschwindigkeiten gltten
//					Bit1 = Zittern auf einem Punkt entfernen
//					Bit2 = Erster Schritt auf "zu weit" prfen
//					Bit3 = Hhenmeterunterschiede berechnen
//	Ergibt die Gesamtlnge in Meter
double CalcDistance(RouteData *pRoute,int iCount,int iFlags)
{
double		dHeighDown;
double		dHeighUp;
double		dDeltaX;
double		dDeltaY;
double		dDeltaT;
double		dDelta;
double		dPause;
double		dTime;
double		dSum;
double		dDif;
double		dLen;
double		dVel;
int			iPos;
int			iOff;
int			iEnd;
int			iNum;





	if((iFlags&4) && iCount>DELTA_CNT)				// Erster Schritt zu weit ?
		{
		for(iPos=2;iPos<10;iPos++)					// Ist es eine Zeitaufzeichnung
			{
			   dDelta  = pRoute[iPos].dGlobalTime-pRoute[iPos-1].dGlobalTime;
			   dDelta *= 86400.0;
			if(dDelta>5.0)break;
			}

		if(iPos>=10)
			{
			dDelta	= DistanceOfPointsEx(DegreeToRadiant(pRoute[1].dCoordX),DegreeToRadiant(pRoute[1].dCoordY),DegreeToRadiant(pRoute[0].dCoordX),DegreeToRadiant(pRoute[0].dCoordY));

			if(dDelta>270.0)						// Erster Schritt zu weit ?
				{
				pRoute[0].dCoordX		= pRoute[1].dCoordX;
				pRoute[0].dCoordY		= pRoute[1].dCoordY;
				pRoute[0].dGlobalTime	= pRoute[1].dGlobalTime-0.0001/86400.0;
				}
			}
		}


	if((iFlags&2) && iCount>DELTA_CNT)				// Zittern auf einem Punkt entfernen
		{
		iEnd = iCount-DELTA_CNT;

		for(iPos=DELTA_CNT;iPos<iEnd;iPos++)
			{
			for(iNum=1;;iNum++)
				{
				if(iNum>=DELTA_CNT)
					{
					pRoute[iPos-DELTA_CNT+1].dCoordX = pRoute[iPos-DELTA_CNT].dCoordX;
					pRoute[iPos-DELTA_CNT+1].dCoordY = pRoute[iPos-DELTA_CNT].dCoordY;
					break;
					}

				dDeltaX = fabs(pRoute[iPos].dCoordX-pRoute[iPos-iNum].dCoordX);
				if(dDeltaX>0.0002)break;
				dDeltaY = fabs(pRoute[iPos].dCoordY-pRoute[iPos-iNum].dCoordY);
				if(dDeltaY>0.0002)break;
				dDeltaX = fabs(pRoute[iPos].dCoordX-pRoute[iPos+iNum].dCoordX);
				if(dDeltaX>0.0002)break;
				dDeltaY = fabs(pRoute[iPos].dCoordY-pRoute[iPos+iNum].dCoordY);
				if(dDeltaY>0.0002)break;
				}
			}
		}



	dHeighDown	= 0;
	dHeighUp	= 0;
	dPause		= 0;
	dTime		= 0;
	dLen		= 0;


	for(iPos=1;iPos<iCount;iPos++)					// Distanzen und Geschwindigkeiten berechnen
		{
		dDelta	= DistanceOfPointsEx(DegreeToRadiant(pRoute[iPos-1].dCoordX),DegreeToRadiant(pRoute[iPos-1].dCoordY),DegreeToRadiant(pRoute[iPos].dCoordX),DegreeToRadiant(pRoute[iPos].dCoordY));
		dDeltaT	= (pRoute[iPos].dGlobalTime-pRoute[iPos-1].dGlobalTime)*86400.0;

		if(dDeltaT<0.0)dDeltaT=0;

		if(dDeltaT>0.01 && dDeltaT<=120.0)
				dVel = dDelta/dDeltaT;
		else	dVel = 0.0;

		dTime  += dDeltaT;
		dLen   += dDelta;

		if(dVel<0.2)								// Pause ist unter 0.2 m/s (0.72 km/h)
			{
			dPause += dDeltaT;
			}

		pRoute[iPos].dTime		= dTime;
		pRoute[iPos].dPause		= dPause;
		pRoute[iPos].dVelocity	= dVel;
		pRoute[iPos].dDistance	= dLen;

		if(iFlags&8)								// Hhennderung berechnen
			{
			   dDelta = pRoute[iPos].dCoordZ-pRoute[iPos-1].dCoordZ;
			if(dDelta>0)
					dHeighUp   += dDelta;
			else	dHeighDown += dDelta;

			pRoute[iPos].dHeighUp	= dHeighUp;
			pRoute[iPos].dHeighDown	= dHeighDown;
			}
		}



	if((iFlags&1) && dTime>10.0)					// Die Geschwindigkeit gltten
		{
		dPause	= 0.0;

		for(iPos=0;iPos<iCount;iPos++)
			{
			dTime	= pRoute[iPos].dTime;

			for(iNum=iPos;iNum>0;iNum--)			// 2.5 Sekunden in die Vergangenheit
				{
				   dDeltaT	= pRoute[iPos].dTime-pRoute[iNum].dTime;
				if(dDeltaT>=2.5)break;
				}

			for(iOff=iPos;iOff<iCount;iOff++)		// 2.5 Sekunden in die Zukunft
				{
				   dDeltaT	= pRoute[iOff].dTime-pRoute[iPos].dTime;
				if(dDeltaT>=2.5)break;
				}

			dDeltaT	= pRoute[iPos].dTime    -pRoute[iNum].dTime;
			dDelta 	= pRoute[iPos].dDistance-pRoute[iNum].dDistance;

			if(dDeltaT>2.5)
				{
				dDelta	= dDelta*2.5/dDeltaT;
				dDeltaT = 2.5;
				}

			dSum	= dDelta;
			dDif	= dDeltaT;
			dDeltaT	= pRoute[iOff].dTime    -pRoute[iPos].dTime;
			dDelta 	= pRoute[iOff].dDistance-pRoute[iPos].dDistance;

			if(dDeltaT>2.5)
				{
				dDelta	= dDelta*2.5/dDeltaT;
				dDeltaT = 2.5;
				}

			dSum   += dDelta;
			dDif   += dDeltaT;

			if(dDif>0.0)							// Die geglttete Geschwindigkeit speichern
				{
				dVel					= dSum/dDif;
				pRoute[iPos].dVelocity	= dVel;
				}
			}
		}




return dLen;
}

//*****************************************************************************
//*
//*		ScaleRoute
//*
//*****************************************************************************
//	Skaliert eine Route in equidistante Abschnitte. Vorher sollte CalcDistance()
//	fr diese Route aufgerufen werden.
//	pRoute		: Zeiger auf die Sttzpunkte
//	iCount		: Ist die Anzahl der Koordinatenpaare
//	pRoutePtr	: Hier wird die neue Route gespeichert
//	dStep		: Ist der Abstand der Abschnitte bzw. die gesamte Anzahl der
//				  Abschnitte wenn der Wert kleiner 0 ist.
//	uMode		: Bestimmt welcher Wert benutzt wird (0=Distanz 1=Zeit)
//	Ergibt die Anzahl der geladenen Koordinatenpaare bzw. kleiner 0 bei einem Fehler.
//					-1 Falschermodus
//					-2 Keine Routendaten vorhanden
//					-3 Es ist kein Abschnittsbereich vorhanden
//					-4 Der neue Bereich ist zu gro oder zu klein
//					-5 Zuwenig Speicher
int ScaleRoute(const RouteData *pRoute,int iCount,RouteData **pRoutePtr,double dStep,unsigned uMode)
{
#define 		ROUTE_NUM	(offsetof(RouteData,dUserData1)/sizeof(double))
#define 		ROUTE_END	(sizeof  (RouteData           )/sizeof(double))
unsigned		uOffset;
unsigned char  *pData;
RouteData	   *pOutput;
double		   *pValues;
double		   *pInput;
double			dPlace;
double			dValue;
double			dDelta;
double			dArea;
double			dBase;
double			dDiff;
double			dLast;
int				iLast;
int				iNow;
int				iMax;
int				iPos;
int				iNum;



	*pRoutePtr = 0;

	if(uMode >=2)return -1;
	if(iCount<=1)return -2;


	switch(uMode)
		{
	case 0:	uOffset = offsetof(RouteData,dDistance);	break;
	case 1:	uOffset = offsetof(RouteData,dTime    );	break;
		}


	pData	= (unsigned char*)pRoute;
	pData  += uOffset;
	dLast	= *(double*)(pData);
	dDelta	= *(double*)(pData+sizeof(RouteData)*(iCount-1));
	dDelta -= dLast;

	if(dDelta<0.0001)
		{
		return -3;
		}

	if(dStep<0.0)
		{
		iMax	= (int)-dStep;
		dStep	= dDelta/(double)iMax;
		}
	else{
		iMax	= (int)(dDelta/dStep+0.01);
		}


	if(iMax<=0 || iMax>0x80000)
		{
		return -4;
		}

		pOutput	= new RouteData[iMax];
	if(!pOutput)
		{
		return -5;
		}


	iCount     -= 1;
	iLast		= 0;
	pOutput[0]	= pRoute[0];
	dPlace		= pRoute[0].dDistance+dStep;
	pValues		= (double*)(pOutput+1);

	for(iPos=1;iPos<iMax;)								// Alle Eintrge erzeugen
		{
		for(iNow=iLast+1;;iNow++)						// Suche nchsten Eintrag nach Grenze
			{
			dValue = *(double*)(pData+iNow*sizeof(RouteData));

			if(iNow>=iCount  )break;
			if(dValue<=dLast )continue;
			if(dValue>=dPlace)break;

			dLast = dValue;
			}

		dDelta	= dValue-dLast;

		while(iPos<iMax)
			{
			pInput	= (double*)(pRoute+iNow);
			dDiff	= dPlace-dLast;
			dDiff  /= dDelta;

			if(dDiff>1.0)dDiff=1.0;
			if(dDiff<0.0)dDiff=0.0;

														// Einen Eintrag skalieren
			for(iNum=0;iNum<ROUTE_NUM;iNum++,pValues++,pInput++)
				{
				dBase		= pInput[-(int)ROUTE_END];
				dArea		= pInput[0]-dBase;
				pValues[0]	= dBase+dArea*dDiff;
				}

			for(;iNum<ROUTE_END;iNum++,pValues++)		// User-Daten auf 0 setzen
				{
				pValues[0]	= 0;
				}

			dPlace += dStep;
			iPos   += 1;

			if(dPlace>dValue && iNow<iCount)			// In den nchsten Abschnitt
				{
				break;
				}
			}

		if(iNow<iCount)
			{
			dLast = dValue;
			iLast = iNow;
			}
		}


	*pRoutePtr = pOutput;


return iMax;
}

//*****************************************************************************
//*
//*		TextToCoord
//*
//*****************************************************************************
//	Wandelt einen Text in eine Koordinate um
//	pText		: Zeiger auf den Text ("-11.456 +11,456 -1122'33.44'' 1122'O )
//	pLen		: Hier wird die Lnge des Textes gespeichert
//	Ergibt den Koordinaten Wert
double TextToCoord(const char *pText,int *pLen)
{
double	dValue;
double	dSub;
int		iNum;
int		iLen;


	dValue	= strtof(pText,&iLen,0x05);
														// Dezimal Format "11.223344"
	if(pText[iLen]!='' || strcwlen(pText,0x1200,iLen)!=iLen)
		{
		if(pLen)
			{
			iLen += strwlen(pText+iLen,0xE0);
			*pLen = iLen;
			}

		return dValue;
		}

	iLen += 1;
	iLen += strwlen(pText+iLen,0xE0);
	dSub  = strtof(pText+iLen,&iNum,0x604);

														// Minuten
	if(iNum && pText[iLen+iNum]=='\'' && pText[iLen+iNum+1]!='\'')
		{
		dSub   *= 1.0/60.0;
		dValue += (dValue<0)? -dSub:dSub;
		iLen   += iNum+1;
		iLen   += strwlen(pText+iLen,0xE0);
		}


	dSub = strtof(pText+iLen,&iNum,0x604);
														// Sekunden
	if(pText[iLen+iNum]=='\'' && pText[iLen+iNum+1]=='\'')
		{
		dSub   *= 1.0/3600.0;
		dValue += (dValue<0)? -dSub:dSub;
		dValue += dSub*(1.0/3600.0);
		iLen   += iNum+2;
		iLen   += strwlen(pText+iLen,0xE0);
		}

	if(pText[iLen]=='N' || pText[iLen]=='n')			// Himmelsrichtungen
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		}
	else if(pText[iLen]=='S' || pText[iLen]=='s')
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		if(dValue>0.0)dValue=-dValue;
		}
	else if(pText[iLen]=='E' || pText[iLen]=='e')
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		}
	else if(pText[iLen]=='W' || pText[iLen]=='w')
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		if(dValue>0.0)dValue=-dValue;
		}


	if(pLen)*pLen = iLen;


return dValue;
}

double TextToCoord(const wchar_t *pText,int *pLen)
{
double	dValue;
double	dSub;
int		iNum;
int		iLen;


	dValue	= strtof(pText,&iLen,0x05);
														// Dezimal Format "11.223344"
	if(pText[iLen]!='' || strcwlen(pText,0x1200,iLen)!=iLen)
		{
		if(pLen)
			{
			iLen += strwlen(pText+iLen,0xE0);
			*pLen = iLen;
			}

		return dValue;
		}

	iLen += 1;
	iLen += strwlen(pText+iLen,0xE0);
	dSub  = strtof(pText+iLen,&iNum,0x604);

														// Minuten
	if(iNum && pText[iLen+iNum]=='\'' && pText[iLen+iNum+1]!='\'')
		{
		dSub   += dSub*(1.0/60.0);
		dValue += (dValue<0)? -dSub:dSub;
		iLen   += iNum+1;
		iLen   += strwlen(pText+iLen,0xE0);
		}


	dSub = strtof(pText+iLen,&iNum,0x604);
														// Sekunden
	if(pText[iLen+iNum]=='\'' && pText[iLen+iNum+1]=='\'')
		{
		dSub   *= 1.0/3600.0;
		dValue += (dValue<0)? -dSub:dSub;
		dValue += dSub*(1.0/3600.0);
		iLen   += iNum+2;
		iLen   += strwlen(pText+iLen,0xE0);
		}

	if(pText[iLen]=='N' || pText[iLen]=='n')			// Himmelsrichtungen
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		}
	else if(pText[iLen]=='S' || pText[iLen]=='s')
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		if(dValue>0.0)dValue=-dValue;
		}
	else if(pText[iLen]=='E' || pText[iLen]=='e')
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		}
	else if(pText[iLen]=='W' || pText[iLen]=='w')
		{
		iLen   += 1;
		iLen   += strwlen(pText+iLen,0xE0);
		if(dValue>0.0)dValue=-dValue;
		}


	if(pLen)*pLen = iLen;


return dValue;
}

//*****************************************************************************
//*
//*		CoordToText
//*
//*****************************************************************************
//	Wantelt eine Koordinate in einen Text um.
//	dValue		: Ist der Koordinatenwert
//	uFormat		: Ist das Format fr den Text
//					Bit0	= 0:X-Koordinate 1:Y-Koordinate
//					Bit1..N	= Format
//				    0/1		= "-11.223344"
//					2/3		= "-11,223344"
//					4/5		= "-11 22' 33''"
//					6/7		= "-11 22' 33.44''"
//					8 		= " 11 22' 33'' W"
//					9 		= " 11 22' 33'' S"
//					10 		= " 11 22' 33.44'' W"
//					11 		= " 11 22' 33.44'' S"
//	pBuffer		: Puffer in dem der Text gespeichert werden soll.
//				  Bei 0 wird ein Puffer zugewiesen
//	pLen		: Hier wird die Lnge des Textegespeichert (bei 0 nicht)
//	Ergibt einen Zeiger auf den Textpuffer.
const char *CoordToTextA(double dValue,unsigned uFormat,char *pBuffer,int *pLen)
{
static	char		cBuffer[8*32];
static	unsigned 	uBufNum=0;
int					iChar;
int					iLen;
int					iVal;
int					iMin;
int					iSec;
int					iSub;


	if(dValue>999.0 || dValue<-999.0)
		{
		if(pLen)*pLen=3;
		if(!pBuffer)return "\?\?\?";
		strcpy(pBuffer,"\?\?\?");
		return pBuffer;
		}

	if(!pBuffer)
		{
		pBuffer = cBuffer+32*(uBufNum&7);
		uBufNum++;
		}

	switch(uFormat)
		{
	default:iLen=sprintf(pBuffer,"%.6f",dValue);			// Dezimal
			break;
	case 2:
	case 3:	iLen=sprintf(pBuffer,"%.6f",dValue);			// Dezimal mit Komma
			strreplace(pBuffer,'.',',');
			break;

	case 4:
	case 5:													// Grad Minuten Sekunden
			iVal	= (int)dValue;
			dValue	= (fabs(dValue-(double)iVal)+0.4999/3600.0)*60.0;
			iMin	= (int)dValue;
			dValue	= (dValue-(double)iMin)*60.0;
			iSec	= (int)dValue;
			iLen	= sprintf(pBuffer,"%i %02i' %02i''",iVal,iMin,iSec);
			break;

	case 6:
	case 7:													// Grad Minuten Sekunden und Komma
			iVal	= (int)dValue;
			dValue	= (fabs(dValue-(double)iVal)+0.004999/3600.0)*60.0;
			iMin	= (int)dValue;
			dValue	= (dValue-(double)iMin)*60.0;
			iSec	= (int)dValue;
			dValue	= (dValue-(double)iSec)*100.0;
			iSub	= (int)dValue;
			iLen	= sprintf(pBuffer,"%i %02i' %02i.%02i''",iVal,iMin,iSec,iSub);
			break;

	case 8:													// Grad Minuten Sekunden und Richtung-X
			if(dValue<0.0)
					{iChar='W';dValue=-dValue;}
			else	{iChar='E';}

			iVal	= (int)dValue;
			dValue	= (fabs(dValue-(double)iVal)+0.4999/3600.0)*60.0;
			iMin	= (int)dValue;
			dValue	= (dValue-(double)iMin)*60.0;
			iSec	= (int)dValue;
			iLen	= sprintf(pBuffer,"%i %02i' %02i'' %c",iVal,iMin,iSec,iChar);
			break;

	case 9:													// Grad Minuten Sekunden und Richtung-Y
			if(dValue<0.0)
					{iChar='S';dValue=-dValue;}
			else	{iChar='N';}

			iVal	= (int)dValue;
			dValue	= (fabs(dValue-(double)iVal)+0.4999/3600.0)*60.0;
			iMin	= (int)dValue;
			dValue	= (dValue-(double)iMin)*60.0;
			iSec	= (int)dValue;
			iLen	= sprintf(pBuffer,"%i %02i' %02i'' %c",iVal,iMin,iSec,iChar);
			break;

	case 10:												// Grad Minuten Sekunden und Richtung-X+Komma
			if(dValue<0.0)
					{iChar='W';dValue=-dValue;}
			else	{iChar='E';}

			iVal	= (int)dValue;
			dValue	= (fabs(dValue-(double)iVal)+0.004999/3600.0)*60.0;
			iMin	= (int)dValue;
			dValue	= (dValue-(double)iMin)*60.0;
			iSec	= (int)dValue;
			dValue	= (dValue-(double)iSec)*100.0;
			iSub	= (int)dValue;
			iLen	= sprintf(pBuffer,"%i %02i' %02i.%02i'' %c",iVal,iMin,iSec,iSub,iChar);
			break;

	case 11:												// Grad Minuten Sekunden und Richtung-Y+Komma
			if(dValue<0.0)
					{iChar='S';dValue=-dValue;}
			else	{iChar='N';}

			iVal	= (int)dValue;
			dValue	= (fabs(dValue-(double)iVal)+0.004999/3600.0)*60.0;
			iMin	= (int)dValue;
			dValue	= (dValue-(double)iMin)*60.0;
			iSec	= (int)dValue;
			dValue	= (dValue-(double)iSec)*100.0;
			iSub	= (int)dValue;
			iLen	= sprintf(pBuffer,"%i %02i' %02i.%02i'' %c",iVal,iMin,iSec,iSub,iChar);
			break;
		}


	if(pLen)*pLen=iLen;


return pBuffer;
}

const wchar_t *CoordToTextW(double dValue,unsigned uFormat,wchar_t *pBuffer,int *pLen)
{
static	wchar_t		cBuffer[8*32];
static	unsigned 	uBufNum=0;
char				cTemp[32];
const char		   *pRet;


	pRet = CoordToTextA(dValue,uFormat,cTemp,pLen);

	if(pBuffer)
		{
		pBuffer = cBuffer+32*(uBufNum&7);
		uBufNum++;
		}

	strawcpy(pBuffer,pRet,32);


return pBuffer;
}


