//*****************************************************************************
//*
//*
//*		DataSanner.cpp: Implementierung der Klasse CDataScanner.
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//
//	Funktion:  siehe "DataScaner.h"
//
#include	<stdio.h>
#include	<stdlib.h>

#include	"DataScanner.h"
#include	"FileFunctions.h"
#include	"StringFunctions.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


//*****************************************************************************
//*
//*		Flags
//*
//*****************************************************************************

#define 	MAX_LOCALS		4					// Anzahl der lokalen  Parameter
#define 	MAX_PARAMS		4					// Anzahl der globalen Parameter

#define 	FLAG_WIDTH		0x0000F000			// Art der Zahlen Ausgabe
#define 	FLAG_WIDTH_EX	0x007FF000			// Art der Zahlen Ausgabe
#define 	FLAG_OCT		0x00010000
#define 	FLAG_DEC		0x00020000
#define 	FLAG_HEX		0x00040000
#define 	FLAG_ASM		0x00080000
#define 	FLAG_SIGNED		0x00100000
#define 	FLAG_UNSIGNED	0x00200000
#define 	FLAG_CHAR		0x00400000
#define 	FLAG_BIN		0x00800000
#define 	FLAG_NONE		0x01000000
#define 	FLAG_MOTOROLA	0x02000000
#define 	FLAG_PREFIX		0x04000000
#define 	FLAG_SUFFIX		0x08000000
#define 	FLAG_SUFFIXH	(FLAG_SUFFIX|FLAG_SAVE)
#define 	FLAG_AFTER		FLAG_UTF8
#define 	FLAG_REGMODE	FLAG_NONE
#define 	FLAG_UTF8		FLAG_SIGNED
#define 	FLAG_UNICODE	FLAG_UNSIGNED
#define 	FLAG_SETSIZE	FLAG_RASTER
#define 	FLAG_RVA		FLAG_RASTER
#define 	FLAG_SAVE		FLAG_ABSOLUT
#define 	FLAG_UP			FLAG_UNSIGNED
#define 	FLAG_DOWN		FLAG_DEC
#define 	FLAG_MARK		FLAG_MOTOROLA
#define 	FLAG_PRINT		F_MARK2

#define 	FLAG_GLOBAL		0x80000000			// Globales Offset verwenden
#define 	FLAG_RASTER		0x40000000			// Raster Verschub
#define 	FLAG_ABSOLUT	0x20000000			// Absoluter Verschub
#define 	FLAG_NOT		0x10000000			// Text nicht ausgeben

#define 	F_NAME			0x00000002			// Name		 im Syntax
#define 	F_MARK1			0x00000004			// Marke1	 im Syntax
#define 	F_VERGL			0x00000008			// Vergleich im Syntax
#define 	F_VAL1			0x00000010			// Zahl1	 im Syntax
#define 	F_VAL2			0x00000020			// Zahl2	 im Syntax
#define 	F_CALL			0x00000040			// CALL/GOTO im Syntax
#define 	F_MARK2			0x00000080			// Marke2	 im Syntax
#define 	F_DUMP			0x00000100			// Bis-Dump	 im Syntax
#define 	F_FLAGS			0x00000200			// Flags	 im Syntax
#define 	F_NOT			0x00000400			// NotFlag	 im Syntax
#define 	F_MARKF			0x00000800			// Mrk vor Z im Syntax
#define 	F_ALL			0x00000FFF			// Alles Syntax Bits

//*****************************************************************************
//*
//*		Befehle
//*
//*****************************************************************************

#define 	CMD_NULL		0x00
#define 	CMD_FLOAT		0x84
#define 	CMD_DOUBLE		0x88
#define 	CMD_BYTE		0x91
#define 	CMD_WORD		0x92
#define 	CMD_RWORD		0x93
#define 	CMD_DWORD		0x94
#define 	CMD_QWORD		0xA8
#define 	CMD_STRING		0x10
#define 	CMD_STRING_P	0x11
#define 	CMD_STRING_D	0x12
#define 	CMD_STRING_B	0x13
#define 	CMD_STRING_K	0x14
#define 	CMD_REST		0x20
#define 	CMD_BIT			0x21
#define 	CMD_IF			0x30
#define 	CMD_LOOP		0x31
#define 	CMD_BLOCK		0x32
#define 	CMD_RETURN		0x33
#define 	CMD_EXIT		0x34
#define 	CMD_SEEK		0x35
#define 	CMD_CALL		0x36
#define 	CMD_GOTO		0x37
#define 	CMD_MODUL		0x38
#define 	CMD_ADD			0x41
#define 	CMD_MUL			0x42
#define 	CMD_SUB			0x43
#define 	CMD_AND			0x44
#define 	CMD_OR			0x45
#define 	CMD_XOR			0x46
#define 	CMD_DIV			0x47
#define 	CMD_MOD			0x48
#define 	CMD_MOV			0x49
#define 	CMD_BIN			0x4A
#define 	CMD_TEXT		0x51
#define 	CMD_PRINT		0x52
#define 	CMD_SPRINT		0x53
#define 	CMD_UP			0x54
#define 	CMD_DOWN		0x55
#define 	CMD_OFFSET		0x60
#define 	CMD_LEN			0x61
#define 	CMD_RVA			0x62

#define 	CNONE			0,0

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	strzlen		wcslen
#define 	strzcpy		wcscpy
#define 	strzicmp	wcsnicmp
#define 	sprintfz	wsprintf
#define 	printfz		wprintf
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	strzlen		strlen
#define 	strzcpy		strcpy
#define 	strzicmp	strnicmp
#define 	sprintfz	sprintf
#define 	printfz		printf
#endif



typedef struct
	{
	char			aName[8];
    unsigned		uVirtualSize;
    unsigned		uVirtualAddress;
    unsigned		uSizeOfRawData;
    unsigned		uPointerToRawData;
    unsigned		uPointerToRelocations;
    unsigned		uPointerToLinenumbers;
    unsigned short	uNumberOfRelocations;
    unsigned short	uNumberOfLinenumbers;
    unsigned		uCharacteristics;
	}ImageSectionHeader;

typedef struct
	{
	int				iValue;									// Variabel Wert (muss an die erste Stelle)
	unsigned char 	ucCmd;									// Befehls Nummer
	unsigned char	ucPrint;								// Kann der Befehl in ander Zeilen schreiebn
	unsigned char	ucDummy1;								// Unbenutzt
	unsigned char	ucDummy2;								// Unbenutzt
	int				iMark1;									// Markennummern
	int				iMark2;									//		*
	int				iNumber1;								// Zahlen Konstanten
	int				iNumber2;								//		*
	int				iDump;									// Bis-Dump Nummer
	int				iSubCmd;								// Sub-Befehl
	int				iFlags;									// Flags
	int				iFlagNr;								// Nummer des Flag-Textes
	int				iOffset;								// Offest auf dem die Marke liegt
	int				iPreMark;								// Nummer der Marke an Zeilenbeginn (-1= keine)
	#if _DEBUG
	int				iLine;									// Zeile des Befehls
	int				iText;									// Text der Zeile
	#endif
	}DataStruct;

typedef struct
	{
	int				iPos;									// Stratposition
	int				iModul;									// Modulnummer
	}BlockStruct;

typedef struct
	{
	int				iAdd;									// Was wird zum Offset addiert
	short			iOff;									// Vergleich-Offset im Speicher
	short			iMark;									// Marke fr Vergleichs-Offset
	}DumpStruct;

typedef struct
	{
	zchar		   *pName;
	unsigned char	ucCmd;
	unsigned char	ucPrint;
	unsigned char	ucDummy1;
	unsigned char	ucDummy2;
	int 	 		iMust;
	int 	 		iNot;
	}CommandData;

static const CommandData	aCommands[]=
	{
//  Name			ID			   print	    muss   nicht
	Z(""), 			CMD_NULL		,1	,CNONE,0x0000,0x0000,	// 0x0002=Name
	Z("BIT"),	 	CMD_BIT			,1	,CNONE,0x0000,0x04E8,	// 0x0004=Marke1
	Z("BYTE"), 		CMD_BYTE		,1	,CNONE,0x0000,0x04E8,	// 0x0008=Vergleich
	Z("WORD"), 		CMD_WORD		,1	,CNONE,0x0000,0x04E8,	// 0x0010=Zahl1
	Z("DWORD"), 	CMD_DWORD		,1	,CNONE,0x0000,0x04E8,	// 0x0020=Zahl2
	Z("RWORD"), 	CMD_RWORD		,1	,CNONE,0x0000,0x04E8,	// 0x0040=CALL/GOTO
	Z("QWORD"), 	CMD_QWORD		,1	,CNONE,0x0000,0x04E8,	// 0x0080=Marke2
	Z("FLOAT"), 	CMD_FLOAT		,1	,CNONE,0x0000,0x04E8,	// 0x0100=Bis-Dump
	Z("DOUBLE"), 	CMD_DOUBLE		,1	,CNONE,0x0000,0x04E8,	// 0x0200=Flags
	Z("STRING"), 	CMD_STRING		,1	,CNONE,0x0000,0x04E8,	// 0x0400=NotFlag
	Z("STRING_P"),	CMD_STRING_P	,1	,CNONE,0x0000,0x04E8,	// 0x0800=Marke vor Zahl
	Z("STRING_D"),	CMD_STRING_D	,1	,CNONE,0x0000,0x04E8,	// 0x1000=Marke vor Zahl ohne Suffix
	Z("STRING_B"),	CMD_STRING_B	,1	,CNONE,0x0000,0x04E8,	// 0x2000=Marke in Flags
	Z("STRING_K"),	CMD_STRING_K	,1	,CNONE,0x0000,0x04E8,
	Z("REST"), 		CMD_REST		,1	,CNONE,0x0010,0x04CC,
	Z("OFFSET"),	CMD_OFFSET		,1	,CNONE,0x0000,0x04E8,
	Z("LEN"),		CMD_LEN 		,1	,CNONE,0x0000,0x05EC,
	Z("IF"), 		CMD_IF			,0	,CNONE,0x0044,0x2002,
	Z("LOOP"), 		CMD_LOOP		,0	,CNONE,0x0000,0x2568,
	Z("BLOCK"), 	CMD_BLOCK		,0	,CNONE,0x0002,0x27CC,
	Z("MODUL"), 	CMD_MODUL		,0	,CNONE,0x0002,0x27FC,
	Z("RETURN"),	CMD_RETURN		,0	,CNONE,0x0000,0x27FC,
	Z("EXIT"), 		CMD_EXIT		,0	,CNONE,0x0000,0x27FC,
	Z("SEEK"),		CMD_SEEK		,0	,CNONE,0x0000,0x25CA,
	Z("TEXT"), 		CMD_TEXT		,0	,CNONE,0x0002,0x25FC,
	Z("CALL"),		CMD_CALL		,0	,CNONE,0x0004,0x25EA,
	Z("GOTO"), 		CMD_GOTO		,0	,CNONE,0x0004,0x25EA,
	Z("PRINT"), 	CMD_PRINT		,1	,CNONE,0x0004,0x04E8,
	Z("SPRINT"), 	CMD_SPRINT		,0	,CNONE,0x0084,0x2468,
	Z("ADD"),	 	CMD_ADD			,0	,CNONE,0x0804,0x254A,
	Z("MUL"),	 	CMD_MUL			,0	,CNONE,0x0804,0x254A,
	Z("SUB"),	 	CMD_SUB			,0	,CNONE,0x0804,0x254A,
	Z("AND"),	 	CMD_AND			,0	,CNONE,0x0804,0x254A,
	Z("DIV"),	 	CMD_DIV			,0	,CNONE,0x0804,0x254A,
	Z("MOD"),	 	CMD_MOD			,0	,CNONE,0x0804,0x254A,
	Z("BIN"),	 	CMD_BIN			,0	,CNONE,0x0804,0x254A,
	Z("MOV"),	 	CMD_MOV			,0	,CNONE,0x0804,0x254A,
	Z("DOWN"),		CMD_DOWN		,0	,CNONE,0x0000,0x25E8,
	Z("UP"),		CMD_UP			,0	,CNONE,0x0000,0x27FC,
	Z("RVA"), 		CMD_RVA			,0	,CNONE,0x0804,0x2148,
	};

static const zchar  *aSubCmds[]=
	{
	Z("=="),
	Z("!="),
	Z(">="),
	Z("<="),
	Z(">" ),
	Z("<" ),
	Z("&" ),
	};

#if _DEBUG
static CStringChain	aLineText;
#endif


#define 	CMD_MAX		(sizeof(aCommands)/sizeof(CommandData))

//*****************************************************************************
//*
//*		UniToStr
//*
//*****************************************************************************
//	Unicode-Strings in normale Strings umkopieren
//	pText	: Zeiger auf den Textpuffer in dem der String abgelegt wird
//	iMax	: Maximale Gre des Text Speichers
//	pMem	: Zeiger auf den
//	iSize	: Maximale Zahl an Unicode Zeichen die eingelesen werden
//	iMode	: Statt '\a'... '\x...' ausgeben
//	Ergibt die Lnge des gespeicherten Textes in Zeichen
static int UniToStr(zchar *pText,int iMax,const void *pMem,int iSize,int iMode)
{
unsigned short *pPtr=(unsigned short*)pMem;
int				iPos;


	if(iMax<=2)
		{
		if(iMax>0)pText[0]=0;
		return 0;
		}

	pText[0]='"';
	pText  +=1;
	iMax   -=3;

	for(iPos=0;iPos<iMax && iPos<iSize;iPos++)
		{
		#if UNICODE
		if(pPtr[iPos]<' ')
		#else
		if(pPtr[iPos]<' ' || pPtr[iPos]>255)
		#endif
				pText[iPos]='.';
		else	pText[iPos]=(zchar)pPtr[iPos];
		}

	pText[iPos  ] = '"';
	pText[iPos+1] = 0;



return iPos+2;
}


//*****************************************************************************
//*
//*		UniToStrM
//*
//*****************************************************************************
//	Unicode Strings in normale Strings kopieren (Motorolar Format)
//	pText	: Zeiger auf den Textpuffer in dem der String abgelegt wird
//	iMax	: Maximale Gre des Text Speichers
//	pMem	: Zeiger auf den
//	iSize	: Maximale Zahl an Unicode Zeichen die eingelesen werden
//	iMode	: Statt '\a'... '\x...' ausgeben
//	Ergibt die Lnge des gespeicherten Textes in Zeichen
static int UniToStrM(zchar *pText,int iMax,void *pMem,int iSize,int iMode)
{
unsigned short *pPtr=(unsigned short*)pMem;
unsigned short	uChar;
int				iPos;


	if(iMax<=2)
		{
		if(iMax>0)pText[0]=0;
		return 0;
		}


	pText[0]='"';
	pText  +=1;
	iMax   -=3;

	for(iPos=0;iPos<iMax && iPos<iSize;iPos++)
		{
		uChar = pPtr[iPos];
		uChar = (uChar>>8)|(uChar<<8);

		#if UNICODE
		if(uChar<' ')
		#else
		if(uChar<' ' || uChar>255)
		#endif
				pText[iPos]='.';
		else	pText[iPos]=(zchar)uChar;
		}

	pText[iPos  ]='"';
	pText[iPos+1]=0;



return iPos+2;
}

//*****************************************************************************
//*
//*		 RvaToFile
//*
//*****************************************************************************
//	Rechnet einen Offset im Speicher in einen Offset in einer Datei um
//	pSecPtr		: Zeiger auf die Instanz bzw. dem Instanz-Handle
//	uRvaOff		: Offset im Speicher der umgerechnet werden soll.
//				  Der Speicher beginnt bei der RVA-Adresse.
//	Ergibt den Dateioffset oder 0 bei einem Fehler.
static unsigned RvaToFile(const void *pSecPtr,unsigned uRvaOff,unsigned uCount=16)
{
ImageSectionHeader *pSections=(ImageSectionHeader*)pSecPtr;
unsigned			uFrom;
unsigned			uNum;
unsigned			uPos;
unsigned			uSec;


	for(uSec=0;uSec<uCount;uSec++)					// Abschnitt suchen
		{
		uNum  = (pSections[uSec].uVirtualSize+0x1FF)&~0x1FF;
		uFrom =  pSections[uSec].uPointerToRawData;
		uPos  =  pSections[uSec].uVirtualAddress;

		if(uRvaOff>=uPos && uRvaOff<uPos+uNum)		// Ist der Offest im Abschnitt
			{
			return uRvaOff+uFrom-uPos;
			}
		}


return 0;
}


//*****************************************************************************
//*
//*		 FileToRva
//*
//*****************************************************************************
//	Rechnet den Offset in der Datei in einen Offset im Speicher um
//	pSecPtr		: Zeiger auf die Instanz bzw. dem Instanz-Handle
//	uOffset		: Offset in der Datei der umgerechnet werden soll.
//				  Der Speicher beginnt bei der RVA-Adresse.
//	Ergibt den Dateioffset oder 0 bei einem Fehler.
static unsigned FileToRva(const void *pSecPtr,unsigned uOffset,unsigned uCount=16)
{
ImageSectionHeader *pSections=(ImageSectionHeader*)pSecPtr;
unsigned			uFrom;
unsigned			uNum;
unsigned			uPos;
unsigned			uSec;


	for(uSec=0;uSec<uCount;uSec++)					// Abschnitt suchen
		{
		uNum  = (pSections[uSec].uVirtualSize+0x1FF)&~0x1FF;
		uFrom =  pSections[uSec].uPointerToRawData;
		uPos  =  pSections[uSec].uVirtualAddress;

		if(uOffset>=uPos && uOffset<uPos+uNum)		// Ist der Offest im Abschnitt
			{
			return uOffset+uFrom-uPos;
			}
		}


return 0;
}

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CDataScanner::CDataScanner()
{
int		i;


	m_sModuls.DataLength(sizeof(int)*4);
	m_sBlocks.DataLength(sizeof(BlockStruct));
	m_sDumps .DataLength(sizeof(DumpStruct ));
	m_sData  .DataLength(sizeof(DataStruct ));

	m_iErrorMode   = 1;
	m_iErrorCount  = 0;
	m_iDefaultScan = 0;
	m_iGlobalFlags = 0;

	for(i=0;i<SCAN_MAX;i++)
		{
		m_sScans[i].DataLength(sizeof(ScanData));
		}

}


//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CDataScanner::~CDataScanner()
{

}

//*****************************************************************************
//*
//*		ErrorOut
//*
//*****************************************************************************
//	Fehler Ausgabe
void CDataScanner::ErrorOut(int iMode)
{
	m_iErrorMode=(iMode)? 1:0;
}

void CDataScanner::ErrorOut(CTextBuffer *pError)
{
			  m_pErrorOut =pError;
	if(pError)m_iErrorMode=2;
	else	  m_iErrorMode=0;

}
//*****************************************************************************
//*
//*		Error
//*
//*****************************************************************************
//	Fehler Ausgabe
void CDataScanner::Error(zchar *pText,int iMode)
{
zchar 	cBuffer[256];
int		iLen;

	   m_iErrorCount++;
	if(m_iErrorMode==0)return;
	if(m_iErrorMode==1)
		{
		printfz(Z("\n%s(%i)\t%s"),(iMode)? Z("Warnung"):Z("Fehler"),m_iLine,pText);
		return;
		}

	iLen=sprintfz(cBuffer,Z("%s(%i)\t%s\r\n"),(iMode)? Z("Warnung"):Z("Fehler"),m_iLine,pText);
	m_pErrorOut->Write(cBuffer,iLen*sizeof(zchar));

}

//*****************************************************************************
//*
//*		Errors
//*
//*****************************************************************************
//	Fehler Ausgabe
int CDataScanner::Errors()
{
return m_iErrorCount;
}

//*****************************************************************************
//*
//*		DumpCheck
//*
//*****************************************************************************
//	Prfen ob ein Dumps mit einem Speicherbereich zusammen passt
//	pMem	: Ist der Zeiger auf den Speicherbereich
//	iNum	: Ist die Nummer des Dumps
//	iOff	: Ist der Offset im Speicherbereich
//	iSize	: Ist die Gre des Speicherbereichs
//	Ergibt 1 wenn der Dump ok ist
int CDataScanner::DumpCheck(char *pMem,int iNum,int iOff,int iSize)
{
unsigned char	ucByte;
DumpStruct	   *pData;
zchar		   *pDump;
int				iLen,i;


	pData = (DumpStruct*)m_sDumps.Data(iNum);
	pDump = m_sDumps.Get(iNum);
	iLen  =	m_sDumps.Len(iNum)>>1;

	iOff +=	pData->iOff;

	if(pData->iMark>=0)
		{
		iOff += m_sData.DataGet(pData->iMark);
		}

	if(iOff<0 || (unsigned)(iOff+iLen)>(unsigned)iSize)
		{
		return 0;
		}

	pMem += iOff;

	for(i=0;i<iLen;i++)
		{
		ucByte  = (unsigned char)pMem [i    ];
		ucByte &= (unsigned char)pDump[i*2+1];
		ucByte ^= (unsigned char)pDump[i*2  ];

		if(ucByte)return 0;
		}


return 1;
}

//*****************************************************************************
//*
//*		DumpRead
//*
//*****************************************************************************
//	Dump einlesen:  Form  >X-8  00 A0 ? A6 ?? ~1??00011 FF   +2345
//	pMem	: Zeiger auf die Textstelle mit dem Dump-Angaben
//	pAdd	: Hier wird die Anzahl der gelesenen Zeichen gespeichert
//	Ergibt die Nummer mit der der Dump gespeichert wurde.
int CDataScanner::DumpRead(zchar *pMem,int *pAdd,CStringChain *pMarks)
{
DumpStruct	sDumpInfo;
zchar		cBuffer[64];
zchar		cChar;
int			iMark;
int 		iVal;
int 		iAnd;
int 		iCnt;
int 		iAdd;
int 		iTxt;
int 		iBit;
int 		iPos;
int			iOff;
int			iLen;
int 		iOk;
int			i;


	iAnd	= 0xFF;
	iMark	=-1;
	iAdd	=-1;
	iOff	= 0;
	iTxt	= 0;
	iCnt	= 0;
	iBit	= 0;
	iVal	= 0;
	iOk		= 0;

	iPos	= strwlen(pMem,0xA0);

	if(pMem[iPos]=='>')									// Offset Eintellungen
		{
		iPos++;

		if(strwchar(pMem[iPos],0x1E))					// Marke vor handen
			{
			iLen  = strwlen(pMem+iPos);
			iMark = pMarks->FindFast(iLen,pMem+iPos,0,-1,1);
			if(iMark>0)iMark = pMarks->DataGet(iMark);
			iPos += iLen;
			}

		iOff  = strtoi(pMem+iPos,0,&iLen);				// Offset einlesen
		iPos += iLen;
		iPos += strwlen(pMem+iPos,0xA0);
		}

	for(;iPos<256;iPos++)
		{
		cChar=pMem[iPos];

		if(cChar==' ' || cChar=='\t' || (iTxt&&iOk))	// Neue Zahl
			{
			if(!iOk)continue;

			cBuffer[iCnt]=iVal;iCnt++;
			cBuffer[iCnt]=iAnd;iCnt++;

			iOk  = 0;
			iBit = 0;
			iVal = 0;
			iAnd = 0xFF;

			if(iCnt>=64)break;
			if(iTxt== 0)continue;
			}

		if(cChar=='\"')									// String ein/aus
			{
			iTxt = !iTxt;
			continue;
			}

		if(iTxt)
			{
			if(cChar!='\\'       ){iVal=cChar      ;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='n' ){iVal='\n';iPos++;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='t' ){iVal='\t';iPos++;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='r' ){iVal='\r';iPos++;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='b' ){iVal='\b';iPos++;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='v' ){iVal='\v';iPos++;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='\\'){iVal='\\';iPos++;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='\''){iVal='\'';iPos++;iAnd=0xFF;iOk=1;continue;}
			if(pMem[iPos+1]=='\"'){iVal='\"';iPos++;iAnd=0xFF;iOk=1;continue;}
			continue;
			}

		if(cChar=='\"' && iTxt)
		if(cChar=='\'')									// Text Zeichen
		if(pMem[iPos+1]>=' ' && pMem[iPos+2]=='\'')
			{
			iVal = pMem[iPos+1];
			iAnd = 0xFF;
			iOk  = 1;
			continue;
			}

		if(cChar=='~' )									// Binr Zahl Anfang
			{
			iBit=1;
			continue;
			}

		if(cChar>='0' && cChar<='1' && iBit)			// Binr Zahl Digit
			{
			i    = (cChar=='0')? 0:1;
			iVal = (iVal<<1)|i;
			iAnd = (iAnd<<1)|1;
			iOk  = 1;
			continue;
			}

		if(cChar=='?' && iBit)							// Binr Zahl Auslassen
			{
			iVal <<= 1;
			iAnd <<= 1;
			iOk    = 1;
			continue;
			}

		if(cChar>='0' && cChar<='9' && !iBit)			// Hex Zahl
			{
			iVal = (iVal<<4)|(cChar-'0');
			iAnd = (iAnd<<4)|0x0F;
			iOk  =  1;
			continue;
			}

		if(cChar>='a' && cChar<='f' && !iBit)			// Hex Zahl
			{
			iVal = (iVal<<4)|(cChar-'a'+10);
			iAnd = (iAnd<<4)|0x0F;
			iOk  =  1;
			continue;
			}

		if(cChar>='A' && cChar<='F' && !iBit)			// Hex Zahl
			{
			iVal = (iVal<<4)|(cChar-'A'+10);
			iAnd = (iAnd<<4)|0x0F;
			iOk  = 1;
			continue;
			}

		if(cChar=='?' && !iBit)							// Leerzahl
			{
			iVal = 0;
			iAnd = 0;
			iOk  = 1;
			continue;
			}

		if(cChar=='+')									// Lngen Addierer
			{
			iAdd  = strtoi (pMem+iPos+1);
			iPos += strwlen(pMem+iPos+1,0x300002BF)+1;
			}

		break;
		}

	if(iOk)
		{
		cBuffer[iCnt]=iVal;iCnt++;
		cBuffer[iCnt]=iAnd;iCnt++;
		}

	iPos+=strwlen(pMem+iPos,0x008000A0);

	if(iCnt<= 0)return  -1;
	if(iAdd==-1)iAdd=iCnt>>1;

	sDumpInfo.iAdd	= iAdd;
	sDumpInfo.iOff	= iOff;
	sDumpInfo.iMark	= iMark;

	i = m_sDumps.Add(iCnt,cBuffer,CSC_END,&sDumpInfo);

	*pAdd=iPos;


return i;
}

//*****************************************************************************
//*
//*		PrintValue(64)
//*
//*****************************************************************************
//	Gibt eine Zahl in eine Puffer aus
//	iValue		: Ist die Zahl die augegeben werden soll
//	iCount		: Wird um 1 erhht wenn eine weiter Zahl ausgegeben wurde
//	iFlags		: Diese Flags bestimmen die Art der Textausgabe (siehe FLAG_???)
//	pBuffer		: Zeiger auf den Buffer in dem die Zahl geschrieben werden soll
//	iBufferSize	: Gre des Puffer in Zeichen
//	iValSize	: Ist die Gre der Zahl in Byte
//	iExtNr		: Ist die Nummer des Zuastztextes der angehngt werden soll
//	Ergibt die Annzahl der gespeicherten Zeichen
static const int  aPvFlags[]={FLAG_HEX,FLAG_BIN,FLAG_OCT,0};
static const int  aPvBase []={0x8010  ,0x8002,0x8008    ,0};
static const int  aPvMask []={0x000F  ,0x0001,0x0007    ,0};
static const int  aPvBits []={0x0004  ,0x0001,0x0003    ,0};
static const char aPvHexCh[]={'x'     ,'b'   ,'o'       ,0};
static const char aPvAsmCh[]={'h'     ,'b'   ,'o'       ,0};
int CDataScanner::PrintValue64(qint iValue,int &iCount,int iFlags,zchar *pBuffer,int iBufferSize,int iValSize,int iExtNr)
{
zchar  *pText;
char   *pByte;
char    cByte;
qint	iVal64;
qint	iMask;
int		iWidth;
int		iBits;
int		iLenT;
int		iLen;
int		iCnt;
int		i,k;



	if(!(iFlags&FLAG_PRINT))						// In anderer Zeile ausgeben
	if( (iFlags&FLAG_NOT  ))return 0;				// Nicht anzeigen
	if(iBufferSize<60	)return 0;					// Puffer zu klein


	iLen  = 0;
	iLenT = 0;
	pText = 0;


	if(iFlags&FLAG_PREFIX)							// Ein '@' Prfix ausgeben
		{
		pBuffer[iLen++]='@';
		}


	if(iFlags&FLAG_MOTOROLA)						// Motorolaformat ?
		{
		pByte=(char*)&iValue;

		for(i=0,k=iValSize-1;i<k;i++,k--)
			{
			cByte    = pByte[i];
			pByte[i] = pByte[k];
			pByte[k] = cByte;
			}
		}


	if(iCount>1)									// Beistrich anhngen
		{
		pBuffer[0] = ',';
		pBuffer[1] = ' ';
		iLen       = 2;
		}

	iCount=0;

	for(i=0;aPvFlags[i];i++)						// Hex/Oct/Bin Zahlen
		{
		if(!(iFlags&aPvFlags[i]))continue;
		if(!(iFlags&(FLAG_ASM|FLAG_NONE)))			// Ox... Zahlen
			{
			pBuffer[iLen++] = '0';
			pBuffer[iLen++] = aPvHexCh[i];
			}

		if(iFlags&FLAG_WIDTH)						// Konstante Breite
			{
			iWidth  =((iFlags&FLAG_WIDTH)>>12)+1;
			iMask   = aPvMask[i];
			iBits   = aPvBits[i];

			iVal64  = iValue&~iMask;
			iMask <<= iBits;
			for(iCnt=1;iVal64;iMask<<=iBits,iCnt++)iVal64&=~iMask;
			for(;iCnt<iWidth;iCnt++)pBuffer[iLen++]='0';
			}

		iLen+=qtostr(iValue,pBuffer+iLen,aPvBase[i]);

		if(iFlags&FLAG_ASM)							// Asm-Format
			{
			pBuffer[iLen++]=aPvAsmCh[i];
			}

		pBuffer[iLen++]=' ';
		iCount++;
		}

	if(iFlags&FLAG_CHAR)							// Als Zeichen ausgeben
		{
		iCount++;
		iLen+=ctostr((int)iValue,pBuffer+iLen);
		pBuffer[iLen++]=' ';
		}

	if(iFlags&FLAG_UNSIGNED)						// Vorzeichenlose Zahl
		{
		iCount++;
		iLen+=qtostr(iValue,pBuffer+iLen,15);
		pBuffer[iLen++]=' ';
		}

	if(iExtNr>=0)									// Einen Zusatztext anhngen
		{
		pText=m_sFlags[iExtNr];

		if(pText)
		while(*pText)								// Text durchlaufen
			{
			if(*pText=='?')
					iVal64=iValue;
			else	iVal64=strtoq(pText);

				pText+=strchar(pText,':');
			if(*pText)pText++;

			iCnt=strchar(pText,':');

			if(iVal64==iValue)						// Vergleiche Werte
				{
				iCount++;
				iLenT=iCnt;
				break;
				}

			pText+=iCnt;
			if(*pText)pText++;
			}
		}


	if(iCount<=0 || (iFlags&FLAG_SIGNED))			// Vorzeichenbehaftet Zahl
		{
		iCount++;
		iLen+=qtostr(iValue,pBuffer+iLen,0);
		pBuffer[iLen++]=' ';
		}

	if(iLenT)
		{
		if(iLenT>=iBufferSize-iLen)iLenT=iBufferSize-iLen;
		memcpy(pBuffer+iLen,pText,iLenT);
		iLen+=iLenT;
		}


return iLen;
}

int CDataScanner::PrintValue(int iValue,int &iCount,int iFlags,zchar *pBuffer,int iBufferSize,int iValSize,int iExtNr)
{
zchar  *pText;
char   *pByte;
char	cByte;
int		iWidth;
int		iMask;
int		iBits;
int		iLenT;
int		iLen;
int		iNum;
int		iCnt;
int		i,k;


	if(!(iFlags&FLAG_PRINT))						// In anderer Zeile ausgeben
	if( (iFlags&FLAG_NOT  ))return 0;				// Nicht anzeigen
	if(  iBufferSize<60	   )return 0;				// Puffer zu klein


	iLen  = 0;
	iLenT = 0;
	pText = 0;


	if(iFlags&FLAG_PREFIX)							// Ein '@' Prfix ausgeben
		{
		pBuffer[iLen++]='@';
		}


	if(iFlags&FLAG_MOTOROLA)						// Motorolaformat ?
		{
		pByte=(char*)&iValue;

		for(i=0,k=iValSize-1;i<k;i++,k--)
			{
			cByte    = pByte[i];
			pByte[i] = pByte[k];
			pByte[k] = cByte;
			}
		}


	if(iCount>1)									// Beistrich anhngen
		{
		pBuffer[0]=',';
		pBuffer[1]=' ';
		iLen=2;
		}

	iCount=0;

	for(i=0;aPvFlags[i];i++)						// Hex/Oct/Bin Zahlen
		{
		if(!(iFlags&aPvFlags[i]))continue;
		if(!(iFlags&(FLAG_ASM|FLAG_NONE)))			// Ox... Zahlen
			{
			pBuffer[iLen++]='0';
			pBuffer[iLen++]=aPvHexCh[i];
			}

		if(iFlags&FLAG_WIDTH)						// Konstante Breite
			{
			iWidth =((iFlags&FLAG_WIDTH)>>12)+1;
			iMask  = aPvMask[i];
			iBits  = aPvBits[i];

			iNum   =iValue&~iMask;
			iMask<<=iBits;
			for(iCnt=1;iNum;iMask<<=iBits,iCnt++)iNum&=~iMask;
			for(;iCnt<iWidth;iCnt++)pBuffer[iLen++]='0';
			}

		iLen+=itostr(iValue,pBuffer+iLen,aPvBase[i]);

		if(iFlags&FLAG_ASM)							// Asm-Format
			{
			pBuffer[iLen++]=aPvAsmCh[i];
			}

		pBuffer[iLen++]=' ';
		iCount++;
		}

	if(iFlags&FLAG_CHAR)							// Als Zeichen ausgeben
		{
		iCount++;
		iLen+=ctostr(iValue,pBuffer+iLen);
		pBuffer[iLen++]=' ';
		}

	if(iFlags&FLAG_UNSIGNED)						// Vorzeichenlose Zahl
		{
		iCount++;
		iLen+=itostr(iValue,pBuffer+iLen,15);
		pBuffer[iLen++]=' ';
		}

	if(iExtNr>=0)									// Einen Zusatztext anhngen
		{
		pText=m_sFlags[iExtNr];

		if(pText)
		while(*pText)								// Text durchlaufen
			{
			if(*pText=='?')
					iNum=iValue;
			else	iNum=strtoi(pText);

				pText+=strchar(pText,':');
			if(*pText)pText++;

			iCnt=strchar(pText,':');

			if(iNum==iValue)						// Vergleiche Werte
				{
				iCount++;
				iLenT=iCnt;
				break;
				}

			pText+=iCnt;
			if(*pText)pText++;
			}
		}


	if(iCount<=0 || (iFlags&FLAG_SIGNED))			// Vorzeichenbehaftet Zahl
		{
		iCount++;
		iLen+=itostr(iValue,pBuffer+iLen,0);
		pBuffer[iLen++]=' ';
		}

	if(iLenT)
		{
		if(iLenT>=iBufferSize-iLen)iLenT=iBufferSize-iLen;
		memcpy(pBuffer+iLen,pText,iLenT);
		iLen+=iLenT;
		}


return iLen;
}


//*****************************************************************************
//*
//*		BlockName
//*
//*****************************************************************************
zchar *CDataScanner::BlockName(unsigned int uBlockNr)
{

	if(uBlockNr>=(unsigned)m_sBlocks.Count())return  0;

return m_sBlocks.Get(uBlockNr);
}

//*****************************************************************************
//*
//*		BlockNumber
//*
//*****************************************************************************
//	Sucht die Block Nummer zum BlockNamen
//	Ist Modul==-1 so werden die Alle Blcke bercksichtigt
//	Ergibt Block Nummer bzw. -1 wenn nicht gefunden
int CDataScanner::BlockNumber(const zchar  *pBlockName,int iModul)
{
	if(iModul<=0)return m_sBlocks.Find(pBlockName);

return m_sBlocks.Find(pBlockName,ModulStart(iModul),ModulEnd(iModul));
}

//*****************************************************************************
//*
//*		BlockNumber
//*
//*****************************************************************************
//	Sucht die Modul Nummer zum ModulNamen
//	Ergibt Modul Nummer bzw. -1 wenn nicht gefunden
int CDataScanner::ModulNumber(const zchar *pModulName)
{
return m_sModuls.Find(pModulName);
}

//*****************************************************************************
//*
//*		ModulName
//*
//*****************************************************************************
//	Ergibt den Modulnamen zu einer Modulnummer
//	uModulNr	: Ist die Nummer des Moduls
zchar *CDataScanner::ModulName(unsigned int uModulNr)
{
	if(uModulNr>=(unsigned) m_sModuls.Count())return  0;

return m_sModuls.Get(uModulNr);
}

//*****************************************************************************
//*
//*		ModulStart
//*
//*****************************************************************************
//	Ergibt den Startblock zu einem Modul
//	uModulNr	: Ist die Nummer des Moduls
int CDataScanner::ModulStart(unsigned int uModulNr)
{
	if(uModulNr>=(unsigned)m_sModuls.Count())return m_sBlocks.Count();

return m_sModuls.DataGet(uModulNr);
}

//*****************************************************************************
//*
//*		ModulEnd
//*
//*****************************************************************************
//	Ergibt den Endblock zu einem Modul
//	uModulNr	: Ist die Nummer des Moduls
int CDataScanner::ModulEnd(unsigned int uModulNr)
{
	   uModulNr++;
	if(uModulNr>=(unsigned) m_sModuls.Count())return  m_sBlocks.Count();

return m_sModuls.DataGet(uModulNr);
}

//*****************************************************************************
//*
//*		Delete
//*
//*****************************************************************************
//	Lscht alle Daten in der Klasse
void CDataScanner::Delete()
{
int i;

	m_sBlocks.Del();							// Block  Daten
	m_sModuls.Del();							// Modul  Daten
	m_sDumps .Del();							// Bis-Dump Daten
	m_sFlags .Del();							// Flag Texte lschen
	m_sData  .Del(20,-1);						// Daten

	for(i=0;i<SCAN_MAX;i++)						// Scanner lschen
		{
		m_sScans  [i].Del();
		m_sStrings[i].Del();
		}
}

//*****************************************************************************
//*
//*		SetScanner
//*
//*****************************************************************************
//	Stellt den Standartscanner ein
//	uNum		: Ist die Nummer des Scanners der Standard werden soll
//	Ergibt die Nummer des neuen Standartscanners
int CDataScanner::SetScanner(unsigned int uNum)
{
int		iOld;

	if(uNum>=SCAN_MAX)
		{
		return m_iDefaultScan;
		}

	iOld =	m_iDefaultScan;
			m_iDefaultScan = uNum;

return  iOld;
}

//*****************************************************************************
//*
//*		LoadModul
//*
//*****************************************************************************
//	Ld ein Modul aus einer Textdatei
//	pFile	: Ist der Name der Moduldatei
//	pName	: Ist der ModulName (Bei 0 DateiName bzw. "Modul ???")
//	Ergibt		0 bei einem Fehler,
//				1 wenn breits geladen,
//				2 wenn neu es neu geladen wurde
int CDataScanner::LoadModul(const zchar *pFile,const zchar *pName)
{
zchar		cBuffer[64];
CTextBuffer	sFile;
int			i;


	m_iLine			= 0;
	m_iErrorCount	= 0;


	#if _DEBUG

	zchar cPath[MAX_PATH];

	GetExePath(cPath                ,MAX_PATH);
	strlcat(cPath,Z("..\\")         ,MAX_PATH);
	strlcat(cPath,GetFileName(pFile),MAX_PATH);

	if(ExistFile(cPath)==1)						// In Debug-Version lokale Module laden
		{
		pFile = cPath;
		}
	else{
		strlcat(cPath,Z("D:\\MyProjects\\HexEditor\\"),MAX_PATH);
		strlcat(cPath,GetFileName(pFile)              ,MAX_PATH);

		if(ExistFile(cPath)==1)
			{
			pFile = cPath;
			}
		}

	#endif


	if(!sFile.OpenAnsi(pFile))					// Datei ffnen
		{
		Error(Z("Kann Datei nicht ffnen."));
		return 0;
		}

	if(!pName)									// Dateinamen als Name
		{
		i=strzlen(pFile);
		for(;i>0;i--)
			{
			if(pFile[i]=='/' || pFile[i]=='\\')break;
			}

		if(pFile[i]=='/' || pFile[i]=='\\')i++;

				  cBuffer[sizeof(cBuffer)-1]=0;
		  strlcpy(cBuffer,pFile+i,sizeof(cBuffer)-1);
		i=strchar(cBuffer,'.');cBuffer[i]=0;

		pName=cBuffer;
		}



return LoadModul(&sFile,pName);
}

//*****************************************************************************
//*
//*		LoadModul
//*
//*****************************************************************************
//	Ld ein Modul aus einer Textdatei aus einem Textpuffer
//	Steht der Modul pName in der Datei wird pName berschrieben
//	Module mit gleichen Namen knnen nicht geladen werden.
//	pInput	: Ist der Puffer mit den Modul-Daten
//	pName	: Ist der pModulName (Bei 0 DateiName bzw. "Modul ???")
//	Ergibt		0 bei einem Fehler,
//				1 wenn breits geladen,
//				2 wenn neu es neu geladen wurde
int CDataScanner::LoadModul(CTextBuffer *pInput,const zchar *pName)
{
zchar		   *pMem;
int				i,k,j,l,iAdd,iLen,iPos,iCmd,iBit,iModulCount;
int				iStartBlock,iStartPos,iBlockBegin,aData[4];
zchar			cText[256],cModuleName[64],cTemp[16];
CStringChain	sMarks(sizeof(int)*2);
DataStruct		sDataS,*pPtr;
BlockStruct		sDataB;



	sDataS.ucCmd	= CMD_NULL;						// Spezielle Marken einfgen
	sDataS.ucPrint	= 0;
	sDataS.iMark1	=-1;
	sDataS.iMark2	=-1;
	sDataS.iNumber1	= 0;
	sDataS.iNumber2	= 0;
	sDataS.iDump	=-1;
	sDataS.iFlags	= 0;
	sDataS.iValue	= 0;
	sDataS.iSubCmd	= 0;
	sDataS.iFlagNr	=-1;
	sDataS.iOffset	= 0;
	sDataS.iPreMark	=-1;

	#if _DEBUG
	sDataS.iLine	= 0;							// Zeile des Befehls
	sDataS.iText	= 0;							// Text der Zeile
	#endif

	strcpy(cText,"LOCAL_");

	if(!m_sData.Count())							// Gibt es die lokalen Marken
		{
		for(i=0;i<MAX_LOCALS;i++)
			{
			cText[5] = '0'+i;
			m_sData.Add(cText,CSC_END,&sDataS);
			}
		}

	aData[1] = -1;

	for(i=0;i<MAX_LOCALS;i++)						// Referenzen fr spezielle Marken
		{
		aData[0] =  i;
		cText[5] = '0'+i;
		sMarks.Add(cText,CSC_END,aData);
		}

	strcpy(cText,"PARAM_");							// Globale Modul-Parameter einfgen

	for(i=0;i<MAX_PARAMS;i++)
		{
		cText[5] = '0'+i;
		aData[0] = m_sData.Count();
		sMarks .Add(cText,CSC_END, aData);			// Referenzen fr spezielle Marken
		m_sData.Add(cText,CSC_END,&sDataS);
		}


	iAdd			= 0;
	iBlockBegin		=-1;
	iStartPos		= m_sData  .Count();
	iStartBlock		= m_sBlocks.Count();
	iModulCount		= m_sModuls.Count();
	sDataB.iModul	= iModulCount;
	sDataB.iPos		= iStartPos;
	iPos			= iStartPos;
	m_iErrorCount	= 0;
	m_iLine			= 1;



//******************** Marken scannen *****************************************


	while(pInput->Text(iAdd,&pMem,1024))
		{
		if(iAdd=pInput->TextEmpty())				// Leerzeichen entfernen
			{
			pInput->Text(iAdd,&pMem,1024);
			}

		if(*pMem==':')								// Marke gefunden
			{
				 pInput->Text(1,&pMem,1024);
			iLen=pInput->TextWordLen();

			if(iLen)
				{
				if(sMarks.FindFast(iLen,pMem,0,-1,1)>=0)
					{
					zchar cMsg[256];
					sprintfz(cMsg,Z("Doppelte Marke '%.*s' gefunden"),iLen,pMem);
					Error(cMsg);
					}

				aData[0] = iPos;
				sMarks.Add(iLen,pMem,CSC_ABCG,aData);

					 pInput->Text(iLen,&pMem,1024);
				iAdd=pInput->TextEmpty();
					 pInput->Text(iAdd,&pMem,1024);
				}
			}

		if(*pMem==';')								// REM-Marke gefunden
			{
			pInput->TextLine();
			m_iLine++;
			iAdd=0;
			continue;
			}

			iLen=pInput->TextWordLen();
		if(!iLen)
			{
			pInput->TextLine();
			m_iLine++;
			iAdd=0;
			continue;
			}

		for(i=0;i<CMD_MAX;i++)						// Suche Kommando
			{
			if(!strzicmp(pMem,aCommands[i].pName,iLen))break;
			}

		if(i<CMD_MAX)iPos++;						// Kommando ok dann Position erhhen

		pInput->TextLine();
		m_iLine++;
		iAdd=0;
		}


//******************** Syntax scannen *****************************************


	iAdd	= 0;
	iPos	= iStartPos;
	m_iLine	= 1;
	pInput->Seek(0);

	while(pInput->Text(iAdd,&pMem,1024))
		{
		if(iAdd=pInput->TextEmpty())				// Leerzeichen entfernen
			{
			pInput->Text(iAdd,&pMem,1024);
			}

		#if _DEBUG
		sDataS.iLine	= m_iLine;
		sDataS.iText	= aLineText.Add(strcwlen(pMem,0x40),pMem);
		#endif

		if(pMem[0]==':')							// Marke entfernen
			{
				 pInput->Text(   1,&pMem,1024);
			iLen=pInput->TextWordLen();

			sDataS.iPreMark = sMarks.FindFast(iLen,pMem,0,-1,1);

				 pInput->Text(iLen,&pMem,1024);
			iAdd=pInput->TextEmpty();
				 pInput->Text(iAdd,&pMem,1024);
			}
		else{
			sDataS.iPreMark = -1;
			}

		if(*pMem==';')								// REM-Marke gefunden
			{
			pInput->TextLine();
			m_iLine++;
			iAdd=0;
			continue;
			}

			iLen=pInput->TextWordLen();				// Befehl suchen
		if(!iLen)
			{
			if(pMem[0]!='\n' && pMem[0]!='\r')Error(Z("Erwarte Befehl"));
			pInput->TextLine();
			m_iLine++;iAdd=0;
			continue;
			}

		for(iCmd=0;iCmd<CMD_MAX;iCmd++)				// Welcher Befehl
			{
			if(!strzicmp(pMem,aCommands[iCmd].pName,iLen))break;
			}

		if(iCmd>=CMD_MAX)
			{
			pInput->TextLine();
			Error(Z("Unbekannter Befehl"));
			m_iLine++;iAdd=0;
			continue;
			}


		pInput->Text(iLen,&pMem,1024);

		sDataS.ucCmd	= aCommands[iCmd].ucCmd;	// Grundeinstellung
		sDataS.ucPrint	= aCommands[iCmd].ucCmd;
		sDataS.iMark1	=-1;
		sDataS.iMark2	=-1;
		sDataS.iNumber1	= 0;
		sDataS.iNumber2	= 0;
		sDataS.iDump	=-1;
		sDataS.iFlags	= 0;
		sDataS.iValue	= 0;
		sDataS.iSubCmd	= 0;
		sDataS.iFlagNr	=-1;
		sDataS.iOffset	= 0;
		iBit			= 0;
		*(int*)cText=*(int*)Z("???");

		for(i=0;i<10;i++)
			{
			//******************************** Leerzeichen entfernen **********
			if(iAdd=pInput->TextEmpty())
				{
				pInput->Text(iAdd,&pMem,1024);
				}
			//******************************** Zeilen Ende ********************
			if(pMem[0]=='\0'){iAdd=0;break;}
			if(pMem[0]=='\n'){iAdd=0;break;}
			if(pMem[0]=='\r'){iAdd=0;break;}
			//******************************** Namen einlesen *****************
			if(pMem[0]=='\"')
				{
				if(iBit&F_NAME)
					{
					Error(Z("Name ist bereits angegeben"));
					iAdd=pInput->TextStringLen();
						 pInput->Text(iAdd,&pMem,1024);
					continue;
					}
				iBit|=F_NAME;
				iAdd=pInput->TextString(cText,sizeof(cText));
					 pInput->Text(iAdd,&pMem,1024);
				continue;
				}
			//******************************** Zahlen einlesen ****************
			if(pMem[0]=='-' || pMem[0]=='+' || (pMem[0]>='0' && pMem[0]<='9'))
				{
				k=strtoi(pMem+(pMem[0]=='+'),0,&iAdd);
				pInput->Text(iAdd,&pMem,1024);

				if(!(iBit&F_VAL1)){iBit|=F_VAL1;sDataS.iNumber1=k;continue;}
				if(!(iBit&F_VAL2)){iBit|=F_VAL2;sDataS.iNumber2=k;continue;}
				Error(Z("Zuviele Zahlen definiert"));
				continue;
				}
			//******************************** Flags einlesen *****************
			if(pMem[0]=='#')
				{
				for(iLen=1,k=0,j=0;iLen<16 && !j;iLen++)
				switch(pMem[iLen])
					{
				case 'b':case 'B':	k|=FLAG_BIN;		break;
				case 'o':case 'O':	k|=FLAG_OCT;		break;
				case 'd':case 'D':	k|=FLAG_DEC;		break;
				case 'h':case 'H':	k|=FLAG_HEX;		break;
				case 'x':case 'X':	k|=FLAG_ASM;		break;
				case 'k':case 'K':	k|=FLAG_NONE;		break;
				case 's':case 'S':	k|=FLAG_SIGNED;		break;
				case 'u':case 'U':	k|=FLAG_UNSIGNED;	break;
				case 'g':case 'G':	k|=FLAG_GLOBAL;		break;
				case 'c':case 'C':	k|=FLAG_CHAR;   	break;
				case 'n':case 'N':	k|=FLAG_NOT;	   	break;
				case 'm':case 'M':	k|=FLAG_MOTOROLA;  	break;
				case 'r':case 'R':	k|=FLAG_RASTER;		break;
				case 'a':case 'A':	k|=FLAG_ABSOLUT;	break;
				case 'z':case 'Z':	k|=FLAG_SETSIZE;	break;	
				case 'v':case 'V':	k|=FLAG_REGMODE;	break;
				case 'p':case 'P':	k|=FLAG_PREFIX;		break;
				case 'f':case 'F':	k|=FLAG_UTF8;		break;
				case 'i':case 'I':	k|=FLAG_SUFFIX;		break;
				case 'j':case 'J':	k|=FLAG_SUFFIXH;	break;
				case '"': 			j=2;				break;
				case '>':
								j = strwlen(pMem+iLen+1,0x0F);					
							if(!j)
								{
								zchar cMsg[256]=Z("Leere Marke in Flags");
								Error(cMsg);
								break;
								}

							l = sMarks.FindFast(j,pMem+iLen+1,0,-1,1);

							if(aCommands[iCmd].iNot&0x2000)
								{
								Error("Marken sind in den Flags nicht erlaubt");
								continue;
								}
							
							if(l==-1)
								{
								zchar   cMsg[256]=Z("Unbekannte Marke :");
								strlcpy(cMsg+18,pMem-iLen+1,256-18,j);
								Error  (cMsg);
								break;
								}
							
							if(sDataS.iFlags&FLAG_PRINT)
								{
								Error(Z("Einen Marke ist in den Flags bereits definiert"));
								break;
								}

							sDataS.iFlags|=FLAG_PRINT;
							sDataS.iMark2=l;
							iLen += j;
							continue;

				default:	if(pMem[iLen]>='0' && pMem[iLen]<='9')
								{
								j= pMem[iLen  ] -'0';
								if(pMem[iLen+1]<='9')
								if(pMem[iLen+1]>='0')
									{
									iLen++;
									j*=10;
									j+=pMem[iLen]-'0';
									}

								if(iCmd==CMD_SEEK)
									{
									k &= ~FLAG_WIDTH_EX;
									k |= ((j-1)<<12)&FLAG_WIDTH_EX;
									}
								else{
									k &= ~FLAG_WIDTH;
									k |= ((j-1)<<12)&FLAG_WIDTH;
									}

								j = 0;
								break;
								}

							j=1;iLen--;
							break;
					}

				if(j==2)			// Zusatztext gefunden
					{
					j=strcwlen(pMem+iLen,0x0140);
					sDataS.iFlagNr=m_sFlags.Add(j,pMem+iLen);
					iLen+=j;
					if(pMem[iLen]=='"')iLen++;
					}

				pInput->Text(iLen,&pMem,1024);

				if(!(iBit&F_FLAGS))
					{
					iBit|=F_FLAGS;
					sDataS.iFlags|=k;
					continue;
					}

				Error(Z("Flags bereits definiert."));
				continue;
				}
			//******************************** Dump einlesen ******************
			if(strwchar(pMem[0],0x06))
				{
			if(!strzicmp(pMem,Z("UNTIL"),5) && (pMem[5]==' ' || pMem[5]=='\t'))
				{
				k=DumpRead(pMem+6,&iAdd,&sMarks);
				iAdd+=6;
				pInput->Text(iAdd,&pMem,1024);

				if(!(iBit&F_DUMP)){iBit|=F_DUMP;sDataS.iDump=k;continue;}
				Error(Z("Bis-Dump bereits definiert."));
				continue;
				}
			//******************************** GOTO/CALL einlesen *************
			if(!strzicmp(pMem,Z("EXIT"),4) && strwchar(pMem[4],0x800000E0))
				{
				pInput->Text(5,&pMem,1024);
				if(!(iBit&F_CALL)){iBit|=F_CALL;sDataS.iSubCmd|=0x20;continue;}
				Error(Z("EXIT ist nicht erlaubt."));
				continue;
				}
			if(!strzicmp(pMem,Z("CALL"),4) && strwchar(pMem[4],0x800000E0))
				{
				pInput->Text(5,&pMem,1024);
				if(!(iBit&F_CALL)){iBit|=F_CALL;sDataS.iSubCmd|=0x40;continue;}
				Error(Z("CALL ist nicht erlaubt."));
				continue;
				}
			if(!strzicmp(pMem,Z("SEEK"),4) && strwchar(pMem[4],0x800000E0))
				{
				pInput->Text(5,&pMem,1024);
				if(!(iBit&F_CALL)){iBit|=F_CALL;sDataS.iSubCmd|=0xA0;continue;}
				Error(Z("SEEK ist nicht erlaubt."));
				continue;
				}
			if(!strzicmp(pMem,Z("GOTO"),4) && strwchar(pMem[4],0x800000E0))
				{
				pInput->Text(5,&pMem,1024);
				if(!(iBit&F_CALL)){iBit|=F_CALL;sDataS.iSubCmd|=0x80;continue;}
				Error(Z("GOTO ist nicht erlaubt."));
				continue;
				}
			if(!strzicmp(pMem,Z("RETURN"),6) && strwchar(pMem[6],0x800000E0))
				{
				pInput->Text(7,&pMem,1024);
				if(!(iBit&F_CALL)){iBit|=F_CALL;sDataS.iSubCmd|=0x60;continue;}
				Error(Z("RETURN ist nicht erlaubt."));
				continue;
				}
			if(!strzicmp(pMem,Z("MOV"),3) && strwchar(pMem[3],0x800000E0))
				{
				pInput->Text(4,&pMem,1024);
				if(!(iBit&F_CALL)){iBit|=F_CALL;sDataS.iSubCmd|=0xC0;continue;}
				Error(Z("MOV ist nicht erlaubt."));
				continue;
				}
			if(!strzicmp(pMem,Z("ADD"),3) && strwchar(pMem[3],0x800000E0))
				{
				pInput->Text(4,&pMem,1024);
				if(!(iBit&F_CALL)){iBit|=F_CALL;sDataS.iSubCmd|=0xE0;continue;}
				Error(Z("ADD ist nicht erlaubt."));
				continue;
				}
			//******************************** Marken einlesen ****************
				iLen=pInput->TextWordLen();
				k =  sMarks.FindFast(iLen,pMem,0,-1,1);
				pInput->Text(iLen,&pMem,1024);
				if(k==-1)
					{
					zchar   cMsg[256]=Z("Unbekannte Marke :");
					strlcpy(cMsg+18,pMem-iLen,256-18,iLen);
					Error  (cMsg);
					k = -1;
					}
				else{
					k = sMarks.DataGet(k);
					}
				if(!(iBit&(F_VAL1|F_VAL2))){iBit|=F_MARKF;}
				if(!(iBit&F_MARK1)){iBit|=F_MARK1;sDataS.iMark1=k;continue;}
				if(!(iBit&F_MARK2)){iBit|=F_MARK2;sDataS.iMark2=k;continue;}
				Error(Z("Zuviele Marken definiert"));
				continue;
				}
			//******************************** NotFlag ************************
			if(pMem[0]=='!' && pMem[1]!='=')
				{
				pInput->Text(iLen,&pMem,1);

				if(!(iBit&F_NOT)){iBit|=F_NOT;;continue;}
				Error(Z("Not Zeichen bereits definiert."));
				continue;
				}
			//******************************** IF Sub aCommands  einlesen *****
			for(k=0;k<7;k++)
				{
				if(aSubCmds[k][0]!=pMem[0])continue;
				if(aSubCmds[k][1])
				if(aSubCmds[k][1]!=pMem[1])continue;

				pInput->Text((aSubCmds[k][1])? 2:1,&pMem,1024);

				if(!(iBit&F_VERGL))
					{
					iBit|=F_VERGL;
					sDataS.iSubCmd|=k;
					break;
					}

				Error(Z("Unerlaubte Vergleichsfunktion."));
				break;
				}
			//******************************** REM Marke **********************
			if(pMem[0]==';')
				{
				iAdd=pInput->TextLineLen(1);
					 pInput->Text(iAdd);
				iAdd=0;
				break;
				}
			if(k<7){iAdd=0;continue;}
			Error(Z("Unbekanntes Zeichen"));
			iAdd=1;
			break;
			}

		if(sDataS.ucCmd==CMD_BLOCK && (iBit&F_NAME))	// Block speichern
			{
			if(!(iBit&F_VAL1) || !sDataS.iNumber1 || !(m_iGlobalFlags&sDataS.iNumber1))
			if(!(iBit&F_VAL2) || !sDataS.iNumber2 ||  (m_iGlobalFlags&sDataS.iNumber2))
				{
				sDataB.iPos=m_sData.Count();
				if(iBlockBegin==-1)iBlockBegin=sDataB.iPos;
				m_sBlocks.Add(cText,CSC_END,&sDataB);
				}
			}

		if(sDataS.ucCmd==CMD_MODUL && (iBit&F_NAME))	// Modul_name speichern
			{
			strlcpy(cModuleName,cText,sizeof(cModuleName));
			cModuleName[sizeof(cModuleName)-1]=0;
			pName=cModuleName;
			}

		pInput->TextLine();								// Daten einfgen
		sDataS.iFlags|=iBit&F_ALL;
		m_sData.Add(cText,CSC_END,&sDataS);

		   k  =  aCommands[iCmd].iMust & iBit;
		if(k !=  aCommands[iCmd].iMust)
			Error(Z("Syntaxfehler, fehlente Angaben"));
		if(iBit &aCommands[iCmd].iNot )
			Error(Z("Syntaxfehler, zuviele  Angaben"));

		m_iLine++;
		iPos++;
		}

//******************** Modul einfgen  ****************************************

	if(m_sModuls.Find(pName)!=-1)						// Modul lschen ?
		{
		m_iLine=0;
		Error(Z("Modul ist bereits geladen."),1);

		m_sData  .Del(iStartPos  ,m_sData  .Count());
		m_sBlocks.Del(iStartBlock,m_sBlocks.Count());
		return 1;
		}

	if(iBlockBegin!=-1)
	if(iBlockBegin==iStartPos+1)						// Erster Block nach Modul
		{
		   pPtr=(DataStruct*)m_sData.Data(iBlockBegin-1);
		if(pPtr->ucCmd==CMD_MODUL)iBlockBegin--;
		}

	if(iBlockBegin!=iStartPos)
		{
		if(!pName)										// Stantartnamen
			{
			itostr(iModulCount+1,cTemp,0);
			pName=cTemp;
			}

		strzcpy(cText,Z("Modul - "));
		strlcat(cText,pName,sizeof(cText));
		m_sBlocks.Add(cText,CSC_END,&iStartPos);
		}

	sDataS.ucCmd	= CMD_EXIT;							// Exit Befehl
	sDataS.ucPrint	= 0;
	sDataS.iFlags	= 0;

	aData[0]		= iStartBlock;
	aData[1]		= iStartPos;
	aData[2]		= m_sData.Count();
	aData[3]		= sMarks .Count();

	m_sData  .Add(Z(""),CSC_END,&sDataS);
	m_sModuls.Add(pName,CSC_END,&aData );				// Modul einfgen


return 2;
}

//*****************************************************************************
//*
//*		SetParam
//*
//*****************************************************************************
//	Einen Parameter eines Moduls setzen.
//	uModulNr		: Ist die Modulnummer
//	uNum			: Ist die Parameternummer (0..3)
//	iValue			: Ist der neue Wert fr die Marke
//	Ergibt die Markenposition oder -1 bei einem Fehler.
int CDataScanner::SetParam(unsigned uModulNr,unsigned uNum,int iValue)
{
DataStruct *pData;
int		   *pInfo;
int			iPos;


	if(uNum>=MAX_PARAMS)return -1;
	if(uModulNr>=(unsigned)m_sModuls.Count())return -1;

	pInfo  = (int*)m_sModuls.Data(uModulNr);
	iPos   = pInfo[1]-4+uNum;

	if(iPos<0)return -1;

	pData  = (DataStruct*)m_sData.Data(iPos);
	pData->iValue = iValue;


return iPos;
}

//*****************************************************************************
//*
//*		GetParam
//*
//*****************************************************************************
//	Einen Parameter eines Moduls auslesen.
//	uModulNr		: Ist die Modulnummer
//	uNum			: Ist die Parameternummer (0..3)
//	iValue			: Hier wird der Wert der Marke gespeichert
//	Ergibt die Markenposition oder -1 bei einem Fehler.
int CDataScanner::GetParam(unsigned uModulNr,unsigned uNum,int &iValue)
{
DataStruct *pData;
int		   *pInfo;
int			iPos;


	iValue = 0;

	if(uNum>=MAX_PARAMS)return -1;
	if(uModulNr>=(unsigned)m_sModuls.Count())return -1;

	pInfo  = (int*)m_sModuls.Data(uModulNr);
	iPos   = pInfo[1]-4+uNum;

	if(iPos<0)return -1;

	pData  = (DataStruct*)m_sData.Data(iPos);
	iValue = pData->iValue;


return iPos;
}

//*****************************************************************************
//*
//*		DeleteScan
//*
//*****************************************************************************
//	Lscht einen Scann
//	uNum	: Ist die Nummer des Scanns
void CDataScanner::DeleteScan(int uNum)
{
	if(uNum>=SCAN_MAX)return;
	if(uNum<0)uNum=m_iDefaultScan;

	m_sScans  [uNum].Del();							// Daten lschen
	m_sStrings[uNum].Del();
}

//*****************************************************************************
//*
//*		Data
//*
//*****************************************************************************
//	Holt eine Datenzeile aus einem Scann
//	uNum		: Ist die Nummer des Scanns
//	iPos		: Ist die Zeile die geholt werden soll
//	pText1		: Hier wird ein Zeiger auf den allgemeinen   Text gespeichert (1.Spalte)
//	pText2		: Hier wird ein Zeiger auf den individuellen Text gespeichert (2.Spalte)
//	pLen1		: Hier wird die Lnge von Text1 gespeichert
//	pLen2		: Hier wird die Lnge von Text2 gespeichert
//	Ergibt einen Zeiger auf die Variabeldaten
ScanData *CDataScanner::Data(unsigned uNum,int iPos,zchar **pText1,zchar **pText2,int *pLen1,int*pLen2)
{
CStringChain   *pString;
CStringChain   *pScan;
zchar		   *pText;
ScanData       *pPtr;
int				iCnt;
int				iLen;


	if(uNum>=SCAN_MAX || iPos<0)						// Falscher Index
		{
		if(pText1)*pText1=Z("");
		if(pText2)*pText2=Z("");
		if(pLen1 )*pLen1 =0;
		if(pLen2 )*pLen2 =0;
		return 0;
		}


	pScan = &m_sScans[uNum];
	iCnt  = pScan->Count();

	if(iPos>=iCnt)										// Auerhalb des Bereichs
 		{
		if(pText1)*pText1=Z("");
		if(pText2)*pText2=Z("");
		if(pLen1 )*pLen1 =0;
		if(pLen2 )*pLen2 =0;
		return 0;
		}

	pPtr	= (ScanData*)pScan->Data(iPos);
	pString	= &m_sStrings[uNum];

	if(pText1)											// Text fr Namen
		{
		if(pPtr->iTextNr<0)
				pText = pString->Get(-pPtr->iTextNr,iLen);
		else	pText = m_sData .Get( pPtr->iTextNr,iLen);

		if(!pText)pText=Z("");

		if(pText1)*pText1=pText;
		if(pLen1 )*pLen1 =iLen;
		}

	if(pText2)											// Text fr Werte
		{
			pText = pScan->Get(iPos,iLen);
		if(!pText)pText=Z("");

		if(pText2)*pText2=pText;
		if(pLen2 )*pLen2 =iLen;
		}


return pPtr;
}


//*****************************************************************************
//*
//*		Scan
//*
//*****************************************************************************
//	Scannt einen Speicherbereich und erzeugt die Zeilenzeilen zum Scan
//	uNum		: Nummer des zubenuzenden Scanners	   (0 bis SCAN_MAX)
//	pMem		: Speicher der zum auslesen ist
//	iMemLen		: Speicher Lnge
//	pBlock		: Blockname mit dem gescannt werden soll
//	iMax		: Maximal Eintagsanzahl fr den Scan
//	iOffBegin	: Start Offset im Speicher
//	Ergibt die erzeugten Textzeilen
int	CDataScanner::Scan(unsigned int uNum,void *pMem,int iMemLen,zchar *pBlock,int iMax,int iOffBegin)
{
int		iNum;


	   iNum=m_sBlocks.Find(pBlock);
	if(iNum==-1)return 0;


return Scan(uNum,pMem,iMemLen,iNum,iMax,iOffBegin);
}

//*****************************************************************************
//*
//*		Scan
//*
//*****************************************************************************
//	Scannt einen Speicherbereich und erzeugt die Zeilenzeilen zum Scan
//	uNum		: Nummer des zubenuzenden Scanners	   (0 bis SCAN_MAX)
//	pMemPtr		: Speicher der zum auslesen ist
//	iMemLen		: Speicher Lnge
//	uBlockNr	: Blocknummer mit dem zu benuzenden Datenformat
//	iMax		: Maximal Eintagsanzahl fr den Scan
//	iOffBegin	: Start Offset im Speicher
//	Ergibt die erzeugten Textzeilen
int	CDataScanner::Scan(unsigned int uNum,void *pMemPtr,int iMemLen,unsigned uBlockNr,int iMax,int iOffBegin)
{
ScanData		 sData;
ScanData	    *pData;
CStringChain	*pScan;
CStringChain	*pStrings;
BlockStruct		*pBlock;
DataStruct		*pDataS;
DataStruct		*pDataO;
int				*pModul;
int				*pMarks;
char			*pMem;
char			*pMemGlobal;
zchar			*pFormat;
zchar			*pText;
zchar			 cText[1024];
int				 i=0,k,j,iNum1,iNum2;
int				 iPos,iCmd,iAdd,iTextPos;
int				 iCnt=0,iLen=0,iOff;
int				 iStartOff,iBitOff;
int				 iLevel,iFlags,iBit;
int				 iUnicode,iMotorola;
int				 iOk,iFlagNr,iDump;
int				 iOutput,iVal;
qint			 iVal64=0;
double			 dValue=0;
int				 iGlobalOff;
int				 iCallNr;
int				 iMark1,iMark2;
int				 iLoopNr,iSource;
int				 aCallOff  [CALL_MAX];
int				 aCallPos  [CALL_MAX];
int				 aCallMode [CALL_MAX];
int				 aCallLoop [CALL_MAX];
int				 aCallPosL [CALL_MAX];
int				 aCallLevel[CALL_MAX];
char		    *aCallPtr  [CALL_MAX];



	if(uNum>=SCAN_MAX)return 0;
	if(iMemLen <0    )return 0;
	if(iMemLen==0    )return 0;
	if(uBlockNr>=(unsigned)m_sBlocks.Count())			// Existiert der Block
		{
		return  0;
		}

	pBlock	= (BlockStruct*)m_sBlocks.Data(uBlockNr);
	pModul	= (int        *)m_sModuls.Data(pBlock->iModul);
	pMarks	= new int[pModul[3]+1];

	if(!pMarks)											// Markenpuffer erzeugen
		{
		return 0;
		}

	memset(pMarks,0,sizeof(int)*(pModul[3]+1));


	for(iPos=0;iPos<4;iPos++)							// Lokale Marken auf 0 setzen
		{
		m_sData.DataSet(iPos,0);
		}

	pMem		= (char*)pMemPtr;
	pMemGlobal	= (char*)pMemPtr;
	iPos		= m_sBlocks.DataGet(uBlockNr);
	iOff		= iOffBegin;
	iLevel		= 0;
	iLoopNr		= 0;
	iBitOff		= 0;
	iCallNr		= 0;
	iGlobalOff	= 0;
	pScan		= &m_sScans  [uNum];
	pStrings	= &m_sStrings[uNum];
	pStrings->Del();
	pScan   ->Del();
	pStrings->Add(0,Z(""));



//******************** Hauptschleife ******************************************

while(iMax>0)
	{
	if(iGlobalOff+iOff>iMemLen)break;					// Bereichs Prfung
	if(iGlobalOff+iOff<0      )break;

		pDataS=(DataStruct*)m_sData.Data(iPos);			// Befehl ermitteln
	if(!pDataS)
		{
		sData.iMemOff  = iPos;
		sData.iMemOffG = 0;
		sData.iMemLen  = 0;
		sData.iTextNr  = 0;
		sData.iLevel   = iLevel;
		pScan->Add(Z("Programmfehler"),CSC_END,&sData);
		break;
		}

	iCmd			= pDataS->ucCmd;
	iMark1			= pDataS->iMark1;
	iMark2			= pDataS->iMark2;
	iNum1			= pDataS->iNumber1;
	iNum2			= pDataS->iNumber2;
	iDump			= pDataS->iDump;
	iFlags			= pDataS->iFlags;
	iFlagNr			= pDataS->iFlagNr;

	cText[0]		= 0;
	iTextPos		= 0;
	sData.iLevel	= iLevel;
	sData.iMemOff	= iOff;
	sData.iMemOffG	= iGlobalOff+iOff;
	pDataS->iOffset	= iGlobalOff+iOff;
	iStartOff		= iOff;

	#ifdef _DEBUG
	zchar	*pLine  = aLineText[pDataS->iText];
	int		 iLine	= pDataS->iLine;
	#endif



switch(iCmd&0xF0)
	{

//******************** Flieskomma Zahlen **************************************

	case 0x80:

		if(iBitOff)										// Bitoffset lschem
			{
			iOff+=(iBitOff+7)>>3;
			iBitOff=0;
			}

		iLen=(iCmd==CMD_FLOAT)? 4:8;

		if(iFlags&FLAG_SUFFIX)							// Suffixe anhngen "...[00]"
			{
			iNum1   = 1;
			iFlags &= ~F_MARK1;
			iFlags &= ~F_VAL1;
			}
		else if(!(iFlags&F_VAL1) && (iFlags&F_MARK1))	// Marke statt Anzahl
			{
			iNum1=m_sData.DataGet(iMark1);
			iFlags|=F_VAL1;
			}

		if(!(iFlags&F_VAL1) && iDump==-1)iNum1=1;		// Zahl gesetzt
		if(!iNum1 &&(iDump!=-1))iNum1=4096;				// Bis-Dump ohne Grenzen

		for(k=0;k<iNum1;k++)
			{
			if(iDump!=-1)								// Bis Dump
			if(DumpCheck(pMemGlobal,iDump,iGlobalOff+iOff,iMemLen))
				{
				iOff+=m_sDumps.DataGet(iDump);
				break;
				}

			dValue=(iCmd==CMD_FLOAT)? *(float*)(pMem+iOff):*(double*)(pMem+iOff);
			iOff+=iLen;
			if(iGlobalOff+iOff>iMemLen){iFlags|=FLAG_NOT;break;}
			if(iFlags & FLAG_NOT        )continue;		// Nicht anzeigen
			if(iTextPos>sizeof(cText)-42)continue;		// Text-Puffer zuklein

			if(k){strzcpy(cText+iTextPos,Z("  ,  "));iTextPos+=5;}
			iTextPos+=sprintfz(cText+iTextPos,Z("%e"),dValue);
			}

		pDataS->iValue=(int)dValue;

	break;

//******************** Ganzzahl Typen *****************************************

	case 0x90:

		if(iBitOff)										// Bitoffset lschem
			{
			iOff+=(iBitOff+7)>>3;
			iBitOff=0;
			}

		iLen=iCmd&0x0F;

		if(iFlags&FLAG_SUFFIX)							// Suffixe anhngen "...[00]"
			{
			iNum1   = 1;
			iFlags &= ~F_MARK1;
			iFlags &= ~F_VAL1;
			}
		else if(!(iFlags&F_VAL1) && (iFlags&F_MARK1))	// Marke statt Anzahl
			{
			iNum1=m_sData.DataGet(iMark1);
			iFlags|=F_VAL1;
			}

		if(!(iFlags&F_VAL1) && iDump==-1)iNum1=1;		// Zahl gesetzt
		if(!iNum1 &&(iDump!=-1))iNum1=4096;				// Bis-Dump ohne Grenzen

		for(k=0,iCnt=0;k<iNum1;k++)
			{
			if(iDump!=-1)								// Bis Dump prfen
			if(DumpCheck(pMemGlobal,iDump,iGlobalOff+iOff,iMemLen))
				{
				iOff+=m_sDumps.DataGet(iDump);
				break;
				}

			if(iGlobalOff+iOff+iLen>iMemLen)			// Speicher berlauf
				{
				iFlags|=FLAG_NOT;
				break;
				}

			i=*(int*)(pMem+iOff);						// Wert ermitteln
			i&=0xFFFFFFFF>>(32-(iLen<<3));
			iOff+=iLen;
			iTextPos+=PrintValue(i,iCnt,iFlags,cText+iTextPos,sizeof(cText)-iTextPos,iLen,iFlagNr);
			}

		pDataS->iValue=i;


		break;

//******************** Ganzzahl Typen 64 Bit **********************************

	case 0xA0:

		if(iBitOff)										// Bitoffset lschem
			{
			iOff+=(iBitOff+7)>>3;
			iBitOff=0;
			}

		iLen=iCmd&0x0F;

		if(iFlags&FLAG_SUFFIX)							// Suffixe anhngen "...[00]"
			{
			iNum1   = 1;
			iFlags &= ~F_MARK1;
			iFlags &= ~F_VAL1;
			}
		else if(!(iFlags&F_VAL1) && (iFlags&F_MARK1))	// Marke statt Anzahl
			{
			iNum1=m_sData.DataGet(iMark1);
			iFlags|=F_VAL1;
			}

		if(!(iFlags&F_VAL1)  && iDump==-1)iNum1=1;		// Zahl gesetzt
		if(!iNum1 &&(iDump!=-1))iNum1=4096;				// Bis-Dump ohne Grenzen

		for(k=0,iCnt=0;k<iNum1;k++)
			{
			if(iDump!=-1)								// Bis Dump prfen
			if(DumpCheck(pMemGlobal,iDump,iGlobalOff+iOff,iMemLen))
				{
				iOff+=m_sDumps.DataGet(iDump);
				break;
				}

			if(iGlobalOff+iOff+iLen>iMemLen)			// Speicher berlauf
				{
				iFlags|=FLAG_NOT;
				break;
				}

			iVal64	 = * (qint*)(pMem+iOff);			// Wert ermitteln
			iVal64	&= ~((qint )0)>>(64-(iLen<<3));
			iOff	+= iLen;
			iTextPos+= PrintValue64(iVal64,iCnt,iFlags,cText+iTextPos,sizeof(cText)-iTextPos,iLen,iFlagNr);
			}

		pDataS->iValue=(int)iVal64;


		break;

//******************** Strings ************************************************

	case 0x10:

		if(iBitOff)										// Bitoffset lschem
			{
			iOff+=(iBitOff+7)>>3;
			iBitOff=0;
			}

		iAdd		= 0;
		iLen		= 4096;
		iOk			= iFlags&F_VAL1;
		iUnicode	= iFlags&FLAG_UNICODE;
		iMotorola	= iFlags&FLAG_MOTOROLA;

		if(iFlags&FLAG_SUFFIX)							// Suffixe anhngen "...[00]"
			{
			iNum1   = 1;
			iFlags &= ~F_MARK1;
			iFlags &= ~F_VAL1;
			}
		else 		if(!iOk && (iFlags&F_MARK1))		// Marke statt Anzahl
			{
			iNum1=m_sData.DataGet(iMark1);
			iOk=1;
			}

		if(iOk && iGlobalOff+iOff+iNum1>iMemLen)iNum1=iMemLen-iOff-iGlobalOff;
		if(       iGlobalOff+iOff+4096 >iMemLen)iLen =iMemLen-iOff-iGlobalOff;

		if(iDump!=-1)									// Bis Dump
			{
			for(k=0,j=(iOk)? iNum1:iLen;k<j;k++)
				{
				if(DumpCheck(pMemGlobal,iDump,iGlobalOff+iOff+k,iMemLen))
					{
					iAdd  = m_sDumps.DataGet(iDump);
					iNum1 = k;
					iOk   = 1;
					break;
					}
				}
			}

		if(!iUnicode)
		{
		switch(iCmd)
			{
		case CMD_STRING:	j =(iOk)? iNum1:iLen;			// C-String
							for(iCnt=0;iCnt<j;iCnt++)
								if(pMem[iOff+iCnt]==0)break;
							iTextPos=memtostr(cText,sizeof(cText),pMem+iOff,iCnt,(iFlags&FLAG_REGMODE)? STR_REGMODE:0);
							if(!iOk)iNum1=iCnt+1;
							break;
		case CMD_STRING_D:	j =(iOk)? iNum1:iLen;			// Dos-String
							for(iCnt=0;iCnt<j;iCnt++)
								if(pMem[iOff+iCnt]=='$')break;
							iTextPos=memtostr(cText,sizeof(cText),pMem+iOff,iCnt,(iFlags&FLAG_REGMODE)? STR_REGMODE:0);
 							if(!iOk)iNum1=iCnt+1;
							break;
		case CMD_STRING_K:	j =(iOk)? iNum1:iLen;			// String-Kette
							k = iOff;
							for(iCnt=0;iCnt<j;iCnt++)
								{
								if(pMem[iOff+iCnt  ]!=0)continue;
								if(pMem[iOff+iCnt+1]==0){j=iCnt;}
								if(iTextPos>1000)continue;
								if(k!=iOff) {strzcpy(cText+iTextPos,Z("  ,  "));iTextPos+=5;}
								iTextPos+=memtostr(cText+iTextPos,sizeof(cText)-iTextPos,pMem+k,iOff+iCnt-k,(iFlags&FLAG_REGMODE)? STR_REGMODE:0);
								k=iOff+iCnt+1;
								}
							if(!iOk)iNum1=iCnt+1;
							break;
		case CMD_STRING_P:	iCnt=2+*(unsigned short*)(pMem+iOff);// Pascal String
							if(iOk && iCnt>iNum1)iCnt=iNum1;
							iTextPos=memtostr(cText,sizeof(cText),pMem+iOff+2,iCnt-2,(iFlags&FLAG_REGMODE)? STR_REGMODE:0);
							if(!iOk)iNum1=iCnt;
							break;
		case CMD_STRING_B:	iCnt=1+*(unsigned char *)(pMem+iOff);// Byte String
							if(iOk && iCnt>iNum1)iCnt=iNum1;
							iTextPos=memtostr(cText,sizeof(cText),pMem+iOff+1,iCnt-1,(iFlags&FLAG_REGMODE)? STR_REGMODE:0);
							if(!iOk)iNum1=iCnt;
							break;
			}

		if(iFlags&FLAG_UTF8)								// In UTF8 umwandeln
			{
			char cUtf[1024];
			strawcpy(cUtf,cText,sizeof(cUtf));
			iTextPos = strfromutf8(cText,cUtf,sizeof(cText)/sizeof(cText[0]));
			}
		}

		if(iUnicode && !iMotorola)
		{
		short *pMemU =(short*)(pMem+iOff);
		int    iLenU =iLen>>1;


		switch(iCmd)
			{
		case CMD_STRING:	j =(iOk)? iNum1:iLenU;			// C-String Unicode
							for(iCnt=0;iCnt<j;iCnt++)
								if(pMemU[iCnt]==0)break;
							iTextPos=UniToStr(cText,sizeof(cText),pMemU,iCnt,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=iCnt+1;
							iNum1<<=1;
							break;
		case CMD_STRING_D:	j =(iOk)? iNum1:iLenU;			// Dos-String Unicode
							for(iCnt=0;iCnt<j;iCnt++)
								if(pMemU[iCnt]=='$')break;
							iTextPos=UniToStr(cText,sizeof(cText),pMemU,iCnt,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=iCnt+1;
							iNum1<<=1;
							break;
		case CMD_STRING_K:	j =(iOk)? iNum1:iLenU;			// String-Kette Unicode
							k = iOff;
							for(iCnt=0;iCnt<j;iCnt++)
								{
								if(pMemU[iCnt  ]!=0)continue;
								if(pMemU[iCnt+1]==0){j=iCnt;}
								if(iTextPos>1000)continue;
								if(k!=iOff){strzcpy(cText+iTextPos,Z("  ,  "));  iTextPos+=5;}
								iTextPos +=UniToStr(cText+iTextPos,sizeof(cText)-iTextPos,pMem+k,(iOff+iCnt+iCnt-k)>>1,iFlags&FLAG_REGMODE);
								k=iOff+iCnt+iCnt+2;
								}
							if(!iOk)iNum1=iCnt+1;
							iNum1<<=1;
							break;
		case CMD_STRING_P:	iCnt=1+*(unsigned short*)(pMem+iOff);// Pascal String Unicode
							if(iOk && iCnt>iNum1)iCnt=iNum1;
							iTextPos=UniToStr(cText,sizeof(cText),pMem+iOff+2,iCnt-1,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=iCnt;
							iNum1<<=1;
							break;
		case CMD_STRING_B:	iCnt=1+*(unsigned char *)(pMem+iOff);// Byte String Unicode
							if(iOk && iCnt>iNum1)iCnt=iNum1;
							iTextPos=UniToStr(cText,sizeof(cText),pMem+iOff+1,iCnt,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=(iCnt<<1)+1;
							else   iNum1<<=1;
							break;
			}
		}

		if(iUnicode && iMotorola)
		{
		short *pMemU=(short*)(pMem+iOff);
		int    iLenU=iLen>>1;


		switch(iCmd)
			{
		case CMD_STRING:	j=(iOk)? iNum1:iLenU;			// C-String Unicode
							for(iCnt=0;iCnt<j;iCnt++)
								if(pMemU[iCnt]==0)break;
							iTextPos=UniToStrM(cText,sizeof(cText),pMemU,iCnt,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=iCnt+1;
							iNum1<<=1;
							break;
		case CMD_STRING_D:	j=(iOk)? iNum1:iLenU;			// Dos-String Unicode
							for(iCnt=0;iCnt<j;iCnt++)
								if(pMemU[iCnt]=='$')break;
							iTextPos=UniToStrM(cText,sizeof(cText),pMemU,iCnt,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=iCnt+1;
							iNum1<<=1;
							break;
		case CMD_STRING_K:	j=(iOk)? iNum1:iLenU;			// String-Kette Unicode
							k=iOff;
							for(iCnt=0;iCnt<j;iCnt++)
								{
								if(pMemU[iCnt  ]!=0)continue;
								if(pMemU[iCnt+1]==0){j=iCnt;}
								if(iTextPos>1000)continue;
								if(k!=iOff){strzcpy(cText+iTextPos,Z("  ,  "));  iTextPos+=5;}
								iTextPos+=UniToStrM(cText+iTextPos,sizeof(cText)-iTextPos,pMem+k,(iOff+iCnt+iCnt-k)>>1,iFlags&FLAG_REGMODE);
								k=iOff+iCnt+iCnt+2;
								}
							if(!iOk)iNum1=iCnt+1;
							iNum1<<=1;
							break;
		case CMD_STRING_P:	iCnt=1+*(unsigned short*)(pMem+iOff);// Pascal String Unicode
							if(iOk && iCnt>iNum1)iCnt=iNum1;
							iTextPos=UniToStrM(cText,sizeof(cText),pMem+iOff+2,iCnt-1,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=iCnt;
							iNum1<<=1;
							break;
		case CMD_STRING_B:	iCnt=1+*(unsigned char *)(pMem+iOff);// Byte String Unicode
							if(iOk && iCnt>iNum1)iCnt=iNum1;
							iTextPos=UniToStrM(cText,sizeof(cText),pMem+iOff+1,iCnt,iFlags&FLAG_REGMODE);
							if(!iOk)iNum1=(iCnt<<1)+1;
							else   iNum1<<=1;
							break;
			}
		}

		pDataS->iValue=(int)iCnt;
		iOff+=iNum1+iAdd;

	break;

//******************** Bit Typen **********************************************

	case 0x20:

		if(iBitOff>=8)									// Bitoffset lschem
			{
			iOff    += (iBitOff)>>3;
			iBitOff &= 7;
			}


		switch(iCmd)										// Welcher Befehl
			{
		case CMD_BIT:	if(!iNum1)iNum1=1;break;
		case CMD_REST:	if(!iNum1)iNum1=4;
						iLen=iOff;
						if(iFlags&FLAG_GLOBAL)iLen+=iGlobalOff;
						iLen=(iNum1-((iLen-iNum2)%iNum1))<<3;
						iNum1=iLen-iBitOff;
						break;
			}

		i=*(int*)(pMem+iOff);								// Wert ermitteln
		i=(i>>iBitOff)&(0xFFFFFFFF>>(32-iNum1));
		iBitOff+=iNum1;

		iLen=(iBitOff+7)>>3;								// Byte Lnge ermitteln
		if(iLen<0)iLen=1;

		if(iBitOff>=8)									// Bitoffset lschem
			{
			iOff+=(iBitOff)>>3;
			iBitOff&=7;
			}

		if(iGlobalOff+iOff>iMemLen)iFlags|=FLAG_NOT;

		iCnt=0;
		iTextPos+=PrintValue(i,iCnt,iFlags,cText+iTextPos,sizeof(cText)-iTextPos,iLen,iFlagNr);
		pDataS->iValue=i;

	break;

//******************** Diverse Funktionen *************************************

	case 0x30:

		switch(iCmd)
			{
		case CMD_IF:	iFlags|=FLAG_NOT;				// Nicht Anzeigen
						if(iDump!=-1)					// If UNTIL-Dumb
							{
							iMark2=iMark1;
							i=DumpCheck(pMemGlobal,iDump,iGlobalOff+iOff,iMemLen);
							if(iFlags&F_NOT)i=!i;
							iOff+=m_sDumps.DataGet(iDump);
							}
						else{							// If Vergleich
							iVal=m_sData.DataGet(iMark1);
							switch(pDataS->iSubCmd&0x0F)
									{
							case 0:	i=iVal==iNum1;	break;
							case 1:	i=iVal!=iNum1;	break;
							case 2:	i=iVal>=iNum1;	break;
							case 3:	i=iVal<=iNum1;	break;
							case 4:	i=iVal> iNum1;	break;
							case 5:	i=iVal< iNum1;	break;
							case 6:	i=iVal& iNum1;	break;
									}
							if(iFlags&F_NOT    )i=!i;
							if(iFlags&FLAG_UP  )m_sData.DataSet(iMark1,iVal+1);
							if(iFlags&FLAG_DOWN)m_sData.DataSet(iMark1,iVal-1);
							}
						if(!i)break;
														// Werte verschieben fr Sub-Befehl
						if(iDump==-1 && pDataS->iSubCmd>=0xA0)
							{
							iMark1 = iMark2;
							iNum1  = iNum2;

							if(iFlags&F_MARK2)
									iFlags&=~F_MARK2;
							else	iFlags&=~F_MARK1;

							if(iFlags&F_VAL2)
									iFlags&=~F_VAL2;
							else	iFlags&=~F_VAL1;
							}

						switch(pDataS->iSubCmd&0xE0)	// Sub - Funktion
							{
						case 0x20:						// EXIT
							iMax=0;
							continue;
						case 0x40:						// CALL
							iMark1=iMark2;
							goto CmdCall;
						case 0x60:						// RETURN
							goto CmdReturn;
							break;
						case 0x80:						// GOTO
							iPos=iMark2;
							continue;
						case 0xA0:						// SEEK
							iMark1=iMark2;
							iFlags &=~(F_MARK1|F_VAL1);
							if(iFlags&F_VAL2 )iFlags|=F_VAL1;
							if(iFlags&F_MARK2)iFlags|=F_MARK1;
							goto CmdSeek;
						case 0xC0:						// MOV
							m_sData.DataSet(iMark1,iNum1);
							break;
						case 0xE0:						// ADD
							m_sData.DataSet(iMark1,m_sData.DataGet(iMark1)+iNum1);
							break;
							}
						continue;						// Call - Funktion
		case CMD_CALL:
		CmdCall:		if(iCallNr>=CALL_MAX)
							{
							strzcpy(cText,Z("CALL berlauf !!!"));
							iTextPos=strzlen(cText);
							iFlags=0;iMax=0;
							break;
							}

						aCallPos  [iCallNr]=iPos+1;		// CALL-Stack
						aCallOff  [iCallNr]=iGlobalOff;
	   					aCallLoop [iCallNr]=iLoopNr;
						aCallLevel[iCallNr]=iLevel;
						aCallPtr  [iCallNr]=pMem;

						if(iFlags&FLAG_GLOBAL)
							{
							aCallMode[iCallNr]=1;
							}
						else{
							aCallMode[iCallNr]=0;
							iGlobalOff+=iOff;
							pMem+=iOff;iOff=0;
							}

						iPos=iMark1;
						iLoopNr=0;
						iCallNr++;
						continue;

		case CMD_RETURN:
		CmdReturn:		if(iCallNr<=0){iMax=0;continue;}// return
						if(iLoopNr>0)					// LOOP Schleife
							{
							iPos=aCallPosL[iCallNr-1];

							if(aCallMode[iCallNr-1]==0)
								{
								iGlobalOff += iOff;
								pMem       += iOff;
								iOff        = 0;
								}

							iLoopNr--;
							continue;
							}

						iCallNr--;						// CALL-Stack
						iPos		= aCallPos  [iCallNr];
						iLoopNr		= aCallLoop [iCallNr];
						iLevel		= aCallLevel[iCallNr];

						if(aCallMode[iCallNr]==0)
							{
							iLen		= pMem-aCallPtr[iCallNr];
							iGlobalOff	= aCallOff     [iCallNr];
							pMem		= aCallPtr     [iCallNr];
							iOff		= iLen+iOff;
							}

						continue;

		case CMD_EXIT:	iMax=0;continue;
		case CMD_LOOP:	if(iFlags&F_MARK2)				// LOOP Funktion
							{
							iVal=m_sData.DataGet(iMark1);
							}
						else{
							iVal=iNum1;
							iMark2=iMark1;
							}

						if(iVal<=0)
							{
							iPos++;
							continue;
							}

						aCallPos  [iCallNr]=iPos+1;		// CALL-Stack
						aCallOff  [iCallNr]=iGlobalOff;
	   					aCallLoop [iCallNr]=iLoopNr;
						aCallLevel[iCallNr]=iLevel;
						aCallPosL [iCallNr]=iMark2;
						aCallPtr  [iCallNr]=pMem;

						if(iFlags&FLAG_GLOBAL)
							{
							aCallMode[iCallNr]=1;
							}
						else{
							aCallMode[iCallNr]=0;
							iGlobalOff+=iOff;
							pMem+=iOff;iOff=0;
							}

						iPos=iMark2;
						iLoopNr=iVal-1;
						iCallNr++;
						continue;

		case CMD_MODUL:
		case CMD_BLOCK:	iPos++;continue;

		case CMD_SEEK:
		CmdSeek:		if(iFlags&F_MARK1)			 	// Verschub Anzahl
							{
							if(iFlags&FLAG_MARK)
								{
								pDataO = (DataStruct*)m_sData.Data(iMark1);
								iVal   = (pDataO)? pDataO->iOffset:0;
								iFlags|= FLAG_ABSOLUT|FLAG_GLOBAL;
								}
							else{
			  					iVal   = m_sData.DataGet(iMark1);
								}
							}
						else{
							iVal=0;
							}
						if(iFlags&F_VAL1)
							{
							iVal+=iNum1;
							}
						if(iFlags&FLAG_WIDTH_EX)		// Wert multiplizieren
							{
							iVal *= ((iFlags&FLAG_WIDTH_EX)>>12)+1;
							}
						if(iBitOff)						// Bitoffset lschem
							{
							iOff+=(iBitOff+7)>>3;
							iBitOff=0;
							}
						if(iFlags&FLAG_ABSOLUT)			// Absolut Verschub
							{
							if(iFlags&FLAG_GLOBAL)iOff=iVal-iGlobalOff;
							else				  iOff=iVal;
							if(iGlobalOff+iOff <0)iOff=-iGlobalOff;
							iPos++;

							if(iFlags&FLAG_BIN)			// Bereichs Prfung
								{
								if(iGlobalOff+iOff>iMemLen)iOff=-iGlobalOff;
								if(iGlobalOff+iOff<0      )iOff=-iGlobalOff;
								}
							continue;
							}
						if(!(iFlags&FLAG_RASTER))		// Normaler Verschub
							{
							iOff+=iVal;
							if(iGlobalOff+iOff<0)iOff=-iGlobalOff;
							iPos++;
							if(iFlags&FLAG_BIN)			// Bereichs Prfung
								{
								if(iGlobalOff+iOff>iMemLen)iOff=-iGlobalOff;
								if(iGlobalOff+iOff<0      )iOff=-iGlobalOff;
								}
							continue;
							}
	 					iLen=iOff;						// Raster Verschub
						if(iVal<=0)iVal=1;
						if(iFlags&FLAG_NOT   ){iOff--;iLen--;}
						if(iFlags&FLAG_GLOBAL)iOff+=iGlobalOff;
						iOff+=iVal-(iLen%iVal);
						iPos++;
						if(iFlags&FLAG_BIN)				// Bereichs Prfung
							{
							if(iGlobalOff+iOff>iMemLen)iOff=-iGlobalOff;
							if(iGlobalOff+iOff<0      )iOff=-iGlobalOff;
							}
						continue;

		case CMD_GOTO:	iPos=iMark1;					// GOTO - Funktion
						continue;

			}

	break;


//******************** Speicher Funktionen ************************************

	case 0x40:
		
		if(iFlags&FLAG_OCT)								// Die Offsets der Marken benutzen
			{
			i = (iFlags&FLAG_GLOBAL)? 0:iGlobalOff;

			if(!(iFlags&F_MARK2))						// Marke1 und Wert1 definiert ?
				{
				pDataO  = (DataStruct*)m_sData.Data(iMark1);
				iSource = (pDataO)? pDataO->iOffset-i:0;
				iVal	= iNum1;
				}
			else if(iFlags&F_VAL1)						// Marke2 und Wert1 definiert ?
				{
				pDataO  = (DataStruct*)m_sData.Data(iMark2);
				iSource = (pDataO)? pDataO->iOffset-i:0;
				iVal	= iNum1;
				}
			else{										// Marke1 und Marke2 definiert ?
				pDataO  = (DataStruct*)m_sData.Data(iMark1);
				iSource	= (pDataO)? pDataO->iOffset-i:0;
				pDataO  = (DataStruct*)m_sData.Data(iMark2);
				iVal	= (pDataO)? pDataO->iOffset-i:0;
				}
			}											
		else{
			if(!(iFlags&F_MARK2))						// Marke1 und Wert1 definiert ?
				{
				iSource = m_sData.DataGet(iMark1);
				iVal	= iNum1;
				}
			else if(iFlags&F_VAL1)						// Marke2 und Wert1 definiert ?
				{
				iSource = m_sData.DataGet(iMark2);
				iVal	= iNum1;
				}
			else{										// Marke1 und Marke2 definiert ?
				iSource	= m_sData.DataGet(iMark1);
				iVal	= m_sData.DataGet(iMark2);
				}
			}


		if(iFlags&FLAG_SAVE)							// Wert in der Zeile speichern
			{
			iMark1 = iPos;
			}

		switch(iCmd)
			{
		case CMD_ADD:	m_sData.DataSet(iMark1,iSource+iVal);		break;
		case CMD_MUL:	m_sData.DataSet(iMark1,iSource*iVal);		break;
		case CMD_SUB:	m_sData.DataSet(iMark1,iSource-iVal);		break;
		case CMD_AND:	m_sData.DataSet(iMark1,iSource&iVal);		break;
		case CMD_OR:	m_sData.DataSet(iMark1,iSource|iVal);		break;
		case CMD_XOR:	m_sData.DataSet(iMark1,iSource^iVal);		break;
		case CMD_DIV:	if(!iVal)iVal=1;
						m_sData.DataSet(iMark1,iSource/iVal);		break;
		case CMD_MOD:	if(!iVal)iVal=1;
						m_sData.DataSet(iMark1,iSource/iVal);		break;
		case CMD_BIN:	m_sData.DataSet(iMark1,(1<<iVal)+iNum1);	break;
		case CMD_MOV:	m_sData.DataSet(iMark1,iVal);				break;
			}

		iPos++;

	continue;

//******************** Ausgabe Funktionen *************************************

	case 0x50:

		switch(iCmd)
				{
		case CMD_TEXT:	break;
		case CMD_PRINT:									//Textzeile ausgeben
		case CMD_SPRINT:iVal = m_sData.DataGet(iMark1);
						iBit = (iCmd==CMD_SPRINT)? iFlags:iFlags&~FLAG_PRINT;
						iTextPos+=PrintValue(iVal,iCnt=0,iBit,cText+iTextPos,sizeof(cText)-iTextPos,sizeof(int),iFlagNr);
						if(iFlags&F_VAL1)
							{
							iStartOff = iOff-iNum1;
								pDataO= (DataStruct*)m_sData.Data(iMark1);
							if(!pDataO)break;
							sData.iMemOffG = pDataO->iOffset;
							sData.iMemOff  = pDataO->iOffset-iGlobalOff;
							}

						if(iCmd!=CMD_SPRINT)break;
														// Text ersetzen
						   pDataO = (DataStruct*)m_sData.Data(iMark2);
						if(pDataO && pDataO->iPreMark>=0 && pMarks[pDataO->iPreMark])
							{
							i = pMarks[pDataO->iPreMark];
							
							if(iFlags&FLAG_SETSIZE)
								{
								pData = (ScanData*)pScan->Data(i-1);
								pData->iMemLen = iVal;
								}
							else{
								pScan->Set(iTextPos,cText,i-1);
								}
							}

						iPos++;
						continue;

		case CMD_UP:	iLevel--;if(iLevel<0)iLevel=0;
						if(iFlags&F_NAME)break;
						iPos++;continue;
		case CMD_DOWN:	iLevel++;
						if(iFlags&F_NAME)break;
						iPos++;continue;
				}

		break;

//******************** Ausgabe Funktionen *************************************

	case 0x60:

		switch(iCmd)
			{
		case CMD_OFFSET:if(iFlags&F_MARK1)				// Offset von der Marke
							{
							pDataO = (DataStruct*)m_sData.Data(iMark1);
							i = (pDataO)? pDataO->iOffset-iGlobalOff:0;
							}
						else{							// Aktueller Offset
							i=(iFlags&FLAG_GLOBAL)? iOff+iGlobalOff:iOff;
							}

						if(iFlags&F_VAL1)i+=iNum1;
						iTextPos+=PrintValue(i,iCnt=0,iFlags,cText+iTextPos,sizeof(cText)-iTextPos,iLen,iFlagNr);
						break;
		case CMD_LEN:						   i =iMemLen-iGlobalOff-iOff;
						if(iFlags&FLAG_GLOBAL )i =iMemLen-iGlobalOff;
						if(iFlags&FLAG_ABSOLUT)i =iMemLen;
						if(iFlags&F_VAL1      )i+=iNum1;
						iTextPos+=PrintValue(i,iCnt=0,iFlags,cText+iTextPos,sizeof(cText)-iTextPos,iLen,iFlagNr);
						break;

		case CMD_RVA:	if(iFlags&F_MARK2)				// RVA Funktion
							{
							i = m_sData.DataGet(iMark2);
							}
						else if(iFlags&F_VAL1)			// Zahlen angabe
							{
							i = iNum1;
							}
						else{							// DWORD einlesen
							if(iBitOff)
								{
								iOff+=(iBitOff+7)>>3;	// Bitoffset lschem
								iBitOff = 0;
								}

							if(iGlobalOff+iOff+4>iMemLen)// Speicher berlauf
								{
								iFlags|=FLAG_NOT;
								i = 0;
								}
							else{
								i=*(int*)(pMem+iOff);	// Wert ermitteln
								iOff+=4;
								}
							}

						if(iFlags&FLAG_GLOBAL)			// Auf Globales Offset umrechnen
							{
							i += iGlobalOff;
							}

						if(!(iFlags&FLAG_AFTER))		// Wert vorher ausgeben
							{
							iTextPos += PrintValue(i,iCnt,iFlags&~FLAG_PRINT,cText+iTextPos,sizeof(cText)-iTextPos,iLen,iFlagNr);
							}


							pDataO = (DataStruct*)m_sData.Data(iMark1);
						if(!pDataO)						// Offset der Tabelle holen
							{
							k = -1;
							}
						else{
							k = pDataO->iOffset;
							}
														// Ist die Section im Speicher
						if((unsigned)k+sizeof(ImageSectionHeader)*16>(unsigned)iMemLen)
							{
							i = 0;
							}
						else if(iFlags&FLAG_RVA)		// Datei-Offset in RVA-Offset
							{
							i = FileToRva(pMemGlobal+k,i,16);
							}
						else{
							i = RvaToFile(pMemGlobal+k,i,16);
							}

						if(iFlags&FLAG_AFTER)			// Wert nacher ausgeben
							{
							iTextPos += PrintValue(i,iCnt,iFlags,cText+iTextPos,sizeof(cText)-iTextPos,iLen,iFlagNr);
							}
														// In Makre2 speichern
						if((iFlags&FLAG_SAVE) && (iFlags&F_MARK2))
							{
							m_sData.DataSet(iMark2,i);
							}

						break;
			}

		pDataS->iValue=i;

	break;

//*****************************************************************************

	}


	if(!(iFlags&FLAG_NOT))								// Den Scan einfgen
		{
		if(iFlags&FLAG_SUFFIX)
			{
			pText         =  m_sData.Get(iPos);
			pFormat		  =  (iFlags&FLAG_SAVE)? "%s[%0*Xh]":"%s[%0*i]";
			iNum1		  =  (pDataS->iFlags&F_VAL1)? pDataS->iNumber1:1;
			iVal		  =  m_sData.DataGet(iMark1);
			sData.iTextNr = -pStrings->AddFormat(pFormat,pText,iNum1,iVal);
			}
		else{											// Namensfeld direkt ausgeben
			sData.iTextNr = iPos;
			}

		sData.iMemLen	  = iOff-iStartOff;
		pScan->Add(iTextPos,cText,CSC_END,&sData);
		iMax--;

		if(pDataS->iPreMark>=0)							// Ausgabeposition von Zeilen mit Marken merken
			{
			pMarks[pDataS->iPreMark] = pScan->Count();
			}
		}

														// Text in eine andere Zeile ausgeben		
	if((iFlags&FLAG_PRINT) && pDataS->ucPrint && iTextPos>0 )
		{
		   pDataO = (DataStruct*)m_sData.Data(iMark2);
		if(iMark2>=0 && iMark2<=pModul[3])
			{
			   iOutput = pMarks[iMark2];
			if(iOutput>0)pScan->Set(iTextPos,cText,iOutput-1);
			}
		}

	iPos++;
	}


	delete pMarks;										// Markenpuffer lschen


return pScan->Count();
}



