//*****************************************************************************
//*
//*
//*		DiskIO.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
#include	<windows.h>
#include	<shlobj.h>
#include	<stdio.h>
#include	<dbt.h>
#include	"DiskIO.h"

#define 	VWIN32_DIOC_DOS_IOCTL		1
#define 	VWIN32_DIOC_DOS_DRIVEINFO	6

#ifndef 	IOCTL_CDROM_BASE
#define		IOCTL_CDROM_BASE						FILE_DEVICE_CD_ROM
#endif 	

#ifndef 	IOCTL_VOLUME_BASE
#define		IOCTL_VOLUME_BASE						((DWORD) 'V')
#endif 	

#ifndef 	IOCTL_DISK_GET_LENGTH_INFO
#define		IOCTL_DISK_GET_LENGTH_INFO				CTL_CODE(IOCTL_DISK_BASE, 0x0017, METHOD_BUFFERED, FILE_READ_ACCESS)
#endif 	

#ifndef 	IOCTL_DISK_UPDATE_PROPERTIES
#define		IOCTL_DISK_UPDATE_PROPERTIES			CTL_CODE(IOCTL_DISK_BASE,0x0050,METHOD_BUFFERED,FILE_ANY_ACCESS)
#endif 	

#ifndef 	IOCTL_CDROM_GET_DRIVE_GEOMETRY
#define		IOCTL_CDROM_GET_DRIVE_GEOMETRY			CTL_CODE(IOCTL_CDROM_BASE, 0x0013, METHOD_BUFFERED, FILE_READ_ACCESS)
#endif 	

#ifndef 	IOCTL_DISK_GET_DRIVE_GEOMETRY_EX
#define		IOCTL_DISK_GET_DRIVE_GEOMETRY_EX		CTL_CODE(IOCTL_DISK_BASE, 0x0028, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif 	

#ifndef 	IOCTL_DISK_GET_PARTITION_INFO_EX
#define		IOCTL_DISK_GET_PARTITION_INFO_EX		CTL_CODE(IOCTL_DISK_BASE, 0x0012, METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif 	

#ifndef 	IOCTL_VOLUME_GET_VOLUME_DISK_EXTENTS
#define		IOCTL_VOLUME_GET_VOLUME_DISK_EXTENTS	CTL_CODE(IOCTL_VOLUME_BASE,0,METHOD_BUFFERED, FILE_ANY_ACCESS)
#endif 	

#ifndef 	FSCTL_ALLOW_EXTENDED_DASD_IO
#define		FSCTL_ALLOW_EXTENDED_DASD_IO			CTL_CODE(FILE_DEVICE_FILE_SYSTEM,32,METHOD_NEITHER, FILE_ANY_ACCESS)
#endif 	


#ifndef 	WIN98_SUPPORT
#define 	WIN98_SUPPORT				0
#endif 	

#ifndef 	VOLUME_NAME_GUID
#define		VOLUME_NAME_GUID			0x01
#endif 	


typedef struct _DISK_EXTENT_
	{
	DWORD			DiskNumber;
	LARGE_INTEGER	StartingOffset;
	LARGE_INTEGER	ExtentLength;
	}DISK_EXTENT_;

typedef struct _DISK_GEOMETRY_EX 
	{
	DISK_GEOMETRY	Geometry;
	LARGE_INTEGER	DiskSize;
	BYTE			Data[1];
	}DISK_GEOMETRY_EX_;

typedef struct _PARTITION_INFORMATION_MBR 
	{
	BYTE    PartitionType;
	BOOLEAN BootIndicator;
	BOOLEAN RecognizedPartition;
	DWORD   HiddenSectors;
	}PARTITION_INFORMATION_MBR_;

typedef struct _PARTITION_INFORMATION_GPT 
	{
	GUID    PartitionType;
	GUID    PartitionId;
	DWORD64 Attributes;
	WCHAR   Name[36];
	}PARTITION_INFORMATION_GPT_;

typedef struct 
	{
	DWORD			PartitionStyle;							// 0=MBR 1=GPT 2=RAW
	LARGE_INTEGER   StartingOffset;
	LARGE_INTEGER   PartitionLength;
	DWORD           PartitionNumber;
	BOOLEAN         RewritePartition;
	union 
		{
		PARTITION_INFORMATION_MBR_ Mbr;
		PARTITION_INFORMATION_GPT_ Gpt;
		};
	}PARTITION_INFORMATION_EX_;

typedef struct _VOLUME_DISK_EXTENTS_
	{
	DWORD			NumberOfDiskExtents;
	DISK_EXTENT_	Extents[8];
	}VOLUME_DISK_EXTENTS_;


typedef struct
	{
    USHORT			Length;
    USHORT			MaximumLength;
    PWSTR  			Buffer;
	}NT_UNICODE_STRING;

typedef NT_UNICODE_STRING *NT_PUNICODE_STRING;

typedef struct
	{
    ULONG 			Length;
    HANDLE			RootDirectory;
    NT_PUNICODE_STRING ObjectName;
    ULONG 			Attributes;
    PVOID 			SecurityDescriptor;
    PVOID 			SecurityQualityOfService;
	}NT_OBJECT_ATTRIBUTES;

typedef NT_OBJECT_ATTRIBUTES *NT_POBJECT_ATTRIBUTES;


typedef LONG (WINAPI *NtCreateSymbolicLinkObjectType)(PHANDLE DirectoryHandle,ACCESS_MASK DesiredAccess,NT_POBJECT_ATTRIBUTES ObjectAttributes,NT_PUNICODE_STRING LinkTarget);
typedef LONG (WINAPI *NtOpenDirectoryObjectType)(PHANDLE DirectoryHandle,ACCESS_MASK DesiredAccess,NT_POBJECT_ATTRIBUTES ObjectAttributes);
typedef LONG (WINAPI *NtQueryDirectoryObjectType)(HANDLE DirectoryHandle,PVOID Buffer,ULONG Length,BOOLEAN ReturnSingleEntry,BOOLEAN RestartScan,PULONG Context,PULONG ReturnLength);
typedef LONG (WINAPI *NtOpenSymbolicLinkObjectType )(PHANDLE LinkHandle,ACCESS_MASK DesiredAccess,NT_POBJECT_ATTRIBUTES ObjectAttributes);
typedef LONG (WINAPI *NtQuerySymbolicLinkObjectType)(HANDLE LinkHandle,NT_PUNICODE_STRING LinkTarget,PULONG ReturnedLength);
typedef BOOL (WINAPI *GetMountPoint)(LPCTSTR pVolumeMountPoint,LPTSTR  pVolumeName,DWORD dwBufferLength);
typedef BOOL (WINAPI *SetMountPoint)(LPCTSTR pVolumeMountPoint,LPCTSTR pVolumeName);
typedef BOOL (WINAPI *DelMountPoint)(LPCTSTR pVolumeMountPoint);
typedef BOOL (WINAPI *GetVolNames  )(LPCTSTR pVolumeName,LPTSTR pVolumePathNames,DWORD dwBufferLength,PDWORD  pReturnLength);

static char iSectors[]=
	{
    0,	// Unknown                 Format is unknown
    15,	// F5_1Pt2_512             5.25" 1.2MB  512 bytes/sector
    18,	// F3_1Pt44_512            3.5"  1.44MB 512 bytes/sector
    36,	// F3_2Pt88_512            3.5"  2.88MB 512 bytes/sector
    0,	// F3_20Pt8_512            3.5"  20.8MB 512 bytes/sector
    9,	// F3_720_512              3.5"  720KB  512 bytes/sector
    9,	// F5_360_512              5.25" 360KB  512 bytes/sector
    8,	// F5_320_512              5.25" 320KB  512 bytes/sector
    };

static char iTracks[]=
	{
    80,	// Unknown                 Format is unknown
    80,	// F5_1Pt2_512             5.25" 1.2MB  512 bytes/sector
    80,	// F3_1Pt44_512            3.5"  1.44MB 512 bytes/sector
    80,	// F3_2Pt88_512            3.5"  2.88MB 512 bytes/sector
    0,	// F3_20Pt8_512            3.5"  20.8MB 512 bytes/sector
    80,	// F3_720_512              3.5"  720KB  512 bytes/sector
    40,	// F5_360_512              5.25" 360KB  512 bytes/sector
    40,	// F5_320_512              5.25" 320KB  512 bytes/sector
    };


#pragma pack(1)


typedef struct _ExtGetDskFreSpcStruc
	{
    WORD	wSize;
    WORD	wLevel;
    DWORD	dwSectorsPerCluster;
    DWORD	dwBytessPerSector;
    DWORD	dwAvailableClusters;
    DWORD	dwTotalClusters;
    DWORD	dwAvailablePhysSectors;
    DWORD	dwTotalPhysSectors;
    DWORD	dwAvailableAllocationUnits;
    DWORD	dwTotalAllocationUnits;
    DWORD	dwRsvd[2];
	}ExtGetDskFreSpcStruc;

typedef struct
	{
	DWORD	dwStartingSector;
	WORD	wNumberOfSectors;
	LPVOID	pBuffer;
	}CONTROL_BLOCK;

typedef struct _DIOC_REGISTERS
	{
	DWORD	reg_EBX;
	DWORD	reg_EDX;
	DWORD	reg_ECX;
	DWORD	reg_EAX;
	DWORD	reg_EDI;
	DWORD	reg_ESI;
	DWORD	reg_Flags;
	}DIOC_REGISTERS;

typedef struct _FVBLOCK
	{
    BYTE	fvSpecFunc;		// special functions = 0
    WORD	fvHead;			// head to format/verify
    WORD	fvCylinder;		// cylinder to format/verify
    WORD	fvTracks;		// tracks to format/verify
	}FVBLOCK;


#pragma pack()

static	OSVERSIONINFO	sInfo;
static	HINSTANCE		hKernel32	= 0;	
static  DWORD			dwOsVersion	= 0;
static	HANDLE			hIoSem		= 0;
static	HANDLE volatile	hThread		= 0;
static	HANDLE			aHandles[64];
static	BYTE			aReparse[64];
static	BYTE			aUnmount[64];
static	DWORD			aDrvList[64];
static	DWORD			aAccess	[64];
static	DWORD  volatile	aDoTick [64];
static	BYTE   volatile	aDoMount[64];
static	CHAR			aDrvGuid[64*64];
static	CHAR			aDrvName[64*64];

BOOL MountPysicalDisk(int iDrive);

//*****************************************************************************
//*
//*		IsWinNt
//*
//*****************************************************************************
static inline BOOL IsWinNt()
{
	
	#if WIN98_SUPPORT

	if(!dwOsVersion)
		{
		dwOsVersion = GetVersion();
		sInfo.dwOSVersionInfoSize=sizeof(sInfo);
		GetVersionEx(&sInfo);
		}

	if((dwOsVersion&0xFF00)==0x0100 && (dwOsVersion & 0x00FF)<=0x0004)
		{
		return FALSE;
		}

	#endif 	


return TRUE;
}


//*****************************************************************************
//*
//*		IsWin7
//*
//*****************************************************************************
static inline BOOL IsWin7()
{
return 0;


	if(!dwOsVersion)
		{
		dwOsVersion = GetVersion();
		sInfo.dwOSVersionInfoSize=sizeof(sInfo);
		GetVersionEx(&sInfo);
		}

	if((dwOsVersion&0xFF00)==0x0100 && (dwOsVersion & 0x00FF)<=0x0004)
		{
		return FALSE;
		}


return (sInfo.dwMajorVersion>=6)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		StrFilter
//*
//*****************************************************************************
//	Vergleich einen Text mit einen Vergleichstext samt Wildcards (* ?)
//	pString		: Ist der Zeiger auf den Text
//	pCompare	: Ist der Zeiger auf den Vergleichstext (mit * ?)
//	Ergibt 0 wenn die Texte bereinstimmen.
static int StrFilter(const wchar_t *pString, const wchar_t *pCompare)
{
int		iDiff;


	while(*pCompare)
		{
		if(*pCompare=='?')								// berspringe einzelnes Zeichen
			{
			if(!*pString)
				{
				return 1;
				}
			}
		else if(*pCompare=='*')							// Rekursiver Vergleich
			{
			if(!StrFilter(pString,pCompare+1))
				{
				return 0;
				}

			if(*pString && !StrFilter(pString+1,pCompare))
				{
				return 0;
				}

			return 1;
			}
		else{											// Vergleiche einzelnes Zeichen
			iDiff = *pCompare-*pString;

			if(iDiff)
				{
				return iDiff;
				}
			}

		pCompare++;
		pString++;
		}


	if(*pString)										// War alles gleich
		{
		return 1;
		}


return 0;
}

//*****************************************************************************
//*
//*		GetVolumeNameForVolumeMountPoint
//*
//*****************************************************************************
static BOOL WINAPI GetVolumeNameForVolumeMountPoint_(LPCTSTR pVolumeMountPoint,LPTSTR pVolumeName,DWORD dwBufferLength)
{
static	GetMountPoint	pGetMountPoint=0;
 
 
	if(pVolumeName)*pVolumeName=0;
 
	if(!hKernel32)
		{
			hKernel32=LoadLibraryA("KERNEL32");
		if(!hKernel32)return FALSE;
		}
 
	if(!pGetMountPoint)
		{
			pGetMountPoint = (GetMountPoint)GetProcAddress(hKernel32,"GetVolumeNameForVolumeMountPointA");
		if(!pGetMountPoint)return FALSE;
		}
 
 
return pGetMountPoint(pVolumeMountPoint,pVolumeName,dwBufferLength);
}
 
//*****************************************************************************
//*
//*		SetVolumeMountPoint
//*
//*****************************************************************************
static BOOL WINAPI SetVolumeMountPoint_(LPCTSTR pVolumeMountPoint,LPCTSTR pVolumeName)
{
static	SetMountPoint	pSetMountPoint=0;
 
 
	if(!hKernel32)
		{
			hKernel32=LoadLibraryA("KERNEL32");
		if(!hKernel32)return FALSE;
		}
 
	if(!pSetMountPoint)
		{
			pSetMountPoint = (SetMountPoint)GetProcAddress(hKernel32,"SetVolumeMountPointA");
		if(!pSetMountPoint)return FALSE;
		}
 
 
return pSetMountPoint(pVolumeMountPoint,pVolumeName);
}
 
//*****************************************************************************
//*
//*		DeleteVolumeMountPoint
//*
//*****************************************************************************
static BOOL WINAPI DeleteVolumeMountPoint_(LPCTSTR pVolumeMountPoint)
{
static	DelMountPoint	pDelMountPoint=0;
 
 
	if(!hKernel32)
		{
			hKernel32=LoadLibraryA("KERNEL32");
		if(!hKernel32)return FALSE;
		}
 
	if(!pDelMountPoint)
		{
			pDelMountPoint = (DelMountPoint)GetProcAddress(hKernel32,"DeleteVolumeMountPointA");
		if(!pDelMountPoint)return FALSE;
		}
 
 
return pDelMountPoint(pVolumeMountPoint);
}
 
//*****************************************************************************
//*
//*		GetVolumeNameForVolumeMountPoint
//*
//*****************************************************************************
static BOOL WINAPI GetVolumePathNamesForVolumeName_(LPCTSTR pVolumeName,LPTSTR pVolumePathNames,DWORD dwBufferLength,PDWORD  pReturnLength)
{
static	GetVolNames	pGetVolNames=0;
 
 
	if(pVolumePathNames)*pVolumePathNames=0;
	if(pReturnLength   )*pReturnLength=0;
 
	if(!hKernel32)
		{
			hKernel32=LoadLibraryA("KERNEL32");
		if(!hKernel32)return FALSE;
		}
 
	if(!pGetVolNames)
		{
			pGetVolNames = (GetVolNames)GetProcAddress(hKernel32,"GetVolumePathNamesForVolumeNameA");
		if(!pGetVolNames)return FALSE;
		}
 
  
return pGetVolNames(pVolumeName,pVolumePathNames,dwBufferLength,pReturnLength);
}
 
//*****************************************************************************
//*
//*		FakeDosNameForDevice
//*
//*****************************************************************************
//	Erzeugt einen DOS-Namen fr einen Pfad
//	pDiskFile		: Zeiger auf den Pfad
//	pDosDevice		: Hier wird der DOS-Name gespeichert (frs Lschen)
//	pDeviceName		: Hier wird der Device-Name gespeichert (fr CreateFile)
//	bNameOnly		: Nur die Namen speichern
//	Ergibt TRUE wenn der Namen erzeugt wurde.
static BOOL FakeDosNameForDevice(const char *pDiskFile,char *pDosDevice,char *pDeviceName,BOOL bNameOnly)
{

	if(pDiskFile[0]=='\\' && pDiskFile[1]=='\\')
		{
		strcpy(pDeviceName,pDiskFile);
		pDosDevice[0]=0;
		return FALSE;
		}

	sprintf(pDosDevice,"diskio%u",GetCurrentProcessId());

	if(bNameOnly==FALSE)
		{
		if(!DefineDosDevice(DDD_RAW_TARGET_PATH,pDosDevice,pDiskFile))
			{
			pDeviceName[0]=0;
			pDosDevice [0]=0;
			return FALSE;
			}
		}

	sprintf(pDeviceName,"\\\\.\\%s",pDosDevice);



return TRUE;
}


//*****************************************************************************
//*
//*		RemoveFakeDosName
//*
//*****************************************************************************
//	Lscht einen DOS-Namen wieder
//	pDiskFile		: Zeiger auf den Pfad
//	pDosDevice		: Zeiger auf den DOS-Namen
//	Ergibt TRUE wenn der Namen gelscht wurde wurde.
static BOOL RemoveFakeDosName(const char *pDiskFile,char *pDosDevice)
{
BOOL bDosLinkRemoved;


	if(!*pDosDevice)return FALSE;

	bDosLinkRemoved = DefineDosDevice(DDD_RAW_TARGET_PATH|DDD_EXACT_MATCH_ON_REMOVE|DDD_REMOVE_DEFINITION,pDosDevice,pDiskFile);


return bDosLinkRemoved;
}
		
//*****************************************************************************
//*
//*		QuerySymbolLink
//*
//*****************************************************************************
//	Sucht den Namen auf den ein symolischer Link zeigt
//	pLinkName		: Ist der Name des symolischen Links
//	pTarget			: Hier wird der Name gespeichert
//	iSize			: Gre des Puffers fr den Namen
//	Ergibt TRUE wenn der Name ausgelesen werden konnte.
static BOOL QuerySymbolLink(const char *pLinkName,char *pTarget,int iSize)
{
static	NtQuerySymbolicLinkObjectType	pQueryLink=0;
static	NtOpenSymbolicLinkObjectType	pOpenLink=0;
static	HMODULE							hNtDll=0;
WCHAR									cBuffer[256];
NT_OBJECT_ATTRIBUTES					sObject;
NT_UNICODE_STRING						sName;
HANDLE									hLink;
int										iPos;



	if(iSize<=0)return FALSE;

	*pTarget = 0;

	if(!hNtDll)
		{
		hNtDll     = GetModuleHandle("ntdll");
		pOpenLink  = (NtOpenSymbolicLinkObjectType )GetProcAddress(hNtDll,"NtOpenSymbolicLinkObject");
		pQueryLink = (NtQuerySymbolicLinkObjectType)GetProcAddress(hNtDll,"NtQuerySymbolicLinkObject");
		}

	if(!pOpenLink || !pOpenLink)
		{
		return FALSE;
		}



	sName.Buffer		= cBuffer;
	sName.MaximumLength	= 2*255;
	sName.Length		= 0;

	for(;sName.Length<sName.MaximumLength;sName.Length+=2)
		{
		cBuffer[sName.Length/2]	= pLinkName[sName.Length/2];
		if(!pLinkName[sName.Length/2])break;
		}

	sObject.Length					 = sizeof(sObject);
	sObject.ObjectName				 = &sName;
	sObject.Attributes				 = 0x40;
	sObject.RootDirectory			 = NULL;
	sObject.SecurityDescriptor		 = NULL;
	sObject.SecurityQualityOfService = NULL;

	pOpenLink(&hLink,1,&sObject);


	if(!hLink)
		{
		return FALSE;
		}

	if(pQueryLink(hLink,&sName,NULL))
		{
		sName.Length = 0;
		}

	CloseHandle(hLink);
	sName.Length/=2;
	iSize--;

	for(iPos=0;iPos<sName.Length;iPos++)
		{
		if(iPos>=iSize)break;
		pTarget[iPos] = (char)cBuffer[iPos];
		}

	pTarget[iPos] = 0;


return (sName.Length>0)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		QuerySourceFromLink
//*
//*****************************************************************************
//	Sucht den Namen der Quelle auf den ein symolischer Link zeigt
//	pSourceDir		: Ist das Verzeichniss in dem gesucht werden soll Quelle
//					  z.B. "\\GLOBAL??"
//	pLinkName		: Ist der Name des symolischen Links
//	pTarget			: Hier wird der Name gespeichert
//	iSize			: Gre des Puffers fr den Namen
//	Ergibt die Anzahl der gefundenen Quellen
static int QuerySourceFromLink(const char *pSourceDir,const char *pLinkName,char *pTarget,int iSize)
{
static	NtQuerySymbolicLinkObjectType	pQueryLink=0;
static	NtOpenSymbolicLinkObjectType	pOpenLink=0;
static	NtQueryDirectoryObjectType		pQueryDir=0;
static	NtOpenDirectoryObjectType		pOpenDir=0;
static	HMODULE							hNtDll=0;
WCHAR									cCmpTxt[256];
WCHAR									cString[256];
WCHAR									cBuffer[256];
DWORD									dwContext;
LPBYTE									pMemory;
NT_UNICODE_STRING					   *pString;
NT_OBJECT_ATTRIBUTES					sObject;
NT_UNICODE_STRING						sName;
NT_UNICODE_STRING						sTemp;
HANDLE									hLink;
HANDLE									hDir;
int										iLen;
int										iPos;
int										iNum;
int 									iRet;


	if(iSize<=1)
		{
		return FALSE;
		}

		iNum = strlen(pLinkName);
	if(!iNum || iNum>=256)
		{
		return FALSE;
		}

	iSize     -= 2;
	pTarget[0] = 0;
	pTarget[1] = 0;

	if(!hNtDll)
		{
		hNtDll     = GetModuleHandle("ntdll");
		pOpenDir   = (NtOpenDirectoryObjectType    )GetProcAddress(hNtDll,"NtOpenDirectoryObject"    );
		pQueryDir  = (NtQueryDirectoryObjectType   )GetProcAddress(hNtDll,"NtQueryDirectoryObject"   );
		pOpenLink  = (NtOpenSymbolicLinkObjectType )GetProcAddress(hNtDll,"NtOpenSymbolicLinkObject" );
		pQueryLink = (NtQuerySymbolicLinkObjectType)GetProcAddress(hNtDll,"NtQuerySymbolicLinkObject");
		}

	if(!pOpenLink || !pOpenLink || !pOpenLink || !pQueryLink)
		{
		return FALSE;
		}


	sName.Buffer		= cBuffer;
	sName.MaximumLength	= 2*255;
	sName.Length		= 0;

	for(;sName.Length<sName.MaximumLength;sName.Length+=2)
		{
		cBuffer[sName.Length/2]	= pSourceDir[sName.Length/2];
		if(!pSourceDir[sName.Length/2])break;
		}

	sObject.Length					 = sizeof(sObject);
	sObject.ObjectName				 = &sName;
	sObject.Attributes				 = 0x00;
	sObject.RootDirectory			 = NULL;
	sObject.SecurityDescriptor		 = NULL;
	sObject.SecurityQualityOfService = NULL;

	pOpenDir(&hDir,1,&sObject);
	if(!hDir)return FALSE;

	iLen	= 0x10000;
	pMemory	= (LPBYTE)VirtualAlloc(NULL,iLen,MEM_COMMIT,PAGE_READWRITE);

	if(!pMemory)
		{
		CloseHandle(hDir);
		return FALSE;
		}

	while(pQueryDir(hDir,pMemory,iLen,FALSE,TRUE,&dwContext,NULL)==0x105)
		{
		VirtualFree(pMemory,iLen,MEM_DECOMMIT|MEM_RELEASE);

		iLen *= 2;

			pMemory=(PBYTE)VirtualAlloc(NULL,iLen,MEM_COMMIT,PAGE_READWRITE);
		if(!pMemory)
			{
			CloseHandle(hDir);
			return FALSE;
			}
		}


	for(iPos=0;iPos<=iNum;iPos++)
		{
		cCmpTxt[iPos] = pLinkName[iPos];
		}


	pString	= (NT_PUNICODE_STRING)pMemory;

	sName.Buffer		= cBuffer;
	sName.MaximumLength	= 2*255;
	sName.Length		= 0;
	sTemp.Buffer		= cString;
	sTemp.MaximumLength	= 2*255;
	sTemp.Length		= 0;


	for(iRet=0;pString->Buffer;pString++,pString++)
		{
		if(pString[1].Buffer[0]!='S' || memcmp(pString[1].Buffer,L"SymbolicLink",24))
			{
			continue;
			}

		if(pString->Buffer[0]=='P' && pString->Buffer[1]=='h' && pString->Buffer[2]=='y' && pString->Buffer[3]=='s' && pString->Buffer[4]=='i')
			{
			sTemp.Buffer		= cString;
			sTemp.MaximumLength	= 2*255;
			sTemp.Length		= 2*swprintf(cString,L"\\GLOBAL??\\%s",pString->Buffer);

			sObject.Length					 = sizeof(sObject);
			sObject.ObjectName				 = &sTemp;
			sObject.Attributes				 = 0x40;
			sObject.RootDirectory			 = NULL;
			sObject.SecurityDescriptor		 = NULL;
			sObject.SecurityQualityOfService = NULL;
			}
		else{
			sObject.Length					 = sizeof(sObject);
			sObject.ObjectName				 = pString;
			sObject.Attributes				 = 0x40;
			sObject.RootDirectory			 = hDir;
			sObject.SecurityDescriptor		 = NULL;
			sObject.SecurityQualityOfService = NULL;
			}


		pOpenLink(&hLink,1,&sObject);

		if(!hLink)continue;

		pQueryLink(hLink,&sTemp,NULL);
		CloseHandle(hLink);


		if(sTemp.Length!=iNum*2 || memcmp(sTemp.Buffer,cCmpTxt,iNum*2))
			{
			continue;
			}

		if(pString->Length>=iSize*2)break;

		for(iPos=0;iPos<pString->Length/2;iPos++)
			{
			pTarget[iPos] = (char)pString->Buffer[iPos];
			}

		pTarget[iPos] = 0;
		pTarget += iPos+1;
		iSize   -= iPos+1;
		iRet    += 1;
		}

	VirtualFree(pMemory,iLen,MEM_DECOMMIT|MEM_RELEASE);
	CloseHandle(hDir);
	pTarget[0]=0;


return iRet;
}

//*****************************************************************************
//*
//*		FindSymbolLinks
//*
//*****************************************************************************
//	Sucht den Namen der Quelle auf den ein symolischer Link zeigt
//	pSourceDir		: Ist das Verzeichniss in dem gesucht werden soll Quelle
//					  z.B. "\\GLOBAL??"
//	pLinkName		: Ist der Name des symolischen Links mit Wildchards (* ?)
//	pTarget			: Hier werden die Texte gespeichert. ("<Name>0<Link>0...")
//	iSize			: Gre des Puffers fr die Texte
//	Ergibt die Anzahl der gefundenen Texte (Anzahl der Links)
static int FindSymbolLinks(const char *pSourceDir,const char *pLinkName,char *pTarget,int iSize)
{
static	NtQuerySymbolicLinkObjectType	pQueryLink=0;
static	NtOpenSymbolicLinkObjectType	pOpenLink=0;
static	NtQueryDirectoryObjectType		pQueryDir=0;
static	NtOpenDirectoryObjectType		pOpenDir=0;
static	HMODULE							hNtDll=0;
WCHAR									cCmpTxt[256];
WCHAR									cString[256];
WCHAR									cBuffer[256];
DWORD									dwContext;
LPBYTE									pMemory;
NT_UNICODE_STRING					   *pString;
NT_OBJECT_ATTRIBUTES					sObject;
NT_UNICODE_STRING						sName;
NT_UNICODE_STRING						sTemp;
HANDLE									hLink;
HANDLE									hDir;
int										iLen;
int										iPos;
int										iNum;
int 									iRet;


	if(iSize<=1)
		{
		return FALSE;
		}

		iNum = strlen(pLinkName);
	if(!iNum || iNum>=256)
		{
		return FALSE;
		}

	iSize     -= 2;
	pTarget[0] = 0;
	pTarget[1] = 0;

	if(!hNtDll)
		{
		hNtDll     = GetModuleHandle("ntdll");
		pOpenDir   = (NtOpenDirectoryObjectType    )GetProcAddress(hNtDll,"NtOpenDirectoryObject"    );
		pQueryDir  = (NtQueryDirectoryObjectType   )GetProcAddress(hNtDll,"NtQueryDirectoryObject"   );
		pOpenLink  = (NtOpenSymbolicLinkObjectType )GetProcAddress(hNtDll,"NtOpenSymbolicLinkObject" );
		pQueryLink = (NtQuerySymbolicLinkObjectType)GetProcAddress(hNtDll,"NtQuerySymbolicLinkObject");
		}

	if(!pOpenLink || !pOpenLink || !pOpenLink || !pQueryLink)
		{
		return FALSE;
		}


	sName.Buffer		= cBuffer;
	sName.MaximumLength	= 2*255;
	sName.Length		= 0;

	for(;sName.Length<sName.MaximumLength;sName.Length+=2)
		{
		cBuffer[sName.Length/2]	= pSourceDir[sName.Length/2];
		if(!pSourceDir[sName.Length/2])break;
		}

	sObject.Length					 = sizeof(sObject);
	sObject.ObjectName				 = &sName;
	sObject.Attributes				 = 0x00;
	sObject.RootDirectory			 = NULL;
	sObject.SecurityDescriptor		 = NULL;
	sObject.SecurityQualityOfService = NULL;

	pOpenDir(&hDir,1,&sObject);
	if(!hDir)return FALSE;

	iLen	= 0x10000;
	pMemory	= (LPBYTE)VirtualAlloc(NULL,iLen,MEM_COMMIT,PAGE_READWRITE);
	
	if(!pMemory)
		{
		CloseHandle(hDir);
		return FALSE;
		}

	while(pQueryDir(hDir,pMemory,iLen,FALSE,TRUE,&dwContext,NULL)==0x105)
		{
		VirtualFree(pMemory,iLen,MEM_DECOMMIT|MEM_RELEASE);
		
		iLen *= 2;

			pMemory=(PBYTE)VirtualAlloc(NULL,iLen,MEM_COMMIT,PAGE_READWRITE);
		if(!pMemory)
			{
			CloseHandle(hDir);
			return FALSE;
			}
		}


	for(iPos=0;iPos<=iNum;iPos++)
		{
		cCmpTxt[iPos] = pLinkName[iPos];
		}

	
	pString	= (NT_PUNICODE_STRING)pMemory;
	
	sName.Buffer		= cBuffer;
	sName.MaximumLength	= 2*255;
	sName.Length		= 0;
	sTemp.Buffer		= cString;
	sTemp.MaximumLength	= 2*255;
	sTemp.Length		= 0;


	for(iRet=0;pString->Buffer;pString++,pString++)
		{
		if(pString[1].Buffer[0]!='S' || memcmp(pString[1].Buffer,L"SymbolicLink",24))
			{
			continue;
			}	

		if(StrFilter(pString->Buffer,cCmpTxt))
			{
			continue;
			}

		if(pString->Buffer[0]=='P' && pString->Buffer[1]=='h' && pString->Buffer[2]=='y' && pString->Buffer[3]=='s' && pString->Buffer[4]=='i')
			{
			sTemp.Buffer		= cString;
			sTemp.MaximumLength	= 2*255;
			sTemp.Length		= 2*swprintf(cString,L"\\GLOBAL??\\%s",pString->Buffer);

			sObject.Length					 = sizeof(sObject);
			sObject.ObjectName				 = &sTemp;
			sObject.Attributes				 = 0x40;
			sObject.RootDirectory			 = NULL;
			sObject.SecurityDescriptor		 = NULL;
			sObject.SecurityQualityOfService = NULL;
			}
		else{
			sObject.Length					 = sizeof(sObject);
			sObject.ObjectName				 = pString;
			sObject.Attributes				 = 0x40;
			sObject.RootDirectory			 = hDir;
			sObject.SecurityDescriptor		 = NULL;
			sObject.SecurityQualityOfService = NULL;
			}


		pOpenLink(&hLink,1,&sObject);

		if(!hLink)
			{
			sTemp.Length = 0;
			}
		else{
			if(pQueryLink(hLink,&sTemp,NULL))sTemp.Length=0;
			CloseHandle(hLink);
			}

		sTemp.Buffer[sTemp.Length/2] = 0;

		if(sTemp.Length+pString->Length+6>iSize*2)
			{
			break;
			}

		if(pString->Length>=iSize*2)
			{
			break;
			}

		for(iPos=0;iPos<pString->Length/2;iPos++)
			{
			pTarget[iPos] = (char)pString->Buffer[iPos];
			}

		pTarget[iPos] = 0;				
		pTarget += iPos+1;
		iSize   -= iPos+1;

		for(iPos=0;iPos<sTemp.Length/2;iPos++)
			{
			pTarget[iPos] = (char)sTemp.Buffer[iPos];
			}

		pTarget[iPos] = 0;				
		pTarget += iPos+1;
		iSize   -= iPos+1;
		iRet    += 1;
		}
	
	VirtualFree(pMemory,iLen,MEM_DECOMMIT|MEM_RELEASE);
	CloseHandle(hDir);
	pTarget[0]=0;


return iRet;
}

//*****************************************************************************
//*
//*		CreateSymbolicLinkObject
//*
//*****************************************************************************
//	Erzeugt einen symolischer Link. 
//	pLinkName		: Ist das Verzeichniss in dem gesucht werden soll Quelle
//					  z.B. "\\GLOBAL??"
//	pTarget			: Ist der Name des Links
//	Ergibt TRUE wenn der Link erzeugt werden konnte.
/*
static BOOL CreateSymbolicLinkObject(const char *pLinkName,const char *pTarget)
{
static	NtCreateSymbolicLinkObjectType	pCreateLink=0;
static	HMODULE							hNtDll=0;
WCHAR									cName[256];
WCHAR									cText[256];
NT_OBJECT_ATTRIBUTES					sObject;
NT_UNICODE_STRING						sName;
NT_UNICODE_STRING						sText;
HANDLE									hLink;
LONG									lRet;



	if(!hNtDll)
		{
		hNtDll		= GetModuleHandle("ntdll");
		pCreateLink = (NtCreateSymbolicLinkObjectType )GetProcAddress(hNtDll,"NtCreateSymbolicLinkObject");
		}

	if(!pCreateLink)
		{
		return FALSE;
		}



	sName.Buffer		= cName;
	sName.MaximumLength	= 2*255;
	sName.Length		= 0;

	for(;sName.Length<sName.MaximumLength;sName.Length+=2)
		{
		cName[sName.Length/2]	= pLinkName[sName.Length/2];
		if(!pLinkName[sName.Length/2])break;
		}

	sText.Buffer		= cText;
	sText.MaximumLength	= 2*255;
	sText.Length		= 0;

	for(;sText.Length<sText.MaximumLength;sText.Length+=2)
		{
		cText[sText.Length/2]	= pTarget[sText.Length/2];
		if(!pTarget[sText.Length/2])break;
		}

	sObject.Length					 = sizeof(sObject);
	sObject.ObjectName				 = &sName;
	sObject.Attributes				 = 0x40;
	sObject.RootDirectory			 = NULL;
	sObject.SecurityDescriptor		 = NULL;
	sObject.SecurityQualityOfService = NULL;
						   
	lRet = pCreateLink(&hLink,0x000F0001,&sObject,&sText);

	if(!hLink)
		{
		return FALSE;
		}

	CloseHandle(hLink);


return TRUE;
}
*/

//*****************************************************************************
//*
//*		MountThread
//*
//*****************************************************************************
static DWORD WINAPI MountThread(LPVOID pParam)
{
char		cDeviceName[64];
DWORD		dwBytes;
HANDLE		hDevice;
unsigned	uNum;
unsigned	uDif;
unsigned	uCnt;



	while(hThread)
		{
		Sleep(2000);

		uCnt = 0;

		WaitForSingleObject(hIoSem,0xFFFFFFFF);

		for(uNum=0;uNum<64;uNum++)
			{
			if(!aDoMount[uNum])continue;

			uCnt++;
			uDif = GetTickCount()-aDoTick[uNum];

			if(uDif<10000)continue;						// Nach 10 Sekunden monten

			aDoMount[uNum] = 0;

			if(uNum>=32)
				{
    			strcpy(cDeviceName,"\\\\.\\PhysicalDrive0");
				itoa(uNum-32,cDeviceName+17,10);
				}
			else{
				strcpy(cDeviceName,"\\\\.\\A:");
				cDeviceName[4] += uNum;
				}


			   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice==INVALID_HANDLE_VALUE)
				{
				if(uNum<32)continue;

				sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",uNum-32);

				   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
				if(hDevice==INVALID_HANDLE_VALUE)
					{
					continue;
					}
				}

			DeviceIoControl(hDevice,IOCTL_STORAGE_LOAD_MEDIA,NULL,0,NULL,0,&dwBytes,0);
			CloseHandle(hDevice);
			}

		if(!uCnt)hThread=0;

		ReleaseSemaphore(hIoSem,1,0);
		}


return 0;
}

//*****************************************************************************
//*
//*		ReadSector
//*
//*****************************************************************************
//	Auslesen eines oder mehrere logischer Sektoren (immer 512 Byte gro !!!)
//	iDrive	: Laufwerksnummer  A=0 B=1 ... Z=25)
//							   0x8000=Pysical_0 0x8001=Pysical_1 ... 
//							   0x4001=Harddisk0\Partition1 	
//	qwStart	: Nummer des ersten Sektors
//	dwCount : Ist die Anzahl der Sektoren (1 bis 0xFFFF)
//	pBuffer	: Zeiger auf den Puffer in dem die Sektoren geschrieben werden.
//	Ergibt TRUE wenn die Sektoren gelesen wurden
BOOL ReadSector(int iDrive,QWORD qwStart,DWORD dwCount,void *pBuffer)
{
CHAR		cDeviceName[64];
HANDLE		hDevice;
BOOL		bResult;
DWORD		dwStart;
DWORD		dwBytes;
DWORD		dwMode;
DWORD		dwPos;
LONG		lHigh;
LONG		lSave;




	if(iDrive&0x8000)										// Physikalisches Laufwerk
		{
		iDrive &=~0x8000;
		dwMode	= 1;
		if(iDrive<0 || iDrive>=32)
			{
			SetLastError(ERROR_INVALID_DRIVE);
			return 0;
			}
		}
	else if(iDrive&0x4000)									// Partition/Harddisk
		{
		iDrive &=~0x4000;
		dwMode	= 2;
		if(iDrive<0 || iDrive>0x3FF)
			{
			SetLastError(ERROR_INVALID_DRIVE);
			return 0;
			}
		
		if((iDrive&0x0F)==0)								// Partition 0 ist physikalisches Laufwerk
			{
			dwMode=1;
			iDrive>>=4;
			}
		}
	else{													// Logisches Laufwerk
		if(iDrive<0 || iDrive>=26)
			{
			SetLastError(ERROR_INVALID_DRIVE);
			return 0;
			}

		dwMode	= 0;
		}


	

	if(IsWinNt())											// WinNT Code
		{
		switch(dwMode)
			{
    	case 1:		sprintf(cDeviceName,"\\\\.\\PhysicalDrive%i",iDrive);
					break;
		
		case 2:		sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition%i",iDrive>>4,iDrive&0x0F);
					break;
			
		default:	sprintf(cDeviceName,"\\\\.\\%c:",iDrive+'A');
					break;
			}

		   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(hDevice==INVALID_HANDLE_VALUE)
			{
			if(dwMode!=1)return FALSE;

			sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",iDrive);

			   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice==INVALID_HANDLE_VALUE)
				{
				return FALSE;
				}
			}

		lHigh	 = (long)(qwStart>>(32-9));
		lSave	 = lHigh;
		qwStart *= 512;
		dwCount *= 512;
		dwStart  = (DWORD)qwStart;

		// Setting the pointer to point to the start of the sector we want to read ..
		   dwPos=SetFilePointer(hDevice,dwStart,&lHigh,FILE_BEGIN);
		if(dwPos!=dwStart || lSave!=lHigh)
			{
			CloseHandle(hDevice);
			return FALSE;
			}

			bResult=ReadFile(hDevice,pBuffer,dwCount,&dwBytes,NULL);
		if(!bResult || dwCount!=dwBytes)
			{
			CloseHandle(hDevice);
			return FALSE;
			}
		}
	else{													// Win95/98 Code
		#if WIN98_SUPPORT

		CONTROL_BLOCK	sBlock;
		DIOC_REGISTERS	sReg;

		if(dwMode)return FALSE;
		if(qwStart>(QWORD)0xFFFFFFFF)
			{
			return FALSE;
			}

		hDevice = CreateFile ( "\\\\.\\vwin32",0,0,NULL,0,FILE_FLAG_DELETE_ON_CLOSE,NULL);


		sBlock.dwStartingSector	= (DWORD)qwStart;
		sBlock.wNumberOfSectors	= (WORD)dwCount;
		sBlock.pBuffer			= pBuffer;

		//-----------------------------------------------------------
		// SI contains read/write mode flags
		// SI=0h for read and SI=1h for write
		// CX must be equal to ffffh for
		// int 21h's 7305h extention
		// DS:BX -> base addr of the
		// control block structure
		// DL must contain the iDrive number
		// (01h=A:, 02h=B: etc)
		//-----------------------------------------------------------

		sReg.reg_ESI = 0x00;
		sReg.reg_ECX = -1;
		sReg.reg_EBX = (DWORD)(&sBlock);
		sReg.reg_EDX = iDrive+1;
		sReg.reg_EAX = 0x7305;


			bResult = DeviceIoControl(hDevice,VWIN32_DIOC_DOS_DRIVEINFO,&(sReg),sizeof(sReg),&(sReg),sizeof(sReg),&dwBytes,0);
		if(!bResult || (sReg.reg_Flags & 0x0001))
			{
			CloseHandle(hDevice);
			return FALSE;
			}
		
		#else 
		
		SetLastError(ERROR_NOT_SUPPORTED);

		return FALSE;

		#endif 	
		}

	CloseHandle(hDevice);


return TRUE;
}

//*****************************************************************************
//*
//*		ReadSector
//*
//*****************************************************************************
//	Auslesen eines oder mehrere logischer Sektoren (immer 512 Byte gro !!!)
//	hDrive	: Ist das Handle des Laufwerkes
//	qwStart	: Nummer des ersten Sektors
//	dwCount : Ist die Anzahl der Sektoren (1 bis 0xFFFF)
//	pBuffer	: Zeiger auf den Puffer in dem die Sektoren geschrieben werden.
//	Ergibt TRUE wenn die Sektoren gelesen wurden
BOOL ReadSector(void *hDrive,QWORD qwStart,DWORD dwCount,void *pBuffer)
{
BOOL			bResult;
DWORD			dwStart;
DWORD			dwBytes;
HANDLE			hDevice;
UINT			uCount;
LONG			lHigh;
UINT			uNr;


	   uNr=(int)hDrive-1;
	if(uNr>=64)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	hDevice=aHandles[(int)hDrive-1];

	if(IsWinNt())											// WinNT Code
		{
		lHigh    = (LONG)(qwStart>>(32-9));
		qwStart *= 512;
		dwCount *= 512;
		dwStart =  (DWORD)qwStart;


		for(uCount=0;uCount<5;uCount++)
			{
			// Setting the pointer to point to the start of the sector we want to read ..
			if(SetFilePointer(hDevice,dwStart,&lHigh,FILE_BEGIN)==0xFFFFFFFF)
			if(GetLastError())
				{
				return FALSE;
				}

				bResult=ReadFile(hDevice,pBuffer,dwCount,&dwBytes,NULL);
			if( bResult && dwCount==dwBytes)break;
			}

		if(uCount>=5)return FALSE;
		}
	else{													// Win95/98 Code
		#if WIN98_SUPPORT

		DIOC_REGISTERS	sReg;
		CONTROL_BLOCK	sBlock;

		if(qwStart>(QWORD)0xFFFFFFFF)return FALSE;

		sBlock.dwStartingSector	= (DWORD)qwStart;
		sBlock.wNumberOfSectors	= (WORD)dwCount;
		sBlock.pBuffer			= pBuffer;

		//-----------------------------------------------------------
		// SI contains read/write mode flags
		// SI=0h for read and SI=1h for write
		// CX must be equal to ffffh for
		// int 21h's 7305h extention
		// DS:BX -> base addr of the
		// control block structure
		// DL must contain the iDrive number
		// (01h=A:, 02h=B: etc)
		//-----------------------------------------------------------

		sReg.reg_ESI = 0x00;
		sReg.reg_ECX = -1;
		sReg.reg_EBX = (DWORD)(&sBlock);
		sReg.reg_EDX = (DWORD)hDrive;
		sReg.reg_EAX = 0x7305;

		for(uCount=0;uCount<5;uCount++)
			{
				bResult = DeviceIoControl(hDevice,VWIN32_DIOC_DOS_DRIVEINFO,&(sReg),sizeof(sReg),&(sReg),sizeof(sReg),&dwBytes,0);
			if( bResult && !(sReg.reg_Flags & 0x0001))break;

			}

		if(uCount>=5)return FALSE;

		#else 

		SetLastError(ERROR_NOT_SUPPORTED);
		
		return FALSE;

		#endif 	
		}



return TRUE;
}

//*****************************************************************************
//*
//*		FlushSector
//*
//*****************************************************************************
//	Schreibt alle Buffer auf die Disk zurck
//	hDrive	: Ist das Handle des Laufwerkes
//	Ergibt TRUE wenn die Sektoren gespeichert wurden
BOOL FlushSector(void *hDrive)
{
HANDLE			hDevice;
unsigned		uNr;


	   uNr=(int)hDrive-1;
	if(uNr>=64)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	hDevice=aHandles[(int)hDrive-1];

	if(IsWinNt())
		{
		return FlushFileBuffers(hDevice);
		}

	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}

//*****************************************************************************
//*
//*		NotifyNewDrives
//*
//*****************************************************************************
//	Explorer bekannt geben das es neue Laufwerke gibt
//	dwBits		: Bits mit den Laufwerken ("A:" = Bit 0)
static void NotifyNewDrives(DWORD dwBits)
{
DEV_BROADCAST_VOLUME	sVolume;
CHAR					cName[4];
DWORD					dwLetter;
DWORD					dwValue;


														
	for(dwLetter=0;dwLetter<26 && dwBits;dwLetter++,dwBits>>=1)
		{
		if(!(dwBits&1))continue;

		sVolume.dbcv_size		= sizeof(DEV_BROADCAST_VOLUME);
		sVolume.dbcv_devicetype	= DBT_DEVTYP_VOLUME;
		sVolume.dbcv_unitmask	= 1<<dwLetter;
		sVolume.dbcv_reserved	= 0;
		sVolume.dbcv_flags		= 0;

		SendMessageTimeout(HWND_BROADCAST,WM_DEVICECHANGE,DBT_DEVICEARRIVAL   ,(LPARAM)&sVolume,SMTO_BLOCK|SMTO_ABORTIFHUNG,4000,&dwValue);

		sVolume.dbcv_flags		= DBTF_MEDIA;

		SendMessageTimeout(HWND_BROADCAST,WM_DEVICECHANGE,DBT_DEVICEARRIVAL   ,(LPARAM)&sVolume,SMTO_BLOCK|SMTO_ABORTIFHUNG,4000,&dwValue);
		SendMessageTimeout(HWND_BROADCAST,WM_DEVICECHANGE,DBT_DEVNODES_CHANGED,(LPARAM)0       ,SMTO_BLOCK|SMTO_ABORTIFHUNG,4000,&dwValue);

		cName[0]	= (char)(dwLetter+'A');
		cName[1]	= ':';
		cName[2]	= '\\';
		cName[3]	= 0;

		SHChangeNotify(SHCNE_DRIVEADD,SHCNF_PATH,cName,NULL);
		}

}

//*****************************************************************************
//*
//*		UnmountDrive
//*
//*****************************************************************************
static void UnmountDrive(HANDLE &hDevice,unsigned uNr,BOOL bPysical)
{
char		cDeviceName[64];
char		cLnkName[256];
char		cDosName[256];
char		cDrvName[256];
char		cPrtName[256];
HANDLE		hLogical;
DWORD		dwBytes;
unsigned	uDrive;
unsigned	uTurn;
unsigned	uPos;
unsigned	uLen;
unsigned	uErr;
unsigned	uSet;
BOOL		bOk;



	if(bPysical)
		{
    	strcpy(cDeviceName,"\\\\.\\PhysicalDrive0");
		itoa(uNr,cDeviceName+17,10);
		uTurn = 0;
		uErr  = 0;
		}
	else{
		strcpy(cDeviceName,"\\\\.\\A:");
		cDeviceName[4] += uNr;
		uTurn = 0;
		uErr  = 0;
		}

Retry:

	if(bPysical)											// Physikalisches Laufwerk
		{
		for(uPos=0;uPos<26;uPos++)
			{
			uDrive = GetDriveOfVolume(uPos)&0x7FFF;

			if(uDrive!=uNr)continue;						// Ist logisches Laufwerk am pysikalischen

			strcpy(cDrvName,"\\\\.\\A:");
			cDrvName[4] += uPos;

			   hLogical =CreateFile(cDrvName,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
			if(hLogical==INVALID_HANDLE_VALUE)continue;

			dwBytes=0;
			DeviceIoControl(hLogical,FSCTL_ALLOW_EXTENDED_DASD_IO,NULL,0,NULL,0,&dwBytes,NULL);
			DeviceIoControl(hLogical,FSCTL_LOCK_VOLUME           ,NULL,0,NULL,0,&dwBytes,NULL);
			DeviceIoControl(hLogical,FSCTL_DISMOUNT_VOLUME       ,NULL,0,NULL,0,&dwBytes,NULL);

			cDosName[0] = 0;
			QueryDosDevice(cDrvName+4,cDosName,sizeof(cDosName));
			DefineDosDevice(DDD_RAW_TARGET_PATH|DDD_REMOVE_DEFINITION|DDD_EXACT_MATCH_ON_REMOVE,cDrvName+4,cDosName);

			CloseHandle(hLogical);
			}


		sprintf(cPrtName,"\\GLOBAL??\\PhysicalDrive%u",uNr);

		if(QuerySymbolLink(cPrtName,cLnkName,sizeof(cLnkName)))
		if(memcmp(cLnkName,"\\Device\\Harddisk",16)==0)
		if(cLnkName[16]>='0' && cLnkName[16]<='9')
			{
			for(uLen=17;;uLen++)							// Suche Ende von
				{
				if(cLnkName[uLen]<'0')break;
				if(cLnkName[uLen]>'9')break;
				}

			if(cLnkName[uLen]=='\\')
				{
				cLnkName[uLen] = 0;


				for(uPos=16;uPos>0;)						// Partitionen aushngen
					{
					uPos--;
					sprintf(cPrtName,"%s\\Partition%u",cLnkName,uPos);

					if(!FakeDosNameForDevice(cPrtName,cDosName,cDrvName,FALSE))
						{
						continue;
						}

					   hLogical =CreateFile(cDrvName,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
					if(hLogical!=INVALID_HANDLE_VALUE)
						{
						dwBytes=0;
						DeviceIoControl(hLogical,FSCTL_ALLOW_EXTENDED_DASD_IO,NULL,0,NULL,0,&dwBytes,NULL);
						DeviceIoControl(hLogical,FSCTL_LOCK_VOLUME           ,NULL,0,NULL,0,&dwBytes,NULL);
						DeviceIoControl(hLogical,FSCTL_DISMOUNT_VOLUME       ,NULL,0,NULL,0,&dwBytes,NULL);
						CloseHandle(hLogical);
						}

					if(QuerySymbolLink(cPrtName,cDrvName,sizeof(cDrvName)))
						{
						   hLogical =CreateFile(cDrvName,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
						if(hLogical!=INVALID_HANDLE_VALUE)
							{
							dwBytes=0;
							DeviceIoControl(hLogical,FSCTL_ALLOW_EXTENDED_DASD_IO,NULL,0,NULL,0,&dwBytes,NULL);
							DeviceIoControl(hLogical,FSCTL_LOCK_VOLUME           ,NULL,0,NULL,0,&dwBytes,NULL);
							DeviceIoControl(hLogical,FSCTL_DISMOUNT_VOLUME       ,NULL,0,NULL,0,&dwBytes,NULL);
							CloseHandle(hLogical);
							}
						}

					RemoveFakeDosName(cPrtName,cDosName);
					}
				}
			}
		}


	if(!DeviceIoControl(hDevice,FSCTL_LOCK_VOLUME,NULL    ,0,NULL,0,&dwBytes,0))uErr=1;
	if(!DeviceIoControl(hDevice,FSCTL_DISMOUNT_VOLUME,NULL,0,NULL,0,&dwBytes,0))uErr=1;

	uTurn++;
	cDosName[0] = 0;
	QueryDosDevice(cDeviceName+4,cDosName,sizeof(cDosName));
	bOk=DefineDosDevice(DDD_RAW_TARGET_PATH|DDD_REMOVE_DEFINITION|DDD_EXACT_MATCH_ON_REMOVE,cDeviceName+4,cDosName);

	if(bOk && bPysical && cDosName[0])
		{
		if(!aDrvName[(uNr+32)*64])Sleep(200);
		strncpy(aDrvName+(uNr+32)*64,cDosName,63);
		}


	if(uTurn<=1 && bPysical && uErr)						// Nochmal versuchen
		{
		CloseHandle(hDevice);
		Sleep(50);

		uSet = (aHandles[uNr+32]==hDevice);
		sprintf(cPrtName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",uNr);

		hDevice =CreateFile(cPrtName,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(uSet)aHandles[uNr+32]=hDevice;

		if(hDevice!=INVALID_HANDLE_VALUE)
			{
			goto Retry;
			}
		}

}

//*****************************************************************************
//*
//*		MountPysicalDisk
//*
//*****************************************************************************
BOOL MountPysicalDisk(int iDrive)
{
char		cLinkBuffer[4096];
char		cDeviceName[128];
char		cLinkName  [128];
LPCTSTR		pMountPoint;
LPCTSTR		pVolumeName;
LPCTSTR		pString;
int			iCount;
int			iBits;
int			iPos;


	iDrive &= ~0x8000;
	
	if(iDrive<0 || iDrive>=32)
		{
		return FALSE;
		}
	
	if(GetVolumeOfDrive(iDrive))					// Gibt es schon Laufwerke am Device
		{
		return TRUE;
		}


	sprintf(cDeviceName,"\\Device\\Harddisk%i\\Partition1",iDrive);
	
	if(!QuerySymbolLink(cDeviceName,cLinkName,sizeof(cLinkName)))
		{
		return FALSE;
		}

	   iCount = QuerySourceFromLink("\\GLOBAL??",cLinkName,cLinkBuffer,sizeof(cLinkBuffer));
	if(iCount<=0)
		{
		return FALSE;
		}

	pMountPoint	= 0;
	pVolumeName	= 0;
	pString		= cLinkBuffer;
													
	for(iPos=0;iPos<iCount;iPos++,pString+=strlen(pString)+1)
		{
		if(pString[1]==':' && pString[0]>='A' && pString[0]<='Z' && pString[2]==0)
			{
			pMountPoint = pString;
			continue;
			}
		
		if(!memicmp(pString,"Volume{",7))
			{
			pVolumeName = pString;
			continue;
			}
		}

	if(pMountPoint==0)								// Kein Laufwerk gefunden
		{
		iBits	= GetLogicalDrives()>>3;
		
		for(iPos=3;iPos<26;iPos++,iBits>>=1)
			{
			if(iBits&1)continue;
			sprintf(cDeviceName,"%c:",'A'+iPos);
			pMountPoint = cDeviceName;
			break;
			}

		}

	if(pMountPoint && pVolumeName)
		{
		sprintf(cDeviceName,"%s\\"        ,pMountPoint);
		sprintf(cLinkName   ,"\\\\?\\%s\\",pVolumeName);

		return SetVolumeMountPoint_(cDeviceName,cLinkName);
		}
	


return FALSE;
}


//*****************************************************************************
//*
//*		WriteSector
//*
//*****************************************************************************
//	Schreiben eines oder mehrere logischer Sektoren (immer 512 Byte gro !!!)
//	iDrive	: Laufwerksnummer (A=0 B=1 ...  0x8000=Pysical_0 0x8001=Pysical_1 ... )
//	qwStart	: Nummer des ersten Sektors
//	dwCount : Ist die Anzahl der Sektoren (1 bis 0xFFFF)
//	pBuffer	: Zeiger auf den Puffer in dem die Sektoren Daten stehen werden.
//	Ergibt TRUE wenn die Sektoren geschrieben wurden
BOOL WriteSector(int iDrive,QWORD qwStart,DWORD dwCount,const void *pBuffer)
{
char		cDeviceName[64];
HANDLE		hDevice;
BOOL		bResult;
DWORD		dwIdNum;
DWORD		dwStart;
DWORD		dwBytes;
DWORD		dwMode;
LONG		lHigh;
unsigned	uNum;



	if(iDrive&0x8000)									// Physikalisches Laufwerk
		{
		iDrive &=~0x8000;
		dwMode	= 1;
		if(iDrive<0 || iDrive>=32)
			{
			SetLastError(ERROR_INVALID_HANDLE);
			return 0;
			}
		}
	else{												// Logisches Laufwerk
		if(iDrive<0 || iDrive>=26)
			{
			SetLastError(ERROR_INVALID_HANDLE);
			return 0;
			}

		dwMode	= 0;
		}


	if(IsWinNt())										// WinNT Code
		{
		switch(dwMode)  
			{
    	case 1:		sprintf(cDeviceName,"\\\\.\\PhysicalDrive%i",iDrive);
					uNum = iDrive+32;
					break;
		
		default:	sprintf(cDeviceName,"\\\\.\\%c:",iDrive+'A');
					uNum = iDrive;
					break;
			}

		   hDevice= CreateFile(cDeviceName,GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(hDevice==INVALID_HANDLE_VALUE)
			{
			if(dwMode!=1)return FALSE;

			sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",iDrive-32);

			   hDevice= CreateFile(cDeviceName,GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice==INVALID_HANDLE_VALUE)
				{
				return FALSE;
				}
			}

		if(IsWin7() && !hIoSem)							// Bei Win7 Laufwerk unmounten
			{
			hIoSem = CreateSemaphore(0,1,0x70000000,0);
			}

		lHigh	 = (LONG)(qwStart>>(32-9));
		qwStart *= 512;
		dwCount *= 512;
		dwStart  = (DWORD)qwStart;

														// Setting the pointer to point to the start of the sector we want to write
		if(SetFilePointer(hDevice,dwStart,&lHigh,FILE_BEGIN)==0xFFFFFFFF)
		if(GetLastError())
			{
			CloseHandle(hDevice);
			return FALSE;
			}

			bResult=WriteFile(hDevice,pBuffer,dwCount,&dwBytes,NULL);
		if(!bResult || dwCount!=dwBytes)
			{
			if(!IsWin7())								// Bei Win7 Laufwerk unmounten
				{
				CloseHandle(hDevice);
				return FALSE;
				}

			WaitForSingleObject(hIoSem,0xFFFFFFFF);
			aDoTick[uNum] = GetTickCount();

			if(!aDoMount[uNum])							// Laufwerk unmonten ?
				{
				aDoMount[uNum] = TRUE;
				UnmountDrive(hDevice,iDrive,dwMode==1);

				if(!hThread)							// Thread zum Mounten starten
					{
					hThread = CreateThread(0,0x1000,MountThread,0,0,&dwIdNum);
					CloseHandle(hThread);
					}
				}

			ReleaseSemaphore(hIoSem,1,0);

				bResult=WriteFile(hDevice,pBuffer,dwCount,&dwBytes,NULL);
			if(!bResult || dwCount!=dwBytes)
				{
				CloseHandle(hDevice);
				return FALSE;
				}
			}
		}
	else{												// Win95/98 Code
		#if WIN98_SUPPORT

		DIOC_REGISTERS	sReg;
		CONTROL_BLOCK	sBlock;

		if(dwMode)return FALSE;
		if(qwStart>(QWORD)0xFFFFFFFF)return FALSE;

		hDevice = CreateFile ( "\\\\.\\vwin32",0,0,NULL,0,FILE_FLAG_DELETE_ON_CLOSE,NULL);

		sBlock.dwStartingSector	= (DWORD )qwStart;
		sBlock.wNumberOfSectors	= (WORD  )dwCount;
		sBlock.pBuffer			= (LPVOID)pBuffer;

		//-----------------------------------------------------------
		// SI contains read/write mode flags
		// SI=0h for read and SI=1h for write
		// CX must be equal to ffffh for
		// int 21h's 7305h extention
		// DS:BX -> base addr of the
		// control block structure
		// DL must contain the iDrive number
		// (01h=A:, 02h=B: etc)
		//-----------------------------------------------------------

		memset(&sReg,0,sizeof(sReg));

		sReg.reg_ESI = 0x01;
		sReg.reg_ECX = -1;
		sReg.reg_EBX = (DWORD)(&sBlock);
		sReg.reg_EDX = iDrive+1;
		sReg.reg_EAX = 0x7305;


			bResult = DeviceIoControl(hDevice,VWIN32_DIOC_DOS_DRIVEINFO,&(sReg),sizeof(sReg),&(sReg),sizeof(sReg),&dwBytes,0);
		if(!bResult || (sReg.reg_Flags & 0x0001))
			{
			CloseHandle(hDevice);
			return FALSE;
			}
		
		#else 
		
		SetLastError(ERROR_NOT_SUPPORTED);
		
		return FALSE;

		#endif 	
		}

	CloseHandle(hDevice);


return TRUE;
}

//*****************************************************************************
//*
//*		WriteSector
//*
//*****************************************************************************
//	Schreiben eines oder mehrere logischer Sektoren (immer 512 Byte gro !!!)
//	hDrive	: Ist das Handle des Laufwerkes
//	qwStart	: Nummer des ersten Sektors
//	dwCount : Ist die Anzahl der Sektoren (1 bis 0xFFFF)
//	pBuffer	: Zeiger auf den Puffer in dem die Sektoren Daten stehen werden.
//	Ergibt TRUE wenn die Sektoren geschrieben wurden
BOOL WriteSector(void *hDrive,QWORD qwStart,DWORD dwCount,const void *pBuffer)
{
HANDLE		hDevice;
BOOL		bResult;
DWORD		dwStart;
DWORD		dwBytes;
int			iCount;
LONG		lHigh;
unsigned	uNr;



	   uNr=(int)hDrive-1;
	if(uNr>=64)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	hDevice = aHandles[uNr];

	if(IsWinNt())											// WinNT Code
		{
		lHigh	 = (LONG)(qwStart>>(32-9));
		qwStart *= 512;
		dwCount *= 512;
		dwStart	 = (DWORD)qwStart;


		for(iCount=0;iCount<5;iCount++)
			{												// Setting the pointer to point to the start of the sector we want to write
			if(SetFilePointer(hDevice,dwStart,&lHigh,FILE_BEGIN)==0xFFFFFFFF)
			if(GetLastError())
				{
				return FALSE;
				}

				bResult=WriteFile(hDevice,pBuffer,dwCount,&dwBytes,NULL);
			if( bResult && dwCount==dwBytes)
				{
				if(qwStart<=3)aReparse[uNr] = TRUE;
				break;
				}

															// Laufwerk unmounten
			if(iCount==1 && aAccess[uNr]>=2 && aUnmount[uNr]==0)
				{
				UnmountDrive(hDevice,uNr&0x1F,uNr>>5);
				aUnmount[uNr] = TRUE;
				}

			if(iCount==33)
				{
				char	cDeviceName[64]="\\\\.\\PhysicalDrive0";
				CloseHandle(hDevice);
				itoa(uNr&0x1F,cDeviceName+17,10);
				hDevice = CreateFile(cDeviceName,GENERIC_WRITE|GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
				aHandles[uNr] = hDevice;
				}
			}


		if(iCount>=5)
			{
			return FALSE;
			}
		}
	else{													// Win95/98 Code
		#if WIN98_SUPPORT

		DIOC_REGISTERS	sReg;
		CONTROL_BLOCK	sBlock;

		if(qwStart>(QWORD)0xFFFFFFFF)return FALSE;

		sBlock.dwStartingSector	= (DWORD )qwStart;
		sBlock.wNumberOfSectors	= (WORD  )dwCount;
		sBlock.pBuffer			= (LPVOID)pBuffer;

		//-----------------------------------------------------------
		// SI contains read/write mode flags
		// SI=0h for read and SI=1h for write
		// CX must be equal to ffffh for
		// int 21h's 7305h extention
		// DS:BX -> base addr of the
		// control block structure
		// DL must contain the iDrive number
		// (01h=A:, 02h=B: etc)
		//-----------------------------------------------------------

		memset(&sReg,0,sizeof(sReg));

		sReg.reg_ESI = 0x01;
		sReg.reg_ECX = -1;
		sReg.reg_EBX = (DWORD)(&sBlock);
		sReg.reg_EDX = (DWORD)hDrive;
		sReg.reg_EAX = 0x7305;

		for(iCount=0;iCount<5;iCount++)
			{
				bResult = DeviceIoControl(hDevice,VWIN32_DIOC_DOS_DRIVEINFO,&(sReg),sizeof(sReg),&(sReg),sizeof(sReg),&dwBytes,0);
			if( bResult && !(sReg.reg_Flags & 0x0001))break;
			}

		if(iCount>=5)
			{
			return FALSE;
			}

		#else 

		SetLastError(ERROR_NOT_SUPPORTED);
				
		return FALSE;

		#endif 	
		}



return TRUE;
}

//*****************************************************************************
//*
//*		GetPartitionInfo
//*
//*****************************************************************************
//	Auslesen der Partitionsinformationen eines Laufwerks
//	iDrive	: Laufwerksnummer  A=0 B=1 ... Z=25)
//							   0x8000=Pysical_0 0x8001=Pysical_1 ... 
//							   0x4001=Harddisk0\Partition1 	
//	qwStart	: Nummer des ersten Sektors
//	dwCount : Ist die Anzahl der Sektoren (1 bis 0xFFFF)
//	pBuffer	: Zeiger auf den Puffer in dem die Sektoren geschrieben werden.
//	Ergibt die Art der Partition oder 0 bei einem Fehler
//				0 = Keine Partition gefunden
//				1 = MBR Partition
//				2 = GPT Partition
//				3 = RAW	Partition
INT	GetPartitionInfo(int iDrive,DWORD *pNumber,QWORD *pStart,QWORD *pSize,GUID *pGuid)
{
CHAR		cDeviceName[64];
HANDLE		hDevice;
DWORD		dwBytes;
DWORD		dwMode;
DWORD		dwRet;



	if(pNumber)*pNumber=0;
	if(pStart )*pStart =0;
	if(pSize  )*pSize  =0;
	if(pGuid  )memset(pGuid,0,sizeof(GUID));


	if(iDrive&0x8000)										// Physikalisches Laufwerk
		{
		iDrive &=~0x8000;
		dwMode	= 1;
		if(iDrive<0 || iDrive>=32)
			{
			SetLastError(ERROR_INVALID_DRIVE);
			return 0;
			}
		}
	else if(iDrive&0x4000)									// Partition/Harddisk
		{
		iDrive &=~0x4000;
		dwMode	= 2;
		if(iDrive<0 || iDrive>0x3FF)
			{
			SetLastError(ERROR_INVALID_DRIVE);
			return 0;
			}
		
		if((iDrive&0x0F)==0)								// Partition 0 ist physikalisches Laufwerk
			{
			dwMode=1;
			iDrive>>=4;
			}
		}
	else{													// Logisches Laufwerk
		if(iDrive<0 || iDrive>=26)
			{
			SetLastError(ERROR_INVALID_DRIVE);
			return 0;
			}

		dwMode	= 0;
		}


	

	if(IsWinNt())											// WinNT Code
		{
		PARTITION_INFORMATION_EX_		sInfo;
	
		switch(dwMode)
			{
    	case 1:		sprintf(cDeviceName,"\\\\.\\PhysicalDrive%i",iDrive);
					break;
		
		case 2:		sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition%i",iDrive>>4,iDrive&0x0F);
					break;
			
		default:	sprintf(cDeviceName,"\\\\.\\%c:",iDrive+'A');
					break;
			}

		   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(hDevice==INVALID_HANDLE_VALUE)
			{
			if(dwMode!=1)return FALSE;

			sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",iDrive);

			   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice==INVALID_HANDLE_VALUE)
				{
				return FALSE;
				}
			}

		
		if(DeviceIoControl(hDevice,IOCTL_DISK_GET_PARTITION_INFO_EX,NULL,0,&sInfo,sizeof(sInfo),&dwBytes,NULL))
			{
			if(pNumber)*pNumber=sInfo.PartitionNumber;
			if(pStart )*pStart =sInfo.StartingOffset .QuadPart>>9;
			if(pSize  )*pSize  =sInfo.PartitionLength.QuadPart>>9;
			if(pGuid)
				{
				if(sInfo.PartitionStyle==1)
					{
					memcpy(pGuid,&sInfo.Gpt.PartitionType,sizeof(GUID));
					}
				else if(sInfo.PartitionStyle==2)
					{
					pGuid->Data1 = sInfo.Mbr.PartitionType;
					}
				}

			dwRet = sInfo.PartitionStyle+1;
			}
		else{
			dwRet = 0;
			}
		
		CloseHandle(hDevice);

		return dwRet;
		}

	
	SetLastError(ERROR_NOT_SUPPORTED);



return 0;
}


//*****************************************************************************
//*
//*		GetPartitionInfo
//*
//*****************************************************************************
//	Auslesen der Partitionsinformationen eines Laufwerks
//	hDrive	: Handle vom Laufwerk
//	qwStart	: Nummer des ersten Sektors
//	dwCount : Ist die Anzahl der Sektoren (1 bis 0xFFFF)
//	pBuffer	: Zeiger auf den Puffer in dem die Sektoren geschrieben werden.
//	Ergibt die Art der Partition oder 0 bei einem Fehler
//				0 = Keine Partition gefunden
//				1 = MBR Partition
//				2 = GPT Partition
//				3 = RAW	Partition
INT	GetPartitionInfo(void *hDrive,DWORD *pNumber,QWORD *pStart,QWORD *pSize,GUID *pGuid)
{
HANDLE		hDevice;
DWORD		dwBytes;
DWORD		dwRet;
UINT		uNr;


	if(pNumber)*pNumber=0;
	if(pStart )*pStart =0;
	if(pSize  )*pSize  =0;
	if(pGuid  )memset(pGuid,0,sizeof(GUID));

	   uNr=(int)hDrive-1;
	if(uNr>=64)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	hDevice = aHandles[(int)hDrive-1];


	

	if(IsWinNt())											// WinNT Code
		{
		PARTITION_INFORMATION_EX_		sInfo;
	
		if(DeviceIoControl(hDevice,IOCTL_DISK_GET_PARTITION_INFO_EX,NULL,0,&sInfo,sizeof(sInfo),&dwBytes,NULL))
			{
			if(pNumber)*pNumber=sInfo.PartitionNumber;
			if(pStart )*pStart =sInfo.StartingOffset .QuadPart>>9;
			if(pSize  )*pSize  =sInfo.PartitionLength.QuadPart>>9;
			if(pGuid)
				{
				if(sInfo.PartitionStyle==1)
					{
					memcpy(pGuid,&sInfo.Gpt.PartitionType,sizeof(GUID));
					}
				else if(sInfo.PartitionStyle==2)
					{
					pGuid->Data1 = sInfo.Mbr.PartitionType;
					}
				}

			dwRet = sInfo.PartitionStyle+1;
			}
		else{
			dwRet = 0;
			}

		return dwRet;
		}

	
	SetLastError(ERROR_NOT_SUPPORTED);



return 0;
}

//*****************************************************************************
//*
//*		FormatDisk
//*
//*****************************************************************************
//	Formatiert eine Diskette
//	iDrive	: Lafwerksnummer (0=A 1=B 2=C ...)
//	iType	: Art der Diskkete (z.B.  F3_1Pt44_512 F3_720_512)
//	Ergibt TRUE wenn die Diskette formatiert wurde.
BOOL FormatDisk(int iDrive,MEDIA_TYPE iType)
{
char			cDeviceName[] = "\\\\.\\A:";
HANDLE			hDevice;



	if(iType<0 || iType>=sizeof(iSectors)/sizeof(iSectors[0]) || !iSectors[iType])
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}
	

	cDeviceName[4] += iDrive;
	
	if(GetDriveType(cDeviceName+4)!=DRIVE_REMOVABLE)
		{
		SetLastError(ERROR_INVALID_CATEGORY);
		return FALSE;
		}


	if(IsWinNt())							// WinNT code
		{
		FORMAT_PARAMETERS	sFormat;
		DWORD				dwBytes;

		// Creating a handle to iDrive a: using CreateFile () function ..

			hDevice =CreateFile(cDeviceName,GENERIC_ALL,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if( hDevice==INVALID_HANDLE_VALUE)return FALSE;

		sFormat.StartCylinderNumber = 0;
		sFormat.StartHeadNumber		= 0;
		sFormat.EndCylinderNumber	= iTracks[iType]-1;
		sFormat.EndHeadNumber		= 1;
		sFormat.MediaType			= iType;

		if(!DeviceIoControl(hDevice,IOCTL_DISK_FORMAT_TRACKS,&sFormat,sizeof(FORMAT_PARAMETERS),NULL,0,&dwBytes,NULL))
			{
			CloseHandle(hDevice);
			return FALSE;
			}
		
		CloseHandle(hDevice);
		}
	else{
		#if WIN98_SUPPORT

		DIOC_REGISTERS	sReg;
		FVBLOCK			sBlock;
		DWORD			dwBytes;
		BOOL			bResult;
		int				iTrack;
		int				iHead;

						 
		hDevice = CreateFile ( "\\\\.\\vwin32",0,0,NULL,0,FILE_FLAG_DELETE_ON_CLOSE,NULL);

		memset(&sReg,0,sizeof(sReg));

		for(iHead=0;iHead<2;iHead++)
		for(iTrack=0;iTrack<iTracks[iType];iTrack++)
			{
			sBlock.fvSpecFunc	= 0;				// special functions = 0
			sBlock.fvHead		= iHead;			// head to format/verify
			sBlock.fvCylinder	= iTrack;			// cylinder to format/verify
			sBlock.fvTracks		= 0;				// tracks to format/verify

			sReg.reg_EAX		= 0x440D;       	// IOCTL for block devices
			sReg.reg_EBX 		= iDrive+1;       	// zero-based iDrive ID
			sReg.reg_ECX 		= 0x0842;			// Get Media ID command 0x??42
			sReg.reg_EDX 		= (DWORD)&sBlock;	// receives media ID info
			sReg.reg_Flags=0x0001;

			//  6 == VWIN32_DIOC_DOS_DRIVEINFO
			bResult = DeviceIoControl(hDevice,VWIN32_DIOC_DOS_IOCTL,&sReg,sizeof(sReg),&(sReg),sizeof(sReg),&dwBytes,0);

			if(!bResult || (sReg.reg_Flags & 0x0001))
				{
				int x=0;
				CloseHandle(hDevice);
				return FALSE;
				}
			}
		
		
		CloseHandle(hDevice);

		#else 
		
		SetLastError(ERROR_NOT_SUPPORTED);

		return FALSE;
 
		#endif 	
		}


return TRUE;
}

//*****************************************************************************
//*
//*		GetPhysicalCount
//*
//*****************************************************************************
//	Ergibt die Gre eines Laufwerks in 512 Byte Sektoren.
//	Ist	pSize ungleich Null so wird dort die wirkliche Sektorgre gespeichert.
//	Ergibt die Anzahl der Sektoren, bzw. 0 bei einem Fehler.
static QWORD GetPhysicalCount(int iDrive,DWORD *pSize)
{
HANDLE			hDevice;
QWORD			qwSize;
QWORD			qwLen;
QWORD			qwMax;


	if(pSize)*pSize=0;

	if(IsWinNt())									// WinNT Code
		{
    	char				cDeviceName[64] = "\\\\.\\PhysicalDrive0";
		DWORD				dwBytes;
		DISK_GEOMETRY_EX_	sGeoEx;
		DISK_GEOMETRY		sDisc;

		itoa(iDrive,cDeviceName+17,10);


		   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_FLAG_WRITE_THROUGH, NULL);
		if(hDevice==INVALID_HANDLE_VALUE)
			{
			   hDevice =CreateFile(cDeviceName,0,FILE_SHARE_READ,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice==INVALID_HANDLE_VALUE)
				{
				sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",iDrive);

				   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_FLAG_WRITE_THROUGH, NULL);
				if(hDevice==INVALID_HANDLE_VALUE)
					{
					   hDevice =CreateFile(cDeviceName,0,FILE_SHARE_READ,NULL,OPEN_EXISTING,0,NULL);
					if(hDevice==INVALID_HANDLE_VALUE)
						{
						return 0;
						}
					}
				}
			}


		if(!DeviceIoControl(hDevice,IOCTL_DISK_GET_LENGTH_INFO,NULL,0,&qwSize,sizeof(qwSize),&dwBytes,NULL))
			{
			qwSize = 0;
			}

		if(!DeviceIoControl(hDevice,IOCTL_DISK_GET_DRIVE_GEOMETRY_EX,NULL,0,&sGeoEx,sizeof(sGeoEx),&dwBytes,NULL))
			{
			qwLen = 0;
			}
		else{
			qwLen = sGeoEx.DiskSize.QuadPart;
			}

		if(!DeviceIoControl(hDevice,IOCTL_DISK_GET_DRIVE_GEOMETRY,NULL,0,&sDisc,sizeof(sDisc),&dwBytes,NULL))
			{
			CloseHandle(hDevice);
			return 0;
			}

		qwMax	= sDisc.Cylinders.QuadPart;
		qwMax  *= sDisc.SectorsPerTrack;
		qwMax  *= sDisc.TracksPerCylinder;
		qwMax  *=(sDisc.BytesPerSector)>>9;
		qwSize /= sDisc.BytesPerSector;
		qwLen  /= sDisc.BytesPerSector;

		if(qwSize>qwMax)qwMax=qwSize;
		if(qwLen >qwMax)qwMax=qwLen;

		if(pSize)*pSize=sDisc.BytesPerSector;
		}
	else{											// Win95/98 Code
		SetLastError(ERROR_NOT_SUPPORTED);
		return 0;
		}

	CloseHandle(hDevice);



return qwMax;
}


//*****************************************************************************
//*
//*		GetSectorCount
//*
//*****************************************************************************
//	Ergibt die Gre eines Laufwerks in 512 Byte Sektoren
//	iDrive	: Laufwerksnummer (A=0 B=1 ...  0x8000=Pysical_0 0x8001=Pysical_1 ... )
//	pSize	: Wenn ungleich Null, dann wird dort die wirkliche
//			  Sektorgre gespeichert.
//	Ergibt 0 bei einem Fehler
QWORD GetSectorCount(int iDrive,DWORD *pSize)
{
char			cDeviceName[] = "\\\\.\\A:";
ULARGE_INTEGER	uDummy;
ULARGE_INTEGER	uTotal;
HANDLE			hDevice;
DWORD			dwBytes;


	if(iDrive&0x8000)
		{
		return GetPhysicalCount(iDrive&~0x8000,pSize);
		}

	if(pSize)*pSize=0;

	cDeviceName[4] += iDrive;

	if(IsWinNt())											// WinNT Code
		{
		PARTITION_INFORMATION	sInfo;
		DISK_GEOMETRY			sDisk;
		BOOL					bDisk;

		   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(hDevice==INVALID_HANDLE_VALUE)
			{
			   hDevice =CreateFile(cDeviceName,0,FILE_SHARE_READ,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice==INVALID_HANDLE_VALUE)return 0;
			}

		bDisk = DeviceIoControl(hDevice,IOCTL_DISK_GET_DRIVE_GEOMETRY,NULL,0,&sDisk,sizeof(sDisk),&dwBytes,NULL);


		if(DeviceIoControl(hDevice,IOCTL_DISK_GET_PARTITION_INFO,NULL,0,&sInfo,sizeof(sInfo),&dwBytes,NULL))
			{
			CloseHandle(hDevice);
			if(pSize)*pSize=(bDisk)? sDisk.BytesPerSector:512;
			return sInfo.PartitionLength.QuadPart>>9;
			}

		if(!bDisk)
			{
			bDisk = DeviceIoControl(hDevice,IOCTL_CDROM_GET_DRIVE_GEOMETRY,NULL,0,&sDisk,sizeof(sDisk),&dwBytes, NULL);
			}

		if(bDisk)
			{
			CloseHandle(hDevice);

			if(pSize)*pSize=sDisk.BytesPerSector;

			if(GetDiskFreeSpaceEx(cDeviceName+4,&uDummy,&uTotal,NULL))
				{
				return uTotal.QuadPart>>9;
				}

			return ((sDisk.Cylinders.QuadPart*
						(__int64)sDisk.TracksPerCylinder*
						(__int64)sDisk.SectorsPerTrack*
						(__int64)sDisk.BytesPerSector)>>9);
			}

		CloseHandle(hDevice);

		if(GetDiskFreeSpaceEx(cDeviceName+4,&uDummy,&uTotal,NULL))
			{
			if(pSize)*pSize=512;
			return uTotal.QuadPart>>9;
			}

		if(pSize)*pSize=0;
		}
	else{												// Win95/98 Code
		#if WIN98_SUPPORT

		BYTE	bBoot[512];
		DWORD	dwReturn;

		if(GetDriveType(cDeviceName+4)==DRIVE_CDROM)
			{
			if(!GetDiskFreeSpaceEx(cDeviceName+4,&uDummy,&uTotal,NULL))return 0;
			if(pSize)*pSize=2048;
			return uTotal.QuadPart>>9;
			}

		if(!ReadSector(iDrive,0,1,bBoot))return 0;		// Read Bootsector

		if(bBoot[510]!=0x55)return 0;
		if(bBoot[511]!=0xAA)return 0;

		if(!memcmp(bBoot+0x52,"FAT32   ",8))
			{
			dwReturn=*(DWORD*)(bBoot+0x20);
			}
		else if(!memcmp(bBoot+0x36,"FAT16   ",8))
			{
			dwReturn=*(WORD*)(bBoot+0x13);
			if(dwReturn==0)dwReturn=*(DWORD*)(bBoot+0x20);
			}
		else if(!memcmp(bBoot+0x36,"FAT12   ",8))
			{
			dwReturn=*(WORD*)(bBoot+0x13);
			if(dwReturn==0)dwReturn=*(DWORD*)(bBoot+0x20);
			}
		else{
			return 0;
			}

		if(dwReturn==0)
			{
			SetLastError(ERROR_INVALID_DATA);
			return 0;
			}

		if(pSize)*pSize=*(WORD*)(bBoot+0x0B);

		return dwReturn;

		#endif 	
		}

	


return 0;
}

//*****************************************************************************
//*
//*		GetSectorCount
//*
//*****************************************************************************
//	Ergibt die Gre eines Laufwerks in 512 Byte Sektoren
//	iDrive	: Laufwerksnummer (A=0 B=1 ...  0x8000=Pysical_0 0x8001=Pysical_1 ... )
//	pSize	: Wenn ungleich Null, dann wird dort die wirkliche
//			  Sektorgre gespeichert.
//	Ergibt 0 bei einem Fehler
QWORD GetSectorCount(void *hDrive,DWORD *pSize)
{
CHAR			cDeviceName[]="\\\\.\\A:";
DWORD			dwBytes;
HANDLE			hDevice;
ULARGE_INTEGER	uDummy;
ULARGE_INTEGER	uTotal;
unsigned		uNr;

		 
	   uNr=(int)hDrive-1;
	if(uNr>=64)return FALSE;

	hDevice = aHandles[(int)hDrive-1];



	if(uNr>=32)												// Ist es ein physikalisches Laufwerk
		{
		return GetPhysicalCount(uNr-32,pSize);
		}

	if(pSize)*pSize=0;

	if(IsWinNt())											// WinNT Code
		{
		PARTITION_INFORMATION	sInfo;
		DISK_GEOMETRY			sDisk;
		BOOL					bDisk;


		bDisk=DeviceIoControl(hDevice,IOCTL_DISK_GET_DRIVE_GEOMETRY,NULL,0,&sDisk,sizeof(sDisk),&dwBytes,NULL);

		if(DeviceIoControl(hDevice,IOCTL_DISK_GET_PARTITION_INFO,NULL,0,&sInfo,sizeof(sInfo),&dwBytes,NULL))
			{
			if(pSize)*pSize=(bDisk)? sDisk.BytesPerSector:512;
			return sInfo.PartitionLength.QuadPart>>9;
			}

		if(!bDisk)
			{
			bDisk = DeviceIoControl(hDevice,IOCTL_CDROM_GET_DRIVE_GEOMETRY,NULL,0,&sDisk,sizeof(sDisk),&dwBytes, NULL);
			}

		if(bDisk)
			{
			if(pSize)*pSize=sDisk.BytesPerSector;

			if(GetDiskFreeSpaceEx(cDeviceName+4,&uDummy,&uTotal,NULL))
				{
				return uTotal.QuadPart>>9;
				}

			return ((sDisk.Cylinders.QuadPart*
						(__int64)sDisk.TracksPerCylinder*
						(__int64)sDisk.SectorsPerTrack*
						(__int64)sDisk.BytesPerSector)>>9);
			}

		if(GetDiskFreeSpaceEx(cDeviceName+4,&uDummy,&uTotal,NULL))
			{
			if(pSize)*pSize=512;
			return uTotal.QuadPart>>9;
			}

		if(pSize)*pSize=0;
		}
	else{												// Win95/98 Code
		#if WIN98_SUPPORT
		return GetSectorCount(uNr,pSize);
		#endif 	
		}


return 0;
}

//*****************************************************************************
//*
//*		HasAccess
//*
//*****************************************************************************
//	Ergibt ob man Zugriff hat auf ein Laufwerk
//	iDrive	: Laufwerksnummer (A=0 B=1 ...  0x8000=Pysical_0 0x8001=Pysical_1 ... )
//	bWrite	: Einen Schreibzugriff prfen
//	Ergibt	0 = Kein Zugriff
//			1 = Zugriff aber keine Disk ist im Laufwerk
//			2 = Zugriff mit Disk
int HasAccess(int iDrive,BOOL bWrite)
{
DWORD	dwFlags;
DWORD	dwMaxChars;
DWORD	dwSerialNbr;
char	cName[]="A:\\";
char	cBuffer[64];
char	cFsName[64];
void   *hDevice;
BOOL	bOk;


		hDevice=OpenDevice(iDrive,(bWrite)? 2:1);
	if(!hDevice)return 0;
	
	CloseDevice(hDevice,TRUE);

	if(iDrive&0x8000)return 2;

	cName[0]+=iDrive;

	   bOk=GetVolumeInformation(cName,cBuffer,sizeof(cBuffer),&dwSerialNbr,&dwMaxChars,&dwFlags,cFsName,sizeof(cFsName));
	if(bOk==0 &&  GetLastError()==ERROR_NOT_READY)return 1;


return 2;
}

//*****************************************************************************
//*
//*		CloseDevice
//*
//*****************************************************************************
//	Schliest ein Laufwerks Device
//	hDrive		: Ist das Handle des Laufwerks
//	bNoMount	: Laufwerke nicht neu Mounten
BOOL CloseDevice(void *hDrive,BOOL bNoMount)
{
char		cName[64];
HANDLE		hDevice;
DWORD		dwLetter;
DWORD		dwBytes;
DWORD		dwBits;
unsigned	uNr;
 
 
 
	   uNr=(int)hDrive-1;
	if(uNr>=64)return FALSE;
 
 
	if(!hDrive)return FALSE;
 
	hDevice = aHandles[uNr];
 
	DeviceIoControl(hDevice,FSCTL_UNLOCK_VOLUME,NULL,0,NULL,0,&dwBytes,0);
 
	if(aUnmount[uNr])
		{
		DeviceIoControl(hDevice,IOCTL_STORAGE_LOAD_MEDIA,NULL,0,NULL,0,&dwBytes,0);
		}
 
	aHandles[uNr] = INVALID_HANDLE_VALUE;

														// "\\.\PhysicalDrive?" weider erstellen	
	if(aDrvName[uNr*64] && uNr>=32 && aAccess[uNr]>=2 && !bNoMount)	
		{
    	strcpy(cName,"\\\\.\\PhysicalDrive0");
		itoa(uNr&0x1F,cName+17,10);
		DefineDosDevice(DDD_RAW_TARGET_PATH,cName+4,aDrvName+uNr*64);
		}
 
 
	if(aDrvList[uNr] && !bNoMount)						// Laufwerke wieder mouten
		{	
		dwBits = aDrvList[uNr];	   
		
		for(dwLetter=0;dwLetter<26;dwLetter++,dwBits>>=1)
			{
			if(!(dwBits&1))continue;
 
			sprintf(cName,"%c:\\",'A'+dwLetter);
			SetVolumeMountPoint_(cName,aDrvGuid+dwLetter*64); 
			}
			
		   dwBits = aDrvList[uNr];	   					// Explorer bekannt geben das es neue Laufwerke gibt
		if(dwBits)
			{
			Sleep(200);
			NotifyNewDrives(dwBits);
			}
														
		aDrvList[uNr] = 0;
		}
 
	if(uNr>=32 && aAccess[uNr]>=2 && !bNoMount)			// Logische Laufwerke prfen
		{
		MountPysicalDisk(uNr-32);
		}
 
	if(aReparse[uNr])									// Die Partitionen neu einlesen
		{
		DeviceIoControl(hDevice,IOCTL_DISK_UPDATE_PROPERTIES,NULL,0,NULL,0,&dwBytes,NULL);
		}

 
return CloseHandle(hDevice);
}

//*****************************************************************************
//*
//*		OpenDevice
//*
//*****************************************************************************
//	ffnet ein Laufwerks Device
//	iDrive		: Laufwerksnummer (A=0 B=1 ...  0x8000=Pysical_0 0x8001=Pysical_1 ... )
//	iAccess		: Zugriffsart (0=Keinen 1=Read 2=Write&Read)
//	bLock		: Das Laufwerk sperren
//	Ergibt 0 bei einem Fehler
void *OpenDevice(int iDrive,int iAccess,BOOL bLock)
{
static const int	aMode[3]={GENERIC_ALL,GENERIC_READ,GENERIC_WRITE|GENERIC_READ};
char				cDeviceName[64];
char				cDriveName [64];
BOOL				bPysical=FALSE;
BOOL				bUnmount=FALSE;
DWORD				dwDrvList;
DWORD				dwCompare;
DWORD				dwLetter;
DWORD				dwBytes;
HANDLE				hDevice;




	if(iDrive&0x8000)										// Physikalisches Laufwerk
		{
		bPysical  =  TRUE;
		iDrive   &= ~0x8000;
		}

	if(iDrive <0 || iDrive>=26)
		{
		SetLastError(ERROR_INVALID_DRIVE);
		return 0;
		}

	if(iAccess<0 || iAccess>=3)
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return 0;
		}

	
	dwDrvList	= 0;


	if(IsWinNt())											// WinNT Code
		{
		if(bPysical)
			{
    		strcpy(cDeviceName,"\\\\.\\PhysicalDrive0");
			itoa(iDrive,cDeviceName+17,10);
			iDrive+=32;
			}
		else{
			strcpy(cDeviceName,"\\\\.\\A:");
			cDeviceName[4] += iDrive;
			}


		   hDevice =CreateFile(cDeviceName,aMode[iAccess],FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(hDevice==INVALID_HANDLE_VALUE)
			{
			if(!bPysical)return NULL;

			sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",iDrive-32);

			   hDevice =CreateFile(cDeviceName,aMode[iAccess],FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice==INVALID_HANDLE_VALUE)
				{
				return NULL;
				}
			}

		if(bLock)
			{
			dwBytes=0;

			if(iAccess==2)									// Logische Laufwerke abmelden
				{
				if(bPysical)									
					{
					dwCompare	= (iDrive-32)|0x8000;
					
					for(dwLetter=0;dwLetter<26;dwLetter++)
						{
						if(GetDriveOfVolume(dwLetter)!=dwCompare)continue;
 
						sprintf(cDriveName,"%c:\\",'A'+dwLetter);
 
						if(GetVolumeNameForVolumeMountPoint_(cDriveName,aDrvGuid+dwLetter*64,64))
						if(DeleteVolumeMountPoint_(cDriveName))
							{
							dwDrvList |= 1<<dwLetter;
							}
						}
					}
				else{
					sprintf(cDriveName,"%c:\\",'A'+iDrive);
 
					if(GetVolumeNameForVolumeMountPoint_(cDriveName,aDrvGuid+iDrive*64,64))
					if(DeleteVolumeMountPoint_(cDriveName))
						{
						dwDrvList	|= 1<<iDrive;
						}
					}
				}

			DeviceIoControl(hDevice,FSCTL_ALLOW_EXTENDED_DASD_IO,NULL,0,NULL,0,&dwBytes,NULL);

			if(!DeviceIoControl(hDevice,FSCTL_LOCK_VOLUME,NULL,0,NULL,0,&dwBytes,NULL))
				{
				CloseHandle(hDevice);
				return FALSE;
				}

			DeviceIoControl(hDevice,FSCTL_DISMOUNT_VOLUME,NULL,0,NULL,0,&dwBytes,NULL);
			}
		}
	else{													// Win98 Code
		#if WIN98_SUPPORT

		if(bPysical)return FALSE;

		hDevice = CreateFile("\\\\.\\vwin32",0,0,NULL,0,FILE_FLAG_DELETE_ON_CLOSE,NULL);

		#else 
		
		return NULL;

		#endif 	
		}

	if(hDevice==INVALID_HANDLE_VALUE)
		{
		return NULL;
		}
	
	aDrvName[iDrive*64] = 0;
	aReparse[iDrive]	= 9;
	aAccess	[iDrive]	= iAccess;
	aHandles[iDrive]	= hDevice;
	aUnmount[iDrive]	= bUnmount;
	aDrvList[iDrive]   |= dwDrvList;


return (void*)(iDrive+1);
}


//*****************************************************************************
//*
//*		GetDriveData
//*
//*****************************************************************************
//	Auslesen der Daten eines Laufwerek
//	iDrive		: Laufwerksnummer (A=0 B=1 ...  0x8000=Pysical_0 0x8001=Pysical_1 ... )
//	pDiscData	: Zeiger auf den Puffer in dem die Sektoren geschrieben werden.
//	Ergibt TRUE wenn die Daten gelesen wurden.
BOOL GetDriveData(int iDrive,DISK_GEOMETRY *pDiscData)
{
BOOL			bPysical=FALSE;
HANDLE			hDevice;
BOOL			bResult;
DWORD			dwBytes;



	pDiscData->Cylinders.QuadPart = 1023;
	pDiscData->TracksPerCylinder  = 255;
	pDiscData->SectorsPerTrack    = 63;


	if(iDrive&0x8000)										// Physikalisches Laufwerk
		{
		bPysical  =  TRUE;
		iDrive   &= ~0x8000;
		}

	if(iDrive<0 || iDrive>=26)
		{
		SetLastError(ERROR_INVALID_DRIVE);
		return 0;
		}

	if(IsWinNt())											// WinNT Code
		{
    	char	cDeviceName[64];

		if(bPysical)
			{
    		strcpy(cDeviceName,"\\\\.\\PhysicalDrive0");
			itoa(iDrive,cDeviceName+17,10);
			}
		else{
			strcpy(cDeviceName,"\\\\.\\A:");
			cDeviceName[4] += iDrive;
			}

		   hDevice = CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(hDevice == INVALID_HANDLE_VALUE)
			{
			if(!bPysical)return FALSE;

			sprintf(cDeviceName,"\\\\?\\GLOBALROOT\\Device\\Harddisk%i\\Partition0",iDrive);

			   hDevice = CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
			if(hDevice == INVALID_HANDLE_VALUE)
				{
				return FALSE;
				}
			}


		bResult=DeviceIoControl(hDevice,IOCTL_DISK_GET_DRIVE_GEOMETRY,NULL,0,pDiscData,sizeof(DISK_GEOMETRY),&dwBytes,NULL);

		CloseHandle(hDevice);

		return bResult;
		}

	SetLastError(ERROR_NOT_SUPPORTED);



return FALSE;
}


//*****************************************************************************
//*
//*		GetDriveOfVolume
//*
//*****************************************************************************
//	Holt die Nummer des physikalischen Laufwerks
//	iDrive		: Laufwerksnummer (A=0 B=1 ...  )
//	pPartition	: Hier wird die Partition gespeichert wenn nicht NULL. 
//					-1    = keine Partition gefunedn
//					 1..N = Partitionsnummer
//	Ergibt die Laufwerksnummer (0x8000+N) oder 0 bei einem Fehler
DWORD GetDriveOfVolume(int iDrive,int *pPartition)
{
VOLUME_DISK_EXTENTS_	sData;
DWORD					dwBytes;
HANDLE					hDevice;
BOOL					bResult;



	if(iDrive<0 || iDrive>=26)
		{
		SetLastError(ERROR_INVALID_DRIVE);
		return 0;
		}


	if(IsWinNt())											// WinNT Code
		{
    	char	cDeviceName[64];
		char	cDeviceLink[64];
		char	cDeviceTemp[64];
		int		iNumber;
		int		iPart;

		strcpy(cDeviceName,"\\\\.\\A:");
		cDeviceName[4] += iDrive;

		   hDevice =CreateFile(cDeviceName,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
		if(hDevice==INVALID_HANDLE_VALUE)return FALSE;


		bResult = DeviceIoControl(hDevice,IOCTL_VOLUME_GET_VOLUME_DISK_EXTENTS,NULL,0,&sData,sizeof(sData),&dwBytes,0);

		CloseHandle(hDevice);

		if(bResult && sData.NumberOfDiskExtents>0)
			{
			iNumber = sData.Extents[0].DiskNumber;

			if(pPartition)										// Partitionsnummer auslesen
				{
				*pPartition = -1;
				sprintf(cDeviceName,"\\GLOBAL??\\%c:",'A'+iDrive);

				if(QuerySymbolLink(cDeviceName,cDeviceLink,sizeof(cDeviceLink)))
					{
					for(iPart=0;iPart<8;iPart++)
						{
						sprintf(cDeviceName,"\\Device\\Harddisk%i\\Partition%i",iNumber,iPart);

						if(!QuerySymbolLink(cDeviceName,cDeviceTemp,sizeof(cDeviceTemp)))
							{
							continue;
							}

						if(!stricmp(cDeviceTemp,cDeviceLink))
							{
							*pPartition = iPart;
							break;
							}
						}
					}
				}

			return iNumber|0x8000;
			}
		}


	if(pPartition)*pPartition=-1;

	SetLastError(ERROR_NOT_SUPPORTED);



return 0;
}


//*****************************************************************************
//*
//*		GetVolumeOfDrive
//*
//*****************************************************************************
//	Sucht alle logischen Laufwerke zu einem physikalischen Laufwerk.
//	iDrive		: Is die Nummer des physikalischen Laufwerks (0..N oder 0x8000..0x8000+N)
//	Ergibt ein Bitfeld mit den logischen Laufwerken. 
DWORD GetVolumeOfDrive(int iDrive)
{
unsigned	uDrive;
unsigned	uBits;
unsigned	uPos;
unsigned	uRet;



	   iDrive &= ~0x80008000;
	if(iDrive>=32)return 0;

	uDrive	= iDrive|0x8000;
	uBits	= GetLogicalDrives();
															 
	for(uPos=0,uRet=0;uPos<26;uPos++,uBits>>=1)				// Alle Laufwerke abschreiten
		{
		if(!(uBits  ))break;
		if(!(uBits&1))continue;
	
		if(GetDriveOfVolume(uPos)==uDrive)
			{
			uRet |= 1<<uPos;
			}
		}


return uRet;
}

//*****************************************************************************
//*
//*		GetDriveGuid
//*
//*****************************************************************************
//	Holt den GUID-Volume-Namen fr ein logisches Laufwerk
//	iDrive				: Laufwerksnummer (A=0 B=1 ...  Z=25 )
//	pVolumeName			: Hier wird der GUID-Volume-Namen gespeichert.
//	iSize				: Ist die Gre des Namenspuffers.
//	Ergibt TRUE wenn erfolgreich.
BOOL GetDriveGuid(int iDrive,LPSTR pVolumeName,int iSize)
{
char	cDriveName[]="A:\\";
CHAR	cBuffer[64*32];
CHAR	cPath  [64];
CHAR	cName  [64];
LPCSTR	pName;
LPCSTR	pLink;
int		iOff;
int		iPos;
int		iNum;
int		iLen;



	if(iSize<=0)											// Prfe die Puffergre
		{
		SetLastError(ERROR_BUFFER_OVERFLOW);
		return FALSE;
		}

	*pVolumeName = 0;

	if(iDrive&0x4000)										// Volume-Namen von Partition ermitteln 
		{
		if((iDrive&0xFFFF8000) || !(iDrive&0x0F))
			{
			SetLastError(ERROR_INVALID_DRIVE);
			return FALSE;
			}

		iDrive &= ~0x4000;

		sprintf(cPath,"\\Device\\Harddisk%i",iDrive>>4);
		sprintf(cName,"Partition%i",iDrive&0x0F);

		   iNum	= FindSymbolLinks(cPath,cName,cBuffer,sizeof(cBuffer));
		if(iNum<0)
			{
			SetLastError(ERROR_PATH_NOT_FOUND);
			return FALSE;
			}

		strncpy(cName,cBuffer+strlen(cBuffer)+1,sizeof(cName));

		iNum = FindSymbolLinks("\\GLOBAL??","Volume{*",cBuffer,sizeof(cBuffer));
		
		for(iOff=0,iPos=0;;iPos++)							// Suce GUID zu Link
			{
			if(iPos>=iNum)
				{
				SetLastError(ERROR_FILE_NOT_FOUND);
				return FALSE;
				}

			pName	= cBuffer+iOff;
			iLen	= strlen(pName);
			iOff   += iLen+1;
			pLink	= cBuffer+iOff;
			iOff   += strlen(pLink)+1;

			if(stricmp(pLink,cName))continue;
			if(iLen+6>=iSize)
				{
				SetLastError(ERROR_BUFFER_OVERFLOW);
				return FALSE;
				}

			memcpy(pVolumeName  ,"\\\\?\\",4);				// Volume-Namen erstellen
			memcpy(pVolumeName+4,pName,iLen);
			pVolumeName[4+iLen  ] = '\\';
			pVolumeName[4+iLen+1] = 0;

			return TRUE;
			}
		}
											
	if((unsigned)iDrive>=26)								// Volume-Namen von logischem Laufwerk
		{
		SetLastError(ERROR_INVALID_DRIVE);
		return FALSE;
		}

	cDriveName[0] += iDrive;


return GetVolumeNameForVolumeMountPoint_(cDriveName,pVolumeName,iSize);
}


//*****************************************************************************
//*
//*		SetDriveGuid
//*
//*****************************************************************************
//	Setzt den GUID-Volume-Namen fr ein logisches Laufwerk.
//	iDrive				: Laufwerksnummer (A=0 B=1 ...  Z=25 )
//	pVolumeName			: Ist der GUID-Volume-Namen fr das Laufwerk.
//						  z.B.: "\\?\Volume{b6f5fe78-275e-11e5-9b96-806e6f6e6963}\"
//	Ergibt TRU wenn erfolgreich.
BOOL SetDriveGuid(int iDrive,LPCSTR pVolumeName)
{
char	cDriveName[]="A:\\";


	if((unsigned)iDrive>=26)
		{
		SetLastError(ERROR_INVALID_DRIVE);
		return FALSE;
		}

	cDriveName[0] += iDrive;


return SetVolumeMountPoint_(cDriveName,pVolumeName);
}


//*****************************************************************************
//*
//*		DelDriveGuid
//*
//*****************************************************************************
//	Lcht ein logisches Laufwerk vom GUID-Volume-Namen.
//	iDrive				: Laufwerksnummer (A=0 B=1 ...  Z=25 )
//	Ergibt TRUE wenn erfolgreich.
BOOL DelDriveGuid(int iDrive)
{
char	cDriveName[]="A:\\";


	if((unsigned)iDrive>=26)
		{
		SetLastError(ERROR_INVALID_DRIVE);
		return FALSE;
		}

	cDriveName[0] += iDrive;


return DeleteVolumeMountPoint_(cDriveName);
}



