//*****************************************************************************
//*
//*
//*		FileBuffer.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Die FileBuffer kann Dateien/Speicherbereiche und Resourcen als einen
//	gepufferten Stream ffnen. Weiters ist es mglich Konervertierungen
//	zwischen den Stream und die Ausgabe zu setzen.
//
//	Funktionen der Klasse
//
//	CFileBuffer(char *pName,int iBufferSize=CFB_BUFFER_SIZE);
//	CFileBuffer(            int iBufferSize=CFB_BUFFER_SIZE);
//
//	bool	Open    (char *pName ,int iMode=0,int iSeekEnd=0);		// Datei ffnen
//	bool	OpenWide(char *pName ,int iMode=0,int iSeekEnd=0);		// Datei ffnen
//	bool	OpenMem (void *pMem=0,int iLen =0,int iMode=CFB_READ );	// Memory ffnen
//	bool	OpenRes (const char *pName,const char *pType=0);		// Resource ffnen
//	bool	OpenRes (unsigned    pName,const char *pType=0);
//	bool	OpenProc(void *pHandle,	CFB_PROC_READ  Read,			// Allgemein ffnen
//									CFB_PROC_CLOSE Close,
//									CFB_PROC_WRITE Write,
//									CFB_PROC_SEEK  Seek)
//
//	bool	Close();								// Datei schliesen
//	bool	Flush();								// Puffer speichern
//
//	int		Read(unsigned char  *pMem,int);			// Datei       lesen
//	int		Read(         char  *pMem,int);			// Datei       lesen
//	int		Read(unsigned ifb64 *pValue);			// Datei Zahl  lesen
//	int		Read(		  ifb64 *pValue);			// Datei Zahl  lesen
//	int		Read(unsigned int   *pValue);			// Datei Zahl  lesen
//	int		Read(		  int   *pValue);			// Datei Zahl  lesen
//	int		Read(unsigned long  *pValue);			// Datei Zahl  lesen
//	int		Read(         long  *pValue);			// Datei Zahl  lesen
//	int		Read(unsigned short *pValue);			// Datei Zahl  lesen
//	int		Read(		  short *pValue);			// Datei Zahl  lesen
//	int		Read(unsigned char  *pValue);			// Datei Zahl  lesen
//	int		Read(		  char  *pValue);			// Datei Zahl  lesen
//	int		Read(unsigned ifb64 &iValue);			// Datei Zahl  lesen
//	int		Read(		  ifb64 &iValue);			// Datei Zahl  lesen
//	int		Read(unsigned int   &iValue);			// Datei Zahl  lesen
//	int		Read(		  int   &iValue);			// Datei Zahl  lesen
//	int		Read(unsigned long  &iValue);			// Datei Zahl  lesen
//	int		Read(         long  &iValue);			// Datei Zahl  lesen
//	int		Read(unsigned short &iValue);			// Datei Zahl  lesen
//	int		Read(		  short &iValue);			// Datei Zahl  lesen
//	int		Read(unsigned char  &iValue);			// Datei Zahl  lesen
//	int		Read(		  char  &iValue);			// Datei Zahl  lesen
//	int		ReadLine(char *,int Size );				// Eine  Zeile lesen
//	int		ReadByte ();							// Datei Byte  lesen
//	int		ReadWord ();							// Datei Word  lesen
//	int		ReadDWord();							// Datei DWord lesen
//  ifb64	ReadQWord();							// Datei QWord lesen
//
//	int		Write(unsigned char  *,int );			// Datei       schreiben
//	int		Write(         char  *,int );			// Datei       schreiben
//	int		Write(unsigned char  *pMem );			// Datei Text  schreiben
//	int		Write(         char  *pMem );			// Datei Text  schreiben
//	int		Write(unsigned int   iValue);			// Datei Zahl  schreiben
//	int		Write(		   int   iValue);			// Datei Zahl  schreiben
//	int		Write(unsigned short iValue);			// Datei Zahl  schreiben
//	int		Write(		   short iValue);			// Datei Zahl  schreiben
//	int		Write(unsigned char  iValue);			// Datei Zahl  schreiben
//	int		Write(		   char  iValue);			// Datei Zahl  schreiben
//	int		Write(unsigned ifb64 iValue);			// Datei Zahl  schreiben
//	int		Write(		   ifb64 iValue);			// Datei Zahl  schreiben
//	void    WriteByte (    int   iValue);			// Datei Byte  schreiben
//	void    WriteWord (    int   iValue);			// Datei Word  schreiben
//	void    WriteDWord(    int   iValue);			// Datei DWord schreiben
//	void    WriteQWord(    ifb64 iValue);			// Datei DWord schreiben
//	void    WriteLF();								// Datei Zeile schreiben
//	int		WriteFormat(char *Fmt,...);				// Mit Format  speichern
//
//
//	char   *Buffer();								// Zeiger auf Puffer holen
//	bool	Eof();									// Datei Ende erreicht
//	int		Tell();									// Datei Position
//	int		Seek(int,int iMode=SEEK_SET)			// Datei Position setzen
//	int		GetLength();							// Dateilnge holen
//
//	int		GetUserData()							// Benutzerdaten lesen
//	void   *GetUserDataPtr()						// Benutzerdaten lesen
//	void	SetUserData(int   Data);				// Benutzerdaten setzen
//	void	SetUserData(void *Data);				// Benutzerdaten setzen
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#ifdef     _WIN32
#include	<io.h>
#include	<wchar.h>
#ifndef		ifb64
#define		ifb64	__int64
#endif
#endif

#if defined(LINUX) || defined(linux)
#include	<unistd.h>
#include	"stdfunk.h"
#ifndef		ifb64
#define		ifb64	long long
#endif
#endif

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#ifdef	   _MSC_VER
#pragma 	warning( disable :4996 )
#endif


#include	<fcntl.h>
#include	<memory.h>
#include	<string.h>

#include	<sys/stat.h>
#include	<sys/types.h>

#include	"FileBuffer.h"
#include	"ProcPrintf.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifndef		SEEK_SET
#define 	SEEK_SET 	0
#endif

#ifndef		SEEK_CUR
#define 	SEEK_CUR 	1
#endif

#ifndef		SEEK_END
#define 	SEEK_END 	2
#endif

#ifndef		wtoi
#define 	wtoi       _wtoi
#endif

#ifndef		wopen
#define 	wopen      _wopen
#endif

#ifndef		wchmod
#define 	wchmod     _wchmod
#endif

#ifndef		wcsicmp
#define 	wcsicmp	   _wcsicmp
#endif


typedef struct
	{
	CFileBuffer	   *pClass;							// Zeiger auf Classe
	void		   *pHandle;						// Datei-Handle
	int				iSeekAdd;						// Letzte Schreibposition
	char			iCheckWr;						// Letztes Schreibeoperation prfen
	}AnsiHandle;

FileBufferConvert	CFileBuffer::sWideToAnsi =		// Konvertierungsfunktione von UTF16 Datei nach ANSI
	{
	CFileBuffer::WideWrite,
	CFileBuffer::WideRead,
	CFileBuffer::WideSeek,
	CFileBuffer::WideConv,
	};

FileBufferConvert	CFileBuffer::sAnsiToWide =		// Konvertierungsfunktione von ANSI Datei nach UTF16
	{
	CFileBuffer::AnsiWrite,
	CFileBuffer::AnsiRead,
	CFileBuffer::AnsiSeek,
	CFileBuffer::AnsiConv,
	};


//*****************************************************************************
//*
//*     WritePrintf
//*
//*****************************************************************************
//	Speicherfunktion fr WriteFormat mit pprintf
//	pParam	: Zeiger auf Klasseninstanz
//	pBuffer	: Zeiger auf den zuspeichernten Puffer
//	iSize	: Anzahl der Zeichen die gespeichert werden sollen
//	Zu speichernte Zeichen
static int WritePrintf(void *pParam,const char *pBuffer,int iSize)
{
return ((CFileBuffer*)pParam)->Write(pBuffer,iSize);
}

static int WritePrintf(void *pParam,const wchar_t *pBuffer,int iSize)
{
return ((CFileBuffer*)pParam)->Write(pBuffer,iSize*2);
}

//*****************************************************************************
//*
//*		Dummy Funktionen fr Speicher Funktionen
//*
//*****************************************************************************

int	_cdecl CFileBuffer::MemClose(void *pHandle                            ){return 0;};
int	_cdecl CFileBuffer::MemWrite(void *pHandle,const void *pMem,int iCount){return 0;};
int	_cdecl CFileBuffer::MemRead (void *pHandle,      void *pMem,int iCount){return 0;};
int	_cdecl CFileBuffer::MemSeek (void *pHandle,      int   iPos,int iMode ){return 0;};


char *_cdecl CFileBuffer::BufferNull(void *pClass){return NULL;}


//*****************************************************************************
//*
//*		BufferGet
//*
//*****************************************************************************
//	Funktion fr das Holen des Pufferzeigers ohne Nullbytes am Ende anzuhngen
//	pClass	: Ist der Zeiger auf die Klasse
//	Ergibt den Zeiger auf den Puffer
char *_cdecl CFileBuffer::BufferNone(void *pClass)
{
CFileBuffer	*pThis=(CFileBuffer*)pClass;


return pThis->pBuffer;
}

//*****************************************************************************
//*
//*		BufferGet
//*
//*****************************************************************************
//	Standartfunktion fr das Holen des Pufferzeigers
//	pClass	: Ist der Zeiger auf die Klasse
//	Ergibt den Zeiger auf den Puffer
char *_cdecl CFileBuffer::BufferGet(void *pClass)
{
CFileBuffer	*pThis=(CFileBuffer*)pClass;


	*(short*)(pThis->pBuffer+pThis->iBufferEnd)=0;


return pThis->pBuffer;
}

//*****************************************************************************
//*
//*     Konstruktor
//*
//*****************************************************************************
CFileBuffer::CFileBuffer(int iBufferSize)
{
	Init(iBufferSize);
}

CFileBuffer::CFileBuffer(const char *pName,int iBufferSize)
{
	Init(iBufferSize);
	Open(pName);								// Datei zum Lesen ffnen
}

CFileBuffer::CFileBuffer(const wchar_t *pName,int iBufferSize)
{
	Init(iBufferSize);
	Open(pName);								// Datei zum Lesen ffnen
}

//*****************************************************************************
//*
//*     Destruktor
//*
//*****************************************************************************
CFileBuffer::~CFileBuffer()
{
	Close();
}


//*****************************************************************************
//*
//*		Init
//*
//*****************************************************************************
//	Die Klasse iniitialisieren
//	iBufferSize	: Ist die Gre des Puffers
void CFileBuffer::Init(int iBufferSize)
{

	pFileHandle     = (void*)-1;				// Dateihandle
	bEofFlag		= true;						//		*
	bMemFlag  		= false;					//		*
	bOpenFlag 		= false;					//		*
	iCloseSeek		=-1;						// Dateiposition fr's schlieen

	if(iBufferSize<512)
		{
		iBufferSize = 512;
		}

	iBufferPos 		= iBufferSize;				// Puffer Lese Position
	iBufferLen 		= iBufferSize;				// Puffer Lnge
	iBufferMin 		= iBufferSize/8;			// Minimal Byte-Zahl
	iBufferEnd 		= 0;						// Puffer Byteanzahl
	iBufferOff		= 0;						// Puffer Offset im Stream
	pBuffer    		= 0;						// Zeiger auf Pufferspeicher

	if(iBufferMin<0x1000)
		{
		iBufferMin  = iBufferSize/2;
		}

	pProcClose 		= 0;						// Dateifunktionen
	pProcWrite 		= 0;						//		*
	pProcRead  		= 0;						//		*
	pProcSeek  		= 0;						//		*
	pProcConv		= 0;						//		*
	pProcBuff		= BufferNull;				//		*

	pUserData 		= 0;						// User-Daten

}

//*****************************************************************************
//*
//*		OpenProc
//*
//*****************************************************************************
//	ffnet einen gepufferen Stream mit speziellen Zugriffsfunktionen
//	pHandle		: Ist ein Zeiger auf einen beliebigen Speicherberich der
//				  als Parameter an die Funktionen bergeben wird.
//	pRead		: Ist die Funktion mit der aus dem Stream gelesen wird.
//	pClose		: Ist die Funktion mit der der Stream geschlossen wird.
//	pWrite		: Ist die Funktion mit der in dem Stream geschrieben wird.
//				  Ist Write gleich Null wird der Schreibmodus aktiviert.
//	Ergibt 'false' bei einem Fehler
bool CFileBuffer::OpenProc(void *pHandle,CFB_PROC_READ  pRead,
										 CFB_PROC_CLOSE pClose,
										 CFB_PROC_WRITE pWrite,
										 CFB_PROC_SEEK  pSeek)
{

	if(bOpenFlag)Close();

		pBuffer = new char[iBufferLen+4];		// Puffer Speicher
	if(!pBuffer)
		{
		iBufferPos  = 0;						// Puffer Lese Position
		iBufferEnd  = 0;						// Puffer Byteanzahl
		bWriteFlag  = 0;						// Puffer schreiben
		iOpenMode	= 0;						// Lese oder Schreib-Modus

		return false;
		}

	iOpenMode   =(pWrite)? 1:0;					// Lese oder Schreib-Modus
	pProcWrite 	= pWrite;						// Dateifunktionen einstellen
	pProcClose 	= pClose;						//		*
	pProcSeek  	= pSeek;						//		*
	pProcRead	= pRead;						//		*

	iBufferPos  = 0;							// Puffer Lese Position
	iBufferEnd  = 0;							// Puffer Byteanzahl
	bWriteFlag  = 0;							// Puffer schreiben
	pFileHandle	= pHandle;						// Dateihandle
	bEofFlag	= false;
	bOpenFlag	= true;


return true;
}

//*****************************************************************************
//*
//*		OpenConv
//*
//*****************************************************************************
//	Eine Datei mit einer Konvertierung ffnen.
//	pName		: Ist der Dateinamen der Datei. Wenn der  Dateinamen die
//				  mit "res://" anfngt wird eine Resource geladen.
//	iMode		: Modus mit dem die Datei geffnet wird.
//					CFB_READ	Datei zum Lesen ffnen
//					CFB_WRITE	Datei zum Schreiben ffnen
//					CFB_CREATE	Datei neuzeugen
//	iSeekEnd	: Wenn dieser Wert ungleich Null ist, dann wird der Dateizeiger
//				  ans Eede der Datei versetzt.
//	pName		: Ist der Name der Datei
//	pConverter	: Ist der Zeiger auf die Konvertierungsfunktionen.
//				  Bei NULL erfollgt keine Konvertierung.
//	Ergibt true wenn die Datei geffnet wurde, bzw. false bei einem Fehler.
bool CFileBuffer::OpenConv(const char *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter)
{

	if(!Open(pName,iMode,iSeekEnd))return false;

	if(!pConverter)return true;
	if(!pConverter->pConv(this,pConverter,0))
		{
		Close();
		return false;
		}

	pProcConv = pConverter->pConv;


return true;
}

bool CFileBuffer::OpenConv(const wchar_t *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter)
{

	if(!Open(pName,iMode,iSeekEnd))return false;

	if(!pConverter)return true;
	if(!pConverter->pConv(this,pConverter,0))
		{
		Close();
		return false;
		}

	pProcConv = pConverter->pConv;


return true;
}

//*****************************************************************************
//*
//*     OpenRes
//*
//*****************************************************************************
//	ffnet eine Resource via Resource-ID
//	uName   : Ist die Resource-ID der Resource
//	pName   : Ist der Name der der Resource
//	pType   : Ist der Resource-Ordner, wenn pType Null ist wird "DATA" benutzt.
//  Ergibt true wenn die Resource geffnet wurde, bzw. false bei einem Fehler.
bool CFileBuffer::OpenRes(unsigned    uName,const char *pType){return OpenRes((char*)MAKEINTRESOURCE(uName),pType);}
bool CFileBuffer::OpenRes(const char *pName,const char *pType)
{
void	   *pData;
unsigned	uSize;


	if(bOpenFlag)Close();
	if(!pType)pType="DATA";

		hResSrc=FindResourceA(NULL,pName,pType);
	if(!hResSrc)return false;


	hResHg  = LoadResource  (NULL,(HRSRC)hResSrc);
	uSize   = SizeofResource(NULL,(HRSRC)hResSrc);
	pData   = LockResource  ((HGLOBAL)   hResHg );


	if(OpenMem(pData,uSize,CFB_READ))
		{
		pProcClose = ResClose;
		pProcBuff  = BufferNone;
		return true;
		}


return false;
}

bool CFileBuffer::OpenRes(unsigned       uName,const wchar_t *pType){return OpenRes((wchar_t*)MAKEINTRESOURCE(uName),pType);}
bool CFileBuffer::OpenRes(const wchar_t *pName,const wchar_t *pType)
{
void	   *pData;
unsigned	uSize;


	if(bOpenFlag)Close();
	if(!pType)pType=(const wchar_t*)L"DATA";


		hResSrc=FindResourceW(NULL,pName,pType);
	if(!hResSrc)return false;


	hResHg  = LoadResource  (NULL,(HRSRC)hResSrc);
	uSize   = SizeofResource(NULL,(HRSRC)hResSrc);
	pData   = LockResource  ((HGLOBAL)   hResHg );


	if(OpenMem(pData,uSize,CFB_READ))
		{
		pProcClose = ResClose;
		pProcBuff  = BufferNone;
		return true;
		}



return false;
}

//*****************************************************************************
//*
//*     OpenMem
//*
//*****************************************************************************
//	ffnet einen gepufferten Stream aus einem Speicherbereich
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	iLen		:	Lnge des Speicherbereich
//	iMode   	:	CFB_READ   = Aus dem Speicher lesen ab Position 0.
//					CFB_WRITE  = Beim Schreiben in den Speicher wchst seine
//								 Gre automatisch mit. Der bergebene Zeiger
//								 pMem wird dabei mit delete gelscht !
//								 Das Schreiben beginnt bei iLen.
//					CFB_CREATE = Neuen Speicherbereich mit der Lnge 0 zum
//								 Schreiben erzeugen. pMem wird ignoriert.
//					CFB_FIXED  = Es wird in einem Speicher mit fixer Lnge
//								 geschrieben.
//	pConverter	: Ist der Zeiger auf die Konvertierungsfunktionen.
//				  Bei NULL erfollg keine Konvertierung.
//	Ergibt false bei einem Fehler
bool CFileBuffer::OpenMem(void *pMem,int iLen,int iMode,FileBufferConvert *pConverter)
{

	if(bOpenFlag)Close();


	iOpenMode = (iMode!=CFB_READ)? 1:0;


	switch(iMode)
		{
	case CFB_READ:	pProcWrite 	= 0;					// Speicher auslesen
					pProcRead	= MemRead;
					pProcBuff	= BufferNone;
					pBuffer		= (char*)pMem;
					iBufferEnd	= iLen;
					iBufferLen	= iLen;
					iBufferPos	= 0;
					break;

	case CFB_FIXED:	if(!pMem)iLen=0;					// Speicher mit fixer Gre

					bFixFlag	= true;
					pProcRead	= MemRead;
					pProcWrite 	= MemWrite;
					pProcBuff	= BufferNone;
					pBuffer		= (char*)pMem;
					iBufferEnd	= iLen;
					iBufferLen	= iLen;
					iBufferPos	= 0;
					break;

	case CFB_CREATE:    pMem=0;iLen=0;					// Speicher erzeugen
	case CFB_WRITE:	if(!pMem)
						{
						pProcBuff  = BufferGet;
						pBuffer	   = new char[4];
						pBuffer[0] = 0;
						iLen	   = 0;
						}
					else{
						pProcBuff  = BufferNone;
						pBuffer    = (char*)pMem;
						}

					pProcRead	= MemRead;
					pProcWrite 	= MemWrite;
					iBufferPos	= iLen;
					iBufferEnd	= iLen;
					iBufferLen	= iLen;
					bFixFlag	= false;
					break;

	default:		return false;
		}

	pProcClose 	= MemClose;
	pProcSeek  	= MemSeek;
	pFileHandle	= this;									// Dateihandle
	bEofFlag	= iOpenMode==CFB_READ && iLen<=0;
	bMemFlag	= true;
	bOpenFlag	= true;
	bStopFlag   = true;
	bWriteFlag	= false;								// Puffer schreiben
	iSeekAdd	= 0;


	if(!pConverter)return true;							// Eine Konvertierung einstellen
	if(!pConverter->pConv(this,pConverter,0))
		{
		Close();
		return false;
		}

	pProcConv = pConverter->pConv;


return true;
}

//*****************************************************************************
//*
//*     Open
//*
//*****************************************************************************
//	ffnet eine Datei als Stream
//	pName		: Ist der Dateinamen der Datei. Wenn der  Dateinamen die
//				  mit "res://" anfngt wird eine Resource geladen.
//	iMode		: Modus mit dem die Datei geffnet wird.
//					CFB_READ	Datei zum Lesen ffnen
//					CFB_WRITE	Datei zum Schreiben ffnen
//					CFB_CREATE	Datei neuzeugen
//	iSeekEnd	: Wenn dieser Wert ungleich Null ist, dann wird der Dateizeiger
//				  ans Ende der Datei versetzt.
//	Ergibt true wenn die Datei geffnet wurde, bzw. false bei einem Fehler.
bool CFileBuffer::Open(const char *pName,int iMode,int iSeekEnd)
{
char   *pType;
char	cTemp[64];
int		iNum;
int		iEnd;
int		i;



	if(bOpenFlag)Close();

	iEnd		= (iSeekEnd)? _O_APPEND:0;
	iOpenMode	= (iMode!=CFB_READ)?  1:0;
	iSeekAdd	=  0;

	pProcWrite	= (CFB_PROC_WRITE)write;					// Dateifunktionen einstellen
	pProcClose	= (CFB_PROC_CLOSE)close;					//		*
	pProcSeek	= (CFB_PROC_SEEK )lseek;					//		*
	pProcRead	= (CFB_PROC_READ )read;						//		*
	pProcBuff	= BufferGet;								//		*


	switch(iMode)											// Datei ffnen
		{
	case CFB_READ:	if(pName[0]=='R'  || pName[0]=='r')
					if(pName[1]=='E'  || pName[1]=='e')
					if(pName[2]=='S'  || pName[2]=='s')
					if(pName[3]==':')
					if(pName[4]=='\\' || pName[4]=='/')
					if(pName[5]=='\\' || pName[5]=='/')
						{
						pType  = 0;
						pName += 6;

						i=(int)strcspn(pName,"\\/");

						if(pName[i] && i<sizeof(cTemp)-1)	// Ist der Pfad der Typname ?
							{
							memcpy(cTemp,pName,i);
							cTemp[i] = 0;
							pName   += i+1;
							iNum     = atoi(cTemp);

							if( iNum!=0                )pType=(char*)MAKEINTRESOURCE(iNum);
							if(!stricmp(cTemp,"CURSOR"))pType=(char*)RT_CURSOR;
							if(!stricmp(cTemp,"BITMAP"))pType=(char*)RT_BITMAP;
							if(!stricmp(cTemp,"ICON"  ))pType=(char*)RT_ICON;
							if(!stricmp(cTemp,"HTML"  ))pType=(char*)RT_HTML;
							if(!pType                  )pType=cTemp;
							}

						   i=atoi(pName);
						if(i!=0)
							{
							i&=0xFFFF;
							pName=(const char*)i;
							}

						return OpenRes(pName,pType);
						}

					pProcWrite = NULL;
					i=open(pName,O_BINARY|O_RDONLY|iEnd);
					if(i==-1)return false;
					break;

	case CFB_WRITE:	i=open(pName,O_BINARY|O_RDWR|iEnd);
					if(i!=-1)break;

	case CFB_CREATE:i=open(pName,O_BINARY|O_RDWR|O_TRUNC|O_CREAT,S_IREAD|S_IWRITE);
					if(i!=-1)break;

					 chmod(pName,S_IREAD |S_IWRITE);
					i=open(pName,O_BINARY|O_RDWR|O_TRUNC|O_CREAT,S_IREAD|S_IWRITE );
					if(i!=-1)break;

	default:		return false;
					}


		pBuffer = new char[iBufferLen+4];				// Puffer Speicher allokieren
	if(!pBuffer)
		{
		pProcWrite	= NULL;								// Dateifunktionen rcksetzen
		pProcClose	= NULL;								//		*
		pProcSeek	= NULL;								//		*
		pProcBuff	= NULL;								//		*
		pProcRead	= NULL;								//		*

		close(i);

		return false;
		}

	iCloseSeek	= tell(i);								// Dateiposition fr's schlieen
	pFileHandle	= (void*)i;								// Dateihandle
	iBufferPos  = 0;									// Puffer Lese Position
	iBufferEnd  = 0;									// Puffer Byteanzahl
	bWriteFlag  = 0;									// Puffer schreiben
	bEofFlag	= false;
	bStopFlag   = false;
	bOpenFlag	= true;


return true;
}

bool CFileBuffer::Open(const wchar_t *pName,int iMode,int iSeekEnd)
{
wchar_t    *pType;
wchar_t		uTemp[64];
int			iNum;
int			iEnd;
int			i;



	if(bOpenFlag)Close();

	iEnd		= (iSeekEnd)? _O_APPEND:0;
	iOpenMode	= (iMode!=CFB_READ)? 1:0;
	iSeekAdd	=  0;

	switch(iMode)											// Datei ffnen
		{
	case CFB_READ:	if(pName[0]=='R'  || pName[0]=='r')
					if(pName[1]=='E'  || pName[1]=='e')
					if(pName[2]=='S'  || pName[2]=='s')
					if(pName[3]==':')
					if(pName[4]=='\\' || pName[4]=='/')
					if(pName[5]=='\\' || pName[5]=='/')
						{
						pType  = 0;
						pName += 6;

						i=(int)wcscspn(pName,L"\\/");

						if(pName[i] && i<sizeof(uTemp)-1)	// Ist der Pfad der Typname ?
							{
							memcpy(uTemp,pName,i);
							uTemp[i] = 0;
							pName   += i+1;
							iNum     = wtoi(uTemp);

							if( iNum!=0){ iNum&=0xFFFF;  pType=(wchar_t*)iNum;}
							if(!wcsicmp(uTemp,L"CURSOR"))pType=(wchar_t*)RT_CURSOR;
							if(!wcsicmp(uTemp,L"BITMAP"))pType=(wchar_t*)RT_BITMAP;
							if(!wcsicmp(uTemp,L"ICON"  ))pType=(wchar_t*)RT_ICON;
							if(!wcsicmp(uTemp,L"HTML"  ))pType=(wchar_t*)RT_HTML;
							if(!pType                   )pType=uTemp;
							}

						   i=wtoi(pName);
						if(i!=0)
							{
							i&=0xFFFF;
							pName=(wchar_t*)i;
							}

						return OpenRes(pName,pType);
						}

					i=wopen(pName,O_BINARY|O_RDONLY|iEnd);
					if(i==-1)return false;
					break;

	case CFB_WRITE:	i=wopen(pName,O_BINARY|O_RDWR|iEnd);
					if(i!=-1)break;

	case CFB_CREATE:i=wopen(pName,O_BINARY|O_RDWR|O_TRUNC|O_CREAT,S_IREAD|S_IWRITE);
					if(i!=-1)break;

					 wchmod(pName,S_IREAD |S_IWRITE);
					i=wopen(pName,O_BINARY|O_RDWR|O_TRUNC|O_CREAT,S_IREAD|S_IWRITE );
					if(i!=-1)break;

	default:		return false;
					}

		pBuffer = new char[iBufferLen+4];				// Puffer Speicher allokieren
	if(!pBuffer)
		{
		pProcWrite	= NULL;								// Dateifunktionen rcksetzen
		pProcClose	= NULL;								//		*
		pProcSeek	= NULL;								//		*
		pProcBuff	= NULL;								//		*
		pProcRead	= NULL;								//		*

		close(i);

		return false;
		}

	pProcWrite	= (CFB_PROC_WRITE)write;				// Dateifunktionen einstellen
	pProcClose	= (CFB_PROC_CLOSE)close;				//		*
	pProcSeek	= (CFB_PROC_SEEK )lseek;				//		*
	pProcRead	= (CFB_PROC_READ )read;					//		*
	pProcBuff	= BufferGet;							//		*


	pFileHandle	= (void*)i;								// Dateihandle
	iBufferPos  = 0;									// Puffer Lese Position
	iBufferEnd  = 0;									// Puffer Byteanzahl
	bWriteFlag  = 0;									// Puffer schreiben
	bEofFlag	= false;
	bStopFlag   = false;
	bOpenFlag	= true;


return true;
}

//*****************************************************************************
//*
//*     Close
//*
//*****************************************************************************
//	Schliet den gerade geffneten Stream.
//	Ergibt true wenn der Stream geschlossen wurde bzw. false bei einem Fehler.
bool CFileBuffer::Close()
{
int		i;

	if(!bOpenFlag)return false;					// Ist der Stream geffnet
	if( iOpenMode)Flush();						// Puffer leeren
	if( pProcConv)pProcConv(this,0,1);			// Konvertierung abbauen


	if(!bMemFlag)
		{
		if(pBuffer){delete pBuffer;pBuffer=0;}	// Pufferspeicher freigeben
		}

	i = -1;

	if(iCloseSeek>=0 && pProcSeek)				// Dateiposition wieder herstellen
		{
		pProcSeek(pFileHandle,iCloseSeek,SEEK_SET);
		}

	if(pProcClose)								// Datei ber Funktion schlieen
			i=pProcClose(pFileHandle);
	else	i=0;


	bEofFlag	= true;
	bMemFlag   	= false;
	bOpenFlag  	= false;
	pFileHandle = (void*)-1;
	iBufferLen 	= CFB_BUFFER_SIZE;				// Puffer Lnge
	iBufferPos 	= CFB_BUFFER_SIZE;				// Puffer Lese Position
	iBufferEnd 	= 0;							// Puffer Byteanzahl
	pProcClose 	= 0;
	pProcWrite 	= 0;
	pProcClose 	= 0;
	pProcSeek  	= 0;
	pProcRead  	= 0;
	pProcBuff	= BufferNull;



return i!=-1;
}

//*****************************************************************************
//*
//*     ResClose
//*
//*****************************************************************************
//	Schliest eine Resource
//	pHandle		: Zeiger auf die CFileBuffer Klasse
//	Ergibt 0 wenn alles ok war
int	_cdecl CFileBuffer::ResClose(void *pHandle)
{
CFileBuffer	*pThis;


	MemClose(pHandle);

	pThis=(CFileBuffer*)pHandle;

	UnlockResource(pThis->hResHg );
	DeleteObject  (pThis->hResSrc);



return 0;
}

//*****************************************************************************
//*
//*     BufferFill
//*
//*****************************************************************************
//	Fllt den Puffer wieder auf
//	iMin	: Ist die minimal bentigte Puffergre. Diese darf nicht
//			  grer sein all die eingestellte minimale Pufferung.
//	Ergibt die Anzahl der Bytes im Puffer bzw. -1 bei einem Fehler.
int CFileBuffer::BufferFill(int iMin)
{
int 	iDiff;
int 	iOff;
int		i,j;



	if(!pProcRead)return -1;


	if((unsigned)iMin>(unsigned)iBufferMin)			// Bentigte Gre prfen
		{
		iMin = iBufferMin;
		}


	if(bWriteFlag && iBufferPos>0 && !bMemFlag)		// Wurde der Puffer als Schreibuffer benutzt
		{
		iDiff = iBufferEnd-iBufferPos;

		if(!Flush())return -1;

		if(iDiff>0 && pProcSeek)
			{
			pProcSeek(pFileHandle,-iDiff,SEEK_CUR);
			}
		}


	   iDiff=iBufferEnd-iBufferPos;
	if(iDiff>=iMin)									// Sind genug Bytes im Puffer
		{
		return iDiff;
		}



	if(bMemFlag)									// Nur ein Speicherblock
		{
		iBufferEnd  = 0;
		iBufferPos  = 0;
		bEofFlag    = true;

		return iDiff;
		}

	if(iOpenMode)									// Im Schreibmodus nur 512 Byte nachladen
		{
		if(iBufferLen<512 || iMin>512)
				j = iBufferLen;
		else	j = 512;

		iOff	    = 0;
		iBufferEnd  = 0;
		iBufferPos  = 0;
		}
	else if(iBufferPos>iBufferLen-iBufferMin)		// Untere Bytes verwerfen
		{
		j = iBufferLen-iBufferMin;

		memmove(pBuffer,pBuffer+j,iBufferEnd-j);

		iBufferPos -= j;
		iBufferEnd -= j;
		iOff	    = iBufferMin;
		j			= iBufferMin;
		}
	else if(bStopFlag)								// Ende der Datei erreicht
		{
		return iBufferEnd-iBufferPos;
		}
	else{											// Puffer oben nachfllen
		iOff	    = iBufferEnd;
		j			= iBufferLen;
		j		   -= iBufferEnd;
		}


	i = pProcRead(pFileHandle,pBuffer+iOff,j);		// Puffer laden

	if(i== 0)bStopFlag=true;
	if(i < j)bEofFlag=true;
	if(i==-1)return -1;

	iBufferEnd += i;
	iBufferOff += i;

	*(int*)(pBuffer+iBufferEnd) = 0;				// Null ans Ende setzen



return iBufferEnd-iBufferPos;
}

//*****************************************************************************
//*
//*     BufferFill
//*
//*****************************************************************************
//	Fllt den Puffer wieder auf.
//	iAdd	: Ist die Zahl an Bytes die im Puffer nicht gebraucht werden.
//	pMem	: Hier wird der Zeiger auf den Puffer abgelegt.
//	iMin	: Ist die minimal bentigte Puffergre. Diese darf nicht
//			  grer sein all die eingestellte minimale Pufferung.
//	Ergibt die Anzahl der Bytes im Puffer bzw. -1 bei einem Fehler.
int CFileBuffer::BufferFill(int iAdd,void *pMem,int iMin)
{
char   *pPtr;
int		iLen;


	if(iAdd>0)Seek(iAdd,SEEK_CUR);

	iLen = BufferFill(iMin);
	pPtr = Buffer();

	if(!pPtr) iLen = -1;
	else	  pPtr+= iBufferPos;
	*((void**)pMem)= pPtr;


return iLen;
}


//*****************************************************************************
//*
//*     Read
//*
//*****************************************************************************
//	Liest Daten aus den Stream aus
//	pMem	: Zeiger auf den Puffer wo die Daten gspeichert werden.
//	iLen	: Anzahl der Bytes die gelesen werden sollen
//	Ergibt die Anzahl der geladenen Bytes bzw. -1 bei einem Fehler
int CFileBuffer::Read(char *pMem,int iLen)
{
int	i,j,iDiff,iNum=0;



	if(!pProcRead)return -1;
	if( iLen<=  0)return  0;


	if(bWriteFlag && iBufferPos>0 && !bMemFlag)		// Wurde der Puffer als Schreibuffer benutzt
		{
		iDiff = iBufferEnd-iBufferPos;

		if(!Flush())return -1;

		if(iDiff>0 && pProcSeek)
			{
			pProcSeek(pFileHandle,-iDiff,SEEK_CUR);
			}
		}


	   iDiff=iBufferEnd-iBufferPos;
	if(iDiff<iLen)									// Puffer aufladen
		{
		memcpy(pMem,pBuffer+iBufferPos,iDiff);		// Puffer leeren

		iBufferEnd = 0;
		iBufferPos = 0;

		pMem += iDiff;
		iNum += iDiff;
		iLen -= iDiff;

		if(bMemFlag)
			{
			bEofFlag = true;
			return iNum;
			}

		if(iLen>iBufferLen)							// Zu groer Block ?
			{
			j = iLen%iBufferLen;
			i = pProcRead(pFileHandle,pMem,iLen-j);

			if(i!=iLen-j)bEofFlag=true;
			if(i==-1)return -1;

			iLen	   -= i;
			iNum 	   += i;
			pMem 	   += i;
			iBufferOff += i;
			}

		if(iOpenMode)								// Im Schreibmodus nur 512 Byte nachladen
			{
			if(iBufferLen<512 || iLen>=128)
					j = iBufferLen;
			else	j = 512;
			}
		else{
			j = iBufferLen;
			}

		i = pProcRead(pFileHandle,pBuffer,j);		// Puffer laden

		if(i<iLen)
			{
			bEofFlag = true;
			iLen	 = i;
			}

		if(i== 0)bStopFlag=true;
		if(i==-1)return -1;

		iBufferEnd	 = i;
		iBufferOff	+= i;

		*(int*)(pBuffer+iBufferEnd) = 0;			// Null ans Ende setzen
		}


	memcpy(pMem,pBuffer+iBufferPos,iLen);			// Puffer kopieren
	iBufferPos += iLen;



return iNum+iLen;
}

//*****************************************************************************
//*
//*     ReadByte
//*
//*****************************************************************************
//	Liest 1 Byte aus dem Stream ein.
//	Ergibt den Wert des Bytes
int CFileBuffer::ReadByte()
{
unsigned char ucVal;


	if(iOpenMode)return -1;

	if(iBufferEnd-iBufferPos<1)
		{
		Read((char*)&ucVal,1);
		}
	else{
		ucVal=pBuffer[iBufferPos];
		iBufferPos++;
		}


return ucVal;
}

//*****************************************************************************
//*
//*     ReadWord
//*
//*****************************************************************************
//	Liest 2 Byte aus dem Stream ein.
//	Ergibt den Wert der Bytes
int CFileBuffer::ReadWord()
{
unsigned short usVal;


	if(iOpenMode)return -1;

	if(iBufferEnd-iBufferPos<2)
		{
		Read((char*)&usVal,2);
		}
	else{
		usVal=*(short*)(pBuffer+iBufferPos);
		iBufferPos+=2;
		}


return usVal;
}

//*****************************************************************************
//*
//*     ReadDWord
//*
//*****************************************************************************
//	Liest 4 Byte aus dem Stream ein.
//	Ergibt den Wert der Bytes
int CFileBuffer::ReadDWord()
{
unsigned int uiVal;


	if(iOpenMode)return -1;

	if(iBufferEnd-iBufferPos<4)
		{
		Read((char*)&uiVal,4);
		}
	else{
		uiVal=*(int*)(pBuffer+iBufferPos);
		iBufferPos+=4;
		}


return uiVal;
}

//*****************************************************************************
//*
//*     ReadQWord
//*
//*****************************************************************************
//	Liest 8 Byte aus dem Stream ein.
//	Ergibt den Wert der Bytes
ifb64 CFileBuffer::ReadQWord()
{
ifb64	iVal;


	if(iOpenMode)return -1;

	if(iBufferEnd-iBufferPos<8)
		{
		Read((char*)&iVal,8);
		}
	else{
		iVal=*(ifb64*)(pBuffer+iBufferPos);
		iBufferPos+=8;
		}


return iVal;
}

//*****************************************************************************
//*
//*		ReadLine
//*
//*****************************************************************************
//	Liest eine ganze Textzeile aus dem Stream ein.
//	Entfernt '\n' und '\r' am Ende der Zeile.
//	pMem	: Ist der Zeiger auf den Puffer wo die Zeile gespeichert wird
//	iMax	: Ist die Gre des Puffers in Zeichen
//	Ergibt die Anzahl der gespeicherten Zeichen im Zeilenpuffer.
//	bzw. -1 bei einem Fehler
int CFileBuffer::ReadLine(char *pMem,int iMax)
{
char   *pText;
int		iDiff;
int		iPos;
int		iNum;
int		i;


	if(iMax<=0)return -1;

	if(!bMemFlag)										// Puffer fllen
		{
		   iDiff=iBufferEnd-iBufferPos;
		if(iDiff<iMax+2)								// Puffer aufladen
			{
			memmove(pBuffer,pBuffer+iBufferPos,iDiff);	// Puffer leeren
			iBufferEnd = iDiff;
			iBufferPos = 0;

			iNum = iBufferLen-iBufferEnd;				// Puffer laden
			i    = pProcRead(pFileHandle,pBuffer+iBufferEnd,iNum);

			if(i==-1)
				{
				pMem[0]=0;
				return -1;
				}

			iBufferOff += i;
			iBufferEnd += i;

			*(int*)(pBuffer+iBufferEnd) = 0;			// Null ans Ende setzen

			if(iBufferEnd<=iBufferPos)
				{
				bEofFlag=true;
				}
			}
		}


	   iNum = iBufferEnd-iBufferPos;
	if(iNum<=0){pMem[0]=0;return -1;}
	if(iNum>=iMax)iNum=iMax-1;

	pText = pBuffer+iBufferPos;

	for(iPos=0;iPos<iNum;iPos++)						// Suche Zeilenende
		{
		if(pText[iPos]=='\r')break;
		if(pText[iPos]=='\n')break;
		}

	memcpy(pMem,pText,iPos);

	pMem[iPos]  = 0;									// Nullzeichen anhngen
	iBufferPos += iPos;


	if(pBuffer[iBufferPos]=='\r')iBufferPos++;
	if(pBuffer[iBufferPos]=='\n')iBufferPos++;



return iPos;
}

int CFileBuffer::ReadLine(unsigned short *pMem,int iMax)
{
unsigned short *pText;
int				iDiff;
int				iPos;
int				iNum;
int				i;


	if(iMax<=0)return -1;

	if(!bMemFlag)										// Puffer fllen
		{
		   iDiff=(iBufferEnd-iBufferPos)/2;
		if(iDiff<iMax+2)								// Puffer aufladen
			{
			memmove(pBuffer,pBuffer+iBufferPos,iDiff);	// Puffer leeren
			iBufferEnd = iDiff;
			iBufferPos = 0;

			iNum = iBufferLen-iBufferEnd;				// Puffer laden
			i    = pProcRead(pFileHandle,pBuffer+iBufferEnd,iNum);

			if(i==-1)
				{
				pMem[0]=0;
				return -1;
				}

			iBufferOff += i;
			iBufferEnd += i;

			*(int*)(pBuffer+iBufferEnd) = 0;			// Null ans Ende setzen

			if(iBufferEnd<=iBufferPos)
				{
				bEofFlag=true;
				}
			}
		}


	   iNum = (iBufferEnd-iBufferPos)/2;
	if(iNum<=0){pMem[0]=0;return -1;}
	if(iNum>=iMax)iNum=iMax-1;

	pText = (unsigned short*)(pBuffer+iBufferPos);

	for(iPos=0;iPos<iNum;iPos++)						// Suche Zeilenende
		{
		if(pText[iPos]=='\r')break;
		if(pText[iPos]=='\n')break;
		}

	memcpy(pMem,pText,iPos*2);

	pMem[iPos]  = 0;									// Nullzeichen anhngen
	iBufferPos += iPos;


	if(pBuffer[iBufferPos]=='\r')iBufferPos++;
	if(pBuffer[iBufferPos]=='\n')iBufferPos++;



return iPos;
}

//*****************************************************************************
//*
//*		Flush
//*
//*****************************************************************************
//	Speichert vorhanden Pufferdaten
//	Ergibt true wenn die Daten gespeichert wurden, bzw. false bei einem Fehler.
bool CFileBuffer::Flush()
{
int		i;

	if(!pProcWrite)return false;						// Schreiberlaubnis vorhanden ?
	if( bMemFlag  )return false;						// Nur Speicher Funktionen ?
	if(!bWriteFlag)return true;							// Ist etwas verndert ?
		bWriteFlag=0;

	   i=pProcWrite(pFileHandle,pBuffer,iBufferPos);	// Puffer schreiben
	if(i==-1)return false;								// Fehler aufgetreten ?

	bStopFlag  = false;
	iBufferPos = 0;
	iBufferEnd = 0;


return true;
}

//*****************************************************************************
//*
//* 	Write
//*
//*****************************************************************************
//	Schreibt Daten in den Stream
//	pMem	: Zeiger auf die Daten die gespeichert werden sollen.
//	iLen	: Anzahl der Bytes die gespeichert werden sollen.
//	Ergibt die Anzahl der gespeicherten Bytes bzw. -1 bei einem Fehler
int CFileBuffer::Write(const char *pMem,int iLen)
{
char   *pNew;
char   *pOld;
int		iSize;
int		iDiff;
int		iNum;
int		i;



	if(!pProcWrite)return -1;
	if( iLen <=  0)return  0;

	if(!bWriteFlag && iBufferPos>0 && !bMemFlag)	// Wurde der Puffer als Lesebuffer benutzt
		{
		   iDiff=iBufferEnd-iBufferPos;
		if(iDiff>0 && pProcSeek)
			{
			pProcSeek(pFileHandle,-iDiff,SEEK_CUR);
			}

		iBufferPos = 0;
		iBufferEnd = 0;
		}


	iNum = 0;

	   iDiff=iBufferLen-iBufferPos;
	if(iDiff<iLen)									// Puffer aufladen
		{
		if(iDiff>0)
			{
			memcpy(pBuffer+iBufferPos,pMem,iDiff);

			iBufferPos += iDiff;
			bWriteFlag  = true;

			iLen -= iDiff;
			pMem += iDiff;
			iNum += iDiff;

			if(iBufferPos>iBufferEnd)iBufferEnd=iBufferPos;
			}

		if(bMemFlag)								// Puffer vergrern
			{
			if(bFixFlag)return -1;

			i	  = iBufferPos+iLen;
			iSize = i+i/2;

			if(iSize<CFB_BUFFER_SIZE)
				{
				iSize=CFB_BUFFER_SIZE;
				}


				pNew = new char[iSize+4];
			if(!pNew)return -1;

			pOld = pBuffer;
			memcpy(pNew,pBuffer,iBufferEnd);

			if(pProcBuff==BufferNone)
				{
				pProcBuff = BufferGet;
				}

			pBuffer    = pNew;
			iBufferLen = iSize;

			delete pOld;
			}
		else{
			if(!Flush())return -1;
			if(iLen>iBufferLen)						// Zu groer Block ?
				{
				i=pProcWrite(pFileHandle,pMem,iLen);
				if(i==-1)return -1;					// Fehler
				bStopFlag = false;
				return iNum+i;
				}
			}
		}


	memcpy(pBuffer+iBufferPos,pMem,iLen);

	   iBufferPos+=iLen;
	if(iBufferPos>iBufferEnd)iBufferEnd=iBufferPos;

	bWriteFlag =true;



return iNum+iLen;
}

//*****************************************************************************
//*
//*    WriteByte
//*
//*****************************************************************************
// Schreibt 1 Byte in den Stream
//	iVal	: Ist der Wert der gespeichert werden soll
void CFileBuffer::WriteByte(int iVal)
{

	if(iBufferLen-iBufferPos<1)							// Puffer aufladen
		{
		Write((char*)&iVal,1);
		return;
		}

	pBuffer[iBufferPos] = (char)iVal;

	   iBufferPos+=1;
	if(iBufferPos>iBufferEnd)iBufferEnd=iBufferPos;

	bWriteFlag =true;

}

//*****************************************************************************
//*
//*    WriteWord
//*
//*****************************************************************************
//	Schreibt 2 Byte in den Stream
//	iVal	: Ist der Wert der gespeichert werden soll
void CFileBuffer::WriteWord(int iVal)
{

	if(iBufferLen-iBufferPos<2)							// Puffer aufladen
		{
		Write((char*)&iVal,2);
		return;
		}

	*(short*)(pBuffer+iBufferPos) = (short)iVal;
	   iBufferPos+=2;
	if(iBufferPos>iBufferEnd)iBufferEnd=iBufferPos;

	bWriteFlag =true;

}

//*****************************************************************************
//*
//*    WriteDWord
//*
//*****************************************************************************
//	Schreibt 4 Byte in den Stream
void CFileBuffer::WriteDWord(int iVal)
{

	if(iBufferLen-iBufferPos<4)							// Puffer aufladen
		{
		Write((char*)&iVal,4);
		return;
		}

	*(int*)(pBuffer+iBufferPos)=iVal;

	   iBufferPos+=4;
	if(iBufferPos>iBufferEnd)iBufferEnd=iBufferPos;

	bWriteFlag =true;

}


//*****************************************************************************
//*
//*    WriteQWord
//*
//*****************************************************************************
//	Schreibt 8 Byte in den Stream
//	iVal	: Ist der Wert der gespeichert werden soll
void CFileBuffer::WriteQWord(ifb64 iVal)
{

	if(iBufferLen-iBufferPos<8)							// Puffer aufladen
		{
		Write((char*)&iVal,8);
		return;
		}

	*(ifb64*)(pBuffer+iBufferPos)=iVal;

	   iBufferPos+=8;
	if(iBufferPos>iBufferEnd)iBufferEnd=iBufferPos;

	bWriteFlag =true;

}

//*****************************************************************************
//*
//*		WriteFormat
//*
//*****************************************************************************
//	Speichert Bytes in einem bestimmen Format (siehe printf)
//	pFormat		: Ist der String der das Format
//	Ergibt die gespeicherten Bytes bzw. -1 bei einem Fehler
int __cdecl	CFileBuffer::WriteFormat(const char *pFormat, ...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = pprintfva(this,WritePrintf,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

int __cdecl	CFileBuffer::WriteFormat(const wchar_t *pFormat, ...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = pprintfvw(this,WritePrintf,pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

//*****************************************************************************
//*
//*		Tell
//*
//*****************************************************************************
//	Liest die Dateiposition aus
int CFileBuffer::Tell()
{

	if(!bOpenFlag )return -1;
	if( bMemFlag  )return iBufferPos;
	if(!pProcSeek )return (iOpenMode)? -1:iBufferOff-iBufferEnd+iBufferPos;
	if( bWriteFlag)return pProcSeek(pFileHandle,0,SEEK_CUR)+iBufferPos;
				   return pProcSeek(pFileHandle,0,SEEK_CUR)-iBufferEnd+iBufferPos;

}

//*****************************************************************************
//*
//*		Seek
//*
//*****************************************************************************
//	Verschiebt den Lese/Schreibzeiger im Stream
//	iPos		: Ist die Position zu der Verschoben werden soll
//	iMode		: Art der Dateizeiger Vernderung
//					SEEK_SET = von Dateibegin berechnet
//					SEEK_END = von Dateiende  berechnet
//					SEEK_CUR = von aktueller Position berechnet
//	Ergibt die alte Dateiposition
int CFileBuffer::Seek(int iPos,int iMode)
{
int		i,j,k;


	if(!pProcSeek)return -1;

	i = Tell();

	if(bMemFlag)
		{
		switch(iMode)
			{
		case SEEK_CUR:	iBufferPos += iPos;			break;
		case SEEK_END:	iBufferPos  = iBufferEnd;	break;
		default:		iBufferPos  = iPos;
			}

		return i;
		}

	if(iOpenMode)										// Ungespeicherte Bytes ablegen
		{
		if(bWriteFlag)Flush();
		iBufferPos = 0;
		iBufferEnd = 0;
		}


	switch(iMode)
		{
	case SEEK_END:	k  = pProcSeek(pFileHandle,iPos,SEEK_END);
					j  = i-iBufferPos;
														// Ist die neue Position im Puffer
					if(k!=-1 && k>=j && k<j+iBufferEnd)
						{
						iBufferPos=k-j;
						return i;
						}

					iBufferEnd = 0;
					iBufferPos = 0;
					iPos       = k;

					break;

	case SEEK_CUR:	iPos+=i;							// Relativer Verschub

	default:		if(i==-1)							// Absolute Position
						{
						iBufferEnd = 0;
						iBufferPos = 0;

						pProcSeek(pFileHandle,iPos,SEEK_SET);

						bStopFlag = false;

						return i;
						}

					j = i-iBufferPos;
					k = j+iBufferEnd;

					if(iPos>=j && iPos<k)				// Ist die neue Position im Puffer
						{
						iBufferPos=iPos-j;
						return i;
						}

					iBufferEnd = 0;
					iBufferPos = 0;

					pProcSeek(pFileHandle,iPos,SEEK_SET);

					break;
		}



	if(iPos<i)
		{
		bStopFlag = false;
		}


return i;
}

//*****************************************************************************
//*
//*		GetRealLength
//*
//*****************************************************************************
//	Ergibt die wikliche Datei/Speicherlnge bzw. -1 bei einem Fehler. Ist ein
//	Konverter  definiert wird die Lnge des nicht konvertertieren Bereiches
//	zurckgegeben.
int CFileBuffer::GetRealLength()
{
int	iLen,iPos;


	if(!pProcConv)return GetLength();

	if(bMemFlag)
		{
		Buffer();
		return iConvPos;
		}


	if(!pOldSeek)return -1;
	if(iOpenMode)Flush();


	iPos=pOldSeek(pFileHandle,0   ,SEEK_CUR);
	iLen=pOldSeek(pFileHandle,0   ,SEEK_END);
		 pOldSeek(pFileHandle,iPos,SEEK_SET);



return (iLen>iPos)? iLen:iPos;
}

//*****************************************************************************
//*
//*		GetLength
//*
//*****************************************************************************
//	Ergibt die Datei/Speicherlnge bzw. -1 bei einem Fehler. Ist ein Konverter
//	definiert wird die Lnge des konvertertieren Bereiches zurckgegeben.
int CFileBuffer::GetLength()
{
int	iLen,iPos;


	if(!pProcSeek)return -1;

	if(bMemFlag)return iBufferEnd;

	iPos=pProcSeek(pFileHandle,0   ,SEEK_CUR);
	iLen=pProcSeek(pFileHandle,0   ,SEEK_END);
		 pProcSeek(pFileHandle,iPos,SEEK_SET);

	if(!iOpenMode)return iLen;

	iPos=Tell();



return (iLen>iPos)? iLen:iPos;
}

//*****************************************************************************
//*
//*		BufferLen
//*
//*****************************************************************************
//	Stellt die Pufferlnge ein. Das funktioniert nur wenn keine Datei offen ist.
//	iLen	: Ist die neue Pufferlnge.
//	iMin	: Ist die minimal bentigte Puffergre, bzw. der nachlade Block.
//			  Diese darf nicht grer sein all die eingestellte halbe Pufferung.
//	Ergibt die alte Pufferlnge bzw. -1 bei einem Fehler
int CFileBuffer::BufferLen(int iLen,int iMin)
{
int iOld;


	if(bOpenFlag)return -1;



	if(iLen<=0)
		{
		iLen = CFB_BUFFER_SIZE;
		}
	else if(iLen<512)
		{
		iLen = 512;
		}

	if(iMin<=0)
		{
		   iMin = iLen/8;
		if(iMin<0x1000)
			{
			iMin = iLen/2;
			}
		}

	if(iMin>iLen/2)
		{
		iMin = iLen/2;
		}


	iOld		= iBufferLen;
	iBufferLen	= iLen;
	iBufferMin	= iMin;


return iOld;
}

//*****************************************************************************
//*
//*		WideWrite
//*
//*****************************************************************************
//	Schreibt Daten im UTF16 Format in eine Datei.
//	pHandle		: Ist das Datei-Handle
//	pMem		: Ist der Zeiger auf die zu speichernten Daten im ANSI-Format
//				  Diese Daten werden von ANSI nach UFT16 konvertiert.
//	iCount		: Ist die Anzahl der Zeichen in den Ansi-Daten
//	Ergibt die Anzahl der gespeicherten UTF16 Zeichen bzw. -1 bei einem Fehler
int	_cdecl CFileBuffer::WideWrite(void *pHandle,const void *pMem,int iCount)
{
CFileBuffer	   *pThis=(CFileBuffer *)pHandle;
unsigned char  *pPtr=(unsigned char*)pMem;
unsigned char	ucTemp[1024];
int				iPos,i,j;


	for(iPos=0,i=0;iPos<iCount;)
		{
		ucTemp[i] = pPtr[iPos];	i++;
		ucTemp[i] = 0;			i++;
		iPos++;

		if(i>=sizeof(ucTemp))
			{
			j=pThis->pOldWrite(pThis->pOldHandle,ucTemp,i);
			if(j!=i)return (j<0)? j:iPos+((j-i)>>1);
			i=0;
			}
		}

	if(i)j=pThis->pOldWrite(pThis->pOldHandle,ucTemp,i);
	else j=0;


return (j<0)? j:iPos+((j-i)>>1);
}


//*****************************************************************************
//*
//*		WideRead
//*
//*****************************************************************************
//	Liest Daten vom UTF16 Format aus einer Datei.
//	pHandle		: Ist das Datei-Handle
//	pMem		: Ist der Zeiger auf den Lesepuffer. (im ANSI-Format)
//				  Diese geladenen Daten werden von UFT16 nach ANSI konvertiert.
//	iCount		: Ist die Gre des der Lesepuffers in Byte.
//	Ergibt die Anzahl der gelesenen UTF16 Zeichen bzw. -1 bei einem Fehler
int	_cdecl CFileBuffer::WideRead(void *pHandle,void *pMem,int iCount)
{
CFileBuffer	   *pThis=(CFileBuffer *)pHandle;
unsigned char  *pPtr=(unsigned char*)pMem;
unsigned char	ucTemp[1024+1];
int				iPos,i,j;


	for(iPos=0,i=0,j=0;iPos<iCount;)
		{
		if(i>=j)
			{
			if(pThis->pOldSeek)							// Auf ein gerades Offset
				{
				   j =        pThis->pOldSeek(pThis->pOldHandle,0,SEEK_CUR);
				if(j>0 && j&1)pThis->pOldSeek(pThis->pOldHandle,1,SEEK_CUR);
				}

			   j=(iCount-iPos)*2;
			if(j>sizeof(ucTemp)-1)j=sizeof(ucTemp)-1;	// Daten lesen
			   j=pThis->pOldRead(pThis->pOldHandle,ucTemp,j);
			if(j <0)return -1;
			if(j==0)return iPos;

			ucTemp[j]=0;
			i=0;
			}

		if(ucTemp[i+1])									// Zeichen ober 255
			{
			pPtr[iPos]=0xFF;
			}
		else{
			pPtr[iPos]=ucTemp[i];
			}

		iPos++;
		i+=2;
		}


return -1;
}


//*****************************************************************************
//*
//*		WideSeek
//*
//*****************************************************************************
//	Seek Funktion fr Unicode-Streams
//	pHandle		: Ist das Datei-Handle
//	iPos		: Ist die Position zu der Verschoben werden soll (mal 0.5)
//	iMode		: Art der Dateizeiger Vernderung
//					SEEK_SET = von Dateibegin berechnet
//					SEEK_END = von Dateiende  berechnet
//					SEEK_CUR = von aktueller Position berechnet
//	Ergibt die Dateiposition umgewandelt auf die Hlfte.
int	_cdecl CFileBuffer::WideSeek(void *pHandle,int iPos,int iMode)
{
CFileBuffer	   *pThis=(CFileBuffer*)pHandle;
int				iRet;


	if(!pThis->pOldSeek)return -1;


	   iRet=pThis->pOldSeek(pThis->pOldHandle,iPos*2,iMode);
	if(iRet==-1)return -1;


return iRet/2;
}


//*****************************************************************************
//*
//*		WideBuffer
//*
//*****************************************************************************
//	Holte den Puffer fr das Speicherholen bei der UTF16->ANSI Konvertierung.
//	pClass		: Ist der Zeiger auf die Klasse
//	Ergibt einen Zeiger auf die Daten im UTF16 Format
char *_cdecl CFileBuffer::WideBuffer(void *pClass)
{
char		   *pOld;
char		   *pNew;
CFileBuffer	   *pThis=(CFileBuffer*)pClass;
int				iBytes;
int				iSize;
int				iPos;
int				iEnd;



	pOld = pThis->pOldBuffer;

	if(pThis->iConvPos/2>=pThis->iBufferEnd)return pOld;

	iPos  = pThis->iConvPos/2;
	iEnd  = pThis->iBufferEnd;
	iSize = iEnd*2;

	if(iSize > pThis->iConvLen)							// Den Puffer vergrern
		{
		if(pThis->bFixFlag)
			{
			iEnd = pThis->iConvLen/2;
			}
		else{
			iBytes	= iSize+iSize/2+1024;
			pNew	= new char [iBytes+4];

			if(!pNew)return pOld;

			memcpy(pNew,pOld,pThis->iConvPos);
			pNew[iSize+1] = 0;
			pNew[iSize  ] = 0;

			pThis->iConvLen   = iBytes;
			pThis->pOldBuffer = pNew;
			delete pOld;
			pOld = pNew;
			}
		}



	for(;iPos<iEnd;iPos++)								// ANSI nach UTF16
		{
		pOld[iPos*2  ] = pThis->pBuffer[iPos];
		pOld[iPos*2+1] = 0;
		}

	pOld[iPos*2  ] = 0;
	pOld[iPos*2+1] = 0;

	pThis->iConvPos = iPos*2;




return pOld;
}

//*****************************************************************************
//*
//*		WideConv
//*
//*****************************************************************************
//	Stellt die Konvertierung von UTF16 (Datei) auf ANSI (Speicher) ein.
//	pClass		: Ist der Zeiger auf die Klasse
//	pData			0 = FileBufferConvert-Zeiger fr Konvertierung
//	iMode		: Ist die Art des Aufrufes
//					0 = Einstellen der Konvertierung
//					1 = Schlieen des Streams
//	Ergibt 1 wenn alles ok sonst 0
int _cdecl CFileBuffer::WideConv(void *pClass,void *pData,int iMode)
{
CFileBuffer		   *pThis=(CFileBuffer*)pClass;
FileBufferConvert  *pConv;
int					iLen;
int					iPos;


	switch(iMode)
		{
	case 0:												// Einstelle der Konvertierung

			pThis->pOldBuffer	= pThis->pBuffer;
			pThis->pOldHandle	= pThis->pFileHandle;
			pThis->pOldWrite	= pThis->pProcWrite;
			pThis->pOldSeek		= pThis->pProcSeek;
			pThis->pOldRead		= pThis->pProcRead;
			pThis->pFileHandle	= pThis;


			if(pThis->bMemFlag)							// Nur Speicherzugriff
				{
				iLen				= pThis->iBufferLen/2;
				pThis->iConvPos		= pThis->iBufferLen;
				pThis->iConvLen		= pThis->iBufferEnd;
				pThis->pBuffer		= new char [iLen+4];

				if(!pThis->pBuffer)
					{
					return 1;
					}

				if(!pThis->iOpenMode)
					{
					pThis->iBufferEnd = iLen;
					pThis->iBufferLen = iLen;
					}
				else{
					iLen			= 0;
					pThis->iConvPos = 0;

					if(pThis->iBufferPos>0)				// Sind schon Daten Puffer
						{
						iLen = pThis->iBufferPos/2;
						pThis->iBufferPos	= iLen;
						}
					}

				for(iPos=0;iPos<iLen;iPos++)			// UTF16 auf ANSI konvertieren
					{
					if(pThis->pOldBuffer[iPos*2+1])
							pThis->pBuffer[iPos] = '\xFF';
					else	pThis->pBuffer[iPos] = pThis->pOldBuffer[iPos*2];
					}

				pThis->pBuffer[iLen  ]	= 0;
				pThis->pBuffer[iLen+1]	= 0;
				pThis->iConvPos			= iLen*2;
				pThis->pProcBuff		= WideBuffer;
				}
			else{										// Zugriff auf eine Datei
				pConv = (FileBufferConvert*)pData;

				pThis->pProcWrite = pConv->pWrite;
				pThis->pProcSeek  = pConv->pSeek;
				pThis->pProcRead  = pConv->pRead;
				}

			return 1;

	case 1:	if(pThis->bMemFlag)							// Schlieen des Streams
				{
				if(pThis->pBuffer)delete pThis->pBuffer;
				   pThis->pBuffer      = pThis->pOldBuffer;
				}

			pThis->pFileHandle	= pThis->pOldHandle;

			return 1;

		}


return 0;
}

//*****************************************************************************
//*
//*		AnsiWrite
//*
//*****************************************************************************
//	Schreibt Daten im UTF16 Format in eine Datei.
//	pHandle		: Ist das Datei-Handle
//	pMem		: Ist der Zeiger auf die zu speichernten Daten im ANSI-Format
//				  Diese Daten werden von ANSI nach UFT16 konvertiert.
//	iCount		: Ist die Anzahl der Zeichen in den Ansi-Daten
//	Ergibt die Anzahl der gespeicherten UTF16 Zeichen bzw. -1 bei einem Fehler
int	_cdecl CFileBuffer::AnsiWrite(void *pHandle,const void *pMem,int iCount)
{
CFileBuffer	   *pThis=(CFileBuffer  *)pHandle;
unsigned char  *pPtr =(unsigned char*)pMem;
unsigned char	ucTemp[1024];
int				iOut,iLen,iNum,iPos,iAdd=0;



	if(iCount <0)return -1;
	if(iCount==0)return  0;


	if(pThis->iSeekAdd)								// Dateizeiger auf ungerader Position
		{
		if(pThis->iCheckWr)							// War das letzte Zeichen ber 255
		if(pThis->pOldSeek && pPtr[0])
			{
			   pThis->pOldSeek (pThis->pOldHandle,-1,SEEK_CUR);
			if(pThis->pOldWrite(pThis->pOldHandle,"\xFF",1)<0)return -1;
			}

		pThis->iSeekAdd=0;
		iCount--;
		iAdd=1;
		pPtr++;
		}

	pThis->iCheckWr = 0;

	for(iPos=0,iLen=0,iOut=0;iPos<iCount;)
		{
		ucTemp[iOut] = pPtr[iPos];

		iPos++;

		if(pPtr[iPos])								// Ist das Zeichen ber 255
			{
			if(iPos<iCount)
					ucTemp[iOut]=0xFF;
			else	pThis->iCheckWr=1;
			}

		iPos++;
		iOut++;

		if(iOut>=sizeof(ucTemp) || iPos>=iCount)	// Daten schreiben
			{
			   iNum=pThis->pOldWrite(pThis->pOldHandle,ucTemp,iOut);
			if(iNum<0)
				{
				if(iLen+iAdd)break;
				return -1;
				}

			iLen+=iNum;
			}
		}

	   iLen *= 2;
	if(iLen>iCount)									// Auf ungerader Position
		{
		pThis->iSeekAdd = iCount&1;
		iLen=iCount;
		}


return iLen+iAdd;
}


//*****************************************************************************
//*
//*		AnsiRead
//*
//*****************************************************************************
//	Liest Daten vom UTF16 Format aus einer Datei.
//	pHandle		: Ist das Datei-Handle
//	pMem		: Ist der Zeiger auf den Lesepuffer. (im ANSI-Format)
//				  Diese geladenen Daten werden von UFT16 nach ANSI konvertiert.
//	iCount		: Ist die Gre des der Lesepuffers in Byte.
//	Ergibt die Anzahl der gelesenen UTF16 Zeichen bzw. -1 bei einem Fehler
int	_cdecl CFileBuffer::AnsiRead(void *pHandle,void *pMem,int iCount)
{
CFileBuffer	   *pThis=(CFileBuffer  *)pHandle;
unsigned char  *pPtr =(unsigned char*)pMem;
int				iPos,iLen,iNum,iAdd=0;


	if(iCount<0)return -1;


	if(pThis->iSeekAdd && iCount>0)					// Dateizeiger auf ungerader Position
		{
		pPtr++;
		iCount--;
		pThis->iSeekAdd = 0;
		pPtr[0]			= 0;
		iAdd			= 1;
		}


	if(iCount<=0)return iAdd;


	   iNum =(iCount+1)/2;							// Daten lesen
	   iLen =pThis->pOldRead(pThis->pOldHandle,pPtr,iNum);
	if(iLen==0)return iAdd;
	if(iLen <0)return -1;

	iPos = iLen-1;

	if(iLen*2>iCount)								// Wird letztes Zeichen nur halb bergeben
		{
		pThis->iSeekAdd = 1;
		iAdd--;
		pPtr[iPos*2] = pPtr[iPos];
		iPos--;
		}

	for(;iPos>=0;iPos--)							// ANSI auf UTF16 konvertieren
		{
		pPtr[iPos*2  ] = pPtr[iPos];
		pPtr[iPos*2+1] = 0;
		}




return iLen*2+iAdd;
}


//*****************************************************************************
//*
//*		AnsiSeek
//*
//*****************************************************************************
//	Seek Funktion fr ANSI-Streams
//	pHandle		: Ist das Datei-Handle
//	iPos		: Ist die Position zu der Verschoben werden soll (mal 0.5)
//	iMode		: Art der Dateizeiger Vernderung
//					SEEK_SET = von Dateibegin berechnet
//					SEEK_END = von Dateiende  berechnet
//					SEEK_CUR = von aktueller Position berechnet
//	Ergibt die Dateiposition umgewandelt auf die Hlfte.
int	_cdecl CFileBuffer::AnsiSeek(void *pHandle,int iPos,int iMode)
{
CFileBuffer	   *pThis=(CFileBuffer*)pHandle;
int				iRet;


	if(!pThis->pOldSeek)return -1;

	switch(iMode)
		{
	case SEEK_END:	pThis->iSeekAdd  = 0;
					break;

	case SEEK_CUR:	if(pThis->iSeekAdd)iPos++;
					pThis->iSeekAdd  = iPos&1;
					break;

	default:		pThis->iSeekAdd  = iPos&1;
					break;
		}


	   iRet=pThis->pOldSeek(pThis->pOldHandle,iPos/2,iMode);
	if(iRet==-1)return -1;



return iRet*2+pThis->iSeekAdd;
}

//*****************************************************************************
//*
//*		AnsiBuffer
//*
//*****************************************************************************
//	Holte den Puffer fr das Speicherholen bei der ANSI->UTF16 Konvertierung.
//	pClass		: Ist der Zeiger auf die Klasse
//	Ergibt einen Zeiger auf die Daten im ANSI Format
char *_cdecl CFileBuffer::AnsiBuffer(void *pClass)
{
char		   *pOld;
char		   *pNew;
CFileBuffer	   *pThis=(CFileBuffer*)pClass;
int				iBytes;
int				iSize;
int				iPos;
int				iEnd;



	pOld = pThis->pOldBuffer;

	if(pThis->iConvPos*2>=pThis->iBufferEnd)return pOld;

	iPos  = pThis->iConvPos*2;
	iEnd  = pThis->iBufferEnd;
	iSize = iEnd/2;

	if(iSize > pThis->iConvLen)							// Den Puffer vergrern
		{
		if(pThis->bFixFlag)
			{
			iEnd = pThis->iConvLen*2;
			}
		else{
			iBytes	= iSize+iSize/2+1024;
			pNew	= new char [iBytes+4];

			if(!pNew)return pOld;

			memcpy(pNew,pOld,pThis->iConvPos);
			pNew[iSize+1] = 0;
			pNew[iSize  ] = 0;

			pThis->iConvLen   = iBytes;
			pThis->pOldBuffer = pNew;
			delete pOld;
			pOld = pNew;
			}
		}



	for(;iPos<iEnd;iPos+=2)								// UTF16 nach ANSI
		{
		if(pThis->pBuffer[iPos+1])
				pOld[iPos/2] = '\xFF';
		else	pOld[iPos/2] = pThis->pBuffer[iPos];;
		}

	pOld[iPos*2  ] = 0;
	pOld[iPos*2+1] = 0;

	pThis->iConvPos = iPos/2;




return pOld;
}

//*****************************************************************************
//*
//*		AnsiConv
//*
//*****************************************************************************
//	Stellt die Konvertierung von ANSI (Datei) auf UTF16 (Speicher) ein.
//	pClass		: Ist der Zeiger auf die Klasse
//	pData			0 = FileBufferConvert-Zeiger fr Konvertierung
//	iMode		: Ist die Art des Aufrufes
//					0 = Einstellen der Konvertierung
//					1 = Schlieen des Streams
//	Ergibt 1 wenn alles ok sonst 0
int _cdecl CFileBuffer::AnsiConv(void *pClass,void *pData,int iMode)
{
CFileBuffer		   *pThis=(CFileBuffer*)pClass;
FileBufferConvert  *pConv;
int					iLen;
int					iPos;


	switch(iMode)
		{
	case 0:												// Einstelle der Konvertierung

			pThis->pOldBuffer	= pThis->pBuffer;
			pThis->pOldHandle	= pThis->pFileHandle;
			pThis->pOldWrite	= pThis->pProcWrite;
			pThis->pOldSeek		= pThis->pProcSeek;
			pThis->pOldRead		= pThis->pProcRead;
			pThis->pFileHandle	= pThis;


			if(pThis->bMemFlag)							// Nur Speicherzugriff
				{
				iLen				= pThis->iBufferLen*2;
				pThis->iConvPos		= pThis->iBufferLen;
				pThis->iConvLen		= pThis->iBufferEnd;
				pThis->pBuffer		= new char [iLen+4];

				if(!pThis->pBuffer)
					{
					return 1;
					}

				if(!pThis->iOpenMode)
					{
					pThis->iBufferEnd = iLen;
					pThis->iBufferLen = iLen;
					}
				else{
					iLen			= 0;
					pThis->iConvPos = 0;

					if(pThis->iBufferPos>0)				// Sind schon Daten Puffer
						{
						iLen = pThis->iBufferPos*2;
						pThis->iBufferPos	= iLen;
						}
					}

				for(iPos=0;iPos<iLen;iPos+=2)			// ANSI auf UTF16 konvertieren
					{
					pThis->pBuffer[iPos  ] = pThis->pOldBuffer[iPos/2];
					pThis->pBuffer[iPos+1] = 0;
					}

				pThis->pBuffer[iLen  ]	= 0;
				pThis->pBuffer[iLen+1]	= 0;
				pThis->iConvPos			= iLen/2;
				pThis->pProcBuff		= AnsiBuffer;
				}
			else{										// Zugriff auf eine Datei
				pConv = (FileBufferConvert*)pData;

				pThis->pProcWrite = pConv->pWrite;
				pThis->pProcSeek  = pConv->pSeek;
				pThis->pProcRead  = pConv->pRead;
				}

			return 1;

	case 1:	if(pThis->bMemFlag)							// Schlieen des Streams
				{
				if(pThis->pBuffer)delete pThis->pBuffer;
				   pThis->pBuffer      = pThis->pOldBuffer;
				}

			pThis->pFileHandle	= pThis->pOldHandle;

			return 1;

		}


return 0;
}

//*****************************************************************************
//*
//*    Inline Funktionen
//*
//*****************************************************************************
#ifdef _DEBUG

bool  CFileBuffer::Eof()   {return bEofFlag;}

int   CFileBuffer::Write   (const unsigned char  *pMem,int iLen){return Write((const char*)pMem,iLen);}
int   CFileBuffer::Write   (const 		   void  *pMem,int iLen){return Write((const char*)pMem,iLen);}
int   CFileBuffer::Write   (const unsigned char  *pMem         ){return Write((const char*)pMem,(int)strlen((const char*)pMem));}
int   CFileBuffer::Write   (const          char  *pMem         ){return Write((const char*)pMem,(int)strlen((const char*)pMem));}
int   CFileBuffer::Write   (const unsigned short *pMem         ){return Write((const char*)pMem,(int)wcslen((const wchar_t*)pMem)*2);}
int   CFileBuffer::Write   (const          short *pMem         ){return Write((const char*)pMem,(int)wcslen((const wchar_t*)pMem)*2);}
int   CFileBuffer::Write   (unsigned   ifb64 iVal){return Write((char*)&iVal,sizeof(unsigned   ifb64));}
int   CFileBuffer::Write   (           ifb64 iVal){return Write((char*)&iVal,sizeof(           ifb64));}
int   CFileBuffer::Write   (unsigned   int   iVal){return Write((char*)&iVal,sizeof(unsigned   int  ));}
int   CFileBuffer::Write   (           int   iVal){return Write((char*)&iVal,sizeof(           int  ));}
int   CFileBuffer::Write   (unsigned   short iVal){return Write((char*)&iVal,sizeof(unsigned   short));}
int   CFileBuffer::Write   (           short iVal){return Write((char*)&iVal,sizeof(           short));}
int   CFileBuffer::Write   (unsigned   char  iVal){return Write((char*)&iVal,sizeof(unsigned   char ));}
int   CFileBuffer::Write   (           char  iVal){return Write((char*)&iVal,sizeof(           char ));}
int   CFileBuffer::WriteLFA(                     ){return Write( "\r\n",2);}
int   CFileBuffer::WriteLFW(                     ){return Write(L"\r\n",4);}
int   CFileBuffer::WriteLF (const char     *pText){return Write( "\r\n",2);}
int   CFileBuffer::WriteLF (const wchar_t  *pText){return Write(L"\r\n",4);}
int   CFileBuffer::WriteLF (                     ){return CFB_WRITELF()   ;}

#ifdef _NATIVE_WCHAR_T_DEFINED
int	  CFileBuffer::Write   (      wchar_t  iValue){return Write((char*)&iValue,sizeof(wchar_t)  );}
int	  CFileBuffer::Write   (const wchar_t *pMem  ){return Write((const char*)pMem,(int)wcslen(pMem)*2);}
#endif

int   CFileBuffer::Read(unsigned   char  *pMem,int iLen){return Read((char*)pMem ,iLen);};
int   CFileBuffer::Read(		   void  *pMem,int iLen){return Read((char*)pMem ,iLen);};
int   CFileBuffer::Read(unsigned   ifb64 *pVal){return Read((char*) pVal,sizeof(unsigned   ifb64));}
int   CFileBuffer::Read(           ifb64 *pVal){return Read((char*) pVal,sizeof(           ifb64));}
int   CFileBuffer::Read(unsigned   int   *pVal){return Read((char*) pVal,sizeof(unsigned   int  ));}
int   CFileBuffer::Read(           int   *pVal){return Read((char*) pVal,sizeof(           int  ));}
int   CFileBuffer::Read(unsigned   long  *pVal){return Read((char*) pVal,sizeof(unsigned   long ));}
int   CFileBuffer::Read(           long  *pVal){return Read((char*) pVal,sizeof(           long ));}
int   CFileBuffer::Read(unsigned   short *pVal){return Read((char*) pVal,sizeof(unsigned   short));}
int   CFileBuffer::Read(           short *pVal){return Read((char*) pVal,sizeof(           short));}
int   CFileBuffer::Read(unsigned   char  *pVal){return Read((char*) pVal,sizeof(unsigned   char ));}
int   CFileBuffer::Read(           char  *pVal){return Read((char*) pVal,sizeof(           char ));}
int   CFileBuffer::Read(unsigned   ifb64 &iVal){return Read((char*)&iVal,sizeof(unsigned   ifb64));}
int   CFileBuffer::Read(           ifb64 &iVal){return Read((char*)&iVal,sizeof(           ifb64));}
int   CFileBuffer::Read(unsigned   int   &iVal){return Read((char*)&iVal,sizeof(unsigned   int  ));}
int   CFileBuffer::Read(           int   &iVal){return Read((char*)&iVal,sizeof(           int  ));}
int   CFileBuffer::Read(unsigned   long  &iVal){return Read((char*)&iVal,sizeof(unsigned   long ));}
int   CFileBuffer::Read(           long  &iVal){return Read((char*)&iVal,sizeof(           long ));}
int   CFileBuffer::Read(unsigned   short &iVal){return Read((char*)&iVal,sizeof(unsigned   short));}
int   CFileBuffer::Read(           short &iVal){return Read((char*)&iVal,sizeof(           short));}
int   CFileBuffer::Read(unsigned   char  &iVal){return Read((char*)&iVal,sizeof(unsigned   char ));}
int   CFileBuffer::Read(           char  &iVal){return Read((char*)&iVal,sizeof(           char ));}
int	  CFileBuffer::ReadLine(unsigned char  *pMem,int iLen){return ReadLine((char          *)pMem,iLen);}
int	  CFileBuffer::ReadLine(         short *pMem,int iLen){return ReadLine((unsigned short*)pMem,iLen);}

char *CFileBuffer::Buffer(                ){return pProcBuff(this);}
void *CFileBuffer::GetUserDataPtr(        ){return       pUserData;}
int	  CFileBuffer::GetUserData(           ){return  (int)pUserData;}
void  CFileBuffer::SetUserData(int   iData){pUserData=(void*)iData;}
void  CFileBuffer::SetUserData(void *pData){pUserData=       pData;}

bool  CFileBuffer::OpenWide(const wchar_t *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
bool  CFileBuffer::OpenWide(const char    *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
bool  CFileBuffer::OpenAnsi(const wchar_t *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
bool  CFileBuffer::OpenAnsi(const char    *pName,int iMode,int iSeekEnd,FileBufferConvert *pConverter){return OpenConv(pName,iMode,iSeekEnd,pConverter);}
bool  CFileBuffer::OpenMem (                                            FileBufferConvert *pConverter){return OpenMem(0,0,CFB_CREATE       ,pConverter);}

#endif




