//*****************************************************************************
//*
//*
//*		FileFinder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Die Klasse CFileFinder sucht Dateien in einem Verzeichnis
//	Die Art der Suche wird mit dem Konstuktor oder mit Set(...) eingestellt
//	CFileFinder kann auch rekursif einen ganzen Baum absuchen.
//	Alle Dateinamen des aktuellen Verzeichnisse werden zwischen
//	gespeichert (ohne Pfad angaben)
//	Die Suche wird gestartet mit dem ersten aufruf von Next()
//
//	Funktionen:
//
//		void	        Set				Neuen Suchbereich festlegen
//		int		    	Read			Dateien suchen (alle auf einmal)
//		char	       *Next			Nchste Datei finden
//		char	       *Path			Ergibt aktuellen Pfad
//		char	       *Name			Ergibt Dateinamen ber Index
//		int 	        NameLen			Ergibt Dateinamen Lnge
//		int		        Error			Fehler Meldung abfragen
//		int		        Count			Anzahl der gefundenen Dateien
//		void	        Sort			Dateien sortieren
//		void	        Rename			Dateinamen in der Liste ndern
//		int 	        Len				Ergibt Dateilnge
//		time_t  	    Time		    Ergibt nderungszeit
//	    FfFindStruct   *Info		    Ergibt Zeiger auf Dateiinformationen
//		int 	        IsReadOnly		Ergibt ob Datei schreibgeschtzt ist
//		int 	        IsArchive		Ergibt ob Datei das Arichive Flag hat
//		int 	        IsDir			Ergibt ob Datei Verzeichnis
//	    int 	        IsNet			Ergibt ob Pfad nur ein Netzzweig ist
//	    int 	        Reset			Auslesezeiger fr Next reseten
//	    int 	        Del				Dateinamen aus der Liste lschen
//	    int 	        Find			Suche Datei in Liste
//		int 	        GetIcon			Icon Nummer holen
//		int			    GetSmallIcon	Icon Nummer holen
//		int			    GetLargeIcon	Icon Nummer holen
//		int 	        GetData			Daten Wert lesen
//		int		        SetData			Daten Wert schreiben
//	    void	        Filter			Dateinamen konvertieren
//		void		    Break			Suche abbrechen
//
//	Die Module	"StringChain.cpp" "MemFunctions" "StringFunctions.cpp"
//	und "ShFind.cpp" werden mit verwendet.
//
#include	"FileFunctions.h"
#include	"MemFunctions.h"
#include	"FileFinder.h"
#include	"DataField.h"
#ifndef		CFF_NONET
#include	"ShFind.h"
#endif
#include	<malloc.h>
#include	<string.h>
#include	<stdlib.h>
#include	<stddef.h>
#include	<stdio.h>
#include	<conio.h>
#include	<errno.h>
#include	<time.h>

#ifndef		CFF_NONET
#include	<lm.h>
#include 	<lmcons.h>
#include 	<lmshare.h>
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
	#ifdef		WIN32
	#include	<Windows.h>
	#endif
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


#if	defined(LINUX) || defined(linux)
	#ifndef 	LINUX
	#define 	LINUX
	#endif
	#define 	SLASH			'/'
	#define 	SLASH_INV		'\\'
	#define 	SLASH_CMP		Z("/*\0")
	#define 	SLASH_TXT		Z("/\0\0")
	#define 	SLASH_NET		Z("//\0")
#else
	#define 	SLASH			'\\'
	#define 	SLASH_INV		'/'
	#define 	SLASH_CMP		Z("\\*\0")
	#define 	SLASH_TXT		Z("\\\0\0")
	#define 	SLASH_NET		Z("\\\\\0")
#endif


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)				L##a
#define 	zchar				wchar_t
#define 	zcharu				short
#define 	zword				long
#define 	strzlen				(int)wcslen
#define 	strzcspn			(int)wcscspn
#define 	strzicmp			wcsicmp
#define 	strzcpy				wcscpy
#define 	strzcat				wcscat
#define 	strzncpy			wcsncpy
#define 	strzncat			wcsncat
#define 	strznicmp			wcsnicmp
#else
#define 	Z(a)				a
#define 	zchar				char
#define 	zcharu				unsigned char
#define 	zword				short
#define 	strzlen				(int)strlen
#define 	strzcspn			(int)strcspn
#define 	strzicmp			stricmp
#define 	strzcpy				strcpy
#define 	strzcat				strcat
#define 	strzncpy			strncpy
#define 	strzncat			strncat
#define 	strznicmp			strnicmp
#endif

static		const int			iTempVal=0x8000;
static		const int			iMinusVal=-1;
#define		TEMP_MEM			&iTempVal
#define		MINUS_MEM			&iMinusVal
#define		offset_of(s,m)  	((int)&(((s*)0)->m))

#ifdef	WIN32

CIconChache	sIconCache;
HIMAGELIST	CIconChache::m_aImageList [2];
BYTE		CIconChache::m_aBaseDrive;
BYTE		CIconChache::m_aDriveType [26];
INT			CIconChache::m_aDriveIconL[26];
INT			CIconChache::m_aDriveIconS[26];
UINT		CIconChache::m_aDriveDoneL[26];
UINT		CIconChache::m_aDriveDoneS[26];
UINT		CIconChache::m_uDriveBits;
UINT		CIconChache::m_uScanLast;

#endif

#ifdef		CFF_NONET
#define 	DelShItems()
#else
zchar		CFileFinder::m_aNetBase[]	= Z("Netzwerksumgebung");
int			CFileFinder::m_iNetBaseLen	= 0;
#endif

#define 	DATA_MAX			4096
#define 	DATA_INIT(a,l)		{if(m_iUserSize && *(char*)(a))memset(a,0,m_iUserSize);						\
								*(int*)(((char*)(a))-sizeof(int))=l; }
#define 	DATA_ALLOC(a,b)		a = (char*)alloca(m_iUserSize+sizeof(FileDataStr)+sizeof(int))+sizeof(int);	\
								if(m_iUserSize){b=(FileDataStr*)((char*)(a)+m_iUserSize);*(char*)(a)=1;}	\
								else		   {b=(FileDataStr*)((char*)(a));}								\
								b->pUserPtr=0;

#define		FfData				CFileFinder::FileDataStr
#define		FfStr				FfFindStruct


#ifdef WIN32
//*****************************************************************************
//*
//*		FfFullPath
//*
//*****************************************************************************
//	converts a relative path to a full path
//	pFilename	: is the relative path
//	pBuffer		: is the buffer for the full path
//	iMax		: is the size of the buffer
//	returns the length in bytes of the full path name.
int FfFullPath(const zchar *pFilename,zchar *pBuffer,int iMax)
{

	if(iMax<=0)return 0;

	pBuffer[0] = 0;

	return GetFullPathName(pFilename,iMax,pBuffer,NULL);

}

//*****************************************************************************
//*
//*		FfClearFindStruct
//*
//*****************************************************************************
//	Lscht eine FfFindStruct Struktur
//	pFind	Zeiger auf die Struktur
static __inline void FfClearFindStruct(FfFindStruct *pFind)
{

	pFind->dwSize   = 0;
	pFind->cName[0] = 0;

}

//*****************************************************************************
//*
//*		FfFileInfo
//*
//*****************************************************************************
//	Holt Informationen zu einer Datei
//	pFilename	: Ist der Name der Datei
//	pInfo		: Hier werden die Datei Infos gespeichert
//	Ergibt 0 wenn ok, oder eine Fehlernummer kleiner 0.
int FfFileInfo(const zchar *pFilename,FfFindStruct *pInfo)
{
WIN32_FILE_ATTRIBUTE_DATA	sData;
int							iAttr;
int							iErr;


	if(!pInfo)return SYS_ERR_PARAM;

													// Existiert die Datei
	if(!GetFileAttributesEx(pFilename,GetFileExInfoStandard,&sData))
		{
		FfClearFindStruct(pInfo);
		iErr=GetLastError();

		switch(iErr)
			{
		case ERROR_FILE_NOT_FOUND:	return SYS_ERR_FILE_NOT_FOUND;
		case ERROR_PATH_NOT_FOUND:	return SYS_ERR_PATH_NOT_FOUND;
			}

		return 0;
		}

	pInfo->cName[0] = 0;

	iAttr=(sData.dwFileAttributes&FILE_ATTRIBUTE_READONLY)? SYS_READ:SYS_READ|SYS_WRITE;
	if(sData.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN   )iAttr|=SYS_HIDDEN;
	if(sData.dwFileAttributes&FILE_ATTRIBUTE_SYSTEM   )iAttr|=SYS_SYSTEM;
	if(sData.dwFileAttributes&FILE_ATTRIBUTE_ARCHIVE  )iAttr|=SYS_ARCHIVE;
	if(sData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)iAttr|=SYS_DIR;

	pInfo->dwAttr = (sData.nFileSizeHigh<<SYS_HIGHSHIFT)|iAttr;
	pInfo->dwSize =  sData.nFileSizeLow;

	FileTimeToUtcTime(&sData.ftCreationTime  ,&pInfo->dwTimeCreate);
	FileTimeToUtcTime(&sData.ftLastWriteTime ,&pInfo->dwTimeWrite);
	FileTimeToUtcTime(&sData.ftLastAccessTime,&pInfo->dwTimeAccess);


return 0;
}

//*****************************************************************************
//*
//*		FfFindFirstEx
//*
//*****************************************************************************
//	Sucht nach einer Datei
//	pFilename	: Ist der Pfad nach dem gesucht werden soll.
//				  Er kann Wildcars wie '*' und '?' enthalten.
//	pFind		: Zeiger auf den Puffer wo das Suchergenis gespeichert wird
//	pError		: Hier wird die Fehlernummer gespeichert
//	Ergint ein Handle bzw. 0 bei einem Fehler.
FfFind	_cdecl FfFindFirstEx(const zchar *pFilename,FfFindStruct *pFind,int *pError)
{
WIN32_FIND_DATA	sFind;
HANDLE			hHandle;
int				iAttr;
int				iErr;

	if(!pFind)
		{
		if(pError)*pError=SYS_ERR_PARAM;
		return 0;
		}

	   hHandle=FindFirstFile(pFilename,&sFind);
	if(hHandle==INVALID_HANDLE_VALUE)
		{
		if(pError)
			{
			iErr=GetLastError();

			     if(iErr==ERROR_FILE_NOT_FOUND)*pError=SYS_ERR_FILE_NOT_FOUND;
			else if(iErr==ERROR_NO_MORE_FILES )*pError=SYS_ERR_FILE_NOT_FOUND;
			else if(iErr==ERROR_DIRECTORY     )*pError=SYS_ERR_PATH_NOT_FOUND;
			else							   *pError=SYS_ERR_UNKNOWN;
			}

		FfClearFindStruct(pFind);
		return 0;
		}

	strzcpy(pFind->cName,sFind.cFileName);

	iAttr=(sFind.dwFileAttributes&FILE_ATTRIBUTE_READONLY)? SYS_READ:SYS_READ|SYS_WRITE;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN   )iAttr|=SYS_HIDDEN;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_SYSTEM   )iAttr|=SYS_SYSTEM;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_ARCHIVE  )iAttr|=SYS_ARCHIVE;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)iAttr|=SYS_DIR;

	pFind->dwAttr = (sFind.nFileSizeHigh<<SYS_HIGHSHIFT)|iAttr;
	pFind->dwSize =  sFind.nFileSizeLow;

	FileTimeToUtcTime(&sFind.ftCreationTime  ,&pFind->dwTimeCreate);
	FileTimeToUtcTime(&sFind.ftLastWriteTime ,&pFind->dwTimeWrite );
	FileTimeToUtcTime(&sFind.ftLastAccessTime,&pFind->dwTimeAccess);



return (FfFind)hHandle;
}

//*****************************************************************************
//*
//*		FfFindNext
//*
//*****************************************************************************
//	Setzt eine Suche nach Dateien fort.
//	hHandle	: Ist das Such-Handle von FfFindFirst(...)
//	pFind	: Zeiger auf den Puffer wo das Suchergenis gespeichert wird
//	Ergibt 0 wenn eine weitere Datei gefunden wurde, bzw eine Fehlernummer.
int _cdecl FfFindNext(FfFind hHandle,FfFindStruct *pFind)
{
WIN32_FIND_DATA	sFind;
int				iAttr;


	if(!pFind || (DWORD)hHandle>=0xFFFFFFE0)
		{
		FfClearFindStruct(pFind);
		return SYS_ERR_PARAM;
		}

	if(!FindNextFile((HANDLE)hHandle,&sFind))
		{
		return SYS_ERR_FILE_NOT_FOUND;
		}

	strzcpy(pFind->cName,sFind.cFileName);

	iAttr=(sFind.dwFileAttributes&FILE_ATTRIBUTE_READONLY)? SYS_READ:SYS_READ|SYS_WRITE;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN   )iAttr|=SYS_HIDDEN;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_SYSTEM   )iAttr|=SYS_SYSTEM;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_ARCHIVE  )iAttr|=SYS_ARCHIVE;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)iAttr|=SYS_DIR;

	pFind->dwAttr = (sFind.nFileSizeHigh<<SYS_HIGHSHIFT)|iAttr;
	pFind->dwSize =  sFind.nFileSizeLow;

	FileTimeToUtcTime(&sFind.ftCreationTime  ,&pFind->dwTimeCreate);
	FileTimeToUtcTime(&sFind.ftLastWriteTime ,&pFind->dwTimeWrite );
	FileTimeToUtcTime(&sFind.ftLastAccessTime,&pFind->dwTimeAccess);



return 0;
}


//*****************************************************************************
//*
//*		FfFindClose
//*
//*****************************************************************************/
//	Schliet ein FfFind Handle
//	hHandle		: Ist das Handle
//	Ergibt 0 wenn alles ok war,bzw. eine Fehlernummer unter 0.
int	_cdecl FfFindClose(FfFind hHandle)
{


	if(!hHandle)
		{
		return SYS_ERR_PARAM;
		}

	if(!FindClose((HANDLE)hHandle))
		{
		return SYS_ERR_PARAM;
		}


return 0;
}
#endif

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
//	Neuen Suchbereich festlegen
//	pPath	: Ausgangspfad fr die Suche. Ist am Pfadende ein Suchfilter
//				  angehngt (z.B ...\*.abc") so wird dies zu den Pluspiltern
//				  hinzu gefgt.
//				  Ist Pfad==":" so werden alle Laufwerke durchsucht
//				  Pfade die mit Verzeichnisen enden brauchen kein '\' am Ende.
//				  (z.B. "C:\WINDOWS\*.txt;*.log")
//	iFlags	: Optionen fr die Suche
//				  CFF_FIND_DIR			Nur Verzeichnise suchen
//				  CFF_FIND_PARTIAL   	Die Suche immer wieder abbrechen
//				  CFF_FIND_SUBDIR		Auch in Unterverz. suchen
//				  CFF_FIND_ADDDIR		Auch Verzeichnisse finden
//				  CFF_FIND_ADDUP		Auch '..' einfgen
//				  CFF_FIND_SUBDIRALL	Auch in Unterverz. und Verzeichnise
//				  CFF_FIND_SORT	  		Dateien sortiert einfgen
//				  CFF_FIND_ALL     		Alles auf Einmal lesen
//				  CFF_FIND_ADD     		Dateien zur den jetztigen anhngen
//	pPlus	: Dateifilter fr die Dateien die gefunden werden sollen
//				  Form "*.c;*.h;abc*.???"
//	pMinus	: Dateifilter fr die Dateien die nicht gefunden werden sollen
//				  Form "*.c;*.h;abc*.???"
CFileFinder::CFileFinder(const zchar *pPath,int iFlags,const zchar *pPlus,const zchar *pMinus)
{

	#ifndef	 CFF_NONET
	m_sShItems.DataLength(sizeof(void*));
	m_sShFinds.DataLength(sizeof(void*));
	#endif

	m_sFolders.DataLength(sizeof(int));
	m_sFiles  .DataLength(offset_of(FileDataStr,sInfo.cName));

	m_iPathNum 		= -1;
	m_iSortMode		= -1;
	m_cDoneFlag		=  0;
	m_pChangeCb		=  0;
	m_pBuffer		=  0;
	m_uBufferPos	=  0;
	m_iFindFlags  	=  0;
	m_iFolderNum  	=  0;
	m_iLastError    =  0;
	m_iAttrFilter	=  0;
	m_iOneFilter	=  0;
	m_iTimeFilter	=  0;
	m_iRdWrFilter	=  0;
	m_iUserSize		=  0;
	m_pFilePath     =  0;
	m_hNextFile		=  0;
	m_pTempUser    	=  0;
	m_pTempPath		=  0;
	m_iInfoOffset	=  offset_of(FileDataStr,sInfo);

	Set(pPath,iFlags,pPlus,pMinus);

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFileFinder::~CFileFinder()
{

	Set(0);

	if(m_pTempPath)
		{
		delete m_pTempPath;
		m_pTempPath  = 0;
		}

	if(m_pTempUser)
		{
		m_pTempUser -= sizeof(int);
		delete m_pTempUser;
		m_pTempUser  = 0;
		}

}


//*****************************************************************************
//*
//*		DelShItems
//*
//*****************************************************************************
//	Netzwerkszweige lschen
#ifndef	 CFF_NONET
void CFileFinder::DelShItems()
{
CShItem	*pShi;
CShFind	*pShf;
int		 i;


	for(i=0;i<m_sShItems.Count();i++)
		{
		pShi=(CShItem*)m_sShItems.DataGetPtr(i);
		delete pShi;
		}

	for(i=0;i<m_sShFinds.Count();i++)
		{
		pShf=(CShFind*)m_sShFinds.DataGetPtr(i);
		delete pShf;
		}


	m_sShItems.Del();
	m_sShFinds.Del();

}
#endif

//*****************************************************************************
//*
//*		Set
//*
//*****************************************************************************
//	Einen neuen Suchbereich festlegen.
//	pPath	: Ist der Ausgangspfad (z.B. "C:\ABC\") fr die Suche.
//			  Wenn am Pfadende ein Suchfilter (z.B ...\*.abc") angehngt ist,
//			  so wird dies zu den pPlus Filtern hinzu gefgt.
//			  Ist Pfad==":" so werden alle Laufwerke durchsucht
//			  Pfade die mit Verzeichnisen enden brauchen kein '\' am Ende.
//			  (z.B. "C:\WINDOWS\*.txt;*.log")
//	iFlags	: Optionen fr die Suche.
//				  CFF_FIND_DIR			Nur Verzeichnise suchen
//				  CFF_FIND_SUBDIR		Auch in Unterverz. suchen
//				  CFF_FIND_ADDDIR		Auch Verzeichnisse finden
//				  CFF_FIND_SUBDIRALL	Auch in Unterverz. und Verzeichnise
//				  CFF_FIND_SORT	  		Dateien sortiert einfgen
//				  CFF_FIND_ALL     		Alles auf Einmal lesen
//				  CFF_FIND_ADD     		Dateien zur den jetztigen anhngen
//				  CFF_FIND_ADDUP		Auch '..' einfgen
//				  CFF_FIND_PARTIAL   	Die Suche immer wieder abbrechen
//	pPlus	: Dateifilter fr die Dateien die gefunden werden sollen
//			  Form "*.c;*.h;abc*.???"
//	pMinus	: Dateifilter fr die Dateien die nicht gefunden werden sollen
//			  Form "*.c;*.h;abc*.???"
void CFileFinder::Set(const zchar *pPath,int iFlags,const zchar *pPlus,const zchar *pMinus)
{
FfFindStruct	sFind;
zchar		   *pOld=0;
int				iCount;
int				iAllFilter=0;
int				iRelOld=0;
int				iLen,i,j;


	if(m_hNextFile)											// Partielle Suche beenden
		{
		FfFindClose(m_hNextFile);
		m_hNextFile = 0;
		}

	m_sPlus .Del();											// Filter lschen
	m_sMinus.Del();
	m_sPlus	.DataLength(sizeof(int));
	m_sMinus.DataLength(sizeof(int));

	if(pPath && (iFlags&CFF_FIND_ADD))						// Suche anhngen
		{
		pOld			= m_pFilePath;
		iRelOld			= m_iStartPathLen;
		m_pFilePath		= 0;
		m_uCurrentState	= 0;
		m_iFolderNum	= m_sFolders.Count();
		iFlags		   |= CFF_FIND_ALL;
		}
	else{
		DelShItems();

		iCount	= m_sFiles.Count();

		if(m_pChangeCb && iCount)
			{
			m_pChangeCb(m_pCbParam,0,iCount,m_uCbDelete^m_uCbLock);
			}

		m_sPlus   .Del();									// Alle String Ketten lschen
		m_sMinus  .Del();
		m_sFiles  .Del();
		m_sFolders.Del();

		m_uCurrentState	=  0xFFFFFFFF;
		m_iStartPathLen	=  0;
		m_iCurrentNum	=  0;
		m_iFolderNum	=  0;
		m_iPathNum		= -1;

		if(m_pChangeCb && iCount)
			{
			m_pChangeCb(m_pCbParam,0,iCount,m_uCbDelete);
			}
		}

	m_iStartFolderNum	= m_sFolders.Count();

	if(!pPath)
		{
		if(m_pFilePath)										// Den alten Suchpfad lschen
			{
			delete m_pFilePath;
			m_pFilePath=0;
			}

		return;
		}

	if(!m_pFilePath)
		{
		m_pFilePath	= new zchar[CFF_PATH_MAX];				// Auf ganzen Pfad ergnzen
		}


//*****************************************************************************


	iFlags &= ~0xFF;

	if(pPath[0]==':')
		{
		m_pFilePath[0] = ':';
		m_pFilePath[1] = 0;

		iLen = strzlen(pPath);

		for(iLen--;iLen>0;iLen--)							// Suche Plusfilter
			{
			if(pPath[iLen]=='\\' || pPath[iLen]=='/' || pPath[iLen]==':')
				{
				ScanFilter(pPath+iLen,m_sPlus,0);
				break;
				}
			}
		}
	else do
		{
		#ifndef CFF_NONET
		if((iFlags&CFF_FIND_NET) &&
		   (pPath[0]=='\\' || pPath[0]=='/') &&
		   (pPath[1]=='\\' || pPath[1]=='/'))
			{
			iLen = NetPathConvert(pPath,m_pFilePath,CFF_PATH_MAX);
			}
		else
		#endif
			{
			iLen = FfFullPath(pPath,m_pFilePath,CFF_PATH_MAX);

			for(i=iLen;i>=0;)
				{
				i--;

				if(m_pFilePath[i]==SLASH_INV)
					{
					m_pFilePath[i]=SLASH;
					}
				}
			}

		j = strzcspn(m_pFilePath,Z(";*?"));

		if(m_pFilePath[j]==0)								// Datei oder Verzeichnis ?
			{
			m_iStartPathLen=j;

			if(j && (m_pFilePath[j-1]=='/' || m_pFilePath[j-1]=='\\'))break;


			if(FfFileInfo(m_pFilePath,&sFind)>=0)			// Ist Pfad vorhanden ?
				{
				if(sFind.dwAttr&SYS_DIR)					// Ist Pfad ein Verzeichnis ?
					{
					m_iStartPathLen  = j+1;
					m_pFilePath[j  ] = SLASH;
					m_pFilePath[j+1] = 0;
					break;
					}
				else{										// Ist Pfad ist eine Datei
					iFlags &= ~CFF_FIND_SUBDIR;
					pPlus	=  NULL;

					if(!(iFlags&(CFF_FIND_SUBDIR|CFF_FIND_ADDDIR)))
						{
						iFlags |= CFF_FIND_ONE;
						}
					}
				}
			else{											// Ist Pfad im Netz vorhanden ?
				if(m_pFilePath[0]=='\\' || m_pFilePath[0]=='/')
				if(m_pFilePath[1]=='\\' || m_pFilePath[1]=='/')
					{
					m_pFilePath[j  ] = SLASH;
					m_pFilePath[j+1] = '*';
					m_pFilePath[j+2] =  0;

					if(FfFileInfo(m_pFilePath,&sFind)>=0)
						{
						m_iStartPathLen  = j+1;
						m_pFilePath[j+1] = 0;
						break;
						}

					if(iFlags&CFF_FIND_NET)
						{
						m_iStartPathLen  = j+1;
						m_pFilePath[j+1] = 0;
						break;
						}
					}
				}

			m_pFilePath[j]=0;
			}


		for(i=iLen-1;i>=0;i--)								// Pfad bis '\' krzen
			{
			if(m_pFilePath[i]=='\\')break;
			if(m_pFilePath[i]=='/' )break;
			}

		iAllFilter       = ScanFilter(m_pFilePath+i+1,m_sPlus);
		m_iStartPathLen  = i+1;								// Lnge vom Anfangspfad
		m_pFilePath[i+1] = 0;
		}while(0);


//*********************** Filter ***********************************************


	if(pPlus && !iAllFilter)
		{
		iAllFilter |= ScanFilter(pPlus,m_sPlus);
		}

	if(pMinus)
		{
		ScanFilter(pMinus,m_sMinus,0);
		}

	if(iAllFilter)											// Plusfilter lschen wenn "*.*" enthalten
		{
		m_sPlus.Del();
		}


	m_iFilerPlus  = m_sPlus .Count();
	m_iFilerMinus = m_sMinus.Count();
															// Nur einzelner Plusfilter
	if(m_iFilerPlus==1 && !(iFlags&(CFF_FIND_SUBDIR|CFF_FIND_ADDDIR)))
		{
		iFlags |= CFF_FIND_ONE;
		}

	if(iFlags&CFF_FIND_ADD)									// Pfadlnge zurcksetzen wenn neuer Pfad ungleich
		{
		if(pOld)
			{
			if(iRelOld==m_iStartPathLen)
			if(strzicmp(pOld,m_pFilePath))m_iStartPathLen=0;
			delete pOld;
			}
		else if(m_sFiles.Count())
			{
			m_iStartPathLen = 0;
			}
		}

	m_pCmpProc = NULL;

	if(iFlags&CFF_FIND_SORT)								// Dateien sortiert einfgen
		{
		if(m_sFiles.Count()<=1 || m_iSortMode>=0)
			{
			m_pCmpProc = CmpProc;
			iFlags |=  CFF_FIND_ALL;
			}
		else{
			iFlags &= ~CFF_FIND_SORT;						// Die Liste ist unsortiert
			}
		}


	m_iFindFlags	= iFlags;
	m_iLastError    = 0;
	m_uCurrentState = 0;


}

//*****************************************************************************
//*
//*		ScanFilter
//*
//*****************************************************************************
//	Speichert alle Filter aus einer Liste in einer StringKette
//	pList		: Ist der Text mit den Filtern
//	sFilter		: Referenz auf die StringKette
//	iAll		: Nach dem Alles-Filter suchen
//	Ergibt 1 wenn ein Filter wie "*.*" oder "*" gefunden wurde
int CFileFinder::ScanFilter(const zchar *pList,CStringChain &sFilter,int iAll)
{
int		iFound=0;
int		iMode;
int		iStop;
int		iLen;



	while(*pList)											// Suchfilter lesen
		{
		if(*pList==';')
			{
			pList++;
			continue;
			}

		for(iLen=0;pList[iLen];iLen++)
			{
			if(pList[iLen]==';')
				{
				break;
				}
			}

		if(iAll && pList[0]=='*')
			{
			if(iLen==3 && pList[1]=='.' && pList[2]=='*')
				{
				iFound = 1;
				pList += 2;
				continue;
				}

			if(iLen==1 && pList[0]=='*')
				{
				iFound = 1;
				pList += 1;
				continue;
				}
			}


		iMode = 0;


		for(iMode=0;;iMode++)								// Zhle '*' am Beginn
			{
			if(pList[iMode]!='*')break;
			}

		for(iStop=iMode;;iStop++)							// Ist es ein Typ "*.xxx" oder "xxx"
			{
			if(iStop>=iLen)
				{
				iMode++;
				break;
				}

			if(pList[iStop]=='*')
				{
				iMode = 0;
				break;
				}

			if(pList[iStop]=='?')
				{
				iMode = 0;
				break;
				}
			}

		sFilter.Add(iLen,pList,CSC_END,&iMode);
		pList += iLen;
		}


return iFound;
}

//*****************************************************************************
//*
//*		Next
//*
//*****************************************************************************
//	Sucht nach der nchsten Datei.
//	iMode	: 0 dann wird der Dateiname ohne Pfad ausgegeben
//			  1 dann wird der vollstntige Dateiname ausgegeben
//			  2 dann wird der ausgehend von Startpfad ausgegeben
//	pLen	: Wenn der Zeiger ungleich 0 ist, wird hier die Pfadlnge gespeichert.
//	Ergibt den nchsten gefundene Dateinamen bzw. 0 bei einem Fehler
zchar *CFileFinder::Next(int iMode,int *pLen)
{
FileDataStr    *pData;
zchar		   *pName;
int				iLen;


	m_cDoneFlag  = 0;
	m_cUserBreak = 0;

	if(m_uCurrentState>=0xF0000000)							// Alles fertig eingelesen ?
		{
		m_iLastError  = CFF_ERROR_NULL;
		if(pLen)*pLen = 0;
		return 0;
		}

	while(m_iCurrentNum>=m_sFiles.Count())					// Nchstes Verzeichnis einlesen ?
		{
		if(Read()<0 && !m_pFilePath)
			{
			m_iLastError  = CFF_ERROR_NULL;
			if(pLen)*pLen = 0;
			return 0;
			}

		if(m_iFindFlags&CFF_FIND_PARTIAL)
			{
			if(m_hNextFile)break;
			}
		else{
			m_iCurrentNum = 0;
			}

		if(m_uCurrentState==0xF0000000)						// Alles fertig eingelesen ?
			{
			m_iLastError  = CFF_ERROR_NULL;
			if(pLen)*pLen = 0;
			return 0;
			}
		}


	pData = DataNone(m_iCurrentNum);						// Nchste Datei holen
	pName = pData->sInfo.cName;

	m_iLastError = CFF_ERROR_OK;							// Kein Fehler

	if(!iMode)												// Dateiname ohne Pfad zurckgeben
		{
		if(pLen)*pLen=m_sFiles.LenNone(m_iCurrentNum);
		m_iCurrentNum++;
		return pName;
		}

	if(m_iFindFlags&(CFF_FIND_ALL|CFF_FIND_TEMP))			// Gibt es mehr als einen Pfad
		{
		BufferCopy(pData->iFolderNum);
		}
	else{
		BufferCopy(m_iFolderNum);
		}

	iLen = m_sFiles.LenNone(m_iCurrentNum);
	memcpy(m_pTempPath+m_iPathLen,pName,iLen+1);			// Namen an Pfad anhngen

	m_iCurrentNum++;


	if(iMode==2)											// Relativer Pfad
		{
		if(pLen)*pLen=m_iPathLen+iLen-m_iStartPathLen;
		return m_pTempPath+m_iStartPathLen;
		}

	if(pLen)												// Lnge des Pfades ermitteln
		{
		*pLen = m_iPathLen+iLen;
		}


return m_pTempPath;
}

//*****************************************************************************
//*
//*		Path
//*
//*****************************************************************************
//	Den Pfad einer gefundenen Datei abfragen, aus dem Dateiinfos
//	pInfo	: Zeiger auf die Dateiinfos
//	iMode	: 0 dann wird der vollstntige Pfadname ausgegeben
//			  1 dann wird der ausgehend von Startpfad ausgegeben
//	pLen	: Wenn der Zeiger ungleich 0 ist, wird hier die Pfadlnge gespeichert.
//	Ergibt Pfad der gewhlten Datei
zchar *CFileFinder::Path(FfStr *pInfo,int iMode,int *pLen)const
{
FileDataStr    *pData;
zchar		   *pPath;


	if(!pInfo)												// Zeiger auf die Dateiinfos prfen
		{
		if(iMode)
			{
			if(pLen)*pLen=0;
			return Z("");
			}

		if(pLen)
			{
			*pLen = strzlen(m_pFilePath);
			}

		return m_pFilePath;
		}

	pData = InfoToData(pInfo);

	pPath = pData->pPath;

	if(pLen)												// Lnge des Pfades ermitteln
		{
		*pLen = m_sFolders.LenNone(pData->iFolderNum);
		}

	if(iMode)												// Relativer Pfad
		{
		if(pLen)*pLen-=m_iStartPathLen;
		return   pPath+m_iStartPathLen;
		}



return pPath;
}

//*****************************************************************************
//*
//*		Link
//*
//*****************************************************************************
//	Den Pfad einer gefundenen Netzwerkslink abfragen, aus dem Dateiinfos
//	pInfo	: Zeiger auf die Dateiinfos
//	pLen	: Wenn der Zeiger ungleich 0 ist, wird hier die Pfadlnge gespeichert.
//	Ergibt Link-Pfad der gewhlten Datei, oder 0 bei einem Fehler
zchar *CFileFinder::Link(FfStr *pInfo,int *pLen)const
{
FileDataStr    *pData;
int				iLen;


	if(!pInfo)												// Zeiger auf die Dateiinfos prfen
		{
		if(pLen)*pLen=0;
		return 0;
		}

	pData = InfoToData(pInfo);

	#ifndef CFF_NONET
															// Ist es ein Netzwerklink
	if((pInfo->dwAttr&SYS_NET) && (pInfo->dwAttr&SYS_ARCHIVE))
		{
		   iLen=m_sShItems.Len(pData->iShItemNum);
		if(iLen>0)
			{
			if(pLen)*pLen=iLen;
			return m_sShItems.Get(pData->iShItemNum);
			}
		}

	#endif

	if(pLen)*pLen=0;

return 0;
}

//*****************************************************************************
//*
//*		Suffix
//*
//*****************************************************************************
//	Das Suffix einer gefundenen Datei abfragen aus dem Dateiinfos abfragen
//	pInfo	: Zeiger auf die Dateiinfos
//	pLen	: Wenn der Zeiger ungleich 0 ist, wird hier die Suffixlnge gespeichert.
//	Ergibt einen Zeiger auf das Suffix
zchar *CFileFinder::Suffix(FfStr *pInfo,int *pLen)const
{
FileDataStr    *pData;
void 		   *pUser;
int				iPos;
int				iLen;


	if(!pInfo)												// Zeiger auf die Dateiinfos prfen
		{
		if(pLen)*pLen=0;
		return 0;
		}

	pUser = InfoToUser(pInfo);
	pData = InfoToData(pInfo);

	if(pData->iSuffixPos<0)									// Suffix-Offset noch nicht berechnet
		{
		iLen = UserToLength(pUser);

		for(iPos=iLen;;iPos--)
			{
			if(iPos<=0){iPos=iLen;break;}
			if(pInfo->cName[iPos-1]=='.')break;
			}

		pData->iSuffixPos = iPos;
		}

	if(pLen)												// Lnge des Suffixes ermitteln
		{
		iLen	= UserToLength(pUser);
		pLen[0]	= iLen-pData->iSuffixPos;
		}


return pInfo->cName+pData->iSuffixPos;
}

//*****************************************************************************
//*
//*		InternalNameFull
//*
//*****************************************************************************
//	Einen vollstntigen Dateinamen abfragen.
//	iNr			: Ist die Nummer der Datei in der Liste.
//	pLen		: Wenn der Zeiger ungleich 0 ist, wird hier die Pfadlnge gespeichert
//	iMode		: Wenn 1 dann wird der ausgehend von Startpfad ausgegeben
//	Ergibt den Dateinamen der gewhlten Datei. Tritt ein Fehler auf so
//	wird NULL zurck gegeben
zchar *CFileFinder::InternalNameFull(int iNr,int *pLen,int iMode)
{
FileDataStr    *pData;
zchar		   *pName;
int				iLen;



	if(!m_sFiles.Check(iNr))								// Index-Nummer prfen
		{
		if(pLen)*pLen=0;
		return 0;
		}


	pData = DataNone(iNr);									// Nchste Datei holen
	pName = pData->sInfo.cName;


	if(m_iFindFlags&(CFF_FIND_ALL|CFF_FIND_TEMP))			// Mehr als ein Pfad vorhanden
		{
		BufferCopy(pData->iFolderNum);
		}
	else{
		BufferCopy(m_iFolderNum);
		}

	iLen = m_sFiles.LenNone(iNr);
	memcpy(m_pTempPath+m_iPathLen,pName,iLen+1);

	if(iMode)												// Relativer Pfad
		{
		if(pLen)*pLen=m_iPathLen+iLen-m_iStartPathLen;
		return m_pTempPath+m_iStartPathLen;
		}

	if(pLen)												// Lnge des Pfades ermitteln
		{
		*pLen = m_iPathLen+iLen;
		}


return m_pTempPath;
}

//*****************************************************************************
//*
//*		InternalName
//*
//*****************************************************************************
//	Einen Dateinamen ohne Pfad abfragen.
//	iNr		: Ist die Nummer der Datei in der Liste.
//	pLen	: Wenn der Zeiger ungleich 0 ist, wird hier die Pfadlnge gespeichert.
//	Ergibt den Dateinamen der gewhlten Datei. Tritt ein Fehler auf so
//	wird NULL zurck gegeben
zchar *CFileFinder::InternalName(int iNr,int *pLen)const
{
zchar	*pName;

	if(!m_sFiles.Check(iNr))								// Index-Nummer prfen
		{
		if(pLen)*pLen=0;
		return 0;
		}

	pName = m_sFiles.GetNone(iNr);

	if(pLen)*pLen=m_sFiles.LenNone(iNr);


return pName;
}

//*****************************************************************************
//*
//*		FullNameFullLen
//*
//*****************************************************************************
//	Die Lnge eines Dateinamens mit Pfad abfragen.
//	iNr		: Ist die Nummer der Datei in der Liste. (-1 fr aktuelle Datei)
//	iMode	: 0 dann wird die Lnge vom vollstntige Dateiname ausgegeben
//			  1 dann wird die Lnge vom relativeb Pfad ausgegeben
//	Ergibt die Lnge vom Dateinamen der gewhlten Datei. Tritt ein Fehler auf
//	so wird 0 zurck gegeben
int	CFileFinder::InternalNameFullLen(int iNr,int iMode)const
{
FileDataStr  *pData;
int			  iLen;


	if(!m_sFiles.Check(iNr))								// Index-Nummer prfen
		{
		return 0;
		}

	iLen = m_sFiles.LenNone(iNr);

	if(m_iFindFlags&CFF_FIND_ALL)
		{
		pData = DataNone(iNr);
		iLen += m_sFolders.LenNone(pData->iFolderNum);
		}
	else{
		iLen += m_iPathLen;
		}


return (iMode)? iLen-m_iStartPathLen:iLen;
}

//*****************************************************************************
//*
//*		Len64
//*
//*****************************************************************************
//	Abfragen der Dateigre. Mit einem 64 Bitwert
//	iNr		: Ist die Nummer der Datei in der Liste. (-1 fr aktuelle Datei)
//	Ergibt die Dateinlnge der gewhlten Datei des aktuell durchsuchten
//	Verzeichnisses. Tritt ein Fehler auf so wird -1 zurck gegeben
iff64 CFileFinder::Len64(FfStr *pInfo)const
{
iff64	iSize;


	if(!pInfo)												// Zeiger prfen
		{
		return -1;
		}

	if(pInfo->dwAttr&SYS_HIGHMASK)
		{
		iSize   = pInfo->dwAttr&SYS_HIGHMASK;
		iSize >>= SYS_HIGHSHIFT;
		iSize   = iSize<<32;
		iSize  += pInfo->dwSize;
		}
	else{
		iSize	= pInfo->dwSize;
		}


return iSize;
}

//*****************************************************************************
//*
//*		InternalTime
//*
//*****************************************************************************
//	Die nderungszeit einer Datei abfragen.
//	pInfo	: Ist der Zeiger auf die Dateiinformationen
//	Ergibt die nderungszeit der gewhlten Datei des aktuell durchsuchten
//	Verzeichnisses. Bei Fehlern oder einer unbekannten Zeit wird -1 zurckgegeben.
time_t CFileFinder::Time(FfStr *pInfo)const
{

	if(pInfo==0)return -1;
	if(pInfo->dwTimeWrite !=(FfUtc)(~0))return pInfo->dwTimeWrite;
	if(pInfo->dwTimeCreate!=(FfUtc)(~0))return pInfo->dwTimeCreate;


return pInfo->dwTimeAccess;
}

time_t CFileFinder::Time(FfStr *pInfo,int iType)const
{
FfUtc	uTime;


	if(pInfo==0)return -1;

	switch(iType)
		{
	case 1:	uTime = pInfo->dwTimeCreate;	break;
	case 2:	uTime = pInfo->dwTimeAccess;	break;
	default:uTime = pInfo->dwTimeWrite;		break;
		}

	if(uTime!=(FfUtc)(~0))return uTime;

	if(pInfo->dwTimeWrite !=(FfUtc)(~0))return pInfo->dwTimeWrite;
	if(pInfo->dwTimeCreate!=(FfUtc)(~0))return pInfo->dwTimeCreate;


return pInfo->dwTimeAccess;
}


//*****************************************************************************
//*
//*		SortProc
//*
//*****************************************************************************
//	Standard Vergleichsfunktion fr CFileFinder::Sort
//	pChain		: Ist die Liste mit den Dateien
//	iPos1		: Nummer der ersten  zu Vergleichenden Datei
//	iPos2		: Nummer der zweiten zu Vergleichenden Datei
int __cdecl CFileFinder::SortProc(CStringChain *pChain,int iPos1,int iPos2)
{
CFileFinder	   *pThis;
void		   *pPtr1;
void		   *pPtr2;


	pThis	= (CFileFinder*)pChain->GetUserPtr();
	pPtr1	= pChain->DataNone(iPos1);
	pPtr2	= pChain->DataNone(iPos2);


return CmpProc(pPtr1,pPtr2,pThis);
}


//*****************************************************************************
//*
//*		CmpProc
//*
//*****************************************************************************
//	Standard Vergleichsfunktion frs sortierte Einfgen
//	pChain		: Ist die Liste mit den Dateien
//	iPos1		: Nummer der ersten  zu Vergleichenden Datei
//	iPos2		: Nummer der zweiten zu Vergleichenden Datei
int __cdecl CFileFinder::CmpProc(const void *pPtr1,const void *pPtr2,void *pParam)
{
CFileFinder	   *pThis =(CFileFinder*)pParam;
FileDataStr    *pData1,*pData2;
zchar		   *pName1,*pName2;
int				iAttr1,iAttr2;
int				iVal1 ,iVal2;
int				iFlags,iMode;
int				iExit ,iLen;



	if(pPtr1==pPtr2)return 0;

	pData1 = pThis->UserToData((void*)pPtr1);
	pData2 = pThis->UserToData((void*)pPtr2);
	iFlags = pThis->m_iSortMode;
	iMode  = iFlags&0xFF;
	iExit  = 0;


	if(!(iFlags&CFF_SORT_NODIR))							// Verzeichnisse an den Anfang
		{
		iAttr1 = pData1->sInfo.dwAttr;
		iAttr2 = pData2->sInfo.dwAttr;

		if((iAttr1^iAttr2)&SYS_DIR)							// Ungleiche Attribute
			{
			return (iAttr1&SYS_DIR)? -1:1;
			}

		if(iAttr1&SYS_DIR)
			{
			if((iAttr1^iAttr2)&SYS_DRIVE)					// Laufwerke zuerst
				{
				return (iAttr1&SYS_DRIVE)? -1:1;
				}

			if((iAttr1^iAttr2)&SYS_NET)						// Netzwerk am Ende
				{
				return (iAttr2&SYS_NET)? -1:1;
				}

			if(iAttr1&SYS_NET)
				{
				if((iAttr1^iAttr2)&SYS_ARCHIVE)				// Links am Ende
					{
					return (iAttr2&SYS_ARCHIVE)? -1:1;
					}
				}
			}
		}


//*****************************************************************************


	switch(iMode)
		{
	case CFF_SORT_SUFFIX:									// Nach Suffix sortieren

		pName1 = pData1->sInfo.cName;
		pName2 = pData2->sInfo.cName;

		if(pData1->iSuffixPos>=0)
			{
			iVal1 = pData1->iSuffixPos;
			}
		else{
			iLen  = strzlen(pName1);

			for(iVal1=iLen;;iVal1--)
				{
				if(iVal1<=0){iVal1=iLen;break;}
				if(pName1[iVal1-1]=='.')break;
				}

			if(pData1!=pThis->m_pTempData)
				{
				pData1->iSuffixPos = iVal1;
				}
			}

		if(pData2->iSuffixPos>=0)
			{
			iVal2 = pData2->iSuffixPos;
			}
		else{
			iLen  = strzlen(pName2);

			for(iVal2=iLen;;iVal2--)
				{
				if(iVal2<=0){iVal2=iLen;break;}
				if(pName2[iVal2-2]=='.')break;
				}

			if(pData2!=pThis->m_pTempData)
				{
				pData2->iSuffixPos = iVal2;
				}
			}

		iExit = strzicmp(pName1+iVal1,pName2+iVal2);

		break;

	case CFF_SORT_TYPE:										// Nach Typ sortieren

		pName1 = pData1->pTypeName;
		pName2 = pData2->pTypeName;

		#ifdef _WIN32
		if(!pName1)
			{
			if(pData1->sInfo.dwAttr&SYS_DIR)
					pName1=Z("Dateiordner");
			else	pName1=pThis->GetType(&pData1->sInfo);
			}

		if(!pName2)
			{
			if(pData2->sInfo.dwAttr&SYS_DIR)
					pName2=Z("Dateiordner");
			else	pName2=pThis->GetType(&pData2->sInfo);
			}

		#endif

		if(pName1==pName2){iExit=0;break;}

		if(!pName1)pName1=Z("");
		if(!pName2)pName2=Z("");

		iExit = strzicmp(pName1,pName2);

		break;

	case CFF_SORT_LEN:										// Nach Dateilnge sortieren

		iExit   = pData1->sInfo.dwAttr&SYS_HIGHMASK;
		iExit  -= pData2->sInfo.dwAttr&SYS_HIGHMASK;

		if(iExit)break;

		if(pData1->sInfo.dwSize==pData2->sInfo.dwSize){iExit=0;break;}
		iExit=(pData1->sInfo.dwSize<pData2->sInfo.dwSize)? -1:1;
		break;

	case CFF_SORT_CREATE:									// Nach Erstellungszeit sortieren

		if(pData1->sInfo.dwTimeCreate==pData2->sInfo.dwTimeCreate){iExit=0;break;}
		iExit=(pData1->sInfo.dwTimeCreate<pData2->sInfo.dwTimeCreate)? -1:1;
		break;

	case CFF_SORT_ACCCESS:									// Nach letztdem Zugriff sortieren

		if(pData1->sInfo.dwTimeAccess==pData2->sInfo.dwTimeAccess){iExit=0;break;}
		iExit=(pData1->sInfo.dwTimeAccess<pData2->sInfo.dwTimeAccess)? -1:1;
		break;

	case CFF_SORT_WRITE:									// Nach letztdem Schreibzugriff sortieren

		if(pData1->sInfo.dwTimeWrite==pData2->sInfo.dwTimeWrite){iExit=0;break;}
		iExit=(pData1->sInfo.dwTimeWrite<pData2->sInfo.dwTimeWrite)? -1:1;
		break;

	case CFL_SORT_ATTRIBUTE:								// Nach Dateiattributen sortieren

		iExit=(pData1->sInfo.dwAttr&0x01FF)-(pData2->sInfo.dwAttr&0x01FF);
		break;

	case CFF_SORT_PATH:										// Nach dem Dateipfad sortieren

		if(pData1->pPath==pData2->pPath){iExit=0;break;}

		if(iFlags&CFF_SORT_VALUE)
				iExit=strvicmp(pData1->pPath,pData2->pPath);
		else	iExit=strzicmp(pData1->pPath,pData2->pPath);
		break;

	case CFF_SORT_DATA:										// Nach Dateilnge sortieren

		if(pData1->pUserPtr==pData2->pUserPtr){iExit=0;break;}
		iExit=(pData1->pUserPtr<pData2->pUserPtr)? -1:1;
		}


//*****************************************************************************


	if(iExit==0)											// Vergleich beendet ?
		{
		pName1 = pData1->sInfo.cName;
		pName2 = pData2->sInfo.cName;

		if(iFlags&CFF_SORT_VALUE)
				iExit  = strvicmp(pName1,pName2);
		else	iExit  = strzicmp(pName1,pName2);

		if(iExit==0 && iMode!=CFF_SORT_PATH)
		if(pData1->pPath!=pData2->pPath)
			{
			if(iFlags&CFF_SORT_VALUE)
					iExit=strvicmp(pData1->pPath,pData2->pPath);
			else	iExit=strzicmp(pData1->pPath,pData2->pPath);
			}
		}

	if(iFlags&CFF_SORT_INVERT)
		{
		iExit = -iExit;
		}


return iExit;
}


//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiert die aktuelle Dateieliste nach einer gewissen Art
//	Fr Next() wird der Auslesezeiger zurckgesetzt
//	iMode	: Besimmt nch welchem Kriterium sortiert werden soll
//				CFF_SORT_NAME		Nach Namen sortieren
//				CFF_SORT_TYPE		Nach Typ sortieren
//				CFF_SORT_LEN		Nach Datei lnge sortieren
//				CFF_SORT_CREATE		Nach Erstellungs Zeit sortieren
//				CFF_SORT_ACCCESS	Nach letztdem Zugriff sortieren
//				CFF_SORT_WRITE		Nach letztdem Schreibzugriff  sortieren
//				CFF_SORT_PATH		Nach Pfad sortieren
//				CFF_SORT_DATA		Nach dem Datenwert sortieren
//	iFlags	: Weitere Sortieroptionen
//				CFF_SORT_VALUE		Bercksichtigt Zahlen in den Texten
//				CFF_SORT_NODIR		Verzeichnisse nicht an den Anfang
//				CFF_SORT_INVERT		Abwrts sortieren
void CFileFinder::Sort(int iMode,int iFlags)
{


	m_iSortMode	 = iMode & 0xFF;
	m_iSortMode |= iFlags&~0xFF;

	if(m_sFiles.Count()<=1)
		{
		Reset();
		return;
		}

	if(m_pChangeCb)
		{
		m_pChangeCb(m_pCbParam,0,m_sFiles.Count(),m_uCbSort^m_uCbLock);
		}

	m_sFiles.SetUserPtr(this);
	m_sFiles.Sort(0,-1,SortProc);

	Reset();

	if(m_pChangeCb)
		{
		m_pChangeCb(m_pCbParam,0,m_sFiles.Count(),m_uCbSort);
		}

}

//*****************************************************************************
//*
//*		Filter
//*
//*****************************************************************************
//	Alle Dateinamen der aktuellen Liste konvertieren.
//	iMode	: Bestimmt wie die Dateinamen konvertiert werden sollen.
//				CFF_FILTER_BIG			0		Alles gros  schreiben
//				CFF_FILTER_LITTLE		1		Alles klein schreiben
//				CFF_FILTER_BEGIN		2		Anfangsbuchstaben gro schreiben
//				CFF_FILTER_PATH    		0x2000	Auch alle Pfade Filtern
//				CFF_FILTER_ONLYBIG		0x8000	Nur wenn alles gro  ist filtern
//				CFF_FILTER_ONLYLITTLE	0x4000	Nur wenn alles klein ist filtern
//				CFF_FILTER_ONLYSAME		0xC000	Nur wenn alles klein ist filtern
//				CFF_FILTER_LETLITTLE	0x1000	Kleingeschriebene Worte kleinlassen
void CFileFinder::Filter(int iMode)
{
zchar	*pName;
int		 iPos;


	iMode &= ~SFT_ANSI;
	iPos   = m_sFiles.Count();

	for(iPos--;iPos>=0;iPos--)
		{
		pName = m_sFiles[iPos];
		strfilter(pName,iMode);
		}

	if(iMode&CFF_FILTER_PATH)
		{
		if(!(iMode&SFT_SCANWORDS))
			{
			iMode|=SFT_SCANPATH;
			}

		iPos = m_sFolders.Count();

		for(iPos--;iPos>=0;iPos--)
			{
			pName=m_sFolders[iPos];
			strfilter(pName,iMode);
			}

		strfilter(m_pFilePath,iMode);
		m_iPathNum = -1;
		}


}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	ndert einen Dateinamen in der aktuellen Liste.
//	Der Dateiname der Datei am Laufwerk wird dabei nicht verndert.
//	iNr		: Ist die Nummer der Datei in der Liste. (-1 fr aktuelle Datei)
//	pName	: Ist der neue Name fr die Datei
void CFileFinder::Rename(int iNr,const zchar *pName)
{
FileDataStr    *pData;


	if(iNr==-1)
		{													// Aktuelle Nummer whlen
		iNr = m_iCurrentNum-1;
		}

	if(!m_sFiles.Check(iNr))								// Index-Nummer prfen
		{
		return;
		}


	pData				= DataNone(iNr);
	pData->pTypeName	=  0;
	pData->iSuffixPos	= -1;
	pData->iIconNumS	= -1;
	pData->iIconNumL	= -1;
	m_iSortMode			= -1;


	if(m_pChangeCb)m_pChangeCb(m_pCbParam,iNr,1,m_uCbChange^m_uCbLock);
	m_sFiles.Set(pName,iNr);
	if(m_pChangeCb)m_pChangeCb(m_pCbParam,iNr,1,m_uCbChange);

}

//*****************************************************************************
//*
//*		InternalDel
//*
//*****************************************************************************
//	Lscht einen Dateinamen aus der Liste.
//	iNr		: Ist die Nummer der Datei in der Liste. (-1 fr aktuelle Datei)
//	Ergibt 1 wenn der Dateinamen aus der Liste entfernt wurde.
//	Tritt ein Fehler auf so wird 0 zurck gegeben
int CFileFinder::InternalDel(int iNr)
{
int		iRet;


	if(m_sFiles.Check(iNr)==0)								// Index-Nummer prfen
		{
		return 0;
		}

	if(m_pChangeCb)
		{
		m_pChangeCb(m_pCbParam,iNr,1,m_uCbDelete^m_uCbLock);
		}

	iRet = m_sFiles.Del(iNr);								// Eintrag lschen

	if(m_pChangeCb)
		{
		m_pChangeCb(m_pCbParam,iNr,(iRet)? 1:0,m_uCbDelete);
		}

	if(iRet==0)												// Gab es einen Fehler
		{
		return 0;
		}

	if(m_iCurrentNum>iNr)
		{
		m_iCurrentNum--;
		}

	if(m_sFiles.Count()==0)									// Auch alle Verzeichnisse lschen
		{
		m_sFolders.Del();
		m_iPathNum = -1;
		}


return 1;
}

//*****************************************************************************
//*
//*		DelAll
//*
//*****************************************************************************
//	Lscht alle Dateinamen aus der Liste.
//	Ergibt 1 wenn Dateien gelscht wurden
int CFileFinder::DelAll()
{
int		iCount;


		iCount = m_sFiles.Count();
	if(!iCount)												// Liste ist leer
		{
		return 0;
		}

	if(m_pChangeCb)
		{
		m_pChangeCb(m_pCbParam,0,iCount,m_uCbDelete^m_uCbLock);
		}

	m_sFiles.Del();											// Alle Eintrge lschen

	if(m_pChangeCb)
		{
		m_pChangeCb(m_pCbParam,0,iCount,m_uCbDelete);
		}

	m_iCurrentNum = 0;
	m_sFolders.Del();										// Pfade lschen
	m_iPathNum = -1;


return 1;
}

//*****************************************************************************
//*
//*		SetCallback
//*
//*****************************************************************************
//	Callback-Funktion fr nderungen einstellen:
//	int _stdcall CbFunction(void *pParam,uint uItem,int iCount,uint uAction)
//	pProc			: Callback fr nderungen
//	pParam			: Parameter fr Callback
//	uInsert			: Aktion frs Einfgen
//	uDelete			: Aktion frs Lschen
//	uSort			: Aktion frs Sortieren
//	uChange			: Aktion frs ndern
//	uLock			: Aktion frs Sperren (uAction=uLock^....)
void CFileFinder::SetCallback(FfChangeCb pProc,void *pParam,unsigned uInsert,unsigned uDelete,unsigned uSort,unsigned uChange,unsigned uLock)
{

	m_pChangeCb	= pProc;
	m_pCbParam	= pParam;
	m_uCbInsert	= uInsert;
	m_uCbDelete	= uDelete;
	m_uCbChange	= uChange;
	m_uCbSort	= uSort;
	m_uCbLock	= uLock;

}

//*****************************************************************************
//*
//*		GetIcon
//*
//*****************************************************************************
//	Die System-Icon-Nummer zu einer Datei abfragen ber die Dateiinfos.
//	pInfo	: Zeiger auf die Dateiinfos
//	iMode	: SHGFI_LARGEICON		Groe Icons
//			  SHGFI_SMALLICON		Kleines Icon
//			  SHGFI_OPENICON		Icon geffnet
//	Ergibt die Iconnummer bzw. -1 bei einem Fehler
#ifdef	WIN32
int	CFileFinder::GetIcon(FfStr *pInfo,int iMode)const
{
zchar			cTemp[MAX_PATH];
SHFILEINFO		sSfi;
FileDataStr    *pData;
zchar		   *pName;
zchar		   *pPath;
void 		   *pUser;
void		   *pIcon;
int				iAttr;
int				iVal;
int				iRet;
int				iLen;
int				iPos;


	if(!pInfo)												// Zeiger auf die Dateiinfos prfen
		{
		return -1;
		}

	pUser = InfoToUser(pInfo);
	pData = InfoToData(pInfo);


	if(iMode&SHGFI_SMALLICON)								// Ist das Icon schon gespeichert ?
		{
		if(pData->iIconNumS!=-1)return pData->iIconNumS;
		}
	else{
		if(pData->iIconNumL!=-1)return pData->iIconNumL;
		}

	#ifndef CFF_NONET
	if(m_iFindFlags&CFF_FIND_NET)							// Ist der Pfad ein Netzlaufwerk
		{
		CShItem	   *pShi;
		int			iIcon;

		if(pData->iShItemNum>=0)
			{
			if(pData->iShItemNum>=m_sShItems.Count())return -1;

			pShi  = (CShItem*)m_sShItems.DataGetPtr(pData->iShItemNum);
			iIcon = pShi->GetIcon(iMode&~SHGFI_USEFILEATTRIBUTES);

			if(pData!=m_pTempData)
				{
				if(iMode&SHGFI_SMALLICON)
						pData->iIconNumS = iIcon;
				else	pData->iIconNumL = iIcon;
				}

			return iIcon;
			}
		}
	#endif

	pIcon = 0;
	pName = pInfo->cName;
	iLen  = UserToLength(pUser);

	if(pData->sInfo.dwAttr&SYS_DRIVE)						// Ist der Pfad ein Laufwerk
		{
		   sSfi.iIcon = CIconChache::GetDriveIcon(*pName);
		if(sSfi.iIcon<=-2)
			{
			memcpy(cTemp,pName,iLen*sizeof(zchar));			// Es ist ein Netzlaufwerk
			cTemp[iLen  ] = '\\';
			cTemp[iLen+1] = 0;

			if(!SHGetFileInfo(cTemp,FILE_ATTRIBUTE_DIRECTORY,&sSfi,sizeof(sSfi),SHGFI_SYSICONINDEX|SHGFI_USEFILEATTRIBUTES))
				{
				return -1;
				}
			}

		if(pData!=m_pTempData)
			{
			if(iMode&SHGFI_SMALLICON)
					pData->iIconNumS = sSfi.iIcon;
			else	pData->iIconNumL = sSfi.iIcon;
			}

		return sSfi.iIcon;
		}

	if(pData->sInfo.dwAttr&SYS_DIR)							// Bei Verzeichnissen immer ganzen Namen
		{
		iMode &= ~SHGFI_USEFILEATTRIBUTES;
		}

	if((iMode&SHGFI_USEFILEATTRIBUTES) && iLen>4)			// Spezielle Icons bei *.exe *.ico *.htm *.html *.lnk
		{
		#ifdef UNICODE
		iVal  = ((BYTE)pName[iLen-4]);
		iVal |= ((BYTE)pName[iLen-3])<<8;
		iVal |= ((BYTE)pName[iLen-2])<<16;
		iVal |= ((BYTE)pName[iLen-1])<<24;
		#else
		iVal = *(int*)(pName+iLen-4)&~0x20202020;
		#endif

		if(iVal==0x4558450E || iVal==0x4F43490E || iVal==0x4D54480E || iVal==0x4C4D5448 || iVal==0x4B4E4C0E )
			{
			iMode &= ~SHGFI_USEFILEATTRIBUTES;
			}
		}

	if(!(iMode&SHGFI_USEFILEATTRIBUTES))					// Ganzen Pfad als Name benutzen
		{
		pPath = Path(pInfo,0,&iVal);
		memcpy(cTemp,pPath,iVal*sizeof(zchar));

		if(iLen==2 && pInfo->cName[0]=='.' && pInfo->cName[1]=='.')
			{
			if(iVal>0)iVal--;
			pName = cTemp;
			iLen  = iVal;
			cTemp  [iVal]=0;
			}
		else{
			memcpy(cTemp+iVal,pName,(iLen+1)*sizeof(zchar));
			pName =cTemp;
			iLen += iVal;
			}
		}
	else{
		iVal  = 0;
		}

	if(iMode&SHGFI_USEFILEATTRIBUTES)						// Spezielle Icons bei *.exe *.ico *.htm *.html *.lnk
		{
		if(pData->iSuffixPos>=0)
			{
			iPos = iVal+pData->iSuffixPos;
			}
		else{
			for(iPos=iLen-1;;iPos--)
				{
				if(iPos<=0          ){iPos=iLen;break;}
				if(pName[iPos]=='.' ){iPos+=1  ;break;}
				if(pName[iPos]=='/' ){iPos=iLen;break;}
				if(pName[iPos]=='\\'){iPos=iLen;break;}
				}

			pData->iSuffixPos = iPos-iVal;
			}

		   iRet = sIconCache.GetIcon(pName+iPos,iMode&3,&pIcon);
		if(iRet>=0)
			{
			if(pData!=m_pTempData)
				{
				if(iMode&SHGFI_SMALLICON)
						pData->iIconNumS = iRet;
				else	pData->iIconNumL = iRet;
				}

			return iRet;
			}
		}


	iAttr=(pData->sInfo.dwAttr&SYS_WRITE)?  0:FILE_ATTRIBUTE_READONLY;
	if(pData->sInfo.dwAttr&SYS_HIDDEN )iAttr|=FILE_ATTRIBUTE_HIDDEN;
	if(pData->sInfo.dwAttr&SYS_SYSTEM )iAttr|=FILE_ATTRIBUTE_SYSTEM;
	if(pData->sInfo.dwAttr&SYS_ARCHIVE)iAttr|=FILE_ATTRIBUTE_ARCHIVE;
	if(pData->sInfo.dwAttr&SYS_DIR    )iAttr|=FILE_ATTRIBUTE_DIRECTORY;


	if(SHGetFileInfo(pName,iAttr,&sSfi,sizeof(sSfi),SHGFI_SYSICONINDEX|iMode))
		{
		if(pData!=m_pTempData)
			{
			if(iMode&SHGFI_SMALLICON)
					pData->iIconNumS = sSfi.iIcon;
			else	pData->iIconNumL = sSfi.iIcon;
			}

		if(pIcon)sIconCache.SetIcon(iMode&3,sSfi.iIcon,pIcon);

		return sSfi.iIcon;
		}



return -1;
}
#endif

//*****************************************************************************
//*
//*		GetType
//*
//*****************************************************************************
//	Die System-Dateityp zu einer Datei abfragen ber die Dateiinfos.
//	pInfo	: Zeiger auf die Dateiinfos
//	iMode	: SHGFI_USEFILEATTRIBUTES Dateiatribute benutzen
//	Ergibt einen Zeiger auf den Typnamen bzw. 0 bei einem Fehler
#ifdef	WIN32
zchar *CFileFinder::GetType(FfStr *pInfo,int iMode)
{
zchar			cTemp[MAX_PATH];
SHFILEINFO		sSfi;
FileDataStr    *pData;
zchar		   *pName;
zchar		   *pText;
zchar		   *pPath;
void 		   *pUser;
void		   *pIcon;
int				iAttr;
int				iVal;
int				iLen;
int				iPos;


	if(!pInfo)												// Zeiger auf die Dateiinfos prfen
		{
		return 0;
		}

	pData = InfoToData(pInfo);

	if(pData->pTypeName)									// Ist das Icon schon gespeichert ?
		{
		return pData->pTypeName;
		}

	#ifndef CFF_NONET
	if(m_iFindFlags&CFF_FIND_NET)							// Ist der Pfad ein Netzlaufwerk
		{
		if(pData->iShItemNum>=0)
			{
			return 0;
			}
		}
	#endif

	pIcon = 0;
	pName = pInfo->cName;
	pUser = InfoToUser(pInfo);
	iLen  = UserToLength(pUser);

	if(pData->sInfo.dwAttr&SYS_DRIVE)						// Ist der Pfad ein Laufwerk
		{
		memcpy(cTemp,pName,iLen*sizeof(zchar));

		cTemp[iLen  ] = '\\';
		cTemp[iLen+1] = '\\';
		cTemp[iLen+2] = 0;

		if(!SHGetFileInfo(cTemp,FILE_ATTRIBUTE_DIRECTORY,&sSfi,sizeof(sSfi),SHGFI_TYPENAME|SHGFI_USEFILEATTRIBUTES))
			{
			return 0;
			}

		BufferNext(sSfi.szTypeName);

		return 0;
		}

	if(pData->sInfo.dwAttr&SYS_DIR)							// Bei Verzeichnissen immer ganzen Namen
		{
		iMode &= ~SHGFI_USEFILEATTRIBUTES;
		}


	if(!(iMode&SHGFI_USEFILEATTRIBUTES))					// Ganzen Pfad als Name benutzen
		{
		pPath = Path(pInfo,0,&iVal);
		memcpy(cTemp     ,pPath,(iVal  )*sizeof(zchar));
		memcpy(cTemp+iVal,pName,(iLen+1)*sizeof(zchar));
		pName =cTemp;
		iLen += iVal;
		}
	else{
		iVal  = 0;
		}

	if(iMode&SHGFI_USEFILEATTRIBUTES)						// Aus Chache holen
		{
		if(pData->iSuffixPos>=0)
			{
			iPos = iVal+pData->iSuffixPos;
			}
		else{
			for(iPos=iLen-1;;iPos--)
				{
				if(iPos<=0          ){iPos=iLen;break;}
				if(pName[iPos]=='.' ){iPos+=1  ;break;}
				if(pName[iPos]=='/' ){iPos=iLen;break;}
				if(pName[iPos]=='\\'){iPos=iLen;break;}
				}

			if(pData!=m_pTempData)
				{
				pData->iSuffixPos = iPos-iVal;
				}
			}

		   pText = sIconCache.GetType(pName+iPos,&pIcon);
		if(pText)
			{
			if(pData!=m_pTempData)
				{
				pData->pTypeName = pText;
				}

			return pText;
			}
		}


	iAttr=(pData->sInfo.dwAttr&SYS_WRITE)?  0:FILE_ATTRIBUTE_READONLY;
	if(pData->sInfo.dwAttr&SYS_HIDDEN )iAttr|=FILE_ATTRIBUTE_HIDDEN;
	if(pData->sInfo.dwAttr&SYS_SYSTEM )iAttr|=FILE_ATTRIBUTE_SYSTEM;
	if(pData->sInfo.dwAttr&SYS_ARCHIVE)iAttr|=FILE_ATTRIBUTE_ARCHIVE;
	if(pData->sInfo.dwAttr&SYS_DIR    )iAttr|=FILE_ATTRIBUTE_DIRECTORY;


	if(!SHGetFileInfo(pName,iAttr,&sSfi,sizeof(sSfi),SHGFI_TYPENAME|iMode))
		{
		return 0;
		}

	if(pIcon)
		{
		   pText = sIconCache.SetType(sSfi.szTypeName,pIcon);
		if(pText)
			{
			if(pData!=m_pTempData)
				{
				pData->pTypeName = pText;
				}

			return pText;
			}
		}



return BufferNext(sSfi.szTypeName);
}
#endif

//*****************************************************************************
//*
//*		BufferNext
//*
//*****************************************************************************
//  Der Pfad in m_sFolders mit der Nummer iFolderPos wird in den
//	Hilfsspeicher m_pTempPath kopiert
zchar *CFileFinder::BufferNext(const zchar *pText)
{
zchar	   *pPtr;
unsigned	uLen;


	if(!m_pTempPath)
		{
		m_pTempPath = new   zchar[CFF_PATH_MAX+256+256];
		m_pBuffer	= m_pTempPath+CFF_PATH_MAX+256;
		}

	   uLen = strzlen(pText);
	if(uLen>64)uLen=64;


	if(m_uBufferPos+uLen>256)
		{
		m_uBufferPos = 0;
		}


	pPtr = m_pFilePath+m_uBufferPos;
	memcpy(pPtr,pText,uLen*sizeof(zchar));
	m_uBufferPos += uLen+1;
	pPtr[uLen] = 0;



return pPtr;
}

//*****************************************************************************
//*
//*		BufferCopy
//*
//*****************************************************************************
//  Der Pfad in m_sFolders mit der Nummer iFolderPos wird in den
//	Hilfsspeicher m_pTempPath kopiert
void CFileFinder::BufferCopy(int iFolderPos)
{
zchar	*m_pFilePath;


	if(m_iPathNum==iFolderPos)return;

	if(!m_pTempPath)
		{
		m_pTempPath = new   zchar[CFF_PATH_MAX+256+256];
		m_pBuffer	= m_pTempPath+CFF_PATH_MAX+256;
		}

	m_iPathNum  = iFolderPos;
	m_pFilePath = m_sFolders[iFolderPos];

	if(!m_pFilePath)
		{
		m_iPathLen		= 1;
		m_pTempPath[0]	= 0;

		return;
		}

	m_iPathLen = m_sFolders.LenNone(iFolderPos);
	memcpy(m_pTempPath,m_pFilePath,m_iPathLen+1);

}

//*****************************************************************************
//*
//*		AllocTempData
//*
//*****************************************************************************
//	Datenzeiger fr das Einfgen allokieren und befllen.
void CFileFinder::AllocTempData()
{

	m_pTempUser  = new char[m_iUserSize+sizeof(FileDataStr)+sizeof(int)*3];
	m_pTempUser	+= sizeof(int);
	m_pTempData  = UserToData(m_pTempUser);

	memset(m_pTempUser,0,m_iUserSize);

	m_pTempData->iSuffixPos	= -1;
	m_pTempData->iShItemNum	= -1;
	m_pTempData->iIconNumS 	= -1;
	m_pTempData->iIconNumL	= -1;
	m_pTempData->pTypeName	=  0;
	m_pTempData->pUserPtr	=  0;

}

//*****************************************************************************
//*
//*		Read
//*
//*****************************************************************************
//	Einen neuen Suchbereich festlegen und sofort mit der Suche beginnen.
//	pPath	: Ist der Ausgangspfad (z.B. "C:\ABC\") fr die Suche.
//			  Wenn am Pfadende ein Suchfilter (z.B ...\*.abc") angehngt ist,
//			  so wird dies zu den pPlus Filtern hinzu gefgt.
//			  Ist Pfad==":" so werden alle Laufwerke durchsucht
//			  Pfade die mit Verzeichnisen enden brauchen kein '\' am Ende.
//			  (z.B. "C:\WINDOWS\*.txt;*.log")
//	iFlags	: Optionen fr die Suche.
//				  CFF_FIND_DIR			Nur Verzeichnise suchen
//				  CFF_FIND_SUBDIR		Auch in Unterverz. suchen
//				  CFF_FIND_ADDDIR		Auch Verzeichnisse finden
//				  CFF_FIND_SUBDIRALL	Auch in Unterverz. und Verzeichnise
//				  CFF_FIND_SORT	  		Dateien sortiert einfgen
//				  CFF_FIND_ALL     		Alles auf Einmal lesen
//				  CFF_FIND_ADD     		Dateien zur den jetztigen anhngen
//				  CFF_FIND_ADDUP		Auch '..' einfgen
//				  CFF_FIND_PARTIAL   	Die Suche immer wieder abbrechen
//	pPlus	: Dateifilter fr die Dateien die gefunden werden sollen
//			  Form "*.c;*.h;abc*.???"
//	pMinus	: Dateifilter fr die Dateien die nicht gefunden werden sollen
//			  Form "*.c;*.h;abc*.???"
//	Ergibt die Anzahl der gefunden Dateien bzw. -1 am bei einem Fehler
int CFileFinder::Read(const zchar *pPath,int iFlags,const zchar *pPlus,const zchar *pMinus)
{
int				iCount;
int				iDir;
int				iLen;
int				iOld;



	m_cDoneFlag  = 0;
	m_cUserBreak = 0;

	if(pPath)												// Suchpfad einstellen
		{
		Set(pPath,iFlags,pPlus,pMinus);
		}

	if(m_pFilePath==0)
		{
		m_iLastError = CFF_ERROR_PATH;						// Kein Suchpfad vorhanden
		return -1;
		}

	if(m_pTempUser==0)										// Sind Temp-Daten vorhanden
		{
		AllocTempData();
		}

	iOld = m_sFiles.Count();


//******************** Parielle Suche *****************************************


	if(m_hNextFile)
		{
		iDir = m_uCurrentState&~0x80000000;

		ReadFolder(iDir);

		if(!m_hNextFile)
			{
			m_uCurrentState=1;
			}

		if(iOld!=m_sFiles.Count())							// Wurden Dateien gefunden
			{
			if(m_sFiles.Count()>1 && !(m_iFindFlags&CFF_FIND_SORT))
				{
				m_iSortMode = -1;
				}

			return m_sFiles.Count();
			}

		m_uCurrentState = 1;
		}


//**************** Erste Suche ************************************************


	if(m_uCurrentState==0)
		{
		m_iLastError = CFF_ERROR_OK;

		iLen = strzlen(m_pFilePath);

		if(iLen==1 && *m_pFilePath==':')					// Laufwerke suchen ?
			{
			iDir = m_sFolders.Add(Z(""),CSC_END,MINUS_MEM);

			ReadDrives(iDir);
			}
		else{												// Suche bei Pfad beginnen
			#ifndef CFF_NONET
			if((m_iFindFlags&CFF_FIND_NET)		  &&
			   (m_pFilePath[0]=='\\' || m_pFilePath[0]=='/') &&
			   (m_pFilePath[1]=='\\' || m_pFilePath[1]=='/'))
				{
				iDir = InitNet(m_pFilePath);
				}
			else
			#endif
				{
				iDir = m_sFolders.Add(m_pFilePath,CSC_END,MINUS_MEM);
				}

			ReadFolder(iDir);
			}

		if(m_hNextFile)										// Parielle Suche
			{
			m_uCurrentState	= iDir|0x80000000;
			}
		else{
			m_iCurrentNum   = 0;
			m_uCurrentState = 1;

			if(m_iFindFlags&CFF_FIND_ALL)					// Alle Verzeichnisse einlesen
				{
				while(m_iFolderNum+1<m_sFolders.Count())
					{
					if(m_cUserBreak)
						{
						m_cDoneFlag = 2;
						break;
						}

					m_iFolderNum++;
					ReadFolder(m_iFolderNum);
					}

				m_uCurrentState = 1;
				}
			}

		if(iOld!=m_sFiles.Count() && m_sFiles.Count()>1 && !(m_iFindFlags&CFF_FIND_SORT))
			{
			m_iSortMode = -1;
			}

		return m_sFiles.Count();
		}


//**************** Nchste Suche ***********************************************


	if(m_uCurrentState==1)
		{
		if(m_iFolderNum+1>=m_sFolders.Count())				// Alle Verzeichnisse durch sucht ?
			{
			m_iLastError    = CFF_ERROR_NULL;
			m_uCurrentState = 0xF0000000;

			return -1;
			}

		iCount = m_sFiles.Count();

		if(m_pChangeCb && iCount)
			{
			m_pChangeCb(m_pCbParam,0,iCount,m_uCbDelete^m_uCbLock);
			}

		m_sFiles  .Del( );									// Aktuelles Verzeichnis lschen
		m_sFolders.Del(0);

		if(m_pChangeCb && iCount)
			{
			m_pChangeCb(m_pCbParam,0,iCount,m_uCbDelete);
			}

		m_iPathNum		= -1;
		ReadFolder(m_iFolderNum);							// Nchstes Verzeichnis

		m_iLastError    = CFF_ERROR_OK;
		m_uCurrentState = 1;
		m_iCurrentNum   = 0;

		if(iOld!=m_sFiles.Count() && m_sFiles.Count()>1 && !(m_iFindFlags&CFF_FIND_SORT))
			{
			m_iSortMode = -1;
			}

		return	m_sFiles.Count();
		}

	m_iLastError = CFF_ERROR_NULL;



return -1;
}


//*****************************************************************************
//*
//*		ReadFolder
//*
//*****************************************************************************
//  Liest fr ein Unterverzeichnis alle Dateien ein
//  Die gefunden Dateien werden bei m_sFiles angehngt
//  Die gefunden Verzeichnisse  werden bei m_sFolders angehngt
//	Der Pfad wird in den Hilfsspeicher m_pTempPath kopiert
//  iFolderPos	: Ist die Nummer des Verzeichnisses in m_sFolders
//  Ergibt die Anzahl der gefundenen Dateien.
int CFileFinder::ReadFolder(int iFolderPos)
{
zchar		   *pPath;
FfFind			hFile;
int				iMulti;
int				iPlace;
int 			iStart;
int				iTurn;
int				iLen;
int				iNum;
int				iPos;



	iMulti = 0;

	if(m_hNextFile)												// Die Suche vortfhren
		{
		BufferCopy(iFolderPos);

		pPath = m_sFolders.GetNone(iFolderPos);

		m_pTempData->pPath		=  pPath;
		m_pTempData->iFolderNum	=  iFolderPos;
		m_pTempData->iShItemNum	= -1;

		if(m_iFindFlags&CFF_FIND_ONE)							// Ein oder mehrere Filter
				strzcpy(m_pTempPath+m_iPathLen,m_sPlus.GetNone(0));
		else	strzcpy(m_pTempPath+m_iPathLen,Z("*"));

		hFile		= m_hNextFile;
		m_hNextFile	= 0;
		iStart		= 1;

		goto Start;
		}


	#ifndef CFF_NONET
	if(m_iFindFlags&CFF_FIND_NET)								// Im Netz suchen ?
		{
		if(m_sFolders.DataGet(iFolderPos)>=0)
			{
			return ReadNet(iFolderPos);
			}

		pPath = m_sFolders.GetNone(iFolderPos);

		if(pPath[0]=='\\' && pPath[1]=='\\')
			{
			iLen = m_sFolders.LenNone(iFolderPos);
			iNum = strcwlen(pPath+2,0x00300000)+2;

			if(iNum+1>=iLen)
				{
				return ReadNetPath(iFolderPos);
				}
			}
		}
	else
	#endif
		{
		pPath = m_sFolders.GetNone(iFolderPos);
		}

	if(m_iStartFolderNum>0 && m_iStartFolderNum<=iFolderPos)	// Doppeltes Verzeichnis
	if(m_iFindFlags&CFF_FIND_ADD)
		{
		iPos = m_sFolders.Find(pPath,0,m_iStartFolderNum,CSC_MASK_TEST);

		if(iPos>=0)
			{
			m_sFolders.Del(iFolderPos);

			if(m_iPathNum>=iFolderPos)
				{
				m_iPathNum	= -1;
				}

			pPath			=  m_sFolders[iPos];
			iFolderPos		=  iPos;
			iMulti			=  1;
			m_iFolderNum   -=  1;
			}
		}



	BufferCopy(iFolderPos);

	m_pTempData->pPath		=  pPath;
	m_pTempData->iFolderNum	=  iFolderPos;
	m_pTempData->iShItemNum	= -1;

	if(m_iFindFlags&CFF_FIND_ONE)								// Ein oder mehrere Filter
			strzcpy(m_pTempPath+m_iPathLen,m_sPlus.GetNone(0));
	else	strzcpy(m_pTempPath+m_iPathLen,Z("*"));

	   hFile=FfFindFirst(m_pTempPath,&m_pTempData->sInfo);		// Erste Datei suchen
	if(hFile==0)
		{
		return 0;
		}

	iStart = 0;

//******************** Verzeichnis einlesen ***********************************
Start:

	iTurn  = 0;
	iNum   = 0;

	for(;;)
		{
		if(m_cUserBreak)
			{
			m_cDoneFlag = 2;
			break;
			}

   		if(iStart && FfFindNext(hFile,&m_pTempData->sInfo))		// Nchste Datei suchen
			{
			break;
			}

		iStart = 1;
		iTurn += 1;

		if(m_pTempData->sInfo.dwAttr&SYS_DIR)					// Verzeichnis gefunden ?
	  		{
			iLen = strzlen(m_pTempData->sInfo.cName);


			if(iLen<=2 && m_pTempData->sInfo.cName[0]=='.')		// "." und ".." berspringen
				{
				if(m_pTempData->sInfo.cName[1]== 0 )continue;
				if(m_pTempData->sInfo.cName[1]=='.')
				if(m_pTempData->sInfo.cName[2]== 0 )
					{
					if(!(m_iFindFlags&CFF_FIND_ADDUP ))continue;
					if(!(m_iFindFlags&CFF_FIND_ADDDIR))continue;

					UserLength(m_pTempUser,iLen);

					iPlace = InsertPos();
					if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert^m_uCbLock);
					m_sFiles.Add(iLen,m_pTempData->sInfo.cName,iPlace  ,m_pTempUser);
					if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert);

					iNum++;

					continue;
					}
				}

			if(m_iFindFlags&CFF_FIND_SUBDIR)					// Auch in Unterverzeichnissen suchen
				{
				memcpy (m_pTempPath+m_iPathLen,m_pTempData->sInfo.cName,iLen);
				*(int*)(m_pTempPath+m_iPathLen+iLen) = *(int*)SLASH_TXT;
				m_sFolders.Add(m_iPathLen+iLen+1,m_pTempPath,CSC_END,MINUS_MEM);
				}

			if(m_iFindFlags&CFF_FIND_ADDDIR)					// Auch Verzeichnisse suchen
				{
				if(m_iFindFlags&CFF_FIND_DIRCMP)				// Auch Verzeichnisnamen prfen
					{
					iLen = strzlen(m_pTempData->sInfo.cName);

					if(!CheckFilter(m_pTempData->sInfo.cName,iLen))continue;
					}

				if(iMulti && ExistFile(m_pTempData->sInfo.cName,iFolderPos,iLen)>=0)
					{
					continue;									// Doppelte Dateinamen
					}

				UserLength(m_pTempUser,iLen);

				iPlace = InsertPos();
				if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert^m_uCbLock);
				m_sFiles.Add(iLen,m_pTempData->sInfo.cName,iPlace  ,m_pTempUser);
				if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert);

				iNum++;

				if(m_iFindFlags&CFF_FIND_FIRST)					// Nur die erste Datei finden
					{
					m_cDoneFlag = 1;
					break;
					}

				continue;
				}

			if(!(m_iFindFlags&CFF_FIND_DIR))
				{
				continue;
				}
			}
		else{
			if(m_iRdWrFilter)									// Via Read-Only-Flag filtern
				{
				if(m_iRdWrFilter==1)
					{if(  m_pTempData->sInfo.dwAttr&SYS_WRITE )continue;}
				else{if(!(m_pTempData->sInfo.dwAttr&SYS_WRITE))continue;}
				}

			if(m_iOneFilter)									// Attribut Eins-Aus-Filter
				{
				if(!(m_pTempData->sInfo.dwAttr&m_iOneFilter))continue;
				}

			if(m_iAttrFilter)									// Attribut Muss-Filter
				{
				if((m_pTempData->sInfo.dwAttr^m_iMustFilter)&m_iAttrFilter)continue;
				}

			if(m_iTimeFilter)									// Ist ein Zeitfiler vorhanden
				{
				switch(m_iTimeFilter)
					{
				case 1:											// 1=erstellt vor
						if(m_pTempData->sInfo.dwTimeCreate>(FfUtc)m_iTimeValue)continue;
						break;
				case 2:											// 2=gendert vor
						if(m_pTempData->sInfo.dwTimeWrite >(FfUtc)m_iTimeValue)continue;
						break;
				case 3:											// 3=erstellt nach
						if(m_pTempData->sInfo.dwTimeCreate<(FfUtc)m_iTimeValue)continue;
						break;
				case 4:											// 4=gendert nach
						if(m_pTempData->sInfo.dwTimeWrite <(FfUtc)m_iTimeValue)continue;
						break;
				case 5:											// 5=erstellt von/bis
						if(m_pTempData->sInfo.dwTimeCreate<(FfUtc)m_iTimeValue)continue;
						if(m_pTempData->sInfo.dwTimeCreate>(FfUtc)m_iTimeUntil)continue;
						break;
				case 6:											// 6=gendert von/bis
						if(m_pTempData->sInfo.dwTimeWrite <(FfUtc)m_iTimeValue)continue;
						if(m_pTempData->sInfo.dwTimeWrite >(FfUtc)m_iTimeUntil)continue;
						break;
				case 7:											// 7=erstellt vor/nach
						if(m_pTempData->sInfo.dwTimeCreate>(FfUtc)m_iTimeValue)break;
						if(m_pTempData->sInfo.dwTimeCreate<(FfUtc)m_iTimeUntil)break;
						continue;
				case 8:											// 8=gendert vor/nach
						if(m_pTempData->sInfo.dwTimeWrite >(FfUtc)m_iTimeValue)break;
						if(m_pTempData->sInfo.dwTimeWrite <(FfUtc)m_iTimeUntil)break;
						continue;
					}
				}

			if(m_iFindFlags&CFF_FIND_DIR)
				{
				continue;
				}
			}


		//***************** Dateinamen auswerten *******************************


		iLen = strzlen(m_pTempData->sInfo.cName);

		if(!CheckFilter(m_pTempData->sInfo.cName,iLen))continue;
																// Suche doppelte Dateinamen
		if(iMulti && ExistFile(m_pTempData->sInfo.cName,iFolderPos,iLen)>=0)
			{
			continue;
			}


		UserLength(m_pTempUser,iLen);

		iPlace = InsertPos();
		if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert^m_uCbLock);
		m_sFiles.Add(iLen,m_pTempData->sInfo.cName,iPlace  ,m_pTempUser);
		if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert);
		iNum++;

		if(m_iFindFlags&CFF_FIND_FIRST)							// Nur die erste Datei finden
			{
			m_cDoneFlag = 1;
			break;
			}

		if(m_iFindFlags&CFF_FIND_PARTIAL)						// Nur die erste Datei finden
			{
			if(iTurn<16)continue;
			m_hNextFile = hFile;
			return iNum;
			}
		}


	FfFindClose(hFile);


return iNum;
}

//*****************************************************************************
//*
//*		CheckFilter
//*
//*****************************************************************************
//	Prft ob ein Dateiname zum Filter passt
//	Ergibt 1 wenn der Filter passt
int	CFileFinder::CheckFilter(const zchar *pName,int iLen) const
{
zchar  *pText;
int		iMode;
int		iSize;
int		iPos;


	if(m_iFilerPlus)											// Plusfilter vergleichen
		{
		for(iPos=0;iPos<m_iFilerPlus;iPos++)
			{
			pText =        m_sPlus.GetNone (iPos);
			iSize =        m_sPlus.LenNone (iPos);
			iMode = *(int*)m_sPlus.DataNone(iPos);

			if(iMode)											// Vom Typ "*.xxx"
				{
				pText += iMode-1;
				iSize -= iMode-1;

				if(iSize>iLen)continue;
				if(!strzicmp(pName+iLen-iSize,pText))
					{
					break;
					}

				continue;
				}

			if(!memxcmp(pText,iSize,pName,iLen,1))
				{
				break;
				}
			}

		if(iPos>=m_iFilerPlus)return 0;
		}

	if(m_iFilerMinus)											// Minusfilter vergleichen
		{
		for(iPos=0;iPos<m_iFilerMinus;iPos++)
			{
			pText =        m_sMinus.GetNone (iPos);
			iSize =        m_sMinus.LenNone (iPos);
			iMode = *(int*)m_sMinus.DataNone(iPos);

			if(iMode)											// Vom Typ "*.xxx"
				{
				pText += iMode-1;
				iSize -= iMode-1;

				if(iSize>iLen)continue;
				if(!strzicmp(pName+iLen-iSize,pText))
					{
					break;
					}

				continue;
				}

			if(!memxcmp(pText,iSize,pName,iLen,1))
				{
				break;
				}
			}

		if(iPos<m_iFilerMinus)return 0;
		}



return 1;
}

//*****************************************************************************
//*
//*		ReadDrives
//*
//*****************************************************************************
//	Liet alle Laufwerke ein, und
//	Ergibt die Anzahl der gefunden Laufwerke bzw. -1 am bei einem Fehler
int	CFileFinder::ReadDrives(int iFolder)
{
zchar		cText[4];
int			iPlace;
int			iCount;
int			iLen;


	m_pTempData->pPath				= m_sFolders[iFolder];
	m_pTempData->iFolderNum 		= iFolder;
	m_pTempData->iShItemNum			= -1;
	m_pTempData->sInfo.dwAttr		= SYS_DIR|SYS_DRIVE;
	m_pTempData->sInfo.dwSize		= 0;
	m_pTempData->sInfo.dwTimeCreate	= 0;
	m_pTempData->sInfo.dwTimeAccess	= 0;
	m_pTempData->sInfo.dwTimeWrite	= 0;
	m_iStartPathLen					= 0;
	iCount							= 0;

	UserLength(m_pTempUser,1);

	#ifdef LINUX

	if(m_iFindFlags&CFF_FIND_SUBDIR)
		{
		m_sFolders.Add(1,Z("/"),CSC_END,MINUS_MEM);
		}

	if(m_iFindFlags&(CFF_FIND_DIR|CFF_FIND_ADDDIR))
		{
		iPlace = InsertPos();
		if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert^m_uCbLock);
		m_sFiles.Add(1,Z("/"),cText          ,iPlace  ,m_pTempUser);
		if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert);

		iCount++;
		}

	#else

	unsigned	uBits;

	UserLength(m_pTempUser,2);

	uBits = CIconChache::GetDriveBits();

													// Laufwerke suchen
	for(iLen='A';uBits && iLen<='Z';iLen++,uBits>>=1)
		{
		if(m_cUserBreak)
			{
			m_cDoneFlag = 2;
			break;
			}

		if(!(uBits&1))								// Gibt es das Laufwerk
			{
			continue;
			}

		cText[0] = iLen;							// "A:\";
		cText[1] = ':';
		cText[2] = '\\';
		cText[3] =  0;

		if(m_iFindFlags&CFF_FIND_SUBDIR)
			{
			m_sFolders.Add(3,cText,CSC_END,MINUS_MEM);
			}
													// "A:"
		if(m_iFindFlags&(CFF_FIND_DIR|CFF_FIND_ADDDIR))
			{
			iPlace = InsertPos();
			if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert^m_uCbLock);
			m_sFiles.Add(2,cText                 ,iPlace  ,m_pTempUser);
			if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert);
			}

		iCount++;
		}

	#endif

	#ifndef CFF_NONET

	if(m_iFindFlags&CFF_FIND_NET)					// Auch Netzzugriff
		{
		zchar		cNetPath[MAX_PATH+1];
		CShFind    *pRoot;
		CShFind    *pShf;
		CShItem    *pShi;
		int			iPos;

		pRoot = new CShFind;
		pRoot->Init(CSIDL_NETWORK);
		iPos  = m_sShFinds.Add(0,Z(""),CSC_END,&pRoot);
		m_pTempData->sInfo.dwAttr = SYS_DIR|SYS_DRIVE|SYS_NET;

		if(m_iFindFlags&CFF_FIND_SUBDIR)
			{
			pShf = new CShFind;
			pShf ->Init(*pRoot);
			pRoot->Path(cNetPath);

			cNetPath[0] = '\\';
			cNetPath[1] = '\\';
			pRoot->Name(cNetPath+2);
			iPos = m_sShFinds.Add(0,Z(""),CSC_END,&pShf);
			m_sFolders.Add(SLASH_NET,CSC_END,&iPos);
			}

		if(m_iFindFlags&(CFF_FIND_DIR|CFF_FIND_ADDDIR))
			{
			pShi = new CShItem;
			pShi->Get(*pRoot);
			m_pTempData->sInfo.dwAttr = SYS_DIR|SYS_DRIVE|SYS_NET;
			m_pTempData->iShItemNum	  = m_sShItems.Add(0,Z(""),CSC_END,&pShi);
			UserLength(m_pTempUser,17);

			iPlace = InsertPos();
			if(m_pChangeCb)m_pChangeCb(m_pCbParam ,iPlace,1,m_uCbInsert^m_uCbLock);
			m_sFiles.Add(17,m_aNetBase            ,iPlace  ,m_pTempUser);
			if(m_pChangeCb)m_pChangeCb(m_pCbParam ,iPlace,1,m_uCbInsert);

			iCount++;
			}
		}

	#endif


return iCount;
}


//*****************************************************************************
//*
//*		ReadNet
//*
//*****************************************************************************
//  Liest fr ein Unterverzeichnis in einem Netzzweig alle Dateien ein
//	Pfade werden in den Hilfsspeicher m_pTempPath kopiert
//  Die gefunden Dateien werden bei m_sFiles angehngt
//  Die gefunden Verzeichnisse  werden bei m_sFolders angehngt
//  iFolderPos	: Ist die Nummer des Verzeichnisses in m_sFolders
//  Ergibt die Anzahl der gefundenen Dateien.
#ifndef CFF_NONET
int CFileFinder::ReadNet(int iFolderPos)
{
int				iCnt,iLen,iNum,iPos,iShfNr;
int				iTempNr=-1,iTempLen=-1,iMulti=0;
zchar		   *pPtr,*pDirPath,*pTempPath=0;
zchar			cNetPath[MAX_PATH];
zchar			cName[256];
int				iPlace;
CShFind		   *pBase;
CShFind		   *pShf;
CShItem		   *pShi;




	   iShfNr = m_sFolders.DataGet(iFolderPos);
	if(iShfNr>=iTempVal)return 0;								// Temporres Verzeichnis

	pBase	 = (CShFind*)m_sShFinds.DataGetPtr(iShfNr);
	pDirPath = m_sFolders.GetNone(iFolderPos);
	iNum	 = 0;


	if(m_iStartFolderNum>0 && m_iStartFolderNum<=iFolderPos)	// Doppeltes Verzeichnis
	if(m_iFindFlags&CFF_FIND_ADD)
		{
		   iPos=m_sFolders.Find(pDirPath,0,m_iStartFolderNum,CSC_MASK_TEST);
		if(iPos>=0)
			{
			m_sFolders.Del(iFolderPos);

			if(m_iPathNum>=iFolderPos)
				{
				m_iPathNum	= -1;
				}

			iFolderPos		= iPos;
			pDirPath		= m_sFolders[iPos];
			m_iFolderNum   -= 1;
			iMulti			= 1;
			}
		}


	BufferCopy(iFolderPos);

	m_pTempData->pPath		= pDirPath;
	m_pTempData->iFolderNum	= iFolderPos;
	m_pTempData->iShItemNum	= -1;

	for(;;)														// Einlese Schleife
		{
		if(m_cUserBreak)
			{
			m_cDoneFlag = 2;
			break;
			}

		if(pBase->Next(     )==0)break;
		if(pBase->Name(cName)==0)continue;
		if(pBase->IsFolder())									// Ordner gefunden ?
	  		{
			if(cName[0]=='.')									// "." und ".." berspringen
				{
				if(cName[1]== 0 )continue;
				if(cName[1]=='.')
				if(cName[2]== 0 )continue;
				}

			if(m_iFindFlags&CFF_FIND_SUBDIR)					// Auch in Unterverzeichnissen suchen
				{
				pShf = new CShFind;
				pShf ->Init(*pBase);
				pBase->Path(cNetPath);

				if(cNetPath[0])									// Realer Pfad ?
					{
					iPos = strzlen(cNetPath);
					cNetPath[iPos  ] = '\\';
					cNetPath[iPos+1] = 0;

					m_sFolders.Add(iPos+1,cNetPath,CSC_END,MINUS_MEM);

					cNetPath[iPos] = 0;

					delete pShf;
					}
				else{
					pBase->Name(m_pTempPath+m_iPathLen);
					strzcat(m_pTempPath+m_iPathLen,SLASH_TXT);
					iPos=m_sShFinds.Add(0,Z(""),CSC_END,&pShf);
					m_sFolders.Add(m_pTempPath ,CSC_END,&iPos);
					}
				}

			if(m_iFindFlags&CFF_FIND_ADDDIR)					// Auch Verzeichnisse suchen
				{
				if(pBase->IsFilesystem() && pBase->IsHasRealDirs())
					{
					m_pTempData->sInfo.dwAttr = 0;
					}
				else if(pBase->IsLink())
					{
					m_pTempData->sInfo.dwAttr = SYS_NET|SYS_ARCHIVE;
					}
				else{
					m_pTempData->sInfo.dwAttr = SYS_NET;
					}

				m_pTempData->sInfo.dwAttr      |= SYS_DIR;
				m_pTempData->sInfo.dwTimeCreate = 0;
				m_pTempData->sInfo.dwTimeAccess = 0;
				m_pTempData->sInfo.dwTimeWrite  = 0;
				m_pTempData->sInfo.dwSize       = 0;

				pShi = new CShItem;
				pShi->Get(*pBase);
				m_pTempData->iShItemNum = m_sShItems.Add(0,Z(""),CSC_END,&pShi);

				if(!(m_pTempData->sInfo.dwAttr&SYS_NET))		// Temorren Verzeichnisnamen erzeugen
					{
					pShi->Path(cNetPath);
					iLen = strzlen(cNetPath);

					for(iLen--;iLen>=0;iLen--)
						{
						if(cNetPath[iLen]=='\\')break;
						if(cNetPath[iLen]=='/' )break;
						}

					cNetPath[iLen+1] = 0;

					if(iTempNr<0 || iTempLen!=iLen || strzicmp(cNetPath,pTempPath))
						{
						iTempLen      = iLen;
						iTempNr		  = m_sFolders.Add(iLen+1,cNetPath,CSC_END,TEMP_MEM);
						pTempPath     = m_sFolders[iTempNr];
						m_iFindFlags |= CFF_FIND_TEMP;
						}

					m_pTempData->iFolderNum = iTempNr;
					m_pTempData->pPath		= pTempPath;
					}

				iLen = strzlen(cName);

				if(iMulti && ExistFile(cName,iFolderPos,iLen)>=0)// Suche doppelte Dateinamen
					{
					continue;
					}

				UserLength(m_pTempUser,iLen);

				iPlace = InsertPos();
				if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert^m_uCbLock);
				m_sFiles.Add(iLen,cName              ,iPlace  ,m_pTempUser);
				if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert);

				m_pTempData->iFolderNum	= iFolderPos;
				m_pTempData->pPath		= pDirPath;
				iNum++;

				continue;
				}

			if(!(m_iFindFlags&CFF_FIND_DIR))continue;
			}
		else{
			if(m_iFindFlags&CFF_FIND_DIR)continue;
			}

	//***************** Dateinamen auswerten *******************************

		iLen = strzlen(cName);


		if(m_iFilerPlus)										// Plusfilter vergleichen
			{
			for(iPos=0;iPos<m_iFilerPlus;iPos++)
				{
				pPtr = m_sPlus.GetNone(iPos);
				iCnt = m_sPlus.LenNone(iPos);

				if(!memxcmp(pPtr,iCnt,cName,iLen,1))
					{
					break;
					}
				}

			if(iPos>=m_iFilerPlus)continue;
			}

		if(m_iFilerMinus)										// Minusfilter vergleichen
			{
			for(iPos=0;iPos<m_iFilerMinus;iPos++)
				{
				pPtr = m_sMinus.GetNone(iPos);
				iCnt = m_sMinus.LenNone(iPos);

				if(!memxcmp(pPtr,iCnt,cName,iLen,1))
					{
					break;
					}
				}

			if(iPos<m_iFilerMinus)continue;
			}


		if(pBase->IsFilesystem() && pBase->IsHasRealDirs())
			{
			m_pTempData->sInfo.dwAttr = 0;
			}
		else if(pBase->IsLink())
			{
			m_pTempData->sInfo.dwAttr = SYS_NET|SYS_ARCHIVE;
			}
		else{
			m_pTempData->sInfo.dwAttr = SYS_NET;
			}

		if(pBase->IsFolder())
			{
			m_pTempData->sInfo.dwAttr |= SYS_DIR;
			}

		m_pTempData->sInfo.dwTimeCreate = 0;
		m_pTempData->sInfo.dwTimeAccess = 0;
		m_pTempData->sInfo.dwTimeWrite  = 0;
		m_pTempData->sInfo.dwSize       = 0;

		pShi = new CShItem;
		pShi->Get(*pBase);
		m_pTempData->iShItemNum = m_sShItems.Add(0,Z(""),CSC_END,&pShi);

		if(!(m_pTempData->sInfo.dwAttr&SYS_NET))				// Temporren Verzeichnisnamen erzeugen
			{
			pShi->Path(cNetPath);
			iCnt = strzlen(cNetPath);

			for(iCnt--;iCnt>=0;iCnt--)
				{
				if(cNetPath[iCnt]=='\\')break;
				if(cNetPath[iCnt]=='/' )break;
				}

			cNetPath[iCnt+1]=0;

			if(iTempNr<0 || iTempLen!=iCnt || strzicmp(cNetPath,pTempPath))
				{
				iTempLen      = iCnt;
				iTempNr       = m_sFolders.Add(iCnt+1,cNetPath,CSC_END,TEMP_MEM);
				pTempPath     = m_sFolders[iTempNr];
				m_iFindFlags |= CFF_FIND_TEMP;
				}

			m_pTempData->iFolderNum = iTempNr;
			m_pTempData->pPath	    = pTempPath;
			}
		else if(m_pTempData->sInfo.dwAttr&SYS_ARCHIVE)			// Ist es ein Link
			{
			pBase->Link(cNetPath);
			iCnt = strzlen(cNetPath);
			m_sShItems.Set(iCnt,cNetPath,m_pTempData->iShItemNum);
			}

		if(iMulti && ExistFile(cName,iFolderPos,iLen)>=0)		// Suche doppelte Dateinamen
			{
			continue;
			}

		UserLength(m_pTempUser,iLen);

		iPlace = InsertPos();
		if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert^m_uCbLock);
		m_sFiles.Add(iLen,cName              ,iPlace  ,m_pTempUser);
		if(m_pChangeCb)m_pChangeCb(m_pCbParam,iPlace,1,m_uCbInsert);

		m_pTempData->iFolderNum	= iFolderPos;
		m_pTempData->pPath		= pDirPath;
		iNum++;
		}



return iNum;
}
#endif

//*****************************************************************************
//*
//*		InitNet
//*
//*****************************************************************************
#ifndef CFF_NONET
int CFileFinder::InitNet(zchar *pPath)
{
zchar			cText[CFF_PATH_MAX];
zchar		   *pName;
FfFindStruct	sInfo;
unsigned		uNum;
int				iDir;



	uNum = strlcpy(cText,pPath,sizeof(cText)-16);
	memcpy(cText+uNum,"*.*",4);

	if(FfFileInfo(cText,&sInfo)>=0)								// Existiert das Verzeichnis
		{
		iDir = m_sFolders.Add(uNum,cText,CSC_END,MINUS_MEM);
		return iDir;
		};


//******************** Suche Netzzweig ****************************************
#define 	NDEPTH	6
CShFind	   *pShfN;
CShFind		pShf[NDEPTH];
int			aPos[NDEPTH];
int			iLevel=0;
int			iVal;


	if(uNum==2)													// Netzwerk
		{
 		pShfN = new CShFind;
		pShfN->Init(CSIDL_NETWORK);

		iVal = m_sShFinds.Add(2,SLASH_NET,CSC_END,&pShfN);
		iVal = m_sFolders.Add(pPath,      CSC_END,&iVal );

		return iVal;
		}


	aPos[0] = 2;
	pShf[0].Init(CSIDL_NETWORK);


	while(iLevel>=0)
		{
		pName = pPath+aPos[iLevel];

		while(pShf[iLevel].Next())								// Netz Root durchlaufen
			{
			if(m_cUserBreak)
				{
				m_cDoneFlag = 2;
				break;
				}

			if(!pShf[iLevel].Name(cText    ))continue;
			if( pShf[iLevel].IsFilesystem ())
			if( pShf[iLevel].IsHasRealDirs())continue;

			   uNum = strzlen(cText);
			if(uNum!=strzcspn(pName,Z("/\\")))continue;
			if(strznicmp(pName,cText,uNum   ))continue;

			if(pName[uNum] && pName[uNum+1])					// Auf nchste Ebene
				{
				iLevel++;

				if(iLevel<NDEPTH && pShf[iLevel].Init(pShf[iLevel-1]))
					{
					aPos[iLevel] = aPos[iLevel-1]+uNum+1;
					pName        = pPath+aPos[iLevel];
					continue;
					}

				iLevel--;
				continue;
				}

			if(pShf[iLevel].IsLink())							// Ist es ein Link auf eine Pfad
			if(pShf[iLevel].Link(cText))
				{
				uNum = strzlen(cText); cText[uNum]='\\';
				iVal = m_sFolders.Add(uNum+1,cText,CSC_END,MINUS_MEM);
				return iVal;
				}

			pShfN = new CShFind;								// Pfad gefunden
			pShfN->Init(pShf[iLevel]);

			iVal = m_sShFinds.Add(2,Z(""),CSC_END,&pShfN);
			iVal = m_sFolders.Add(pPath  ,CSC_END,&iVal );

			return iVal;
			}

		iLevel--;
		}

	iDir = m_sFolders.Add(pPath,CSC_END,MINUS_MEM);


return iDir;
}
#endif

//*****************************************************************************
//*
//*		Insert
//*
//*****************************************************************************
//  Einen Dateinamen einfgen. Das geht nur wenn nicht in
//  Unterverzeichnissen gesucht wurde !!!
//	pInfo	: Ist der Zeiger auf die Dateiinfos die eingefgt werden sollen.
//	iNr		: Ist die Einfgeposition. (-1 fr das Ende,-2 fr sortiert)
//	Ergibt die Listenposition wo die Datei eingefgt wurde bzw. -1 bei einem Fehler
int	CFileFinder::Insert(FfStr *pInfo,int iNr)
{

	m_pTempData->sInfo.dwAttr		= pInfo->dwAttr;
	m_pTempData->sInfo.dwSize		= pInfo->dwSize;
	m_pTempData->sInfo.dwTimeWrite	= pInfo->dwTimeWrite;
	m_pTempData->sInfo.dwTimeCreate	= pInfo->dwTimeCreate;
	m_pTempData->sInfo.dwTimeAccess	= pInfo->dwTimeAccess;


return Insert(pInfo->cName,iNr,0);
}

//*****************************************************************************
//*
//*		Insert
//*
//*****************************************************************************
//  Einen Dateinamen einfgen. Das geht nur wenn nicht in
//  Unterverzeichnissen gesucht wurde !!!
//	pName	: Ist der Dateiname der eingefgt werden soll
//	iNr		: Ist die Einfgeposition. (-1 fr das Ende,-2 fr sortiert)
//	iAttr	: Sind die Datei Attribute (SYS_READ,SYS_WRITE,SYS_DIR ...)
//	iLen	: Ist die Dateilnge
//	uWrite	: Ist die nderungszeit der Datei
//	uCreate	: Ist die Erstellungszeit der Datei
//	uAccess	: Ist die Zeit des letzten Dateizugriffs
//	Ergibt die Listenposition wo die Datei eingefgt wurde bzw. -1 bei einem Fehler
int	CFileFinder::Insert(const zchar *pName,int iNr,int iAttr,iff64 iLen,unsigned uWrite,unsigned uCreate,unsigned uAccess)
{

	m_pTempData->sInfo.dwSize		= (unsigned)(iLen);
	m_pTempData->sInfo.dwAttr		= (unsigned)(iLen>>(32-SYS_HIGHSHIFT));
	m_pTempData->sInfo.dwAttr	   |= iAttr&~SYS_HIGHMASK;
	m_pTempData->sInfo.dwTimeWrite	= uWrite;
	m_pTempData->sInfo.dwTimeCreate	= uCreate;
	m_pTempData->sInfo.dwTimeAccess	= uAccess;


return Insert(pName,iNr,0);
}

//*****************************************************************************
//*
//*		Insert
//*
//*****************************************************************************
//  Einen Dateinamen einfgen. Das geht nur wenn nicht in
//  Unterverzeichnissen gesucht wurde !!!
//	pName	: Ist der Dateiname der eingefgt werden soll
//	iNr		: Ist die Einfgeposition. (-1 fr das Ende,-2 fr sortiert)
//	iExist	: Wenn !=0 so wird nach einer Datei mit diesem Namen gesucht,
//			  und dabei werden alle Dateiinformationen geholt.
//			  Bei Bit1 wird auch geprft ob die Datei noch nicht in der Liste ist.
//			  Bei Bit2 werden keine Verzeichnisse eingefgt
//			  Bei Bit3 werden keine Dateien eingefgt
//	Ergibt die Listenposition wo die Datei eingefgt wurde bzw. -1 bei einem Fehler
int CFileFinder::Insert(const zchar *pName,int iNr,int iExist)
{
zchar			cTemp[MAX_PATH];
zchar		   *pPath;
FfComparCb		pProc;
int				iOff;
int				iPos;
int				iLen;



	if(!m_pTempUser)											// Sind Temp-Daten vorhanden
		{
		AllocTempData();
		}

	if(iNr==-2 && m_iSortMode<0)
		{
		iNr = -1;
		}

	iLen = strcwlen(pName,0x01F00000);

	if(pName[iLen] || !m_sFolders.Count() || iExist)			// Datei mit Pfadangabe
		{
		if(pName[iLen]=='*')return -1;
		if(pName[iLen]=='?')return -1;

		iLen = FfFullPath(pName,cTemp,sizeof(cTemp));

		for(iOff=iLen;iOff>=0;)									// Trennzeichen anpassen
			{
			iOff--;

			if(cTemp[iOff]==SLASH_INV)
				{
				cTemp[iOff]=SLASH_INV;
				}
			}

		if(iExist)
		do	{
			if(iLen>2              && cTemp[iLen-2]==':')
			if(cTemp[iLen-1]=='\\' || cTemp[iLen-1]=='/')
				{
				iLen       -= 1;
				cTemp[iLen]	= 0;
				}

			if(iLen>1 && cTemp[iLen-1]==':')					// Ist Laufwerkspfad ?
				{
				if(iExist&4)return -1;
				m_pTempData->sInfo.dwAttr		= SYS_DIR|SYS_DRIVE;
				m_pTempData->sInfo.dwSize		= 0;
				m_pTempData->sInfo.dwTimeWrite	= 0;
				m_pTempData->sInfo.dwTimeAccess	= 0;
				m_pTempData->sInfo.dwTimeCreate	= 0;
				pName = cTemp;
				if(iNr!=-2)break;
				memcpy(m_pTempData->sInfo.cName,pName,(iLen+1)*sizeof(zchar));
				break;
				}

			if(FfFileInfo(cTemp,&m_pTempData->sInfo)<0)			// Daten einlesen
				{
				return -1;
				}
																// Keine Verzeichnisse
			if((iExist&4) &&  (m_pTempData->sInfo.dwAttr&SYS_DIR))
				{
				return -1;
				}
																// Nur Verzeichnisse
			if((iExist&8) && !(m_pTempData->sInfo.dwAttr&SYS_DIR))
				{
				return -1;
				}

			}while(0);


		for(iOff=iLen;iOff>0;)									// Suche das Pfadende
			{
			iOff--;

			if(cTemp[iOff]=='\\' || cTemp[iOff]=='/')
				{
				iOff++;
				break;
				}
			}

		iLen -= iOff;
		pName = cTemp+iOff;
		iPos  = m_sFolders.Find(iOff,cTemp,0,-1,CSC_MASK_TEST);	// Verzeichnis suchen

		if(iPos<0)
			{
			iPos = m_sFolders.Add(iOff,cTemp);
			}
		else if((iExist&2) && ExistFile(pName,iPos,iLen)>=0)	// Bereits in der Liste
			{
			return -1;
			}

		if(iNr==-2 && iExist==0)								// Dateiname kopieren bei Sortierung
			{
			memcpy(m_pTempData->sInfo.cName,pName,(iLen+1)*sizeof(zchar));
			}

		pPath = m_sFolders.GetNone(iPos);
		}
	else{
		if(m_sFolders.Count()==0)
			{
			return -1;
			}

		if(iNr==-2)
			{
			memcpy(m_pTempData->sInfo.cName,pName,(iLen+1)*sizeof(zchar));
			}

		iPos  = m_sFolders.Count()-1;
		pPath = m_sFolders.GetNone(iPos);
		}

	if(!pName[0])return -1;

	m_pTempData->pPath      =  pPath;
	m_pTempData->iFolderNum =  iPos;
	m_pTempData->iShItemNum = -1;

	if(iNr==-2)													// Sortiert einfgen
		{
		pProc	   = m_pCmpProc;
		m_pCmpProc = CmpProc;
		iNr		   = InsertPos();
		m_pCmpProc = pProc;
		}
	else{
		m_iSortMode = -1;

		if(!m_sFiles.Check(iNr))
			{
			iNr = m_sFiles.Count();								// Am Ende einfgen
			}
		}


	UserLength(m_pTempUser,iLen);

	if(m_pChangeCb)m_pChangeCb(m_pCbParam,iNr,1,m_uCbInsert^m_uCbLock);
	m_sFiles.Add(iLen,pName              ,iNr  ,m_pTempUser);
	if(m_pChangeCb)m_pChangeCb(m_pCbParam,iNr,1,m_uCbInsert);



return iNr;
}

//*****************************************************************************
//*
//*		NetPathConvert
//*
//*****************************************************************************
//	Konvertiert einen Netzwerkspfad in den realen Pfad
#ifndef CFF_NONET
int CFileFinder::NetPathConvert(const zchar *pPath,zchar *pBuffer,int iSize)
{
const zchar	   *pPtr;
zchar			cText[CFF_PATH_MAX];
zchar			cChar;
unsigned		uLen;
unsigned		uNum;
int				iOff;
int				iOut;
int				i;


	if(iSize<=3)
		{
		if(iSize>0)*pBuffer=0;
		return 0;
		}

	iSize--;
																// Ist es ein Netzpfad "\\...."
	if((pPath[0]!='\\' && pPath[0]!='/') || (pPath[1]!='\\' && pPath[1]!='/') || pPath[2]==0)
		{
		   i=strzlen(pPath);
		if(i>iSize)
			{
			*pBuffer=0;
			return 0;
			}

		for(iOff=i;iOff>=0;iOff--)								// Pfad kopieren
			{
			if(pPath[iOff]==SLASH_INV)
					pBuffer[iOff]=SLASH;
			else	pBuffer[iOff]=pPath[iOff];
			}

		return i;
		}


	for(i=2,iOff=0;pPath[i];i++)								// Suchen Ende vom Server-Namen "\\<server>\<share>\..."
		{
		if(i>CFF_PATH_MAX-8)
			{
			*pBuffer=0;
			return 0;
			}

		if(pPath[i]=='/' || pPath[i]=='\\')						// Nchstes Segment
			{
			cText[i] = SLASH;
			iOff++;
			if(iOff>=2)break;									// Nach Share-Segment
			continue;
			}

		cText[i] = pPath[i];
		}

	cText[0  ] = SLASH;
	cText[1  ] = SLASH;
    cText[i  ] = SLASH;
	cText[i+1] = '*';
	cText[i+2] = 0;


	if(GetFileAttributes(cText)!=0xFFFFFFFF)					// Existiert der Server-Pfad
		{
		for(i=0;;i++)											// Pfad kopieren
			{
			if(i>iSize)
				{
				*pBuffer=0;
				return 0;
				}

			if(pPath[i]==SLASH_INV)
					pBuffer[i]=SLASH;
			else	pBuffer[i]=pPath[i];

			if(!pPath[i])break;
			}

		return i;
		}


	#define NDEPTH 6
	CShFind	aShf[NDEPTH];
	int		aPos[NDEPTH];
	int		iLevel=0;


	aPos[0]=2;
	aShf[0].Init(CSIDL_NETWORK);


	if(!m_iNetBaseLen)
		{
		m_iNetBaseLen = strzlen(m_aNetBase);
		}

	if(!strznicmp(pPath+aPos[0],m_aNetBase,m_iNetBaseLen))		// Ist der Basis-Netz-Pfad vorhanden
		{
		pPtr  = pPath+2+m_iNetBaseLen;
		cChar = pPtr[0];

		if(cChar=='\\' || cChar=='/')
			{
			aPos[0] += m_iNetBaseLen+1;

			if(pPtr[1]==0)
				{
				cChar = 0;
				}
			else if(pPtr[1]=='*' && (pPtr[2]==0 || strzicmp(pPtr+2,Z(".*"))))
				{
				cChar = 0;
				}
			}

		if(cChar==0)											// Basis-Netz-Pfad zu "\\"
			{
			pBuffer[0] = '\\';
			pBuffer[1] = '\\';
			pBuffer[2] = 0;
			return 2;
			}
		}


	iOff = iSize+1;

	while(iLevel>=0)
		{
		pPtr = pPath+aPos[iLevel];
		uLen = strzcspn(pPtr,Z("/\\"));

		while(aShf[iLevel].Next())								// Netz Root durchlaufen
			{
			if(aShf[iLevel].Name(cText)==0)continue;
			if(iLevel>0)
			if(aShf[iLevel-1].IsFilesystem ())
			if(aShf[iLevel-1].IsHasRealDirs())continue;

			uNum =strzlen(cText);
			if(uNum!=uLen)continue;
			if(strznicmp(pPtr,cText,uNum))continue;

			if(aShf[iLevel].IsFilesystem ())
			if(aShf[iLevel].IsHasRealDirs())
				{
				if(!aShf[iLevel].Path(cText)){*pBuffer=0;return 0;}
				i=strlcat(cText,pPtr+uNum,sizeof(cText));
				if(i>=iSize){*pBuffer=0;return 0;}
				memmove(pBuffer,cText,i+1);
				return i;
				}

			if(pPtr[uLen]==0 || pPtr[uLen+1]==0)				// Ende des Pfades erreicht
				{
				break;
				}

			pPtr += uLen+1;
			uLen  = strzcspn(pPtr,Z("/\\"));

			if(pPtr[0]=='*' && pPtr[uLen]==0)					// Ende mit "*" oder "*.*"
			if(uLen==1 || (uLen==3 && pPtr[1]=='.' && pPtr[2]=='*'))
				{
				iOff = pPtr-pPath-1;
				break;
				}

			   iLevel++;
			if(iLevel<NDEPTH)
			if(aShf[iLevel].Init(aShf[iLevel-1])==0)			// Auf nchste Ebene
				{
				iLevel--;
				break;
				}

			aPos[iLevel] = pPtr-pPath;
			}

		iLevel--;
		}

	pBuffer[0] = '\\';
	pBuffer[1] = '\\';

	for(i=aPos[0],iOut=2;;i++,iOut++)							// Pfad kopieren
		{
		if(i>=iOff)
			{
			pBuffer[iOut] = 0;
			break;
			}

		if(i>iSize)
			{
			*pBuffer=0;
			return 0;
			}

		   cChar =pPath[i];
		if(cChar==SLASH_INV)
				pBuffer[iOut]=SLASH;
		else	pBuffer[iOut]=cChar;

		if(!cChar)break;
		}




return iOut;
}
#endif

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht eine Datei mit Pfad in der Dateiliste
//	pName		: Ist der Name der Datei nach der gesucht werden soll.
//	pPath		: Ist der Pfad der Datei nach der gesucht werden soll.
//				  Der Pfad muss mit eine Slash oder Backslash enden.
//	iLenN		: Lnge des Namens
//	iLenP		: Lnge des Pfades
//	Ergibt die Dateiposition bzw. -1 wenn nichts gefunden wurde.
int CFileFinder::Find(const zchar *pName,const zchar *pPath,int iLenN,int iLenP)const
{
zchar  *pText;
int		iPos;
int		iLen;


	iPos = m_sFolders.Count();

	for(;;)														// Suche den passenden Folder
		{
		if(iPos<=0)return -1;

		iPos--;

		   iLen =m_sFolders.LenNone(iPos);
		if(iLen!=iLenP)continue;

		pText = m_sFolders.GetNone(iPos);

		if(!CmpFileNameN(pText,pPath,iLen,iLenP))
			{
			break;
			}
		}


return ExistFile(pName,iPos,iLenN);
}


//*****************************************************************************
//*
//*		ExistFile
//*
//*****************************************************************************
//	Suche eine Datei mit der Verzeichnisnummer.
//	Es wird dabei nur in alten Dateien gesucht.
//	pName		: Ist der Name der Datei nach der gesucht werden soll.
//	iFolderPos	: Ist die Verzeichnisnummer
//	iLen		: Ist die Lnge des Namens
//	Ergibt die Dateiposition bzw. -1 wenn nichts gefunden wurde.
int CFileFinder::ExistFile(const zchar *pName,int iFolderPos,int iLen)const
{
FileDataStr    *pData;
void 		   *pUser;
unsigned iff64	uMask;
unsigned iff64	uTest;
unsigned iff64	uVal;
int				iPos;
int				iNum;


	uTest = *(unsigned iff64*)(pName);
	uMask = 0xDFDFDFDFDFDFDFDF;

	if(iLen*sizeof(zchar)<7)
		{
		uMask >>= 8*(iLen*sizeof(zchar)-7);
		}

	for(iPos=0;iPos<m_sFiles.Count();iPos++)
		{
		   pUser = m_sFiles.DataNone(iPos);
		   pData = UserToData(pUser);
		if(pData->iFolderNum!=iFolderPos)						// Anderes Verzeichnis ?
			{
			continue;
			}

		   iNum	 = UserToLength(pUser);
		if(iNum!=iLen)											// Dateilnge ungleich ?
			{
			continue;
			}

		uVal  = *(unsigned iff64*)(pData->sInfo.cName);			// Schnelltest fr 8 Byte
		uVal ^= uTest;
		uVal &= uMask;

		if(uVal)continue;

		if(!strzicmp(pName,pData->sInfo.cName))					// Namen vergleichen
			{
			return iPos;
			}
		}


return -1;
}

//*****************************************************************************
//*
//*		ReadNetPath
//*
//*****************************************************************************
//  Liest fr ein Netzlaufwerk alle Shares ein
//  iFolderPos	: Ist die Nummer des Verzeichnisses in m_sFolders
//	Es wird in den Hilfsspeicher m_pTempPath kopiert
//  Die gefunden Dateien werden bei m_sFiles angehngt
//  Die gefunden Verzeichnisse  werden bei m_sFolders angehngt
//  Ergibt die Anzahl der gefundenen Dateien.
#ifndef CFF_NONET
int CFileFinder::ReadNetPath(int iFolderPos)
{
zchar		   *pText;
HANDLE			hNetwork;
int				iMulti=0;
int				iCount;
int				iPlace;
int				iCnt;
int				iLen;
int				iPos;



	pText = m_sFolders.GetNone(iFolderPos);

	if(m_iStartFolderNum>0 && m_iStartFolderNum<=iFolderPos)		// Doppeltes Verzeichnis
	if(m_iFindFlags&CFF_FIND_ADD)
		{
		iPos=m_sFolders.Find(pText,0,m_iStartFolderNum,CSC_MASK_TEST);
		if(iPos>=0)
			{
			m_sFolders.Del(iFolderPos);

			if(m_iPathNum>=iFolderPos)
				{
				m_iPathNum	= -1;
				}

			pText			= m_sFolders[iPos];
			iFolderPos		= iPos;
			iMulti			= 1;
			m_iFolderNum   -= 1;
			}
		}


	BufferCopy(iFolderPos);

	m_pTempData->pPath		= pText;
	m_pTempData->iFolderNum = iFolderPos;
	m_pTempData->iShItemNum = -1;



	hNetwork = FindFirstNetwork(m_pTempPath);
	if(!hNetwork)return 0;



	for(;;)															// Einlese Schleife
		{
		if(m_cUserBreak)
			{
			m_cDoneFlag = 2;
			break;
			}

		if(!FindNextNetwork(hNetwork,m_pTempData->sInfo.cName,sizeof(m_pTempData->sInfo.cName)/sizeof(TCHAR)))
			{
			break;
			}

		m_pTempData->sInfo.dwAttr		= SYS_DIR|SYS_NET;
		m_pTempData->sInfo.dwTimeCreate	= 0;
		m_pTempData->sInfo.dwTimeAccess	= 0;
		m_pTempData->sInfo.dwTimeWrite	= 0;
		m_pTempData->sInfo.dwSize		= 0;

		if(m_pTempData->sInfo.dwAttr&SYS_DIR)						// Verzeichnis gefunden ?
	  		{
			iLen = strzlen(m_pTempData->sInfo.cName);

			if(m_iFindFlags&CFF_FIND_SUBDIR)						// Auch in Unterverzeichnissen suchen
				{
				memcpy (m_pTempPath+m_iPathLen,m_pTempData->sInfo.cName,iLen);
				*(int*)(m_pTempPath+m_iPathLen+iLen) = *(int*)SLASH_TXT;
				m_sFolders.Add(m_iPathLen+iLen+1,m_pTempPath,CSC_END,MINUS_MEM);
				}

			if(m_iFindFlags&CFF_FIND_ADDDIR)						// Auch Verzeichnisse suchen
				{													// Suche doppelte Dateinamen
				if(iMulti && ExistFile(m_pTempData->sInfo.cName,iFolderPos,iLen)>=0)
					{
					continue;
					}

				UserLength(m_pTempUser,iLen);

				iPlace = InsertPos();
				if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert^m_uCbLock);
				m_sFiles.Add(iLen,m_pTempData->sInfo.cName,iPlace  ,m_pTempUser);
				if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert);
				iCount++;
				continue;
				}

			if((m_iFindFlags&CFF_FIND_DIR)==0)
				{
				continue;
				}
			}
		else if(m_iFindFlags&CFF_FIND_DIR)
			{
			continue;
			}

	//***************** Dateinamen auswerten *******************************

		iLen = strzlen(m_pTempData->sInfo.cName);

		if(m_iFilerPlus)											// Plusfilter vergleichen
			{
			for(iPos=0;iPos<m_iFilerPlus;iPos++)
				{
				pText = m_sPlus.GetNone(iPos);
				iCnt  = m_sPlus.LenNone(iPos);

				if(!memxcmp(pText,iCnt,m_pTempData->sInfo.cName,iLen,1))
					{
					break;
					}
				}

			if(iPos>=m_iFilerPlus)continue;
			}

		if(m_iFilerMinus)											// Minusfilter vergleichen
			{
			for(iPos=0;iPos<m_iFilerMinus;iPos++)
				{
				pText = m_sMinus.GetNone(iPos);
				iCnt  = m_sMinus.LenNone(iPos);

				if(!memxcmp(pText,iCnt,m_pTempData->sInfo.cName,iLen,1))
					{
					break;
					}
				}

			if(iPos<m_iFilerMinus)continue;
			}
																	// Suche doppelte Dateinamen
		if(iMulti && ExistFile(m_pTempData->sInfo.cName,iFolderPos,iLen)>=0)
			{
			continue;
			}

		UserLength(m_pTempUser,iLen);

		iPlace = InsertPos();
		if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert^m_uCbLock);
		m_sFiles.Add(iLen,m_pTempData->sInfo.cName,iPlace  ,m_pTempUser);
		if(m_pChangeCb)m_pChangeCb(m_pCbParam     ,iPlace,1,m_uCbInsert);
		iCount++;
		}

	FindCloseNetwork(hNetwork);



return iCount;
}
#endif

//*****************************************************************************
//*
//*		GetSizeAll
//*
//*****************************************************************************
//	Gre von allen gefunden Dateien holen. Achtung, bei der Suche
//	sollte CFF_FIND_ALL gesetzt sein
//	Ergibt die Gre aller gefunden Dateien in Byte.
iff64 CFileFinder::GetSizeAll()const
{
FileDataStr	   *pData;
iff64			iSize;
iff64			iAdd;
int				iPos;


	iSize = 0;

	for(iPos=0;iPos<m_sFiles.Count();iPos++)
		{
		   pData = DataNone(iPos);								// Zeiger auf Dateidaten
		if(pData->sInfo.dwAttr&(SYS_DIR|SYS_NET))continue;

		iSize += pData->sInfo.dwSize;

		if(pData->sInfo.dwAttr&SYS_HIGHMASK)					// Lnge ber 32-Bit
			{
			iAdd   = pData->sInfo.dwAttr&SYS_HIGHMASK;
			iAdd >>= SYS_HIGHSHIFT;
			iAdd   = iAdd<<32;
			iSize += iAdd;
			}
		}


return iSize;
}

//*****************************************************************************
//*
//*		SetUserDataSize
//*
//*****************************************************************************
//	Stellt die Gre des User-Daten-Feldes ein. Das ist nur mglich
//	zur Zeit keine Dateien in der Liste sind.
//	iSize		: Ist die neue Gre in Byte (4..4096)
//	Ergibt 1 wenn die Gre verndert wurde, bzw. 0 bei einem Fehler
int	CFileFinder::SetUserDataSize(int iSize)
{


	iSize -= sizeof(void*);										// Gre von pUserPtr abziehen

	if(iSize<0 || iSize>DATA_MAX || m_sFiles.Count())
		{
		return 0;
		}


	iSize +=  3;												// Aufrunden auf 4 Byte
	iSize &= ~3;


	if(m_iUserSize==iSize)										// Kiene nderungen
		{
		return 1;
		}

	m_iUserSize	  = iSize;
	m_iInfoOffset = iSize+offset_of(FileDataStr,sInfo);

	m_sFiles.DataLength(offset_of(FileDataStr,sInfo.cName)+m_iUserSize);


	if(m_pTempUser)
		{
		m_pTempUser -= sizeof(int);
		delete m_pTempUser;
		m_pTempUser  = 0;
		}


return 1;
}

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
#ifdef	_DEBUG
void  	CFileFinder::SortClear   (                  )		{m_iSortMode=-1;}
void   	CFileFinder::Break       (                  )		{m_cUserBreak=1;}
char   	CFileFinder::DoneFlag    (                  )const	{return m_cDoneFlag;}
int		CFileFinder::Error       (                  )const	{return m_iLastError;}
int	   	CFileFinder::InfoOffset  (                  )const	{return m_iInfoOffset;}
int	   	CFileFinder::Count       (                  )const	{return m_sFiles.Count();}
int		CFileFinder::Reset       (                  )		{if(m_uCurrentState>0)m_uCurrentState=1;m_iCurrentNum=0;return m_sFiles.Count();}
int		CFileFinder::NameLen     (int iNr,int iMode )const	{if(iMode==0)return m_sFiles.Len(ItemIndex(iNr));return NameFullLen(iNr,iMode-1);}
int		CFileFinder::NameFullLen (int iNr,int iMode )const	{return InternalNameFullLen(ItemIndex(iNr),iMode);}
int		CFileFinder::NamePostLen (int iNr           )const	{return InternalNameFullLen(ItemIndex(iNr),1    );}
zchar  *CFileFinder::Name        (FfStr*pInfo,int*pL)const	{if(!pInfo){*pL=0;return 0;}*pL=InfoToLength(pInfo);return pInfo->cName;}
zchar  *CFileFinder::Name        (int iNr           )const	{                    return m_sFiles.Get  (ItemIndex(iNr)      );}
zchar  *CFileFinder::Name        (int iNr,int iMode,int *pLen)		{if(iMode==0)return m_sFiles.GetEx(ItemIndex(iNr),pLen );return InternalNameFull(ItemIndex(iNr),pLen,iMode-1);}
zchar  *CFileFinder::NameFull    (int iNr,int *pLen,int iMode)		{return InternalNameFull(ItemIndex(iNr),pLen,iMode     );}
zchar  *CFileFinder::NamePost    (int iNr,int *pLen          )		{return InternalNameFull(ItemIndex(iNr),pLen,1         );}
zchar  *CFileFinder::Suffix		 (int iNr,          int *pLen)const	{void *pUser=m_sFiles.Data(ItemIndex(iNr));return Suffix((pUser)? UserToInfo(pUser):0,      pLen);}
zchar  *CFileFinder::Link        (int iNr,          int *pLen)const	{void *pUser=m_sFiles.Data(ItemIndex(iNr));return Link  ((pUser)? UserToInfo(pUser):0,      pLen);}
zchar  *CFileFinder::Path        (int iNr,int iMode,int *pLen)const	{void *pUser=m_sFiles.Data(ItemIndex(iNr));return Path  ((pUser)? UserToInfo(pUser):0,iMode,pLen);}
zchar  *CFileFinder::CurrentPath (					)const  {return m_pTempPath;}
void   *CFileFinder::UserData    (int iNr           )const	{return               m_sFiles.Data(ItemIndex(iNr));}
void   *CFileFinder::GetDataPtr  (FfStr *pInfo      )const	{if(!pInfo)return 0;return      InfoToData(pInfo)->pUserPtr;}
int     CFileFinder::GetData     (FfStr *pInfo      )const	{if(!pInfo)return 0;return (int)InfoToData(pInfo)->pUserPtr;}
FfData *CFileFinder::Data        (int iNr           )const	{return UserToData(m_sFiles.Data    (iNr));}
FfData *CFileFinder::DataNone    (int iNr           )const	{return UserToData(m_sFiles.DataNone(iNr));}
void   *CFileFinder::SetData     (void *pNew,int iNr)		{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);void	 *pOld=pData->pUserPtr;pData->pUserPtr=pNew;return pOld;}
void   *CFileFinder::GetDataPtr  (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->pUserPtr                ;}
FfStr  *CFileFinder::Info        (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return &pData->sInfo                   ;}
int 	CFileFinder::IsReadOnly  (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_WRITE  ;}
int 	CFileFinder::IsArchive   (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_ARCHIVE;}
int 	CFileFinder::IsDrive     (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_DRIVE  ;}
int		CFileFinder::IsDir       (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_DIR    ;}
int 	CFileFinder::IsNet       (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_NET    ;}
int		CFileFinder::Len	     (int iNr			)const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return-1;FfData *pData=DataNone(iNr);return  pData->sInfo.dwSize            ;}
int		CFileFinder::Del         (int iNr			)		{return InternalDel(   ItemIndex(iNr)   );}
int		CFileFinder::Attribute	 (int iNr			)const	{return Attribute(Info(ItemIndex(iNr))  );}
iff64	CFileFinder::Len64       (int iNr			)const	{return Len64(Info(ItemIndex(iNr))      );}
time_t	CFileFinder::Time        (int iNr			)const	{return Time (Info(ItemIndex(iNr))      );}
time_t	CFileFinder::Time        (int iNr,int iMode )const	{return Time (Info(ItemIndex(iNr)),iMode);}
time_t 	CFileFinder::TimeWrite   (int iNr			)const	{return Time (Info(ItemIndex(iNr))      );}
time_t 	CFileFinder::TimeCreate  (int iNr			)const	{return Time (Info(ItemIndex(iNr)),    1);}
time_t 	CFileFinder::TimeAccess  (int iNr			)const	{return Time (Info(ItemIndex(iNr)),    2);}
time_t 	CFileFinder::TimeWrite   (FfStr *pInfo      )const	{return Time (pInfo,0);}
time_t 	CFileFinder::TimeCreate  (FfStr *pInfo      )const	{return Time (pInfo,1);}
time_t 	CFileFinder::TimeAccess  (FfStr *pInfo      )const	{return Time (pInfo,2);}
int		CFileFinder::Len	     (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwSize:-1;}
int     CFileFinder::IsNet       (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_NET    :0;}
int     CFileFinder::IsDir       (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_DIR    :0;}
int     CFileFinder::IsDrive     (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_DRIVE  :0;}
int		CFileFinder::IsArchive   (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_ARCHIVE:0;}
int		CFileFinder::IsReadOnly  (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_WRITE  :0;}
int		CFileFinder::Attribute	 (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr           :-1;}
void   *CFileFinder::UserData    (FfStr *pInfo      )const	{return (pInfo)? InfoToUser(pInfo):0;}
int    	CFileFinder::GetData     (           int iNr)const	{return (int)GetDataPtr           (iNr   );}
int    	CFileFinder::SetData     (int iValue,int iNr)		{return (int)SetData((void*)iValue,iNr   );}
int   	CFileFinder::ItemIndex   (int iNr           )const	{return (iNr!=-1)? iNr:m_iCurrentNum-1;}
int   	CFileFinder::NextIndex   (                  )const	{return m_iCurrentNum;}
int		CFileFinder::NameLen     (FfStr  *pInfo     )const	{if(!pInfo)return 0;return InfoToLength(pInfo);}
int		CFileFinder::InsertPos	 (                  )const	{if(!m_pCmpProc)return m_sFiles.Count();return m_sFiles.FindFast((void*)this,m_pTempUser,0,m_sFiles.Count(),0x0C,m_pCmpProc);}
FfStr  *CFileFinder::InfoOfUser  (void   *pUser     )const	{return (pUser)? UserToInfo(pUser):0;}
void   *CFileFinder::UserOfInfo  (FfStr  *pInfo     )const	{return (pInfo)? InfoToUser(pInfo):0;}
FfData *CFileFinder::UserToData  (void   *pUser     )const	{return (FfData*)(((char*)pUser)+m_iUserSize  );}
FfStr  *CFileFinder::UserToInfo  (void   *pUser     )const	{return (FfStr *)(((char*)pUser)+m_iInfoOffset);}
FfData *CFileFinder::InfoToData  (FfStr  *pInfo     )const	{return (FfData*)(((char*)pInfo)-((int)(&(((FfData*)0)->sInfo))));}
void   *CFileFinder::InfoToUser  (FfStr  *pInfo     )const	{return (void  *)(((char*)pInfo)-m_iInfoOffset);}
FfStr  *CFileFinder::DataToInfo  (FfData *pData     )const	{return (FfStr *)        &pData->sInfo         ;}
void   *CFileFinder::DataToUser  (FfData *pData     )const	{return (void  *)(((char*)pData)-m_iUserSize  );}
int		CFileFinder::UserToLength(void   *pUser     )const	{return *(int*)(((char*)pUser)              -sizeof(int));}
int		CFileFinder::InfoToLength(FfStr  *pInfo     )const	{return *(int*)(((char*)pInfo)-m_iInfoOffset-sizeof(int));}
int		CFileFinder::DataToLength(FfData *pData     )const	{return *(int*)(((char*)pData)-m_iUserSize  -sizeof(int));}
void 	CFileFinder::UserLength  (void   *pUsr,int i)const	{       *(int*)(((char*)pUsr )-sizeof(int))=i            ;}
void	CFileFinder::TimeFilter  (int iMd,time_t iTi,time_t iTu){m_iTimeFilter=iMd;m_iTimeValue=iTi;m_iTimeUntil=iTu;}
void	CFileFinder::ProtectFilter(int iMode        )		{m_iRdWrFilter=iMode;}
void	CFileFinder::AttributeFilter(int iOne,int iMask,int iMust){m_iOneFilter=iOne;m_iAttrFilter=iMask;m_iMustFilter=iMust;}
void  **CFileFinder::GetFieldPointer(int **pCount)const{return m_sFiles.GetFieldPointer(pCount);}
void	CFileFinder::SetData     (FfPtr &pInfo,void*pValue){if(!pInfo)return;*(void**)(((char*)pInfo)-((int)(&(((FfData*)0)->sInfo))))=pValue;}
void	CFileFinder::SetData     (FfPtr &pInfo,int  iValue){if(!pInfo)return; *(int  *)(((char*)pInfo)-((int)(&(((FfData*)0)->sInfo))))=iValue;}
int		CFileFinder::Find        (const zchar *pName         )const	{return m_sFiles.Find(     pName,0,-1,CSC_MASK_TEST);}
int		CFileFinder::Find        (const zchar *pName,int iLen)const	{return m_sFiles.Find(iLen,pName,0,-1,CSC_MASK_TEST);}

#ifdef	WIN32
zchar  *CFileFinder::GetType     (int    iNr  ,int iMode)		{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return  0;return GetType(&DataNone(iNr)->sInfo,iMode);}
int		CFileFinder::GetIcon     (int    iNr  ,int iMode)const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return -1;return GetIcon(&DataNone(iNr)->sInfo,iMode);}
int	   	CFileFinder::GetSmallIcon(int    iNr  ,int iMode)const	{return GetIcon(iNr  ,iMode|SHGFI_SMALLICON);}
int	   	CFileFinder::GetLargeIcon(int    iNr  ,int iMode)const	{return GetIcon(iNr  ,iMode|SHGFI_LARGEICON);}
int		CFileFinder::GetSmallIcon(FfStr *pInfo,int iMode)const	{return GetIcon(pInfo,iMode|SHGFI_SMALLICON);}
int		CFileFinder::GetLargeIcon(FfStr *pInfo,int iMode)const	{return GetIcon(pInfo,iMode|SHGFI_LARGEICON);}
#endif

#endif


#ifdef	WIN32

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CIconChache::CIconChache()
{

	m_pList		= 0;
	m_pNames	= 0;

	InitializeCriticalSection(&m_sSynch);

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CIconChache::~CIconChache()
{

	if(m_pList)
		{
		Clear();
		}

	DeleteCriticalSection(&m_sSynch);

}


//*****************************************************************************
//*
//*		Clear
//*
//*****************************************************************************
//	Lscht alle
void CIconChache::Clear()
{
int		i;



	EnterCriticalSection(&m_sSynch);

	if(m_pList)
		{
		for(i=0;i<27;i++)
			{
			if(!m_pList[i])continue;
			delete m_pList[i];
			m_pList[i]=0;
			}

		delete m_pList;
		m_pList=0;
		}

	if(m_pNames)
		{
		m_pNames->Del();
		delete m_pNames;
		m_pNames = 0;
		}

	LeaveCriticalSection(&m_sSynch);

}

//*****************************************************************************
//*
//*		GetItem
//*
//*****************************************************************************
void *CIconChache::GetItem(const char *pSuffux)
{
void	   *pHandle;
unsigned	uEntry;
IconData	sData;
BYTE		bChar;
int			iNum;
int			i;


	for(i=0;i<CIC_NAME_MAX;i++)									// Suffixnamen abspeichern
		{
		if(!pSuffux[i])break;
		bChar = (BYTE)((pSuffux[i]&0xFFFFFF80)? 255:pSuffux[i]);
		if(bChar<='Z' && bChar>='A')bChar+=0x20;
		sData.aName[i] = bChar;
		}

	for(;i<CIC_NAME_MAX;i++)									// Rest mit 0 fllen
		{
		sData.aName[i] = 0;
		}


	EnterCriticalSection(&m_sSynch);


	if(sData.aName[0]>='a' && sData.aName[0]<='z')				// Slot fr das Suchen
			uEntry = sData.aName[0]-'a'+1;
	else	uEntry = 0;

	if(!m_pList)
		{
		m_pList = new CDataListPtr[27];
		memset(m_pList,0,sizeof(CDataListPtr)*27);
		}


	if(!m_pList[uEntry])										// Slot noch nicht initialisiert ?
		{
		m_pList[uEntry] = new CDataList;
		m_pList[uEntry]->Len(sizeof(IconData));

		sData.aIcon[0]  = -1;
		sData.aIcon[1]  = -1;
		sData.aIcon[2]  = -1;
		sData.aIcon[3]  = -1;
		sData.pType		=  0;

		iNum    = m_pList[uEntry]->Add(&sData);
		pHandle = m_pList[uEntry]->Get(iNum);

		LeaveCriticalSection(&m_sSynch);

		return pHandle;
		}

																// Such nach dem Suffix
	iNum = m_pList[uEntry]->FindFast(sData.aName,0,0x7FFFFFFF,CIC_NAME_MAX,offsetof(IconData,aName),0x00);


	if(iNum<0)													// Das Suffix ist noch nicht eingetragen
		{
		iNum = m_pList[uEntry]->FindFast(sData.aName,0,0x7FFFFFFF,CIC_NAME_MAX,offsetof(IconData,aName),0x08);

		sData.aIcon[0]  = -1;
		sData.aIcon[1]  = -1;
		sData.aIcon[2]  = -1;
		sData.aIcon[3]  = -1;
		sData.pType		=  0;

		iNum	= m_pList[uEntry]->Add(&sData,iNum);
		pHandle = m_pList[uEntry]->Get(iNum);

		LeaveCriticalSection(&m_sSynch);

		return pHandle;
		}


	pHandle = m_pList[uEntry]->Get(iNum);

	LeaveCriticalSection(&m_sSynch);


return pHandle;
}


//*****************************************************************************
//*
//*		GetItem
//*
//*****************************************************************************
void *CIconChache::GetItem(const wchar_t *pSuffux)
{
char		aName[CIC_NAME_MAX+1];
BYTE		bChar;
int			iPos;



	for(iPos=0;iPos<CIC_NAME_MAX;iPos++)						// Suffixnamen abspeichern
		{
		if(!pSuffux[iPos])break;
		bChar = (BYTE)((pSuffux[iPos]&0xFFFFFF80)? 255:pSuffux[iPos]);
		if(bChar<='Z' && bChar>='A')bChar+=0x20;
		aName[iPos] = bChar;
		}

	aName[iPos] = 0;


return GetItem(aName);
}

//*****************************************************************************
//*
//*		NameAdd
//*
//*****************************************************************************
zchar *CIconChache::NameAdd(const zchar *pName,int iLen)
{
zchar  *pPtr;
int		iPos;


	EnterCriticalSection(&m_sSynch);

	if(!m_pNames)
		{
		m_pNames = new CDataPointer;
		m_pNames->Len(1);
		}

	iPos =         m_pNames->Add(pName,0x7FFFFFFF,(iLen+1)*sizeof(zchar));
	pPtr = (zchar*)m_pNames->Get(iPos);

	pPtr[iLen] = 0;

	LeaveCriticalSection(&m_sSynch);


return pPtr;
}

//*****************************************************************************
//*
//*		GetDriveIcon
//*
//*****************************************************************************
//	Das Icon eines Laufwerks holen
//	uLetter		: Ist der Laufwerksbuchstaben (A..Z oder a..z)
//	iMode		: Ist der Modus fr das Icon
//					SHGFI_SMALLICON
//					SHGFI_LARGEICON
//	Ergibt die Icon-Nummer oder eine Fehlernummer kleiner 0.
//						-1 = Icon nicht gefunden
//						-2 = Laufwerk nicht gefunden
int CIconChache::GetDriveIcon(unsigned uLetter,int iMode)
{
WCHAR		cPath[4];
SHFILEINFOW	sInfo;
unsigned	uSlot;
unsigned	uTick;
unsigned	uDiff;



	   uSlot = (uLetter&~0x20)-'A';
	if(uSlot>=26)return -2;

	iMode &= SHGFI_SMALLICON;


	if(m_aDriveType[uSlot]==DRIVE_FIXED)
		{
		if(iMode)sInfo.iIcon = m_aDriveIconS[uSlot];
		else	 sInfo.iIcon = m_aDriveIconL[uSlot];

		return   sInfo.iIcon;
		}

	uTick = GetTickCount();

	if(iMode)uDiff = uTick-m_aDriveDoneS[uSlot];
	else	 uDiff = uTick-m_aDriveDoneL[uSlot];
																// Wechsellaufwerke nur alle 3 Sekunden holen
	if(uDiff<3000 && m_aDriveType[uSlot]!=DRIVE_UNKNOWN)
		{
		if(iMode)sInfo.iIcon = m_aDriveIconS[uSlot];
		else	 sInfo.iIcon = m_aDriveIconL[uSlot];

		return   sInfo.iIcon;
		}

	cPath[0] = uLetter;
	cPath[1] = ':';
	cPath[2] = '\\';
	cPath[3] = 0;


	if(!SHGetFileInfoW(cPath,FILE_ATTRIBUTE_DIRECTORY,&sInfo,sizeof(sInfo),SHGFI_SYSICONINDEX|SHGFI_USEFILEATTRIBUTES|iMode))
		{
		return -1;
		}

	m_aDriveType[uSlot] = GetDriveTypeW(cPath);					// Icon in Laufwerks-Cache speichern

	if(iMode&SHGFI_SMALLICON)
		{
		m_aDriveDoneS[uSlot] = uTick;
		m_aDriveIconS[uSlot] = sInfo.iIcon;
		}
	else{
		m_aDriveDoneL[uSlot] = uTick;
		m_aDriveIconL[uSlot] = sInfo.iIcon;
		}




return sInfo.iIcon;
}

//*****************************************************************************
//*
//*		GetDriveBits
//*
//*****************************************************************************
//	Holt ein Bitfeld mit allen Laufwerksbuchstaben
unsigned CIconChache::GetDriveBits()
{
unsigned	uTick;
unsigned	uDiff;
unsigned	uBits;


	uTick = GetTickCount();
	uDiff = uTick-m_uScanLast;

	if(uDiff<1000 && m_uDriveBits)					// Nur alle 1000 Millisekunden neu suchen
		{
		uBits = m_uDriveBits;
		}
	else{
		uBits = GetLogicalDrives();

		m_uDriveBits = uBits;
		m_uScanLast  = uTick;
		}


return uBits;
}

//*****************************************************************************
//*
//*		GetBaseDrive
//*
//*****************************************************************************
//	Holt den Buchstaben der erste Festplatten Laufwerks.
unsigned CIconChache::GetBaseDrive()
{
WCHAR		cPath[64];
unsigned	uBits;
unsigned	uPos;


	if(m_aBaseDrive)
		{
		return m_aBaseDrive;
		}

	if(GetSystemDirectoryW(cPath,64))
		{
		m_aBaseDrive = (BYTE)cPath[0]&~0x20;
		return m_aBaseDrive;
		}

	uBits = GetDriveBits()>>2;

	for(uPos=2;uPos<26;uPos++,uBits>>=1)
		{
		if(uBits&1)
			{
			m_aBaseDrive = 'A'+uPos;
			return m_aBaseDrive;
			}
		}

	m_aBaseDrive = 'C';


return 'C';
}

//*****************************************************************************
//*
//*		GetPathIcon
//*
//*****************************************************************************
//	Das Standart-Icon eines Pfades holen
//	iMode		: Zustzliche Modus-Bits
//					SHGFI_SMALLICON
//					SHGFI_OPENICON
//	Ergibt die Icon-Nummer oder -1 bei einem Fehler
int CIconChache::GetPathIcon(int iMode)
{
SHFILEINFOW	sInfo;


	if(!SHGetFileInfoW(L"Hallo",_A_SUBDIR,&sInfo,sizeof(sInfo),SHGFI_SYSICONINDEX|SHGFI_USEFILEATTRIBUTES|iMode))
		{
		return -1;
		}


return sInfo.iIcon;
}

//*****************************************************************************
//*
//*		GetImageList
//*
//*****************************************************************************
//	Holt die System-Icon-Liste.
//	iLarge		: Wenn ungleich 0 so wird die Liste mit den groen Icons geholt
//	Ergibt ein Handle auf die System-Icon-Liste.
HIMAGELIST CIconChache::GetImageList(int iLarge)
{
SHFILEINFOW		sInfo;
unsigned		uMode;
unsigned		uPos;


	if(iLarge)
		{
		uMode  = SHGFI_LARGEICON;
		uPos   = 0;
		}
	else{
		uMode  = SHGFI_SMALLICON;
		uPos   = 1;
		}

	if(!m_aImageList[uPos])
		{
		m_aImageList[uPos] = (HIMAGELIST)SHGetFileInfoW(L"",0,&sInfo,sizeof(sInfo),SHGFI_PIDL|SHGFI_SMALLICON|SHGFI_SYSICONINDEX|uMode);
		}


return m_aImageList[uPos];
}

//*****************************************************************************
//*
//*		CIconChache inline
//*
//*****************************************************************************
#ifdef _DEBUG
int    CIconChache::GetIcon(const char    *pSuffix,int iType,void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->aIcon[iType]:-1;}
int    CIconChache::GetIcon(const wchar_t *pSuffix,int iType,void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->aIcon[iType]:-1;}
zchar *CIconChache::GetType(const char    *pSuffix,          void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->pType       : 0;}
zchar *CIconChache::GetType(const wchar_t *pSuffix,          void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->pType       : 0;}
void   CIconChache::SetIcon(int iType,int  iIcon,void *pHandle){((IconData*)pHandle)->aIcon[iType]=iIcon;}
zchar *CIconChache::SetType(const zchar   *pName,void *pHandle){return SetType(pName,pHandle,strzlen(pName));}
zchar *CIconChache::SetType(const zchar   *pName,void *pHandle,int iLen){zchar *pPtr=NameAdd(pName,iLen);((IconData*)pHandle)->pType=pPtr;return pPtr;}
#endif


#endif



