//*****************************************************************************
//*
//*
//*		FileFinder.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Die Klasse CFileFinder sucht Dateien in einem Verzeichnis
//	Die Art der Suche wird mit dem Konstuktor oder mit Set(...) eingestellt
//	CFileFinder kann auch rekursif einen ganzen Baum absuchen.
//	Alle Dateinamen des aktuellen Verzeichnisse werden zwischen
//	gespeichert (ohne Pfad angaben)
//	Die Suche wird gestartet mit dem ersten aufruf von Next()
//
//	Funktionen:
//
//		void	        Set				Neuen Suchbereich festlegen
//		int		    	Read			Dateien suchen (alle auf einmal)
//		char	       *Next			Nchste Datei finden
//		char	       *Path			Ergibt aktuellen Pfad
//		char	       *Name			Ergibt Dateinamen ber Index
//		int 	        NameLen			Ergibt Dateinamen Lnge
//		int		        Error			Fehler Meldung abfragen
//		int		        Count			Anzahl der gefundenen Dateien
//		void	        Sort			Dateien sortieren
//		void	        Rename			Dateinamen in der Liste ndern
//		int 	        Len				Ergibt Dateilnge
//		time_t  	    Time		    Ergibt nderungszeit
//	    FfFindStruct   *Info		    Ergibt Zeiger auf Dateiinformationen
//		int 	        IsReadOnly		Ergibt ob Datei schreibgeschtzt ist
//		int 	        IsDir			Ergibt ob Datei Verzeichnis
//	    int 	        IsNet			Ergibt ob Pfad nur ein Netzzweig ist
//	    int 	        Reset			Auslesezeiger fr Next reseten
//	    int 	        DelAll			Gesamte Liste lschen
//	    int 	        Del				Dateinamen aus der Liste lschen
//	    int 	        Find			Suche Datei in Liste
//		int 	        GetIcon			Icon Nummer holen
//		int			    GetSmallIcon	Icon Nummer holen
//		int			    GetLargeIcon	Icon Nummer holen
//		int 	        GetData			Daten Wert lesen
//		int		        SetData			Daten Wert schreiben
//	    void	        Filter			Dateinamen konvertieren
//		void		    Break			Suche abbrechen
//
//	Die Module	"StringChain.cpp" "MemFunctions" "StringFunctions.cpp"
//	und "ShFind.cpp" werden mit verwendet.
//


#ifndef		__FILEFINDER_H__
#define     __FILEFINDER_H__

#if _MSC_VER >= 1000
#pragma once
#endif

#if defined(linux) || defined(LINUX)
	#ifndef 	 _cdecl
	#define 	 _cdecl
	#endif
	#ifndef 	__cdecl
	#define 	__cdecl
	#endif
	#ifndef 	 _stdcall
	#define 	 _stdcall
	#endif
	#ifndef 	__stdcall
	#define 	__stdcall
	#endif
	#define		CFF_NONET
	#ifndef		_WCHAR_T_DEFINED
	typedef		unsigned short wchar_t;
	#define		_WCHAR_T_DEFINED	1
	#endif
#else
	#ifndef		_WCHAR_T_DEFINED
	#include	<wchar.h>
	#endif
#endif



#include	"StringChain.h"
#include	"StringFunctions.h"
#include	<time.h>

#ifdef	WIN32
	#include	<io.h>
	#include	<shlobj.h>
	#include	<shellapi.h>
	#pragma comment(lib, "netapi32.lib")
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	zchar			wchar_t
#define 	strzlen			(int)wcslen
#else
#define 	zchar			char
#define 	strzlen			(int)strlen
#endif
#ifndef		iff64
#define		iff64			__int64
#endif

#define 	CIC_NAME_MAX	8

class		CDataList;
class		CDataPointer;
class		CIconChache;
extern		CIconChache		sIconCache;



//*****************************************************************************
//*
//*		Suchfunktionen
//*
//*****************************************************************************

typedef void		   *FfFind;
typedef	unsigned     	FfUtc;
typedef unsigned		FfDW;

typedef	struct
	{
	FfDW	dwAttr;
	FfDW	dwSize;
	FfUtc 	dwTimeCreate;
	FfUtc 	dwTimeAccess;
	FfUtc 	dwTimeWrite;
	zchar	cName[256];
	}FfFindStruct;

#define	FfFindFirst(n,f)	FfFindFirstEx(n,f,0)
int		_cdecl FfFileInfo	(const zchar *pFilename,FfFindStruct *pInfo);
int		_cdecl FfFullPath   (const zchar *pFilename,zchar *pBuffer,int iMax);
FfFind	_cdecl FfFindFirstEx(const zchar *pFilename,FfFindStruct *pFind,int *pError);
int		_cdecl FfFindNext   (FfFind	   hHandle     ,FfFindStruct *pFind);
int		_cdecl FfFindClose  (FfFind	   hHandle);


#if !defined(__FILEUTILITY_H__) && !defined(__SYSTEM_H__) && !defined(AZ_LIB_NO_SYSFIND)

#define	SysFileInfo				FfFileInfo
#define	SysFullPath				FfFullPath
#define	SysFindFirst			FfFindFirst
#define	SysFindFirstEx			FfFindFirstEx
#define	SysFindNext				FfFindNext
#define	SysFindClose			FfFindClose
#define SysFindStruct			FfFindStruct
#define	SysFind					FfFind
#define	SysUtc					FfUtc
#define	SysDW					FfDW

#define SYS_READ				0x0001								// Flags fr dwAttr in FfFindStruct
#define SYS_WRITE				0x0002								//		*
#define SYS_DIR					0x0010								//		*
#define SYS_HIDDEN				0x0020								//		*
#define SYS_ARCHIVE				0x0040								//		*
#define SYS_SYSTEM				0x0080								//		*
#define SYS_VOLUME 				0x0100								//		*
#define SYS_HIGHMASK			0xFC00								// Maske fr die 64-Bit-Gre
#define SYS_HIGHSHIFT			10									// Shift fr die 64-Bit-Gre

#define SYS_ERR_PARAM			-1									// Fehler bei der Suche nach einer Datei
#define SYS_ERR_FILE_NOT_FOUND	-3									// Es wurden keine Dateien gefunden
#define SYS_ERR_UNKNOWN			-6									// Unbekannter Fehler
#define SYS_ERR_PATH_NOT_FOUND	-7									// Der Pfad wurde nicht gefunden

#else !defined(__FILEUTILITY_H__) && !defined(__SYSTEM_H__)

#include "System/FileUtility.h"

#endif

#define	SYS_DRIVE				0x4000								// Flag fr Laufwerke
#define	SYS_NET					0x8000								// Flag fr Netzpfad

#define FfData					CFileFinder::FileDataStr
#define FfPtr					CFileFinder::FfFindPtr
#define FfStr					FfFindStruct



//*****************************************************************************
//*
//*		Globale Konstanten
//*
//*****************************************************************************

#define	CFF_PATH_MAX			(MAX_PATH+260)						// Maximale Pfad lnge

#define	CFF_ERROR_OK   			0									// Kein Fehler
#define	CFF_ERROR_NULL			1									// Keine Datei gefunden
#define	CFF_ERROR_PATH			2									// Pfad ist zu lang
#define	CFF_ERROR_ACCESS		3									// Fehler beim Zugriff auf LW

#define	CFF_SORT_NAME			0									// Nach Namen sortieren
#define	CFF_SORT_TYPE			1									// Nach Typ sortieren
#define	CFF_SORT_CREATE			2									// Nach Erstellungs Zeit sortieren
#define	CFF_SORT_ACCCESS		3									// Nach letztdem Zugriff sortieren
#define	CFF_SORT_WRITE			4									// Nach letztdem Schreibzugriff sortieren
#define	CFF_SORT_LEN			5									// Nach Datei lnge sortieren
#define	CFF_SORT_PATH			6									// Nach Pfad sortieren
#define	CFF_SORT_SUFFIX			7									// Nach Dateisuffix sortieren
#define	CFL_SORT_ATTRIBUTE		8									// Nach Dateiattributen
#define	CFF_SORT_DATA			9									// Nach dem Datenwert sortieren
#define	CFF_SORT_VALUE			0x00002000							// Bercksichtigt Zahlen in den Texten
#define	CFF_SORT_NODIR			0x00004000							// Verzeichnisse nicht an den Anfang
#define	CFF_SORT_INVERT			0x00008000							// Abwrts sortieren

#define	CFF_FIND_DIRCMP			0x00040000							// Verzeichnisse mit Plus/Minus vergleichen
#define	CFF_FIND_PARTIAL   		0x00020000							// Die Suche immer wieder abbrechen
#define	CFF_FIND_ADDUP			0x00010000							// Auch '..' einfgen
#define	CFF_FIND_DIR			0x00008000							// Nur  Verzeichnise suchen
#define	CFF_FIND_ADDDIR			0x00004000							// Auch Verzeichnisse finden
#define	CFF_FIND_SUBDIR			0x00002000							// Auch in Unterverz. suchen
#define	CFF_FIND_FILE_DIR		0x00006000							// Auch in Unterverz. und Verz. Add
#define	CFF_FIND_ALL     		0x00001000							// Alles auf Einmal lesen
#define	CFF_FIND_ADD     		0x00000800							// Dateien zur den jetztigen anhngen
#define	CFF_FIND_NET     		0x00000400							// Auch Netzwerk durchsuchen
#define	CFF_FIND_FIRST     		0x00000200							// Nur bis zum ersten Eintrag suchen
#define	CFF_FIND_SORT	  		0x00000100							// Dateien sortiert einfgen
#define	CFF_FIND_ONE     		0x00000001							// Nur ein Filter definiert (nur intern)
#define	CFF_FIND_TEMP			0x00000002							// Es gibt temporre Verzeichnisse (nur intern)

#define	CFF_FILTER_BIG			SFT_BIG								// Alles gro  schreiben
#define	CFF_FILTER_LITTLE		SFT_LITTLE							// Alles klein schreiben
#define	CFF_FILTER_BEGIN		SFT_BEGIN							// Anfangsbuchstaben gro schreiben
#define	CFF_FILTER_ONLYLITTLE	SFT_ONLYLITTLE						// Nur wenn alles klein ist filtern
#define	CFF_FILTER_ONLYBIG		SFT_ONLYBIG							// Nur wenn alles gro  ist filtern
#define	CFF_FILTER_ONLYSAME		SFT_ONLYSAME						// Nur alles gro oder alles klein
#define CFF_FILTER_LETLITTLE	SFT_LETLITTLE						// Kleingeschriebene Worte kleinlassen
#define CFF_FILTER_WORD			SFT_SCANWORDS						// Wort weise prfen
#define CFF_FILTER_POINT		SFT_POINTLITTLE						// Nach setzem Punkt klein
#define	CFF_FILTER_PATH    		0x0100								// Auch alle Pfade Filtern
#define	CFF_FILTER_DEFAULT		(CFF_FILTER_PATH|CFF_FILTER_POINT|CFF_FILTER_BEGIN|CFF_FILTER_ONLYBIG)

//*****************************************************************************
//*
//*		Klassen Elemente
//*
//*****************************************************************************
class CFileFinder
{
typedef int (__stdcall *FfChangeCb)(void *pParam,LPARAM lParam,int iCount,unsigned uAction);
typedef int (__cdecl   *FfComparCb)(const void *pPtr1,const void *pPtr2,void *pParam);

typedef FfStr *FfFindPtr;
typedef struct
		{
		void		   *pUserPtr;									// User-Daten (muss am Anfang sein !!)
		zchar		   *pPath;										// Zeiger auf Verzeichnispfad
		zchar		   *pTypeName;									// Ist der Windows-Typname
		int				iSuffixPos;									// Offset des Suffixes
		int				iFolderNum;									// Nummer des Verzeichnisses
		int				iShItemNum;									// Nummer in Shell
		int				iIconNumS;									// Nummer des small Icons
		int				iIconNumL;									// Nummer des large Icons
		FfFindStruct	sInfo;										// Dateiinformationen
		}FileDataStr;


public:

   ~CFileFinder ();
	CFileFinder (const zchar *pPath=0,int iFlags=0,const zchar *pPlus=0,const zchar *pMinus=0);
	int		Read(const zchar *pPath=0,int iFlags=0,const zchar *pPlus=0,const zchar *pMinus=0);	// Neunen Suchbereich festlegen und einlesen
	void	Set (const zchar *pPath  ,int iFlags=0,const zchar *pPlus=0,const zchar *pMinus=0);	// Neunen Suchbereich festlegen

	zchar  *Suffix     (int iNr=-1            ,int *pLen=0)const;	// Ergibt einen Zeiger aus das Sufix
	zchar  *Link	   (int iNr=-1			  ,int *pLen=0)const;	// Ergibt Link-Pfad einer Datei aus Info-Zeiger
	zchar  *Next       (           int iMode=0,int *pLen=0);		// Nchste Datei finden
	zchar  *Path       (int iNr=-1,int iMode=0,int *pLen=0)const;	// Ergibt Pfad einer Datei
	zchar  *Path       (FfStr*pPtr,int iMode=0,int *pLen=0)const;	// Ergibt Pfad einer Datei aus Info-Zeiger
	zchar  *Link	   (FfStr*pPtr,            int *pLen=0)const;	// Ergibt Link-Pfad einer Datei aus Info-Zeiger
	zchar  *Name       (FfStr*pPtr,            int *pLen  )const;	// Ergibt Dateinamen ber Index mit Lnge
	zchar  *Name       (int iNr=-1                        )const;	// Ergibt Dateinamen ber Index
	zchar  *Name       (int iNr   ,int iMode  ,int *pLen=0);		// Ergibt Dateinamen ber Index
	zchar  *NameFull   (int iNr=-1,int *pLen=0,int iMode=0);		// Ergibt vollen Dateinamen ber Index
	zchar  *NamePost   (int iNr=-1,int *pLen=0);					// Ergibt relativen Dateinamen ber Index
	int		NamePostLen(int iNr=-1            )const;				// Ergibt die Lnge vom Dateinamen mit relativen Pfad
	int		NameFullLen(int iNr=-1,int iMode=0)const;				// Ergibt die Lnge vom Dateinamen mit Pfad
	int		NameLen    (int iNr=-1,int iMode=0)const;				// Ergibt die Lnge vom Dateinamen
	FfStr  *Info       (int iNr=-1)const;							// Ergibt Dateiinfo  ber Index
	void   *UserData   (int iNr=-1)const;							// Ergibt einen Zeiger auf die User-Daten
	int     IsNet      (int iNr=-1)const;							// Ergibt ob Eintrag ein Netzzweig ist
    int     IsDir      (int iNr=-1)const;							// Ergibt ob die Datei Verzeichnis
	int     IsDrive    (int iNr=-1)const;							// Ergibt ob die Datei ein Laufwerk ist
	int     IsArchive  (int iNr=-1)const;							// Ergibt ob die Datei ein Archiv-Flag hat
	int		IsReadOnly (int iNr=-1)const;							// Ergibt ob Datei schreibgeschtzt ist
	int		Len        (int iNr=-1)const;							// Ergibt Dateilnge ber Index
	iff64	Len64      (int iNr=-1)const;		 					// Ergibt Dateilnge ber Index
   	int		Del        (int iNr=-1);								// Lscht einen Dateinamen aus der Liste
	int		Attribute  (int iNr=-1)const;							// Ergibt Die Dateiattribute
	time_t  TimeWrite  (int iNr=-1)const;							// Ergibt nderungszeit
	time_t  TimeCreate (int iNr=-1)const;							// Ergibt Erstellungszeit
	time_t  TimeAccess (int iNr=-1)const;							// Ergibt Zugriffszeit
	time_t  Time       (int iNr=-1)const;							// Ergibt nderungszeit
	time_t  Time       (int iNr,int iType)const;					// Ergibt Dateizeit (0=Write 1=Create 2=Access)
	zchar  *CurrentPath(				 )const;					// Ergibt den aktuellen Suchpfad
	zchar  *Suffix     (FfStr *pInfo,int *pLen=0)const;				// Ergibt einen Zeiger aus das Sufix
	int		Len        (FfStr *pInfo)const;							// Ergibt Dateilnge ber Index
	iff64	Len64      (FfStr *pInfo)const;							// Ergibt Dateilnge ber Index
	int		Attribute  (FfStr *pInfo)const;							// Ergibt Die Dateiattribute
	time_t  TimeWrite  (FfStr *pInfo)const;							// Ergibt nderungszeit
	time_t  TimeCreate (FfStr *pInfo)const;							// Ergibt Erstellungszeit
	time_t  TimeAccess (FfStr *pInfo)const;							// Ergibt Zugriffszeit
	time_t  Time       (FfStr *pInfo)const;							// Ergibt nderungszeit
	time_t  Time       (FfStr *pInfo,int iType)const;				// Ergibt Dateizeit (0=Write 1=Create 2=Access)
	int     IsNet      (FfStr *pInfo)const;							// Ergibt ob Eintrag ein Netzzweig ist
    int     IsDir      (FfStr *pInfo)const;							// Ergibt ob Datei Verzeichnis
	int     IsDrive    (FfStr *pInfo)const;							// Ergibt ob Datei ein Laufwerk ist
	int     IsArchive  (FfStr *pInfo)const;							// Ergibt ob die Datei ein Archiv-Flag hat
	int		IsReadOnly (FfStr *pInfo)const;							// Ergibt ob Datei schreibgeschtzt ist
	int		NameLen    (FfStr *pInfo)const;							// Ergibt die Lnge vom Dateinamen aus Info-Zeiger
	void   *UserData   (FfStr *pInfo)const;							// Ergibt einen Zeiger auf die User-Daten
	void   *GetDataPtr (FfStr *pInfo)const;							// Datenwert lesen     (als Zeiger)
	int     GetData    (FfStr *pInfo)const;							// Datenwert lesen
	void    SetData    (FfPtr &pInfo,void *pValue);					// Datenwert schreiben (als Zeiger)
	void    SetData    (FfPtr &pInfo,int   iValue);					// Datenwert schreiben
	int		Insert     (FfStr *pInfo,int iN=-1);					// Fgt einen Dateinamen hinzu einen Dateinamen
	int		Insert     (const zchar *pN,int iN=-1,int iF=0);		// Fgt einen Dateinamen hinzu einen Dateinamen
	int		Insert     (const zchar *pN,int iNr,int iAddr,iff64 iLen,unsigned uWrite=0,unsigned uCreate=0,unsigned uAccess=0);
	void    Rename     (int iNr,const zchar *pName);				// Dateinamen ndern in der Liste umbenennen
	void    Filter	   (int iMode=CFF_FILTER_DEFAULT);				// Dateinamen konvertieren
	void	Break      ();											// Suche abbrechen
	int   	NextIndex  ()const;										// Den Index des Nchsten Eintrages holen
	int	    Error      ()const;										// Fehlermeldung abfragen
	int	    Count      ()const;										// Anzahl der gefundenen Dateien abfragen
	int     Reset      ();											// Auslesezeiger fr Next zurcksetzen
	int		DelAll     ();											// Gesamte Liste lschen
	int     GetData    (			  int iNr   =-1)const;			// Datenwert lesen
	void   *GetDataPtr (              int iNr   =-1)const;			// Datenwert lesen     (als Zeiger)
	void   *SetData    (void *pValue ,int iNr   =-1);				// Datenwert schreiben (als Zeiger)
	int     SetData    (int   iValue ,int iNr   =-1);				// Datenwert schreiben
	void    Sort       (int   iMode=0,int iFlags= 0);				// Dateien sortieren
	int	    Find       (const zchar *pName         )const;			// Suche eine Datei in der Liste
	int	    Find       (const zchar *pName,int iLen)const;			// Suche eine Datei in der Liste
	int		Find	   (const zchar *pName,const zchar *pPath,int iLenN,int iLenP)const;
	int		CheckFilter(const zchar *pName,int iLen)const;		// Einen Dateinamen prfen
	char   	DoneFlag   ()const;										// Wie wurde ein Aufruf beendet
	void	SortClear  ();											// Den Sortiermodus zurck setzen
	iff64	GetSizeAll ()const;										// Gre von allen Dateien holen
	int		InfoOffset ()const;										// Offset zwischen User-Daten und Dateiinfos
	FfStr  *InfoOfUser (void  *pUser)const;							// Dateiinfozeiger aus User-Daten berechnen
	void   *UserOfInfo (FfStr *pInfo)const;							// User-Daten aus Dateiinfozeiger berechnen
	int		SetUserDataSize(int iSize);								// Gre der User-Daten einstellen
	void	ProtectFilter  (int iMode);								// Schreibschutzfilter fr die Suche einstellen
	void	TimeFilter     (int iMode,time_t iTime=0,time_t iEnd=0);// Zeitfilter          fr die Suche einstellen
	void  **GetFieldPointer(int **pCount)const;						// Zeiger auf Datenfeld holen
	void	AttributeFilter(int iOne,int Mask,int iMust);			// Attribut Filter fr Dateien
	void	SetCallback    (FfChangeCb pProc,void *pParam=0,unsigned uInsert=1,unsigned uDelete=2,unsigned uSort=3,unsigned uChange=4,unsigned uLock=0x80000000);

#ifndef CFF_NONET
static	int NetPathConvert(const zchar *pPath,zchar *pBuffer,int Size);
#endif

#ifdef	WIN32
	zchar  *GetType     (int   iNr=-1,int iMode=SHGFI_USEFILEATTRIBUTES);
	int		GetIcon     (int   iNr=-1,int iMode=SHGFI_USEFILEATTRIBUTES|SHGFI_SMALLICON)const;
	int		GetSmallIcon(int   iNr=-1,int iMode=SHGFI_USEFILEATTRIBUTES)const;
	int		GetLargeIcon(int   iNr=-1,int iMode=SHGFI_USEFILEATTRIBUTES)const;
	zchar  *GetType     (FfStr *pInfo,int iMode=SHGFI_USEFILEATTRIBUTES);
	int		GetIcon     (FfStr *pInfo,int iMode=SHGFI_USEFILEATTRIBUTES|SHGFI_SMALLICON)const;
	int		GetSmallIcon(FfStr *pInfo,int iMode=SHGFI_USEFILEATTRIBUTES)const;
	int		GetLargeIcon(FfStr *pInfo,int iMode=SHGFI_USEFILEATTRIBUTES)const;
#endif

protected:

	int  	ExistFile          (const zchar *pName,int iFolder,int iLen)const;
	int		ScanFilter         (const zchar *pList,CStringChain &sFilter,int iAll=1);
	zchar  *BufferNext		   (const zchar *pText);				// Text in inkrementellen Puffer speichern
	FfData *Data               (int iNr)const;						// Zeiger auf Dateidaten holen
	FfData *DataNone           (int iNr)const;						// Zeiger auf Dateidaten holen ohne Indexprfung
	int	    ItemIndex          (int iNr)const;						// Den Index einer Nummer berechnen
	void 	BufferCopy         (int iFolder);						// Pfad nach m_pTempPath kopieren
	int	 	ReadFolder         (int iFolder);						// Dateien  einlesen
	int		InternalNameFullLen(int iNr,int iMode)const;			// Ergibt die Lnge vom Dateinamen
	zchar  *InternalName       (int iNr,int *pLen)const;			// Ergibt Dateinamen ber Index
	zchar  *InternalNameFull   (int iNr,int *pLen,int iMode);		// Ergibt vollen Dateinamen ber Index
	int		InternalDel		   (int iNr      );						// Lscht einen Dateinamen aus der Liste
	FfData *UserToData         (void   *pUser)const;				// Dateidatenzeiger aus User-Datenzeiger berechnen ohne Prfung
	FfStr  *UserToInfo         (void   *pUser)const;				// Dateiinfozeiger  aus User-Datenzeiger berechnen ohne Prfung
	FfData *InfoToData         (FfStr  *pInfo)const;				// Dateidatenzeiger aus Dateiinfozeiger  berechnen ohne Prfung
	void   *InfoToUser         (FfStr  *pInfo)const;				// User-Datenzeiger aus Dateiinfozeiger  berechnen ohne Prfung
	FfStr  *DataToInfo         (FfData *pData)const;				// Dateiinfozeiger  aus Dateidatenzeiger berechnen ohne Prfung
	void   *DataToUser         (FfData *pData)const;				// User-Datenzeiger aus Dateidatenzeiger berechnen ohne Prfung
	int		UserToLength       (void   *pUser)const;				// Dateinamenslnge aus User-Datenzeiger berechnen ohne Prfung
	int		InfoToLength       (FfStr  *pInfo)const;				// Dateinamenslnge aus Dateiinfozeiger  berechnen ohne Prfung
	int		DataToLength       (FfData *pData)const;				// Dateinamenslnge aus Dateidatenzeiger berechnen ohne Prfung
	void 	UserLength         (void   *pUser,int iLen)const;		// Dateinamenslnge via User-Datenzeiger setzen
	void 	AllocTempData	   (			 );						// Zwischenspeicher allokieren
	int		InsertPos		   (             )const;				// Einfgeposition fr Dateien

#ifndef CFF_NONET
	int  	InitNet    (zchar *pPath  );							// Netz Startverzeichnis suchen
	int	 	ReadNet    (int    iFolder);							// Netz      einlesen
	int  	ReadNetPath(int    iFolder);							// Netzpfad  einlesen
	void 	DelShItems ();											// Netzzweige lschen
#endif
	int	 	ReadDrives (int    iFolder);							// Laufwerke einlesen

static int __cdecl SortProc(CStringChain *pChain,int iPos1,int iPos2);
static int __cdecl CmpProc (const void *pPtr1,const void *pPtr2,void *pParam);


//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************
protected:

	char 			m_cDoneFlag;									// Wie wurde ein Aufruf bbeendet (0=ok 1=First 2=Break)
	char 			m_cUserBreak;									// Variabel fr Break()
	int				m_iStartPathLen;								// Startpfadlnge fr relative Pfadausgabe
	unsigned 		m_uCurrentState;								// Aktueller Zustand (0=Neustart 1=Einlesen 0xF0000000=alles eingelesen)
	int				m_iCurrentNum;									// Aktueller Auslesezhler
	int				m_iFolderNum;									// Verzeichnis Zhlernummer
	int				m_iLastError;									// Fehler Nummer der letzten Aktion
	int				m_iFindFlags;									// Diverse iFlags fr die Suche (CFF_FIND_???)
	int				m_iFilerPlus;									// Anzahl der Plus-Filter
	int				m_iFilerMinus;									// Anzahl der Minus-Filter
	int				m_iAttrFilter;									// Bestimmt welche Attribute geprft werden
	int				m_iMustFilter;									// Muss-Wert der Attribute
	int				m_iOneFilter;									// Attribut Eins-Aus-Filter
	int				m_iPathLen;										// Lnge des Puffers bis "...\"
	int				m_iPathNum;										// Nummer des Pfades von sFolder
	int				m_iSortMode;									// Aktueller Sortiermodus der Liste (-1=unsortiert)
	int				m_iUserSize;									// Gre der extra User-Daten
	int				m_iInfoOffset;									// Offset auf die Dateiinfos vom User-Datenfeld
	int				m_iStartFolderNum;								// Verzeichnis Anzahl am Begin der Suche
	int				m_iRdWrFilter;									// Schreibschutz-Filter 0=keine 1=nur schreibgeschtzte 2=nicht schreibgeschtzte
	int				m_iTimeFilter;									// Zeitfiltermodus 	0=egal  1=erstellt vor   2=erstellt nach  3=gendert vor  4=gendert nach
	time_t			m_iTimeValue;									// Zeitwert fr den Zeitfilter
	time_t			m_iTimeUntil;									// Zeitwert fr den Von-Bis-Zeitfilter
	zchar		   *m_pFilePath;									// Ausgangspfad
	zchar		   *m_pTempPath;									// Zwischenspeicher fr Pfad
	zchar		   *m_pBuffer;										// Zwischenspeicher inkremetel
	char		   *m_pTempUser;									// User-Puffer fr's Einfgen
	FfData		   *m_pTempData;									// Datenpuffer fr's Einfgen
	FfFind			m_hNextFile;									// Handle fr's weiter Suchen
	FfComparCb		m_pCmpProc;										// Vergleichsfunktion frs Einfgen
	unsigned		m_uBufferPos;									// Zwischenspeicherposition

	void		   *m_pCbParam;										// Parameter fr Callback
	FfChangeCb		m_pChangeCb;									// Callback fr nderungen
	unsigned		m_uCbInsert;									// Callback Aktion frs Einfgen
	unsigned		m_uCbDelete;									// Callback Aktion frs Lschen
	unsigned		m_uCbChange;									// Callback Aktion frs ndern
	unsigned		m_uCbSort;										// Callback Aktion frs Sortieren
	unsigned		m_uCbLock;										// Callback Aktion frs Sperren

#ifndef CFF_NONET
	static zchar	m_aNetBase[];									// Basispfad fr's Netztwerk
	static int		m_iNetBaseLen;									// Lnge vom Basispfad
	CStringChain	m_sShItems;										// Liste mit Netzdateien
	CStringChain	m_sShFinds;										// Liste mit Netzdateien fr Suche
#endif
	CStringChain    m_sFolders;										// Liste mit Verzeichnisnamen
	CStringChain    m_sFiles;										// Liste mit gefundene Dateien
	CStringChain	m_sMinus;										// Liste mit Dateisuchfilter (minus)
	CStringChain	m_sPlus;										// Liste mit Dateisuchfilter (plus)


};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
#ifndef	_DEBUG
inline	void  	CFileFinder::SortClear   (                  )		{m_iSortMode=-1;}
inline	void   	CFileFinder::Break       (                  )		{m_cUserBreak=1;}
inline	char   	CFileFinder::DoneFlag    (                  )const	{return m_cDoneFlag;}
inline	int		CFileFinder::Error       (                  )const	{return m_iLastError;}
inline	int	   	CFileFinder::InfoOffset  (                  )const	{return m_iInfoOffset;}
inline	int	   	CFileFinder::Count       (                  )const	{return m_sFiles.Count();}
inline	int		CFileFinder::Reset       (                  )		{if(m_uCurrentState>0)m_uCurrentState=1;m_iCurrentNum=0;return m_sFiles.Count();}
inline	int		CFileFinder::NameLen     (int iNr,int iMode )const	{if(iMode==0)return m_sFiles.Len(ItemIndex(iNr));return NameFullLen(iNr,iMode-1);}
inline	int		CFileFinder::NameFullLen (int iNr,int iMode )const	{return InternalNameFullLen(ItemIndex(iNr),iMode);}
inline	int		CFileFinder::NamePostLen (int iNr           )const	{return InternalNameFullLen(ItemIndex(iNr),1    );}
inline	zchar  *CFileFinder::Name        (FfStr*pInfo,int*pL)const	{if(!pInfo){*pL=0;return 0;}*pL=InfoToLength(pInfo);return pInfo->cName;}
inline	zchar  *CFileFinder::Name        (int iNr           )const	{                    return m_sFiles.Get  (ItemIndex(iNr)      );}
inline	zchar  *CFileFinder::Name        (int iNr,int iMode,int *pLen)		{if(iMode==0)return m_sFiles.GetEx(ItemIndex(iNr),pLen );return InternalNameFull(ItemIndex(iNr),pLen,iMode-1);}
inline	zchar  *CFileFinder::NameFull    (int iNr,int *pLen,int iMode)		{return InternalNameFull(ItemIndex(iNr),pLen,iMode     );}
inline	zchar  *CFileFinder::NamePost    (int iNr,int *pLen          )		{return InternalNameFull(ItemIndex(iNr),pLen,1         );}
inline	zchar  *CFileFinder::Suffix		 (int iNr,          int *pLen)const	{void *pUser=m_sFiles.Data(ItemIndex(iNr));return Suffix((pUser)? UserToInfo(pUser):0,      pLen);}
inline	zchar  *CFileFinder::Link        (int iNr,          int *pLen)const	{void *pUser=m_sFiles.Data(ItemIndex(iNr));return Link  ((pUser)? UserToInfo(pUser):0,      pLen);}
inline	zchar  *CFileFinder::Path        (int iNr,int iMode,int *pLen)const	{void *pUser=m_sFiles.Data(ItemIndex(iNr));return Path  ((pUser)? UserToInfo(pUser):0,iMode,pLen);}
inline	zchar  *CFileFinder::CurrentPath (					)const  {return m_pTempPath;}
inline	void   *CFileFinder::UserData    (int iNr           )const	{return               m_sFiles.Data(ItemIndex(iNr));}
inline	void   *CFileFinder::GetDataPtr  (FfStr *pInfo      )const	{if(!pInfo)return 0;return      InfoToData(pInfo)->pUserPtr;}
inline	int     CFileFinder::GetData     (FfStr *pInfo      )const	{if(!pInfo)return 0;return (int)InfoToData(pInfo)->pUserPtr;}
inline	FfData *CFileFinder::Data        (int iNr           )const	{return UserToData(m_sFiles.Data    (iNr));}
inline	FfData *CFileFinder::DataNone    (int iNr           )const	{return UserToData(m_sFiles.DataNone(iNr));}
inline	void   *CFileFinder::SetData     (void *pNew,int iNr)		{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);void	 *pOld=pData->pUserPtr;pData->pUserPtr=pNew;return pOld;}
inline	void   *CFileFinder::GetDataPtr  (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->pUserPtr                ;}
inline	FfStr  *CFileFinder::Info        (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return &pData->sInfo                   ;}
inline	int 	CFileFinder::IsReadOnly  (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_WRITE  ;}
inline	int 	CFileFinder::IsArchive   (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_ARCHIVE;}
inline	int 	CFileFinder::IsDrive     (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_DRIVE  ;}
inline	int		CFileFinder::IsDir       (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_DIR    ;}
inline	int 	CFileFinder::IsNet       (int iNr           )const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return 0;FfData *pData=DataNone(iNr);return  pData->sInfo.dwAttr&SYS_NET    ;}
inline	int		CFileFinder::Len	     (int iNr			)const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return-1;FfData *pData=DataNone(iNr);return  pData->sInfo.dwSize            ;}
inline	int		CFileFinder::Del         (int iNr			)		{return InternalDel(   ItemIndex(iNr)   );}
inline	int		CFileFinder::Attribute	 (int iNr			)const	{return Attribute(Info(ItemIndex(iNr))  );}
inline	iff64	CFileFinder::Len64       (int iNr			)const	{return Len64(Info(ItemIndex(iNr))      );}
inline	time_t	CFileFinder::Time        (int iNr			)const	{return Time (Info(ItemIndex(iNr))      );}
inline	time_t	CFileFinder::Time        (int iNr,int iMode )const	{return Time (Info(ItemIndex(iNr)),iMode);}
inline	time_t 	CFileFinder::TimeWrite   (int iNr			)const	{return Time (Info(ItemIndex(iNr)),    0);}
inline	time_t 	CFileFinder::TimeCreate  (int iNr			)const	{return Time (Info(ItemIndex(iNr)),    1);}
inline	time_t 	CFileFinder::TimeAccess  (int iNr			)const	{return Time (Info(ItemIndex(iNr)),    2);}
inline	time_t 	CFileFinder::TimeWrite   (FfStr *pInfo      )const	{return Time (pInfo,0);}
inline	time_t 	CFileFinder::TimeCreate  (FfStr *pInfo      )const	{return Time (pInfo,1);}
inline	time_t 	CFileFinder::TimeAccess  (FfStr *pInfo      )const	{return Time (pInfo,2);}
inline	int		CFileFinder::Len	     (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwSize:-1;}
inline	int     CFileFinder::IsNet       (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_NET    :0;}
inline	int     CFileFinder::IsDir       (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_DIR    :0;}
inline	int     CFileFinder::IsDrive     (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_DRIVE  :0;}
inline	int		CFileFinder::IsArchive   (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_ARCHIVE:0;}
inline	int		CFileFinder::IsReadOnly  (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr&SYS_WRITE  :0;}
inline	int		CFileFinder::Attribute	 (FfStr *pInfo      )const	{return (pInfo)? pInfo->dwAttr           :-1;}
inline	void   *CFileFinder::UserData    (FfStr *pInfo      )const	{return (pInfo)? InfoToUser(pInfo):0;}
inline	int    	CFileFinder::GetData     (           int iNr)const	{return (int)GetDataPtr           (iNr   );}
inline	int    	CFileFinder::SetData     (int iValue,int iNr)		{return (int)SetData((void*)iValue,iNr   );}
inline	int   	CFileFinder::ItemIndex   (int iNr           )const	{return (iNr!=-1)? iNr:m_iCurrentNum-1;}
inline	int   	CFileFinder::NextIndex   (                  )const	{return  m_iCurrentNum;}
inline	int		CFileFinder::NameLen     (FfStr  *pInfo     )const	{if(!pInfo)return 0;return InfoToLength(pInfo);}
inline	int		CFileFinder::InsertPos	 (                  )const	{if(!m_pCmpProc)return m_sFiles.Count();return m_sFiles.FindFast((void*)this,m_pTempUser,0,m_sFiles.Count(),0x0C,m_pCmpProc);}
inline	FfStr  *CFileFinder::InfoOfUser  (void   *pUser     )const	{return (pUser)? UserToInfo(pUser):0;}
inline	void   *CFileFinder::UserOfInfo  (FfStr  *pInfo     )const	{return (pInfo)? InfoToUser(pInfo):0;}
inline	FfData *CFileFinder::UserToData  (void   *pUser     )const	{return (FfData*)(((char*)pUser)+m_iUserSize  );}
inline	FfStr  *CFileFinder::UserToInfo  (void   *pUser     )const	{return (FfStr *)(((char*)pUser)+m_iInfoOffset);}
inline	FfData *CFileFinder::InfoToData  (FfStr  *pInfo     )const	{return (FfData*)(((char*)pInfo)-((int)(&(((FfData*)0)->sInfo))));}
inline	void   *CFileFinder::InfoToUser  (FfStr  *pInfo     )const	{return (void  *)(((char*)pInfo)-m_iInfoOffset);}
inline	FfStr  *CFileFinder::DataToInfo  (FfData *pData     )const	{return (FfStr *)        &pData->sInfo         ;}
inline	void   *CFileFinder::DataToUser  (FfData *pData     )const	{return (void  *)(((char*)pData)-m_iUserSize  );}
inline	int		CFileFinder::UserToLength(void   *pUser     )const	{return *(int*)(((char*)pUser)              -sizeof(int));}
inline	int		CFileFinder::InfoToLength(FfStr  *pInfo     )const	{return *(int*)(((char*)pInfo)-m_iInfoOffset-sizeof(int));}
inline	int		CFileFinder::DataToLength(FfData *pData     )const	{return *(int*)(((char*)pData)-m_iUserSize  -sizeof(int));}
inline	void 	CFileFinder::UserLength  (void   *pUsr,int i)const	{       *(int*)(((char*)pUsr )-sizeof(int))=i            ;}
inline	void	CFileFinder::TimeFilter  (int iMd,time_t iTi,time_t iTu){m_iTimeFilter=iMd;m_iTimeValue=iTi;m_iTimeUntil=iTu;}
inline	void	CFileFinder::ProtectFilter(int iMode        )		{m_iRdWrFilter=iMode;}
inline	void	CFileFinder::AttributeFilter(int iOne,int iMask,int iMust){m_iOneFilter=iOne;m_iAttrFilter=iMask;m_iMustFilter=iMust;}
inline	void  **CFileFinder::GetFieldPointer(int **pCount)const{return m_sFiles.GetFieldPointer(pCount);}
inline	void	CFileFinder::SetData     (FfPtr &pInfo,void*pValue){if(!pInfo)return;*(void**)(((char*)pInfo)-((int)(&(((FfData*)0)->sInfo))))=pValue;}
inline	void	CFileFinder::SetData     (FfPtr &pInfo,int  iValue){if(!pInfo)return; *(int  *)(((char*)pInfo)-((int)(&(((FfData*)0)->sInfo))))=iValue;}
inline	int		CFileFinder::Find        (const zchar *pName         )const	{return m_sFiles.Find(     pName,0,-1,CSC_MASK_TEST);}
inline	int		CFileFinder::Find        (const zchar *pName,int iLen)const	{return m_sFiles.Find(iLen,pName,0,-1,CSC_MASK_TEST);}

#ifdef	WIN32
inline	zchar  *CFileFinder::GetType     (int    iNr  ,int iMode)		{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return  0;return GetType(&DataNone(iNr)->sInfo,iMode);}
inline	int		CFileFinder::GetIcon     (int    iNr  ,int iMode)const	{iNr=ItemIndex(iNr);if(!m_sFiles.Check(iNr))return -1;return GetIcon(&DataNone(iNr)->sInfo,iMode);}
inline	int	   	CFileFinder::GetSmallIcon(int    iNr  ,int iMode)const	{return GetIcon(iNr  ,iMode|SHGFI_SMALLICON);}
inline	int	   	CFileFinder::GetLargeIcon(int    iNr  ,int iMode)const	{return GetIcon(iNr  ,iMode|SHGFI_LARGEICON);}
inline	int		CFileFinder::GetSmallIcon(FfStr *pInfo,int iMode)const	{return GetIcon(pInfo,iMode|SHGFI_SMALLICON);}
inline	int		CFileFinder::GetLargeIcon(FfStr *pInfo,int iMode)const	{return GetIcon(pInfo,iMode|SHGFI_LARGEICON);}
#endif

#endif

//*****************************************************************************
//*
//*		Klasse CIconChache
//*
//*****************************************************************************
#ifdef	WIN32
class CIconChache
{
public:

	CIconChache();
   ~CIconChache();

	void   *GetItem(const char    *pSuffix);
	void   *GetItem(const wchar_t *pSuffix);
	int		GetIcon(const char    *pSuffix,int iType,void **pHandle=0);
	int		GetIcon(const wchar_t *pSuffix,int iType,void **pHandle=0);
	zchar  *GetType(const char    *pSuffix          ,void **pHandle=0);
	zchar  *GetType(const wchar_t *pSuffix          ,void **pHandle=0);

	void 	SetIcon(int   iType,int iIcon ,void *pHandle);
	zchar  *SetType(const zchar   *pName  ,void *pHandle);
	zchar  *SetType(const zchar   *pName  ,void *pHandle,int iLen);

	static	int			GetDriveIcon(unsigned uLetter,int iMode=SHGFI_SMALLICON);
	static	int			GetPathIcon (                 int iMode=SHGFI_SMALLICON);
	static	unsigned	GetDriveBits();
	static	unsigned	GetBaseDrive();
	static	HIMAGELIST	GetImageList(int iLarge=0);

protected:

	zchar  *NameAdd(const zchar *pName,int iLen);

	typedef CDataList *CDataListPtr;
	typedef struct
		{
		BYTE 			aName[CIC_NAME_MAX];
		INT				aIcon[4];
		zchar		   *pType;
		}IconData;

	void				Clear();
	CDataList		  **m_pList;
	CDataPointer	   *m_pNames;
	CRITICAL_SECTION	m_sSynch;
	friend				CFileFinder;

	static	BYTE		m_aBaseDrive;					// Buchstabe des Systemlaufwerks
	static	BYTE		m_aDriveType [26];				// Ist der Typ vom Laufwerk
	static	INT			m_aDriveIconS[26];				// Icon vom Laufwerk (schmal)
	static	INT			m_aDriveIconL[26];				// Icon vom Laufwerk
	static	UINT		m_aDriveDoneS[26];				// Wann wurde zuletzt das Icon geholt
	static	UINT		m_aDriveDoneL[26];				// Wann wurde zuletzt das Icon geholt
	static	UINT		m_uDriveBits;					// Bits mit vorhandnen Laufwerken
	static	UINT		m_uScanLast;					// Wann wurde zuletzt nach Laufwerken gesucht
	static	HIMAGELIST	m_aImageList[2];				// Handle der Image-Listen (large=0 small=1)


};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
#ifndef		_DEBUG
inline int    CIconChache::GetIcon(const char    *pSuffix,int iType,void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->aIcon[iType]:-1;}
inline int    CIconChache::GetIcon(const wchar_t *pSuffix,int iType,void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->aIcon[iType]:-1;}
inline zchar *CIconChache::GetType(const char    *pSuffix,          void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->pType       : 0;}
inline zchar *CIconChache::GetType(const wchar_t *pSuffix,          void **pHandle){IconData *pData=(IconData*)GetItem(pSuffix);*pHandle=pData;return (pData)? pData->pType       : 0;}
inline void   CIconChache::SetIcon(int iType,int  iIcon,void *pHandle){((IconData*)pHandle)->aIcon[iType]=iIcon;}
inline zchar *CIconChache::SetType(const zchar   *pName,void *pHandle){return SetType(pName,pHandle,strzlen(pName));}
inline zchar *CIconChache::SetType(const zchar   *pName,void *pHandle,int iLen){zchar *pPtr=NameAdd(pName,iLen);((IconData*)pHandle)->pType=pPtr;return pPtr;}
#endif

#endif

#undef	strzlen
#undef	FfData
#undef	FfStr

#endif
