//*****************************************************************************
//*
//*
//*		FileList.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2017
//
//	Das ist eine Klasse um schnell Dateien in einer sortierten Liste zu
//	speicheren. Der Pfad und der Dateiname werden getrennt gespeichert,
//	weiters kann ein golbaler Basispfad definiert werden.
//	Pfade und Dateinamen werden immer sortiert abgespeichert.
//
//
//	Datnenfeld in m_sPaths
//
//		[0]			= CRC vom Pfad
//		[1] 		= Referenzzhler
//
//	Datnenfeld in m_sFiles
//
//		[0..N-1]	= User-Daten
//		[N..N+3]	= Zeiger zum Pfad (im Namensteil)
//		[N+4...] 	= Dateinamen
//
#ifdef		GLOBAL_CONFIG
#define 	STRINGCHAIN_CPP
#include	<GlobalConfig.h>
#undef		STRINGCHAIN_CPP
#endif

#include	<stdio.h>
#include	<stdarg.h>
#include	<string.h>
#include	"FileList.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif


#ifdef		Z
#undef 		Z
#endif

#ifdef		u8
#undef 		u8
#endif

#ifdef		u16
#undef 		u16
#endif

#ifdef		i64
#undef 		i64
#endif

#ifdef		u64
#undef 		u64
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		strzlen
#undef 		strzlen
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	ztype		wchar_t
#define 	strzlen		(int)wcslen
#define 	strzicmp	wcsicmp
#define 	strznicmp	wcsnicmp
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	ztype		unsigned char
#define 	strzlen		(int)strlen
#define 	strzicmp	stricmp
#define 	strznicmp	strnicmp
#endif

#ifndef	   _MSC_VER
#define		i64	 		long long
#define		u64	 		unsigned long long
#define		u16	 		unsigned short
#define		u8	 		unsigned char
#else
#define		i64	 		__int64
#define		u64	 		unsigned __int64
#define		u16	 		unsigned short
#define		u8	 		unsigned char
#endif

#define 	NUMDATA		CFL_NUMDATA
#define 	GETDATA(p)	(*(int**)(p))
#define 	GETNAME(p)	((zchar*)(p))+NUMDATA
#define 	ENTRIES(a)	(sizeof(a)/sizeof((a)[0]))
#define 	SLASHEX		'/'
#define 	SLASH		'\\'

#if CFL_HEAP_CHECK
extern "C"	void		*_crtheap;
#endif


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
//	iDataLen	: Lnge des Datenteils
//	pBasePath	: Ist der Basispfad (bei NULL wird er nicht gesetzt)
//	iBaseSize	: Ist die Lnge des Basispfades
CFileList::CFileList(int iDataLen,const zchar *pBasePath,int iBaseSize)
{

	m_iBaseSize = 0;

	m_sFiles.DataLength(iDataLen);
	m_sPaths.DataLength(sizeof(int)*2);

	if(pBasePath)SetBasePath(iBaseSize,pBasePath);


	#if CFL_HEAP_CHECK
	m_pHeapHandle = _crtheap;									// Ist das Handle des benutzen Heaps
	#endif

}

CFileList::CFileList(int iDataLen,const zchar *pBasePath)
{

	m_iBaseSize = 0;

	m_sFiles.DataLength(iDataLen);
	m_sPaths.DataLength(sizeof(int)*2);

	if(pBasePath)SetBasePath(strzlen(pBasePath),pBasePath);

	#if CFL_HEAP_CHECK
	m_pHeapHandle = _crtheap;									// Ist das Handle des benutzen Heaps
	#endif

}

CFileList::CFileList(const zchar *pBasePath,int iBaseSize)
{

	m_iBaseSize = 0;
	m_sPaths.DataLength(sizeof(int)*2);

	if(pBasePath)SetBasePath(iBaseSize,pBasePath);

	#if CFL_HEAP_CHECK
	m_pHeapHandle = _crtheap;									// Ist das Handle des benutzen Heaps
	#endif

}

CFileList::CFileList(const zchar *pBasePath)
{

	m_iBaseSize = 0;
	m_sPaths.DataLength(sizeof(int)*2);

	if(pBasePath)SetBasePath(strzlen(pBasePath),pBasePath);

	#if CFL_HEAP_CHECK
	m_pHeapHandle = _crtheap;									// Ist das Handle des benutzen Heaps
	#endif

}

CFileList::CFileList(int iDataLen)
{

	m_iBaseSize = 0;

	m_sFiles.DataLength(iDataLen);
	m_sPaths.DataLength(sizeof(int)*2);

	#if CFL_HEAP_CHECK
	m_pHeapHandle = _crtheap;									// Ist das Handle des benutzen Heaps
	#endif

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFileList::~CFileList()
{


}

//*****************************************************************************
//*
//*		IsFullPath
//*
//*****************************************************************************
//	Prft ob ein Pfad eine voller ist ("D:\..." oder "/...")
inline int CFileList::IsFullPath(const zchar *pPath)
{
zchar	cSign;


	cSign = pPath[0]&~0x20;

	if(m_cBuffer[1]==':' && pPath[1]==':' && cSign<='Z' && cSign>='A' )
		{
		return 1;
		}

	if(m_cBuffer[1]!=':' && pPath[0]=='/' && pPath[0]=='\\')
		{
		return 1;
		}


return 0;
}

//*****************************************************************************
//*
//*		SetBasePath
//*
//*****************************************************************************
//	Setzt den Basispfad, ein Backslash wird automatisch angehngt
//	iSize		: Ist die Lnge des Pfades in Zeichen
//	pPath		: Ist der Zeiger auf den Pfad
//	Ergibt die neue Lnge des Basispfades oder 0 bei einem Fehler
int CFileList::SetBasePath(int iSize,const zchar *pPath)
{
int		iPos;


	if((unsigned)iSize>=CFL_MAX_PATH-2)					// Prfe die Lnge
		{
		return 0;
		}

	m_iBaseSize = iSize;
	memcpy(m_cBuffer,pPath,iSize*sizeof(zchar));
														// Backslash anhngen
	if(m_iBaseSize>2 && m_cBuffer[m_iBaseSize-1]!='\\' && m_cBuffer[m_iBaseSize-1]!='/')
		{
		m_cBuffer[m_iBaseSize] = SLASH;
		m_iBaseSize++;
		}

	for(iPos=iSize-1;iPos>=0;iPos--)					// Den Pfad korrigieren
		{
		if(m_cBuffer[iPos]!=SLASHEX)continue;
		m_cBuffer[m_iBaseSize] = SLASH;
		}


return m_iBaseSize;
}

//*****************************************************************************
//*
//*		Del
//*
//*****************************************************************************
//	Einen Eintrag lschen
//	iPos	: Ist die Nummer des Eintrages
//	Ergibt 1 wenn der Eintrag gelscht wurde, bzw. 0 wenn nicht.
int CFileList::Del(int iPos)
{
zchar	   *pName;
int		   *pData;


	#if CFL_HEAP_CHECK
	if(m_pHeapHandle!=_crtheap)							// Wird der falsche Heap benutzt
		{
		_asm int 3;
		}
	#endif

	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName = m_sFiles.GetNone(iPos);
	pData = GETDATA(pName);

	m_sFiles.Del(iPos);

	pData[1]--;											// Referenzzhler verringern

	if(pData[1]<=0)										// Auch den Pfad lschen
		{
		iPos = m_sPaths.FindFast(pData[1],pData[-1],(zchar*)(pData+2),0,-1,0x01);
		m_sPaths.Del(iPos);
		}


return 1;
}

//*****************************************************************************
//*
//*		CmpPath
//*
//*****************************************************************************
//	Vergeicht zwei Pfade
//	pPath1		: Zeiger auf den ersten Pfad
//	pPath2		: Zeiger auf den zweiten Pfad
//	iSize		: Ist die Vergleichslnge
int	CFileList::CmpPath(const zchar *pPath1,const zchar *pPath2,int iSize)
{
unsigned char 	ucSign1;
unsigned char 	ucSign2;
int				iDiff;
int				iPos;



	for(iPos=0;iPos<iSize;iPos++)
		{
		if(pPath1[iPos]==pPath2[iPos])continue;

		ucSign1 = pPath1[iPos];
		ucSign2 = pPath2[iPos];

		     if(ucSign1<='Z' && ucSign1>='A' )ucSign1+=0x20;
		else if(ucSign1=='/'                 )ucSign1 ='\\';

		     if(ucSign2<='Z' && ucSign2>='A' )ucSign2+=0x20;
		else if(ucSign2=='/'                 )ucSign2 ='\\';

		iDiff = ucSign1-ucSign2;

		if(iDiff)return iDiff;
		}


return 0;
}

//*****************************************************************************
//*
//*		CmpFile
//*
//*****************************************************************************
//	Vergleicht zwei Dateinamen
int CFileList::CmpFile(const zchar *pFile1,const zchar *pFile2,int iSize)
{
char   *pPath1;
char   *pPath2;


	pPath1	= *(char**)pFile1;
	pPath2	= *(char**)pFile2;

	if(pPath1!=pPath2)									// Zeiger auf den Pfad vergleichen
		{
		return (pPath1<pPath2)? -1:1;
		}

return strzicmp(pFile1+sizeof(char*),pFile2+sizeof(char*));
}

//*****************************************************************************
//*
//*		CmpName
//*
//*****************************************************************************
//	Vergleicht zwei Dateinamen, der erste Name wird ohne Pfad bergeben
int CFileList::CmpName(const zchar *pFile1,const zchar *pFile2,int iSize)
{
int		iRet;


	   iRet = strznicmp(pFile1,pFile2+sizeof(char*),iSize-sizeof(char*));
	if(iRet)return iRet;

	if(pFile2[iSize])return -1;


return 0;
}

//*****************************************************************************
//*
//*		GetBase
//*
//*****************************************************************************
//	Eine Pfad holen, der Pfad wird einmalig in einem in einem temprren Puffer gespeichert.
//	iPos		: Ist die Nummer des Eintrages
//	pLen		: Hier wird die Lnge gespeichert
//	Ergibt einen Zeiger auf den Pfad oder NULL.
zchar *CFileList::GetBase(int iPos,int *pLen)
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iSize;
int		iNum;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName	= m_sFiles.GetNone(iPos);
	iSize	= m_sFiles.LenNone(iPos);
	pData	= GETDATA (pName);
	pPath	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iSize  -= NUMDATA;
														// Den Pfad zusammen bauen
	memcpy(m_cBuffer+m_iBaseSize     ,pPath,iNum *sizeof(zchar));
	memcpy(m_cBuffer+m_iBaseSize+iNum,pName,iSize*sizeof(zchar)+sizeof(zchar));

	if(pLen)*pLen=m_iBaseSize+iNum+iSize;


return m_cBuffer;
}

//*****************************************************************************
//*
//*		GetBase
//*
//*****************************************************************************
//	Eine Pfad holen, der Pfad wird einmalig in einem in einem temprren
//	Puffer gespeichert, der nur bis zum nchsten Zugriff auf die Klasse gltig ist.
//	iPos		: Ist die Nummer des Eintrages
//	iLen		: Hier wird die Lnge gespeichert
//	iOff		: Hier wird der Offset des Dateinamens gespeichert
//	Ergibt einen Zeiger auf den Pfad oder NULL.
zchar *CFileList::GetBase(int iPos,int &iLen,int &iOff)
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iSize;
int		iNum;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName	= m_sFiles.GetNone(iPos);
	iSize	= m_sFiles.LenNone(iPos);
	pData	= GETDATA (pName);
	pPath	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iSize  -= NUMDATA;
														// Den Pfad zusammen bauen
	memcpy(m_cBuffer+m_iBaseSize     ,pPath,iNum *sizeof(zchar));
	memcpy(m_cBuffer+m_iBaseSize+iNum,pName,iSize*sizeof(zchar)+sizeof(zchar));

	iOff	= m_iBaseSize+iNum;
	iLen	= m_iBaseSize+iNum+iSize;


return m_cBuffer;
}



//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Eine Dateipfad holen, der Pfad wird einmalig in einem in einem temprren
//	Puffer gespeichert, der nur bis zum nchsten Zugriff auf die Klasse gltig ist.
//	iPos		: Ist die Nummer des Eintrages
//	pLen		: Hier wird die Lnge gespeichert
//	Ergibt einen Zeiger auf den Dateipfad (Pfad und Name )oder NULL bei einem Fehler.
zchar *CFileList::Get(int iPos,int *pLen)
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iSize;
int		iNum;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName	= m_sFiles.GetNone(iPos);
	iSize	= m_sFiles.LenNone(iPos);
	pData	= GETDATA (pName);
	pPath	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iSize  -= NUMDATA;
														// Den Pfad zusammen bauen
	memcpy(m_cBuffer+m_iBaseSize     ,pPath,iNum *sizeof(zchar));
	memcpy(m_cBuffer+m_iBaseSize+iNum,pName,iSize*sizeof(zchar)+sizeof(zchar));

	if(pLen)*pLen=iNum+iSize;


return m_cBuffer+m_iBaseSize;
}

//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Einen Dateipfad holen ohne Basispfad, der Pfad wird einmalig in einem temprren
//	Puffer gespeichert, der nur bis zum nchsten Zugriff auf die Klasse gltig ist.
//	iPos		: Ist die Nummer des Eintrages
//	iLen		: Hier wird die Lnge gespeichert
//	iOff		: Hier wird der Offset des Dateinamens gespeichert
//	Ergibt einen Zeiger auf den Dateipfad (Pfad und Name )oder NULL bei einem Fehler.
zchar *CFileList::Get(int iPos,int &iLen,int &iOff)
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iSize;
int		iNum;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName	= m_sFiles.GetNone(iPos);
	iSize	= m_sFiles.LenNone(iPos);
	pData	= GETDATA (pName);
	pPath	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iSize  -= NUMDATA;
														// Den Pfad zusammen bauen
	memcpy(m_cBuffer+m_iBaseSize     ,pPath,iNum *sizeof(zchar));
	memcpy(m_cBuffer+m_iBaseSize+iNum,pName,iSize*sizeof(zchar)+sizeof(zchar));

	iLen	= iNum+iSize;
	iOff	= iNum;


return m_cBuffer+m_iBaseSize;
}

//*****************************************************************************
//*
//*		GetBase
//*
//*****************************************************************************
//	Eine Pfad (Base+Pfad+Name) holen, der Pfad wird einmalig in einem
//	iPos		: Ist die Nummer des Eintrages
//	pBuffer		: Hier wird der Pfad gespeichert (Base+Pfad+Name)
//	iSize		: Ist die Gre des Pfadpuffers
//	Ergibt Lnge der erstellen Pfades.
int CFileList::GetBase(int iPos,zchar *pBuffer,int iSize)const
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iLen;
int		iNum;
int		iAll;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName 	= m_sFiles.GetNone(iPos);
	iLen  	= m_sFiles.LenNone(iPos);
	pData 	= GETDATA (pName);
	pPath 	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iLen   -= NUMDATA;
	iAll	= m_iBaseSize+iNum+iLen;

	if(iAll>=iSize)return NULL;
														// Den Pfad zusammen bauen
	memcpy(pBuffer,m_cBuffer,      m_iBaseSize*sizeof(zchar));
	memcpy(pBuffer+m_iBaseSize     ,pPath,iNum*sizeof(zchar));
	memcpy(pBuffer+m_iBaseSize+iNum,pName,iLen*sizeof(zchar)+sizeof(zchar));


return iAll;
}

//*****************************************************************************
//*
//*		GetBase
//*
//*****************************************************************************
//	Eine Pfad (Base+Pfad+Name) holen, der Pfad wird einmalig in einem
//	iPos		: Ist die Nummer des Eintrages
//	pBuffer		: Hier wird der Pfad gespeichert (Base+Pfad+Name)
//	iOff		: Hier wird der Datei-Offset gespeichert
//	iSize		: Ist die Gre des Pfadpuffers
//	Ergibt Lnge der erstellen Pfades.
int CFileList::GetBase(int iPos,zchar *pBuffer,int &iOff,int iSize)const
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iLen;
int		iNum;
int		iAll;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName 	= m_sFiles.GetNone(iPos);
	iLen  	= m_sFiles.LenNone(iPos);
	pData 	= GETDATA (pName);
	pPath 	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iLen   -= NUMDATA;
	iAll	= m_iBaseSize+iNum+iLen;

	if(iAll>=iSize)return NULL;
														// Den Pfad zusammen bauen
	memcpy(pBuffer,m_cBuffer,      m_iBaseSize*sizeof(zchar));
	memcpy(pBuffer+m_iBaseSize     ,pPath,iNum*sizeof(zchar));
	memcpy(pBuffer+m_iBaseSize+iNum,pName,iLen*sizeof(zchar)+sizeof(zchar));

	iOff	= m_iBaseSize+iNum;


return iAll;
}

//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Einen Dateipfad holen ohne Basispfad, der Pfad wird einmalig in einem
//	iPos		: Ist die Nummer des Eintrages
//	pBuffer		: Hier wird der Pfad gespeichert (Pfad+Name)
//	iSize		: Ist die Gre des Pfadpuffers
//	Ergibt Lnge der erstellen Pfades.
int CFileList::Get(int iPos,zchar *pBuffer,int iSize)const
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iLen;
int		iNum;
int		iAll;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName 	= m_sFiles.GetNone(iPos);
	iLen  	= m_sFiles.LenNone(iPos);
	pData 	= GETDATA (pName);
	pPath 	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iLen   -= NUMDATA;
	iAll	= iNum+iLen;

	if(iAll>=iSize)return NULL;
														// Den Pfad zusammen bauen
	memcpy(pBuffer     ,pPath,iNum*sizeof(zchar));
	memcpy(pBuffer+iNum,pName,iLen*sizeof(zchar)+sizeof(zchar));


return iAll;
}


//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Einen Dateipfad holen ohne Basispfad, der Pfad wird einmalig in einem
//	iPos		: Ist die Nummer des Eintrages
//	pBuffer		: Hier wird der Pfad gespeichert (Pfad+Name)
//	iOff		: Hier wird der Datei-Offset gespeichert
//	iSize		: Ist die Gre des Pfadpuffers
//	Ergibt Lnge der erstellen Pfades.
int CFileList::Get(int iPos,zchar *pBuffer,int &iOff,int iSize)const
{
zchar  *pPath;
zchar  *pName;
int	   *pData;
int		iLen;
int		iNum;
int		iAll;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName 	= m_sFiles.GetNone(iPos);
	iLen  	= m_sFiles.LenNone(iPos);
	pData 	= GETDATA (pName);
	pPath 	= (zchar*)(pData+2);
	iNum	= pData[-1];
	pName  += NUMDATA;
	iLen   -= NUMDATA;
	iAll	= iNum+iLen;

	if(iAll>=iSize)return NULL;
														// Den Pfad zusammen bauen
	memcpy(pBuffer     ,pPath,iNum*sizeof(zchar));
	memcpy(pBuffer+iNum,pName,iLen*sizeof(zchar)+sizeof(zchar));

	iOff	= iNum;


return iAll;
}


//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht nach einen Pfad in der Liste.
//	iSize		: Ist die Lnge des Pfades mit Dateinamen
//	pPath		: Ist der Zeiger auf den Pfad mit Dateinamen
//	Ergibt die Position wo der Eintrag gefunden wurde, bzw. -1 wenn nicht.
int CFileList::Find(int iSize,const zchar *pPath)
{
int 	   *pData;
int			iStop;
int			iPos;
int			iNum;
unsigned 	uTag;



	m_iTargetOff = 0;

	if(m_iBaseSize && *pPath && IsFullPath(pPath))			// Den Vollen Pfad prfen
		{
		if(CmpPath(pPath,m_cBuffer,m_iBaseSize))
			{
			return -1;
			}

		m_iTargetOff += m_iBaseSize;
		pPath        += m_iBaseSize;
		iSize		 -= m_iBaseSize;
		}
	else if(iSize>0 && (*pPath=='/' || *pPath=='\\'))		// Slash am Beginn entfernen
		{
		m_iTargetOff += 1;
		pPath		 += 1;
		iSize		 -= 1;
		}


	for(iStop=iSize;iStop>0;iStop--)						// Sucht das Ende vom Pfad
		{
		if(pPath[iStop-1]=='\\')break;
		if(pPath[iStop-1]=='/' )break;
		}


	m_uPathCrc		= 0;
	m_iTargetLen	= iStop;
	m_iTargetOff   += iStop;

	for(iPos=0;iPos<iStop;iPos++)							// CRC vom Pfad berechnen
		{
		uTag = (ztype)pPath[iPos];

		if(uTag<='Z' && uTag>='A')
			{
			m_cTarget[iPos] = (zchar)uTag;
			uTag		   +=0x20;
			}
		else if(uTag==SLASHEX)
			{
			m_cTarget[iPos]	= SLASH;
			uTag			= SLASH;
			}
		else{
			m_cTarget[iPos] = (zchar)uTag;
			}

		m_uPathCrc  = (m_uPathCrc<<6)|(m_uPathCrc>>26);
		m_uPathCrc += uTag;
		}



															// Suche den Pfad
	   m_iPathPos = m_sPaths.FindFast(m_uPathCrc,iStop,m_cTarget,0,-1,0x01);
	if(m_iPathPos<0)return m_iPathPos;

	pData = (int*)m_sPaths.DataNone(m_iPathPos);

	m_iFileOff = (iPos+NUMDATA-1)&~(NUMDATA-1);
	iNum	   =  iSize-iStop;

	GETDATA(m_cTarget+m_iFileOff) = pData;					// Dateinamen mit Pfadzeiger erstellen
	memcpy (m_cTarget+m_iFileOff+NUMDATA,pPath+iStop,iNum);
	m_cTarget[m_iFileOff+NUMDATA+iNum] = 0;

	m_iFileLen = iNum+NUMDATA;
	m_iFilePos = m_sFiles.FindFast(m_iFileLen,m_cTarget+m_iFileOff,0,-1,0x01,CmpFile);




return m_iFilePos;
}


//*****************************************************************************
//*
//*		FindEx
//*
//*****************************************************************************
//	Sucht nach einen Pfad mit Dateinamen in der Liste.
//	iSize		: Ist die Lnge des Pfades ohne Dateinamen
//	pPath		: Ist der Zeiger auf den Pfad ohne Dateinamen
//	iLen		: Ist die Lnge des Dateinamen ohne Pfad
//	pFile		: Ist der Zeiger auf den Dateinamen ohne Pfad
//	Ergibt die Position wo der Eintrag gefunden wurde, bzw. -1 wenn nicht.
int CFileList::FindEx(int iSize,const zchar *pPath,int iLen,const zchar *pFile)
{
int 	   *pData;
int			iPos;
unsigned	uTag;



	m_iTargetOff = 0;

	if(m_iBaseSize && *pPath && IsFullPath(pPath))			// Den Vollen Pfad prfen
		{
		if(CmpPath(pPath,m_cBuffer,m_iBaseSize))
			{
			return -1;
			}

		m_iTargetOff += m_iBaseSize;
		pPath        += m_iBaseSize;
		iSize		 -= m_iBaseSize;
		}
	else if(iSize>0 && (*pPath=='/' || *pPath=='\\'))		// Slash am Beginn entfernen
		{
		m_iTargetOff += 1;
		pPath		 += 1;
		iSize		 -= 1;
		}


	m_uPathCrc = 0;

	for(iPos=0;iPos<iSize;iPos++)							// CRC berechnen
		{
		uTag = (ztype)pPath[iPos];

		if(uTag<='Z' && uTag>='A')
			{
			m_cTarget[iPos] = (zchar)uTag;
			uTag		   +=0x20;
			}
		else if(uTag==SLASHEX)
			{
			m_cTarget[iPos]	= SLASH;
			uTag			= SLASH;
			}
		else{
			m_cTarget[iPos] = (zchar)uTag;
			}

		m_uPathCrc  = (m_uPathCrc<<6)|(m_uPathCrc>>26);
		m_uPathCrc += uTag;
		}


	if(iSize>0 && m_cTarget[iSize-1]!=SLASH)				// Slash am Ende anhngen
		{
		m_cTarget[iSize]	= SLASH;
		m_uPathCrc			= (m_uPathCrc<<6)|(m_uPathCrc>>26);
		m_uPathCrc		   += SLASH;

		iSize++;
		}


	m_iTargetLen	= iSize;
	m_iTargetOff   += iSize;

															// Suche den Pfad
	   m_iPathPos = m_sPaths.FindFast(m_uPathCrc,iSize,m_cTarget,0,-1,0x01);
	if(m_iPathPos<0)return m_iPathPos;

	pData = (int*)m_sPaths.DataNone(m_iPathPos);

	m_iFileOff  = (iPos+NUMDATA-1)&~(NUMDATA-1);

	GETDATA(m_cTarget+m_iFileOff) = pData;					// Dateinamen mit Pfadzeiger erstellen
	memcpy (m_cTarget+m_iFileOff+NUMDATA,pFile,iLen);
	m_cTarget[m_iFileOff+NUMDATA+iLen] = 0;

	m_iFileLen = iLen+NUMDATA;
	m_iFilePos = m_sFiles.FindFast(m_iFileLen,m_cTarget+m_iFileOff,0,-1,0x01,CmpFile);


return m_iFilePos;
}



//*****************************************************************************
//*
//*		FindPath
//*
//*****************************************************************************
//	Sucht nach einen Pfad ohne Datei in der Liste.
//	iSize		: Ist die Lnge des Pfades ohne Dateinamen
//	pPath		: Ist der Zeiger auf den Pfad ohne Dateinamen
//	Ergibt die Position wo der Eintrag gefunden wurde, bzw. -1 wenn nicht.
int CFileList::FindPath(int iSize,const zchar *pPath)
{
int			iStop;
int			iPos;
unsigned 	uTag;



	m_iTargetOff = 0;

	if(m_iBaseSize && *pPath && IsFullPath(pPath))			// Den Vollen Pfad prfen
		{
		if(CmpPath(pPath,m_cBuffer,m_iBaseSize))
			{
			return -1;
			}

		m_iTargetOff += m_iBaseSize;
		pPath        += m_iBaseSize;
		iSize		 -= m_iBaseSize;
		}
	else if(iSize>0 && (*pPath=='/' || *pPath=='\\'))		// Slash am Beginn entfernen
		{
		m_iTargetOff += 1;
		pPath		 += 1;
		iSize		 -= 1;
		}


	for(iStop=iSize;iStop>0;iStop--)						// Such das Ende vom Pfad
		{
		if(pPath[iStop-1]=='\\')break;
		if(pPath[iStop-1]=='/' )break;
		}


	m_uPathCrc = 0;

	for(iPos=0;iPos<iStop;iPos++)							// CRC berechnen
		{
		uTag = (ztype)pPath[iPos];

		if(uTag<='Z' && uTag>='A')
			{
			m_cTarget[iPos] = (zchar)uTag;
			uTag		   +=0x20;
			}
		else if(uTag==SLASHEX)
			{
			m_cTarget[iPos]	= SLASH;
			uTag			= SLASH;
			}
		else{
			m_cTarget[iPos] = (zchar)uTag;
			}

		m_uPathCrc  = (m_uPathCrc<<6)|(m_uPathCrc>>26);
		m_uPathCrc += uTag;
		}


	if(iStop>0 && m_cTarget[iStop-1]!=SLASH)				// Slash am Ende anhngen
		{
		m_cTarget[iStop]	= SLASH;
		m_uPathCrc			= (m_uPathCrc<<6)|(m_uPathCrc>>26);
		m_uPathCrc		   += SLASH;

		iStop++;
		}


	m_iTargetLen	= iStop;
	m_iTargetOff   += iStop;

															// Suche den Pfad
	m_iPathPos = m_sPaths.FindFast(m_uPathCrc,iStop,m_cTarget,0,-1,0x01);



return m_iPathPos;
}


//*****************************************************************************
//*
//*		FindPath
//*
//*****************************************************************************
//	Such nach einen Pfad ohne Dateinamen in der Liste.
//	Fr die Datei wird der Bereich ermittelt.
//	iSize		: Ist die Lnge des Pfades ohne Dateinamen
//	pPath		: Ist der Zeiger auf den Pfad ohne Dateinamen
//	iFrom		: Position der ersten Datei
//	iLast		: Position der letzten Datei (N+1)
//	Ergibt die Position wo der Eintrag gefunden wurde, bzw. -1 wenn nicht.
int CFileList::FindPath(int iSize,const zchar *pPath,int &iFrom,int &iLast)
{
char  	   *pData[2];
int			iPos;
unsigned 	uTag;



	m_iTargetOff = 0;

	if(m_iBaseSize && *pPath && IsFullPath(pPath))			// Den Vollen Pfad prfen
		{
		if(CmpPath(pPath,m_cBuffer,m_iBaseSize))
			{
			return -1;
			}

		m_iTargetOff += m_iBaseSize;
		pPath        += m_iBaseSize;
		iSize		 -= m_iBaseSize;
		}
	else if(iSize>0 && (*pPath=='/' || *pPath=='\\'))		// Slash am Beginn entfernen
		{
		m_iTargetOff += 1;
		pPath		 += 1;
		iSize		 -= 1;
		}


	m_uPathCrc = 0;

	for(iPos=0;iPos<iSize;iPos++)							// CRC berechnen
		{
		uTag = (ztype)pPath[iPos];

		if(uTag<='Z' && uTag>='A')
			{
			m_cTarget[iPos] = (zchar)uTag;
			uTag		   += 0x20;
			}
		else if(uTag==SLASHEX)
			{
			m_cTarget[iPos]	= SLASH;
			uTag			= SLASH;
			}
		else{
			m_cTarget[iPos] = (zchar)uTag;
			}

		m_uPathCrc  = (m_uPathCrc<<6)|(m_uPathCrc>>26);
		m_uPathCrc += uTag;
		}


	if(iSize>0 && m_cTarget[iSize-1]!=SLASH)				// Slash am Ende anhngen
		{
		m_cTarget[iSize]	= SLASH;
		m_uPathCrc			= (m_uPathCrc<<6)|(m_uPathCrc>>26);
		m_uPathCrc		   += SLASH;

		iSize++;
		}


	m_iTargetLen	= iSize;
	m_iTargetOff   += iSize;

															// Suche den Pfad
	   m_iPathPos = m_sPaths.FindFast(m_uPathCrc,iSize,m_cTarget,0,-1,0x01);
	if(m_iPathPos<0)
		{
		iFrom = -1;
		iLast = -1;

		return -1;
		}


	pData[0]  = (char*)m_sPaths.DataNone(m_iPathPos);
	pData[1]  = 0;
	iFrom	  = m_sFiles.FindFast(NUMDATA,(char*)pData,    0,-1,0x09,CmpFile);
	pData[0] += 1;
	iLast	  = m_sFiles.FindFast(NUMDATA,(char*)pData,iFrom,-1,0x09,CmpFile);



return m_iPathPos;
}

//*****************************************************************************
//*
//*		Add
//*
//*****************************************************************************
//	Einen neuen Dateipfad einfgen (Pfad und Dateinamen)
//	iSize		: Ist die Lnge des Dateipfades (Pfad und Dateinamen)
//	pPath		: Ist der Zeiger auf den Pfad mit Dateinamen
//	pPtr		: Ist der Datenblock der miteingefgt wird
//	Ergibt die Einfgeposition
int CFileList::Add(int iSize,const zchar *pPath,const void *pPtr)
{
int			aData[2];
int		   *pData;
int			iPos;
int			iNum;


	#if CFL_HEAP_CHECK
	if(m_pHeapHandle!=_crtheap)								// Wird der falsche Heap benutzt
		{
		_asm int 3;
		}
	#endif


	   iPos = Find(iSize,pPath);
	if(iPos>=0)return iPos;

	if(m_iPathPos<0)										// Einen neuen Pfad erstellen
		{
		aData[1]	= 0;
		aData[0]	= m_uPathCrc;
		m_iPathPos	= m_sPaths.Add(m_iTargetLen,m_cTarget,CSC_UABCG,aData);
		pData		= (int*)m_sPaths.DataNone(m_iPathPos);

		m_iFileOff  = (m_iTargetLen+NUMDATA-1)&~(NUMDATA-1);
		iNum		=  iSize-m_iTargetOff;

		GETDATA(m_cTarget+m_iFileOff) = pData;				// Dateinamen mit Pfadzeiger erstellen
		memcpy (m_cTarget+m_iFileOff+NUMDATA,pPath+m_iTargetOff,iNum);
		m_cTarget[m_iFileOff+NUMDATA+iNum] = 0;

		m_iFileLen	= iNum+NUMDATA;
		}

	pData = (int*)m_sPaths.Data(m_iPathPos);
	pData[1]++;												// Referenzzhler vom Pfad erhhen

	m_iFilePos = m_sFiles.Add(m_iFileLen,m_cTarget+m_iFileOff,CmpFile,pPtr);



return m_iFilePos;
}


//*****************************************************************************
//*
//*		AddEx
//*
//*****************************************************************************
//	Einen neuen Pfad mit Dateinamen einfgen
//	iSize		: Ist die Lnge des Pfades
//	pPath		: Ist der Zeiger auf den Pfad ohne Dateinamen
//	pPtr		: Ist der Datenblock der miteingefgt wird
//	iLen		: Ist die Lnge des Dateinamens
//	pFile		: Ist der Zeiger auf den Dateinamen
//	Ergibt die Einfgeposition
int CFileList::AddEx(int iSize,const zchar *pPath,int iLen,const zchar *pFile,const void *pPtr)
{
int			aData[2];
int		   *pData;
int			iPos;


	#if CFL_HEAP_CHECK
	if(m_pHeapHandle!=_crtheap)								// Wird der falsche Heap benutzt
		{
		_asm int 3;
		}
	#endif


	   iPos = FindEx(iSize,pPath,iLen,pFile);
	if(iPos>=0)return iPos;

	if(m_iPathPos<0)										// Einen neuen Pfad erstellen
		{
		aData[1]	= 0;
		aData[0]	= m_uPathCrc;
		m_iPathPos	= m_sPaths.Add(m_iTargetLen,m_cTarget,CSC_UABCG,aData);
		pData		= (int*)m_sPaths.DataNone(m_iPathPos);

		m_iFileOff  = (m_iTargetLen+NUMDATA-1)&~(NUMDATA-1);

		GETDATA(m_cTarget+m_iFileOff) = pData;				// Dateinamen mit Pfadzeiger erstellen
		memcpy (m_cTarget+m_iFileOff+NUMDATA,pFile,iLen);
		m_cTarget[m_iFileOff+NUMDATA+iLen] = 0;

		m_iFileLen	= iLen+NUMDATA;
		}

	pData = (int*)m_sPaths.Data(m_iPathPos);
	pData[1]++;												// Referenzzhler vom Pfad erhhen

	m_iFilePos = m_sFiles.Add(m_iFileLen,m_cTarget+m_iFileOff,CmpFile,pPtr);


return m_iFilePos;
}


//*****************************************************************************
//*
//*		GetPath
//*
//*****************************************************************************
//	Den Pfad zu einer Datei holen (ohne Dateinamen, und Basispfad)
zchar *CFileList::GetPath(int iPos,int &iLen)const
{
zchar  *pName;
int	   *pData;


	if(!m_sFiles.Check(iPos))							// Existiert der Eintrag
		{
		iLen = 0;
		return 0;
		}

	pName = m_sFiles.Get(iPos);
	pData = GETDATA(pName);
	iLen  = pData[-1];


return (zchar*)(pData+2);
}

zchar *CFileList::GetPath(int iPos)const
{
zchar  *pName;
int	   *pData;


	if(!m_sFiles.Check(iPos))return 0;					// Existiert der Eintrag

	pName = m_sFiles.Get(iPos);
	pData = GETDATA(pName);


return (zchar*)(pData+2);
}

