//*****************************************************************************
//*
//*
//*		FileList.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2017
//
//	Das ist eine Klasse um schnell Dateien in einer sortierten Liste zu
//	speicheren. Der Pfad und der Dateiname werden getrennt gespeichert,
//	weiters kann ein golbaler Basispfad definiert werden.
//	Pfade und Dateinamen werden immer sortiert abgespeichert.
//
#ifndef 	__FILELIEST_H__
#define 	__FILELIEST_H__



#if _MSC_VER >= 1000
#pragma once
#pragma	warning( disable :4514 )
#pragma warning( disable :4996 )
#else
#define 	 _cdecl
#define 	__cdecl
#endif


#ifndef		 strcpy
#include	<string.h>
#endif


#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if		_MSC_VER > 1299
	#ifndef		WINVER
	#define		WINVER 	0x0501
	#endif
	#endif
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#endif


#ifndef _WCHAR_T_DEFINED
	#if _MSC_VER
	#include	<wchar.h>
	#else
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif
#endif


#ifndef		CFL_HEAP_CHECK
#define 	CFL_HEAP_CHECK	0
#else
#if			CFL_HEAP_CHECK	&& (!defined(_DEBUG) || !defined(_MSC_VER))
#undef		CFL_HEAP_CHECK
#define 	CFL_HEAP_CHECK	0
#endif
#endif



#include	"StringChain.h"


#ifdef		Z
#undef 		Z
#endif

#ifdef		u8
#undef 		u8
#endif

#ifdef		u16
#undef 		u16
#endif

#ifdef		i64
#undef 		i64
#endif

#ifdef		u64
#undef 		u64
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		strzlen
#undef 		strzlen
#endif


#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	strzlen		(int)wcslen
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	strzlen		(int)strlen
#endif

#ifndef	   _MSC_VER
#define		i64	 		long long
#define		u64	 		unsigned long long
#define		u16	 		unsigned short
#define		u8	 		unsigned char
#else
#define		i64	 		__int64
#define		u64	 		unsigned __int64
#define		u16	 		unsigned short
#define		u8	 		unsigned char
#endif


#define 	CFL_MAX_PATH	256
#define 	CFL_NUMDATA		(sizeof(void*)/sizeof(zchar))


//*****************************************************************************
//*
//*		Klasse CFileList
//*
//*****************************************************************************
class CFileList
{
public:

	CFileList(int iDataLen,const char *pBasePath,int iBaseSize);
	CFileList(int iDataLen,const char *pBasePath              );
	CFileList(             const char *pBasePath,int iBaseSize);
	CFileList(             const char *pBasePath              );
	CFileList(int iDataLen=0);

   ~CFileList(void);

	int		Del     ();												// Alles lschen
	int 	Del     (int iPos);										// Einen Eintrag lschen
	int 	Del     (int iSize,const zchar *pPath);					// Einen Eintrag suchen und lschen (mit Lnge)
	int 	Del     (          const zchar *pPath);					// Einen Eintrag suchen und lschen
	int 	Add     (int iSize,const zchar *pPath,const void *pPtr=0);	// Einen neuen Pfad einfgen (mit Lnge)
	int 	Add     (          const zchar *pPath,const void *pPtr=0);	// Einen neuen Pfad einfgen
	int 	AddEx   (int iSize,const zchar *pPath,int iLen,const zchar *pFile,const void *pPtr=0);	// Einen neuen Pfad einfgen (mit Lnge)
	int 	AddEx   (          const zchar *pPath,int iLen,const zchar *pFile,const void *pPtr=0);	// Einen neuen Pfad einfgen
	int 	AddEx   (int iSize,const zchar *pPath,         const zchar *pFile,const void *pPtr=0);	// Einen neuen Pfad einfgen
	int 	AddEx   (          const zchar *pPath,         const zchar *pFile,const void *pPtr=0);	// Einen neuen Pfad einfgen

	int 	Find    (int iSize,const zchar *pPath);					// Nach einen Pfad suchen (mit Lnge)
	int 	Find    (          const zchar *pPath);					// Nach einen Pfad suchen
	int 	FindEx  (int iSize,const zchar *pPath,int iLen,const zchar *pFile);	// Nach einen Pfad und Datei suchen (mit Lnge)
	int 	FindEx  (          const zchar *pPath,int iLen,const zchar *pFile);	// Nach einen Pfad und Datei suchen
	int 	FindEx  (int iSize,const zchar *pPath         ,const zchar *pFile);	// Nach einen Pfad und Datei suchen
	int 	FindEx  (          const zchar *pPath         ,const zchar *pFile);	// Nach einen Pfad und Datei suchen
	int 	FindName(int iLen ,const zchar *pFile,int iFrom,int iLast);			// Nach einer Datei suchen in Bereich (mit Lnge)
	int 	FindName(          const zchar *pFile,int iFrom,int iLast);			// Nach einer Datei suchen in Bereich

	zchar  *Get     (int iPos,int *pLen=0);							// Einen Dateipfad ohne Basispfad holen
	zchar  *Get     (int iPos,int &iLen,int &iOff);					// Einen Dateipfad ohne Basispfad holen und Dateinamens-Offset
	int		Get	    (int iPos,zchar *pBuf,          int iSize)const;// Einen Dateipfad ohne Basispfad via Puffer holen
	int		Get	    (int iPos,zchar *pBuf,int &iOff,int iSize)const;// Einen Dateipfad ohne Basispfad via Puffer holen
	zchar  *GetBase (int iPos,int *pLen=0);							// Einen Dateipfad mit  Basispfad holen
	zchar  *GetBase (int iPos,int &iLen,int &iOff);					// Einen Dateipfad mit  Basispfad holen
	int		GetBase (int iPos,zchar *pBuf          ,int iSize)const;// Einen Dateipfad mit  Basispfad via Puffer holen
	int		GetBase (int iPos,zchar *pBuf,int &iOff,int iSize)const;// Einen Dateipfad mit  Basispfad via Puffer holen
	zchar  *GetPath (int iPos,int &iLen)const;						// Einen Pfad ohne Basispfad holen (ohne Dateinamen)
	zchar  *GetPath (int iPos          )const;						// Einen Pfad ohne Basispfad holen (ohne Dateinamen)
	zchar  *GetName (int iPos          )const;						// Nur den Dateinamen holen
	zchar  *GetName (int iPos,int &iLen)const; 						// Nur den Dateinamen holen
	zchar  *GetName (int iPos,int &iLen,int   &iData)const; 		// Nur den Dateinamen holen
	zchar  *GetName (int iPos,int &iLen,void **pData)const; 		// Nur den Dateinamen holen

	int		FindPath    (int iSize,const zchar *pPath);				// Einen Pfad ohne Datei suchen (mit Lnge)
	int		FindPath    (          const zchar *pPath);				// Einen Pfad ohne Datei suchen
	int		FindPath	(int iSize,const zchar *pPath,int &iFrom,int &iLast); // Einen Pfad ohne Datei suchen (mit Lnge und Bereich)
	int		FindPath	(          const zchar *pPath,int &iFrom,int &iLast); // Einen Pfad ohne Datei suchen (mit Bereich)

	int		SetBasePath	(          const zchar *pPath);				// Den Basispfad einstellen
	int		SetBasePath	(int iSize,const zchar *pPath);				// Den Basispfad einstellen (mit Lnge)

	int	  	DataLength	(int iLen=-1);								// Verndert Datenlnge
	void   *Data	  	(int iPos)const;							// Datenzeiger holen
	void   *DataEx	  	(int iPos)const;							// Datenzeiger holen mit Lnge
	void   *DataNone    (int iPos)const;							// Datenzeiger holen ohne Indexprfung
	void   *DataSet 	(int iPos,const void *pData );				// Daten  setzen
	void   *DataGet 	(int iPos,      void *pData )const;			// Daten  holen

	int		DataGet		(int iPos)const;							// Daten  holen
	i64 	DataGetQ	(int iPos)const;							// Daten  holen
	u16 	DataGetW	(int iPos)const;							// Daten  holen
	u8   	DataGetB	(int iPos)const;							// Daten  holen
	int   	DataGet 	(int iPos,unsigned uIndex)const;			// Daten  holen
	i64  	DataGetQ	(int iPos,unsigned uIndex)const;			// Daten  holen
	u16 	DataGetW	(int iPos,unsigned uIndex)const;			// Daten  holen
	u8   	DataGetB	(int iPos,unsigned uIndex)const;			// Daten  holen

	int		DataSet		(int iPos,int iValue);						// Daten  setzen
	i64 	DataSetQ	(int iPos,i64 iValue);						// Daten  setzen
	u16 	DataSetW	(int iPos,u16 iValue);						// Daten  setzen
	u8   	DataSetB	(int iPos,u8  iValue);						// Daten  setzen
	int   	DataSet 	(int iPos,int iValue,unsigned uIndex);		// Daten  setzen
	i64  	DataSetQ	(int iPos,i64 iValue,unsigned uIndex);		// Daten  setzen
	u16 	DataSetW	(int iPos,u16 iValue,unsigned uIndex);		// Daten  setzen
	u8   	DataSetB	(int iPos,u8  iValue,unsigned uIndex);		// Daten  setzen

	void   *DataGetPtr 	(int iPos                   )const;			// Daten  holen
	void   *DataGetPtr 	(int iPos,unsigned    uIndex)const;			// Daten  holen

	bool	Check      (int   iPos )const;							// Einen Index prfen
	void	SetUserData(int   iData);								// Einstellen der Userdaten
	void	SetUserPtr (void *pPtr );								// Einstellen der Userdaten
	void	GetUserPtr (void *pPtr )const;							// Abfragen der Userdaten
	int		GetUserData()const;										// Abfragen der Userdaten
	void   *GetUserPtr ()const;										// Abfragen der Userdaten
	int	  	PathCount  ()const;										// Ergibt Anzahl der Pfade
	int	  	Count 	   ()const;										// Ergibt Anzahl

protected:

	zchar			m_cBuffer[CFL_MAX_PATH  ];						// Speicher fr Pfad
	zchar			m_cTarget[CFL_MAX_PATH+8];						// Speicher fr's Einfgen
	int				m_iTargetLen;									// Lnge fr's Einfgen
	int				m_iTargetOff;									// Offset im Pfad
	int				m_iPathPos;										// Pfadposition fr's Einfgen
	int				m_iFilePos;										// Dateiposition fr's Einfgen
	int				m_iFileOff;										// Lnge des Dateinamens fr's Einfgen
	int				m_iFileLen;										// Lnge des Dateinamens fr's Einfgen
	int				m_iBaseSize;									// Lnge des Basis Pfades
	unsigned		m_uPathCrc;										// CRC des letzten Pfades
	CStringChain	m_sFiles;										// Liste mit Dateien und Daten
	CStringChain	m_sPaths;										// Liste mit Pfaden

	#if				CFL_HEAP_CHECK
	void		   *m_pHeapHandle;									// Ist das Handle des benutzen Heaps
	#endif


	static int		CmpName(const zchar *pFile1,const zchar *pFile2,int iSize);
	static int		CmpFile(const zchar *pFile1,const zchar *pFile2,int iSize);
	static int		CmpPath(const zchar *pPath1,const zchar *pPath2,int iSize);
		   int		IsFullPath(const zchar *pPath);

};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	int	  	CFileList::FindEx     (int iSize,const zchar *pPath,         const zchar *pFile                 ){return FindEx(iSize         ,pPath,strzlen(pFile),pFile);}
inline	int	  	CFileList::FindEx     (          const zchar *pPath,int iLen,const zchar *pFile                 ){return FindEx(strzlen(pPath),pPath,iLen          ,pFile);}
inline	int	  	CFileList::FindEx     (          const zchar *pPath,         const zchar *pFile                 ){return FindEx(strzlen(pPath),pPath,strzlen(pFile),pFile);}
inline	int	  	CFileList::Find       (          const zchar *pPath                                             ){return Find  (strzlen(pPath),pPath                     );}
inline	int	  	CFileList::AddEx      (int iSize,const zchar *pPath,         const zchar *pFile,const void *pPtr){return AddEx (iSize         ,pPath,strzlen(pFile),pFile,pPtr);}
inline	int	  	CFileList::AddEx      (          const zchar *pPath,int iLen,const zchar *pFile,const void *pPtr){return AddEx (               pPath,iLen          ,pFile,pPtr);}
inline	int	  	CFileList::AddEx      (          const zchar *pPath,         const zchar *pFile,const void *pPtr){return AddEx (               pPath,strzlen(pFile),pFile,pPtr);}
inline	int	  	CFileList::Add        (          const zchar *pPath                            ,const void *pPtr){return Add   (strzlen(pPath),pPath                     ,pPtr);}
inline	zchar  *CFileList::GetName    (int iPos                       )const{zchar *pName=m_sFiles.Get(iPos           );if(!pName)return 0;                  return pName+CFL_NUMDATA;}
inline	zchar  *CFileList::GetName    (int iPos,int &iLen             )const{zchar *pName=m_sFiles.Get(iPos,iLen      );if(!pName)return 0;iLen-=CFL_NUMDATA;return pName+CFL_NUMDATA;}
inline	zchar  *CFileList::GetName    (int iPos,int &iLen,int   &iData)const{zchar *pName=m_sFiles.Get(iPos,iLen,iData);if(!pName)return 0;iLen-=CFL_NUMDATA;return pName+CFL_NUMDATA;}
inline	zchar  *CFileList::GetName    (int iPos,int &iLen,void **pData)const{zchar *pName=m_sFiles.Get(iPos,iLen,pData);if(!pName)return 0;iLen-=CFL_NUMDATA;return pName+CFL_NUMDATA;}
inline	int		CFileList::FindName   (int iLen,const zchar *pFile,int iFrom,int iLast){if(iFrom<0)return -1;return m_sFiles.FindFast(iLen+CFL_NUMDATA,pFile,iFrom,iLast,0x01,CmpName);}
inline	int		CFileList::FindName	  (const zchar *pFile,int  iFrom,int  iLast){return FindName(strzlen(pFile),pFile,iFrom,iLast);}
inline	int		CFileList::FindPath	  (const zchar *pPath,int &iFrom,int &iLast){return FindPath(strzlen(pPath),pPath,iFrom,iLast);}
inline	int		CFileList::FindPath   (const zchar *pPath){return FindPath   (strzlen(pPath),pPath);}
inline	int	  	CFileList::SetBasePath(const zchar *pPath){return SetBasePath(strzlen(pPath),pPath);}
inline	int	  	CFileList::Del        (const zchar *pPath){return Del        (strzlen(pPath),pPath);}
inline	int	  	CFileList::DataLength (int    iLen){return m_sFiles.DataLength(iLen);}
inline	void	CFileList::SetUserData(int   iData)		{m_sFiles.SetUserData(iData);}
inline	void	CFileList::SetUserPtr (void *pPtr )		{m_sFiles.SetUserPtr (pPtr );}
inline	void	CFileList::GetUserPtr (void *pPtr )const{m_sFiles.GetUserPtr (pPtr );}
inline	int		CFileList::GetUserData()const{return m_sFiles.GetUserData();}
inline	void   *CFileList::GetUserPtr ()const{return m_sFiles.GetUserPtr ();}
inline	int	  	CFileList::Count 	  ()const{return m_sFiles.Count();}
inline	int	  	CFileList::PathCount  ()const{return m_sPaths.Count();}
inline	bool	CFileList::Check      (int iPos)const{return m_sFiles.Check   (iPos);}
inline	void   *CFileList::Data	  	  (int iPos)const{return m_sFiles.Data    (iPos);}
inline	void   *CFileList::DataEx	  (int iPos)const{return m_sFiles.DataEx  (iPos);}
inline	void   *CFileList::DataNone   (int iPos)const{return m_sFiles.DataNone(iPos);}
inline	void   *CFileList::DataSet 	  (int iPos,const void *pData )     {return m_sFiles.DataSet   (iPos,pData );}
inline	void   *CFileList::DataGet 	  (int iPos,      void *pData )const{return m_sFiles.DataGet   (iPos,pData );}
inline	void   *CFileList::DataGetPtr (int iPos                   )const{return m_sFiles.DataGetPtr(iPos       );}
inline	void   *CFileList::DataGetPtr (int iPos,unsigned    uIndex)const{return m_sFiles.DataGetPtr(iPos,uIndex);}
inline	int		CFileList::DataGet    (int iPos                   )const{return m_sFiles.DataGet   (iPos       );}
inline	int		CFileList::DataGet    (int iPos,unsigned    uIndex)const{return m_sFiles.DataGet   (iPos,uIndex);}
inline	u8		CFileList::DataGetB   (int iPos                   )const{return m_sFiles.DataGetB  (iPos       );}
inline	u8		CFileList::DataGetB   (int iPos,unsigned    uIndex)const{return m_sFiles.DataGetB  (iPos,uIndex);}
inline	u16		CFileList::DataGetW   (int iPos                   )const{return m_sFiles.DataGetW  (iPos       );}
inline	u16		CFileList::DataGetW   (int iPos,unsigned    uIndex)const{return m_sFiles.DataGetW  (iPos,uIndex);}
inline	i64		CFileList::DataGetQ   (int iPos                   )const{return m_sFiles.DataGetQ  (iPos       );}
inline	i64		CFileList::DataGetQ   (int iPos,unsigned    uIndex)const{return m_sFiles.DataGetQ  (iPos,uIndex);}
inline	int		CFileList::DataSet    (int iPos,int iValue                ){return m_sFiles.DataSet   (iPos,iValue       );}
inline	int		CFileList::DataSet    (int iPos,int iValue,unsigned uIndex){return m_sFiles.DataSet   (iPos,iValue,uIndex);}
inline	u8		CFileList::DataSetB   (int iPos,u8  iValue                ){return m_sFiles.DataSetB  (iPos,iValue       );}
inline	u8		CFileList::DataSetB   (int iPos,u8  iValue,unsigned uIndex){return m_sFiles.DataSetB  (iPos,iValue,uIndex);}
inline	u16		CFileList::DataSetW   (int iPos,u16 iValue                ){return m_sFiles.DataSetW  (iPos,iValue       );}
inline	u16		CFileList::DataSetW   (int iPos,u16 iValue,unsigned uIndex){return m_sFiles.DataSetW  (iPos,iValue,uIndex);}
inline	i64		CFileList::DataSetQ   (int iPos,i64 iValue                ){return m_sFiles.DataSetQ  (iPos,iValue       );}
inline	i64		CFileList::DataSetQ   (int iPos,i64 iValue,unsigned uIndex){return m_sFiles.DataSetQ  (iPos,iValue,uIndex);}
inline	int		CFileList::Del		  (int iSize,const zchar*pPath){int iPos;iPos = Find(iSize,pPath);if(iPos<0)return 0;return Del(iPos);}
inline	int		CFileList::Del        (){m_sPaths.Del();return m_sFiles.Del();}


#ifdef		strzlen
#undef 		strzlen
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		i64
#undef 		i64
#endif

#ifdef		u16
#undef 		u16
#endif

#ifdef		u8
#undef 		u8
#endif

#ifdef		Z
#undef 		Z
#endif


#endif
