//*****************************************************************************
//*
//*
//*		exfat_io.c
//*
//*
//*****************************************************************************
#include	<windows.h>
#include	<string.h>
#include	<errno.h>

#include	"exfat.h"


//*****************************************************************************
//*
//*		exfat_generic_pread
//*
//*****************************************************************************
EXFAT_STATIC ssize_t exfat_generic_pread(const struct exfat *ef,struct exfat_node *node,void *buffer,size_t size,exfat_off_t offset)
{
char	   *bufp=(char*)buffer;
cluster_t	cluster;
exfat_off_t	remainder;
off_t		loffset;
off_t		lsize;
off_t		csize;
int			num;


	if(offset>=node->size)
		{
		return 0;
		}

	if(size==0)
		{
		return 0;
		}

	csize	= CLUSTER_SIZE(*ef->sb);
	cluster = exfat_advance_cluster(ef, node, (cluster_t)(offset/csize));

	if(CLUSTER_INVALID(cluster))
		{
		exfat_error1("invalid cluster 0x%x while reading", cluster);
		return -1;
		}

	loffset 	= (off_t)offset%csize;
	remainder	= MIN(size, node->size - offset);
	num			= 1;
	
	while(remainder>0)
		{
		if(CLUSTER_INVALID(cluster))
			{
			exfat_error1("invalid cluster 0x%x while reading", cluster);
			return -1;
			}

		if(loffset || remainder<=csize)
			{
			lsize = (off_t)MIN(csize-loffset, remainder);
			num   = 1;
			}
		else if(remainder<0x10000000)
			{
			lsize = (off_t)((remainder+csize-1)/csize);
			if(num>lsize)num=lsize;
			lsize = csize*num;
			if(lsize>(off_t)remainder)lsize=(off_t)remainder;
			}  
		else{
			lsize = csize*num;
			}

		if(exfat_pread(ef->dev, bufp, lsize,exfat_c2o(ef, cluster) + loffset) < 0)
			{
			exfat_error1("failed to read cluster %#x", cluster);
			return -1;
			}

		loffset 	= 0;
		cluster	   += num-1;	
		bufp	   += lsize;
		remainder  -= lsize;
		num			= exfat_next_clusters(ef,node,cluster,&cluster);
		}

	if(!ef->ro && !ef->noatime)
		{
		exfat_update_atime(node);
		}


return (off_t)(MIN(size,node->size-offset) - remainder);
}


//*****************************************************************************
//*
//*		exfat_generic_pwrite
//*
//*****************************************************************************
EXFAT_STATIC ssize_t exfat_generic_pwrite(struct exfat *ef,struct exfat_node *node,const void *buffer,size_t size,exfat_off_t offset)
{
const char	   *bufp=(char*)buffer;
cluster_t		cluster;
exfat_off_t		remainder;
off_t			loffset;
off_t			lsize;
off_t			csize;
int				num;


	if(offset>node->size)
	if(exfat_truncate(ef, node, offset, true) != 0)
		{
		return -1;
		}

	if(offset+size>node->size)
	if(exfat_truncate(ef, node, offset + size, false) != 0)
		{
		return -1;
		}

	if(size==0)
		{
		return 0;
		}

	csize	= CLUSTER_SIZE(*ef->sb);
	cluster = exfat_advance_cluster(ef,node,(uint32_t)(offset/csize));

	if(CLUSTER_INVALID(cluster))
		{
		exfat_error1("invalid cluster 0x%x while writing", cluster);
		return -1;
		}

	loffset 	= (off_t)offset%csize;
	remainder	= size;
	num			= 1;

	while(remainder>0)
		{
		if(CLUSTER_INVALID(cluster))
			{
			exfat_error1("invalid cluster 0x%x while writing", cluster);
			return -1;
			}

		if(loffset || remainder<=csize)
			{
			lsize = (off_t)MIN(csize-loffset, remainder);
			num   = 1;
			}
		else if(remainder<0x10000000)
			{
			lsize = (off_t)((remainder+csize-1)/csize);
			if(num>lsize)num=lsize;
			lsize = csize*num;
			if(lsize>(off_t)remainder)lsize=(off_t)remainder;
			}  
		else{
			lsize = csize*num;
			}

		if(exfat_pwrite(ef->dev, bufp, lsize,exfat_c2o(ef, cluster) + loffset) < 0)
			{
			exfat_error1("failed to write cluster %#x", cluster);
			return -1;
			}

		loffset 	= 0;
		cluster	   += num-1;	
		bufp	   += lsize;
		remainder  -= lsize;
		num			= exfat_next_clusters(ef,node,cluster,&cluster);
		}

	exfat_update_mtime(node);


return (off_t)(size - remainder);
}
