//*****************************************************************************
//*
//*
//*		exfat_platform.h
//*
//*
//*****************************************************************************
/*
	platform.h (14.05.13)
	OS-specific code (libc-specific in fact). Note that systems with the
	same kernel can use different libc implementations.

	Free exFAT implementation.
	Copyright (C) 2010-2016  Andrew Nayenko

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef		PLATFORM_H_INCLUDED 		
#define		PLATFORM_H_INCLUDED 		

#if	defined(_MSC_VER) && (defined(_M_IX86) || defined(_M_X64) || defined(_M_AMD64))

	#include	<time.h>
	
	#define 	NO_STDINT_H					1

	#ifndef		__BYTE_ORDER			
	#define 	__BYTE_ORDER				1
	#endif 	

	#ifndef		__LITTLE_ENDIAN			
	#define 	__LITTLE_ENDIAN				1
	#endif 	

	#ifndef		__BIG_ENDIAN			
	#define 	__BIG_ENDIAN				0
	#endif 	

	#define		exfat_bswap16(x)			(x)
	#define		exfat_bswap32(x)			(x)
	#define		exfat_bswap64(x)			(x)
	#define		EXFAT_BYTE_ORDER			__BYTE_ORDER
	#define		EXFAT_LITTLE_ENDIAN 		__LITTLE_ENDIAN
	#define		EXFAT_BIG_ENDIAN			__BIG_ENDIAN
	#define 	EXFAT_INLINE				__inline

	#define 	PRId64						"I64i"
	#define 	PRIu64						"I64u"

	typedef		char 						int8_t;
	typedef		short						int16_t;
	typedef		int 						int32_t;
	typedef		__int64						int64_t;

	typedef		unsigned char 				uint8_t;
	typedef		unsigned short				uint16_t;
	typedef		unsigned int 				uint32_t;
	typedef		unsigned __int64			uint64_t;

	typedef		unsigned __int64			exfat_off_t;
	typedef		int							ssize_t;
	typedef		int							uid_t;
	typedef		int							gid_t;

	struct timespec
		{
        time_t   tv_sec;					// seconds 
        long     tv_nsec;					// nanoseconds 
		}; 

	#ifndef		__cplusplus
	typedef		char						bool;
	#define 	false						0
	#define 	true						1
	#endif 	

#elif defined(__linux__) || defined(__GLIBC__) || defined(__GNU__)

	#include	<stdint.h>
	#include	<stdbool.h>
	#include	<endian.h>
	#include	<byteswap.h>
	#include	<inttypes.h>

	#define		exfat_bswap16(x)			bswap_16(x)
	#define		exfat_bswap32(x)			bswap_32(x)
	#define		exfat_bswap64(x)			bswap_64(x)
	#define		EXFAT_BYTE_ORDER			__BYTE_ORDER
	#define		EXFAT_LITTLE_ENDIAN 		__LITTLE_ENDIAN
	#define		EXFAT_BIG_ENDIAN			__BIG_ENDIAN
	#define 	EXFAT_INLINE				inline

	typedef		off_t						exfat_off_t;

#elif defined(__APPLE__)

	#include	<stdint.h>
	#include	<stdbool.h>
	#include	<inttypes.h>
	#include	<machine/endian.h>
	#include	<libkern/OSByteOrder.h>

	#define		exfat_bswap16(x)			OSSwapInt16(x)
	#define		exfat_bswap32(x)			OSSwapInt32(x)
	#define		exfat_bswap64(x)			OSSwapInt64(x)
	#define		EXFAT_BYTE_ORDER			BYTE_ORDER
	#define		EXFAT_LITTLE_ENDIAN 		LITTLE_ENDIAN
	#define		EXFAT_BIG_ENDIAN			BIG_ENDIAN
	#define 	EXFAT_INLINE				inline

	typedef		off_t						exfat_off_t;

#elif defined(__FreeBSD__) || defined(__DragonFlyBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)

	#include	<stdint.h>
	#include	<stdbool.h>
	#include	<inttypes.h>
	#include	<sys/endian.h>

	#define		exfat_bswap16(x)			bswap16(x)
	#define		exfat_bswap32(x)			bswap32(x)
	#define		exfat_bswap64(x)			bswap64(x)
	#define		EXFAT_BYTE_ORDER			_BYTE_ORDER
	#define		EXFAT_LITTLE_ENDIAN 		_LITTLE_ENDIAN
	#define		EXFAT_BIG_ENDIAN			_BIG_ENDIAN
	#define 	EXFAT_INLINE				inline
													
	typedef		off_t						exfat_off_t;

#else

	#error		"Unknown platform"

#endif

#endif // ifndef PLATFORM_H_INCLUDED 
