/*---------------------------------------------------------------------------/
/  FatFs - FAT file system module include file  R0.08b    (C)ChaN, 2011
/----------------------------------------------------------------------------/
/ FatFs module is a generic FAT file system module for small embedded systems.
/ This is a free software that opened for education, research and commercial
/ developments under license policy of following trems.
/
/  Copyright (C) 2011, ChaN, all right reserved.
/
/ * The FatFs module is a free software and there is NO WARRANTY.
/ * No restriction on use. You can use, modify and redistribute it for
/   personal, non-profit or commercial product UNDER YOUR RESPONSIBILITY.
/ * Redistributions of source code must retain the above copyright notice.
/
/----------------------------------------------------------------------------*/

#ifndef _FATFS
#define _FATFS	8237	/* Revision ID */

#ifdef __cplusplus
extern "C" {
#endif

#include "ff_integer.h"	/* Basic integer types */
#include "ff_conf.h"	/* FatFs configuration options */


#if _FATFS != _FFCONF
#error Wrong configuration file (ffconf.h).
#endif

#ifndef	FS_API 
#define FS_API 		
#endif

#ifndef	FS_STATIC
#define FS_STATIC		
#endif

#ifdef	FS_SEC32
typedef	DWORD		SECOFF;
#else 	
typedef	QWORD		SECOFF;
#endif

#ifdef	FS_CNT8
typedef	BYTE		SECCNT;
#else 	
typedef	DWORD		SECCNT;
#endif

typedef	unsigned char (_cdecl *disk_init_cb	 )(void*,DWORD);
typedef	unsigned char (_cdecl *disk_status_cb)(void*,DWORD);
typedef	unsigned char (_cdecl *disk_ioctl_cb )(void*,DWORD,BYTE, void*);
typedef	unsigned char (_cdecl *disk_write_cb )(void*,DWORD,const BYTE*,SECOFF,SECCNT);
typedef	unsigned char (_cdecl *disk_read_cb	 )(void*,DWORD,      BYTE*,SECOFF,SECCNT);

		   
				   
/* File system object structure (FATFS) */

typedef struct 
	{
	BYTE	fs_type;		/* FAT sub-type (0:Not mounted) */
	BYTE	drv;			/* Physical drive number */
	BYTE	csize;			/* Sectors per cluster (1,2,4...128) */
	BYTE	n_fats;			/* Number of FAT copies (1,2) */
	BYTE	wflag;			/* win[] dirty flag (1:must be written back) */
	BYTE	fsi_flag;		/* fsinfo dirty flag (1:must be written back) */
	WORD	id;				/* File system mount ID */
	WORD	n_rootdir;		/* Number of root directory entries (FAT12/16) */
	#if _MAX_SS != 512
	WORD	ssize;			/* Bytes per sector (512,1024,2048,4096) */
	#endif
	#if _FS_REENTRANT
	_SYNC_t	sobj;			/* Identifier of sync object */
	#endif
	#if !_FS_READONLY
	DWORD	last_clust;		/* Last allocated cluster */
	DWORD	free_clust;		/* Number of free clusters */
	DWORD	fsi_sector;		/* fsinfo sector (FAT32) */
	#endif
	#if _FS_RPATH
	DWORD	cdir;			/* Current directory start cluster (0:root) */
	#endif
	DWORD	n_fatent;		/* Number of FAT entries (= number of clusters + 2) */
	DWORD	fsize;			/* Sectors per FAT */
	DWORD	fatbase;		/* FAT start sector */
	DWORD	dirbase;		/* Root directory start sector (FAT32:Cluster#) */
	DWORD	database;		/* Data start sector */
	DWORD	winsect;		/* Current sector appearing in the win[] */
	BYTE	win[_MAX_SS];	/* Disk access window for Directory, FAT (and Data on tiny cfg) */
	} FATFS;



/* File object structure (FIL) */

typedef struct 
	{
	FATFS*	fs;				/* Pointer to the owner file system object */
	WORD	id;				/* Owner file system mount ID */
	BYTE	flag;			/* File status flags */
	BYTE	pad1;
	DWORD	fptr;			/* File read/write pointer (0 on file open) */
	DWORD	fsize;			/* File size */
	DWORD	sclust;			/* File start cluster (0 when fsize==0) */
	DWORD	clust;			/* Current cluster */
	DWORD	dsect;			/* Current data sector */
	#if _FS_USESETTIME && !_FS_READONLY
	DWORD	newtime;		/* New time change to set */
	#if _FS_USECRTTIME
	DWORD	newcrea;		/* New time create to set */
	#endif 	
	#endif
	#if _FS_GETFNAME && _USE_LFN
	DWORD	lfn_sect;		/* Sector containing the long filename */
	WORD	lfn_idx;		/* Index containing the long filename */
	WORD	dir_idx;		/* Index of the name*/
	#endif 	
	#if !_FS_READONLY
	DWORD	dir_sect;		/* Sector containing the directory entry */
	BYTE*	dir_ptr;		/* Pointer to the directory entry in the window */
	#endif
	#if _USE_FASTSEEK
	DWORD*	cltbl;			/* Pointer to the cluster link map table (null on file open) */
	#endif
	#if _FS_SHARE
	UINT	lockid;			/* File lock ID (index of file semaphore table) */
	#endif
	#if !_FS_TINY
	BYTE	buf[_MAX_SS];	/* File data read/write buffer */
	#endif
	} FIL;



/* Directory object structure (DIR) */

typedef struct 
	{
	FATFS*	fs;				/* Pointer to the owner file system object */
	WORD	id;				/* Owner file system mount ID */
	WORD	index;			/* Current read/write index number */
	DWORD	sclust;			/* Table start cluster (0:Root dir) */
	DWORD	clust;			/* Current cluster */
	DWORD	sect;			/* Current sector */
	BYTE*	dir;			/* Pointer to the current SFN entry in the win[] */
	BYTE*	fn;				/* Pointer to the SFN (in/out) {file[8],ext[3],status[1]} */
	#if    _USE_LFN
	WCHAR*	lfn;			/* Pointer to the LFN working buffer */
	WORD	lfn_idx;		/* Last matched LFN index number (0xFFFF:No LFN) */
	#endif
	#if _FS_GETFNAME && _USE_LFN
	DWORD	lfn_sect;		/* Sector containing the long filename */
	#endif 	
	} DIR;



/* File status structure (FILINFO) */

typedef struct 
	{
	DWORD	fsize;			/* File size */
	WORD	fdate;			/* Last modified date */
	WORD	ftime;			/* Last modified time */
	#if _FS_USECRTTIME
	WORD	cdate;			/* Creation date */
	WORD	ctime;			/* Creation time */
	#endif 	
	BYTE	fattrib;		/* Attribute */
	XCHAR	fname[13];		/* Short file name (8.3 format) */
	#if _USE_LFN
	union
		{
	XCHAR*	lfname;			/* Pointer to the LFN buffer */
	ZCHAR*	lname;			/* Pointer to the LFN buffer */
		};
	UINT 	lfsize;			/* Size of LFN buffer in XCHAR */
	#endif
	} FILINFO;



/* Drive data structure (FATDATA) */

typedef struct 
	{
	#if _VOLUMES
	FATFS		   *FatFs[_VOLUMES];	/* Pointer to the file system objects (logical drives) */
	#else
	#error Number of drives must not be 0.
	#endif

	WORD			Fsid;				/* File system mount ID */

	#if _FS_RPATH
	BYTE			CurrVol;			/* Current drive */
	#endif

	#if _FS_SHARE
	FILESEM			Files[_FS_SHARE];	/* File lock semaphores */
	#endif

	#if _FS_USERPTR
	void		   *user_ptr;
	#endif 	
		
	#if _FS_USERDATA
	int 			user_data;
	#endif 	

	disk_init_cb	disk_init_proc;
	disk_status_cb	disk_status_proc;
	disk_ioctl_cb	disk_ioctl_proc;
	disk_write_cb	disk_write_proc;
	disk_read_cb	disk_read_proc;
	}FATDATA;


/* File function return code (FRESULT) */

typedef enum 
	{
	FR_OK = 0,				/* (0) Succeeded */
	FR_DISK_ERR,			/* (1) A hard error occured in the low level disk I/O layer */
	FR_INT_ERR,				/* (2) Assertion failed */
	FR_NOT_READY,			/* (3) The physical drive cannot work */
	FR_NO_FILE,				/* (4) Could not find the file */
	FR_NO_PATH,				/* (5) Could not find the path */
	FR_INVALID_NAME,		/* (6) The path name format is invalid */
	FR_DENIED,				/* (7) Acces denied due to prohibited access or directory full */
	FR_EXIST,				/* (8) Acces denied due to prohibited access */
	FR_INVALID_OBJECT,		/* (9) The file/directory object is invalid */
	FR_WRITE_PROTECTED,		/* (10) The physical drive is write protected */
	FR_INVALID_DRIVE,		/* (11) The logical drive number is invalid */
	FR_NOT_ENABLED,			/* (12) The volume has no work area */
	FR_NO_FILESYSTEM,		/* (13) There is no valid FAT volume on the physical drive */
	FR_MKFS_ABORTED,		/* (14) The f_mkfs() aborted due to any parameter error */
	FR_TIMEOUT,				/* (15) Could not get a grant to access the volume within defined period */
	FR_LOCKED,				/* (16) The operation is rejected according to the file shareing policy */
	FR_NOT_ENOUGH_CORE,		/* (17) LFN working buffer could not be allocated */
	FR_TOO_MANY_OPEN_FILES	/* (18) Number of open files > _FS_SHARE */
	} FRESULT;



/*--------------------------------------------------------------*/
/* FatFs module application interface                           */

FRESULT f_mount	   (FATDATA*,BYTE, FATFS*);						/* Mount/Unmount a logical drive */
FRESULT f_open     (FATDATA*,FIL*, const ZCHAR*, BYTE);			/* Open or create a file */
FRESULT f_read     (FATDATA*,FIL*, void*, UINT, UINT*);			/* Read data from a file */
FRESULT f_lseek    (FATDATA*,FIL*, DWORD);						/* Move file pointer of a file object */
FRESULT f_close    (FATDATA*,FIL*);								/* Close an open file object */
FRESULT f_opendir  (FATDATA*,DIR*, const ZCHAR*);				/* Open an existing directory */
FRESULT f_readdir  (FATDATA*,DIR*, FILINFO*);					/* Read a directory item */
FRESULT f_stat     (FATDATA*,const ZCHAR*, FILINFO*);			/* Get file status */
FRESULT f_write    (FATDATA*,FIL*, const void*, UINT, UINT*);	/* Write data to a file */
FRESULT f_getfree  (FATDATA*,const ZCHAR*, DWORD*, FATFS**);	/* Get number of free clusters on the drive */
FRESULT f_truncate (FATDATA*,FIL*);								/* Truncate file */
FRESULT f_sync     (FATDATA*,FIL*);								/* Flush cached data of a writing file */
FRESULT f_unlink   (FATDATA*,const ZCHAR*);						/* Delete an existing file or directory */
FRESULT	f_mkdir    (FATDATA*,const ZCHAR*);						/* Create a new directory */
FRESULT f_chmod    (FATDATA*,const ZCHAR*, BYTE, BYTE);			/* Change attriburte of the file/dir */
FRESULT f_utime    (FATDATA*,const ZCHAR*, const FILINFO*);		/* Change timestamp of the file/dir */
FRESULT f_rename   (FATDATA*,const ZCHAR*, const ZCHAR*);		/* Rename/Move a file or directory */
FRESULT f_forward  (FATDATA*,FIL*, UINT(*)(const BYTE*,UINT), UINT, UINT*);	/* Forward data to the stream */
FRESULT f_mkfs     (FATDATA*,BYTE, BYTE, UINT);					/* Create a file system on the drive */
FRESULT f_chdrive  (FATDATA*,BYTE);								/* Change current drive */
FRESULT f_chdir    (FATDATA*,const ZCHAR*);						/* Change current directory */
FRESULT f_getcwd   (FATDATA*,ZCHAR*, UINT);						/* Get current directory */
int		f_putc     (FATDATA*,ZCHAR, FIL*);						/* Put a character to the file */
int 	f_puts     (FATDATA*,const ZCHAR*, FIL*);				/* Put a string to the file */
int 	f_printf   (FATDATA*,FIL*, const ZCHAR*, ...);			/* Put a formatted string to the file */
ZCHAR  *f_gets     (FATDATA*,ZCHAR*, int, FIL*);				/* Get a string from the file */

#ifndef EOF
#define EOF (-1)
#endif

#define f_eof(fp)   (((fp)->fptr == (fp)->fsize) ? 1 : 0)
#define f_error(fp) (((fp)->flag &    FA__ERROR) ? 1 : 0)
#define f_size(fp)  (((fp)->fsize))
#define f_tell(fp)  (((fp)->fptr))




/*--------------------------------------------------------------*/
/* Additional user defined functions                            */

/* RTC function */
#if !_FS_READONLY
DWORD get_fattime(void);
#endif

/* Unicode support functions */
#if _USE_LFN							/* Unicode - OEM code conversion */
WCHAR ff_convert (WCHAR, UINT);			/* OEM-Unicode bidirectional conversion */
WCHAR ff_wtoupper(WCHAR);				/* Unicode upper-case conversion */
#if _USE_LFN == 3						/* Memory functions */
void* ff_memalloc(UINT);				/* Allocate memory block */
void  ff_memfree (void*);				/* Free memory block */
#endif
#endif

/* Sync functions */
#if _FS_REENTRANT
int  ff_cre_syncobj(BYTE, _SYNC_t*);	/* Create a sync object */
int  ff_req_grant  (_SYNC_t);			/* Lock sync object */
void ff_rel_grant  (_SYNC_t);			/* Unlock sync object */
int  ff_del_syncobj(_SYNC_t);			/* Delete a sync object */
#endif




/*--------------------------------------------------------------*/
/* Flags and offset address                                     */


/* File access control and file status flags (FIL.flag) */

#define	FA_READ				0x01
#define	FA_OPEN_EXISTING	0x00
#define FA__ERROR			0x80

#if   !_FS_READONLY
#define	FA_WRITE			0x02
#define	FA_CREATE_NEW		0x04
#define	FA_CREATE_ALWAYS	0x08
#define	FA_OPEN_ALWAYS		0x10
#define FA__WRITTEN			0x20
#define FA__DIRTY			0x40
#endif


/* FAT sub type (FATFS.fs_type) */

#define FS_FAT12			0x01
#define FS_FAT16			0x02
#define FS_FAT32			0x03


/* File attribute bits for directory entry */

#define	AM_RDO				0x01	/* Read only */
#define	AM_HID				0x02	/* Hidden */
#define	AM_SYS				0x04	/* System */
#define	AM_VOL				0x08	/* Volume label */
#define AM_LFN				0x0F	/* LFN entry */
#define AM_DIR				0x10	/* Directory */
#define AM_ARC				0x20	/* Archive */
#define AM_MASK				0x3F	/* Mask of defined bits */


/* Fast seek function */
#define CREATE_LINKMAP	0xFFFFFFFF



/*--------------------------------*/
/* Multi-byte word access macros  */

#if    _WORD_ACCESS == 1	/* Enable word access to the FAT structure */
#define	LD_WORD(ptr)		(WORD)(*(WORD*)(BYTE*)(ptr))
#define	LD_DWORD(ptr)		(DWORD)(*(DWORD*)(BYTE*)(ptr))
#define	ST_WORD(ptr,val)	*(WORD*)(BYTE*)(ptr)=(WORD)(val)
#define	ST_DWORD(ptr,val)	*(DWORD*)(BYTE*)(ptr)=(DWORD)(val)
#else						/* Use byte-by-byte access to the FAT structure */
#define	LD_WORD(ptr)		(WORD)(((WORD)*((BYTE*)(ptr)+1)<<8)|(WORD)*(BYTE*)(ptr))
#define	LD_DWORD(ptr)		(DWORD)(((DWORD)*((BYTE*)(ptr)+3)<<24)|((DWORD)*((BYTE*)(ptr)+2)<<16)|((WORD)*((BYTE*)(ptr)+1)<<8)|*(BYTE*)(ptr))
#define	ST_WORD(ptr,val)	*(BYTE*)(ptr)=(BYTE)(val); *((BYTE*)(ptr)+1)=(BYTE)((WORD)(val)>>8)
#define	ST_DWORD(ptr,val)	*(BYTE*)(ptr)=(BYTE)(val); *((BYTE*)(ptr)+1)=(BYTE)((WORD)(val)>>8); *((BYTE*)(ptr)+2)=(BYTE)((DWORD)(val)>>16); *((BYTE*)(ptr)+3)=(BYTE)((DWORD)(val)>>24)
#endif

#ifdef __cplusplus
}
#endif

#endif /* _FATFS */
