//*****************************************************************************
//*
//*
//*		ff_misc.cpp
//*
//*
//*****************************************************************************
#include	<string.h>
#include	"ff.h"

//*****************************************************************************
//*
//*		f_chain
//*
//*****************************************************************************
//	Liest eine Sektorkette von einem Verzeichniss ein
//	pFat		: Zeiger auf die FAT-Daten
//	pPath		: Ist der Pfad des Verzeichnisses (z.B.: "0:Test")
//	pList		: Hier wird die Sektorkette gespeichert
//					[0] = Laufwerk
//					[1] = Sektoren je Cluster
//					[2] = Anzahl der Cluster
//					[3] = Gre in Byte
//					[X] = Liste mit Sektoren relativ zu Disk-Anfang
//	uMode		: Was soll analysiert werden
//					0 = Verzeichnisse
//					1 = Dateien
//	Ergibt FR_OK wenn ok, oder eine Fehlernummer. 
FRESULT f_chain(FATDATA *pFat,const ZCHAR *pPath,DWORD *pList,unsigned uSize,unsigned uMode)
{
BYTE		aSecData[512*3];
unsigned	uCluster;
unsigned	uDrive;
unsigned	uBase;
unsigned	uFat;
unsigned	uNow;
unsigned	uMax;
unsigned	uPos;
FRESULT		iRet;
DIR			sDir;
FIL			sDat;



	if(uSize<5)return FR_INT_ERR;

	if(uMode)												// Dateien
		{
		   iRet = f_open(pFat,&sDat,pPath,FA_OPEN_EXISTING);
		if(iRet!=FR_OK)return iRet;

		sDir.fs		= sDat.fs;
		sDir.clust	= sDat.sclust;
		sDir.sect	= sDat.fs->database;

		f_close(pFat,&sDat);
		}
	else{
		   iRet = f_opendir(pFat,&sDir,pPath);
		if(iRet!=FR_OK)return iRet;
		}


	uDrive		= sDir.fs->drv;								// Laufwerk		
	pList[0]	= uDrive;									
	
	if(sDir.fs->fs_type!=FS_FAT32 && sDir.sect<sDir.fs->database) 
		{
		uMax		= sDir.fs->database-sDir.sect;			// Anzahl der Sektoren vom Verzeichniss	
		pList[1]	= 1;									// Sektoren je Cluster
		pList[2]	= uMax;									// Gre des Verzeichnisses in Cluster

		if(uMax+4>uSize)uMax=uSize-4;

		for(uPos=0;uPos<uMax;uPos++)
			{
			pList[4+uPos] = sDir.sect+uPos;
			}
		}
	else{
		pList[1]	= sDir.fs->csize;						// Sektoren je Cluster
		pList[2]	= 0;
		uCluster	= sDir.clust;
		uBase		= sDir.fs->database-pList[1]*2;
		uNow		= 0xFFFFFFFF;

		switch(sDir.fs->fs_type)							// Maximale Cluster-Nummer
			{
		case FS_FAT12:	uMax = 0x00000FF6;	break;		
		case FS_FAT16:	uMax = 0x0000FFF6;	break;		
		case FS_FAT32:	uMax = 0x0FFFFFF7;	break;		
		default:		return FR_NO_FILESYSTEM;
			}

		if(uMax>sDir.fs->n_fatent)
			{
			uMax = sDir.fs->n_fatent;
			}

		for(uPos=4;uPos<uSize;uPos++)						// Cluster-Kette abschreiten
			{
			if(uCluster==0 || uCluster>=uMax)
				{
				break;
				}

			switch(sDir.fs->fs_type)						// Sektor in FAT-Tabelle
				{
			case FS_FAT12:	uFat = sDir.fs->fatbase+(uCluster/1024)*3;	break;		
			case FS_FAT16:	uFat = sDir.fs->fatbase+(uCluster/ 768)*3;	break;
			case FS_FAT32:	uFat = sDir.fs->fatbase+(uCluster/ 384)*3;	break;
				}

			if(uFat!=uNow)
				{
				if(pFat->disk_read_proc(pFat,(BYTE)uDrive,aSecData,uFat,3))
					{
					return FR_DISK_ERR;
					}

				uNow = uFat;
				}

			pList[uPos] = uBase+uCluster*pList[1];
			
			switch(sDir.fs->fs_type)						// Nchster Cluster
				{
			case FS_FAT12:	uCluster = *(DWORD*)(aSecData+(((uCluster/2)%512)*3)); if(uCluster&1)uCluster>>=12; uCluster&=0x0FFF;	break;
			case FS_FAT16:	uCluster = ((WORD *) aSecData)[uCluster%768];							break;
			case FS_FAT32:	uCluster = ((DWORD*) aSecData)[uCluster%384];	uCluster &= 0x0FFFFFFF;	break;
				}
			}	

		pList[2] = uPos-4;
		}		
	

	if(uMode)												// Gre in Byte
		{
		pList[3] = sDat.fsize;
		}
	else{
		pList[3] = pList[2]*pList[1]*512;					
		}


return FR_OK;
}

//*****************************************************************************
//*
//*		f_chain_rd
//*
//*****************************************************************************
//	Liest eine Sektorkette von einem Laufwerk ein
//	pFat		: Zeiger auf die FAT-Daten
//	pList		: Zeiger auf die Sektorkette
//					[0] = Laufwerk
//					[1] = Sektoren je Cluster
//					[2] = Anzahl der Cluster
//					[3] = Gre in Byte
//					[X] = Liste mit Sektoren relativ zu Disk-Anfang
//	pData		: Zeiger auf den Zielpuffer
//	Ergibt FR_OK wenn ok, oder eine Fehlernummer. 
FRESULT f_chain_rd(FATDATA *pFat,DWORD *pList,BYTE *pData)
{
unsigned	uEnd;
unsigned	uPos;


	uEnd = 4+pList[2];
	
	for(uPos=4;uPos<uEnd;uPos++)
		{
		if(pFat->disk_read_proc(pFat,(BYTE)pList[0],pData,pList[uPos],(BYTE)pList[1]))
			{
			return FR_DISK_ERR;
			}

		pData += 512*pList[1];
		}
	
	
return FR_OK;
}

//*****************************************************************************
//*
//*		f_chain_wr
//*
//*****************************************************************************
//	Schreibt eine Sektorkette zu einem Laufwerk.
//	pFat		: Zeiger auf die FAT-Daten
//	pList		: Zeiger auf die Sektorkette
//					[0] = Laufwerk
//					[1] = Sektoren je Cluster
//					[2] = Anzahl der Cluster
//					[3] = Gre in Byte
//					[X] = Liste mit Sektoren relativ zu Disk-Anfang
//	pData		: Zeiger auf den Zielpuffer
//	Ergibt FR_OK wenn ok, oder eine Fehlernummer. 
FRESULT f_chain_wr(FATDATA *pFat,DWORD *pList,const BYTE *pData)
{
unsigned	uEnd;
unsigned	uPos;


	uEnd = 4+pList[2];
	
	for(uPos=4;uPos<uEnd;uPos++)
		{
		if(pFat->disk_write_proc(pFat,(BYTE)pList[0],pData,pList[uPos],(BYTE)pList[1]))
			{
			return FR_DISK_ERR;
			}

		pData += 512*pList[1];
		}
	
	
return FR_OK;
}

//*****************************************************************************
//*
//*		f_lfnentry
//*
//*****************************************************************************
//	Liest eine Sektorkette von einem Verzeichniss ein
//	pData		: Zeiger auf Verzeichniss daten
//	iSize		: Gre der Daten in Byte
//	pName		: Hier wird der Datei name gespeichert
//	pLen		: Hier wird die Lnge des Abschnittes gespeichert
//	Ergibt den Typ des Abschnittes
//					0 = Ende
//					1 = Unbenutzt
//					2 = Voume-Name
//					3 = Langer Voume-Name
//					4 = Steuer-Name
//					5 = Langer Steuer-Name
//					6 = Name
//					7 = Langer Name
int f_lfnentry(const BYTE *pData,int iSize,unsigned short *pName,int *pLen)
{
BYTE	ucCrc;
int		iStep;
int		iSum;
int		iPos;
int		iLen;	
int		iOff;	


	if(iSize<32 || *pData==0)								// Ende erreicht
		{
		pName[0] = 0;
		pLen [0] = 0;

		return 0;
		}

															
	if(*pData==0xE5 || *pData<=0x20)						// Unbenutzter Bereich
		{
		for(iPos=32;iPos<iSize;iPos+=32)
			{
			if( pData[iPos]==0x00)break;
			if( pData[iPos]==0xE5)continue;
			if( pData[iPos]>=0x20)break;
			if( pData[iPos] &0x40)
			if((pData[iPos+11]&AM_MASK)!=AM_LFN)
				{
				break;
				}
			}

		
		pName[0] = 0;
		pLen [0] = iPos;
		
		return 1;		
		}
	

	if(!(*pData&0x40) || (pData[11]&AM_MASK)!=AM_LFN)		// Kurzer Name
		{
		for(iLen=8;iLen>1;iLen--)
			{
			if(pData[iLen]!=' ')break;
			}
		
		for(iOff=0;iOff<=iLen;iOff++)
			{
			pName[iOff] = pData[iOff];
			}
															
		for(iLen=10;iLen>=8;iLen--)
			{
			if(pData[iLen]!=' ')break;
			}
		
		if(iLen>8)											// Suffix vorhanden
			{
			pName[iOff] = '.';		
			iOff++;

			for(iPos=8;iPos<=iLen;iPos++,iOff++)
				{
				pName[iOff] = pData[iPos];
				}
			}

		pName[iOff] = 0;
		pLen[0]		= 32;

		if(pData[11]&AM_VOL)return 2;

		if(pName[0]=='.')
			{
			if(pName[1]==0)return 4;
			if(pName[1]=='.' && pName[2]==0)return 4;
			}
		
		return 6;		
		} 
	
	
	iStep	= pData[ 0]&0x1F;
	iSum	= pData[13];
	iPos	= 0;

	if(iStep==0 || iStep>20)								// Ungltige Lnge
		{
		pName[0] = 0;
		pLen [0] = 32;

		return 0;
		}

	pName[iStep*13] = 0;
	
	
	for(ucCrc=0x40;iStep>0;iStep--)							// Langen Namen aufbauen
		{
		iOff   = (iStep-1)*13;
		ucCrc |=  iStep;

		if(pData[iPos+13]!=iSum || pData[iPos]!=ucCrc || iPos+64>iSize)
			{
			pName[0] = 0;
			pLen [0] = iPos;
			return 1;
			}

		for(iLen=1;iLen<11;iLen+=2,iOff++)
			{
			pName[iOff] = *(WORD*)(pData+iPos+iLen);
			}

		for(iLen=14;iLen<26;iLen+=2,iOff++)
			{
			pName[iOff] = *(WORD*)(pData+iPos+iLen);
			}

		for(iLen=28;iLen<32;iLen+=2,iOff++)
			{
			pName[iOff] = *(WORD*)(pData+iPos+iLen);
			}
		
		iPos += 32;
		ucCrc = 0;
		}

	
    
	for(ucCrc=0,iOff=0;iOff<11;iOff++)						// Checksumme prfen		
		{
		ucCrc  = (ucCrc<<7)+(ucCrc>>1);
		ucCrc += pData[iPos+iOff]; 
		}

	if(ucCrc!=iSum)
		{
		pName[0] = 0;
		pLen [0] = iPos;
		return 1;
		}
	
  	pLen[0] = iPos+32;

	if(pData[iPos+11]&AM_VOL)return 3;

	if(pName[0]=='.')
		{
		if(pName[1]==0)return 5;
		if(pName[1]=='.' && pName[2]==0)return 5;
		}


return 7;
}

//*****************************************************************************
//*
//*		f_lfnentry
//*
//*****************************************************************************
//	Erzeugt einen Link auf eine zweite Datei
//	pFat			: Zeiger auf die FAT-Daten
//	pSource			: Quellee auf die der Link erzeugt wird
//	pTarget			: Ist die Zeildatei, sie muss 0 Byte lang sein
//	iCreateAlways	: Die Zieldatei/Link immer erstellen.
//	Ergibt FR_OK wenn ok, oder eine Fehlernummer. 
#if !_FS_READONLY
FRESULT f_link(FATDATA *pFat,const ZCHAR *pSource,const ZCHAR *pTraget,int iCreateAlways)
{
BYTE		aSector[512];
BYTE		aData[32];
BYTE		ucDrive;
unsigned	uCluster;
unsigned	uSector;
unsigned	uSize;
unsigned	uPos;
FRESULT		iRet;
FIL			sDat;



	   iRet = f_open(pFat,&sDat,pSource,FA_OPEN_EXISTING);
	if(iRet!=FR_OK)return iRet;

	uCluster = sDat.sclust;
	uSize    = sDat.fsize;

	f_close(pFat,&sDat);

	   iRet = f_open(pFat,&sDat,pTraget,(iCreateAlways)? FA_CREATE_ALWAYS|FA_WRITE:FA_OPEN_EXISTING|FA_WRITE);
	if(iRet!=FR_OK)return iRet;


	if(sDat.fsize)											// Ist die Datei nicht leer
		{
		f_close(pFat,&sDat);
		return FR_INT_ERR; 
		}

	ucDrive	= sDat.fs->drv;
	uSector	= sDat.dir_sect;
	memcpy(aData,sDat.dir_ptr,32);
	f_close(pFat,&sDat);
															
	if(pFat->disk_read_proc(pFat,ucDrive,aSector,uSector,1))// Verzeichnisssektor lesen
		{
		return FR_DISK_ERR;		
		}

	for(uPos=0;uPos<512;uPos+=32)
		{
		if(memcmp(aSector+uPos,aData,11))continue;

		aSector[uPos+26] = (BYTE)(uCluster    );
		aSector[uPos+27] = (BYTE)(uCluster>> 8);
		aSector[uPos+20] = (BYTE)(uCluster>>16);
		aSector[uPos+21] = (BYTE)(uCluster>>24);
		aSector[uPos+28] = (BYTE)(uSize       );
		aSector[uPos+29] = (BYTE)(uSize   >> 8);
		aSector[uPos+30] = (BYTE)(uSize   >>16);
		aSector[uPos+31] = (BYTE)(uSize   >>24);
															// Verzeichnisssektor speichern
		if(pFat->disk_write_proc(pFat,ucDrive,aSector,uSector,1))
			{
			return FR_DISK_ERR;		
			}

		return FR_OK;
		}


return FR_INVALID_NAME;
}

#endif 	