//*****************************************************************************
//*
//*
//*		AzFtpClient.cpp
//*
//*
//*****************************************************************************
#include	"AzFtpClient.h"
#include	<StringFunctions.h>
#include	<System/SocketUtility.h>
#include	<tchar.h>

#ifndef		LOG_RECV_ON
#define 	LOG_RECV_ON		0
#endif 	
#ifndef		LOG_SEND_ON
#define 	LOG_SEND_ON		0
#endif 	
#ifndef		LOG_LIST_ON
#define 	LOG_LIST_ON		0
#endif 	
#ifndef		LOG_CONN_ON
#define 	LOG_CONN_ON		0
#endif 	

#if			LOG_RECV_ON
#define 	LOG_RECV(p,l)	printf("RECV:  %.*s\n",l,p);
#else
#define 	LOG_RECV(p,l)
#endif

#if			LOG_SEND_ON
#define 	LOG_SEND(p,l)	printf("SEND:  %.*s\n",(l<2 || (p)[(l)-2]!='\r' || (p)[(l)-1]!='\n')? l:l-2,p);
#else
#define 	LOG_SEND(p,l)
#endif

#if			LOG_LIST_ON
#define 	LOG_LIST(p,l)	printf("LIST:  %.*s\n",(l<2 || (p)[(l)-2]!='\r' || (p)[(l)-1]!='\n')? l:l-2,p);
#else
#define 	LOG_LIST(p,l)
#endif

#if			LOG_CONN_ON
#define 	LOG_CONN(i,p,c)	{char cAddr[64];	IpToString(&i,cAddr); printf("CONN:  %s to %s:%u\n",c,cAddr,p);}
#define 	LOG_CONN_EX(c)	{printf("CONN:  %s\n",c);}
#else
#define 	LOG_CONN(i,p,c)
#define 	LOG_CONN_EX(c)
#endif
							
#define 	UNLOCK(d)		ReleaseSemaphore(d->hLockSem,1,NULL)
#define 	LOCK(d)			WaitForSingleObject(d->hLockSem,INFINITE);
#define 	WAIT(d,t)	  ((WaitForSingleObject(d->hLockSem,t)==WAIT_OBJECT_0)? 1:0)

#define 	SIGNAL(d)		ReleaseSemaphore(d->hWaitSem,1,NULL)
#define 	READY(d)		WaitForSingleObject(d->hWaitSem,INFINITE);

#define 	THREAD(p,d,n)	{HANDLE	hThread;DWORD dwId;hThread=CreateThread(NULL,n,p,d,0,&dwId);CloseHandle(hThread);}
#define 	THREAD_DEF		DWORD WINAPI 
#define 	THREAD_EXIT		return 0


#ifndef		FILE_ATTRIBUTE_LINK
#define 	FILE_ATTRIBUTE_LINK		0x0200
#endif 	

#ifndef		AZFTP_MAX_PATH
#define 	AZFTP_MAX_PATH			512					// Maximale Pfadlnge
#endif 	 	 


typedef struct
	{
	char			cServer  [64];						// Ist der Name des Servers
	char			cUsername[64];						// Ist der Username fr die Anmeldung
	char			cPassword[64];						// Ist das Passwort fr die Anmeldung
	char			cLastErr[128];						// Letzte Fehlermeldung
	char			cCurrDir[AZFTP_MAX_PATH];			// Ist das aktuelle Verzeichnis (z.B. "/root/mydir/")
	char			cBuffer [AZFTP_MAX_PATH+128];		// Empfangspuffer fr eine Zeile
	int volatile 	iBreakDone;							// Wurde ein Abruchkommando gesendet
	int volatile	iConnStop;							// Abbruch vom Conn-Thread 
	int volatile 	iDoBreak;							// Abruchkommando senden
	int				iDirLen;							// Ist die Lnge des aktuellen Verzeichnises
	int				iPosNow;							// Ist die aktuelle Lnge bei Dateioperationen
	int				iBufOff;							// Offset der gelesenen Bytes im Puffer
	int				iBufNum;							// Anzahl der Bytes im Puffer
	int				iBufLen;							// Ist die Lnge des Puffers.
	int				iBufTxt;							// Ist der Begin des Textes im Puffer
	int				iBufEnd;							// Ist das Ende des Textes im Puffer
	int				iError;								// Nummer der letzten Fehlermeldung
	int				iFlags;								// Diverse Flags (siehe AZFTP_???)
	int				iSize;								// Ist die Gre der Datei
	int				iPort;								// Ist die TCP-Portnummer
	int				iDataPort;							// Port fr die passive Verbindung
	int				iListenPort;						// Port fr die aktive Verbindung
	int				iConnect;							// Wird gerade eine Verbindung hergestellt
	IpAddr			iConnAddr;							// IP fr das ffnen einer Verbindung
	TcpHandle		hTcp;								// Ist das Handle der Verbindung
	TcpHandle		hConn;								// Ist der die Verbindungen die geffnet wird
	TcpHandle		hListen;							// Ist der Listen-Port fr aktive Verbindungen
	TcpCancel		sTcpCancel;							// Abbruch beim verbinden
	HANDLE			hLockSem;							// Zugriffs Semaphore
	HANDLE			hWaitSem;							// Warte Semaphore
	IpAddr			iAddrSrv;							// IP-Adresse des Servers
	unsigned		uTimeout;							// Timeout in Millisekunden
	}AzFtpData;


typedef struct
	{
	int				iOff;								// Offset im Speicher
	int				iSize;								// Gre des Speichers
	char		   *pMem;								// Zeiger auf Speicher
	}MemParam;

typedef struct
	{
	WIN32_FIND_DATA	sData;								// Dateiinfos
	char 			cBadTime;							// Ist die Zeit nicht ermittelbar
	char 			cDummy;								// Unbenutzt
	unsigned short	uIsLink;							// Ist die Datei ein Link (Offset in Pfad)
	void		   *pFirst;								// Zeiger auf ersten Speicher
	void		   *pNext;								// Zeiger auf nchsten Speicher
	}AzFtpFind,*AzFtpFindPtr;

static unsigned char 	ucListenIndex = 0;				// Indxe fr TCP-Port vom aktiven Modus

static	const char     *pMonthNames[]=
							{
							"Jan","Feb","Mar","Apr",
							"May","Jun","Jul","Aug",
							"Sep","Oct","Nov","Dec",0,
							};


//*****************************************************************************
//*
//*		FileRead
//*
//*****************************************************************************
static int FileRead(void *pParam,void *pBuffer,int iSize)
{

	if(pParam==0)return -1;

	if(iSize<=0)
		{
		fclose((FILE*)pParam);
		return 0;
		}


return fread(pBuffer,1,iSize,(FILE*)pParam);
}

//*****************************************************************************
//*
//*		FileWrite
//*
//*****************************************************************************
static int FileWrite(void *pParam,const void *pBuffer,int iSize)
{

	if(pParam==0)return -1;

	if(iSize<=0)
		{
		fclose((FILE*)pParam);
		return 0;
		}


return fwrite(pBuffer,1,iSize,(FILE*)pParam);
}

//*****************************************************************************
//*
//*		MemRead
//*
//*****************************************************************************
static int MemRead(void *pParam,void *pBuffer,int iSize)
{
MemParam   *pData=(MemParam*)pParam;
int			iLen;


	if(iSize<=0)return 0;

	   iLen = pData->iSize-pData->iOff;
	if(iLen<=0)return 0;

	if(iLen>iSize)iLen=iSize;

	memcpy(pBuffer,pData->pMem+pData->iOff,iLen);
	pData->iOff += iLen;


return iLen;
}

//*****************************************************************************
//*
//*		MemWrite
//*
//*****************************************************************************
static int MemWrite(void *pParam,const void *pBuffer,int iSize)
{
MemParam   *pData=(MemParam*)pParam;
int			iLen;


	if(iSize<=0)return 0;

	   iLen = pData->iSize-pData->iOff;
	if(iLen<=0)return 0;

	if(iLen>iSize)iLen=iSize;

	memcpy(pData->pMem+pData->iOff,pBuffer,iLen);
	pData->iOff += iLen;


return iLen;
}

//*****************************************************************************
//*
//*		AzFtpFileName
//*
//*****************************************************************************
//	Erzeugt einen FTP-Dateinamen relativ zum aktuellen Verzeichnis
//	pData		: Zeiger auf die FTP-Client-Daten.
//	pPath		: Ist der Dateiname.
//	pBuffer		: Ist der Puffer in dem der Dateinamen gespeichert wird.
//	iSize		: Ist die Gre des Puffers
//	Ergibt die FTP-Fehlernummer der Anfrage oder 0 bei einem Fehler.
static int AzFtpFileName(AzFtpData *pData,LPCTSTR pPath,char *pBuffer,int iSize)
{
int		iLen;
int		iPos;



	if(pData->iFlags&AZFTP_NO_UTF8)						// Ohne UTF8
		{
		if(*pPath=='/' || *pPath=='\\')					// Globaler Pfad
			{
			iLen = strawcpy(pBuffer,pPath,iSize);
			iPos = 0;
			}
		else{
			if(pPath[0]=='.' && (pPath[1]=='/' || pPath[1]=='\\'))
				{
				pPath+=2;
				}

			iPos = strlcpy (pBuffer,pData->cCurrDir,iSize);
			iLen = strawcpy(pBuffer+iPos,pPath,iSize-iPos)+iPos;
			}

		for(iPos=0;iPos<iLen;iPos++)
			{
			if(pBuffer[iPos]=='\\')pData->cBuffer[iPos]='/';
			}
		}
	else{												// UTF8
		if(*pPath=='/' || *pPath=='\\')					// Globaler Pfad
			{
			iLen = strtoutf8(pBuffer,pPath,iSize);
			iPos = 0;
			}
		else{
			if(pPath[0]=='.' && (pPath[1]=='/' || pPath[1]=='\\'))
				{
				pPath+=2;
				}

			iPos = strlcpy(pBuffer,pData->cCurrDir,iSize);
			
			if(iPos<=0 || (pBuffer[iPos-1]!='/' && pBuffer[iPos-1]!='\\'))
			if(iPos+1<iSize)
				{
				pBuffer[iPos] = '/';
				iPos++;
				}
			
			iLen = strtoutf8(pBuffer+iPos,pPath,iSize-iPos)+iPos;
			}

		for(iPos=0;iPos<iLen;iPos++)
			{
			if(pBuffer[iPos]=='\\')pData->cBuffer[iPos]='/';
			}
		}



return iLen;
}

//*****************************************************************************
//*
//*		AzFtpDecodeResponse
//*
//*****************************************************************************
//	Holt die Fehlernummer aus einer Antwort.
//	pData		: Zeiger auf die FTP-Client-Daten.
//	iSetErr		: Fehlerinfos abspeichern.
//	Ergibt die FTP-Fehlernummer der Anfrage oder 0 bei einem Fehler.
static int AzFtpDecodeResponse(AzFtpData *pData,int iSetErr=1)
{
char   *pText;
int		iPos;
int		iLen;
int		iErr;


	pText	= pData->cBuffer;
	pText[pData->iBufLen] = 0;
	iLen	= pData->iBufLen;
	iErr	= atoi(pText);

	for(iPos=0;iPos<iLen;iPos++)						// berspringe den Fehlercode
		{
		if((unsigned char)pText[iPos]<'0')break;
		if((unsigned char)pText[iPos]>'9')break;
		}

	for(;iPos<iLen;iPos++)								// Suche Fehlertext
		{
		if((unsigned char)pText[iPos]>' ')break;
		}


	for(;iLen>iPos;iLen--)								// Suche Textende
		{
		if((unsigned char)pText[iLen-1]>' ')break;
		}


	pData->iBufTxt = iPos;
	pData->iBufEnd = iLen;

	if(iSetErr)											// Fehlerinfos setzen
		{
		iLen         -= iPos;
		pData->iError = iErr;
		strlcpy(pData->cLastErr,pText+iPos,sizeof(pData->cLastErr),iLen);
		}



return iErr;
}

//*****************************************************************************
//*
//*		AzFtpResvLine
//*
//*****************************************************************************
//	Empfngt die nchste Zeile vom Server.
//	pData		: Zeiger auf die FTP-Client-Daten.
//	iSetErr		: Fehlerinfos abspeichern.
//	iMulti		: Mehrfachzeilen ignorieren
//	Ergibt die FTP-Fehlernummer der Anfrage oder 0 wenn ok.
static int AzFtpResvLine(AzFtpData *pData,int iSetErr=1,int iMulti=1)
{
unsigned 	uStart;
int			iFlag;
int			iDiff;
int			iPos;
int			iLen;
int			iNum;
int			iErr;



	pData->iBufLen = 0;

	uStart	= GetTickCount();
	uStart += pData->uTimeout;
	iPos	= 0;
	iErr	= 0;

	if(pData->iBufOff<pData->iBufNum)					// Sind noch Bytes im Puffer
		{
		iLen = pData->iBufNum-pData->iBufOff;
		memmove(pData->cBuffer,pData->cBuffer+pData->iBufOff,iLen);
		}
	else{
		iLen = 0;
		}


	for(;;)
		{
		   iDiff = uStart-GetTickCount();
		if(iDiff<100)iDiff=100;

		if(iPos>=iLen)
			{
			if(iDiff>2500)								// Nur in kleinen Stcken warten
				{
				iDiff = 2500;
				iFlag = 1;
				}
			else{
				iFlag = 0;
				}

			iNum = sizeof(pData->cBuffer)-1-iLen;
			iNum = TcpGetNext(pData->hTcp,pData->cBuffer+iLen,iNum,iDiff);

			if(iNum<0)									// Abgebrochene Verbindung
				{
				if(iSetErr)
					{
					strcpy(pData->cLastErr,"connection brocken");
					pData->iError = 997;
					}

				TcpClose(pData->hTcp);
				pData->hTcp = 0;
				iErr = 997;

				break;
				}

			if(iNum==0 || pData->iDoBreak)				// Timeout
				{
				if(iFlag && !pData->iDoBreak)
					{
					   iDiff = uStart-GetTickCount();
					if(iDiff>0)continue;
					}

				if(iSetErr)
					{
					strcpy(pData->cLastErr,(pData->iDoBreak)? "user break":"receive timeout");
					pData->iError = 996;
					}

				TcpClose(pData->hTcp);
				pData->hTcp = 0;
				iErr = 996;

				LOG_RECV(pData->cBuffer,iPos)

				break;
				}

			iLen += iNum;
			}

		for(;iPos<iLen;iPos++)							// Suche Zeilenende
			{
			if(pData->cBuffer[iPos]=='\r')
				{
				if(iPos+1>=iLen                )break;
				if(pData->cBuffer[iPos+1]!='\n')continue;

				pData->iBufNum = iLen;
				pData->iBufLen = iPos;
				pData->iBufOff = iPos+2;
				pData->cBuffer[iPos  ]=0;
				pData->cBuffer[iPos+1]=0;

				LOG_RECV(pData->cBuffer,iPos)

				return 0;
				}
			
			if(pData->cBuffer[iPos]=='\n')
				{
				pData->iBufNum = iLen;
				pData->iBufLen = iPos;
				pData->iBufOff = iPos+1;
				pData->cBuffer[iPos  ]=0;

				LOG_RECV(pData->cBuffer,iPos)

				return 0;
				}
			}
		}

	pData->iBufNum = 0;
	pData->iBufLen = 0;
	pData->iBufOff = 0;



return iErr;
}

//*****************************************************************************
//*
//*		AzFtpResvAnwswer
//*
//*****************************************************************************
//	Empfngt die Antwort un berspringt Antworten mit "123-...."
//	pData		: Zeiger auf die FTP-Client-Daten.
//	iSetErr		: Fehlerinfos abspeichern.
//	iMulti		: Mehrfachzeilen ignorieren
//	Ergibt die FTP-Fehlernummer der Anfrage oder 0 wenn ok.
static int AzFtpResvAnwswer(AzFtpData *pData,int iSetErr=1,int iMulti=1)
{
int		iErr;

	
	for(;;)
		{
		   iErr = AzFtpResvLine(pData,iSetErr,iMulti);
		if(iErr)return iErr;

		if(pData->cBuffer[3]!='-' || !pData->cBuffer[0] || !pData->cBuffer[1] || !pData->cBuffer[2])
			{
			break;										// Keine weiteren Antworten	
			}
		}


return iErr;	
}


//*****************************************************************************
//*
//*		AzFtpSendResv
//*
//*****************************************************************************
//	Sendet eine Zeile und wertet die Antwort aus.
//	pData		: Zeiger auf die FTP-Client-Daten.
//	pLine		: Zeiger auf den Zeilentext.
//	iLen		: Ist die Lnge der Zeile.
//	iMode		: Diverse Bits
//					Bit0 = Fehlerinfos abspeichern.
//					Bit1 = Vielfachzeilen nicht berspringen
//	iClose		: Bei Fehlern die Verbindung schlieen.
//	Ergibt die FTP-Fehlernummer der Anfrage oder 0 bei einem Fehler.
static int AzFtpSendResv(AzFtpData *pData,const char *pLine,int iLen,int iMode=1,int iClose=1)
{
int		iMulti;
int		iNum;
int		iErr;



	iNum = TcpPut(pData->hTcp,pLine,iLen);

	LOG_SEND(pLine,iLen-2)

	if(iNum!=iLen)
		{
		TcpClose(pData->hTcp);
		pData->hTcp=0;

		if(iMode&1)
			{
			strcpy(pData->cLastErr,"can't send data");
			pData->iError = 998;
			}

		return 998;
		}

	iMulti = 0;

Next:
	   iErr = AzFtpResvLine(pData,iMode&1,1);
	if(iErr)return iErr;
	   iErr = AzFtpDecodeResponse(pData,iMode&1);
	if(iErr==0 &&  iMulti!=0 && pData->iBufLen>0)
	if(pData->cBuffer[0]<'0' || pData->cBuffer[0]>'9')
		{
		iMulti = 1;
		goto Next;
		}

	if(iErr/100!=2)
		{
		if(iClose)
			{
			TcpClose(pData->hTcp);
			pData->hTcp=0;
			}

		return iErr;
		}

	if(!(iMode&2))										// Gibt es weite Zeilen ("220-.....\r\n")
		{
		if(pData->iBufLen>=4 && pData->cBuffer[3]=='-')
		if(atoi(pData->cBuffer)>=100)
			{
			iMulti = 1;
			goto Next;
			}
		}





return iErr;
}


//*****************************************************************************
//*
//*		AzFtpDoConnect
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis vom FTP Server.
//	pData		: Zeiger auf die FTP-Client-Daten.
//	iSetErr		: Die Fehlernummern setzen
//	Ergibt die FTP-Fehlernummer der Anfrage.
static int AzFtpDoConnect(AzFtpData	*pData,int iSetErr=1)
{
int			iForce;
int			iTurn;
int			iErr;
int			iLen;
int			iOff;


	if(pData->hTcp)
		{
		TcpClose(pData->hTcp);
		pData->hTcp=0;
		}

	if(!IpHostname(&pData->iAddrSrv,pData->cServer))	// Hostname in IP umwandeln
		{
		if(iSetErr)
			{
			strcpy(pData->cLastErr,"wrong hostname");
			pData->iError = 999;
			}

		return 999;
		}

														// Zum Server verbinden
		pData->hTcp = TcpOpenWait(&pData->iAddrSrv,pData->iPort,pData->uTimeout*2,&pData->sTcpCancel);
	if(!pData->hTcp)
		{
		

		if(iSetErr)
			{
			strcpy(pData->cLastErr,"can't connect to target");
			pData->iError = 995;
			}

		return 995;
		}

	
	if(pData->iConnect)pData->iConnect=2;

	LOG_CONN(pData->iAddrSrv,pData->iPort,pData->cServer)


	TcpSendTimeout(pData->hTcp,pData->uTimeout);

	iErr = AzFtpSendResv(pData,"",0);					// Server-Antwort
	if(iErr/100!=2)
		{
		if(iErr!=0 || (pData->cBuffer[0]>='0' && pData->cBuffer[0]<='9'))
			{	
			return iErr;
			}
		}


	iForce = 0;
	iTurn  = 0;

Retry:
														
	if(pData->cUsername[0])								// User anmelden
		{
		iLen = sprintf(pData->cBuffer,"USER %s\r\n",pData->cUsername);
		iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);
		if(iErr/100!=2)
			{
			if(iErr/100!=3)
				{
				TcpClose(pData->hTcp);
				pData->hTcp=0;
				return iErr;
				}
			else{
				iForce = 1;
				}
			}
		}

	if(pData->cPassword[0] || iForce)					// Passwort bergeben
		{
		iLen = sprintf(pData->cBuffer,"PASS %s\r\n",pData->cPassword);
		iErr = AzFtpSendResv(pData,pData->cBuffer,iLen);
		if(iErr/100!=2)
			{
			return iErr;
			}
		}


	iErr = AzFtpSendResv(pData,"TYPE I\r\n",8,1,0);		// Binre bertragung
	if(iErr/100!=2)
		{
		if(iErr=530 && iTurn==0 && pData->cUsername[0]==0 && !(pData->iFlags&AZFTP_NO_ANONYMOUS))
			{
			strcpy(pData->cUsername,"anonymous");
			strcpy(pData->cPassword,"anonymous");
			iTurn = 1;
			goto Retry;
			}

		if(iErr/100!=3 || pData->cPassword[0]==0)
			{
			TcpClose(pData->hTcp);
			pData->hTcp=0;
			return iErr;
			}
		}

	iErr = AzFtpSendResv(pData,"FEAT\r\n",6,2,0);		// Features auslesen
	if(iErr==211)
		{
		for(;;)
			{
			if(AzFtpResvAnwswer(pData,0))break;
			if(pData->cBuffer[0]>='0' && pData->cBuffer[0]<='9' && pData->cBuffer[3]!='-')
				{
				break;
				}

			iErr = atoi(pData->cBuffer);
			if(iErr>=100 && iErr<=999)
					iOff = 4;
			else	iOff = 1;

			if(iErr>0 && iErr!=211 && iSetErr)
				{
				strcpy(pData->cLastErr,"wrong response for FEAD command");
				pData->iError = iErr;
				}

			if(!strcmp(pData->cBuffer+iOff,"EPRT"))pData->iFlags|=AZFTP_USE_EPRT;
			if(!strcmp(pData->cBuffer+iOff,"EPSV"))pData->iFlags|=AZFTP_USE_EPSV;
			if(!strcmp(pData->cBuffer+iOff,"SIZE"))pData->iFlags|=AZFTP_USE_SIZE;
			if(!strcmp(pData->cBuffer+iOff,"MDTM"))pData->iFlags|=AZFTP_USE_MDTM;
			}
		}


	iErr = AzFtpPwd((AzFtpHandle)pData,0,12345);		// Aktuelles Verzeichnis holen



return pData->iError;
}

//*****************************************************************************
//*
//*		AzFtpConnect
//*
//*****************************************************************************
//	Eine Verbindung zu einem FTP-Server aufbauen
//	pServer		: Ipadresse oder Hostnamen des Servers
//	pUserName	: Ist der Username fr die Anmeldung
//	pPassword	: Ist das Passwort fr die Anmeldung
//	usPort		: Ist die TCP-Portnummer.(bei 0 wird 21 benutzt)
//	iFlags		: Diverse Flags:
//						AZFTP_NO_UTF8		0x0001	Dateinamen nicht in UTF8 konvertieren
//						AZFTP_PASSIVE		0x0002	Passiven Modus verwenden
//						AZFTP_NO_ANONYMOUS	0x0010	Anonymous-Login sperren bei User ""
//	uTimeout	: Ist der Timeout in Millisekunden
//	pError		: Hier wird die FTP Fehlernummer gespeichert
//	pCancelPtr	: Hier wird das Abbruch-Handle gespeichert (bei 0 nicht)
//	Ergibt ein Server-Handle oder 0 bei einem Fehler.
AzFtpHandle	AzFtpConnect(LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,unsigned short usPort,int iFlags,unsigned uTimeout,int *pError,void **pCancelPtr)
{
AzFtpData	   *pData;
int				iErr;



		pData = new AzFtpData;
	if(!pData)
		{
		if(pError)*pError=985;
		return 0;
		}

	memset(pData,0,sizeof(AzFtpData));
	if(pCancelPtr)*pCancelPtr=pData;

	strawcpy(pData->cServer  ,pServer  ,sizeof(pData->cServer  ));
	strawcpy(pData->cUsername,pUserName,sizeof(pData->cUsername));
	strawcpy(pData->cPassword,pPassword,sizeof(pData->cPassword));

	if(!usPort)usPort=21;

	pData->iConnect	= 1;
	pData->iPort	= usPort;
	pData->iFlags	= iFlags;
	pData->uTimeout	= uTimeout;
	pData->hLockSem	= CreateSemaphore(0,1,0x70000000,0);
	pData->hWaitSem	= CreateSemaphore(0,0,0x70000000,0);


	LOCK(pData);

	   iErr = AzFtpDoConnect(pData);
	if(iErr/100!=2)
		{
		UNLOCK(pData);
		pData->iConnect	= 0;
		if(pCancelPtr)*pCancelPtr=0;
		CloseHandle(pData->hLockSem);
		CloseHandle(pData->hWaitSem);
		if(pError)*pError=iErr;
		delete pData;
		return 0;
		}

	UNLOCK(pData);

	pData->iConnect	= 2;
	if(pError)*pError=iErr;


return (AzFtpHandle)pData;
}

//*****************************************************************************
//*
//*		AzFtpPwd
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis vom FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pBuffer		: Puffer in dem das Verzeichnis gespeichert wird.
//	iSize		: Gre des Puffers in Zeichen.
//	Ergibt die FTP-Fehlernummer der Anfrage.
int AzFtpPwd(AzFtpHandle hClient,LPTSTR pBuffer,int iSize)
{
AzFtpData	   *pData=(AzFtpData*)hClient;
char		   *pText;
int				iErr;
int				iLen;
int				iNum;


	if(pBuffer || iSize!=12345)
		{
		LOCK(pData);

		if(pData->iFlags&AZFTP_RECONNECT)				// Neu verbinden nach Unterberechungen
		if(TcpIsBroken(pData->hTcp))
			{
			   iErr = AzFtpDoConnect(pData);
			if(iErr/100!=2)
				{
				UNLOCK(pData);
				return iErr;
				}
			}
		}

	iErr = AzFtpSendResv(pData,"PWD\r\n",5,(pBuffer)? 1:0,0);
	if(iErr/100!=2)
		{
		UNLOCK(pData);
		return iErr;
		}

	pText = pData->cBuffer+pData->iBufTxt;
	iLen  = pData->iBufEnd-pData->iBufTxt;

	if(iLen>0 && pText[0]=='"')							// Ist der Pfad zwischen Anfhrungszeichen
		{
		iLen--;
		pText++;

		for(iNum=0;iNum<iLen;iNum++)
			{
			if(pText[iNum]=='"')break;
			}

		iLen = iNum;
		}

	pData->iDirLen = strlcpy(pData->cCurrDir,pText,sizeof(pData->cCurrDir),iLen);

	if(pBuffer)strawcpy(pBuffer,pText,iSize,iLen);

	if(pBuffer || iSize!=12345)
		{
		UNLOCK(pData);
		}



return iErr;
}

//*****************************************************************************
//*
//*		AzFtpCwd
//*
//*****************************************************************************
//	ndert das  aktuelle Verzeichnis vom FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pDirName	: Neuer Verzeichnisnamen. (z.B.: "/" "/aaa/bbb" "/xxx/yyy/")
//	Ergibt die FTP-Fehlernummer der Anfrage.
int	AzFtpCwd(AzFtpHandle hClient,LPCTSTR pDirName)
{
AzFtpData  *pData=(AzFtpData*)hClient;
int			iErr;
int			iLen;



	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}

	strcpy(pData->cBuffer,"CWD ");
	iLen = AzFtpFileName(pData,pDirName,pData->cBuffer+4,sizeof(pData->cBuffer)-8)+4;

	if(iLen>5 && pData->cBuffer[iLen-1]=='/')			// "/xxx/yyy/" zu "/xxx/yyy"
		{
		iLen--;
		}

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

														// Verzeichnis bergeben
	iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);
	if(iErr/100==2)
		{
		AzFtpPwd((AzFtpHandle)pData,0,12345);			// Aktuelles Verzeichnis holen
		}


	UNLOCK(pData);


return iErr;
}


//*****************************************************************************
//*
//*		AzFtpRmDir
//*
//*****************************************************************************
//	Lscht ein Verzeichnis vom FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pDirName	: Ist der Verzeichnisnamen. (z.B.: "/" "/aaa/bbb" "/xxx/yyy/")
//	Ergibt die FTP-Fehlernummer der Anfrage.
int AzFtpRmDir(AzFtpHandle hClient,LPCTSTR pDirName)
{
AzFtpData  *pData=(AzFtpData*)hClient;
int			iErr;
int			iLen;



	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}

	strcpy(pData->cBuffer,"RMD ");
	iLen = AzFtpFileName(pData,pDirName,pData->cBuffer+4,sizeof(pData->cBuffer)-8)+4;

	if(iLen>5 && pData->cBuffer[iLen-1]=='/')			// "/xxx/yyy/" zu "/xxx/yyy"
		{
		iLen--;
		}

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	UNLOCK(pData);


return iErr;
}

//*****************************************************************************
//*
//*		AzFtpMkDir
//*
//*****************************************************************************
//	Erstellt ein Verzeichnis vom FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pDirName	: Ist der Verzeichnisnamen. (z.B.: "/" "/aaa/bbb" "/xxx/yyy/")
//	Ergibt die FTP-Fehlernummer der Anfrage.
int AzFtpMkDir(AzFtpHandle hClient,LPCTSTR pDirName)
{
AzFtpData  *pData=(AzFtpData*)hClient;
int			iErr;
int			iLen;



	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}

	strcpy(pData->cBuffer,"MKD ");
	iLen = AzFtpFileName(pData,pDirName,pData->cBuffer+4,sizeof(pData->cBuffer)-8)+4;

	if(iLen>5 && pData->cBuffer[iLen-1]=='/')			// "/xxx/yyy/" zu "/xxx/yyy"
		{
		iLen--;
		}

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	UNLOCK(pData);


return iErr;
}

//*****************************************************************************
//*
//*		AzFtpRename
//*
//*****************************************************************************
//	Lscht eine Datei vom FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pFileName	: Ist der Dateinamen.
//	Ergibt die FTP-Fehlernummer der Anfrage.
int AzFtpRename(AzFtpHandle hClient,LPCTSTR pFileName)
{
AzFtpData  *pData=(AzFtpData*)hClient;
int			iErr;
int			iLen;



	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}

	strcpy(pData->cBuffer,"DELE ");
	iLen = AzFtpFileName(pData,pFileName,pData->cBuffer+5,sizeof(pData->cBuffer)-8)+5;

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	UNLOCK(pData);


return iErr;
}

//*****************************************************************************
//*
//*		AzFtpRemove
//*
//*****************************************************************************
//	Lscht eine Datei vom FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pFileName	: Ist der Dateinamen.
//	Ergibt die FTP-Fehlernummer der Anfrage.
int AzFtpRemove(AzFtpHandle hClient,LPCTSTR pFileName)
{
AzFtpData  *pData=(AzFtpData*)hClient;
int			iErr;
int			iLen;


	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}

	strcpy(pData->cBuffer,"DELE ");
	iLen = AzFtpFileName(pData,pFileName,pData->cBuffer+5,sizeof(pData->cBuffer)-8)+5;

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	UNLOCK(pData);



return iErr;
}


//*****************************************************************************
//*
//*		AzFtpRename
//*
//*****************************************************************************
//	Lscht eine Datei vom FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pFileName	: Ist der Dateinamen.
//	Ergibt die FTP-Fehlernummer der Anfrage.
int AzFtpRename(AzFtpHandle hClient,LPCTSTR pExisting,LPCTSTR pNewName)
{
AzFtpData  *pData=(AzFtpData*)hClient;
int			iErr;
int			iLen;


	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}

	strcpy(pData->cBuffer,"RNFR ");
	iLen = AzFtpFileName(pData,pExisting,pData->cBuffer+5,sizeof(pData->cBuffer)-8)+5;

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	if(iErr!=350)
		{
		UNLOCK(pData);
		return iErr;
		}

	strcpy(pData->cBuffer,"RNTO ");
	iLen = AzFtpFileName(pData,pNewName,pData->cBuffer+5,sizeof(pData->cBuffer)-8)+5;

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	UNLOCK(pData);



return iErr;
}

//*****************************************************************************
//*
//*		AzFtpChmod
//*
//*****************************************************************************
//	ndert die Zugriffrechte einer Datei am FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pFileName	: Ist der Dateinamen.
//	iPermission	: Sind die neuen Zugriffsrechte
//					0x0001 = Other Execute
//					0x0002 = Other Write
//					0x0004 = Other Read
//					0x0008 = Group Execute
//					0x0010 = Group Write
//					0x0020 = Group Read
//					0x0040 = Owner Execute
//					0x0080 = Owner Write
//					0x0100 = Owner Read
//	Ergibt die FTP-Fehlernummer der Anfrage.	(200=OK)
int AzFtpChmod(AzFtpHandle hClient,LPCTSTR pFileName,int iPermission)
{
AzFtpData  *pData=(AzFtpData*)hClient;
int			iErr;
int			iLen;


	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}


	iLen  = sprintf(pData->cBuffer,"SITE chmod %03o ",iPermission);
	iLen += AzFtpFileName(pData,pFileName,pData->cBuffer+iLen,sizeof(pData->cBuffer)-iLen-4);

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr  = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	UNLOCK(pData);



return iErr;
}


//*****************************************************************************
//*
//*		AzFtpGetTime
//*
//*****************************************************************************
//	ndert die nderungszeit einer Datei am FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pFileName	: Ist der Dateinamen.
//	iPermission	: Sind die neuen Zugriffsrechte
//	Ergibt die FTP-Fehlernummer der Anfrage. (213==ok)
int	AzFtpGetTime(AzFtpHandle hClient,LPCTSTR pRemoteFile,time_t &iTime)
{
AzFtpData  *pData=(AzFtpData*)hClient;
struct tm   sTime;
int			iErr;
int			iLen;


	LOCK(pData);

	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}


	iLen   = sprintf(pData->cBuffer,"MDTM ");
	iLen += AzFtpFileName(pData,pRemoteFile,pData->cBuffer+iLen,sizeof(pData->cBuffer)-iLen-4);

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr  = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);

	if(iErr==213)
		{
		sTime.tm_sec		= atoi(pData->cLastErr+12);   pData->cLastErr[12]=0;
		sTime.tm_min		= atoi(pData->cLastErr+10);   pData->cLastErr[10]=0;
		sTime.tm_hour		= atoi(pData->cLastErr+ 8);   pData->cLastErr[ 8]=0;
		sTime.tm_wday		= 0;
		sTime.tm_mday		= atoi(pData->cLastErr+ 6);   pData->cLastErr[ 6]=0;
		sTime.tm_mon		= atoi(pData->cLastErr+ 4)-1; pData->cLastErr[ 4]=0;
		sTime.tm_year		= atoi(pData->cLastErr+ 0)-1900;
		sTime.tm_isdst		= 0;

		iTime = mktime(&sTime);
		}
	else{
		iTime = -1;
		}

	UNLOCK(pData);




return iErr;
}

//*****************************************************************************
//*
//*		AzFtpSetTime
//*
//*****************************************************************************
//	ndert die nderungszeit einer Datei am FTP Server.
//	hClient		: Handle vom FTP-Client.
//	pFileName	: Ist der Dateinamen.
//	iPermission	: Sind die neuen Zugriffsrechte
//	Ergibt die FTP-Fehlernummer der Anfrage.
int AzFtpSetTime(AzFtpHandle hClient,LPCTSTR pRemoteFile,time_t iTime)
{
AzFtpData  *pData=(AzFtpData*)hClient;
struct tm  *pTime;
int			iErr;
int			iLen;




	if(iTime>0)											// Die Zeitangabe prfen
			pTime = gmtime(&iTime);
	else	pTime = NULL;

	if(!pTime)
		{
		strcpy(pData->cLastErr,"wrong time");
		pData->iError = 986;
		return 986;
		}


	LOCK(pData);


	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			UNLOCK(pData);
			return iErr;
			}
		}


	iLen   = sprintf(pData->cBuffer,"MDTM %04u%02u%02u%02u%02u%02u ",pTime->tm_year+1900,pTime->tm_mon+1,pTime->tm_mday,pTime->tm_hour,pTime->tm_min,pTime->tm_sec);
	iLen += AzFtpFileName(pData,pRemoteFile,pData->cBuffer+iLen,sizeof(pData->cBuffer)-iLen-4);

	pData->cBuffer[iLen] = '\r'; iLen++;
	pData->cBuffer[iLen] = '\n'; iLen++;

	iErr  = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);


	UNLOCK(pData);




return iErr;
}

//*****************************************************************************
//*
//*		AzFtpClose
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
//	hClient		: Handle vom FTP-Client.
//	uTimeout	: Timeout in Millisekunden nach dem ein Abbruch erfolgt	
//					0 = nie abbrechen
//	Ergibt 1 wenn erfolgreich, ansonsten 0
int AzFtpClose(AzFtpHandle hClient,unsigned uTimeout)
{
AzFtpData  *pData=(AzFtpData*)hClient;


	if(!hClient)return 0;

	if(uTimeout)
		{
		if(uTimeout<50)uTimeout=50;

		if(!WAIT(pData,uTimeout))						// Abbrechen wenn es zu lange dauert
			{
			AzFtpAbort(hClient);
			LOCK(pData);
			}
		}
	else{
		LOCK(pData);
		}
														// Ein QUIT-Kommando senden
	if(pData->hTcp && !pData->iDoBreak && !TcpIsBroken(pData->hTcp))
		{
		TcpPut(pData->hTcp,"QUIT\r\n",6);
		LOG_SEND("QUIT",4)
		TcpGetNext(pData->hTcp,pData->cBuffer,sizeof(pData->cBuffer),200);
		TcpClose(pData->hTcp);
		pData->hTcp=0;
		}

	if(pData->hListen)									// Server fr Daten-Verbindung schlieen
		{
		TcpClose(pData->hListen);
		pData->hListen=0;
		}

	UNLOCK(pData);

	CloseHandle(pData->hLockSem);
	CloseHandle(pData->hWaitSem);
	delete pData;


return 1;
}

//*****************************************************************************
//*
//*		AzFtpAbort
//*
//*****************************************************************************
//	Bricht die aktuelle betrtragung ab
//	hClient		: Handle vom FTP-Client.
//	Ergibt 1 wenn erfolgreich, ansonsten 0
int	AzFtpAbort(AzFtpHandle hClient)
{
AzFtpData  *pData=(AzFtpData*)hClient;


	if(hClient==0)return 0;

	if(pData->iConnect==4)								// ffne Datenverbindung Listen
		{
		TcpShutdown(pData->hListen);
		TcpShutdown(pData->hTcp);
		}
	else if(pData->iConnect==3)							// ffne Datenverbindung
		{
		TcpOpenCancel(&pData->sTcpCancel);
		TcpShutdown(pData->hTcp);
		}
	else if(pData->iConnect==2 && pData->hTcp)
		{
		TcpShutdown(pData->hTcp);
		}
	else{
		TcpOpenCancel(&pData->sTcpCancel);
		}
	

	if(pData->iDoBreak)return 2;

	pData->iDoBreak = 1;


return 1;
}

//*****************************************************************************
//*
//*		AzFtpPutFile
//*
//*****************************************************************************
//	bertrgt eine Datei zum FTP-Server
//	hClient			: Handle vom FTP-Client.
//	pRemoteFile		: Dateiname am FTP-server
//	pLocalFile		: Lokaler Dateiname
//	Ergibt die FTP-Fehlernummer der Anfrage. (226=OK)
int AzFtpPutFile(AzFtpHandle hClient,LPCTSTR pRemoteFile,LPCTSTR pLocalFile)
{
AzFtpData  *pData=(AzFtpData*)hClient;
FILE	   *pFile;


	LOCK(pData);


		pFile = _tfopen(pLocalFile,_T("rb"));
	if(!pFile)
		{
		strcpy(pData->cLastErr,"local file not found");
		pData->iError = 994;
		UNLOCK(pData);
		return 994;
		}


	fseek(pFile,0,SEEK_END);
	pData->iSize = ftell(pFile);
	fseek(pFile,0,SEEK_SET);


return AzFtpPutProc(hClient,pRemoteFile,pFile,FileRead);
}

//*****************************************************************************
//*
//*		AzFtpGetFile
//*
//*****************************************************************************
//	bertrgt eine Datei vom FTP-Server
//	hClient			: Handle vom FTP-Client.
//	pRemoteFile		: Dateiname am FTP-server
//	pLocalFile		: Lokaler Dateiname
//	Ergibt die FTP-Fehlernummer der Anfrage.	(226=OK)
int	AzFtpGetFile(AzFtpHandle hClient,LPCTSTR pRemoteFile,LPCTSTR pLocalFile)
{
AzFtpData  *pData=(AzFtpData*)hClient;
FILE	   *pFile;


		pFile = _tfopen(pLocalFile,_T("wb"));
	if(!pFile)
		{
		strcpy(pData->cLastErr,"can't create local file");
		pData->iError = 993;
		return 993;
		}


return AzFtpGetProc(hClient,pRemoteFile,pFile,FileWrite);
}

//*****************************************************************************
//*
//*		AzFtpPutMem
//*
//*****************************************************************************
//	bertrgt einen Speicherbereich als Datei zum FTP-Server
//	hClient			: Handle vom FTP-Client.
//	pRemoteFile		: Dateiname am FTP-server
//	pMem			: Zeiger auf den Speicherbereich.
//	iSize			: Gre des Speicherbereiches in Byte.
//	Ergibt die FTP-Fehlernummer der Anfrage. (226=OK)
int	AzFtpPutMem(AzFtpHandle hClient,LPCTSTR pRemoteFile,const void *pMem,int iSize)
{
AzFtpData  *pData=(AzFtpData*)hClient;
MemParam	sInfo;


	LOCK(pData);

	pData->iSize = iSize;
	sInfo.iSize  = iSize;
	sInfo.pMem   = (char*)pMem;
	sInfo.iOff   = 0;


return AzFtpPutProc(hClient,pRemoteFile,&sInfo,MemRead);
}

//*****************************************************************************
//*
//*		AzFtpSetSize
//*
//*****************************************************************************
//	Stetzt die Gre einer zu bertragenden Datei (fr AzFtpPutProc(...))
//	hClient			: Handle vom FTP-Client.
//	uSize			: Zu bertragente Bytes.
void AzFtpSetSize(AzFtpHandle hClient,UINT uSize)
{
AzFtpData  *pData=(AzFtpData*)hClient;


	pData->iSize = uSize;

}

//*****************************************************************************
//*
//*		AzFtpGetMem
//*
//*****************************************************************************
//	bertrgt eine Datei vom FTP-Server in einen einen Speicherbereich.
//	hClient			: Handle vom FTP-Client.
//	pRemoteFile		: Dateiname am FTP-server
//	pMem			: Zeiger auf den Speicherbereich.
//	iSize			: Gre des Speicherbereiches in Byte.
//	Ergibt die FTP-Fehlernummer der Anfrage.	(226=OK)
int	AzFtpGetMem(AzFtpHandle hClient,LPCTSTR pRemoteFile,void *pMem,int iSize)
{
AzFtpData  *pData=(AzFtpData*)hClient;
MemParam	sInfo;


	pData->iSize = iSize;
	sInfo.iSize  = iSize;
	sInfo.pMem   = (char*)pMem;
	sInfo.iOff   = 0;


return AzFtpGetProc(hClient,pRemoteFile,&sInfo,MemWrite);
}


//*****************************************************************************
//*
//*		AzFtpOpenThread
//*
//*****************************************************************************
//	ffnet einen Datenverbindung zum Server.
static THREAD_DEF AzFtpOpenThread(void *pParam)
{
AzFtpData  *pData=(AzFtpData*)pParam;
int			iTurn;
int			iMax;


	pData->hConn = 0;
	iMax		 = pData->uTimeout/1024+5;

	for(iTurn=0;iTurn<iMax;iTurn++)
		{
		if(pData->iConnStop)break;
		   pData->hConn = TcpOpenWait(&pData->iConnAddr,pData->iDataPort,1000,&pData->sTcpCancel);
		if(pData->hConn)break;
		}

	SIGNAL(pData);

	THREAD_EXIT;
}

//*****************************************************************************
//*
//*		AzFtpOpenDataConn
//*
//*****************************************************************************
//	ffnet einen Datenverbindung.
//	pData		: Zeiger auf die FTP-Client-Daten.
//	hConn		: Hier wird das Handle der
//	pLine		: FTP-Commando fr's Starten
//	iSize		: Lnge des FTP-Kommandos
//	Ergibt die FTP-Fehlernummer der Anfrage. (226=OK)
static int AzFtpOpenDataConn(AzFtpData *pData,TcpHandle &hConn,const char *pLine,int iSize)
{
char			cIpText[64];
unsigned		uStart;
unsigned		uDiff;
unsigned short 	uPort;
IpAddr			iAddr;
int				iFlag;
int				iErr;
int				iPos;
int				iLen;
int				iNum;
int				iMax;


	iFlag				= 0;
	hConn				= 0;
	pData->iBreakDone	= 0;
	pData->iDoBreak		= 0;


//******************** Passiver Modus *****************************************

	for(;;)												// Empfangspuffer leeren
		{
		   iNum = TcpGetNext(pData->hTcp,cIpText,sizeof(cIpText),0);
		if(iNum<=0)break;
		}

	if(pData->iFlags&AZFTP_PASSIVE)
		{												// Erweiteter Modus
Next:	if((pData->iFlags&AZFTP_USE_EPSV) && IsIpV6(&pData->iAddrSrv))
			{
			iErr = AzFtpSendResv(pData,"EPSV\r\n",6,1,0);
			if(iErr!=229)
				{
				return iErr;
				}

			iPos = strlchar(pData->cLastErr,'(');

			for(iNum=0;iNum<3;iNum++)
				{
				iPos += strlchar(pData->cLastErr+iPos,'|');
				}

			pData->iDataPort = atoi(pData->cLastErr+iPos);

			if(iPos<0 || pData->iDataPort<=0 || pData->iDataPort>0xFFFF)
				{
				strcpy(pData->cLastErr,"can't get passive data port");
				pData->iError=992;
				return 992;
				}
			}
		else{
			iErr = AzFtpSendResv(pData,"PASV\r\n",6,1,0);
			if(iErr!=227)
				{
				return iErr;
				}

			iPos = strlchar(pData->cLastErr,'(');

			for(iNum=0;iNum<4;iNum++)
				{
				iPos += strlchar(pData->cLastErr+iPos,',');
				}

			pData->iDataPort  = atoi    (pData->cLastErr+iPos)*256;
			iPos             += strlchar(pData->cLastErr+iPos,',');
			pData->iDataPort += atoi    (pData->cLastErr+iPos);

			if(!pData->cLastErr[iPos] || pData->iDataPort<=0 || pData->iDataPort>0xFFFF)
				{
				strcpy(pData->cLastErr,"can't get passive data port");
				pData->iError=992;
				return 992;
				}
			}

														// Hostname in IP umwandeln
		if(!IpHostname(&pData->iConnAddr,pData->cServer))
			{
			strcpy(pData->cLastErr,"wrong hostname");
			pData->iError = 999;
			return 999;
			}

		iLen = TcpPut(pData->hTcp,pLine,iSize);			// FTP-Kommando senden

		LOG_SEND(pLine,iSize-2);

		if(iLen!=iSize)
			{
			TcpClose(pData->hTcp);
			pData->hTcp = 0;

			strcpy(pData->cLastErr,"can't send data");
			pData->iError   = 998;
			pData->iDoBreak = 0;
			return 998;
			}

		pData->hConn	 = 0;
		pData->iConnStop = 0;
		pData->iConnect  = 3;

		THREAD(AzFtpOpenThread,pData,0x1000);			// ffne Datenverbindung zum Server via Thread

		    iErr = AzFtpResvAnwswer(pData,1);			// Erste Antwort auf STOR/RETR/LIST
		if(!iErr)
			{
			iErr = AzFtpDecodeResponse(pData,1);
			}

		   iErr = AzFtpDecodeResponse(pData,1);
		if(iErr/100!=1)
			{
			pData->iConnStop = 1;
			TcpOpenCancel(&pData->sTcpCancel);
			READY(pData);		
			if(pData->hConn)TcpClose(pData->hConn);
			return pData->iError;
			}

		READY(pData);									// Warte auf den Thread
														
			hConn = pData->hConn;
		if(!hConn)
			{
			strcpy(pData->cLastErr,"can't open passive data connection");
			LOG_CONN_EX("don't get passive data connection");
			pData->iConnect = 0;
			pData->iError   = 991;
			TcpClose(pData->hTcp);
			pData->hTcp = 0;
			return 991;
			}

		if(iFlag)pData->iFlags|=AZFTP_PASSIVE;

		pData->iConnect = 2;

		LOG_CONN(iAddr,pData->iDataPort,"open passive data connection");

		return 0;
		}


//******************** Aktiver Modus ******************************************


	uStart = GetTickCount();

Retry:

	if(!pData->hListen)
		{
		if((pData->iFlags&AZFTP_USE_EPRT) && IsIpV6(&pData->iAddrSrv))
			{
			if(!IpAddress(&iAddr,"::1",0))
				{
				IpAddress(&iAddr,"0.0.0.0");
				}
			}
		else{
			IpAddress(&iAddr,"0.0.0.0");
			}


		for(iPos=0;iPos<16;iPos++)
			{
			ucListenIndex	   += 1;
			pData->iListenPort	= 0xF700+ucListenIndex;
			pData->hListen		= TcpOpen(&iAddr,pData->iListenPort,TCP_SERVER);

			if(pData->hListen)break;
			}

		if(!pData->hListen)
			{
			strcpy(pData->cLastErr,"can't open listen port");
			LOG_CONN_EX("can't open listen port");
			pData->iError=990;
			return 990;
			}

		LOG_CONN(iAddr,pData->iListenPort,"open open listen port");
		}


	TcpGetLocal(pData->hTcp,&iAddr);
	IpToString(&iAddr,cIpText);

	if(pData->iFlags&AZFTP_USE_EPRT && IsIpV6(&pData->iAddrSrv))
		{
		iLen = sprintf(pData->cBuffer,"EPRT |%c|%s|%i|\r\n",IsIpV6(&iAddr)? '2':'1',cIpText,pData->iListenPort);
		iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);
		if(iErr/100!=2)
			{
			return iErr;
			}
		}
	else{
		for(iPos=0;cIpText[iPos];iPos++)
			{
			if(cIpText[iPos]=='.')cIpText[iPos]=',';
			}

		iLen = sprintf(pData->cBuffer,"PORT %s,%i,%i\r\n",cIpText,pData->iListenPort/256,pData->iListenPort&0xFF);
		iErr = AzFtpSendResv(pData,pData->cBuffer,iLen,1,0);
		if(iErr/100!=2)
			{
			if(iErr/100==5)
				{
				iFlag = 1;								// Passiven Modus versuchen
				goto Next;
				}

			return iErr;
			}
		}

	for(;;)												// Unbenutzte Datenverbindungen lschen
		{
		pData->iConnect = 4;
		hConn = TcpAcceptWait(pData->hListen,&iAddr,&uPort,0);
		pData->iConnect = 0;

		if(!hConn)break;
		
		TcpClose(hConn);

		LOG_CONN(iAddr,uPort,"refuse unused data connection");
		}


	iLen   = TcpPut(pData->hTcp,pLine,iSize);			// FTP-Kommando senden

	LOG_SEND(pLine,iSize-2);

	if(iLen!=iSize)
		{
		TcpClose(pData->hTcp);
		pData->hTcp = 0;

		strcpy(pData->cLastErr,"can't send data");
		pData->iError   = 998;
		pData->iDoBreak = 0;
		return 998;
		}



	for(;;)												// Warte auf Datenverbindung vom Server
		{
		   hConn = TcpAcceptWait(pData->hListen,&iAddr,&uPort,1000);
		if(hConn)break;

		   uDiff = GetTickCount()-uStart;
		if(uDiff>pData->uTimeout+5000)break;

		   iLen = TcpGetCount(pData->hTcp);
		if(iLen>0)
			{
			if(pData->iBufOff==pData->iBufNum)			// Ist der Puffer leer
				{
				pData->iBufOff = 0;
				pData->iBufNum = 0;
				}

			iMax = sizeof(pData->cBuffer)-pData->iBufNum;
			iNum = TcpGetNext(pData->hTcp,pData->cBuffer+pData->iBufNum,iLen,0);

			if(iNum>0)pData->iBufNum+=iNum;


			if(pData->iBufNum-pData->iBufOff>3) 		// Existiert eine Fehlermeldung
			if(strwlen(pData->cBuffer+pData->iBufOff,0x01,4)==3)
				{
				if(pData->cBuffer[pData->iBufOff]!='1')break;
				}
			}
		}


	if(!hConn)											// Keine Verbindung vorhanden
		{
		strcpy(pData->cLastErr,"don't get data connection");
		LOG_CONN_EX("don't get data connection");
		pData->iError=984;
		TcpClose(pData->hTcp);
		pData->hTcp = 0;
		return 984;
		}

	LOG_CONN(iAddr,uPort,"open data connection");



	   iErr = AzFtpResvAnwswer(pData,1);				// Erste Antwort auf STOR/RETR/LIST
	if(iErr)return pData->iError;
	   iErr = AzFtpDecodeResponse(pData,1);

	if(iErr==425)										// Keine Verbindung geffnet
		{
		   uDiff = GetTickCount()-uStart;
		if(uDiff>pData->uTimeout*2)						// Nochmals versuchen
			{
			TcpClose(hConn);
			TcpClose(pData->hListen);
			pData->hListen = 0;
			LOG_CONN_EX("retry data connection");
			goto Retry;
			}
		}

	if(iErr/100!=1)
		{
		return pData->iError;
		}



return 0;
}

//*****************************************************************************
//*
//*		AzFtpPutProc
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion zum FTP-Server.
//	hClient			: Handle vom FTP-Client.
//	pRemoteFile		: Dateiname am FTP-server
//	pParam			: Ist der Parameter der der Funktion bergeben wird.
//	pLocalFile		: Lokaler Dateiname
//	Ergibt die FTP-Fehlernummer der Anfrage. (226=OK)
int	AzFtpPutProc(AzFtpHandle hClient,LPCTSTR pRemoteFile,void *pParam,AzFtpFileCallback pProc)
{
AzFtpData  *pData=(AzFtpData*)hClient;
char		cBuffer[1024];
TcpHandle	hConn;
int			iErr;
int			iOff;
int			iLen;
int			iNum;



	if(pProc!=FileRead && pProc!=MemRead)
		{
		LOCK(pData);
		if(!pData->iSize)pData->iSize=0x7FFFFFFF;
		}
	else{
		pData->iSize = 0x7FFFFFFF;
		}


	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			pProc(pParam,NULL,-1);						// Einen Fehler melden
			UNLOCK(pData);
			return iErr;
			}
		}

	pData->iPosNow = 0;

	strcpy(cBuffer,"STOR ");
	iLen = AzFtpFileName(pData,pRemoteFile,cBuffer+5,sizeof(cBuffer)-8)+5;

	cBuffer[iLen] = '\r'; iLen++;
	cBuffer[iLen] = '\n'; iLen++;


														// Datenverbindung ffnen
	   iErr = AzFtpOpenDataConn(pData,hConn,cBuffer,iLen);
	if(iErr)
		{
		pProc(pParam,NULL,-1);							// Einen Fehler melden
		pData->iDoBreak = 0;
		UNLOCK(pData);
		return iErr;
		}

	for(iNum=0,iOff=0;iOff<pData->iSize;iOff+=iLen)		// pertragungsschleife
		{
		pData->iPosNow = iOff;

		if(pData->iDoBreak && !pData->iBreakDone)
			{
			pData->iBreakDone = 1;
			TcpPut(pData->hTcp,"ABOR\r\n",6);
			LOG_SEND("ABOR",4);
			iNum=0;
			break;
			}

		   iNum = pProc(pParam,cBuffer,sizeof(cBuffer));
		if(iNum<=0)break;

		   iLen = TcpPut(hConn,cBuffer,iNum);
		if(iLen!=iNum)
			{
			break;
			}
		}

	TcpClose(hConn);

	if(iNum<0)											// Lokale Schreibfehler
		{
		strcpy(pData->cLastErr,"can't read from local file");
		pData->iError = 989;
		}

	if(iNum>0)											// bertragungsfehler
		{
		strcpy(pData->cLastErr,"can't send data file data");
		pData->iError = 988;
		}


	   iErr = AzFtpResvAnwswer(pData,(!iNum)? 1:0);		// Zweite Antwort auf STOR
	if(iErr)
		{
		goto Exit;
		}


	   iErr = AzFtpDecodeResponse(pData,1);
	if(iErr!=226)
		{
		goto Exit;
		}


Exit:

	if(pData->hListen)									// Listen-Port schlieen
		{
		TcpClose(pData->hListen);
		pData->hListen=0;
		}


	if(pData->iBreakDone)								// Antwort von ABOR holen
		{
		AzFtpResvAnwswer(pData,0);
		pData->iBreakDone=0;
		}

	
	pProc(pParam,NULL,(pData->iError)? -1:0);			// Datei schlieen
	pData->iDoBreak		= 0;


	UNLOCK(pData);




return pData->iError;
}


//*****************************************************************************
//*
//*		AzFtpGetProc
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion vom FTP-Server.
//	hClient			: Handle vom FTP-Client.
//	pRemoteFile		: Dateiname am FTP-server
//	pParam			: Ist der Parameter der der Funktion bergeben wird.
//	pLocalFile		: Lokaler Dateiname
//	Ergibt die FTP-Fehlernummer der Anfrage. (226=OK)
int	AzFtpGetProc(AzFtpHandle hClient,LPCTSTR pRemoteFile,void *pParam,AzFtpReadCallback pProc)
{
AzFtpData  *pData=(AzFtpData*)hClient;
char		cBuffer[1024];
TcpHandle	hConn;
int			iFlag;
int			iErr;
int			iOff;
int			iLen;
int			iNum;



	LOCK(pData);


	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			pProc(pParam,NULL,-1);						// Einen Fehler melden
			UNLOCK(pData);
			return iErr;
			}
		}

	pData->iPosNow = 0;
	pData->iSize   = 0x7FFFFFFF;
	iFlag		   = 0;

	strcpy(cBuffer,"SIZE ");
	iLen = AzFtpFileName(pData,pRemoteFile,cBuffer+5,sizeof(cBuffer)-8)+5;

	cBuffer[iLen] = '\r'; iLen++;
	cBuffer[iLen] = '\n'; iLen++;


	if(pData->iFlags&AZFTP_USE_SIZE)					// Dateigre abfragen
		{
		   iErr = AzFtpSendResv(pData,cBuffer,iLen,1,0);
		if(iErr==213)
			{
			pData->iSize = atoi(pData->cLastErr);
			iFlag		 = 1;
			}
		}

	memcpy(cBuffer,"RETR",4);

														// Datenverbindung ffnen
	   iErr = AzFtpOpenDataConn(pData,hConn,cBuffer,iLen);
	if(iErr)
		{
		pProc(pParam,NULL,-1);							// Einen Fehler melden
		pData->iDoBreak = 0;
		UNLOCK(pData);
		return iErr;
		}


	for(iNum=0,iOff=0;iOff<pData->iSize;iOff+=iLen)		// pertragungsschleife
		{
		pData->iPosNow = iOff;

		if(pData->iDoBreak && !pData->iBreakDone)
			{
			pData->iBreakDone = 1;
			TcpPut(pData->hTcp,"ABOR\r\n",6);
			LOG_SEND("ABOR",4);
			iNum=0;
			break;
			}

		   iLen = pData->iSize-iOff;
		if(iLen>sizeof(cBuffer))iLen=sizeof(cBuffer);

		   iNum = TcpGetWait(hConn,cBuffer,iLen,pData->uTimeout);
		if(iNum<=0)break;

		   iLen = pProc(pParam,cBuffer,iNum);
		if(iLen!=iNum)
			{
			strcpy(pData->cLastErr,"can't write to local file");
			pData->iError = 987;
			iFlag = 2;
			break;
			}
		}

	TcpClose(hConn);

	   iErr = AzFtpResvAnwswer(pData,(iFlag!=2)? 1:0);	// Zweite Antwort auf RETR
	if(iErr)
		{
		goto Exit;
		}

	   iErr = AzFtpDecodeResponse(pData,(iFlag!=2)? 1:0);
	if(iErr!=226)
		{
		goto Exit;
		}

	if(iFlag==1 && iOff<pData->iSize)
		{
		strcpy(pData->cLastErr,"connection closed; transfer aborted");
		pData->iError = 426;
		}

Exit:

	if(pData->hListen)									// Listen-Port schlieen
		{
		TcpClose(pData->hListen);
		pData->hListen=0;
		}


	if(pData->iBreakDone)								// Antwort von ABOR holen
		{
		AzFtpResvAnwswer(pData,0);
		pData->iBreakDone=0;
		}

	pData->iDoBreak = 0;
	pProc(pParam,NULL,(pData->iError)? -1:0);			// Datei schlieen

	UNLOCK(pData);




return pData->iError;
}

//*****************************************************************************
//*
//*		AzFtpParseEntry
//*
//*****************************************************************************
//	Erzeugt aus Listeneintrgen Dateiinformationen.
//	pData		: Zeiger auf die Daten des FTP-Clients
//	pBuffer		: Zeiger auf die Listeneintrge
//	pChain		: Zeiger auf eine verkettete Liste mit Dateiinformationen.
//	Ergibt die Anzahl der verarbeiteten Bytes in den Listeneintrgen
static int AzFtpParseEntry(AzFtpData  *pData,const char *pBuffer,AzFtpFindPtr &pChain)
{
AzFtpFind  *pFind;
struct tm	sData;
SYSTEMTIME	sTime;
SYSTEMTIME	sNowT;
unsigned 	uSize;
int			iAttr;
int			iLink;
int			iBad;
int			iPos;
int			iRet;
int			iLen;
int			iNum;
int			iOff;



	iRet = 0;


Next:

	for(;;iRet+=iLen,pBuffer+=iLen)
		{
		if(pBuffer[0]>='0' && pBuffer[0]<='9')			// Windows-Version "MM-DD-YY HH:MM[AM|PM]  <DIR> ... Name\r\n"
			{
			iLen	= 1;
			iLink	= 0;
			uSize	= 0;
			iAttr	= 0770;

			if(!(pData->iFlags&AZFTP_NO_ANIS))			// Bei Windows-Version ANSI verwenden
				{
				pData->iFlags |= AZFTP_NO_UTF8;
				}

														// Datum und Zeit berspringen
			while((unsigned char)pBuffer[iLen]> ' ')iLen++;
			while((unsigned char)pBuffer[iLen]==' ')iLen++;
			while((unsigned char)pBuffer[iLen]> ' ')iLen++;
			while((unsigned char)pBuffer[iLen]==' ')iLen++;

			iNum = strtoutc(pBuffer,&sData,STR_DATE|STR_TIME|STR_FORMAT_MDY|STR_NOWEEKDAY,iLen);
			iBad = ((iNum&0x7C)!=0x7C)? 1:0;

			sTime.wYear		= sData.tm_year+1900;
			sTime.wMonth	= sData.tm_mon+1;
			sTime.wDay		= sData.tm_mday;
			sTime.wHour		= sData.tm_hour;
			sTime.wMinute	= sData.tm_min;

			if((unsigned char)pBuffer[iLen]<' ')break;

			while(pBuffer[iLen]=='<')					// Flags auswerten
				{
				iLen++;

				if(!strnicmp(pBuffer+iLen,"DIR>",4))
					{
					iLen  += 3;
					iAttr |= 0x4000;
					}

				for(;;)
					{
					if(pBuffer[iLen]=='>'){iLen++;break;}
					if((unsigned char)pBuffer[iLen]<=' ')break;
					iLen++;
					}

				while((unsigned char)pBuffer[iLen]==' ')iLen++;
				}

			if(!(iAttr&0x0200))							// Dateilnge berspringen
				{
				uSize = atoi(pBuffer+iLen);
				while(pBuffer[iLen]>='0' && pBuffer[iLen]<='9')iLen++;
				while((unsigned char)pBuffer[iLen]==' ')iLen++;
				}


			iPos  = iLen;								// Namen berspringen
			while((unsigned char)pBuffer[iLen]>13)iLen++;
			if(pBuffer[iLen]!='\r' || pBuffer[iLen+1]!='\n')break;

			iNum  = iLen-iPos;

			if(iLen==iPos)break;

			iLen += 2;
			}
		else{
			if(pBuffer[0]=='d')							// Verzeichnis, Link oder Datei
				{
				iAttr = 0x4000;
				iLink = 0;
				}
			else if(pBuffer[0]=='l')
				{
				iAttr = 0xA000;
				iLink = 1;
				iOff  = 0;
				}
			else if(pBuffer[0]=='-')
				{
				iAttr = 0;
				iLink = 0;
				}
			else{
				iLen  = 0;
				break;
				}


			for(iLen=1;iLen<10;iLen++)					// Zugriffsrechte prfen
				{
				if(pBuffer[iLen]=="drwxrwxrwx"[iLen])
					{
					iAttr |= 0x200>>iLen;
					continue;
					}

				if(pBuffer[iLen]!='-')break;
				}

			if(iAttr&0x0200)							// Verzeichnissattribut korrigieren
				{
				iAttr &=~0xF200;
				iAttr |= 0x4FFF;
				}

			if(iLen!=10 || pBuffer[iLen]!=' ')break;
														// Hard-Link-Count berspringen
			while((unsigned char)pBuffer[iLen]==' ')iLen++;
			while((unsigned char)pBuffer[iLen]> ' ')iLen++;
			if(pBuffer[iLen]!=' ')break;
														// Besitzer berspringen
			while((unsigned char)pBuffer[iLen]==' ')iLen++;
			while((unsigned char)pBuffer[iLen]> ' ')iLen++;
			if(pBuffer[iLen]!=' ')break;
														// Gruppe berspringen
			while((unsigned char)pBuffer[iLen]==' ')iLen++;
			while((unsigned char)pBuffer[iLen]> ' ')iLen++;
			if(pBuffer[iLen]!=' ')break;

			while(pBuffer[iLen]==' ')iLen++;
			if(pBuffer[iLen]<'0')break;
			if(pBuffer[iLen]>'9')break;
			uSize = atoi(pBuffer+iLen);
														// Dateilnge berspringen
			while(pBuffer[iLen]>='0' && pBuffer[iLen]<='9')iLen++;
			if(pBuffer[iLen]!=' ')break;
			while(pBuffer[iLen]==' ')iLen++;

			iPos  = iLen;								// Monat berspringen
			while((unsigned char)pBuffer[iLen]>' ')iLen++;
			if(pBuffer[iLen]!=' ' || iPos+3!=iLen)break;
			while(pBuffer[iLen]==' ')iLen++;
			sTime.wMonth		= strlsubtext(pBuffer+iPos,pMonthNames,3)+1;

			if(pBuffer[iLen]<'0')break;
			if(pBuffer[iLen]>'9')break;
			sTime.wDay			= atoi(pBuffer+iLen);
														// Tag berspringen
			while(pBuffer[iLen]>='0' && pBuffer[iLen]<='9')iLen++;
			if(pBuffer[iLen]!=' ')break;
			while(pBuffer[iLen]==' ')iLen++;


			if(pBuffer[iLen]<'0')break;
			if(pBuffer[iLen]>'9')break;
			sTime.wHour			= atoi(pBuffer+iLen);


			if(sTime.wHour>1000)
				{
				sTime.wYear		= sTime.wHour;
				sTime.wHour		= 0;
				sTime.wMinute	= 0;
				iBad			= 1;
				}
			else{										// Stunde berspringen
				while(pBuffer[iLen]>='0' && pBuffer[iLen]<='9')iLen++;
				if(pBuffer[iLen]!=':')break;
				iLen++;

				if(pBuffer[iLen]<'0')break;
				if(pBuffer[iLen]>'9')break;
				sTime.wMinute	= atoi(pBuffer+iLen);

				GetSystemTime(&sNowT);					// Das Jahr schtzen
				sTime.wYear = sNowT.wYear;

				if(sNowT.wMonth<=6 && sNowT.wMonth+2<sTime.wMonth)
					{
					sTime.wYear--;
					}

				iBad = (sTime.wMonth)? 0:1;
				}
														// Minuten/Jahr berspringen
			while(pBuffer[iLen]>='0' && pBuffer[iLen]<='9')iLen++;
			if(pBuffer[iLen]!=' ')break;
			iLen++;

			iPos  = iLen;								// Namen berspringen
			while((unsigned char)pBuffer[iLen]>13)iLen++;
			if(pBuffer[iLen]!='\r' || pBuffer[iLen+1]!='\n')break;

			iNum  = iLen-iPos;

			if(iLen==iPos)break;

			if(iLink)									// Links mit " -> " korrigieren
				{
				if(pBuffer[iLen]=='/'){iAttr&=~0xF000;iAttr|=0x4000;}
				iOff = strlfind(pBuffer+iPos," -> ",iNum);
				if(iOff>0){iNum=iOff;iLink=iLen-iOff-iPos-4;}
				else iLink=0;
				}

			iLen += 2;
			}

		pFind = new AzFtpFind;							// Listeneintrag erzeugen

		if(pChain)
			{
			pChain->pNext  = pFind;
			pFind ->pFirst = pChain->pFirst;
			}
		else{
			pFind ->pFirst = pFind;
			}

		pChain = pFind;

		pFind->pNext	= NULL;
		pFind->cBadTime	= iBad;

		LOG_LIST(pBuffer,iLen-2);



		if(pData->iFlags&AZFTP_NO_UTF8)					// Dateinamen kopieren
				strawcpy   (pFind->sData.cFileName,pBuffer+iPos,sizeof(pChain->sData.cFileName)/sizeof(TCHAR),iNum);
		else	strfromutf8(pFind->sData.cFileName,pBuffer+iPos,sizeof(pChain->sData.cFileName)/sizeof(TCHAR),iNum);

		if(iLink)
			{
			strawcpy(pFind->sData.cFileName+iNum+1,pBuffer+iPos+iOff+4,sizeof(pChain->sData.cFileName)/sizeof(TCHAR)-iNum-1,iLink);
			pFind->uIsLink  = (unsigned short)iNum+1;
			}
		else{
			pFind->uIsLink	= 0;
			}


		sTime.wSecond		= 0;
		sTime.wDayOfWeek	= 0;
		sTime.wMilliseconds	= 0;

		if(!SystemTimeToFileTime(&sTime,&pFind->sData.ftLastWriteTime))
			{
			iBad = 1;
			}

		SystemTimeToFileTime(&sNowT,&pFind->sData.ftLastAccessTime);

		pFind->sData.ftCreationTime			= pChain->sData.ftLastWriteTime;
		pFind->sData.dwFileAttributes		= iAttr<<16;
		pFind->sData.nFileSizeLow			= uSize;
		pFind->sData.nFileSizeHigh			= 0;
		pFind->sData.dwReserved0			= 0;
		pFind->sData.dwReserved1			= 0;
		pFind->sData.cAlternateFileName[0]	= 0;

		if(!(iAttr&0x0080))pFind->sData.dwFileAttributes|=FILE_ATTRIBUTE_READONLY;
		if(  iAttr&0x4000 )pFind->sData.dwFileAttributes|=FILE_ATTRIBUTE_DIRECTORY;
		if(  iAttr&0x0001 )pFind->sData.dwFileAttributes|=FILE_ATTRIBUTE_HIDDEN;		// Samba-Hack
		if(  iAttr&0x0008 )pFind->sData.dwFileAttributes|=FILE_ATTRIBUTE_SYSTEM;
		if(  iAttr&0x0040 )pFind->sData.dwFileAttributes|=FILE_ATTRIBUTE_ARCHIVE;
		if(  iLink        )pFind->sData.dwFileAttributes|=FILE_ATTRIBUTE_LINK;
		}




	for(;pBuffer[iLen];iLen++)							// Vor auf die Nchste Zeile
		{
		if(pBuffer[iLen  ]=='\r')
		if(pBuffer[iLen+1]=='\n')
			{
			pBuffer += iLen+2;
			iRet    += iLen+2;

			goto Next;
			}
		}



return iRet;
}

//*****************************************************************************
//*
//*		AzFtpCompressName
//*
//*****************************************************************************
//	Entfernt ".." und "." aus einem Dateifad.
//	pBuffer		: Zeiger auf den Pfad
//	iLen		: Lnge des Pfades
//	Ergibt die neue Lnge
static int AzFtpCompressName(char *pBuffer,int iLen)
{
int		iPos;
int		iOff;


	for(iPos=0;iPos<iLen;iPos++)						// Pfad komprimieren
		{
		if(pBuffer[iPos  ]!='/')continue;
		if(pBuffer[iPos+1]!='.')continue;

		if(pBuffer[iPos+2]=='/' || pBuffer[iPos+2]==0)	// "/." entfernen
			{
			memmove(pBuffer+iPos,pBuffer+iPos+2,iLen-iPos-1);
			iLen -= 2;
			iPos -= 1;
			continue;
			}

		if(pBuffer[iPos+2]=='.' && iPos>0)
		if(pBuffer[iPos+3]=='/' || pBuffer[iPos+3]==0)	// "/.." entfernen	
			{
			for(iOff=iPos-1;iOff>0;iOff--)
				{
				if(pBuffer[iOff]=='/')break;
				}

			memmove(pBuffer+iOff,pBuffer+iPos+3,iLen-iPos-2);
			iLen -= 3+iPos-iOff;
			iPos  = iOff-1;

			if(iLen>0)continue;

			pBuffer[iLen] = '/'; 
			iLen = 1;
			break;
			}
		}

return iLen;
}


//*****************************************************************************
//*
//*		AzFtpCheckLink
//*
//*****************************************************************************
//	Prft ob ein Link eine Datei ist. 
//	pData			: Handle vom FTP-Client.
//	pRemoteFile		: Dateipfad am FTP-server
//	pFileName		: Dateiname ohne Pfad
//	pFind			: Zeiger auf die Dateiinfos
//	Ergibt -1 bei einem Fehler, 0 fr ein Verzeichnis und 1 fr eine Datei
static int AzFtpCheckLink(AzFtpData	*pData,LPCTSTR pRemotePath,LPCTSTR pFileName,AzFtpFind *pFind)
{
AzFtpFind	   *pNext;
AzFtpFind	   *pChain=0;
char			cBuffer[1024];
TcpHandle		hConn;
int				iBreakDone;
int				iByte;
int				iRet;
int				iErr;
int				iLen;
int				iNum;



	   iLen = strlen(pFileName);
	if(iLen>0 && pFileName[iLen-1]=='/')				// Slash am Ende nur bei Verzeichnissen
		{
		pFind->sData.dwFileAttributes  &= 0x0FFFFFFF;
		pFind->sData.dwFileAttributes  |= FILE_ATTRIBUTE_DIRECTORY|(0x4000<<16);
		pFind->sData.nFileSizeLow	    = 0;
		pFind->sData.nFileSizeHigh		= 0;
		pFind->cBadTime					= 0;
		return 0;
		}


	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			return -1;
			}
		}

	pData->iPosNow = 0;
	pData->iSize   = 0;
	pChain         = 0;
	iBreakDone	   = 0;


	strcpy(cBuffer,"LIST -al ");
	
	if(pFileName[0]=='/')								// Pfad zu Link erzeugen
		{
		iLen  = AzFtpFileName(pData,pFileName,cBuffer+9,sizeof(cBuffer)-12)+9;
		}
	else{
		iLen  = AzFtpFileName(pData,pRemotePath,cBuffer+9,sizeof(cBuffer)-12)+9;
		cBuffer[iLen] = '/'; iLen++;
		iLen +=	strawcpy(cBuffer+iLen,pFileName,sizeof(cBuffer)-12-iLen);
		}

	if(iLen>9 && cBuffer[iLen-1]=='/')					// Slasch am Ende entferen
		{
		iLen--;
		} 

	iLen = AzFtpCompressName(cBuffer+9,iLen-9)+9;
	cBuffer[iLen] = '\r'; iLen++;
	cBuffer[iLen] = '\n'; iLen++;


														// Datenverbindung ffnen
	   iErr = AzFtpOpenDataConn(pData,hConn,cBuffer,iLen);
	if(iErr)
		{
		return -1;
		}


	for(iByte=0;;pData->iPosNow++)						// pertragungsschleife
		{
		if(pChain || (pData->iDoBreak && !pData->iBreakDone))
			{
/*			TcpPut(pData->hTcp,"ABOR\r\n",6);			// Macht Probleme bei Datenverbindungen
			LOG_SEND("ABOR",4);
			iBreakDone = 1;
			break;
*/			}

		   iLen = sizeof(cBuffer)-1-iByte;
		   iNum = TcpGetNext(hConn,cBuffer+iByte,iLen,pData->uTimeout);
		if(iNum<=0)break;

		iByte += iNum;

		cBuffer[iByte] = 0;

		iNum   = AzFtpParseEntry(pData,cBuffer,pChain);

		if(iNum==0 && iByte>512)
			{
			iByte = 0;
			break;
			}

		memmove(cBuffer,cBuffer+iNum,iByte-iNum);
		iByte -= iNum;
		}


	TcpClose(hConn);

	iErr = AzFtpResvAnwswer(pData,1);					// Zweite Antwort auf LIST
	iErr = AzFtpDecodeResponse(pData,1);

	strcpy(cBuffer,"MDTM ");
	if(pRemotePath[0]=='.' && pRemotePath[1]==0)pRemotePath=_T("");

	
	if(pFileName[0]=='/')								// Pfad zu Link erzeugen
		{
		iLen  = AzFtpFileName(pData,pFileName,cBuffer+5,sizeof(cBuffer)-8)+5;
		}
	else{
		iLen  = AzFtpFileName(pData,pRemotePath,cBuffer+5,sizeof(cBuffer)-8)+5;
		cBuffer[iLen] = '/'; iLen++;
		iLen +=	strawcpy(cBuffer+iLen,pFileName,sizeof(cBuffer)-12-iLen);
		}

	iLen = AzFtpCompressName(cBuffer+5,iLen-5)+5;

	if(iLen>5 && cBuffer[iLen-1]!='/'){cBuffer[iLen]='/';iLen++;}
	if(pChain)pChain=(AzFtpFind*)pChain->pFirst;



	if(pData->hListen)									// Listen-Port schlieen
		{
		TcpClose(pData->hListen);
		pData->hListen=0;
		}


	if(pChain)											// Ergebnisse ausgeben
		{
		if(pChain->uIsLink==0)
		if(pChain->cBadTime)
		if(pData->iFlags&AZFTP_USE_MDTM)				// nderungszeit holen
			{
			if(pData->iFlags&AZFTP_NO_UTF8)
					iNum = strawcpy (cBuffer+iLen,pChain->sData.cFileName,sizeof(cBuffer)-4-iLen)+iLen;
			else	iNum = strtoutf8(cBuffer+iLen,pChain->sData.cFileName,sizeof(cBuffer)-4-iLen)+iLen;

			cBuffer[iNum] = '\r'; iNum++;
			cBuffer[iNum] = '\n'; iNum++;
			iRet  = AzFtpSendResv(pData,cBuffer,iNum,1,0);

			if(iRet==213)
				{
				SYSTEMTIME	sTime;

				sTime.wMilliseconds	= 0; 						pData->cLastErr[14]=0;
				sTime.wSecond		= atoi(pData->cLastErr+12); pData->cLastErr[12]=0;
				sTime.wMinute		= atoi(pData->cLastErr+10); pData->cLastErr[10]=0;
				sTime.wHour			= atoi(pData->cLastErr+ 8); pData->cLastErr[ 8]=0;
				sTime.wDayOfWeek	= 0;
				sTime.wDay			= atoi(pData->cLastErr+ 6); pData->cLastErr[ 6]=0;
				sTime.wMonth		= atoi(pData->cLastErr+ 4); pData->cLastErr[ 4]=0;
				sTime.wYear			= atoi(pData->cLastErr+ 0);

				SystemTimeToFileTime(&sTime,&pFind->sData.ftLastWriteTime);
				}
			}

		if((pChain->sData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) || pChain->pNext)
			{
			pFind->sData.dwFileAttributes  &= 0x0FFFFFFF;
			pFind->sData.dwFileAttributes  |= FILE_ATTRIBUTE_DIRECTORY|(0x4000<<16);
			pFind->sData.nFileSizeLow	    = 0;
			pFind->sData.nFileSizeHigh		= 0;
			pFind->cBadTime					= 0;
			iErr							= 0;
			}
		else{
			pFind->sData.nFileSizeLow		= pChain->sData.nFileSizeLow;
			pFind->sData.nFileSizeHigh		= pChain->sData.nFileSizeHigh;
			pFind->cBadTime					= 0;
			iErr							= 1;
			}

		while(pChain)									// Kette freigeben
			{
			pNext  = (AzFtpFind*)pChain->pNext;
			delete pChain;
			pChain = pNext;
			}
		}
	else{
		iErr = 1;
		}


	if(iBreakDone)										// Antwort von ABOR holen
		{
		AzFtpResvAnwswer(pData,0);
		pData->iBreakDone=0;
		}

	pData->iDoBreak	= 0;



return iErr;
}

//*****************************************************************************
//*
//*		AzFtpList
//*
//*****************************************************************************
//	Liest Verzeichniseintrge via Callback-Funktion vom FTP-Server ein.
//	hClient			: Handle vom FTP-Client.
//	pRemoteFile		: Dateiname am FTP-server
//	pParam			: Ist der Parameter der der Funktion bergeben wird.
//	pProc			: Callback zum Melden der Dateiliste
//	Ergibt die FTP-Fehlernummer der Anfrage. (226=OK)
int	AzFtpList(AzFtpHandle hClient,LPCTSTR pRemotePath,void *pParam,AzFtpFindCallback pProc)
{
AzFtpData	   *pData=(AzFtpData*)hClient;
AzFtpFind	   *pNext;
AzFtpFind	   *pChain;
char			cBuffer[1024];
TcpHandle		hConn;
int				iByte;
int				iRet;
int				iErr;
int				iLen;
int				iNum;
int 			iOk;



	LOCK(pData);


	if(pData->iFlags&AZFTP_RECONNECT)					// Neu verbinden nach Unterberechungen
	if(TcpIsBroken(pData->hTcp))
		{
		   iErr = AzFtpDoConnect(pData);
		if(iErr/100!=2)
			{
			pProc(pParam,NULL);							// Ende anzeigen
			UNLOCK(pData);
			return iErr;
			}
		}

	pData->iPosNow = 0;
	pData->iSize   = 0;
	pChain         = 0;


	strcpy(cBuffer,"LIST -al ");
	iLen = AzFtpFileName(pData,pRemotePath,cBuffer+9,sizeof(cBuffer)-12)+9;

	cBuffer[iLen] = '\r'; iLen++;
	cBuffer[iLen] = '\n'; iLen++;



														// Datenverbindung ffnen
	   iErr = AzFtpOpenDataConn(pData,hConn,cBuffer,iLen);
	if(iErr)
		{
		pProc(pParam,NULL);								// Ende anzeigen
		pData->iDoBreak = 0;
		UNLOCK(pData);
		return iErr;
		}


	for(iByte=0;;pData->iPosNow++)						// bertragungsschleife
		{
		if(pData->iDoBreak && !pData->iBreakDone)
			{
			pData->iBreakDone = 1;
			TcpPut(pData->hTcp,"ABOR\r\n",6);
			LOG_SEND("ABOR",4);
			iNum=0;
			break;
			}


		   iLen = sizeof(cBuffer)-1-iByte;
		   iNum = TcpGetNext(hConn,cBuffer+iByte,iLen,pData->uTimeout);
		if(iNum<=0)break;


		iByte += iNum;

		cBuffer[iByte] = 0;

		iNum   = AzFtpParseEntry(pData,cBuffer,pChain);

		if(iNum==0 && iByte>512)
			{
			iByte = 0;
			break;
			}

		memmove(cBuffer,cBuffer+iNum,iByte-iNum);
		iByte -= iNum;
		}


	if(!pData->iBreakDone)								// Gab es einen Abbruch
		{
		iOk				= 1;
		cBuffer[iByte]	= 0;
		AzFtpParseEntry(pData,cBuffer,pChain);
		}
	else{
		iOk				= 0;
		}


	TcpClose(hConn);

	   iErr = AzFtpResvAnwswer(pData,1);				// Zweite Antwort auf LIST
	if(iErr)
		{
		iOk = 0;
		goto Exit;
		}

	   iErr = AzFtpDecodeResponse(pData,1);
	if(iErr!=226)
		{
		iOk = 0;
		goto Exit;
		}


	strcpy(cBuffer,"MDTM ");
	if(pRemotePath[0]=='.' && pRemotePath[1]==0)pRemotePath=_T("");
	iLen  = AzFtpFileName(pData,pRemotePath,cBuffer+5,sizeof(cBuffer)-8)+5;
	if(iLen>5 && cBuffer[iLen-1]!='/'){cBuffer[iLen]='/';iLen++;}
	if(pChain)pChain=(AzFtpFind*)pChain->pFirst;


Exit:


	if(pData->hListen)									// Listen-Port schlieen
		{
		TcpClose(pData->hListen);
		pData->hListen=0;
		}


	while(pChain && iOk)								// Ergebnisse ausgeben
		{
														// Links auflsen
		if(pChain->uIsLink && (pData->iFlags&AZFTP_LINK_RESOLVE))
			{
			AzFtpCheckLink(pData,pRemotePath,pChain->sData.cFileName+pChain->uIsLink,pChain);
			pChain->sData.dwFileAttributes |= FILE_ATTRIBUTE_LINK;
			}
		
		if(pChain->cBadTime)
		if(pData->iFlags&AZFTP_USE_MDTM)				// nderungszeit holen
			{
			if(pData->iFlags&AZFTP_NO_UTF8)
					iNum = strawcpy (cBuffer+iLen,pChain->sData.cFileName,sizeof(cBuffer)-4-iLen)+iLen;
			else	iNum = strtoutf8(cBuffer+iLen,pChain->sData.cFileName,sizeof(cBuffer)-4-iLen)+iLen;

			cBuffer[iNum] = '\r'; iNum++;
			cBuffer[iNum] = '\n'; iNum++;
			iRet  = AzFtpSendResv(pData,cBuffer,iNum,1,0);

			if(iRet==213)
				{
				SYSTEMTIME	sTime;

				sTime.wMilliseconds	= 0; 						pData->cLastErr[14]=0;
				sTime.wSecond		= atoi(pData->cLastErr+12); pData->cLastErr[12]=0;
				sTime.wMinute		= atoi(pData->cLastErr+10); pData->cLastErr[10]=0;
				sTime.wHour			= atoi(pData->cLastErr+ 8); pData->cLastErr[ 8]=0;
				sTime.wDayOfWeek	= 0;
				sTime.wDay			= atoi(pData->cLastErr+ 6); pData->cLastErr[ 6]=0;
				sTime.wMonth		= atoi(pData->cLastErr+ 4); pData->cLastErr[ 4]=0;
				sTime.wYear			= atoi(pData->cLastErr+ 0);

				SystemTimeToFileTime(&sTime,&pChain->sData.ftLastWriteTime);
				}
			}

		pNext  = (AzFtpFind*)pChain->pNext;
		iOk=pProc(pParam,&pChain->sData);
		delete pChain;
		pChain = pNext;
		}

	while(pChain)										// Resliche Kette freigeben
		{
		pNext  = (AzFtpFind*)pChain->pNext;
		delete pChain;
		pChain = pNext;
		}



	pProc(pParam,NULL);									// Ende anzeigen

	if(pData->iBreakDone)								// Antwort von ABOR holen
		{
		AzFtpResvAnwswer(pData,0);
		pData->iBreakDone=0;
		}

	pData->iDoBreak	= 0;


	UNLOCK(pData);

	pData->iError = iErr;


return iErr;
}

//*****************************************************************************
//*
//*		AzFtpIsConnected
//*
//*****************************************************************************
//	Prft ob die Verbindung zum FTP-Server ok ist.
//	hClient		: Handle vom FTP-Client.
//	Ergibt 1 wenn die Verbindung ok ist, bzw. 0 wenn nicht.
int	AzFtpIsConnected(AzFtpHandle hClient)
{
AzFtpData	   *pData=(AzFtpData*)hClient;


	if(TcpIsBroken(pData->hTcp))
		{
		return 0;
		}


return 0;
}

//*****************************************************************************
//*
//*		AzFtpReconnect
//*
//*****************************************************************************
//	Die Verbindung zum FTP-server neu aufbauen.
//	hClient		: Handle vom FTP-Client.
//	Ergibt die FTP-Fehlernummer der Anfrage.
int	AzFtpReconnect(AzFtpHandle hClient)
{
AzFtpData	   *pData=(AzFtpData*)hClient;
int				iErr;



	LOCK(pData);

	iErr = AzFtpDoConnect(pData);

	UNLOCK(pData);


return iErr;
}

//*****************************************************************************
//*
//*		AzFtpSetTimeout
//*
//*****************************************************************************
//	Stellt den Timeout-Wert ein.
//	hClient			: Handle vom FTP-Client.
//	uTimeout	: Timeout in Millisekunden
//	Ergibt 1 wenn das Timeout gesetzt wurde.
int AzFtpSetTimeout(AzFtpHandle hClient,UINT uTimeout)
{
AzFtpData	   *pData=(AzFtpData*)hClient;


	if(!hClient)return 0;

	if(uTimeout<  1000)uTimeout=1000;
	if(uTimeout>200000)uTimeout=200000;

	pData->uTimeout = uTimeout;


return 1;
}

//*****************************************************************************
//*
//*		AzFtpGetTimeout
//*
//*****************************************************************************
//	Den Timeout-Wert abfragen.
//	hClient		: Handle vom FTP-Client.
//	Ergibt das Timeout in Millisekunden, bzw. 0 bei einem Fehler
int AzFtpGetTimeout(AzFtpHandle hClient)
{
AzFtpData	   *pData=(AzFtpData*)hClient;


	if(!hClient)return 0;


return pData->uTimeout;
}

//*****************************************************************************
//*
//*		AzFtpGetProgress
//*
//*****************************************************************************
//	Den Wert der Verlaufsanzeige abfragen.
//	hClient		: Handle vom FTP-Client.
//	pPos		: Hier wird die aktuelle Verlaufsposition gespeichert,
//				  wenn der Zeiger nicht 0 ist.
//	pMax		: Hier wird die maximale Verlaufsposition gespeichert,
//				  wenn der Zeiger nicht 0 ist.
//				  Falls
//	Ergibt 1 wenn die Werte abgefragt wurden.
int AzFtpGetProgress(AzFtpHandle hClient,UINT *pPos,UINT *pMax)
{
AzFtpData	   *pData=(AzFtpData*)hClient;


	if(!hClient)
		{
		if(pPos)*pPos=0;
		if(pMax)*pMax=0;
		return 0;
		}


	if(pPos)*pPos=pData->iPosNow;
	if(pMax)*pMax=pData->iSize;


return 1;
}


//*****************************************************************************
//*
//*		AzFtpSetProgress
//*
//*****************************************************************************
//	Den Wert fr die Verlaufsanzeige setzen. 
//	hClient		: Handle vom FTP-Client.
//	uPos		: Hier wird die aktuelle Verlaufsposition eingestellt
//	uMax		: Hier wird die maximale Verlaufsposition eingestellt
//	Ergibt 1 wenn die Werte abgefragt wurden.
int AzFtpSetProgress(AzFtpHandle hClient,UINT uPos,UINT uMax)
{
AzFtpData	   *pData=(AzFtpData*)hClient;


	if(!hClient)
		{
		return 0;
		}


	if(uPos!=0xFFFFFFFF)pData->iPosNow=uPos;
	if(uMax!=0xFFFFFFFF)pData->iSize  =uMax;


return 1;
}
