//*****************************************************************************
//*
//*
//*		DiskTools.h
//*
//*
//*****************************************************************************
#ifndef 	__DISKTOOLS_H__
#define 	__DISKTOOLS_H__

#include	<stdlib.h>


#ifndef 	DPD_MAX_PARTIONS
#define 	DPD_MAX_PARTIONS			8					// Anzahl an Partitionen (logische+physikalische)
#endif														
															
#ifndef 	DPD_HAS_FAT										
#define 	DPD_HAS_FAT					1					// FAT-Partitionen erlauben
#endif														
															
#ifndef 	DPD_HAS_EXFAT									
#define 	DPD_HAS_EXFAT				1					// EXFAT-Partitionen erlauben
#endif														
															
#ifndef 	DPD_HAS_EXT										
#define 	DPD_HAS_EXT					1					// EXT2..4-Partitionen erlauben
#endif														
															
#ifndef 	DPD_BASE_LETTER									
#define 	DPD_BASE_LETTER				'0'					// Erster Laufwerksbuchstabe
#endif														
															
#ifndef		DPD_MAX_PATH									
#define 	DPD_MAX_PATH				512					// Maximale Lnge eines Dateipfades
#endif														
															
#ifndef 	DISK_MAX_FILES									
#define 	DISK_MAX_FILES				16					// Maximale Anzahl an Datei-Handels
#endif														
															
#ifndef 	DISK_MAX_FIND									
#define 	DISK_MAX_FIND				8					// Maximale Anzahl an Such-Handels
#endif														
															
#ifndef 	DPD_CACHE_SIZE									
#define 	DPD_CACHE_SIZE				0					// Anzahl der Sektoren fr Cache
#endif														
															
#ifndef 	DISK_HAS_SHORT_NAME								
#define 	DISK_HAS_SHORT_NAME			0					
#endif														
															
#define 	DPD_TYPE_NONE				0					// Slot ist unbenutzt
#define 	DPD_TYPE_PHYSICAL			1					// Slot ist eine pysikalische Partition
#define 	DPD_TYPE_EMPTY				2					// Slot ist eine leere Partition
#define 	DPD_TYPE_FAT				3					// Slot ist eine FAT Partition
#define 	DPD_TYPE_EXFAT				4					// Slot ist eine EXFAT Partition
#define 	DPD_TYPE_EXT				5					// Slot ist eine EXT? Partition
															
#define 	DPD_MODE_NONE				0					// Kein Zugriff auf die Partition
#define 	DPD_MODE_FILE				2					// Zugriff auf Image-Datei
#define 	DPD_MODE_FILE_RO			3					// Zugriff auf Image-Datei (ReadOnly)
#define 	DPD_MODE_DISK				4					// Zugriff auf pysikalisches Laufwerk
#define 	DPD_MODE_DISK_RO			5					// Zugriff auf pysikalisches Laufwerk (ReadOnly)
															
#define		DPD_CTRL_SYNC				0					// Alle Puffer leeren
#define		DPD_CTRL_SECTOR_COUNT		1					// Abfragen der Partitionsgre (32-Bit)
#define		DPD_CTRL_SECTOR_SIZE 		2					// Abfragen der Gre eines Sektors in Byte
#define		DPD_CTRL_BLOCK_SIZE			3					// Abfragen der Blockgre in Sektoren
#define		DPD_CTRL_POWER				4					// ???
#define		DPD_CTRL_LOCK				5					// Das Laufwerk sperren
#define		DPD_CTRL_EJECT				6					// Die Diskette auswerfen
#define		DPD_CTRL_SECTOR_CNT64		7					// Abfragen der Partitionsgre (64-Bit)
#define		DPD_CTRL_READONLY			8					// Ist die Partition ReadOnly
															
#define 	DPD_OK						0 					// 0: Successful
#define 	DPD_ERROR					1					// 1: R/W Error
#define 	DPD_READONLY				2					// 2: Write Protected
#define 	DPD_NOTREADY				3 					// 3: Not Ready
#define 	DPD_PARAMERR				4					// 4: Invalid Parameter
															
#define 	DPD_COMMON					0					// Laufwerk ohne Schreibschutz
#define		DPD_NOINIT					1					// Laufwerk nicht initialisiert
#define		DPD_NODISK					2					// Kein Medium im Laufwerk
#define		DPD_PROTECT 				4					// Laufwerk hat Schreibschutz
															
#define 	DPD_ERR_NONE				0					// Kein Fehler
#define 	DPD_ERR_PARAM			   -1					// Falscher Parameter
#define 	DPD_ERR_INVALID_FILENAME   -2					// Ungltiger Dateiname
#define 	DPD_ERR_FILE_NOT_FOUND	   -3					// Die Datei wurde nicht gefunden
#define 	DPD_ERR_ACCESS_DENIED	   -4					// Zugriff verweigert
#define 	DPD_ERR_NO_HANDLES		   -5					// Keine freien Handels
#define 	DPD_ERR_UNKNOWN			   -6					// Unbekannter Fehler
#define 	DPD_ERR_PATH_NOT_FOUND	   -7					// Der Pfad wurde nicht gefunden
#define 	DPD_ERR_DISKFULL		   -8					// Die Partitione ist voll
#define		DPD_ERR_ALREADY_EXISTS	   -9					// Die Datei existiert bereits
															
#define		DPD_FLAG_READONLY 			0x01				// Laufwerk im Readolny-Modus ffnen
#define		DPD_FLAG_NO_EMPTY			0x02				// Nur formatierte Laufwerke mounten
#define		DPD_FLAG_SINGLE				0x04				// Keine Subpfade bei nur einer Partition
#define		DPD_FLAG_INIT				0x08				// Die Laufwerksdaten initialisieren
															
#define     DPD_READ        			0x0001				// Leserechte fr die Datei
#define     DPD_WRITE       			0x0002      		// Schreibrechte fr die Datei
#define     DPD_CREATE      			0x0004      		// Die Datei erstellen wenn sie nicht existiert
#define     DPD_TRUNC       			0x0008      		// Die Datei auf die Gre 0 krzen
#define     DPD_DIR         			0x0010      		// Die Date ist ein Verzeichniss
#define     DPD_HIDDEN      			0x0020      		// Die Datei ist versteckt
#define     DPD_ARCHIVE     			0x0040      		// Das Archive-Flag ist gesetzt
#define     DPD_SYSTEM      			0x0080      		// Das System-Flag ist gesetzt
#define     DPD_VOLUME      			0x0100      		// Das Volume-Flag ist gesetzt
#define     DPD_EXEC	    			0x0200      		// Ausfhrungsrechte fr die Datei
#define     DPD_HIGHMASK            	0xFC00      		// Maske fr 64 Bit-Dateigre
#define     DPD_HIGHSHIFT           	10          		// Shift fr 64 Bit-Dateigre
															
															

#ifndef 	DISK_USEUFT16
typedef		char				DCHAR;
#else
typedef		wchar_t				DCHAR;
#endif

#ifndef		QWORD_DEFINED
#define		QWORD_DEFINED
typedef		unsigned __int64	QWORD;
#endif

typedef		unsigned short     *DiskFile;					// Handels fr File-Funktionen
typedef		unsigned		   *DiskFind;					// Handels fr Find-Funktionen
typedef		DWORD				DiskUtc;					// Typ fr die Dateizeit
typedef		__int64				DiskPos;					// 64Bit Datei Offset

typedef	unsigned char (_cdecl  *DiskInitCb   )(void*,DWORD);
typedef	unsigned char (_cdecl  *DiskStatusCb )(void*,DWORD);
typedef	unsigned char (_cdecl  *DiskIoctlCb  )(void*,DWORD,      unsigned char ,void*);
typedef	unsigned char (_cdecl  *DiskWriteCb  )(void*,DWORD,const unsigned char*,QWORD,DWORD);
typedef	unsigned char (_cdecl  *DiskReadCb   )(void*,DWORD,      unsigned char*,QWORD,DWORD);

extern "C" unsigned char _cdecl DiskInitProc  (void *pPtr,DWORD uDrive);
extern "C" unsigned char _cdecl DiskStatusProc(void *pPtr,DWORD uDrive);
extern "C" unsigned char _cdecl DiskIoctlProc (void *pPtr,DWORD uDrive,      unsigned char ucCommand,void *pBuffer);
extern "C" unsigned char _cdecl DiskReadProc  (void *pPtr,DWORD uDrive,      unsigned char *pBuffer,QWORD uSecOffset,DWORD uSecCount);
extern "C" unsigned char _cdecl DiskWriteProc (void *pPtr,DWORD uDrive,const unsigned char *pBuffer,QWORD uSecOffset,DWORD uSecCount);


typedef	struct
	{
	unsigned		uDriveCount;							// Anzahl aller Partitionen
	unsigned		uPartitionCount;						// Anzahl der logischen Partitionen
	unsigned		uCurrenrSlot;							// Partition mit dem aktuellen Verzeichniss (0=keines 1=[0] ...)
	unsigned		uCurrenrPathLen;						// Lnge des Aktuellen Pfades
	unsigned		uSingleSlot;							// Slot der einzigen Partition ([Slot]=N-1)
	DCHAR			aCurrentPath[DPD_MAX_PATH];				// Aktueller Pfad auf Partition (immer '/' am Ende)
	void	       *aFindPtr  [DISK_MAX_FIND];				// Find Handles
	void	       *aLocalData[DPD_MAX_PARTIONS];			// Speicher fr Daten
	void	       *aCacheData[DPD_MAX_PARTIONS];			// Speicher fr Sektoren-Cache
	void	       *aDrivesPtr[DPD_MAX_PARTIONS];			// File-Handle fr Zugriff
	BYTE		 	aDriveType[DPD_MAX_PARTIONS];			// Art der Partition (physikalisch,leer,FAT,EXFAT,...)
	BYTE		 	aDriveSign[DPD_MAX_PARTIONS];			// Laufwerksbuchstabe (0=aus 1='0' 2='1' ...)
	BYTE		 	aDriveMode[DPD_MAX_PARTIONS];			// Art des Zugriffes (FILE,DISK,ReadOnly)
	BYTE		 	aDriveHide[DPD_MAX_PARTIONS];			// War die Partition versteckt
	DWORD 			aMountFlag[DPD_MAX_PARTIONS];			// Flags beim Mounten
	DWORD 			aRefCount [DPD_MAX_PARTIONS];			// Referenz zu logischen Laufwerk
	QWORD			aStartSec [DPD_MAX_PARTIONS];			// Erster Sektor der Partition
	QWORD			aCountSec [DPD_MAX_PARTIONS];			// Gre der Partition
	unsigned char	aLetterToSlot[64];						// Laufwerk zu Slot Info ([0]='0' [1]='1' ... [17]='A'...)
	}DiskPartionData;

typedef	struct
	{
	DWORD			dwAttr;									// Dateiattribute (DPD_???)
	DWORD			dwSize;									// Dateigre
	DiskUtc 		dwTimeCreate;							// Erstellzeit der Datei
	DiskUtc			dwTimeAccess;							// Letzter Zugriff auf die Datei
	DiskUtc			dwTimeWrite;							// Letzte nderung der Datei
	DCHAR			cName[256];								// Name der Datei
    #if     		DISK_HAS_SHORT_NAME
    char    		cShortName[16];
    #endif
	}DiskFindStruct;


int DiskMount  (DiskPartionData *pData,LPCTSTR pPathName,DWORD dwFlags=0,unsigned *pSlot=0,unsigned uPartition=0xFFFFFFFF);
int DiskUnmount(DiskPartionData *pData,unsigned uSlot=0);

#define		DiskFileOpen(d,n,f)		DiskFileOpenEx(d,n,f,0)
#define		DiskFindFirst(d,n,f)	DiskFindFirstEx(d,n,f,0)
#define 	DiskFileSeek(d,h,p,m)	((int)DiskFileSeekEx(d,h,p,m))
#define 	DiskFileTell(d,h)		((int)DiskFileTellEx(d,h))
#define 	DiskFileLength(d,n)		((int)DiskFileLengthEx(d,n))
#define 	DiskGetFileSize(d,h)	((int)DiskGetFileSizeEx(d,h))


DiskFile	DiskFileOpenEx    (DiskPartionData *pData,const DCHAR *pFilename,int iMode,int *pError);
DiskPos 	DiskFileSeekEx    (DiskPartionData *pData,DiskFile     hFile,DiskPos 	 iPos   ,int iMode);
int 		DiskFileRead      (DiskPartionData *pData,DiskFile     hFile,      void *pBuffer,int iSize);
int 		DiskFileWrite     (DiskPartionData *pData,DiskFile     hFile,const void *pBuffer,int iSize);
int     	DiskFileClose     (DiskPartionData *pData,DiskFile     hFile);
int 		DiskFileFlush     (DiskPartionData *pData,DiskFile	   hFile);
DiskPos		DiskFileTellEx    (DiskPartionData *pData,DiskFile	   hFile);

int     	DiskGetFileTime   (DiskPartionData *pData,DiskFile	   hFile,DiskUtc *pWrite,DiskUtc *pCreate,DiskUtc *pAccess);
int     	DiskSetFileTime   (DiskPartionData *pData,DiskFile	   hFile,DiskUtc  uWrite,DiskUtc  uCreate,DiskUtc  uAccess);
int     	DiskGetFileInfo   (DiskPartionData *pData,DiskFile	   hFile,DiskFindStruct *pFind);
int     	DiskSetFileSize   (DiskPartionData *pData,DiskFile	   hFile,DWORD uSize);
DiskPos    	DiskGetFileSizeEx (DiskPartionData *pData,DiskFile	   hFile);

int			DiskGetCurrentPath(DiskPartionData *pData,      DCHAR *pFilename,int iSize);
int			DiskSetCurrentPath(DiskPartionData *pData,const DCHAR *pFilename);
int 		DiskGetAttributes (DiskPartionData *pData,const DCHAR *pFilename);
int 		DiskSetAttributes (DiskPartionData *pData,const DCHAR *pFilename,int iAttr);

DiskPos		DiskFileLengthEx  (DiskPartionData *pData,const DCHAR *pFilename);
int 		DiskFileExist 	  (DiskPartionData *pData,const DCHAR *pFilename);
int			DiskMkdir	 	  (DiskPartionData *pData,const DCHAR *pFilename);
int			DiskRmdir	 	  (DiskPartionData *pData,const DCHAR *pFilename);
int			DiskDelete	 	  (DiskPartionData *pData,const DCHAR *pFilename);
int			DiskRename	 	  (DiskPartionData *pData,const DCHAR *pFilename,const DCHAR *pFilenameNew);
int			DiskFullPath  	  (DiskPartionData *pData,const DCHAR *pFilename,DCHAR *pBuffer,int iMax);

DiskFind	DiskFindFirstEx   (DiskPartionData *pData,const DCHAR *pFilename,DiskFindStruct *pFind,int *pError);
int			DiskFindNext  	  (DiskPartionData *pData,DiskFind     hHandle  ,DiskFindStruct *pFind);
int			DiskFindClose 	  (DiskPartionData *pData,DiskFind     hHandle);



#endif
