//*****************************************************************************
//*
//*
//*		FtpArchive.cpp
//*
//*
//*****************************************************************************
//
//	Emuliert ein Archive (*.zip) als FTP-Verbindung.
//
#define 	_WIN32_WINNT	0x0400

#include	<Afx.h>
#include	"FtpTools.h"
#include	"FtpArchive.h"
#include	"../StringChain.h"
#include	"../MemFunctions.h"
#include	"../FileFunctions.h"
#include	"../StringFunctions.h"
#include	"../Archive/ArchiveFile.h"

#pragma		comment(lib,"mpr.lib")


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpArchive::CFtpArchive(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive) : CFtpBase(pSession,pServer,pUserName,pPassword,dwContext,nPort,bPassive)
{

	m_pBaseUrl	  = _T("arch://");
	m_cCurrDir[0] = '/';
	m_cCurrDir[1] = 0;
	m_bCancel	  = 0;
	m_uDoneSet	  = 0;
	m_iDoneNow	  =-1;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpArchive::~CFtpArchive()
{

	if(m_pSession)Close();

}


//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpArchive::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;
DWORD		   *pVal=(DWORD*)pBuffer;
DWORD			uDiff;
DWORD			uLen;
__int64			iSize;
__int64			iPos;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	uLen = *pLen;


	switch(dwOption)
		{
	case INTERNET_OPTION_MODE:						// Verbindungsmodus

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_ARCH;

		return TRUE;

	case INTERNET_OPTION_READONLY:					// Keine Schreiberlaubnis

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 1;

		return TRUE;

	case INTERNET_OPTION_PROGRESS:					// Verlaufsanzeige

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		iPos    = pArchive->Done(&iSize);

		if(m_iDoneNow!=iPos || m_iDoneMax!=iSize)
			{
			pVal[0]	= (DWORD)iPos;
			}
		else{
			   uDiff = GetTickCount()-m_uTickSet;
			if(uDiff>250)
					pVal[0]=(DWORD)iPos;
			else	pVal[0]=m_uDoneSet;
			}

		return TRUE;

	case INTERNET_OPTION_PROGRESS2:					// Verlaufsanzeige mit Maximalwert

		if(uLen<sizeof(DWORD)*2)
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		iPos    = pArchive->Done(&iSize);

		if(m_iDoneNow!=iPos || m_iDoneMax!=iSize)
			{
			pVal[0] = (DWORD)iPos;
			pVal[1] = (DWORD)iSize;
			}
		else{
			   uDiff = GetTickCount()-m_uTickSet;
			if(uDiff>250)
					pVal[0]=(DWORD)iPos;
			else	pVal[0]=m_uDoneSet;

			pVal[1] = (DWORD)iSize;
			}

		return TRUE;

	case INTERNET_OPTION_CONNTYPE:					// Art der Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_ARCH;

		return TRUE;

	case INTERNET_OPTION_URLFORMAT:					// Format des URL-Namens

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 1;

		return TRUE;

	case INTERNET_OPTION_ERROPEN:					// Wo gab es den Fehler

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_dwErrOpen;

		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpArchive::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{

	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	switch(dwOption)
		{
	case INTERNET_OPTION_PROGRSET:					// Verlaufsanzeige einstellen

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_uTickSet = GetTickCount();
		m_uDoneSet = *(int*)pBuffer;
		m_iDoneNow = ((CArchiveFile*)m_pSession)->Done(&m_iDoneMax);

		return TRUE;

	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}
		
		m_dwPutSize = *(LPDWORD)pBuffer;		
		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpArchive::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{

	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	return NULL;
}

//*****************************************************************************
//*
//*		SetCurrentDirectory
//*
//*****************************************************************************
//	Stellt das aktuelle Verzeichnis ein.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpArchive::SetCurrentDirectory(LPCTSTR pDirName)
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;
TCHAR			cBuffer[CFB_MAX_PATH];
LPCTSTR			pName;
LPCTSTR			pBase;
int				iLen;
int				iNum;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	if(pDirName[0]=='/' || pDirName[0]=='\\')				// "/" und "//" sind Root-Verzeichnis
		{
		if(pDirName[1]==0)
			{
			m_cCurrDir[0] = '/';
			m_cCurrDir[1] = 0;
			return TRUE;
			}

		if(pDirName[2]==0 && (pDirName[1]=='/' || pDirName[1]=='\\'))
			{
			m_cCurrDir[0] = '/';
			m_cCurrDir[1] = 0;
			return TRUE;
			}
		}

	PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer)/sizeof(TCHAR),0);
	PathConvert (cBuffer,'/');
	DelBackSlash(cBuffer);

		pBase = cBuffer;
	if(*pBase=='/')pBase++;
	iLen = _tcslen(pBase);

	m_sSemaphor.Lock();

	for(iNum=0;iNum<pArchive->Count();iNum++)			// Suche das Verzeichnis
		{
		if(m_bCancel)break;

		if(pArchive->IsDel(iNum))continue;

		pName = pArchive->Name(iNum);

		if(*pName=='/')pName++;

		if(_tcsnicmp(pName,pBase,iLen))continue;

		if(pName[iLen]=='/')break;
		if(pName[iLen]==0)
			{
			if(pArchive->IsDir(iNum))break;
			SetLastError(ERROR_PATH_NOT_FOUND);
			m_sSemaphor.Unlock();
			return FALSE;
			}
		}

	strlcpy(m_cCurrDir,cBuffer,sizeof(m_cCurrDir)/sizeof(TCHAR));

	m_sSemaphor.Unlock();



return TRUE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis.
//	pDirName		: Puffer fr den Namen
//	pLen			: Gre des Puffers
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpArchive::GetCurrentDirectory(LPTSTR pDirName,LPDWORD pLen) const
{


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	*pLen = strlcpy(pDirName,m_cCurrDir,*pLen);



return TRUE;
}


//*****************************************************************************
//*
//*		RemoveDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des zu lschenden Verzeichnisses
//	Ergibt TRUE wenn das Verzeichnis gelscht wurde.
BOOL CFtpArchive::RemoveDirectory(LPCTSTR pDirName)
{

	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}

//*****************************************************************************
//*
//*		CreateDirectory
//*
//*****************************************************************************
//	Ein Verzeichnis erstellen.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpArchive::CreateDirectory(LPCTSTR pDirName)
{

	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	SetLastError(ERROR_NOT_SUPPORTED);



return FALSE;
}

//*****************************************************************************
//*
//*		Remove
//*
//*****************************************************************************
//	Lscht eine Datei..
//	pFileName		: Ist der Name der Datei
//	Ergibt TRUE wenn die Datei gelscht wurde.
BOOL CFtpArchive::Remove(LPCTSTR pFileName)
{


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	SetLastError(ERROR_NOT_SUPPORTED);



return FALSE;
}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	Verschiebt eine Datei..
//	pExisting		: Ist der Name der Datei.
//	pNewName		: Ist der Name der Zieldatei.
//	Ergibt TRUE wenn die Datei vershoben wurde.
BOOL CFtpArchive::Rename(LPCTSTR pExisting,LPCTSTR pNewName)
{


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
void CFtpArchive::Close()
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;


	if(!m_pSession)return;

	pArchive->Close();
	delete pArchive;

	m_pSession=0;

}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertgt eine Datei zum FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	pRemoteFile		: Name der Datei am FTP-Server.
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpArchive::PutFile(LPCTSTR pLocalFile,LPCTSTR pRemoteFile,DWORD dwFlags,DWORD dwContext)
{


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpArchive::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei vom FTP-Server.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	bFailIfExists	: Die lokale Datei nicht berschreiben
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpArchive::GetFile(LPCTSTR pRemoteFile,LPCTSTR pLocalFile,BOOL bFailIfExists,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;
TCHAR			cBuffer[CFB_MAX_PATH];
int				iRet;
int				iNum;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(bFailIfExists && ExistFile(pLocalFile))
		{
		SetLastError(ERROR_FILE_EXISTS);
		m_dwErrOpen = 1;
		return FALSE;
		}

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer)/sizeof(TCHAR),0);

	m_sSemaphor.Lock();

	iNum = pArchive->Find(cBuffer);						// Existiert die Datei

	if(iNum<0 || pArchive->IsDir(iNum) || pArchive->IsDel(iNum))
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		m_sSemaphor.Unlock();
		return FALSE;
		}


	m_bCancel		= 0;
	m_bAbortAllowed	= 1;

	iRet = pArchive->Extract(iNum,pLocalFile);

	switch(iRet)
		{
	case CAF_ERROR_OK:		iRet=TRUE;	break;
	case CAF_ERROR_FCREATE: iRet=FALSE;	SetLastError(ERROR_ACCESS_DENIED);	break;
	default:				iRet=FALSE;	SetLastError(ERROR_INVALID_ACCESS);	break;
		}

	m_bAbortAllowed	= 0;

	m_sSemaphor.Unlock();



return iRet;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpArchive::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;
TCHAR			cBuffer[CFB_MAX_PATH];
int				iRet;
int				iNum;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer)/sizeof(TCHAR),0);

	m_sSemaphor.Lock();

	iNum = pArchive->Find(cBuffer);						// Existiert die Datei

	if(iNum<0 || pArchive->IsDir(iNum) || pArchive->IsDel(iNum))
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		m_sSemaphor.Unlock();
		return FALSE;
		}


	m_bCancel		= 0;
	m_bAbortAllowed	= 1;

	iRet = pArchive->ExtractProc(iNum,pParam,pProc);

	switch(iRet)
		{
	case CAF_ERROR_OK:		iRet=TRUE;	break;
	case CAF_ERROR_FCREATE: iRet=FALSE;	SetLastError(ERROR_ACCESS_DENIED);	break;
	default:				iRet=FALSE;	SetLastError(ERROR_INVALID_ACCESS);	break;
		}

	m_bAbortAllowed	= 0;

	m_sSemaphor.Unlock();



return iRet;
}

//*****************************************************************************
//*
//*		List
//*
//*****************************************************************************
//	Liste alle Dateien in einem Verzeichnis via Callback auf.
//	pRemotePath		: Ist der Name des Verzeichnisses.
//	pParam			: Parameter fr den Callback.
//	pProc			: Zeiger auf den Callback.
//	pFilter			: Filter der benutzt werden kann.
//	Ergibt wenn TRUE alles ok war, bzw. FALSE bei einem Fehler.
BOOL CFtpArchive::List(LPCTSTR pRemotePath,void *pParam,FtpFindCallback pProc,LPCTSTR pFilter)
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;
TCHAR			cBuffer[CFB_MAX_PATH];
CStringChain	sFolders;
LPCTSTR			pName;
LPCTSTR			pBase;
WIN32_FIND_DATA	sData;
time_t			iTimeC;
time_t			iTimeW;
time_t			iTimeA;
__int64			iSize;
int				iAttr;
int				iEnd;
int				iLen;
int				iNum;
int				iOff;



	if(!m_pSession)
		{
		pProc(pParam,NULL);
		return FALSE;
		}

	if(!*pRemotePath)
		{
		iLen=strlcpy(cBuffer,m_cCurrDir,sizeof(cBuffer)/sizeof(TCHAR));
		}
	else{
		iLen=PathFullMake(cBuffer,m_cCurrDir,pRemotePath,sizeof(cBuffer)/sizeof(TCHAR),0);
		}


	m_sSemaphor.Lock();

	iLen = DelBackSlash(cBuffer);
	PathConvert(cBuffer,'/');

	memset(&sData,0,sizeof(sData));
	cBuffer[iLen] = '/';
	iLen++;

		pBase = cBuffer;
	if(*pBase=='/'){pBase++;iLen--;}


	if(iLen==0)
		{
		iNum = 0;
		}
	else{
		for(iNum=0;iNum<pArchive->Count();iNum++)			// Suche das Verzeichnis
			{
			if(m_bCancel)break;

			if(pArchive->IsDel(iNum))continue;

			pName = pArchive->Name(iNum,iEnd);

			if(*pName=='/')
				{
				pName++;
				iEnd--;
				}

			if(iEnd<iLen-1)
				{
				continue;
				}

			if(pName[iLen-1]!='/')
				{
				continue;
				}

			if(_tcsnicmp(pName,pBase,iLen-1))
				{
				continue;
				}

			if(pName[iLen-1]=='/')
				{
				break;
				}
			}

		if(iNum>=pArchive->Count())							// Verzeichnis nicht gefunden
			{
			SetLastError(ERROR_PATH_NOT_FOUND);
			pProc(pParam,NULL);
			m_sSemaphor.Unlock();
			return FALSE;
			}
		}


	for(;iNum<pArchive->Count();iNum++)						// Alle Dateien suchen
		{
		if(m_bCancel)break;

		pName = pArchive->Name(iNum,iEnd);

		if(pArchive->IsDel(iNum))continue;

		if(*pName=='/')
			{
			pName++;
			iEnd--;
			}

		if(iEnd<iLen-1)
			{
			break;
			}

		if(iLen>0 && pName[iLen-1]!='/')
			{
			break;
			}

		if(_tcsnicmp(pName,pBase,iLen))
			{
			break;
			}

		iOff = strcwlen(pName+iLen,0x00300000);

		if(pName[iLen+iOff] || pArchive->IsDir(iNum))		// Verzeichnisse einfgen
			{
			if(sFolders.FindFast(iOff,pName+iLen,0,-1,1)>=0)
				{
				continue;
				}

			sFolders.Add(iOff,pName+iLen,CSC_ABCG);

			iTimeW				   = 946684800;
			iTimeC				   = 946684800;
			iTimeA				   = 946684800;
			sData.dwFileAttributes = FILE_ATTRIBUTE_DIRECTORY;
			sData.nFileSizeLow	   = 0;
			}
		else{												// Dateien einfgen
			iSize				   = pArchive->Len(iNum);
			iTimeW				   = pArchive->Time(iNum,CAF_TIME_W);
			iTimeC				   = pArchive->Time(iNum,CAF_TIME_C);
			iTimeA				   = pArchive->Time(iNum,CAF_TIME_A);
			sData.nFileSizeHigh    = (unsigned)(iSize>>32);
			sData.nFileSizeLow	   = (unsigned)iSize;
			sData.dwFileAttributes = (sData.nFileSizeHigh&0x3F)<<10;

			if(pArchive->IsReadOnly(iNum))
				{
				sData.dwFileAttributes |= FILE_ATTRIBUTE_READONLY;
				}

			if(iTimeC<0)iTimeC=iTimeW;
			if(iTimeA<0)iTimeA=iTimeW;
			}

		iAttr = pArchive->Attribute(iNum);

		if(!(iAttr&0x80000000))								// UNIX Attribute setzen
			{
			sData.dwFileAttributes |= iAttr<<16;
			}

		if(pFilter && memxcmp(pFilter,pName+iLen,iOff,0x03))// Filter prfen
			{
			continue;
			}

		strlcpy(sData.cFileName,pName+iLen,sizeof(sData.cFileName)/sizeof(TCHAR),iOff);

		UtcTimeToFileTime(iTimeW,&sData.ftLastWriteTime );
		UtcTimeToFileTime(iTimeC,&sData.ftCreationTime  );
		UtcTimeToFileTime(iTimeA,&sData.ftLastAccessTime);

		pProc(pParam,&sData);
		}



	pProc(pParam,NULL);

	m_sSemaphor.Unlock();



return (m_bCancel)? FALSE:TRUE;
}

//*****************************************************************************
//*
//*		SetAttributes
//*
//*****************************************************************************
//	Einstellen der Dateiattribute
//	pFileName		: Ist der Name der Datei.
//	dwAttr			: Sind die neuen Attribute.
//					  In den Bits 16..31 knnen Unix-Attribute verpackt werden.
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpArchive::SetAttributes(LPCTSTR pFileName,DWORD dwAttr)
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}

//*****************************************************************************
//*
//*		SetModifyTime
//*
//*****************************************************************************
//	Einstellen der nderungszeit einer Datei.
//	pFileName		: Ist der Name der Datei.
//	sFileTime		: Ist die neue nderungszeit
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpArchive::SetModifyTime(LPCTSTR pFileName,FILETIME sFileTime)
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}

//*****************************************************************************
//*
//*		DoAbort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpArchive::DoAbort()
{
CArchiveFile   *pArchive=(CArchiveFile*)m_pSession;


	if(!m_pSession)return;

	pArchive->Break();

	m_bCancel		= TRUE;
	m_bAbortAllowed = 0;


}


