//*****************************************************************************
//*
//*
//*		FtpAxComm.cpp
//*
//*
//*****************************************************************************
//
//	Ist eine FTP-Verbindung ber die AX-Comm-API.
//
#include	<Afx.h>
#include	<stddef.h>
#include	"FtpTools.h"
#include	"FtpAxComm.h"
#include	"../MemFunctions.h"
#include	"../FileFunctions.h"
#include	"../StringFunctions.h"
//#include	"D:\\azechner_automationx_plc5\\src\\applications\\CommLib\\CommInterface.h"



//*****************************************************************************
//*
//*		Error Codes
//*
//*****************************************************************************

#define 	ERR_NONE					0x0000			// Keinfehler vorhanden
#define 	ERR_FAIL					0x0001			// Ein allgemeiner Fehler ist passiert
#define 	ERR_PARAM					0x0002			// Ein falscher Parameter wurde bergeben
#define 	ERR_CANT_LOAD_DLL			0x0003			// Die DLL konnte nicht geladen werden
#define 	ERR_NOT_ALL_PROCS_FOUND		0x0004			// In der DLL wurden nicht alle Funktionen gefunden
#define 	ERR_NO_MEMORY				0x0005			// Zuwenig Speicher
#define 	ERR_UNKNOWN_INTERFACE		0x0006			// Der Name des Interfaces wurde nicht gefunden
#define 	ERR_UNKNOWN_COMMAND			0x0007			// Es wurde ein unbekanntes Kommando empfangen
#define 	ERR_ALREADY_RUNNING			0x0008			// Das Interface luft breits
#define 	ERR_ALREADY_STOPED			0x0009			// Das Interface wurde bereits angehalten
#define 	ERR_NOT_STOPED				0x000A			// Das Interface wurde noch nicht angehalten
#define 	ERR_NO_CONNECTION			0x000B			// Es ist gerade keine Verbindung offen
#define 	ERR_START_THREAD			0x000C			// Ein Thread konnte nicht gestartet werden
#define 	ERR_BUFFER_FULL				0x000D			// Der Sendepuffer ist zu voll
#define 	ERR_NO_FREE_SLOTS			0x000E			// Es gibt keine freien Slots mehr
#define 	ERR_ROUTING_ERROR			0x000F			// Das Paket konnte nicht weitergeleitet werden
#define 	ERR_WRONG_VERSION			0x0010			// Die Open-Funktion hat die falsche Versionsnummer
#define 	ERR_WRONG_FORMAT			0x0011			// Das Paket enthielt einen Formatfehler
#define 	ERR_NO_DEVICE_CONN			0x0012			// Es sind keine Device-Verbindungen erlaubt
#define 	ERR_UNKNOWN_BROKER			0x0013			// Der Broker ist nicht bekannt
#define 	ERR_WRONG_URL				0x0014			// Es wurde eine falsche URL angeben
#define 	ERR_WRONG_SESSION_ID		0x0015			// Die Session ID hat eine falsche Lnge
#define 	ERR_OUT_OF_HANDELS			0x0016			// Es wurden zu viele Sessions geffnet
#define 	ERR_TIMEOUT					0x0017			// Von der Gegenstelle ist keine Antwort gekommen
#define 	ERR_WRONG_USERID			0x0018			// Die benutzte User-ID ist flasch
#define 	ERR_WORNG_RESPONSE			0x0019			// Eine falsche Rckmeldung wurde empfangen
#define 	ERR_WORNG_HANDEL			0x001A			// Es wurde ein falsches Verbindungs-Handle benutzt
#define 	ERR_NO_SESSION				0x001B			// Es wurde keine Session bei dieser Verbindung geffnet
#define 	ERR_SESSION_NOT_FOUND		0x001C			// Es wurde keine Session unter diesem Namen am Device gefunden
#define 	ERR_SESSION_WRONG_VER		0x001D			// Die Session am Device hat eine andere Versionsnummer
#define 	ERR_SESSION_WRONG_PRM		0x001E			// Die Session am Device hat andere Einstellungen
#define 	ERR_CANT_CREATE_CFG			0x001F			// Es konnte keine Konfigurationsdatei erstelt werden
#define 	ERR_TOO_MUCH_SESSIONS		0x0020			// Es sind zu viele Sessions am Device angemeldet
#define 	ERR_FORMAT					0x0021			// Das Paket hat ein falsches Format
#define 	ERR_SESSION_FILE_ERR		0x0022			// Es konnte eine Datei zur Session nicht geffnet werden
#define 	ERR_FILTERD					0x0023			// Die Nachricht wurde vom Filter bernommen
#define 	ERR_WRONG_RANGE				0x0024			// Es wurde ein flalscher Range angegeben (|<von>-<bis>|)
#define		ERR_EXISTS_AS_FILE			0x0025			// Der Pfad ist eine Datei
#define		ERR_NOT_ONLINE				0x0026			// Die Verbindung ist nicht online
#define 	ERR_LOCAL_FILE				0x0027			// Die lokale Datei konnten nicht geffnet werden
#define 	ERR_LOCAL_READ				0x0028			// Von der lokalen Datei konnten nicht gelesen werden
#define 	ERR_WRITE_FILE				0x0029			// Es konnte nicht in die Datei geschrieben werden
#define 	ERR_UNKNOWN_INSTANCE		0x002A			// Die Instanz konnte nicht gefunden werden
#define 	ERR_CONN_LOCKED				0x002B			// Die Verbindung ist gesperrt
#define 	ERR_NOT_SUPPORTED			0x002C			// Die Funktion wird nicht untersttzt
#define 	ERR_MODUL_NOT_LOADED		0x002D			// Ein Modul konnte nicht geladen werden
#define 	ERR_INTERFACE_CLOSED		0x002E			// Das Interface wurde gelscht
#define 	ERR_DOUBLE_SESSION_ID		0x002F			// Der Name fr die Session ist bereits benutzt
#define 	ERR_OUT_OFFSLOTS			0x0030			// Keine freien Slots fr Verbindungen

#define 	ERR_WRONG_FILE_PARAM		0x0060			// Falscher Dateiparameter
#define 	ERR_INVALID_FILENAME		0x0061			// Falscher Dateiname
#define 	ERR_FILE_NOT_FOUND			0x0062			// Die Datei wurde nicht gefunden
#define 	ERR_ACCESS_DENIED			0x0063			// Kein Zugriff auf die Datei
#define 	ERR_NO_HANDLES				0x0064			// Keine freien Handels mer
#define 	ERR_UNKNOWN					0x0065			// Unbekander Dateifehler
#define 	ERR_PATH_NOT_FOUND			0x0066			// Dateipfad wurde nicht geunden
#define 	ERR_DISKFULL				0x0067			// Das Laufwerk ist voll
#define		ERR_ALREADY_EXISTS			0x0068			// Die Datei existiert bereits


#define     CI_FILE_READ        		0x0001          // Leserechte fr die Datei
#define     CI_FILE_WRITE       		0x0002          // Schreibrechte fr die Datei
#define     CI_FILE_CREATE      		0x0004          // Die Datei neu erstellen wenn sie nicht existiert
#define     CI_FILE_TRUNC       		0x0008          // Die Dateigre auf 0 setzen
#define     CI_FILE_DIR         		0x0010          // Die Datei ist eine Verzeichnis
#define     CI_FILE_HIDDEN      		0x0020          // Die Datei ist versteckt
#define     CI_FILE_ARCHIVE     		0x0040          // Archiv-Flag fr Dateien (wird beim Schreiben gesetzt)
#define     CI_FILE_SYSTEM      		0x0080          // System-Flag fr Dateien
#define     CI_FILE_VOLUME      		0x0100          // Volume-Flag fr Disknamen
#define     CI_FILE_EXEC	    		0x0200          // Ausfhrungsrechte fr Dateien
#define     CI_FILE_SHARE_RD    		0x0400          // Es knnen mehr als eine Datei gleichzeitig Lesen
#define     CI_FILE_SHARE_WR    		0x0800          // Es knnen mehr als eine Datei gleichzeitig Schreiben
#define     CI_FILE_HIGHMASK        	0xFC00          // Maske fr die 64-Bit-Dateigre
#define     CI_FILE_HIGHSHIFT       	10              // Shift fr die 64-Bit-Dateigre

#define 	CI_QOS_CURRENT				0x0000			// Nur den aktuellen Wert einer Variable holen
#define 	CI_QOS_TREND				0x0010			// Jede nderung einer Variable bertragen
#define 	CI_QOS_ALWAYS				0x0020			// Auch am Standby die Daten bertragen (extra Flag)
#define 	CI_QOS_DELAY				0x0040			// Kleine Antwort-Pakete packen und zeitverzgert senden

#define 	CI_INF_AXROOTPATH 			0x0000			// Den AX-Root-Pfad abfragen
#define 	CI_INF_AXBINDIR 			0x0001			// Den Pfad vom AX Binary abfragen
#define 	CI_INF_AXPROJECTDIR 		0x0002			// Den Pfad vom AX Project abfragen
#define 	CI_INF_HOSTNAME 			0x0003			// Den Hostnamen abfragen
#define 	CI_INF_AXREMOTEHOST 		0x0004			// Den Hostnamen vom Remote-Rechner abfragen
#define 	CI_INF_HOSTSTANDALONE 		0x0005			// Ist es ein Standalone-System
#define 	CI_INF_CFGASMASTER 			0x0006			// Ist das Gert als Master konfiguriert
#define 	CI_INF_ISMASTER 			0x0007			// Ist das Gert gerade Master
#define 	CI_INF_CYCLETIME 			0x0008			// Zykluszeit abfragen
#define 	CI_INF_CYCLECOUNTER 		0x0009			// Zykluszhler abfragen
#define 	CI_INF_SYSTEMTIME 			0x000A			// Systemzeit abfragen
#define 	CI_INF_SPSID 				0x000B			// SPS-Id abfragen
#define 	CI_INF_TASKLIST				0x000C			// Task-List abfragen
#define 	CI_INF_KILLTASK				0x000D			// Einen Prozess beenden
#define 	CI_INF_WARMSTARTIMAGE		0x000E			// Einen Warmstart machen
#define 	CI_INF_COLDSTARTIMAGE		0x000F			// Einen Kaltstart machen
#define 	CI_INF_IMAGEINIT			0x0010			// Namen des aktuellen Images abfragen
#define 	CI_INF_CURRENTCODEVER		0x0011			// Aktuelle Code-Version abfragen
#define 	CI_INF_CURRENTVERSION		0x0012			// Aktuelle AX-Version abfragen
#define 	CI_INF_QUERYVARIABLE		0x0013			// Eine AX-Variable abfragen
#define 	CI_INF_VARIABLETYPE			0x0014			// Den Typ einer AX-Variable abfragen
#define 	CI_INF_GETOSVERSION			0x0015			// Die Version des Betreibsystems abfragen
#define 	CI_INF_GETFORCEDVARS		0x0016			// Variabeln die Gefoced sind abfragen
#define 	CI_INF_ROLLBACKIMAGE		0x0017			// Das alte Image wieder herstellen
#define 	CI_INF_GENERATEIMAGE		0x0018			// Ein Image aus XML-Dateien (in *.zip) erstellen
#define 	CI_INF_GETIMAGESTATE		0x0019			// Verlauf der Image-Erstellung abfragen
#define 	CI_INF_SESSIONINFO			0x001A			// Infos zu Sessions abfragen
#define 	CI_INF_REMOVESESSION		0x001B			// Eine Session lschen
#define 	CI_INF_GETIMAGENAME			0x001C			// Pfad der erzeugen Image-Datei holen




#define 	CI_API
typedef		DWORD	CiDW;
typedef		DWORD	CiUtc;
typedef		INT		CiError;
typedef		LPVOID	CiConnHandle;

typedef	void (_cdecl *CiCallback)(void *pUserParam,CiConnHandle hConn,unsigned uGroup,unsigned uAckId,unsigned char *pData,unsigned uSize);


typedef	struct
	{
	WIN32_FIND_DATA	sFind;
	LPVOID			pNext;
	}LinkEntry;

typedef	struct
	{
	FtpFindCallback	pProc;
	LPVOID			pParam;
	BOOL			bBreak;
	BOOL			bUseUtf8;
	LinkEntry	   *pLinkLast;
	LinkEntry	   *pLinkFirst;
	}FindCbData;


typedef	struct
	{
	CiDW	dwAttr;										// Dateiattribute und Dateigre (Bit 32..37 siehe CI_FILE_???)
	CiDW	dwSize;										// Dateigre      (low 32 bits)
	CiUtc 	dwTimeCreate;								// Erstellungszeit (Sekunden seit 1.Jan.1970)
	CiUtc	dwTimeAccess;								// Letzter Zugriff (Sekunden seit 1.Jan.1970)
	CiUtc	dwTimeWrite;								// Letzte nderung (Sekunden seit 1.Jan.1970)
	char	cName[256];									// Dateiname
	}CiFindStruct;										
														
														
typedef struct											
	{													
	__int64			iNow;								// Aktueller Offset bei einer Dateioperation
	__int64			iMax;								// Maximale  Offset bei einer Dateioperation (Dateigre)
	}CiProgressData;									
														
typedef struct											
	{													
	unsigned char 	ucQualityOfService;					// Ist der QOS-Level der Verbindung (CI_QOS_???, Bit 0..1=Priority/3=high)
	unsigned char 	aRouting[3];						// Sind die Nummern der Broker fr das Routing (0=Ende)
	unsigned short 	usLifetick;							// Nach dieser Zeit ohne Lifetick wird eine Verbindungsunterbrechung gemeldet (in Sekunden, 15..120s)
	unsigned short 	usCycleTime;						// Zykluszeit in Millisekunden fr CI_QOS_CURRENT
	unsigned		uSessionTimeout;					// Wann soll die Session verworfen werden (in Sekunden)
	unsigned		uAsynchTimeout;						// In welcher Zeit muss CiGetVars/CiSetVars antworten (in Millisekunden)
	unsigned char 	ucGetPriority;						// Prioritt fr CiGetVars(...)
	unsigned char 	ucSetPriority;						// Prioritt fr CiSetVars(...)
	unsigned short	usSendDelay;						// Zeit in Millisekunden um die kleine Antworten verzgert versand werden (nur mit CI_QOS_DELAY)
	CiCallback		pCallback;							// Callback-Funktion fr Antworten
	void		   *pCbParam;							// User-Parameter fr den Callback
	}CiConnParameter;									
														

typedef void (_cdecl *CiListCb  )(void *pUserParam,CiFindStruct *pFind);
typedef int  (_cdecl *CiReadCb  )(void *pUserParam,      void *pBuffer,int iSize);
typedef int  (_cdecl *CiWriteCb )(void *pUserParam,const void *pBuffer,int iSize);

CI_API	void		(_cdecl *CiInit        )();
CI_API	void		(_cdecl *CiExit        )();
CI_API	int 		(_cdecl *CiGetVersion  )();
CI_API	int 		(_cdecl *CiIsOnline    )(CiConnHandle  hConn);
CI_API	int			(_cdecl *CiErrorToText )(CiError iError,char *pBuffer,int iSize);
CI_API	CiError		(_cdecl *CiDisconnect  )(CiConnHandle  hConn);
CI_API	CiError		(_cdecl *CiConnect     )(CiConnHandle *pHandle,const char *pServerUrl,CiConnParameter *pParams,const char *pSessionId,unsigned uSessionVer,unsigned uUserId);
CI_API	CiError		(_cdecl *CiConnectWait )(CiConnHandle *pHandle,const char *pServerUrl,CiConnParameter *pParams,const char *pSessionId,unsigned uSessionVer);

CI_API	CiError 	(_cdecl *CiListFiles   )(CiConnHandle  hConn,const char *pPath,CiListCb pCallback,void *pUserParam);
CI_API	CiError 	(_cdecl *CiCreateFolder)(CiConnHandle  hConn,const char *pPath,int iRecursive);
CI_API	CiError 	(_cdecl *CiDeleteFolder)(CiConnHandle  hConn,const char *pPath);
CI_API	CiError 	(_cdecl *CiDeleteFile  )(CiConnHandle  hConn,const char *pPath);
CI_API	CiError 	(_cdecl *CiRenameFile  )(CiConnHandle  hConn,const char *pPath,const char *pNewFileName);
CI_API	CiError 	(_cdecl *CiUploadFile  )(CiConnHandle  hConn,const char *pPath,const char *pLocalFile,unsigned uFlags                 ,CiProgressData *pProgress=0);
CI_API	CiError 	(_cdecl *CiUploadProc  )(CiConnHandle  hConn,const char *pPath,CiReadCb    pCallback ,unsigned uFlags,void *pUserParam,CiProgressData *pProgress=0,int iFileSize=-1);
CI_API	CiError 	(_cdecl *CiDownloadFile)(CiConnHandle  hConn,const char *pPath,const char *pLocalFile                                 ,CiProgressData *pProgress=0);
CI_API	CiError 	(_cdecl *CiDownloadProc)(CiConnHandle  hConn,const char *pPath,CiWriteCb   pCallback                 ,void *pUserParam,CiProgressData *pProgress=0);
CI_API	CiError 	(_cdecl *CiGetFileAttr )(CiConnHandle  hConn,const char *pPath,unsigned *pAttr);
CI_API	CiError 	(_cdecl *CiSetFileAttr )(CiConnHandle  hConn,const char *pPath,unsigned  uAttr);
CI_API	CiError		(_cdecl *CiGetInfos    )(CiConnHandle  hConn,unsigned uInfoId,char *pBuffer,int *pSize,const char *pOutput=0);


static	HMODULE		hCommDll = 0;
static	BOOL		bDllDone = 0;

//*****************************************************************************
//*
//*		CiCommDllLoad
//*
//*****************************************************************************
//	Lade die "CommClient32.dll" samt allen Funktionspointeren.
//	pPath		: Ist der Pfad in dem die DL liegt (oder 0)
//	Ergibt 1 wenn ok oder 0 bei einem Fehler.
int	CiCommDllLoad(LPCTSTR pPath)
{
#define 	LoadProc(a)			*(FARPROC*)(&a)=GetProcAddress(hCommDll,#a); if(!a)return 0;
char		cPath[MAX_PATH];
int			iLen;



	if(bDllDone)return 1;

	if(!hCommDll)
		{
		if(pPath && *pPath)								// DLL aus eine bestimmten Pfad laden
			{
			iLen = strawcpy(cPath,pPath,MAX_PATH);
			if(iLen>0 && cPath[iLen-1]=='\\')iLen--;
			strlcat(cPath+iLen,"\\CommClient32.dll",MAX_PATH-iLen);
			hCommDll = LoadLibraryA(cPath);
			}

		if(!hCommDll)
			{
				hCommDll = LoadLibraryA("CommClient32.dll");
			if(!hCommDll)
				{
					hCommDll = LoadLibraryA("D:\\azechner_automationx_plc5\\binDebug\\CommClient32.dll");
				if(!hCommDll)
					{
					return 0;
					}
				}
			}
		}


	LoadProc(CiInit        );							// Funktionen Laden
	LoadProc(CiExit        );
	LoadProc(CiGetVersion  );
	LoadProc(CiIsOnline    );
	LoadProc(CiErrorToText );
	LoadProc(CiDisconnect  );
	LoadProc(CiConnect     );
	LoadProc(CiConnectWait );
	LoadProc(CiListFiles   );
	LoadProc(CiCreateFolder);
	LoadProc(CiDeleteFolder);
	LoadProc(CiDeleteFile  );
	LoadProc(CiRenameFile  );
	LoadProc(CiUploadFile  );
	LoadProc(CiUploadProc  );
	LoadProc(CiDownloadFile);
	LoadProc(CiDownloadProc);
	LoadProc(CiGetFileAttr );
	LoadProc(CiSetFileAttr );
	LoadProc(CiGetInfos    );


	CiInit();


	bDllDone = 1;



return 1;
}

//*****************************************************************************
//*
//*		CiCommDllUnload
//*
//*****************************************************************************
//	Entlade die "CommClient32.dll" ergibt 1 wenn ok oder 0 bei einem Fehler.
int	CiCommDllUnload()
{

	if(!bDllDone)return 0;

	CiExit();

	bDllDone = 0;
	CiInit   = 0;

	FreeLibrary(hCommDll);

	hCommDll = NULL;


return 1;
}

//*****************************************************************************
//*
//*		CiMyCallback
//*
//*****************************************************************************
static void CiMyCallback(void *pUserParam,void *hConn,unsigned uGroup,unsigned uAckId,unsigned char *pData,unsigned uSize)
{

}


//*****************************************************************************
//*
//*		CiDoConnect
//*
//*****************************************************************************
int CiDoConnect(void **pSession,const char *pServer,int nPort,const char *pUser,const char *pPassWd)
{
char				cServer[256];
CiConnParameter		sParam;
CiConnHandle		hConn;
unsigned			uErr;


	if(!bDllDone)
		{
		SetLastError(ERROR_INVALID_DLL);
		*pSession = 0;
		return 0;
		}


	if(!nPort)nPort=9200;
	sprintf(cServer,"%s:%u",pServer,nPort);

	memset(&sParam,0,sizeof(sParam));

	sParam.ucQualityOfService	= CI_QOS_ALWAYS;
	sParam.usLifetick			= 15;
	sParam.pCallback			= CiMyCallback;

	uErr = CiConnectWait(&hConn,cServer,&sParam,"",0);

	if(uErr!=ERR_NONE)
		{
		switch(uErr)
			{
		case ERR_OUT_OF_HANDELS:	SetLastError(ERROR_NO_MORE_USER_HANDLES);	break;
		case ERR_PARAM:				SetLastError(ERROR_INVALID_PARAMETER);		break;
		case ERR_WRONG_URL:			SetLastError(ERROR_RESOURCE_NAME_NOT_FOUND);break;
		case ERR_OUT_OFFSLOTS:		SetLastError(ERROR_NO_MORE_USER_HANDLES);	break;
		case ERR_NO_MEMORY:			SetLastError(ERROR_OUTOFMEMORY);			break;
		default:					SetLastError(ERROR_TIMEOUT);				break;
			}

		return 0;
		}

	*pSession = hConn;



return 1;
}


//*****************************************************************************
//*
//*		CiToFind
//*
//*****************************************************************************
static void _cdecl CiToFind(void *pParam,CiFindStruct *pEntry)
{
FindCbData		   *pData=(FindCbData*)pParam;
LinkEntry		   *pLink;
WIN32_FIND_DATA		sFind;
int					iLen;


	if(!pEntry || pData->bBreak)
		{
		return;
		}

    UtcTimeToFileTime(pEntry->dwTimeCreate,&sFind.ftCreationTime  );
	UtcTimeToFileTime(pEntry->dwTimeAccess,&sFind.ftLastAccessTime);
    UtcTimeToFileTime(pEntry->dwTimeWrite ,&sFind.ftLastWriteTime );

    sFind.dwReserved0			= 0;
    sFind.dwReserved1			= 0;
	sFind.cAlternateFileName[0]	= 0;
    sFind.nFileSizeHigh			=(pEntry->dwAttr&CI_FILE_HIGHMASK)>>CI_FILE_HIGHSHIFT;
    sFind.nFileSizeLow			= pEntry->dwSize;
	sFind.dwFileAttributes		= pEntry->dwAttr&(0xFFFF0000|CI_FILE_HIGHMASK);

	if(!(pEntry->dwAttr&(CI_FILE_READ|CI_FILE_WRITE)))sFind.dwFileAttributes|=FILE_ATTRIBUTE_READONLY;
	if(  pEntry->dwAttr& CI_FILE_DIR    )sFind.dwFileAttributes|=FILE_ATTRIBUTE_DIRECTORY;
	if(  pEntry->dwAttr& CI_FILE_HIDDEN )sFind.dwFileAttributes|=FILE_ATTRIBUTE_HIDDEN;
	if(  pEntry->dwAttr& CI_FILE_SYSTEM )sFind.dwFileAttributes|=FILE_ATTRIBUTE_SYSTEM;
	if(  pEntry->dwAttr& CI_FILE_ARCHIVE)sFind.dwFileAttributes|=FILE_ATTRIBUTE_ARCHIVE;


	if(pData->bUseUtf8)
			iLen=strfromutf8(sFind.cFileName,pEntry->cName,sizeof(sFind.cFileName)/sizeof(sFind.cFileName[0]));
	else	iLen=strawcpy   (sFind.cFileName,pEntry->cName,sizeof(sFind.cFileName)/sizeof(sFind.cFileName[0]));


	if((pEntry->dwAttr&0xF0000000)==0xA0000000)		// Ist es ein Link
		{
		pLink			= new LinkEntry;
		pLink->pNext	= 0;

		memcpy(&pLink->sFind,&sFind,offsetof(WIN32_FIND_DATA,cFileName)+(iLen+1)*sizeof(TCHAR));

		if(pData->pLinkLast)
			{
			pData->pLinkLast->pNext = pLink;
			}
		else{
			pData->pLinkFirst = pLink;
			}

		pData->pLinkLast = pLink;

		return;
		}

	if(!pData->pProc(pData->pParam,&sFind))
		{
		pData->bBreak = 1;
		}


}

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpAxComm::CFtpAxComm(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive) : CFtpBase(pSession,pServer,pUserName,pPassword,dwContext,nPort,bPassive)
{

	m_pBaseUrl	  = _T("axcomm://");
	m_uTimeout	  = 10000;
	m_bReconnect  = FALSE;
	m_bConnDone	  = FALSE;
	m_bUseUtf8	  = FALSE;
	m_cCurrDir[0] = 0;
	m_aProgess[0] = 0;
	m_aProgess[1] = 0;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpAxComm::~CFtpAxComm()
{

	if(m_pSession)Close();

}



//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpAxComm::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{
DWORD	   *pVal=(DWORD*)pBuffer;
DWORD		uLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	uLen = *pLen;


	switch(dwOption)
		{
	case INTERNET_OPTION_MODE:						// Verbindungsmodus

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_AXCOMM;

		return TRUE;

	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:		// Timeout
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLen<sizeof(m_uTimeout))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		*pVal = m_uTimeout;
		*pLen = sizeof(m_uTimeout);

		return TRUE;

	case INTERNET_OPTION_PROGRESS:					// Verlaufsanzeige

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = (DWORD)m_aProgess[0];

		return TRUE;

	case INTERNET_OPTION_PROGRESS2:					// Verlaufsanzeige mit Maximalwert

		if(uLen<sizeof(DWORD)*2)
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = (DWORD)m_aProgess[0];
		pVal[1] = (DWORD)m_aProgess[1];

		return TRUE;

	case INTERNET_OPTION_UTF8:						// UTF8 Konvertierung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_bUseUtf8;

		return TRUE;

	case INTERNET_OPTION_CONNTYPE:					// Art der Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_AXCOMM;

		return TRUE;

	case INTERNET_OPTION_URLFORMAT:					// Format des URL-Namens

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 0;

		return TRUE;

	case INTERNET_OPTION_ERROPEN:					// Wo gab es den Fehler

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_dwErrOpen;

		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpAxComm::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{
int		iCount;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	switch(dwOption)
		{
	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLength<sizeof(m_uTimeout))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_uTimeout = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_PROGRSET:					// Verlaufsanzeige setzen

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_aProgess[0] = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_UTF8:						// UTF8 Konvertierung

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_bUseUtf8 = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_RECONNECT:					// Neuverbinden

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}

		iCount = *(int*)pBuffer;

		for(;;)
			{
			if(Reconnect(TRUE))return TRUE;
			iCount--;
			if(iCount<=0)break;
			Sleep(500);
			}

		return FALSE;

	case INTERNET_OPTION_SETSTAT:					// Attribute und Zeiten setzen

		if(uLength<sizeof(WIN32_FIND_DATA))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return SetStat((LPWIN32_FIND_DATA)pBuffer);

	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}
		
		m_dwPutSize = *(LPDWORD)pBuffer;		
		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpAxComm::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{
	if(!m_pSession)return NULL;

	return NULL;
}

//*****************************************************************************
//*
//*		Reconnect
//*
//*****************************************************************************
//	Erzeugt wieder eine Verbindung falls die alte abgebochen ist.
//	bForce		: Immer neu verbinden
BOOL CFtpAxComm::Reconnect(BOOL bForce)
{
CiConnParameter sParam;
char			cServer[64];
char			cPaswd [64];
char			cUser  [64];
char			cUrl   [96];
int				iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	if(!bForce && !m_bReconnect)return TRUE;

	if(!bForce && CiIsOnline(m_pSession))
		{
		return TRUE;
		}

	strawcpy(cServer,m_sServerName,sizeof(cServer));
	strawcpy(cPaswd ,m_sPassword  ,sizeof(cPaswd ));
	strawcpy(cUser  ,m_sUserName  ,sizeof(cUser  ));

	if(m_bConnDone)
		{
		if(m_pSession)CiDisconnect(m_pSession);

		m_bConnDone = FALSE;
		m_pSession  = NULL;

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}
		}

	m_bConnDone = TRUE;

	if(m_wPort)
			sprintf(cUrl,"%s:%u",cServer,m_wPort);
	else	sprintf(cUrl,"%s"   ,cServer);


	sParam.ucQualityOfService = CI_QOS_CURRENT;
    sParam.aRouting[0]        = 0;
    sParam.usLifetick         = 5000;
    sParam.usCycleTime        = 2000;
    sParam.uSessionTimeout    = 0;
    sParam.pCallback          = CiMyCallback;
    sParam.pCbParam           = this;



	iErr = CiConnectWait(&m_pSession,cUrl,&sParam,NULL,0);

	if(iErr==ERR_NONE)return TRUE;


	SetError(iErr);



return FALSE;
}

//*****************************************************************************
//*
//*		SetCurrentDirectory
//*
//*****************************************************************************
//	Stellt das aktuelle Verzeichnis ein.
//	pDirName		: Ist der Name des neuen Verzeichnisses.
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpAxComm::SetCurrentDirectory(LPCTSTR pDirName)
{
char 		cBuffer[CFB_MAX_PATH];
unsigned	uAttrib;
int			iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	DelBackSlash(cBuffer);

	if(!cBuffer[0])
		{
		cBuffer[0] = '/';
		cBuffer[1] = 0;
		}



	   iErr = CiGetFileAttr(m_pSession,cBuffer,&uAttrib);
	if(iErr==ERR_NONE)
		{
		if(!(uAttrib&CI_FILE_DIR))
			{
			SetLastError(ERROR_PATH_NOT_FOUND);
			return FALSE;
			}

		if(m_bUseUtf8)
				strfromutf8(m_cCurrDir,cBuffer,sizeof(m_cCurrDir)/sizeof(TCHAR));
		else	strawcpy   (m_cCurrDir,cBuffer,sizeof(m_cCurrDir)/sizeof(TCHAR));

		return TRUE;
		}

	SetError(iErr);



return FALSE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis.
//	pDirName		: Puffer fr den Namen
//	pLen			: Gre des Puffers
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpAxComm::GetCurrentDirectory(LPTSTR pDirName,LPDWORD pLen) const
{


	if(!m_pSession)return FALSE;

	*pLen = strlcpy(pDirName,m_cCurrDir,*pLen);



return TRUE;
}


//*****************************************************************************
//*
//*		RemoveDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des zu lschenden Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis gelscht wurde.
BOOL CFtpAxComm::RemoveDirectory(LPCTSTR pDirName)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;
int		iLen;


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = CiDeleteFolder(m_pSession,cBuffer);

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		CreateDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpAxComm::CreateDirectory(LPCTSTR pDirName)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;
int		iLen;


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = CiCreateFolder(m_pSession,cBuffer,0);

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		Remove
//*
//*****************************************************************************
//	Lscht eine Datei..
//	pFileName		: Ist der Name der Datei
//	Ergibt TRUE wenn die Datei gelscht wurde.
BOOL CFtpAxComm::Remove(LPCTSTR pFileName)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');

	iErr = CiDeleteFile(m_pSession,cBuffer);

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	Verschiebt eine Datei..
//	pExisting		: Ist der Name der Datei.
//	pNewName		: Ist der Name der Zieldatei.
//	Ergibt TRUE wenn die Datei vershoben wurde.
BOOL CFtpAxComm::Rename(LPCTSTR pExisting,LPCTSTR pNewName)
{
char 	cBuffer[CFB_MAX_PATH];
char 	cTarget[CFB_MAX_PATH];
int		iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pExisting,sizeof(cBuffer));
	PathConvert (cBuffer,'/');

	PathFullMake(cTarget,m_cCurrDir,pNewName,sizeof(cTarget));
	PathConvert (cTarget,'/');

	iErr = CiRenameFile(m_pSession,cBuffer,cTarget);

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
void CFtpAxComm::Close()
{

	if(!m_pSession)return;

	CiDisconnect(m_pSession);

	m_pSession	  = NULL;
	m_bReconnect  = FALSE;
	m_bConnDone	  = FALSE;

}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertgt eine Datei zum FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	pRemoteFile		: Name der Datei am FTP-Server.
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpAxComm::PutFile(LPCTSTR pLocalFile,LPCTSTR pRemoteFile,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;
int		iRet;


	if(!Reconnect())return FALSE;

	   iRet = ExistFile(pLocalFile);
	if(iRet!=1)
		{
		SetLastError((iRet)? ERROR_ACCESS_DENIED:ERROR_FILE_NOT_FOUND);
		m_dwErrOpen = 1;
		return FALSE;
		}

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	m_dwErrOpen = 2;

	#ifdef UNICODE
	char 	 cLocal[CFB_MAX_PATH];
	strawcpy(cLocal,pLocalFile,sizeof(cLocal));
	pLocalFile = (LPCTSTR)cLocal;
	#endif

	m_bAbortAllowed = 1;
	iErr = CiUploadFile(m_pSession,cBuffer,pLocalFile,0,(CiProgressData*)m_aProgess);
	m_bAbortAllowed = 0;

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpAxComm::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(!CiUploadProc)
		{
		return CFtpBase::PutFile(pRemoteFile,pParam,pProc,dwFlags,dwContext);
		}

	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	m_dwErrOpen = 2;

	m_bAbortAllowed = 1;
	iErr = CiUploadProc(m_pSession,cBuffer,pProc,0,pParam,(CiProgressData*)m_aProgess);
	m_bAbortAllowed = 0;

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei vom FTP-Server.
//	pRemoteFile		: Name der Datei am FTP-Server
//	pLocalFile		: Name der lokalen Datei
//	bFailIfExists	: Einen Fehler zurck melden wenn die Datei existiert
//	dwFlags			: Unbenutzt
//	dwContext		: Unbenutzt
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpAxComm::GetFile(LPCTSTR pRemoteFile,LPCTSTR pLocalFile,BOOL bFailIfExists,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(bFailIfExists && ExistFile(pLocalFile))
		{
		SetLastError(ERROR_FILE_EXISTS);
		m_dwErrOpen = 1;
		return FALSE;
		}

	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');

	#ifdef UNICODE
	char 	 cLocal[CFB_MAX_PATH];
	strawcpy(cLocal,pLocalFile,sizeof(cLocal));
	pLocalFile = (LPCTSTR)cLocal;
	#endif

	m_dwErrOpen     = 2;
	m_bAbortAllowed = 1;
	iErr = CiDownloadFile(m_pSession,cBuffer,(char*)pLocalFile,(CiProgressData*)m_aProgess);
	m_bAbortAllowed = 0;

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpAxComm::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(!CiDownloadProc)
		{
		return CFtpBase::GetFile(pRemoteFile,pParam,pProc,dwAttributes,dwFlags,dwContext);
		}


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');


	m_dwErrOpen     = 2;
	m_bAbortAllowed = 1;
	iErr = CiDownloadProc(m_pSession,cBuffer,pProc,pParam,(CiProgressData*)m_aProgess);
	m_bAbortAllowed = 0;

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		List
//*
//*****************************************************************************
//	Liste alle Dateien in einem Verzeichnis via Callback auf.
//	pRemotePath		: Ist der Name des Verzeichnisses.
//	pParam			: Parameter fr den Callback.
//	pProc			: Zeiger auf den Callback.
//	pFilter			: Filter der benutzt werden kann.
//	Ergibt wenn TRUE alles ok war, bzw. FALSE bei einem Fehler.
BOOL CFtpAxComm::List(LPCTSTR pRemotePath,void *pParam,FtpFindCallback pProc,LPCTSTR pFilter)
{
char 			cBuffer[CFB_MAX_PATH];
//char 			cLnName[CFB_MAX_PATH];
LinkEntry	   *pLink;
LinkEntry	   *pNext;
FindCbData		sData;
//CiFindStruct 	sAttr;
//int			iRet;
//int			iNum;
int				iLen;
int				iErr;


	if(!Reconnect())
		{
		pProc(pParam,NULL);
		return FALSE;
		}

	if(!*pRemotePath)
		{
		if(m_bUseUtf8)
				iLen=strtoutf8(cBuffer,m_cCurrDir,sizeof(cBuffer));
		else	iLen=strawcpy (cBuffer,m_cCurrDir,sizeof(cBuffer));
		}
	else{
		       PathFullMake(cBuffer,m_cCurrDir,pRemotePath,sizeof(cBuffer));
		iLen = PathConvert (cBuffer,'/');
		}

	if(!pFilter)pFilter=_T("*");

	if(!iLen || cBuffer[iLen-1]!='/')				// Slash anhngen
		{
		cBuffer[iLen] = '/';
		iLen++;
		}

													// Filter anhngen
	iLen += strawcpy(cBuffer+iLen,pFilter,sizeof(cBuffer)-iLen);


	sData.pProc		 = pProc;
	sData.pParam	 = pParam;
	sData.bUseUtf8	 = m_bUseUtf8;
	sData.bBreak	 = 0;
	sData.pLinkLast	 = 0;
	sData.pLinkFirst = 0;

	m_bAbortAllowed = 1;
	iErr = CiListFiles(m_pSession,cBuffer,CiToFind,&sData);
	m_bAbortAllowed = 0;


	pLink = sData.pLinkFirst;

	if(pLink && iLen>0 && cBuffer[iLen-1]!='/')		// Slash an Pfad anhngen
		{
		cBuffer[iLen] = '/';
		iLen++;
		}

	while(pLink)									// Wurden Soft-Links gefunden
		{
		pNext = (LinkEntry*)pLink->pNext;
		/*
		if(iErr==ERR_NONE)
			{
			if(m_bUseUtf8)
					strtoutf8(cBuffer+iLen,pLink->sFind.cFileName,sizeof(cBuffer)-iLen);
			else	strawcpy (cBuffer+iLen,pLink->sFind.cFileName,sizeof(cBuffer)-iLen);

			m_bAbortAllowed = 1;
			// REM AZ ist nicht implemntiert
			//iRet  = CommReadLink(m_pSession,cBuffer,cLnName,sizeof(cLnName));
			iRet  = ERR_FAIL;
			m_bAbortAllowed = 0;

			if(iRet==ERR_NONE)
				{
				if(cLnName[0]!='/')
					{
					iNum = strlen(cLnName);
					if(iNum+iLen<sizeof(cLnName))
						{
						memmove(cLnName+iLen,cLnName,iNum+1);
						memcpy (cLnName,cBuffer,iLen);
						}
					}

				// REM AZ ist nicht implemntiert
				//   iRet = CommStat(m_pSession,cLnName,&sAttr);
				//if(iRet==SSH2_FX_USERBREAK)iErr=SSH2_FX_USERBREAK;

				if(iRet==ERR_NONE)
					{
					UtcTimeToFileTime(sAttr.dwTimeWrite ,&pLink->sFind.ftLastWriteTime );
					UtcTimeToFileTime(sAttr.dwTimeAccess,&pLink->sFind.ftLastAccessTime);

					pLink->sFind.ftCreationTime		= pLink->sFind.ftLastWriteTime;
					pLink->sFind.nFileSizeHigh		=(sAttr.dwAttr&CI_FILE_HIGHMASK)>>CI_FILE_HIGHSHIFT;
					pLink->sFind.nFileSizeLow		= sAttr.dwSize;
					pLink->sFind.dwFileAttributes	= sAttr.dwAttr&(0xFFFF0000|CI_FILE_HIGHMASK);
		 			pLink->sFind.dwFileAttributes  |= FILE_ATTRIBUTE_LINK;

					if(sAttr.dwAttr&CI_FILE_DIR)
						{
						pLink->sFind.dwFileAttributes |= FILE_ATTRIBUTE_DIRECTORY;
						}
					}
				}

			if(!pProc(pParam,&pLink->sFind))
				{
				// REM AZ Userabbruch
				}
			}
		*/

		delete pLink;
		pLink = pNext;
		}


	pProc(pParam,NULL);

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		SetAttributes
//*
//*****************************************************************************
//	Einstellen der Dateiattribute
//	pFileName		: Ist der Name der Datei.
//	dwAttr			: Sind die neuen Attribute.
//					  In den Bits 16..31 knnen Unix-Attribute verpackt werden.
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpAxComm::SetAttributes(LPCTSTR pFileName,DWORD dwAttr)
{
char 		cBuffer[CFB_MAX_PATH];
unsigned 	uAttrib;
int			iErr;


	if(!Reconnect())return FALSE;


	uAttrib = (dwAttr&FILE_ATTRIBUTE_READONLY)? CI_FILE_READ:CI_FILE_READ|CI_FILE_WRITE;
	if(dwAttr&FILE_ATTRIBUTE_DIRECTORY)uAttrib|=CI_FILE_DIR;
	if(dwAttr&FILE_ATTRIBUTE_HIDDEN   )uAttrib|=CI_FILE_HIDDEN;
	if(dwAttr&FILE_ATTRIBUTE_SYSTEM   )uAttrib|=CI_FILE_SYSTEM;
	if(dwAttr&FILE_ATTRIBUTE_ARCHIVE  )uAttrib|=CI_FILE_ARCHIVE;
	if(dwAttr&FILE_ATTRIBUTE_EXEC	  )uAttrib|=CI_FILE_EXEC;

	if(dwAttr&0xFFFF0000)							// Gibt es Unix Attribute		
		{
		if(dwAttr&(0444<<16))uAttrib|=CI_FILE_READ;
		if(dwAttr&(0222<<16))uAttrib|=CI_FILE_WRITE;
		if(dwAttr&(0111<<16))uAttrib|=CI_FILE_EXEC;
		}


	PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');


	iErr = CiSetFileAttr(m_pSession,cBuffer,uAttrib);

	if(iErr==ERR_NONE)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		SetModifyTime
//*
//*****************************************************************************
//	Einstellen der nderungszeit einer Datei.
//	pFileName		: Ist der Name der Datei.
//	sFileTime		: Ist die neue nderungszeit
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpAxComm::SetModifyTime(LPCTSTR pFileName,FILETIME sFileTime)
{

	SetLastError(ERROR_NOT_SUPPORTED);


return FALSE;
}

//*****************************************************************************
//*
//*		DoAbort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpAxComm::DoAbort()
{


	if(!m_pSession|| !m_bAbortAllowed)return;

	CiDisconnect(m_pSession);

	m_pSession		= NULL;
	m_bReconnect    = TRUE;
	m_bAbortAllowed = 0;


}

//*****************************************************************************
//*
//*		Init
//*
//*****************************************************************************
//	Initialisiert die SFTP Verbindung.
//	iReconnect	: Automatisch wieder verbinden
//	uTimeout	: Timeout in Millisekunden
//	bUtf8		: UTF8 benutzen
//	Ergibt TRUE wenn die Verbindung erzeugt werden konnte.
BOOL CFtpAxComm::Init(int iReconnect,unsigned uTimeout,BOOL bUtf8)
{
char		cBuffer[CFB_MAX_PATH];
int			iSize;
int			iErr;


	if(!m_pSession)return FALSE;

	iSize = CFB_MAX_PATH;
	iErr  = CiGetInfos(m_pSession,CI_INF_AXROOTPATH,cBuffer,&iSize);

	if(iErr!=ERR_NONE)
		{
		SetError(iErr);
		return FALSE;
		}

	strawcpy(m_cCurrDir,cBuffer,(sizeof(m_cCurrDir)/sizeof(TCHAR)));

	m_bReconnect	= (iReconnect)? TRUE:FALSE;
	m_uTimeout		=  uTimeout;
	m_bUseUtf8		=  bUtf8;


return TRUE;
}

//*****************************************************************************
//*
//*		PathFullMake
//*
//*****************************************************************************
int CFtpAxComm::PathFullMake(char *pBuffer,LPCTSTR pCurrDir,LPCTSTR pPath,int iSize)
{
char		cTemp[CFB_MAX_PATH];
const char *pDir;


	if(m_bUseUtf8)
		{
		strtoutf8(cTemp,pCurrDir,sizeof(cTemp));
		pDir = cTemp;
		}
	else{
		#ifdef UNICODE
		strawcpy(cTemp,pCurrDir,sizeof(cTemp));
		pDir = cTemp;
		#else
		pDir = pCurrDir;
		#endif
		}


return ::PathFullMake(pBuffer,pDir,pPath,iSize,0);
}


//*****************************************************************************
//*
//*		SetStat
//*
//*****************************************************************************
//	Einstellen der Dateiattribute und der Zeiten.
//	pInfo		: Sind die neuen Zeiten und Attribute
//				  Wenn pInfo->dwFileAttributes ungleich 0 und -1 ist werden die
//				  Attribute	verndert. Zie Zeiten werden gesetzt wenn sie ungleich
//				  0 sind. In pInfo->cFileName wird der Dateinamge bergeben.
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpAxComm::SetStat(LPWIN32_FIND_DATA pInfo)
{
char 		cBuffer[CFB_MAX_PATH];



	if(!m_pSession)return FALSE;

	if(!Reconnect())return FALSE;


	PathFullMake(cBuffer,m_cCurrDir,pInfo->cFileName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');


	if(pInfo->dwFileAttributes && pInfo->dwFileAttributes!=0xFFFFFFFF)
		{
		return SetAttributes(cBuffer,pInfo->dwFileAttributes);
		}


	SetLastError(ERROR_NOT_SUPPORTED);



return FALSE;
}

//*****************************************************************************
//*
//*		SetError
//*
//*****************************************************************************
//	Stellt die Windows Fehlernummer ein
//	iError		: Ist die FTP Fehlernummer
void CFtpAxComm::SetError(int iError)
{

	switch(iError)
		{
	case ERR_NONE				:	SetLastError(ERROR_SUCCESS);								return;
	case ERR_FAIL				:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_PARAM				:	SetLastError(ERROR_INVALID_PARAMETER);						return;
	case ERR_CANT_LOAD_DLL		:	SetLastError(ERROR_INVALID_DLL);							return;
	case ERR_NOT_ALL_PROCS_FOUND:	SetLastError(ERROR_PROC_NOT_FOUND);							return;
	case ERR_NO_MEMORY			:	SetLastError(ERROR_OUTOFMEMORY);							return;
	case ERR_UNKNOWN_INTERFACE	:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_UNKNOWN_COMMAND	:	SetLastError(ERROR_BAD_COMMAND);		m_bReconnect=TRUE;	return;
	case ERR_ALREADY_RUNNING	:	SetLastError(ERROR_SERVICE_ALREADY_RUNNING);				return;
	case ERR_ALREADY_STOPED		:	SetLastError(ERROR_SERVICE_ALREADY_RUNNING);				return;
	case ERR_NOT_STOPED			:	SetLastError(ERROR_SERVICE_NOT_ACTIVE);						return;
	case ERR_NO_CONNECTION		:	SetLastError(ERROR_NOT_CONNECTED);		m_bReconnect=TRUE;	return;
	case ERR_START_THREAD		:	SetLastError(ERROR_INVALID_THREAD_ID);						return;
	case ERR_BUFFER_FULL		:	SetLastError(ERROR_BUFFER_OVERFLOW);						return;
	case ERR_NO_FREE_SLOTS		:	SetLastError(ERROR_NO_SYSTEM_RESOURCES);					return;
	case ERR_ROUTING_ERROR		:	SetLastError(ERROR_POINT_NOT_FOUND);						return;
	case ERR_WRONG_VERSION		:	SetLastError(ERROR_NO_SUCH_PACKAGE);						return;
	case ERR_WRONG_FORMAT		:	SetLastError(ERROR_BAD_DESCRIPTOR_FORMAT);					return;
	case ERR_NO_DEVICE_CONN		:	SetLastError(ERROR_HOST_UNREACHABLE);						return;
	case ERR_UNKNOWN_BROKER		:	SetLastError(ERROR_NETWORK_UNREACHABLE);					return;
	case ERR_WRONG_URL			:	SetLastError(ERROR_RESOURCE_NAME_NOT_FOUND);				return;
	case ERR_WRONG_SESSION_ID	:	SetLastError(ERROR_SESSION_CREDENTIAL_CONFLICT);			return;
	case ERR_OUT_OF_HANDELS		:	SetLastError(ERROR_NO_MORE_USER_HANDLES);					return;
	case ERR_TIMEOUT			:	SetLastError(ERROR_TIMEOUT);								return;
	case ERR_WRONG_USERID		:	SetLastError(ERROR_USER_MAPPED_FILE);						return;
	case ERR_WORNG_RESPONSE		:	SetLastError(ERROR_BAD_NET_RESP);							return;
	case ERR_WORNG_HANDEL		:	SetLastError(ERROR_INVALID_HANDLE);							return;
	case ERR_NO_SESSION			:	SetLastError(ERROR_INVALID_HANDLE);							return;
	case ERR_SESSION_NOT_FOUND	:	SetLastError(ERROR_NO_SUCH_LOGON_SESSION);					return;
	case ERR_SESSION_WRONG_VER	:	SetLastError(ERROR_OLD_WIN_VERSION);						return;
	case ERR_SESSION_WRONG_PRM	:	SetLastError(ERROR_INVALID_PARAMETER);						return;
	case ERR_CANT_CREATE_CFG	:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_TOO_MUCH_SESSIONS	:	SetLastError(ERROR_NO_MORE_USER_HANDLES);					return;
	case ERR_FORMAT				:	SetLastError(ERROR_INVALID_PARAMETER);						return;
	case ERR_SESSION_FILE_ERR	:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_FILTERD			:	SetLastError(ERROR_INVALID_FILTER_PROC);					return;
	case ERR_WRONG_RANGE		:	SetLastError(ERROR_INVALID_PARAMETER);						return;
	case ERR_EXISTS_AS_FILE		:	SetLastError(ERROR_FILE_NOT_FOUND);							return;
	case ERR_NOT_ONLINE			:	SetLastError(ERROR_RESOURCE_NOT_ONLINE);					return;
	case ERR_LOCAL_FILE			:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_LOCAL_READ			:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_WRITE_FILE			:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_UNKNOWN_INSTANCE	:	SetLastError(ERROR_UNKNOWN_COMPONENT);						return;
	case ERR_CONN_LOCKED		:	SetLastError(ERROR_LOCK_VIOLATION);							return;
	case ERR_NOT_SUPPORTED		:	SetLastError(ERROR_NOT_SUPPORTED);							return;
	case ERR_MODUL_NOT_LOADED	:	SetLastError(ERROR_INVALID_DLL);							return;
	case ERR_INTERFACE_CLOSED	:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_DOUBLE_SESSION_ID	:	SetLastError(ERROR_INVALID_ACCESS);							return;
	case ERR_OUT_OFFSLOTS		:	SetLastError(ERROR_NO_MORE_USER_HANDLES);					return;
		}

	SetLastError(ERROR_INVALID_ACCESS);

}
