//*****************************************************************************
//*
//*
//*		FtpBase.cpp
//*
//*
//*****************************************************************************
//
//	Basisklasse fr FTP-Verbindungen
//
#include	"FtpBase.h"
#include	<MemFunctions.h>
#include	<FileFunctions.h>
#include	<StringFunctions.h>


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpBase::CFtpBase(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive)
{

	ASSERT(pServer !=0);
	ASSERT(pSession!=0);

	m_dwErrOpen		= 0;
	m_dwPutSize		= 0;
	m_wPort			= nPort;
	m_bPassive		= bPassive;
	m_pSession		= pSession;
	m_dwContext		= dwContext;
	m_pBaseUrl		= _T("ftp://");
	m_bAbortAllowed	= FALSE;

	if(pServer  )m_sServerName=pServer;
	if(pUserName)m_sUserName=pUserName;
	if(pPassword)m_sPassword=pPassword;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpBase::~CFtpBase()
{


}


//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//					INTERNET_OPTION_???
//	sRefString	: Hier werden Texte abgelegt.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpBase::QueryOption(DWORD dwOption,CString &sRefString) const
{
DWORD	dwLength;
LPTSTR	pBuffer;
BOOL	bRet;


	dwLength = 0;

	if(!QueryOption(dwOption,NULL,&dwLength) && GetLastError()!=ERROR_INSUFFICIENT_BUFFER)
		{
		sRefString.Empty();
		return FALSE;
		}

	pBuffer = sRefString.GetBuffer(dwLength);
	bRet	= QueryOption(dwOption,pBuffer,&dwLength);

	sRefString.ReleaseBuffer();


return bRet;
}

//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpBase::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{

	SetLastError(ERROR_INVALID_NAME);

return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpBase::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{

	switch(dwOption)
		{
	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

			if(uLength<sizeof(DWORD))
				{
				SetLastError(ERROR_INVALID_PARAMETER);
				return FALSE;
				}
			
			m_dwPutSize = *(LPDWORD)pBuffer;		
			return TRUE;
		}
	

	SetLastError(ERROR_INVALID_NAME);



return FALSE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectoryAsURL
//*
//*****************************************************************************
//	Holt das aktelle Verzeichnis des Servers als URL-String.
//	sDirName	: CString in dem der URL-String gespeichert wird.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpBase::GetCurrentDirectoryAsURL(CString &sDirName) const
{
CString		sDirectory;


	if(!GetCurrentDirectory(sDirectory))
		{
		return FALSE;
		}

	sDirName  = m_pBaseUrl;
	sDirName += GetServerName();

	if(sDirectory.GetLength()<=0 || (sDirectory[0]!='/' && sDirectory[0]!='\\'))
		{
		sDirName+='/';
		}

	sDirName += sDirectory;


return TRUE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectoryAsURL
//*
//*****************************************************************************
//	Holt das aktelle Verzeichnis des Servers als URL-String.
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpBase::GetCurrentDirectoryAsURL(LPTSTR pBuffer,LPDWORD pLen) const
{
CString sTemp;
int		iLen;


	ASSERT( pLen != 0);
	ASSERT(*pLen != 0);
	ASSERT_POINTER(pLen,DWORD);
	ASSERT(AfxIsValidAddress(pBuffer, *pLen));


	if(pLen==NULL || !GetCurrentDirectoryAsURL(sTemp))
		{
		return FALSE;
		}

	if(pBuffer==NULL)
		{
		*pLen = sTemp.GetLength();
		}
	else{
		iLen  = *pLen;
		if(iLen<0)iLen=0;
		lstrcpyn(pBuffer,(LPCTSTR)sTemp,iLen);
		}


return TRUE;
}

//*****************************************************************************
//*
//*		Abort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpBase::Abort()
{

	if(!m_bAbortAllowed)return;

	DoAbort();

}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis am Server.
//	sDirName		: Hier wird das Verzeichnis abgelegt
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpBase::GetCurrentDirectory(CString &sDirName) const
{
LPTSTR	pTarget;
DWORD	dwLen;
BOOL	bRet;


	ASSERT_VALID(this);
	ASSERT(m_pSession!=NULL);

	dwLen	= INTERNET_MAX_PATH_LENGTH;
	pTarget = sDirName.GetBufferSetLength(dwLen);
	bRet	= GetCurrentDirectory(pTarget,&dwLen);

	if (bRet)
			sDirName.ReleaseBuffer(dwLen);
	else	sDirName.ReleaseBuffer(0);


return bRet;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis am Server.
//	pDirName		: Hier wird das Verzeichnis abgelegt
//	uLen			: Ist die Gre des Puffers in Zeichen
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpBase::GetCurrentDirectory(LPTSTR pDirName,DWORD uLen) const
{
return GetCurrentDirectory(pDirName,&uLen);
}

//*****************************************************************************
//*
//*		CreatePath
//*
//*****************************************************************************
//	Erzeugt eine Pfad rekursiv. Dabei werden alle einzel Pfade erzeugt
//	pDirName	: Ist der Pfadname
//	Ergibt TRUE wenn der Pfad erzeugt werden konnte.
BOOL CFtpBase::CreatePath(LPCTSTR pDirName)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		aOffset[32];
int		iLen;
int		iNum;


	iLen = strlcpy(cBuffer,pDirName,sizeof(cBuffer)/sizeof(TCHAR));
	if(iLen>=sizeof(cBuffer)/sizeof(TCHAR)-2)return FALSE;
	iLen = DelBackSlash(cBuffer,iLen);
	if(iLen<=0 || cBuffer[iLen-1]==':')return FALSE;

	iNum = 0;

	while(iNum>=0)									// Suche rekursive nach erstem Verzeichnis
		{
		if(CreateDirectory(cBuffer))break;

		for(iLen--;iLen>0;iLen--)
			{
			if(cBuffer[iLen]=='/' )break;
			if(cBuffer[iLen]=='\\')break;
			}

		if(iLen<1 || cBuffer[iLen-1]==':')return FALSE;

		aOffset[iNum] = iLen;
		cBuffer[iLen] = 0;
		iNum++;

		if(CreateDirectory(cBuffer))break;
		}

	while(iNum>0)									// Verzeichnisse Rekursive erzeugen
		{
		iNum--;
		iLen = aOffset[iNum];
		cBuffer[iLen] = '/';

		if(!CreateDirectory(cBuffer))return FALSE;
		}


return TRUE;
}


//*****************************************************************************
//*
//*		RemovePath
//*
//*****************************************************************************
//	Lscht einen Pfad rekursiv. Dabei werden alle Unterverzeichnisse mitgelscht.
//	pDirName	: Ist der Pfadname
//	Ergibt TRUE wenn der Pfad gelscht werden.
BOOL CFtpBase::RemovePath(LPCTSTR pDirName)
{
CString			sName;
CFtpBaseFind	sFind(this);
TCHAR			cBuffer[CFB_MAX_PATH];
int				iLen;
BOOL			bOk;


	if(!*pDirName)return FALSE;

	   iLen = strlcpy(cBuffer,pDirName,sizeof(cBuffer)/sizeof(TCHAR));
	if(iLen>=sizeof(cBuffer)/sizeof(TCHAR)-2)return FALSE;
	   iLen = AddBackSlash('/',cBuffer,iLen);
	if(iLen<=0 || cBuffer[iLen-1]==':')return FALSE;

	cBuffer[iLen] = 0;
	bOk = sFind.FindFile(cBuffer,INTERNET_FLAG_NO_DOTS);

	while(bOk)
		{
		bOk	  = sFind.FindNextFile();
		sName = sFind.GetFileName();

		if(sName.GetLength()+iLen+2<sizeof(cBuffer)/sizeof(TCHAR))
			{
			strcpy(cBuffer+iLen,sName);

			if(sFind.IsDirectory())
					RemovePath(cBuffer);
			else	Remove    (cBuffer);
			}
		}

	cBuffer[iLen-1] = 0;


return RemoveDirectory(cBuffer);
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpBase::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{
	return NULL;
}

//*****************************************************************************
//*
//*		GetLocalPath
//*
//*****************************************************************************
//	Holt den lokalen Pfad zu einer FTP-Datei, falls er existiert
//	pFileName		: Name der FTP-Datei zu der der lokale Pfad geholt werden soll.
//	pBuffer			: Puffer fr den lokalen Namen
//	iSize			: Gre des Puffers in Zeichen.
//	Ergibt die lokale Pfadlnge in Zeichen bzw. 0 wenn es keine lokalen Pfad gibt.
int CFtpBase::GetLocalPath(LPCTSTR pFileName,LPTSTR pBuffer,int iSize)
{

	if(iSize>0)pBuffer[0]=0;

return 0;
}

//*****************************************************************************
//*
//*		SetStat
//*
//*****************************************************************************
//	Einstellen der Dateiattribute und der Zeiten.
//	pInfo		: Sind die neuen Zeiten und Attribute
//				  Wenn pInfo->dwFileAttributes ungleich 0 und -1 ist werden die
//				  Attribute	verndert. Zie Zeiten werden gesetzt wenn sie ungleich
//				  0 sind. In pInfo->cFileName wird der Dateinamge bergeben.
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpBase::SetStat(LPWIN32_FIND_DATA pInfo)
{
LPCTSTR		pFileName;
BOOL		bDone;




	pFileName = pInfo->cFileName;


	if(pInfo->ftLastWriteTime.dwHighDateTime)		// Zeiten setzen
		{
			bDone = SetModifyTime(pFileName,pInfo->ftLastWriteTime);
		if(!bDone)return FALSE;
		}

													// Attribute setzen
	if(pInfo->dwFileAttributes && pInfo->dwFileAttributes!=0xFFFFFFFF)
		{
		if(!SetAttributes(pFileName,pInfo->dwFileAttributes&~FILE_SET_ATTRIBUTE))return FALSE;
		}



return TRUE;
}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpBase::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{
char		aBuffer[4096];
TCHAR		cPath[MAX_PATH];	
TCHAR		cName[MAX_PATH];	
DWORD		uError;
HANDLE		hFile;
INT			iSize;
BOOL		bOk;


	GetTempPath(MAX_PATH,cPath);
	GetTempFileName(cPath,_T("Ftp"),0,cName);

	   hFile = CreateFile(cName,GENERIC_READ|GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		return FALSE;
		}


	for(;;)											
		{
		   iSize = pProc(pParam,aBuffer,sizeof(aBuffer));
		if(iSize<=0)
			{
			if(iSize==0)break;
			CloseHandle(hFile);
			DeleteFile(cName);
			SetLastError(ERROR_READ_FAULT);
			return FALSE;
			}
		}

	CloseHandle(hFile);

		bOk = PutFile(cName,pRemoteFile,dwFlags,dwContext);
	if(!bOk)
		{
		uError = GetLastError();
		}
	else{
		uError = ERROR_SUCCESS;
		}
	
	
	DeleteFile(cName);
	SetLastError(uError);


return bOk;
}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei aus einem Speicher.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pMemory			: Zeiger auf den Speicher
//	iSize			: Ist die Gre des Speichers
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpBase::PutFile(LPCTSTR pRemoteFile,const LPVOID pMemory,INT iSize,DWORD dwFlags,DWORD dwContext)
{
MemoryData	sData;


	if(iSize<0)
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}

	sData.pMem	= (LPBYTE)pMemory;
	sData.iLen	= iSize;
	sData.iPos	= 0;


return PutFile(pRemoteFile,&sData,MemRead,dwFlags,dwContext);
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpBase::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
TCHAR		aBuffer[4096/sizeof(TCHAR)];
TCHAR		cName[MAX_PATH];	
DWORD		uError;
DWORD		uBytes;
HANDLE		hFile;
INT			iSize;
BOOL		bOk;



	GetTempPath(MAX_PATH,aBuffer);
	GetTempFileName(aBuffer,_T("Ftp"),0,cName);


		bOk = GetFile(pRemoteFile,cName,dwAttributes,dwFlags,dwContext);
	if(!bOk)
		{
		return FALSE;
		}


	   hFile = CreateFile(cName,GENERIC_READ,0,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		return FALSE;
		}


	for(;;)											
		{
		if(!ReadFile(hFile,aBuffer,sizeof(aBuffer),&uBytes,NULL))
			{
			uError = GetLastError();
			bOk	   = FALSE;
			break;
			}

		   iSize = pProc(pParam,aBuffer,uBytes);
		if(iSize^uBytes)
			{
			uError = ERROR_READ_FAULT;
			bOk	   = FALSE;
			break;
			}
		
		if(uBytes<sizeof(aBuffer))
			{
			uError = ERROR_SUCCESS;
			bOk	   = TRUE;
			break;
			}
		}


	CloseHandle (hFile);
	DeleteFile  (cName);
	SetLastError(uError);


return bOk;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei und legt sie in einem Speicher ab.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pMemory			: Zeiger auf den Speicher
//	iSize			: Ist die maximale Gre des Speichers
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt die Gre der geholten Datei oder -1 bei einem Fehler.
BOOL CFtpBase::GetFile(LPCTSTR pRemoteFile,LPVOID pMemory,INT iSize,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
MemoryData	sData;
BOOL		bOk;

	if(iSize<0)
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}

	sData.pMem	= (LPBYTE)pMemory;
	sData.iLen	= iSize;
	sData.iPos	= 0;

		bOk = GetFile(pRemoteFile,(LPVOID)&sData,MemWrite,dwAttributes,dwFlags,dwContext);
	if(!bOk)return -1;


return sData.iPos;
}

//*****************************************************************************
//*
//*		MemRead
//*
//*****************************************************************************
int __cdecl CFtpBase::MemRead(void *pParam,void *pBuffer,int iSize)
{
MemoryData *pData=(MemoryData*)pParam;
int			iMax;


	   iMax = pData->iLen-pData->iPos;
	if(iSize>iMax)iSize=iMax;

	memcpy(pBuffer,pData->pMem+pData->iPos,iSize);
	pData->iPos += iSize;
	

return iSize;
}


//*****************************************************************************
//*
//*		MemWrite
//*
//*****************************************************************************
int __cdecl CFtpBase::MemWrite(void *pParam,const void *pBuffer,int iSize)
{
MemoryData *pData=(MemoryData*)pParam;
int			iMax;


	   iMax = pData->iLen-pData->iPos;
	if(iSize>iMax)iSize=iMax;

	memcpy(pData->pMem+pData->iPos,pBuffer,iSize);
	pData->iPos += iSize;
	

return iSize;
}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpBaseFind::CFtpBaseFind(CFtpBase *pConnection,DWORD dwContext)
{

	ASSERT(pConnection!=NULL);

	m_pConnection = pConnection;
	m_bClose	  = FALSE;
	m_bDone		  = FALSE;
	m_pPlus		  = 0;
	m_pMinus	  = 0;
	m_iReadPos	  = 0;

}


//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpBaseFind::~CFtpBaseFind()
{


}


//*****************************************************************************
//*
//*		ListCallback
//*
//*****************************************************************************
//	Callback fr die List-Funktion
//	Ergibt 0 bei einem Abbruch bzw 1. um mit der Suche weiter zu machen.
int _cdecl CFtpBaseFind::ListCallback(void *pParam,WIN32_FIND_DATA *pFind)
{
CFtpBaseFind   *pThis=(CFtpBaseFind*)pParam;
int				iLen;


	if(!pFind)												// Ist die Suche beendet
		{
		pThis->m_bDone = TRUE;

		return FALSE;
		}


	if(pThis->m_bClose)										// Benzterabbruch
		{
		return FALSE;
		}

	if(pFind->dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)	// Verzeichnis gefunden
		{
		if(pThis->m_bNoDirs)return TRUE;
		if(pThis->m_bNoDots && pFind->cFileName[0]=='.')
			{
			if(pFind->cFileName[1]== 0 )return TRUE;
			if(pFind->cFileName[1]=='.')
			if(pFind->cFileName[2]== 0 )return TRUE;
			}

		if(pThis->m_bAllDirs)goto DoIt;
		}
	else{
		if(pThis->m_bOnlyDirs)return TRUE;
		}


	iLen = _tcslen(pFind->cFileName);

	if(pThis->m_pFilter && memxcmp(pThis->m_pFilter,pFind->cFileName,iLen,0x03))
		{
		return TRUE;
		}

	if(pThis->m_pPlus && memfilter(pFind->cFileName,pThis->m_pPlus,1)<0)
		{
		return TRUE;
		}

	if(pThis->m_pMinus && memfilter(pFind->cFileName,pThis->m_pMinus,1)>0)
		{
		return TRUE;
		}

DoIt:

	pThis->m_sList.Add(pFind->cFileName,CSC_END,pFind);


return TRUE;
}

//*****************************************************************************
//*
//*		FindFile
//*
//*****************************************************************************
//	Suche die Dateien, erst ab dem ersten Aufruf von FindNext(...)
//	knnen die Daten abgefragt werden.
//	pName		: Suchpfad mit Namensfilter, bzw. aktueller Pfad bei NULL.
//				  Wenn der Suchpfad keine Wildcards	(*?) enthlt, so wird
//				  der Suchpfad immer als Verzeichnis interpretiert.
//	dwFlags		: Diverse Flags fr die Suche.
//					INTERNET_FLAG_ALL_DIRS	Alle Verzeichnisse suchen auch bei Filtern
//					INTERNET_FLAG_NO_DIRS	Keine Verzeichnisse finden
//					INTERNET_FLAG_NO_DOTS	Keine "." und ".." Verzeichnisse
//					INTERNET_FLAG_ONLY_DIRS	Nur Verzeichnisse finden
//					INTERNET_FLAG_RELOAD	Die Dateiliste neu Laden
//	Ergibt TRUE wenn Dateien gefunden wurden.
BOOL CFtpBaseFind::FindFile(LPCTSTR pName,DWORD dwFlags)
{
LPCTSTR	    pFilter; 	
CFtpBase   *pBase;
LPCTSTR		pFile;
CString		sPath;
DWORD		uVal;
int			iLen;
int			iRet;


	ASSERT((dwFlags & INTERNET_FLAG_ASYNC) == 0);
	ASSERT(m_pConnection != NULL);
	ASSERT_VALID(m_pConnection);


	if(!m_pConnection)
		{
		return FALSE;
		}

											// Bestehende Liste neu auslesen
	if(!(dwFlags&INTERNET_FLAG_RELOAD) && m_sList.Count()>0)
		{
		m_iReadPos = 0;

		return TRUE;
		}


	if(!pName)pName = _T("");

	pFile		= ::GetFileName(pName);
	iLen		= strcwlen(pFile,0x01800000);
	GetFilePathR(sPath,pName);

	if(pFile[iLen])							// Sind Wildcards (*,?) enthalten
		{
		m_pFilter = pFile;
		}
	else{
		m_pFilter = NULL;
		sPath    += pFile;
		pFile    += iLen;
		}

	m_iReadPos	= 0;
	m_pMinus	= NULL;
	m_pPlus		= NULL;
	m_bDone		= FALSE;
	m_bNoDots	= (dwFlags&INTERNET_FLAG_NO_DOTS  )? 1:0;
	m_bNoDirs	= (dwFlags&INTERNET_FLAG_NO_DIRS  )? 1:0;
	m_bAllDirs	= (dwFlags&INTERNET_FLAG_ALL_DIRS )? 1:0;
	m_bOnlyDirs	= (dwFlags&INTERNET_FLAG_ONLY_DIRS)? 1:0;


	if(*pName=='/' || *pName=='\\')			// Globaler Pfad "/....."
		{
		m_sRoot = sPath;
		}									// Globaler Pfad mit Laufwerk "D:/...."
	else if(*pName && pName[1]==':' && (pName[2]=='/' || pName[2]=='\\'))
		{
		m_sRoot = sPath;
		}
	else{									// Lokaler Pfad
		m_pConnection->GetCurrentDirectory(m_sRoot);
		AddBackSlash(m_sRoot,'/');
		m_sRoot += sPath;
		}


	if(m_sRoot.GetLength()>1)
		{
		::DelBackSlash(m_sRoot);
		}

	::PathConvert(m_sRoot,'/');


	m_sList.Del();
	m_sList.DataLength(sizeof(m_sFindData)-sizeof(m_sFindData.cFileName));

	pBase	= (CFtpBase*)m_pConnection;
	pFilter	= m_pFilter;
	
											// Kann die Verbindung nur Verzeichnisse auflisten
	if(!m_bAllDirs && m_bOnlyDirs && !pFilter) 
		{
		if(pBase->QueryOption(INTERNET_OPTION_ONLYDIR,uVal) && uVal)
			{
			pFilter = CFB_ONLYDIR;
			}
		}
	else if(m_bAllDirs)
		{
		pFilter = NULL;
		}

											// Die Suche starten
	iRet  = pBase->List(m_sRoot,this,ListCallback,pFilter);


	if(!iRet || m_bClose || !m_sList.Count())
		{
		m_sList.Del();
		return 0;
		}



return TRUE;
}

//*****************************************************************************
//*
//*		FindFile
//*
//*****************************************************************************
//	Suche die Dateien, erst ab dem ersten Aufruf von FindNext(...)
//	knnen die Daten abgefragt werden.
//	pName		: Wenn der Suchpfad , bzw. aktueller Pfad bei NULL.
//				  Wildcards	(*?) im Suchpfas sind nicht erlaubt.
//				  der Suchpfad immer als Verzeichnis interpretiert.
//	pPlus		: Ist der Plusfilter fr die Suche (z.B,: "*.gif;*.jpg")
//	pMinus		: Ist der Plusfilter fr die Suche (z.B,: "*.gif;*.jpg")
//	dwFlags		: Diverse Flags fr die Suche.
//					INTERNET_FLAG_ALL_DIRS	Alle Verzeichnisse suchen auch bei filtern
//					INTERNET_FLAG_NO_DIRS	Keine Verzeichnisse finden
//					INTERNET_FLAG_NO_DOTS	Keine "." und ".." Verzeichnisse
//					INTERNET_FLAG_ONLY_DIRS	Nur Verzeichnisse finden
//					INTERNET_FLAG_RELOAD	Die Dateiliste neu Laden
//	Ergibt TRUE wenn Dateien gefunden wurden.
BOOL CFtpBaseFind::FindFile(LPCTSTR pName,LPCTSTR pPlus,LPCTSTR pMinus,DWORD dwFlags)
{
LPCTSTR	    pFilter; 	
CFtpBase   *pBase;
DWORD		uVal;
int			iLen;
int			iRet;



	ASSERT((dwFlags & INTERNET_FLAG_ASYNC) == 0);
	ASSERT(m_pConnection != NULL);
	ASSERT_VALID(m_pConnection);


	if(!m_pConnection)
		{
		return FALSE;
		}

											// Bestehende Liste neu auslesen
	if(!(dwFlags&INTERNET_FLAG_RELOAD) && m_sList.Count()>0)
		{
		m_iReadPos = 0;

		return TRUE;
		}


	if(!pName)pName = _T("");
	if(!pPlus)pPlus = _T("");


	m_iReadPos	= 0;
	m_pMinus	= pMinus;
	m_pPlus		= pPlus;
	m_bDone		= FALSE;
	m_bNoDots	= (dwFlags&INTERNET_FLAG_NO_DOTS  )? 1:0;
	m_bNoDirs	= (dwFlags&INTERNET_FLAG_NO_DIRS  )? 1:0;
	m_bAllDirs	= (dwFlags&INTERNET_FLAG_ALL_DIRS )? 1:0;
	m_bOnlyDirs	= (dwFlags&INTERNET_FLAG_ONLY_DIRS)? 1:0;

	iLen		= strchar(pPlus,';');

	if(pPlus[iLen] || !iLen)				// Mehrfacher Filter
		{
		m_pFilter = NULL;
		}
	else{									// Einfacher Filter
		m_pFilter = pPlus;
		m_pPlus   = 0;
		}


	if(*pName=='/' || *pName=='\\')			// Globaler Pfad "/....."
		{
		m_sRoot = pName;
		}									// Globaler Pfad mit Laufwerk "D:/...."
	else if(*pName && pName[1]==':' && (pName[2]=='/' || pName[2]=='\\'))
		{
		m_sRoot = pName;
		}
	else{									// Lokaler Pfad
		m_pConnection->GetCurrentDirectory(m_sRoot);
		AddBackSlash(m_sRoot,'/');
		m_sRoot += pName;
		}

	if(m_sRoot.GetLength()!=1 || (m_sRoot[0]!='/' && m_sRoot[0]!='\\'))
		{
		::DelBackSlash(m_sRoot);
		}

	::PathConvert(m_sRoot,'/');


	m_sList.Del();
	m_sList.DataLength(sizeof(m_sFindData)-sizeof(m_sFindData.cFileName));

											
	pBase	= (CFtpBase*)m_pConnection;
	pFilter	= m_pFilter;

											// Kann die Verbindung nur Verzeichnisse auflisten
	if(!m_bAllDirs && m_bOnlyDirs && !pFilter) 
		{
		if(pBase->QueryOption(INTERNET_OPTION_READONLY,uVal) && uVal)
			{
			pFilter = CFB_ONLYDIR;
			}
		}
	else if(m_bAllDirs)
		{
		pFilter = NULL;
		}
											// Die Suche starten
 	iRet  = pBase->List(m_sRoot,this,ListCallback,(m_bAllDirs)? NULL:m_pFilter);


	if(!iRet || m_bClose || !m_sList.Count())
		{
		m_sList.Del();
		return 0;
		}




return TRUE;
}

//*****************************************************************************
//*
//*		FindNextFile
//*
//*****************************************************************************
//	Die nchsten Dateidaten holen.
//	Diese Funktion muss mindestens einmal nach einen erfolgreichen FindFile(...)
//	Aufruf ausgefhrt werden, um Dateidaten zu erhalten.
//	Ergibt TRUE wenn daten vorhanden sind.
BOOL CFtpBaseFind::FindNextFile()
{


	ASSERT_VALID(this);
	ASSERT(m_bDone);

	if(m_bDone==0)return FALSE;

	if(m_iReadPos>=m_sList.Count())
		{
		return FALSE;
		}


	memcpy(&m_sFindData,m_sList.Data(m_iReadPos),sizeof(m_sFindData)-sizeof(m_sFindData.cFileName));
	strlcpy(m_sFindData.cFileName,m_sList[m_iReadPos],sizeof(m_sFindData.cFileName)/sizeof(TCHAR),m_sList.Len(m_iReadPos));

	m_iReadPos++;

	if(m_iReadPos>=m_sList.Count())
		{
		return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
BOOL CFtpBaseFind::Close()
{

	m_bClose = TRUE;
	m_sList.Del();


return TRUE;
}

//*****************************************************************************
//*
//*		GetFileURL
//*
//*****************************************************************************
CString CFtpBaseFind::GetFileURL() const
{
CString		sText;
CString		sPath;
LPCSTR		pPath;


	ASSERT(m_bDone);
	ASSERT_VALID(this);
	ASSERT(m_iReadPos>0);


	if(!m_bDone)
		{
		sPath	= GetFilePath();
		pPath	= sPath;

		if(*pPath=='/' )pPath++;
		if(*pPath=='\\')pPath++;

		sText	= m_pConnection->m_pBaseUrl;
		sText  += m_pConnection->m_sServerName;
		sText  += _T("/");
		sText  += pPath;
		}


return sText;
}

//*****************************************************************************
//*
//*		GetFilePath
//*
//*****************************************************************************
CString CFtpBaseFind::GetFilePath() const
{
CString		sText;


	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	sText  = m_sRoot;
	AddBackSlash(sText,'/');
	sText += GetFileName();


return sText;
}


//*****************************************************************************
//*
//*		GetRoot
//*
//*****************************************************************************
CString CFtpBaseFind::GetRoot() const
{
	ASSERT_VALID(this);
	ASSERT(m_bDone);

	return m_sRoot;
}


//*****************************************************************************
//*
//*		GetFileTitle
//*
//*****************************************************************************
CString CFtpBaseFind::GetFileTitle() const
{
	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	CString sFullName = GetFileName();
	CString sResult;

	_tsplitpath(sFullName,NULL,NULL,sResult.GetBuffer(_MAX_PATH),NULL);
	sResult.ReleaseBuffer();


return sResult;
}


//*****************************************************************************
//*
//*		GetFileName
//*
//*****************************************************************************
CString CFtpBaseFind::GetFileName() const
{
	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	CString sText;

	if(m_iReadPos>0)
		{
		sText = m_sFindData.cFileName;
		}


return sText;
}

//*****************************************************************************
//*
//*		GetLength
//*
//*****************************************************************************
DWORD CFtpBaseFind::GetLength() const
{
	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);


return m_sFindData.nFileSizeLow;
}

//*****************************************************************************
//*
//*		GetLength64
//*
//*****************************************************************************
__int64 CFtpBaseFind::GetLength64() const
{
__int64		iRet;

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);


	iRet   = m_sFindData.nFileSizeHigh;
	iRet <<= 32;
	iRet  += m_sFindData.nFileSizeLow;

return iRet;
}


//*****************************************************************************
//*
//*		MatchesMask
//*
//*****************************************************************************
BOOL CFtpBaseFind::MatchesMask(DWORD dwMask) const
{

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

return (m_sFindData.dwFileAttributes&dwMask)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		GetLastAccessTime
//*
//*****************************************************************************
BOOL CFtpBaseFind::GetLastAccessTime(FILETIME* pTimeStamp) const
{

	ASSERT(pTimeStamp != NULL);
	ASSERT(m_bDone);
	ASSERT_VALID(this);

	*pTimeStamp = m_sFindData.ftLastAccessTime;

return TRUE;
}

//*****************************************************************************
//*
//*		GetLastWriteTime
//*
//*****************************************************************************
BOOL CFtpBaseFind::GetLastWriteTime(FILETIME* pTimeStamp) const
{

	ASSERT(pTimeStamp != NULL);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);
	ASSERT_VALID(this);

	*pTimeStamp = m_sFindData.ftLastWriteTime;

return TRUE;
}

//*****************************************************************************
//*
//*		GetCreationTime
//*
//*****************************************************************************
BOOL CFtpBaseFind::GetCreationTime(FILETIME *pTimeStamp) const
{

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	*pTimeStamp = m_sFindData.ftCreationTime;

return TRUE;
}

//*****************************************************************************
//*
//*		GetLastAccessTime
//*
//*****************************************************************************
BOOL CFtpBaseFind::GetLastAccessTime(CTime &sTime) const
{
time_t	iTime;
BOOL	bOk;

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	bOk   = FileTimeToUtcTime(&m_sFindData.ftLastAccessTime,&iTime);
	sTime = iTime;

return bOk;
}

//*****************************************************************************
//*
//*		GetLastWriteTime
//*
//*****************************************************************************
BOOL CFtpBaseFind::GetLastWriteTime(CTime &sTime) const
{
time_t	iTime;
BOOL	bOk;

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	bOk   = FileTimeToUtcTime(&m_sFindData.ftLastWriteTime,&iTime);
	sTime = iTime;

return bOk;
}

//*****************************************************************************
//*
//*		GetCreationTime
//*
//*****************************************************************************
BOOL CFtpBaseFind::GetCreationTime(CTime &sTime) const
{
time_t	iTime;
BOOL	bOk;

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	bOk   = FileTimeToUtcTime(&m_sFindData.ftCreationTime,&iTime);
	sTime = iTime;

return bOk;
}

//*****************************************************************************
//*
//*		GetLastWriteTime
//*
//*****************************************************************************
time_t CFtpBaseFind::GetLastWriteTime() const
{
time_t	iTime;

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	FileTimeToUtcTime(&m_sFindData.ftLastWriteTime,&iTime);

return iTime;
}

//*****************************************************************************
//*
//*		GetLastAccessTime
//*
//*****************************************************************************
time_t CFtpBaseFind::GetLastAccessTime() const
{
time_t	iTime;

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	FileTimeToUtcTime(&m_sFindData.ftLastAccessTime,&iTime);

return iTime;
}

//*****************************************************************************
//*
//*		GetCreationTime
//*
//*****************************************************************************
time_t CFtpBaseFind::GetCreationTime() const
{
time_t	iTime;

	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	FileTimeToUtcTime(&m_sFindData.ftCreationTime,&iTime);

return iTime;
}

//*****************************************************************************
//*
//*		IsDots
//*
//*****************************************************************************
BOOL CFtpBaseFind::IsDots() const
{
	ASSERT_VALID(this);
	ASSERT(m_bDone);
	ASSERT(m_iReadPos>0);

	if(m_sFindData.cFileName[0]!='.')return FALSE;
	if(m_sFindData.cFileName[1]== 0 )return TRUE;
	if(m_sFindData.cFileName[1]=='.' && m_sFindData.cFileName[2]==0)
		{
		return TRUE;
		}

return FALSE;
}


