//*****************************************************************************
//*
//*
//*		FtpCommon.cpp
//*
//*
//*****************************************************************************
//
//	Ist eine normale FTP-Verbindung via FTP-Protokoll.
//
#include	"FtpTools.h"
#include	"FtpCommon.h"
#include	"AzFtpClient.h"
#include	"../MemFunctions.h"
#include	"../FileFunctions.h"


typedef	struct
	{
	FtpFindCallback	pProc;
	LPVOID			pParam;
	LPCTSTR			pFilter;
	}FilterCbData;


//*****************************************************************************
//*
//*		FilterCallback
//*
//*****************************************************************************
static int _cdecl FilterCallback(void *pParam,WIN32_FIND_DATA *pFind)
{
FilterCbData	   *pData=(FilterCbData*)pParam;


	if(pFind && memxcmp(pData->pFilter,pFind->cFileName,_tcslen(pFind->cFileName),0x03))
		{
		return 1;
		}


return pData->pProc(pData->pParam,pFind);
}

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpCommon::CFtpCommon(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive) : CFtpBase(pSession,pServer,pUserName,pPassword,dwContext,nPort,bPassive)
{
	m_bRetry = FALSE;
	m_bBreak = FALSE;
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpCommon::~CFtpCommon()
{

	if(m_pSession)Close();

}


//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpCommon::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{
DWORD	   *pVal=(DWORD*)pBuffer;
DWORD		uLen;



	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	uLen = *pLen;


	switch(dwOption)
		{
	case INTERNET_OPTION_MODE:						// Verbindungsmodus

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_FTP;

		return TRUE;

	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:		// Timeout
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		*pVal = AzFtpGetTimeout((AzFtpHandle)m_pSession);
		*pLen = sizeof(DWORD);

		return TRUE;

	case INTERNET_OPTION_PROGRESS:					// Verlaufsanzeige

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return AzFtpGetProgress((AzFtpHandle)m_pSession,(LPUINT)pVal,0);

	case INTERNET_OPTION_PROGRESS2:					// Verlaufsanzeige mit Maximalwert

		if(uLen<sizeof(DWORD)*2)
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return AzFtpGetProgress((AzFtpHandle)m_pSession,(LPUINT)pVal,(LPUINT)pVal+1);

	case INTERNET_OPTION_CONNTYPE:					// Art der Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_FTP;

		return TRUE;

	case INTERNET_OPTION_URLFORMAT:					// Format des URL-Namens

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 0;

		return TRUE;

	case INTERNET_OPTION_ERROPEN:					// Wo gab es den Fehler

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_dwErrOpen;

		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpCommon::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{
int		iCount;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	switch(dwOption)
		{
	case INTERNET_OPTION_RETRY:

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_bRetry = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		AzFtpSetTimeout((AzFtpHandle)m_pSession,*(DWORD*)pBuffer);

		return TRUE;

	case INTERNET_OPTION_PROGRSET:					// Verlaufsanzeige einstellen

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		AzFtpSetProgress((AzFtpHandle)m_pSession,*(int*)pBuffer);

		return TRUE;

	case INTERNET_OPTION_RECONNECT:					// Neuverbinden

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}

		iCount = *(int*)pBuffer;

		for(;;)
			{
			if(!AzFtpReconnect((AzFtpHandle)m_pSession))break;
			iCount--;
			if(iCount<=0)break;
			Sleep(500);
			}

		SetLastError(ERROR_NOT_CONNECTED);
		return FALSE;

	case INTERNET_OPTION_SETSTAT:					// Attribute und Zeiten setzen

		if(uLength<sizeof(WIN32_FIND_DATA))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return SetStat((LPWIN32_FIND_DATA)pBuffer);

	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}
		
		m_dwPutSize = *(LPDWORD)pBuffer;		
		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpCommon::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{
	if(!m_pSession)return NULL;

	return NULL;
}


//*****************************************************************************
//*
//*		SetCurrentDirectory
//*
//*****************************************************************************
//	Stellt das aktuelle Verzeichnis ein.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpCommon::SetCurrentDirectory(LPCTSTR pDirName)
{
int		iErr;


	if(!m_pSession)return FALSE;

	iErr = AzFtpCwd((AzFtpHandle)m_pSession,pDirName);

	if(iErr/100==2)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis.
//	pDirName		: Puffer fr den Namen
//	pLen			: Gre des Puffers
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpCommon::GetCurrentDirectory(LPTSTR pDirName,LPDWORD pLen) const
{
int		iErr;
int		iLen;


	if(!m_pSession)return FALSE;

	iLen  = *pLen;
	iErr  = AzFtpPwd((AzFtpHandle)m_pSession,pDirName,iLen);

	if(iErr/100!=2)
		{
		SetError(iErr);
		return FALSE;
		}

	*pLen = _tcslen(pDirName);



return TRUE;
}


//*****************************************************************************
//*
//*		RemoveDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des zu lschenden Verzeichnisses
//	Ergibt TRUE wenn das Verzeichnis gelscht wurde.
BOOL CFtpCommon::RemoveDirectory(LPCTSTR pDirName)
{
int		iErr;


	if(!m_pSession)return FALSE;

	iErr = AzFtpRmDir((AzFtpHandle)m_pSession,pDirName);

	if(iErr/100==2)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		CreateDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpCommon::CreateDirectory(LPCTSTR pDirName)
{
int		iErr;


	if(!m_pSession)return FALSE;

	iErr = AzFtpMkDir((AzFtpHandle)m_pSession,pDirName);

	if(iErr/100==2)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		Remove
//*
//*****************************************************************************
//	Lscht eine Datei..
//	pFileName		: Ist der Name der Datei
//	Ergibt TRUE wenn die Datei gelscht wurde.
BOOL CFtpCommon::Remove(LPCTSTR pFileName)
{
int		iErr;


	if(!m_pSession)return FALSE;

	iErr = AzFtpRemove((AzFtpHandle)m_pSession,pFileName);

	if(iErr/100==2)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	Verschiebt eine Datei..
//	pExisting		: Ist der Name der Datei.
//	pNewName		: Ist der Name der Zieldatei.
//	Ergibt TRUE wenn die Datei vershoben wurde.
BOOL CFtpCommon::Rename(LPCTSTR pExisting,LPCTSTR pNewName)
{
int		iErr;


	if(!m_pSession)return FALSE;

	iErr = AzFtpRename((AzFtpHandle)m_pSession,pExisting,pNewName);

	if(iErr/100==2)return TRUE;

	SetError(iErr);


return FALSE;
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
void CFtpCommon::Close()
{

	if(m_pSession)
		{
		AzFtpClose((AzFtpHandle)m_pSession,100);
		m_pSession=0;
		}

}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertgt eine Datei zum FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	pRemoteFile		: Name der Datei am FTP-Server.
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpCommon::PutFile(LPCTSTR pLocalFile,LPCTSTR pRemoteFile,DWORD dwFlags,DWORD dwContext)
{
int		iFlag;
int		iErr;


	if(!m_pSession)return FALSE;

	iFlag = 0;

Retry:

	m_bAbortAllowed = 1;
	iErr = AzFtpPutFile((AzFtpHandle)m_pSession,pRemoteFile,pLocalFile);
	m_bAbortAllowed = 0;

	if(m_bRetry && !iFlag && !m_bBreak && (iErr==984 || iErr==991 || iErr==996))
		{
		iFlag = 1;
		goto Retry;
		}

	if(iErr==226)return TRUE;

	SetError(iErr);

	m_bBreak = FALSE;


return FALSE;
}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpCommon::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{
int		iFlag;
int		iErr;


	if(!m_pSession)return FALSE;

	iFlag = 0;

Retry:

	if(m_dwPutSize)
		{
		AzFtpSetSize((AzFtpHandle)m_pSession,m_dwPutSize);
		}

	m_bAbortAllowed = 1;
	iErr = AzFtpPutProc((AzFtpHandle)m_pSession,pRemoteFile,pParam,pProc);
	m_bAbortAllowed = 0;

	if(m_bRetry && !iFlag && !m_bBreak && (iErr==984 || iErr==991 || iErr==996))
		{
		iFlag = 1;
		goto Retry;
		}

	if(iErr==226)return TRUE;

	SetError(iErr);

	m_bBreak	= FALSE;
	m_dwPutSize	= 0;


return FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei vom FTP-Server.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	bFailIfExists	:
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpCommon::GetFile(LPCTSTR pRemoteFile,LPCTSTR pLocalFile,BOOL bFailIfExists,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
int		iFlag;
int		iErr;


	if(!m_pSession)return FALSE;

	if(bFailIfExists && ExistFile(pLocalFile))
		{
		SetLastError(ERROR_FILE_EXISTS);
		return FALSE;
		}

	iFlag = 0;

Retry:

	m_bAbortAllowed = 1;
	iErr = AzFtpGetFile((AzFtpHandle)m_pSession,pRemoteFile,pLocalFile);
	m_bAbortAllowed = 0;

	if(m_bRetry && !iFlag && !m_bBreak && (iErr==984 || iErr==991 || iErr==996))
		{
		iFlag = 1;
		goto Retry;
		}


	if(iErr==226)return TRUE;

	SetError(iErr);

	m_bBreak = FALSE;


return FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpCommon::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
int		iFlag;
int		iErr;


	if(!m_pSession)return FALSE;

	iFlag = 0;

Retry:

	m_bAbortAllowed = 1;
	iErr = AzFtpGetProc((AzFtpHandle)m_pSession,pRemoteFile,pParam,pProc);
	m_bAbortAllowed = 0;

	if(m_bRetry && !iFlag && !m_bBreak && (iErr==984 || iErr==991 || iErr==996))
		{
		iFlag = 1;
		goto Retry;
		}


	if(iErr==226)return TRUE;

	SetError(iErr);

	m_bBreak = FALSE;


return FALSE;
}

//*****************************************************************************
//*
//*		List
//*
//*****************************************************************************
//	Liste alle Dateien in einem Verzeichnis via Callback auf.
//	pRemotePath		: Ist der Name des Verzeichnisses.
//	pParam			: Parameter fr den Callback.
//	pProc			: Zeiger auf den Callback.
//	pFilter			: Filter der benutzt werden kann.
//	Ergibt wenn TRUE alles ok war, bzw. FALSE bei einem Fehler.
BOOL CFtpCommon::List(LPCTSTR pRemotePath,void *pParam,FtpFindCallback pProc,LPCTSTR pFilter)
{
FilterCbData	sData;
int				iErr;


	if(!m_pSession)
		{
		pProc(pParam,NULL);
		return FALSE;
		}

	m_bAbortAllowed = 1;

	sData.pProc		= pProc;
	sData.pParam	= pParam;
	sData.pFilter	= pFilter;

	if(pFilter)
			iErr = AzFtpList((AzFtpHandle)m_pSession,pRemotePath,&sData,FilterCallback);
	else	iErr = AzFtpList((AzFtpHandle)m_pSession,pRemotePath,pParam,pProc);

	m_bAbortAllowed = 0;


	if(iErr==226)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		SetAttributes
//*
//*****************************************************************************
//	Einstellen der Dateiattribute
//	pFileName		: Ist der Name der Datei.
//	dwAttr			: Sind die neuen Attribute.
//					  In den Bits 16..31 knnen Unix-Attribute verpackt werden.
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpCommon::SetAttributes(LPCTSTR pFileName,DWORD dwAttr)
{
int		iAttr;
int		iErr;


	if(!m_pSession)return FALSE;


	if(dwAttr&0xFFFF0000)									// UNIX Attribute vorhanden
		{
		iAttr = dwAttr>>16;
		}
	else{
		iAttr = (dwAttr&FILE_ATTRIBUTE_READONLY)? 0x0120:0x01B4;
		if(dwAttr&FILE_ATTRIBUTE_DIRECTORY)iAttr|=0x0200;
		if(dwAttr&FILE_ATTRIBUTE_HIDDEN   )iAttr|=0x0001;	// Samba-Hack
		if(dwAttr&FILE_ATTRIBUTE_SYSTEM   )iAttr|=0x0008;
		if(dwAttr&FILE_ATTRIBUTE_ARCHIVE  )iAttr|=0x0040;
		if(dwAttr&FILE_ATTRIBUTE_EXEC	  )iAttr|=0x0049;
		}

	iErr = AzFtpChmod((AzFtpHandle)m_pSession,pFileName,iAttr);

	if(iErr==200)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		SetModifyTime
//*
//*****************************************************************************
//	Einstellen der nderungszeit einer Datei.
//	pFileName		: Ist der Name der Datei.
//	sFileTime		: Ist die neue nderungszeit
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpCommon::SetModifyTime(LPCTSTR pFileName,FILETIME sFileTime)
{
CTime	sTime(sFileTime,0);
time_t	iTime;
int		iErr;


	if(!m_pSession)return FALSE;

	iTime = sTime.GetTime();
	iErr  = AzFtpSetTime((AzFtpHandle)m_pSession,pFileName,iTime);

	if(iErr/100==2)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		DoAbort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpCommon::DoAbort()
{

	if(!m_pSession)return;

	if(AzFtpAbort((AzFtpHandle)m_pSession))
		{
		m_bAbortAllowed = 0;
		m_bBreak = TRUE;
		}

}

//*****************************************************************************
//*
//*		SetError
//*
//*****************************************************************************
//	Stellt die Windows Fehlernummer ein
//	iError		: Ist die FTP Fehlernummer
void CFtpCommon::SetError(int iError) const
{

	switch(iError)
		{
	case 503:
	case 500:
	case 501:	SetLastError(ERROR_INVALID_PARAMETER);			return;
	case 421:
	case 502:
	case 504:	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);		return;
	case 425:	SetLastError(ERROR_OPEN_FAILED);				return;
	case 426:	SetLastError(ERROR_GRACEFUL_DISCONNECT);		return;
	case 332:
	case 532:
	case 530:	SetLastError(ERROR_LOGON_FAILURE);				return;
	case 331:	SetLastError(ERROR_WRONG_PASSWORD);				return;
	case 550:
	case 553:
	case 452:
	case 450:	SetLastError(ERROR_INVALID_ACCESS);				return;
	case 451:
	case 551:	SetLastError(ERROR_OPERATION_ABORTED);			return;
	case 552:	SetLastError(ERROR_DISK_FULL);					return;
	case 997:	SetLastError(ERROR_GRACEFUL_DISCONNECT);		return;
	case 996:	SetLastError(ERROR_TIMEOUT);					return;
	case 987:
	case 998:	SetLastError(ERROR_WRITE_FAULT);				return;
	case 999:	SetLastError(ERROR_BAD_NETPATH);				return;
	case 995:	SetLastError(ERROR_HOST_UNREACHABLE);			return;
	case 994:	SetLastError(ERROR_FILE_NOT_FOUND);				return;
	case 993:	SetLastError(ERROR_CANNOT_MAKE);				return;
	case 992:	SetLastError(ERROR_PROTOCOL_UNREACHABLE);		return;
	case 986:	SetLastError(ERROR_INVALID_TIME);				return;
	case 989:	SetLastError(ERROR_READ_FAULT);					return;
	case 988:	SetLastError(ERROR_NET_WRITE_FAULT);			return;
	case 985:	SetLastError(ERROR_OUTOFMEMORY);				return;
	case 984:	SetLastError(ERROR_REQ_NOT_ACCEP);				return;
	case 991:	SetLastError(ERROR_REQ_NOT_ACCEP);				return;
		}

	SetLastError(ERROR_INVALID_ACCESS);


}


