//*****************************************************************************
//*
//*
//*		FtpConsole.cpp
//*
//*
//*****************************************************************************
//
//	Ist eine Security-FTP-Verbindung ber SSH (Konsole)
//
#include	<Afx.h>
#include	<stddef.h>
#include	"FtpTools.h"
#include	"FtpConsole.h"
#include	"System/FileUtility.h"
#include	"../BitFunctions.h"
#include	"../MemFunctions.h"
#include	"../FileFunctions.h"
#include	"../StringFunctions.h"
#include	"../Tools/AzCommLib/AzCommDll.h"

#ifndef		SSH_LGGING_ON
#define 	SSH_LGGING_ON	0
#endif 	

#define 	SSH_TIMEOUT		255

typedef	struct
	{
	WIN32_FIND_DATA	sFind;
	LPVOID			pNext;
	}LinkEntry;


BYTE	CFtpConsole::m_aHexOut[] =						// Daten fr HexOut Programm
	{
	#include 	"FtpHexOut.h"
	};


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpConsole::CFtpConsole(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive) : CFtpBase(pSession,pServer,pUserName,pPassword,dwContext,nPort,bPassive)
{

	m_pBaseUrl			= _T("ssh://");
	m_bUseHexOut		= bPassive;
	m_uTimeout	  		= 10000;
	m_bReconnect  		= FALSE;
	m_bUserBreak		= FALSE;
	m_bConnDone	  		= FALSE;
	m_bUseUtf8	  		= FALSE;
	m_pLsText			= "-la";
	m_uLsMode			= 0;
//	m_cCurrentPath[0]	= 0;
	m_cPrefix [0]		= 0;			
	m_cCurrDir[0] 		= 0;
	m_uProgressOff		= 0;
	m_uProgressSize 	= 0;
	m_uHexOutVesion		= 0;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpConsole::~CFtpConsole()
{

	if(m_pSession)Close();

}


//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpConsole::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{
DWORD	   *pVal=(DWORD*)pBuffer;
DWORD		uLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	uLen = *pLen;


	switch(dwOption)
		{
	case INTERNET_OPTION_MODE:						// Verbindungsmodus

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_SFTP;

		return TRUE;

	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:		// Timeout
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLen<sizeof(m_uTimeout))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		*pVal = m_uTimeout;
		*pLen = sizeof(m_uTimeout);

		return TRUE;

	case INTERNET_OPTION_PROGRESS:					// Verlaufsanzeige

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_uProgressOff;

		return TRUE;

	case INTERNET_OPTION_PROGRESS2:					// Verlaufsanzeige mit Maximalwert

		if(uLen<sizeof(DWORD)*2)
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_uProgressOff;
		pVal[1] = m_uProgressSize;

		return TRUE;

	case INTERNET_OPTION_UTF8:						// UTF8 Konvertierung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_bUseUtf8;

		return TRUE;

	case INTERNET_OPTION_CONNTYPE:					// Art der Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_SFTP;

		return TRUE;

	case INTERNET_OPTION_URLFORMAT:					// Format des URL-Namens

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 0;

		return TRUE;

	case INTERNET_OPTION_ERROPEN:					// Wo gab es den Fehler

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_dwErrOpen;

		return TRUE;

	case INTERNET_OPTION_ONLYDIR:					// Knnen nur Verzeichnisse aufgelistet werden

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_uHexOutVesion;

		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpConsole::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{
int		iCount;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	switch(dwOption)
		{
	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLength<sizeof(m_uTimeout))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_uTimeout = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_PROGRSET:					// Verlaufsanzeige setzen

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_uProgressOff = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_UTF8:						// UTF8 Konvertierung

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_bUseUtf8 = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_RECONNECT:					// Neuverbinden

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}

		iCount = *(int*)pBuffer;

		for(;;)
			{
			if(Reconnect(TRUE))return TRUE;
			iCount--;
			if(iCount<=0)break;
			Sleep(500);
			}

		return FALSE;

	case INTERNET_OPTION_SETSTAT:					// Attribute und Zeiten setzen

		if(uLength<sizeof(WIN32_FIND_DATA))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return SetStat((LPWIN32_FIND_DATA)pBuffer);

	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}
		
		m_dwPutSize = *(LPDWORD)pBuffer;		
		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpConsole::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{
	if(!m_pSession)return NULL;

	return NULL;
}

//*****************************************************************************
//*
//*		Reconnect
//*
//*****************************************************************************
//	Erzeugt wieder eine Verbindung falls die alte abgebochen ist.
//	bForce		: Immer neu verbinden
BOOL CFtpConsole::Reconnect(BOOL bForce)
{
char	cServer[64];
char	cPaswd [64];
char	cUser  [64];
int		iErr;
int		iPos;	


	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	if(!bForce && !m_bReconnect)return TRUE;

	if(!bForce && SshIsConnected(m_pSession))
		{
		return TRUE;
		}

	strawcpy(cServer,m_sServerName,sizeof(cServer));
	strawcpy(cPaswd ,m_sPassword  ,sizeof(cPaswd ));
	strawcpy(cUser  ,m_sUserName  ,sizeof(cUser  ));

	if(m_bConnDone)
		{
		if(m_pSession)SshClose(m_pSession);

		m_bConnDone = FALSE;
		m_pSession  = SshCreate();

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}
		}

	for(iPos=0;cUser[iPos];iPos++)					// Superuser-Weiterleitung entfernen
		{
		if(cUser[iPos]!='|')continue;
		cUser[iPos] = 0;
		break;
		}

	
	m_bConnDone = TRUE;

	iErr = SshConnect(m_pSession,cServer,m_wPort,cUser,cPaswd,m_uTimeout);

	if(iErr==SSH2_FX_ALLREADY_CONNECTED)
		{
		SshBreak(m_pSession);

		if(m_pSession)SshClose(m_pSession);

		m_bConnDone = FALSE;
		m_pSession  = SshCreate();

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}

		iErr = SshConnect(m_pSession,cServer,m_wPort,cUser,cPaswd,m_uTimeout);
		}

	if(iErr==SSH2_FX_OK)							// Alles ok
		{
		SuLogin();
		return TRUE;
		}


	SetError(iErr);



return FALSE;
}


//*****************************************************************************
//*
//*		HextOutText
//*
//*****************************************************************************
//	Wandelt eine Pfad fr das "hexout" Programm um (*?- ;=)
//	pBuffer		: Zeiger auf Textpuffer
//	pPath		: Ist der Quellpfad der Umgewandelt werden soll
//	iSize		: Ist die Gre des Textpuffers
//	Ergibt die Lnge des umgewandelten Texte.
int	CFtpConsole::HextOutText(char *pBuffer,const char *pPath,int iSize)
{
char	cSign;
int		iPos;

	
	if(iSize<=1)return 0;

	iSize -= 2;

	for(iPos=0;iPos<iSize;iPos++)
		{
		if(!*pPath)break;

		switch(*pPath)
			{
		case '*':	cSign='=';	break;
		case '?':	cSign='+';	break;
		case '-':	cSign='-';	break;
		case ' ':	cSign='_';	break;
		case ';':	cSign='#';	break;
		case '=':	cSign='$';	break;
		case '"':	cSign='&';	break;

		default:
			
			pBuffer[iPos] = *pPath;
			pPath++;
			continue;
			}

		pBuffer[iPos] = '='; iPos++;
		pBuffer[iPos] = cSign;
		
		pPath++;
		}

	pBuffer[iPos] = 0;


return iPos;
}

//*****************************************************************************
//*
//*		SetCurrentDirectory
//*
//*****************************************************************************
//	Stellt das aktuelle Verzeichnis ein.
//	pDirName		: Ist der Name des neuen Verzeichnisses.
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpConsole::SetCurrentDirectory(LPCTSTR pDirName)
{
char 		cBuffer[CFB_MAX_PATH];
char 		cData[512];
int			iAdd;
int			iErr;
int			iLen;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer)-8);
	PathConvert (cBuffer,'/');
	DelBackSlash(cBuffer);

	if(!cBuffer[0])
		{
		cBuffer[0] = '/';
		cBuffer[1] = 0;
		}


	ClearBuffer();

	iAdd = (m_cPrefix[0])? 0:1;

	   iErr = LineCommand(cBuffer,cData,sizeof(cData),"\\ls -ald "+iAdd,"\"");
	if(iErr<0)return FALSE;


	   iLen = strwlen(cData,0x02000007);	
	if(iLen!=10 || cData[0]!='d')
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}


	if(m_bUseUtf8)
			strfromutf8(m_cCurrDir,cBuffer,sizeof(m_cCurrDir)/sizeof(TCHAR));
	else	strawcpy   (m_cCurrDir,cBuffer,sizeof(m_cCurrDir)/sizeof(TCHAR));



return TRUE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis.
//	pDirName		: Puffer fr den Namen
//	pLen			: Gre des Puffers
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpConsole::GetCurrentDirectory(LPTSTR pDirName,LPDWORD pLen) const
{


	if(!m_pSession)return FALSE;

	*pLen = strlcpy(pDirName,m_cCurrDir,*pLen);



return TRUE;
}


//*****************************************************************************
//*
//*		RemoveDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des zu lschenden Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis gelscht wurde.
BOOL CFtpConsole::RemoveDirectory(LPCTSTR pDirName)
{
char 	cBuffer[CFB_MAX_PATH];
char 	cData[512];
int		iErr;
int		iLen;


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = LineCommand(cBuffer,cData,sizeof(cData),"rmdir ","\"");

	if(iErr     <0)return FALSE;
	if(cData[0]==0)return TRUE;


	switch(GetLastError())
		{
	case ERROR_FILE_NOT_FOUND:	SetLastError(ERROR_PATH_NOT_FOUND);		break;
	case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
		}


	SetLastError(ERROR_ACCESS_DENIED);	



return FALSE;
}

//*****************************************************************************
//*
//*		CreateDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpConsole::CreateDirectory(LPCTSTR pDirName)
{
char 	cBuffer[CFB_MAX_PATH];
char 	cData[512];
int		iErr;
int		iLen;


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = LineCommand(cBuffer,cData,sizeof(cData),"mkdir ","\"");

	if(iErr     <0)return FALSE;
	if(cData[0]==0)return TRUE;

	switch(GetLastError())
		{
	case ERROR_FILE_NOT_FOUND:	SetLastError(ERROR_PATH_NOT_FOUND);		break;
	case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
		}



return FALSE;
}

//*****************************************************************************
//*
//*		GetStat
//*
//*****************************************************************************
//	Die Attibute einer Datei abfragen
//	pFileName	: Ist der Dateiname
//	pInfo		: Hier werden die Attribute gespeichert
//	Ergibt 0 bei einer Datei, 1 bei einem Verzeichnis, bzw. den Offset
//	des Link-Namens und -1 bei einem Fehler.
int CFtpConsole::GetStat(LPCTSTR pFileName,LPWIN32_FIND_DATA pInfo)
{
char 	cBuffer[CFB_MAX_PATH];
char 	cData[512];
int		iErr;
int		iLen;


	if(!Reconnect())return -1;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);


	if(m_uHexOutVesion)
		{
		iLen = strlcpy(cData,"hexout -l ",sizeof(cData));
		HextOutText(cData+iLen,cBuffer   ,sizeof(cData)-iLen);		
		iErr = LineCommand(cData , cData ,sizeof(cData));
		}
	else{	
		sprintf(cData,"%sls %sd ",m_cPrefix,m_pLsText);
		iErr = LineCommand(cBuffer,cData,sizeof(cData),cData,"\"");
		}


	if(iErr<0)return -1;

	iLen = LineToAttrib(cData,pInfo);

	if(iLen>=0)return iLen;

	switch(GetLastError())
		{
	case ERROR_FILE_NOT_FOUND:	SetLastError(ERROR_PATH_NOT_FOUND);		break;
	case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
		}



return -1;
}

//*****************************************************************************
//*
//*		SetStat
//*
//*****************************************************************************
//	Einstellen der Dateiattribute und der Zeiten.
//	pInfo		: Sind die neuen Zeiten und Attribute
//				  Wenn pInfo->dwFileAttributes ungleich 0 und -1 ist werden die
//				  Attribute	verndert. Zie Zeiten werden gesetzt wenn sie ungleich
//				  0 sind. In pInfo->cFileName wird der Dateinamge bergeben.
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpConsole::SetStat(LPWIN32_FIND_DATA pInfo)
{
BOOL		bOkTime;
BOOL		bOkAttr;



	if(!m_pSession)return FALSE;

	if(!Reconnect())return FALSE;



	if(pInfo->ftLastWriteTime.dwHighDateTime)			// Zeiten setzen
		{
		bOkTime = SetModifyTime(pInfo->cFileName,pInfo->ftLastWriteTime);
		}
	else{
		bOkTime = TRUE;
		}

														// Attribute setzen
	if(pInfo->dwFileAttributes && pInfo->dwFileAttributes!=0xFFFFFFFF)
		{
		bOkAttr = SetAttributes(pInfo->cFileName,pInfo->dwFileAttributes);
		}
	else{
		bOkAttr = TRUE;
		}


return bOkAttr && bOkTime;
}

//*****************************************************************************
//*
//*		TruncFile
//*
//*****************************************************************************
//	Krtzt eine Datei auf eine bestimmte Gre. Wenn die Datei bereits
//	krze ist wird ein Fehler ausgegeben.
//	pFileName	: Ist der Dateiname
//	iSize		: Ist die neue Dateigre
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpConsole::TruncFile(LPCTSTR pFileName,int iSize)
{
char 			cBuffer[CFB_MAX_PATH];
char 			cData[1024];
WIN32_FIND_DATA sInfo;
int				iErr;
int				iLen;
int				iNum;


	
	if(iSize<0)												// Prfe die Gre
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);


	iNum = sprintf(cData,"perl -we \"my \\$ff_x;open(\\$ff_x,\\\"+<\\\",\\\"%s\\\") && truncate(\\$ff_x,%i) && close(\\$ff_x);\"",cBuffer,iSize);
	iErr = LineCommand(cData,cData,sizeof(cData));

	if(iErr<0)
		{
		return FALSE;
		}

	if(cData[0])
		{
		switch(GetLastError())
			{
		case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
			}

		return FALSE;
		}

	   iErr = GetStat(cBuffer,&sInfo);
	if(iErr<0)
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		return FALSE;
		}

	if(sInfo.nFileSizeLow^iSize)							// Prfe die Dateigre
		{
		SetLastError(ERROR_HANDLE_EOF);
		return FALSE;
		}


return TRUE;
}


//*****************************************************************************
//*
//*		Remove
//*
//*****************************************************************************
//	Lscht eine Datei..
//	pFileName		: Ist der Name der Datei
//	Ergibt TRUE wenn die Datei gelscht wurde.
BOOL CFtpConsole::Remove(LPCTSTR pFileName)
{
char 	cBuffer[CFB_MAX_PATH];
char 	cData[512];
int		iErr;
int		iLen;


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = LineCommand(cBuffer,cData,sizeof(cData),"rm -f ","\"");

	if(iErr     <0)return FALSE;
	if(cData[0]==0)return TRUE;


	switch(GetLastError())
		{
	case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
		}


return FALSE;
}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	Verschiebt eine Datei..
//	pExisting		: Ist der Name der Datei.
//	pNewName		: Ist der Name der Zieldatei.
//	Ergibt TRUE wenn die Datei vershoben wurde.
BOOL CFtpConsole::Rename(LPCTSTR pExisting,LPCTSTR pNewName)
{
char 			cBuffer[CFB_MAX_PATH*2];
char 			cData[512];
WIN32_FIND_DATA sInfo;
int				iLast;
int				iErr;
int				iLen;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pExisting,sizeof(cBuffer)/2);
	iLen = PathConvert(cBuffer,'/');

	   iErr = GetStat(cBuffer,&sInfo);
	if(iErr<0)
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		return FALSE;
		}

	cBuffer[iLen] = '"';iLen++;
	cBuffer[iLen] = ' ';iLen++;
	cBuffer[iLen] = '"';iLen++;
	

	PathFullMake(cBuffer+iLen,m_cCurrDir,pNewName,sizeof(cBuffer)-iLen);
	PathConvert (cBuffer+iLen,'/');

	   iErr = GetStat(cBuffer,&sInfo);						// Existiert die neuen Datei bereits
	if(iErr>=0)
		{
		SetLastError(ERROR_FILE_EXISTS);
		return FALSE;
		}


	iErr = LineCommand(cBuffer,cData,sizeof(cData),"mv -f ","\"");

	if(iErr     <0)return FALSE;
	if(cData[0]==0)return TRUE;

	
	iLast = GetLastError();


	   iErr = GetStat(pNewName,&sInfo);
	if(iErr>=0)												// Existiert die neuen Datei
		{
		SetLastError(ERROR_SUCCESS);
		return TRUE;
		}

	if(iLast==ERROR_SUCCESS)
			SetLastError(ERROR_ACCESS_DENIED);
	else	SetLastError(iLast);
		



return FALSE;
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
void CFtpConsole::Close()
{

	if(!m_pSession)return;

	SshClose(m_pSession);

	m_pSession	  = NULL;
	m_bReconnect  = FALSE;
	m_bConnDone	  = FALSE;

}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertgt eine Datei zum FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	pRemoteFile		: Name der Datei am FTP-Server.
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpConsole::PutFile(LPCTSTR pLocalFile,LPCTSTR pRemoteFile,DWORD dwFlags,DWORD dwContext)
{
char 			cBuffer[CFB_MAX_PATH];
char 			cData[1600];
char 			cTemp[256];
char 			cCode[20];
static unsigned uCounter;
WIN32_FIND_DATA sInfo;
FILE		   *pFile;
int				iFlag;
int				iLine;
int				iSize;
int				iEnd;
int				iErr;
int				iRet;
int				iLen;
int				iAdd;
int				iNum;
int				iPos;
int				iCnt;


	m_uProgressSize	= 0;
	m_uProgressOff	= 0;
	m_bUserBreak	= 0;

	if(!Reconnect())return FALSE;

	   iRet = LengthFile(pLocalFile);
	if(iRet<0)
		{
		SetLastError((iRet)? ERROR_ACCESS_DENIED:ERROR_FILE_NOT_FOUND);
		m_dwErrOpen = 1;
		return FALSE;
		}

	m_uProgressSize = iRet;
	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	m_dwErrOpen = 2;

		pFile = _tfopen(pLocalFile,"rb");
	if(!pFile)
		{
		return FALSE;
		}

	
	m_bAbortAllowed = 1;
	iCnt = 0;


//******************** Ausgabe mit hexout *************************************


	if(m_uHexOutVesion)
		{
		uCounter++;	

		iLen  = sprintf(cCode,"aacc%08Xggtt",uCounter);
		iLen  = sprintf    (cData,"echo %s;hexout -w -e ",cCode);
		iLen += HextOutText(cData+iLen,cBuffer,sizeof(cData)-128-iLen);		
		iLen += sprintf    (cData+iLen,"\n");

		ClearBuffer();

		   iErr = SshWrite(m_pSession,cData,iLen);		// Kommandos senden
		if(iErr!=iLen)
			{
			fclose(pFile);
			SetError(iErr);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return FALSE;
			}
		
		iEnd	=-1;
		iErr	= 0;
		iPos	= 0;
		iFlag	= 0;

		m_bAbortAllowed	= TRUE;


		for(;;)											// Hex-Dump herunterladen
			{
			if((iFlag!=2 || iCnt>64) && !SshWait(m_pSession,m_uTimeout,1))
				{
				if(m_bUserBreak){iErr=0;iPos=iEnd;break;}
				fclose(pFile);
				SetLastError(ERROR_TIMEOUT);
				m_bAbortAllowed	= FALSE;
				m_bReconnect    = TRUE;
				return FALSE;
				}

			iSize = SshReadCount(m_pSession,&iLine);	// Lnge der nchsten empfangenen Zeile holen

			if(iLine>=sizeof(cData))
				{
				iLine = sizeof(cData)-1;
				}

			if(iLine>0)
				{
				iSize = SshRead(m_pSession,cData,iLine);// Nchste Zeile einlesen
				if(iCnt>0)iCnt--;

				#if SSH_LGGING_ON
				printf("%.*s",iSize,cData);
				#endif 	

				if(iSize<0)
					{
					fclose(pFile);
					SetError(iSize);
					m_bReconnect= TRUE;
					break;
					}

				cData[iSize] = 0;
				iSize = strcut(cData);
				}
			else{
				iSize    = 0;	
				cData[0] = 0;
				}

			if(m_bUserBreak)							// Gab es einen User-Abbruch
				{
				iFlag = 5;
				break;
				}

			if(iFlag==0)								// Anfangstag empfangen
				{
				if(iSize>=16 && !memcmp(cData,cCode,16))
					{
					iFlag = 1;
					}

				continue;
				}
			
			if(iFlag==1)								// Startlnge von 'hexout' empfangen
				{
				if(iSize>=4 && !memcmp(cData,"LEN<",4))
					{
					iPos  = atoi(cData+4);
					iFlag = 2;
					}

				continue;
				}
			
			if(iFlag>=2)								// Startlnge von 'hexout' empfangen
				{
				if(iFlag<=3 && iSize>=4 && !memcmp(cData,"END<",4))
					{
					iEnd  = atoi(cData+4);
					iFlag = 4;

					if(iEnd<0)
						{
						iErr = DecodeError(iEnd);
						iEnd = -1;
						}
					
					break;
					}

				if(iFlag>=3)continue;
				}

														// Daten aus Datei einlesen
			   iAdd = fread(cTemp,1,sizeof(cTemp),pFile);
			if(iAdd<0)
				{
				SetLastError(ERROR_WRITE_FAULT);
				iErr  = -1;
				break;
				}
			else if(iAdd==0)
				{
				memcpy(cData,">STOP<\n",8);
				iLen  = 7;
				iFlag = 3;
				}
			else{
				iCnt += 1;
				iPos += iAdd;
				iLen  = memtobase64(cTemp,iAdd,cData+1,0)+2;
				cData[0     ] = '>';
				cData[iLen-1] = '\n';
				}

			   iErr = SshWrite(m_pSession,cData,iLen);	// Daten als Base64-Text bertragen
			if(iErr!=iLen)
				{
				m_bReconnect= TRUE;
				SetError(iErr);
				break;
				}

			m_uProgressOff += iAdd;
			iErr			= 0;
			}


		m_bAbortAllowed	= FALSE;

		if(iErr)										// Gab es Fehler
			{
			fclose(pFile);
			return FALSE;
			}

		if(iPos!=iEnd)									// Wurde alles bertragen
			{
			fclose(pFile);
			SetLastError(ERROR_HANDLE_EOF);
			return FALSE;
			}

		cData[0] = 0;

		goto Done;
		}
	


//******************** Ausgabe mit echo ***************************************


	for(iPos=0;;iPos+=iLen)								// Die Datei bertragen
		{
		if(m_bUserBreak)break;

		   iLen = fread(cTemp,1,sizeof(cTemp),pFile);	// Daten aus Datei einlesen
		if(iLen<=0)break;

		iNum  = strlcpy(cData,"echo -ne \"",sizeof(cData));
		iNum += MemToText(cTemp,cData+iNum ,iLen);
		iNum += strlcpy(cData+iNum,"\" >>",sizeof(cData)-iNum,(iPos)? 4:3);

		iErr  = LineCommand(cBuffer,cData,sizeof(cData),cData,"\"");

		if(iErr || cData[0]!=0)							// Gab es einen Fehler
			{
			if(iCnt==0)
				{
				SshBreak(m_pSession);
				m_bReconnect = TRUE;

				if(TruncFile(cBuffer,iPos))				// Datei krzen und weiter machen
					{
					fseek(pFile,iPos,SEEK_SET);
					iCnt = 16;
					continue;
					}
				}

			break;
			}
		
		m_uProgressOff += iLen;
		if(iCnt>0)iCnt--;
		}

	m_bAbortAllowed = 0;


//******************** Ende ***************************************************


Done:

	fclose(pFile);

	
	if(m_bUserBreak)									// Gab es einen User-Abbruch
		{
		SetLastError(ERROR_CANCELLED);
		return FALSE;
		}

	if(iErr<0)
		{
		return FALSE;
		}

	if(cData[0])
		{
		switch(GetLastError())
			{
		case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
			}
		
		return FALSE;
		}

	if(GetStat(cBuffer,&sInfo)<0)						// Dateigre einlesen	
		{
		return FALSE;
		}

	if(sInfo.nFileSizeLow^iRet)							// Prfe die Dateigre
		{
		SetLastError(ERROR_HANDLE_EOF);
		return FALSE;
		}



return TRUE;
}


//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpConsole::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{
char 			cBuffer[CFB_MAX_PATH];
char 			cData[1600];
char 			cTemp[256];
char 			cCode[20];
static unsigned uCounter;
WIN32_FIND_DATA sInfo;
int				iFlag;
int				iSize;
int				iLine;
int				iAdd;
int				iErr;
int				iEnd;
int				iRet;
int				iLen;
int				iNum;
int				iPos;
int				iCnt;


	m_uProgressSize	= 0;
	m_uProgressOff	= 0;
	m_bUserBreak	= 0;

	if(!Reconnect())
		{
		return FALSE;
		}


	m_uProgressSize = m_dwPutSize;
	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	m_dwErrOpen = 2;

	m_bAbortAllowed = 1;
	iRet = m_dwPutSize;
	iCnt = 0;
	iLen = 0;


//******************** Ausgabe mit hexout *************************************


	if(m_uHexOutVesion)
		{
		uCounter++;	

		iLen  = sprintf(cCode,"aacc%08Xggtt",uCounter);
		iLen  = sprintf    (cData,"echo %s;hexout -w -e ",cCode);
		iLen += HextOutText(cData+iLen,cBuffer,sizeof(cData)-128-iLen);		
		iLen += sprintf    (cData+iLen,"\n");

		ClearBuffer();

		   iErr = SshWrite(m_pSession,cData,iLen);		// Kommandos senden
		if(iErr!=iLen)
			{
			SetError(iErr);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return FALSE;
			}
		
		iEnd	=-1;
		iErr	= 0;
		iPos	= 0;
		iFlag	= 0;

		m_bAbortAllowed	= TRUE;


		for(;;)											// Hex-Dump herunterladen
			{
			if((iFlag!=2 || iCnt>64) && !SshWait(m_pSession,m_uTimeout,1))
				{
				if(m_bUserBreak){iErr=0;iPos=iEnd;break;}
				SetLastError(ERROR_TIMEOUT);
				m_bAbortAllowed	= FALSE;
				m_bReconnect    = TRUE;
				return FALSE;
				}

			iSize = SshReadCount(m_pSession,&iLine);	// Lnge der nchsten empfangenen Zeile holen

			if(iLine>=sizeof(cData))
				{
				iLine = sizeof(cData)-1;
				}

			if(iLine>0)
				{
				iSize = SshRead(m_pSession,cData,iLine);// Nchste Zeile einlesen
				if(iCnt>0)iCnt--;

				#if SSH_LGGING_ON
				printf("%.*s",iSize,cData);
				#endif 	

				if(iSize<0)
					{
					SetError(iSize);
					m_bReconnect= TRUE;
					break;
					}

				cData[iSize] = 0;
				iSize = strcut(cData);
				}
			else{
				iSize    = 0;	
				cData[0] = 0;
				}

			if(m_bUserBreak)							// Gab es einen User-Abbruch
				{
				iFlag = 5;
				continue;
				}

			if(iFlag==0)								// Anfangstag empfangen
				{
				if(iSize>=16 && !memcmp(cData,cCode,16))
					{
					iFlag = 1;
					}

				continue;
				}
			
			if(iFlag==1)								// Startlnge von 'hexout' empfangen
				{
				if(iSize>=4 && !memcmp(cData,"LEN<",4))
					{
					iPos  = atoi(cData+4);
					iFlag = 2;
					}

				continue;
				}
			
			if(iFlag>=2)								// Startlnge von 'hexout' empfangen
				{
				if(iFlag<=3 && iSize>=4 && !memcmp(cData,"END<",4))
					{
					iEnd  = atoi(cData+4);
					iFlag = 4;

					if(iEnd<0)
						{
						iErr = DecodeError(iEnd);
						iEnd = -1;
						}
					
					break;
					}

				if(iFlag>=3)continue;
				}

														
			   iAdd = pProc(pParam,cTemp,sizeof(cTemp));// Daten fr Datei einlesen
			if(iAdd<0)
				{
				SetLastError(ERROR_WRITE_FAULT);
				iErr  = -1;
				break;
				}
			else if(iAdd==0)
				{
				memcpy(cData,">STOP<\n",8);
				iLen  = 7;
				iFlag = 3;
				}
			else{
				iCnt += 1;
				iPos += iAdd;
				iLen  = memtobase64(cTemp,iAdd,cData+1,0)+2;
				cData[0     ] = '>';
				cData[iLen-1] = '\n';
				}

			   iErr = SshWrite(m_pSession,cData,iLen);	// Daten als Base64-Text bertragen
			if(iErr!=iLen)
				{
				m_bReconnect= TRUE;
				SetError(iErr);
				break;
				}

			m_uProgressOff += iAdd;
			iErr			= 0;
			}


		m_bAbortAllowed	= FALSE;

		if(iErr)										// Gab es Fehler
			{
			return FALSE;
			}

		if(iPos!=iEnd)									// Wurde alles bertragen
			{
			SetLastError(ERROR_HANDLE_EOF);
			return FALSE;
			}

		cData[0] = 0;

		goto Done;
		}
	


//******************** Ausgabe mit echo ***************************************


	for(iPos=0;;iPos+=iLen)								// Die Datei bertragen
		{
		if(m_bUserBreak)break;

		   iLen = pProc(pParam,cTemp,sizeof(cTemp));
		if(iLen<=0)break;

Retry:	iNum  = strlcpy(cData,"echo -ne \"",sizeof(cData));
		iNum += MemToText(cTemp,cData+iNum ,iLen);
		iNum += strlcpy(cData+iNum,"\" >>",sizeof(cData)-iNum,(iPos)? 4:3);

		iErr  = LineCommand(cBuffer,cData,sizeof(cData),cData,"\"");

		if(iErr || cData[0]!=0)							// Gab es einen Fehler
			{
			if(iCnt==0)
				{
				SshBreak(m_pSession);
				m_bReconnect = TRUE;

				if(TruncFile(cBuffer,iPos))				// Datei krzen und weiter machen
					{
					iCnt = 16;
					goto Retry;
					}
				}

			break;
			}
		
		m_uProgressOff += iLen;
		if(iCnt>0)iCnt--;
		}


//******************** Ende ***************************************************

Done:


	m_bAbortAllowed = 0;
	m_dwPutSize     = 0;

	if(m_bUserBreak)									// Gab es einen User-Abbruch
		{
		SetLastError(ERROR_CANCELLED);
		return FALSE;
		}

	if(iErr<0)
		{
		return FALSE;
		}
	else if(iLen<0)										// Lesefehler bei Callback
		{
		SetLastError(ERROR_READ_FAULT);		
		return FALSE;
		}	

	if(cData[0])
		{
		switch(GetLastError())
			{
		case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
			}
		
		return FALSE;
		}

	if(GetStat(cBuffer,&sInfo)<0)						// Dateigre einlesen	
		{
		return FALSE;
		}

	if(iRet && (sInfo.nFileSizeLow^iRet))				// Prfe die Dateigre
		{
		SetLastError(ERROR_HANDLE_EOF);
		return FALSE;
		}



return TRUE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei vom FTP-Server.
//	pRemoteFile		: Name der Datei am FTP-Server
//	pLocalFile		: Name der lokalen Datei
//	bFailIfExists	: Einen Fehler zurck melden wenn die Datei existiert
//	dwFlags			: Unbenutzt
//	dwContext		: Unbenutzt
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpConsole::GetFile(LPCTSTR pRemoteFile,LPCTSTR pLocalFile,BOOL bFailIfExists,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
static unsigned uCounter;
char 			cBuffer[CFB_MAX_PATH];
char 			cData[1024];
char			cTemp[260];;
char 			cCode[20];
FILE		   *pFile; 
WIN32_FIND_DATA sInfo;
int				iSize;
int				iLine;
int				iByte;
int				iFlag;
int				iMax;
int				iOff;
int				iErr;
int				iLen;
int				iVal;
int 			iPos;
int 			iEnd;



	m_uProgressSize	= 0;
	m_uProgressOff	= 0;
	m_bUserBreak	= 0;


	if(bFailIfExists && ExistFile(pLocalFile))
		{
		SetLastError(ERROR_FILE_EXISTS);
		m_dwErrOpen = 1;
		return FALSE;
		}

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	
	if(GetStat(cBuffer,&sInfo)<0)						// Dateigre einlesen	
		{
		return FALSE;
		}

	
	if(sInfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)	// Ist die Datei ein Verzeichniss
		{
		SetLastError(ERROR_ACCESS_DENIED);
		return FALSE;
		}

	m_uProgressSize	= sInfo.nFileSizeLow;
	
	
		pFile = fopen(pLocalFile,"wb");
	if(!pFile)											// Lokale Datei erstellen
		{
		return FALSE;
		}

	if(sInfo.nFileSizeLow==0 && sInfo.nFileSizeHigh==0)	// Leere Datei
		{
		fclose(pFile);
		return TRUE;
		}

	uCounter++;	
	iLen = sprintf(cCode,"aadd%08Xggtt",uCounter);

	if(m_uHexOutVesion)									// Datei via "hexout" einlesen
		{
		iLen  = sprintf    (cData,"echo %s;hexout -r256 -e ",cCode);
		iLen += HextOutText(cData+iLen,cBuffer,sizeof(cData)-128-iLen);		
		iLen += sprintf    (cData+iLen,";echo ddoonnee\r\n");
		}
	else{												// Datei via "hexdump" einlesen
		iLen = sprintf(cData,"echo %s;hexdump -v -e '\"END<%%08_Ad>\\n\"' -e '256/1 \"%%02x\"' -e '\"\\n\"' %.400s;echo ddoonnee\r\n",cCode,cBuffer);
		}


	ClearBuffer();

	   iErr = SshWrite(m_pSession,cData,iLen);			// Kommandos senden
	if(iErr!=iLen)
		{
		fclose(pFile);
		SetError(iErr);
		return FALSE;
		}
	
	
	iMax	=-1;
	iEnd	=-1;
	iErr	= 0;
	iPos	= 0;
	iFlag	= 0;

	m_bAbortAllowed	= TRUE;


	for(;;)												// Hex-Dump herunterladen
		{
		if(!SshWait(m_pSession,m_uTimeout,1))
			{
			if(m_bUserBreak)break;
			fclose(pFile);
			SetLastError(ERROR_TIMEOUT);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return -SSH_TIMEOUT;
			}

		iSize = SshReadCount(m_pSession,&iLine);		// Lnge der nchsten empfangenen Zeile holen

		if(iLine>=sizeof(cData))
			{
			iLine = sizeof(cData)-1;
			}

		iSize = SshRead(m_pSession,cData,iLine);		// Nchste Zeile einlesen

		#if SSH_LGGING_ON
		printf("%.*s",iSize,cData);
		#endif 	

		if(iSize<0)
			{
			fclose(pFile);
			SetError(iSize);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return FALSE;
			}

		cData[iSize] = 0;
		iSize = strcut(cData);

		if(m_bUserBreak)								// Gab es einen User-Abbruch
			{
			break;
			}

		if(iSize>=8 && !memcmp(cData,"ddoonnee",8))		// Schluss-Tag
			{
			break;
			}

		if(!iFlag)
			{
			if(iSize>=16 && !memcmp(cData,cCode,16))
				{
				iFlag = 1;
				}

			continue;
			}

		if(cData[0]=='E' && !memcmp(cData,"END<",4))	// Ende erreicht
			{
			iEnd  = atoi(cData+4);
			iFlag = 0;

			if(iEnd<0 && m_uHexOutVesion)
				{
				iErr = DecodeError(iEnd);
				iEnd = -1;
				}

			continue;
			}

		if(cData[0]=='L' && !memcmp(cData,"LEN<",4))	// Datei Lnge am Beginn (nur bei "hexout")
			{
			iMax  = atoi(cData+4);
			continue;
			}
		

		if(m_uHexOutVesion)
			{
			if(cData[0]!='>')continue;
			iByte = base64tomem(cTemp,sizeof(cTemp),cData+1);
			}
		else{
			iByte = dumptomem(cTemp,sizeof(cTemp),cData,&iOff);

			if(cData[iOff])
				{
				iVal = DecodeError(cData);
				if(iVal)iErr=iVal;
				continue;
				}
			}


		fwrite(cTemp,1,iByte,pFile);
		m_uProgressOff += iByte;
		iPos		   += iByte;
		}


	m_bAbortAllowed	= FALSE;

	fclose(pFile);

	if(iErr)											// Gab es Fehler
		{
		SetLastError(iErr);
		return FALSE;
		}

	if(m_bUserBreak)									// Gab es einen Abbruch vom User
		{
		SetLastError(ERROR_CANCELLED);
		return FALSE;
		}

	if(iPos!=iEnd)										// Nicht alles Empfangen
		{
		SetLastError(ERROR_HANDLE_EOF);
		return FALSE;
		}

	if(iMax>=0 && iMax!=iPos)							// Nicht alles Empfangen
		{
		SetLastError(ERROR_HANDLE_EOF);
		return FALSE;
		}



return TRUE; 
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpConsole::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
static unsigned uCounter;
char 			cBuffer[CFB_MAX_PATH];
char 			cData[1024];
char			cTemp[260];;
char 			cCode[20];
WIN32_FIND_DATA sInfo;
int				iSize;
int				iLine;
int				iByte;
int				iFlag;
int				iMax;
int				iOff;
int				iErr;
int				iRet;
int				iLen;
int				iVal;
int 			iPos;
int 			iEnd;



	m_uProgressSize	= 0;
	m_uProgressOff	= 0;
	m_bUserBreak	= 0;


	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	
	if(GetStat(cBuffer,&sInfo)<0)						// Dateigre einlesen	
		{
		return FALSE;
		}

	
	if(sInfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)	// Ist die Datei ein Verzeichniss
		{
		pProc(pParam,NULL,-1);
		SetLastError(ERROR_ACCESS_DENIED);
		return FALSE;
		}

	m_uProgressSize	= sInfo.nFileSizeLow;
	
	if(m_uProgressSize)									// Leere Datei
		{
		pProc(pParam,NULL,0);							// Das Ende signalisieren
		return TRUE;
		}


	uCounter++;	
	iLen = sprintf(cCode,"aabb%08Xggtt",uCounter);

	if(m_uHexOutVesion)									// Datei via "hexout" einlesen
		{
		iLen  = sprintf    (cData,"echo %s;hexout -r256 -e ",cCode);
		iLen += HextOutText(cData+iLen,cBuffer,sizeof(cData)-128-iLen);		
		iLen += sprintf    (cData+iLen,";echo ddoonnee\r\n");
		}
	else{												// Datei via "hexdump" einlesen
		iLen = sprintf(cData,"echo %s;hexdump -v -e '\"END<%%08_Ad>\\n\"' -e '256/1 \"%%02x\"' -e '\"\\n\"' %.400s;echo ddoonnee\r\n",cCode,cBuffer);
		}


	ClearBuffer();


	   iErr = SshWrite(m_pSession,cData,iLen);			// Kommandos senden
	if(iErr!=iLen)										
		{
		pProc(pParam,NULL,-1);
		SetError(iErr);
		return FALSE;
		}
	
	iEnd	=-1;
	iErr	= 0;
	iPos	= 0;
	iFlag	= 0;

	m_bAbortAllowed	= TRUE;


	for(;;)												// Hex-Dump herunterladen
		{
		if(!SshWait(m_pSession,m_uTimeout,1))
			{
			if(m_bUserBreak)break;
			SetLastError(ERROR_TIMEOUT);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return -SSH_TIMEOUT;
			}

		iSize = SshReadCount(m_pSession,&iLine);		// Lnge der nchsten empfangenen Zeile holen

		if(iLine>=sizeof(cData))
			{
			iLine = sizeof(cData)-1;
			}

		iSize = SshRead(m_pSession,cData,iLine);		// Nchste Zeile einlesen

		#if SSH_LGGING_ON
		printf("%.*s",iSize,cData);
		#endif 	

		if(iSize<0)
			{
			SetError(iSize);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return FALSE;
			}

		cData[iSize] = 0;
		iSize = strcut(cData);

		if(m_bUserBreak)								// Gab es einen User-Abbruch
			{
			break;
			}

		if(iSize>=8 && !memcmp(cData,"ddoonnee",8))		// Schluss-Tag
			{
			break;
			}

		if(!iFlag)
			{
			if(iSize>=16 && !memcmp(cData,cCode,16))
				{
				iFlag = 1;
				}

			continue;
			}


		if(cData[0]=='E' && !memcmp(cData,"END<",4))	// Ende erreicht
			{
			iEnd  = atoi(cData+4);
			iFlag = 0;

			if(iEnd<0 && m_uHexOutVesion)
				{
				iErr = DecodeError(iEnd);
				iEnd = -1;
				}

			continue;
			}

		if(cData[0]=='L' && !memcmp(cData,"LEN<",4))	// Datei Lnge am Beginn (nur bei "hexout")
			{
			iMax  = atoi(cData+4);
			continue;
			}
		

		if(m_uHexOutVesion)
			{
			if(cData[0]!='>')continue;
			iByte = base64tomem(cTemp,sizeof(cTemp),cData+1);
			}
		else{
			iByte = dumptomem(cTemp,sizeof(cTemp),cData,&iOff);

			if(cData[iOff])
				{
				iVal = DecodeError(cData);
				if(iVal)iErr=iVal;
				continue;
				}
			}

		   iRet = pProc(pParam,cTemp,iByte);
		if(iRet!=iByte)
			{
			iErr = ERROR_WRITE_FAULT; 
			break;
			}

		m_uProgressOff += iByte;
		iPos		   += iByte;
		}

	m_bAbortAllowed	= FALSE;

	if(iErr)											// Gab es Fehler
		{
		pProc(pParam,NULL,-1);
		SetLastError(iErr);
		return FALSE;
		}

	if(m_bUserBreak)									// Gab es einen Abbruch vom User
		{
		pProc(pParam,NULL,-1);
		SetLastError(ERROR_CANCELLED);
		return FALSE;
		}

	if(iPos!=iEnd)										// Nicht alles Empfangen
		{
		pProc(pParam,NULL,-1);
		SetLastError(ERROR_HANDLE_EOF);
		return FALSE;
		}

	if(iMax>=0 && iMax!=iPos)							// Nicht alles Empfangen
		{
		pProc(pParam,NULL,-1);
		SetLastError(ERROR_HANDLE_EOF);
		return FALSE;
		}


	pProc(pParam,NULL,0);								// Das Ende signalisieren



return TRUE; 
}


//*****************************************************************************
//*
//*		GetLinkTarget
//*
//*****************************************************************************
//	Holt den Zielpfad zu einen Datei-Link
//	pLinkPath	: Ist der Pfad zum Datei-Link
//	pTarget		: Hier wird der Zielpfad gespeichert
//	iSize		: Ist die Gre des Puffers fr den Zielpfad
//	Ergibt TRUE wenn der Zielpfad ermittelt werden konnte, bzw. FALSE bei einem Fehler. 
int CFtpConsole::GetLinkTarget(const char *pLinkPath,char *pTarget,int iSize)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;
int		iLen;


	if(iSize<=0)
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}

	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pLinkPath,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = LineCommand(cBuffer,pTarget,iSize,"readlink -fv ","\"");

	if(iErr<=0)
		{
		pTarget[0] = 0;
		return FALSE;
		}

	

return TRUE;
}

//*****************************************************************************
//*
//*		List
//*
//*****************************************************************************
//	Liste alle Dateien in einem Verzeichnis via Callback auf.
//	pRemotePath		: Ist der Name des Verzeichnisses.
//	pParam			: Parameter fr den Callback.
//	pProc			: Zeiger auf den Callback.
//	pFilter			: Filter der benutzt werden kann.
//	Ergibt TRUE wenn alles ok war oder FALSE bei einem Fehler.
BOOL CFtpConsole::List(LPCTSTR pRemotePath,void *pParam,FtpFindCallback pProc,LPCTSTR pFilter)
{
static unsigned uCounter;
char 			cBuffer[CFB_MAX_PATH+64];
char 			cLnName[CFB_MAX_PATH+64];
char 			cData[1024];
char 			cCode[20];
const char     *pExtra;
LinkEntry	   *pLink;
LinkEntry	   *pNext;
LinkEntry	   *pTemp;
WIN32_FIND_DATA sInfo;
int				iFlag;
int				iLine;
int				iSize;
int				iOff;
int				iLen;
int				iErr;
int				iNum;


	m_bUserBreak = 0;

	if(!Reconnect())
		{
		pProc(pParam,NULL);
		return FALSE;
		}


	uCounter++;
	sprintf(cCode,"abcd%08Xvvww",uCounter);

	if(!*pRemotePath)								// Aktuellen Pfad benutzen	
		{
		if(m_bUseUtf8)
				iOff=strtoutf8(cBuffer,m_cCurrDir,CFB_MAX_PATH-1);
		else	iOff=strawcpy (cBuffer,m_cCurrDir,CFB_MAX_PATH-1);
		}
	else{
		iOff = PathFullMake(cBuffer,m_cCurrDir,pRemotePath,CFB_MAX_PATH-1);
		iOff = PathConvert (cBuffer,'/');
		}


	if(pFilter==CFB_ONLYDIR)						// Nur Verzeichnisse auflisten
		{
		pExtra  = "-o ";
		pFilter	= NULL;
		}
	else{
		pExtra  = "";
		}

	if(m_uHexOutVesion)								// "hexout" benutzen
		{
		if(iOff==0 || cBuffer[iOff-1]!='/')
			{
			cBuffer[iOff] = '/';
			iOff++;
			}

		if(!pFilter || !*pFilter)
			{
			cBuffer[iOff] = '*';iOff++;
			cBuffer[iOff] = 0;
			}
		else{
			if(m_bUseUtf8)
					iOff=strtoutf8(cBuffer+iOff,pFilter,CFB_MAX_PATH-iOff);
			else	iOff=strawcpy (cBuffer+iOff,pFilter,CFB_MAX_PATH-iOff);
			}

		HextOutText(cLnName,cBuffer,sizeof(cLnName));
		iLen = sprintf(cData,"echo %s;hexout -l %s%s;echo ssttoopp\r\n",cCode,pExtra,cLnName);
		}
	else{											// "ls" benutzen
		iOff = AddBackSlash('/',cBuffer,iOff);
		iLen = sprintf(cData,"echo %s;%sls %s %s;echo ssttoopp\r\n",cCode,m_cPrefix,m_pLsText,cBuffer);
		}


	ClearBuffer();

	   iErr = SshWrite(m_pSession,cData,iLen);		// Kommandos senden
	if(iErr!=iLen)
		{
		pProc(pParam,NULL);
		SetError(iErr);
		return FALSE;
		}
	
	if(!pFilter)pFilter=_T("");
	
	m_bAbortAllowed = 1;
	pLink			= 0;
	pNext			= 0;

	for(iFlag=0,iNum=0;;)							// Die Dateien einlesen
		{
		if(!SshWait(m_pSession,m_uTimeout,1))
			{
			for(;pLink;pLink=pNext)					// Links lschen
				{
				pNext = (LinkEntry*)pLink->pNext;
				delete pLink;
				}

			pProc(pParam,NULL);	
			SetLastError(ERROR_TIMEOUT);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return -SSH_TIMEOUT;
			}

		iSize = SshReadCount(m_pSession,&iLine);	// Lnge der nchsten Zeile holen

		if(iLine>=sizeof(cData))
			{
			iLine = sizeof(cData)-1;
			}

		iSize = SshRead(m_pSession,cData,iLine);

		#if SSH_LGGING_ON
		printf("%.*s",iSize,cData);
		#endif 	

		if(iSize<0)
			{
			for(;pLink;pLink=pNext)					// Links lschen
				{
				pNext = (LinkEntry*)pLink->pNext;
				delete pLink;
				}

			pProc(pParam,NULL);
			SetError(iSize);
			m_bAbortAllowed	= FALSE;
			m_bReconnect    = TRUE;
			return FALSE;
			}

		cData[iSize] = 0;
		iSize = strcut(cData);
													// Wurde der Start gefunden
		if(!iFlag && iSize>=16 && !memcmp(cData,cCode,16))
			{
			iFlag=1;
			continue;
			}
		
		if(iFlag)
			{										// Ist das Ende erreicht
			if(iSize>=8 && iSize<=12 && !memcmp(cData,"ssttoopp",8))
				{
				break;
				}

			if(iFlag==1 && !m_bUserBreak)
				{
				   iLen = LineToAttrib(cData,&sInfo);
				if(iLen>=0)							// Einen Listeneintrag gefunden
					{
					if(pFilter[0]==0 || !memxcmp(pFilter,sInfo.cFileName,_tcslen(sInfo.cFileName),0x03))
						{
						if(iLen<=1)
							{
							if(pProc(pParam,&sInfo)==0)iFlag=2;
							}
						else{						// Link gefunden
							if(!pLink)
								{
								pLink			= new LinkEntry;
								pNext			= pLink;
								pLink->pNext	= NULL;
								pLink->sFind.cFileName[0] = 0;
								}

							pTemp			= new LinkEntry;						
							pTemp->pNext	= NULL;
							pTemp->sFind.cFileName[0] = 0;

							strlcpy( pNext->sFind.cFileName,sInfo.cFileName,MAX_PATH);
							memcpy (&pNext->sFind,&sInfo,offsetof(WIN32_FIND_DATA,cFileName));
							pNext->sFind.cAlternateFileName[0] = 0;
							pNext->pNext	= pTemp;
							pNext			= pTemp;
							}
						}
					
					iNum++;
					}
				}
			}
		}

	if(pLink)										// Links auflsen
		{
		pNext = pLink;

		for(;pNext->sFind.cFileName[0];pNext=(LinkEntry*)pNext->pNext)
			{
			if(iFlag!=1 || m_bUserBreak)break;

			strlcpy(cBuffer+iOff,pNext->sFind.cFileName,sizeof(cBuffer)-iOff);
			
			if(!GetLinkTarget(cBuffer,cLnName,sizeof(cLnName)))
				{
				if(pProc(pParam,&pNext->sFind)==0)iFlag=2;
				continue;
				}

			iLen = GetStat(cLnName,&sInfo);

			if(iLen<0)
				{
				if(pProc(pParam,&pNext->sFind)==0)iFlag=2;
				continue;
				}

			strlcpy(sInfo.cFileName,pNext->sFind.cFileName,MAX_PATH);
			sInfo.dwFileAttributes |= FILE_ATTRIBUTE_LINK;
			if(pProc(pParam,&sInfo)==0)iFlag=2;
			}
		}


	for(;pLink;pLink=pNext)							// Links lschen
		{
		pNext = (LinkEntry*)pLink->pNext;
		delete pLink;
		}

	
	m_bAbortAllowed = 0;

	pProc(pParam,NULL);								// Das Ende signalisieren


	if(m_bUserBreak)
		{
		SetLastError(ERROR_CANCELLED);
		return FALSE;
		}

	if(iNum==0)	
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		return FALSE;
		}
	

return TRUE;
}

//*****************************************************************************
//*
//*		SetAttributes
//*
//*****************************************************************************
//	Einstellen der Dateiattribute
//	pFileName		: Ist der Name der Datei.
//	dwAttr			: Sind die neuen Attribute.
//					  In den Bits 16..31 knnen Unix-Attribute verpackt werden.
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpConsole::SetAttributes(LPCTSTR pFileName,DWORD dwAttr)
{
char 		cBuffer[CFB_MAX_PATH];
char 		cData[512];
unsigned 	uAttrib;
int			iErr;
int			iLen;


	if(dwAttr==0xFFFFFFFF)							// Parameter prfen
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}

	if(!Reconnect())return FALSE;


	uAttrib = SSH2_FILEXFER_ATTR_PERMISSIONS;


	if(dwAttr&0xFFFF0000)							// UNIX Attribute vorhanden
		{
		uAttrib = dwAttr>>16;
		}
	else{
		uAttrib = (dwAttr&FILE_ATTRIBUTE_READONLY)? 0x0120:0x01B4;
		if(dwAttr&FILE_ATTRIBUTE_DIRECTORY)uAttrib|=0x0200;
		if(dwAttr&FILE_ATTRIBUTE_HIDDEN   )uAttrib|=0x0001;	// Samba-Hack
		if(dwAttr&FILE_ATTRIBUTE_SYSTEM   )uAttrib|=0x0008;
		if(dwAttr&FILE_ATTRIBUTE_ARCHIVE  )uAttrib|=0x0040;
		if(dwAttr&FILE_ATTRIBUTE_EXEC     )uAttrib|=0x0049;
		}


	iLen = PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	sprintf(cData,"chmod 0%2o ",uAttrib&0x1FF);

	iErr = LineCommand(cBuffer,cData,sizeof(cData),cData,"\"");

	if(iErr     <0)return FALSE;
	if(cData[0]==0)return TRUE;


	switch(GetLastError())
		{
	case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
		}


return FALSE;
}

//*****************************************************************************
//*
//*		SetModifyTime
//*
//*****************************************************************************
//	Einstellen der nderungszeit einer Datei.
//	pFileName		: Ist der Name der Datei.
//	sFileTime		: Ist die neue nderungszeit
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpConsole::SetModifyTime(LPCTSTR pFileName,FILETIME sFileTime)
{
char 		cBuffer[CFB_MAX_PATH];
char		cData[512];
SYSTEMTIME	sSysTime;
int			iErr;
int			iLen;

	
	if(!FileTimeToSystemTime(&sFileTime,&sSysTime))
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	sprintf(cData,"touch -m -t %04u%02u%02u%02u%02u.%02u ",sSysTime.wYear,sSysTime.wMonth,sSysTime.wDay,sSysTime.wHour,sSysTime.wMinute,sSysTime.wSecond);

	iErr = LineCommand(cBuffer,cData,sizeof(cData),cData,"\"");

	if(iErr     <0)return FALSE;
	if(cData[0]==0)return TRUE;


	switch(GetLastError())
		{
	case ERROR_SUCCESS:			SetLastError(ERROR_ACCESS_DENIED);		break;
		}


return FALSE;
}

//*****************************************************************************
//*
//*		DoAbort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpConsole::DoAbort()
{


	if(!m_pSession|| !m_bAbortAllowed)return;

	SshBreak(m_pSession);
	
	m_bUserBreak	= TRUE;
	m_bReconnect    = TRUE;
	m_bAbortAllowed = 0;


}

//*****************************************************************************
//*
//*		ClearBuffer
//*
//*****************************************************************************
//	Lscht alle Bytes aus dem Emfangspuffer
//	Ergibt die Anzahl der gelschen Bytes.
int	CFtpConsole::ClearBuffer()
{
char	cBuffer[256];
int		iBytes;
int		iSize;


	iBytes = 0;

	for(;;)
		{
		   iSize = SshReadCount(m_pSession,NULL);			
		if(iSize<=0)break;

		if(iSize>sizeof(cBuffer))iSize=sizeof(cBuffer);

		   iSize = SshRead(m_pSession,cBuffer,iSize);
		if(iSize<=0)break;

		#if SSH_LGGING_ON
		printf("%.*s",iSize,cBuffer);
		#endif 	

		iBytes += iSize;
		}


return iBytes;
}

//*****************************************************************************
//*
//*		MemToText
//*
//*****************************************************************************
//	Wandelt einen Speicherbereich in einen Text mit Escape-Sequenzen um
//	pMem		: Zeiger auf den Speicherbereich
//	pText		: Zeiger auf den Textpuffer (Gre = iLen*4+1)
//	iLen		: Lnge des Speicherbereichs
//	Ergibt die Lnge des Textes in Zeichen.
int CFtpConsole::MemToText(const char *pMem,char *pText,int iLen)
{
static const BYTE	aSpecialChars[] =
	{	
	0xFF,0xFF,0xFF,0xFF, 0xDF,0x00,0x00,0x50, 0x00,0x00,0x00,0x38, 0x01,0x00,0x00,0x90,
	0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF, 
	};

int		iBits;
int		iVal1;
int		iVal2;
int		iVal3;
int		iFlag;
int		iPos;
int		iNum;


	iNum	= 0;
	iFlag	= 0;

	for(iPos=0;iPos<iLen;iPos++)
		{
		iBits = (BYTE)pMem[iPos];

		if(!bit_read(aSpecialChars,iBits))
			{
			if(!iFlag || iBits>'7' || iBits<'0')
				{
				pText[iNum] = iBits;
				iNum++;
				iFlag=0;
				continue;
				}
			}

		if(iBits<=13 && ((0x3F80>>iBits)&1))
			{
			pText[iNum]	= '\\';						iNum++;
			pText[iNum]	= "_______abtnvfr"[iBits];	iNum++;
			iFlag = 0;
			continue;
			}

		iVal1	= (iBits>>6);
		iVal2	= (iBits>>3)&7;
		iVal3	= (iBits   )&7;

		pText[iNum]	= '\\'; iNum++;
		pText[iNum]	= '0' ; iNum++;

		if(iVal1                  ){pText[iNum] = "01234567"[iVal1]; iNum++;}
		if(iVal2 || iVal1         ){pText[iNum] = "01234567"[iVal2]; iNum++;}
		if(iVal3 || iVal2 || iVal1){pText[iNum] = "01234567"[iVal3]; iNum++;}

		iFlag		= 1;
		}
	

	pText[iNum] = 0;


return iNum;
}

//*****************************************************************************
//*
//*		LineToAttrib
//*
//*****************************************************************************
//	Wandlet eine Zeile von einem "ls -al" Kommando in Dateiattribute um
//	pLine		: Zeiger auf die Zeile
//					"drwx------ 10 root root  4096 2019-12-03 18:45:14.000000000 +0200 MyDir"
//					"-rw-rw-rw- 10 root root 12519 1. Nov 2011                   MyFile"
//					"lrw-rw-rw- 10 root      12519 3. Oct 20:11                  MyFile -> dir/test.txt"
//	pInfo		: Zeiger fr die Attribute
//	Ergibt 0 bei einer Datei, 1 bei einem Verzeichnis, bzw. den Offset
//	des Link-Namens und -1 bei einem Fehler.
int CFtpConsole::LineToAttrib(const char *pLine,LPWIN32_FIND_DATA pInfo)
{
const char *pStart;
__int64		iSize;
unsigned	uTime;
unsigned	uFlag;
int			iAttr;
int			iMode;
int			iOff;
int			iPos;
int			iAdd;
int			iEnd;
int			iLen;
int			iVal;
int			iNum;



	if(pLine[0]=='>')									// Ausgabe von "hexout"
		{
		if(pLine[1]>'9' || pLine[1]<'0')
			{
			return -1;
			}

		iAttr = strtoi(pLine+1,0,&iLen);
		iOff  = iLen+1; 

		uFlag  = (iAttr&SYS_WRITE)? 0:FILE_ATTRIBUTE_READONLY;
		uFlag |= (iAttr&0xFFFFFC00);
		
		if(iAttr&SYS_SYSTEM )uFlag|=FILE_ATTRIBUTE_SYSTEM;
		if(iAttr&SYS_HIDDEN )uFlag|=FILE_ATTRIBUTE_HIDDEN;
		if(iAttr&SYS_ARCHIVE)uFlag|=FILE_ATTRIBUTE_ARCHIVE;
		if(iAttr&SYS_DIR    )uFlag|=FILE_ATTRIBUTE_DIRECTORY;

		if((uFlag&0xF0000000)==0xA0000000)uFlag|=FILE_ATTRIBUTE_LINK;

		pInfo->dwFileAttributes	= uFlag;
		pInfo->nFileSizeHigh	= (uFlag>>10)&0x3F;
		pInfo->nFileSizeLow		=  strtoi(pLine+iOff,0,&iLen);	iOff+=iLen; 
		uTime	 			 	=  strtoi(pLine+iOff,0,&iLen);	iOff+=iLen; 
		UtcTimeToFileTime(uTime,&pInfo->ftLastWriteTime);
		uTime	 			 	=  strtoi(pLine+iOff,0,&iLen);	iOff+=iLen; 
		UtcTimeToFileTime(uTime,&pInfo->ftCreationTime);
		iOff				   += strwlen(pLine+iOff,0xE0);
		
		pInfo->ftLastAccessTime.dwLowDateTime	= 0;
		pInfo->ftLastAccessTime.dwHighDateTime	= 0;
		pInfo->cAlternateFileName[0]			= 0;
		pInfo->dwReserved0						= 0;
		pInfo->dwReserved1						= 0;
		
		if(m_bUseUtf8)
				iOff=strfromutf8(pInfo->cFileName,pLine+iOff,MAX_PATH);
		else	iOff=strawcpy   (pInfo->cFileName,pLine+iOff,MAX_PATH);

		return (iAttr&SYS_DIR)? 1:0;
		}


	if(pLine[0]=='-')									// Verzeichnis, Link oder Datei
		{
		iAttr = 0x0000;
		iMode = 0;
		}
	else if(pLine[0]=='d')
		{
		iAttr = 0x4000;
		iMode = 1;
		}
	else if(pLine[0]=='l')
		{
		iAttr = 0xA000;
		iMode = 2;
		}
	else if(pLine[0]!='c' && pLine[0]!='b' && pLine[0]!='s' && pLine[0]!='p')
		{
		return -1;
		}
	else{
		iAttr = 0;
		iMode = 0;
		}


	   iLen = strwlen(pLine,0x02000006);
	if(iLen!=10)										// Prfe die Lnge
		{
		return -1;
		}



	for(iLen=1;iLen<10;iLen++)							// Zugriffsrechte decodieren
		{
		if(pLine[iLen]=="drwxrwxrwx"[iLen])
			{
			iAttr |= 0x200>>iLen;
			continue;
			}

		if(pLine[iLen]!='-')break;
		}

	pStart = pLine;
	pLine += 10;
	pLine += strwlen(pLine,0xE0);

	if(!strwchar(*pLine,0x01))							// Keine Inode gefunden
		{
		return -1;
		}

	pLine += strwlen(pLine,0x01);
	pLine += strwlen(pLine,0xE0);

	for(iPos=0;iPos<2;iPos++)							// Besitzer und Gruppe berspringen
		{
		if(!strwchar(*pLine,0x06))break;
		pLine += strcwlen(pLine,0xE0);
		pLine += strwlen (pLine,0xE0);
		}

	if(!strwchar(*pLine,0x01))							// Keine Gre gefunden
		{
		return -1;
		}

	iSize	= strtoq(pLine,10,&iLen);
	pLine  += iLen;
	pLine  += strwlen(pLine,0xE0);

	if(iMode==1)										// Verzeichnisse haben keine Gre
		{
		iSize = 0;
		}

	if(iMode==2)										// Link
		{
		iEnd = strfind(pLine,"->");

		if(iEnd<0)
			{
			iOff   = 0;
			iMode  = 0;
			iAttr &= 0x0FFF;
			iEnd   = strlen(pLine);
			}
		else{
			iOff   = iEnd+2;
			iOff  += strwlen(pLine+iOff,0xE0);
			}
		}
	else{
		iEnd = strlen(pLine);
		iOff = 0;
		}


	iEnd -= strbwlen(pLine+iEnd,0xE0,iEnd);

	for(iPos=iEnd;iPos>0;iPos=iNum)						// Suche Anfang vom Dateinamen
		{
		iPos -= strdwlen(pLine+iPos,0x002000E0,iPos);

		if(pLine[iPos-1]=='/')
			{
			break;
			}

		iNum  = strbwlen(pLine+iPos,0xE0,iPos);
		iNum  = iPos-iNum;

		if(iNum<=0 || !strwchar(pLine[iNum-1],0x01))	// Ist eine Zahl vorher
			{
			continue;
			}

		iAdd = 	strbwlen(pLine+iNum,0x01,iNum);

		if(iNum>5 && iAdd==4 && (pLine[iNum-5]=='+' || pLine[iNum-5]=='-'))
			{
			break;										// Zeitszone gefunden "+0200"
			}

		if(iNum>5 && iAdd==2 && pLine[iNum-3]==':' && strwchar(pLine[iNum-4],0x01) && strwchar(pLine[iNum-5],0x01) && strwchar(pLine[iNum-6],0xA0))
			{
			break;										// Zeit gefunden "20:13"
			}

		if(iNum>5 && iAdd==4)
			{
			iVal = atoi(pLine+iNum-4);

			if(iVal>1900 && iVal<2199)
				{
				break;									// Jahr gefunden
				}
			}
		}


	if(m_bUseUtf8)
			strfromutf8(pInfo->cFileName,pLine+iPos,sizeof(pInfo->cFileName)/sizeof(pInfo->cFileName[0]),iEnd-iPos);
	else	strawcpy   (pInfo->cFileName,pLine+iPos,sizeof(pInfo->cFileName)/sizeof(pInfo->cFileName[0]),iEnd-iPos);

	uTime = (unsigned)strtoutc(pLine,STR_DATE|STR_TIME|STR_NOW_DATE,iPos,&iVal);
	UtcTimeToFileTime(uTime,&pInfo->ftLastWriteTime);

	pInfo->ftCreationTime.dwLowDateTime		= 0;
	pInfo->ftCreationTime.dwHighDateTime	= 0;
	pInfo->ftLastAccessTime.dwLowDateTime	= 0;
	pInfo->ftLastAccessTime.dwHighDateTime	= 0;
	pInfo->dwFileAttributes					= iAttr<<16;
	pInfo->nFileSizeLow						= (DWORD)(iSize);
	pInfo->nFileSizeHigh					= (DWORD)(iSize>>32);
	pInfo->dwReserved0						= 0;
	pInfo->dwReserved1						= 0;
	pInfo->cAlternateFileName[0]			= 0;

	if(!(iAttr&0x0080))pInfo->dwFileAttributes|=FILE_ATTRIBUTE_READONLY;
	if(  iAttr&0x4000 )pInfo->dwFileAttributes|=FILE_ATTRIBUTE_DIRECTORY;
	if(  iAttr&0x0001 )pInfo->dwFileAttributes|=FILE_ATTRIBUTE_HIDDEN;		// Samba-Hack
	if(  iAttr&0x0008 )pInfo->dwFileAttributes|=FILE_ATTRIBUTE_SYSTEM;
	if(  iAttr&0x0040 )pInfo->dwFileAttributes|=FILE_ATTRIBUTE_ARCHIVE;
	if(  iMode==2     )pInfo->dwFileAttributes|=FILE_ATTRIBUTE_LINK;


	if(iMode==2)										// Offset des Linknamens
		{
		return pLine-pStart+iOff;
		}


return iMode;
}

//*****************************************************************************
//*
//*		LineCommand
//*
//*****************************************************************************
//	Holt die Antwort eines einzelnen Kommmandos
//	pCmd		: Ist das Kommando das gesendet wird
//	pBuffer		: Hier wird die Antwort abgelegt
//	iMax		: Puffergre fr die Antwort
//	pExtra		: Ist ein Extra-Text der vor das Kommando gestellt wird
//	pFrame		: Frame der um pCmd ausgegebn wird	
//	Ergibt die Lnge der Antwort oder eine Fehlernummer kleiner 0.
//	Die Fehlernummer wird ebenfalls gesetzt.
int CFtpConsole::LineCommand(const char *pCmd,char *pBuffer,int iMax,const char *pExtra,const char *pFrame)
{
static unsigned uCounter;
char 			cData[2048];
char 			cCode[20];
int				iSize;
int				iLine;
int				iFlag;
int				iErr;
int				iLen;

	
	uCounter++;	
	iLen = sprintf(cCode,"aaee%08Xxxyy",uCounter);
	iLen = sprintf(cData,"echo -n %s;%.1200s%s%.750s%s;echo\r\n",cCode,pExtra,pFrame,pCmd,pFrame);

	ClearBuffer();

	   iErr = SshWrite(m_pSession,cData,iLen);
	if(iErr!=iLen)
		{
		SetError(iErr);
		return iErr;
		}


	for(iFlag=0;;)
		{
		if(!SshWait(m_pSession,m_uTimeout,1))
			{
			SetLastError(ERROR_TIMEOUT);
			m_bReconnect = TRUE;
			return -SSH_TIMEOUT;
			}

		iSize = SshReadCount(m_pSession,&iLine);

		if(iLine>=sizeof(cData))
			{
			iLine = sizeof(cData)-1;
			}

		iSize = SshRead(m_pSession,cData,iLine);

		#if SSH_LGGING_ON
		printf("%.*s",iSize,cData);
		#endif 	

		if(iSize<0)
			{
			SetError(iSize);
			m_bReconnect = TRUE;
			return FALSE;
			}

		cData[iSize] = 0;
		iSize = strcut(cData);

		if(!iFlag && iSize>=16 && !memcmp(cData,cCode,16))
			{
			break;
			}
		}
	

	iLen = strlcpy(pBuffer,cData+16,iMax,iSize-16);
	iErr = DecodeError(pBuffer);

	SetLastError(iErr);



return iLen; 
}

//*****************************************************************************
//*
//*		Init
//*
//*****************************************************************************
//	Initialisiert die SFTP Verbindung.
//	iReconnect	: Automatisch wieder verbinden
//	uTimeout	: Timeout in Millisekunden
//	bUtf8		: UTF8 benutzen
//	Ergibt TRUE wenn die Verbindung erzeugt werden konnte.
BOOL CFtpConsole::Init(int iReconnect,unsigned uTimeout,BOOL bUtf8)
{
static const char  *aInit[]={"\\ls -alogd --full-time .","\\ls -algd --full-time .","\\ls -algd .",0};
static const char  *aUsed[]={"--full-time -alog"        ,"--full-time -alg"        ,"-alg"          };
char				cBuffer[CFB_MAX_PATH+256];
unsigned			uOffset;
const char		   *pText;	
int					iErr;
int					iPos;



	if(!m_pSession)return FALSE;

	m_uHexOutVesion = 0;
	m_cPrefix[1]    = 0;

	SshWait(m_pSession,200,1);
	ClearBuffer();

	
	if(m_bUseHexOut)								// Soll das Programm "hexout" benutzt werden
		{
		   iErr = LineCommand("hexout -v",cBuffer,sizeof(cBuffer));
		if(iErr>5 && cBuffer[0]=='v' && !memcmp(cBuffer,"version ",8))
			{
			m_uHexOutVesion = atoi(cBuffer+8);
			m_pLsText	 = "-la";
			m_uLsMode    = 3;
			goto Done;
			}
		}

	
	for(iPos=0;;iPos++)								// Das Format fr das "ls" Kommando ermitteln
		{
			pText = aInit[iPos/2];
		if(!pText)
			{
			m_cPrefix[0] = 0;
			m_pLsText	 = "-al";
			m_uLsMode    = 3;
			break;
			}

		if(iPos&1)pText++;							// Farben nicht unterdrcken

		   iErr = LineCommand(pText,cBuffer,sizeof(cBuffer));
		if(iErr>10 && cBuffer[0]=='d' && strwlen(cBuffer,0x02000006)==10)
			{
			m_cPrefix[0] = (iPos&1)? 0:'\\';
			m_pLsText    = aUsed[iPos/2];
			m_uLsMode    = iPos/2;
			break;
			}
		}

Done:
													// Aktuelles Verzeichniss holen
	   iErr = LineCommand("pwd",cBuffer,sizeof(cBuffer));
	if(iErr<0)
		{
		return FALSE;
		}

	if(bUtf8)
			strfromutf8(m_cCurrDir,cBuffer,(sizeof(m_cCurrDir)/sizeof(TCHAR)));
	else	strawcpy   (m_cCurrDir,cBuffer,(sizeof(m_cCurrDir)/sizeof(TCHAR)));

	ClearBuffer();

	SuLogin();




	if(m_bUseHexOut && m_uHexOutVesion==0)			// Das 'hexout' Programm kopieren							
		{
		   iErr = LineCommand("uname -m",cBuffer,sizeof(cBuffer));
		if(iErr>2)
			{
			if((strifind(cBuffer,"386")>=0) ||
			   (strifind(cBuffer,"486")>=0) ||
			   (strifind(cBuffer,"586")>=0) ||
			   (strifind(cBuffer,"686")>=0) ||
			   (strifind(cBuffer,"x86")>=0))
				{
				m_uHexOutVesion = 0x8000;
				}
			}

		if(m_uHexOutVesion==0x8000)					// Das 'hexout' Programm kopieren
			{
			m_uHexOutVesion = 0;
			uOffset			= 0;

			if(PutFile("/bin/hexout",&uOffset,CopyHexOut))
				{
				SetAttributes("/bin/hexout",(0775<<16)|FILE_ATTRIBUTE_EXEC);

				   iErr = LineCommand("hexout -v",cBuffer,sizeof(cBuffer));
				if(iErr>5 && cBuffer[0]=='v' && !memcmp(cBuffer,"version ",8))
					{
					m_uHexOutVesion = atoi(cBuffer+8);
					m_pLsText	 = "-la";
					m_uLsMode    = 3;
					}
				}
			}
		}


	m_bReconnect	= (iReconnect)? TRUE:FALSE;
	m_uTimeout		=  uTimeout;
	m_bUseUtf8		=  bUtf8;
	m_bConnDone		=  TRUE;


return TRUE;
}

//*****************************************************************************
//*
//*		SuLogin
//*
//*****************************************************************************
//	Erweitert das Login um 'su' oder 'sudo'.
//	Der User-Name muss im Format  "<User>|<root>|<Pass>" angegeben werden
int CFtpConsole::SuLogin()
{
char	cData[256];
char	cText[128];
int		iLine;
int		iSize;
int		iTurn;
int		iLast;
int		iPos;
int		iLen;
int		iNum;
int		iErr;



	strawcpy(cText,m_sUserName,sizeof(cText));
	iPos = strchar(cText,'|');

	if(!cText[iPos])return 1;						// Normalse Login

	iPos += 1;
	iLen  = strchar(cText+iPos,'|'); 

	if(cText[iPos+iLen])							// Benutze 'su'
		{
		iNum  = sprintf(cData,"su -l %.*s\n",iLen,cText+iPos);
		iPos += iLen+1;
		}
	else{
		iNum  = sprintf(cData,"sudo -i\n");
		}


	ClearBuffer();

	   iErr = SshWrite(m_pSession,cData,iNum);
	if(iErr!=iNum)
		{
		return 0;
		}


	for(iTurn=0,iLast=0;iTurn<10;iTurn++)			// Warte auf Passwortabfrage
		{
		SshWait(m_pSession,(iTurn)? 100:1000,1);
		iSize = SshReadCount(m_pSession,&iLine);
		if(iLine>=sizeof(cData))iLine=sizeof(cData)-1;
		if(iLine==0 && iLast==0 && iTurn>1)iLine=iSize;
		
		iNum  = SshRead(m_pSession,cData,iLine);
		if(iNum<0)break;
		cData[iNum]=0;

		if(strifind(cData,"passwor")>=0)
			{
			break;
			}

		iLast = iNum;
		}


	Sleep(500);

	iNum  = sprintf(cData,"%s\n",cText+iPos);		

	   iErr = SshWrite(m_pSession,cData,iNum);		// Das Passwort senden
	if(iErr!=iNum)
		{
		return 0;
		}

	SshWait(m_pSession,200,1);

	ClearBuffer();



return 1;
}

//*****************************************************************************
//*
//*		PathFullMake
//*
//*****************************************************************************
int CFtpConsole::PathFullMake(char *pBuffer,LPCTSTR pCurrDir,LPCTSTR pPath,int iSize)
{
char		cTemp[CFB_MAX_PATH];
const char *pDir;


	if(m_bUseUtf8)
		{
		strtoutf8(cTemp,pCurrDir,sizeof(cTemp));
		pDir = cTemp;
		}
	else{
		#ifdef UNICODE
		strawcpy(cTemp,pCurrDir,sizeof(cTemp));
		pDir = cTemp;
		#else
		pDir = pCurrDir;
		#endif
		}


return ::PathFullMake(pBuffer,pDir,pPath,iSize,0);
}

//*****************************************************************************
//*
//*		CopyHexOut
//*
//*****************************************************************************
int _cdecl CFtpConsole::CopyHexOut(void *pParam,void *pMem,int iLen)
{
unsigned   *pOffset=(unsigned*)pParam;
int			iMax;
	

	if(iLen<=0)return 0;

	iMax = sizeof(m_aHexOut)-*pOffset;

	if(iMax<   0)iMax=0;
	if(iLen>iMax)iLen=iMax;

	memcpy(pMem,m_aHexOut+*pOffset,iLen);

	pOffset[0] += iLen;


return iLen;
}

//*****************************************************************************
//*
//*		DecodeError
//*
//*****************************************************************************
//	Sucht aus einem Text oder einer Fehlernummer eine Fehlermeldung.
int	CFtpConsole::DecodeError(const char *pBuffer)
{

	if(strifind(pBuffer,"No such file or directory")>=0)
		{
		return ERROR_FILE_NOT_FOUND;
		}
	else if(strifind(pBuffer,"Datei oder Verzeichnis nicht gefunden")>=0)
		{
		return ERROR_FILE_NOT_FOUND;
		}
	else if(strifind(pBuffer,"Directory not empty")>=0)
		{
		return ERROR_DIR_NOT_EMPTY;
		}
	else if(strifind(pBuffer,"Verzeichnis ist nicht leer")>=0)
		{
		return ERROR_DIR_NOT_EMPTY;
		}
	else if(strifind(pBuffer,"File exists")>=0)
		{
		return ERROR_ALREADY_EXISTS;
		}
	else if(strifind(pBuffer,"Die Datei existiert bereits")>=0)
		{
		return ERROR_ALREADY_EXISTS;
		}
	else if(strifind(pBuffer,"Permission denied")>=0)
		{
		return ERROR_ACCESS_DENIED;
		}
	else if(strifind(pBuffer,"invalid option")>=0)
		{
		return ERROR_INVALID_PARAMETER;
		}


return ERROR_SUCCESS;
}

int	CFtpConsole::DecodeError(int iErr)
{

	switch(iErr)
		{
	case SYS_ERR_PARAM:				return ERROR_INVALID_PARAMETER;
	case SYS_ERR_INVALID_FILENAME:	return ERROR_INVALID_NAME;
	case SYS_ERR_FILE_NOT_FOUND:	return ERROR_FILE_NOT_FOUND;
	case SYS_ERR_ACCESS_DENIED:		return ERROR_ACCESS_DENIED;
	case SYS_ERR_NO_HANDLES:		return ERROR_NO_MORE_SEARCH_HANDLES;
	case SYS_ERR_UNKNOWN:			return ERROR_INVALID_ACCESS;
	case SYS_ERR_PATH_NOT_FOUND:	return ERROR_PATH_NOT_FOUND;
	case SYS_ERR_DISKFULL:			return ERROR_HANDLE_DISK_FULL;
	case SYS_ERR_ALREADY_EXISTS:	return ERROR_ALREADY_EXISTS;
		}


return ERROR_SUCCESS;
}

//*****************************************************************************
//*
//*		SetError
//*
//*****************************************************************************
//	Stellt die Windows Fehlernummer ein
//	iError		: Ist die FTP Fehlernummer
void CFtpConsole::SetError(int iError)
{

	switch(iError)
		{
	case -SSH_CONN_BROKEN:			SetLastError(ERROR_GRACEFUL_DISCONNECT);m_bReconnect=TRUE;	return;
	case -SSH_CHANNEL_FAILURE:		SetLastError(ERROR_INVALID_ACCESS);							return;
	case -SSH_VERSION_NOT_SUPPORDED:SetLastError(ERROR_CALL_NOT_IMPLEMENTED);					return;
	case -SSH_UNKNOWN_VERSION:		SetLastError(ERROR_CALL_NOT_IMPLEMENTED);					return;
	case -SSH_AUTH_FAILURE:			SetLastError(ERROR_SERVICE_LOGON_FAILED);					return;
	case -SSH_INTEGRITY_ERROR: 		SetLastError(ERROR_INTERNAL_ERROR);							return;
	case -SSH_AUTH_ERROR:			SetLastError(ERROR_SERVICE_LOGON_FAILED);					return;
	case -SSH_PERMISSION_DENIED:	SetLastError(ERROR_ACCESS_DENIED);							return;
	case -SSH_USERAUTH_ERROR:		SetLastError(ERROR_SERVICE_LOGON_FAILED);					return;
	case -SSH_INTERNAL_DISSCONNECT:	SetLastError(ERROR_GRACEFUL_DISCONNECT);m_bReconnect=TRUE;	return;
	case -SSH_KEY_ERROR:			SetLastError(ERROR_LOCAL_USER_SESSION_KEY);					return;
	case -SSH_OUT_OF_MEMORY:		SetLastError(ERROR_OUTOFMEMORY);							return; 
	case -SSH_PADDING_ERROR:		SetLastError(ERROR_INTERNAL_ERROR);							return;
	case -SSH_REMOTE_CLOSE:			SetLastError(ERROR_GRACEFUL_DISCONNECT);m_bReconnect=TRUE;	return;
	case -SSH_ZIP_ERROR:			SetLastError(ERROR_INTERNAL_ERROR);							return;
	case -SSH_USER_BREAK:			SetLastError(ERROR_CANCELLED);								return;
	case -SSH_WRONG_PARAM:			SetLastError(ERROR_INVALID_PARAMETER);						return; 
	case -SSH_CONNECT_ERROR:		SetLastError(ERROR_NOT_CONNECTED);		m_bReconnect=TRUE;	return;
	case -SSH_CHIPER_ERROR:			SetLastError(ERROR_INTERNAL_ERROR);							return;
	case -SSH_COMPRESS_ERROR:		SetLastError(ERROR_INTERNAL_ERROR);							return;
	case -SSH_INIT_ERROR:			SetLastError(ERROR_INTERNAL_ERROR);							return;
	case -SSH_ALLREADY_CONNECTED:	SetLastError(ERROR_ALREADY_ASSIGNED);						return;
	case -SSH_RESOURCE_ERROR:		SetLastError(ERROR_INTERNAL_ERROR);							return;
	case -SSH_DLL_NOT_LOADED:		SetLastError(ERROR_INVALID_DLL);							return;
	case -SSH_CANT_CONNECT:			SetLastError(ERROR_HOST_UNREACHABLE);						return;
	case -SSH_TIMEOUT:				SetLastError(ERROR_TIMEOUT);			m_bReconnect=TRUE;	return;

	case SSH2_FX_OK:				SetLastError(ERROR_SUCCESS);								return;
	case SSH2_FX_EOF:				SetLastError(ERROR_HANDLE_EOF);								return;
	case SSH2_FX_NO_SUCH_FILE:		SetLastError(ERROR_FILE_NOT_FOUND);							return;
	case SSH2_FX_PERMISSION_DENIED:	SetLastError(ERROR_ACCESS_DENIED);							return;
	case SSH2_FX_FAILURE:			SetLastError(ERROR_INVALID_ACCESS);							return;
	case SSH2_FX_BAD_MESSAGE:		SetLastError(ERROR_BAD_COMMAND);		m_bReconnect=TRUE;	return;
	case SSH2_FX_NO_CONNECTION:		SetLastError(ERROR_NOT_CONNECTED);		m_bReconnect=TRUE;	return;
	case SSH2_FX_CONNECTION_LOST:	SetLastError(ERROR_GRACEFUL_DISCONNECT);m_bReconnect=TRUE;	return;
	case SSH2_FX_OP_UNSUPPORTED:	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);					return;
	case SSH2_FX_TIMEOUT:			SetLastError(ERROR_TIMEOUT);								return;
	case SSH2_FX_USERBREAK:			SetLastError(ERROR_CANCELLED);								return;
	case SSH2_FX_BUFFER_SIZE:		SetLastError(ERROR_BUFFER_OVERFLOW);						return;
	case SSH2_FX_CANT_CREATE_FILE:	SetLastError(ERROR_WRITE_FAULT);							return;
	case SSH2_FX_CANT_OPEN_FILE:	SetLastError(ERROR_FILE_INVALID);							return;
	case SSH2_FX_WRITE_ERROR:		SetLastError(ERROR_WRITE_FAULT);							return;
	case SSH2_FX_DLL_ERROR:			SetLastError(ERROR_INVALID_DLL);							return;
	case SSH2_FX_WRONG_PARAM:		SetLastError(ERROR_INVALID_PARAMETER);						return;
	case SSH2_FX_ALLREADY_CONNECTED:SetLastError(ERROR_ALREADY_ASSIGNED);						return;
	case SSH2_FX_RESOURCE_ERROR:	SetLastError(ERROR_INTERNAL_ERROR);							return;
	case SSH2_FX_CANT_CONNECT:		SetLastError(ERROR_HOST_UNREACHABLE);						return;
		}

	SetLastError(ERROR_INVALID_ACCESS);

}

