//*****************************************************************************
//*
//*
//*		FtpDrives.cpp
//*
//*
//*****************************************************************************
//
//	Ist eine Verbindung zu einem FAT-Lauferk oder FAT-Image.
//
#include	<stddef.h>
#include	"FtpTools.h"
#include	"FtpDrives.h"
#include	"../DiskIO.h"
#include	"../MemFunctions.h"
#include	"../FileFunctions.h"
#include	"../StringFunctions.h"

typedef	struct
	{
	FtpFindCallback	pProc;
	LPVOID			pParam;
	LPCTSTR			pFilter;
	}FilterCbData;


#define THIS_DISK	((DiskPartionData*)m_pSession)
#define BUFFER_SIZE	0x20000

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpDrives::CFtpDrives(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive) : CFtpBase(pSession,pServer,pUserName,pPassword,dwContext,nPort,bPassive)
{

	m_pBaseUrl	  = _T("disk://");
	m_uPartition  = nPort-1;
	m_uTimeout	  = 10000;
	m_bReadOnly	  = FALSE;
	m_bRetry	  = FALSE;
	m_bBreak      = FALSE;
	m_pBuffer     = NULL;
	m_lLocked	  = 0;


}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpDrives::~CFtpDrives()
{

	Close();

}


//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpDrives::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{
DWORD	   *pVal=(DWORD*)pBuffer;
DWORD		uLen;



	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	uLen = *pLen;


	switch(dwOption)
		{
	case INTERNET_OPTION_MODE:						// Verbindungsmodus

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_FTP;

		return TRUE;

	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:		// Timeout
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		*pVal = m_uTimeout;
		*pLen = sizeof(m_uTimeout);

		return TRUE;

	case INTERNET_OPTION_PROGRESS:					// Verlaufsanzeige

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		*pVal = (DWORD)m_uProgress;

		return 1;

	case INTERNET_OPTION_PROGRESS2:					// Verlaufsanzeige mit Maximalwert

		if(uLen<sizeof(DWORD)*2)
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = (DWORD)m_uProgress;
		pVal[1] = (DWORD)m_uProgrMax;

		return 1;

	case INTERNET_OPTION_CONNTYPE:					// Art der Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_DISK;

		return TRUE;

	case INTERNET_OPTION_URLFORMAT:					// Format des URL-Namens

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 1;

		return TRUE;

	case INTERNET_OPTION_ERROPEN:					// Wo gab es den Fehler

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_dwErrOpen;

		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpDrives::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	switch(dwOption)
		{
	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_uTimeout = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_PROGRSET:					// Verlaufsanzeige einstellen

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_uProgress = *(int*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_SETSTAT:					// Attribute und Zeiten setzen

		if(uLength<sizeof(WIN32_FIND_DATA))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return SetStat((LPWIN32_FIND_DATA)pBuffer);

	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}
		
		m_dwPutSize = *(LPDWORD)pBuffer;		
		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpDrives::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{
	if(!m_pSession)return NULL;

	return NULL;
}


//*****************************************************************************
//*
//*		SetCurrentDirectory
//*
//*****************************************************************************
//	Stellt das aktuelle Verzeichnis ein.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpDrives::SetCurrentDirectory(LPCTSTR pDirName)
{
int  	iErr;



	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	   iErr = DiskSetCurrentPath(THIS_DISK,pDirName);
	if(iErr)
		{
		SetError(iErr);
		return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis.
//	pDirName		: Puffer fr den Namen
//	pLen			: Gre des Puffers
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpDrives::GetCurrentDirectory(LPTSTR pDirName,LPDWORD pLen) const
{
int  	iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	   iErr = DiskGetCurrentPath(THIS_DISK,pDirName,*pLen);
	if(iErr)
		{
		SetError(iErr);
		return FALSE;
		}

	*pLen = _tcslen(pDirName);


return TRUE;
}


//*****************************************************************************
//*
//*		RemoveDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des zu lschenden Verzeichnisses
//	Ergibt TRUE wenn das Verzeichnis gelscht wurde.
BOOL CFtpDrives::RemoveDirectory(LPCTSTR pDirName)
{
int    		iErr;



	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	   iErr = DiskRmdir(THIS_DISK,pDirName);
	if(iErr)
		{
		SetError(iErr);
		return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		CreateDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpDrives::CreateDirectory(LPCTSTR pDirName)
{
int    		iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	   iErr = DiskMkdir(THIS_DISK,pDirName);
	if(iErr)
		{
		SetError(iErr);
		return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		Remove
//*
//*****************************************************************************
//	Lscht eine Datei..
//	pFileName		: Ist der Name der Datei
//	Ergibt TRUE wenn die Datei gelscht wurde.
BOOL CFtpDrives::Remove(LPCTSTR pFileName)
{
int    		iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	   iErr = DiskDelete(THIS_DISK,pFileName);
	if(iErr)
		{
		SetError(iErr);
		return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	Verschiebt eine Datei..
//	pExisting		: Ist der Name der Datei.
//	pNewName		: Ist der Name der Zieldatei.
//	Ergibt TRUE wenn die Datei vershoben wurde.
BOOL CFtpDrives::Rename(LPCTSTR pExisting,LPCTSTR pNewName)
{
int    		iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	   iErr = DiskRename(THIS_DISK,pExisting,pNewName);
	if(iErr)
		{
		SetError(iErr);
		return FALSE;
		}


return TRUE;
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
void CFtpDrives::Close()
{

	if(!m_pSession)return;
	DiskUnmount(THIS_DISK);
	delete m_pSession;
	m_pSession = NULL;


}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertgt eine Datei zum FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	pRemoteFile		: Name der Datei am FTP-Server.
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpDrives::PutFile(LPCTSTR pLocalFile,LPCTSTR pRemoteFile,DWORD dwFlags,DWORD dwContext)
{
LPBYTE 		pBuffer;
QWORD		qwDelta;
DWORD		dwBytes;
DWORD		dwHigh;
HANDLE		hFile;
DiskFile	hDisk;
int			iErr;
int			iRet;


	m_uProgrMax	= 0;
	m_uProgress = 0;

	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}
											// ffne die lokale Quelldatei
	   hFile = CreateFile(pLocalFile,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		m_dwErrOpen = 1;
		return FALSE;
		}

	m_dwErrOpen = 2;
											// Datei Gre holen
	   m_uProgrMax = GetFileSize(hFile,&dwHigh);
	if(m_uProgrMax==0xFFFFFFFF && GetLastError()!=NO_ERROR)
		{
		CloseHandle(hFile);
		return FALSE;
		}


	m_uProgrMax += (QWORD)dwHigh<<32;

											// Erstelle die Zieldatei auf Disk
		hDisk = DiskFileOpenEx(THIS_DISK,pRemoteFile,DPD_WRITE|DPD_CREATE,&iErr);
	if(!hDisk)
		{
		CloseHandle(hFile);
		SetError(iErr);
		return FALSE;
		}


	iRet = TRUE;


	if(!InterlockedExchange(&m_lLocked,1))	// Puffer holen
		{
		if(!m_pBuffer)m_pBuffer=new BYTE[BUFFER_SIZE];
		pBuffer = m_pBuffer;
		}
	else{
		pBuffer = new BYTE[BUFFER_SIZE];
		}


	for(;m_uProgress<m_uProgrMax;m_uProgress+=dwBytes)
		{
		if(m_bBreak)						// User-Abbruch
			{
			SetLastError(ERROR_OPERATION_ABORTED);
			iRet = FALSE;
			break;
			}

		   qwDelta = m_uProgrMax-m_uProgress;
		if(qwDelta>BUFFER_SIZE)
				dwBytes = BUFFER_SIZE;
		else	dwBytes = (DWORD)qwDelta;

											// Daten aus lokalen Ziel laden
		if(!ReadFile(hFile,pBuffer,dwBytes,&dwBytes,NULL))
			{
			SetLastError(ERROR_READ_FAULT);
			iRet = FALSE;
			break;
			}
											// Daten in Ziel auf Disk speichern
		   iErr = DiskFileWrite(THIS_DISK,hDisk,pBuffer,dwBytes);
		if(iErr<0)
			{
			SetError(iErr);
			iRet = FALSE;
			break;
			}

		if((unsigned)iErr<dwBytes)			// Wurde alles gespeichert
			{
			SetLastError(ERROR_DISK_FULL);
			iRet = FALSE;
			break;
			}
		}


	if(pBuffer!=m_pBuffer)					// Puffer wieder freigeben
		{
		delete pBuffer;
		}
	else{
		m_lLocked = 0;
		}


	DiskFileClose(THIS_DISK,hDisk);
	CloseHandle(hFile);
	m_bBreak = FALSE;




return iRet;
}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpDrives::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{
LPBYTE 		pBuffer;
DiskFile	hDisk;
int			iErr;
int			iRet;
int			iLen;


	m_uProgrMax	= 0;
	m_uProgress = 0;

	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	m_dwErrOpen = 2;
	m_uProgrMax = m_dwPutSize;

											// Erstelle die Zieldatei auf Disk
		hDisk = DiskFileOpenEx(THIS_DISK,pRemoteFile,DPD_WRITE|DPD_CREATE,&iErr);
	if(!hDisk)
		{
		SetError(iErr);
		return FALSE;
		}


	iRet = TRUE;


	if(!InterlockedExchange(&m_lLocked,1))	// Puffer holen
		{
		if(!m_pBuffer)m_pBuffer=new BYTE[BUFFER_SIZE];
		pBuffer = m_pBuffer;
		}
	else{
		pBuffer = new BYTE[BUFFER_SIZE];
		}


	for(;m_uProgress<m_uProgrMax;m_uProgress+=iLen)
		{
		if(m_bBreak)						// User-Abbruch
			{
			SetLastError(ERROR_OPERATION_ABORTED);
			iRet = FALSE;
			break;
			}
											// Daten via Callback holen
		   iLen = pProc(pParam,pBuffer,BUFFER_SIZE);
		if(iLen<=0)
			{
			SetLastError((iLen<0)? ERROR_READ_FAULT:ERROR_SUCCESS);
			iRet = (iLen<0)? FALSE:TRUE;
			break;
			}
											// Daten in Ziel auf Disk speichern
		   iErr = DiskFileWrite(THIS_DISK,hDisk,pBuffer,iLen);
		if(iErr<0)
			{
			SetError(iErr);
			iRet = FALSE;
			break;
			}

		if(iErr!=iLen)						// Wurde alles gespeichert
			{
			SetLastError(ERROR_DISK_FULL);
			iRet = FALSE;
			break;
			}
		}


	if(pBuffer!=m_pBuffer)					// Puffer wieder freigeben
		{
		delete pBuffer;
		}
	else{
		m_lLocked = 0;
		}


	DiskFileClose(THIS_DISK,hDisk);
	m_bBreak = FALSE;




return iRet;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei vom FTP-Server.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	bFailIfExists	: Eine vorhandene Datei nicht berschreiben
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpDrives::GetFile(LPCTSTR pRemoteFile,LPCTSTR pLocalFile,BOOL bFailIfExists,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
LPBYTE 		pBuffer;
QWORD		qwDelta;
DWORD		dwBytes;
HANDLE		hFile;
DiskFile	hDisk;
int			iErr;
int			iRet;


	m_uProgrMax	= 0;
	m_uProgress = 0;
	m_dwErrOpen = 0;

	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	if(pRemoteFile[0]=='/' || pRemoteFile[0]=='\\')
	if(pRemoteFile[1]!= 0  && pRemoteFile[2]==':' )
		{
		pRemoteFile++;
		}

											// ffne die Datei auf der DIsk
		hDisk = DiskFileOpenEx(THIS_DISK,pRemoteFile,DPD_READ,&iErr);
	if(!hDisk)
		{
		m_dwErrOpen = 2;
		SetError(iErr);
		return FALSE;
		}
											// Gre der Datei abfragen
   m_uProgrMax = DiskGetFileSizeEx(THIS_DISK,hDisk);

											// Erstelle die lokale Zieldatei
	   hFile = CreateFile(pLocalFile,GENERIC_READ|GENERIC_WRITE,0,NULL,(bFailIfExists)? CREATE_NEW:CREATE_ALWAYS,FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		DiskFileClose(THIS_DISK,hDisk);
		m_dwErrOpen = 1;
		return FALSE;
		}

	m_dwErrOpen = 2;
	iRet		= TRUE;

	if(!InterlockedExchange(&m_lLocked,1))	// Puffer holen
		{
		if(!m_pBuffer)m_pBuffer=new BYTE[BUFFER_SIZE];
		pBuffer = m_pBuffer;
		}
	else{
		pBuffer = new BYTE[BUFFER_SIZE];
		}

	for(;m_uProgress<m_uProgrMax;m_uProgress+=dwBytes)
		{
		if(m_bBreak)						// User-Abbruch
			{
			SetLastError(ERROR_OPERATION_ABORTED);
			iRet = FALSE;
			break;
			}

		   qwDelta = m_uProgrMax-m_uProgress;
		if(qwDelta>BUFFER_SIZE)
				dwBytes = BUFFER_SIZE;
		else	dwBytes = (DWORD)qwDelta;

											// Daten von Disk laden
		   iErr = DiskFileRead(THIS_DISK,hDisk,pBuffer,dwBytes);
		if(iErr<0)
			{
			SetError(iErr);
			iRet = FALSE;
			break;
			}

		if((unsigned)iErr<dwBytes)			// Wurde alles gespeichert
			{
			SetLastError(ERROR_READ_FAULT);
			iRet = FALSE;
			break;
			}
											// Daten im lokalen Ziel speichern
		if(!WriteFile(hFile,pBuffer,dwBytes,&dwBytes,NULL))
			{
			iRet = FALSE;
			break;
			}
		}


	if(pBuffer!=m_pBuffer)					// Puffer wieder freigeben
		{
		delete pBuffer;
		}
	else{
		m_lLocked = 0;
		}


	DiskFileClose(THIS_DISK,hDisk);
	CloseHandle(hFile);
	m_bBreak = FALSE;



return iRet;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpDrives::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
LPBYTE 		pBuffer;
QWORD		qwDelta;
DWORD		dwBytes;
DiskFile	hDisk;
int			iErr;
int			iRet;
int			iLen;


	m_uProgrMax	= 0;
	m_uProgress = 0;
	m_dwErrOpen = 0;

	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	if(pRemoteFile[0]=='/' || pRemoteFile[0]=='\\')
	if(pRemoteFile[1]!= 0  && pRemoteFile[2]==':' )
		{
		pRemoteFile++;
		}

											// ffne die Datei auf der DIsk
		hDisk = DiskFileOpenEx(THIS_DISK,pRemoteFile,DPD_READ,&iErr);
	if(!hDisk)
		{
		m_dwErrOpen = 2;
		SetError(iErr);
		return FALSE;
		}
											// Gre der Datei abfragen
	m_uProgrMax = DiskGetFileSizeEx(THIS_DISK,hDisk);
	m_dwErrOpen = 2;
	iRet		= TRUE;

	if(!InterlockedExchange(&m_lLocked,1))	// Puffer holen
		{
		if(!m_pBuffer)m_pBuffer=new BYTE[BUFFER_SIZE];
		pBuffer = m_pBuffer;
		}
	else{
		pBuffer = new BYTE[BUFFER_SIZE];
		}

	for(;;m_uProgress+=iLen)
		{
		if(m_uProgress>=m_uProgrMax)		// Ende erreicht
			{
			pProc(pParam,NULL,0);
			break;
			}

		if(m_bBreak)						// User-Abbruch
			{
			pProc(pParam,NULL,-1);
			SetLastError(ERROR_OPERATION_ABORTED);
			iRet = FALSE;
			break;
			}

		   qwDelta = m_uProgrMax-m_uProgress;
		if(qwDelta>BUFFER_SIZE)
				dwBytes = BUFFER_SIZE;
		else	dwBytes = (DWORD)qwDelta;

											// Daten von Disk laden
		   iErr = DiskFileRead(THIS_DISK,hDisk,pBuffer,dwBytes);
		if(iErr<0)
			{
			pProc(pParam,NULL,-1);
			SetError(iErr);
			iRet = FALSE;
			break;
			}

		if((unsigned)iErr<dwBytes)			// Wurde alles gespeichert
			{
			pProc(pParam,NULL,-1);
			SetLastError(ERROR_READ_FAULT);
			iRet = FALSE;
			break;
			}
											// Daten lokalen speichern via Callback
		   iLen = pProc(pParam,pBuffer,iErr);
		if(iLen!=iErr)
			{
			SetLastError(ERROR_WRITE_FAULT);
			iRet = FALSE;
			break;
			}
		}


	if(pBuffer!=m_pBuffer)					// Puffer wieder freigeben
		{
		delete pBuffer;
		}
	else{
		m_lLocked = 0;
		}


	DiskFileClose(THIS_DISK,hDisk);
	m_bBreak = FALSE;



return iRet;
}

//*****************************************************************************
//*
//*		List
//*
//*****************************************************************************
//	Listet alle Dateien in einem Verzeichnis via Callback auf.
//	pRemotePath		: Ist der Name des Verzeichnisses.
//	pParam			: Parameter fr den Callback.
//	pProc			: Zeiger auf den Callback.
//	pFilter			: Filter der benutzt werden kann.
//	Ergibt wenn TRUE alles ok war, bzw. FALSE bei einem Fehler.
BOOL CFtpDrives::List(LPCTSTR pRemotePath,void *pParam,FtpFindCallback pProc,LPCTSTR pFilter)
{
TCHAR			aBuffer[512];
WIN32_FIND_DATA	sData;
DiskFindStruct	sFind;
DiskFind		hDisk;
int    			iErr;
int				iRet;
int				iLen;


	if(!m_pSession)
		{
		pProc(pParam,NULL);
		return FALSE;
		}


	   iLen = strlcpy(aBuffer,pRemotePath,510);
	if(iLen>0 && aBuffer[iLen-1]!='/' && aBuffer[iLen-1]!='\\')
		{
		aBuffer[iLen] = '/';
		iLen++;
		}

	if(!pFilter)pFilter=_T("*");

	strlcpy(aBuffer+iLen,pFilter,512-iLen);

		hDisk = DiskFindFirstEx(THIS_DISK,aBuffer,&sFind,&iErr);
	if(!hDisk)
		{
		SetError(iErr);
		pProc(pParam,NULL);
		return FALSE;
		}



	sData.dwReserved0			= 0;
	sData.dwReserved1			= 0;
	sData.cAlternateFileName[0]	= 0;

	for(;;)
		{
		if(m_bBreak)break;					// User-Abbruch

		strlcpy(sData.cFileName,sFind.cName,MAX_PATH);

		sData.nFileSizeHigh	= (sFind.dwAttr>>DPD_HIGHSHIFT)&DPD_HIGHMASK;
		sData.nFileSizeLow	= (sFind.dwSize);

		UtcTimeToFileTime(sFind.dwTimeAccess,&sData.ftLastAccessTime);
		UtcTimeToFileTime(sFind.dwTimeCreate,&sData.ftCreationTime  );
		UtcTimeToFileTime(sFind.dwTimeWrite ,&sData.ftLastWriteTime );

		if(sFind.dwAttr&DPD_WRITE  )sData.dwFileAttributes = 0;
		else                        sData.dwFileAttributes =FILE_ATTRIBUTE_READONLY;
		if(sFind.dwAttr&DPD_HIDDEN )sData.dwFileAttributes|=FILE_ATTRIBUTE_HIDDEN;
		if(sFind.dwAttr&DPD_SYSTEM )sData.dwFileAttributes|=FILE_ATTRIBUTE_SYSTEM;
		if(sFind.dwAttr&DPD_ARCHIVE)sData.dwFileAttributes|=FILE_ATTRIBUTE_ARCHIVE;
		if(sFind.dwAttr&DPD_DIR    )sData.dwFileAttributes|=FILE_ATTRIBUTE_DIRECTORY;


		if(pProc(pParam,&sData)==0)break;
		if(DiskFindNext(THIS_DISK,hDisk,&sFind)<0)break;
		}

	DiskFindClose(THIS_DISK,hDisk);
	pProc(pParam,NULL);

	iRet = (m_bBreak)? FALSE:TRUE;
	m_bBreak = FALSE;


return iRet;
}

//*****************************************************************************
//*
//*		SetAttributes
//*
//*****************************************************************************
//	Einstellen der Dateiattribute
//	pFileName		: Ist der Name der Datei.
//	dwAttr			: Sind die neuen Attribute.
//					  In den Bits 16..31 knnen Unix-Attribute verpackt werden.
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpDrives::SetAttributes(LPCTSTR pFileName,DWORD dwAttr)
{
BYTE		uFlags;
int    		iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	if(dwAttr&FILE_ATTRIBUTE_READONLY)uFlags =DPD_READ;
	else                              uFlags =DPD_READ|DPD_WRITE;
	if(dwAttr&FILE_ATTRIBUTE_HIDDEN  )uFlags|=DPD_HIDDEN;
	if(dwAttr&FILE_ATTRIBUTE_SYSTEM  )uFlags|=DPD_SYSTEM;
	if(dwAttr&FILE_ATTRIBUTE_ARCHIVE )uFlags|=DPD_ARCHIVE;


	    iErr = DiskSetAttributes(THIS_DISK,pFileName,uFlags);
	if(!iErr)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		SetModifyTime
//*
//*****************************************************************************
//	Einstellen der nderungszeit einer Datei.
//	pFileName		: Ist der Name der Datei.
//	sFileTime		: Ist die neue nderungszeit
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpDrives::SetModifyTime(LPCTSTR pFileName,FILETIME sFileTime)
{
DiskFile	hDisk;
time_t		iTime;
int    		iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}


	if(!FileTimeToUtcTime(&sFileTime,&iTime))
		{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
		}


		hDisk = DiskFileOpenEx(THIS_DISK,pFileName,DPD_READ|DPD_WRITE,&iErr);
	if(!hDisk)
		{
		SetError(iErr);
		return FALSE;
		}

	iErr = DiskSetFileTime(THIS_DISK,hDisk,iTime,0,0);
	DiskFileClose(THIS_DISK,hDisk);

	if(!iErr)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		DoAbort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpDrives::DoAbort()
{

	if(!m_pSession)return;

	m_bBreak = TRUE;

}

//*****************************************************************************
//*
//*		SetError
//*
//*****************************************************************************
//	Stellt die Windows Fehlernummer ein
//	iError		: Ist die FTP Fehlernummer
void CFtpDrives::SetError(int iError) const
{


	switch(iError)
		{
	case DPD_ERR_NONE:			   SetLastError(ERROR_SUCCESS);				return;	// Kein Fehler
	case DPD_ERR_PARAM:			   SetLastError(ERROR_INVALID_PARAMETER);	return;	// Falscher Parameter
	case DPD_ERR_INVALID_FILENAME: SetLastError(ERROR_INVALID_NAME);		return;	// Ungltiger Dateiname
	case DPD_ERR_FILE_NOT_FOUND:   SetLastError(ERROR_FILE_NOT_FOUND);		return;	// Die Datei wurde nicht gefunden
	case DPD_ERR_ACCESS_DENIED:	   SetLastError(ERROR_ACCESS_DENIED);		return;	// Zugriff verweigert
	case DPD_ERR_NO_HANDLES:	   SetLastError(ERROR_TOO_MANY_OPEN_FILES);	return;	// Keine freien Handels
	case DPD_ERR_UNKNOWN:		   SetLastError(ERROR_INVALID_ACCESS);		return;	// Unbekannter Fehler
	case DPD_ERR_PATH_NOT_FOUND:   SetLastError(ERROR_PATH_NOT_FOUND);		return;	// Der Pfad wurde nicht gefunden
	case DPD_ERR_DISKFULL:		   SetLastError(ERROR_DISK_FULL);			return;	// Die Partitione ist voll
	case DPD_ERR_ALREADY_EXISTS:   SetLastError(ERROR_FILE_EXISTS);			return;	// Die Datei existiert bereits
		}


	SetLastError(ERROR_INVALID_ACCESS);

}

//*****************************************************************************
//*
//*		Init
//*
//*****************************************************************************
//	iReadOnly	: Nur Read-Only Zugriff
BOOL CFtpDrives::Init(int iReadOnly)
{
TCHAR	cPath[CFB_MAX_PATH];
UINT	uFlags;
UINT	uLen;



	uLen    = strhtmldecode(m_sServerName,cPath,CFB_MAX_PATH,0x04);
	uFlags	= DPD_FLAG_INIT|DPD_FLAG_NO_EMPTY|DPD_FLAG_SINGLE;

	if(iReadOnly)uFlags|=DPD_FLAG_READONLY;

	if(DiskMount(THIS_DISK,cPath,uFlags,NULL,m_uPartition)<=1)
		{
		SetLastError(ERROR_PARTITION_FAILURE);
		return FALSE;
		}




return TRUE;
}

//*****************************************************************************
//*
//*		SetStat
//*
//*****************************************************************************
//	Einstellen der Dateiattribute und der Zeiten.
//	pInfo		: Sind die neuen Zeiten und Attribute
//				  Wenn pInfo->dwFileAttributes ungleich 0 und -1 ist werden die
//				  Attribute	verndert. Die Zeiten werden gesetzt wenn sie ungleich
//				  0 sind. In pInfo->cFileName wird der Dateinamge bergeben.
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpDrives::SetStat(LPWIN32_FIND_DATA pInfo)
{
DiskFile	hDisk;
time_t		iTimeC;
time_t		iTimeW;
time_t		iTimeA;
int			iFlags;
int    		iErr;



	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}


	FileTimeToUtcTime(&pInfo->ftCreationTime  ,&iTimeC);
	FileTimeToUtcTime(&pInfo->ftLastWriteTime ,&iTimeW);
	FileTimeToUtcTime(&pInfo->ftLastAccessTime,&iTimeA);


	if(iTimeC|iTimeW|iTimeA)							// Zeiten setzen
		{
			hDisk = DiskFileOpenEx(THIS_DISK,pInfo->cFileName,DPD_READ|DPD_WRITE,&iErr);
		if(!hDisk)
			{
			SetError(iErr);
			return FALSE;
			}

		iErr = DiskSetFileTime(THIS_DISK,hDisk,iTimeW,iTimeC,iTimeA);
		DiskFileClose(THIS_DISK,hDisk);
		}

														// Attribute setzen
	if(pInfo->dwFileAttributes && pInfo->dwFileAttributes!=0xFFFFFFFF && iErr==0)
		{
		if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_READONLY)iFlags =DPD_READ;
		else				                               iFlags =DPD_READ|DPD_WRITE;
		if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_HIDDEN  )iFlags|=DPD_HIDDEN;
		if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_SYSTEM  )iFlags|=DPD_SYSTEM;
		if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_ARCHIVE )iFlags|=DPD_ARCHIVE;

	    iErr = DiskSetAttributes(THIS_DISK,pInfo->cFileName,iFlags);
		}


	if(!iErr)return TRUE;

	SetError(iErr);


return FALSE;
}
