//*****************************************************************************
//*
//*
//*		FtpNetwork.cpp
//*
//*
//*****************************************************************************
//
//	Emuliert ein Netzlaufwerk oder Laufwerk als FTP-Verbindung
//
#include	<Afx.h>
#include	"FtpTools.h"
#include	"FtpNetwork.h"
#include	"../ShFind.h"
#include	"../MemFunctions.h"
#include	"../FileFunctions.h"
#include	"../StringFunctions.h"


#pragma		comment(lib,"mpr.lib")



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpNetwork::CFtpNetwork(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive) : CFtpBase(pSession,pServer,pUserName,pPassword,dwContext,nPort,bPassive)
{

	m_cBaseDir[0] = 0;
	m_cCurrDir[0] = 0;
	m_bLogoff	  = 0;
	m_bCancel	  = 0;

	if(pServer[0])
		{
		m_bIsDrive = 0;
		}
	else{
		m_bIsDrive = 1;
		}

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpNetwork::~CFtpNetwork()
{

	if(m_pSession)Close();

}


//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpNetwork::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{
DWORD	   *pVal=(DWORD*)pBuffer;
DWORD		uLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	uLen = *pLen;


	switch(dwOption)
		{
	case INTERNET_OPTION_MODE:						// Verbindungsmodus

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_SMB;

		return TRUE;

	case INTERNET_OPTION_PROGRESS:					// Verlaufsanzeige

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = (unsigned)m_iProgressPos;

		return TRUE;

	case INTERNET_OPTION_PROGRESS2:					// Verlaufsanzeige mit Maximalwert

		if(uLen<sizeof(DWORD)*2)
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = (unsigned)m_iProgressPos;
		pVal[1] = (unsigned)m_iProgressSize;

		return TRUE;

	case INTERNET_OPTION_LOCAL:						// Ist es eine lokale Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = (unsigned)m_bIsDrive;

		return TRUE;
	
	case INTERNET_OPTION_PATHMODE:					// Pfadmodus abfragen (0=ftp:// 1=//... 2=D:\...)

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = (m_bIsDrive)? 2:1;

		return TRUE;

	case INTERNET_OPTION_CONNTYPE:					// Art der Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_SMB;

		return TRUE;

	case INTERNET_OPTION_URLFORMAT:					// Format des URL-Namens

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 0;

		return TRUE;

	case INTERNET_OPTION_ERROPEN:					// Wo gab es den Fehler

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_dwErrOpen;

		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpNetwork::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{

	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	switch(dwOption)
		{
	case INTERNET_OPTION_PROGRSET:					// Verlaufsanzeige einstellen

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_iProgressPos = *(int*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_SETSTAT:					// Attribute und Zeiten setzen

		if(uLength<sizeof(WIN32_FIND_DATA))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return SetStat((LPWIN32_FIND_DATA)pBuffer);

	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}
		
		m_dwPutSize = *(LPDWORD)pBuffer;		
		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpNetwork::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{

	if(!m_pSession)return NULL;

	if(pFileName[0]=='/' || pFileName[0]=='\\')
	if(pFileName[1]!= 0  && pFileName[2]==':' )
		{
		pFileName++;
		}

	return NULL;
}

//*****************************************************************************
//*
//*		SetCurrentDirectory
//*
//*****************************************************************************
//	Stellt das aktuelle Verzeichnis ein.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/" "/D:/aaa" "D:/aaaa"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpNetwork::SetCurrentDirectory(LPCTSTR pDirName)
{
TCHAR	cBuffer[CFB_MAX_PATH];
TCHAR	cTemp  [CFB_MAX_PATH];
UINT	uAttr;
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pDirName[0]=='/' || pDirName[0]=='\\')				// "/" und "//" sind Root-Verzeichnis
		{
		if(pDirName[1]==0)
			{
			m_cCurrDir[0] = '/';
			m_cCurrDir[1] = 0;
			return TRUE;
			}

		if(pDirName[2]==0 && (pDirName[1]=='/' || pDirName[1]=='\\'))
			{
			m_cCurrDir[0] = '/';
			m_cCurrDir[1] = 0;
			return TRUE;
			}
		}


	if(m_bIsDrive)
		{
		if(pDirName[0]=='/' || pDirName[0]=='\\')
			{
			if(pDirName[1] && pDirName[2]==':')				// "/D:..." zu "D:..."
				{
				pDirName++;
				}
			else if(m_cCurrDir[0] && m_cCurrDir[1]==':')	// "/xxxxx" zu "D:/xxxxx"
				{
				cTemp[0]      = m_cCurrDir[0];
				cTemp[1]      = ':';
				strlcpy(cTemp+2,pDirName,sizeof(cTemp)/sizeof(TCHAR)-2);
				pDirName      = cTemp;
				}
			}

		iLen = 0;
		}
	else{													// Kein Laufwerk
		iLen = strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
		}


	PathFullMake(cBuffer+iLen,m_cCurrDir,pDirName,sizeof(cBuffer)/sizeof(TCHAR),0);
	DelBackSlash(cBuffer+iLen);
	PathConvert (cBuffer+iLen,'/');
															// Im Root Rerzeichniess
	if(cBuffer[iLen]==0 && (m_cCurrDir[0]==0 || m_cCurrDir[1]==0))
		{
		m_cCurrDir[0] = '/';
		m_cCurrDir[1] = 0;

		return TRUE;
		}


	uAttr = GetFileAttributes(cBuffer);
															// Ist es ein Verzeichnis
	if(uAttr==0xFFFFFFFF || !(uAttr&FILE_ATTRIBUTE_DIRECTORY))
		{
		return FALSE;
		}


	strlcpy(m_cCurrDir,cBuffer+iLen,sizeof(m_cCurrDir)/sizeof(TCHAR));



return TRUE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis.
//	pDirName		: Puffer fr den Namen
//	pLen			: Gre des Puffers
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpNetwork::GetCurrentDirectory(LPTSTR pDirName,LPDWORD pLen) const
{


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(m_bIsDrive && m_cCurrDir[0]!='/' && m_cCurrDir[0]!='\\')
		{
		*pDirName = '/';
		*pLen = strlcpy(pDirName+1,m_cCurrDir,*pLen-1)+1;
		}
	else{
		*pLen = strlcpy(pDirName,m_cCurrDir,*pLen);
		}



return TRUE;
}


//*****************************************************************************
//*
//*		RemoveDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des zu lschenden Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/" "D:/aaaa" "/D:/aaa/"
//	Ergibt TRUE wenn das Verzeichnis gelscht wurde.
BOOL CFtpNetwork::RemoveDirectory(LPCTSTR pDirName)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pDirName[0]=='/' || pDirName[0]=='\\')
	if(pDirName[1]!= 0  && pDirName[2]==':' )
		{
		pDirName++;
		}

	iLen = strlcpy     (cBuffer     ,m_cBaseDir         ,sizeof(cBuffer)/sizeof(TCHAR));
	iLen = PathFullMake(cBuffer+iLen,m_cCurrDir,pDirName,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(iLen>2 && cBuffer[iLen-2]!=':')DelBackSlash(cBuffer,iLen);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}

	if(iLen<=2)										// Laufwerk kann nicht gelscht werden
		{
		SetLastError(ERROR_ACCESS_DENIED);
		return FALSE;
		}


return ::RemoveDirectory(cBuffer);
}

//*****************************************************************************
//*
//*		CreateDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/" "D:/aaaa" "/D:/aaa/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpNetwork::CreateDirectory(LPCTSTR pDirName)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pDirName[0]=='/' || pDirName[0]=='\\')
	if(pDirName[1]!= 0  && pDirName[2]==':' )
		{
		pDirName++;
		}

	iLen = strlcpy     (cBuffer     ,m_cBaseDir         ,sizeof(cBuffer)/sizeof(TCHAR));
	iLen = PathFullMake(cBuffer+iLen,m_cCurrDir,pDirName,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(iLen>2 && cBuffer[iLen-2]!=':')
		{
		iLen = DelBackSlash(cBuffer,iLen);
		}

	if(m_bIsDrive && (iLen<2 || cBuffer[1]!=':'))	// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}


return ::CreateDirectory(cBuffer,0);
}

//*****************************************************************************
//*
//*		Remove
//*
//*****************************************************************************
//	Lscht eine Datei..
//	pFileName		: Ist der Name der Datei
//	Ergibt TRUE wenn die Datei gelscht wurde.
BOOL CFtpNetwork::Remove(LPCTSTR pFileName)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pFileName[0]=='/' || pFileName[0]=='\\')
	if(pFileName[1]!= 0  && pFileName[2]==':' )
		{
		pFileName++;
		}

	iLen = strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	iLen = PathFullMake(cBuffer+iLen,m_cCurrDir,pFileName,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && (iLen<2 || cBuffer[1]!=':'))	// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		return FALSE;
		}


return DeleteFile(cBuffer);
}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	Verschiebt eine Datei..
//	pExisting		: Ist der Name der Datei.
//	pNewName		: Ist der Name der Zieldatei.
//	Ergibt TRUE wenn die Datei vershoben wurde.
BOOL CFtpNetwork::Rename(LPCTSTR pExisting,LPCTSTR pNewName)
{
TCHAR	cBuffer[CFB_MAX_PATH];
TCHAR	cTarget[CFB_MAX_PATH];
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pExisting[0]=='/' || pExisting[0]=='\\')
	if(pExisting[1]!= 0  && pExisting[2]==':' )
		{
		pExisting++;
		}

	if(pNewName[0]=='/' || pNewName[0]=='\\')
	if(pNewName[1]!= 0  && pNewName[2]==':' )
		{
		pNewName++;
		}

	iLen=strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pExisting,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	iLen=strlcpy(cTarget,m_cBaseDir,sizeof(cTarget)/sizeof(TCHAR));
	PathFullMake(cTarget+iLen,m_cCurrDir,pNewName,sizeof(cTarget)/sizeof(TCHAR)-iLen,0);


	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		return FALSE;
		}

	if(m_bIsDrive && cTarget[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}


return MoveFile(cBuffer,cTarget);
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
void CFtpNetwork::Close()
{

	if(!m_pSession)return;

	if(m_bLogoff)
		{
		m_cBaseDir[2] = 0;
		WNetCancelConnection2(m_cBaseDir,0,TRUE);
		}

	m_pSession = 0;

}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertgt eine Datei zum FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	pRemoteFile		: Name der Datei am FTP-Server.
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpNetwork::PutFile(LPCTSTR pLocalFile,LPCTSTR pRemoteFile,DWORD dwFlags,DWORD dwContext)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		iRet;
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pRemoteFile[0]=='/' || pRemoteFile[0]=='\\')
	if(pRemoteFile[1]!= 0  && pRemoteFile[2]==':' )
		{
		pRemoteFile++;
		}


	iLen=strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pRemoteFile,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}

	m_bCancel			= 0;
	m_bAbortAllowed		= 1;
	m_iProgressSize		= 0;
	m_iProgressPos		= 0;

	iRet = CopyFileProgress(pLocalFile,cBuffer,&m_iProgressPos,&m_iProgressSize,&m_bCancel);

	m_bAbortAllowed				= 0;


return (iRet>0)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpNetwork::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{
TCHAR	cBuffer[0x4000/sizeof(TCHAR)];
DWORD	dwBytes;
HANDLE	hFile;
int		iRet;
int		iLen;
int		iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pRemoteFile[0]=='/' || pRemoteFile[0]=='\\')
	if(pRemoteFile[1]!= 0  && pRemoteFile[2]==':' )
		{
		pRemoteFile++;
		}


	iLen=strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pRemoteFile,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}

	m_bCancel			= 0;
	m_bAbortAllowed		= 1;
	m_iProgressSize		= 0;
	m_iProgressPos		= 0;

	   hFile = CreateFile(cBuffer,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		return FALSE;
		}

	m_iProgressSize = m_dwPutSize;

	for(;;)
		{  
		if(m_bCancel)								// Gibt ese eine User-Abbruch
			{
			iErr = ERROR_CANCELLED;
			iRet = 0;
			break;
			}
													// Daten lesen via Callback															
		   iLen = pProc(pParam,cBuffer,sizeof(cBuffer));	
		if(iLen<=0)
			{
			iErr = (iLen)? ERROR_WRITE_FAULT:ERROR_SUCCESS;
			iRet = (iLen)? 0:1;
			break;
			}

													// Daten aus Datei lesen
			iRet = WriteFile(hFile,cBuffer,iLen,&dwBytes,NULL);
		if(!iRet || (dwBytes^iLen))
			{
			iErr = GetLastError();
			break;
			}
		
		m_iProgressPos += dwBytes;
		}

	CloseHandle(hFile);
	SetLastError(iErr);


	m_bAbortAllowed				= 0;


return (iRet>0)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei vom FTP-Server.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	bFailIfExists	: Eine vorhandene Datei nicht berschreiben
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpNetwork::GetFile(LPCTSTR pRemoteFile,LPCTSTR pLocalFile,BOOL bFailIfExists,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		iRet;
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pRemoteFile[0]=='/' || pRemoteFile[0]=='\\')
	if(pRemoteFile[1]!= 0  && pRemoteFile[2]==':' )
		{
		pRemoteFile++;
		}

	iLen=strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pRemoteFile,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}

	m_bCancel			= 0;
	m_bAbortAllowed		= 1;
	m_iProgressSize		= 0;
	m_iProgressPos		= 0;


	iRet = CopyFileProgress(cBuffer,pLocalFile,&m_iProgressPos,&m_iProgressSize,&m_bCancel,(bFailIfExists)? CFP_FAILIFEXIST:0);

	m_bAbortAllowed				= 0;


return (iRet==1)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpNetwork::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
TCHAR	cBuffer[0x4000/sizeof(TCHAR)];	
DWORD	dwBytes;
HANDLE	hFile;
int		iErr;
int		iRet;
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pRemoteFile[0]=='/' || pRemoteFile[0]=='\\')
	if(pRemoteFile[1]!= 0  && pRemoteFile[2]==':' )
		{
		pRemoteFile++;
		}

	iLen=strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pRemoteFile,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}

	m_bCancel			= 0;
	m_bAbortAllowed		= 1;
	m_iProgressSize		= 0;
	m_iProgressPos		= 0;

	   hFile = CreateFile(cBuffer,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		return FALSE;
		}

	LengthFile(hFile,&m_iProgressSize);

	for(;;)
		{  
		if(m_bCancel)								// Gibt ese eine User-Abbruch
			{
			pProc(pParam,NULL,-1);
			iErr = ERROR_CANCELLED;
			iRet = 0;
			break;
			}
													// Daten aus Datei lesen
			iRet = ReadFile(hFile,cBuffer,sizeof(cBuffer),&dwBytes,NULL);
		if(!iRet || !dwBytes)
			{
			pProc(pParam,NULL,(!iRet && !dwBytes)? -1:0);
			iErr = GetLastError();
			break;
			}

		   iRet = pProc(pParam,cBuffer,dwBytes);	// Daten schreiben via Callback
		if(iRet^dwBytes)
			{
			iErr = ERROR_WRITE_FAULT;
			iRet = 0;
			break;
			}
		
		m_iProgressPos += dwBytes;
		}

	CloseHandle(hFile);
	SetLastError(iErr);

	m_bAbortAllowed	= 0;



return (iRet)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		List
//*
//*****************************************************************************
//	Liste alle Dateien in einem Verzeichnis via Callback auf.
//	pRemotePath		: Ist der Name des Verzeichnisses.
//	pParam			: Parameter fr den Callback.
//	pProc			: Zeiger auf den Callback.
//	pFilter			: Filter der benutzt werden kann.
//	Ergibt wenn TRUE alles ok war, bzw. FALSE bei einem Fehler.
BOOL CFtpNetwork::List(LPCTSTR pRemotePath,void *pParam,FtpFindCallback pProc,LPCTSTR pFilter)
{
TCHAR			cBuffer[CFB_MAX_PATH];
WIN32_FIND_DATA	sData;
HANDLE			hFind;
unsigned		uBits;
int				iErr;
int				iLen;
int				iPos;

	if(!m_pSession)
		{
		pProc(pParam,NULL);
		return FALSE;
		}

	if(!*pRemotePath)
		{
		iLen=strlcpy(cBuffer     ,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
		iLen=strlcpy(cBuffer+iLen,m_cCurrDir,sizeof(cBuffer)/sizeof(TCHAR)-iLen)+iLen;
		}
	else if(!m_bIsDrive && strwchar(*pRemotePath,0x06) && pRemotePath[1]==':')
		{
		iLen=strlcpy(cBuffer     ,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
		cBuffer[iLen] = '\\'; iLen++;
		iLen=strlcpy(cBuffer+iLen,m_cCurrDir,sizeof(cBuffer)/sizeof(TCHAR)-iLen)+iLen;
		}
	else{
		iLen=strlcpy(cBuffer,m_cBaseDir                      ,sizeof(cBuffer)/sizeof(TCHAR));
		if(iLen>1 && strwchar(cBuffer[iLen-1],0x00300000) && strwchar(*pRemotePath,0x00300000))iLen--;
		iLen=PathFullMake(cBuffer+iLen,m_cCurrDir,pRemotePath,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0)+iLen;
		}

													// "/..." in "..." umwandeln
	if(m_bIsDrive && (cBuffer[0]=='/' || cBuffer[0]=='\\') && (cBuffer[1]!='/' && cBuffer[1]!='\\'))
		{
		memmove(cBuffer,cBuffer+1,iLen*sizeof(TCHAR));
		iLen--;
		}


	if(!cBuffer[0] && m_bIsDrive)					// Nur Laufwerke ausgeben
		{
		if(!pFilter)pFilter=_T("");
		uBits = GetLogicalDrives();

		memset(&sData,0,sizeof(sData));
		sData.cFileName[0] 		= 'A';
		sData.cFileName[1] 		= ':';
		sData.cFileName[2]		=  0;
		sData.dwFileAttributes	= FILE_ATTRIBUTE_DIRECTORY;
		sData.ftCreationTime  .dwHighDateTime = 0x01BF53EB;
		sData.ftCreationTime  .dwLowDateTime  = 0x256D4000;
		sData.ftLastAccessTime.dwHighDateTime = 0x01BF53EB;
		sData.ftLastAccessTime.dwLowDateTime  = 0x256D4000;
		sData.ftLastWriteTime .dwHighDateTime = 0x01BF53EB;
		sData.ftLastWriteTime .dwLowDateTime  = 0x256D4000;

		for(;sData.cFileName[0]<='Z';sData.cFileName[0]++)
			{
			if(uBits&1)
			if(pFilter[0]==0 || !memxcmp(pFilter,sData.cFileName,2,0x03))
				{
				if(!pProc(pParam,&sData))break;
				}

			uBits>>=1;
			}


		pProc(pParam,NULL);

		return TRUE;
		}


	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}

	if(iLen==2 && cBuffer[1]==':')					// "D:" zu "D:\"
		{
		cBuffer[2] = '\\';
		cBuffer[3] = 0;
		iLen	   = 3;
		}


	if(!pFilter)pFilter=_T("*");

													// Shares eines Netzlaufwerkes auflisten
	if(!m_bIsDrive && iLen>2 && strwchar(cBuffer[0],0x00300000) && strwchar(cBuffer[1],0x00300000))
		{
		iPos = strcwlen(cBuffer+2,0x00300000)+2;
		
		if(!cBuffer[iPos] || !cBuffer[iPos+1])
			{
			HANDLE	hNet;
				
				
			   hNet = FindFirstNetwork(cBuffer);
			if(hNet)
				{
				sData.dwFileAttributes					= FILE_ATTRIBUTE_DIRECTORY;
				sData.dwReserved0						= 0;
				sData.dwReserved1						= 0;
				sData.nFileSizeLow						= 0;
				sData.nFileSizeHigh						= 0;
				sData.cAlternateFileName[0]				= 0;
				sData.ftCreationTime.dwLowDateTime		= 0;
				sData.ftCreationTime.dwHighDateTime		= 0;
				sData.ftLastWriteTime.dwLowDateTime		= 0;
				sData.ftLastWriteTime.dwHighDateTime	= 0;
				sData.ftLastAccessTime.dwLowDateTime	= 0;
				sData.ftLastAccessTime.dwHighDateTime	= 0;

				for(;;)
					{
					if(m_bCancel)
						{
						break;
						}

					if(!FindNextNetwork(hNet,sData.cFileName,sizeof(sData.cFileName)))
						{
						break;
						}
													// Den Filter prfen
					if(*pFilter && strwcmp(sData.cFileName,pFilter))
						{
						continue;	
						}
					
					if(pProc(pParam,&sData)==0)break;
					}

				FindCloseNetwork(hNet);
				}

			pProc(pParam,NULL);
			
			return (m_bCancel)? FALSE:TRUE;
			}
		}

													// Filter anhngen
	if(*pFilter && iLen<sizeof(cBuffer)/sizeof(TCHAR)-4)
		{
		if(iLen>0 && cBuffer[iLen-1]!='\\' && cBuffer[iLen-1]!='/')
			{
			cBuffer[iLen] = '\\';
			iLen++;
			}

		strlcpy(cBuffer+iLen,pFilter,sizeof(cBuffer)/sizeof(TCHAR)-iLen);
		}


	m_bCancel = 0;

	   hFind = FindFirstFile(cBuffer,&sData);
	if(hFind==INVALID_HANDLE_VALUE)
		{
		pProc(pParam,NULL);

		if(iLen>0 && cBuffer[iLen-1]!='\\' && cBuffer[iLen-1]!='/')
			{
			iLen--;
			}

		cBuffer[iLen] = 0;
		iErr = GetLastError();

		if(iErr==ERROR_PATH_NOT_FOUND)
			{
			return FALSE;
			}

		if(ExistFile(cBuffer)>=2)
			{
			SetLastError(iErr);
			return FALSE;
			}

		return TRUE;
		}


	do	{
		if(m_bCancel)break;
		sData.dwFileAttributes  &= 0x00FF;
		sData.dwFileAttributes  |= (sData.nFileSizeHigh&0x3F)<<10;
		if(pProc(pParam,&sData)==0)break;
		}while(FindNextFile(hFind,&sData));


	FindClose(hFind);

	pProc(pParam,NULL);


return (m_bCancel)? FALSE:TRUE;
}

//*****************************************************************************
//*
//*		SetAttributes
//*
//*****************************************************************************
//	Einstellen der Dateiattribute
//	pFileName		: Ist der Name der Datei.
//	dwAttr			: Sind die neuen Attribute.
//					  In den Bits 16..31 knnen Unix-Attribute verpackt werden.
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpNetwork::SetAttributes(LPCTSTR pFileName,DWORD dwAttr)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	if(pFileName[0]=='/' || pFileName[0]=='\\')
	if(pFileName[1]!= 0  && pFileName[2]==':' )
		{
		pFileName++;
		}

	dwAttr &= 0xFFFF&~(FILE_ATTRIBUTE_EXEC|FILE_ATTRIBUTE_LINK|FILE_SET_ATTRIBUTE);

	iLen=strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pFileName,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}




return SetFileAttributes(cBuffer,dwAttr);
}

//*****************************************************************************
//*
//*		SetModifyTime
//*
//*****************************************************************************
//	Einstellen der nderungszeit einer Datei.
//	pFileName		: Ist der Name der Datei.
//	sFileTime		: Ist die neue nderungszeit
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpNetwork::SetModifyTime(LPCTSTR pFileName,FILETIME sFileTime)
{
TCHAR	cBuffer[CFB_MAX_PATH];
int		iRet;
int		iLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	if(pFileName[0]=='/' || pFileName[0]=='\\')
	if(pFileName[1]!= 0  && pFileName[2]==':' )
		{
		pFileName++;
		}


	iLen = strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pFileName,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}


	iRet = ChangeFileTime(cBuffer,&sFileTime);



return iRet;
}

//*****************************************************************************
//*
//*		DoAbort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpNetwork::DoAbort()
{

	if(!m_pSession)return;

	m_bCancel		= TRUE;
	m_bAbortAllowed = 0;


}

//*****************************************************************************
//*
//*		Init
//*
//*****************************************************************************
//	Initialisiert die FTP Verbindung.
//	pPath		: Ist der Startpfad fr die Verbindung
//					"D:\xxx"
//					"/D:\xxx"
//					"Share\xxxx"
//	Ergibt TRUE wenn die Verbindung erzeugt werden konnte.
BOOL CFtpNetwork::Init(LPCTSTR pPath)
{
NETRESOURCE sNetRes;
LPTSTR		pFilePart;
TCHAR		cDrive[4];
CString		sPath;
unsigned	uBits;
int			iRet;
int			iBit;
int			iLen;
int			iNum;


	if(!m_pSession)
		{
		SetLastError(ERROR_RESOURCE_NOT_PRESENT);
		return FALSE;
		}


	if(!pPath)pPath=_T("");

	if(m_bIsDrive)										// Ist "D:/xxxx"
		{
		if(!*pPath)
			{
			iRet = GetFullPathName(_T("."),sizeof(m_cCurrDir)/sizeof(TCHAR),m_cCurrDir,&pFilePart);
			}
		else{
			if(strwchar(pPath[0],0x00300000) && strwchar(pPath[1],0x06) && pPath[2]==':')pPath++;
			iRet = GetFullPathName(pPath,sizeof(m_cCurrDir)/sizeof(TCHAR),m_cCurrDir,&pFilePart);
			}

		if(!iRet || iRet>=sizeof(m_cCurrDir)/sizeof(TCHAR))
			{
			return FALSE;
			}

		DelBackSlash(m_cCurrDir);
		if(ExistFile(m_cCurrDir)<2)
			{
			SetLastError(ERROR_PATH_NOT_FOUND);
			return FALSE;
			}

		PathConvert(m_cCurrDir,'/');

		m_pBaseUrl = _T("file://");
		return TRUE;
		}


	m_pBaseUrl = _T("smb://");

	if(strwchar(pPath[0],0x00300000))					// Slash am Begin entfernen
		{
		pPath++;
		}

	   iLen	= strcwlen(pPath,0x00300000);				// share an Server-Namen anfgen
	if(iLen<=0)
		{
		SetLastError(ERROR_INVALID_SHARENAME);
		return FALSE;
		}


	iNum = m_sServerName.GetLength();					// IP am Anfang berspringen

	if(_tcsnicmp(pPath,m_sServerName,iNum)==0)
	if(strwchar(pPath[0],0x01) && strwchar(pPath[iNum],0x80300000))
		{
		pPath += iNum;
		iLen  -= iNum;
		}


	strlcpy(m_cCurrDir,pPath,sizeof(m_cCurrDir)/sizeof(TCHAR),iLen);
	m_sServerName  = m_cCurrDir;
	pPath         += iLen;

	strlcpy(m_cCurrDir,pPath,sizeof(m_cCurrDir)/sizeof(TCHAR));
	DelBackSlash(m_cCurrDir);


	if(!m_cCurrDir[0])
		{
		m_cCurrDir[0] = '/';
		m_cCurrDir[1] = 0;
		}

	if(m_sUserName==_T(""))								// Keine Anmeldung ntig
		{
		m_cBaseDir[0] = '\\';
		m_cBaseDir[1] = '\\';
		strlcpy(m_cBaseDir+2,m_sServerName,sizeof(m_cBaseDir)/sizeof(TCHAR)-2);
		PathConvert(m_cBaseDir+2,'\\');

		if(!m_bIsDrive)
			{
			iLen = strcwlen(m_cBaseDir+2,0x00300000);	// Share am Ende entfernen
			m_cBaseDir[2+iLen] = 0;
			}

		return TRUE;
		}


	uBits	= GetLogicalDrives();


	for(cDrive[0]='Z';cDrive[0]>'F';cDrive[0]--)		// Suche freies Laufwerk
		{
		iBit = 1<<(cDrive[0]-'A');
		if(uBits&iBit)continue;
		break;
		}

	if(cDrive[0]<='F')
		{
		SetLastError(ERROR_NO_MORE_SEARCH_HANDLES);
		return FALSE;
		}


	cDrive[1]	= ':';									// Netzlaufwerk mounten
	cDrive[2]	= 0;
	sPath		= _T("\\\\");
	sPath	   += m_sServerName;
	PathConvert(sPath,'\\');

	strlcpy(m_cBaseDir,sPath,sizeof(m_cBaseDir)/sizeof(TCHAR));


	sNetRes.dwType			= RESOURCETYPE_DISK;
	sNetRes.lpRemoteName	= sPath.GetBuffer(CFB_MAX_PATH);
	sNetRes.lpProvider		= NULL;
	sNetRes.lpLocalName		= cDrive;

	iRet = WNetAddConnection2(&sNetRes,m_sPassword,m_sUserName,CONNECT_TEMPORARY|CONNECT_PROMPT);

	if(iRet!=NO_ERROR)									// War das Mounten ok
		{
		return FALSE;
		}

	m_bLogoff = TRUE;


return TRUE;
}




//*****************************************************************************
//*
//*		GetLocalPath
//*
//*****************************************************************************
//	Holt den lokalen Pfad zu einer FTP-Datei, falls er existiert
//	pFileName		: Name der FTP-Datei zu der der lokale Pfad geholt werden soll.
//	pBuffer			: Puffer fr den lokalen Namen
//	iSize			: Gre des Puffers in Zeichen.
//	Ergibt die lokale Pfadlnge in Zeichen bzw. 0 wenn es keine lokalen Pfad gibt.
int CFtpNetwork::GetLocalPath(LPCTSTR pFileName,LPTSTR pBuffer,int iSize)
{
int		iLen;


	iLen  = strlcpy     (pBuffer,m_cBaseDir,iSize);
	iLen += PathFullMake(pBuffer+iLen,m_cCurrDir,pFileName,iSize-iLen,0);

return iLen;
}

//*****************************************************************************
//*
//*		SetStat
//*
//*****************************************************************************
//	Einstellen der Dateiattribute und der Zeiten.
//	pInfo		: Sind die neuen Zeiten und Attribute
//				  Wenn pInfo->dwFileAttributes ungleich 0 und -1 ist werden die
//				  Attribute	verndert. Zie Zeiten werden gesetzt wenn sie ungleich
//				  0 sind. In pInfo->cFileName wird der Dateinamge bergeben.
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpNetwork::SetStat(LPWIN32_FIND_DATA pInfo)
{
TCHAR		cBuffer[CFB_MAX_PATH];
LPCTSTR		pFileName;
DWORD		dwAttr;
HANDLE		hFile;
BOOL		bDone;
int			iLen;



	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	pFileName = pInfo->cFileName;

	if(pFileName[0]=='/' || pFileName[0]=='\\')
	if(pFileName[1]!= 0  && pFileName[2]==':' )
		{
		pFileName++;
		}



	iLen=strlcpy(cBuffer,m_cBaseDir,sizeof(cBuffer)/sizeof(TCHAR));
	PathFullMake(cBuffer+iLen,m_cCurrDir,pFileName,sizeof(cBuffer)/sizeof(TCHAR)-iLen,0);

	if(m_bIsDrive && cBuffer[1]!=':')				// Prfe die Laufwerksangabe "C:..."
		{
		SetLastError(ERROR_PATH_NOT_FOUND);
		return FALSE;
		}


													// Zeiten setzen
	if(pInfo->ftCreationTime.dwHighDateTime|pInfo->ftLastAccessTime.dwHighDateTime|pInfo->ftLastWriteTime.dwHighDateTime)
		{
		   hFile=CreateFileA(cBuffer,GENERIC_WRITE,0,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_NO_BUFFERING,0);
		if(hFile==INVALID_HANDLE_VALUE)return FALSE;

		bDone = SetFileTime(hFile,(!pInfo->ftCreationTime  .dwHighDateTime)? NULL:&pInfo->ftCreationTime,
								  (!pInfo->ftLastAccessTime.dwHighDateTime)? NULL:&pInfo->ftLastAccessTime,
								  (!pInfo->ftLastWriteTime .dwHighDateTime)? NULL:&pInfo->ftLastWriteTime);

		CloseHandle(hFile);

		if(!bDone)return FALSE;
		}

													// Attribute setzen
	if(pInfo->dwFileAttributes && pInfo->dwFileAttributes!=0xFFFFFFFF)
		{
		dwAttr = pInfo->dwFileAttributes&(0xFFFF&~FILE_SET_ATTRIBUTE);
		if(!SetFileAttributes(cBuffer,dwAttr))return FALSE;
		}



return TRUE;
}








