//*****************************************************************************
//*
//*
//*		FtpSecurity.cpp
//*
//*
//*****************************************************************************
//
//	Ist eine Security-FTP-Verbindung ber SFTP
//
#include	<Afx.h>
#include	<stddef.h>
#include	"FtpTools.h"
#include	"FtpSecurity.h"
#include	"../MemFunctions.h"
#include	"../FileFunctions.h"
#include	"../StringFunctions.h"
#include	"../Tools/AzCommLib/AzCommDll.h"

typedef	struct
	{
	WIN32_FIND_DATA	sFind;
	LPVOID			pNext;
	}LinkEntry;

typedef	struct
	{
	FtpFindCallback	pProc;
	LPVOID			pParam;
	LPCTSTR			pFilter;
	BOOL			bUseUtf8;
	LinkEntry	   *pLinkLast;
	LinkEntry	   *pLinkFirst;
	}FindCbData;


//*****************************************************************************
//*
//*		SftpToFind
//*
//*****************************************************************************
static int _cdecl SftpToFind(void *pParam,sftp_dirent *pEntry)
{
FindCbData		   *pData=(FindCbData*)pParam;
LinkEntry		   *pLink;
WIN32_FIND_DATA		sFind;
int					iLen;


	if(!pEntry)return 0;

    UtcTimeToFileTime(pEntry->attr.mtime,&sFind.ftLastWriteTime );
    UtcTimeToFileTime(pEntry->attr.atime,&sFind.ftLastAccessTime);

    sFind.ftCreationTime		= sFind.ftLastWriteTime;
    sFind.dwReserved0			= 0;
    sFind.dwReserved1			= 0;
	sFind.cAlternateFileName[0]	= 0;
    sFind.nFileSizeHigh			= (DWORD)(pEntry->attr.size>>32);
    sFind.nFileSizeLow			= (DWORD)(pEntry->attr.size);
	sFind.dwFileAttributes		= pEntry->attr.perm<<16;
	sFind.dwFileAttributes     |= (sFind.nFileSizeHigh&0x3F)<<10;

	if(!(pEntry->attr.perm&0x0080))sFind.dwFileAttributes|=FILE_ATTRIBUTE_READONLY;
	if(  pEntry->attr.perm&0x4000 )sFind.dwFileAttributes|=FILE_ATTRIBUTE_DIRECTORY;
	if(  pEntry->attr.perm&0x0001 )sFind.dwFileAttributes|=FILE_ATTRIBUTE_HIDDEN;		// Samba-Hack
	if(  pEntry->attr.perm&0x0008 )sFind.dwFileAttributes|=FILE_ATTRIBUTE_SYSTEM;
	if(  pEntry->attr.perm&0x0040 )sFind.dwFileAttributes|=FILE_ATTRIBUTE_ARCHIVE;

	if(pData->bUseUtf8)
			iLen=strfromutf8(sFind.cFileName,pEntry->filename,sizeof(sFind.cFileName)/sizeof(sFind.cFileName[0]));
	else	iLen=strawcpy   (sFind.cFileName,pEntry->filename,sizeof(sFind.cFileName)/sizeof(sFind.cFileName[0]));


	if(pData->pFilter && !memxcmp(pData->pFilter,sFind.cFileName,2,0x03))
		{
		return 0;
		}

	if((pEntry->attr.perm&0xF000)==0xA000)				// Ist es ein Link
		{
		pLink			= new LinkEntry;
		pLink->pNext	= 0;

		memcpy(&pLink->sFind,&sFind,offsetof(WIN32_FIND_DATA,cFileName)+(iLen+1)*sizeof(TCHAR));

		if(pData->pLinkLast)
			{
			pData->pLinkLast->pNext = pLink;
			}
		else{
			pData->pLinkFirst = pLink;
			}

		pData->pLinkLast = pLink;

		return 0;
		}

	if(!pData->pProc(pData->pParam,&sFind))
		{
		return 1;
		}



return 0;
}

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CFtpSecurity::CFtpSecurity(CInternetSession *pSession,LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,DWORD dwContext,INTERNET_PORT nPort,BOOL bPassive) : CFtpBase(pSession,pServer,pUserName,pPassword,dwContext,nPort,bPassive)
{
	m_pBaseUrl	  = _T("sftp://");
	m_uTimeout	  = 10000;
	m_bReconnect  = FALSE;
	m_bConnDone	  = FALSE;
	m_bUseUtf8	  = FALSE;
	m_cCurrDir[0] = 0;
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CFtpSecurity::~CFtpSecurity()
{

	if(m_pSession)Close();

}


//*****************************************************************************
//*
//*		QueryOption
//*
//*****************************************************************************
//	Abfragen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	pLen		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpSecurity::QueryOption(DWORD dwOption,LPVOID pBuffer,LPDWORD pLen) const
{
LONGLONG	iOff;
LONGLONG	iSize;
DWORD	   *pVal=(DWORD*)pBuffer;
DWORD		uLen;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}

	uLen = *pLen;


	switch(dwOption)
		{
	case INTERNET_OPTION_MODE:						// Verbindungsmodus

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_SFTP;

		return TRUE;

	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:		// Timeout
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLen<sizeof(m_uTimeout))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		*pVal = m_uTimeout;
		*pLen = sizeof(m_uTimeout);

		return TRUE;

	case INTERNET_OPTION_PROGRESS:					// Verlaufsanzeige

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		iOff    = SftpProgress(m_pSession);
		pVal[0] = (DWORD)iOff;

		return TRUE;

	case INTERNET_OPTION_PROGRESS2:					// Verlaufsanzeige mit Maximalwert

		if(uLen<sizeof(DWORD)*2)
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		iOff    = SftpProgress(m_pSession,(int64_sftp*)&iSize);
		pVal[0] = (DWORD)iOff;

		if(m_bAbortAllowed==2 && m_dwPutSize)
				pVal[1] = m_dwPutSize;
		else	pVal[1] = (DWORD)iSize;

		return TRUE;

	case INTERNET_OPTION_UTF8:						// UTF8 Konvertierung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_bUseUtf8;

		return TRUE;

	case INTERNET_OPTION_CONNTYPE:					// Art der Verbindung

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = FTP_MODE_SFTP;

		return TRUE;

	case INTERNET_OPTION_URLFORMAT:					// Format des URL-Namens

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = 0;

		return TRUE;

	case INTERNET_OPTION_ERROPEN:					// Wo gab es den Fehler

		if(uLen<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		pVal[0] = m_dwErrOpen;

		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		SetOption
//*
//*****************************************************************************
//	Setzen einer Option.
//	dwOption	: Ist die Nummer der Option (siehe InternetQueryOption(...))
//						INTERNET_OPTION_???
//	pBuffer		: Zeiger auf den Textpuffer fr den URL-String. Wenn der
//				  Zeiger 0 ist wird in *pLen die mindest Gre gespeichert.
//	uLength		: *pLen enthllt Gre des Puffer in Zeichen, bzw. wird hier
//				  die Mindestgre gespeichert wenn pBuffer NULL ist.
//	dwFlags		: Unbenutzt, sollte 0 sein.
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei Fehlern.
BOOL CFtpSecurity::SetOption(DWORD dwOption,LPVOID pBuffer,DWORD uLength,DWORD dwFlags)
{
int		iCount;


	if(!m_pSession)
		{
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
		}


	switch(dwOption)
		{
	case INTERNET_OPTION_DATA_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_RECEIVE_TIMEOUT:
	case INTERNET_OPTION_SEND_TIMEOUT:

		if(uLength<sizeof(m_uTimeout))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_uTimeout = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_PROGRSET:					// Verlaufsanzeige setzen

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		SftpProgressSet(m_pSession,*(DWORD*)pBuffer);

		return TRUE;

	case INTERNET_OPTION_UTF8:						// UTF8 Konvertierung

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		m_bUseUtf8 = *(DWORD*)pBuffer;

		return TRUE;

	case INTERNET_OPTION_RECONNECT:					// Neuverbinden

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}

		iCount = *(int*)pBuffer;

		for(;;)
			{
			if(Reconnect(TRUE))return TRUE;
			iCount--;
			if(iCount<=0)break;
			Sleep(500);
			}

		return FALSE;

	case INTERNET_OPTION_SETSTAT:					// Attribute und Zeiten setzen

		if(uLength<sizeof(WIN32_FIND_DATA))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}

		return SetStat((LPWIN32_FIND_DATA)pBuffer);

	case INTERNET_OPTION_PUTSIZE:					// Gre der zu bertragenden Datei

		if(uLength<sizeof(DWORD))
			{
			SetLastError(ERROR_INVALID_PARAMETER);
			return FALSE;
			}
		
		m_dwPutSize = *(LPDWORD)pBuffer;		
		return TRUE;
		}


	SetLastError(ERROR_INVALID_NAME);


return FALSE;
}

//*****************************************************************************
//*
//*		OpenFile
//*
//*****************************************************************************
CInternetFile *CFtpSecurity::OpenFile(LPCTSTR pFileName,DWORD dwAccess,DWORD dwFlags,DWORD dwContext)
{
	if(!m_pSession)return NULL;

	return NULL;
}

//*****************************************************************************
//*
//*		Reconnect
//*
//*****************************************************************************
//	Erzeugt wieder eine Verbindung falls die alte abgebochen ist.
//	bForce		: Immer neu verbinden
BOOL CFtpSecurity::Reconnect(BOOL bForce)
{
char	cServer[64];
char	cPaswd [64];
char	cUser  [64];
int		iErr;


	if(!m_pSession)
		{
		SetLastError(ERROR_INTERNAL_ERROR);
		return FALSE;
		}

	if(!bForce && !m_bReconnect)return TRUE;

	if(!bForce && SftpIsConnected(m_pSession))
		{
		return TRUE;
		}

	strawcpy(cServer,m_sServerName,sizeof(cServer));
	strawcpy(cPaswd ,m_sPassword  ,sizeof(cPaswd ));
	strawcpy(cUser  ,m_sUserName  ,sizeof(cUser  ));

	if(m_bConnDone)
		{
		if(m_pSession)SftpClose(m_pSession);

		m_bConnDone = FALSE;
		m_pSession  = SftpCreate();

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}
		}

	m_bConnDone = TRUE;

	iErr = SftpConnect(m_pSession,cServer,m_wPort,cUser,cPaswd,m_uTimeout);

	if(iErr==SSH2_FX_ALLREADY_CONNECTED)
		{
		SftpBreak(m_pSession);

		if(m_pSession)SftpClose(m_pSession);

		m_bConnDone = FALSE;
		m_pSession  = SftpCreate();

		if(!m_pSession)
			{
			SetLastError(ERROR_INTERNAL_ERROR);
			return FALSE;
			}

		iErr = SftpConnect(m_pSession,cServer,m_wPort,cUser,cPaswd,m_uTimeout);
		}

	if(iErr==SSH2_FX_OK)return TRUE;


	SetError(iErr);



return FALSE;
}

//*****************************************************************************
//*
//*		SetCurrentDirectory
//*
//*****************************************************************************
//	Stellt das aktuelle Verzeichnis ein.
//	pDirName		: Ist der Name des neuen Verzeichnisses.
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpSecurity::SetCurrentDirectory(LPCTSTR pDirName)
{
char 		cBuffer[CFB_MAX_PATH];
sftp_attr	sAttrib;
int			iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	DelBackSlash(cBuffer);

	if(!cBuffer[0])
		{
		cBuffer[0] = '/';
		cBuffer[1] = 0;
		}



	   iErr = SftpStat(m_pSession,cBuffer,&sAttrib);
	if(iErr==SSH2_FX_OK)
		{
		if(!(sAttrib.perm&SFTP_ATTR_IFDIR))
			{
			SetLastError(ERROR_PATH_NOT_FOUND);
			return FALSE;
			}

		if(m_bUseUtf8)
				strfromutf8(m_cCurrDir,cBuffer,sizeof(m_cCurrDir)/sizeof(TCHAR));
		else	strawcpy   (m_cCurrDir,cBuffer,sizeof(m_cCurrDir)/sizeof(TCHAR));

		return TRUE;
		}

	SetError(iErr);



return FALSE;
}

//*****************************************************************************
//*
//*		GetCurrentDirectory
//*
//*****************************************************************************
//	Holt das aktuelle Verzeichnis.
//	pDirName		: Puffer fr den Namen
//	pLen			: Gre des Puffers
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpSecurity::GetCurrentDirectory(LPTSTR pDirName,LPDWORD pLen) const
{


	if(!m_pSession)return FALSE;

	*pLen = strlcpy(pDirName,m_cCurrDir,*pLen);



return TRUE;
}


//*****************************************************************************
//*
//*		RemoveDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des zu lschenden Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis gelscht wurde.
BOOL CFtpSecurity::RemoveDirectory(LPCTSTR pDirName)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;
int		iLen;


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = SftpRmdir(m_pSession,cBuffer);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		CreateDirectory
//*
//*****************************************************************************
//	Lscht ein Verzeichnis.
//	pDirName		: Ist der Name des neuen Verzeichnisses
//					  z.B.: "/" "/aaa/bbb" "/xxx/yyy/"
//	Ergibt TRUE wenn das Verzeichnis eingestellt wurde.
BOOL CFtpSecurity::CreateDirectory(LPCTSTR pDirName)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;
int		iLen;


	if(!Reconnect())return FALSE;

	iLen = PathFullMake(cBuffer,m_cCurrDir,pDirName,sizeof(cBuffer));
	iLen = PathConvert (cBuffer,'/');
	if(iLen>1)DelBackSlash(cBuffer,iLen);

	iErr = SftpMkdir(m_pSession,cBuffer);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		Remove
//*
//*****************************************************************************
//	Lscht eine Datei..
//	pFileName		: Ist der Name der Datei
//	Ergibt TRUE wenn die Datei gelscht wurde.
BOOL CFtpSecurity::Remove(LPCTSTR pFileName)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');

	iErr = SftpRemove(m_pSession,cBuffer);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		Rename
//*
//*****************************************************************************
//	Verschiebt eine Datei..
//	pExisting		: Ist der Name der Datei.
//	pNewName		: Ist der Name der Zieldatei.
//	Ergibt TRUE wenn die Datei vershoben wurde.
BOOL CFtpSecurity::Rename(LPCTSTR pExisting,LPCTSTR pNewName)
{
char 	cBuffer[CFB_MAX_PATH];
char 	cTarget[CFB_MAX_PATH];
int		iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pExisting,sizeof(cBuffer));
	PathConvert (cBuffer,'/');

	PathFullMake(cTarget,m_cCurrDir,pNewName,sizeof(cTarget));
	PathConvert (cTarget,'/');

	iErr = SftpRename(m_pSession,cBuffer,cTarget);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}


//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet die FTP-Verbindung
void CFtpSecurity::Close()
{

	if(!m_pSession)return;

	SftpClose(m_pSession);

	m_pSession	  = NULL;
	m_bReconnect  = FALSE;
	m_bConnDone	  = FALSE;

}

//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertgt eine Datei zum FTP-Server.
//	pLocalFile		: Name der lokalen Dateil
//	pRemoteFile		: Name der Datei am FTP-Server.
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpSecurity::PutFile(LPCTSTR pLocalFile,LPCTSTR pRemoteFile,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;
int		iRet;


	if(!Reconnect())return FALSE;

	   iRet = ExistFile(pLocalFile);
	if(iRet!=1)
		{
		SetLastError((iRet)? ERROR_ACCESS_DENIED:ERROR_FILE_NOT_FOUND);
		m_dwErrOpen = 1;
		return FALSE;
		}

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	m_dwErrOpen = 2;

	#ifdef UNICODE
	char 	 cLocal[CFB_MAX_PATH];
	strawcpy(cLocal,pLocalFile,sizeof(cLocal));
	pLocalFile = (LPCTSTR)cLocal;
	#endif

	m_bAbortAllowed = 1;
	iErr = SftpUploadFile(m_pSession,cBuffer,pLocalFile);
	m_bAbortAllowed = 0;

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}


//*****************************************************************************
//*
//*		PutFile
//*
//*****************************************************************************
//	bertrgt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpSecurity::PutFile(LPCTSTR pRemoteFile,LPVOID pParam,InpCall pProc,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');
	m_dwErrOpen = 2;

	#ifdef UNICODE
	char 	 cLocal[CFB_MAX_PATH];
	strawcpy(cLocal,pLocalFile,sizeof(cLocal));
	pLocalFile = (LPCTSTR)cLocal;
	#endif

	m_bAbortAllowed = 2;
	iErr = SftpUploadProc(m_pSession,cBuffer,(sftp_upload_proc)pProc,pParam);
	m_bAbortAllowed = 0;
	m_dwPutSize		= 0;

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei vom FTP-Server.
//	pRemoteFile		: Name der Datei am FTP-Server
//	pLocalFile		: Name der lokalen Datei
//	bFailIfExists	: Einen Fehler zurck melden wenn die Datei existiert
//	dwFlags			: Unbenutzt
//	dwContext		: Unbenutzt
//	Ergibt TRUE wenn die Datei bertragen wurde
BOOL CFtpSecurity::GetFile(LPCTSTR pRemoteFile,LPCTSTR pLocalFile,BOOL bFailIfExists,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(bFailIfExists && ExistFile(pLocalFile))
		{
		SetLastError(ERROR_FILE_EXISTS);
		m_dwErrOpen = 1;
		return FALSE;
		}

	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');

	#ifdef UNICODE
	char 	 cLocal[CFB_MAX_PATH];
	strawcpy(cLocal,pLocalFile,sizeof(cLocal));
	pLocalFile = (LPCTSTR)cLocal;
	#endif

	m_dwErrOpen     = 2;
	m_bAbortAllowed = 1;
	iErr = SftpDownloadFile(m_pSession,cBuffer,(char*)pLocalFile,NULL);
	m_bAbortAllowed = 0;

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		GetFile
//*
//*****************************************************************************
//	Holt eine Datei via Callback-Funktion.
//	pRemoteFile		: Name der Datei am FTP-Server.
//	pParam			: User-Parameter fr den Callback
//	pProc			: Ist die Callback-Funktion
//	dwAttributes	: Sind die Attribute fr die Datei
//	dwFlags			: Unbenutzt.
//	dwContext		: Unbenutzt.
//	Ergibt TRUE wenn die Datei bertragen wurde.
BOOL CFtpSecurity::GetFile(LPCTSTR pRemoteFile,LPVOID pParam,OutCall pProc,DWORD dwAttributes,DWORD dwFlags,DWORD dwContext)
{
char 	cBuffer[CFB_MAX_PATH];
int		iErr;


	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pRemoteFile,sizeof(cBuffer));
	PathConvert (cBuffer,'/');

	#ifdef UNICODE
	char 	 cLocal[CFB_MAX_PATH];
	strawcpy(cLocal,pLocalFile,sizeof(cLocal));
	pLocalFile = (LPCTSTR)cLocal;
	#endif

	m_dwErrOpen     = 2;
	m_bAbortAllowed = 1;
	iErr = SftpDownloadProc(m_pSession,cBuffer,(sftp_download_proc)pProc,pParam);
	m_bAbortAllowed = 0;

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		List
//*
//*****************************************************************************
//	Liste alle Dateien in einem Verzeichnis via Callback auf.
//	pRemotePath		: Ist der Name des Verzeichnisses.
//	pParam			: Parameter fr den Callback.
//	pProc			: Zeiger auf den Callback.
//	pFilter			: Filter der benutzt werden kann.
//	Ergibt wenn TRUE alles ok war, bzw. FALSE bei einem Fehler.
BOOL CFtpSecurity::List(LPCTSTR pRemotePath,void *pParam,FtpFindCallback pProc,LPCTSTR pFilter)
{
char 		cBuffer[CFB_MAX_PATH];
char 		cLnName[CFB_MAX_PATH];
LinkEntry  *pLink;
LinkEntry  *pNext;
FindCbData	sData;
sftp_attr	sAttr;
int			iLen;
int			iErr;
int			iRet;
int			iNum;


	if(!Reconnect())
		{
		pProc(pParam,NULL);
		return FALSE;
		}

	if(!*pRemotePath)
		{
		if(m_bUseUtf8)
				iLen=strtoutf8(cBuffer,m_cCurrDir,sizeof(cBuffer));
		else	iLen=strawcpy (cBuffer,m_cCurrDir,sizeof(cBuffer));
		}
	else{
		       PathFullMake(cBuffer,m_cCurrDir,pRemotePath,sizeof(cBuffer));
		iLen = PathConvert (cBuffer,'/');
		}

	sData.pProc		 = pProc;
	sData.pParam	 = pParam;
	sData.pFilter	 = pFilter;
	sData.bUseUtf8	 = m_bUseUtf8;
	sData.pLinkLast	 = 0;
	sData.pLinkFirst = 0;

	m_bAbortAllowed = 1;
	iErr = SftpReadDir(m_pSession,cBuffer,SftpToFind,&sData);
	m_bAbortAllowed = 0;


	pLink = sData.pLinkFirst;

	if(pLink && iLen>0 && cBuffer[iLen-1]!='/')		// Slash an Pfad anhngen
		{
		cBuffer[iLen] = '/';
		iLen++;
		}

	while(pLink)									// Wurden Soft-Links gefunden
		{
		pNext = (LinkEntry*)pLink->pNext;

		if(iErr==SSH2_FX_OK)
			{
			if(m_bUseUtf8)
					strtoutf8(cBuffer+iLen,pLink->sFind.cFileName,sizeof(cBuffer)-iLen);
			else	strawcpy (cBuffer+iLen,pLink->sFind.cFileName,sizeof(cBuffer)-iLen);

			m_bAbortAllowed = 1;
			iRet  = SftpReadLink(m_pSession,cBuffer,cLnName,sizeof(cLnName));
			m_bAbortAllowed = 0;

			if(iRet==SSH2_FX_OK)
				{
				if(cLnName[0]!='/')
					{
					iNum = strlen(cLnName);
					if(iNum+iLen<sizeof(cLnName))
						{
						memmove(cLnName+iLen,cLnName,iNum+1);
						memcpy (cLnName,cBuffer,iLen);
						}
					}

				   iRet = SftpStat(m_pSession,cLnName,&sAttr);
				if(iRet==SSH2_FX_USERBREAK)iErr=SSH2_FX_USERBREAK;

				if(iRet==SSH2_FX_OK)
					{
					UtcTimeToFileTime(sAttr.mtime,&pLink->sFind.ftLastWriteTime );
					UtcTimeToFileTime(sAttr.atime,&pLink->sFind.ftLastAccessTime);

					pLink->sFind.ftCreationTime		= pLink->sFind.ftLastWriteTime;
					pLink->sFind.nFileSizeHigh		= (DWORD)(sAttr.size>>32);
					pLink->sFind.nFileSizeLow		= (DWORD)(sAttr.size);
					pLink->sFind.dwFileAttributes	= sAttr.perm<<16;
					pLink->sFind.dwFileAttributes  |= (pLink->sFind.nFileSizeHigh&0x3F)<<10;
					pLink->sFind.dwFileAttributes  |= FILE_ATTRIBUTE_LINK;

					if((sAttr.perm&0xF000)==0x4000)
						{
						pLink->sFind.dwFileAttributes |= FILE_ATTRIBUTE_DIRECTORY;
						}
					}
				}

			if(!pProc(pParam,&pLink->sFind))
				{
				iErr = SSH2_FX_USERBREAK;
				}
			}

		delete pLink;
		pLink = pNext;
		}


	pProc(pParam,NULL);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		SetAttributes
//*
//*****************************************************************************
//	Einstellen der Dateiattribute
//	pFileName		: Ist der Name der Datei.
//	dwAttr			: Sind die neuen Attribute.
//					  In den Bits 16..31 knnen Unix-Attribute verpackt werden.
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpSecurity::SetAttributes(LPCTSTR pFileName,DWORD dwAttr)
{
char 		cBuffer[CFB_MAX_PATH];
sftp_attr	sAttrib;
int			iErr;


	if(!Reconnect())return FALSE;


	sAttrib.flags = SSH2_FILEXFER_ATTR_PERMISSIONS;


	if(dwAttr&0xFFFF0000)									// UNIX Attribute vorhanden
		{
		sAttrib.perm = dwAttr>>16;
		}
	else{
		sAttrib.perm = (dwAttr&FILE_ATTRIBUTE_READONLY)? 0x0120:0x01B4;
		if(dwAttr&FILE_ATTRIBUTE_DIRECTORY)sAttrib.perm|=0x0200;
		if(dwAttr&FILE_ATTRIBUTE_HIDDEN   )sAttrib.perm|=0x0001;	// Samba-Hack
		if(dwAttr&FILE_ATTRIBUTE_SYSTEM   )sAttrib.perm|=0x0008;
		if(dwAttr&FILE_ATTRIBUTE_ARCHIVE  )sAttrib.perm|=0x0040;
		if(dwAttr&FILE_ATTRIBUTE_EXEC     )sAttrib.perm|=0x0049;
		}


	PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');


	iErr = SftpSetStat(m_pSession,cBuffer,&sAttrib);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		SetModifyTime
//*
//*****************************************************************************
//	Einstellen der nderungszeit einer Datei.
//	pFileName		: Ist der Name der Datei.
//	sFileTime		: Ist die neue nderungszeit
//	Ergibt wenn alles ok War oder eine Fehlernummer
BOOL CFtpSecurity::SetModifyTime(LPCTSTR pFileName,FILETIME sFileTime)
{
char 		cBuffer[CFB_MAX_PATH];
sftp_attr	sAttrib;
CTime		sTime(sFileTime,0);
int			iErr;


	if(!m_pSession)return FALSE;

	sAttrib.flags = SSH2_FILEXFER_ATTR_ACMODTIME;
	sAttrib.atime = sTime.GetTime();
	sAttrib.mtime =	sAttrib.atime;

	if(!Reconnect())return FALSE;

	PathFullMake(cBuffer,m_cCurrDir,pFileName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');


	iErr = SftpSetStat(m_pSession,cBuffer,&sAttrib);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}

//*****************************************************************************
//*
//*		DoAbort
//*
//*****************************************************************************
//	Eine Put/Get/List operation abbrechen
void CFtpSecurity::DoAbort()
{


	if(!m_pSession|| !m_bAbortAllowed)return;

	SftpBreak(m_pSession);

	m_bReconnect    = TRUE;
	m_bAbortAllowed = 0;


}

//*****************************************************************************
//*
//*		Init
//*
//*****************************************************************************
//	Initialisiert die SFTP Verbindung.
//	iReconnect	: Automatisch wieder verbinden
//	uTimeout	: Timeout in Millisekunden
//	bUtf8		: UTF8 benutzen
//	Ergibt TRUE wenn die Verbindung erzeugt werden konnte.
BOOL CFtpSecurity::Init(int iReconnect,unsigned uTimeout,BOOL bUtf8)
{
char		cBuffer[CFB_MAX_PATH];
int			iErr;



	if(!m_pSession)return FALSE;

	iErr = SftpRealPath(m_pSession,".",cBuffer,sizeof(cBuffer));

	if(iErr!=SSH2_FX_OK)
		{
		SetError(iErr);
		return FALSE;
		}

	strawcpy(m_cCurrDir,cBuffer,(sizeof(m_cCurrDir)/sizeof(TCHAR)));

	m_bReconnect	= (iReconnect)? TRUE:FALSE;
	m_uTimeout		=  uTimeout;
	m_bUseUtf8		=  bUtf8;
	m_bConnDone		=  TRUE;


return TRUE;
}

//*****************************************************************************
//*
//*		PathFullMake
//*
//*****************************************************************************
int CFtpSecurity::PathFullMake(char *pBuffer,LPCTSTR pCurrDir,LPCTSTR pPath,int iSize)
{
char		cTemp[CFB_MAX_PATH];
const char *pDir;


	if(m_bUseUtf8)
		{
		strtoutf8(cTemp,pCurrDir,sizeof(cTemp));
		pDir = cTemp;
		}
	else{
		#ifdef UNICODE
		strawcpy(cTemp,pCurrDir,sizeof(cTemp));
		pDir = cTemp;
		#else
		pDir = pCurrDir;
		#endif
		}


return ::PathFullMake(pBuffer,pDir,pPath,iSize,0);
}

//*****************************************************************************
//*
//*		SetError
//*
//*****************************************************************************
//	Stellt die Windows Fehlernummer ein
//	iError		: Ist die FTP Fehlernummer
void CFtpSecurity::SetError(int iError)
{

	switch(iError)
		{
	case SSH2_FX_OK:				SetLastError(ERROR_SUCCESS);								return;
	case SSH2_FX_EOF:				SetLastError(ERROR_HANDLE_EOF);								return;
	case SSH2_FX_NO_SUCH_FILE:		SetLastError(ERROR_FILE_NOT_FOUND);							return;
	case SSH2_FX_PERMISSION_DENIED:	SetLastError(ERROR_ACCESS_DENIED);							return;
	case SSH2_FX_FAILURE:			SetLastError(ERROR_INVALID_ACCESS);							return;
	case SSH2_FX_BAD_MESSAGE:		SetLastError(ERROR_BAD_COMMAND);		m_bReconnect=TRUE;	return;
	case SSH2_FX_NO_CONNECTION:		SetLastError(ERROR_NOT_CONNECTED);		m_bReconnect=TRUE;	return;
	case SSH2_FX_CONNECTION_LOST:	SetLastError(ERROR_GRACEFUL_DISCONNECT);m_bReconnect=TRUE;	return;
	case SSH2_FX_OP_UNSUPPORTED:	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);					return;
	case SSH2_FX_TIMEOUT:			SetLastError(ERROR_TIMEOUT);								return;
	case SSH2_FX_USERBREAK:			SetLastError(ERROR_CANCELLED);								return;
	case SSH2_FX_BUFFER_SIZE:		SetLastError(ERROR_BUFFER_OVERFLOW);						return;
	case SSH2_FX_CANT_CREATE_FILE:	SetLastError(ERROR_WRITE_FAULT);							return;
	case SSH2_FX_CANT_OPEN_FILE:	SetLastError(ERROR_FILE_INVALID);							return;
	case SSH2_FX_WRITE_ERROR:		SetLastError(ERROR_WRITE_FAULT);							return;
	case SSH2_FX_DLL_ERROR:			SetLastError(ERROR_INVALID_DLL);							return;
	case SSH2_FX_WRONG_PARAM:		SetLastError(ERROR_INVALID_PARAMETER);						return;
	case SSH2_FX_ALLREADY_CONNECTED:SetLastError(ERROR_ALREADY_ASSIGNED);						return;
	case SSH2_FX_RESOURCE_ERROR:	SetLastError(ERROR_INTERNAL_ERROR);							return;
	case SSH2_FX_CANT_CONNECT:		SetLastError(ERROR_HOST_UNREACHABLE);						return;
		}

	SetLastError(ERROR_INVALID_ACCESS);

}

//*****************************************************************************
//*
//*		SetStat
//*
//*****************************************************************************
//	Einstellen der Dateiattribute und der Zeiten.
//	pInfo		: Sind die neuen Zeiten und Attribute
//				  Wenn pInfo->dwFileAttributes ungleich 0 und -1 ist werden die
//				  Attribute	verndert. Zie Zeiten werden gesetzt wenn sie ungleich
//				  0 sind. In pInfo->cFileName wird der Dateinamge bergeben.
//	Ergibt TRUE wenn alles ok war oder eine Fehlernummer
BOOL CFtpSecurity::SetStat(LPWIN32_FIND_DATA pInfo)
{
char 		cBuffer[CFB_MAX_PATH];
CTime		sTime(pInfo->ftLastWriteTime,0);
sftp_attr	sAttrib;
int    		iErr;



	if(!m_pSession)return FALSE;

	if(!Reconnect())return FALSE;


	sAttrib.flags = 0;

	if(pInfo->ftLastWriteTime.dwHighDateTime)			// Zeiten setzen
		{
		sAttrib.flags |= SSH2_FILEXFER_ATTR_PERMISSIONS;
		sAttrib.mtime  = sTime.GetTime();
		sAttrib.atime  = sAttrib.mtime;

		if(pInfo->ftLastAccessTime.dwHighDateTime)
			{
			CTime sAccess(pInfo->ftLastAccessTime,0);
			sAttrib.atime = sAccess.GetTime();
			}
		}

														// Attribute setzen
	if(pInfo->dwFileAttributes && pInfo->dwFileAttributes!=0xFFFFFFFF)
		{
		sAttrib.flags |= SSH2_FILEXFER_ATTR_PERMISSIONS;


		if(pInfo->dwFileAttributes&0xFFFF0000)			// UNIX Attribute vorhanden
			{
			sAttrib.perm = pInfo->dwFileAttributes>>16;
			}
		else{
			sAttrib.perm = (pInfo->dwFileAttributes&FILE_ATTRIBUTE_READONLY)? 0x0120:0x01B4;
			if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)sAttrib.perm|=0x0200;
			if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_HIDDEN   )sAttrib.perm|=0x0001;	// Samba-Hack
			if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_SYSTEM   )sAttrib.perm|=0x0008;
			if(pInfo->dwFileAttributes&FILE_ATTRIBUTE_ARCHIVE  )sAttrib.perm|=0x0040;
			}
		}

	PathFullMake(cBuffer,m_cCurrDir,pInfo->cFileName,sizeof(cBuffer));
	PathConvert (cBuffer,'/');


	iErr = SftpSetStat(m_pSession,cBuffer,&sAttrib);

	if(iErr==SSH2_FX_OK)return TRUE;

	SetError(iErr);


return FALSE;
}
