//*****************************************************************************
//*
//*
//*		FtpTools.cpp
//*
//*
//*****************************************************************************
//
//	Erzeugt diverse FTP Verbindungen
//
//			*) FTP
//			*) SFTP
//			*) SMB
//			*) SSH
//			*) ARCHIVE
//			*) AX-COMM
//
//	Spezielle Define's
//
//			FTP_NO_DISC			Keine FAT Verbindungen
//			FTP_NO_SSH			Keine SSH Verbindungen
//			FTP_NO_SMB			Keine SMB Verbindungen
//			FTP_NO_SFTP			Keine SFTP Verbindungen
//			FTP_NO_ARCH			Keine Archiv Verbindungen
//			FTP_NO_AXCOMM		Keine AX-Comm Verbindungen
//			FTP_OLNY_ZIP		Nur das ZIP-Format wird untersttzt
//
#include	<Afx.h>
#include	<AfxWin.h>
#include	"FtpTools.h"
#include	"DiskTools.h"
#include	"FtpDrives.h"
#include	"FtpCommon.h"
#include	"FtpAxComm.h"
#include	"FtpArchive.h"
#include	"FtpNetwork.h"
#include	"FtpConsole.h"
#include	"FtpSecurity.h"
#include	"AzFtpClient.h"
#include	"../MemFunctions.h"
#include	"../StringFunctions.h"
#include	"../Tools/AzCommLib/AzCommDll.h"


typedef struct
	{
	int		iMode;										// Art der Verbindung
	void   *pPtr;										// Ist das Abbruch-Handle
	}CancelData;


#ifndef		FTP_NO_ARCH
#ifdef		FTP_OLNY_ZIP
#include	"../Archive/ZipFile.h"
#else
#include	"../AZ_Archive.h"
#endif
#endif 	

#ifdef		FTP_OLNY_ZIP
#define 	FILTER_ALL			_T("*.zip")
#else
#define 	FILTER_ALL			_T("*.zip;*.tar;*.gz;*.tgz;*.bz2;*.rpm;*.rar;*.cab;*.chm;*.7z;*.iso;*.nrg;*.jar;*.uue;*.cpio;*.msi")
#endif

#define 	WM_MY_NOTIFY_MSG			0x4440
#define 	WM_MY_ASYNC_NETEVENT_MSG	0x4441
#define 	WM_MY_AGNT_CALLBK_MSG		0x4442
#define 	MY_TIMER_ID					0x4443

#ifndef		FTP_TIMEOUT
#define 	FTP_TIMEOUT					30000
#endif

#ifdef		FTP_NO_DISC
#undef		FTP_MODE_DISK
#define 	FTP_MODE_DISK				FTP_MODE_NONE
#endif

#ifdef		FTP_NO_ARCH
#undef		FTP_MODE_ARCH
#define 	FTP_MODE_ARCH				FTP_MODE_NONE
#endif

#ifdef		FTP_NO_SMB
#undef		FTP_MODE_SMB
#define 	FTP_MODE_SMB				FTP_MODE_NONE
#endif

#ifdef		FTP_NO_SFTP
#undef		FTP_MODE_SFTP
#define 	FTP_MODE_SFTP				FTP_MODE_NONE
#else
static int	iAzCommInit=0;
#endif

#ifdef		FTP_NO_AXCOMM
#undef		FTP_MODE_AXCOMM
#define 	FTP_MODE_AXCOMM				FTP_MODE_NONE
#else
static		TCHAR cAxDllPath[MAX_PATH];
#endif

#ifdef		FTP_NO_SSH
#undef		FTP_MODE_SSH
#define 	FTP_MODE_SSH				FTP_MODE_NONE
#endif


#ifndef		FTP_NO_ARCH
#ifndef		FTP_OLNY_ZIP


//*****************************************************************************
//*
//*		CheckTarHeader
//*
//*****************************************************************************
//	Prft den Header einer UUE-Datei
static int CheckUueHeader(char *pHeader,int iLen)
{
int		iPos;



	iPos = strlfind(pHeader,"begin",iLen-5);

	if(iPos<0)return 0;
	if(iPos>0 && pHeader[iPos-1]!='\n')return 0;

	pHeader += iPos+5;

		iPos = strwlen(pHeader,0xA0);
	if(!iPos || iPos>8)return 0;

	pHeader += iPos;

		iPos = strwlen(pHeader,0x00020000);
	if(!iPos || iPos>15)return 0;

	pHeader += iPos;

		iPos = strcwlen(pHeader,0x40);
	if(!iPos || iPos>255)return 0;

	pHeader += iPos;

		iPos = strwlen(pHeader,0x40);
	if(!iPos || iPos>2)return 0;

	if(pHeader[iPos]<' ' || pHeader[iPos]>0x60)return 0;



return 1;
}


//*****************************************************************************
//*
//*		CheckTarHeader
//*
//*****************************************************************************
//	Prft den Header einer TAR-Datei
static int CheckTarHeader(void *pHeader)
{
unsigned char  *pData=(unsigned char*)pHeader;
unsigned		uSum;
unsigned		uPos;



	uSum = 0;



	for(uPos=148;uPos<156;uPos++)
		{
		if(pData[uPos]!=' ')break;
		}

	for(;uPos<156;uPos++)
		{
		if(pData[uPos]<'0')break;
		if(pData[uPos]>'9')break;

		uSum *= 8;
		uSum += pData[uPos]-'0';
		}

	for(uPos=0;uPos<148;uPos++)
		{
		uSum -= pData[uPos];
		}

	for(uPos=156;uPos<512;uPos++)
		{
		uSum -= pData[uPos];
		}

	uSum -= ' '*8;


return (uSum)? 0:1;
}

//*****************************************************************************
//*
//*		CheckIsoHeader
//*
//*****************************************************************************
//	Prft den Header einer ISO-Datei
static int CheckIsoHeader(void *pFilePtr)
{
FILE	   *pFile=(FILE*)pFilePtr;
char		cData[16]={0,0,0,0};


	if(fseek (pFile,16*2048,SEEK_SET))return 0;
	if(fread (cData,sizeof(cData),1,pFile)!=1)return 0;
	if(memcmp(cData+1,"CD001",5)==0)return 1;
	if(memcmp(cData+1,"BEA"  ,3)==0)return 1;
	if(fseek (pFile,-12,SEEK_END) )return 0;
	if(fread (cData,12,1,pFile)!=1)return 0;
	if(memcmp(cData  ,"NER5",4)==0)return 1;
	if(memcmp(cData+4,"NERO",4)==0)return 1;


return (memcmp(cData+1,"CD001",5))? 0:1;
}

//*****************************************************************************
//*
//*		CheckHxsHeader
//*
//*****************************************************************************
//	Prft den Header einer HXS-Datei
static int CheckHxsHeader(void *pFilePtr,char *pHeader)
{
FILE 	   *pFile=(FILE*)pFilePtr;
char		cBuffer[512+4];
unsigned	uOff;
unsigned	uPos;



	memcpy(cBuffer,pHeader,512);
	pHeader = cBuffer;

	fseek(pFile,512,SEEK_SET);

	for(uOff=0;uOff<0x8000;uOff+=504)
		{
		for(uPos=0;uPos<=504;uPos++)
			{
			if(!memcmp(pHeader+uPos,"ITOLITLS",8))return 1;
			}

		memcpy(pHeader,pHeader+504,8);
		if(fread(pHeader+8,504,1,pFile)!=1)return 0;
		}



return 0;
}

#endif

//*****************************************************************************
//*
//*		OpenArchive
//*
//*****************************************************************************
static CArchiveFile *OpenArchive(LPCTSTR pPathName)
{
CArchiveFile   *pArchive;
FILE           *pFile;
CString			sText;
char			cBuffer[512+4];
int				iError;
int				iLen;


	if(!*pPathName)return 0;

	if((pPathName[0]=='r'  || pPathName[0]=='R') &&
	   (pPathName[1]=='e'  || pPathName[1]=='E') &&
	   (pPathName[2]=='s'  || pPathName[2]=='S') &&
	   (pPathName[3]==':'                      ) &&
	   (pPathName[4]=='\\' || pPathName[4]=='/') &&
	   (pPathName[5]=='\\' || pPathName[5]=='/'))
		{
		pArchive = new CZipFile;
		}
	else{
			pFile = _tfopen(pPathName,_T("rb"));
		if(!pFile)
			{
			sText.Format(_T("Die Datei %s konnte nicht geffnet werden !"),pPathName);
			AfxGetMainWnd()->MessageBox(sText,_T("Fehler"),MB_OK|MB_ICONERROR);
			SetCursor(LoadCursor(NULL,IDC_ARROW));
			return 0;
			}

		iLen = fread(cBuffer,1,512,pFile);
		cBuffer[iLen] = 0;

		if(iLen>4 && cBuffer[0]=='P' && cBuffer[1]=='K')		// Archiveformat feststellen
			{
			pArchive = new CZipFile;
			}
		#ifndef FTP_OLNY_ZIP
		else if(iLen>4 && cBuffer[0]=='B' && cBuffer[1]=='Z')
			{
			pArchive = new CBz2File;
			}
		else if(iLen>=512 && !strcmp(cBuffer+0x101,"ustar  "))
			{
			pArchive = new CTarFile;
			}
		else if(iLen>=512 && !memcmp(cBuffer,"\xFD""7zXZ\0",6))
			{
			pArchive = new CXzsFile;
			}
		else if(iLen>4 && cBuffer[0]=='\x1F' && cBuffer[1]=='\x8B')
			{
			pArchive = new CTgzFile;
			}
		else if(iLen>4 && cBuffer[0]=='\x1F' && cBuffer[1]=='\x9D')
			{
			pArchive = new CLzwFile;
			}
		else if(iLen>4 && cBuffer[0]=='\xED' && cBuffer[1]=='\xAB' && cBuffer[2]=='\xEE' && cBuffer[3]=='\xDB')
			{
			pArchive = new CRpmFile;
			}
		else if(iLen>4 && cBuffer[0]=='M' && cBuffer[1]=='S' && cBuffer[2]=='C' && cBuffer[3]=='F')
			{
			pArchive = new CCabFile;
			}
		else if(iLen>4 && cBuffer[0]=='I' && cBuffer[1]=='T' && cBuffer[2]=='S' && cBuffer[3]=='F')
			{
			pArchive = new CChmFile;
			}
		else if(iLen>4 && cBuffer[0]=='R' && cBuffer[1]=='a' && cBuffer[2]=='r' && cBuffer[3]=='!')
			{
			pArchive = new CRarFile;
			}
		else if(iLen>4 && cBuffer[0]=='U' && cBuffer[1]=='n' && cBuffer[2]=='i' && cBuffer[3]=='q')
			{
			pArchive = new CRarFile;
			}
		else if(iLen>6 && cBuffer[0]=='7' && cBuffer[1]=='z' && cBuffer[2]=='\xBC' && cBuffer[3]=='\xAF' && cBuffer[4]=='\x27' && cBuffer[5]=='\x1C')
			{
			pArchive = new CZp7File;
			}
		else if(iLen>=512 && CheckTarHeader(cBuffer))
			{
			pArchive = new CTarFile;
			}
		else if(iLen>8 && cBuffer[0]=='\xD0' && cBuffer[1]=='\xCF' && cBuffer[2]=='\x11' && cBuffer[3]=='\xE0')
			{
			pArchive = new CMsiFile;
			}
		else if(iLen>6 && cBuffer[0]=='0' && cBuffer[1]=='7' && cBuffer[2]=='0' && cBuffer[3]=='7' && cBuffer[4]=='0' && cBuffer[5]>='1' && cBuffer[5]<='7')
			{
			pArchive = new CPioFile;
			}
		else if(iLen>6 && cBuffer[0]=='\xC7' && cBuffer[1]=='\x71')
			{
			pArchive = new CPioFile;
			}
		else if(iLen>6 && cBuffer[0]=='\x71' && cBuffer[1]=='\xC7')
			{
			pArchive = new CPioFile;
			}
		else if(iLen>=512 && cBuffer[0]=='M' && cBuffer[1]=='Z' && CheckHxsHeader(pFile,cBuffer))
			{
			pArchive = new CChmFile;
			}
		else if(iLen>=512 && cBuffer[0]=='M' && cBuffer[1]=='Z')
			{
			pArchive = new CExeFile;
			}
		else if(iLen>=512 && cBuffer[0]=='P' && cBuffer[1]=='E')
			{
			pArchive = new CExeFile;
			}
		else if(iLen>=512 && memcmp(cBuffer,"\x00\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\x00",12)==0)
		 	{
			pArchive = new CIsoFile;
			}
		else if(iLen>=64 && CheckUueHeader(cBuffer,iLen))
			{
			pArchive = new CUueFile;
			}
		else if(iLen>=512 && CheckIsoHeader(pFile))
			{
			pArchive = new CIsoFile;
			}
		else if(iLen>=512 && cBuffer[510]=='\x55' && cBuffer[511]=='\xAA')
			{
			pArchive = new CFatFile;
			}
		#endif
		else{
			fclose(pFile);
			return 0;
			}


		fclose(pFile);
		}

	   iError = pArchive->Open(pPathName,CAF_READ);
	if(iError!=CAF_ERROR_OK)
		{
		delete pArchive;
		return 0;
		}


	pArchive->Flags(CAF_FLAG_SORT);


return pArchive;
}

#endif

//*****************************************************************************
//*
//*		GetFtpConnectionEx
//*
//*****************************************************************************
//	Erzeugt eine FTP-Verbindung
//	pServer		: Ist der Servername ("" bei Laufwerken)
//	pUserName	: Ist der Username oder NULL wenn keiner vorhanden.
//	pPassword	: Ist das Passwort oder NULL wenn keines vorhanden.
//	nPort		: Ist die TCP-Portnummer. (0=default)
//	iMode		: Bestimmt die Art der Verbindung
//					FTP_MODE_FTP		0x0001	Normale FTP Verbindung
//					FTP_MODE_SFTP		0x0002	Serurity SFTP Verbindung
//					FTP_MODE_SMB		0x0003	Verzeichnis oder Netzlaufwerk
//					FTP_MODE_ARCH		0x0004	Eine Archivedatei (z.B. *.zip)
//					FTP_MODE_DISK		0x0005	Ein FAT-Image oder Laufwerk
//					FTP_MODE_AXCOMM		0x0006	Eine AX-Comm-Server Verbindung ffnen
//					FTP_MODE_SSH		0x0007	Eine Serurity SSH Verbindung
//					FTP_MODE_RECONN		0x0100	Die Verbindung bei Abbrchen neu aufbauen
//					FTP_MODE_PASSIVE	0x0200	Eine passiver Verbindung aufbauen
//					FTP_MODE_RETRY		0x0400	Die Daten Verbindung versuchen erneut aufzubauen
//					FTP_MODE_RESOLVE	0x0800	Soft-Links auf FTP-Verbindunen auflsen
//					FTP_MODE_READONLY	0x1000	Nur lesend auf die Verbindung zugreifen
//	pPath		: Ist der Pfad aus URL.
//	pCancelPtr	: Hier wird ein Handle fr den Abbruch gespeichert
//	Ergibt einen Zeiger auf die Verbindung.
//	Bei einem Fehler wird eine CInternetException geworfen.
CFtpBase *GetFtpConnectionEx(LPCTSTR pServer,LPCTSTR pUserName,LPCTSTR pPassword,INTERNET_PORT nPort,int iMode,LPCTSTR pPath,void **pCancelPtr)
{
#ifndef				FTP_NO_ARCH
CArchiveFile	   *pArchive=0;
#endif 	
void			   *pSession=0;
CInternetException *pException;
CFtpCommon		   *pConnF=0;
CFtpDrives		   *pConnD=0;
CFtpArchive		   *pConnA=0;
CFtpNetwork		   *pConnN=0;
CFtpSecurity	   *pConnS=0;
CFtpConsole		   *pConnC=0;
CFtpAxComm		   *pConnX=0;
CancelData		   *pData=0;
void			  **pCancel=0;
char				cServer[64];
char				cPaswd[64];
char				cUser[64];
CString				sText;
AzFtpHandle			hFtp;
int					iFlag;
int					iErr;
int					iRet;




	ASSERT(pServer!=0);

	cServer[0] = 0;
	cPaswd [0] = 0;
	cUser  [0] = 0;
	iRet       = 0;

	if(pCancelPtr)*pCancelPtr=NULL;								// Abbruch-Handle


	switch(iMode&0xFF)
		{
	case FTP_MODE_FTP:											// FTP

			if(!pUserName)pUserName=_T("");

									   iFlag =0;
			if(iMode&FTP_MODE_PASSIVE )iFlag|=AZFTP_PASSIVE;
			if(iMode&FTP_MODE_RECONN  )iFlag|=AZFTP_RECONNECT;
			if(iMode&FTP_MODE_RESOLVE )iFlag|=AZFTP_LINK_RESOLVE;

			if(pCancelPtr)
				{
				pData		 = new CancelData;
				pData->iMode = FTP_MODE_FTP;
				pData->pPtr	 = 0;
				pCancel		 = &pData->pPtr;
			   *pCancelPtr	 = pData;
				}

			hFtp = AzFtpConnect(pServer,pUserName,pPassword,nPort,iFlag,FTP_TIMEOUT,&iErr,pCancel);

			if(pCancelPtr)
				{
			   *pCancelPtr	 = 0;
				pData->pPtr	 = 0;
				pData->iMode = FTP_MODE_NONE;
				delete(pData);
				}

			if(!hFtp)
				{
				if(iErr==530)
					{
					iErr = ERROR_INVALID_PASSWORD;
					}
				else if(iErr/100==3)
					{
					iErr = ERROR_BAD_USERNAME;
					}
				else if(iErr==986)
					{
					iErr = ERROR_OUTOFMEMORY;
					}
				else if(iErr==500)
					{
					iErr = ERROR_GEN_FAILURE;
					}
				else{
					iErr = ERROR_CONNECTION_INVALID;
					}


				pException = new CInternetException(iErr);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", iErr);
				THROW(pException);
				return 0;
				}

			pConnF = new CFtpCommon((CInternetSession*)hFtp,pServer,pUserName,pPassword,0,nPort,(iMode&FTP_MODE_PASSIVE)? 1:0);

			if(pConnF && (iMode&FTP_MODE_RETRY))
				{
				iFlag = 1;
				pConnF->SetOption(INTERNET_OPTION_RETRY,&iFlag,sizeof(iFlag));
				}

			if(pPath && *pPath)									// set path
				{
				pConnF->SetCurrentDirectory(pPath);
				}

			return pConnF;

	#ifndef FTP_NO_SFTP

	case FTP_MODE_SFTP:											// SFTP

			if(!pUserName)pUserName=_T("");

			if(!iAzCommInit)
				{
					iRet = AzCommLoadDll();						// load DLL
				if(!iRet)
					{
					pException = new CInternetException(ERROR_INVALID_DLL);
					pException->m_dwContext = 0;

					TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_INVALID_DLL);
					THROW(pException);

					return 0;
					}

				iAzCommInit = 1;
				}

				pSession = SftpCreate();
			if(!pSession)
				{
				pException = new CInternetException(ERROR_STATIC_INIT);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_STATIC_INIT);
				THROW(pException);

				return 0;
				}

			strawcpy(cServer,pServer  ,sizeof(cServer));
			strawcpy(cPaswd ,pPassword,sizeof(cPaswd ));
			strawcpy(cUser  ,pUserName,sizeof(cUser  ));

			if(pCancelPtr)
				{
				pData		 = new CancelData;
				pData->iMode = FTP_MODE_SFTP;
				pData->pPtr	 = pSession;
			   *pCancelPtr	 = pData;
				}

																// connect to the target
			iRet = SftpConnect(pSession,cServer,nPort,cUser,cPaswd);

			if(pCancelPtr)
				{
			   *pCancelPtr	 = 0;
				pData->pPtr	 = 0;
				pData->iMode = FTP_MODE_NONE;
				delete(pData);
				}

			if(iRet!=SSH2_FX_OK)
				{
				pSession = 0;

				switch(iRet)
					{
				case SSH2_FX_CANT_LOGIN:	iErr=ERROR_BAD_USERNAME;		break;
				default:					iErr=ERROR_CONNECTION_INVALID;	break;
					}

				pException = new CInternetException(iErr);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", iErr);
				THROW(pException);

				return 0;
				}

			pConnS = new CFtpSecurity((CInternetSession*)pSession,pServer,pUserName,pPassword,0,nPort,(iMode&FTP_MODE_PASSIVE)? 1:0);

			if(!pConnS->Init(iMode&FTP_MODE_RECONN,FTP_TIMEOUT,TRUE))
				{
				delete pConnS;

				pException = new CInternetException(ERROR_DEVICE_REINITIALIZATION_NEEDED);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_DEVICE_REINITIALIZATION_NEEDED);
				THROW(pException);

				return 0;
				}

			if(pPath && *pPath)									// set path
				{
				pConnS->SetCurrentDirectory(pPath);
				}

			return pConnS;

	#endif

	#ifndef FTP_NO_SSH

	case FTP_MODE_SSH:											// SSH (Console)

			if(!pUserName)pUserName=_T("");

			if(!iAzCommInit)
				{
					iRet = AzCommLoadDll();						// load DLL
				if(!iRet)
					{
					pException = new CInternetException(ERROR_INVALID_DLL);
					pException->m_dwContext = 0;

					TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_INVALID_DLL);
					THROW(pException);

					return 0;
					}

				iAzCommInit = 1;
				}

				pSession = SshCreate();
			if(!pSession)
				{
				pException = new CInternetException(ERROR_STATIC_INIT);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_STATIC_INIT);
				THROW(pException);

				return 0;
				}

			strawcpy(cServer,pServer  ,sizeof(cServer));
			strawcpy(cPaswd ,pPassword,sizeof(cPaswd ));
			strawcpy(cUser  ,pUserName,sizeof(cUser  ));

			if(pCancelPtr)
				{
				pData		 = new CancelData;
				pData->iMode = FTP_MODE_SSH;
				pData->pPtr	 = pSession;
			   *pCancelPtr	 = pData;
				}

			
			for(iRet=0;cUser[iRet];iRet++)						// Entferne Supper-User Weiterleitung
				{
				if(cUser[iRet]!='|')continue;
				cUser[iRet] = 0;
				break;
				}

																// connect to the target
			iRet = SshConnect(pSession,cServer,nPort,cUser,cPaswd);

			if(pCancelPtr)
				{
			   *pCancelPtr	 = 0;
				pData->pPtr	 = 0;
				pData->iMode = FTP_MODE_NONE;
				delete(pData);
				}

			if(iRet!=SSH2_FX_OK)
				{
				pSession = 0;

				switch(iRet)
					{
				case SSH2_FX_CANT_LOGIN:	iErr=ERROR_BAD_USERNAME;		break;
				default:					iErr=ERROR_CONNECTION_INVALID;	break;
					}

				pException = new CInternetException(iErr);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", iErr);
				THROW(pException);

				return 0;
				}

			pConnC = new CFtpConsole((CInternetSession*)pSession,pServer,pUserName,pPassword,0,nPort,(iMode&FTP_MODE_HEXOUT)? 1:0);

			if(!pConnC->Init(iMode&FTP_MODE_RECONN,FTP_TIMEOUT,TRUE))
				{
				delete pConnC;

				pException = new CInternetException(ERROR_BAD_NETPATH);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_BAD_NETPATH);
				THROW(pException);

				return 0;
				}

			if(pPath && *pPath)									// set path
				{
				pConnC->SetCurrentDirectory(pPath);
				}

			return pConnC;

	#endif

	#ifndef FTP_NO_AXCOMM

	case FTP_MODE_AXCOMM:										// AXCOMM (AX-COMM-Client)

			if(!pUserName)pUserName=_T("");

				iRet = CiCommDllLoad(cAxDllPath);				// load DLL
			if(!iRet)
				{
				pException = new CInternetException(ERROR_INVALID_DLL);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d (can't load DLL)\n", ERROR_INVALID_DLL);
				THROW(pException);

				return 0;
				}


			strawcpy(cServer,pServer  ,sizeof(cServer));
			strawcpy(cPaswd ,pPassword,sizeof(cPaswd ));
			strawcpy(cUser  ,pUserName,sizeof(cUser  ));

																// connect to the target
				iRet = CiDoConnect(&pSession,cServer,nPort,cUser,cPaswd);
			if(!iRet)
				{
				pException = new CInternetException(GetLastError());
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", iErr);
				THROW(pException);

				return 0;
				}

			pConnX = new CFtpAxComm((CInternetSession*)pSession,pServer,pUserName,pPassword,0,nPort,0);

			if(!pConnX->Init(iMode&FTP_MODE_RECONN,FTP_TIMEOUT,TRUE))
				{
				delete pConnX;

				pException = new CInternetException(ERROR_BAD_NETPATH);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_BAD_NETPATH);
				THROW(pException);

				return 0;
				}

			if(pPath && *pPath)									// set path
				{
				pConnX->SetCurrentDirectory(pPath);
				}

			return pConnX;

	#endif

	#ifndef FTP_NO_SMB

	case FTP_MODE_SMB:											// SMB (Netzlaufwerke)

			if(!pUserName)pUserName=_T("");
			if(!pPath    )pPath    =_T("");

			iRet = strwlen(pServer,0x00300000);
																// Share an Server anhngen
			if(pServer[iRet] && !strwchar(pServer[iRet+1],0x00700000))
				{
				int		iOff;
				int		iNum;

				iOff  = strwlen (pPath,0x00300000);
				iNum  = strcwlen(pPath+iOff,0x00300000);

				if(iNum>0)
					{
					iRet += strcwlen(pServer+iRet,0x00300000);

					if(pServer[iRet]==0)
						{
						sText.Format("%s/%.*s",pServer,iNum,pPath+iOff);
						pServer = sText;
						}
					else if(pServer[iRet+1]==0)
						{
						sText.Format("%s%.*s",pServer,iNum,pPath+iOff);
						pServer = sText;
						}
					}
				}

			pConnN = new CFtpNetwork((CInternetSession*)1,pServer,pUserName,pPassword,0,nPort,(iMode&FTP_MODE_PASSIVE)? 1:0);

			if(!pConnN->Init(pServer))
				{
				delete pConnN;

				pException = new CInternetException(GetLastError());
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", GetLastError());
				THROW(pException);

				return 0;
				}

			if(pPath && *pPath)									// set path
				{
				pConnN->SetCurrentDirectory(pPath);
				}

			return pConnN;


	#endif

	#ifndef FTP_NO_ARCH

	case FTP_MODE_ARCH:											// ARCHIVE

			if(!pUserName)pUserName=_T("");
			if(!pPath    )pPath    =_T("");

			if(!*pServer && !*pPath)break;
			if(!*pServer)pServer=pPath;
			if(!*pPath  )pPath=pServer;

			sText = pPath;
			sText.Replace("%20"," " );
			sText.Replace("%25","%" );
			sText.Replace("%3A",":" );
			sText.Replace("%2F","/" );
			sText.Replace("%5C","\\");
			pPath = sText;

			if(strwchar(pPath[0],0x00300000))
				{
				if(strwchar(pPath[1],0x00300000) && strwchar(pPath[2],0x00300000))
					{
					pPath++;									// "///..." zu "//..."
					}
				else if(strwchar(pPath[1],0x06) && pPath[2]==':')
					{
					pPath++;									// "/D:/..." zu "D:/..."
					}
				}

				pArchive = OpenArchive(pPath);
			if(!pArchive)
				{
				pException = new CInternetException(ERROR_FILE_NOT_FOUND);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_FILE_NOT_FOUND);
				THROW(pException);

				return 0;
				}

			sText = pServer;
			sText.Replace("%" ,"%25");
			sText.Replace(":" ,"%3A");
			sText.Replace("/" ,"%2F");
			sText.Replace("\\","%2F");

			pConnA = new CFtpArchive((CInternetSession*)pArchive,sText,pUserName,pPassword,0,nPort,(iMode&FTP_MODE_PASSIVE)? 1:0);

			if(!pConnA)
				{
				pException = new CInternetException(ERROR_OUTOFMEMORY);
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", ERROR_OUTOFMEMORY);
				THROW(pException);

				return 0;
				}

			return pConnA;

	#endif

	#ifndef FTP_NO_DISC

	case FTP_MODE_DISK:											// FAT

			if(!pUserName)pUserName=_T("");

			sText = pServer;
			sText.Replace("%" ,"%25");
			sText.Replace(":" ,"%3A");
			sText.Replace("/" ,"%2F");
			sText.Replace("\\","%2F");

			pSession = new DiskPartionData;
			pConnD   = new CFtpDrives((CInternetSession*)pSession,sText,pUserName,pPassword,0,nPort,(iMode&FTP_MODE_PASSIVE)? 1:0);

			if(!pConnD->Init(iMode&FTP_MODE_READONLY))
				{
				delete pConnD;

				pException = new CInternetException(GetLastError());
				pException->m_dwContext = 0;

				TRACE1("Warning: throwing CInternetException for error %d\n", GetLastError());
				THROW(pException);

				return 0;
				}

			return pConnD;

	#endif
		}


	pException = new CInternetException(ERROR_INVALID_PARAMETER);
	pException->m_dwContext = 0;

	TRACE1("Warning: throwing CInternetException for error %d\n", iErr);
	THROW(pException);



return 0;
}

//*****************************************************************************
//*
//*		GetFtpConnType
//*
//*****************************************************************************
//	Holt den Typ einer Verbindung aus eine URL.
//	pUrl		: Zeiger auf die URL.
//	bOnlyProt	: Nur Protokollangabe auswerten
//	Ergibt den Typ der Verbindung
//					FTP_MODE_NONE	0x0000	Keine Type spezifizeirt
//					FTP_MODE_FTP	0x0001	Normale FTP Verbindung
//					FTP_MODE_SFTP	0x0002	Serurity SFTP Verbindung
//					FTP_MODE_SMB	0x0003	Verzeichnis oder Netzlaufwerk
//					FTP_MODE_ARCH	0x0004	Eine Archivedatei (z.B. *.zip)
//					FTP_MODE_DISK	0x0005	Ein FAT-Image oder Laufwerk
//					FTP_MODE_AXCOMM	0x0006	Eine AX-Comm-API Verbindung
unsigned GetFtpConnType(LPCTSTR pUrl,BOOL bOnlyProt)
{
TCHAR		cPath[256]=_T("");
TCHAR		cServ[ 64]=_T("");
TCHAR		cProt[ 16]=_T("");



	strparseurl(pUrl,cServ,64,0,0,0,cProt,cPath,256);

	if(!_tcsicmp(cProt,_T("ftp"   )))return FTP_MODE_FTP;
	if(!_tcsicmp(cProt,_T("sftp"  )))return FTP_MODE_SFTP;
	if(!_tcsicmp(cProt,_T("file"  )))return FTP_MODE_SMB;
	if(!_tcsicmp(cProt,_T("smb"   )))return FTP_MODE_SMB;
	if(!_tcsicmp(cProt,_T("arch"  )))return FTP_MODE_ARCH;
	if(!_tcsicmp(cProt,_T("disk"  )))return FTP_MODE_DISK;
	if(!_tcsicmp(cProt,_T("axcomm")))return FTP_MODE_AXCOMM;


	if(cProt[0] || bOnlyProt)return FTP_MODE_NONE;


	if(memfilter(cPath,FILTER_ALL,1)>1)
		{
		return FTP_MODE_ARCH;
		}



return FTP_MODE_SMB;
}

//*****************************************************************************
//*
//*		GetFtpConnCancel
//*
//*****************************************************************************
//	Bricht den aktuellen Verbindungsaufbau ab
//	Ergibt 1 wenn erfolgreich, ansonsten 0
unsigned GetFtpConnCancel(LPVOID pCandelPtr)
{
CancelData	*pData=(CancelData*)pCandelPtr;


	if(!pData || !pData->pPtr)return 0;

	switch(pData->iMode)
		{
	case FTP_MODE_FTP:											// FTP

			return (AzFtpAbort((AzFtpHandle)pData->pPtr))? 1:0;

	#ifndef FTP_NO_SFTP

	case FTP_MODE_SFTP:											// SFTP

			SftpBreak(pData->pPtr);
			return 1;
	#endif

	#ifndef FTP_NO_SSH

	case FTP_MODE_SSH:											// SSH 

			SshBreak(pData->pPtr);
			return 1;
	#endif

	#ifndef FTP_NO_SMB

	case FTP_MODE_SMB:											// SMB

			return 0;
	#endif

	#ifndef FTP_NO_ARCH

	case FTP_MODE_ARCH:											// ARCHIVE

			return 0;
	#endif

	#ifndef FTP_NO_DISC

	case FTP_MODE_DISK:											// DISK

			return 0;
	#endif

	#ifndef FTP_NO_AXCOMM

	case FTP_MODE_AXCOMM:										// AXCOMM

			return 0;
	#endif
		}



return 0;
}


//*****************************************************************************
//*
//*		FtpShutdown
//*
//*****************************************************************************
//	Alle FTP-Resourcen wieder freigeben 
unsigned FtpShutdown()
{

	#ifdef	FTP_NO_SFTP
	
	if(iAzCommInit)
		{
		AzCommUnloadDll();
		iAzCommInit = 0;
		}

	#endif 			

	#ifndef FTP_NO_AXCOMM
	
	CiCommDllUnload();
	
	#endif


return 1;	
}

//*****************************************************************************
//*
//*		FtpAxCommPath
//*
//*****************************************************************************
//	Den Pfad fr die AX-COMM-DLL setzen
LPCTSTR FtpAxCommPath(LPCTSTR pPath)
{

	#ifndef FTP_NO_AXCOMM

	if(pPath)strlcpy(cAxDllPath,pPath,MAX_PATH);
	return cAxDllPath;

	#else

	return _T("");

	#endif 	
}