//*****************************************************************************
//*
//*
//*		HardwareBreakPoint.cpp
//*
//*
//*****************************************************************************
#include	<windows.h>
#include	<tlhelp32.h>
#include	<stdlib.h>

#include	"HardwareBreakPoint.h"


#define 	v			volatile
#define 	HWBP_API	extern "C"
#ifndef	   _WIN64
#ifndef		DWORD_PTR
#define		DWORD_PTR	DWORD
#endif
#endif		

typedef struct
	{
	void			   *pPointer;
	HANDLE				hThread;
	HWBP_TYPE			iType;
	HWBP_SIZE			iSize;
	int					iRegister;
	char 				cAction;
	bool				bOk;
	}BreakPoint;


typedef struct
	{
    HANDLE				UniqueProcess;
    HANDLE				UniqueThread;
	}CLIENT_ID;

typedef struct
	{
	ULONG               ExitStatus;
	PVOID				TebBaseAddress;
	CLIENT_ID			ClientId;
	ULONG				AffinityMask;
	LONG				Priority;
	LONG				BasePriority;
	}THREAD_BASIC_INFORMATION;


typedef enum _THREADINFOCLASS
	{
    ThreadBasicInformation,
    ThreadTimes,
    ThreadPriority,
    ThreadBasePriority,
    ThreadAffinityMask,
    ThreadImpersonationToken,
    ThreadDescriptorTableEntry,
    ThreadEnableAlignmentFaultFixup,
    ThreadEventPair_Reusable,
    ThreadQuerySetWin32StartAddress,
    ThreadZeroTlsCell,
    ThreadPerformanceCount,
    ThreadAmILastThread,
    ThreadIdealProcessor,
    ThreadPriorityBoost,
    ThreadSetTlsArrayAddress,
    ThreadIsIoPending,
    ThreadHideFromDebugger,
    ThreadBreakOnTermination,
    ThreadSwitchLegacyState,
    ThreadIsTerminated,
    ThreadLastSystemCall,
    ThreadIoPriority,
    ThreadCycleTime,
    ThreadPagePriority,
    ThreadActualBasePriority,
    ThreadTebInformation,
    ThreadCSwitchMon,
    ThreadCSwitchPmu,
    ThreadWow64Context,
    ThreadGroupInformation,
    ThreadUmsInformation,
    ThreadCounterProfiling,
    ThreadIdealProcessorEx,
    MaxThreadInfoClass
	}THREADINFOCLASS;

static BreakPoint v    *pBreakPoint = 0;
static int volatile		iInitDone	= 0;
static HANDLE v			hCurrentTh	= 0;
static HANDLE			hBpThread	= 0;
static HANDLE			hLockSem	= 0;
static HANDLE			hNextSem	= 0;
static HANDLE			hDoneSem	= 0;
static DWORD			dwBpThread	= 0;

static DWORD WINAPI 	HardwareBreakPointThread(LPVOID pParam);




//*****************************************************************************
//*
//*		OpenThread
//*
//*****************************************************************************
//	Ersatzfunktion fr OpenThread in VisualStuatio 6.0
#ifndef		OpenThread
#define		OpenThread	OpenThreadDll

typedef HANDLE (WINAPI *TypeOpenThread)(DWORD,BOOL,DWORD);
static	TypeOpenThread	OpenThreadProc=0;

static HANDLE OpenThreadDll(DWORD dwDesiredAccess,BOOL bInheritHandle,DWORD dwThreadId)
{


	if(!OpenThreadProc)
		{
			OpenThreadProc=(TypeOpenThread)GetProcAddress(GetModuleHandle("KERNEL32"),"OpenThread");
		if(!OpenThreadProc)return FALSE;
		}


return OpenThreadProc(dwDesiredAccess,bInheritHandle,dwThreadId);
}

#endif

//*****************************************************************************
//*
//*		GetThreadId
//*
//*****************************************************************************
//	Ersatzfunktion fr GetThreadId in VisualStuatio 6.0
#ifndef		GetThreadId
#define		GetThreadId	GetThreadId

typedef ULONG (WINAPI  *TypeNtQueryInfo)(HANDLE,THREADINFOCLASS,PVOID,ULONG,PULONG);
static	TypeNtQueryInfo	NtQueryInformationThread=0;


static DWORD GetThreadId(HANDLE hThread)
{
THREAD_BASIC_INFORMATION	sInfo;
ULONG						uSize;
ULONG						uRet;

	if(!NtQueryInformationThread)
		{
		NtQueryInformationThread = (TypeNtQueryInfo)GetProcAddress(GetModuleHandle("ntdll.dll"),"NtQueryInformationThread");
		if(!NtQueryInformationThread)return 0;
		}

       uRet = NtQueryInformationThread(hThread,ThreadBasicInformation,&sInfo,sizeof(sInfo),&uSize);
	if(uRet)
		{
		return 0;
		}


return (DWORD)sInfo.ClientId.UniqueThread;
}

#endif


//*****************************************************************************
//*
//*		SetBits
//*
//*****************************************************************************
//	ndert einige Bits in einem Wert
//	dwValue		: Ist der Wert der gendert werden soll
//	iStart		: Ist das Bit ab dem gendert wird
//	iBits		: Ist die Anzahl an Bits die gendert werden sollen
//	iNewValue	: Ist der neue Wert fr die zu nderenden Bits
static inline void SetBits(DWORD_PTR &dwValue,int iStart,int iBits,int iNewValue)
{
DWORD_PTR	dwMask;


	dwMask      = (1<<iBits)-1;
	dwMask    <<=  iStart;
	iNewValue <<=  iStart;
	iNewValue  &=  dwMask;
	dwValue     = ~dwMask&dwValue;
	dwValue    |=  iNewValue;

}

//*****************************************************************************
//*
//*		HardwareBreakPointInit
//*
//*****************************************************************************
//	Bendet den HardwareBreakPointThread
HWBP_API void HardwareBreakPointInit()
{

	if(iInitDone==0)
		{
		iInitDone = 1;
		hLockSem  = CreateSemaphore(NULL,1,0x70000000,NULL);
		hNextSem  = CreateSemaphore(NULL,0,0x70000000,NULL);
		hDoneSem  = CreateSemaphore(NULL,0,0x70000000,NULL);
		hBpThread = CreateThread(0,0,HardwareBreakPointThread,NULL,CREATE_SUSPENDED ,&dwBpThread);

		SetThreadPriority(hBpThread,THREAD_PRIORITY_HIGHEST);
		ResumeThread(hBpThread);
		atexit(HardwareBreakPointExit);

		if(!OpenThreadProc)
			{
			OpenThreadProc=(TypeOpenThread)GetProcAddress(GetModuleHandle("KERNEL32"),"OpenThread");
			}

		if(!NtQueryInformationThread)
			{
			NtQueryInformationThread = (TypeNtQueryInfo)GetProcAddress(GetModuleHandle("ntdll.dll"),"NtQueryInformationThread");
			}
		}

}

//*****************************************************************************
//*
//*		HardwareBreakPointExit
//*
//*****************************************************************************
//	Bendet den HardwareBreakPointThread
void __cdecl HardwareBreakPointExit()
{

	if(iInitDone==0)return;
	if(iInitDone==1)
		{
		iInitDone=2;
		ReleaseSemaphore(hNextSem,1,NULL);
		}

}

//*****************************************************************************
//*
//*		HardwareBreakPointThread
//*
//*****************************************************************************
//	Thread der die Breakpoints setzt und freigibt.
static DWORD WINAPI HardwareBreakPointThread(LPVOID pParam)
{
CONTEXT			sContext={0};
THREADENTRY32	sThreadData;
DWORD			dwMyProcess;
DWORD			dwThreadId;
HANDLE			hSnapshot;
HANDLE			hBpTread;
bool			bDr0Busy;
bool 			bDr1Busy;
bool 			bDr2Busy;
bool 			bDr3Busy;
int				iFlagBit;
int				iMode;
int				iSize;
int				iOk;


	sThreadData.dwSize = sizeof(sThreadData);
	dwMyProcess	= GetCurrentProcessId();
	hSnapshot	= 0;
	hBpTread	= 0;

	for(;;)
		{
		if(hSnapshot)
			{
			if(hBpTread)
				{
				CloseHandle(hBpTread);
				hBpTread = NULL;
				}

			if(!Thread32Next(hSnapshot,&sThreadData))			// Nchsten Thread holen
				{
				ReleaseSemaphore(hDoneSem,1,NULL);
				CloseHandle(hSnapshot);
				hSnapshot = 0;
				continue;
				}

			if(dwMyProcess!=sThreadData.th32OwnerProcessID)		// Nur vom eigenen Prozess
				{
				hBpTread = NULL;
				continue;
				}

			if(dwBpThread==sThreadData.th32ThreadID)			// Eigenen Thread auslassen
				{
				hBpTread = NULL;
				continue;
				}

				hBpTread = OpenThread(THREAD_ALL_ACCESS,0,sThreadData.th32ThreadID);
			if(!hBpTread)
				{
				continue;
				}
			}
		else{
			WaitForSingleObject(hNextSem,INFINITE);

			if(iInitDone==2)break;

			hBpTread = pBreakPoint->hThread;

			if(hBpTread==HWBP_ALL_THREADS)						// Fr alle Thread setzen
				{
					hSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD,dwMyProcess);
				if(!hSnapshot)
					{
					pBreakPoint->bOk = false;
					ReleaseSemaphore(hDoneSem,1,NULL);
					continue;
					}

				if(!Thread32First(hSnapshot,&sThreadData))
					{
					pBreakPoint->bOk = false;
					ReleaseSemaphore(hDoneSem,1,NULL);
					CloseHandle(hSnapshot);
					hSnapshot = 0;
					continue;
					}

				if(dwMyProcess!=sThreadData.th32OwnerProcessID)	// Nur vom eigenen Prozess
					{
					hBpTread = NULL;
					continue;
					}

				if(dwBpThread==sThreadData.th32ThreadID)		// Eigenen Thread auslassen
					{
					hBpTread = NULL;
					continue;
					}

					hBpTread = OpenThread(THREAD_ALL_ACCESS,0,sThreadData.th32ThreadID);
				if(!hBpTread)
					{
					continue;
					}
				}

			   dwThreadId = GetThreadId(hBpTread);
			if(dwThreadId==dwBpThread)							// Eigenen Thread auslassen
				{
				ReleaseSemaphore(hDoneSem,1,NULL);
				continue;
				}
			}


		sContext.ContextFlags = CONTEXT_DEBUG_REGISTERS;		// Context-Daten vom Thread holen

		   iOk = SuspendThread(hBpTread);
		if(iOk<0 || !GetThreadContext(hBpTread,&sContext))
			{
			ResumeThread(hBpTread);

			if(!hSnapshot)
				{
				ReleaseSemaphore(hDoneSem,1,NULL);
				}

			continue;
			}

		iFlagBit = 0;
		bDr0Busy = false;
		bDr1Busy = false;
		bDr2Busy = false;
		bDr3Busy = false;

		if(sContext.Dr7&0x01)									// Welche Breakpoints sind busy
			{
			bDr0Busy = true;
			}

		if(sContext.Dr7&0x04)
			{
			bDr1Busy = true;
			}

		if(sContext.Dr7&0x10)
			{
			bDr2Busy = true;
			}

		if(sContext.Dr7&0x40)
			{
			bDr3Busy = true;
			}


		if(pBreakPoint->cAction==1)								// Breakpoint entfernen
			{
			if(pBreakPoint->iRegister==0)
				{
				sContext.Dr0	= 0;
				iFlagBit        = 0;
				bDr0Busy		= false;
				}

			if(pBreakPoint->iRegister==1)
				{
				sContext.Dr1	= 0;
				iFlagBit		= 2;
				bDr1Busy		= false;
				}

			if(pBreakPoint->iRegister==2)
				{
				sContext.Dr2	= 0;
				iFlagBit		= 4;
				bDr2Busy		= false;
				}

			if(pBreakPoint->iRegister==3)
				{
				sContext.Dr3	= 0;
				iFlagBit		= 6;
				bDr3Busy		= false;
				}

			sContext.Dr7 &= ~(1<<iFlagBit);
			}
		else{
			if(pBreakPoint->cAction==2)							// Breakpoint aktualisieren
				{
				if(pBreakPoint->iRegister==0)
					{
					sContext.Dr0	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr0Busy          = true;
					}
				else if(pBreakPoint->iRegister==1)
					{
					sContext.Dr1	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr1Busy          = true;
					}
				else if(pBreakPoint->iRegister==2)
					{
					sContext.Dr2	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr2Busy          = true;
					}
				else if(pBreakPoint->iRegister==3)
					{
					sContext.Dr3	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr3Busy		  = true;
					}
				else{											// Kein Breakpoint frei
					ResumeThread(hBpTread);

					if(!hSnapshot)
						{
						ReleaseSemaphore(hDoneSem,1,NULL);
						}

					continue;
					}
				}
			else{												// Breakpoint setzen
				if(!bDr0Busy)
					{
					pBreakPoint->iRegister = 0;
					sContext.Dr0	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr0Busy          = true;
					}
				else if(!bDr1Busy)
					{
					pBreakPoint->iRegister = 1;
					sContext.Dr1	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr1Busy          = true;
					}
				else if(!bDr2Busy)
					{
					pBreakPoint->iRegister = 2;
					sContext.Dr2	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr2Busy          = true;
					}
				else if(!bDr3Busy)
					{
					pBreakPoint->iRegister = 3;
					sContext.Dr3	  = (DWORD_PTR)pBreakPoint->pPointer;
					bDr3Busy		  = true;
					}
				else{											// Kein Breakpoint frei
					ResumeThread(hBpTread);

					if(!hSnapshot)
						{
						ReleaseSemaphore(hDoneSem,1,NULL);
						}

					continue;
					}
				}

			sContext.Dr6 = 0;
			iMode		 = 0;
			iSize		 = 0;

			if(pBreakPoint->iType==HWBP_TYPE_CODE)
				{
				iMode = 0;
				}

			if(pBreakPoint->iType==HWBP_TYPE_RDWR)
				{
				iMode = 3;
				}

			if(pBreakPoint->iType==HWBP_TYPE_WRITE)
				{
				iMode = 1;
				}

			if(pBreakPoint->iSize==HWBP_SIZE_1)
				{
				iSize = 0;
				}

			if(pBreakPoint->iSize==HWBP_SIZE_2)
				{
				iSize = 1;
				}

			if(pBreakPoint->iSize==HWBP_SIZE_4)
				{
				iSize = 3;
				}

			if(pBreakPoint->iSize==HWBP_SIZE_8)
				{
				iSize = 2;
				}

			SetBits(sContext.Dr7,16+pBreakPoint->iRegister*4,2,iMode);
			SetBits(sContext.Dr7,18+pBreakPoint->iRegister*4,2,iSize);
			SetBits(sContext.Dr7,   pBreakPoint->iRegister*2,1,1);
			}


		iOk	= SetThreadContext(hBpTread,&sContext);				// Daten wieder in den Thread Context

GetThreadContext(hBpTread,&sContext);
GetThreadId(hBpTread);


		ResumeThread(hBpTread);

		if(iOk)
			{
			pBreakPoint->bOk = true;
			}

		if(!hSnapshot)
			{
			ReleaseSemaphore(hDoneSem,1,NULL);
			}
		}


	ReleaseSemaphore(hLockSem,1,NULL);
	CloseHandle(hBpThread);
	CloseHandle(hLockSem);
	CloseHandle(hNextSem);
	CloseHandle(hDoneSem);


	dwBpThread= 0;
	hBpThread = 0;
	hLockSem  = 0;
	hNextSem  = 0;
	hDoneSem  = 0;
	iInitDone = 0;


return 0;
}

//*****************************************************************************
//*
//*		HardwareBreakPointThread
//*
//*****************************************************************************
//	Ergibt die Thread-ID des Threads fr das ndern der BreakPoints.
unsigned HardwareBreakPointThread()
{
return dwBpThread;
}

//*****************************************************************************
//*
//*		HardwareBreakpointSet
//*
//*****************************************************************************
//	Setzt einen neuen Hardware-Breakpoint
//	hThread		: Ist der Thread in dem der Breakpoint gesetzt werden soll.
//				 	HWBP_ALL_THREADS steht fr alle Thread
//	iType		: Bstimm die Art des Breakpoints
//					HWBP_TYPE_CODE		= Wenn Code aufgerufen wird
//					HWBP_TYPE_WRITE		= Wenn auf Speicher geschrieben wird
//					HWBP_TYPE_READWRITE	= Wenn von Speicher gelesen wird
//	iSize		: Bestimmt die Gre der Speicherstelle (sihe HWBP_SIZE_?)
//	pPtr		: Ist der Zeiger auf die Speicherstelle
//	Ergibt das Handle auf den Breakpoints oder NULL bei einem Fehler.
HWBP_API void *HardwareBreakpointSet(void *hThread,HWBP_TYPE iType,HWBP_SIZE iSize,void *pPtr)
{
DWORD		dwProcess;
HANDLE		hOpenTh;
BreakPoint *pPoint;



	if(iInitDone==0)
		{
		HardwareBreakPointInit();
		}

	if(hThread==hBpThread)										// Ist es der Breakpoint-Thread
		{
		return NULL;
		}

	pPoint = new BreakPoint;
	memset(pPoint,0,sizeof(BreakPoint));


	WaitForSingleObject(hLockSem,INFINITE);						// Warte bis Resourcen frei sind


	pPoint->pPointer	= pPtr;
	pPoint->iSize		= iSize;
	pPoint->iType		= iType;
	pBreakPoint			= pPoint;
	hCurrentTh			= GetCurrentThread();

	if(hThread==hCurrentTh || hThread==HWBP_CUR_THREAD)			// Im eigenen Thread setzen
		{
		dwProcess		= GetCurrentThreadId();

		if(dwProcess==dwBpThread)
			{
			ReleaseSemaphore(hLockSem,1,NULL);
			delete pPoint;
			return NULL;
			}

		hOpenTh			= OpenThread(THREAD_ALL_ACCESS,0,dwProcess);
		pPoint->hThread	= hOpenTh;
		}
	else{														// In anderen Thread setzen
		pPoint->hThread	= hThread;
		hOpenTh			= NULL;
		}


	ReleaseSemaphore(hNextSem,1,NULL);
	WaitForSingleObject(hDoneSem,INFINITE);						// Warte auf die Antwort
	ReleaseSemaphore(hLockSem,1,NULL);


	if(hOpenTh)
		{
		pPoint->hThread	= hThread;
		CloseHandle(hOpenTh);
		}

	if(pPoint->bOk==0)
		{
		delete pPoint;
		return NULL;
		}



return pPoint;
}

//*****************************************************************************
//*
//*		HardwareBreakpointUpdate
//*
//*****************************************************************************
//	Aktualisiert einen Hardware-Breakpoint in eine Thread
//	hBreakPoint	: Ist das Handle des Breakpoints
//	hThread		: Ist der Thread in dem der Breakpoint gesetzt werden soll.
//				 	HWBP_ALL_THREADS steht fr alle Thread
//					HWBP_CUR_THREAD  steht fr den akuellen Thread
//	Ergibt 1 wenn der Breakpoint aktualisiert wurde
int HardwareBreakpointUpdate(void *hBreakPoint,void *hThread)
{
DWORD		dwProcess;
HANDLE		hOpenTh;
BreakPoint *pPoint;



	if(hBreakPoint==0)return 0;

	if(iInitDone==0)
		{
		HardwareBreakPointInit();
		}


	WaitForSingleObject(hLockSem,INFINITE);						// Warte bis Resourcen frei sind


	pPoint				= (BreakPoint*)hBreakPoint;
	pPoint->cAction		= 2;
	pBreakPoint			= pPoint;
	hCurrentTh			= GetCurrentThread();

	if(hThread==hCurrentTh || hThread==HWBP_CUR_THREAD)
		{
		dwProcess		= GetCurrentThreadId();
		hOpenTh			= OpenThread(THREAD_ALL_ACCESS,0,dwProcess);
		pPoint->hThread	= hOpenTh;
		}
	else{
		pPoint->hThread	= hThread;
		hOpenTh			= NULL;
		}


	ReleaseSemaphore(hNextSem,1,NULL);
	WaitForSingleObject(hDoneSem,INFINITE);						// Warte auf die Antwort
	ReleaseSemaphore(hLockSem,1,NULL);


	if(hOpenTh)
		{
		pPoint->hThread	= hThread;
		CloseHandle(hOpenTh);
		}


return pPoint->bOk;
}

//*****************************************************************************
//*
//*		HardwareBreakpointDel
//*
//*****************************************************************************
//	Entfernt einen Hardware-Breakpoint
//	hBreakPoint	: Ist das Handle des Breakpoints
//	Ergibt 1 wenn der Breakpint gesetzt wurde oder 0 bei einem Fehler
HWBP_API int HardwareBreakpointDel(void *hBreakPoint)
{
BreakPoint *pPoint=(BreakPoint*)hBreakPoint;
DWORD		dwProcess;
HANDLE		hOpenTh;



	if(iInitDone!=1)return FALSE;
	if(pPoint==NULL)return FALSE;


	WaitForSingleObject(hLockSem,INFINITE);						// Warte bis Resourcen frei sind


	hCurrentTh			= GetCurrentThread();
	pBreakPoint			= pPoint;
	pPoint->cAction		= 1;

	if(pPoint->hThread==hCurrentTh)
		{
		dwProcess		= GetCurrentThreadId();
		hOpenTh			= OpenThread(THREAD_ALL_ACCESS,0,dwProcess);
		pPoint->hThread	= hOpenTh;
		}
	else{
		hOpenTh			= NULL;
		}


	ReleaseSemaphore(hNextSem,1,NULL);
	WaitForSingleObject(hDoneSem,INFINITE);						// Warte auf die Antwort
	ReleaseSemaphore(hLockSem,1,NULL);

	if(hOpenTh)
		{
		CloseHandle(hOpenTh);
		}

	delete pPoint;



return TRUE;
}
