//*****************************************************************************
//*
//*
//*		HexFind.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//
//	Mit dieser Klasse Knnen binre Daten in eineer Liste von Dateien
//	gesucht werden. Bei der Sucher knnen verschiedene Optionen
//	eingestellt werden.
//
#include	<stdlib.h>
#include	<malloc.h>

#include	"HexFind.h"
#include	"MapFile.h"
#include	"MemFunctions.h"
#include	"FileFunctions.h"
#include	"StringFunctions.h"


#define 	MAX_COUNT	1000			// Maximal gefunden Anzahl an Positionen
#define 	GRUPPE_MAX	16				// Maximale Gre fr eine Gruppensuche

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

int			StrToBin(const char *Str ,void *Mem,int iSize);

#ifndef 	int64
#define 	int64	__int64
#endif


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CHexFind::CHexFind()
{
	sPositions.DataLength(sizeof(PosType));
	iMaxCount	= MAX_COUNT;
	iSearchFile	= 0;							// Aktuelle Suchdatei
	iSearchPos	= 0;							// Aktuelle Suchposition
	iSearchByte = 0;							// Abgesuchte Bytes
	iStop1      = 0;							// Flags fr Break
	iSortMode	=-1;							// Sortiermodus
	iSortDir	= 0;							// Sortierrichtung
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CHexFind::~CHexFind()
{

}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Datei nach einem oder mehreren Speicherbereichen.
//	Die Speicherbereiche sind ber einen Text im folgenden Format definiert:
//
//  Format: [Bereichs-Lnge]                    /00
//          [Art-Byte]:[Lnge]   [/G] :Hex-Dump /00
//			[Art-Byte]:[Lnge]   [/G] :Hex-Dump /00 ...
//          /00
//          /G       :  fr Gro/Kleinschreibung ignorieren
//          Art-Byte :  'D'-Dezimal  'H'-Hexadez. 'B'=Binr 'T'=Text 'U'=Hex-Dump 'M'=Masken-Dump
//
//			Bei Masken-Dumps muss die Lnge halbiert werden.
//			Die obere Hlft enthlt die Maske, die unter die Daten.
//
//	pMem		: Zeiger auf die Daten aus der Datei.
//	iSize		: Ist die Gre der Daten aus der Datei.
//	pFilename	: Ist die Datei die durchsucht werden soll.
//	pText		: Enthlt die Suchdaten wie im Format oben beschrieben.
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_SAMEPOS  Gruppen Suche mit der gleichen Reihenfolge
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(const void *pMem,int iSize,const char *pFilename,const char *pText,int iMode)
{
PosType		sPosition;
char		cPath[MAX_PATH],cTemp[64];
int			iNum,iMax,j,i;


	iStop1		= 0;
	iSearchPos	= 0;									// Aktuelle Suchposition

	if(!pText || !*pText || !pMem || iSize<=0)return 0;
	if(sPositions.Count()       >=  iMaxCount)return 0;

	iNum = 0;

	while(iSearchPos<iSize)
		{
		if(iStop1)break;

		   iSearchPos=FindTxt(pMem,pText,iSearchPos,iSize,iMode);
		if(iSearchPos<0)
			{
			iSearchPos=iSize;
			break;
			}

		if(!iNum)										// Neue Datei
			{
			_fullpath(cPath,pFilename,sizeof(cPath));
			   sPosition.iFile = sFiles.Find(cPath,0,-1,1);
			if(sPosition.iFile<0)sPosition.iFile=sFiles.Add(cPath);
			}

		iNum++;
		sPosition.iPos = iSearchPos;
		sPosition.iLen = iFindLen;

		iMax=16;										// Anzeigetext erstellen
		if(iSize-iSearchPos<iMax)iMax=iSize-iSearchPos;
		i=htostr((char*)pMem+iSearchPos,cTemp,iMax);

		cTemp[i     ]=' ';i++;
		cTemp[i     ]=' ';i++;
		cTemp[i+iMax]= 0 ;

		memcpy(cTemp+i,(char*)pMem+iSearchPos,iMax);

		for(j=i;j<i+iMax;j++)
			{
			if(cTemp[j]>=0 && cTemp[j]<=' ')cTemp[j]='.';
			}

		i+=iMax;

		   sPositions.Add(i,cTemp,CSC_END,&sPosition);
		if(sPositions.Count()>= iMaxCount)break;;

		iSearchPos++;
		}


return iNum;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Datei nach einen Speicherbereich
//	pMem		: Zeiger auf die Daten aus der Datei.
//	iSize		: Ist die Gre der Daten aus der Datei.
//	pFilename	: Ist die Datei die durschsucht werden soll
//	pBin		: Ist der Zeiger auf den zu suchenden Speicherbereich
//	iLen		: Ist die Lnge der Speicherbereiches in Byte
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_BIN			Eins zu eins Suche
//					CHF_NOCASE		Gros/Kleinbuchstaben ignorieren
//					CHF_MASK		Masken-Suche, die Bit-Maske kommt nach Bytes (Bytes+iSize)
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(const void *pMem,int iSize,const char *pFilename,void *pBin,int iLen,int iMode)
{
PosType		sPosition;
char		cPath[MAX_PATH],cTemp[80];
int			iNum,iMax,i,j;


	iStop1		= 0;
	iSearchPos	= 0;								// Aktuelle Suchposition

	if(pBin==0 || iLen<=0 || !pMem || iSize<=0)return 0;
	if(sPositions.Count()         >= iMaxCount)return 0;

	iNum = 0;
	sPosition.iLen  = iLen;

	while(iSearchPos<iSize)
		{
		if(iStop1)break;

		   iSearchPos=FindBin(pMem,(const char*)pBin,iSize,iSearchPos,iLen,iMode);
		if(iSearchPos<0)break;

		if(!iNum)									// Neue Datei
			{
			_fullpath(cPath,pFilename,sizeof(cPath));
			   sPosition.iFile = sFiles.Find(cPath,0,-1,1);
			if(sPosition.iFile<0)sPosition.iFile=sFiles.Add(cPath);
			}

		iNum++;
		sPosition.iPos = iSearchPos;

		iMax=16;									// Anzeige Text erstellen
		if(iSize-iSearchPos<iMax)iMax=iSize-iSearchPos;
		i=htostr((char*)pMem+iSearchPos,cTemp,iMax);

		cTemp[i     ]=' ';i++;
		cTemp[i     ]=' ';i++;
		cTemp[i+iMax]= 0 ;

		memcpy(cTemp+i,(char*)pMem+iSearchPos,iMax);

		for(j=i;j<i+iMax;j++)
			{
			if(cTemp[j]>=0 && cTemp[j]<=' ')cTemp[j]='.';
			}

		i+=iMax;

		   sPositions.Add(i,cTemp,CSC_END,&sPosition);
		if(sPositions.Count() >= iMaxCount)break;;

		iSearchPos++;
		}



return iNum;
}


//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Datei nach einem oder mehreren Speicherbereichen.
//	Die Speicherbereiche sind ber einen Text im folgenden Format definiert:
//
//  Format: [Bereichs-Lnge]                    /00
//          [Art-Byte]:[Lnge]   [/G] :Hex-Dump /00
//			[Art-Byte]:[Lnge]   [/G] :Hex-Dump /00 ...
//          /00
//          /G       :  fr Gro/Kleinschreibung ignorieren
//          Art-Byte :  'D'-Dezimal  'H'-Hexadez. 'B'=Binr 'T'=Text 'U'=Hex-Dump 'M'=Masken-Dump
//
//			Bei Masken-Dumps muss die Lnge halbiert werden.
//			Die obere Hlft enthlt die Maske, die unter die Daten.
//
//	pFilename	: Ist die Datei die durchsucht werden soll.
//	pText		: Enthlt die Suchdaten wie im Format oben beschrieben.
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_SAMEPOS  Gruppen Suche mit der gleichen Reihenfolge
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(const char *pFilename,const char *pText,int iMode)
{
CMapFile	sMapFile;
CacheItem  *pItem;
char	   *pPtr;
char	   *pMem;
int			iPos;
int			iLen;


	iStop1		= 0;
	iSearchPos	= 0;										// Aktuelle Suchposition

	if(!pText || !*pText)return 0;
	if(sPositions.Count() >= iMaxCount)return 0;

		pPtr=sMapFile.Open(pFilename);
	if(!pPtr)return 0;

	iLen = sMapFile.GetLength();

	if(pCache && iLen+pCache->GetUserData()<iMaxCache)		// In den Cache einfgen
		{
		pMem = new char[iLen];

		if(pMem)
			{
			iPos   = pCache->Add(pFilename,CSC_ABCG);
			pItem  = (CacheItem*)pCache->Data(iPos);

			pItem->iLen		= iLen;							// Lnge der Cache-Daten
			pItem->iFlag	= 1;							// Wurde die Datei benutzt
			pItem->uTime	= FileTime(pFilename);			// nderungszeit der Datei
			pItem->pMem		= pMem;							// Zeiger auf die Daten

			memcpy(pMem,pPtr,iLen);

			iPos	= iLen+pCache->GetUserData();
			pCache->SetUserData(iPos);
			}
		}


return Find(pPtr,iLen,pFilename,pText,iMode);
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Datei nach einen Speicherbereich
//	pFilename	: Ist die Datei die durschsucht werden soll
//	pBin		: Ist der Zeiger auf den zu suchenden Speicherbereich
//	iSize		: Ist die Lnge der Speicherbereiches in Byte
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_BIN			Eins zu eins Suche
//					CHF_NOCASE		Gros/Kleinbuchstaben ignorieren
//					CHF_MASK		Masken-Suche, die Bit-Maske kommt nach Bytes (Bytes+iSize)
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(const char *pFilename,void *pBin,int iSize,int iMode)
{
CMapFile	sMapFile;
CacheItem  *pItem;
char	   *pPtr;
char	   *pMem;
int			iPos;
int			iLen;


	iStop1		= 0;
	iSearchPos	= 0;								// Aktuelle Suchposition

	if(pBin==0 || iSize<=0)return 0;
	if(sPositions.Count() >= iMaxCount)return 0;

		pPtr=sMapFile.Open(pFilename);
	if(!pPtr)return 0;

	iLen = sMapFile.GetLength();


	if(pCache && iLen+pCache->GetUserData()<iMaxCache)		// In den Cache einfgen
		{
		pMem = new char[iLen];

		if(pMem)
			{
			iPos   = pCache->Add(pFilename,CSC_ABCG);
			pItem  = (CacheItem*)pCache->Data(iPos);

			pItem->iLen		= iLen;							// Lnge der Cache-Daten
			pItem->iFlag	= 1;							// Wurde die Datei benutzt
			pItem->uTime	= FileTime(pFilename);			// nderungszeit der Datei
			pItem->pMem		= pMem;							// Zeiger auf die Daten

			memcpy(pMem,pPtr,iLen);

			iPos	= iLen+pCache->GetUserData();
			pCache->SetUserData(iPos);
			}
		}


return Find(pPtr,iLen,pFilename,pBin,iSize,iMode);
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in meheren Dateien nach einem oder mehreren Speicherbereichen.
//	Die Speicherbereiche sind ber einen Text im folgenden Format definiert:
//
//  Format: [Bereichs-Lnge]                    /00
//          [Art-Byte]:[Lnge]   [/G] :Hex-Dump /00
//			[Art-Byte]:[Lnge]   [/G] :Hex-Dump /00 ...
//          /00
//          /G       :  fr Gro/Kleinschreibung ignorieren
//          Art-Byte :  'D'-Dezimal  'H'-Hexadez. 'B'=Binr 'T'=Text 'U'=Hex-Dump 'M'=Masken-Dump
//
//
//			Bei Masken-Dumps muss die Lnge halbiert werden.
//			Die obere Hlft enthlt die Maske, die unter die Daten.
//
//	pFiles		: Ist die Liste mit den zu durchsuchenden Dateien.
//	pText		: Enthlt die Suchdaten wie im Format oben beschrieben.
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_SAMEPOS  Gruppen Suche mit der gleichen Reihenfolge
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(CFileFinder *pFiles,const char *pText,int iMode)
{
CacheItem  *pItem;
char	   *pName;
int			iLen;
int			iNum;
int			iPos;
int			i;



	iNum		= 0;
	iStop2      = 0;
	iSearchFile	= 0;									// Aktuelle Suchdatei
	iSearchPos	= 0;									// Aktuelle Suchposition
	iSearchByte = 0;									// Abgesuchte Bytes

	pFiles->Reset();

	for(i=0;i<pFiles->Count();i++)
		{
		if(iStop2)break;
		iSearchFile	= i;								// Aktuelle Suchdatei
		pName=pFiles->Name(i,1);
		if(sPositions.Count() >= iMaxCount)break;

		if(pCache)										// Gibt es einen Datei-Cache
			{
			   iPos = pCache->FindFast(pName,0,-1,0x01);
			if(iPos>=0)
				{
				pItem = (CacheItem*)pCache->Data(iPos);

				if(!CmpFileTime(pItem->uTime,pName))	// Hat sich die Datei inzwischen verndert
					{
					pItem->iFlag = 1;
					iNum+=Find(pItem->pMem,pItem->iLen,pName,pText,iMode);
					iSearchByte+=(int64)((unsigned)iSearchPos);
					iSearchPos  =0;
					continue;
					}

				iLen  = pItem->iLen;					// Datei aus dem Cache lschen
				iLen -= pCache->GetUserData();
				pCache->SetUserData(iLen);

				delete pItem->pMem;
				pCache->Del(iPos);
				}
			}

		iNum+=Find(pName,pText,iMode);
		iSearchByte+=(int64)((unsigned)iSearchPos);
		iSearchPos  =0;
		}

return iNum;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in meheren Dateien nach einem Speicherbereiche.
//	pFiles		: Ist die Liste mit den zu durchsuchenden Dateien.
//	pBin		: Ist der Zeiger auf den zu suchenden Speicherbereich
//	iSize		: Ist die Lnge der Speicherbereiches in Byte
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_BIN			Eins zu eins Suche
//					CHF_NOCASE		Gros/Kleinbuchstaben ignorieren
//					CHF_MASK		Masken-Suche, die Bit-Maske kommt nach Bytes (Bytes+iSize)
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(CFileFinder *pFiles,void *pBin,int iSize,int iMode)
{
CacheItem  *pItem;
char	   *pName;
int			iNum;
int			iPos;
int			iLen;
int			i;


	iNum		= 0;
	iStop2		= 0;
	iSearchFile	= 0;									// Aktuelle Suchdatei
	iSearchPos	= 0;									// Aktuelle Suchposition

	for(i=0;i<pFiles->Count();i++)
		{
		if(iStop2)break;

		iSearchFile	= 0;								// Aktuelle Suchdatei
		pName=pFiles->Name(i,1);

		if(sPositions.Count() >= iMaxCount)break;

		if(pCache)										// Gibt es einen Datei-Cache
			{
			   iPos = pCache->FindFast(pName,0,-1,0x01);
			if(iPos>=0)
				{
				pItem = (CacheItem*)pCache->Data(iPos);

				if(!CmpFileTime(pItem->uTime,pName))	// Hat sich die Datei inzwischen verndert
					{
					pItem->iFlag = 1;
					iNum+=Find(pItem->pMem,pItem->iLen,pName,pBin,iSize,iMode);
					iSearchByte+=(int64)((unsigned)iSearchPos);
					iSearchPos  =0;
					continue;
					}

				iLen  = pItem->iLen;					// Datei aus dem Cache lschen
				iLen -= pCache->GetUserData();
				pCache->SetUserData(iLen);

				delete pItem->pMem;
				pCache->Del(iPos);
				}
			}

		iNum        += Find(pName,pBin,iSize,iMode);
		iSearchByte += (int64)((unsigned)iSearchPos);
		iSearchPos   = 0;
		}

return iNum;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in meheren Dateien nach einem oder mehreren Speicherbereichen.
//	Die Speicherbereiche sind ber einen Text im folgenden Format definiert:
//
//  Format: [Bereichs-Lnge]                    /00
//          [iMode-Byte]:[Lnge]   [/G] :Hex-Dump /00
//			[iMode-Byte]:[Lnge]   [/G] :Hex-Dump /00 ...
//          /00
//          /G       :  fr Gro/Kleinschreibung ignorieren
//          iMode-Byte :  'D'-Dezimal  'H'-Hexadez. 'B'=Binr 'T'=Text 'U'=Hex-Dump 'M'=Masken-Dump
//
//
//			Bei Masken-Dumps muss die Lnge halbiert werden.
//			Die obere Hlft enthlt die Maske, die unter die Daten.
//
//	pFiles		: Ist die Liste mit den zu durchsuchenden Dateien.
//	pText		: Enthlt die Suchdaten wie im Format oben beschrieben.
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_SAMEPOS  Gruppen Suche mit der gleichen Reihenfolge
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(CStringChain *pFiles,const char *pText,int iMode)
{
CacheItem  *pItem;
char	   *pName;
int			iNum;
int			iPos;
int			iLen;
int			i;


	iNum		= 0;
	iStop2      = 0;
	iSearchFile	= 0;									// Aktuelle Suchdatei
	iSearchPos	= 0;									// Aktuelle Suchposition
	iSearchByte = 0;									// Abgesuchte Bytes

	for(i=0;i<pFiles->Count();i++)
		{
		if(iStop2)break;
		iSearchFile	= i;								// Aktuelle Suchdatei
		pName=pFiles->Get(i);
		if(sPositions.Count() >= iMaxCount)break;


		if(pCache)										// Gibt es einen Datei-Cache
			{
			   iPos = pCache->FindFast(pName,0,-1,0x01);
			if(iPos>=0)
				{
				pItem = (CacheItem*)pCache->Data(iPos);

				if(!CmpFileTime(pItem->uTime,pName))	// Hat sich die Datei inzwischen verndert
					{
					pItem->iFlag = 1;
					iNum+=Find(pItem->pMem,pItem->iLen,pName,pText,iMode);
					iSearchByte+=(int64)((unsigned)iSearchPos);
					iSearchPos  =0;
					continue;
					}

				iLen  = pItem->iLen;					// Datei aus dem Cache lschen
				iLen -= pCache->GetUserData();
				pCache->SetUserData(iLen);

				delete pItem->pMem;
				pCache->Del(iPos);
				}
			}

		iNum+=Find(pName,pText,iMode);
		iSearchByte+=(int64)((unsigned)iSearchPos);
		iSearchPos  =0;
		}

return iNum;
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in meheren Dateien nach einem Speicherbereiche.
//	pFiles		: Ist die Liste mit den zu durchsuchenden Dateien.
//	pBin		: Ist der Zeiger auf den zu suchenden Speicherbereich
//	iSize		: Ist die Lnge der Speicherbereiches in Byte
//	iMode		: Bestimmt die Art der Suche. Folgende Flags sind erlaubt:
//					CHF_BIN			Eins zu eins Suche
//					CHF_NOCASE		Gros/Kleinbuchstaben ignorieren
//					CHF_MASK		Masken-Suche, die Bit-Maske kommt nach Bytes (Bytes+iSize)
//	Ergibt die Anzahl der gefunden Stellen
int	CHexFind::Find(CStringChain *pFiles,void *pBin,int iSize,int iMode)
{
CacheItem  *pItem;
char	   *pName;
int			iNum;
int			iPos;
int			iLen;
int			i;


	iNum		= 0;
	iStop2		= 0;
	iSearchFile	= 0;									// Aktuelle Suchdatei
	iSearchPos	= 0;									// Aktuelle Suchposition

	for(i=0;i<pFiles->Count();i++)
		{
		if(iStop2)break;

		iSearchFile	= 0;								// Aktuelle Suchdatei
		pName=pFiles->Get(i);

		if(sPositions.Count() >= iMaxCount)break;

		if(pCache)										// Gibt es einen Datei-Cache
			{
			   iPos = pCache->FindFast(pName,0,-1,0x01);
			if(iPos>=0)
				{
				pItem = (CacheItem*)pCache->Data(iPos);

				if(!CmpFileTime(pItem->uTime,pName))	// Hat sich die Datei inzwischen verndert
					{
					pItem->iFlag = 1;
					iNum+=Find(pItem->pMem,pItem->iLen,pName,pBin,iSize,iMode);
					iSearchByte+=(int64)((unsigned)iSearchPos);
					iSearchPos  =0;
					continue;
					}

				iLen  = pItem->iLen;					// Datei aus dem Cache lschen
				iLen -= pCache->GetUserData();
				pCache->SetUserData(iLen);

				delete pItem->pMem;
				pCache->Del(iPos);
				}
			}

		iNum        += Find(pName,pBin,iSize,iMode);
		iSearchByte += (int64)((unsigned)iSearchPos);
		iSearchPos   = 0;
		}

return iNum;
}

//*****************************************************************************
//*
//*		Delete
//*
//*****************************************************************************
//	Lschen die Liste der gefundenen Positionen, und die Dateiliste.
//	Ergibt 1 wenn alles ok war, sonst 0.
int	CHexFind::Delete()
{
	sPositions.Del();
	sFiles	  .Del();

return 1;
}

//*****************************************************************************
//*
//*		Delete
//*
//*****************************************************************************
//	Lschen eines einzelnen Eintrages, in der Liste der gefundenen Positionen.
//	iNum	: Ist die Nummer des Eintrages.
//	Ergibt 1 wenn der Eintrag gelscht wurde, sonst 0
int	CHexFind::Delete(int iNum)
{

	if((unsigned)iNum>=(unsigned)sPositions.Count())return 0;

		sPositions.Del(iNum);
	if(!sPositions.Count())sFiles.Del();


return 1;
}


//*****************************************************************************
//*
//*		GetFileName
//*
//*****************************************************************************
//	Holt den Dateinamen aus der Liste mit den gefundenen Positionen.
//	iNum	: Ist die Nummer des Eintrages in der Liste.
//	iLen	: Hier wird die Lnge des Dateinamens gespeichert.
//  Ergibt einen Zeiger auf den Dateinamen oder NULL bei einem Fehler.
char  *CHexFind::GetFileName(int iNum)
{
PosType	*pPosition;

		pPosition=(PosType*)sPositions.Data(iNum);
	if(!pPosition)return 0;

return sFiles[pPosition->iFile];
}

//*****************************************************************************
//*
//*		GetFileName
//*
//*****************************************************************************
//	Holt den Dateinamen samt Lnge aus der Liste mit den gefundenen Positionen.
//	iNum	: Ist die Nummer des Eintrages in der Liste.
//  Ergibt einen Zeiger auf den Dateinamen oder NULL bei einem Fehler.
char  *CHexFind::GetFileName(int iNum,int &iLen)
{
PosType	*pPosition;


		pPosition=(PosType*)sPositions.Data(iNum);
	if(!pPosition)
		{
		iLen = 0;
		return 0;
		}


return sFiles.Get(pPosition->iFile,iLen);
}


//*****************************************************************************
//*
//*		GetFileName
//*
//*****************************************************************************
//	Holt den Dateinamen aus der Liste mit den gefundenen Positionen.
//	Der Dateiname wird in einen Puffer gespeichert.
//	iNum		: Ist die Nummer des Eintrages in der Liste.
//	pBuffer		: Ist det Zeiger auf den Puffer
//	iSize		: Ist die Gre des Puffers
//  Ergibt die Lnge des gespeicherten Dateinamens.
int CHexFind::GetFileName(int iNum,char *pBuffer,int iSize)
{
PosType	*pPosition;


	if(iSize<=0)return 0;

	pBuffer[0]=0;

		pPosition=(PosType*)sPositions.Data(iNum);
	if(!pPosition)return 0;


return sFiles.Get(pPosition->iFile,pBuffer,iSize);
}


//*****************************************************************************
//*
//*		GetPos
//*
//*****************************************************************************
//	Holt die Byte-Position aus der Liste mit den gefundenen Positionen.
//	Auf einer solchen Position wurde ein Speicherbereich gefunden.
//	iNum	: Ist die Nummer des Eintrages in der Liste.
//	Ergibt die Byte-Position in der Datei bzw. -1 bei einem Fehler
int  CHexFind::GetPos(int iNum)
{
PosType	*pPosition;


		pPosition=(PosType*)sPositions.Data(iNum);
	if(!pPosition)return -1;


return pPosition->iPos;
}

//*****************************************************************************
//*
//*		GetLength
//*
//*****************************************************************************
//	Holt die Byte-Lnge aus der Liste mit den gefundenen Positionen.
//	Diese Lnge bestimmt wieviele Bytes ein Speicherbereich lang ist.
//	iNum	: Ist die Nummer des Eintrages in der Liste.
//	Ergibt die Byte-Lnge in der Datei bzw. -1 bei einem Fehler
int  CHexFind::GetLength(int iNum)
{
PosType	*pPosition;


		pPosition=(PosType*)sPositions.Data(iNum);
	if(!pPosition)return -1;


return pPosition->iLen;
}


//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	Holt den "Dateinamen(Zeile): Text" aus der Liste mit den gefundenen
//	Positionen. Der Text wird in einen Puffer gespeichert.
//	iNum		: Ist die Nummer des Eintrages in der Liste.
//	pBuffer		: Ist det Zeiger auf den Puffer
//	iSize		: Ist die Gre des Puffers
//	iStart		: Ist die Gre des Leeraums der vor den Text gesetzt wird.
//  Ergibt die Lnge des gespeicherten Textes samt Leerraum.
int CHexFind::GetText(int iNum,char *pBuffer,int iSize,int iStart)
{
PosType		*pPosition;
int			 iPos;


	if(iSize<=0)return 0;
	pBuffer[0]=0;

		pPosition=(PosType*)sPositions.Data(iNum);
	if(!pPosition)return 0;

	   iPos=sFiles.Get(pPosition->iFile,pBuffer,iSize);		// Dateiname anhngen
	if(iPos+16>iSize)return iPos;

	pBuffer[iPos]='(';iPos++;								// Zeilennummer anhngen
	iPos+=itostr(pPosition->iPos,pBuffer+iPos);
	pBuffer[iPos]=')';iPos++;
	pBuffer[iPos]=':';iPos++;
	pBuffer[iPos]=' ';iPos++;

	if(iStart+1>iSize)iStart=iSize-1;						// Leerraum einfgen
	for(;iPos<iStart;iPos++)pBuffer[iPos]=' ';


	iPos+=sPositions.Get(iNum,pBuffer+iPos,iSize-iPos);		// Zeilentext anhngen

return iPos;
}


//*****************************************************************************
//*
//*		FindBin
//*
//*****************************************************************************
//	Sucht in einem Speicherbereich nach einem Abschnitt.
//	pMem		: Ist der Zeiger auf den Speicher der durchsucht werden soll.
//	pBin		: Ist der Zeiger auf die zusuchenden Datenabschnitt.
//	iSize	    : Ist die Lnge der Suchdaten in Bytes
//	iStart		: Ist der Offset fr den Beginn der Suche in pMem
//	iLen		: Ist die Lnge des Speichers pMem in Byte
//	iMode		:	0	Eins zu eins Suche
//					1	Gros/Kleinbuchstaben ignorieren
//					2	Masken-Suche, die Bit-Maske kommt nach Bytes (pBin+iSize)
//  Ergibt die Offsetadresse in pMem bzw. -1 wenn nichts gefunden wurde
int	 CHexFind::FindBin(const void *pMem,const void *pBin,int iSize,int iStart,int iLen,int iMode)
{
char	cChar,cByte;
char   *pMask,*pData,*pPtr;



	pData = (char*)pBin;
	pPtr  = (char*)pMem;

	iMode &= 0xFF;
	iLen  -= iSize-1;

	if(iSize <=0   )return iStart;
	if(iLen  <=0   )return -1;
	if(iStart>=iLen)return -1;								// Bereichsprfung


//******************** Suchen *************************************************
	if(iMode==CHF_BIN)
		{
		if(iSize>sizeof(int))
			{
			int		iData;

			iData  =*(int*)pData;
			iSize  -=sizeof(int);
			pData  +=sizeof(int);

			for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
				{
				if(iStop1)return -1;
				if(*(int*)(pPtr+iSearchPos)             !=   iData )continue;
				if(memcmp (pPtr+iSearchPos+sizeof(int),pData,iSize))continue;
 				return iSearchPos;
				}
			}
		else if(iSize==sizeof(int))
			{
			int iData;

			iData=*(int*)pData;

			for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
				{
				if(iStop1)return -1;
				if(*(int*)(pPtr+iSearchPos) != iData)continue;
 				return iSearchPos;
				}
			}
		else if(iSize==2)
			{
			short iData;

			iData = *(short*)pData;

			for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
				{
				if(iStop1)return -1;
				if(*(short*)(pPtr+iSearchPos) != iData)continue;
 				return iSearchPos;
				}
			}
		else if(iSize==1)
			{
			char iData;

			iData = *(char*)pData;

			for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
				{
				if(iStop1)return -1;
				if(*(char*)(pPtr+iSearchPos) != iData)continue;
 				return iSearchPos;
				}
			}
		else{
			char iData[3];

			iData[0]=pData[0];
			iData[1]=pData[1];
			iData[2]=pData[2];

			for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
				{
				if(iStop1)return -1;
				if(pPtr[iSearchPos  ] != iData[0])continue;
				if(pPtr[iSearchPos+1] != iData[1])continue;
				if(pPtr[iSearchPos+2] != iData[2])continue;
				return iSearchPos;
				}
			}
		}

//******************** Suchen (igoriere Gros/Kleinbuchstaben) *****************

	if(iMode==CHF_NOCASE)
		{
		iSize--;
		cByte=pData[0];

		if(cByte>='A' && cByte<='Z')cByte+=32;

		for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
			{
			if(iStop1)return -1;

			   cChar=pPtr[iSearchPos];
			if(cChar!=cByte)
				{
				if(cChar>='A' && cChar<='Z')cChar+=32;
				if(cChar!=cByte)continue;
				}

			if(memicmp(pPtr+iSearchPos+1,pData+1,iSize))continue;

			return iSearchPos;
			}
		}

//******************** Masken-Suche *******************************************

	if(iMode==CHF_MASK)
		{
		pMask= (char*)pData+iSize;

		if(iSize>=sizeof(int))
			{
			int iData;
			int iMask;

			iData  = *(int*)pData;
			iMask  = *(int*)pMask;
			iSize -= sizeof(int);
			pData += sizeof(int);
			pMask += sizeof(int);

			for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
				{
				if(iStop1)return -1;
				if((*(int*)(pPtr+iSearchPos)           ^iData)&iMask       )continue;
				if( memmcmp(pPtr+iSearchPos+sizeof(int),pData, pMask,iSize))continue;
 				return iSearchPos;
				}
			}
		else{
			char iData;
			char iMask;

			iData = *pData;
			iMask = *pMask;

			iSize--;
			pData++;
			pMask++;

			for(iSearchPos=iStart;iSearchPos<iLen;iSearchPos++)
				{
				if(iStop1)return -1;
				if((pPtr[iSearchPos]^iData)        &iMask       )continue;
				if( memmcmp(pPtr+iSearchPos+1,pData,pMask,iSize))continue;
 				return iSearchPos;
				}
			}
		}


return -1;
}

//*****************************************************************************
//*
//*		FindTxt
//*
//*****************************************************************************
//	Sucht in einem Speicherbereich nach einem oder meheren Abschnitten.
//	Die Abschnitte sind ber einen Text im folgenden Format definiert:
//
//  Format: [Bereichs-Lnge]                    /00
//          [Art-Byte]:[Lnge]   [/G] :Hex-Dump /00
//			[Art-Byte]:[Lnge]   [/G] :Hex-Dump /00 ...
//          /00
//          /G       :  fr Gro/Kleinschreibung ignorieren
//          Art-Byte :  'D'-Dezimal  'H'-Hexadez. 'B'=Binr 'T'=Text 'U'=Hex-Dump 'M'=Masken-Dump
//
//			Bei Masken-Dumps muss die Lnge halbiert werden.
//			Die obere Hlft enthlt die Maske, die unter die Daten.
//
//	pMem		: Ist der Zeiger auf den Speicher der durchsucht werden soll.
//	pBin		: Ist der Zeiger auf die zusuchenden Datenabschnitt.
//	iSize	    : Ist die Lnge der Suchdaten in Bytes
//	iStart		: Ist der Offset fr den Beginn der Suche in pMem
//	iLen		: Ist die Lnge des Speichers pMem in Byte
//
//	Mem		: Ist der Zeiger auf den Speicher der durchsucht werden soll.
//	Text	: Enthlt die Suchdaten wie im Format oben beschrieben.
//	Start	: Ist die Position fr den Beginn der Suche
//	Laenge	: Ist die Lnge des Speichers Mem
//	iMode	: CHF_SAMEPOS  Gruppensuche mit der gleichen Reihenfolge
//  Ergibt die Offsetadresse in Mem bzw. -1 wenn nichts gefunden wurde
int	 CHexFind::FindTxt(const void *pMem,const char *pText,int iStart,int iLen,int iMode)
{
int		iStop;
int		i,j,iPos,iLast;
int		iEnd,iNum,iArea,iFrom;
int		iEndHi,iEndLo;
int		iPosHi,iPosLo;
char   *pData,iType,*pPtr;
int		aMode[GRUPPE_MAX];
int		aAddr[GRUPPE_MAX];
int		aSize[GRUPPE_MAX];



	iPosHi = 0;
	iPosLo = 0;
	pPtr   = (char*)pMem;

//******************** Text entschlsseln **************************************

	if(!pText)return -1;
	if(iLen<0)return -1;

	iStop=iStart+iLen;
	iArea=strtoi(pText,0xC000);									// Bereichslnge holen

	for(iNum=-1,i=0;pText[i];iNum++)i+=strlen(pText+i)+1;
	if (iNum< 1        )return -1;
	if (iNum>GRUPPE_MAX)iNum=GRUPPE_MAX;

	pData=(char*)alloca(256*iNum);

	for(i=0;i<iNum;i++)											// Text decodieren
		{
		aMode[i]=0;
		pText  +=strlen (pText)+1;
		pText  +=strwlen(pText,0xA0);							// Leerzeichen berspringen
		iType   =pText[0];
		pText  +=strchar(pText,':');if(pText[0]==':')pText++;
		aSize[i]=strtoi (pText,0xC000);							// Lnge holen
		iLast   =strchar(pText,':');
		iPos    =strchar(pText,'G');if(iPos<iLast && 'G'== pText[iPos])aMode[i]|=1;
		pText  +=iLast;StrToBin(pText,pData+i*256,256);

		if(iType=='M')											// Masken-Dump
			{
			aMode[i]   =2;
			aSize[i] >>=1;
			}
		}

	iFindLen=aSize[0];

//******************** Suchen **************************************************

	do	{
		   aAddr[0]=FindBin(pPtr,pData,aSize[0],iStart,iLen,aMode[0]);
		if(aAddr[0]==-1)break;
		if(iNum    <= 1)break;

		//******************* Suche mit richtiger Reihenfolge ******************
		if(iMode&CHF_SAMEPOS)
			{
			   iEnd=aAddr[0]+iArea;
			if(iEnd>iStop)iEnd=iStop;

			for(i=1;i<iNum;i++)
				{
				iFrom = aAddr[i-1]+aSize[i-1];
				j     = iEnd-iFrom-aSize[i]+1;

				if(j<0)break;
				   j=FindBin(pPtr,pData+i*256,aSize[i],iFrom,iEnd,aMode[i]);
				if(j==-1)break;

				aAddr[i]=j;
				}
			}
		//******************* Suche ohne richtiger Reihenfolge *****************
		else{
			iFrom  = aAddr[0];
			iEndHi = aAddr[0]+iArea;
			iEndLo = aAddr[0]-iArea;

			if(iEndLo<iStart)iEndLo=iStart;
			if(iEndHi>iStop )iEndHi=iStop;

			for(i=1;i<iNum;i++)									// Suche ohne richtiger Reihenfolge
				{
				iPosHi = -1;
				iPosLo = -1;

				   j= iEndHi-iFrom-aSize[i];
				if(j>0)iPosHi=FindBin(pPtr,pData+i*256,aSize[i],iFrom+aSize[0],j,aMode[i]);

				if(iPosHi==-1)
					{
					   j= iFrom-iEndLo-1;
					if(j<0)iPosLo=FindBin(pPtr,pData+i*256,aSize[i],iEndLo,j,aMode[i]);
					}

				if(iPosHi==-1)
					{
					if(iPosLo==-1)break;
					aAddr[i]=iPosLo;
					}
				else{
					if(iPosLo==-1)aAddr[i]=iPosHi;
					else aAddr[i]=(iFrom-iPosLo < iPosHi-iFrom)? iPosLo:iPosHi;
					}
				}
			}

		//******************** Nchste Suche ***********************************

		if(i>=iNum)break;										// Alles gefunden ?

		j =aAddr[0]-iStart;
		j+=(iLen<0)? -1:1;
		if (iLen*iLen<j*j){aAddr[0]=-1;break;}

		iStart += j;
		iLen   -= j;

		}while(1);





return aAddr[0];
}

//*****************************************************************************
//*
//*		SortPath
//*
//*****************************************************************************
int __cdecl CHexFind::SortPath(CStringChain *pList,int iPos1,int iPos2)
{
CHexFind  *pThis=(CHexFind*)pList->GetUserPtr();
const char *pPath1;
const char *pPath2;
PosType	   *pType1;
PosType	   *pType2;
int			iCmp;


	pType1	= (PosType*)pList->Data(iPos1);
	pType2	= (PosType*)pList->Data(iPos2);
	pPath1	= pThis->sFiles.Get(pType1->iFile);
	pPath2	= pThis->sFiles.Get(pType2->iFile);

	   iCmp	= strvicmp(pPath1,pPath2);
	if(iCmp)return iCmp;

	   iCmp	= pType1->iPos-pType2->iPos;
	if(iCmp)return iCmp;


return pType1->iLen-pType2->iLen;
}

//*****************************************************************************
//*
//*		SortName
//*
//*****************************************************************************
int __cdecl CHexFind::SortName(CStringChain *pList,int iPos1,int iPos2)
{
CHexFind  *pThis=(CHexFind*)pList->GetUserPtr();
const char *pPath1;
const char *pPath2;
const char *pName1;
const char *pName2;
PosType	   *pType1;
PosType	   *pType2;
int			iCmp;


	pType1	= (PosType*)pList->Data(iPos1);
	pType2	= (PosType*)pList->Data(iPos2);
	pPath1	= pThis->sFiles.Get(pType1->iFile);
	pPath2	= pThis->sFiles.Get(pType2->iFile);
	pName1	= ::GetFileName(pPath1);
	pName2	= ::GetFileName(pPath2);

	   iCmp	= strvicmp(pName1,pName2);
	if(iCmp)return iCmp;

	   iCmp	= strvicmp(pPath1,pPath2);
	if(iCmp)return iCmp;

	   iCmp	= pType1->iPos-pType2->iPos;
	if(iCmp)return iCmp;


return pType1->iLen-pType2->iLen;
}

//*****************************************************************************
//*
//*		SortType
//*
//*****************************************************************************
int __cdecl CHexFind::SortType(CStringChain *pList,int iPos1,int iPos2)
{
CHexFind  *pThis=(CHexFind*)pList->GetUserPtr();
const char *pPath1;
const char *pPath2;
const char *pName1;
const char *pName2;
PosType	   *pType1;
PosType	   *pType2;
int			iCmp;


	pType1	= (PosType*)pList->Data(iPos1);
	pType2	= (PosType*)pList->Data(iPos2);
	pPath1	= pThis->sFiles.Get(pType1->iFile);
	pPath2	= pThis->sFiles.Get(pType2->iFile);
	pName1	= GetFileType(pPath1);
	pName2	= GetFileType(pPath2);

	   iCmp	= stricmp(pName1,pName2);
	if(iCmp)return iCmp;

	   iCmp	= strvicmp(pPath1,pPath2);
	if(iCmp)return iCmp;

	   iCmp	= pType1->iPos-pType2->iPos;
	if(iCmp)return iCmp;


return pType1->iLen-pType2->iLen;
}

//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiere das Ergbniss
//	iMode		: Ist die Sortierart
//					CHF_PATH	0	Nach dem ganzen Pfad sortieren
//					CHF_NAME	1	Nach dem Dateinamen sortieren
//					CHF_TYPE	2	Nach dem Dateityp sortieren
//	iDir		: Ist die Sortierrichtung (-1=toogle)
//	Errgibt 1 wenn die Reihenfolge verndert wurde, ansonsten 0
int CHexFind::Sort(int iMode,int iDir)
{

	if(iDir<0)
		{
		iDir = (iSortMode==iMode)? !iSortDir:0;
		}

	if(iSortMode==iMode && iSortDir==iDir)				// Sortierrichtung
		{
		return 0;
		}


	sPositions.SetUserPtr(this);

	switch(iMode)
		{
	case CHF_PATH:										// Nach dem ganzen Pfad sortieren

			sPositions.Sort(0,-1,SortPath,iDir);
			break;

	case CHF_NAME:										// Nach dem Dateinamen sortieren

			sPositions.Sort(0,-1,SortName,iDir);
			break;

	case CHF_TYPE:										// Nach dem Dateityp sortieren

			sPositions.Sort(0,-1,SortType,iDir);
			break;

	default:	return 0;
		}


	iSortMode	= iMode;
	iSortDir	= iDir;


return 1;
}

