//*****************************************************************************
//*
//*
//*		HexFind.h: Schnittstelle fr die Klasse CHexFind.
//*
//*
//*****************************************************************************
#ifndef	__HEXFIND_H__
#define	__HEXFIND_H__

#if _MSC_VER > 1000
#pragma once
#endif

#include	"StringChain.h"
#include	"FileFinder.h"


// Konstanten fr Find
#define 	CHF_SAMEPOS		0x8000	// Gruppen suche mit der gleichen Reihenfolge
#define 	CHF_BIN			0x0000	// Eins zu eins Suche
#define 	CHF_NOCASE		0x0001	// Gros/Kleinbuchstaben ignorieren
#define 	CHF_MASK		0x0002	// Masken-Suche, die Bit-Maske kommt nach Bytes (Bytes+Len)

// Konstanten fr SORT
#define 	CHF_PATH		0x0000		// Nach dem ganzen Pfad sortieren
#define 	CHF_NAME		0x0001		// Nach dem Dateinamen sortieren
#define 	CHF_TYPE		0x0002		// Nach dem Dateityp sortieren


#ifndef 	int64
#define 	int64	__int64
#endif


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
class CHexFind
{
public:

	 CHexFind();
	~CHexFind();

	int		Find(const void *pMem,int iSize,const char   *pFilename,const char *pText         ,int iMode=0);
	int		Find(const void *pMem,int iSize,const char   *pFilename,void       *pBin ,int iLen,int iMode=0);
	int		Find(                           const char   *pFilename,const char *pText,         int iMode=0);
	int		Find(                           const char   *pFilename,void       *pBin ,int iLen,int iMode=0);
	int		Find(                           CFileFinder  *pFiles   ,const char *pText,         int iMode=0);
	int		Find(                           CFileFinder  *pFiles   ,void       *pBin ,int iLen,int iMode=0);
	int		Find(                           CStringChain *pFiles   ,const char *pText,         int iMode=0);
	int		Find(                           CStringChain *pFiles   ,void       *pBin ,int iLen,int iMode=0);
	
	int		Count();										// Anzahl der gefunden Eintrge
	int 	Sort(int iMode,int iDir=0);						// Sortiere das Ergbniss
	void	Break();										// Suche abbrechen
	int		Delete();										// Alles lschen
	int		Delete     (int iNr);							// Einen Eintrag lschen
	char   *GetFileText(int iNr);							// Ergibt den Zeilentext
	int     GetFileText(int iNr,char *pBuffer,int iSize);	// Speichert den Zeilentext in einem Puffer
	char   *GetFileName(int iNr);							// Ergibt den Dateinamen
	char   *GetFileName(int iNr,int &iLen);					// Ergibt den Dateinamen mit Lnge
	int     GetFileName(int iNr,char *pBuffer,int iSize);	// Speichert den Dateinamen in einem Puffer
	int     GetSearchFile();								// Ergibt die Suchdate
	int     GetSearchPos ();								// Ergibt die Suchposition
	int64	GetSearchByte();								// Ergibt abgesuchte Bytes
	int     GetPos     (int iNr);							// Ergibt die gefundene Byte Position
	int     GetLength  (int iNr);							// Ergibt Byte Anzahl
	void    SetChache  (CStringChain *pNew,int iMax);		// Den Datei-Cache einstellen
	int     GetText    (int iNr,char *pBuffer,int iSize,int iStart=0);
	int		GetMaxCount();
	int		SetMaxCount(int iMax);

	int		FindBin(const void *pMem,const void *pBytes,int iLen,int iStart,int iSize,int iFlags);
	int		FindTxt(const void *pMem,const char *pText ,         int iStart,int iSize,int iMode );


protected:

	static	int __cdecl SortPath(CStringChain *pList,int iPos1,int iPos2);
	static	int	__cdecl SortName(CStringChain *pList,int iPos1,int iPos2);
	static	int	__cdecl SortType(CStringChain *pList,int iPos1,int iPos2);

//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************

protected:

	typedef	struct											// Suchergebnisse
		{													
		int iPos;											// Byte Position
		int iLen;											// Byte Anzahl
		int iFile;											// Dateinummer
		}PosType;

	typedef	struct											// Daten fr Suchergebnis
		{													
		int			iLen;									// Lnge der Cache-Daten
		int			iFlag;									// Wurde die Datei benutzt
		unsigned 	uTime;									// nderungszeit der Datei
		char	   *pMem;									// Zeiger auf die Daten
		}CacheItem;

	CStringChain   *pCache;									// Cache fr die Dateien
	CStringChain	sFiles;									// Durchsuchte Dateien
	CStringChain	sPositions;								// Gefundene Positionen
	int				iMaxCache;								// Maximale Cache-Gre
	int				iMaxCount;								// Maximale Positionsanzahl
	int				iFindFlags;								// Suchen Flags
	int				iFindLen;								// Anzahl der gesuchten Bytes
	int				iSearchFile;							// Aktuelle Suchdatei
	int				iSearchPos;								// Aktuelle Suchposition
	int64			iSearchByte;							// Abgesuchte Bytes
	volatile int	iStop1,iStop2;							// Flags fr Break
	int				iSortMode;								// Sortiermodus
	int				iSortDir;								// Sortierrichtung

};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

inline	void    CHexFind::SetChache(CStringChain *pNew,int iMax){pCache=pNew;iMaxCache=iMax;}
inline	int		CHexFind::Count(){return sPositions.Count();}
inline	char   *CHexFind::GetFileText(int iNr){return sPositions[iNr];}
inline	int		CHexFind::GetFileText(int iNr,char *Puffer,int Size){return sPositions.Get(iNr,Puffer,Size);}
inline	int		CHexFind::GetMaxCount(){return iMaxCount;}
inline	int		CHexFind::SetMaxCount(int iMax){int iOld=iMaxCount;iMaxCount=iMax;return iOld;}
inline	void	CHexFind::Break(){iStop1=1;iStop2=1;}
inline	int     CHexFind::GetSearchPos (){return iSearchPos;}
inline	int     CHexFind::GetSearchFile(){return iSearchFile;}
inline	int64	CHexFind::GetSearchByte(){return iSearchByte;};


#ifdef 	int64
#undef 	int64
#endif


#endif
