//*****************************************************************************
//*
//*
//*		HttpClient.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#ifdef      GLOBAL_CONFIG
#define     HTTPCLIENT_CPP
#include    <GlobalConfig.h>
#undef      HTTPCLIENT_CPP
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<stddef.h>
#include	<string.h>
#include	<malloc.h>
#include	"HttpClient.h"
#include	"SslFunctions.h"
#include	"System/SocketUtility.h"

#if        _MSC_VER >= 1000
#pragma 	warning( disable :4996 )
#endif

#ifdef		WIN32	
#include	<windows.h>
#endif 	

#ifdef		linux	
#include	<pwd.h>
#endif 	

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifndef 	HCL_USE_SSL
#define 	HCL_USE_SSL			0
#endif

#ifndef 	ADD_HEADER
#define 	ADD_HEADER			768
#endif 	
#ifndef 	MAX_HEADER
#define 	MAX_HEADER			2048
#endif 	
#ifndef 	MAX_RESV_BYTE
#define 	MAX_RESV_BYTE		0x100000
#endif 	
#ifndef		MAX_RESV_FILE
#define 	MAX_RESV_FILE		0x020000
#endif 	

static	char   cHttpGet[]	=	" HTTP/1.0\r\nUser-Agent: Mozilla/5.0 (Windows NT 5.1)\r\n";





//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
CHttpClient::CHttpClient()
{

	hTcp				= 0;
	pData				= 0;
	iError				= 0;
	iToOpen				= 0;
	iDataMax			= 0;
	iDataStop			= 0;
	iDataStart			= 0;
	iDataLength			= 0;
	uCookiePos			= 0;
	uCookieCount		= 0;
	iReconnectMax		= 2;
	iReconnectLevel		= 0;
	iEncodeRequests		= 0x11;
	iConnTimeout		= 60000;
	iNewConnection		= 0;
	iHttpVersion		= 0;
	iProgressType		= 0;
	iProgressPos		= 0;
	iProgressEnd		= 0;
	iDataState			= 0;
	cServername[0]		= 0;

	memset(pCookies,0,sizeof(pCookies));

	aFieldName[0]		= "Accept";
	aFieldText[0]		= "text/html, image/gif, image/x-xbitmap, image/jpeg, image/png, */*";
	aFieldSize[0]		= 65;
	aFieldFree[0]		= 0;
	
	aFieldName[1]		= "Accept-Language";
	aFieldText[1]		= "de";
	aFieldSize[1]		= 2;
	aFieldFree[1]		= 0;

	aFieldName[2]		= "Accept-Charset";
	aFieldText[2]		= "iso-8859-1,*,utf-8";
	aFieldSize[2]		= 18;
	aFieldFree[2]		= 0;

	aFieldName[3]		= "Keep-Alive";
	aFieldText[3]		= "300";
	aFieldSize[3]		= 3;
	aFieldFree[3]		= 0;

	aFieldName[4]		= "Connection";
	aFieldText[4]		= "keep-alive";
	aFieldSize[4]		= 10;
	aFieldFree[4]		= 0;

	iFieldCount			= 5;


	#if HCL_USE_NTLM
	cDomain	 [0]		= 0;
	cHostName[0]		= 0;
	cUserName[0]		= 0;
	cPassWord[0]		= 0;
	#endif

}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
CHttpClient::~CHttpClient()
{


	if(hTcp)Disconnect();

	if(pData)
		{
		delete pData;
		pData = NULL;
		}

	DeleteAllCookies();

	while(iFieldCount>0)							// Extra-Felder lschen	
		{
		iFieldCount--;

		if(aFieldFree[iFieldCount])
			{
			delete aFieldText[iFieldCount];
			}
		}

}



//*****************************************************************************
//*
//*		Connect
//*
//*****************************************************************************
//	Baut eine Verbindung zu einem Webserver auf
//	pServerName	: Name des Servers oder eine URL (z.B. http://www.servername/xxx )
//	iPort		: Ist das TCP-Port fr die Verbindung (80 ist normal)
//	uTimeout	: Gibt das Timeout an fr die Verbindung (0=default)
//	iSsl		: Eine SSL Verbindung aufbauen
//	Ergibt:		 0 wenn ok
//				-6 wenn die Verbindung nicht geffnet wurde
//				-7 wenn der Servername falsch ist
//				-8 wenn der Servername nicht aufgelst werden konnte
int CHttpClient::Connect(const char *pServerName,int iPort,unsigned uTimeout,int iSsl)
{
char	cChar;
IpAddr	iAddr;
int		iTurn;
int		i;


	if(hTcp)Disconnect();

	if(!pServerName)
		{
		iError=HCL_WRONG_SERVERNAME;
		return HCL_WRONG_SERVERNAME;
		}

	if(iPort==443)iSsl=1;

	if(*pServerName=='h' || *pServerName=='H')		// Protokol auswerten
		{
		if(!strnicmp(pServerName,"http:",5))
			{
			pServerName += 5;
			}

		if(!strnicmp(pServerName,"https:",5))
			{
			pServerName += 6;
			iSsl = 1;
			}

		if(*pServerName=='/' || *pServerName=='\\')pServerName++;
		if(*pServerName=='/' || *pServerName=='\\')pServerName++;
		}

													// Verbindungsparameter sichern
	if(cServername!=pServerName)
		{
		strncpy(cServername,pServerName,sizeof(cServername));
		iServerPort	= iPort;		
		iServerSsl  = iSsl;			
		}

	for(i=0,iTurn=0;i<sizeof(cServername)-1;i++)
		{
		cChar=pServerName[i];

		if(cChar== 0  )break;
		if(cChar=='/' )break;
		if(cChar=='\\')break;
		if(cChar!=':' )continue;

		cServername[i] = 0;

		if(iTurn>0)break;

		if(cChar==':' && iPort==0)
			{
			iPort = atoi(pServerName+i+1);
			if(iPort==443)iSsl=1;
			}

		for(i++;pServerName[i]=='/';i++);
		pServerName+=i;
		iTurn++;
		i--;
		}

	cServername[i] = 0;
	if(!iPort)iPort=(iSsl)? 443:80;

	if(!IpHostname(&iAddr,cServername))
		{
		iError=HCL_SERVER_NOT_FOUND;
		return HCL_SERVER_NOT_FOUND;
		}

	if(!uTimeout)
		{
		uTimeout = iConnTimeout;
		}
	else{
		   iConnTimeout = uTimeout;
		if(iConnTimeout<2500)
			{
			iConnTimeout=2500;
			}
		}

	iToOpen = 1;

	#if HCL_USE_SSL
	if(iSsl)hTcp = SslOpenWait(&iAddr,iPort,uTimeout,&sCancel,TCP_CLOSEDOWN); else
	#endif

	hTcp = TcpOpenWait(&iAddr,iPort,uTimeout,&sCancel,0,TCP_CLOSEDOWN);


	iToOpen = 0;
	iError  = (hTcp)? HCL_OK:HCL_NOCONNECTION;



return iError;
}


//*****************************************************************************
//*
//*		Disconnect
//*
//*****************************************************************************
//	Schliest die Verbindung zum Server
//	Ergibt:     0 wenn ok
//			   -1 Falsches Handle
int CHttpClient::Disconnect()
{

	if(!hTcp)return HCL_WRONG_HANDLE;
	TcpClose(hTcp);
	iHttpVersion=0;
	hTcp=0;


return HCL_OK;
}

//*****************************************************************************
//*
//*		Shutdown
//*
//*****************************************************************************
//	Unterbricht die Verbindung zum Server
//	Ergibt:     0 wenn ok
//			   -1 Falsches Handle
int CHttpClient::Shutdown()
{


	if(iToOpen)
		{
		TcpOpenCancel(&sCancel);
		return HCL_OK;
		}

	if(hTcp)
		{
		TcpShutdown(hTcp);
		return HCL_OK;
		}


return HCL_WRONG_HANDLE;
}

//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Liest eine Datei von einer HTTP Verbindung ein, an die Datei wird ein
//	Null-Byte angehngt
//	pFilename: Dateinamen der Datei wo die HTTP Daten gespeichert werden.
//			   Bei NULL werden die Daten im Speicher gehalten
//	pPath	 : Name der gewnschten Datei ohne Servernamen
//	uTimeout : Gibt die Wartezeit an bis auf das letzte Byte gewartet wird.
//	uFlags	 : Diverse Flags fr die Steuerung
//				Bit0	= Auch groe Daten nicht im Speicher halten (fr groe Dateien)
//				Bit4..7 = NTLM-Login-Modus (1=NTLM_V1 2=NTLM_V2+Session 3=NTLM_V2)
//	Ergibt   : Die Lnge der Daten
//			   -1  Falsches Handle
//			   -2  Fehler beim Senden
//			   -3  Fehler beim Lesen
//			   -4  Zuwenig Speicher
//			   -5  Timeout
//			   -9  Es gab zu viele Umleitungen
//			   -10 Das Ziel der Umleitung wurde nicht gefunden
//			   -11 Die Anfrage wurde zu gro
//			   -14 Es wurde eine zugroe Datei empfangen
//			   -15 Es wurde kein HTTP-Header gefunden
//			   -16 Es wurde eine HTTP Fehlermedung empfangen
//			   -17 Es gab eine Fehler im bertragungsformat
//			   -18 Die Weiterleitung zu einem anderen Server schlug fehl
int CHttpClient::Get(const char *pFilename,const char *pPath,unsigned uTimeout,unsigned uFlags)
{
char	cGet[sizeof(cHttpGet)+ADD_HEADER+256*HCL_COOKIES_MAX+4];
int		iLen,iTurn,iVersion;



	iError		= HCL_OK;
	iDataLength	= 0;
	iDataMinus	= 0;
	iDataStop	= 0;
	iDataStart	= 0;
	iDataState  = 0;
	iChunkLen	= 0;

													// Die Verbindung neu aufbauen ?
	if(iNewConnection || (hTcp && TcpIsBroken(hTcp)))
		{
		iVersion=iHttpVersion;
		Disconnect();
		Connect(cServername,iServerPort,0,iServerSsl);
		iHttpVersion=iVersion;
		iNewConnection=0;
		}

	if(!hTcp)
		{
		iError=HCL_WRONG_HANDLE;
		return HCL_WRONG_HANDLE;
		}


	for(iTurn=0;;iTurn++)							// Puffer leeren
		{
		if(TcpGetNext(hTcp,cGet,sizeof(cGet),(iTurn)? 2000:0)<=0)break;
		}


		iLen=CreateRequest("GET",pPath,cGet,sizeof(cGet),0x01);
	if(!iLen)
		{
		iError=HCL_REQUEST_TOO_LARGE;
		return HCL_REQUEST_TOO_LARGE;
		}

	iProgressType	= 1;
	iProgressPos	= 0;
	iProgressEnd	= iLen;


	if(!TcpPut(hTcp,cGet,iLen))						// Header senden
		{
		iProgressType	= 3;
		iError=HCL_SEND_ERROR;
		return HCL_SEND_ERROR;
		}

	

#if HCL_USE_NTLM
Next:
#endif 	

	if(Read(pFilename,uTimeout,uFlags)<0)
		{
		iProgressType = 3;
		iDataLength   = iDataStart;
		pData[iDataLength]=0;

		#if HCL_USE_NTLM
													// Anmeldung ber NTLM
		if(iNtlmPos && iError==HCL_ERROR_CODE  && iDataState==401)
			{
			int		iSize;

			   iSize = HandleNtLm(cGet,iLen,sizeof(cGet),uFlags);
			if(iSize>0)
				{
				if(!TcpPut(hTcp,cGet,iSize))
					{
					iError=HCL_SEND_ERROR;
					return HCL_SEND_ERROR;
					}

				iError = 0;
				goto Next;
				}
			else if(iSize<0)
				{
				iError = iSize;
				}
			}

		#endif

		return iError;
		}


	pData[iDataLength]=0;
	iProgressType = 3;



return iDataLength-iDataStart+iDataMinus;
}

//*****************************************************************************
//*
//*		Reconnect
//*
//*****************************************************************************
//	Wird aufgerufen wenn ein Server eine Anfrage weiterleitet
//	pFilename: Dateinamen der Datei wo die HTTP Daten gespeichert werden.
//			   Bei NULL werden die Daten im Speicher gehalten
//	Des Ziel der Weiterleitung muss im aktuellen Header stehen
int	CHttpClient::Reconnect(const char *pFilename,unsigned uTimeout)
{
int		iRet,iLen,iSsl,iVersion;
char   *pLocation,*pValue;
char	cOldServer[256];
char	cOld;


	if(iReconnectLevel+1>=iReconnectMax)
		{
		iError=HCL_RECONNECT_LOOP;
		return HCL_RECONNECT_LOOP;
		}

	pData[iDataStart]=0;

	    pValue=strstr(pData,"Connection:");
	if( pValue)
		{
		pValue += 11;
		pValue += strspn(pValue," \t");

		if(!memicmp(pValue,"close",5))
			{
			iVersion     = iHttpVersion;
			Disconnect();  
			iHttpVersion = iVersion;
			
			if(Connect(cServername,iServerPort,0,iServerSsl)!=HCL_OK)
				{
				return iError;
				}
			}
		}

		pLocation=strstr(pData,"Location:");
	if(!pLocation)
		{
		iError=HCL_LOCATION_NOT_FOUND;
		return HCL_LOCATION_NOT_FOUND;
		}

	pLocation      += 9;
	pLocation	   += strspn(pLocation," \t");
	iLen			= strcspn(pLocation,";\n\r");
	pLocation[iLen] = 0;
	cOldServer[0  ] = 0;

												
	if(*pLocation=='h' || *pLocation=='H')		// Ist auch ein Server angegeben
		{
		if(!memicmp(pLocation,"http://",7))
			{
			iLen = 7;
			iSsl = 0;
			}
		else if(!memicmp(pLocation,"https://",8))
			{
			iLen = 8;
			iSsl = 1;
			}
		else{
			iLen = 0;
			}

		if(iLen)
			{
			pLocation+=iLen;
			iLen=strlen(cServername);

			if(pLocation[iLen]!='/' || strncmp(pLocation,cServername,iLen))
				{
				iLen=strcspn(pLocation,"/");
				cOld=pLocation[iLen];
				pLocation[iLen]=0;
				strcpy(cOldServer,cServername);

				if(Connect(pLocation,0,0,iSsl))	// Verbinden zum anderen Server
					{
					iNewConnection=1;
					strcpy(cServername,cOldServer);
					return HCL_WRONG_SERVER;
					}

				pLocation[iLen]=cOld;
				pLocation+=iLen;
				}


			iLen = strlen(cServername);

			if(pLocation[iLen]!='/' || memcmp(pLocation,cServername,iLen))
				{
				for(iLen=0;pLocation[iLen];iLen++)
					{
					if(pLocation[iLen]=='/')break;
					}

				// REM AZ hier msste eine neue Verbindung kommen
				}

			pLocation += iLen;
			}
		}




	iReconnectLevel++;
	iRet=Get(pFilename,pLocation,uTimeout);
	iReconnectLevel--;

	if(cOldServer[0])							// Verbinden zum anderen Server wieder abbauen
		{
		iNewConnection=1;
		strcpy(cServername,cOldServer);
		}


return iRet;
}

//*****************************************************************************
//*
//*		AddCookie
//*
//*****************************************************************************
//	Fgt ein Cookie in die Liste ein.
//	pText	: Ist der Cookie-Text
//	pName	: Ist der Cookie-Name (Bei 0 kein Name)
//	Ergibt:	Die einfge Position bzw -1 bei einem Fehler
int	CHttpClient::AddCookie(const char *pText,const char *pName)
{
unsigned	uPos,uLen,uCnt;
char	   *pTemp,*pAlloc;



	if(!pText)return -1;

	  pText+=strspn (pText," \t");
		uLen=strcspn(pText,";\n\r");
	if(!uLen)return -1;

	for(uCnt=0,uPos=uCookiePos;uCnt<HCL_COOKIES_MAX;uCnt++,uPos++)
		{
		if(uPos>=HCL_COOKIES_MAX)uPos=0;
		if(!pCookies[uPos])break;
		}


	if(uPos>=HCL_COOKIES_MAX)uPos=0;

	if(pName)
			uCnt = strlen(pName)+1;
	else	uCnt = 0;


	pTemp  = pCookies[uPos];
	pAlloc = new char[uLen+uCnt+256];


	memcpy(pAlloc+uCnt,pName,uCnt);
	memcpy(pAlloc     ,pText,uLen);

	if(uCnt)pAlloc[uCnt-1]='=';

	uLen += uCnt;
	pAlloc[uLen]=0;


	pCookies[uPos]=pAlloc;
	uCookiePos=uPos+1;
	if(uCookiePos>=HCL_COOKIES_MAX)uCookiePos=0;

	if(pTemp)delete pTemp;
	else	uCookieCount++;


return 0;
}

//*****************************************************************************
//*
//*		Post
//*
//*****************************************************************************
//	Schickt eine Nachricht zum Server
//	pFilename: Dateinamen der Datei wo die HTTP Daten gespeichert werden.
//			   Bei NULL werden die Daten im Speicher gehalten
//	pPath	 : Name der gewnschten Datei ohne Servernamen
//	pMessage : Zeiger auf die Daten die geschickt werden sollen
//	iSize	 : Lnge der Daten die geschickt werden.
//			   Bei -1 wird die Lnge mit strlen(pData) ermittelt
//	uTimeout : Gibt die Wartezeit an bis auf das letzte Byte gewartet wird.
//	pContent : Ist der Text fr den Content-Type. (maximal 80 Zeichen) 
//			   Bei NULL wird "application/x-www-form-urlencoded" benutzt, falls
//			   In den Extra-Feldern kein Content-Type definiert ist.	
//	Ergibt   : Die Lnge der Daten
//			   -1  Falsches Handle
//			   -2  Fehler beim Senden
//			   -3  Fehler beim Lesen
//			   -4  Zuwenig Speicher
//			   -5  Timeout
//			   -9  Es gab zu viele Umleitungen
//			   -10 Das Ziel der Umleitung wurde nicht gefunden
//			   -11 Die Anfrage wurde zu gro
//			   -14 Es wurde eine zugroe Datei empfangen
//			   -15 Es wurde kein HTTP-Header gefunden
//			   -16 Es wurde eine HTTP Fehlermedung empfangen
//			   -17 Es gab eine Fehler im bertragungsformat
//			   -18 Die Weiterleitung zu einem anderen Server schlug fehl
int CHttpClient::Post(const char *pFilename,const char *pPath,const char *pMessage,int iSize,unsigned uTimeout,const char *pContent)
{
char   *pBuffer;
char	cPost[sizeof(cHttpGet)+ADD_HEADER+64+256*HCL_COOKIES_MAX+4];
int		iVersion;
int		iOff;
int		iNum;
int		iLen;




	iError		= HCL_OK;
	iDataMinus  = 0;
	iDataLength	= 0;
	iDataStop	= 0;
	iDataStart	= 0;
	iDataState  = 0;
	iChunkLen	= 0;

													// Die Verbindung neu aufbauen ?
	if(iNewConnection || (hTcp && TcpIsBroken(hTcp)))
		{
		iVersion=iHttpVersion;
		Disconnect();
		Connect(cServername,iServerPort,0,iServerSsl);
		iHttpVersion=iVersion;
		iNewConnection=0;
		}

	if(!hTcp)
		{
		iError=HCL_WRONG_HANDLE;
		return HCL_WRONG_HANDLE;
		}

	iProgressType	= 1;
	iProgressPos	= 0;
	iProgressEnd	= 0;


	while(TcpGetNext(hTcp,cPost,sizeof(cPost),0)>0);


	if(iSize<0)iSize=strlen(pMessage);
													// Header erstellen
		iLen=CreateRequest("POST",pPath,cPost,sizeof(cPost)-128,(pContent)? 0x02:0x04);
	if(!iLen)
		{
		iProgressType	= 3;
		iError=HCL_REQUEST_TOO_LARGE;
		return HCL_REQUEST_TOO_LARGE;
		}


	if((iEncodeRequests&0x0F) && iSize>0)
		{
		pBuffer = (char*)alloca(iSize*3+1);
		iSize	= EncodeText(iEncodeRequests&0x0F,pMessage,iSize,pBuffer,iSize*3+1);
		}
	else{
		pBuffer = (char*)pMessage;
		}

	if(pContent)
		{
		iLen += sprintf(cPost+iLen,"Content-Type: %.80s\r\n",pContent);
		}

													// Lnge der Post-Daten
	iLen += sprintf(cPost+iLen,"Content-Length: %i\r\n\r\n",iSize);

	iProgressEnd = iLen;
	if(iSize>0)iProgressEnd+=iSize;

	if(!TcpPut(hTcp,cPost,iLen))					// Header Senden
		{
		iProgressType	= 3;
		iError=HCL_SEND_ERROR;
		return HCL_SEND_ERROR;
		}

	iProgressPos = iLen;

	if(iSize>0)										// Post-Daten senden
		{
		if(iSize<4096)
			{
			if(!TcpPut(hTcp,pBuffer,iSize))
				{
				iProgressType = 3;
				iError=HCL_SEND_ERROR;
				return HCL_SEND_ERROR;
				}
		
			iProgressPos += iSize;
			}
		else{
			for(iOff=0;iOff<iSize;iOff+=iNum)
				{
				   iNum = iSize-iOff;
				if(iNum>4096)iNum=4096;

				if(!TcpPut(hTcp,pBuffer+iOff,iNum))
					{
					iProgressType = 3;
					iError=HCL_SEND_ERROR;
					return HCL_SEND_ERROR;
					}
			
				iProgressPos += iNum;
				}
			}
		}

	if(Read(pFilename,uTimeout,0)<0)				// Antwort einlesen
		{
		iProgressType = 3;
		pData[iDataLength]=0;
		return iError;
		}

	iProgressType = 3;
	pData[iDataLength]=0;



return (iError)? iError:iDataLength-iDataStart;
}

//*****************************************************************************
//*
//*		CreateRequest
//*
//*****************************************************************************
//	Erzeugt einen HTTP Request
//	pType	: Art des Requestes ("GET","POST", ... )
//	pPath	: Pfad des oder URL
//	pData	: Zielspeicher fr den Request
//	iMax	: Gre des Zielspeichers
//	iFlags	: Diverse Flags
//				Bit0 = Den Block mit CR/LF abschlieen
//				Bit1 = Keinen "Content-Type" anhngen
//				Bit2 = Standart "Content-Type" anhngen wenn nicht definiert
//	Ergibt die Lnge des Requestes bzw 0 bei eine Fehler
int	CHttpClient::CreateRequest(const char *pType,const char *pPath,char *pData,int iMax,int iFlags)
{
const char *pName;	
int			iHost;
int			iType;
int			iPos;
int			iLen;
int			iNum;
int			iCnt;
int			i;



	   iLen=strlen(pType);
	if(iLen<=0)return 0;
	if(iLen+1>iMax)return 0;

	memcpy(pData,pType,iLen);
	pData[iLen]=' ';
	iLen++;


	   iNum=strlen(pPath);
	if(iNum<=0){iNum=1;pPath="/";}
	if(iLen+iNum>iMax)return 0;

	iLen+=EncodeText((iEncodeRequests>>4)&0x0F,pPath,iNum,pData+iLen,iMax-iLen);

	memcpy(pData+iLen,cHttpGet ,sizeof(cHttpGet));
	pData[iLen+8]  += iHttpVersion;
	iLen		   += sizeof(cHttpGet)-1;


	iHost	= 1;
	iType	= 1;

	for(iPos=0;iPos<iFieldCount;iPos++)		// Extra-Felder anhngen
		{
		pName = aFieldName[iPos];
		iNum  = strlen(pName);

		if(iLen+iNum+aFieldSize[iPos]+4>iMax)
			{
			return 0;						// Header wird zu lang
			}

		if(iNum==4 && (*pName&~0x20)=='H' && !stricmp(pName,"Host"))
			{
			iHost = 0;
			}

		if(iNum==12 && (*pName&~0x20)=='C' && !stricmp(pName,"Content-Type"))
			{
			iType = 0;
			if(iFlags&0x02)continue;
			}
		
		memcpy(pData+iLen,aFieldName[iPos],iNum);
		iLen+=iNum;
		
		pData[iLen] = ':';	iLen++;
		pData[iLen] = ' ';	iLen++;

		memcpy(pData+iLen,aFieldText[iPos],aFieldSize[iPos]);
		iLen+=aFieldSize[iPos];

		pData[iLen] = '\r';	iLen++;
		pData[iLen] = '\n';	iLen++;
		}


	if(iHost)								// Servernamen anhngen
		{
		iNum = strlen(cServername);

		if(iLen+iNum+8>iMax)
			{
			return 0;						// Header wird zu lang
			}

		memcpy(pData+iLen,"Host: ",6);			
		iLen+=6;
		memcpy(pData+iLen,cServername,iNum);
		iLen+=iNum;
		memcpy(pData+iLen,"\r\n",2);
		iLen+=2;
		}


	if(iType && (iFlags&0x04))				// Standart "Content-Type" anhngen wenn nicht definiert
		{
		if(iLen+49>iMax)
			{
			return 0;						// Header wird zu lang
			}

		memcpy(pData+iLen,"Content-Type: application/x-www-form-urlencoded\r\n",51);			
		iLen+=49;
		}


											

											// Cookies anhngen
	for(i=uCookiePos-1,iCnt=0,iPos=0;iPos<HCL_COOKIES_MAX;i--,iPos++)
		{
		if(i<0)i=HCL_COOKIES_MAX-1;

		if(!pCookies[i])continue;

		if(iCnt==0)
			{
			memcpy(pData+iLen,"Cookie: ",8);
			iLen+=8;
			}
		else{
			memcpy(pData+iLen,"; ",2);
			iLen+=2;
			}

		iNum=strlen(pCookies[i]);
		if(iLen+iNum>iMax)return 0;
		memcpy(pData+iLen,pCookies[i],iNum);
		iLen+=iNum;
		iCnt+=1;
		}

	if(iCnt>0)
		{
		memcpy(pData+iLen,"\r\n",2);
		iLen+=2;
		}


	if(iFlags&1)							// Den Block mit CR/LF abschlieen ?
		{
		if(iLen+2>iMax)return 0;
		memcpy(pData+iLen,"\r\n",2);
		iLen+=2;
		}



return iLen;
}


//*****************************************************************************
//*
//*		DeleteAllCookies
//*
//*****************************************************************************
//	Lscht alle Cookies
//	Ergibt die Anzahl der gelschten Cookies
int CHttpClient::DeleteAllCookies()
{
int		i,iCount=0;


	for(i=0;i<HCL_COOKIES_MAX;i++)
		{
		if(pCookies[i])continue;
		delete pCookies[i];
		pCookies[i]=0;
		iCount++;
		}

	uCookieCount=0;

return iCount;
}


//*****************************************************************************
//*
//*		SaveHtml
//*
//*****************************************************************************
//	Speichert die HTML Daten in einer Datei.
//	pPath	: Pfad der Html-Datei
//	Ergibt 0 wenn ok
int CHttpClient::SaveHtml(const char *pPath)
{
FILE	*pFile;
int		 iOk;


	if(!iDataStart || iDataLength<=iDataStart)return HCL_NO_DATA;

		pFile=fopen(pPath,"wb");
	if(!pFile)return HCL_FILE_ERROR;

	iOk=fwrite(pData+iDataStart,iDataLength-iDataStart,1,pFile);
	fclose(pFile);

return (iOk)? HCL_OK:HCL_FILE_ERROR;
}



//*****************************************************************************
//*
//*		SaveAll
//*
//*****************************************************************************
//	Speichert die HTML Daten mit dem Request-Header in einer Datei.
//	pPath	: Pfad der Html-Datei
//	Ergibt 0 wenn ok
int CHttpClient::SaveAll(const char *pPath)
{
FILE	*pFile;
int		 iOk;


	if(!iDataStart || iDataLength<=0)return HCL_NO_DATA;

		pFile=fopen(pPath,"wb");
	if(!pFile)return HCL_FILE_ERROR;

	iOk=fwrite(pData,iDataLength,1,pFile);
	fclose(pFile);

return (iOk)? HCL_OK:HCL_FILE_ERROR;
}


//*****************************************************************************
//*
//*		GetNextChunk
//*
//*****************************************************************************
//	List einen Chunk ein im Format "Lnge\n....."
//	hTcp		: Ist das TCP Handle der HTTP Verbindung
//	pBuffer		: Puffer in dem die Daten gespeichert werden
//	iSize		: Gre des Puffers in Byte
//	uTimeout	: Timeout in Millisekunden
//	Ergibt  -2 wenn alle Daten empfangen wurden
//			-1 bei Fehlern
//			 0 bei einem Timeout
//			bzw. die Lnge der empfangenen Daten
int	CHttpClient::GetNextChunk(TcpHandle hTcp,char *pBuffer,int iSize,unsigned  uTimeout)
{
char	cLength[8];
int		iNum;
int		iPos;
int		iAdd;


	if(iChunkLen>0)							// Hat dieser Chunk noch Daten
		{
		if(iSize>iChunkLen)iSize=iChunkLen;

		   iNum =TcpGetNext(hTcp,pBuffer,iSize,uTimeout);
		if(iNum<0)return -1;

		iChunkLen-=iNum;

		return iNum;
		}


	   iNum = TcpGetNext(hTcp,cLength,sizeof(cLength)-1,uTimeout);
	if(iNum <0)return -1;
	if(iNum==0)return  0;

	cLength[iNum]=0;
	iPos=0;

	iChunkLen=strtol(cLength+iPos,NULL,16);

	if(cLength[iPos]=='\r')iPos++;
	if(cLength[iPos]=='\n')iPos++;

	for(;iPos<iNum;iPos++)
		{
		if(cLength[iPos]=='\n')break;
		}

	if(cLength[iPos]=='\n')iPos++;

	if(iNum>iPos)							// Sind noch Bytes hinter der Lngenangabe
		{
		   iAdd = iNum-iPos;
		if(iAdd>iSize)return -1;

		memcpy(pBuffer,cLength+iPos,iAdd);
		pBuffer   += iAdd;
		iSize     -= iAdd;
		iChunkLen -= iAdd;

		if(iSize<=0)return iNum;
		}


	if(iSize>iChunkLen)iSize=iChunkLen;

	   iNum =TcpGetNext(hTcp,pBuffer,iSize,uTimeout);
	if(iNum <0)return -1;
	if(iNum==0)

	iChunkLen-=iNum;



return iNum+iAdd;
}


//*****************************************************************************
//*
//*		CheckBuffer
//*
//*****************************************************************************
//	Prft ob der gro genug ist
//	Ergibt 1 wenn ok oder 0 bei einem Fehler
int CHttpClient::CheckBuffer()
{
char   *pNew;
int		iMax;


	if(iDataLength+4096>iDataMax)		// Den Speicher vergrern
		{
		iMax = iDataMax+iDataMax/2;

		if(iMax>MAX_RESV_BYTE)
			{
			iError=HCL_RESEIVE_TOO_LARGE;
			return 0;
			}

			pNew  = new char[iMax+16];
		if(!pNew)
			{
			iError=HCL_OUT_OF_MEMORY;
			return 0;
			}

		memcpy(pNew,pData,iDataLength);
		iDataMax = iMax;
		delete pData;
		pData =pNew;
		}


return 1;
}

//*****************************************************************************
//*
//*		Read
//*
//*****************************************************************************
//	Liest eine Datei von einer HTTP Verbindung ein, an die Datei wird ein
//	Null-Byte angehngt.
//	pFilename: Dateinamen der Datei wo die HTTP Daten gespeichert werden.
//			   Bei NULL werden die Daten im Speicher gehalten. Ist ein
//			   Dateinamen angegeben ist die Speichergre auf 1MB begrenzt.
//	uTimeout : Gibt die Wartezeit an bis auf das letzte Byte gewartet wird.
//	uFlags	 : Diverse Flags fr die Steuerung
//				Bit0	= Auch groe Daten nicht im Speicher halten (fr groe Dateien)
//	Ergibt   : Die Lnge der Daten
//			   -1  Falsches Handle
//			   -2  Fehler beim Senden
//			   -3  Fehler beim Lesen
//			   -4  Zuwenig Speicher
//			   -5  Timeout
//			   -9  Es gab zu viele Umleitungen
//			   -10 Das Ziel der Umleitung wurde nicht gefunden
//			   -11 Die Anfrage wurde zu gro
//			   -14 Es wurde eine zugroe Datei empfangen
//			   -15 Es wurde kein HTTP-Header gefunden
//			   -16 Es wurde eine HTTP Fehlermedung empfangen
//			   -17 Es gab eine Fehler im bertragungsformat
//			   -18 Die Weiterleitung zu einem anderen Server schlug fehl
int CHttpClient::Read(const char *pFilename,unsigned uTimeout,unsigned uFlags)
{
FILE   *pFile;
char   *pValue;
int		iChunked;
int		iLength;
int		iCheck;
int		iWrite;
int		iFlag;
int		iLen;
int		iNum;
int		iAdd;
int		iPos;
int		iMax;
int		iOff;



	if(!iDataMax)									// Einen neuen Datenspeicher allokieren
		{
		iDataMax = 8192;

			pData=new char[iDataMax+16];
		if(!pData)
			{
			iDataMax=0;
			iError=HCL_OUT_OF_MEMORY;
			return HCL_OUT_OF_MEMORY;
			}
		}


	iFlag			= 0;
	iCheck			= 0;
	iLength			= 0;
	iChunked		= 0;
	iProgressType	= 2;
	iProgressPos	= iProgressEnd;

	#if HCL_USE_NTLM
	iNtlmPos = 0;
	#endif


	for(;;) 										// Den Header einlesen
		{
		   iMax = iDataMax-iLength;
		if(iMax>MAX_HEADER)iMax=MAX_HEADER;


		   iNum = TcpGetNext(hTcp,pData+iLength,iMax,uTimeout);
		if(iNum<0)
			{
			iError=HCL_READ_ERROR;
			return HCL_READ_ERROR;
			}

		if(iNum==0)									// Gab es eine Timeout
			{
			iError=HCL_TIMEOUT;
			return HCL_TIMEOUT;
			}

		iLength		+= iNum;

		for(;iCheck<iLength;iCheck++)				// Header-Ende suchen
			{
			if(pData[iCheck  ]!='\r')continue;
			if(pData[iCheck+1]!='\n')continue;
			if(pData[iCheck+2]!='\r')continue;
			if(pData[iCheck+3]!='\n')continue;

			break;
			}

		if(iCheck<iLength)break;					// Wurde das Headerende gefunden

		if(iCheck>=MAX_HEADER)
			{
			iError=HCL_NO_HEADER;
			return HCL_NO_HEADER;
			}

		   iCheck-=3;
		if(iCheck<0)iCheck=0;
		}


	iProgressPos += iLength;


//******************** Header verarbeiten *************************************


	pData[iLength]=0;
	iDataLength=iLength;

	if(!memcmp(pData,"HTTP/",5))					// HTTP Version abfragen
		{
		iLen=strcspn(pData+5," \t\n\r");
		iDataState=atoi(pData+iLen+5);
		if(!memcmp(pData+5,"1.1",3))iHttpVersion=1;
		}

	iDataStart  = iCheck+4;

	for(iPos=0;iPos<iDataStart;iPos+=iAdd)			// Header Zeilen weise auswerten
		{
		for(iAdd=0;;iAdd++)
			{
			if(pData[iPos+iAdd]=='\n')break;
			}

		for(;;iAdd++)
			{
			if(pData[iPos+iAdd]!='\n')
			if(pData[iPos+iAdd]!='\r')break;
			}

		if(memicmp(pData+iPos,"Set-Cookie:",11)==0)
			{
			pValue=pData+iPos+11;
			AddCookie(pValue);

			for(iOff=iPos+11;;iOff++)
				{
				if(pData[iOff]=='\r')break;
				if(pData[iOff]=='\n')break;
				if(pData[iOff]!=';' )continue;

				iOff++;
				iOff+=strspn(pData+iOff," \t");

				if(pData[iOff]=='\r')break;
				if(pData[iOff]=='\n')break;

				if(!memicmp(pData+iOff,"path="  ,5))continue;
				if(!memicmp(pData+iOff,"domain=",7))continue;

				AddCookie(pData+iOff);
				}

			continue;
			}

		if(memicmp(pData+iPos,"Connection:",11)==0)
			{
			pValue=pData+iPos+11;
			pValue+=strspn(pValue," \t");
			if(!memicmp(pValue,"close",5))
				{
				iNewConnection=1;
				}

			continue;
			}

		if(memicmp(pData+iPos,"Transfer-Encoding:",18)==0)
			{
			pValue=pData+iPos+18;
			pValue+=strspn(pValue," \t");
			if(!memicmp(pValue,"chunked",7))
				{
				iChunked=1;
				}

			continue;
			}

		if(memicmp(pData+iPos,"Content-Length:",15)==0)
			{
			pValue = pData+iPos+15;
			iFlag  = 1;			


			   iLen = atoi(pValue);
			if(iLen)
				{
				iDataStop	  = iDataStart+iLen;
				iProgressEnd += iDataStart+iLen;
				}

			continue;
			}

		#if HCL_USE_NTLM

		if(memicmp(pData+iPos,"WWW-Authenticate:",17)==0)
			{
			pValue = pData+iPos+17;

			while(*pValue==' ')pValue++;

			if(!memicmp(pValue,"NTLM",4))
				{
				iNtlmPos = (int)(pValue-pData+4);
				while(pData[iNtlmPos]==' ')iNtlmPos++;
				}

			continue;
			}

		#endif
		}

	if(iDataState>=300 && iDataState<=399)			// Umleitung
		{
		return Reconnect(pFilename,uTimeout);
		}


	if(iDataState!=200)								// Wurde ein Fehler gemeldet
		{
		iError = HCL_ERROR_CODE;

		if(!iDataStop)
			{
			return iError;
			}
		}


//*****************************************************************************


	if(pFilename)									// Alles in einer Datei speichern
		{
			pFile = fopen(pFilename,"wb");
		if(!pFile)
			{
			iError = HCL_FILE_ERROR;
			return iError;
			}
		}
	else{
		uFlags &= ~1;								// Groe Daten knnen nur in Dateien gespeichert werden
		pFile   = NULL;
		}


	if(!iChunked && !iFlag)							// Lesen bis zum Verbindungsabbruch
		{
		if(pFile)
			{
			iNum    = iLength-iDataStart;
			fwrite(pData+iDataStart,iNum,1,pFile);
			}


		for(;;)
			{
			iMax = iDataMax-iDataLength;
			iNum = TcpGetNext(hTcp,pData+iLength,iMax,uTimeout);

			if(iNum<0)
				{
				if(iDataLength<=iDataStart)
					{
					iError=HCL_READ_ERROR;
					}

				break;
				}

			if(iNum==0)								// Gab es eine Timeout
				{
				if(iDataLength<=iDataStart)
					{
					iError=HCL_TIMEOUT;
					}

				break;
				}

			if(pFile)
				{
				fwrite(pData+iLength,iNum,1,pFile);
				}

			iLength     += iNum;
			iDataLength += iNum;

													// Puffer zurcksetzen bei groen Dateien
			if(iDataLength>MAX_RESV_BYTE/2 && (uFlags&1))
				{
				iDataMinus += iDataLength-MAX_RESV_BYTE/2;
				iLength		= MAX_RESV_BYTE/2;
				iDataLength	= MAX_RESV_BYTE/2;
				}

			if(!CheckBuffer())break;
			}


		if(pFile)fclose(pFile);

		iNewConnection=1;

		return iError;
		}


	if(iFlag && !iChunked)							// Lesen einer fixen Lnge
		{
		if(pFile)
			{
			iNum    = iLength-iDataStart;
			fwrite(pData+iDataStart,iNum,1,pFile);
			}

		while(iDataLength+iDataMinus<iDataStop)
			{
			iMax = iDataMax-iLength;
			iNum = TcpGetNext(hTcp,pData+iLength,iMax,uTimeout);

			if(iNum<0)
				{
				iError=HCL_READ_ERROR;
				break;
				}

			if(iNum==0)								// Gab es eine Timeout
				{
				iError=HCL_TIMEOUT;
				break;
				}

			if(pFile)
				{
				fwrite(pData+iLength,iNum,1,pFile);
				}

			iLength		 += iNum;
			iDataLength	 += iNum;
			iProgressPos += iNum;
													// Puffer zurcksetzen bei groen Dateien
			if(iDataLength>MAX_RESV_BYTE/2 && (uFlags&1))
				{
				iDataMinus += iDataLength-MAX_RESV_BYTE/2;
				iLength		= MAX_RESV_BYTE/2;
				iDataLength	= MAX_RESV_BYTE/2;
				}

			if(!CheckBuffer())break;
			}


		if(pFile)fclose(pFile);

		return iError;
		}

//******************** Chunked Einlesemodus ***********************************


	pData[iLength]	= 0;
	iChunkLen		= 0;
	iPos			= iDataStart;
	iWrite			= iDataStart;
	iNum			= iPos;

													// Gelesene Bytes auswerten
	for(;;)
		{
		iNum = iLength-iPos;

		if(iNum>=iChunkLen)							// Daten+Chunk vollstndig geladen
			{
			iPos	   += iChunkLen;
			iChunkLen	= 0;

			for(iNum=iPos;iNum<iLength;iNum++)		// Suche Ende vom Chunk ("12345\r\n")
				{
				if(pData[iNum]=='\n')break;
				}

			if(pData[iNum]!='\n')					// Ist Chunk vollstndig
				{
				iMax = iDataMax-iLength;
				iNum = TcpGetNext(hTcp,pData+iLength,iMax,uTimeout);

				if(iNum<0)
					{
					if(pFile)fclose(pFile);
					iError=HCL_READ_ERROR;
					return HCL_READ_ERROR;
					}

				if(iNum==0)							// Gab es einen Timeout
					{
					if(pFile)fclose(pFile);
					iError=HCL_TIMEOUT;
					return HCL_TIMEOUT;
					}

				iLength		 += iNum;
				iDataLength	 += iNum;
				iProgressPos += iNum;

				continue;
				}

			iNum++;
			iChunkLen  = strtol(pData+iPos,NULL,16)+2;
			iAdd       = iLength-iNum;

			if(iPos-2>=iDataStart)					// "\r\n" von letzten Daten lschen
				{
				iPos -= 2;
				}

			memmove(pData+iPos,pData+iNum,iAdd);	// Chunk entfernen

			iLength		-= iNum-iPos;
			iDataLength	-= iNum-iPos;
			pData[iLength]	= 0;


			if(iChunkLen>2)							// Ende der HTML-Datei erreicht
				{
				continue;
				}

			   iNum = iLength-iPos;
			if(iNum<=4)								// "\r\n\r\n" vom Ende entfernen
				{
				iLength		-= iNum;
				iDataLength	-= iNum;
				}

			break;
			}

													// Puffer zurcksetzen bei groen Dateien
		if(iDataLength>MAX_RESV_BYTE/2 && (uFlags&1))
			{
			if(pFile && iWrite<iDataLength)
				{
				fwrite(pData+iWrite,iDataLength-iWrite,1,pFile);
				}

			iDataMinus += iDataLength-MAX_RESV_BYTE/2;
			iLength		= MAX_RESV_BYTE/2;
			iDataLength	= MAX_RESV_BYTE/2;
			iWrite		= MAX_RESV_BYTE/2;
			}


		if(!CheckBuffer())break;


		iMax = iDataMax-iLength;					// Nchsten Daten einlesen
		iNum = TcpGetNext(hTcp,pData+iLength,iMax,uTimeout);

		if(iNum<0)
			{
			if(pFile)fclose(pFile);
			iError=HCL_READ_ERROR;
			return HCL_READ_ERROR;
			}

		if(iNum==0)									// Gab es einen Timeout
			{
			if(pFile)fclose(pFile);
			iError=HCL_TIMEOUT;
			return HCL_TIMEOUT;
			}

		iLength		 += iNum;
		iDataLength	 += iNum;
		iProgressPos += iNum;

		pData[iLength]	= 0;
		}


	if(pFile)										// Daten in Datei speichern
		{
		if(iWrite<iDataLength)
			{
			fwrite(pData+iWrite,iDataLength-iWrite,1,pFile);
			}

		fclose(pFile);

		if(iDataLength>MAX_RESV_FILE)iDataLength=MAX_RESV_FILE;
		}


return iError;
}



//*****************************************************************************
//*
//*		EncodeText
//*
//*****************************************************************************
//	Codiert einen Text
//	iMode		: Ist der Modus in dem cotiert wird
//					0 = keine Codierung
//					1 = Zeichen kleiner gleich ' ' und '%' werden codiert
//					2 = Nur Texzeichen uncodiert lassen
//					3 = Zeichen kleiner '?' (0x39) ohne Zahlen werden codiert
//					4 = Zeichen kleiner '?' ohne Zahlen+'='+'&' werden codiert
//	pText		: Ist der Text der codiert werden soll
//	iSize		: Ist die Lnge des Textes, bei -1 wird strlen() benutzt
//	pBuffer		: Hier wird der Text gespeichert
//	iMax		: Ist die Gre des Puffers
//	Ergibt die Anzahl der gespeicherten Bytes.
int CHttpClient::EncodeText(int iMode,const char *pText,int iSize,char *pBuffer,int iMax)
{
unsigned	uMask;
char		cChar;
int			iLen;
int			i;


	if(iSize<0)iSize=strlen(pText);
	if(iMax <0)iMax =0;

	switch(iMode)
		{
	default:									// Modus 0
	case 0:
			if(iSize>iMax)iSize=iMax;
			else   pBuffer[iSize]=0;
			memcpy(pBuffer,pText,iSize);
			return iSize;

	case 1:										// Modus 1

			uMask = 0x00000021;
			break;

	case 2:										// Modus 2

			iLen = 0;

			for(i=0;i<iSize;i++)
				{
				if(iLen>=iMax)break;

				cChar=pText[i];

				if(cChar==' ')
					{
					pBuffer[iLen]='+';
					iLen++;
					continue;
					}

				if(cChar>='a' && cChar<='z')
					{
					pBuffer[iLen]=cChar;
					iLen++;
					continue;
					}

				if(cChar>='A' && cChar<='Z')
					{
					pBuffer[iLen]=cChar;
					iLen++;
					continue;
					}

				if(cChar>='0' && cChar<='9')
					{
					pBuffer[iLen]=cChar;
					iLen++;
					continue;
					}

				pBuffer[iLen  ]='%';
				pBuffer[iLen+1]="0123456789ABCDEF"[(cChar>>4)&0x0F];
				pBuffer[iLen+2]="0123456789ABCDEF"[ cChar    &0x0F];

				iLen += 3;
				}

			return iLen;

	case 3:										// Modus 3

			uMask = 0xFC00FFFF;
			break;

	case 4:										// Modus 3

			uMask = 0xDC00FFBF;
			break;
		}


	iLen = 0;

	for(i=0;i<iSize;i++)
		{
		if(iLen>=iMax)break;

		cChar = pText[i];

		if(cChar>=0x40 || !(cChar>=0x20 && ((uMask>>(cChar&0x1F))&1)))
			{
			pBuffer[iLen]=cChar;
			iLen++;
			continue;
			}

		pBuffer[iLen  ]='%';
		pBuffer[iLen+1]="0123456789ABCDEF"[(cChar>>4)&0x0F];
		pBuffer[iLen+2]="0123456789ABCDEF"[ cChar    &0x0F];

		iLen += 3;
		}


return iLen;
}


//*****************************************************************************
//*
//*		LoginData
//*
//*****************************************************************************
//	Einstellen der Anmeldedaten	(bei NULL werden die Namen nicht verndert)
//	pUser		: Ist der Username
//	pPasswd		: Ist das Passwort
//	pDomain		: Ist die Domne
//	pHost		: Ist der Hostname ("*" um den lokalen Host zu benutzen)
//	Ergibt 0 wenn ok.
#if HCL_USE_NTLM
int CHttpClient::LoginData(const char *pUser,const char *pPasswd,const char *pDomain,const char *pHost)
{
char	cTemp[64];


	if(pUser)
		{
		if(pUser[0]=='*' && !pUser[1])			// Aktuellen User benutzen			
			{
			#ifdef WIN32
			DWORD  dwSize=sizeof(cTemp);
			GetUserNameA(cTemp,&dwSize);
			pUser=cTemp;
			#endif 	

			#ifdef linux	
			struct passwd *pPass;
 			pPass = getpwuid(geteuid());
			if(pPass)pUser=pPass->pw_name;
			#endif 	
			}

		strncpy(cUserName,pUser  ,sizeof(cUserName)-1);
		cUserName[sizeof(cUserName)-1] = 0;
		}


	if(pPasswd)									// Passwort setzen
		{
		strncpy(cPassWord,pPasswd,sizeof(cPassWord)-1);
		cPassWord[sizeof(cPassWord)-1] = 0;
		}


	if(pDomain)									// Domne setzen	
		{
		strncpy(cDomain  ,pDomain,sizeof(cDomain  )-1);
		cDomain[sizeof(cDomain  )-1] = 0;
		}


	if(pHost)
		{
		if(pHost[0]=='*' && !pHost[1])			// Aktuellen Host benutzen
			{
				pHost=IpGetHostname();
			if(!pHost)pHost="";
			}
	
		strncpy(cHostName,pHost  ,sizeof(cHostName)-1);
		cHostName[sizeof(cHostName)-1] = 0;
		}


return 0;
}
#endif

//*****************************************************************************
//*
//*		SetField
//*
//*****************************************************************************
//	Ein extrafel im HTTP-Header einfgen nder oder lschen
//	pName		: Ist der Name des Feldes
//	pText		: Ist der Inhalt vom Feld ohne CR/LF am Ende.
//				  NULL lscht das Feld.
//	iAlloc		: Sollen die Daten ein einem Speicher allokiert werden
//	Ergibt HCL_OK wenn ok oder HCL_TOO_MUCH_FIELDS wenn kein Platz mehr ist,
//	oder HCL_LOCATION_NOT_FOUND wenn ein Feld nicht gefunden wurde.
int CHttpClient::SetField(const char *pName,const char *pText,int iAlloc)
{
int		iPos;
int		iLen;


	for(iPos=0;;iPos++)							// Suche den Slot fr das Feld	
		{
		if(iPos>=iFieldCount)
			{
			if(!pText)
				{
				return HCL_LOCATION_NOT_FOUND;
				}

			if(iFieldCount>=HCL_FIELD_COUNT)
				{
				return HCL_TOO_MUCH_FIELDS;
				}

			aFieldName[iPos] = pName;	
			aFieldFree[iPos] = 0;	
			iFieldCount++;	
			}

		if(!stricmp(pName,aFieldName[iPos]))
			{
			break;
			}
		}

	
	if(!pText)									// Ein Feld lschen
		{
		if(iPos==0)
			{									// "Accept" nicht lschen 
			return HCL_LOCATION_NOT_FOUND;		
			}

		if(aFieldFree[iPos])
			{
			delete aFieldText[iPos];
			}

		iFieldCount--;

		aFieldName[iPos] = aFieldName[iFieldCount];	
		aFieldText[iPos] = aFieldText[iFieldCount];	
		aFieldFree[iPos] = aFieldFree[iFieldCount];	
		aFieldSize[iPos] = aFieldSize[iFieldCount];	

		return HCL_OK;
		}
	

	iLen = strlen(pText);

	aFieldSize[iPos]	 = iLen;

	if(iAlloc)									// Speicher fr den Inhalt allokieren
		{
		aFieldFree[iPos] = 1;
		aFieldText[iPos] = new char[iLen+1];
		memcpy(aFieldText[iPos],pText,iLen+1);
		}
	else{
		aFieldFree[iPos] = 0;
		aFieldText[iPos] = (char*)pText;
		}
	


return HCL_OK;
}
