//*****************************************************************************
//*
//*
//*		HttpClient.h
//*
//*
//*****************************************************************************
#ifndef 	__HTTPCLIENT_H__
#define 	__HTTPCLIENT_H__

#include	"System/SocketUtility.h"

#ifndef 	HCL_COOKIES_MAX
#define 	HCL_COOKIES_MAX			16	// Maximale Anzahl der Cookies
#endif

#ifndef 	HCL_USE_NTLM
#define 	HCL_USE_NTLM			 0	// Anmeldung ber NTLM
#endif

#ifndef 	HCL_FIELD_COUNT
#define 	HCL_FIELD_COUNT			 12	// Anzahl der mglichen Extrafelder
#endif

#define 	HCL_OK					 0	// Alles ok
#define 	HCL_WRONG_HANDLE		-1	// Falsches Handle
#define 	HCL_SEND_ERROR			-2	// Fehler beim Senden
#define 	HCL_READ_ERROR		    -3	// Fehler beim Lesen
#define 	HCL_OUT_OF_MEMORY	    -4	// Zuwenig Speicher
#define 	HCL_TIMEOUT				-5	// Timeout
#define 	HCL_NOCONNECTION		-6	// Es konnte keine TCP Verbindung aufgebaut werden
#define 	HCL_WRONG_SERVERNAME	-7	// Falscher Servername
#define 	HCL_SERVER_NOT_FOUND	-8	// Der Servername konnte nicht aufgelst werden
#define 	HCL_RECONNECT_LOOP		-9	// Es gab zu viele Umleitungen
#define 	HCL_LOCATION_NOT_FOUND	-10	// Das Ziel der Umleitung wurde nicht gefunden
#define 	HCL_REQUEST_TOO_LARGE	-11	// Die Anfrage wurde zu gro
#define 	HCL_FILE_ERROR			-12	// Die Daten konnten nicht geschrieben werden
#define 	HCL_NO_DATA				-13	// Es sind keine Daten vorhanden
#define 	HCL_RESEIVE_TOO_LARGE	-14	// Es wurde eine zugroe Datei empfangen
#define 	HCL_NO_HEADER			-15	// Es wurde kein HTTP-Header gefunden
#define 	HCL_ERROR_CODE			-16	// Es wurde eine HTTP Fehlermedung empfangen
#define 	HCL_WRONG_FORMAT		-17	// Es gab eine Fehler im bertragungsformat
#define 	HCL_WRONG_SERVER		-18	// Die Weiterleitung zu einem anderen Server schlug fehl
#define 	HCL_HEADER_TOO_BIG		-19	// Der Request-Header ist zu gro
#define 	HCL_TOO_MUCH_FIELDS		-20	// Zu viele Extra-Felder

#define 	v	volatile


//*****************************************************************************
//*
//*		CHttpClient
//*
//*****************************************************************************
class CHttpClient
{
public:

    CHttpClient();
   ~CHttpClient();

	int			Shutdown();
	int			Disconnect();
	int			Connect(const char *pServerName,int iPort=0,unsigned uTimeout=0,int iSsl=0);
	int			Get (const char *pFilename ,const char *pPath,unsigned uTimeout=5000,unsigned uFlags=0);
	int			Get (                       const char *pPath,unsigned uTimeout=5000,unsigned uFlags=0){return Get(NULL,pPath,uTimeout,uFlags);}
	int			Post(const char *pFilename ,const char *pPath,const char *pMessage,int iSize=-1,unsigned uTimeout=5000,const char *pContent=0);
	int			Post(                       const char *pPath,const char *pMessage,int iSize=-1,unsigned uTimeout=5000,const char *pContent=0){return Post(NULL,pPath,pMessage,iSize,uTimeout,pContent);}
	int			SetField (const char *pName,const char *pText=0,int iAlloc=1);
	int			AddCookie(const char *pText,const char *pName=0);
	int			Progress(int *pSize=0,int *pInfo=0);
	void 		ProgressReset();
	int			CookieCount();
	int			DeleteAllCookies();
	int			SaveHtml(const char *pPath);
	int			SaveAll(const char *pPath);
	void 		SetVersion(int iVersion);
	void 		ForceReconnect();

	#if			HCL_USE_NTLM
	int			LoginData(const char *pUser,const char *pPasswd,const char *pDomain="",const char *pHost="*");
	#endif


	const char *GetCookie(unsigned uNum);
	const char *Data();							// Zeiger auf Daten ohne Header
	const char *FullData();						// Zeiger auf Daten mit Header
	int			Start();						// Gre des Headers
	int			Length();						// Lnge mit Header
	int			DataLength();					// Lnge ohne Header
	int			IsInMem();						// Passt die gesammte Datei in den Speicher
	int			State();						// Ergibt den letzten HTTP Rckgabewert
	void 		Encode(int iEncode);			// Einstellen ob Requests-Messages codiert werden sollen
	static int	EncodeText(int iMode,const char *pText,int iSize,char *pPuffer,int iMax);

protected:


	int			CheckBuffer();
	int			Read		 (const char *pFilename,unsigned uTimeout,unsigned uFlags);
	int			Reconnect	 (const char *pFilename,unsigned uTimeout=5000);
	int			CreateRequest(const char *pType,const char *pPath,char *pData,int iMax,int iFlags=0x01);
	int			GetNextChunk (TcpHandle hTcp,char *pBuffer,int iSize,unsigned  uTimeout);

	#if			HCL_USE_NTLM
	int			HandleNtLm   (char *pRequest,int iLen,int iMax,unsigned &uFlags);
	#endif 	

	TcpHandle	hTcp;							// TCP-Handle fr Serververbindung
	TcpCancel	sCancel;						// Abbruch Struktur fr TcpOpen
	int			iError;							// Aktuelle Fehlernummer
	char       *pData;							// Zeiger auf Datenspeicher
	int			iToOpen;						// Wird gerade eine Verbindung aufgebaut
	int			iChunkLen;						// Restliche Lnge des des aktuellen Chunks
	int			iDataMax;						// Gre des Datenspeichers
	int			iDataStart;						// Begin der HTTP-Daten im Datenspeicher
	int			iDataStop;						// Vom Server angegebene Lnge der Daten
	int			iDataMinus;						// Sind die Bytes die nicht in den Buffer passen
	int			iDataLength;					// Empfangene Lnge der Daten
	int			iDataState;						// Empfangener Status
	int			iHttpVersion;					// Nummer der Http-Version
	int			iConnTimeout;					// Timeout fr's wieder Verbinden
	int			iReconnectMax;					// Maximale Umleitungs Ebene
	int			iReconnectLevel;				// Aktuelle Umleitungs Ebene
	int			iNewConnection;					// Beim nchstenmal neu verbinden
	int			iEncodeRequests;				// Sollen Requests-Messages codiert werden (Bit0..3=Post Bit4..7=Pfade)
	int	v		iProgressType;					// 0=Nichts 1=Write 2=Read 3=Done 4=Response
	int	v		iProgressPos;					// Aktuelle Verlaufsposition
	int	v		iProgressEnd;					// Ende der Verlaufsposition
	unsigned	uCookieCount;					// Anzahl der einfgen Cookies
	unsigned	uCookiePos;						// Nchste einfge Position fr Cookies

	char	   *pCookies[HCL_COOKIES_MAX];		// Zeiger auf Cookiedaten
	char		cServername[256];				// Name des Servers
	int			iServerPort;					// Ist das benutzte TCP-Port
	int			iServerSsl;						// Ist der SSL Modus

	const char *aFieldName[HCL_FIELD_COUNT];	// Sind die Namen der Extra-Felder
	char	   *aFieldText[HCL_FIELD_COUNT];	// Ist der Inhalt der Extra-Felder (ohne CR/LF)
	char		aFieldFree[HCL_FIELD_COUNT];	// Ist die Gre der Inhalte in Zeichen
	int			aFieldSize[HCL_FIELD_COUNT];	// Den Speicher der Inhalte freigeben
	int			iFieldCount;					// Anzahl der Extra-Felder

	#if			HCL_USE_NTLM
	char		cDomain  [64];					// Domne   fr Login
	char		cHostName[64];					// Hostname fr Login
	char		cUserName[64];					// Username fr Login
	char		cPassWord[64];					// Password fr Login
	int 		iNtlmPos;						// Wo Wurde NTLM gefunden (0=garnicht)
	#endif

};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	const char *CHttpClient::Data       (){return (iDataStart )? pData+iDataStart:0;}
inline	const char *CHttpClient::FullData   (){return (iDataLength)? pData:0;}
inline	int			CHttpClient::IsInMem    (){return !iDataMinus;}
inline	int			CHttpClient::Start      (){return  iDataStart;}
inline	int			CHttpClient::Length     (){return  iDataLength;}
inline	int			CHttpClient::DataLength (){return  iDataLength-iDataStart+iDataMinus;}
inline	int			CHttpClient::State      (){return  iDataState;}
inline	int			CHttpClient::CookieCount(){return  uCookieCount;}
inline	int			CHttpClient::Progress(int *pSize,int *pInfo){if(pSize)*pSize=iProgressEnd;if(pInfo)*pInfo=iProgressType;return iProgressPos;}
inline	void 		CHttpClient::ProgressReset(){iProgressEnd=0;iProgressType=0;iProgressPos=0;}
inline	void 		CHttpClient::ForceReconnect(){iNewConnection=1;}
inline	void 		CHttpClient::Encode(int iEncode){iEncodeRequests=iEncode;}
inline	void 		CHttpClient::SetVersion(int iVersion){iHttpVersion=iVersion;};
inline	const char *CHttpClient::GetCookie(unsigned uNum){return (uNum<uCookieCount)? pCookies[uNum]:0;}

#undef	v

#endif

