//*****************************************************************************
//*
//*
//*		HttpClientNtlm.cpp
//*
//*
//*****************************************************************************
//
//	Das ist ein Modud fr CHttpClient um ein Login via NTLM zu ermglichen.
//	Es muss das Makro HCL_USE_NTLM auf eins gesetzt werden.
//	Hilfreiche Infos giibr es unter:	https://github.com/dyduyu/ntlm
//	
//	Das Modul braucht die Open-SSL Bibliothek, und die Enviorment-Variable
//	$(OPENSSL_PATH) muss gesetzt werden.
//	
//	(C) Copyright Anton Zechner 2007
//
#ifdef      GLOBAL_CONFIG
#define     HTTPCLIENT_CPP
#include    <GlobalConfig.h>
#undef      HTTPCLIENT_CPP
#endif

#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<stddef.h>
#include	<string.h>
#include	<malloc.h>
#include	"HttpClient.h"
#include	"SslFunctions.h"
#include	"System/SocketUtility.h"
#include	"openssl/des.h"
#include	"openssl/md4.h"
#include	"openssl/md5.h"
#include	"openssl/hmac.h"


#if        _MSC_VER >= 1000
#pragma 	warning( disable :4996 )
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		WIN32
#include	<windows.h>
#else
#include	<time.h>
#endif 	

#ifdef	   _MSC_VER
#define		HttpInt64	__int64
#else
#define		HttpInt64	long long 
#endif 		

#ifdef		NTLM_TEST
#ifndef	   _DEBUG
#undef		NTLM_TEST
#endif 	
#endif 		


#define 	NTLM_FLAG_UNICODE			0x00000001	// Text sind in Unicode
#define 	NTLM_FLAG_OEM               0x00000002	// OEM-Strings werden untersttzt	
#define 	NTLM_FLAG_REQUEST_TARGET    0x00000004	// Server Authentifikation in Type2 Nachricht
#define 	NTLM_FLAG_DIGITAL_SIGNATURE	0x00000010	// Eine digitale Signatur einfgen
#define 	NTLM_FLAG_COMM_ENCRYPT		0x00000020	// Die Kommunikation soll verschlsselt werden
#define 	NTLM_FLAG_DATAGRAM_AUTH		0x00000040	// Authentifikation via Datagramm
#define 	NTLM_FLAG_LANMAN_SESSKEY	0x00000080	// LAN-Manager Session-Key verwenden
#define 	NTLM_FLAG_NETWARE			0x00000100	// Unbekannt
#define 	NTLM_FLAG_NEGOTIATE_NTLM    0x00000200	// NTLM Authentifikation verwenden
#define 	NTLM_FLAG_DOMAIN_SUPP		0x00001000	// Sende die Domain in Type1 Nachricht
#define 	NTLM_FLAG_WORKSTATION_SUPP  0x00002000	// Sende die Workstation in Type1 Nachricht
#define 	NTLM_FLAG_LOCAL_CALL		0x00004000	// Server und Client sind auf der selben Maschiene
#define 	NTLM_FLAG_ALWAYS_SIGN       0x00008000	// Die Kommunication muss siegiert werden
#define 	NTLM_FLAG_TYPE_DOMAIN		0x00010000	// Bereich ist eine Domain 
#define 	NTLM_FLAG_TYPE_SERVER		0x00020000	// Bereich ist ein Server
#define 	NTLM_FLAG_TYPE_SHARE		0x00040000	// Bereich ist ein Share
#define 	NTLM_FLAG_NEGOTIATE_NTLM2	0x00080000	// NTLM2 Signierung benutzen
#define 	NTLM_FLAG_TARGET_INFO 		0x00800000	// Einen Target-Informationsblock einfgen
#define 	NTLM_FLAG_128BIT_ENCRYP		0x20000000	// 128-bit Verschlsselung wird untersttzt
#define 	NTLM_FLAG_56BIT_ENCRYP		0x80000000	// 56-bit Verschlsselung wird untersttzt

typedef  struct										
	{
    char			cProtocol[8];					// "NTLMSSP\0"
    unsigned int 	uiType;	         				// Nachrichtentyp 0x01
    unsigned int 	uiFlags;       					// Diverse Flags: 0xB203
    unsigned short	usDomainSize;					// Lnge Domain-Namen
    unsigned short	usDomainAlloc;       			// Allokierte Lnge Domain-Namen
    unsigned int 	uiDomainOffset;					// Offset Domain-Namen
    unsigned short	usHostSize;						// Lnge Host-Namen
    unsigned short	usHostAlloc;					// Allokierte Lnge Host-Namen
    unsigned int 	uiHostOffset;					// Offset Host-Namen (immer 0x28)
	unsigned char	ucOsMajorVer;					// OS-Versions Hi
	unsigned char	ucOsMinorVer;					// OS-Versions Low
	unsigned short	usOsBuild;						// OS-Build-Nummer
	unsigned int 	uiOsRevision;					// Revision immer 0x000F
													// Daten: Host+Domain oder nichts				
    }NtlmType1Msg;

typedef  struct 
	{
    char			cProtocol[8];					// "NTLMSSP\0"
    unsigned int 	uiType;		       				// Nachrichtentyp 0x02
    unsigned short	usTargetSize;					// Lnge Domain-Namen
    unsigned short	usTargetAlloc;       			// Allokierte Lnge Domain-Namen
    unsigned int 	uiTargetOffset;					// Offset Domain-Namen
    unsigned int 	uiFlags;       					// Diverse Flags: 0x8201
    unsigned char	ucChallenge[8];					// Challenge Bytes
	unsigned char	ucContext[8];					// Context Bytes
    unsigned short	usItemsSize;					// Lnge Domain-Namen (optional)
    unsigned short	usItemsAlloc;       			// Allokierte Lnge Domain-Namen
    unsigned int 	uiItemsOffset;					// Offset Domain-Namen
	unsigned char	ucOsMajorVer;					// OS-Versions Hi	  (optional)
	unsigned char	ucOsMinorVer;					// OS-Versions Low
	unsigned short	usOsBuild;						// OS-Build-Nummer
	unsigned int 	uiOsRevision;					// Revision immer 0x000F
													// Daten: Domain + [2,*]DomainName + [1,*]ServerName + [4,*]DnsDomain + [3,*]DnsServer + "\0"
    }NtlmType2Msg;

typedef  struct 
	{
    char			cProtocol[8];					// "NTLMSSP\0"
    unsigned int 	uiType;		       				// Nachrichtentyp 0x03
    unsigned short	usLmRepSize;					// Lnge LW-Antwort
    unsigned short	usLmRespAlloc;					// Allokierte Lnge LW-Antwort
    unsigned int 	uiLmRespOffset;					// Offset LW-Antwort
    unsigned short	usNtRepSize;					// Lnge NTLW-Antwort
    unsigned short	usNtRespAlloc;					// Allokierte Lnge NTLW-Antwort
    unsigned int 	uiNtRespOffset;					// Offset NTLW-Antwort
    unsigned short	usDomainSize;					// Lnge Domain-Namen
    unsigned short	usDomainAlloc;       			// Allokierte Lnge Domain-Namen
    unsigned int 	uiDomainOffset;					// Offset Domain-Namen
    unsigned short	usUserSize;						// Lnge Domain-Namen
    unsigned short	usUserAlloc;       				// Allokierte Lnge Domain-Namen
    unsigned int 	uiUserOffset;					// Offset Domain-Namen
    unsigned short	usHostSize;						// Lnge Host-Namen
    unsigned short	usHostAlloc;	       			// Allokierte Lnge Host-Namen
    unsigned int 	uiHostOffset;					// Offset Host-Namen
    unsigned short	usSessionSize;					// Lnge Session-Key (optional)
    unsigned short	usSessionAlloc;	       			// Allokierte Lnge Session-Key
    unsigned int 	uiSessionOffset;				// Offset Session-Key
    unsigned int 	uiFlags;       					// Diverse Flags (optional)
	unsigned char	ucOsMajorVer;					// OS-Versions Hi  (optional)
	unsigned char	ucOsMinorVer;					// OS-Versions Low (optional)
	unsigned short	usOsBuild;						// OS-Build-Nummer (optional)
	unsigned int 	uiOsRevision;					// Revision immer 0x000F (optional)
	unsigned char	aDummy[16];						// Unbekannt muss frei bleiben
													// Daten: ...
    }NtlmType3Msg;

int _cdecl memtobase64(const void *pMem,int iSize,      char *pTarget,int iLineBreak);
int _cdecl base64tomem(      void *pMem,int iSize,const char *pBase64,int iLength);



const static unsigned char 	aDesMagic[8] = 	{0x4B,0x47,0x53,0x21,0x40,0x23,0x24,0x25};	//KGS!@$%



//*****************************************************************************
//*
//*		CreateChallange
//*
//*****************************************************************************
//	Erzeigt eine 8 Byte Zufallzahl
//	pChallenge		: Zeiger auf den 8-Byte-Puffer
static void CreateChallange(unsigned char *pChallenge)
{
unsigned	uRand;
unsigned	uPos;


	#ifdef NTLM_TEST

	memcpy(pChallenge,"\x56\xFF\x69\x06\xC1\x5E\xEB\x54",8);
	return;

	#endif 	

	#ifdef WIN32	

	GetSystemTimeAsFileTime((LPFILETIME)pChallenge);
	*(LPDWORD)pChallenge ^= GetTickCount();

	#else

	*(time_t*)pChallenge	= time(0);

	#endif 	

	for(uPos=0;uPos<8;uPos++)
		{
		uRand = rand();
		pChallenge[uPos] ^= (unsigned char)uRand;
		pChallenge[uPos] += uRand>>8;
		}

}

//*****************************************************************************
//*
//*		DesEncoder
//*
//*****************************************************************************
//	Berechnet eine Checksummen.
//	pKey		: Zeiger auf einen 7 Byte Schlssel
//	pInput		: Zeiger auf die 8 Byte DES-Eingabedaten
//	pOutput		: Zeiger auf den 8 Byte DES-Ausgabebuffer
static void DesEncoder(const unsigned char *pKey,const unsigned char *pInput,unsigned char *pOutput)
{
unsigned char	aBlock[8];
DES_ks			sKey;


    aBlock[0] = (pKey[0]     );
    aBlock[1] = (pKey[0] << 7) | (pKey[1] >> 1);
    aBlock[2] = (pKey[1] << 6) | (pKey[2] >> 2);
    aBlock[3] = (pKey[2] << 5) | (pKey[3] >> 3);
    aBlock[4] = (pKey[3] << 4) | (pKey[4] >> 4);
    aBlock[5] = (pKey[4] << 3) | (pKey[5] >> 5);
    aBlock[6] = (pKey[5] << 2) | (pKey[6] >> 6);
    aBlock[7] = (pKey[6] << 1);

    DES_set_odd_parity((DES_cblock*)aBlock);
	DES_set_key((DES_cblock*)aBlock,&sKey);
    DES_ecb_encrypt((DES_cblock*)pInput,(DES_cblock*)pOutput,&sKey,DES_ENCRYPT);

}

//*****************************************************************************
//*
//*		NtLmHash
//*
//*****************************************************************************
//	Berechnet den NTLM-Hash aus einem Password
//	pPassword	: Zeiger auf das Passwort
//	pHash		: Zeiger auf 16 Byte-Puffer fr die MD4-Checksumme
//	Ergibt die Lnge des Passworts
static int NtLmHash(const char *pPassword,unsigned char *pHash)
{
unsigned char	aPassWd[128];
unsigned		uLen;


	for(uLen=0;uLen<sizeof(aPassWd)/2;uLen++)		// Password in Unicode umwandeln
		{
		if(!pPassword[uLen])break;
		aPassWd[uLen*2  ] = pPassword[uLen];
		aPassWd[uLen*2+1] = 0;
		}

	MD4(aPassWd,uLen*2,pHash);						// MD4-Summe vom Unicode-Password erstellen

	/*
	Md4Context sMd4Ctx;
	Md4Init	 (&sMd4Ctx);							
	Md4Update(&sMd4Ctx,aPassWd,uLen*2);
	Md4Final (&sMd4Ctx,pHash);
	*/


return uLen;
}


//*****************************************************************************
//*
//*		NtLm2Hash
//*
//*****************************************************************************
//	Berechnet den NTLM-Hash aus einem Password
//	pPassword	: Zeiger auf das Passwort
//	pUserName	: Zeiger auf den User-Namen
//	pDomain		: Ist die Domain
//	pHash		: Zeiger auf 16 Byte-Puffer fr die HASH-Daten
//	Ergibt die Lnge des Passworts
static int NtLm2Hash(const char *pPassword,const char *pUserName,const char *pDomain,unsigned char *pHash)
{
unsigned char	aNameBuff[512];
unsigned char	aKeyBytes[16]; 
unsigned		uPassLen;
unsigned		uHashLen;
unsigned		uOffset;
unsigned char	ucSign;
HMAC_CTX		sHamc;



    uPassLen = NtLmHash(pPassword,aKeyBytes);


	for(uOffset=0;uOffset<sizeof(aNameBuff);)
		{
			ucSign = *pUserName;
		if(!ucSign)break;

		if(ucSign>='a' && ucSign<='z')
			{
			ucSign -= 0x20;
			}

		aNameBuff[uOffset] = ucSign;	uOffset++;
		aNameBuff[uOffset] = 0;			uOffset++;

		pUserName++;
		}

	for(;uOffset<sizeof(aNameBuff);)
		{
			ucSign = *pDomain;
		if(!ucSign)break;

		aNameBuff[uOffset] = ucSign;	uOffset++;
		aNameBuff[uOffset] = 0;			uOffset++;

		pDomain++;
		}

	uHashLen = 16;	
    
    HMAC_Init  (&sHamc,aKeyBytes,16,EVP_md5());
    HMAC_Update(&sHamc,aNameBuff   , uOffset );
    HMAC_Final (&sHamc,pHash       ,&uHashLen);
    

return uPassLen;    
}


//*****************************************************************************
//*
//*		Lm1Response
//*
//*****************************************************************************
//	Wird aufgerufen wenn Version 1 des Lanman-Protokolls benutzt wird.
//	pLmResp		: Zeiger auf einen 24-Byte-Puffer. Hiier wird die Lanmanager-
//				  Antwort abgelegt. ([0..7]=Client-Challenge [8..23]=Null)
//	pChallenge	: Server-Challenge aus der Typ2-Nachricht vom Server.
//				  Das ist eine 8-Byte Zufallszahl.
//	pPassword	: Ist das Passwort
void Lm1Response(unsigned char *pLmResp,const unsigned char *pChallenge,const char *pPassword)
{
unsigned char 	aPassBuffer[14];
unsigned char 	aKeyBytes[21];
unsigned		uPassLen;


													// Passwort in Grobuchstaben umwandeln
	for(uPassLen=0;uPassLen<sizeof(aPassBuffer);uPassLen++)
		{
		aPassBuffer[uPassLen] = pPassword[uPassLen];

		if(aPassBuffer[uPassLen]==0)break;
		if(aPassBuffer[uPassLen]>='a' && aPassBuffer[uPassLen]<='z')
			{
			aPassBuffer[uPassLen] -= 0x20;
			}
		}

	for(;uPassLen<14;uPassLen++)					// Passwort-Puffer mit 0 fllen
		{
		aPassBuffer[uPassLen] = 0;
		}


    DesEncoder(aPassBuffer+7*0,aDesMagic,aKeyBytes+8*0);
    DesEncoder(aPassBuffer+7*1,aDesMagic,aKeyBytes+8*1);
    memset(aKeyBytes+16,0,5);
    
    DesEncoder(aKeyBytes+7*0,pChallenge,pLmResp+8*0);
    DesEncoder(aKeyBytes+7*1,pChallenge,pLmResp+8*1);
    DesEncoder(aKeyBytes+7*2,pChallenge,pLmResp+8*2);
    
}
	 
//*****************************************************************************
//*
//*		NtLm1Response
//*
//*****************************************************************************
//	Wird aufgerufen wenn Version 1 des NT-Lanman-Protokolls benutzt wird.
//	pLmResp		: Zeiger auf einen 24-Byte-Puffer. Hiier wird die Lanmanager-
//				  Antwort abgelegt. ([0..7]=Client-Challenge [8..23]=Null)
//	pResponse	: Ist ein 24-Byte-Puffer fr die Rckgabe der NTLW-Antwort
//	pChallenge	: Server-Challenge aus der Typ2-Nachricht vom Server.
//				  Das ist eine 8-Byte Zufallszahl.
//	pPassword	: Ist das Passwort
void NtLm1Response(unsigned char *pLmResp,unsigned char *pResponse,const unsigned char *pChallenge,const char *pPassword)
{
unsigned char	aKeyBytes[21];


	Lm1Response(pLmResp,pChallenge,pPassword);

	NtLmHash(pPassword,aKeyBytes);					// MD4-Summe vom Unicode-Password erstellen
	memset(aKeyBytes+16,0,5);
    
    
    DesEncoder(aKeyBytes+7*0,pChallenge,pResponse+8*0);
    DesEncoder(aKeyBytes+7*1,pChallenge,pResponse+8*1);
    DesEncoder(aKeyBytes+7*2,pChallenge,pResponse+8*2);
    
}


//*****************************************************************************
//*
//*		NtLm2SessResp
//*
//*****************************************************************************
//	Wird aufgerufen wenn das NTLM_FLAG_NEGOTIATE_NTLM2 Flag (0x00080000) in
//	der Typ2-Nachricht empfangen wurde.	(Session-Response)
//	pLmResp		: Zeiger auf einen 24-Byte-Puffer. Hiier wird die Lanmanager-
//				  Antwort abgelegt. ([0..7]=Client-Challenge [8..23]=Null)
//	pResponse	: Ist ein 24-Byte-Puffer fr die Rckgabe der NTLW-Antwort
//	pChallenge	: Server-Challenge aus der Typ2-Nachricht vom Server.
//				  Das ist eine 8-Byte Zufallszahl.
//	pPassword	: Ist das Passwort
void NtLm2SessResp(unsigned char *pLmResp,unsigned char *pResponse,const unsigned char *pChallenge,const char *pPassword)
{
unsigned char	aSessionKey[16];
unsigned char	aInputData[16];
unsigned char	aKeyBytes[21];



	CreateChallange(pLmResp);						// In den unteren 8 Byte eine Zufallszahl erstellen

	memset(pLmResp+8,0,16);

	memcpy(aInputData  ,pChallenge,8);
	memcpy(aInputData+8,pLmResp   ,8);

	MD5(aInputData,16,aSessionKey);					// MD5-Summe aus Server und Client-Challange berechnen		

	/*
	Md5Context sMd5Ctx;
	Md5Init	 (&sMd5Ctx);							
	Md5Update(&sMd5Ctx,pChallenge,8);
	Md5Update(&sMd5Ctx,pLmResp   ,8);
	Md5Final (&sMd5Ctx,aSessionKey );
	*/

	NtLmHash(pPassword,aKeyBytes);					// MD4-Summe vom Unicode-Password erstellen
	memset(aKeyBytes+16,0,5);

	DesEncoder(aKeyBytes+7*0,aSessionKey,pResponse+8*0);
	DesEncoder(aKeyBytes+7*1,aSessionKey,pResponse+8*1);
	DesEncoder(aKeyBytes+7*2,aSessionKey,pResponse+8*2);


}


//*****************************************************************************
//*
//*		NtLm2Response
//*
//*****************************************************************************
//	Wird aufgerufen wenn das NTLM_FLAG_NEGOTIATE_NTLM2 Flag (0x00080000) in
//	der Typ2-Nachricht empfangen wurde.	(Session-Response)
//	pLmResp			: Zeiger auf einen 24-Byte-Puffer. Hiier wird die Lanmanager-
//					  Antwort abgelegt. ([0..7]=Client-Challenge [8..23]=Null)
//	pResponse		: Ist ein Puffer fr die Rckgabe der NTLW-Antwort.
//					  Der Puffer muss mindestens 16+28+uTargetInfoLen gro sein.
//	uMaxResp		: Ist die Gre des Rckgabepuffers in Byte.
//	pChallenge		: Server-Challenge aus der Typ2-Nachricht vom Server.
//					  Das ist eine 8-Byte Zufallszahl.
//	pPassword		: Ist das Passwort
//	pUserName		: Ist der Username fr das Login
//	pDomain			: Ist die benutzte Domne
//	pTargetInfo		: Zeiger auf Domain-Namen in Type2-Nachricht (siehe uiItemsOffset)
//	uTargetInfoLen	: Lnge  vom Domain-Namen in Type2-Nachricht (siehe usItemsSize) 
//	Ergibt die Lnge der NTLW-Antwort in Byte. 
int NtLm2Response(unsigned char *pLmResp,unsigned char *pResponse,unsigned uMaxResp,const unsigned char *pChallenge,const char *pPassword,const char *pUserName,const char *pDomain,const unsigned char *pTargetInfo,unsigned uTargetInfoLen)
{
unsigned char 	aBlobData[256];
unsigned char	aKeyBytes[16]; 
unsigned char	aBaseData[16]; 
unsigned		uDataLen;
unsigned		uHashLen;
HMAC_CTX		sHamc;


	CreateChallange(aBaseData);						// 8-Byte Zufallszahl erstellen
	memcpy(aBaseData+8,pChallenge,8);


	NtLm2Hash(pPassword,pUserName,pDomain,aKeyBytes);

	uHashLen = 16;
	uDataLen = 16;

    HMAC_Init  (&sHamc,aKeyBytes,16,EVP_md5());
    HMAC_Update(&sHamc,aBaseData   , uDataLen);
    HMAC_Final (&sHamc,pLmResp     ,&uHashLen);
	
	memcpy(pLmResp+16,aBaseData,8);


	if(uTargetInfoLen>sizeof(aBlobData)-16-28)
		{
		uTargetInfoLen = 0;
		}
													// Domain aus Type2-Nachricht einfgen
	memcpy(aBlobData+16+28,pTargetInfo,uTargetInfoLen);



	#ifdef NTLM_TEST								// Aktuellen Zeitstempel einfgen

	memcpy(aBlobData+16+ 8,"\x00\x9D\xB5\xA7\x07\x92\xD3\x01",8);
	
	#elif defined(WIN32)
	
	GetSystemTimeAsFileTime((LPFILETIME)(aBlobData+16+8));
	
	#else
	
	HttpInt64	iTimeStamp;

	iTimeStamp	= time(NULL);
	iTimeStamp += 11644473600;
	iTimeStamp *= 10000000;
	*(HttpInt64*)(aBlobData+16+8) = iTimeStamp;
	
	#endif 	

	memcpy(aBlobData+16+16,aBaseData,8);			// 8-Byte Zufallszahl kopieren


    aBlobData[16+0] = 0x01;
    aBlobData[16+1] = 0x01;
	memset(aBlobData+16+2 ,0,6);		  
	memset(aBlobData+16+24,0,4);		  
	memcpy(aBlobData+8,pChallenge,8);

    
	uHashLen = 16;
	uDataLen = 8+28+uTargetInfoLen;


    HMAC_Init  (&sHamc,aKeyBytes,16,EVP_md5());
    HMAC_Update(&sHamc,aBlobData+8 , uDataLen);
    HMAC_Final (&sHamc,aBlobData   ,&uHashLen);

	   uDataLen += 8;
	if(uDataLen>uMaxResp)return 0;

	memcpy(pResponse,aBlobData,uDataLen);
	

return uDataLen;
}



//*****************************************************************************
//*
//*		HandleNtLm
//*
//*****************************************************************************
//	Den Login via NTLM Protokoll abwickeln
//	pRequest	: Ist der Request-Puffer mit dem originalen Request
//	iLen		: Lnge des Requests
//	iSize		: Gre des Request-Puffers
//	uFlags		: Flags vom GET-Aufruf
//					Bit4..7 = NTLM-Login-Modus (1=NTLM_V1 2=NTLM_V2+Session 3=NTLM_V2)
//					Bit24	= Anfrage wurde gesendet
//					Bit25	= Antwort wurde gesendet
//	Ergibt die neuen Lnge vom Request bzw. eine Fehlernummer
int	CHttpClient::HandleNtLm(char *pRequest,int iLen,int iMax,unsigned &uFlags)
{
char			cBuffer[1024];
char			cTemp[1024];
const char	   *pHost;
char		   *pNtlm;
int				iType;
int				iMode;
int				iByte;
int				iNum;
int				iAdd;
int				iOff;
int				iPos;
int				iAll;



	pNtlm	= pData+iNtlmPos;
	iMode	= (uFlags>>4)&0x0F;


	if(pData[iNtlmPos]<' ')							// Ist es eine neue Anfrage
		{
		NtlmType1Msg	*pOut;

		if(uFlags&0x03000000)
			{
			return 0;								// Schon versucht
			}

		if(iMax-iLen<sizeof(NtlmType1Msg)+sizeof(NtlmType1Msg)/2+26)
			{
			return HCL_HEADER_TOO_BIG;
			}


		uFlags |= 0x01000000;
		pHost   = (cHostName[0]=='*')? IpGetHostname():cHostName;
		pOut	= (NtlmType1Msg*)cBuffer;

		if(!pHost)
			{
			pHost = "";
			iNum  = 0;
			}
		else{
			iNum  = strlen(pHost);
			}

		iAdd = strlen(cDomain);

		iAdd = 0;									// REM AZ Host und Domain nicht anhngen
		iNum = 0;


		memset(pOut,0,sizeof(NtlmType1Msg));
		memcpy(pOut->cProtocol,"NTLMSSP",8);
		
		pOut->uiType				= 1;
		pOut->uiFlags				= 0x00088205;
		pOut->ucOsMajorVer			= 6;			// OS-Versions Hi
		pOut->ucOsMinorVer			= 1;			// OS-Versions Low
		pOut->usOsBuild				= 0x1DB1;		// OS-Build-Nummer
		pOut->uiOsRevision			= 0x0F000000;
		iAll						= sizeof(NtlmType1Msg);


		switch(iMode)
			{
		case 1:	pOut->uiFlags = 0x00008207;	break;	// NTLM V1
		case 2:	pOut->uiFlags = 0x00088207;	break;	// NTLM V2+Session
		case 3:	pOut->uiFlags = 0x00088207;	break;	// NTLM V2
			}

		if(iNum)									// Hostnamen einfgen
			{
			pOut->uiFlags		   |= NTLM_FLAG_TYPE_SERVER;
			pOut->usHostSize	 	= iNum;
			pOut->usHostAlloc		= iNum;
			pOut->uiHostOffset		= iAll;

			for(iPos=0;iPos<iNum;iPos++,iAll++)
				{
				if(pHost[iPos]<'a' || pHost[iPos]>'z')
						cBuffer[iAll] = pHost[iPos];
				else	cBuffer[iAll] = pHost[iPos]-0x20;
				}
			}

		if(iAdd)									// Domainnamen einfgen
			{
			pOut->uiFlags	       |= NTLM_FLAG_TYPE_DOMAIN;
			pOut->usDomainSize		= iAdd;
			pOut->usDomainAlloc		= iAdd;
			pOut->uiDomainOffset	= iAll;

			for(iPos=0;iPos<iNum;iPos++,iAll++)
				{
				if(cDomain[iPos]<'a' || cDomain[iPos]>'z')
						cBuffer[iAll] = cDomain[iPos];
				else	cBuffer[iAll] = cDomain[iPos]-0x20;
				}
			}

		iOff  = iLen-2;
		memcpy(pRequest+iOff,"Authorization: NTLM ",20);	iOff += 20;
		iAdd  = memtobase64(pOut,iAll,pRequest+iOff, 0);	iOff += iAdd;
		memcpy(pRequest+iOff,"\r\n\r\n",4);					iOff += 4;

		return iOff;
		}



	if(uFlags&0x01000000)							// Zeiter Schritt
		{	  
		char			cDomainX[128]="";
		unsigned char	aNtResp[1024];
		unsigned char	aLmResp[24];
		unsigned char  *pTarget;	
		const char     *pText;
		NtlmType2Msg   *pMsg;
		NtlmType3Msg   *pOut;
		unsigned		uBits;
		int				iSize;
		int				iLenNt;
		int				iLenLm;
		int				iPlus;
	


		if(uFlags&0x02000000)						// Antwort schon versandt
			{
			return 0;								
			}


		for(iOff=0;pNtlm[iOff]>' ';iOff++);
		pMsg	= (NtlmType2Msg*)cTemp;

		   iAdd = base64tomem(pMsg,sizeof(cTemp),pNtlm,iOff);
		if(iAdd<offsetof(NtlmType2Msg,usItemsSize) || pMsg->uiType!=2)
			{
			return 0;								// Keine gltige Type2-Message
			}	

		uFlags |= 0x02000000;


		if(!(pMsg->uiFlags&NTLM_FLAG_NEGOTIATE_NTLM2))
			{
			iMode = 1;
			}

		if(iAdd>sizeof(NtlmType2Msg))				// Gibt es Eintrge nach dem Header
			{
			iOff = pMsg->uiItemsOffset;

			while(iOff+4<=iAdd)
				{
				iType = *(unsigned short*)(cTemp+iOff  );
				iByte = *(unsigned short*)(cTemp+iOff+2);
				pText =  (const    char *)(cTemp+iOff+4);

				if(!iType)break;

				if(iType==2)
					{
					for(iPos=0;iPos<sizeof(cDomainX)-1;iPos++)
						{
						if(iPos*2>=iByte)break;
						cDomainX[iPos] = pText[iPos*2];
						}
					
					cDomainX[iPos] = 0;					
					break;
					}

				iOff += iByte+4;
				}
			}


		switch(iMode)
			{
		case 1:										// NTLM V1

			NtLm1Response(aLmResp,aNtResp,pMsg->ucChallenge,cPassWord);
			uBits  = 0x00008207;
			iLenLm = 24;
			iLenNt = 24;
			break;

		case 2:										// NTLM V2 Session
		default:

			NtLm2SessResp(aLmResp,aNtResp,pMsg->ucChallenge,cPassWord);
			uBits  = 0x00088207;
			iLenLm = 24;
			iLenNt = 24;
			break;
		
		case 3:										// NTLM V2
			
			pTarget = ((unsigned char*)pMsg)+pMsg->uiItemsOffset;
			iSize	= pMsg->uiItemsOffset;
			uBits	= 0x00088207;

			if(iAdd<offsetof(NtlmType2Msg,ucOsMajorVer) || (int)(iSize+pMsg->uiItemsOffset)>iAdd)
				{
				iSize = 0;
				}

			iLenNt = NtLm2Response(aLmResp,aNtResp,sizeof(aNtResp),pMsg->ucChallenge,cPassWord,cUserName,cDomain,pTarget,iSize);
			iLenLm = 24;

			break;
			} 

		if(!(pMsg->uiFlags&NTLM_FLAG_UNICODE))		// Kein Unicode
			{
			uBits &= ~NTLM_FLAG_UNICODE;
			iPlus  = 1;
			}
		else{
			uBits &= ~NTLM_FLAG_OEM;
			iPlus  = 2;
			}
		
		pOut	= (NtlmType3Msg*)cBuffer;
		memset(pOut,0,sizeof(NtlmType3Msg));
		memcpy(pOut->cProtocol,"NTLMSSP",8);
		
		pOut->uiType				= 3;
		pOut->uiFlags				= uBits;
		pOut->ucOsMajorVer			= 6;			// OS-Versions Hi
		pOut->ucOsMinorVer			= 1;			// OS-Versions Low
		pOut->usOsBuild				= 0x1DB1;		// OS-Build-Nummer
		pOut->uiOsRevision			= 0x0F000000;	// OS-Revision
		iAll						= sizeof(NtlmType3Msg);

		if(iAll+iLenLm>sizeof(cBuffer))
			{
			return 0;
			}

		pOut->usLmRepSize		= iLenLm;
		pOut->usLmRespAlloc		= iLenLm;
		pOut->uiLmRespOffset	= iAll;
		
		memcpy(cBuffer+iAll,aLmResp,iLenLm);
		iAll += iLenLm;	

		if(iAll+iLenNt>sizeof(cBuffer))
			{
			return 0;
			}

		pOut->usNtRepSize		= iLenNt;
		pOut->usNtRespAlloc		= iLenNt;
		pOut->uiNtRespOffset	= iAll;
		
		memcpy(cBuffer+iAll,aNtResp,iLenNt);
		iAll += iLenNt;	

		pText = cDomain;
		iByte = strlen(pText);

		if(!iByte && cDomainX[0])					// Domne auf Typ2-Nachricht verwenden
			{
			pText = cDomainX;
			iByte = strlen(pText);
			}
		
		if(iAll+iByte*2>sizeof(cBuffer))
			{
			return 0;
			}

		pOut->usDomainSize		= iByte*iPlus;
		pOut->usDomainAlloc		= iByte*iPlus;
		pOut->uiDomainOffset	= iAll;
		
		for(iPos=0;iPos<iByte;iPos++)				// Domne einfgen
			{
			cBuffer[iAll] = pText[iPos];iAll++; if(iPlus==1)continue;
			cBuffer[iAll] = 0;			iAll++;
			}
		
		pText = cUserName;
		iByte = strlen(pText);

		if(iAll+iByte*2>sizeof(cBuffer))
			{
			return 0;
			}

		pOut->usUserSize		= iByte*iPlus;
		pOut->usUserAlloc		= iByte*iPlus;
		pOut->uiUserOffset		= iAll;
		
		for(iPos=0;iPos<iByte;iPos++)				// User-Namen einfgen
			{
			cBuffer[iAll] = pText[iPos];iAll++; if(iPlus==1)continue;
			cBuffer[iAll] = 0;			iAll++;
			}
		

		pText = (cHostName[0]=='*')? IpGetHostname():cHostName;
		iByte = strlen(pText);

		if(iAll+iByte*2>sizeof(cBuffer))
			{
			return 0;
			}

		pOut->usHostSize		= iByte*iPlus;
		pOut->usHostAlloc		= iByte*iPlus;
		pOut->uiHostOffset		= iAll;
		
		for(iPos=0;iPos<iByte;iPos++)				// Host-Namen einfgen
			{
			cBuffer[iAll] = pText[iPos];iAll++; if(iPlus==1)continue;
			cBuffer[iAll] = 0;			iAll++;
			}

		pOut->usSessionSize		= 0;
		pOut->usSessionAlloc	= 0;
		pOut->uiSessionOffset	= iAll;

		if(iMax-iLen<iAll+iAll/2)
			{
			return HCL_HEADER_TOO_BIG;
			}

		iOff  = iLen-2;
		memcpy(pRequest+iOff,"Authorization: NTLM ",20);	iOff += 20;
		iAdd  = memtobase64(pOut,iAll,pRequest+iOff, 0);	iOff += iAdd;
		memcpy(pRequest+iOff,"\r\n\r\n",4);					iOff += 4;

		return iOff;
		}	



				



return 0;
}




//*****************************************************************************
//*
//*		NtlmTest
//*
//*****************************************************************************
//	Testfunktion fr NTLM, make_type3_msg ist aus https://github.com/dyduyu/ntlm
#ifdef NTLM_TEST
char *make_type3_msg(const char *username, const char *pPassword, const char *domain, const char *host, char *msg2_b64_buff, int ntlm_resp_type);

void NtlmTest()
{
unsigned char	aBuffer[1024];
unsigned char	aNtResp[1024];
unsigned char	aLmResp[24];
unsigned char  *pLmResp;
unsigned char  *pNtResp;
NtlmType1Msg   *pNow1; 
NtlmType2Msg   *pNow2;
NtlmType3Msg   *pNow3;
NtlmType3Msg   *pMsg3;
char		   *pBas64;
int				iMode;
int				iLen;


unsigned char	aNtlmTyp1[] =
	{
	0x4E,0x54,0x4C,0x4D,0x53,0x53,0x50,0x00,0x01,0x00,0x00,0x00,0x07,0x82,0x08,0xA2,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x06,0x01,0xB1,0x1D,0x00,0x00,0x00,0x0F,
	};

unsigned char	aNtlmTyp2[] =
	{
	0x4E,0x54,0x4C,0x4D,0x53,0x53,0x50,0x00,0x02,0x00,0x00,0x00,0x0E,0x00,0x0E,0x00,
	0x38,0x00,0x00,0x00,0x05,0x82,0x89,0xA2,0xF4,0xB7,0xDA,0x5E,0x50,0xA3,0x10,0xA2,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xAC,0x00,0xAC,0x00,0x46,0x00,0x00,0x00,
	0x06,0x01,0xB1,0x1D,0x00,0x00,0x00,0x0F,0x47,0x00,0x41,0x00,0x57,0x00,0x32,0x00,
	0x30,0x00,0x30,0x00,0x30,0x00,0x02,0x00,0x0E,0x00,0x47,0x00,0x41,0x00,0x57,0x00,
	0x32,0x00,0x30,0x00,0x30,0x00,0x30,0x00,0x01,0x00,0x10,0x00,0x41,0x00,0x58,0x00,
	0x2D,0x00,0x53,0x00,0x52,0x00,0x56,0x00,0x31,0x00,0x36,0x00,0x04,0x00,0x20,0x00,
	0x67,0x00,0x61,0x00,0x77,0x00,0x2E,0x00,0x67,0x00,0x61,0x00,0x77,0x00,0x67,0x00,
	0x72,0x00,0x6F,0x00,0x75,0x00,0x70,0x00,0x2E,0x00,0x63,0x00,0x6F,0x00,0x6D,0x00,
	0x03,0x00,0x32,0x00,0x41,0x00,0x58,0x00,0x2D,0x00,0x53,0x00,0x52,0x00,0x56,0x00,
	0x31,0x00,0x36,0x00,0x2E,0x00,0x67,0x00,0x61,0x00,0x77,0x00,0x2E,0x00,0x67,0x00,
	0x61,0x00,0x77,0x00,0x67,0x00,0x72,0x00,0x6F,0x00,0x75,0x00,0x70,0x00,0x2E,0x00,
	0x63,0x00,0x6F,0x00,0x6D,0x00,0x05,0x00,0x18,0x00,0x67,0x00,0x61,0x00,0x77,0x00,
	0x67,0x00,0x72,0x00,0x6F,0x00,0x75,0x00,0x70,0x00,0x2E,0x00,0x63,0x00,0x6F,0x00,
	0x6D,0x00,0x07,0x00,0x08,0x00,0x80,0x55,0x44,0x99,0x8E,0x3E,0xD3,0x01,0x00,0x00,
	0x00,0x00,
	};

unsigned char	aNtlmTyp3[] =
	{
	0x4E,0x54,0x4C,0x4D,0x53,0x53,0x50,0x00,0x03,0x00,0x00,0x00,0x18,0x00,0x18,0x00,
	0x88,0x00,0x00,0x00,0x18,0x00,0x18,0x00,0xA0,0x00,0x00,0x00,0x0E,0x00,0x0E,0x00,
	0x58,0x00,0x00,0x00,0x12,0x00,0x12,0x00,0x66,0x00,0x00,0x00,0x10,0x00,0x10,0x00,
	0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xB8,0x00,0x00,0x00,0x05,0x82,0x88,0xA2,
	0x06,0x01,0xB1,0x1D,0x00,0x00,0x00,0x0F,0x8C,0x81,0x76,0xDC,0x42,0x0E,0x30,0xD1,
	0xB2,0xE0,0xFB,0x7B,0x1A,0xDA,0xBA,0xE1,0x67,0x00,0x61,0x00,0x77,0x00,0x32,0x00,
	0x30,0x00,0x30,0x00,0x30,0x00,0x61,0x00,0x75,0x00,0x74,0x00,0x78,0x00,0x2D,0x00,
	0x7A,0x00,0x65,0x00,0x61,0x00,0x6E,0x00,0x41,0x00,0x58,0x00,0x2D,0x00,0x50,0x00,
	0x43,0x00,0x2D,0x00,0x33,0x00,0x36,0x00,0x56,0xFF,0x69,0x06,0xC1,0x5E,0xEB,0x54,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x28,0x78,0x03,0x5D,0xF7,0x07,0x5E,0x4B,0xDA,0x73,0x3B,0x22,0x72,0x37,0xCD,0x4E,
	0xE3,0xDD,0xB7,0x1C,0xC7,0xB2,0x48,0xB9,
	};



	iMode	= 3;										// Modus 1..3 zum prfen

	pNow1   = (NtlmType1Msg*)aNtlmTyp1;
	pNow2   = (NtlmType2Msg*)aNtlmTyp2;
	pNow3   = (NtlmType3Msg*)aNtlmTyp3;
	pBas64  = make_type3_msg("autx-zean","E3e3e3e3","gaw2000","ax-pc-36",(char*)pNow2,iMode);	// Letzte Zahl ist der Modus (1..3)
	iLen    = base64tomem(aBuffer,sizeof(aBuffer),pBas64,0xFFFF);
	pMsg3   = (NtlmType3Msg*)aBuffer;	
	pLmResp = aBuffer+pMsg3->uiLmRespOffset;
	pNtResp = aBuffer+pMsg3->uiNtRespOffset;

	switch(iMode)
		{
	case 1:	NtLm1Response(aLmResp,aNtResp,pNow2->ucChallenge,"E3e3e3e3");
			break;

	case 2:	NtLm2SessResp(aLmResp,aNtResp,pNow2->ucChallenge,"E3e3e3e3");
			break;

	case 3:	NtLm2Response(aLmResp,aNtResp,sizeof(aNtResp),pNow2->ucChallenge,"E3e3e3e3","autx-zean","gaw2000",aNtlmTyp2+pNow2->uiItemsOffset,pNow2->usItemsSize);
			break;
		}


}

#endif 	

