//*****************************************************************************
//*
//*
//*		BmpDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2008
//
//	Ld BMP-Bilder.
//

#include "BmpDecoder.h"
									   

typedef struct	
	{						  
	unsigned char	ucCnt;
	unsigned char	ucCmd;
	}RleCode;

//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
BmpDecoder::BmpDecoder()
{
	m_iMeterX = 0;											// Pixel per Meter
	m_iMeterY = 0;											//		*
	m_pBuffer = 0;
}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
BmpDecoder::~BmpDecoder()
{
}

//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
void BmpDecoder::ReadImage(const char *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw EBmpFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void BmpDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw EBmpFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void BmpDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage)
{
BITMAPINFOHEADER*pHeader; 
BITMAPFILEHEADER sFileHeader;
bool			 bOs2Format;
unsigned	 	 uHeaderSize;
unsigned	 	 uColorCount;
unsigned		 uBytesRead;
unsigned		 uCompress;
unsigned	 	 uHeight;
unsigned	 	 uWidth;
unsigned	 	 uColors;
unsigned char	 aColor[4];
unsigned		 uCheck;
unsigned		 uPos;
unsigned		 uNum;


	
	uCheck		= 1;
	uBytesRead	= 0;
	m_pBuffer	= pFile;


	   uNum = m_pBuffer->Read(&sFileHeader,sizeof(sFileHeader));
	if(uNum!= sizeof(sFileHeader))throw EBmpFileReadError();

	
	uBytesRead += uNum;


	if(sFileHeader.bfType!= *(unsigned short*)"BM")		// Datei-Header prfen
		{
		pHeader = (BITMAPINFOHEADER*)&sFileHeader; 
		
		if(pHeader->biSize!=sizeof(BITMAPINFOHEADER) || pHeader->biWidth>0x8000 || pHeader->biHeight>0x8000)
			{
			throw EBmpNotABmpFile();
			}

		uCheck	   = 0;	
		uBytesRead = 0;
		sFileHeader.bfOffBits = sizeof(BITMAPINFOHEADER);
		m_pBuffer->Seek(0-sizeof(sFileHeader),SEEK_CUR);// Aus Resourcen laden 
		}
	
	m_pBuffer->Read(&uHeaderSize,sizeof(uHeaderSize));
	

	if(uHeaderSize==sizeof(BITMAPCOREHEADER))			// OS/2 Format Header
		{
		BITMAPCOREHEADER sHeader;

		sHeader.bcSize	= uHeaderSize;
		
		m_pBuffer->Read(&sHeader.bcWidth,sizeof(sHeader)-sizeof(uHeaderSize));
		
		uBytesRead     += sizeof(sHeader);
		uWidth       	= sHeader.bcWidth;
		uHeight      	= sHeader.bcHeight;
		m_uBitCount		= sHeader.bcBitCount;
		uColors      	= 0;
		uCompress 		= BI_RGB;
		bOs2Format		= true;
		m_iMeterX		= 0;							// Pixel per Meter
		m_iMeterY		= 0;							//		*
		}
	else if(uHeaderSize>=sizeof(BITMAPINFOHEADER))		// Normal Format
		{
		BITMAPINFOHEADER	sHeader;

		sHeader.biSize	= uHeaderSize;
		
		m_pBuffer->Read(&sHeader.biWidth,sizeof(sHeader)-sizeof(uHeaderSize)) ;

		uBytesRead     += sizeof(sHeader);
		uWidth			= sHeader.biWidth;
		uHeight			= sHeader.biHeight;
		m_uBitCount		= sHeader.biBitCount;
		uCompress		= sHeader.biCompression;
		uColors			= sHeader.biClrUsed;
		bOs2Format		= false;

		for(uPos=0;uPos<uHeaderSize-sizeof(BITMAPINFOHEADER);uPos++)
			{
			uBytesRead++;
			m_pBuffer->ReadByte();
			}

		m_iMeterX		= sHeader.biXPelsPerMeter;		// Pixel per Meter
		m_iMeterY		= sHeader.biYPelsPerMeter;		//		*
		}
	else{
		throw EBmpCorruptFile();
		}



//******************** Farben-Format ******************************************


	switch(m_uBitCount)
		{
	case 1:		if(uCompress!= BI_RGB)throw EBmpNotSupported();
				uColorCount  = 1 << m_uBitCount;	break;
	case 4:		if(uCompress!= BI_RGB && uCompress != BI_RLE4)throw EBmpNotSupported();
				uColorCount  = 1 << m_uBitCount;	break;
	case 8:		if(uCompress!= BI_RGB && uCompress != BI_RLE8)throw EBmpNotSupported();
				uColorCount  = 1 << m_uBitCount;	break;
	case 24:	uColorCount  = 0;					break;
	case 32:	uColorCount  = 0;					break;
	default:	throw EBmpNotSupported();
		}

	if(uColors)uColorCount=uColors;						// Speziele Farbanzahl

														// Neues Bild erstellen
	if(!sImage.SetSize(m_uBitCount,uWidth,uHeight,BI_NOT_INIT))		
		{
		throw EBmpUnknownError();
		}


//******************** Farben laden *******************************************


	if(bOs2Format)
		{
		for(uPos=0;uPos<uColorCount;uPos++)
			{
			m_pBuffer->Read(aColor,3) ;
			sImage.SetColor(uPos,aColor[2],aColor[1],aColor[0]);
			uBytesRead += 3;
			}
		}
	else{
		for(uPos=0;uPos<uColorCount;uPos++)
			{
			m_pBuffer->Read(aColor,4) ;
			sImage.SetColor(uPos,aColor[2],aColor[1],aColor[0],aColor[3]);
			uBytesRead += 4;
			}
		}


//******************** Header fertig laden ************************************


	if(uCheck)
		{
		if(uBytesRead>sFileHeader.bfOffBits)
			{
			throw EBmpCorruptFile();
			}

		for(uPos=uBytesRead;uPos<sFileHeader.bfOffBits;uPos++)
			{
			m_pBuffer->ReadByte();
			}
		}


//******************** Bild Laden *********************************************


	CallProgressFunction(0);

	
	if(m_uBitCount<=8)
		{
		switch(uCompress)
				{
		case BI_RGB:	if(ReadBIT8(uWidth,uHeight,sImage))throw EBmpFileReadError();	break;
		case BI_RLE8:	if(ReadRLE8(uWidth,uHeight,sImage))throw EBmpFileReadError();	break;
		case BI_RLE4:	if(ReadRLE4(uWidth,uHeight,sImage))throw EBmpFileReadError();	break;
		default:		throw EBmpCorruptFile();										break;
				}
		}
	else{
		switch(m_uBitCount)
				{
		case 24:		if(Read24(uWidth,uHeight,sImage))throw EBmpFileReadError();		break;
		case 32:		if(Read32(uWidth,uHeight,sImage))throw EBmpFileReadError();		break;
		default:		throw EBmpCorruptFile();										break;
				}
		}

	CallProgressFunction(100);

}

//*****************************************************************************
//*
//*		GetPixelPerMeter
//*
//*****************************************************************************
void BmpDecoder::GetPixelPerMeter(int &iMeterX,int &iMeterY)
{
	iMeterX	= m_iMeterX;									// Pixel per Meter
	iMeterY = m_iMeterY;									//		*
}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void BmpDecoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel);

	if(bCancel)throw EGraphicsAbort();
}


//*****************************************************************************
//*
//*		ReadBIT8
//*
//*****************************************************************************
//	Liest ein 8 Bit Bild ein
//	Ergibt 0 wenn ok bzw. -1 bei einem Fehler
int BmpDecoder::ReadBIT8(unsigned uWidth,unsigned uHeight,BitmapImage &sImage)
{
unsigned char	aPads[4];
unsigned		uBitWidth;
unsigned		uRowWidth;
unsigned		uRowSize;
unsigned		uPadSize;
unsigned		uIndex;
unsigned		uVal;
unsigned		uPos;




	uBitWidth =  m_uBitCount * uWidth;
	uRowWidth = (uBitWidth+7) /  8;
	uRowSize  = (uRowWidth+3) & ~3;
	uPadSize  =  uRowSize -  uRowWidth;

	for(uPos=0;uPos<uHeight;uPos++)							// Read in each uRow.
		{
		CallProgressFunction(uPos*100/uHeight);

		   uIndex = (uHeight-uPos-1) ;
		   uVal = m_pBuffer->Read(sImage[uIndex], uRowWidth) ;
		if(uVal!=uRowWidth)return -1;

		m_pBuffer->Read(aPads,uPadSize);					// Skip over the aPads bytes.
		}


return 0;
}


//*****************************************************************************
//*
//*		ReadRLE8
//*
//*****************************************************************************
//	Liest nach dem RLE8-Format ein Bild ein
//	Ergibt 0 wenn ok bzw. -1 bei einem Fehler
int BmpDecoder::ReadRLE8(unsigned uWidth,unsigned uHeight,BitmapImage &sImage)
{
bool        bDone;
RleCode		sCode;
unsigned	uPos;
unsigned	uRow;
unsigned	uCol;



	bDone	= false;
	uRow  	= uHeight-1;								// Current row
	uCol  	= 0;										// Current column


	while(!m_pBuffer->Eof() && !bDone)
		{
		CallProgressFunction((uHeight-uRow-1)*100/uHeight);

		m_pBuffer->Read(&sCode,sizeof(sCode));

		if(sCode.ucCnt==0)
			{
			switch(sCode.ucCmd)							// A byte count of zero means that this is a special instruction.
				{
			case 0: uRow -= 1;uCol = 0;		break;		// Nchste Reihe
			case 1: bDone = true ;			break;		// Bildende
			case 2: uCol += m_pBuffer->ReadByte();		// Springe zu eine neuen Position
					uRow -= m_pBuffer->ReadByte();
					break;
			default:									// Store absolute data. The command code is the number of absolute bytes to store.
					if(uRow>=uHeight || uCol+sCode.ucCmd>uWidth)
						{
						return -1;
						}

					for(uPos=0;uPos<sCode.ucCmd;uPos++)
						{
						sImage [uRow][uCol] = m_pBuffer->ReadByte();
						uCol++;
						}
					
					if(sCode.ucCmd&1)					// An odd number of bytes is followed by a aPads byte.
						{
						m_pBuffer->ReadByte();
						}
					
					break;
				}
			}
		else{											// Store a run of the same color value.
			if(uRow>=uHeight || uCol+sCode.ucCnt>uWidth)
				{
				return -1;
				}

			for(uPos=0;uPos<sCode.ucCnt;uPos++)
				{
				sImage[uRow][uCol] = sCode.ucCmd;
				uCol++;
				}
			}
		}


return (bDone)? 0:-1;
}

//*****************************************************************************
//*
//*		ReadRLE4
//*
//*****************************************************************************
//	Liest nach dem RLE4-Format ein Bild ein
//	Ergibt 0 wenn ok bzw. -1 bei einem Fehler
int BmpDecoder::ReadRLE4(unsigned uWidth,unsigned uHeight,BitmapImage &sImage)
{
bool			bDone;
RleCode			sCode;
unsigned char	ucData;
unsigned char	ucHigh;
unsigned char	ucLow;
unsigned		uPos;
unsigned		uRow;
unsigned		uCol;


	bDone	= false ;
	uRow 	= uHeight-1;
	uCol 	= 0 ;


	while(!m_pBuffer->Eof() && !bDone)
		{
		CallProgressFunction((uHeight-uRow-1)*100/uHeight);


		m_pBuffer->Read(&sCode,sizeof(sCode));

		if(sCode.ucCnt==0)									// Spezial Code
			{
			switch(sCode.ucCmd)
					{
			case 0:	uRow--;uCol=0;	break;					// Nchste Reihe
			case 1: bDone = true ;	break;					// Bildende
			case 2:	
					uCol += m_pBuffer->ReadByte();			// Springe zu eine neuen Position
					uRow -= m_pBuffer->ReadByte();
					break;
			default:
					if(uRow>=uHeight || uCol+sCode.ucCmd>uWidth)
						{
						return -1;
						}

					for(uPos=0;uPos<sCode.ucCmd;uPos++)		// Verschiedene Pixel ausgeben
						{
						if((uPos&1)==0)
							{
							ucData	= m_pBuffer->ReadByte();
							ucHigh	= (ucData&0xF0)>>4;
							ucLow	=  ucData&0x0F;
							}

						if(uCol&1)
							{	
							if(uPos&1)sImage[uRow][uCol/2] |= ucLow;
							else      sImage[uRow][uCol/2] |= ucHigh;		
							}
						else{	
							if(uPos&1)sImage[uRow][uCol/2]  = ucLow <<4;
							else      sImage[uRow][uCol/2]  = ucHigh<<4; 
							}

						uCol++;
						}
															
					if(((sCode.ucCmd-1)&3)<2)				// Auf WORD-Grenze springen
						{
						m_pBuffer->ReadByte();	
						}
						
					break;
					}
			}
		else{												// Fixe Farbpaare ausgeben
			ucHigh = sCode.ucCmd>> 4;
			ucLow  = sCode.ucCmd&0x0F;

			if(uRow>=uHeight || uCol+sCode.ucCnt>uWidth)
				{
				return -1;
				}

			for(uPos=0;uPos<sCode.ucCnt;uPos++)
				{
				if(uCol&1)
					{	
					if(uPos&1)sImage[uRow][uCol/2] |= ucLow;
					else      sImage[uRow][uCol/2] |= ucHigh;	  
					}
				else{	
					if(uPos&1)sImage[uRow][uCol/2]  = ucLow <<4;
					else      sImage[uRow][uCol/2]  = ucHigh<<4;  
					}
				
				uCol++;
				}
			}
		}


return  (bDone)? 0:-1;
}

//*****************************************************************************
//*
//*		Read24
//*
//*****************************************************************************
//	Liest nach dem 24Bit-Format ein Bild ein (Byte 0=Blau 1=Grn 2=Rot)
//	Ergibt 0 wenn ok bzw. -1 bei einem Fehler
int BmpDecoder::Read24(unsigned uWidth,unsigned uHeight,BitmapImage &sImage)
{
unsigned char	aPads[4];
unsigned		uRowSize;
unsigned		uPadSize;
unsigned		uIndex;
unsigned		uLen;
unsigned		uRow;


	uLen		=  uWidth*3;
	uRowSize	= (uWidth*3+3)&~3;
	uPadSize	=  uRowSize-uWidth*3;


	for(uRow=0;uRow<uHeight;uRow++)
		{
		CallProgressFunction(uRow*100/uHeight);

		uIndex	= uHeight-uRow-1;

		if(m_pBuffer->Read(sImage[uIndex],uLen)!=(int)uLen    )return -1;
		if(m_pBuffer->Read(aPads, uPadSize    )!=(int)uPadSize)return -1;
		}


return 0;
}

//*****************************************************************************
//*
//*		Read32
//*
//*****************************************************************************
//	Liest nach dem 32Bit-Format ein Bild ein (Byte 0=Blau 1=Grn 2=Rot 3=Alpha)
//	Ergibt 0 wenn ok bzw. -1 bei einem Fehler
int BmpDecoder::Read32(unsigned uWidth,unsigned uHeight,BitmapImage &sImage)
{
unsigned		uIndex;
unsigned		uLen;
unsigned		uRow;


	uLen = uWidth*4;

	for(uRow=0;uRow<uHeight;uRow++)
		{
		CallProgressFunction(uRow*100/uHeight);

		uIndex	= uHeight-uRow-1;

		if(m_pBuffer->Read(sImage[uIndex],uLen)!=(int)uLen)return -1;
		}


return 0;
}
