//*****************************************************************************
//*
//*
//*		BmpEncoder.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"BmpEncoder.h"
#include	"..\FileBuffer.h"


static const unsigned short  usSignature = 'B'|('M'<<8);

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
BmpEncoder::BmpEncoder()
{
	Initialize();
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
BmpEncoder::~BmpEncoder()
{
}


//*****************************************************************************
//*
//*		Initialize
//*
//*****************************************************************************
void BmpEncoder::Initialize()
{
	m_iMeterX	= 0;								// Pixel per Meter
	m_iMeterY	= 0;								//		*
	m_iFormat	= 0;								// Format fr das Speichern
	m_iModeBits	= 0;								// Bit0=Color compress
}

//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
//	Speichert das Bild ab.
//	pName		: Dateiname der Bilddatei
//  sImage		: Klasse in der das Bild gespeichert wird
//	iMode		: Diverse Modus-Bits
//					Bit0 = Alpha-Kanal speichern (0xFF=sichtbar)
void BmpEncoder::WriteImage(const char *pName,const BitmapImage &sImage,int iMode)
{

	if((iMode&1) && sImage.BitCount()==32)m_iFormat=32;

	WriteImage(pName,sImage);

}

void BmpEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage,int iMode)
{

	if((iMode&1) && sImage.BitCount()==32)m_iFormat=32;

	WriteImage(pName,sImage);

}

void BmpEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage,int iMode)
{

	if((iMode&1) && sImage.BitCount()==32)m_iFormat=32;

	WriteImage(pFile,sImage);

}

//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
//	Speichert das Bild ab.
//	pName		: Dateiname der Bilddatei
//  sImage		: Klasse in der das Bild gespeichert wird
void BmpEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EBmpFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void BmpEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EBmpFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void BmpEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
static const char	aPads[4]={0,0,0,0,};
BITMAPFILEHEADER	sFileHeader;
BITMAPINFOHEADER	sInfoHeader;
unsigned char		ucMaxColor;
unsigned			uSpaceNeed;
unsigned			uColorSize;
unsigned			uDataSize;
unsigned			uOutWidth;
unsigned			uBitWidth;
unsigned			uOutBits;
unsigned			uPadSize;
unsigned			uColors;
unsigned			uIndex;
unsigned			uValue;
unsigned			uPosY;
unsigned			uPosX;
unsigned			uPos;
unsigned char	   *pPtr;
int					iAnz;


	
	if(!sImage.IsOk())
		{
		throw EBmpDataError();
		}

	switch(sImage.BitCount())
	  {
	  case 1:  
	  case 4:  
	  case 8:	if(m_iFormat==0){uOutBits=sImage.BitCount();break;}
	  case 12: 
	  case 15: 
	  case 16: 
	  case 24: 
	  case 32:	uOutBits=(m_iFormat)? m_iFormat:24;break;
	  default:	throw EBmpNotSupported();
	  }


	sFileHeader.bfType			= usSignature;
	sFileHeader.bfSize			= 0;
	sFileHeader.bfReserved1		= 0;
	sFileHeader.bfReserved2		= 0;
	sFileHeader.bfOffBits		= 0;

	sInfoHeader.biSize		   	= sizeof(BITMAPINFOHEADER);
	sInfoHeader.biWidth		   	= sImage.Width();
	sInfoHeader.biHeight	   	= sImage.Height();
	sInfoHeader.biPlanes	   	= 1;
	sInfoHeader.biBitCount	   	= 0;
	sInfoHeader.biCompression	= BI_RGB;
	sInfoHeader.biSizeImage	   	= 0;
	sInfoHeader.biXPelsPerMeter = m_iMeterX;
	sInfoHeader.biYPelsPerMeter = m_iMeterY;
	sInfoHeader.biClrUsed	   	= 0;
	sInfoHeader.biClrImportant  = 0;

	uColors		= ((uOutBits<=8)? sImage.ColorCount():0);
	uBitWidth   = ((uOutBits   ) *sImage.Width());
	uOutWidth	= ((uBitWidth+7)/ 8);						
	uPadSize	= ((uOutWidth+3)&~3)-uOutWidth;
	uDataSize	= sImage.Height() * (uOutWidth+uPadSize);

	if((m_iModeBits&1) && uOutBits==8)					// Nur die bentigen Farben speichern
		{
		ucMaxColor	= 1;
		pPtr		= sImage.ImageData(); 
		
		for(uPosY=sInfoHeader.biHeight;uPosY>0;uPosY--,pPtr+=uPadSize)
		for(uPosX=sInfoHeader.biWidth;uPosX>0;uPosX--)
			{
			if(ucMaxColor<*pPtr)ucMaxColor=*pPtr;
			pPtr++;
			}

		if(ucMaxColor<255)								// Weniger als 256 Farben
			{
			uColors						= ucMaxColor+1;	
			sInfoHeader.biClrUsed	   	= uColors;
			sInfoHeader.biClrImportant  = uColors;
			}		
		}

	uColorSize  = uColors *sizeof(RGBQUAD);
	uSpaceNeed	= sizeof(BITMAPFILEHEADER);
	uSpaceNeed += sizeof(BITMAPINFOHEADER);
	uSpaceNeed += uColorSize + uDataSize;

	sFileHeader.bfOffBits	= sizeof(BITMAPFILEHEADER);	// Fill in the remaining header fields.
	sFileHeader.bfOffBits  += sizeof(BITMAPINFOHEADER);
	sFileHeader.bfOffBits  += uColorSize;
	sFileHeader.bfSize		= uSpaceNeed;
	sInfoHeader.biBitCount	= uOutBits;



//******************** Datei Header schreiben *********************************


	   iAnz =pFile->Write(&sFileHeader,sizeof(BITMAPFILEHEADER  ));
	if(iAnz!=sizeof(BITMAPFILEHEADER))throw   EBmpFileWriteError();
	   iAnz =pFile->Write(&sInfoHeader,sizeof(BITMAPINFOHEADER  ));
	if(iAnz!=sizeof(BITMAPINFOHEADER))throw   EBmpFileWriteError();

	for(uPos=0;uPos<uColors;uPos++)
		{
		uValue = sImage.GetColor(uPos);
		pFile->Write(&uValue,4);
		}

	
	CallProgressFunction(0);

	if(uOutBits<=8)
		{
		for(uPos=0;uPos<sImage.Height();uPos++)
			{
			CallProgressFunction(uPos*100/sImage.Height());
			
			uIndex	= sImage.Height()-uPos-1;
			pPtr	= sImage.GetRow(uIndex);

			if(pFile->Write(pPtr ,uOutWidth)!=(int)uOutWidth)throw EBmpFileWriteError();
			if(pFile->Write(aPads,uPadSize )!=(int)uPadSize )throw EBmpFileWriteError();
			}
		}
	else if(uOutBits==32)
		{
		for(uPos=0;uPos<sImage.Height();uPos++)
			{
			CallProgressFunction(uPos*100/sImage.Height());

			uIndex	= sImage.Height()-uPos-1;
			pPtr	= sImage.GetRow4(uIndex);

			if(pFile->Write(pPtr,uOutWidth)!=(int)uOutWidth)throw EBmpFileWriteError();
			}
		}
	else{
		for(uPos=0;uPos<sImage.Height();uPos++)
			{
			CallProgressFunction(uPos*100/sImage.Height());

			uIndex	= sImage.Height()-uPos-1;
			pPtr	= sImage.GetRow3(uIndex);

			if(pFile->Write(pPtr,uOutWidth)!=(int)uOutWidth)throw EBmpFileWriteError();
			if(pFile->Write(aPads,uPadSize)!=(int)uPadSize )throw EBmpFileWriteError();
			}
		}

	CallProgressFunction (100);
}


//*****************************************************************************
//*
//*		SetPixelPerMeter
//*
//*****************************************************************************
void BmpEncoder::SetPixelPerMeter(int iMeterX,int iMeterY)
{
	m_iMeterX = iMeterX;								// Pixel per Meter
	m_iMeterY = iMeterY;								//		*
}

//*****************************************************************************
//*
//*		SetFormat
//*
//*****************************************************************************
//	Setzt das Format fr das Speichern  
//	iFormat		: Ist das Format (0,24 oder 32)
//	Ergibt das eingestellte Format
int BmpEncoder::SetFormat(int iFormat)
{

	if(iFormat!=24 && iFormat!=32)
		{
		m_iFormat = 0;
		}
	else{
		m_iFormat = iFormat;
		}


return m_iFormat;
}

//*****************************************************************************
//*
//*		SetMode
//*
//*****************************************************************************
//	Stellt diverse Modusbits ein 
//	iMode		: Sind die neuen Bits
//					Bit0 = Nur bentige Farben speichern
//	iMask		: Welche bis sollen verndert werden
//	Ergibt den eingestellten Modus.
int BmpEncoder::SetMode(int iMode,int iMask)
{

	m_iModeBits &= ~iMask;
	m_iModeBits |=  iMask&iMode;


return m_iModeBits;
}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void BmpEncoder::CallProgressFunction (unsigned int percent)
{
bool	bCancel;


	if(pProgressProc == NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,percent,bCancel);

	if(bCancel)throw EGraphicsAbort();

}
