//*****************************************************************************
//*
//*
//*		EpsDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright (c) 2002 AZ Software
//	All rights reserved.
//	Author:  Anton Zechner
//
#if		    _MSC_VER > 1299
#ifndef		 WINVER
#define		 WINVER 	0x0501
#endif
#endif

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif 	

#include	"EpsDecoder.h"
#include	"TiffDecoder.h"




//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
EpsDecoder::EpsDecoder ()
{
	m_iMeterX = 2835;								// 72 DPI
	m_iMeterY = 2835;								// 72 DPI
}


//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
EpsDecoder::~EpsDecoder ()
{
}



//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
//
//  Description:
//
//    This function calls the progress function if it has been
//    defined.
//
//  Parameters:
//    uPercent:  The completion percentage (0..100)
//
void EpsDecoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;
	if(uPercent>100)uPercent=100;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel) ;

	if(bCancel)throw EGraphicsAbort();

}

//*****************************************************************************
//*
//*		GetPixelPerMeter
//*
//*****************************************************************************
void EpsDecoder::GetPixelPerMeter(int &iMeterX,int &iMeterY)
{

	iMeterX = m_iMeterX;					// Pixel per Meter
	iMeterY = m_iMeterY;					//		*

}


//*****************************************************************************
//*
//*		CheckImageMode
//*
//*****************************************************************************
int EpsDecoder::CheckImageMode(const char *pMem,int *pValues)
{
int		iPos;


	pValues[4] = 0;

	for(iPos=0;;iPos++)
		{
		if(pMem[iPos]==' ' )continue;
		if(pMem[iPos]=='\t')continue;
		if(pMem[iPos]=='\n')continue;
		if(pMem[iPos]=='\r')continue;
		break;
		}
	
	if(strncmp(pMem+iPos,"false",5)==0)
		{
		iPos      += 5; 
		pValues[3] = 1;
		pValues[4] = atoi(pMem+iPos);

		for(;;iPos++)
			{
			if(pMem[iPos]>='0' )
			if(pMem[iPos]<='9' )continue;
			if(pMem[iPos]==' ' )continue;
			if(pMem[iPos]=='\t')continue;
			if(pMem[iPos]=='\n')continue;
			if(pMem[iPos]=='\r')continue;
			break;
			}
		
		if(strncmp(pMem+iPos,"colorimage",10))
			{
			return iPos;
			}
		
		iPos += 10;
		}
	else if(strncmp(pMem+iPos,"true",4)==0)
		{
		iPos      += 4;
		pValues[3] = 0;
		pValues[4] = atoi(pMem+iPos);

		for(;;iPos++)
			{
			if(pMem[iPos]>='0' )
			if(pMem[iPos]<='9' )continue;
			if(pMem[iPos]==' ' )continue;
			if(pMem[iPos]=='\t')continue;
			if(pMem[iPos]=='\n')continue;
			if(pMem[iPos]=='\r')continue;
			break;
			}
		
		if(strncmp(pMem+iPos,"colorimage",10))
			{
			return iPos;
			}
		
		iPos += 10;
		}
	else if(strncmp(pMem+iPos,"image",4)==0)
		{
		pValues[3] = 0;
		pValues[4] = 1;
		iPos      += 4;
		}


return iPos;
}

//*****************************************************************************
//*
//*		CheckImageSize
//*
//*****************************************************************************
int EpsDecoder::CheckImageSize(const char *pMem,int *pValues)
{
int		iFrom;
int		iPos;
int		iNum;


	for(iPos=0;;iPos++)
		{
		if(pMem[iPos]==' ' )continue;
		if(pMem[iPos]=='\t')continue;
		break;
		}
											
	if(pMem[iPos]<'0' || pMem[iPos]>'9')			// Suche "X Y Bit [a b c d e f]
		{
		return 0;
		}

	pValues[0] = atoi(pMem+iPos);

	for(iNum=0;;iPos++)
		{
		if(pMem[iPos]==' ' )continue;
		if(pMem[iPos]=='\t')continue;
		if(pMem[iPos]< '0' )break;
		if(pMem[iPos]> '9' )break;
		
		if(pMem[iPos+1]<'0' || pMem[iPos+1]>'9')
			{
			iNum++;
			pValues[iNum] = atoi(pMem+iPos+1);
			}
		}
	
	if(iNum!=3)return 0;

	for(;;iPos++)
		{
		if(pMem[iPos]==' ' )continue;
		if(pMem[iPos]=='\t')continue;
		if(pMem[iPos]=='\n')continue;
		if(pMem[iPos]=='\r')continue;
		break;
		}

	if(pMem[iPos]!='[')return 0;
	
	for(iNum=0,iPos++;;iPos++)
		{
		if(pMem[iPos]==' ' )continue;
		if(pMem[iPos]=='\t')continue;
		if(pMem[iPos]!='-' )
		if(pMem[iPos]< '0' )break;
		if(pMem[iPos]> '9' )break;
		
		if(pMem[iPos+1]<'0' || pMem[iPos+1]>'9')
			{
			iNum++;
			}
		}

	if(iNum!=6 || pMem[iPos]!=']')return 0;

	for(iPos++;;iPos++)
		{
		if(pMem[iPos]==' ' )continue;
		if(pMem[iPos]=='\t')continue;
		if(pMem[iPos]=='\n')continue;
		if(pMem[iPos]=='\r')continue;
		break;
		}

	iFrom = iPos;
	iNum  = 0;						
													
	while(pMem[iPos]=='{' && pMem[iPos+1]!='<')		// berspringe "{.....}"	
		{
		for(iNum=0;;iPos++)
			{
			if(pMem[iPos]== 0 )break;
			if(pMem[iPos]=='}')
				{
				if(iNum>=6)return 0;
				iNum++;
				iPos++;
				break;
				}
			if(iPos-iFrom>256 )break;
			}

		for(;;iPos++)
			{
			if(pMem[iPos]==' ' )continue;
			if(pMem[iPos]=='\t')continue;
			if(pMem[iPos]=='\n')continue;
			if(pMem[iPos]=='\r')continue;
			break;
			}
		}
	

return iPos;
}

//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
void EpsDecoder::ReadImage(const char *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;


	if(!sBuffer.Open(pName))
		{
		throw EEpsFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void EpsDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;


	if(!sBuffer.Open(pName))
		{
		throw EEpsFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void EpsDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage)
{
char		   	cHeader[32],cChar,*pMem;
int				iPos,iLen,iMmX,iMmY,iSeek;
int				iNull1,iNull2,iAdd;
int				aValues[5];


	iAdd	 	= 0;
	m_pInput 	= pFile;
	m_pImage	= &sImage;
	m_iFileEnd	= m_pInput->GetLength();


	if(m_pInput->Read(cHeader,32)!=32)				// Die ersten 4 Bytes 
		{
		throw EEpsFileReadError();
		}


	if(!memcmp(cHeader,"\xC5\xD0\xD3\xC6",4))		// Spezial Header
		{
		   iPos=*(int*)(cHeader+20);
		if(iPos!=0)									// Ist Tiff-Image vorhanden
			{
			TiffDecoder	sTiff;
			m_pInput->Seek(iPos);
			sTiff.ReadImage(m_pInput,sImage);
			return;
			}

		iPos       =        *(int*)(cHeader+4);
		m_iFileEnd = iPos + *(int*)(cHeader+8);
		m_pInput->Seek(iPos);
		}
	else if(cHeader[0]!='%' || cHeader[1]!='!')		// "%!...."
		{
		throw EEpsNotAEpsFile();
		}
	else{
		m_pInput->Seek(0);
		}



	for(;;)											// Suche Bitmap
		{
		   m_pInput->BufferFill(iAdd,&pMem);
		if(m_pInput->Tell()>=m_iFileEnd)break;

		iLen = (int)strcspn(pMem     ,"\r\n");
		iLen+= (int)strspn (pMem+iLen,"\r\n");
		iAdd = iLen;

		for(;iLen>14;iLen--)						// Leerzeichen am Ende
			{
			   cChar =  pMem[iLen-1];
			if(cChar == ' ' )continue;
			if(cChar == '\t')continue;
			if(cChar == '\n')continue;
			if(cChar == '\r')continue;
			break;
			}

		if(iLen<16 || memcmp(pMem+iLen-4,"@BMP",4))
			{
				iPos = CheckImageSize(pMem,aValues);
			if(!iPos)continue;

			if(pMem[iPos]=='{' && pMem[iPos+1]=='<') 
				{
				iPos  += 2;
				iSeek  = iPos;
				iSeek += pFile->Tell();

				for(;;)								// Suche Ende von Bitmap-Daten
					{
					   m_pInput->Seek(iPos,SEEK_CUR);
					if(m_pInput->Tell()>=m_iFileEnd)break;

					pMem = m_pInput->Buffer();

					for(iPos=0;;iPos++)
						{
						if(pMem[iPos  ]=='>' )
						if(pMem[iPos+1]=='}' ){       break;}
						if(pMem[iPos  ]=='\n'){iPos++;break;}
						}

					if(pMem[iPos]=='>' )break;					
					}

				if(pMem[iPos]!='>' || pMem[iPos+1]!='}')
					{
					throw EEpsCorruptFile();
					}

				iPos  += 2;
				}
			else{
				iSeek  = 0;
				}
			
			iPos += CheckImageMode(pMem+iPos,aValues);
			if(!aValues[4])continue;

			if(iSeek)pFile->Seek(iSeek);
			else  	 pFile->Seek(iPos,SEEK_CUR);

			m_iSizeX = aValues[0];
			m_iSizeY = aValues[1];
			m_iBits  = aValues[2];
			m_iInter =!aValues[3];
			m_iPlane = aValues[4];
			m_iInvers= 0;

			Read();

			return;
			}

		iMmX		= 0;
		iMmY		= 0;
		m_iBits		= 0;
		m_iPlane	= 0;
		m_iSizeX	= 0;
		m_iSizeY	= 0;
		m_iInter	= 0;
		m_iInvers	= 1;

		sscanf(pMem,"%i %i %i %i %i %i %i %i",	&m_iSizeX,&m_iSizeY,
												&m_iBits ,&m_iPlane,
												&iNull1  ,&iNull2,
												&iMmX    ,&iMmY);

		if(m_iSizeX<=0 || m_iSizeY<=0 || m_iBits<=0 || m_iBits>8)
			{
			throw EEpsCorruptFile();
			}

		if(iMmY<0)iMmY=-iMmY;
		if(iMmX  )m_iMeterX	= (int)((double)m_iSizeX/(double)iMmX*1.0e6);
		if(iMmY  )m_iMeterY	= (int)((double)m_iSizeY/(double)iMmY*1.0e6);

		m_pInput->Seek(iAdd,SEEK_SET);
		pMem = m_pInput->Buffer();
		iAdd = 0;

		if(m_iBits==8 && m_iPlane==3){Read();return;}
		if(m_iBits==8 && m_iPlane==1){Read();return;}
		if(m_iBits==1 && m_iPlane==1){Read();return;}

		throw EEpsNotSupported();
		}

	throw EEpsCorruptFile();
}




//*****************************************************************************
//*
//*		Read
//*
//*****************************************************************************
//	List die Bilddaten ein
void EpsDecoder::Read()
{
unsigned char  *pRow;
unsigned char  *pLine,ucByte;
char		   *pMem ,cChar;
int				iPosY,iPosX;
int				iLineSize;
int				iOffset;
int				iSwap;
int				iNext;
int				iAdd;
int				iPos;
int				iLen;

	iAdd	= 0;
	iSwap	= 0;

	if(!m_pImage->SetSize(m_iBits*m_iPlane,m_iSizeX,m_iSizeY))
		{
		throw EEpsUnknownError();
		}

	if(m_iPlane!=3)m_iInter=0;
	if(m_iPlane==3 && m_iBits==8 && m_iInter==0)iSwap=1;
	if(m_iPlane==1 && m_iBits==8)						// Monochrom Pallete
		{
		for(iPos=0;iPos<256;iPos++)
			{
			m_pImage->SetColor(iPos,iPos,iPos,iPos);
			}
		}


	if(m_iInvers)
		{
		iOffset	= m_iSizeY-1;
		iNext	= -1;	
		}
	else{
		iOffset	= 0;
		iNext	= 1;	
		}

	iLineSize	= (m_iSizeX*m_iBits*m_iPlane+7)/8;
	pLine		= new unsigned char[iLineSize*2+16];



	for(iPosY=0;iPosY<m_iSizeY;iPosY++)					// Bildzeilen laden
		{												
		iPosX=0;

		for(;;)
			{
			iLen = m_pInput->BufferFill(iAdd,&pMem,512);

			if(m_pInput->Tell()>=m_iFileEnd)break;
			
			iAdd = 0;

			for(iPos=0;iPos+4<iLen;)
				{
				iAdd++;
				cChar=pMem[iPos];
				iPos++;

				if(cChar=='\n')continue;
				if(cChar=='\r')continue;
				if(cChar=='\t')continue;
				if(cChar==' ' )continue;

				if(cChar<'0' || cChar>'f'  ||
				  (cChar<'A' && cChar>'F') ||
				  (cChar<'a' && cChar>'f'))
					{
					delete pLine;
					throw EEpsCorruptFile();
					}

					 if(cChar <= '9' )ucByte=cChar-'0';
				else if(cChar <= 'F' )ucByte=cChar-'A'+10;
				else				  ucByte=cChar-'a'+10;

				iAdd++;
				cChar=pMem[iPos];
				iPos++;
				ucByte<<=4;

				if(cChar=='\n')continue;
				if(cChar=='\r')continue;
				if(cChar=='\t')continue;
				if(cChar==' ' )continue;

				if(cChar<'0' || cChar>'f'  ||
				  (cChar<'A' && cChar>'F') ||
				  (cChar<'a' && cChar>'f'))
					{
					delete pLine;
					throw EEpsCorruptFile();
					}

					 if(cChar <= '9' )ucByte+=cChar-'0';
				else if(cChar <= 'F' )ucByte+=cChar-'A'+10;
				else				  ucByte+=cChar-'a'+10;

				pLine[iPosX]=ucByte;
				iPosX++;

				if(iPosX>=iLineSize)break;
				}

			if(iPosX>=iLineSize)break;
			}

		

		CallProgressFunction((100*iPosY)/m_iSizeY);

		if(m_iInter)									// Kommen die Farben Zeilenweise
			{
			pRow = pLine+iLineSize;
			
			for(iPos=0;iPos<m_iSizeX;iPos++)
				{
				pRow[iPos*3+2] = pLine[iPos           ];
				pRow[iPos*3+1] = pLine[iPos+m_iSizeX  ];
				pRow[iPos*3+0] = pLine[iPos+m_iSizeX*2];
				}
			}
		else{
			pRow = pLine;
			if(iSwap)m_pImage->SwapRGB3(pLine,m_iSizeX);
			}

		
		memcpy(m_pImage->GetRow(iOffset),pRow,iLineSize);
		iOffset+= iNext;
		}

	
	delete pLine;

	CallProgressFunction(100);


}



