//*****************************************************************************
//*
//*		EpsEncoder.cpp
//*
//*****************************************************************************
//
// Copyright (c) 2011 AZ Software
// All rights reserved.
// Author:  Anton Zechner
//
#include "EpsEncoder.h"

static	char *pEpsHeader1=
	{
	"%%!PS-Adobe-3.0 EPSF-3.0\n"
	"%%%%LanguageLevel: 2\n"
	"%%%%Creator: AZ-Tools\n"
	"%%%%Pages: 1\n"
	"%%%%BoundingBox: %i %i %i %i\n"
	"%%%%EndComments\n"
	"%%%%BeginSetup\n"
	"/readstring {\n"
	"  currentfile exch readhexstring pop\n"
	"} bind def\n"
	"/rpicstr %i string def\n"
	"/gpicstr %i string def\n"
	"/bpicstr %i string def\n"
	"%%%%EndSetup\n"
	"%%%%Page: 1 1\n"
	"gsave\n"
	"%g %g translate\n"
	"%g %g scale\n"
	"%i %i 8\n"
	"[ %i 0 0 -%i 0 %i ]\n"
	"{ rpicstr readstring }\n"
	"{ gpicstr readstring }\n"
	"{ bpicstr readstring }\n"
	"true 3\n"
	"colorimage\n"
	};

static	char *pEpsHeader2=
	{
	"%%!PS-Adobe-3.0 EPSF-3.0\n"
	"%%%%LanguageLevel: 2\n"
	"%%%%Creator: AZ-Tools\n"
	"%%%%Pages: 1\n"
	"%%%%BoundingBox: %i %i %i %i\n"
	"%%%%EndComments\n"
	"%%%%BeginSetup\n"
	"/readstring {\n"
	"  currentfile exch readhexstring pop\n"
	"} bind def\n"
	"/picstr %i string def\n"
	"%%%%Dummy %i %i\n"
	"%%%%EndSetup\n"
	"%%%%Page: 1 1\n"
	"gsave\n"
	"%g %g translate\n"
	"%g %g scale\n"
	"%i %i 8\n"
	"[ %i 0 0 -%i 0 %i ]\n"
	"{ picstr readstring }\n"
	"image\n"
	};

static	char *pEpsHeader3=
	{
	"%%!PS-Adobe-3.0 EPSF-3.0\n"
	"%%%%LanguageLevel: 1\n"
	"%%%%Creator: AZ-Tools\n"
	"%%%%Pages: 1\n"
	"%%%%BoundingBox: %i %i %i %i\n"
	"%%%%EndComments\n"
	"%%%%BeginSetup\n"
	"/readstring {\n"
	"  currentfile exch readhexstring pop\n"
	"} bind def\n"
	"/picstr %i string def\n"
	"%%%%Dummy %i %i\n"
	"%%%%EndSetup\n"
	"%%%%Page: 1 1\n"
	"gsave\n"
	"%g %g translate\n"
	"%g %g scale\n"
	"%i %i 1\n"
	"[ %i 0 0 -%i 0 %i ]\n"
	"{ picstr readstring }\n"
	"image\n"
	};

static	char pEpsHeader4[]=
	{
	"currentdict /inputf undef\n"
	"currentdict /picstr undef\n"
	"currentdict /rpicstr undef\n"
	"currentdict /gpicstr undef\n"
	"currentdict /bpicstr undef\n"
	"grestore\n"
	"showpage\n"
	"%%Trailer\n"
	};

#define 	BOUNDING_X		595						// A4 Seite in 1/72 Zoll-Schritten
#define 	BOUNDING_Y		841						//			*
#define 	BOUNDING_FRAME	15						// Kleinster Rahmen


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
EpsEncoder::EpsEncoder ()
{
	m_iMeterX = 0;									// 72 DPI
	m_iMeterY = 0;									// 72 DPI
}


//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
EpsEncoder::~EpsEncoder ()
{
}


//*****************************************************************************
//*
//*		CallProgressFunction 
//*
//*****************************************************************************
//
//  Description:
//
//    This function calls the progress function if it has been
//    defined.
//
//  Parameters:
//    uPercent:  The completion percentage (0..100)
//
void EpsEncoder::CallProgressFunction (unsigned int uPercent)
{
bool	bCancel;


  if(pProgressProc==NULL)return;
  if(uPercent>100)uPercent=100;

  bCancel = false;
  pProgressProc(*this,pProgressData,1,1,uPercent,bCancel) ;


  if(bCancel)throw EGraphicsAbort();

}

//*****************************************************************************
//*
//*		SetPixelPerMeter
//*
//*****************************************************************************
//	Stellt die Pixel je Meter ein, bei 0 wird das Bild auf eine A4 Seite eingepass
void EpsEncoder::SetPixelPerMeter(int iMeterX,int iMeterY)
{
	m_iMeterX = iMeterX;						// Pixel per Meter
	m_iMeterY = iMeterY;						//		*
}


//*****************************************************************************
//*
//*		WriteImage 
//*
//*****************************************************************************
void EpsEncoder::WriteImage(const char *Name,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;

	if(!sBuffer.Open(Name,CFB_CREATE))throw EEpsFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void EpsEncoder::WriteImage(const wchar_t *Name,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;

	if(!sBuffer.Open(Name,CFB_CREATE))throw EEpsFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void EpsEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
unsigned char  *pLine;
const char     *pFormat;
double			dScale,dScaleX,dScaleY;
char			cBuffer[1024],aBytes[2];
int				iCol,iRow,iVal,iMode,iLineSize;
int				iStartX,iStartY,iPos,iLen;
int				iEndX,iEndY,iByte,iOff;


	m_iSizeX	= sImage.Width();
	m_iSizeY	= sImage.Height();
	m_iBits		= sImage.BitCount();
	m_iPlane	= 1;
	iMode		= 0;

	if(!m_iSizeX || !m_iSizeY)
		{
		throw EEpsDataError(); 
		}

	switch(m_iBits)									// Bildformat 
			{
	case 1:	iMode	= 1;		
			pFormat = pEpsHeader3;
			break;
	
	case 8:	for(iPos=0;iPos<256;iPos++)				
				{
				iVal = iPos+(iPos<<8)+(iPos<<16);
				if(sImage.GetColor(iPos)!=(COLORRGB)iVal)break;
				}
			
			if(iPos>=256)							// Ist es ein Graustufenbild
				{
				pFormat = pEpsHeader2;
				iMode   = 1;
				break;
				}
	
	default:m_iBits  = 8;
			m_iPlane = 3;
			pFormat  = pEpsHeader1;
			}

	iLineSize = (m_iSizeX*m_iBits*m_iPlane+7)/8;

	if(m_iMeterX>0 && m_iMeterY>0)					// Vorgegebene DPI verwenden
		{
		dScaleX  = 2834.0/(double)m_iMeterX;
		dScaleY  = 2834.0/(double)m_iMeterY;
		iEndX	 = (int)( (double)m_iSizeX*dScaleX);
		iEndY	 = (int)( (double)m_iSizeY*dScaleY);

		if(iEndX>BOUNDING_X-BOUNDING_FRAME)
			{
			iStartX	 = BOUNDING_FRAME;
			iEndX   += BOUNDING_FRAME;
			}
		else{
			iStartX	 =(BOUNDING_X-iEndX)/2;
			iEndX   += iStartX;
			}

		if(iEndY>BOUNDING_Y-BOUNDING_FRAME)
			{
			iStartY	 = BOUNDING_FRAME;
			iEndY   += BOUNDING_FRAME;
			}
		else{
			iStartY	 =(BOUNDING_Y-iEndY)/2;
			iEndY   += iStartY;
			}

		dScaleX	 = (double)m_iSizeX*dScaleX;
		dScaleY  = (double)m_iSizeY*dScaleY;
		}
	else{											// Automatisch auf A4 einpassen
		dScaleX  = (BOUNDING_X-BOUNDING_FRAME*2);
		dScaleY  = (BOUNDING_Y-BOUNDING_FRAME*2);
		dScaleX /=  m_iSizeX;
		dScaleY /=  m_iSizeY;

		if(dScaleX<dScaleY)
			{
			dScale   = dScaleX;
			iStartX  = BOUNDING_FRAME;
			iEndX    = BOUNDING_X-BOUNDING_FRAME;
			iStartY  =(BOUNDING_Y-(int)(dScale*(double)m_iSizeY))/2;
			iEndY	 = BOUNDING_Y-iStartY;
			}
		else{
			dScale   = dScaleY;
			iStartY  = BOUNDING_FRAME;
			iEndY    = BOUNDING_Y-BOUNDING_FRAME;
			iStartX  =(BOUNDING_X-(int)(dScale*(double)m_iSizeX))/2;
			iEndX	 = BOUNDING_X-iStartX;
			}

		dScaleX	 = (double)m_iSizeX*dScale;
		dScaleY  = (double)m_iSizeY*dScale;
		}
	
													// Header erstellen
	iLen = sprintf(cBuffer,pFormat,	iStartX,iStartY,iEndX,iEndY,
									iLineSize/m_iPlane,iLineSize/m_iPlane,iLineSize/m_iPlane,
									(double)iStartX+0.25,(double)iStartY+0.25,
									dScaleX,dScaleY,
									m_iSizeX,m_iSizeY,m_iSizeX,m_iSizeY,m_iSizeY);

	if(pFile->Write(cBuffer,iLen)!=iLen)
		{
		throw EEpsFileWriteError();
		}

	for(iPos=0,iRow=0;iRow<m_iSizeY;iRow++)			// Alle Zeilen speichern
		{
		if(iMode)pLine = sImage.GetRow(iRow);
		else	 pLine = sImage.GetRow3Temp(iRow);

		for(iCol=0,iOff=2;iCol<iLineSize;iCol++)
			{
			if(iMode)								// Zeile eins zu eins speichern
				{
				iByte = pLine[iCol];
				}
			else{									// Rote-Zeile dann Grne dann Blaue
				iByte = pLine[iOff];
				iOff += 3;
				
				if(iOff>=m_iSizeX*3)
					{
					iOff -= m_iSizeX*3+1;
					}
				}

			iVal      =  iByte>>4;
			aBytes[0] = (iVal<10)? '0'+iVal:'a'+iVal-10;
			iVal      =  iByte&0x0F;
			aBytes[1] = (iVal<10)? '0'+iVal:'a'+iVal-10;

			if(pFile->Write(aBytes,2)!=2)
				{
				throw EEpsFileWriteError();
				}

			   iPos++;
			if(iPos<30)continue;					// Dreisig Bytes pro Zeile
			if(pFile->Write("\n",1)!=1)
				{
				throw EEpsFileWriteError();
				}
			
			iPos = 0;
			}
		
		CallProgressFunction((100*(m_iSizeY-iRow))/m_iSizeY);
		}	
	
	if(iPos>0)										// Sind noch Bytes im Puffer
		{
		if(pFile->Write("\n",1)!=1)
			{
			throw EEpsFileWriteError();
			}
		}

	if(pFile->Write(pEpsHeader4,sizeof(pEpsHeader4))!=sizeof(pEpsHeader4))
		{
		throw EEpsFileWriteError();
		}

}




