//*****************************************************************************
//*
//*
//*		GifDecoder/GifEncoder
//*
//*
//*****************************************************************************
#ifndef __GIF_H__
#define __GIF_H__

#if _MSC_VER >= 1000
#pragma once
#endif


#include "DataType.h"

const int GifBitsUsed = 12 ; // Up to 12 bits can be used in compresson
							 // Largest number of data bytes in a data block.
const unsigned int GifInputBufferSize = 255 ;
const int GifMaxColors = 256 ;

#pragma pack(push,1)

struct GifHeader
	{
	BYTE1 file_signature [3] ;
	BYTE1 file_version	 [3] ;
	} ;




struct GifScreenDescriptor
	{
	UBYTE2   logical_screen_width ;
  	UBYTE2   logical_screen_height ;
  	unsigned global_color_table_size	: 3 ;
  	unsigned sort_flag					: 1 ;
  	unsigned color_resolution			: 3 ;
  	unsigned global_color_table_flag	: 1 ;
  	UBYTE1   background_color_index ;
  	UBYTE1   pixel_aspect_ratio ;
	};


struct GifImageDescriptor
	{
  	UBYTE2 		left_position ;
  	UBYTE2 		top_position ;
  	UBYTE2 		image_width ;
  	UBYTE2 		image_height ;
  	unsigned 	color_table_size 		: 3 ;
  	unsigned 	reserved 				: 2 ;
  	unsigned 	sort_flag 				: 1 ;
  	unsigned 	interlace_flag			: 1 ;
  	unsigned 	local_color_table_flag 	: 1 ;
	};


struct GifColorTableEntry
	{
  	UBYTE1 red ;
  	UBYTE1 green ;
  	UBYTE1 blue ;
	} ;

struct GifGraphicControlBlock
	{
  	UBYTE1 		block_size ;
  	UBYTE1 		transparent_color_flag 	: 1 ;
  	UBYTE1		user_input_flag			: 1 ;
  	UBYTE1		disposal_method 		: 3 ;
  	UBYTE1	 	reserved 				: 3 ;
  	UBYTE2		delay_time ;
  	UBYTE1		transparent_color ;
	} ;


struct GifApplicationBlock
{
  UBYTE1 block_size ;
  UBYTE1 application_name [8] ;
  UBYTE1 authentication_code [3] ;
} ;


enum GifBlockType
{
  GifTerminator = 0x0,
  GifImageSeparator = 0x2C,
  GifExtension = 0x21,
  GifCommentExtension = 0xFE,
  GifPlainTextExtension = 0x01,
  GifApplicationExtension = 0xFF,
  GifGraphicControlExtension=0xF9,
  GifTrailer = 0x3B,
} ;


#pragma pack(pop)



#endif
