//*****************************************************************************
//*
//*
//*		GifDecoder.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//

#include "GifDecoder.h"

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif 	



#define		MAX_CODE		4097
#define		MAX_LZW_BITS    12



struct GifScreenDescriptor
	{
	unsigned short   	usLogicalScreenWidth;
  	unsigned short   	usLogicalScreenHeight;
  	unsigned char		ucGlobalColorTableSize;
  	unsigned char		ucGlobalColorTableFlag;
  	unsigned char   	ucBackgroundColorIndex;
  	unsigned char   	ucPixelAspectRatio;
  	unsigned char		ucColorResolution;
  	unsigned char		ucSortFlag;
	};

struct GifImageDescriptor
	{
  	unsigned short 		usLeftPosition;
  	unsigned short 		usTopPosition;
  	unsigned short 		uImageWidth;
  	unsigned short 		uImageHeight;
  	unsigned char 		ucColorTableSize;
  	unsigned char 		ucReserved;
  	unsigned char 		ucSortFlag;
  	unsigned char 		ucInterlaceFlag;
  	unsigned char 		ucColorTableFlag;
	};

struct GifColorTableEntry
	{
  	unsigned char		ucRed;
  	unsigned char		ucGreen;
  	unsigned char		ucBlue;
	} ;

struct GifGraphicControlBlock
	{
  	unsigned char		ucBlockSize ;
  	unsigned char		ucTransparentFlag;
  	unsigned char		ucUserInputFlag;
  	unsigned char		ucDisposalMethod;
  	unsigned char		ucReserved;
  	unsigned short		usDelayTime;
  	unsigned char		ucTransparentColor;
	} ;

enum GifBlockType
	{
	GifTerminator				 = 0x00,
	GifImageSeparator			 = 0x2C,
	GifExtension				 = 0x21,
	GifCommentExtension			 = 0xFE,
	GifPlainTextExtension		 = 0x01,
	GifApplicationExtension		 = 0xFF,
	GifGraphicControlExtension   = 0xF9,
	GifTrailer					 = 0x3B,
	};






//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
GifDecoder::GifDecoder()
{

	Initialize();

}


//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
GifDecoder::~GifDecoder()
{
	Reset();
}

//*****************************************************************************
//*
//*		Initialize
//*
//*****************************************************************************
void GifDecoder::Initialize()
{

	m_bMoreImages       = false;
	m_bMultipleImages	= false;
	m_pColorTable		= NULL;
	m_pCurrentImg       = NULL;
	m_iTransparentColor	= -1;
	m_pFile             = 0;
	m_uColorCount		= 0;
	m_uImageTop		    = 0;
	m_uImageLeft		= 0;
	m_uScreenWidth	    = 0;
	m_uScreenHeight     = 0;


}

//*****************************************************************************
//*
//*		Reset
//*
//*****************************************************************************
void GifDecoder::Reset()
{

  delete m_pColorTable;
		 m_pColorTable=NULL;

}


//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
//	Ld ein Bild aus einer Datei
//	pName		: Ist der Dateiname des Bildes.
//	sImage		: In dieser Klasse wird das Bild gespeichert
void GifDecoder::ReadImage(const char *pName, BitmapImage &sImage)
{
CFileBuffer	sFile;


	if(!sFile.Open(pName))
		{
		throw EGifError("Fehler beim ffnen der Datei");
		}

	try	{
		ReadImage(&sFile,sImage);
		}
	catch(...)
		{
		sFile.Close();
		throw;
		}

	sFile.Close();

}

void GifDecoder::ReadImage(const wchar_t *pName, BitmapImage &sImage)
{
CFileBuffer	sFile;


	if(!sFile.Open(pName))
		{
		throw EGifError("Fehler beim ffnen der Datei");
		}

	try	{
		ReadImage(&sFile,sImage);
		}
	catch(...)
		{
		sFile.Close();
		throw;
		}

	sFile.Close();

}

//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
//	Ld ein Bild aus einer Datei
//	pFile		: Ist das Filepuffer Objekt aus dem das Bild geladen wird.
//	sImage		: In dieser Klasse wird das Bild gespeichert
void GifDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage)
{

	m_pFile = pFile;

	ReadStreamHeader();
	ReadToNextImage ();
	ReadNextImage   (sImage);

	if(!m_bMoreImages && !m_bMultipleImages)			// clear color table
		{
		Reset();
		}

}


//*****************************************************************************
//*
//*		ReadToNextImage
//*
//*****************************************************************************
void GifDecoder::ReadToNextImage()
{
GifGraphicControlBlock	sBlock;
unsigned char			ucBuffer[256] ;
unsigned char			ucType;
unsigned char			ucSize;
unsigned char			ucData;



	for(;;)
		{
		ucType=m_pFile->ReadByte();

		if(m_pFile->Eof())break;


		switch(ucType)
			{
		case GifImageSeparator:								// Image Descriptor

			m_bMoreImages = true;
			return;


		case GifTerminator:									// Do nothing block

			break;


		case GifExtension:									// Extension introducer

			m_pFile->Read(&ucType,1);

			switch(ucType)
				{
			case GifPlainTextExtension:						// Plain Text Label

				m_pFile->Read(&ucSize,1);

				if(ucSize!=12)
					{
					throw EGifCorruptFile();
					}

				m_pFile->Read(ucBuffer,12);
				m_pFile->Read(&ucSize ,1 );					// Read in the Plain Text Data Blocks
				m_pFile->Read(ucBuffer,ucSize);
				m_pFile->Read(&ucSize ,1 );

				break;


			case GifGraphicControlExtension:				// Graphic Control Extension

				sBlock.ucBlockSize			= m_pFile->ReadByte();
				ucData						= m_pFile->ReadByte();
				sBlock.ucTransparentFlag 	= (ucData>>0)&1;
				sBlock.ucUserInputFlag		= (ucData>>1)&1;
				sBlock.ucDisposalMethod 	= (ucData>>2)&7;
				sBlock.ucReserved 			= (ucData>>5)&7;
				sBlock.usDelayTime			= m_pFile->ReadWord();
				sBlock.ucTransparentColor	= m_pFile->ReadByte();

				m_pFile->Read(&ucData,1);

				if(ucData!=0)
					{
					throw EGifCorruptFile();
					}

				if(sBlock.ucTransparentFlag)
						m_iTransparentColor=sBlock.ucTransparentColor;
				else	m_iTransparentColor=-1;

				break;


			case GifCommentExtension:   					// Comment Extention

				m_pFile->Read(&ucSize,1);					// Read in the Comment Data Blocks

				while(ucSize>0)
					{
					if(m_pFile->Read(ucBuffer,ucSize)!=ucSize)
						{
						throw EGifCorruptFile();
						}

					ucSize=0xFF;
					m_pFile->Read(&ucSize,1);
					}

				break;


			case GifApplicationExtension:
															// These is nothing we can really do with this
				ucSize=0xFF;								// other than to skip over it.
				m_pFile->Read(&ucSize,1);

				if(ucSize!=11)
					{
					throw EGifCorruptFile();
					}

				while(ucSize>0)
					{
					if(m_pFile->Read(ucBuffer,ucSize)!=ucSize)
						{
						throw EGifCorruptFile();
						}

					ucSize=0xFF;
					m_pFile->Read(&ucSize,1);
					}
			}

			break;

		case GifTrailer:

			m_bMoreImages = false;
			return ;

		default:

			throw EGifCorruptFile();
			}
		}


	throw EGifCorruptFile();

}


//*****************************************************************************
//*
//*		ReadStreamHeader
//*
//*****************************************************************************
//   This function reads the stream ucHeader and determines
//   if this is actually a GIF stream.
void GifDecoder::ReadStreamHeader()
{
GifScreenDescriptor sDesc;
unsigned char		ucHeader[6];
unsigned char		ucValue;
unsigned			uColor;



	if(m_pFile->Read(ucHeader,6)!=6)
		{
    	throw EGifFileReadError();
		}

	if(memcmp(ucHeader,"GIF87a",6) && memcmp(ucHeader,"GIF89a",6))
		{
    	throw EGifNotAGifFile ();
		}


	sDesc.usLogicalScreenWidth   	= m_pFile->ReadWord();
	sDesc.usLogicalScreenHeight  	= m_pFile->ReadWord();
	ucValue							= m_pFile->ReadByte();
	sDesc.ucGlobalColorTableSize	= (ucValue>>0)&7;
	sDesc.ucSortFlag				= (ucValue>>3)&1;
	sDesc.ucColorResolution			= (ucValue>>4)&7;
	sDesc.ucGlobalColorTableFlag	= (ucValue>>7)&1; ;
	sDesc.ucBackgroundColorIndex 	= m_pFile->ReadByte();
	sDesc.ucPixelAspectRatio		= m_pFile->ReadByte();


	if(m_pFile->Eof())
		{
		throw EGifFileReadError();
		}


	m_uScreenWidth  = sDesc.usLogicalScreenWidth;
	m_uScreenHeight = sDesc.usLogicalScreenHeight;


	if(sDesc.ucGlobalColorTableFlag)							// If a global color table is present then read it in.
		{
    	m_uColorCount = 1 << (sDesc.ucGlobalColorTableSize+1);
    	m_pColorTable = new GifColorTableEntry[m_uColorCount];

		for(uColor=0;uColor<m_uColorCount;uColor++)
			{
			if(m_pFile->Read(m_pColorTable+uColor,3)==3)continue;
			throw EGifFileReadError();
			}
		}


}



//*****************************************************************************
//*
//*		ReadNextImage
//*
//*****************************************************************************
//	This function reads the next image from the GIF stream.
//  image: The image to read
void GifDecoder::ReadNextImage(BitmapImage &sImage)
{
GifImageDescriptor	sDesc;
GifColorTableEntry	sColor;
unsigned			uColorCount;
unsigned			uValue;
unsigned			uPos;




	if(!m_bMoreImages)
		{
		throw EGifNoMoreImages();
		}


	m_bMoreImages = false;


  	sDesc.usLeftPosition	=  m_pFile->ReadWord();		// The thing we should have read from the stream is the code for an
  	sDesc.usTopPosition		=  m_pFile->ReadWord();
  	sDesc.uImageWidth		=  m_pFile->ReadWord();
  	sDesc.uImageHeight		=  m_pFile->ReadWord();
	uValue					=  m_pFile->ReadByte();

  	sDesc.ucColorTableSize	= (uValue>>0)&7;
  	sDesc.ucReserved 		= (uValue>>3)&3;
  	sDesc.ucSortFlag 		= (uValue>>5)&1;
  	sDesc.ucInterlaceFlag	= (uValue>>6)&1;
  	sDesc.ucColorTableFlag	= (uValue>>7)&1;

	m_uImageWidth			= sDesc.uImageWidth;		// Data needed to keep track of where we are writing the bytes to.
	m_uImageHeight			= sDesc.uImageHeight;
	m_uImageTop				= sDesc.usTopPosition;
	m_uImageLeft			= sDesc.usLeftPosition;
	m_uCurrentRow			= 0;
	m_uCurrentColumn		= 0 ;
	m_bInterlaced			= (sDesc.ucInterlaceFlag)? 1:0;
	m_uInterlaceGroup		= (m_bInterlaced)? 1:0;



	if(!sDesc.ucColorTableFlag || sDesc.ucColorTableSize==0)
		{
		if(m_pColorTable==NULL)							// If there is no local color table there must be a global color table.
			{
			throw EGifCorruptFile();
			}

		uColorCount = m_uColorCount ;					// Find the number of colors and allocate the table ;
		sImage.SetSize(8,m_uImageWidth,m_uImageHeight,BI_NOT_INIT);

		for(uPos=0;uPos<uColorCount;uPos++)
			{
			sImage.ColorMap(uPos).ucRed   = m_pColorTable[uPos].ucRed;
			sImage.ColorMap(uPos).ucGreen = m_pColorTable[uPos].ucGreen;
			sImage.ColorMap(uPos).ucBlue  = m_pColorTable[uPos].ucBlue;
			}
		}
	else{
		uColorCount  = 1 << (sDesc.ucColorTableSize+1);
		sImage.SetSize(8,m_uImageWidth,m_uImageHeight,BI_NOT_INIT);

		for(uPos=0;uPos<uColorCount;uPos++)
			{
			m_pFile->Read(&sColor,3);
			sImage.ColorMap(uPos).ucRed   = sColor.ucRed;
			sImage.ColorMap(uPos).ucGreen = sColor.ucGreen;
			sImage.ColorMap(uPos).ucBlue  = sColor.ucBlue;
			}

		if(m_pColorTable)
			{
			for(;uPos<256 && uPos<m_uColorCount;uPos++)
				{
				sImage.ColorMap(uPos).ucRed   = m_pColorTable[uPos].ucRed ;
				sImage.ColorMap(uPos).ucGreen = m_pColorTable[uPos].ucGreen ;
				sImage.ColorMap(uPos).ucBlue  = m_pColorTable[uPos].ucBlue ;
				}
			}
		}


	try	{
	    m_pCurrentImg = &sImage;
		ReadCompressedData();
	    CallProgressFunction(100);
		}
	catch (EGraphicsAbort&)
		{
		}


	m_pCurrentImg = NULL ;
	ReadToNextImage ();



}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void GifDecoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;



	if(pProgressProc==NULL)return;


	bCancel = false;

	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel);

	if(bCancel)
		{
		throw EGraphicsAbort();
		}

}

//*****************************************************************************
//*
//*		OutputRow
//*
//*****************************************************************************
void GifDecoder::OutputRow(int &iRow,unsigned char *pBuffer)
{
int				iDrowStart;
int				iDrowEnd;
int				iRowShift;
int				iRowDup;
unsigned char  *pMem;


	iDrowStart		= iRow;
	iDrowEnd		= iRow;
	m_bProgressive	= 0;

	if(m_bProgressive && m_bInterlaced && m_uInterlaceGroup<4)
	    {
		iRowShift = 0;
	    iRowDup	  = 0;

		switch(m_uInterlaceGroup)
			{
	    case 1:		iRowDup=7; iRowShift=3;	break;
	    case 2:		iRowDup=3; iRowShift=1;	break;
	    case 3:		iRowDup=1; iRowShift=0;	break;
	    default:							break;
		    }

		iDrowStart -= iRowShift;
	    iDrowEnd    = iRowDup;
		iDrowEnd   += iDrowStart;

													// Extend if bottom edge isn't covered because of the shift upward.
		if(((int)m_uImageHeight-1)-iDrowEnd <= iRowShift)
			{
	        iDrowEnd = m_uImageHeight - 1;
			}

	    if(iDrowStart < 0)							// Clamp first and last rows to upper and lower edge of image.
			{
	        iDrowStart = 0;
			}

		if(iDrowEnd >= (int)m_uImageHeight)
			{
	        iDrowEnd = m_uImageHeight-1;
			}
	    }



	if((unsigned)iRow < m_uImageHeight)				// Reihe uusgeben
		{
		   pMem=(*m_pCurrentImg)[m_uImageTop+iDrowStart];
		if(pMem)memcpy(pMem,pBuffer,m_uImageWidth);
	    }

	if(!m_bInterlaced)
		{
		iRow++;
		}
	else{
	    do	{
	        switch(m_uInterlaceGroup)
				{
	        case 1:	iRow += 8;
					if((unsigned)iRow >= m_uImageHeight)
						{
						m_uInterlaceGroup++;
						iRow = 4;
						}
					break;

	        case 2: iRow += 8;
					if((unsigned)iRow >= m_uImageHeight)
						{
						m_uInterlaceGroup++;
						iRow = 2;
						}
					break;

	        case 3: iRow += 4;
					if((unsigned)iRow >= m_uImageHeight)
						{
						m_uInterlaceGroup++;
						iRow = 1;
						}
					break;

	        case 4:	iRow += 2;
					if((unsigned)iRow >= m_uImageHeight)
						{
						m_uInterlaceGroup++;
						iRow = 0;
						}
					break;

	        default:;
				}
			}while((unsigned)iRow>m_uImageHeight-1);
	    }

}

//*****************************************************************************
//*
//*		ReadCompressedData
//*
//*****************************************************************************
void GifDecoder::ReadCompressedData ()
{
unsigned char	ucFirst;
unsigned short *pPrefix;
unsigned char  *pBuffer;
unsigned char  *pSuffix;
unsigned char  *pStackp;
unsigned char  *pStack;
unsigned char  *pRowEnd;
unsigned char  *pRowP;
int				iEndCode;
int				iDecoded;
int				iClearCode;
int				iCodeSize;
int				iCodeMask;
int				iDataSize;
int				iRowsRem;
int				iOldCode;
int				iInCode;
int				iCount;
int				iStop;
int				iCode;
int				iDate;
int				iBits;
int				iExit;
int				iRow;
int				i;


	iExit	 = 0;
	iStop	 = 0;
	iDecoded = 0;

	pPrefix  = new unsigned short [MAX_CODE];
	pSuffix  = new unsigned char  [MAX_CODE];
	pStack	 = new unsigned char  [MAX_CODE];
	pBuffer	 = new unsigned char  [m_uImageWidth];


	   iDataSize = m_pFile->ReadByte();
	if(iDataSize>MAX_LZW_BITS)iStop=-1;

	iRow        = 0;
	iRowsRem	= m_uImageHeight;
	pRowEnd     = pBuffer + m_uImageWidth;
	pRowP       = pBuffer;
	pStackp     = pStack;

	iClearCode	=  1 << iDataSize;
	iEndCode	=       iClearCode+ 2;
	iOldCode    = -1;
	iCodeSize   =       iDataSize  + 1;
	iCodeMask   = (1 << iCodeSize) - 1;

	iDate		= 0;
	iBits		= 0;



	if(iClearCode >= MAX_CODE)iStop=-1;

	for(i=0;i<iClearCode;i++)
		{
		pSuffix[i] = i;
		}



//******************** Haupt-Schleife *****************************************
try {
	while(!iStop)
		{
		if(m_pFile->Eof())								// Unerwartetes Datei ende
			{
			iStop=-1;
			break;
			}

		CallProgressFunction((100*(m_uImageHeight-iRowsRem))/m_uImageHeight);

		   iCount = m_pFile->ReadByte();				// Neue Blocklnge
		if(iCount == 0)									// Bild Ende
			{
			iDecoded++;
			iStop=1;
			break;
			}

		if(iRowsRem == 0)
			{
			if(iCount)iCount=m_pFile->ReadByte();
			iStop=1;
			break;
			}

		while(iCount-- > 0)
			{
		    iDate |= m_pFile->ReadByte()<<iBits;		// Nchstes Byte laden
		    iBits += 8;

		    while(iBits >= iCodeSize)					// Genug Bits vorhanden ?
				{
			    iCode   = iCodeMask&iDate ;				// Code auslesen
		        iDate >>= iCodeSize;
		        iBits  -= iCodeSize;


		        if(iCode == iClearCode)					// Nchste Gre
					{
		            iCodeSize = iDataSize  + 1;
		            iCodeMask = (1<<iCodeSize) - 1;
		            iEndCode  = iClearCode + 2;
		            iOldCode  = -1;
		            continue;
					}

														// Stream Ende ?
		        if(iCode == iClearCode+1)
					{
					iExit=0;
					goto END;
					}

				if(iOldCode == -1)
					{
		              *pRowP++= pSuffix[iCode];
		            if(pRowP >= pRowEnd)
						{
						OutputRow(iRow,pBuffer);
							pRowP = pBuffer;
							iRowsRem--;
						if(!iRowsRem)goto END;                                                         \
						}
		            ucFirst = iOldCode = iCode;
		            continue;
					}

		        if(iCode>iEndCode)						// Falscher Code ?
					{
					iExit=-1;
					goto END;
					}


		        iInCode = iCode;

		        if(iCode==iEndCode)
					{
		            *pStackp++ = ucFirst;
		            iCode      = iOldCode;
					}

		        while(iCode>iClearCode)
					{
		            if(iCode==pPrefix[iCode])
						{
						iExit=-1;
						goto END;
						}
		            *pStackp++= pSuffix[iCode];
		               iCode  = pPrefix[iCode];

					}

		        *pStackp++ = ucFirst = pSuffix[iCode];

		        pPrefix[iEndCode] = iOldCode;
		        pSuffix[iEndCode] = ucFirst;

		           iEndCode++;						// Neue Schlange
		        if(iEndCode >= MAX_CODE)
					{
					iExit=-1;
					goto END;
					}

				if(!(iEndCode&iCodeMask) && iEndCode<4096)
					{
		            iCodeSize++;
		            iCodeMask += iEndCode;
					}

				iOldCode = iInCode;

		        do {
				   *pRowP++ = *--pStackp;

				   if(pRowP>=pRowEnd)
						{
						OutputRow(iRow,pBuffer);
						pRowP = pBuffer;
						iRowsRem--;

						if(!iRowsRem)goto END;                                                         \
						}
					}while(pStackp>pStack);
				}
			}

		END:

		if(iExit<0)
			{
			iStop=-1;
			break;
			}
		}

//******************** LWZ-Ende ***********************************************
	}
	catch(EGraphicsAbort &)
		{
		delete pStack;
		delete pSuffix;
		delete pPrefix;
		delete pBuffer;
		throw EGraphicsAbort();
		}
	catch(...)
		{
		iStop=-1;
		}

	delete pStack;
	delete pSuffix;
	delete pPrefix;
	delete pBuffer;

	if(iStop<0)
		{
		throw EGifCorruptFile();
		}

}


