//*****************************************************************************
//*
//*
//*		GifEncoder.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//

#include "GifEncoder.h"


CFileBuffer		  *m_pOut;

#define		MAXCODE(n)		((1<<(n)) - 1)



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
GifEncoder::GifEncoder ()
{
	Initialize();
}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
GifEncoder::~GifEncoder ()
{
}

//*****************************************************************************
//*
//*		Initialize
//*
//*****************************************************************************
void GifEncoder::Initialize ()
{
	m_pImage		= NULL;
	m_pImgList		= NULL;
	m_uBgColor		= 0;
	m_uImageCount	= 1;
	m_uTranparence	= 0x100;
	m_uDelay		= 50;
}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
int GifEncoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc == NULL)return 0;
	if(uPercent>100)uPercent = 100;


	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel);


return (bCancel)? 1:0;
}

//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
void GifEncoder::WriteImage(const char *pName,const BitmapImage **pImages,int iImageCount)
{
	m_pImgList		= pImages;
	m_uImageCount	= iImageCount;

	if(m_uImageCount<0 || !m_pImgList || !m_pImgList[0])
		{
		throw EGifError("Es wurden keine Bilder angegeben");
		}

	WriteImage(pName,*m_pImgList[0]);
}

void GifEncoder::WriteImage(const wchar_t *pName,const BitmapImage **pImages,int iImageCount)
{
	m_pImgList		= pImages;
	m_uImageCount	= iImageCount;

	if(m_uImageCount<0 || !m_pImgList || !m_pImgList[0])
		{
		throw EGifError("Es wurden keine Bilder angegeben");
		}

	WriteImage(pName,*m_pImgList[0]);
}

void GifEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage **pImages,int iImageCount)
{
	m_pImgList		= pImages;
	m_uImageCount	= iImageCount;

	if(m_uImageCount<0 || !m_pImgList || !m_pImgList[0])
		{
		throw EGifError("Es wurden keine Bilder angegeben");
		}

	WriteImage(pFile,*m_pImgList[0]);
}

void GifEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{
CFileBuffer		sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EGifError("Fehler beim Erzeugen der Datei");

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void GifEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{
CFileBuffer		sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EGifError("Fehler beim Erzeugen der Datei");

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void GifEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
char			cComment[240];
unsigned char	aColors[256];
BitmapImage		sNewImage;
unsigned		uHeight;
unsigned		uColors;
unsigned		uPixel;
unsigned		uValue;
unsigned		uWidth;
unsigned		uSize;
unsigned		uBits;
unsigned		uPosX;
unsigned		uPosY;
unsigned		uLen;
unsigned		uOff;
unsigned		uPos;
char		   *pPtr;




	uWidth	= sImage.Width ();
	uHeight	= sImage.Height();

	if(uWidth<=0 || uHeight<=0)										// Bild Gre Prfen
		{
		throw EGifError("Das Bils ist leer");
		}

	m_pBuffer = pFile;
																	// Ist das Bild ein 8 Bit Bild
	if(sImage.BitCount()>8)
		{
		if(!sNewImage.ConvertTo8Bit(sImage))
			{
			throw EGifError("Kann Format nicht verarbeiten");
			}

		m_pImage = &sNewImage;
		}
	else{
		m_pImage = &sImage;
		}


	if(m_uImageCount<=0)
		{
		m_uImageCount=1;
		}


//******************** Header speichern ***************************************


	m_pBuffer->Write    ("GIF89a",6);
	m_pBuffer->WriteWord(uWidth );
	m_pBuffer->WriteWord(uHeight);


	uBits	= m_pImage->BitCount();
	uColors = 1<<uBits;

	if(uColors>16)													// Faben optimieren
		{
		memset(aColors,0,sizeof(aColors));

		for(uPosY=0;uPosY<m_pImage->Height();uPosY++)
		for(uPosX=0;uPosX<m_pImage->Width ();uPosX++)
			{
			uPixel = m_pImage->GetPixel(uPosX,uPosY);
			aColors[uPixel] = 1;
			if(uPixel>=128)break;
			}

		for(uPos=uColors-1;uPos>0;uPos--)							// Suche die hchste Farbnummer
			{
			if(aColors[uPos])break;
			}

		for(uBits=1;;uBits++)
			{
			   uColors = 1<<uBits;
			if(uBits  >= 8 )break;
			if(uColors>uPos)break;
			}
		}


	uValue  = 0x80;													// Yes, there is a color map
	uValue |= (uBits-1)<<4;											// OR in the resolution
	uValue |= (uBits-1);


	m_pBuffer->WriteByte(uValue);
	m_pBuffer->WriteByte(m_uBgColor);
	m_pBuffer->WriteByte(0);


	for(uPos=0;uPos<uColors;uPos++)									// Farben speichern
		{
		unsigned char	ucRed,ucGreen,ucBlue;

		m_pImage->GetColor(uPos,ucRed,ucGreen,ucBlue);
		m_pBuffer->WriteByte(ucRed  );
		m_pBuffer->WriteByte(ucGreen);
		m_pBuffer->WriteByte(ucBlue );
		}

	if(m_uImageCount>1)												// Mehere Bilder
		{
		m_pBuffer->WriteByte(0x21);
		m_pBuffer->WriteByte(0xFF);
		m_pBuffer->WriteByte(0x0B);
		m_pBuffer->Write    ("NETSCAPE2.0");
		m_pBuffer->WriteByte(0x03);
		m_pBuffer->WriteByte(0x01);
		m_pBuffer->WriteByte(0x00);									// 0 = ewig durchlaufen
		m_pBuffer->WriteByte(0x03);									// ???
		m_pBuffer->WriteByte(0x00);
		}


	   pPtr = GetComment();
	if(pPtr)														// Kommentar speichern
		{
		for(uLen=0;*pPtr;pPtr++)
			{
			if(*pPtr&0x80)continue;
			cComment[uLen]=*pPtr;
			uLen++;
			if(uLen>=sizeof(cComment)-1)break;
			}

		if(uLen>0)
			{
			m_pBuffer->WriteByte(0x21);
			m_pBuffer->WriteByte(0xFE);
			m_pBuffer->WriteByte(uLen);
			m_pBuffer->Write    (cComment,uLen);
			m_pBuffer->WriteByte(0x00);
			}
		}

//******************** Speichere Bilder ***************************************

	m_uTickVal 	= 0;
	m_uTick		= (100<<24)/(uHeight*m_uImageCount);


	for(uPos=0;;uPos++)												// Bilder Schleife
		{
		uOff = 0;

		if(m_uTranparence<256 || m_uImageCount>1)
			{
			uValue  = (m_uTranparence<256)? 1   :0;
			uValue |= (m_uImageCount >1  )? 3<<2:0;					// Bildabspielmodus 3=durchlaufen

			m_pBuffer->WriteByte(0x21);
			m_pBuffer->WriteByte(0xF9);
			m_pBuffer->WriteByte(0x04);
			m_pBuffer->WriteByte(uValue);
			m_pBuffer->WriteWord(m_uDelay);
			m_pBuffer->WriteByte(m_uTranparence);
			m_pBuffer->WriteByte(0x00);
			}


		m_pBuffer->WriteByte(0x2C   );
		m_pBuffer->WriteWord(0x0000 );
		m_pBuffer->WriteWord(0x0000 );
		m_pBuffer->WriteWord(uWidth );
		m_pBuffer->WriteWord(uHeight);
		m_pBuffer->WriteByte(0x00   );

		   uSize =m_pImage->BitCount();
		if(uSize<=1)uSize=2;

		m_pBuffer->WriteByte(uSize);								// InitCodeSize

		Compress(uSize+1,uWidth,uHeight);
		m_pBuffer->WriteByte(0x00);

		uPos++;

		if(uPos>=m_uImageCount)
			{
			m_pBuffer->WriteByte(0x3B);								// Endtag speichern
			break;
			}

		m_pBuffer->WriteByte(0x00);									// Weiteres Bild
		m_pImage = m_pImgList[uPos];

		if(!m_pImage)
			{
			throw EGifError("Ein Bild Fehlt in der Liste");
			}

		if(m_pImage->BitCount()>8)									// In 8-Bit Convertieren
			{
			if(!sNewImage.ConvertTo8Bit(*m_pImage))
				{
				throw EGifError("Kann Format nicht verarbeiten");
				}

			m_pImage = &sNewImage;
			}
		}

	CallProgressFunction(100);

}



//*****************************************************************************
//*
//*		CharOut
//*
//*****************************************************************************
//	Gibt ein einzenes Byte in den Buffer aus
inline void GifEncoder::CharOut(int iChar)
{

	aBuffer[iBufSize++] = iChar;

	if(iBufSize>=254)
		{
		m_pBuffer->WriteByte(iBufSize);
		m_pBuffer->Write(aBuffer,iBufSize);
		iBufSize=0;
		}
}

//*****************************************************************************
//*
//*		OutputCode
//*
//*****************************************************************************
//	Gibt einen Code aus
void GifEncoder::OutputCode(int iCode)
{
static const unsigned	aBitMask[] =
		{
		0x0000, 0x0001, 0x0003, 0x0007,
		0x000F, 0x001F, 0x003F, 0x007F,
		0x00FF, 0x01FF, 0x03FF, 0x07FF,
		0x0FFF, 0x1FFF, 0x3FFF, 0x7FFF,
		0xFFFF
		};


	uCurAccu &= aBitMask[iCurBits];

	if (iCurBits > 0)
			uCurAccu |= (iCode << iCurBits);
	else	uCurAccu  = iCode;

	iCurBits += iBitCount;

	while(iCurBits >= 8)
		{
		CharOut(uCurAccu);
		uCurAccu >>= 8;
		iCurBits -= 8;
		}

	if(iFreeCode>iMaxCode || iClearFlag)
		{
		if(iClearFlag)
			{
			iMaxCode   = MAXCODE(iBitCount=iInitBits);
			iClearFlag = 0;
			}
		else{
			iBitCount++;

			if(iBitCount == iMaxBits)
					iMaxCode = iLastCode;
			else	iMaxCode = MAXCODE (iBitCount);
			}
		}

	if(iCode==iEofCode)								// At EOF, write the rest of the buffer.
		{
		while(iCurBits>0)
			{
			CharOut(uCurAccu);
			uCurAccu >>= 8;
			iCurBits  -= 8;
			}

		m_pBuffer->WriteByte(iBufSize);
		m_pBuffer->Write(aBuffer,iBufSize);
		iBufSize=0;
		}



}

//*****************************************************************************
//*
//*		ResetHash
//*
//*****************************************************************************
// reset code table
void GifEncoder::ResetHash(int iSize)
{
register int   *pHeap;
register int 	iVal;
register int 	i;


	iVal	= -1;
	i		= iSize - 16;
	pHeap	= aHashTab + iSize;


	do	{														// might use Sys V memset(3) here
		*(pHeap - 16) = iVal;
		*(pHeap - 15) = iVal;
		*(pHeap - 14) = iVal;
		*(pHeap - 13) = iVal;
		*(pHeap - 12) = iVal;
		*(pHeap - 11) = iVal;
		*(pHeap - 10) = iVal;
		*(pHeap -  9) = iVal;
		*(pHeap -  8) = iVal;
		*(pHeap -  7) = iVal;
		*(pHeap -  6) = iVal;
		*(pHeap -  5) = iVal;
		*(pHeap -  4) = iVal;
		*(pHeap -  3) = iVal;
		*(pHeap -  2) = iVal;
		*(pHeap -  1) = iVal;
		  pHeap -= 16;

		}while((i-=16) >= 0);

	for(i += 16; i > 0; --i)
		{
		*--pHeap = iVal;
		}

}

//*****************************************************************************
//*
//*		ResetBlock
//*
//*****************************************************************************
//	table clear for block compress
void GifEncoder::ResetBlock()
{
	ResetHash(GIF_HSIZE);

	iFreeCode	= iClearCode + 2;
	iClearFlag	= 1;

	OutputCode(iClearCode);
}


//*****************************************************************************
//*
//*		Compress
//*
//*****************************************************************************
void GifEncoder::Compress(int iInitCode,unsigned uWidth,unsigned uHeight)
{
int			iVal;
int			iCode;
int 		iChar;
int 		iDisp;
int 		iSize;
int 		iEntry;
int 		iShift;
unsigned	uPosX;
unsigned	uPosY;




	iMaxBits	=    12;										// user settable max # bits/code
	iLastCode	= 1<<12;										// should NEVER generate this code

	iInitBits	=  iInitCode;									// Set up the globals:  iInitBits - initial number of bits
	iCurBits	=  0;
	uCurAccu	=  0;
	iClearFlag	=  0;

	iClearCode	= (1<<(iInitCode-1));
	iEofCode 	=  iClearCode + 1;
	iFreeCode	=  iClearCode + 2;

	iBitCount	=  iInitBits;									// Had some problems here... should be okay now.	--Adam
	iMaxCode	=  MAXCODE(iBitCount);
	iBufSize	=  0;

	uPosX		= 1;
	uPosY		= 0;
	iShift		= 0;
	iEntry		= m_pImage->GetPixel(0,0);



	for(iCode=GIF_HSIZE;iCode<0x10000;iCode*=2)
		{
		iShift++;
		}

	iShift = 8 - iShift;										// set hash code range bound

	iSize = GIF_HSIZE;
	ResetHash(iSize);											// clear hash table

	OutputCode(iClearCode);



	for(;;)
		{
		if(uPosX>=uWidth)
			{
			uPosX=0;
			uPosY++;
			if(uPosY>=uHeight)break;

			m_uTickVal += m_uTick;

			if(CallProgressFunction(m_uTickVal>>24))
				{
				throw EGraphicsAbort();
				}
			}

		iChar	= m_pImage->GetPixel(uPosX,uPosY);
		uPosX  += 1;
		iCode	= ((int) iChar << iMaxBits) + iEntry;
		iVal	= ((int) iChar << iShift  ) ^ iEntry;			// xor hashing

		if(aHashTab[iVal] == iCode)
			{
			iEntry = aCodeTab[iVal];
			continue;
			}
		else if(aHashTab[iVal] < 0)								// empty slot
			{
			goto nomatch;
			}

		iDisp = iSize-iVal;										// secondary hash (after G. Knott)

		if(iVal==0)
			{
			iDisp = 1;
			}

probe:

		if((iVal -= iDisp) < 0)
			{
			iVal += iSize;
			}

		if(aHashTab[iVal]==iCode)
			{
			iEntry = aCodeTab[iVal];
			continue;
			}

		if(aHashTab[iVal] > 0)
			{
			goto probe;
			}

nomatch:

		OutputCode(iEntry);
		iEntry = iChar;

		if(iFreeCode<iLastCode)
			{
			aCodeTab[iVal] = iFreeCode++;							// code -> hashtable
			aHashTab[iVal] = iCode;
			}
		else{
			ResetBlock();
			}
		}


	OutputCode(iEntry);											   // Put out the final code.
	OutputCode(iEofCode);
}


