//*****************************************************************************
//*
//*
//*		IconDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
#include "IconDecoder.h"
#include "PngDecoder.h"

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif

#define 	 MAX_ICONS	32

//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
IconDecoder::IconDecoder ()
{
	pFileBuf	 = 0;
	ulColorTrans = 0x40C0C0;
	ulColorXor   = 0xC040C0;
	uIconFound	 = 0;
	uIconType	 = 0;
	uIconNum	 = 0;
	uIconId		 = 0;
	uColors	  	 = 0;
	uHeight	  	 = 0;
	uWidth	  	 = 0;

}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
IconDecoder::~IconDecoder ()
{
}


//*****************************************************************************
//*
//*		CompressImage
//*
//*****************************************************************************
//	Konvertiert ein Bild so das die Farben 255 und 254 unbenutzt sind
void IconDecoder::CompressImage(BitmapImage *pImage)
{
unsigned char 	r,g,b,r1,g1,b1;
unsigned		uColorCount[256];
int				iMax1,iMax2,iNr1,iNr2,iNew1,iNew2;
int				iWidth,iHeight;
int				i,j,k;




	iWidth  = pImage->Width();
	iHeight = pImage->Height();

	memset(uColorCount,0,sizeof(uColorCount));

	for(j=0;j<iHeight;j++)				// Zhle die Pixel je Farbe
	for(i=0;i<iWidth ;i++)
		{
		k=pImage->GetPixel(i,j);
		uColorCount[k&0xFF]++;
		}

	if(uColorCount[255]==0 && uColorCount[254]==0)return;

	iMax1 = uColorCount[255];
	iMax2 = uColorCount[254];
	iNr1  = 255;
	iNr2  = 254;

	if(uColorCount[255])				// Suche kleinste Farbe
	for(i=253;i>=16;i--)
		{
		if(uColorCount[i]>=(unsigned)iMax1)continue;
		iMax1=uColorCount[i];
		iNr1=i;
		}

	if(uColorCount[254])
	for(i=253;i>=16;i--)
		{
		if(uColorCount[i]>=(unsigned)iMax2)continue;
		if(i==iNr1)continue;
		iMax2=uColorCount[i];
		iNr2=i;
		}


	iNew1 = 255;
	iNew2 = 254;

	if(uColorCount[iNr1])				// Suche beste Farbe
		{
		pImage->GetColor(iNr1,r1,g1,b1);

		iNew1=255;
		iMax1=256*256*4;
		for(i=253;i>=0;i--)
			{
			if(i==iNr1)continue;
			if(i==iNr2)continue;
			pImage->GetColor(i,r,g,b);
			j =(r-r1)*(r-r1);
			j+=(g-g1)*(g-g1);
			j+=(b-b1)*(b-b1);
			if(j>=iMax1)continue;
			iNew1=i;
			iMax1=j;
			}
		}

	if(uColorCount[iNr2])				// Suche beste Farbe
		{
		pImage->GetColor(iNr1,r1,g1,b1);

		iNew2=254;
		iMax2=256*256*4;
		for(i=253;i>=0;i--)
			{
			if(i==iNr1)continue;
			if(i==iNr2)continue;
			pImage->GetColor(i,r,g,b);
			j =(r-r1)*(r-r1);
			j+=(g-g1)*(g-g1);
			j+=(b-b1)*(b-b1);
			if(j>=iMax2)continue;
			iNew2=i;
			iMax2=j;
			}
		}

	i=pImage->GetColor(255 );			// Tausche Farben
	j=pImage->GetColor(iNr1);
	  pImage->SetColor(255 ,j);
	  pImage->SetColor(iNr1,i);
	i=pImage->GetColor(254 );
	j=pImage->GetColor(iNr2);
	  pImage->SetColor(254 ,j);
	  pImage->SetColor(iNr2,i);

	for(j=0;j<iHeight;j++)				// Tausche Pixel
	for(i=0;i<iWidth ;i++)
		{
		k=pImage->GetPixel(i,j);
		if(k==255 )pImage->SetPixel(i,j,iNr1 );
		if(k==iNr1)pImage->SetPixel(i,j,iNew1);
		if(k==254 )pImage->SetPixel(i,j,iNr2 );
		if(k==iNr2)pImage->SetPixel(i,j,iNew2);
		}


}

//*****************************************************************************
//*
//*		GetInfo
//*
//*****************************************************************************
//	Liest Informationen ber alle Icons einer Datei ein.
//	pName		: Ist der Dateiname der Icon-Datei.
//	pFile		:
//	pInfo		: Hier werden die Informatione gespeichert
//	iCount		: Ist die Gre des pInfo Arrays
//	uFilter		: 1=Nir Icons  2=Nur Cursors  3=Alles
//	Ergibt die Anzahl der vorhandenen Icons oder -1 bei einem Fehler.
int IconDecoder::GetInfo(const char *pName,IconInfo *pInfo,int iCount,unsigned uFilter)
{
CFileBuffer		sBuf;
int				iRet;


	if(!sBuf.Open(pName))return -1;

	try	{
		iRet = GetInfo(&sBuf,pInfo,iCount,uFilter);
		}
	catch(...)
		{
		iRet = -1;
		}

	sBuf.Close();


return iRet;
}

int IconDecoder::GetInfo(const wchar_t *pName,IconInfo *pInfo,int iCount,unsigned uFilter)
{
CFileBuffer		sBuf;
int				iRet;


	if(!sBuf.Open(pName))return -1;

	try	{
		iRet = GetInfo(&sBuf,pInfo,iCount,uFilter);
		}
	catch(...)
		{
		iRet = -1;
		}

	sBuf.Close();


return iRet;
}

int IconDecoder::GetInfo(CFileBuffer *pFile,IconInfo *pInfo,int iCount,unsigned uFilter)
{
unsigned short	aHeader[6];
unsigned int 	uLength;


	if(!pInfo)										// Kein Pointer
		{
		pInfo	= (IconInfo*)1;
		iCount	= 0;
		}

	pFileBuf	= pFile;
	uIconFound	= 0;
	uIconNum    = 0;
	uColors	  	= 0;
	uHeight	  	= 0;
	uWidth	  	= 0;

	if(pFileBuf->Read(&aHeader,6)!=6)
		{
		return -1;
		}

	if(aHeader[0]==0x0000 && (aHeader[1]==0x0001 || aHeader[1]==0x0002) && aHeader[2]<=MAX_ICONS)
		{
		return ReadImageIco((ITEMDIR*)aHeader,0,0,iCount,pInfo);
		}


	if(aHeader[0]==0x5A4D)							// MZ Sign
		{
		   pFileBuf->Seek(27*2,SEEK_CUR);
		if(pFileBuf->Read(&uLength,4)!=4)
			{
			throw EIconError("Datei konnte nicht geffnet werden !");
			}

		if(uLength<=30*2-4)
			{
			throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
			}

		   pFileBuf->Seek(uLength-30*2-4,SEEK_CUR);
		if(pFileBuf->Read(&aHeader,4)!=4)
			{
			throw EIconError("Datei konnte nicht geffnet werden !");
			}

		if(aHeader[0]==0x4550 && aHeader[1]==0)		// PE Sign 64-Bit
			{
			return ReadImageExe(uLength,0,0,iCount|(uFilter<<24),-1,pInfo);
			}

		if(aHeader[0]==0x454E)						// NE Sign 16-Bit
			{
			return ReadImageE16(uLength,0,0,iCount|(uFilter<<24),-1,pInfo);
			}
		}


return -1;
}


//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
void IconDecoder::ReadImage(const char    *pName,BitmapImage &sImage){ReadImage(pName,&sImage,0,-1,0);}
void IconDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage){ReadImage(pName,&sImage,0,-1,0);}
void IconDecoder::ReadImage(CFileBuffer   *pFile,BitmapImage &sImage){ReadImage(pFile,&sImage,0,-1,0);}
void IconDecoder::ReadImage(const char    *pName,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr)
{
CFileBuffer		sBuf;


	uIconFound=0;

	if(!sBuf.Open(pName))throw EIconError("Datei konnte nicht geffnet werden !");

	try	{
		ReadImage(&sBuf,pImageXor,pImageAnd,iBitmapNr,iIconNr);
		}
	catch(...)
		{
		sBuf.Close();
		throw;
		}

	sBuf.Close();

}
void IconDecoder::ReadImage(const wchar_t *pName,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr)
{
CFileBuffer		sBuf;


	uIconFound=0;

	if(!sBuf.Open(pName))throw EIconError("Datei konnte nicht geffnet werden !");

	try	{
		ReadImage(&sBuf,pImageXor,pImageAnd,iBitmapNr,iIconNr);
		}
	catch(...)
		{
		sBuf.Close();
		throw;
		}

	sBuf.Close();

}

void IconDecoder::ReadImage(CFileBuffer *pFile,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr)
{
unsigned short	aHeader[6];
unsigned int 	uLength;


	pFileBuf	= pFile;
	uIconFound	= 0;
	uIconNum    = 0;
	uColors	  	= 0;
	uHeight	  	= 0;
	uWidth	  	= 0;

	if(pFileBuf->Read(&aHeader,6)!=6)
		{
		throw EIconError("Datei konnte nicht geffnet werden !");
		}

	if(aHeader[0]==0x0000 && (aHeader[1]==0x0001 || aHeader[1]==0x0002) && aHeader[2]<=MAX_ICONS)
		{
		if(iIconNr!=0 || aHeader[2]<1)
			{
			throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
			}
		ReadImageIco((ITEMDIR*)aHeader,pImageXor,pImageAnd,iBitmapNr,0);
		return;
		}


	if(aHeader[0]==0x5A4D)							// MZ Sign
		{
		   pFileBuf->Seek(27*2,SEEK_CUR);
		if(pFileBuf->Read(&uLength,4)!=4)
			{
			throw EIconError("Datei konnte nicht geffnet werden !");
			}

		if(uLength<=30*2-4)
			{
			throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
			}

		   pFileBuf->Seek(uLength-30*2-4,SEEK_CUR);
		if(pFileBuf->Read(&aHeader,4)!=4)
			{
			throw EIconError("Datei konnte nicht geffnet werden !");
			}

		if(aHeader[0]==0x4550 && aHeader[1]==0)		// PE Sign 64-Bit
			{
			ReadImageExe(uLength,pImageXor,pImageAnd,iBitmapNr,iIconNr,0);
			return;
			}

		if(aHeader[0]==0x454E)						// NE Sign 16-Bit
			{
			ReadImageE16(uLength,pImageXor,pImageAnd,iBitmapNr,iIconNr,0);
			return;
			}
		}


	throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");

}

//*****************************************************************************
//*
//*		GetIconData
//*
//*****************************************************************************
//	Holt Icon oder Cursor Daten (fr Icon/Cursor-Dateien)
//	iType		: Ist der Header-Typ (1=Icons 2=Cursors)
//	pArray		: Zeiger das Aarry mit den Icon/Cursor eintrgen
//	pValX/Y/B	: Hier werden Breite Hhe BistCount gespeichert
inline void IconDecoder::GetIconData(int iType,void *pArray,int *pValX,int *pValY,int *pValB)
{
LPCURSORDIRENTRY	pCursors;
LPICONDIRENTRY		pEntries;
unsigned			uColors;
unsigned			uBits;


	if(iType==2)										// Cursor
		{
		pCursors = (LPCURSORDIRENTRY)pArray;
		uColors	 =  pCursors->bColorCount;

		for(uBits=0;uColors;uBits++)
			{
			uColors >>= 1;
			}

		if(uBits==0)uBits=1;

		if(pValX)*pValX = pCursors->bWidth;
		if(pValY)*pValY = pCursors->bHeight;
		if(pValB)*pValB = uBits;
		}
	else{												// Icons
		pEntries = (LPICONDIRENTRY)pArray;

		if(pValX)*pValX = pEntries->bWidth;
		if(pValY)*pValY = pEntries->bHeight;
		if(pValB)*pValB = pEntries->wBitCount;
		}

	if(pValX && !*pValX)*pValX=256;
	if(pValY && !*pValY)*pValY=256;

}

//*****************************************************************************
//*
//*		GetIconData
//*
//*****************************************************************************
//	Holt Icon oder Cursor Daten	(fr Exe-Dateien)
//	pHeader		: Zeiger auf den Header
//	iIndex		: Index des Eintrages
//	pValX/Y/B	: Hier werden Breite Hhe BistCount gespeichert
inline void IconDecoder::GetIconData(void *pHeader,int iIndex,int *pValX,int *pValY,int *pValB)
{
LPGITEMDIR			pDir=(LPGITEMDIR)pHeader;
LPGCURSORDIRENTRY	pCursors;
LPGICONDIRENTRY		pEntries;


	if(pDir->idType==2)									// Cursor
		{
		pCursors = pDir->idCursors;

		if(pValX)*pValX = pCursors[iIndex].wWidth;
		if(pValY)*pValY = pCursors[iIndex].wHeight;
		if(pValB)*pValB = pCursors[iIndex].wBitCount;
		}
	else{												// Icons
		pEntries = pDir->idEntries;

		if(pValX)*pValX = pEntries[iIndex].bWidth;
		if(pValY)*pValY = pEntries[iIndex].bHeight;
		if(pValB)*pValB = pEntries[iIndex].wBitCount;
		if(pValX && !*pValX)*pValX=256;
		if(pValY && !*pValY)*pValY=256;
		}


}

//*****************************************************************************
//*
//*		ReadImageIco
//*
//*****************************************************************************
//	Liest ein Icon aus eine *.ICO Datei
//	Header		: Zeiger auf den 6 Byte groen Datei-Header
//	pImageXor	: Bitmap fr die XOR Daten (Farbdaten 2,16,256,2^32-farbig)
//	pImageAnd	: Bitmap fr die AND-Daten (2-Farbig)
//				  Ist kein  AND-Image angegeben so wird das XORImage immer
//				  Im 256 Farbmodus geladen, die durchsichtien Pixel werden
//				  auf 255 gesetzt die XOR-Pixel werden auf 254 gesetzt.
//				  Bei 32-Bit-Images wird die Maske im Alpha-Kanal abgelgt.
//	iBitmapNr	: Nummer des Icons das geladen werden soll.
//				  Bei -1 wird das grte Icon mit den meisten Farben geladen
//				  Ist Bit 0x80000000 gesetzt bestimmen die ersten drei Byte
//				  die Breite,Hhe und die Farbzahl, ist die Farbzahl 0xFF
//				  wird das Bild mit der grten Farbe benutzt.
//	pInfo		: Sollen nur die Infos ber die einzelnen Icons geladen werden.
//				  Dabei ist iBitmapNr die Array gre
//	Ergibt die Anzahl der vorhandnen Iconinformationen.
int IconDecoder::ReadImageIco(ITEMDIR *pIconDir,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,IconInfo *pInfo)
{
ICONDIRENTRY		aIcons[MAX_ICONS];
BITMAPINFOHEADER   *pHeader,sDummy;
unsigned int 	   *pColors,uRgb,uPos;
unsigned char	   *pData,*pPixels;
int					iSize,iNr,iColor,iLen,iMask;
int					iWidth,iHeight,iBits,iMul;
int					iValX,iValY,iValB;
int					i,j,k,iSet1,iSet2;


	iLen		= sizeof(ICONDIRENTRY)*pIconDir->idCount;
	iBitmapNr  &= ~0x03000000;
	uIconFound	= 1;

	if(pFileBuf->Read(aIcons,iLen)!=iLen)
		{
		throw EIconError("Fehler beim lesen aus der Datei !");
		}

	for(i=0;i<pIconDir->idCount;i++)				// Bit-Count prfen
		{
		if(aIcons[i].wBitCount!=0)continue;

		uPos=pFileBuf->Tell();
		     pFileBuf->Seek(aIcons[i].dwImageOffset-iLen-6,SEEK_CUR);
		if(  pFileBuf->Read(&sDummy,sizeof(sDummy))==sizeof(sDummy))
			{
			aIcons[i].wBitCount=sDummy.biBitCount;
			}

		pFileBuf->Seek(uPos,SEEK_SET);
		}

	if(pInfo)										// Nur Iconinformationen laden
		{
		for(i=0;i<pIconDir->idCount;i++)
			{
			if(i>=iBitmapNr)break;

			GetIconData(pIconDir->idType,aIcons+i,&iValX,&iValY,&iValB);

			pInfo[i].uId		= 0;
			pInfo[i].usNumber	= 0;
			pInfo[i].usBitmap	= i;
			pInfo[i].usType		= pIconDir->idType;
			pInfo[i].usWidth	= iValX;
			pInfo[i].usHeight	= iValY;
			pInfo[i].usBits		= iValB;
			}

		return pIconDir->idCount;
		}


	if(iBitmapNr<0)
		{
		iNr    =  1000;
		iSize  = -1;
		iColor = -1;

		if((iBitmapNr&0xFF000000)==0x80000000)
			{
			for(i=0;i<pIconDir->idCount;i++)		// Suchen ein spezielles Subbild
				{
				GetIconData(pIconDir->idType,aIcons+i,&iValX,&iValY,&iValB);

				if((iBitmapNr&0x000000FF)!= iValX    )continue;
				if((iBitmapNr&0x0000FF00)!=(iValY<<8))continue;
				if((iBitmapNr&0x00FF0000)!=0x00FF0000)
					{
					if((iBitmapNr&0x00FF0000)!=iValB<<16)continue;
					iNr = i;
					break;
					}
				else{
					if(iValB<=iColor)continue;
					iColor = iValB;
					iNr    = i;
					}
				}
			}
		else{
			for(i=0;i<pIconDir->idCount;i++)		// Suche das grte Subbild
				{
				GetIconData(pIconDir->idType,aIcons+i,&iValX,&iValY,&iValB);

				j = iValX*iValY;

				if(j< iSize                 )continue;
				if(j==iSize && iValB<=iColor)continue;

				iColor = iValB;
				iSize  = j;
				iNr    = i;
				}
			}

		iBitmapNr=iNr;
		}

	if(iBitmapNr>=pIconDir->idCount)
		{
		throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
		}

//*****************************************************************************


	GetIconData(pIconDir->idType,aIcons+iBitmapNr,&iValX,&iValY,&iValB);

	switch(iValB)
		{
	case   1:	iBits=1;	break;
	case   4:	iBits=4;	break;
	case   8:	iBits=8;	break;
	case  24:	iBits=24;	break;
	case  32:	iBits=32;	break;
	default:	throw EIconError("Das Farbformat wird nicht untersttzt !");
		}


	uIconNum = iBitmapNr;
	iWidth   = iValX;
	iHeight  = iValY;
	iSize    = aIcons[iBitmapNr].dwBytesInRes;

	if(iSize>0x40000)								// Ist da Bild zu gro ?
		{
		throw EIconError("Fehlerhaftes Dateiformat !");
		}

	pData = new unsigned char[iSize];

	   pFileBuf->Seek(aIcons[iBitmapNr].dwImageOffset-iLen-6,SEEK_CUR);
	if(pFileBuf->Read(pData,iSize)!=iSize)
		{
		delete pData;
		throw EIconError("Fehler beim lesen aus der Datei !");
		}

													// Ist das Bild im PNG-Format
	if(!memcmp(pData,"\x89\x50\x4E\x47\x0D\x0A\x1A\x0A",8))
		{
		ReadPng(pData,iSize,pImageXor,pImageAnd,1);
		return 0;
		}


	i  = ((iWidth*iBits+31)/32)*iHeight*4;
	i += 40+((iBits<=8)? (1<<iBits)*4:0);

	if(pIconDir->idType!=2)							// Gre der Maske bei Icons
		{
		i   += ((iWidth+31)/32)*iHeight*4;
		iMul = 2;
		}
	else{
		iMul = 1;
		}

	if(iSize<i)										// Ist da Bild zu gro ?
		{
		throw EIconError("Fehlerhaftes Dateiformat !");
		}


	pHeader = (BITMAPINFOHEADER*)pData;
	iColor  = (iBits>8)? 0:1<<pHeader->biBitCount;

	if(pHeader->biSize   != 40        ||
	   pHeader->biWidth  != iWidth    ||
	   pHeader->biHeight != iHeight*iMul)
		{
		delete pData;
		throw EIconError("Fehlerhaftes Dateiformat !");
		}

	if(pHeader->biCompression)
		{
		delete pData;
		throw EIconError("Komprimierte Icons werden nicht untersttzt !");
		}

	if(pHeader->biBitCount != 1 &&
	   pHeader->biBitCount != 4 &&
	   pHeader->biBitCount != 8 &&
	   pHeader->biBitCount !=24 &&
	   pHeader->biBitCount !=32 &&
	   pHeader->biPlanes   != 1)
		{
		delete pData;
		throw EIconError("Das Farbformat wird nicht untersttzt !");
		}

													// And Maske am Bildende von Cursor
	if(pIconDir->idType==2 && !(iHeight&1) && iBits==1)
		{
		iHeight /= 2;
		iMul     = 2;
		}

	if(pImageAnd)									// Ein leeres Bild erstellen
		{
		pImageXor->SetSize(iBits,iWidth,iHeight);
		pImageAnd->SetSize(1    ,iWidth,iHeight);
		}
	else{
		pImageXor->SetSize((iBits<=8)? 8:iBits,iWidth,iHeight);
		}

	pColors = (unsigned*)(pData+40);



	for(i=0;i<iColor;i++)							// Farben einstellen
		{
		pImageXor->SetColor(i,pColors[i]);
		}

	pPixels	= (unsigned char*)(pColors+iColor);
	iSize   = ((iWidth*iBits+31)/32)*4;


	for(j=iHeight-1;j>=0;j--)						// Pixel laden
		{
		switch(iBits)
			{
		case 1:	for(i=0;i<iWidth;i++)
					{
					k=(pPixels[i/8]>>(~i&7))&1;
					pImageXor->SetPixel(i,j,k);
					}
				break;

		case 4:	for(i=0;i<iWidth;i++)
					{
					if(i&1)	k=pPixels[i/2]&0x0F;
					else	k=pPixels[i/2]>>4;
					pImageXor->SetPixel(i,j,k);
					}
				break;

		case 8:	for(i=0;i<iWidth;i++)
					{
					pImageXor->SetPixel(i,j,pPixels[i]);
					}
				break;

		case 24:for(i=0;i<iWidth;i++)
					{
					uRgb=*(DWORD*)(pPixels+i*3);
					pImageXor->SetPixel(i,j,uRgb);
					}
				break;

		case 32:for(i=0;i<iWidth;i++)
					{
					uRgb=*(DWORD*)(pPixels+i*4);
					pImageXor->SetPixel(i,j,uRgb);
					}
				break;
			}

		pPixels+=iSize;
		}



	iSize = ((iWidth+31)/32)*4;


	if(pImageAnd && iMul!=1)						// Maske laden
		{
		for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
		for(i=0;i<iWidth;i++)
			{
			k=(pPixels[i/8]>>(~i&7))&1;
			pImageAnd->SetPixel(i,j,k);
			}
		}
	else if(iMul!=1)								// Die Maske sind die Farben 255/254
		{
		if(iBits<=8)
			{
			CompressImage(pImageXor);

			iSet1=0;
			iSet2=2;
			iMask=(1<<iBits)-1;

			for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
			for(i=0;i<iWidth;i++)
				{
				k=(pPixels[i/8]>>(~i&7))&1;
				if(!k)continue;
				k=pImageXor->GetPixel(i,j);
				if(k==iMask){iSet2=1;pImageXor->SetPixel(i,j,254);}
				else        {iSet1=1;pImageXor->SetPixel(i,j,255);}
				}

			if(iSet1)pImageXor->SetColor(255,ulColorTrans);
			if(iSet2)pImageXor->SetColor(254,ulColorXor  );
			}
		else{
			for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
			for(i=0;i<iWidth;i++)
				{
				k=(pPixels[i/8]>>(~i&7))&1;
				if(!k)continue;
				k=pImageXor->GetPixel(i,j);
				pImageXor->SetPixel(i,j,ulColorTrans);
				}
			}
		}

	uIconType	= 0;
	uIconId		= 0;
	uColors	  	= iBits;
	uHeight	  	= iHeight;
	uWidth	  	= iWidth;

	delete pData;



return 0;
}


//*****************************************************************************
//*
//*		ReadImageExe
//*
//*****************************************************************************
//	Liest ein Icon aus einer *.EXE Datei
//	iOffset		: Ist der Datei Offset des PE-Headers
//	pImageXor	: Bitmap fr die XOR Daten (Farbdaten 2,16,256-farbig)
//	pImageAnd	: Bitmap fr die AND-Daten (2-Farbig)
//				  Ist kein  AND-Image angegeben so wird das XORIMage immer
//				  Im 256 Farbmodus geladen, die durchsichtien Pixel werden
//				  auf 255 gesetzt die XOR-Pixel werden auf 254 gesetzt.
//	iBitmapNr	: Nummer des Icons das geladen werden soll.
//				  Bei -1 wird das grte Icon mit den meisten Farben geladen
//				  Ist Bit 0x80000000 gesetzt, bestimmen die ersten drei Bytes
//				  die Breite,Hhe und die Farbbitzahl,
//				  Ist die Farbzahl 0xFF wird das Bild mit der grten Farbe benutzt.
//				  Mit Bit 0x01000000 werden auch Icons geladen.
//				  Mit Bit 0x02000000 werden auch Cursor geladen.
//	iIconNr		: Ist die Nummer des Icons das geladen werden soll
//				  Ist iIconNr kleiner 0 so wird das Icon mit der ID -iIconNr
//				  geladen.
//	pInfo		: Sollen nur die Infos ber die einzelnen Icons geladen werden.
//				  Dabei ist iBitmapNr die Array gre
//	Ergibt die Anzahl der vorhandnen Iconinformationen.
int IconDecoder::ReadImageExe(int iOffset,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr,IconInfo *pInfo)
{
unsigned char  *pData;
unsigned char 	ucBuffer[40];
unsigned short	uOffset;
unsigned  		uFilter;
unsigned  		uVirt;
unsigned  		uRgb;
unsigned  		uPng;
int				iPos;
int				iLen;
int				iSize;
int				i,j,k;


	pFileBuf->Seek(16,SEEK_CUR);

	if(pFileBuf->Read(&uOffset,2)!=2)
		{
		throw EIconError("Fehler beim Lesen der Datei !");
		}

	pFileBuf->Seek(uOffset+2,SEEK_CUR);				// Gre optional Header einlesen

	for(i=0;i<16;i++)								// Abschnitte einlesen
		{
		if(pFileBuf->Read(ucBuffer,40)!=40)
			{
			throw EIconError("Fehler beim Lesen der Datei !");
			}

		if(*(int*)ucBuffer    == 0x7273722E)		// Suche ".rsrc"
		if(       ucBuffer[4] == 0x63 )
		if(       ucBuffer[5] == 0x00 )break;

		if(ucBuffer[0]==0 || i==15)
			{
			throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
			}
		}

	uVirt = *(unsigned*)(ucBuffer+12);
	iSize = *(unsigned*)(ucBuffer+16);
	iPos  = *(unsigned*)(ucBuffer+20);

	pFileBuf->Seek(iPos-iOffset-uOffset-i*40-64,SEEK_CUR);

	pData = new unsigned char[iSize];

	if(pFileBuf->Read(pData,iSize)!=iSize)
		{
		delete pData;
		throw EIconError("Fehler beim Lesen der Datei !");
		}


//******************** Suche Icon *********************************************
GICONDIRENTRY  *pIcons;
GITEMDIR	   *pDir;
unsigned int   *pLong,uAddr,uType,uType0;
unsigned int   *pPtr,*pPtrA[16];
unsigned char  *pRsrc;
unsigned 		uRecource;
int				iOk=0,iId,iCount,iLevel=0,iNum[16];
int				iColor,iNr,iAll=0,iInfoPos=0;
int				iValX,iValY,iValB;


	uFilter		= (0x03000000&iBitmapNr)>>24;
	iBitmapNr  &= ~0x03000000;
	pPtrA[0]	= (unsigned*)pData;
	iNum [0] 	= 0;
	iNr		 	= 0;
	uRecource	= 0;

	if(!uFilter)uFilter=0x01;


	while(iLevel>=0 && !iOk)						// Recourcen durchlaufen
		{
		pPtr   = pPtrA[iLevel];
		pLong  = pPtr+4;
		iCount =(pPtr[3]&0xFFFF)+(pPtr[3]>>16);

		for(i=iNum[iLevel];i<iCount;i++)
			{
						 uAddr =pLong[i*2+1];
			if(iLevel==1)uType =pLong[i*2];
			if(iLevel==0)uType0=pLong[i*2];
			if(uAddr&0x80000000)
				{
				if(iLevel>=15)continue;
				iNum[iLevel]=i+1;
				iLevel++;
				uAddr&=0x7FFFFFFF;
				pPtrA[iLevel]=(unsigned*)(pData+uAddr);
				iNum [iLevel]=0;
				iLevel++;
				break;
				}
													// Nur Icons und Cursor
			     if(uType0==(DWORD)RT_GROUP_ICON   && (uFilter&0x01))uRecource=1;
			else if(uType0==(DWORD)RT_GROUP_CURSOR && (uFilter&0x02))uRecource=2;
			else{
				continue;
				}

			pRsrc = pData+uAddr;
			iPos  = *(unsigned*) pRsrc-uVirt;
			iLen  = *(unsigned*)(pRsrc+4);

			if(iPos<         0)continue;
			if(iPos+iLen>iSize)continue;

			   pDir = (GITEMDIR*)(pData+iPos);
			if(pDir->idType!=uRecource)continue;

			if(pInfo)								// Nur Icon-Infos abfragen
				{
				for(j=0;j<pDir->idCount;j++)
					{
					iAll++;

					if(iInfoPos>=iBitmapNr)continue;

					GetIconData(pDir,j,&iValX,&iValY,&iValB);

					pInfo[iInfoPos].uId		 = uType;
					pInfo[iInfoPos].usType   = pDir->idType;
					pInfo[iInfoPos].usNumber = iNr;
					pInfo[iInfoPos].usBitmap = j;
					pInfo[iInfoPos].usWidth	 = iValX;
					pInfo[iInfoPos].usHeight = iValY;
					pInfo[iInfoPos].usBits	 = iValB;

					iInfoPos++;
					}

				iNr++;
				continue;
				}

			if(iIconNr>=0)							// ber die Nummer laden
				{
				if(iIconNr!=0){iIconNr--;continue;}
				}
			else{									// ber ID laden
				if(-iIconNr!=(int)uType)continue;
				}

			uIconId		= uType;
			iOk			= 1;
			break;
			}

		iLevel--;
		}

	if(!iOk)
		{
		delete pData;
		if(pInfo)return iAll;
		throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
		}

//******************** Suche Subbild ******************************************

	uIconFound	=  1;
	iId			= -1;
	pIcons		= pDir->idEntries;

	if(iBitmapNr<0)
		{
		j      = -1;
		iNr	   = -1;
		iLen   = -1;
		iColor = -1;

		if((iBitmapNr&0xFF000000)==0x80000000)
			{
			for(i=0;i<pDir->idCount;i++)			// Suchen ein spezielles Subbild
				{
				GetIconData(pDir,i,&iValX,&iValY,&iValB);

				if((iBitmapNr&0x000000FF)!= iValX    )continue;
				if((iBitmapNr&0x0000FF00)!=(iValY<<8))continue;
				if((iBitmapNr&0x00FF0000)!=0x00FF0000)
					{
					if((iBitmapNr&0x00FF0000)!=iValB<<16)continue;
					iId    = pIcons[i].nID;
					iNr    = i;
					break;
					}
				else{
					if(iValB<=iColor)continue;
					iId	   = pIcons[i].nID;
					iColor = iValB;
					iNr    = i;
					}
				}

			iBitmapNr=iNr;
			}
		else{
			for(i=0;i<pDir->idCount;i++)			// Suche das grte Subbild
				{
				GetIconData(pDir,i,&iValX,&iValY,&iValB);

				j = iValX*iValY;

				if(j< iLen                 )continue;
				if(j==iLen && iValB<=iColor)continue;

				iId       = pIcons[i].nID;
				iColor    = iValB;
				iLen      = j;
				iBitmapNr = i;
				}
			}
		}
	else{
		if(iBitmapNr<pDir->idCount)
			{
			iId = pIcons[iBitmapNr].nID;
			}
		}

	if(iId<0)
		{
		delete pData;
		throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
		}

//******************** Suche Icon ID ******************************************

	pPtrA[0]	= (unsigned*)pData;
	iNum [0]	= 0;
	iLevel		= 0;
	iOk			= 0;

	while(iLevel>=0 && !iOk)						// Recourcen durchlaufen
		{
		pPtr	=  pPtrA[iLevel];
		pLong	=  pPtr+4;
		iCount	=  pPtr[3]&0xFFFF;
		iCount +=  pPtr[3]>>16;

		for(i=iNum[iLevel];(int)i<iCount;i++)
			{
						 uAddr =pLong[i*2+1];
			if(iLevel==1)uType =pLong[i*2];
			if(iLevel==0)uType0=pLong[i*2];
			if(uAddr&0x80000000)
				{
				if(iLevel>=15)continue;
				iNum[iLevel]=i+1;
				iLevel++;
				uAddr&=0x7FFFFFFF;
				pPtrA[iLevel]=(unsigned*)(pData+uAddr);
				iNum [iLevel]=0;
				iLevel++;
				break;
				}

													// Nur Icons und Cursor
			     if(uType0==(DWORD)RT_ICON   && uRecource==1);
			else if(uType0==(DWORD)RT_CURSOR && uRecource==2);
			else{
				continue;
				}

			pRsrc =              pData+uAddr;
			iPos  = *(unsigned*) pRsrc-uVirt;
			iLen  = *(unsigned*)(pRsrc+4);


			if(iPos<         0)continue;
			if(iPos+iLen>iSize)continue;

			pRsrc = pData+iPos;

			if(iId==(int)uType)						// Icon gefunden
				{
				iOk = 1;
				break;
				}
			}

		iLevel--;
		}

	if(!iOk)
		{
		delete pData;
		throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
		}

//******************** Kopiere Iocn Daten *************************************
BITMAPINFOHEADER   *pHeader;
unsigned int 	   *pColors;
unsigned char	   *pPixels;
int					iHeight;
int					iWidth;
int					iSet1;
int					iSet2;
int					iBits;
int					iMask;
int					iMul;


	if(uRecource==2)								// Ist es ein Cursor
		{
		pRsrc += 4;
		}

	uPng    = (memcmp(pRsrc,"\x89\x50\x4E\x47\x0D\x0A\x1A\x0A",8))? 0:1;
	pHeader = (BITMAPINFOHEADER*)pRsrc;
	iBits	= pHeader->biBitCount;

	if(uPng || iBits<=0 || iBits>32)				// BitCount aus dem Bild-Header holen
		{
		switch(pIcons[iBitmapNr].wBitCount)
			{
		case   1:	iBits=1;	break;
		case   4:	iBits=4;	break;
		case   8:	iBits=8;	break;
		case  24:	iBits=24;	break;
		case  32:	iBits=32;	break;
		case   0:	iBits	= pHeader->biBitCount;
					if(iBits>0 && iBits<=32)break;
		default:	throw EIconError("Das Farbformat wird nicht untersttzt !");
			}
		}


	GetIconData(pDir,iBitmapNr,&iWidth,&iHeight);
	iSize    = pIcons[iBitmapNr].dwBytesInRes;
	uIconNum = iBitmapNr;
	uType	 = 0;

	if(iSize>iLen || iSize>0x40000)
		{
		throw EIconError("Fehlerhaftes Dateiformat !");
		}


	if(uPng)										// Ist das Bild im PNG-Format
		{
		ReadPng(pData,iSize,pImageXor,pImageAnd);
		return 0;
		}

	if(uRecource==2)
		{
		i    = ((iWidth*iBits+31)/32)*iHeight*4;
		i   += 44+((iBits<=8)? (1<<iBits)*4:0);
		iMul = 1;
		}
	else{
		i    = ((iWidth*iBits+31)/32)*iHeight*4;	// Gre mit Maske
		i   += ((iWidth      +31)/32)*iHeight*4;
		i   += 40+((iBits<=8)? (1<<iBits)*4:0);
		iMul = 2;
		}


	if(iSize<i)										// Ist da Bild zu gro ?
		{
		throw EIconError("Fehlerhaftes Dateiformat !");
		}

	iColor  = (iBits>8)? 0:1<<pHeader->biBitCount;


	if(pHeader->biSize   != 40        ||
	   pHeader->biWidth  != iWidth    ||
	   pHeader->biHeight != iHeight*iMul)
		{
		delete pData;
		throw EIconError("Fehlerhaftes Dateiformat !");
		}

	if(pHeader->biCompression)
		{
		delete pData;
		throw EIconError("Komprimierte Icons werden nicht untersttzt !");
		}

	if(pHeader->biBitCount != 2 &&
	   pHeader->biBitCount != 4 &&
	   pHeader->biBitCount != 8 &&
	   pHeader->biPlanes   != 1)
		{
		delete pData;
		throw EIconError("Das Farbformat wird nicht untersttzt !");
		}


	if(uRecource==2 && !(iHeight&1) && iBits==1)	// And Maske am Bildende von Cursor
		{
		iHeight /= 2;
		iMul     = 2;
		}


	if(pImageAnd)									// Ein leeres Bild erstellen
		{
		pImageXor->SetSize(iBits,iWidth,iHeight);
		pImageAnd->SetSize(1    ,iWidth,iHeight);
		}
	else{
		pImageXor->SetSize((iBits<=8)? 8:iBits,iWidth,iHeight);
		}

	pColors = (unsigned*)(pRsrc+40);



	for(i=0;i<iColor;i++)							// Farben einstellen
		{
		pImageXor->SetColor(i,pColors[i]);
		}

	pPixels = (unsigned char*)(pColors+iColor);
	iSize   = ((iWidth*iBits+31)/32)*4;


	for(j=iHeight-1;j>=0;j--)						// Pixel laden
		{
		switch(iBits)
			{
		case 1:	for(i=0;i<iWidth;i++)
					{
					k=(pPixels[i/8]>>(~i&7))&1;
					pImageXor->SetPixel(i,j,k);
					}
				break;

		case 4:	for(i=0;i<iWidth;i++)
					{
					if(i&1)	k=pPixels[i/2]&0x0F;
					else	k=pPixels[i/2]>>4;
					pImageXor->SetPixel(i,j,k);
					}
				break;

		case 8:	for(i=0;i<iWidth;i++)
					{
					pImageXor->SetPixel(i,j,pPixels[i]);
					}
				break;

		case 24:for(i=0;i<iWidth;i++)
					{
					uRgb=*(DWORD*)(pPixels+i*3);
					pImageXor->SetPixel(i,j,uRgb);
					}
				break;

		case 32:for(i=0;i<iWidth;i++)
					{
					uRgb=*(DWORD*)(pPixels+i*4);
					pImageXor->SetPixel(i,j,uRgb);
					}
				break;
			}

		pPixels += iSize;
		}


	iSize = ((iWidth+31)/32)*4;

	if(pImageAnd && iMul!=1)						// Maske laden
		{
		if(iBits==32)
			{
			j = iHeight*iSize/4;

			for(i=j;i>0;)
				{
				i--;
				if(((DWORD*)pPixels)[i])break;
				}

			if(i==0)								// Maske aus Alpha-Werten
				{
				pPixels  = (unsigned char*)(pColors+iColor);
				pPixels += 3;

				for(j=iHeight-1;j>=0;j--)
				for(i=0;i<iWidth;i++)
					{
					if(*pPixels==0)
						{
						pImageAnd->SetPixel(i,j,1);
						}

					pPixels += 4;
					}

				pPixels = 0;
				}
			}

		if(pPixels)									// Maske aus Bits holen
			{
			for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
			for(i=0;i<iWidth;i++)
				{
				k=(pPixels[i/8]>>(~i&7))&1;
				pImageAnd->SetPixel(i,j,k);
				}
			}
		}
	else if(iMul!=1)
		{
		if(iBits<=8)
			{
			CompressImage(pImageXor);

			iSet1=0;
			iSet2=2;
			iMask=(1<<iBits)-1;

			for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
			for(i=0;i<iWidth;i++)
				{
				k=(pPixels[i/8]>>(~i&7))&1;
				if(!k)continue;
				k=pImageXor->GetPixel(i,j);
				if(k==iMask){iSet2=1;pImageXor->SetPixel(i,j,254);}
				else        {iSet1=1;pImageXor->SetPixel(i,j,255);}
				}

			if(iSet1)pImageXor->SetColor(255,ulColorTrans);
			if(iSet2)pImageXor->SetColor(254,ulColorXor  );
			}
		else{
			for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
			for(i=0;i<iWidth;i++)
				{
				k=(pPixels[i/8]>>(~i&7))&1;
				if(!k)continue;
				k=pImageXor->GetPixel(i,j);
				pImageXor->SetPixel(i,j,ulColorTrans);
				}
			}
		}

	uIconType	= 0;
	uColors	  	= iBits;
	uHeight	  	= iHeight;
	uWidth	  	= iWidth;

	delete pData;

return 0;
}

//*****************************************************************************
//*
//*		ReadImageE16
//*
//*****************************************************************************
//	Liest ein Icon aus einer 16 Bit *.EXE Datei
//	iOffset		: Ist der Datei Offset des NE-Headers
//	pImageXor	: Bitmap fr die XOR Daten (Farbdaten 2,16,256-farbig)
//	pImageAnd	: Bitmap fr die AND-Daten (2-Farbig)
//				  Ist kein  AND-Image angegeben so wird das XORIMage immer
//				  Im 256 Farbmodus geladen, die durchsichtien Pixel werden
//				  auf 255 gesetzt die XOR-Pixel werden auf 254 gesetzt.
//	iBitmapNr	: Nummer des Icons das geladen werden soll.
//				  Bei -1 wird das grte Icon mit den meisten Farben geladen
//				  Ist Bit 0x80000000 gesetzt bestimmt das ersten drei Byte
//				  die Breite,Hhe und die Farbzahl, ist die Farbzahl 0xFF
//				  wird das Bild mit der grten Farbe benutzt.
//				  Mit Bit 0x01000000 werden keine Icons geladen.
//				  Mit Bit 0x02000000 werden auch Cursor geladen.
//	iIconNr		: Ist die Nummer des Icons das geladen werden soll
//				  Ist iIconNr kleiner 0 so wird das Icon mit der ID -iIconNr
//				  geladen.
//	pInfo		: Sollen nur die Infos ber die einzelnen Icons geladen werden.
//				  Dabei ist iBitmapNr die Array gre
//	Ergibt die Anzahl der vorhandnen Iconinformationen.
int IconDecoder::ReadImageE16(int iOffset,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr,IconInfo *pInfo)
{
GITEMDIR	   *pDir;
GICONDIRENTRY  *pIcons;
unsigned short  sData[132];
int				iAlignSize,iNr;
int				iCount,iStart;
int				iValX,iValY;
int				iPos,iLen;
int				iFilter;
int				iColor;
int				i,j,k;
int				iValB;
int				iId;


	iStart = pFileBuf->Tell()-4;

	pFileBuf->Seek(30,SEEK_CUR);

	if(pFileBuf->Read(sData,2)!=2)
		{
		throw EIconError("Fehler beim Lesen der Datei !");
		}


	pFileBuf->Seek(sData[0]-38,SEEK_CUR);


//******************** Suche Icon *********************************************


	if(pFileBuf->Read(sData,2)!=2)
		{
		throw EIconError("Fehler beim Lesen der Datei !");
		}

	iFilter		= (0x03000000&iBitmapNr)>>24;
	iBitmapNr  &= ~0x03000000;
	iPos		= pFileBuf->Tell();
	iAlignSize	= 1<<sData[0];

	if(!iFilter)iFilter=0x01;

	for(;;)
		{
		if(pFileBuf->Read(sData,8)!=8)
			{
			throw EIconError("Fehler beim Lesen der Datei !");
			}

		if(!sData[0])break;
		if( sData[0]==0x800e)break;

		iCount=sData[1];
		pFileBuf->Seek(iCount*12,SEEK_CUR);
		}

	if(!sData[0])
		{
		throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
		}


	iCount = sData[1];

	if(pInfo)										// Nur Icon-Informationen laden
		{
		for(i=0,j=0;i<iCount;i++)
			{
			if(pFileBuf->Read(sData,12)!=12)
				{
				return -1;
				}

			iPos = pFileBuf->Tell();
			iNr	 = sData[3]&0x7FFF;					// Icon ID
			iLen = sData[1]*iAlignSize;

			if(iLen>sizeof(sData))continue;
													// Zum Icon springen
			pFileBuf->Seek(iStart-iOffset+sData[0]*iAlignSize);

			if(pFileBuf->Read(sData,iLen)!=iLen)
				{
				return -1;
				}

			pDir = (GITEMDIR*)sData;

			for(k=0;k<pDir->idCount;k++)			// Alle einzelnen Icons auswerten
				{
				if(j<iBitmapNr)
					{
					GetIconData(pDir,k,&iValX,&iValY,&iValB);

					pInfo[j].uId		= iNr;
					pInfo[j].usType		= pDir->idType;
					pInfo[j].usNumber	= i;
					pInfo[j].usBitmap	= k;
					pInfo[j].usWidth	= iValX;
					pInfo[j].usHeight	= iValY;
					pInfo[j].usBits		= iValB;
					}

				j++;
				}

			pFileBuf->Seek(iPos);					// Zurck zum nchsten Icon
			}

		return j;
		}


	if(iIconNr>=0)									// Icon via Nummer laden
		{
		if(iIconNr>=iCount)
			{
			throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
			}

		   pFileBuf->Seek(iIconNr*12,SEEK_CUR);
		if(pFileBuf->Read(sData,12)!=12)
			{
			throw EIconError("Fehler beim Lesen der Datei !");
			}

		uIconId = sData[3]&0x7FFF;
		}
	else{											// Icon via ID laden
		for(i=0;i<iCount;i++)
			{
			if(pFileBuf->Read(sData,12)!=12)
				{
				throw EIconError("Fehler beim Lesen der Datei !");
				}
			sData[3]&=0x7FFF;
			if(-iIconNr==sData[3])break;
			}

		if(i>=iCount)
			{
			throw EIconError("Fehler beim Lesen der Datei !");
			}

		uIconId = sData[3]&0x7FFF;
		}


	pFileBuf->Seek(iStart-iOffset+sData[0]*iAlignSize);
	iLen = sData[1]*iAlignSize;

	if(iLen>sizeof(sData))
		{
		throw EIconError("Zugroer Datenblock !");
		}

	if(pFileBuf->Read(sData,iLen)!=iLen)
		{
		throw EIconError("Fehler beim Lesen der Datei !");
		}


//******************** Suche Subbild ******************************************

	uIconFound=1;
	pDir   = (GITEMDIR*)sData;
	pIcons = pDir->idEntries;
	iId	   = -1;

	if(pDir->idType!=1)
		{
		throw EIconError("Fehlerhaftes Dateiformat !");
		}

	if(iBitmapNr<0)
		{
		j      = -1;
		iNr	   = -1;
		iLen   = -1;
		iColor = -1;

		if((iBitmapNr&0xFF000000)==0x80000000)
			{
			for(i=0;i<pDir->idCount;i++)			// Suchen ein spezielles Subbild
				{
				GetIconData(pDir,i,&iValX,&iValY,&iValB);

				if((iBitmapNr&0x000000FF)!= iValX    )continue;
				if((iBitmapNr&0x0000FF00)!=(iValY<<8))continue;
				if((iBitmapNr&0x00FF0000)!=0x00FF0000)
					{
					if((iBitmapNr&0x00FF0000)!=iValB<<16)continue;
					iId    = pIcons[i].nID;
					iNr	   = i;
					break;
					}
				else{
					if(iValB<=iColor)continue;
					iId	   = pIcons[i].nID;
					iColor = iValB;
					iNr	   = i;
					}
				}

			iBitmapNr = iNr;
			}
		else{
			for(i=0;i<pDir->idCount;i++)			// Suche das grte Subbild
				{
				GetIconData(pDir,i,&iValX,&iValY,&iValB);

				j = iValX*iValY;

				if(j< iLen                 )continue;
				if(j==iLen && iValB<=iColor)continue;

				iId       = pIcons[i].nID;
				iColor    = iValB;
				iLen      = j;
				iBitmapNr = i;
				}
			}
		}
	else{
		if(iBitmapNr<pDir->idCount)
			{
			iId = pIcons[iBitmapNr].nID;
			}
		}

	if(iId<0)
		{
		throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
		}


	GetIconData(pDir,iBitmapNr,&iValX,&iValY,&iValB);



//******************** Suche Icon ID ******************************************
unsigned char  *pData;


	pFileBuf->Seek(iPos);

	for(;;)
		{
		if(pFileBuf->Read(sData,8)!=8)
			{
			throw EIconError("Fehler beim Lesen der Datei !");
			}

		if(!sData[0])break;
		if( sData[0]==0x8003)break;

		iCount=sData[1];
		pFileBuf->Seek(iCount*12,SEEK_CUR);
		}

	if(!sData[0])
		{
		throw EIconError("Das Icon konnte in der Datei nicht gefunden werden !");
		}


	iCount=sData[1];

	for(i=0;i<iCount;i++)
		{
		if(pFileBuf->Read(sData,12)!=12)
			{
			throw EIconError("Fehler beim Lesen der Datei !");
			}
		sData[3]&=0x7FFF;
		if(iId==sData[3])break;
		}

	if(i>=iCount)
		{
		throw EIconError("Fehler beim Lesen der Datei !");
		}


	pFileBuf->Seek(iStart-iOffset+sData[0]*iAlignSize);
	iLen = sData[1]*iAlignSize;

	if(iLen>0x20000)
		{
		throw EIconError("Zugroer Datenblock !");
		}

	pData=new unsigned char[iLen];

	if(pFileBuf->Read(pData,iLen)!=iLen)
		{
		delete pData;
		throw EIconError("Fehler beim Lesen der Datei !");
		}

//******************** Kopiere Icon Daten *************************************
BITMAPINFOHEADER   *pHeader;
unsigned int 	   *pColors;
unsigned char	   *pPixels;
int					iHeight;
int					iWidth;
int					iSize;
int					iBits;
int					iMask;
int					iSet1;
int					iSet2;



	switch(iValB)									// Prfe das Farbformat
		{
	case   1:	iBits=1;	break;
	case   4:	iBits=4;	break;
	case   8:	iBits=8;	break;
	default:	throw EIconError("Das Farbformat wird nicht untersttzt !");
		}

	uIconNum = iBitmapNr;
	iWidth   = iValX;
	iHeight  = iValY;
	iSize    = pIcons[iBitmapNr].dwBytesInRes;

	if(!iWidth )iWidth =256;
	if(!iHeight)iHeight=256;

	if(iSize>iLen || iSize>0x20000)
		{
		throw EIconError("Fehlerhaftes Dateiformat !");
		}

													// Ist das Bild im PNG-Format
	if(!memcmp(pData,"\x89\x50\x4E\x47\x0D\x0A\x1A\x0A",9))
		{
		ReadPng(pData,iSize,pImageXor,pImageAnd);
		return 0;
		}


	i  =	((iWidth*iBits+31)/32)*iHeight*4;
	i +=	((iWidth      +31)/32)*iHeight*4;
	i +=    40+((iBits<=8)? (1<<iBits)*4:0);

	if(iSize<i)										// Ist da Bild zu gro ?
		{
		throw EIconError("Fehlerhaftes Dateiformat !");
		}


	pHeader = (BITMAPINFOHEADER*)pData;
	iColor  = 1<<pHeader->biBitCount;

	if(pHeader->biSize   != 40        ||
	   pHeader->biWidth  != iWidth    ||
	   pHeader->biHeight != iHeight*2)
		{
		delete pData;
		throw EIconError("Fehlerhaftes Dateiformat !");
		}

	if(pHeader->biCompression)
		{
		delete pData;
		throw EIconError("Komprimierte Icons werden nicht untersttzt !");
		}

	if(pHeader->biBitCount != 2 &&
	   pHeader->biBitCount != 4 &&
	   pHeader->biBitCount != 8 &&
	   pHeader->biPlanes   != 1)
		{
		delete pData;
		throw EIconError("Das Farbformat wird nicht untersttzt !");
		}



	if(pImageAnd)								// Ein leeres Bild erstellen
		{
		pImageXor->SetSize(iBits,iWidth,iHeight);
		pImageAnd->SetSize(1    ,iWidth,iHeight);
		}
	else{
		pImageXor->SetSize(8,iWidth,iHeight);
		}

	pColors=(unsigned*)(pData+40);



	for(i=0;i<iColor;i++)						// Farben einstellen
		{
		pImageXor->SetColor(i,pColors[i]);
		}

	pPixels	= (unsigned char*)(pColors+iColor);
	iSize	= ((iWidth*iBits+31)/32)*4;


	for(j=iHeight-1;j>=0;j--)					// Pixel laden
		{
		switch(iBits)
			{
		case 1:	for(i=0;i<iWidth;i++)
					{
					k=(pPixels[i/8]>>(~i&7))&1;
					pImageXor->SetPixel(i,j,k);
					}
				break;

		case 4:	for(i=0;i<iWidth;i++)
					{
					if(i&1)	k=pPixels[i/2]&0x0F;
					else	k=pPixels[i/2]>>4;
					pImageXor->SetPixel(i,j,k);
					}
				break;

		case 8:	for(i=0;i<iWidth;i++)
					{
					pImageXor->SetPixel(i,j,pPixels[i]);
					}
				break;
			}

		pPixels+=iSize;
		}


	iSize = ((iWidth+31)/32)*4;

	if(pImageAnd)								// Maske laden
		{
		if(iBits==32)
			{
			j = iHeight*iSize/4;

			for(i=j;i>0;)
				{
				i--;
				if(((DWORD*)pPixels)[i])break;
				}

			if(i==0)							// Maske aus Alpha-Werten
				{
				pPixels  = (unsigned char*)(pColors+iColor);
				pPixels += 3;

				for(j=iHeight-1;j>=0;j--)
				for(i=0;i<iWidth;i++)
					{
					if(*pPixels==0)
						{
						pImageAnd->SetPixel(i,j,1);
						}

					pPixels += 4;
					}

				pPixels = 0;
				}
			}

		if(pPixels)								// Maske aus Bits holen
			{
			for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
			for(i=0;i<iWidth;i++)
				{
				k=(pPixels[i/8]>>(~i&7))&1;
				pImageAnd->SetPixel(i,j,k);
				}
			}
		}
	else{
		CompressImage(pImageXor);

		iSet1=0;
		iSet2=2;
		iMask=(1<<iBits)-1;

		for(j=iHeight-1;j>=0;j--,pPixels+=iSize)
		for(i=0;i<iWidth;i++)
			{
			k=(pPixels[i/8]>>(~i&7))&1;
			if(!k)continue;
			k=pImageXor->GetPixel(i,j);
			if(k==iMask){iSet2=1;pImageXor->SetPixel(i,j,254);}
			else        {iSet1=1;pImageXor->SetPixel(i,j,255);}
			}

		if(iSet1)pImageXor->SetColor(255,ulColorTrans);
		if(iSet2)pImageXor->SetColor(254,ulColorXor  );
		}

	uIconType	= 0;
	uColors	  	= iBits;
	uHeight	  	= iHeight;
	uWidth	  	= iWidth;

	delete pData;


return 0;
}

//*****************************************************************************
//*
//*		ReadPng
//*
//*****************************************************************************
//	Liet ein PNG-Bild aus einem Speicherbereich ein.
//	pData		: Zeiger auf den Speicherbereich.
//	iSize		: Gre des Speicherbereiches.
//	pImageXor	: Bitmap fr die XOR Daten (Farbdaten 2,16,256-farbig)
//	pImageAnd	: Bitmap fr die AND-Daten (2-Farbig)
//				  Ist kein  AND-Image angegeben so wird das XOR-Image immer
//				  Im 256 Farbmodus geladen, die durchsichtien Pixel werden
//				  auf 255 gesetzt die XOR-Pixel werden auf 254 gesetzt.
//	iDel		_ Muss der Speicherbereich mit delete gelscht werden
//	pInfo		: Sollen nur die Infos ber die einzelnen Icons geladen werden.
//				  Dabei ist iBitmapNr die Array gre
//	Ergibt die Anzahl der vorhandnen Iconinformationen.
void IconDecoder::ReadPng(void *pData,int iSize,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iDel)
{
PngDecoder	sPng;
CFileBuffer	sMem;
unsigned   *pRow;
int			i,j;




	if(!sMem.OpenMem(pData,iSize))
		{
		if(iDel)delete pData;
		throw EIconError("Zu wenig Speicher");
		}


	try	{
		sPng.ReadImage(&sMem,*pImageXor);
		}
	catch(const char *pError)
		{
		if(iDel)delete pData;
		throw pError;
		}
	catch(...)
		{
		if(iDel)delete pData;
		throw "Fehler beim Lesen aus der Datei !";
		}



	if(pImageAnd)										// Maskenbild erstellen
		{
		pImageAnd->SetSize(1,pImageXor->Width(),pImageXor->Height());

		if(pImageXor->BitCount()==32 && sPng.HasAlpha())
			{
			for(j=pImageXor->Height()-1;j>=0;j--)
				{
				pRow = (unsigned*)pImageXor->GetRow(j);

				for(i=pImageXor->Width()-1;i>=0;i--)
					{
					if((pRow[i]>>24)!=0x00)continue;
					pImageAnd->SetPixel(i,j,1);
					}
				}
			}
		}
	else{												// Die Maske sind die Farben 255/254
		if(pImageXor->BitCount()<=8)
			{
			CompressImage(pImageXor);
			}
		}

	uColors	  	= pImageXor->BitCount();
	uHeight	  	= pImageXor->Height();
	uWidth	  	= pImageXor->Width();
	uIconType	= 1;

	if(iDel)delete pData;

	uIconId=0;

}

