//*****************************************************************************
//*
//*
//*		IconDecoder.h
//*
//*
//*****************************************************************************
#ifndef __ICONDECODER_H__
#define __ICONDECODER_H__

#if _MSC_VER >= 1000
#pragma once
#endif

#include "Icon.h"
#include "BitmapImage.h"
#include "GrException.h"
#include "..\FileBuffer.h"

typedef struct
	{
	unsigned	 	uId;							// ID des Icons (Resource-ID)
	unsigned short 	usType;							// Typ (1=Icon 2=Cursor)
	unsigned short 	usNumber;						// Icon-Nummer (fortlaufende Resource-Nummer)
	unsigned short 	usBitmap;						// Ist die Bitmap-Nummer (Unternummer des Icons)
	unsigned short	usWidth;						// Breite in Pixel
	unsigned short	usHeight;						// Hhe in Pixel
	unsigned short 	usBits;							// Farbtiefe in Bits
	}IconInfo;

//*****************************************************************************
//*
//*		Klasse
//*
//*****************************************************************************
class IconDecoder : public BitmapImageDecoder
{
public:
			  IconDecoder();
	virtual  ~IconDecoder();

	virtual void  ReadImage(const wchar_t *pName,BitmapImage &sImage);
	virtual void  ReadImage(const char    *pName,BitmapImage &sImage);
	virtual void  ReadImage(CFileBuffer   *pFile,BitmapImage &sImage);
	void		  ReadImage(const wchar_t *pName,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr=0);
	void		  ReadImage(const char    *pName,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr=0);
	void		  ReadImage(CFileBuffer   *pFile,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr=0);

	int 		  GetInfo  (const wchar_t *pName,IconInfo    *pInfo,int iCount,unsigned uFilter=0x01);
	int 		  GetInfo  (const char    *pName,IconInfo    *pInfo,int iCount,unsigned uFilter=0x01);
	int 		  GetInfo  (CFileBuffer   *pFile,IconInfo    *pInfo,int iCount,unsigned uFilter=0x01);

	unsigned long SetTransparentColor(unsigned long uColor);
	unsigned long SetXorColor        (unsigned long uColor);

	unsigned	  GetIconType();			// Ergibt 0 fr BMP oder 1 fr PNG
	unsigned	  GetIconNum ();
	unsigned	  GetIconId  ();
	unsigned	  GetWidth   ();
	unsigned	  GetHeight  ();
	unsigned	  GetPlanes  ();
	unsigned	  FoundIcon  ();			// Ergibt 1 wenn die IconNummer vorhanden war

protected:

	void ReadPng     (void *pData,int iSize  ,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iDel=0);
	int  ReadImageE16(int             iOffset,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr,IconInfo *pInfo);
	int  ReadImageExe(int             iOffset,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr,int iIconNr,IconInfo *pInfo);
	int  ReadImageIco(ITEMDIR        *pHeader,BitmapImage *pImageXor,BitmapImage *pImageAnd,int iBitmapNr            ,IconInfo *pInfo);
	void GetIconData (int			  iType  ,void *pArray,int *pValX,int *pValY=0,int *pValB=0);
	void GetIconData (void           *pHeader,int   iIndex,int *pValX,int *pValY=0,int *pValB=0);
	void CompressImage(BitmapImage   *pImage);

	CFileBuffer	   *pFileBuf;

	unsigned long	ulColorTrans;
	unsigned long	ulColorXor;
	unsigned		uIconId;
	unsigned		uIconNum;
	unsigned		uIconFound;
	unsigned		uIconType;
	unsigned		uColors;
	unsigned		uHeight;
	unsigned		uWidth;
};



//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline unsigned	     IconDecoder::GetWidth   (){return uWidth;}
inline unsigned	     IconDecoder::GetHeight  (){return uHeight;}
inline unsigned	     IconDecoder::GetPlanes  (){return uColors;}
inline unsigned	     IconDecoder::FoundIcon  (){return uIconFound;}
inline unsigned	     IconDecoder::GetIconId  (){return uIconId;}
inline unsigned	     IconDecoder::GetIconNum (){return uIconNum;}
inline unsigned	     IconDecoder::GetIconType(){return uIconType;}
inline unsigned long IconDecoder::SetTransparentColor(unsigned long uColor)
{
unsigned long	uOld;

	uOld=ulColorTrans;
	ulColorTrans=uColor;

return uOld;
}
inline unsigned long IconDecoder::SetXorColor(unsigned long uColor)
{
unsigned long uOld;

	uOld=ulColorXor;
	ulColorXor=uColor;

return uOld;
}


#endif
