//*****************************************************************************
//*
//*
//*		IconEncoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
#include "IconEncoder.h"
#include "PngEncoder.h"
#include "..\FileBuffer.h"

#define 	MAX_ICONS		32


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
IconEncoder::IconEncoder()
{
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
IconEncoder::~IconEncoder()
{
}


//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
//	Speichert ein Icon
//	pFilename	: Dateiname des Icons (*.ico)
//  pImageXor	: Bild mit den Farbdaten
//				  Ist kein Maskenbild angegeben so ist die Farbe 255 fr
//				  transpatente Pixel und 254 fr XorPixel
//	pImageAnd	: Bild mit den Maskendaten
//				  Wenn die Maske auf 1 ist dann beteutet die Fatbe 0
//				  in den Bilddaten einen transpatenten Pixel und die
//				  Farbe 15 einen XorPixel
//	iBits		: Mit iBits gibt man die Bits je Pixel an (1,4,8,24,32)
//				  Bei 32-Bit wird auch der Alpha-Channel gespeichert
//	iNew		: Wenn neu ungleich 0 ist dan wird eine Datei neu erzeugt.
//	iMask		: Ist die Farbe fr die Maske. (-1=default)
//	iType		: Art der Datei (1=Icon 2=Cursor) Fr eine Cursor-Datei wird
//				  die Hot-Spot-Position in den Bytes 3 und 4 angegeben.
//				  (siehe ICO_CURSOR(x,y))	
void IconEncoder::WriteImage(const char *pFilename,const BitmapImage &sImage)
{
	WriteImage(pFilename,&sImage,sImage.BitCount(),1,-1,1);
}

void IconEncoder::WriteImage(const wchar_t *pFilename,const BitmapImage &sImage)
{
	WriteImage(pFilename,&sImage,sImage.BitCount(),1,-1,1);
}

void IconEncoder::WriteImage(const char *pFilename,const BitmapImage *pImageXor,int iBits,int iNew,int iMask,int iType)
{
CFileBuffer	 sFileBuf;


	if(!sFileBuf.Open(pFilename,(iNew)? CFB_CREATE:CFB_WRITE))
		{
		throw EIconError("Datei konnte nicht geffnet werden !");
		}

	try	{
		WriteImage(&sFileBuf,pImageXor,iBits,iNew,-1,iType);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();
}

void IconEncoder::WriteImage(const wchar_t *pFilename,const BitmapImage *pImageXor,int iBits,int iNew,int iMask,int iType)
{
CFileBuffer	 sFileBuf;


	if(!sFileBuf.Open(pFilename,(iNew)? CFB_CREATE:CFB_WRITE))
		{
		throw EIconError("Datei konnte nicht geffnet werden !");
		}

	try	{
		WriteImage(&sFileBuf,pImageXor,iBits,iNew,-1,iType);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();
}

void IconEncoder::WriteImage(const char  *pFilename,const BitmapImage *pImageXor,const BitmapImage *pImageAnd,int iNew,int iType)
{
	CFileBuffer	sFileBuf;


	if(!sFileBuf.Open(pFilename,(iNew)? CFB_CREATE:CFB_WRITE))
		{
		throw EIconError("Datei konnte nicht geffnet werden !");
		}

	try	{
		WriteImage(&sFileBuf,pImageXor,pImageAnd,iNew,iType);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();
}

void IconEncoder::WriteImage(const wchar_t *pFilename,const BitmapImage *pImageXor,const BitmapImage *pImageAnd,int iNew,int iType)
{
	CFileBuffer	sFileBuf;


	if(!sFileBuf.Open(pFilename,(iNew)? CFB_CREATE:CFB_WRITE))
		{
		throw EIconError("Datei konnte nicht geffnet werden !");
		}

	try	{
		WriteImage(&sFileBuf,pImageXor,pImageAnd,iNew,iType);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();
}

void IconEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
	WriteImage(pFile,&sImage,sImage.BitCount(),-1,0,1);
}

void IconEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage *pImageXor,int iBits,int iNew,int iMask,int iType)
{
BitmapImage	 sXor,sAnd;
unsigned	 uColMask;
unsigned	 uColVal;
unsigned	 uPixel;
int			 iHeight;
int			 iWidth;
int			 iColor;
int			 x,y,z;



	if(iBits!=32 && iBits!=24 && iBits!=8 && iBits!=4 && iBits!=1)
		{
		throw EIconError("Falsches Farbformat !");
		}


	iColor	 = pImageXor->ColorCount();
	iWidth	 = pImageXor->Width();
	iHeight  = pImageXor->Height();


	if(iBits<=8)
		{
		for(z=0;z<iColor;z++)
			{
			sXor.SetColor(z,pImageXor->GetColor(z));
			}

		sAnd.SetSize(1    ,iWidth,iHeight);
		sXor.SetSize(iBits,iWidth,iHeight);

		uColMask	= (iMask&0xFF);
		uColVal		= (uColMask-1)&0xFF;

		for(y=0;y<iHeight;y++)
		for(x=0;x<iWidth ;x++)
			{
			uPixel=pImageXor->GetPixel(x,y);
			if(uPixel==uColMask){uPixel= 0;sAnd.SetPixel(x,y,1);}
			if(uPixel==uColVal ){uPixel=15;sAnd.SetPixel(x,y,1);}
			sXor.SetPixel(x,y,uPixel);
			}

		try	{
			WriteImage(pFile,&sXor,&sAnd,iNew,iType);
			}
		catch(...)
			{
			throw;
			}

		}
	else{
		if(iMask>=0)
			{
			uColMask = 0x00FFFFFF;
			uColVal  = iMask;
			}
		else if(pImageXor->BitCount()<=8)
			{
			uColMask = 0x00FFFFFF;
			uColVal  = pImageXor->GetColor(255);
			}
		else{
			uColMask = 0xFF000000;
			uColVal  = 0xFF000000;
			}

		sAnd.SetSize(1    ,iWidth,iHeight);
		sXor.SetSize(iBits,iWidth,iHeight);

		for(y=0;y<iHeight;y++)
		for(x=0;x<iWidth ;x++)
			{
			uPixel=pImageXor->GetRGB(x,y);

			if((uPixel&uColMask)==uColVal)
				{
				sAnd.SetPixel(x,y,1);
				uPixel |= 0xFF000000;
				}

			sXor.SetPixel(x,y,uPixel);
			}

		try	{
			WriteImage(pFile,&sXor,&sAnd,iNew,iType);
			}
		catch(...)
			{
			throw;
			}
		}

}

void IconEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage *pImageXor,const BitmapImage *pImageAnd,int iNew,int iType)
{
ITEMDIR				sDir;
ICONDIRENTRY		aIcons[MAX_ICONS+1];
unsigned char	   *pMem  [MAX_ICONS+1];
BITMAPINFOHEADER   *pHeader;
unsigned		   *pRowPtr;
unsigned int 	   *pColors;
unsigned char	   *pData,*pPixels;
int					iSize,iColor,iRow,i;
int					iWidth,iHeight,iBits;
int					iCount=0,iOk,iPos;



	iBits		= pImageXor->BitCount();
	iWidth		= pImageXor->Width();
	iHeight		= pImageXor->Height();
	iColor		= (iBits>8)? 0:1<<iBits;
	sDir.idType	= (BYTE)iType;


	if((iBits!=32 && iBits!=24 && iBits!=8 && iBits!=4 && iBits!=1) || pImageAnd->BitCount()!=1)
		{
		throw EIconError("Falsches Farbformat !");
		}

	if(iWidth>256 || iHeight>256)
		{
		throw EIconError("Das Bild ist fr das Icon zu gro !");
		}

	iPos=pFile->Tell();


//******************** Die vorhandenen Bilder laden ***************************


	if(!iNew)
	do	{
		if(iPos<0)
			{
			throw EIconError("Zugriffsfehler auf Datei !");
			}

		if(pFile->Read(&sDir,6)!=6)break;

		   iCount=sDir.idCount;
		if(iCount>=MAX_ICONS-1)
			{
			throw EIconError("Zu viele Bilder im Icon !");
			}

		if(sDir.idType!=(BYTE)iType)
			{
			throw EIconError("Falsches Icon/Cursor Format !");
			}

		iSize = sizeof(ICONDIRENTRY)*iCount;
		
		if(pFile->Read(aIcons,iSize)!=iSize)
			{
			iCount=0;
			break;
			}

		for(i=0;i<iCount;i++)
			{
			   iSize=aIcons[i].dwBytesInRes;
			if(iSize>0x40100)break;

			pMem[i]=new unsigned char[iSize];

			   pFile->Seek(iPos+aIcons[i].dwImageOffset);
			if(pFile->Read(pMem[i],iSize)!=iSize)
				{
				delete pMem[i];
				break;
				}
			}

		}while(0);


//******************** Doppelte Bilder entfernen ******************************


	for(i=0;i<iCount;i++)
		{
		if( sDir.idType!=2)
		if( iBits !=aIcons[i].wBitCount    )continue;
		if((iWidth ^aIcons[i].bWidth )&0xFF)continue;
		if((iHeight^aIcons[i].bHeight)&0xFF)continue;
		delete pMem[i];
		iCount--;
		
		aIcons[i] = aIcons[iCount];
		pMem  [i] = pMem  [iCount]; 
		}


//******************** Bilddaten erzeugen *************************************


	if(iWidth>128 || iHeight>128)						// PNG erzeugen
		{
		CFileBuffer		sMem;
		PngEncoder		sPng;
		BitmapImage		sImg;


		if(iBits!=32)
			{
			sImg.ConvertToRGB(*pImageXor,32);
			pImageXor = &sImg;
			}

		for(iRow=pImageXor->Height()-1;iRow>=0;iRow--)	// Maske einstellen
			{
			pRowPtr = (unsigned*)pImageXor->GetRow(iRow);

			for(i=pImageXor->Width()-1;i>=0;i--)
				{
				if(pImageAnd->GetPixel(i,iRow))			
					{
					pRowPtr[i] &= 0x00FFFFFF;			// Pixel transparent
					}
				else if(!(pRowPtr[i]&0xFF000000))		
					{
					pRowPtr[i] |= 0xFF000000;			// Pixel sichtbar
					}
				}
			}

		try	{											// PNG in einem Speicher erzeugen
			sMem.OpenMem();
			sPng.WriteImage(&sMem,*pImageXor,1);
			}
		catch(...)
			{
			throw EIconError("Fehler beim Speichern des PNG-Bildes !");
			}			   		

		aIcons[iCount].bColorCount  = iColor;
		aIcons[iCount].bWidth       = iWidth;
		aIcons[iCount].bHeight      = iHeight;
		aIcons[iCount].bReserved	= 0;
		aIcons[iCount].wPlanes		= (sDir.idType==2)? (BYTE)(iType>>16):0;
		aIcons[iCount].wBitCount    = (sDir.idType==2)? (BYTE)(iType>>24):iBits;
		aIcons[iCount].dwBytesInRes = sMem.GetLength();
		pMem  [iCount]              = (unsigned char*)sMem.Buffer();

		iCount++;
		}
	else{												// BMP erzeugen
		i  =	((iWidth*iBits+31)/32)*iHeight*4;
		i +=	((iWidth      +31)/32)*iHeight*4;
		i +=    40+iColor*4;

		pMem  [iCount] = new unsigned char[i];
		aIcons[iCount].bColorCount  = iColor;
		aIcons[iCount].bWidth       = iWidth;
		aIcons[iCount].bHeight      = iHeight;
		aIcons[iCount].wPlanes		= (sDir.idType==2)? (BYTE)(iType>>16):0;
		aIcons[iCount].wBitCount    = (sDir.idType==2)? (BYTE)(iType>>24):iBits;
		aIcons[iCount].bReserved	= 0;
		aIcons[iCount].dwBytesInRes = i;

		pData = pMem[iCount];
		iCount++;

		pHeader = (BITMAPINFOHEADER*)pData;
		memset(pHeader,0,40);

		pHeader->biSize		 = 40;
		pHeader->biBitCount  = iBits;
		pHeader->biPlanes    = 1;
		pHeader->biWidth	 = iWidth;
		pHeader->biHeight	 = iHeight*2;
		pHeader->biSizeImage = ((iWidth*iBits+31)/32)*iHeight*4;


		pColors = (unsigned*)(pData+40);

		for(i=0;i<iColor;i++)						// Farben einstellen
			{
			pColors[i]=pImageXor->GetColor(i);
			}

		pPixels = (unsigned char*)(pColors+iColor);
		iSize   = ((iWidth*iBits+31)/32)*4;


		for(i=iHeight-1;i>=0;i--)					// Pixel erstellen
			{
			memcpy(pPixels,pImageXor->GetRow(i),iSize);
			pPixels+=iSize;
			}

		iSize=((iWidth+31)/32)*4;

		for(i=iHeight-1;i>=0;i--)					// Maske erstellen
			{
			memcpy(pPixels,pImageAnd->GetRow(i),iSize);
			pPixels+=iSize;
			}
		}


//******************** Bilder speichern ***************************************


	if(iPos>=0)pFile->Seek(iPos);

	iPos = 6+sizeof(ICONDIRENTRY)*iCount;

	for(i=0;i<iCount-1;i++)							// Sortiere Bilder
		{
		if(sDir.idType==2)
		if(aIcons[i].wBitCount <= aIcons[i+1].wBitCount)continue;
		if(aIcons[i].bHeight   <= aIcons[i+1].bHeight  )continue;
		if(aIcons[i].bWidth    <= aIcons[i+1].bWidth   )continue;

		#define TMP  MAX_ICONS	

		aIcons[TMP]=aIcons[i  ];
		aIcons[i  ]=aIcons[i+1];
		aIcons[i+1]=aIcons[TMP];
		pMem  [TMP]=pMem  [i  ];
		pMem  [i  ]=pMem  [i+1];
		pMem  [i+1]=pMem  [TMP];

		i = -1;
		}

	for(i=0;i<iCount;i++)							// Berechne Datenoffsets
		{
		aIcons[i].dwImageOffset = iPos;
		iPos+=aIcons[i].dwBytesInRes;
		}


	do	{											// Speichere Bilder
		iOk				= 0;
		sDir.idReserved = 0;

		if(pFile->Write(&sDir  ,4)!=4)break;
		if(pFile->Write(&iCount,2)!=2)break;

		iSize=sizeof(ICONDIRENTRY)*iCount;
		if(pFile->Write(aIcons,iSize)!=iSize)break;

		for(i=0;i<iCount;i++)
			{
			iSize=aIcons[i].dwBytesInRes;
			if(pFile->Write(pMem[i],iSize)!=iSize)break;
			}

		if(i>=iCount)iOk=1;
		}while(0);

	for(i=0;i<iCount;i++)
		{
		delete pMem[i];
		}

	if(!iOk)
		{
		throw EIconError("Fehler beim Schreiben in die Datei !");
		}
}
