//*****************************************************************************
//*
//*
//*		ImageLoader.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"BmpDecoder.h"
#include	"EpsDecoder.h"
#include	"GifDecoder.h"
#include	"PngDecoder.h"
#include	"PnmDecoder.h"
#include	"TlsDecoder.h"
#include	"XpmDecoder.h"
#include	"XwdDecoder.h"
#include	"JpegDecoder.h"
#include	"TiffDecoder.h"
#include	"IconDecoder.h"
#include	"ImageLoader.h"

#ifndef _WCHAR_T_DEFINED
	#if _MSC_VER 
	#include	<wchar.h>
	#else
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif 	
#endif 	

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	fzopen		_wfopen
#define 	strzcpy		wcscpy
#define 	strzncpy	wcsncpy
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	fzopen		fopen
#define 	strzcpy		strcpy
#define 	strzncpy	strncpy
#endif

static	int		iTransparency   = -1;
static	zchar	cLastError[512] = Z("");

//*****************************************************************************
//*
//*		ImageLoaderError
//*
//*****************************************************************************
//	Ergibt den Text der letzten Fehlermeldung
const zchar *ImageLoaderError()
{
return cLastError;
}

//*****************************************************************************
//*
//*		ImageLoaderTransparency
//*
//*****************************************************************************
//	Holt die Transperente Farbe von Bildern
int ImageLoaderTransparency()
{
return iTransparency;
}

//*****************************************************************************
//*
//*		ImageLoader
//*
//*****************************************************************************
//	Ld ein Bild und ermittelt automatisch den Typ der Datei.
//	pName		: Ist der Dateiname des Bildes.
//	pImage		: Zeiger auf die Klasse in die das Bild geladen werden soll.	
//	pComment	: Hier wird der Zeiger auf den Kommentar gespeichert.
//	Ergibt:	 0	Bild geladen
//			-1	Fehler beim Bildladen
//			-2	Unbekanntes Dateiformat
//			-3	Kann Datei nicht ffnen
//			-4  Das Bild ist zu gro
//			-5  Falsches Farbformat
//			-6	Zuwenig Speicher
//			-7	Unbekannter Fehler
int ImageLoader(const zchar *pName,BitmapImage *pImage,void *pComment)
{
char			  **pComPt=(char**)pComment;
const char		   *pText;
BitmapImageDecoder *pDecoder=0;
int					i,iType=-1;
char				cBuffer[65];
CFileBuffer			sFile;


	if(pComment)
		{
		*pComPt = 0;
		}

	if(!sFile.Open(pName))
		{
		strzcpy(cLastError,Z("Kann Datei nicht ffnen."));
		return -3;
		}

	sFile.Read(cBuffer,64);									// Header einlesen
	
	cBuffer[64]   = 0;
	cLastError[0] = 0;

	if(*(int*)cBuffer==40 && !cBuffer[6] && !cBuffer[7] && !cBuffer[10] && !cBuffer[11])
		{
		iType = 0;											// BMP aus Resourcen
		}

															
	if(cBuffer[0]==0 && cBuffer[1]==0 && cBuffer[2]>= 1 && cBuffer[2]<=2) 
	if(cBuffer[3]==0 && cBuffer[5]==0 && cBuffer[4]<=32 && cBuffer[4]>=1)
		{
		iType = 10;											// ICON-Herader
		}
	
	for(i=0;i<64;i++)										// Null zu Leerzeichen
		{
		if(!cBuffer[i])cBuffer[i]=' ';
		}


	if(!strncmp(cBuffer,"II"     ,2))iType=4;				// Format ermitteln
	if(!strncmp(cBuffer,"MM"     ,2))iType=4;
	if(!strncmp(cBuffer,"GIF87a" ,6))iType=2;
	if(!strncmp(cBuffer,"GIF89a" ,6))iType=2;
	if(!strncmp(cBuffer,"\x89PNG",4))iType=3;
	if(!strncmp(cBuffer,"BM"     ,2))iType=0;
	if(!strncmp(cBuffer,"TLSP"   ,4))iType=9;
	if( strstr (cBuffer,"JFIF"     ))iType=1;
	if( strstr (cBuffer,"XPM"      ))iType=5;
	if( strstr (cBuffer,"char"     ))iType=5;
	if( cBuffer[0]=='\xFF' && cBuffer[1]=='\xD8')iType=1;
	if( cBuffer[0]=='\xFF' && cBuffer[1]=='\xDB')iType=1;
	if( cBuffer[0]=='%'    && cBuffer[1]=='!'   )iType=8;
	if( cBuffer[0]=='P'    && cBuffer[1]>='1'    && cBuffer[1]<='6')iType=7;
	if( cBuffer[4]=='\x20' && cBuffer[5]=='\x20' && cBuffer[6]=='\x20')
	if( cBuffer[7]=='\x07' || cBuffer[7]=='\x06')iType=6;
	

	try	{
		switch(iType)
				{
		case  0:{pDecoder = new  BmpDecoder;	break;}
		case  1:{pDecoder = new JpegDecoder;	break;}
		case  2:{pDecoder = new  GifDecoder;	break;}
		case  3:{pDecoder = new  PngDecoder;	break;}
		case  4:{pDecoder = new TiffDecoder;	break;}
		case  5:{pDecoder = new  XpmDecoder;	break;}
		case  6:{pDecoder = new  XwdDecoder;	break;}
		case  7:{pDecoder = new  PnmDecoder;	break;}
		case  8:{pDecoder = new  EpsDecoder	;	break;}
		case  9:{pDecoder = new  TlsDecoder;	break;}
		case 10:{pDecoder = new IconDecoder;	break;}

		default:strzcpy(cLastError,Z("Unbekanntes Datei Format."));
				return -2 ;
				}

		sFile.Seek(0);
		pDecoder->ReadImage(&sFile,*pImage);

		if(pComment)
			{
			pText = pDecoder->GetComment();

			if(pText && *pText)
				{
				i		= strlen(pText)+1;
				*pComPt = new char [i+1];
				memcpy(*pComPt,pText,i);
				}
			}
		}
	catch(EGraphicsException &sErr)
		{
		strzncpy(cLastError,sErr.what(),sizeof(cLastError)/sizeof(zchar)-1);
		cLastError[sizeof(cLastError)-1]=0;
		if(pDecoder)delete pDecoder;
		i = sErr.code();
		if(i==EG_CODE_TOOBIG    )return -4;
		if(i==EG_CODE_WRONGCOLOR)return -5;
		if(i==EG_CODE_MEMORY    )return -6;
		return -1;
		}
	catch(...)
		{
		strzncpy(cLastError,Z("Unbekannter Fehler"),sizeof(cLastError)/sizeof(zchar)-1);
		if(pDecoder)delete pDecoder;
		return -7;
		}

	if(iType==2)
			iTransparency = ((GifDecoder*)pDecoder)->TransparentColor();
	else	iTransparency = -1;

	if(pDecoder)delete pDecoder;


return 0;
}

//*****************************************************************************
//*
//*		ImageLoader
//*
//*****************************************************************************
//	Ld ein Bild aus einem Speicher und ermittelt automatisch den Typ.
//	pName		: Ist der Dateiname des Bildes.
//	pImage		: Zeiger auf die Klasse in die das Bild geladen werden soll.	
//	pComment	: Hier wird der Zeiger auf den Kommentar gespeichert.
//	Ergibt:	 0	Bild geladen
//			-1	Fehler beim Bildladen
//			-2	Unbekanntes Dateiformat
//			-3	Kann Datei nicht ffnen
//			-4	Unbekannter Fehler
int ImageLoader(const void *pMem,int iSize,BitmapImage *pImage,void *pComment)
{
char			  **pComPt=(char**)pComment;
const char		   *pText;
CFileBuffer			sFileBuffer;
BitmapImageDecoder *pDecoder=0;
int					i,iType=-1;
char				cBuffer[65];


	if(pComment)
		{
		*pComPt = 0;
		}

	cLastError[0] = 0;

	if(!pMem || iSize<64)									// Prfe den Puffer
		{
		strzcpy(cLastError,Z("Unbekanntes Datei Format."));
		return -2;
		}

	memcpy(cBuffer,pMem,64);
	cBuffer[64] = 0;

	if(*(int*)cBuffer==40 && !cBuffer[6] && !cBuffer[7] && !cBuffer[10] && !cBuffer[11])
		{
		iType = 0;											// BMP aus Resourcen
		}

															
	if(cBuffer[0]==0 && cBuffer[1]==0 && cBuffer[2]>= 1 && cBuffer[2]<=2) 
	if(cBuffer[3]==0 && cBuffer[5]==0 && cBuffer[4]<=32 && cBuffer[4]>=1)
		{
		iType = 10;											// ICON-Herader
		}

	for(i=0;i<64;i++)										// Null zu Leerzeichen
		{
		if(!cBuffer[i])cBuffer[i]=' ';
		}


	if(!strncmp(cBuffer,"II"     ,2))iType=4;				// Format ermitteln
	if(!strncmp(cBuffer,"MM"     ,2))iType=4;
	if(!strncmp(cBuffer,"GIF87a" ,6))iType=2;
	if(!strncmp(cBuffer,"GIF89a" ,6))iType=2;
	if(!strncmp(cBuffer,"\x89PNG",4))iType=3;
	if(!strncmp(cBuffer,"BM"     ,2))iType=0;
	if(!strncmp(cBuffer,"TLSP"   ,4))iType=9;
	if( strstr (cBuffer,"JFIF"     ))iType=1;
	if( strstr (cBuffer,"XPM"      ))iType=5;
	if( cBuffer[0]=='\xFF' && cBuffer[1]=='\xD8')iType=1;
	if( cBuffer[0]=='\xFF' && cBuffer[1]=='\xDB')iType=1;
	if( cBuffer[0]=='%'    && cBuffer[1]=='!'   )iType=8;
	if( cBuffer[0]=='P'    && cBuffer[1]>='1'    && cBuffer[1]<='6')iType=7;
	if( cBuffer[4]=='\x20' && cBuffer[5]=='\x20' && cBuffer[6]=='\x20')
	if( cBuffer[7]=='\x07' || cBuffer[7]=='\x06')iType=6;

	try	{
		switch(iType)
				{
		case  0:{pDecoder = new  BmpDecoder;	break;}
		case  1:{pDecoder = new JpegDecoder;	break;}
		case  2:{pDecoder = new  GifDecoder;	break;}
		case  3:{pDecoder = new  PngDecoder;	break;}
		case  4:{pDecoder = new TiffDecoder;	break;}
		case  5:{pDecoder = new  XpmDecoder;	break;}
		case  6:{pDecoder = new  XwdDecoder;	break;}
		case  7:{pDecoder = new  PnmDecoder;	break;}
		case  8:{pDecoder = new  EpsDecoder	;	break;}
		case  9:{pDecoder = new  TlsDecoder;	break;}
		case 10:{pDecoder = new IconDecoder;	break;}

		default:strzcpy(cLastError,Z("Unbekanntes Datei Format."));
				return -2 ;
				}

		sFileBuffer.OpenMem((void*)pMem,iSize);

		pDecoder->ReadImage(&sFileBuffer,*pImage);

		if(pComment)
			{
			pText = pDecoder->GetComment();

			if(pText && *pText)
				{
				i		= strlen(pText)+1;
				*pComPt = new char [i+1];
				memcpy(*pComPt,pText,i);
				}
			}
		}
	catch(EGraphicsException &sErr)
		{
		strzncpy(cLastError,sErr.what(),sizeof(cLastError)/sizeof(zchar)-1);
		cLastError[sizeof(cLastError)-1]=0;
		if(pDecoder)delete pDecoder;
		return -1;
		}
	catch(...)
		{
		strzncpy(cLastError,Z("Unbekannter Fehler"),sizeof(cLastError)/sizeof(zchar)-1);
		if(pDecoder)delete pDecoder;
		return -4;
		}

	if(iType==2)
			iTransparency = ((GifDecoder*)pDecoder)->TransparentColor();
	else	iTransparency = -1;

	if(pDecoder)delete pDecoder;


return 0;
}

