//*****************************************************************************
//*
//*
//*		ImageSaver
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
   
#include	"ImageSaver.h"
#include	"BmpEncoder.h"
#include	"EpsEncoder.h"
#include	"GifEncoder.h"
#include	"PngEncoder.h"
#include	"PnmEncoder.h"
#include	"TlsEncoder.h"
#include	"XpmEncoder.h"
#include	"XwdEncoder.h"
#include	"JpegEncoder.h"
#include	"TiffEncoder.h"

#ifndef _WCHAR_T_DEFINED
	#if _MSC_VER 
	#include	<wchar.h>
	#else
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif 	
#endif 	

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	strzncpy	wcsncpy
#define 	strzicmp	wcsicmp
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	strzncpy	strncpy
#define 	strzicmp	stricmp
#endif


static	zchar	cLastError[512]=Z("");
static	int		iSetTransparency=-2;

//*****************************************************************************
//*
//*		ImageSaverError
//*
//*****************************************************************************
const zchar *ImageSaverError()
{
return cLastError;
}

//*****************************************************************************
//*
//*		ImageSaverTransparency
//*
//*****************************************************************************
//	Stellt die Transperente Farbe von Bildern ein
void ImageSaverTransparency(int iValue)
{
	iSetTransparency = iValue;
}

//*****************************************************************************
//*
//*		ImageSaver
//*
//*****************************************************************************
//	Ld ein Bild
//	pName		: Ist der Dateiname des Bildes.
//	pImage		: Zeiger auf die Klasse in die das Bild gespeichert werden soll.	
//	pComment	: Ist der Kommentar der eingefgt werden soll
//	Ergibt:	 0	Bild wurde gespeichert
//			-1	Fehler beim Bild speichern
//			-2	Unbekanntes Dateiformat
//			-3	Kann Datei nicht ffnen
//			-4	Unbekannter Fehler
int ImageSaver(const zchar *pName,const BitmapImage *pImage,const zchar *pComment)
{
BitmapImageEncoder *pEncoder=0;
int					i,iType=-1;

	
	
	for(i=0;pName[i];i++);

	for(i--;i>0;i--)									// Suche Suffix
		{
		if(pName[i]=='.')break;
		}


		 if(!strzicmp(pName+i,Z(".PNG" )))iType=3;
	else if(!strzicmp(pName+i,Z(".GIF" )))iType=2;
	else if(!strzicmp(pName+i,Z(".BMP" )))iType=1;
	else if(!strzicmp(pName+i,Z(".JPG" )))iType=0;
	else if(!strzicmp(pName+i,Z(".JPEG")))iType=0;
	else if(!strzicmp(pName+i,Z(".XWD" )))iType=6;
	else if(!strzicmp(pName+i,Z(".XPM" )))iType=5;
	else if(!strzicmp(pName+i,Z(".PNM" )))iType=7;
	else if(!strzicmp(pName+i,Z(".TIFF")))iType=4;
	else if(!strzicmp(pName+i,Z(".TIF" )))iType=4;
	else if(!strzicmp(pName+i,Z(".PS"  )))iType=8;
	else if(!strzicmp(pName+i,Z(".EPS" )))iType=8;
	else if(!strzicmp(pName+i,Z(".PRN" )))iType=8;
	else if(!strzicmp(pName+i,Z(".TLSP")))iType=9;

	try	{
		switch(iType)
			{
		case 0:	{pEncoder = new JpegEncoder;	break;}
		case 1:	{pEncoder = new  BmpEncoder;	if(iSetTransparency>=-1 && pImage->BitCount()==32)((BmpEncoder*)pEncoder)->SetFormat((iSetTransparency>=0)? 32:24);break;}
		case 2:	{pEncoder = new  GifEncoder;	if(iSetTransparency>=-1                          )((GifEncoder*)pEncoder)->SetTransparentColor(iSetTransparency)  ;break;}
		case 3:	{pEncoder = new  PngEncoder;	if(iSetTransparency>=-1 && pImage->BitCount()==32)((PngEncoder*)pEncoder)->SetMode((iSetTransparency>=0)? 1:0  )  ;break;}
		case 4:	{pEncoder = new TiffEncoder;	break;}
		case 5:	{pEncoder = new  XpmEncoder;	break;}
		case 6:	{pEncoder = new  XwdEncoder;	break;}
		case 7:	{pEncoder = new  PnmEncoder;	break;}
		case 8:	{pEncoder = new  EpsEncoder;	break;}
		case 9:	{pEncoder = new  TlsEncoder;	break;}

		default:strzncpy(cLastError,Z("Unbekanntes Datei Format."),sizeof(cLastError)/sizeof(zchar)-1);
				return -2 ;
			}

		if(pComment)
			{
			pEncoder->SetComment(pComment);
			}

		pEncoder->WriteImage(pName,*pImage);	
		}
	catch(EGraphicsException &sErr)
		{
		strzncpy(cLastError,sErr.what(),sizeof(cLastError)/sizeof(zchar)-1);
		cLastError[sizeof(cLastError)]=0;
		if(pEncoder)delete pEncoder;
		return -1;
		}
	catch(...)
		{
		strzncpy(cLastError,Z("Unbekannter Fehler"),sizeof(cLastError)/sizeof(zchar)-1);
		if(pEncoder)delete pEncoder;
		return -4;
		}

	if(pEncoder)delete pEncoder;


return 0;
}
