//*****************************************************************************
//*
//*
//*		JpegDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Klasse zum Lesen von *.jpeg Bildern
//
//
//	(C) Copyright Anton Zechner 2007
//
#include	<stdio.h>
#include	"BitmapImage.h"
#include	"JpegDecoder.h"
#include	"GrException.h"
#include	"Jpeg\JpegLib.h"
#include	"..\FileBuffer.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif 	






//*****************************************************************************
//*
//*	    Konstructor
//*
//*****************************************************************************
JpegDecoder::JpegDecoder()
{
	Initialize();
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
JpegDecoder::~JpegDecoder()
{
}

//*****************************************************************************
//*
//*		Initialize
//*
//*****************************************************************************
void JpegDecoder::Initialize()
{
	cErrorString[0]=0;
}

//*****************************************************************************
//*
//*		DoCopy
//*
//*****************************************************************************
void JpegDecoder::DoCopy(const JpegDecoder &Source)
{
	BitmapImageDecoder::DoCopy(Source);
}

//*****************************************************************************
//*
//*		UpdateImage
//*
//*****************************************************************************
void JpegDecoder::UpdateImage()
{
}


//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
//	Ld die Datei Name ins sImage
void JpegDecoder::ReadImage(const char *pName,BitmapImage &sImage)
{
CFileBuffer	sFileBuf;


	if(!sFileBuf.Open(pName))throw EJpegOpenError();

	try	{
		ReadImage(&sFileBuf,sImage);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();

}

void JpegDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage)
{
CFileBuffer	sFileBuf;


	if(!sFileBuf.Open(pName))throw EJpegOpenError();

	try	{
		ReadImage(&sFileBuf,sImage);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();

}

void JpegDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage)
{
struct jpeg_decompress_struct sInfo;
struct		jpeg_error_mgr	  sErr;
JSAMPARRAY		sBuffer;
int				iRowStride;
int				iPercent;
bool			bCancel;
unsigned int	uPos;
unsigned char  *pMemB;
unsigned char  *pMem;




	bCancel			= false;
	sInfo.err		= jpeg_std_error(&sErr);
	sErr.error_exit = &JpegError;

	//jpeg_set_marker_processor (&sInfo, JPEG_COM, ReadComment);
	jpeg_create_decompress    (&sInfo);
	jpeg_stdio_src            (&sInfo,pFile);
	jpeg_save_markers         (&sInfo,JPEG_COM, 0x1000);
	jpeg_read_header          (&sInfo,TRUE);
	jpeg_start_decompress     (&sInfo);


	iRowStride  =   sInfo.output_width * sInfo.output_components;
	sBuffer     = (*sInfo.mem->alloc_sarray)((j_common_ptr)&sInfo,JPOOL_IMAGE,iRowStride,1);



//******************** Bilder Zeilen laden ************************************

	try {
		switch(sInfo.output_components)
				{
		case 3:										// RGB - Farben

				if(!sImage.SetSize(24,sInfo.output_width,sInfo.output_height,BI_NOT_INIT))
					{
					throw sImage.LastError();
					}

				while(sInfo.output_scanline<sInfo.output_height)
					{
					if(pProgressProc)				// Verlauf Funktion
						{
						iPercent=(100*sInfo.output_scanline)/sInfo.output_height;
						pProgressProc (*this, pProgressData, 1, 1, iPercent, bCancel) ;
						if(bCancel)break;
						}

					pMem=sImage[sInfo.output_scanline];
					jpeg_read_scanlines(&sInfo, sBuffer, 1);
					pMemB=*sBuffer;

					for(uPos=0;uPos<sInfo.output_width;uPos++,pMem+=3,pMemB+=3)
						{
						pMem[0]=pMemB[2];
						pMem[1]=pMemB[1];
						pMem[2]=pMemB[0];
						}
					}
				break;

		case 1:										// Graustuffen

				if(sImage.SetSize(24,sInfo.output_width,sInfo.output_height,BI_NOT_INIT))
					{
					throw sImage.LastError();
					}

				while(sInfo.output_scanline < sInfo.output_height)
					{
					if(pProgressProc)			// Verlauf Funktion
						{
						iPercent=(100*sInfo.output_scanline)/sInfo.output_height;
						pProgressProc (*this, pProgressData, 1, 1, iPercent, bCancel) ;
						if(bCancel)break;
						}

					pMem=sImage[sInfo.output_scanline];
					jpeg_read_scanlines(&sInfo,sBuffer,1);

					for(uPos=0;uPos<sInfo.output_width;uPos++,pMem+=3)
						{
						pMem[0]=pMem[1]=pMem[2]=(*sBuffer)[uPos];
						}
					}


		default:break;
				}
		}
	catch(int iError)
		{
		SetComment((char*)0);
		
		try	{
			jpeg_finish_decompress (&sInfo);
			jpeg_destroy_decompress(&sInfo);
			}
		catch(...)
			{
			}

		ThrowImageError(iError);
		}
	catch(EGraphicsException &sExep)
		{
		SetComment((char*)0);
		jpeg_finish_decompress (&sInfo);
		jpeg_destroy_decompress(&sInfo);
		throw sExep;
		}

//****************** Ende *****************************************************

	if(!bCancel && pProgressProc)					// Verlaufs Funktion
		{
		iPercent=100;
		pProgressProc(*this, pProgressData, 1, 1, 100, bCancel) ;
		bCancel=false;
		}

	if(sInfo.marker_list && sInfo.marker_list->data && sInfo.marker_list->data_length>0)
		{
		jpeg_saved_marker_ptr pMarker;
		char	             *pPtr;
		int					  iLen;

		iLen	= 0;
		pMarker	= sInfo.marker_list;

		while(pMarker)
			{
			iLen   += pMarker->data_length;
			pMarker = pMarker->next;
			}

		pPtr	   = new char[iLen+1];
		pPtr[iLen] = 0;
		iLen	   = 0;
		pMarker	   = sInfo.marker_list;

		while(pMarker)
			{
			memcpy(pPtr+iLen,pMarker->data,pMarker->data_length);
			iLen	+= pMarker->data_length;
			pMarker  = pMarker->next;
			}

		SetComment(pPtr);
		delete pPtr;
		}
	else{
		SetComment((char*)0);
		}


	jpeg_finish_decompress (&sInfo);
	jpeg_destroy_decompress(&sInfo);


	if(bCancel)throw EJpegAbort();

}




//*****************************************************************************
//*
//*		JpegError
//*
//*****************************************************************************
void JpegDecoder::JpegError(jpeg_common_struct *pInfo)
{
JpegDecoder	 *pDecoder;


	pDecoder = (JpegDecoder*)pInfo->err;
	sprintf(pDecoder->cErrorString,"Fehler %i beim Laden der Jpeg Datei",pInfo->err->msg_code);

	throw EJpegError(pDecoder->cErrorString);
}

//*****************************************************************************
//*
//*		ThrowImageError
//*
//*****************************************************************************
void JpegDecoder::ThrowImageError(int iError)
{


	switch(iError)
		{
	case 1:	throw EGraphicsPicSize();
	case 2:	throw EGraphicsWrongColor();
	case 3:	throw EGraphicsMemory();
		}

	throw EGraphicsException("Unknown error");

};




