//*****************************************************************************
//*
//*
//*		JpegEncoder.cpp
//*
//*
//*****************************************************************************
//
//	Klasse zum Schreiben von *.jpeg Dateien
//
//	(C) Copyright Anton Zechner 2007
//
#include	<math.h>
#include	<string>
#include	<stdio.h>

#include	"BitmapImage.h"
#include	"JpegEncoder.h"
#include	"Jpeg\JpegLib.h"
#include	"..\FileBuffer.h"



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
JpegEncoder::JpegEncoder()
{
	Initialize();
}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
JpegEncoder::~JpegEncoder()
{
}

//*****************************************************************************
//*
//*		DoCopy
//*
//*****************************************************************************
void JpegEncoder::DoCopy(const JpegEncoder &sSource)
{

	m_bProgressiveMode	= sSource.m_bProgressiveMode;
	m_iImageQuality		= sSource.m_iImageQuality;
	m_bGrayScale		= sSource.m_bGrayScale;

	BitmapImageEncoder::DoCopy(sSource);
}

//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
void JpegEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{
CFileBuffer	sFileBuf;


	if(!sFileBuf.Open(pName,CFB_CREATE))throw EJpegOpenError();

	try	{
		WriteImage(&sFileBuf,sImage);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();
}

void JpegEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{
CFileBuffer	sFileBuf;


	if(!sFileBuf.Open(pName,CFB_CREATE))throw EJpegOpenError();

	try	{
		WriteImage(&sFileBuf,sImage);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();
}

void JpegEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
struct			jpeg_compress_struct sInfo;
struct			jpeg_error_mgr		 sErr;
unsigned char  *pBuffer;
unsigned char  *pMem;
unsigned char  *pMemB;
unsigned char   ucMemR;
unsigned char   ucMemB;
unsigned char	ucMemG;
bool			bCancel;
JSAMPROW		aRowPointer[1];
int				iImageHeight;
int				iImageWidth;
int				iRowStride;
int				iPercent;
int				iGray;
int				i;


	pMemB	= 0;
	ucMemR	= 0;
	ucMemB	= 0;
	ucMemG	= 0;
	bCancel	= false;

	sInfo.err = jpeg_std_error(&sErr);
	jpeg_create_compress(&sInfo);

	if(!sImage.IsOk())throw EJpegBadData();

	iImageWidth  = sImage.Width();
	iImageHeight = sImage.Height();


	jpeg_stdio_dest(&sInfo, pFile);


	sInfo.image_width      =  iImageWidth;			// image width and height, in pixels */
	sInfo.image_height     =  iImageHeight;			//				*
	sInfo.input_components = (m_bGrayScale)? 1:3;	// of color components per pixel */
	sInfo.in_color_space   = (m_bGrayScale)? JCS_GRAYSCALE:JCS_RGB;
	sInfo.progressive_mode =  m_bProgressiveMode;

	jpeg_set_defaults  (&sInfo);
	jpeg_set_quality   (&sInfo, m_iImageQuality, TRUE);
	jpeg_start_compress(&sInfo, TRUE);
	iRowStride	= iImageWidth * sInfo.input_components;
	pBuffer		= new unsigned char[iRowStride+16];


	if(this->iCommentLen>0)
		{
		jpeg_write_marker(&sInfo,JPEG_COM,(JOCTET*)pComment,iCommentLen);
		}

//******************** Zeilen schreiben ***************************************
	try {
		if(!m_bGrayScale)
			{
			while(sInfo.next_scanline < sInfo.image_height)
				{
				if(pProgressProc)			// Verlauf Funktion
					{
					iPercent=(100*sInfo.next_scanline)/sInfo.image_height;
					pProgressProc(*this,pProgressData,1,1,iPercent,bCancel);
					if(bCancel)break;
					}

				 pMem=sImage.GetRow3Temp(sInfo.next_scanline);
					  sImage.SwapRGB3(pMem,sInfo.image_width);

				aRowPointer[0] = pMem;
				jpeg_write_scanlines(&sInfo, aRowPointer, 1);
				}
			}
		if(m_bGrayScale)
			{
			while (sInfo.next_scanline < sInfo.image_height)
				{
				if(pProgressProc)			// Verlauf Funktion
					{
					iPercent =   (100*sInfo.next_scanline)/sInfo.image_height;
					pProgressProc(*this,pProgressData,1,1,iPercent,bCancel);
					if(bCancel)break;
					}

				pMem=sImage.GetRow3(sInfo.next_scanline);

				for(i=0;i<(int)sInfo.image_width;i++,pMem+=3,pMemB++)
					{
					iGray    = (int)pMem[2]*77 + (int)pMem[1]*151 + (int)pMem[0]*29;
					pMemB[0] = iGray>>8;
					}

				aRowPointer[0] = pBuffer;
				jpeg_write_scanlines(&sInfo, aRowPointer, 1);
				}
			}
		}
	catch(EGraphicsException &ee)
		{
		jpeg_finish_compress (&sInfo);
		jpeg_destroy_compress(&sInfo);
		throw ee;
		}

//******************** Ende ***************************************************

	jpeg_finish_compress (&sInfo);
	jpeg_destroy_compress(&sInfo);


	if(pBuffer)delete pBuffer;
}


//*****************************************************************************
//*
//*		Initialize
//*
//*****************************************************************************
void JpegEncoder::Initialize()
{

	m_iImageQuality		= 75;
	m_bProgressiveMode	= false;
	m_bGrayScale		= false;

}


//*****************************************************************************
//*
//*		SetProgressive
//*
//*****************************************************************************
void JpegEncoder::SetProgressive(bool bValue)
{
	m_bProgressiveMode = bValue;
}


//*****************************************************************************
//*
//*		GetQuality
//*
//*****************************************************************************
unsigned int JpegEncoder::GetQuality() const
{
	return m_iImageQuality;
}


//*****************************************************************************
//*
//*		SetQuality
//*
//*****************************************************************************
void JpegEncoder::SetQuality(unsigned uValue)
{

	if(uValue<1  )uValue=1;
	if(uValue>100)uValue=100;

	m_iImageQuality = uValue;

}

//*****************************************************************************
//*
//*		GetGrayscale
//*
//*****************************************************************************
bool JpegEncoder::GetGrayscale() const
{
return m_bGrayScale;
}

//*****************************************************************************
//*
//*		SetGrayscale
//*
//*****************************************************************************
void JpegEncoder::SetGrayscale(bool bValue)
{
	m_bGrayScale = bValue;
}
