//*****************************************************************************
//*
//*
//*		pngzutil.c
//*
//*
//*****************************************************************************
#include	<malloc.h>

static volatile int	crc_table_empty = 1;


#define BASE 65521UL							// largest prime smaller than 65536 
#define NMAX 5552


#define DO1(buf,i)  {adler += (buf)[i]; sum2 += adler;}
#define DO2(buf,i)  DO1(buf,i); DO1(buf,i+1);
#define DO4(buf,i)  DO2(buf,i); DO2(buf,i+2);
#define DO8(buf,i)  DO4(buf,i); DO4(buf,i+4);
#define DO16(buf)   DO8(buf,0); DO8(buf,8);


#ifdef NO_DIVIDE

	#define MOD(a) \
		do { \
			if (a >= (BASE << 16)) a -= (BASE << 16); \
			if (a >= (BASE << 15)) a -= (BASE << 15); \
			if (a >= (BASE << 14)) a -= (BASE << 14); \
			if (a >= (BASE << 13)) a -= (BASE << 13); \
			if (a >= (BASE << 12)) a -= (BASE << 12); \
			if (a >= (BASE << 11)) a -= (BASE << 11); \
			if (a >= (BASE << 10)) a -= (BASE << 10); \
			if (a >= (BASE <<  9)) a -= (BASE <<  9); \
			if (a >= (BASE <<  8)) a -= (BASE <<  8); \
			if (a >= (BASE <<  7)) a -= (BASE <<  7); \
			if (a >= (BASE <<  6)) a -= (BASE <<  6); \
			if (a >= (BASE <<  5)) a -= (BASE <<  5); \
			if (a >= (BASE <<  4)) a -= (BASE <<  4); \
			if (a >= (BASE <<  3)) a -= (BASE <<  3); \
			if (a >= (BASE <<  2)) a -= (BASE <<  2); \
			if (a >= (BASE <<  1)) a -= (BASE <<  1); \
			if (a >=  BASE) a -= BASE; \
		} while (0)
	#define MOD4(a) \
		do { \
			if (a >= (BASE << 4)) a -= (BASE << 4); \
			if (a >= (BASE << 3)) a -= (BASE << 3); \
			if (a >= (BASE << 2)) a -= (BASE << 2); \
			if (a >= (BASE << 1)) a -= (BASE << 1); \
			if (a >= BASE) a -= BASE; \
		} while (0)

#else

	#define MOD(a)  a %= BASE
	#define MOD4(a) a %= BASE

#endif

#define		REV(w)			(((w)>>24)+(((w)>>8)&0xff00)+ 	\
							(((w)&0xff00)<<8)+(((w)&0xff)<<24))

#define		CDO1 			crc = crc_table[((int)crc ^ (*buf++)) & 0xff] ^ (crc >> 8)
#define		CDO8 			CDO1; CDO1; CDO1; CDO1; CDO1; CDO1; CDO1; CDO1

static	unsigned long		crc_table[256];


//*****************************************************************************
//*
//*		make_crc_table
//*
//*****************************************************************************
// 
// Generate tables for a byte-wise 32-bit CRC calculation on the polynomial:
// x^32+x^26+x^23+x^22+x^16+x^12+x^11+x^10+x^8+x^7+x^5+x^4+x^2+x+1.
// 
// Polynomials over GF(2) are represented in binary, one bit per coefficient,
// with the lowest powers in the most significant bit.  Then adding polynomials
// is just exklusive-or, and multiplying a polynomial by x is a right shift by
// one.  If we call the above polynomial p, and represent a byte as the
// polynomial q, also with the lowest power in the most significant bit (so the
// byte 0xb1 is the polynomial x^7+x^3+x+1), then the CRC is (q*x^32) mod p,
// where a mod b means the remainder after dividing a by b.
// 
// This calculation is done using the shift-register method of multiplying and
// taking the remainder.  The register is initialized to zero, and for each
// incoming bit, x^32 is added mod p to the register if the bit is a one (where
// x^32 mod p is p+x^32 = x^26+...+1), and the register is multiplied mod p by
// x (which is shifting right by one and adding x^32 mod p if the bit shifted
// out is a one).  We start with the highest power (least significant bit) of
// q and repeat for all eight bits of q.
// 
// The first table is simply the CRC of all possible eight bit values.  This is
// all the information needed to generate CRCs on data a byte at a time for all
// combinations of CRC register values and incoming bytes.  The remaining tables
// allow for word-at-a-time CRC calculation for both big-endian and little-
// endian machines, where a word is four bytes.
// 
static void make_crc_table()
{
static volatile int			first = 1;							// flag to limit concurrent making 
static const unsigned char	p[] = {0,1,2,4,5,7,8,10,11,12,16,22,23,26};
unsigned long	c;
unsigned long	poly;											// polynomial exklusive-or pattern 
int				n, k;




	if(first) 
		{
		first = 0;

		
		poly = 0;												// make exklusive-or pattern from polynomial (0xedb88320UL) 

		for(n = 0; n < sizeof(p)/sizeof(unsigned char); n++)
			{
			poly |= 1UL << (31 - p[n]);
			}

		
		for(n = 0; n < 256; n++)								// generate a crc for every 8-bit value 
			{
			c = (unsigned long)n;
			for(k = 0; k < 8; k++)
				{
				c = c & 1 ? poly ^ (c >> 1) : c >> 1;
				}

			crc_table[n] = c;
			}

		crc_table_empty = 0;
		}
	else{
		while(crc_table_empty);					// wait for the other guy to finish (not efficient, but rare) 
		}

}




//*****************************************************************************
//*
//*		zcalloc
//*
//*****************************************************************************
void* zcalloc(void *opaque,unsigned items, unsigned size)
{

return malloc(items*size);
}

//*****************************************************************************
//*
//*		zcfree
//*
//*****************************************************************************
void  zcfree (void* opaque, void *ptr)
{

	free(ptr);

}



//*****************************************************************************
//*
//*		png_adler32
//*
//*****************************************************************************
unsigned png_adler32(unsigned adler,const unsigned char *buf,unsigned  len)
{
unsigned	sum2;
unsigned	n;


	
	sum2	= (adler >> 16) & 0xFFFF;							// split Adler-32 into component sums 
	adler  &=  0xFFFF;

	
	if(len == 1)												// in case user likes doing a byte at a time, keep it fast 
		{
		   adler += buf[0];
		if(adler >= BASE)
			{
			adler -= BASE;
			}

		   sum2 += adler;
		if(sum2 >= BASE)
			{
			sum2 -= BASE;
			}

		return adler | (sum2 << 16);
		}

	
	if(!buf)													// initial Adler-32 value (deferred check for len == 1 speed) 
		{
		return 1L;
		}

	
	if(len < 16)												// in case short lengths are provided, keep it somewhat fast 
		{
		while(len--) 
			{
			adler  += *buf++;
			sum2   +=  adler;
			}

		if(adler >= BASE)
			{
			adler -= BASE;
			}

		MOD4(sum2); 											// only added so many BASE's 
		
		return adler | (sum2 << 16);
		}

	
	while(len >= NMAX)											// do length NMAX blocks -- requires just one modulo operation 
		{
		len    -= NMAX;
		n		= NMAX / 16;									// NMAX is divisible by 16 
		
		do	{ 
			DO16(buf);											// 16 sums unrolled 
			buf += 16;
			}while(--n);
		
		MOD(adler);
		MOD(sum2);
		}

																// do remaining bytes (less than NMAX, still just one modulo) 
	if(len) 													// avoid modulos if none remaining 
		{
		while(len >= 16) 
			{
			len -= 16;
			DO16(buf);
			buf += 16;
			}

		while(len--) 
			{
			adler  += *buf++;
			sum2   +=  adler;
			}

		MOD(adler);
		MOD(sum2);
		}

	

return adler | (sum2 << 16);									// return recombined sums 
}


//*****************************************************************************
//*
//*		png_crc32
//*
//*****************************************************************************
unsigned long png_crc32(unsigned long crc,const unsigned char *buf,unsigned len)
{

	
	if(!buf)return 0;

	if(crc_table_empty)
		{
		make_crc_table();
		}


	crc = crc ^ 0xFFFFFFFF;

	while(len>=8) 
		{
		CDO8;
		len -= 8;
		}

	if(len) 
		{
		do	{
			CDO1;
			len--;
			}while(len);
		}


return crc ^ 0xFFFFFFFF;
}
