//*****************************************************************************
//*
//*
//*		PngDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright (c) 2007 Anton Zechner
//
//	Das ist eine Klasse zum Laden von PNG Bildern, basierend auf libpng
// 


#include	"PngDecoder.h"
#include	"Png/png.h"
#include	"Png/pngpriv.h"

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
PngDecoder::PngDecoder()
{
	iColorType = 0;
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
PngDecoder::~PngDecoder()
{
}


//*****************************************************************************
//*
//*		ReadImage 
//*
//*****************************************************************************
//	Ldd ein PNG-Bild aus einer Datei oder einem Stream
//  pName		: Ist der Dateiname des Bildes	
//	pFile		: Ist der Zeiger auf den Stream
//	sImage		: Ist die Bildklasse in die das Bild geladen wird.
void PngDecoder::ReadImage(const char *pName, BitmapImage &sImage)
{
CFileBuffer		sFile;

	
	if(!sFile.Open(pName))throw EGraphicsException("Fehler beim ffnen der Datei");

	try	{
		ReadImage(&sFile,sImage);
		}
	catch(...)
		{
		sFile.Close();
		throw;
		}

	sFile.Close();

}

void PngDecoder::ReadImage(const wchar_t *pName, BitmapImage &sImage)
{
CFileBuffer		sFile;

	
	if(!sFile.Open(pName))throw EGraphicsException("Fehler beim ffnen der Datei");

	try	{
		ReadImage(&sFile,sImage);
		}
	catch(...)
		{
		sFile.Close();
		throw;
		}

	sFile.Close();

}

void PngDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage)
{
png_uint_32		uWidth;
png_uint_32		uHeight;
png_structp		pPngPtr;
png_bytep		pRowPtr;
png_uint_32p	pOutput;
png_infop		pInfoPtr;
png_textp		pTextPtr;
int				iInterlaceType;
unsigned		uPassCount;
unsigned		uBitDepth;
unsigned		uColCmp;
unsigned		uColor;
unsigned		uCount;
unsigned		uIndex;
unsigned		uTrans;
unsigned		uPass;
unsigned		uPosY;
unsigned		uPosX;
unsigned 		uRow;
unsigned 		uRgb;
int				i;



	pStream		= pFile;
	iColorType	= 0;

		pPngPtr = png_create_read_struct(PNG_LIBPNG_VER_STRING,this,(png_error_ptr)ErrorProc,NULL);
	if(!pPngPtr)
		{
		throw EGraphicsException("Interner Fehler");		
		}

	try	{
		CallProgressFunction(0);

		
			pInfoPtr = png_create_info_struct(pPngPtr);			// Allocate/initialize the memory for image information.  
		if(!pInfoPtr)
			{
			png_destroy_read_struct(&pPngPtr,NULL,NULL);
			throw EGraphicsException("Zu wenig Speicher") ;		
			}

		
		png_set_read_fn(pPngPtr,this,(png_rw_ptr)ReadBuffer);
		png_read_info  (pPngPtr,pInfoPtr);
		png_get_IHDR   (pPngPtr,pInfoPtr,&uWidth,&uHeight,&i,&iColorType,&iInterlaceType,0,0);

		uBitDepth = pInfoPtr->pixel_depth;
		uCount    = pInfoPtr->num_palette;
		uTrans    = pInfoPtr->num_trans;

		   
		if(uTrans && (!pInfoPtr->trans_alpha || uBitDepth>8))	// Gibt es eine Aphamaske in der Palette
			{
			uTrans = 0;
			}

		for(i=pInfoPtr->num_text-1;i>=0;i--)					// Den Kommentar auslesen
			{
			pTextPtr = pInfoPtr->text+i;
			
			if(i==0 || strcmp(pTextPtr->key,"Comment"))
				{
				SetComment(pTextPtr->text);
				break;
				}
			}
																
		if(!uHeight || !uWidth)									// Ein leere Bild erzeugen	
			{
			throw EGraphicsException("Das Format wird nicht untersttzt");		
			}

		if(!sImage.SetSize(uBitDepth,uWidth,uHeight,BI_NOT_INIT))
			{
			switch(sImage.LastError())
				{
			case 1:	throw EGraphicsPicSize();
			case 2:	throw EGraphicsWrongColor();
			case 3:	throw EGraphicsMemory();
				}

			throw EGraphicsException("Unknown error");
			}

																// Alle Palleteneintrge auslesen
		if(iColorType&PNG_COLOR_MASK_PALETTE && pInfoPtr->palette)
			{
			for(uColor=0;uColor<uCount;uColor++)
				{
				sImage.SetColor(uColor,pInfoPtr->palette[uColor].red,pInfoPtr->palette[uColor].green,pInfoPtr->palette[uColor].blue);
				}
			}
		
		if(iColorType&PNG_COLOR_MASK_COLOR)						// RGB auf GBR Ausgabe
			{
			png_set_bgr(pPngPtr);
			}

		
		uPassCount = png_set_interlace_handling(pPngPtr);
		

		for(uPass=0;uPass<uPassCount;uPass++)					// Bild zeilenweise einlesen
			{
			for(uRow=0;uRow<uHeight;uRow++)
				{
				CallProgressFunction((100*uRow*uPass)/uHeight/uPassCount);

				pRowPtr = sImage[uRow];

				png_read_row(pPngPtr,pRowPtr,NULL);
				}
			}


																// Alle Palleteneintrge auslesen
		if((iColorType&PNG_COLOR_MASK_PALETTE) && pInfoPtr->palette)
			{
			unsigned char	aUsed[256];
			unsigned char  *pAlpha;	
			unsigned		uCheck;
			unsigned		uSame;

			uSame	= 0;
			uCheck	= 0;
			pAlpha	= pInfoPtr->trans_alpha;
			uColCmp = sImage.GetColor(0);

			for(uIndex=0;uIndex<uTrans;uIndex++)				// Sind die Transparenzen undurchsichtig
				{
				if(pAlpha[uIndex]==0xFF)uCheck++;
				}

			if(uCheck>=uTrans)
				{
				uTrans = 0;
				}

			if(uTrans)											// Gibt es Tranparenzangaben 
				{
				uCheck = 0;

				memset(aUsed,0,sizeof(aUsed));

				for(uPosY=uHeight;uPosY>0;)						// Welche Farb-Indizes werden benutz
					{
					uPosY  -= 1;
					pRowPtr = sImage.GetRow(uPosY);

					for(uPosX=uWidth;uPosX>0;)
						{
						uPosX--;
						uIndex = pRowPtr[uPosX];
						aUsed[uIndex] = 1;

						if(uIndex>=uTrans || pAlpha[uIndex]!=0xFF)
							{
							uCheck++;
							}
						}
					}

				if(uCheck==0)									// Werden keine transparenten Farben benutzt 
					{
					uTrans = 0;
					}
				else if(uTrans)
					{
					uColCmp |= (pAlpha[0])<<24;
					}

				for(uIndex=0;uIndex<uCount;uIndex++)			// Sind alle Farben gleich
					{
					if(aUsed[uIndex]==0){uSame++;continue;}

					uRgb = sImage.GetColor(uIndex); 

					if(uTrans)
						{
						uRgb |= (uIndex<uTrans)? pAlpha[uIndex]<<24:0xFF000000;
						}

					sImage.SetColor(uIndex,uRgb); 
					   
					if(uRgb==uColCmp)uSame++; 
					}
				
				if(uTrans && uSame<uCount)						// Wegen Transparenz auf 32 Bit gehen
					{
					BitmapImage	sPicture;

					if(!sPicture.SetSize(32,uWidth,uHeight,BI_NOT_INIT))
						{
						switch(sImage.LastError())
							{
						case 1:	throw EGraphicsPicSize();
						case 2:	throw EGraphicsWrongColor();
						case 3:	throw EGraphicsMemory();
							}

						throw EGraphicsException("Unknown error");
						}

					for(uPosY=uHeight;uPosY>0;)					// Welche Farb-Indizes werden benutz
						{
						uPosY  -= 1;
						pOutput = (png_uint_32p)sPicture.GetRow (uPosY);
						pRowPtr =			    sImage  .GetRow1(uPosY);

						for(uPosX=uWidth;uPosX>0;)
							{
							uPosX--;
							uIndex = pRowPtr[uPosX];
							pOutput[uPosX] = sImage.GetColor(uIndex);
							}
						}
											
					sImage.Swap(sPicture);
					iColorType |= 4;
					}
				}
			else{
				for(uColor=0;uColor<uCount;uColor++)
					{
					   uRgb =sImage.GetColor(uColor); 
					if(uRgb==sImage.GetColor(0))uSame++; 
					}
				}
			
			if(uSame>1 && uSame==uCount)						// Sind alle Farben gleich
				{				  
				uSame   = 0;

				for(uColor=0;uColor<pInfoPtr->num_trans;uColor++)
					{
					uTrans = 255-pInfoPtr->trans_alpha[uColor];
					sImage.SetColor(uColor,uTrans,uTrans,uTrans);
					   uRgb =sImage.GetColor(uColor); 
					if(uRgb==sImage.GetColor(0))uSame++; 
					}
																// Sind alle Transparenzen gleich
				if(pInfoPtr->num_trans>1 && uSame<pInfoPtr->num_trans)		
					{

					}
				else if(uBitDepth==8)
					{
					for(uColor=0;uColor<256;uColor++)
						{
						sImage.SetColor(uColor,uColor,uColor,uColor);
						}
					}
				else if(uBitDepth==4)
					{
					for(uColor=0;uColor<16;uColor++)
						{
						uRgb = uColor*17;
						sImage.SetColor(uColor,uRgb,uRgb,uRgb);
						}
					}
				else if(uBitDepth==1)
					{
					for(uColor=0;uColor<2;uColor++)
						{
						uRgb = uColor*255;
						sImage.SetColor(uColor,uRgb,uRgb,uRgb);
						}
					}
				}
			}	


  
		CallProgressFunction(100);

		png_destroy_info_struct( pPngPtr,&pInfoPtr);
		png_destroy_read_struct(&pPngPtr,NULL,NULL);
		}
	catch (...)
		{
		png_destroy_info_struct( pPngPtr,&pInfoPtr);
		png_destroy_read_struct(&pPngPtr,NULL,NULL);

		throw;
		}


}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void PngDecoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;
	pProgressProc(*this, pProgressData, 1, 1, uPercent, bCancel) ;

	if(bCancel)throw EGraphicsAbort();
}




//*****************************************************************************
//*
//*		ReadBuffer
//*
//*****************************************************************************
void _cdecl PngDecoder::ReadBuffer(void *pPtr,unsigned char *pBuffer,unsigned uLength)
{
PngDecoder	   *pThis;
png_struct	   *pPngPtr;
unsigned 		uCount;


	pPngPtr = (png_struct*) pPtr;
	pThis   = (PngDecoder*)(pPngPtr->io_ptr);
	
   	
	   uCount = pThis->pStream->Read(pBuffer,uLength);
	if(uCount!=uLength)throw EGraphicsException("Fehler beim Lesen");		

}


//*****************************************************************************
//*
//*		ErrorProc
//*
//*****************************************************************************
void _cdecl PngDecoder::ErrorProc(void *pPtr,const char *pErrorText)
{
png_struct	   *pPngPtr=(png_struct*)pPtr;

	throw EGraphicsException(pErrorText);		
}