//*****************************************************************************
//*
//*
//*		PngEncoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright (c) 2007 Anton Zechner
//
//	Das ist eine Klasse zum Speichern von PNG Bildern, basierend auf libpng
// 

#include	"PngEncoder.h"
#include	"Png/png.h"
#include	"Png/pngpriv.h"

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
PngEncoder::PngEncoder()
{

	iDefaultMode = 0;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
PngEncoder::~PngEncoder()
{


}

//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
//	Speichhert ein PNG-Bild aus einer Datei oder einem Stream
//  pName		: Ist der Dateiname des Bildes	
//	pFile		: Ist der Zeiger auf den Stream
//	sImage		: Ist die Bildklasse in die das Bild geladen wird.
//	iMode		: Diverse Modus-Bits
//					Bit0 = Alpha-Kanal speichern (0xFF=sichtbar)
void PngEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{

	WriteImage(pName,sImage,iDefaultMode);

}

void PngEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{

	WriteImage(pName,sImage,iDefaultMode);

}

void PngEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{

	WriteImage(pFile,sImage,iDefaultMode);

}

void PngEncoder::WriteImage(const char *pName,const BitmapImage &sImage,int iMode)
{
CFileBuffer	sFile;


	if(!sFile.Open(pName,CFB_CREATE))throw EGraphicsException("Fehler beim Erzeugen der Datei") ;

	try	{
		WriteImage(&sFile,sImage,iMode);
		}
	catch(...)
		{
		sFile.Close();
		throw;
		}

	sFile.Close();

}

void PngEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage,int iMode)
{
CFileBuffer	sFile;


	if(!sFile.Open(pName,CFB_CREATE))throw EGraphicsException("Fehler beim Erzeugen der Datei") ;

	try	{
		WriteImage(&sFile,sImage,iMode);
		}
	catch(...)
		{
		sFile.Close();
		throw;
		}

	sFile.Close();

}

void PngEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage,int iMode)
{
png_structp		pPngPtr;
png_infop		pInfoPtr;
png_colorp		pPalette;
png_bytep		pRowPtr;
png_text		sText;
unsigned		uColor;
unsigned		uRow;
unsigned		uMax;



	pStream = pFile;
	
	CallProgressFunction(0);

	if(!sImage.IsOk())
		{
		throw EGraphicsException("Ungltiges Bild");		
		}

		pPngPtr = png_create_write_struct(PNG_LIBPNG_VER_STRING,this,(png_error_ptr)ErrorProc,NULL);
	if(!pPngPtr)
		{
		throw EGraphicsException("Interner Fehler");		
		}


	try	{
			pInfoPtr = png_create_info_struct(pPngPtr);
		if(!pInfoPtr)
			{
			throw EGraphicsException("Interner Fehler");		
			}		


		png_set_write_fn(pPngPtr,this,(png_rw_ptr)WriteBuffer,NULL);

		
		if(GetComment())									// Kommentar speichern
			{
			sText.key  = "Comment";
			sText.text = GetComment();
			sText.compression = PNG_TEXT_COMPRESSION_NONE;

			png_set_text(pPngPtr,pInfoPtr,&sText,1);
			}
		

		if(sImage.BitCount()<=8)
			{
			png_set_IHDR(pPngPtr,pInfoPtr,sImage.Width(),sImage.Height(),sImage.BitCount(),PNG_COLOR_TYPE_PALETTE,PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,PNG_FILTER_TYPE_DEFAULT);
			
			uMax	 = 1<<sImage.BitCount();
			pPalette = (png_colorp)png_malloc(pPngPtr,uMax*sizeof(png_color));

			for(uColor=0;uColor<uMax;uColor++)
				{
				sImage.GetColor(uColor,pPalette[uColor].red,pPalette[uColor].green,pPalette[uColor].blue);
				}

			png_set_PLTE  (pPngPtr,pInfoPtr,pPalette,uMax);
			png_write_info(pPngPtr,pInfoPtr);

			for(uRow=0;uRow<sImage.Height();uRow++)
				{
				CallProgressFunction((100*uRow)/sImage.Height());
				pRowPtr = sImage[uRow];				
				png_write_rows(pPngPtr, &pRowPtr, 1);
				}
			}
		else if(sImage.BitCount()==32 && (iMode&0x01))
			{
			png_set_IHDR  (pPngPtr,pInfoPtr,sImage.Width(),sImage.Height(),8,PNG_COLOR_TYPE_RGB_ALPHA,PNG_INTERLACE_NONE,PNG_COMPRESSION_TYPE_BASE,PNG_FILTER_TYPE_DEFAULT);
			png_write_info(pPngPtr,pInfoPtr);
			png_set_bgr   (pPngPtr);
			
			for(uRow=0;uRow<sImage.Height();uRow++)
				{
				CallProgressFunction((100*uRow)/sImage.Height());
				pRowPtr = sImage.GetRow4(uRow);				
				png_write_row(pPngPtr,pRowPtr);
				}
			}
		else{
			png_set_IHDR  (pPngPtr,pInfoPtr,sImage.Width(),sImage.Height(),8,PNG_COLOR_TYPE_RGB,PNG_INTERLACE_NONE,PNG_COMPRESSION_TYPE_BASE,PNG_FILTER_TYPE_DEFAULT);
			png_write_info(pPngPtr,pInfoPtr);
			png_set_bgr   (pPngPtr);
			
			for(uRow=0;uRow<sImage.Height();uRow++)
				{
				CallProgressFunction((100*uRow)/sImage.Height());
				pRowPtr = sImage.GetRow3(uRow);				
				png_write_rows(pPngPtr, &pRowPtr, 1);
				}
			}
		
		png_write_end           ( pPngPtr, pInfoPtr);		
		png_destroy_info_struct ( pPngPtr,&pInfoPtr);
		png_destroy_write_struct(&pPngPtr,NULL);
		CallProgressFunction(100);
		}
	catch(...)
		{
		png_destroy_info_struct ( pPngPtr,&pInfoPtr);
		png_destroy_write_struct(&pPngPtr,NULL);
		throw;
		}

	
  
}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void PngEncoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel) ;

	if(bCancel)throw EGraphicsAbort();
}


//*****************************************************************************
//*
//*		WriteBuffer
//*
//*****************************************************************************
void _cdecl PngEncoder::WriteBuffer(void *pPtr,unsigned char *pBuffer,unsigned uLength)
{
PngEncoder	   *pThis;
png_struct	   *pPngPtr;
unsigned 		uCount;

	
	pPngPtr =(png_struct*) pPtr;
	pThis   =(PngEncoder*)(pPngPtr->io_ptr);
	
   	
	   uCount = pThis->pStream->Write(pBuffer,uLength);
	if(uCount!=uLength)throw EGraphicsException("Fehler beim Schreiben");		

}

//*****************************************************************************
//*
//*		ErrorProc
//*
//*****************************************************************************
void _cdecl PngEncoder::ErrorProc(void *pPtr,const char *pErrorText)
{
png_struct	   *pPngPtr=(png_struct*)pPtr;

	throw EGraphicsException(pErrorText);		
}

