//*****************************************************************************
//*
//*
//*		PnmDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2011
//
//	Ld PNM-Bilder.
//

#include "PnmDecoder.h"


//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
PnmDecoder::PnmDecoder()
{
}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
PnmDecoder::~PnmDecoder()
{
}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void PnmDecoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel);

	if(bCancel)throw EGraphicsAbort();
}

//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
void PnmDecoder::ReadImage(const char *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw EPnmFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void PnmDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw EPnmFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void PnmDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage)
{
char			cBuffer[4096];
unsigned		uHeight;
unsigned		uWidth; 
unsigned		uColor;
unsigned		uMode;
unsigned		uLine;
unsigned		uLen;
unsigned		uRem;
unsigned		uNum;
unsigned		uPos;
unsigned		uCol;
unsigned		uOff;
unsigned		uRow;
unsigned		uIdx;	
unsigned char  *pRow;	


	

	if(pFile->Read(cBuffer,2)!=2)							// Read ID
		{
		throw EPnmFileReadError();	
		}	


	if(cBuffer[0]!='P' || cBuffer[1]<'1' || cBuffer[1]>'6')	// Check ID
		{
		throw EPnmNotAPnmFile();
		}


	uMode  = cBuffer[1]-'1';
	uColor = 0;
	uNum   = 0;
	uLen   = 0;
	uPos   = 0;
	uRem   = 0;
	uCol   = 0;
	uRow   = 0;
	uIdx   = 0;
	uOff   = 0;


	for(;;uPos++)
		{
		if(uLen-uPos<256)									// Puffer auffllen		
			{
			memmove(cBuffer,cBuffer+uPos,uLen-uPos);
			uOff += uPos;
			uLen -= uPos;
			uPos  = 0;
			uLen += pFile->Read(cBuffer+uLen,sizeof(cBuffer)-uLen);

			if(uPos>=uLen)
				{
				throw EPnmCorruptFile();
				}
			}

		if(cBuffer[uPos]=='#')								// Kommentar gefunden
			{
			uRem = 1;
			continue;
			}

		if(cBuffer[uPos]=='\n' || cBuffer[uPos]=='\r')
			{
			uRem = 0;
			continue;
			}

		if(cBuffer[uPos]<'0' || cBuffer[uPos]>'9' || uRem)
			{
			continue;
			}

		if(uNum==0)uWidth =atoi(cBuffer+uPos);				// Zahl gefunden
		if(uNum==1)uHeight=atoi(cBuffer+uPos);
		
		for(;;)
			{												// Zahl berspringen
			uPos++;

			if(uPos>=uLen)
				{
				throw EPnmCorruptFile();
				}

			if(cBuffer[uPos]<'0')break;
			if(cBuffer[uPos]>'9')break;
			}

		uNum++;
		if(uNum>=3 || (uNum==2 && (uMode==0 || uMode==3)))
			{
			break;
			}
		}
	
	
	

//******************** ASCII Laden ********************************************

	
	if(uMode<3)			
		{
		CallProgressFunction(0);
									
		switch(uMode)										// Bild erzeugen
			{
		case 0:	sImage.SetSize(1 ,uWidth,uHeight,BI_NOT_INIT);						break;
		case 1:	sImage.SetSize(8 ,uWidth,uHeight,BI_NOT_INIT);		
				for(uNum=0;uNum<256;uNum++)sImage.SetColor(uNum,uNum,uNum,uNum);	break;
		case 2:	sImage.SetSize(24,uWidth,uHeight,BI_NOT_INIT);						break;
			}

		
		for(;;uPos++)
			{
			if(uLen-uPos<16)								// Puffer auffllen		
				{
				memmove(cBuffer,cBuffer+uPos,uLen-uPos);
				uLen -= uPos;
				uPos  = 0;
				uLen += pFile->Read(cBuffer+uLen,sizeof(cBuffer)-uLen);

				if(uPos>=uLen)
					{
					throw EPnmCorruptFile();
					}
				}

			if(cBuffer[uPos]=='#')							// Kommentar gefunden
				{
				uRem = 1;
				continue;
				}

			if(cBuffer[uPos]=='\n' || cBuffer[uPos]=='\r')	// Zeilenende
				{
				uRem = 0;
				continue;
				}
																		
			if(cBuffer[uPos]<'0' || cBuffer[uPos]>'9' || uRem)
				{
				continue;
				}

			uNum = atoi(cBuffer+uPos);


			if(uMode==2)									// RGB-Bild
				{
				uColor <<= 8;
				uColor  |= uNum;
				uIdx++;
				
				if(uIdx>=3)
					{
					sImage.SetPixel(uCol,uRow,uColor);
					uCol++;

					if(uCol>=uWidth)
						{
						uCol=0;
						uRow++;
						CallProgressFunction(uRow*100/uHeight);

						if(uRow>=uHeight)
							{
							CallProgressFunction(100);
							return; 
							}
						}
					
					uIdx   = 0;
					uColor = 0;
					}
				}
			else{											// Index-Bild
				sImage.SetPixel(uCol,uRow,uNum);
				uCol++;

				if(uCol>=uWidth)
					{
					uCol=0;
					uRow++;
					CallProgressFunction(uRow*100/uHeight);

					if(uRow>=uHeight)
						{
						CallProgressFunction(100);
						return;
						}
					}
				}
			
			for(;;)											// Zahl berspringen
				{
				uPos++;

				if(uPos>=uLen)
					{
					throw EPnmCorruptFile();
					}

				if(cBuffer[uPos]<'0')break;
				if(cBuffer[uPos]>'9')break;
				}
			}
		}

 
//******************** Bild Laden *********************************************


	for(;;)											// Zahl berspringen
		{
		uPos++;

		if(uPos>=uLen)
			{
			throw EPnmCorruptFile();
			}

		if(cBuffer[uPos]<'0')break;
		if(cBuffer[uPos]>'9')break;
		}
	
	pFile->Seek(uOff+uPos+2);

	uMode -= 3;

	switch(uMode)											// Bild erzeugen
		{
	case 0:	sImage.SetSize(1 ,uWidth,uHeight);	uLine=uWidth/8;				break;
	case 1:	sImage.SetSize(8 ,uWidth,uHeight);	uLine=uWidth;		
			for(uNum=0;uNum<256;uNum++)sImage.SetColor(uNum,uNum,uNum,uNum);break;
	case 2:	sImage.SetSize(24,uWidth,uHeight);	uLine=uWidth*3;				break;
		}

	
	CallProgressFunction(0);


	for(uRow=0;uRow<uHeight;uRow++)							// Alle Bildzeilen laden
		{
		CallProgressFunction(uRow*100/uHeight);
		pRow = sImage[uRow];
		if(pFile->Read(pRow,uLine)!=(int)uLine)break;

		if(uMode!=2)continue;
		
		for(uPos=0;uPos<uLine;uPos+=3)
			{
			uNum         = pRow[uPos  ];
			pRow[uPos  ] = pRow[uPos+2];
			pRow[uPos+2] = uNum;
			}
		}

	if(uRow<uHeight)										// Gab es einen Fehler
		{
		throw EPnmCorruptFile();
		}


	CallProgressFunction(100);

}

