//*****************************************************************************
//*
//*
//*		PnmEncoder.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"PnmEncoder.h"
#include	"..\FileBuffer.h"




//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
PnmEncoder::PnmEncoder()
{

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
PnmEncoder::~PnmEncoder()
{
}

//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
//	Speichert das Bild ab.
//	pName		: Dateiname der Bilddatei
//  sImage		: Klasse in der das Bild gespeichert wird
void PnmEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EPnmFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void PnmEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EPnmFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void PnmEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
char				cHeader[64];
unsigned			uSize;
unsigned			uPos;
unsigned			uNum;
unsigned			uOff;
unsigned char	   *pPtr;



	if(!sImage.IsOk())
		{
		throw EPnmDataError();
		}


	if(sImage.BitCount()==1)
		{
		uSize = sprintf(cHeader,"P4 %i %i ",sImage.Width(),sImage.Height());

		if(pFile->Write(cHeader,uSize)!=(int)uSize)
			{
			throw EPnmFileWriteError();
			}
		
		uSize = (sImage.Width()+7)/8;
		
		for(uPos=0;uPos<sImage.Height();uPos++)
			{
			CallProgressFunction((100*uPos)/sImage.Height());

			pPtr = sImage[uPos];

			if(pFile->Write(pPtr,uSize)!=(int)uSize)
				{
				throw EPnmFileWriteError();
				}
			}
		}
	else{
		uSize = sprintf(cHeader,"P6 %i %i 255 ",sImage.Width(),sImage.Height());
		
		if(pFile->Write(cHeader,uSize)!=(int)uSize)
			{
			throw EPnmFileWriteError();
			}
		
		uSize = sImage.Width()*3;
		
		for(uPos=0;uPos<sImage.Height();uPos++)
			{
			CallProgressFunction((100*uPos)/sImage.Height());

			pPtr = sImage.GetRow3Temp(uPos);

			for(uOff=0;uOff<uSize;uOff+=3)
				{
				uNum         = pPtr[uOff  ];
				pPtr[uOff  ] = pPtr[uOff+2];
				pPtr[uOff+2] = uNum;
				}

			if(pFile->Write(pPtr,uSize)!=(int)uSize)
				{
				throw EPnmFileWriteError();
				}
			}
		}


	CallProgressFunction(100);

}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void PnmEncoder::CallProgressFunction (unsigned int percent)
{
bool	bCancel;


	if(pProgressProc == NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,percent,bCancel);

	if(bCancel)throw EGraphicsAbort();

}
