//*****************************************************************************
//*
//*
//*		TiffDecoder.h
//*
//*
//*****************************************************************************
//
//	Copyright (c) 2002 AZ Software
//	All rights reserved.
//	Author:  Anton Zechner
//

#ifndef __TIFFDECODER_H__
#define __TIFFDECODER_H__

#include "Tiff.h"
#include "BitmapImage.h"
#include "..\FileBuffer.h"



//*****************************************************************************
//*
//*		Klasse TiffDecoder
//*
//*****************************************************************************
class TiffDecoder : public BitmapImageDecoder
{
public:

		         TiffDecoder();
	virtual     ~TiffDecoder();
	virtual void ReadImage (const wchar_t *pName,BitmapImage &sImage);
	virtual void ReadImage (const char    *pName,BitmapImage &sImage);
	virtual void ReadImage (CFileBuffer   *pFile,BitmapImage &sImage);
	void		 GetPixelPerMeter(int &iMeterX,int &iMeterY);

private:

	void		Initialize () ;
	void		CallProgressFunction (unsigned uPercent);
	void		ReadRGB();
	void		ReadPal();
	void		ReadMono();

	int			DecompressHuffman (unsigned char *pBuffer,int iMax);
	int			DecompressPackBits(unsigned char *pBuffer,int iMax);
	int			DecompressLZW     (unsigned char *pBuffer,int iMax);
	void		DecompressLZWReset();

	inline	unsigned int   BtoL(unsigned int   iVal);
	inline	unsigned short BtoL(unsigned short sVal);
	inline			 int   BtoL(		 int   iVal);
	inline			 short BtoL(		 short sVal);

	CFileBuffer  *m_pInput;
	BitmapImage	 *m_pImage;

	int			m_iBigEndian;					// Byteorder wechseln
	int			m_iBaseSeek;					// Anfangs Offset in der Datei

	int			m_iWidth;						// Bildauflsung
	int			m_iHeight;						//		*
	int			m_iCompressType;				// Compresions Type
	int			m_iImageType;					// Bild Typ
	int			m_iMeterX;						// Auflsung in Pixel je Meter
	int			m_iMeterY;						//		*
	int			m_iBitPerSample;				// Bits je Pixel
	int			m_iSamplesPerPixel;				// Sample je Pixel
	int			m_iFillOrder;					// Bitleserichtung	0:7zu0 1:0zu7
	int			m_iDifference;					// LZW Differnz endcodieren

	int			m_iRowsPerStrip;				// Reihen je Speicher Block
	int 	   *m_pRowOffsets;					// Offsets der Speicher Blcke
	int			m_iRowOffsetPos;				// Offset  der Speicher Blcke in Datei
	int			m_iRowOffsetCnt;				// Anzahl  der Speicher Blcke
	int 	   *m_pRowByteCnt;					// Lngen  der Speicher Blcke
	int		    m_iRowBytePos;					// Offset  der Speicher Blocklngen in Datei
	int		    m_iRowByteCnt;					// Anzahl  der Speicher Blcke


	//**************** LZW-Daten **********************************************
	
	int			m_iOldCode;
	int			m_iEndCode;
	int			m_iCodeSize;
	int			m_iCodeMask;
	int			m_iLzwDate;
	int			m_iLzwBits;
	int			m_iLzwRest;
	
	unsigned short *m_pPrefix;
	unsigned char  *m_pCharF;
	unsigned char  *m_pCharC;



};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline unsigned int TiffDecoder::BtoL(unsigned uVal)
{
union
	{
	unsigned char b[4];
	unsigned int  l;
	}u;

	u.l= uVal;

return (u.b[0]<<24)|(u.b[1]<<16)|(u.b[2]<<8)|(u.b[3]);
}

inline int TiffDecoder::BtoL(int iVal)
{
union
	{
	unsigned char b[4];
	unsigned int  l;
	}u;

	u.l = iVal;

return (u.b[0]<<24)|(u.b[1]<<16)|(u.b[2]<<8)|(u.b[3]);
}

inline unsigned short TiffDecoder::BtoL(unsigned short sVal)
{
union
	{
	unsigned char  b[2];
	unsigned short s;
	}u;

	u.s = sVal;

return (u.b[0]<<8)|(u.b[1]);
}

inline short TiffDecoder::BtoL(short sVal)
{
union
	{
	unsigned char  b[2];
	unsigned short s;
	}u;

	u.s = sVal;

return (u.b[0]<<8)|(u.b[1]);
}

#endif

