//*****************************************************************************
//*
//*
//*		TiffEncoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright (c) 2002 AZ Software
//	All rights reserved.
//	Author:  Anton Zechner
//
#include "TiffEncoder.h"

typedef	struct
	{
	unsigned short	usSign;
	unsigned short	usNumber;
	unsigned int	uiOffset;
	}TiffHeader;

typedef	struct
	{
	unsigned short	usTag;
	unsigned short	usType;
	unsigned int	uiCount;
	union	{
			unsigned int	uiOffset;
			unsigned int 	uiValue;
			unsigned short 	usValue;
			};
	}TiffEntry;


static char *pVersionStr="AZ Grafik Lib";




#define RES_CODES	2
					
#define HASH_FREE	0xFFFF
#define NEXT_FIRST	0xFFFF

#define MAXBITS		12
#define MAXSTR		(1 << MAXBITS)

#define HASHSIZE	9973
#define HASHSTEP	2039
#define HASH(i,l)	(((l<<8)^i)%HASHSIZE)

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
TiffEncoder::TiffEncoder ()
{
	m_pRowOffsets	= 0;
	m_pRowByteCnt	= 0;
	m_pOutput		= 0;
	m_pStingChar	= 0;
	m_pStingNext	= 0;
	m_pStingHash	= 0;

	Initialize();

}


//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
TiffEncoder::~TiffEncoder ()
{
	FreeBuffers();
}


//*****************************************************************************
//*
//*		FreeBuffers
//*
//*****************************************************************************
void TiffEncoder::FreeBuffers()
{

	if(m_pRowOffsets ){delete m_pRowOffsets;m_pRowOffsets=0;}
	if(m_pRowByteCnt ){delete m_pRowByteCnt;m_pRowByteCnt=0;}
	if(m_pStingChar  ){delete m_pStingChar ;m_pStingChar =0;}
	if(m_pStingNext  ){delete m_pStingNext ;m_pStingNext =0;}
	if(m_pStingNext  ){delete m_pStingNext ;m_pStingNext =0;}

}

//*****************************************************************************
//*
//*		Initialize
//*
//*****************************************************************************
void TiffEncoder::Initialize ()
{

	m_iMeterX			= 2835;							// 72 DPI
	m_iMeterY			= 2835;							// 72 DPI
	m_iWidth			= 0;							// Bildauflsung
	m_iHeight			= 0;							//		*
	m_iCompressType		= 1;							// Compresions iType
	m_iRowsPerStrip		= 0;							// Reihen je Speicher Block
	m_iBitPerSample		= 1;							// Bits   je Pixel
	m_iSamplesPerPixel	= 1;							// Sample je Pixel
	m_iRowOffsetPos		= 0;							// Offset der Speicher Blcke in Datei
	m_iRowBytePos		= 0;							// Offset der Speicher Blocklngen in Datei
	m_iSave32Bit		= 0;							// RGB 24 als 32 Bit speichern
	m_iDifferenz		= 1;							// Differenz Codiert bei LZW
	m_iImageType		= 1;							// Bild Typ

}


//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
//
//  Description:
//
//    This function calls the progress function if it has been
//    defined.
//
//  Parameters:
//    uPercent:  The completion percentage (0..100)
//
void TiffEncoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;
	
	if(pProgressProc==0)return;
	if(uPercent>100)uPercent=100;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel);

	if(bCancel)throw EGraphicsAbort();

}

//*****************************************************************************
//*
//*		SetPixelPerMeter
//*
//*****************************************************************************
void TiffEncoder::SetPixelPerMeter(int iMeterX,int iMeterY)
{
	m_iMeterX = iMeterX;					// Pixel per Meter
	m_iMeterY = iMeterY;					//		*
}


//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
void TiffEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{
CFileBuffer	sFileBuf;



	if(!sFileBuf.Open(pName,CFB_CREATE))throw ETiffError("Fehler beim Erzeugen der Datei");

	try	{
		WriteImage(&sFileBuf,sImage);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();


}

void TiffEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{
CFileBuffer	sFileBuf;



	if(!sFileBuf.Open(pName,CFB_CREATE))throw ETiffError("Fehler beim Erzeugen der Datei");

	try	{
		WriteImage(&sFileBuf,sImage);
		}
	catch(...)
		{
		sFileBuf.Close();
		throw;
		}

	sFileBuf.Close();


}

void TiffEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
TiffHeader		sHeader;
TiffEntry		sEntry;
unsigned short *pPtr;
unsigned char   ucRgb  [3];
int				aRes   [4];
char	   		cTest  [2]={1,0};
short			aBitLen[3]={8,8,8};
short			sCount;
int				iCommentPos=0;
int				iCommentCnt=0;
int				iSoftwarePos=0;
int				iSoftwareCnt=0;
int				iColorPos=0;
int				iBitPpPos=0;
int				iBitsPos=0;
int				iBitsCnt=0;
int				iResPosX=0;
int				iResPosY=0;
int				iResMode=0;
int				iEntryPos;
int				iNum;



	m_pOutput	= pFile;
	m_pImage	= &sImage;

	m_iColorCnt	= 0;
	m_iWidth	= m_pImage->Width();
	m_iHeight	= m_pImage->Height();

	if(m_iWidth<=0 || m_iHeight<=0)
		{
		throw ETiffError("Leeres Bild bergeben");
		}

	m_iBaseSeek=m_pOutput->Tell();

	sHeader.usSign   = (*((short*)cTest)==1)? 0x4949:0x4D4D;
	sHeader.usNumber = 42;
	sHeader.uiOffset = 0;

	WriteData(&sHeader,8);

	switch(m_pImage->BitCount())
			{
	case 32:
	case 24:
	case 16:
	case 15:
	case 12:WriteRGB();
			iBitPpPos = m_pOutput->Tell()-m_iBaseSeek;
			WriteData(aBitLen,6);
			break;
	case  8:
	case  4:m_iColorCnt=m_pImage->ColorCount();
			WritePal();			break;
	case  1:WriteMono();		break;
	default:throw ETiffError("Modus ist nicht untersttzt");
			}

//******************** Farben Speichern ****************************************

	if(m_iColorCnt>0)
		{
		iColorPos	= m_pOutput->Tell()-m_iBaseSeek;
		pPtr		= new unsigned short[m_iColorCnt*3];
		
		memset(pPtr,0,m_iColorCnt*6);

		for(iNum=0;iNum<m_iColorCnt;iNum++)
			{
			m_pImage->GetColor(iNum,ucRgb[0],ucRgb[1],ucRgb[2]);
			
			pPtr[iNum              ]=ucRgb[0];
			pPtr[iNum+m_iColorCnt  ]=ucRgb[1];
			pPtr[iNum+m_iColorCnt*2]=ucRgb[2];
			}

		WriteData(pPtr,m_iColorCnt*6);

		delete pPtr;
		}

//******************** Speicher Blcke speichern *******************************

	m_iRowOffsetPos = m_pOutput->Tell()-m_iBaseSeek;
	WriteData(m_pRowOffsets,m_iRowOffsetCnt*4);

	m_iRowBytePos   = m_pOutput->Tell()-m_iBaseSeek;
	WriteData(m_pRowByteCnt,m_iRowOffsetCnt*4);

//******************** Software Speichern *************************************

	if(pVersionStr)
		{
		iSoftwareCnt = (int)strlen(pVersionStr)+1;
		iSoftwarePos = m_pOutput->Tell()-m_iBaseSeek;
		WriteData(pVersionStr,iSoftwareCnt);
		}

//******************** Kommentar Speichern ************************************

	if(pComment && *pComment)
		{
		iCommentCnt = (int)strlen(pComment)+1;
		iCommentPos = m_pOutput->Tell()-m_iBaseSeek;
		WriteData(pComment,iCommentCnt);
		}

//******************** Auflsung Speichern ************************************

	if(m_iMeterX>0 && m_iMeterY>0)
		{
		aRes[0]	 = m_iMeterX;
		aRes[2]	 = m_iMeterY;
		aRes[1]	 = 100;
		aRes[3]	 = 100;
		
		iResPosX = m_pOutput->Tell()-m_iBaseSeek;
		iResPosY = iResPosX+8;
	
		WriteData(aRes,16);
		}

//******************** Entrys speichern ***************************************

	sCount = 12;

	if(m_iDifferenz!=1)sCount++;
	if(iSoftwarePos   )sCount++;
	if(iCommentPos    )sCount++;
	if(iResPosX       )sCount++;
	if(iResPosY       )sCount++;
	if(iColorPos      )sCount++;

	iEntryPos = m_pOutput->Tell()-m_iBaseSeek;

	WriteData(&sCount,2);


	sEntry.usTag    = 254;								// Neues Sub-File
	sEntry.usType   = 4;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = 0;
	WriteData(&sEntry,12);

	sEntry.usTag    = 256;								// Bildbreite
	sEntry.usType   = 4;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = m_pImage->Width();
	WriteData(&sEntry,12);

	sEntry.usTag    = 257;								// Bildhhe
	sEntry.usType   = 4;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = m_pImage->Height();
	WriteData(&sEntry,12);

	sEntry.usTag    = 258;								// Bit je Pixel
	sEntry.usType   = 3;
	sEntry.uiValue  = 0;
	sEntry.uiCount  = (iBitPpPos)? 3:1;
	if(iBitPpPos)sEntry.uiOffset = iBitPpPos;
	else		 sEntry.usValue  = m_iBitPerSample;
	WriteData(&sEntry,12);

	sEntry.usTag    = 259;								// Compresions Typ
	sEntry.usType   = 3;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = 0;
	sEntry.usValue  = m_iCompressType;
	WriteData(&sEntry,12);

	sEntry.usTag    = 262;								// Bild Typ
	sEntry.usType   = 3;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = 0;
	sEntry.usValue  = m_iImageType;
	WriteData(&sEntry,12);

	sEntry.usTag    = 266;								// Fill Order
	sEntry.usType   = 3;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = 0;
	sEntry.usValue  = 1;
	WriteData(&sEntry,12);

	sEntry.usTag    = 273;								// Block Offsets
	sEntry.usType   = 4;
	sEntry.uiCount  = m_iRowOffsetCnt;
	sEntry.uiOffset = m_iRowOffsetPos;
	WriteData(&sEntry,12);

	sEntry.usTag    = 279;								// Block Lngen
	sEntry.usType   = 4;
	sEntry.uiCount  = m_iRowOffsetCnt;
	sEntry.uiOffset = m_iRowBytePos;
	WriteData(&sEntry,12);

	sEntry.usTag    = 278;								// Zeilen je Block
	sEntry.usType   = 4;
	sEntry.uiCount  = 1;
	sEntry.uiOffset = m_iRowsPerStrip;
	WriteData(&sEntry,12);

	sEntry.usTag    = 277;								// Samples je Pixel
	sEntry.usType   = 3;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = 0;
	sEntry.usValue  = m_iSamplesPerPixel;
	WriteData(&sEntry,12);

	sEntry.usTag    = 296;								// Auflsung in P/cm
	sEntry.usType   = 3;
	sEntry.uiCount  = 1;
	sEntry.uiValue  = 0;
	sEntry.usValue  = 3;
	WriteData(&sEntry,12);

	if(iResPosX)
		{
		sEntry.usTag    = 282;							// Auflsung in X
		sEntry.usType   = 5;
		sEntry.uiCount  = 1;
		sEntry.uiOffset = iResPosX;
		WriteData(&sEntry,12);
		}

	if(iResPosY)
		{
		sEntry.usTag    = 283;							// Auflsung in Y
		sEntry.usType   = 5;
		sEntry.uiCount  = 1;
		sEntry.uiOffset = iResPosY;
		WriteData(&sEntry,12);
		}

	if(iColorPos)
		{
		sEntry.usTag    = 320;							// Farbenpallete
		sEntry.usType   = 3;
		sEntry.uiCount  = m_iColorCnt*3;
		sEntry.uiOffset = iColorPos;
		WriteData(&sEntry,12);
		}

	if(iSoftwarePos)sCount++;
		{
		sEntry.usTag    = 305;							// Software String
		sEntry.usType   = 2;
		sEntry.uiCount  = iSoftwareCnt;
		sEntry.uiOffset = iSoftwarePos;
		WriteData(&sEntry,12);
		}

	if(iCommentPos)
		{
		sEntry.usTag    = 270;							// Komentar String
		sEntry.usType   = 2;
		sEntry.uiCount  = iCommentCnt;
		sEntry.uiOffset = iCommentPos;
		WriteData(&sEntry,12);
		}

	if(m_iDifferenz!=1)
		{
		sEntry.usTag    = 317;							// Komentar String
		sEntry.usType   = 3;
		sEntry.uiCount  = 1;
		sEntry.uiValue  = 0;
		sEntry.usValue  = m_iDifferenz;
		WriteData(&sEntry,12);
		}

	sEntry.uiOffset = 0;
	WriteData(&sEntry.uiOffset,4);
	m_pOutput->Seek(m_iBaseSeek+4);						// Speichere Entryoffset
	WriteData(&iEntryPos,4);

	if(!m_pOutput->Flush())throw ETiffError("Fehler beim Schreiben");

	FreeBuffers();
}


//*****************************************************************************
//*
//*		CompressPackBits
//*
//*****************************************************************************
//	Ergibt die Anzahl an geschriebene Bytes bzw. -1 bei einem Fehler
int TiffEncoder::CompressPackBits(unsigned char *pBuffer,int iMax)
{
unsigned char	ucCount;
unsigned char	ucLast;
int				iBytes;
int				iPos;
int				iLen;


	iPos	= 0;
	iBytes	= 0;


	while(iPos<iMax)
		{
		ucLast=pBuffer[iPos];

		for(iLen=iPos;iLen+1<iMax;iLen++)				// Suche Doppel Zeichen
			{
			if(ucLast==pBuffer[iLen+1])break;
			   ucLast =pBuffer[iLen+1];
			}

		if(iLen>iPos)									// Speichere ungleiche Zeichen
			{
			iLen    = iLen-iPos;
			ucCount = iLen-1;
			if(m_pOutput->Write(&ucCount,1       )!=   1)return -1;
			if(m_pOutput->Write(pBuffer+iPos,iLen)!=iLen)return -1;
			iPos   += iLen;
			iBytes += iLen+1;
			continue;
			}

		for(iLen=iPos+1;iLen<iMax;iLen++)				// Zhle Doppel Zeichen
			{
			if(iLen>=iPos+128)break;
			if(ucLast!=pBuffer[iLen])break;
			}

		iLen    =  iLen-iPos;
		ucCount = -iLen+1;
		
		if(m_pOutput->Write(&ucCount,1)!=1)return -1;
		if(m_pOutput->Write(&ucLast ,1)!=1)return -1;
		
		iBytes += 2;
		iPos   += iLen;
		}


return iBytes;
}

//*****************************************************************************
//*
//*		CompressLZWReset
//*
//*****************************************************************************
//	Startet eine neue Komprimierung
void TiffEncoder::CompressLZWReset()
{

    if(!m_pStingHash)m_pStingHash = new unsigned short [HASHSIZE];
	if(!m_pStingNext)m_pStingNext = new unsigned int   [MAXSTR];
	if(!m_pStingChar)m_pStingChar = new unsigned char  [MAXSTR];

	m_iLwzPrefix	=  0xFFFF;
    m_iLwzCount		=  0;
	m_iLwzDate		=  0;
	m_iLwzBits		=  0;
	m_iCodeSize		=  9;
    m_iEndCode		= (1<<9)-1;

	CompressLZWInit();
	WriteBits(256);
}


//*****************************************************************************
//*
//*		CompressLZWFlush
//*
//*****************************************************************************
void TiffEncoder::CompressLZWFlush()
{
char cByte;


    if(m_iLwzPrefix != 0xFFFF)							// Letzten Code ausgeben
        {
		WriteBits(m_iLwzPrefix);
		m_iLwzPrefix = 0xFFFF;
		}

	WriteBits(257);										// Exit Code speichern
    WriteBits(256);m_iCodeSize=9;						// Clear Code speichern

	if(m_iLwzBits>0)									// Restbits speichern
		{
		cByte = m_iLwzDate << (8-m_iLwzBits);
		WriteData(&cByte,1);
		}

	m_iLwzBits = 0;

}

//*****************************************************************************
//*
//*		CompressLZWAdd
//*
//*****************************************************************************
//  DESCRIPTION:    Add a string consisting of the string of index plus
//                  the byte ucByte.
//
//                  If a string of length 1 is wanted, the index should
//                  be 0xFFFF.
//
//  PARAMETERS:     uIndex   - index to first part of string, or 0xFFFF is
//                            only 1 byte is wanted
//                  ucByte    - Last byte in new string
//
//  RETURNS:        index to new string, or 0xFFFF if no more room
unsigned TiffEncoder::CompressLZWAdd(unsigned uIndex, unsigned char ucByte)
{
unsigned uHashIndex;


														
    if (m_iLwzCount >= MAXSTR)return 0xFFFF;			// Check if there is more room

    
    uHashIndex = HASH(uIndex, ucByte);					// Search the string table until a free position is found
    
	while(m_pStingHash[uHashIndex] != 0xFFFF)
		{
        uHashIndex = (uHashIndex+HASHSTEP)%HASHSIZE;
		}

    
    m_pStingHash[uHashIndex ] =  m_iLwzCount;			// Insert new string
    m_pStingChar[m_iLwzCount] =  ucByte;
    m_pStingNext[m_iLwzCount] = (uIndex!=0xFFFF) ? uIndex:NEXT_FIRST;


return m_iLwzCount++;
}


//*****************************************************************************
//*
//*		CompressLZWFind
//*
//*****************************************************************************
//  DESCRIPTION:    Find index of string consisting of the string of index
//                  plus the byte ucByte.
//                  If a string of length 1 is wanted, the index should be 0xFFFF.
//
//  PARAMETERS:     uIndex   - index to first part of string, or 0xFFFF is
//                            only 1 byte is wanted
//                  ucByte    - Last byte in string
//
//  RETURNS:        uIndex to string, or 0xFFFF if not found
unsigned TiffEncoder::CompressLZWFind(unsigned uIndex,unsigned char ucByte)
{
unsigned	uHashIndex;
unsigned	uNextIndex;


														//  Check if index is 0xFFFF. In that case we need only
														//  return ucByte, since all one-character strings has their
														//  bytevalue as their index
    if(uIndex==0xFFFF)return ucByte;

														//  Search the string table until the string is found, or
														//  we find HASH_FREE. In that case the string does not exist.
    uHashIndex = HASH(uIndex, ucByte);

    while((uNextIndex=m_pStingHash[uHashIndex])!=0xFFFF)
		{
        if(m_pStingNext[uNextIndex]==uIndex)
		if(m_pStingChar[uNextIndex]==ucByte)
			{
            return uNextIndex;
			}

        uHashIndex = (uHashIndex+HASHSTEP)%HASHSIZE;
		}


return 0xFFFF;											// No match is found
}

//*****************************************************************************
//*
//*		CompressLZWInit
//*
//*****************************************************************************
//	Setzt die StringTabel zurck
void TiffEncoder::CompressLZWInit()
{
unsigned short *pWord;
int				iValW;
int				iValQ;


    m_iLwzCount = 0;
    pWord		= m_pStingHash;							// Mark entire hashtable as free

	for(iValQ=0;iValQ<HASHSIZE;iValQ++)
		{
        *pWord++ = HASH_FREE;
		}

    
    iValW = (1<<8)+RES_CODES;							// Insert 2^8 one-character strings, and reserved codes
    
	for(iValQ=0;iValQ<iValW;iValQ++)
		{
		CompressLZWAdd(0xFFFF,iValQ);
		}



}

//*****************************************************************************
//*
//*		CompressLZW
//*
//*****************************************************************************
//	Ergibt die Anzahl an eingelesenen Bytes bzw. -1 bei einem Fehler
int TiffEncoder::CompressLZW(unsigned char *pBuffer,int iMax)
{
int				iPos;
int				iBytes;
int				iClearCode;
int				iExitCode;
unsigned char 	ucByte;
unsigned char 	ucLast;
unsigned		uIndex;
unsigned		uCode;
char			cLzwPos;
char			cLzwLast[4];


	iPos		= 0;
	iBytes		= 0;
	cLzwPos     = 0;
	cLzwLast[0] = 0;
	cLzwLast[1] = 0;
	cLzwLast[2] = 0;
	cLzwLast[3] = 0;
    iClearCode	= 256;
    iExitCode	= 257;


    while(iPos<iMax)
		{
		ucByte = pBuffer[iPos];								// Byte holen
		iPos++;

		if(m_iDifferenz==2)									// Differenz Signale
			{
			if(cLzwPos>=m_iSamplesPerPixel)cLzwPos=0;
			
			ucLast			  = cLzwLast[cLzwPos];
			cLzwLast[cLzwPos] = ucByte;
			ucByte           -= ucLast;    
			cLzwPos          += 1;
			}

		uIndex = CompressLZWFind(m_iLwzPrefix,ucByte);

		if(uIndex!=0xFFFF)
			{
            m_iLwzPrefix   = uIndex;
			}
		else{
            iBytes += WriteBits(m_iLwzPrefix);
			uCode   = CompressLZWAdd(m_iLwzPrefix,ucByte);

            if(uCode>=(unsigned)m_iEndCode)					// Code Gre erhhen
				{
				if(uCode>=4093)
					{
					iBytes += WriteBits(iClearCode);
					CompressLZWInit();
					m_iCodeSize = 8;
					}
			
				m_iCodeSize++;
				m_iEndCode = (1<<m_iCodeSize)-1;
				
				if(m_iEndCode>4093)
					{
					m_iEndCode=4093;
					}
				}

            m_iLwzPrefix = ucByte;
			}
		}

return iBytes;
}



//*****************************************************************************
//*
//*		WriteBits
//*
//*****************************************************************************
//	Speichert Size Bits aus iCode
int TiffEncoder::WriteBits(int iCode)
{
char	cCount=0;
char	cByte;

	m_iLwzDate <<= m_iCodeSize;
	m_iLwzBits  += m_iCodeSize;
	m_iLwzDate  |= iCode;

	while(m_iLwzBits>=8)					// Bits speichern
		{
		cByte = m_iLwzDate>>(m_iLwzBits-8);
		WriteData(&cByte,1);
		m_iLwzBits -= 8;
		cCount++;
		}

return cCount;
}


//*****************************************************************************
//*
//*		WriteRGB
//*
//*****************************************************************************
//	List RGB Bilder ein
void TiffEncoder::WriteRGB()
{
unsigned char  *pLine;
int				iLineSize;
int				iBlock;
int				iPosY;
int				iLen;



	iPosY = 0;

	if(m_iSave32Bit && m_pImage->BitCount()==32)
		{
		m_iSamplesPerPixel=4;
		}
	else{
		m_iSamplesPerPixel=3;
		}

	if(m_iCompressType==5)m_iDifferenz=2;
	if(m_iCompressType!=1 && m_iCompressType!=5 && m_iCompressType!=32773)
		{
		throw ETiffError("Modus ist nicht untersttzt");
		}

	m_iImageType = 2;
	iLineSize    = m_iSamplesPerPixel * m_iWidth;

	   m_iRowsPerStrip  = 0x2000/iLineSize;
	if(m_iRowsPerStrip <= 0      )m_iRowsPerStrip=1;
	if(m_iRowsPerStrip >m_iHeight)m_iRowsPerStrip=m_iHeight;


	m_iRowOffsetCnt = (m_iHeight+m_iRowsPerStrip-1)/m_iRowsPerStrip;
	m_pRowOffsets   = new int[m_iRowOffsetCnt];
	m_pRowByteCnt   = new int[m_iRowOffsetCnt];

	for(iBlock=0;iBlock<m_iRowOffsetCnt;iBlock++)	// Blockweise speichern
		{
		if(iPosY>=m_iHeight)break;

		m_pRowOffsets[iBlock] = m_pOutput->Tell()-m_iBaseSeek;
		if(m_iCompressType==5)CompressLZWReset();

		for(;iPosY<m_iHeight;)
			{
			//******** Zeile aus Bild holen ***********************************
			switch(m_iSamplesPerPixel)				// Zeile in Bild einfgen
					{
			case 4:	pLine=m_pImage->GetRow4Temp(iPosY);
					      m_pImage->SwapRGB4(pLine,m_iWidth);
					break;
			case 3:	pLine=m_pImage->GetRow3Temp(iPosY);
					      m_pImage->SwapRGB3(pLine,m_iWidth);
					break;
					}

			//******** Zeile laden ********************************************
			switch(m_iCompressType)
					{
			case 1:	iLen=m_pOutput->Write(pLine,iLineSize);
					if(iLen!=iLineSize)iLen=-1;
					break;
			case 5:	iLen = CompressLZW     (pLine,iLineSize);	break;
			default:iLen = CompressPackBits(pLine,iLineSize);	break;
					}

			if(iLen<0)
				{
				throw ETiffError("Fehler beim Schreiben");
				}

			iPosY++;
			CallProgressFunction((iPosY*100)/m_iHeight);// Nchste Zeile
			if(iPosY%m_iRowsPerStrip==0)break;
			}

		if(m_iCompressType==5)CompressLZWFlush();

		m_pRowByteCnt[iBlock]  = m_pOutput->Tell()-m_iBaseSeek;
		m_pRowByteCnt[iBlock] -= m_pRowOffsets[iBlock];
		}

}


//*****************************************************************************
//*
//*		WritePal
//*
//*****************************************************************************
//	List 8 Bit Pallete ein
void TiffEncoder::WritePal()
{
unsigned char  *pLine;
unsigned		uValue;
int				iLineSize;
int				iBlock;
int				iPosY;
int				iNum;
int				iLen;

	
	iPosY				= 0;
	m_iSamplesPerPixel	= 1;
	m_iBitPerSample		= m_pImage->BitCount();

	if((m_iCompressType  != 1 && m_iCompressType != 5 && m_iCompressType != 32773) ||
	   (m_iBitPerSample != 8 && m_iBitPerSample!= 4))
		{
		throw ETiffError("Modus ist nicht untersttzt");
		}

	m_iImageType = 3;
	iLineSize    = (m_iWidth*m_iBitPerSample+7)/8;

	if(m_iColorCnt==(1<<m_iBitPerSample))			// Prfe ob Monochrom
		{
		for(iNum=0;iNum<m_iColorCnt;iNum++)
			{
			uValue=iNum<<(8-m_iBitPerSample);
			uValue=uValue|(uValue<<8)|(uValue<<16);
			if(uValue!=m_pImage->GetColor(iNum))break;
			}
		
		if(iNum==m_iColorCnt)
			{
			m_iImageType = 1;
			m_iColorCnt  = 0;
			}
		}



	   m_iRowsPerStrip  = 0x2000/iLineSize;
	if(m_iRowsPerStrip <=0        )m_iRowsPerStrip=1;
	if(m_iRowsPerStrip > m_iHeight)m_iRowsPerStrip=m_iHeight;

	m_iRowOffsetCnt = (m_iHeight+m_iRowsPerStrip-1)/m_iRowsPerStrip;
	m_pRowOffsets   = new int[m_iRowOffsetCnt];
	m_pRowByteCnt   = new int[m_iRowOffsetCnt];

	for(iBlock=0;iBlock<m_iRowOffsetCnt;iBlock++)	// Blockweise speichern
		{
		if(iPosY>=m_iHeight)break;

		m_pRowOffsets[iBlock] = m_pOutput->Tell()-m_iBaseSeek;
		if(m_iCompressType==5)CompressLZWReset();

		for(;iPosY<m_iHeight;)
			{
			//******** Zeile aus Bild holen ***********************************
			
			pLine=m_pImage->GetRow(iPosY);

			//******** Zeile laden ********************************************
			switch(m_iCompressType)
					{
			case 1:	iLen=m_pOutput->Write(pLine,iLineSize);
					if(iLen!=iLineSize)iLen=-1;
					break;
			case 5:	iLen = CompressLZW     (pLine,iLineSize);	break;
			default:iLen = CompressPackBits(pLine,iLineSize);	break;
					}

			if(iLen<0)
				{
				throw ETiffError("Fehler beim Schreiben");
				}

			iPosY++;
			CallProgressFunction((iPosY*100)/m_iHeight);	// Nchste Zeile
			if(iPosY%m_iRowsPerStrip==0)break;
			}

		if(m_iCompressType==5)CompressLZWFlush();

		m_pRowByteCnt[iBlock]  = m_pOutput->Tell()-m_iBaseSeek;
		m_pRowByteCnt[iBlock] -= m_pRowOffsets[iBlock];
		}

}


//*****************************************************************************
//*
//*		WriteMono
//*
//*****************************************************************************
//	List eine SW-Grafik ein
void TiffEncoder::WriteMono()
{
unsigned char  *pLine;
int				iLineSize;
int				iPosY;
int				iBlock;
int				iLen;


	iPosY				= 0;
	m_iBitPerSample		= 1;
	m_iSamplesPerPixel	= 1;

	if((m_iCompressType != 1 && m_iCompressType != 5 && m_iCompressType != 32773))
		{
		throw ETiffError("Modus ist nicht untersttzt");
		}

	m_iImageType = 1;
	iLineSize    = (m_iWidth+7)/8;

	   m_iRowsPerStrip  = 0x2000/iLineSize;
	if(m_iRowsPerStrip <=0       )m_iRowsPerStrip=1;
	if(m_iRowsPerStrip >m_iHeight)m_iRowsPerStrip=m_iHeight;
	
	m_iRowsPerStrip=161*2;

	m_iRowOffsetCnt = (m_iHeight+m_iRowsPerStrip-1)/m_iRowsPerStrip;
	m_pRowOffsets   = new int[m_iRowOffsetCnt];
	m_pRowByteCnt   = new int[m_iRowOffsetCnt];

	for(iBlock=0;iBlock<m_iRowOffsetCnt;iBlock++)	// Blockweise speichern
		{
		if(iPosY>=m_iHeight)break;

		m_pRowOffsets[iBlock] = m_pOutput->Tell()-m_iBaseSeek;
		if(m_iCompressType==5)CompressLZWReset();

		for(;iPosY<m_iHeight;)
			{

			//******** Zeile aus Bild holen ***********************************
			
			pLine=m_pImage->GetRow(iPosY);

			//******** Zeile laden ********************************************
			
			switch(m_iCompressType)
					{
			case 1:	iLen=m_pOutput->Write(pLine,iLineSize);
					if(iLen!=iLineSize)iLen=-1;
					break;
			case 5:	iLen = CompressLZW     (pLine,iLineSize);	break;
			default:iLen = CompressPackBits(pLine,iLineSize);	break;
					}

			if(iLen<0)
				{
				throw ETiffError("Fehler beim Schreiben");
				}

			iPosY++;								// Nchste Zeile
			CallProgressFunction((iPosY*100)/m_iHeight);
			if(iPosY%m_iRowsPerStrip==0)break;
			}

		if(m_iCompressType==5)CompressLZWFlush();

		m_pRowByteCnt[iBlock]  = m_pOutput->Tell()-m_iBaseSeek;
		m_pRowByteCnt[iBlock] -= m_pRowOffsets[iBlock];
		}

}


