//*****************************************************************************
//*
//*
//*		TiffEncoder.h
//*
//*
//*****************************************************************************
//
//	Copyright (c) 2002 AZ Software
//	All rights reserved.
//	Author:  Anton Zechner
//

#ifndef __TIFFENCODER_H__
#define __TIFFENCODER_H__

#include "Tiff.h"
#include "BitmapImage.h"
#include "..\FileBuffer.h"


//*****************************************************************************
//*
//*		Klasse TiffEncoder
//*
//*****************************************************************************
class TiffEncoder : public BitmapImageEncoder
{
public:

				 TiffEncoder();
	virtual     ~TiffEncoder();
	virtual void WriteImage (const wchar_t *pName,const BitmapImage &sImage);
	virtual void WriteImage (const char    *pName,const BitmapImage &sImage);
	virtual void WriteImage (CFileBuffer   *pFile,const BitmapImage &sImage);
	void		 SetPixelPerMeter(int iMeterX,int iMeterY);
	void		 SetCompressType (int iType);
	void		 SetSaveType     (int iType);

private:

	void		Initialize ();
	void		FreeBuffers();
	void		CallProgressFunction (unsigned iPercent);
	void		WriteRGB();
	void		WritePal();
	void		WriteMono();
	void		WriteData(void *pPtr,int iCount);
	int 		WriteBits(int iCode);

	int			CompressPackBits(unsigned char *pBuffer,int iMax);
	int			CompressLZW     (unsigned char *pBuffer,int iMax);
	void		CompressLZWReset();
	void 		CompressLZWFlush();
	void 		CompressLZWInit ();
	unsigned	CompressLZWFind (unsigned uIndex,unsigned char ucByte);
	unsigned	CompressLZWAdd  (unsigned uIndex,unsigned char ucByte);

		  CFileBuffer    *m_pOutput;
	const BitmapImage	 *m_pImage;

	int			m_iBaseSeek;					// Anfangs Offset in der Datei
	int			m_iWidth;						// Bildauflsung
	int			m_iHeight;						//		*
	int			m_iCompressType;				// Compresions iType
	int			m_iImageType;					// Bild Typ
	int			m_iMeterX;						// Auflsung in Pixel je Meter
	int			m_iMeterY;						//		*
	int			m_iBitPerSample;				// Bits je Pixel
	int			m_iSamplesPerPixel;				// Sample je Pixel
	int			m_iDifferenz;					// Differenz Codiert bei LZW
	int			m_iSave32Bit;					// RGB 24 als 32 Bit speichern
	int			m_iColorCnt;					// Anzahl der zuspeichernten Farben

	int			m_iRowsPerStrip;				// Reihen je Speicher Block
	int 	   *m_pRowOffsets;					// Offsets der Speicher Blcke
	int			m_iRowOffsetPos;				// Offset  der Speicher Blcke in Datei
	int			m_iRowOffsetCnt;				// Anzahl  der Speicher Blcke
	int 	   *m_pRowByteCnt;					// Lngen  der Speicher Blcke
	int		    m_iRowBytePos;					// Offset  der Speicher Blocklngen in Datei

	//**************** LZW-Daten **********************************************
	
	int			m_iEndCode;
	int			m_iCodeSize;
	int			m_iLwzDate;
	int			m_iLwzBits;
	int			m_iLwzPrefix;
	int			m_iLwzCount;

	unsigned int   *m_pStingNext;
	unsigned char  *m_pStingChar;
    unsigned short *m_pStingHash;


};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	void TiffEncoder::SetCompressType(int iType){m_iCompressType=iType;}
inline	void TiffEncoder::SetSaveType(int iType){m_iSave32Bit=iType;}
inline	void TiffEncoder::WriteData(void *pPtr,int iCount){if(m_pOutput->Write(pPtr,iCount)!=iCount)throw ETiffError("Fehler beim Schreiben");}

#endif
