//*****************************************************************************
//*
//*
//*		Tls.h
//*
//*
//*****************************************************************************
#ifndef __TLS_H__
#define __TLS_H__


#define TLS_SIZE_MAX_X	0x1000
#define TLS_SIZE_MAX_Y	0x1000

typedef struct
	{
	unsigned		uSignature;							// "TLSP"
	unsigned short 	usSizeX;							// Bildbreite
	unsigned short 	usSizeY;							// Bildhhe
	unsigned char	ucFormat;							// 1=RGB-3Bit 4=RGB-24Bit 
	unsigned char	ucSequPics;							// Zustzliche Sequenzbilder
	unsigned short	usCrc;								// CRC16 mit Polygon x^16 + x^12 + x^5 + 1
	unsigned		uPicureId;							// Eindeutige Bild-ID
	}TlsFileHeader;

typedef struct
	{
	unsigned		aAreaErrors[16];					// Erlaubte Fehler pro Bereich
	}TlsAreaHeader;

typedef struct
	{
	unsigned short 	usShowTime;							// Anzeigezeit in Millisekunden
	unsigned char	ucNextPicture;						// Nchstes Bild
	unsigned char	ucUnused;							// Unbenutzt
	}TlsSequenceHeader;

unsigned TlsCrcCalc(const void *pPtr,unsigned uDataLen,unsigned short uCrc=0);

#include "GrException.h"

#ifdef	ERROR_DEUTSCH
	#define	ET_ERR_OF	"Fehler beim ffnen der TLS-Datei !"
	#define	ET_ERR_RF	"Fehler beim Lesen der TLS-Datei !"
	#define	ET_ERR_WF	"Fehler beim Schreiben TLS-Datei !"
	#define	ET_ERR_WS	"Die Datei ist keine TLS-Datei !"
	#define	ET_ERR_CS	"Beschdigte TLS-Datei !"
	#define	ET_ERR_US	"Das Bild wurde in der Sequenz nicht gefunden !"
	#define	ET_ERR_UE	"Fehler beim Unknown Error"
	#define	ET_ERR_CR	"Falsche CRC-Summe !"
	#define	EB_ERR_DE	"Falsche Bild-Daten !"
#else
	#define	ET_ERR_OF	"Error opening TLS file"
	#define	ET_ERR_RF	"Error reading TLS file"
	#define	ET_ERR_WF	"Error writing TLS file"
	#define	ET_ERR_WS	"Not a Bitmap stream"
	#define	ET_ERR_CS	"Corrupt Bitmap stream"
	#define	ET_ERR_US	"Can't fild picture in sequence"
	#define	ET_ERR_UE	"Unknown Error"
	#define	ET_ERR_CR	"Wrong CRC value"
	#define	EB_ERR_DE	"Bad input data"
#endif

class ETlsFileOpenError : public EGraphicsException
{
public:
  ETlsFileOpenError () : EGraphicsException (ET_ERR_OF) {}
  ETlsFileOpenError (const ETlsFileOpenError &be) : EGraphicsException (be) {}
  ETlsFileOpenError &operator=(const ETlsFileOpenError &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this ;
  }
} ;

class ETlsFileReadError : public EGraphicsException
{
public:
  ETlsFileReadError () : EGraphicsException (ET_ERR_RF) {}
  ETlsFileReadError (const ETlsFileReadError &be) : EGraphicsException (be) {}
  ETlsFileReadError &operator=(const ETlsFileReadError &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this;
  }
};

class ETlsFileWriteError : public EGraphicsException
{
public:
  ETlsFileWriteError () : EGraphicsException (ET_ERR_WF) {}
  ETlsFileWriteError (const ETlsFileWriteError &be) : EGraphicsException (be) {}
  ETlsFileWriteError &operator=(const ETlsFileWriteError &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this;
  }
} ;

class ETlsNotATlsFile : public EGraphicsException
{
public:
  ETlsNotATlsFile () : EGraphicsException (ET_ERR_WS) {}
  ETlsNotATlsFile (const ETlsNotATlsFile &be) : EGraphicsException (be) {}
  ETlsNotATlsFile &operator=(const ETlsNotATlsFile &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this ;
  }
} ;

class ETlsCorruptFile : public EGraphicsException
{
public:
  ETlsCorruptFile () : EGraphicsException (ET_ERR_CS) {}
  ETlsCorruptFile (const ETlsCorruptFile &be) : EGraphicsException (be) {}
  ETlsCorruptFile &operator=(const ETlsCorruptFile &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this ;
  }
} ;

class ETlsNotFound : public EGraphicsException
{
public:
  ETlsNotFound () : EGraphicsException (ET_ERR_US) {}
  ETlsNotFound (const ETlsNotFound &be) : EGraphicsException (be) {}
  ETlsNotFound &operator=(const ETlsNotFound &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this ;
  }
} ;

class ETlsUnknownError : public EGraphicsException
{
public:
  ETlsUnknownError () : EGraphicsException (ET_ERR_UE) {}
  ETlsUnknownError (const ETlsUnknownError &be) : EGraphicsException (be) {}
  ETlsUnknownError &operator=(const ETlsUnknownError &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this ;
  }
} ;

class ETlsCrcError : public EGraphicsException
{
public:
  ETlsCrcError () : EGraphicsException (ET_ERR_CR) {}
  ETlsCrcError (const ETlsCrcError &be) : EGraphicsException (be) {}
  ETlsCrcError &operator=(const ETlsCrcError &ge)
  {
    this->EGraphicsException::operator=(ge);
    return *this ;
  }
} ;

class ETlsDataError : public EGraphicsException
{
public:
  ETlsDataError () : EGraphicsException (EB_ERR_DE) {}
  ETlsDataError (const ETlsDataError &be) : EGraphicsException (be) {}
  ETlsDataError &operator=(const ETlsDataError &ge)
  {
    this->EGraphicsException::operator=(ge) ;
    return *this ;
  }
} ;

#endif
