//*****************************************************************************
//*
//*
//*		TlsDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2013
//
//	Ld TLSP-Bilder.
//

#include "TlsDecoder.h"


typedef struct
	{
	unsigned char	ucCnt;
	unsigned char	ucCmd;
	}RleCode;

static const unsigned   uSignature = 'T'|('L'<<8)|('S'<<16)|('P'<<24);

static const unsigned short aCrcTable[] =
    {
    0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50A5, 0x60C6, 0x70E7,
    0x8108, 0x9129, 0xA14A, 0xB16B, 0xC18C, 0xD1AD, 0xE1CE, 0xF1EF,
    0x1231, 0x0210, 0x3273, 0x2252, 0x52B5, 0x4294, 0x72F7, 0x62D6,
    0x9339, 0x8318, 0xB37B, 0xA35A, 0xD3BD, 0xC39C, 0xF3FF, 0xE3DE,
    0x2462, 0x3443, 0x0420, 0x1401, 0x64E6, 0x74C7, 0x44A4, 0x5485,
    0xA56A, 0xB54B, 0x8528, 0x9509, 0xE5EE, 0xF5CF, 0xC5AC, 0xD58D,
    0x3653, 0x2672, 0x1611, 0x0630, 0x76D7, 0x66F6, 0x5695, 0x46B4,
    0xB75B, 0xA77A, 0x9719, 0x8738, 0xF7DF, 0xE7FE, 0xD79D, 0xC7BC,
    0x48C4, 0x58E5, 0x6886, 0x78A7, 0x0840, 0x1861, 0x2802, 0x3823,
    0xC9CC, 0xD9ED, 0xE98E, 0xF9AF, 0x8948, 0x9969, 0xA90A, 0xB92B,
    0x5AF5, 0x4AD4, 0x7AB7, 0x6A96, 0x1A71, 0x0A50, 0x3A33, 0x2A12,
    0xDBFD, 0xCBDC, 0xFBBF, 0xEB9E, 0x9B79, 0x8B58, 0xBB3B, 0xAB1A,
    0x6CA6, 0x7C87, 0x4CE4, 0x5CC5, 0x2C22, 0x3C03, 0x0C60, 0x1C41,
    0xEDAE, 0xFD8F, 0xCDEC, 0xDDCD, 0xAD2A, 0xBD0B, 0x8D68, 0x9D49,
    0x7E97, 0x6EB6, 0x5ED5, 0x4EF4, 0x3E13, 0x2E32, 0x1E51, 0x0E70,
    0xFF9F, 0xEFBE, 0xDFDD, 0xCFFC, 0xBF1B, 0xAF3A, 0x9F59, 0x8F78,
    0x9188, 0x81A9, 0xB1CA, 0xA1EB, 0xD10C, 0xC12D, 0xF14E, 0xE16F,
    0x1080, 0x00A1, 0x30C2, 0x20E3, 0x5004, 0x4025, 0x7046, 0x6067,
    0x83B9, 0x9398, 0xA3FB, 0xB3DA, 0xC33D, 0xD31C, 0xE37F, 0xF35E,
    0x02B1, 0x1290, 0x22F3, 0x32D2, 0x4235, 0x5214, 0x6277, 0x7256,
    0xB5EA, 0xA5CB, 0x95A8, 0x8589, 0xF56E, 0xE54F, 0xD52C, 0xC50D,
    0x34E2, 0x24C3, 0x14A0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
    0xA7DB, 0xB7FA, 0x8799, 0x97B8, 0xE75F, 0xF77E, 0xC71D, 0xD73C,
    0x26D3, 0x36F2, 0x0691, 0x16B0, 0x6657, 0x7676, 0x4615, 0x5634,
    0xD94C, 0xC96D, 0xF90E, 0xE92F, 0x99C8, 0x89E9, 0xB98A, 0xA9AB,
    0x5844, 0x4865, 0x7806, 0x6827, 0x18C0, 0x08E1, 0x3882, 0x28A3,
    0xCB7D, 0xDB5C, 0xEB3F, 0xFB1E, 0x8BF9, 0x9BD8, 0xABBB, 0xBB9A,
    0x4A75, 0x5A54, 0x6A37, 0x7A16, 0x0AF1, 0x1AD0, 0x2AB3, 0x3A92,
    0xFD2E, 0xED0F, 0xDD6C, 0xCD4D, 0xBDAA, 0xAD8B, 0x9DE8, 0x8DC9,
    0x7C26, 0x6C07, 0x5C64, 0x4C45, 0x3CA2, 0x2C83, 0x1CE0, 0x0CC1,
    0xEF1F, 0xFF3E, 0xCF5D, 0xDF7C, 0xAF9B, 0xBFBA, 0x8FD9, 0x9FF8,
    0x6E17, 0x7E36, 0x4E55, 0x5E74, 0x2E93, 0x3EB2, 0x0ED1, 0x1EF0
    };


//*****************************************************************************
//*
//*		TlsCrcCalc
//*
//*****************************************************************************
unsigned TlsCrcCalc(const void *pPtr,unsigned uDataLen,unsigned short uCrc)
{
unsigned char  *pMem=(unsigned char*)pPtr;
unsigned		uPos;


    for(uPos=0;uPos<uDataLen;uPos++)
        {
        uCrc = (uCrc<<8) ^ aCrcTable[(uCrc>>8)^pMem [uPos]];
        }

return uCrc;
}



//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
TlsDecoder::TlsDecoder()
{
	m_pBuffer					= 0;
	m_uLoadNumber				= 0;
	m_sFileHeader.uPicureId		= 0;
	m_sSequence.usShowTime		= 0;
	m_sSequence.ucNextPicture	= 0;
}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
TlsDecoder::~TlsDecoder()
{
}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void TlsDecoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel);

	if(bCancel)throw EGraphicsAbort();
}

//*****************************************************************************
//*
//*		SelectPicture
//*
//*****************************************************************************
void TlsDecoder::SelectPicture(unsigned uLoadNumber)
{

	m_uLoadNumber = uLoadNumber;

}

//*****************************************************************************
//*
//*		GetPicureId
//*
//*****************************************************************************
unsigned TlsDecoder::GetPicureId()
{

	return m_sFileHeader.uPicureId;

}

//*****************************************************************************
//*
//*		GetSequence
//*
//*****************************************************************************
void TlsDecoder::GetSequence(unsigned &uTime,unsigned &uNext)
{

	uTime = m_sSequence.usShowTime;
	uNext = m_sSequence.ucNextPicture;

}

//*****************************************************************************
//*
//*		GetErrorAreas
//*
//*****************************************************************************
void TlsDecoder::GetErrorAreas(unsigned *pArea)
{

	memcpy(pArea,m_sAreas.aAreaErrors,16*4);

}

//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
void TlsDecoder::ReadImage(const char *pName,BitmapImage &sImage){ReadImage(pName,sImage,m_uLoadNumber);}
void TlsDecoder::ReadImage(const char *pName,BitmapImage &sImage,unsigned uPictureNr)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw ETlsFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage,uPictureNr);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void TlsDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage){ReadImage(pName,sImage,m_uLoadNumber);}
void TlsDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage,unsigned uPictureNr)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw ETlsFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage,uPictureNr);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void TlsDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage){ReadImage(pFile,sImage,m_uLoadNumber);}
void TlsDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage,unsigned uPictureNr)
{
void		   *pPointer;
unsigned short 	usCalcCrc;
unsigned short 	usHeaderCrc;
unsigned		uBytesRead;
unsigned	 	uBitCount;
unsigned		uLineSize;
unsigned		uNumber;
unsigned		uLines;
unsigned		uStart;
unsigned		uStep;
unsigned		uPos;
unsigned		uNum;
unsigned		uOff;
unsigned		uRow;
unsigned		uCol;
unsigned		uBit;



	if(uPictureNr>256)uPictureNr=256;

	uBytesRead	= 0;
	m_pBuffer	= pFile;


	   uNum = m_pBuffer->Read(&m_sFileHeader,sizeof(m_sFileHeader));
	if(uNum!= sizeof(m_sFileHeader))throw ETlsFileReadError();

	uBytesRead += uNum;

	if(m_sFileHeader.uSignature!=uSignature)				// Passt der Dateityp
		{
		throw ETlsNotATlsFile();
		}

	if(m_sFileHeader.ucFormat==1)							// Bildformat prfen
		{
		uBitCount = 8;
		}
	else if(m_sFileHeader.ucFormat==4)
		{
		uBitCount = 32;
		}
	else{
		throw ETlsCorruptFile();
		}

	if(m_sFileHeader.usSizeX>TLS_SIZE_MAX_X || m_sFileHeader.usSizeY>TLS_SIZE_MAX_Y)
		{
		throw ETlsCorruptFile();
		}

	if(m_sFileHeader.usSizeX==0 || m_sFileHeader.usSizeY==0)
		{
		throw ETlsCorruptFile();
		}

	if(uPictureNr>m_sFileHeader.ucSequPics)					// Die Bildnummer wurde nicht gefunden
		{
		throw ETlsNotFound();
		}


	   uNum = m_pBuffer->Read(&m_sAreas,sizeof(m_sAreas));
	if(uNum!= sizeof(m_sAreas))throw ETlsCorruptFile();


	usHeaderCrc = m_sFileHeader.usCrc;
	m_sFileHeader.usCrc = 0;


//******************** Bild laden *********************************************


	CallProgressFunction(0);

															// Neues Bild erstellen
	if(!sImage.SetSize(uBitCount,m_sFileHeader.usSizeX,m_sFileHeader.usSizeY,BI_NOT_INIT))
		{
		throw ETlsUnknownError();
		}

	uLines		=   m_sFileHeader.usSizeY*(m_sFileHeader.ucSequPics+1);
	uLineSize	= ((m_sFileHeader.usSizeX*uBitCount+31)/32)*4;
	usCalcCrc	= 	TlsCrcCalc(&m_sFileHeader,sizeof(m_sFileHeader),0);
	usCalcCrc	= 	TlsCrcCalc(&m_sAreas     ,sizeof(m_sAreas),usCalcCrc);
	uStart		=   m_sFileHeader.usSizeY*uPictureNr;
	uStep		=  	m_sFileHeader.usSizeY;
	uNumber		=   0;
	uOff		=   0;

	for(uRow=0;uRow<uLines;uRow++)							// Alle Zeilen laden
		{
		CallProgressFunction(uRow*100/uLines);

		if(uRow<uStart || uOff>=m_sFileHeader.usSizeY)		// In den unbenutzten Bildern
			{
			if(m_pBuffer->BufferFill(0,&pPointer,uLineSize)<0)
				{
				throw ETlsCorruptFile();
				}

			usCalcCrc = TlsCrcCalc(pPointer,uLineSize,usCalcCrc);

			if(m_pBuffer->BufferFill(uLineSize,&pPointer,0)<0)
				{
				throw ETlsCorruptFile();
				}
			}
		else{												// Im richtigen Bild
			if(m_pBuffer->Read(sImage[uOff],uLineSize)!=(int)uLineSize)
				{
				throw ETlsCorruptFile();
				}


			usCalcCrc = TlsCrcCalc(sImage[uOff],uLineSize,usCalcCrc);
			uOff++;
			}

		   uStep--;
		if(uStep>0)continue;								// Ende eines Bildes;

		if(m_sFileHeader.ucSequPics==0)						// Keine Sequenz
			{
			break;
			}


		if(uNumber==uPictureNr)								// Sequenz-Header laden
			{
			if(m_pBuffer->Read(&m_sSequence,sizeof(m_sSequence))!=sizeof(m_sSequence))
				{
				throw ETlsCorruptFile();
				}

			usCalcCrc = TlsCrcCalc(&m_sSequence,sizeof(m_sSequence),usCalcCrc);
			}
		else{
			if(m_pBuffer->BufferFill(0,&pPointer,sizeof(m_sSequence))<0)
				{
				throw ETlsCorruptFile();
				}

			usCalcCrc = TlsCrcCalc(pPointer,sizeof(m_sSequence),usCalcCrc);

			if(m_pBuffer->BufferFill(sizeof(m_sSequence),&pPointer,0)<0)
				{
				throw ETlsCorruptFile();
				}
			}

		uNumber++;
		if(uNumber>m_sFileHeader.ucSequPics)break;

		if(uNumber==uPictureNr)								// Fehlerbereiche laden
			{
			if(m_pBuffer->Read(&m_sAreas,sizeof(m_sAreas))!=sizeof(m_sAreas))
				{
				throw ETlsCorruptFile();
				}

			usCalcCrc = TlsCrcCalc(&m_sAreas,sizeof(m_sAreas),usCalcCrc);
			}
		else{
			if(m_pBuffer->BufferFill(0,&pPointer,sizeof(m_sAreas))<0)
				{
				throw ETlsCorruptFile();
				}

			usCalcCrc = TlsCrcCalc(pPointer,sizeof(m_sAreas),usCalcCrc);

			if(m_pBuffer->BufferFill(sizeof(m_sAreas),&pPointer,0)<0)
				{
				throw ETlsCorruptFile();
				}
			}

		uStep = m_sFileHeader.usSizeY;
		}


	if(usCalcCrc!=usHeaderCrc)								// CRC prfen
		{
		throw ETlsCrcError();
		}


	CallProgressFunction(100);

	if(uBitCount==8)										// Farben mit Bereichen einstellen
		{
		for(uPos=0;uPos<11*16;uPos++)
			{
			uOff  = (uPos >> 4)*3;
			uBit  = (m_sAreas.aAreaErrors[uPos&0x0F]>>uOff)&0x07;
			uNum  = (uPos&0x8F);
			uNum |= (uBit << 4);

			sImage.SetColor(uPos,(uPos&0x40)? 255:0,(uPos&0x20)? 255:0,(uPos&0x10)? 255:0,uNum);
			}

		for(;uPos<256;uPos++)
			{
			uNum  = (uPos&0x8F);
			sImage.SetColor(uPos,(uPos&0x40)? 255:0,(uPos&0x20)? 255:0,(uPos&0x10)? 255:0,uNum);
			}
		}
	else{													// Bereich in die ersten Pixel einfgen
		for(uPos=0;uPos<11*16;uPos++)
			{
			uCol	= (uPos%m_sFileHeader.usSizeX);
			uRow	= (uPos/m_sFileHeader.usSizeX);
			uOff	= (uPos >> 4)*3;
			uBit	= (m_sAreas.aAreaErrors[uPos&0x0F]>>uOff)&0x07;
			uNum	= sImage.GetPixel(uCol,uRow);
			uNum   &= 0x8FFFFFFF;
			uNum   |= uBit<<28;

			sImage.SetPixel(uCol,uRow,uNum);
			}

		for(;uPos<11*16+11*2;uPos++)						// CRC-Bits einfgen
			{
			uCol	= (uPos%m_sFileHeader.usSizeX);
			uRow	= (uPos/m_sFileHeader.usSizeX);
			uBit	= (uPos-11*16)&0x07;
			uNum	= sImage.GetPixel(uCol,uRow);
			uNum   &= 0x8FFFFFFF;
			uNum   |= uBit<<28;

			sImage.SetPixel(uCol,uRow,uNum);
			}
		}

}

