//*****************************************************************************
//*
//*
//*		XpmDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2011
//
#include <string.h>
#include "XpmDecoder.h"

#define 	MAX_BUFFER		8192

typedef	struct
	{
	unsigned		uColor;
	char		   *pName;
	}XpmColor;

#define 	XRGB(r,g,b)	((b)+(g)*256+(r)*256*256)

static	XpmColor	aColors[]={
  { XRGB(240,248,255),	"aliceblue" },
  { XRGB(250,235,215),	"antiquewhite" },
  { XRGB(255,239,219),	"antiquewhite1" },
  { XRGB(238,223,204),	"antiquewhite2" },
  { XRGB(205,192,176),	"antiquewhite3" },
  { XRGB(139,131,120),	"antiquewhite4" },
  { XRGB(0  ,255,255),	"aqua" },
  { XRGB(127,255,212),	"aquamarine" },
  { XRGB(127,255,212),	"aquamarine1" },
  { XRGB(118,238,198),	"aquamarine2" },
  { XRGB(102,205,170),	"aquamarine3" },
  { XRGB( 69,139,116),	"aquamarine4" },
  { XRGB(240,255,255),	"azure" },
  { XRGB(240,255,255),	"azure1" },
  { XRGB(224,238,238),	"azure2" },
  { XRGB(193,205,205),	"azure3" },
  { XRGB(131,139,139),	"azure4" },
  { XRGB(245,245,220),	"beige" },
  { XRGB(255,228,196),	"bisque" },
  { XRGB(255,228,196),	"bisque1" },
  { XRGB(238,213,183),	"bisque2" },
  { XRGB(205,183,158),	"bisque3" },
  { XRGB(139,125,107),	"bisque4" },
  { XRGB(  0,  0,  0),	"black" },
  { XRGB(255,235,205),	"blanchedalmond" },
  { XRGB(  0,  0,255),	"blue" },
  { XRGB(  0,  0,255),	"blue1" },
  { XRGB(  0,  0,238),	"blue2" },
  { XRGB(  0,  0,205),	"blue3" },
  { XRGB(  0,  0,139),	"blue4" },
  { XRGB(138, 43,226),	"blueviolet" },
  { XRGB(165, 42, 42),	"brown" },
  { XRGB(255, 64, 64),	"brown1" },
  { XRGB(238, 59, 59),	"brown2" },
  { XRGB(205, 51, 51),	"brown3" },
  { XRGB(139, 35, 35),	"brown4" },
  { XRGB(222,184,135),	"burlywood" },
  { XRGB(255,211,155),	"burlywood1" },
  { XRGB(238,197,145),	"burlywood2" },
  { XRGB(205,170,125),	"burlywood3" },
  { XRGB(139,115, 85),	"burlywood4" },
  { XRGB( 95,158,160),	"cadetblue" },
  { XRGB(152,245,255),	"cadetblue1" },
  { XRGB(142,229,238),	"cadetblue2" },
  { XRGB(122,197,205),	"cadetblue3" },
  { XRGB( 83,134,139),	"cadetblue4" },
  { XRGB(127,255,  0),	"chartreuse" },
  { XRGB(127,255,  0),	"chartreuse1" },
  { XRGB(118,238,  0),	"chartreuse2" },
  { XRGB(102,205,  0),	"chartreuse3" },
  { XRGB( 69,139,  0),	"chartreuse4" },
  { XRGB(210,105, 30),	"chocolate" },
  { XRGB(255,127, 36),	"chocolate1" },
  { XRGB(238,118, 33),	"chocolate2" },
  { XRGB(205,102, 29),	"chocolate3" },
  { XRGB(139, 69, 19),	"chocolate4" },
  { XRGB(255,127, 80),	"coral" },
  { XRGB(255,114, 86),	"coral1" },
  { XRGB(238,106, 80),	"coral2" },
  { XRGB(205, 91, 69),	"coral3" },
  { XRGB(139, 62, 47),	"coral4" },
  { XRGB(100,149,237),	"cornflowerblue" },
  { XRGB(255,248,220),	"cornsilk" },
  { XRGB(255,248,220),	"cornsilk1" },
  { XRGB(238,232,205),	"cornsilk2" },
  { XRGB(205,200,177),	"cornsilk3" },
  { XRGB(139,136,120),	"cornsilk4" },
  { XRGB(  0,255,255),	"cyan" },
  { XRGB(  0,255,255),	"cyan1" },
  { XRGB(  0,238,238),	"cyan2" },
  { XRGB(  0,205,205),	"cyan3" },
  { XRGB(  0,139,139),	"cyan4" },
  { XRGB(  0,  0,139),	"darkblue" },
  { XRGB(  0,139,139),	"darkcyan" },
  { XRGB(184,134, 11),	"darkgoldenrod" },
  { XRGB(255,185, 15),	"darkgoldenrod1" },
  { XRGB(238,173, 14),	"darkgoldenrod2" },
  { XRGB(205,149, 12),	"darkgoldenrod3" },
  { XRGB(139,101,  8),	"darkgoldenrod4" },
  { XRGB(169,169,169),	"darkgray" },
  { XRGB(  0,100,  0),	"darkgreen" },
  { XRGB(169,169,169),	"darkgrey" },
  { XRGB(189,183,107),	"darkkhaki" },
  { XRGB(139,  0,139),	"darkmagenta" },
  { XRGB( 85,107, 47),	"darkolivegreen" },
  { XRGB(202,255,112),	"darkolivegreen1" },
  { XRGB(188,238,104),	"darkolivegreen2" },
  { XRGB(162,205, 90),	"darkolivegreen3" },
  { XRGB(110,139, 61),	"darkolivegreen4" },
  { XRGB(255,140,  0),	"darkorange" },
  { XRGB(255,127,  0),	"darkorange1" },
  { XRGB(238,118,  0),	"darkorange2" },
  { XRGB(205,102,  0),	"darkorange3" },
  { XRGB(139, 69,  0),	"darkorange4" },
  { XRGB(153, 50,204),	"darkorchid" },
  { XRGB(191, 62,255),	"darkorchid1" },
  { XRGB(178, 58,238),	"darkorchid2" },
  { XRGB(154, 50,205),	"darkorchid3" },
  { XRGB(104, 34,139),	"darkorchid4" },
  { XRGB(139,  0,  0),	"darkred" },
  { XRGB(233,150,122),	"darksalmon" },
  { XRGB(143,188,143),	"darkseagreen" },
  { XRGB(193,255,193),	"darkseagreen1" },
  { XRGB(180,238,180),	"darkseagreen2" },
  { XRGB(155,205,155),	"darkseagreen3" },
  { XRGB(105,139,105),	"darkseagreen4" },
  { XRGB( 72, 61,139),	"darkslateblue" },
  { XRGB( 47, 79, 79),	"darkslategray" },
  { XRGB(151,255,255),	"darkslategray1" },
  { XRGB(141,238,238),	"darkslategray2" },
  { XRGB(121,205,205),	"darkslategray3" },
  { XRGB( 82,139,139),	"darkslategray4" },
  { XRGB( 47, 79, 79),	"darkslategrey" },
  { XRGB(  0,206,209),	"darkturquoise" },
  { XRGB(148,  0,211),	"darkviolet" },
  { XRGB(255, 20,147),	"deeppink" },
  { XRGB(255, 20,147),	"deeppink1" },
  { XRGB(238, 18,137),	"deeppink2" },
  { XRGB(205, 16,118),	"deeppink3" },
  { XRGB(139, 10, 80),	"deeppink4" },
  { XRGB(  0,191,255),	"deepskyblue" },
  { XRGB(  0,191,255),	"deepskyblue1" },
  { XRGB(  0,178,238),	"deepskyblue2" },
  { XRGB(  0,154,205),	"deepskyblue3" },
  { XRGB(  0,104,139),	"deepskyblue4" },
  { XRGB(105,105,105),	"dimgray" },
  { XRGB(105,105,105),	"dimgrey" },
  { XRGB( 30,144,255),	"dodgerblue" },
  { XRGB( 30,144,255),	"dodgerblue1" },
  { XRGB( 28,134,238),	"dodgerblue2" },
  { XRGB( 24,116,205),	"dodgerblue3" },
  { XRGB( 16, 78,139),	"dodgerblue4" },
  { XRGB(178, 34, 34),	"firebrick" },
  { XRGB(255, 48, 48),	"firebrick1" },
  { XRGB(238, 44, 44),	"firebrick2" },
  { XRGB(205, 38, 38),	"firebrick3" },
  { XRGB(139, 26, 26),	"firebrick4" },
  { XRGB(255,250,240),	"floralwhite" },
  { XRGB( 34,139, 34),	"forestgreen" },
  { XRGB(220,220,220),	"gainsboro" },
  { XRGB(248,248,255),	"ghostwhite" },
  { XRGB(255,215,  0),	"gold" },
  { XRGB(255,215,  0),	"gold1" },
  { XRGB(238,201,  0),	"gold2" },
  { XRGB(205,173,  0),	"gold3" },
  { XRGB(139,117,  0),	"gold4" },
  { XRGB(218,165, 32),	"goldenrod" },
  { XRGB(255,193, 37),	"goldenrod1" },
  { XRGB(238,180, 34),	"goldenrod2" },
  { XRGB(205,155, 29),	"goldenrod3" },
  { XRGB(139,105, 20),	"goldenrod4" },
  { XRGB(190,190,190),	"gray" },
  { XRGB(  0,  0,  0),	"gray0" },
  { XRGB(  3,  3,  3),	"gray1" },
  { XRGB( 26, 26, 26),	"gray10" },
  { XRGB(255,255,255),	"gray100" },
  { XRGB( 28, 28, 28),	"gray11" },
  { XRGB( 31, 31, 31),	"gray12" },
  { XRGB( 33, 33, 33),	"gray13" },
  { XRGB( 36, 36, 36),	"gray14" },
  { XRGB( 38, 38, 38),	"gray15" },
  { XRGB( 41, 41, 41),	"gray16" },
  { XRGB( 43, 43, 43),	"gray17" },
  { XRGB( 46, 46, 46),	"gray18" },
  { XRGB( 48, 48, 48),	"gray19" },
  { XRGB(  5,  5,  5),	"gray2" },
  { XRGB( 51, 51, 51),	"gray20" },
  { XRGB( 54, 54, 54),	"gray21" },
  { XRGB( 56, 56, 56),	"gray22" },
  { XRGB( 59, 59, 59),	"gray23" },
  { XRGB( 61, 61, 61),	"gray24" },
  { XRGB( 64, 64, 64),	"gray25" },
  { XRGB( 66, 66, 66),	"gray26" },
  { XRGB( 69, 69, 69),	"gray27" },
  { XRGB( 71, 71, 71),	"gray28" },
  { XRGB( 74, 74, 74),	"gray29" },
  { XRGB(  8,  8,  8),	"gray3" },
  { XRGB( 77, 77, 77),	"gray30" },
  { XRGB( 79, 79, 79),	"gray31" },
  { XRGB( 82, 82, 82),	"gray32" },
  { XRGB( 84, 84, 84),	"gray33" },
  { XRGB( 87, 87, 87),	"gray34" },
  { XRGB( 89, 89, 89),	"gray35" },
  { XRGB( 92, 92, 92),	"gray36" },
  { XRGB( 94, 94, 94),	"gray37" },
  { XRGB( 97, 97, 97),	"gray38" },
  { XRGB( 99, 99, 99),	"gray39" },
  { XRGB( 10, 10, 10),	"gray4" },
  { XRGB(102,102,102),	"gray40" },
  { XRGB(105,105,105),	"gray41" },
  { XRGB(107,107,107),	"gray42" },
  { XRGB(110,110,110),	"gray43" },
  { XRGB(112,112,112),	"gray44" },
  { XRGB(115,115,115),	"gray45" },
  { XRGB(117,117,117),	"gray46" },
  { XRGB(120,120,120),	"gray47" },
  { XRGB(122,122,122),	"gray48" },
  { XRGB(125,125,125),	"gray49" },
  { XRGB( 13, 13, 13),	"gray5" },
  { XRGB(127,127,127),	"gray50" },
  { XRGB(130,130,130),	"gray51" },
  { XRGB(133,133,133),	"gray52" },
  { XRGB(135,135,135),	"gray53" },
  { XRGB(138,138,138),	"gray54" },
  { XRGB(140,140,140),	"gray55" },
  { XRGB(143,143,143),	"gray56" },
  { XRGB(145,145,145),	"gray57" },
  { XRGB(148,148,148),	"gray58" },
  { XRGB(150,150,150),	"gray59" },
  { XRGB( 15, 15, 15),	"gray6" },
  { XRGB(153,153,153),	"gray60" },
  { XRGB(156,156,156),	"gray61" },
  { XRGB(158,158,158),	"gray62" },
  { XRGB(161,161,161),	"gray63" },
  { XRGB(163,163,163),	"gray64" },
  { XRGB(166,166,166),	"gray65" },
  { XRGB(168,168,168),	"gray66" },
  { XRGB(171,171,171),	"gray67" },
  { XRGB(173,173,173),	"gray68" },
  { XRGB(176,176,176),	"gray69" },
  { XRGB( 18, 18, 18),	"gray7" },
  { XRGB(179,179,179),	"gray70" },
  { XRGB(181,181,181),	"gray71" },
  { XRGB(184,184,184),	"gray72" },
  { XRGB(186,186,186),	"gray73" },
  { XRGB(189,189,189),	"gray74" },
  { XRGB(191,191,191),	"gray75" },
  { XRGB(194,194,194),	"gray76" },
  { XRGB(196,196,196),	"gray77" },
  { XRGB(199,199,199),	"gray78" },
  { XRGB(201,201,201),	"gray79" },
  { XRGB( 20, 20, 20),	"gray8" },
  { XRGB(204,204,204),	"gray80" },
  { XRGB(207,207,207),	"gray81" },
  { XRGB(209,209,209),	"gray82" },
  { XRGB(212,212,212),	"gray83" },
  { XRGB(214,214,214),	"gray84" },
  { XRGB(217,217,217),	"gray85" },
  { XRGB(219,219,219),	"gray86" },
  { XRGB(222,222,222),	"gray87" },
  { XRGB(224,224,224),	"gray88" },
  { XRGB(227,227,227),	"gray89" },
  { XRGB( 23, 23, 23),	"gray9" },
  { XRGB(229,229,229),	"gray90" },
  { XRGB(232,232,232),	"gray91" },
  { XRGB(235,235,235),	"gray92" },
  { XRGB(237,237,237),	"gray93" },
  { XRGB(240,240,240),	"gray94" },
  { XRGB(242,242,242),	"gray95" },
  { XRGB(245,245,245),	"gray96" },
  { XRGB(247,247,247),	"gray97" },
  { XRGB(250,250,250),	"gray98" },
  { XRGB(252,252,252),	"gray99" },
  { XRGB(  0,255,  0),	"green" },
  { XRGB(  0,255,  0),	"green1" },
  { XRGB(  0,238,  0),	"green2" },
  { XRGB(  0,205,  0),	"green3" },
  { XRGB(  0,139,  0),	"green4" },
  { XRGB(173,255, 47),	"greenyellow" },
  { XRGB(190,190,190),	"grey" },
  { XRGB(  0,  0,  0),	"grey0" },
  { XRGB(  3,  3,  3),	"grey1" },
  { XRGB( 26, 26, 26),	"grey10" },
  { XRGB(255,255,255),	"grey100" },
  { XRGB( 28, 28, 28),	"grey11" },
  { XRGB( 31, 31, 31),	"grey12" },
  { XRGB( 33, 33, 33),	"grey13" },
  { XRGB( 36, 36, 36),	"grey14" },
  { XRGB( 38, 38, 38),	"grey15" },
  { XRGB( 41, 41, 41),	"grey16" },
  { XRGB( 43, 43, 43),	"grey17" },
  { XRGB( 46, 46, 46),	"grey18" },
  { XRGB( 48, 48, 48),	"grey19" },
  { XRGB(  5,  5,  5),	"grey2" },
  { XRGB( 51, 51, 51),	"grey20" },
  { XRGB( 54, 54, 54),	"grey21" },
  { XRGB( 56, 56, 56),	"grey22" },
  { XRGB( 59, 59, 59),	"grey23" },
  { XRGB( 61, 61, 61),	"grey24" },
  { XRGB( 64, 64, 64),	"grey25" },
  { XRGB( 66, 66, 66),	"grey26" },
  { XRGB( 69, 69, 69),	"grey27" },
  { XRGB( 71, 71, 71),	"grey28" },
  { XRGB( 74, 74, 74),	"grey29" },
  { XRGB(  8,  8,  8),	"grey3" },
  { XRGB( 77, 77, 77),	"grey30" },
  { XRGB( 79, 79, 79),	"grey31" },
  { XRGB( 82, 82, 82),	"grey32" },
  { XRGB( 84, 84, 84),	"grey33" },
  { XRGB( 87, 87, 87),	"grey34" },
  { XRGB( 89, 89, 89),	"grey35" },
  { XRGB( 92, 92, 92),	"grey36" },
  { XRGB( 94, 94, 94),	"grey37" },
  { XRGB( 97, 97, 97),	"grey38" },
  { XRGB( 99, 99, 99),	"grey39" },
  { XRGB( 10, 10, 10),	"grey4" },
  { XRGB(102,102,102),	"grey40" },
  { XRGB(105,105,105),	"grey41" },
  { XRGB(107,107,107),	"grey42" },
  { XRGB(110,110,110),	"grey43" },
  { XRGB(112,112,112),	"grey44" },
  { XRGB(115,115,115),	"grey45" },
  { XRGB(117,117,117),	"grey46" },
  { XRGB(120,120,120),	"grey47" },
  { XRGB(122,122,122),	"grey48" },
  { XRGB(125,125,125),	"grey49" },
  { XRGB( 13, 13, 13),	"grey5" },
  { XRGB(127,127,127),	"grey50" },
  { XRGB(130,130,130),	"grey51" },
  { XRGB(133,133,133),	"grey52" },
  { XRGB(135,135,135),	"grey53" },
  { XRGB(138,138,138),	"grey54" },
  { XRGB(140,140,140),	"grey55" },
  { XRGB(143,143,143),	"grey56" },
  { XRGB(145,145,145),	"grey57" },
  { XRGB(148,148,148),	"grey58" },
  { XRGB(150,150,150),	"grey59" },
  { XRGB( 15, 15, 15),	"grey6" },
  { XRGB(153,153,153),	"grey60" },
  { XRGB(156,156,156),	"grey61" },
  { XRGB(158,158,158),	"grey62" },
  { XRGB(161,161,161),	"grey63" },
  { XRGB(163,163,163),	"grey64" },
  { XRGB(166,166,166),	"grey65" },
  { XRGB(168,168,168),	"grey66" },
  { XRGB(171,171,171),	"grey67" },
  { XRGB(173,173,173),	"grey68" },
  { XRGB(176,176,176),	"grey69" },
  { XRGB( 18, 18, 18),	"grey7" },
  { XRGB(179,179,179),	"grey70" },
  { XRGB(181,181,181),	"grey71" },
  { XRGB(184,184,184),	"grey72" },
  { XRGB(186,186,186),	"grey73" },
  { XRGB(189,189,189),	"grey74" },
  { XRGB(191,191,191),	"grey75" },
  { XRGB(194,194,194),	"grey76" },
  { XRGB(196,196,196),	"grey77" },
  { XRGB(199,199,199),	"grey78" },
  { XRGB(201,201,201),	"grey79" },
  { XRGB( 20, 20, 20),	"grey8" },
  { XRGB(204,204,204),	"grey80" },
  { XRGB(207,207,207),	"grey81" },
  { XRGB(209,209,209),	"grey82" },
  { XRGB(212,212,212),	"grey83" },
  { XRGB(214,214,214),	"grey84" },
  { XRGB(217,217,217),	"grey85" },
  { XRGB(219,219,219),	"grey86" },
  { XRGB(222,222,222),	"grey87" },
  { XRGB(224,224,224),	"grey88" },
  { XRGB(227,227,227),	"grey89" },
  { XRGB( 23, 23, 23),	"grey9" },
  { XRGB(229,229,229),	"grey90" },
  { XRGB(232,232,232),	"grey91" },
  { XRGB(235,235,235),	"grey92" },
  { XRGB(237,237,237),	"grey93" },
  { XRGB(240,240,240),	"grey94" },
  { XRGB(242,242,242),	"grey95" },
  { XRGB(245,245,245),	"grey96" },
  { XRGB(247,247,247),	"grey97" },
  { XRGB(250,250,250),	"grey98" },
  { XRGB(252,252,252),	"grey99" },
  { XRGB(240,255,240),	"honeydew" },
  { XRGB(240,255,240),	"honeydew1" },
  { XRGB(224,238,224),	"honeydew2" },
  { XRGB(193,205,193),	"honeydew3" },
  { XRGB(131,139,131),	"honeydew4" },
  { XRGB(255,105,180),	"hotpink" },
  { XRGB(255,110,180),	"hotpink1" },
  { XRGB(238,106,167),	"hotpink2" },
  { XRGB(205, 96,144),	"hotpink3" },
  { XRGB(139, 58, 98),	"hotpink4" },
  { XRGB(205, 92, 92),	"indianred" },
  { XRGB(255,106,106),	"indianred1" },
  { XRGB(238, 99, 99),	"indianred2" },
  { XRGB(205, 85, 85),	"indianred3" },
  { XRGB(139, 58, 58),	"indianred4" },
  { XRGB(255,255,240),	"ivory" },
  { XRGB(255,255,240),	"ivory1" },
  { XRGB(238,238,224),	"ivory2" },
  { XRGB(205,205,193),	"ivory3" },
  { XRGB(139,139,131),	"ivory4" },
  { XRGB(240,230,140),	"khaki" },
  { XRGB(255,246,143),	"khaki1" },
  { XRGB(238,230,133),	"khaki2" },
  { XRGB(205,198,115),	"khaki3" },
  { XRGB(139,134, 78),	"khaki4" },
  { XRGB(230,230,250),	"lavender" },
  { XRGB(255,240,245),	"lavenderblush" },
  { XRGB(255,240,245),	"lavenderblush1" },
  { XRGB(238,224,229),	"lavenderblush2" },
  { XRGB(205,193,197),	"lavenderblush3" },
  { XRGB(139,131,134),	"lavenderblush4" },
  { XRGB(124,252,  0),	"lawngreen" },
  { XRGB(255,250,205),	"lemonchiffon" },
  { XRGB(255,250,205),	"lemonchiffon1" },
  { XRGB(238,233,191),	"lemonchiffon2" },
  { XRGB(205,201,165),	"lemonchiffon3" },
  { XRGB(139,137,112),	"lemonchiffon4" },
  { XRGB(173,216,230),	"lightblue" },
  { XRGB(191,239,255),	"lightblue1" },
  { XRGB(178,223,238),	"lightblue2" },
  { XRGB(154,192,205),	"lightblue3" },
  { XRGB(104,131,139),	"lightblue4" },
  { XRGB(240,128,128),	"lightcoral" },
  { XRGB(224,255,255),	"lightcyan" },
  { XRGB(224,255,255),	"lightcyan1" },
  { XRGB(209,238,238),	"lightcyan2" },
  { XRGB(180,205,205),	"lightcyan3" },
  { XRGB(122,139,139),	"lightcyan4" },
  { XRGB(238,221,130),	"lightgoldenrod" },
  { XRGB(255,236,139),	"lightgoldenrod1" },
  { XRGB(238,220,130),	"lightgoldenrod2" },
  { XRGB(205,190,112),	"lightgoldenrod3" },
  { XRGB(139,129, 76),	"lightgoldenrod4" },
  { XRGB(250,250,210),	"lightgoldenrodyellow" },
  { XRGB(211,211,211),	"lightgray" },
  { XRGB(144,238,144),	"lightgreen" },
  { XRGB(211,211,211),	"lightgrey" },
  { XRGB(255,182,193),	"lightpink" },
  { XRGB(255,174,185),	"lightpink1" },
  { XRGB(238,162,173),	"lightpink2" },
  { XRGB(205,140,149),	"lightpink3" },
  { XRGB(139, 95,101),	"lightpink4" },
  { XRGB(255,160,122),	"lightsalmon" },
  { XRGB(255,160,122),	"lightsalmon1" },
  { XRGB(238,149,114),	"lightsalmon2" },
  { XRGB(205,129, 98),	"lightsalmon3" },
  { XRGB(139, 87, 66),	"lightsalmon4" },
  { XRGB( 32,178,170),	"lightseagreen" },
  { XRGB(135,206,250),	"lightskyblue" },
  { XRGB(176,226,255),	"lightskyblue1" },
  { XRGB(164,211,238),	"lightskyblue2" },
  { XRGB(141,182,205),	"lightskyblue3" },
  { XRGB( 96,123,139),	"lightskyblue4" },
  { XRGB(132,112,255),	"lightslateblue" },
  { XRGB(119,136,153),	"lightslategray" },
  { XRGB(119,136,153),	"lightslategrey" },
  { XRGB(176,196,222),	"lightsteelblue" },
  { XRGB(202,225,255),	"lightsteelblue1" },
  { XRGB(188,210,238),	"lightsteelblue2" },
  { XRGB(162,181,205),	"lightsteelblue3" },
  { XRGB(110,123,139),	"lightsteelblue4" },
  { XRGB(255,255,224),	"lightyellow" },
  { XRGB(255,255,224),	"lightyellow1" },
  { XRGB(238,238,209),	"lightyellow2" },
  { XRGB(205,205,180),	"lightyellow3" },
  { XRGB(139,139,122),	"lightyellow4" },
  { XRGB( 50,205, 50),	"limegreen" },
  { XRGB(250,240,230),	"linen" },
  { XRGB(255,  0,255),	"magenta" },
  { XRGB(255,  0,255),	"magenta1" },
  { XRGB(238,  0,238),	"magenta2" },
  { XRGB(205,  0,205),	"magenta3" },
  { XRGB(139,  0,139),	"magenta4" },
  { XRGB(176, 48, 96),	"maroon" },
  { XRGB(255, 52,179),	"maroon1" },
  { XRGB(238, 48,167),	"maroon2" },
  { XRGB(205, 41,144),	"maroon3" },
  { XRGB(139, 28, 98),	"maroon4" },
  { XRGB(102,205,170),	"mediumaquamarine" },
  { XRGB(  0,  0,205),	"mediumblue" },
  { XRGB(186, 85,211),	"mediumorchid" },
  { XRGB(224,102,255),	"mediumorchid1" },
  { XRGB(209, 95,238),	"mediumorchid2" },
  { XRGB(180, 82,205),	"mediumorchid3" },
  { XRGB(122, 55,139),	"mediumorchid4" },
  { XRGB(147,112,219),	"mediumpurple" },
  { XRGB(171,130,255),	"mediumpurple1" },
  { XRGB(159,121,238),	"mediumpurple2" },
  { XRGB(137,104,205),	"mediumpurple3" },
  { XRGB( 93, 71,139),	"mediumpurple4" },
  { XRGB( 60,179,113),	"mediumseagreen" },
  { XRGB(123,104,238),	"mediumslateblue" },
  { XRGB(  0,250,154),	"mediumspringgreen" },
  { XRGB( 72,209,204),	"mediumturquoise" },
  { XRGB(199, 21,133),	"mediumvioletred" },
  { XRGB( 25, 25,112),	"midnightblue" },
  { XRGB(245,255,250),	"mintcream" },
  { XRGB(255,228,225),	"mistyrose" },
  { XRGB(255,228,225),	"mistyrose1" },
  { XRGB(238,213,210),	"mistyrose2" },
  { XRGB(205,183,181),	"mistyrose3" },
  { XRGB(139,125,123),	"mistyrose4" },
  { XRGB(255,228,181),	"moccasin" },
  { XRGB(255,222,173),	"navajowhite" },
  { XRGB(255,222,173),	"navajowhite1" },
  { XRGB(238,207,161),	"navajowhite2" },
  { XRGB(205,179,139),	"navajowhite3" },
  { XRGB(139,121, 94),	"navajowhite4" },
  { XRGB(  0,  0,128),	"navy" },
  { XRGB(  0,  0,128),	"navyblue" },
  { XRGB(253,245,230),	"oldlace" },
  { XRGB(107,142, 35),	"olivedrab" },
  { XRGB(192,255, 62),	"olivedrab1" },
  { XRGB(179,238, 58),	"olivedrab2" },
  { XRGB(154,205, 50),	"olivedrab3" },
  { XRGB(105,139, 34),	"olivedrab4" },
  { XRGB(255,165,  0),	"orange" },
  { XRGB(255,165,  0),	"orange1" },
  { XRGB(238,154,  0),	"orange2" },
  { XRGB(205,133,  0),	"orange3" },
  { XRGB(139, 90,  0),	"orange4" },
  { XRGB(255, 69,  0),	"orangered" },
  { XRGB(255, 69,  0),	"orangered1" },
  { XRGB(238, 64,  0),	"orangered2" },
  { XRGB(205, 55,  0),	"orangered3" },
  { XRGB(139, 37,  0),	"orangered4" },
  { XRGB(218,112,214),	"orchid" },
  { XRGB(255,131,250),	"orchid1" },
  { XRGB(238,122,233),	"orchid2" },
  { XRGB(205,105,201),	"orchid3" },
  { XRGB(139, 71,137),	"orchid4" },
  { XRGB(238,232,170),	"palegoldenrod" },
  { XRGB(152,251,152),	"palegreen" },
  { XRGB(154,255,154),	"palegreen1" },
  { XRGB(144,238,144),	"palegreen2" },
  { XRGB(124,205,124),	"palegreen3" },
  { XRGB( 84,139, 84),	"palegreen4" },
  { XRGB(175,238,238),	"paleturquoise" },
  { XRGB(187,255,255),	"paleturquoise1" },
  { XRGB(174,238,238),	"paleturquoise2" },
  { XRGB(150,205,205),	"paleturquoise3" },
  { XRGB(102,139,139),	"paleturquoise4" },
  { XRGB(219,112,147),	"palevioletred" },
  { XRGB(255,130,171),	"palevioletred1" },
  { XRGB(238,121,159),	"palevioletred2" },
  { XRGB(205,104,137),	"palevioletred3" },
  { XRGB(139, 71, 93),	"palevioletred4" },
  { XRGB(255,239,213),	"papayawhip" },
  { XRGB(255,218,185),	"peachpuff" },
  { XRGB(255,218,185),	"peachpuff1" },
  { XRGB(238,203,173),	"peachpuff2" },
  { XRGB(205,175,149),	"peachpuff3" },
  { XRGB(139,119,101),	"peachpuff4" },
  { XRGB(205,133, 63),	"peru" },
  { XRGB(255,192,203),	"pink" },
  { XRGB(255,181,197),	"pink1" },
  { XRGB(238,169,184),	"pink2" },
  { XRGB(205,145,158),	"pink3" },
  { XRGB(139, 99,108),	"pink4" },
  { XRGB(221,160,221),	"plum" },
  { XRGB(255,187,255),	"plum1" },
  { XRGB(238,174,238),	"plum2" },
  { XRGB(205,150,205),	"plum3" },
  { XRGB(139,102,139),	"plum4" },
  { XRGB(176,224,230),	"powderblue" },
  { XRGB(160, 32,240),	"purple" },
  { XRGB(155, 48,255),	"purple1" },
  { XRGB(145, 44,238),	"purple2" },
  { XRGB(125, 38,205),	"purple3" },
  { XRGB( 85, 26,139),	"purple4" },
  { XRGB(255,  0,  0),	"red" },
  { XRGB(255,  0,  0),	"red1" },
  { XRGB(238,  0,  0),	"red2" },
  { XRGB(205,  0,  0),	"red3" },
  { XRGB(139,  0,  0),	"red4" },
  { XRGB(188,143,143),	"rosybrown" },
  { XRGB(255,193,193),	"rosybrown1" },
  { XRGB(238,180,180),	"rosybrown2" },
  { XRGB(205,155,155),	"rosybrown3" },
  { XRGB(139,105,105),	"rosybrown4" },
  { XRGB( 65,105,225),	"royalblue" },
  { XRGB( 72,118,255),	"royalblue1" },
  { XRGB( 67,110,238),	"royalblue2" },
  { XRGB( 58, 95,205),	"royalblue3" },
  { XRGB( 39, 64,139),	"royalblue4" },
  { XRGB(139, 69, 19),	"saddlebrown" },
  { XRGB(250,128,114),	"salmon" },
  { XRGB(255,140,105),	"salmon1" },
  { XRGB(238,130, 98),	"salmon2" },
  { XRGB(205,112, 84),	"salmon3" },
  { XRGB(139, 76, 57),	"salmon4" },
  { XRGB(244,164, 96),	"sandybrown" },
  { XRGB( 46,139, 87),	"seagreen" },
  { XRGB( 84,255,159),	"seagreen1" },
  { XRGB( 78,238,148),	"seagreen2" },
  { XRGB( 67,205,128),	"seagreen3" },
  { XRGB( 46,139, 87),	"seagreen4" },
  { XRGB(255,245,238),	"seashell" },
  { XRGB(255,245,238),	"seashell1" },
  { XRGB(238,229,222),	"seashell2" },
  { XRGB(205,197,191),	"seashell3" },
  { XRGB(139,134,130),	"seashell4" },
  { XRGB(160, 82, 45),	"sienna" },
  { XRGB(255,130, 71),	"sienna1" },
  { XRGB(238,121, 66),	"sienna2" },
  { XRGB(205,104, 57),	"sienna3" },
  { XRGB(139, 71, 38),	"sienna4" },
  { XRGB(135,206,235),	"skyblue" },
  { XRGB(135,206,255),	"skyblue1" },
  { XRGB(126,192,238),	"skyblue2" },
  { XRGB(108,166,205),	"skyblue3" },
  { XRGB( 74,112,139),	"skyblue4" },
  { XRGB(106, 90,205),	"slateblue" },
  { XRGB(131,111,255),	"slateblue1" },
  { XRGB(122,103,238),	"slateblue2" },
  { XRGB(105, 89,205),	"slateblue3" },
  { XRGB( 71, 60,139),	"slateblue4" },
  { XRGB(112,128,144),	"slategray" },
  { XRGB(198,226,255),	"slategray1" },
  { XRGB(185,211,238),	"slategray2" },
  { XRGB(159,182,205),	"slategray3" },
  { XRGB(108,123,139),	"slategray4" },
  { XRGB(112,128,144),	"slategrey" },
  { XRGB(255,250,250),	"snow" },
  { XRGB(255,250,250),	"snow1" },
  { XRGB(238,233,233),	"snow2" },
  { XRGB(205,201,201),	"snow3" },
  { XRGB(139,137,137),	"snow4" },
  { XRGB(  0,255,127),	"springgreen" },
  { XRGB(  0,255,127),	"springgreen1" },
  { XRGB(  0,238,118),	"springgreen2" },
  { XRGB(  0,205,102),	"springgreen3" },
  { XRGB(  0,139, 69),	"springgreen4" },
  { XRGB( 70,130,180),	"steelblue" },
  { XRGB( 99,184,255),	"steelblue1" },
  { XRGB( 92,172,238),	"steelblue2" },
  { XRGB( 79,148,205),	"steelblue3" },
  { XRGB( 54,100,139),	"steelblue4" },
  { XRGB(210,180,140),	"tan" },
  { XRGB(255,165, 79),	"tan1" },
  { XRGB(238,154, 73),	"tan2" },
  { XRGB(205,133, 63),	"tan3" },
  { XRGB(139, 90, 43),	"tan4" },
  { XRGB(216,191,216),	"thistle" },
  { XRGB(255,225,255),	"thistle1" },
  { XRGB(238,210,238),	"thistle2" },
  { XRGB(205,181,205),	"thistle3" },
  { XRGB(139,123,139),	"thistle4" },
  { XRGB(255, 99, 71),	"tomato" },
  { XRGB(255, 99, 71),	"tomato1" },
  { XRGB(238, 92, 66),	"tomato2" },
  { XRGB(205, 79, 57),	"tomato3" },
  { XRGB(139, 54, 38),	"tomato4" },
  { XRGB( 64,224,208),	"turquoise" },
  { XRGB(  0,245,255),	"turquoise1" },
  { XRGB(  0,229,238),	"turquoise2" },
  { XRGB(  0,197,205),	"turquoise3" },
  { XRGB(  0,134,139),	"turquoise4" },
  { XRGB(238,130,238),	"violet" },
  { XRGB(208, 32,144),	"violetred" },
  { XRGB(255, 62,150),	"violetred1" },
  { XRGB(238, 58,140),	"violetred2" },
  { XRGB(205, 50,120),	"violetred3" },
  { XRGB(139, 34, 82),	"violetred4" },
  { XRGB(245,222,179),	"wheat" },
  { XRGB(255,231,186),	"wheat1" },
  { XRGB(238,216,174),	"wheat2" },
  { XRGB(205,186,150),	"wheat3" },
  { XRGB(139,126,102),	"wheat4" },
  { XRGB(255,255,255),	"white" },
  { XRGB(245,245,245),	"whitesmoke" },
  { XRGB(255,255,  0),	"yellow" },
  { XRGB(255,255,  0),	"yellow1" },
  { XRGB(238,238,  0),	"yellow2" },
  { XRGB(205,205,  0),	"yellow3" },
  { XRGB(139,139,  0),	"yellow4" },
  { XRGB(154,205, 50),	"yellowgreen" }
  };

#define 	MAX_COLORS	(sizeof(aColors)/sizeof(aColors[0]))

//*****************************************************************************
//*
//*		StrToVal
//*
//*****************************************************************************
//	Converts a hex dump to a value
//	pString		: string with the hex dump
//	pBuffer		: pointer to the data buffer
//	iSize		; size of the data buffer
//	returns the count of stored bytes in the buffer
static unsigned StrToVal(const char *pString)
{
unsigned int	ulValue=0;
unsigned char	ucByte;
char			cChar;




	for(;;)
		{
		   cChar = *pString;
		if(cChar==0)break;


		if(cChar< '0')break;
		if(cChar<='9'){ucByte=cChar-'0';}
		else{
			if(cChar< 'A')break;
			if(cChar<='F'){ucByte=cChar-'A'+10;}
			else{
				if(cChar<'a')break;
				if(cChar>'f')break;
				ucByte=cChar-'a'+10;
				}
			}

		ulValue <<= 4;
		ulValue  += ucByte;

		pString++;
		}


return ulValue;
}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
//
// Class Constructor
//
// Nothing gets done here except initializing variables to a known state.
//
XpmDecoder::XpmDecoder()
{

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
//
// Class Destructor
//
XpmDecoder::~XpmDecoder()
{

}

//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
//
//  Description:
//
//    This function reads a PNG image.
//
//  Parameters:
//	  Name:	  Filename
//    image:  The output image
//
void XpmDecoder::ReadImage(const char *pName, BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName))throw EXpmError("Fehler beim ffnen der Datei");

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void XpmDecoder::ReadImage(const wchar_t *pName, BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName))throw EXpmError("Fehler beim ffnen der Datei");

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void XpmDecoder::ReadImage(CFileBuffer *pFile, BitmapImage &sImage)
{
char   *pPtr;
char	cBuffer[MAX_BUFFER];
int		iValues[4],iPos,i;



	do	{												// Leezeilen berspringen
		if(pFile->ReadLine(cBuffer,sizeof(cBuffer))<0)
			{
			throw EXpmError("Fehler beim Lesen aus der Datei");
			}

		pPtr  = cBuffer;
		pPtr += strspn(pPtr,"\r\n\t ");
		}while(pPtr[0]==0 || pPtr[0]=='#');


	if(pPtr[0]=='/' || pPtr[1]=='*')					// Existiert ein /* XPM */ Tag
		{
 		pPtr += 2;
		pPtr += strspn(pPtr,"\r\n\t ");

		if(pPtr[0]!='X' || pPtr[1]!='P' || pPtr[2]!='M')
			{
			throw EXpmError("Unbekantes Format");
			}

 		pPtr += 3;
		pPtr += strspn(pPtr,"\r\n\t ");

		if(pPtr[0]!='*' || pPtr[1]!='/')
			{
			throw EXpmError("Unbekantes Format");
			}

		pPtr += 2;

		for(;;)											// Kommentare berspringen
			{
			if(pPtr[0]==0)
				{
				if(ReadLine(pFile,cBuffer,sizeof(cBuffer))<0)
					{
					throw EXpmError("Unbekantes Format");
					}

				pPtr=cBuffer;
				}

			pPtr += strspn(pPtr,"\r\n\t ");

			if(pPtr[0]==0                  )continue;
			if(pPtr[0]=='/' && pPtr[1]=='/')continue;
			if(pPtr[0]!='/' || pPtr[1]!='*')break;

			pPtr += 2;

			for(;;)										//   "/*....*/" berspringen
				{
				   pPtr += strcspn(pPtr,"*");
				if(pPtr[0]== 0 )
					{
					if(ReadLine(pFile,cBuffer,sizeof(cBuffer))<0)break;
					pPtr=cBuffer;
					}

				if(pPtr[1]!='/'){pPtr++;continue;}

				pPtr += 2;
				pPtr += strspn(pPtr,"\r\n\t ");
				break;
				}
			}
		}

	iPos = 0;

	if(!strncmp(pPtr+iPos,"static",6))					// Beginn vom Array	  static char *xxxxx[] = {
		{
		iPos = 6;

		while(pPtr[iPos]==' ' || pPtr[iPos]=='\t')
			{
			iPos++;
			}
		}

	if(!strncmp(pPtr+iPos,"const",5))					// Beginn vom Array	  const char *xxxxx[] = {
		{
		iPos += 5;

		while(pPtr[iPos]==' ' || pPtr[iPos]=='\t')
			{
			iPos++;
			}
		}

	if(!strncmp(pPtr+iPos,"char",4))					// Beginn vom Array	  char *xxxxx[] = {
		{
		iPos += 4;

		while(pPtr[iPos]==' ' || pPtr[iPos]=='\t')
			{
			iPos++;
			}
		}

	if(iPos)
		{
		pPtr += iPos;

		while(pPtr[0]!='=')
			{
			   pPtr++;
			if(pPtr[0])continue;

			if(pFile->ReadLine(cBuffer,sizeof(cBuffer))<0)
				{
				throw EXpmError("Unbekantes Format");
				}

			pPtr=cBuffer;
			}

		pPtr += 1;
		pPtr += strspn(pPtr,"\r\n\t ");

		while(pPtr[0]!='{')
			{
			   pPtr+=strspn(pPtr,"\r\n\t ");
			if(pPtr[0])
				{
				throw EXpmError("Unbekantes Format");
				}

			if(pFile->ReadLine(cBuffer,sizeof(cBuffer))<0)
				{
				throw EXpmError("Unbekantes Format");
				}

			pPtr=cBuffer;
			}

		pPtr++;

		while(pPtr[0]!='"')
			{
			   pPtr+=strspn(pPtr,"\r\n\t ");
			if(pPtr[0]=='"')break;
			if(pPtr[0])
				{
				if(pPtr[0]=='/' && pPtr[1]=='/')		// "//...." berspringen
					{
					pPtr += strlen(pPtr);
					continue;
					}
				
				if(pPtr[0]=='/' && pPtr[1]=='*')		// "/*....*/" berspringen
					{
					for(pPtr+=2;;pPtr++)
						{
						if(pPtr[0]==0)
							{
							if(pFile->ReadLine(cBuffer,sizeof(cBuffer))<0)
								{
								throw EXpmError("Unbekantes Format");
								}
							
							pPtr = cBuffer;		
							}

						if(pPtr[0]=='*' && pPtr[1]=='/')
							{
							pPtr += 2;
							break;
							}
						}
					continue;
					}

				throw EXpmError("Unbekantes Format");
				}

			if(pFile->ReadLine(cBuffer,sizeof(cBuffer))<0)
				{
				throw EXpmError("Unbekantes Format");
				}

			pPtr=cBuffer;
			}

		}


	if(*pPtr!='"')throw EXpmError("Unbekantes Format");
	pPtr++;

	for(i=0;i<4;i++)
		{
		pPtr+=strspn(pPtr,"\r\n\t ");
		iValues[i]=atoi(pPtr);
		pPtr+=strcspn(pPtr,"\r\n\t ");
		pPtr+=strspn(pPtr,"\r\n\t ");
		}

	if(iValues[0]<=0 || iValues[0]>8000 || i<4 ||
	   iValues[1]<=0 || iValues[1]>8000)
		{
		throw EXpmError("Unbekantes Format");
		}

	if(iValues[2]<=0 || iValues[2]>MAX_XPM_COLORS)
		{
		throw EXpmError("Zu viele Farben definiert");
		}

	for(m_iColorCnt=0;m_iColorCnt<iValues[2];)
		{
		ReadLine(pFile,cBuffer,sizeof(cBuffer));

		pPtr=cBuffer;
		pPtr+=strspn(pPtr,"\r\n\t ");

		if(pPtr[0]==0                  ){i--;continue;}
		if(pPtr[0]=='/' && pPtr[1]=='*'){i--;continue;}
		if(pPtr[0]=='/' && pPtr[1]=='/'){i--;continue;}

		ReadColor(cBuffer,iValues[3]);
		}

	if(m_iColorCnt!=iValues[2])
		{
		throw EXpmError("Zu wenige Farben definiert");
		}

	if(iValues[2]>256)
		{
		sImage.SetSize(24,iValues[0],iValues[1],BI_NOT_INIT);

		for(i=0;i<iValues[1];i++)
			{
			ReadLine(pFile,cBuffer,sizeof(cBuffer));

			pPtr=cBuffer;
			pPtr+=strspn(pPtr,"\r\n\t ");

			if(pPtr[0]==0                  ){i--;continue;}
			if(pPtr[0]=='/' && pPtr[1]=='*'){i--;continue;}
			if(pPtr[0]=='/' && pPtr[1]=='/'){i--;continue;}

			SetColorLineRgb(sImage,i,cBuffer,iValues[3]);
			CallProgressFunction((99*i)/iValues[1]);
			}
		}
	else{
		sImage.SetSize(8,iValues[0],iValues[1],BI_NOT_INIT);

		for(i=0;i<m_iColorCnt;i++)
			{
			sImage.SetColor(i,m_aColorRgb[i]);
			}

		for(i=0;i<iValues[1];i++)
			{
			ReadLine(pFile,cBuffer,sizeof(cBuffer));

			pPtr=cBuffer;
			pPtr+=strspn(pPtr,"\r\n\t ");

			if(pPtr[0]==0                  ){i--;continue;}
			if(pPtr[0]=='/' && pPtr[1]=='*'){i--;continue;}
			if(pPtr[0]=='/' && pPtr[1]=='/'){i--;continue;}

			SetColorLine8(sImage,i,cBuffer,iValues[3]);
			CallProgressFunction((99*i)/iValues[1]);
			}
		}

	CallProgressFunction(100);
}


//*****************************************************************************
//*
//*		ReadLine
//*
//*****************************************************************************
int	 XpmDecoder::ReadLine(CFileBuffer *pFile,char *pBuffer,int iMax)
{
char *pPtr;
int   iSize;

	for(;;)
		{
		iSize=pFile->ReadLine(pBuffer,iMax);
		if(iSize<0)throw EXpmError("Fehler beim Lesen aus der Datei");
		pPtr=pBuffer+strcspn(pBuffer,"\r\n\t ");
		if(pPtr[0]!='/' || pPtr[1]!='*')break;
		}

return iSize;
}

//*****************************************************************************
//*
//*		ReadColor
//*
//*****************************************************************************
bool XpmDecoder::ReadColor(const char *pPtr,int iCharsPerPixel)
{
char		cType;



		pPtr+=strspn(pPtr,"\r\n\t ");
	if( pPtr[0]== 0 )return true;
	if( pPtr[0]!='"')return false;

	pPtr++;

	m_aColorVal[m_iColorCnt]=*(int*)pPtr;
	m_aColorMask [m_iColorCnt]=~(-1<<(8*iCharsPerPixel));

	for(;iCharsPerPixel>0;iCharsPerPixel--)
		{
		if(*pPtr)pPtr++;
		}

	pPtr+=strspn(pPtr,"\r\n\t ");
	cType=*pPtr;
	pPtr++;
	pPtr+=strspn(pPtr,"\r\n\t ");

	switch(cType)
		{
	case 's':
	case 'c':
	case 'g':

			if(!strnicmp(pPtr,"None",4))
				{
				m_aColorRgb[m_iColorCnt]=-1;
				m_iColorCnt++;
				break;
				}

			if(pPtr[0]!='#')
				{
				int iLen,iPos;

				   iLen=(int)strcspn(pPtr," \t\n\r\",;");
				if(iLen<=0)
					{
					throw EXpmError("Nicht untersttztes Farbformat");
					}

				for(iPos=0;iPos<MAX_COLORS;iPos++)
					{
					if(strnicmp(pPtr,aColors[iPos].pName,iLen))continue;
					if(aColors[iPos].pName[iLen]==0)break;
					}

				if(iPos>=MAX_COLORS)
					{
					throw EXpmError("Nicht untersttztes Farbformat");
					}

				m_aColorRgb[m_iColorCnt]=aColors[iPos].uColor;
				m_iColorCnt++;
				pPtr+=iLen;

				break;
				}
			pPtr++;
			m_aColorRgb[m_iColorCnt]=StrToVal(pPtr);
			m_iColorCnt++;
			break;


	default:throw EXpmError("Nicht untersttztes Farbformat");
		}


return true;
}

//*****************************************************************************
//*
//*		SetColorLineRgb
//*
//*****************************************************************************
void XpmDecoder::SetColorLineRgb(BitmapImage &sImage,int iLine,const char *pPtr,int iCharsPerPixel)
{
int		i,iXpos,iAdd;


		pPtr+=strspn(pPtr,"\r\n\t ");
	if( pPtr[0]== 0 || pPtr[0]!='"')throw EXpmError("Fehler in der Datei");

	pPtr++;

	for(iXpos=0;iXpos<(int)sImage.Width();iXpos++)
		{
		for(i=0;i<m_iColorCnt;i++)
			{
			if(!((*(int*)pPtr^m_aColorVal[i])&m_aColorMask[i]))break;
			}

		for(iAdd=0;iAdd<iCharsPerPixel;iAdd++)
			{
			if(*pPtr)pPtr++;
			}

		if(i>=m_iColorCnt)break;
		if(m_aColorRgb[i]==-1)continue;

		sImage.SetPixel(iXpos,iLine,m_aColorRgb[i]);
		}

}

//*****************************************************************************
//*
//*		SetColorLine8
//*
//*****************************************************************************
void XpmDecoder::SetColorLine8(BitmapImage &sImage,int iLine,const char *pPtr,int iCharsPerPixel)
{
int		i,iXpos,iAdd;


		pPtr+=strspn(pPtr,"\r\n\t ");
	if( pPtr[0]== 0 || pPtr[0]!='"')throw EXpmError("Fehler in der Datei");

	pPtr++;

	for(iXpos=0;iXpos<(int)sImage.Width();iXpos++)
		{
		for(i=0;i<m_iColorCnt;i++)
			{
			if(!((*(int*)pPtr^m_aColorVal[i])&m_aColorMask[i]))break;
			}

		for(iAdd=0;iAdd<iCharsPerPixel;iAdd++)
			{
			if(*pPtr)pPtr++;
			}

		if(i>=m_iColorCnt)break;

		sImage.SetPixel(iXpos,iLine,i);
		}

}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void XpmDecoder::CallProgressFunction(unsigned int iProzent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;

	pProgressProc(*this,pProgressData,1,1,iProzent,bCancel);

	if(bCancel)throw EGraphicsAbort();
}
