//*****************************************************************************
//*
//*
//*		XpmEncoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2011
//
#include "XpmEncoder.h"


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#ifdef		_DEBUG
#define new	 DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif
#endif 	



const char *XpmEncoder::pChars="0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ#_";

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
//
//  Description:
//
//    Class Default Constructor
//
XpmEncoder::XpmEncoder()
{
	cName[0]=0;
}


//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
//
//  Description:
//
//    Class Destructor
//
XpmEncoder::~XpmEncoder()
{

}


//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
//
//  Description:
//
//    This function writes an image to a PNG stream.
//
//  Parameters:
//    Name:   The filename
//    image:  The image to outpu
//
void XpmEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;
int			i,iLen;



	if(!sBuffer.Open(pName,CFB_CREATE))
		{
		throw EXpmError("Fehler beim Erzeugen der Dateio") ;
		}

	try	{
		iLen=(int)strlen(pName);
		for(i=iLen;i>0;i--)
			{
			if(pName[i]=='\\'){i++;break;}
			if(pName[i]=='/' ){i++;break;}
			}

		iLen-=i;
		if(iLen>=sizeof(cName)-1)iLen=sizeof(cName)-1;
		memcpy(cName,pName+i,iLen);
		cName[iLen]=0;

		for(i=0;i<iLen;i++)
			{
			if(cName[i]>='a' && cName[i]<='j'     )continue;
			if(cName[i]>='A' && cName[i]<='Z'     )continue;
			if(cName[i]>='0' && cName[i]<='9' && i)continue;
			if(cName[i]=='_'                      )continue;

			cName[i]='_';
			}

		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void XpmEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;
int			i,iLen;



	if(!sBuffer.Open(pName,CFB_CREATE))
		{
		throw EXpmError("Fehler beim Erzeugen der Dateio") ;
		}

	try	{
		for(iLen=0;pName[iLen];iLen++);

		for(i=iLen;i>0;i--)
			{
			if(pName[i]=='\\'){i++;break;}
			if(pName[i]=='/' ){i++;break;}
			}

		iLen-=i;
		if(iLen>=sizeof(cName)-1)iLen=sizeof(cName)-1;
		memcpy(cName,pName+i,iLen);
		cName[iLen]=0;

		for(i=0;i<iLen;i++)
			{
			if(cName[i]>='a' && cName[i]<='j'     )continue;
			if(cName[i]>='A' && cName[i]<='Z'     )continue;
			if(cName[i]>='0' && cName[i]<='9' && i)continue;
			if(cName[i]=='_'                      )continue;

			cName[i]='_';
			}

		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void XpmEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
unsigned int		uColor[MAX_XPM_COLORS];
unsigned int 		x,y,j,uNr,uRgb,uVal;
	  BitmapImage  *pNewImage=0;
const BitmapImage  *pImage;
char				cText[8];
int					i,iChars;


	pImage=&sImage;
	if(!cName[0])strcpy(cName,"unkown_xpm");

	try	{
		if(sImage.BitCount()>8)						// Zhle die Farben
			{
			uNr=0;

			for(y=0;y<sImage.Height();y++)
			for(x=0;x<sImage.Width() ;x++)
				{
				uRgb=sImage.GetPixel(x,y);
				for(j=0;j<uNr;j++)
					{
					if(uColor[j]==uRgb)break;
					}

				if(j<uNr)continue;

				uColor[uNr]=uRgb;
				uNr++;

				if(uNr>=MAX_XPM_COLORS)break;
				}

			if(uNr>=MAX_XPM_COLORS)
				{
				pNewImage=new BitmapImage;

				if(!pNewImage->ConvertTo8Bit(*pImage))
					{
					throw EXpmError("Kann Format nicht verarbeiten");
					}

				pImage=pNewImage;
				}
			}

		if(pImage->BitCount()<=8)					// 256 Farben
			{
			uNr=pImage->ColorCount();

			for(j=0;j<uNr;j++)
				{
				uColor[j]=pImage->GetColor(j);
				}
			}

		for(iChars=0;uNr>>iChars;iChars++);
		iChars+=5;
		iChars/=6;

		if(pFile->WriteFormat("/* XPM */\nstatic char * %s[] = {\n\"%i %i %i %i\",\n",
			cName,pImage->Width(),pImage->Height(),uNr,iChars)<=0)
			{
			throw EXpmError("Fehler beim Schreiben in die Datei");
			}

		cText[iChars]=0;

		for(j=0;j<uNr;j++)							// Farben speichern
			{
			uVal=j;

			for(i=0;i<iChars;i++)
				{
				cText[i]=pChars[uVal&0x3F];
				uVal>>=6;
				}

			if(pFile->WriteFormat("\"%s\tc #%06X\",\n",cText,uColor[j]&0xFFFFFF)<=0)
				{
				throw EXpmError("Fehler beim Schreiben in die Datei");
				}
			}



		for(y=0;y<pImage->Height();y++)				// Pixel speichern
			{
			pFile->WriteByte('"');

			for(x=0;x<pImage->Width();x++)
				{
				uRgb=pImage->GetRGB(x,y);

				for(j=0;j<uNr;j++)
					{
					if(uColor[j]==uRgb)break;
					}

				if(j>uNr)j=uNr;

				uVal=j;

				for(i=0;i<iChars;i++)
					{
					cText[i]=pChars[uVal&0x3F];
					uVal>>=6;
					}

				pFile->Write(cText,iChars);
				}

			CallProgressFunction((99*y)/pImage->Height());

			if(y<pImage->Height()-1)
					pFile->Write("\",\n" ,3);
			else	pFile->Write("\"};\n",4);
			}
		}
	catch(EGraphicsException e)
		{
		if(pNewImage)delete pNewImage;
		throw e;
		}
	catch(...)
		{
		if(pNewImage)delete pNewImage;
		throw "";
		}


	if(pNewImage)delete pNewImage;

	CallProgressFunction(100);

}


//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void XpmEncoder::CallProgressFunction(unsigned int iProzent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;

	pProgressProc(*this,pProgressData,1,1,iProzent,bCancel);

	if(bCancel)throw EGraphicsAbort();
}
