//*****************************************************************************
//*
//*
//*		XwdDecoder.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2011
//
//	Ld XWD-Bilder.
//

#include "XwdDecoder.h"


static inline unsigned BTOL(unsigned a)	{return ((((a)&0xFF000000)>>24)|(((a)&0x00FF0000)>>8)|(((a)&0x0000FF00)<<8)|(((a)&0x000000FF)<<24));}
static inline unsigned BTOW(unsigned a)	{return	((((a)&0xFF00)>>8)|(((a)&0x00FF)<<8));}


//*****************************************************************************
//*
//*		Konstructor
//*
//*****************************************************************************
XwdDecoder::XwdDecoder()
{
}

//*****************************************************************************
//*
//*		Destructor
//*
//*****************************************************************************
XwdDecoder::~XwdDecoder()
{
}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void XwdDecoder::CallProgressFunction(unsigned uPercent)
{
bool	bCancel;


	if(pProgressProc==NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,uPercent,bCancel);

	if(bCancel)throw EGraphicsAbort();
}

//*****************************************************************************
//*
//*		ReadImage
//*
//*****************************************************************************
void XwdDecoder::ReadImage(const char *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw EXwdFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void XwdDecoder::ReadImage(const wchar_t *pName,BitmapImage &sImage)
{
CFileBuffer		sBuffer;

	if(!sBuffer.Open(pName))
		{
		throw EXwdFileOpenError();
		}

	try	{
		ReadImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();

}

void XwdDecoder::ReadImage(CFileBuffer *pFile,BitmapImage &sImage)
{
unsigned	 	uHeaderSize;
unsigned		uFileVersion;
unsigned		uDisplayType;								// Display type
unsigned		uDisplayPlanes;								// Number of display planes 
unsigned		uPixmapFormat;								// Pixmap format
unsigned		uPixmapDepth;    							// Pixmap depth in pixels 
unsigned		uPixmapWidth;    							// Pixmap width in pixels 
unsigned		uPixmapHeight;   							// Pixmap height in pixels
unsigned		uXOffset;        							// Bitmap X offset
unsigned		uByteOrder;      							// Byte order of image data
unsigned		uBitmapUnit;     							// Bitmap base data size
unsigned		uBitmapBitOrder; 							// Bit-order of image data
unsigned		uBitmapPad;      							// Bitmap scan-line pad
unsigned		uBitsPerPixel;   							// Bits per pixel
unsigned		uBytesPerLine;   							// Bytes per scan-line
unsigned		uVisualClass;    							// Class of the image
unsigned		uRedMask;        							// Red mask
unsigned		uGreenMask;      							// Green mask
unsigned		uBlueMask;       							// Blue mask
unsigned		uBitsPerRgb;     							// Size of each color mask in bits 
unsigned		uNumberOfColors; 							// Number of colors in image 
unsigned		uColorMapEntries;							// Number of entries in color map
unsigned		uWindowWidth;    							// Window width
unsigned		uWindowHeight;   							// Window height
unsigned		uWindowX;        							// Window upper left X coordinate
unsigned		uWindowY;        							// Window upper left Y coordinate
unsigned		uWindowBorder;								// Window border width
unsigned char  	aColor[12];	
unsigned		aSort[4];
unsigned		uPadSize;
unsigned		uDoSort;
unsigned		uLen;
unsigned		uPos;
unsigned		uOff;
unsigned		uRow;
unsigned char  *pRow;	


	
	uDoSort		 = 0;	
	uHeaderSize  = BTOL(pFile->ReadDWord());
	uFileVersion = BTOL(pFile->ReadDWord());

	if((unsigned)pFile->GetLength()<=uHeaderSize)
		{
		throw EXwdFileReadError();
		}

	if(uHeaderSize>0x100 || uFileVersion<6 || uFileVersion>7)
		{
		throw EXwdNotAXwdFile();
		}

	if(uFileVersion==7)
		{
		uPixmapFormat    = BTOL(pFile->ReadDWord()); 
		uPixmapDepth     = BTOL(pFile->ReadDWord()); 
		uPixmapWidth     = BTOL(pFile->ReadDWord()); 
		uPixmapHeight    = BTOL(pFile->ReadDWord()); 
		uXOffset         = BTOL(pFile->ReadDWord()); 
		uByteOrder       = BTOL(pFile->ReadDWord()); 
		uBitmapUnit      = BTOL(pFile->ReadDWord()); 
		uBitmapBitOrder  = BTOL(pFile->ReadDWord()); 
		uBitmapPad       = BTOL(pFile->ReadDWord()); 
		uBitsPerPixel    = BTOL(pFile->ReadDWord()); 
		uBytesPerLine    = BTOL(pFile->ReadDWord()); 
		uVisualClass     = BTOL(pFile->ReadDWord()); 
		uRedMask         = BTOL(pFile->ReadDWord()); 
		uGreenMask       = BTOL(pFile->ReadDWord()); 
		uBlueMask        = BTOL(pFile->ReadDWord()); 
		uBitsPerRgb      = BTOL(pFile->ReadDWord()); 
		uNumberOfColors  = BTOL(pFile->ReadDWord()); 
		uColorMapEntries = BTOL(pFile->ReadDWord()); 
		uWindowWidth     = BTOL(pFile->ReadDWord()); 
		uWindowHeight    = BTOL(pFile->ReadDWord()); 
		uWindowX         = BTOL(pFile->ReadDWord()); 
		uWindowY         = BTOL(pFile->ReadDWord()); 
		uWindowBorder    = BTOL(pFile->ReadDWord());

		if(uPixmapWidth==0 || uPixmapHeight==0 || uBitsPerPixel>32 || uPixmapDepth>32 || uPixmapWidth>0x7FFF || uPixmapHeight>0x7FFF)
			{
			throw EXwdNotAXwdFile();
			}

		if(uBytesPerLine/uPixmapWidth*8>uBitsPerPixel)
		if(uBytesPerLine%uPixmapWidth==0)
			{
			uBitsPerPixel = uBytesPerLine/uPixmapWidth*8;
			}

															// Sind die Bilddaten im BigEndian-Format
		if(uByteOrder)
			{
			uRedMask   = BTOL(uRedMask  ); 
			uBlueMask  = BTOL(uBlueMask ); 
			uGreenMask = BTOL(uGreenMask); 
			}

		pFile->Seek(uHeaderSize);

		sImage.SetSize(uBitsPerPixel,uPixmapWidth,uPixmapHeight,BI_NOT_INIT);

		if(uBitsPerPixel<=8)
			{
			for(uPos=0;uPos<uColorMapEntries;uPos++)
				{
				if(pFile->Read(aColor,12)!=12)
					{
					throw EXwdFileReadError();
					}

				sImage.SetColor(aColor[3],aColor[5],aColor[7],aColor[9]);
				}
			}
		else if(uColorMapEntries)
			{
			pFile->Seek(uColorMapEntries*12,SEEK_CUR);
			}
		
															// Muss etwas umsortiert werden
		if(uBitsPerPixel>=24 && (uRedMask!=0xFF0000 || uGreenMask!=0x00FF00 || uBlueMask!=0x0000FF))
			{
			uDoSort  = uBitsPerPixel/8;
			aSort[0] = 0;
			aSort[1] = 1;
			aSort[2] = 2;

			if(uRedMask  ==0xFF000000)aSort[2]=3;
			if(uRedMask  ==0x0000FF00)aSort[2]=1;
			if(uRedMask  ==0x000000FF)aSort[2]=0;
			if(uBlueMask ==0xFF000000)aSort[0]=3;
			if(uBlueMask ==0x0000FF00)aSort[0]=1;
			if(uBlueMask ==0x00FF0000)aSort[0]=2;
			if(uGreenMask==0xFF000000)aSort[1]=3;
			if(uGreenMask==0x00FF0000)aSort[1]=2;
			if(uGreenMask==0x000000FF)aSort[1]=0;
			}
		}
	else{
		uDisplayType     = BTOL(pFile->ReadDWord()); 
		uDisplayPlanes   = BTOL(pFile->ReadDWord()); 
		uPixmapFormat    = BTOL(pFile->ReadDWord()); 
		uPixmapWidth     = BTOL(pFile->ReadDWord()); 
		uPixmapHeight    = BTOL(pFile->ReadDWord()); 
		uWindowWidth     = BTOW(pFile->ReadWord()); 
		uWindowHeight    = BTOW(pFile->ReadWord()); 
		uWindowX         = BTOW(pFile->ReadWord()); 
		uWindowY         = BTOW(pFile->ReadWord()); 
		uWindowBorder    = BTOW(pFile->ReadWord()); 
		uColorMapEntries = BTOW(pFile->ReadWord()); 	

		if(uPixmapWidth==0 || uPixmapHeight==0)
			{
			throw EXwdNotAXwdFile();
			}

		if(uColorMapEntries<=2)
			{
			uBitsPerPixel = 1;
			}
		else if(uColorMapEntries<=16)
			{
			uBitsPerPixel = 4;
			}
		else{
			uBitsPerPixel = 8;
			}

		uBytesPerLine = (uPixmapWidth*uBitsPerPixel+7)/8;
			
		sImage.SetSize(uBitsPerPixel,uPixmapWidth,uPixmapHeight,BI_NOT_INIT);

		if(uBitsPerPixel<=8)
			{
			for(uPos=0;uPos<uColorMapEntries;uPos++)
				{
				if(pFile->Read(aColor,8)!=8)
					{
					throw EXwdFileReadError();
					}

				sImage.SetColor(aColor[1],aColor[3],aColor[5],aColor[7]);
				}
			}
		else if(uColorMapEntries)
			{
			pFile->Seek(uColorMapEntries*8,SEEK_CUR);
			}
		}

 
//******************** Bild Laden *********************************************


	CallProgressFunction(0);

	   uLen		=((uPixmapWidth*uBitsPerPixel+7)/8  );
	if(uLen>uBytesPerLine)uLen=uBytesPerLine;

	uPadSize	= uBytesPerLine-uLen;


	for(uRow=0;uRow<uPixmapHeight;uRow++)					// Alle Bildzeilen laden
		{
		CallProgressFunction(uRow*100/uPixmapHeight);

		pRow = sImage[uRow];

		if(pFile->Read(pRow,uLen      )!=(int)uLen    )break;
		if(pFile->Read(aColor,uPadSize)!=(int)uPadSize)break;
		
		if(uDoSort==0)continue;

		for(uOff=0;uOff<uLen;uOff+=uDoSort)					// Umsortieren der Bytes
			{
			aColor[0] = pRow[uOff+0];
			aColor[1] = pRow[uOff+1];
			aColor[2] = pRow[uOff+2];
			aColor[3] = pRow[uOff+3];

			pRow[uOff+0] = aColor[aSort[0]];
			pRow[uOff+1] = aColor[aSort[1]];
			pRow[uOff+2] = aColor[aSort[2]];

			if(uDoSort>=4)pRow[uOff+3]=0;
			}
		}

	if(uRow<uPixmapHeight)									// Gab es einen Fehler
		{
		throw EXwdFileReadError();
		}


	CallProgressFunction(100);

}

