//*****************************************************************************
//*
//*
//*		XwdEncoder.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"XwdEncoder.h"
#include	"..\FileBuffer.h"


static inline void BTOL(void *p,unsigned a)	{*(unsigned      *)p = ((((a)&0xFF000000)>>24)|(((a)&0x00FF0000)>>8)|(((a)&0x0000FF00)<<8)|(((a)&0x000000FF)<<24));}
static inline void BTOW(void *p,unsigned a)	{*(unsigned short*)p = ((((a)&0xFF00)>>8)|(((a)&0x00FF)<<8));}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
XwdEncoder::XwdEncoder()
{

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
XwdEncoder::~XwdEncoder()
{
}

//*****************************************************************************
//*
//*		WriteImage
//*
//*****************************************************************************
//	Speichert das Bild ab.
//	pName		: Dateiname der Bilddatei
//  sImage		: Klasse in der das Bild gespeichert wird
void XwdEncoder::WriteImage(const char *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EXwdFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void XwdEncoder::WriteImage(const wchar_t *pName,const BitmapImage &sImage)
{
CFileBuffer	sBuffer;


	if(!sBuffer.Open(pName,CFB_CREATE))throw EXwdFileOpenError();

	try	{
		WriteImage(&sBuffer,sImage);
		}
	catch(...)
		{
		sBuffer.Close();
		throw;
		}

	sBuffer.Close();
}

void XwdEncoder::WriteImage(CFileBuffer *pFile,const BitmapImage &sImage)
{
unsigned			uSize;
unsigned			uPos;
unsigned char	   *pPtr;



	
	if(!sImage.IsOk())
		{
		throw EXwdDataError();
		}

	if(sImage.BitCount()>8)
		{
		unsigned char aHeader24[] = 
			{
			0x00,0x00,0x00,0x6B,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x18,
			0x00,0x00,0x04,0x80,0x00,0x00,0x03,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
			0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x20,
			0x00,0x00,0x12,0x00,0x00,0x00,0x00,0x04,0x00,0xFF,0x00,0x00,0x00,0x00,0xFF,0x00,
			0x00,0x00,0x00,0xFF,0x00,0x00,0x00,0x08,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,
			0x00,0x00,0x04,0x80,0x00,0x00,0x03,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
			0x00,0x00,0x00,0x00,0x78,0x77,0x64,0x75,0x6D,0x70,0x00,
			};

		uSize = sImage.Width()*4;
		
		BTOL(aHeader24+12*4,uSize);
		BTOL(aHeader24+ 4*4,sImage.Width ());
		BTOL(aHeader24+ 5*4,sImage.Height());
		BTOL(aHeader24+20*4,sImage.Width ());
		BTOL(aHeader24+21*4,sImage.Height());
		BTOL(aHeader24+22*4,sImage.Width ());
		BTOL(aHeader24+23*4,sImage.Height());

		CallProgressFunction(0);

		if(pFile->Write(aHeader24,sizeof(aHeader24))!=sizeof(aHeader24))
			{
			throw EXwdFileWriteError();
			}

		for(uPos=0;uPos<sImage.Height();uPos++)
			{
			CallProgressFunction(uPos*100/sImage.Height());

			pPtr	= sImage.GetRow4(uPos);

			if(pFile->Write(pPtr,uSize)!=(int)uSize)throw EXwdFileWriteError();
			}
		
		CallProgressFunction(100);
		}
	else{
		unsigned char aColors8[] = { 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x94 };
		unsigned char aHeader8[] = 
			{
			0x00,0x00,0x00,0x6B,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x08,
			0x00,0x00,0x02,0x08,0x00,0x00,0x01,0xFE,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
			0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x08,
			0x00,0x00,0x02,0x08,0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
			0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x00,
			0x00,0x00,0x02,0x08,0x00,0x00,0x01,0xFE,0x00,0x00,0x01,0xE0,0x00,0x00,0x00,0xA0,
			0x00,0x00,0x00,0x00,0x78,0x77,0x64,0x75,0x6D,0x70,0x00,
			};

		uSize = sImage.Width();
		
		BTOL(aHeader8+12*4,uSize);
		BTOL(aHeader8+ 4*4,sImage.Width ());
		BTOL(aHeader8+ 5*4,sImage.Height());
		BTOL(aHeader8+20*4,sImage.Width ());
		BTOL(aHeader8+21*4,sImage.Height());
		BTOL(aHeader8+22*4,sImage.Width ());
		BTOL(aHeader8+23*4,sImage.Height());

		CallProgressFunction(0);

		if(pFile->Write(aHeader8,sizeof(aHeader8))!=sizeof(aHeader8))
			{
			throw   EXwdFileWriteError();
			}

		for(uPos=0;uPos<256;uPos++)						
			{
			sImage.GetColor(uPos,aColors8[4],aColors8[6],aColors8[8]);
			
			aColors8[3] = uPos;
			aColors8[5] = aColors8[4];
			aColors8[7] = aColors8[6];
			aColors8[9] = aColors8[8];
			
			if(pFile->Write(aColors8,12)!=12)throw EXwdFileWriteError();
			}

		for(uPos=0;uPos<sImage.Height();uPos++)
			{
			CallProgressFunction(uPos*100/sImage.Height());

			pPtr	= sImage.GetRow1(uPos);

			if(pFile->Write(pPtr,uSize)!=(int)uSize)throw EXwdFileWriteError();
			}
		
		CallProgressFunction(100);
		}

}

//*****************************************************************************
//*
//*		CallProgressFunction
//*
//*****************************************************************************
void XwdEncoder::CallProgressFunction (unsigned int percent)
{
bool	bCancel;


	if(pProgressProc == NULL)return;

	bCancel = false;
	pProgressProc(*this,pProgressData,1,1,percent,bCancel);

	if(bCancel)throw EGraphicsAbort();

}
