//*****************************************************************************
//*
//*
//*		LicenseTools.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2009
//
//
//	Diese Datei enthlt Funktionen fr die Lizenzverwaltung.
//
#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#ifndef	 __AFX_H__
	#include <Afx.h>
	#include <AfxSock.h>
	#endif
#else
	#include <winsock2.h>
	#include <windows.h>
#endif

#include	<tchar.h>
#include	<stdio.h>
#include	<string.h>
#include	"LicenseTools.h"
#include	"BitFunctions.h"
#include	"MemFunctions.h"
#include	"StringFunctions.h"

#ifndef		KEY_WOW64_64KEY
#define		KEY_WOW64_64KEY		0x0100					// 32-Bit Programme nicht auf WOW64 Umleitung gehen
#endif

#ifndef		KEY_WOW64_32KEY
#define		KEY_WOW64_32KEY		0x0200					// 64-Bit Programme immer auf WOW64 Umleitung gehen
#endif


static unsigned  	uMinTime   = 0;						// Minimal erlaubte Zeit
static int 			iTimeDiff  = 0;						// Zeit-Offset in Sekunden (falls lokale Zeit nicht stimmt)
static int 			iLastError = 0;						// Letzer Fehler von LicenseCheck(...)/LicenseCheckIp(...)
static char 		cMyHost[64]= "";					// Hoastname vom Rechner


//*****************************************************************************
//*
//*		LicenseHostName
//*
//*****************************************************************************
//	Hostnamen vom Rechner abfragen
const char *LicenseHostName()
{

	if(!cMyHost[0])
		{
		WORD    wVersion;
		WSADATA sData;

		wVersion = 0x0002;
		WSAStartup(wVersion,&sData);

		gethostname((char*)cMyHost,sizeof(cMyHost));
		}


return cMyHost;
}

//******************************************************************************
//*
//*		LicenseLastError
//*
//******************************************************************************
//	Holt den letzer Fehler von LicenseCheck(...)/LicenseCheckIp(...)
//			0 = Kein Fehler
//			1 = Kann Lizensdaten in Registry nicht finden
//			2 = Kann Lizensdatei nicht ffnen
//			3 = Die Lizensdaten sind nicht vollstndig
//			4 = Die Lizensdaten sind abgelaufen
//			5 = Die Lizensdaten sind ungltig
//			6 = Die Lizensdaten wurden in der Datei nicht gefunden
//			7 = Die IP-Listendatei konnte nicht geffnet werden
//			8 = Ungltige IP-Adresse zur Prfung bergeben
//			9 = Die IP-Adresse ist nicht in der IP-Liste
//		   10 = Die IP-Liste hat eine falsche Prfsumme
//		   11 = Die Lizens sind abgelaufen weil Zeit zurckgesetz
//		   12 = Der Zeitstempel konnte nicht gespeichert werden
int LicenseLastError()
{
	return iLastError;
}

//******************************************************************************
//*
//*		LicenseSetTime
//*
//******************************************************************************
//	Eine andere lokale Zeit fr einstellen, falls lokale Zeit nicht stimmt.
//	uUtcTime	: Ist die neue UTC-Zeit (Sekunden seit 1.1.1970)
void LicenseSetTime(unsigned int uUtcTime)
{
unsigned 	uNow;



	if(uUtcTime>0xFFFF0000 || uUtcTime<=0xFFFF)			// Prfe die Zeit
		{
		iTimeDiff = 0;
		return;
		}

	uNow		= (unsigned)time(0);
	iTimeDiff	= uUtcTime-uNow;

}

//******************************************************************************
//*
//*		LicenseMinTime
//*
//******************************************************************************
//	Setzt die minimal eralaubte Zeit fr einstellen, falls lokale Zeit nicht stimmt.
//	uUtcTime	: Ist die neue UTC-Zeit (Sekunden seit 1.1.1970)
void LicenseMinTime(unsigned int uUtcTime)
{

	if(uUtcTime<4*3600)
		{
		uUtcTime = 0;
		}
	else{
		uMinTime = uUtcTime-4*3600;
		}
}

//*****************************************************************************
//*
//*		DiskGetId
//*
//*****************************************************************************
//	Auslesen der Seriennummer der ersten Fesplatte
//	pBuffer		: Zeiger auf einen 4-Byte-Buffer fr die ID, oder 0.
//	Ergibt die ID der Festplatte bzw. 0 wenn nichts gefunden wurde.
unsigned DiskGetId(unsigned char *pBuffer)
{
TCHAR			cName[32]="C:\\";
static DWORD	uSerial=0;
unsigned		uBits;
unsigned		uDisk;


	if(!uSerial)
		{
		uBits	= GetLogicalDrives();

		for(uDisk=2;uDisk<8;uDisk++)
			{
			if(!((uBits>>uDisk)&1))continue;

			cName[0] = 'A'+uDisk;

			if(GetDriveType(cName)!=DRIVE_FIXED)continue;
			if(GetVolumeInformation(cName,NULL,0,&uSerial,NULL,NULL,NULL,0))
				{
				break;
				}
			}
		}

	if(pBuffer)
		{
		pBuffer[0] = (unsigned char)(uSerial>> 0);
		pBuffer[1] = (unsigned char)(uSerial>> 8);
		pBuffer[2] = (unsigned char)(uSerial>>16);
		pBuffer[3] = (unsigned char)(uSerial>>24);
		}


return uSerial;
}



//*****************************************************************************
//*
//*		LicenseCheck
//*
//*****************************************************************************
//	Prft einen Lizenztext. Der Lizenztext ist ein HexDump.
//	Die dekodierten Byte haben folgenden Aufbau:
//			0.. 3	= uKey
//			4.. 7	= Null
//			8..11	= Freischaltebits
//		   12..13	= Datum (Tage seit 1.1.1970)
//		   14..15	= Gltig bis (Tage seit 1.1.1970)
//		   16..n-2	= Lizenztext
//		       n-1	= Prfsumme ber alle Bytes
//	pLicense	: Hexdump mit der Lizenz.
//	uKey		: Ist ein User-Wert fr die Verschlsselung.
//	pName		: Hier wird der Namen der Lizenz abhelegt.
//	iSize		: Gre des Namensspeichers.
//	pUntil		: Hier wird die UTC Zeit gespeichert bis wann die Liezens
//				  gltig ist. (0 = fr immer)
//	pExtension	: Zeiger auf die erweiterten Daten erstes DWORD sind Bits
//	uExSize		: Gre der erweiterten Daten (minimal 4)
//	Ergibt 0 bei einer falschen Lizenz bzw. die Freischaltebits.
int LicenseCheck(const TCHAR *pLicense,unsigned uKey,TCHAR *pName,int iSize,unsigned *pUntil,unsigned *pExtension,unsigned uExSize)
{
const char	   *pHost;
unsigned char  *pMac;
unsigned char   ucMac [6];
unsigned char   ucKey [4];
unsigned char   ucDisk[4];
unsigned char 	ucMem [128];
unsigned char 	ucData[132];
char			cTemp [256];
unsigned		uUntil;
unsigned		uDone;
unsigned		uSize;
unsigned		uDisk;
unsigned		uCard;
unsigned		uNow;
unsigned		uRun;
unsigned		uAdd;
unsigned		uNum;
unsigned 		uLen;
unsigned		uOff;
unsigned		uPos;



	if(pExtension)										// Erweiterte Daten lschen
		{
		memset(pExtension,0,uExSize);
		}

	if(pName)
		{
		strlcpy(pName,_T("---"),iSize);
		}

	if(pUntil)
		{
		*pUntil = 0;
		}

	uLen = strlcpy(cTemp,pLicense,sizeof(cTemp)/sizeof(cTemp[0]));

	while(uLen>0)
		{
		uLen--;

		if(cTemp[uLen]=='-')cTemp[uLen]=' ';
		if(cTemp[uLen]==':')cTemp[uLen]=' ';
		}

	   uLen = dumptomem(ucMem,sizeof(ucMem),cTemp);
	if(uLen<15)											// Zuwenig Daten
		{
		if(pUntil )*pUntil=0;
		if(iSize>0)*pName=0;
		iLastError = 3;
		return 0;
		}


	uDone		= 0;
	uDisk		= DiskGetId(ucDisk);
	ucDisk[0] 	= (unsigned char)(uDisk>> 0);
	ucDisk[1] 	= (unsigned char)(uDisk>> 8);
	ucDisk[2] 	= (unsigned char)(uDisk>>16);
	ucDisk[3] 	= (unsigned char)(uDisk>>24);
	ucKey [0] 	= (unsigned char)(uKey >> 0);
	ucKey [1] 	= (unsigned char)(uKey >> 8);
	ucKey [2] 	= (unsigned char)(uKey >>16);
	ucKey [3] 	= (unsigned char)(uKey >>24);


	for(uCard=0,uRun=1;uRun;uCard++)					// Alle MAC-Adressen prfen
		{
			pMac = MacOfCard(uCard);
		if(!pMac)
			{
			if(!uDone)
				{
				memset(ucMac,0,sizeof(ucMac));
				pHost    = LicenseHostName();
				ucMac[0] = 0xD3;
				ucMac[1] = 0xD3;

				for(uPos=0;*pHost;uPos++)
					{
					ucMac[2+(uPos&3)] += *pHost;
					pHost++;
					}

				pMac  = ucMac;
				uDone = 1;
				}
			else{
				pMac  = (unsigned char*)"\0\0\0\0\0\0";
				uRun  = 0;
				}
			}

		memcpy(ucData,ucMem,uLen);
		uAdd = uKey%13;

		for(uNum=0;uNum<uLen;uNum++)
			{
			ucData[uNum] += ucKey [(uNum+uAdd)%4];
			ucData[uNum] ^= ucDisk[(uNum+uAdd)%4];
			ucData[uNum] -= pMac  [(uNum+uAdd)%6];
			}

		uNum = uLen-1;

		while(uNum>0)									// Checksumme dekodieren
			{
			uNum--;
			ucData[uLen-1] += ucData[uNum];
			}

		ucData[uLen-1] -= uKey%319;

		if(ucData[uLen-1]!=0x00)continue;				// Ergebnis prfen
		if(ucData[0]!=ucKey [0])continue;
		if(ucData[1]!=ucKey [1])continue;
		if(ucData[2]!=ucKey [2])continue;
		if(ucData[3]!=ucKey [3])continue;
		if(ucData[4]!=ucDisk[0])continue;
		if(ucData[5]!=ucDisk[1])continue;
		if(ucData[6]!=ucDisk[2])continue;
		if(ucData[7]!=ucDisk[3])continue;

   		uNum  = ucData[ 8];								// Lizenzbits holen
		uNum |= ucData[ 9]<<8;
		uNum |= ucData[10]<<16;
		uNum |= ucData[11]<<24;

		uLen -= 1;										// Checksumme abschneiden
		uOff  = 16;										// Offset auf Usertext
		uSize = ucData[uOff+1];

		if(ucData[uOff]==1 && uOff+uSize+1<uLen)		// Erweiterten Daten vorhanden
			{
			uOff += 2+uSize;

			if(pExtension && uExSize>4)
				{
				dumptomem(pExtension+1,uExSize-4,(char*)ucData+18);
				}
			}

		if(pName)										// Den Usertext speichern
			{
			for(;uOff<uLen;uOff++)
				{
				if(iSize<=1)break;
				pName[0] = ucData[uOff];
				pName++;
				iSize--;
				}

			if(iSize>=16)
				{
				pName[0] = ',';
				pName[1] = ' ';
				pName+=2;
				pName+=strtimeset(pName,(ucData[12]+ucData[13]*256)*24*3600,STR_DATE|STR_LONG|STR_ABC);
				}

			if(iSize>0)*pName=0;
			}

		uUntil  = ucData[14];							// Lizenzdatum holen
		uUntil |= ucData[15]<<8;
		uUntil *= 3600*24;

		if(pUntil)
			{
			*pUntil = uUntil;
			}

		uNow  = (unsigned)time(0);
		uNow += iTimeDiff;

		if(uUntil && uNow>uUntil)						// Ist die Lizens abgelaufen
			{
			iLastError = 11;
			return 0;
			}

		if(uUntil && uNow>uUntil)						// Ist die Lizens abgelaufen
			{
			iLastError = 4;
			return 0;
			}

		if(pExtension && uExSize>=4)					// Lizenzbits in erweiterten Daten speichern
			{
			*pExtension = uNum;
			}

		iLastError = 0;

		return uNum;
		}


	iLastError = 5;


return 0;
}

//*****************************************************************************
//*
//*		LicenseCheckEx
//*
//*****************************************************************************
//	Prft einen Lizenztext zu einer MAC und Disk-ID. Der Lizenztext ist ein HexDump.
//	Die dekodierten Byte haben folgenden Aufbau:
//			0.. 3	= uKey
//			4.. 7	= Null
//			8..11	= Freischaltebits
//		   12..13	= Datum (Tage seit 1.1.1970)
//		   14..15	= Gltig bis (Tage seit 1.1.1970)
//		   16..n-2	= Lizenztext
//		       n-1	= Prfsumme ber alle Bytes
//	pLicense	: Hexdump mit der Lizenz.
//	uKey		: Ist ein User-Wert fr die Verschlsselung.
//	pName		: Hier wird der Namen der Lizenz abhelegt.
//	iSize		: Gre des Namensspeichers.
//	pUntil		: Hier wird die UTZ Zeit gespeichert bis wann die Liezens
//				  gltig ist. (0 = fr immer)
//	pExtension	: Zeiger auf die erweiterten Daten erstes DWORD sind Bits
//	uExSize		: Gre der erweiterten Daten (minimal 4)
//	Ergibt 0 bei einer falschen Lizenz bzw. die Freischaltebits.
int LicenseCheckEx(const TCHAR *pLicense,unsigned uKey,const unsigned char *pMac,unsigned uDisk,TCHAR *pName,int iSize,unsigned *pUntil,unsigned *pExtension,unsigned uExSize)
{
unsigned char   ucKey [4];
unsigned char   ucDisk[4];
unsigned char 	ucMem [128];
unsigned char 	ucData[132];
char			cTemp [256];
unsigned		uUntil;
unsigned		uSize;
unsigned 		uNow;
unsigned		uAdd;
unsigned		uNum;
unsigned 		uLen;
unsigned		uOff;



	if(pExtension)										// Erweiterten Date lschen
		{
		memset(pExtension,0,uExSize);
		}

	if(pName)
		{
		strlcpy(pName,_T("---"),iSize);
		}

	if(pUntil)
		{
		*pUntil = 0;
		}

	uLen = strlcpy(cTemp,pLicense,sizeof(cTemp)/sizeof(cTemp[0]));

	while(uLen>0)
		{
		uLen--;

		if(cTemp[uLen]=='-')cTemp[uLen]=' ';
		if(cTemp[uLen]==':')cTemp[uLen]=' ';
		}

	   uLen = dumptomem(ucMem,sizeof(ucMem),cTemp);
	if(uLen<15)											// Zuwenig Daten
		{
		if(iSize>0)*pName=0;
		return 0;
		}


	ucDisk[0] 	= (unsigned char)(uDisk>> 0);
	ucDisk[1] 	= (unsigned char)(uDisk>> 8);
	ucDisk[2] 	= (unsigned char)(uDisk>>16);
	ucDisk[3] 	= (unsigned char)(uDisk>>24);
	ucKey [0] 	= (unsigned char)(uKey >> 0);
	ucKey [1] 	= (unsigned char)(uKey >> 8);
	ucKey [2] 	= (unsigned char)(uKey >>16);
	ucKey [3] 	= (unsigned char)(uKey >>24);


	memcpy(ucData,ucMem,uLen);
	uAdd = uKey%13;

	for(uNum=0;uNum<uLen;uNum++)
		{
		ucData[uNum] += ucKey [(uNum+uAdd)%4];
		ucData[uNum] ^= ucDisk[(uNum+uAdd)%4];
		ucData[uNum] -= pMac  [(uNum+uAdd)%6];
		}

	uNum = uLen-1;

	while(uNum>0)										// Checksumme dekodieren
		{
		uNum--;
		ucData[uLen-1] += ucData[uNum];
		}

	ucData[uLen-1] -= uKey%319;

	if(ucData[uLen-1]!=0x00)return 0;					// Ergebnis prfen
	if(ucData[0]!=ucKey [0])return 0;
	if(ucData[1]!=ucKey [1])return 0;
	if(ucData[2]!=ucKey [2])return 0;
	if(ucData[3]!=ucKey [3])return 0;
	if(ucData[4]!=ucDisk[0])return 0;
	if(ucData[5]!=ucDisk[1])return 0;
	if(ucData[6]!=ucDisk[2])return 0;
	if(ucData[7]!=ucDisk[3])return 0;

	uNum  = ucData[ 8];									// Lizenzbits holen
	uNum |= ucData[ 9]<<8;
	uNum |= ucData[10]<<16;
	uNum |= ucData[11]<<24;

	uLen -= 1;											// Checksumme abschneiden
	uOff  = 16;											// Offset auf Usertext
	uSize = ucData[uOff+1];

	if(ucData[uOff]==1 && uOff+uSize+1<uLen)			// Erweiterten Daten vorhanden
		{
		uOff += 2+uSize;

		if(pExtension && uExSize>4)
			{
			dumptomem(pExtension+1,uExSize-4,(char*)ucData+18);
			}
		}

	if(pName)											// Den Usertext speichern
		{
		for(;uOff<uLen;uOff++)
			{
			if(iSize<=1)break;
			pName[0] = ucData[uOff];
			pName++;
			iSize--;
			}

		if(iSize>=16)
			{
			pName[0] = ',';
			pName[1] = ' ';
			pName+=2;
			pName+=strtimeset(pName,(ucData[12]+ucData[13]*256)*24*3600,STR_DATE|STR_LONG|STR_ABC);
			}

		if(iSize>0)*pName=0;
		}

	uUntil  = ucData[14];								// Lizenzdatum holen
	uUntil |= ucData[15]<<8;
	uUntil *= 3600*24;

	if(pUntil)
		{
		*pUntil = uUntil;
		}

	uNow	= (unsigned)time(0);
	uNow   += iTimeDiff;

	if(uUntil && uNow>uUntil)							// Ist die Lizens abgelaufen
		{
		iLastError = 11;
		return 0;
		}

	if(uUntil && uNow>uUntil)							// Ist die Lizens abgelaufen
		{
		return 0;
		}

	if(pExtension && uExSize>=4)						// Lizenzbits in erweiterten Daten speichern
		{
		*pExtension = uNum;
		}


return uNum;
}


//*****************************************************************************
//*
//*		LicenseMake
//*
//*****************************************************************************
//	Erzeugt einen Lizenztext. Der Lizenztext ist ein HexDump.
//	Die dekodierten Byte haben folgenden Aufbau:
//			0.. 3	= uKey
//			4.. 7	= Null
//			8..11	= Freischalte Bits
//		   12..13	= Datum (Tage seit 1.1.1970)
//		   14..15	= Gltig bis (Tage seit 1.1.1970)
//		   16..n-2	= Lizenztext
//		       n-1	= Prfsumme ber alle Bytes
//	pMac		: Zeiger auf die MAC-Adresse
//	uDisc		: Ist die Seriennummer der Festplatte
//	uKey		: Ist ein User-Wert fr die Verschlsselung
//	uBits		: Sind die Freischalte Bits, darf nicht 0 sein.
//	pText		: Speicher fr den Lizenztext
//	iSize		: Ist die Gre des Textspeichers
//	pNamen		: Ist der Namen der fr die Lizenz.
//	uUntil		: Ist die UTC Zeit bis wann die Lizenz gltig ist. (0=immer)
//	pExtension	: Zeiger auf die erweiterten Daten erstes DWORD sind Bits
//	uExSize		: Gre der erweiterten Daten (minimal 4)
//	Ergibt die Lnge des Hex-Dumps.
int LicenseMake(const unsigned char *pMac,unsigned uDisk,unsigned uKey,unsigned uBits,TCHAR *pText,int iSize,const TCHAR *pName,unsigned uUntil,const unsigned *pExtension,unsigned uExSize)
{
unsigned char 	ucMem[128];
unsigned		uChar;
unsigned		uDate;
unsigned 		uNow;
unsigned		uLen;
unsigned		uNum;
unsigned		uAdd;



	if(!uBits)uBits=0x80000000;

	ucMem[ 0] 	= (unsigned char)(uKey>> 0);
	ucMem[ 1] 	= (unsigned char)(uKey>> 8);
	ucMem[ 2] 	= (unsigned char)(uKey>>16);
	ucMem[ 3] 	= (unsigned char)(uKey>>24);

	ucMem[ 4] 	= (unsigned char)(uDisk>> 0);
	ucMem[ 5] 	= (unsigned char)(uDisk>> 8);
	ucMem[ 6] 	= (unsigned char)(uDisk>>16);
	ucMem[ 7] 	= (unsigned char)(uDisk>>24);

	ucMem[ 8] 	= (unsigned char)(uBits>> 0);
	ucMem[ 9] 	= (unsigned char)(uBits>> 8);
	ucMem[10] 	= (unsigned char)(uBits>>16);
	ucMem[11] 	= (unsigned char)(uBits>>24);

	uNow		= (unsigned)time(0);
	uNow       += iTimeDiff;
	uDate	  	= (unsigned     )(uNow/(24*3600));
	ucMem[12] 	= (unsigned char)(uDate>>0);
	ucMem[13]	= (unsigned char)(uDate>>8);
	uUntil	  	= uUntil/(24*3600);
	ucMem[14] 	= (unsigned char)(uUntil>>0);
	ucMem[15]	= (unsigned char)(uUntil>>8);

	uLen		= 16;

	if(pExtension && uExSize>4)								// Erweiterte Daten anhngen
		{
		if(uExSize>36)uExSize=36;
		uAdd = memtodump((char*)ucMem+18,pExtension+1,uExSize-4,0);
		ucMem[16]   = 0x01;
		ucMem[17]   = uAdd+1;
		uLen       += uAdd+2;
		ucMem[uLen] = '|';
		uLen       += 1;
		}

	for(;uLen<126;uLen++)									// Usertext anhngen
		{
			uChar = *pName;
		if(!uChar)break;

		#ifdef UNICODE
		if(uChar>255)uChar=255;
		#endif

		ucMem[uLen] = (unsigned char)uChar;
		pName++;
		}

	ucMem[uLen] = uKey%319;									// Prfsumme speichern
	uNum		= uLen;

	while(uNum>0)
		{
		uNum--;
		ucMem[uLen] -= ucMem[uNum];
		}

	uAdd		= uKey%13;
	uLen		= uLen+1;

	for(uNum=0;uNum<uLen;uNum++)							// Byte kodieren
		{
		ucMem[uNum] += (unsigned char)(pMac   [((uNum+uAdd)%6)]  );
		ucMem[uNum] ^= (unsigned char)(uDisk>>(((uNum+uAdd)%4)*8));
		ucMem[uNum] -= (unsigned char)(uKey >>(((uNum+uAdd)%4)*8));
		}


	memtodump(pText,ucMem,uLen,HEX_NULL|4);


return uLen;
}

//******************************************************************************
//*
//*		LicenseMinOfReg
//*
//******************************************************************************
//	Holt letzte Aufrufzeit aus der Registry und
//	setzt die aktuelle Zeit wenn sie neuer ist.
//	pRegPath	: Ist der Pfad in der Registry (z.B. "SOFTWARE\\automationX\\Policies")
//	pRegKey		: Ist der Schlsselnamen in der Registry.
//	pMinTime	: Hier wird die UTC-Zeit gespeichert (0 = nicht holen)
//	Ergibt 0 bei einem Fehler.
//					0 = Registry Wert konnte nicht geladen oder gespeichert werden
//
int LicenseMinOfReg(const TCHAR *pRegPath,const TCHAR *pRegKey,unsigned *pMinTime)
{
TCHAR			cText[1024];
DWORD			dwSize=0;
DWORD			dwType=0;
DWORD			dwValue;
HKEY			hKey;
UINT			uNow;
INT				iRet;
LONG			lOk;




	if(pMinTime)
		{
		*pMinTime = 0;
		}
														// Registry-Key ffnen
	lOk = RegOpenKeyEx(HKEY_LOCAL_MACHINE,pRegPath,0,KEY_READ|KEY_WRITE|KEY_WOW64_64KEY,&hKey);

	if(lOk!=ERROR_SUCCESS)
		{
		lOk = RegOpenKeyEx(HKEY_LOCAL_MACHINE,pRegPath,0,KEY_READ|KEY_WRITE|KEY_WOW64_32KEY,&hKey);

		if(lOk!=ERROR_SUCCESS)
			{
			lOk = RegOpenKeyEx(HKEY_LOCAL_MACHINE,pRegPath,0,KEY_READ|KEY_WRITE,&hKey);
			iLastError = 1;
			return 0;
			}
		}

	uNow	= (unsigned)time(0);
	uNow   += iTimeDiff;

	cText[0]= 0;
	dwType	= REG_DWORD;
	dwSize	= sizeof(dwValue);
	iRet	= 1;
														// Minimalzeit einlesen
	   lOk	= RegQueryValueEx(hKey,pRegKey,0,&dwType,(unsigned char*)&dwValue,&dwSize);
	if(lOk==ERROR_SUCCESS)
		{
		if(dwValue>uNow)
			{
			uNow = dwValue;								// Zeit wurde zurck gesetzt
			iRet = 3;
			}
		else{
			iRet = 2;
			}
		}
														// Minimalzeit speichern
	   lOk	= RegSetValueEx(hKey,pRegKey,0,dwType,(unsigned char*)&dwValue,sizeof(dwValue));
	if(lOk==ERROR_SUCCESS)
		{
		iLastError = 12;
		iRet = 0;
		}

	RegCloseKey(hKey);

	if(pMinTime)
		{
		*pMinTime = uNow;
		}

	LicenseMinTime(uNow);								// Neue minimal Zeit bergeben



return iRet;
}


//******************************************************************************
//*
//*		LicenseOfReg
//*
//******************************************************************************
//	Holt eine Lizenz aus der Registry und prft den Schlssel
//	Die dekodierten Byte haben folgenden Aufbau:
//			0.. 3	= Key
//			4.. 7	= Null
//			8..11	= Freischaltebits
//		   12..13	= Datum (Tage seit 1.1.1970)
//		   14..15	= Gltig bis (Tage seit 1.1.1970)
//		   16..n-2	= Lizenztext
//		       n-1	= Prfsumme ber alle Bytes
//	pRegPath	: Ist der Pfad in der Registry (z.B. "SOFTWARE\\automationX\\Policies")
//	pRegKey		: Ist der Schlsselnamen in der Registry.
//	uKey		: Ist ein User-Wert fr die Verschlsselung.
//	pName		: Hier wird der Namen der Lizenz abhelegt.
//	iSize		: Gre des Namensspeichers.
//	pUntil		: Hier wird die UTC Zeit gespeichert bis wann die Lizenz
//				  gltig ist. (0 = fr immer)
//	pExtension	: Zeiger auf die erweiterten Daten erstes DWORD sind Bits
//	uExSize		: Gre der erweiterten Daten (minimal 4)
//	Ergibt 0 bei einer falschen Lizenz bzw. die Freischaltebits.
int LicenseOfReg(const TCHAR *pRegPath,const TCHAR *pRegKey,unsigned uKey,TCHAR *pName,int iSize,unsigned *pUntil,unsigned *pExtension,unsigned uExSize)
{
TCHAR			cText[1024];
DWORD			dwSize=0;
DWORD			dwType=0;
HKEY			hKey;
INT				iRet;
LONG			lOk1;
LONG			lOk;



	if(pExtension)										// Erweiterten Date lschen
		{
		memset(pExtension,0,uExSize);
		}

	if(pName)
		{
		strlcpy(pName,_T("---"),iSize);
		}

	if(pUntil)
		{
		*pUntil = 0;
		}

	lOk = RegOpenKeyEx(HKEY_LOCAL_MACHINE,pRegPath,0,KEY_READ|KEY_WOW64_64KEY,&hKey);

	if(lOk!=ERROR_SUCCESS)
		{
		lOk = RegOpenKeyEx(HKEY_LOCAL_MACHINE,pRegPath,0,KEY_READ|KEY_WOW64_32KEY,&hKey);

		if(lOk!=ERROR_SUCCESS)
			{
			lOk = RegOpenKeyEx(HKEY_LOCAL_MACHINE,pRegPath,0,KEY_READ,&hKey);
			iLastError = 1;
			return 0;
			}
		}


	cText[0]= 0;
	dwType	= REG_SZ;
	dwSize	= sizeof(cText);
	lOk1	= RegQueryValueEx(hKey,pRegKey,0,&dwType,(unsigned char*)cText,&dwSize);

	if(lOk1!=ERROR_SUCCESS)
		{
		iLastError = 1;
		iRet       = 0;
		}
	else{
		iRet = LicenseCheck(cText,uKey,pName,iSize,pUntil,pExtension,uExSize);
		}

	RegCloseKey(hKey);



return iRet;
}

//*****************************************************************************
//*
//*		LicenseOfFile
//*
//*****************************************************************************
//	Holt eine Lizenz aus einer Datei und prft den Schlssel
//	Die dekodierten Byte haben folgenden Aufbau:
//			0.. 3	= uKey
//			4.. 7	= Null
//			8..11	= Freischaltebits
//		   12..13	= Datum (Tage seit 1.1.1970)
//		   14..15	= Gltig bis (Tage seit 1.1.1970)
//		   16..n-2	= Lizenztext
//		       n-1	= Prfsumme ber alle Bytes
//	pPath		: Ist der Dateiname
//	pItem		: Ist der Schlsselnamen in der Datei.  (z.B. KEY = .....)
//	uKey		: Ist ein User-Wert fr die Verschlsselung.
//	pName		: Hier wird der Namen der Lizenz abhelegt.
//	iSize		: Gre des Namensspeichers.
//	pUntil		: Hier wird die UTC Zeit gespeichert bis wann die Lizenz
//				  gltig ist. (0 = fr immer)
//	pExtension	: Zeiger auf die erweiterten Daten erstes DWORD sind Bits
//	uExSize		: Gre der erweiterten Daten (minimal 4)
//	Ergibt 0 bei einer falschen Lizenz bzw. die Freischaltebits.
int LicenseOfFile(const TCHAR *pPath,const TCHAR *pItem,unsigned uKey,TCHAR *pName,int iSize,unsigned *pUntil,unsigned *pExtension,unsigned uExSize)
{
TCHAR	cText[1024];
TCHAR	cItem[64];
FILE   *pFile;
INT		iRet;
INT		iPos;
INT		iLen;


	if(pExtension)											// Erweiterten Daten lschen
		{
		memset(pExtension,0,uExSize);
		}

	if(pName)
		{
		strlcpy(pName,_T("---"),iSize);
		}

	if(pUntil)
		{
		*pUntil = 0;
		}

		pFile = _tfopen(pPath,"rb");
	if(!pFile)
		{
		iLastError = 2;
		return 0;
		}

	iLen		= _tcslen(pItem);
	iRet		= 0;
	cText[0   ] = 0;
	cText[1023] = 0;
	iLastError  = 6;

	while(fgets((char*)cText,1023,pFile))					// Datei Zeilenweise laden
		{
		#if UNICODE
		iPos  = strlen((char*)cText)+1;
		for(;iPos>=0;iPos--)cText[iPos]=((char*)cText)[iPos];
		#endif

		iPos  = strwlen(cText,0xE0);
		strgetstr(cItem,cText,64);

		if(_tcscmp(cItem,pItem))continue;					// Key gefunden

		iPos += iLen;
		iPos += strwlen(cText+iPos,0xE0);

		if(cText[iPos]!='=')continue;
															// Lizenz prfen
		iRet  = LicenseCheck(cText+iPos+1,uKey,pName,iSize,pUntil,pExtension,uExSize);

		break;
		}

	fclose(pFile);




return iRet;
}


//*****************************************************************************
//*
//*		LicenseCrc
//*
//*****************************************************************************
//	Berechnet die Checksumme eines Tetxes. Leerzeichen und Kommentrare werden
//	nicht berechnet. Kommentare sind "// ...." und "; ...."
//	pText	: Zeiger auf den Tetx.
//	iSize	: Ist die Lnge der Zeile (-1 steht fr mit strlen(pText))
//	uCrc	: Ist die
unsigned LicenseCrc(const char *pText,int iSize,unsigned uCrc)
{
int		iPos;

	if(iSize<0)iSize=strlen(pText);


	for(iPos=0;iPos<iSize;iPos++)
		{
		if(strwchara(pText[iPos],0x802020E0))						// Leerzeichen und ";/\0"
			{
			if(strwchara(pText[iPos],0x800000E0))					// Leerzeichen
				{
				continue;
				}
																	// Kommentrare berspringen
			if(pText[iPos]==';' || (iPos+1<iSize && pText[iPos+1]=='/'))
				{
				iPos += strcwlen(pText+iPos+1,0x40,iSize-iPos);
				continue;
				}
			}

		uCrc  = bit_rol_dw(uCrc,5);
		uCrc += pText[iPos];
		}


return uCrc;
}

unsigned LicenseCrc(const wchar_t *pText,int iSize,unsigned uCrc)
{

int		iPos;

	if(iSize<0)iSize=wcslen(pText);


	for(iPos=0;iPos<iSize;iPos++)
		{
		if(strwcharw(pText[iPos],0x802020E0))						// Leerzeichen und ";/\0"
			{
			if(strwcharw(pText[iPos],0x800000E0))					// Leerzeichen
				{
				continue;
				}
																	// Kommentrare berspringen
			if(pText[iPos]==';' || (iPos+1<iSize && pText[iPos+1]=='/'))
				{
				iPos += strcwlen(pText+iPos+1,0x40,iSize-iPos);
				continue;
				}
			}

		uCrc  = bit_rol_dw(uCrc,5);

		if(pText[iPos]<255)
				uCrc += (char)pText[iPos];
		else	uCrc -= 1;
		}


return uCrc;
}

static unsigned char	aToNullList[32] =
	{
	31,18, 5,24,11,30,17, 4,23,10,29,16, 3,22, 9,28,
	15, 2,21, 8,27,14, 1,20, 7,26,13, 0,19, 6,25,12,
	};


//*****************************************************************************
//*
//*		LicenseSetCrc
//*
//*****************************************************************************
int LicenseSetCrc(char *pBuffer,unsigned uCrc,unsigned uToCrc,const char *pPre)
{
char       *pStart=pBuffer;
unsigned 	uDiff;
unsigned	uPos;
unsigned	uOff;



	for(;*pPre;pPre++,pBuffer++)
		{
		uCrc	= bit_rol_dw(uCrc,5);
		uCrc   += *pPre;
	   *pBuffer	= *pPre;
		}


	uDiff = uToCrc-uCrc;

	memset(pBuffer,'0',32);

	for(uPos=0;uPos<32;uPos++,uDiff>>=1)
		{
		if(!(uDiff&1))continue;

		uOff = aToNullList[uPos];
		pBuffer[uOff]++;
		}


	pBuffer[32] = '\n';
	pBuffer[33] = 0;


return pBuffer-pStart+33;
}

int LicenseSetCrc(wchar_t *pBuffer,unsigned uCrc,unsigned uToCrc,const wchar_t *pPre)
{
wchar_t    *pStart=pBuffer;
unsigned 	uDiff;
unsigned	uPos;
unsigned	uOff;



	for(;*pPre;pPre++,pBuffer++)
		{
		uCrc	= bit_rol_dw(uCrc,5);
		uCrc   += *pPre;
	   *pBuffer	= *pPre;
		}


	uDiff = uToCrc-uCrc;

	memset(pBuffer,'0',32);

	for(uPos=0;uPos<32;uPos++,uDiff>>=1)
		{
		if(!(uDiff&1))continue;

		uOff = aToNullList[uPos];
		pBuffer[uOff]++;
		}


	pBuffer[32] = '\n';
	pBuffer[33] = 0;


return pBuffer-pStart+33;
}

//*****************************************************************************
//*
//*		LicenseCrcFile
//*
//*****************************************************************************
//	Berechnet die CRC-Summe von einer Datei
//	pFile		: Ist das FILE-Handle
//	Ergibt die CRC Summe.
unsigned LicenseCrcFile(FILE *pFile)
{
char		cBuffer[1024];
unsigned	uCrc;


	uCrc = 0;
	cBuffer[sizeof(cBuffer)-1] = 0;

	while(fgets(cBuffer,sizeof(cBuffer)-1,pFile))
		{
		uCrc = LicenseCrc(cBuffer,-1,uCrc);
		}


return uCrc;
}

//*****************************************************************************
//*
//*		LicenseCrcFile
//*
//*****************************************************************************
//	Berechnet die CRC-Summe von einer Datei
//	pFileName		: Ist der Dateiname von der Datei
//	Ergibt die CRC Summe bzw. 0 wenn die Datei nicht gefunden wurde.
unsigned LicenseCrcFile(const char *pFileName)
{
FILE	   *pFile;
unsigned	uCrc;


		pFile = fopen(pFileName,"rb");
	if(!pFile)return 0;


	uCrc = LicenseCrcFile(pFile);

	fclose(pFile);



return uCrc;
}

unsigned LicenseCrcFile(const wchar_t *pFileName)
{
FILE	   *pFile;
unsigned	uCrc;


		pFile = _wfopen(pFileName,L"rb");
	if(!pFile)return 0;


	uCrc = LicenseCrcFile(pFile);

	fclose(pFile);



return uCrc;
}

//*****************************************************************************
//*
//*		LicenseTextToIpAddress
//*
//*****************************************************************************
//	Wandel eine IP-Adresse aus einem Text in eine binre Adresse um
//	uAddr		: Hier wird die Adresse gespeichert
//	pText		: Zeiger auf die Adresse ("127.0.0.1" oder "192.168.0.1/24")
//	Ergibt die Lnge der Adresse wenn sie gltig ist
static unsigned LicenseTextToIpAddress(unsigned &uAddr,const char *pText)
{
const char *pStart=pText;
unsigned	uAll;
unsigned	uSub;
unsigned	uVal;



	uAll	= 0;
	uAddr	= 0;

	for(uSub=0;;uSub+=8)
		{
		for(uVal=0;*pText>='0' && *pText<='9';pText++)
			{
			uVal *= 10;
			uVal += *pText-'0';
			}

		if(uVal>255)return 0;

		uAll |= uVal<<uSub;

		if(uSub == 3*8)break;
		if(*pText!='.')break;

		pText++;
		}


	if(!strwchara(*pText,0x802000E0))
		{
		return 0;
		}


	uAddr = uAll;



return pText-pStart;
}

static unsigned LicenseTextToIpAddress(unsigned &uAddr,const wchar_t *pText)
{
const wchar_t  *pStart=pText;
unsigned		uAll;
unsigned		uSub;
unsigned		uVal;



	uAll	= 0;
	uAddr	= 0;

	for(uSub=0;;uSub+=8)
		{
		for(uVal=0;*pText>='0' && *pText<='9';pText++)
			{
			uVal *= 10;
			uVal += *pText-'0';
			}

		if(uVal>255)return 0;

		uAll |= uVal<<uSub;

		if(uSub == 3*8)break;
		if(*pText!='.')break;

		pText++;
		}


	if(!strwcharw(*pText,0x802000E0))
		{
		return 0;
		}


	uAddr = uAll;



return pText-pStart;
}


//*****************************************************************************
//*
//*		LicenseCheckIp
//*
//*****************************************************************************
//	Prft ob eine IP-Adresse in eine IP-Liste ist
//	pAddr		: Ist die die prfende Adresse als Text
//	uAddr		: Ist die die prfende Adresse
//	uCrc		: Ist die CRC-Summe fr die Datei
//	pListFile	: Ist der Name der Datei mit der II-Liste
//	Ergibt 0 wenn die Adresse ok ist oder eine Fehlernummer
//					0 wenn die Adresse gltig ist,
//					1 wenn die Adresse nicht in der Liste ist
//					2 wenn die Listendatei eine falsche CRC hat
//					3 wenn die Listendatei nicht gefunden wurde
//					4 wenn die IP-Adresse ungltig ist
int LicenseCheckIp(unsigned uAddr,unsigned uCrc,const char *pListFile)
{
FILE   *pFile;
int		iRet;


		pFile = fopen(pListFile,"rb");
	if(!pFile)
		{
		iLastError = 7;
		return 3;
		}

	iRet = LicenseCheckIp(uAddr,uCrc,pFile);

	fclose(pFile);


return iRet;
}

int LicenseCheckIp(unsigned uAddr,unsigned uCrc,const wchar_t *pListFile)
{
FILE   *pFile;
int		iRet;


		pFile = _wfopen(pListFile,L"rb");
	if(!pFile)
		{
		iLastError = 7;
		return 3;
		}

	iRet = LicenseCheckIp(uAddr,uCrc,pFile);

	fclose(pFile);


return iRet;
}

int LicenseCheckIp(const char *pAddr,unsigned uCrc,const char *pListFile)
{
FILE	   *pFile;
unsigned 	uAddr;
int			iRet;


	if(!LicenseTextToIpAddress(uAddr,pAddr))
		{
		iLastError = 8;
		return 4;
		}

		pFile = fopen(pListFile,"rb");
	if(!pFile)
		{
		iLastError = 7;
		return 3;
		}

	iRet = LicenseCheckIp(uAddr,uCrc,pFile);

	fclose(pFile);


return iRet;
}

int LicenseCheckIp(const wchar_t *pAddr,unsigned uCrc,const wchar_t *pListFile)
{
FILE	   *pFile;
unsigned 	uAddr;
int			iRet;


	if(!LicenseTextToIpAddress(uAddr,pAddr))
		{
		iLastError = 8;
		return 4;
		}

		pFile = _wfopen(pListFile,L"rb");
	if(!pFile)
		{
		iLastError = 7;
		return 3;
		}

	iRet = LicenseCheckIp(uAddr,uCrc,pFile);

	fclose(pFile);


return iRet;
}


//*****************************************************************************
//*
//*		LicenseCheckIp
//*
//*****************************************************************************
//	Prft ob eine IP-Adresse in eine IP-Liste ist
//	uAddr		: Ist die die prfende Adresse
//	uCrc		: Ist die CRC-Summe fr die Datei
//	pFile		: File-Handel mit der IP-Liste
//	Ergibt 0 wenn die Adresse ok ist oder eine Fehlernummer
//					0 wenn die Adresse gltig ist,
//					1 wenn die Adresse nicht in der Liste ist
//					2 wenn die Listendatei eine falsche CRC hat
int LicenseCheckIp(unsigned uAddr,unsigned uCrc,FILE *pFile)
{
char		cBuffer[256];
unsigned 	uFileCrc;
unsigned	uIpAddr;
unsigned	uSubNet;
unsigned	uPos;
unsigned	uLen;
unsigned	uRet;



	uRet		= 1;
	uFileCrc	= 0;
	iLastError  = 9;

	cBuffer[sizeof(cBuffer)-1] = 0;

	while(fgets(cBuffer,sizeof(cBuffer)-1,pFile))			// Datei Zeilenweise laden
		{
		uFileCrc = LicenseCrc(cBuffer,-1,uFileCrc);

		if(uRet==0)continue;


		uPos	 = strwlen(cBuffer,0xE0);
		uLen	 = LicenseTextToIpAddress(uIpAddr,cBuffer+uPos);

		if(!uLen)continue;

		uPos    += uLen;
		uPos	+= strwlen(cBuffer+uPos,0xE0);
		uLen	 = LicenseTextToIpAddress(uSubNet,cBuffer+uPos);

		if(uLen)											// Mit Subnet-Maske
			{
			if(((uAddr^uIpAddr)&uSubNet)==0)
				{
				iLastError	= 0;
				uRet		= 0;
				}
			}
		else{
			if(uAddr==uIpAddr)
				{
				iLastError	= 0;
				uRet		= 0;
				}
			}
		}

	if(uCrc!=uFileCrc)										// Passt die Datei-CRC
		{
		iLastError  = 10;
		uRet = 2;
		}


return uRet;
}

