//*****************************************************************************
//*
//*
//*		PrintfLogger.cpp
//*
//*
//*****************************************************************************
//
//	Diese Modul kann in Echtzeit Daten in einem Programm aufzeichen.
//	Dazu wird das Remote-Interface verwendet. Datenpakete werden in einem
//	Shared-Memory gespeichert, und von einem anderen Prozess ausgelesen.
//	Dieser Prozess kann die Datenpakete via TCP weiterleiten.
//	Copyright Anton Zechner (C) 2016
//
//	Als Empfnger kann das Terminal-Programm mit der ID 18227 benutzt werden.
//
//	Beispiel:
//
//
//		PrintfLoggerInit(1);
//
//		...
//		PrintfLogger("Output %i %i",123,222);
//		...
//
//		PrintfLoggerExit();
//

#include	"../System/RemoteInterface.h"
#include	"../PeFunctions.h"
#include	"PrintfLogger.h"
#include	<stdio.h>

#ifndef 	REMOTE_KEY
#define 	REMOTE_KEY			18227				// SHM Key-Nummer fr Remote-Interface
#endif

#ifndef 	REMOTE_MAX
#define 	REMOTE_MAX			0x10000				// Anzahl der Sendepakete
#endif

#ifndef 	REMOTE_SIZE
#define 	REMOTE_SIZE			128					// Gre der Pakete
#endif


static	RemoteHandle	hRemote = 0;				// Handle fr LOG-Schnittstelle

//*****************************************************************************
//*
//*		PrintfLoggerInit
//*
//*****************************************************************************
//	Initialisiert den Profile-Logger
//	iEnable		: Schnittstelle freigeben
PLOG_API void PrintfLoggerInit(int iEnable)
{

	if(hRemote)return;

	hRemote = RemoteInit(REMOTE_KEY,REMOTE_SIZE*REMOTE_MAX,0x1000,REMOTE_SIZE);

	if(iEnable)
		{
		RemoteEnable(hRemote);
		}


}

//*****************************************************************************
//*
//*		PrintfLoggerExit
//*
//*****************************************************************************
//	Initialisiert den Profile-Logger
PLOG_API void PrintfLoggerExit()
{

	RemoteExit(hRemote);
	hRemote = 0;

}

//*****************************************************************************
//*
//*		PrintfLogger
//*
//*****************************************************************************
//	Profiler Marke ausgeben
//	pFormat		: Format fr Ausgabe
//	iVal1..4	: Werte fr die Marke
PLOG_API void PrintfLogger(const char *pFormat,int iVal1,int iVal2,int iVal3,int iVal4)
{
char			cBuffer[1024];
static unsigned uLast=0;
unsigned		uTime;
unsigned 		uLen;

	
	if(!RemoteIsEnabled(hRemote))return;
	
	uTime = GetTickCount();
	uLen  = sprintf(cBuffer,"\n%05u ",(uTime-uLast)%10000);
	uLen += sprintf(cBuffer+uLen,pFormat,iVal1,iVal2,iVal3,iVal4);
	uLast = uTime;

	RemoteSend(hRemote,cBuffer,uLen);


}

