//*****************************************************************************
//*
//*		
//*		ProfileLogger.cpp	
//*		
//*
//*****************************************************************************
//
//	Diese Modul kann in Echtzeit Daten in einem Programm aufzeichen.
//	Dazu wird das Remote-Interface verwendet. Datenpakete werden in einem 
//	Shared-Memory gespeichert, und von einem anderen Prozess ausgelesen.
//	Dieser Prozess kann die Datenpakete via TCP weiterleiten.
//	Copyright Anton Zechner (C) 2010
//
//	Als Empfnger kann das Terminal-Programm mit der ID 18227 benutzt werden.
//
//	Beispiel:
//
//
//		ProfileLoggerInit(PLOG_ALL);
//	
//		for(;;)
//			{
//			ProfileLoggerMark("MyMark");
//			iRet = Function();
//			ProfileLoggerMark("MyMark out",iRet);
//			}
//	
//		ProfileLoggerExit();
//

#include	"../System/RemoteInterface.h"
#include	"../PeFunctions.h"
#include	"ProfileLogger.h"
#include	<stdio.h>

#ifndef 	REMOTE_KEY
#define 	REMOTE_KEY			18227				// SHM Key-Nummer fr Remote-Interface		
#endif 	

#ifndef 	REMOTE_MAX
#define 	REMOTE_MAX			0x20000				// Anzahl der Sendepakete
#endif 	

#ifndef 	REMOTE_SIZE
#define 	REMOTE_SIZE			64					// Gre der Pakete
#endif 	



#define		DefineProc(proc)			static Type##proc p##proc
#define		GetProcAddr(dll,proc)		p##proc = (Type##proc)GetProcAddress(GetModuleHandle(#dll),#proc)
#define		ChangeProc(where,dll,proc)	{																	\
										if(where)															\
												SetImportProc(where,#dll,#proc,New##proc,(void**)&p##proc);	\
										else	SetImportProc(      #dll,#proc,New##proc,(void**)&p##proc);	\
										}


typedef	VOID  (_stdcall *TypeSleep                   )(DWORD);
typedef	DWORD (_stdcall *TypeSleepEx                 )(DWORD,BOOL);
typedef	DWORD (_stdcall *TypeWaitForInputIdle        )(HANDLE,DWORD);
typedef	DWORD (_stdcall *TypeWaitForSingleObject     )(HANDLE,DWORD);
typedef	DWORD (_stdcall *TypeWaitForSingleObject     )(HANDLE,DWORD);
typedef	DWORD (_stdcall *TypeWaitForSingleObjectEx   )(HANDLE,DWORD,BOOL);
typedef	DWORD (_stdcall *TypeWaitForMultipleObjects  )(DWORD,HANDLE*,BOOL,DWORD);
typedef	DWORD (_stdcall *TypeWaitForMultipleObjectsEx)(DWORD,HANDLE*,BOOL,DWORD,BOOL);


DefineProc(Sleep					);
DefineProc(SleepEx                 	);
DefineProc(WaitForInputIdle        	);
DefineProc(WaitForSingleObject     	);
DefineProc(WaitForSingleObjectEx   	);
DefineProc(WaitForMultipleObjects  	);
DefineProc(WaitForMultipleObjectsEx	);

static	int					iInitDone	= 0;
static	RemoteHandle		hRemote		= 0;
static	CRITICAL_SECTION	sCritical;

//*****************************************************************************
//*
//*		NewSleep
//*
//*****************************************************************************
static VOID _stdcall NewSleep(DWORD dwMilliSeconds)
{
static	unsigned last=1;


	if(!dwMilliSeconds && !last)
		{
		pSleep(dwMilliSeconds);
		return;
		}
		

	ProfileLoggerMark("Sleep",dwMilliSeconds);
	pSleep(dwMilliSeconds);
	ProfileLoggerMark("Sleep out");
	last = dwMilliSeconds;
}

//*****************************************************************************
//*
//*		NewSleepEx
//*
//*****************************************************************************
static DWORD _stdcall NewSleepEx(DWORD dwMilliSeconds,BOOL bAlertable)
{
DWORD	uRet;

	ProfileLoggerMark("SleepEx",dwMilliSeconds,bAlertable);
	uRet = pSleepEx(dwMilliSeconds,bAlertable);
	ProfileLoggerMark("SleepEx out =",uRet);


return uRet;
}

//*****************************************************************************
//*
//*		NewWaitForInputIdle
//*
//*****************************************************************************
static DWORD _stdcall NewWaitForInputIdle(HANDLE hProcess,DWORD dwMilliSeconds)
{
DWORD	uRet; 

	ProfileLoggerMark("WaitForInputIdle",dwMilliSeconds);
	uRet = pWaitForInputIdle(hProcess,dwMilliSeconds);
	ProfileLoggerMark("WaitForInputIdle out",uRet);


return uRet;
}

//*****************************************************************************
//*
//*		NewWaitForSingleObject
//*
//*****************************************************************************
static DWORD _stdcall NewWaitForSingleObject(HANDLE hHandle,DWORD dwMilliSeconds)
{
DWORD	uRet; 

	ProfileLoggerMark("WaitForSingleObject",dwMilliSeconds);
	uRet = pWaitForSingleObject(hHandle,dwMilliSeconds);
	ProfileLoggerMark("WaitForSingleObject out ",uRet);

	
return uRet;
}

//*****************************************************************************
//*
//*		NewWaitForSingleObjectEx
//*
//*****************************************************************************
static DWORD _stdcall NewWaitForSingleObjectEx(HANDLE hHandle,DWORD dwMilliSeconds,BOOL bAlertable)
{
DWORD	uRet; 


	ProfileLoggerMark("WaitForSingleObjectEx",dwMilliSeconds);
	uRet = pWaitForSingleObjectEx(hHandle,dwMilliSeconds,bAlertable);
	ProfileLoggerMark("WaitForSingleObjectEx out",uRet);


return uRet;
}

//*****************************************************************************
//*
//*		NewWaitForMultipleObjects
//*
//*****************************************************************************
static DWORD _stdcall NewWaitForMultipleObjects(DWORD nCount,HANDLE *pHandles,BOOL bWaitAll,DWORD dwMilliSeconds)
{
DWORD	uRet; 

	ProfileLoggerMark("WaitForMultipleObjects",dwMilliSeconds);
	uRet = pWaitForMultipleObjects(nCount,pHandles,bWaitAll,dwMilliSeconds);
	ProfileLoggerMark("WaitForMultipleObjects out",uRet);
	

return uRet;
}


//*****************************************************************************
//*
//*		NewWaitForMultipleObjectsEx
//*
//*****************************************************************************
static DWORD _stdcall NewWaitForMultipleObjectsEx(DWORD nCount,HANDLE *pHandles,BOOL bWaitAll,DWORD dwMilliSeconds,BOOL bAlertable)
{
DWORD	uRet; 

	ProfileLoggerMark("WaitForMultipleObjectsEx",dwMilliSeconds);
	uRet = pWaitForMultipleObjectsEx(nCount,pHandles,bWaitAll,dwMilliSeconds,bAlertable);
	ProfileLoggerMark("WaitForMultipleObjectsEx out",uRet);

return uRet;
}


//*****************************************************************************
//*
//*		ProfileLoggerInit
//*
//*****************************************************************************
//	Initialisiert den Profile-Logger 
//	iMode		: Was soll getraced werden:
//					PLOG_SLEEP		0x00000001	Sleep tracen
//					PLOG_WAIT		0x00000002	WaitForSingleObject/WaitForMultipleObjects... tracen
//					PLOG_ALL		0x000000FF	Alle Funktionen tracen
//					PLOG_START		0x00000100	Aufzeichnung sofort starten
//	pModule		: Ist der Name der oder 0 fr alles
PLOG_API void ProfileLoggerInit(int iMode,const char *pModule)
{

	if(iInitDone)return;

	iInitDone = 1;

	InitializeCriticalSection(&sCritical);

	GetProcAddr(KERNEL32,Sleep						);
	GetProcAddr(KERNEL32,SleepEx                 	);
//	GetProcAddr(KERNEL32,WaitForInputIdle        	);
	GetProcAddr(KERNEL32,WaitForSingleObject     	);
	GetProcAddr(KERNEL32,WaitForSingleObjectEx   	);
	GetProcAddr(KERNEL32,WaitForMultipleObjects  	);
	GetProcAddr(KERNEL32,WaitForMultipleObjectsEx	);

	
	if(iMode&PLOG_SLEEP)
		{
		ChangeProc(pModule,KERNEL32,Sleep);
		ChangeProc(pModule,KERNEL32,SleepEx);
		}

	if(iMode&PLOG_WAIT)
		{
//		ChangeProc(pModule,KERNEL32,WaitForInputIdle        	);
		ChangeProc(pModule,KERNEL32,WaitForSingleObject     	);
		ChangeProc(pModule,KERNEL32,WaitForSingleObjectEx   	);
		ChangeProc(pModule,KERNEL32,WaitForMultipleObjects  	);
		ChangeProc(pModule,KERNEL32,WaitForMultipleObjectsEx	);
		}

	hRemote = RemoteInit(REMOTE_KEY,REMOTE_SIZE*REMOTE_MAX,0x1000,REMOTE_SIZE);
	
	if(iMode&PLOG_START)
		{
		RemoteEnable(hRemote);
		}


}

//*****************************************************************************
//*
//*		ProfileLoggerExit
//*
//*****************************************************************************
//	Initialisiert den Profile-Logger 
PLOG_API void ProfileLoggerExit()
{

	RemoteExit(hRemote);
	hRemote = 0;

}

//*****************************************************************************
//*
//*		ProfileLoggerMark
//*
//*****************************************************************************
//	Profiler Marke ausgeben
//	pName		: Name der Marke
//	iVal1..4	: Werte fr die Marke
PLOG_API void ProfileLoggerMark(const char *pName,int iVal1,int iVal2,int iVal3,int iVal4)
{
char	cBuffer[1024];
UINT	uThread;
UINT	uLen;


	

	
	uThread = GetCurrentThreadId();

	if(iVal4!=PLOG_NONE)
		{
		uLen = sprintf(cBuffer,"%04X \"%s\"  %i %i %i %i",uThread,pName,iVal1,iVal2,iVal3,iVal4);
		}
	else if(iVal3!=PLOG_NONE)
		{
		uLen = sprintf(cBuffer,"%04X \"%s\"  %i %i %i",uThread,pName,iVal1,iVal2,iVal3);
		}
	else if(iVal2!=PLOG_NONE)
		{
		uLen = sprintf(cBuffer,"%04X \"%s\"  %i %i",uThread,pName,iVal1,iVal2);
		}
	else if(iVal1!=PLOG_NONE)
		{
		uLen = sprintf(cBuffer,"%04X \"%s\"  %i",uThread,pName,iVal1);
		}
	else{
		uLen = sprintf(cBuffer,"%04X \"%s\"",uThread,pName);
		}

	EnterCriticalSection(&sCritical);

	RemoteSend(hRemote,cBuffer,uLen,uThread);

	LeaveCriticalSection(&sCritical);


}

