//*****************************************************************************
//*                                                                          
//*                                                                          
//*      MapFile.h                                                     
//*                                                                          
//*                                                                          
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2007
//
//
//	Die Klasse CMapFile erzeugt bzw. ffnet ein MemoryMappedFile
//	Die Klasse ist fr UNICODE und ANSI geeignet.
//	
//	Funktionen:		char *Open(...)		Datei ffnen
//					char *Map (...)		Mapping einstellen
//					char *GetMem()		Speicherzeiger
//					int	  GetLength()	Speicherlnge
//					int	  Close()		Datei schliesen
//					int	  Unmap()		Mapping aufheben
//
   
#ifndef	__MAPFILE_H__
#define __MAPFILE_H__

	  
#define	CMF_READ 		0x0000				// Lesen erlaubt
#define	CMF_WRITE		0x0001				// Schreiben erlaubt
#define	CMF_NOMAP		0x0002				// Kein automatisches Mapping
#define	CMF_SHAREDWRITE	0x0005				// Mehrfach Schreiben erlaubt
#define	CMF_NOSHARE		0x0008				// Kein Datei-Sharing
#define	CMF_CREATE		0x0011				// Ein vorhandenes Objekt ffnen oder die Datei erzeugen
#define	CMF_NEW			0x0021				// Eine Datei erzeugen und auf 0 setzen


#ifndef _WCHAR_T_DEFINED
	#if _MSC_VER 
	#include	<wchar.h>
	#else
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif 	
#endif 	



//*****************************************************************************
//*                                                                          
//*    CMapFile                                                     
//*                                                                          
//*****************************************************************************
class CMapFile
{
public:


	~CMapFile();														
	 CMapFile();														
	 CMapFile(const char    *pFilename,int iMode=CMF_READ,const char     *pName=0,unsigned uSize=0);
	 CMapFile(const wchar_t *pFilename,int iMode=CMF_READ,const wchar_t  *pName=0,unsigned uSize=0);	
	 CMapFile(int iMode,const char    *pName,int iLen);					
	 CMapFile(int iMode,const wchar_t *pName,int iLen);					

	 int   Close  ();													// Datei schlieen
	 int   Unmap  ();													// Datei unmappen
	 char *Open   (const char    *pFile,          int iMode=CMF_READ,const char    *pName=0,unsigned uSize=0);	
	 char *Open   (const wchar_t *pFile,          int iMode=CMF_READ,const wchar_t *pName=0,unsigned uSize=0);
	 char *Open   (int iMode,const char    *pName,int iLen=0);			// Datei ffnen 
	 char *Open   (int iMode,const wchar_t *pName,int iLen=0);			// Datei ffnen 
	 char *Map	  (int iOffset=0,int iLen=0);							// Datei mappen
	 char *SetSize(int iLen);											// Gre einstellen
	 

protected:
	
	int				iFileLength;										// Lnge des Files
	int				iOpenMode;											// Open Modus
	wchar_t        *pObjName;											// Zeiger auf Objektnamem
	char		   *pMapMem;											// Zeiger auf Speicher 
	int				iMapLen;											// Lnge des Speichers
	int				iMapOff;											// Offset des Speichers
	void		   *hMapObj;											// Memorymaped Objekt
	void		   *hMapFile;											// Memorymaped File


public:
	
	inline char *GetMem	   (){return pMapMem;}							// Speicherbereichs-Zeiger
	inline int   GetLength (){return iMapLen;}							// Speicherbereichs-Lnge 
	inline char *Open      (const char    *pFile,int *pLen,int iMode=CMF_READ){char *pMem=Open(pFile,iMode);*pLen=GetLength();return pMem;};
	inline char *Open      (const wchar_t *pFile,int *pLen,int iMode=CMF_READ){char *pMem=Open(pFile,iMode);*pLen=GetLength();return pMem;};
																		
};



#endif

