//*****************************************************************************
//*
//*
//*		MemFunctions.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//
//	Funktionen:
//
//		int   memgcmp			Vergleicht zwei Speicherbereiche
//  	int   memfcmp			Vergleicht zwei Speicherbereiche (mit ? )
//  	int   memxcmp			Vergleicht zwei Speicherbereiche (mit ? und *)
//  	int   memycmp			Vergleicht zwei Speicherbereiche (mit ? und * /...)
//  	int   memmcmp			Vergleicht zwei Speicherbereiche  mit einer Maske
//		int	  meminsert			Einen Speicherbereich in einen anderen Speicherbereich einfgen	
//		int	  memtofind			Rename-pFormat in Find-pFormat konvertieren
//		int   memrename			Einen String laut pFormat umbenennen
//  	void  memxchg			Zwei Speicherbereiche austauschen
//		char *memupper			In iModebuchstaben  umwandeln
//		char *memlower			In Kleinbuchstaben umwandeln
//  	char *memcfg			iMode/Kleinscgreibung einstellen
//		int   memtobase64		Speicherbereich in Base64 pText konvertieren
//		int   base64tomem		Base64 pText in Speicherbereich konvertieren
//		int   memtomask			Erzeugt einen Maken-Hex-Dump aus einem Speicherbereich und einer Maske.
//		int   memtohex			Erzeugt aus einen Speicherbereich eine HEX-Zeile.
//		int   memtodump			Erzeugt einen Hex-Dump aus einem Speicherbereich.
//		int	  dumptomem			Wandelt einen Hex-Dump in einen Speicherbereich um.
//
//		#define NO_ASM			Ausschalten der Assembler-Optimierung

#ifndef	__MEMFUNCTIONS_H__
#define __MEMFUNCTIONS_H__

#if _MSC_VER >= 1000
#pragma once
#endif  

#include	<string.h>
#include	<memory.h>

#ifndef _WCHAR_T_DEFINED
	#if _MSC_VER
	#include	<wchar.h>
	#else
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif
#endif

#ifndef	   _MSC_VER						// Assembler verwenden ?
#ifndef		NO_MEM_ASM
#define		NO_MEM_ASM
#endif
#endif

#ifndef	   _WIN32
#ifndef		NO_MEM_ASM
#define		NO_MEM_ASM
#endif
#endif

#ifdef	   _WIN64
#ifndef		NO_MEM_ASM
#define		NO_MEM_ASM
#endif
#endif

#ifdef		NO_ASM
#ifndef		NO_MEM_ASM
#define		NO_MEM_ASM
#endif
#endif

// Konstanten fr memtodump dumptomem und htostr 

#define		HEX_NULL		0x80000000			// Null Byte anhngen
#define		HEX_INVERT		0x40000000			// Reihenfolge um drehen
#define		HEX_DPOINT		0x20000000			// Einen Dopelpunkt am Anfang
#define		HEX_ENDCUT		0x10000000			// Keinen Beistrich nach dem letzten Byte
#define 	HEX_ASCII		0x10000000			// Auch ASCII Zeichen erlauben
#define		HEX_DPBIN		0x01000000			// Dopelpunkt als binres Trenzeichen
#define		HEX_BYTE		0x00000000			// Immer Bytes ausgeben (00 11 22 33 44...)
#define		HEX_WORD		0x02000000			// Immer Words ausgeben (0000 1111 2222...)
#define		HEX_DWORD		0x04000000			// Immer DWords ausgeben (00000000 11111111...)
#define		HEX_QWORD		0x06000000			// Immer QWords ausgeben (0011223344556677...)
#define 	HEX_SPACEONLY	0x08000000			// Beim extra Leerzeichen nur Space einfgen
#define		HEX_COMMA_OK	0x00080000			// Ein Beistrich zwischen jedem Byte ist erlaubt
#define		HEX_POINT_OK	0x00040000			// Ein Punkt zwischen jedem Byte ist erlaubt
#define		HEX_MINUS_OK	0x00020000			// Ein Minus zwischen jedem Byte ist erlaubt
#define		HEX_DOUBLE_OK	0x00010000			// Ein Doppelpunkt zwischen jedem Byte ist erlaubt
#define		HEX_NSPACE		0x000000FF			// Extra Leerzeichen alle N-Byte
#define		HEX_SPACE		(' '<<8)			// Ein Leerzeichen nach jedem Byte
#define		HEX_COMMA 		(','<<8)			// Einen Beistrich nach jedem Byte
#define		HEX_POINT 		('.'<<8)			// Einen Punkt nach jedem Byte
#define		HEX_MINUS 		('-'<<8)			// Eine  Minus nach jedem Byte
#define		HEX_DOFILL		0x40000000			// Einen Hexdump auffllen
#define 	HEX_ZERO		HEX_DOFILL			// Mit 0'en auffllen	
#define 	HEX_FILL(a)		((((a)&0xFF)<<20)|HEX_DOFILL)	



#ifndef	intq
#if defined(LINUX) || defined(linux)
#define intq		long long
#else
#define intq		__int64
#endif
#endif

typedef struct
	{
	const wchar_t *pToUpperCase;
	const wchar_t *pToLowerCase;
	}CaseConfig;

extern	CaseConfig	sUniCaseData;
#define	UNICASE	  (&sUniCaseData)

extern	const wchar_t *pToUpperCase;	// Vergleichs Buchstaben fr
extern	const wchar_t *pToLowerCase;	// Gro/Kleinschreibung


//******************** Algemeine Funktionen ***********************************


	void        _cdecl memsetw    (      void *pBuffer,int  iVal,unsigned uCount);
	void        _cdecl memsetdw   (      void *pBuffer,int  iVal,unsigned uCount);
	void        _cdecl memsetqw   (      void *pBuffer,intq iVal,unsigned uCount);

	int         _cdecl memcmpw    (const void *pBuffer1,const void *pBuffer2,unsigned uCount);
	int         _cdecl memicmpw   (const void *pBuffer1,const void *pBuffer2,unsigned uCount);

	void 		_cdecl memcfg     (CaseConfig *pConfig);
	void        _cdecl memxchg    (      void *pBuffer1,      void *pBuffer2,                   int iCount);
	int         _cdecl memmcmp    (const void *pBuffer1,const void *pBuffer2,const void *pMaske,int iCount);

	int         _cdecl memreplace (      void *pBuffer1,const void *pBuffer2,int iSize,int iCount,int iOld,int iOffset=0);
	int         _cdecl memreplace (      void *pBuffer1,const void *pBuffer2,int iSize,int iCount,int iOld,int iOffset,int iMax);
	int         _cdecl meminsert  (      void *pBuffer1,const void *pBuffer2,int iSize,int iCount         ,int iOffset=0);
	int         _cdecl meminsert  (      void *pBuffer1,const void *pBuffer2,int iSize,int iCount         ,int iOffset,int iMax);
	int         _cdecl memslide   (      void *pBuffer1,                     int iFrom,int iCount         ,int iOffset=0);
	int         _cdecl memslide   (      void *pBuffer1,                     int iFrom,int iCount         ,int iOffset,int iMax);
	int         _cdecl memremove  (      void *pBuffer1                     ,int iSize,int iCount         ,int iOffset=0);


//******************** ANSI ***************************************************


	char *      _cdecl memupper   (char *pBuffer,int iLen);
	char *      _cdecl memupper   (char *pBuffer);

	char *      _cdecl memlower   (char *pBuffer,int iLen);
	char *      _cdecl memlower   (char *pBuffer);

	int         _cdecl memgcmp    (const char *pBuffer1,const char *pBuffer2,int iCount,int iMode);
	int         _cdecl memgcmp    (const char *pBuffer1,const char *pBuffer2,int iCount);
	int         _cdecl memgcmp    (const char *pBuffer1,const char *pBuffer2);

	int         _cdecl memfcmp    (const char *pBuffer1,const char *pBuffer2,int iCount);
	int         _cdecl memfcmp    (const char *pBuffer1,const char *pBuffer2,int iCount,int iMode);

	int	        _cdecl memxcmp    (const char *pFind,int iFindiLen,const char *pText ,int iTextLen,int iMode=0);
	int	        _cdecl memxcmp    (const char *pFind,              const char *pText ,int iTextLen,int iMode=0);
	int	        _cdecl memxcmp    (const char *pFind,int iFindiLen,const char *pText );
	int	        _cdecl memxcmp    (const char *pFind,              const char *pText );

	int         _cdecl memycmp    (const char *pFind,int iFindiLen,const char *pText ,int iTextLen,int iMode=0);
	int         _cdecl memycmp    (const char *pFind,              const char *pText ,int iTextLen,int iMode=0);
	int         _cdecl memycmp    (const char *pFind,int iFindiLen,const char *pText );
	int         _cdecl memycmp    (const char *pFind,              const char *pText );

	int			_cdecl memfilter  (const char *pText              ,const char *pFilter,int iMode=0,char cSign=';');
	int			_cdecl memfilter  (const char *pText,int iSize    ,const char *pFilter,int iMode=0,char cSign=';');

	int         _cdecl memtofind  (const char *pFormat,char *pTarget,int iLen,int iMode=0);
	int         _cdecl memrename  (const char *pFormat,int iSizeQ,const char *pSource,char *pTarget,int iLen,int iMode=0);
	int	        _cdecl memrename  (const char *pFormat,           const char *pSource,char *pTarget,int iLen,int iMode=0);

	int         _cdecl memtobase64(const void *pMem,int iSize,int *pRest,      char *pTarget,int iLineBreak=64);
	int         _cdecl memtobase64(const void *pMem,int iSize,                 char *pTarget,int iLineBreak=64);
	int			_cdecl base64tomem(      void *pMem,int iSize,int *pRest,const char *pBase64,int iLength=0x10000000);
	int         _cdecl base64tomem(      void *pMem,int iSize,           const char *pBase64,int iLength=0x10000000);
	int			_cdecl base64size (                                      const char *pBase64,int iLength=0x10000000);
	int	        _cdecl dumptomem  (      void *pMem,               int iMax,const char *pStr,int *pCount=0);
	int         _cdecl masktomem  (      void *pMem,void *pMask   ,int iMax,const char *pStr,int *pCount=0);
	int         _cdecl htostr     (const void *pHex,char *pTarget ,int iCount,int iMode=HEX_SPACE|HEX_NULL|4);
	int         _cdecl memtohex   (      char *pStr,unsigned uAddr,const void *pMem,int iSize,int iLine=16,int iMode=0x01220404);
	int         _cdecl memtodump  (      char *pStr,               const void *pMem,int iLen,int iMode=HEX_SPACE|HEX_NULL);
	int         _cdecl memtomask  (      char *pStr,int iMax,      const void *pMem,const void *pMask,int iCount,int iMode=HEX_SPACE|HEX_NULL);


	#ifndef _DEBUG

	inline int  _cdecl memxcmp    (const char *pFind,              const char *pText ,int iTextLen,int iMode){return memxcmp(pFind,(int)strlen(pFind),pText, iTextLen     ,iMode);}
	inline int  _cdecl memxcmp    (const char *pFind,int iFindiLen,const char *pText                        ){return memxcmp(pFind,iFindiLen	     ,pText,(int)strlen(pText),0);}
	inline int  _cdecl memxcmp    (const char *pFind,              const char *pText			            ){return memxcmp(pFind,(int)strlen(pFind),pText,(int)strlen(pText),0);}

	inline int  _cdecl memycmp    (const char *pFind,              const char *pText ,int iTextLen,int iMode){return memycmp(pFind,(int)strlen(pFind),pText, iTextLen     ,iMode);}
	inline int  _cdecl memycmp    (const char *pFind,int iFindiLen,const char *pText                        ){return memycmp(pFind,iFindiLen	     ,pText,(int)strlen(pText),0);}
	inline int  _cdecl memycmp    (const char *pFind,              const char *pText			            ){return memycmp(pFind,(int)strlen(pFind),pText,(int)strlen(pText),0);}

	inline int  _cdecl memfilter  (const char *pText  ,const char *pFilter,int iMode,char cSign){return memfilter(pText,(int)strlen(pText),pFilter,iMode,cSign);}
	inline int	_cdecl memrename  (const char *pFormat,const char *pSource,char *pTarget,int iLen,int iMode){return memrename(pFormat,(int)strlen(pSource),pSource,pTarget,iLen,iMode);}
	inline int  _cdecl memtodump  (      char *pStr   ,const void *pMem,int iLen,int iMode){return htostr((const char*)pMem,pStr,iLen,iMode);}

	#endif


//******************** UNICODE ************************************************


	wchar_t *   _cdecl memupper   (      wchar_t *pBuffer,int iLen);
	wchar_t *   _cdecl memupper   (      wchar_t *pBuffer);

	wchar_t *   _cdecl memlower   (      wchar_t *pBuffer,int iLen);
	wchar_t *   _cdecl memlower	  (      wchar_t *pBuffer);

	int         _cdecl memgcmp    (const wchar_t *pBuffer1,const wchar_t *pBuffer2,int iCount,int iMode);
	int         _cdecl memgcmp    (const wchar_t *pBuffer1,const wchar_t *pBuffer2,int iCount);
	int         _cdecl memgcmp    (const wchar_t *pBuffer1,const wchar_t *pBuffer2);

	int         _cdecl memfcmp    (const wchar_t *pBuffer1,const wchar_t *pBuffer2,int iCount);
	int         _cdecl memfcmp    (const wchar_t *pBuffer1,const wchar_t *pBuffer2,int iCount,int iMode);

	int	        _cdecl memxcmp    (const wchar_t *pFind,int iFindiLen,const wchar_t *pText ,int iTextLen,int iMode=0);
	int	        _cdecl memxcmp    (const wchar_t *pFind,              const wchar_t *pText ,int iTextLen,int iMode=0);
	int	        _cdecl memxcmp    (const wchar_t *pFind,int iFindiLen,const wchar_t *pText );
	int	        _cdecl memxcmp    (const wchar_t *pFind,              const wchar_t *pText );

	int         _cdecl memycmp    (const wchar_t *pFind,int iFindiLen,const wchar_t *pText ,int iTextLen,int iMode=0);
	int         _cdecl memycmp    (const wchar_t *pFind,              const wchar_t *pText ,int iTextLen,int iMode=0);
	int         _cdecl memycmp    (const wchar_t *pFind,int iFindiLen,const wchar_t *pText );
	int         _cdecl memycmp    (const wchar_t *pFind,              const wchar_t *pText );

	int			_cdecl memfilter  (const wchar_t *pText              ,const wchar_t *pFilter,int iMode=0,wchar_t cSign=';');
	int			_cdecl memfilter  (const wchar_t *pText,int iSize    ,const wchar_t *pFilter,int iMode=0,wchar_t cSign=';');

	int         _cdecl memtofind  (const wchar_t *pFormat                                   ,wchar_t *pTarget,int iLen,int iMode=0);
	int         _cdecl memrename  (const wchar_t *pFormat,int iSizeQ ,const wchar_t *pSource,wchar_t *pTarget,int iLen,int iMode=0);
	int	        _cdecl memrename  (const wchar_t *pFormat,            const wchar_t *pSource,wchar_t *pTarget,int iLen,int iMode=0);

	int         _cdecl memtobase64(const    void  *pMem,int iSize,int *pRest,      wchar_t *pTarget,int iLineBreak=64);
	int         _cdecl memtobase64(const    void  *pMem,int iSize           ,      wchar_t *pTarget,int iLineBreak=64);
	int			_cdecl base64tomem(         void  *pMem,int iSize,int *pRest,const wchar_t *pBase64,int iLength=0x10000000);
	int         _cdecl base64tomem(         void  *pMem,int iSize           ,const wchar_t *pBase64,int iLength=0x10000000);
	int			_cdecl base64size (                                          const wchar_t *pBase64,int iLength=0x10000000);
	int	        _cdecl dumptomem  (         void  *pMem,            int iMax,const wchar_t *pStr,int *pCount=0);
	int         _cdecl masktomem  (         void  *pMem,void *pMask,int iMax,const wchar_t *pStr,int *pCount=0);
	int         _cdecl htostr     (const    void  *pHex,wchar_t *pTarget    ,int iCount,int iMode=HEX_SPACE|HEX_NULL|4);
	int         _cdecl memtohex   (       wchar_t *pStr,unsigned uAddr,const void *pMem,int iSize,int iLine=16,int iMode=0x01220404);
	int         _cdecl memtodump  (       wchar_t *pStr,               const void *pMem,int iLen,int iMode=HEX_SPACE|HEX_NULL);
	int         _cdecl memtomask  (       wchar_t *pStr,int iMax,      const void *pMem,const void *pMask,int iCount,int iMode=HEX_SPACE|HEX_NULL);

	#ifndef _DEBUG

	inline int  _cdecl memxcmp    (const wchar_t *pFind,              const wchar_t *pText ,int iTextLen,int iMode){return memxcmp(pFind,(int)wcslen(pFind),pText, iTextLen        ,iMode);}
	inline int  _cdecl memxcmp    (const wchar_t *pFind,int iFindiLen,const wchar_t *pText                        ){return memxcmp(pFind,iFindiLen	       ,pText,(int)wcslen(pText),0   );}
	inline int  _cdecl memxcmp    (const wchar_t *pFind,              const wchar_t *pText			              ){return memxcmp(pFind,(int)wcslen(pFind),pText,(int)wcslen(pText),0   );}

	inline int  _cdecl memycmp    (const wchar_t *pFind,              const wchar_t *pText ,int iTextLen,int iMode){return memycmp(pFind,(int)wcslen(pFind),pText, iTextLen        ,iMode);}
	inline int  _cdecl memycmp    (const wchar_t *pFind,int iFindiLen,const wchar_t *pText                        ){return memycmp(pFind,iFindiLen	       ,pText,(int)wcslen(pText),0   );}
	inline int  _cdecl memycmp    (const wchar_t *pFind,              const wchar_t *pText			              ){return memycmp(pFind,(int)wcslen(pFind),pText,(int)wcslen(pText),0   );}

	inline int  _cdecl memfilter  (const wchar_t *pText  ,const wchar_t *pFilter,int iMode,wchar_t  cSign){return memfilter(pText,(int)wcslen(pText),pFilter,iMode,cSign);}
	inline int	_cdecl memrename  (const wchar_t *pFormat,const wchar_t *pSource,wchar_t *pTarget,int iLen,int iMode){return memrename(pFormat,(int)wcslen(pSource),pSource,pTarget,iLen,iMode);}
	inline int  _cdecl memtodump  (      wchar_t *pStr   ,const void    *pMem,int iLen,int iMode){return htostr((const char*)pMem,pStr,iLen,iMode);}

	#endif



#endif

 