//*****************************************************************************
//*
//*
//*		NetFunctions.h
//*
//*
//*****************************************************************************
//	
//	Mit diesen Funktionen knne die IP-Einstellungen des Rechners gendert werden.
//
//	z.B.:
//
//			NetCardInfo *pInfo;
//			int          i;
//	
//			NetInit();
//			NetUpdateCards();
//	
//			for(i=0;;i++)								// Karten auflisten
//				{
//				pInfo = NetGetCardInfo(i);
//				if(!pInfo)break;
//				printf("%s -> %s\n",pInfo->cDesciption,pInfo->cFirstIp);
//				}
//			
//			pInfo = NetGetCardInfo(0);					// Karte [0] ndern
//	
//			NetSetIpAddress(pInfo->cMac,"192.168.0.100\0","255.255.255.0\0",3);
//			
//			NetExit();
//	
//
#ifndef 	__NETFUNCTIONS_H__
#define 	__NETFUNCTIONS_H__

#include	<tchar.h>


typedef struct
	{
	TCHAR		cDesciption[128];						// Beschreibung fr die Karte
	TCHAR		cInfoName  [64];						// Ist der Namen der im Exporer sichtbar ist
	TCHAR		cName      [64];						// Ist der Gertenamen "{CF879B1D-3C21-442E-A353-9E5FC581A2B6}"
	TCHAR		cGateway   [64];						// Ist der Gateway fr die Karte
	TCHAR		cFirstIp   [24];						// Ist die erste IP-Adresse als Text
	TCHAR		cNameSrv1  [64];						// Ist die IP-Adresse des ersten Name-Servers
	TCHAR		cNameSrv2  [64];						// Ist die IP-Adresse des zweiten Name-Servers
	TCHAR		cMac       [24];						// Ist die MAC-Adresse als Text "00-11-22-33-44-55"
	BYTE		aMac       [8];							// Ist die MAC-Adresse binr 
	DWORD		uIpCount;								// Anzahl der zugeordneten IP-Adressen
	DWORD		uIndex;									// Ist der Gerte Index
	DWORD		uNameSrv1;								// Ist die IP-Adresse des zweiten Name-Servers binr
	DWORD		uNameSrv2;								// Ist die IP-Adresse des zweiten Name-Servers binr
	DWORD		uGate;									// Ist die Gateway IP-Adresse binr 
	DWORD		uAddr;									// Ist die erste IP-Adresse binr 
	DWORD		uDhcp;									// Ist DHCP freigegeben
	}NetCardInfo;


void			NetInit();
void			NetExit();
unsigned		NetUpdateCards     ();
NetCardInfo    *NetGetCardInfo     (unsigned uCard);     
unsigned    	NetGetSubNet       (unsigned uAddr);
unsigned     	NetIsLocalNet	   (unsigned uAddr,LPTSTR  pInterface);
unsigned    	NetTextToIpAddress (DWORD   &uAddr,LPCTSTR pText,DWORD *pSubNet=0);
unsigned    	NetIpAddressToText (unsigned uAddr,LPTSTR  pText);
unsigned		NetCheckIpAddress  (LPCTSTR  pIpAddr,unsigned uIsMask=0);
int         	NetSetIpDefault	   (LPCTSTR  pInterface);
int         	NetSetGateway      (LPCTSTR  pInterface,LPCTSTR pIpAddr);
int         	NetSetNameServer   (LPCTSTR  pInterface,LPCTSTR pIpAddr1,LPCTSTR pIpAddr2=0);
int         	NetSetIpAddress    (LPCTSTR  pInterface,LPCTSTR pIpAddr,LPCTSTR pSubNet,DWORD uFlags);
int         	NetDelIpAddress    (LPCTSTR  pInterface,LPCTSTR pIpAddr,DWORD   uFlags);
int         	NetGetIpAddresses  (LPCTSTR  pInterface,DWORD *pList,int iCount,DWORD uFlags=1);
int         	NetGetPackets      (LPCTSTR  pInterface,DWORD &uOut,DWORD &uInp,DWORD &uOutByte,DWORD &uInpByte);
int         	NetGetLinkSpeed    (LPCTSTR  pInterface);
int         	NetGetConnectStatus(LPCTSTR  pInterface);
											 



#endif 	
