//*****************************************************************************
//*
//*
//*		PdfTools.cpp
//*
//*
//*****************************************************************************
#include	<StringFunctions.h>
#include	<FileFunctions.h>
#include	<MemFunctions.h>
#include	<FileFinder.h>
#include	<ProcPrintf.h>
#include	<DataField.h>
#include	<MapFile.h>
#include	<string.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<math.h>

#include	"PdfTools.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	MAP_MAX			256
#define 	LINE_MAX		100
#define 	ENTRIES(a)		(sizeof(a)/sizeof(a[0]))
#ifndef 	PDF_DLL_PATH
#define 	PDF_DLL_PATH	""
#endif

typedef	int (_cdecl *PdfProcType)(int iArgc,char **pArgv);
typedef struct
	{
	int		iSize;											// Gre der Daten
	int		iStart;											// Beginn der letzten Zeile
	int		iOffset;										// Offset in der Datei
	char   *pMemory;										// Zeiger auf die Daten
	char   *pDealloc;										// Speiche deallokiereen
	}PdfFile;

HMODULE				hMuPdfDll	= 0;						// Handle der MuDll
PdfProcType			pPdfClean	= 0;						// Funktions-Pointer der MuDll
PdfProcType			pPdfConvert	= 0;						//			*
static char			aCachePath[MAX_PATH];					// Ist der Cache-Pfad fr Text-Exporte
static CStringChain sFonts  (sizeof(int)*4);				// Liste aller Fonts der aktuellen Seite
static CDataList	sObjects(sizeof(int)*4);				// Liste aller Objekte in der PDF-Datei
static CDataList	sMaps	(MAP_MAX);						// Sind die Maps de Unicode
static PdfFile		sData;									// Globales Dateiobjekt
static int			iLine;									// Aktuelle Spalte in der Zeile



//*****************************************************************************
//*
//*		PdfGetCachePath
//*
//*****************************************************************************
//	Holt den Cache-Pfad und erstellt ihn wenn ntig.
//	pLen	: Wenn ungleich NULL wird hier die Pfadlnge gespeichert.
//	Ergibt einen Zeiger auf den Pfad oder NULL bei einem Fehler.
const char *PdfGetCachePath(int *pLen)
{
int		iLen;


	if(!CreateTempPath(aCachePath,sizeof(aCachePath),"TextCache",iLen))
		{
		if(pLen)*pLen=0;
		return 0;
		}

	if(pLen)*pLen=iLen;


return aCachePath;
}

//*****************************************************************************
//*
//*		PdfCheckInfo
//*
//*****************************************************************************
//	Prft ob das Info-File zur PDF-Datei passt
//	pPath		: Pfad zur PDF-Datei
//	pTemp		: Pfad zur Temp-Datei im Cache
//	Ergibt 1 wenn die Info-Datei passt, ansonnsten 0
static int PdfCheckInfo(const char *pPath,const char *pTemp)
{
char	cData[256];
char	cInfo[256];
FILE   *pFile; 
int		iSize;
int		iPos;
int		iRet;


	strlcpy(cInfo,pTemp,sizeof(cInfo));

	   iPos = strifind(cInfo,".txt.tmp");
	if(iPos<0)return 0;

	memcpy(cInfo+iPos,".inf",4);

		pFile = fopen(cInfo,"rb");
	if(!pFile)return 0;										// Info-Datei vorhanden

	iRet = fread(cData,128,1,pFile);
	fclose(pFile);
															// Ist das PDF gleich der Info-Datei
	if(iRet!=1)return 0;
	
	   iSize = LengthFile(pPath);
	if(iSize<=1204)return 0;

	if(*(int*)cData!=iSize)
		{
		return 0;
		}

	pFile = fopen(pPath,"rb");								// Daten aus der Mitter des PDF's laden
	fseek(pFile,iSize/2,SEEK_SET);
	iRet  = fread(cData+128,124,1,pFile);
	fclose(pFile);

	if(iRet!=1 || memcmp(cData+4,cData+128,124))
		{
		return 0;				
		}


return 1;
}


//*****************************************************************************
//*
//*		PdfCreateInfo
//*
//*****************************************************************************
//	Erzeugt ein neues Info-File aus eine PDF-Datei.
//	pPath		: Pfad zur PDF-Datei
//	pTemp		: Pfad zur Temp-Datei im Cache
//	Ergibt 1 wenn die Info-Datei erstellt wurde, ansonnsten 0
static int PdfCreateInfo(const char *pPath,const char *pTemp)
{
char	cData[256];
char	cInfo[256];
FILE   *pFile; 
int		iSize;
int		iPos;
int		iRet;


	strlcpy(cInfo,pTemp,sizeof(cInfo));

	   iPos = strifind(cInfo,".txt.tmp");
	if(iPos<0)return 0;

	memcpy(cInfo+iPos,".inf",4);

	   iSize = LengthFile(pPath);
	if(iSize<=1204)return 0;

	pFile = fopen(pPath,"rb");
	fseek(pFile,iSize/2,SEEK_SET);
	iRet  = fread(cData+4,124,1,pFile);
	fclose(pFile);

	if(iRet==1 && iSize>1024)								// Info-File erstellen
		{
		*(int*)cData = iSize; 

		   pFile = fopen(cInfo,"wb");
		if(pFile)									
			{
			fwrite(cData,1,128,pFile);
			fclose(pFile);
			return 1;
			}
		}


return 0;
}


//*****************************************************************************
//*
//*		PdfUpdateCache
//*
//*****************************************************************************
//	Aktuallisiert den Cache der Textdateien fr einzelne Dateien
//	pFiles			: Liste mit den PDF-Dateien
//	pCachePath		: Pfad mit den Text-Cache-Dateien
//	Ergibt die Anzahl der upgedateten Dateien bzw. -1 bei einem Fehler.
int PdfUpdateCache(const CStringChain *pFiles,const char *pCachePath)
{
char			cTarget[256];
unsigned 		uCount;
const char	   *pName;
const char	   *pPath;
int				iPos;
int				iLen;
int				iOff;



	if(!pCachePath)											// Cache-Pfad erstellen
		{
			pCachePath = PdfGetCachePath(&iPos);
		if(!pCachePath)return -1;
		}

	iPos = strlcpy(cTarget,pCachePath,sizeof(cTarget));
	iPos = AddBackSlash(cTarget,iPos);

	uCount = 0;

	for(iOff=0;;iOff++)										// Alle PDF-Dateien suchen
		{
			pPath = pFiles->Get(iOff);
		if(!pPath)break;

		pName = GetFileName(pPath);
		iLen  = strlen(pName);

		if(!CmpFileType(pName,"pdf",iLen))					// Ist es eine PDF Datei
			{
			continue;
			}

		strlcpy(cTarget+iPos,pName     ,sizeof(cTarget),iLen);
		strlcat(cTarget+iPos,".txt.tmp",sizeof(cTarget),8);

		if(ExistFile(cTarget))
			{
			if(CmpFileTime(cTarget,pPath)>=0)				// Ist die PDF-Datei lter
				{
				continue;
				}

			if(PdfCheckInfo(pPath,cTarget))					// Ist das Info-File ok
				{
				ChangeFileTime(cTarget,time(0));
				continue;
				}
			}

		if(PdfExtractText(pPath,cTarget,0x04)<=0)			// Die Texte aus der PDF-Datei holen
			{
			continue;
			}

		PdfCreateInfo(pPath,cTarget);
		uCount++;
		}



return uCount;
}

//*****************************************************************************
//*
//*		PdfUpdateCache
//*
//*****************************************************************************
//	Aktuallisiert den Cache der Textdateien
//	pPdfPath		: Name der Textausgabedatei
//	pCachePath		: Pfad mit den Text-Cache-Dateien
//	Ergibt die Anzahl der upgedateten Dateien bzw. -1 bei einem Fehler.
int PdfUpdateCache(const char *pPdfPath,const char *pCachePath)
{
char			cTarget[256];
unsigned 		uCount;
CFileFinder		sFind;
const char	   *pPath;
int				iPos;
int				iLen;



	if(!pCachePath)											// Cache-Pfad erstellen
		{
			pCachePath = PdfGetCachePath(&iPos);
		if(!pCachePath)return -1;
		}

	iPos = strlcpy(cTarget,pCachePath,sizeof(cTarget));
	iPos = AddBackSlash(cTarget,iPos);
	
	uCount = 0;
	sFind.Set(pPdfPath,CFF_FIND_SUBDIR,"*.pdf");


	for(;;)													// Alle PDF-Dateien suchen
		{
			pPath = sFind.Next(2,&iLen);
		if(!pPath)break;

		strlcpy(cTarget+iPos,pPath     ,sizeof(cTarget),iLen);
		strlcat(cTarget+iPos,".txt.tmp",sizeof(cTarget),8);

		pPath = sFind.NameFull();

		if(ExistFile(cTarget))
			{
			if(CmpFileTime(cTarget,pPath)>=0)				// Ist die PDF-Datei lter
				{
				continue;
				}

			if(PdfCheckInfo(pPath,cTarget))					// Ist das Info-File ok
				{
				ChangeFileTime(cTarget,time(0));
				continue;
				}
			}

		if(PdfExtractText(pPath,cTarget,0x04)<=0)			// Die Texte aus der PDF-Datei holen
			{
			continue;
			}
		
		PdfCreateInfo(pPath,cTarget);
		uCount++;
		}



return uCount;
}

//*****************************************************************************
//*
//*		PdfLine
//*
//*****************************************************************************
//	Liest die nchste Zeile von einer Datei ein
static int PdfLine(char *pLine,int iSize,PdfFile *pData)
{
char   *pPtr;
int		iMax;
int		iPos;



	iSize  -= 2;
	iMax	= pData->iSize-pData->iOffset;

	if(iSize>iMax)
		{
		iSize = iMax;
		}

	pPtr = pData->pMemory+pData->iOffset;



	for(iPos=0;;iPos++)										// Kopiere eine Zeile
		{
		if(iPos>=iSize)
			{
			pData->iOffset += iPos;
			pLine[iPos] = '\0';
			return iPos;
			}

		pLine[iPos] = pPtr[iPos];

		if(pLine[iPos]=='\n')
			{
			break;
			}

		if(pLine[iPos]=='\r')
			{
			break;
			}
		}

	pData->iStart	= pData->iOffset;
	pData->iOffset += iPos;

	while(pData->iOffset<pData->iSize)						// Vor auf die nchste Zeile
		{
		if(pData->pMemory[pData->iOffset]!='\n')
		if(pData->pMemory[pData->iOffset]!='\r')
			{
			break;
			}

		pData->iOffset++;
		}


	pLine[iPos] = '\n';	iPos++;
	pLine[iPos] = '\0';



return iPos;
}

//*****************************************************************************
//*
//*		PdfLoadDll
//*
//*****************************************************************************
//	Ld die PDF-DLL
//	Ergibt 1 wenn ok bzw. 0 bei einem Fehler
int PdfLoadDll()
{
char				cPath[MAX_PATH];
static unsigned		uDone=0;
unsigned			uDiff;
unsigned			uLen;


	
	if(pPdfClean && pPdfConvert)
		{
		return 1;
		}


	if(uDone)												// Nur alle 5min neu versuchne
		{
		   uDiff = GetTickCount()-uDone;
		if(uDiff<60000*5)return 0;
		}


	uDone	= GetTickCount();


	if(!hMuPdfDll)											// DLL laden
		{
			hMuPdfDll = LoadLibraryA("mupdf.dll");
		if(!hMuPdfDll)
			{
			uLen = GetExePath(cPath,sizeof(cPath));
			strlcpy(cPath+uLen,PDF_DLL_PATH "mupdf.dll",sizeof(cPath)-uLen);
			hMuPdfDll = LoadLibraryA(cPath);
			if(!hMuPdfDll)return 0;
			}

		pPdfClean	= (PdfProcType)GetProcAddress(hMuPdfDll,"PdfClean"  );
		pPdfConvert	= (PdfProcType)GetProcAddress(hMuPdfDll,"PdfConvert");
		}



return (pPdfClean && pPdfConvert)? 1:0;
}

//*****************************************************************************
//*
//*		PdfExtractText
//*
//*****************************************************************************
//	Extrahiert nur den Text ohne Code aus einer PDF-Datei. 
//	pPdfFile		: Ist die Quell-PDF-Datei
//	pOutFile		: Ist die Ausgabedatei
//	uFlags			: Diverse Flags
//						Bit0 : Ausgabedatei nur berschreiben wenn lter als PDF
//						Bit1 : Ausgabedatei nie berschreiben
//						Bit2 : Seiten-Tags einfgen ("<--nepage ???>")
//	Ergibt folgende Werte
//						3 = Die PDF-Datei existiert nicht
//						2 = Die Ausgabedatei hat bereits existiert
//						1 = Die Ausgabedatei wurde erstellt
//						0 = Fehler bei der Erstellung
//					   -1 = Fehler in der MuPdfDll
//					   -2 = PdfConvert() Funktion ist nicht in der DLL
//					   -3 = Die DLL konnte nicht geffnet werden
//					   -4 Der Pfad fr die Cache -Datei konnte nicht erstellt werden
int PdfExtractText(const char *pPdfFile,const char *pOutFile,unsigned uFlags)
{
char	cPath[MAX_PATH];
char	cName[MAX_PATH];
char   *aList[9];
int		iRet;
int		iNum;



	if(!PdfLoadDll())										// DLL laden
		{
		return (hMuPdfDll)? -2:-3;
		}

	if(uFlags&0x02)											// Nie berschreiben
		{
		if(ExistFile(pOutFile)==1)
			{
			return 2;
			}
		}

	if(ExistFile(pPdfFile)!=1)								// Existiert die PDF-Datei
		{
		return 3;
		}

	if(uFlags&0x01)											// Ist die Ausgabedatei neuer
		{
		if(CmpFileTime(pPdfFile,pOutFile)<=0)
			{
			return 2;
			}
		}

	if(!CreatePath(pOutFile,CP_PATH_LEN))					// Pfad erstellen
		{
		return -4;
		}


	strtoutf8(cPath,pOutFile,sizeof(cPath)-3);				// Es mssen UTF8-Namen bergeben werden
	strtoutf8(cName,pPdfFile,sizeof(cName)  );

	iNum = 0;

	aList[iNum++]	= "convert"; if(uFlags&0x04)
	aList[iNum++]	= "-t";
	aList[iNum++]	= "-F";
	aList[iNum++]	= "txt";
	aList[iNum++]	= "-o";
	aList[iNum++]	= cPath;
	aList[iNum++]	= cName;
	aList[iNum  ]	= NULL;

	DeleteFile(pOutFile);

	iRet = pPdfConvert(iNum,aList);

	if(ExistFile(pOutFile)==1)
		{
		return 1;
		}

	if(iRet<0)return -1;


return 0;
}


//*****************************************************************************
//*
//*		PdfExtract
//*
//*****************************************************************************
//	Extrahiert den Text-Code aus einer PDF-Datei. Dieser Text-Code kann auch
//	mit einem PDF-Viewer geffnet werden.
//	pPdfFile		: Ist die Quell-PDF-Datei
//	pOutFile		: Ist die Ausgabedatei
//	uFlags			: Diverse Flags
//						Bit0 : Ausgabedatei nur berschreiben wenn lter als PDF
//						Bit1 : Ausgabedatei nie berschreiben
//						Bit2 : Ausgabe nicht linearisieren
//						Bit3 : Streams wieder komprimieren
//	Ergibt folgende Werte
//						3 = Die PDF-Datei existiert nicht
//						2 = Die Ausgabedatei hat bereits existiert
//						1 = Die Ausgabedatei wurde erstellt
//						0 = Fehler bei der Erstellung
//					   -1 = Fehler in der MuPdfDll
//					   -2 = PdfClean() Funktion ist nicht in der DLL
//					   -3 = Die DLL konnte nicht geffnet werden
//					   -4 Der Pfad fr die Cache -Datei konnte nicht erstellt werden
int PdfExtract(const char *pPdfFile,const char *pOutFile,unsigned uFlags)
{
char	cPath[MAX_PATH];
char	cName[MAX_PATH];
char   *aList[8];
int		iRet;



	if(!PdfLoadDll())										// DLL laden
		{
		return (hMuPdfDll)? -2:-3;
		}

	if(uFlags&0x02)											// Nie berschreiben
		{
		if(ExistFile(pOutFile)==1)
			{
			return 2;
			}
		}

	if(ExistFile(pPdfFile)!=1)								// Existiert die PDF-Datei
		{
		return 3;
		}

	if(uFlags&0x01)											// Ist die Ausgabedatei neuer
		{
		if(CmpFileTime(pPdfFile,pOutFile)<=0)
			{
			return 2;
			}
		}

	if(!CreatePath(pOutFile,CP_PATH_LEN))					// Pfad erstellen
		{
		return -4;
		}


	strcpy(cPath,"-o:");									// Es mssen UTF8-Namen bergeben werden
	strtoutf8(cPath+3,pOutFile,sizeof(cPath)-3);
	strtoutf8(cName  ,pPdfFile,sizeof(cName)  );

	aList[0]	= "clean";
	aList[1]	= "-d";
	aList[2]	= "-r";
	aList[3]	= (uFlags&0x08)? "-z":"-a";
	aList[4]	= (uFlags&0x04)? "-d":"-l";
	aList[5]	= cName;
	aList[6]	= cPath;
	aList[7]	= NULL;

	DeleteFile(pOutFile);

	iRet = pPdfClean(7,aList);

	if(ExistFile(pOutFile)==1)
		{
		return 1;
		}

	if(iRet<0)return -1;


return 0;
}

//*****************************************************************************
//*
//*		PdfString
//*
//*****************************************************************************
//	Konvertiert die Escape-Sequenzen in PDF-Strings
//	pOut		: Ist der Ausgabepuffer
//	pTxt		: Zeiger auf den PDF-Strings
//	iSize		: Lnge des PDF-Strings
//	Ergibt die Lnge des estellten Textes
static int PdfString(char *pOut,const char *pTxt,int iSize)
{
int			iOff;
int			iNum;
int			iAdd;



	for(iOff=0,iNum=0;iOff<iSize;iOff++)
		{
		if(pTxt[iOff]!='\\')							// Ist es eine Escape-Sequenz
			{
			pOut[iNum] = pTxt[iOff];
			iNum++;
			continue;
			}

		if(iOff+1>=iSize)
			{
			break;
			}

		iOff++;

		if(pTxt[iOff]=='n')
			{
			pOut[iNum] = '\n';
			iNum++;
			continue;
			}

		if(pTxt[iOff]=='r')
			{
			pOut[iNum] = '\r';
			iNum++;
			continue;
			}

		if(pTxt[iOff]=='t')
			{
			pOut[iNum] = '\t';
			iNum++;
			continue;
			}

		if(pTxt[iOff]=='b')
			{
			pOut[iNum] = '\b';
			iNum++;
			continue;
			}

		if(pTxt[iOff]<'0' || pTxt[iOff]>'7')
			{
			pOut[iNum] = pTxt[iOff];
			iNum++;
			continue;
			}

		pOut[iNum] = strtoc(pTxt+iOff,&iAdd);
		iOff+=iAdd-1;
		iNum++;
		continue;
		}

	pOut[iNum] = 0;


return iNum;
}

//*****************************************************************************
//*
//*		PdfCheckText
//*
//*****************************************************************************
//	Prft ob es Zeilenumbrche gibt  (z.B. "... 1 0 0 1 21.8 <Y-Coord> Tm [(...")
//	pLine		: Zeiger auf die Zeile
//	iLen		: Lnge der Zeile (Position bei '(')
//	dLast		: Letzte Y-Koordinate
//	Ergibt 1 wenn man in einer neuen Zeile ist
static int PdfCheckText(const char *pLine,int iLen,double &dLast)
{
double	dCoor;
int		iOff;


	if(iLen<5)return 0;

	iOff = iLen-1;

	if(iOff>0 && pLine[iOff]=='[')
		{
		iOff--;
		}

	iOff -= strbwlen(pLine+iOff,0xE0,iOff);
															// ".... Tm [(..." gefunden
	if(iOff>4 && pLine[iOff-2]=='T' && pLine[iOff-1]=='m' && strwchar(pLine[iOff-3],0xE0))
		{
		iOff -= 3;
		iOff -= strbwlen(pLine+iOff,0xE0,iOff);

		if(strwchar(pLine[iOff-1],0x01))
			{
			iOff -= strbwlen(pLine+iOff,0x02000201,iOff);
			dCoor = atof(pLine+iOff);

			if(fabs(dLast-dCoor)>0.001)						// Y-Koordinate mit in Zeile
				{
				dLast = dCoor;
				return 1;
				}
			}
		}

return 0;
}

//*****************************************************************************
//*
//*		PdfLoadObject
//*
//*****************************************************************************
//	Schliet eine Datei.
//	pFile		: Zeiger auf File-Daten
static void PdfFileClose(PdfFile *pFile)
{

	if(!pFile)return;

	if(pFile->pDealloc)
		{
		delete pFile->pDealloc;
		pFile->pDealloc = 0;
		}

	pFile->pMemory	= 0;
	pFile->iOffset	= 0;
	pFile->iStart	= 0;
	pFile->iSize	= 0;


}

//*****************************************************************************
//*
//*		PdfLoadObject
//*
//*****************************************************************************
//	Einen Stream laden
//	iObj		: Ist die Objektnummer vom zu ladenden Stream
//	pFile		: Hier werden die File-Daten des neuen Stream gespeichert
//	Ergibt 1 wenn der Stream geladen wurde.
static int PdfLoadObject(int iObj,PdfFile *pFile)
{
char		cLine[512];
char	   *pDump;
int		   *pData;
int			iMode;
int			iMax;
int			iNum;
int			iPos;
int			iLen;



	if(iObj<0)												// Daten prfen
		{
		return 0;
		}

	   iPos = sObjects.FindFast(iObj);
	if(iPos<0)												// Wurde das Objekt gefunden
		{
		pFile->pDealloc	= 0;
		pFile->pMemory	= 0;
		pFile->iOffset	= 0;
		pFile->iSize	= 0;
		return 0;
		}

	pData = (int*)sObjects.Get(iPos);
	iMode = pData[1];

	if(iMode==1)iMode=0;

	pFile->pMemory	= sData.pMemory+pData[3];
	pFile->iSize	= pData[2];
	pFile->iStart	= 0;
	pFile->iOffset	= 0;
	pFile->pDealloc	= 0;

	if(pFile->iSize+sData.iOffset>sData.iSize)				// Prfe die Stream-Gre
		{
		pFile->iSize = sData.iSize-sData.iOffset;
		}

	if(pData[1]!=1)return 1;

	iLen	= 0;
	iMax	= pData[2]/2+4;
	pDump	= new char[iMax];


	while(PdfLine(cLine,sizeof(cLine),pFile))				// HEX-Dump einlesen
		{
		   iNum = iMax-iLen;
		if(iNum>1024)iNum=1024;

		iLen += dumptomem(pDump+iLen,iNum,cLine);
		}

	pFile->pDealloc		= pDump;
	pFile->pMemory		= pDump;
	pFile->iSize		= iLen;
	pFile->iOffset		= 0;
	pDump[iLen]			= 0;



return 1;
}

//*****************************************************************************
//*
//*		PdfScanStreams
//*
//*****************************************************************************
//	Konvertiert ein extrahierte PDF-Datei (enthlt nur Text) in eine
//	Textdatei mit Seiten Marken.
//	pFile		: Zeiger auf die Dateidaten
//	Ergibt folgende Werte
//						1 = Die Ausgabedatei wurde erstellt
//						0 = Fehler bei der Erstellung
//					   -1 = Die Datei konnte nicht geffnet werden
//					   -2 = Die Ausgabedatei konnte nicht erstellt werden
static int PdfScanObjects(PdfFile *pFile)
{
char		cLine[256];
unsigned 	aData[4];
int			iStart;
int			iMode;
int			iSize;
int			iPos;
int			iObj;
int			iOff;
int			iLen;




	iOff	= pFile->iOffset;
	iStart	= -1;
	iSize	= -1;
	iMode	= -1;
	iObj	= -1;

	sObjects.Del();
	pFile->iOffset = 0;


	for(;;)
		{
		if(!PdfLine(cLine,sizeof(cLine),pFile))				// Dateiende erreicht
			{
			break;
			}

 	   iPos = strwlen(cLine,0xA0);

	   if(cLine[iPos]=='/')
			{
			iPos += 1;
			iLen  = strwlen(cLine+iPos,0x0F);

															// Gre eines Streams
			if(iLen==6 && cLine[iPos]=='L' && !memcmp(cLine+iPos,"Length",iLen))
				{
				iSize = atoi(cLine+iPos+iLen);
				continue;
				}

															// Gre eines Streams
			if(iLen==6 && cLine[iPos]=='F' && !memcmp(cLine+iPos,"Filter",iLen))
				{
				iPos += iLen;
				iMode = 0;

				   iLen  = strfind(cLine+iPos,"/ASCIIHexDecode",15);
				if(iLen>=0 && !strwchar(cLine[iPos+iLen+15],0x0F))
					{
					iMode = 1;
					}

				continue;
				}

			continue;
			}

		   iPos=strfind(cLine,"obj",3);						// Beginnt ein Neues Object
		if(iPos>=0 && !strwchar(cLine[iPos+3],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-3],0x0F))
			{
			iPos = strwlen(cLine,0xA0);

			if(strwchar(cLine[iPos],0x01))
				{
				iPos += strwlen(cLine+iPos,0x01);
				iPos += strwlen(cLine+iPos,0xA0);

				if(strwchar(cLine[iPos],0x01))
					{
					iObj	= atoi(cLine);
					iStart	= pFile->iOffset;
					}
				}
			}

		   iPos=strfind(cLine,"endstream",9);				// End von einem Stream
		if(iPos>=0 && !strwchar(cLine[iPos+9],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-1],0x0F))
			{
			if(iObj>=0)
				{
				aData[0]	= iObj;
				aData[1]	= -1;
				aData[2]	= pFile->iStart-iStart;
				aData[3]	= iStart;
				iPos		= sObjects.FindFast(iObj,0,0x7FFFFFFF,0,0x08);

				sObjects.Add(aData,iPos);
				}

			iMode = -1;
			iSize = -1;
			iObj  = -1;
			}

		   iPos=strfind(cLine,"endobj",6);					// End von einem Stream
		if(iPos>=0 && !strwchar(cLine[iPos+6],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-1],0x0F))
			{
			if(iObj>=0)
				{
				aData[0]	= iObj;
				aData[1]	= -1;
				aData[2]	= pFile->iStart-iStart;
				aData[3]	= iStart;
				iPos		= sObjects.FindFast(iObj,0,0x7FFFFFFF,0,0x08);

				sObjects.Add(aData,iPos);
				}

			iMode = -1;
			iSize = -1;
			iObj  = -1;
			}

		   iPos=strfind(cLine,"stream",6);					// Beginnt ein Stream
		if(iPos>=0 && !strwchar(cLine[iPos+6],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-1],0x0F))
			{
			if(iSize>0 && iObj>=0)
				{
				aData[0]	= iObj;
				aData[1]	= iMode;
				aData[2]	= iSize;
				aData[3]	= pFile->iOffset;
				iPos		= sObjects.FindFast(iObj,0,0x7FFFFFFF,0,0x08);

				sObjects.Add(aData,iPos);

				pFile->iOffset += iSize;

				iMode = -1;
				iSize = -1;
				iObj  = -1;
				}
			else{
				iStart = pFile->iOffset;
				}
			}
		}

   pFile->iOffset = iOff;									// Datei-Offset wieder herstellen


return sObjects.Count();
}

//*****************************************************************************
//*
//*		PdfScanMap
//*
//*****************************************************************************
//	Liet alle die Unicode Maps eines Fonts ein
//	pData		: Zeiger auf die Fontdaten ([0]=ID,[1]=Map-ID [2]=Map-NR)
//	Ergibt die Anzahl der gefundenen Maps
static int PdfScanMap(int *pData)
{
unsigned char	aChars[MAP_MAX];
char			cLine[256];
unsigned 		aNums[4];
PdfFile			sFile;
int				iFlag;
int				iNum;
int				iLen;
int				iPos;



	pData[1] = 0;

	if(!PdfLoadObject(pData[0],&sFile))						// Objekt mit Resource laden
		{
		return 0;
		}



	for(;;)													// Font durchlaufen
		{
		if(!PdfLine(cLine,sizeof(cLine),&sFile))			// Dateiende erreicht
			{
			PdfFileClose(&sFile);
			return 0;
			}

	   iPos = strwlen(cLine,0xA0);

	   if(cLine[iPos]=='/')
			{
			iPos += 1;
			iLen  = strwlen(cLine+iPos,0x0F);

															// Font gefunden
			if(iLen==9 && cLine[iPos]=='T' && !memcmp(cLine+iPos,"ToUnicode",iLen))
				{
				iPos += iLen;
				iPos += strwlen(cLine+iPos,0xA0);

				if(strwchar(cLine[iPos],0x01))				// Objektnummer "... /Font 7 0 R..."
					{
					pData[1] = atoi(cLine+iPos);
					PdfFileClose(&sFile);

					if(!PdfLoadObject(pData[1],&sFile))		// Das Objekt mit der Map laden
						{
						return 0;
						}

					break;
					}

				continue;
				}

			continue;
			}
		}


	for(iPos=0;iPos<MAP_MAX;iPos++)							// Map initialisieren
		{
		aChars[iPos] = iPos;
		}

	iFlag = 0;

	for(;;)													// Map durchlaufen
		{
		if(!PdfLine(cLine,sizeof(cLine),&sFile))			// Dateiende erreicht
			{
			break;
			}

	   iPos = strwlen(cLine,0xA0);

	   if(cLine[iPos]=='/')
			{
			iPos += 1;
			iLen  = strwlen(cLine+iPos,0x0F);
			continue;
			}



		   iPos=strfind(cLine,"beginbfrange",12);			// Anfag von der Zeichen-Map
		if(iPos>=0 && !strwchar(cLine[iPos+12],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos- 1],0x0F))
			{
			iFlag = 1;
			}


		   iPos=strfind(cLine,"endbfrange",10);				// Ende von der Zeichen-Map
		if(iPos>=0 && !strwchar(cLine[iPos+10],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos- 1],0x0F))
			{
			iFlag = 0;
			}

		if(iFlag)
			{
			for(iPos=0,iNum=0;iNum<3;iNum++)
				{
				iPos += strwlen(cLine,0xA0);

				if(cLine[iPos]!='<')break;
				iPos += 1;

				aNums[iNum] = strtoi(cLine+iPos,0x1010,&iLen);
				if(iLen<=0)break;
				iPos += iLen;

				if(cLine[iPos]!='>')break;
				iPos += 1;
				}

			if(iNum==3)
				{
				while(aNums[0]<=aNums[1])
					{
					if(aNums[0]>=MAP_MAX)break;
					aChars[aNums[0]] = aNums[2];
					aNums[0]++;
					aNums[2]++;
					}
				}
			}
		}


	PdfFileClose(&sFile);

	pData[2] = sMaps.Add(aChars);



return 1;
}

//*****************************************************************************
//*
//*		PdfScanFonts
//*
//*****************************************************************************
//	Liet alle Fonts aus einem Resourcen-Objekt ein
//	iRes		: Ist die Resourcen-ID
//	pFonts		: Hier werden die Positionen der Streams gespeichert
//	Ergibt die Anzahl der gefundenen Fonts
static int PdfScanFonts(int iRes)
{
char		cLine[256];
int		 	aData[4];
PdfFile		sFile;
int			iFont;
int			iLen;
int			iPos;



	iFont	= -1;
	sMaps .Del();
	sFonts.Del();

	if(!PdfLoadObject(iRes,&sFile))							// Objekt mit Resource laden
		{
		return 0;
		}



	for(;;)													// Resource durchlaufen
		{
		if(!PdfLine(cLine,sizeof(cLine),&sFile))			// Dateiende erreicht
			{
			break;
			}

	   iPos = strwlen(cLine,0xA0);

	   if(cLine[iPos]=='/')
			{
			iPos += 1;
			iLen  = strwlen(cLine+iPos,0x0F);

															// Font gefunden
			if(iLen==4 && cLine[iPos]=='F' && !memcmp(cLine+iPos,"Font",iLen))
				{
				iPos += iLen;
				iPos += strwlen(cLine+iPos,0xA0);

				if(cLine[iPos]=='<' && cLine[iPos+1]=='<')	// Direkt eingebettet  "... /Font << ... >>"
					{
					break;
					}

				if(strwchar(cLine[iPos],0x01))				// Objektnummer "... /Font 7 0 R..."
					{
					iFont = atoi(cLine+iPos);
					PdfFileClose(&sFile);

					PdfLoadObject(iFont,&sFile);			// Das Objekt mit dem Font laden

					break;
					}

				continue;
				}

			continue;
			}
		}



	for(;;)													// Font durchlaufen
		{
		if(!PdfLine(cLine,sizeof(cLine),&sFile))			// Dateiende erreicht
			{
			break;
			}

	   iPos = strwlen(cLine,0xA0);

	   if(cLine[iPos]=='/')
			{
			iPos += 1;
			iLen  = strwlen(cLine+iPos,0x0F);

			aData[0] = atoi(cLine+iPos+iLen);
			aData[1] = -1;
			aData[2] = -1;

			if(PdfScanMap(aData))							// Hat der Font eine Unicode-Map
				{
				sFonts.Add(iLen,cLine+iPos,CSC_ABC,aData);
				}

			continue;
			}

		if(strfind(cLine,">>")>=0)
			{
			break;
			}
		}


	PdfFileClose(&sFile);



return sFonts.Count();
}

//*****************************************************************************
//*
//*		PdfFindFont
//*
//*****************************************************************************
//	Sucht die Font-Angaben in einen Format-Text
//	pLine		: Zeiger auf den Text
//	iSize		: Lnge des Textes
//	iFont		: Hier wird der Font Index gespeichert
//	Ergibt 1 wenn sich der Font verndert hat
int PdfFindFont(const char *pLine,int iSize,int &iFont)
{
int		iPos;
int		iRet;
int		iLen;
int		iOff;


	iRet = 0;

	for(iOff=0;iOff<iSize;iOff++)
		{
		if(pLine[iOff]!='/')continue;					// ".../<Font> ...."

		iOff++;

		iLen = strwlen(pLine+iOff,0x0F,iSize-iOff);
		iPos = sFonts.FindFast(iLen,pLine+iOff);

		if(iPos>=0)
			{
			iPos = sFonts.DataGet(iPos,2);
			}

		if(iPos!=iFont)
			{
			iFont = iPos;
			iRet  = 1;
			}
		}


return iRet;
}

//*****************************************************************************
//*
//*		PdfPlayStream
//*
//*****************************************************************************
//	Gibt die Texte eines Streams aus
//	pFile		: Zeiger auf die Dateidaten
//	pOutput		: Ausgabe fr den Text
//	Ergibt folgende Werte
//						1 = Die Ausgabedatei wurde erstellt
//						0 = Fehler bei der Erstellung
//					   -1 = Die Datei konnte nicht geffnet werden
//					   -2 = Die Ausgabedatei konnte nicht erstellt werden
static int PdfPlayStream(PdfFile *pFile,FILE *pOutput)
{
char		cText[1024];
char	   *pCharMap;
char	   *pSpace;
char	   *pLine;
double		dLast;
int			iJump;
int			iFont;
int			iOut;
int			iOff;
int			iMax;
int			iEnd;
int			iFix;
int			iNum;
int			iLen;
int			iPos;


	dLast	 = 0;
	iFont    =-1;
	iJump	 = 1;
	iFix	 = 0;
	pCharMap = 0;

	for(;;)													// Alle Textteile augeben
		{
		pLine	= pFile->pMemory+pFile->iOffset;
		iLen	= pFile->iSize  -pFile->iOffset;

		if(iLen<=0)break;

		iPos	= strchar(0x80000000,pLine,'(',iLen,'\\');	// Offset des Nchsten Textes
		iJump  |= PdfCheckText(pLine,iPos,dLast);

		if(iPos>=iLen)break;

		if(sFonts.Count())
			{
			if(PdfFindFont(pLine,iPos,iFont))
				{
				pCharMap = (char*)sMaps[iFont];
				}
			}

		pFile->iOffset += iPos;
		pLine		   += iPos;
		iLen		   -= iPos;

		if(iJump && iLine)									// Leeraum einfgen
			{
			fprintf(pOutput," ");
			iJump	= 0;
			}

		if(pFile->iOffset>0 && pLine[-1]=='[')				// Beginnt ein Array  "...[(..."
			{
			iFix	= 1;
			iJump	= 0;
			}

		   iEnd = strchar(0x80000000,pLine,')',iLen,'\\');
		if(iEnd<iLen && pLine[iEnd]==')')
				iMax = iEnd+1;
		else	iMax = iEnd;

		pFile->iOffset += iMax;


		for(iOff=1,iOut=0;iOff<iEnd;iOff+=iNum)				// "(...)" Bereich speichern
			{
			   iNum  = iEnd-iOff;
			if(iNum>sizeof(cText)/4*3)
				{
				iNum = sizeof(cText)/4*3;
				}

			if(iOff+iNum<iEnd)								// Suche eine gute Bruchlinie
				{
				while(iNum>16 && strwchar(pLine[iOff+iNum-1],0x00100041))
					{
					iNum--;
					}
				}
															// Textabschnitt dekodieren
			iOut	= PdfString(cText,pLine+iOff,iNum);
			iOff   += iNum;

			if(pCharMap)									// Unicode-Zeichen konvertieren
				{
				for(iNum=0;iNum<iOut;iNum++)
					{
					cText[iNum] = pCharMap[(unsigned char)cText[iNum]];
					}
				}

			for(iNum=0;iNum<iOut;iNum++,iLine++)
				{
				if(strwchar(cText[iNum],0xE0) && iLine>=LINE_MAX)
					{
					cText[iNum] = '\n';
					iLine = 0;
					}
				}

			fwrite(cText,iOut,1,pOutput);
			}

		pLine  += iMax;
		iLen   -= iMax;
		iNum	= strwlen(pLine,0xE0,iLen);
		pSpace	= "";

		if(iOut>0 && strwchar(cText[iOut-1],0xE0))			// Wurde vorher eine Leerstele augeben
			{
			pSpace = "";
			}
		else if(iLen<=0 || iFix)							// Ende vom Stream oder kein Array
			{
			pSpace = (iFix)? "":" ";
			}
		else if(iLen>0 && pLine[0]==']')					// Ende vom Array "....)]"
			{
			pLine  += 1;
			iLen   -= 1;
			iFix	= 0;
			}
		else if(strwchar(pLine[0],0x02000001))				// Sind wir in einem Array "...)12(..."
			{
			iOff  = strtoni(pLine,iLen,0,&iNum);
			iOff += strwlen(pLine+iOff,0xE0,iLen-iOff);

			if(iOff<iLen && pLine[iOff]=='(')
				{
				iNum  = abs(iNum);
				if(iNum< 65)pSpace = "" ;					// Breite des Leerraums prfen
				if(iNum>200)pSpace = " ";
				}
			}

		if(*pSpace)											// Wird ein Leeraum eingefgt
			{
			if(iLine>=LINE_MAX)
				{
				pSpace	= "\n";
				iLine	= 0;
				}
			else{
				iLine++;
				}

			fwrite(pSpace,1,1,pOutput);
			}
		}



return 1;
}

//*****************************************************************************
//*
//*		PdfScanPages
//*
//*****************************************************************************
//	Such die Seiten-Inizes in einer PDF-Datei
//	pData		: Zeiger auf die Daten der PDF-Datei
//	iObj		: Ist die Objektnummer der Seiten-Inizes
//	Ergibt einen Zeiger auf die Seiten ([0]=Anzahl  [1..n]=Objekt der Seite)
short *PdfScanPages(PdfFile *pData,int iObj)
{
char		cLine[1024];
char       *pText;
short      *pList;
short      *pTemp;
PdfFile		sData;
int			iSize;
int			iCnt;
int			iNum;
int			iPos;
int			iLen;


	pText	= pData->pMemory;
	iSize	= pData->iSize-20;

	for(iPos=0;;iPos++)									// Suche das Objekt
		{
		if(iPos>=iSize)return 0;

		if(pText[iPos]!='\n')continue;

		   iLen = strwlen(pText+iPos+1,0x01,iSize-iPos);
		if(iLen<=0 || iLen>8 || iLen>=iSize)continue;

		if(atoi(pText+iPos)!=iObj)continue;

		iPos += iLen+1;
		iLen  = strwlen(pText+iPos,0xA0,iSize-iPos);
		
		if(iLen<=0 || iLen>8 || iLen>=iSize)continue;
		
		iPos += iLen;
		iLen  = strwlen(pText+iPos,0x01,iSize-iPos);

		if(iLen<=0 || iLen>8 || iLen>=iSize)continue;

		iPos += iLen;
		iLen  = strwlen(pText+iPos,0xA0,iSize-iPos);
		
		if(iLen<=0 || iLen>8 || iLen>=iSize)continue;

		iPos += iLen;

		if(!memcmp(pText+iPos,"obj",3) && !strwchar(pText[iPos+3],0x1F))
			{
			break;
			}
		}

	iCnt			= 0;
	sData			= *pData;
	sData.iOffset	= iPos;

	for(;;)												// Suche die Liste mit den Seiten
		{
		if(!PdfLine(cLine,sizeof(cLine),&sData))		// Dateiende erreicht
			{
			return 0;
			}

		iPos = strwlen(cLine,0xA0);	
		
		if(cLine[iPos]=='>' && cLine[iPos+1]=='>')
			{
			return 0;
			}

		if(cLine[iPos]=='/' && !memcmp(cLine+iPos+1,"Count",5) && !strwchar(cLine[iPos+6],0x1F))
			{
			iCnt = atoi(cLine+iPos+6);
			if(iCnt<0 || iCnt>10000)iCnt=0;
			continue;
			}
														
		if(cLine[iPos]=='/' && !memcmp(cLine+iPos+1,"Kids",4) && !strwchar(cLine[iPos+5],0x1F))
			{
			iPos = strchar(cLine,'[');
			if(cLine[iPos]=='['){iPos++;break;}
			
			if(!PdfLine(cLine,sizeof(cLine),&sData))
				{
				return 0;
				}
			
			iPos = strchar(cLine,'[');
			if(cLine[iPos]=='['){iPos++;break;}
			}
		}

	
	if(!iCnt)iCnt=2000;

	pList	 = new short[iCnt+8]; 
	pList[0] = 0;

	for(iNum=1;;)										// Seitennummern einlesen
		{
		if(iNum>iCnt)
			{
			iCnt += 1000;
			pTemp = new short[iCnt+4]; 

			memcpy(pTemp,pList,sizeof(short)*iNum);
			delete pList;
			pList = pTemp;
			}

		pList[iNum]  = atoi(cLine+iPos);
		pList[ 0  ] += 1;
		iNum		+= 1;

		for(;cLine[iPos];iPos++)						// Zur nchsten Seite
			{
			if(cLine[iPos]=='R')break;
			if(cLine[iPos]==']')break;
			}			
		
		if(cLine[iPos]!='R')break;

		iPos += 1;
		iPos += strwlen(cLine+iPos,0xE0);	

		if(cLine[iPos]==0)
			{
			if(!PdfLine(cLine,sizeof(cLine),&sData))
				{
				break;
				}

			iPos = -1;
			}
		}


return pList;
}


//*****************************************************************************
//*
//*		PdfToText
//*
//*****************************************************************************
//	Konvertiert ein extrahierte PDF-Datei (enthlt nur Text) in eine
//	Textdatei mit Seiten Marken.
//	pPdfFile		: Name der extrahierten PDF-Datei
//	pTxtFile		: Name der Textausgabedatei
//	Ergibt folgende Werte
//						1 = Die Ausgabedatei wurde erstellt
//						0 = Fehler bei der Erstellung
//					   -1 = Die Datei konnte nicht geffnet werden
//					   -2 = Die Ausgabedatei konnte nicht erstellt werden
int PdfToText(const char *pPdfFile,const char *pTxtFile,unsigned uFlags)
{
int			aContents[64];
char		cLine[1024];
char		cTemp[256];
char		cText[64];
int			aData[4];
short      *pPages;	
char	   *pLine;
FILE	   *pFile;
CMapFile	sFile;
PdfFile		sNext;
double		dLast;
int			iPage;
int			iMode;
int			iInfo;
int			iFlag;
int			iSize;
int			iType;
int			iPos;
int			iIdx;
int			iObj;
int			iNum;
int			iOff;
int			iSel;
int			iRes;
int			iLen;



		sData.pMemory = sFile.Open(pPdfFile,&sData.iSize,CMF_READ);
	if(!sData.pMemory)
		{
		return -1;
		}

	sData.pDealloc  = 0;
	sData.iOffset	= 0;
	sData.iStart	= 0;

															// Format prfen
	if(!PdfLine(cLine,sizeof(cLine),&sData) || strncmp(cLine,"%PDF",4))
		{
		return 0;
		}

	
	strlcpy(cTemp,pTxtFile,sizeof(cTemp));
	SetFileName(cTemp,"PdfToText.tmp",sizeof(cTemp));


		pFile = fopen(cTemp,"wb");							// Ausgabedatei ffnen
	if(!pFile)
		{
		return -2;
		}


	pPages	= 0;
	dLast	= 0;
	iPage	= 0;
	iMode	= 0;
	iFlag	= 0;
	iType	=-1;
	iInfo	=-1;
	iSize	=-1;
	iSel	=-1;
	iRes	=-1;

	PdfScanObjects(&sData);									// Positionen der Streams suchen


	for(;;)
		{
		if(!PdfLine(cLine,sizeof(cLine),&sData))			// Dateiende erreicht
			{
			break;
			}

 	   iPos = strwlen(cLine,0xA0);

	   if(cLine[iPos]=='/')
			{
			iPos += 1;
			iLen  = strwlen(cLine+iPos,0x0F);
															// Typangabe gfunden
			if(iLen==4 && cLine[iPos]=='T' && !memcmp(cLine+iPos,"Type",iLen))
				{
				iPos += iLen;

				   iOff = strfind(cLine+iPos,"/Page",5);
				if(iOff>=0 && !strwchar(cLine[iPos+iOff+5],0x0F))
					{
					if(iFlag!=2)iPage++;					// Neue Seite gefunden
					
					if(pPages)
						{
						for(iOff=1;iOff<pPages[0];iOff++)
							{
							if(pPages[iOff]!=iObj)continue;
							iPage = iOff;
							break;
							}
						}

					iFlag = 1;
					iType = 0;
					}

				   iOff = strfind(cLine+iPos,"/Catalog",8);
				if(iOff>=0 && !strwchar(cLine[iPos+iOff+8],0x0F))
					{
					iType = 1;								// Catalog gefunden
					}

				continue;
				}

															// Existiert ein Seiten Tag
			if(iLen==5 && cLine[iPos]=='P' && !memcmp(cLine+iPos,"Pages",iLen))
				{
				iInfo = atoi(cLine+iPos+iLen);
				continue;
				}

															// Existiert ein Seiten Tag
			if(iLen==13 && cLine[iPos]=='p' && !memcmp(cLine+iPos,"pdftk_PageNum",iLen))
				{
				iPage = atoi(cLine+iPos+iLen);
				iMode = 1;
				iFlag = 2;
				continue;
				}

															// Content einer Seite
			if(iLen==8 && cLine[iPos]=='C' && !memcmp(cLine+iPos,"Contents",iLen))
				{
				iPos += iLen;
				iPos += strwlen(cLine+iPos,0xE0);

				if(cLine[iPos]=='[')						// Array mit Content
					{
					iPos++;
					pLine	= sData.pMemory+sData.iStart+iPos;
					iNum	= sData.iSize  -sData.iStart+iPos; if(iNum>4096)iNum=4096;
					iNum	= strchar(pLine,']',iNum);

					for(iOff=0,iIdx=0;iOff<iNum;iIdx++)
						{
						iOff += strgetstr(cText,pLine+iOff,sizeof(cText),'\\','(',iNum-iOff)+1;
						iOff += strwlen(pLine+iOff,0xE0,iNum-iOff);

						if(iIdx%3==0)
							{
							if(iIdx>=63*3)break;
							aContents[iIdx/3] = atoi(cText);
							}
						}

					iIdx /= 3;
					aContents[iIdx] = -1;
					}
				else{										// Einzelner Content
					aContents[0] = atoi(cLine+iPos);
					aContents[1] = -1;
					}

				continue;
				}

															// Resourcen einer Seite
			if(iLen==9 && cLine[iPos]=='R' && !memcmp(cLine+iPos,"Resources",iLen))
				{
				iPos += iLen;
				iPos += strwlen(cLine+iPos,0xE0);

				if(strwchar(cLine[iPos],0x01))				// Mit Objektnummer "/Resources 2 0 R"
					{
					iRes = atoi(cLine+iPos);
					continue;
					}

				if(cLine[iPos]=='<' && cLine[iPos+1]=='<')	// Eingebettet "/Resources << ... >>"
					{
					pLine		= sData.pMemory+sData.iStart+iPos;
					iNum		= sData.iSize  -sData.iStart+iPos; if(iNum>4096)iNum=4096;
					iNum		= strlfind(pLine,">>",iNum,2)+2;

					iRes		= sObjects.Count()+0x1000000;
					aData[0]	= iRes;
					aData[1]	= -1;
					aData[2]	= iNum;
					aData[3]	= sData.iStart+iPos;
					iPos		= sObjects.FindFast(iRes,0,0x7FFFFFFF,0,0x08);

					sObjects.Add(aData,iPos);
					continue;
					}

				continue;
				}

															// Gre eines Streams
			if(iLen==6 && cLine[iPos]=='L' && !memcmp(cLine+iPos,"Length",iLen))
				{
				iSize = atoi(cLine+iPos+iLen);
				continue;
				}

															// Filter eines Streams
			if(iLen==6 && cLine[iPos]=='F' && !memcmp(cLine+iPos,"Filter",iLen))
				{
				iPos += iLen;
				iSel  = 0;

				   iOff = strfind(cLine+iPos,"/ASCIIHexDecode",15);
				if(iOff>=0 && !strwchar(cLine[iPos+iOff+15],0x0F))
					{
					iSel = 1;
					}

				continue;
				}

			continue;
			}


		if(iFlag)											// Seiten-Tag abspeichern
			{
			if(strfind(cLine,">>",2)>=0)
				{
				iFlag=3;
				}
			}

		   iPos=strfind(cLine,"endobj",6);
		if(iPos>=0 && !strwchar(cLine[iPos+6],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-1],0x0F))
			{
			if(iType==0 && iFlag==3)						// Texte der Seite ausgeben
				{
				iLine = 0;
				iFlag = 0;
				fprintf(pFile,"\n<!--newpage %i>\n",iPage);

				if(iRes>=0)
					{
					PdfScanFonts(iRes);
					}

				for(iIdx=0;aContents[iIdx]>=0;iIdx++)		// Texte einlesen
					{
					if(!PdfLoadObject(aContents[iIdx],&sNext))
						{
						continue;
						}

					if(sNext.pDealloc)
						{
						pLine = sNext.pDealloc;
						iLen  =	sNext.iSize-4;

						for(iPos=0;iPos<iLen;iPos++)		// "ID .... EI" aufrumen
							{
							if(pLine[iPos+0x01]!='B')continue;
							if(pLine[iPos+0x02]!='I')continue;
							if(pLine[iPos+0x03]!=' ')
							if(pLine[iPos+0x03]!=10 )
							if(pLine[iPos+0x03]!=13 )continue;
							if(pLine[iPos     ]!=10 )
							if(pLine[iPos     ]!=13 )continue;

							for(iNum=iPos+4;iNum<iLen;iNum++)
								{
								if(pLine[iNum+0x01]!='E')continue;
								if(pLine[iNum+0x02]!='I')continue;
								if(pLine[iNum+0x03]!=' ')
								if(pLine[iNum+0x03]!=10 )
								if(pLine[iNum+0x03]!=13 )
								if(pLine[iNum+0x03]!= 0 )continue;
								if(pLine[iNum     ]!=10 )
								if(pLine[iNum     ]!=13 )continue;

								memset(pLine+iPos+4,' ',iNum-iPos-4);
								break;
								}
							}
						}

					PdfPlayStream(&sNext,pFile);			// Texte ausgeben
					PdfFileClose (&sNext);
					}
				}

			if(iType==1 && iInfo>0)							// Catalog mit Seiten-Indizes
				{
				pPages = PdfScanPages(&sData,iInfo);				
				} 

			iSize	= -1;
			iType	= -1;
			iInfo	= -1;
			iSel	= -1;
			iObj	= -1;
			iRes	= -1;

			continue;
			}

		   iPos=strfind(cLine,"endstream",9);				// Ende von einem Stream
		if(iPos>=0 && !strwchar(cLine[iPos+9],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-1],0x0F))
			{
			iSize	= -1;
			iType	= -1;
			iInfo	= -1;
			iSel	= -1;
			iObj	= -1;
			iRes	= -1;
			}


		   iPos=strfind(cLine,"stream",6);					// Beginnt ein Stream
		if(iPos>=0 && !strwchar(cLine[iPos+6],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-1],0x0F))
			{
			if(iSize>0)sData.iOffset += iSize;

			iSize	= -1;
			iType	= -1;
			iInfo	= -1;
			iSel	= -1;
			iObj	= -1;
			iRes	= -1;
			}


		   iPos=strfind(cLine,"obj",3);						// Beginnt ein Neues Object
		if(iPos>=0 && !strwchar(cLine[iPos+3],0x0F))
		if(iPos==0 || !strwchar(cLine[iPos-3],0x0F))
			{
			iPos	= strwlen(cLine,0xA0);

			if(strwchar(cLine[iPos],0x01))
				{
				iPos += strwlen(cLine+iPos,0x01);
				iPos += strwlen(cLine+iPos,0xA0);

				if(strwchar(cLine[iPos],0x01))
					{
					iObj  = atoi(cLine);
					iType = -1;
					iInfo = -1;
					}
				}
			}
		}

	fclose(pFile);

	if(pPages)delete pPages;

	sObjects.Del();
	sData.iSize = 0;

	DeleteFile(pTxtFile);

	if(!MoveFile(cTemp,pTxtFile))							// Datei umbenennen	
		{
		return -2;
		}



return 1;
}



