//*****************************************************************************
//*
//*
//*		RandFunctions.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2015
//
//	Funktionen:
//
//		void	 RandInit (const void *pInitData)	: Zufallsgenerator initialisieren 
//		unsigned RandShift(int iBit);				: Zufallszahl genrieren
//

#include	"RandFunctions.h"
#include	<string.h>


//*****************************************************************************
//*
//*		RandShift
//*
//*****************************************************************************
//	Ergibt eine Zufallszahl die iBit lang ist.
//	Die Schieberegister Lnge ist 128 Bit, Polynom: 1+x+x^127

static unsigned				aShiftVal[ 4]={0x127CBA41,0x9AF35180,0x920BA291,0x91D4E127};
static unsigned char const	aBackBits[64]=
	{
	0x00,0x0B,0x0D,0x06,0x01,0x0A,0x0C,0x07,0x03,0x08,0x0E,0x05,0x02,0x09,0x0F,0x04,
	0x06,0x0D,0x0B,0x00,0x07,0x0C,0x0A,0x01,0x05,0x0E,0x08,0x03,0x04,0x0F,0x09,0x02,
	0x0D,0x06,0x00,0x0B,0x0C,0x07,0x01,0x0A,0x0E,0x05,0x03,0x08,0x0F,0x04,0x02,0x09,
	0x0B,0x00,0x06,0x0D,0x0A,0x01,0x07,0x0C,0x08,0x03,0x05,0x0E,0x09,0x02,0x04,0x0F,
	};

extern "C" unsigned _cdecl RandShift(int iBit)
{
unsigned	uValue=0;
unsigned	uMask;
unsigned	uBack;
unsigned	uNum;


	/*
	for(;iBit>0;iBit--)
		{
		uBack=(aShiftVal[3]>>31)&1;
		if(aShiftVal[0]&1)uBack^=1;
		if(aShiftVal[0]&2)uBack^=1;

		aShiftVal[3] <<= 1;
		aShiftVal[3]  |= (aShiftVal[2]>>31)&1;
		aShiftVal[2] <<= 1;
		aShiftVal[2]  |= (aShiftVal[1]>>31)&1;
		aShiftVal[1] <<= 1;
		aShiftVal[1]  |= (aShiftVal[0]>>31)&1;
		aShiftVal[0] <<= 1;
		aShiftVal[0]  |= uBack;

		uValue<<=1;
		uValue|=uBack;
		}
	*/

	for(;iBit>4;iBit-=4)
		{
		uBack=((aShiftVal[3]>>26)&0x3C) | (aShiftVal[0]&0x03);
		uBack=aBackBits[uBack];

		aShiftVal[3] <<= 4;
		aShiftVal[3]  |= (aShiftVal[2]>>28)&0x0F;
		aShiftVal[2] <<= 4;
		aShiftVal[2]  |= (aShiftVal[1]>>28)&0x0F;
		aShiftVal[1] <<= 4;
		aShiftVal[1]  |= (aShiftVal[0]>>28)&0x0F;
		aShiftVal[0] <<= 4;
		aShiftVal[0]  |= uBack;

		uValue	     <<= 4;
		uValue        |= uBack;
		}


	uNum   = 4-iBit;
	uBack  = ((aShiftVal[3]>>26)&0x3C) | (aShiftVal[0]&0x03);
	uBack  = aBackBits[uBack];
	uBack>>= uNum;
	uMask  = 0x0F>>uNum;
	uNum  += 28;

	aShiftVal[3] <<= iBit;
	aShiftVal[3]  |= (aShiftVal[2]>>uNum)&uMask;
	aShiftVal[2] <<= iBit;
	aShiftVal[2]  |= (aShiftVal[1]>>uNum)&uMask;
	aShiftVal[1] <<= iBit;
	aShiftVal[1]  |= (aShiftVal[0]>>uNum)&uMask;
	aShiftVal[0] <<= iBit;
	aShiftVal[0]  |= uBack;

	uValue       <<= iBit;
	uValue        |= uBack;


return uValue;
}

//*****************************************************************************
//*
//*		RandInit
//*
//*****************************************************************************
//	Initialisiert das Schieberegister
//	pInitData	: Ist ein Zeiger auf einen 64 Byte Datenblock
extern "C" void _cdecl RandInit(const void *pInitData)
{

	memcpy(aShiftVal,pInitData,sizeof(aShiftVal));
	aShiftVal[1]|=1;

}
