//*****************************************************************************
//*
//*
//*		RegKeys.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//	Diese Klasse ermglicht den Zugriff auf die Registry:
//
//	Funktionen der Klasse CRegKeys
//
//		bool Close ();												// Schliet Schlssel
//		bool Open  (HKEY   hKeyNow=HKEY_CURRENT_USER,				// ffnet Schlssel
//					char  *pPath=0,
//					int	   iMode=CRK_CREATE);
//		bool Open  (char  *pPath=0,
//					int	   iMode=CRK_CREATE);
//		bool Create(char  *pPath);									// Erzeugt Unterschlssel
//		bool Create(HKEY   hKeyNow=HKEY_CURRENT_USER, 				// Erzeugt Schlssel
//					char  *pPath=0);
//		bool Up	   (char  *pPath=CRK_NEXT,int iOnlyIfExist=0);		// Pfad aufsteigen
//		bool Down  ();												// Pfad aufabsteigen
//		bool Next  ();												// Nchsten Unterschlssel ffnen
//		bool First ();												// Zum ersten Unterschlssel
//
//																	// Werte auslesen
//		bool Get(		  int	&iVal,char  *pName=CRK_NEXT);
//		bool Get(unsigned int	&uVal,char  *pName=CRK_NEXT);
//		bool Get(		  long	&lVal,char  *pName=CRK_NEXT);
//		bool Get(unsigned long	&uVal,char  *pName=CRK_NEXT);
//		bool Get(		  char **pVal,char  *pName=CRK_NEXT);
//		bool Get(unsigned char **pVal,char  *pName=CRK_NEXT);
//		bool Get(		  void **pVal,char  *pName=CRK_NEXT,DWORD dwType=REG_BINARY);
//		bool Get(		  char  *pVal,char  *pName=CRK_NEXT);
//		bool Get(unsigned char  *pVal,char  *pName=CRK_NEXT);
//		bool Get(		  char  *pVal,DWORD dwLen,char  *pName=CRK_NEXT);
//		bool Get(unsigned char  *pVal,DWORD dwLen,char  *pName=CRK_NEXT);
//		bool Get(		  void	*pVal,DWORD dwLen,char  *pName=CRK_NEXT,DWORD dwType=REG_NONE);
//
//																	// Werte schreiben
//		bool Set(		  int	 iVal,char  *pName=CRK_NEXT);
//		bool Set(		  char 	*pVal,char  *pName=CRK_NEXT);
//		bool Set(unsigned char 	*pVal,char  *pName=CRK_NEXT);
//		bool Set(		  void	*pVal,DWORD dwLen,char *pName=CRK_NEXT,DWORD dwType=REG_NONE);
//
//		bool Del    (char  *pName=CRK_NEXT);						// Wert lschen
//		bool DelKey (char  *pName=CRK_NEXT);						// Key	lschen
//
//		bool Load   (char  *pFilename);								// Zweig speichern
//		bool Save   (char  *pFilename);								// Zweig speichern
//		bool Replace(char  *pFilenameOld,char *pFilenameNew);		// Zweig austauschen
//
//		int		Path(char  *,int,int iMode=0);						// Ergibt Pfad
//		int		GetLength(const char  *pName);						// Ergibt die Lnge eines Wertes
//		DWORD	Type ();											// Ergibt Type	der Werte
//		DWORD	Len  ();											// Ergibt Lnge der Werte
//		HKEY	Key  ();											// Ergibt Aktuellen Schlssel
//		char   *Name ();											// Ergibt Name
//		int		Error();											// Ergibt letzten Fehler
//
//		int	    KeyCount  ();										// Ergibt Anzahl der SubKeys
//		int	    ValueCount();										// Ergibt Werte Anzahl
//		char   *ValueName (int iNr=-1);								// Ergibt Wert-Namen
//		char   *KeySubName(int iNr=-1);								// Ergibt SubKey-Namen
//		char   *KeyName   (int iNr=-1);								// Ergibt hKey-Namen
//
//		int	    RegPath(char  *pPath,int iLen,zchar  *pName=0);		// Ergibt Standard Pfad

#include	"RegKeys.h"
#include	"StringChain.h"
#include	<stdio.h>

#ifdef	_MSC_VER
#pragma warning( disable :4996 )
#endif

#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)			L##a
#define 	zmode			1
#define 	zchar			wchar_t
#define 	zcharx			unsigned char 
#define 	zcharu			short
#define 	zword			long
#define 	strzlen			(int)wcslen
#define 	strzcpy			wcscpy
#define 	strzncpy		wcsncpy
#define 	strzncat		wcsncat
#define 	strzicmp		wcsicmp
#define 	strznicmp		wcsnicmp
#define 	szprintf		swprintf
#else
#define 	Z(a)			a
#define 	zmode			0
#define 	zchar			char
#define 	zcharx			wchar_t
#define 	zcharu			unsigned char
#define 	zword			short
#define 	strzlen			(int)strlen
#define 	strzcpy			strcpy
#define 	strzncpy		strncpy
#define 	strzncat		strncat
#define 	strzicmp		stricmp
#define 	strznicmp		strnicmp
#define 	szprintf		sprintf
#endif


#pragma warning( disable :4035 )

typedef	LONG	(WINAPI *RegQueryType)(HKEY,LPVOID,LPDWORD,LPDWORD,LPBYTE,LPDWORD);
typedef	LONG    (WINAPI *RegSetType  )(HKEY,LPVOID,DWORD,DWORD,CONST BYTE* ,DWORD);




#define	Check()													\
	if(iLevel<            0){iError=ERROR_KEY  ;return false;}	\
	if(iLevel>=CRK_KEYS_END){iError=ERROR_LIMIT;return false;}
#define	Exit(nr)	        {iError=nr         ;return false;}	\


//*****************************************************************************
//*
//*		Variablen
//*
//*****************************************************************************

const zchar *CRegKeys::aRootName[]=
					{
					Z("HKEY_CLASSES_ROOT"),
					Z("HKEY_CURRENT_USER"),
					Z("HKEY_LOCAL_MACHINE"),
					Z("HKEY_USERS"),
					Z("HKEY_CURRENT_CONFIG"),
					Z("HKEY_DYN_DATA"),
					NULL
					};

const HKEY	CRegKeys::aRootKeys[]=
					{
					HKEY_CLASSES_ROOT,
					HKEY_CURRENT_USER,
					HKEY_LOCAL_MACHINE,
					HKEY_USERS,
					HKEY_CURRENT_CONFIG,
					HKEY_DYN_DATA
					};


//*****************************************************************************
//*
//*		Konstrukor
//*
//*****************************************************************************
CRegKeys::CRegKeys()
{

	cName[0]	=  0;
	dwNameLen	=  0;
	iLevel		= -1;
	hKeyNow		=  0;

}

//*****************************************************************************
//*
//*		Destrukor
//*
//*****************************************************************************
CRegKeys::~CRegKeys()
{
	if(iLevel>=0)Close();
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet alle Schlssel
//	Ergibt 'false' bei einem Fehler
bool CRegKeys::Close()
{
int		iErr=ERROR_SUCCESS;
bool	bExit=true;


	while(iLevel>=0)
		{
		if(!Down())
			{
			iErr  = iError;
			bExit = false;
			}
		}

	hKeyNow	 = 0;
	hKeyInit = 0;
	iError   = iErr;


return bExit;
}

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
//  ffnet eine Schlssel
//	hKey	: Ausgangsschlssel =	HKEY_CURRENT_USER
//									HKEY_LOCAL_MACHINE
//									... weitere
//									0 == Schlssel aus Pfad holen.
//	pPath	: Name des Schlssels bei 0 wird er von AfxGetApp()->m_pszRegistryKey...
//			  genommen.
//	iMode	: CRK_READ		0	Schlssel nur lesen
//		      CRK_WRITE		1	Schlssel auch schreiben
//		      CRK_CREATE	2	Schlssel neu erzeugen wenn nicht vorhanden.
//			  Bit 6..31			Erweiterte Flags (CRK_WOW64_64KEY oder CRK_WOW64_32KEY)
// Ergibt 'false' bei einem Fehler
bool CRegKeys::Open(HKEY hKey,const zchar *pPath,int iMode)
{
zchar	cBuffer[1024];
int		iExFlags;


	if(iLevel>=0)Close();

	iExFlags  =  iMode&~0x3F;
	iMode    &=  0x3F;
	iOpenMode =  iMode;
	iFlags    = (iMode==CRK_READ)? KEY_READ:KEY_ALL_ACCESS;
	iFlags   |=  iExFlags&~(0x3F|CRK_WOW64_AUTO);


	if(!iExFlags && hKey==HKEY_LOCAL_MACHINE)			// Nicht auf WOW64 Umleitung gehen
		{
		iFlags |= CRK_WOW64_64KEY;
		}


	if(!hKey)											// ROOT-Key aus dem Pfad holen
		{
		pPath += GetInitKey(hKey,pPath);
		}

	if(!pPath)											// Standard Pfad
		{
		if(hKey==HKEY_CURRENT_USER || hKey==HKEY_LOCAL_MACHINE || hKey==HKEY_CURRENT_CONFIG)
			{
			RegPath(cBuffer,sizeof(cBuffer)/sizeof(zchar));
			}
		else{
			cBuffer[0] = 0;
			}

		pPath = cBuffer;
		}



	iError = RegOpenKeyEx(hKey,pPath,0,iFlags,&aKey[0]);// ffne Schlssel



	if(iError==ERROR_ACCESS_DENIED && hKey==HKEY_LOCAL_MACHINE)
	if(iFlags& CRK_WOW64_64KEY)
		{
		iFlags &= ~CRK_WOW64_64KEY;						// Win200 mag CRK_WOW64_64KEY nicht
		iError  =  RegOpenKeyEx(hKey,pPath,0,iFlags,&aKey[0]);
		}

	if(iError==ERROR_SUCCESS)
		{
			    aStr[0]=new zchar[strzlen(pPath)+1];
		strzcpy(aStr[0],pPath);
		hKeyNow=aKey[0];
			    aSub[0]=0;
			    aPos[0]=0;

		iLevel   = 0;
		hKeyInit = hKey;

		return true;
		}

	if(iMode!=CRK_CREATE)								// Neuen Schlssel erzeugen
		{
		iError=ERROR_MODE;
		return false;
		}



return Create(hKey,pPath,iExFlags);
}

//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
//	Erzeugt neuen Schlssel alle alten Schlssel werden geschlossen
//	hKey	: Ausgangsschlssel:	HKEY_CURRENT_USER
//									HKEY_LOCAL_MACHINE
//									... weitere
//									0 == Schlssel aus Pfad holen.
//	pPath	: Name des Schlssels.
//		      Bei NULL wird er von AfxGetApp()->m_pszRegistryKey genommen.
//	iExMode	: Erweiterte Flags (CRK_WOW64_64KEY oder CRK_WOW64_32KEY)
//	Ergibt 'false' bei einem Fehler
bool CRegKeys::Create(HKEY hKey,const zchar *pPath,int iExMode)
{
DWORD	dwType;
zchar	cBuffer[1024];


	if(iLevel>=0)Close();

	iOpenMode = CRK_CREATE;
	iFlags    = KEY_ALL_ACCESS;
	iFlags   |= iExMode&~(0x3F|CRK_WOW64_AUTO);

	if(hKey==HKEY_LOCAL_MACHINE)
		{												// Nicht auf WOW64 Umleitung gehen
		iFlags |= CRK_WOW64_64KEY;
		}


	if(!hKey)											// ROOT-Key aus dem Pfad holen
		{
		pPath += GetInitKey(hKey,pPath);
		}

	if(!pPath)											// Standard Pfad
		{
		if(hKey==HKEY_CURRENT_USER || hKey==HKEY_LOCAL_MACHINE || hKey==HKEY_CURRENT_CONFIG)
			{
			RegPath(cBuffer,sizeof(cBuffer)/sizeof(zchar));
			}
		else{
			cBuffer[0] = 0;
			}

		pPath = cBuffer;
		}


	iError = RegCreateKeyEx(hKey,pPath,0,0,REG_OPTION_NON_VOLATILE,iFlags,0,&aKey[0],&dwType);

	if(iError==ERROR_SUCCESS)
		{
			    aStr[0]=new zchar[strzlen(pPath)+1];
		strzcpy(aStr[0],pPath);
			    aSub[0]=0;
			    aPos[0]=0;
		hKeyNow=aKey[0];

		hKeyInit = hKey;
		iLevel   = 0;

		return true;
		}


return false;
}

//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
//	Erzeugt einen neuen Unterschlssel im aktuellen Schlssel
//	pPath	: Name des Schlssels.
//			  Bei NULL wird er von AfxGetApp()->m_pszRegistryKey genommen.
//	Ergibt 'false' bei einem Fehler
bool CRegKeys::Create(const zchar *pPath)
{
DWORD	dwMode;


	Check();										// Sind Ebenen ok
	if(iOpenMode!=CRK_CREATE)Exit(ERROR_MODE);		// Zugriffs-Erlaubnis ?

	if(!pPath)
		{
	#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
		   pPath=(zchar*)AfxGetApp()->m_pszRegistryKey;
	#else
		   pPath=Z("");
	#endif
		}


	iError = RegCreateKeyEx(hKeyNow,pPath,0,0,REG_OPTION_NON_VOLATILE,iFlags,0,&aKey[iLevel+1],&dwMode);

	if(iError==ERROR_SUCCESS)
		{
		iLevel++;

			    aStr[iLevel]=new zchar[strzlen(pPath)+1];
		strzcpy(aStr[iLevel],pPath);
		hKeyNow=aKey[iLevel];
			    aSub[iLevel]=0;
			    aPos[iLevel]=0;

		return true;
		}


return false;
}

//*****************************************************************************
//*
//*		Down
//*
//*****************************************************************************
//	Springt einen Pfad nach unten. Dadurch wird der Pfad krzer.
bool CRegKeys::Down()
{

	if(iLevel<0){iError=ERROR_KEY  ;return false;}	// Sind Ebenen ok


	  iError=RegCloseKey  (aKey[iLevel]);
	if(aStr[iLevel])delete aStr[iLevel];
			  iLevel--;
	hKeyNow =(iLevel<0)? 0:aKey[iLevel];


return iError==ERROR_SUCCESS;
}

//*****************************************************************************
//*
//*		Up
//*
//*****************************************************************************
//	ffnet einen neuen Unterschlssel
// 	pPfad			: Ist der name des Unterschlssels der geffnet werden soll.
//					  Bei NULL wird der nchste Schlssel genommen.
//	iOnlyIfExist	: Nur wenn der Key existiert, ihn dann ffnen
bool CRegKeys::Up(const zchar *pPath,int iOnlyIfExist)
{

	Check();										// Sind Ebenen ok


	if(!pPath || pPath==CRK_NEXT)					// Nchsten Subkey
		{
			pPath=KeySubName();
		if(!pPath)return false;
		aSub[iLevel]++;
		}
													// ffne Schlssel
	iError = RegOpenKeyEx(hKeyNow,pPath,0,iFlags,&aKey[iLevel+1]);

	if(iError==ERROR_SUCCESS)
		{
		iLevel++;

			    aStr[iLevel]=new zchar[strzlen(pPath)+1];
		strzcpy(aStr[iLevel],pPath);
		hKeyNow=aKey[iLevel];
   			    aSub[iLevel]=0;
			    aPos[iLevel]=0;

		return true;
		}

	if(iOpenMode!=CRK_CREATE || iOnlyIfExist)
		{
		if(iError!=ERROR_ACCESS_DENIED)iError=ERROR_MODE;
		return false;
		}


return Create(pPath);
}


//*****************************************************************************
//*
//*		First
//*
//*****************************************************************************
//	Springt zurck zum ersten Wertnamen
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::First()
{

	Check();									// Sind Ebenen ok
	aSub[iLevel]=0;


return true;
}

//*****************************************************************************
//*
//*		GetLength
//*
//*****************************************************************************
//	Liest die Lnge eines Wertes aus
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//  Ergibt die Datenlnge des Wertes oder -1 bei einem Fehler
int CRegKeys::GetLength(const zchar *pName)
{
DWORD	dwSize;


	Check();												// Sind Ebenen ok

	dwLength = 0;
	dwType   = REG_BINARY;

	if(pName!=CRK_NEXT)
		{
		if(pName)
			{
			   dwNameLen = strzlen(pName);
			if(dwNameLen>=sizeof(cName)/sizeof(cName[0]))
				{
				dwNameLen = sizeof(cName)/sizeof(cName[0])-1;
				}

			memcpy(cName,pName,(dwNameLen+1)*sizeof(zchar));
			pName = cName;
			}
		else{
			cName[0]  = 0;
			dwNameLen = 0;
			}

		iError = RegQueryValueEx(hKeyNow,pName,0,&dwType,NULL,&dwLength);
		}
	else{
		dwSize = 256;
		iError = RegEnumValue(hKeyNow,aPos[iLevel],cName,&dwSize,0,&dwType,NULL,&dwLength);
		if(iError==ERROR_SUCCESS)dwNameLen=dwSize;
		}


	if(iError!=ERROR_SUCCESS && iError!=ERROR_MORE_DATA)	// Gab es Fehler
		{
		dwLength = 0;
		dwSize   = 0;
		return -1;
		}


return dwLength;
}

//*****************************************************************************
//*
//*		Get(void*...
//*
//*****************************************************************************
//	Liest einen Wert aus.
//	pVal		: Zeiger auf den Zielpuffer in dem der Wert gespeichert wird.
//	uLen		: Ist die Lnges des Zielpuffers in Byte.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Get(void *pVal,DWORD uLen,const zchar *pName,DWORD uType)
{
zchar      *pText;
unsigned	uOff;
DWORD		dwSize;


	Check();											// Sind Ebenen ok

	dwLength = uLen;
	dwType   = uType;

	if(pName!=CRK_NEXT)
		{
		if(pName)
			{
			   dwNameLen = strzlen(pName);
			if(dwNameLen>=sizeof(cName)/sizeof(cName[0]))
				{
				dwNameLen = sizeof(cName)/sizeof(cName[0])-1;
				}

			memcpy(cName,pName,(dwNameLen+1)*sizeof(zchar));
			pName = cName;
			}
		else{
			cName[0]  = 0;
			dwNameLen = 0;
			}

		iError=RegQueryValueEx(hKeyNow,pName,0,&dwType,(unsigned char*)pVal,&dwLength);
		}
	else{
		dwSize=256;
		iError=RegEnumValue(hKeyNow,aPos[iLevel],cName,&dwSize,0,&dwType,(unsigned char*)pVal,&dwLength);
		if(iError==ERROR_SUCCESS)dwNameLen=dwSize;
		aPos[iLevel]++;
		}


	if(iError!=ERROR_SUCCESS)							// Gab es Fehler
		{
		dwLength=0;

		if(uLen>0)
			{
			if(uType==REG_SZ || uType==REG_EXPAND_SZ)
				{
				*(zchar*)pVal=0;
				dwLength=sizeof(zchar);
				}

			if(uType==REG_MULTI_SZ && uLen>=2*sizeof(zchar))
				{
				*(zword*)pVal=0;
				dwLength=2*sizeof(zchar);
				}
			}

		return false;
		}


	if(dwLength==0 && uLen>0)
		{
		if(uType==REG_SZ || uType==REG_EXPAND_SZ)
			{
			*(zchar*)pVal=0;
			dwLength=1*sizeof(zchar);
			}
		else if(uType==REG_MULTI_SZ && uLen>=2*sizeof(zchar))
			{
			*(zword*)pVal=0;
			dwLength=2*sizeof(zchar);
			}
		}
	else if(uType==REG_SZ || uType==REG_EXPAND_SZ)		// Nullzeichen am Ende prfen (Win7)
		{
		if(uLen<sizeof(zchar))
			{
			iError   = ERROR_MORE_DATA;
			dwLength = 0;
			return false;
			}

		uOff = (dwLength+sizeof(zchar)-1)/sizeof(zchar);

		   pText = (zchar*)pVal;
		if(pText[uOff-1])								// Windows7 zhlt 0 Bytes nicht mit
			{
			if(dwLength+sizeof(zchar)>uLen)
				{
				*(zchar*)pVal=0;
				iError   = ERROR_MORE_DATA;
				dwLength = 0;
				return false;
				}

			pText[uOff] = 0;
			dwLength += sizeof(zchar);
			}
		}
	else if(uType==REG_MULTI_SZ)						// Doppel-Nullzeichen am Ende prfen (Win7)
		{
		if(uLen<sizeof(zchar)*2)
			{
			if(uLen>=sizeof(zchar))*(zchar*)pVal=0;
			iError   = ERROR_MORE_DATA;
			dwLength = 0;
			return false;
			}

		uOff = (dwLength+sizeof(zchar)-1)/sizeof(zchar);

		   pText = (zchar*)pVal;
		if(pText[uOff-1])								// Kein Nullbyte am Ende
			{
			if(dwLength+sizeof(zchar)*2>uLen)
				{
				*(zword*)pVal=0;
				iError   = ERROR_MORE_DATA;
				dwLength = 0;
				return false;
				}

			pText[uOff  ] = 0;
			pText[uOff+1] = 0;
			dwLength += sizeof(zchar)*2;
			}
		else if(pText[uOff-2])							// Nur ein Nullbyte am Ende
			{
			if(dwLength+sizeof(zchar)>uLen)
				{
				*(zword*)pVal=0;
				iError   = ERROR_MORE_DATA;
				dwLength = 0;
				return false;
				}

			pText[uOff] = 0;
			dwLength += sizeof(zchar);
			}
		}


return true;
}

//*****************************************************************************
//*
//*		Get(void**...
//*
//*****************************************************************************
//	Liest einen Wert aus und reserviert einen Speicherbereich dafr.
//	pVal		: Hier wird der Zeiger auf den Speicherbereich abgelegt
//				  oder NULL bei einem Fehler.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Get(void **pVal,const zchar *pName,DWORD uType)
{
zchar      *pText;
DWORD		dwSize;
unsigned    uOff;


	*pVal=NULL;

	Check();											// Sind Ebenen ok

	dwLength = 0;
	dwType   = uType;

	if(pName!=CRK_NEXT)
		{
		if(pName)
			{
			   dwNameLen = strzlen(pName);
			if(dwNameLen>=sizeof(cName)/sizeof(cName[0]))
				{
				dwNameLen = sizeof(cName)/sizeof(cName[0])-1;
				}

			memcpy(cName,pName,(dwNameLen+1)*sizeof(zchar));
			pName = cName;
			}
		else{
			cName[0]  = 0;
			dwNameLen = 0;
			}

		iError=RegQueryValueEx(hKeyNow,pName,0,&dwType,(unsigned char*)NULL,&dwLength);
		}
	else{
		dwSize=256;
		iError=RegEnumValue(hKeyNow,aPos[iLevel],cName,&dwSize,0,&dwType,(unsigned char*)NULL,&dwLength);
		if(iError==ERROR_SUCCESS)dwNameLen=dwSize;
		aPos[iLevel]++;
		pName=cName;
		}

	if(iError!=ERROR_MORE_DATA)
	if(iError!=ERROR_SUCCESS)return false;

	dwSize	  = dwLength+4;
	*pVal	  = new unsigned char[dwSize];
	iError    = RegQueryValueEx(hKeyNow,pName,0,&dwType,(unsigned char*)*pVal,&dwSize);

	if(dwSize==0)
		{
		*(DWORD*)*pVal=0;
		if(uType==REG_SZ || uType==REG_EXPAND_SZ)dwLength=1*sizeof(zchar);
		if(uType==REG_MULTI_SZ                  )dwLength=2*sizeof(zchar);
		}
	else if(uType==REG_SZ || uType==REG_EXPAND_SZ)		// Nullzeichen am Ende prfen (Win7)
		{
		uOff = (dwLength+sizeof(zchar)-1)/sizeof(zchar);

		   pText = (zchar*)*pVal;
		if(pText[uOff-1])								// Windows7 zhlt 0 Bytes nicht mit
			{
			pText[uOff] = 0;
			dwLength += sizeof(zchar);
			}
		}
	else if(uType==REG_MULTI_SZ)						// Doppel-Nullzeichen am Ende prfen (Win7)
		{
		uOff = (dwLength+sizeof(zchar)-1)/sizeof(zchar);

		   pText = (zchar*)*pVal;
		if(pText[uOff-1])								// Kein Nullbyte am Ende
			{
			pText[uOff  ] = 0;
			pText[uOff+1] = 0;
			dwLength += sizeof(zchar)*2;
			}
		else if(pText[uOff-2])							// Nur ein Nullbyte am Ende
			{
			pText[uOff] = 0;
			dwLength += sizeof(zchar);
			}
		}




return true;
}

//*****************************************************************************
//*
//*		GetEx(void*...
//*
//*****************************************************************************
//	Liest einen Wert aus im ANSI oder UNICODE-Format, ohne Lngenprfung.
//	pVal		: Zeiger auf den Zielpuffer in dem der Wert gespeichert wird.
//	uLen		: Ist die Lnges des Zielpuffers in Byte.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//					CRK_USE_AUTO	0x10000000		Unicode oder Ansi benutzen (fr GetEx)
//					CRK_USE_ANSI	0x10000000		Ansi benutzen    (fr GetEx)
//					CRK_USE_WIDE	0x20000000		Unicode benutzen (fr GetEx)
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::GetEx(void *pVal,DWORD uLen,const zchar *pName,DWORD uType)
{
zcharx			cTemp[260];
LPVOID			pNamePtr;
RegQueryType    pQuery;	
unsigned		uRest;
unsigned		uMode;
unsigned		uOff;


	if(pName==CRK_NEXT)									// Im EX-Modus ist CRK_NEXT nicht untersttzt
		{
		Exit(ERROR_NOT_SUPPORTED);
		}


	Check();											// Sind Ebenen ok

	dwLength = uLen;
	dwType   = uType;

	if(pName)
		{
		   dwNameLen = strzlen(pName);
		if(dwNameLen>=sizeof(cName)/sizeof(cName[0]))
			{
			dwNameLen = sizeof(cName)/sizeof(cName[0])-1;
			}

		memcpy(cName,pName,(dwNameLen+1)*sizeof(zchar));
		pName = cName;
		}
	else{
		cName[0]  = 0;
		dwNameLen = 0;
		}



	uMode  = (uType>>28)&3;
	uType &= 0xCFFFFFFF;


	switch(uMode)
		{
	case 0:		pQuery = (RegQueryType)RegQueryValueEx;		uMode=zmode;break;
	case 1:		pQuery = (RegQueryType)RegQueryValueExA;	uMode=0;	break;
	case 2:		pQuery = (RegQueryType)RegQueryValueExW;	uMode=1;	break;
	default:	Exit(ERROR_NOT_SUPPORTED);
		}

	if(uMode!=zmode && pName)						// UNICODE<-->ANSI Umwandlung
		{
		for(uOff=0;;uOff++)
			{
			if(uOff>=256)Exit(ERROR_FILE_NOT_FOUND);
			cTemp[uOff] = (zcharx)pName[uOff];
			if(!pName[uOff])break;
			}
		
		pNamePtr = cTemp;
		}
	else{
		pNamePtr = (LPVOID)pName;
		}



	iError = pQuery(hKeyNow,pNamePtr,0,&dwType,(unsigned char*)pVal,&dwLength);


	if(iError!=ERROR_SUCCESS)							// Gab es Fehler
		{
		     if(uLen>=2)*(LPWORD)pVal = 0; 
		else if(uLen==1)*(LPBYTE)pVal = 0; 
		dwLength = 0;
		return false;
		}

	uRest = uLen-dwLength;

		 if(uRest>=4)*(LPDWORD)((char*)pVal+dwLength) = 0; 
	else if(uRest>=2)*(LPWORD )((char*)pVal+dwLength) = 0; 
	else if(uRest==1)*(LPBYTE )((char*)pVal+dwLength) = 0; 



return true;
}

//*****************************************************************************
//*
//*		GetEx(void**...
//*
//*****************************************************************************
//	Liest einen Wert aus und reserviert einen Speicherbereich dafr,.
//	Das kann im ANSI oder UNICODE-Format erfolgen, ohne Lngenprfung.
//	pVal		: Hier wird der Zeiger auf den Speicherbereich abgelegt
//				  oder NULL bei einem Fehler.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//					CRK_USE_AUTO	0x10000000		Unicode oder Ansi benutzen (fr GetEx)
//					CRK_USE_ANSI	0x10000000		Ansi benutzen    (fr GetEx)
//					CRK_USE_WIDE	0x20000000		Unicode benutzen (fr GetEx)
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::GetEx(void **pVal,const zchar *pName,DWORD uType)
{
zcharx			cTemp[260];
LPVOID			pNamePtr;
DWORD			dwSize;
RegQueryType    pQuery;	
unsigned		uMode;
unsigned		uOff;


	*pVal=NULL;

	if(pName==CRK_NEXT)									// Im EX-Modus ist CRK_NEXT nicht untersttzt
		{
		Exit(ERROR_NOT_SUPPORTED);
		}


	Check();											// Sind Ebenen ok

	dwLength = 0;
	dwType   = uType;

	if(pName)
		{
		   dwNameLen = strzlen(pName);
		if(dwNameLen>=sizeof(cName)/sizeof(cName[0]))
			{
			dwNameLen = sizeof(cName)/sizeof(cName[0])-1;
			}

		memcpy(cName,pName,(dwNameLen+1)*sizeof(zchar));
		pName = cName;
		}
	else{
		cName[0]  = 0;
		dwNameLen = 0;
		}

	uMode  = (uType>>28)&3;
	uType &= 0xCFFFFFFF;


	switch(uMode)
		{
	case 0:		pQuery = (RegQueryType)RegQueryValueEx;		uMode=zmode;break;
	case 1:		pQuery = (RegQueryType)RegQueryValueExA;	uMode=0;	break;
	case 2:		pQuery = (RegQueryType)RegQueryValueExW;	uMode=1;	break;
	default:	Exit(ERROR_NOT_SUPPORTED);
		}

	if(uMode!=zmode && pName)						// UNICODE<-->ANSI Umwandlung
		{
		for(uOff=0;;uOff++)
			{
			if(uOff>=256)Exit(ERROR_FILE_NOT_FOUND);
			cTemp[uOff] = (zcharx)pName[uOff];
			if(!pName[uOff])break;
			}
		
		pNamePtr = cTemp;
		}
	else{
		pNamePtr = (LPVOID)pName;
		}


													// Gre holen
	iError = pQuery(hKeyNow,pNamePtr,0,&dwType,(unsigned char*)NULL,&dwLength);

	if(iError!=ERROR_MORE_DATA)
	if(iError!=ERROR_SUCCESS)return false;

	dwSize	  = dwLength+4;
	*pVal     = new zchar[dwSize];					// Daten holen
	iError    = pQuery(hKeyNow,pNamePtr,0,&dwType,(unsigned char*)*pVal,&dwSize);

	*(LPDWORD)((char*)(*pVal)+dwLength) = 0; 



return true;
}

//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Liest einen Fliekommawert aus.
//	pVal		: Zeiger auf den Zielpuffer in dem der Wert gespeichert wird.
//	uLen		: Ist die Lnges des Zielpuffers in Byte.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Get(float &dVal,const zchar *pName)
{
zchar	cText[64]=Z("");


	if(!Get(cText,64,pName))return false;

	#ifdef		UNICODE
	swscanf(cText,L"%lf",&dVal);
	#else
	dVal = (float)atof(cText);
	#endif



return true;
}

//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Liest einen Fliekommawert aus.
//	pVal		: Zeiger auf den Zielpuffer in dem der Wert gespeichert wird.
//	uLen		: Ist die Lnges des Zielpuffers in Byte.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Get(double &dVal,const zchar *pName)
{
zchar	cText[64]=Z("");


	if(!Get(cText,64,pName))return false;

	#ifdef		UNICODE
	swscanf(cText,L"%lf",&dVal);
	#else
	dVal = atof(cText);
	#endif


return true;
}

//*****************************************************************************
//*
//*		Get
//*
//*****************************************************************************
//	Holt einen Text in ein CString-Objekt
//	sStr		: Ist das CString-Objekt
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//  Ergibt 'false' bei einem Fehler
#if defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
bool CRegKeys::Get(CString &sStr,const zchar *pName)
{
zchar *pPtr;
bool   bOk;

	bOk=Get(&pPtr,pName);
	if(pPtr){sStr=(dwLength>0)? pPtr:Z("");delete pPtr;}

return bOk;
}
#endif


//*****************************************************************************
//*
//*		Set(void*...
//*
//*****************************************************************************
//	Speichert einen Wert in der Registry.
//	pVal		: Zeiger auf den Quellpuffer mit dem Wert.
//	uLen		: Ist die Lnges des Quellpuffers in Byte.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Set(const void *pVal,DWORD uLen,const zchar *pName,DWORD uType)
{
DWORD	dwSize;



	Check();											// Sind Ebenen ok


	if(iOpenMode==CRK_READ)Exit(ERROR_MODE);
	if(!pVal)Exit(ERROR_DATA);

	if(pName!=CRK_NEXT)
		{
		if(pName)
			{
			   dwNameLen = strzlen(pName);
			if(dwNameLen>=sizeof(cName)/sizeof(cName[0]))
				{
				dwNameLen = sizeof(cName)/sizeof(cName[0])-1;
				}

			memcpy(cName,pName,(dwNameLen+1)*sizeof(zchar));
			pName = cName;
			}
		else{
			cName[0]  = 0;
			dwNameLen = 0;
			}
		}
	else{
		dwLength = 0;
		dwSize   = 256;
		iError   = RegEnumValue(hKeyNow,aPos[iLevel],cName,&dwSize,0,&dwType,(unsigned char*)pVal,&dwLength);
		if(iError==ERROR_SUCCESS)dwNameLen=dwSize;
		pName    = cName;
		aPos[iLevel]++;
		}


	dwLength = uLen;
	dwType   = uType;
	iError   = RegSetValueEx(hKeyNow,pName,0,dwType,(unsigned char*)pVal,dwLength);


return iError==ERROR_SUCCESS;
}


//*****************************************************************************
//*
//*		SetEx(void*...
//*
//*****************************************************************************
//	Speichert einen Wert in der Registry im ANSI oder UNICODE-Format.
//	pVal		: Zeiger auf den Quellpuffer mit dem Wert.
//	uLen		: Ist die Lnges des Quellpuffers in Byte.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
//	uType		: Ist der Typ des Wertes der geholt werden soll.
//					REG_BINARY
//					REG_DWORD
//					...
//					CRK_USE_AUTO	0x10000000		Unicode oder Ansi benutzen (fr GetEx)
//					CRK_USE_ANSI	0x10000000		Ansi benutzen    (fr GetEx)
//					CRK_USE_WIDE	0x20000000		Unicode benutzen (fr GetEx)
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::SetEx(const void *pVal,DWORD uLen,const zchar *pName,DWORD uType)
{
zcharx		cTemp[260];
LPVOID		pNamePtr;
RegSetType  pSetVal;	
unsigned	uMode;
unsigned	uOff;



	if(pName==CRK_NEXT)									// Im EX-Modus ist CRK_NEXT nicht untersttzt
		{
		Exit(ERROR_NOT_SUPPORTED);
		}

	Check();											// Sind Ebenen ok


	if(iOpenMode==CRK_READ)Exit(ERROR_MODE);
	if(!pVal)Exit(ERROR_DATA);

	if(pName)
		{
		   dwNameLen = strzlen(pName);
		if(dwNameLen>=sizeof(cName)/sizeof(cName[0]))
			{
			dwNameLen = sizeof(cName)/sizeof(cName[0])-1;
			}

		memcpy(cName,pName,(dwNameLen+1)*sizeof(zchar));
		pName = cName;
		}
	else{
		cName[0]  = 0;
		dwNameLen = 0;
		}

	uMode  = (uType>>28)&3;
	uType &= 0xCFFFFFFF;


	switch(uMode)
		{
	case 0:		pSetVal = (RegSetType)RegSetValueEx;	uMode=zmode;break;
	case 1:		pSetVal = (RegSetType)RegSetValueExA;	uMode=0;	break;
	case 2:		pSetVal = (RegSetType)RegSetValueExW;	uMode=1;	break;
	default:	Exit(ERROR_NOT_SUPPORTED);
		}

	if(uMode!=zmode && pName)							// UNICODE<-->ANSI Umwandlung
		{
		for(uOff=0;;uOff++)
			{
			if(uOff>=256)Exit(ERROR_FILE_NOT_FOUND);
			cTemp[uOff] = (zcharx)pName[uOff];
			if(!pName[uOff])break;
			}
		
		pNamePtr = cTemp;
		}
	else{
		pNamePtr = (LPVOID)pName;
		}



	dwLength = uLen;
	dwType   = uType;
	iError   = pSetVal(hKeyNow,pNamePtr,0,dwType,(unsigned char*)pVal,dwLength);


return iError==ERROR_SUCCESS;
}



//*****************************************************************************
//*
//*		Set
//*
//*****************************************************************************
//	Speichert einen Fliekommawert in der Registry.
//	dVal		: Ist der Wert.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Set(double &dVal,const zchar *pName)
{
zchar	cText[64];
int		iLen;


	iLen = szprintf(cText,Z("%g"),dVal);


return Set(cText,iLen*sizeof(zchar),pName,REG_SZ);
}


//*****************************************************************************
//*
//*		SetMulti
//*
//*****************************************************************************
//	Speichert einen Multi-String in der Registry.
//	pVal		: Zeiger auf den Multi-String.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::SetMulti(const zchar *pVal,const zchar *pName)
{
int iPos;


	for(iPos=0;pVal[iPos];)						// Lnge berechnen
		{
		iPos+=(int)strzlen(pVal+iPos)+1;
		}


return SetMulti(pVal,iPos+1,pName);
}

//*****************************************************************************
//*
//*		SetMulti
//*
//*****************************************************************************
//	Speichert einen Multi-String in der Registry.
//	pChain		: Zeiger auf den String-Kette mit den zuspeichernten Texten.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//	iStart		: Ist die Nummer des ersten einzufgenden Textes. 
//				  Wenn sie kleiner 0 ist wird sie auf 0 gesetzt. 
//	iStop		: Ist die Nummer des Textes bis zu Eingefgt wird. 
//				  Wenn sie ber der Anzahl an Texten ist, wird sie auf diese Anzahl gesetzt. 
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::SetMulti(const CStringChain *pChain,const zchar *pName,int iStart,int iStop)
{
zchar  *pMem;	
bool	bRet;


	pChain->Chain(iStart,iStop,&pMem);
	bRet  = SetMulti(pMem,pName);
	delete pMem;


return bRet;
}

//*****************************************************************************
//*
//*		GetMulti
//*
//*****************************************************************************
//	Leist einen Multi-String aus der Registry in eine String-Kette ein.
//	pChain		: Zeiger auf den String-Kette in der die Texte gespeichert werden.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::GetMulti(CStringChain *pChain,const zchar *pName)
{
zchar  *pMem;	
bool	bRet;

	
	bRet  = GetMulti(&pMem,pName);
	pChain->Del();
	pChain->KAdd(pMem);
	
	delete pMem;


return bRet;
}


//*****************************************************************************
//*
//*		Del
//*
//*****************************************************************************
//	Lscht einen Schlsselwert. (nicht einen Pfad)
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
// 					Ist pName==CRK_ALL  werden alle Werte gelscht
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Del(const zchar *pName)
{
int		iNum;
bool	bExit;


	Check();									// Sind Ebenen ok

	if(iOpenMode==CRK_READ)Exit(ERROR_MODE);

	if(pName==CRK_ALL)							// Alles lschen
		{
		bExit = true;
		iNum  = ValueCount();

		for(iNum--;iNum>=0;iNum--)
			{
			if(!Del(ValueName(iNum)))bExit=false;
			}

		return bExit;
		}

	if(pName==CRK_NEXT)
		{
		pName = ValueName(aPos[iLevel]);
		}

	iError = RegDeleteValue(hKeyNow,pName);


return iError==ERROR_SUCCESS;
}

//*****************************************************************************
//*
//*		DelKey
//*
//*****************************************************************************
//	Lscht einen Unterschlssel. (nicht einen Wert) Dabei werden auch
//	weitere Unterschlssel rekursiv gelscht.
// 	pName		: Ist der Name des Wertes der ausgelesen werden soll.
//					Ist pName==NULL     wird der Standard Wert genommen
// 					Ist pName==CRK_NEXT wird der nchste  Wert genommen
// 					Ist pName==CRK_ALL  werden alle Unterschlssel gelscht
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::DelKey(const zchar *pName)
{
zchar	cTemp[260];
bool	bExit;
int		iNum;


	Check();									// Sind Ebenen ok

	if(pName==CRK_ALL)
		{
		bExit = true;
		iNum  = KeyCount();

		for(iNum--;iNum>=0;iNum--)
			{
			KeySubName(cTemp,sizeof(cTemp)/sizeof(zchar),iNum);
			if(!DelKey(cTemp))bExit=false;
			}

		return bExit;
		}


	if(pName==CRK_NEXT)
		{
		KeySubName(cTemp,sizeof(cTemp),aSub[iLevel]);

		   iError =RegDeleteKey(hKeyNow,cTemp);
		if(iError==ERROR_SUCCESS)return true;
		
		if(!Up(cTemp,1))return false;			// Rekursives Lschen

		DelKey(CRK_ALL);

		if(!Down())return false;

		   iError =RegDeleteKey(hKeyNow,cTemp);
		if(iError==ERROR_SUCCESS)return true;

		aSub[iLevel]++;

		return false;
		}


	   iError =RegDeleteKey(hKeyNow,pName);
	if(iError==ERROR_SUCCESS)return true;

	if(!Up(pName,1))return false;				// Rekursives Lschen

	DelKey(CRK_ALL);

	if(!Down())return false;

	iError = RegDeleteKey(hKeyNow,pName);



return iError==ERROR_SUCCESS;
}

//*****************************************************************************
//*
//*		ValueCount
//*
//*****************************************************************************
//  Ergibt die Anzahl der Werte im aktuellen Key
int	 CRegKeys::ValueCount()
{
DWORD	dwNum;


	if(iLevel<0)Exit(ERROR_KEY);

	   iError=RegQueryInfoKey(hKeyNow,0,0,0,0,0,0,&dwNum,0,0,0,0);
	if(iError!=ERROR_SUCCESS)dwNum=0;


return dwNum;
}

//*****************************************************************************
//*
//*		KeyCount
//*
//*****************************************************************************
//  Ergibt die Anzahl der SubKeys im aktuellen Key
int	 CRegKeys::KeyCount()
{
DWORD	dwNum;


	if(iLevel<0)Exit(ERROR_KEY);

	   iError=RegQueryInfoKey(hKeyNow,0,0,0,&dwNum,0,0,0,0,0,0,0);
	if(iError!=ERROR_SUCCESS)dwNum=0;


return dwNum;
}

//*****************************************************************************
//*
//*		ValueName
//*
//*****************************************************************************
//  Ergibt den Namen eines Wertes und speichert ihn in einem Puffer
//	Der Speicherbereich fr den Puffer wird dabei extra angelegt.
//	pName	: Hier wird der Zeiger auf den Puffer abgelegt.
//	iNumber	: Ist die Nummer des Wertes. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt die Lnge des Namens oder 0 bei einem Fehler.
int CRegKeys::ValueName(zchar **pName,int iNumber)
{
int		 iLen;
zchar	*pPtr;


		pPtr=ValueName(&iLen,iNumber);
	if(!pPtr)
		{
		pName[0]=NULL;
		return 0;
		}

    iLen++;
	pName[0] = new zchar[iLen];
	memcpy(pName[0],pPtr,iLen*sizeof(zchar));


return iLen-1;
}

//*****************************************************************************
//*
//*		ValueName
//*
//*****************************************************************************
//  Speichert den Namen eines Wertes in einem Puffer.
//	pName	: Ist der Zeiger auf den Puffer.
//	iSize	: Ist die Gre des Puffers
//	iNumber	: Ist die Nummer des Wertes. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt die Lnge des Namens
int CRegKeys::ValueName(zchar *pName,int iSize,int iNumber)
{
int		 iLen;
zchar	*pPtr;


		pPtr=ValueName(&iLen,iNumber);
	if(!pPtr)
		{
		pName[0]=NULL;
		return 0;
		}

	iLen++;

	if(iLen>iSize)
		{
		pName[0]=NULL;
		return 0;
		}

	memcpy(pName,pPtr,iLen*sizeof(zchar));


return iLen-1;
}

//*****************************************************************************
//*
//*		ValueName
//*
//*****************************************************************************
//  Speichert den Namen eines Wertes im internen Namespuffer.
//	iNumber	: Ist die Nummer des Wertes. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt den Zeiger auf den Namen oder NULL bei einem Fehler
zchar *CRegKeys::ValueName(int iNumber)
{
DWORD	dwSize;


	if(iLevel  < 0)Exit(ERROR_KEY);
	if(iNumber==-1)iNumber=aPos[iLevel];


	dwLength = 0;
	dwSize   = 256;
	iError   = RegEnumValue(hKeyNow,iNumber,cName,&dwSize,0,&dwType,0,&dwLength);

	if(iError!=ERROR_MORE_DATA)
	if(iError!=ERROR_SUCCESS)
		{
		dwNameLen = 0;
		cName[0]  = 0;
		return 0;
		}
	
	dwNameLen = dwSize;


return cName;
}

//*****************************************************************************
//*
//*		ValueName
//*
//*****************************************************************************
//  Speichert den Namen eines Wertes im internen Namespuffer.
//	pSize   : Hier wird die Lnge des Namens gespeichert
//	iNumber	: Ist die Nummer des Wertes. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt den Zeiger auf den Namen oder NULL bei einem Fehler
zchar *CRegKeys::ValueName(int *pSize,int iNumber)
{
DWORD	dwSize;


	if(iLevel<0)
		{
		*pSize = 0;
		Exit(ERROR_KEY);
		}

	if(iNumber==-1)iNumber=aPos[iLevel];


	dwLength = 0;
	dwSize   = 256;
	iError   = RegEnumValue(hKeyNow,iNumber,cName,&dwSize,0,&dwType,0,&dwLength);

	if(iError!=ERROR_MORE_DATA)
	if(iError!=ERROR_SUCCESS)
		{
		dwNameLen = 0;
		cName[0]  = 0;
		*pSize    = 0;
		return 0;
		}
	
	dwNameLen = dwSize;
	*pSize    = dwSize;


return cName;
}



//*****************************************************************************
//*
//*		KeySubName
//*
//*****************************************************************************
//  Liest den Namen eines Sub-Schlssels aus.
//	iNumber	: Ist die Nummer des Sub-Schlssels. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt einen Zeiger auf den Namen des Sub-Schlssels oder NULL bei einem Fehler.
zchar *CRegKeys::KeySubName(int iNumber)
{
DWORD	dwSize;


	if(iLevel<0)Exit(ERROR_KEY);
	if(iNumber==-1)iNumber=aSub[iLevel];


	dwLength = 0;
	dwSize	 = 256;
	iError	 = RegEnumKeyEx(hKeyNow,iNumber,cName,&dwSize,0,0,0,0);


	if(iError!=ERROR_SUCCESS)
		{
		dwNameLen = 0;
		cName[0]  = 0;
		return 0;
		}

	dwNameLen = dwSize;



return cName;
}

//*****************************************************************************
//*
//*		KeySubName
//*
//*****************************************************************************
//  Liest den Namen eines Sub-Schlssels aus.
//	pSize	: Hier wird die Lnge des Namen gespeichert
//	iNumber	: Ist die Nummer des Sub-Schlssels. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt einen Zeiger auf den Namen des Sub-Schlssels oder NULL bei einem Fehler.
zchar *CRegKeys::KeySubName(int *pSize,int iNumber)
{
DWORD	dwSize;


	if(iLevel<0)
		{
		*pSize = 0;
		Exit(ERROR_KEY);
		}

	if(iNumber==-1)iNumber=aSub[iLevel];


	dwLength = 0;
	dwSize	 = 256;
	iError	 = RegEnumKeyEx(hKeyNow,iNumber,cName,&dwSize,0,0,0,0);


	if(iError!=ERROR_SUCCESS)
		{
		dwNameLen = 0;
		cName[0]  = 0;
		*pSize    = 0;
		return 0;
		}

	dwNameLen = dwSize;
	*pSize	  = dwSize;


return cName;
}

//*****************************************************************************
//*
//*		KeySubName
//*
//*****************************************************************************
//  Liest den Namen eines Sub-Schlssels aus, und speichert ihn in einem Puffer
//	Der Speicherbereich fr den Puffer wird dabei extra angelegt.
//	pName	: Hier wird der Zeiger auf den Puffer abgelegt.
//	iNumber	: Ist die Nummer des Sub-Schlssels. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt die Lnge des Namens oder 0 bei einem Fehler.
int	CRegKeys::KeySubName(zchar **pName,int iNumber)
{
int		 iLen;
zchar	*pPtr;



		pPtr=KeySubName(&iLen,iNumber);
	if(!pPtr)
		{
		pName[0]=NULL;
		return 0;
		}

	iLen++;
	pName[0]=new zchar[iLen];
	memcpy(pName[0],pPtr,iLen*sizeof(zchar));


return iLen-1;
}

//*****************************************************************************
//*
//*		KeySubName
//*
//*****************************************************************************
//  Speichert den Namen eines Sub-Schlssels in einem Puffer.
//	pName	: Ist der Zeiger auf den Puffer.
//	iSize	: Ist die Gre des Puffers
//	iNumber	: Ist die Nummer des Sub-Schlssels. Bei -1 wird die aktuelle
//			  Position genommen.
//	Ergibt die Lnge des Namens oder 0 bei einem Fehler.
int	CRegKeys::KeySubName(zchar *pName,int iSize,int iNumber)
{
int		 iLen;
zchar	*pPtr;


		pPtr=KeySubName(&iLen,iNumber);
	if(!pPtr)
		{
		pName[0]=NULL;
		return 0;
		}

	iLen++;

	if(iLen>iSize)
		{
		pName[0]=0;
		return 0;
		}

	memcpy(pName,pPtr,iLen*sizeof(zchar));


return iLen-1;
}

//*****************************************************************************
//*
//*		KeyName
//*
//*****************************************************************************
//  Holt den Namen eines Pfadteiles des aktuellen Pfades
//	iNumber		: Ist die Nummer des Pfadteiles
//					=0  Pfadteiles von Open(...)
//					>0 	Pfadteiles von Up  (...)
//  Ergibt einen Zeiger auf den Namen
zchar *CRegKeys::KeyName(int iNumber)
{


	if(iLevel<0)Exit(ERROR_KEY);
	
	if(iNumber==-1)
		{
		iNumber = iLevel;
		}
	else if(iNumber<0 || iNumber>iLevel)
		{
		Exit(ERROR_INVALID_LEVEL);
		}


return aStr[iNumber];
}

//*****************************************************************************
//*
//*		KeyName
//*
//*****************************************************************************
//  Holt den Namen eines Pfadteiles des aktuellen Pfades
//	pSize		: Hier wir die Lnge des Namnes gespeichert
//	iNumber		: Ist die Nummer des Pfadteiles
//					=0  Pfadteiles von Open(...)
//					>0 	Pfadteiles von Up  (...)
//  Ergibt einen Zeiger auf den Namen
zchar *CRegKeys::KeyName(int *pSize,int iNumber)
{
zchar *pName;


	if(iLevel<0)
		{
		*pSize = 0;
		Exit(ERROR_KEY);
		}

	if(iNumber==-1)
		{
		iNumber = iLevel;
		}
	else if(iNumber<0 || iNumber>iLevel)
		{
		Exit(ERROR_INVALID_LEVEL);
		}

	 pName = aStr[iNumber];
	*pSize = strzlen(pName);


return pName;
}

//*****************************************************************************
//*
//*		KeyName
//*
//*****************************************************************************
//  Speichert den Namen eines Pfadteiles des aktuellen Pfades, in einem Puffer.
//	pName	: Ist der Zeiger auf den Puffer.
//	iSize	: Ist die Gre des Puffers
//	iNumber	: Ist die Nummer des Pfadteiles
//				=0  Pfadteiles von Open(...)
//				>0 	Pfadteiles von Up  (...)
//	Ergibt die Lnge des Namens
int CRegKeys::KeyName(zchar *pName,int iSize,int iNumber)
{
int		 iLen;
zchar	*pPtr;


		pPtr=KeyName(&iLen,iNumber);
	if(!pPtr)
		{
		pName[0]=NULL;
		return 0;
		}

	iLen++;
	memcpy(pName,pPtr,iLen*sizeof(zchar));


return iLen-1;
}

//*****************************************************************************
//*
//*		KeyName
//*
//*****************************************************************************
//  Speichert den Namen eines Pfadteiles des aktuellen Pfades, in einem Puffer.
//	Der Speicherbereich fr den Puffer wird dabei extra angelegt.
//	pName	: Hier wird der Zeiger auf den Puffer abgelegt.
//	iNumber	: Ist die Nummer des Pfadteiles
//				=0  Pfadteiles von Open(...)
//				>0 	Pfadteiles von Up  (...)
//	Ergibt die Lnge des Namensint CRegKeys::KeyName(zchar **pName,int iNumber)
int CRegKeys::KeyName(zchar **pName,int iNumber)
{
int		 iLen;
zchar	*pPtr;


		pPtr=KeyName(&iLen,iNumber);
	if(!pPtr)
		{
		pName[0]=0;
		return 0;
		}

	iLen++;
	pName[0]=new zchar[iLen];
	memcpy(*pName,pPtr,iLen*sizeof(zchar));


return iLen-1;
}

//*****************************************************************************
//*
//*		KeyIndex
//*
//*****************************************************************************
//	Setzte den Index eines Pfadteilenames
//	iNumber	: Ist die Nummer des Pfadteiles
//				=0  Pfadteiles von Open(...)
//				>0 	Pfadteiles von Up  (...)
//	Ergibt den Index des Namens oder -1 bei einem Fehler.
int CRegKeys::KeyIndex(int iNumber)							
{
zchar	 cTemp[257];
INT		 iError;
DWORD	 dwSize;
DWORD	 dwNum;
DWORD	 dwPos;
zchar	*pPtr;
HKEY	 hKey;
int		 iLen;


	if(iLevel<=0)
		{
		return -1;
		}

	if(iNumber==-1)
		{
		iNumber = iLevel-1;
		}
	else if(iNumber<0 || iNumber>iLevel)
		{
		return -1;
		}

	pPtr = aStr[iNumber+1];
	hKey = aKey[iNumber  ];

	   iError=RegQueryInfoKey(hKey,0,0,0,&dwNum,0,0,0,0,0,0,0);
	if(iError!=ERROR_SUCCESS || dwNum==0)
		{
		return -1;
		}

	
	iLen = strzlen(pPtr);


	for(dwPos=0;dwPos<dwNum;dwPos++)
		{
		dwSize	 = 256;
		iError	 = RegEnumKeyEx(hKey,dwPos,cTemp,&dwSize,0,0,0,0);

		if(iError!=ERROR_SUCCESS)
			{
			break;
			}

		if(!(dwSize^iLen) && !strzicmp(pPtr,cTemp))
			{
			aSub[iNumber] = dwPos+1;
			return dwPos;
			}
		}



return -1;
}

		
//*****************************************************************************
//*
//*		Save
//*
//*****************************************************************************
//  Speichert den aktuellen Zweig aus den Baum in einer Datei.
//	Die Datei ist ein Windows eigenes Format.
//	pFilename	: Ist der Dateiname
//  Ergibt 'false' bei einem Fehler.
bool CRegKeys::Save(const zchar *pFilename)
{


	if(iLevel<0)Exit(ERROR_KEY);
	iError=RegSaveKey(hKeyNow,pFilename,0);


return iError==ERROR_SUCCESS;
}

//*****************************************************************************
//*
//*		Load
//*
//*****************************************************************************
//  Ld eine Zweig in den Baum der Registry aus einer Datei.
//	Die Datei ist ein Windows eigenes Format.
//	pPath		: Ist der Unterpfad der Registry
//	pFilename	: Ist der Dateiname mit den Daten
//  Ergibt 'false' bei einem Fehler.
bool CRegKeys::Load(const zchar *pPath,const zchar *pFilename)
{


	if(iLevel<0)Exit(ERROR_KEY);
	if(iOpenMode!=CRK_CREATE)Exit(ERROR_MODE);
	iError=RegLoadKey(hKeyNow,pPath,pFilename);


return iError==ERROR_SUCCESS;
}

//*****************************************************************************
//*
//*		Replace
//*
//*****************************************************************************
//  Tauscht einen Zweig im Baum aus. Der eine Zweig wird aus einer Datei
//	geladen, der aktuelle wird in einer Datei gespeichert.
//	pPath		: Ist der Unterpfad der Registry
//	pFileOld	: Ist die Datei aus der die Daten geladen werden
//	pFileNew	: Ist die Datei in der der Registy-Pfad gespeichert wird.
//  Ergibt 'false' bei einem Fehler
bool CRegKeys::Replace(const zchar *pPath,const zchar *pFileOld,const zchar *pFileNew)
{

	if(iLevel<0)Exit(ERROR_KEY);
	if(iOpenMode!=CRK_CREATE)Exit(ERROR_MODE);
	iError=RegReplaceKey(hKeyNow,pPath,pFileOld,pFileNew);


return iError==ERROR_SUCCESS;
}

//*****************************************************************************
//*
//*		RegPath
//*
//*****************************************************************************
//	Erzeugt den Standartpfad --> "Software\m_pszRegistryKey\m_pszAppName\Name"
//	und speichert ihn in einem Puffer
//	pPath	: Ist der Zeiger auf den Puffer
//	iLen	: Ist die Gre des Puffers
//	pName	: Ist der Unterpfad der an den Standartpfad angehngt wird.
//			  Bei NULL wird kein Unterpfad angehngt.
//	Ergibt die Lnge des erzeuten Pfades.
int CRegKeys::RegPath(zchar *pPath,int iLen,const zchar *pName)
{
const zchar   *pTemp1,*pTemp2;


	if(iLen<=0)return 0;

	#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	pTemp1=AfxGetApp()->m_pszRegistryKey;
	pTemp2=AfxGetApp()->m_pszAppName;
	#else
	pTemp1=Z("");
	pTemp2=Z("");
	#endif
				strzncpy(pPath,Z("Software"),iLen);		// Pfad erstellen
	if( pTemp1){strzncat(pPath,Z("\\")	    ,iLen);
				strzncat(pPath,pTemp1	    ,iLen); }
	if( pTemp2){strzncat(pPath,Z("\\") 	    ,iLen);
				strzncat(pPath,pTemp2	    ,iLen); }
	if( pName ){strzncat(pPath,Z("\\") 	    ,iLen);
				strzncat(pPath,pName 	    ,iLen); }

	pPath[iLen-1]=0;



return (int)strzlen(pPath);
}

//*****************************************************************************
//*
//*		Path
//*
//*****************************************************************************
// 	Speichert den aktuellen Schlsselpfad in einem Puffer
//	pMem		: Ist der Zeiger auf den Puffer
//	iLen		: Ist die Lnge des Puffers
//	iMode		: 0 = Gesamten Pfad mit  HKEY_... speichern
//				  1 = Gesamten Pfad ohne HKEY_... speichern
//				  2 = Pfad ab Ebene 0 speichern
//	Ergibt die Lnge des Pfades in Zeichen bzw. -1 bei einem Fehler
int	CRegKeys::Path(zchar *pMem,int iLen,int iMode)
{
int		i,j,iPos=0;



	if(iLen  <=0)return -1;
	pMem[0]=0;
	if(iLevel< 0)return -1;

	if(iMode==0)									// HKEY_... Namen
		{
		for(i=0;aRootName[i];i++)
			{
			if(hKeyInit!=aRootKeys[i])continue;
			j=(int)strzlen(aRootName[i]);
			if(j+2>iLen)return -1;
			memcpy(pMem,aRootName[i],j*sizeof(zchar));
			iPos+=j;
			if(!*aStr[0])break;
			pMem[iPos]='\\';
			iPos++;
			break;
			}

		if(!aRootName[i])return -1;
		}


	for(i=(iMode<=1)? 0:1;i<=iLevel;i++)			// Sub-Pfad Namen
		{
		j=(int)strzlen(aStr[i]);
		if(j<=0)continue;
		if(iPos+j+2>iLen)return -1;
		memcpy(pMem+iPos,aStr[i],j*sizeof(zchar));
		iPos+=j;
		if(i==iLevel)break;
		pMem[iPos]='\\';
		iPos++;
		}

	if(iPos>=iLen)return -1;
	pMem[iPos]=0;


return 0;
}

//*****************************************************************************
//*
//*		GetInitKey
//*
//*****************************************************************************
//	Einen Init-Key holen der zu einem Pfad passt
//	hKey	: Hier wird der gefundene Init-Key abgelegt.
//	pPath	: Ist der Pfad nach dem gesucht wird. Vom Pfad wird nur das
//			  erste Segment (..1...\xxxxxx\yyyyyy) verglichen.
//	Ergibt die Lnge des ersten Segmentes mit Backslash, bzw. 0 bei einem Fehler.
int CRegKeys::GetInitKey(HKEY &hKey,const zchar *pPath)
{
int		i,j;


	if(pPath)
		{
		for(i=0;aRootName[i];i++)
			{
			j=(int)strzlen(aRootName[i]);
			if(strznicmp(pPath,aRootName[i],j))continue;
			if(pPath[j]=='\\'){hKey=aRootKeys[i];return j+1;}
			if(pPath[j]== 0  ){hKey=aRootKeys[i];return j;  }
			}
		}

	hKey = HKEY_CURRENT_USER;


return 0;
}

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
#ifdef		_DEBUG

bool   CRegKeys::Open (const zchar  *pPath,int iMode){return Open(0,pPath,iMode);}
bool   CRegKeys::Open (){return Open(HKEY_CURRENT_USER,0,CRK_CREATE);}
bool   CRegKeys::Jump (){Down(); return Up(CRK_NEXT);}
bool   CRegKeys::Next (){        return Up(CRK_NEXT);}
int    CRegKeys::Level(){        return iLevel      ;}

bool   CRegKeys::SetMulti(const  zchar   *pVal,DWORD dwLen,const zchar  *pName){return Set((void*)pVal,dwLen                              ,pName,REG_MULTI_SZ);}
bool   CRegKeys::Set(const       zchar   *pVal,const zchar  *pName){return Set((const void*) pVal,(int)strzlen(        pVal)*sizeof(zchar),pName,REG_SZ      );}
bool   CRegKeys::Set(const       zcharu  *pVal,const zchar  *pName){return Set((const void*) pVal,(int)strzlen((zchar*)pVal)*sizeof(zchar),pName,REG_SZ      );}
bool   CRegKeys::Set(            int      iVal,const zchar  *pName){return Set((const void*)&iVal,sizeof (        iVal)                   ,pName,REG_DWORD   );}
bool   CRegKeys::Set(float               &fVal,const zchar  *pName){double dVal=fVal;return Set(dVal,pName);}

bool   CRegKeys::GetMulti(       zchar  **pVal,const zchar  *pName           ){return Get((void**) pVal                              ,pName,REG_MULTI_SZ);}
bool   CRegKeys::Get(            int     &iVal,const zchar  *pName           ){return Get((void* )&iVal,sizeof(iVal)                 ,pName,REG_DWORD   );}
bool   CRegKeys::Get(unsigned    int     &uVal,const zchar  *pName           ){return Get((void* )&uVal,sizeof(uVal)                 ,pName,REG_DWORD   );}
bool   CRegKeys::Get(            long    &lVal,const zchar  *pName           ){return Get((void* )&lVal,sizeof(lVal)                 ,pName,REG_DWORD   );}
bool   CRegKeys::Get(unsigned    long    &uVal,const zchar  *pName           ){return Get((void* )&uVal,sizeof(uVal)                 ,pName,REG_DWORD   );}
bool   CRegKeys::Get(            zchar   *pVal,const zchar  *pName           ){return Get((void* ) pVal,260*sizeof(zchar)            ,pName,REG_SZ      );}
bool   CRegKeys::Get(            zcharu  *pVal,const zchar  *pName           ){return Get((void* ) pVal,260*sizeof(zchar)            ,pName,REG_SZ      );}
bool   CRegKeys::Get(            zchar  **pVal,const zchar *pName,DWORD uType){return Get((void**) pVal,                              pName,uType       );}
bool   CRegKeys::Get(            zcharu **pVal,const zchar *pName,DWORD uType){return Get((void**) pVal,                              pName,uType       );}
bool   CRegKeys::Get(            zchar   *pVal,DWORD dwLen,const zchar *pName){return Get((void* ) pVal,dwLen                        ,pName,REG_SZ      );}
bool   CRegKeys::Get(            zcharu  *pVal,DWORD dwLen,const zchar *pName){return Get((void* ) pVal,dwLen                        ,pName,REG_SZ      );}
bool   CRegKeys::GetMulti(       zchar   *pVal,DWORD dwLen,const zchar *pName){return Get((void* ) pVal,dwLen                        ,pName,REG_MULTI_SZ);}

DWORD  CRegKeys::Type (){return  dwType   ;}
DWORD  CRegKeys::Len  (){return  dwLength ;}
HKEY   CRegKeys::Key  (){return  hKeyNow  ;}
int    CRegKeys::Error(){return  iError   ;}
zchar *CRegKeys::Name (){return  cName    ;}
zchar *CRegKeys::Name (int &iLen){iLen=dwNameLen;return cName;}

#if defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
bool CRegKeys::Set(CString  sStr,const zchar *pName){return Set((const void*)((LPCTSTR)sStr),sStr.GetLength(),pName,REG_SZ);}
#endif

#endif

