//*****************************************************************************
//*
//*
//*		RegText.cpp
//*
//*
//*****************************************************************************
//
//	In/Exportieren von Registry Eintrgen in Text Dateien
//
//	Funktionen:
//
//		bool Export(char *Datei,HKEY hKey,char *pPath="",int iMode=0)
// 		bool Export(char *Datei,          char *pPath   ,int iMode=0)
// 		bool Import(char *Datei)
//
//	Format fr Konfigurations Registry (*.crg)
//
//		CFGEDIT4
//
//		[HKEY_LOCAL_MACHINE\$(Var1)\AZ\$(Var2)\Text]
//
//		"Type1"	= "Text $(Var1) Text $$(kein Symbol nichts)  $ normaler Text"
//		"Type2"	= dword:$(Var1)
//		"Type3"	= hex:00,$(Var1),00,$(Var2)
//
//		Bei allen $(???) Symbolen wird $(???) durch den Text der
//		Variabel ??? ersetzt.
//
#include	"RegText.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)			L##a
#define 	zchar			wchar_t
#define 	strzlen			wcslen
#define 	strznicmp		wcsnicmp
#else
#define 	Z(a)			a
#define 	zchar			char
#define 	strzlen			strlen
#define 	strznicmp		strnicmp
#endif




#define 	PUFFER_LEN	4096
#define 	PUFFER_READ	(PUFFER_LEN-1024)
#define		PUFFER_ADD(pMem,iOld,iNew)						\
			{												\
			zchar  *pPtr;									\
			pPtr = new zchar[iNew+1];			 			\
			memcpy(pPtr,pMem,iOld*sizeof(zchar));			\
			iOld = iNew;									\
			if(pMem)delete pMem;							\
			pMem = pPtr;									\
			}

char     *(_cdecl *CRegText::m_pGetVarNameA)(const char   *)=0;
wchar_t  *(_cdecl *CRegText::m_pGetVarNameW)(const wchar_t*)=0;

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CRegText::CRegText()
{

	m_bIsOpen   = 0;
	m_pTempText = 0;									// Puffer fr LineCfg
	m_iTempSize = 0;									// Puffergre

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CRegText::~CRegText()
{
	if(m_bIsOpen)Close();
	if(m_pTempText)delete m_pTempText;
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Ergibt 'false' bei einem Fehler
bool CRegText::Close()
{

	if(!m_bIsOpen)return false;

	m_bEofFlag = true;
	m_bIsOpen  = false;
	m_sStream.Close();


return true;
}

//*****************************************************************************
//*
//*		Export
//*
//*****************************************************************************
//	Einen Registry-Zweig in eine Datei exportieren.
//	pFile		: Ist der Name der Datei in die Exportiert wird.
//	hKey		: Bestimmt den Registry-Abschnitt (HKLM,HKCU...)
//				  Wenn hKey nicht angegeben ist, wird HKCU benutzt.
//	pPath		: Ist der Pfad des Registry-Zweigs.
//				  Wenn kein Pfad angegeben ist wird
//					HKCU/Software/<Vendor>/<App> benutzt.
//	iMode		: Ist der Wert ungleich 0 werden die Daten an die Datei angehngt.
//	Ergibt 'false' bei einem Fehler.
bool CRegText::Export(const zchar *pFile,const zchar *pPath,int iMode)
{
	if(!m_sKey.Open(pPath,CRK_READ|CRK_WOW64_64KEY))return false;
	return Export(pFile,iMode);
}

bool CRegText::Export(const zchar *pFile,HKEY hKey,const zchar *pPath,int iMode)
{
	if(!m_sKey.Open(hKey,pPath,CRK_READ|CRK_WOW64_64KEY))return false;
	return Export(pFile,iMode);
}

bool CRegText::Export(const zchar *pFile,int iMode)
{

	if(m_bIsOpen)Close();

	if(!iMode)
		{
		if(!m_sStream.Open(pFile,CFB_CREATE))return false;
		m_bIsOpen=1;
		}
	else{
		if(!m_sStream.Open(pFile,CFB_WRITE,1))return false;
		m_bIsOpen=1;
		}


	m_bEofFlag = false;
	m_iFileLen = m_sStream.GetLength();						// Dateilnge
	m_iFilePos = m_sStream.Tell();							// Dateileseposition


	if(!iMode)
		{
		#ifdef UNICODE
		m_sStream.Write("\xFF\xFE",2);
		m_sStream.Write(L"Windows Registry Editor Version 5.00");
		#else
		m_sStream.Write("REGEDIT4");
		#endif
		}


//********************* Daten auslesen *****************************************


	for(;;)
		{
		WriteKey();
		if(m_sKey.Up())continue;

		for(;;)
			{
			if(!m_sKey.Down())break;
			if( m_sKey.Up  ())break;
			}

		if(m_sKey.Level()<=0)break;
		}

	Close();


return true;
}

//*****************************************************************************
//*
//*		WriteKey
//*
//*****************************************************************************
//	Speichert alle Schlsselwerte von m_sKey
void CRegText::WriteKey()
{
zchar	cBuffer[4096];
zchar  *pName=Z("@");
zchar  *pMem=0;
int		iNum=0;
int		iPos;
int		iOff;
int		iLen;



	m_sKey   .Path(cBuffer,sizeof(cBuffer));				// Schlsselnamen
	m_sStream.Write(Z("\r\n\r\n["));
	m_sStream.Write(cBuffer );
	m_sStream.Write(Z("]"));

	m_sKey.Get((void**)&pMem,0,REG_BINARY);

	for(;;)
		{
		if(!iNum)											// Kein Standardwert ?
			{
			if(m_sKey.Len()<=0 || !pMem || !pMem[0])
				{
				if(pMem){delete pMem;pMem=0;}
				if(!m_sKey.Get((void**)&pMem))break;
				pName=m_sKey.Name();
				iNum++;
				continue;
				}
			}

		m_sStream.Write(Z("\r\n\""),(iNum)? 3:2);			// Key-Namen speichern
		m_sStream.Write(pName);

		if(iNum)m_sStream.Write(Z("\""));

		switch(m_sKey.Type())
			{
		case REG_DWORD:

			htostr(pMem,cBuffer,4,HEX_NULL|HEX_INVERT);
			m_sStream.Write(Z("=dword:"));
			m_sStream.Write(cBuffer);
			break;

		case REG_SZ:

			iLen = m_sKey.Len()-1;
			if(iLen<0)iLen=0;
			m_sStream.Write(Z("=\""));

			for(iPos=0;iPos<iLen;iPos+=512)
				{
				iOff=512;
				if(iOff+iPos>iLen)iOff=iLen-iPos;
				MemToStr(cBuffer,sizeof(cBuffer),pMem+iPos,iOff);
				m_sStream.Write(cBuffer);
				}

			m_sStream.Write(Z("\""));
			break;

		default:

			m_sStream.Write(Z("=hex"));
			if(m_sKey.Type())
				{
				itostr(m_sKey.Type(),cBuffer);
				m_sStream.Write(Z("("));
				m_sStream.Write(cBuffer);
				m_sStream.Write(Z(")"));
				}

			m_sStream.Write(Z(":"));

			cBuffer[2]	= ',';
			cBuffer[3]	= 0 ;
			iLen		= m_sKey.Len()-1;
			iPos		= (78-(int)strzlen(pName)-((iNum)? 7:5))/3;

			for(iOff=0;iOff<=iLen;iOff++)
				{
				cBuffer[0]="0123456789abcdef"[(pMem[iOff]>>4)&0x0F];
				cBuffer[1]="0123456789abcdef"[ pMem[iOff]    &0x0F];

				iPos--;
				m_sStream.Write(cBuffer,(iOff==iLen)? 2:3);
				if(iPos<=0)
					{
					m_sStream.Write(Z("\\\r\n  "));
					iPos=25;
					}
				}

			break;
			}


		if(pMem)
			{
			delete pMem;
			pMem=0;
			}

		if(!m_sKey.Get((void**)&pMem))break;				// Nchster Wert

		pName = m_sKey.Name();
		iNum++;
		}

	if(pMem)delete pMem;

}

//*****************************************************************************
//*
//*		GetFormat
//*
//*****************************************************************************
//	Das Dateiformat bestimmen.
//	pPtr		: Zeiger auf die ersten Bytes in der Datei
//	iLen		: Ist die Lnge der Daten
//	Ergibt 0 fr ANSI oder 1 fr UNICODE.
int	CRegText::GetFormat(const char *pPtr,int iLen)
{


	if(iLen>2 && pPtr[0]=='\xFF' && pPtr[1]=='\xFE')
		{
		return 1;
		}

	if(iLen>6 && pPtr[0] && pPtr[2] && pPtr[4] && !pPtr[1] && !pPtr[3] && !pPtr[5])
		{
		return 1;
		}


return 0;
}


//*****************************************************************************
//*
//*		Import
//*
//*****************************************************************************
//	Registry-Zweig aus einer Datei oder einem Speicherbereich importieren.
//	pFile		: Datei aus der der Registry-Zweig importiert werden soll.
//	pMem		: Zeiger auf den Speicherbereich aus dem importiert werden soll.
//	iLen		: Lnge des Speicherbereiches in Byte
//	iFormat		: Ist das Format des Speichers
//				   -1 = automatisch
//					0 = Ansi-Text
//					1 = Unicode-Text
//	Ergibt 'false' bei einem Fehler.
bool CRegText::Import(int iLen,const void *pMem,int iFormat)
{
char	*pPtr;



	if(m_bIsOpen)Close();

	pPtr=(char*)pMem;


	if(iFormat<0)											// Automatisches Format
		{
		iFormat = GetFormat(pPtr,iLen);
		}


	switch(iFormat)											// Stream ffnen
		{
	case 0:	if(!m_sStream.OpenMem((void*)pMem,iLen,CFB_READ,CFB_ANSITOMODE))return false;	break;
	case 1:	if(!m_sStream.OpenMem((void*)pMem,iLen,CFB_READ,CFB_WIDETOMODE))return false;	break;
	default:return false;
		}

	m_bIsOpen  = true;


return Import();
}

bool CRegText::Import(const zchar *pFile)
{
char	cTemp[16];
int		iFormat;
int		iLen;



	if(m_bIsOpen)Close();
	if(m_sStream.Open(pFile,CFB_READ)==0)return false;


	iLen	= m_sStream.Read(cTemp,sizeof(cTemp));
	iFormat	= GetFormat(cTemp,iLen);


	switch(iFormat)											// Stream ffnen
		{
	case 0:													// Ansi-Format
			#ifdef UNICODE

			   m_sStream.Close();
			if(m_sStream.OpenAnsi(pFile,CFB_READ)==0)return false;

			#else

			m_sStream.Seek(0);

			#endif
			break;

	case 1:													// Unicode-Format
			#ifdef UNICODE

			if(iLen>2 && cTemp[0]=='\xFF' && cTemp[1]=='\xFE')
					m_sStream.Seek(2);
			else	m_sStream.Seek(0);

			#else

			   m_sStream.Close();
			if(m_sStream.OpenWide(pFile,CFB_READ)==0)return false;

			if(iLen>2 && cTemp[0]=='\xFF' && cTemp[1]=='\xFE')
					m_sStream.Seek(1);
			else	m_sStream.Seek(0);

			#endif

			break;

	default:return false;
		}


	m_bIsOpen  = true;



return Import();
}

//*****************************************************************************
//*
//*		Import
//*
//*****************************************************************************
//	Registry-Zweig aus einem Stream importieren.
//	Ergibt 'false' bei einem Fehler.
bool CRegText::Import()
{
int			 iNum,iOff,iCnt,iLen,iVal,iSub,iSize,iType;
bool		 bPathOk,bOk=0,bCfgFlag=0;
zchar		 cBuffer[260],*pName,*pData=0;
zchar		 cChar,*pText,*pTemp;



	m_bEofFlag  = false;
	m_iFileLen  = m_sStream.GetLength();					// Dateilnge
	m_iFilePos  = m_sStream.Tell();							// Dateileseposition
	iCnt		= m_sStream.Text(0,&pText);
	iNum		= strwlen(pText,0xE0);



	if(!strznicmp(pText+iNum,Z("Windows Registry Editor Version 5.00"),36)){bOk=1;iNum+=36-8;}
	if(!strznicmp(pText+iNum,Z("REGEDIT4"),8)){bOk=1;}
	if(!strznicmp(pText+iNum,Z("CFGEDIT4"),8)){bOk=1;bCfgFlag=1;}
	if(!m_pGetVarNameA && !m_pGetVarNameW)bCfgFlag=0;


	if(!bOk || strcwlen(pText+iNum+8,0xE0))
		{
		Close();
		return false;
		}

	m_sStream.Text(iNum+8);

	bPathOk = false;

	for(;;)
		{
			iCnt=m_sStream.Text(0,&pText);
		if(!iCnt)break;

		iNum = m_sStream.TextEmpty(1);						// Leerzeichen berspringen
		m_sStream.Text(iNum);

		if(iNum>=iCnt || !pText[iNum])
			{
			m_sStream.Text(1);
			continue;
			}

		iCnt = m_sStream.Text(0,&pText);

		if(pText[0]=='[')									// Neuer Pfad
			{
			bPathOk	= false;
			iNum    = strchar(0x40,pText+1,']')+1;

			if(pText[iNum]!=']')
				{
				m_sStream.Text(iNum);
				continue;
				}

			pText[iNum] = 0;
			if(bCfgFlag)pText=LineCfg(pText);

			if(pText[1]=='-')								// Pfad lschen
				{
				bPathOk = 0;

				iSub = strdwlen(pText+iNum,0x00100000,iNum);
				iSub = iNum-iSub;

				if(iSub<3 || !pText[iSub])
					{
					m_sStream.Text(iNum+1);
					continue;
					}

				pText[iSub-1] = 0;

				if(!m_sKey.Open(pText+2,CRK_READ|CRK_WOW64_64KEY))
					{
					m_sStream.Text(iNum+1);
					continue;
					}

				m_sKey.DelKey(pText+iSub);
				m_sKey.Close();
				m_sStream.Text(iNum+1);
				continue;
				}
															// Pfad erstellen
			bPathOk=m_sKey.Open(pText+1,CRK_CREATE|CRK_WOW64_64KEY);
			m_sStream.Text(iNum+1);
			continue;
			}

		if(!bPathOk)										// Kein Schlssel offen
			{
			iNum = strcwlen(pText+1,0x40)+1;
			m_sStream.Text(iNum);
			continue;
			}


		iLen = m_sStream.TextLineLen();

		if(pText[0]=='\"')									// Namen laden
			{
			iNum=m_sStream.TextString(cBuffer,sizeof(cBuffer),&iCnt);
			if(pText[iCnt+1]!='\"' || iNum>=iLen){m_sStream.Text(iLen);continue;}
			m_sStream.Text(iNum,&pText);iLen-=iNum;
			pName=cBuffer;
			}
		else{
			if(pText[0]!='@' || iLen<=0){m_sStream.Text(iLen);continue;}
			m_sStream.Text(1,&pText);iLen--;
			pName=0;
			}

		iNum=m_sStream.TextChar(&cChar);
		if(cChar!='=' || iNum>=iLen){m_sStream.Text(iLen);continue;}
		m_sStream.Text(iNum,&pText);iLen-=iNum;

		if(*pText=='-')										// Wert lschen einlesen
			{
			m_sKey.Del(pName);
			m_sStream.Text(iLen);
			continue;
			}

		if(!strznicmp(pText,Z("dword:"),6))					// Zahl einlesen
			{
			pText+=6;
			if(bCfgFlag)pText=LineCfg(pText);
			iVal=strtoi(pText,0xC000+16,&iCnt);
			if(iCnt)m_sKey.Set(iVal,pName);
			m_sStream.Text(iLen);
			continue;
			}

		if(!strznicmp(pText,Z("hex"),3))					// Hex Dump einlesen
			{
			iType = (pText[3]!='(')? 0:strtoi(pText+4);
			iNum  = strcwlen(pText,0x00400040);

			if(pText[iNum]!=':')
				{
				m_sStream.Text(iLen);
				continue;
				}

			iNum += 1;
			iNum += strwlen(pText+iNum,0xA0);

			m_sStream.Text(iNum,&pText);
			iLen -= iNum;
			iSize = 0;
			iOff  = 0;

			for(;;) 										// Bytes einlesen
				{
				iNum=strcwlen(pText,0x00103040);
				if(bCfgFlag)pText=LineCfg(pText,10);
				iVal=strtoi(pText,0xC000+16,&iCnt);

				if(iCnt<=0)break;
				if(iOff<=iSize)PUFFER_ADD(pData,iOff,iOff+256);

				pData[iSize]=iVal;
				iSize++;

				     m_sStream.Text(iNum,&pText);iLen-=iNum;
				iNum=m_sStream.TextChar(&cChar);

				if(cChar==',')
					{
					iCnt=m_sStream.Text(iNum,&pText);
					iLen-=iNum;
					if(!iCnt)break;
					}

				if(pText[0]!='\\')continue;					// Weitere Zeilen angngen

					iCnt=m_sStream.Text(1,&pText);
				if(!iCnt)break;

				       m_sStream.Text(iLen,&pText);			// Vor auf neue Zeile
				iNum = m_sStream.TextEmpty();
				       m_sStream.Text(iNum,&pText);
				iLen = m_sStream.TextLineLen();
				}

			m_sKey.Set((void*)pData,iSize,pName,iType);
			if(pData){delete pData;pData=0;}
			m_sStream.Text(iLen);

			continue;
			}

		if(pText[0]=='\"')									// String einlesen
			{
			iOff=0;iSize=0;

			for(;;)
				{
				iNum   = m_sStream.TextLineLen(1);
				PUFFER_ADD(pData,iOff,iSize+iNum+1);
				memcpy(pData+iSize,pText,iNum);
				iSize += iNum;
				pData[iSize]=0;

				iCnt = m_sStream.Text(iNum,&pText);
				iLen = m_sStream.TextLineLen();

				if(iCnt    ==  0 )break;
				if(pText[0]=='\r')break;
				if(pText[0]=='\n')break;
				}

			pTemp	= new zchar[iSize+1];
			pText	= pTemp;
			iNum	= StrToMem(pText,iSize,pData);
			pText[iNum]=0;

			if(bCfgFlag)pText=LineCfg(pText);
			m_sKey.Set(pText,pName);
			if(pData){delete pData;pData=0;}
			m_sStream.Text(iLen);
			delete pTemp;

			continue;
			}


		m_sStream.Text(iLen);
		}



	if(pData)delete pData;

	Close();



return true;
}

//*****************************************************************************
//*
//*		MemToStr
//*
//*****************************************************************************
//	Wandelt einen Speicherbereich in einen String um.
//	Am Anfang und am Ende wird ein Anfhrungszeichen eingefgt.
//	Zeichen '\n' und '\r' '\\' '\"' werden in Zeichenzetten umgewandelt.
//	pStr		: Ist der Zeiger auf den Textpuffer.
//	iMax		: Ist die Gre des Textpuffers.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	iSize		: Ist die Gre des Speicherbereich.
//	Ergibt die Lnge des Strings in Zeichen.
int CRegText::MemToStr(zchar *pStr,int iMax,const zchar *pMem,int iSize)
{
int			iLen=0,iAll;
unsigned 	ucByte;


	iAll=iMax;
	iMax--;

	while(iLen<iSize)
		{
		if(iMax<=2)break;
		ucByte = pMem[iLen];
		iLen++;

		if(ucByte!='\r' && ucByte!='\\' && ucByte!='\n' && ucByte!='\"')
			{
			*pStr =ucByte;
			iMax--;pStr++;
			continue;
			}

		if(iMax<=2)break;

 		*pStr='\\';
		iMax--;
		pStr++;

		if(ucByte=='\\')*pStr='\\';
		if(ucByte=='\"')*pStr='\"';
		if(ucByte=='\r')*pStr='r' ;
		if(ucByte=='\n')*pStr='n' ;

		iMax--;
		pStr++;
		}


	   iMax++;
	if(iMax>0)*pStr=0;



return iAll-iMax;
}

//*****************************************************************************
//*
//*		StrToMem
//*
//*****************************************************************************
//	Wandelt einen String in einen Speicherbereich um.
//	Spezial Zeichen // /n /r /" werden bercksichtigt.
//	Fngt der String mit '"' an so werden nur die Zeichen zwischen den
//	Anfhrungszeichen ausgewertet.
//	pMem		: Ist der Zeiger auf den Speicherbereich.
//	iLen		: Ist die maximale Gre des Speicherbereiches.
//	pStr		: Ist der Zeiger auf den String.
//	Ergibt die Lnge des pMem-Bereiches
int	CRegText::StrToMem(zchar *pMem,int iSize,const zchar *pStr)
{
int			iStop=0;
int			iLen=0;
zchar		cChar;


	if(*pStr=='\"'){iStop=1;pStr++;}					// Anfhrungs-Zeichen

	while(iLen<iSize)
		{
			cChar=*pStr;
		if(!cChar)break;
		if( cChar=='"' && iStop )break;
		if( cChar=='\\')								// Escape Sequenzen
			{
				 if(pStr[1]=='\\'){pStr++;cChar='\\';}
			else if(pStr[1]=='"' ){pStr++;cChar='"' ;}
			else if(pStr[1]=='r' ){pStr++;cChar='\r';}
			else if(pStr[1]=='n' ){pStr++;cChar='\n';}
			}

		pMem[iLen]=cChar;
		pStr++;
		iLen++;
		}


return iLen;
}


//*****************************************************************************
//*
//*		LineCfg
//*
//*****************************************************************************
//	Konvertiert alle $(Var) Symbole in einer Zeile
//	pLine	: Ist der Zeiger auf die Zeile.
//	iMax	: Ist die Zeilenlnge,
//	Ergibt einen Zeiger auf die neue Zeile
zchar *CRegText::LineCfg(zchar *pLine,int iMax)
{
char	   *pPtrA=0;
wchar_t	   *pPtrW=0;
zchar	   *pVar =0;
zchar		cName[128];
zchar		cChar;
int			iPos;
int			iNow;
int			iLen;
int			iOut;



	for(iPos=0;iPos<iMax;iPos++)						// Suche Symbolangabe
		{
		cChar = pLine[iPos];

		if(cChar== 0  )return pLine;
		if(cChar=='\r')return pLine;
		if(cChar=='\n')return pLine;
		if(cChar!='$' )continue;
		if(pLine[iPos+1]=='(')break;
		if(pLine[iPos+1]=='$')break;
		}


	if(!m_pTempText || iPos+64>m_iTempSize)				// Neuen Puffer erzeugen
		{
		if(m_pTempText)delete m_pTempText;
		m_pTempText = new zchar[iPos+512+1];
		m_iTempSize = iPos+512;
		}

	iNow = iPos;
	memcpy(m_pTempText,pLine,iPos);

	for(;iPos<iMax;iPos++)								// Suche Symbolangabe
		{
		cChar = pLine[iPos];

		if(cChar== 0  )break;
		if(cChar=='\r')break;
		if(cChar=='\n')break;

		if(cChar!='$' || pLine[iPos+1]!='(')			// Zeichen speichern
			{
			if(cChar=='$' && pLine[iPos+1]=='$')iPos++;

			m_pTempText[iNow]=cChar;
			iNow++;

			if(iNow>=m_iTempSize)						// Puffer vergrern
				{
				PUFFER_ADD(m_pTempText,m_iTempSize,iNow+256);
				}

			continue;
			}

		iPos+=2;										// Hole Variabelnamen

		for(iLen=0;iLen<sizeof(cName)-1;iLen++)
			{
			if(pLine[iPos+iLen]==')')break;
			}

		if(pLine[iPos+iLen]!=')')continue;

		memcpy(cName,pLine+iPos,iLen);
		cName[iLen]=0;
		iPos+=iLen;

		if(m_pGetVarNameA)								// Ersatztext holen
			{
			#ifdef UNICODE

			pPtrA = (char*)cName;

			for(iOut=1;iOut<=iLen;iOut++)
				{
				pPtrA[iOut] = pPtrA[iOut*2];
				}

				pPtrA = m_pGetVarNameA(pPtrA);
			if(!pPtrA)continue;

			iLen = strlen(pPtrA);
			iOut = 1;

			#else

				pVar=m_pGetVarNameA(cName);				// Ersatztext holen
			if(!pVar)continue;

			iLen = (int)strlen(pVar);
			iOut = 0;

			#endif
			}
		else if(m_pGetVarNameW)
			{
			#ifdef UNICODE

				pVar=m_pGetVarNameW(cName);				// Ersatztext holen
			if(!pVar)continue;

			iLen = wcslen(pVar);
			iOut = 0;

			#else

			if(iLen>=sizeof(cName)/2)continue;

			pPtrW = (wchar_t*)cName;

			for(iOut=iLen;iOut>=0;iOut--)
				{
				pPtrW[iOut] = cName[iOut];
				}

				pPtrW = m_pGetVarNameW(pPtrW);
			if(!pPtrW)continue;

			iLen =  (int)wcslen(pPtrW);
			iOut = -1;

			#endif
			}
		else{
			continue;
			}


 		if(iNow+iLen>=m_iTempSize)						// Puffer vergrern
			{
			PUFFER_ADD(m_pTempText,m_iTempSize,iNow+iLen+256);
			}

		if(iOut==0)										// Ersatztext einfgen eins zu eins
			{
			memcpy(m_pTempText+iNow,pVar,iLen*sizeof(TCHAR));
			iNow+=iLen;
			}
		else if(iOut>0)									// Ersatztext einfgen ANSI-->WIDE
			{
			for(iOut=0;iOut<=iLen;iOut++,iNow++)
				{
				m_pTempText[iNow] = pPtrA[iOut];
				}
			}
		else if(iOut<0)									// Ersatztext einfgen WIDE-->ANSI
			{
			for(iOut=0;iOut<=iLen;iOut++,iNow++)
				{
				m_pTempText[iNow] = (char)((pPtrW[iOut]<256)? pPtrW[iOut]:255);
				}
			}
		}


	m_pTempText[iNow]=0;


return m_pTempText;
}
