//*****************************************************************************
//*
//*
//*		ResourceHacker.cpp
//*
//*
//*****************************************************************************
#include	<windows.h>
#include	<winbase.h>
#include	<imagehlp.h>
#include	<malloc.h>
#include	<string.h>
#include	<wchar.h>
#include	<tchar.h>

#include	"AZ_Lib.h"
#include	"ResourceHacker.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifndef	   _WIN64
#ifndef		DWORD_PTR
#define		DWORD_PTR				DWORD
#endif
#endif

#ifndef 	PRINT_ENTRIES
#define 	PRINT_ENTRIES			0
#endif

#define 	MAX_LEVELS				3
#define 	WORD_SIGN(a,b)			( WORD)(((unsigned char)(a)+((unsigned char)(b)<<8)))
#define 	DWORD_SIGN(a,b,c,d)		(DWORD)(((unsigned char)(a)+((unsigned char)(b)<<8)+((unsigned char)(c)<<16)+((unsigned char)(d)<<24)))
#define		PTR_ADD_64(ptr,add)		(((const char*)(ptr))+(ULONGLONG)(add))
#define		PTR_ADD_32(ptr,add)		(((const char*)(ptr))+(DWORD    )(add))

typedef IMAGE_RESOURCE_DATA_ENTRY	DataEntry;
typedef IMAGE_RESOURCE_DIRECTORY	DataDir;

typedef struct
	{
	UINT			uLevels;						// Anzahl der Ebenen im Namen
	BOOL			aIsId [MAX_LEVELS];				// Ist ein Name als ID definiert
	DWORD			aIds  [MAX_LEVELS];				// ID's der namen (0xFFFFFFFF bei Texten)
	LPWSTR			aNames[MAX_LEVELS];				// Namen der Texte (<LEN><Text...><0>)
	DataEntry		sData;							// Resourcen Eintrag
	unsigned char  *pDataPtr;						// Zeiger auf die Daten
	BOOL			bFreePtr;						// Den Datenzeiger deallokieren
	}ResourceItem;


//*****************************************************************************
//*
//*		ResourceFileToMem
//*
//*****************************************************************************
//	Rechnet den Offset in der Datei in einen Offset im Speicher um
//	pInstance		: Zeiger auf die Instanz bzw. dem Instanz-Handle
//	uOffset			: Offset im Speicher der umgerechnet werden soll.
//	Ergibt den Speicheroffset (ab RVA) oder 0xFFFFFFFF bei einem Fehler.
static DWORD ResourceFileToMem(const void *pInstance,DWORD uOffset)
{
IMAGE_OPTIONAL_HEADER  *pOption;
IMAGE_SECTION_HEADER   *pSections;
IMAGE_FILE_HEADER	   *pImage;
IMAGE_DOS_HEADER	   *pDos;
char			       *pMem;
CMapFile		   		sMap;
DWORD				    uCount;
DWORD				    uFrom;
DWORD				    uMax;
DWORD				    uOff;
DWORD				    uSec;
DWORD				    uNum;
DWORD				    uPos;




		pMem = (char*)pInstance;
	if(!pMem)return 0xFFFFFFFF;

	if(IsBadReadPtr(pMem,sizeof(IMAGE_DOS_HEADER)))
		{
		return 0xFFFFFFFF;
		}

	if(*(DWORD*)pMem==DWORD_SIGN('P','E',0,0))		// Kein DOS-Header
		{
		uOff = 0;
		}
	else if(*(WORD*)pMem==WORD_SIGN('M','Z'))		// DOS-Header
		{
		pDos = (IMAGE_DOS_HEADER*)pMem;
		uOff = pDos->e_lfanew;
		}
	else{											// Unbekannte Datei
		return 0xFFFFFFFF;
		}

	if(IsBadReadPtr(pMem,uOff+4+sizeof(IMAGE_SECTION_HEADER)*16))
		{
		return 0xFFFFFFFF;
		}


	uOff	   += 4;								// Zeiger holen
	pImage		= (IMAGE_FILE_HEADER*)(pMem+uOff);
	uOff	   += sizeof(IMAGE_FILE_HEADER);
	pOption		= (IMAGE_OPTIONAL_HEADER*)(pMem+uOff);
	uOff	   += pImage->SizeOfOptionalHeader;
	pSections	= (IMAGE_SECTION_HEADER*)(pMem+uOff);
	uMax		= 0x2000000;
	uCount		= pImage->NumberOfSections;

	if(uCount>16)uCount=16;


	for(uSec=0;uSec<uCount;uSec++)					// Abschnitt suchen
		{
		uNum  = (pSections[uSec].SizeOfRawData+0x1FF)&~0x1FF;
		uFrom =  pSections[uSec].PointerToRawData;
		uPos  =  pSections[uSec].VirtualAddress;

		if(uOffset>=uFrom && uOffset<uFrom+uNum)	// Ist der Offest im Abschnitt
			{
			return uOffset+uPos-uFrom;
			}

		if(uFrom<uMax)uMax=uFrom;
		}

	if(uOffset>=uMax)return 0xFFFFFFFF;


return uOffset;
}

//*****************************************************************************
//*
//*		ResourceMemToFile
//*
//*****************************************************************************
//	Rechnet den Offset im Speicher in einen Offset in einer Datei um
//	pInstance		: Zeiger auf die Instanz bzw. dem Instanz-Handle
//	uOffset			: Offset im Speicher der umgerechnet werden soll.
//					  Der Speicher beginnt bei der RVA-Adresse.
//	Ergibt den Dateioffset oder 0xFFFFFFFF bei einem Fehler.
static DWORD ResourceMemToFile(const void *pInstance,DWORD uOffset)
{
IMAGE_OPTIONAL_HEADER  *pOption;
IMAGE_SECTION_HEADER   *pSections;
IMAGE_FILE_HEADER	   *pImage;
IMAGE_DOS_HEADER	   *pDos;
char			       *pMem;
CMapFile		   		sMap;
DWORD				    uCount;
DWORD				    uFrom;
DWORD				    uMax;
DWORD				    uOff;
DWORD				    uSec;
DWORD				    uNum;
DWORD				    uPos;




		pMem = (char*)pInstance;
	if(!pMem)return 0xFFFFFFFF;

	if(IsBadReadPtr(pMem,sizeof(IMAGE_DOS_HEADER)))
		{
		return 0xFFFFFFFF;
		}

	if(*(DWORD*)pMem==DWORD_SIGN('P','E',0,0))		// Kein DOS-Header
		{
		uOff = 0;
		}
	else if(*(WORD*)pMem==WORD_SIGN('M','Z'))		// DOS-Header
		{
		pDos = (IMAGE_DOS_HEADER*)pMem;
		uOff = pDos->e_lfanew;
		}
	else{											// Unbekannte Datei
		return 0xFFFFFFFF;
		}

	if(IsBadReadPtr(pMem,uOff+4+sizeof(IMAGE_SECTION_HEADER)*16))
		{
		return 0xFFFFFFFF;
		}


	uOff	   += 4;								// Zeiger holen
	pImage		= (IMAGE_FILE_HEADER*)(pMem+uOff);
	uOff	   += sizeof(IMAGE_FILE_HEADER);
	pOption		= (IMAGE_OPTIONAL_HEADER*)(pMem+uOff);
	uOff	   += pImage->SizeOfOptionalHeader;
	pSections	= (IMAGE_SECTION_HEADER*)(pMem+uOff);
	uMax		= 0x2000000;
	uCount		= pImage->NumberOfSections;

	if(uCount>16)uCount=16;


	for(uSec=0;uSec<uCount;uSec++)					// Abschnitt suchen
		{
		uNum  = (pSections[uSec].Misc.VirtualSize+0x1FF)&~0x1FF;
		uFrom =  pSections[uSec].PointerToRawData;
		uPos  =  pSections[uSec].VirtualAddress;

		if(uOffset>=uPos && uOffset<uPos+uNum)		// Ist der Offest im Abschnitt
			{
			return uOffset+uFrom-uPos;
			}

		if(uPos<uMax)uMax=uPos;
		}

	if(uOffset>=uMax)return 0xFFFFFFFF;


return uOffset;
}


//*****************************************************************************
//*
//*		ResourceSort
//*
//*****************************************************************************
//	Sortierfunktion fr die Resourcen-Liste
static int ResourceSort(const void *pPtr1,const void *pPtr2)
{
ResourceItem   *pItem1=(ResourceItem*)pPtr1;
ResourceItem   *pItem2=(ResourceItem*)pPtr2;
unsigned		uLevel;
unsigned		uMin;
int				iDiff;


	uMin	= (pItem1->uLevels<pItem2->uLevels)? pItem1->uLevels:pItem2->uLevels;

	for(uLevel=0;uLevel<uMin;uLevel++)
		{
		iDiff	= pItem1->aIsId[uLevel];
		iDiff  -= pItem2->aIsId[uLevel];

		if(iDiff)return iDiff;

		if(pItem1->aIsId[uLevel])
			{
			iDiff	= pItem1->aIds[uLevel];
			iDiff  -= pItem2->aIds[uLevel];
			}
		else{
			iDiff	= wcscmp(pItem1->aNames[uLevel]+1,pItem2->aNames[uLevel]+1);
			}

		if(iDiff)return iDiff;
		}

	iDiff	= pItem1->uLevels;
	iDiff  -= pItem2->uLevels;

	if(iDiff)return iDiff;

	iDiff	= pItem1->pDataPtr-pItem2->pDataPtr;


return iDiff;
}

//*****************************************************************************
//*
//*		ResourceFind
//*
//*****************************************************************************
//	Suchfunktion fr die Resourcen-Liste
static int ResourceFind(const void *pPtr1,const void *pPtr2)
{
ResourceItem   *pItem1=(ResourceItem*)pPtr1;
ResourceItem   *pItem2=(ResourceItem*)pPtr2;
unsigned		uLevel;
unsigned		uMin;
int				iDiff;


	uMin	= (pItem1->uLevels<pItem2->uLevels)? pItem1->uLevels:pItem2->uLevels;

	for(uLevel=0;uLevel<uMin;uLevel++)
		{
		   iDiff = wcscmp(pItem1->aNames[uLevel]+1,pItem2->aNames[uLevel]+1);
		if(iDiff)
			{
			if(uLevel!=2 || pItem1->aIds[2]!=(UINT)RES_DEFAULT)
				{
				return iDiff;
				}
			}
		}

	iDiff	= pItem1->uLevels;
	iDiff  -= pItem2->uLevels;


return iDiff;
}

//*****************************************************************************
//*
//*		ResourceLoad
//*
//*****************************************************************************
//	Ld eine PE-Datei aus einer Datei.
//	pFilename		: Ist der Dateiname der DLL
//	pError			: Hier wird eine Fehlernummer gespeichert.
//						 0 = kein Fehler
//						 1 = Die Datei ist zuklein
//						 2 = Die Datei wurde nicht gefunden
//						 3 = Die Daten aus der Datei konnten nicht geladen werden
//						 4 = Unbekanntes Datei-Format
//						 5 = Fehler im Dateiformat
//						 6 = Die EXE-Datei ist zu gro
//						 7 = Die Abschnittstabelle ist unplausibel
//						 8 = Es wurden keine Resourcen gefunden
//						 9 = Die Resourcen sind nicht am Ende der Datei
//						10 = Die Resourcen sind sind fehlerhaft
//						11 = Zu viele Ebenen in den Resourcen
//	Ergibt einen Zeiger auf die Resource-Daten bzw. 0 bei einem Fehler.
ResourceData *ResourceLoad(LPCTSTR pFilename,int *pError)
{
ResourceData		       *pResource;
WCHAR						aBuffer [16];
DWORD				    	aLevel  [MAX_LEVELS];
LPWSTR						aNames	[MAX_LEVELS];
IMAGE_RESOURCE_DIRECTORY   *aResDirs[MAX_LEVELS];
IMAGE_RESOURCE_DIRECTORY   *pResDir;
IMAGE_RESOURCE_DATA_ENTRY  *pResData;
IMAGE_OPTIONAL_HEADER      *pOption;
IMAGE_SECTION_HEADER       *pSections;
IMAGE_NT_HEADERS32	       *pPeHeader;
IMAGE_NT_HEADERS64	       *pPeHdr64;
IMAGE_FILE_HEADER	       *pImage;
IMAGE_DOS_HEADER	       *pDos;
char			           *pMem;
DWORD				       *pEntry;
WCHAR					   *pName;
ResourceItem				sItem;
CMapFile		   			sMap;
DWORD				    	uLevel;
DWORD				    	u64Bit;
DWORD				    	uCount;
DWORD				    	uDelta;
DWORD						uPeOff;
DWORD						uPeEnd;
DWORD				    	uFrom;
DWORD				    	uPage;
DWORD				    	uSize;
DWORD				    	uName;
DWORD				    	uChar;
DWORD				    	uData;
DWORD				    	uAll;
DWORD				    	uSeg;
DWORD				    	uRsc;
DWORD				    	uEnd;
DWORD						uNum;
DWORD				    	uRva;
DWORD				    	uOff;
DWORD				    	uDir;
DWORD				    	uLen;
DWORD				    	uCnt;
DWORD				    	uIdx;
DWORD				    	uPos;
DWORD				    	uMax;
DWORD				    	uOk;
int							iErr;


	if(!pError)pError=&iErr;

	   uSize = LengthFile(pFilename);
	if(uSize<=sizeof(IMAGE_DOS_HEADER) || uSize>=0x80000000)
		{
		*pError = (uSize>=0)? 1:2;
		return 0;
		}

		pMem=new char[uSize+4];						// ffne die Datei
	if(!pMem)return 0;

	if(!LoadMemory(pFilename,pMem,uSize+4,1))
		{
		*pError = 3;
		delete pMem;
		return 0;
		}


	if(*(DWORD*)pMem==DWORD_SIGN('P','E',0,0))		// Kein DOS-Header
		{
		uOff = 0;
		}
	else if(*(WORD*)pMem==WORD_SIGN('M','Z'))		// DOS-Header
		{
		pDos = (IMAGE_DOS_HEADER*)pMem;
		uOff = pDos->e_lfanew;
		}
	else{											// Unbekannte Datei
		*pError = 4;
		delete pMem;
		return 0;
		}

	if(uOff+4+sizeof(IMAGE_SECTION_HEADER)*16>uSize)
		{
		*pError = 1;
		delete pMem;
		return 0;
		}


	uPeOff		= uOff;
	pPeHeader	= (IMAGE_NT_HEADERS32*)(pMem+uOff);
	uOff	   += 4;								// Zeiger holen
	pImage		= (IMAGE_FILE_HEADER*)(pMem+uOff);
	uOff	   += sizeof(IMAGE_FILE_HEADER);
	pOption		= (IMAGE_OPTIONAL_HEADER*)(pMem+uOff);
	uOff	   += pImage->SizeOfOptionalHeader;
	pSections	= (IMAGE_SECTION_HEADER*)(pMem+uOff);


	if(uOff+4+sizeof(IMAGE_FILE_HEADER)>uSize)
		{
		*pError = 1;
		delete pMem;
		return 0;
		}


	   uLen = pOption->SizeOfImage;
	if(uLen==0)
		{
		*pError = 5;
		delete pMem;
		return 0;
		}

	if(uLen>0x3000000)
		{
		*pError = 6;
		delete pMem;
		return 0;
		}

	   uCount = pImage->NumberOfSections;
	if(uCount>16)uCount=16;

	uOk		= 1;
	uMax	= 0;
	uPage	= 0;
	uPeEnd	= uOff+sizeof(IMAGE_SECTION_HEADER)*uCount;


	for(uIdx=0;uIdx<uCount;uIdx++)					// Daten kopieren
		{
			uCnt =pSections[uIdx].SizeOfRawData;
			uPos =pSections[uIdx].VirtualAddress;
			uFrom=pSections[uIdx].PointerToRawData;
		if(	uFrom+uCnt > uSize){uOk=0;break;}
		if(	uFrom+uCnt < uFrom){uOk=0;break;}
		if(	uFrom+uCnt > uMax ){uMax=uFrom+uCnt;}
		if(	uPos +uCnt > uLen ){uOk=0;break;}
		if(	uPos +uCnt < uPos ){uOk=0;break;}
		}

	if(!uOk)										// Gab es Fehler beim Laden
		{
		*pError = 7;
		delete pMem;
		return 0;
		}


	if(pImage->Machine==0x8664)						// 64-Bit
		{
		u64Bit   = 1;
		pPeHdr64 = (IMAGE_NT_HEADERS64*)pPeHeader;

			uRva=pPeHdr64->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].VirtualAddress;
		if(!uRva)
			{
			*pError = 8;
			delete pMem;
			return 0;
			}

		uDir = (char*)(pPeHdr64->OptionalHeader.DataDirectory+IMAGE_DIRECTORY_ENTRY_RESOURCE)-pMem;
		uNum = pPeHdr64->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].Size;
		uSeg = pPeHdr64->OptionalHeader.FileAlignment;
		uAll = pPeHdr64->OptionalHeader.SizeOfImage;
		}
	else{											// 32-Bit
		u64Bit   = 0;

			uRva=pPeHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].VirtualAddress;
		if(!uRva)
			{
			*pError = 8;
			delete pMem;
			return 0;
			}

		uDir = (char*)(pPeHeader->OptionalHeader.DataDirectory+IMAGE_DIRECTORY_ENTRY_RESOURCE)-pMem;
		uNum = pPeHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].Size;
		uSeg = pPeHeader->OptionalHeader.FileAlignment;
		uAll = pPeHeader->OptionalHeader.SizeOfImage;
		}

	if(!uSeg)uSeg=0x1000;
	uOff	= ResourceMemToFile(pMem,uRva);
	uRsc	= 0;
	uOk		= 1;

	for(uIdx=0;uIdx<uCount;uIdx++)					// Prfe ob Resourcen ganz hinten sind
		{
		uCnt =pSections[uIdx].SizeOfRawData;
		uPos =pSections[uIdx].VirtualAddress;
		uFrom=pSections[uIdx].PointerToRawData;

		if(uAll==uPos+uCnt)
			{
			uAll = uFrom+uCnt;
			if(uOk)uOk=2;
			}

		if(!strcmp((char*)pSections[uIdx].Name,".rsrc"))
			{
			uRsc = (char*)pSections[uIdx].Name-pMem;

			if(uFrom==uOff)
			if(uFrom+uCnt>=uOff+uNum)
				{
				uEnd = (uOff+uNum+uSeg-1)&~(uSeg-1);

				if(uFrom+uCnt>uOff+uNum && uFrom+uCnt==uEnd)
					{
					uNum = uEnd-uOff;
					continue;
					}

				continue;
				}

			uOk = 0;
			break;
			}

		if(uFrom+uCnt<=uOff)						// Segment vor den Resourcen
			{
			continue;
			}

		if(uFrom>=uOff+uNum)						// Segment nach den Resourcen
			{
			uOk=0;
			break;
			}
		}
													// Sind die Resourcen nicht am Ende oder zu klein
	if(uOk==0 || uNum<sizeof(IMAGE_RESOURCE_DIRECTORY))
		{
		*pError = 9;
		delete pMem;
		return 0;
		}

	if(uOk!=2)										// Lnge krzen
		{
		uAll = uSize&(uSeg-1);
		}

	uDelta      = uRva-uOff;

													// Prfe alle Eintrge
	pResDir		= (IMAGE_RESOURCE_DIRECTORY*)PTR_ADD_32(pMem,uOff);
	uCount		= pResDir->NumberOfNamedEntries+pResDir->NumberOfIdEntries;
	uPos		= uOff+sizeof(IMAGE_RESOURCE_DIRECTORY);
	uPos	   += uCount*8;
	uEnd		= uOff+uNum;
	uLevel		= 0;

	if(uPos>uEnd)
		{
		*pError = 10;
		delete pMem;
		return 0;
		}


	for(uPos=0;;uPos++)
		{											// Ebene nach unten
		if(uPos>=uCount)
			{
			if(uLevel==0)break;
			uLevel--;
			pResDir = aResDirs[uLevel];
			uPos    = aLevel  [uLevel];
			uCount	= pResDir->NumberOfNamedEntries+pResDir->NumberOfIdEntries;
			continue;
			}

		pEntry  = (DWORD*)PTR_ADD_32(pResDir,sizeof(IMAGE_RESOURCE_DIRECTORY));
		pEntry += uPos*2;
		uName   = pEntry[0];

		if(uName&0x80000000)						// Namenseintrag
			{
			uName  = pEntry[0]&0x7FFFFFFF;
			uName += uOff;

			if(uName+1>=uEnd || (uName&1))
				{
				*pError = 10;
				delete pMem;
				return 0;
				}

			uChar = *(WORD*)(pMem+uName);

			if(uName+uChar*2+2>uAll)
				{
				*pError = 10;
				delete pMem;
				return 0;
				}

			#if PRINT_ENTRIES
			printf("%.*s%.*S\t",uLevel*4,"                                    ",uChar,pMem+uName+2);
			#endif
			}
		else{										// ID-Eintrag
			#if PRINT_ENTRIES
			printf("%.*s%i\t"  ,uLevel*4,"                                    ",pEntry[0]);
			#endif

			if(!uPage && uLevel==2)
				{
				uPage = uName;
				}
			}

		uData = pEntry[1];

		if(uData&0x80000000)						// Neues Unterverzeichniss
			{
			if(uLevel>=MAX_LEVELS-1)
				{
				*pError = 11;
				delete pMem;
				return 0;
				}

			aResDirs[uLevel] = pResDir;				// Auf die nchste Ebene
			aLevel  [uLevel] = uPos;
			uLevel			+= 1;

			uData			&= 0x7FFFFFFF;
			uData			+= uOff;

			if(uData+sizeof(IMAGE_RESOURCE_DIRECTORY)>uEnd)
				{
				*pError = 10;
				delete pMem;
				return 0;
				}

			pResDir	= (IMAGE_RESOURCE_DIRECTORY*)PTR_ADD_32(pMem,uData);
			uCount	= pResDir->NumberOfNamedEntries+pResDir->NumberOfIdEntries;
			uPos	= 0-1;

			#if PRINT_ENTRIES
			printf("\n");
			#endif

			continue;
			}

		uData  += uOff;								// Resource gefunden

		if(uData+sizeof(IMAGE_RESOURCE_DATA_ENTRY)>uEnd)
			{
			*pError = 10;
			delete pMem;
			return 0;
			}

		pResData = (IMAGE_RESOURCE_DATA_ENTRY*)PTR_ADD_32(pMem,uData);

		#if PRINT_ENTRIES
		printf("p:%08Xh s:%08Xh c:%i\n",pResData->OffsetToData,pResData->Size,pResData->CodePage);
		#endif

		uData	 = pResData->OffsetToData-uDelta;

		if(uData<uOff || uData+pResData->Size>uEnd || pResData->Reserved==0xFFFFFFFF)
			{
			*pError = 10;
			delete pMem;
			return 0;
			}
		}

	if(!uPage)uPage=1031;


//******************** Resourcen laden ****************************************


	pResource				= new ResourceData;
	pResource->pFileData	= (unsigned char*)pMem;
	pResource->uFileAlign	= uSeg;
	pResource->uPeLength	= uSize;
	pResource->uPeOffset	= uPeOff;
	pResource->uPeHeader	= uPeEnd;
	pResource->uIs64Bit		= u64Bit;
	pResource->uOffset		= uOff;
	pResource->uDelta		= uDelta;
	pResource->uSize		= uNum;
	pResource->uDirPos		= uDir;
	pResource->uRscPos		= uRsc;
	pResource->uCodePage	= uPage;
	pResource->sResList.Len(sizeof(ResourceItem));
	pResource->sMemList.Len(1);

	*pError = 0;

	memset(&sItem,0,sizeof(sItem));

	pResDir		= (IMAGE_RESOURCE_DIRECTORY*)PTR_ADD_32(pMem,uOff);
	uCount		= pResDir->NumberOfNamedEntries+pResDir->NumberOfIdEntries;
	uPos		= uOff+sizeof(IMAGE_RESOURCE_DIRECTORY);
	uPos	   += uCount*8;
	uLevel		= 0;


	#if PRINT_ENTRIES
	printf("\n");
	#endif


	for(uPos=0;;uPos++)
		{											// Resourcen suchen
		if(uPos>=uCount)
			{
			if(uLevel==0)break;

			sItem.aIsId [uLevel] = 0;
			sItem.aIds  [uLevel] = 0;
			sItem.aNames[uLevel] = 0;

			uLevel--;
			pResDir = aResDirs[uLevel];
			pName	= aNames  [uLevel];
			uPos    = aLevel  [uLevel];
			uCount	= pResDir->NumberOfNamedEntries+pResDir->NumberOfIdEntries;
			continue;
			}


		pEntry  = (DWORD*)PTR_ADD_32(pResDir,sizeof(IMAGE_RESOURCE_DIRECTORY));
		pEntry += uPos*2;
		uName   = pEntry[0];

		if(uName&0x80000000)						// Namenseintrag
			{
			uName  = pEntry[0]&0x7FFFFFFF;
			uName += uOff;
			pName  = (WCHAR*)(pMem+uName);
			uChar  = *pName;

			if(uChar>0 && strwlen(pName+1,0x01,uChar)-uChar==0)
				{
				uName = strtoi(pName+1);
				}
			else{
				uName = 0xFFFFFFFF;
				}

			sItem.aIsId[uLevel] = FALSE;
			}
		else{										// ID-Eintrag
			uChar	   = itostr(uName,aBuffer+1);
			aBuffer[0] = (WCHAR)uChar;
			pName	   = aBuffer;

			sItem.aIsId[uLevel] = TRUE;
			}


		uIdx				 = pResource->sMemList.Add(pName,0x7FFFFFFF,uChar*2+4);
		pName				 = (WCHAR*)pResource->sMemList[uIdx];
		pName[uChar+1]		 = 0;
		sItem.aNames[uLevel] = pName;
		sItem.aIds 	[uLevel] = uName;

		uData = pEntry[1];


		if(uData&0x80000000)						// Neues Unterverzeichniss
			{
			aResDirs[uLevel] = pResDir;				// Auf nchste Ebene
			aLevel  [uLevel] = uPos;
			uLevel          += 1;

			uData			&= 0x7FFFFFFF;
			uData			+= uOff;
			pResDir			 = (IMAGE_RESOURCE_DIRECTORY*)PTR_ADD_32(pMem,uData);

			pName			 = aNames[uLevel];
			uCount			 = pResDir->NumberOfNamedEntries+pResDir->NumberOfIdEntries;
			uPos			 = 0-1;

			continue;
			}

		uData   += uOff;							// Resource gefunden
		pResData = (IMAGE_RESOURCE_DATA_ENTRY*)PTR_ADD_32(pMem,uData);
		uData	 = pResData->OffsetToData-uDelta;

		sItem.uLevels	= uLevel+1;
		sItem.pDataPtr	= pResource->pFileData+uData;
		sItem.bFreePtr	= FALSE;
		sItem.sData		=*pResData;

		uIdx = pResource->sResList.FindFast(&sItem,0,0x7FFFFFFF,0x08,ResourceSort);

		pResource->sResList.Add(&sItem,uIdx);

		#if PRINT_ENTRIES
		for(uIdx=0;uIdx<sItem.uLevels;uIdx++)
			{
			printf("%s%S",(uIdx)? ".":"",sItem.aNames[uIdx]+1);
			}
		printf("\n");
		#endif
		}


	#if PRINT_ENTRIES
	printf("\n");
	#endif


return pResource;
}

//*****************************************************************************
//*
//*		ResourceFree
//*
//*****************************************************************************
//	Die Resource-Eintrge wieder freigen
//	pResource	: Zeiger auf die Resource-Daten
void ResourceFree(ResourceData *pResource)
{
ResourceItem   *pEntry;
int				iIdx;



	if(!pResource)return;

	for(iIdx=0;iIdx<pResource->sResList.Count();iIdx++)
		{
			pEntry = (ResourceItem*)pResource->sResList[iIdx];
		if(!pEntry->bFreePtr)continue;

		delete pEntry->pDataPtr;
		}

	delete pResource->pFileData;
	delete pResource;

}

//*****************************************************************************
//*
//*		ResourceFind
//*
//*****************************************************************************
//	Sucht eine Resource
//	pResource	: Zeiger auf die Resource-Daten
//	pNames		: Ist der Level-Name der Resource
//	uLevels		: sind die Ebenen im Resource-Namen
//	Ergibt den Index der Resource inder Liste, oder -1 wenn sie nicht gefunden wurde.
int ResourceFind(ResourceData *pResource,LPCTSTR *pNames,unsigned uLevels)
{
TCHAR				aBuffer[16];
ResourceItem		sItem;
WCHAR			   *pName;
LPCTSTR				pText;
unsigned			uOff;
unsigned			uNum;
unsigned			uPos;




	if(pResource==0)return 0;
	if(uLevels  ==0)return 0;
	if(uLevels>MAX_LEVELS)return 0;


	for(uPos=0;uPos<uLevels;uPos++)					// Namensliste erzeugen
		{
		pText = pNames[uPos];

		if((DWORD_PTR)pText<=0xFFFF)
			{
			if(pText==RES_NONE)
				{
				if(uPos==0)return 0;
				uLevels = uPos;
				break;
				}

			sItem.aIds[uPos] = (DWORD)pText;
			pText			 = aBuffer;
			_itot(sItem.aIds[uPos],aBuffer,10);
			}
		else if(pText[0]>='0' && pText[0]<='9')
			{
			sItem.aIds[uPos] = _ttoi(pText);
			}
		else{
			sItem.aIds [uPos] = 0xFFFFFFFF;
			}

		uNum	= _tcslen(pText);
		pName	= (LPWSTR)alloca(uNum*2+4);
		pName[0]= uNum;

		for(uOff=0;uOff<=uNum;uOff++)
			{
			#ifdef UNICODE
			pName[uOff+1] = pText[uOff];
			#else
			pName[uOff+1] = (UCHAR)pText[uOff];
			#endif
			}

		sItem.aNames[uPos] = pName;
		}

	sItem.uLevels = uLevels;



return pResource->sResList.Find(&sItem,0,0x7FFFFFFF,ResourceFind);
}


//*****************************************************************************
//*
//*		ResourceDelete
//*
//*****************************************************************************
//	Lscht eine Resource
//	pResource	: Zeiger auf die Resource-Daten
//	pNames		: Ist der Level-Name der Resource
//	uLevels		: sind die Ebenen im Resource-Namen
//	Ergibt 1 wenn die Resource gelscht wurde, und 0 wenn sie nicht gefunden wurde.
int ResourceDelete(ResourceData *pResource,LPCTSTR *pNames,unsigned uLevels)
{
ResourceItem   *pEntry;
int				iIndex;


	   iIndex = ResourceFind(pResource,pNames,uLevels);
	if(iIndex<0)return 0;

	   pEntry = (ResourceItem*)pResource->sResList[iIndex];
	if(pEntry->bFreePtr)delete pEntry->pDataPtr;

	pResource->sResList.Del(iIndex);


return 1;
}


//*****************************************************************************
//*
//*		ResourceEntry
//*
//*****************************************************************************
//	Holt den Zeiger auf eine Resource
//	pResource	: Zeiger auf die Resource-Daten
//	pNames		: Ist der Level-Name der Resource
//	uLevels		: sind die Ebenen im Resource-Namen
//	Ergibt 1 wenn die Resource gelscht wurde, und 0 wenn sie nicht gefunden wurde.
LPBYTE ResourceEntry(ResourceData *pResource,LPCTSTR *pNames,unsigned uLevels,int *pSize)
{
ResourceItem   *pEntry;
int				iIndex;


	   iIndex = ResourceFind(pResource,pNames,uLevels);
	if(iIndex<0)
		{
		if(pSize)*pSize=0;
		return 0;
		}

	pEntry = (ResourceItem*)pResource->sResList[iIndex];
	if(pSize)*pSize=pEntry->sData.Size;


return pEntry->pDataPtr;
}


//*****************************************************************************
//*
//*		ResourceInsert
//*
//*****************************************************************************
//	Fgt eine neue Resource ein
//	pResource	: Zeiger auf die Resource-Daten
//	pNames		: Ist der Level-Name der Resource
//	uLevels		: Sind die Ebenen im Resource-Namen
//	pData		: Zeiger auf die Resource
//	uSize		: Gre der Resource in Byte. Wenn RES_FILE angegeben ist
//				  dann wird die Resource aus einer Datei geladen.
//				  "pData" ist der Pfad auf diese Datei.
//	Ergibt 1 wenn die Resource eingefgt wurde, bzw. 0 bei einem Fehler.
int ResourceInsert(ResourceData *pResource,LPCTSTR *pNames,unsigned uLevels,LPCVOID pData,UINT uSize)
{
TCHAR				aBuffer[16];
unsigned char	   *pNewData;
ResourceItem       *pEntry;
ResourceItem		sItem;
WCHAR			   *pName;
LPCTSTR				pText;
unsigned			uOff;
unsigned			uNum;
unsigned			uSet;
unsigned			uPos;
int					iIdx;
int					iLen;


	if(uSize    ==0)return 0;
	if(pData    ==0)return 0;
	if(pResource==0)return 0;
	if(uLevels  ==0)return 0;
	if(uLevels>MAX_LEVELS)return 0;


	memset(&sItem,0,sizeof(sItem));


	for(uPos=0;uPos<uLevels;uPos++)					// Namensliste erzeugen
		{
		pText = pNames[uPos];

		if((DWORD_PTR)pText<=0xFFFF)
			{
			if(pText==RES_NONE)
				{
				if(uPos==0)return 0;
				uLevels = uPos;
				break;
				}

			if(pText==RES_DEFAULT)
				{
				if(uPos!=2)return 0;
				pText = (LPCTSTR)pResource->uCodePage;
				}

			sItem.aIsId[uPos] = TRUE;
			sItem.aIds [uPos] = (DWORD)pText;
			pText			  = aBuffer;
			_itot(sItem.aIds[uPos],aBuffer,10);
			}
		else if(pText[0]>='0' && pText[0]<='9')
			{
			sItem.aIsId[uPos] = TRUE;
			sItem.aIds [uPos] = _ttoi(pText);
			}
		else{
			sItem.aIds [uPos] = 0xFFFFFFFF;
			sItem.aIsId[uPos] = FALSE;
			}

		uNum	= _tcslen(pText);
		pName	= (LPWSTR)alloca(uNum*2+4);
		pName[0]= uNum;

		for(uOff=0;uOff<=uNum;uOff++)
			{
			#ifdef UNICODE
			pName[uOff+1] = pText[uOff];
			#else
			pName[uOff+1] = (UCHAR)pText[uOff];
			#endif
			}

		uSet	= pResource->sMemList.Add(pName,0x7FFFFFFF,uNum*2+4);
		pName	= (WCHAR*)pResource->sMemList[uSet];

		sItem.aNames[uPos] = pName;
		}

	if(uSize==RES_FILE)								// Resource aus Datei laden
		{
		   iLen = LengthFile((LPCTSTR)pData);
		if(iLen<=0)return 0;

			pNewData = new unsigned char[iLen+4];
		if(!pNewData)								// Resource umkopiern
			{
			return 0;
			}

			uSize = LoadMemory((LPCTSTR)pData,pNewData,iLen+4,1);
		if(!uSize)
			{
			delete pNewData;
			return 0;
			}

		if(sItem.aIds[0]==(UINT)RT_BITMAP)			// Header bei Bitmaps entfernen
		if(uSize>14 && pNewData[0]=='B' && pNewData[1]=='M')
			{
			uSize-=14;
			memmove(pNewData,pNewData+14,uSize);
			}
		}
	else{
			pNewData = new unsigned char[uSize];
		if(!pNewData)								// Resource umkopiern
			{
			return 0;
			}

		memcpy(pNewData,pData,uSize);
		}

	sItem.uLevels		= uLevels;

	   iIdx = pResource->sResList.Find(&sItem,0,0x7FFFFFFF,ResourceFind);
	if(iIdx>=0)										// Bestehende Resource lschen
		{
		   pEntry = (ResourceItem*)pResource->sResList[iIdx];
		if(pEntry->bFreePtr)delete pEntry->pDataPtr;
		pResource->sResList.Del(iIdx);
		}



	sItem.bFreePtr		= TRUE;
	sItem.pDataPtr		= pNewData;
	sItem.sData.Size	= uSize;
	sItem.sData.CodePage= (uLevels>=3)? sItem.aIds[2]:pResource->uCodePage;


	iIdx = pResource->sResList.FindFast(&sItem,0,0x7FFFFFFF,0x08,ResourceSort);

	pResource->sResList.Add(&sItem,iIdx);


return 1;
}


//*****************************************************************************
//*
//*		ResourceSave
//*
//*****************************************************************************
//	Speichert die Daten als neue EXE-Datei.
//	pResource	: Zeiger auf die Resource-Daten
//	pFilename	: Ist der Name der EXE-Datei
//	Ergibt 1 wenn die EXE-Datei gespeichert wurde, bzw. 0 bei einem Fehler.
int ResourceSave(ResourceData *pResource,LPCTSTR pFilename)
{
ResourceItem   *pEntry;
ResourceItem   *pLast;
ResourceItem   *pNext;
unsigned char  *pMem;
DataDir		   *pDir;
DataEntry	   *pData;
WCHAR		   *pName;
DWORD		   *pList;
DWORD		   *aDirs [MAX_LEVELS];
LPWSTR			aNames[MAX_LEVELS];
unsigned 		aSizes[MAX_LEVELS];
unsigned 		aUsed [MAX_LEVELS];
unsigned 		aIds  [MAX_LEVELS];
BOOL			aIsId [MAX_LEVELS];
unsigned		uCntN;
unsigned		uCntI;
unsigned		uLevel;
unsigned		uSize;
unsigned		uCmp;
unsigned		uNow;
unsigned		uLen;
unsigned		uOff;
unsigned		uPos;
int				iIdx;
int				iNum;



	if(!pResource)return 0;

	uSize = pResource->uFileAlign+256;

	for(iIdx=0;iIdx<pResource->sResList.Count();iIdx++)
		{
		pEntry = (ResourceItem*)pResource->sResList[iIdx];
		uSize += pEntry->sData.Size+4+8+sizeof(DataEntry)+sizeof(DataDir)*MAX_LEVELS;

		for(uPos=0;uPos<pEntry->uLevels;uPos++)
			{
			if(pEntry->aIsId[uPos])continue;
			pName =	pEntry->aNames[uPos];
			uLen  = pName[0]*2+2;
			if(uLen&2)uLen+=2;
			uSize+= uLen;
			}
		}


		pMem = new unsigned char[uSize];
	if(!pMem)return 0;

	memset(pMem,0,uSize);
	memset(aSizes,0,sizeof(aSizes));

	uLevel	= 0;
	uOff	= 0;

	for(iIdx=0;iIdx<pResource->sResList.Count();iIdx++)
		{
		pEntry = (ResourceItem*)pResource->sResList[iIdx];

		for(uPos=0;uPos<pEntry->uLevels;uPos++)		// Wieviele Namensebenen sind gleich
			{
			if(uPos>=uLevel)break;

			if(aIsId[uPos]!=pEntry->aIsId[uPos])break;

			if(aIsId[uPos])
				{
				if(aIds[uPos]!=pEntry->aIds[uPos])break;
				continue;
				}

			if(wcscmp(aNames[uPos]+1,pEntry->aNames[uPos]+1))
				{
				break;
				}
			}


		if(uPos>=uLevel && aSizes[uPos]==0)			// Neues Directory anlegen
			{
			uCntN = 0;
			uCntI = 0;
			pLast = 0;
													// Zhlt die Eintrge auf gleicher Ebene
			for(iNum=iIdx;iNum<pResource->sResList.Count();iNum++)
				{
				pNext = (ResourceItem*)pResource->sResList[iNum];

				for(uNow=uPos;uNow>0;uNow=uCmp)
					{
					uCmp = uNow-1;

					if(pNext->aIsId[uCmp]!=pEntry->aIsId[uCmp])break;

					if(pNext->aIsId[uCmp])
						{
						if(pNext->aIds[uCmp]!=pEntry->aIds[uCmp])break;
						continue;
						}

					if(wcscmp(pNext->aNames[uCmp]+1,pEntry->aNames[uCmp]+1))
						{
						break;
						}
					}

				if(uNow>0)break;

				if(pLast && pNext->aIsId[uPos]==pLast->aIsId[uPos])
					{
					if(pNext->aIsId[uPos])
						{
						if(pNext->aIds[uPos]==pLast->aIds[uPos])continue;
						}
					else if(!wcscmp(pNext->aNames[uPos]+1,pLast->aNames[uPos]+1))
						{
						continue;
						}
					}

				if(pNext->aIsId[uPos])
						uCntI++;
				else	uCntN++;

				pLast = pNext;
				}

			pDir						= (DataDir*)(pMem+uOff);
			pDir->NumberOfIdEntries		= uCntI;
			pDir->NumberOfNamedEntries	= uCntN;
			uOff					   += sizeof(DataDir);
			aSizes[uPos]				= uCntI+uCntN;
			aDirs [uPos]				= (LPDWORD)(pMem+uOff);
			aUsed [uPos]				= 0;
			uOff					   += sizeof(DWORD)*2*(uCntI+uCntN);

			uLevel++;
			}
		else{
			while(uLevel>uPos+1)					// Ebenen absteigen
				{
				uLevel--;
				aSizes[uLevel] = 0;
				}
			}

		aNames[uPos] = pEntry->aNames[uPos];
		aIsId [uPos] = pEntry->aIsId [uPos];
		aIds  [uPos] = pEntry->aIds  [uPos];
		pList        = aDirs         [uPos];
		pList       += aUsed         [uPos]*2;
		aUsed [uPos]+= 1;


		if(pEntry->aIsId[uPos])						// ID eintragen
			{
			pList[0] = pEntry->aIds[uPos];
			}
		else{										// Namen einfgen
			pList[0] = uOff|0x80000000;
			pName	 = pEntry->aNames[uPos];
			uLen	 = pName[0]*2+2;
			if(uLen&2)uLen+=2;
			memcpy(pMem+uOff,pName,uLen);
			uOff    += uLen;
			}


		if(uPos+1<pEntry->uLevels)					// Directory-Eintrag einfgen
			{
			pList[1] = uOff|0x80000000;
			iIdx--;
			continue;
			}

		pList[1]					= uOff;
		pEntry->sData.OffsetToData	= uOff;
		pData						= (DataEntry*)(pMem+uOff);
		uOff					   += sizeof(DataEntry);
		pData->OffsetToData			= pResource->uOffset+uOff+pResource->uDelta;
		pData->CodePage				= pEntry->sData.CodePage;
		pData->Size					= pEntry->sData.Size;

		memcpy(pMem+uOff,pEntry->pDataPtr,pEntry->sData.Size);
		uOff += (pEntry->sData.Size+3)&~3;

		#if PRINT_ENTRIES
		printf("%i.%i.%i\t%S.%S.%S\n",aUsed[0]-1,aUsed[1]-1,aUsed[2]-1,pEntry->aNames[0]+1,pEntry->aNames[1]+1,pEntry->aNames[2]+1);
		#endif
		}


//******************** EXE-Datei speichern ************************************
IMAGE_SECTION_HEADER   *pSection;
IMAGE_NT_HEADERS32     *pPeHeader;
IMAGE_NT_HEADERS64     *pPeHdr64;
FILE				   *pFile;
unsigned char		   *pPeData;
unsigned				uMask;
unsigned				uEnd;
int						iDiff;


		pPeData	= new unsigned char[pResource->uPeHeader];
	if(!pPeData)
		{
		delete pMem;
		return 0;
		}

	memcpy(pPeData,pResource->pFileData,pResource->uPeHeader);

	uMask	= pResource->uFileAlign-1;
	uOff   += pResource->uFileAlign-((pResource->uOffset+uOff)&uMask);
	iDiff	= uOff-pResource->uSize;


	if(pResource->uIs64Bit)
		{
		pPeHdr64  = (IMAGE_NT_HEADERS64*)PTR_ADD_32(pPeData,pResource->uPeOffset);
		pPeHdr64->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_SECURITY].VirtualAddress = 0;
		pPeHdr64->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_SECURITY].Size			  = 0;
		pPeHdr64->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].Size          += iDiff;
		pPeHdr64->OptionalHeader.CheckSum		= 0;
		pPeHdr64->OptionalHeader.SizeOfImage   += iDiff;
		}
	else{
		pPeHeader = (IMAGE_NT_HEADERS32*)PTR_ADD_32(pPeData,pResource->uPeOffset);
		pPeHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_SECURITY].VirtualAddress = 0;
		pPeHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_SECURITY].Size		   = 0;
		pPeHeader->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].Size          += iDiff;
		pPeHeader->OptionalHeader.CheckSum		= 0;
		pPeHeader->OptionalHeader.SizeOfImage  += iDiff;
		}

	if(pResource->uRscPos)
		{
		pSection = (IMAGE_SECTION_HEADER*)PTR_ADD_32(pPeData,pResource->uRscPos);
		pSection->SizeOfRawData	    += iDiff;
		pSection->Misc.VirtualSize 	+= iDiff;
		}


		pFile = _tfopen(pFilename,_T("wb"));
	if(!pFile)
		{
		delete pPeData;
		delete pMem;
		return 0;
		}

	uEnd = pResource->uOffset+pResource->uSize;
	fwrite(pPeData,pResource->uPeHeader,1,pFile);
	fwrite(pResource->pFileData+pResource->uPeHeader,pResource->uOffset-pResource->uPeHeader,1,pFile);
	fwrite(pMem,uOff,1,pFile);
	fwrite(pResource->pFileData+uEnd,pResource->uPeLength-uEnd,1,pFile);
	fclose(pFile);



	delete pPeData;
	delete pMem;


return 1;
}
