//*****************************************************************************
//*
//*
//*		SrtmFunctions.cpp
//*
//*
//*****************************************************************************
#include	<math.h>
#include	<stdio.h>
#include	<string.h>
#include	"SrtmFunctions.h"
#include	"FileFunctions.h"
#include	"HttpClient.h"
#include	"ZipFile.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifndef 	HTG_MAX
#define 	HTG_MAX			4						// Maximale Zahl der Cache-Puffer
#endif

#ifndef 	HTG_SIDE
#define 	HTG_SIDE		1201					// Auflsung einer Kachel
#endif

#define 	HGT_SERVER		"dds.cr.usgs.gov:443"	// Download-Server

static	char				cHgtFile[32];			// Ist die letzte Datei
static	char				cHgtPath[256];			// Ist der Cache-Pfad
static	char				cHgtLast[256];			// Ist die letzte Datei
static 	unsigned			aHgtVal [HTG_MAX];		// ID-der Cache-Puffer
static 	unsigned			aHgtNum [HTG_MAX];		// Welche Puffer wurde zuletzt geladen
static 	unsigned			uHgtNext;				// Nchster Cache-Puffer
static 	unsigned			uHgtLastGet;			// Tick wann wurde zuletzt eine Datei geholt
static 	unsigned			uHgtLastNum;			// Zhler fr Datei holen
static 	unsigned short 	   *pHgtPoints;				// Zeiger auf den Cache-Puffer
static 	const char		   *aHgtLocations[] =		// Download-Pfade
			{
			"/srtm/version2_1/SRTM3/Eurasia",
			"/srtm/version2_1/SRTM3/North_America",
			"/srtm/version2_1/SRTM3/South_America",
			"/srtm/version2_1/SRTM3/Africa",
			"/srtm/version2_1/SRTM3/Australia",
			"/srtm/version2_1/SRTM3/Islands",
			NULL
			};



//*****************************************************************************
//*
//*		SrtmInit
//*
//*****************************************************************************
//	Initialisiert die SRTM-Funktionen
//	pCachePath		: Ist das Cache-Verzeichnis fr die SRTM-Dateien.
//					  Bei NULL wird der Standartpfad benutzt
//	Ergibt 1 wenn ok oder 0 bei einem Fehler
int SrtmInit(const char *pCachePath)
{
unsigned	uLen;



	if(!pCachePath || !*pCachePath)
		{
		if(ExistFile("D:\\Temp\\Srtm")==2)
			{
			strcpy(cHgtPath,"D:\\Temp\\Srtm");
			}
		else if(ExistFile("C:\\Temp\\Srtm")==2)
			{
			strcpy(cHgtPath,"C:\\Temp\\Srtm");
			}
		/*
		else if(ExistFile("D:\\Temp\\SrtmToy\\Strm3")==2)
			{
			strcpy(cHgtPath,"D:\\Temp\\SrtmToy\\Strm3");
			}
		*/
		else{
			uLen=GetExePath(cHgtPath,sizeof(cHgtPath)-32);
			uLen=DelBackSlash(cHgtPath,uLen);
			strcpy(cHgtPath+uLen,"\\Srtm");
			CreatePath(cHgtPath);
			}
		}
	else{
		strlcpy(cHgtPath,pCachePath,sizeof(cHgtPath)-32);
		}

	if(ExistFile(cHgtPath)!=2)
		{
		cHgtPath[0] = 0;
		return 0;
		}


return 1;
}

//*****************************************************************************
//*
//*		SrtmExit
//*
//*****************************************************************************
//	Deinitialisiert die SRTM-Funktionen
//	Ergibt 1 wenn ok oder 0 bei einem Fehler
int SrtmExit()
{

	if(!cHgtPath[0])return 0;

	memset(aHgtNum,0,sizeof(aHgtNum));
	memset(aHgtVal,0,sizeof(aHgtVal));

	cHgtPath[0] = 0;

	if(pHgtPoints)
		{
		delete pHgtPoints;
		pHgtPoints = 0;
		}


return 1;
}

//*****************************************************************************
//*
//*		SrtmGetHeight
//*
//*****************************************************************************
//	Holt die Hhe von einer globalen Koordinate
//	dCoorX		: Lngegrad
//	dCoorY		: Breitengrad
//	Ergibt die Hhe an diesem Punkt in Metern oder eine Fehlernummer kleiner -10000:
//					SRTM_ERR_NO_MEMORY		-10000	Zuwenig speicher
//					SRTM_ERR_WRONG_FORMAT	-10001	Die Datei hat die falsche Gre
//					SRTM_ERR_CANT_OPEN		-10002	Die Datei konnte nicht geffnet werden
//					SRTM_ERR_CANT_FIND		-10003  Eine SRTM-Datei konnte nicht gefunden werden
//					SRTM_ERR_READ			-10004	Fehler beim Lesen der Daten
//					SRTM_ERR_NOT_INIT		-10005	Die SRTM-Funktionen wurden nicht initilaisiert
double SrtmGetHeight(double dCoorX,double dCoorY)
{
static double	dLast=100;
FILE		   *pFile;
unsigned char  *pData;
unsigned short *pHeights;
unsigned char	ucByte;
char			cPath[256];
char			cTemp[256];
char			cCharX;
char			cCharY;
unsigned		uValY;
unsigned		uValX;
unsigned		uVal;
unsigned		uPos;
unsigned		uMin;
unsigned		uNum;
unsigned		uOff;
double			dVal;
double			dRestX;
double			dRestY;
double			dPosZ1;
double			dPosZ2;
double			dPosZ3;
double			dPosZ;
int				iXpos;
int				iYpos;



	if(!cHgtPath[0])								// Wurden die Funktionen ini
		{
		return SRTM_ERR_NOT_INIT;
		}

	if(dCoorX>=0.0)
		{
		uValX  =  (int)dCoorX;
		cCharX = 'E';
		}
	else{
		dCoorX = -dCoorX;
		uValX  =  (int)dCoorX+1;
		cCharX = 'W';
		}

	if(dCoorY>=0.0)
		{
		uValY  =  (int)dCoorY;
		cCharY = 'N';
		}
	else{
		dCoorY = -dCoorY;
		uValY  =  (int)dCoorY+1;
		cCharY = 'S';
		}

	uVal	= (uValX    )+(cCharX<< 8);
	uVal   |= (uValY<<16)+(cCharX<<24);
	uNum	= 0xFFFFFFFF;
	uMin	= 0;

	for(uPos=0;uPos<HTG_MAX;uPos++)					// Suche Kachel im Cache
		{
		if(uVal==aHgtVal[uPos])break;

		if(aHgtNum[uPos]<uNum)
			{
			uNum = aHgtNum[uPos];
			uMin = uPos;
			}
		}

	if(uPos>=HTG_MAX)								// Nachladen der Datei
		{
		uHgtNext	   += 1;
		uPos			= uMin;
		aHgtNum[uPos]	= uHgtNext;
		aHgtVal[uPos]   = 0;

		if(!pHgtPoints)
			{
				pHgtPoints = new WORD[HTG_SIDE*HTG_SIDE*HTG_MAX];
			if(!pHgtPoints)
				{
				return SRTM_ERR_NO_MEMORY;
				}
			}


		sprintf(cHgtFile,"%c%02u%c%03u.hgt",cCharY,uValY,cCharX,uValX);
		sprintf(cPath,"%s/%s",cHgtPath,cHgtFile);


		if(ExistFile(cPath)==1)
			{
			if(LengthFile(cPath)!=HTG_SIDE*HTG_SIDE*2)
				{
				return SRTM_ERR_WRONG_FORMAT;
				}

				pFile = fopen(cPath,"rb");
			if(!pFile)
				{
				return SRTM_ERR_CANT_OPEN;
				}

			fread (pHgtPoints+HTG_SIDE*HTG_SIDE*uPos,HTG_SIDE*HTG_SIDE*2,1,pFile);
			fclose(pFile);
			}
		else{										// Aus Zip-Datei laden
			strcat(cPath,".zip");

			if(ExistFile(cPath)!=1)
				{
				CHttpClient	sHttp;
				unsigned	uDiff;

				uDiff = GetTickCount()-uHgtLastGet;
													// Wude die Datei kurz vorher geholt
				if(cHgtLast[0] && uDiff<10000 && !stricmp(cPath,cHgtLast))
				if(uHgtLastNum>1 || (uHgtLastNum==1 && uDiff<5000))
					{
					return SRTM_ERR_CANT_FIND;
					}

				if(!stricmp(cHgtLast,cPath) && uHgtLastNum<=1)
					{
					uHgtLastNum++;
					}
				else{
					strlcpy(cHgtLast,cPath,sizeof(cHgtLast));
					uHgtLastGet = GetTickCount();
					uHgtLastNum = 1;
					}

				for(uOff=0;;uOff++)					// Datei vom HTTP-Server holen
					{
					if(uOff==0 && sHttp.Connect(HGT_SERVER))
						{
						return SRTM_ERR_CANT_FIND;
						}

					if(!aHgtLocations[uOff])
						{
						return SRTM_ERR_CANT_FIND;
						}

					sprintf(cTemp,"%s/%c%02u%c%03u.hgt.zip",aHgtLocations[uOff],cCharY,uValY,cCharX,uValX);

					if(sHttp.Get(cPath,cTemp,20000,1)>0)
						{
						break;
						}
					}
				}

			CZipFile	sZip;

			if(sZip.Open(cPath)!=CAF_ERROR_OK)
				{
				return SRTM_ERR_CANT_OPEN;
				}

			if(sZip.Len(0)!=HTG_SIDE*HTG_SIDE*2)
				{
				return SRTM_ERR_WRONG_FORMAT;
				}

			if(sZip.ExtractMem(0,pHgtPoints+HTG_SIDE*HTG_SIDE*uPos,HTG_SIDE*HTG_SIDE*2)!=CAF_ERROR_OK)
				{
				return SRTM_ERR_READ;
				}
			}

		aHgtVal[uPos] = uVal;
		pData         = (unsigned char*)(pHgtPoints+HTG_SIDE*HTG_SIDE*uPos);

		for(uOff=0;uOff<HTG_SIDE*HTG_SIDE;uOff++)	// Big-Endian in Little-Endian
			{
			ucByte			= pData[uOff*2+0];
			pData[uOff*2+0]	= pData[uOff*2+1];
			pData[uOff*2+1] = ucByte;
			}
		}


	pHeights = pHgtPoints+HTG_SIDE*HTG_SIDE*uPos;

	dCoorX  = modf(dCoorX,&dVal);
	dCoorY  = modf(dCoorY,&dVal);
	dCoorX *= HTG_SIDE-1;
	dCoorY *= HTG_SIDE-1;

	if(cCharY=='N')dCoorY=(double)(HTG_SIDE-1)-dCoorY;
	if(cCharX=='W')dCoorX=(double)(HTG_SIDE-1)-dCoorX;

	iXpos   = (int)dCoorX;
	iYpos   = (int)dCoorY;

	dRestX	= dCoorX-(double)iXpos;
	dRestY	= dCoorY-(double)iYpos;


	if(dRestX>dRestY)								// In welchem Dreieck liegt der Punkt
		{
		dPosZ1 = pHeights[iXpos+iYpos*HTG_SIDE           ];
		dPosZ2 = pHeights[iXpos+iYpos*HTG_SIDE+1         ];
		dPosZ3 = pHeights[iXpos+iYpos*HTG_SIDE+1+HTG_SIDE];

		if(dPosZ1>30000.0)
				dPosZ1 = dLast;
		else	dLast  = dPosZ1;

		if(dPosZ2>30000.0)
				dPosZ2 = dLast;
		else	dLast  = dPosZ2;

		if(dPosZ3>30000.0)
				dPosZ3 = dLast;
		else	dLast  = dPosZ3;

		dPosZ  = dRestX*(dPosZ2-dPosZ1) + dPosZ1;
		dPosZ += dRestY*(dPosZ3-dPosZ2);
		}
	else{
		dPosZ1 = pHeights[iXpos+iYpos*HTG_SIDE           ];
		dPosZ2 = pHeights[iXpos+iYpos*HTG_SIDE+  HTG_SIDE];
		dPosZ3 = pHeights[iXpos+iYpos*HTG_SIDE+1+HTG_SIDE];


		if(dPosZ1>30000.0)
				dPosZ1 = dLast;
		else	dLast  = dPosZ1;

		if(dPosZ2>30000.0)
				dPosZ2 = dLast;
		else	dLast  = dPosZ2;

		if(dPosZ3>30000.0)
				dPosZ3 = dLast;
		else	dLast  = dPosZ3;

		dPosZ  = dRestX*(dPosZ3-dPosZ2) + dPosZ1;
		dPosZ += dRestY*(dPosZ2-dPosZ1);
		}



return dPosZ;
}

//*****************************************************************************
//*
//*		SrtmPath
//*
//*****************************************************************************
//	Holt den Cache-Pfad fr die HTG-Dateien
const char *SrtmPath()
{
return cHgtPath;
}

//*****************************************************************************
//*
//*		SrtmFile
//*
//*****************************************************************************
//	Holt den letzten Dateinamen
const char *SrtmFile()
{
return cHgtFile;
}

//*****************************************************************************
//*
//*		SrtmIsLoaded
//*
//*****************************************************************************
//	Prft ob eine Koordinate im Cache liegt.
//	dCoorX		: Lngegrad
//	dCoorY		: Breitengrad
//	Ergibt 1 wenn eine Datei fr diese Koordinate eine Datei im Cache ist
//	bzw. 0 wenn nicht, oder eine Fehlernummer die kleiner 0 ist:
//					SRTM_ERR_NOT_INIT	-10005	Die SRTM-Funktionen wurden nicht initilaisiert
int SrtmIsLoaded(double dCoorX,double dCoorY)
{
char		cCharX;
char		cCharY;
unsigned	uValX;
unsigned	uValY;
unsigned	uVal;
unsigned	uNum;
unsigned	uPos;


	if(!cHgtPath[0])								// Wurden die Funktionen ini
		{
		return SRTM_ERR_NOT_INIT;
		}

	if(dCoorX>=0.0)
		{
		uValX  =  (int)dCoorX;
		cCharX = 'E';
		}
	else{
		dCoorX = -dCoorX;
		uValX  =  (int)dCoorX+1;
		cCharX = 'W';
		}

	if(dCoorY>=0.0)
		{
		uValY  =  (int)dCoorY;
		cCharY = 'N';
		}
	else{
		dCoorY = -dCoorY;
		uValY  =  (int)dCoorY+1;
		cCharY = 'S';
		}

	uVal	= (uValX    )+(cCharX<< 8);
	uVal   |= (uValY<<16)+(cCharX<<24);
	uNum	= 0xFFFFFFFF;

	for(uPos=0;uPos<HTG_MAX;uPos++)					// Suche Kachel im Cache
		{
		if(uVal==aHgtVal[uPos])
			{
			return 1;
			}
		}


return 0;
}
