//*****************************************************************************
//*
//*
//*		Klasse zum Starten von Programmen
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"FileFunctions.h"
#include	"StringFunctions.h"
#include	"StartProg.h"
#include	<io.h>


#ifdef		 AZ_LIB_ADDAFX
#include	<Afx.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifdef		UNICODE
#define 	Z(a)			L##a
#define 	zchar			wchar_t
#define 	itoa_		   _itow
#define 	strzlen			wcslen
#define 	strzicmp	   _wcsicmp
#define 	strznicmp	   _wcsnicmp
#define 	vsiprintf		vswprintf
#else
#define 	Z(a)			a
#define 	zchar			char
#define 	itoa_	    	itoa
#define 	strzlen			strlen
#define 	strzicmp		stricmp
#define 	strznicmp		strnicmp

#define 	strzcmp			strcmp
#define 	strzupr			strupr
#define 	vsiprintf		vsprintf
#endif

#ifndef		CSP_NO_COMMNAD
#define 	CSP_NO_COMMNAD	0
#endif


//*****************************************************************************
//*
//*		GetIntHandle
//*
//*****************************************************************************
#define		FOPEN           0x01		// file handle open
#define		_osfhnd(i)	( _pioinfo (i)->osfhnd)
#define		_osfile(i)  ( _pioinfo (i)->osfile )
#define		_pioinfo(i) (__pioinfo[(i) >> 5] + ((i) & 0x1F))

typedef struct
	{
    long	osfhnd;						// underlying OS file HANDLE
    char	osfile;						// attributes of file (e.g., open in text mode?)
    char	pipech;						// one char buffer for handles opened on pipes
#ifdef _MT
	int		lockinitflag;
    CRITICAL_SECTION lock;
#endif
    }ioinfo;


#ifdef  _DLL
	extern  "C" __declspec(dllimport) ioinfo * __pioinfo[];
	#define _nhandle 64
#else
	extern  "C"  ioinfo  *__pioinfo[];
	extern	"C"	 int	   _nhandle;
#endif




static HANDLE GetIntHandle(int iHandle)
{

	if(((unsigned)iHandle>=(unsigned)_nhandle) ||!(_osfile(iHandle) & FOPEN) )
		{
		return INVALID_HANDLE_VALUE;
		}


return (HANDLE)_osfhnd(iHandle);
}

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CStartProg::CStartProg(const zchar *pFilename){CStartProg();SetFilename(pFilename);}
CStartProg::CStartProg()
{

	pCloseInp	    = NULL;							// Funktion fr das Schlieen der IN-Pipe
	pCloseOut	    = NULL;							// Funktion fr das Schlieen der OUT-Pipe
	pCloseErr	    = NULL;							// Funktion fr das Schlieen der ERR-Pipe

	iParamLen		= 0;							// Bytelnge aller Parameter
	iParamCount		= 0;							// Anzahl der Parameter
	cProgFile[0]	= 0;							// Programm-Dateiname
	cProgDir[0]		= 0;							// Start-Verzeichnis
	pProcessId		= 0;							// Hier wird die Prozess-Id gespeichert
	uProgExit		= 0;							// Ergebnistwert des Programmes
	hProgWnd		= 0;							// Fenster-Handle von Programm
	pProgEnv		= 0;							// Zeiger auf EnvironmentStrings
	hEvent			= 0;							// Event um das Warten zu bebennden
	iBreak			= 0;							// Wurde das Warten angehalten
	iRunning		= 0;							// Luft das Programm noch
	uDefWait		= INFINITE;						// Standart Wartezeit

	memset(&sPi,0,sizeof(sPi));						// Process info
	memset(&sSi,0,sizeof(sSi));						// Startup info
	memset(&aCloseValues,0,sizeof(aCloseValues));	// Handle fr Pipes

	sSi.cb		   = sizeof(sSi);
	sSi.hStdInput  = INVALID_HANDLE_VALUE;
	sSi.hStdOutput = INVALID_HANDLE_VALUE;
	sSi.hStdError  = INVALID_HANDLE_VALUE;

}


//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CStartProg::~CStartProg()
{
int i;


	iRunning = 0;
	ClosePipes();

	for(i=0;i<iParamCount;i++)delete pParams[i];
	if(sPi.hProcess)CloseHandle(sPi.hProcess);
	if(sPi.hThread )CloseHandle(sPi.hThread);
	if(hEvent	   )CloseHandle(hEvent);
	if(pProgEnv)delete pProgEnv;
}


//*****************************************************************************
//*
//*		Reset
//*
//*****************************************************************************
//	Lscht alle bergabe-Parameter und Enviorment-Strings
//	bClosePipes		: Sollen alle erstellten Pipe geschlossen werden
void CStartProg::Reset(BOOL bClosePipes)
{
int i;


	if(bClosePipes)
		{
		sSi.dwFlags &= ~STARTF_USESTDHANDLES;
		ClosePipes();
		}

	for(i=0;i<iParamCount;i++)
		{
		delete pParams[i];
		}

	if(sPi.hProcess || sPi.hThread)
		{
		CloseHandle(sPi.hThread);
		CloseHandle(sPi.hProcess);
		memset(&sPi,0,sizeof(sPi));
		}

	iParamCount	= 0;

	if(pProgEnv)
		{
		delete pProgEnv;
		pProgEnv = 0;
		}

	uProgExit	= 0;
	hProgWnd	= 0;
	iRunning	= 0;
	iBreak	    = 0;

}

//*****************************************************************************
//*
//*		SetFilename
//*
//*****************************************************************************
//	Setz das den Dateinamen fr die EXE-Datei
//	pFilename	: Ist der Name der zustardenden EXE oder BAT-Datei
//				  Ein '#' bewirkt das an den Anfang der Pfad der aktuellen
//				  EXE-Datei gesetzt wird.
//				  Ein '*' am Pfadanfang bewirkt das an den Anfang ein Pfad aus
//				  der PATH Eviormentvariabel gesucht wird.
//				  Ein '?' am Pfadanfang bewirkt das der Pfad unverndert bergeben wird
//	Ergibt einen Zeiger auf den Dateinamen
zchar *CStartProg::SetFilename(const zchar *pFilename)
{
zchar	cBuffer[MAX_PATH];
zchar  *pSuffix;
int		iLen;


	if(*pFilename=='#')										// Pfad von aktueller EXE-Datei benutzen
		{
		GetExePath (cBuffer,sizeof(cBuffer)/sizeof(zchar));
		AddFileName(cBuffer,pFilename+1);
		pFilename = cBuffer;
		}
	else if(*pFilename=='*')								// Pfade aus PATH' Variabel benutzen
		{
		pFilename++;
		pSuffix = GetFileType(pFilename);

		if(*pSuffix)
				iLen=FindPathFile(cBuffer,pFilename,sizeof(cBuffer)/sizeof(zchar));
		else	iLen=FindPathFile(cBuffer,pFilename,sizeof(cBuffer)/sizeof(zchar),Z("exe;bat;com"));

		if(iLen)pFilename=cBuffer;
		}
	else if(*pFilename=='?')									// Soll eins zu eins benutzt werden
		{
		strlcpy(cProgFile,pFilename+1,sizeof(cProgFile));
		return cProgFile;
		}


	GetFullPathName(pFilename,sizeof(cProgFile)/sizeof(zchar),cProgFile,0);



return cProgFile;
}


//*****************************************************************************
//*
//*		SetDirectory
//*
//*****************************************************************************
//	Setzt das Verzeichnis in dem die EXE-Datei ausgefhrt werden soll.
//	Wird ein Dateiname bergeben so wird nur der Pfad davon verwendet.
//	pDirPath	: Is der Name des Verzeichnisses
//				  Ein '#' bewirkt das an den Anfang der Pfad der aktuellen
//				  Exe-Datei gesetzt wird
//	Ergibt einen Zeiger auf das Start-Verzeichnis
zchar *CStartProg::SetDirectory(const zchar *pDirPath)
{
zchar	cBuffer[MAX_PATH];


	if(*pDirPath=='#')
		{
		GetExePath (cBuffer,sizeof(cBuffer)/sizeof(zchar));
		AddFileName(cBuffer,pDirPath+1);
		pDirPath  = cBuffer;
		}
	else{
		if(ExistFile(pDirPath)==1)
			{
			GetFilePath(cBuffer,pDirPath,sizeof(cBuffer)/sizeof(zchar));
			pDirPath  = cBuffer;
			}
		}

	GetFullPathName(pDirPath,sizeof(cProgDir)/sizeof(zchar),cProgDir,0);
	DelBackSlash(cProgDir);


return cProgDir;
}


zchar *CStartProg::SetDirectory(const zchar *pDirPath,int iSize)
{
zchar	cBuffer[MAX_PATH];

	
	if(iSize<0)iSize=0;
	if(iSize>=MAX_PATH)iSize=MAX_PATH-1;

	memcpy(cBuffer,pDirPath,iSize*sizeof(zchar));
	cBuffer[iSize] = 0;


return SetDirectory(cBuffer);
}

//*****************************************************************************
//*
//*		SetParam
//*
//*****************************************************************************
//	Hngt einen einzelnen Parameter an. Sind Leerzeichen in pParam werden
//	automatisch Anfhrungszeichen angehngt.
//	pParam		: Ist der Zeiger auf den Parametertext.
//	iLen		: Ist die Lnge des Parameters.
//	Ergibt die Position des angehngten Blockes bzw. -1 bei einem Fehler.
int CStartProg::SetParam(const zchar *pParam,int iLen)
{
zchar	*pPtr;
int		 iEmpty=0;



	if(iParamCount>=CSP_PARAM_MAX)return -1;
	if(iLen<=0)return -1;
	if(strcwlen(pParam,0xA0,iLen)<iLen)iEmpty=1;


	pPtr=new zchar[iLen+2+iEmpty*2];					// Parameter speichern
	memcpy(pPtr+iEmpty+1,pParam,iLen);
	pPtr[iLen+iEmpty*2+1]=0;
	pPtr[0]=' ';

	if(iEmpty)											// Anfhrungszeichen einfgen
		{
		pPtr[     1] = '"';
		pPtr[iLen+2] = '"';
		}

	iParamLen += iLen+iEmpty*2+1;
	pParams[iParamCount] = pPtr;
	iParamCount++;



return iParamCount-1;
}

//*****************************************************************************
//*
//*		SetParamEx
//*
//*****************************************************************************
//	Hngt einen einzelnen Parameter an. Sind Leerzeichen in pParam werden
//	automatisch Anfhrungszeichen angehngt. Prfixe und Sufixe knnen angehngt werden
//	pParam		: Ist der Zeiger auf den Parametertext.
//	iLen		: Ist die Lnge des Parameters.
//	pPrefix		: Ist der Zeiger auf den Prfixtext.
//	iLenP		: Ist die Lnge des Prfixtext.
//	pSuffix		: Ist der Zeiger auf den Suffixtext.
//	iLenS		: Ist die Lnge des Suffixtext.
//	Ergibt die Position des angehngten Blockes bzw. -1 bei einem Fehler.
int	CStartProg::SetParamEx(const zchar *pParam,int iLen,const zchar *pPrefix,int iLenP,const zchar *pSuffix,int iLenS)
{
zchar	*pPtr;
int		 iEmpty=0;
int		 iSize;


	if(iParamCount>=CSP_PARAM_MAX   )return -1;
	if(iLen<=0 || iLenP<0 || iLenS<0)return -1;
	
	if(strcwlen(pParam ,0xA0,iLen )<iLen  ||			// Suche nach Leerzeichen
	   strcwlen(pPrefix,0xA0,iLenP)<iLenP ||
	   strcwlen(pSuffix,0xA0,iLenS)<iLenS)
		{
		iEmpty = 1;
		}


	pPtr = new zchar[iLen+iLenP+iLenS+2+iEmpty*2];		// Parameter speichern
	
										iSize =iEmpty+1;
	memcpy(pPtr+iSize,pPrefix,iLenP);	iSize+=iLenP;
	memcpy(pPtr+iSize,pParam ,iLen );	iSize+=iLen ;
	memcpy(pPtr+iSize,pSuffix,iLenS);	iSize+=iLenS;

	pPtr[iSize+iEmpty]=0;
	pPtr[0]=' ';

	if(iEmpty)											// Anfhrungszeichen einfgen
		{
		pPtr[      1] = '"';
		pPtr[iSize+2] = '"';
		}

	iParamLen += iSize+iEmpty;
	pParams[iParamCount] = pPtr;
	iParamCount++;

			  

return iParamCount-1;
}

//*****************************************************************************
//*
//*		SetParamMulti
//*
//*****************************************************************************
//	Hngt mehrere Parameter an. Leerzeichen und Anfhrungszeichen trennen dabei
//	die einzelnen Parameter. Leerzeichen am Anfang werden bersprungen.
//	pParam		: Ist der Teiger auf den Text der Parameter.
//	iLen		: Ist die Lnge des Parameters, bei -1 wird die Stringlnge benutzt
//	Ergibt die Position des ersten angehngten Blockes bzw. -1 bei einem Fehler
int CStartProg::SetParamMulti(const zchar *pParam,int iLen)
{
int		iPos;
int		iAdd;
int		iNum;



	if(iParamCount>=CSP_PARAM_MAX)return -1;
	if(iLen< 0)iLen=(int)strzlen(pParam);
	if(iLen<=0)return -1;


	for(iNum=0,iPos=0;iPos<iLen;iPos++)
		{
		   iPos+=strwlen(pParam+iPos,0xA0,iLen-iPos);	// Leerzeichen auslassen
		if(iPos>=iLen)break;

		if(pParam[iPos]=='"')							// String einfgen
			{
			if(iPos+2>=iLen)break;
			iPos++;
			iAdd=strcwlen(pParam+iPos,0x0100,iLen-iPos);
			if(iAdd>0 && SetParam(pParam+iPos,iAdd)>0)iNum++;
			iPos+=iAdd;
			continue;
			}

		iAdd=strcwlen(pParam+iPos,0xA0,iLen-iPos);		// Wort einfgen
		if(iAdd>0 && SetParam(pParam+iPos,iAdd)>0)iNum++;
		iPos+=iAdd-1;
		}

	if(iNum<=0)return -1;


return iParamCount-iNum;
}

//*****************************************************************************
//*
//*		SetParamFormat
//*
//*****************************************************************************
//	Hngt meherer Parameter an ber eine Formatangabe ein.
//	Die Parameter mssen mit ' ' oder '\t' getrennt sein
//	Blcke knnen mit '"' zusammen gehngt werden.
//
//	Die maximale Puffergre ist 2048 Bytes
//	Ergibt die Position des ersten angehngten Blockes bzw. -1 bei einem Fehler
int	_cdecl CStartProg::SetParamFormat(const zchar  *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = SetParamFormat(&pFormat,pArgs);			// Formatstring erzeugen
	va_end(pArgs);

return iRet;
}

int _cdecl CStartProg::SetParamFormat(const zchar **pFormat,va_list pArgs)
{
zchar cBuffer[2048],*pPtr;
int   iNum,iPos,iLen;



	if(iParamCount>=CSP_PARAM_MAX)return -1;

	iPos = iParamCount;
	iNum = vsiprintf(cBuffer,*pFormat,pArgs);		// Formatstring erzeugen
	pPtr = cBuffer;

	while(*pPtr)
		{
		pPtr+=strwlen(pPtr,0xA0);					// Leerzeichen berspringen

		if(*pPtr=='"')
			{
			pPtr++;
			iLen=strchar(pPtr,'"');

			if(iLen>0 && SetParam(pPtr,iLen)<0)return -1;

			   pPtr+=iLen;
			if(pPtr[0]=='"')pPtr++;
			}
		else{
			   iLen=strcwlen(pPtr,0xA0);
			if(iLen>0 && SetParam(pPtr,iLen)<0)return -1;
			pPtr+=iLen;
			}
		}

	if(iPos==iParamCount)return -1;


return iPos;
}

//*****************************************************************************
//*
//*		SetStdIn / SetStdOut / SetStdErr
//*
//*****************************************************************************
void	CStartProg::SetStdIn (int iHandle){SetStdIn (GetIntHandle(iHandle));}
void	CStartProg::SetStdOut(int iHandle){SetStdOut(GetIntHandle(iHandle));}
void	CStartProg::SetStdErr(int iHandle){SetStdErr(GetIntHandle(iHandle));}

//*****************************************************************************
//*
//*		PipeCreate
//*
//*****************************************************************************
//	Erzeugt eine Pipe, sie kann mit Namen oder oder ohne Namen sein.
//	pReadPipe	: Hier wird das Handle zum Lesen gespeichert.
//	pWritePipe	: Hier wird das Handle zum Lesen gespeichert.
//	dwSize		: Ist die Gre der Pipe, bzw. folgende Flags sind erlaubt
//					CSP_PIPE_NAMED		= Eine Pipe mit Namen erstellen
//					CSP_PIPE_OVERLAPPED	= Eine Pipe mit Overlapped-Flag
//	iMode		: Stream fr elchen die Pipe erstell werden soll
//					0	= Input
//					1	= Output
//					2	= Error
//	dwTimeout	: Timeout fr Pipes mit Namen
//	Ergibt TRUE wenn erfolgreich, bzw. FALSE bei einem Fehler.
BOOL CStartProg::PipeCreate(HANDLE *pReadPipe,HANDLE *pWritePipe,DWORD dwSize,int iMode,DWORD dwTimeout)
{
SECURITY_ATTRIBUTES sAttr = {sizeof(SECURITY_ATTRIBUTES),NULL,TRUE};
char				cName[64];
static UINT			uOffset;
HANDLE				hPipeA;
HANDLE				hPipeB;
unsigned			uValue;
unsigned			uFlags;
unsigned			uOverl;
unsigned			uTurn;


	if(!(dwSize&CSP_PIPE_NAMED))					// Pipe ohne Namen
		{
		return CreatePipe(pReadPipe,pWritePipe,&sAttr,dwSize&~CSP_PIPE_OVERLAPPED);
		}

	iMode	&=  3;
	uOverl	 =	dwSize&CSP_PIPE_OVERLAPPED;
	uFlags   =  uOverl|((iMode)? PIPE_ACCESS_INBOUND:PIPE_ACCESS_OUTBOUND);
	dwSize  &= ~(CSP_PIPE_OVERLAPPED|CSP_PIPE_NAMED);
	uOffset	 =  GetCurrentThreadId();

	for(uTurn=0;uTurn<8;uTurn++,uOffset--)			// Pipe mit Namen erstellen
		{
		uValue  = GetTickCount()+uOffset;
		sprintf(cName,"\\\\.\\pipe\\SP_%08X_%c",uValue,"IOE"[iMode]);

		   hPipeA = CreateNamedPipeA(cName,uFlags,PIPE_TYPE_BYTE|PIPE_READMODE_BYTE|PIPE_WAIT,1,dwSize,dwSize,dwTimeout,&sAttr);
		if(hPipeA==INVALID_HANDLE_VALUE)continue;


		if(uOverl)									// Mit Overlapped-Funktion
			{
			OVERLAPPED	sOverlapped={0,0,0,0,NULL};

				sOverlapped.hEvent = aCloseValues[iMode*3+2];
			if(!sOverlapped.hEvent)
				{
				sOverlapped.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
				}
			else{
				ResetEvent(sOverlapped.hEvent);
				}

			if(!ConnectNamedPipe(hPipeA,&sOverlapped) && GetLastError()!=ERROR_IO_PENDING)
				{
				if(!aCloseValues[iMode*3+2])CloseHandle(sOverlapped.hEvent);
				CloseHandle(hPipeA);
				return FALSE;
				}

			aCloseValues[iMode*3+2] = sOverlapped.hEvent;
			}

		   hPipeB = CreateFileA(cName,(iMode)? GENERIC_WRITE:GENERIC_READ,0,&sAttr,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(hPipeB==INVALID_HANDLE_VALUE)
			{
			CloseHandle(hPipeA);
			continue;
			}

		if(iMode)									// Out+Err
			{
			*pReadPipe	= hPipeA;
			*pWritePipe	= hPipeB;
			}
		else{										// Inp
			*pReadPipe	= hPipeB;
			*pWritePipe	= hPipeA;
			}

		return TRUE;
		}


return FALSE;
}

//*****************************************************************************
//*
//*		SetStdInPipe
//*
//*****************************************************************************
//	Erzeugt eine Pipe zum Schreiben von Daten zum Programm.
//	hHandle		: Dort wird das Handle gespeichert in das geschrieben werden kann.
//	dwSize		: Gre der Pipe in Byte, bzw. folgende Flags sind erlaubt
//					CSP_PIPE_NAMED		= Eine Pipe mit Namen erstellen
//					CSP_PIPE_OVERLAPPED	= Eine Pipe mit Overlapped-Flag
//	pDelMode	: Wenn dieser Wert 0 ist wird die Pipe automatisch geschlossen
//				  beim Schlieen des Programms bzw. beim Lschen der Klasse.
//				  ansonsten wird hier das Handle der zweiten Pipe-Seite gespeichert.
//	dwTimeout	: Timeout fr Pipes mit Namen
//	Ergibt TRUE wenn die Pipe erzeugt wurde
BOOL CStartProg::SetStdInPipe(HANDLE &hHandle,DWORD dwSize,HANDLE *pDelMode,DWORD dwTimeout)
{
HANDLE		hTemp;


	if(pCloseInp)ClosePipes(1);

	if(!PipeCreate(&hTemp,&hHandle,dwSize,0,dwTimeout))
		{
		hHandle = INVALID_HANDLE_VALUE;
		return FALSE;
		}

	aCloseValues[0] = hTemp;
	aCloseValues[1] = hHandle;

	if(pDelMode)
		{
		*pDelMode = hTemp;
		}
	else{
		pCloseInp = CloseWin;
		}

	SetStdIn(hTemp);


return TRUE;
}

//*****************************************************************************
//*
//*		SetStdOutPipe
//*
//*****************************************************************************
//	Erzeugt eine Pipe zum Lesen der Ausgaben des Programmes.
//	hHandle		: Dort wird das Handle aus dem gelesen werden kann gespeichert
//	dwSize		: Gre der Pipe in Byte, bzw. folgende Flags sind erlaubt
//					CSP_PIPE_NAMED		= Eine Pipe mit Namen erstellen
//					CSP_PIPE_OVERLAPPED	= Eine Pipe mit Overlapped-Flag
//	pDelMode	: Wenn dieser Wert 0 ist wird die Pipe automatisch geschlossen
//				  beim Schlieen des Programms bzw. beim Lschen der Klasse.
//				  ansonsten wird hier das Handle der zweiten Pipe Seite gespeichert.
//	dwTimeout	: Timeout fr Pipes mit Namen
//	Ergibt TRUE wenn die Pipe erzeugt wurde
BOOL CStartProg::SetStdOutPipe(HANDLE &hHandle,DWORD dwSize,HANDLE *pDelMode,DWORD dwTimeout)
{
HANDLE		hTemp;



	if(pCloseOut)ClosePipes(2);

	if(!PipeCreate(&hHandle,&hTemp,dwSize,1,dwTimeout))
		{
		hHandle = INVALID_HANDLE_VALUE;
		return FALSE;
		}

	aCloseValues[3] = hTemp;
	aCloseValues[4] = hHandle;

	if(pDelMode)
		{
		*pDelMode = hTemp;
		}
	else{
		pCloseOut = CloseWin;
		}


	SetStdOut(hTemp);


return TRUE;
}

//*****************************************************************************
//*
//*		SetStdErrPipe
//*
//*****************************************************************************
//	Erzeugt eine Pipe zum Lesen der Fehlerausgaben des Programmes.
//	hHandle		: Dort wird das Handle aus dem gelesen werden kann gespeichert
//	dwSize		: Gre der Pipe in Byte, bzw. folgende Flags sind erlaubt
//					CSP_PIPE_NAMED		= Eine Pipe mit Namen erstellen
//					CSP_PIPE_OVERLAPPED	= Eine Pipe mit Overlapped-Flag
//	pDelMode	: Wenn dieser Wert 0 ist wird die Pipe automatisch geschlossen
//				  beim Schlieen des Programms bzw. beim Lschen der Klasse.
//				  ansonsten wird hier das Handle der zweiten Pipe Seite gespeichert.
//	dwTimeout	: Timeout fr Pipes mit Namen
//	Ergibt TRUE wenn die Pipe erzeugt wurde
BOOL CStartProg::SetStdErrPipe(HANDLE &hHandle,DWORD dwSize,HANDLE *pDelMode,DWORD dwTimeout)
{
HANDLE		hTemp;


	if(pCloseErr)ClosePipes(4);

	if(!PipeCreate(&hHandle,&hTemp,dwSize,2,dwTimeout))
		{
		hHandle=INVALID_HANDLE_VALUE;
		return FALSE;
		}

	aCloseValues[6] = hTemp;
	aCloseValues[7] = hHandle;

	if(pDelMode)
		{
		*pDelMode = hTemp;
		}
	else{
		pCloseErr = CloseWin;
		}

	SetStdErr(hTemp);


return TRUE;
}


//*****************************************************************************
//*
//*		SetStdInPipe
//*
//*****************************************************************************
//	Erzeugt eine Pipe zum Schreinben von Daten zum Programm.
//	iHandle		: Dort wird das Handle gespeichert in das geschrieben werden kann.
//	dwSize		: Gre der Pipe in Byte
//	pDelMode	: Wenn dieser Wert 0 ist wird die Pipe automatisch geschlossen
//				  beim Schlieen des Programms bzw. beim Lschen der Klasse.
//				  ansonsten wird hier das Handle der zweiten Pipe Seite gespeichert.
//	Ergibt TRUE wenn die Pipe erzeugt wurde
BOOL CStartProg::SetStdInPipe(int &iHandle,DWORD dwSize,int *pDelMode)
{
int aTemp[2];


	if(pCloseInp)ClosePipes(1);

	if(_pipe(aTemp,dwSize,0))
		{
		iHandle=-1;
		return FALSE;
		}

	aCloseValues[0] = (HANDLE)aTemp[0];
	aCloseValues[1] = (HANDLE)aTemp[1];

	if(pDelMode)
		{
		*pDelMode = aTemp[0];
		}
	else{
		pCloseInp = CloseInt;
		}


	iHandle	=aTemp[1];
	SetStdIn(aTemp[0]);


return TRUE;
}

//*****************************************************************************
//*
//*		SetStdOutPipe
//*
//*****************************************************************************
//	Erzeugt eine Pipe zum Lesen der Ausgaben des Programmes.
//	iHandle		: Dort wird das Handle aus dem gelesen werden kann gespeichert
//	dwSize		: Gre der Pipe in Byte
//	pDelMode	: Wenn dieser Wert 0 ist wird die Pipe automatisch geschlossen
//				  beim Schlieen des Programms bzw. beim Lschen der Klasse,
//				  ansonsten wird hier das Handle der zweiten Pipe Seite gespeichert.
//	Ergibt TRUE wenn die Pipe erzeugt wurde
BOOL CStartProg::SetStdOutPipe(int &iHandle,DWORD dwSize,int *pDelMode)
{
int aTemp[2];


	if(pCloseOut)ClosePipes(2);

	if(_pipe(aTemp,dwSize,0))
		{
		iHandle=-1;
		return FALSE;
		}

	aCloseValues[3] = (HANDLE)aTemp[0];
	aCloseValues[4] = (HANDLE)aTemp[1];

	if(pDelMode)
		{
		*pDelMode = aTemp[1];
		}
	else{
		pCloseOut = CloseInt;
		}


	iHandle	 =aTemp[0];
	SetStdOut(aTemp[1]);


return TRUE;
}

//*****************************************************************************
//*
//*		SetStdErrPipe
//*
//*****************************************************************************
//	Erzeugt eine Pipe zum Lesen der Fehlerausgaben des Programmes.
//	iHandle		: Dort wird das Handle aus dem gelesen werden kann gespeichert
//	dwSize		: Gre der Pipe in Byte
//	pDelMode	: Wenn dieser Wert 0 ist wird die Pipe automatisch geschlossen
//				  beim Schlieen des Programms bzw. beim Lschen der Klasse.
//				  ansonsten wird hier das Handle der zweiten Pipe Seite gespeichert.
//	Ergibt TRUE wenn die Pipe erzeugt wurde
BOOL CStartProg::SetStdErrPipe(int &iHandle,DWORD dwSize,int *pDelMode)
{
int aTemp[2];


	if(pCloseErr)ClosePipes(4);

	if(_pipe(aTemp,dwSize,0))
		{
		iHandle=-1;
		return FALSE;
		}

	aCloseValues[6] = (HANDLE)aTemp[0];
	aCloseValues[7] = (HANDLE)aTemp[1];

	if(pDelMode)
		{
		*pDelMode = aTemp[1];
		}
	else{
		pCloseErr = CloseInt;
		}


	iHandle	 =aTemp[0];
	SetStdErr(aTemp[1]);


return TRUE;
}

//*****************************************************************************
//*
//*		SetShow
//*
//*****************************************************************************
//	Set den Anzeige-Modus fr das Programm
//	Mode:	CSP_HIDE            oder	SW_HIDE
//			CSP_SHOWNORMAL      oder	SW_SHOWNORMAL
//			CSP_NORMAL          oder	SW_NORMAL
//			CSP_SHOWMINIMIZED   oder	SW_SHOWMINIMIZED
//			CSP_SHOWMAXIMIZED   oder	SW_SHOWMAXIMIZED
//			CSP_MAXIMIZE        oder	SW_MAXIMIZE
//			CSP_SHOWNOACTIVATE  oder	SW_SHOWNOACTIVATE
//			CSP_SHOW            oder	SW_SHOW
//			CSP_MINIMIZE        oder	SW_MINIMIZE
//			CSP_SHOWMINNOACTIVE oder	SW_SHOWMINNOACTIVE
//			CSP_SHOWNA          oder	SW_SHOWNA
//			CSP_RESTORE         oder	SW_RESTORE
//			CSP_SHOWDEFAULT     oder	SW_SHOWDEFAULT
//			CSP_FORCEMINIMIZE   oder	SW_FORCEMINIMIZE
void CStartProg::SetShow(int iMode)
{

	sSi.dwFlags		|= STARTF_USESHOWWINDOW;
	sSi.wShowWindow	 = iMode&0x0F;

}

//*****************************************************************************
//*
//*		SetShowRect
//*
//*****************************************************************************
void CStartProg::SetShowRect(LPRECT pRect)
{
	SetShowPos (pRect->left ,pRect->top);
	SetShowSize(pRect->right-pRect->left,pRect->bottom-pRect->top);
}

//*****************************************************************************
//*
//*		SetEnvironment
//*
//*****************************************************************************
//	Setzt eine Environment-Variabel
//	pName			: Ist der Name der zu setzenten Environment-Variabel
//	pValue/iValue	: Ist der neue Wert fr die Environment-Variabel
void CStartProg::SetEnvironment(const zchar *pName,int          iValue){zchar cTemp[16];SetEnvironment(pName,itoa_(iValue,cTemp,10));}
void CStartProg::SetEnvironment(const zchar *pName,const zchar *pValue)
{
zchar	*pPtr,*pNew;
int		 iPos=0,iNum=0,iLen,iLenN,iLenV;




		pPtr=(pProgEnv)? pProgEnv:(zchar*)GetEnvironmentStrings();
	if(!pPtr)pPtr=Z("");


	iLenN = (int)strzlen(pName);
	iLenV = (int)strzlen(pValue)+1;

	while(pPtr[iPos])								// Lnge berechnen
		{
		iLen=(int)strzlen(pPtr+iPos)+1;

		if(iLen<=iLenN || pPtr[iPos+iLenN]!='=' || strznicmp(pPtr+iPos,pName,iLenN))
			{
			iNum+=iLen;
			}


		iPos+=iLen;
		}


	iNum += iLenN+iLenV+4;
	pNew  = new zchar[iNum];
	iPos  = 0;


	while(pPtr[iPos])								// Strings kopieren berechnen
		{
		iLen=(int)strzlen(pPtr+iPos)+1;

		if(iLen<=iLenN || pPtr[iPos+iLenN]!='=' || strznicmp(pPtr+iPos,pName,iLenN))
			{
			memcpy(pNew+iPos,pPtr+iPos,iLen);
			}

		iPos+=iLen;
		}


													// Neue Variabel setzen
	memcpy(pNew+iPos,pName, iLenN);iPos+=iLenN;pNew[iPos]='=';iPos++;
	memcpy(pNew+iPos,pValue,iLenV);iPos+=iLenV;pNew[iPos]= 0 ;

	if(pProgEnv)delete pProgEnv;
	   pProgEnv=pNew;

}

void CStartProg::SetEnvironment(zchar *pName)
{


	if(pProgEnv)delete pProgEnv;
	   pProgEnv=pName;


}


//*****************************************************************************
//*
//*		IsRunning
//*
//*****************************************************************************
//	Ergibt 1 wenn das Programm noch luft
BOOL CStartProg::IsRunning()
{
int		i;


	if(!sPi.hProcess || !iRunning)return 0;

	i = WaitForSingleObject(sPi.hProcess,0);

	switch(i)
		{
	case WAIT_OBJECT_0:		break;
	case WAIT_TIMEOUT:		return 1;
	case WAIT_ABANDONED:	return 0;
	case WAIT_FAILED:		return 0;
	default:				return 0;
		}

	GetExitCodeProcess  (sPi.hProcess,&uProgExit);
	CloseHandle         (sPi.hProcess);
	CloseHandle         (sPi.hThread);
	memset(&sPi,0,sizeof(sPi));

	hProgWnd = 0;


return 0;
}


//*****************************************************************************
//*
//*		Break
//*
//*****************************************************************************
//	Das Waren anhalten (fr Wait,WaitForInit und GetMainWindow)
void CStartProg::Break()
{

	iBreak = 1;
	if(hEvent)SetEvent(hEvent);

}

//*****************************************************************************
//*
//*		Wait
//*
//*****************************************************************************
//	Warte auf Program Ende
//	dwTime	: Ist die maximale Wartezeit in Millisekunden .
//			  INFINITE fr unendlich.
//  Ergibt 1 wenn Porgramm benendet wurde sonst 0
BOOL CStartProg::Wait(DWORD dwTime)
{
HANDLE	aList[2];


	if(!sPi.hProcess || iBreak || !iRunning)return 0;

	if(!hEvent)
		{
		hEvent = CreateEventW(NULL,TRUE,FALSE,NULL);
		}
	else{
		ResetEvent(hEvent);
		}

	if(iBreak)return 0;

	aList[0] = sPi.hProcess;
	aList[1] = hEvent;

	if(WaitForMultipleObjects(2,aList,FALSE,dwTime)!=WAIT_OBJECT_0)
		{
		return 0;
		}

	if(WaitForSingleObject(sPi.hProcess,dwTime)==WAIT_TIMEOUT)
		{
		return 0;
		}

	GetExitCodeProcess  (sPi.hProcess,&uProgExit);
	memset(&sPi,0,sizeof(sPi));

	if(pProcessId)									// Prozess-ID in Zeiger rcksetzen
		{
		*pProcessId = 0;
		}

	iRunning = 0;
	hProgWnd = 0;


return 1;
}

//*****************************************************************************
//*
//*		WaitForInit
//*
//*****************************************************************************
//	Warte bis Program-Initialisierung abgeschlossen ist
//	dwTime	: Ist die maximale Wartezeit in Millisekunden.
//			  INFINITE fr unendlich.
//  Ergibt 1 wenn Porgramm itialisiert wurde sonst 0
BOOL CStartProg::WaitForInit(DWORD dwTime)
{
DWORD	uRet;


	if(!sPi.hProcess || !iRunning)
		{
		return 0;
		}


	if(dwTime==INFINITE)
		{
		for(;;)
			{
			   uRet = WaitForInputIdle(sPi.hProcess,5000);
			if(uRet==0x00000000)return 0;
			if(uRet==0xFFFFFFFF)return 0;
			if(IsRunning() == 0)return 0;
			if(iBreak		   )return 0;
			}

		}
	else if(WaitForInputIdle(sPi.hProcess,dwTime)!=0x00000000)
		{
		return 0;
		}


return 1;
}


//*****************************************************************************
//*
//*		GetMainWindow
//*
//*****************************************************************************
//	Holl das Fenster-Handle auf das Hauptfensetr des gestarteten Programms.
//	iMaxTime	: Ist die maximale Wartezeit bis zum ffnen des Fensters.
//	Ergibt ein Handle auf das Haupt Fenster des Przesses bzw 0 bei einem Fehler.
HWND CStartProg::GetMainWindow(int iMaxTime)
{
HWND	hWnd;
DWORD	dwId;
UINT	uStart;
int		iDiff;
zchar	cName[64];



	uStart = GetTickCount();

	while(!IsRunning())
		{
		iDiff = GetTickCount()-uStart;
		if(iDiff>iMaxTime)return 0;
		if(iBreak		 )return 0;
		Sleep(100);
		}

	if(!hProgWnd)
		{
		hWnd=0;

		while(iMaxTime>=0)							// Warte bis Fenster aktiv ist
			{
			if(WaitForInit(1000))break;
			if(!IsRunning())return 0;

			iMaxTime-=1000;

			for(;;)
				{
					hWnd=FindWindowEx(0,hWnd,0,0);
				if(!hWnd)break;
				if(!GetWindowThreadProcessId(hWnd,&dwId))continue;
				if(dwId!=sPi.dwProcessId)continue;

				cName[0]=0;
				GetWindowText(hWnd,cName,sizeof(cName)/sizeof(zchar));
				if(cName[0]=='M' && cName[1]==0)continue;
				if(!strzicmp(cName,Z("Default IME")))continue;

				cName[0]=0;
				GetClassName(hWnd,cName,sizeof(cName)/sizeof(zchar));
				if(!strzicmp(cName,Z("tooltips_class64")))continue;
				if(!strzicmp(cName,Z("tooltips_class32")))continue;
				if(!strzicmp(cName,Z("MSCTFIME UI"     )))continue;
				if(!strzicmp(cName,Z("IME"             )))continue;

				iMaxTime = -1;
				break;
				}
			}

		if(!hWnd)
		for(;;)
			{
				hWnd=FindWindowEx(0,hWnd,0,0);
			if(!hWnd)break;
			if(!GetWindowThreadProcessId(hWnd,&dwId))continue;
			if(dwId!=sPi.dwProcessId)continue;

			cName[0]=0;
			GetWindowText(hWnd,cName,sizeof(cName)/sizeof(zchar));
			if(cName[0]=='M' && cName[1]==0)continue;
			if(!strzicmp(cName,Z("Default IME")))continue;

			cName[0]=0;
			GetClassName(hWnd,cName,sizeof(cName)/sizeof(zchar));
			if(!strzicmp(cName,Z("tooltips_class64")))continue;
			if(!strzicmp(cName,Z("tooltips_class32")))continue;
			if(!strzicmp(cName,Z("MSCTFIME UI"     )))continue;
			if(!strzicmp(cName,Z("IME"             )))continue;

			break;
			}

		if(!hWnd)
		for(;;)
			{
				hWnd=FindWindowEx(0,hWnd,0,0);
			if(!hWnd)break;
			if(!GetWindowThreadProcessId(hWnd,&dwId))continue;
			if(dwId!=sPi.dwProcessId)continue;

			cName[0]=0;
			GetClassName(hWnd,cName,sizeof(cName)/sizeof(zchar));
			if(!strzicmp(cName,Z("tooltips_class64")))continue;
			if(!strzicmp(cName,Z("tooltips_class32")))continue;
			if(!strzicmp(cName,Z("MSCTFIME UI"     )))continue;
			if(!strzicmp(cName,Z("IME"             )))continue;


			break;
			}

		if(!hWnd)
		for(;;)
			{
				hWnd=FindWindowEx(0,hWnd,0,0);
			if(!hWnd)break;
			if(!GetWindowThreadProcessId(hWnd,&dwId))continue;
			if(dwId!=sPi.dwProcessId)continue;
			break;
			}

		if(!hWnd)return 0;
		hProgWnd=hWnd;
		}


return hProgWnd;
}

//*****************************************************************************
//*
//*		SetFocus
//*
//*****************************************************************************
//	Gibt den Fokus ans Haupt fensters des Processes
void CStartProg::SetFocus()
{
HWND	hWnd;


	   hWnd=GetMainWindow();
	if(hWnd)
		{
		::SetForegroundWindow(hWnd);
		::SetActiveWindow    (hWnd);
		::SetFocus           (hWnd);
		}

}

//*****************************************************************************
//*
//*		PostMessage
//*
//*****************************************************************************
//	Schickt eine Nachricht an andere Programm ohne auf die Antwort zu warten.
//	uMsg		: Ist die Nummer der Nachricht
//	wParam		: Erster  bergabeparameter
//	lParam		: Zweiter bergabeparameter
//	iMaxTime	: Ist die maximale Wartezeit bis zum ffnen des Fensters.
//	Ergibt 0 bei einem Fehler
BOOL CStartProg::PostMessage(UINT uMsg,WPARAM wParam,LPARAM lParam,int iMaxTime)
{

	if(!IsRunning())return 0;

return ::PostMessage(GetMainWindow(iMaxTime),uMsg,wParam,lParam);
}


//*****************************************************************************
//*
//*		SendMessage
//*
//*****************************************************************************
//	Schickt eine Nachricht an andere Programm.
//	uMsg		: Ist die Nummer der Nachricht
//	wParam		: Erster  bergabeparameter
//	lParam		: Zweiter bergabeparameter
//	iMaxTime	: Ist die maximale Wartezeit bis zum ffnen des Fensters.
//	Ergibt 0 bei einem Fehler bzw. den Rckgabewert von SendMessage.
DWORD CStartProg::SendMessage(UINT uMsg,WPARAM wParam,LPARAM lParam,int iMaxTime)
{

	if(!IsRunning())return 0;

return (DWORD)::SendMessage(GetMainWindow(iMaxTime),uMsg,wParam,lParam);
}

//*****************************************************************************
//*
//*		Start
//*
//*****************************************************************************
//	Startet das Programm
//	pFilename	: Ist das Programm das gestartet werden soll
//				  Ein '#' bewirkt das an den Anfang der Pfad der aktuellen
//				  EXE-Datei gesetzt wird.
//				  Ein '*' am Pfadanfang bewirkt das an den Anfang ein Pfad aus
//				  der PATH Eviormentvariabel gesucht wird.
//				  Ein '?' am Pfadanfang bewirkt das der Pfad eins zu eins
//				  bernommen wird.
//	iMode		: CSP_NOWAIT   = Nicht warten bis Programm beendet ist
//				  CSP_WAIT	   = Warten bis Programm beendet ist
//				  CSP_SETSHOW  = Anzeige Modus setzen
//				  CSP_NOHANLES = Die File-Handle nicht teilen
//				  CSP_SYSTEM   = Datei ber Command.com auftufen
//	pFormat		: Ist die Formatangabe mit der Parameter bergeben werden knnen
//				  bzw. NULL falls keine Parameter bergeben werden sollen.
//	Ergibt 1 wenn das Programm gestartet wurde ansonsten 0.
BOOL _cdecl CStartProg::Start(		 			     int iMode,const zchar  *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = Start(0,iMode,&pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

BOOL _cdecl CStartProg::Start(const zchar *pFilename,int iMode,const zchar  *pFormat,...)
{
va_list		pArgs;
int			iRet;

	va_start(pArgs,pFormat);
	iRet = Start(pFilename,iMode,&pFormat,pArgs);
	va_end(pArgs);

return iRet;
}

BOOL _cdecl CStartProg::Start(const zchar *pFilename,int iMode,const zchar **pFormat,va_list pArgs)
{
int		 i,iLen,iIsBat=0,iAdd=0;
zchar	*pCmd,*pPtr,*pFile,*pTmp,*pDir;
zchar   *pAddPtr[4],cTemp[MAX_PATH];

	
	if(pProcessId)									// Prozess-ID in Zeiger rcksetzen
		{
		*pProcessId = 0;
		}
	
	cTemp[0]  = 0;
	iBreak    = 0;
	hProgWnd  = 0;
	iRunning  = 0;
	uProgExit = 0;

	if(sPi.hProcess){CloseHandle(sPi.hProcess);CloseHandle(sPi.hThread);memset(&sPi,0,sizeof(sPi));}
	if(pFilename   )SetFilename   (pFilename);
	if(pFormat[0]  )SetParamFormat(pFormat,pArgs);
	if(iMode&CSP_SETSHOW)SetShow  (iMode);
	if(cProgFile[0]==0)
		{
		SetLastError(ERROR_FILE_NOT_FOUND);
		return 0;
		}

	pDir  = (cProgDir[0])? cProgDir:0;
	pFile =  cProgFile;

	if(pDir && ExistFile(pDir)<2)pDir=0;			// Prfe ob das Verzeichnis existiert

	#if !CSP_NO_COMMNAD

	if(iMode&CSP_SYSTEM)
		{
		iIsBat = 1;
		}
	else{
		iIsBat = CmpFileType(pFile,Z("BAT;CMD"));	// Ein Batch File ffnen ?
		}

	if(iIsBat)
		{
		if(ExistFile(Z("C:\\COMMAND.COM"))==1)
			{
			pFile=Z("C:\\COMMAND.COM");
			}
		else{
			iLen = GetSystemDirectory(cTemp  ,sizeof(cTemp)/sizeof(zchar));
			strlcpy(cTemp+iLen,Z("\\CMD.EXE"),sizeof(cTemp)/sizeof(zchar)-iLen);

			if(ExistFile(cTemp)!=1)
				{
				strlcpy(cTemp+iLen,Z("\\COMMAND.COM"),sizeof(cTemp)/sizeof(zchar)-iLen);
				}

			pFile=cTemp;
			}

		pAddPtr[3] = Z(" /C ");
		pAddPtr[2] = Z("\""  );
		pAddPtr[1] = cProgFile;
		pAddPtr[0] = Z("\""  );
		iAdd       = (int)strzlen(cProgFile)+6;
		}

	#endif

	iLen = (int)strzlen(pFile);
	pPtr = new zchar[iLen+4+iParamLen+iAdd];
	pCmd = pPtr;


	*pPtr = '"';pPtr++;								// Kommandozeile erzeugen
	memcpy(pPtr,pFile,iLen);pPtr+=iLen;
	*pPtr = '"';pPtr++;
	*pPtr = ' ';pPtr++;

	for(i=(iIsBat)? -4:0;i<iParamCount;i++)
		{
		pTmp=(i>=0)? pParams[i]:pAddPtr[-i-1];
		iLen=(int)strzlen(pTmp);
		memcpy(pPtr,pTmp,iLen);
		pPtr+=iLen;
		}

	*pPtr=0;

	if(sSi.dwFlags&STARTF_USESTDHANDLES)			// In/Output Handles verwenden
		{
		iMode &= ~CSP_NOHANLES;
		}


	i=CreateProcess(pFile,							// Das Programm starten
					pCmd,
					NULL,NULL,
					(iMode&CSP_NOHANLES)? FALSE:TRUE,
					CREATE_DEFAULT_ERROR_MODE,
					(pProgEnv)? pProgEnv:GetEnvironmentStrings(),
					pDir,
					&sSi,							// StartUpInfo
					&sPi);

	  

	delete pCmd;

	if(i==0)										// Gab es Fehler ?
		{
		sPi.hProcess=0;
		return 0;
		}

	if(pProcessId)									// Prozess-ID in Zeiger speichern
		{
		*pProcessId = sPi.dwProcessId;
		}

	iRunning = 1;

	if((iMode&CSP_WAIT) && !Wait(uDefWait))			// Warte auf Programmende
		{
		SetLastError(ERROR_TIMEOUT);
		}
	else{
		SetLastError(ERROR_SUCCESS);
		}


return 1;
};

//*****************************************************************************
//*
//*		Terminate
//*
//*****************************************************************************
//	Beendet ein gestartetes Programmm
//	iExitCode	: Ist der Exit-Code fr das Programm
//	Ergibt 1 wenn das Programm beendet wurde
BOOL CStartProg::Terminate(int iExitCode)
{

	if(!sPi.hProcess)return 0;
	if(!TerminateProcess(sPi.hProcess,iExitCode))return 0;

	GetExitCodeProcess (sPi.hProcess,&uProgExit);
	CloseHandle        (sPi.hProcess);
	CloseHandle        (sPi.hThread);
	memset(&sPi,0,sizeof(sPi));
	hProgWnd=0;


return 1;
}


//*****************************************************************************
//*
//*		CloseInt
//*
//*****************************************************************************
//	Schliet Pipe-Handels vom Typ int
void __cdecl CStartProg::CloseInt(HANDLE &hHandle)
{
	close((int)hHandle);
	hHandle = INVALID_HANDLE_VALUE;
}


//*****************************************************************************
//*
//*		CloseWin
//*
//*****************************************************************************
//	Schliet Pipe-Handels vom Typ HANDLE
void __cdecl CStartProg::CloseWin(HANDLE &hHandle)
{
	CloseHandle(hHandle);
	hHandle = INVALID_HANDLE_VALUE;
}


//*****************************************************************************
//*
//*		ClosePipes
//*
//*****************************************************************************
//	Schliee alle offen Pipe-Handels
//	iFlags		: Welche Handles sollen geschlossen werden
//					Bit0 = Input
//					Bit1 = Output
//					Bit2 = Error
void CStartProg::ClosePipes(int iFlags)
{

	if((iFlags&1) && pCloseInp)
		{
		sSi.hStdInput  = INVALID_HANDLE_VALUE;

		pCloseInp(aCloseValues[0]);
		pCloseInp(aCloseValues[1]);
		pCloseInp=NULL;

		if(aCloseValues[2])
			{
			CloseHandle(aCloseValues[2]);
			aCloseValues[2]=0;
			}
		}

	if((iFlags&2) && pCloseOut)
		{
		sSi.hStdOutput = INVALID_HANDLE_VALUE;

		pCloseOut(aCloseValues[3]);
		pCloseOut(aCloseValues[4]);
		pCloseOut=NULL;

		if(aCloseValues[5])
			{
			CloseHandle(aCloseValues[5]);
			aCloseValues[5]=0;
			}
		}

	if((iFlags&4) && pCloseErr)
		{
		sSi.hStdError  = INVALID_HANDLE_VALUE;

		pCloseErr(aCloseValues[6]);
		pCloseErr(aCloseValues[7]);
		pCloseErr=NULL;

		if(aCloseValues[8])
			{
			CloseHandle(aCloseValues[8]);
			aCloseValues[8]=0;
			}
		}

}

//*****************************************************************************
//*
//*		GetHandle
//*
//*****************************************************************************
//	Holt das Handle zu einer Pipe oder einem Prozess
//	uType		: Ist der Typ des Handles (siehe CSP_TYPE_???)
//					0 = Input
//					1 = Output
//					2 = Error
//					3 = Prozess Thread Wait
//	uIndex		: Ist der Unterindex fr das Handle
//					0 = Pipe zu Prozess
//					1 = Lokale Pipe
//					2 = Event fr Pipe (bei Overlapped)
//					0 = (Type==4) Prozess
//					1 = (Type==4) Thread
//					2 = (Type==4) Wait-Event
//	Ergibt das Handle oder NULL bei einem Fehler.
HANDLE CStartProg::GetHandle(UINT uType,UINT uIndex)
{

	if(uType>3 || uIndex>=3)return NULL;


	if(uType<3)
		{
		return aCloseValues[uType*3+uIndex];
		}


	if(uIndex==0)return sPi.hProcess;
	if(uIndex==1)return sPi.hThread;


return hEvent;
}

