//*****************************************************************************
//*
//*
//*		StartProgEx.cpp
//*
//*
//*****************************************************************************
//
//	Verschidene Funktionen arbeiten mit Consolen-Programmen
//	Die empfangenen Daten werden ber Pipes umgeleitet.
//
#include	"StartProgEx.h"
#include	"WinExtension.h"
#include	"StringFunctions.h"
#include	<tchar.h>

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	LOCK()			EnterCriticalSection(&m_sSynch);
#define 	UNLOCK()		LeaveCriticalSection(&m_sSynch);

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CStartProgEx::CStartProgEx()
{

	m_pLoggerErr	= 0;
	m_pLoggerOut	= 0;
	m_cPortState	= 0;
	m_uByteOut		= 0;
	m_uByteErr		= 0;
	m_uLineOut		= 0;
	m_uLineErr		= 0;
	m_bRunning		= false;
	m_bReset		= false;
	m_bExit			= false;
	m_bWait			= false;
	m_hOutFile		= INVALID_HANDLE_VALUE;
	m_hErrFile		= INVALID_HANDLE_VALUE;
	m_hNextTrig		= CreateEvent(NULL,FALSE,FALSE,NULL);
	m_hReadTrig		= CreateEvent(NULL,FALSE,FALSE,NULL);
	m_hPortTrig		= CreateEvent(NULL,FALSE,FALSE,NULL);
	m_hDoneTrig		= CreateEvent(NULL,FALSE,FALSE,NULL);
	m_hMyThread		= NULL;

	SetStdOutPipe(m_hPipeRead ,CPL_LINESIZE*16|CSP_PIPE_NAMED|CSP_PIPE_OVERLAPPED,0,INFINITE);
	SetStdErrPipe(m_hPipeError,CPL_LINESIZE*16|CSP_PIPE_NAMED|CSP_PIPE_OVERLAPPED,0,INFINITE);
	SetStdInPipe (m_hPipeWrite,CPL_LINESIZE*16|CSP_PIPE_NAMED|CSP_PIPE_OVERLAPPED,0,INFINITE);


	InitializeCriticalSection(&m_sSynch);


}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CStartProgEx::~CStartProgEx()
{

	if(m_bRunning)											// Warte auf das Ende des Prozesses
		{
		TerminateProcess(sPi.hProcess,-1);
		WaitForSingleObject(sPi.hProcess,15000);
		m_bRunning = 0;
		}

	while(m_cPortState && m_cPortState<127)					// Warte bis der Thread beendet wurden
		{
		m_bExit = true;
		SetEvent(m_hPortTrig);
		SetEvent(m_hNextTrig);
		WaitForSingleObject(m_hDoneTrig,50);
		}

	if(m_hOutFile!=INVALID_HANDLE_VALUE)
		{
		CloseHandle(m_hOutFile);
		m_hOutFile = INVALID_HANDLE_VALUE;
		}

	if(m_hErrFile!=INVALID_HANDLE_VALUE)
		{
		CloseHandle(m_hErrFile);
		m_hErrFile = INVALID_HANDLE_VALUE;
		}

	if(m_hPortTrig)
		{
		CloseHandle(m_hPortTrig);
		m_hPortTrig = NULL;
		}

	if(m_hDoneTrig)
		{
		CloseHandle(m_hDoneTrig);
		m_hDoneTrig = NULL;
		}

	if(m_hNextTrig)
		{
		CloseHandle(m_hNextTrig);
		m_hNextTrig = NULL;
		}

	if(m_hReadTrig)
		{
		CloseHandle(m_hReadTrig);
		m_hReadTrig = NULL;
		}

	if(m_hMyThread)
		{
		WaitForSingleObject(m_hMyThread,15000);
		m_hMyThread = NULL;
		}

	DeleteCriticalSection(&m_sSynch);

}


//*****************************************************************************
//*
//*		Start
//*
//*****************************************************************************
//	Startet das Programm
//	pFilename	: Ist das Programm das gestartet werden soll
//				  Ein '#' bewirkt das an den Anfang der Pfad der aktuellen
//				  EXE-Datei gesetzt wird.
//				  Ein '*' am Pfadanfang bewirkt das an den Anfang ein Pfad aus
//				  der PATH Eviormentvariabel gesucht wird.
//				  Ein '?' am Pfadanfang bewirkt das der Pfad eins zu eins
//				  bernommen wird.
//	iMode		: CSP_NOWAIT   = Nicht warten bis Programm beendet ist
//				  CSP_WAIT	   = Warten bis Programm beendet ist
//				  CSP_SETSHOW  = Anzeige Modus setzen
//				  CSP_NOHANLES = Die File-Handle nicht teilen
//				  CSP_SYSTEM   = Datei ber Command.com auftufen
//	pFormat		: Ist die Formatangabe mit der Parameter bergeben werden knnen
//				  bzw. NULL falls keine Parameter bergeben werden sollen.
//	Ergibt 1 wenn das Programm gestartet wurde ansonsten 0.
BOOL _cdecl CStartProgEx::Start(int iMode,const zchar  *pFormat,...)
{
va_list		pArgs;
int			iRet;

	if(!StartIo())return 0;

	va_start(pArgs,pFormat);
	iRet = CStartProg::Start(0,iMode,&pFormat,pArgs);
	va_end(pArgs);


	if(iRet)											// Starte das Auslesen
		{
		m_bWait	   = 0;
		m_bReset   = 0;
		m_bRunning = 1;
		SetEvent(m_hPortTrig);
		}



return iRet;
}

BOOL _cdecl CStartProgEx::Start(const zchar *pFilename,int iMode,const zchar  *pFormat,...)
{
va_list		pArgs;
int			iRet;


	if(!StartIo())return 0;

	va_start(pArgs,pFormat);
	iRet = CStartProg::Start(pFilename,iMode,&pFormat,pArgs);
	va_end(pArgs);


	if(iRet)											// Starte das Auslesen
		{
		m_bWait	   = 0;
		m_bReset   = 0;
		m_bRunning = 1;
		SetEvent(m_hPortTrig);
		}


return iRet;
}


//*****************************************************************************
//*
//*		StartIo
//*
//*****************************************************************************
//	Startet das IO-Port
BOOL CStartProgEx::StartIo()
{
DWORD		dwThreadId;



	if(m_hPipeRead ==INVALID_HANDLE_VALUE)return FALSE;
	if(m_hPipeWrite==INVALID_HANDLE_VALUE)return FALSE;
	if(m_hPipeError==INVALID_HANDLE_VALUE)return FALSE;


	m_bWait		= false;
	m_iReadMode = 0;

	ResetEvent(m_hNextTrig);
	ResetEvent(m_hReadTrig);


	if(!m_cPortState)									// Starte Empfangs-Thread
		{
		if(m_hMyThread)
			{
			WaitForSingleObject(m_hMyThread,2000);
			CloseHandle(m_hMyThread);
			}

		m_cPortState = 1;
		m_hMyThread  = CreateThread(NULL,0x8000,ReadThread,this,0,&dwThreadId);
		}




return TRUE;
}

//*****************************************************************************
//*
//*		ResetPipe
//*
//*****************************************************************************
//	Bricht das Einlesen von der Pipe ab.
BOOL CStartProgEx::ResetPipe()
{

	LOCK();

	if(m_bWait)
		{
		m_bWait = false;
		SetEvent(m_hNextTrig);
		}

	m_bReset = 1;

	UNLOCK();

	SetEvent(m_hPortTrig);






return 0;
}



//*****************************************************************************
//*
//*		ReadThread
//*
//*****************************************************************************
//	Thread der auf Daten vom Programm und Ereignisse wartet
DWORD WINAPI CStartProgEx::ReadThread(LPVOID pParam){return ((CStartProgEx*)pParam)->ReadThread();}
DWORD CStartProgEx::ReadThread()
{
OVERLAPPED		sOverlOut={0,0,0,0,NULL};
OVERLAPPED		sOverlErr={0,0,0,0,NULL};
HANDLE			aWaitList[4];
DWORD			dwCount;
DWORD			dwRest;
LPBYTE			pData;
BOOL			bOut;
BOOL			bErr;
UINT			uNum;
UINT			uMax;
UINT			uRet;
UINT			uErr;



	while(!m_bExit)											// Thread laufen lassen bis Instanz gelscht wird
		{
		m_cPortState	= 1;

		WaitForSingleObject(m_hPortTrig,5000);
		if(m_bExit)break;

		m_uByteOut	= 0;
		m_uByteErr	= 0;
		m_uLineErr	= 0;
		m_uLineOut	= 0;
		m_cLastErr  = 0;
		m_cLastOut	= 0;
		bOut		= 1;
		bErr		= 1;


		while(!m_bExit && m_bRunning && !m_bReset)			// Daten vom Prozess lesen
			{
			m_cPortState = 2;

			if(bOut)
				{
				sOverlOut.hEvent	= aCloseValues[5];		// Event von Pite (Output)
				uMax	= (CPL_LINESIZE-m_uLineOut)*sizeof(TCHAR);
				pData	= (LPBYTE)(m_cLineOut+m_uLineOut);


				if(ReadFile(m_hPipeRead,pData,uMax,&dwCount,&sOverlOut))
					{
					if(dwCount)NewDataRecived(dwCount);
					continue;
					}

				if(GetLastError()!=ERROR_IO_PENDING)		// Gab es einen Fehler beim Lesen
					{
					if(m_bRunning)Terminate(-1);
					m_bExit = TRUE;
					break;
					}

				bOut = 0;
				}

			if(bErr)
				{
				sOverlErr.hEvent	= aCloseValues[8];		// Event von Pipe (Error)
				uMax	= (CPL_LINESIZE-m_uLineErr)*sizeof(TCHAR);
				pData	= (LPBYTE)(m_cLineErr+m_uLineErr);

				if(ReadFile(m_hPipeError,pData,uMax,&dwCount,&sOverlErr))
					{
					if(dwCount)NewInfoRecived(dwCount);
					continue;
					}

				if(GetLastError()!=ERROR_IO_PENDING)		// Gab es einen Fehler beim Lesen
					{
					if(m_bRunning)Terminate(-1);
					m_bExit = TRUE;
					break;
					}

				bErr = 0;
				}

			aWaitList[0]	= aCloseValues[5];
			aWaitList[1]	= aCloseValues[8];
			aWaitList[2]	= m_hPortTrig;
			aWaitList[3]	= sPi.hProcess;
			m_cPortState	= 3;
			uNum			= 4;
			uErr			= 0;

Retry:
			uRet = WaitForMultipleObjects(uNum,aWaitList,FALSE,5000);

			if(uRet==3)										// Der Prozess wurde beendet
				{
				m_bRunning	= 0;
				uNum		= 2;

				if(CancelIo(m_hPipeRead))break;

				goto Retry;
				}

			if(uRet==2)										// Port-Tigger aktiviert
				{
				if((m_bExit || m_bReset) && m_bRunning)
					{
					Terminate(-1);
					}

				goto Retry;
				}

			if(uRet==1)										// Daten aus Error-Pipe einlesen
				{
				if(GetOverlappedResult(m_hPipeError,&sOverlErr,&dwCount,FALSE))
					{
					if(dwCount)NewInfoRecived(dwCount);
					bErr = 1;
					continue;
					}

				if(GetLastError()==ERROR_IO_INCOMPLETE)
					{
					Sleep(20);
					goto Retry;
					}

				uErr++;
				if(uErr>2)
					{
					CancelIo(m_hPipeError);
					CancelIo(m_hPipeRead);
					bOut = 1;
					bErr = 1;
					continue;
					}

				goto Retry;
				}

			if(uRet==0)										// Daten aus Output-Pipe einlesen
				{
				if(GetOverlappedResult(m_hPipeRead,&sOverlOut,&dwCount,FALSE))
					{
					if(dwCount)NewDataRecived(dwCount);
					bOut = 1;
					continue;
					}

				if(GetLastError()==ERROR_IO_INCOMPLETE)
					{
					Sleep(20);
					goto Retry;
					}

				uErr++;
				if(uErr>2)
					{
					CancelIo(m_hPipeError);
					CancelIo(m_hPipeRead);
					bOut = 1;
					bErr = 1;
					continue;
					}

				goto Retry;
				}

			if(m_bExit && uRet==WAIT_TIMEOUT)
				{
				   uErr++;
				if(uErr>2)
					{
					CancelIo(m_hPipeError);
					CancelIo(m_hPipeRead);
					bOut = 1;
					bErr = 1;
					continue;
					}
				}

			goto Retry;
			}



		if(m_bRunning)										// Warte auf das Ende des Prozesses
			{
			Terminate(-1);

			for(;;)
				{
				   uRet = WaitForSingleObject(sPi.hProcess,5000);
				if(uRet==0)break;
				Terminate(-1);
				}

			m_bRunning = 0;
			}

		dwRest = 0;
		PeekNamedPipe(m_hPipeRead,NULL,0,NULL,&dwRest,NULL);

		while(dwRest)										// Restliche Bytes aus der Queue holen
			{
			if(m_bExit)
				{
				m_uLineOut = 0;
				}

			uMax   = (CPL_LINESIZE-m_uLineOut)*sizeof(TCHAR);

			if(uMax>dwRest)uMax=dwRest;

			pData  = (LPBYTE)(m_cLineOut+m_uLineOut);

			if(ReadFile(m_hPipeRead,pData,uMax,&dwCount,&sOverlOut)==0)
			if(GetLastError()!=ERROR_IO_PENDING || !GetOverlappedResult(m_hPipeRead,&sOverlOut,&dwCount,TRUE))
				{
				break;
				}

			dwRest -= dwCount;
			if(dwCount)NewDataRecived(dwCount);
			}

		dwRest = 0;
		PeekNamedPipe(m_hPipeError,NULL,0,NULL,&dwRest,NULL);

		while(dwRest)										// Restliche Bytes aus der Queue holen
			{
			if(m_bExit)
				{
				m_uLineErr = 0;
				}

			uMax   = (CPL_LINESIZE-m_uLineErr)*sizeof(TCHAR);

			if(uMax>dwRest)uMax=dwRest;

			pData  = (LPBYTE)(m_cLineErr+m_uLineErr);

			if(ReadFile(m_hPipeError,pData,uMax,&dwCount,&sOverlErr)==0)
			if(GetLastError()!=ERROR_IO_PENDING || !GetOverlappedResult(m_hPipeError,&sOverlErr,&dwCount,TRUE))
				{
				break;
				}

			dwRest -= dwCount;
			if(dwCount)NewInfoRecived(dwCount);
			}

		NewInfoRecived(0);
		NewDataRecived(0);
		}

	if(m_bRunning)											// Warte auf das Ende des Prozesses
		{
		Terminate(-1);

		for(;;)
			{
			   uRet = WaitForSingleObject(sPi.hProcess,5000);
			if(uRet==0)break;
			Terminate(-1);
			}

		m_bRunning = 0;
		}

	NewInfoRecived(0);
	NewDataRecived(0);

	m_cPortState = 127;
	SetEvent(m_hDoneTrig);


return 0;
}



//*****************************************************************************
//*
//*		NewDataRecived
//*
//*****************************************************************************
//	Wird aufgerunen wenn neue Daten aus der Output-Pipe empfangen wurden
//	uBytes			: Ist die Anzahl der neuen Bytes, bzw. 0 am Ende.
VOID CStartProgEx::NewDataRecived(unsigned uBytes)
{
DWORD		dwDone;
INT			iStart;
INT			iFlag;
TCHAR		cSign;



	m_uByteOut += uBytes;

	if(m_hOutFile!=INVALID_HANDLE_VALUE)					// In Datei speichern
		{
		if(uBytes)
			{
			WriteFile(m_hOutFile,m_cLineOut,uBytes,&dwDone,NULL);
			m_uLineOut = 0;
			return;
			}

		CloseHandle(m_hOutFile);

		if(m_pLoggerErr)
			{
			dwDone = sprintf(m_cLineErr,"<file done: %i byte>\n",m_uByteErr);
			m_pLoggerErr(m_pLogParamErr,m_cLineErr,dwDone);
			}

		m_hOutFile    = INVALID_HANDLE_VALUE;
		m_cLineOut[0] = 0;
		m_uLineOut	  = 0;

		LOCK();

		m_bWait			 = false;
		m_iReadMode      = 0x204;

		ResetEvent(m_hNextTrig);
		SetEvent  (m_hReadTrig);

		UNLOCK();

		return;
		}



	if(uBytes==0)											// Ende erreicht
		{
		m_cLineOut[m_uLineOut] = 0;

		if(m_uLineOut>0)
			{
			if(m_pLoggerOut)m_pLoggerOut(m_pLogParamOut,m_cLineOut,m_uLineOut);

			LOCK();

			m_bWait		= true;
			m_iReadMode = 0x003;

			ResetEvent(m_hNextTrig);
			SetEvent  (m_hReadTrig);

			UNLOCK();

			WaitForSingleObject(m_hNextTrig,INFINITE);

			m_uLineOut       = 0;
			m_cLineOut[0] = 0;
			}

		LOCK();

		m_bWait		= false;
		m_iReadMode = 0x004;
		ResetEvent(m_hNextTrig);
		SetEvent  (m_hReadTrig);

		UNLOCK();

		return;
		}



	iStart = 0;

	for(;uBytes>0;uBytes--)									// Suche Zeilenumbrche
		{
		if(m_bExit)
			{
			break;											// Soll der Thread beendet werden
			}

		cSign = m_cLineOut[m_uLineOut];
		iFlag = strwchar(cSign,0x40);

		if(!iFlag && m_uLineOut<CPL_LINESIZE-1)
			{
			m_uLineOut++;
			continue;
			}

															// Zum Logger ausgeben
		if(m_pLoggerOut)m_pLoggerOut(m_pLogParamOut,m_cLineOut,m_uLineOut);

		LOCK();

		m_bWait		= true;
		m_iReadMode = (iFlag)? 0x001:0x002;
		m_cLineOut[m_uLineOut] = 0;

		ResetEvent(m_hNextTrig);
		SetEvent  (m_hReadTrig);

		UNLOCK();

		WaitForSingleObject(m_hNextTrig,INFINITE);

		m_uLineOut++;
		m_cLastOut = cSign;
															// "\r\n" abfangen
		if(cSign=='\r' && uBytes>1 && m_cLineOut[m_uLineOut]=='\n')
			{
			m_cLastOut = 0;
			m_uLineOut++;
			uBytes--;
			}

		memmove(m_cLineOut,m_cLineOut+m_uLineOut,uBytes-1);
		m_uLineOut = 0;
		}



}


//*****************************************************************************
//*
//*		NewInfoRecived
//*
//*****************************************************************************
//	Wird aufgerunen wenn neue Daten aus der Error-Pipe empfangen wurden
//	uBytes			: Ist die Anzahl der neuen Bytes, bzw. 0 am Ende.
VOID CStartProgEx::NewInfoRecived(unsigned uBytes)
{
DWORD		dwDone;
INT			iStart;
INT			iFlag;
TCHAR		cSign;


	m_uByteErr += uBytes;

	if(m_hErrFile!=INVALID_HANDLE_VALUE)					// In Datei speichern
		{
		if(uBytes)
			{
			WriteFile(m_hErrFile,m_cLineErr,uBytes,&dwDone,NULL);
			m_uLineErr = 0;
			return;
			}

		CloseHandle(m_hErrFile);

		if(m_pLoggerErr)
			{
			dwDone = sprintf(m_cLineErr,"<file done: %i byte>\n",m_uByteErr);
			m_pLoggerErr(m_pLogParamErr,m_cLineErr,dwDone);
			}

		m_hErrFile    = INVALID_HANDLE_VALUE;
		m_cLineErr[0] = 0;
		m_uLineErr	  = 0;

		LOCK();

		m_bWait			 = false;
		m_iReadMode      = 0x304;

		ResetEvent(m_hNextTrig);
		SetEvent  (m_hReadTrig);

		UNLOCK();

		return;
		}

	if(uBytes==0)											// Ende erreicht
		{
		m_cLineErr[m_uLineErr] = 0;

		if(m_uLineErr>0)
			{
			if(m_pLoggerErr)m_pLoggerErr(m_pLogParamErr,m_cLineErr,m_uLineErr);

			LOCK();

			m_bWait		= true;
			m_iReadMode = 0x103;

			ResetEvent(m_hNextTrig);
			SetEvent  (m_hReadTrig);

			UNLOCK();

			WaitForSingleObject(m_hNextTrig,INFINITE);

			m_uLineErr = 0;
			m_cLineErr[0] = 0;
			}

		return;
		}



	iStart = 0;

	for(;uBytes>0;uBytes--)									// Suche Zeilenumbrche
		{
		cSign = m_cLineErr[m_uLineErr];
		iFlag = strwchar(cSign,0x40);

		if(!iFlag && m_uLineErr<CPL_LINESIZE-1)
			{
			m_uLineErr++;
			continue;
			}

		if(m_pLoggerErr)m_pLoggerErr(m_pLogParamErr,m_cLineErr,m_uLineErr);

		LOCK()

		m_bWait		= true;
		m_iReadMode = (iFlag)? 0x101:0x102;
		m_cLineErr[m_uLineErr] = 0;

		ResetEvent(m_hNextTrig);
		SetEvent  (m_hReadTrig);

		UNLOCK();

		WaitForSingleObject(m_hNextTrig,INFINITE);


		m_uLineErr++;
		m_cLastErr = cSign;
															// "\r\n" abfangen
		if(cSign=='\r' && uBytes>1 && m_cLineErr[m_uLineErr]=='\n')
			{
			m_cLastErr = 0;
			m_uLineErr++;
			uBytes--;
			}

		memmove(m_cLineErr,m_cLineErr+m_uLineErr,uBytes-1);
		m_uLineErr = 0;
		}


}


//*****************************************************************************
//*
//*		ReadNext
//*
//*****************************************************************************
//	Holt einen Zeiger auf die nchste Zeile	bzw. die nchste Meldung.
//	pLine		: Hier wird der Zeiger gespeichert
//	pSize		: Hier wird die Textlnge gespeichert. (wenn != NULL)
//	uTimeout	: Ist der Timeout in Millisekunden
//	Eribt einen Statuswert ber die Aktion und im zweiten Byte die Quelle:
//					0x??00 = Timeout
//					0x??01 = Zeile empfangen
//					0x??02 = Zu lange Zeile
//					0x??03 = Letzte Zeile
//					0x??04 = Ende ereicht
//					0x00?? = Zeile aus Output
//					0x01?? = Zeile aus Error
//					0x02?? = Datei aus Output
//					0x03?? = Datei aus Error
INT CStartProgEx::ReadNext(const zchar *&pLine,int *pSize,UINT uTimeout)
{


	LOCK();

	if(m_bWait)
		{
		m_bWait = false;
		SetEvent(m_hNextTrig);
		}

	UNLOCK();


	if(WaitForSingleObject(m_hReadTrig,uTimeout)!=WAIT_OBJECT_0)
		{
		if(pSize)*pSize=0;
		pLine=_T("");
		return 0;
		}

	if(m_iReadMode&0x100)									// Aus Error-Pipe
		{
		pLine = m_cLineErr;

		if(m_uLineErr>0 && m_cLineErr[0]=='\n' && m_cLastErr=='\r')
			{
			if(pSize)*pSize=m_uLineErr-1;
			pLine++;
			}
		else{
			if(pSize)*pSize=m_uLineErr;
			}
		}
	else{													// Aus Output-Pipe
		pLine = m_cLineOut;

		if(m_uLineOut>0 && m_cLineOut[0]=='\n' && m_cLastOut=='\r')
			{
			if(pSize)*pSize=m_uLineOut-1;
			pLine++;
			}
		else{
			if(pSize)*pSize=m_uLineOut;
			}
		}



return m_iReadMode;
}

//*****************************************************************************
//*
//*		SetOutput
//*
//*****************************************************************************
//	Setzt als Ausgabe eine Datei
//	pFilename	: Ist die Dateiname
//	bAppend		: Am Ende der Datei anhngen (TRUE) oder neue erstellen.
//	Ergibt TRUE wenn die Datei geffnet werden konnte
BOOL CStartProgEx::SetOutput(const zchar *pFilename,BOOL bAppend)
{
HANDLE	hFile;


	if(m_hOutFile!=INVALID_HANDLE_VALUE)
		{
		CloseHandle(m_hOutFile);
		m_hOutFile = INVALID_HANDLE_VALUE;
		}

	   hFile = CreateFile(pFilename,GENERIC_WRITE,0,NULL,(bAppend)? OPEN_ALWAYS:CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)return FALSE;

	if(bAppend)SetFilePointer(hFile,0,NULL,FILE_END);


	m_hOutFile = hFile;


return TRUE;
}


//*****************************************************************************
//*
//*		SetError
//*
//*****************************************************************************
//	Setzt als Ausgabe fr die Error-Pipe eine Datei
//	pFilename	: Ist die Dateiname
//	bAppend		: Am Ende der Datei anhngen (TRUE) oder neue erstellen.
//	Ergibt TRUE wenn die Datei geffnet werden konnte
BOOL CStartProgEx::SetError(const zchar *pFilename,BOOL bAppend)
{
HANDLE	hFile;


	if(m_hErrFile!=INVALID_HANDLE_VALUE)
		{
		CloseHandle(m_hErrFile);
		m_hErrFile = INVALID_HANDLE_VALUE;
		}

	   hFile = CreateFile(pFilename,GENERIC_WRITE,0,NULL,(bAppend)? OPEN_ALWAYS:CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)return FALSE;

	if(bAppend)SetFilePointer(hFile,0,NULL,FILE_END);


	m_hErrFile = hFile;


return TRUE;
}

//*****************************************************************************
//*
//*		SetLogger
//*
//*****************************************************************************
//	Stellt den Logger-Callback fr Error und Output ein.
//	pProc		: Ist die Callback-Funktion. (NULL=deaktivieren)
//	pParam		: User-Parameter fr die Callback-Funktion.
//	iMode		: Wo soll der Callback installiert werden (Bit0=Output Bit1=Error)
VOID CStartProgEx::SetLogger(CplLoggerType pProc,void *pParam,int iMode)
{

	if(iMode&1)
		{
		m_pLoggerOut	= pProc;
		m_pLogParamOut	= pParam;
		}

	if(iMode&2)
		{
		m_pLoggerErr	= pProc;
		m_pLogParamErr	= pParam;
		}

}
