//*****************************************************************************
//*
//*
//*		StrXCmp.h
//*
//*
//*****************************************************************************
//
//	Funktionen zum Vergleichen der hnlichkeit von Texten
//	(C) Copyright Anton Zechner 2007
//
//
//	Funktionen:
//
//		void strxinit(char *pGrosKlein=0)		: Initialisierung
//		int  strxcmp (char *pStr1,char *pStr2)	: Vergleichen

#include <string.h>
#include <malloc.h>

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#ifndef _WCHAR_T_DEFINED
	#if _MSC_VER
	#include	<wchar.h>
	#else
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif
#endif


//*****************************************************************************
//*
//*		Globale fr strxcmp
//*
//*****************************************************************************
static char	cStrxCmpCase[256];

#define 	AEIOU ((1<<('A'-'A'))|(1<<('E'-'A'))|(1<<('I'-'A'))|(1<<('O'-'A'))|(1<<('U'-'A')))
#define 	STRXW_MAX	8

//*****************************************************************************
//*
//*		strxinit
//*
//*****************************************************************************
//	Stellt die spezielle Buchstaben fr die Grokleinumwandlung ein
//	pGrosKlein	: Ist ein Zeiger auf die Umwandlungsbuchstaben (z.B. "AOUS")
void _cdecl strxinit(const char *pGrosKlein)
{
int				i;
unsigned char 	ucChar1;
unsigned char	ucChar2;
static	 int    iInitOk=0;



	if(iInitOk)return;



	memset(cStrxCmpCase,' ',sizeof(cStrxCmpCase));

	for(i='0';i<='9';i++)cStrxCmpCase[i]='0';
	for(i='A';i<='Z';i++)cStrxCmpCase[i]=i;
	for(i='a';i<='z';i++)cStrxCmpCase[i]=i+'A'-'a';

	cStrxCmpCase['C']='K';
	cStrxCmpCase['Y']='I';
	cStrxCmpCase['P']='B';
	cStrxCmpCase['T']='D';
	cStrxCmpCase['V']='F';

	if(pGrosKlein)
	for(;;)
		{
		ucChar1 = *pGrosKlein;pGrosKlein++;if(!ucChar1)break;
		ucChar2 = *pGrosKlein;pGrosKlein++;if(!ucChar2)break;
		cStrxCmpCase[ucChar1]=ucChar2;
		}

}

void _cdecl strxinit(const wchar_t *pGrosKlein)
{
int				i;
wchar_t 		ucChar1;
wchar_t			ucChar2;
static int  	iInitOk=0;



	if(iInitOk)return;



	memset(cStrxCmpCase,' ',sizeof(cStrxCmpCase));

	for(i='0';i<='9';i++)cStrxCmpCase[i]='0';
	for(i='A';i<='Z';i++)cStrxCmpCase[i]=i;
	for(i='a';i<='z';i++)cStrxCmpCase[i]=i+'A'-'a';

	cStrxCmpCase['C']='K';
	cStrxCmpCase['Y']='I';
	cStrxCmpCase['P']='B';
	cStrxCmpCase['T']='D';
	cStrxCmpCase['V']='F';

	if(pGrosKlein)
	for(;;)
		{
		ucChar1 = *pGrosKlein;pGrosKlein++;if(!ucChar1)break;
		ucChar2 = *pGrosKlein;pGrosKlein++;if(!ucChar2)break;

		if(ucChar1<255 && ucChar2<255)
			{
			cStrxCmpCase[ucChar1]=(char)ucChar2;
			}
		}

}

//*****************************************************************************
//*
//*		strxcmp
//*
//*****************************************************************************
//	Vergleicht zwei Strings auf ihre hnlichkeit
//	pText1	: Ist der Zeiger auf den ersten  String
//	pText2	: Ist der Zeiger auf den zweiten String
//	Ergibt		  0 : Wenn absolut gleich
//			  1-254 : Abweichung bis 25%
//			    255 : Ungleich
int _cdecl strxcmp(const char *pText1,const char *pText2)
{
int				i,j,iMin,iDivCase;
int 			iDiv,iNum,iMax;
int 			iAeiou,iRet=255;
int				iLen1, iLen2;
int 			iPos1, iPos2;
unsigned char  *pStr1,*pStr2;
unsigned char   ucChar;




	iLen1 = strlen(pText1);
	iLen2 = strlen(pText2);

	pStr1 = (unsigned char*)alloca(iLen1+1);
	pStr2 = (unsigned char*)alloca(iLen2+1);

	pStr1[iLen1]=0;
	pStr2[iLen2]=0;


	do	{
		iMin	 = 0;
		iDiv	 = 0;
		iDivCase = 0;
		iNum	 = (iLen1<iLen2)? iLen1:iLen2;

		for(i=0;i<iNum;i++)							// Gleiche Teile kopieren
			{
			if(pText2[i]!=pText1[i])iDiv++;
			pStr1[i]=cStrxCmpCase[(unsigned char)pText1[i]];
			pStr2[i]=cStrxCmpCase[(unsigned char)pText2[i]];
			if(pStr2[i]!=pStr1[i])iDivCase++;
			if(iDivCase==0)iMin++;
			}

		for(i=iNum;i<iLen1;i++)						// Rest kopieren
			{
			pStr1[i]=cStrxCmpCase[(unsigned char)pText1[i]];
			}

		for(i=iNum;i<iLen2;i++)
			{
			pStr2[i]=cStrxCmpCase[(unsigned char)pText2[i]];
			}

		if(iLen1==iLen2)							// Prfe auf Gleichheit
			{
			if(iDiv    ==0){iRet=0;break;}
			if(iDivCase==0){iRet=1;break;}
			}

//******************** Gleiche entfernen ****************************************

		iAeiou=0;

		for(i=0,iPos1=0;i<iLen1;i++)
			{
			ucChar=pStr1[i];
			while(ucChar==pStr1[i+1])i++;

			if(ucChar==' ')							// Space am Anfang berspringen
				{
				if(i>=iLen1)continue;				// Space am Ende   berspringen
				if(iPos1==0)continue;				// Space am Anfang berspringen
				}

			if(ucChar=='U' && iPos1)				// Q krzen
			if(pStr1[iPos1-1]=='Q')continue;
			if(ucChar=='H' && iPos1>0)				// Stumme H berspringen
				{
				if(pStr1[iPos1-1]!='P'){pStr1[iPos1-1]='F';continue;}
				if(pStr1[iPos1-1]!=' ')continue;
				}

			if(ucChar=='T')ucChar='D';				// D und T angleichen
			if(ucChar=='P')ucChar='B';				// B und P angleichen

			j=(1<<(ucChar-'A'))&AEIOU;				// Weiter Konsonanten
			if(iAeiou && j)continue;
			iAeiou=j;
			pStr1[iPos1]=ucChar;
			iPos1++;
			}

		iAeiou=0;

		for(i=0,iPos2=0;i<iLen2;i++)
			{
			ucChar=pStr2[i];
			while(ucChar==pStr2[i+1])i++;

			if(ucChar==' ')
				{
				if(i>=iLen2 )continue;				// Space am Ende   berspringen
				if(iPos2== 0)continue;				// Space am Anfang berspringen
				}

			if(ucChar=='U' && iPos2)				// Q krzen
			if(pStr2[iPos2-1]=='Q')continue;
			if(ucChar=='H' && iPos2)				// Stumme H berspringen
				{
				if(pStr2[iPos2-1]!='P'){pStr2[iPos2-1]='F';continue;}
				if(pStr2[iPos2-1]!=' ')continue;
				}

			if(ucChar=='T')ucChar='D';				// D und T angleichen
			if(ucChar=='P')ucChar='B';				// B und P angleichen

			j=(1<<(ucChar-'A'))&AEIOU;				// Weiter Konsonanten
			if(iAeiou && j)continue;
			iAeiou=j;
			pStr2[iPos2]=ucChar;
			iPos2++;
			}

		iLen1=iPos1;
		iLen2=iPos2;
		pStr1[iLen1]=0;
		pStr2[iLen2]=0;

//******************** Vergleiche  ********************************************

		iDiv=0;										// Vergleiche gleiche Lnge
		iNum=(iLen1<iLen2)? iLen1:iLen2;

		for(i=0;i<iNum;i++)
			{
			if(pStr1[i]!=pStr2[i])iDiv++;
			}

		iMax =(iLen1<iLen2)? iLen2:iLen1;

		if(iNum<8)iDiv+=iMax-iNum;
		if(iNum>0)
			{
			if(iMin+3>=iMax)iNum+=3;
			iDiv=(iDiv*1000)/iNum;
			if(iDiv<=250){iRet=1+iDiv;break;}
			}


		if(iNum>2)									// Verschub Vergleich
		if(pStr1[0]==pStr2[0])
		if(pStr1[1]==pStr2[1])
			{
			iDiv	= 0;
			iPos1	= 2;
			iPos2	= 2;
			iMax	= 2+iMax/8;

			while(iPos1<iLen1 && iPos2<iLen2)
				{
				if(pStr1[iPos1]==pStr2[iPos2])		// Gleiche Reihe
					{
					iPos1++;
					iPos2++;
					continue;
					}

				iDiv++;
				if(iDiv>iMax)break;

				if(pStr1[iPos1  ]==pStr2[iPos2+1])	// Veschubsprung
				if(pStr1[iPos1+1]==pStr2[iPos2+2])
					{
					iPos1++;
					iPos2+=2;
					continue;
					}

				if(pStr1[iPos1+1]==pStr2[iPos2  ])	// Veschubsprung
				if(pStr1[iPos1+2]==pStr2[iPos2+1])
					{
					iPos1+=2;
					iPos2++;
					continue;
					}

				iPos1++;
				iPos2++;
				}

			iDiv+=(iLen1-iPos1)+(iLen2-iPos2);

			if(iDiv<iMax)							// hnlich ?
				{
				iDiv=(iDiv*700)/iNum;
				if(iDiv<250){iRet=1+iDiv;break;}
				}
			}
		}while(0);


return iRet;
}

int _cdecl strxcmp(const wchar_t *pText1,const wchar_t *pText2)
{
int				i,j,iMin,iDivCase;
int 			iDiv,iNum,iMax;
int 			iAeiou,iRet=255;
int				iLen1, iLen2;
int 			iPos1, iPos2;
wchar_t		   *pStr1,*pStr2;
wchar_t			usChar;




	iLen1 = wcslen(pText1);
	iLen2 = wcslen(pText2);

	pStr1 = (wchar_t*)alloca((iLen1+1)*sizeof(wchar_t));
	pStr2 = (wchar_t*)alloca((iLen2+1)*sizeof(wchar_t));

	pStr1[iLen1] = 0;
	pStr2[iLen2] = 0;


	do	{
		iMin	 = 0;
		iDiv	 = 0;
		iDivCase = 0;
		iNum	 = (iLen1<iLen2)? iLen1:iLen2;

		for(i=0;i<iNum;i++)							// Gleiche Teile kopieren
			{
			if(pText2[i]!=pText1[i])iDiv++;
			pStr1[i]=cStrxCmpCase[(unsigned char)pText1[i]];
			pStr2[i]=cStrxCmpCase[(unsigned char)pText2[i]];
			if(pStr2[i]!=pStr1[i])iDivCase++;
			if(iDivCase==0)iMin++;
			}

		for(i=iNum;i<iLen1;i++)						// Rest kopieren
			{
			pStr1[i]=cStrxCmpCase[(unsigned char)pText1[i]];
			}

		for(i=iNum;i<iLen2;i++)
			{
			pStr2[i]=cStrxCmpCase[(unsigned char)pText2[i]];
			}

		if(iLen1==iLen2)							// Prfe auf Gleichheit
			{
			if(iDiv    ==0){iRet=0;break;}
			if(iDivCase==0){iRet=1;break;}
			}

//******************** Gleiche entfernen ****************************************

		iAeiou=0;

		for(i=0,iPos1=0;i<iLen1;i++)
			{
			usChar=pStr1[i];
			while(usChar==pStr1[i+1])i++;

			if(usChar==' ')							// Space am Anfang berspringen
				{
				if(i>=iLen1)continue;				// Space am Ende   berspringen
				if(iPos1==0)continue;				// Space am Anfang berspringen
				}

			if(usChar=='U' && iPos1)				// Q krzen
			if(pStr1[iPos1-1]=='Q')continue;
			if(usChar=='H' && iPos1>0)				// Stumme H berspringen
				{
				if(pStr1[iPos1-1]!='P'){pStr1[iPos1-1]='F';continue;}
				if(pStr1[iPos1-1]!=' ')continue;
				}

			if(usChar=='T')usChar='D';				// D und T angleichen
			if(usChar=='P')usChar='B';				// B und P angleichen

			j=(1<<(usChar-'A'))&AEIOU;				// Weiter Konsonanten
			if(iAeiou && j)continue;
			iAeiou=j;
			pStr1[iPos1]=usChar;
			iPos1++;
			}

		iAeiou=0;

		for(i=0,iPos2=0;i<iLen2;i++)
			{
			usChar=pStr2[i];
			while(usChar==pStr2[i+1])i++;

			if(usChar==' ')
				{
				if(i>=iLen2 )continue;				// Space am Ende   berspringen
				if(iPos2== 0)continue;				// Space am Anfang berspringen
				}

			if(usChar=='U' && iPos2)				// Q krzen
			if(pStr2[iPos2-1]=='Q')continue;
			if(usChar=='H' && iPos2)				// Stumme H berspringen
				{
				if(pStr2[iPos2-1]!='P'){pStr2[iPos2-1]='F';continue;}
				if(pStr2[iPos2-1]!=' ')continue;
				}

			if(usChar=='T')usChar='D';				// D und T angleichen
			if(usChar=='P')usChar='B';				// B und P angleichen

			j=(1<<(usChar-'A'))&AEIOU;				// Weiter Konsonanten
			if(iAeiou && j)continue;
			iAeiou=j;
			pStr2[iPos2]=usChar;
			iPos2++;
			}

		iLen1=iPos1;
		iLen2=iPos2;
		pStr1[iLen1]=0;
		pStr2[iLen2]=0;

//******************** Vergleiche  ********************************************

		iDiv=0;										// Vergleiche gleiche Lnge
		iNum=(iLen1<iLen2)? iLen1:iLen2;

		for(i=0;i<iNum;i++)
			{
			if(pStr1[i]!=pStr2[i])iDiv++;
			}

		iMax =(iLen1<iLen2)? iLen2:iLen1;

		if(iNum<8)iDiv+=iMax-iNum;
		if(iNum>0)
			{
			if(iMin+3>=iMax)iNum+=3;
			iDiv=(iDiv*1000)/iNum;
			if(iDiv<=250){iRet=1+iDiv;break;}
			}


		if(iNum>2)									// Verschub Vergleich
		if(pStr1[0]==pStr2[0])
		if(pStr1[1]==pStr2[1])
			{
			iDiv	= 0;
			iPos1	= 2;
			iPos2	= 2;
			iMax	= 2+iMax/8;

			while(iPos1<iLen1 && iPos2<iLen2)
				{
				if(pStr1[iPos1]==pStr2[iPos2])		// Gleiche Reihe
					{
					iPos1++;
					iPos2++;
					continue;
					}

				iDiv++;
				if(iDiv>iMax)break;

				if(pStr1[iPos1  ]==pStr2[iPos2+1])	// Veschubsprung
				if(pStr1[iPos1+1]==pStr2[iPos2+2])
					{
					iPos1++;
					iPos2+=2;
					continue;
					}

				if(pStr1[iPos1+1]==pStr2[iPos2  ])	// Veschubsprung
				if(pStr1[iPos1+2]==pStr2[iPos2+1])
					{
					iPos1+=2;
					iPos2++;
					continue;
					}

				iPos1++;
				iPos2++;
				}

			iDiv+=(iLen1-iPos1)+(iLen2-iPos2);

			if(iDiv<iMax)							// hnlich ?
				{
				iDiv=(iDiv*700)/iNum;
				if(iDiv<250){iRet=1+iDiv;break;}
				}
			}
		}while(0);



return iRet;
}

//*****************************************************************************
//*
//*		strxwcmp
//*
//*****************************************************************************
//	Vergleicht zwei Strings und sucht einzelne Wrter
//	pText1	: Ist der Zeiger auf den ersten  String
//	pText2	: Ist der Zeiger auf den zweiten String
//	Ergibt		  0 : Wenn absolut gleich
//			  1-254 : Abweichung bis 25%
//			    255 : Ungleich
int _cdecl strxwcmp(const char *pText1,const char *pText2)
{
char            cBuffer1[64],cBuffer2[64];
char           *pPtr1[STRXW_MAX],*pPtr2[STRXW_MAX];
char           *pStr1,*pStr2;
int				iDivMax,iDiv;
int				iLen1,iLen2;
int 			iNum1,iNum2;
int				i,j;




	while(*pText1==' '||*pText1=='\t')pText1++;	// Leerzeichen entfernen
	while(*pText2==' '||*pText2=='\t')pText2++;

	iLen1=strlen(pText1);
	iLen2=strlen(pText2);

	if(!iLen1 || !iLen2)return 255;				// Keine Wrter

	pStr1=(iLen1<=63)? cBuffer1:new char[iLen1+1];
	pStr2=(iLen2<=63)? cBuffer2:new char[iLen2+1];

	memcpy(pStr1,pText1,iLen1+1);
	memcpy(pStr2,pText2,iLen2+1);

	pPtr1[0]=pStr1;

	for(i=0,iNum1=1;i<iLen1;i++)				// Wrter suchen
		{
		if(pStr1[i]!=' '	&& pStr1[i]!='\t')continue;

		pStr1[i]=0;i++;
		while(pStr1[i+1]==' ' || pStr1[i]=='\t')i++;
		if(i>=iLen1)break;

		pPtr1[iNum1]=pStr1+i;
		iNum1++;
		if(iNum1>=STRXW_MAX)break;
		}

	pPtr2[0]=pStr2;

	for(i=0,iNum2=1;i<iLen2;i++)				// Wrter suchen
		{
		if(pStr2[i]!=' '	&& pStr2[i]!='\t')continue;

		pStr2[i]=0;i++;
		while(pStr2[i+1]==' ' || pStr2[i]=='\t')i++;
		if(i>=iLen2)break;


		pPtr2[iNum2]=pStr2+i;
		iNum2++;
		if(iNum2>=STRXW_MAX)break;
		}

//******************** Vergleich **********************************************

	iDivMax=255;


	if(iNum1>iNum2)
		{
		for(i=0;i<iNum2;i++)
		for(j=0;j<iNum1;j++)
			{
			   iDiv=strxcmp(pPtr1[j],pPtr2[i]);
			if(iDiv<iDivMax)iDivMax=iDiv;
			}
		if(iDivMax>250)iDivMax=255;
		}
	else{
		for(i=0;i<iNum1;i++)
		for(j=0;j<iNum2;j++)
			{
			   iDiv=strxcmp(pPtr2[j],pPtr1[i]);
			if(iDiv<iDivMax)iDivMax=iDiv;
			}
		if(iDivMax>250)iDivMax=255;
		}


	if(pStr1!=cBuffer1)delete pStr1;			// Puffer freigeben
	if(pStr2!=cBuffer2)delete pStr2;


return iDivMax;
}

int _cdecl strxwcmp(const wchar_t *pText1,const wchar_t *pText2)
{
wchar_t         usBuffer1[64];
wchar_t			usBuffer2[64];
wchar_t        *pPtr1[STRXW_MAX];
wchar_t		   *pPtr2[STRXW_MAX];
wchar_t        *pStr1,*pStr2;
int				iLen1, iLen2;
int 			iNum1, iNum2;
int				iDivMax,iDiv;
int				i,j;




	while(*pText1==' '||*pText1=='\t')pText1++;	// Leerzeichen entfernen
	while(*pText2==' '||*pText2=='\t')pText2++;

	iLen1 = wcslen(pText1);
	iLen2 = wcslen(pText2);

	if(!iLen1 || !iLen2)return 255;				// Keine Wrter

	pStr1 = (iLen1<=63)? usBuffer1:new wchar_t[iLen1+1];
	pStr2 = (iLen2<=63)? usBuffer2:new wchar_t[iLen2+1];

	memcpy(pStr1,pText1,(iLen1+1)*sizeof(wchar_t));
	memcpy(pStr2,pText2,(iLen2+1)*sizeof(wchar_t));

	pPtr1[0]=pStr1;

	for(i=0,iNum1=1;i<iLen1;i++)				// Wrter suchen
		{
		if(pStr1[i]!=' '	&& pStr1[i]!='\t')continue;

		pStr1[i]=0;i++;
		while(pStr1[i+1]==' ' || pStr1[i]=='\t')i++;
		if(i>=iLen1)break;

		pPtr1[iNum1]=pStr1+i;
		iNum1++;
		if(iNum1>=STRXW_MAX)break;
		}

	pPtr2[0]=pStr2;

	for(i=0,iNum2=1;i<iLen2;i++)				// Wrter suchen
		{
		if(pStr2[i]!=' '	&& pStr2[i]!='\t')continue;

		pStr2[i]=0;i++;
		while(pStr2[i+1]==' ' || pStr2[i]=='\t')i++;
		if(i>=iLen2)break;


		pPtr2[iNum2]=pStr2+i;
		iNum2++;
		if(iNum2>=STRXW_MAX)break;
		}

//******************** Vergleich **********************************************

	iDivMax=255;


	if(iNum1>iNum2)
		{
		for(i=0;i<iNum2;i++)
		for(j=0;j<iNum1;j++)
			{
			   iDiv=strxcmp(pPtr1[j],pPtr2[i]);
			if(iDiv<iDivMax)iDivMax=iDiv;
			}
		if(iDivMax>250)iDivMax=255;
		}
	else{
		for(i=0;i<iNum1;i++)
		for(j=0;j<iNum2;j++)
			{
			   iDiv=strxcmp(pPtr2[j],pPtr1[i]);
			if(iDiv<iDivMax)iDivMax=iDiv;
			}

		if(iDivMax>250)iDivMax=255;
		}


	if(pStr1!=usBuffer1)delete pStr1;			// Puffer freigeben
	if(pStr2!=usBuffer2)delete pStr2;


return iDivMax;
}
