//*****************************************************************************
//*
//*
//*		ComUtility.c
//*
//*
//*****************************************************************************
#ifdef		GLOBAL_CONFIG
#define 	COMUTILITY_C
#include	<GlobalConfig.h>
#undef		COMUTILITY_C
#endif

#include	<windows.h>
#include	<stdio.h>
#include	"ComUtility.h"


#ifndef		MAX_COM_PORTS
#define 	MAX_COM_PORTS		8					// count of simultaneously open ports
#endif

#ifndef		MAX_USED_COM
#define 	MAX_USED_COM		32					// highest com port for assign list		
#endif

#ifndef		COM_ASSIGN_LIST
#define		COM_ASSIGN_LIST		0					// activate the assingn list
#endif

#ifndef		COM_REMOTE_LOGGING
#define		COM_REMOTE_LOGGING	1					// activate remote logging via shm
#endif

#if			COM_REMOTE_LOGGING
#include	"RemoteInterface.h"
static		RemoteHandle hRemote = 0;
#endif

static	const BYTE			aModeP[]={NOPARITY,ODDPARITY,EVENPARITY,SPACEPARITY,MARKPARITY,NOPARITY,NOPARITY,NOPARITY};
static	const BYTE			aModeS[]={ONESTOPBIT,TWOSTOPBITS};

static	HANDLE				aComFile  [MAX_COM_PORTS];
static	OVERLAPPED			aComOverRd[MAX_COM_PORTS];
static	OVERLAPPED			aComOverWr[MAX_COM_PORTS];
static	OVERLAPPED			aComOverWt[MAX_COM_PORTS];
static	DWORD				aComEvent [MAX_COM_PORTS];
static	DWORD				aComLastWr[MAX_COM_PORTS];
static	DWORD				aComByteWr[MAX_COM_PORTS];
static	unsigned int 		aComMask  [MAX_COM_PORTS];
static	unsigned int 		aUsPerByte[MAX_COM_PORTS];
static	unsigned int 		aUsedPort [MAX_COM_PORTS];
static	unsigned char		aUseRs485 [MAX_COM_PORTS];
static	unsigned char		aIsPending[MAX_COM_PORTS];
static	unsigned char		aIsOpen	  [MAX_COM_PORTS];
static	unsigned			uNextSlot = 0;
static	HANDLE				hComSem   = 0;

#if		COM_ASSIGN_LIST
static	unsigned char		aAssign   [MAX_USED_COM];
#endif


//*****************************************************************************
//*
//*		ComInit
//*
//*****************************************************************************
//	Initializes the COM ports assigments
static void ComInit()
{
#if		COM_ASSIGN_LIST
char	cBuffer[256];
FILE   *pFile;
char   *pStr;
int		iNr;
#endif
 
 
	if(!hComSem)
		{
		hComSem = CreateSemaphore(NULL,1,0x10000000,NULL);
		}

	memset(aComFile,(int)INVALID_HANDLE_VALUE,sizeof(aComFile));
 
	#if COM_ASSIGN_LIST
 
	memset(aAssign,0,sizeof(aAssign));

		pFile = fopen("Config/System/ComAssign.cfg","rb");
	if(!pFile)
		{
			pFile = fopen("Config/System/ComAssign.cfg","wb");
		if(!pFile)return;

		for(iNr=1;iNr<=MAX_USED_COM;iNr++)
			{
			fprintf(pFile,"COM%i=%i\n",iNr,iNr);
			}

		fclose(pFile);
		return;
		}


	while(fgets(cBuffer,sizeof(cBuffer),pFile))
		{
			pStr = strstr(cBuffer,"COM");
		if(!pStr)continue;

		   iNr = atoi(pStr+3);
		if(iNr<=0 || iNr>MAX_USED_COM)continue;

			pStr = strstr(cBuffer,"=");
		if(!pStr)continue;

		aAssign[iNr-1] = atoi(pStr+1);
		}

	fclose(pFile);
 
	#endif

}

//*****************************************************************************
//*
//*		ComOpen
//*
//*****************************************************************************
//	Opens a serial COM port
//	uPort		: Is the number of the port (0=COM1 1=COM2 ...)
//	iBaud		: Is the baudrate
//	iParity		: Is the parity mode
//					0x00 = no parity bit	(PARITY__NONE)
//					0x01 = odd parity		(PARITY__ODD)
//					0x02 = even parity		(PARITY__EVEN)
//					0x03 = mark with 0		(PARITY__ZERO)
//					0x04 = mark with 1		(PARITY__ONE)
//	iStopbits	: Selects the count of stop bits
//					0x00 = one stopbit		(STOPBITS_ONE)
//					0x01 = two stopbits		(STOPBITS_TWO)
//	iDataBits	: Selects the count of data bits
//					0x00 = 8 databits
//					0x01 = 7 databits
//					0x07 = 7 databits		(DATABITS_SEVEN)
//					0x08 = 8 databits		(DATABITS_EIGHT)
//	iFlags		: Some flags
//					0x04 = RTS_ON			: switches on the RTS line
//				    0x08 = DTR_ON			: switches on the DTR line
//				  	0x10 = RTS_HANDSHAKE	: switches on the RDS handshake
//				  	0x20 = DTR_HANDSHAKE	: switches on the DTR handshake
//				  	0x40 = RS485_ON			: Switches on the RS485 option (not used in windows)
//	Returns a handle for the port or -1 if an error occurs
int ComOpen(unsigned uPort,int iBaud,int iParity,int iStopbits,int iDatabits,int iFlags)
{
static int			iIsInit=0;
char				cName[12]="\\\\.\\COM";
int					iBitsPerByte;
UINT				uValue;
HANDLE				hFile;
UINT				uStep;
UINT				uNr;
COMMTIMEOUTS		sTo;
DCB					sDcb;


	if(!iIsInit)
		{
		ComInit();
		iIsInit=1;
		}

	if(uPort>=256)										// check port number
		{
		return -1;
		}
 
 
	#if COM_ASSIGN_LIST
 
	if(uPort<MAX_USED_COM && aAssign[uPort])			// is an other port assigned
		{
		uPort	= aAssign[uPort]-1;
		}
 
	#endif

	WaitForSingleObject(hComSem,INFINITE);

	uNr			= uNextSlot%MAX_COM_PORTS;
	uNextSlot	= uNextSlot+1;
 
 
	for(uStep=0;;uStep++)								// find a free slot
		{
		if(uStep>=MAX_COM_PORTS)
			{
			return -1;
			}
 
		if(aComFile[uNr]==INVALID_HANDLE_VALUE)
			{
			break;
			}
 
		uNr = (uNr+1)%MAX_COM_PORTS;
		}
 
 
	ReleaseSemaphore(hComSem,1,NULL);
	
	sprintf(cName+7,"%i",uPort+1);


	   hFile= CreateFile(cName,GENERIC_READ|GENERIC_WRITE,0,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,0);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		hFile = NULL;
		return -1;
		}

	aComMask[uNr] = 0xFFFF;

	if(iFlags&RS485_ON     )iFlags&=~(DTR_HANDSHAKE|RTS_HANDSHAKE);
	if(iFlags&DTR_HANDSHAKE){aComMask[uNr]&=~DTR_ON;iFlags&=~DTR_ON;}
	if(iFlags&RTS_HANDSHAKE){aComMask[uNr]&=~RTS_ON;iFlags&=~RTS_ON;}


	memset(&sDcb,0,sizeof(sDcb));
	sDcb.DCBlength		 = sizeof(sDcb);
	sDcb.BaudRate     	 = iBaud;
	sDcb.fParity      	 = (iParity!=0)? TRUE:FALSE;
	sDcb.fBinary      	 = TRUE;
	sDcb.Parity       	 = aModeP[iParity  &7];
	sDcb.StopBits     	 = aModeS[iStopbits&1];
	sDcb.fOutxCtsFlow 	 = FALSE;
	sDcb.fOutxDsrFlow 	 = FALSE;
	sDcb.fDtrControl	 = (iFlags&DTR_HANDSHAKE)? DTR_CONTROL_HANDSHAKE:((iFlags&DTR_ON)? DTR_CONTROL_ENABLE:DTR_CONTROL_DISABLE);
	sDcb.fRtsControl	 = (iFlags&RTS_HANDSHAKE)? RTS_CONTROL_HANDSHAKE:((iFlags&RTS_ON)? RTS_CONTROL_ENABLE:RTS_CONTROL_DISABLE);
	sDcb.fDsrSensitivity = FALSE;
	sDcb.fAbortOnError	 = FALSE;
	sDcb.ByteSize     	 = (BYTE)((iDatabits&1)? 7:8);

	if(!SetCommState(hFile,&sDcb))
		{
		CloseHandle(hFile);
		return -1;
		}


	sTo.ReadIntervalTimeout		   = MAXDWORD; 			// 0 ms read-timeout
	sTo.ReadTotalTimeoutMultiplier = 0;
	sTo.ReadTotalTimeoutConstant   = 0;
	sTo.WriteTotalTimeoutMultiplier= 12000/iBaud+1;		// ms write timeout per byte
	sTo.WriteTotalTimeoutConstant  = sTo.WriteTotalTimeoutMultiplier+1;

	if(!SetCommTimeouts((HANDLE)hFile,&sTo))
		{
		CloseHandle(hFile);
		return -1;
		}

	               iBitsPerByte=10;						// calculate time for one byte
	if(iDatabits&1)iBitsPerByte--;
	if(iStopbits  )iBitsPerByte++;
	if(iParity!=0 )iBitsPerByte++;

	memset(aComOverRd+uNr,0,sizeof(OVERLAPPED));
	memset(aComOverWt+uNr,0,sizeof(OVERLAPPED));
	memset(aComOverWr+uNr,0,sizeof(OVERLAPPED));

	if(iBaud<=115200)
			uValue =  (iBaud/300);
	else	uValue = ((iBaud-115200)/19200)+0x180;

	aComOverRd[uNr].hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	aComOverWt[uNr].hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	aComOverWr[uNr].hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	aUsPerByte[uNr]  = (1000*1024*iBitsPerByte)/iBaud;
	aUseRs485 [uNr]  = (iFlags&RS485_ON)? 1:0;
	aComFile  [uNr]  = hFile;
	aIsOpen   [uNr]  = TRUE;
	aIsPending[uNr]  = 0;
	aComByteWr[uNr]  = 0;
	aComMask  [uNr]  = 0xFFFF;
	aUsedPort [uNr]  = (uPort<<16);
	aUsedPort [uNr] |= (iDatabits&1);
	aUsedPort [uNr] |= (iStopbits)? 2:0;
	aUsedPort [uNr] |= (iParity<<2);
	aUsedPort [uNr] |=  uValue <<5;

	SetupComm(hFile,2048,2048);
	SetCommMask(hFile,EV_RXCHAR);


	#if COM_REMOTE_LOGGING
	if(!hRemote)hRemote	= RemoteInit(31279,RemoteSize(0x10000,0x1000),0x1000,0x400,1);
	#endif


return uNr;
}



//*****************************************************************************
//*
//*		ComClose
//*
//*****************************************************************************
//	Closes a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns 0 if all ok, or -1 if an error occur
int	ComClose(int iHandle)
{

	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen[iHandle])return -1;
	if(!CloseHandle(aComFile[iHandle]))return -1;

	CloseHandle(aComOverRd[iHandle].hEvent);
	CloseHandle(aComOverWr[iHandle].hEvent);
	CloseHandle(aComOverWt[iHandle].hEvent);

	aComFile[iHandle] = NULL;
	aIsOpen [iHandle] = FALSE;
	

return 0;
}

//******************************************************************************
//*
//*		ComGetState
//*
//******************************************************************************
//	Gets the state of the CTS and DSR line
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns the state of the of the CTS and DSR line (CTS_ON=1,DSR_ON=2)
int	ComGetState(int iHandle)
{
int		iBits;
DWORD	dwState;



	if((unsigned) iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen  [iHandle])return -1;
	if( aUseRs485[iHandle])return DSR_ON|CTS_ON;

	if(!GetCommModemStatus(aComFile[iHandle],&dwState))
		{
		return -1;
		}

						 iBits =0;
	if(dwState&MS_DSR_ON)iBits|=DSR_ON;
	if(dwState&MS_CTS_ON)iBits|=CTS_ON;



return iBits;
}


//******************************************************************************
//*
//*		ComSetState
//*
//******************************************************************************
//	Set the state of the RTS and DTR lines
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	iBits	: Is the new state off the lines  (RTS_ON,DTR_ON)
//	iMask 	: Which lines should change       (RTS_ON,DTR_ON)
void ComSetState(int iHandle,int iBits,int iMask)
{
DCB		sDcb;



	if((unsigned)iHandle>=MAX_COM_PORTS)return;
	if(!aIsOpen[iHandle])return;

	sDcb.DCBlength=sizeof(sDcb);

		iMask&=aComMask[iHandle];
	if(!iMask)return;

	if(!GetCommState(aComFile[iHandle],&sDcb))return;

	if(iMask&RTS_ON)
		{
		sDcb.fRtsControl = (iBits&RTS_ON)? RTS_CONTROL_ENABLE:RTS_CONTROL_DISABLE;
		}

	if(iMask&DTR_ON)
		{
		sDcb.fDtrControl = (iBits&DTR_ON)? DTR_CONTROL_ENABLE:DTR_CONTROL_DISABLE;
		}


	SetCommState(aComFile[iHandle],&sDcb);

}

//*****************************************************************************
//*
//*		ComGetReadCount
//*
//*****************************************************************************
//	Gets the count of chars in the read buffer of a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns the count of chars in the read buffer
int ComGetReadCount(int iHandle)
{
COMSTAT		sComStat;
DWORD		dwErrorFlags;


	dwErrorFlags=0;

	if(!ClearCommError(aComFile[iHandle],&dwErrorFlags,&sComStat))return 0;

return sComStat.cbInQue;
}

//*****************************************************************************
//*
//*		ComCheckPort
//*
//*****************************************************************************
//	Checks if a com port is valid
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns 1 if ok or 0 if not.
int ComCheckPort(int iHandle)
{
COMSTAT		sComStat;
DWORD		dwErrorFlags;


	if(aComFile[iHandle]==0)return 0;

	dwErrorFlags=0;

	if(!ClearCommError(aComFile[iHandle],&dwErrorFlags,&sComStat))return 0;

return 1;
}


//*****************************************************************************
//*
//*		ComGetWriteCount
//*
//*****************************************************************************
//	Gets the count of chars in the read buffer of a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns the count of chars in the write buffer
int ComGetWriteCount(int iHandle)
{
COMSTAT		sComStat;
DWORD		dwErrorFlags;
DWORD		dwTime;
int			iDiff;


	dwErrorFlags=0;

	if(!ClearCommError(aComFile[iHandle],&dwErrorFlags,&sComStat))
		{
		return 0;
		}

	if(sComStat.cbOutQue==0 && aComByteWr[iHandle])			// Bug in XP und 2000
		{
		dwTime  =  aComLastWr[iHandle];
		dwTime += (aUsPerByte[iHandle]*aComByteWr[iHandle])/1024;
		iDiff   = dwTime-GetTickCount();

		if(iDiff>0 && aUsPerByte[iHandle])
			{
			iDiff				*= 1024;
			sComStat.cbOutQue	/= aUsPerByte[iHandle];
			if(sComStat.cbOutQue>aComByteWr[iHandle])
				{
				sComStat.cbOutQue = aComByteWr[iHandle];
				}
			}
		}


return sComStat.cbOutQue;
}



//*****************************************************************************
//*
//*		ComWaitUntilSend
//*
//*****************************************************************************
//	Waits until all bytes in the send queue are transmitted
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	uTimeout	: Is the timeout in miliseconts
//	Returns 1 if all bytes are sended
int ComWaitUntilSend(int iHandle,unsigned uTimeout)
{
DWORD			dwTime;
int				iDiff;



	if((unsigned)iHandle>=MAX_COM_PORTS)return 0;
	if(!aIsOpen[iHandle])return 0;

	if(aComByteWr[iHandle])
		{
		dwTime  =  aComLastWr[iHandle];
		dwTime += (aUsPerByte[iHandle]*aComByteWr[iHandle])/1024;
		iDiff	= dwTime-GetTickCount();

		if(iDiff<=0)
			{
			return 1;
			}
		else if(iDiff>(int)uTimeout)
			{
			Sleep(uTimeout);
			}
		else if(iDiff>0)
			{
			Sleep(iDiff);
			aComByteWr[iHandle]=0;
			}
		}


return 1;
}

//*****************************************************************************
//*
//*		ComWaitUntilResv
//*
//*****************************************************************************
//	Waits until a byte was reveived
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	uTimeout	: Is the timeout in miliseconts
//	Returns 1 if a byet was received
int	ComWaitUntilResv(int iHandle,unsigned uTimeout)
{
COMSTAT		sComStat;
DWORD		dwErrorFlags;
DWORD		dwCount;
DWORD		dwEnd;
int			iDiff;
int			iWait;
int			iStop;


	if((unsigned)iHandle>=MAX_COM_PORTS)return 0;
	if(!aIsOpen[iHandle])return 0;

	iStop	= 0;
	dwEnd	= GetTickCount()+uTimeout;

	for(;;)
		{
		aComEvent[iHandle]	= 0;
		dwErrorFlags		= 0;
															// Is someting in the receive buffer
		if(ClearCommError(aComFile[iHandle],&dwErrorFlags,&sComStat) && sComStat.cbInQue>0)
			{
			return 1;
			}

		if(iStop)break;

		   iDiff = dwEnd-GetTickCount();
		if(iDiff<=0)
			{
			iStop = 1;
			iDiff = 0;
			}

		if(iDiff>500)iDiff=500;
		iWait = 0;

															
															// Start new wait call
		if(!aIsPending[iHandle] && !WaitCommEvent(aComFile[iHandle],&aComEvent[iHandle],&aComOverWt[iHandle]))
			{
			if(GetLastError()==ERROR_IO_PENDING)
				{
				aIsPending[iHandle]=1;
				}
			else if(iDiff>0)
				{
				iWait = (iDiff>100)? 50:10;
				}
			}

		if(iWait)Sleep(iWait);

															// Wait for the next event
		if(aIsPending[iHandle] && WaitForMultipleObjects(1,&aComOverWt[iHandle].hEvent,TRUE,iDiff)==WAIT_OBJECT_0)
			{
			if(aIsPending[iHandle] && !GetOverlappedResult(aComFile[iHandle],&aComOverWt[iHandle],&dwCount,FALSE))
				{
				iWait = (iDiff>100)? 50:10;
				}

			aIsPending[iHandle] = 0;

			if(iWait)Sleep(iWait);
			}
		}


return 0;
}

//*****************************************************************************
//*
//*		ComWrite
//*
//*****************************************************************************
//	Sends some chars on a serial port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	pBuffer	: Is a pointer to a puffer with the chars
//	iSize	: Is the size of the buffer
//	Returns the count of sended chars, or -1 if an error occurs
int	ComWrite(int iHandle,const void *pBuffer,int iSize)
{
DWORD			dwCount;
DWORD			dwTime;
DWORD			dwWait;
DWORD			dwTurn;



	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen[iHandle])return -1;
	if(iSize<=0)return 0;


	aComLastWr[iHandle] = GetTickCount();
	aComByteWr[iHandle] = iSize;


	if(!WriteFile(aComFile[iHandle],pBuffer,iSize,&dwCount,&aComOverWr[iHandle]))
		{
		if(GetLastError()!=ERROR_IO_PENDING)
			{
			return -1;
			}

		dwWait	= GetTickCount()+(aUsPerByte[iHandle]*iSize)/1024+1000;
		
		
		for(dwTurn=0;;dwTurn++)
			{
			dwTime = dwWait-GetTickCount(); 

			if(dwTime&0x80000000)
				{
				if(dwTurn)return -1;
				dwTime = 10;
				}
			else if(dwTurn==0 && dwTime>10)
				{
				dwTime = 10;
				}
			else if(dwTime>30)
				{
				dwTime = 30;
				}

			if(WaitForMultipleObjects(1,&aComOverWr[iHandle].hEvent,TRUE,dwTime)!=WAIT_OBJECT_0)
				{
				continue;
				}

			if(GetOverlappedResult(aComFile[iHandle],&aComOverWr[iHandle],&dwCount,FALSE))
				{
				break;
				}
			}
		}

	aComByteWr[iHandle] = dwCount;

	#if COM_REMOTE_LOGGING
	if(dwCount>0)RemoteSendSub(hRemote,pBuffer,dwCount,aUsedPort[iHandle]|0x8000,0x7F);
	#endif


return dwCount;
}


//*****************************************************************************
//*
//*		ComRead
//*
//*****************************************************************************
//	Read some chars from a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	pBuffer	: Is a pointer to a puffer for the received chars
//	iSize	: Is the size of the buffer
//	Returns the count of received chars, or -1 if an error occurs
int ComRead(int iHandle,void *pBuffer,int iSize)
{
unsigned char  *pData=(unsigned char*)pBuffer;
DWORD			dwCount;
DWORD			dwTime;



	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen[iHandle])return -1;
	if(iSize==0)return 0;


	if(!ReadFile(aComFile[iHandle],pData,iSize,&dwCount,&aComOverRd[iHandle]))
		{
		if(GetLastError()!=ERROR_IO_PENDING)
			{
			return -1;
			}

		dwTime = (aUsPerByte[iHandle]*iSize)/1024+1000;

		if(WaitForMultipleObjects(1,&aComOverRd[iHandle].hEvent,TRUE,dwTime)!=WAIT_OBJECT_0)
			{
			return -1;
			}

		if(!GetOverlappedResult(aComFile[iHandle],&aComOverRd[iHandle],&dwCount,FALSE))
			{
			return -1;
			}
		}


	#if COM_REMOTE_LOGGING
	if(dwCount>0)RemoteSendSub(hRemote,pBuffer,dwCount,aUsedPort[iHandle],0x7F);
	#endif


return dwCount;
}

//*****************************************************************************
//*
//*		ComReadTimeout
//*
//*****************************************************************************
//	Read some chars from a serial COM port , with a timeout
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	pBuffer		: Is a pointer to a puffer for the received chars
//	iSize		: Is the size of the buffer
//	uTimeout	: Is the time out value in ms
//	Returns the count of received chars, or -1 if an error occurs
int  ComReadTimeout(int iHandle,void *pBuffer,int iSize,unsigned uTimeout)
{
char	   *pData=(char*)pBuffer;
unsigned 	uStart;
unsigned 	uCount;
int  		iDelta;
int			iNum=0;
int			iRet;


	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen[iHandle])return -1;
	if(iSize<=0)
		{
		if(uTimeout)Sleep(uTimeout);
		return 0;
		}


	uStart = GetTickCount()+uTimeout;

	if(uTimeout==0)
		{
		   uCount = ComGetReadCount(iHandle);
		if(uCount==0)return 0;
		}
	else if(!ComWaitUntilResv(iHandle,uTimeout))		// wait until first byte
		{
		return 0;
		}

	for(;;)
		{
		   uCount = ComGetReadCount(iHandle);
		if(uCount>(unsigned)iSize)uCount=iSize;

		   iRet = ComRead(iHandle,pData,uCount);
		if(iRet<0)
			{
			return (iNum==0)? -1:iNum;
			}

		uStart +=(iRet*aUsPerByte[iHandle])/1024;
		pData  += iRet;
		iNum   += iRet;
		iSize  -= iRet;

		if(iSize<=0)break;

		   iDelta = uStart-GetTickCount();
		if(iDelta<0)break;

		if(!ComWaitUntilResv(iHandle,iDelta))			// wait until next byte
			{
			break;
			}
		}


return iNum;
}

//*****************************************************************************
//*
//*		ComReadUntil
//*
//*****************************************************************************
//	Read some chars from a serial COM port, with a timeout and a stop byte
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	pBuffer		: Is a pointer to a puffer for the received chars
//	iSize		: Is the size of the buffer
//	uTimeout	: Is the time out value in ms
//	ucStop		: If this byte is received the function is stoped
//	Returns the count of received chars, or -1 if an error occurs
int  ComReadUntil(int iHandle,void *pBuffer,int iSize,unsigned uTimeout,unsigned char ucStop)
{
unsigned char  *pData=(unsigned char*)pBuffer;
unsigned 		uStart;
unsigned		uCount;
int 			iDelta;
int				iNum=0;
int				iRet;
int				iPos;


	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen[iHandle])return -1;
	if(iSize<=0)
		{
		if(uTimeout)Sleep(uTimeout);
		return 0;
		}

	uStart = GetTickCount()+uTimeout+20;

	if(uTimeout==0)
		{
		   uCount = ComGetReadCount(iHandle);
		if(uCount==0)return 0;
		}
	else if(!ComWaitUntilResv(iHandle,uTimeout))		// wait until first byte
		{
		return 0;
		}

	for(;;)
		{
		   uCount = ComGetReadCount(iHandle);
		if(uCount>(unsigned)iSize)uCount=iSize;

		   iRet = ComRead(iHandle,pData,uCount);
		if(iRet<0)
			{
			return (iNum==0)? -1:iNum;
			}

		uStart +=(iRet*aUsPerByte[iHandle])/1024;
		pData  += iRet;
		iNum   += iRet;
		iSize  -= iRet;

		if(iSize<=0)break;

		for(iPos=1;iPos<=iRet;iPos++)					// is the stop byte in the data
			{
			if(pData[-iPos]==ucStop)
				{
				return iNum;
				}
			}

		   iDelta = uStart-GetTickCount();
		if(iDelta<=0)
			{
			break;
			}

		if(!ComWaitUntilResv(iHandle,iDelta))			// wait until next byte
			{
			break;
			}
		}


return iNum;
}

//*****************************************************************************
//*
//*		ComGetByteSendTime
//*
//*****************************************************************************
//	Returns the time which is needed to send one byte in s
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	Returns the time to send one byte or -1 on error
int	ComGetByteSendTime(int iHandle)
{


	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen[iHandle])return -1;


return aUsPerByte[iHandle];
}


//*****************************************************************************
//*
//*		ComChangeMode
//*
//*****************************************************************************
//	Changes the mode of a com port
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	iBaud		: Is the baudrate (-1 means no change)
//	iParity		: Is the parity mode (-1 means no change)
//					0x00 = no parity bit	(PARITY__NONE)
//					0x01 = odd parity		(PARITY__ODD)
//					0x02 = even parity		(PARITY__EVEN)
//					0x03 = mark with 0		(PARITY__ZERO)
//					0x04 = mark with 1		(PARITY__ONE)
//	iStopbits	: Selects the count of stop bits (-1 means no change)
//					0x00 = one stopbit		(STOPBITS_ONE)
//					0x01 = two stopbits		(STOPBITS_TWO)
//	iDataBits	: Selects the count of data bits (-1 means no change)
//					0x00 = 8 databits
//					0x01 = 7 databits
//					0x07 = 7 databits		(DATABITS_SEVEN)
//					0x08 = 8 databits		(DATABITS_EIGHT)
//	iFlags		: Some flags (-1 means no change)
//					0x04 = RTS_ON			: switches on the RTS line
//				    0x08 = DTR_ON			: switches on the DTR line
//				  	0x10 = RTS_HANDSHAKE	: switches on the RDS handshake
//				  	0x20 = DTR_HANDSHAKE	: switches on the DTR handshake
//				  	0x40 = RS485_ON			: Switches on the RS485 option (not used in windows)
//	Returns 0 if ok or -1 if an error occurs
int ComChangeMode(int iHandle,int iBaud,int iParity,int iStopbits,int iDatabits,int iFlags)
{
INT					iBitsPerByte;
UINT				uValue;	
HANDLE				hFile;
UINT				uFlag;
DCB					sDcb;
COMMTIMEOUTS		sTo;




	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!aIsOpen[iHandle])return -1;

	hFile				 = aComFile[iHandle];
	sDcb.DCBlength		 = sizeof(sDcb);

	if(!GetCommState(hFile,&sDcb))return -1;			// get current states	
												
	if(iBaud<0)
		{
		iBaud			 = sDcb.BaudRate;
		uFlag			 = 0;
		}
	else{
		sDcb.BaudRate	 = iBaud;
		uFlag			 = 1;
		}

	if(iParity<0)
		{
		iParity			 = sDcb.fParity;
		}
	else{
		sDcb.fParity  	 = (iParity!=0)? TRUE:FALSE;
		sDcb.Parity   	 = aModeP[iParity  &7];
		}

	if(iStopbits<0)
		{
		iStopbits		 = (sDcb.StopBits==ONESTOPBIT)? 0:1;
		}
	else{
		sDcb.StopBits  	 = aModeS[iStopbits&1];
		}

	if(iDatabits<0)
		{
		iDatabits		 = iDatabits&1;
		}
	else{
		sDcb.ByteSize    = (BYTE)((iDatabits&1)? 7:8);
		}


	if(iFlags>=0)
		{
		if(iFlags&RS485_ON)
			{
			iFlags &= ~(DTR_HANDSHAKE|RTS_HANDSHAKE);
			}

		sDcb.fDtrControl = (iFlags&DTR_HANDSHAKE)? DTR_CONTROL_HANDSHAKE:((iFlags&DTR_ON)? DTR_CONTROL_ENABLE:DTR_CONTROL_DISABLE);
		sDcb.fRtsControl = (iFlags&RTS_HANDSHAKE)? RTS_CONTROL_HANDSHAKE:((iFlags&RTS_ON)? RTS_CONTROL_ENABLE:RTS_CONTROL_DISABLE);
		}

	if(!SetCommState(hFile,&sDcb))
		{
		CloseHandle(hFile);
		return -1;
		}

	if(uFlag)											// baudrate changed
		{
		sTo.ReadIntervalTimeout		   = MAXDWORD; 		// 0 ms read-timeout
		sTo.ReadTotalTimeoutMultiplier = 0;
		sTo.ReadTotalTimeoutConstant   = 0;
		sTo.WriteTotalTimeoutMultiplier= 12000/iBaud+1;	// ms write timeout per byte
		sTo.WriteTotalTimeoutConstant  = sTo.WriteTotalTimeoutMultiplier+1;

		if(!SetCommTimeouts((HANDLE)hFile,&sTo))
			{
			CloseHandle(hFile);
			return -1;
			}
		}

	               iBitsPerByte=10;						// calculate time for one byte
	if(iDatabits&1)iBitsPerByte--;
	if(iStopbits  )iBitsPerByte++;
	if(iParity!=0 )iBitsPerByte++;

	if(iBaud<=115200)
			uValue =  (iBaud/300);
	else	uValue = ((iBaud-115200)/19200)+0x180;

	aUsPerByte[iHandle]  = (1000*1024*iBitsPerByte)/iBaud;
	aUsedPort [iHandle] &= 0xFFFF0000;
	aUsedPort [iHandle] |= (iDatabits&1);
	aUsedPort [iHandle] |= (iStopbits)? 2:0;
	aUsedPort [iHandle] |= (iParity<<2);
	aUsedPort [iHandle] |=  uValue <<5;


return 0;
}
