//******************************************************************************
//*
//*
//*		ComUtilityLinux.c
//*
//*
//******************************************************************************
#ifdef		GLOBAL_CONFIG
#define 	COMUTILITYLINUX_C
#include	<GlobalConfig.h>
#undef		COMUTILITYLINUX_C
#endif

#include	<unistd.h>
#include	<memory.h>
#include	<stdio.h>
#include	<errno.h>
#include	<termio.h>
#include	<sys/time.h>
#include	<sys/fcntl.h>
#include	<sys/ioctl.h>
#include	<sys/resource.h>
#include	<linux/serial.h>
#include	<pthread.h>
#include	<sched.h>
#include	<setjmp.h>
#include	<semaphore.h>
#include	<time.h>
#include	"ComUtility.h"

#include	<linux/version.h>
#if 		LINUX_VERSION_CODE < 0x020400
#warning	"LINUX Version NOT OK"
#endif

#ifndef		COM_REMOTE_LOGGING
#define		COM_REMOTE_LOGGING	1
#endif

#if			COM_REMOTE_LOGGING
#include	"RemoteInterface.h"
static		RemoteHandle hRemote = 0;
#endif


typedef	struct
	{
	unsigned		iBaud;
	speed_t 		iCode;
	}TcSpeed;

static TcSpeed aTcSpeeds[] =
	{
	#ifdef B300
		{300, B300},
	#endif
	#ifdef B600
		{600, B600},
	#endif
	#ifdef B1200
		{1200, B1200},
	#endif
	#ifdef B1800
		{1800, B1800},
	#endif
	#ifdef B2400
		{2400, B2400},
	#endif
	#ifdef B4800
		{4800, B4800},
	#endif
	#ifdef B9600
		{9600, B9600},
	#endif
	#ifdef B19200
		{19200, B19200},
	#endif
	#ifdef B38400
		{38400, B38400},
	#endif
	#ifdef B57600
		{57600, B57600},
	#endif
	#ifdef B115200
		{115200, B115200},
	#endif
	};

#define		MAX_SPEED		(sizeof(aTcSpeeds)/sizeof(aTcSpeeds[0]))
#ifndef		MAX_COM_PORTS
#define 	MAX_COM_PORTS	32
#endif

static	unsigned char 	aIsOpen	  [MAX_COM_PORTS*2];
static	unsigned int 	aUsPerByte[MAX_COM_PORTS*2];
static	unsigned int  	aUsedPort [MAX_COM_PORTS*2];
static	int				aComFile  [MAX_COM_PORTS*2];
static	int				aComBaud  [MAX_COM_PORTS*2];



//*****************************************************************************
//*
//*		ComSleep
//*
//*****************************************************************************
// Call this funktion form the thread
static void ComSleep( unsigned uMilliSec)
{
struct timeval		sTime;
struct timespec 	sSpec;
pthread_mutex_t 	sMutex;
pthread_cond_t  	sCond;


	gettimeofday(&sTime,0);


	sSpec.tv_nsec  =(sTime.tv_usec + (uMilliSec % 1000) * 1000) * 1000;
	sSpec.tv_sec   = sTime.tv_sec  + (uMilliSec / 1000) + (sSpec.tv_nsec / 1000000000);
	sSpec.tv_nsec %= 1000000000;


	pthread_mutex_init    (&sMutex,0);
	pthread_cond_init     (&sCond ,0);

	pthread_mutex_lock    (&sMutex);
	pthread_cond_timedwait(&sCond,&sMutex,&sSpec);
	pthread_mutex_unlock  (&sMutex);

	pthread_cond_destroy  (&sCond );
	pthread_mutex_destroy (&sMutex);

}

//*****************************************************************************
//*
//*		ComOpen
//*
//*****************************************************************************
//	Opens a serial COM port
//	uNr			: Is the number of the  COM ports (0=COM1 1=COM2 ... 64=USB1 65=USB2 ...)
//	iBaud		: Is the baudrate
//	iParity		: Is the parity mode
//					0x00 = no parity bit	(PARITY__NONE)
//					0x01 = odd parity		(PARITY__ODD)
//					0x02 = even parity		(PARITY__EVEN)
//					0x03 = mark with 0		(PARITY__ZERO) (not supported in linux)
//					0x04 = mark with 1		(PARITY__ONE)  (not supported in linux)
//	iStopbits	: Selects the count of stop bits
//					0x00 = one stopbit		(STOPBITS_ONE)
//					0x01 = two stopbits		(STOPBITS_TWO)
//	iDataBits	: Selects the count of data bits
//					0x00 = 8 databits
//					0x01 = 7 databits
//					0x07 = 7 databits		(DATABITS_SEVEN)
//					0x08 = 8 databits		(DATABITS_EIGHT)
//	iFlags		: Some flags
//					0x04 = RTS_ON			: switches on the RTS line
//				    0x08 = DTR_ON			: switches on the DTR line
//				  	0x10 = RTS_HANDSHAKE	: switches on the RDS handshake
//				  	0x20 = DTR_HANDSHAKE	: switches on the DTR handshake
//				  	0x40 = RS485_ON			: Switches on the RS485 option (not used in linux)
//	Ergibt ein Handel wenn eine Schnittstelle geffnet wurde sonst -1
int ComOpen(unsigned uNr,int iBaud,int iParity,int iStopbits,int iDatabits,int iFlags)
{
struct		serial_struct sData;
struct      termios sPortInfo;
char		cNameUsb[]="/dev/ttyUSB00";
char		cName   []="/dev/ttyS00";
int			iBitsPerByte;
int			iHandle;
int			iBaudNr;
int 		iValue;
int			i;



	if(uNr>=MAX_COM_PORTS)
		{
		if(uNr>=1000 && uNr<1000+MAX_COM_PORTS)			// is it a USB port (1000...)
			{
			uNr -= 1000;
			}
		else if(uNr>=64 && uNr<64+MAX_COM_PORTS)		// is it a USB port (64...)
			{
			uNr -= 64;
			}
		else{
			return -1;									// wrong pom port number
			}

		if(aIsOpen[uNr+MAX_COM_PORTS])
			{
			printf("WARING: USB_COM%u is allready opened !\n\a",uNr+1001);
			return -1;
			}

		if(uNr<10)
			{
			cNameUsb[11] += uNr;
			cNameUsb[12]  = 0;
			}
		else{
			cNameUsb[11] += uNr%10;
			cNameUsb[12] += uNr/10;
			}
														// open USB device
	    iHandle = open(cNameUsb,O_RDWR|O_NONBLOCK|O_NOCTTY);
		uNr    += MAX_COM_PORTS;
		aUsedPort[uNr] = (uNr-MAX_COM_PORTS+64)<<16;
		}
	else{
		if(aIsOpen[uNr])
			{
			printf("WARING: COM%u is allready opened !\n\a",uNr+1);
			return -1;
			}

		if(uNr<10)
			{
			cName[ 9] += uNr;
			cName[10]  = 0;
			}
		else{
			cName[ 9] += uNr%10;
			cName[10] += uNr/10;
			}
														// open com port
	    iHandle = open(cName,O_RDWR|O_NONBLOCK|O_NOCTTY);
		aUsedPort[uNr] = uNr<<16;
		}


    if(iHandle<0)return -1;


	if(iFlags&RS485_ON)
		{
		iFlags &= ~(DTR_HANDSHAKE|RTS_HANDSHAKE);
		}

	if(iFlags&DTR_HANDSHAKE){iFlags&=~DTR_ON;}
	if(iFlags&RTS_HANDSHAKE){iFlags&=~RTS_ON;}

	tcgetattr(iHandle, &sPortInfo);
	cfmakeraw(&sPortInfo);
	sPortInfo.c_cc[VMIN] = 0;
	sPortInfo.c_cc[VTIME]= 0;

	if(iStopbits!=STOPBITS_ONE)							// set stop bits
    		sPortInfo.c_cflag |=  CSTOPB;
    else	sPortInfo.c_cflag &= ~CSTOPB;

	sPortInfo.c_cflag &= ~CSIZE; 						// set data bits
	sPortInfo.c_cflag |= (iDatabits&1)? CS7:CS8;

	if(iFlags&RTS_HANDSHAKE)							// set RTS handshake
			sPortInfo.c_cflag |=  CRTSCTS;
	else	sPortInfo.c_cflag &= ~CRTSCTS;

	switch(iParity)										// set parity bits
		{
	case PARITY__NONE:  sPortInfo.c_cflag =  sPortInfo.c_cflag & ~PARENB;				break;
	case PARITY__ODD :	sPortInfo.c_cflag =  sPortInfo.c_cflag |  PARENB  |  PARODD;	break;
	case PARITY__EVEN:	sPortInfo.c_cflag = (sPortInfo.c_cflag |  PARENB) & ~PARODD;	break;
	default:			close(iHandle);
						return -1;
		}

	tcsetattr(iHandle,TCSANOW,&sPortInfo);

	for(i=0;i<MAX_SPEED;i++)
		{
		if(aTcSpeeds[i].iBaud==iBaud)break;
		}

	if(i>=MAX_SPEED)
		{
		errno=-EFAULT;
		close(iHandle);
		return -1;
		}

	ioctl(iHandle,TIOCGSERIAL,&sData);					// remove special baudrates
	sData.flags &= ~ASYNC_SPD_MASK;
	ioctl(iHandle,TIOCSSERIAL,&sData);

	iBaudNr=aTcSpeeds[i].iCode;
    tcgetattr(iHandle,&sPortInfo);
    cfsetospeed(&sPortInfo, iBaudNr);
    cfsetispeed(&sPortInfo, iBaudNr);
    tcsetattr(iHandle,TCSANOW,&sPortInfo);


	i  = (iFlags&RTS_ON)? 0:TIOCM_RTS;					// turn off RTS DTR
	i |= (iFlags&DTR_ON)? 0:TIOCM_DTR;

	ioctl(iHandle,TIOCMBIC,&i);

	i  = (iFlags&RTS_ON)? TIOCM_RTS:0;					// turn on RTS DTR
	i |= (iFlags&DTR_ON)? TIOCM_DTR:0;

	ioctl(iHandle,TIOCMBIS,&i);



	               iBitsPerByte=10;						// calculate time for one byte
	if(iDatabits&1)iBitsPerByte--;
	if(iStopbits  )iBitsPerByte++;
	if(iParity!=0 )iBitsPerByte++;

	if(iBaud<=115200)
			iValue =  (iBaud/300);
	else	iValue = ((iBaud-115200)/19200)+0x180;

	aUsPerByte[uNr]  = (1000*1024*iBitsPerByte)/iBaud;
	aComFile  [uNr]  = iHandle;
	aComBaud  [uNr]  = iBaud;
	aIsOpen   [uNr]  = 1;
	aUsedPort [uNr] |= (iDatabits&1);
	aUsedPort [uNr] |= (iStopbits)? 2:0;
	aUsedPort [uNr] |= (iParity<<2);
	aUsedPort [uNr] |=  iValue <<5;



	#if COM_REMOTE_LOGGING
	if(!hRemote)hRemote	= RemoteInit(31279,RemoteSize(0x10000,0x1000),0x1000,0x400,1);
	#endif


return uNr;
}

//*****************************************************************************
//*
//*		ComClose
//*
//*****************************************************************************
//	Closes a serial COM port
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	Returns 0 if all ok, or -1 if an error occur
int	ComClose(int iIndex)
{


	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return -1;
		}


	close(aComFile[iIndex]);

	aComFile[iIndex] = -1;
	aIsOpen [iIndex] =  0;


return 0;
}



//******************************************************************************
//*
//*		ComSetState
//*
//******************************************************************************
//	Set the state of the RTS and DTR lines
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	iBits		: Is the new state off the lines  (RTS_ON,DTR_ON)
//	iMask 		: Which lines should change       (RTS_ON,DTR_ON)
void ComSetState(int iIndex,int iBits,int iMask)
{
int		i;



	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return;
		}

	if(iMask&RTS_ON)
		{
		i  = TIOCM_RTS;

		if(iBits&RTS_ON)
				ioctl(aComFile[iIndex], TIOCMBIS,&i);
		else	ioctl(aComFile[iIndex], TIOCMBIC,&i);

		}

	if(iMask&DTR_ON)
		{
		i  = TIOCM_DTR;

		if(iBits&DTR_ON)
				ioctl(aComFile[iIndex],TIOCMBIS,&i);
		else	ioctl(aComFile[iIndex],TIOCMBIC,&i);
		}


}

//******************************************************************************
//*
//*		ComGetState
//*
//******************************************************************************
//	Get the states of serveral lines
//	iIndex	: Is the handle of the com port (look at ComOpen)
//	Returns the state of the of the CTS and DSR line (CTS_ON=1,DSR_ON=2)
int	ComGetState(int iIndex)
{
int		i,iRet=0;


	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return 0;
		}


	if(ioctl(aComFile[iIndex],TIOCMGET,&i)<0)return 0;


	if(i&TIOCM_CTS)iRet|=CTS_ON;
	if(i&TIOCM_DSR)iRet|=DSR_ON;


return iRet;
}

//*****************************************************************************
//*
//*		ComGetReadCount
//*
//*****************************************************************************
//	Ergibt die Anzahl der Bytes die im Lesepuffer der Schnittstelle sind
//	iIndex		: Ist das Hande fr das Com-Port (siehe ComOpen)
//	Ergibt die Anzahl der Bytes im Buffer
int ComGetReadCount(int iIndex)
{
long	lLength;


	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return 0;
		}

	if(ioctl(aComFile[iIndex],TIOCINQ,&lLength)<0)return 0;


return lLength;
}


//*****************************************************************************
//*
//*		ComWaitGetWriteCount
//*
//*****************************************************************************
//	Ergibt die Anzahl der Bytes die im Schreibpuffer der Schnittstelle sind
//	iIndex		: Ist das Hande fr das Com-Port (siehe ComOpen)
//	Ergibt die Anzahl der Bytes im Buffer
int ComGetWriteCount(int iIndex)
{
long	lLength;


	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return 0;
		}

	if(ioctl(aComFile[iIndex],TIOCOUTQ,&lLength)<0)return 0;


return lLength;
}

//*****************************************************************************
//*
//*		ComWaitUntilSend
//*
//*****************************************************************************
//	Waits until all bytes in the send queue are transmitted
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	uTimeout	: Is the timeout in miliseconts
//	Returns 1 if all bytes are sended
int ComWaitUntilSend(int iIndex,unsigned uTimeout)
{
long			lLength;
unsigned long 	ulTime;
unsigned long 	ulUs;



	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return 0;
		}


	ulUs = aUsPerByte[iIndex];


	for(;;)
		{
		lLength=0;

		if(ioctl(aComFile[iIndex],TIOCOUTQ,&lLength)<0)break;

		if(lLength<=0)break;

		   ulTime = ((lLength*ulUs)>>10)+5;
		if(ulTime<10)ulTime=10;

		ComSleep(ulTime);

		if(uTimeout<ulTime)break;

		uTimeout -= ulTime;
		}


return 1;
}

//*****************************************************************************
//*
//*		ComWaitUntilResv
//*
//*****************************************************************************
//	Waits until a byte was reveived
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	uTimeout	: Is the timeout in miliseconts
//	Returns 1 if a byet was received
int	ComWaitUntilResv(int iIndex,unsigned uTimeout)
{
struct timeval	sWait;
int 			iRet;
fd_set			sRead;



	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return -1;
		}


	sWait.tv_sec  =  uTimeout/1000;
	sWait.tv_usec = (uTimeout%1000)*1000;

	FD_ZERO(&sRead);
	FD_SET(aComFile[iIndex],&sRead);
															// wait unti a byte was received
		iRet=select(aComFile[iIndex]+1,&sRead,NULL,NULL,&sWait);
	if(!iRet)
		{
		return 0;
		}


return 1;
}

//*****************************************************************************
//*
//*		ComWrite
//*
//*****************************************************************************
//	Sends some chars on a serial port
//	iIndex	: Is the handle of the com port (look at ComOpen)
//	pBuffer	: Is a pointer to a puffer with the chars
//	iSize	: Is the size of the buffer
//	Returns the count of sended chars, or -1 if an error occurs
int	ComWrite(int iIndex,const void *pBuffer,int iSize)
{
int		iCount;


	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return -1;
		}

	if(iSize<=0)return 0;

	iCount = write(aComFile[iIndex],pBuffer,iSize);

	if(iCount<=0)return 0;

	#if COM_REMOTE_LOGGING
	RemoteSendSub(hRemote,pBuffer,iCount,aUsedPort[iIndex]|0x8000,0x7F);
	#endif


return iCount;
}

//*****************************************************************************
//*
//*		ComRead
//*
//*****************************************************************************
//	Read some chars from a serial COM port
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	pBuffer		: Is a pointer to a puffer for the received chars
//	iSize		: Is the size of the buffer
//	Returns the count of received chars, or -1 if an error occurs
int ComRead(int iIndex,void *pBuffer,int iSize)
{
int		iCount;


	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return -1;
		}

	if(iSize<=0)return 0;

	iCount = read(aComFile[iIndex],pBuffer,iSize);

	if(iCount<0 && errno==EAGAIN)
		{
		return 0;
		}

	#if COM_REMOTE_LOGGING
	if(iCount>0)RemoteSendSub(hRemote,pBuffer,iCount,aUsedPort[iIndex],0x7F);
	#endif


return iCount;
}

//******************************************************************************
//*
//*		ComReadTimeout
//*
//******************************************************************************
//	Liest Daten von der seriellen Schnittstelle ein. Sollen bis zu einem
//	Timeot nicht alle Bytes empfangen worden sie so bricht die Funktion ab
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	pBuffer		: Is a pointer to a puffer for the received chars
//	iSize		: Is the size of the buffer
//	uTimeout	: Is the time out value in ms
//	Returns the count of received chars, or -1 if an error occurs
int	ComReadTimeout(int iIndex,void *pBuffer,int iSize,unsigned uTimeout)
{
struct timeval	sStart,sTime,sWait;
unsigned char  *pData=(unsigned char*)pBuffer;
unsigned 		uBytes;
unsigned long 	ulMsWait;
unsigned long 	ulDiff;
unsigned long 	ulUs;
int 			iRet;
fd_set			sRead;



	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return -1;
		}

	if(iSize<=0)
		{
		if(uTimeout)
			{
			sWait.tv_sec  =  uTimeout/1000;
			sWait.tv_usec = (uTimeout%1000)*1000;
	
			select(0,NULL,NULL,NULL,&sWait);
			}
			
		return 0;
		}
	
	ulUs = aUsPerByte[iIndex];

	gettimeofday(&sStart,NULL);


	for(ulDiff=0,uBytes=0;uBytes<(unsigned)iSize;)
		{
		ulMsWait      =  uTimeout+((uBytes*ulUs)>>10)-ulDiff;
		sWait.tv_sec  =  ulMsWait/1000;
		sWait.tv_usec = (ulMsWait%1000)*1000;

		FD_ZERO(&sRead);
		FD_SET(aComFile[iIndex],&sRead);
															// wait unti a byte was received
			iRet=select(aComFile[iIndex]+1,&sRead,NULL,NULL,&sWait);
		if(!iRet)
			{
			gettimeofday(&sTime,NULL);						// check for timeout

			ulDiff  = (sTime.tv_sec  - sStart.tv_sec )*1000;
			ulDiff += (sTime.tv_usec - sStart.tv_usec)/1000;

			if(ulDiff>=uTimeout+((uBytes*ulUs)>>10))break;
			continue;
			}

 		if(iRet==-1)
			{
			if(uBytes>0)break;
			return -1;
			}
															// read the data from com port
  		   iRet = read(aComFile[iIndex],pData+uBytes,iSize-uBytes);
		if(iRet==-1)
			{
			if(errno==EAGAIN)continue;
			if(uBytes>0)break;
			return -1;
			}

		if(iRet==0)											// no data check for timeout
			{
			gettimeofday(&sTime,NULL);

			ulDiff = (sTime.tv_sec  - sStart.tv_sec )*1000;
			ulDiff+= (sTime.tv_usec - sStart.tv_usec)/1000;

			if(ulDiff>=uTimeout+((uBytes*ulUs)>>10))break;

			sWait.tv_sec  =  0;
			sWait.tv_usec =  20000;
															// wait 20ms
			select(0,NULL,NULL,NULL,&sWait);

			continue;
			}

		uBytes+=iRet;
		}

	#if COM_REMOTE_LOGGING
	if(uBytes>0)RemoteSendSub(hRemote,pBuffer,uBytes,aUsedPort[iIndex],0x7F);
	#endif


return uBytes;
}


//*****************************************************************************
//*
//*		ComReadUntil
//*
//*****************************************************************************
//	Read some chars from a serial COM port, with a timeout and a stop byte.
//	Attention, if some bytes after the stop byte are pressent in the input
//	buffer, this bytes will be received too.
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	pBuffer		: Is a pointer to a puffer for the received chars
//	iSize		: Is the size of the buffer
//	uTimeout	: Is the time out value in ms
//	ucStop		: If this byte is received the function is stoped
//	Returns the count of received chars, or -1 if an error occurs
int  ComReadUntil(int iIndex,void *pBuffer,int iSize,unsigned uTimeout,unsigned char ucStop)
{
struct timeval	sStart,sTime,sWait;
unsigned char  *pData=(unsigned char*)pBuffer;
unsigned char  *pStart;
unsigned 		uBytes;
unsigned long 	ulMsWait;
unsigned long 	ulDiff;
unsigned long 	ulUs;
int 			iRet;
int 			iPos;
fd_set			sRead;



	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return -1;
		}

	if(iSize<=0)
		{
		if(uTimeout)
			{
			sWait.tv_sec  =  uTimeout/1000;
			sWait.tv_usec = (uTimeout%1000)*1000;
	
			select(0,NULL,NULL,NULL,&sWait);
			}
			
		return 0;
		}
	
	ulUs = aUsPerByte[iIndex];

	gettimeofday(&sStart,NULL);

	for(ulDiff=0,uBytes=0;uBytes<(unsigned)iSize;)
		{
		ulMsWait      =  uTimeout+((uBytes*ulUs)>>10)-ulDiff;
		sWait.tv_sec  =  ulMsWait/1000;
		sWait.tv_usec = (ulMsWait%1000)*1000;

		FD_ZERO(&sRead);
		FD_SET(aComFile[iIndex],&sRead);
															// wait unti a byte was received
			iRet=select(aComFile[iIndex]+1,&sRead,NULL,NULL,&sWait);
		if(!iRet)
			{
			gettimeofday(&sTime,NULL);						// check for timeout

			ulDiff = (sTime.tv_sec  - sStart.tv_sec )*1000;
			ulDiff+= (sTime.tv_usec - sStart.tv_usec)/1000;

			if(ulDiff>=uTimeout+((uBytes*ulUs)>>10))break;
			continue;
			}

 		if(iRet==-1)
			{
			if(uBytes>0)break;
			return -1;
			}
															// read the data from com port
  		   iRet = read(aComFile[iIndex],pData+uBytes,iSize-uBytes);
		if(iRet==-1)
			{
			if(errno==EAGAIN)continue;
			if(uBytes>0)break;
			return -1;
			}

		if(iRet==0)											// no data check for timeout
			{
			gettimeofday(&sTime,NULL);

			ulDiff = (sTime.tv_sec  - sStart.tv_sec )*1000;
			ulDiff+= (sTime.tv_usec - sStart.tv_usec)/1000;

			if(ulDiff>=uTimeout+((uBytes*ulUs)>>10))break;

			sWait.tv_sec  =  0;
			sWait.tv_usec =  20000;
															// wait 20ms
			select(0,NULL,NULL,NULL,&sWait);

			continue;
			}


		pStart = pData+uBytes;

		for(iPos=0;iPos<iRet;iPos++)						// is a stop byte in the received data
			{
			if(pStart[iPos]==ucStop)return uBytes+iRet;
			}

		uBytes+=iRet;
		}

	#if COM_REMOTE_LOGGING
	if(uBytes>0)RemoteSendSub(hRemote,pBuffer,uBytes,aUsedPort[iIndex],0x7F);
	#endif


return uBytes;
}


//*****************************************************************************
//*
//*		ComGetByteSendTime
//*
//*****************************************************************************
//	Returns the time which is needed to send one byte in us
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	Returns the time to send one byte or -1 on error
int	ComGetByteSendTime(int iHandle)
{

	if((unsigned)iHandle>=MAX_COM_PORTS*2 || !aIsOpen[iHandle])
		{
		return -1;
		}

return aUsPerByte[iHandle];
}


//*****************************************************************************
//*
//*		ComChangeMode
//*
//*****************************************************************************
//	Changes the mode of a com port
//	iIndex		: Is the handle of the com port (look at ComOpen)
//	iBaud		: Is the baudrate (-1 means no change)
//	iParity		: Is the parity mode (-1 means no change)
//					0x00 = no parity bit	(PARITY__NONE)
//					0x01 = odd parity		(PARITY__ODD)
//					0x02 = even parity		(PARITY__EVEN)
//					0x03 = mark with 0		(PARITY__ZERO)
//					0x04 = mark with 1		(PARITY__ONE)
//	iStopbits	: Selects the count of stop bits (-1 means no change)
//					0x00 = one stopbit		(STOPBITS_ONE)
//					0x01 = two stopbits		(STOPBITS_TWO)
//	iDataBits	: Selects the count of data bits (-1 means no change)
//					0x00 = 8 databits
//					0x01 = 7 databits
//					0x07 = 7 databits		(DATABITS_SEVEN)
//					0x08 = 8 databits		(DATABITS_EIGHT)
//	iFlags		: Some flags (-1 means no change)
//					0x04 = RTS_ON			: switches on the RTS line
//				    0x08 = DTR_ON			: switches on the DTR line
//				  	0x10 = RTS_HANDSHAKE	: switches on the RDS handshake
//				  	0x20 = DTR_HANDSHAKE	: switches on the DTR handshake
//				  	0x40 = RS485_ON			: Switches on the RS485 option (not used in windows)
//	Returns 0 if ok or -1 if an error occurs
int ComChangeMode(int iIndex,int iBaud,int iParity,int iStopbits,int iDatabits,int iFlags)
{
struct serial_struct	sData;
struct termios			sPortInfo;
int 					iBitsPerByte;
int 					iHandle;
int						iBaudNr;
int 	 				iValue;
int 					i;




	if((unsigned)iIndex>=MAX_COM_PORTS*2 || !aIsOpen[iIndex])
		{
		return -1;
		}

	iHandle = aComFile[iIndex];
	tcgetattr(iHandle, &sPortInfo);
	cfmakeraw(&sPortInfo);
	sPortInfo.c_cc[VMIN ]= 0;
	sPortInfo.c_cc[VTIME]= 0;


	if(iFlags>=0)
		{
		if(iFlags&RS485_ON)
			{
			iFlags &= ~(DTR_HANDSHAKE|RTS_HANDSHAKE);
			}

		if(iFlags&DTR_HANDSHAKE){iFlags&=~DTR_ON;}
		if(iFlags&RTS_HANDSHAKE){iFlags&=~RTS_ON;}

		if(iFlags&RTS_HANDSHAKE)							// set RTS handshake
				sPortInfo.c_cflag |=  CRTSCTS;
		else	sPortInfo.c_cflag &= ~CRTSCTS;
		}

	if(iStopbits<0)											// set stop bits
		{
		iStopbits = (sPortInfo.c_cflag&CSTOPB)? 1:0;
		}
	else{
		if(iStopbits!=STOPBITS_ONE)
	    		sPortInfo.c_cflag |=  CSTOPB;
	    else	sPortInfo.c_cflag &= ~CSTOPB;
		}

	if(iDatabits<0)											// set data bits
		{
		iDatabits = ((sPortInfo.c_cflag&CSIZE)==CS7)? 1:0;
		}
	else{
		sPortInfo.c_cflag &= ~CSIZE;
		sPortInfo.c_cflag |= (iDatabits&1)? CS7:CS8;
		}

	if(iParity<0)											// set parity bits
		{
		iParity	= (sPortInfo.c_cflag&PARENB)? 1:0;
		}
	else{
		switch(iParity)
			{
		case PARITY__NONE:  sPortInfo.c_cflag =  sPortInfo.c_cflag & ~PARENB;				break;
		case PARITY__ODD :	sPortInfo.c_cflag =  sPortInfo.c_cflag |  PARENB  |  PARODD;	break;
		case PARITY__EVEN:	sPortInfo.c_cflag = (sPortInfo.c_cflag |  PARENB) & ~PARODD;	break;
			}
		}

	tcsetattr(iHandle,TCSANOW,&sPortInfo);

	if(iBaud<0)												// change baudrate
		{
		iBaud = aComBaud[iIndex];
		}
	else{
		for(i=0;i<MAX_SPEED;i++)
			{
			if(aTcSpeeds[i].iBaud==iBaud)break;
			}

		if(i>=MAX_SPEED)
			{
			errno=-EFAULT;
			return -1;
			}

		ioctl(iHandle,TIOCGSERIAL,&sData);					// remove special baudrates
		sData.flags &= ~ASYNC_SPD_MASK;
		ioctl(iHandle,TIOCSSERIAL,&sData);

		iBaudNr=aTcSpeeds[i].iCode;
	    tcgetattr(iHandle,&sPortInfo);
	    cfsetospeed(&sPortInfo, iBaudNr);
	    cfsetispeed(&sPortInfo, iBaudNr);
	    tcsetattr(iHandle,TCSANOW,&sPortInfo);
		}

	if(iFlags>=0)
		{
		i  = (iFlags&RTS_ON)? 0:TIOCM_RTS;					// turn off RTS DTR
		i |= (iFlags&DTR_ON)? 0:TIOCM_DTR;

		ioctl(iHandle,TIOCMBIC,&i);

		i  = (iFlags&RTS_ON)? TIOCM_RTS:0;					// turn on RTS DTR
		i |= (iFlags&DTR_ON)? TIOCM_DTR:0;

		ioctl(iHandle,TIOCMBIS,&i);
		}


	               iBitsPerByte=10;							// calculate time for one byte
	if(iDatabits&1)iBitsPerByte--;
	if(iStopbits  )iBitsPerByte++;
	if(iParity!=0 )iBitsPerByte++;

	if(iBaud<=115200)
			iValue =  (iBaud/300);
	else	iValue = ((iBaud-115200)/19200)+0x180;

	aUsPerByte[iIndex]  = (1000*1024*iBitsPerByte)/iBaud;
	aComFile  [iIndex]  = iHandle;
	aComBaud  [iIndex]  = iBaud;
	aIsOpen   [iIndex]  = 1;
	aUsedPort [iIndex] |= (iDatabits&1);
	aUsedPort [iIndex] |= (iStopbits)? 2:0;
	aUsedPort [iIndex] |= (iParity<<2);
	aUsedPort [iIndex] |=  iValue <<5;


return 0;
}
