//*****************************************************************************
//*
//*
//*		ComUtilityRt.c
//*
//*
//*****************************************************************************
#ifdef		GLOBAL_CONFIG
#define 	COMUTILITYRT_C
#include	<GlobalConfig.h>
#undef		COMUTILITYRT_C
#endif

#include	<rttarget.h>
#include 	<rtk32.h>
#include 	<clock.h>
#include 	<rtcom.h>
#include 	<rttextio.h>
#include 	<memory.h>
#include 	"ComUtility.h"

#ifndef		SWAP_COM
#define 	SWAP_COM			0
#endif
#ifndef		DEBUG_PORT
#define 	DEBUG_PORT			99
#endif
#ifndef		MAX_COM_PORTS
#define 	MAX_COM_PORTS		10
#endif
#ifndef		COM_BUFFER_SIZE
#define 	COM_BUFFER_SIZE		1024
#endif


static	unsigned char	ucIrqNr    [MAX_COM_PORTS]={     3,    4,    5,    6,    7,    9,   11,   11,   15,   15 };
static	unsigned int	uiPortAdr  [MAX_COM_PORTS]={ 0x2F8,0x3F8,0x3E8,0x2E8,0x3A8,0x2A8,0x180,0x188,0x190,0x198 };
static	unsigned char	ucIsOpen   [MAX_COM_PORTS];
static	unsigned char	ucUseRs485 [MAX_COM_PORTS];
static	unsigned int	uiUsPerByte[MAX_COM_PORTS];
static	unsigned int	uiTkPerByte[MAX_COM_PORTS];
static	unsigned int	uiSendEnd  [MAX_COM_PORTS];
static	unsigned int	uiWaitMax  [MAX_COM_PORTS];
static	unsigned char	ucCheckCom [MAX_COM_PORTS];	// 0=not checked  1=don't exists  2=exists


typedef struct
	{
	unsigned	uNr1;
	unsigned	uNr2;
	}SwapList;



#if	SWAP_COM
SwapList	sSwap[]=
	{
	1,3,
//	6,7,
	};
#endif

//*****************************************************************************
//*
//*		ComInit
//*
//*****************************************************************************
//	Initialize all COM Ports
static void ComInit()
{
int		i;
char	iIrqSet[256];


	memset(iIrqSet,0,sizeof(iIrqSet));

	#if _DEBUG
	for(i=0;i<MAX_COM_PORTS;i++)					// disable the used download board
		{
		int j;

		if(i!=DEBUG_PORT)continue;

		for(j=0;j<MAX_COM_PORTS;j++)				// disable ports with the same IRQ
			{
			if(j==i)continue;
			if(ucIrqNr[i]!=ucIrqNr[j])continue;
			ucIrqNr  [j] = 0;
			uiPortAdr[j] = 0;
			}

		ucIrqNr  [i] = 0;
		uiPortAdr[i] = 0;
		}
	#endif


	if(MAX_COM_PORTS>6)
		{
		COMSetBoardType(COM_BOARD_GENERIC,MAX_COM_PORTS-6,ucIrqNr[6],6);
		}

	for(i=0;i<MAX_COM_PORTS;i++)					// set IRQ stack to 1024 byte
		{
		if(!ucIrqNr[i])continue;
		if(iIrqSet[ucIrqNr[i]])continue;

		if(RTKConfig.DefaultIntStackSize>=1024)continue;
		RTKSetIRQStack(ucIrqNr[i],2048);
		iIrqSet[ucIrqNr[i]]=1;
		}

	for(i=0;i<MAX_COM_PORTS;i++)					// init ports
		{
		if(!uiPortAdr[i])continue;
		COMSetIOBase    (i,uiPortAdr[i]);
		if(i<6)COMSetIRQ(i,ucIrqNr  [i]);
		COMModemControl (i,0,COM_RTS|COM_DTR);
		COMAllocateBuffers(i,2048,2048);
		}


}



//*****************************************************************************
//*
//*		ComOpen
//*
//*****************************************************************************
//	Opens a serial COM port
//	uNr			: Is the number of the port (0=COM1 1=COM2 ...)
//	iBaud		: Is the baudrate
//	iParity		: 0 = no parity bit		(PARITY__NONE)
//				  1 = odd parity		(PARITY__ODD)
//				  2 = even parity		(PARITY__EVEN)
//				  3	= mark with 0		(PARITY__ZERO)
//				  4 = mark with 1		(PARITY__ONE)
//	iStopbits	: 0 = one stopbit
//				  1 = two stopbits
//	iDataBits	: 0 = 8 databits
//				  1 = 7 databits
//				  7 = 7 databits
//				  8 = 8 databits
//	iFlags		: Some Flags
//				  1 = RS485_ON		: Switshes on the RS485 option
//				  2 = DTR_HANDSHAKE	: Switshes on the DTR handshake
//				  4 = RTS_HANDSHAKE	: Switshes on the RDS handshake
//	Returns a handle for the port or -1 if an error occurs
int ComOpen(unsigned uNr,int iBaud,int iParity,int iStopbits,int iDatabits,int iFlags)
{
static int const 	aPatityMode[8]={PARITY_NONE,PARITY_ODD,PARITY_EVEN,PARITY_SPACE,PARITY_MARK,PARITY_NONE,PARITY_NONE,PARITY_NONE};
static int			iIsInit=0;
unsigned char		ucByte,ucBits;
int					iProtocol=COM_PROT_NONE;
int					iBitsPerByte;

	#if	SWAP_COM
	{
	int i;
	for(i=0;i<sizeof(sSwap)/sizeof(sSwap[0]);i++)
		{
		if(uNr==sSwap[i].uNr1-1){uNr=sSwap[i].uNr2-1;break;}
		if(uNr==sSwap[i].uNr2-1){uNr=sSwap[i].uNr1-1;break;}
		};
	}
	#endif

	if(!iIsInit)
		{
		ComInit();
		iIsInit=1;
		}

	if(uNr>=MAX_COM_PORTS)return -1;
	if(uiPortAdr[uNr]==0 )return -1;
	if(ucIsOpen [uNr]    )return -1;

	if(ucCheckCom[uNr]==0)								// check if port exists
		{
		ucCheckCom[uNr]=2;
		ucByte=RTIn(uiPortAdr[uNr]+3);
		RTOut(uiPortAdr[uNr]+3,0xAA);
		if((RTIn(uiPortAdr[uNr]+3)&0xBF)!=0xAA)ucCheckCom[uNr]=1;
		RTOut(uiPortAdr[uNr]+3,0x15);
		if((RTIn(uiPortAdr[uNr]+3)&0xBF)!=0x15)ucCheckCom[uNr]=1;
		RTOut(uiPortAdr[uNr]+3,ucByte);
		}

	if(ucCheckCom[uNr]==1)								// didn't exist com port
		{
		return -1;
		}

	if(iFlags&RS485_ON)
		{
		iFlags &= ~(DTR_HANDSHAKE|RTS_HANDSHAKE);
		}

	if(iFlags&2)iProtocol=COM_PROT_DTR_DSR;
	if(iFlags&4)iProtocol=COM_PROT_RTS_CTS;


	COMPortInit(uNr,iBaud,aPatityMode[iParity&7],(iStopbits)? 2:1,(iDatabits&1)? 7:8);
	COMSetProtocol(uNr,iProtocol,0,0);
	if(uNr>=6)COMSetBoardType(COM_BOARD_GENERIC,MAX_COM_PORTS-6,ucIrqNr[uNr],6);
	COMEnableInterrupt(uNr,1024);
	COMModemControl(uNr,0,COM_RTS|COM_DTR);


	if(iFlags&RS485_ON)									// enable "Auto RS485 Control" on XR16L2750
		{
		ucByte=RTIn(uiPortAdr[uNr]+3);
		RTOut(uiPortAdr[uNr]+3,0xBF);
		ucBits=RTIn(uiPortAdr[uNr]+1)|0x08;
		RTOut(uiPortAdr[uNr]+1,ucBits);
		RTOut(uiPortAdr[uNr]+3,ucByte);
		COMModemControl(uNr,1,COM_RTS);
		ucUseRs485[uNr]=1;
		}
	else{
		ucByte=RTIn(uiPortAdr[uNr]+3);					// disable "Auto RS485 Control" on XR16L2750
		RTOut(uiPortAdr[uNr]+3,0xBF);
		ucBits=RTIn(uiPortAdr[uNr]+1)&~0x08;
		RTOut(uiPortAdr[uNr]+1,ucBits);
		RTOut(uiPortAdr[uNr]+3,ucByte);

		ucUseRs485[uNr]=0;
		}

	RTKClearMailbox(COMReceiveBuffer[uNr]);
	RTKClearMailbox(COMSendBuffer   [uNr]);

	if(iFlags&RTS_ON)
			COMModemControl(uNr,1,COM_RTS);
	else	COMModemControl(uNr,0,COM_RTS);


	if(iFlags&DTR_ON)
			COMModemControl(uNr,1,COM_DTR);
	else	COMModemControl(uNr,0,COM_DTR);



	               iBitsPerByte=10;						// calculate time for one byte
	if(iDatabits&1)iBitsPerByte--;
	if(iStopbits  )iBitsPerByte++;
	if(iParity!=0 )iBitsPerByte++;


	ucIsOpen   [uNr] = 1;
	uiUsPerByte[uNr] = (1000*1024*iBitsPerByte)/iBaud;
	uiTkPerByte[uNr] = CLKMicroSecsToTicks(((1000*1024*1000)/iBaud)*iBitsPerByte);
	uiWaitMax  [uNr] = (uiTkPerByte[uNr]*512)/1024;
	uiSendEnd  [uNr] = RTKGetTime();



return uNr;
}


//*****************************************************************************
//*
//*		ComClose
//*
//*****************************************************************************
//	Closes a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns 0 if all ok, or -1 if an error occur
int	ComClose(int iHandle)
{

	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!ucIsOpen[iHandle])return -1;

	RTKClearMailbox(COMReceiveBuffer[iHandle]);
	ucIsOpen[iHandle]=0;

return 0;
}


//*****************************************************************************
//*
//*		ComRead
//*
//*****************************************************************************
//	Read some chars from a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	pBuffer	: Is a pointer to a puffer for the received chars
//	iSize	: Is the size of the buffer
//	Returns the count of received chars, or -1 if an error occurs
int ComRead(int iHandle,void *pBuffer,int iSize)
{
COMData			uData;
unsigned char  *pPtr=(unsigned char*)pBuffer;
int				iPos;


	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!ucIsOpen[iHandle])return -1;
	if(iSize<=0)return 0;

	for(iPos=0;iPos<iSize;iPos++)
		{
		if(!RTKGetCond(COMReceiveBuffer[iHandle],&uData))break;
		pPtr[iPos]=(unsigned char)uData;
		}


return iPos;
}


//*****************************************************************************
//*
//*		ComWrite
//*
//*****************************************************************************
//	Sends some chars on a serial port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	pBuffer	: Is a pointer to a puffer with the chars
//	iSize	: Is the size of the buffer
//	Returns the count of sended chars, or -1 if an error occurs
int	ComWrite(int iHandle,const void *pBuffer,int iSize)
{
unsigned	uTime;
unsigned	uWait;
unsigned	uRet;
unsigned 	uDif;


	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!ucIsOpen[iHandle])return -1;
	if(iSize<=0)return 0;

	uRet  = COMSendBlockTimed(iHandle,(void*)pBuffer,iSize,uiWaitMax[iHandle]);
	uTime = RTKGetTime();
	uWait = (uiTkPerByte[iHandle]*uRet+1023)/1024;
	uDif  = uiSendEnd[iHandle]-uTime;

	if(uDif>uiWaitMax[iHandle])								// Some bytes are in the buffer
		{
		uiSendEnd[iHandle]  = uWait+uTime;
		}
	else{													// All bytes sended
		uiSendEnd[iHandle] += uWait;
		}

	if(uRet!=(unsigned)iSize)uWait/= 2;
	if(uRet==        0)uWait = 1;

	COMWaitSendBufferEmpty(iHandle,uWait);


return uRet;
}

//*****************************************************************************
//*
//*		ComGetReadCount
//*
//*****************************************************************************
//	Gets the count of chars in the read buffer of a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns the count of chars in the read buffer
int ComGetReadCount(int iHandle)
{

	if((unsigned)iHandle>=MAX_COM_PORTS)return 0;
	if(!ucIsOpen[iHandle])return 0;

return RTKMessages(COMReceiveBuffer[iHandle]);
}

//*****************************************************************************
//*
//*		ComGetWriteCount
//*
//*****************************************************************************
//	Gets the count of chars in the read buffer of a serial COM port
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returns the count of chars in the write buffer
int ComGetWriteCount(int iHandle)
{

	if((unsigned)iHandle>=MAX_COM_PORTS)return 0;
	if(!ucIsOpen[iHandle])return 0;

return RTKMessages(COMSendBuffer[iHandle]);
}

//******************************************************************************
//*
//*		ComGetState
//*
//******************************************************************************
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	Returtns the state of the of the CTS and DSR line (CTS_ON=1,DSR_ON=2)
int	ComGetState(int iHandle)
{
int		iFalgs,iBits=0;



	if( (unsigned) iHandle>=MAX_COM_PORTS)return 0;
	if(!ucIsOpen  [iHandle])return 0;
	if( ucUseRs485[iHandle])return DSR_ON|CTS_ON;

	iFalgs=COMModemStatus(iHandle);

	if(iFalgs&COM_DSR)iBits|=DSR_ON;
	if(iFalgs&COM_CTS)iBits|=CTS_ON;


return iBits;
}

//******************************************************************************
//*
//*		ComSetState
//*
//******************************************************************************
//	Set the state of the RTS and DTR lines
//	iHandle	: Is the handle of the com port (look at ComOpen)
//	iBits	: Is the new state off the lines  (RTS_ON,DTR_ON)
//	iMask 	: Which lines should change       (RTS_ON,DTR_ON)
void ComSetState(int iHandle,int iBits,int iMask)
{



	if((unsigned)iHandle>=MAX_COM_PORTS)return;
	if(!ucIsOpen[iHandle])return;

	if(iMask&RTS_ON)
		{
		if(iBits&RTS_ON)
				COMModemControl(iHandle,1,COM_RTS);
		else	COMModemControl(iHandle,0,COM_RTS);
		}

	if(iMask&DTR_ON)
		{
		if(iBits&DTR_ON)
				COMModemControl(iHandle,1,COM_DTR);
		else	COMModemControl(iHandle,0,COM_DTR);
		}


}


//*****************************************************************************
//*
//*		ComWaitUntilSend
//*
//*****************************************************************************
//	Waits until all bytes in the send queue are transmitted
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	uTimeout	: Is the timeout in miliseconts
//	Returns 1 if all bytes are sended
int ComWaitUntilSend(int iHandle,unsigned uTimeout)
{
unsigned 	uDelay;
unsigned	uStop;
unsigned	uTime;
unsigned	uMax;
unsigned	uEnd;
int			iDif;



	if((unsigned)iHandle>=MAX_COM_PORTS)return 0;
	if(!ucIsOpen[iHandle])return 0;


	uMax   = CLKMilliSecsToTicks(uTimeout);
	uEnd   = RTKGetTime()+uMax;


	if(!COMWaitSendBufferEmpty(iHandle,uMax))			// Wain until all byte transmitted to the UART
		{
		return 0;
		}



	uTime  = RTKGetTime();
	uStop  = uiSendEnd[iHandle]-uTime;


	if(uStop<uiWaitMax[iHandle])						// Before calculated end
		{
		iDif   = uEnd-uTime;

		if(iDif<=0 || (unsigned)iDif<uStop)				// Timeout
			{
			RTKDelayUntil(uEnd);
			return 0;
			}

		RTKDelayUntil(uiSendEnd[iHandle]);
		}


		uDelay = CLKMilliSecsToTicks(3);
	if(!uDelay)uDelay=1;


	while(!(COMLineStatus(iHandle)&COM_TX_SHIFT_EMPTY))	// Wait until send register empty
		{
		uTime  = RTKGetTime();
		iDif   = uEnd-uTime;

		if(iDif<=0)return 0;
		if(iDif<(int)uDelay)uDelay=iDif;

		RTKDelay(uDelay);
		}





return 1;
}

//*****************************************************************************
//*
//*		ComWaitUntilResv
//*
//*****************************************************************************
//	Waits until a byte was reveived
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	uTimeout	: Is the timeout in miliseconts
//	Returns 1 if a byet was received
int	ComWaitUntilResv(int iHandle,unsigned uTimeout)
{
COMData			uData;
int				iDelta;


	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!ucIsOpen[iHandle])return -1;


	iDelta = CLKMilliSecsToTicks(uTimeout);


	if(!RTKGetTimed(COMReceiveBuffer[iHandle],&uData,iDelta))
		{
		return 0;
		}


	RTKPutFrontTimed(COMReceiveBuffer[iHandle],&uData,CLKMilliSecsToTicks(10));


return 1;
}


//*****************************************************************************
//*
//*		ComReadTimeout
//*
//*****************************************************************************
//	Read some chars from a serial COM port , with a timeout
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	pBuffer		: Is a pointer to a puffer for the received chars
//	iSize		: Is the size of the buffer
//	uTimeout	: Is the time out value in ms
//	Returns the count of received chars, or -1 if an error occurs
int  ComReadTimeout(int iHandle,void *pBuffer,int iSize,unsigned uTimeout)
{
COMData			uData;
unsigned char  *pPtr=(unsigned char*)pBuffer;
unsigned	 	uStop;
int				iDelta;
int 			iWait;
int				iPos;


	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!ucIsOpen[iHandle])return -1;

	   iWait = CLKMilliSecsToTicks(uTimeout);
	if(iWait==0 && uTimeout>0)
		{
		iWait = 1;
		}

	if(iSize<=0)
		{
		if(uTimeout)RTKDelay(iWait);
		return 0;
		}

	if(uTimeout==0)									// read all bytes which are in the buffer
		{
		for(iPos=0;iPos<iSize;iPos++)
			{
			if(!RTKGetCond(COMReceiveBuffer[iHandle],&uData))break;
			pPtr[iPos]=(unsigned char)uData;
			}

		return iPos;
		}


	uStop = RTKGetTime()+iWait;

	for(iPos=0;iPos<iSize;iPos++)					// read bytes with timeout
		{
		   iDelta = uStop-RTKGetTime();
		   iDelta+= CLKMilliSecsToTicks((iPos*uiUsPerByte[iHandle])>>10);
		if(iDelta<=0)break;

		if(!RTKGetTimed(COMReceiveBuffer[iHandle],&uData,iDelta))break;
		pPtr[iPos]=(unsigned char)uData;
		}


return iPos;
}



//*****************************************************************************
//*
//*		ComReadUntil
//*
//*****************************************************************************
//	Read some chars from a serial COM port, with a timeout and a stop byte
//	iHandle		: Is the handle of the com port (look at ComOpen)
//	pBuffer		: Is a pointer to a puffer for the received chars
//	iSize		: Is the size of the buffer
//	uTimeout	: Is the time out value in ms
//	ucStop		: If this byte is received the function is stoped
//	Returns the count of received chars, or -1 if an error occurs
int  ComReadUntil(int iHandle,void *pBuffer,int iSize,unsigned uTimeout,unsigned char ucStop)
{
COMData			uData;
unsigned char  *pPtr=(unsigned char*)pBuffer;
unsigned	 	uEnd;
int 			iWait;
int				iPos;



	if((unsigned)iHandle>=MAX_COM_PORTS)return -1;
	if(!ucIsOpen[iHandle])return -1;				// is the port open

	   iWait = CLKMilliSecsToTicks(uTimeout);
	if(iWait==0 && uTimeout>0)
		{
		iWait = 1;
		}

	if(iSize<=0)
		{
		if(uTimeout)RTKDelay(iWait);
		return 0;
		}


	uEnd = RTKGetTime()+iWait;


	for(iPos=0;iPos<iSize;iPos++)					// read bytes with timeout
		{
		if(!RTKGetTimed(COMReceiveBuffer[iHandle],&uData,iWait))
			{
			break;
			}

		pPtr[iPos]=(unsigned char)uData;

		if(((unsigned char)uData)==ucStop)			// is it the stop byte
			{
			iPos++;
			break;
			}

		   iWait = uEnd-RTKGetTime();
		   iWait+= (uiUsPerByte[iHandle]*iPos)/1024;
		if(iWait<0)break;
		}


return iPos;
}
