//*****************************************************************************
//*
//*
//*		EventUtility.c
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//





//*****************************************************************************
//*
//*
//*		WINDOWS
//*
//*
//*****************************************************************************
#if (defined(_WIN32) || defined(_WIN64)) && !defined(RTTARGET)
#include	<stdio.h>
#include	<string.h>
#include	<windows.h>
#include	"EventUtility.h"


#if        _MSC_VER >= 1000
#pragma		warning( disable :4035 )
#pragma		warning( disable :4996 )
#pragma		warning( disable :4514 )
#endif

//*****************************************************************************
//*
//*		EventOpen
//*
//*****************************************************************************
//	Opens an existing global puls event
//	iKey	: is the name of the event, 0 isn't allowed
//	Returns a handle to an event or NULL if an error occurs
Event EventOpen(int iKey)
{
char	cName[32];
HANDLE	hEvent;


	if(!iKey)return NULL;

	sprintf(cName,"Global\\EVE_%08X",iKey);
	hEvent = OpenEventA(SEMAPHORE_ALL_ACCESS,FALSE,cName);


return (Event)hEvent;
}

//*****************************************************************************
//*
//*		EventOpenRS
//*
//*****************************************************************************
//	Opens an existing global set/reset event
//	iKey	: is the name of the event, 0 isn't allowed
//	Returns a handle to an event or NULL if an error occurs
Event EventOpenRS(int iKey)
{
char	cName[32];
HANDLE	hEvent;


	if(!iKey)return NULL;

	sprintf(cName,"Global\\ERS_%08X",iKey);
	hEvent = OpenEventA(SEMAPHORE_ALL_ACCESS,FALSE,cName);


return (Event)hEvent;
}

//*****************************************************************************
//*
//*		EventCreate
//*
//*****************************************************************************
//	Creates a new pulse event with the init value of 0
//	iKey	: is the name of the event, 0 is for a local event
//	Returns a handle to an event or NULL if an error occurs
Event EventCreate(int iKey)
{
SECURITY_ATTRIBUTES sAttr;
char				sDesc[SECURITY_DESCRIPTOR_MIN_LENGTH];
char				cName[32];
HANDLE				hEvent;


	if(!iKey)												// create local  event
		{
		hEvent = CreateEventA(NULL,TRUE,0,NULL);
		return (Event)hEvent;
		}

	sprintf(cName,"Global\\EVE_%08X",iKey);

	   hEvent = OpenEventA(EVENT_ALL_ACCESS,FALSE,cName);
	if(hEvent)												// the event allready exists
		{
		CloseHandle(hEvent);
		return NULL;
		}

	sAttr.nLength				= sizeof(sAttr);			// create security descriptor
	sAttr.bInheritHandle		= FALSE;
	sAttr.lpSecurityDescriptor	= &sDesc;
	InitializeSecurityDescriptor(sAttr.lpSecurityDescriptor,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl   (sAttr.lpSecurityDescriptor,TRUE,NULL,FALSE);

	hEvent = CreateEventA(&sAttr,TRUE,0,cName);				// create global event



return (Event)hEvent;
}

//*****************************************************************************
//*
//*		EventCreateRS
//*
//*****************************************************************************
//	Creates a new set/reset event with the init value of 0
//	iKey	: is the name of the event, 0 is for a local event
//	Returns a handle to an event or NULL if an error occurs
Event EventCreateRS(int iKey)
{
SECURITY_ATTRIBUTES sAttr;
char				sDesc[SECURITY_DESCRIPTOR_MIN_LENGTH];
char				cName[32];
HANDLE				hEvent;


	if(!iKey)												// create local  event
		{
		hEvent = CreateEventA(NULL,TRUE,0,NULL);
		return (Event)hEvent;
		}

	sprintf(cName,"Global\\ERS_%08X",iKey);

	   hEvent = OpenEventA(EVENT_ALL_ACCESS,FALSE,cName);
	if(hEvent)												// the event allready exists
		{
		CloseHandle(hEvent);
		return NULL;
		}

	sAttr.nLength				= sizeof(sAttr);			// create security descriptor
	sAttr.bInheritHandle		= FALSE;
	sAttr.lpSecurityDescriptor	= &sDesc;
	InitializeSecurityDescriptor(sAttr.lpSecurityDescriptor,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl   (sAttr.lpSecurityDescriptor,TRUE,NULL,FALSE);

	hEvent = CreateEventA(&sAttr,TRUE,0,cName);				// create global event



return (Event)hEvent;
}


//*****************************************************************************
//*
//*		EventClose
//*
//*****************************************************************************
//	Closes an event which was opened with EventOpen
//	hEvent	: ist the handle of the event
//	Return 1 if the event was closed or 0 if an error occurs
int EventClose(Event hEvent)
{

return CloseHandle((HANDLE)hEvent);
}


#endif

//*****************************************************************************
//*
//*
//*		LINUX
//*
//*
//*****************************************************************************
//
//	Note:	Add -lpthread to your makefile
//

#ifdef 	linux
#ifndef	LINUX
#define LINUX	1
#endif
#endif


#ifdef	LINUX

#include 	<errno.h>
#include	<stdio.h>
#include 	<unistd.h>
#include 	<sys/ipc.h>
#include 	<sys/sem.h>
#include	<sys/stat.h>
#include 	<sys/time.h>
#include 	<sys/types.h>
#include	"EventUtility.h"


extern	int semtimedop(int,struct sembuf*,unsigned,struct timespec*);

//******************************************************************************
//*
//*		EventClean
//*
//******************************************************************************
//	Cleans all unused events
//	Returns the count of closed events
static int EventClean()
{
char 			 cPath[32];
struct ipc_perm *pIpc;
struct semid_ds  sData;
struct seminfo	 sInfo;
struct stat   	 sStat;
int 			 iMaxId;
int 			 iSemId;
int 			 iPid;
int 			 iPos;
int  			 iRet;
int 			 iId;



	   iMaxId = semctl(0,0,SEM_INFO,&sInfo);
	if(iMaxId<0)
		{
		return 0;
		}


	iRet = 0;

	for(iId=0;iId<=iMaxId;iId++)							// loop over all semaphores
		{
		   iSemId = semctl(iId,0,SEM_STAT,&sData);
		if(iSemId<0)
			{
			continue;
			}

		pIpc = &sData.sem_perm;

		if(sData.sem_nsems!=2 /*|| pIpc->__key!=0*/)		// only private events
			{
			continue;
			}

		for(iPos=0;iPos<sData.sem_nsems;iPos++)				// check the processes
			{
			   iPid  = semctl(iSemId,iPos,GETPID,&sData);
			if(iPid<0)break;

			sprintf(cPath,"/proc/%i",iPid);

			if(stat(cPath,&sStat)!=-1)
				{
				break;
				}
			}

		if(iPos>=sData.sem_nsems)
			{
			if(semctl(iSemId,0,IPC_RMID,0)!=-1)				// close event
				{
				iRet++;
				}
			}
		}


return iRet;
}

//*****************************************************************************
//*
//*		EventOpen
//*
//*****************************************************************************
//	Opens an existing global puls event
//	iKey	: is the name of the event, 0 isn't allowed
//	Returns a handle to an event or NULL if an error occurs
Event EventOpen(int iKey)
{
struct sembuf	sOp;
int				iSem;



	if(!iKey)return NULL;

	   iSem = semget(iKey,2,0666);
	if(iSem==-1)return NULL;


	sOp.sem_num = 1;										// increment open count
	sOp.sem_op  = 1;
	sOp.sem_flg = SEM_UNDO;

	semop(iSem,&sOp,0);



return (Event)((intptr_t)iSem+1);
}

//*****************************************************************************
//*
//*		EventOpenRS
//*
//*****************************************************************************
//	Opens an existing global set/reset event
//	iKey	: is the name of the event, 0 isn't allowed
//	Returns a handle to an event or NULL if an error occurs
Event EventOpenRS(int iKey)
{
struct sembuf	sOp;
int				iSem;


	if(!iKey)return NULL;

	iKey ^= 0x1234;

	   iSem = semget(iKey,2,0666);
	if(iSem==-1)return NULL;


	sOp.sem_num = 1;										// increment open count
	sOp.sem_op  = 1;
	sOp.sem_flg = SEM_UNDO;

	semop(iSem,&sOp,0);



return (Event)(((intptr_t)iSem^0x40000000)+1);
}

//*****************************************************************************
//*
//*		EventCreate
//*
//*****************************************************************************
//	Creates a new puls event with the init value of 0
//	iKey	: is the name of the event, 0 is for a local event
//	Returns a handle to an event or NULL if an error occurs
Event EventCreate(int iKey)
{
struct sembuf	sOp;
int				iInc;
int 			iSem;



	if(!iKey)												// create a local event
		{
		   iSem = semget(IPC_PRIVATE,2,IPC_CREAT|IPC_EXCL|0666);
		if(iSem==-1)
			{
			if(errno!=ENOSPC || !EventClean())
				{
				return NULL;
				}
															// second try if events are cleaned
			   iSem = semget(IPC_PRIVATE,2,IPC_CREAT|IPC_EXCL|0666);
			if(iSem==-1)
				{
				return NULL;
				}
			}
		}
	else{
		   iSem = semget(iKey,2,IPC_CREAT|IPC_EXCL|0666);
		if(iSem==-1)
			{
			   iSem = semget(iKey,2,0666);					// did the semaphore allready exist ?
			if(iSem==-1)return NULL;

			   iInc = semctl(iSem,1,GETVAL,0);				// if open count is zero the semaphore can be used
			if(iInc!=0)return NULL;
			}
		}


	sOp.sem_num = 1;										// increment open count
	sOp.sem_op  = 1;
	sOp.sem_flg = SEM_UNDO;

	semop(iSem,&sOp,1);


	semctl(iSem,0,SETVAL,0);								// set starting value



return (Event)((intptr_t)iSem+1);
}

//*****************************************************************************
//*
//*		EventCreateRS
//*
//*****************************************************************************
//	Creates a new set/reset event with the init value of 0
//	iKey	: is the name of the event, 0 is for a local event
//	Returns a handle to an event or NULL if an error occurs
Event EventCreateRS(int iKey)
{
struct sembuf	sOp;
int				iInc;
int 			iSem;



	if(!iKey)												// create a local event
		{
		   iSem = semget(IPC_PRIVATE,2,IPC_CREAT|IPC_EXCL|0666);
		if(iSem==-1)
			{
			if(errno!=ENOSPC || !EventClean())
				{
				return NULL;
				}
															// second try if events are cleand
			   iSem = semget(IPC_PRIVATE,2,IPC_CREAT|IPC_EXCL|0666);
			if(iSem==-1)
				{
				return NULL;
				}
			}
		}
	else{
		iKey ^= 0x1234;

		   iSem = semget(iKey,2,IPC_CREAT|IPC_EXCL|0666);
		if(iSem==-1)
			{
			   iSem = semget(iKey,2,0666);					// did the semaphore allready exist ?
			if(iSem==-1)return NULL;

			   iInc = semctl(iSem,1,GETVAL,0);				// if open count is zero the semaphore can be used
			if(iInc!=0)return NULL;
			}
		}


	sOp.sem_num = 1;										// increment open count
	sOp.sem_op  = 1;
	sOp.sem_flg = SEM_UNDO;

	semop(iSem,&sOp,1);


	semctl(iSem,0,SETVAL,0);								// set starting value



return (Event)(((intptr_t)iSem^0x40000000)+1);
}



//*****************************************************************************
//*
//*		EventClose
//*
//*****************************************************************************
//	Closes an event.
//	hEvent	: ist the handle of the event
//	Return 1 if the event was closed or 0 if an error occurs
int EventClose(Event hEvent)
{
struct timespec	sTimeout;
struct sembuf	sOp;
int 			iSem;
int 			iInc;


	if(!hEvent)return 0;

	iSem  = ((intptr_t)hEvent)-1;

	if(iSem&0x40000000)										// is it an set/reset event
		{
		iSem &= ~0x40000000;
		}

	sTimeout.tv_sec	 =  0;									// decrement open count
	sTimeout.tv_nsec =  0;
	sOp.sem_num      =  1;
	sOp.sem_op		 = -1;
	sOp.sem_flg		 = SEM_UNDO;


	if(semtimedop(iSem,&sOp,1,&sTimeout)==-1)
		{
		return 0;
		}

	   iInc = semctl(iSem,1,GETVAL,0);						// if open count is not zero the event can be deleded
	if(iInc>0)return 1;

	if(semctl(iSem,0,IPC_RMID,0)==-1)						// close event
		{
		return 0;
		}



return 1;
}


//*****************************************************************************
//*
//*		EventWait
//*
//*****************************************************************************
//	Waits until the event is not released. With the EventPule Funktion
//	the event could be relesed.
//	hEvent	: is the handle of the event
void EventWait(Event hEvent)
{
struct sembuf	sOp;
int 			iSem;


	if(!hEvent)return;

	iSem = ((intptr_t)hEvent)-1;

	if(iSem&0x40000000)										// is it an set/reset event
		{
		iSem &= ~0x40000000;

		if(semctl(iSem,0,GETVAL)>0)							// is the event in the SET state
			{
			return;
			}
		}

	sOp.sem_num  = 0;
	sOp.sem_op   = -1;
	sOp.sem_flg  = SEM_UNDO;

	semop(iSem,&sOp,1);


}


//*****************************************************************************
//*
//*		EventWaitTimed
//*
//*****************************************************************************
//	Locks an event, the event value is decremented by 1. If the
//  event value is 0 the function waits until the value is above zero.
//	If the event is locked longer than uWaitTime (in ms) the function
//	breaks the waiting.
//	hEvent		: is the handle of the event
//	uWaitTime	: wait time in milliseconds
//	Returns 1 if the evend was happend, or 0 at a timeout
int EventWaitTimed(Event hEvent,unsigned uWaitTime)
{
struct sembuf	sOp;
struct timespec	sTimeout;
int 			iSem;



	if(!hEvent)return 0;

	iSem  = ((intptr_t)hEvent)-1;

	if(iSem&0x40000000)										// is it an set/reset event
		{
		iSem &= ~0x40000000;

		if(semctl(iSem,0,GETVAL)>0)							// is the event in the SET state
			{
			return 0;
			}
		}


	if(uWaitTime==0)										// is no timeout used (only get state)
		{
		sOp.sem_num = 0;
		sOp.sem_op  = -1;
		sOp.sem_flg = SEM_UNDO|IPC_NOWAIT;

		if(semop(iSem,&sOp,1)!=-1)
			{
			return 1;
			}

		return 0;
		}


	sTimeout.tv_sec	 = uWaitTime/1000;
	sTimeout.tv_nsec =(uWaitTime%1000)*1000000;
	sOp.sem_num      = 0;
	sOp.sem_op		 = -1;
	sOp.sem_flg		 = SEM_UNDO;


	if(semtimedop(iSem,&sOp,1,&sTimeout)!=-1)				// wait with timeout
		{
		return 1;
		}



return 0;
}


//*****************************************************************************
//*
//*		EventPulse
//*
//*****************************************************************************
//	Unlocks all waiting threads on an event.
//	hEvent	: is the handle of the event
void EventPulse(Event hEvent)
{
struct sembuf	sOp;
int				iNum;



	if(!hEvent)return;

	   iNum = semctl(((intptr_t)hEvent)-1,0,GETNCNT);		// get count of waiting threads
	if(iNum<=0)
		{
		semctl(((intptr_t)hEvent)-1,0,SETVAL,0);			// set value to zero
		return;
		}

	sOp.sem_num = 0;
	sOp.sem_op  = iNum;
	sOp.sem_flg = IPC_NOWAIT;

	semop(((intptr_t)hEvent)-1,&sOp,1);						// unlock all waiting threads

}


//*****************************************************************************
//*
//*		EventSet
//*
//*****************************************************************************
//	Sets an event, the semaphore value is set to one.
//	hEvent		: is the handle of the event
void EventSet(Event hEvent)
{
struct sembuf	sOp;
int				iNum;
int 			iSem;

	if(!hEvent)return;

	iSem  = ((intptr_t)hEvent)-1;
	iSem ^= 0x40000000;


	   iNum = semctl(iSem,0,GETNCNT);						// get count of waiting threads
	if(iNum<0)iNum=0;

	sOp.sem_num = 0;
	sOp.sem_op  = iNum+0x10000;
	sOp.sem_flg = IPC_NOWAIT;

	semop(iSem,&sOp,1);										// unlock all waiting threads

	semctl(iSem,0,SETVAL,1);								// set value to one

}

//*****************************************************************************
//*
//*		EventReset
//*
//*****************************************************************************
//	Resets an event, the semaphore value is set to zero.
//	hEvent		: is the handle of the event
void EventReset(Event hEvent)
{
int 	iSem;

	if(!hEvent)return;

	iSem  = ((intptr_t)hEvent)-1;
	iSem ^= 0x40000000;

	semctl(iSem,0,SETVAL,0);								// set value to zero

}


#endif



//*****************************************************************************
//*
//*
//*		RTTARGET
//*
//*
//*****************************************************************************
#if defined(RTTARGET)
#include	<rtk32.h>
#include	<clock.h>
#include	<stdio.h>
#include	"EventUtility.h"


//*****************************************************************************
//*
//*		EventOpen
//*
//*****************************************************************************
//	Opens an existing global puls event
//	iKey	: is the name of the event, 0 isn't allowed
//	Returns a handle to an event or NULL if an error occurs
Event EventOpen(int iKey)
{
char			cName[32];
RTKSemaphore	hEvent;


	if(!iKey)return NULL;

	sprintf(cName,"Global\\EVE_%08X",iKey);

	hEvent = RTKOpenSemaphore(ST_EVENT,0,SF_SEARCH|SF_FAIL_NOT_FOUND,cName);


return (Event)hEvent;
}

//*****************************************************************************
//*
//*		EventOpenRS
//*
//*****************************************************************************
//	Opens an existing global set/reset event
//	iKey	: is the name of the event, 0 isn't allowed
//	Returns a handle to an event or NULL if an error occurs
Event EventOpenRS(int iKey)
{
char			cName[32];
RTKSemaphore	hEvent;


	if(!iKey)return NULL;

	sprintf(cName,"Global\\ERS_%08X",iKey);

	hEvent = RTKOpenSemaphore(ST_EVENT,0,SF_SEARCH|SF_FAIL_NOT_FOUND,cName);


return (Event)hEvent;
}

//*****************************************************************************
//*
//*		EventCreate
//*
//*****************************************************************************
//	Creates a new pulse event with the init value of 0
//	iKey	: is the name of the event, 0 is for a local event
//	Returns a handle to an event or NULL if an error occurs
Event EventCreate(int iKey)
{
char				cName[32];
RTKBool				bCreate;
RTKSemaphore		hEvent;


	if(!iKey)												// create local event
		{
		hEvent = RTKOpenSemaphoreEx(ST_EVENT,0,0,NULL,&bCreate);
		return (Event)hEvent;
		}

	sprintf(cName,"Global\\EVE_%08X",iKey);

	   hEvent =RTKOpenSemaphore(ST_EVENT,0,SF_SEARCH|SF_FAIL_NOT_FOUND,cName);
	if(hEvent!=RTK_NO_SEMAPHORE)
		{
		return NULL;
		}

	hEvent = RTKOpenSemaphoreEx(ST_EVENT,0,SF_COPY_NAME,cName,&bCreate);


return (Event)hEvent;
}

//*****************************************************************************
//*
//*		EventCreateRS
//*
//*****************************************************************************
//	Creates a new set/reset event with the init value of 0
//	iKey	: is the name of the event, 0 is for a local event
//	Returns a handle to an event or NULL if an error occurs
Event EventCreateRS(int iKey)
{
char				cName[32];
RTKBool				bCreate;
RTKSemaphore		hEvent;


	if(!iKey)												// create local  event
		{
		hEvent = RTKOpenSemaphoreEx(ST_EVENT,0,0,NULL,&bCreate);
		return (Event)hEvent;
		}

	sprintf(cName,"Global\\ERS_%08X",iKey);

	   hEvent =RTKOpenSemaphore(ST_EVENT,0,SF_SEARCH|SF_FAIL_NOT_FOUND,cName);
	if(hEvent!=RTK_NO_SEMAPHORE)
		{
		return NULL;
		}

	hEvent = RTKOpenSemaphoreEx(ST_EVENT,0,SF_COPY_NAME,cName,&bCreate);



return (Event)hEvent;
}


//*****************************************************************************
//*
//*		EventClose
//*
//*****************************************************************************
//	Closes an event which was opened with EventOpen
//	hEvent	: ist the handle of the event
//	Return 1 if the event was closed or 0 if an error occurs
int EventClose(Event hEvent)
{

	if(!hEvent)return 0;
	RTKDeleteSemaphore((RTKSemaphore*)&hEvent);

return 1;
}


#endif


