//*****************************************************************************
//*
//*
//*		ExchangeUtility.h
//*
//*
//*****************************************************************************
//
//	Module with several atomic exchange functions for X86,X64,ARM
//
//	(C) Copyright Anton Zechner 2015
//
//	void           *ExchangePtr		(ptr,val)
//	unsigned char	ExchangeByte    (ptr,val)
//	unsigned short 	ExchangeWord    (ptr,val)
//	unsigned int 	ExchangeDword   (ptr,val)
//
//	void		   *ExchangeAddPtr  (ptr,val)
//	unsigned char	ExchangeAddByte (ptr,val)
//	unsigned short 	ExchangeAddWord (ptr,val)
//	unsigned int 	ExchangeAddDword(ptr,val)
//
//	void		   *ExchangeSubPtr	(ptr,val)
//	unsigned char	ExchangeSubByte (ptr,val)
//	unsigned short 	ExchangeSubWord (ptr,val)
//	unsigned int 	ExchangeSubDword(ptr,val)
//
//	void		   *ExchangeIncPtr	(ptr)
//	unsigned char	ExchangeIncByte (ptr)
//	unsigned short 	ExchangeIncWord (ptr)
//	unsigned int 	ExchangeIncDword(ptr)
//
//	void		   *ExchangeDecPtr	(ptr)
//	unsigned char	ExchangeDecByte (ptr)
//	unsigned short 	ExchangeDecWord (ptr)
//	unsigned int 	ExchangeDecDword(ptr)
//
#ifndef 	__EXCHANGEUTILITY_H__
#define 	__EXCHANGEUTILITY_H__

#ifdef		GLOBAL_CONFIG
#define 	EXCHANGEUTILITY_H
#include	<GlobalConfig.h>
#undef 		EXCHANGEUTILITY_H
#endif

//*****************************************************************************
//*
//*		Microsoft Compiler
//*
//*****************************************************************************

#ifdef	_MSC_VER

#pragma once

//
// intel/amd 64bit cpu's
//
#if defined(_M_X64) ||  defined(_M_AMD64)

#include	<intrin.h>

#define		ExchangePtr(ptr,val)			((void         *)_InterlockedExchange64((__int64*)(ptr),(__int64)(val)))
#define		ExchangeByte(ptr,val)			((unsigned char )_InterlockedExchange8 ((char   *)(ptr),val))
#define		ExchangeWord(ptr,val)			((unsigned short)_InterlockedExchange16((short  *)(ptr),val))
#define		ExchangeDword(ptr,val)			((unsigned int  )_InterlockedExchange  ((long   *)(ptr),val))

#define		ExchangeAddPtr(ptr,val)			((void         *)_InterlockedExchangeAdd64((__int64*)(ptr),val))
#define		ExchangeAddByte(ptr,val)		((unsigned char )_InterlockedExchangeAdd8 ((char   *)(ptr),val))
#define		ExchangeAddWord(ptr,val)		((unsigned short)_InterlockedExchangeAdd16((short  *)(ptr),val))
#define		ExchangeAddDword(ptr,val)		((unsigned int  )_InterlockedExchangeAdd  ((long   *)(ptr),val))

//
// intel/amd 32bit cpu's with visual sudio 2003...
//
#elif defined(_M_IX86) && _MSC_VER>=1300

#include	<intrin.h>

#define		ExchangePtr(ptr,val)			((void         *)_InterlockedExchange  ((long   *)(ptr),(long)(val)))
#define		ExchangeByte(ptr,val)			((unsigned char )_InterlockedExchange8 ((char   *)(ptr),val))
#define		ExchangeWord(ptr,val)			((unsigned short)_InterlockedExchange16((short  *)(ptr),val))
#define		ExchangeDword(ptr,val)			((unsigned int  )_InterlockedExchange  ((long   *)(ptr),val))

#define		ExchangeAddPtr(ptr,val)			((void         *)_InterlockedExchangeAdd  ((long   *)(ptr),val))
#define		ExchangeAddByte(ptr,val)		((unsigned char )_InterlockedExchangeAdd8 ((char   *)(ptr),val))
#define		ExchangeAddWord(ptr,val)		((unsigned short)_InterlockedExchangeAdd16((short  *)(ptr),val))
#define		ExchangeAddDword(ptr,val)		((unsigned int  )_InterlockedExchangeAdd  ((long   *)(ptr),val))

//
// intel/amd 32bit cpu's with visual sudio 6.0
//
#elif defined(_M_IX86)

#pragma warning(push)
#pragma warning(disable:4035)

static __inline void *__ExchangePtr(void *pPtr,void *pVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	eax,pVal
		lock 	xchg dword ptr [edx],eax
		}
	}

static __inline unsigned char __ExchangeByte(unsigned char *pPtr,unsigned char ucVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	al ,ucVal
		lock 	xchg byte ptr [edx],al
		}
	}

static __inline unsigned short __ExchangeWord(unsigned short *pPtr,unsigned short usVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	ax ,usVal
		lock 	xchg word ptr [edx],ax
		}
	}

static __inline unsigned int __ExchangeDword(unsigned int *pPtr,unsigned int uiVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	eax,uiVal
		lock 	xchg dword ptr [edx],eax
		}
	}

static __inline void *__ExchangeAddPtr(void *pPtr,unsigned int uiVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	eax,uiVal
		lock 	xadd dword ptr [edx],eax
		}
	}

static __inline unsigned char __ExchangeAddByte(unsigned char *pPtr,unsigned char ucVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	al ,ucVal
		lock 	xadd byte ptr [edx],al
		}
	}

static __inline unsigned short __ExchangeAddWord(unsigned short *pPtr,unsigned short usVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	ax ,usVal
		lock 	xadd word ptr [edx],ax
		}
	}

static __inline unsigned int __ExchangeAddDword(unsigned int *pPtr,unsigned int uiVal)
	{
	_asm{
		mov	 	edx,pPtr
		mov  	eax,uiVal
		lock 	xadd dword ptr [edx],eax
		}
	}

#pragma warning(pop)

#define		ExchangePtr(ptr,val)			__ExchangePtr  ((void            *)(ptr),val)
#define		ExchangeByte(ptr,val)			__ExchangeByte ((unsigned char   *)(ptr),val)
#define		ExchangeWord(ptr,val)			__ExchangeWord ((unsigned short  *)(ptr),val)
#define		ExchangeDword(ptr,val)			__ExchangeDword((unsigned int    *)(ptr),val)

#define		ExchangeAddPtr(ptr,val)			__ExchangeAddByte ((void          *)(ptr),val)
#define		ExchangeAddByte(ptr,val)		__ExchangeAddByte ((unsigned char *)(ptr),val)
#define		ExchangeAddWord(ptr,val)		__ExchangeAddWord ((unsigned short*)(ptr),val)
#define		ExchangeAddDword(ptr,val)		__ExchangeAddDword((unsigned int  *)(ptr),val)

//
// arm 64bit cpu's
//
#elif defined(_M_ARM64)

#include	<intrin.h>
#include	<immintrin.h>

#define		ExchangePtr(ptr,val)			((void         *)_InterlockedExchange64_acq((__int64*)(ptr),(__int64)(val)))
#define		ExchangeByte(ptr,val)			((unsigned char )_InterlockedExchange8_acq ((char   *)(ptr),val))
#define		ExchangeWord(ptr,val)			((unsigned short)_InterlockedExchange16_acq((short  *)(ptr),val))
#define		ExchangeDword(ptr,val)			((unsigned int  )_InterlockedExchange_acq  ((long   *)(ptr),val))

#define		ExchangeAddPtr(ptr,val)			((void         *)_InterlockedExchangeAdd64_acq((__int64*)(ptr),val))
#define		ExchangeAddByte(ptr,val)		((unsigned char )_InterlockedExchangeAdd8_acq ((char   *)(ptr),val))
#define		ExchangeAddWord(ptr,val)		((unsigned short)_InterlockedExchangeAdd16_acq((short  *)(ptr),val))
#define		ExchangeAddDword(ptr,val)		((unsigned int  )_InterlockedExchangeAdd_acq  ((long   *)(ptr),val))

//
// arm 32bit cpu's
//
#elif defined(_M_ARM) ||  defined(_M_ARMT)

#include	<intrin.h>

#define		ExchangePtr(ptr,val)			((void         *)_InterlockedExchange_acq  ((long   *)(ptr),val))
#define		ExchangeByte(ptr,val)			((unsigned char )_InterlockedExchange8_acq ((char   *)(ptr),val))
#define		ExchangeWord(ptr,val)			((unsigned short)_InterlockedExchange16_acq((short  *)(ptr),val))
#define		ExchangeDword(ptr,val)			((unsigned int  )_InterlockedExchange_acq  ((long   *)(ptr),val))

#define		ExchangeAddPtr(ptr,val)			((void         *)_InterlockedExchangeAdd_acq  ((long   *)(ptr),val))
#define		ExchangeAddByte(ptr,val)		((unsigned char )_InterlockedExchangeAdd8_acq ((char   *)(ptr),val))
#define		ExchangeAddWord(ptr,val)		((unsigned short)_InterlockedExchangeAdd16_acq((short  *)(ptr),val))
#define		ExchangeAddDword(ptr,val)		((unsigned int  )_InterlockedExchangeAdd_acq  ((long   *)(ptr),val))

#else

#error	unknown cpu type

#endif


//******************************************************************************
//*
//*		IAR Compiler
//*
//******************************************************************************
#elif defined(__ICCARM__)

#include	<intrinsics.h>

#define		E_DISABLE_IRQ()		__disable_irq()
#define		E_ENABLE_IRQ()		__enable_irq()
#define		E_READ_IRQ()		((__get_CPSR()>>7)&1)

#define 	E_LDREXB		__LDREXB
#define 	E_LDREXH		__LDREXH
#define 	E_LDREX 		__LDREX
#define 	E_STREXB		__STREXB
#define 	E_STREXH		__STREXH
#define 	E_STREX 		__STREX

static inline void *__ExchangePtr(void *pPtr,void *pVal)
	{
	void *pOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	pOld          = *(void**)pPtr;
	*(void**)pPtr = pVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		pOld =  (unsigned*)E_LDREX((unsigned long*)pPtr);
		if(!E_STREX((unsigned)pVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return pOld;
	}

static inline unsigned char __ExchangeByte(unsigned char *pPtr,unsigned char ucVal)
	{
	unsigned char ucOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	ucOld = *pPtr;
	*pPtr = ucVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		ucOld   = E_LDREXB((unsigned char*)pPtr);
		if(!E_STREXB(ucVal,(unsigned char*)pPtr))
			{
			break;
			}
		}

	#endif

	return ucOld;
	}

static inline unsigned short __ExchangeWord(unsigned short *pPtr,unsigned short usVal)
	{
	unsigned short	usOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	usOld = *pPtr;
	*pPtr = usVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		usOld   = E_LDREXH((unsigned short*)pPtr);
		if(!E_STREXH(usVal,(unsigned short*)pPtr))
			{
			break;
			}
		}

	#endif

	return usOld;
	}

static inline unsigned int __ExchangeDword(unsigned int *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *pPtr;
	*pPtr = uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld   = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return uiOld;
	}

static inline void *__ExchangeAddPtr(void *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *(unsigned*)pPtr;
	*(unsigned*)pPtr = uiOld+uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld         = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiOld+uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return (void*)uiOld;
	}

static inline unsigned char __ExchangeAddByte(unsigned char *pPtr,unsigned char ucVal)
	{
	unsigned char ucOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	ucOld = *pPtr;
	*pPtr = uiOld+ucVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		ucOld         = E_LDREXB((unsigned char*)pPtr);
		if(!E_STREXB(ucOld+ucVal,(unsigned char*)pPtr))
			{
			break;
			}
		}

	#endif

	return ucOld;
	}

static inline unsigned short __ExchangeAddWord(unsigned short *pPtr,unsigned short usVal)
	{
	unsigned short	usOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	usOld = *pPtr;
	*pPtr = usOld+usVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		usOld         = E_LDREXH((unsigned short*)pPtr);
		if(!E_STREXH(usOld+usVal,(unsigned short*)pPtr))
			{
			break;
			}
		}

	#endif

	return usOld;
	}

static inline unsigned int __ExchangeAddDword(unsigned int *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *pPtr;
	*pPtr = uiOld+uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld         = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiOld+uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return uiOld;
	}

#define		ExchangePtr(ptr,val)			__ExchangePtr  ((void            *)(ptr),val)
#define		ExchangeByte(ptr,val)			__ExchangeByte ((unsigned char   *)(ptr),val)
#define		ExchangeWord(ptr,val)			__ExchangeWord ((unsigned short  *)(ptr),val)
#define		ExchangeDword(ptr,val)			__ExchangeDword((unsigned int    *)(ptr),val)

#define		ExchangeAddPtr(ptr,val)			__ExchangeAddPtr  ((void          *)(ptr),val)
#define		ExchangeAddByte(ptr,val)		__ExchangeAddByte ((unsigned char *)(ptr),val)
#define		ExchangeAddWord(ptr,val)		__ExchangeAddWord ((unsigned short*)(ptr),val)
#define		ExchangeAddDword(ptr,val)		__ExchangeAddDword((unsigned int  *)(ptr),val)



//******************************************************************************
//*
//*		Keil/Realview Compiler
//*
//******************************************************************************
#elif defined(__CC_ARM)

#define 	E_STREXB(value,ptr)     	   	__strex(value,(uint8_t *)(ptr))
#define 	E_STREXH(value,ptr) 	       	__strex(value,(uint16_t*)(ptr))
#define 	E_STREX(value,ptr)		       	__strex(value,(uint32_t*)(ptr))
#define 	E_LDREXB(ptr)                   ((uint8_t )__ldrex((uint8_t *)(ptr)))
#define 	E_LDREXH(ptr)                   ((uint16_t)__ldrex((uint16_t*)(ptr)))
#define 	E_LDREX(ptr)                    ((uint32_t)__ldrex((uint32_t*)(ptr)))

static __inline void *__ExchangePtr(void *pPtr,void *pVal)
	{
	void *pOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	pOld          = *(void**)pPtr;
	*(void**)pPtr = pVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		pOld =  (unsigned*)E_LDREX((unsigned char*)pPtr);
		if(!E_STREX((unsigned)pVal,(unsigned char*)pPtr))
			{
			break;
			}
		}

	#endif

	return pOld;
	}

static __inline unsigned char __ExchangeByte(unsigned char *pPtr,unsigned char ucVal)
	{
	unsigned char ucOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	ucOld = *pPtr;
	*pPtr = ucVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		ucOld   = E_LDREXB((unsigned char*)pPtr);
		if(!E_STREXB(ucVal,(unsigned char*)pPtr))
			{
			break;
			}
		}

	#endif

	return ucOld;
	}

static __inline unsigned short __ExchangeWord(unsigned short *pPtr,unsigned short usVal)
	{
	unsigned short	usOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	usOld = *pPtr;
	*pPtr = usVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		usOld   = E_LDREXH((unsigned short*)pPtr);
		if(!E_STREXH(usVal,(unsigned short*)pPtr))
			{
			break;
			}
		}

	#endif

	return usOld;
	}

static __inline unsigned int __ExchangeDword(unsigned int *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *pPtr;
	*pPtr = uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld   = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return uiOld;
	}

static __inline void *__ExchangeAddPtr(void *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *(unsigned*)pPtr;
	*(unsigned*)pPtr = uiOld+uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld         = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiOld+uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return (void*)uiOld;
	}

static __inline unsigned char __ExchangeAddByte(unsigned char *pPtr,unsigned char ucVal)
	{
	unsigned char ucOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	ucOld = *pPtr;
	*pPtr = uiOld+ucVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		ucOld         = E_LDREXB((unsigned char*)pPtr);
		if(!E_STREXB(ucOld+ucVal,(unsigned char*)pPtr))
			{
			break;
			}
		}

	#endif

	return ucOld;
	}

static __inline unsigned short __ExchangeAddWord(unsigned short *pPtr,unsigned short usVal)
	{
	unsigned short	usOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	usOld = *pPtr;
	*pPtr = usOld+usVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		usOld         = E_LDREXH((unsigned short*)pPtr);
		if(!E_STREXH(usOld+usVal,(unsigned short*)pPtr))
			{
			break;
			}
		}

	#endif

	return usOld;
	}

static __inline unsigned int __ExchangeAddDword(unsigned int *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *pPtr;
	*pPtr = uiOld+uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld         = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiOld+uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return uiOld;
	}

#define		ExchangePtr(ptr,val)			__ExchangePtr  ((void            *)(ptr),val)
#define		ExchangeByte(ptr,val)			__ExchangeByte ((unsigned char   *)(ptr),val)
#define		ExchangeWord(ptr,val)			__ExchangeWord ((unsigned short  *)(ptr),val)
#define		ExchangeDword(ptr,val)			__ExchangeDword((unsigned int    *)(ptr),val)

#define		ExchangeAddPtr(ptr,val)			__ExchangeAddPtr  ((void          *)(ptr),val)
#define		ExchangeAddByte(ptr,val)		__ExchangeAddByte ((unsigned char *)(ptr),val)
#define		ExchangeAddWord(ptr,val)		__ExchangeAddWord ((unsigned short*)(ptr),val)
#define		ExchangeAddDword(ptr,val)		__ExchangeAddDword((unsigned int  *)(ptr),val)

//*****************************************************************************
//*
//*		GNU Compiler
//*
//*****************************************************************************
#elif defined(__GNUC__)


#ifdef __ATOMIC_RELAXED

__attribute__((always_inline)) static inline  void *ExchangePtr(void *pPtr,unsigned int uiVal)
	{
	if(sizeof(pPtr)==8)
		{
		unsigned long long	ulVal=(int)uiVal;
		unsigned long long  ulRet;
	
		__atomic_exchange((unsigned long long*)pPtr,&ulVal,&ulRet,__ATOMIC_RELAXED);	
	
		return (void*)ulRet;
		}
	else{
		unsigned int  uiRet;
	
		__atomic_exchange((unsigned int*)pPtr,&uiVal,&uiRet,__ATOMIC_RELAXED);	
	
		return (void*)uiRet;
		}
	}

__attribute__((always_inline)) static inline  unsigned char ExchangeByte(unsigned char *pPtr,unsigned char ucVal)
	{
	unsigned char ucRet;
	
	__atomic_exchange(pPtr,&ucVal,&ucRet,__ATOMIC_RELAXED);	
	
	return ucRet;
	}

__attribute__((always_inline)) static inline  unsigned short ExchangeWord(unsigned short *pPtr,unsigned short usVal)
	{
	unsigned short usRet;
	
	__atomic_exchange(pPtr,&usVal,&usRet,__ATOMIC_RELAXED);	
	
	return usRet;
	}

__attribute__((always_inline)) static inline  unsigned int ExchangeDword(unsigned int *pPtr,unsigned int uiVal)
	{
	unsigned int  uiRet;
	
	__atomic_exchange(pPtr,&uiVal,&uiRet,__ATOMIC_RELAXED);	
	
	return uiRet;
	}

__attribute__((always_inline)) static inline  void *ExchangeAddPtr(void *pPtr,unsigned int uiVal)
	{
	if(sizeof(pPtr)==8)
			return (void*)__sync_fetch_and_add((unsigned long long*)pPtr,(unsigned long long)uiVal);	
	else	return (void*)__sync_fetch_and_add((unsigned int      *)pPtr,(unsigned int      )uiVal);			
	}

__attribute__((always_inline)) static inline  unsigned char ExchangeAddByte(unsigned char *pPtr,unsigned char ucVal)
	{
	return __sync_fetch_and_add(pPtr,ucVal,__ATOMIC_RELAXED);	
	}

__attribute__((always_inline)) static inline  unsigned short ExchangeAddWord(unsigned short *pPtr,unsigned short usVal)
	{
	return __sync_fetch_and_add(pPtr,usVal,__ATOMIC_RELAXED);	
	}

__attribute__((always_inline)) static inline  unsigned int ExchangeAddDword(unsigned int *pPtr,unsigned int uiVal)
	{
	return __sync_fetch_and_add(pPtr,uiVal,__ATOMIC_RELAXED);	
	}


#else

//
// intel/amd 32/64bit cpu's
//
#if defined(__x86_64__) || defined(__amd64__) || defined(__i386__) || defined(__i486__) || defined(__i586__)

__attribute__((always_inline)) static inline unsigned char __ExchangeByte(unsigned char *pPtr,unsigned char ucVal)
	{
    asm volatile (
        "lock; xchgb %b0,%1 \n"
        : "=q" (ucVal)
        : "m"  (*pPtr) , "0" (ucVal)
		: "memory"
        );

	return  ucVal;
	}

__attribute__((always_inline)) static inline unsigned short __ExchangeWord(unsigned short *pPtr,unsigned short usVal)
	{
    asm volatile (
        "lock; xchgw %w0,%1 \n"
        : "=r" (usVal)
        : "m"  (*pPtr) , "0" (usVal)
		: "memory"
        );

	return  usVal;
	}

__attribute__((always_inline)) static inline unsigned int __ExchangeDword(unsigned int *pPtr,unsigned int uiVal)
	{
    asm volatile (
        "lock; xchgl %0,%1 \n"
        : "=r" (uiVal)
        : "m"  (*pPtr) , "0" (uiVal)
		: "memory"
        );

	return  uiVal;
	}

#if defined(__x86_64__) || defined(__amd64__)
__attribute__((always_inline)) static inline unsigned long long __ExchangeQword(unsigned long long *pPtr,unsigned long long ulVal)
	{
    asm volatile (
        "lock; xchgq %0,%1 \n"
        : "=r" (ulVal)
        : "m"  (*pPtr) , "0" (ulVal)
		: "memory"
        );

	return  ulVal;
	}
#endif

__attribute__((always_inline)) static inline unsigned char __ExchangeAddByte(unsigned char *pPtr,unsigned char ucVal)
	{
    asm volatile (
        "lock; xaddb %b0,%1 \n"
        : "=q" (ucVal)
        : "m"  (*pPtr) , "0" (ucVal)
		: "memory"
        );

	return  ucVal;
	}

__attribute__((always_inline)) static inline unsigned short __ExchangeAddWord(unsigned short *pPtr,unsigned short usVal)
	{
    asm volatile (
        "lock; xaddw %w0,%1 \n"
        : "=r" (usVal)
        : "m"  (*pPtr) , "0" (usVal)
		: "memory"
        );

	return  usVal;
	}

__attribute__((always_inline)) static inline unsigned int __ExchangeAddDword(unsigned int *pPtr,unsigned int uiVal)
	{
    asm volatile (
        "lock; xaddl %0,%1 \n"
        : "=r" (uiVal)
        : "m"  (*pPtr) , "0" (uiVal)
		: "memory"
        );

	return  uiVal;
	}

#if defined(__x86_64__) || defined(__amd64__)
__attribute__((always_inline)) static inline unsigned long long __ExchangeAddQword(unsigned long long *pPtr,unsigned long long ulVal)
	{
    asm volatile (
        "lock; xaddq %0,%1 \n"
        : "=r" (ulVal)
        : "m"  (*pPtr) , "0" (ulVal)
		: "memory"
        );

	return  ulVal;
	}

#endif

#define		ExchangeByte(ptr,val)			__ExchangeByte ((unsigned char   *)(ptr),val)
#define		ExchangeWord(ptr,val)			__ExchangeWord ((unsigned short  *)(ptr),val)
#define		ExchangeDword(ptr,val)			__ExchangeDword((unsigned int    *)(ptr),val)

#define		ExchangeAddByte(ptr,val)		__ExchangeAddByte ((unsigned char *)(ptr),val)
#define		ExchangeAddWord(ptr,val)		__ExchangeAddWord ((unsigned short*)(ptr),val)
#define		ExchangeAddDword(ptr,val)		__ExchangeAddDword((unsigned int  *)(ptr),val)

#if 		defined(__x86_64__) || defined(__amd64__)
#define		ExchangePtr(ptr,val)			((void*)__ExchangeQword   ((unsigned long long*)(ptr),(unsigned long long)(val)))
#define		ExchangeAddPtr(ptr,val)			((void*)__ExchangeAddQword((unsigned long long*)(ptr),(         long long)(val)))
#else
#define		ExchangePtr(ptr,val)			((void*)__ExchangeDword   ((unsigned int *)(ptr),(unsigned int)(val)))
#define		ExchangeAddPtr(ptr,val)			((void*)__ExchangeAddDword((unsigned int *)(ptr),(unsigned int)(val)))
#endif


//
// arm 32/64bit cpu's
//
#elif defined(__arm__) || defined(__aarch64__)

#define		__ASM	asm
#include	<inttypes.h>


__attribute__((always_inline)) static inline  void bit_enable_irq(void)
	{
	__ASM volatile ("cpsie i" : : : "memory");
	}

__attribute__((always_inline)) static inline  void bit_disable_irq(void)
	{
	__ASM volatile ("cpsid i" : : : "memory");
	}

__attribute__((always_inline)) static inline  uint32_t bit_get_cpsr(void)
	{
	uint32_t uRet;
	__ASM volatile ("mrs %0,CPSR":"=r" (uRet):) ;
	return uRet;
	}

__attribute__((always_inline)) static inline  uint32_t E_STREXB(uint8_t value, volatile uint8_t *addr)
	{
	uint32_t result;
	__ASM volatile ("strexb %0, %2, %1" : "=&r" (result), "=Q" (*addr) : "r" ((uint32_t)value));
	return(result);
	}

__attribute__((always_inline)) static inline  uint8_t E_LDREXB(volatile uint8_t *addr)
	{
	uint32_t result;
	__ASM volatile ("ldrexb %0, %1" : "=r" (result) : "Q" (*addr));
	return ((uint8_t) result);
	}

__attribute__((always_inline)) static inline  uint32_t E_STREXH(uint16_t value, volatile uint16_t *addr)
	{
	uint32_t result;
	__ASM volatile ("strexh %0, %2, %1" : "=&r" (result), "=Q" (*addr) : "r" ((uint32_t)value));
	return(result);
	}

__attribute__((always_inline)) static inline  uint16_t E_LDREXH(volatile uint16_t *addr)
	{
	uint32_t result;
	__ASM volatile ("ldrexh %0, %1" : "=r" (result) : "Q" (*addr));
	return ((uint16_t) result);
	}

__attribute__((always_inline)) static inline  uint32_t E_STREX(uint32_t value, volatile uint32_t *addr)
	{
	uint32_t result;
	__ASM volatile ("strex %0, %2, %1" : "=&r" (result), "=Q" (*addr) : "r" ((uint32_t)value));
	return(result);
	}

__attribute__((always_inline)) static inline  uint32_t E_LDREX(volatile uint32_t *addr)
	{
	uint32_t result;
	__ASM volatile ("ldrex %0, %1" : "=r" (result) : "Q" (*addr));
	return ((uint32_t) result);
	}

#if defined(__aarch64__)

__attribute__((always_inline)) static inline  uint64_t E_LDREXD(volatile uint64_t *addr)
	{
	uint32_t result;
	__ASM volatile ("ldrexd %0, %1" : "=r" (result) : "Q" (*addr));
	return ((uint64_t) result);
	}

__attribute__((always_inline)) static inline  uint32_t E_STREXD(uint64_t value, volatile uint64_t *addr)
	{
	uint32_t result;
	__ASM volatile ("strexd %0, %2, %1" : "=&r" (result), "=Q" (*addr) : "r" ((uint32_t)value));
	return(result);
	}

#define		E_DISABLE_IRQ()		__disable_irq()
#define		E_ENABLE_IRQ()		__enable_irq()
#define		E_READ_IRQ()		((bit_get_cpsr()>>7)&1)

__attribute__((always_inline)) static inline unsigned long long __ExchangeQword(unsigned long long *pPtr,unsigned long long ulVal)
	{
	unsigned long ulOld;

	for(;;)
		{
		ulOld   = E_LDREXD((unsigned long long*)pPtr);
		if(!E_STREXD(ulVal,(unsigned long long*)pPtr))
			{
			break;
			}
		}

	return ulOld;
	}

__attribute__((always_inline)) static inline unsigned int __ExchangeAddDword(unsigned int *pPtr,long long llVal)
	{
	unsigned long long	ulOld;

	for(;;)
		{
		ulOld         = E_LDREXD((unsigned long*)pPtr);
		if(!E_STREXD(ulOld+llVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	return ulOld;
	}

#endif


__attribute__((always_inline)) static inline unsigned char __ExchangeByte(unsigned char *pPtr,unsigned char ucVal)
	{
	unsigned char ucOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	ucOld = *pPtr;
	*pPtr = ucVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		ucOld   = E_LDREXB((unsigned char*)pPtr);
		if(!E_STREXB(ucVal,(unsigned char*)pPtr))
			{
			break;
			}
		}

	#endif

	return ucOld;
	}

__attribute__((always_inline)) static inline unsigned short __ExchangeWord(unsigned short *pPtr,unsigned short usVal)
	{
	unsigned short	usOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	usOld = *pPtr;
	*pPtr = usVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		usOld   = E_LDREXH((unsigned short*)pPtr);
		if(!E_STREXH(usVal,(unsigned short*)pPtr))
			{
			break;
			}
		}

	#endif

	return usOld;
	}

__attribute__((always_inline)) static inline unsigned int __ExchangeDword(unsigned int *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *pPtr;
	*pPtr = uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld   = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return uiOld;
	}


__attribute__((always_inline)) static inline unsigned char __ExchangeAddByte(unsigned char *pPtr,unsigned char ucVal)
	{
	unsigned char ucOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	ucOld = *pPtr;
	*pPtr = uiOld+ucVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		ucOld         = E_LDREXB((unsigned char*)pPtr);
		if(!E_STREXB(ucOld+ucVal,(unsigned char*)pPtr))
			{
			break;
			}
		}

	#endif

	return ucOld;
	}

__attribute__((always_inline)) static inline unsigned short __ExchangeAddWord(unsigned short *pPtr,unsigned short usVal)
	{
	unsigned short	usOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	usOld = *pPtr;
	*pPtr = usOld+usVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		usOld         = E_LDREXH((unsigned short*)pPtr);
		if(!E_STREXH(usOld+usVal,(unsigned short*)pPtr))
			{
			break;
			}
		}

	#endif

	return usOld;
	}

__attribute__((always_inline)) static inline unsigned int __ExchangeAddDword(unsigned int *pPtr,unsigned int uiVal)
	{
	unsigned int uiOld;

	#ifdef EXCHANGE_USE_IRQ

	int 	iIrq;

	   iIrq=E_READ_IRQ();
	if(iIrq)E_DISABLE_IRQ();

	uiOld = *pPtr;
	*pPtr = uiOld+uiVal;

	if(iIrq)E_ENABLE_IRQ();

	#else

	for(;;)
		{
		uiOld         = E_LDREX((unsigned long*)pPtr);
		if(!E_STREX(uiOld+uiVal,(unsigned long*)pPtr))
			{
			break;
			}
		}

	#endif

	return uiOld;
	}

#define		ExchangeByte(ptr,val)			__ExchangeByte ((unsigned char   *)(ptr),val)
#define		ExchangeWord(ptr,val)			__ExchangeWord ((unsigned short  *)(ptr),val)
#define		ExchangeDword(ptr,val)			__ExchangeDword((unsigned int    *)(ptr),val)

#define		ExchangeAddByte(ptr,val)		__ExchangeAddByte ((unsigned char *)(ptr),val)
#define		ExchangeAddWord(ptr,val)		__ExchangeAddWord ((unsigned short*)(ptr),val)
#define		ExchangeAddDword(ptr,val)		__ExchangeAddDword((unsigned int  *)(ptr),val)

#if 		defined(__aarch64__)
#define		ExchangePtr(ptr,val)			((void*)__ExchangeQword((unsigned long long*)(ptr),(unsigned long long)(val)))
#define		ExchangeAddPtr(ptr,val)			((void*)__ExchangeQword((unsigned long long*)(ptr),(         long long)(val)))
#else
#define		ExchangePtr(ptr,val)			((void*)__ExchangeDword((unsigned int *)(ptr),(unsigned int)(val)))
#define		ExchangeAddPtr(ptr,val)			((void*)__ExchangeDword((unsigned int *)(ptr),(unsigned int)(val)))
#endif


#else

#error	unknown cpu type

#endif
#endif

//******************************************************************************
//*
//*		Wrong compiler
//*
//******************************************************************************
#else

#error 	Wrong compiler

#endif

#define	ExchangeSubPtr(ptr,val)			ExchangeAddPtr  (ptr,-(int)(val))
#define	ExchangeSubByte(ptr,val)		ExchangeAddByte (ptr,-(int)(val))
#define	ExchangeSubWord(ptr,val)		ExchangeAddWord (ptr,-(int)(val))
#define	ExchangeSubDword(ptr,val)		ExchangeAddDword(ptr,-(int)(val))

#define	ExchangeDecPtr(ptr)				ExchangeAddPtr  (ptr,-1)
#define	ExchangeDecByte(ptr)			ExchangeAddByte (ptr,-1)
#define	ExchangeDecWord(ptr)			ExchangeAddWord (ptr,-1)
#define	ExchangeDecDword(ptr)			ExchangeAddDword(ptr,-1)

#define	ExchangeIncPtr(ptr)				ExchangeAddPtr  (ptr, 1)
#define	ExchangeIncByte(ptr)			ExchangeAddByte (ptr, 1)
#define	ExchangeIncWord(ptr)			ExchangeAddWord (ptr, 1)
#define	ExchangeIncDword(ptr)			ExchangeAddDword(ptr, 1)


#endif



