//*****************************************************************************
//*
//*
//*		FileUtility.c
//*
//*
//*****************************************************************************
//
//  This is a library fore file and directory access for WINDOWS/LINUX/RTTARGET,
//
//	Global Macros:
//
//		GLOBAL_CONFIG			Enable the configuration via GlobalConfig.h header file (1=on 0=off)
//		SYS_USE_GLOBAL_PATH		Converts paths beginnting with "/C:..." to "C:..."  (0=off 1=on)
//		SYS_DRIVE_SLOTS			Slots for the drive serarch in "/..." (only when SYS_USE_GLOBAL_PATH is 1)
//
//
//  Copyright  2018    Anton Zechner
//
//
#ifdef		GLOBAL_CONFIG
#define 	FILEUTILITY_C
#include	<GlobalConfig.h>
#undef		FILEUTILITY_C
#endif


#include	<stdio.h>
#include	<windows.h>
#include	"FileUtility.h"

#ifdef		_MSC_VER
#pragma 	warning( disable :4035 )
#pragma 	warning( disable :4996 )
#endif

#ifndef 	SYS_USE_GLOBAL_PATH
#define 	SYS_USE_GLOBAL_PATH		0
#endif

#ifndef 	SYS_DRIVE_SLOTS
#define 	SYS_DRIVE_SLOTS			4
#endif

#define 	SYS_DRIVE_HANDLE		0xFFEEBB00
#define 	IS_DRIVE_HANDLE(h)		((((DWORD)h)&0xFFFFFF00)==SYS_DRIVE_HANDLE)
#define 	NUM_OF_DRIVE_HANDLE(h)	((((DWORD)h)&0x000000FF))
#define 	GET_DRIVE_HANDLE(s)		((HANDLE)(SYS_DRIVE_HANDLE|(s)))


#if			SYS_USE_GLOBAL_PATH
static		DWORD	dwDriveBits	= 0;					// bits wiht logical drives
static		CHAR	aDriveFilter[SYS_DRIVE_SLOTS*16];	// filter for drive search
#define 	CHECK_GLOBAL_PATH(p)	if((p[0]=='/' || p[0]=='\\') && p[1] && p[2]==':')p++;
#else
#define 	CHECK_GLOBAL_PATH(p)
#endif


typedef struct
    {
    ULONG           uStatus;
    ULONG           uInformation;
    }IO_STATUS_BLOCK_;


typedef struct _FILE_NAME_INFORMATION
    {
    ULONG           uFileNameLength;
    WCHAR           aFileName[256];
    }FILE_NAME_INFORMATION_;

typedef LONG  (__stdcall *NtQueryInformationFileType)(HANDLE hFileHandle,IO_STATUS_BLOCK_ *pIoStatusBlock,PVOID pFileInformation,ULONG uLength,UINT uInfoMode);


static HMODULE						hNtDll					= 0;
static NtQueryInformationFileType   pNtQueryInformationFile	= 0;



//*****************************************************************************
//*
//*		StrCmpEx
//*
//*****************************************************************************
//	Compares tow strings with wildcards ('*' and '?')
//	pString		: pointer to the string without wildcards
//	pCompare	: pointer to the string with wildcards
//	Returns 0 if the strings are equal.
#if SYS_USE_GLOBAL_PATH
static int StrCmpEx(const char *pString,const char *pCompare)
{
int		iChar2;
int		iChar1;
int		iDiff;


	while(*pCompare)
		{
		if(*pCompare=='?')								// step over a single sign
			{
			if(!*pString)
				{
				return 1;
				}
			}
		else if (*pCompare=='*')						// start recurcive search
			{
			if(!StrCmpEx(pString,pCompare+1))
				{
				return 0;
				}

			if(*pString && !StrCmpEx(pString+1,pCompare))
				{
				return 0;
				}

			return 1;
			}
		else{											// compare one sign
			iChar2	= *pCompare;
			iChar1	= *pString;

			if(iChar1<='Z' && iChar1>='A')iChar1+='a'-'A';
			if(iChar2<='Z' && iChar2>='A')iChar2+='a'-'A';

			iDiff = iChar2-iChar1;

			if(iDiff)
				{
				return iDiff;
				}
			}

		pCompare++;
		pString++;
		}


	if(*pString)										// was it all equal
		{
		return 1;
		}


return 0;
}
#endif

//*****************************************************************************
//*
//*		SysTimeConvertFS	   FILETIME --> SysUtc
//*
//*****************************************************************************
//	Converts a FILETIME to an SysUtc type  (used in FileUtility.c)
//	pFileTime	: pointer to a windows file time item
//	pSysTime	: pointer to a system time item
//	Returns 1 if the conversion was ok or 0 if an error ocurrs
static int SysTimeConvertFS(FILETIME *pFileTime,SysUtc *pSysTime)
{
union	{
		__int64	i64;
		DWORD	dw[2];
		}sTime;


	sTime.i64  = *(__int64*)pFileTime;
	sTime.i64 -= 0x019db1ded53e8000;

	if(sTime.dw[1]&0x80000000)							// Zeit vor Mi 1.Jan.1970
		{
		*pSysTime = 0;
		return 0;
		}

	sTime.i64 /= 10000000;								// 100ns zu Sekunden umwandeln

	if(sTime.dw[1])										// ber Jahr 2106
		{
		*pSysTime = 0xFFFFFFFF;
		return 0;
		}

	*pSysTime = sTime.dw[0];							// UTC Zeit speichern


return 1;
}


//*****************************************************************************
//*
//*     SysTimeConvertSF        SysUtc  --> LONGTIME
//*
//*****************************************************************************
//	Converts a SysUtc to a FILETIME type  (used in FileUtility.c)
//	uSysTime	: a system time item
//	pWftTime	: pointer to a windows file time item
//	Returns 1 if the conversion was ok or 0 if an error ocurrs
static int SysTimeConvertSF(SysUtc uSysTime,void  *pWftTime)
{
union   {
        __int64     i64;
        DWORD       dw[2];
        }uVal;
 
 
    if(uSysTime==0xFFFFFFFF || uSysTime==0)
        {
        ((SysDW*)pWftTime)[1] = 0xFFFFFFFF;
        ((SysDW*)pWftTime)[0] = 0xFFFFFFFF;
        return 0;
        }
 
 
    uVal.i64  = uSysTime;
    uVal.i64 *= 10000000;                       // Sekunden zu 100ns umwandeln
    uVal.i64 += (__int64)0x019db1ded53e8000L;
 
    ((SysDW*)pWftTime)[1] = uVal.dw[1];         // Zeit speichern
    ((SysDW*)pWftTime)[0] = uVal.dw[0];
 
 
return 1;
}


//*****************************************************************************
//*
//*		SysError
//*
//*****************************************************************************
static int SysError()
{
int iErr;

	iErr=GetLastError();

	if(iErr==ERROR_ACCESS_DENIED 	)return SYS_ERR_ACCESS_DENIED;
	if(iErr==ERROR_FILE_NOT_FOUND	)return SYS_ERR_FILE_NOT_FOUND;
	if(iErr==ERROR_NO_MORE_FILES	)return SYS_ERR_FILE_NOT_FOUND;
	if(iErr==ERROR_DISK_FULL     	)return SYS_ERR_DISKFULL;
	if(iErr==ERROR_DIRECTORY     	)return SYS_ERR_PATH_NOT_FOUND;
	if(iErr==ERROR_PATH_NOT_FOUND	)return SYS_ERR_PATH_NOT_FOUND;
	if(iErr==ERROR_NO_MORE_FILES	)return SYS_ERR_NO_HANDLES;
	if(iErr==ERROR_INVALID_PARAMETER)return SYS_ERR_PARAM;
	if(iErr==ERROR_INVALID_NAME		)return SYS_ERR_INVALID_FILENAME;
	if(iErr==ERROR_ALREADY_EXISTS   )return SYS_ERR_ALREADY_EXISTS;


return SYS_ERR_UNKNOWN;
}

//*****************************************************************************
//*
//*		SysClearFindStruct
//*
//*****************************************************************************
static __inline void SysClearFindStruct(SysFindStruct *pFind)
{
	pFind->dwSize=0;
	pFind->cName[0]=0;
}


//*****************************************************************************
//*
//*		SysFileExist
//*
//*****************************************************************************
//	Checks if a file exists
//	pFilename	: pointer to file name
//	Returns	: 	0 = file din't exist
//				1 = file exists
//				2 =	file exists and is a directory
int SysFileExist(const char *pFilename)
{
DWORD dwAttr;
int	  iRet;


	CHECK_GLOBAL_PATH(pFilename)

	   dwAttr=GetFileAttributes(pFilename);
	if(dwAttr==0xFFFFFFFF)return 0;

	iRet=(dwAttr&FILE_ATTRIBUTE_DIRECTORY)? 2:1;


return iRet;
}


//*****************************************************************************
//*
//*		SysGetAttributes
//*
//*****************************************************************************
//	get the the attribute from a file
//	pFilename	: pointer to file name
//	return		: <0 = error
//				  SYS_READ
//				  SYS_WRITE
//				  SYS_DIR
//				  SYS_HIDDEN
//				  SYS_ARCHIVE
int SysGetAttributes(const char *pFilename)
{
DWORD		dwAttr;
int			iRet;


	CHECK_GLOBAL_PATH(pFilename)

	   dwAttr=GetFileAttributes(pFilename);
	if(dwAttr==0xFFFFFFFF)return SysError();

	iRet = SYS_READ|SYS_WRITE;

	if(dwAttr&FILE_ATTRIBUTE_READONLY )iRet&=~SYS_WRITE;
	if(dwAttr&FILE_ATTRIBUTE_DIRECTORY)iRet|= SYS_DIR;
	if(dwAttr&FILE_ATTRIBUTE_HIDDEN   )iRet|= SYS_HIDDEN;
	if(dwAttr&FILE_ATTRIBUTE_ARCHIVE  )iRet|= SYS_ARCHIVE;


return iRet;
}


//*****************************************************************************
//*
//*		SysSetAttributes
//*
//*****************************************************************************
//	set the the attribute from a file
//	pFilename	: pointer to file name
//	return		: <0 = error
//				   1 = ok
int SysSetAttributes(const char *pFilename,SysDW dwAttr)
{
int iAttr;


	if(dwAttr&(SYS_DIR|SYS_DIR))
		{
		return SYS_ERR_PARAM;
		}

	CHECK_GLOBAL_PATH(pFilename)

	iAttr=(dwAttr&SYS_WRITE)? FILE_ATTRIBUTE_NORMAL:FILE_ATTRIBUTE_READONLY;

	if(!SetFileAttributes(pFilename,iAttr))return SysError();


return SYS_ERR_NONE;
}



//*****************************************************************************
//*
//*		SysFindFirstEx
//*
//*****************************************************************************
//	search for a file
//	pFilename	: find path (with '*' and '?' )
//	pFind		: pointer to file structer
//	pError		: if this pointer isn't zero, here the error number will be sored
//	returns a find handle or 0 if an error occurs
SysFind	SysFindFirstEx(const char *pFilename,SysFindStruct *pFind,int *pError)
{
WIN32_FIND_DATA	sFind;
HANDLE			hHandle;
int				iAttr;



	#if SYS_USE_GLOBAL_PATH

	if(pFilename[0]=='/' || pFilename[0]=='\\')			// is it a global path "/..."
		{
		int		iPos;
		int		iErr;

		if(pFilename[1]==0)
			{
			if(pError)*pError=SYS_ERR_PATH_NOT_FOUND;
			SysClearFindStruct(pFind);
			return NULL;
			}

		for(iPos=1;;iPos++)								// to the next sub path
			{
			if(pFilename[iPos]== 0  )break;
			if(pFilename[iPos]=='/' )break;
			if(pFilename[iPos]=='\\')break;
			}

		if(pFilename[iPos])								// are we in the root path
			{
			if(pFilename[2]==':')pFilename++;
			goto Done;
			}

		for(iPos=SYS_DRIVE_SLOTS-1;;iPos--)				// search for an empty slot
			{
			if(aDriveFilter[iPos*16]==0)break;
			if(iPos>0)continue;

			if(pError)*pError=SYS_ERR_NO_HANDLES;
			SysClearFindStruct(pFind);
			return NULL;
			}

		aDriveFilter[iPos*16]	= 1;					// fill slot
		strncpy(aDriveFilter+iPos*16+1,pFilename+1,14);
		dwDriveBits				= GetLogicalDrives();
		hHandle					= GET_DRIVE_HANDLE(iPos);
		iErr					= SysFindNext((SysFind)hHandle,pFind);

		if(pError)*pError=iErr;

		if(iErr)										// nothing found
			{
			aDriveFilter[iPos] = 0;
			return 0;
			}

		return (SysFind)hHandle;
Done:;	}

	#endif

	if(!pFind)											// check parameters
		{
		if(pError)*pError=SYS_ERR_PARAM;
		SysClearFindStruct(pFind);
		return 0;
		}

	   hHandle=FindFirstFile(pFilename,&sFind);			// find the first file
	if(hHandle==INVALID_HANDLE_VALUE)
		{
		if(pError)*pError=SysError();
		SysClearFindStruct(pFind);
		return 0;
		}

	strcpy(pFind->cName,sFind.cFileName);

	iAttr=(sFind.dwFileAttributes&FILE_ATTRIBUTE_READONLY)? SYS_READ:SYS_READ|SYS_WRITE;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)iAttr|=SYS_DIR;

	pFind->dwAttr = (sFind.nFileSizeHigh<<SYS_HIGHSHIFT)|iAttr;
	pFind->dwSize =  sFind.nFileSizeLow;

	SysTimeConvertFS(&sFind.ftCreationTime  ,&pFind->dwTimeCreate);
	SysTimeConvertFS(&sFind.ftLastWriteTime ,&pFind->dwTimeWrite);
	SysTimeConvertFS(&sFind.ftLastAccessTime,&pFind->dwTimeAccess);

	if(pError)*pError=SYS_ERR_NONE;



return (SysFind)hHandle;
}


//*****************************************************************************
//*
//*		SysFindNext
//*
//*****************************************************************************
//	search for the next file
//	hHandle	: find handle
//	pFind	: pointer to file structer
//	returns 0 if a file was found or an error code below zero
int SysFindNext(SysFind	hHandle,SysFindStruct *pFind)
{
WIN32_FIND_DATA	sFind;
int				iAttr;


	if(!pFind || !hHandle)
		{
		SysClearFindStruct(pFind);
		return SYS_ERR_PARAM;
		}

	#if SYS_USE_GLOBAL_PATH

	if(IS_DRIVE_HANDLE(hHandle))						// search for a drive
		{
		int		iSlot;
		int		iPos;

		   iSlot = NUM_OF_DRIVE_HANDLE(hHandle);
		if(iSlot>=SYS_DRIVE_SLOTS || !aDriveFilter[iSlot*16])
			{
			SysClearFindStruct(pFind);
			return SYS_ERR_PARAM;
			}

		iPos = (BYTE)aDriveFilter[iSlot*16]-1;

		pFind->cName[0] = 'A'+iPos;
		pFind->cName[1] = ':';
		pFind->cName[2] =  0 ;

		for(;iPos<26;iPos++,pFind->cName[0]++)			// find drive to a filter
			{
			aDriveFilter[iSlot*16]++;

			if(!((dwDriveBits>>iPos)&1))
				{
				continue;
				}

			if(StrCmpEx(pFind->cName,aDriveFilter+iSlot*16+1))
				{
				continue;
				}

			pFind->dwAttr		= SYS_DIR;				// drive fits to the filter
			pFind->dwSize		= 0;
			pFind->dwTimeAccess	= 0;
			pFind->dwTimeCreate	= 0;
			pFind->dwTimeWrite	= 0;

			return 0;
			}

		SysClearFindStruct(pFind);

		return SYS_ERR_FILE_NOT_FOUND;
		}

	#endif


	if(!FindNextFile((HANDLE)hHandle,&sFind))			// find the next filr
		{
		SysClearFindStruct(pFind);
		return SysError();
		}

	strcpy(pFind->cName,sFind.cFileName);

	iAttr=(sFind.dwFileAttributes&FILE_ATTRIBUTE_READONLY)? SYS_READ:SYS_READ|SYS_WRITE;
	if(sFind.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)iAttr|=SYS_DIR;

	pFind->dwAttr = (sFind.nFileSizeHigh<<SYS_HIGHSHIFT)|iAttr;
	pFind->dwSize =  sFind.nFileSizeLow;

	SysTimeConvertFS(&sFind.ftCreationTime  ,&pFind->dwTimeCreate);
	SysTimeConvertFS(&sFind.ftLastWriteTime ,&pFind->dwTimeWrite);
	SysTimeConvertFS(&sFind.ftLastAccessTime,&pFind->dwTimeAccess);



return 0;
}


//*****************************************************************************
//*
//*		SysFindClose
//*
//*****************************************************************************
//	closes a SysFind handle
//	hHandle	: find handle
//	returns 0 if all okor an error number below zero
int	SysFindClose(SysFind hHandle)
{


	if(!hHandle)
		{
		return SYS_ERR_PARAM;
		}

	#if SYS_USE_GLOBAL_PATH

	if(IS_DRIVE_HANDLE(hHandle))						// Laufwerke auflisten
		{
		int		iSlot;

		   iSlot = NUM_OF_DRIVE_HANDLE(hHandle);
		if(iSlot>=SYS_DRIVE_SLOTS)
			{
			return SYS_ERR_PARAM;
			}

		aDriveFilter[iSlot*16] = 0;						// Slot freigeben
		return 0;
		}

	#endif

	if(!FindClose((HANDLE)hHandle))						// Such-Handle schlieen
		{
		return SysError();
		}


return 0;
}


//*****************************************************************************
//*
//*		SysMkdir
//*
//*****************************************************************************
//	create a new directory
//	pFilename	: is the name of the directory
//	return 0 if all ok or an error number below zero
int	SysMkdir(const char *pFilename)
{

	CHECK_GLOBAL_PATH(pFilename)

	if(CreateDirectory(pFilename,0))return 0;


return SysError();
}


//*****************************************************************************
//*
//*		SysRmdir
//*
//*****************************************************************************
//	removes a directory
//	pFilename	: is the name of the directory
//	return 0 if all ok or an error number below zero
int	SysRmdir(const char *pFilename)
{

	CHECK_GLOBAL_PATH(pFilename)

	if(RemoveDirectory(pFilename))return 0;


return SysError();
}


//*****************************************************************************
//*
//*		SysDelete
//*
//*****************************************************************************
//	delets a file
//	pFilename	: is the name of the file
//	return 0 if all ok or an error number below zero
int	SysDelete(const char *pFilename)
{

	CHECK_GLOBAL_PATH(pFilename)

	if(DeleteFile(pFilename))return 0;


return SysError();
}



//*****************************************************************************
//*
//*		SysRename
//*
//*****************************************************************************
//	renames file
//	pFilename	: is the name of the file
//	return 0 if all ok or an error number below zero
int SysRename(const char *pFilename,const char *pFilenameNew)
{

	CHECK_GLOBAL_PATH(pFilename)
	CHECK_GLOBAL_PATH(pFilenameNew)

	if(MoveFile(pFilename,pFilenameNew))return 0;


return SysError();
}


//*****************************************************************************
//*
//*		SysFileLength
//*
//*****************************************************************************
//	get the length of a file
//	pFilename	: is the name of the file
//	return the file length or an errornumber below zero
int SysFileLength(const char *pFilename)
{
WIN32_FILE_ATTRIBUTE_DATA sData;



	CHECK_GLOBAL_PATH(pFilename)
													// Existiert die Datei
	if(!GetFileAttributesEx(pFilename,GetFileExInfoStandard,&sData))
		{
		return SysError();
		}

	if(sData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)
		{
		return 0;
		}


return sData.nFileSizeLow;
}

//*****************************************************************************
//*
//*		SysFileLengthEx
//*
//*****************************************************************************
//	get the length of a file (64 bit version)
//	pFilename	: is the name of the file
//	return the file length or an errornumber below zero
SysPos SysFileLengthEx(const char *pFilename)
{
WIN32_FILE_ATTRIBUTE_DATA	sData;
SysPos						lSize;



	CHECK_GLOBAL_PATH(pFilename)
													// Existiert die Datei
	if(!GetFileAttributesEx(pFilename,GetFileExInfoStandard,&sData))
		{
		return SysError();
		}

	if(sData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)
		{
		return 0;
		}

	lSize	= sData.nFileSizeHigh;
	lSize <<= 32;
	lSize  |= sData.nFileSizeLow;


return lSize;
}


//*****************************************************************************
//*
//*		SysFileInfo
//*
//*****************************************************************************
//	gets informations of a file
//	pFilename	: is the name of the file
//	pInfo		: here the file info will be stored
//	returns 0 if ok or an errornumber below zero
int SysFileInfo(const char *pFilename,SysFindStruct *pInfo)
{
WIN32_FILE_ATTRIBUTE_DATA	sData;
int							iAttr;


	if(!pInfo)return SYS_ERR_PARAM;

	CHECK_GLOBAL_PATH(pFilename)
													// Existiert die Datei
	if(!GetFileAttributesEx(pFilename,GetFileExInfoStandard,&sData))
		{
		SysClearFindStruct(pInfo);
		return SysError();
		}

	pInfo->cName[0] = 0;

	iAttr=(sData.dwFileAttributes&FILE_ATTRIBUTE_READONLY)? SYS_READ:SYS_READ|SYS_WRITE;
	if(sData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)iAttr|=SYS_DIR;

	pInfo->dwAttr = (sData.nFileSizeHigh<<SYS_HIGHSHIFT)|iAttr;
	pInfo->dwSize =  sData.nFileSizeLow;

	SysTimeConvertFS(&sData.ftCreationTime  ,&pInfo->dwTimeCreate);
	SysTimeConvertFS(&sData.ftLastWriteTime ,&pInfo->dwTimeWrite );
	SysTimeConvertFS(&sData.ftLastAccessTime,&pInfo->dwTimeAccess);


return SYS_ERR_NONE;
}


//*****************************************************************************
//*
//*		SysFullPath
//*
//*****************************************************************************
//	converts a relative path to a full path
//	pFilename	: is the relative path
//	pBuffer		: is the buffer for the full path
//	iMax		: is the size of the buffer
//	returns the length in bytes of the full path name.
int SysFullPath(const char *pFilename,char *pBuffer,int iMax)
{

	CHECK_GLOBAL_PATH(pFilename)

	if(iMax<=0)return 0;

	pBuffer[0] = 0;


return GetFullPathName(pFilename,iMax,pBuffer,NULL);
}



//*****************************************************************************
//*
//*		SysFileOpenEx
//*
//*****************************************************************************
//	ffnet eine Datei.
//	pFilename		: Ist der Name der Datei.
//	iMode			: Bestimmt wie die Datei geffnet werden soll.
//						SYS_READ	= Datei zum Lesen ffnen
//						SYS_WRITE	= Datei zum Schreiben ffnen und nicht auf 0 krzen
//						SYS_CREATE	= Datei zum Schreiben ffnen und auf 0 krzen
//	pError			: Hier wird die Fehlernummer gespeichert wenn der Zeiger
//					  ungleich NULL ist.
//	Ergibt ein File-Handle bzw. 0 bei einem Fehler
SysFile SysFileOpenEx(const char *pFilename,int iMode,int *pError)
{
DWORD   dwAttribut=FILE_ATTRIBUTE_NORMAL;
DWORD   dwCreation=OPEN_EXISTING;
DWORD   dwAccess=0;
DWORD   dwShare=0;
HANDLE  hFile;



	CHECK_GLOBAL_PATH(pFilename)


    if(iMode&SYS_READ    )dwAccess  |=GENERIC_READ;
    if(iMode&SYS_WRITE   )dwAccess  |=GENERIC_WRITE;
    if(iMode&SYS_SYSTEM  )dwAttribut|=FILE_ATTRIBUTE_SYSTEM;
    if(iMode&SYS_HIDDEN  )dwAttribut|=FILE_ATTRIBUTE_HIDDEN;
    if(iMode&SYS_ARCHIVE )dwAttribut|=FILE_ATTRIBUTE_ARCHIVE;
    if(iMode&SYS_SHARE_RD)dwShare   |=FILE_SHARE_READ;
    if(iMode&SYS_SHARE_WR)dwShare   |=FILE_SHARE_WRITE;
    if(iMode&SYS_TRUNC   )
        {
        dwCreation = (iMode&SYS_CREATE)? CREATE_ALWAYS:TRUNCATE_EXISTING;
        }
    else{
        if(iMode&SYS_CREATE)dwCreation=OPEN_ALWAYS;
        }


    hFile = CreateFile(pFilename,dwAccess,dwShare,0,dwCreation,dwAttribut,0);

    if(hFile==INVALID_HANDLE_VALUE)
        {
		if(pError)*pError=SysError();
        return 0;
        }

	if(pError)*pError=SYS_ERR_NONE;


return (SysFile)hFile;
}

//*****************************************************************************
//*
//*		SysFileRead
//*
//*****************************************************************************
//	Liest Daten aus einer Datei
//	hFile		: File-Handle der Datei
//	pData		: Zeiger auf den Lesepuffer.
//	iSize		: Ist die Anzahl der zuladenden Bytes
//	Ergibt die Anzahl der geladenen Bytes bzw. einen Fehlercode kleiner 0.
int SysFileRead(SysFile hFile,void *pData,int iSize)
{
HANDLE      hHandle=(HANDLE)hFile;
DWORD       dwByte;


    if(!ReadFile(hHandle,pData,iSize,&dwByte,NULL))
        {
        return SysError();
        }

return dwByte;
}


//*****************************************************************************
//*
//*		SysFileWrite
//*
//*****************************************************************************
//	Speichert Daten in einer Datei
//	hFile		: File-Handle der Datei
//	pData		: Zeiger auf den Schreibpuffer.
//	iSize		: Ist die Anzahl der speichernden Bytes
//	Ergibt die Anzahl der geladenen Bytes bzw. einen Fehlercode kleiner 0.
int SysFileWrite(SysFile hFile,const void *pData,int iSize)
{
HANDLE      hHandle=(HANDLE)hFile;
DWORD       dwByte;


    if(!WriteFile(hHandle,pData,iSize,&dwByte,NULL))
        {
        return SysError();
        }



return dwByte;
}


//*****************************************************************************
//*
//*		SysFileClose
//*
//*****************************************************************************
//	Schliet eine geffnete Datei.
//	hFile		: File-Handle der Datei
//	Ergbibt 0 wenn die Datei geschlossen werden konnte, bzw einen Fehlercode.
int SysFileClose(SysFile hFile)
{
HANDLE      hHandle=(HANDLE)hFile;


    if(CloseHandle(hHandle))return 0;


return SysError();
}


//*****************************************************************************
//*
//*		SysFileSeek
//*
//*****************************************************************************
//	Verschiebt die Lese/Schreibposition in einer Datei
//	hFile		: File-Handle der Datei
//	iPos		: Neue Lese/Schreibposition
//	iMode		: Bestimmt wie die Position bestimmt werden soll.
//					SYS_SEEK_SET	0	Absolute Position
//					SYS_SEEK_CUR	1	Relativ zur aktuellen Position
//					SYS_SEEK_END	2	Relativ zum Datei Ende
//	Ergibt die neue Position in der Datei.
int SysFileSeek(SysFile hFile,int iPos,int iMode)
{
HANDLE      		hHandle=(HANDLE)hFile;
DWORD               dwPos;


    dwPos = SetFilePointer(hHandle,iPos,0,iMode);


return (dwPos==0xFFFFFFFF)? SysError():dwPos;
}

//*****************************************************************************
//*
//*		SysFileSeekEx
//*
//*****************************************************************************
//	Verschiet die Lese/Schreibposition in einer Datei
//	hFile		: File-Handle der Datei
//	iPos		: Neue Lese/Schreibposition
//	iMode		: Bestimmt wie die Position bestimmt werden soll.
//					SYS_SEEK_SET	0	Absolute Position
//					SYS_SEEK_CUR	1	Relativ zur aktuellen Position
//					SYS_SEEK_END	2	Relativ zum Datei Ende
//	Ergibt die neue Position in der Datei.
SysPos SysFileSeekEx(SysFile hHandle,SysPos iPos,int iMode)
{
DWORD               dwPos;
LONG				lHigh;


	lHigh = (LONG)(iPos>>32);
    dwPos = SetFilePointer(hHandle,(DWORD)iPos,&lHigh,iMode);

	if(dwPos==0xFFFFFFFF && GetLastError()!=NO_ERROR)
		{
		return SysError();
		}

	iPos	= lHigh;
	iPos  <<= 32;
	iPos   |= dwPos;


return iPos;
}

//*****************************************************************************
//*
//*     SysFileTell
//*
//*****************************************************************************/
//  return the file position or an error code below zero
int SysFileTell(SysFile hFile)
{
HANDLE      		hHandle=(HANDLE)hFile;
DWORD               dwPos;



    dwPos = SetFilePointer(hHandle,0,0,FILE_CURRENT);


return (dwPos==0xFFFFFFFF)? SysError():dwPos;
}

//*****************************************************************************
//*
//*     SysFileTellEx
//*
//*****************************************************************************/
//  return the file position or an error code below zero (64 bit version)
SysPos SysFileTellEx(SysFile hFile)
{
HANDLE      		hHandle=(HANDLE)hFile;
DWORD               dwPos;
LONG				lHigh;
SysPos				iPos;



	lHigh = 0;
    dwPos = SetFilePointer(hHandle,0,&lHigh,FILE_CURRENT);

	if(dwPos==0xFFFFFFFF && GetLastError()!=NO_ERROR)
		{
		return SysError();
		}


	iPos	= lHigh;
	iPos  <<= 32;
	iPos   |= dwPos;


return iPos;
}

//*****************************************************************************
//*
//*		SysFileFlush
//*
//*****************************************************************************
//	Leert alle Zwischenpuffer einer Datei.
//	hFile		: File-Handle der Datei
//	Ergbibt 1 wenn die Puffer gellert werden konnten, bzw. 0 bei einem Fehler.
int SysFileFlush(SysFile hFile)
{
HANDLE      hHandle=(HANDLE)hFile;



    if(FlushFileBuffers(hHandle))return 0;



return SysError();
}

//*****************************************************************************
//*
//*		SysSetCurrentPath
//*
//*****************************************************************************
//	Stellt fr den Prozess das aktuelle Verzeichnis ein
//	pFilename	: Ist der Zeiger auf den Verzeichnisnamen.
//				  Der Pfad kann auch '/' am Ende haben.
//	Ergibt 0 wenn ok, bzw eine Fehlernummer.
int SysSetCurrentPath(const char *pFilename)
{
char	cPath[260];
int		iLen;


	CHECK_GLOBAL_PATH(pFilename)


	   iLen = GetFullPathName(pFilename,sizeof(cPath),cPath,NULL);
	if(iLen<=0)return SysError();

	if(iLen>0 && (cPath[iLen-1]=='\\' || cPath[iLen-1]=='/'))
		{
		cPath[iLen-1]=0;
		}

	if(!SetCurrentDirectory(cPath))
		{
		return SysError();
		}


return SYS_ERR_NONE;
}


//*****************************************************************************
//*
//*		SysFileCopy
//*
//*****************************************************************************
//	Copyies a file from pSource to pTarget.
//	pSource			: Path of the source file
//	pTarget			: Path of the target file
//	iFailIfExists	: fails if the target file exists
//	Rertuns the file size, or an error number below zero.
int SysFileCopy(const char *pSource,const char *pTarget,int iFailIfExists)
{
char		cBuffer[4096];
SysFile		hSource;
SysFile		hTarget;
int			iError;
int			iPos;
int			iLen;
int			iNum;


	if(iFailIfExists && SysFileExist(pTarget)!=0)	// exists the target
		{
		return SYS_ERR_ALREADY_EXISTS;
		}

													// open files
		hSource = SysFileOpenEx(pSource,SYS_READ,&iError);
	if(!hSource)return iError;

		hTarget = SysFileOpenEx(pTarget,SYS_CREATE|SYS_WRITE,&iError);
	if(!hTarget)
		{
		SysFileClose(hSource);
		return iError;
		}


	iLen = sizeof(cBuffer);

	for(iPos=0;iLen==sizeof(cBuffer);iPos+=iLen)	// copy data
		{
		   iLen = SysFileRead(hSource,cBuffer,sizeof(cBuffer));
		if(iLen<=0){iError=iLen;break;}

		   iNum = SysFileWrite(hTarget,cBuffer,iLen);
		if(iNum<=0){iError=iNum;break;}
		if(iNum!=iLen)
			{
			iError = SYS_ERR_DISKFULL;
			break;
			}
		}


	SysFileClose(hSource);
	SysFileClose(hTarget);

	if(iError<0)									// all ok
		{
		SysDelete(pTarget);
		return iError;
		}


return iPos;
}



//*****************************************************************************
//*
//*     SysGetFileTime
//*
//*****************************************************************************/
//  Gets all times of a opened file
//	hFile		: handle of the file
//	pWrite		: here the write time is stored (NULL means not storing)
//	pCreate		: here the create time is stored (NULL means not storing)
//	pAccess		: here the last access time is stored (NULL means not storing)
//	Returns 0 if ok, or an error value below zero.
int SysGetFileTime(SysFile hFile,SysUtc *pWrite,SysUtc *pCreate,SysUtc *pAccess)
{
HANDLE      hHandle=(HANDLE)hFile;
FILETIME    sLastAccessTime;
FILETIME    sLastWriteTime;
FILETIME    sCreationTime;
 
 
 
    if(!GetFileTime(hHandle,&sCreationTime,&sLastAccessTime,&sLastWriteTime))
        {
        return SysError();
        }
 
 
    SysTimeConvertFS(&sLastWriteTime ,pWrite );
    SysTimeConvertFS(&sCreationTime  ,pCreate);
    SysTimeConvertFS(&sLastAccessTime,pAccess);
 
 
return 0;
}
 
 
//*****************************************************************************
//*
//*     SysSetFileTime
//*
//*****************************************************************************/
//  Sets all times of an opened file
//	hFile		: handle of the file
//	dwWrite		: it the new write time (0 means not changing)
//	dwCreate	: it the new create time (0 means not changing)
//	dwAccess	: it the new last access time (0 means not changing)
//	Returns 0 if ok, or an error value below zero.
int SysSetFileTime(SysFile hFile,SysUtc dwWrite,SysUtc dwCreate,SysUtc dwAccess)
{
HANDLE      hHandle=(HANDLE)hFile;
FILETIME    sLastAccessTime;
FILETIME    sLastWriteTime;
FILETIME    sCreationTime;
 
 
 
    if(!dwWrite && !dwCreate && !dwAccess)return 0;
 
 
    SysTimeConvertSF(dwWrite ,&sLastWriteTime );
    SysTimeConvertSF(dwCreate,&sCreationTime  );
    SysTimeConvertSF(dwAccess,&sLastAccessTime);
 
 
    if(!SetFileTime(hFile,(dwCreate)? &sCreationTime:NULL,(dwAccess)? &sLastAccessTime:NULL,&sLastWriteTime))
        {
        return SysError();
        }
 
 
 
return 0;
}

//*****************************************************************************
//*
//*		SysGetFileSize
//*
//*****************************************************************************
//	Gets the size of an opend file.
//	hFile		: handle of the file
//	Returns the size of the file, or an error value below zero.
int SysGetFileSize(SysFile hFile)
{
HANDLE      hHandle=(HANDLE)hFile;
DWORD		dwSize;



	   dwSize = GetFileSize(hHandle,NULL);
	if(dwSize==0xFFFFFFFF)
		{
		return SysError();
		}


return dwSize;
}

//*****************************************************************************
//*
//*		SysSetFileSize
//*
//*****************************************************************************
//	Sets the size of an opend file.
//	hFile		: handle of the file
//	uSize		: is the new file size
//	Returns the size of the file, or an error value below zero.
int SysSetFileSize(SysFile hFile,SysDW uSize)
{
HANDLE      hHandle=(HANDLE)hFile;
DWORD       dwPos;
int         iErr;
int         iOk;


    dwPos = SetFilePointer(hHandle,0,0,FILE_CURRENT);

    SetFilePointer(hFile,uSize,0,FILE_BEGIN);
    iOk=SetEndOfFile(hFile);

    if(!iOk)
        {
        iErr = SysError();
        }
    else{
		if(uSize>=0x80000000)uSize=0x7FFFFFFF;
        iErr = dwPos;
        }

    SetFilePointer(hFile,dwPos,0,FILE_BEGIN);



return iErr;
}


//*****************************************************************************
//*
//*     SysGetFileInfo
//*
//*****************************************************************************
//  Get informations of an opened file.
//	hFile	: handle of the file
//  pFind   : pointer to file structer
//  returns 0 if a file was found or an error code below zero
int SysGetFileInfo(SysFile hFile,SysFindStruct *pFind)
{
static int                  iDone=0;
HANDLE						hHandle;
BY_HANDLE_FILE_INFORMATION  sInfo;
IO_STATUS_BLOCK_            sBlock;
FILE_NAME_INFORMATION_      sData;
ULONG                       uRet;
ULONG                       uPos;
ULONG						uOut;
int                         iAttr;


    hHandle = (HANDLE)hFile;

    if(!GetFileInformationByHandle(hHandle,&sInfo))
        {
        return SysError();
        }

    if(hNtDll==0 && iDone<=0)
        {
        hNtDll = GetModuleHandle("NTDLL");
        iDone  = 1;
        }

    if(hNtDll && !pNtQueryInformationFile && iDone<=1)
        {
        pNtQueryInformationFile = (NtQueryInformationFileType)GetProcAddress(hNtDll,"NtQueryInformationFile");
        iDone  = 2;
        }


    sData.uFileNameLength   = 0;
    sData.aFileName[0]      = 0;

    if(pNtQueryInformationFile)
        {
        uRet = pNtQueryInformationFile(hHandle,&sBlock,&sData,sizeof(sData),9);   // FileNameInformation
        }

    uPos = (sData.aFileName[0]=='\\')? 1:0;
    sData.uFileNameLength /= 2;



    for(uOut=0;uOut<sizeof(pFind->cName)-1;uPos++,uOut++)
        {
        if(uPos>=sData.uFileNameLength)break;
        if(sData.aFileName[uPos]<255)
            {
            pFind->cName[uOut] = (char)sData.aFileName[uPos];
            }
        else{
            pFind->cName[uOut] = '\xFF';
            }
        }

    pFind->cName[uOut] = 0;

  



    iAttr=(sInfo.dwFileAttributes&FILE_ATTRIBUTE_READONLY)? SYS_READ:SYS_READ|SYS_WRITE;
    if(sInfo.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN   )iAttr|=SYS_HIDDEN;
    if(sInfo.dwFileAttributes&FILE_ATTRIBUTE_SYSTEM   )iAttr|=SYS_SYSTEM;
    if(sInfo.dwFileAttributes&FILE_ATTRIBUTE_ARCHIVE  )iAttr|=SYS_ARCHIVE;
    if(sInfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)iAttr|=SYS_DIR;

    pFind->dwAttr = iAttr;
    pFind->dwSize = sInfo.nFileSizeLow;

	SysTimeConvertFS(&sInfo.ftCreationTime  ,&pFind->dwTimeCreate);
	SysTimeConvertFS(&sInfo.ftLastWriteTime ,&pFind->dwTimeWrite);
	SysTimeConvertFS(&sInfo.ftLastAccessTime,&pFind->dwTimeAccess);



return 0;
}


