//*****************************************************************************
//*
//*
//*		FileUtility.h
//*
//*
//*****************************************************************************
//
//  This is a library fore file and directory access for WINDOWS,
//
//	Global Macros:
//	
//		GLOBAL_CONFIG			Enable the configuration via GlobalConfig.h header file (1=on 0=off)
//
//  Copyright  2018    Anton Zechner
//
//
#ifndef	  __FILEUTILITY_H__
#define   __FILEUTILITY_H__

#ifdef		GLOBAL_CONFIG
#define 	FILEUTILITY_H
#include	<GlobalConfig.h>
#undef		FILEUTILITY_H
#endif

#ifdef		_MSC_VER
#pragma     once
#pragma 	warning( disable :4035 )
#pragma 	warning( disable :4996 )
#pragma 	warning( disable :4514 )
#endif


#define 	SYS_ERR_NONE				 0
#define 	SYS_ERR_PARAM				-1
#define 	SYS_ERR_INVALID_FILENAME	-2
#define 	SYS_ERR_FILE_NOT_FOUND		-3
#define 	SYS_ERR_ACCESS_DENIED		-4
#define 	SYS_ERR_NO_HANDLES			-5
#define 	SYS_ERR_UNKNOWN				-6
#define 	SYS_ERR_PATH_NOT_FOUND		-7
#define 	SYS_ERR_DISKFULL			-8
#define		SYS_ERR_ALREADY_EXISTS		-9


// Constands for FileAttributes

#define     SYS_READ        		0x0001                          // read access for files
#define     SYS_WRITE       		0x0002                          // write access for files
#define     SYS_CREATE      		0x0004                          // create the file if it didn't exist
#define     SYS_TRUNC       		0x0008                          // set the file size to zero
#define     SYS_DIR         		0x0010                          // the file is a directory
#define     SYS_HIDDEN      		0x0020                          // the hidden flag for files
#define     SYS_ARCHIVE     		0x0040                          // the archive flag for files (set at writes)
#define     SYS_SYSTEM      		0x0080                          // the system flag for files
#define     SYS_VOLUME      		0x0100                          // the volume flag for disknames
#define     SYS_EXEC	    		0x0200                          // execute acces for the file
#define     SYS_SHARE_RD    		0x0400                          // read access for more than one file
#define     SYS_SHARE_WR    		0x0800                          // write access for more than one file
#define     SYS_HIGHMASK            0xFC00                          // mask  for the 64 bit size
#define     SYS_HIGHSHIFT           10                              // shift for the 64 bit size


// Constands for SysFileSeek
#define 	SYS_SEEK_SET			0
#define 	SYS_SEEK_CUR			1
#define 	SYS_SEEK_END			2


#ifdef	__cplusplus
extern "C" {
#endif

	typedef		unsigned char  *SysFile;
	typedef		unsigned       *SysFind;							// Handels for Find Functions
	typedef		unsigned		SysUtc;								// UCT time (seconds since 1.Jan.1970)
	typedef		unsigned		SysDW;								// Four byte unsigned type

	#ifdef _MSC_VER
	#pragma     once
	typedef		__int64			SysPos;								// Eight byte type for file positions
	#else
	typedef		long long 		SysPos;								// Eight byte type for file positions
	#endif

	typedef	struct
		{
		SysDW	dwAttr;												// file attributes and high size (see at SYS_???)
		SysDW	dwSize;												// file size (low 32 bits)
		SysUtc 	dwTimeCreate;										// creation time (sec since 1.Jan.1970)
		SysUtc	dwTimeAccess;										// las access    (sec since 1.Jan.1970)
		SysUtc	dwTimeWrite;										// last change   (sec since 1.Jan.1970)
		char	cName[256];											// filename
		}SysFindStruct;

	typedef struct
	    {
	    SysDW   dwFreeUnits;                            			// free clusters on disk
	    SysDW   dwTotalUnits;                           			// count of clusters on disk
	    SysDW   dwBytesPerSector;                       			// bytes per sector
	    SysDW   dwSectorsPerUnit;                       			// sectors per cluster
	    SysDW   dwSerialNumber;                         			// serial number of disk
	    char    cName  [16];                            			// name of disk
	    char    cFsType[16];                            			// file format on disk
	    }SysVolumeInfo;

	int		SysSetCurrentPath(const char *pFilename);
	int 	SysGetAttributes (const char *pFilename);
	int 	SysSetAttributes (const char *pFilename,SysDW dwAttr);

	SysPos 	SysFileLengthEx	 (const char *pFilename);
	int 	SysFileLength	 (const char *pFilename);
	int 	SysFileExist 	 (const char *pFilename);
	int		SysMkdir	 	 (const char *pFilename);
	int		SysRmdir	 	 (const char *pFilename);
	int		SysDelete	 	 (const char *pFilename);
	int		SysFileCopy		 (const char *pFilename,const char *pFilenameNew,int iFailIfExists);
	int		SysRename	 	 (const char *pFilename,const char *pFilenameNew);
	int		SysFullPath  	 (const char *pFilename,char *pBuffer,int iMax);
	int		SysFileInfo		 (const char *pFilename,SysFindStruct *pInfo);

	#define	SysFindFirst(n,f) SysFindFirstEx(n,f,0)
	SysFind	SysFindFirstEx 	 (const char *pFilename,SysFindStruct *pFind,int *pError);
	int		SysFindNext  	 (SysFind     hHandle  ,SysFindStruct *pFind);
	int		SysFindClose 	 (SysFind     hHandle);

    int     SysGetFileTime   (SysFile     hHandle  ,SysUtc *pWrite,SysUtc *pCreate,SysUtc *pAccess);
    int     SysSetFileTime   (SysFile     hHandle  ,SysUtc  uWrite,SysUtc  uCreate,SysUtc  uAccess);
	int     SysGetFileInfo   (SysFile	  hHandle  ,SysFindStruct *pFind);
	int     SysSetFileSize   (SysFile     hHandle  ,SysDW   uSize);
	int     SysGetFileSize   (SysFile     hHandle);

	#define	SysFileOpen(n,f)  SysFileOpenEx(n,f,0)
	SysFile SysFileOpenEx	 (const char *pFilename,int iMode,int *pError);
	int 	SysFileSeek  	 (SysFile     hFile,int 		iPos ,int iMode);
	SysPos 	SysFileSeekEx  	 (SysFile     hFile,SysPos 		iPos ,int iMode);
	int 	SysFileRead  	 (SysFile     hFile,      void *pData,int iSize);
	int 	SysFileWrite 	 (SysFile     hFile,const void *pData,int iSize);
	int     SysFileClose 	 (SysFile     hFile);
	int 	SysFileFlush 	 (SysFile	  hFile);
	int 	SysFileTell 	 (SysFile	  hFile);
	SysPos 	SysFileTellEx 	 (SysFile	  hFile);


#ifdef	__cplusplus
}
#endif

#endif
