//*****************************************************************************
//*
//*
//*		LibraryLibraryUtility.c
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"LibraryUtility.h"
#include	<string.h>
#include	<stdio.h>

//*****************************************************************************
//*
//*
//*		WINDOWS
//*
//*
//*****************************************************************************
#if defined(_WIN32) || defined(_WIN64)
#include	<windows.h>

#if _MSC_VER >= 1000
#pragma	warning( disable :4514 )
#pragma warning( disable :4996 )
#endif


//*****************************************************************************
//*
//*		LibraryLoad
//*
//*****************************************************************************
//	Loads a DLL with a specific name
//	pLibraryName	: is the name of the DLL
//	Returns a handle to the DLL or NULL if an error occurs
Library LibraryLoad(const char *pLibraryName)
{
char		cPath[256];
HMODULE		hDll;
int			iLen;
int			iPos;
int			iPtr;
int			iOff;


	   hDll=LoadLibrary(pLibraryName);
	if(hDll)return (Library)hDll;

	iPtr = -1;
	iLen = (int)strlen(pLibraryName);

	if(iLen+8>sizeof(cPath))return 0;


	for(iPos=iLen;iPos>0;iPos--)
		{
		if(pLibraryName[iPos]=='/' ){iPos++;break;}
		if(pLibraryName[iPos]=='\\'){iPos++;break;}
		if(pLibraryName[iPos]==':' ){iPos++;break;}
		if(pLibraryName[iPos]=='.' && iPtr<0)iPtr=iPos;
		}


	if(iLen>6 && !stricmp(pLibraryName+iLen-3,".so"))		// exchange ".so with ".dll"
		{
		iOff = iPos;
		memcpy(cPath,pLibraryName,iPos);
		if(!memicmp(pLibraryName+iOff,"lib",3))iOff+=3;
		memcpy(cPath+iPos,pLibraryName+iOff,iLen-iOff-2);
		memcpy(cPath+iPos+iLen-iOff-2,"dll",4);

		return (Library)LoadLibrary(cPath);
		}


	if(iPtr>=0)return 0;									// try with ".dll" extension

	iOff = iPos;
	memcpy(cPath,pLibraryName,iPos);
	if(!memicmp(pLibraryName+iOff,"lib",3))iOff+=3;
	memcpy(cPath+iPos,pLibraryName+iOff,iLen-iOff);
	memcpy(cPath+iPos+iLen-iOff,".dll",5);



return (Library)LoadLibrary(cPath);
}


//*****************************************************************************
//*
//*		LibraryProc
//*
//*****************************************************************************
//	Gets a pointer to a function of a DLL
//	hLibrary		: is the handle of the DLL
//	pProcnameName	: is a pointer to the name of the function
//	Returns a pointer to the function or NULL if an error occurs
void *LibraryProc(Library hLibrary,const char *pProcnameName)
{
return GetProcAddress((HMODULE)hLibrary,pProcnameName);
}

//*****************************************************************************
//*
//*		LibraryUnload
//*
//*****************************************************************************
//	Unload a DLL
//	hLibrary		: is the handle of the DLL
//	Returns 1 if the DLL was unloaded or 0 if an error occurs
int LibraryUnload(Library hLibrary)
{
return FreeLibrary((HMODULE)hLibrary);
}

#endif



//*****************************************************************************
//*
//*
//*		LINUX
//*
//*
//*****************************************************************************
//
//	Note:	Add -ldl to your makefile
//
#ifdef 	linux
#ifndef	LINUX
#define LINUX	1
#endif
#endif



#ifdef	LINUX

#include 	<unistd.h>
#include	<dlfcn.h>


//*****************************************************************************
//*
//*		LibraryGetExePath
//*
//*****************************************************************************
//	Get the path of the current exe file with a backslash at the end
//	pPath	: pBuffer for the pPath
//	iSize	: size of the pBuffer in bytes
//	Returns the size of the stored pPath
static int LibraryGetExePath(char *pPath,int iSize)
{
int		iPid;
int		iLen;
char	cPath[64];



	if(iSize<=0)return 0;

	iPid = getpid();
	sprintf(cPath,"/proc/%i/exe",iPid);

	iLen = readlink(cPath,pPath,iSize);

	if(iLen>=iSize)
		{
		pPath[0]=0;
		return 0;
		}

	if(iLen<=0)
		{
		pPath[0]=0;
		return 0;
		}

	for(;iLen>0;iLen--)						// remove filename from path
		{
		if(pPath[iLen-1]=='\\')break;
		if(pPath[iLen-1]=='/' )break;
		}

	pPath[iLen] = 0;


return iLen;
}

//*****************************************************************************
//*
//*		LibraryLoad
//*
//*****************************************************************************
//	Loads a shared object with a specific name
//	pLibraryName	: is the name of the shared object
//	Returns a handle to the shared object or NULL if an error occurs
Library LibraryLoad(const char *pLibraryName)
{
char		cPath[256];
char 		cTemp[256];
Library		hDll;
int			iLen;
int			iPos;
int			iPtr;
int			iOff;
int 		iAll;



	   hDll = (Library)dlopen(pLibraryName,RTLD_NOW);
	if(hDll)return hDll;

	iPtr = -1;
	iAll =  0;
	iLen =  strlen(pLibraryName);

	if(iLen+8>(int)sizeof(cPath))return 0;


	for(iPos=iLen;iPos>0;iPos--)							// find end of path
		{
		if(pLibraryName[iPos]=='/' ){iPos++;break;}
		if(pLibraryName[iPos]==':' ){iPos++;break;}
		if(pLibraryName[iPos]=='.' && iPtr<0)iPtr=iPos;
		}


	if(iPos==0)												// no path in filename
		{
		cPath[0] = '.';
		cPath[1] = '/';

		memcpy(cPath+2,pLibraryName,iLen+1);

		   hDll = (Library)dlopen(cPath,RTLD_NOW);			// try local with original name
		if(hDll)return hDll;

															// try EXE path with original name
		iAll = LibraryGetExePath(cTemp,sizeof(cTemp)-iLen-1);

		if(iAll>0)
			{
			memcpy(cTemp+iAll,cPath+2,iLen+1);
			hDll = (Library)dlopen(cTemp,RTLD_NOW);
			if(hDll)return hDll;
			}
		}


	if(iLen>4 && !strcasecmp(pLibraryName+iLen-4,".dll"))	// exchange ".dll" with ".so"
		{
		iLen -= 4;
		}
	else if(iLen>3 && !strcmp(pLibraryName+iLen-3,".so"))	// remove so
		{
		iLen -= 4;
		}


	iOff = iPos;
	memcpy(cPath,pLibraryName,iPos);

	if(memcmp(pLibraryName+iPos,"lib",3))					// add "lib" at the begin
		{
		memcpy(cPath+iPos,"lib",3);
		iOff+=3;
		}

	memcpy(cPath+iOff,pLibraryName+iPos,iLen-iPos);
	memcpy(cPath+iOff+iLen-iPos,".so",4);
	iLen = iOff+iLen-iPos+4;

	hDll = (Library)dlopen(cPath,RTLD_NOW);


	if(!hDll && iPos==0)									// try local path
		{
		memmove(cPath+2,cPath,iLen);

		cPath[0] = '.';
		cPath[1] = '/';

		hDll = (Library)dlopen(cPath,RTLD_NOW);

		if(!hDll && iAll)									// try EXE path
			{
			memcpy(cTemp+iAll,cPath+2,iLen+1);
			hDll = (Library)dlopen(cTemp,RTLD_NOW);
			}
		}



return hDll;
}


//*****************************************************************************
//*
//*		LibraryProc
//*
//*****************************************************************************
//	Gets a pointer to a function of a shared object
//	hLibrary		: is the handle of the shared object
//	pProcnameName	: is a pointer to the name of the function
//	Returns a pointer to the function or NULL if an error occurs
void *LibraryProc(Library hLibrary,const char *pProcnameName)
{
return dlsym(hLibrary,pProcnameName);
}

//*****************************************************************************
//*
//*		LibraryUnload
//*
//*****************************************************************************
//	Unload a shared object
//	hLibrary		: is the handle of the shared object
//	Returns 1 if the shared object was unloaded or 0 if an error occurs
int LibraryUnload(Library hLibrary)
{
return dlclose(hLibrary);
}

#endif



