//*****************************************************************************
//*
//*
//*		MacAddrUtility.c
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2010
//
#include	"MacAddrUtility.h"

#define		MAX_MAC_ADDR	16
static		unsigned char	aMacBuffer[MAX_MAC_ADDR*8];
static		unsigned 		uCardCount	= 0;
static		unsigned 		uInitDone	= 0;


//*****************************************************************************
//*
//*
//*		WINDOWS
//*
//*
//*****************************************************************************
#if defined(_WIN32) || defined(_WIN64)
#include	<windows.h>
#include	<malloc.h>
#include	<time.h>

#ifdef	   _MSC_VER
#pragma		warning( disable :4996 )
#endif


#define 	MIB_IF_TYPE_OTHER		1
#define 	MIB_IF_TYPE_ETHERNET 	6
#define 	MIB_IF_TYPE_TOKEN_RING 	9
#define 	MIB_IF_TYPE_FDDI 		15
#define 	MIB_IF_TYPE_PPP 		23
#define 	MIB_IF_TYPE_LOOPBACK 	24
#define 	MIB_IF_TYPE_SLIP 		28

#define		MAX_ADAPTER_NAME_LENGTH			256
#define 	MAX_ADAPTER_DESCRIPTION_LENGTH	128
#define 	MAX_ADAPTER_ADDRESS_LENGTH		8


typedef struct
	{
	char String[4 * 4];
	}IP_ADDRESS_STRING, *PIP_ADDRESS_STRING, IP_MASK_STRING, *PIP_MASK_STRING;


typedef struct _IP_ADDR_STRING
	{
	struct _IP_ADDR_STRING* Next;
	IP_ADDRESS_STRING		IpAddress;
	IP_MASK_STRING			IpMask;
	DWORD					Context;
	} IP_ADDR_STRING, *PIP_ADDR_STRING;


typedef struct _IP_ADAPTER_INFO
	{
	struct _IP_ADAPTER_INFO *Next;
	DWORD			ComboIndex;
	char			AdapterName[MAX_ADAPTER_NAME_LENGTH + 4];
	char 			Description[MAX_ADAPTER_DESCRIPTION_LENGTH + 4];
	UINT 			AddressLength;
	BYTE 			Address[MAX_ADAPTER_ADDRESS_LENGTH];
	DWORD 			Index;
	UINT 			Type;
	UINT 			DhcpEnabled;
	PIP_ADDR_STRING CurrentIpAddress;
	IP_ADDR_STRING	IpAddressList;
	IP_ADDR_STRING	GatewayList;
	IP_ADDR_STRING	DhcpServer;
	BOOL 			HaveWins;
	IP_ADDR_STRING	PrimaryWinsServer;
	IP_ADDR_STRING	SecondaryWinsServer;
	time_t 			LeaseObtained;
	time_t 			LeaseExpires;
	}IP_ADAPTER_INFO, *PIP_ADAPTER_INFO;



//*****************************************************************************
//*
//*		GetNameOfCard
//*
//*****************************************************************************
static int GetNameOfCard(const char *pName,char *pStr,int iBufferSize)
{
CHAR			cRegPath[256];
DWORD			dwType;
DWORD			dwSize;
HKEY			hKey;
int				iRet;


	strcpy(cRegPath,"SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}\\");
	strcat(cRegPath,pName);
	strcat(cRegPath,"\\Connection");
	pStr[0] = 0;

	if(!RegOpenKeyExA(HKEY_LOCAL_MACHINE,cRegPath,0,KEY_READ,&hKey)==ERROR_SUCCESS)
		{
		return 0;
		}

	dwSize = iBufferSize-2;
	dwType = REG_SZ;
	iRet   = 0;

	if(RegQueryValueExA(hKey,"Name",0,&dwType,(LPBYTE)pStr,&dwSize)==ERROR_SUCCESS)
		{
		pStr[dwSize]=0;
		iRet = dwSize;
		}

	RegCloseKey(hKey);


return iRet;
}

//*****************************************************************************
//*
//*		GetAdaptersInfo
//*
//*****************************************************************************
static DWORD GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo,PULONG pOutBufLen)
{
typedef DWORD (_stdcall *GetAdInfo)(PIP_ADAPTER_INFO,PULONG);
static	GetAdInfo	 pGetInfo =0;
static	HMODULE		 hIphlpApi=0;


	if(!hIphlpApi)
		{
		hIphlpApi=LoadLibrary("iphlpapi.dll");
		}

	if(!hIphlpApi)return ERROR_NOT_SUPPORTED;
	if(!pGetInfo)pGetInfo=(GetAdInfo)GetProcAddress(hIphlpApi,"GetAdaptersInfo");
	if(!pGetInfo)return ERROR_NOT_SUPPORTED;


return pGetInfo(pAdapterInfo,pOutBufLen);
}

//*****************************************************************************
//*
//*		ScanEthernetCards
//*
//*****************************************************************************
//	Auslesen der MAC-Addressen
static void ScanEthernetCards()
{
HKEY				hBase;
HKEY				hClass;
HKEY				hEntry;
DWORD				uBus;
DWORD				uType;
DWORD				uSize;
DWORD				uValue;
DWORD				uCardIdx;
DWORD				uBaseIdx;
DWORD				uClassIdx;
char				cName[128];
PIP_ADAPTER_INFO	aList[MAX_MAC_ADDR];
PIP_ADAPTER_INFO	pAlloc=0;
PIP_ADAPTER_INFO	pInfo=0;
unsigned long		ulLen=0;
unsigned			uCard=0;
unsigned			uOffset;
int					nError;



	memset(aMacBuffer,0,sizeof(aMacBuffer));
	uCardCount = 0;

	nError = GetAdaptersInfo(pInfo,&ulLen);

	if(nError==ERROR_BUFFER_OVERFLOW)
		{
		pInfo  = (PIP_ADAPTER_INFO)malloc(ulLen);
		nError = GetAdaptersInfo(pInfo,&ulLen);
		pAlloc = pInfo;
		}

	if(nError)
		{
		if(pAlloc)free(pAlloc);
		return;
		}


	while(pInfo)											// Suche Ethernet-Karten
		{
		if(pInfo->Type==MIB_IF_TYPE_ETHERNET)
			{
			if(uCardCount>=MAX_MAC_ADDR)break;

			memcpy(aMacBuffer+8*uCardCount,pInfo->Address,6);
			aMacBuffer[uCardCount*8+6] = uCardCount|0x20;
			aMacBuffer[uCardCount*8+7] = 1;
			aList     [uCardCount    ] = pInfo;

														   // Sortierung festlegen
			for(uOffset=0;pInfo->Description[uOffset];uOffset++)
				{
				if(pInfo->Description[uOffset]!='V')
				if(pInfo->Description[uOffset]!='v')continue;

															// VmWare an den Schluss
				if(!strnicmp(pInfo->Description+uOffset,"VMware",6))
					{
					aMacBuffer[uCardCount*8+6] |= 0xC0;
					}
															// Virtual an den Schluss
				if(!strnicmp(pInfo->Description+uOffset,"Virt",4))
					{
					aMacBuffer[uCardCount*8+6] |= 0x80;
					}
															// VLan an den Schluss
				if(!strnicmp(pInfo->Description+uOffset,"VLan",4))
					{
					aMacBuffer[uCardCount*8+6] |= 0x40;
					}
				}

			uCardCount++;
			}


		pInfo = pInfo->Next;
		}

															// Auf welchem Bus sind die Karten
	if(RegOpenKeyExA(HKEY_LOCAL_MACHINE,"System\\CurrentControlSet\\Control\\Class",0,KEY_READ,&hBase)==ERROR_SUCCESS)
		{
		for(uBaseIdx=0;uBaseIdx<256;uBaseIdx++)
			{
			if(RegEnumKeyA(hBase,uBaseIdx,cName,sizeof(cName))!=ERROR_SUCCESS)break;
			if(RegOpenKeyExA(hBase,cName,0,KEY_READ,&hClass  )==ERROR_SUCCESS)
				{
				uType = REG_SZ;
				uSize = sizeof(cName);

				if(RegQueryValueEx(hClass,"Class",NULL,&uType,(LPBYTE)cName,&uSize)!=ERROR_SUCCESS || stricmp(cName,"Net"))
					{
					RegCloseKey(hClass);
					continue;
					}

				for(uClassIdx=0;uClassIdx<256;uClassIdx++)
					{
					if(RegEnumKeyA(hClass,uClassIdx,cName,sizeof(cName))!=ERROR_SUCCESS)break;
					if(RegOpenKeyExA(hClass,cName,0,KEY_READ,&hEntry   )==ERROR_SUCCESS)
						{
						uBus  = 0;
						uType = REG_DWORD;
						uSize = sizeof(uBus);

						RegQueryValueEx(hEntry,"BusType",NULL,&uType,(LPBYTE)&uBus,&uSize);

						uType = REG_SZ;
						uSize = sizeof(cName);

						if(RegQueryValueEx(hEntry,"NetCfgInstanceId",NULL,&uType,(LPBYTE)cName,&uSize)==ERROR_SUCCESS)
							{
							for(uCardIdx=0;uCardIdx<uCardCount;uCardIdx++)
								{
								if(strcmp(cName,aList[uCardIdx]->AdapterName))continue;
								if(uBus=='5')aMacBuffer[uCardIdx*8+6]&=~0x20;
								break;
								}
							}

						RegCloseKey(hEntry);
						}
					}

				RegCloseKey(hClass);
				}
			}

		RegCloseKey(hBase);
		}



	if(pAlloc)free(pAlloc);


	for(uCardIdx=1;uCardIdx<uCardCount;uCardIdx++)			// Sortiere die Karten
		{
		if(aMacBuffer[uCardIdx*8+6]>aMacBuffer[uCardIdx*8+6-8])continue;

		uValue                             = *(DWORD*)(aMacBuffer+uCardIdx*8  );
		*(DWORD*)(aMacBuffer+uCardIdx*8  ) = *(DWORD*)(aMacBuffer+uCardIdx*8-8);
		*(DWORD*)(aMacBuffer+uCardIdx*8-8) = uValue;

		uValue                             = *(DWORD*)(aMacBuffer+uCardIdx*8+4);
		*(DWORD*)(aMacBuffer+uCardIdx*8+4) = *(DWORD*)(aMacBuffer+uCardIdx*8-4);
		*(DWORD*)(aMacBuffer+uCardIdx*8-4) = uValue;

		uCardIdx=0;
		}


}




#endif

//*****************************************************************************
//*
//*
//*		LINUX
//*
//*
//*****************************************************************************
#ifdef 	linux
#ifndef	LINUX
#define LINUX	1
#endif
#endif

#ifdef	LINUX


#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<memory.h>
#include	<sys/time.h>
#include	<sys/types.h>
#include	<sys/ioctl.h>
#include	<sys/socket.h>
#include	<arpa/inet.h>
#include	<linux/if_packet.h>
#include	<linux/if_ether.h>
#include	<linux/if_arp.h>


//******************************************************************************
//*
//*		ScanEthernetCards
//*
//******************************************************************************
//	Sucht nach den Ethernetkarten eth? und speichert die MAC Adressen
static void ScanEthernetCards()
{
FILE			   *pFile;
char 				cPath[64];
struct ifreq		sIfreq;
int					iSocket;
int					iIndex;
int					iCard;
int 				iVal;
int 				iPos;




	memset(aMacBuffer,0,sizeof(aMacBuffer));

	iSocket = socket(AF_PACKET,SOCK_RAW,htons(ETH_P_ALL));

	if(iSocket>=0)								// MAC's via Socket holen
		{
		for(iCard=0;iCard<MAX_MAC_ADDR;iCard++)
			{
			memset(&sIfreq,0,sizeof(sIfreq));
			sprintf(sIfreq.ifr_name,"eth%i",iCard);

			if(ioctl(iSocket,SIOCGIFINDEX,&sIfreq)<0)
				{
				sprintf(sIfreq.ifr_name,"Eth%i",iCard);

				if(ioctl(iSocket,SIOCGIFINDEX,&sIfreq)<0)
					{
					continue;
					}
				}

			iIndex = sIfreq.ifr_ifindex;

			if(ioctl(iSocket,SIOCGIFHWADDR,&sIfreq)<0)
				{
				continue;
				}

			memcpy(aMacBuffer+iCard*8,sIfreq.ifr_hwaddr.sa_data,6);
			aMacBuffer[iCard*8+7] = 1;
			uCardCount = iCard;
			}


		close(iSocket);
		}


	for(iCard=0;iCard<MAX_MAC_ADDR;iCard++)		// MAC's  von "/sys/class/net" holen
		{
		if(aMacBuffer[iCard*8+7])continue;

		sprintf(cPath,"/sys/class/net/eth%i/address",iCard);

			pFile = fopen(cPath,"rb");
		if(!pFile)
			{
			continue;
			}

		memset(cPath,0,33);
		fread (cPath,1,32,pFile);
		fclose(pFile);

		for(iPos=0;iPos<6;iPos++)				// MAC aus Text dekodieren
			{
			if(cPath[iPos*3+0]<='9' && cPath[iPos*3+0]>='0')
				{
				iVal = cPath[iPos*3+0]-'9';
				}
			else if(cPath[iPos*3+0]<='f' && cPath[iPos*3+0]>='a')
				{
				iVal = cPath[iPos*3+0]-'a'+10;
				}
			else if(cPath[iPos*3+0]<='F' && cPath[iPos*3+0]>='A')
				{
				iVal = cPath[iPos*3+0]-'A'+10;
				}
			else{
				break;
				}

			iVal <<= 4;

			if(cPath[iPos*3+1]<='9' && cPath[iPos*3+1]>='0')
				{
				iVal |= cPath[iPos*3+1]-'9';
				}
			else if(cPath[iPos*3+1]<='f' && cPath[iPos*3+1]>='a')
				{
				iVal |= cPath[iPos*3+1]-'a'+10;
				}
			else if(cPath[iPos*3+1]<='F' && cPath[iPos*3+1]>='A')
				{
				iVal |= cPath[iPos*3+1]-'A'+10;
				}
			else{
				break;
				}

			if(cPath[iPos*3+2]==0 && iPos<5)
				{
				break;
				}

			aMacBuffer[iCard*8+iPos] = iVal;
			}

		if(iPos>=6)
			{
			aMacBuffer[iCard*8+7] = 1;
			}

		if(uCardCount<(unsigned)iCard)
			{
			uCardCount = iCard;
			}
		}

}


#endif


//******************************************************************************
//*
//*		MacOfCard
//*
//******************************************************************************
//	Holt die MAC-Adresse von einer Ethernet-Karte
//	uCard		: Ist die Nummer der Karte (0=eth0)
//	Ergibt einen Zeiger auf die 6-Byte-MAC-Adresse, oder 0 wenn die Karte
//	nicht existiert.
unsigned char *MacOfCard(unsigned uCard)
{

	if(!uInitDone)
		{
		uInitDone=1;
		ScanEthernetCards();
		}


	if(uCard>=uCardCount || !aMacBuffer[uCard*8+7])
		{
		return 0;
		}



return aMacBuffer+uCard*8;
}


//******************************************************************************
//*
//*		MacGetCard
//*
//******************************************************************************
//	Sucht die Ethernet-Karte zu einer MAC-Adresse
//	pMac		: Zeiger auf die MAC-Adresse
//	Ergibt die Kartennummer oder -1 wenn die MAC nicht gefunden wurde.
int MacGetCard(const unsigned char *pMac)
{
int 	iCard;

	if(!uInitDone)
		{
		uInitDone=1;
		ScanEthernetCards();
		}


	for(iCard=uCardCount-1;iCard>=0;iCard--)
		{
		if(!memcmp(pMac,aMacBuffer+iCard*8,6))break;;
		}



return iCard;
}

//******************************************************************************
//*
//*		MacAsText
//*
//******************************************************************************
//	Holt die MAC-Adresse von einer Ethernet-Karte als Text.
//	pMac		: Zeiger auf die MAC-Adresse
//	Ergibt die Kartennummer oder -1 wenn die MAC nicht gefunden wurde.
//	Ergibt einen Zeiger auf die 6-Byte-MAC-Adresse, oder 0 wenn die Karte
//	nicht existiert.
unsigned char *MacAsText(unsigned uCard,char *pText,int iSize)
{
unsigned char  *pMac;
unsigned 		uOff;



	if(!uInitDone)
		{
		uInitDone=1;
		ScanEthernetCards();
		}


	if(uCard>=uCardCount || !aMacBuffer[uCard*8+7])
		{
		if(iSize>0)pText[0]=0;
		return 0;
		}


	pMac = aMacBuffer+uCard*8;


	for(uOff=0;uOff<6;uOff++)
		{
		if(iSize<4)break;

		pText[0] = "0123456789ABCDEF"[pMac[uOff]>>  4];
		pText[1] = "0123456789ABCDEF"[pMac[uOff]&0x0F];
		pText[2] = (uOff==5)? 0:'-';

		pText   += 3;
		iSize   -= 3;
		}

	if(iSize>0)pText[0]=0;


return pMac;
}



