//*****************************************************************************
//*
//*
//*		Mailslots.h
//*
//*
//*****************************************************************************
//
//  This is a library for a mailslot support for WINDOWS,FREERTOS,LINUX and RTTARGET.
//	Mailslots are thread save FIFO buffers
//	The elenent size and FIFO size could conigured individually.
//	Note: MailslotCreate use malloc, use this function only at startup
//
//	Global Macros:
//	
//		GLOBAL_CONFIG		Enable the configuration via GlobalConfig.h header file (1=on 0=off)
//		MAX_MAILSLOTS		Maximum count of mailslots
//
//
//  Copyright  2018    Anton Zechner
//
//
#ifndef		__MAILSLOTS_H__
#define 	__MAILSLOTS_H__

#ifdef		GLOBAL_CONFIG
#define 	MAILSLOTS_H
#include	<GlobalConfig.h>
#undef		MAILSLOTS_H
#endif

#ifdef		_MSC_VER
#pragma     once
#pragma 	warning( disable :4035 )
#pragma 	warning( disable :4996 )
#pragma 	warning( disable :4514 )
#endif

#ifdef		__cplusplus
extern		"C" {
#define 	D(a)	a
#else
#define 	D(a)
#endif



	typedef unsigned Mailslot;


	Mailslot	MailslotCreate (int iItemSize,int iQueueSize);
	void		MailslotDelete (Mailslot hMailslot);
	int			MailslotGetWait(Mailslot hMailslot,      void *pData,               unsigned uTimeout);
	int			MailslotPutWait(Mailslot hMailslot,const void *pData,unsigned uSize,unsigned uTimeout);
	int			MailslotPeek   (Mailslot hMailslot,      void *pData,unsigned uPosition              );
	int			MailslotClear  (Mailslot hMailslot,int iCount);
	int			MailslotCount  (Mailslot hMailslot);
	int			MailslotInfo   (Mailslot hMailslot,int *pItemSize D(=0),int *pSlotCount D(=0));

	#define 	MailslotGet(h,p)	MailslotGetWait(h,p,0)
	#define 	MailslotPut(h,p,s)	MailslotPutWait(h,p,s,0)




#ifdef	__cplusplus
}
#endif

#undef	D

#endif
