//*****************************************************************************
//*
//*
//*		MemFileUtility.c
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"MemFileUtility.h"





//*****************************************************************************
//*
//*
//*		WINDOWS
//*
//*
//*****************************************************************************
#if defined(_WIN32) || defined(_WIN64)

#include		<windows.h>

SYSTEM_INFO		sSysInfo;

//*****************************************************************************
//*
//*		MemMap
//*
//*****************************************************************************
//	Blendet eine Datei in einen Speicherbereich ein.
//	pFilename	: Ist der Name der Datei
//	uLength		: Anzahl der Bytes die eingeblendet werden sollen
//	uOffset		: Ist der Offset in der Datei ab der etwas eingeblendet werden soll.
//	uWrite		: Soll der eingeblendete Speicherbereiche beschreibbar sein.
//	Ergibt einen Zeiger auf den	Speicherbereich, bzw. 0 bei einem Fehler
unsigned char *MemMap(const char *pFilename,unsigned uLength,unsigned uOffset,unsigned uWrite)
{
SECURITY_DESCRIPTOR		sSecurity;
SECURITY_ATTRIBUTES		sAttrib;
DWORD					uAccess;
DWORD					uMode;
DWORD					uAdd;
HANDLE					hFile;
HANDLE					hMem;
LPVOID					pPtr;




	if(sSysInfo.dwPageSize==0)
		{
		GetSystemInfo(&sSysInfo);
		}


	sAttrib.nLength					= sizeof(sAttrib);		// create security descriptor
	sAttrib.lpSecurityDescriptor	= &sSecurity;
	sAttrib.bInheritHandle			= FALSE;

	InitializeSecurityDescriptor(&sSecurity,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl   (&sSecurity,TRUE,(PACL)NULL,FALSE);


	uAdd     = 	(sSysInfo.dwPageSize-1)&uOffset;
	uLength +=   uAdd;
	uOffset &= ~(sSysInfo.dwPageSize-1);

	uAccess	 = (uWrite)? GENERIC_READ|GENERIC_WRITE      :GENERIC_READ;
	uMode	 = (uWrite)? FILE_SHARE_READ|FILE_SHARE_WRITE:FILE_SHARE_READ;

	   hFile = CreateFile(pFilename,uAccess,uMode,&sAttrib,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hFile == INVALID_HANDLE_VALUE) 
		{
		return 0;
		}

	
	uAccess	= (uWrite)? PAGE_READWRITE:PAGE_READONLY;

		hMem = CreateFileMapping(hFile,&sAttrib,uAccess,0,uLength,NULL);
	if(!hMem)
		{
		CloseHandle(hFile);
		return 0;
		}

	
	uAccess	= (uWrite)? FILE_MAP_WRITE:FILE_MAP_READ;
	pPtr    = MapViewOfFile(hMem,uAccess,0,uOffset,uLength);

	CloseHandle(hFile);
	CloseHandle(hMem);


	if(!pPtr)return 0;



return (unsigned char*)pPtr+uAdd;
}


//*****************************************************************************
//*
//*		MemUnmap
//*
//*****************************************************************************
//	Blendet eine Datei aus einen Speicherbereich wieder aus.
//	pMemory		: Ist der Zeiger auf den Speicherbereich der ausgeblendet 
//				  werden soll.
//	Ergibt 1 wenn ok, bzw. 0 bei einem Fehler
int MemUnmap(void *pMemory)
{
unsigned char	*pPtr;


	if(!pMemory)return 0;

	pPtr  = (unsigned char*)pMemory;
	pPtr -=	((unsigned)pPtr)&(sSysInfo.dwPageSize-1);

	if(UnmapViewOfFile(pPtr))return 1;



return 0;
}


#endif 	


//*****************************************************************************
//*
//*
//*		LINUX
//*
//*
//*****************************************************************************
#ifdef 	linux
#ifndef	LINUX
#define LINUX	1
#endif
#endif


#ifdef	LINUX


#define MAX_MAPS	16

static	void	   *aMmfPtr[MAX_MAPS];	
static	unsigned	aMmfLen[MAX_MAPS];
static	unsigned	uPageSize=0;




#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sys/mman.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<errno.h>




//*****************************************************************************
//*
//*		MemMap
//*
//*****************************************************************************
//	Blendet eine Datei in einen Speicherbereich ein.
//	pFilename	: Ist der Name der Datei
//	uLength		: Anzahl der Bytes die eingeblendet werden sollen
//	uOffset		: Ist der Offset in der Datei ab der etwas eingeblendet werden soll.
//	uWrite		: Soll der eingeblendete Speicherbereiche beschreibbar sein.
//	Ergibt einen Zeiger auf den	Speicherbereich, bzw. 0 bei einem Fehler
unsigned char *MemMap(const char *pFilename,unsigned uLength,unsigned uOffset,unsigned uWrite)
{
unsigned 	uAdd;
int			iAccess;
int			iFile;
int			iPos;
void	   *pPtr;		 	



	
	iAccess = (uWrite)? O_RDWR:O_RDONLY;

	   iFile=open(pFilename,O_RDWR,S_IRUSR|S_IWUSR);
	if(iFile<0)return 0;


	if(!uPageSize)
		{
		uPageSize=getpagesize(); 
		}


	uAdd     = 	(uPageSize-1)&uOffset;
	uLength +=   uAdd;
	uLength +=  (uPageSize-1);
	uLength &= ~(uPageSize-1);
	uOffset &= ~(uPageSize-1);
	iAccess  =  (uWrite)? PROT_READ|PROT_WRITE:PROT_READ;
	
	pPtr = mmap(NULL,uLength,iAccess,MAP_SHARED,iFile,uOffset);
	close(iFile);

	if(MAP_FAILED==pPtr)return 0;
	

	for(iPos=0;iPos<MAX_MAPS;iPos++)
		{
		if(aMmfPtr[iPos]!=0)continue;

		aMmfPtr[iPos] = pPtr;
		aMmfLen[iPos] = uLength;

		return (unsigned char*)pPtr+uAdd;
		}


	munmap(pPtr,uLength);



return 0;
}

//*****************************************************************************
//*
//*		MemUnmap
//*
//*****************************************************************************
//	Blendet eine Datei aus einen Speicherbereich wieder aus.
//	pMemory		: Ist der Zeiger auf den Speicherbereich der ausgeblendet 
//				  werden soll.
//	Ergibt 1 wenn ok, bzw. 0 bei einem Fehler
int MemUnmap(void *pMemory)
{
unsigned char  *pPtr;
int				iPos;



	if(!pMemory)return 0;


	for(iPos=0;iPos<MAX_MAPS;iPos++)
		{
		if(aMmfPtr[iPos]!=pMemory)continue;

		pPtr  = (unsigned char*)pMemory;
		pPtr -=	((intptr_t)pPtr)&(uPageSize-1);
		
		if(munmap(pPtr,aMmfLen[iPos]))break;
		
		aMmfPtr[iPos] = 0;

		return 1;
		}


			

return 0;
}



#endif 	









