//*****************************************************************************
//*
//*
//*		ProcessUtility.c
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"ProcessUtility.h"






//*****************************************************************************
//*
//*
//*		WINDOWS
//*
//*
//*****************************************************************************
#if defined(_WIN32) || defined(_WIN64)
#include	<stdio.h>
#include	<string.h>
#include	<windows.h>
#include	<tlhelp32.h>

#if			_MSC_VER >= 1000
#pragma		once
#pragma		warning( disable :4514 )
#pragma		warning( disable :4996 )
#endif

#ifndef		TH32CS_SNAPMODULE32
#define		TH32CS_SNAPMODULE32					0x00000010
#endif

#ifndef		PROCESS_QUERY_LIMITED_INFORMATION
#define		PROCESS_QUERY_LIMITED_INFORMATION	0x00001000
#endif

//*****************************************************************************
//*
//*		ProcessFileName
//*
//*****************************************************************************
static DWORD WINAPI ProcessFileName(unsigned uId,LPTSTR pFilename,int iSize)
{
typedef BOOL  (_stdcall *QueryFullProcessImageNameType)(HANDLE,DWORD,LPTSTR,LPDWORD);
typedef DWORD (_stdcall *GetModuleFileNameExType      )(HANDLE,HMODULE,LPTSTR,DWORD);
static GetModuleFileNameExType			pGetN=0;
static QueryFullProcessImageNameType	pProc=0;
static BOOL								bDone=0;
DWORD									dwByte;
HMODULE									hExe;
HMODULE									hDll;



	if(iSize<=0)return 0;

	if(!pProc && !bDone)
		{
		bDone = 1;

		   hDll = GetModuleHandle("KERNEL32");
		if(hDll)
			{											// Exist only on Vista and higher
			pProc = (QueryFullProcessImageNameType)GetProcAddress(hDll,"QueryFullProcessImageNameA");
			}

		   hDll = GetModuleHandle("Psapi.dll");
		if(hDll)
			{											// Exist only on Vista and higher
			pGetN = (GetModuleFileNameExType)GetProcAddress(hDll,"GetModuleFileNameExA");
			}
		}


		hExe = OpenProcess(PROCESS_QUERY_INFORMATION,FALSE,uId);
	if(!hExe)
		{
			hExe = OpenProcess(PROCESS_QUERY_LIMITED_INFORMATION,FALSE,uId);
		if(!hExe)
			{
			return 0;
			}
		}


	dwByte = iSize;
	iSize  = 0;

	if(pProc && pProc(hExe,0,pFilename,&dwByte))		// Get name via "QueryFullProcessImageName"
		{
		iSize =  dwByte;
		}
	else if(pGetN)
		{
		iSize =  pGetN(hExe,0,pFilename,dwByte);		// Get name via "GetModuleFileNameEx"
		if(iSize<=0)pFilename[0]=0;
		}
	else{
		pFilename[0] = 0;
		}


	CloseHandle(hExe);


return iSize;
}

//*****************************************************************************
//*
//*		ProcessExists
//*
//*****************************************************************************
//	Checks if an process is running
//	uId	: is the process id of the process which should be checked
//	Returns 1 if the process is running otherwise 0
int	ProcessExists(unsigned uId)
{
HANDLE			hProcess;
DWORD			uCode;
UINT			uRet;


	if(uId==0)return 0;

		hProcess = OpenProcess(SYNCHRONIZE|PROCESS_QUERY_INFORMATION,FALSE,uId);
	if(!hProcess)
		{
			hProcess = OpenProcess(SYNCHRONIZE|PROCESS_QUERY_LIMITED_INFORMATION,FALSE,uId);
		if(!hProcess)return 0;
		}

	uRet = 0;

	if(!GetExitCodeProcess(hProcess,&uCode))
		{
		uRet = 1;
		}
	else if(uCode==STILL_ACTIVE)
		{
		uRet = 1;
		}

	CloseHandle(hProcess);


return uRet;
}

//*****************************************************************************
//*
//*		ProcessGetExePath
//*
//*****************************************************************************
//	Get the path of the current exe file with a backslash at the end
//	pPath	: buffer for the path
//	iSize	: size of the pBuffer in bytes
//	Returns the size of the stored path
int ProcessGetExePath(char *pPath,int iSize)
{
int		iLen;

	if(iSize<=0)return 0;


		iLen=GetModuleFileName(NULL,pPath,iSize);
	if(!iLen)
		{
		pPath[0]=0;
		return 0;
		}

	for(;iLen>0;iLen--)						// remove filename from path
		{
		if(pPath[iLen-1]=='\\')break;
		if(pPath[iLen-1]=='/' )break;
		}

	pPath[iLen] = 0;


return iLen;
}

//*****************************************************************************
//*
//*		ProcessGetExeName
//*
//*****************************************************************************
//	Get the path of the current exe without the EXE suffix
//	pPath	: buffer for the path (the *.exe suffix isn't stored)
//	iSize	: size of the pBuffer in bytes
//	Returns the size of the stored path
int ProcessGetExeName(char *pPath,int iSize)
{
int		iLen;


	if(iSize<=0)return 0;


		iLen=GetModuleFileName(NULL,pPath,iSize);
	if(!iLen)
		{
		pPath[0]=0;
		return 0;
		}

	if(iLen>4 && !stricmp(pPath+iLen-4,".exe"))	// remove *.exe suffix
		{
		iLen-=4;
		pPath[iLen]=0;
		}

return iLen;
}


//*****************************************************************************
//*
//*		ProcessRunningOwn
//*
//*****************************************************************************
//	Returns 1 if a other instance of the currend process is running
int	ProcessRunningOwn()
{
DWORD			dwId;
HANDLE			hSnapshot;
HANDLE			hModules;
PROCESSENTRY32	sProcess;
MODULEENTRY32	sModule;
char			cBuffer[MAX_PATH];
char		   *pName;
int				iLen,iRet=0;



	iLen = GetModuleFileName(NULL,cBuffer,sizeof(cBuffer));
	dwId = GetCurrentProcessId();

	for(;iLen>0;iLen--)
		{
		if(cBuffer[iLen-1]=='\\')break;
		if(cBuffer[iLen-1]=='/' )break;
		}

	pName           = cBuffer+iLen;
	hSnapshot       = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS|TH32CS_SNAPMODULE32,0);
	sProcess.dwSize = sizeof(sProcess);


	if(!hSnapshot || hSnapshot==INVALID_HANDLE_VALUE)
		{
		return 0;
		}

	if(!Process32First(hSnapshot,&sProcess))
		{
		CloseHandle(hSnapshot);
		return 0;
		}

	do	{
		if(sProcess.th32ProcessID==dwId)continue;	// ignore the own process

		if(strnicmp(pName,sProcess.szExeFile,15))	// compare process name
			{
			continue;
			}


		if(!ProcessFileName(sProcess.th32ProcessID,sModule.szExePath,sizeof(sModule.szExePath))>0)
			{
			hModules = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE|TH32CS_SNAPMODULE32,sProcess.th32ProcessID);

			if(!hModules || hModules==INVALID_HANDLE_VALUE)
				{
				continue;
				}

			sModule.dwSize = sizeof(sModule);

			if(!Module32First(hModules,&sModule))
				{
				CloseHandle(hModules);
				continue;
				}

			do	{
				if(stricmp(cBuffer,sModule.szExePath))	// compare full name
					{
					continue;
					}

				iRet=1;
				break;
				}while(Module32Next(hModules,&sModule));


			CloseHandle(hModules);
			}
		else{
			if(!stricmp(cBuffer,sModule.szExePath))		// compare name
				{
				iRet=1;
				}
			}

		if(iRet)break;


		}while(Process32Next(hSnapshot,&sProcess));

	CloseHandle(hSnapshot);


return iRet;
}


//*****************************************************************************
//*
//*		ProcessExeNameFormId
//*
//*****************************************************************************
//	Gets the path of the exe file from a process id.
//	uId	: is the process id
//	pPath	: here the path of the process is stored (without the *.exe suffix)
//	iSize	: size of the path pBuffer in bytes
//	Returns the size of the path or 0 if an error occurs
int	ProcessExeNameFormId(unsigned uId,char *pPath,int iSize)
{
HANDLE			hModules;
MODULEENTRY32	sModule;
int				iLen;


	if(iSize<=0)return 0;
	if(uId  ==0)return 0;


	if(!ProcessFileName(uId,sModule.szExePath,sizeof(sModule.szExePath)))
		{
		hModules       = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE|TH32CS_SNAPMODULE32,uId);
		sModule.dwSize = sizeof(sModule);

		if(!hModules || hModules==INVALID_HANDLE_VALUE)
			{
			pPath[0] = 0;
			return 0;
			}

		if(!Module32First(hModules,&sModule))
			{
			CloseHandle(hModules);
			pPath[0] = 0;
			return 0;
			}

		CloseHandle(hModules);
		}


	if(!strnicmp(sModule.szExePath,"\\SystemRoot\\",12))
		{
		iLen=GetWindowsDirectory(pPath,iSize);

		if(!iLen || iLen+2>=iSize)
			{
			pPath[0] = 0;
			return 0;
			}

		strncpy(pPath+iLen,sModule.szExePath+11,iSize-iLen);
		pPath[iSize-1]=0;

		iLen=strlen(pPath);

		if(iLen>4 && !stricmp(pPath+iLen-4,".exe"))	// remove *.exe suffix
			{
			iLen-=4;
			pPath[iLen]=0;
			}

		return iLen;
		}


	if(sModule.szExePath[0]=='\\' &&
	   sModule.szExePath[1]=='?'  &&
	   sModule.szExePath[2]=='?'  &&
	   sModule.szExePath[3]=='\\' &&
	   sModule.szExePath[5]==':')
		{
		strncpy(pPath,sModule.szExePath+4,iSize);
		}
	else{
		strncpy(pPath,sModule.szExePath,iSize);
		}

	pPath[iSize-1]=0;
	iLen=strlen(pPath);

	if(iLen>4 && !stricmp(pPath+iLen-4,".exe"))	// remove *.exe suffix
		{
		iLen-=4;
		pPath[iLen]=0;
		}


return iLen;
}

//*****************************************************************************
//*
//*		ProcessEnum
//*
//*****************************************************************************
//	Enumberates all running processes.
//	uId	: is the process id where the enumberation starts (0 for the first)
//	pPath	: here the path of the process is stored (if NULL none is stored)
//	iSize	: size of the path pBuffer in bytes
//	Returns the proccess id of the next process or 0 if no more processe where
//	found.
unsigned ProcessEnum(unsigned uId,char *pPath,int iSize)
{
static HANDLE	hSnapshot=0;
PROCESSENTRY32	sProcess;



	if(hSnapshot && !uId)							// close last enumeration
		{
		CloseHandle(hSnapshot);
		hSnapshot=0;
		}


	if(!hSnapshot)									// create new snapshot
		{
		hSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS|TH32CS_SNAPMODULE32,0);

		if(!hSnapshot || hSnapshot==INVALID_HANDLE_VALUE)
			{
			if(pPath && iSize>0)pPath[0]=0;
			return 0;
			}


		sProcess.dwSize = sizeof(sProcess);

		if(!Process32First(hSnapshot,&sProcess))
			{
			if(pPath && iSize>0)pPath[0]=0;
			CloseHandle(hSnapshot);
			hSnapshot=0;
			return 0;
			}
		}
	else{
		sProcess.dwSize = sizeof(sProcess);
		if(!Process32Next(hSnapshot,&sProcess))
			{
			if(pPath && iSize>0)pPath[0]=0;
			CloseHandle(hSnapshot);
			hSnapshot=0;
			return 0;
			}
		}


	if(!strnicmp("[System Process]",sProcess.szExeFile,15))
		{
		return ProcessEnum(1,pPath,iSize);
		}

	if(!strnicmp("System",sProcess.szExeFile,15))
		{
		return ProcessEnum(1,pPath,iSize);
		}


	if(pPath)										// get path of the process
		{
		ProcessExeNameFormId(sProcess.th32ProcessID,pPath,iSize);
		}


return sProcess.th32ProcessID;
}



//*****************************************************************************
//*
//*		ProcessKill
//*
//*****************************************************************************
//	Terminates the process with the specific process id
//	uId	: is the process id
//	Returns 1 if the process was killed or 0 if an error ocurrs
int	ProcessKill(unsigned uId)
{
HANDLE	hProcess;
int		iRet;


		hProcess = OpenProcess(PROCESS_TERMINATE,FALSE,uId);
	if(!hProcess)return 0;

	iRet = (TerminateProcess(hProcess,0))? 1:0;

	CloseHandle(hProcess);


return iRet;
}


//*****************************************************************************
//*
//*		ProcessIdFromExeName
//*
//*****************************************************************************
//	Find the process id to a path of a process.
//	If the process didn't have a *.EXE suffix this suffix is added.
//	pPath	: Is the path of the process
//	Returns the process id or 0 if no runnin process was found
unsigned ProcessIdFromExeName(const char *pPath)
{
int				iLen,iPos;
int				iSufix=-1;
HANDLE			hSnapshot;
PROCESSENTRY32	sProcess;
char			cName   [MAX_PATH];
char			cBuffer [MAX_PATH];
char			cCompare[MAX_PATH];
unsigned		uRet=0;



	iLen = strlen(pPath);

	for(iPos=iLen;iPos>0;iPos--)				// find suffix and filename
		{
		if(iSufix<0 && pPath[iPos]=='.')iSufix=iPos;
		if(pPath[iPos-1]=='\\')break;
		if(pPath[iPos-1]=='/' )break;
		}


	if(iLen-iPos+6>sizeof(cName))				// buffer overrun ?
		{
		return 0;
		}


	strcpy(cName,pPath+iPos);
	if(iSufix<0)strcat(cName,".exe");
	else		cName[iSufix-iPos]=0;

	if(iPos>0)									// create full path name
		{
		int	i,iMax;

		iMax=GetFullPathName(pPath,sizeof(cBuffer)-5,cBuffer,NULL);
		for(i=0;i<iMax;i++)
			{
			if(cBuffer[i]=='/')cBuffer[i]='\\';
			}

		if(iMax>4 && !stricmp(cBuffer+iMax-4,".exe"))
			{
			iMax-=4;
			cBuffer[iMax]=0;
			}
		}



	hSnapshot       = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS|TH32CS_SNAPMODULE32,0);
	sProcess.dwSize = sizeof(sProcess);

	if(!hSnapshot || hSnapshot==INVALID_HANDLE_VALUE)
		{
		return 0;
		}

	if(!Process32First(hSnapshot,&sProcess))
		{
		CloseHandle(hSnapshot);
		return 0;
		}


	do	{
		if(strnicmp(cName     ,sProcess.szExeFile,15))
		if(strnicmp(pPath+iPos,sProcess.szExeFile,15))
			{
			continue;
			}

		if(iPos>0)
			{
			ProcessExeNameFormId(sProcess.th32ProcessID,cCompare,sizeof(cCompare));
			if(stricmp(cCompare,cBuffer))continue;
			}

		uRet=sProcess.th32ProcessID;
		break;

		}while(Process32Next(hSnapshot,&sProcess));

	CloseHandle(hSnapshot);


return uRet;
}

//*****************************************************************************
//*
//*		ProcessOwnId
//*
//*****************************************************************************
//	Returns the own process id
unsigned ProcessOwnId()
{

return GetCurrentProcessId();
}

//*****************************************************************************
//*
//*		ProcessStart
//*
//*****************************************************************************
//	Starts a new process
//	iWait	: wait until the process is finished
//	pPath	: name of the process file, the path must not contain a *.exe suffix
//	pArgs	: aruments for the process (NULL for no args)
//	pError	: if this pointer isn't zero, an error code will be stored
//				0 = process succefull started
//				1 = file not found
//				2 = error at starting process
//				3 = filename too long
//				4 = file hasn't an execute prermission
//				5 = argument string too long
//	pWorkingDir	: is the working directory for the process, or NULL for default
//	pExitCode	: here the exit code of the proccess will be stored
//				  if this pointer is zero nothing will be stored
//	Returns the process id of started processes or 0 if an error ocurrs
unsigned ProcessStart(int iWait,const char *pPath,const char *pArgs,int *pError,const char *pWorkingDir,int *pExitCode)
{
int						iOk,iLen;
char					cArgs[1024];
char					cPath[MAX_PATH];
DWORD					dwExit;
STARTUPINFO				sSi;
PROCESS_INFORMATION		sPi;



	if(pExitCode)*pExitCode=-1;

	cPath[0] = 0;
	iLen=GetFullPathName(pPath,sizeof(cPath),cPath,NULL);
	if(iLen+5>=sizeof(cPath))
		{
		if(pError)*pError=3;
		return 0;
		}

	strcpy(cPath+iLen,".exe");


	if(GetFileAttributes(cPath)==0xFFFFFFFF)	// exists the file
		{
		cPath[iLen]=0;

		if(GetFileAttributes(cPath)==0xFFFFFFFF)
			{
			if(pError)*pError=1;
			return 0;
			}
		}

	if(!pArgs)pArgs="";

	if(strlen(pArgs)+strlen(pArgs)+4>sizeof(cArgs))
		{
		if(pError)*pError=5;
		return 0;
		}

	sprintf(cArgs,"\"%s\" %s",cPath,pArgs);

	memset(&sSi,0,sizeof(sSi));
	memset(&sPi,0,sizeof(sPi));
	sSi.cb       =sizeof(sSi);

		iOk=CreateProcess(cPath,cArgs,NULL,NULL,FALSE,CREATE_DEFAULT_ERROR_MODE,NULL,pWorkingDir,&sSi,&sPi);
	if(!iOk)
		{
		if(pError)
			{
			*pError=(GetLastError()==ERROR_ACCESS_DENIED)? 4:2;
			}
		return 0;
		}

	if(!sPi.hProcess)
		{
		if(pError)*pError=2;
		return 0;
		}

	if(iWait)									// wait until process has finished
		{
		WaitForSingleObject(sPi.hProcess,INFINITE);

		if(pExitCode && GetExitCodeProcess(sPi.hProcess,&dwExit))
			{
			*pExitCode = dwExit;
			}
		}

	CloseHandle(sPi.hProcess);
	CloseHandle(sPi.hThread);

	if(pError)*pError=0;


return sPi.dwProcessId;
}

#endif

//*****************************************************************************
//*
//*
//*		LINUX
//*
//*
//*****************************************************************************
#ifdef 	linux
#ifndef	LINUX
#define LINUX	1
#endif
#endif

#ifdef	LINUX

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/types.h>

#ifndef __USE_GNU
#define environ __environ
#endif


static	char  	cSignalDone;

//******************************************************************************
//*
//*		SignalChildHandler
//*
//******************************************************************************
//	Signal handle for SIGCHLD (end of schild process), prevent us from zombies
static void SignalChildHandler(int iSignal)
{
int 	iState;
int 	iPid;


    for(;;)
    	{
		   iPid = waitpid(-1,&iState,WNOHANG);
		if(iPid<=0)
			{
	    	signal(SIGCHLD,SignalChildHandler);		// Reinstall signal handler, since Linux resets to default after
	    	break;
			}
    	}

}


//*****************************************************************************
//*
//*		make_fullpath
//*
//*****************************************************************************
//	Get the full path of a relative path
//	pBuffer		: is the target buffer for the full path
//	pPath		: is the source path
//	iMax		: is the size of the target buffer in bytes
//	Returns the size of the new path
static int make_fullpath(char *pBuffer,const char *pPath,int iMax)
{
char	*pStr;
int		 iLen,iSlash;
int		 iPos=0,iAdd;



	if(iMax<=0)return 0;

	pBuffer[0]=0;
	iLen=strlen(pPath);
	if(iLen+1>=iMax)return 0;

	if(pPath[0]!='/')
		{
		pStr=getcwd(pBuffer,iMax);
	    if(pStr==NULL)return 0;
    	iPos=strlen(pBuffer);
		if(iPos+1+iLen>=iMax)return 0;
		pBuffer[iPos]='/';
		iPos++;
		memcpy(pBuffer+iPos,pPath,iLen+1);
		iLen+=iPos;
		}
	else{
		memcpy(pBuffer,pPath,iLen+1);
		iPos=1;
		}


	while(iPos<iLen)
		{
		if(iPos>0 && pBuffer[iPos]=='/')				// filter "//"
			{
			memmove(pBuffer+iPos,pBuffer+iPos+1,iLen-iPos);
			iLen-=1;
			continue;
			}
														// filter "./"
		if(iPos>0 && pBuffer[iPos]=='.' && pBuffer[iPos+1]=='/')
			{
			memmove(pBuffer+iPos,pBuffer+iPos+2,iLen-iPos-1);
			iLen-=2;
			continue;
			}
														// filter "."
		if(iPos>1 && pBuffer[iPos]=='.' && pBuffer[iPos+1]==0)
			{
			iPos--;
			pBuffer[iPos]=0;
			iLen=iPos;
			break;
			}

														// filter "../"
		if(iPos>0 && pBuffer[iPos]=='.' && pBuffer[iPos+1]=='.' && pBuffer[iPos+2]=='/')
			{
			iSlash=iPos;

			for(iPos--;iPos>0;iPos--)
				{
				if(pBuffer[iPos-1]=='/')break;
				}

			iAdd = iSlash-iPos+3;

			memmove(pBuffer+iPos,pBuffer+iSlash+3,iLen-iSlash-2);
			iLen-=iAdd;
			continue;
			}
														// filter ".."
		if(iPos>1 && pBuffer[iPos]=='.' && pBuffer[iPos+1]=='.' && pBuffer[iPos+2]==0)
			{
			for(iPos-=2;iPos>0;iPos--)
				{
				if(pBuffer[iPos]=='/')break;
				}

			if(iPos<=0)
				{
				pBuffer[0]='/';
				iPos=1;
				}

			pBuffer[iPos]=0;
			iLen=iPos;
			break;
			}

		for(;pBuffer[iPos];iPos++)						// ton nect slash
			{
			if(pBuffer[iPos]!='/')continue;
			iPos++;
			break;
			}
		}



return iLen;
}

//*****************************************************************************
//*
//*		ProcessExists
//*
//*****************************************************************************
//	Checks if an process is running
//	uId	: is the process id of the process which should be checked
//	Returns 1 if the process is running otherwise 0
int	ProcessExists(unsigned uId)
{
struct stat 	sStat;
char			cPath[64];



	if(uId==0)return 0;

	sprintf(cPath,"/proc/%i/exe",uId);


	if(stat(cPath,&sStat)==-1)
		{
		return 0;
		}



return 1;
}

//*****************************************************************************
//*
//*		ProcessGetExeName
//*
//*****************************************************************************
//	Get the path of the current exe without the EXE suffix
//	pPath	: buffer for the path
//	iSize	: size of the pBuffer in bytes
//	Returns the size of the stored path
int ProcessGetExeName(char *pPath,int iSize)
{
int		iPid;
int		iLen;
char	cPath[64];



	if(iSize<=0)return 0;

	iPid = getpid();
	sprintf(cPath,"/proc/%i/exe",iPid);

	iLen = readlink(cPath,pPath,iSize);

	if(iLen>=iSize || iLen<=0)
		{
		pPath[0]=0;
		return 0;
		}

	pPath[iLen]=0;



return iLen;
}


//*****************************************************************************
//*
//*		ProcessGetExePath
//*
//*****************************************************************************
//	Get the path of the current exe file with a backslash at the end
//	pPath	: buffer for the path
//	iSize	: size of the pBuffer in bytes
//	Returns the size of the stored path
int ProcessGetExePath(char *pPath,int iSize)
{
int		iLen;


	if(iSize<=0)return 0;


		iLen=ProcessGetExeName(pPath,iSize);
	if(!iLen)return 0;


	for(;iLen>0;iLen--)									// remove filename from path
		{
		if(pPath[iLen-1]=='\\')break;
		if(pPath[iLen-1]=='/' )break;
		}

	pPath[iLen] = 0;


return iLen;
}

//*****************************************************************************
//*
//*		ProcessOwnId
//*
//*****************************************************************************
//	Returns the own process id
unsigned ProcessOwnId()
{
return getpid();
}


//*****************************************************************************
//*
//*		ProcessRunningOwn
//*
//*****************************************************************************
//	Returns 1 if a other instance of the currend process is running
int	ProcessRunningOwn()
{
int				iPid;
int				iLen;
int				iNum;
char			cFile[64];
char			cName[256];
char			cPath[256];
DIR 		   *pDir;
struct dirent  *pResult;
struct dirent   sEntry;




	if(!ProcessGetExeName(cName,sizeof(cName)))return 0;
	iPid = getpid();

		pDir=opendir("/proc");
	if(!pDir)return 0;

	for(;;)
		{
	    readdir_r(pDir,&sEntry,&pResult);

		if(!pResult)break;

			iNum = atoi(pResult->d_name);
		if(!iNum)continue;

		if(iNum==iPid)continue;

		sprintf(cFile,"/proc/%i/exe",iNum);

		   iLen = readlink(cFile,cPath,sizeof(cPath));
		if(iLen<=0 || iLen>=sizeof(cPath))continue;

		cPath[iLen]=0;
		if(strcmp(cPath,cName))continue;

		closedir(pDir);

		return 1;
		};


	closedir(pDir);

return 0;
}

//*****************************************************************************
//*
//*		ProcessExeNameFormId
//*
//*****************************************************************************
//	Gets the path of the exe file from a process id.
//	uId	: is the process id
//	pPath	: here the path of the process is stored
//	iSize	: size of the path pBuffer in bytes
//	Returns the size of the path or 0 if an error occurs
int	ProcessExeNameFormId(unsigned uId,char *pPath,int iSize)
{
int		iLen;
char	cPath[64];



	if(iSize<=0)return 0;

	sprintf(cPath,"/proc/%i/exe",uId);

	iLen = readlink(cPath,pPath,iSize);

	if(iLen>=iSize || iLen<=0)
		{
		pPath[0]=0;
		return 0;
		}

	pPath[iLen]=0;


return iLen;
}

//*****************************************************************************
//*
//*		ProcessEnum
//*
//*****************************************************************************
//	Enumberates all running processes.
//	uId	: is the process id where the enumberation starts (0 for the first)
//	pPath	: here the path of the process is stored (if NULL none is stored)
//	iSize	: size of the path pBuffer in bytes
//	Returns the proccess id of the next process or 0 if no more processe where
//	found.
unsigned ProcessEnum(unsigned uId,char *pPath,int iSize)
{
int				iLen;
int				iNum;
char			cFile[64];
static DIR 	   *pDir=NULL;
struct dirent  *pResult;
struct dirent   sEntry;




	if(pDir && !uId)									// close old enum dir
		{
		closedir(pDir);
		pDir=NULL;
		}

	if(!pDir)											// open new enum directory
		{
			pDir=opendir("/proc");
		if(!pDir)return 0;
		}


	for(;;)
		{
	    readdir_r(pDir,&sEntry,&pResult);

		if(!pResult)break;

			iNum = atoi(pResult->d_name);
		if(!iNum)continue;

		sprintf(cFile,"/proc/%i/exe",iNum);


		if(pPath)										// copy path of process
			{
			   iLen = readlink(cFile,pPath,iSize);
			if(iLen<=0 || iLen>=iSize)continue;

			pPath[iLen]=0;
			}

		return iNum;
		};


	closedir(pDir);


return 0;
}

//*****************************************************************************
//*
//*		ProcessKill
//*
//*****************************************************************************
//	Terminates the process with the specific process id
//	uId	: is the process id
//	Returns 1 if the process was killed or 0 if an error ocurrs
int	ProcessKill(unsigned uId)
{
int	iPid;


	   iPid = uId;
	if(iPid<=0)return 0;

	if(kill(iPid,SIGKILL)==-1)return 0;


return 1;
}

//*****************************************************************************
//*
//*		ProcessIdFromExeName
//*
//*****************************************************************************
//	Find the process id to a path of a process.
//	If the process didn't have a *.EXE suffix this suffix is added.
//	pPath	: Is the path of the process
//	Returns the process id or 0 if no runnin process was found
unsigned ProcessIdFromExeName(const char *pPath)
{
int				iLen;
int				iNum;
char			cFile[64];
char			cPath[256];
char			cFull[256];
const char	   *pName;
DIR 		   *pDir;
struct dirent  *pResult;
struct dirent   sEntry;



	iLen = strlen(pPath);
	for(;iLen>0;iLen--)
		{
		if(pPath[iLen-1]=='/')break;
		}

	pName = pPath+iLen;

	if(iLen>0)										// convert to full path
		{
		make_fullpath(cFull,pPath,sizeof(cFull));
		pPath=cFull;
		}

		pDir=opendir("/proc");
	if(!pDir)return 0;

	for(;;) 										// find process in /proc dir
		{
	    readdir_r(pDir,&sEntry,&pResult);

		if(!pResult)break;

			iNum = atoi(pResult->d_name);
		if(!iNum)continue;

		sprintf(cFile,"/proc/%i/exe",iNum);

		   iLen = readlink(cFile,cPath,sizeof(cPath));
		if(iLen<=0 || iLen>=sizeof(cPath))continue;

		cPath[iLen]=0;


		if(pName==pPath)
			{
			for(;iLen>0;iLen--)
				{
				if(cPath[iLen-1]=='/')break;
				}

			if(strcmp(cPath+iLen,pName))continue;
			}
		else{
			if(strcmp(cPath,pPath))continue;
			}


		closedir(pDir);

		return iNum;
		};


	closedir(pDir);

return 0;
}


//*****************************************************************************
//*
//*		ProcessStart
//*
//*****************************************************************************
//	Starts a new process
//	iWait		: wait until the process is finished
//	pPath		: name of the process file
//	pArgs		: aruments for the process (NULL for no args)
//	pError		: if this pointer isn't zero, an error code will be stored
//					0 = process succefull started
//					1 = file not found
//					2 = error at starting process
//					3 = filename too long
//					4 = file hasn't an execute prermission
//					5 = argument string too long
//	pWorkingDir	: is the working directory for the process, or NULL for default
//	pExitCode	: here the exit code of the proccess will be stored
//				  if this pointer is zero nothing will be stored
//	Returns the process id of started processes or 0 if an error ocurrs
unsigned ProcessStart(int iWait,const char *pPath,const char *pArgs,int *pError,const char *pWorkingDir,int *pExitCode)
{
int			i,iNum,iPos;
int			iSize;
int			iPid,iState;
char       *pList[16];
char		cArgs[1024];
char		cPath[256];
struct stat sStat;


	if(pExitCode)*pExitCode=-1;

	if(!make_fullpath(cPath,pPath,sizeof(cPath)))
		{
		if(pError)*pError=3;
		return 0;
		}

	if(stat(cPath,&sStat)==-1)					// exists the file
		{
		if(pError)*pError=1;
		return 0;
		}

	if(!(sStat.st_mode&S_IRWXU))				// is it an execuable file
		{
		if(pError)*pError=4;
		return 0;
		}


	if(!cSignalDone)							// install signal handler
		{
		cSignalDone = 1;
		signal(SIGCHLD,SignalChildHandler);
		}


	   iPid = fork();
	if(iPid == -1)
		{
		if(pError)*pError=2;
		return 0;
		}

	if(iPid ==  0)								// is it the new task
		{
		if(!pArgs)pArgs="";

		pList[0] = cPath;
		iNum     = 1;
		iSize	 = 0;

		for(iPos=0;pArgs[iPos];iPos++)			// copy arguments
			{
			if(pArgs[iPos]==' ' )continue;
			if(pArgs[iPos]=='\t')continue;

			if(pArgs[iPos]!='"')				// normal argument
				{
				for(i=iPos;pArgs[i];i++)
					{
					if(pArgs[i]==' ' )break;
					if(pArgs[i]=='\t')break;
					}

				i-=iPos;

				if(iSize+i+1>=sizeof(cArgs))
					{
					if(pError)*pError=5;
					return 0;
					}

				memcpy(cArgs+iSize,pArgs+iPos,i);
				pList[iNum] = cArgs+iSize;
				cArgs[iSize+i]=0;
				iSize+=i+1;
				iNum++;

				if(iNum+1>=sizeof(pList)/sizeof(pList[0]))break;

				iPos+=i-1;
				continue;
				}

			iPos++;								// "..." argument
			for(i=iPos;pArgs[i];i++)
				{
				if(pArgs[i]=='"')break;
				}

			i-=iPos;
			if(iSize+i+1>=sizeof(cArgs))break;

			memcpy(cArgs+iSize,pArgs+iPos,i);
			pList[iNum] = cArgs+iSize;
			cArgs[iSize+i]=0;
			iSize+=i+1;
			iNum++;

			if(iNum+1>=sizeof(pList)/sizeof(pList[0]))break;

			iPos+=i;
			continue;
			}

		pList[iNum]=NULL;

		if(pWorkingDir)							// set working directory
			{
			chdir(pWorkingDir);
			}

		execve(cPath,pList,environ);
		_exit(-1);
		return 0;
		}


	if(iWait)									// wait until process is stopped
		{
		iState=0;
		waitpid(iPid,&iState,0);

		if(pExitCode)
			{
			*pExitCode = WEXITSTATUS(iState);
			}
		}

	if(pError)*pError=0;


return iPid;
}

#endif


