//*****************************************************************************
//*
//*
//*		RemoteIntServer.c
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2009
//
//	Mit diesem Interface knnen ber Shared-Memory und einem TCP-Prozess
//	Pakete an eine Remote-Station versendet werden.
//
//	Diese Datei ist der Server auf dem Remote-System.
//
//		RemoteThread(...)		Starten einen eigenen Thread fr den Server
//		RemoteDaemon(...)		Ist die Funktion fr einen exteren Thread
//
#ifdef		GLOBAL_CONFIG
#define 	REMOTEINTSERVER_C
#include	<GlobalConfig.h>
#undef		REMOTEINTSERVER_C
#endif

#include	"RemoteInterface.h"
#include	"ThreadUtility.h"
#include	"SocketUtility.h"
#include	"ShmUtility.h"
#include	<malloc.h>
#include	<stdio.h>
#include	<time.h>


static int volatile			iStopThreads = 0;
static int volatile			iThreadCount = 0;

#ifndef		LINUX
#ifdef		linux
#define 	LINUX			1
#endif
#endif


#define 	LOG_RECV_ON		0
#define 	LOG_SEND_ON		0
#define 	LOG_CONN_ON		0


#if			LOG_RECV_ON
#define 	LOG_RECV(p,l)	printf("recv:  %s %i\n"   ,p,l);
#define 	LOG_RECV_X(p,l)	printf("recv:  %s %08Xh\n",p,l);
#else
#define 	LOG_RECV(p,l)
#define 	LOG_RECV_X(p,l)
#endif

#if			LOG_SEND_ON
#define 	LOG_SEND(p,l)	printf("send:  %s %i\n"   ,p,l);
#else
#define 	LOG_SEND(p,l)
#endif

#if			LOG_CONN_ON
#define 	LOG_CONN(i,p,c)	{char cAddr[64];	IpToString(&i,cAddr); printf("conn:  %s to %s:%u\n",c,cAddr,p);}
#define 	LOG_CONN_2(c,i)	{printf("conn:  %s %i\n",c,i);}
#define 	LOG_CONN_1(c)	{printf("conn:  %s\n",c);}
#else
#define 	LOG_CONN(i,p,c)
#define 	LOG_CONN_2(c,i)
#define 	LOG_CONN_1(c)
#endif




//******************************************************************************
//*
//*		RemoteSendIn
//*
//******************************************************************************
//	Speichert ein Paket im Empfangsringpuffer.
//	pRemote			: Zeiger auf die Pufferdaten.
//	pData			: Zeiger auf die Paketdaten sammt Header.
//	uSize			: Gre des Paketes in Byte ohne Header.
//	Ergibt 0 wenn das Paket gespeichert wurde, ansonsten -1.
static int	RemoteSendIn(RemoteHandle pRemote,const void *pData,unsigned uSize)
{
unsigned   *pOut;
unsigned   *pPtr;
unsigned 	uFree;
unsigned 	uOff;
unsigned 	uMax;


	if(pRemote->usMaxPacket<uSize)return -1;


	   uFree=pRemote->uRecvWr-pRemote->uRecvRd;
	if(uFree>pRemote->uRecvSize)uFree+=pRemote->uRecvSize;

	uFree = pRemote->uRecvSize-uFree;

	if(uFree<=pRemote->usMaxPacket+32U)return -1;


	pOut = pRemote->aData+pRemote->uSendSize/sizeof(unsigned);
	uOff = pRemote->uRecvWr                 /sizeof(unsigned);
	uMax = pRemote->uRecvSize               /sizeof(unsigned);
	pPtr = (unsigned*)pData;


	pOut[uOff] = uSize|(pPtr[0]&0xFFFF0000);
	uOff++;
	pPtr++;
	if(uOff>=uMax)uOff=0;

	uSize = (uSize+sizeof(unsigned)-1)/sizeof(unsigned)+2;

	while(uSize>0)												// Die Paketdaten kopieren
		{
		pOut[uOff] = *pPtr;

		uSize--;
		pPtr++;
		uOff++;

		if(uOff>=uMax)uOff=0;
		}


	pRemote->uRecvWr = uOff*sizeof(unsigned);					// Neue Schreibposition fr Ringpuffer



return 0;
}

//******************************************************************************
//*
//*		RemoteRecvIn
//*
//******************************************************************************
//	Liet ein Paket aus dem Senderingpuffer.
//	pRemote			: Zeiger auf die Pufferdaten.
//	pData			: Zeiger auf den Puffer fr die Paketdaten.
//					  Hier wird auch der 12 Byte-Header gespeichert.
//	iSize			: Gre des Puffers in Byte.
//	Ergibt die Paketlnge ohne Header wenn ein Paket empfangen wurde, ansonsten -1.
static int RemoteRecvIn(RemoteHandle pRemote,const void *pData,int iSize)
{
unsigned   *pInp;
unsigned   *pPtr;
unsigned 	uUsed;
unsigned 	uSize;
unsigned 	uOff;
unsigned 	uMax;
unsigned 	uCnt;



	   uUsed=pRemote->uSendWr-pRemote->uSendRd;
	if(uUsed>pRemote->uSendSize)uUsed+=pRemote->uSendSize;
	if(uUsed==0)return -1;


	pInp	= pRemote->aData;
	uOff	= pRemote->uSendRd  /sizeof(unsigned);
	uMax 	= pRemote->uSendSize/sizeof(unsigned);

	uSize   = pInp[uOff]&0xFFFF;
	uCnt	= (uSize+sizeof(unsigned)-1)/sizeof(unsigned)+3;
	pPtr	= (unsigned*)pData;

	while(uCnt>0)												// Die Paketdaten kopieren
		{
		if(iSize>=sizeof(unsigned))
			{
			*pPtr  = pInp[uOff];
			iSize -= sizeof(unsigned);
			}

		uCnt--;
		pPtr++;
		uOff++;

		if(uOff>=uMax)uOff=0;
		}


	pRemote->uSendRd = uOff*sizeof(unsigned);					// Neue Leseposition fr Ringpuffer



return uSize;
}

//******************************************************************************
//*
//*		RemoteDaemon
//*
//******************************************************************************
//	bertrgt den Inhalt eines Kommunikationspuffers an einen TCP-Client.
//	Wenn uSendSize und uRecvSize Null sind wird die Gre automatisch ermittelt.
//	uPort			: Ist das TCP-Port fr den Server. (0=Standartport)
//	uId				: Ist die ID fr den Puffer
//	Ergibt 0 wenn der Daemon gelaufen ist,oder eine Fehlernummer kleiner 0.
//						 0 = Alles ok
//						-1 = Threads von anderen Serveren laufen noch
//						-2 = SHM konnte nicht geffnet werden
//						-3 = Falsches Format fr SHM Bereich
//						-4 = Das Sever-Socket konnte nicht geffnet werden
//						-5 = Falsche ID
int RemoteDaemon(unsigned uPort,unsigned uId)
{
unsigned		aBuffer[0x10010/sizeof(unsigned)];
TcpHandle		hTcp;
TcpHandle		hNew;
TcpHandle		hServer;
RemoteHandle	pRemote;
unsigned		uMaxPacket;
unsigned		uSendSize;
unsigned		uRecvSize;
unsigned		uMarker;
unsigned		uUsed;
unsigned		uTime;
unsigned		uLast;
unsigned		uDiff;
unsigned		uTick;
unsigned		uSend;
unsigned		uMask;
unsigned		uBits;
IpAddr			iAddr;
IpAddr			iCome;
int				iTurn;
int				iLen;
int				iNum;



	if(uId>=0x10000)return -5;


	LOG_CONN_1("start daemon");

	uMarker	= uId+(~uId<<16);


	for(iTurn=0;;iTurn++)										// Warten bis Kommunikationspuffer ok ist
		{
		if(iStopThreads)
			{
			LOG_CONN_1("stop threads A");
			return  0;
			}

		if(iTurn>4*6*2)											// Timeout nach 2 Minuten
			{
			LOG_CONN_1("timeout daemon");
			return -1;
			}

			pRemote = (RemoteHandle)ShmOpen(uId,sizeof(RemoteStruct),SHM_WRITE);
		if(!pRemote)
			{
			LOG_CONN_2("can't open shm ->",uId);

			for(iNum=0;iNum<25;iNum++)							// 2.5 Sekunden warten
				{
				if(iStopThreads)break;
				ThreadSleep(100);
				}

			continue;
			}

		for(iNum=0;iNum<30;iNum++)								// Prfen ob die ID stimmt
			{
			if(iStopThreads)
				{
				LOG_CONN_1("stop threads B");
				ShmClose(uId);
				return 0;
				}

			if(pRemote->uId==uMarker)							// Stimmt die ID
				{
				break;
				}

			ThreadSleep(100);
			}

		if(pRemote->uId==uMarker)
			{
			break;
			}


		ShmClose(uId);

		for(iNum=0;iNum<10;iNum++)								// 5 Sekunden warten
			{
			if(iStopThreads)break;
			ThreadSleep(500);
			}

		iTurn++;
		}


	uSendSize   = pRemote->uSendSize;							// Die Gren automatisch ermitteln
	uRecvSize   = pRemote->uRecvSize;
	uMaxPacket  = pRemote->usMaxPacket;


	ShmClose(uId);


		pRemote = (RemoteHandle)ShmOpen(uId,sizeof(RemoteStruct)+uSendSize+uRecvSize,SHM_WRITE);
	if(!pRemote)
		{
		LOG_CONN_2("can't open shm",uId);
		return -2;
		}

	if((uMaxPacket<0x0010 || uMaxPacket>0x0000FFF0)	||
	   (uRecvSize <0x1000 || uRecvSize >0x10000000) ||
	   (uSendSize <0x1000 || uSendSize >0x10000000) ||
	   (uSendSize&3)      ||(uRecvSize&3))
		{
		LOG_CONN_2("not init shm",uId);
		ShmClose(uId);
		return -3;
		}


	LOG_CONN_2("open shm",uId);


//******************** TCP-Server laufen lassen *******************************


	IpAddress(&iAddr,"0.0.0.0",0);

	if(!uPort)uPort=REMOTE_PORT;

	for(iTurn=0;;iTurn++)
		{
		   hServer = TcpOpen(&iAddr,(unsigned short)uPort,TCP_SERVER,0);
		if(hServer)break;										// Konnte der TCP-Server registiert werden
		
		if(iTurn>60*2*2)										
			{													// Nach 2 Minuten einen Fehler ausgeben
			LOG_CONN(iAddr,uPort,"can't install server at port");
			ShmClose(uId);
			return -4;
			}

		ThreadSleep(500);
		}

	hTcp = 0;

	while(!iStopThreads)
		{
		if(pRemote->uId!=uMarker)								// Wurde der SHM-Bereich geschlossen
			{
			LOG_CONN_2("wrong marker shm",uId);
			break;
			}

		if(!hTcp)												// Kein Client ist Verbunden
			{
			pRemote->ucEnabled &= ~1;

				hTcp = TcpAcceptWait(hServer,&iCome,0,250);
			if(!hTcp)continue;

			LOG_CONN(iAddr,uPort,"accept connection");

			if(pRemote->ucEnabled&2)							// Ist der Server schon freigegeben
				{
				for(;;)											// Sendepuffer auf die Hlfte beschrnken
					{
					if(pRemote->uSendWr<pRemote->uSendRd)
							uUsed = pRemote->uSendWr-pRemote->uSendRd+uSendSize;
					else	uUsed = pRemote->uSendWr-pRemote->uSendRd;

					if(uUsed<uSendSize/2)break;

					   iNum = RemoteRecvIn(pRemote,aBuffer,sizeof(aBuffer));
					if(iNum<=0)break;
					}
				}
			else{
				pRemote->uSendRd = pRemote->uSendWr;			// Sendepuffer lschen
				}

			uTime = ThreadTickCount();

			if(!(pRemote->ucEnabled&2))							// Ist der Server schon enabel
				{
				pRemote->uTime		= time(0);
				pRemote->uTimeSub	= uTime;
				}

			pRemote->uRecvRd	= 0;
			pRemote->uRecvWr	= 0;
			pRemote->ucEnabled |= 1;

			uLast = uTime;
			uTick = uTime;
			uSend = uTime;
			}

		uTime = ThreadTickCount();
		uDiff = uTime-uLast;

		if(uDiff>15000)											// Auf neue Verbindungen prfen
			{
			uLast = uTime;
			hNew  = TcpAcceptWait(hServer,&iCome,0,0);

			if(hNew)
				{
				LOG_CONN(iAddr,uPort,"accept new connection");

				uTime				= ThreadTickCount();
				pRemote->uTime		= time(0);
				pRemote->uTimeSub	= uTime;

				TcpClose(hTcp);
				hTcp = hNew;
				}
			}

		uDiff = uTime-uTick;

		if(uDiff>REMOTE_TIMEOUT)								// Kein Lifetick angekommen
			{
			LOG_CONN_1("timeout");
			TcpClose(hTcp);
			hTcp = 0;
			continue;
			}

		iLen = TcpGetNext(hTcp,aBuffer,sizeof(unsigned),250);	// Lnge empfangen

		if(iLen<0)												// Gab es einen Verbindungsabbruch
			{
			LOG_CONN_1("broken A");
			TcpClose(hTcp);
			hTcp = 0;
			continue;
			}

		if(iLen==sizeof(unsigned))								// Wurde TCP seitig etwas empfangen
			{
			uTick=uTime;

			if(aBuffer[0]<0xFFFFFFFE)							// Paket empfangen (TCP->SHM)
				{
				iNum = sizeof(unsigned)*2+(aBuffer[0]&0xFFFF);

				LOG_RECV("header",aBuffer[0]&0xFFFF);

				iLen = TcpGetWait(hTcp,aBuffer+1,iNum,100);

				if(iNum!=iLen)
					{											// Fehler beim Empfangen
					TcpClose(hTcp);
					hTcp = 0;
					continue;
					}

				LOG_RECV("data  ",iLen-sizeof(unsigned)*2);

				RemoteSendIn(pRemote,aBuffer,aBuffer[0]&0xFFFF);
				}
			else if(aBuffer[0]==0xFFFFFFFE)						// Einstellung fr Sub-Filter
				{
				iLen = TcpGetWait(hTcp,aBuffer+1,4,100);

				if(iLen!=4)
					{											// Fehler beim Empfangen
					TcpClose(hTcp);
					hTcp = 0;
					continue;
					}

				LOG_RECV_X("filter",aBuffer[1]);

				uMask  = aBuffer[1]>>16;
				uBits  = aBuffer[1]&uMask;
				uBits |= pRemote->usSubEnable&~uMask;

				pRemote->usSubEnable = uBits;
				}
			else{
				LOG_RECV("livetick",-1);
				}
			}


		for(;;)													// Vorhandene Pakete senden (SHM->TCP)
			{
			   iNum = RemoteRecvIn(pRemote,aBuffer,sizeof(aBuffer));
			if(iNum<0)break;									// Kein Paket vorhanden ?

			uSend = uTime;
			iLen  = TcpPut(hTcp,aBuffer,iNum+sizeof(unsigned)*3);

			if(iLen!=(int)(iNum+sizeof(unsigned)*3))			// Fehler beim Senden
				{
				LOG_CONN_1("broken B");
				TcpClose(hTcp);
				hTcp = 0;
				}

			LOG_SEND("packet",iLen-sizeof(unsigned)*3);
			}


		uDiff = uTime-uSend;

		if(uDiff>REMOTE_LIFETICK)								// Einen Lifetick senden
			{
			aBuffer[0] = 0xFFFFFFFF;
			iLen = TcpPut(hTcp,aBuffer,sizeof(unsigned));

			if(iLen<0)
				{
				LOG_CONN_1("broken C");
				TcpClose(hTcp);
				hTcp = 0;
				}

			uSend = uTime;
			}
		}


	pRemote->ucEnabled &= ~1;									// SHM freigeben
	ShmClose(uId);


	if(hTcp)TcpClose(hTcp);										// Verbindungen wieder abbauen
	TcpClose(hServer);

	LOG_CONN_1("close all");



return 0;
}

//*****************************************************************************
//*
//*		RemoteThreadProc
//*
//*****************************************************************************
static void _cdecl RemoteThreadProc(void *pParam)
{
RemoteHandle	pRemote=(RemoteHandle)pParam;


	iThreadCount++;

	while(!iStopThreads)
		{
		RemoteDaemon(pRemote->usSubEnable,pRemote->uId);
		ThreadSleep(1000);
		}

	free(pRemote);

	iThreadCount--;

}

//*****************************************************************************
//*
//*		RemoteThread
//*
//*****************************************************************************
//	Erzeugt einen Kommunikationspuffer der ber einen eigenen Thread verfgt.
//	Wenn uSendSize und uRecvSize Null sind wird die Gre automatisch ermittelt.
//	uId				: Ist die ID fr den Puffer
void RemoteThread(unsigned uPort,unsigned uId)
{
RemoteHandle	 pRemote;


	pRemote				 = (RemoteHandle)malloc(sizeof(*pRemote));
	pRemote->uId		 = uId;
	pRemote->usSubEnable = uPort;

	ThreadStart(RemoteThreadProc,0x14000,pRemote,0);


}




//******************************************************************************
//*
//*		RemoteStop
//*
//******************************************************************************
//	Stopt alle Kommunikationsverbindungen.
void RemoteStop()
{

	iStopThreads = 1;

}

//******************************************************************************
//*
//*		RemoteStart
//*
//******************************************************************************
//	Erlaubt das Kommunikationsverbindungen wieder gestartet werden knnen.
void RemoteStart()
{

	iStopThreads = 0;

}

//******************************************************************************
//*
//*		RemoteStart
//*
//******************************************************************************
//	Stopt alle Kommunikationsverbindungen und wartet bis alle Threads beendet wurden.
//	uTimeout		: Ist die maximale Wartezeit in Millisekunden.
//	Ergibt die Anzahl der laufenden Threads
int RemoteStopWait(unsigned uTimeout)
{
unsigned	uTime;


	iStopThreads = 1;


	for(uTime=0;uTime<uTimeout;uTime+=100)
		{
		if(iThreadCount<=0)break;
		ThreadSleep(100);
		iStopThreads = 1;
		}


return iThreadCount;
}
