//*****************************************************************************
//*
//*
//*		RemoteInterface.h
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2009
//
//	Mit diesem Interface knnen ber Shared-Memory und einem TCP-Prozess
//	Pakete an eine Remote-Station versendet werden.
//
#ifndef   __REMOTEINTERFACE_H__
#define   __REMOTEINTERFACE_H__

#ifdef		GLOBAL_CONFIG
#define 	REMOTEINTERFACE_H
#include	<GlobalConfig.h>
#undef		REMOTEINTERFACE_H
#endif

#ifdef	__cplusplus
extern	"C" {
#endif


#ifdef		LINUX
#ifndef		linux
#define		linux	1
#endif
#endif

#ifdef		_MSC_VER
#pragma     once
#pragma 	warning( disable :4035 )
#pragma 	warning( disable :4996 )
#pragma 	warning( disable :4514 )
#ifndef		__cplusplus
#ifndef 	inline
#define 	inline	__inline
#endif
#endif
#endif

#ifndef		__cplusplus
#ifndef		D
#define 	D(a)
#endif
#else
#ifndef		D
#define 	D(a)	a
#endif
#endif

#ifndef		REMOTE_PORT
#define 	REMOTE_PORT		12871							// Standard TCP-Port fr den Server
#endif
#ifndef		REMOTE_TIMEOUT
#define 	REMOTE_TIMEOUT	15000							// Verbindungs-Timeout in Millisekunden
#endif
#ifndef		REMOTE_LIFETICK
#define 	REMOTE_LIFETICK	7000							// Lifetick sende Zeit in Millisekunden
#endif

typedef struct
	{
	unsigned		volatile	uId;						// Bit 0..15 = SHM-ID Bit 16..31 = inverse ID
	unsigned 					uTime;						// UTC-Zeit vom letzten Start fr Zeitstempel
	unsigned 					uTimeSub;					// Millisekundenkorrektur fr Zeitstempel
	unsigned char 	volatile	ucEnabled;					// Hat sich die Gegenstelle verbunden (0=nein 1=ja 2..3=immer ein)
	unsigned char 	volatile	ucSemaphore;				// Beim Senden eine Semaphore verwenden
	unsigned short 	volatile	usSubEnable;				// Welcher unterbereich soll freigegeben werden
	unsigned short 	volatile	usMaxPacket;				// Maximale Paketgre
	unsigned char	volatile	ucAttached;					// Wieviele Prozesse sind angehngt
	unsigned char				ucUnused;					// Unbenutzt
	unsigned 					uSendRd;					// Schreibposition im Ringpuffer frs Senden
	unsigned 					uSendWr;					// Leseposition    im Ringpuffer frs Senden
	unsigned 					uSendSize;					// Ringpuffer frs Senden
	unsigned 					uRecvRd;					// Schreibposition im Ringpuffer frs Empfangen
	unsigned 					uRecvWr;					// Leseposition    im Ringpuffer frs Empfangen
	unsigned 					uRecvSize;					// Ringpuffer frs Empfangen
	unsigned	 				aData[1];					// Ringpuffer Sende ... Empfangen
	}RemoteStruct,*RemoteHandle;

typedef struct
	{
	unsigned 					uUtcTime;					// UTC-Referenzzeit wann die Aufzeichnung begonnen hat
	unsigned 					uMsOffset;					// Offset in ms wann das Paket erzeugt wurde.
	}RemoteTimeStemp;

typedef void *RemoteConn;

//*****************************************************************************
//*
//*		Server API
//*
//*****************************************************************************

int				RemoteDaemon  (unsigned uPort,unsigned uId);
void 			RemoteThread  (unsigned uPort,unsigned uId);
int 			RemoteStopWait(unsigned uTimeout D(=20000));
void			RemoteStart   ();
void			RemoteStop    ();

//*****************************************************************************
//*
//*		Client API
//*
//*****************************************************************************

RemoteConn		RemoteConnect   (const char *pHostname,unsigned uPort D(=0));
int				RemoteDisconnect(RemoteConn hHandle);
int				RemotePutPacket (RemoteConn hHandle,const void *pData,unsigned uSize,unsigned  uPacketId);
int				RemoteGetPacket (RemoteConn hHandle,      void *pData,int      iSize,unsigned *pPacketId,RemoteTimeStemp *pTime);
int				RemoteSetFilter (RemoteConn hHandle,unsigned uSubId);
int				RemoteWait      (RemoteConn hHandle,int iTimeout);

//*****************************************************************************
//*
//*		Echtzeit Funktionen
//*
//*****************************************************************************

unsigned	    RemoteTickCount();
RemoteHandle	RemoteInit  (unsigned uId,unsigned uSendSize D(=0x80000),unsigned uRecvSize D(=0x80000),unsigned uMaxPacket D(=0x0800),unsigned uUseSem D(=0));
int 			RemoteExit  (RemoteHandle pRemote);
int				RemoteSendEx(RemoteHandle pRemote,const void *pData,unsigned uSize,unsigned  uPacketId);
int				RemoteRecvEx(RemoteHandle pRemote,      void *pData,int      iSize,unsigned *pPacketId,RemoteTimeStemp *pTime);
void			RemoteEnable(RemoteHandle pRemote,unsigned uEnable D(=1),unsigned uSubIdMask D(=0));
#define			RemoteSize(uMax,uSlot)	((uMax)-(uSlot)-sizeof(RemoteStruct))

//******************************************************************************
//*
//*		RemoteIsEnabled
//*
//******************************************************************************
//	Prft ob das Remote Interface freigegben ist
//	pRemote			: Zeiger auf die Pufferdaten.
//	Ergibt 1 wenn ja, bzw. 0 wenn das Interface gesperrt ist.
static inline int RemoteIsEnabled(RemoteHandle pRemote)
{

	if(pRemote           ==0)return 0;
	if(pRemote->ucEnabled==0)return 0;

return 1;
}


//******************************************************************************
//*
//*		RemoteSend
//*
//******************************************************************************
//	Speichert ein Paket im Senderingpuffer.
//	pRemote			: Zeiger auf die Pufferdaten.
//	pData			: Zeiger auf die Paketdaten.
//	uSize			: Gre des Paketes in Byte.
//	uPacketId		: User ID fr das Paket (16-Bit).
//	Ergibt 0 wenn das Paket gespeichert wurde, ansonsten -1.
static inline int RemoteSend(RemoteHandle pRemote,const void *pData,unsigned uSize,unsigned uPacketId D(=0))
{

	if(pRemote           ==0)return -1;
	if(pRemote->ucEnabled==0)return -1;

return RemoteSendEx(pRemote,pData,uSize,uPacketId);
}

//******************************************************************************
//*
//*		RemoteSendBit
//*
//******************************************************************************
//	Speichert ein Paket im Senderingpuffer mit Bitmaske als Freigabe.
//	pRemote			: Zeiger auf die Pufferdaten.
//	pData			: Zeiger auf die Paketdaten.
//	uSize			: Gre des Paketes in Byte.
//	uPacketId		: User ID fr das Paket (16-Bit).
//						Bits  0..15 = ID die bertragen wird
//						Bits 16..31 = Bitmaske fr den Sendefilter.
//	Ergibt 0 wenn das Paket gespeichert wurde, ansonsten -1.
static inline int RemoteSendBit(RemoteHandle pRemote,const void *pData,unsigned uSize,unsigned uPacketId D(=0))
{
unsigned 	usMask;

	if(pRemote           ==0)return -1;
	if(pRemote->ucEnabled==0)return -1;

	usMask = (unsigned short)(uPacketId>>16);

	if(pRemote->usSubEnable&usMask)return -1;

return RemoteSendEx(pRemote,pData,uSize,uPacketId);
}

//******************************************************************************
//*
//*		RemoteSendSub
//*
//******************************************************************************
//	Speichert ein Paket im Senderingpuffer mit eine Unter-ID als Freigabe.
//	pRemote			: Zeiger auf die Pufferdaten.
//	pData			: Zeiger auf die Paketdaten.
//	uSize			: Gre des Paketes in Byte.
//	uPacketId		: User ID fr das Paket (16-Bit).
//						Bits  0..15 = ID die bertragen wird
//						Bits 16..31 = Unter-ID fr den Sendefilter.
//	uMask			: Welche Bits in der Unter-ID mssen ausgewertet werden.
//	Ergibt 0 wenn das Paket gespeichert wurde, ansonsten -1.
static inline int RemoteSendSub(RemoteHandle pRemote,const void *pData,unsigned uSize,unsigned uPacketId D(=0),unsigned uMask D(=0xFFFF))
{
unsigned 	usSubId;

	if(pRemote           ==0)return -1;
	if(pRemote->ucEnabled==0)return -1;

	usSubId = (unsigned short)(uPacketId>>16);

	if((pRemote->usSubEnable^usSubId)&uMask)return -1;

return RemoteSendEx(pRemote,pData,uSize,uPacketId);
}

//******************************************************************************
//*
//*		RemoteRecvEx
//*
//******************************************************************************
//	Liet ein Paket aus dem Empfangsringpuffer.
//	pRemote			: Zeiger auf die Pufferdaten.
//	pData			: Zeiger auf den Puffer fr die Paketdaten.
//	uSize			: Gre des Puffers in Byte.
//	pPacketId		: Hier wird die User ID vom das Paket (16-Bit) gespeichert.
//					  Wenn der Zeiger 0 ist wird nichts gespeichert.
//	pTime			: Hier wird der Zeitstempel gespeichert.
//					  Wenn der Zeiger 0 ist wird nichts gespeichert.
//	Ergibt die Paketlnge wenn ein Paket empfangen wurde, ansonsten -1.
static inline int RemoteRecv(RemoteHandle pRemote,void *pData,int iSize,unsigned *pPacketId D(=0),RemoteTimeStemp *pTime D(=0))
{

	if(pRemote           ==0)return -1;
	if(pRemote->ucEnabled==0)return -1;
	if(pRemote->uRecvWr  ==pRemote->uRecvRd)return -1;

return RemoteRecvEx(pRemote,pData,iSize,pPacketId,pTime);
}



#ifdef	__cplusplus
}
#endif

#undef	D

#endif


