//*****************************************************************************
//*
//*
//*		SemaphoreUtility.c
//*
//*
//*****************************************************************************
//
//	Module for global semaphores. Supports WINDOWS,LINUX,RTTARGET.
//
//	(C) Copyright Anton Zechner 2007
//




//*****************************************************************************
//*
//*
//*		WINDOWS
//*
//*
//*****************************************************************************
#if  (defined(_WIN32) || defined(_WIN64)) && !defined(RTTARGET)
#include	<stdio.h>
#include	<string.h>
#include	<windows.h>
#include	"SemaphoreUtility.h"


#if        _MSC_VER >= 1000
#pragma		warning( disable :4035 )
#pragma		warning( disable :4996 )
#pragma		warning( disable :4514 )
#endif


//*****************************************************************************
//*
//*		SemaphoreOpen
//*
//*****************************************************************************
//	Opens an existing global semaphore
//	iKey	: is the name of the semaphore, 0 isn't allowed
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreOpen(int iKey)
{
char	cName[32];
HANDLE	hSem;


	if(!iKey)return NULL;

	sprintf(cName,"Global\\SEM_%08X",iKey);
	hSem = OpenSemaphoreA(SEMAPHORE_ALL_ACCESS,FALSE,cName);


return (Semaphore)hSem;
}

//*****************************************************************************
//*
//*		SemaphoreCreate
//*
//*****************************************************************************
//	Creates a new semaphore with the init value of 1
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreate(int iKey)
{
return SemaphoreCreateEx(iKey,1);
}


//*****************************************************************************
//*
//*		SemaphoreCreateEx
//*
//*****************************************************************************
//	Creates a new semaphore
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	iNum	: is the inital value of the semaphore (0=locked 1..=free)
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreateEx(int iKey,int iNum)
{
SECURITY_ATTRIBUTES sAttr;
char				sDesc[SECURITY_DESCRIPTOR_MIN_LENGTH];
char				cName[32];
HANDLE				hSem;


	if(!iKey)												// create local  semaphore
		{
		hSem = CreateSemaphoreA(NULL,iNum,0x70000000,NULL);
		return (Semaphore)hSem;
		}

	sprintf(cName,"Global\\SEM_%08X",iKey);

	   hSem = OpenSemaphoreA(SEMAPHORE_ALL_ACCESS,FALSE,cName);
	if(hSem)												// the semaphore allready exists
		{
		CloseHandle(hSem);
		return NULL;
		}

	sAttr.nLength				= sizeof(sAttr);			// create security descriptor
	sAttr.bInheritHandle		= FALSE;
	sAttr.lpSecurityDescriptor	= &sDesc;
	InitializeSecurityDescriptor(sAttr.lpSecurityDescriptor,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl   (sAttr.lpSecurityDescriptor,TRUE,NULL,FALSE);

	hSem = CreateSemaphoreA(&sAttr,iNum,0x70000000,cName);	// create global semaphore


return (Semaphore)hSem;
}


//*****************************************************************************
//*
//*		SemaphoreClose
//*
//*****************************************************************************
//	Closes a semaphore. If all handels are closed the semaphor will be deleted.
//	hSemaphore	: ist the handle of the semaphore
//	Return 1 if the semaphore was closed or 0 if an error occurs
int SemaphoreClose(Semaphore hSemaphore)
{

return CloseHandle((HANDLE)hSemaphore);
}



#endif


//*****************************************************************************
//*
//*
//*		LINUX
//*
//*
//*****************************************************************************
//
//	Note:	Add -lpthread to your makefile
//

#ifdef 	linux
#ifndef	LINUX
#define LINUX	1
#endif
#endif


#ifdef	LINUX

#define	   _GNU_SOURCE 

#include 	<errno.h>
#include	<stdio.h>
#include 	<unistd.h>
#include 	<sys/ipc.h>
#include 	<sys/sem.h>
#include	<sys/stat.h>
#include 	<sys/time.h>
#include 	<sys/types.h>
#include	<sys/syscall.h>
#include	"SemaphoreUtility.h"



//******************************************************************************
//*
//*		SemaphoreClean
//*
//******************************************************************************
//	Cleans all unused semaphores
//	Returns the count of closed semaphores
static int SemaphoreClean()
{
char 			 cPath[32];
struct ipc_perm *pIpc;
struct semid_ds  sData;
struct seminfo	 sInfo;
struct stat   	 sStat;
int 			 iMaxId;
int 			 iSemId;
int 			 iPid;
int 			 iPos;
int  			 iRet;
int 			 iId;



	   iMaxId = semctl(0,0,SEM_INFO,&sInfo);
	if(iMaxId<0)
		{
		return 0;
		}


	iRet = 0;

	for(iId=0;iId<=iMaxId;iId++)									// loop over all semahores
		{
		   iSemId = semctl(iId,0,SEM_STAT,&sData);
		if(iSemId<0)
			{
			continue;
			}

		pIpc = &sData.sem_perm;

		if(sData.sem_nsems!=2 /*|| pIpc->__key!=0*/)				// only private events
			{
			continue;
			}

		for(iPos=0;iPos<sData.sem_nsems;iPos++)						// check the processes
			{
			   iPid  = semctl(iSemId,iPos,GETPID,&sData);
			if(iPid<0)break;

			sprintf(cPath,"/proc/%i",iPid);

			if(stat(cPath,&sStat)!=-1)
				{
				break;
				}
			}

		if(iPos>=sData.sem_nsems)
			{
			if(semctl(iSemId,0,IPC_RMID,0)!=-1)						// close event
				{
				iRet++;
				}
			}
		}


return iRet;
}

//*****************************************************************************
//*
//*		SemaphoreCreate
//*
//*****************************************************************************
//	Creates a new semaphore with the init value of 1
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreate(int iKey)
{
	return SemaphoreCreateEx(iKey,1);

}

//*****************************************************************************
//*
//*		SemaphoreCreateEx
//*
//*****************************************************************************
//	Creates a new semaphore
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	iNum	: is the inital value of the semaphore (0=locked 1..=free)
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreateEx(int iKey,int iNum)
{
struct sembuf	sOp;
int				iInc;
int 			iSem;



	if(!iKey)												// create a local semaphore
		{
		   iSem = semget(IPC_PRIVATE,2,IPC_CREAT|IPC_EXCL|0666);
		if(iSem==-1)return NULL;
			{
			if(errno!=ENOSPC || !SemaphoreClean())
				{
				return NULL;
				}
															// second try if semaphores are cleaned
			   iSem = semget(IPC_PRIVATE,2,IPC_CREAT|IPC_EXCL|0666);
			if(iSem==-1)
				{
				return NULL;
				}
			}
		}
	else{
		   iSem = semget(iKey,2,IPC_CREAT|IPC_EXCL|0666);
		if(iSem==-1)
			{
			   iSem = semget(iKey,2,0666);					// did the semaphore allready exist ?
			if(iSem==-1)return NULL;

			   iInc = semctl(iSem,1,GETVAL,0);				// if open count is zero the semaphore can be used
			if(iInc!=0)return NULL;
			}
		}


	sOp.sem_num = 1;										// increment open count
	sOp.sem_op  = 1;
	sOp.sem_flg = SEM_UNDO;

	semop(iSem,&sOp,1);


	semctl(iSem,0,SETVAL,iNum);								// set starting value



return (Semaphore)((intptr_t)iSem+1);
}

//*****************************************************************************
//*
//*		SemaphoreOpen
//*
//*****************************************************************************
//	Opens an existing semaphore
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreOpen(int iKey)
{
struct sembuf	sOp;
int				hSem;


	if(!iKey)return NULL;

	   hSem = semget(iKey,2,0666);
	if(hSem==-1)return NULL;


	sOp.sem_op  = 1;
	sOp.sem_num = 1;										// increment open count
	sOp.sem_flg = SEM_UNDO;

	semop(hSem,&sOp,1);




return (Semaphore)((intptr_t)hSem+1);
}

//*****************************************************************************
//*
//*		SemaphoreClose
//*
//*****************************************************************************
//	Closes a semaphore. If all handels are closed the semaphor will be deleted.
//	If the last handle of a semaphore was closed the semaphore will be removed.
//	hSemaphore	: ist the handle of the semaphore
//	Return 1 if the semaphore was closed or 0 if an error occurs
int SemaphoreClose(Semaphore hSemaphore)
{
struct timespec	sTimeout;
struct sembuf	sOp;
int 			iInc;


	if(!hSemaphore)return 0;


	sTimeout.tv_sec	 =  0;									// decrement open count
	sTimeout.tv_nsec =  0;
	sOp.sem_op		 = -1;
	sOp.sem_num      =  1;
	sOp.sem_flg		 =  SEM_UNDO;


	if(semtimedop(((intptr_t)hSemaphore)-1,&sOp,1,&sTimeout)==-1)
		{
		return 0;
		}

	   iInc = semctl(((intptr_t)hSemaphore)-1,1,GETVAL,0);	// if open count is zero the semaphore can be deleded
	if(iInc>0)return 1;



	if(semctl(((intptr_t)hSemaphore)-1,0,IPC_RMID,0)==-1)	// close semaphore
		{
		return 0;
		}



return 1;
}


//*****************************************************************************
//*
//*		SemaphoreLock
//*
//*****************************************************************************
//	Locks a semaphore, the semaphore value is decremented by 1. If the
//  semaphore value is 0 the function waits unti the value is above zero.
//	hSemaphore	: is the handle of the semaphore
void SemaphoreLock(Semaphore hSemaphore)
{
struct sembuf	sOp;


	if(!hSemaphore)return;

	sOp.sem_op  = -1;
	sOp.sem_num =  0;
	sOp.sem_flg =  0;

	semop(((intptr_t)hSemaphore)-1,&sOp,1);


}


//*****************************************************************************
//*
//*		SemaphoreLockTimed
//*
//*****************************************************************************
//	Locks a semaphore, the semaphore value is decremented by 1. If the
//  semaphore value is 0 the function waits unti the value is above zero.
//	If the semaphore is locked longer than uWaitTime (in ms) the function
//	breakes the waiting.
//	hSemaphore	: is the handle of the semaphore
//	uWaitTime	: waite time in milliseconds
//	Returns 1 if the semophore is locked, or 0 at a timeout
int SemaphoreLockTimed(Semaphore hSemaphore,unsigned uWaitTime)
{
struct sembuf	sOp;
struct timespec	sTimeout;



	if(!hSemaphore)return 0;

	if(uWaitTime==0)
		{
		sOp.sem_op  = -1;
		sOp.sem_num =  0;
		sOp.sem_flg =  IPC_NOWAIT;

		if(semop(((intptr_t)hSemaphore)-1,&sOp,1)!=-1)
			{
			return 1;
			}

		return 0;
		}


	sTimeout.tv_sec	 = uWaitTime/1000;
	sTimeout.tv_nsec =(uWaitTime%1000)*1000000;
	sOp.sem_op		 = -1;
	sOp.sem_num      =  0;
	sOp.sem_flg		 =  0;

	if(semtimedop(((intptr_t)hSemaphore)-1,&sOp,1,&sTimeout)!=-1)
		{
		return 1;
		}


return 0;
}


//*****************************************************************************
//*
//*		SemaphoreUnlockEx
//*
//*****************************************************************************
//	Unlocks a semaphore, the semaphore value is incremented by 1.
//	hSemaphore	: is the handle of the semaphore
void SemaphoreUnlockEx(Semaphore hSemaphore,int iCount)
{
struct sembuf	sOp;



	if(!hSemaphore || iCount<=0)return;

	sOp.sem_op  = iCount;
	sOp.sem_num = 0;
	sOp.sem_flg = 0;

	semop(((intptr_t)hSemaphore)-1,&sOp,1);
}




#endif


//******************************************************************************
//*
//*
//*		RTTARGET
//*
//*
//******************************************************************************
#ifdef		RTTARGET

#include	<rttarget.h>
#include 	<rtk32.h>
#include 	<clock.h>
#include 	<stdio.h>

#define 	RemoteTickCount()		CLKTicksToMilliSecs(RTKGetTime())
#define		SemLock(h,t)			RTKWaitTimed((RTKSemaphore)h,CLKMilliSecsToTicks(t))
#define		SemUnlock(h)			RTKWait((RTKSemaphore)h)



//*****************************************************************************
//*
//*		SemaphoreOpen
//*
//*****************************************************************************
//	Opens an existing global semaphore
//	iKey	: is the name of the semaphore, 0 isn't allowed
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreOpen(int iKey)
{
char		cName[32];
Semaphore	hSem;


	if(!iKey)return NULL;

	sprintf(cName,"SEM_%08X",iKey);
	hSem = (Semaphore)RTKOpenSemaphoreA(ST_COUNTING,0,0,cName)



return hSem;
}

//*****************************************************************************
//*
//*		SemaphoreCreate
//*
//*****************************************************************************
//	Creates a new semaphore with the init value of 1
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreate(int iKey)
{
return SemaphoreCreateEx(iKey,1);
}


//*****************************************************************************
//*
//*		SemaphoreCreateEx
//*
//*****************************************************************************
//	Creates a new semaphore
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	iNum	: is the inital value of the semaphore (0=locked 1..=free)
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreateEx(int iKey,int iNum)
{
char		cName[32];
Semaphore	hSem;



	if(!iKey)												// create local  semaphore
		{
		hSem = (Semaphore)RTKOpenSemaphore(ST_COUNTING,iNum,0x70000000,NULL)
		return hSem;
		}

	sprintf(cName,"SEM_%08X",iKey);

	   hSem = (Semaphore)RTKOpenSemaphore(ST_COUNTING,0,0,cName)
	if(hSem)												// the semaphore allready exists
		{
		RTKDeleteSemaphore(&hSem);
		return NULL;
		}


	hSem = (Semaphore)RTKOpenSemaphore(ST_COUNTING,iNum,0x70000000,cName)


return hSem;
}


//*****************************************************************************
//*
//*		SemaphoreClose
//*
//*****************************************************************************
//	Closes a semaphore. If all handels are closed the semaphor will be deleted.
//	hSemaphore	: ist the handle of the semaphore
//	Return 1 if the semaphore was closed or 0 if an error occurs
int SemaphoreClose(Semaphore hSemaphore)
{

return RTKDeleteSemaphore(&hSemaphore);
}


//*****************************************************************************
//*
//*		SemaphoreUnlockEx
//*
//*****************************************************************************
void SemaphoreUnlockEx(ThreadSemaphore hSemaphore,int iCount)
{

	for(;iCount>0;iCount--)RTKSignal(hSemaphore);

}

#endif

//******************************************************************************
//*
//*
//*		FREERTOS
//*
//*
//******************************************************************************
#ifdef		FREERTOS

#include	<FreeRTOS.h>
#include	"task.h"
#include	"queue.h"
#include	"semphr.h"

#ifdef		portUSING_MPU_WRAPPERS
#if			portUSING_MPU_WRAPPERS
#error		"wrong FreeRtos configuration, portUSING_MPU_WRAPPERS isn't allowed"
#endif
#endif

#if			configUSE_COUNTING_SEMAPHORES==0
#error		"wrong FreeRtos configuration, configUSE_COUNTING_SEMAPHORES must be 1"
#endif

#if			configUSE_16_BIT_TICKS!=0
#error		"wrong FreeRtos configuration, configUSE_16_BIT_TICKS must be 0"
#endif


// REM AZ Fehlt noch
#define 	RemoteTickCount()		CLKTicksToMilliSecs(RTKGetTime())
#define		SemLock(h,t)			RTKWaitTimed((RTKSemaphore)h,CLKMilliSecsToTicks(t))
#define		SemUnlock(h)			RTKWait((RTKSemaphore)h)



//*****************************************************************************
//*
//*		SemaphoreOpen
//*
//*****************************************************************************
//	Opens an existing global semaphore
//	iKey	: is the name of the semaphore, 0 isn't allowed
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreOpen(int iKey)
{
char		cName[32];
Semaphore	hSem;


	if(!iKey)return NULL;

	sprintf(cName,"SEM_%08X",iKey);
	hSem = (Semaphore)RTKOpenSemaphoreA(ST_COUNTING,0,0,cName)



return hSem;
}

//*****************************************************************************
//*
//*		SemaphoreCreate
//*
//*****************************************************************************
//	Creates a new semaphore with the init value of 1
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreate(int iKey)
{
return SemaphoreCreateEx(iKey,1);
}


//*****************************************************************************
//*
//*		SemaphoreCreateEx
//*
//*****************************************************************************
//	Creates a new semaphore
//	iKey	: is the name of the semaphore, 0 is for a local semaphore
//	iNum	: is the inital value of the semaphore (0=locked 1..=free)
//	Returns a handle to a semaphore or NULL if an error occurs
Semaphore SemaphoreCreateEx(int iKey,int iNum)
{
char		cName[32];
Semaphore	hSem;



	if(!iKey)												// create local  semaphore
		{
		hSem = (Semaphore)RTKOpenSemaphore(ST_COUNTING,iNum,0x70000000,NULL)
		return hSem;
		}

	sprintf(cName,"SEM_%08X",iKey);

	   hSem = (Semaphore)RTKOpenSemaphore(ST_COUNTING,0,0,cName)
	if(hSem)												// the semaphore allready exists
		{
		RTKDeleteSemaphore(&hSem);
		return NULL;
		}


	hSem = (Semaphore)RTKOpenSemaphore(ST_COUNTING,iNum,0x70000000,cName)


return hSem;
}


//*****************************************************************************
//*
//*		SemaphoreClose
//*
//*****************************************************************************
//	Closes a semaphore. If all handels are closed the semaphor will be deleted.
//	hSemaphore	: ist the handle of the semaphore
//	Return 1 if the semaphore was closed or 0 if an error occurs
int SemaphoreClose(Semaphore hSemaphore)
{

return RTKDeleteSemaphore(&hSemaphore);
}


//*****************************************************************************
//*
//*		SemaphoreUnlockEx
//*
//*****************************************************************************
void SemaphoreUnlockEx(ThreadSemaphore hSemaphore,int iCount)
{

	for(;iCount>0;iCount--)RTKSignal(hSemaphore);

}



#endif

