//*****************************************************************************
//*
//*
//*		SemaphoreUtility.h
//*
//*
//*****************************************************************************
//
//
//	Module for global semaphores. Supports WINDOWS,LINUX,RTTARGET.
//
//	(C) Copyright Anton Zechner 2007
//
#ifndef 	__SEMAPHOREUTILTY_H__
#define 	__SEMAPHOREUTILTY_H__

#ifdef		GLOBAL_CONFIG
#define 	SHMUTILITY_H
#include	<GlobalConfig.h>
#undef		SHMUTILITY_H
#endif

#ifdef 		linux
#ifndef		LINUX
#define 	LINUX	1
#endif
#endif

#ifdef		_MSC_VER
#pragma     once
#pragma 	warning( disable :4035 )
#pragma 	warning( disable :4996 )
#pragma 	warning( disable :4514 )
#endif


#ifdef	__cplusplus
extern "C" {
#endif

	typedef	void *Semaphore;


	Semaphore	SemaphoreOpen       (int iKey);
	Semaphore	SemaphoreCreate     (int iKey);
	Semaphore	SemaphoreCreateEx	(int iKey,int iNum);
	int			SemaphoreClose      (Semaphore hSemaphore);
//	void		SemaphoreLock       (Semaphore hSemaphore);
//	int 		SemaphoreLockTimed  (Semaphore hSemaphore,unsigned uWaitTime);
//	void		SemaphoreUnlockEx   (Semaphore hSemaphore,int iCount);
//	void		SemaphoreUnlock     (Semaphore hSemaphore);



//******************** LINUX **************************************************
#ifdef	LINUX


	int			SemaphoreLockTimed  (Semaphore hSemaphore,unsigned uWaitTime);
	void		SemaphoreLock       (Semaphore hSemaphore);
	void		SemaphoreUnlockEx   (Semaphore hSemaphore,int iCount);
	int 		SemaphoreCount  	(Semaphore hSemaphore);
	#define		SemaphoreUnlock(h)   SemaphoreUnlockEx(h,1)


//******************** WINDOWS ************************************************
#elif defined(WIN32) && !defined(RTTARGET) && !defined(FREERTOS)


	#ifndef _WINDOWS_
	extern	unsigned long	__declspec(dllimport)   __stdcall	WaitForMultipleObjects(unsigned long,void **,int,unsigned long);
	extern	unsigned long	__declspec(dllimport)   __stdcall	WaitForSingleObject(void*,unsigned long);
	extern	int				__declspec(dllimport)   __stdcall	ReleaseSemaphore(void*,long,long*);
	#endif

	int 		SemaphoreCount(Semaphore hSemaphore);

	#define		SemaphoreLock(h)				WaitForSingleObject(h,0xFFFFFFFF)
	#define		SemaphoreLockTimed(h,t)		  ((WaitForSingleObject(h,t)==0)? 1:0)
	#define		SemaphoreUnlock(h)				ReleaseSemaphore(h,1,NULL)
	#define		SemaphoreUnlockEx(h,c)			ReleaseSemaphore(h,c,NULL)



//******************** RTTARGET ***********************************************
#elif defined(RTTARGET)


	#ifdef		   _MSC_VER
	#ifndef		   _MT
	#error			"Multithread not activated"
	#endif
	#endif

	#ifndef __THREAD_C__
	#ifndef _CLOCK_H
	extern	long			__declspec(dllimport)   _cdecl	CLKMilliSecsToTicks(unsigned long);
	#endif
	#ifndef _RTK32_H
	extern	void 			__declspec(dllimport)   _cdecl	RTKSignal(void*);
	extern	unsigned long	__declspec(dllimport)   _cdecl	RTKWaitTimed(void*,long);
	extern	void 			__declspec(dllimport)   _cdecl	RTKWait(void*);
	#endif
	#endif

	void		SemaphoreUnlockEx(void *hSemaphore,int iCount);

	#define 	SemaphoreLock(h)				RTKWait(h)
	#define 	SemaphoreLockTimed(h,t)			RTKWaitTimed(h,CLKMilliSecsToTicks(t))
	#define 	SemaphoreUnlock(h)				RTKSignal(h)
	#define 	SemaphoreCount(h)	      ((int)RTKSemaValue(h))


//********************* FREERTOS **********************************************
#elif defined(FREERTOS)


	#ifdef		   _MSC_VER
	#ifndef		   _MT
	#error			"Multithread not activated"
	#endif
	#endif

    #ifndef       __THREAD_C__
    #ifndef         QUEUE_H

	extern void		vQueueDelete(void *);
	extern void	   *xQueueCreateCountingSemaphore(const unsigned long,const unsigned long);
	extern long		xQueueGenericReceive(void*,const void *,unsigned long,const long);
	extern long		xQueueGenericSend   (void*,const void *,unsigned long,const long);
	extern unsigned uxQueueMessagesWaitingFromISR(void*);

    #endif
    #endif

	#ifndef	      __THREAD_C__
	#ifndef			SEMAPHORE_H

	#ifndef			xSemaphoreCreateCounting
	#define			xSemaphoreCreateCounting(c,n)	xQueueCreateCountingSemaphore(c,n)
	#endif

	#ifndef			vSemaphoreDelete
	#define			vSemaphoreDelete(h)				vQueueDelete(h)
	#endif

	#ifndef			xSemaphoreTake
	#define			xSemaphoreTake(h,t)				xQueueGenericReceive(h,NULL,t,0)
	#endif

	#ifndef			xSemaphoreGive
	#define			xSemaphoreGive(h)				xQueueGenericSend(h,0,0,0)
	#endif

	#endif
	#endif


	#ifdef			portMAX_DELAY
	#define 		SYS_MAX_DELAY					portMAX_DELAY
	#else
	#define 		SYS_MAX_DELAY					0xFFFFFFFF
	#endif

	#ifdef			configTICK_RATE_HZ
	#ifdef			SYS_TICK_RATE_HZ
	#undef			SYS_TICK_RATE_HZ
	#endif
	#define 		SYS_TICK_RATE_HZ				configTICK_RATE_HZ
	#endif


	#ifdef			SYS_TICK_RATE_HZ

		#if			SYS_TICK_RATE_HZ==1000
		#define 	SYS_MS_TO_TICKS(m)				(m)
		#else
		#define 	SYS_MS_TO_TICKS(m)				(((m)*SYS_TICK_RATE_HZ)/1000)
		#endif

	#else

		#define 	SYS_MS_TO_TICKS(m)				SemaphoreSysToTicks(m)

	#endif

	typedef void   *Semaphore;
	#define 		SemaphoreCount(h)				uxQueueMessagesWaitingFromISR(h)
	#define			SemaphoreLock(h)				xSemaphoreTake(h,SYS_MAX_DELAY);}
	#define 		SemaphoreLockTimed(h,t)		   (xSemaphoreTake(h,SYS_MS_TO_TICKS(t)))? 1:0)
	#define			SemaphoreUnlock(h)				xSemaphoreGive(h)
	#define 		SemaphoreUnlockEx(h,c)			{int iSemDown=c;for(;iSemDown>0;iSemDown--)SemaphoreUnlock(h);}

	extern int 		SemaphoreSysToTicks				(unsigned uMillSec);



//*****************************************************************************
#else

#error wrong OS System

#endif



#ifdef	__cplusplus
}
#endif




#endif



