//*****************************************************************************
//*
//*
//*		ShmUtility.c
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//	This file contains functions for the access to the shared memory
//
#include	"ShmUtility.h"

#ifdef		GLOBAL_CONFIG
#define 	SHMUTILITY_C
#include	<GlobalConfig.h>
#undef		SHMUTILITY_C
#endif

#ifndef		MAX_SHM_AREAS
#define 	MAX_SHM_AREAS		32
#endif


//*****************************************************************************
//*
//*
//*		WINDOWS
//*
//*
//*****************************************************************************
#if			defined(_WIN32) && !defined(RTTARGET)
#include	<windows.h>
#include	<string.h>
#include	<stdio.h>


#pragma 	comment(lib, "advapi32.lib")


#ifdef	   _MSC_VER
#pragma		warning( disable :4996)
#pragma		warning (disable: 4244)
#pragma		warning (disable: 4311)
#pragma		warning (disable: 4312)
#endif

#ifndef	   _WIN64
#ifndef		DWORD_PTR
#define 	DWORD_PTR			DWORD
#endif
#endif

static		HANDLE				hShmSemaphor=0;

#define 	SHM_INIT()			if(!hShmSemaphor)hShmSemaphor=CreateSemaphoreW(0,1,0x70000000,0)
#define 	SHM_DO_LOCK()		WaitForSingleObject(hShmSemaphor,0xFFFFFFFF)
#define 	SHM_DO_UNLOCK()		ReleaseSemaphore(hShmSemaphor,1,0)


typedef	struct
	{
	unsigned		uKey;
	unsigned	 	uSize;
	unsigned		uCount;
	unsigned  		uOffset;
	}ShmEntry;

typedef	struct
	{
	unsigned		uMemSize;
	unsigned long 	uDataCount;
	unsigned long 	uItemCount;
	ShmEntry		sItems[1];
	}ShmData;



static	HANDLE			aShmHandles[MAX_SHM_AREAS];	// handles of the shared memory areas
static	void	 	   *aShmPointer[MAX_SHM_AREAS];	// pointers to the shared memory areas
static	unsigned 		aShmCount  [MAX_SHM_AREAS];	// open count for the shared memory areas
static	unsigned 		aShmSize   [MAX_SHM_AREAS];	// is the size of the shared memory areas
static	unsigned 		aShmKeys   [MAX_SHM_AREAS];	// unique keys of the shared memory areas
static	int				iShmMaxItem = 0;			// maximum item number in arrays
static	HINSTANCE		hShmDll	    = 0;			// DLL handle for execute areas
static	unsigned char  *pExecMemory = 0;			// Memory pointer for execute areas
static	ShmData		   *pExecData   = 0;			// Data  pointer for execute areas
static	HANDLE			hExecSem	= 0;			// Semaphore for execute areas


//******************************************************************************
//*
//*		UnProtect
//*
//******************************************************************************
//	Removes the write protection of a memory area
//	pMem		: pointer to the memory area
//	iSize		: is the size of the memory area
//	Returns 1 if ok, or 0 if an error occurs.
static BOOL UnProtect(void *pMem,int iSize)
{
MEMORY_BASIC_INFORMATION	sInfo;
DWORD						uAttr;
DWORD						uOld;


	VirtualQuery(pMem,&sInfo,sizeof(sInfo));

	if(sInfo.Protect&(PAGE_EXECUTE|PAGE_EXECUTE_READ|PAGE_EXECUTE_READWRITE|PAGE_EXECUTE_WRITECOPY))
			uAttr = PAGE_EXECUTE_WRITECOPY;
	else	uAttr = PAGE_WRITECOPY;


return VirtualProtect(pMem,iSize,uAttr,&uOld);
}



//*****************************************************************************
//*
//*		ShmCreate
//*
//*****************************************************************************
//	Creates a new shared memory area
//	uKey		: Is an unique key number for the area
//	uSize		: Is the size of the area in bytes
//	uAccess		: Are the access rights for memory area
//					SHM_WRITE	0x0001
//					SHM_EXECUTE	0x0002
//				  Note SHM_EXECUTE must be used with the SHM_WRITE flag
//	Returns a pointer to the memory area or NULL if an error ocurrs
void *ShmCreate(unsigned uKey,unsigned uSize,unsigned uAccess)
{
SECURITY_ATTRIBUTES sAttr;
char				sDesc[SECURITY_DESCRIPTOR_MIN_LENGTH];
char				cName[32];
unsigned			uFree;
unsigned			uPos;
unsigned			uOff;
HANDLE  			hShm;
void			   *pMem;
int					iAccess;
int					iMode;
int					iNum;
int					i;



	if(uSize<=0 || uKey==0)return NULL;
	if(uSize< 4)uSize=4;


	SHM_INIT();
	SHM_DO_LOCK();

	for(i=0;i<iShmMaxItem;i++)						// is the key already in use
		{
		if(aShmKeys[i]==uKey)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}
		}

	for(i=0;i<iShmMaxItem;i++)						// search for empty handle storage
		{
		if(aShmKeys[i]==0)break;
		}

	if(i>=MAX_SHM_AREAS)							// no more free handels
		{
		SHM_DO_UNLOCK();
		return NULL;
		}


	iNum = i;


//******************** execude area *******************************************


	if(uAccess&SHM_EXECUTE)							// execute access ?
		{
		if(!(uAccess&SHM_WRITE))					// check write access
			{
			SHM_DO_UNLOCK();
			return NULL;
			}

		if(!hShmDll)hShmDll=LoadLibraryA("ShmExec.dll");
		if(!hShmDll)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}

		if(!pExecMemory)
			{
				pExecMemory=(unsigned char*)GetProcAddress(hShmDll,"ExecMemory");
			if(!pExecMemory)
				{
				SHM_DO_UNLOCK();
				return NULL;
				}
			}

		if(!pExecData)
			{
				pExecData=(ShmData*)GetProcAddress(hShmDll,"ExecData");
			if(!pExecData)
				{
				SHM_DO_UNLOCK();
				return NULL;
				}
			}

		if(!hExecSem)								// create semaphore
			{
			sAttr.nLength				= sizeof(sAttr);
			sAttr.bInheritHandle		= FALSE;
			sAttr.lpSecurityDescriptor	= &sDesc;

			InitializeSecurityDescriptor(sAttr.lpSecurityDescriptor,SECURITY_DESCRIPTOR_REVISION);
			SetSecurityDescriptorDacl   (sAttr.lpSecurityDescriptor,TRUE,NULL,FALSE);

						 hExecSem = OpenSemaphoreA(SEMAPHORE_ALL_ACCESS,FALSE,"Global\\ShmExecSem");
			if(!hExecSem)hExecSem = CreateSemaphoreA(&sAttr,1,0x7000000,"Global\\ShmExecSem");
			if(!hExecSem)
				{
				SHM_DO_UNLOCK();
				return NULL;
				}
			}


		WaitForSingleObject(hExecSem,INFINITE);

													// exists the key ?
		for(uPos=0;uPos<pExecData->uItemCount;uPos++)
			{
			if(uKey!=pExecData->sItems[uPos].uKey)continue;
			ReleaseSemaphore(hExecSem,1,NULL);
			SHM_DO_UNLOCK();
			return NULL;
			}
													// enought items left
		if(pExecData->uItemCount*sizeof(ShmEntry)+sizeof(ShmData)>sizeof(long)*pExecData->uDataCount)
			{
			ReleaseSemaphore(hExecSem,1,NULL);
			SHM_DO_UNLOCK();
			return NULL;
			}
													// find empty region
		for(uPos=0,uOff=0;uPos<pExecData->uItemCount;uPos++)
			{
			   uFree = pExecData->sItems[uPos].uOffset-uOff;
			if(uFree>=uSize)break;

			uOff  = pExecData->sItems[uPos].uOffset;
			uOff += pExecData->sItems[uPos].uSize;
			}

		if(uPos>=pExecData->uItemCount)				// end of list
			{
			   uFree = pExecData->uMemSize-uOff;
			if(uFree<uSize)
				{
				ReleaseSemaphore(hExecSem,1,NULL);
				SHM_DO_UNLOCK();
				return NULL;
				}
			}

		memmove(pExecData->sItems+uPos+1,pExecData->sItems+uPos,sizeof(ShmEntry)*(pExecData->uItemCount-uPos));
		pExecData->sItems[uPos].uKey    = uKey;
		pExecData->sItems[uPos].uSize   = uSize;
		pExecData->sItems[uPos].uOffset = uOff;
		pExecData->sItems[uPos].uCount  = 1;

		pExecData->uItemCount++;

		((unsigned*)(pExecMemory+uOff))[0] = 0;		// set first 4 bytes to zero


		ReleaseSemaphore(hExecSem,1,NULL);

		if(iNum>=iShmMaxItem)iShmMaxItem=iNum+1;

		aShmHandles[iNum] = (HANDLE)0xFFFFFFFB;		// handles of the shared memory areas
		aShmPointer[iNum] = pExecMemory+uOff;		// pointers to the shared memory areas
		aShmCount  [iNum] = 1;						// open count for the shared memory areas
		aShmSize   [iNum] = uSize;					// is the size of the shared memory areas
		aShmKeys   [iNum] = uKey;					// unique keys of the shared memory areas

		SHM_DO_UNLOCK();

		return pExecMemory+uOff;
		}


//******************** normal area ********************************************


	sprintf(cName,"Global\\SHM_%X",uKey);


	iMode   = (uAccess&SHM_WRITE)? PAGE_READWRITE:PAGE_READONLY;
	iAccess	= (uAccess&SHM_WRITE)? FILE_MAP_WRITE:FILE_MAP_READ;

	   hShm = OpenFileMappingA(iAccess,TRUE,cName);
	if(hShm)										// exist the area ?
		{
		CloseHandle(hShm);
		SHM_DO_UNLOCK();
		return NULL;
		}


	sAttr.nLength				= sizeof(sAttr);	// create security descriptor
	sAttr.bInheritHandle		= FALSE;
	sAttr.lpSecurityDescriptor	= &sDesc;

	InitializeSecurityDescriptor(sAttr.lpSecurityDescriptor,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl   (sAttr.lpSecurityDescriptor,TRUE,NULL,FALSE);


		hShm = CreateFileMappingA(INVALID_HANDLE_VALUE,&sAttr,iMode,0,uSize,cName);
	if(!hShm)
		{
		SHM_DO_UNLOCK();
		return NULL;
		}

		pMem = MapViewOfFileEx(hShm,iAccess,0,0,uSize,NULL);
	if(!pMem)
		{
		SHM_DO_UNLOCK();
		CloseHandle(hShm);
		return 0;
		}

	if(uAccess&SHM_WRITE)
		{
		((unsigned*)pMem)[0] = 0;					// set first 4 bytes to zero
		}

	if(iNum>=iShmMaxItem)iShmMaxItem=iNum+1;

	aShmHandles[iNum] = hShm;						// handles of the shared memory areas
	aShmPointer[iNum] = pMem;						// pointers to the shared memory areas
	aShmCount  [iNum] = 1;							// open count for the shared memory areas
	aShmSize   [iNum] = uSize;						// is the size of the shared memory areas
	aShmKeys   [iNum] = uKey;						// unique keys of the shared memory areas

	SHM_DO_UNLOCK();



return pMem;
}

//*****************************************************************************
//*
//*		ShmOpen
//*
//*****************************************************************************
//	Opens an existing shared memory area
//	uKey		: Is an unique key number for the area
//	uSize		: Is the size of the area in bytes
//	uAccess		: Are the access rights for memory area
//					SHM_WRITE	0x0001
//					SHM_EXECUTE	0x0002
//				  Note SHM_WRITE and SHM_EXECUTE aren't allowed.
//	Returns a pointer to the memory area or NULL if an error ocurrs
void *ShmOpen(unsigned uKey,unsigned uSize,unsigned uAccess)
{
SECURITY_ATTRIBUTES			sAttr;
MEMORY_BASIC_INFORMATION	sInfo;
char						sDesc[SECURITY_DESCRIPTOR_MIN_LENGTH];
char						cName[32];
DWORD						dwProdect;
unsigned					uPos,uOff;
unsigned 					uBytes;
unsigned 					uExec;
HANDLE						hShm;
void					   *pMem;
int							iAccess;
int							iMode;
int							iNum;
int							i;



	if(uSize<=0 || uKey==0)return NULL;


	SHM_INIT();
	SHM_DO_LOCK();


	for(i=0;i<iShmMaxItem;i++)						// is the key already in use
		{
		if(aShmKeys[i]==uKey)
			{
			uExec = (aShmHandles[i]==(HANDLE)0xFFFFFFFB)? SHM_EXECUTE:0;

			if(uExec!=(uAccess&SHM_EXECUTE))pMem=NULL;
			else if(aShmSize[i]<uSize      )pMem=NULL;
			else{
				aShmCount[i]++;
				pMem = aShmPointer[i];
				}

			SHM_DO_UNLOCK();
			return pMem;
			}
		}

	for(i=0;i<iShmMaxItem;i++)						// search for empty handle storage
		{
		if(aShmKeys[i]==0)break;
		}

	if(i>=MAX_SHM_AREAS)							// no more free handels
		{
		SHM_DO_UNLOCK();
		return NULL;
		}

	iNum = i;


//******************** execute area *******************************************


	if(uAccess&SHM_EXECUTE)							// execute access ?
		{
		if(uAccess&SHM_WRITE)						// don't allow excute and write
			{
			SHM_DO_UNLOCK();
			return NULL;
			}

		if(!hShmDll)hShmDll=LoadLibraryA("ShmExec.dll");
		if(!hShmDll)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}

		if(!pExecMemory)
			{
				pExecMemory=(unsigned char*)GetProcAddress(hShmDll,"ExecMemory");
			if(!pExecMemory)
				{
				SHM_DO_UNLOCK();
				return NULL;
				}
			}

		if(!pExecData)
			{
				pExecData=(ShmData*)GetProcAddress(hShmDll,"ExecData");
			if(!pExecData)
				{
				SHM_DO_UNLOCK();
				return NULL;
				}
			}

		if(!hExecSem)								// create Semaphore
			{
			sAttr.nLength				= sizeof(sAttr);
			sAttr.bInheritHandle		= FALSE;
			sAttr.lpSecurityDescriptor	= &sDesc;
			InitializeSecurityDescriptor(sAttr.lpSecurityDescriptor,SECURITY_DESCRIPTOR_REVISION);
			SetSecurityDescriptorDacl   (sAttr.lpSecurityDescriptor,TRUE,NULL,FALSE);

						 hExecSem = OpenSemaphoreA(SEMAPHORE_ALL_ACCESS,FALSE,"Global\\ShmExecSem");
			if(!hExecSem)hExecSem = CreateSemaphoreA(&sAttr,1,0x7000000,"Global\\ShmExecSem");
			if(!hExecSem)
				{
				SHM_DO_UNLOCK();
				return NULL;
				}
			}


		WaitForSingleObject(hExecSem,INFINITE);

													// exists the key ?
		for(uPos=0;uPos<pExecData->uItemCount;uPos++)
			{
			if(uKey==pExecData->sItems[uPos].uKey)break;
			}

		if(uPos>=pExecData->uItemCount)				// found key ?
			{
			ReleaseSemaphore(hExecSem,1,NULL);
			SHM_DO_UNLOCK();
			return NULL;
			}

		if(pExecData->sItems[uPos].uSize<uSize)		// check size of region
			{
			ReleaseSemaphore(hExecSem,1,NULL);
			SHM_DO_UNLOCK();
			return NULL;
			}

		uOff = pExecData->sItems[uPos].uOffset;


		dwProdect = PAGE_EXECUTE_READ;

		if(!VirtualProtect(pExecMemory+uOff,uSize,dwProdect,&dwProdect))
			{
			ReleaseSemaphore(hExecSem,1,NULL);
			SHM_DO_UNLOCK();
			return NULL;
			}

		pExecData->sItems[uPos].uCount++;
		uSize = pExecData->sItems[uPos].uSize;
		ReleaseSemaphore(hExecSem,1,NULL);


		aShmHandles[iNum] = (HANDLE)0xFFFFFFFB;		// handles of the shared memory areas
		aShmPointer[iNum] = pExecMemory+uOff;		// pointers to the shared memory areas
		aShmCount  [iNum] = 1;						// open count for the shared memory areas
		aShmSize   [iNum] = uSize;					// is the size of the shared memory areas
		aShmKeys   [iNum] = uKey;					// unique keys of the shared memory areas

		if(iNum>=iShmMaxItem)iShmMaxItem=iNum+1;

		SHM_DO_UNLOCK();

		return pExecMemory+uOff;
		}


//******************** normal area ********************************************


	sprintf(cName,"Global\\SHM_%X",uKey);


	iMode   = (uAccess&SHM_WRITE)? PAGE_READWRITE:PAGE_READONLY;
	iAccess	= (uAccess&SHM_WRITE)? FILE_MAP_WRITE:FILE_MAP_READ;


		hShm = OpenFileMappingA(iAccess,TRUE,cName);
	if(!hShm)										// is mapping object ok ?
		{
		SHM_DO_UNLOCK();
		return NULL;
		}

		pMem = MapViewOfFileEx(hShm,iAccess,0,0,0,NULL);
	if(!pMem)										// no pointer to shm area ?
		{
		CloseHandle(hShm);
		SHM_DO_UNLOCK();
		return NULL;
		}

		uBytes = (unsigned)VirtualQuery(pMem,&sInfo,sizeof(sInfo));
	if(!uBytes || sInfo.RegionSize<uSize)			// check size of shm area
		{
		UnmapViewOfFile(pMem);
		CloseHandle(hShm);
		SHM_DO_UNLOCK();
		return NULL;
		}

	if(iNum>=iShmMaxItem)iShmMaxItem=iNum+1;

	aShmHandles[iNum] = hShm;						// handles of the shared memory areas
	aShmPointer[iNum] = pMem;						// pointers to the shared memory areas
	aShmCount  [iNum] = 1;							// open count for the shared memory areas
	aShmSize   [iNum] = (UINT)sInfo.RegionSize;		// is the size of the shared memory areas
	aShmKeys   [iNum] = uKey;						// unique keys of the shared memory areas

	SHM_DO_UNLOCK();


return pMem;
}

//*****************************************************************************
//*
//*		ShmClose
//*
//*****************************************************************************
//	Closes an existing shared memory area, if all handels are removed the
//	shared memory will be deleted
//	uKey	: Is an unique key number for the area
//	Returns a 1 if all ok or 0 if an error ocurrs
int ShmClose(unsigned uKey)
{
int			i;
unsigned 	uPos;


	if(!uKey)return 0;

	SHM_INIT();
	SHM_DO_LOCK();


	for(i=0;i<iShmMaxItem;i++)						// search for handle storage
		{
		if(aShmKeys[i]==uKey)break;
		}

	if(i>=iShmMaxItem)
		{
		SHM_DO_UNLOCK();
		return 0;
		}

	aShmCount[i]--;

	if(aShmCount[i])								// decrement access count
		{
		SHM_DO_UNLOCK();
		return 1;
		}


	if(aShmHandles[i]==(HANDLE)0xFFFFFFFB)			// execute area ?
		{
		WaitForSingleObject(hExecSem,INFINITE);

													// exists the key ?
		for(uPos=0;uPos<pExecData->uItemCount;uPos++)
			{
			if(uKey==pExecData->sItems[uPos].uKey)break;
			}

		aShmKeys[i] = 0;
		if(i+1==iShmMaxItem)iShmMaxItem--;

		if(uPos>=pExecData->uItemCount)				// found key ?
			{
			ReleaseSemaphore(hExecSem,1,NULL);
			SHM_DO_UNLOCK();
			return 0;
			}

		pExecData->sItems[uPos].uCount--;

		if(pExecData->sItems[uPos].uCount>0)		// only detach
			{
			ReleaseSemaphore(hExecSem,1,NULL);
			SHM_DO_UNLOCK();
			return 1;
			}

		memmove(pExecData->sItems+uPos,pExecData->sItems+uPos+1,sizeof(ShmEntry)*(pExecData->uItemCount-uPos-1));
		pExecData->uItemCount--;

		ReleaseSemaphore(hExecSem,1,NULL);
		SHM_DO_UNLOCK();

		return 1;
		}


	UnmapViewOfFile(aShmPointer[i]);				// remove the shared memory area
	CloseHandle(aShmHandles[i]);
	aShmKeys[i]=0;

	if(i+1==iShmMaxItem)iShmMaxItem--;

	SHM_DO_UNLOCK();


return 1;
}


//*****************************************************************************
//*
//*		Serveral types for memory locking
//*
//*****************************************************************************
#ifndef _WIN64

static	LPVOID  _stdcall  VirtualAllocNew( LPVOID lpAddress,DWORD dwSize,DWORD flAllocationType,DWORD flProtect);
typedef LPVOID (_stdcall *VirtualAllocType)( LPVOID,DWORD,DWORD,DWORD);
static	VirtualAllocType  VirtualAllocOld=0;

static	HANDLE  _stdcall  HeapCreateNew(DWORD flOptions,DWORD dwInitialSize,DWORD dwMaximumSize);
typedef HANDLE (_stdcall *HeapCreateType)(DWORD,DWORD,DWORD);
static	HeapCreateType    HeapCreateOld=0;

#define PTR_ADD(ptr,add)	(((const char*)(ptr))+(DWORD)(add))
#define WORD_SIGN(a,b)		((unsigned char)(a)+((unsigned char)(b)<<8))
#define DWORD_SIGN(a,b,c,d)	((unsigned char)(a)+((unsigned char)(b)<<8)+((unsigned char)(c)<<16)+((unsigned char)(d)<<24))


//*****************************************************************************
//*
//*		GetPeHeader
//*
//*****************************************************************************
static IMAGE_NT_HEADERS *GetPeHeader(IMAGE_DOS_HEADER *pDosHeader)
{
IMAGE_NT_HEADERS	*pPe;

	if(!pDosHeader)return 0;
	if(IsBadReadPtr(pDosHeader,sizeof(*pDosHeader)))return 0;
	if(*(DWORD*)pDosHeader==DWORD_SIGN('P','E',0,0))return (IMAGE_NT_HEADERS*)pDosHeader;
	if(pDosHeader->e_magic!=WORD_SIGN ('M','Z')    )return 0;

	pPe=(IMAGE_NT_HEADERS*)PTR_ADD(pDosHeader,pDosHeader->e_lfanew);
	if(!IsBadReadPtr(pPe,sizeof(*pPe)))
	if(pPe->Signature==DWORD_SIGN('P','E',0,0))return pPe;

	if(!pDosHeader->e_cp)return 0;

	pPe=(IMAGE_NT_HEADERS*)PTR_ADD(pDosHeader,pDosHeader->e_cblp+pDosHeader->e_cp*16-16+64);
	if(!IsBadReadPtr(pPe,sizeof(*pPe)))
	if(pPe->Signature==DWORD_SIGN('P','E',0,0))return pPe;


return 0;
}

//*****************************************************************************
//*
//*		GetEDataPtr
//*
//*****************************************************************************
static IMAGE_EXPORT_DIRECTORY *GetEDataPtr(void *pInstance)
{
LONG				 pPtr;
IMAGE_NT_HEADERS	*pPe;

		pPe=GetPeHeader((IMAGE_DOS_HEADER*)pInstance);
	if(!pPe )return 0;
		pPtr=pPe->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress;
	if(!pPtr)return 0;

return (IMAGE_EXPORT_DIRECTORY*)PTR_ADD(pPtr,pInstance);
};


//*****************************************************************************
//*
//*		SetExportProcA
//*
//*****************************************************************************
//	ndert den Zeiger auf einen Export-Funktion
//	Edata	: Zeiger auf den Edata-Abschnitt
//	Base	: Handel der Instanze
//	Name	: Name der Export-Funktion
//			  bzw. wenn Name < 0x10000 die Ordinal-Nummer
//	NewProc	: Ist der Zeiger auf die neue Funktion
//	OldProc	: Dort wird der Wert der alten Funktion gespeichert wenn != 0
//	Ergibt TRUE wenn die Funktion verndert wurde.
static BOOL SetExportProcA(IMAGE_EXPORT_DIRECTORY *pEdata,void *pBase,const char *pName,const void *pNewProc,void **pOldProc)
{
DWORD	*pNames;
DWORD	*pAddrs;
WORD	*pOrtinal;
char    *pPtr;
unsigned uCount,uCname,uIndex,uOrdNr;


	if(pOldProc)*pOldProc=0;
	if(!pEdata )return FALSE;

	uCount=pEdata->NumberOfFunctions;
	uCname=pEdata->NumberOfNames;

	pNames  =(DWORD*)PTR_ADD(pBase,pEdata->AddressOfNames);
	pAddrs  =(DWORD*)PTR_ADD(pBase,pEdata->AddressOfFunctions);
	pOrtinal=( WORD*)PTR_ADD(pBase,pEdata->AddressOfNameOrdinals);
	uOrdNr  =(DWORD )pName;

	if(!(uOrdNr&0xFFFF0000))
		{
		uOrdNr -= pEdata->Base;

		for(uIndex=0;;uIndex++)
			{
			if(uIndex>=uCname)return FALSE;
			if(pOrtinal[uIndex]==uOrdNr)break;
			}
		}
	else{
		for(uIndex=0;;uIndex++)
			{
			if(uIndex>=uCname)return FALSE;
			pPtr=(char*)pBase+pNames[uIndex];
			if(!strcmp(pName,pPtr))break;
			}
		}

	   uIndex = pOrtinal[uIndex];
	if(uIndex>=uCount)return FALSE;

	//if(IsBadWritePtr(pAddrs+uIndex,4))
	if(!UnProtect(pAddrs+uIndex,4))
		{
		return FALSE;
		}

	if(pOldProc)*pOldProc=(void*)(pAddrs[uIndex]+(DWORD)pBase);
	pAddrs[uIndex]=(DWORD)pNewProc-(DWORD)pBase;


return TRUE;
}

//*****************************************************************************
//*
//*		SetExportProcB
//*
//*****************************************************************************
//	ndert eine Funktion in der Export-Tabelle eines Moduls
//	hDll		: Handle   des Moduls mit der Export-Tabelle
//	pProc		: Name der Funktion die gendert werden soll
//	pNewPorc	: Zeiger auf die neue Funktion
//	pOldProc	: Dort wird der wert der alten Funktion gespeichert
//	Ergibt TRUE wen die Funktion gesetzt wurde
static BOOL SetExportProcB(HMODULE hDll,const char *pProc,void *pNewPorc,void **pOldProc)
{
IMAGE_EXPORT_DIRECTORY	*pEData;
char					 cPath[256];



	if(!hDll)return FALSE;
	if(!GetModuleFileName(hDll,cPath,sizeof(cPath)))return FALSE;

		pEData=GetEDataPtr(hDll);
	if(!pEData)return FALSE;


return SetExportProcA(pEData,hDll,pProc,pNewPorc,pOldProc);
}

//*****************************************************************************
//*
//*		SetExportProcC
//*
//*****************************************************************************
//	ndert eine Funktion in der Export-Tabelle eines Moduls
//	pDllName	: Dll-Name des Moduls mit der Export-Tabelle
//	pProc		: Name der Funktion die gendert werden soll
//	pNewPorc	: Zeiger auf die neue Funktion
//	pOldProc	: Dort wird der wert der alten Funktion gespeichert
//	Ergibt TRUE wen die Funktion gesetzt wurde
static BOOL SetExportProcC(const char *pDllName,const char *pProc,void *pNewPorc,void **pOldProc)
{
HMODULE hDll;

		hDll=GetModuleHandle(pDllName);
	if(!hDll)return FALSE;


return SetExportProcB(hDll,pProc,pNewPorc,pOldProc);
}

//*****************************************************************************
//*
//*		SetImportProcA
//*
//*****************************************************************************
//	ndert alle Import-Tabellen in einem Modul
//	hDll		: Handle des Modules mit den Import-Tabellen
//	pDllName	: Dll-Name der Funktion die gendert werden soll
//	pProc		: Name der Funktion die gendert werden soll
//	pNewPorc	: Zeiger auf die neue Funktion
//	pOldProc	: Dort wird der wert der alten Funktion gespeichert
//	Ergibt die Anzahl der genderten Funktionen
static BOOL SetImportProcA(HMODULE hDll,const char *pDllName,const char *pProc,void *pNewPorc,void **pOldProc)
{
IMAGE_IMPORT_DESCRIPTOR	*pIData;
IMAGE_THUNK_DATA		*pNameThunk;
IMAGE_THUNK_DATA		*pAddrThunk;
IMAGE_NT_HEADERS		*pPe;
char					*pName;
char					 cPath[256];
DWORD					 dwPos;
int						 i;



	if(!hDll)return FALSE;
	if(!GetModuleFileName(hDll,cPath,sizeof(cPath)))return FALSE;



	pPe    = (IMAGE_NT_HEADERS*)PTR_ADD(hDll,((IMAGE_DOS_HEADER*)hDll)->e_lfanew);
	pIData = (IMAGE_IMPORT_DESCRIPTOR*)PTR_ADD(hDll,pPe->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress);

	for(i=0;;i++)						// Suche Import Dll's
		{
		if(i>=256)break;
		if(!pIData[i].Name	      )break;
		if(!pIData[i].FirstThunk  )break;
		if( pIData[i].Name==0xFFFF)continue;

		pName=(char*)PTR_ADD(hDll,pIData[i].Name);
		if(stricmp(pName,pDllName))continue;


		pNameThunk=(IMAGE_THUNK_DATA*)PTR_ADD(hDll,pIData[i].OriginalFirstThunk);
		pAddrThunk=(IMAGE_THUNK_DATA*)PTR_ADD(hDll,pIData[i].FirstThunk);

		for(i=0;;i++)					// Suche Funktion
			{
			if(i>=4096)break;
			if(!pNameThunk[i].u1.AddressOfData)break;

			   dwPos=(DWORD)pNameThunk[i].u1.Ordinal;
			if(dwPos&0x80000000)continue;
			pName=(char*)PTR_ADD(hDll,dwPos+2);

			if(!strcmp(pName,pProc))   // Setze Funktions Pointer
				{
				if(pOldProc)*pOldProc=(void*)pAddrThunk[i].u1.Function;
				if(UnProtect(&pAddrThunk[i].u1.Function,sizeof(DWORD)))
					{
					*(DWORD*)&pAddrThunk[i].u1.Function = (DWORD)pNewPorc;
					return TRUE;
					}
					
				return FALSE;
				}
			}

		break;
		}



return FALSE;
}

//*****************************************************************************
//*
//*		SetImportProcB
//*
//*****************************************************************************
//	ndert alle Import-Tabellen in einem Prozess
//	pDllName	: Dll-Name der Funktion die gendert werden soll
//	pProc		: Name der Funktion die gendert werden soll
//	pNewPorc	: Zeiger auf die neue Funktion
//	pOldProc	: Dort wird der wert der alten Funktion gespeichert
//	Ergibt die Anzahl der genderten Funktionen
static int SetImportProcB(const char *pDllName,const char *pProc,void *pNewPorc,void **pOldProc)
{
MEMORY_BASIC_INFORMATION 	 sMi;
SYSTEM_INFO					 sSi;
BOOL						 bOk,bExit=FALSE;
DWORD						 dwPtr,dwOld;
char						 cPath[256];
int							 iSize,iAdd,iMask,iAnz=0;


	GetSystemInfo(&sSi);

	dwPtr = (DWORD)sSi.lpMinimumApplicationAddress;
	iAdd  =   sSi.dwPageSize;
	iMask = ~(iAdd-1);

	for(;;) 									// Suche alle Module
		{
		sMi.BaseAddress=0;
		bOk=(BOOL)VirtualQuery((void*)dwPtr,&sMi,sizeof(sMi));

		if(GetModuleFileName((HMODULE)dwPtr,cPath,sizeof(cPath)))
			{
			if(SetImportProcA((HMODULE)dwPtr,pDllName,pProc,pNewPorc,pOldProc))
				{
				iAnz++;
				}
			}

		dwOld = dwPtr;
		iSize = sMi.RegionSize&iMask;

		if(!iSize)iSize=iAdd;
			dwPtr+=iSize;
		if(!dwPtr      )break;
		if( dwPtr<dwOld)break;
		if( dwPtr>=(DWORD)sSi.lpMaximumApplicationAddress)break;
		}

return iAnz;
}



//*****************************************************************************
//*
//*		VirtualAllocNew
//*
//*****************************************************************************
//	Neue Funktion fr VirtualAlloc
//	Mit dieser Funktion wird jeder allokierte Speicher automatisch gelockt
static LPVOID _stdcall VirtualAllocNew(LPVOID pAddress,DWORD dwSize,DWORD flAllocationType,DWORD flProtect)
{
LPVOID	pMem;


	   pMem=VirtualAllocOld(pAddress,dwSize,flAllocationType,flProtect);
	if(pMem)VirtualLock(pMem,dwSize);


return pMem;
}


//*****************************************************************************
//*
//*		HeapCreateNew
//*
//*****************************************************************************
//	Neue Funktion fr HeapCreate
//	Mit dieser Funktion wird jeder allokierte Heap automatisch gelockt
static HANDLE _stdcall HeapCreateNew(DWORD flOptions,DWORD dwInitialSize,DWORD dwMaximumSize)
{
HANDLE	hHeap;


	   hHeap=HeapCreateOld(flOptions,dwInitialSize,dwMaximumSize);
	if(hHeap)VirtualLock(hHeap,dwMaximumSize);


return hHeap;
}

#endif

//*****************************************************************************
//*
//*		ShmLockMemory
//*
//*****************************************************************************
//	Locks all memory of a process
//	Returns 1 if all ok or 0 if an error occurs
int ShmLockMemory()
{
#ifndef _WIN64
SYSTEM_INFO					 sSi;
MEMORY_BASIC_INFORMATION 	 sMi;
IMAGE_DOS_HEADER			*pDosHeader;
IMAGE_NT_HEADERS			*pPeHeader;
DWORD_PTR					 dwPos,dwMask,dwMax;
DWORD_PTR					 dwOld,dwSize,dwPtr;
DWORD_PTR					 dwMaxStack,dwCurStack;
DWORD_PTR					 dwAdd,dwLock,dwLocked;
long						 lStackAdd;
void						*pStackAddr;
int							 iRetValue=1;
DWORD						 dwType,dwValue=0;
HKEY						 hKey;
LONG						 lOk;
#define 					 EXTRA_LOCK_MEM	(0x100000*16)	// 16 MB Reserve






	   lOk=RegOpenKeyExA(HKEY_LOCAL_MACHINE,"SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Memory Management",0,KEY_READ|KEY_WRITE,&hKey);
	if(lOk==ERROR_SUCCESS)
		{
		dwSize =sizeof(DWORD);							// Auslagerung des Systems verhintern
		dwType =REG_DWORD;
		dwValue=1;
		RegSetValueExA(hKey,"DisablePagingExecutive",0,dwType,(const LPBYTE)&dwValue,dwSize);
		RegCloseKey(hKey);
		}

	if(!VirtualAllocOld)								// Tauschen der VirtualAlloc Funktion
		{
		if(SetExportProcC("KERNEL32.DLL","VirtualAlloc",VirtualAllocNew,(void**)&VirtualAllocOld))
			{
			SetImportProcB("KERNEL32.DLL","VirtualAlloc",VirtualAllocNew,NULL);
			}
		if(SetExportProcC("KERNEL32.DLL","HeapCreate",HeapCreateNew,(void**)&HeapCreateOld))
			{
			SetImportProcB("KERNEL32.DLL","HeapCreate",HeapCreateNew,NULL);
			}
		}


	pDosHeader = (IMAGE_DOS_HEADER*)GetModuleHandle(0);
	pPeHeader  = (IMAGE_NT_HEADERS*)(((BYTE*)pDosHeader)+pDosHeader->e_lfanew);
	dwMaxStack = (DWORD)pPeHeader->OptionalHeader.SizeOfStackReserve;


	GetSystemInfo(&sSi);
	dwAdd  =   sSi.dwPageSize;
	dwMask = ~(sSi.dwPageSize-1);

	dwLock =  0;
	dwPtr  =  0;

	#if _WIN64
	pStackAddr = 0;
	/* REM AZ nicht untersttzt
	_asm{												// Hole den Stackpointer
		mov		rax,rsp;
		mov		pStackAddr,rax;
		}
	*/
	#else
	_asm{
		mov		eax,esp;
		mov		pStackAddr,eax;
		}
	#endif

	for(;;) 											// Suche Stack Bereich
		{
		sMi.BaseAddress=0;
		VirtualQuery((void*)dwPtr,&sMi,sizeof(sMi));

		dwOld   = dwPtr;
		dwSize  = sMi.RegionSize&dwMask;

		if(!(sMi.State&(MEM_FREE|MEM_RESERVE)))
		if(sMi.Protect&(PAGE_READWRITE|PAGE_WRITECOPY|PAGE_EXECUTE|PAGE_EXECUTE_READ|PAGE_EXECUTE_READWRITE|PAGE_EXECUTE_WRITECOPY))
			{
			if((DWORD_PTR)sMi.BaseAddress               <=(DWORD_PTR)pStackAddr)
			if((DWORD_PTR)sMi.BaseAddress+sMi.RegionSize> (DWORD_PTR)pStackAddr)
				{
				dwCurStack = (DWORD)((DWORD_PTR)sMi.BaseAddress+sMi.RegionSize-(DWORD_PTR)pStackAddr);
				lStackAdd  = (long)(dwMaxStack-dwCurStack-0x4000);


				if(lStackAdd>=1024)						// Den Stack aufs Maximum anwachsen lassen
				#if _WIN64
				/* REM AZ nicht untersttzt
				_asm{
					mov		rax,rsp
					mov		rbx,rsp
					xor		rcx,rcx
					mov		ecx,lStackAdd
					sub		rbx,256
					sub		rsp,rcx;
					shr		rcx,11
				XL:	mov		dword ptr [rbx],0
					sub		rbx,2048
					loop	XL
					mov		rsp,rax
					}
				*/
				#else
				_asm{
					mov		eax,esp
					mov		ebx,esp
					mov		ecx,lStackAdd
					sub		ebx,256
					sub		esp,ecx;
					shr		ecx,11
				XL:	mov		dword ptr [ebx],0
					sub		ebx,2048
					loop	XL
					mov		esp,eax
					}
				#endif

				break;
				}
			}

		if(!dwSize)dwSize=dwAdd;
			dwPtr+=dwSize;
		if(!dwPtr)break;
		if( dwPtr<dwOld)break;
		if( dwPtr>=(DWORD_PTR)sSi.lpMaximumApplicationAddress)break;
		}



	dwLock =  0;
	dwPtr  =  0;
	dwMax  =  EXTRA_LOCK_MEM;

	SetProcessWorkingSetSize(GetCurrentProcess(),dwMax+EXTRA_LOCK_MEM,dwMax+EXTRA_LOCK_MEM);

	for(;;) 												// Bereche WorkingSetSize
		{
		sMi.BaseAddress=0;
		VirtualQuery((void*)dwPtr,&sMi,sizeof(sMi));

		dwOld   = dwPtr;
		dwSize  = sMi.RegionSize&dwMask;

		if(!(sMi.State&(MEM_FREE|MEM_RESERVE)))
		if(!(sMi.Protect&PAGE_GUARD))
			{
			if(dwMax<sMi.RegionSize)
				{
				dwMax=(DWORD)sMi.RegionSize+0x100000*4;
				SetProcessWorkingSetSize(GetCurrentProcess(),dwMax+EXTRA_LOCK_MEM,dwMax+EXTRA_LOCK_MEM);
				}

			if(VirtualLock((void*)dwPtr,sMi.RegionSize))
				{
				dwLock+=(DWORD)sMi.RegionSize;
				VirtualUnlock((void*)dwPtr,sMi.RegionSize);
				}
			else{
				if(GetLastError()!=ERROR_INVALID_PARAMETER)
				if(GetLastError()!=ERROR_NOACCESS)
					{
					dwLock+=(DWORD)sMi.RegionSize;
					}
				}
			}

		if(!dwSize)dwSize=dwAdd;
			dwPtr += dwSize;
		if(!dwPtr)break;
		if( dwPtr<dwOld)break;
		if( dwPtr>=(DWORD_PTR)sSi.lpMaximumApplicationAddress)break;
		}


	if(!SetProcessWorkingSetSize(GetCurrentProcess(),dwLock+EXTRA_LOCK_MEM,dwLock+EXTRA_LOCK_MEM))
		{
		iRetValue=0;
		}


	dwLocked = 0;
	dwPtr    = 0;

	for(;;) 												// Den gesamten Speicher sperren
		{
		sMi.BaseAddress=0;
		VirtualQuery((void*)dwPtr,&sMi,sizeof(sMi));

		dwOld   = dwPtr;
		dwSize  = sMi.RegionSize&dwMask;

		if(!(sMi.State&(MEM_FREE|MEM_RESERVE)))
		if(!(sMi.Protect&PAGE_GUARD))
		__try
			{
			volatile BYTE *pData,bData;

			if(!IsBadReadPtr((void*)dwPtr,sMi.RegionSize))
				{
				if(VirtualLock((void*)dwPtr,sMi.RegionSize))
					{
					dwLocked += (DWORD)sMi.RegionSize;
					}

				pData=(BYTE*)dwPtr;

				for(dwPos=0;dwPos<sMi.RegionSize;dwPos+=dwAdd)
					{
					bData=pData[dwPos];
					}
				}
			}
		__except(0)
			{

			}

		if(!dwSize)dwSize=dwAdd;
			dwPtr += dwSize;
		if(!dwPtr)break;
		if( dwPtr<dwOld)break;
		if( dwPtr>=(DWORD_PTR)sSi.lpMaximumApplicationAddress)break;
		}




return iRetValue;

#else
// REM AZ fr 64-Bim muss noch etwas gemacht werden
return 0;

#endif
}

#endif


//*****************************************************************************
//*
//*
//*		LINUX
//*
//*
//*****************************************************************************
#ifdef		linux
#ifndef		LINUX
#define 	LINUX	1
#endif
#endif

#ifdef		LINUX

#define	   _GNU_SOURCE 


#include 	<errno.h>
#include 	<unistd.h>
#include    <stdlib.h>
#include	<stdio.h>
#include	<sys/io.h>
#include	<sys/types.h>
#include	<sys/ipc.h>
#include	<sys/shm.h>
#include 	<sys/sem.h>
#include	<sys/mman.h>
#include 	<sys/time.h>



#define 	SHM_SEM_KEY			0xCB38


#define		SHM_DO_LOCK()		{struct	sembuf sOper;sOper.sem_num=0;sOper.sem_op=-1;sOper.sem_flg=SEM_UNDO;semop(iShmSemahor,&sOper,1);}
#define		SHM_DO_UNLOCK()		{struct	sembuf sOper;sOper.sem_num=0;sOper.sem_op= 1;sOper.sem_flg=SEM_UNDO;semop(iShmSemahor,&sOper,1);}


static	int 			iShmSemahor =-1;			// handles of the shared memory semaphor
static	int 			aShmHandles[MAX_SHM_AREAS];	// handles of the shared memory areas
static	void	 	   *aShmPointer[MAX_SHM_AREAS];	// pointers to the shared memory areas
static	unsigned 		aShmCount  [MAX_SHM_AREAS];	// open count for the shared memory areas
static	unsigned 		aShmSize   [MAX_SHM_AREAS];	// is the size of the shared memory areas
static	unsigned 		aShmKeys   [MAX_SHM_AREAS];	// unique keys of the shared memory areas
static	unsigned char 	aShmExec   [MAX_SHM_AREAS];	// exeution posible in  the shared memory areas
static	int				iShmMaxItem = 0;			// maximum item number in arrays



//*****************************************************************************
//*
//*		ShmExit
//*
//*****************************************************************************
//	Remove shm area at procces shutdown
static void ShmExit(void)
{
int i;


	for(i=0;i<iShmMaxItem;i++)						// search for used handles
		{
		if(aShmKeys[i]!=0)continue;
		ShmClose(aShmKeys[i]);
		}

}

//******************************************************************************
//*
//*		ShmSemaphorInit
//*
//******************************************************************************
//	Initializes the semaphor for the api
static void ShmSemaphorInit()
{
struct sembuf	sOp;
int 			iSem;



	if(iShmSemahor<0)								// exists the semaphorere
		{
		   iSem = semget(SHM_SEM_KEY,2,IPC_CREAT|IPC_EXCL|0666);
		if(iSem==-1)
			{
			   iSem = semget(SHM_SEM_KEY,2,0666);	// did the semaphore already exist ?
			if(iSem==-1)return;
			}

		sOp.sem_num = 1;							// increment open count
		sOp.sem_op  = 1;
		sOp.sem_flg = SEM_UNDO;

		semop (iSem,&sOp    ,1);
		semctl(iSem,0,SETVAL,1);					// set starting value

		iShmSemahor = iSem;

		atexit(ShmExit);
		}


}



//*****************************************************************************
//*
//*		ShmCreate
//*
//*****************************************************************************
//	Creates a new shared memory area
//	uKey	: Is an unique key number for the area
//	uSize	: Is the size of the area in bytes
//	uAccess	: Are the access rights for memory area
//					SHM_WRITE	0x0001
//					SHM_EXECUTE	0x0002
//			  Note SHM_EXECUTE must be used with the SHM_WRITE flag
//	Returns a pointer to the memory area or NULL if an error ocurrs
void *ShmCreate(unsigned uKey,unsigned uSize,unsigned uAccess)
{
struct shmid_ds	sStat;
void		   *pMem;
int				iAccess;
int				iExec;
int				iMode;
int				iShm;
int				iNum;
int				i;




	if(iShmSemahor<0)								// initialize the semaphore
		{
		ShmSemaphorInit();
		if(iShmSemahor<0)return NULL;
		}



	if(uSize<=0 || uKey==0)return NULL;
	if(uSize<4)uSize=4;

	   iExec =    uAccess&SHM_EXECUTE;
	if(iExec && !(uAccess&SHM_WRITE))				// check write access
		{
		return NULL;
		}

	SHM_DO_LOCK();

	for(i=0;i<iShmMaxItem;i++)						// is the key already in use
		{
		if(aShmKeys[i]==uKey)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}
		}

	for(i=0;i<iShmMaxItem;i++)						// search for empty handle storage
		{
		if(aShmKeys[i]==0)break;
		}

	if(i>=MAX_SHM_AREAS)							// no more free handles
		{
		SHM_DO_UNLOCK();
		return NULL;
		}


	iNum = i;


	iAccess	= (uAccess&SHM_WRITE)? 0666:0444;

	   iShm = shmget(uKey,uSize,IPC_CREAT|IPC_EXCL|iAccess);
	if(iShm==-1)									// can't create shm ?
		{
		struct shmid_ds		sStat;


		   iShm = shmget(uKey,0,iAccess);
		if(iShm==-1)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}

		if(shmctl(iShm,IPC_STAT,&sStat)==-1)		// get statics	
			{
			SHM_DO_UNLOCK();
			return NULL;
			}
			
		if(sStat.shm_nattch>0)						// an other proccess is attached ?
			{
			SHM_DO_UNLOCK();
			return NULL;
			}
		
		if(sStat.shm_segsz<uSize)					// check the size of the shared memory
			{
			shmctl(iShm,IPC_RMID,NULL);
													// try to recreate it
				iShm = shmget(uKey,uSize,IPC_CREAT|IPC_EXCL|iAccess);
			if(iShm==-1)							
				{
				SHM_DO_UNLOCK();					// can't create shm ?
				return NULL;
				}
			}
		}


	if(shmctl(iShm,IPC_STAT,&sStat)!=-1)			// check attached proccesses ?
		{
		if(sStat.shm_nattch>0)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}
		}


	iMode  = (uAccess&SHM_WRITE)? 0:SHM_RDONLY;
	pMem   = shmat (iShm,NULL,iMode);				// get memory pointer to shm


	if(pMem==0 || pMem==(void*)-1)
		{
		shmctl(iShm,IPC_RMID,NULL);
		SHM_DO_UNLOCK();
		return NULL;
		}

	if(iExec)										// execute access to shm ?
		{
		iMode = (uAccess&SHM_WRITE)? PROT_EXEC|PROT_WRITE|PROT_READ:PROT_EXEC|PROT_READ;
		mprotect(pMem,uSize,iMode);
		}

	if(iNum>=iShmMaxItem)iShmMaxItem=iNum+1;

	if(uAccess&SHM_WRITE)
		{
		((unsigned*)pMem)[0] = 0;					// set first 4 bytes to zero
		}

	aShmHandles[iNum] = iShm;						// handles of the shared memory areas
	aShmPointer[iNum] = pMem;						// pointers to the shared memory areas
	aShmCount  [iNum] = 1;							// open count for the shared memory areas
	aShmSize   [iNum] = uSize;						// is the size of the shared memory areas
	aShmKeys   [iNum] = uKey;						// unique keys of the shared memory areas
	aShmExec   [iNum] = iExec;						// exeution posible in  the shared memory areas


	SHM_DO_UNLOCK();



return pMem;
}

//*****************************************************************************
//*
//*		ShmOpen
//*
//*****************************************************************************
//	Opens an existing shared memory area
//	uKey		: Is an unique key number for the area
//	uSize		: Is the size of the area in bytes
//	uAccess		: Are the access rights for memory area
//					SHM_WRITE	0x0001
//					SHM_EXECUTE	0x0002
//				  Note SHM_WRITE and SHM_EXECUTE aren't allowed.
//	Returns a pointer to the memory area or NULL if an error ocurrs
void *ShmOpen(unsigned uKey,unsigned uSize,unsigned uAccess)
{
struct shmid_ds	sStat;
void		   *pMem;
int				iAccess;
int				iExec;
int				iMode;
int				iShm;
int				iNum;
int				i;



	if(iShmSemahor<0)								// initialize the semaphore
		{
		ShmSemaphorInit();
		if(iShmSemahor<0)return NULL;
		}


	if(uSize<=0 || uKey==0)return NULL;
	if(uSize< 4)uSize=4;

	   iExec = uAccess&SHM_EXECUTE;
	if(iExec)										// don't allow excute and write
		{
		if(uAccess&SHM_WRITE)return NULL;
		}


	SHM_DO_LOCK();

	for(i=0;i<iShmMaxItem;i++)						// is the key already in use
		{
		if(aShmKeys[i]==uKey)
			{
			     if(aShmSize[i] <uSize)pMem=NULL;
			else if(aShmExec[i]!=iExec)pMem=NULL;
			else{
				aShmCount[i]++;
				pMem = aShmPointer[i];
				}

			SHM_DO_UNLOCK();

			return pMem;
			}
		}


	for(i=0;i<iShmMaxItem;i++)						// search for empty handle storage
		{
		if(aShmKeys[i]==0)break;
		}

	if(i>=MAX_SHM_AREAS)							// no free handels
		{
		SHM_DO_UNLOCK();
		return NULL;
		}

	iNum    = i;
	iAccess	= (uAccess&SHM_WRITE)? 0666:0444;

	   iShm = shmget(uKey,uSize,iAccess);			// create shm
	if(iShm==-1)
		{
		SHM_DO_UNLOCK();
		return NULL;
		}

	if(shmctl(iShm,IPC_STAT,&sStat)==-1)			// get size of shm area
		{
		sStat.shm_segsz = uSize;
		}
	else{
		if(uSize>sStat.shm_segsz)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}

		uSize = sStat.shm_segsz;
		}

	iMode  = (uAccess&SHM_WRITE)? 0:SHM_RDONLY;
	pMem   = shmat(iShm,NULL,iMode);				// get memory pointer to shm

	if(pMem==0 || pMem==(void*)-1)
		{
	    shmctl(iShm,IPC_RMID,NULL);
		SHM_DO_UNLOCK();
		return NULL;
		}

	if(iExec)										// execute access to shm ?
		{
		iMode  = (uAccess&SHM_WRITE)? PROT_EXEC|PROT_WRITE|PROT_READ:PROT_EXEC|PROT_READ;
		mprotect(pMem,uSize,iMode);
		}


	if(iNum>=iShmMaxItem)iShmMaxItem=iNum+1;



	aShmHandles[iNum] = iShm;						// handles of the shared memory areas
	aShmPointer[iNum] = pMem;						// pointers to the shared memory areas
	aShmCount  [iNum] = 1;							// open count for the shared memory areas
	aShmSize   [iNum] = uSize;						// is the size of the shared memory areas
	aShmKeys   [iNum] = uKey;						// unique keys of the shared memory areas


	SHM_DO_UNLOCK();



return pMem;
}

//*****************************************************************************
//*
//*		ShmClose
//*
//*****************************************************************************
//	Closes an existing shared memory area. If the last handle was closed
//	the shared memory area will be removed
//	uKey	: Is an unique key number for the area
//	Returns a 1 if all ok or 0 if an error ocurrs
int ShmClose(unsigned uKey)
{
struct shmid_ds		sStat;
int					i;


	if(iShmSemahor<0 || !uKey)return 0;


	SHM_DO_LOCK();

	for(i=0;i<iShmMaxItem;i++)						// search for handle storage
		{
		if(aShmKeys[i]==uKey)break;
		}

	if(i>=iShmMaxItem)
		{
		SHM_DO_UNLOCK();
		return 0;
		}

	aShmCount[i]--;

	if(aShmCount[i])								// decrement access count
		{
		SHM_DO_UNLOCK();
		return 1;
		}


	aShmKeys[i] = 0;

	shmdt (aShmPointer[i]);

	if(shmctl(aShmHandles[i],IPC_STAT,&sStat)!=-1)	// more than one proccess attached ?
		{
		if(sStat.shm_nattch==0)
			{
			shmctl(aShmHandles[i],IPC_RMID,NULL);
			}
		}
 

	if(i+1==iShmMaxItem)iShmMaxItem--;


	SHM_DO_UNLOCK();


return 1;
}


//*****************************************************************************
//*
//*		ShmLockMemory
//*
//*****************************************************************************
//	Locks all memory of a process
//	Returns 1 if all ok or 0 if an error occurs
int ShmLockMemory()
{
int		iRet;

	iRet = mlockall(MCL_CURRENT|MCL_FUTURE);


return !iRet;
}



#endif

//*****************************************************************************
//*
//*
//*		RTTARGET
//*
//*
//*****************************************************************************
#if			defined(RTTARGET)
#include	<malloc.h>
#include	<stdio.h>


static		RTKSemaphore		hShmSemaphor=0;

#define 	SHM_INIT()			if(!hShmSemaphor)hShmSemaphor=RTKOpenSemaphore(ST_COUNTING,1,0,0)
#define 	SHM_DO_LOCK()		RTKWait(hShmSemaphor)
#define 	SHM_DO_UNLOCK()		RTKSignal(hShmSemaphor)


typedef	struct
	{
	unsigned		uKey;
	unsigned	 	uSize;
	unsigned		uCount;
	unsigned  		uOffset;
	}ShmEntry;


static	void	   *aShmPointer [MAX_SHM_AREAS];	// pointers to the shared memory areas
static	unsigned 	aShmCount   [MAX_SHM_AREAS];	// open count for the shared memory areas
static	unsigned 	aShmSize    [MAX_SHM_AREAS];	// is the size of the shared memory areas
static	unsigned 	aShmKeys    [MAX_SHM_AREAS];	// unique keys of the shared memory areas
static	char  		aShmExec	[MAX_SHM_AREAS];	// exeution posible in  the shared memory areas
static	int			iShmMaxItem = 0;				// maximum item number in arrays


//*****************************************************************************
//*
//*		ShmCreate
//*
//*****************************************************************************
//	Creates a new shared memory area
//	uKey		: Is an unique key number for the area
//	uSize		: Is the size of the area in bytes
//	uAccess		: Are the access rights for memory area
//					SHM_WRITE	0x0001
//					SHM_EXECUTE	0x0002
//				  Note SHM_EXECUTE must be used with the SHM_WRITE flag
//	Returns a pointer to the memory area or NULL if an error ocurrs
void *ShmCreate(unsigned uKey,unsigned uSize,unsigned uAccess)
{
void	   *pMem;
int			iExec;
int			iNum;
int			i;



	if(uSize<=0 || uKey==0)return NULL;
	if(uSize< 4)uSize=4;

	   iExec =    uAccess&SHM_EXECUTE;				// execute access ?
	if(iExec && !(uAccess&SHM_WRITE))				// check write access
		{
		return NULL;
		}

	SHM_INIT();
	SHM_DO_LOCK();

	for(i=0;i<iShmMaxItem;i++)						// is the key already in use
		{
		if(aShmKeys[i]==uKey)
			{
			SHM_DO_UNLOCK();
			return NULL;
			}
		}

	for(i=0;i<iShmMaxItem;i++)						// search for empty handle storage
		{
		if(aShmKeys[i]==0)break;
		}

	if(i>=MAX_SHM_AREAS)							// no more free handels
		{
		SHM_DO_UNLOCK();
		return NULL;
		}

	iNum  = i;

		pMem = malloc(uSize);
	if(!pMem)
		{
		SHM_DO_UNLOCK();
		return NULL;
		}

	if(uAccess&SHM_WRITE)
		{
		((unsigned*)pMem)[0] = 0;					// set first 4 bytes to zero
		}

	if(iNum>=iShmMaxItem)iShmMaxItem=iNum+1;

	aShmPointer[iNum] = pMem;						// pointers to the shared memory areas
	aShmCount  [iNum] = 1;							// open count for the shared memory areas
	aShmSize   [iNum] = uSize;						// is the size of the shared memory areas
	aShmKeys   [iNum] = uKey;						// unique keys of the shared memory areas
	aShmExec   [iNum] = iExec;						// exeution posible in  the shared memory areas


 	SHM_DO_UNLOCK();


return pMem;
}

//*****************************************************************************
//*
//*		ShmOpen
//*
//*****************************************************************************
//	Opens an existing shared memory area
//	uKey		: Is an unique key number for the area
//	uSize		: Is the size of the area in bytes
//	uAccess		: Are the access rights for memory area
//					SHM_WRITE	0x0001
//					SHM_EXECUTE	0x0002
//				  Note SHM_WRITE and SHM_EXECUTE aren't allowed.
//	Returns a pointer to the memory area or NULL if an error ocurrs
void *ShmOpen(unsigned uKey,unsigned uSize,unsigned uAccess)
{
void   *pMem;
int		i,iExec;



	if(uSize<=0 || uKey==0)return NULL;

	   iExec = uAccess&SHM_EXECUTE;
	if(iExec)										// don't allow excute and write
		{
		if(uAccess&SHM_WRITE)return NULL;
		}

	SHM_INIT();
	SHM_DO_LOCK();

	pMem = NULL;

	for(i=0;i<iShmMaxItem;i++)						// is the key already in use
		{
		if(aShmKeys[i]!=uKey )continue;
		if(aShmExec[i]!=iExec)break;
		if(aShmSize[i]< uSize)break;

		aShmCount[i]++;
		pMem = aShmPointer[i];

		break;
		}

	SHM_DO_UNLOCK();


return pMem;
}

//*****************************************************************************
//*
//*		ShmClose
//*
//*****************************************************************************
//	Closes an existing shared memory area, if all handels are removed the
//	shared memory will be deleted
//	uKey	: Is an unique key number for the area
//	Returns a 1 if all ok or 0 if an error ocurrs
int ShmClose(unsigned uKey)
{
void	   *pMem;
int			i;


	if(!uKey)return 0;

	SHM_INIT();
	SHM_DO_LOCK();

	for(i=0;i<iShmMaxItem;i++)						// search for handle storage
		{
		if(aShmKeys[i]==uKey)break;
		}

	if(i>=iShmMaxItem)
		{
		SHM_DO_UNLOCK();
		return 0;
		}

	aShmCount[i]--;

	if(aShmCount[i])								// decrement access count
		{
		SHM_DO_UNLOCK();
		return 1;
		}

	pMem = aShmPointer[i]

	aShmKeys[i] = 0;

	if(i+1==iShmMaxItem)iShmMaxItem--;

	free(pMem);										// remove shm area

	SHM_DO_UNLOCK();


return 1;
}



//*****************************************************************************
//*
//*		ShmLockMemory
//*
//*****************************************************************************
//	Locks all memory of a process
//	Returns 1 if all ok or 0 if an error occurs
int ShmLockMemory()
{
return 1;
}

#endif

