//*****************************************************************************
//*
//*
//*     SocketUtility.h
//*
//*
//*****************************************************************************
//
//  This is a UDP/TCP/EVENT library with callback support for
//
//      LWIP
//      LINUX
//      WINDOWS
//		FREERTOS
//      RT-TARGET
//
//	Global Macros:
//
//		GLOBAL_CONFIG		Enable the configuration via GlobalConfig.h header file (1=on 0=off)
//		SU_USE_IPV6			Enable the IP-V6 support (0=disable 1=enable)
//		SU_HAS_SUBCALLS		Enable the callback support (0=off 1..3=levels)
//		NO_DYNAMIC_MEM		Use static memory for TCP handles (0=no 1=yes)
//		SOCKETS_MAX			Maximum count of static TCP handles (only if NO_DYNAMIC_MEM)
//		RECV_TIMEOUT		Default receive  timeout in seconds
//		SEND_TIMEOUT		Default transmit timeout in seconds
//		SAE_API				Is the API prefix for the functions
//		LWIP				Use the LWIP stack (only for RTTARGET,FREERTOS and WINDOWS)
//		FREERTOS			Operating system is FreeRTOS
//		RTTARGET			Operating system is RtTarget
//
//  Copyright  2018    Anton Zechner
//
//
#ifndef     __SOCKETUTILITY_H_H__
#define     __SOCKETUTILITY_H_H__

#ifdef      GLOBAL_CONFIG
#define     SOCKETUTILITY_H
#include    <GlobalConfig.h>
#undef      SOCKETUTILITY_H
#endif

#ifdef      _MSC_VER
#pragma     once
#pragma 	warning( disable :4035 )
#pragma 	warning( disable :4996 )
#pragma 	warning( disable :4514 )
#define 	SUBAPI	_cdecl
#ifndef     __cplusplus
#ifndef     inline
#define     inline  __inline
#endif
#endif
#else
#define 	SUBAPI
#endif

#ifdef      __cplusplus
extern      "C" {
#endif


#ifndef     __cplusplus
#ifndef     D
#define     D(a)
#endif
#else
#ifndef     D
#define     D(a)    a
#endif
#endif

#define     TCP_CLIENT      0						// open a client connection
#define     TCP_SERVER      1						// open a server socket
#define		TCP_BLOCKING	0x00000000				// sets the socket in the blocking mode
#define     TCP_NOBLOCKING  0x00000001				// sets the socket in the no blocking mode
#define     TCP_NODELAYACK  0x00000002				// disable nagle algorithm
#define     TCP_KEEP_ALIVE  0x00000004				// enable sending of keepalive packets
#define     TCP_REUSEADDR   0x00000008				// allows to reuse an used tcp port
#define     TCP_CLOSEDOWN   0x00000010				// close socket at TcpShutdown(...)
#define     TCP_DATABROKEN  0x00000020				// no broken signal if data received
#define     TCP_USENAGLE	0x00000040				// enable the nagle algorithm
#define     TCP_OVERLAPPED  0x00000080				// the socket is used in overlaped mode (for SocketInterface.h)
#define     TCP_USEPORT(a)  ((a)<<16)				// select this local port for the connection
#define		UDP_BLOCKING	0x00000000				// activate the blocking mode
#define     UDP_NOBLOCKING  0x00000001				// activate none-blocking mode
#define     UDP_BCASTSEND   0x00000002				// allow broadcast sending (255.255.255.255)
#define     UDP_BCASTRECV   0x00000004				// allow broadcast receiving
#define     UDP_REUSEADDR   0x00000008				// enable reuse of the address
#define     UDP_CLOSEDOWN   0x00000010				// close socket at UdpShutdown(...)
#define 	SU_SUB_TIMEOUT	0x76540000				// timeouts only for sub calls
#define     SU_IPV4_TAG     0xFFFF0000				// marker for ipv4 addresses in ipv6
#define     SU_IPV4_ID      0xD93B3BD9				// marker for ipv4 addresses is IpAddr
#ifndef     SU_USE_IPV6
#ifdef      RTTARGET
#define     SU_USE_IPV6     0
#else
#define     SU_USE_IPV6     1
#endif
#endif

#ifndef		SU_HAS_SUBCALLS
#define 	SU_HAS_SUBCALLS	0						// allow subcalls on a socket
#endif



    typedef unsigned short  IpPort;

    typedef union
        {
        unsigned            aAddr[ 4];
        unsigned short      aWord[ 8];
        unsigned char       aByte[16];
        }IpAddr6;

    typedef union
        {
        unsigned            aAddr[ 1];
        unsigned short      aWord[ 2];
        unsigned char       aByte[ 4];
        }IpAddr4;

    typedef struct
        {
        const IpAddr6       sAddr;
        const int           iSocket;
        const IpPort        usPort;
        const unsigned      aData[1];
        }UdpStruct6,*UdpHandle6;

    typedef struct
        {
        const IpAddr6       sAddr;
        const int           iSocket;
        const IpPort        usPort;
        const unsigned      aData[1];
        }TcpStruct6,*TcpHandle6;

    typedef struct
        {
        const IpAddr6       sAddr;
        const int           iSocket;
        const IpPort        usPort;
        const unsigned      aData[1];
        }EvnStruct6,*EvnHandle6;

    typedef struct
        {
        const IpAddr4       sAddr;
        const int           iSocket;
        const IpPort        usPort;
        const unsigned      aData[1];
        }UdpStruct4,*UdpHandle4;

    typedef struct
        {
        const IpAddr4       sAddr;
        const int           iSocket;
        const IpPort        usPort;
        const unsigned      aData[1];
        }TcpStruct4,*TcpHandle4;

    typedef struct
        {
        const IpAddr4       sAddr;
        const int           iSocket;
        const IpPort        usPort;
        const unsigned      aData[1];
        }EvnStruct4,*EvnHandle4;

	typedef	int (SUBAPI *SubSend)(void *pCallHandle,const void *pBuffer,int iSize);
	typedef	int (SUBAPI *SubRecv)(void *pCallHandle,      void *pBuffer,int iSize);
	typedef	int (SUBAPI *SubCall)(void *pCallHandle);

    typedef struct
        {
		void			   *pHandle;				// handle for the subcall
		SubCall				pCountOut;				// callbacks for count of input
		SubCall				pCountInp;				// callbacks for count of buffer
		SubSend				pOutWr;					// callbacks for write (TCP .... [] <--- OUT)
		SubRecv				pOutRd;					// callbacks for read  (TCP .... [] ---> OUT)
		SubSend				pInpWr;					// callbacks for read  (TCP ---> [] .... OUT)
		SubRecv				pInpRd;					// callbacks for write (TCP <--- [] .... OUT)
		SubCall				pClose;					// callbacks for close (could be NULL)
		}TcpSubCalls;

    typedef struct TcpCancelStr
        {
        int volatile        iSocket;
        int volatile        iValue;
		#ifdef __cplusplus
		TcpCancelStr(){iSocket=-1;iValue=0;}
		#endif
        }TcpCancel;

    #if SU_USE_IPV6
    typedef IpAddr6         IpAddr;
    typedef TcpHandle6      TcpHandle;
    typedef UdpHandle6      UdpHandle;
	typedef EvnHandle6      EvnHandle;
    #else
    typedef IpAddr4         IpAddr;
    typedef TcpHandle4      TcpHandle;
    typedef UdpHandle4      UdpHandle;
	typedef EvnHandle4      EvnHandle;
    #endif

	int         IpAddressExA  (IpAddr *pAddr,const char *pIp,int        *pLen  D(=0));
    int         IpAddressA    (IpAddr *pAddr,const char *pIp,const char *pMask D(=0));
    int         IpHostnameA   (IpAddr *pAddr,const char *pHostname);
    int         IpToStringA   (IpAddr *pAddr,char *pIp);
	int         IpSetBcast    (IpAddr *pAddr,unsigned uIpV4,unsigned uBits);
    int         IpIsV6AddressA(const char *pName);
    int         IpSetHostname (const char *pName);
    const char *IpGetHostname ();


    #if     SU_USE_IPV6
    static inline int IsIpV6Ex(const IpAddr *pAddr)
        {
        return (pAddr->aAddr[0]==SU_IPV4_ID || pAddr->aAddr[2]==SU_IPV4_TAG)? 0:1;
        }
    #else
    #define     IsIpV6Ex(p) 0
    #endif

    #if     SU_USE_IPV6
    static inline int IsIpV6(const IpAddr *pAddr)
        {
        return (pAddr->aAddr[0]==SU_IPV4_ID)? 0:1;
        }
    #else
    #define     IsIpV6(p)   0
    #endif


    #if     SU_USE_IPV6
    static inline int IsIpAny(const IpAddr *pAddr)
        {
        if(pAddr->aAddr[1]!=0         )return 0;
        if(pAddr->aAddr[0]==SU_IPV4_ID)return 1;
        if(pAddr->aAddr[0]!=0         )return 0;
        if(pAddr->aAddr[2]!=0         )return 0;
        if(pAddr->aAddr[3]!=0         )return 0;
        return 1;
        }
    #else
    static inline int IsIpAny(const IpAddr *pAddr)
        {
        return (pAddr && pAddr->aAddr[0]==0)? 1:0;
        }
    #endif

    #if     SU_USE_IPV6
    static inline int IsIpLoop(const IpAddr *pAddr)
        {
        if(pAddr->aAddr[0]==SU_IPV4_ID)
            {
            return (pAddr->aByte[4]==127)? 1:0;
            }

		if(pAddr->aAddr[2]==SU_IPV4_TAG)
		if(pAddr->aAddr[1]==0x00000000)
		if(pAddr->aAddr[0]==0x00000000)
		if(pAddr->aByte[12]==127      )return 1;

		if(pAddr->aAddr[3]!=0x01000000)return 0;
        if(pAddr->aAddr[2]!=0         )return 0;
        if(pAddr->aAddr[1]!=0         )return 0;
        if(pAddr->aAddr[0]!=0         )return 0;
        return 1;
        }
    #else
    static inline int IsIpLoop(const IpAddr *pAddr)
        {
        return (pAddr && pAddr->aByte[0]==127)? 1:0;
        }
    #endif

    #if     SU_USE_IPV6
    static inline unsigned GetIpV4(const IpAddr *pAddr)
        {
        if(pAddr->aAddr[0]!=SU_IPV4_ID)return 0;
        return pAddr->aAddr[1];
        }
    #else
    #define     GetIpV4(p)  (*(unsigned*)(p))
    #endif

    #if     SU_USE_IPV6
    static inline void SetIpV4(IpAddr *pAddr,unsigned uVal)
        {
        pAddr->aAddr[0] = SU_IPV4_ID;
		pAddr->aAddr[1] = uVal;
        }
    #else
    #define     SetIpV4(p,v)  *(unsigned*)(p)=(v)
    #endif

    static inline int IpCompareV4(const unsigned *pAddrA,const unsigned *pAddrB)
        {
        unsigned char   *pPtrA,*pPtrB;
        if(*pAddrA==*pAddrB)return 0;
        pPtrA=(unsigned char*)pAddrA;
        pPtrB=(unsigned char*)pAddrB;
        if(pPtrA[0]!=pPtrB[0])return pPtrA[0]-pPtrB[0];
        if(pPtrA[1]!=pPtrB[1])return pPtrA[1]-pPtrB[1];
        if(pPtrA[2]!=pPtrB[2])return pPtrA[2]-pPtrB[2];
                              return pPtrA[3]-pPtrB[3];
        }

    #if     SU_USE_IPV6
    static inline int IpCompare(const IpAddr *pAddrA,const IpAddr *pAddrB)
        {
        unsigned char   *pPtrA;
        unsigned char   *pPtrB;
        unsigned         uOffA;
        unsigned         uOffB;
        unsigned         uPos;


        if(pAddrA->aAddr[0]==SU_IPV4_ID)
            {
            uOffA = 1;
            }
        else if(pAddrA->aAddr[2]==SU_IPV4_TAG && !pAddrA->aAddr[1] && !pAddrA->aAddr[0])
            {
            uOffA = 3;
            }
        else{
            uOffA = 0;
            }


        if(pAddrB->aAddr[0]==SU_IPV4_ID)
            {
            uOffB = 1;
            }
        else if(pAddrB->aAddr[2]==SU_IPV4_TAG && !pAddrB->aAddr[1] && !pAddrB->aAddr[0])
            {
            uOffB = 3;
            }
        else{
            uOffB = 0;
            }


        if(uOffA && uOffB)
            {
            return IpCompareV4(pAddrA->aAddr+uOffA,pAddrB->aAddr+uOffB);
            }

        if(uOffA || uOffB)return -1;

        pPtrA=(unsigned char*)pAddrA;
        pPtrB=(unsigned char*)pAddrB;

        for(uPos=0;uPos<15;uPos++)
            {
            if(pPtrA[uPos]!=pPtrB[uPos])return pPtrA[uPos]-pPtrB[uPos];
            }

        return pPtrA[15]-pPtrB[15];
        }
    #else
    static inline int IpCompare(IpAddr *pAddrA,IpAddr *pAddrB)
        {
        return IpCompareV4(pAddrA->aAddr,pAddrB->aAddr);
        }
    #endif


//******************** Events *************************************************


    EvnHandle   EvnOpen       (int iSignaled D(=0));
	void	    EvnSignal     (EvnHandle hHandle);
	void	    EvnReset      (EvnHandle hHandle);
	int         EvnClose      (EvnHandle hHandle);
    int			EvnWait		  (EvnHandle hHandle,int iTimeout);
	int         EvnWaitMulti  (void     *pHandle,int iCount,int iTimeout,int iOffset D(=0));


//******************** UDP ****************************************************


    UdpHandle   UdpOpen      (IpAddr   *pAddr,IpPort uPort,int iFlags D(=0));
    int         UdpClose     (UdpHandle hHandle);
    int         UdpShutdown  (UdpHandle hHandle);
    int         UdpCloseDown (UdpHandle hHandle,int iOn);
    int         UdpBlock     (UdpHandle hHandle,int iOn);
    int         UdpPut       (UdpHandle hHandle,IpAddr *pTarget,const void *pMsg,int iLen);
    int         UdpPutTo     (UdpHandle hHandle,IpAddr *pTarget,const void *pMsg,int iLen,             unsigned short  uPort);
    int         UdpGet       (UdpHandle hHandle,IpAddr *pFrom  ,      void *pMsg,int iLen,             unsigned short *pPort D(=0));
    int         UdpGetWait   (UdpHandle hHandle,IpAddr *pFrom  ,      void *pMsg,int iLen,int iTimeout,unsigned short *pPort D(=0));
    int         UdpGetLocal  (UdpHandle hHandle,IpAddr *pAddr,unsigned short *pPort D(=0));
    int         UdpGetCount  (UdpHandle hHandle);
    int         UdpSetBuffers(UdpHandle hHandle,int iRecvSize,int iSendSize);
    int         UdpWait      (UdpHandle hHandle,int iTimeout);

    static inline int UdpWaitMulti(UdpHandle *pHandle,int iCount,int iTimeout,int iOffset D(=0))
		{
		return EvnWaitMulti(pHandle,iCount,iTimeout,iOffset);
		}

    static inline int UdpGetAddr(UdpHandle hHandle,IpAddr *pAddr)
        {
        #if SU_USE_IPV6

        if(!hHandle)
            {
            pAddr->aAddr[0]=SU_IPV4_ID;
            pAddr->aAddr[1]=0;
            return 0;
            }

        pAddr->aAddr[0]=((IpAddr*)hHandle)->aAddr[0];
        pAddr->aAddr[1]=((IpAddr*)hHandle)->aAddr[1];
        pAddr->aAddr[2]=((IpAddr*)hHandle)->aAddr[2];
        pAddr->aAddr[3]=((IpAddr*)hHandle)->aAddr[3];
        return 1;

        #else

        if(!hHandle)
            {
            pAddr->aAddr[0]=0;
            return 0;
            }

        pAddr->aAddr[0] = ((IpAddr*)hHandle)->aAddr[0];
        return 1;

        #endif
        }

    static inline int UdpGetPort(UdpHandle hHandle)
        {
        if(!hHandle)return 0;
        return *(unsigned short*)(((char*)hHandle)+sizeof(IpAddr)+sizeof(int));
        }


//******************** TCP ****************************************************


    int         TcpOpenCancel (TcpCancel*pCancel);
	int			TcpCancelDone (TcpCancel*pCancel);
    TcpHandle   TcpOpenWait   (IpAddr   *pAddr,unsigned short uPort,int iTimeout,TcpCancel *pCancel D(=0),int iServer D(=0),int iFalgs D(=0));
    TcpHandle   TcpAccept     (TcpHandle hHandle,IpAddr *pAddr D(=0),unsigned short *uPort D(=0));
    TcpHandle   TcpAcceptWait (TcpHandle hHandle,IpAddr *pAddr      ,unsigned short *uPort,int iTimeout);
    int         TcpGetLocal   (TcpHandle hHandle,IpAddr *pAddr      ,unsigned short *pPort D(=0));
    int         TcpClose      (TcpHandle hHandle);
    int         TcpShutdown   (TcpHandle hHandle);
    int         TcpSetFlags   (TcpHandle hHandle,int iFlags);
    int         TcpCloseDown  (TcpHandle hHandle,int iOn);
    int         TcpBlock      (TcpHandle hHandle,int iOn);
    int         TcpPut        (TcpHandle hHandle,const void *pMsg,int iLen);
    int         TcpGet        (TcpHandle hHandle,      void *pMsg,int iLen);
    int         TcpGetWait    (TcpHandle hHandle,      void *pMsg,int iLen,int iTimeout);
    int         TcpGetNext    (TcpHandle hHandle,      void *pMsg,int iLen,int iTimeout);
    int         TcpGetCount   (TcpHandle hHandle);
	int         TcpIsBrokenEx (TcpHandle hHandle,int iMode);
    int         TcpSetBuffers (TcpHandle hHandle,int iRecvSize,int iSendSize);
    int         TcpSendTimeout(TcpHandle hHandle,int iTimeout);
    int         TcpRecvTimeout(TcpHandle hHandle,int iTimeout);
	int			TcpWaitServer (TcpHandle hHandle,int iTimeout);
	int         TcpWaitSend   (TcpHandle hHandle,int iTimeout);
    int         TcpWait       (TcpHandle hHandle,int iTimeout);
	int         TcpSubFlash   (TcpHandle hHandle);
	int			TcpAddSubCall (TcpHandle hHandle,const TcpSubCalls *pNewCall,void *pHandleInp D(=0));
	int			TcpSetCancel  (TcpHandle hHandle,TcpCancel *pCancel,int iOn);


    static inline int TcpWaitMulti(TcpHandle *pHandle,int iCount,int iTimeout,int iOffset D(=0))
		{
		return EvnWaitMulti(pHandle,iCount,iTimeout,iOffset);
		}

    static inline TcpHandle TcpOpen(IpAddr *pAddr,unsigned short uPort,int iServer D(=0),int iFlags D(=0))
        {
        return TcpOpenWait(pAddr,uPort,0,0,iServer,iFlags);
        }

    static inline int TcpGetAddr(TcpHandle hHandle,IpAddr *pAddr)
        {
        #if SU_USE_IPV6

        if(!hHandle)
            {
            pAddr->aAddr[0]=SU_IPV4_ID;
            pAddr->aAddr[1]=0;
            return 0;
            }

        pAddr->aAddr[0]=((IpAddr*)hHandle)->aAddr[0];
        pAddr->aAddr[1]=((IpAddr*)hHandle)->aAddr[1];
        pAddr->aAddr[2]=((IpAddr*)hHandle)->aAddr[2];
        pAddr->aAddr[3]=((IpAddr*)hHandle)->aAddr[3];
        return 1;

        #else

        if(!hHandle)
            {
            pAddr->aAddr[0]=0;
            return 0;
            }

        *pAddr=*(IpAddr*)hHandle;
        return 1;

        #endif
        }

    static inline int TcpGetPort(TcpHandle hHandle)
        {
        if(!hHandle)return 0;
        return *(unsigned short*)(((char*)hHandle)+sizeof(IpAddr)+sizeof(int));
        }

	static inline int TcpIsBroken(TcpHandle hHandle)
		{
		return TcpIsBrokenEx(hHandle,0);
		}



#ifdef  __cplusplus
}
#endif


//******************** Unicode ************************************************


#if defined(__cplusplus) && defined(WIN32)

    #include    <wchar.h>

	extern "C"    int IpAddressExW  (IpAddr *pAddr,const wchar_t *pIp,int           *pLen =0);
    extern "C"    int IpAddressW    (IpAddr *pAddr,const wchar_t *pIp,const wchar_t *pMask=0);
    extern "C"    int IpHostnameW   (IpAddr *pAddr,const wchar_t *pHostname);
    extern "C"    int IpToStringW   (IpAddr *pAddr,      wchar_t *pIp);
    extern "C"    int IpIsV6AddressW(              const wchar_t *pName);

    static inline int IpAddressEx   (IpAddr *pAddr,const wchar_t *pIp,int           *pLen =0){return IpAddressExW(pAddr,pIp,pLen );}
    static inline int IpAddress     (IpAddr *pAddr,const wchar_t *pIp,const wchar_t *pMask=0){return IpAddressW  (pAddr,pIp,pMask);}
    static inline int IpHostname    (IpAddr *pAddr,const wchar_t *pHostname                 ){return IpHostnameW (pAddr,pHostname);}
    static inline int IpToString    (IpAddr *pAddr,      wchar_t *pIp                       ){return IpToStringW (pAddr,pIp);}
    static inline int IpIsV6Address (              const wchar_t *pName                     ){return IpIsV6AddressW(pName);}

	static inline int IpAddressEx   (IpAddr *pAddr,const char    *pIp,int           *pLen =0){return IpAddressExA(pAddr,pIp,pLen );}
    static inline int IpAddress     (IpAddr *pAddr,const char    *pIp,const char    *pMask=0){return IpAddressA  (pAddr,pIp,pMask);}
    static inline int IpHostname    (IpAddr *pAddr,const char    *pHostname                 ){return IpHostnameA (pAddr,pHostname);}
    static inline int IpHostname    (IpAddr *pAddr                                          ){return IpHostnameA (pAddr,NULL);}
    static inline int IpToString    (IpAddr *pAddr,      char    *pIp                       ){return IpToStringA (pAddr,pIp);}
    static inline int IpIsV6Address (              const char    *pName                     ){return IpIsV6AddressA(pName);}

#else

    #define IpAddress       IpAddressA
	#define IpAddressEx     IpAddressExA
    #define IpHostname      IpHostnameA
    #define IpToString      IpToStringA
    #define IpIsV6Address   IpIsV6AddressA

#endif





#undef  D

#endif


