//*****************************************************************************
//*
//*
//*     ThreadUtility.h
//*
//*
//*****************************************************************************
//
//	Thread Module for WIN32,LINUX,FREERTOS,CIMSIS and RTTARGET
//
//	Global Macros:
//
//		GLOBAL_CONFIG		Enable the configuration via GlobalConfig.h header file (1=on 0=off)
//		THREAD_CHECK		Store ckeck points with ThreadCheck()/ThreadCheckSub() (1=on 0=off)
//		THREAD_TLS_SIZE		Size of TLS data for the threads (0=none, 1...n=size)
//		THREAD_NAMES		Size of thread names (0=no names, 1...n=size, not for CIMSIS)
//		FREERTOS			Operating system is FreeRTOS
//		RTTARGET			Operating system is RtTarget
//		RTL					Operating system is Cimsis (Keil RTX)
//
//
//	Copyright Anton Zechner (c) 2018
//
//
#ifndef   __THREAD_H__
#define   __THREAD_H__

#ifdef      GLOBAL_CONFIG
#define     THREAD_H
#include    <GlobalConfig.h>
#undef      THREAD_H
#endif

#if         defined(_CMSIS_OS_H) || defined(RTL)
#include    "cmsis_os.h"
#endif

#ifdef    __cplusplus
extern      "C" {
#endif

#ifdef     _MSC_VER
#pragma     once
#pragma		warning( disable :4035 )
#pragma		warning( disable :4996 )
#pragma		warning( disable :4514 )
#endif

#define     THREAD_GLOBAL   0x80000000                  // absolute global priority for threads

#ifndef     THREAD_CHECK
#ifdef      RTTARGET
#define     THREAD_CHECK    1                           // store ckeck points
#else
#define     THREAD_CHECK    0
#endif
#endif

#ifdef    __ICCARM__
#ifndef   __cdecl
#define   __cdecl
#endif
#ifndef    _cdecl
#define    _cdecl
#endif
#endif

#ifdef    __GNUC__
#ifndef   __cdecl
#define   __cdecl
#endif
#ifndef    _cdecl
#define    _cdecl
#endif
#endif

#ifdef      LINUX
#ifndef     linux
#define     linux           1
#endif
#endif


//********************* LINUX **************************************************
#ifdef  linux

    typedef void   *ThreadMutex;
    typedef void   *ThreadSemaphore;

    ThreadSemaphore ThreadSemaphoreInitEx           (int iNum);
    #define         ThreadSemaphoreInit(a)           ThreadSemaphoreInitEx(1)
    #define         ThreadSemaphoreRelease(a)        ThreadSemaphoreReleaseEx(a,1)
    #define         ThreadSemaphoreReleaseIsr(a)     ThreadSemaphoreReleaseEx(a,1)
    void            ThreadSemaphoreExit             (ThreadSemaphore hSemaphore);
    extern  void    ThreadSemaphoreWait             (ThreadSemaphore hSemaphore);
    extern  int     ThreadSemaphoreTimed            (ThreadSemaphore hSemaphore,unsigned uMillisec);
    extern  int     ThreadSemaphoreCount            (ThreadSemaphore hSemaphore);
    extern  void    ThreadSemaphoreReleaseEx        (ThreadSemaphore hSemaphore,int iCount);
    #define         ThreadMutexInit()                ThreadMutexInitEx(0)
    ThreadMutex     ThreadMutexInitEx               (int         iLock);
    extern  int     ThreadMutexTake                 (ThreadMutex hMutex);
    extern  void    ThreadMutexWait                 (ThreadMutex hMutex);
    extern  void    ThreadMutexRelease              (ThreadMutex hMutex);
    extern  void    ThreadMutexExit                 (ThreadMutex    hMutex);

    extern  void    ThreadSleep                     (unsigned uMillSec);
    extern  char   *ThreadTlsPointer                ();
    unsigned        ThreadTickCount                 ();


//********************* RTTARGET **********************************************
#elif defined(RTTARGET)

    #ifdef         _MSC_VER
    #ifndef        _MT
    #error          "Multithread not activated"
    #endif
    #endif

    #ifndef       __EVENTUTILITY_H__
    #ifndef       __THREAD_C__
    #ifndef        _CLOCK_H
    extern          unsigned long   __declspec(dllimport)   _cdecl  CLKTicksToMilliSecs(         long);
    extern          long            __declspec(dllimport)   _cdecl  CLKMilliSecsToTicks(unsigned long);
    #endif
    #ifndef        _RTK32_H
    struct          rtkTCBType;
    struct          rtkSemaType;
    extern          void            __declspec(dllimport)   _cdecl  RTKDelay      (long);
    extern          void            __declspec(dllimport)   _cdecl  RTKSignal     (struct rtkSemaType*);
    extern          int             __declspec(dllimport)   _cdecl  RTKWaitTimed  (struct rtkSemaType*,long);
    extern          void            __declspec(dllimport)   _cdecl  RTKWait       (struct rtkSemaType*);
    extern          void            __declspec(dllimport)  *_cdecl  RTKGetUserData(struct rtkTCBType *,int);
    extern          void            __declspec(dllimport)   _cdecl  RTKResetEvent (struct rtkSemaType*);
    extern          void            __declspec(dllimport)   _cdecl  RTKPulse      (struct rtkSemaType*);
    #endif
    #endif
    #endif

    typedef void   *ThreadMutex;
	typedef void   *ThreadSemaphore;

    #define         ThreadSemaphoreInit()           ThreadSemaphoreInitEx(1)
    #define         ThreadSemaphoreWait(h)          RTKWait     ((struct rtkSemaType*)(h))
    #define         ThreadSemaphoreTimed(h,t)       RTKWaitTimed((struct rtkSemaType*)(h),CLKMilliSecsToTicks((long)(t)))
    #define         ThreadSemaphoreRelease(h)       RTKSignal   ((struct rtkSemaType*)(h))
    #define         ThreadSemaphoreCount(h)   ((int)RTKSemaValue((struct rtkSemaType*)(h)))
    #define         ThreadSemaphoreReleaseIsr(h)    RTKSignal(h)        // REM AZ noch testen
    #define         ThreadMutexTake(h)              RTKWaitCond ((struct rtkSemaType*)(h))
    #define         ThreadMutexWait(h)              RTKWait     ((struct rtkSemaType*)(h))
    #define         ThreadMutexRelease(h)           RTKSignal   ((struct rtkSemaType*)(h))
    #define         ThreadMutexInit()               ThreadMutexInitEx(0)

    #define         ThreadSleep(t)                  RTKDelay(CLKMilliSecsToTicks(t))
    #define         ThreadTlsPointer()              ((char*)RTKGetUserData(RTKCurrentTaskHandle(),iThreadTlsNum))
    void            ThreadSemaphoreReleaseEx        (ThreadSemaphore hSemaphore,int iCount);
    void            ThreadSemaphoreExit             (ThreadSemaphore hSemHandle);
    void           *ThreadSemaphoreInitEx           (int iInitCount);
    void            ThreadMutexExit                 (ThreadSemaphore hSemHandle);
    void           *ThreadMutexInitEx               (int iLock);
    unsigned        ThreadTickCount();
    extern int      iThreadTlsNum;


//********************* FREERTOS **********************************************
#elif defined(FREERTOS)

    #ifdef         _MSC_VER
    #ifndef        _MT
    #error          "Multithread not activated"
    #endif
    #endif

    #ifdef        __ICCARM__
    #ifndef        _cdecl
    #define        _cdecl
    #endif
    #ifndef       __cdecl
    #define       __cdecl
    #endif
    #endif
    #ifndef       __THREAD_C__
    #ifndef         INC_TASK_H

    void            vTaskDelay(const unsigned long);
    unsigned long   xTaskGetTickCount(void);
    void           *xTaskGetApplicationTaskTag(void*);
    void           *xTaskGetCurrentTaskHandle(void);

    #endif
    #endif

    #ifndef       __THREAD_C__
    #ifndef         QUEUE_H

    extern void     vQueueDelete(void *);
    extern void    *xQueueCreateCountingSemaphore(const unsigned long,const unsigned long);
    extern long     xQueueGenericReceive    (void*,const void *,unsigned long,const long);
    extern long     xQueueGenericSend       (void*,const void *,unsigned long,const long);
	extern long     xQueueTakeMutexRecursive(void *,unsigned long);
	extern void    *xQueueCreateMutex       (const unsigned char);
	extern long     xQueueGiveMutexRecursive(void *);
    extern unsigned uxQueueMessagesWaitingFromISR(void*);

    #endif
    #endif

    #ifndef       __THREAD_C__
    #ifndef         SEMAPHORE_H

    #ifndef         xSemaphoreCreateCounting
    #define         xSemaphoreCreateCounting(c,n)   xQueueCreateCountingSemaphore(c,n)
    #endif

    #ifndef         vSemaphoreDelete
    #define         vSemaphoreDelete(h)             vQueueDelete(h)
    #endif

    #ifndef         xSemaphoreTake
    #define         xSemaphoreTake(h,t)             xQueueGenericReceive(h,0,t,0)
    #endif

    #ifndef         xSemaphoreGive
    #define         xSemaphoreGive(h)               xQueueGenericSend(h,0,0,0)
    #endif

    #endif
    #endif


    #ifdef          portMAX_DELAY
    #define         SYS_MAX_DELAY                   portMAX_DELAY
    #else
    #define         SYS_MAX_DELAY                   0xFFFFFFFF
    #endif

    #ifdef          configTICK_RATE_HZ
    #ifdef          SYS_TICK_RATE_HZ
    #undef          SYS_TICK_RATE_HZ
    #endif
    #define         SYS_TICK_RATE_HZ                configTICK_RATE_HZ
    #endif


    #ifdef          SYS_TICK_RATE_HZ

        #if         SYS_TICK_RATE_HZ==1000
        #define     ThreadSleep(t)                  vTaskDelay(t)
        #define     ThreadTickCount()               xTaskGetTickCount()
        #define     SYS_MS_TO_TICKS(m)              (m)
        #else
        #define     ThreadSleep(t)                  ThreadSleepInternal(t)
        #define     ThreadTickCount()               ThreadTickCountInternal()
        #define     SYS_MS_TO_TICKS(m)              (((m)*SYS_TICK_RATE_HZ+999)/1000)
        #endif

    #else

        #define     ThreadSleep(t)                  ThreadSleepInternal(t)
        #define     ThreadTickCount()               ThreadTickCountInternal()
        #define     SYS_MS_TO_TICKS(m)              ThreadSysToTicks(m)

    #endif

    typedef void   *ThreadMutex;
    typedef void   *ThreadSemaphore;
    #define         ThreadSemaphoreCount(h)         uxQueueMessagesWaitingFromISR(h)
    #define         ThreadSemaphoreInit()           xSemaphoreCreateCounting(0x70000000,1)
    #define         ThreadSemaphoreInitEx(n)        xSemaphoreCreateCounting(0x70000000,n)
    #define         ThreadSemaphoreExit(h)          vSemaphoreDelete(h)
    #define         ThreadSemaphoreWait(h)          xSemaphoreTake(h,SYS_MAX_DELAY);
    #define         ThreadSemaphoreTimed(h,t)      (xSemaphoreTake(h,SYS_MS_TO_TICKS(t))? 1:0)
    #define         ThreadSemaphoreRelease(h)       xSemaphoreGive(h)
    #define         ThreadSemaphoreReleaseEx(h,c)   {int iSemDown=c;for(;iSemDown>0;iSemDown--)ThreadSemaphoreRelease(h);}

    #define         ThreadMutexInit()               xQueueCreateMutex(4)
    #define         ThreadMutexTake(h)              xQueueTakeMutexRecursive(h,0)
    #define         ThreadMutexWait(h)              xQueueTakeMutexRecursive(h,SYS_MAX_DELAY)
    #define         ThreadMutexRelease(h)           xQueueGiveMutexRecursive(h)
    #define         ThreadMutexExit(h)              vSemaphoreDelete(h)

    #define         ThreadTlsPointer()              ((char*)xTaskGetApplicationTaskTag(xTaskGetCurrentTaskHandle()))

    extern unsigned ThreadTickCountInternal         ();
    extern void    *ThreadMutexInitEx               (int iLock);
    extern void     ThreadSleepInternal             (unsigned uMillSec);
    extern int      ThreadSysToTicks                (unsigned uMillSec);
    extern void     ThreadSemaphoreReleaseIsr       (ThreadSemaphore hSemaphore);

    #ifndef         __THREAD_C__
    #define         main    FreeRtosMainTask
    #endif

//********************* CIMSIS or Keil RTX ************************************
#elif defined(_CMSIS_OS_H) || defined(RTL)

    #ifndef        _cdecl
    #define        _cdecl
    #endif

    #ifndef       __cdecl
    #define       __cdecl
    #endif

    #ifndef         THREAD_CHECK
    #define         THREAD_CHECK    0
    #else
    #undef          THREAD_CHECK
    #define         THREAD_CHECK    0
    #endif

    #ifndef         THREAD_NAMES
    #define         THREAD_NAMES    0
    #else
    #undef          THREAD_NAMES
    #define         THREAD_NAMES    0
    #endif

    #ifdef          THREAD_TLS_SIZE
    #if             THREAD_TLS_SIZE>0
    #error          RTL not supports TLS memory
    #endif
    #endif

    typedef void   *ThreadMutex;
    typedef void   *ThreadSemaphore;
    void            ThreadSemaphoreExit             (ThreadSemaphore hSemHandle);
    #define         ThreadSemaphoreInit()           ThreadSemaphoreInitEx(1)
    ThreadSemaphore ThreadSemaphoreInitEx           (int iNum);

    extern  int     ThreadSemaphoreCount            (ThreadSemaphore h);
    extern  void    ThreadSemaphoreReleaseEx        (ThreadSemaphore hSemHandle,int iCount);
    #define         ThreadSemaphoreWait(h)          osSemaphoreWait   ((osSemaphoreId)h,osWaitForever)
    #define         ThreadSemaphoreTimed(h,t)       osSemaphoreWait   ((osSemaphoreId)h,t)
    #define         ThreadSemaphoreRelease(h)       osSemaphoreRelease((osSemaphoreId)h)

    #define         ThreadSleep(uMillSec)           osDelay(uMillSec)
    #define         ThreadTlsPointer()              0
    unsigned        ThreadTickCount(void);

//********************* WINDOWS ***********************************************
#else

    #ifdef         _MSC_VER
    #ifndef        _MT
    #error          "Multithread not activated"
    #endif
    #endif

	#define 		CSEC_TYPE		struct _RTL_CRITICAL_SECTION
	#define 		SEMW_TYPE		struct _SECURITY_ATTRIBUTES

    #ifndef       __THREAD_C__
    #if            !defined(_WINDOWS_) && !defined(_WINBASE_)
	#if			   _MSC_VER>=1300
	#include		<wchar.h>
	#define			WCHAR_TYPE	wchar_t
	#else
	#define			WCHAR_TYPE  unsigned short
	#endif

    extern          void            __declspec(dllimport) * __stdcall   CreateSemaphoreW(SEMW_TYPE*,long,long,const WCHAR_TYPE*);
    extern          unsigned long   __declspec(dllimport)   __stdcall   WaitForSingleObject(void *,unsigned long);
    extern          unsigned long   __declspec(dllimport)   __stdcall   GetTickCount(void);
    extern          int             __declspec(dllimport)   __stdcall   ReleaseSemaphore(void *,long,long*);
    extern          int             __declspec(dllimport)   __stdcall   CloseHandle(void*);
    extern          void            __declspec(dllimport)   __stdcall   Sleep(unsigned long);
    extern          void            __declspec(dllimport) * __stdcall   TlsGetValue(unsigned long);
    extern          void            __declspec(dllimport)   __stdcall   EnterCriticalSection(CSEC_TYPE*);
    extern          void            __declspec(dllimport)   __stdcall   LeaveCriticalSection(CSEC_TYPE*);
    extern          int             __declspec(dllimport)   __stdcall   TryEnterCriticalSection(CSEC_TYPE*);
	#undef			WCHAR_TYPE
    #endif
    #endif

    #ifdef         _DEBUG
    int             ThreadHandleError(const char *pFile,int iLine);
    #define         ThreadHandleCheck(a,b,c)    (a || !ThreadHandleError(b,c))
    #else
    #define         ThreadHandleCheck(a,b,c)    1
    #endif

    typedef void   *ThreadMutex;
    typedef void   *ThreadSemaphore;
    #define         ThreadSemaphoreInit()           CreateSemaphoreW(0,1,0x70000000,0)
    #define         ThreadSemaphoreInitEx(n)        CreateSemaphoreW(0,n,0x70000000,0)
    #define         ThreadSemaphoreExit(h)          CloseHandle(h)
    #define         ThreadSemaphoreWait(h)          {if(ThreadHandleCheck(h,__FILE__,__LINE__))   WaitForSingleObject(h,0xFFFFFFFF);}
    #define         ThreadSemaphoreTimed(h,t)       (  (ThreadHandleCheck(h,__FILE__,__LINE__) && WaitForSingleObject(h,t)==0)? 1:0)
    #define         ThreadSemaphoreRelease(h)       ReleaseSemaphore(h,1,0)
    #define         ThreadSemaphoreReleaseIsr(h)    ReleaseSemaphore(h,1,0)
    #define         ThreadSemaphoreReleaseEx(h,c)   ReleaseSemaphore(h,c,0)
    #define         ThreadMutexInit()               ThreadMutexInitEx(0)
    #define         ThreadMutexTake(h)              TryEnterCriticalSection((CSEC_TYPE*)(h))
    #define         ThreadMutexWait(h)              EnterCriticalSection((CSEC_TYPE*)(h))
    #define         ThreadMutexRelease(h)           LeaveCriticalSection((CSEC_TYPE*)(h))

    #define         ThreadTickCount()               GetTickCount()
    #define         ThreadSleep(t)                  Sleep(t)
    #define         ThreadTlsPointer()              ((char*)TlsGetValue(iThreadTlsNum))

    extern void    *ThreadMutexInitEx               (int             iLock);
    extern void     ThreadMutexExit                 (ThreadMutex     hMutex);
    extern int      ThreadSemaphoreCount            (ThreadSemaphore hSemaphore);
    extern int      iThreadTlsNum;

#endif

//******************** General Functions **************************************


extern unsigned     ThreadSecCount(void);
extern void        *ThreadStartEx (void(_cdecl  *pStartAddress)(void*),unsigned uStackSize,void *pParameter,int iPriority,const char *pName);
#define             ThreadStart(pStartAddress,uStackSize,pParameter,iPriority) ThreadStartEx(pStartAddress,uStackSize,pParameter,iPriority,0)
extern void         ThreadSetName (const char *pName);
extern const char  *ThreadGetName (void);




#if THREAD_CHECK
extern void         ThreadCheckSet   (const char *pFile,int   iLine);
extern void         ThreadCheckSetSub(const char *pFile,int   iLine  ,unsigned uHandle);
extern int          ThreadCheckGet   (void       *hTask,char *pBuffer,int iSize,int iAddPath);
extern int          ThreadCheckGetSub(void       *hTask,char *pBuffer,int iSize,int iAddPath);
#define             ThreadCheck()       ThreadCheckSet(__FILE__,__LINE__)
#define             ThreadCheckSub(h)   ThreadCheckSet(__FILE__,__LINE__,h)
#else
#define             ThreadCheck()
#define             ThreadCheckSub(h)
#endif


#ifdef  __cplusplus
}
#endif

#endif
