//*****************************************************************************
//*
//*
//*		ThreadUtilityRtl.c
//*
//*
//*****************************************************************************
//
//	Thread Module for Keil CMSIS Realtime OS for Cortex-M CPU's
//	Copyright Anton Zechner (c) 2013
//
#define 	__THREAD_C__
#include	"ThreadUtility.h"

#ifndef  	RTL
#error 	 	This file is only for the Keil-RTL-OS
#endif

#ifndef		THREAD_PRIOR_EX
#define		THREAD_PRIOR_EX			1
#endif

#ifndef		THREAD_BASE_PRIOR
#define		THREAD_BASE_PRIOR		5
#endif

#ifndef		THREAD_SEM_MAX
#define		THREAD_SEM_MAX			32
#endif


typedef struct
	{
	unsigned char	cb_type;                 	// Control Block Type
	unsigned char	mask;                    	// Semaphore token mask
	unsigned short	tokens;                  	// Semaphore tokens
	void		   *p_lnk;           			// Chain of tasks waiting for tokens
	unsigned char	used; 	                	// Is the semahore used
	unsigned char	dummy[3];                 	// Unused bytes
	}P_SCB_;

typedef struct
	{
	unsigned char	cb_type;                	// Control Block Type
	unsigned char	state;                  	// Task state
	unsigned char	prio;                    	// Execution priority
	unsigned char	task_id;                 	// Task ID value for optimized TCB access
	void		   *p_lnk;           			// Link pointer for ready/sem. wait list
	void		   *p_rlnk;          			// Link pointer for sem./mbx lst backwards
	void		   *p_dlnk;          			// Link pointer for delay list
	void		   *p_blnk;          			// Link pointer for delay list backwards
	unsigned short	delta_time;              	// Time until time out
	unsigned short	interval_time;           	// Time interval for periodic waits
	unsigned short	events;                  	// Event flags
	unsigned short	waits;                   	// Wait flags
	void          **msg;                   		// Direct message passing when task waits
	void		   *p_mlnk;         			// Link pointer for mutex owner list
	unsigned char   prio_base;               	// Base priority
	unsigned char   stack_frame;             	// Stack frame: 0=Basic, 1=Extended
	unsigned short	priv_stack;              	// Private stack size, 0= system assigned
	unsigned		tsk_stack;               	// Current task Stack pointer (R13)
	unsigned	   *stack;                  	// Pointer to Task Stack memory block
	void		   *ptask;                   	// Task entry address
	}P_TCB_;

static P_SCB_				aSemahoreList[THREAD_SEM_MAX];	// List with all semaphores
static unsigned volatile	uSemahoreNext;		// Next free semaphore


//*****************************************************************************
//*
//*		Compiler defines
//*
//*****************************************************************************
#ifdef	__ICCARM__								// IAR Compiler

	#include <intrinsics.h>

	#define	SVC_Setup(f)   		__asm("mov r12,%0\n" :: "r"(&f): "r12");

	#define SVC_2_1(f,t,t1,t2,...)                                                 	\
			t f (t1 a1, t2 a2);                                                    	\
			_Pragma("swi_number=0") __swi t _##f(t1 a1, t2 a2);                   	\
			static inline t __##f (t1 a1, t2 a2) 									\
				{                                         							\
				SVC_Setup(f);                                                       \
				return _##f(a1,a2);                                                 \
				}

#elif defined(__CC_ARM)							// KEIL Compiler

	#define __STREXB(value, ptr)              __strex(value,ptr)
	#define __LDREXB(ptr)                     ((uint8_t )__ldrex(ptr))

	#define SVC_2_1(f,t,t1,t2,...)													\
		__svc_indirect(0) t  _##f (t(*)(t1,t2),t1,t2); t     f (t1 a1, t2 a2);		\
		__attribute__((always_inline)) static __inline   t __##f (t1 a1, t2 a2)		\
				{        								                           	\
				return _##f(f,a1,a2);                                               \
				}

#elif defined (__GNUC__) 						// GNU Compiler

	__attribute__((always_inline)) static inline  uint32_t __STREXB(uint8_t value, volatile uint8_t *addr)
				{
				uint32_t result;
				__ASM volatile ("strexb %0, %2, %1" : "=&r" (result), "=Q" (*addr) : "r" ((uint32_t)value));
				return(result);
				}

	__attribute__((always_inline)) static inline  uint8_t __LDREXB(volatile uint8_t *addr)
				{
				uint32_t result;
				__ASM volatile ("ldrexb %0, %1" : "=r" (result) : "Q" (*addr));
				return ((uint8_t) result);
				}

	#define SVC_ArgN(n) 		register int __r##n __asm("r"#n);
	#define SVC_ArgR(n,t,a) 	register t   __r##n __asm("r"#n) = a;
	#define SVC_Arg2(t1,t2)                                                        	\
			SVC_ArgR(0,t1,a1)                                                       \
			SVC_ArgR(1,t2,a2)                                                       \
			SVC_ArgN(2)                                                             \
			SVC_ArgN(3)

	#define SVC_Call(f)                                                            	\
		__asm volatile                                                              \
				(                                                                   \
				"ldr r12,="#f"\n\t"                                                 \
				"svc 0"                                                             \
				:               "=r" (__r0), "=r" (__r1), "=r" (__r2), "=r" (__r3)  \
				:                "r" (__r0),  "r" (__r1),  "r" (__r2),  "r" (__r3)  \
				: "r12", "lr", "cc"                                                 \
				);


	#define SVC_2_1(f,t,t1,t2,rv)                                                  	\
		__attribute__((always_inline)) static inline  t __##f (t1 a1, t2 a2)        \
				{                                        							\
				SVC_Arg2(t1,t2);                                                    \
				SVC_Call(f);                                                        \
				return (t) rv;                                                      \
				}


#else

	#error 	Wrong compiler

#endif


//*****************************************************************************
//*
//*		Library call via SVC
//*
//*****************************************************************************

extern int rt_tsk_prio(unsigned uThreadId,int iPror);
SVC_2_1(rt_tsk_prio,int,unsigned,int,int);

//*****************************************************************************
//*
//*		ThreadTickCount
//*
//*****************************************************************************
//	Returns the current time in ms  ... must be defined in RTX_Conf_CM.c ...
//	extern unsigned		os_time;
//
//	unsigned ThreadTickCount(void)
//	{
//	unsigned 	uTick;
//
//		#if OS_TICK == 1000
//
//		uTick	= os_time;
//
//		#elif OS_TICK%1000 == 0
//
//		uTick	= os_time;
//		uTick  *= OS_TICK/1000;
//
//		#else
//
//		#error wrong OS_TICK value
//
//		#endif
//
//	return uTick;
//	}


//*****************************************************************************
//*
//*		ThreadStartEx
//*
//*****************************************************************************
//	Start a new thread
//	pStartAddress	: void Proc(void *pParam)
//	uStackSize		: Size of Stack
//	pParam			: Is send to Proc
//	iPriority		: -256 to +256  ( 0 = normaly )
//	pName			: Is the name of the thread
//	Returns the thread ID or 0 if an error occurs.
void *ThreadStartEx(void( _cdecl  *pStartAddress)(void*),
					unsigned       uStackSize,
					void          *pParam,
					int            iPriority,
					const char    *pName)
{
osThreadId	uTaskId;
int			iPrior;



	if(!uStackSize)return 0;

	#if THREAD_PRIOR_EX

	if((iPriority&0xFFFF0000)==THREAD_GLOBAL)
		{
		iPrior  	= iPriority&0xFFFF;
		}
	else{
		P_TCB_		 *pThread;
		pThread   	= (P_TCB_*)osThreadGetId();
		iPrior  	= pThread->prio_base;
		iPrior	   += iPriority;
		}

	if(iPrior<  1)iPrior=1;
	if(iPrior>255)iPrior=255;

	{
	P_TCB_		   *pThread;
	osThreadDef_t 	sThread;
	static char 	cThreadInit=0;
	sThread.pthread		= (os_pthread)pStartAddress;
	sThread.tpriority	= osPriorityIdle;
	sThread.instances	= 1;
	sThread.stacksize	= uStackSize;


	if(!cThreadInit)
		{
		cThreadInit			= 1;
		pThread 			= (P_TCB_*)osThreadGetId();
		pThread->prio		= THREAD_BASE_PRIOR;
		pThread->prio_base	= THREAD_BASE_PRIOR;
		}

	uTaskId	= osThreadCreate(&sThread,pParam);

	if(!uTaskId)return 0;

	pThread = (P_TCB_*)uTaskId;
	__rt_tsk_prio(pThread->task_id,iPrior);
	}

	#else

	if((iPriority&0xFFFF0000)==THREAD_GLOBAL)
		{
		iPrior  	= iPriority&0xFFFF;
		iPrior	   /= 64;
		}
	else{
		osThreadId	  uMyTaskId;
		uMyTaskId	= osThreadGetId();
		iPrior  	= osThreadGetPriority(uMyTaskId);
		iPrior	   += iPriority/64;
		}

	if(iPrior<-3)iPrior=-3;
	if(iPrior> 3)iPrior= 3;

	{
	osThreadDef_t 	sThread;
	sThread.pthread		= (os_pthread)pStartAddress;
	sThread.tpriority	= (osPriority)iPrior;
	sThread.instances	= 1;
	sThread.stacksize	= uStackSize;

	uTaskId	= osThreadCreate(&sThread,pParam);
	}

	#endif


return (void*)uTaskId;
}


//*****************************************************************************
//*
//*		ThreadSemaphoreInitEx
//*
//*****************************************************************************
//	Create a new semaphore
//	iNum	: Is the init value of the semaphore
//	Returns a hHandle to the semaphore
ThreadSemaphore ThreadSemaphoreInitEx(int iNum)
{
osSemaphoreDef_t 	sSemaphore;
osSemaphoreId  		hSemaphore;
unsigned			uCount;
unsigned			uPos;


	if(iNum>0xF000)iNum=0xF000;

	uPos = uSemahoreNext+1;

	for(uCount=0;;uCount++,uPos++)
		{
		if(uCount>=THREAD_SEM_MAX)
			{
			return 0;
			}

		if(uPos>=THREAD_SEM_MAX)
			{
			uPos = 0;
			}

		if(__LDREXB(  &aSemahoreList[uPos].used)==0)
		if(__STREXB(1,&aSemahoreList[uPos].used)==0)
			{
			break;
			}
		}

	uSemahoreNext = uPos;
	sSemaphore.semaphore = aSemahoreList+uPos;

		hSemaphore = osSemaphoreCreate(&sSemaphore,iNum);
	if(!hSemaphore)
		{
		aSemahoreList[uPos].used = 0;
		return 0;
		}


return (ThreadSemaphore)hSemaphore;
}

//*****************************************************************************
//*
//*		ThreadSemaphoreReleaseEx
//*
//*****************************************************************************
//	Destroys a semaphore
void ThreadSemaphoreExit(ThreadSemaphore hSemHandle)
{
unsigned	uPos;

       uPos = (P_SCB_*)hSemHandle-aSemahoreList;
	if(uPos<THREAD_SEM_MAX)
		{
		osSemaphoreRelease((osSemaphoreId)hSemHandle);
		aSemahoreList[uPos].used = 0;
		}

}

//*****************************************************************************
//*
//*		ThreadSemaphoreReleaseEx
//*
//*****************************************************************************
//	Releases a semapore
void ThreadSemaphoreReleaseEx(ThreadSemaphore hSemHandle,int iCount)
{

	if(!hSemHandle || iCount<=0)return;

	while(iCount>0)
		{
		osSemaphoreRelease((osSemaphoreId)hSemHandle);
		iCount--;
		}

}

//*****************************************************************************
//*
//*		ThreadSemaphoreCount
//*
//*****************************************************************************
//	Gets the value of a semaphore
//	hSemaphore		: handle of the semaphore
//	Returns the semophore value between 0 and 0xFFFF
int ThreadSemaphoreCount(ThreadSemaphore hSemaphore)
{
P_SCB_	*pSem=(P_SCB_*)hSemaphore;

	if(!hSemaphore)return 0;

return	pSem->tokens;
}

//*****************************************************************************
//*
//*
//*		GLOBAL Functions
//*
//*
//*****************************************************************************


//*****************************************************************************
//*
//*		ThreadSecCount
//*
//*****************************************************************************
//	returns a monoton rising second counter
unsigned ThreadSecCount(void)
{
unsigned		uTick;
unsigned		uNext;
unsigned		uUpper;
static unsigned uHigh[8]={0,0,0,0,0,0,0,0};
static unsigned	uLast=0;



	uTick	= ThreadTickCount();
	uUpper	= uTick>>29;

	if(uUpper!=uLast)
		{
		uLast  =  uUpper;
		uNext  = (uUpper-4)&7;

		if(uUpper>=4)
			{
			uHigh[uNext] = uHigh[uUpper]+1;
			}
		else{
			uHigh[uNext] = uHigh[uUpper];
			}
		}


return (uHigh[uUpper]<<22)|(uTick/1024);
}


