//*****************************************************************************
//*
//*
//*		TimeUtility.c
//*
//*
//*****************************************************************************
//
//  This is a library with time support for WINDOWS,LINUX and RTTARGET
//
//	Global Macros:
//	
//		GLOBAL_CONFIG		Enable the configuration via GlobalConfig.h header file (1=on 0=off)
//		SYS_TIME_HAS_FT		use the windows FILETIME structure as SysTime (1=on 0=off)
//		SYS_TIME_HAS_NS		add a nanosecont support to the library (0=seconds 1=ns)
//		SYS_BASE_YEAR		is the base year for SysTime
//
//	(C) Copyright Anton Zechner 2018
//
//
#ifdef		GLOBAL_CONFIG
#define 	TIMEUTILITY_C
#include	<GlobalConfig.h>
#undef		TIMEUTILITY_C
#endif

#include	"TimeUtility.h"
#include	<string.h>

#ifdef		_MSC_VER
#define		tuint64				__int64
#define		BASE_OFFSET			0x014F373BFDE04000L
#define		TIME_OFFSET			0x019DB1DED53E8000L
#else
#define		tuint64				long long
#define		BASE_OFFSET			0x014F373BFDE04000LL
#define		TIME_OFFSET			0x019DB1DED53E8000LL
#endif




#define 	YEAR_DAYS			(365)					// days in   1 year
#define		YEAR_DAYS_4			(YEAR_DAYS*4+1)			// days in   4 years
#define		YEAR_DAYS_100		(YEAR_DAYS_4*25-1)		// days in 100 years
#define		YEAR_DAYS_400		(YEAR_DAYS_100*4+1)		// days in 400 years
#define 	IS_LEAP_YEAR(y)		(((y)%4)? 0:(((y)%100)? 1:(((y)%400)? 0:1)))
#define 	DIF_OFFSET(y)		((y)*365+(y)/4-(y)/100+(y)/400)
#define 	DAY_OFFSET(y)		(DIF_OFFSET(y)+1-IS_LEAP_YEAR(y))
#define 	FILETIME_OFFSET		(BASE_OFFSET+(tuint64)(DAY_OFFSET(SYS_BASE_YEAR)-DAY_OFFSET(1900))*(tuint64)24L*(tuint64)3600L*(tuint64)10000000)


static const unsigned	aMonDays [] = {  0,  0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365, 365, 365 };
static const unsigned	aMonEnd  [] = { 31, 31, 28, 31, 30,  31,  30,  31,  31,  30,  31,  30,  31 , 31,  31,  31 };
	   int				iInitTimeZone	= 0;			// is time zone initalized
	   int 				iTimeZone		= 0;			// current time zone (difference in secounds)
	   int 				iHasDst			= 1;			// hase daylight saving

	   #if   SYS_TIME_HAS_FT
	   SysTime			iTimeZoneFt		= 0;			// current time zone (difference in 100 nanosecs)
	   #elif SYS_TIME_HAS_NS
	   SysTime			iTimeZoneNs		= 0;			// current time zone (difference in nano secounds)
	   #endif

static void				SysConvertDaylightSaving(SysTimeStruct *pData,int iOn);
extern void				SysInitTimeZone(void);



//*****************************************************************************
//*
//*		SysTimeGlobal
//*
//*****************************************************************************
//	Gets the current time and store the global time in a time date structure
//	pData	: pointer to the time structure
//	Returns the current global time in seconds from 1.Jan.2000 and
SysTime SysTimeGlobal(SysTimeStruct *pData)
{
SysTime	iTime;


	iTime  = SysTimeGet();

	SysTimeGlobalEx(pData,iTime);


return iTime;
}

//*****************************************************************************
//*
//*		SysTimeGlobalEx
//*
//*****************************************************************************
//	Converts a global time to a time structure
//	uTime		: global time in nano seconds from 1.Jan.SYS_BASE_YEAR and
//	pData		: pointer to the time structure
void SysTimeGlobalEx(SysTimeStruct *pData,SysTime iTime)
{
static const unsigned 	aDaysOfYear[] = {0,31,61,92,122,153,184,214,245,275,306,337,366};
SysTime					iSecTime;
unsigned				uMinutes;
unsigned				uHours;
unsigned				uMonth;
unsigned				uDays;
unsigned				uYear;
unsigned				uTemp;


	#if SYS_TIME_HAS_NS!=0 ||  SYS_BASE_YEAR<1970

	if(iTime<0)											// Is the timevalue correct 0
		{
		pData->usYear	 = SYS_BASE_YEAR;
		pData->ucMonth	 = 1;
		pData->ucDay	 = 1;
		pData->ucHour	 = 0;
		pData->ucMinute	 = 0;
		pData->ucSecond	 = 0;
		pData->ucIsDst	 = 0;

		#if SYS_TIME_HAS_NS
		pData->uiNanoSec = 0;
		#endif

		return;
		}

	#endif

	#if   SYS_TIME_HAS_FT
	pData->uiNanoSec = (unsigned int)((iTime%10000000L)*100);
	iSecTime		 =                 iTime/10000000L;
	#elif SYS_TIME_HAS_NS
	pData->uiNanoSec = (unsigned int )(iTime%1000000000L);
	iSecTime		 =                 iTime/1000000000L;
	#else
	iSecTime		 =                 iTime;
	#endif

	pData->ucSecond	 = (unsigned char)(iSecTime%60);
	uMinutes		 = (unsigned int )(iSecTime/60);
	pData->ucMinute	 = (unsigned char)(uMinutes%60);
	uHours           =                 uMinutes/60;
	pData->ucHour	 = (unsigned char)(uHours%24);
	uDays			 =                 uHours/24;

														// Differenz to 1.March.0000
	uDays      += DIF_OFFSET(SYS_BASE_YEAR)-31-28-IS_LEAP_YEAR(SYS_BASE_YEAR);

	uTemp		= uDays/YEAR_DAYS_400;
	uDays      -= uTemp*YEAR_DAYS_400;
	uYear		= uTemp*400;

	   uTemp    = uDays/YEAR_DAYS_100;
	if(uTemp==4)
		{
	  	uDays  -= 3*YEAR_DAYS_100;
		uYear  += 3*100;
		}
	else{
		uDays   -= uTemp*YEAR_DAYS_100;
		uYear	+= uTemp*100;
		}

	uTemp		= uDays/YEAR_DAYS_4;
	uDays      -= uTemp*YEAR_DAYS_4;
	uYear	   += uTemp*4;

	   uTemp    = uDays/YEAR_DAYS;
	if(uTemp==4)
		{
		uDays  -= 3*YEAR_DAYS;
		uYear  += 3;
		}
	else{
		uDays  -= uTemp*YEAR_DAYS;
		uYear  += uTemp;
		}

	uMonth	 = (uDays*3)/92;

	if(uDays>=aDaysOfYear[uMonth+1])
		{
		uMonth++;
		}

	uDays  -= aDaysOfYear[uMonth]-1;
	uMonth += 3;

	if(uMonth>12)
		{
		uMonth -= 12;
		uYear  +=  1;
		}

	pData->usYear	= (unsigned short)uYear;
	pData->ucMonth	= (unsigned char )uMonth;
	pData->ucDay	= (unsigned char )uDays;
	pData->ucIsDst	= 0;

}

//*****************************************************************************
//*
//*		SysTimeLocal
//*
//*****************************************************************************
//	Gets the current time and store the local time in a time date structure
//	pData		: pointer to the time structure
//	iNoDst		: don't set daylight saving
//	Returns the current global time in seconds from 1.Jan.2000 and
SysTime SysTimeLocal(SysTimeStruct *pData,int iNoDst)
{
SysTime	iTime;



	if(!iInitTimeZone)
		{
		SysInitTimeZone();
		}

	iTime  = SysTimeGet();

	#if   SYS_TIME_HAS_FT
	SysTimeGlobalEx(pData,iTime-(SysTime)iTimeZoneFt);
	#elif SYS_TIME_HAS_NS
	SysTimeGlobalEx(pData,iTime-(SysTime)iTimeZoneNs);
	#else
	SysTimeGlobalEx(pData,iTime-(SysTime)iTimeZone);
	#endif

	if(!iNoDst && SysIsDaylightSaving(pData))
		{
		SysConvertDaylightSaving(pData,1);
		}


return iTime;
}

//*****************************************************************************
//*
//*		SysTimeLocalEx
//*
//*****************************************************************************
//	Converts a SysTime value in a local time structure
//	uTime		: global time in seconds from 1.Jan.2000
//	pData		: pointer to the time structure for the local time
//	uTime		: global time value that will convertet
//	iNoDst		: don't set daylight saving
void SysTimeLocalEx(SysTimeStruct *pData,SysTime iTime,int iNoDst)
{

	if(!iInitTimeZone)
		{
		SysInitTimeZone();
		}

	#if   SYS_TIME_HAS_FT
	iTime -= (SysTime)iTimeZoneFt;
	#elif SYS_TIME_HAS_NS
	iTime -= (SysTime)iTimeZoneNs;
	#else
	iTime -= (SysTime)iTimeZone;
	#endif

	SysTimeGlobalEx(pData,iTime);

	if(!iNoDst && SysIsDaylightSaving(pData))
		{
		SysConvertDaylightSaving(pData,1);
		}

}

//*****************************************************************************
//*
//*		SysTimeFromLocal
//*
//*****************************************************************************
//	Converts the local time from a time structure in a SysTime value
//	pData		: pointer to the time structure
//				  if pData->ucIsDst is on SYS_TIME_DST_AUTO, the daylight saving
//				  will be subtracted automaically.
//	Returns a time in nanoseconds from 1.Jan.SYS_BASE_YEAR
SysTime	SysTimeFromLocal(const SysTimeStruct *pData)
{
SysTime		iRet;


	if(!iInitTimeZone)
		{
		SysInitTimeZone();
		}

	iRet = SysTimeFromGlobal(pData);

	#if   SYS_TIME_HAS_FT
	iRet += (SysTime)iTimeZone*(SysTime)10000000;
	#elif SYS_TIME_HAS_NS
	iRet += (SysTime)iTimeZone*(SysTime)1000000000;
	#else
	iRet += (SysTime)iTimeZone;
	#endif


return iRet;
}

//*****************************************************************************
//*
//*		SysTimeFromGlobal
//*
//*****************************************************************************
//	Converts the global time from a time structure in a SysTime value
//	pData		: pointer to the time structure
//	Returns a time in nanoseconds from 1.Jan.SYS_BASE_YEAR
SysTime	SysTimeFromGlobal(const SysTimeStruct *pData)
{
unsigned	uTime;
unsigned	uDate;
unsigned	uLeap;
unsigned 	uYear;
unsigned	uVal;
SysTime		iRet;




	uYear   =  pData->usYear;
	uLeap	=  IS_LEAP_YEAR(uYear);

	uTime   =  pData->ucSecond;							// calculate time offset
	uTime  +=  pData->ucMinute*60;
	uTime  +=  pData->ucHour*3600;

	uDate   =  pData->ucDay-1;
	uDate  +=  aMonDays[pData->ucMonth&0x0F];
	uDate  +=  DIF_OFFSET(uYear)+1-uLeap;
	uDate  -=  DAY_OFFSET(SYS_BASE_YEAR);

	if(uLeap && pData->ucMonth>=3 && uLeap)				// is it a leap year and > 1.March
		{
		uDate++;
		}

	if(pData->ucIsDst)									// is daylight saving active
		{
		if(pData->ucIsDst==SYS_TIME_DST_AUTO)
			{
			uVal   = SysIsDaylightSaving(pData);
			uTime -= uVal*60;
			}
		else{
			uTime -= pData->ucIsDst*60;
			}
		}

	#if   SYS_TIME_HAS_FT
	iRet  = (SysTime)pData->uiNanoSec/100;
	iRet += (SysTime)uTime*(SysTime)10000000;
	iRet += (SysTime)uDate*(SysTime)10000000*(SysTime)(24*3600);
	#elif SYS_TIME_HAS_NS
	iRet  = (SysTime)pData->uiNanoSec;
	iRet += (SysTime)uTime*(SysTime)1000000000;
	iRet += (SysTime)uDate*(SysTime)1000000000*(SysTime)(24*3600);
	#else
	iRet  = (SysTime)uTime;
	iRet += (SysTime)uDate*(SysTime)(24*3600);
	#endif


return iRet;
}


//*****************************************************************************
//*
//*		SysWeekday
//*
//*****************************************************************************
//	Returns the Weekday of a date.
//	uDay		: day of month (1-31)
//	uMonth		: mounth of the year (1=Jan ... )
//	uYear		: Is the year (i.e. 2000)
//	Returns 1=Mo 2=Di 3=Mi 4=Do 5=Fr 6=Sa 7=So
unsigned SysWeekday(unsigned uDay,unsigned uMonth,unsigned uYear)
{
static const unsigned char	ucYearDays[16] = {0%7,306%7,337%7,0%7,31%7,61%7,92%7,122%7,153%7,184%7,214%7,245%7,275%7,365%7,365%7,365%7};
unsigned					uVal;


	uMonth   = uMonth&0x0F;
	uDay    &= 0x1F;

	if(uMonth<3)uYear--;

	uVal     =  1;										// 1.March.0000 = Mo
	uVal	+= (uYear% 4)*YEAR_DAYS;	uYear/=4;
	uVal	+= (uYear%25)*YEAR_DAYS_4;	uYear/=25;
	uVal	+= (uYear% 4)*YEAR_DAYS_100;uYear/=4;
	uVal	+= (uYear   )*YEAR_DAYS_400;
	uVal    += ucYearDays[uMonth];
	uVal	+= uDay;
	uVal	%= 7;


return	uVal+1;
}

//*****************************************************************************
//*
//*		SysWeekdayEx
//*
//*****************************************************************************
//	Returns the Weekday of a time value.
//	iTime		: Time witch is converted
//	Returns 1=Mo 2=Di 3=Mi 4=Do 5=Fr 6=Sa 7=So
unsigned SysWeekdayEx(SysTime iTime)
{
SysTimeStruct	sData;


	SysTimeLocalEx(&sData,iTime,0);


return SysWeekday(sData.ucDay,sData.ucMonth,sData.usYear);
}

//*****************************************************************************
//*
//*		SysMounthDays
//*
//*****************************************************************************
//	Calculates the the day count of a month
//	uMonth		: mounth of the year (1=Jan ... )
//	uYear		: year, i.e. 2000 to 2100
//	Returns 1..31 if ok or 0 if fails
unsigned SysMounthDays(unsigned uMonth,unsigned uYear)
{

	uMonth--;

	if(uMonth>=12)return 0;

	if(uMonth!=12)return aMonEnd[uMonth+1];


return (IS_LEAP_YEAR(uYear))? 29:28;
}


//*****************************************************************************
//*
//*		SysIsDaylightSaving
//*
//*****************************************************************************
//	Calculates if a time/date have a daylight saving
//	pData		: Date for the calulation
//	Returns the minute difference if the daylight saving is active.
unsigned SysIsDaylightSaving(const SysTimeStruct *pData)
{
unsigned	uVal;


	if(!iHasDst)return 0;

	if(pData->ucMonth<=3)								// check begin
        {
        if(pData->ucMonth<3 || pData->ucDay<25)
			{
			return  0;
			}

        uVal = SysWeekday(25,3,pData->usYear);
        uVal = (7-uVal)+25;

        if(pData->ucDay>uVal)return 60;
        if(pData->ucDay<uVal)return  0;

		uVal = (pData->ucIsDst)? 1:2;

        if(pData->ucHour<uVal)return 0;

		return 60;
        }

	if(pData->ucMonth<=10)								// check end
        {
        if(pData->ucMonth<10 || pData->ucDay<25)
			{
			return 60;
			}

        uVal=SysWeekday(25,10,pData->usYear);
        uVal=(7-uVal)+25;

        if(pData->ucDay>uVal)return  0;
        if(pData->ucDay<uVal)return 60;

		uVal = (pData->ucIsDst)? 1:2;

        if(pData->ucHour<uVal)return 60;

		return 0;
        }


return 0;
}


//*****************************************************************************
//*
//*		SysConvertDaylightSaving
//*
//*****************************************************************************
//	Converts a time structure to a structure with or without daylight saving.
//	pData		: pointer to time structure
//	iOn			: 0=convert to no daylight saving,  1=convert to daylight saving on
//				  SYS_TIME_DST_AUTO = automaticaly convert
static void SysConvertDaylightSaving(SysTimeStruct *pData,int iOn)
{


	if(pData->ucIsDst==SYS_TIME_DST_AUTO)				// is auto daylight saving active
		{
		pData->ucIsDst = SysIsDaylightSaving(pData);
		}

	if(iOn==SYS_TIME_DST_AUTO)							// automaticaly convert
		{
		iOn	= SysIsDaylightSaving(pData);
		}

	iOn = (iOn)? 60:0;

	if(iOn!=pData->ucIsDst)								// convert
		{
		SysTimeShift(pData,(iOn-pData->ucIsDst)*60);
		pData->ucIsDst	= iOn;
		return;
		}


}

//*****************************************************************************
//*
//*		SysTimeToUtc
//*
//*****************************************************************************
//	Converts a SysTime value to UTC time_t value
//	iTime		: Time witch is converted to UTC
//	Returns a UTC time value (seconds from 1.Jan.1970)
unsigned SysTimeToUtc(SysTime iTime)
{

	#if   SYS_TIME_HAS_FT
	iTime /= 10000000L;
	#elif SYS_TIME_HAS_NS
	iTime /= 1000000000L;
	#endif

	iTime += ((SysTime)DAY_OFFSET(SYS_BASE_YEAR)-(SysTime)DAY_OFFSET(1970))*(SysTime)(24*3600);


return (unsigned)iTime;
}

//*****************************************************************************
//*
//*		SysTimeFromUtc
//*
//*****************************************************************************
//	Converts a UTC value to a SysTime value
//	uTime	: UTC Time witch shold be converted (seconds from 1.Jan.1970)
//	Returns a SysTime time value
SysTime SysTimeFromUtc(unsigned uTime)
{
SysTime		iRet;


	iRet  =  (SysTime)uTime;
	iRet -= ((SysTime)DAY_OFFSET(SYS_BASE_YEAR)-(SysTime)DAY_OFFSET(1970))*(SysTime)(24*3600);

	#if   SYS_TIME_HAS_FT
	iRet *= 10000000L;
	#elif SYS_TIME_HAS_NS
	iRet *= 1000000000L;
	#endif



return iRet;
}

//******************************************************************************
//*
//*		SysIsLeapYear
//*
//******************************************************************************
//	Chech if a year is a leap year
//	uYear		: Is the year value (i.e. 2007)
//	Returns 1 if the year is a leapyear
unsigned SysIsLeapYear(unsigned uYear)
{
unsigned 	uCalc;


	if(uYear&3)return 0;

	uCalc = (uYear/4)%100;

	if(uCalc%25 || uCalc==0)
		{
		return 1;
		}


return 0;
}

//*****************************************************************************
//*
//*		SysTimeShift
//*
//*****************************************************************************
//	Converts a time structure to a structure with a time shift
//	pData		: pointer to time structure
//	iShift		: time shift in seconds
void SysTimeShift(SysTimeStruct *pData,int iShift)
{
int		iDays;


	if(iShift==0)return;

	if(iShift>=0)								// shift in future
		{
		   iShift +=   pData->ucSecond;
		if(iShift< 60){pData->ucSecond=iShift;return;}
		   pData->ucSecond=iShift%60;
		   iShift/=60;

		   iShift +=   pData->ucMinute;
		if(iShift< 60){pData->ucMinute=iShift;return;}
		   pData->ucMinute=iShift%60;
		   iShift/=60;

		   iShift +=   pData->ucHour;
		if(iShift< 24){pData->ucHour=iShift;return;}
		   pData->ucHour=iShift%24;
		   iShift/=24;
		   iShift +=   pData->ucDay;

		if(iShift>=366)							// difference more than one year
			{
			if(pData->ucMonth<=2)
				{
				pData->usYear--;
				}

			do	{
				pData->usYear += 1;

				if(SysIsLeapYear(pData->usYear))
						iShift -= 366;
				else	iShift -= 365;
				}while(iShift>=366);

			if(pData->ucMonth<=2)
				{
				pData->usYear++;
				}
			}

		for(;;)									// shift days in one year
			{
			if(iShift<=28){pData->ucDay=iShift;return;}

			if(pData->ucMonth==2)				// day in this month
					iDays = SysIsLeapYear(pData->usYear)? 29:28;
			else	iDays = aMonEnd[pData->ucMonth];

			if(iShift<=iDays){pData->ucDay=iShift;return;}

			iShift -= iDays;
			pData->ucMonth++;

			if(pData->ucMonth<=12)
				{
				continue;
				}

			pData->ucMonth = 1;
			pData->usYear += 1;
			}
		}
	else{										// shift to past
		   iShift+=    pData->ucSecond;
		if(iShift>= 0){pData->ucSecond=iShift;return;}
		               pData->ucSecond=60+iShift%60;
		   iShift/= 60;

		if(pData->ucSecond==60)pData->ucSecond=0;
		else   iShift-= 1;

		   iShift+=    pData->ucMinute;
		if(iShift>= 0){pData->ucMinute=iShift;return;}
		               pData->ucMinute=60+iShift%60;
		   iShift/= 60;

		if(pData->ucMinute==60)pData->ucMinute=0;
		else   iShift-= 1;

		   iShift+=    pData->ucHour;
		if(iShift>= 0){pData->ucHour=iShift;return;}
		               pData->ucHour=24+iShift%24;
		   iShift/=24;

		if(pData->ucHour==24)pData->ucHour=0;
		else   iShift-= 1;

		   iShift += pData->ucDay;

		if(iShift<=-366)						// difference more than one year
			{
			if(pData->ucMonth<=2)
				{
				pData->usYear--;
				}

			do	{
				if(SysIsLeapYear(pData->usYear))
						iShift += 366;
				else	iShift += 365;

				pData->usYear -= 1;

				}while(iShift<=-366);

			if(pData->ucMonth<=2)
				{
				pData->usYear++;
				}
			}

		for(;;)									// shift days in one year
			{
			if(iShift>=1)
				{
				pData->ucDay = iShift;
				return;
				}

			pData->ucMonth--;

			if(pData->ucMonth<=0)
				{
				pData->ucMonth = 12;
				pData->usYear -= 1;
				}

			if(pData->ucMonth==2)
					iDays = SysIsLeapYear(pData->usYear)? 29:28;
			else	iDays = aMonEnd[pData->ucMonth];

			iShift += iDays;
			}
		}
}


//*****************************************************************************
//*
//*		SysTimeConvert
//*
//*****************************************************************************
//	Converts time structures between local and global time
//	pData		: pointer to the target time
//	pFrom		: pointer to the input time
//	iMode		: defines the conversion mode
//					SYS_IS_GLOBAL			input is a global time
//					SYS_IS_LOCAL			input is a local  time
//					SYS_TO_GLOBAL			target should be a global time
//					SYS_TO_LOCAL			target should be a local  time
//					SYS_NO_DST				set no daylight saving
//					SYS_LOCAL_TO_GLOBAL		from local to global
//					SYS_GLOBAL_TO_LOCAL		from global to local
void SysTimeConvert(SysTimeStruct *pData,const SysTimeStruct *pFrom,int iMode)
{


	memcpy(pData,pFrom,sizeof(SysTimeStruct));


	if(!iInitTimeZone)
		{
		SysInitTimeZone();
		}


	if((iMode&3)==SYS_LOCAL_TO_GLOBAL)						// convert time zone
		{
		SysTimeShift(pData,iTimeZone);
		}
	else if((iMode&3)==SYS_GLOBAL_TO_LOCAL)
		{
		SysTimeShift(pData,-iTimeZone);
		}


	if((iMode&SYS_TO_LOCAL) && !(iMode&SYS_NO_DST))			// set daylight saving for local time
		{
		SysConvertDaylightSaving(pData,SYS_TIME_DST_AUTO);
		}
	else{
		SysConvertDaylightSaving(pData,0);
		}

}


//******************************************************************************
//*
//*		SysCalendarWeek
//*
//******************************************************************************
//	Returns the calender week of a date
//	uDay		: day of month (1-31)
//	uMonth		: mounth of the year (1=Jan ... )
//	uYear		: year, values from 2000 to 2100 are allowed
//	uMode		: 0 = first week is the week which has minimal 4 days in the new year.
//						 (Mo=new week,DIN 1355,ISO 8601)
//				  1 = first week is the week with the 1. Jan (So=new week)
//				  2 = first week is the week which has minimal 7 days in the new year.
//				  bit 8-10 is the start day of a week (0=Mo 1=Di ... 6=So)
//				  standard values
//					middel europe	= 0x0000
//					north amerika	= 0x0601
//	Returns the calender 1 .. 52
unsigned SysCalendarWeek(unsigned uDay,unsigned uMonth,unsigned uYear,unsigned uMode)
{
int 				iWeekDay;
int 				iYearDay;
int 				iLeapYear;
unsigned 			uCalc;



	iYearDay   = (uDay&0x1F)-1+aMonDays[uMonth&0x0F];
	iWeekDay   =  SysWeekday(1,1,uYear);				// weekday of 1.Jan	 (Mo=1 So=7)
	iLeapYear  =  0;

	if(!(uYear&3) && uMonth>=3)							// is it a leap year and > 1.March
		{
		   uCalc = (uYear/4)%100;
		if(uCalc%25 || uCalc==0)
			{
			iYearDay++;
			iLeapYear=1;
			}
		}



	   iWeekDay-= (uMode>>8)&0x07;						// which weekday is the first of week
	if(iWeekDay<=0)iWeekDay+=7;



	switch(uMode&0xFF)
		{
	case 2:												// first week is the week which has minimal 7 days in the new year. (Mo=new Week)

				iYearDay += (iWeekDay>1)? iWeekDay-1:iWeekDay+6;
				iYearDay /=	7;

				if(iYearDay==0)
					{
					if(iWeekDay<6)iYearDay=52;
					else{
						if(iWeekDay==7)iYearDay=53;
						else{
							   uCalc = ((uYear-1)/4)%100;
							if(uCalc%25 || uCalc==0)
									iYearDay=53;
							else	iYearDay=52;
							}
						}
					}

				break;

	case 1:												// first week is the week with the 1. Jan (So=new week)

				iYearDay += iWeekDay+6;
				iYearDay /=	7;

				break;

	default:											// first week is the week which has minimal 4 days in the new year. (Mo=new week,DIN 1355,ISO 8601)

				iYearDay += (iWeekDay>4)? iWeekDay-1:iWeekDay+6;
				iYearDay /=	7;

				if(iYearDay>=53)
					{
					if(!iLeapYear)
						{
						if(iWeekDay!=4)iYearDay=1;
						}
					else{
						if(iWeekDay<3 || iWeekDay>4)iYearDay=1;
						}
					}
				else if(iYearDay==0)
					{
					if(iWeekDay<5 || iWeekDay>6)
						{
						iYearDay=52;
						}
					else{
						if(iWeekDay==5)
							{
							iYearDay=53;
							}
						else{
							   uCalc = ((uYear-1)/4)%100;
							if(uCalc%25 || uCalc==0)
									iYearDay=53;
							else	iYearDay=52;
							}
						}
					}

				break;
		}


return iYearDay;
}


//******************************************************************************
//*
//*		SysWeekBegin
//*
//******************************************************************************
//	Returns the begin of a calender week in a year
//	uYear		: year, values from 2000 to 2100 are allowed
//	uWeek		: is the week 1..54
//	uMode		: 0 = first week is the week which has minimal 4 days in the new year.
//						 (Mo=new week,DIN 1355,ISO 8601)
//				  1 = first week is the week with the 1. Jan (So=new week)
//				  2 = first week is the week which has minimal 7 days in the new year.
//				  bit 8-10 is the start day of a week (0=Mo 1=Di ... 6=So)
//				  standard values
//					middel europe	= 0x0000
//					north amerika	= 0x0601
//	pData		: Here the begin date will be stored
void SysWeekBegin(unsigned uYear,unsigned uWeek,unsigned uMode,SysTimeStruct *pData)
{
int 				iDiffDay;
int 				iWeekDay;
int 				iYearDay;



	iWeekDay = SysWeekday(1,1,uYear)-1;					// weekday of 1.Jan	 (Mo=0 So=6)
	iDiffDay = iWeekDay-((uMode>>8)&0x07);

	if(iDiffDay>0)iDiffDay-=7;



	switch(uMode&0xFF)
		{
	case 2:												// first week is the week which has minimal 7 days in the new year. (Mo=new Week)
				iYearDay = (iDiffDay>=0)? iDiffDay:iDiffDay+7;
				break;
	case 1:												// first week is the week with the 1. Jan (So=new week)
				iYearDay =  iDiffDay;
				break;
	default:											// first week is the week which has minimal 4 days in the new year. (Mo=new week,DIN 1355,ISO 8601)
				iYearDay = (iDiffDay>=-3)? iDiffDay:iDiffDay+7;
				break;
		}


	pData->usYear	 = uYear;
	pData->ucMonth	 = 1;
	pData->ucDay	 = 1;
	pData->ucHour	 = 0;
	pData->ucMinute	 = 0;
	pData->ucSecond	 = 0;
	pData->ucIsDst	 = 0;
	#if SYS_TIME_HAS_NS
	pData->uiNanoSec = 0;
	#endif

	iYearDay	    += uWeek*7;

	SysTimeShift(pData,iYearDay*(3600*24));


}

//*****************************************************************************
//*
//*		SysTimeInfo
//*
//*****************************************************************************
//	Gets the current time zone and daylight saving
//	pDaylightSaving		: in not zero her the daylight saving will be stored
//	Returns the time zone (seconds to UTC time)
int SysTimeInfo(int *pDaylightSaving)
{

	if(!iInitTimeZone)
		{
		SysInitTimeZone();
		}

	if(pDaylightSaving)*pDaylightSaving=iHasDst;


return iTimeZone;
}


//******************************************************************************
//*
//*		SysFtToTime
//*
//******************************************************************************
//	Converts a SysTime value in a FILETIME time
//	iTime		: Is the SysTime value
//	Returns the FILETIME (100ns since 1601)
#if SYS_TIME_HAS_FT==0
SysTick SysTimeToFt(SysTime iTime)
{
SysTick		iTick;


	iTick	= (SysTick)iTime;

	#if SYS_TIME_HAS_NS
	iTick  /= (tuint64)100;
	iTick  += FILETIME_OFFSET;
	#else
	iTick  *= (tuint64)10000000;
	iTick  += FILETIME_OFFSET;
	#endif


return iTick;
}
#endif

//******************************************************************************
//*
//*		SysFtToTime
//*
//******************************************************************************
//	Converts a FILETIME value in a SysTime time
//	iTime		: Is the FILETIME value. (100ns since 1601)
//	Returns the SysTime
#if SYS_TIME_HAS_FT==0
SysTime SysFtToTime(SysTick iTime)
{

	#if SYS_TIME_HAS_NS
	iTime  -= FILETIME_OFFSET;
	iTime  *= (tuint64)100;
	#else
	iTime  -= FILETIME_OFFSET;
	iTime  /= (tuint64)10000000;
	#endif

	if(iTime>>32)return 0;


return (SysTime)iTime;
}
#endif


//*****************************************************************************
//*
//*		SysUtcToFt
//*
//*****************************************************************************
//	Converts a UTC time value in a FILETIME
//	uUtcTime	: Is the UTC time value
//	Returns the FILETIME (100ns since 1601)
SysTick SysUtcToFt(unsigned uUtcTime)
{
SysTick		uFileTime;


	uFileTime  = uUtcTime;
	uFileTime *= 10000000;
	uFileTime += TIME_OFFSET;


return uFileTime;
}


//*****************************************************************************
//*
//*		SysFtToUtc
//*
//*****************************************************************************
//	Converts a FILETIME value in a UTC time
//	iFileTime	: Is the FILETIME value
//	Returns the UTC time (seconds since 1.Jam.1970)
unsigned SysFtToUtc(SysTick iFileTime)
{

	iFileTime -= TIME_OFFSET;

	if(iFileTime<0)							// before 1.Jan.1970
		{
		return 0;
		}

	iFileTime /= 10000000;					// 100ns to seconds

	if(iFileTime>0xFFFFFFFF)
		{
		return 0;
		}


return (unsigned)iFileTime;
}
