//*****************************************************************************
//*
//*
//*		TimeUtility.h
//*
//*
//*****************************************************************************
//
//  This is a library with time support for WINDOWS,LINUX and RTTARGET
//
//	Global Macros:
//	
//		GLOBAL_CONFIG		Enable the configuration via GlobalConfig.h header file (1=on 0=off)
//		SYS_TIME_HAS_FT		use the windows FILETIME structure as SysTime (1=on 0=off)
//		SYS_TIME_HAS_NS		add a nanosecont support to the library (0=seconds 1=ns)
//		SYS_BASE_YEAR		is the base year for SysTime
//
//	(C) Copyright Anton Zechner 2018
//
//
#ifndef	  __TIMEUTILITY_H__
#define   __TIMEUTILITY_H__

#ifdef		GLOBAL_CONFIG
#define 	TIMEUTILITY_H
#include	<GlobalConfig.h>
#undef		TIMEUTILITY_H
#endif


// constands for SysCalendarWeek

#define		SYS_CW_MODE_ISO		0x0000								// first week is the week which has minimal 4 days in the new year
#define		SYS_CW_MODE_1JAN	0x0001								// first week is the week with the 1. Jan (So=new week)
#define		SYS_CW_MODE_FULL	0x0002								// first week is the week which has minimal 7 days in the new year.
#define		SYS_CW_FIRST_MO		0x0000								// first week day is monday
#define		SYS_CW_FIRST_TU		0x0100								// first week day is tuesday
#define		SYS_CW_FIRST_WE		0x0200								// first week day is wendsday
#define		SYS_CW_FIRST_TH		0x0300								// first week day is thuesday
#define		SYS_CW_FIRST_FR		0x0400								// first week day is friday
#define		SYS_CW_FIRST_SA		0x0500								// first week day is saterday
#define		SYS_CW_FIRST_SU		0x0600								// first week day is sunday
#define		SYS_CW_EUROPE		(SYS_CW_MODE_ISO |SYS_CW_FIRST_MO)	// ISO mode for europe
#define		SYS_CW_US			(SYS_CW_MODE_1JAN|SYS_CW_FIRST_SO)	// mode for north amerika

// constands for SysWeekday

#define		SYS_WD_DAY_MO		0x0001								// week day is monday
#define		SYS_WD_DAY_TU		0x0002								// week day is tuesday
#define		SYS_WD_DAY_WE		0x0003								// week day is wendsday
#define		SYS_WD_DAY_TH		0x0004								// week day is thuesday
#define		SYS_WD_DAY_FR		0x0005								// week day is friday
#define		SYS_WD_DAY_SA		0x0006								// week day is saterday
#define		SYS_WD_DAY_SU		0x0007								// week day is sunday

// constands for SysTimeConvert

#define 	SYS_IS_GLOBAL		0x0000								// input is a global time
#define 	SYS_IS_LOCAL		0x0001								// input is a local  time
#define 	SYS_TO_GLOBAL		0x0000								// target should be a global time
#define 	SYS_TO_LOCAL		0x0002								// target should be a local  time
#define 	SYS_NO_DST			0x0004								// set no daylight saving
#define 	SYS_LOCAL_TO_GLOBAL	(SYS_IS_LOCAL |SYS_TO_GLOBAL)		// from local to global
#define 	SYS_GLOBAL_TO_LOCAL	(SYS_IS_GLOBAL|SYS_TO_LOCAL )		// from global to local

// constands for SysTime

#define		SYS_TIME_DST_AUTO	255									// automatically detect daylight saving

#ifndef		SYS_TIME_HAS_FT
#define		SYS_TIME_HAS_FT		0									// supports the windows FILETIME structure
#endif

#ifndef		SYS_TIME_HAS_NS
#define		SYS_TIME_HAS_NS		0									// supports the timestructure nanoseconds
#endif

#ifndef		SYS_BASE_YEAR
#if			SYS_TIME_HAS_NS
#define 	SYS_BASE_YEAR		1900								// the year where SysTime begins
#else
#define 	SYS_BASE_YEAR		2000								// the year where SysTime begins
#endif
#endif

#if			SYS_TIME_HAS_NS
#define 	SYS_LAST_YEAR		(SYS_BASE_YEAR+292)					// the last presentable year
#else
#define 	SYS_LAST_YEAR		(SYS_BASE_YEAR+135)					// the last presentable year
#endif


#if			SYS_TIME_HAS_FT
#undef		SYS_TIME_HAS_NS
#undef		SYS_BASE_YEAR
#undef		SYS_LAST_YEAR
#define 	SYS_TIME_HAS_NS		1
#define 	SYS_BASE_YEAR		1601
#define 	SYS_LAST_YEAR		9999
#endif


#ifdef		linux
#ifndef		LINUX
#define		LINUX				1
#endif
#endif

#ifdef	  __cplusplus
extern		"C" {
#define 	D(a)				a
#else
#define		D(a)
#endif


	#if SYS_TIME_HAS_NS==0 && SYS_BASE_YEAR>=1970

		typedef		unsigned  	SysTime;

	#elif defined(_MSC_VER)

		typedef		__int64		SysTime;

	#else

		typedef		long long 	SysTime;

	#endif

	#ifdef _MSC_VER

		typedef		__int64		SysTick;

	#else

		typedef		long long 	SysTick;

	#endif


	typedef	struct
		{
		unsigned short			usYear;								// Year (ie. 2006)
		unsigned char			ucMonth;							// Mounth (1..12)
		unsigned char			ucDay;								// Day (1..31)
		unsigned char			ucHour;								// Hour (0..24)
		unsigned char			ucMinute;							// Minute (0..59)
		unsigned char			ucSecond;							// Secound (0..59)
		unsigned char			ucIsDst;							// Daylight saving in minutes
		#if SYS_TIME_HAS_NS
		unsigned    			uiNanoSec;							// Nanoseconds (0..999999999)
		#endif
		}SysTimeStruct;

	void		SysWeekBegin   (unsigned uYear,unsigned uWeek ,unsigned uMode,SysTimeStruct *pData);// Returns the begin of a calender week in a year
	unsigned	SysCalendarWeek(unsigned uDay ,unsigned uMonth,unsigned uYear,unsigned uMode);		// Returns the calender week of a date
	unsigned	SysWeekday     (unsigned uDay ,unsigned uMonth,unsigned uYear);	// Returns the weekday of a date (1=Mo ... 7=So)
	unsigned	SysMounthDays  (               unsigned uMonth,unsigned uYear);	// Returns the the day count of a month
	unsigned	SysIsDaylightSaving(const SysTimeStruct *pData);				// Calculates if a time/date have a daylight saving
	unsigned	SysIsLeapYear  (unsigned uYear);								// Returns 1 if the year is a leapyear
	unsigned	SysWeekdayEx   (SysTime  uTime);								// Returns the weekday of a date (1=Mo ... 7=So)


	SysTick 	SysTimeTick      ();											// Returns a nanosecounds counter
	SysTime		SysTimeGet       ();											// Returns the current time in [nano]secounds since 1.Jan.SYS_BASE_YEAR
	SysTime		SysTimeGetMs	 (unsigned *pMilli);							// Returns the current time and a milli second value
	SysTime		SysTimeLocal     (SysTimeStruct *pData			  ,int i D(=0));// Returns the current time and store the local  time in a time date structure
	SysTime		SysTimeGlobal    (SysTimeStruct *pData);			            // Returns the current time and store the global time in a time date structure
	void 		SysTimeLocalEx   (SysTimeStruct *pData,SysTime iTm,int i D(=0));// Converts a SysTime value in a local  time structure
	void		SysTimeGlobalEx  (SysTimeStruct *pData,SysTime iTm);			// Converts a SysTime value in a global time structure
	void		SysTimeShift     (SysTimeStruct *pData,int iShift);				// Adds a time offset to a time structure
	void 		SysTimeConvert   (SysTimeStruct *pData,const SysTimeStruct *pFrom,int iMode);	// converts local global time
	SysTime		SysTimeFromLocal (const SysTimeStruct *pData);					// Converts the local  time from a time structure in a SysTime value
	SysTime		SysTimeFromGlobal(const SysTimeStruct *pData);					// Converts the global time from a time structure in a SysTime value
	SysTime		SysTimeDataSet   (const SysTimeStruct *pData,int iLocal D(=0));	// Gets the system time of a time structure
	void		SysTimeZone	     (int  iZone,int iDaylightSaving);				// Sets the current time zone
	int 		SysTimeInfo	     (int *pDaylightSaving D(=0));					// Gets the current time zone and daylight saving
	SysTime 	SysTimeFromUtc   (unsigned uTime);								// Converts a UTC time in a SysTime value
	unsigned  	SysTimeToUtc     (SysTime  iTime);								// Converts a SysTime  value in a UTC time
	unsigned  	SysFtToUtc       (SysTick  iTime);								// Converts a FILETIME value in a UTC time
	SysTick  	SysUtcToFt       (unsigned uTime);								// Converts a UTC time value in a FILETIME


	#if			SYS_TIME_HAS_FT
	#define		SysFtToTime(t)	 (t)											// Converts a FILETIME value in a SysTime time
	#define		SysTimeToFt(t)	 (t)											// Converts a SysTime value in a FILETIME time
	#else
	SysTime  	SysFtToTime      (SysTick iTime);								// Converts a FILETIME value in a SysTime time
	SysTick  	SysTimeToFt      (SysTime iTime);								// Converts a SysTime value in a FILETIME time
	#endif


#ifdef	__cplusplus
}
#endif

#undef	D

#endif
