//*****************************************************************************
//*
//*
//*		TaskbarTools.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//
//	Funktionen fr die Steuerung der Taskbar
//
//		BOOL	TaskbarHide(HWND hWnd)		Taste in der Taskbar verstecken
//		BOOL	TaskbarShow(HWND hWnd)		Taste in der Taskbar anzeigen
//
//	Nachrichten Verarbeitung:
//
//		ON_MESSAGE(TB_NOTIFYICON,OnProc)
//
//		void OnProc(WPARAM wParam,LPARAM lParam)	// lParam ist die ID
//			{
//			switch(lParam)
//				{
//			case WM_LBUTTONDOWN:
//			case WM_RBUTTONDOWN:
//				}
//			}

#include	<windows.h>
#include	<shellapi.h>
#include	<shlwapi.h>
#include	<shlobj.h>
#include	<string.h>
#include	"TaskbarTools.h"

#if        _MSC_VER >= 1000
#pragma		warning( disable :4996 )
#endif


#ifndef _WCHAR_T_DEFINED
	#if _MSC_VER
	#include	<wchar.h>
	#else
	typedef	 unsigned short	wchar_t;
	#define _WCHAR_T_DEFINED	1
	#endif
#endif


const IID	IID_ITaskbarList3_	= {0xEA1AFB91,0x9E28,0x4B86,0x90,0xE9,0x9E,0x9F,0x8A,0x5E,0xEF,0xAF};



typedef enum THUMBBUTTONMASK
	{
	THB_BITMAP   		= 0x00000001,
	THB_ICON     		= 0x00000002,
	THB_TOOLTIP  		= 0x00000004,
	THB_FLAGS    		= 0x00000008
	}THUMBBUTTONMASK;

typedef enum THUMBBUTTONFLAGS
	{
	THBF_ENABLED        = 0x00000000,
	THBF_DISABLED       = 0x00000001,
	THBF_DISMISSONCLICK = 0x00000002,
	THBF_NOBACKGROUND   = 0x00000004,
	THBF_HIDDEN         = 0x00000008,
	THBF_NONINTERACTIVE = 0x00000010
	}THUMBBUTTONFLAGS;

typedef struct THUMBBUTTON
	{
	THUMBBUTTONMASK  	dwMask;
	UINT             	iId;
	UINT             	iBitmap;
	HICON            	hIcon;
	WCHAR				szTip[260];
	THUMBBUTTONFLAGS	dwFlags;
	}THUMBBUTTON,*LPTHUMBBUTTON;

typedef struct ITaskbarListVtbl
    {
	BEGIN_INTERFACE

	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *QueryInterface		)( VOID __RPC_FAR *pThis,IID *pRiid,void __RPC_FAR *__RPC_FAR *pObject);
	ULONG   ( STDMETHODCALLTYPE __RPC_FAR *AddRef         		)( VOID __RPC_FAR *pThis);
	ULONG   ( STDMETHODCALLTYPE __RPC_FAR *Release        		)( VOID __RPC_FAR *pThis);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *HrInit	      		)( VOID __RPC_FAR *pThis);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *AddTab		  		)( VOID __RPC_FAR *pThis,HWND hWnd);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *DeleteTab	  		)( VOID __RPC_FAR *pThis,HWND hWnd);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *ActivateTab	  		)( VOID __RPC_FAR *pThis,HWND hWnd);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetActiveAlt	  		)( VOID __RPC_FAR *pThis,HWND hWnd);

    END_INTERFACE
    } ITaskbarListVtbl;

typedef struct ITaskbarList3Vtbl
    {
    BEGIN_INTERFACE

	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *QueryInterface		)(VOID __RPC_FAR *pThis,IID *pRiid,void __RPC_FAR *__RPC_FAR *pObject);
	ULONG   ( STDMETHODCALLTYPE __RPC_FAR *AddRef				)(VOID __RPC_FAR *pThis);
	ULONG   ( STDMETHODCALLTYPE __RPC_FAR *Release				)(VOID __RPC_FAR *pThis);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *HrInit				)(VOID __RPC_FAR *pThis);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *AddTab				)(VOID __RPC_FAR *pThis,HWND hWnd);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *DeleteTab			)(VOID __RPC_FAR *pThis,HWND hWnd);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *ActivateTab			)(VOID __RPC_FAR *pThis,HWND hWnd);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetActiveAlt			)(VOID __RPC_FAR *pThis,HWND hWnd);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *MarkFullscreenWindow	)(VOID __RPC_FAR *pThis,HWND hWnd,BOOL      bFullScreen);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetProgressValue		)(VOID __RPC_FAR *pThis,HWND hWnd,ULONGLONG uCompleted,ULONGLONG uTotal);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetProgressState		)(VOID __RPC_FAR *pThis,HWND hWnd,DWORD     dwTbpFlags);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *RegisterTab			)(VOID __RPC_FAR *pThis,HWND hwndTab,HWND   hWndMdi);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *UnregisterTab		)(VOID __RPC_FAR *pThis,HWND hwndTab);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetTabOrder			)(VOID __RPC_FAR *pThis,HWND hwndTab,HWND hWndInsertBefore);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetTabActive			)(VOID __RPC_FAR *pThis,HWND hwndTab,HWND hWndMdi,DWORD dwReserved);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *ThumbBarAddButtons	)(VOID __RPC_FAR *pThis,HWND hWnd,UINT uButtons,LPTHUMBBUTTON pButton);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *ThumbBarUpdateButtons)(VOID __RPC_FAR *pThis,HWND hWnd,UINT uButtons,LPTHUMBBUTTON pButton);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *ThumbBarSetImageList	)(VOID __RPC_FAR *pThis,HWND hWnd,HIMAGELIST hImageList);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetOverlayIcon		)(VOID __RPC_FAR *pThis,HWND hWnd,HICON   hIcon,LPCWSTR pDescription);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetThumbnailTooltip	)(VOID __RPC_FAR *pThis,HWND hWnd,LPCWSTR pTip);
	HRESULT ( STDMETHODCALLTYPE __RPC_FAR *SetThumbnailClip		)(VOID __RPC_FAR *pThis,HWND hWnd,LPRECT  pClip);

    END_INTERFACE
    } ITaskbarList3Vtbl;




#if		UNICODE
#define strzncpy	wcsncpy
#else
#define strzncpy	strncpy
#endif


typedef struct
	{
	ITaskbarList3Vtbl	*pVtbl;
	}ITaskbarList_;


static	ITaskbarList_	*pList		 = 0;
static	int				 iExitAdd	 = 0;
static	int				 iIsVersion3 = 0;


//*****************************************************************************
//*
//*		TaskbarInit
//*
//*****************************************************************************
//	Initialisiert die Taskbar-Routienen
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarInit()
{
HRESULT		uRet;


	if(pList)return TRUE;

	uRet = CoInitialize(NULL);

	if(!SUCCEEDED(uRet))
		{
		return FALSE;
		}

	uRet = CoCreateInstance(CLSID_TaskbarList,NULL,CLSCTX_INPROC_SERVER,IID_ITaskbarList3_,(void**)&pList);

	if(!SUCCEEDED(uRet))
		{
		uRet = CoCreateInstance(CLSID_TaskbarList,NULL,CLSCTX_INPROC_SERVER,IID_ITaskbarList,(void**)&pList);

		if(!SUCCEEDED(uRet))
			{
			return FALSE;
			}

		iIsVersion3 = 0;
		}
	else{
		iIsVersion3 = 1;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TaskbarExit
//*
//*****************************************************************************
//	Deinitialisiert die Taskbar Routienen
void __cdecl TaskbarExit(void)
{

	if(!pList)return;

	if(!IsBadReadPtr(pList->pVtbl,sizeof(ITaskbarListVtbl)))
		{
		pList->pVtbl->Release(pList);
		}

	CoUninitialize();
	pList=0;

}

//*****************************************************************************
//*
//*		TaskbarHide
//*
//*****************************************************************************
//	Versteckt die Taskleiste die zu hWnd gehrt
//	hWnd	: Ist das Handle vom Hauptfenster oder dem Dialog, vom dem
//			  die Taskleiste ausgeblendet werden soll.
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarHide(HWND hWnd)
{
HRESULT uRet;


	if(!pList)
		{
		if(!TaskbarInit())return FALSE;
		if(!iExitAdd){atexit(TaskbarExit);iExitAdd=1;}
		}

	uRet=pList->pVtbl->DeleteTab(pList,hWnd);
	if(!SUCCEEDED(uRet))return FALSE;


return TRUE;
}


//*****************************************************************************
//*
//*		TaskbarShow
//*
//*****************************************************************************
//	Erstellt eine Taste in der Taskleiste die zu hWnd gehrt
//	hWnd	: Ist das Handle vom Hauptfenster oder dem Dialog, vom dem
//			  die Taskleiste angezeigt werden soll.
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarShow(HWND hWnd)
{
HRESULT uRet;


	if(!pList)
		{
		if(!TaskbarInit())return FALSE;
		if(!iExitAdd){atexit(TaskbarExit);iExitAdd=1;}
		}

	uRet=pList->pVtbl->AddTab     (pList,hWnd);
	uRet=pList->pVtbl->ActivateTab(pList,hWnd);

	if(!SUCCEEDED(uRet))return FALSE;


return TRUE;
}


//*****************************************************************************
//*
//*		TaskbarProgress
//*
//*****************************************************************************
//	Setzt die Position der Progress-Bar im Taskbar-Fenster
//	hWnd	: Ist das Handle vom Hauptfenster oder dem Dialog, in dem
//			  die Progressanzeige angezeigt werden soll.
//	iNow	: Ist die aktuelle Position
//	iMax	: Ist der minimale Wert im Bereich
//	iMin	: Ist der maximale Wert im Bereich
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarProgress(HWND hWnd,LONGLONG iNow,LONGLONG iMax,LONGLONG iMin)
{
HRESULT uRet;



	if(!pList)
		{
		if(!TaskbarInit())return FALSE;
		if(!iExitAdd){atexit(TaskbarExit);iExitAdd=1;}
		}

	if(!iIsVersion3)
		{
		return FALSE;
		}


	iMax -= iMin;
	iNow -= iMin;


	uRet=pList->pVtbl->SetProgressValue(pList,hWnd,iNow,iMax);
	if(!SUCCEEDED(uRet))return FALSE;


return TRUE;
}


//*****************************************************************************
//*
//*		TaskbarSetState
//*
//*****************************************************************************
//	Stellt den Modus der Progress-Bar im Taskbar-Fenster ein
//	hWnd	: Ist das Handle vom Hauptfenster oder dem Dialog, in dem
//			  die Progressanzeige angezeigt werden soll.
//	dwFlags	: Diverse Flags fr den Progress-Bar-Status
//				TB_NOPROGRESS		0x0000	Progress-Bar abschalten
//				TB_TICKER			0x0001	Progress-Bar in den Ticker-Modus umschalten
//				TB_NORMAL			0x0002	Normale Farbe fr Progress-Bar
//				TB_ERROR			0x0004	Rote    Farbe fr Progress-Bar
//				TB_PAUSED			0x0008	Gelbe   Farbe fr Progress-Bar
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarSetState(HWND hWnd,DWORD dwFlags)
{
HRESULT uRet;



	if(!pList)
		{
		if(!TaskbarInit())return FALSE;
		if(!iExitAdd){atexit(TaskbarExit);iExitAdd=1;}
		}

	if(!iIsVersion3)
		{
		return FALSE;
		}


	uRet=pList->pVtbl->SetProgressState(pList,hWnd,dwFlags);
	if(!SUCCEEDED(uRet))return FALSE;


return TRUE;
}


//*****************************************************************************
//*
//*		TaskbarIconSet
//*
//*****************************************************************************
//	ndert das Icon der in der Taskbar
//	hWnd	: Handle vom Hauptfenster
//	hIcon	: Resourcen Name fr Icon
//	pText	: Text der ber Icon eingeblendet wird
//			  Ist Text gleich Null, so wird der Icon-Text nicht verndert
//	uId		: User uId
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarIconSet(HWND hWnd,HICON hIcon,LPCTSTR pText,LPARAM uId)
{
NOTIFYICONDATA	sNotify;
BOOL			iRet;


	sNotify.cbSize			 = sizeof(NOTIFYICONDATA);
	sNotify.hWnd			 = hWnd;
	sNotify.uID				 = uId;
	sNotify.uFlags			 = NIF_MESSAGE|NIF_ICON;
	sNotify.uCallbackMessage = TB_NOTIFYICON;
	sNotify.hIcon			 = hIcon;

	if(pText)
		{
		sNotify.uFlags |= NIF_TIP;
		strzncpy(sNotify.szTip,pText,sizeof(sNotify.szTip)/sizeof(*pText));
		}
	else{
		sNotify.szTip[0] = 0;
		}

	iRet = Shell_NotifyIcon(NIM_MODIFY,&sNotify);


return iRet;
}

BOOL TaskbarIconSet(HWND hWnd,LPCTSTR pIcon,LPCTSTR pText,LPARAM uId)
{
HICON	hIcon;
BOOL	iRet;


	hIcon	= (HICON)LoadImage(GetModuleHandle(NULL),pIcon,IMAGE_ICON,16,16,0);
	iRet	= TaskbarIconSet(hWnd,hIcon,pText,uId);

	if(hIcon)DestroyIcon(hIcon);


return iRet;
}


//*****************************************************************************
//*
//*		TaskbarIconAdd
//*
//*****************************************************************************
//	Fgt ein Icon in die Taskbar ein
//	hWnd	: Handle vom Hauptfenster
//	hIcon	: Resourcen Name fr Icon
//	pText	: Text der ber Icon eingeblendet wird
//			  Ist Text gleich Null, so wird der Icontext nicht verndert
//	uId		: User uId
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarIconAdd(HWND hWnd, HICON hIcon,LPCTSTR pText,LPARAM uId)
{
NOTIFYICONDATA	sNotify;
BOOL			iRet;



	sNotify.cbSize			 = sizeof(NOTIFYICONDATA);
	sNotify.hWnd			 = hWnd;
	sNotify.uID				 = uId;
	sNotify.uFlags			 = NIF_MESSAGE|NIF_ICON;
	sNotify.uCallbackMessage = TB_NOTIFYICON;
	sNotify.hIcon			 = hIcon;

	if(pText)
		{
		sNotify.uFlags |= NIF_TIP;
		strzncpy(sNotify.szTip,pText,sizeof(sNotify.szTip)/sizeof(sNotify.szTip[0]));
		}
	else{
		sNotify.szTip[0] = 0;
		}

	iRet = Shell_NotifyIcon(NIM_ADD,&sNotify);


return iRet;
}

BOOL TaskbarIconAdd(HWND hWnd,LPCTSTR pIcon,LPCTSTR pText,LPARAM uId)
{
HICON	hIcon;
BOOL	iRet;


	hIcon	= (HICON)LoadImage(GetModuleHandle(NULL),pIcon,IMAGE_ICON,16,16,0);
	iRet	= TaskbarIconAdd(hWnd,hIcon,pText,uId);

	if(hIcon)DestroyIcon(hIcon);


return iRet;
}


//*****************************************************************************
//*
//*		TaskbarIconDel
//*
//*****************************************************************************
//	Entfernt ein Icon aus der Taskbar
//	hWnd	: Handle vom Hauptfenster
//	uId		: User uId
//	Ergibt TRUE wenn erfolgreich
BOOL TaskbarIconDel(HWND hWnd,LPARAM uId)
{
NOTIFYICONDATA	sNotify;
BOOL			iRet;


	sNotify.cbSize				= sizeof(NOTIFYICONDATA);
	sNotify.hWnd				= hWnd;
	sNotify.uID					= uId;
	sNotify.uFlags				= NIF_MESSAGE|NIF_ICON;
	sNotify.uCallbackMessage	= TB_NOTIFYICON;
	sNotify.hIcon				= 0;
	sNotify.szTip[0]			= 0;

	iRet = Shell_NotifyIcon(NIM_DELETE,&sNotify);



return iRet;
}


