//*****************************************************************************
//*
//*
//*		TcpStream.cpp
//*
//*
//*****************************************************************************
//
//	Klasse um einzelne TCP-Streams aus Ethernet-Paketen zusammen zu bauen:
//
//		for(;;)
//			{
//			unsigned char  *pData;
//			CTcpStreams		sStream;
//			int				iIndex;
//			int				iSize;
//
//				pData = WiresharkRead(pHandle);				// Paket laden
//			if(!pData)break;
//															// Paket einfgen
//			   iIndex=sStream.Put(pData,pHandle->uSize,&pHandle->uTime);
//			if(iIndex>=0)
//				{
//					pData	= sStream.Data(iIndex,iSize);	// Daten aus holen
//				if(!pData || iSize<=0)break;
//
//					iSkip = AnalyseData(iSize,pData);		// Daten aus Stream auswerten
//				if(!iSkip)continue;
//
//				printf("Packet ID     = %i\n",sStream.Label          (iIndex,0));
//				printf("Packet Port1  = %i\n",sStream.GetSourcePort  (iIndex/2));
//				printf("Packet Port1  = %i\n",sStream.GetTargetPort  (iIndex/2));
//				printf("Packet Stream = %i\n",sStream.GetStreamNumber(iIndex/2));
//
//				sStream.Skip(iIndex,iSkip);					// Daten aus Stream lschen
//				}
//
#include	<stddef.h>
#include	<string.h>
#include	<assert.h>

#include	"TcpStream.h"
#include	"StringFunctions.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	CTS_BASE_SIZE	offsetof(StreamEntry,aData)
#ifndef		CTS_MAX_SEG
#define 	CTS_MAX_SEG		0x2000
#endif

#define 	GET_WORD(w)		((unsigned short)((((unsigned short)(w))>>8) | (((unsigned short)(w))<<8)))
#define 	GET_DWORD(w)	((GET_WORD(w)<<16) | GET_WORD((w)>>16))

#define 	PTR_WORD(p)	   *((unsigned short*)(p))
#define 	PTR_DWORD(p)   *((unsigned int  *)(p))

#define 	TYPE_IP			0x0800
#define 	TYPE_IPV6		0x86DD
#define		TYPE_VLAN		0x8100
#define 	TYPE_TCP		0x0006



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
//	uDataSize	   : Ist die maximale Paketgre die ausgelesen wird
CTcpStream::CTcpStream(unsigned uDataSize)
{

	   m_uDataSize	= uDataSize*4;
	if(m_uDataSize<CTS_MAX_SEG)
		{
		m_uDataSize = CTS_MAX_SEG;
		}

	m_pData			= new unsigned char[m_uDataSize+4];
	m_uDataRead		= 0;
	m_uDataUsed		= 0;
	m_uDataAll		= 0;
	m_uCopyPos		= 0;
	m_uCount		= 0;
	m_uClosed		= 0;
	m_uLastTime		= 0;
	m_uForwardCnt	= 0;


	m_sDataList.Len(1);

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CTcpStream::~CTcpStream()
{

	while(m_uForwardCnt>0)
		{
		m_uForwardCnt--;
		delete m_aForwardPtr[m_uForwardCnt];
		}

	delete m_pData;

	m_pData		= 0;
	m_uDataRead	= 0;
	m_uDataUsed	= 0;
	m_uDataAll	= 0;
	m_uCount	= 0;


}

//*****************************************************************************
//*
//*		Ignore
//*
//*****************************************************************************
//	Daten ignorieren, z.B. bei Keepalive-Paketen
//	pData		: Zeiger auf die einzufgenden Daten
//	iSize		: Gre der einzufgenden Daten
//	uOffset		: Offset im TCP-Stream
//	Ergibt die neue Lnge des gesammten Puffers, bzw.0 wenn das Paket nicht passt.
int	CTcpStream::Ignore(int iSize,unsigned uOffset)
{


	if(iSize<=0 || iSize>CTS_MAX_SEG || m_uClosed)	// Datengre prfen
		{
		return 0;
		}

	if(m_uCount==0)									// TCP-Offset prfen
		{
		return 0;
		}

	if(uOffset!=m_uOffset)							// Passt der Offset
		{
		return 0;
		}

	m_uOffset = uOffset+iSize;						// Neuen Offset bernehmen


return 1;
}


//*****************************************************************************
//*
//*		Put
//*
//*****************************************************************************
//	Daten in den Stream einfgen
//	pData		: Zeiger auf die einzufgenden Daten
//	iSize		: Gre der einzufgenden Daten
//	uOffset		: Offset im TCP-Stream
//	uTcpPorts	: Sind die TCP-Ports aus dem Datenpaket
//					(big endian,0..1=Source 2..3=Target)
//	pInfo		: Zeiger auf diverse Infos
//					[0] UTC-Zeit wannn die Daten empfangen wurden
//					[1] s wannn die Daten empfangen wurden
//					[2] Paket-ID
//	Ergibt die neue Lnge des gesammten Puffers, bzw.0 wenn das Paket nicht passt.
//	oder die negative Anzahl der Offset Fehler
int	CTcpStream::Put(const void *pData,int iSize,unsigned uOffset,unsigned uTcpPorts,unsigned *pInfo)
{
StreamEntry	   *pEntry;
int				iDiff;
int				iRet;

	assert(iSize<=CTS_MAX_SEG);


	if(iSize<=0 || iSize>CTS_MAX_SEG || m_uClosed)	// Datengre prfen
		{
		return 0;
		}

	if(m_uCount)									// TCP-Offset prfen
		{
		if(uOffset!=m_uOffset)
			{
			   iDiff = uOffset-m_uOffset;
			if(iDiff<= 0)
				{
				return 0;
				}
													// Paket in den Zukunftspuffer eintragen
			   iRet = PutForward(pData,iSize,uOffset,uTcpPorts,pInfo);
			if(iRet>=0)
				{
				return iRet;
				}

			if(iDiff<64)
				{
				return 0;
				}

			m_uErrorCnt++;

			return -(int)m_uErrorCnt;
			}
		}

	m_uErrorCnt = 0;


		pEntry = (StreamEntry*)new char[CTS_BASE_SIZE+iSize];
	if(!pEntry)
		{
		return 0;
		}

	if((m_uCopyPos^m_sDataList.Count())==0)			// Daten an Lesepuffer anhngen
	if( m_uDataUsed+iSize<m_uDataSize)
		{
		memcpy(m_pData+m_uDataUsed,pData,iSize);
		m_uDataUsed += iSize;
		m_uCopyPos  += 1;
		}

	uTcpPorts			= GET_DWORD(uTcpPorts);

	pEntry->uLenData	= iSize;					// Datenpuffer einfgen
	pEntry->uTimeSec	= pInfo[0];
	pEntry->uTimeUsec	= pInfo[1];
	pEntry->uPacketId	= pInfo[2];
	pEntry->uTargetPort	= (unsigned short)(uTcpPorts);
	pEntry->uSourcePort	= (unsigned short)(uTcpPorts>>16);
	m_uLastTime			= pInfo[0];

	memcpy(pEntry->aData,pData,iSize);

	m_sDataList.Add(pEntry);
	m_uOffset	= uOffset+iSize;
	m_uDataAll += iSize;
	m_uCount   += 1;



return m_uDataAll;
}



//*****************************************************************************
//*
//*		PutForward
//*
//*****************************************************************************
//	Daten in den Zukunftspuffer des Streams einfgen
//	pData		: Zeiger auf die einzufgenden Daten
//	iSize		: Gre der einzufgenden Daten
//	uOffset		: Offset im TCP-Stream
//	uTcpPorts	: Sind die TCP-Ports aus dem Datenpaket
//					(big endian,0..1=Source 2..3=Target)
//	pInfo		: Zeiger auf diverse Infos
//					[0] UTC-Zeit wannn die Daten empfangen wurden
//					[1] s wannn die Daten empfangen wurden
//					[2] Paket-ID
//	Ergibt die neue Lnge des gesammten Puffers, 0 wenn das Paket gepuffert ist,
//	oder -1 wenn der Puffer ungltig ist.
int	CTcpStream::PutForward(const void *pData,int iSize,unsigned uOffset,unsigned uTcpPorts,unsigned *pInfo)
{
unsigned char	aData[256];
StreamForward  *pForward;
unsigned		uIndex;
unsigned		uPlace;
int				iDelta;
int				iDiff;



	if(m_uForwardCnt>=CTS_FORWARD_COUNT)
		{
		return -1;
		}


	if(m_uForwardCnt>0)								// Einfgeposition suchen
		{
		iDiff  = uOffset-m_uOffset;

		for(uIndex=m_uForwardCnt;uIndex>0;uIndex--)
			{
			pForward	= m_aForwardPtr[uIndex-1];
			iDelta		= pForward->uOffset-m_uOffset;

			if(iDelta <iDiff)break;
			if(iDelta==iDiff)return 0;
			if(iDelta >    0)continue;

			delete pForward;

			for(uPlace=uIndex;uPlace;)				// Alle Puffer vorher freigeben
				{
				uPlace--;
				delete m_aForwardPtr[uPlace];
				}

			uIndex++;

			for(uPlace=uIndex;uPlace<m_uForwardCnt;uPlace++)
				{
				m_aForwardPtr[uPlace-uIndex] = m_aForwardPtr[uPlace];
				}

			m_uForwardCnt -= uIndex;

			return 0;
			}
		}
	else{											// Erster Eintrag im Puffer
		uIndex = 0;
		}




		pForward = (StreamForward*)new char[CTS_BASE_SIZE+iSize+sizeof(unsigned)];
	if(!pForward)
		{
		return 0;
		}

	pForward->uLenData		= iSize;				// Datenpuffer einfgen
	pForward->uTimeSec		= pInfo[0];
	pForward->uTimeUsec		= pInfo[1];
	pForward->uPacketId		= pInfo[2];
	pForward->uTcpPorts		= uTcpPorts;
	pForward->uOffset		= uOffset;

	memcpy(pForward->aData,pData,iSize);


	for(uPlace=m_uForwardCnt;uPlace>uIndex;uPlace++)// Den neuen Puffer einfgen
		{
		m_aForwardPtr[uPlace] = m_aForwardPtr[uPlace-1];
		}

	m_aForwardPtr[uIndex] = pForward;
	m_uForwardCnt++;


	if(m_uForwardCnt<=1)return 0;


	pForward = m_aForwardPtr[0];
													// Sind die ersten beiden Puffer direkt hinter einander
	if(pForward->uOffset+pForward->uLenData!=m_aForwardPtr[1]->uOffset)
		{
		return 0;
		}


	iDiff 	= pForward->uOffset-m_uOffset;
	iDelta	= (iDiff>sizeof(aData))? sizeof(aData):iDiff;

	memset(aData,0x9D,iDelta);

	while(iDiff>0)									// Leeraum im Stream fllen
		{
		iDelta = (iDiff>sizeof(aData))? sizeof(aData):iDiff;
		iDiff -= iDelta;

		Put(aData,iDelta,m_uOffset,pForward->uTcpPorts,&pForward->uTimeSec);
		}


	for(uIndex=0;;)									// Alle direkten Puffer einfgen
		{
		if(pForward->uOffset!=m_uOffset)break;
		Put(pForward->aData,pForward->uLenData,pForward->uOffset,pForward->uTcpPorts,&pForward->uTimeSec);

		   uIndex++;
		if(uIndex>=m_uForwardCnt)break;

		pForward = m_aForwardPtr[uIndex];
		}



	for(uPlace=uIndex;uPlace;)						// Alle eingefgten Puffer freigeben
		{
		uPlace--;
		delete m_aForwardPtr[uPlace];
		}

	for(uPlace=uIndex;uPlace<m_uForwardCnt;uPlace++)
		{
		m_aForwardPtr[uPlace-uIndex] = m_aForwardPtr[uPlace];
		}

	m_uForwardCnt -= uIndex;




return m_uDataAll;
}


//*****************************************************************************
//*
//*		Skip
//*
//*****************************************************************************
//	Bytes aus dem Puffer lschen
//	iSize		: Ist die Anzahl der Bytes
//	Ergibt die Zahl der gelschten Bytes
int	CTcpStream::Skip(int iSize)
{
StreamEntry	   *pEntry;
unsigned		uNext;
unsigned		uPos;
unsigned		uOff;
int				iMax;


	iMax = m_uDataUsed-m_uDataRead;

	if(iSize>=iMax)iSize=iMax;
	if(iSize<=   0)return 0;

	m_uDataRead += iSize;
	m_uDone     += iSize;

													// Ist der Lesepuffer zu klein
	if(m_uDataRead>=m_uDataSize/4*3	|| m_uDataRead==m_uDataUsed)
		{
		uOff = 0;

		for(uPos=0;uPos<m_uCopyPos;uPos++)			// Datenpuffer freigeben
			{
			pEntry = (StreamEntry*)m_sDataList[uPos];
			uNext  = uOff+pEntry->uLenData;
			if(uNext>m_uDataRead)break;
			uOff   = uNext;
			}

		if(uOff>m_uDataUsed)
			{
			uOff = m_uDataUsed;
			}

		m_sDataList.Del(0,uPos);
		m_uCopyPos   -= uPos;
		m_uDataUsed  -= uOff;
		m_uDataRead  -= uOff;

		memmove(m_pData,m_pData+uOff,m_uDataUsed);
		}


	while(m_uCopyPos<(unsigned)m_sDataList.Count())	// Datenpuffer fllen
		{
		pEntry = (StreamEntry*)m_sDataList[m_uCopyPos];
		uOff   = m_uDataUsed+pEntry->uLenData;

		if(uOff>m_uDataSize)break;

		memcpy(m_pData+m_uDataUsed,pEntry->aData,pEntry->uLenData);
		m_uDataUsed = uOff;
		m_uCopyPos += 1;
		}

													// Alles aus dem Stream
	if(m_uClosed==1 && m_uDataUsed==m_uDataRead && m_sDataList.Count()==0)
		{
		m_uClosed = 2;
		}



return iSize;
}

//*****************************************************************************
//*
//*		Time
//*
//*****************************************************************************
//	Holt die Zeit eines Bytes aus dem Lesebuffer
//	uByte	: Ist der Byte-Offset im Lesepuffer
//	pUsec	: Ist ein Zeiger auf den Mikrosekundenspeicher (0=nicht speichern)
//	Ergibt die Empfangszeit
unsigned CTcpStream::Time(unsigned uByte,void *pUsec) const
{
StreamEntry	   *pEntry;
unsigned		uPos;
unsigned		uOff;


	uOff   = 0;
	pEntry = 0;
	uByte += m_uDataRead;

	for(uPos=0;uPos<m_uCopyPos;uPos++)				// Datenpuffer freigeben
		{
		pEntry = (StreamEntry*)m_sDataList[uPos];
		uOff  += pEntry->uLenData;
		if(uOff>=uByte)break;
		}

	if(pEntry)										// Zeit aus Datenpuffer holen
		{
		if(pUsec)*(unsigned*)pUsec=pEntry->uTimeUsec;
		return pEntry->uTimeSec;
		}

	if(pUsec)
		{
		*(unsigned*)pUsec=0;
		}


return 0;
}

//*****************************************************************************
//*
//*		Ports
//*
//*****************************************************************************
//	Holt die realen TCP-Ports eines Bytes aus dem Lesebuffer
//	uByte	: Ist der Byte-Offset im Lesepuffer
//	pTarget	: Ist ein Zeiger auf den Taget-Port-Speicher (0=nicht speichern)
//	Ergibt das Source-Port
unsigned CTcpStream::Ports(unsigned uByte,unsigned short *pTarget)const
{
StreamEntry	   *pEntry;
unsigned		uPos;
unsigned		uOff;


	uOff   = 0;
	pEntry = 0;
	uByte += m_uDataRead;

	for(uPos=0;uPos<m_uCopyPos;uPos++)				// Datenpuffer freigeben
		{
		pEntry = (StreamEntry*)m_sDataList[uPos];
		uOff  += pEntry->uLenData;
		if(uOff>=uByte)break;
		}

	if(pEntry)										// Zeit aus Datenpuffer holen
		{
		if(pTarget)*pTarget=pEntry->uTargetPort;
		return pEntry->uSourcePort;
		}

	if(pTarget)
		{
		*pTarget=0;
		}


return 0;
}

//*****************************************************************************
//*
//*		Info
//*
//*****************************************************************************
//	Holt die Informationen zu einem Byte aus dem Lesebuffer
//	uByte	: Ist der Byte-Offset im Lesepuffer
//	uSize	: Hier wird die Ge des restlichen Segments gespeichert
//	Ergibt einen Zeiger auf die Informationen bzw. 0 bei einem Fehler
StreamInfo *CTcpStream::Info(unsigned uByte)const
{
StreamEntry	   *pEntry;
unsigned		uPos;
unsigned		uOff;


	uOff   = 0;
	pEntry = 0;
	uByte += m_uDataRead;

	for(uPos=0;uPos<m_uCopyPos;uPos++)				// Datenpuffer freigeben
		{
		pEntry = (StreamEntry*)m_sDataList[uPos];
		uOff  += pEntry->uLenData;

		if(uOff>uByte)							   // Zeit aus Datenpuffer holen
			{
			return (StreamInfo*)&pEntry->uTimeSec;
			}
		}


return 0;
}

StreamInfo *CTcpStream::Info(unsigned uByte,unsigned &uSize)const
{
StreamEntry	   *pEntry;
unsigned		uPos;
unsigned		uOff;


	uOff   = 0;
	pEntry = 0;
	uByte += m_uDataRead;

	for(uPos=0;uPos<m_uCopyPos;uPos++)				// Datenpuffer freigeben
		{
		pEntry = (StreamEntry*)m_sDataList[uPos];
		uOff  += pEntry->uLenData;

		if(uOff>uByte)								// Zeit aus Datenpuffer holen
			{
			uSize = uOff-uByte;
			return (StreamInfo*)&pEntry->uTimeSec;
			}
		}

	uSize = 0;



return 0;
}


//*****************************************************************************
//*
//*		Label
//*
//*****************************************************************************
//	Holt die Paket-ID eines Bytes aus dem Lesebuffer
//	uByte	: Ist der Byte-Offset im Lesepuffer
//	Ergibt die Paket-ID.
unsigned CTcpStream::Label(unsigned uByte) const
{
StreamEntry	   *pEntry;
unsigned		uPos;
unsigned		uOff;


	uByte += m_uDataRead;
	uOff   = 0;

	for(uPos=0;uPos<m_uCopyPos;uPos++)				// Datenpuffer freigeben
		{
		pEntry = (StreamEntry*)m_sDataList[uPos];
		uOff  += pEntry->uLenData;

		if(uOff>=uByte)
			{
			return pEntry->uPacketId;
			}
		}



return 0;
}


//*****************************************************************************
//*
//*		Reset
//*
//*****************************************************************************
//	Alles im Stream lschen
//	uNewSize		: Neue Lesebpuffergre (0=keine nderung)
void CTcpStream::Reset(unsigned uNewSize)
{
unsigned	uDataUsed;




	if(uNewSize)
		{
		   uDataUsed = uNewSize*4;
		if(uDataUsed<4096)uDataUsed=4096;
		if(uDataUsed!=m_uDataSize)
			{
			delete m_pData;
			m_pData		= new unsigned char[uDataUsed+4];
			m_uDataSize	= uNewSize;
			}
		}

	m_sDataList.Del();
	m_uDataRead	= 0;
	m_uDataUsed	= 0;
	m_uDataAll	= 0;
	m_uCopyPos	= 0;
	m_uCount	= 0;
	m_uDone		= 0;
	m_uClosed	= 0;
	m_uLastTime	= 0;
	m_uErrorCnt	= 0;


}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schliet einen Stream
//	uFinish		: Wurde der Stream bereits ganz gelesen
void CTcpStream::Close(unsigned uFinish)
{

	while(m_uForwardCnt>0)
		{
		m_uForwardCnt--;
		delete m_aForwardPtr[m_uForwardCnt];
		}

	if(uFinish)
		{
		m_uClosed=3;
		return;
		}

	if(m_uClosed)
		{
		return;
		}

	if(m_uDataUsed==m_uDataRead && m_sDataList.Count()==0)
		{
		m_uClosed = 2;
		}
	else{
		m_uClosed = 1;
		}


}


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CTcpStreams::CTcpStreams()
{

	m_uFilterCnt	= 0;
	m_uStreamCnt	= 0;
	m_uStreamNum	= 0;
	m_uFinSize		= 0;
	m_uSynSize		= 0;
	m_uRstSize		= 0;
	m_uErrSize		= 0;

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CTcpStreams::~CTcpStreams()
{

	m_uFilterCnt	= 0;
	m_uStreamCnt	= 0;

}

//*****************************************************************************
//*
//*		Reset
//*
//*****************************************************************************
//	Alle Streams resetieren
//	uNewSize		: Neue Lesebpuffergre (0=keine nderung)
void CTcpStreams::Reset(unsigned uNewSize)
{
unsigned	uStream;


	for(uStream=0;uStream<CTS_STREAM_COUNT*2;uStream++)
		{
		m_aStreams[uStream].Reset(uNewSize);
		}

	m_uStreamCnt = 0;

}

//*****************************************************************************
//*
//*		ClearFilters
//*
//*****************************************************************************
//	Lsch alle vorhandenen Filter
void CTcpStreams::ClearFilters()
{

	m_uFilterCnt = 0;

}

//*****************************************************************************
//*
//*		AddFilter
//*
//*****************************************************************************
//	Einen Filter hinzufgen
//	usSourcePort	: TCP-Port   der Quelle (0=nicht prfen)
//	uSourceIp		: IP-Adresse der Quelle (0=nicht prfen)
//	usTargetPort	: TCP-Port   des Ziels  (0=nicht prfen)
//	uTargetIp		: IP-Adresse der Ziels  (0=nicht prfen)
//	Ergibt 1 wenn der Filder hinzugefgt wurde
int	CTcpStreams::AddFilter(unsigned short usSourcePort,unsigned uSourceIp,unsigned short usTargetPort,unsigned uTargetIp)
{

	if(m_uFilterCnt>=CTS_FILTER_COUNT)
		{
		return 0;
		}

	if(usSourcePort==0)
		{
		return 0;
		}

	usSourcePort = btol_w(usSourcePort);
	usTargetPort = btol_w(usTargetPort);

	m_aSourceIp  [m_uFilterCnt]	= uSourceIp;
	m_aTargetIp  [m_uFilterCnt]	= uTargetIp;
	m_aSourcePort[m_uFilterCnt]	= usSourcePort| (usTargetPort<<16);
	m_aTargetPort[m_uFilterCnt]	= usTargetPort| (usSourcePort<<16);

	m_uFilterCnt++;


return 1;
}

//*****************************************************************************
//*
//*		AddFilter
//*
//*****************************************************************************
//	Einen Filter hinzufgen
//	usSourcePort	: TCP-Port   der Quelle (0=nicht prfen)
//	pSourceIp		: IP-Adresse der Quelle (0=nicht prfen)
//	usTargetPort	: TCP-Port   des Ziels  (0=nicht prfen)
//	pTargetIp		: IP-Adresse der Ziels  (0=nicht prfen)
//	Ergibt 1 wenn der Filder hinzugefgt wurde
int	CTcpStreams::AddFilter(unsigned short usSourcePort,const char *pSourceIp,unsigned short usTargetPort,const char *pTargetIp)
{
unsigned	uSourceIp;
unsigned	uTargetIp;
unsigned	uPos;
unsigned	uVal;


	if(m_uFilterCnt>=CTS_FILTER_COUNT)
		{
		return 0;
		}

	if(usSourcePort==0)
		{
		return 0;
		}

	uSourceIp	= 0;
	uTargetIp	= 0;

	if(pSourceIp)
		{
		for(uPos=0;uPos<4;uPos++)
			{
			uVal		= atoi(pSourceIp)&0xFF;
			pSourceIp  += strwlen(pSourceIp,0x01);
			uSourceIp  |= uVal<<(uPos*8);
			if(*pSourceIp=='.')pSourceIp++;
			}
		}


	if(pTargetIp)
		{
		for(uPos=0;uPos<4;uPos++)
			{
			uVal		= atoi(pTargetIp)&0xFF;
			pTargetIp  += strwlen(pTargetIp,0x01);
			uTargetIp  |= uVal<<(uPos*8);
			if(*pTargetIp=='.')pTargetIp++;
			}
		}


return AddFilter(usSourcePort,uSourceIp,usTargetPort,uTargetIp);
}

//*****************************************************************************
//*
//*		SetFinPacket
//*
//*****************************************************************************
//	Stellt das Paket ein das bei FIN-Flags eingefgt wird
//	uSize	: Gre des Pakets
//	pData	: Zeiger auf Paket
void CTcpStreams::SetFinPacket(unsigned uSize,const void *pData)
{

	if(uSize>sizeof(m_aFinData))
		{
		uSize = 0;
		}

	m_uFinSize = uSize;
	memcpy(m_aFinData,pData,uSize);

}

//*****************************************************************************
//*
//*		SetSynPacket
//*
//*****************************************************************************
//	Stellt das Paket ein das bei SYN-Flags eingefgt wird
//	uSize	: Gre des Pakets
//	pData	: Zeiger auf Paket
void CTcpStreams::SetSynPacket(unsigned uSize,const void *pData)
{

	if(uSize>sizeof(m_aSynData))
		{
		uSize = 0;
		}

	m_uSynSize = uSize;
	memcpy(m_aSynData,pData,uSize);

}


//*****************************************************************************
//*
//*		SetRstPacket
//*
//*****************************************************************************
//	Stellt das Paket ein das bei RST-Flags eingefgt wird
//	uSize	: Gre des Pakets
//	pData	: Zeiger auf Paket
void CTcpStreams::SetRstPacket(unsigned uSize,const void *pData)
{

	if(uSize>sizeof(m_aRstData))
		{
		uSize = 0;
		}

	m_uRstSize = uSize;
	memcpy(m_aRstData,pData,uSize);

}


//*****************************************************************************
//*
//*		SetErrPacket
//*
//*****************************************************************************
//	Stellt das Paket ein das bei Stream-Abbrchen eingefgt werden soll
//	uSize	: Gre des Pakets
//	pData	: Zeiger auf Paket
void CTcpStreams::SetErrPacket(unsigned uSize,const void *pData)
{

	if(uSize>sizeof(m_aErrData))
		{
		uSize = 0;
		}

	m_uErrSize = uSize;
	memcpy(m_aErrData,pData,uSize);

}


//*****************************************************************************
//*
//*		FindFreeStream
//*
//*****************************************************************************
//	Sucht den nchsten freien Stream
//	uTime		: UTC-Zeit des zuletzt empfangenen Paketes
//	uTimeout	: Timeout bei dem aktive Streams ohne Daten benedet werden
//				  (0=offene Streams nie beenden)
unsigned CTcpStreams::FindFreeStream(unsigned uTime,unsigned uTimeout)
{
unsigned		uStream;
unsigned		uClose0;
unsigned		uClose1;
unsigned		uDelta0;
unsigned		uDelta1;
unsigned		uValue;
unsigned		uBest;
unsigned		uDiff;


	if(m_uStreamCnt<CTS_STREAM_COUNT)
		{
		return m_uStreamCnt;
		}

	for(uStream=0;uStream<CTS_STREAM_COUNT;uStream++)
		{
		if(m_aStreamUsed[uStream]==0)
			{
			return uStream;
			}
		}


													// Suche die am besten geschlossenen Streams
	for(uStream=0;uStream<CTS_STREAM_COUNT;uStream++)
		{
		uClose0 = m_aStreams[uStream*2+0].Closed();
		uClose1 = m_aStreams[uStream*2+1].Closed();

		if(uClose0==0 && uClose1==0)continue;

		uDelta0 = uTime-m_aStreams[uStream*2+0].m_uLastTime;
		uDelta1 = uTime-m_aStreams[uStream*2+1].m_uLastTime;

		if(uDelta0>10 && uDelta1>10){uClose0=2;uClose1=2;}
		if(uClose0==3 && uClose1==2)uClose1=3;
		if(uClose0==2 && uClose1==3)uClose0=3;

		if(uClose0 && uClose1==0 && uDelta1>10)uClose1=1;

		if(uClose0>=2 && uClose1>=2)
			{
			return uStream;
			}
		}


	if(!uTimeout)return CTS_STREAM_COUNT;


	uBest	= CTS_STREAM_COUNT;
	uDiff	= 0;
													// Suche Stream mit schon lange keine Daten
	for(uStream=0;uStream<CTS_STREAM_COUNT;uStream++)
		{
		uDelta0 = uTime-m_aStreams[uStream*2+0].m_uLastTime;
		uDelta1 = uTime-m_aStreams[uStream*2+1].m_uLastTime;

		if(uDelta0<5 && uDelta1>20)uDelta1=5;
		if(uDelta1<5 && uDelta0>20)uDelta0=5;

		   uValue = uDelta0+uDelta1;
		if(uValue>uDiff)
			{
			uDiff	= uValue;
			uBest	= uStream;
			}
		}


	if(uDiff>=uTimeout*2)
		{
		return uBest;
		}


return CTS_STREAM_COUNT;
}

//*****************************************************************************
//*
//*		InitStream
//*
//*****************************************************************************
//	Initialisiert einen neuen Stream
//	uStream			: Ist die Sream-Nummer
//	pIpHeader		: Zeiger auf IP-Header
//	pTcpHeader		: Zeiger auf TCP-Header
//	uFilter			: Ist  der Filter der angesprochen hat
//	Ergibt 0 wenn der Stream nicht initialisiert wurde oder
//						1 = wenn Quell-IP gleich Server-IP
//						2 = wenn Quell-IP gleich Client-IP
unsigned CTcpStreams::InitStream(unsigned uStream,IpHeader *pIpHeader,TcpHeader *pTcpHeader,unsigned uFilter)
{
unsigned	uPortsX;
unsigned	uRet;


	if(uStream>=CTS_STREAM_COUNT)					// Prfe die Stream-Nummer
		{
		return 0;
		}

	if(m_uStreamCnt<=uStream)
		{
		m_uStreamCnt = uStream+1;
		}


	uPortsX  = pTcpHeader->uTcpPorts>>16;
	uPortsX |= pTcpHeader->uTcpPorts<<16;

													// Richtung via Filter bestimmen
	if(uFilter>=m_uFilterCnt || (unsigned short)pTcpHeader->uTcpPorts==m_aSourcePort[uFilter])
		{
		m_aSourceUsed[uStream]	= pIpHeader->uIpSource;
		m_aTargetUsed[uStream]	= pIpHeader->uIpTarget;
		m_aPortsUsed [uStream]	= pTcpHeader->uTcpPorts;
		m_aPortsUsedX[uStream]	= uPortsX;
		uRet					= 1;
		}
	else{
		m_aTargetUsed[uStream]	= pIpHeader->uIpSource;
		m_aSourceUsed[uStream]	= pIpHeader->uIpTarget;
		m_aPortsUsedX[uStream]	= pTcpHeader->uTcpPorts;
		m_aPortsUsed [uStream]	= uPortsX;
		uRet					= 2;
		}

	m_uStreamNum++;									// Stream initialisieren
	m_aStreamUsed[uStream  ]	= 1;
	m_aStreamNum [uStream  ]	= m_uStreamNum;

	m_aStreams[uStream*2+0].Reset();
	m_aStreams[uStream*2+1].Reset();
	

return uRet;
}

//*****************************************************************************
//*
//*		Put
//*
//*****************************************************************************
//	Fgt ein Packet in die Streams ein
//	pDataPtr	: Zeiger auf das Packet
//	iSize		: Gre des Packetes
//	pInfo		: Zeiger auf diverse Infos
//					[0] UTC-Zeit wannn die Daten empfangen wurden
//					[1] s wannn die Daten empfangen wurden
//					[2] Paket-ID
//					[3] TCP-Ports (big endian,0..1=Source 2..3=Target)
//	Ergibt die Stream-Nummer (Bit0=Sub-Stream) in dem das Paket eingefgt
//	wurde, bzw. -1 wenn das Paket nicht eingefgt wurde.
int CTcpStreams::Put(const void *pDataPtr,int iSize,unsigned *pInfo)
{
unsigned char  *pData=(unsigned char*)pDataPtr;
TcpHeader	   *pTcpHeader;
IpV6Header	   *pV6Header;
IpHeader	   *pIpHeader;
IpHeader		sDummy;
unsigned short	uPortSr;
unsigned short	uPortTg;
unsigned		uIpSize;
unsigned		uFilter;
unsigned		uOffset;
unsigned		uStream;
unsigned		uSubBit;
int				iLength;
int				iClose;
int				iVal;
int				iRet;



	if(iSize<54)return -1;

	pData += 12;
	iSize -= 12;

	if(PTR_WORD(pData)==GET_WORD(TYPE_VLAN))		// VLAN-Tag berspringen
		{
		pData += 4;
		iSize -= 4;
		}

	if(PTR_WORD(pData)==GET_WORD(TYPE_IP))			// Ist es ein IP-V4-Paket
		{
		pData += 2;
		iSize -= 2;

		pIpHeader	= (IpHeader*)pData;

		if(pIpHeader->ucProtocol!=TYPE_TCP)			// Ist es ein TCP Paket
			{
			return -1;
			}

		uIpSize		= (pIpHeader->ucVersionLength&0x0F)*4;
		uOffset	    =  uIpSize;
		iLength		=  GET_WORD(pIpHeader->usTotalLength);
		}
	else if(PTR_WORD(pData)==GET_WORD(TYPE_IPV6))	// Ist es ein IP-V6-Paket
		{
		pData += 2;
		iSize -= 2;

		pV6Header	= (IpV6Header*)pData;

		if(pV6Header->ucProtocol!=TYPE_TCP)			// Ist es ein TCP Paket
			{
			return -1;
			}

		uOffset	    =  40;
		iLength		=  GET_WORD(pV6Header->usLength)+40;
		pIpHeader	= &sDummy;

		sDummy.ucVersionLength	= 0x4A;				// Dummy-Header erstellen
		sDummy.ucProtocol		= TYPE_TCP;
		sDummy.usTotalLength	= GET_WORD(iLength);
		sDummy.uIpSource		= IpV4ToIpV6(pV6Header->aIpSource);
		sDummy.uIpTarget		= IpV4ToIpV6(pV6Header->aIpTarget);
		}
	else{
		return -1;
		}


	if(!iLength)									// Vielleicht TCP-Segment-Offloading
		{
		iLength = iSize;
		}

	pTcpHeader	= (TcpHeader*)(pData+uOffset);
	uOffset	   += (pTcpHeader->usFlagsOffset&0xF0)/4;
	iLength	   -=  uOffset;
	pData	   +=  uOffset;
	iSize	   -=  uOffset;

	if(iLength<0 || iSize<iLength)					// Prfe die Gre
		{
		return -1;
		}

	uPortSr = (unsigned short)(pTcpHeader->uTcpPorts);
	uPortTg = (unsigned short)(pTcpHeader->uTcpPorts>>16);

	for(uFilter=0;uFilter<m_uFilterCnt;uFilter++)	// Filter prfen
		{
		if(!(m_aSourcePort[uFilter]==uPortSr && (m_aTargetPort[uFilter]==0 || m_aTargetPort[uFilter]==uPortTg)) &&
		   !(m_aSourcePort[uFilter]==uPortTg && (m_aTargetPort[uFilter]==0 || m_aTargetPort[uFilter]==uPortSr)))
			{
			continue;								// Kein Port passt
			}

		if(m_aSourceIp[uFilter]==0)
			{
			if(m_aTargetIp[uFilter]==0)break;
			if(m_aTargetIp[uFilter]==pIpHeader->uIpSource)break;
			if(m_aTargetIp[uFilter]==pIpHeader->uIpTarget)break;
			}
		else if(m_aSourceIp[uFilter]==pIpHeader->uIpSource)
			{
			if(m_aTargetIp[uFilter]==0)break;
			if(m_aTargetIp[uFilter]==pIpHeader->uIpTarget)break;
			}
		else if(m_aSourceIp[uFilter]==pIpHeader->uIpTarget)
			{
			if(m_aTargetIp[uFilter]==0)break;
			if(m_aTargetIp[uFilter]==pIpHeader->uIpSource)break;
			}
		}

	if(uFilter>=m_uFilterCnt && m_uFilterCnt)		// Falscher Filter
		{
		return -1;
		}


	for(uStream=0;uStream<m_uStreamCnt;uStream++)	// Existiert ein passender Stream
		{
		if(!m_aStreamUsed[uStream])continue;

		if(m_aStreams[uStream].m_uClosed && m_uSynSize)
		if(m_aStreams[uStream].m_uDataAll==m_uSynSize)
		if(m_aStreams[uStream].m_uCount==1)
			{
			continue;
			}

		if(!m_aSourceUsed[uStream] || m_aSourceUsed[uStream]==pIpHeader->uIpSource)
		if(!m_aTargetUsed[uStream] || m_aTargetUsed[uStream]==pIpHeader->uIpTarget)
		if(!m_aPortsUsed [uStream] || m_aPortsUsed [uStream]==pTcpHeader->uTcpPorts)
			{
			uSubBit = 0;
			break;
			}

		if(!m_aSourceUsed[uStream] || m_aSourceUsed[uStream]==pIpHeader->uIpTarget)
		if(!m_aTargetUsed[uStream] || m_aTargetUsed[uStream]==pIpHeader->uIpSource)
		if(!m_aPortsUsed [uStream] || m_aPortsUsedX[uStream]==pTcpHeader->uTcpPorts)
			{
			uSubBit = 1;
			break;
			}
		}

	if(uStream<m_uStreamCnt)						// Einen passenden Stream gefunden
		{
		uStream = uSubBit+uStream*2;
		iRet	= -1;

		if(iLength==1)								// Ist es ein Keep-Alive Paket ?
		if((pTcpHeader->usFlagsOffset&0x1000) && !(pTcpHeader->usFlagsOffset&0x0800))
			{
			   uOffset = GET_DWORD(pTcpHeader->uSeqNumber)+1;
			if(uOffset==m_aStreams[uStream].m_uOffset)
				{
//				m_aStreams[uStream].Ignore(iLength,uOffset);
				iLength = 0;
				}
			}

		if(iLength)									// Daten vorhanden
			{
			uOffset = GET_DWORD(pTcpHeader->uSeqNumber);
			iVal    = m_aStreams[uStream].Put(pData,iLength,uOffset,pTcpHeader->uTcpPorts,pInfo);

			if(iVal>0)
				{
				iRet = uStream;
				}
			else if(iVal<0)							// Zuviele Offset Fehler
				{
				if(m_uErrSize)
					{
					m_aStreams[uStream].Put(m_aErrData,m_uErrSize,m_aStreams[uStream].m_uOffset,pTcpHeader->uTcpPorts,pInfo);
					}

				m_aStreams[uStream].Close();
				}
			}

		if(pTcpHeader->usFlagsOffset&0x0100)		// FIN-Flag vorhanden
		if(m_aStreams[uStream].Closed()==0)
			{
			if(m_uFinSize)
				{
				m_aStreams[uStream].Put(m_aFinData,m_uFinSize,m_aStreams[uStream].m_uOffset,pTcpHeader->uTcpPorts,pInfo);
				}

			m_aStreams[uStream].Close();

			return uStream;
			}

		if(pTcpHeader->usFlagsOffset&0x0400)		// RST-Flag vorhanden
		if(m_aStreams[uStream].Closed()==0)
			{
			if(m_uRstSize)
				{
				m_aStreams[uStream].Put(m_aRstData,m_uRstSize,m_aStreams[uStream].m_uOffset,pTcpHeader->uTcpPorts,pInfo);
				}

			m_aStreams[uStream].Close();

			return uStream;
			}

		if(pTcpHeader->usFlagsOffset &0x0200)		// SYN-Flag vorhanden
			{
			iClose = m_aStreams[uStream].Closed();

			if(iClose==0)
				{
				if(m_aStreams[uStream].m_uCount  ==0)
				if(m_aStreams[uStream].m_uDataAll==0)
					{
					m_aStreams[uStream].m_uOffset = GET_DWORD(pTcpHeader->uSeqNumber)+1;
					m_aStreams[uStream].m_uCount  = 1;
					}
				}									// Zweiter Versuch eines SYN bei Retransmitt
			else if(iClose==2 && m_aStreams[uStream].m_uCount<=1)
				{
				if(m_aStreams[uStream].m_uDataAll && m_aStreams[uStream].m_uDataAll==m_uRstSize)
					{
					m_aStreams[uStream].Skip(m_uRstSize);
					m_aStreams[uStream].m_uDataAll = 0;
					}

				if(m_aStreams[uStream].m_uDataAll==0)
					{
					m_aStreams[uStream].m_uOffset = GET_DWORD(pTcpHeader->uSeqNumber)+1;
					m_aStreams[uStream].m_uCount  = 1;
					m_aStreams[uStream].m_uClosed = 0;
					}
				}
			}


		return iRet;
		}


	if(pTcpHeader->usFlagsOffset&0x0200)			// SYN-Flag vorhanden
		{
		if(m_uSynSize)
			{
			uStream = FindFreeStream(pInfo[0],30);
			uSubBit = InitStream(uStream,pIpHeader,pTcpHeader,uFilter);

			if(uSubBit)
				{
				uOffset  = GET_DWORD(pTcpHeader->uSeqNumber);
				uOffset -= m_uSynSize-1;
				uStream	 = uStream*2+uSubBit-1;

				if(m_aStreams[uStream].Put(m_aSynData,m_uSynSize,uOffset,pTcpHeader->uTcpPorts,pInfo))
					{
					return uStream;
					}
				}
			}
		}											// Daten vorhanden
	else if((pTcpHeader->usFlagsOffset&0x0800) || iLength>1)
		{
		uStream = FindFreeStream(pInfo[0],60);
		uSubBit = InitStream(uStream,pIpHeader,pTcpHeader,uFilter);

		if(uSubBit)
			{
			uOffset  = GET_DWORD(pTcpHeader->uSeqNumber);
			uStream	 = uStream*2+uSubBit-1;

			if(m_aStreams[uStream].Put(pData,iLength,uOffset,pTcpHeader->uTcpPorts,pInfo))
				{
				return uStream;
				}
			}
		}



return -1;
}


//*****************************************************************************
//*
//*		IsTcpPaket
//*
//*****************************************************************************
//	Prft ob ein Paket ein TCP-Packet ist
//	pDataPtr	: Zeiger auf das Packet
//	iSize		: Gre des Packetes
//	pInfo		: Hier werden dieverse Infos gespeichert wenn nicht 0.
//					[0] Source-IP
//					[1] Target-IP
//					[2] Source-Target-Port
//					[3] Datenlnge
//	Ergibt die den Offset der Daten, bzw. -1 wenn das Paket kein TCP-Paket ist.
int CTcpStreams::IsTcpPaket(const void *pDataPtr,int iSize,unsigned *pInfo)
{
unsigned char  *pData=(unsigned char*)pDataPtr;
TcpHeader	   *pTcpHeader;
IpV6Header	   *pV6Header;
IpHeader	   *pIpHeader;
IpHeader		sDummy;
unsigned short	uPortSr;
unsigned short	uPortTg;
unsigned		uIpSize;
unsigned		uOffset;
int				iLength;
int				iStart;



	if(iSize<54)return -1;

	iStart  = 14;
	pData  += 12;
	iSize  -= 12;

	if(PTR_WORD(pData)==GET_WORD(TYPE_VLAN))		// VLAN-Tag berspringen
		{
		iStart	+= 4;
		pData   += 4;
		iSize   -= 4;
		}

	if(PTR_WORD(pData)==GET_WORD(TYPE_IP))			// Ist es ein IP-Paket
		{
		pData   += 2;
		iSize   -= 2;

		pIpHeader	= (IpHeader*)pData;

		if(pIpHeader->ucProtocol!=TYPE_TCP)			// Ist es ein TCP Paket
			{
			return -1;
			}

		uIpSize		= (pIpHeader->ucVersionLength&0x0F)*4;
		uOffset	    =  uIpSize;
		iLength		=  GET_WORD(pIpHeader->usTotalLength);
		}
	else if(PTR_WORD(pData)==GET_WORD(TYPE_IPV6))	// Ist es ein IP-V6-Paket
		{
		pData += 2;
		iSize -= 2;

		pV6Header	= (IpV6Header*)pData;

		if(pV6Header->ucProtocol!=TYPE_TCP)			// Ist es ein TCP Paket
			{
			return -1;
			}

		uOffset	    =  40;
		iLength		=  GET_WORD(pV6Header->usLength)+40;
		pIpHeader	= &sDummy;

		sDummy.ucVersionLength	= 0x4A;				// Dummy-Header erstellen
		sDummy.ucProtocol		= TYPE_TCP;
		sDummy.usTotalLength	= GET_WORD(iLength);
		sDummy.uIpSource		= IpV4ToIpV6(pV6Header->aIpSource);
		sDummy.uIpTarget		= IpV4ToIpV6(pV6Header->aIpTarget);
		}
	else{
		return -1;
		}


	if(!iLength)									// Vielleicht TCP-Segment-Offloading
		{
		iLength = iSize;
		}

	pTcpHeader	= (TcpHeader*)(pData+uOffset);
	uOffset	   += (pTcpHeader->usFlagsOffset&0xF0)/4;
	iLength	   -=  uOffset;
	pData	   +=  uOffset;
	iSize	   -=  uOffset;

	if(iLength<0 || iSize<iLength)					// Prfe die Gre
		{
		return -1;
		}

	if(pInfo)										// Infos speichern
		{
		uPortSr  = GET_WORD(pTcpHeader->uTcpPorts);
		uPortTg  = GET_WORD(pTcpHeader->uTcpPorts>>16);
		pInfo[0] = pIpHeader->uIpSource;
		pInfo[1] = pIpHeader->uIpTarget;
		pInfo[2] = uPortSr|(uPortTg<<16);
		pInfo[3] = iLength;
		}


return uOffset+iStart;
}

//*****************************************************************************
//*
//*		IpV4ToIpV6
//*
//*****************************************************************************
//	Eine IP-V6-Adresse in ein Pseudo IP-V4-Adresse umwandeln
unsigned CTcpStreams::IpV4ToIpV6(const void *pIpV6)
{
unsigned   *pAddr=(unsigned*)pIpV6;


	if(pAddr[0]>=0xFF000000)						// Multicast "FF??:..."
		{
		return 0xFFFFFFFF;
		}

	if(pAddr[3]==1 && !pAddr[0])					// Loopback "::1"
		{
		return 0x7F000001;
		}


return pAddr[3]|0xF0000000;							// Pseudo IP-V4
}
