//*****************************************************************************
//*
//*
//*		TcpStream.h
//*
//*
//*****************************************************************************
//
//	Klasse um einzelne TCP-Streams aus Ethernet-Paketen zusammen zu bauen:
//	
//		for(;;)
//			{
//			unsigned char  *pData;
//			CTcpStreams		sStream;
//			int				iIndex;	
//			int				iSize;
//			
//				pData = WiresharkRead(pHandle);				// Paket laden
//			if(!pData)break;
//															// Paket einfgen
//			   iIndex=sStream.Put(pData,pHandle->uSize,&pHandle->uTime);
//			if(iIndex>=0)	
//				{
//					pData	= sStream.Data(iIndex,iSize);	// Daten aus holen
//				if(!pData || iSize<=0)break;
//		
//					iSkip = AnalyseData(iSize,pData);		// Daten aus Stream auswerten
//				if(!iSkip)continue;
//		
//				printf("Packet ID     = %i\n",sStream.Label          (iIndex,0));
//				printf("Packet Port1  = %i\n",sStream.GetSourcePort  (iIndex/2));
//				printf("Packet Port1  = %i\n",sStream.GetTargetPort  (iIndex/2));
//				printf("Packet Stream = %i\n",sStream.GetStreamNumber(iIndex/2));
//		
//				sStream.Skip(iIndex,iSkip);					// Daten aus Stream lschen
//				}
//
#ifndef 	__TCPSTREAM_H__
#define 	__TCPSTREAM_H__


#include	"DataField.h"
#include	"BitFunctions.h"


#ifndef 	CTS_STREAM_COUNT
#define 	CTS_STREAM_COUNT	8						// Maximale Anzahl der gleiczeitigen Streams
#endif 			

#ifndef 	CTS_FILTER_COUNT
#define 	CTS_FILTER_COUNT	8						// Maximale Anzahl der Filter
#endif 			

#ifndef 	CTS_FORWARD_COUNT
#define 	CTS_FORWARD_COUNT	4						// Wie gro ist der Zukunftspuffer
#endif 			

#if	 		CTS_FORWARD_COUNT <=0
#define 	CTS_FORWARD_ARRAY	1
#else 	
#define 	CTS_FORWARD_ARRAY	CTS_FORWARD_COUNT
#endif 	

class		CTcpStreams;

typedef struct
	{
	unsigned 		uTimeSec;							// Empfangszeit des Paketes
	unsigned 		uTimeUsec;							//		*
	unsigned 		uPacketId;							// User-ID des Paketes (z.B.: Wireshark-Index)
	unsigned short	uTargetPort;						// TCP-Target Port
	unsigned short	uSourcePort;						// TCP-Source Port
	}StreamInfo;

typedef struct
	{
	unsigned 		uLenData;							// Lnge der Daten	
	unsigned 		uTimeSec;							// Empfangszeit des Paketes
	unsigned 		uTimeUsec;							//		*
	unsigned 		uPacketId;							// User-ID des Paketes (z.B.: Wireshark-Index)
	unsigned short	uTargetPort;						// TCP-Target Port
	unsigned short	uSourcePort;						// TCP-Source Port
	unsigned char	aData[4];							// Daten des Paktetes
	}StreamEntry;

typedef struct
	{
	unsigned		uOffset;							// Offset des Paketes im Stream
	unsigned 		uLenData;							// Lnge der Daten	
	unsigned 		uTimeSec;							// Empfangszeit des Paketes
	unsigned 		uTimeUsec;							//		*
	unsigned 		uPacketId;							// User-ID des Paketes (z.B.: Wireshark-Index)
	unsigned 		uTcpPorts;							// TCP-Target/Source Ports (big endian)
	unsigned char	aData[4];							// Daten des Paktetes
	}StreamForward;

//*****************************************************************************
//*
//*		Klasse CTcpStream
//*
//*****************************************************************************
class CTcpStream
{
public:

	CTcpStream(unsigned uDataSize=8192);
   ~CTcpStream();
   
	int				Put(const void *pData,int iSize,unsigned uOffset,unsigned uTcpPorts,unsigned *pInfo);
	int				Ignore(int iSize,unsigned uOffset);
	int				Skip(int iSize);
	void 			Close(unsigned uFinish=0);
	void 			Reset(unsigned uNewSize=0);
	unsigned		Done   ()const;
	unsigned		Closed ()const;
	unsigned		Offset ()const;
	unsigned		Packets()const;
	unsigned		Forward()const;
	unsigned		Label(unsigned  uByte)const;
	unsigned		Time (unsigned  uByte,void     *pUsec=0)const;
	unsigned		Time (unsigned  uByte,unsigned &uUsec  )const;
	unsigned		Time (unsigned  uByte,int      &iUsec  )const;
	unsigned		Ports(unsigned  uByte,unsigned short *pTarget=0)const;
	StreamInfo	   *Info (unsigned  uByte,unsigned &uSize)const;
	StreamInfo	   *Info (unsigned  uByte,int      &iSize)const;
	StreamInfo	   *Info (unsigned  uByte)const;
	unsigned char  *Data (unsigned &uSize)const;
	unsigned char  *Data (int      &iSize)const;
	unsigned char  *Data ()const;

protected:

	int				PutForward(const void *pData,int iSize,unsigned uOffset,unsigned uTcpPorts,unsigned *pInfo);

	unsigned char  *m_pData;							// Zeiger auf internen Datenpuffer
	unsigned		m_uDataRead;						// Offset im Datenpuffer
	unsigned		m_uDataSize;						// Gres des Datenpuffers
	unsigned		m_uDataUsed;						// Benutzte Bytes im Datenpuffers
	unsigned		m_uDataAll;							// Alle Bytes in allen Datenpuffern
	unsigned		m_uClosed;							// Ist der Stream geschlossen (0=nein 1=ja 2=alles ausgelesen 3=fertig)
	unsigned		m_uCopyPos;							// Zuletzt eingefgter Datenpuffer
	unsigned		m_uOffset;							// Aktuelles Offset im TCP-Stream	
	unsigned		m_uCount;							// Anzahl der akzeptierten TCP-Pakete	
	unsigned		m_uDone;							// Ingesamt gelesenen Bytes
	unsigned		m_uErrorCnt;						// Anzahl der Offset-Fehler
	unsigned		m_uLastTime;						// Zeit der letzten Eingabe
	unsigned		m_uForwardCnt;						// Anzahl der Pakete in die 
	StreamForward  *m_aForwardPtr[CTS_FORWARD_COUNT];	// Zeiger auf Pakete in die Zukunft	
	CDataPointer	m_sDataList;						// Puffer mit Paketdaten


	friend	CTcpStreams;

};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline StreamInfo	 *CTcpStream::Info(unsigned  uByte,int &iSize)const{return Info(uByte,*(unsigned*)&iSize);}
inline unsigned char *CTcpStream::Data(unsigned &uSize)const{uSize=m_uDataUsed-m_uDataRead;return m_pData+m_uDataRead;}
inline unsigned char *CTcpStream::Data(int      &iSize)const{iSize=m_uDataUsed-m_uDataRead;return m_pData+m_uDataRead;}
inline unsigned char *CTcpStream::Data(               )const{                              return m_pData+m_uDataRead;}
inline unsigned		  CTcpStream::Time(unsigned uByte,unsigned &uUsec)const{return Time(uByte,&uUsec);}
inline unsigned		  CTcpStream::Time(unsigned uByte,int      &iUsec)const{return Time(uByte,&iUsec);}
inline unsigned       CTcpStream::Packets()const{return m_sDataList.Count();}
inline unsigned       CTcpStream::Forward()const{return m_uForwardCnt;}
inline unsigned		  CTcpStream::Closed ()const{return m_uClosed;}
inline unsigned		  CTcpStream::Offset ()const{return m_uOffset;}
inline unsigned		  CTcpStream::Done   ()const{return m_uDone;}



//*****************************************************************************
//*
//*		Klasse CTcpStreams
//*
//*****************************************************************************
class CTcpStreams
{
public:

	CTcpStreams();
   ~CTcpStreams();
	
	int				AddFilter(unsigned short usSourcePort,unsigned    uSourceIp=0,unsigned short usTargetPort=0,unsigned    uTargetIp=0);
	int				AddFilter(unsigned short usSourcePort,const char *pSourceIp  ,unsigned short usTargetPort=0,const char *pTargetIp=0);
	void 			ClearFilters();
	int				Put(const void *pData,int iSize,unsigned *pInfo);
	void 			Close  (unsigned uFinish=0);
	unsigned		Closed (unsigned uStream) const;
	unsigned		Done   (unsigned uStream) const;
	unsigned		Offset (unsigned uStream) const;
	unsigned		Packets(unsigned uStream) const;
	unsigned		Forward(unsigned uStream) const;
	unsigned		Label(unsigned uStream,unsigned  uByte)const;
	unsigned		Time (unsigned uStream,unsigned  uByte,void     *pUsec=0)const;
	unsigned		Time (unsigned uStream,unsigned  uByte,unsigned &uUsec)const;
	unsigned		Time (unsigned uStream,unsigned  uByte,int      &iUsec)const;
	unsigned short  Ports(unsigned uStream,unsigned  uByte,unsigned short *pTarget=0)const;
	StreamInfo	   *Info (unsigned uStream,unsigned  uByte,unsigned &uSize)const;
	StreamInfo	   *Info (unsigned uStream,unsigned  uByte)const;
	unsigned char  *Data (unsigned uStream,unsigned &uSize)const;
	unsigned char  *Data (unsigned uStream,int      &iSize)const;
	unsigned char  *Data (unsigned uStream) const;
	unsigned		GetSourceIp    (unsigned uStream);
	unsigned		GetTargetIp    (unsigned uStream);
	unsigned short 	GetSourcePort  (unsigned uStream);
	unsigned short 	GetTargetPort  (unsigned uStream);
	unsigned		GetStreamNumber(unsigned uStream);	// Fortlaufende Stream-Nummer
	int				Skip(unsigned uStream,int iSize);
	void 			Close(unsigned uStream,unsigned uFinish=0);
	void 			Reset(unsigned uNewSize=0);
	void			SetFinPacket(unsigned uSize,const void *pData);
	void			SetSynPacket(unsigned uSize,const void *pData);
	void			SetRstPacket(unsigned uSize,const void *pData);
	void			SetErrPacket(unsigned uSize,const void *pData);
	static int		IsTcpPaket(const void *pDataPtr,int iSize,unsigned *pInfo=0);
	static unsigned IpV4ToIpV6(const void *pIpV6);		

protected:

	typedef struct
		{
		unsigned char		ucVersionLength;			// 4Lh Version/Lnge (Lnge=L*4)
		unsigned char		ucTypeOfService;			// Typ vom Service
		unsigned short		usTotalLength;				// Lnge IP-Header plus Daten
		unsigned short		usFragmenId;				// ID wird immer um eins erhht beim Senden
		unsigned short		usFragmenOffset;			// FOOOh Flags und Offset (auf 0 setzen)
		unsigned char		ucTimeToLive;				// TTL Zeit fr Paket
		unsigned char		ucProtocol;					// 1=ICMP 6=TCP 17=UDP 
		unsigned short		usCrcSum;					// CRC fr den IP-Header
		unsigned int 		uIpSource;					// IP-Adresse vom Ziel
		unsigned int 		uIpTarget;					// IP-Adresse vom Quelle
		}IpHeader;										
														
	typedef struct
		{
		unsigned char		ucVersionClass;				// Version (=6) und 4-Bit der Klasse
		unsigned char		ucClassLevel;				// 4-Bit der Klasse und 4 Bit Level
		unsigned short		usFlowLevel;				// Rest vom Flow-Level
		unsigned short		usLength;					// Lnge der nach folgenten Daten
		unsigned char		ucProtocol;					// 1=ICMP 6=TCP 17=UDP 
		unsigned char 		ucHopLimith;				// Hop-Grenze
		unsigned short 		aIpSource[8];				// IP-V6-Adresse vom Ziel
		unsigned short 		aIpTarget[8];				// IP-V6-Adresse vom Quelle
		}IpV6Header;										

	typedef struct										
		{												
		unsigned			uTcpPorts;					// UDP-Ports
		unsigned int 		uSeqNumber;					// Sequenznummer
		unsigned int 		uAckNumber;					// Acknowledge-Nummer
		unsigned short		usFlagsOffset;				// Data-Offset und Flags
		unsigned short		usWindow;					// Gre des Empfangsfenster
		unsigned short		usTcpCrc;					// Checksumme
		unsigned short		usUrgentPtr;				// Urgent-Pointer
		}TcpHeader;


	unsigned		FindFreeStream(unsigned uTime,unsigned uTimeout=60);
	unsigned		InitStream(unsigned uStream,IpHeader *pIpHeader,TcpHeader *pTcpHeader,unsigned uFilter=0);

	unsigned		m_uStreamNum;						// Fortlaufende Streamnummer 
	unsigned char	m_uFinSize;							// Gre fr TAG fr Schlieen
	unsigned char	m_uSynSize;							// Gre fr TAG fr ffnen
	unsigned char	m_uRstSize;							// Gre fr TAG fr Ablehnen
	unsigned char	m_uErrSize;							// Gre fr TAG fr Ablehnen
	unsigned char	m_aFinData[256];					// Einzufgendes TAG fr Schlieen
	unsigned char	m_aSynData[256];					// Einzufgendes TAG fr ffnen
	unsigned char	m_aRstData[256];					// Einzufgendes TAG fr Ablehnen
	unsigned char	m_aErrData[256];					// Einzufgendes TAG fr Stream-Abbrche
														
	unsigned		m_uFilterCnt;						// Anzahl aller Filter	
	unsigned		m_aSourceIp  [CTS_FILTER_COUNT];	// Filter IP-Quelle   (0=alles)
	unsigned		m_aTargetIp  [CTS_FILTER_COUNT];	// Filter IP-Ziel	  (0=alles)
	unsigned short 	m_aSourcePort[CTS_FILTER_COUNT];	// Filter Port-Quelle (0=alles)
	unsigned short 	m_aTargetPort[CTS_FILTER_COUNT];	// Filter Port-Ziel	  (0=alles)
														
	unsigned		m_uStreamCnt;						// Anzahl der benutzen Streams
	unsigned		m_aSourceUsed[CTS_STREAM_COUNT];	// Benutzte IP-Quelle 
	unsigned		m_aTargetUsed[CTS_STREAM_COUNT];	// Benutzte IP-Ziel
	unsigned		m_aPortsUsed [CTS_STREAM_COUNT];	// Benutzte Ports
	unsigned		m_aPortsUsedX[CTS_STREAM_COUNT];	// Benutzte Ports (gedreht)
	unsigned char 	m_aStreamUsed[CTS_STREAM_COUNT];	// Ist der Stream benutzt
	unsigned	 	m_aStreamNum [CTS_STREAM_COUNT];	// Fortlaufende Streamnummer 
	CTcpStream		m_aStreams [2*CTS_STREAM_COUNT];	// Puffer fr Streams
	
};
																					 
inline unsigned char *CTcpStreams::Data   (unsigned uStream,unsigned &uSize                )const{if(uStream>=CTS_STREAM_COUNT*2){       uSize=0;return 0;}return m_aStreams[uStream].Data   (uSize      );}
inline unsigned char *CTcpStreams::Data   (unsigned uStream,int      &iSize                )const{if(uStream>=CTS_STREAM_COUNT*2){       iSize=0;return 0;}return m_aStreams[uStream].Data   (iSize      );}
inline unsigned char *CTcpStreams::Data   (unsigned uStream                                )const{if(uStream>=CTS_STREAM_COUNT*2){               return 0;}return m_aStreams[uStream].Data   (           );}
inline unsigned		  CTcpStreams::Time   (unsigned uStream,unsigned uByte,unsigned  &uUsec)const{if(uStream>=CTS_STREAM_COUNT*2){       uUsec=0;return 0;}return m_aStreams[uStream].Time   (uByte,uUsec);}
inline unsigned		  CTcpStreams::Time   (unsigned uStream,unsigned uByte,int       &iUsec)const{if(uStream>=CTS_STREAM_COUNT*2){       iUsec=0;return 0;}return m_aStreams[uStream].Time   (uByte,iUsec);}
inline unsigned		  CTcpStreams::Time   (unsigned uStream,unsigned uByte,void      *pUsec)const{if(uStream>=CTS_STREAM_COUNT*2){*(int*)pUsec=0;return 0;}return m_aStreams[uStream].Time   (uByte,pUsec);}
inline unsigned short CTcpStreams::Ports  (unsigned uStream,unsigned uByte,unsigned short*p)const{if(uStream>=CTS_STREAM_COUNT*2){      *p    =0;return 0;}return m_aStreams[uStream].Ports  (uByte,p    );}
inline StreamInfo    *CTcpStreams::Info   (unsigned uStream,unsigned uByte                 )const{if(uStream>=CTS_STREAM_COUNT*2){               return 0;}return m_aStreams[uStream].Info   (uByte      );}
inline StreamInfo    *CTcpStreams::Info   (unsigned uStream,unsigned uByte,unsigned  &uSize)const{if(uStream>=CTS_STREAM_COUNT*2){               return 0;}return m_aStreams[uStream].Info   (uByte,uSize);}
inline unsigned		  CTcpStreams::Done   (unsigned uStream                                )const{if(uStream>=CTS_STREAM_COUNT*2)return 0;                 return m_aStreams[uStream].Done   (           );}
inline unsigned		  CTcpStreams::Closed (unsigned uStream                                )const{if(uStream>=CTS_STREAM_COUNT*2)return 3;                 return m_aStreams[uStream].Closed (           );}
inline unsigned		  CTcpStreams::Offset (unsigned uStream                                )const{if(uStream>=CTS_STREAM_COUNT*2)return 0;                 return m_aStreams[uStream].Offset (           );}
inline unsigned		  CTcpStreams::Packets(unsigned uStream                                )const{if(uStream>=CTS_STREAM_COUNT*2)return 0;                 return m_aStreams[uStream].Packets(           );}
inline unsigned		  CTcpStreams::Forward(unsigned uStream                                )const{if(uStream>=CTS_STREAM_COUNT*2)return 0;                 return m_aStreams[uStream].Forward(           );}
inline int			  CTcpStreams::Skip   (unsigned uStream,int iSize                      )     {if(uStream>=CTS_STREAM_COUNT*2)return 3;                 return m_aStreams[uStream].Skip   (iSize      );}
inline void 		  CTcpStreams::Close  (unsigned uStream,unsigned uFinish               )     {if(uStream>=CTS_STREAM_COUNT*2)return;                          m_aStreams[uStream].Close  (uFinish    );}

inline unsigned		  CTcpStreams::Label  (unsigned uStream,unsigned uByte)const{if(uStream>=CTS_STREAM_COUNT*2)return 0xFFFFFFFF;return m_aStreams[uStream].Label(uByte);}
inline unsigned		  CTcpStreams::GetStreamNumber(unsigned uStream){if(uStream>=CTS_STREAM_COUNT)return 0;return         m_aStreamNum [uStream] ;}
inline unsigned		  CTcpStreams::GetSourceIp    (unsigned uStream){if(uStream>=CTS_STREAM_COUNT)return 0;return btol_dw(m_aSourceUsed[uStream]);}
inline unsigned		  CTcpStreams::GetTargetIp    (unsigned uStream){if(uStream>=CTS_STREAM_COUNT)return 0;return btol_dw(m_aTargetUsed[uStream]);}
inline unsigned short CTcpStreams::GetSourcePort  (unsigned uStream){if(uStream>=CTS_STREAM_COUNT)return 0;return btol_w (m_aPortsUsed [uStream]);}
inline unsigned	short CTcpStreams::GetTargetPort  (unsigned uStream){if(uStream>=CTS_STREAM_COUNT)return 0;return btol_w (m_aPortsUsedX[uStream]);}


#endif 	

