//*****************************************************************************
//*
//*
//*		TextBuffer.cpp
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"TextBuffer.h"
#include	"MemFunctions.h"


#ifdef		zchar
#undef 		zchar
#endif

#ifdef		UNICODE
#define 	Z(a)		L##a
#define 	zchar		wchar_t
#define 	zcharu		short
#else
#define 	Z(a)		a
#define 	zchar		char
#define 	zcharu		unsigned char
#endif


#ifndef		SEEK_SET
#define 	SEEK_SET	0
#endif

#ifndef		SEEK_CUR
#define 	SEEK_CUR	1
#endif

#ifndef		SEEK_END
#define 	SEEK_END	2
#endif


//*****************************************************************************
//*
//*     Konstruktor
//*
//*****************************************************************************
CTextBuffer::CTextBuffer(int iBufferSize):CFileBuffer(iBufferSize)
{
}

CTextBuffer::CTextBuffer(const wchar_t *pName,int iBufferSize):CFileBuffer(pName,iBufferSize)
{
}

CTextBuffer::CTextBuffer(const char    *pName,int iBufferSize):CFileBuffer(pName,iBufferSize)
{
}

//*****************************************************************************
//*
//*     Destruktor
//*
//*****************************************************************************
CTextBuffer::~CTextBuffer()
{

}

//*****************************************************************************
//*
//*		Text
//*
//*****************************************************************************
//	Fllt den Textspeicher
//	iAdd	: Bestimmt den Vorschub in der Datei, um diese Anzahl wird die
//			  Ausleseposition nach vor verschoben.
//	pMem	: Hier wird ein Zeiger auf Lesepuffer abgelegt
//	iMin	: Minimale Anzahl an Zeichen die im Textpuffer sein mssen.
//			  Wenn dieser Wert Null ist wird  die halbe Pufferlnge benutzt.
//	Ergibt die Anzahl der geladenen Zeichen im Puffer, bzw. -1 bei einem Fehler.
int CTextBuffer::Text(int iAdd,zchar **pMem,int iMin)
{
int	iCnt,iNum,iDiff;



	if( iOpenMode)return -1;
	if(!pProcRead)return -1;

	if(iAdd<0)iAdd=0;									// Vorschub
	

	   iBufferPos+=iAdd*=sizeof(zchar);
	if(iBufferPos>=iBufferEnd)
		{
		iBufferPos=iBufferEnd;
		}

	if(iMin>0)											// Minimale Byteanzahl
		{
		   iBufferMin =iMin*sizeof(zchar);
		if(iBufferMin>=iBufferLen/2)
			{
			iBufferMin=iBufferLen/2;
			}
		}

	if(!bMemFlag)
		{
		   iDiff=iBufferEnd-iBufferPos;
		if(iDiff<iBufferMin)							// Puffer aufladen
			{
			memmove(pBuffer,pBuffer+iBufferPos,iDiff);	// Puffer leeren

			iBufferEnd = iDiff;
			iBufferPos = 0;

			iNum = iBufferLen-iBufferEnd;				// Puffer laden
			iCnt = pProcRead(pFileHandle,pBuffer+iBufferEnd,iNum);

			if(iCnt==-1)return -1;

			iBufferEnd          += iCnt;
			pBuffer[iBufferEnd]  = 0;
			#if UNICODE
			pBuffer[iBufferEnd+1]= 0;
			#endif 	

			if(iBufferEnd<=iBufferPos)bEofFlag=true;
			}
		}


	if(pMem)*pMem=(zchar*)(pBuffer+iBufferPos);


return (iBufferEnd-iBufferPos)/sizeof(zchar);
}

//*****************************************************************************
//*
//*		TextLineLen
//*
//*****************************************************************************
//	Zhlt die Zeichen bis zum Zeilenende von der aktuellen Leseposition aus.
//	iMode	: 0=Zeilenumbrche mitzhlen  1=nicht zhlen
//	iMax	: Ist die maximale Position bis zu der gezhlt wird.
//	Ergibt die Lnge der Zeile
int CTextBuffer::TextLineLen(int iMode,int iMax)
{
zchar	*pText;
int		 iPos,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iNum>iMax)iNum=iMax;

	for(iPos=0;iPos<iNum;iPos++)
		{
		if(pText[iPos]=='\r')break;
		if(pText[iPos]=='\n')break;
		}

	if(!iMode)											// Zeilenumbrche mitzhlen
		{
		if(iPos<iNum && pText[iPos]=='\r')iPos++;
		if(iPos<iNum && pText[iPos]=='\n')iPos++;
		}


return iPos;
}

//*****************************************************************************
//*
//*		TextLine
//*
//*****************************************************************************
//	Holt den Zeiger von der nchsten Textzeile.
//	Dabei wird auch der Lesezeiger vorgeschoben.
//	pMem	: Hier wird ein Zeiger auf Lesepuffer abgelegt
//	iMin	: Minimale Anzahl an Zeichen die im Textpuffer sein mssen
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge der Textzeile gespeichert.
//	iAdd	: Bestimmt den Vorschub in der Datei, um diese Anzahl wird die
//			  Ausleseposition nach vor verschoben. Wenn dieser kleiner 0 ist
//			  wird der Vorschub mit TextLineLen() ermittelt.
//	Ergibt die Anzahl der geladenen Zeichen im Puffer, bzw. -1 bei einem Fehler.
int CTextBuffer::TextLine(zchar **pMem,int iMin,int *pCount,int iAdd)
{
int		iRet;				  


	if(iAdd<0)iAdd = TextLineLen();						// Hole Lnge der aktuellen Zeile
	iRet = Text(iAdd,pMem,iMin);
	if(pCount)*pCount=TextLineLen();


return iRet;
}

//*****************************************************************************
//*
//*		TextLine
//*
//*****************************************************************************
//	Liest eine Textzeile ein und speichert sie in einem Puffer.
//	Dabei werden Zeilenumbrche am Ende der Zeile mit kopiert.
//	pMem	: Zeiger auf den Puffer in dem die Textzeile gespeichert wird.
//	iMax	: Ist die Gre des Puffers mit Nullzeichen.
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge der Textzeile gespeichert.
//	iAdd	: Bestimmt den Vorschub in der Datei, um diese Anzahl wird die
//			  Ausleseposition nach vor verschoben. Wenn dieser kleiner 0 ist
//			  wird der Vorschub mit TextLineLen() ermittelt.
//	Ergibt die Anzahl der eingelesenen Zeichen.
int CTextBuffer::TextLine(zchar *pMem,int iMax,int *pCount,int iAdd)
{
int		iLen;

	if(iAdd<0)iAdd=TextLineLen();
	
	Text(iAdd);											// Auslesezeiger verschieben
	
	if(iMax<=0)return 0;

	iLen = TextLineLen(0,iMax-1);

	if(pCount)*pCount=iLen;								
	memcpy(pBuffer+iBufferPos,pMem,iLen*sizeof(zchar));	// Zeile kopieren
	pMem[iLen]=0;



return iLen;
}

//*****************************************************************************
//*
//*		TextString
//*
//*****************************************************************************
//	Liest einen String in zwischen Anfhrungszeichen aus. (  "ABC... " )
//	Leerzeichen vor und nach dem String werden bersprungen.
//	pMem	: Zeiger auf den Puffer in dem der String gespeichert wird.
//	iMax	: Ist die Gre des Puffers mit Nullzeichen.
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge des Strings gespeichert.
//	iCrypt	: Wenn ungleich Null werden  \" Folgen nicht als Ende erkannt.
//	Ergibt die Anzahl der eingelesenen Zeichen
int CTextBuffer::TextString(zchar *pMem,int iMax,int *pCount,int iCrypt)
{
zchar	*pText,cChar;
int		 iNum,iStart;
int		 i,iPos;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iMax>0) pMem[0]=0;
	if(pCount)*pCount =0;

	for(iPos=0;iPos<iNum;iPos++)						// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}

	if(pText[iPos]!='\"')return iPos;					// Kein String gefunden.
	if(iPos+1>=iNum     )return iPos;

	iPos++;iStart=iPos;
	for(;iPos<iNum;iPos++)								// Leerzeichen berspringen
		{
		   cChar=pText[iPos];
		if(cChar== 0  )break;
		if(cChar=='\n')break;
		if(cChar=='\r')break;
		if(iCrypt)
		if(cChar=='\\')
		if(pText[iPos+1]=='\"'){iPos++;continue;}
		if(cChar=='\"')break;
		}

	if(pText[iPos]!='\"')return iPos;					// Kein '"' am Ende


	   i=iPos-iStart;
	if(i   >=iMax)i=iMax-1;
	if(i   >0    )memcpy(pMem,pText+iStart,i*sizeof(zchar));
	if(iMax>0    ) pMem[i]=0;
	if(pCount    )*pCount=i;

	if(  iPos<iNum)iPos++;
	for(;iPos<iNum;iPos++)								// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}


return iPos;
}

//*****************************************************************************
//*
//*		TextString
//*
//*****************************************************************************
//	Liest einen String zwischen zwei Zeichen ein und Speichert ihn in einem Puffer.
//	Leerzeichen vor und nach dem String werden bersprungen.
//	pMem	: Zeiger auf den Puffer in dem der String gespeichert wird.
//	iMax	: Ist die Gre des Puffers mit Nullzeichen.
//	cFrom	: Ist das Zeichen bei dem der String beginnt, (Bei 0 vom Anfang weg)
//	cTo		: Ist das Zeichen bei dem der String endet.   (Bei 0 zum Zeilenende)
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge des Strings gespeichert.
//	Ergibt die Anzahl der eingelesenen Zeichen
int CTextBuffer::TextString(zchar *pMem,int iMax,zchar cFrom,zchar cTo,int *pCount)
{
zchar   *pText,cChar;
int		iNum,iStart;
int		i,iPos;


	iPos  = 0;
	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iMax>0) pMem[0]=0;
	if(pCount)*pCount =0;

	if(cFrom)
		{
		for(;iPos<iNum;iPos++)							// Leerzeichen berspringen
			{
			if(pText[iPos]==' ' )continue;
			if(pText[iPos]=='\t')continue;
			break;
			}

		if(pText[iPos]!=cFrom)return iPos;				// Kein String gefunden.
		if(iPos+1>=iNum)	  return iPos;
		iPos++;
		}

	for(iStart=iPos;iPos<iNum;iPos++)					// String Lnge scannen
		{
		   cChar=pText[iPos];
		if(cChar== 0  )break;
		if(cChar=='\n')break;
		if(cChar=='\r')break;
		if(cChar==cTo )break;
		}

	if(cTo && pText[iPos]!=cTo)return iPos;				// Kein Ende gefunden


	   i    =iPos-iStart;
	if(i   >=iMax)i=iMax-1;
	if(i   >0    )memcpy(pMem,pText+iStart,i*sizeof(zchar));
	if(iMax>0    )pMem[i]=0;
	if(pCount    )*pCount=i;

	if ( iPos<iNum)iPos++;
	for(;iPos<iNum;iPos++)								// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}



return iPos;
}

//*****************************************************************************
//*
//*		TextChar
//*
//*****************************************************************************
//	Liest einen einzelnes Nicht-Text-Zeichen aus. (  ;  oder  * )
//	pMem	: Hier wird das Zeichen gespeichert.
//	Ergibt die Anzahl der eingelesenen Zeichen
int CTextBuffer::TextChar(zchar *pMem)
{
zchar	*pText;
int		 iPos,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	*pMem=0;

	for(iPos=0;iPos<iNum;iPos++)						// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}

	if(pText[iPos]>='a' && pText[iPos]<='z')return iPos;
	if(pText[iPos]>='A' && pText[iPos]<='Z')return iPos;
	if(pText[iPos]>='0' && pText[iPos]<='9')return iPos;
	if(pText[iPos]=='_' || pText[iPos]<=' ')return iPos;
	if(pText[iPos]>=128 || iPos>=iNum      )return iPos;

	*pMem=pText[iPos];

	if(  iPos<iNum)iPos++;
	for(;iPos<iNum;iPos++)								// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}


return iPos;
}

//*****************************************************************************
//*
//*		TextWord
//*
//*****************************************************************************
//	Liest ein Wort aus dem Stream ein. (  ABC  oder  123abc  oder A_B_C_9 oder  )
//	Leerzeichen vor und nach der Zahl werden bersprungen.
//	pMem	: Zeiger auf den Puffer in dem das Wort gespeichert wird.
//	iMax	: Ist die Gre des Puffers mit Nullzeichen.
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge des Wortes gespeichert.
//	Ergibt die Anzahl der eingelesenen Zeichen
int CTextBuffer::TextWord(zchar *pMem,int iMax,int *pCount)
{
zchar	*pText,cChar;
int		 i,iPos,iNum,iStart;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iMax>0) pMem[0]=0;
	if(pCount)*pCount =0;

	for(iPos=0;iPos<iNum;iPos++)						// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}

	for(iStart=iPos;iPos<iNum;iPos++)
		{
		cChar=pText[iPos];
		if(((cChar>='0')&&(cChar<='9'))	||
		   ((cChar>='A')&&(cChar<='Z'))	||
		   ((cChar>='a')&&(cChar<='z'))	||
			(cChar=='_')||(cChar<  0 ))continue;

		break;
		}

	   i    =iPos-iStart;
	if(i   >=iMax)i=iMax-1;
	if(i   >0    )memcpy(pMem,pText+iStart,i*sizeof(zchar));
	if(iMax>0    )pMem[i]=0;
	if(pCount    )*pCount=i;

	for(;iPos<iNum;iPos++)								// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}



return iPos;
}


//*****************************************************************************
//*
//*		TextWordEx
//*
//*****************************************************************************
//	Liest ein Wort aus dem Stream ein. Dabei werden auch Texte zwischen
//	Anfhrungszeichen bercksichtigt. (z.B:   abc  "hallo 123"  xxxx )
//	Leerzeichen vor und nach der Zahl werden bersprungen.
//	pMem	: Zeiger auf den Puffer in dem das Wort gespeichert wird.
//	iMax	: Ist die Gre des Puffers mit Nullzeichen.
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge des Wortes gespeichert.
//	iCrypt	: Wenn ungleich Null werden  \" Folgen nicht als Ende erkannt.
//	Ergibt die Anzahl der eingelesenen Zeichen
int	CTextBuffer::TextWordEx(zchar *pMem,int iMax,int *pCount,int iCrypt)
{
zchar	*pText;
int		 iPos,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);


	for(iPos=0;iPos<iNum;iPos++)							// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}


	if(pText[iPos]=='"')
			iPos=TextString(pMem,iMax,pCount,iCrypt);
	else	iPos=TextWord  (pMem,iMax,pCount       );


return iPos;
}

//*****************************************************************************
//*
//*		TextChar
//*
//*****************************************************************************
//	Liest eine Zeichenreihe aus ohne 'A' bis 'Z'  '0' bis '9' '_' und >=128
//	pMem	: Zeiger auf den Puffer in dem das Wort gespeichert wird.
//	iMax	: Ist die Gre des Puffers mit Nullzeichen.
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge der Reihe gespeichert.
//	Ergibt die Anzahl der eingelesenen Zeichen
int CTextBuffer::TextChar(zchar *pMem,int iMax,int *pCount)
{
zchar	*pText,cChar;
int		 i,iPos,iNum,iStart;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iMax>0) pMem[0]=0;
	if(pCount)*pCount =0;

	for(iPos=0;iPos<iNum;iPos++)							// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}

	for(iStart=iPos;iPos<iNum;iPos++)
		{
		cChar=pText[iPos];
		if((cChar>='0' && cChar<='9')||
		   (cChar>='A' && cChar<='Z')||
		   (cChar>='a' && cChar<='z')||
			cChar=='_' || cChar<=' ' || cChar>=128)break;
		}

	   i    =iPos-iStart;
	if(i   >=iMax)i=iMax-1;
	if(i   >0    )memcpy(pMem,pText+iStart,i*sizeof(zchar));
	if(iMax>0    )pMem[i]=0;
	if(pCount    )*pCount=i;

	for(;iPos<iNum;iPos++)									// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}



return iPos;
}

//*****************************************************************************
//*
//*		TextValue
//*
//*****************************************************************************
//	Liest eine Zahl aus ( 123 0FFF )
//	Leerzeichen vor und nach dem String werden bersprungen.
//	pMem	: Zeiger auf den Puffer in dem das Zahlenwort gespeichert wird.
//	iMax	: Ist die Gre des Puffers mit Nullzeichen.
//	pCount	: Wenn nicht Null, dann wird hier wird die Lnge der Zahl gespeichert.
//	iHex	: Bestimmt die Art der eingelesene Zahlen
//				0 = dezimal     "123456"
//				1 = hexadezimal "0FFFFF"
//				2 = C++ Format  "0xFFFF"
//	Ergibt die Anzahl der eingelesenen Zeichen
int CTextBuffer::TextValue(zchar *pMem,int iMax,int *pCount,int iHex)
{
zchar	*pText,cChar;
int		 i,iPos,iNum,iStart;



	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iMax>0) pMem[0]=0;
	if(pCount)*pCount =0;

	for(iPos=0;iPos<iNum;iPos++)							// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}

	iStart=iPos;

	if(iHex==2            && pText[iPos  ]=='0')			// C++ Format
	if(pText[iPos+1]=='x' || pText[iPos+1]=='X')
		{
		iPos+=2;
		}

	for(;iPos<iNum;iPos++)
		{
		cChar=pText[iPos];
		if (cChar>='0' && cChar<='9')continue;
		if(iHex)
		if((cChar>='A' && cChar<='F')	||
		   (cChar>='a' && cChar<='f'))continue;

		break;
		}

	   i    =iPos-iStart;
	if(i   >=iMax)i=iMax-1;
	if(i   >0    )memcpy(pMem,pText+iStart,i*sizeof(zchar));
	if(iMax>0    )pMem[i]=0;
	if(pCount    )*pCount=i;

	for(;iPos<iNum;iPos++)									// Leerzeichen berspringen
		{
		if(pText[iPos]==' ' )continue;
		if(pText[iPos]=='\t')continue;
		break;
		}


return iPos;
}

//*****************************************************************************
//*
//*		TextWordLen
//*
//*****************************************************************************
//	Berechnet die Anzahl der kommenden Buchstaben. (0-9,A-Z,a-z,_ und <0)
//	iMax	: Ist die maximale Position bis zu der gesucht wird.
//	Ergibt die Anzahl der kommenden Buchstaben
int CTextBuffer::TextWordLen(int iMax)
{
zchar   *pText,cChar;
int		iPos ,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iNum>iMax)iNum=iMax;

	for(iPos=0;iPos<iNum;iPos++)
		{
		cChar=pText[iPos];
		if((cChar>='0' && cChar<='9') ||
		   (cChar>='A' && cChar<='Z') ||
		   (cChar>='a' && cChar<='z') ||
		   (cChar=='_' || cChar<  0 ))continue;

		break;
		}


return iPos;
}

//*****************************************************************************
//*
//*		TextWordLenEx
//*
//*****************************************************************************
//	Berechnet die Anzahl der kommenden Buchstaben, bzw. Eines Strings "..."
//	iCrypt	: Wenn ungleich Null so werden \" bzw.  \' bersprungen.
//	iMax	: Ist die maximale Position bis zu der gesucht wird.
//	Ergibt die Anzahl der kommenden Buchstaben
int CTextBuffer::TextWordLenEx(int iCrypt,int iMax)
{
int		iLen;



	if(pBuffer[iBufferPos]=='"')
			iLen=TextStringLen(iCrypt,iMax);
	else	iLen=TextWordLen  (       iMax);



return iLen;
}

//*****************************************************************************
//*
//*		TextCharLen
//*
//*****************************************************************************
//	Berechnet die Anzahl der kommenden Nicht-Text-Zeichen bis 0-9,A-Z,a-z,_ und <0
//	iMax	: Ist die maximale Position bis zu der gesucht wird.
//	Ergibt die Anzahl der kommenden Nicht-Text-Zeichen.
int CTextBuffer::TextCharLen(int iMax)
{
zchar	cChar,*pText;
int		iPos,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iNum>iMax)iNum=iMax;

	for(iPos=0;iPos<iNum;iPos++)
		{
		cChar=pText[iPos];
		if((cChar>='0' && cChar<='9') ||
		   (cChar>='A' && cChar<='Z') ||
		   (cChar>='a' && cChar<='z') ||
		   (cChar=='_' || cChar<=' '  || cChar>=128))break;
		}


return iPos;
}

//*****************************************************************************
//*
//*		TextValueLen
//*
//*****************************************************************************
//	Berechnet die Anzahl der kommenden Zahlenzeichen (0-9 bzw. A-F)
//	iHex	: Bestimmt die Art der eingelesene Zahlen
//				0 = dezimal     "123456"
//				1 = hexadezimal "0FFFFF"
//				2 = C++ Format  "0xFFFF"
//	iMax	: Ist die maximale Position bis zu der gesucht wird.
//	Ergibt die Anzahl der kommenden Zahlenzeichen
int CTextBuffer::TextValueLen(int iHex,int iMax)
{
zchar   *pText,cChar;
int		iPos,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iNum>iMax)iNum=iMax;

	iPos=0;

	if(iHex==2       && pText[0]=='0')				// C++ Format
	if(pText[1]=='x' || pText[1]=='X')
		{
		iPos=2;
		}

	for(;iPos<iNum;iPos++)
		{
		cChar=pText[iPos];
		if ((cChar>='0')&&(cChar<='9'))continue;
		if(iHex)
		if(((cChar>='A')&&(cChar<='F'))	||
		   ((cChar>='a')&&(cChar<='f')))continue;

		break;
		}


return iPos;
}


//*****************************************************************************
//*
//*		TextStringLen
//*
//*****************************************************************************
//	Berechnet die Lnge eines folgenden Strings zwischen "..." oder '...'
//	iCrypt	: Wenn ungleich Null so werden \" bzw.  \' bersprungen.
//	iMax	: Ist die maximale Position bis zu der gesucht wird.
//	Ergibt die Lnge des kommenden Strings mit " oder '
int CTextBuffer::TextStringLen(int iCrypt,int iMax)
{
zchar	cMode,cChar,*pText;
int		iPos,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);
	cMode = pText[0];

	if(cMode!='\'' && cMode!='\"')return 0;
	if(iNum>iMax)iNum=iMax;

	for(iPos=0;iPos<iNum;iPos++)
		{
		   cChar =pText[iPos];
		if(cChar>='\n')break;
		if(cChar>='\r')break;
		if(cChar=='\\' && iCrypt)
		if(pText[iPos+1] ==cMode){iPos++;continue;}
		if(cChar         ==cMode){iPos++;break;   }
		}


return (iPos>iNum)? iNum:iPos;
}

//*****************************************************************************
//*
//*		TextUpper
//*
//*****************************************************************************
//	Wandelt die ersten Zeichen im Puffer in Grobuchstaben um.
//	iLen	: Ist die Anzahl der Zeichen die umgewandelt werden sollen
//	Ergibt die Anzahl bearbeiteten Zeichen.
int CTextBuffer::TextUpper(int iLen)
{
zchar	*pText;
int		 iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iLen>iNum)iLen=iNum;

	memupper(pText,iLen);


return iLen;
}

//*****************************************************************************
//*
//*		TextLower
//*
//*****************************************************************************
//	Wandelt die ersten Zeichen im Puffer in Kleinbuchstaben um.
//	iLen	: Ist die Anzahl der Zeichen die umgewandelt werden sollen
//	Ergibt die Anzahl bearbeiteten Zeichen.
int CTextBuffer::TextLower(int iLen)
{
zchar	*pText;
int		 iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);

	if(iLen>iNum)iLen=iNum;

	memlower(pText,iLen);


return iLen;
}


//*****************************************************************************
//*
//*		TextEmpty
//*
//*****************************************************************************
//	Bestimmt die Anzahl der Leerzeichen am Anfang des Puffers
//	iMode	: Wenn ungleich 0 werden auch \r\n bercksichtigt
//	Ergibt die Anzahl der gefundenen Leerzeichen
int CTextBuffer::TextEmpty(int iMode)
{
zchar	*pText,cChar;
int		 iPos,iNum;


	iNum  = (iBufferEnd-iBufferPos)/sizeof(zchar);
	pText = (zchar*)(pBuffer+iBufferPos);


	for(iPos=0;iPos<iNum;iPos++)				// Leerzeichen berspringen
		{
		cChar=pText[iPos];
		if(cChar==' ' )continue;
		if(cChar=='\t')continue;
		if(!iMode)break;
		if(cChar=='\r')continue;
		if(cChar=='\n')continue;
		break;
		}


return iPos;
}


#ifdef _DEBUG
CTextBuffer *CTextBuffer::Convert(CFileBuffer *pPtr  ){return (CTextBuffer*)pPtr   ;}
CTextBuffer *CTextBuffer::Convert(CFileBuffer &sClass){return (CTextBuffer*)&sClass;}
#endif



