//*****************************************************************************
//*
//*
//*		Klasse fr text Suche in Dateien
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
//
//	Copyright Anton Zechner (C) 2003
//
#include	"MapFile.h"
#include	"MemFunctions.h"
#include	"FileFunctions.h"
#include	"StringFunctions.h"
#include	"TextFind.h"
#include	<stdlib.h>


#define 	MAX_COUNT	1000								// Maximal gefunden Anzahl an Positionen

CStringChain	sCache;  


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CTextFind::CTextFind()
{

	sPositions.DataLength(sizeof(PosType));
	iMaxCount	= MAX_COUNT;
	iLineCount  = 10;										// Zeilenanzahl fr Multi-Line-Suche
	iSearchFile	= 0;										// Aktuelle Suchdatei
	iSearchPos	= 0;										// Aktuelle Suchposition
	iSearchByte = 0;										// Abgesuchte Bytes
	iSortMode	=-1;										// Sortiermodus
	iSortDir	= 0;										// Sortierrichtung
	pCache		= 0;										// Cache fr die Dateien

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CTextFind::~CTextFind()
{

}


//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Datei nach einen Text
//	pPtr		: Zeiger auf die Daten aus der Datei.
//	iLen		: Ist die Gre der Daten aus der Datei.
//	pFilename	: Ist der Zeiger auf den Dateinamen.
//	pText		: Ist der Zeiger auf den Text.
//	iMode		: CTF_NOCASE	Groklein-Buchstaben nicht unterscheiden
//				  CTF_WORD		Nur ganze Wrter suchen
//			      CTF_SPECIAL	Spezieler Ausdruck (mit '*' '?')
//	Ergibt die Anzahl der gefunden Stellen
int	CTextFind::Find(const void *pPtr,int iLen,const char *pFilename,const char *pText,int iMode)
{
int			iCase,iWord,iFlag,iSpec,iLenT,iNumB,iMult;
int			iNum,iLine,iMax,iEnd,iSize,i,j;
char	   *pMem=(char*)pPtr; 
char	    cPath[MAX_PATH];
PosType		sTypeP;


	iStop1		= 0;
	iSearchPos	= 0;								// Aktuelle Suchposition
	iSortMode	=-1;								// Sortiermodus
	iSortDir	= 0;								// Sortierrichtung

	iWord = (iMode&CTF_WORD    )? 1:0;
	iCase = (iMode&CTF_NOCASE  )? 1:0;
	iSpec = (iMode&CTF_SPECIAL )? 2:0;
	iMult = (iMode&CTF_MULTLINE)? 1:0;


	if(iSpec)iCase|=4;


	if(!pText || !*pText || !pMem || iLen<=0)return 0;
	if(sPositions.Count()       >= iMaxCount)return 0;


	for(iLenT=0;pText[iLenT];iLenT++)				// Lnge vom Suchtext ermitteln
		{
		if(iSpec==2 && strwchar(pText[iLenT],0x01900000))
			{
			iSpec = 1;								// Gibt es Spezialzeichen im Suchtext (*?\)
			}
		}


	iSpec =	iSpec&1; 
	iLenT = strlen(pText);							
	iNumB = iLenT;
	iNum  = 0;
	iLine = 1;
	



	if(strwchar(*pText,0x3000000F))					// Beginnt der Suchbegriff mit einem Buchstaben
			iFlag=iWord;
	else	iFlag=0;
	
	if(!iSpec && iWord && !strwchar(pText[iLenT-1],0x3000000F))
		{
		iWord = 0;									// Wortende nicht prfen
		}		


	while(iSearchPos<iLen)
		{
		if(iStop1)break;
													// Zeilenlnge holen
		iMax = strcwlen(pMem+iSearchPos,0x40,iLen-iSearchPos);
		iEnd = iSearchPos+iMax;
		i    = iSearchPos;

		if(!iSpec)iEnd-=iLenT-1;

		while(i<iEnd)
			{
			if(iSpec)								// Spezieller Ausdruck mit (*?)
				{
				iCase &= ~0x18;
				iSize  =  iEnd;

				if(iMult)							// Suche ber mehrere Zeilen
				for(j=0;j<iLineCount;j++)
					{
					iSize += strwlen (pMem+iSize,0x40,iLen-iSize);
					iSize += strcwlen(pMem+iSize,0x40,iLen-iSize);
					}

				for(;i<iEnd;i++)
					{
					if(!iFlag || i<=iSearchPos || !(iCase&0x10))		
						{
						   j=memycmp(pText,iLenT,pMem+i,iSize-i,iCase);
						if(j)
							{
							iNumB=j;
		  					if(iWord && i+iNumB<iSize)
							if(strwchar(pMem[i+iNumB  ],0x3000000F))
							if(strwchar(pMem[i+iNumB-1],0x3000000F))
								{
								continue;			// Kein Wortende
								}

							break;
							}
						}

					if(strwchar(pMem[i],0x3000000F))// War vorher ein Buchstabe
							iCase|= 0x10;
					else	iCase&=~0x10;
							iCase|= 0x08;
					}
				}
			else{
				for(;i<iEnd;i++)
					{
					if(iFlag && i>iSearchPos && strwchar(pMem[i-1],0x3000000F))
						{
						continue;					// Kein Wortbegin
						}

					if(iWord && i<iEnd-1  && strwchar(pMem[i+iLenT],0x3000000F))
						{
						continue;					// Kein Wortende
						}
					
					if(!memgcmp(pMem+i,pText,iLenT,iCase))break;
					}
				}

			if(i>=iEnd)break;						// Nichts gefunden ?

			if(!iNum)								// Neue Datei
				{
				_fullpath(cPath,pFilename,sizeof(cPath));
				   sTypeP.iFile=sFiles.Find(cPath,0,-1,1);
				if(sTypeP.iFile<0)sTypeP.iFile=sFiles.Add(cPath);
				}

			iNum++;
			sTypeP.iLine = iLine;
			sTypeP.iPos  = i;
			sTypeP.iLen	 = iNumB;

			sPositions.Add(iMax,pMem+iSearchPos,CSC_END,&sTypeP);

			if(sPositions.Count() >= iMaxCount)return iNum;

			i+=iNumB;
			}

		iLine++;									// Nchste iLine
		iSearchPos+=iMax;

		if(iSearchPos<iLen && pMem[iSearchPos]=='\r')iSearchPos++;
		if(iSearchPos<iLen && pMem[iSearchPos]=='\n')iSearchPos++;
		if(iSearchPos<iLen && pMem[iSearchPos]==  0 )iSearchPos++;
		}


return iNum;
}


//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Datei nach einen Text
//	pFilename	: Ist der Zeiger auf den Dateinamen.
//	pText		: Ist der Zeiger auf den Text.
//	iMode		: CTF_NOCASE	Groklein-Buchstaben nicht unterscheiden
//				  CTF_WORD		Nur ganze Wrter suchen
//			      CTF_SPECIAL	Spezieler Ausdruck (mit '*' '?')
//	Ergibt die Anzahl der gefunden Stellen
int	CTextFind::Find(const char *pFilename,const char *pText,int iMode)
{
CacheItem  *pItem;
char	   *pMem;
char	   *pPtr;
CMapFile	sMap;
int			iLen;
int			iPos;


	iStop1		= 0;
	iSearchPos	= 0;										// Aktuelle Suchposition
	iSortMode	=-1;										// Sortiermodus
	iSortDir	= 0;										// Sortierrichtung


	if(!pText || !*pText)return 0;
	if(sPositions.Count() >= iMaxCount)return 0;

		pMem=sMap.Open(pFilename);
	if(!pMem)return 0;

	iLen = sMap.GetLength();

	if(pCache && iLen+pCache->GetUserData()<iMaxCache)		// In den Cache einfgen
		{
		pPtr	= new char[iLen];

		if(pPtr)
			{
			iPos   = pCache->Add(pFilename,CSC_ABCG);
			pItem  = (CacheItem*)pCache->Data(iPos);

			pItem->iLen		= iLen;							// Lnge der Cache-Daten
			pItem->iFlag	= 1;							// Wurde die Datei benutzt
			pItem->uTime	= FileTime(pFilename);			// nderungszeit der Datei
			pItem->pMem		= pPtr;							// Zeiger auf die Daten

			memcpy(pPtr,pMem,iLen);

			iPos	= iLen+pCache->GetUserData();
			pCache->SetUserData(iPos);
			}
		}



return Find(pMem,iLen,pFilename,pText,iMode);
}

//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Dateiliste nach einem Text
//	pFiles		: Zeiger auf die Dateiliste
//	pText		: Ist der Zeiger auf den Text.
//	iMode		: CTF_NOCASE	Groklein-Buchstaben nicht unterscheiden
//				  CTF_WORD		Nur ganze Wrter suchen
//				  CTF_SPECIAL	Spezieler Ausdruck (mit '*' '?')
//	Ergibt die Anzahl der gefunden Stellen
int	CTextFind::Find(CFileFinder *pFlies,const char *pText,int iMode)
{
CacheItem  *pItem;
char	   *pName;
int			iPos;
int			iNum;
int			iLen;
int			i;



	iNum		= 0;	
	iStop2		= 0;
	iSearchFile	= 0;										// Aktuelle Suchdatei
	iSearchPos	= 0;										// Aktuelle Suchposition
	iSearchByte = 0;										// Abgesuchte Bytes

	for(i=0;i<pFlies->Count();i++)
		{
		if(iStop2)break;
		iSearchFile	= i;
		pName=pFlies->Name(i,1);
		if(sPositions.Count() >= iMaxCount)break;
		
		if(pCache)											// Gibt es einen Datei-Cache
			{
			   iPos = pCache->FindFast(pName,0,-1,0x01);
			if(iPos>=0)
				{
				pItem = (CacheItem*)pCache->Data(iPos);
										
				if(!CmpFileTime(pItem->uTime,pName))		// Hat sich die Datei inzwischen verndert
					{
					pItem->iFlag = 1;
					iNum+=Find(pItem->pMem,pItem->iLen,pName,pText,iMode);
					iSearchByte+=(int64)((unsigned)iSearchPos);
					iSearchPos  =0;
					continue;
					}

				iLen  = pItem->iLen;						// Datei aus dem Cache lschen	
				iLen -= pCache->GetUserData();
				pCache->SetUserData(iLen);

				delete pItem->pMem;
				pCache->Del(iPos);
				}
			}

		iNum+=Find(pName,pText,iMode);
		iSearchByte+=(int64)((unsigned)iSearchPos);
		iSearchPos  = 0;
		}


return iNum;
}


//*****************************************************************************
//*
//*		Find
//*
//*****************************************************************************
//	Sucht in einer Dateiliste nach einem Text
//	pFiles		: Zeiger auf die Dateiliste
//	pText		: Ist der Zeiger auf den Text.
//	iMode		: CTF_NOCASE	Groklein-Buchstaben nicht unterscheiden
//				  CTF_WORD		Nur ganze Wrter suchen
//				  CTF_SPECIAL	Spezieler Ausdruck (mit '*' '?')
//	Ergibt die Anzahl der gefunden Stellen
int	CTextFind::Find(CStringChain *pFlies,const char *pText,int iMode)
{
CacheItem  *pItem;
char	   *pName;
int			iPos;
int			iNum;
int			iLen;
int			i;




	iNum		= 0;
	iStop2		= 0;
	iSearchFile	= 0;										// Aktuelle Suchdatei
	iSearchPos	= 0;										// Aktuelle Suchposition
	iSearchByte = 0;										// Abgesuchte Bytes
															
	for(i=0;i<pFlies->Count();i++)
		{
		if(iStop2)break;
		iSearchFile=i;
		pName=pFlies->Get(i);
		if(sPositions.Count() >= iMaxCount)break;
		
		if(pCache)											// Gibt es einen Datei-Cache
			{
			   iPos = pCache->FindFast(pName,0,-1,0x01);
			if(iPos>=0)
				{
				pItem = (CacheItem*)pCache->Data(iPos);
										
				if(!CmpFileTime(pItem->uTime,pName))		// Hat sich die Datei inzwischen verndert
					{
					pItem->iFlag = 1;
					iNum+=Find(pItem->pMem,pItem->iLen,pName,pText,iMode);
					iSearchByte+=(int64)((unsigned)iSearchPos);
					iSearchPos  =0;
					continue;
					}

				iLen  = pItem->iLen;						// Datei aus dem Cache lschen	
				iLen -= pCache->GetUserData();
				pCache->SetUserData(iLen);

				delete pItem->pMem;
				pCache->Del(iPos);
				}
			}
		
		iNum+=Find(pName,pText,iMode);
		iSearchByte+=(int64)((unsigned)iSearchPos);
		iSearchPos  =0;
		}

return iNum;
}


//*****************************************************************************
//*
//*		Delete
//*
//*****************************************************************************
//	Alle gefunden Eintrge lschen
//	Ergibt 1 wenn alles ok ist sonst 0
int	CTextFind::Delete()
{

	sPositions.Del();
	sFiles	  .Del();


return 1;
}


//*****************************************************************************
//*
//*		Delete
//*
//*****************************************************************************
//	Einen einzelnen gefunden Eintrag lschen
//	iNr		: Ist die Nummer des zu lschenden Eintrages
//	Ergibt 1 wenn alles ok ist sonst 0
int	CTextFind::Delete(int iNr)
{


	if((unsigned)iNr>=(unsigned)sPositions.Count())return 0;

		sPositions.Del(iNr);
	if(!sPositions.Count())sFiles.Del();


return 1;
}


//*****************************************************************************
//*
//*		GetFileName
//*
//*****************************************************************************
//	Einen Dateinamen eines gefunden Eintrages abfragen
//	iNr		: Ist die Nummer des abzufragenden Eintrages
//  Ergibt einen Zeiger auf den Dateinamen.
char  *CTextFind::GetFileName(int iNr)
{
PosType	 *pType;


		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return 0;


return sFiles[pType->iFile];
}

//*****************************************************************************
//*
//*		GetFileName
//*
//*****************************************************************************
//	Einen Dateinamen samt Lnge eines gefunden Eintrages abfragen
//	iNr		: Ist die Nummer des abzufragenden Eintrages
//	iLen	: Hier wird die Lnge des Dateinamens gespeichert.
//  Ergibt einen Zeiger auf den Dateinamen.
char  *CTextFind::GetFileName(int iNr,int &iLen)
{
PosType	 *pType;


		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)
		{
		iLen = 0;
		return 0;
		}


return sFiles.Get(pType->iFile,iLen);
}

//*****************************************************************************
//*
//*		GetFileName
//*
//*****************************************************************************
//	Einen Dateinamen eines gefunden Eintrages in einen Puffer speichern
//	iNr			: Ist die Nummer des abzufragenden Eintrages
//	pBuffer		: Ist der Zeiger auf den Puffer
//	iSize		: Ist die Gre von Puffer
//  Ergibt den Dateinamen bzw die Stringlnge wenn Puffer angegeben ist
int CTextFind::GetFileName(int iNr,char *pBuffer,int iSize)
{
PosType	 *pType;



	if(iSize<=0)return 0;
	pBuffer[0]=0;

		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return 0;



return sFiles.Get(pType->iFile,pBuffer,iSize);
}


//*****************************************************************************
//*
//*		GetLineNr
//*
//*****************************************************************************
//	Die  Zeilennummer eines gefunden Eintrages abfragen.
//	iNr			: Ist die Nummer des abzufragenden Eintrages
//	Ergibt die Zeilennummer vom Eintrag
int  CTextFind::GetLineNr(int iNr)
{
PosType	 *pType;


		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return 0;


return pType->iLine;
}

//*****************************************************************************
//*
//*		GetPos
//*
//*****************************************************************************
//	Den Offset in einer Datei von einem gefunden Eintrag abfragen.
//	iNr			: Ist die Nummer des abzufragenden Eintrages
//	Ergibt das Byte-Offset vom dem Eintrag bzw. -1 bei einem Fehler
int  CTextFind::GetPos(int iNr)
{
PosType	*pType;


		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return -1;


return pType->iPos;
}

//*****************************************************************************
//*
//*		SetPos
//*
//*****************************************************************************
//	Speichert einen neuen Datei-Offset fr einen gefunden Eintrag.
//	iNr			: Ist die Nummer des zu ndernten Eintrages
//	Ergibt den neuen Offset vom Eintrag bzw. -1 bei einem Fehler
int CTextFind::SetPos(int iNr,int iPos)
{
PosType	*pType;


		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return -1;

	pType->iPos=iPos;


return iPos;
}

//*****************************************************************************
//*
//*		GetLength
//*
//*****************************************************************************
//	Die Gre eines gefundenen Eintrages abfragen.
//	iNr			: Ist die Nummer des abzufragenden Eintrages
//	Ergibt die Byte-Anzahl vom dem Eintrag bzw. -1 bei einem Fehler
int  CTextFind::GetLength(int iNr)
{
PosType	*pType;


		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return -1;


return pType->iLen;
}

//*****************************************************************************
//*
//*		SetLength
//*
//*****************************************************************************
//	Setzt die Gre vom einem gefundenen Eintrag.
//	iNr			: Ist die Nummer des zu ndernten Eintrages
//	iLen		: Ist die neue Gre in Byte.
//	Ergibt die neue Gre bzw. -1 bei einem Fehler
int  CTextFind::SetLength(int iNr,int iLen)
{
PosType	*pType;


		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return -1;
		pType->iLen=iLen;


return iLen;
}

//*****************************************************************************
//*
//*		GetText
//*
//*****************************************************************************
//	In einen Puffer wird der Text im Format "Dateiname(Zeile): Text" gespeichert.
//	iNr			: Ist die Nummer des abzufragenden Eintrages.
//	pBuffer		: Ist der Zeiger auf den Puffer.
//	iSize		: Ist die Gre der Puffers.
//	iStart		: Ist die Position bei der Text in den Puffer kopiert wird.
//				  Vorher wird wenn ntig ein Leerraum eingefgt.
//  Ergibt die Stringlnge
int CTextFind::GetText(int iNr,char *pBuffer,int iSize,int iStart)
{
PosType	    *pType;
int			 iPos;


	if(iSize<=0)return 0;

	pBuffer[0]=0;

		pType=(PosType*)sPositions.Data(iNr);
	if(!pType)return 0;

	   iPos=sFiles.Get(pType->iFile,pBuffer,iSize);		// Dateiname anhngen
	if(iPos+16>iSize)return iPos;

	pBuffer[iPos]='(';iPos++;							// Zeilennummer anhngen
	iPos+=itostr(pType->iLine,pBuffer+iPos);
	pBuffer[iPos]=')';iPos++;
	pBuffer[iPos]=':';iPos++;
	pBuffer[iPos]=' ';iPos++;

	if(iStart+1>iSize)iStart=iSize-1;					// Leerraum einfgen
	for(;iPos<iStart;iPos++)pBuffer[iPos]=' ';


	iPos+=sPositions.Get(iNr,pBuffer+iPos,iSize-iPos);	// Zeilentext anhngen


return iPos;
}

//*****************************************************************************
//*
//*		SortPath
//*
//*****************************************************************************
int __cdecl CTextFind::SortPath(CStringChain *pList,int iPos1,int iPos2)
{
CTextFind  *pThis=(CTextFind*)pList->GetUserPtr();
const char *pPath1;
const char *pPath2;
PosType	   *pType1;
PosType	   *pType2;
int			iCmp;

	
	pType1	= (PosType*)pList->Data(iPos1);
	pType2	= (PosType*)pList->Data(iPos2);
	pPath1	= pThis->sFiles.Get(pType1->iFile);
	pPath2	= pThis->sFiles.Get(pType2->iFile);
	
	   iCmp	= strvicmp(pPath1,pPath2);
	if(iCmp)return iCmp;

	   iCmp	= pType1->iLine-pType2->iLine;
	if(iCmp)return iCmp;

	   iCmp	= pType1->iPos-pType2->iPos;
	if(iCmp)return iCmp;


return pType1->iLen-pType2->iLen;
}

//*****************************************************************************
//*
//*		SortName
//*
//*****************************************************************************
int __cdecl CTextFind::SortName(CStringChain *pList,int iPos1,int iPos2)
{
CTextFind  *pThis=(CTextFind*)pList->GetUserPtr();
const char *pPath1;
const char *pPath2;
const char *pName1;
const char *pName2;
PosType	   *pType1;
PosType	   *pType2;
int			iCmp;

	
	pType1	= (PosType*)pList->Data(iPos1);
	pType2	= (PosType*)pList->Data(iPos2);
	pPath1	= pThis->sFiles.Get(pType1->iFile);
	pPath2	= pThis->sFiles.Get(pType2->iFile);
	pName1	= ::GetFileName(pPath1);
	pName2	= ::GetFileName(pPath2);

	   iCmp	= strvicmp(pName1,pName2);
	if(iCmp)return iCmp;
	
	   iCmp	= strvicmp(pPath1,pPath2);
	if(iCmp)return iCmp;

	   iCmp	= pType1->iLine-pType2->iLine;
	if(iCmp)return iCmp;

	   iCmp	= pType1->iPos-pType2->iPos;
	if(iCmp)return iCmp;


return pType1->iLen-pType2->iLen;
}

//*****************************************************************************
//*
//*		SortType
//*
//*****************************************************************************
int __cdecl CTextFind::SortType(CStringChain *pList,int iPos1,int iPos2)
{
CTextFind  *pThis=(CTextFind*)pList->GetUserPtr();
const char *pPath1;
const char *pPath2;
const char *pName1;
const char *pName2;
PosType	   *pType1;
PosType	   *pType2;
int			iCmp;

	
	pType1	= (PosType*)pList->Data(iPos1);
	pType2	= (PosType*)pList->Data(iPos2);
	pPath1	= pThis->sFiles.Get(pType1->iFile);
	pPath2	= pThis->sFiles.Get(pType2->iFile);
	pName1	= GetFileType(pPath1);
	pName2	= GetFileType(pPath2);

	   iCmp	= stricmp(pName1,pName2);
	if(iCmp)return iCmp;
	
	   iCmp	= strvicmp(pPath1,pPath2);
	if(iCmp)return iCmp;

	   iCmp	= pType1->iLine-pType2->iLine;
	if(iCmp)return iCmp;

	   iCmp	= pType1->iPos-pType2->iPos;
	if(iCmp)return iCmp;


return pType1->iLen-pType2->iLen;
}

//*****************************************************************************
//*
//*		Sort
//*
//*****************************************************************************
//	Sortiere das Ergbniss
//	iMode		: Ist die Sortierart
//					CTF_PATH	0	Nach dem ganzen Pfad sortieren
//					CTF_NAME	1	Nach dem Dateinamen sortieren
//					CTF_TYPE	2	Nach dem Dateityp sortieren
//	iDir		: Ist die Sortierrichtung (-1=toogle)
//	Errgibt 1 wenn die Reihenfolge verndert wurde, ansonsten 0
int CTextFind::Sort(int iMode,int iDir)
{

	if(iDir<0)
		{
		iDir = (iSortMode==iMode)? !iSortDir:0;
		}

	if(iSortMode==iMode && iSortDir==iDir)				// Sortierrichtung
		{
		return 0;
		}


	sPositions.SetUserPtr(this);
	
	switch(iMode)
		{
	case CTF_PATH:										// Nach dem ganzen Pfad sortieren

			sPositions.Sort(0,-1,SortPath,iDir);
			break;

	case CTF_NAME:										// Nach dem Dateinamen sortieren

			sPositions.Sort(0,-1,SortName,iDir);
			break;

	case CTF_TYPE:										// Nach dem Dateityp sortieren

			sPositions.Sort(0,-1,SortType,iDir);
			break;

	default:	return 0;
		}


	iSortMode	= iMode;
	iSortDir	= iDir;


return 1;
}
