//*****************************************************************************
//*
//*
//*		TextFind.h: Schnittstelle fr die Klasse CTextFind.
//*
//*
//*****************************************************************************
#ifndef	__TEXTFIND_H__
#define	__TEXTFIND_H__

#if _MSC_VER > 1000
#pragma once
#endif

#include	"StringChain.h"
#include	"FileFinder.h"


// Konstanten fr Find
#define 	CTF_NOCASE		0x0001		// Groklein-Buchstaben nicht unterscheiden
#define 	CTF_WORD		0x0002		// Nur ganze Wrter Suchen
#define 	CTF_SPECIAL		0x0004		// Spezieler Ausdruck (mit '*' '?')
#define 	CTF_MULTLINE	0x0008		// bermehre Zeilen suchen

// Konstanten fr SORT
#define 	CTF_PATH		0x0000		// Nach dem ganzen Pfad sortieren
#define 	CTF_NAME		0x0001		// Nach dem Dateinamen sortieren
#define 	CTF_TYPE		0x0002		// Nach dem Dateityp sortieren

#ifndef 	int64
#define 	int64	__int64
#endif


//*****************************************************************************
//*
//*		Funktionen
//*
//*****************************************************************************
class CTextFind
{
public:

	 CTextFind();
	~CTextFind();

	int		Find(const void *pMem,int iSize,const char   *pFilename,const char *pText,int iMode=0);
	int		Find(                           const char   *pFilename,const char *pText,int iMode=0);
	int		Find(                           CFileFinder  *pFiles   ,const char *pText,int iMode=0);
	int		Find(                           CStringChain *pFlies   ,const char *pText,int iMode=0);

	int		Count();										// Anzahl der gefunden Eintrge
	int 	Sort(int iMode,int iDir=0);						// Sortiere das Ergbniss
	void	Break();										// Suche abbrechen
	int		Delete();										// Alles lschen
	int		Delete     (int iNr);							// Einen Eintrag lschen
	char   *GetFileText(int iNr);							// Ergibt den Zeilentext
	int     GetFileText(int iNr,char *pBuffer,int iSize);	// Zeilentext in einem Puffer speichern
	char   *GetFileName(int iNr);							// Ergibt den Dateinamen
	char   *GetFileName(int iNr,int &iLen);					// Ergibt den Dateinamen
	int     GetFileName(int iNr,char *pBuffer,int iSize);	// Dateinamen in einem Puffer speichern
	int     GetLineNr  (int iNr);							// Ergibt die Zeilennummer
	int     GetPos	   (int iNr);							// Ergibt Byte Position
	int     SetPos	   (int iNr,int iPos);					// Setzt  Byte Position
	int     GetLength  (int iNr);							// Ergibt Byte Anzahl
	int     SetLength  (int iNr,int iLen);					// Setzt  Byte Anzahl
	int     GetText    (int iNr,char *pBuffer,int iSize,int iStart=0);
	void    SetChache  (CStringChain *pCache,int iMax);
	int		GetLineCount ();
	int		SetLineCount (int iLines);
	int		GetMaxCount  ();
	int		SetMaxCount  (int iMax);
	int     GetSearchFile();								// Ergibt die Suchdatei
	int     GetSearchPos ();								// Ergibt die Suchposition
	int64	GetSearchByte();								// Ergibt abgesuchte Bytes

protected:
	
	static	int	__cdecl SortPath(CStringChain *pList,int iPos1,int iPos2);
	static	int	__cdecl SortName(CStringChain *pList,int iPos1,int iPos2);
	static	int	__cdecl SortType(CStringChain *pList,int iPos1,int iPos2);

//*****************************************************************************
//*
//*		Variabeln
//*
//*****************************************************************************

protected:

	typedef	struct											// Daten fr Suchergebnis
		{													
		int   iLine;										// iNr der Text iLine
		int   iFile;										// iNr der Date
		int   iPos;											// Byte Position
		int   iLen;											// Byte Anzahl
		}PosType;											
															
	typedef	struct											// Daten fr Suchergebnis
		{													
		int			iLen;									// Lnge der Cache-Daten
		int			iFlag;									// Wurde die Datei benutzt
		unsigned 	uTime;									// nderungszeit der Datei
		char	   *pMem;									// Zeiger auf die Daten
		}CacheItem;

	CStringChain   *pCache;									// Cache fr die Dateien
	CStringChain	sFiles;									// Durchsuchte Dateien
	CStringChain	sPositions;								// Gefundene Positionen
	int				iMaxCache;								// Maximale Cache-Gre
	int				iMaxCount;								// Maximale Positions Anzahl
	int				iLineCount;								// Zeilenanzahl fr Multi-Line-Suche
	int				iSearchFile;							// Aktuelle Suchdatei
	int				iSearchPos;								// Aktuelle Suchposition
	int64			iSearchByte;							// Abgesuchte Bytes
	volatile int	iStop1,iStop2;							// Flags fr Break
	int				iSortMode;								// Sortiermodus
	int				iSortDir;								// Sortierrichtung
														
};
						
//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

inline	void    CTextFind::SetChache(CStringChain *pNew,int iMax){pCache=pNew;iMaxCache=iMax;}
inline	int		CTextFind::Count(){return sPositions.Count();}
inline	char   *CTextFind::GetFileText(int iNr){return sPositions[iNr];}
inline	int		CTextFind::GetFileText(int iNr,char *Puffer,int Size){return sPositions.Get(iNr,Puffer,Size);}
inline	int		CTextFind::GetMaxCount(){return iMaxCount;}
inline	int		CTextFind::SetMaxCount(int iMax){int iOld=iMaxCount;iMaxCount=iMax;return iOld;}
inline	int		CTextFind::GetLineCount(){return iLineCount;}
inline	int		CTextFind::SetLineCount(int Lines){int iOld=iLineCount;iLineCount=Lines;return iOld;}
inline	void	CTextFind::Break(){iStop1=1;iStop2=1;}
inline	int     CTextFind::GetSearchPos (){return iSearchPos ;}
inline	int     CTextFind::GetSearchFile(){return iSearchFile;}
inline	int64   CTextFind::GetSearchByte(){return iSearchByte;}

#endif
