//*****************************************************************************
//*
//*
//*		Diverse Zeit Funktionen
//*
//*
//*****************************************************************************
//
//	(C) Copyright Anton Zechner 2007
//
#include	"TimeConvert.h"
#include	<math.h>

#define 	OLE_FOUR		1460.0
#define 	OLE_1980		29221.0
#define 	OLE_2030		47484.0
#define 	UTC_FOUR		126230400
#define 	UTC_1980		315532800
#define 	UTC_2030		1893456000
#define 	FTG_FOUR		((__int64)0x0047C0F0D84C000)
#define 	FTG_1980		((__int64)0x1A8E79FE1D58000)
#define 	FTG_2030		((__int64)0x1E0F6C0A005C000)



static int	 iInitDone=0;
const  int   aTpPDays [] = {  0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366, 366, 366,   0};
const  int   aTpDays  [] = {  0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365, 365, 365,   0};
const  int   aTpMPDays[] = { 31, 29, 31, 30,  31,  30,  31,  31,  30,  31,  30,  31,   0,   0,   0,   0};
const  int   aTpMDays [] = { 31, 28, 31, 30,  31,  30,  31,  31,  30,  31,  30,  31,   0,   0,   0,   0};
const  char *aTpWday  [] = { "So " ,"Mo " ,"Di " ,"Mi " ,"Do ","Fr ","Sa ","?? "};
const  char *aTpMonth [] = { "???.","Jan.","Feb.","Mar.",
						     "Apr.","Mai.","Jun.","Jul.",
						     "Aug.","Sep.","Okt.","Nov.",
						     "Dec.","???.","???.","???."};

extern	time_t __cdecl _mkgmtime(struct tm *tb);



//*****************************************************************************
//*
//*		TimePrint
//*
//*****************************************************************************
//	Wantelt eine Zeit in eienen Zeitstring um.
//	pStr		: Zeiger auf den Textpuffer in den die Zeit gespeichert wird.
//	sSysTime	: Systemzeit die in einen Text umgewandelt werden soll.
//	iMode		: Bestimmt die Art der Umwandlung
//					TP_MILISEK		0x0001	Millisekunden angeben
//					TP_WDAY		 	0x0002	Wochentag angeben
//					TP_TIME 		0x000C	Zeit angeben
//					TP_TIMESHORT 	0x0004	Zeit angeben ohne Sekunden
//					TP_DATE 		0x0030	Datum angeben mit Jahr 4 stellig
//					TP_DATE2		0x0050	Datum angeben mit Jahr 2 stellig
//					TP_DATE_ISO 	0x0100	Datum angeben mit [JJ]JJ-MM-TT
//					TP_DATESHORT	0x0010	Datum angeben ohne Jahr
//					TP_ONE_SPACE 	0x0200	Nur ein Leerzeichen wischen Datum-Zeit
//					TP_ABC 			0x0080	Monatsnamen ausschreiben
//					TP_ALL   		0x00BE	Alle Flags ohne Millisekunden
//					TP_TOGLOBAL		0x4000	Lokale  in globale Zeit umwandeln
//					TP_TOLOCAL		0x8000	Globale in lokale  Zeit umwandeln
//	Ergibt die Lnge des gespeicherten Textes.
int TimePrint(char *pStr,const SYSTEMTIME &sSysTime,int iMode)
{
char				*pTxt=pStr;
const	SYSTEMTIME  *pSys=&sSysTime;
		SYSTEMTIME   sSystem;
		FILETIME	 sGlobal;
		FILETIME	 sLocal;



	if(iMode&TP_TOGLOBAL)							// Lokale in globale Zeit umwandeln
		{
		if(SystemTimeToFileTime   (&sSysTime,&sLocal ) &&
		   LocalFileTimeToFileTime(&sLocal  ,&sGlobal) &&
		   FileTimeToSystemTime   (&sGlobal ,&sSystem))
			{
			pSys = &sSystem;
			}
		}

	if(iMode&TP_TOLOCAL)							// Globale in lokale  Zeit umwandeln
		{
		if(SystemTimeToTzSpecificLocalTime(NULL,(LPSYSTEMTIME)&sSysTime,&sSystem))
/*
		if(SystemTimeToFileTime   (&sSysTime,&sGlobal) &&
		   FileTimeToLocalFileTime(&sGlobal ,&sLocal ) &&
		   FileTimeToSystemTime   (&sLocal,  &sSystem))*/
			{
			pSys = &sSystem;
			}
		}

	if(iMode&TP_WDAY)
		{
		*(int*)pTxt=*(int*)aTpWday[pSys->wDayOfWeek&7];pTxt+=3;
		}

	if(iMode&TP_DATE)
		{
		if(iMode&TP_DATE_ISO)
			{
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/1000)%10;
				pTxt[1]='0'+(pSys->wYear/100 )%10;
				pTxt[2]='0'+(pSys->wYear/10  )%10;
				pTxt[3]='0'+(pSys->wYear     )%10;
				pTxt[4]='-';
				pTxt+=5;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/10)%10;
				pTxt[1]='0'+(pSys->wYear   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}
			if(iMode&TP_ABC)
				{
				*(int*)pTxt=*(int*)aTpMonth[pSys->wMonth&15];
				pTxt[3]='-';
				pTxt+=4;
				}
			else{
				pTxt[0]='0'+(pSys->wMonth/10)%10;
				pTxt[1]='0'+(pSys->wMonth   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}

			pTxt[0]='0'+(pSys->wDay/10)%10;
			pTxt[1]='0'+(pSys->wDay   )%10;
			pTxt+=2;
			}
		else{
			pTxt[0]='0'+(pSys->wDay/10)%10;
			pTxt[1]='0'+(pSys->wDay   )%10;
			pTxt[2]='.';
			pTxt+=3;

			if(iMode&TP_ABC)
				{
				*(int*)pTxt=*(int*)aTpMonth[pSys->wMonth&15];pTxt+=4;
				}
			else{
				pTxt[0]='0'+(pSys->wMonth/10)%10;
				pTxt[1]='0'+(pSys->wMonth   )%10;
				pTxt[2]='.';
				pTxt+=3;
				}
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/1000)%10;
				pTxt[1]='0'+(pSys->wYear/100 )%10;
				pTxt[2]='0'+(pSys->wYear/10  )%10;
				pTxt[3]='0'+(pSys->wYear     )%10;
				pTxt+=4;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/10)%10;
				pTxt[1]='0'+(pSys->wYear   )%10;
				pTxt+=2;
				}
			}

		if(iMode&TP_TIME)
			{
			*(short*)pTxt=0x2020;
			pTxt+=(iMode&TP_ONE_SPACE)? 1:2;
			}
		}

	if(iMode&TP_TIME)
		{
		pTxt[0]='0'+(pSys->wHour  /10)%10;
		pTxt[1]='0'+(pSys->wHour     )%10;
		pTxt[2]=':';
		pTxt[3]='0'+(pSys->wMinute/10)%10;
		pTxt[4]='0'+(pSys->wMinute   )%10;
		pTxt+=5;

		if(iMode&(TP_TIME&~TP_TIMESHORT))
			{
			pTxt[0]=':';
			pTxt[1]='0'+(pSys->wSecond/10)%10;
			pTxt[2]='0'+(pSys->wSecond   )%10;
			pTxt+=3;
			}

		if(  iMode&(TP_MILLISECEX|TP_MILLISEC))
		if(!(iMode& TP_MILLISECEX) || pSys->wMilliseconds)
			{
			pTxt[0]='.';
			pTxt[1]='0'+(pSys->wMilliseconds/100)%10;
			pTxt[2]='0'+(pSys->wMilliseconds/10 )%10;
			pTxt[3]='0'+(pSys->wMilliseconds    )%10;
			pTxt+=4;
			}
		}

	pTxt[0]=0;



return pTxt-pStr;
}

int TimePrint(wchar_t *pStr,const SYSTEMTIME &sSysTime,int iMode)
{
wchar_t				*pTxt=pStr;
const	SYSTEMTIME  *pSys=&sSysTime;
		SYSTEMTIME   sSystem;
		FILETIME	 sGlobal;
		FILETIME	 sLocal;



	if(iMode&TP_TOGLOBAL)							// Lokale in globale Zeit umwandeln
		{
		if(SystemTimeToFileTime   (&sSysTime,&sLocal ) &&
		   LocalFileTimeToFileTime(&sLocal  ,&sGlobal) &&
		   FileTimeToSystemTime   (&sGlobal ,&sSystem))
			{
			pSys = &sSystem;
			}
		}

	if(iMode&TP_TOLOCAL)							// Globale in lokale  Zeit umwandeln
		{
		if(SystemTimeToFileTime   (&sSysTime,&sGlobal) &&
		   FileTimeToLocalFileTime(&sGlobal ,&sLocal ) &&
		   FileTimeToSystemTime   (&sLocal,  &sSystem))
			{
			pSys = &sSystem;
			}
		}

	if(iMode&TP_WDAY)
		{
		pTxt[0] = aTpWday[pSys->wDayOfWeek&7][0];
		pTxt[1] = aTpWday[pSys->wDayOfWeek&7][1];
		pTxt[2] = aTpWday[pSys->wDayOfWeek&7][2];
		pTxt+=3;
		}

	if(iMode&TP_DATE)
		{
		if(iMode&TP_DATE_ISO)
			{
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/1000)%10;
				pTxt[1]='0'+(pSys->wYear/100 )%10;
				pTxt[2]='0'+(pSys->wYear/10  )%10;
				pTxt[3]='0'+(pSys->wYear     )%10;
				pTxt[4]='-';
				pTxt+=5;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/10)%10;
				pTxt[1]='0'+(pSys->wYear   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}
			if(iMode&TP_ABC)
				{
				pTxt[0] = aTpMonth[pSys->wMonth&15][0];
				pTxt[1] = aTpMonth[pSys->wMonth&15][1];
				pTxt[2] = aTpMonth[pSys->wMonth&15][2];
				pTxt[3]='-';
				pTxt+=4;
				}
			else{
				pTxt[0]='0'+(pSys->wMonth/10)%10;
				pTxt[1]='0'+(pSys->wMonth   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}

			pTxt[0]='0'+(pSys->wDay/10)%10;
			pTxt[1]='0'+(pSys->wDay   )%10;
			pTxt+=2;
			}
		else{
			pTxt[0]='0'+(pSys->wDay/10)%10;
			pTxt[1]='0'+(pSys->wDay   )%10;
			pTxt[2]='.';
			pTxt+=3;

			if(iMode&TP_ABC)
				{
				pTxt[0] = aTpMonth[pSys->wMonth&15][0];
				pTxt[1] = aTpMonth[pSys->wMonth&15][1];
				pTxt[2] = aTpMonth[pSys->wMonth&15][2];
				pTxt[3] = aTpMonth[pSys->wMonth&15][3];
				pTxt+=4;
				}
			else{
				pTxt[0]='0'+(pSys->wMonth/10)%10;
				pTxt[1]='0'+(pSys->wMonth   )%10;
				pTxt[2]='.';
				pTxt+=3;
				}
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/1000)%10;
				pTxt[1]='0'+(pSys->wYear/100 )%10;
				pTxt[2]='0'+(pSys->wYear/10  )%10;
				pTxt[3]='0'+(pSys->wYear     )%10;
				pTxt+=4;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pSys->wYear/10)%10;
				pTxt[1]='0'+(pSys->wYear   )%10;
				pTxt+=2;
				}
			}

		if(iMode&TP_TIME)
			{
			*(int*)pTxt=0x00200020;
			pTxt+=(iMode&TP_ONE_SPACE)? 1:2;
			}
		}

	if(iMode&TP_TIME)
		{
		pTxt[0]='0'+(pSys->wHour  /10)%10;
		pTxt[1]='0'+(pSys->wHour     )%10;
		pTxt[2]=':';
		pTxt[3]='0'+(pSys->wMinute/10)%10;
		pTxt[4]='0'+(pSys->wMinute   )%10;
		pTxt+=5;

		if(iMode&(TP_TIME&~TP_TIMESHORT))
			{
			pTxt[0]=':';
			pTxt[1]='0'+(pSys->wSecond/10)%10;
			pTxt[2]='0'+(pSys->wSecond   )%10;
			pTxt+=3;
			}

		if(  iMode&(TP_MILLISECEX|TP_MILLISEC))
		if(!(iMode& TP_MILLISECEX) || pSys->wMilliseconds)
			{
			pTxt[0]='.';
			pTxt[1]='0'+(pSys->wMilliseconds/100)%10;
			pTxt[2]='0'+(pSys->wMilliseconds/10 )%10;
			pTxt[3]='0'+(pSys->wMilliseconds    )%10;
			pTxt+=4;
			}
		}

	pTxt[0]=0;



return pTxt-pStr;
}

//*****************************************************************************
//*
//*		TimePrint
//*
//*****************************************************************************
//	Wantelt eine DBTIME Zeit in eienen Zeitstring um.
//	pStr		: Zeiger auf den Textpuffer in den die Zeit gespeichert wird.
//	sSysTime	: Systemzeit die in einen Text umgewandelt werden soll.
//	iMode		: Bestimmt die Art der Umwandlung
//					TP_TIME 		0x000C	Zeit angeben
//					TP_TIMESHORT 	0x0004	Zeit angeben ohne Sekunden
//					TP_ALL   		0x00BE	Alle Flags ohne Millisekunden
//	Ergibt die Lnge des Strings
int TimePrint(char *pStr,DBTIME &sDbTime,int iMode)
{
char	*pTxt=pStr;


	pTxt[0]='0'+(sDbTime.hour  /10)%10;
	pTxt[1]='0'+(sDbTime.hour     )%10;
	pTxt[2]=':';
	pTxt[3]='0'+(sDbTime.minute/10)%10;
	pTxt[4]='0'+(sDbTime.minute   )%10;
	pTxt+=5;

	if(iMode&(TP_TIME&~TP_TIMESHORT))
		{
		pTxt[0]=':';
		pTxt[1]='0'+(sDbTime.second/10)%10;
		pTxt[2]='0'+(sDbTime.second   )%10;
		pTxt+=3;
		}

	pTxt[0]=0;


return pTxt-pStr;
}

int TimePrint(wchar_t *pStr,DBTIME &sDbTime,int iMode)
{
wchar_t	*pTxt=pStr;


	pTxt[0]='0'+(sDbTime.hour  /10)%10;
	pTxt[1]='0'+(sDbTime.hour     )%10;
	pTxt[2]=':';
	pTxt[3]='0'+(sDbTime.minute/10)%10;
	pTxt[4]='0'+(sDbTime.minute   )%10;
	pTxt+=5;

	if(iMode&(TP_TIME&~TP_TIMESHORT))
		{
		pTxt[0]=':';
		pTxt[1]='0'+(sDbTime.second/10)%10;
		pTxt[2]='0'+(sDbTime.second   )%10;
		pTxt+=3;
		}

	pTxt[0]=0;


return pTxt-pStr;
}

//*****************************************************************************
//*
//*		TimePrint
//*
//*****************************************************************************
//	Wantelt eine Zeit in eienen Zeitstring um.
//	pStr		: Zeiger auf den Textpuffer in den die Zeit gespeichert wird.
//	sTmTime		: struct tm Zeit die in einen Text umgewandelt werden soll.
//	iMode		: Bestimmt die Art der Umwandlung
//					TP_MILISEK		0x0001	Millisekunden angeben
//					TP_WDAY		 	0x0002	Wochentag angeben
//					TP_TIME 		0x000C	Zeit angeben
//					TP_TIMESHORT 	0x0004	Zeit angeben ohne Sekunden
//					TP_DATE 		0x0030	Datum angeben mit Jahr 4 stellig
//					TP_DATE2		0x0050	Datum angeben mit Jahr 2 stellig
//					TP_DATE_ISO 	0x0100	Datum angeben mit [JJ]JJ-MM-TT
//					TP_DATESHORT	0x0010	Datum angeben ohne Jahr
//					TP_ONE_SPACE 	0x0200	Nur ein Leerzeichen wischen Datum-Zeit
//					TP_ABC 			0x0080	Monatsnamen ausschreiben
//					TP_ALL   		0x00BE	Alle Flags ohne Millisekunden
//					TP_TOGLOBAL		0x4000	Lokale  in globale Zeit umwandeln
//					TP_TOLOCAL		0x8000	Globale in lokale  Zeit umwandeln
//	Ergibt die Lnge des gespeicherten Textes.
int TimePrint(char *pStr,const TMTIME &sTmTime,int iMode)
{
char				*pTxt=pStr;
const	TMTIME      *pTm=&sTmTime;
		TMTIME	     sTmTemp;



	if(iMode&TP_TOGLOBAL)							// Lokale in globale Zeit umwandeln
		{
		if(TimeConvert(sTmTime,sTmTemp,1,0))
			{
			pTm = &sTmTemp;
			}
		}

	if(iMode&TP_TOLOCAL)							// Globale in lokale  Zeit umwandeln
		{
		if(TimeConvert(sTmTime,sTmTemp,0,1))
			{
			pTm = &sTmTemp;
			}
		}

	if(iMode&TP_WDAY)
		{
		*(int*)pTxt=*(int*)aTpWday[pTm->tm_wday&7];pTxt+=3;
		}

	if(iMode&TP_DATE)
		{
		if(iMode&TP_DATE_ISO)
			{
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+((pTm->tm_year+1900)/1000)%10;
				pTxt[1]='0'+((pTm->tm_year+1900)/100 )%10;
				pTxt[2]='0'+((pTm->tm_year+1900)/10  )%10;
				pTxt[3]='0'+((pTm->tm_year+1900)     )%10;
				pTxt[4]='-';
				pTxt+=5;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pTm->tm_year+1900/10)%10;
				pTxt[1]='0'+(pTm->tm_year+1900   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}
			if(iMode&TP_ABC)
				{
				*(int*)pTxt=*(int*)aTpMonth[pTm->tm_mon+1&15];
				pTxt[3]='-';
				pTxt+=4;
				}
			else{
				pTxt[0]='0'+((pTm->tm_mon+1)/10)%10;
				pTxt[1]='0'+((pTm->tm_mon+1)   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}

			pTxt[0]='0'+(pTm->tm_mday/10)%10;
			pTxt[1]='0'+(pTm->tm_mday   )%10;
			pTxt+=2;
			}
		else{
			pTxt[0]='0'+(pTm->tm_mday/10)%10;
			pTxt[1]='0'+(pTm->tm_mday   )%10;
			pTxt[2]='.';
			pTxt+=3;

			if(iMode&TP_ABC)
				{
				*(int*)pTxt=*(int*)aTpMonth[pTm->tm_mon+1&15];
				pTxt+=4;
				}
			else{
				pTxt[0]='0'+((pTm->tm_mon+1)/10)%10;
				pTxt[1]='0'+((pTm->tm_mon+1)   )%10;
				pTxt[2]='.';
				pTxt+=3;
				}
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+((pTm->tm_year+1900)/1000)%10;
				pTxt[1]='0'+((pTm->tm_year+1900)/100 )%10;
				pTxt[2]='0'+((pTm->tm_year+1900)/10  )%10;
				pTxt[3]='0'+((pTm->tm_year+1900)     )%10;
				pTxt+=4;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[1]='0'+(pTm->tm_year+1900/10)%10;
				pTxt[2]='0'+(pTm->tm_year+1900   )%10;
				pTxt+=2;
				}
			}

		if(iMode&TP_TIME)
			{
			*(short*)pTxt=0x2020;
			pTxt+=(iMode&TP_ONE_SPACE)? 1:2;
			}
		}

	if(iMode&TP_TIME)
		{
		pTxt[0]='0'+(pTm->tm_hour/10)%10;
		pTxt[1]='0'+(pTm->tm_hour   )%10;
		pTxt[2]=':';
		pTxt[3]='0'+(pTm->tm_min /10)%10;
		pTxt[4]='0'+(pTm->tm_min    )%10;
		pTxt+=5;

		if(iMode&(TP_TIME&~TP_TIMESHORT))
			{
			pTxt[0]=':';
			pTxt[1]='0'+(pTm->tm_sec/10)%10;
			pTxt[2]='0'+(pTm->tm_sec   )%10;
			pTxt+=3;
			}

		if(  iMode&(TP_MILLISECEX|TP_MILLISEC))
		if(!(iMode& TP_MILLISECEX) || pTm->tm_yday)
			{
			pTxt[0]='.';
			pTxt[1]='0'+(pTm->tm_yday/100000000)%10;
			pTxt[2]='0'+(pTm->tm_yday/10000000 )%10;
			pTxt[3]='0'+(pTm->tm_yday/1000000  )%10;
			pTxt+=4;
			}
		}

	pTxt[0]=0;



return pTxt-pStr;
}

int TimePrint(wchar_t *pStr,const TMTIME &sTmTime,int iMode)
{
wchar_t				*pTxt=pStr;
const	TMTIME      *pTm=&sTmTime;
		TMTIME	     sTmTemp;



	if(iMode&TP_TOGLOBAL)							// Lokale in globale Zeit umwandeln
		{
		if(TimeConvert(sTmTime,sTmTemp,1,0))
			{
			pTm = &sTmTemp;
			}
		}

	if(iMode&TP_TOLOCAL)							// Globale in lokale  Zeit umwandeln
		{
		if(TimeConvert(sTmTime,sTmTemp,0,1))
			{
			pTm = &sTmTemp;
			}
		}

	if(iMode&TP_WDAY)
		{
		pTxt[0] = aTpWday[pTm->tm_wday&7][0];
		pTxt[1] = aTpWday[pTm->tm_wday&7][1];
		pTxt[2] = aTpWday[pTm->tm_wday&7][2];
		pTxt+=3;
		}


	if(iMode&TP_DATE)
		{
		if(iMode&TP_DATE_ISO)
			{
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+((pTm->tm_year+1900)/1000)%10;
				pTxt[1]='0'+((pTm->tm_year+1900)/100 )%10;
				pTxt[2]='0'+((pTm->tm_year+1900)/10  )%10;
				pTxt[3]='0'+((pTm->tm_year+1900)     )%10;
				pTxt[4]='-';
				pTxt+=5;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[0]='0'+(pTm->tm_year+1900/10)%10;
				pTxt[1]='0'+(pTm->tm_year+1900   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}
			if(iMode&TP_ABC)
				{
				pTxt[0]=aTpMonth[pTm->tm_mon+1&15][0];
				pTxt[1]=aTpMonth[pTm->tm_mon+1&15][1];
				pTxt[2]=aTpMonth[pTm->tm_mon+1&15][2];
				pTxt[3]='-';
				pTxt+=4;
				}
			else{
				pTxt[0]='0'+((pTm->tm_mon+1)/10)%10;
				pTxt[1]='0'+((pTm->tm_mon+1)   )%10;
				pTxt[2]='-';
				pTxt+=3;
				}

			pTxt[0]='0'+(pTm->tm_mday/10)%10;
			pTxt[1]='0'+(pTm->tm_mday   )%10;
			pTxt+=2;
			}
		else{
			pTxt[0]='0'+(pTm->tm_mday/10)%10;
			pTxt[1]='0'+(pTm->tm_mday   )%10;
			pTxt[2]='.';pTxt+=3;

			if(iMode&TP_ABC)
				{
				pTxt[0]=aTpMonth[pTm->tm_mon+1&15][0];
				pTxt[1]=aTpMonth[pTm->tm_mon+1&15][1];
				pTxt[2]=aTpMonth[pTm->tm_mon+1&15][2];
				pTxt[3]=aTpMonth[pTm->tm_mon+1&15][3];
				pTxt+=4;
				}
			else{
				pTxt[0]='0'+((pTm->tm_mon+1)/10)%10;
				pTxt[1]='0'+((pTm->tm_mon+1)   )%10;
				pTxt[2]='.';pTxt+=3;
				}
			if(iMode&(TP_DATE&~TP_DATESHORT))
				{
				pTxt[0]='0'+((pTm->tm_year+1900)/1000)%10;
				pTxt[1]='0'+((pTm->tm_year+1900)/100 )%10;
				pTxt[2]='0'+((pTm->tm_year+1900)/10  )%10;
				pTxt[3]='0'+((pTm->tm_year+1900)     )%10;
				pTxt+=4;
				}
			else if(iMode&(TP_DATE2&~TP_DATESHORT))
				{
				pTxt[1]='0'+(pTm->tm_year+1900/10)%10;
				pTxt[2]='0'+(pTm->tm_year+1900   )%10;
				pTxt+=2;
				}
			}

		if(iMode&TP_TIME)
			{
			*(int*)pTxt=0x00200020;
			pTxt+=(iMode&TP_ONE_SPACE)? 1:2;
			}
		}

	if(iMode&TP_TIME)
		{
		pTxt[0]='0'+(pTm->tm_hour/10)%10;
		pTxt[1]='0'+(pTm->tm_hour   )%10;
		pTxt[2]=':';
		pTxt[3]='0'+(pTm->tm_min /10)%10;
		pTxt[4]='0'+(pTm->tm_min    )%10;
		pTxt+=5;

		if(iMode&(TP_TIME&~TP_TIMESHORT))
			{
			pTxt[0]=':';
			pTxt[1]='0'+(pTm->tm_sec/10)%10;
			pTxt[2]='0'+(pTm->tm_sec   )%10;
			pTxt+=3;
			}

		if(  iMode&(TP_MILLISECEX|TP_MILLISEC))
		if(!(iMode& TP_MILLISECEX) || pTm->tm_yday)
			{
			pTxt[0]='.';
			pTxt[1]='0'+(pTm->tm_yday/100000000)%10;
			pTxt[2]='0'+(pTm->tm_yday/10000000 )%10;
			pTxt[3]='0'+(pTm->tm_yday/1000000  )%10;
			pTxt+=4;
			}
		}

	pTxt[0]=0;



return pTxt-pStr;
}

//*****************************************************************************
//*
//*		TimeConvert	   	time_t --> FILETIME
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,FILETIME &sFileTime)
{
union	{
		__int64	i64;
		DWORD	dw[2];
		}sTime;


	sTime.i64  = iUtcTime;
	sTime.i64 *= 10000000;					// Sekunden zu 100ns umwandeln
	sTime.i64 += 0x019DB1DED53E8000;

	sFileTime.dwHighDateTime = sTime.dw[1];	// Zeit speichern
	sFileTime.dwLowDateTime  = sTime.dw[0];


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   	time_t --> double
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,double &dTime)
{
double	dVal;


	dVal   = (double)iUtcTime;
	dVal  /= 86400.0;						// In Tage umwandeln
	dTime  = dVal + 25569.0;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert		time_t --> struct tm
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,TMTIME &sTmTime,int iLocal)
{
time_t		iTime;
int			iDiff;


	iTime = iUtcTime;

	if(iTime<3600*24*2)
		{
		iDiff  = (iTime-UTC_1980)/UTC_FOUR;
		iTime -=  iDiff*UTC_FOUR;
		}
	else{
		iDiff  = 0;
		}


	if(iLocal)
			sTmTime = *localtime(&iTime);
	else	sTmTime = *gmtime   (&iTime);

	sTmTime.tm_year	+= iDiff*4;
	iInitDone = 1;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   time_t --> SYSTEMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,SYSTEMTIME &sSysTime)
{
FILETIME sFileTime;

	TimeConvert(iUtcTime,sFileTime);

return FileTimeToSystemTime(&sFileTime,&sSysTime);
}

//*****************************************************************************
//*
//*		TimeConvert	   time_t --> DBDATE
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,DBDATE &sDbDate)
{
struct tm *pTm;


	   pTm=gmtime( &iUtcTime );
	if(pTm==NULL)
		{
		sDbDate.day   = 0;
		sDbDate.month = 0;
		sDbDate.year  = 0;
		return FALSE;
		}

	sDbDate.day   = pTm->tm_mday;
	sDbDate.month = pTm->tm_mon+1;
	sDbDate.year  = pTm->tm_year+1900;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   time_t --> DBTIMESTAMP
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,DBTIMESTAMP &sDbStamp)
{
struct tm *pTm;


	   pTm=gmtime(&iUtcTime);
	if(pTm==NULL)
		{
		sDbStamp.fraction = 0;
		sDbStamp.second   = 0;
		sDbStamp.minute   = 0;
		sDbStamp.hour     = 0;
		sDbStamp.day      = 0;
		sDbStamp.month    = 0;
		sDbStamp.year     = 0;
		return FALSE;
		}

	sDbStamp.fraction = 0;
	sDbStamp.second   = pTm->tm_sec;
	sDbStamp.minute   = pTm->tm_min;
	sDbStamp.hour     = pTm->tm_hour;
	sDbStamp.day      = pTm->tm_mday;
	sDbStamp.month    = pTm->tm_mon+1;
	sDbStamp.year     = pTm->tm_year+1900;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   time_t --> DOSTIME
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,DOSTIME &sDosTime)
{
struct tm *pTm;


	   pTm=gmtime( &iUtcTime );
	if(pTm==NULL || pTm->tm_year<80 || pTm->tm_year>143)
		{
		sDosTime.wDate = 0;
		sDosTime.wTime = 0;
		return FALSE;
		}

	sDosTime.wDate = (pTm->tm_mday  ) | ((pTm->tm_mon+1)<<5) | ((pTm->tm_year-80)<< 9);
	sDosTime.wTime = (pTm->tm_sec>>1) | ( pTm->tm_min   <<5) | ( pTm->tm_hour    <<11);


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   time_t --> DBTIME
//*
//*****************************************************************************
BOOL TimeConvert(const time_t iUtcTime,DBTIME &sDbTime)
{
time_t	iTime=iUtcTime;


	sDbTime.second =  iTime       % 60;
	sDbTime.minute = (iTime/  60) % 60;
	sDbTime.hour   = (iTime/3600) % 24;


return TRUE;
}


//*****************************************************************************
//*
//*		TimeConvert	   FILETIME	--> time_t
//*
//*****************************************************************************
BOOL TimeConvert(const FILETIME &sFileTime,time_t &iUtcTime)
{
union	{
		__int64	i64;
		DWORD	dw[2];
		}sTime;


	sTime.i64  = *(__int64*)&sFileTime;
	sTime.i64 -= 0x019DB1DED53E8000;

	if(sTime.dw[1]&0x80000000)				// Zeit vor Mi 1.Jan.1970
		{
		iUtcTime=0;
		return FALSE;
		}

	sTime.i64 /= 10000000;					// 100ns zu Sekunden umwandeln

	if(sizeof(time_t)==4)
		{									// ber Jan.2038
		if(sTime.dw[1] || (sTime.dw[0]&0x80000000))
			{
			iUtcTime = 0x7FFFFFFF;
			return FALSE;
			}

		iUtcTime = (time_t)sTime.dw[0];		// Unix Zeit speichern
		}
	else{
		iUtcTime = (time_t)sTime.i64;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   FILETIME	--> double
//*
//*****************************************************************************
BOOL TimeConvert(const FILETIME &sFileTime,double &dTime)
{
__int64	iTime;
double	dVal;


	iTime  = *(__int64*)&sFileTime;
	iTime -= 0x019DB1DED53E8000;			// Auf 1.Jan.1970 schieben

	dVal   = (double)iTime;
	dVal  /= 864000000000.0;				// In Tage umwandeln
	dTime  = dVal + 25569.0;				// Auf 1.Jan.1900 schieben

return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   FILETIME --> DBDATE
//*
//*****************************************************************************
BOOL TimeConvert(const FILETIME &sFileTime,DBDATE &sDbDate)
{
SYSTEMTIME  sSysTime;

	if(!FileTimeToSystemTime(&sFileTime,&sSysTime))
		{
		sDbDate.day   = 0;
		sDbDate.month = 0;
		sDbDate.year  = 0;
		return FALSE;
		}

	sDbDate.day   = sSysTime.wDay;
	sDbDate.month = sSysTime.wMonth;
	sDbDate.year  = sSysTime.wYear;

return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   sFileTime --> DBTIME
//*
//*****************************************************************************
BOOL TimeConvert(const FILETIME &sFileTime,DBTIME &sDbTime)
{
SYSTEMTIME  sSysTime;


	if(!FileTimeToSystemTime(&sFileTime,&sSysTime))
		{
		sDbTime.hour   = 0;
		sDbTime.minute = 0;
		sDbTime.second = 0;
		return FALSE;
		}

	sDbTime.hour   = sSysTime.wHour;
	sDbTime.minute = sSysTime.wMinute;
	sDbTime.second = sSysTime.wSecond;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   FILETIME --> DBTIMESTAMP
//*
//*****************************************************************************
BOOL TimeConvert(const FILETIME &sFileTime,DBTIMESTAMP &sDbStamp)
{
SYSTEMTIME  sSysTime;

	if(!FileTimeToSystemTime(&sFileTime,&sSysTime))
		{
		sDbStamp.fraction = 0;
		sDbStamp.second   = 0;
		sDbStamp.minute   = 0;
		sDbStamp.hour     = 0;
		sDbStamp.day      = 0;
		sDbStamp.month    = 0;
		sDbStamp.year     = 0;
		return FALSE;
		}

	sDbStamp.fraction = sSysTime.wMilliseconds*1000000;
	sDbStamp.second   = sSysTime.wSecond;
	sDbStamp.minute   = sSysTime.wMinute;
	sDbStamp.hour     = sSysTime.wHour;
	sDbStamp.day      = sSysTime.wDay;
	sDbStamp.month    = sSysTime.wMonth;
	sDbStamp.year     = sSysTime.wYear;

return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   FILETIME --> TMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const FILETIME &sFileTime,TMTIME &sTmTime,int iLocal)
{
__int64		iLong;
time_t		iTime;
int			iDiff;
BOOL		bOk;


	iLong = *(__int64*)(&sFileTime);

	if(iLong<FTG_1980 || iLong>FTG_2030)
		{
		iDiff  = (int)((iLong-FTG_1980)/FTG_FOUR);
		iLong -= (__int64)iDiff*FTG_FOUR;
		}
	else{
		iDiff  = 0;
		}


	if(!TimeConvert(*(FILETIME*)&iLong,iTime))
		{
		memset(&sTmTime,0,sizeof(sTmTime));
		return FALSE;
		}


	bOk = TimeConvert(iTime,sTmTime,iLocal);

	sTmTime.tm_year	+= iDiff*4;


return bOk;
}

//*****************************************************************************
//*
//*		TimeConvert	   double --> time_t
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,time_t &iUtcTime)
{
double	dVal;


	dVal  = dTime;

	if(dVal>=1.0 || dVal<0.0)
		{
		dVal -= 25569.0;
		}

	if(dVal<0.0)							// Zeit vor Mi 1.Jan.1970
		{
		iUtcTime=0;
		return FALSE;
		}

	dVal *= 86400.0;						// Tage zu Sekunden umwandeln

	if(sizeof(time_t)==4)
		{
		if(dVal>2147483647.0)				// ber Jahr 2038
			{
			iUtcTime=0x7FFFFFFF;
			return FALSE;
			}

		iUtcTime = (unsigned)dVal;			// Unix Zeit speichern
		}
	else{
		iUtcTime = (time_t)((__int64)dVal);	// Unix Zeit speichern
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   double --> FILETIME
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,FILETIME &sFileTime)
{
double	dVal;
__int64	iTime;


	dVal    = dTime;
	dVal   -= 25569.0;
	dVal   *= 864000000000.0;				// Tage zu 100ns umwandeln
	iTime   = (__int64)(dVal);

	iTime  += 0x019DB1DED53E8000;			// Auf 1.Jan.1601 schieben
	*(__int64*)&sFileTime = iTime;


return TRUE;
}


//*****************************************************************************
//*
//*		TimeConvert	   double --> DBDATE
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,DBDATE &sDbDate)
{
FILETIME	sFileTime;

	TimeConvert(dTime,sFileTime);

return TimeConvert(sFileTime,sDbDate);
}

//*****************************************************************************
//*
//*		TimeConvert	   double --> DBTIMESTAMP
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,DBTIMESTAMP &sDbStamp)
{
FILETIME	sFileTime;

	TimeConvert(dTime,sFileTime);

return TimeConvert(sFileTime,sDbStamp);
}

//*****************************************************************************
//*
//*		TimeConvert	   double --> SYSTEMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,SYSTEMTIME &sSysTime)
{
double	dTmp;
double	dVal;
BOOL	bOk;


	dVal = modf(dTime*86400.0,&dTmp);
	bOk  = VariantTimeToSystemTime((dTmp+0.3)/86400.0,&sSysTime);

	sSysTime.wMilliseconds = (WORD)(dVal*1000.0+0.3);

	if(bOk)return TRUE;

	memset(&sSysTime,0,sizeof(sSysTime));


return FALSE;
}

//*****************************************************************************
//*
//*		TimeConvert	   double --> DOSTIME
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,DOSTIME &sDosTime)
{
SYSTEMTIME	sSysTime;
BOOL		bOk;
double		dTmp;
double		dVal;


	dVal = modf(dTime*86400.0,&dTmp);
	bOk  = VariantTimeToSystemTime((dTmp+0.3)/86400.0,&sSysTime);

	if(!bOk || sSysTime.wYear<1980 || sSysTime.wYear>2043)
		{
		sDosTime.wDate = 0;
		sDosTime.wTime = 0;
		return FALSE;
		}

	sDosTime.wDate = (sSysTime.wDay      ) | (sSysTime.wMonth <<5) | ((sSysTime.wYear-1980)<< 9);
	sDosTime.wTime = (sSysTime.wSecond>>1) | (sSysTime.wMinute<<5) | ( sSysTime.wHour      <<11);


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   double --> DBTIME
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,DBTIME &sDbTime)
{
double	dTmp,dVal;


	dVal  = modf(dTime,&dTmp);
	dVal *= 24.0;
	sDbTime.hour   = (WORD)dVal;
	dVal -= sDbTime.hour;
	dVal *= 60.0;
	sDbTime.minute = (WORD)dVal;
	dVal -= sDbTime.minute;
	dVal *= 60.0;
	sDbTime.second = (WORD)dVal;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   double --> TMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const double &dTime,TMTIME &sTmTime,int iLocal)
{
double		dLong;
time_t		iTime;
int			iDiff;
BOOL		bOk;


	dLong = dTime;

	if(dLong<OLE_1980 || dLong>OLE_2030)
		{
		iDiff  = (int)((dLong-OLE_1980)/OLE_FOUR);
		dLong -= (double)iDiff*OLE_FOUR;
		}
	else{
		iDiff  = 0;
		}


	if(!TimeConvert(dLong,iTime))
		{
		memset(&sTmTime,0,sizeof(sTmTime));
		return FALSE;
		}


	bOk = TimeConvert(iTime,sTmTime,iLocal);

	sTmTime.tm_year	+= iDiff*4;


return bOk;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBDATE --> SYSTEMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBDATE &sDbDate,SYSTEMTIME &sSysTime)
{
struct tm	sTm;


	sTm.tm_isdst = 0;
	sTm.tm_sec   = 0;
	sTm.tm_min   = 0;
	sTm.tm_hour  = 0;
	sTm.tm_mday  = sDbDate.day;
	sTm.tm_mon   = sDbDate.month - 1;
	sTm.tm_year  = sDbDate.year  - 1900;

	mktime(&sTm);

	sSysTime.wMilliseconds = 0;
	sSysTime.wMinute	   = 0;
	sSysTime.wSecond	   = 0;
	sSysTime.wHour	   	   = 0;
	sSysTime.wDayOfWeek    = sTm.tm_wday;
	sSysTime.wDay          = sDbDate.day;
	sSysTime.wMonth        = sDbDate.month;
	sSysTime.wYear         = sDbDate.year;

	if(sDbDate.month <=0)return FALSE;
	if(sDbDate.month >12)return FALSE;
	if(sDbDate.day   <=0)return FALSE;
	if(sDbDate.day   >aTpMDays[sDbDate.month-1])
		{
		if(sDbDate.month!= 2)return FALSE;
		if(sDbDate.day  !=29)return FALSE;
		if(sDbDate.year   &3)return FALSE;
		}



return TRUE;
}


//*****************************************************************************
//*
//*		TimeConvert	   DBDATE --> time_t
//*
//*****************************************************************************
BOOL TimeConvert(const DBDATE &sDbDate,time_t &iUtcTime)
{
time_t	iYear,iDays;


	   iYear = sDbDate.year;
	if(iYear<1970 || iYear>2038)
		{
		iUtcTime=0xFFFFFFFF;
		return FALSE;
		}

	   iYear -= 1900;
	if(iYear&3)iDays  = aTpDays  [(sDbDate.month-1)&15];
	else	   iDays  = aTpPDays [(sDbDate.month-1)&15];
			   iDays += sDbDate.day-1;
			   iDays += (iYear-70  )*365;
			   iDays += (iYear-70+1)>>2;

	iUtcTime= iDays*24*3600;

	if(sDbDate.month <=0)return FALSE;
	if(sDbDate.month >12)return FALSE;
	if(sDbDate.day   <=0)return FALSE;
	if(sDbDate.day   >aTpMDays[sDbDate.month-1])
		{
		if(sDbDate.month!= 2)return FALSE;
		if(sDbDate.day  !=29)return FALSE;
		if(sDbDate.year   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBDATE --> FILETIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBDATE &sDbDate,FILETIME &sFileTime)
{
SYSTEMTIME sSysTime;


	sSysTime.wMilliseconds = 0;
	sSysTime.wMinute	   = 0;
	sSysTime.wSecond	   = 0;
	sSysTime.wHour	   	   = 0;
	sSysTime.wDayOfWeek    = 0;
	sSysTime.wDay          = sDbDate.day;
	sSysTime.wMonth        = sDbDate.month;
	sSysTime.wYear         = sDbDate.year;

	if(SystemTimeToFileTime(&sSysTime,&sFileTime))
		{
		return TRUE;
		}

	sFileTime.dwHighDateTime = 0;
	sFileTime.dwLowDateTime  = 0;


return FALSE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBDATE --> double
//*
//*****************************************************************************
BOOL TimeConvert(const DBDATE &sDbDate,double &dTime)
{
SYSTEMTIME sSysTime;


	sSysTime.wMilliseconds = 0;
	sSysTime.wMinute	   = 0;
	sSysTime.wSecond	   = 0;
	sSysTime.wHour	   	   = 0;
	sSysTime.wDayOfWeek    = 0;
	sSysTime.wDay          = sDbDate.day;
	sSysTime.wMonth        = sDbDate.month;
	sSysTime.wYear         = sDbDate.year;


	if(sSysTime.wMonth < 1 || sSysTime.wMonth >12 ||
	   sSysTime.wDay   < 1 || sSysTime.wDay   >31)
		{
		dTime  = 0.0;
		return FALSE;
		}

	if(SystemTimeToVariantTime(&sSysTime,&dTime))
		{
		return TRUE;
		}


	dTime = 0.0;


return FALSE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBDATE --> DBTIMESTAMP
//*
//*****************************************************************************
BOOL TimeConvert(const DBDATE &sDbDate,DBTIMESTAMP &sDbStamp)
{

	sDbStamp.fraction = 0;
	sDbStamp.second   = 0;
	sDbStamp.minute   = 0;
	sDbStamp.hour     = 0;
	sDbStamp.day      = sDbDate.day;
	sDbStamp.month    = sDbDate.month;
	sDbStamp.year     = sDbDate.year;


	if(sDbDate.month <=0)return FALSE;
	if(sDbDate.month >12)return FALSE;
	if(sDbDate.day   <=0)return FALSE;
	if(sDbDate.day   >aTpMDays[sDbDate.month-1])
		{
		if(sDbDate.month!= 2)return FALSE;
		if(sDbDate.day  !=29)return FALSE;
		if(sDbDate.year   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBDATE --> DOSTIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBDATE &sDbDate,DOSTIME &sDosTime)
{

	if(sDbDate.year<1980 || sDbDate.year>2043)
		{
		sDosTime.wDate = 0;
		sDosTime.wTime = 0;
		return FALSE;
		}


	sDosTime.wDate = (sDbDate.day) | (sDbDate.month<<5) | ((sDbDate.year-1980)<<9);
	sDosTime.wTime = 0;


	if(sDbDate.month <=0)return FALSE;
	if(sDbDate.month >12)return FALSE;
	if(sDbDate.day   <=0)return FALSE;
	if(sDbDate.day   >aTpMDays[sDbDate.month-1])
		{
		if(sDbDate.month!= 2)return FALSE;
		if(sDbDate.day  !=29)return FALSE;
		if(sDbDate.year   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert		SYSTEMTIME --> SYSTEMTIME : iMode
//*
//*****************************************************************************
//	iMode	: Ist der Verarbeitungsmodus
//				0 = Eins zu eins
//				1 = Lokal zu Global
//				2 = Global zu Lokal
BOOL TimeConvert(const SYSTEMTIME &sSysTime,SYSTEMTIME &sSysTimeX,int iMode)
{
FILETIME	sGlobal;
FILETIME	sLocal;


	switch(iMode)
		{
	case 1:	if(!SystemTimeToFileTime   (&sSysTime,&sLocal   ))break;
			if(!LocalFileTimeToFileTime(&sLocal  ,&sGlobal  ))break;
			if(!FileTimeToSystemTime   (&sGlobal ,&sSysTimeX))break;

			return TRUE;

	case 2:	if(!SystemTimeToFileTime   (&sSysTime,&sGlobal  ))break;
			if(!FileTimeToLocalFileTime(&sGlobal ,&sLocal   ))break;
			if(!FileTimeToSystemTime   (&sGlobal ,&sSysTimeX))break;

			return TRUE;
		}


	memmove(&sSysTimeX,&sSysTime,sizeof(sSysTimeX));


return (iMode==0)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		TimeConvert	   SYSTEMTIME --> double
//*
//*****************************************************************************
BOOL TimeConvert(const SYSTEMTIME &sSysTime,double &dTime)
{
BOOL		bOk;


	if(sSysTime.wMonth < 1 || sSysTime.wMonth >12 ||
	   sSysTime.wDay   < 1 || sSysTime.wDay   >31 ||
	   sSysTime.wMinute>59 || sSysTime.wSecond>59 ||
	   sSysTime.wHour  >23)
		{
		dTime  = 0.0;
		return FALSE;
		}

	bOk = SystemTimeToVariantTime((SYSTEMTIME*)&sSysTime,&dTime);

	if(bOk)
		{
		if(sSysTime.wMilliseconds<1000)
			{
			dTime += (double)sSysTime.wMilliseconds/86400000.0;
			}
		}
	else{
		dTime  = 0.0;
		}


return bOk;
}

//*****************************************************************************
//*
//*		TimeConvert	   SYSTEMTIME --> time_t
//*
//*****************************************************************************
BOOL TimeConvert(const SYSTEMTIME &sSysTime,time_t &iUtcTime)
{
FILETIME sFileTime;


	if(!SystemTimeToFileTime(&sSysTime,&sFileTime))
		{
		iUtcTime = 0;
		return FALSE;
		}


return TimeConvert(sFileTime,iUtcTime);
}


//*****************************************************************************
//*
//*		TimeConvert	   SYSEMTIME --> DBTIMESTAMP
//*
//*****************************************************************************
BOOL TimeConvert(const SYSTEMTIME &sSysTime,DBTIMESTAMP &sDbStamp)
{

	sDbStamp.fraction = sSysTime.wMilliseconds*1000000;
	sDbStamp.second   = sSysTime.wSecond;
	sDbStamp.minute   = sSysTime.wMinute;
	sDbStamp.hour     = sSysTime.wHour;
	sDbStamp.day      = sSysTime.wDay;
	sDbStamp.month    = sSysTime.wMonth;
	sDbStamp.year     = sSysTime.wYear;

	if(sSysTime.wMilliseconds>999)return FALSE;
	if(sSysTime.wSecond>59)return FALSE;
	if(sSysTime.wMinute>59)return FALSE;
	if(sSysTime.wHour  >23)return FALSE;
	if(sSysTime.wMonth <=0)return FALSE;
	if(sSysTime.wMonth >12)return FALSE;
	if(sSysTime.wDay   <=0)return FALSE;
	if(sSysTime.wDay   >aTpMDays[sSysTime.wMonth-1])
		{
		if(sSysTime.wMonth!= 2)return FALSE;
		if(sSysTime.wDay  !=29)return FALSE;
		if(sSysTime.wYear   &3)return FALSE;
		}

return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   SYSTEMTIME --> DOSTIME
//*
//*****************************************************************************
BOOL TimeConvert(const SYSTEMTIME &sSysTime,DOSTIME &sDosTime)
{

	if(sSysTime.wYear<1980 || sSysTime.wYear>2043)
		{
		sDosTime.wDate = 0;
		sDosTime.wTime = 0;
		return FALSE;
		}

	sDosTime.wDate = (sSysTime.wDay      ) | (sSysTime.wMonth <<5) | ((sSysTime.wYear-1980)<< 9);
	sDosTime.wTime = (sSysTime.wSecond>>1) | (sSysTime.wMinute<<5) | ( sSysTime.wHour      <<11);


	if(sSysTime.wSecond>59)return FALSE;
	if(sSysTime.wMinute>59)return FALSE;
	if(sSysTime.wHour  >23)return FALSE;
	if(sSysTime.wMonth <=0)return FALSE;
	if(sSysTime.wMonth >12)return FALSE;
	if(sSysTime.wDay   <=0)return FALSE;
	if(sSysTime.wDay   >aTpMDays[sSysTime.wMonth-1])
		{
		if(sSysTime.wMonth!= 2)return FALSE;
		if(sSysTime.wDay  !=29)return FALSE;
		if(sSysTime.wYear   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   SYSTEMTIME --> TMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const SYSTEMTIME  &sSysTime,TMTIME &sTmTime,int iLocal)
{
FILETIME	iTime;


	if(!iLocal)
		{
		sTmTime.tm_isdst	= 0;
		sTmTime.tm_sec		= sSysTime.wSecond;
		sTmTime.tm_min		= sSysTime.wMinute;
		sTmTime.tm_hour		= sSysTime.wHour;
		sTmTime.tm_mday		= sSysTime.wDay;
		sTmTime.tm_mon		= sSysTime.wMonth-1;
		sTmTime.tm_year		= sSysTime.wYear-1900;

		mktime(&sTmTime);

		return TRUE;
		}

	if(!TimeConvert(sSysTime,iTime))
		{
		memset(&sTmTime,0,sizeof(sTmTime));
		return FALSE;
		}


return TimeConvert(iTime,sTmTime,iLocal);
}

//*****************************************************************************
//*
//*		TimeConvert		DBTIMESTAMP --> DBTIMESTAMP : iMode
//*
//*****************************************************************************
//	iMode	: Ist der Verarbeitungsmodus
//				0 = Eins zu eins
//				1 = Lokal zu Global
//				2 = Global zu Lokal
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp,DBTIMESTAMP &sDbStampX,int iMode)
{
SYSTEMTIME	sSysTime;
FILETIME	sGlobal;
FILETIME	sLocal;


	switch(iMode)
		{
	case 1:	sSysTime.wSecond	   = sDbStamp.second;
			sSysTime.wMinute	   = sDbStamp.minute;
			sSysTime.wHour	   	   = sDbStamp.hour;
			sSysTime.wDay          = sDbStamp.day;
			sSysTime.wMonth        = sDbStamp.month;
			sSysTime.wYear         = sDbStamp.year;
			sSysTime.wDayOfWeek    = 0;
			sSysTime.wMilliseconds = 0;

			if(!SystemTimeToFileTime   (&sSysTime,&sLocal  ))break;
			if(!LocalFileTimeToFileTime(&sLocal  ,&sGlobal ))break;
			if(!FileTimeToSystemTime   (&sGlobal ,&sSysTime))break;

			sDbStampX.second	   = sSysTime.wSecond;
			sDbStampX.minute	   = sSysTime.wMinute;
			sDbStampX.hour		   = sSysTime.wHour;
			sDbStampX.day		   = sSysTime.wDay;
			sDbStampX.month		   = sSysTime.wMonth;
			sDbStampX.year		   = sSysTime.wYear;
			sDbStampX.fraction	   = sDbStamp.fraction;

			return TRUE;

	case 2:	sSysTime.wSecond	   = sDbStamp.second;
			sSysTime.wMinute	   = sDbStamp.minute;
			sSysTime.wHour	   	   = sDbStamp.hour;
			sSysTime.wDay          = sDbStamp.day;
			sSysTime.wMonth        = sDbStamp.month;
			sSysTime.wYear         = sDbStamp.year;
			sSysTime.wDayOfWeek    = 0;
			sSysTime.wMilliseconds = 0;

			if(!SystemTimeToFileTime   (&sSysTime,&sGlobal ))break;
			if(!FileTimeToLocalFileTime(&sGlobal ,&sLocal  ))break;
			if(!FileTimeToSystemTime   (&sGlobal ,&sSysTime))break;

			sDbStampX.second	   = sSysTime.wSecond;
			sDbStampX.minute	   = sSysTime.wMinute;
			sDbStampX.hour		   = sSysTime.wHour;
			sDbStampX.day		   = sSysTime.wDay;
			sDbStampX.month		   = sSysTime.wMonth;
			sDbStampX.year		   = sSysTime.wYear;
			sDbStampX.fraction	   = sDbStamp.fraction;

			return TRUE;
		}


	memmove(&sDbStampX,&sDbStamp,sizeof(sDbStampX));


return (iMode==0)? TRUE:FALSE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> SYSTEMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp,SYSTEMTIME &sSysTime)
{
struct tm sTm;


	sTm.tm_isdst = 0;
	sTm.tm_sec   = 0;
	sTm.tm_min   = 0;
	sTm.tm_hour  = 0;
	sTm.tm_mday  = sDbStamp.day;
	sTm.tm_mon   = sDbStamp.month - 1;
	sTm.tm_year  = sDbStamp.year - 1900;

	mktime(&sTm);

	sSysTime.wMilliseconds = (WORD)(sDbStamp.fraction/1000000);
	sSysTime.wSecond	   = sDbStamp.second;
	sSysTime.wMinute	   = sDbStamp.minute;
	sSysTime.wHour	   	   = sDbStamp.hour;
	sSysTime.wDayOfWeek    = sTm.tm_wday;
	sSysTime.wDay          = sDbStamp.day;
	sSysTime.wMonth        = sDbStamp.month;
	sSysTime.wYear         = sDbStamp.year;


	if(sSysTime.wMilliseconds>999)return FALSE;
	if(sSysTime.wSecond>59)return FALSE;
	if(sSysTime.wMinute>59)return FALSE;
	if(sSysTime.wHour  >23)return FALSE;
	if(sSysTime.wMonth <=0)return FALSE;
	if(sSysTime.wMonth >12)return FALSE;
	if(sSysTime.wDay   <=0)return FALSE;
	if(sSysTime.wDay   >aTpMDays[sSysTime.wMonth-1])
		{
		if(sSysTime.wMonth!= 2)return FALSE;
		if(sSysTime.wDay  !=29)return FALSE;
		if(sSysTime.wYear   &3)return FALSE;
		}



return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> time_t
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp,time_t &iUtcTime)
{
time_t	iTime,iYear,iDays;


	   iYear = sDbStamp.year;
	if(iYear<1970 || iYear>2038)
		{
		iUtcTime=0xFFFFFFFF;
		return FALSE;
		}

	   iYear -= 1900;
	if(iYear&3)iDays  = aTpDays  [(sDbStamp.month-1)&15];
	else	   iDays  = aTpPDays [(sDbStamp.month-1)&15];
			   iDays += sDbStamp.day-1;
			   iDays += (iYear-70  )*365;
			   iDays += (iYear-70+1)>>2;

			   iTime  = iDays*24;
			   iTime += sDbStamp.hour;
			   iTime *= 60;
			   iTime += sDbStamp.minute;
			   iTime *= 60;
			   iTime += sDbStamp.second;
	iUtcTime = iTime;


	if(sDbStamp.second>59)return FALSE;
	if(sDbStamp.minute>59)return FALSE;
	if(sDbStamp.hour  >23)return FALSE;
	if(sDbStamp.month <=0)return FALSE;
	if(sDbStamp.month >12)return FALSE;
	if(sDbStamp.day   <=0)return FALSE;
	if(sDbStamp.day   >aTpMDays[sDbStamp.month-1])
		{
		if(sDbStamp.month!= 2)return FALSE;
		if(sDbStamp.day  !=29)return FALSE;
		if(sDbStamp.year   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> FILETIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp,FILETIME &sFileTime)
{
SYSTEMTIME sSysTime;


	sSysTime.wMilliseconds = (WORD)(sDbStamp.fraction/1000000);
	sSysTime.wSecond	   = sDbStamp.second;
	sSysTime.wMinute	   = sDbStamp.minute;
	sSysTime.wHour	   	   = sDbStamp.hour;
	sSysTime.wDayOfWeek    = 0;
	sSysTime.wDay          = sDbStamp.day;
	sSysTime.wMonth        = sDbStamp.month;
	sSysTime.wYear         = sDbStamp.year;


	if(SystemTimeToFileTime(&sSysTime,&sFileTime))
		{
		return TRUE;
		}

	sFileTime.dwHighDateTime = 0;
	sFileTime.dwLowDateTime  = 0;


return FALSE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> double
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp,double &dTime)
{
BOOL		bOk;
SYSTEMTIME	sSysTime;


	sSysTime.wMilliseconds = 0;
	sSysTime.wSecond	   = sDbStamp.second;
	sSysTime.wMinute	   = sDbStamp.minute;
	sSysTime.wHour	   	   = sDbStamp.hour;
	sSysTime.wDayOfWeek    = 0;
	sSysTime.wDay          = sDbStamp.day;
	sSysTime.wMonth        = sDbStamp.month;
	sSysTime.wYear         = sDbStamp.year;

	if(sSysTime.wMonth < 1 || sSysTime.wMonth >12 ||
	   sSysTime.wDay   < 1 || sSysTime.wDay   >31 ||
	   sSysTime.wMinute>59 || sSysTime.wSecond>59 ||
	   sSysTime.wHour  >23)
		{
		dTime  = 0.0;
		return FALSE;
		}


	bOk = SystemTimeToVariantTime(&sSysTime,&dTime);

	if(bOk)
		{
		if(sDbStamp.fraction<1000000000)
			{
			dTime += (double)sDbStamp.fraction/86400000000000.0;
			}
		}
	else{
		dTime  = 0.0;
		}


return bOk;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> DBDATE
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp ,DBDATE &sDbDate)
{

	sDbDate.day   = sDbStamp.day;
	sDbDate.month = sDbStamp.month;
	sDbDate.year  = sDbStamp.year;


	if(sDbStamp.month <=0)return FALSE;
	if(sDbStamp.month >12)return FALSE;
	if(sDbStamp.day   <=0)return FALSE;
	if(sDbStamp.day   >aTpMDays[sDbStamp.month-1])
		{
		if(sDbStamp.month!= 2)return FALSE;
		if(sDbStamp.day  !=29)return FALSE;
		if(sDbStamp.year   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> DOSTIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp,DOSTIME &sDosTime)
{

	if(sDbStamp.year<1980 || sDbStamp.year>2043)
		{
		sDosTime.wDate = 0;
		sDosTime.wTime = 0;
		return FALSE;
		}

	sDosTime.wDate = (sDbStamp.day      ) | (sDbStamp.month <<5) | ((sDbStamp.year-1980)<< 9);
	sDosTime.wTime = (sDbStamp.second>>1) | (sDbStamp.minute<<5) | ( sDbStamp.hour      <<11);

return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> DBTIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp ,DBTIME &sDbTime)
{

	sDbTime.hour	= sDbStamp.hour;
	sDbTime.minute	= sDbStamp.minute;
	sDbTime.second	= sDbStamp.second;

	if(sDbTime.second>59)return FALSE;
	if(sDbTime.minute>59)return FALSE;
	if(sDbTime.hour  >23)return FALSE;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIMESTAMP --> TMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIMESTAMP &sDbStamp,TMTIME &sTmTime,int iLocal)
{
FILETIME	iTime;


	if(!iLocal)
		{
		sTmTime.tm_isdst	= 0;
		sTmTime.tm_sec		= sDbStamp.second;
		sTmTime.tm_min		= sDbStamp.minute;
		sTmTime.tm_hour		= sDbStamp.hour;
		sTmTime.tm_mday		= sDbStamp.day;
		sTmTime.tm_mon		= sDbStamp.month-1;
		sTmTime.tm_year		= sDbStamp.year-1900;

		mktime(&sTmTime);

		return TRUE;
		}

	if(!TimeConvert(sDbStamp,iTime))
		{
		memset(&sTmTime,0,sizeof(sTmTime));
		return FALSE;
		}


return TimeConvert(iTime,sTmTime,iLocal);
}

//*****************************************************************************
//*
//*		TimeConvert	   DOSTIME --> SYSTEMTIME
//*
//*****************************************************************************
BOOL TimeConvert(const DOSTIME &sDosTime,SYSTEMTIME &sSysTime)
{
struct tm sTm;

	sSysTime.wDay          = (sDosTime.wDate   )&0x1F;
	sSysTime.wMonth        = (sDosTime.wDate>>5)&0x0F;
	sSysTime.wYear         = (sDosTime.wDate>>9)+1980;

	sTm.tm_isdst = 0;
	sTm.tm_sec   = 0;
	sTm.tm_min   = 0;
	sTm.tm_hour  = 0;
	sTm.tm_mday  = sSysTime.wDay;
	sTm.tm_mon   = sSysTime.wMonth - 1;
	sTm.tm_year  = sSysTime.wYear  - 1900;

	mktime(&sTm);

	sSysTime.wMilliseconds = 0;
	sSysTime.wSecond	   = (sDosTime.wTime<< 1)&0x3F;
	sSysTime.wMinute	   = (sDosTime.wTime>> 5)&0x3F;
	sSysTime.wHour	   	   = (sDosTime.wTime>>11);
	sSysTime.wDayOfWeek    = sTm.tm_wday;

	if(sSysTime.wSecond>59)return FALSE;
	if(sSysTime.wMinute>59)return FALSE;
	if(sSysTime.wHour  >23)return FALSE;
	if(sSysTime.wMonth <=0)return FALSE;
	if(sSysTime.wMonth >12)return FALSE;
	if(sSysTime.wDay   <=0)return FALSE;
	if(sSysTime.wDay   >aTpMDays[sSysTime.wMonth-1])
		{
		if(sSysTime.wMonth!= 2)return FALSE;
		if(sSysTime.wDay  !=29)return FALSE;
		if(sSysTime.wYear   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DOSTIME --> time_t
//*
//*****************************************************************************
BOOL TimeConvert(const DOSTIME &sDosTime,time_t &iUtcTime)
{
time_t		iTime;
int			iYear;
int			iMonth;
int			iHour;
int			iDays;
int			iDay;
int			iSec;
int			iMin;


	   iYear = (sDosTime.wDate>>9)+80;
	if(iYear>138)
		{
		iUtcTime=0xFFFFFFFF;
		return FALSE;
		}

	iHour	= ((sDosTime.wTime>>11));
	iDay	= ((sDosTime.wDate    )-1)&0x1F;
	iMonth	= ((sDosTime.wDate>> 5)-1)&0x0F;
	iMin	= ((sDosTime.wTime>> 5)  )&0x3F;
	iSec	= ((sDosTime.wTime<< 1)  )&0x3F;

	if(iYear&3)iDays  = aTpDays  [iMonth];
	else	   iDays  = aTpPDays [iMonth];
			   iDays += iDay;
			   iDays += (iYear-70  )*365;
			   iDays += (iYear-70+1)>>2;

			   iTime  = iDays*24;
			   iTime += iHour;
			   iTime *= 60;
			   iTime += iMin;
			   iTime *= 60;
			   iTime += iSec;
	iUtcTime = iTime;


	if(iSec  >59)return FALSE;
	if(iMin  >59)return FALSE;
	if(iHour >23)return FALSE;
	if(iMonth<=0)return FALSE;
	if(iMonth>12)return FALSE;
	if(iDay  <=0)return FALSE;
	if(iDay>aTpMDays[iMonth-1])
		{
		if(iMonth!= 2)return FALSE;
		if(iDay  !=29)return FALSE;
		if(iYear   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DOSTIME --> double
//*
//*****************************************************************************
BOOL TimeConvert(const DOSTIME &sDosTime ,double &Time)
{
SYSTEMTIME sSysTime;

	if(!	TimeConvert(sDosTime,sSysTime))return FALSE;
	return  TimeConvert(sSysTime,Time);
}

//*****************************************************************************
//*
//*		TimeConvert	   DOSTIME --> DBDATE
//*
//*****************************************************************************
BOOL TimeConvert(const DOSTIME &sDosTime ,DBDATE &sDbDate)
{

	sDbDate.day         = (sDosTime.wDate    )&0x1F;
	sDbDate.month       = (sDosTime.wDate>> 5)&0x0F;
	sDbDate.year        = (sDosTime.wDate>> 9)+1980;

	if(sDbDate.month<=0)return FALSE;
	if(sDbDate.month>12)return FALSE;
	if(sDbDate.day  <=0)return FALSE;
	if(sDbDate.day>aTpMDays[sDbDate.month-1])
		{
		if(sDbDate.month!= 2)return FALSE;
		if(sDbDate.day  !=29)return FALSE;
		if(sDbDate.year   &3)return FALSE;
		}

return TRUE;
}


//*****************************************************************************
//*
//*		TimeConvert	   DOSTIME --> DBTIMESTAMP
//*
//*****************************************************************************
BOOL TimeConvert(const DOSTIME &sDosTime ,DBTIMESTAMP &sDbStamp)
{

	sDbStamp.fraction	= 0;
	sDbStamp.second		= (sDosTime.wTime<< 1)&0x3F;
	sDbStamp.minute		= (sDosTime.wTime>> 5)&0x3F;
	sDbStamp.hour	   	= (sDosTime.wTime>>11);
	sDbStamp.day        = (sDosTime.wDate    )&0x1F;
	sDbStamp.month      = (sDosTime.wDate>> 5)&0x0F;
	sDbStamp.year       = (sDosTime.wDate>> 9)+1980;


	if(sDbStamp.second>59)return FALSE;
	if(sDbStamp.minute>59)return FALSE;
	if(sDbStamp.hour  >23)return FALSE;
	if(sDbStamp.month <=0)return FALSE;
	if(sDbStamp.month >12)return FALSE;
	if(sDbStamp.day   <=0)return FALSE;
	if(sDbStamp.day>aTpMDays[sDbStamp.month-1])
		{
		if(sDbStamp.month!= 2)return FALSE;
		if(sDbStamp.day  !=29)return FALSE;
		if(sDbStamp.year   &3)return FALSE;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIME --> time_t
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIME &sDbTime,time_t &iUtcTime)
{
time_t	iTime;

			   iTime  = sDbTime.hour;
			   iTime *= 60;
			   iTime += sDbTime.minute;
			   iTime *= 60;
			   iTime += sDbTime.second;
	iUtcTime = iTime;

	if(sDbTime.second>59)return FALSE;
	if(sDbTime.minute>59)return FALSE;
	if(sDbTime.hour  >23)return FALSE;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIME --> double
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIME &sDbTime,double &dTime)
{
double	dVal;


	dVal  = sDbTime.hour;
	dVal *= 60.0;
	dVal += sDbTime.minute;
	dVal *= 60.0;
	dVal += sDbTime.second;

	dTime = dVal/86400.0;


return TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert	   DBTIME --> DBTIMESTAMP
//*
//*****************************************************************************
BOOL TimeConvert(const DBTIME &sDbTime,DBTIMESTAMP &sDbStamp)
{

	sDbStamp.fraction = 0;
	sDbStamp.second   = sDbTime.second;
	sDbStamp.minute   = sDbTime.minute;
	sDbStamp.hour     = sDbTime.hour;
	sDbStamp.day      = 0;
	sDbStamp.month    = 0;
	sDbStamp.year     = 0;

	if(sDbTime.second>59)return FALSE;
	if(sDbTime.minute>59)return FALSE;
	if(sDbTime.hour  >23)return FALSE;


return TRUE;
}


//*****************************************************************************
//*
//*		TimeConvert
//*
//*****************************************************************************
BOOL TimeConvert(const TMTIME &sTmTime,time_t &iUtcTime,int iLocal)
{
TMTIME	sTime;

	if(!iInitDone)
		{
		time(0);
		iInitDone=1;
		}

	sTime    = sTmTime;
	iUtcTime = mktime(&sTime);
	if(!iLocal)iUtcTime-=_timezone;	// REM AZ prfen und init


return (iUtcTime<=0)? FALSE:TRUE;
}

//*****************************************************************************
//*
//*		TimeConvert
//*
//*****************************************************************************
BOOL TimeConvert(const TMTIME &sTmTime,FILETIME &sFileTime,int iLocal)
{
SYSTEMTIME	sSysTime;
TMTIME		sTmTemp;
__int64		iDiff;
BOOL		bOk;



	sSysTime.wMilliseconds = 0;
	sSysTime.wDayOfWeek    = 0;
	sSysTime.wSecond	   = sTmTime.tm_sec;
	sSysTime.wMinute	   = sTmTime.tm_min;
	sSysTime.wHour	   	   = sTmTime.tm_hour;
	sSysTime.wDay          = sTmTime.tm_mday;
	sSysTime.wMonth        = sTmTime.tm_mon+1;
	sSysTime.wYear         = sTmTime.tm_year+1900;

	bOk = SystemTimeToFileTime(&sSysTime,&sFileTime);

	if(!bOk)
		{
		sFileTime.dwHighDateTime = 0;
		sFileTime.dwLowDateTime  = 0;
		return FALSE;
		}

	if(!iLocal)return TRUE;

	memset(&sTmTemp,0,sizeof(sTmTemp));

	sTmTemp.tm_isdst	= sTmTime.tm_isdst;
	sTmTemp.tm_year		= 2000;
	sTmTemp.tm_mday		= 1;

	iDiff				= mktime(&sTmTemp);
	iDiff			   -= 946684800;
	iDiff			   *= 10000000;

	*(__int64*)(&sFileTime) += iDiff;


return TRUE;

}




