//*****************************************************************************
//*
//*
//*		UdpMessage.cpp
//*
//*
//*****************************************************************************
//
//	Das ist eine Library um Nachrichten via UDP zu versenden.
//
//
//	Server Beispiel:
//	
//		MsgHandle		hServer;
//		char			aBuffer[256];
//		unsigned short 	usPort;
//		MsgAddr			aAddr;
//		int				iSize;
//		int				iFlag;
//	
//		hServer = MessageOpen(12345,MSD_SERVER_LOCAL);
//		iFlag	= MessageWait(hServer,10000);
//		iSize	= MessageRecv(hServer,aBuffer,sizeof(aBuffer),&usPort,&aAddr);
//		iFlag	= MessageSend(hClient,aBuffer,iSize,usPort,&aAddr);
//		...
//		MessageClose(hServer);
//
//	
//	Client Beispiel:
//	
//		MsgHandle		hClient;
//		char			aBuffer[256]="Hallo";
//		unsigned short 	usPort;
//		MsgAddr			aAddr;
//		int				iSize;
//		int				iFlag;
//	
//		hClient = MessageOpen(MSG_AUTO_PORT,MSD_SERVER_LOCAL);
//		iFlag	= MessageSend(hClient,aBuffer,5,12345,MSD_TARGET_LOCAL);
//		iFlag	= MessageWait(hClient,10000);
//		iSize	= MessageRecv(hClient,aBuffer,sizeof(aBuffer),&usPort,&aAddr);
//		...
//		MessageClose(hClient);
//	
#include	"UdpMessage.h"


#define 	MSG_API				extern "C"
#define     GET_V4_ADDR(p)		(*(unsigned*)(p))
#define     SET_V4_ADDR(p,a)	(*(unsigned*)(p)=a)

static		unsigned			uNextPort;


//*****************************************************************************
//*
//*		WINDOWS
//*
//*****************************************************************************
#ifdef WIN32

#pragma     comment(lib,"wsock32.lib")
#pragma     comment(lib,"ws2_32.lib")
#include    <winsock2.h>
#include    <windows.h>

#define     LEN_TYPE		int*

static void InitSocket()
	{
    WORD    wVersion;
    WSADATA sWsaData;


    wVersion = 0x0202;
    if(WSAStartup(wVersion,&sWsaData))
        {
        wVersion = 0x0002;
        WSAStartup(wVersion,&sWsaData);
        }
    }

//*****************************************************************************
//*
//*		LINUX
//*
//*****************************************************************************
#elif defined(linux)

#include    <unistd.h>
#include    <sys/stat.h>
#include    <sys/time.h>
#include    <sys/times.h>
#include    <sys/types.h>
#include    <sys/ioctl.h>
#include    <sys/socket.h>
#include    <linux/if_arp.h>
#include    <netinet/tcp.h>
#include    <netinet/in.h>
#include    <arpa/inet.h>
#include    <semaphore.h>
#include    <pthread.h>
#include    <ifaddrs.h>
#include    <string.h>
#include    <signal.h>
#include    <malloc.h>
#include    <fcntl.h>
#include    <netdb.h>
#include    <sched.h>
#include    <time.h>


#define		InitSocket()	

#define     LEN_TYPE        socklen_t*

#ifndef     SOCKET_ERROR
#define     SOCKET_ERROR    (-1)
#endif

#ifndef     INVALID_SOCKET
#define     INVALID_SOCKET  (~0)
#endif


#else
#error	unknow operating system
#endif 	




//*****************************************************************************
//*
//*     MessageOpen
//*
//*****************************************************************************/
//  ffnet ein UDP-Port fr den Nachrichtenversand
//	usPort		: Ist die UDP-Portnummer, bei MSG_AUTO_PORT (0) wird
//				  automatisch eine freie Portnummer gesucht. 
//	pServer		: Ist die IP-Adresse auf der das UDP-Port eingerichte werden soll
//					MSD_SERVER_LOCAL	= 127.0.0.1
//					MSD_SERVER_ALL		= 0.0.0.0
//	Ergibt ein Handle zum Nachrichtenport oder 0 bei einem Fehler.
MSG_API MsgHandle MessageOpen(unsigned short usPort,MsgAddr *pServer)
{
unsigned long           lNoneBlocking;
struct sockaddr_in      sData;
SOCKET                  iSock;
unsigned                uAddr;
unsigned                uTurn;
int						iSize;
int                     iErr;



    InitSocket();

    
	iSock	= socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
  
	if(iSock==INVALID_SOCKET)
		{
        return NULL;
        }

	
	memset(&sData,0,sizeof(sData));                 // Adresse erzeugen

	if(pServer==MSD_SERVER_ALL)
		{
		uAddr	= 0x00000000;
		}
	else if(pServer==MSD_SERVER_LOCAL)
		{
		uAddr	= 0x0100007F;
		}
	else{
        uAddr = GET_V4_ADDR(pServer);
		}


	if(usPort!=MSG_AUTO_PORT)						// Fixes Port
		{
        sData.sin_family      =  AF_INET;
        sData.sin_addr.s_addr =  uAddr;
        sData.sin_port        =  htons(usPort);

        iErr  = bind(iSock,(const struct sockaddr*)&sData,sizeof(sData));
		}
	else for(uTurn=0;uTurn<32;uTurn++)				// Suche ein freies Port
		{
		uNextPort++;

		if(uNextPort<51000)uNextPort=51000;
		if(uNextPort>52000)uNextPort=51000;
		
        sData.sin_family      =  AF_INET;
        sData.sin_addr.s_addr =  uAddr;
        sData.sin_port        =  htons(uNextPort);

			iErr  = bind(iSock,(const struct sockaddr*)&sData,sizeof(sData));
		if(!iErr)break;
		}


    if(iErr)										// Gab es Fehler
        {
        closesocket(iSock);
        return NULL;
        }

    lNoneBlocking = 1;

	if(ioctlsocket(iSock,FIONBIO,&lNoneBlocking))
		{
        closesocket(iSock);
        return NULL;
        }

	iSize = 0x8000;									// Puffer einstellen	
    setsockopt(iSock,SOL_SOCKET,SO_RCVBUF,(char*)&iSize,sizeof(iSize));
    setsockopt(iSock,SOL_SOCKET,SO_SNDBUF,(char*)&iSize,sizeof(iSize));


return (MsgHandle)iSock;
}


//*****************************************************************************
//*
//*		MessageClose
//*
//*****************************************************************************
//  Schliet ein UDP-Port fr den Nachrichtenversand
//	hHandle		: Ist das Handle zum zum UDP-Socket
//	Ergibt 1 wenn ok, oder 0 bei einem Fehler.
MSG_API int MessageClose(MsgHandle hHandle)
{

    if(!hHandle)
        {
        return 0;
        }

    shutdown   ((SOCKET)hHandle,2);
    closesocket((SOCKET)hHandle  );


return 1;
}

//*****************************************************************************
//*
//*		MessageRecv
//*
//*****************************************************************************
//	Empfng die nchste Nachricht, dabei wird nicht gewartet
//	hHandle		: Ist das Handle zum zum UDP-Socket
//	pBuffer		: Zeiger auf den Empfangspuffer
//	iSize		: Gre des Empfangspuffers in Byte 
//	pPort		: Hier wird die UDP-Portnummer des Sender gespeichert 
//				  Dieser Zeiger kann auch NULL sein.
//	pSource		: Hier wird die IP-Adresse des Sender gespeichert 
//				  Dieser Zeiger kann auch NULL sein.
//	Ergibt die Lnge der empfangen Nachricht, 0 wenn nichts empfangen wurde
//	oder -1 bei einem Fehler.	
MSG_API int MessageRecv(MsgHandle hHandle,void *pBuffer,int iSize,unsigned short *pPort,MsgAddr *pSource)
{
struct sockaddr_in      sInput;
int						iBytes;
unsigned				uAddr;



	if(!hHandle)return -1;

	iSize  = sizeof(sInput);
	iBytes = recvfrom((SOCKET)hHandle,(char*)pBuffer,iSize,0,(struct sockaddr*)&sInput,(LEN_TYPE)&iSize);
	
	if(iBytes<0)
		{
		if(pPort)*pPort=0;
		return 0;
		}

	if(pPort)
		{
		*pPort = htons(((struct sockaddr_in*)&sInput)->sin_port);
		}

	if(pSource)
		{
		uAddr=((struct sockaddr_in*)&sInput)->sin_addr.s_addr;
		SET_V4_ADDR(pSource,((struct sockaddr_in*)&sInput)->sin_addr.s_addr);
		}


return iBytes;
}


//*****************************************************************************
//*
//*		MessageSend
//*
//*****************************************************************************
//	Sendet eine Nachricht, dabei wird nicht gewartet bis sie versand wurde.
//	hHandle		: Ist das Handle zum zum UDP-Socket
//	pBuffer		: Zeiger auf die Nachrichtendaten
//	iSize		: Gre der Nachrichtendaten in Byte 
//	usPort		: Hier wird die UDP-Portnummer des Ziels bergeben
//	pTarget		: Hier wird die IP-Adresse des Ziels bergeben
//					MSD_TARGET_LOCAL = 127.0.0.1
//	Ergibt die Lnge der gesendeten Nachricht, 0 wenn nichts versant wurde
//	oder -1 bei einem Fehler.	
MSG_API int MessageSend(MsgHandle hHandle,const void *pBuffer,int iSize,unsigned short usPort,MsgAddr *pTarget)
{
struct sockaddr_in  sInput;
int					iBytes;
unsigned			uAddr;



	if(!hHandle)return -1;

	if(pTarget==MSD_TARGET_LOCAL)
		{
		uAddr	= 0x0100007F;
		sInput.sin_addr.s_addr  =  GET_V4_ADDR(&uAddr);
		}
    else{
		sInput.sin_addr.s_addr  =  GET_V4_ADDR(pTarget);
		}

    sInput.sin_family       =  AF_INET;
    sInput.sin_port         =  htons(usPort);

	iBytes = sendto((SOCKET)hHandle,(char*)pBuffer,iSize,0,(const struct sockaddr*)&sInput,sizeof(sInput));

	if(iBytes<0)iBytes=0;


return iBytes;
}


//*****************************************************************************
//*
//*		MessageWait
//*
//*****************************************************************************
//	Wartet auf die nchste Nachricht am UDP-Socket.
//	hHandle		: Ist das Handle zum zum UDP-Socket
//	iTimeout	: Wartezeit in Millisekunden
//	Ergibt 1 wenn eine Nachricht im Puffer liegt, 0 wenn nichts empfangen wurde
//	oder -1 bei einem Fehler.	
int MessageWait(MsgHandle hHandle,int iTimeout)
{
struct timeval  sTimeVal;
fd_set          sFdRead;
fd_set          sFdErr;
int             iRet;



	if(!hHandle)return -1;

	if(iTimeout<0)iTimeout=0;

    sTimeVal.tv_sec  =  iTimeout/1000;
    sTimeVal.tv_usec = (iTimeout%1000)*1000;

    FD_ZERO(&sFdRead);
    FD_ZERO(&sFdErr );

    FD_SET((SOCKET)hHandle,&sFdRead);
    FD_SET((SOCKET)hHandle,&sFdErr );

    iRet = select((SOCKET)hHandle+1,&sFdRead,0,&sFdErr,&sTimeVal);


return iRet;
}


//*****************************************************************************
//*
//*     MessageAddr
//*
//*****************************************************************************
//  Eine IP-Adresse in ein MsgAddr-Objekt umwandeln.
//  pAddr       : Zeiger auf das MsgAddr-Objekt
//  pIp         : Zeiger auf die IP-Adrresse z.B.. "168.192.0.1"
//  Ergibt 1 wenn ok oder 0 bei einem Fehler
MSG_API int MessageAddr(MsgAddr *pAddr,const char *pIp)
{
unsigned    uIpAddr;




    if(!pIp || !*pIp)
        {
        SET_V4_ADDR(pAddr,INADDR_NONE);
        return 0;
        }

    uIpAddr = inet_addr(pIp);

	if(uIpAddr==INADDR_NONE)
		{
        SET_V4_ADDR(pAddr,INADDR_NONE);
        return 0;
        }


	SET_V4_ADDR(pAddr,uIpAddr);



return 1;
}


