//*****************************************************************************
//*
//*
//*		UdpMessage.h
//*
//*
//*****************************************************************************
//
//	Das ist eine Library um Nachrichten via UDP zu versenden.
//
//
//	Server Beispiel:
//	
//		MsgHandle		hServer;
//		char			aBuffer[256];
//		unsigned short 	usPort;
//		MsgAddr			aAddr;
//		int				iSize;
//		int				iFlag;
//	
//		hServer = MessageOpen(12345,MSD_SERVER_LOCAL);
//		iFlag	= MessageWait(hServer,10000);
//		iSize	= MessageRecv(hServer,aBuffer,sizeof(aBuffer),&usPort,&aAddr);
//		iFlag	= MessageSend(hClient,aBuffer,iSize,usPort,&aAddr);
//		...
//		MessageClose(hServer);
//
//	
//	Client Beispiel:
//	
//		MsgHandle		hClient;
//		char			aBuffer[256]="Hallo";
//		unsigned short 	usPort;
//		MsgAddr			aAddr;
//		int				iSize;
//		int				iFlag;
//	
//		hClient = MessageOpen(MSG_AUTO_PORT,MSD_SERVER_LOCAL);
//		iFlag	= MessageSend(hClient,aBuffer,5,12345,MSD_TARGET_LOCAL);
//		iFlag	= MessageWait(hClient,10000);
//		iSize	= MessageRecv(hClient,aBuffer,sizeof(aBuffer),&usPort,&aAddr);
//		...
//		MessageClose(hClient);
//	
#ifndef 	__UDPMESSAGE_H__
#define 	__UDPMESSAGE_H__


#define 	MSG_AUTO_PORT		0				// Automatisch ein freies UDP Port suchen
#define 	MSD_TARGET_LOCAL	((MsgAddr*)0)	// Nachricht zur IP 127.0.0.1 senden
#define 	MSD_SERVER_LOCAL	((MsgAddr*)0)	// Server auf der IP 127.0.0.1 einrichten
#define 	MSD_SERVER_ALL		((MsgAddr*)1)	// Server auf der IP 127.0.0.1 einrichten

#ifdef    __cplusplus 
extern "C"	{
#define 	D(a)	a
#else	
#define 	D(a)
#endif 	

typedef	void	   *MsgHandle;
typedef	unsigned	MsgAddr;



MsgHandle	MessageOpen (unsigned short usPort D(=MSG_AUTO_PORT),MsgAddr *pServer D(=MSD_SERVER_LOCAL));
int			MessageSend (MsgHandle hHandle,const void *pBuffer,int iSize,unsigned short usPort      ,MsgAddr *pTarget D(=MSD_TARGET_LOCAL));
int			MessageRecv (MsgHandle hHandle,      void *pBuffer,int iSize,unsigned short *pPort D(=0),MsgAddr *pSource D(=0));
int			MessageWait (MsgHandle hHandle,int iTimeout);
int			MessageClose(MsgHandle hHandle);
int			MessageAddr (MsgAddr  *pAddr,const char *pIpAddr);


#undef		D

#ifdef   __cplusplus
};
#endif 	


#endif
