//*****************************************************************************
//*
//*
//*		TestArchive.cpp
//*
//*
//*****************************************************************************
#ifdef		UNICODE
#define    _UNICODE
#endif 		

#define 	ENTRIES(a)		(sizeof(a)/sizeof((a)[0]))


#include	<string.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	<conio.h>
#include	<tchar.h>
#include	<time.h>


#include	"TarFile.h"
#include	"PioFile.h"
#include	"Bz2File.h"
#include	"TgzFile.h"
#include	"XzsFile.h"
#include	"LzwFile.h"
#include	"LzmFile.h"
#include	"UueFile.h"
#include	"ZipFile.h"
#include	"CabFile.h"
#include	"FatFile.h"
#include	"ExeFile.h"
#include	"ChmFile.h"
#include	"IsoFile.h"
#include	"RarFile.h"
#include	"RpmFile.h"
#include	"Zp7File.h"
#include	"MsiFile.h"


typedef struct	
	{
	LPCTSTR			pPath;
	int				iSize;
	unsigned		uCrc;	
	}FileItem;


#include	"ArchivChecks/ExeCheck.h"
#include	"ArchivChecks/ChmCheck.h"
#include	"ArchivChecks/IsoCheck.h"
#include	"ArchivChecks/RarCheck.h"
#include	"ArchivChecks/RpmCheck.h"
#include	"ArchivChecks/Zp7Check.h"
#include	"ArchivChecks/MsiCheck.h"

typedef struct	
	{
	CArchiveFile   *pArcive;
	LPCTSTR			pSuffix;
	}ArchiveItem;	

TCHAR	cErrorText[256];
char   *pErroText="";
char   *aFileData[14];
int		aFileSize[14]={0,0,37,512,784,1024,2923,4096,4096,4096,55555,55555,55555,123456};
char 	aTempFile[500000];
char 	aTempData[500000];
int		iMemWrMax;
int		iMemRdPos;
int		iMemLast;


//*****************************************************************************
//*
//*		ArchToMem
//*
//*****************************************************************************
int ArchToMem(void *pParam,const char *pMem,int iLen)
{

	if(pParam!=aTempData)
		{
		return -1;
		}

	if(iMemRdPos+iLen>sizeof(aTempData))
		{
		return -1;
		}

	memcpy(aTempData+iMemRdPos,pMem,iLen);

	iMemRdPos += iLen;
	iMemLast   = iLen;


return iLen;
}

//*****************************************************************************
//*
//*		MemToArch
//*
//*****************************************************************************
int MemToArch(void *pParam,char *pMem,int iLen)
{

	if(iMemRdPos+iLen>sizeof(aTempData))
		{
		return -1;
		}

	if(iMemRdPos+iLen>iMemWrMax)
		{
		iLen = iMemWrMax-iMemRdPos;
		}
	

	memcpy(pMem,(char*)pParam+iMemRdPos,iLen);

	iMemRdPos += iLen;


return iLen;
}

//*****************************************************************************
//*
//*		CreateCmpInfo
//*
//*****************************************************************************
#ifdef _UNICODE
int CreateCmpInfo(const char *pArchPath,const char *pFilePath,const char *pPrefix){return 0;}
#else
int CreateCmpInfo(const char *pArchPath,const char *pFilePath,const char *pPrefix)
{
char		cText[1024];
char	   *pName;	
char	   *pPath;	
CFileFinder	sFind;
FILE	   *pFile;
int			iLen;
int			iPos;
int			iOff;
int			iCrc;
int			iNum;


	   iLen = LoadMemory(pArchPath,aTempFile,sizeof(aTempFile));
	if(iLen<=0)return 0;

	sprintf(cText,"D:\\%sCheck.h",pPrefix);
	
		pFile = fopen(cText,"wb");
	if(!pFile)return 0;

	fprintf(pFile,"//*****************************************************************************\n");
	fprintf(pFile,"//*\n");
	fprintf(pFile,"//*\n");
	fprintf(pFile,"//*\t\t%sCheck.h\n",pPrefix);
	fprintf(pFile,"//*\n");
	fprintf(pFile,"//*\n");
	fprintf(pFile,"//*****************************************************************************\n\n\n");

	fprintf(pFile,"const unsigned char a%sData[] =\n",pPrefix);
	fprintf(pFile,"\t{\n");

	for(iPos=0;iPos<iLen;iPos+=iOff)
		{
		cText[0] = '\n';

		for(iOff=0;iOff<64 && iPos+iOff<iLen;iOff++)
			{
			sprintf(cText+iOff*5,"0x%02X,",(BYTE)aTempFile[iPos+iOff]);
			}

		fprintf(pFile,"\t%s\n",cText);
		}

	fprintf(pFile,"\t};\n\n\n");

	fprintf(pFile,"FileItem	a%sFiles[] =\n",pPrefix);
	fprintf(pFile,"\t{\n");

	
	sFind.Set(pFilePath,CFF_FIND_SUBDIR,"*");

	for(;;)
		{
			pName = sFind.Next(2);
		if(!pName)break;

		pPath = sFind.NameFull();
		iLen  = LoadMemory(pPath,aTempFile,sizeof(aTempFile));
		iCrc  = Crc32(0,aTempFile,iLen);	

		strlcpy(cText,pName,sizeof(cText));
		PathConvert(cText,'/');

		   iNum = 32-strlen(pName);
		if(iNum<0)iNum=0;

		fprintf(pFile,"\t{_T(\"%s\")%.*s\t,%10i  ,0x%08X},\n",cText,iNum,"                                                            ",iLen,iCrc);
		}

	fprintf(pFile,"\t};\n\n\n");
	fclose (pFile);


return 1;
}
#endif 	

//*****************************************************************************
//*
//*		TestReadOnlyArchive
//*
//*****************************************************************************
int TestReadOnlyArchive(CArchiveFile *pArchive,const unsigned char *pData,int iSize,FileItem *pItems,int iCount,LPCTSTR pSuffix)
{
int			iErrors=0;
unsigned 	uCrc;
int			iRet;
int			iIdx;
int			iNum;
int			iLen;
	

	_tprintf(_T("\n\n--------------------- %s -----------------%.*s\n"),pSuffix,5-_tcslen(pSuffix),_T("-------"));


	try	{
		   iRet = pArchive->OpenMem(pData,iSize);
		if(iRet!=CAF_ERROR_OK)
			{
			throw 0xA000;
			}

		for(iIdx=0;iIdx<iCount;iIdx++)					// Dateien prfen
			{
			iNum = pArchive->Find(pItems[iIdx].pPath);

			if(iNum<0)
				{
				pErroText = "Find(...)";
				throw 0xA001;
				}
			
			if(pArchive->Len(iNum)!=pItems[iIdx].iSize)
				{
				pErroText = "Len(...)";
				throw 0xA002;
				}
			
			memset(aTempData,0,sizeof(aTempData));

			   iRet = pArchive->ExtractMem(iNum,aTempData,sizeof(aTempData));
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "ExtractMem(...)";
				throw 0xA003;
				}
														// Passt der Speicher
			   uCrc = Crc32(0,aTempData,pItems[iIdx].iSize);
			if(uCrc!=pItems[iIdx].uCrc)
				{
				pErroText = "Wrong-CRC";
				throw 0xA004;
				}


			iMemLast  = 1;
			iMemRdPos = 0;
			memset(aTempData,0,sizeof(aTempData));
			
			   iRet = pArchive->ExtractProc(iNum,aTempData,ArchToMem);
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "ExtractProc(...)";
				throw 0xA005;
				}

			if(iMemLast!=0)								// Wurde ein Callback mit 0 aufgerufen
				{
				pErroText = "No-Callback-Zero-Call";
				throw 0xA006;
				}

			if(iMemRdPos!=pItems[iIdx].iSize)
				{
				pErroText = "Wrong-File-Size";
				throw 0xA007;
				}				

			   uCrc = Crc32(0,aTempData,pItems[iIdx].iSize);
			if(uCrc!=pItems[iIdx].uCrc)
				{
				pErroText = "Wrong-CRC";
				throw 0xA008;
				}

			   iRet = pArchive->Extract(iNum,_T("Temp.Arch.bin"));
			if(iRet!=CAF_ERROR_OK)
				{
				DeleteFile(_T("Temp.Arch.bin"));
				pErroText = "Extract(...)";
				throw 0xA009;
				}

			if(ExistFile("Temp.Arch.bin")!=1)
				{
				pErroText = "ExistFile(...)";
				throw 0xA00A;
				}

			memset(aTempData,0,sizeof(aTempData));
			iLen = LoadMemory("Temp.Arch.bin",aTempData,sizeof(aTempData));
			DeleteFile(_T("Temp.Arch.bin"));

			if(iLen!=pItems[iIdx].iSize)
				{
				pErroText = "Len(...)";
				throw 0xA00B;
				}
			
			   uCrc = Crc32(0,aTempData,iLen);			// Passt der Speicher
			if(uCrc!=pItems[iIdx].uCrc)
				{
				pErroText = "Wrong-CRC";
				throw 0xA00C;
				}
			}
		}
	catch(int iError)
		{
		iErrors++;
		strawcpy(cErrorText,pErroText,ENTRIES(cErrorText));
		_tprintf(_T("Error %04Xh for %s at:  %s\n"),iError,pSuffix,cErrorText);
		}
	catch(...)
		{
		iErrors++;
		_tprintf(_T("Unknown exeption for %s\n"),pSuffix);
		}

	


return iErrors;
}

//******************************************************************************
//*
//*		main
//*
//******************************************************************************
int main(int iArgc,char **pArgv)
{
TCHAR			cFileName[MAX_PATH];
TCHAR			cFilePath[MAX_PATH];
TCHAR			cFileTemp[MAX_PATH];
CHAR		   *pFileData;
CArchiveFile   *pArchive;
LPCTSTR			pSuffix;
int				iPathLen;
int				iErrors;
int				iSize;
int				iPos;
int				iNum;
int				iIdx;
int				iOff;
int				iRet;

					  

//CreateCmpInfo("D:\\Test.msi","D:\\___","Msi");
//return 0;

	CCabFile	sCab;
	CZipFile	sZip;
	CUueFile	sUue;
	CLzmFile	sLzm;
	CLzwFile	sLzw;
	CXzsFile	sXzs;
	CTgzFile	sTgz;
	CBz2File	sBz2;
	CPioFile	sPio;
	CTarFile	sTar;
	CFatFile	sFat;
	CExeFile	sExe;
	CChmFile	sChm;
	CIsoFile	sIso;	
	CRarFile	sRar;	
	CRpmFile	sRpm;	
	CZp7File	sZp7;
	CMsiFile	sMsi;

//iRet=sMsi.Open("D:\\Test.msi");
//sMsi.ExtractAll("D:\\___");
//return 0;

	iErrors  = 0;

	iErrors += TestReadOnlyArchive(&sMsi,aMsiData,sizeof(aMsiData),aMsiFiles,ENTRIES(aMsiFiles),_T("msi"));
	iErrors += TestReadOnlyArchive(&sZp7,aZp7Data,sizeof(aZp7Data),aZp7Files,ENTRIES(aZp7Files),_T("7z" ));
	iErrors += TestReadOnlyArchive(&sRpm,aRpmData,sizeof(aRpmData),aRpmFiles,ENTRIES(aRpmFiles),_T("rar"));
	iErrors += TestReadOnlyArchive(&sRar,aRarData,sizeof(aRarData),aRarFiles,ENTRIES(aRarFiles),_T("rar"));
	iErrors += TestReadOnlyArchive(&sIso,aIsoData,sizeof(aIsoData),aIsoFiles,ENTRIES(aIsoFiles),_T("iso"));
	iErrors += TestReadOnlyArchive(&sChm,aChmData,sizeof(aChmData),aChmFiles,ENTRIES(aChmFiles),_T("chm"));
	iErrors += TestReadOnlyArchive(&sExe,aExeData,sizeof(aExeData),aExeFiles,ENTRIES(aExeFiles),_T("exe"));

	ArchiveItem	aItems[] =
	{
	&sFat	,_T("fat" ),
	&sCab	,_T("cab" ),
	&sZip	,_T("zip" ),
	&sUue	,_T("uue" ),
	&sLzm	,_T("lzma"),
	&sLzw	,_T("lzw" ),
	&sXzs	,_T("xz"  ),
	&sTgz	,_T("tgz" ),
	&sBz2	,_T("bz2" ),
	&sPio	,_T("cpio"),
	&sTar	,_T("tar" ),
	};



//	srand(time(0));
	iPathLen = GetTempPath(MAX_PATH,cFileName);


iPathLen = strlcpy(cFileName,_T("D:\\"),MAX_PATH);

	
	strlcpy(cFilePath,cFileName,MAX_PATH);
	strlcpy(cFileTemp,cFileName,MAX_PATH);
	



//******************** Test ***************************************************


	
	for(iPos=0;iPos<ENTRIES(aItems);iPos++)				// Die Archiv Testen
		{
		for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)		// Dateien erstellen	
			{
			if(iIdx==0)continue;

			iSize			= aFileSize[iIdx];
			pFileData		= new char[iSize];
			aFileData[iIdx]	= pFileData;
			 
			for(iOff=0;iOff<iSize;iOff++)
				{
				if((iOff/1024)%5>3)
					{
					pFileData[iOff] = iOff/13;
					}
				else{
					pFileData[iOff] = rand();
					}
				}

			_stprintf (cFileName+iPathLen,_T("Test%i.tmp" ),iIdx);
			SaveMemory(cFileName,pFileData,iSize);
			_stprintf (cFileName+iPathLen,_T("Test%i.orig"),iIdx);
			SaveMemory(cFileName,pFileData,iSize);
			}
		
		
		pArchive	= aItems[iPos].pArcive;
		pSuffix		= aItems[iPos].pSuffix;

		printf("\n\n--------------------- %s -----------------%.*s\n",pSuffix,5-_tcslen(pSuffix),"-------");

		try	{
			_stprintf(cFilePath+iPathLen,_T("TestArch.all.%s"),pSuffix);
			sFat.SetOption(0,sizeof(aTempFile)/512-4);

			   iRet = pArchive->Open(cFilePath,CAF_CREATE);
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "Open(...,CAF_CREATE)";
				throw 0x8001;
				}

			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Dateien einfgen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);

				if(iIdx==0)
					{
					pErroText = "AddPath(...)";
					iRet = pArchive->AddPath(cFileName);
					iNum = 0x8000; 
					}
				else if(iIdx==9 || iIdx==12)
					{
					pErroText = "AddMem(...)";
					iRet = pArchive->AddMem(cFileName,aFileData[iIdx],aFileSize[iIdx]);
					iNum = 0x8021; 
					}
				else if(iIdx==7 || iIdx==10)
					{
					iMemRdPos	= 0;
					iMemWrMax	= aFileSize[iIdx];

					pErroText = "AddProc(...)";
					iRet = pArchive->AddProc(cFileName,aFileData[iIdx],MemToArch);

					if(iRet==CAF_ERROR_COMP)			// Unkomprimiert versuchen	
						{
						FfFindStruct	sInfo;

						sInfo.dwAttr		= SYS_READ|SYS_WRITE|SYS_VOLUME;	
						sInfo.dwSize		= aFileSize[iIdx];	
						sInfo.dwTimeAccess	= 0;
						sInfo.dwTimeCreate	= 0;
						sInfo.dwTimeWrite	= 0;
						sInfo.cName[0]		= 0;
						iMemRdPos			= 0;

						iRet = pArchive->AddProc(cFileName,aFileData[iIdx],MemToArch,(CArchiveFile::FileData*)&sInfo);
						}

					iNum = 0x8022; 						
					}
				else{
					pErroText = "AddFile(...)";
					iRet = pArchive->AddFile(cFileName);
					iNum = 0x8023; 						
					}

				if(iRet!=CAF_ERROR_OK)
					{
					throw iNum;
					}
				}


			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Dateien prfen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);
				iNum = pArchive->Find(cFileName+iPathLen);

				if(iNum<0)
					{
					pErroText = "Find(...)";
					throw 0x8003;
					}
				
				if(pArchive->Len(iNum)!=aFileSize[iIdx])
					{
					pErroText = "Len(...)";
					throw 0x804C;
					}

				if(pArchive->IsReadOnly(iNum))
					{
					pErroText = "IsReadOnly(...)";
					throw 0x804D;
					}

				if(iNum>=2 && pArchive->IsDir(iNum))
					{
					pErroText = "IsDir(...)";
					throw 0x804E;
					}
				}

			   iRet = pArchive->Close();				// Achive schliesen
			if(iRet!=CAF_ERROR_OK)				
				{
				throw 0x8004;
				}

			_stprintf(cFileName+iPathLen,_T("TestArch.all.%s"),pSuffix);
			_stprintf(cFilePath+iPathLen,_T("TestArch.tmp.%s"),pSuffix);

			if(!CopyFile(cFileName,cFilePath,FALSE))
				{
				pErroText = "CopyFile(...)";
				throw 0x8005;
				}

			if(LengthFile(cFilePath)<=0x8000)			// Prfe Dateigre		
				{
				pErroText = "LengthFile(...)";
				throw 0x8006;
				}



			   iRet = pArchive->Open(cFilePath,CAF_READ);
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "Open(...,CAF_READ)";
				throw 0x8007;
				}

			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Dateien prfen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);
				iNum = pArchive->Find(cFileName+iPathLen);

				if(iNum<0)
					{
					if(pArchive==&sCab)continue;
					pErroText = "Find(...)";
					throw 0x8008;
					}
				
				if(pArchive->Len(iNum)!=aFileSize[iIdx])
					{
					pErroText = "Len(...)";
					throw 0x8009;
					}

				   iRet = pArchive->Test(iNum);
				if(iRet!=CAF_ERROR_OK)				
					{
					pErroText = "Test(...)";
					throw 0x8024;
					}

				
				if(iIdx==0)								// Pfad prfen
					{
					if(!pArchive->IsDir(iNum))
						{
						pErroText = "IsDir(...)";
						throw 0x8013;
						}

					   iRet = pArchive->Extract(iNum,cFileName);
					if(iRet!=CAF_ERROR_OK)
						{
						pErroText = "Extract(...)";
						throw 0x8015;
						}

					if(ExistFile(cFileName)!=2)
						{
						pErroText = "ExistFile(...)";
						throw 0x8046;
						}

					RemoveDirectory(cFileName);

					continue;
					}
					
				if(pArchive->IsDir(iNum))				// Typ prfen
					{
					throw 0x8014;
					}

				memset(aTempData,0,sizeof(aTempData));
			
				   iRet = pArchive->ExtractMem(iNum,aTempData,sizeof(aTempData));
				if(iRet!=CAF_ERROR_OK)
					{
					pErroText = "ExtractMem(...)";
					throw 0x800A;
					}
														// Passt der Speicher
				if(memcmp(aTempData,aFileData[iIdx],aFileSize[iIdx]))
					{
					pErroText = "memcmp(...)";
					throw 0x800B;
					}				
				
				iMemLast  = 1;
				iMemRdPos = 0;
				memset(aTempData,0,sizeof(aTempData));
				
				   iRet = pArchive->ExtractProc(iNum,aTempData,ArchToMem);
				if(iRet!=CAF_ERROR_OK)
					{
					pErroText = "ExtractProc(...)";
					throw 0x800C;
					}

				if(iMemLast!=0)							// Wurde ein Callback mit 0 aufgerufen
					{
					pErroText = "No-Callback-Zero-Call";
					throw 0x804B;
					}

				if(iMemRdPos!=aFileSize[iIdx])
					{
					pErroText = "Wrong-File-Size";
					throw 0x800D;
					}				
														// Passt der Proc-Export 
				if(memcmp(aTempData,aFileData[iIdx],aFileSize[iIdx]))
					{
					pErroText = "memcmp(...)";
					throw 0x800E;
					}				
				
				DeleteFile(cFileName);
				_stprintf(cFileTemp+iPathLen,_T("Test%i.orig"),iIdx);
				
				   iRet = pArchive->Extract(iNum,cFileName);
				if(iRet!=CAF_ERROR_OK)
					{
					pErroText = "Extract(...)";
					throw 0x800F;
					}

				if(ExistFile(cFileName)!=1)
					{
					pErroText = "ExistFile(...)";
					throw 0x8010;
					}

				if(LengthFile(cFileName)!=aFileSize[iIdx])	
					{
					pErroText = "LengthFile(...)";
					throw 0x8011;
					}
														
				if(!CmpFileData(cFileName,cFileTemp))	// Passt der Datei-Export
					{
					pErroText = "CmpFileData(...)";
					throw 0x8012;
					}
				}

			   iRet = pArchive->Change(CAF_WRITE);		// Zum Schreibmodus
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Change(...)";
				throw 0x8016;
				}

			   iRet = pArchive->Del(_T("Test0.tmp"));
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Del(Test0.tmp)";
				if(pArchive!=&sCab)
				throw 0x8017;
				}

			   iRet = pArchive->Del(_T("Test2.tmp"));
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Del(Test2.tmp)";
				throw 0x8018;
				}

			   iRet = pArchive->Del(_T("Test5.tmp"));
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Del(Test5.tmp)";
				throw 0x8018;
				}

			_stprintf(cFileName+iPathLen,_T("Test5.tmp"));

			   iRet = pArchive->AddFile(cFileName,_T("Test5.tmp"));
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "AddFile(Test5.tmp)";
				throw 0x8050;
				}
		
			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Prfen ob jetzt im Schreibmodus die Dateien fehlen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);
				iNum = pArchive->Find(cFileName+iPathLen);

				if(iNum<0)
					{
					if(iIdx==0)continue;
					if(iIdx==2)continue;
					pErroText = "Find()";
					throw 0x8019;
					}
				else{
					if(pArchive->Len(iNum)!=aFileSize[iIdx])
						{
						pErroText = "Len()";
						throw 0x8020;
						}

					if(pArchive->IsDel(iNum))
						{
						pErroText = "IsDel()";
						throw 0x804F;
						}

					if(iIdx!=0)
					if(iIdx!=2)continue;
					throw 0x801E;
					}
				}
			
			   iRet = pArchive->Change(CAF_READ);		// Zum Lesemodus
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Change()";
				throw 0x8044;
				}

			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Prfen ob jetzt im Lesemodus die Dateien fehlen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);
				iNum = pArchive->Find(cFileName+iPathLen);

				if(iNum<0)
					{
					if(iIdx==0)continue;
					if(iIdx==2)continue;
					pErroText = "Find()";
					throw 0x8047;
					}
				else{
					if(pArchive->Len(iNum)!=aFileSize[iIdx])
						{
						pErroText = "Len()";
						throw 0x8020;
						}

					if(iIdx!=0)
					if(iIdx!=2)continue;
					pErroText = "Find()>=0";
					throw 0x8048;
					}
				}
			
			   iRet = pArchive->Close();				// Achive schliesen
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Close()";
				throw 0x801A;
				}
			
			
			   iRet = pArchive->Open(cFilePath,CAF_READ);
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "Open(...,CAF_READ)";
				throw 0x801B;
				}

			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Prfen ob jetzt die Dateien fehlen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);
				iNum = pArchive->Find(cFileName+iPathLen);

				if(iNum<0)
					{
					if(iIdx==0)continue;
					if(iIdx==2)continue;
					pErroText = "Find()";
					throw 0x801C;
					}
				else{
					if(pArchive->Len(iNum)!=aFileSize[iIdx])
						{
						pErroText = "Len()";
						throw 0x801F;
						}

					if(iIdx!=0)
					if(iIdx!=2)continue;
					pErroText = "Find()>=0";
					throw 0x801D;
					}
				}

			   iRet = pArchive->Close();				// Achive schliesen
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Close()";
				throw 0x8025;
				}
			
			   iSize = LoadMemory(cFilePath,aTempFile,sizeof(aTempFile));
			if(iSize<=0)
				{
				pErroText = "LoadMemory()";
				throw 0x8026;
				}

														// Aus Speicher ffnen
			   iRet = pArchive->OpenMem(aTempFile,iSize);
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "OpenMem(...)";
				throw 0x8027;
				}

			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Prfen ob jetzt die Dateien fehlen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);
				iNum = pArchive->Find(cFileName+iPathLen);

				if(iNum<0)
					{
					if(iIdx==0)continue;
					if(iIdx==2)continue;
					pErroText = "Find()";
					throw 0x8028;
					}
				else{
					if(pArchive->Len(iNum)!=aFileSize[iIdx])
						{
						pErroText = "Len()";
						throw 0x8029;
						}

					if(iIdx!=0)
					if(iIdx!=2)continue;
					pErroText = "Find()>=0";
					throw 0x802A;
					}

				if(pArchive->IsDir(iNum))				// Typ prfen
					{
					pErroText = "IsDir()";
					throw 0x802B;
					}

				memset(aTempData,0,sizeof(aTempData));						
			
				   iRet = pArchive->ExtractMem(iNum,aTempData,sizeof(aTempData));
				if(iRet!=CAF_ERROR_OK)
					{
					pErroText = "ExtractMem()";
					throw 0x802C;
					}
														// Passt der Speicher
				if(memcmp(aTempData,aFileData[iIdx],aFileSize[iIdx]))
					{
					throw 0x802D;
					}				
				
				iMemRdPos = 0;
				memset(aTempData,0,sizeof(aTempData));
				
				   iRet = pArchive->ExtractProc(iNum,aTempData,ArchToMem);
				if(iRet!=CAF_ERROR_OK)
					{
					pErroText = "ExtractProc()";
					throw 0x802E;
					}

				if(iMemRdPos!=aFileSize[iIdx])
					{
					pErroText = "Worng-File-Size";
					throw 0x802F;
					}				
														// Passt der Proc-Export 
				if(memcmp(aTempData,aFileData[iIdx],aFileSize[iIdx]))
					{
					pErroText = "memcmp(...)";
					throw 0x8030;
					}				
				
				DeleteFile(cFileName);
				_stprintf(cFileTemp+iPathLen,_T("Test%i.orig"),iIdx);
				
				   iRet = pArchive->Extract(iNum,cFileName);
				if(iRet!=CAF_ERROR_OK)
					{
					pErroText = "Extract(...)";
					throw 0x8031;
					}

				if(ExistFile(cFileName)!=1)
					{
					pErroText = "ExistFile(...)";
					throw 0x8032;
					}

				if(LengthFile(cFileName)!=aFileSize[iIdx])	
					{
					pErroText = "LengthFile(...)";
					throw 0x8033;
					}
														
				if(!CmpFileData(cFileName,cFileTemp))	// Passt der Datei-Export
					{
					pErroText = "CmpFileData(...)";
					throw 0x8034;
					}
				}

			   iRet = pArchive->Close();				// Achive schliesen
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Close(...)";
				throw 0x803B;
				}

			sFat.SetOption(0,sizeof(aTempFile)/512-4);
			memset(aTempFile,0,sizeof(aTempFile));
														// Im Speicher erstellen
			   iRet = pArchive->OpenMem(aTempFile,sizeof(aTempFile),0);
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "OpenMem(CAF_CREATE)";
				throw 0x8035;
				}

			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Dateien einfgen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);

				if(iIdx==0)
					{
					pErroText = "AddPath(...)";
					iRet = pArchive->AddPath(cFileName);
					iNum = 0x8036; 
					}
				else if(iIdx==9 || iIdx==12)
					{
					pErroText = "AddMem(...)";
					iRet = pArchive->AddMem(cFileName,aFileData[iIdx],aFileSize[iIdx]);
					iNum = 0x8037; 
					}
				else if(iIdx==7 || iIdx==10)
					{
					pErroText = "AddProc(...)";
					iMemRdPos	= 0;
					iMemWrMax	= aFileSize[iIdx];
					iRet = pArchive->AddProc(cFileName,aFileData[iIdx],MemToArch);

					if(iRet==CAF_ERROR_COMP)			// Unkomprimiert versuchen	
						{
						FfFindStruct	sInfo;

						sInfo.dwAttr		= SYS_READ|SYS_WRITE|SYS_VOLUME;	
						sInfo.dwSize		= aFileSize[iIdx];	
						sInfo.dwTimeAccess	= 0;
						sInfo.dwTimeCreate	= 0;
						sInfo.dwTimeWrite	= 0;
						sInfo.cName[0]		= 0;
						iMemRdPos			= 0;

						iRet = pArchive->AddProc(cFileName,aFileData[iIdx],MemToArch,(CArchiveFile::FileData*)&sInfo);
						}

					iNum = 0x8038; 						
					}
				else{
					pErroText = "AddFile(...)";
					iRet = pArchive->AddFile(cFileName);
					iNum = 0x8039; 						
					}

				if(iRet!=CAF_ERROR_OK)
					{
					throw iNum;
					}
				}

			   iRet = pArchive->Flush();				// Die neuen Dateien wegschreien
			if(iRet!=CAF_ERROR_OK)	
				{
				pErroText = "Flush(...)";
				throw 0x8045;
				}

			
			   iRet = pArchive->Close();				// Achive schliesen
			if(iRet!=CAF_ERROR_OK)				
				{
				pErroText = "Close(...)";
				throw 0x803C;
				}

			   iSize = pArchive->GetMemSize();			// Passt die erzeugte Speichergre
			if(iSize<=0x8000 || iSize>=sizeof(aTempFile))
				{
				pErroText = "Worong-Memory-Size";
				iNum = 0x803A;
				}

			_stprintf(cFilePath+iPathLen,_T("TestArch.mem.%s"),pSuffix);
			SaveMemory(cFilePath,aTempFile,iSize);

			   iRet = pArchive->Open(cFilePath,CAF_READ);
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "Open(...,CAF_READ)";
				throw 0x8051;
				}

			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Dateien lschen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp"),iIdx);
				if(iIdx==0)
						RemoveDirectory(cFileName);
				else	DeleteFile     (cFileName);
				}

			cFileName[iPathLen  ] = 0;

			   iRet = pArchive->ExtractAll(cFileName);	// Alles extrahieren
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "ExtractAll(...)";
				throw 0x803D;
				}


			for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)	// Dateien prfen
				{
				_stprintf(cFileName+iPathLen,_T("Test%i.tmp" ),iIdx);
				_stprintf(cFileTemp+iPathLen,_T("Test%i.orig"),iIdx);
				iNum = pArchive->Find(cFileName+iPathLen);

				if(iNum<0)
					{
					if(pArchive==&sCab)continue;
					pErroText = "Find(...)";
					throw 0x803E;
					}
				
				if(pArchive->Len(iNum)!=aFileSize[iIdx])
					{
					pErroText = "Len(...)";
					throw 0x803F;
					}

				if(iIdx==0)								// Pfad prfen
					{
					if(!pArchive->IsDir(iNum))
						{
						pErroText = "IsDir(...)";
						throw 0x8040;
						}

					if(ExistFile(cFileName)!=2)
						{
						pErroText = "ExistFile(...)";
						throw 0x804A;
						}

					continue;
					}
					
				if(pArchive->IsDir(iNum))				// Typ prfen
					{
					pErroText = "IsDir(...)";
					throw 0x8041;
					}

				memset(aTempData,0,sizeof(aTempData));
			
				   iRet = pArchive->ExtractMem(iNum,aTempData,sizeof(aTempData));
				if(iRet!=CAF_ERROR_OK)
					{
					pErroText = "ExtractMem(...)";
					throw 0x8042;
					}
														// Passt der Speicher
				if(memcmp(aTempData,aFileData[iIdx],aFileSize[iIdx]))
					{
					pErroText = "memcmp(...)";
					throw 0x8043;
					}				
													
				if(!CmpFileData(cFileName,cFileTemp))	// Passt der Datei-Export
					{
					pErroText = "CmpFileData(...)";
					throw 0x8049;
					}
				}

			   iRet = pArchive->Close();
			if(iRet!=CAF_ERROR_OK)
				{
				pErroText = "Close(...)";
				throw 0x8052;
				}
			}
		catch(int iError)
			{
			iErrors++;
			strawcpy(cErrorText,pErroText,ENTRIES(cErrorText));
			printf("Error %04Xh at %i for %s at: %s\n",iError,iPos,pSuffix,cErrorText);
			}
		catch(...)
			{
			iErrors++;
			_tprintf(_T("Unknown exeption at %i for %s\n"),iPos,pSuffix);
			}
		}



//******************** Speicher freigeben *************************************


	for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)			// Dateien lschen
		{
		_stprintf(cFileName+iPathLen,_T("Test%i.tmp" ),iIdx);
		RemoveFile     (cFileName);
		RemoveDirectory(cFileName);
		_stprintf(cFileName+iPathLen,_T("Test%i.orig"),iIdx);
		RemoveFile     (cFileName);
		RemoveDirectory(cFileName);
		}

	for(iIdx=0;iIdx<ENTRIES(aFileSize);iIdx++)			// Speicher freigeben
		{
		delete aFileData[iIdx];
		}



	_tprintf(_T("\n\n---------------------------------------------\n"));

	_tprintf(_T("\nEs gab %i Fehler\n"),iErrors);

	_getch();

return 0;
}

