//*****************************************************************************
//*
//*
//*     WinDebug.c
//*
//*
//*****************************************************************************
//
//  Dieses Modul dient zum Mittloggen von Core-Dumps.
//  Bei einem Dump wird im Verzeichnis der Programm-Datei ein File
//  mit dem Namen des Programms und der Endung *.core angelegt.
//  Es enthlt Informationen wo der Dump statt gefunden hat.
//  Ist das Debuggen aktiviert so zeig ein Fenster an wo der Dump
//  war, und dann wird der Debugger gestartet.
//	ACHTUNG: fr Stackinfos muss das *.pdb File im selben Verzeichnis
//	liegen wie die EXE-Datei.
//
//      *) Mit dem Registry Eintrag
//
//              [HKEY_LOCAL_MACHINE\SOFTWARE\AZ\Debug]
//                  "Enable"=dword:1
//
//         wird das Debuggen fr alle Prozesse freigegeben.
//		   Ist "Enable" auf 2 gesetzt so wird der Debugger direkt gestartet
//		   ohne ein Core-File zu schreiben.
//
//      *) Mit der Datei .\cache\axdebug.ini kann fr einzelne Prozesse
//         das Debuggen freigegeben werden, z.B.:
//
//              sps=runtime
//              flash==runtime
//
//      *) Fr die meisten Prozesse kann das Debuggen mit der Systemvariabel
//         $$_ax_stack_dbg freigegeben werden
//
//
//  So kann man das Mittloggen aktivieren:
//
//          #include "WinDebug.h"
//          ...
//          int  iDebuggerOn=0;
//          int *pDebuggerOn=&iDebuggerOn;
//          ...
//          WinInitCoreDumpInfo(&pDebuggerOn);
//
//  ber die Variable iDebuggerOn kann man das Debbugen ein- bzw. ausschalten
//
//          iDebuggerOn=0;      // Debuggen aus
//          iDebuggerOn=1;      // Debuggen ein
//
#undef	 UNICODE

#include <windows.h>
#include <winerror.h>
#include <mmsystem.h>
#include <imagehlp.h>
#include <winreg.h>

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <time.h>

#include "WinDebug.h"






#define MAX_PIDS			64
#define SYM_BUFF_SIZE		512
#define BUFF_SIZE			5000
#define REGISTRY_PATH		"SOFTWARE\\AZ\\Debug"
//#define CORE_PATH			"D:\\MyLog"

#pragma data_seg(".smem")
DWORD	dwDupedPids[MAX_PIDS]={0};
#pragma data_seg()

#pragma comment(lib, "imagehlp.lib")
#pragma comment(lib, "kernel32.lib")
#pragma comment(lib, "advapi32.lib")
#pragma comment(lib, "user32.lib")


#if _MSC_VER >= 1300
#pragma comment ( lib, "dbghelp.lib" )
#include <DbgHelp.h>
#endif


typedef BOOL (__stdcall *SymGetLine_Type)(HANDLE,DWORD,PDWORD,PIMAGEHLP_LINE);
typedef int  (__cdecl   *SortProcType   )(const void*,const void*);



static STACKFRAME			sStackFrame;
static IMAGEHLP_LINE		sHelpLine;
static BYTE					aSymbols[SYM_BUFF_SIZE];
static PIMAGEHLP_SYMBOL		pSym = (PIMAGEHLP_SYMBOL)&aSymbols;
static int				  **pEnableDebugger=0;
static BOOL					bViewDebugBox=0;
static char					cTextString[50000]={0};
static SymGetLine_Type		SymGetLineFromAddr_=0;

static void RemoveOldest();
static BOOL WriteMiniDump(LPEXCEPTION_POINTERS pExceptionInfo);
static int  ExceptionHandlerFunction(int* pEnable, LPEXCEPTION_POINTERS pExInfo, unsigned long ulExceptionCode);
static int  InternalGetStackTraceString(EXCEPTION_POINTERS *pExPtrs,char *pTraceString,int iTraceStringLen);

#ifdef	_MSC_VER
#pragma warning( disable :4996 )
#endif

#if	   _MSC_VER < 1300
BOOL WINAPI IsDebuggerPresent(VOID);

typedef enum
	{
	ModuleCallback,
	ThreadCallback,
	ThreadExCallback,
	IncludeThreadCallback,
	IncludeModuleCallback,
	MemoryCallback,
	CancelCallback,
	WriteKernelMinidumpCallback,
	KernelMinidumpStatusCallback,
	RemoveMemoryCallback,
	IncludeVmRegionCallback,
	IoStartCallback,
	IoWriteAllCallback,
	IoFinishCallback,
	ReadMemoryFailureCallback,
	SecondaryFlagsCallback
	}MINIDUMP_CALLBACK_TYPE;

typedef enum
	{
	ModuleWriteModule						= 0x0001,
	ModuleWriteDataSeg        				= 0x0002,
	ModuleWriteMiscRecord     				= 0x0004,
	ModuleWriteCvRecord       				= 0x0008,
	ModuleReferencedByMemory  				= 0x0010,
	ModuleWriteTlsData        				= 0x0020,
	ModuleWriteCodeSegs       				= 0x0040
	}MODULE_WRITE_FLAGS;


typedef enum
	{
	MiniDumpNormal                          = 0x00000000,
	MiniDumpWithDataSegs                    = 0x00000001,
	MiniDumpWithFullMemory                  = 0x00000002,
	MiniDumpWithHandleData                  = 0x00000004,
	MiniDumpFilterMemory                    = 0x00000008,
	MiniDumpScanMemory                      = 0x00000010,
	MiniDumpWithUnloadedModules             = 0x00000020,
	MiniDumpWithIndirectlyReferencedMemory  = 0x00000040,
	MiniDumpFilterModulePaths               = 0x00000080,
	MiniDumpWithProcessThreadData           = 0x00000100,
	MiniDumpWithPrivateReadWriteMemory      = 0x00000200,
	MiniDumpWithoutOptionalData             = 0x00000400,
	MiniDumpWithFullMemoryInfo              = 0x00000800,
	MiniDumpWithThreadInfo                  = 0x00001000,
	MiniDumpWithCodeSegs                    = 0x00002000,
	MiniDumpWithoutAuxiliaryState           = 0x00004000,
	MiniDumpWithFullAuxiliaryState          = 0x00008000,
	MiniDumpWithPrivateWriteCopyMemory      = 0x00010000,
	MiniDumpIgnoreInaccessibleMemory        = 0x00020000,
	MiniDumpWithTokenInformation            = 0x00040000,
	MiniDumpWithModuleHeaders               = 0x00080000,
	MiniDumpFilterTriage                    = 0x00100000,
	MiniDumpValidTypeFlags                  = 0x001fffff
	}MINIDUMP_TYPE;

typedef struct
	{
	ULONG64					BaseAddress;
	ULONG64					AllocationBase;
	ULONG32					AllocationProtect;
	ULONG32					__alignment1;
	ULONG64					RegionSize;
	ULONG32					State;
	ULONG32					Protect;
	ULONG32					Type;
	ULONG32					__alignment2;
	} MINIDUMP_MEMORY_INFO,*PMINIDUMP_MEMORY_INFO;

typedef struct
	{
	ULONG					ProcessId;
	HANDLE					ProcessHandle;
	ULONG					CallbackType;
	LPVOID					Param;
	}MINIDUMP_CALLBACK_INPUT,*PMINIDUMP_CALLBACK_INPUT;

typedef struct _MINIDUMP_USER_STREAM_INFORMATION
	{
	ULONG					UserStreamCount;
	LPVOID					UserStreamArray;
	}MINIDUMP_USER_STREAM_INFORMATION, *PMINIDUMP_USER_STREAM_INFORMATION;
typedef struct
	{
	union
		{
		ULONG				ModuleWriteFlags;
		ULONG				ThreadWriteFlags;
		ULONG				SecondaryFlags;
		struct
			{
		ULONG64 			MemoryBase;
		ULONG   			MemorySize;
			};

		struct
			{
		BOOL				CheckCancel;
		BOOL				Cancel;
			};
		HANDLE				Handle;
		};

	struct
		{
		MINIDUMP_MEMORY_INFO VmRegion;
		BOOL                 Continue;
		};
	HRESULT Status;
	}MINIDUMP_CALLBACK_OUTPUT, *PMINIDUMP_CALLBACK_OUTPUT;

typedef  BOOL (CALLBACK *MINIDUMP_CALLBACK_ROUTINE)(PVOID CallbackParam,const PMINIDUMP_CALLBACK_INPUT CallbackInput,PMINIDUMP_CALLBACK_OUTPUT CallbackOutput);

typedef struct
	{
	DWORD					ThreadId;
	LPEXCEPTION_POINTERS	ExceptionPointers;
	BOOL					ClientPointers;
	}MINIDUMP_EXCEPTION_INFORMATION, *PMINIDUMP_EXCEPTION_INFORMATION;

typedef struct
	{
	MINIDUMP_CALLBACK_ROUTINE CallbackRoutine;
	PVOID                     CallbackParam;
	}MINIDUMP_CALLBACK_INFORMATION, *PMINIDUMP_CALLBACK_INFORMATION;

typedef  BOOL (WINAPI *MiniDumpWriteDumpType)(HANDLE hProcess,DWORD ProcessId,HANDLE hFile,MINIDUMP_TYPE DumpType,PMINIDUMP_EXCEPTION_INFORMATION ExceptionParam,PMINIDUMP_USER_STREAM_INFORMATION UserStreamParam,PMINIDUMP_CALLBACK_INFORMATION CallbackParam);

#endif


#ifndef	KEY_WOW64_64KEY
#define	KEY_WOW64_64KEY	  0x0100				// 32-Bit Programme nicht auf WOW64 Umleitung gehen
#endif

//*****************************************************************************
//*
//*		UnhandledExceptionFilter
//*
//*****************************************************************************
//	Diese Funktion wird aufgerufen wenn ein Core-Dump passiert
static LONG WINAPI UnhandledExceptionProc(LPEXCEPTION_POINTERS pExceptionInfo)
{
int		*pPtr=0;


	if(pEnableDebugger)
	if(!IsBadReadPtr( pEnableDebugger,4))
	if(!IsBadReadPtr(*pEnableDebugger,4))
		{
		pPtr=*pEnableDebugger;
		}

	SetErrorMode(SEM_NOALIGNMENTFAULTEXCEPT);
	#if	_MSC_VER >= 1300
	WriteMiniDump(pExceptionInfo);
	#endif
	ExceptionHandlerFunction(pPtr,pExceptionInfo,pExceptionInfo->ExceptionRecord->ExceptionCode);



return EXCEPTION_CONTINUE_SEARCH;
}

//*****************************************************************************
//*
//*		WinStopOnError
//*
//*****************************************************************************
void WinStopOnError(int iEnable)
{

	if(!iEnable)
		{
		SetErrorMode(0);
		}
	else{
		SetErrorMode(SEM_FAILCRITICALERRORS|SEM_NOGPFAULTERRORBOX|SEM_NOOPENFILEERRORBOX);
		}

}



//*****************************************************************************
//*
//*		InitSymEngine
//*
//*****************************************************************************
static LONG InitSymEngine(void)
{
DWORD	dwOpts;



    dwOpts = SymGetOptions();					// Set up the symbol engine.


												// Always defer loading to make life faster.
    SymSetOptions(dwOpts|SYMOPT_DEFERRED_LOADS|SYMOPT_LOAD_LINES);

												// Initialize the symbol engine.
    if(!SymInitialize(GetCurrentProcess(),NULL,TRUE))
		{
		return GetLastError();
		}


return 0;
}


//*****************************************************************************
//*
//*		CleanupSymEngine
//*
//*****************************************************************************
// Cleans up the symbol engine if needed.
static void CleanupSymEngine (void)
{
    SymCleanup(GetCurrentProcess()) ;
}


//*****************************************************************************
//*
//*		GetCoreName
//*
//*****************************************************************************
//	Holt den Dateinamen fr eine CORE-Datei
//	pBuffer		: Puffer fr den Dateinamen
//	pSuffix		: Suffix fr die Datei (z.B. ".core")
//	Ergibt die Lnge des Dateinamens in Zeichen
static int GetCoreName(char *pBuffer,const char *pSuffix)
{
char		cPath[256];
SYSTEMTIME	sTime;
int			iLen;
int			iPos;


	#ifdef	CORE_PATH

	strcpy(cPath,CORE_PATH);
	iLen = strlen(cPath);

	#else

	iLen = GetModuleFileName(GetModuleHandle(NULL),cPath,sizeof(cPath));

	for(iLen--;iLen>0;iLen--)
		{
		if(cPath[iLen]=='/' )break;
		if(cPath[iLen]=='\\')break;
		if(cPath[iLen]!='.' )continue;
		   cPath[iLen]=0;
		   break;
		}

	#endif

	iPos = iLen;

	if(!pSuffix)
		{
		iLen = sprintf(pBuffer,"%s__",cPath);
		}
	else{
		GetLocalTime(&sTime);
		iLen = sprintf(pBuffer,"%s__%04u-%02u-%02u__%02u%02u%02u%s",cPath,sTime.wYear,sTime.wMonth,sTime.wDay,sTime.wHour,sTime.wMinute,sTime.wSecond,pSuffix);
		}

	for(;iPos<iLen;iPos++)
		{
		if(pBuffer[iPos]==':'   )pBuffer[iPos]='-';
		if(pBuffer[iPos]!='/' && pBuffer[iPos]!='\\')continue;
		   pBuffer[iPos]='_';
		}


return iLen;
}


//*****************************************************************************
//*
//*		GetFirstStackString
//*
//*****************************************************************************
static int GetFirstStackString(EXCEPTION_POINTERS *pExPtrs,char *pTraceString,int iTraceStringLen)
{


    ZeroMemory(&sStackFrame,sizeof(STACKFRAME));

    sStackFrame.AddrPC    .Offset   = pExPtrs->ContextRecord->Eip ;
    sStackFrame.AddrPC    .Mode     = AddrModeFlat                ;
    sStackFrame.AddrStack .Offset   = pExPtrs->ContextRecord->Esp ;
    sStackFrame.AddrStack .Mode     = AddrModeFlat                ;
    sStackFrame.AddrFrame .Offset   = pExPtrs->ContextRecord->Ebp ;
    sStackFrame.AddrFrame .Mode     = AddrModeFlat                ;


return InternalGetStackTraceString(pExPtrs,pTraceString,iTraceStringLen);
}


//*****************************************************************************
//*
//*		GetNextStackString
//*
//*****************************************************************************
static int GetNextStackString(EXCEPTION_POINTERS *pExPtrs,char *pTraceString,int iTraceStringLen)
{
return InternalGetStackTraceString(pExPtrs,pTraceString,iTraceStringLen);
}


//*****************************************************************************
//*
//*		GetStatckParam
//*
//*****************************************************************************
//	Erzeugt einen Text mit dem angegeben Parameter
//	dwParam	: Wert des Parameters
//	pBuffer	: Zeiger auf Zielpuffer
//	iMax	: Gre des Zielpuffers
static int GetStatckParam(DWORD dwParam,int iNr,char *pBuffer,int iMax)
{
#define 	HEX_SIZE	20
char	   *pStart=pBuffer;
BYTE	   *pData,bByte;
int			i;


	if(iMax<10+HEX_SIZE*3+HEX_SIZE/4+20+4)return 0;

	pBuffer+=sprintf(pBuffer,"\n\tParam%i=%08X : ",iNr+1,dwParam);
	pData=(LPBYTE)dwParam;

	if(IsBadReadPtr(pData,HEX_SIZE))return pBuffer-pStart;

	for(i=0;i<HEX_SIZE;i++)
		{
		pBuffer+=sprintf(pBuffer,"%02X ",pData[i]);
		if((i&3)==3){pBuffer[0]=' ';pBuffer++;}
		}

	pBuffer[0]=(char)160;
	pBuffer++;

	for(i=0;i<HEX_SIZE;i++)
		{
		bByte=pData[i];
		if(bByte<' ')bByte='.';
		pBuffer[0]=bByte;
		pBuffer++;
		}


return pBuffer-pStart;
}


//*****************************************************************************
//*
//*		InternalGetStackTraceString
//*
//*****************************************************************************
//	Erzeugt die Core-Infos fr einen Aufruf
static int InternalGetStackTraceString(EXCEPTION_POINTERS *pExPtrs,char *pTraceString,int iTraceStringLen)
{
char		cText[BUFF_SIZE];
DWORD		dwTemp,dwDisp;
BOOL		bSwRet=FALSE;
int			i,iCurr=0;
HMODULE		hDll;




	if(!pExPtrs || !pTraceString)return 0;



	if(!SymGetLineFromAddr_)					// Unter NT ist SymGetLineFromAddr nicht vorhanden !!!
		{
		   hDll=GetModuleHandle("imagehlp.dll");
		if(hDll)SymGetLineFromAddr_=(SymGetLine_Type)GetProcAddress(hDll,"SymGetLineFromAddr");
		}




												// If the source and line functions are used, then
												//  StackWalk can access violate.
    bSwRet = StackWalk ( IMAGE_FILE_MACHINE_I386,
                         GetCurrentProcess(),
                         GetCurrentThread(),
                         &sStackFrame,
                         pExPtrs->ContextRecord,
                         NULL,
                         SymFunctionTableAccess,
                         SymGetModuleBase,
                         NULL) ;

    if(bSwRet==FALSE || sStackFrame.AddrFrame.Offset==0)
		{
		iCurr += sprintf(cText+iCurr,"\tCan't walk through stack (err=%i)\n",GetLastError());
		return 0;
		}



												// At a minimum, put the address in.
    iCurr += sprintf(cText+iCurr,"%04X:%08X",pExPtrs->ContextRecord->SegCs,sStackFrame.AddrPC.Offset);



    ZeroMemory(pSym,SYM_BUFF_SIZE) ;
    pSym->SizeOfStruct  = sizeof ( IMAGEHLP_SYMBOL ) ;
    pSym->MaxNameLength = SYM_BUFF_SIZE - sizeof ( IMAGEHLP_SYMBOL ) ;



    if(SymGetSymFromAddr(GetCurrentProcess(),sStackFrame.AddrPC.Offset,&dwDisp,pSym))
		{
        dwTemp =  strlen(pSym->Name ) ;
        if(dwTemp > (DWORD)(BUFF_SIZE-iCurr-20))return 0;

		if(dwDisp>0)
				iCurr += sprintf(cText+iCurr,",\"%s()\" %s%d byte(s)\n",pSym->Name,(dwDisp&0x80000000)? "":"+",dwDisp);
        else	iCurr += sprintf(cText+iCurr,",%s",pSym->Name);


		if(SymGetLineFromAddr_)
		if(SymGetLineFromAddr_(GetCurrentProcess(),sStackFrame.AddrPC.Offset,&dwDisp,&sHelpLine))
			{
			dwTemp = strlen(sHelpLine.FileName);
			if(dwTemp > (DWORD)(BUFF_SIZE-iCurr-20))return 0;
			iCurr += sprintf(cText+iCurr,"\n\tCall   %s(%i)",sHelpLine.FileName,sHelpLine.LineNumber);
			}

		for(i=0;i<4;i++)
			{
			iCurr+=GetStatckParam(sStackFrame.Params[i],i,cText+iCurr,BUFF_SIZE-iCurr-4);
			}

		cText[iCurr]='\n';
		iCurr++;
		}
	else{
		iCurr += sprintf(cText+iCurr,"\tError at SymGetSymFromAddr(...) err=%i %s\n",GetLastError(),(GetLastError()==487)? "(can't load *.pdb file)":"");
		return 0;
		}


	if(strlen(cText)<(unsigned)iTraceStringLen)
		{
		strcpy(pTraceString,cText);
		}
	else{
		strncpy(pTraceString,cText,iTraceStringLen-1);
		pTraceString[iTraceStringLen-1] = '\0';
		}


return 1;
}


//*****************************************************************************
//*
//*		ExceptionHandlerFunction
//*
//*****************************************************************************
//	Haupfunktion die die Core-Dump-Infos erzeugt
static int ExceptionHandlerFunction(int *pEnable,LPEXCEPTION_POINTERS pExInfo,unsigned long ulExceptionCode)
{
int							i=0,j=0,x=0;
char						cStackString [5000]={0};
char						cBuffer      [1000]={0};
char						cCoreFileName[ 256]={0};
char						cModuleName  [ 256]={0};
char						cText        [  32];
struct _EXCEPTION_RECORD   *pNextRecord		= NULL;
LONG						ulError			= 0;
BOOL						bSymLoaded      = TRUE;
BOOL						bStartDebugger  = FALSE;
BOOL						bViewMessageBox = FALSE;
BOOL						bStartDirect    = FALSE;
DWORD						dwSize,dwType;
DWORD						dwValue=0;
HANDLE						hFile;
HKEY						hKey;
UINT						uLen;
LONG						lOk;


	   lOk=RegOpenKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_PATH,0,KEY_READ|KEY_WOW64_64KEY,&hKey);
	if(lOk==ERROR_SUCCESS)
		{
		dwSize	= sizeof(DWORD);
		dwType	= REG_DWORD;
		lOk		= RegQueryValueEx(hKey,"Enable",0,&dwType,(unsigned char*)&dwValue,&dwSize);

		if(lOk==ERROR_SUCCESS && dwValue!=0)bStartDebugger=TRUE;
		if(lOk==ERROR_SUCCESS && dwValue==2)bStartDirect  =TRUE;

		RegCloseKey(hKey);
		}

	if(bStartDirect || IsDebuggerPresent())							// Is the Process in the debugger
		{
		return EXCEPTION_EXECUTE_HANDLER;
		}

	SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_BELOW_NORMAL);


	if(!bStartDebugger && !(pEnable && *pEnable))					// Write Process Dump Flag
		{
		for(i=0;i<MAX_PIDS;i++)
			{
			if(dwDupedPids[i])continue;
			dwDupedPids[i]=GetCurrentProcessId();
			break;
			}
		}



	if(pExInfo)
		{
		sprintf(cTextString,"ERROR_CODE=%08X\n",ulExceptionCode);

		if(ulExceptionCode == EXCEPTION_ACCESS_VIOLATION)			// For that look in MSDN why
			{
			sprintf(cBuffer,"Process tried to %s Address [0%Xh]\n",
				(pExInfo->ExceptionRecord->ExceptionInformation[0] ? "write to" : "read from"),
				 pExInfo->ExceptionRecord->ExceptionInformation[1] );
			strcat(cTextString,cBuffer);
			}

		if(ulExceptionCode == EXCEPTION_BREAKPOINT)
			{
			strcat(cTextString,"User breakpoint\n");
			}

		if(ulExceptionCode == EXCEPTION_INT_DIVIDE_BY_ZERO)
			{
			strcat(cTextString,"Division by zero\n");
			}
		if(ulExceptionCode == EXCEPTION_FLT_DIVIDE_BY_ZERO)
			{
			strcat(cTextString,"Floating piont division by zero\n");
			}

		if(ulExceptionCode == EXCEPTION_STACK_OVERFLOW)
			{
			strcat(cTextString,"Stack overflow\n");
			}

		if(ulExceptionCode == EXCEPTION_IN_PAGE_ERROR)
			{
			sprintf(cBuffer,"Page error at Address [0%Xh]\n",pExInfo->ExceptionRecord->ExceptionInformation[1]);
			strcat(cTextString,cBuffer);
			}


		uLen   = sprintf(cBuffer     ,"\n\tEAX=%08X"  ,pExInfo->ContextRecord->Eax);
		uLen  += sprintf(cBuffer+uLen,"\n\tEBX=%08X"  ,pExInfo->ContextRecord->Ebx);
		uLen  += sprintf(cBuffer+uLen,"\n\tECX=%08X"  ,pExInfo->ContextRecord->Ecx);
		uLen  += sprintf(cBuffer+uLen,"\n\tEDI=%08X"  ,pExInfo->ContextRecord->Edx);
		uLen  += sprintf(cBuffer+uLen,"\n\tEDI=%08X"  ,pExInfo->ContextRecord->Edi);
		uLen  += sprintf(cBuffer+uLen,"\n\tESI=%08X"  ,pExInfo->ContextRecord->Esi);
		uLen  += sprintf(cBuffer+uLen,"\n\tEBP=%08X"  ,pExInfo->ContextRecord->Ebp);
		uLen  += sprintf(cBuffer+uLen,"\n\tESP=%08X"  ,pExInfo->ContextRecord->Esp);
		uLen  += sprintf(cBuffer+uLen,"\n\tEIP=%08X"  ,pExInfo->ContextRecord->Eip);
		uLen  += sprintf(cBuffer+uLen,"\n\tFlags=%08X",pExInfo->ContextRecord->EFlags);

		cBuffer[uLen] = '\n'; uLen++;
		cBuffer[uLen] = '\n'; uLen++;
		cBuffer[uLen] = 0;


		strcat(cTextString,cBuffer);
		ulError = InitSymEngine();

		if(ulError)
			{
			sprintf(cBuffer,"Symbol Information not loaded. ErrorCode = [%d]\n",ulError);
			strcat(cTextString,cBuffer);
			bSymLoaded = FALSE;
			}
		else{
			GetFirstStackString(pExInfo,cStackString,sizeof(cStackString));

			sprintf(cBuffer,"%s\n",cStackString);
			strcat(cTextString,cBuffer);

			while(GetNextStackString(pExInfo,cStackString,sizeof(cStackString)))
				{
				i=sprintf(cBuffer,"%s\n",cStackString);
				if(strlen(cTextString)+i+10>sizeof(cTextString))break;
				strcat(cTextString,cBuffer);
				}
			}


//******************** Write Core File ****************************************


		GetCoreName(cCoreFileName,".core");


		   hFile = CreateFileA(cCoreFileName,GENERIC_WRITE,FILE_SHARE_READ,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
		if(hFile!=INVALID_HANDLE_VALUE)
			{
			uLen = strlen(cTextString);
			WriteFile(hFile,cTextString,uLen,&dwSize,NULL);
			uLen = sprintf(cBuffer,"\n\n\n\tDate : %s\n\tTime : %s\n",_strdate(cText),_strtime(cText+16));
			WriteFile(hFile,cBuffer,uLen,&dwSize,NULL);
			CloseHandle(hFile);
			}


		if(bSymLoaded)
			{
			CleanupSymEngine();
			}

		bViewMessageBox = TRUE;

		RemoveOldest();
		}


//******************** Start Debugger and view Infos **************************


	if(bStartDebugger || (pEnable && *pEnable))
		{
		if(bViewMessageBox)
			{
			for(i=0,x=0;cTextString[i];i++)
				{
				if(cTextString[i]!='\n')continue;

				if(!strnicmp(cTextString+i+1,"\tCall ",12))
					{
					x=strlen(cTextString+i+1);
					memmove(cTextString+i,cTextString+i+1,x+1);
					i--;
					continue;
					}

				if(cTextString[i+1]=='\t')
					{
					j=strcspn(cTextString+i+1,"\n")+1;
					x=strlen(cTextString+i+j);
					memmove(cTextString+i,cTextString+i+j,x+1);
					i--;
					continue;
					}

				if(cTextString[i+1]=='\n')
					{
					x=strlen(cTextString+i+1);
					memmove(cTextString+i,cTextString+i+1,x+1);
					i--;
					continue;
					}
				}


			i = GetModuleFileName(GetModuleHandle(NULL),cModuleName,sizeof(cModuleName));

			for(;i>0;i--)
				{
				if(cModuleName[i]!='/' && cModuleName[i]!='\\')continue;
				i++;
				break;
				}

			sprintf(cBuffer,"Internal Error in %s",cModuleName+i);
			MessageBox(GetForegroundWindow(),cTextString,cBuffer,MB_OK|MB_ICONERROR);
			}

		WinStopOnError(0);
		return EXCEPTION_EXECUTE_HANDLER;
		}


	for(i=0;i<MAX_PIDS;i++)							// Remove Process Dump Flag
		{
		if(dwDupedPids[i]!=GetCurrentProcessId())continue;
		dwDupedPids[i]=0;
		}


	TerminateProcess(GetCurrentProcess(),-1);


return EXCEPTION_CONTINUE_SEARCH;
}

//*****************************************************************************
//*
//*		WinInitCoreDumpInfo
//*
//*****************************************************************************
//	Installiert das Mitloggen von Core-Dumps
//	pEnabled		: Zeiger auf Variabel auf die das Mitloggen ein/ausschaltet.
//					  Bein NULL ist das Mitloggen immer an.
void WinInitCoreDumpInfo(int **pEnabled)
{


	pEnableDebugger = pEnabled;
	SetErrorMode(SEM_NOALIGNMENTFAULTEXCEPT);
	SetUnhandledExceptionFilter(UnhandledExceptionProc);


}


//*****************************************************************************
//*
//*		WinIsProcessDumped
//*
//*****************************************************************************
//	Ergibt 1 wenn der Prozesss mit der ID ulPid gedumpt ist, und das Dump-Info
//	gerade geschrieben wird, und das Debuggen ausgeschalten ist.
int WinIsProcessDumped(unsigned long ulPid)
{
int i;


	for(i=0;i<MAX_PIDS;i++)							// Remove Process Dump Flag
		{
		if(dwDupedPids[i]==ulPid)return 1;
		}


return 0;
}

//*****************************************************************************
//*
//*		WinRaiseException
//*
//*****************************************************************************
//	Simuliert eine Exeption.
void WinRaiseException()
{

	//RaiseException(0x12345678,0,0,0);
	_asm{
		xor eax,eax;
		xor edx,edx;
		div eax;
		}

}

//#if	_MSC_VER >= 1300

//*****************************************************************************
//*
//*		MiniDumpCallback
//*
//*****************************************************************************
static BOOL CALLBACK MiniDumpCallback(LPVOID pParam,const PMINIDUMP_CALLBACK_INPUT pInput,PMINIDUMP_CALLBACK_OUTPUT pOutput)
{
BOOL	bRet;



	if(!pInput || !pOutput)										// Check parameters
		{
		return FALSE;
		}



	switch(pInput->CallbackType)								// Process the callbacks
		{
	case IncludeModuleCallback:									// Include the module into the dump

		bRet = TRUE;
		break;

	case IncludeThreadCallback:									// Include the thread into the dump

		bRet = TRUE;
		break;

	case ModuleCallback:										// Does the module have ModuleReferencedByMemory flag set ?

		if(!(pOutput->ModuleWriteFlags & ModuleReferencedByMemory))
			{
			pOutput->ModuleWriteFlags &= (~ModuleWriteModule);
			}

		bRet = TRUE;
		break;

	case ThreadCallback:										// Include all thread information into the minidump

		bRet = TRUE;
		break;

	case ThreadExCallback:										// Include this information

		bRet = TRUE;
		break;

	case MemoryCallback:										// We do not include any information here -> return FALSE

		bRet = FALSE;
		break;

	default:

		bRet = FALSE;
		break;
	}


return bRet;
}


//*****************************************************************************
//*
//*		WriteMiniDump
//*
//*****************************************************************************
static BOOL WriteMiniDump(LPEXCEPTION_POINTERS pExceptionInfo)
{
#if _MSC_VER < 1300
MiniDumpWriteDumpType			MiniDumpWriteDump;
#endif
CHAR							cPath[256];
HMODULE							hDebugDll;
MINIDUMP_CALLBACK_INFORMATION	sCbInfo;
MINIDUMP_EXCEPTION_INFORMATION	sInfo;
MINIDUMP_TYPE					iType;
HANDLE							hFile;
BOOL							bRet;



	GetCoreName(cPath,".mdmp");


	   hFile = CreateFileA(cPath,GENERIC_READ|GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
		{
		return FALSE;
		}


	#if _MSC_VER < 1300

	MiniDumpWriteDump	= NULL;

	   hDebugDll = LoadLibraryA("Dbghelp.dll");
	if(hDebugDll)
		{
		MiniDumpWriteDump = (MiniDumpWriteDumpType)GetProcAddress(hDebugDll,"MiniDumpWriteDump");
		}

	if(!MiniDumpWriteDump)
		{
		if(hDebugDll)FreeLibrary(hDebugDll);

			hDebugDll = LoadLibraryA("Dbgcore.dll");
		if(!hDebugDll)return FALSE;

			MiniDumpWriteDump = (MiniDumpWriteDumpType)GetProcAddress(hDebugDll,"MiniDumpWriteDump");
		if(!MiniDumpWriteDump)
			{
			FreeLibrary(hDebugDll);
			return FALSE;
			}
		}

	#endif

	sInfo.ThreadId			= GetCurrentThreadId();
	sInfo.ExceptionPointers = pExceptionInfo;
	sInfo.ClientPointers	= FALSE;

	sCbInfo.CallbackRoutine = (MINIDUMP_CALLBACK_ROUTINE)MiniDumpCallback;
	sCbInfo.CallbackParam	= 0;							// this example does not use the context


	iType =	MiniDumpWithCodeSegs				|			// the contents of all code sections of all executable modules loaded by the process will be included
			MiniDumpWithFullMemory				|			// the contents of every readable page in the process address space is included in the dump.
			MiniDumpWithFullMemoryInfo			|			// includes info on virtual memory layout.
			MiniDumpWithHandleData				|			// includes info about all handles in the process handle table.
			MiniDumpWithThreadInfo				|			// includes thread times, start address and affinity.
			MiniDumpWithProcessThreadData		|			// includes contents of process and thread environment blocks.
			MiniDumpWithUnloadedModules			|			// includes info from recently unloaded modules if supported by OS.
		//	MiniDumpWithFulAuxiliaryState		|			// requests that aux data providers include their state in the dump.
			MiniDumpIgnoreInaccessibleMemory	|			// ignore memory read failures.
			MiniDumpWithTokenInformation;		 			// includes security token related data.

	bRet = MiniDumpWriteDump(GetCurrentProcess(),GetCurrentProcessId(),hFile,iType,(pExceptionInfo)? &sInfo:0,0,&sCbInfo);

	CloseHandle(hFile);
	FreeLibrary(hDebugDll);



return bRet;
}

//#endif

//*****************************************************************************
//*
//*		RemoveOldest
//*
//*****************************************************************************
//	Die ltesten Dateien lschen
static void RemoveOldest()
{
char				cPath[256];
WIN32_FIND_DATAA	sInfo;
HANDLE				hFind;
int					iNum;
int					iLen;



	iLen = GetCoreName(cPath,NULL);
	strcpy(cPath+iLen,"*");

	   hFind = FindFirstFileA(cPath,&sInfo);
	if(hFind==INVALID_HANDLE_VALUE)return;

	for(iNum=0;iNum<sizeof(cTextString)/64;iNum++)			// Suche alle Dateien
		{
		strcpy(cTextString+iNum*64,sInfo.cFileName);
		if(!FindNextFileA(hFind,&sInfo))break;
		}


	iNum -= 32;
	if(iNum<=0)return;

	FindClose(hFind);

	qsort(cTextString,iNum,64,(SortProcType)stricmp);

	for(;iLen>0;iLen--)										// Suche Pfadende
		{
		if(cPath[iLen-1]=='\\')break;
		}


	for(iNum--;iNum>=0;iNum--)								// Die ltesten Dateien lschen
		{
		strcpy(cPath+iLen,cTextString+iNum*64);
		DeleteFileA(cPath);
		}


}


//*****************************************************************************
//*
//*		WinDelExChain
//*
//*****************************************************************************
//	Exeption Handler-Kette zurcksetzen
unsigned WinDelExChain()
{
LPDWORD		pNext;
DWORD		uBase;


	_asm{
		mov	eax,fs:[0]
		mov uBase,eax
		}

	pNext = (LPDWORD)uBase;
	if(uBase==0xFFFFFFFF)
		{
		return 0xFFFFFFFF;
		}

	for(;;)
		{
		if(*pNext==0xFFFFFFFF)
			{
			break;
			}

		pNext = (LPDWORD)*pNext;
		}


	_asm{
		mov eax,pNext
		mov	fs:[0],eax
		}


return uBase;
}

//*****************************************************************************
//*
//*		WinSetExChain
//*
//*****************************************************************************
//	Exeption Handler-Kette auf Wert setzen
void WinSetExChain(unsigned uBase)
{

	_asm{
		mov eax,uBase
		mov	fs:[0],eax
		}

}
