//*****************************************************************************
//*
//*
//*		WiresharkFile.cpp
//*
//*
//*****************************************************************************
//
//	Liet diverse Pakteformate ein:  PCAPNG PCAP NETMON NSS
//
//
//
//	z.B.:
//			WiresharkFile	*file;
//			WiresharkPacket	*info;
//			unsigned char   *data;
//
//				file = WiresharkOpen("C:\\Text.pcap");
//			if(!file)return -1;
//
//			for(;;)
//				{
//					data = WiresharkRead(file);
//				if(!data)break;
//
//				info = WiresharkInfo(data);
//
//				printf("Packet#=%i  len=%i \n",file->uNumber,info->uLenPacket);
//
//				...
//				}
//
//			WiresharkClose(file);
//
//
#include	<string.h>
#include	<stddef.h>
#include	<stdio.h>
#include	<time.h>

#include	"BitFunctions.h"
#include	"FileFunctions.h"
#include	"WiresharkFile.h"

#define 	GET_OFF(p,i)		(*(LPDWORD)(((char*)p)+i*4))
#define 	GET_WORD(w)			((unsigned short)((((unsigned short)(w))>>8) | (((unsigned short)(w))<<8)))
#define 	GET_DWORD(w)		((GET_WORD(w)<<16) | GET_WORD((w)>>16))

#define 	PTR_WORD(p)	   		*((unsigned short*)(p))
#define 	PTR_DWORD(p)   		*((unsigned int  *)(p))

#define 	TYPE_IP				0x0800
#define		TYPE_VLAN			0x8100
#define 	TYPE_TCP			0x0006
#define 	TYPE_UDP			0x0011

#define 	TCP_PACKET			"\x08\x00\x45\x00\x00\x28\x62\xB6\x40\x00\x80\x06\x00\x00\x7F\x00"	\
								"\x00\x01\xC0\xA8\x58\x21\xF3\xB6\xBB\x94\x57\xF0\x02\x72\x00\x00"	\
								"\x00\x00\x50\x00\x20\x00\x4D\x3C\x00\x00"

#define 	UDP_PACKET			"\x08\x00\x45\x00\x00\x4E\x61\x4B\x00\x00\x80\x11\xC2\x44\x0A\x00"	\
								"\x01\x11\x0A\x00\x01\xFF\x00\x89\x00\x89\x00\x08\x6F\x15"


//*****************************************************************************
//*
//*		WiresharkSystime
//*
//*****************************************************************************
//	Eine Windows Systemzeit umwandlen
static void WiresharkSystime(WiresharkFile *pHandle,NetMonTime *pDate)
{
struct tm		sTimeTm;


	sTimeTm.tm_year	  = pDate->wYear-1900;
	sTimeTm.tm_mon	  = pDate->wMonth-1;
	sTimeTm.tm_mday	  = pDate->wDay;
	sTimeTm.tm_hour	  = pDate->wHour;
	sTimeTm.tm_min	  = pDate->wMinute;
	sTimeTm.tm_sec	  = pDate->wSecond;
	sTimeTm.tm_isdst  = -1;

	pHandle->uTime	  = (unsigned)mktime(&sTimeTm);
	pHandle->uUsec	  = pDate->wMilliseconds*1000;

}

//*****************************************************************************
//*
//*		WiresharkOptions
//*
//*****************************************************************************
//	Scannt nach Optionen in einem Interface Description Block (Typ1)
//	vom neunen PCAP Next Generation Dump File Format.
//	pHandle		: Handle der Wireshark-Datei
//	pData		: Zeiger auf den Block (Typ=0x00000001)
//	iLen		: Ist die Lnge des Blocks
//	iBigEndian	: Sind die Daten im BigEndian-Format
void WiresharkOptions(WiresharkFile *pHandle,const unsigned char *pData,int iLen,int iBigEndian)
{
int		iResolution;
int		iCode;
int		iByte;
int		iSize;
int		iOff;
int		iNum;


	iSize = *(int*)(pData+4);

	if(iBigEndian)iSize=btol_dw(iSize);
	if(iSize>iLen || iSize<20)return;						// Prfe die gre

	iSize          -= 4;
	iResolution		= 6;									// sec	Auflsung


	for(iOff=16;iOff<iSize;)								// Alle Parameter scannen
		{
		if(iBigEndian)
			{
			iCode = pData[iOff+1]+pData[iOff+0]*256;
			iByte = pData[iOff+3]+pData[iOff+2]*256;
			}
		else{
			iCode = pData[iOff+0]+pData[iOff+1]*256;
			iByte = pData[iOff+2]+pData[iOff+3]*256;
			}

		iOff += 4;

		if(iOff+iByte>iSize)break;

		if(iCode==9 && iByte==1)							// if_tsresol Parameter
			{
			iResolution = pData[iOff];
			}

		iOff += (iByte+3)&~3;
		}


	if(iResolution&0x80)									// Auflsung 2^n
		{
		pHandle->uTimeShift	= iResolution&0x7F;
		pHandle->uTimeDiff	= 0;
		}
	else{													// Auflsung 10^n
		pHandle->uTimeShift	= 0;
		pHandle->uTimeDiff	= 1;

		for(iNum=iResolution;iNum>0;iNum--)
			{
			pHandle->uTimeDiff *= 10;
			}
		}


}


//*****************************************************************************
//*
//*		WiresharkNetMon
//*
//*****************************************************************************
//	Initalisiert Network Monitor-Dateien
static void WiresharkNetMon(WiresharkFile *pHandle)
{
NetMonHeader   *pHeader;
FILE		   *pFile;
int				iLen;


	pHeader	= (NetMonHeader*)pHandle->aBuffer;
	pFile	= pHandle->pFile;
														// Rest vom Header laden
	if(fread(pHandle->aBuffer+24,sizeof(NetMonHeader)-24,1,pFile)!=1)
		{
		return;
		}

	if(pHeader->uFrameTableOffset>=0x40000000)			// Prfe Offset der Frame-Tabelle
		{
		return;
		}

	if(pHeader->uFrameTableLength>=0x01000000)
		{
		return;
		}

	if(pHeader->uFrameTableOffset+pHeader->uFrameTableLength>=pHandle->uBytes)
		{
		return;
		}


	iLen	= pHeader->uFrameTableLength-16;
	iLen   &= ~3;

		pHandle->pUserData = new unsigned char[iLen];
	if(!pHandle->pUserData)
		{
		return;
		}


	fseek(pFile,pHeader->uFrameTableOffset+16,SEEK_SET);// Frame-Tabelle laden

	if(fread(pHandle->pUserData,iLen,1,pFile)!=1)
		{
		delete pHandle->pUserData;
		pHandle->pUserData = 0;
		fseek(pFile,24,SEEK_SET);
		return;
		}



	WiresharkSystime(pHandle,(NetMonTime*)(pHandle->aBuffer+8));

	pHandle->uUserUtc	= pHandle->uTime;
	pHandle->uUserUsec	= pHandle->uUsec;
	pHandle->uUserVal	= iLen/4;

	fseek(pFile,24,SEEK_SET);

}


//*****************************************************************************
//*
//*		WiresharkOpen
//*
//*****************************************************************************
//	ffnet eine Wireshark (*.pcap) oder Netspy-Datei (*.nns)
WiresharkFile *WiresharkOpen(const char *pFilename)
{
WiresharkFile  *pHandle;
FILE		   *pFile;
int				iTag;
int				iNum;


		pFile=fopen(pFilename,"rb");
	if(!pFile)
		{
		return 0;
		}

	pHandle			 	= new WiresharkFile;
	pHandle->pFile	 	= pFile;
	pHandle->uUsed	 	= 0;
	pHandle->uRead	 	= 0;
	pHandle->uCard	 	= 0;
	pHandle->uNumber 	= 0;
	pHandle->uOffset 	= 0;
	pHandle->uUserVal	= 0;
	pHandle->uUserIdx	= 0;
	pHandle->uUserUtc	= 0;
	pHandle->uUserUsec	= 0;
	pHandle->pUserData	= 0;
	pHandle->uTimeShift	= 0;
	pHandle->uTimeDiff	= 1000000;
	pHandle->uBytes	 	= LengthFile(pFile);
	pHandle->uType	 	= (CmpFileType(pFilename,"nss"))? 4:0;


	if(pHandle->uType==0)								// Wireshark-Header	berspringen
		{
		   iNum = fread(pHandle->aBuffer,1,24,pFile);
		if(iNum<24)
			{
			delete pHandle;
			fclose(pFile);
			return 0;
			}

		if(*(unsigned*)pHandle->aBuffer==0x0A0D0D0A)	// Neues PCAPNG-Format
			{
			iTag		   = *(int*)(pHandle->aBuffer+8);
			pHandle->uUsed = 24;
			pHandle->uType = (iTag==0x4D3C2B1A)? 3:2;
			}											// Altes PCAP-Format
		else if(*(unsigned*)pHandle->aBuffer==0xA1B2C3D4)
			{
			pHandle->uType = 0;
			}
		else if(*(unsigned*)pHandle->aBuffer==0xD4C3B2A1)
			{
			pHandle->uType = 1;
			}											// NetMon Format
		else if(*(unsigned*)pHandle->aBuffer==0x55424D47)
			{
			pHandle->uType = 5;
			WiresharkNetMon(pHandle);
			}
		else{											// Unbekanntes Format
			delete pHandle;
			fclose(pFile);
			return 0;
			}

		pHandle->uOffset = 24;
		}



return pHandle;
}

//*****************************************************************************
//*
//*		WiresharkClose
//*
//*****************************************************************************
//	Schiet eine eine Wireshark (*.pcap) Datei
//	pHandle		: Handle der Wireshark-Datei
//	Ergibt 1 wenn wenn ok oder 0 bei einem Fehler
int WiresharkClose(WiresharkFile *pHandle)
{

	if(!pHandle)return 0;

	if(pHandle->pUserData)								// Userdaten lschen
		{
		delete pHandle->pUserData;
		pHandle->pUserData = 0;
		}

	fclose(pHandle->pFile);
	delete pHandle;


return 1;
}

//*****************************************************************************
//*
//*		WiresharkRead
//*
//*****************************************************************************
//	Liest das nchste Paket ein
//	pHandle		: Handle der Wireshark-Datei
//	Ergibt einen Zeiger auf das Paket, bzw. 0 am Ende, und noch folgende Werte:
//		pHandle->uNumber  = Ist die Paketnummer
//		pHandle->uTime	  = UTC-Empfangszeit in Sekunden
//		pHandle->uUsec	  = Empfangszeit Mikorsekundenanteil
//		pHandle->uSize	  = Datenlnge
//		pHandle->uCard	  = Ethernetschnittstelle
//		*(int*)(pData-16) = UTC-Empfangszeit in Sekunden
//		*(int*)(pData-12) = Empfangszeit Mikorsekundenanteil
//		*(int*)(pData- 8) = Paketlnge (Lnge in Datei)
//		*(int*)(pData- 4) = Datenlnge (Lnge bei Empfang)
unsigned char *WiresharkRead(WiresharkFile *pHandle)
{
unsigned char  *pData;
NetMonPacket   *pNetMon;
unsigned		uOffset;
unsigned		uStart;
wsin64			iTime;
int				iCard;
int				iMax;
int				iNum;
int				iLen;
int				iTyp;


	if(!pHandle)return 0;


	for(;;)
		{
		if(pHandle->uRead>pHandle->uUsed)				// Prfe die Leseposition
			{
			pHandle->uRead = pHandle->uUsed;
			}


		if(pHandle->uUsed-pHandle->uRead<WSF_BUF_SIZE/2)// Den Puffer nachladen
			{
			pHandle->uUsed -= pHandle->uRead;
			memmove(pHandle->aBuffer,pHandle->aBuffer+pHandle->uRead,pHandle->uUsed);
			pHandle->uRead  = 0;

			iNum   = WSF_BUF_SIZE-pHandle->uUsed;
			iNum   = fread(pHandle->aBuffer+pHandle->uUsed,1,iNum,pHandle->pFile);
			pHandle->uOffset += iNum;
			pHandle->uUsed   += iNum;

			if(iNum<=0 && pHandle->uUsed<=pHandle->uRead)
				{
				return 0;
				}
			}


		switch(pHandle->uType)
			{
		case 0:											// Altes Whireshark-Format (LittleEndian)

			pData			  = pHandle->aBuffer+pHandle->uRead+16;
			pHandle->uTime	  = *(int*)(pData-16);
			pHandle->uUsec	  = *(int*)(pData-12);
			iMax			  = *(int*)(pData- 8);
			iLen			  = *(int*)(pData- 4);

			if(iLen>iMax)iLen=iMax;

			iLen += 16;
			iNum  =	pHandle->uUsed-pHandle->uRead;

			if(iLen>iNum)								// Paket zu gro fr Puffer
				{
				fseek(pHandle->pFile,iLen-iNum,SEEK_CUR);
				iLen = iNum;
				iMax = iLen-16;
				}

			pHandle->uSize	  = iMax;
			pHandle->uRead   += iLen;
			pHandle->uNumber += 1;

			return pData;

		case 1:											// Altes Whireshark-Format (BigEndian)

			pData			  = pHandle->aBuffer+pHandle->uRead+16;
			pHandle->uTime	  = btol_dw(*(int*)(pData-16));
			pHandle->uUsec	  = btol_dw(*(int*)(pData-12));
			iMax			  = btol_dw(*(int*)(pData- 8));
			iLen			  = btol_dw(*(int*)(pData- 4));

			if(iLen>iMax)iLen=iMax;

			iLen += 16;
			iNum  =	pHandle->uUsed-pHandle->uRead;

			if(iLen>iNum)								// Paket zu gro fr Puffer
				{
				fseek(pHandle->pFile,iLen-iNum,SEEK_CUR);
				iLen = iNum;
				iMax = iLen-16;
				}

			pHandle->uSize	  = iMax;
			pHandle->uRead   += iLen;
			pHandle->uNumber += 1;

			*(int*)(pData-16) = pHandle->uTime;
			*(int*)(pData-12) = pHandle->uUsec;
			*(int*)(pData- 8) = iLen;
			*(int*)(pData- 4) = iMax;

			return pData;

		case 2:											// Neues Whireshark-Format (LittleEndian)

			pData	= pHandle->aBuffer+pHandle->uRead;
			iCard   = (*(int*)(pData+8));
			iLen	= (*(int*)(pData+4)+3)&~3;
			iTyp	= (*(int*)(pData  ));
			iNum	= pHandle->uUsed-pHandle->uRead;

			if(iLen>iNum)								// Paket zu gro fr Puffer
				{
				fseek(pHandle->pFile,iLen-iNum,SEEK_CUR);
				iLen = iNum;
				}

			pHandle->uRead += iLen;


			if(iTyp==1 && iLen>=20)						// Optionen scannen
				{
				WiresharkOptions(pHandle,pData,iLen,0);
				}
			else if((iTyp==6 || iTyp==2) && iLen>=32)	// Datenpaket gefunden
				{
				pData		     += 28;
				iMax			  =	iLen-32;
				iLen			  = *(int*)(pData-8);
//				iLen			  = (iLen+3)&~3;
				iTime			  = *(unsigned*)(pData-16);
				iTime			<<= 32;
				iTime			 |= *(unsigned*)(pData-12);
				if(iLen>iMax)iLen = iMax;

				if(pHandle->uTimeDiff)					// Zeit korrigieren
					{
					pHandle->uTime	= (unsigned)(iTime/pHandle->uTimeDiff);
					pHandle->uUsec	= (unsigned)(iTime%pHandle->uTimeDiff);
					}
				else{
					pHandle->uTime	= (unsigned)(iTime>>pHandle->uTimeShift);
					pHandle->uUsec	= bit_shift((unsigned)iTime,16-pHandle->uTimeShift);
					pHandle->uUsec	= (((unsigned)(pHandle->uUsec&0xFFFF))*15625)/1024;
					}

														// Ist es ein Loopback-Paket ohne MACS
				if(*(int*)pData==0x00000002 && pData[4]==0x45)					
					{
					pData -= 14-4;
					iLen  += 14-4;

					memset(pData  ,0x02,6);
					memset(pData+6,0x04,6);
					pData[12] = 0x08;
					pData[13] = 0x00;
					pData[15] = 0x00;
					}

				pHandle->uNumber += 1;
				pHandle->uSize	  = iLen;
				pHandle->uCard	  = (iTyp==6)? iCard:0;

				*(int*)(pData-16) = pHandle->uTime;		// PCAP-Header erstellen
				*(int*)(pData-12) = pHandle->uUsec;
				*(int*)(pData- 8) = iLen;
				*(int*)(pData- 4) = iLen;

				return pData;
				}

			break;

		case 3:											// Neues Whireshark-Format (BigEndian)

			pData	= pHandle->aBuffer+pHandle->uRead;
			iLen	= (btol_dw(*(int*)(pData+4))+3)&~3;
			iTyp	= (btol_dw(*(int*)(pData )));
			iNum	= pHandle->uUsed-pHandle->uRead;

			if(iLen>iNum)								// Paket zu gro fr Puffer
				{
				fseek(pHandle->pFile,iLen-iNum,SEEK_CUR);
				iLen = iNum;
				}

			pHandle->uRead += iLen;

			if(iTyp==1 && iLen>=20)						// Optionen scannen
				{
				WiresharkOptions(pHandle,pData,iLen,1);
				}
			else if((iTyp==6 || iTyp==2) && iLen>=32)	// Datenpaket gefunden
				{
				pData		     += 28;
				iMax			  =	iLen-32;
				iLen			  = btol_dw(*(int*)(pData-8));
				iTime			  = btol_dw(*(unsigned*)(pData-16));
				iTime			<<= 32;
				iTime			 |= btol_dw(*(unsigned*)(pData-12));
				if(iLen>iMax)iLen=iMax;

				if(pHandle->uTimeDiff)					// Zeit korrigieren
					{
					pHandle->uTime	= (unsigned)(iTime/pHandle->uTimeDiff);
					pHandle->uUsec	= (unsigned)(iTime%pHandle->uTimeDiff);
					}
				else{
					pHandle->uTime	= (unsigned)(iTime>>pHandle->uTimeShift);
					pHandle->uUsec	= bit_shift((unsigned)iTime,16-pHandle->uTimeShift);
					pHandle->uUsec	= (((unsigned)(pHandle->uUsec&0xFFFF))*15625)/1024;
					}

														// Ist es ein Loopback-Paket ohne MACS
				if(*(int*)pData==0x00000002 && pData[4]==0x45)					
					{
					pData -= 14-4;
					iLen  += 14-4;

					memset(pData  ,0x02,6);
					memset(pData+6,0x04,6);
					pData[12] = 0x08;
					pData[13] = 0x00;
					pData[15] = 0x00;
					}


				pHandle->uNumber += 1;
				pHandle->uSize	  = iLen;

				*(int*)(pData-16) = pHandle->uTime;		// PCAP-Header erstellen
				*(int*)(pData-12) = pHandle->uUsec;
				*(int*)(pData- 8) = iLen;
				*(int*)(pData- 4) = iLen;

				return pData;
				}

			break;

		case 4:											// Netspy Format

			iLen = *(int*)(pHandle->aBuffer+pHandle->uRead+sizeof(NetMonTime));
			iNum = 	pHandle->uUsed-pHandle->uRead;

			if(iLen>iNum)								// Paket zu gro fr Puffer
				{
				fseek(pHandle->pFile,iLen-iNum,SEEK_CUR);
				iLen = iNum;
				}

			if(iLen<sizeof(NetMonTime)+4)
				{
				return 0;
				}

			pData			  = pHandle->aBuffer+pHandle->uRead;

			WiresharkSystime(pHandle,(NetMonTime*)pData);

			pData            += 4+sizeof(NetMonTime);
			pHandle->uRead   += iLen+4+sizeof(NetMonTime);
			pHandle->uNumber += 1;

			*(int*)(pData- 4) = iLen+4+sizeof(NetMonTime);
			*(int*)(pData- 8) = iLen+4+sizeof(NetMonTime);
			*(int*)(pData-12) = pHandle->uUsec;
			*(int*)(pData-16) = pHandle->uTime;

			return pData;

		case 5:											// NetMon Format

			for(;;)
				{
				if(pHandle->uUserIdx>=pHandle->uUserVal)
					{
					return 0;
					}

				   uOffset = GET_OFF(pHandle->pUserData,pHandle->uUserIdx);
				if(uOffset<pHandle->uBytes)break;

				pHandle->uUserIdx++;
				}


			uStart	= pHandle->uOffset-pHandle->uUsed;
														// Ist das Paket im Puffer
			if(uOffset<uStart || uOffset>uStart+WSF_BUF_SIZE/4*3)
				{
				fseek(pHandle->pFile,uOffset,SEEK_SET);
				pHandle->uOffset  = uOffset;
				pHandle->uUsed    = 0;
				pHandle->uRead	  = 0;
				continue;
				}

			uOffset	   -= uStart;
			pNetMon		= (NetMonPacket*)(pHandle->aBuffer+uOffset);
			iLen		= pNetMon->uLenPacket;
			iNum		= pHandle->uUsed-pHandle->uRead;

			if(iLen>iNum)								// Paket zu gro fr Puffer
				{
				fseek(pHandle->pFile,iLen-iNum,SEEK_CUR);
				iLen = iNum;
				}

			pData			  = pNetMon->ucData;
			iTime			  = pNetMon->iTimeOffset;	// Zeit-Offset berechnen
			iTime		     += pHandle->uUserUsec;
			pHandle->uUsec	  = (unsigned)(iTime%1000000);
			pHandle->uTime	  = (unsigned)(iTime/1000000);
			pHandle->uTime   += pHandle->uUserUtc;
			pHandle->uSize	  = iLen;
			pHandle->uRead   += iLen+16;
			pHandle->uNumber += 1;
			pHandle->uUserIdx+= 1;

			*(int*)(pData- 4) = iLen+16;
			*(int*)(pData- 8) = iLen+16;
			*(int*)(pData-12) = pHandle->uUsec;
			*(int*)(pData-16) = pHandle->uTime;

			return pData;
			}
		}


return 0;
}


//*****************************************************************************
//*
//*		WiresharkCreate
//*
//*****************************************************************************
//	Erstell einne neue PCAP-Datei
//	pFilename		: Ist der Name fr die Datei
//	Ergibt das Datei-Handle oder NULL bei einem Fehler
void *WiresharkCreate(const char *pFilename)
{
char	 cHeader[]="\xD4\xC3\xB2\xA1\x02\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\xFF\xFF\x00\x00\x01\x00\x00\x00";
FILE	*pFile;


		pFile = fopen(pFilename,"wb");
	if(!pFile)return 0;

	time(0);
	*(int*)(cHeader+8) = _timezone;

	fwrite(cHeader,1,24,pFile);


return pFile;
}


//*****************************************************************************
//*
//*		WiresharkWrite
//*
//*****************************************************************************
//	Speichert ein Paket in einer PCAP-Datei
//	pHandle			: Ist das Datei Handle
//	pPacket			: Zeiger auf die Paketdaten mit divversen TCP/IP... Headeren
//	pInfo			: Zeiger auf doe Paket-Infos (bei 0 werden sie aus pPacket geholt)
//	iSize			: Ist die neue Paketgre in Byte (bei -1 wird sie aus pInfo geholt)
//	Ergibt die Anzahl der geschriebenen Bytes.
int WiresharkWrite(void *pHandle,const unsigned char *pPacket,const WiresharkPacket *pInfo,int iSize)
{
WiresharkPacket		sInfo;



	if(!pHandle || !pPacket)return 0;
	if(!pInfo)pInfo=WiresharkInfo((unsigned char*)pPacket);		// Infos aus dem Paket holen

	if(iSize>=0)
		{														// Andere Paketgre
		sInfo				= *pInfo;
		sInfo.uLenData		= iSize;
		sInfo.uLenPacket	= iSize;
		pInfo				= &sInfo;
		}
	else{
		iSize				= pInfo->uLenPacket;
		}

	if(fwrite(pInfo,16,1,(FILE*)pHandle)!=1)					// Header speichern
		{
		return 0;
		}

	if(fwrite(pPacket,iSize,1,(FILE*)pHandle)!=1)				// Daten speichern
		{
		return 0;
		}


return iSize+16;
}



//*****************************************************************************
//*
//*		WiresharkDone
//*
//*****************************************************************************
//	Schliet eine PCAP-Datei die mit WiresharkCreate erstellt wurde.
//	pHandle			: Ist das Datei Handle
//	Ergibt 1 wenn die Datei geschlossen wurde, bzw. 0 bei einem Fehler.
int WiresharkDone(void *pHandle)
{

	if(!pHandle)return 0;
	fclose((FILE*)pHandle);

return 1;
}


//*****************************************************************************
//*
//*		WiresharkHeader
//*
//*****************************************************************************
//	Sucht einen IP-Header in einem Wireshark-Paket
//	pPacket			: Zeiger auf die Paketdaten
//	iSize			: Ist die Paketgre in Byte (bei -1 wird sie aus pPacketlt)
//	pIp/pTcp/pUdp	: Hier werden die einzelnen Header gespeichert
//	pOff...			: Hier werden die Offsets der einzelnen Header gespeichert
//	pOffDat			: Hier wird der Offset der Nutzdaten gespeichert
//	pLength			: Hier wird die Lnge  der Nutzdaten gespeichert
//	Ergibt den Typ des Paketes (0=Unbekannt 1=IP 2=TCP 3=UDP)
int WiresharkGetIpHeader(const unsigned char *pPacket,int iSize,NetIpHeader **pIp,NetTcpHeader **pTcp,NetUdpHeader **pUdp,int *pOffIp,int *pOffTcp,int *pOffUdp,int *pOffDat,int *pLength)
{
unsigned char  *pData=(unsigned char*)pPacket;
NetTcpHeader   *pTcpHeader;
NetUdpHeader   *pUdpHeader;
NetIpHeader	   *pIpHeader;
unsigned		uIpSize;
unsigned		uOffset;
int				iLength;
int				iVal;



	if(pOffIp )*pOffIp =0;
	if(pIp    )*pIp    =0;
	if(pOffTcp)*pOffTcp=0;
	if(pTcp   )*pTcp   =0;
	if(pOffUdp)*pOffUdp=0;
	if(pUdp   )*pUdp   =0;
	if(pOffDat)*pOffDat=0;
	if(pLength)*pLength=0;


	if(iSize==-1)									// Lnge aus Paket holen
		{
		iSize = *(int*)(pData-8);
		}

	if(iSize<54)
		{
		return 0;
		}

	pData  = (unsigned char*)pPacket+12;
	iSize -= 12;

	if(PTR_WORD(pData)==GET_WORD(TYPE_VLAN))		// VLAN-Tag berspringen
		{
		pData += 4;
		iSize -= 4;
		}

	if(PTR_WORD(pData)!=GET_WORD(TYPE_IP))			// Ist es ein IP-Paket
		{
		return 0;
		}

	pData	   += 2;
	iSize	   -= 2;
	pIpHeader	= (NetIpHeader*)pData;

	if(pOffIp)*pOffIp=pData-pPacket;
	if(pIp   )*pIp   =pIpHeader;

	uIpSize		= (pIpHeader->ucVersionLength&0x0F)*4;
	uOffset	    =  uIpSize;
	iLength		=  GET_WORD(pIpHeader->usTotalLength);


	if(!iLength)									// Vielleicht TCP-Segment-Offloading
		{
		iLength = iSize;
		}



	if(pIpHeader->ucProtocol==TYPE_TCP)				// Ist es ein TCP Paket
		{
		pTcpHeader	= (NetTcpHeader*)(pData+uOffset);

		if(pOffTcp)*pOffTcp=uOffset+pData-pPacket;
		if(pTcp   )*pTcp   =pTcpHeader;

		uOffset	   += (pTcpHeader->usFlagsOffset&0xF0)/4;
		iLength	   -=  uOffset;
		pData	   +=  uOffset;
		iSize	   -=  uOffset;

		if(pOffDat)*pOffDat=pData-pPacket;
		if(pLength)*pLength=iLength;

		return 2;
		}
	else if(pIpHeader->ucProtocol==TYPE_UDP)		// Ist es ein UDP Paket
		{
		pUdpHeader	= (NetUdpHeader*)(pData+uOffset);

		if(pOffUdp)*pOffUdp=uOffset+pData-pPacket;
		if(pUdp   )*pUdp   =pUdpHeader;

		iVal		=  btol_w(pUdpHeader->usLength)-8;
		uOffset	   +=  8;
		iLength	   -=  uOffset;
		pData	   +=  uOffset;
		iSize	   -=  uOffset;

		if(iLength>iVal)iLength=iVal;

		if(pOffDat)*pOffDat=pData-pPacket;
		if(pLength)*pLength=iLength;

		return 3;
		}




return 1;
}


//*****************************************************************************
//*
//*		WiresharkCrc
//*
//*****************************************************************************
//	Berechnet die Checksumme eines Speicherbereiches.
//	Dieser Algorithmus ist unabhngig von der Byte-Order
unsigned short WiresharkCrc(const void *pPtr,int iLen,unsigned uCrc=0)
{
unsigned short *pData=(unsigned short*)pPtr;
unsigned short	uRet;
unsigned char	uVal;


	while(iLen>1)
		{
		uCrc  += *pData;
		pData +=  1;
		iLen  -=  2;
		}

	if(iLen==1)
		{
		uVal  = *(unsigned char*)pData;
		uCrc +=  uVal;
		}

	uCrc	= (uCrc>>16)+(uCrc&0xFFFF);
	uCrc   += (uCrc>>16);
	uRet	= ~uCrc;



return uRet;
}


//*****************************************************************************
//*
//*		WiresharkCreateStream
//*
//*****************************************************************************
//	Erstellt einen neuen Stream im Wireshark-File direkt aus den Werten.
//	pHandle			: Ist das Datei Handle
//	iDir			: Welche Richtung hat das Paket (0=TAR->Server, 1=TAR->Client)
//					  Mit UDP_STREAM wird ein UDP-Stream erzeugt
//	pPacket			: Zeiger auf die Paketdaten
//	pStream			: Hier werden die Daten fr den Straem gespeichert
//	usPortSrc/Tar	: Neue Ports fr den Stream (0=aus Dummy einfgen)
//	uIpSrc/Tar		: Neue IP's  fr den Stream (0=aus Dummy einfgen)
//	pMacSrc/Tar		: Neue MAC's fr den Stream (0=aus Dummy einfgen)
//	Ergibt den Typ des Paketes (0=Fehler 1=IP 2=TCP 3=UDP)
int WiresharkCreateStream(void *pHandle,int iDir,WiresharkStream *pStream,unsigned short usPortSrc,unsigned short usPortTar,unsigned uIpSrc,unsigned uIpTar,const void *pMacSrc,const void *pMacTar)
{
unsigned char		aData[64];
NetTcpHeader	   *pTcpHeader;
NetUdpHeader	   *pUdpHeader;
NetIpHeader		   *pIpHeader;



	if(uIpSrc==0xFFFFFFFF)uIpSrc=0;
	if(uIpTar==0xFFFFFFFF)uIpTar=0;
	if(usPortSrc==0xFFFF )usPortSrc=0;
	if(usPortTar==0xFFFF )usPortTar=0;


	if(!uIpSrc   )uIpSrc=0x0100007F;				// Standartwerte benutzen
	if(!uIpTar   )uIpSrc=0x0200007F;
	if(!usPortSrc)usPortSrc=49999;
	if(!usPortTar)usPortTar=48888;


	if(pMacTar)										// Ziel-MAC-Adresse
		{
		memcpy(aData+0,pMacTar,6);
		}
	else{
		aData[0] = 0x00;
		aData[1] = 0x22;
		*(unsigned*)(aData+2)	= uIpTar;
		}


	if(pMacSrc)										// Quell-MAC-Adresse
		{
		memcpy(aData+6,pMacSrc,6);
		}
	else{
		aData[6] = 0x00;
		aData[7] = 0x11;
		*(unsigned*)(aData+8)	= uIpSrc;
		}

	if((iDir&~0xFF)==UDP_STREAM)					// UDP
		{
		iDir &= 0xFF;
		memcpy(aData+12,UDP_PACKET,30);

		pUdpHeader				= (NetUdpHeader*)(aData+34);
		pUdpHeader->usPortsSrc	= btol_w(usPortTar);
		pUdpHeader->usPortsTar	= btol_w(usPortSrc);
		}
	else{											// TCP
		memcpy(aData+12,TCP_PACKET,42);

		pTcpHeader				= (NetTcpHeader*)(aData+34);
		pTcpHeader->usPortsSrc	= btol_w(usPortTar);
		pTcpHeader->usPortsTar	= btol_w(usPortSrc);
		pTcpHeader->uSeqNumber	= 0;
		pTcpHeader->uAckNumber	= 0;
		}

	pIpHeader					= (NetIpHeader *)(aData+14);
	pIpHeader->uIpSource		= uIpTar;
	pIpHeader->uIpTarget		= uIpSrc;



return WiresharkCreateStream(pHandle,iDir,aData,pStream,usPortSrc,usPortTar,uIpSrc,uIpTar);
}

//*****************************************************************************
//*
//*		WiresharkCreateStream
//*
//*****************************************************************************
//	Erstellt einen neuen Stream im Wireshark-File aus Paketdaten.
//	pHandle			: Ist das Datei Handle
//	iDir			: Welche Richtung hat das Paket (0=zum Server, 1=zum Client)
//	pPacket			: Zeiger auf die Paketdaten
//	pStream			: Hier werden die Daten fr den Straem gespeichert
//	usPortSrc/Tar	: Neue Ports fr den Stream (0=aus Paket holen)
//	uIpSrc/Tar		: Neue IP's  fr den Stream (0=aus Paket holen)
//	Ergibt den Typ des Paketes (0=Fehler 1=IP 2=TCP 3=UDP)
int WiresharkCreateStream(void *pHandle,int iDir,const unsigned char *pPacket,WiresharkStream *pStream,unsigned short usPortSrc,unsigned short usPortTar,unsigned uIpSrc,unsigned uIpTar)
{
unsigned char		aData[128];
NetUdpHeader	   *pUdpHeader;
NetTcpHeader	   *pTcpHeader;
NetIpHeader		   *pIpHeader;
WiresharkPacket	   *pInfo;
unsigned			uType;
unsigned			uSize;
unsigned			uCrc;



	if(!pHandle || !pPacket || !pStream)			// Prfe Datei-Handle und Paket
		{
		return 0;
		}


	uType = WiresharkGetIpHeader(pPacket,-1,&pIpHeader,&pTcpHeader,&pUdpHeader);


	if(uType!=2 && uType!=3)						// Nur TCP und UDP
		{
		return 0;
		}

	pInfo					 = WiresharkInfo(pPacket);
	pStream->sInfo.uTimeSec	 = pInfo->uTimeSec;
	pStream->sInfo.uTimeUsec = pInfo->uTimeUsec;
	pStream->uType			 = uType;

	if(uIpSrc==0xFFFFFFFF)uIpSrc=0;
	if(uIpTar==0xFFFFFFFF)uIpTar=0;
	if(usPortSrc==0xFFFF )usPortSrc=0;
	if(usPortTar==0xFFFF )usPortTar=0;


	if(uType==3)									// UDP
		{
		memcpy(pStream->aMacTar,pPacket+((iDir)? 0:6),6);
		memcpy(pStream->aMacSrc,pPacket+((iDir)? 6:0),6);

		if(iDir)pStream->usPortTar	= (usPortSrc)? usPortSrc:btol_w(pUdpHeader->usPortsSrc);
		else	pStream->usPortSrc	= (usPortSrc)? usPortSrc:btol_w(pUdpHeader->usPortsSrc);
		if(iDir)pStream->usPortSrc	= (usPortTar)? usPortTar:btol_w(pUdpHeader->usPortsTar);
		else	pStream->usPortTar	= (usPortTar)? usPortTar:btol_w(pUdpHeader->usPortsTar);
		if(iDir)pStream->uIpAddrTar	= (uIpSrc   )? uIpSrc:pIpHeader->uIpSource;
		else	pStream->uIpAddrSrc	= (uIpSrc   )? uIpSrc:pIpHeader->uIpSource;
		if(iDir)pStream->uIpAddrSrc	= (uIpTar   )? uIpTar:pIpHeader->uIpTarget;
		else	pStream->uIpAddrTar	= (uIpTar   )? uIpTar:pIpHeader->uIpTarget;

		pStream->uOffsetSrc = 0;
		pStream->uOffsetTar = 0;

		return 2;
		}

	if(uType==2)									// TCP
		{
		memcpy(pStream->aMacTar,pPacket+((iDir)? 0:6),6);
		memcpy(pStream->aMacSrc,pPacket+((iDir)? 6:0),6);

		if(iDir)pStream->usPortTar	= (usPortSrc)? usPortSrc:btol_w(pTcpHeader->usPortsSrc);
		else	pStream->usPortSrc	= (usPortSrc)? usPortSrc:btol_w(pTcpHeader->usPortsSrc);
		if(iDir)pStream->usPortSrc	= (usPortTar)? usPortTar:btol_w(pTcpHeader->usPortsTar);
		else	pStream->usPortTar	= (usPortTar)? usPortTar:btol_w(pTcpHeader->usPortsTar);
		if(iDir)pStream->uIpAddrTar	= (uIpSrc   )? uIpSrc:pIpHeader->uIpSource;
		else	pStream->uIpAddrSrc	= (uIpSrc   )? uIpSrc:pIpHeader->uIpSource;
		if(iDir)pStream->uIpAddrSrc	= (uIpTar   )? uIpTar:pIpHeader->uIpTarget;
		else	pStream->uIpAddrTar	= (uIpTar   )? uIpTar:pIpHeader->uIpTarget;

		memcpy(aData+ 0,pStream->aMacTar,6);
		memcpy(aData+ 6,pStream->aMacSrc,6);
		memcpy(aData+12,TCP_PACKET,42);

		pStream->uOffsetSrc			= 0;
		pStream->uOffsetTar			= 0;

		pIpHeader					= (NetIpHeader *)(aData+14);
		pTcpHeader					= (NetTcpHeader*)(aData+34);

		pIpHeader->uIpSource		= pStream->uIpAddrTar;
		pIpHeader->uIpTarget		= pStream->uIpAddrSrc;
		pIpHeader->usCrcSum			= WiresharkCrc(pIpHeader,20);

		pTcpHeader->usPortsSrc		= btol_w (pStream->usPortTar );
		pTcpHeader->usPortsTar		= btol_w (pStream->usPortSrc );
		pTcpHeader->uSeqNumber		= btol_dw(pStream->uOffsetTar);
		pTcpHeader->uAckNumber		= 0x00000000;
		pTcpHeader->usFlagsOffset  |= 0x0200;		// SYN setzen
		pTcpHeader->usTcpCrc		= 0;

		uSize						= 0;
		uCrc					    = GET_WORD(6);
		uCrc					   += GET_WORD(20+uSize);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource>>16);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget>>16);
		pTcpHeader->usTcpCrc		= WiresharkCrc(pTcpHeader,20+uSize,uCrc);


													// Erstes SYS-Paket speichern
		WiresharkWrite(pHandle,aData,&pStream->sInfo,12+42);

		pStream->uOffsetTar		   += 1;

		   pStream->sInfo.uTimeUsec+=10000;
		if(pStream->sInfo.uTimeUsec>=1000000)
			{
			pStream->sInfo.uTimeUsec %= 1000000;
			pStream->sInfo.uTimeSec  += 1;
			}

		memcpy(aData+ 0,pStream->aMacSrc,6);
		memcpy(aData+ 6,pStream->aMacTar,6);
		memcpy(aData+12,TCP_PACKET,42);

		pIpHeader->uIpSource		= pStream->uIpAddrSrc;
		pIpHeader->uIpTarget		= pStream->uIpAddrTar;
		pIpHeader->usCrcSum			= WiresharkCrc(pIpHeader,20);

		pTcpHeader->usPortsSrc		= btol_w (pStream->usPortSrc );
		pTcpHeader->usPortsTar		= btol_w (pStream->usPortTar );
		pTcpHeader->uSeqNumber		= btol_dw(pStream->uOffsetSrc);
		pTcpHeader->uAckNumber		= btol_dw(pStream->uOffsetTar);
		pTcpHeader->usFlagsOffset  |= 0x1200;		// SYN+ACK setzen
		pTcpHeader->usTcpCrc		= 0;

		uSize						= 0;
		uCrc					    = GET_WORD(6);
		uCrc					   += GET_WORD(20+uSize);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource>>16);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget>>16);
		pTcpHeader->usTcpCrc		= WiresharkCrc(pTcpHeader,20+uSize,uCrc);

													// Zweites SYS-Paket speichern
		WiresharkWrite(pHandle,aData,&pStream->sInfo,12+42);

		pStream->uOffsetSrc		   += 1;

		return 3;
		}



return 0;
}


//*****************************************************************************
//*
//*		WiresharkWriteStream
//*
//*****************************************************************************
//	Speichert ein Paket eines Streams in einer PCAP-Datei
//	pHandle			: Ist das Datei Handle
//	iDir			: Welche Richtung im Stream (0=zum Server, 1=zum Client)
//	pData			: Zeiger auf die Nutzdaten
//	iSize			: Ist die Gre der Nutzdaten
//	pStream			: Hier werden die Daten fr den Straem gespeichert
//	Ergibt die Anzahl der geschriebenen Bytes.
int WiresharkWriteStream(void *pHandle,int iDir,const unsigned char *pData,int iSize,WiresharkStream *pStream)
{
unsigned char	   *pMemory;
NetUdpHeader	   *pUdpHeader;
NetTcpHeader	   *pTcpHeader;
NetIpHeader		   *pIpHeader;
unsigned			uSize;
unsigned			uCrc;


													// Prfe die Parameter
	if(!pHandle || !pData || iSize<=0 || iSize>0xFFFF || !pStream)
		{
		return 0;
		}


	if(pStream->uType==2)							// TCP
		{
			pMemory	= new unsigned char[64+iSize];
		if(!pMemory)return 0;

		memcpy(pMemory+ 0,(iDir)? pStream->aMacSrc:pStream->aMacTar,6);
		memcpy(pMemory+ 6,(iDir)? pStream->aMacTar:pStream->aMacSrc,6);
		memcpy(pMemory+12,TCP_PACKET,42);
		memcpy(pMemory+54,pData,iSize);

		pIpHeader					= (NetIpHeader *)(pMemory+14);
		pTcpHeader					= (NetTcpHeader*)(pMemory+34);

		pIpHeader->uIpSource		= (iDir)? pStream->uIpAddrTar:pStream->uIpAddrSrc;
		pIpHeader->uIpTarget		= (iDir)? pStream->uIpAddrSrc:pStream->uIpAddrTar;
		pIpHeader->usTotalLength	= btol_w(40+iSize);
		pIpHeader->usCrcSum			= WiresharkCrc(pIpHeader,20);

		pTcpHeader->usPortsSrc		= (iDir)? btol_w (pStream->usPortTar ):btol_w (pStream->usPortSrc );
		pTcpHeader->usPortsTar		= (iDir)? btol_w (pStream->usPortSrc ):btol_w (pStream->usPortTar );
		pTcpHeader->uSeqNumber		= (iDir)? btol_dw(pStream->uOffsetTar):btol_dw(pStream->uOffsetSrc);
		pTcpHeader->uAckNumber		= (iDir)? btol_dw(pStream->uOffsetSrc):btol_dw(pStream->uOffsetTar);
		pTcpHeader->usFlagsOffset  |= 0x1000;		// ACK setzen
		pTcpHeader->usTcpCrc		= 0;

		uSize						= 0;
		uCrc					    = GET_WORD(6);
		uCrc					   += GET_WORD(20+iSize);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource>>16);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget>>16);
		pTcpHeader->usTcpCrc		= WiresharkCrc(pTcpHeader,20+iSize,uCrc);

		WiresharkWrite(pHandle,pMemory,&pStream->sInfo,12+42+iSize);

		if(iDir)pStream->uOffsetTar+=iSize;
		else	pStream->uOffsetSrc+=iSize;

		   pStream->sInfo.uTimeUsec+=10000;
		if(pStream->sInfo.uTimeUsec>=1000000)
			{
			pStream->sInfo.uTimeUsec %= 1000000;
			pStream->sInfo.uTimeSec  += 1;
			}

		delete pMemory;

		return iSize;
		}


	if(pStream->uType==3)							// UDP
		{
			pMemory	= new unsigned char[64+iSize];
		if(!pMemory)return 0;

		memcpy(pMemory+ 0,(iDir)? pStream->aMacSrc:pStream->aMacTar,6);
		memcpy(pMemory+ 6,(iDir)? pStream->aMacTar:pStream->aMacSrc,6);
		memcpy(pMemory+12,TCP_PACKET,42);
		memcpy(pMemory+42,pData,iSize);
		pMemory[42+iSize] = 0;

		pIpHeader					= (NetIpHeader *)(pMemory+14);
		pUdpHeader					= (NetUdpHeader*)(pMemory+34);

		pIpHeader->uIpSource		= (iDir)? pStream->uIpAddrTar:pStream->uIpAddrSrc;
		pIpHeader->uIpTarget		= (iDir)? pStream->uIpAddrSrc:pStream->uIpAddrTar;
		pIpHeader->usTotalLength	= btol_w(28+iSize);
		pIpHeader->usCrcSum			= WiresharkCrc(pIpHeader,20);

		pUdpHeader->usPortsSrc		= (iDir)? btol_w(pStream->usPortTar):btol_w(pStream->usPortSrc);
		pUdpHeader->usPortsTar		= (iDir)? btol_w(pStream->usPortSrc):btol_w(pStream->usPortTar);
		pUdpHeader->usLength		= btol_w(8+iSize);
		pUdpHeader->usCheckSum		= 0;

		uSize						= 0;
		uCrc					    = GET_WORD(17);
		uCrc					   += GET_WORD(8+iSize);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource);
		uCrc					   += (unsigned short)(pIpHeader->uIpSource>>16);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget);
		uCrc					   += (unsigned short)(pIpHeader->uIpTarget>>16);
		pUdpHeader->usCheckSum		= WiresharkCrc(pUdpHeader,8+((iSize+1)&~1),uCrc);

		WiresharkWrite(pHandle,pMemory,&pStream->sInfo,12+42+iSize);

		if(iDir)pStream->uOffsetTar+=iSize;
		else	pStream->uOffsetSrc+=iSize;

		   pStream->sInfo.uTimeUsec+=10000;
		if(pStream->sInfo.uTimeUsec>=1000000)
			{
			pStream->sInfo.uTimeUsec %= 1000000;
			pStream->sInfo.uTimeSec  += 1;
			}

		delete pMemory;

		return iSize;
		}


return 0;
}

//*****************************************************************************
//*
//*		WiresharkChangeStream
//*
//*****************************************************************************
//	ndert die Daten in einem Stream fr eine PCAP-Aufzeichnung.
//	pHandle			: Ist das Datei Handle
//	iDir			: Welche Richtung hat haben die Daten (0=TAR->Server, 1=TAR->Client)
//	pStream			: Hier werden die Daten fr den Straem gespeichert
//	usPortSrc/Tar	: Neue Ports fr den Stream (0=nicht ndern)
//	uIpSrc/Tar		: Neue IP's  fr den Stream (0=nicht ndern)
//	pMacSrc/Tar		: Neue MAC's fr den Stream (0=nicht ndern)
//	Ergibt den Typ des Paketes (0=Fehler 1=IP 2=TCP 3=UDP)
int WiresharkChangeStream(int iDir,WiresharkStream *pStream,unsigned short usPortSrc,unsigned short usPortTar,unsigned uIpSrc,unsigned uIpTar,const void *pMacSrc,const void *pMacTar)
{

	if(!pStream)return 0;

	if(pMacSrc)
		{
		memcpy((iDir)? pStream->aMacTar:pStream->aMacSrc,pMacSrc,6);
		}

	if(pMacTar)
		{
		memcpy((iDir)? pStream->aMacSrc:pStream->aMacTar,pMacTar,6);
		}

	if(uIpSrc)
		{
		if(iDir)pStream->uIpAddrTar=uIpSrc;
		else	pStream->uIpAddrSrc=uIpSrc;
		}

 	if(uIpTar)
		{
		if(iDir)pStream->uIpAddrSrc=uIpTar;
		else	pStream->uIpAddrTar=uIpTar;
		}

	if(usPortSrc)
		{
		if(iDir)pStream->usPortTar=btol_w(usPortSrc);
		else	pStream->usPortSrc=btol_w(usPortSrc);
		}

 	if(usPortTar)
		{
		if(iDir)pStream->usPortSrc=btol_w(usPortTar);
		else	pStream->usPortTar=btol_w(usPortTar);
		}



return pStream->uType;
}
