//*****************************************************************************
//*
//*
//*		WiresharkFile.h
//*
//*
//*****************************************************************************
//
//	Liet diverse Pakteformate ein:  PCAPNG PCAP NETMON NSS
//
//	
//	z.B.:	
//			WiresharkFile	*file;	
//			WiresharkPacket	*info;
//			unsigned char   *data;
//	
//				file = WiresharkOpen("C:\\Text.pcap");
//			if(!file)return -1;
//			
//			for(;;)
//				{
//					data = WiresharkRead(file);
//				if(!data)break;
//				
//				info = WiresharkInfo(data);
//
//				printf("Packet#=%i  len=%i \n",file->uNumber,info->uLenPacket);
//
//				...
//				}
//
//			WiresharkClose(file);
//	
//
#ifndef 	__WIRESHARKFILE_H__
#define 	__WIRESHARKFILE_H__

#ifndef		WSF_BUF_SIZE
#define 	WSF_BUF_SIZE	0x20000
#endif

#include	<stdio.h>

#ifdef		_MSC_VER
#define 	wsin64			__int64
#else	
#define 	wsin64			long long 
#endif 	

#define 	UDP_STREAM		0x6700						// Eine UDD-Stream erstellen
			  

typedef	struct
	{
	FILE	       *pFile;								// Datei-Handle
	unsigned 		uType;								// Dateiformat
	unsigned 		uUsed;								// Bytes im Puffer
	unsigned 		uRead;								// Leseposition im Puffer
	unsigned		uSize;								// Gre des Paketes
	unsigned		uCard;								// Ist das ETH-Interface
	unsigned		uTime;								// Zeit des Empfangs
	unsigned		uUsec;								// Zeit des Empfangs (s)
	unsigned		uNumber;							// Ist die Packetnummer
	unsigned		uOffset;							// Offset in der Datei
	unsigned		uBytes;								// Gre der Datei
	unsigned		uUserVal;							// User-Value
	unsigned		uUserIdx;							// User-Index
	unsigned		uUserUtc;							// User-Startzeit
	unsigned		uUserUsec;							// User-Startzeit s
	unsigned		uTimeDiff;							// Timestamp-Auflsung 10^n
	unsigned		uTimeShift;							// Timestamp-Auflsung  2^n
	unsigned char  *pUserData;							// User-Daten
	unsigned char	aDummy[32];							// Sicherheitspuffer
	unsigned char	aBuffer[WSF_BUF_SIZE];				// Puffer fr Pakete
	}WiresharkFile;


typedef struct
	{
	unsigned 		uTimeSec;							// Empfangszeit UTC
	unsigned 		uTimeUsec;							// Empfangszeit S
	unsigned 		uLenData;							// Paketlnge in Datei
	unsigned 		uLenPacket;							// Paketlnge empfangen
	unsigned char 	ucData[4];							// Paketdaten
	}WiresharkPacket,WsInfo;


typedef struct
	{
	unsigned 		uBlockType;							// Ist der Blocktyp (6)
	unsigned 		uLenBlock;							// Blocklnge in Datei
	unsigned 		uNetwork;							// Nummer der Netzwerkskarte
	unsigned 		uTimeHigh;							// Empfangszeit (S oder 1/1024)
	unsigned 		uTimeLow;							// Empfangszeit unteres Wort
	unsigned 		uLenData;							// Paketlnge empfangen
	unsigned 		uLenPacket;							// Paketlnge in Datei
	unsigned char 	ucData[4];							// Paketdaten
	}WiresharkPacketEx;


typedef struct
	{
    unsigned int	uMagicNumber;						// Alter Datei-Tag (0xA1B2C3D4)
    unsigned short	uVersionMajor;						// Major Versionsnumber (0x0002)
    unsigned short	uVersionMinor;						// Minor Versionsnumber (0x0004)
    unsigned int	uThisZone;							// GMT Zeitzohne
    unsigned int	uSigFigs;							// Genauigkeit der Zeitspempel
    unsigned int	uSnapLen;							// Maximale Paketlnge
    unsigned int	uNetwork;							// Nummer der Netzwerkskarte
	}WiresharkHeader;


typedef struct
	{
	unsigned char	aMacSrc[6];							// MAC-Adresse Quelle	(Client)
	unsigned char	aMacTar[6];							// MAC-Adresse Ziel		(Server)
    unsigned short	usPortSrc;							// TCP-Port Quelle		(Client)
    unsigned short	usPortTar;							// TCP-Port Ziel		(Server)
    unsigned int	uIpAddrSrc;							// IP-Adresse Quelle	(Client)
    unsigned int	uIpAddrTar;							// IP-Adresse Ziel      (Server)
    unsigned int	uOffsetSrc;							// Stream-Offset Quelle	
    unsigned int	uOffsetTar;							// Stream-Offset Ziel	
	unsigned int	uType;								// 1=IP 2=TCP 3=UDP
	WiresharkPacket	sInfo;								// Infos zum Stream
	}WiresharkStream;


typedef struct
	{
	wsin64			iTimeOffset;						// Mikrosekunden seit Start
	unsigned 		uLenData;							// Paketlnge empfangen
	unsigned 		uLenPacket;							// Paketlnge in Datei
	unsigned char	ucData[4];							// Paketdaten
	}NetMonPacket;


typedef struct 
	{
    unsigned short	wYear;								// Jahr (z.B. 2000)	
    unsigned short	wMonth;								// Monat (1..12)
    unsigned short	wDayOfWeek;							// Wochentag (0=So 1=Mo ...)
    unsigned short	wDay;								// Tag (1..31)
    unsigned short	wHour;								// Stunde
    unsigned short	wMinute;							// Minute
    unsigned short	wSecond;							// Sekunde
    unsigned short	wMilliseconds;						// Millisekunden
	}NetMonTime;
	

typedef struct
	{
	unsigned int	uSignature;							// Immer "GMBU"
	unsigned char	ucBcdVerMinor;						// Version
	unsigned char	ucBcdVerMajor;						//		*
	unsigned short	usMacType;							// Topologietyp
	unsigned short	usStartYear;						// Startzeit
	unsigned short	usStartMonth;						//		*
	unsigned short	usStartDayOfWeek;					//		*
	unsigned short	usStartDay;							//		*
	unsigned short	usStartHour;						//		*
	unsigned short	usStartMinute;						//		*
	unsigned short	usStartSecond;						//		*
	unsigned short	usStartMilliseconds;				//		*
  	unsigned int	uFrameTableOffset;					// Offset der Frametabelle (realtiv zum Dateianfang)
	unsigned int	uFrameTableLength;					// Lnge der Frametabelle
	unsigned int	uUserDataOffset;					// Offset der User-Daten
	unsigned int	uUserDataLength;					// Lnge der User-Daten
	unsigned int	uCommentDataOffset;					// Offset des Kommentars
	unsigned int	uCommentDataLength;					// Lnge des Kommentars
	unsigned int	uStatisticsOffset;					// Offset der Staistik
	unsigned int	uStatisticsLength;					// Lnge der Staistik
	unsigned int	uNetworkInfoOffset;					// Offset der Infos
	unsigned int	uNetworkInfoLength;					// Lnge der Infos
	unsigned int	uConversationStatsOffset;			// Unbenutzt
	unsigned int	uConversationStatsLength;			// Unbenutzt
	}NetMonHeader;


typedef struct
	{
	unsigned int	uFrameTableLength;					// Unbekannt
	unsigned int	uStartIndex;						//		*
	unsigned int	uEndIndex;							//		*
	unsigned int	uFrameCount;						//		*
	unsigned int	uFrames[1];							// Dateioffsets der Frames
	}NetMonFrameTable;


typedef struct
	{
	unsigned char	ucVersionLength;					// 4Lh Version/Lnge (Lnge=L*4)
	unsigned char	ucTypeOfService;					// Typ vom Service
	unsigned short	usTotalLength;						// Lnge IP-Header plus Daten
	unsigned short	usFragmenId;						// ID wird immer um eins erhht beim Senden
	unsigned short	usFragmenOffset;					// FOOOh Flags und Offset (auf 0 setzen)
	unsigned char	ucTimeToLive;						// TTL Zeit fr Paket
	unsigned char	ucProtocol;							// 1=ICMP 6=TCP 17=UDP
	unsigned short	usCrcSum;							// CRC fr den IP-Header
	unsigned int 	uIpSource;							// IP-Adresse vom Ziel
	unsigned int 	uIpTarget;							// IP-Adresse vom Quelle
	}NetIpHeader;		


typedef struct
	{
	unsigned short 	usPortsSrc;							// Port-Source
	unsigned short 	usPortsTar;							// Port-Target
	unsigned int 	uSeqNumber;							// Sequenznummer
	unsigned int 	uAckNumber;							// Acknowledge-Nummer
	unsigned short	usFlagsOffset;						// Data-Offset und Flags
	unsigned short	usWindow;							// Gre des Empfangsfenster
	unsigned short	usTcpCrc;							// Checksumme
	unsigned short	usUrgentPtr;						// Urgent-Pointer
	}NetTcpHeader;


typedef struct
	{
	unsigned short 	usPortsSrc;							// Port-Source
	unsigned short 	usPortsTar;							// Port-Target
	unsigned short 	usLength;							// Datenlnge + Header
	unsigned short 	usCheckSum;							// Check-Summe
	}NetUdpHeader;


//******************** Lese Funktionen ****************************************
	
inline WsInfo      *WiresharkInfo  (const unsigned char *pPacket){return (WiresharkPacket*)(pPacket-16);}
WiresharkFile	   *WiresharkOpen  (const char    *pFilename);
unsigned char	   *WiresharkRead  (WiresharkFile *pHandle);
int 				WiresharkClose (WiresharkFile *pHandle);

//******************** Schreib-Funktionen *************************************

void			   *WiresharkCreate(const char    *pFilename);
int					WiresharkWrite (void          *pHandle,const unsigned char *pPacket,const WiresharkPacket *pInfo=0,int iSize=-1);
int				    WiresharkDone  (void          *pHandle);
int					WiresharkGetIpHeader (const unsigned char *pPacket,int iSize=-1,NetIpHeader **pIp=0,NetTcpHeader **pTcp=0,NetUdpHeader **pUdp=0,int *pOffIp=0,int *pOffTcp=0,int *pOffUdp=0,int *pOffDat=0,int *pLength=0);
int					WiresharkCreateStream(void *pHandle,int iDir                                     ,WiresharkStream *pStream,unsigned short usPortSrc=0,unsigned short usPortTar=0,unsigned uIpSrc=0,unsigned uIpTar=0,const void *pMacSrc=0,const void *pMacTar=0);
int					WiresharkCreateStream(void *pHandle,int iDir,const unsigned char *pPacket        ,WiresharkStream *pStream,unsigned short usPortSrc=0,unsigned short usPortTar=0,unsigned uIpSrc=0,unsigned uIpTar=0);
int					WiresharkWriteStream (void *pHandle,int iDir,const unsigned char *pData,int iSize,WiresharkStream *pStream);
int					WiresharkChangeStream(              int iDir                                     ,WiresharkStream *pStream,unsigned short usPortSrc=0,unsigned short usPortTar=0,unsigned uIpSrc=0,unsigned uIpTar=0,const void *pMacSrc=0,const void *pMacTar=0);




#endif
