//*****************************************************************************
//*
//*
//*		XmlParser.cpp
//*
//*
//*****************************************************************************
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<memory.h>

#include	"XmlParser.h"

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif


//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CXmlParser::CXmlParser()
{

	pXmlData	= NULL;									// Zeiger auf die Xml-Daten
	Init();
}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CXmlParser::~CXmlParser()
{
	if(pXmlData)delete pXmlData;
}



//*****************************************************************************
//*
//*		Init
//*
//*****************************************************************************
//	Initialisiert alle Daten in der Klasse
void CXmlParser::Init()
{

	if(pXmlData)										// Zeiger auf die Xml-Daten lschen
		{
		delete pXmlData;
		pXmlData  = NULL;
		}

	uTagPos		= 0;									// Startposition vom aktuellen Tag
	uXmlSize	= 0;									// Lnge der Xml-Daten in Byte
	uXmlPos		= 0;									// Aktuelle Position fr's parsen
	iXmlLevel	= 0;									// Aktuelle Hierachieebene in der XML-Datei
	iDataFlag	= 0;									// Ist der Datenbereich bereitsgesannt
	iHasCData	= 0;									// War ein CDATA Bereich forhanden
	iHasAnd		= 0;									// War ein '&' Zeichen forhanden

}


//*****************************************************************************
//*
//*		DecodeText
//*
//*****************************************************************************
//	Dekodiert einen XML-Text und speichert ihn in einem Puffer.
//	Zeilesprnge werden entfernt und &???; Zeichen dekodiert.
//	pText		: Zeiger auf zu dekodierenden Text
//	iTextSize	: Ist die Lnge des Textes in Byte
//	pTarget		: Zeiger auf Zielpuffer
//	iMax		: Gre des Ziel Puffers
//	Ergibt die Lnge des gespeicherten Textes ohne Null-Byte am Ende
int CXmlParser::DecodeText(const char *pText,int iTextSize,char *pTarget,int iMax)
{
int		i;
int		iAdd;
int		iOut;
int		iPos;
char	cChar;


	if(iMax<=0)return 0;

	iPos = 0;
	iOut = 0;

	while(iPos<iTextSize)						// Eingangs-Zeichenschleife
		{
		   cChar = pText[iPos];
		if(cChar=='\n'){iPos++;continue;}
		if(cChar=='\r'){iPos++;continue;}

		iAdd = 1;

		if(cChar=='<' && iPos+8<iTextSize)		// CDATA Bereiche filtern
		if(pText[iPos+1]=='!')
		if(pText[iPos+2]=='[')
		if(memcmp(pText+iPos+3,"CDATA[",6)==0)
			{
			iPos+=9;

			while(iPos<iTextSize)
				{
				   cChar = pText[iPos];
				if(cChar==']' && iPos+2<iTextSize)
				if(pText[iPos+1]==']' && pText[iPos+2]=='>')
					{
					iPos+=3;
					break;
					}

				pTarget[iOut]=cChar;			// Zeichen ausgeben
				iOut++;
				if(iOut>=iMax)break;
				iPos++;
				}


			continue;
			}

		if(cChar=='&' )							// Spezialzeichen dekodieren
		do	{
			if(iPos+3<iTextSize)
				{
				if(!memcmp(pText+iPos+1,"lt;"   ,3)){iAdd=4;cChar='<';break;}
				if(!memcmp(pText+iPos+1,"gt;"   ,3)){iAdd=4;cChar='>';break;}
				}

			if(iPos+4<iTextSize)
				{
				if(!memcmp(pText+iPos+1,"amp;"  ,4)){iAdd=5;cChar='&';break;}
				}

			if(iPos+5<iTextSize)
				{
				if(!memcmp(pText+iPos+1,"nbsp;" ,5)){iAdd=6;cChar=' ';break;}
				if(!memcmp(pText+iPos+1,"quot;" ,5)){iAdd=6;cChar='"';break;}
				if(!memcmp(pText+iPos+1,"Auml;" ,5)){iAdd=6;cChar='';break;}
				if(!memcmp(pText+iPos+1,"Ouml;" ,5)){iAdd=6;cChar='';break;}
				if(!memcmp(pText+iPos+1,"Uuml;" ,5)){iAdd=6;cChar='';break;}
				if(!memcmp(pText+iPos+1,"auml;" ,5)){iAdd=6;cChar='';break;}
				if(!memcmp(pText+iPos+1,"ouml;" ,5)){iAdd=6;cChar='';break;}
				if(!memcmp(pText+iPos+1,"uuml;" ,5)){iAdd=6;cChar='';break;}
				}

			if(iPos+6<iTextSize)
				{
				if(!memcmp(pText+iPos+1,"szlig;",6)){iAdd=7;cChar='';break;}
				}

			if(pText[iPos+1]=='#')				// Spezialzeichen als Zahl dekodieren
				{
				for(i=iPos+2;i<iTextSize;i++)	// Suche das Ende der Zahl
					{
					if(pText[i]<'0')break;
					if(pText[i]>'9')break;
					}

				if(i<iTextSize && pText[i]==';')
					{
					cChar = atoi(pText+iPos+2);
					iAdd  = i-iPos+1;
					}
				}
			}while(0);

		pTarget[iOut]=cChar;					// Zeichen ausgeben
		iOut++;
		if(iOut>=iMax)break;
		iPos+=iAdd;
		}



	pTarget[iOut]=0;							// Ein Null-Byte am Ende anhngen


return iOut;
}

//*****************************************************************************
//*
//*		Open
//*
//*****************************************************************************
//	ffnet eine Xml-Datei und ld die Daten in den Speicher.
//	pXmlFile	: Ist der name der Xml-Datei
//	Ergibt		: 0 = Die Xml-Datei wurde geladen
//				  1 = Die Xml-Datei konnte nicht geladen werden
//				  2 = Es gab zu wenig Speicher um die Datei zu laden
int CXmlParser::Open(const char *pXmlFile)
{
FILE	*pFile;
int		 iSize;


	Init();


		pFile = fopen(pXmlFile,"rb");
	if(!pFile)return 1;

	if(fseek(pFile,0,SEEK_END))							// Bestimme die Gre der Xml-Datei
		{
		fclose(pFile);
		return 1;
		}

	   iSize = ftell(pFile);
	if(iSize<=0)
		{
		fclose(pFile);
		return 1;
		}

		pXmlData = new char[iSize+1];					// Datenspeicher allokieren
	if(!pXmlData)
		{
		fclose(pFile);
		return 2;
		}

	if(fseek(pFile,0,SEEK_SET))							// Die Daten vom Dateianfang an laden
		{
		fclose(pFile);
		return 1;
		}

	uXmlSize = fread(pXmlData,1,iSize,pFile);
	pXmlData[uXmlSize] = 0;
	fclose(pFile);


return 0;
}

//*****************************************************************************
//*
//*		Close
//*
//*****************************************************************************
//	Schiet die Xml-Datei
//	Erbibt 1 bei einem Fehler ansonsten 0
int CXmlParser::Close()
{

	if(!pXmlData)return 0;
	Init();

return 1;
}

//*****************************************************************************
//*
//*		LevelName
//*
//*****************************************************************************
//	Liest den Namen eines Level-Tags aus
//	iLevel		: Ist die Nummer des Levels
//	pName		: Hier wird der Name des Tags gespeichert
//	iNameSize	: Ist die Gre des Namenpuffers in Byte
//	Ergibt die Lnge des TAG-Namens oder 0 wenn nichts gefunden wurde
int CXmlParser::LevelName(int iLevel,char *pName,int iNameSize)
{
int		iSize;



	if(iNameSize<=0)return 0;

	if(iLevel<0 || iLevel>=iXmlLevel)
		{
		pName[0]=0;
		return 0;
		}

	   iSize = aLevelSize[iLevel];
	if(iSize>=iNameSize)iSize=iNameSize-1;

	memcpy(pName,pXmlData+aLevelPos[iLevel],iSize);
	pName[iSize]=0;



return iSize;
}

//*****************************************************************************
//*
//*		NextTag
//*
//*****************************************************************************
//	Liest das nchte TAG aus der XmlDatei ein. <TAG ...Atributte...>
//	pName		: Hier wird der Name des nchten Tags gespeichert
//	iNameSize	: Ist die Gre des Namenpuffers in Byte
//	pAttr		: Im Inhalt dieses Zeigers (*pAttr) werden die Attribute gespeichert.
//				  Falls der Zeiger NULL ist werden keine Atributte gespeichert.
//				  Wenn der Inhalt des Zeigers NULL ist dann wird ein neuer
//				  Speicherbereich mit new angelegt.
//	iAttrSize	: Ist die Gre des Atributtpuffers in Byte
//				  Wenn diese Gre kleiner 1 ist werden keine Atributte gespeichert.
//	Ergibt die Lnge des TAG-Namens oder 0 wenn nichts gefunden wurde
int CXmlParser::NextTag(char *pName,int iNameSize,char **pAttr,int iAttrSize)
{
unsigned	uPos;
unsigned	uEnd;
unsigned	uAttrPos;
unsigned	uAttrEnd;
char		cChar;
int			iSize;


	if(iDataFlag)										// Gab es zuvor einen Datenscann
		{
		uXmlPos		= uXmlDataEnd;
		iDataFlag	= 0;
		}


	for(;uXmlPos<uXmlSize;uXmlPos++)					// Suche das nchste TAG
		{
		if(pXmlData[uXmlPos]!='<')continue;

		uXmlPos++;
		uPos=uXmlPos;

		if(pXmlData[uPos  ]=='!')						// CDATA Tags berspringen
		if(pXmlData[uPos+1]=='[')
		if(memcmp(pXmlData+uPos+2,"CDATA[",6)==0)
			{
			for(uXmlPos+=8;uXmlPos<uXmlSize;uXmlPos++)
				{
				if(pXmlData[uXmlPos  ]!=']')continue;
				if(pXmlData[uXmlPos+1]!=']')continue;
				if(pXmlData[uXmlPos+2]!='>')continue;
				uXmlPos+=2;
				break;
				}
			continue;
			}

		for(;uXmlPos<uXmlSize;uXmlPos++)				// Suche das Ende des TAG's
			{
			if(pXmlData[uXmlPos]>= 0 )
			if(pXmlData[uXmlPos]<=' ')break;
			if(pXmlData[uXmlPos]=='>')break;
			}

		uTagPos = uXmlPos;
		uEnd    = uXmlPos;

		if(pAttr && iAttrSize>0)
			{
			for(;uXmlPos<uXmlSize;uXmlPos++)			// Suche den Anfang der Attribute
				{
				   cChar = pXmlData[uXmlPos];
				if(cChar!=' ' )
				if(cChar!='\t')
				if(cChar!='\r')
				if(cChar!='\n')break;
				}

			uAttrPos = uXmlPos;

			for(;uXmlPos<uXmlSize;uXmlPos++)			// Suche das Ende der Attribute
				{
				if(pXmlData[uXmlPos]=='>')break;
				}

			uAttrEnd = uXmlPos;

			if(pXmlData[uAttrEnd]=='>')uAttrEnd--;

			for(;uAttrEnd>uAttrPos;uAttrEnd--)			// Leerzeichen am Ende berspringen
				{
				   cChar=pXmlData[uAttrEnd];
				if(cChar==' ' )continue;
				if(cChar=='\t')continue;
				if(cChar=='\r')continue;
				if(cChar=='\n')continue;
				break;
				}

			iSize  = uAttrEnd-uAttrPos;

			if(!*pAttr)
				{
				*pAttr	  = new char[iSize+1];
				iAttrSize = iSize+1;
				}

			DecodeText(pXmlData+uAttrPos,iSize,*pAttr,iAttrSize);
			}


		for(;uXmlPos<uXmlSize;uXmlPos++)				// Suche das Ende des Tags
			{
			if(pXmlData[uXmlPos]=='>')
				{
				uXmlPos++;
				break;
				}
			}


		iSize = uEnd-uPos;


		if(pXmlData[uPos]=='/')							// Die Level-Ebene anpassen
			{
			if(iXmlLevel>0)iXmlLevel--;
			}
		else if(pXmlData[uPos]!='?' && pXmlData[uPos]!='!')
			{
			if(iXmlLevel<XML_PARSER_LEVELS)
				{
				aLevelPos [iXmlLevel]=uPos;				// Positionen der Level-Tags
				aLevelSize[iXmlLevel]=iSize;			// Lngen der Level-Tags
				}

			iXmlLevel++;
			}

		if(iSize==0)iSize=1;							// Einen "<>" Tag verhintern
		if(iSize>=iNameSize)iSize=iNameSize-1;			// Den Tag speichern

		memcpy(pName,pXmlData+uPos,iSize);
		pName[iSize]=0;

		return iSize;
		}



	if(iNameSize>0)pName[0]=0;
	if(pAttr && iAttrSize>0)pAttr=0;


return 0;
}



//*****************************************************************************
//*
//*		GetData
//*
//*****************************************************************************
//	Liest den Nchsten Datenbereich aus der XmlDatei ein. <TAG>...Data...<TAG>
//	Bei CDATA-Bereichen wird nur der Inhalt kopiert
//	pData		: Im Inhalt dieses Zeigers (*pAttr) werden die Daten gespeichert.
//				  Falls der Zeiger NULL ist werden keine Daten gespeichert.
//				  Wenn der Inhalt des Zeigers NULL ist dann wird ein neuer
//				  Speicherbereich mit new angelegt.
//	iDataSize	: Ist die Gre des Atributtpuffers in Byte
//				  Wenn diese Gre kleiner 1 ist werden keine Daten gespeichert.
//	Ergibt die Lnge des Datenbereiches oder 0 wenn nichts gefunden wurde
int	CXmlParser::GetData(char **pData,int iDataSize)
{
unsigned	uPos;
unsigned	uEnd;
unsigned	uNow;
char		cChar;
int			iSize;




	if(iDataFlag)										// Wurde bereits vorher gescannt
		{
		uNow = uXmlDataEnd;
		uPos = uXmlPos;
		}
	else{
		for(;uXmlPos<uXmlSize;uXmlPos++)				// Leerzeichen am Anfang berspringen
			{
			   cChar = pXmlData[uXmlPos];
			if(cChar!=' ' )
			if(cChar!='\t')
			if(cChar!='\r')
			if(cChar!='\n')break;
			}

		uPos		= uXmlPos;
		iHasCData	= 0;
		iHasAnd		= 0;

		for(uNow=uPos;uNow<uXmlSize;uNow++)				// Suche das nchsen TAG
			{
			   cChar=pXmlData[uNow];
			if(cChar           =='&')iHasAnd=1;
			if(cChar           !='<')continue;
			if(pXmlData[uNow+1]!='!')break;
			if(pXmlData[uNow+2]!='[')break;
			if(memcmp(pXmlData+uNow+3,"CDATA[",6))break;

			if(uNow+9<=uXmlSize)uNow+=9;

			for(;uNow<uXmlSize;uNow++)					// Suche das Ende von CDATA
				{
				if(pXmlData[uNow  ]==']')
				if(pXmlData[uNow+1]==']')
				if(pXmlData[uNow+2]=='>')break;
				}

			if(uNow+3<=uXmlSize)uNow+=2;

			iHasCData = 1;
			}

		iDataFlag	= 1;
		uXmlDataEnd	= uNow;
		}

	if(iDataSize==-12345678)							// Den Zeiger direkt zurckgeben
		{
		return uPos;
		}

	for(uEnd=uNow;uEnd>uPos;uEnd--)						// Leerzeichen am Ende berspringen
		{
		   cChar = pXmlData[uEnd-1];
		if(cChar!=' ' )
		if(cChar!='\t')
		if(cChar!='\r')
		if(cChar!='\n')break;
		}

	iSize  = uEnd-uPos;

	if(!*pData)											// Einen neuen Speicher allokieren
		{
		*pData	  = new char[iSize+1];
		iDataSize = iSize+1;
		}

	iSize = DecodeText(pXmlData+uPos,iSize,*pData,iDataSize);



return iSize;
}


//*****************************************************************************
//*
//*		GetDirectLength
//*
//*****************************************************************************
//	Ergibt die Lnge des nchsten Datenbereichs <TAG> Zahl <TAG> in Byte
int CXmlParser::GetDirectLength()
{

	if(!iDataFlag)
		{
		GetData((char**)NULL,-12345678);
		}


return uXmlDataEnd-uXmlPos;
}

//*****************************************************************************
//*
//*		GetValue
//*
//*****************************************************************************
//	Ergibt den Zahlenwert aus dem nchsten Datenbereich <TAG> Zahl <TAG>
//	Es werden Hexdezimalzahlen in der Form  0x1234 und 123h erkannt
//	Weiters werden Anfhrungszeichen bersprungen
int	CXmlParser::GetValue()
{
const char *pText;
char		cTemp[32];



		pText = GetDirectData();
	if(!pText)return 0;

	if(!iHasCData && !iHasAnd)return GetValue(pText);

	GetData(cTemp,sizeof(cTemp));



return GetValue(cTemp);
}

//*****************************************************************************
//*
//*		GetValue
//*
//*****************************************************************************
//	Holt einen Zahlenwert aus einem Text
//	Es werden Hexdezimalzahlen in der Form  0x1234 und 123h erkannt
//	Weiters werden Anfhrungszeichen bersprungen "123"
//	pText	: Ist der Zeiger auf den Zahlentext
//	Ergibt
int	CXmlParser::GetValue(const char *pText)
{
unsigned	uDigits[16];
unsigned	uCount;
unsigned	uMinus;
unsigned	uBase;
unsigned	uDec;
unsigned	uBin;
unsigned	uVal;
unsigned	uMax;
unsigned	uRet;
char		cChar;


	while(1)											// Leerzeichen am Anfang berspringen
		{
		   cChar=pText[0];
		if(cChar >' ' )break;
		if(cChar!=' ' )
		if(cChar!='\t')
		if(cChar!='\n')
		if(cChar!='\r')break;
		pText++;
		}

	if(pText[0]=='"')pText++;


	uBase=10;

	if(pText[0]=='-')									// Ist ein Minuszeichen am Anfang
		{
		pText++;
		uMinus=1;

		while(1)
			{
			   cChar=pText[0];
			if(cChar!='\n')
			if(cChar!='\r')break;
			pText++;
			}
		}
	else{
		uMinus=0;
		}


	if(pText[0]=='0')
		{
		if(pText[1]=='x' || pText[1]=='X')				// Format 0x1234
			{
			pText+=2;
			uBase=16;
			}
		else if(pText[1]=='b' || pText[1]=='B')			// Format 0b1111
			{
			pText+=2;
			uBase=2;
			}
		}


	uMax = 0;
	uDec = 0;
	uBin = 0;

	for(uCount=0;uCount<16;uCount++,pText++)			// Zahlen einlesen
		{
		uVal = pText[0];

		if(uVal< '0')break;
		if(uVal<='9')
			{
			uVal-='0';
			uDigits[uCount]=uVal;
			if(uCount==uDec)uDec++;
			if(uCount==uBin && uMax<2)uBin++;
			uMax|=uVal;
			continue;
			}

		if(uVal< 'A')break;
		if(uVal<='F')
			{
			uVal-='A'-10;
			uDigits[uCount]=uVal;
			uMax|=uVal;
			continue;
			}

		if(uVal< 'a')break;
		if(uVal<='f')
			{
			uVal-='a'-10;
			uDigits[uCount]=uVal;
			uMax|=uVal;
			continue;
			}

		break;
		}

	if(uBase==10)
		{
		if(pText[0]=='h' || pText[0]=='H')				// Format 1234h
			{
			uBase  = 16;
			}
		else if(uDigits[uBin]==11)						// Format 1111b
			{
			uCount = uBin;
			uBase  = 2;
			}
		else{
			uCount = uDec;
			}
		}
	else{
		if(uBase==2)uCount=uBin;
		}


	for(uRet=0,uMax=0;uMax<uCount;uMax++)				// Den Zahlenwert berechner
		{
		uRet *= uBase;
		uRet += uDigits[uMax];
		}



return (uMinus)? -(int)uRet:(int)uRet;
}


//*****************************************************************************
//*
//*		GetBinary
//*
//*****************************************************************************
//	Holt einen Hexdump aus dem nchsten Datenbereich <TAG> Bereich <TAG>
//	Als Hexdump wird erkannt:  ":112233" ":11 22 33" "112233" und "11 22 33"
//	pMem	: Hier wird der Hexdump im Binrformat
//	iMax	: Ist die maximale Gre in Byte des Puffers pMem
//	Ergibt die Anzahl der gespeicherten Bytes
int	CXmlParser::GetBinary(void *pMem,int iMax)
{
unsigned char   *pData=(unsigned char*)pMem;
const char      *pText;
char			*pTemp=NULL;
char			 cTemp[1028];
char			 cChar;
int				 iLen;
unsigned		 uVal;
unsigned		 uNum;


		pText = GetDirectData();
	if(!pText)return 0;

	if(iHasCData || iHasAnd)							// Daten dekodieren
		{
		   iLen = GetDirectLength();
		if(iLen<sizeof(cTemp))
			{
			GetData(cTemp,sizeof(cTemp));
			pText=cTemp;
			}
		else{
			GetData(&pTemp,1);
			pText=pTemp;
			}
		}


	uNum=0;

	while(pText[0]<=' ')								// Leerzeichen am Anfang berspringen
		{
		   cChar = pText[0];
		if(cChar!=' ' )
		if(cChar!='\t')
		if(cChar!='\n')
		if(cChar!='\r')
			{
			if(cChar!=':' || uNum)break;
			uNum=1;
			}
		pText++;
		}



	for(iLen=0;iLen<iMax;iLen++)
		{
		uVal = pText[0];
		pText++;

		if(uVal< '0')break;								// Erstese Hex-Digit auswerten
		if(uVal<='9')
			{
			uVal-='0';
			}
		else{
			if(uVal< 'A')break;
			if(uVal<='F')
				{
				uVal-='A'-10;
				}
			else{
				if(uVal< 'a')break;
				if(uVal<='f')
					{
					uVal-='a'-10;
					}
				}
			}


		uNum = pText[0];
		pText++;


		if(uNum<='9')									// Zweites Hex-Digit auswerten
			{
			if(uNum>='0')
				{
				uVal<<=4;
				uVal +=uNum-'0';
				}
			else{
				uNum=0xFF;
				}
			}
		else{
			if(uNum<='F')
				{
				if(uNum>='A')
					{
					uVal<<=4;
					uVal +=uNum-'A'+10;
					}
				else{
					uNum=0xFF;
					}
				}
			else{
				if(uNum>='a' && uNum<='f')
					{
					uVal<<=4;
					uVal +=uNum-'a'+10;
					}
				else{
					uNum=0xFF;
					}
				}
			}

		pData[iLen] = uVal;
		if(uNum==0xFF)break;

		while(pText[0]<=' ')							// Leerzeichen berspringen
			{
			   cChar = pText[0];
			if(cChar!=' ' )
			if(cChar!='\t')
			if(cChar!='\n')
			if(cChar!='\r')break;
			pText++;
			}
		}

	if(pTemp)delete pTemp;



return iLen;
}



//*****************************************************************************
//*
//*		LevelDown
//*
//*****************************************************************************
//	Springt an den Anfang des nchtsen Tags	auf einem bestimmten Level
//	iLevel	: Ist das Level das erreicht werden soll
//	Ergibt 0 wenn das Level erreicht wurde sonst 1
int	CXmlParser::LevelDown(int iLevel)
{
char	cTemp[4];


	if(iLevel>=iXmlLevel)return 1;

	while(uXmlPos<uXmlSize)
		{
		NextTag(cTemp,sizeof(cTemp));
		if(iXmlLevel==iLevel)return 0;
		}


return 1;
}


//*****************************************************************************
//*
//*		GotoTag
//*
//*****************************************************************************
//	Springt in der Xml-Datei vor auf einen bestimmten Tag
//	Der letzte Tag-Namen kann auch ein spezial Tag sein	 <?...> <!...>  <!...>
//	pPath	: Ist der Pfad des Tags mit '\\' Zeigen getrennt
//	rgibt 0 wenn der Tag gefunden wurde sonst 0
int	CXmlParser::GotoTag(const char *pPath)
{
const char *pTag;
char		cName[XML_PARSER_TAGS];
unsigned	uPos [XML_PARSER_LEVELS];
unsigned	uSize[XML_PARSER_LEVELS];
unsigned	uOff;
unsigned	uNow;
char		cChar;
int			iCount;
int			iLast;
int			iAdd;
int			iLen;
int			i;


	for(uOff=0,uNow=0,iCount=0;;uNow++)					// Scanne Pfad
		{
		   cChar = pPath[uNow];
		if(cChar==0)
			{
			uPos [iCount]=uOff;
			uSize[iCount]=uNow-uOff;
			iCount++;
			break;
			}

		if(cChar=='\\')
			{
			uPos [iCount]=uOff;
			uSize[iCount]=uNow-uOff;
			iCount++;
			uNow++;
			uOff=uNow;
			}
		}


	if(iCount<=0)return 0;							// Zu kurzer Pfad

	pTag  = pPath+uPos [iCount-1];
	iLast =       uSize[iCount-1];

	   cChar = pTag[0];
	if(cChar=='?' || cChar=='!' || cChar=='/')		// Gibt es eine Spezial-Tag am Ende
			iAdd=1;
	else	iAdd=0;



	while(uXmlPos<uXmlSize)							// Suche Tag
		{
		iLen=NextTag(cName,sizeof(cName));

		if(iXmlLevel+iAdd!=iCount)continue;			// Stimmt die Ebene
		if(iLen!=iLast)continue;					// Stimmt die Lnge
		if(memcmp(cName,pTag,iLen))continue;		// Stimmt der Tag

		if(iCount==1)return 0;
		if(memcmp(uSize,aLevelSize,sizeof(uSize[0])*(iCount-1)))continue;

		for(i=iCount-2;i>=0;i--)					// Vergleiche alle Namen mit Pfad
			{
			if(memcmp(pXmlData+aLevelPos[i],pPath+uPos[i],uSize[i]))break;
			}

		if(i<0)return 0;
		}


return 1;
}



//*****************************************************************************
//*
//*		GetTagPath
//*
//*****************************************************************************
//	Erzeugt einen Pfad fr das aktuelle Tag
//	pBuffer	: Hier wird der Pfad gespeichert
//	iMax	: Ist die Gre des Puffers in Byte
//	iLevel	: Ist das Level bis zu dem der Pfad erzeugt werden soll
//	Ergibt die Lnge der gespeicherten Pfades
int CXmlParser::GetTagPath(char *pBuffer,int iMax,int iLevel)
{
int		iSize;
int		iPos;
int		i;


	if(iMax<=0)return 0;

	iMax--;

	if(iLevel>iXmlLevel)iLevel=iXmlLevel;
	if(iLevel>XML_PARSER_LEVELS)iLevel=XML_PARSER_LEVELS;

	iPos=0;

	if(iLevel>0)
	for(i=0;;i++)										// Den Pfad aufbauen
		{
		iSize = aLevelSize[i];
		if(iPos+iSize>iMax)iSize=iMax-iPos;
		memcpy(pBuffer+iPos,pXmlData+aLevelPos[i],iSize);
		iPos+=iSize;

		if(i+1>=iLevel)break;

		if(iPos<iMax)									// Ein '\\' Zeichen anhngen
			{
			pBuffer[iPos]='\\';
			iPos++;
			}
		}

	pBuffer[iPos]=0;


return iPos;
}


//*****************************************************************************
//*
//*		Reset
//*
//*****************************************************************************
//	Setzt den Auslesezeiger auf eine Ebene zurck
//	iLevel	: Ist die Position des der Ebene (-1= ist der Dateianfang)
//	Ergibt 0 wenn der Auslesezeiger zurckgesetz wurde sonst 1
int CXmlParser::Reset(int iLevel)
{


	if(iLevel<0)
		{
		uTagPos		= 0;								// Startposition vom aktuellen Tag
		uXmlPos		= 0;								// Aktuelle Position fr's parsen
		iXmlLevel	= 0;								// Aktuelle Hierachieebene in der XML-Datei
		iDataFlag	= 0;								// Ist der Datenbereich bereitsgesannt
		iHasCData	= 0;								// War ein CDATA Bereich forhanden
		iHasAnd		= 0;								// War ein '&' Zeichen forhanden
		return 1;
		}

	if(iLevel>=iXmlLevel        )return 1;
	if(iLevel>=XML_PARSER_LEVELS)return 1;


	uTagPos		= 0;									// Startposition vom aktuellen Tag
	uXmlPos		= aLevelPos[iLevel]-1;					// Aktuelle Position fr's parsen
	iXmlLevel	= iLevel;								// Aktuelle Hierachieebene in der XML-Datei
	iDataFlag	= 0;									// Ist der Datenbereich bereitsgesannt
	iHasCData	= 0;									// War ein CDATA Bereich forhanden
	iHasAnd		= 0;									// War ein '&' Zeichen forhanden


return 0;
}


//*****************************************************************************
//*
//*		GetAttr
//*
//*****************************************************************************
//	Ergibt den Wert eines Attributs des aktuellen Tags
//	pName		: Ist der Name des Attributs
//	uLen		: Ist die Lnge des Attributnames, bei XML_AUTOSIZE 
//				  wird er mit strlen(...) ermittelt. 
//	Ergibt den Wert des Attributs.
int	CXmlParser::GetAttr(const char *pName,unsigned uLen)
{
unsigned char	ucSign;
unsigned		uSize;
unsigned		uEnd;
unsigned		uPos;



	if(!uTagPos)										// Sind wir auf einen Tag
		{
		return 0;
		}


	if(uLen==XML_AUTOSIZE)								// Lnge des Attributnamens ermitteln
		{
		uLen = strlen(pName);
		}
	

	for(uPos=uTagPos;uPos<uXmlPos;uPos++)				// Attribute suchen
		{
		   ucSign = pXmlData[uPos];
		if(ucSign<=' ')continue;

		for(uEnd=uPos;uEnd<uXmlPos;uEnd++)				// Lnge des Attributnamens ermitteln
			{
			ucSign = pXmlData[uEnd];

			if(ucSign<'0' || ucSign>'z')break;
			if(ucSign>'9' && ucSign<'A')break;
			if(ucSign>'Z' && ucSign<'a')break;
			}
		
		uSize = uEnd-uPos;

		for(;uEnd<uXmlPos;uEnd++)						// Leerzeichen vor nach '=' berspringen
			{
			   ucSign = pXmlData[uEnd];
			if(ucSign>' ')break;
			}

		if(pXmlData[uEnd]!='=')
			{
			return 0;
			}

		for(uEnd++;uEnd<uXmlPos;uEnd++)					// Leerzeichen nach '=' berspringen
			{
			   ucSign = pXmlData[uEnd];
			if(ucSign>' ')break;
			}

		if(uLen==uSize && !memcmp(pName,pXmlData+uPos,uLen))
			{
			if(pXmlData[uEnd]=='"')uEnd++;
			return atoi(pXmlData+uEnd);
			}

		uPos = uEnd;
		

		if(pXmlData[uPos]=='"')
			{
			for(uPos++;uPos<uXmlPos;uPos++)				// Vor zum nchsten Attribut
				{
				   ucSign = pXmlData[uPos];
				if(ucSign=='"'){uPos++;break;}
				}

			continue;
			}


		for(;uPos<uXmlPos;uPos++)						// Vor zum nchsten Attribut
			{
			   ucSign = pXmlData[uPos];
			if(ucSign<=' ')break;
			}
		}


return 0;
}

//*****************************************************************************
//*
//*		GetAttr
//*
//*****************************************************************************
//	Ergibt den Text eines Attributs des aktuellen Tags
//	pName		: Ist der Name des Attributs
//	pData		: Hier wird der Text gespeichert
//	iMax		: Ist die Gre des Text speichert
//	uLen		: Ist die Lnge des Attributnames, bei XML_AUTOSIZE 
//				  wird er mit strlen(...) ermittelt. 
//	Ergibt die Lnge des gefundenen Textes oder, -1 bei einem Fehler.
int	CXmlParser::GetAttr(const char *pName,char *pData,int iMax,unsigned uLen)
{
unsigned char	ucSign;
unsigned		uSize;
unsigned		uEnd;
unsigned		uPos;
unsigned		uOff;

													
	if(iMax<=0)											// Prfe die Puffer gre
		{
		return -1;
		}


	if(!uTagPos)										// Sind wir auf einen Tag
		{
		return 0;
		}

	iMax -= 1;

	if(uLen==XML_AUTOSIZE)								// Lnge des Attributnamens ermitteln
		{
		uLen = strlen(pName);
		}
	

	for(uPos=uTagPos;uPos<uXmlPos;uPos++)				// Attribute suchen
		{
		   ucSign = pXmlData[uPos];
		if(ucSign<=' ')continue;

		for(uEnd=uPos;uEnd<uXmlPos;uEnd++)				// Lnge des Attributnamens ermitteln
			{
			ucSign = pXmlData[uEnd];

			if(ucSign<'0' || ucSign>'z')break;
			if(ucSign>'9' && ucSign<'A')break;
			if(ucSign>'Z' && ucSign<'a')break;
			}
		
		uSize = uEnd-uPos;

		for(;uEnd<uXmlPos;uEnd++)						// Leerzeichen vor nach '=' berspringen
			{
			   ucSign = pXmlData[uEnd];
			if(ucSign>' ')break;
			}

		if(pXmlData[uEnd]!='=')
			{
			return 0;
			}

		for(uEnd++;uEnd<uXmlPos;uEnd++)					// Leerzeichen nach '=' berspringen
			{
			   ucSign = pXmlData[uEnd];
			if(ucSign>' ')break;
			}

		if(uLen==uSize && !memcmp(pName,pXmlData+uPos,uLen))
			{
			uOff = 0;

			if(pXmlData[uEnd]=='"')
				{
				for(uEnd++;uEnd<uXmlPos;uEnd++)			// Text zwischen Anfhrungszeichen holen
					{
					   ucSign = pXmlData[uEnd];
					if(ucSign=='"')break;

					pData[uOff] = ucSign;
					uOff++;
					if(uOff>=(unsigned)iMax)break;
					}
				
				pData[uOff] = 0;

				return uOff;	
				}

			for(;uEnd<uXmlPos;uEnd++)					// Nchstes Wort holen
				{
				   ucSign = pXmlData[uEnd];
				if(ucSign<=' ')break;

				pData[uOff] = ucSign;
				uOff++;
				if(uOff>=(unsigned)iMax)break;
				}
			
			pData[uOff] = 0;

			return uOff;	
			}

		uPos = uEnd;
		

		if(pXmlData[uPos]=='"')
			{
			for(uPos++;uPos<uXmlPos;uPos++)				// Vor zum nchsten Attribut
				{
				   ucSign = pXmlData[uPos];
				if(ucSign=='"'){uPos++;break;}
				}

			continue;
			}


		for(;uPos<uXmlPos;uPos++)						// Vor zum nchsten Attribut
			{
			   ucSign = pXmlData[uPos];
			if(ucSign<=' ')break;
			}
		}


	*pData = 0;


return -1;
}


