//*****************************************************************************
//*
//*
//*		XmlParser.h
//*
//*
//*****************************************************************************
//
//	Mit dieser Klasse knne XML-Dateien geparst werden.
//	Die XML Datei wird dabei in einem Speicher Block geladen und Tag 
//	fr Tag durchgegangen.
//
#ifndef 	__XMPPARSER_H__
#define 	__XMPPARSER_H__


#define 	XML_PARSER_LEVELS	32
#define 	XML_PARSER_TAGS		64
#define 	XML_PARSER_DATA		260
#define 	XML_AUTOSIZE		0x12345678


//*****************************************************************************
//*
//*		Klasse CXmlParser
//*
//*****************************************************************************
class CXmlParser
{
protected:

	char		   *pXmlData;							// Zeiger auf die Xml-Daten
	unsigned		uXmlSize;							// Lnge der Xml-Daten in Byte
	unsigned		uXmlPos;							// Aktuelle Position fr's parsen
	unsigned		uXmlDataEnd;						// Ende des letzten Datenscanns
	unsigned		uTagPos;							// Startposition vom aktuellen Tag
	int				iXmlLevel;							// Aktuelle Hierachieebene in der XML-Datei
	int				iDataFlag;							// Ist der Datenbereich bereitsgesannt
	int				iHasCData;							// War ein CDATA Bereich forhanden
	int				iHasAnd;							// War ein '&' Zeichen forhanden
	unsigned		aLevelPos [XML_PARSER_LEVELS];		// Positionen der Level-Tags	
	unsigned		aLevelSize[XML_PARSER_LEVELS];		// Lngen der Level-Tags
	char			cDataTemp [XML_PARSER_DATA  ];		// Temporrer Daten-Speicher	
	char			cTagTemp  [XML_PARSER_TAGS  ];		// Temporrer Tag-Name-Speicher	

protected:

	static int	DecodeText(const char *pText,int iTextSize,char *pTarget,int iMax);
	int			NextTag(char  *pName,int iNameSize,char **pAttr,int iAttrSize);
	int			GetData(char **pData,int iDataSize);
	void		Init();	


public:		

	CXmlParser();
   ~CXmlParser();

	int			Close();
	int			Reset(int iLevel=-1);
	int			Level(){return iXmlLevel;}
	int			Open   (const char *pXmlFile);		
	int			GotoTag(const char *pPath);		
	int			NextTag(char  *pName,int iNameSize,char  *pAttr,int iAttrSize);
	int			NextTag(char  *pName,int iNameSize,char **pAttr);
	int			NextTag(char  *pName,int iNameSize);
	const char *NextTag(char **pAttr);
	const char *NextTag();
	const char *LevelName(int iLevel);
	int			LevelName(int iLevel,char  *pName,int iNameSize);
	int			LevelDown(int iLevel);
	int			LevelDown();
	int			GetAttr(const char *pName,unsigned uLen=XML_AUTOSIZE);
	int			GetAttr(const char *pName                     ,unsigned uLen=XML_AUTOSIZE);
	int			GetAttr(const char *pName,char *pData,int iMax,unsigned uLen=XML_AUTOSIZE);
	int			GetData(char  *pData,int iDataSize);
	int			GetData(char **pData);
	const char *GetData();
	const char *GetDirectData();
	int			GetDirectLength();
	int			GetValue();
	static int	GetValue(const char *pText);
	int			GetBinary(void *pMem,int iMax);
	int			GetTagPath(char *pBuffer,int iMax,int iLevel=XML_PARSER_LEVELS);

};	


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline int	CXmlParser::NextTag(char *pName,int iNameSize)
	{
	return NextTag(pName,iNameSize,(char**)NULL,0);
	}

inline int	CXmlParser::NextTag(char *pName,int iNameSize,char *pAttr,int iAttrSize)
	{
	char **pPtr;
	pPtr = (pAttr)? &pAttr:NULL;
	return NextTag(pName,iNameSize,pPtr,iAttrSize);
	}

inline int CXmlParser::NextTag(char *pName,int iNameSize,char **pAttr)
	{
	*pAttr=NULL;
	return NextTag(pName,iNameSize,pAttr,1);
	}

inline const char *CXmlParser::NextTag(char **pAttr)
	{
	int iRet;
	*pAttr=NULL;
	iRet=NextTag(cTagTemp,sizeof(cTagTemp),pAttr,1);
	return (iRet>0)? cTagTemp:NULL; 
	}

inline const char *CXmlParser::NextTag()
	{
	int iRet;
	iRet=NextTag(cTagTemp,sizeof(cTagTemp));
	return (iRet>0)? cTagTemp:NULL; 
	}

inline const char *CXmlParser::LevelName(int iLevel)
	{
	int iRet;
	iRet=LevelName(iLevel,cTagTemp,sizeof(cTagTemp));
	return (iRet>0)? cTagTemp:NULL; 
	}

inline int	CXmlParser::GetData(char *pData,int iDataSize)
	{
	return GetData(&pData,iDataSize);
	}

inline int	CXmlParser::GetData(char **pData)
	{
	*pData=NULL;
	return GetData(pData,1);
	}

inline const char *CXmlParser::GetData()
	{
	GetData(cDataTemp,sizeof(cDataTemp));
	return cDataTemp;
	}

inline const char *CXmlParser::GetDirectData()
	{
	unsigned uPos;
	uPos=GetData((char**)NULL,-12345678);
	return pXmlData+uPos;
	}

inline int  CXmlParser::LevelDown()
	{
	return LevelDown(iXmlLevel-1);
	}

#endif 			  
