//*****************************************************************************
//*
//*
//*		ToolbarWndEx.cpp
//*
//*
//*****************************************************************************
//
//	Dieser Code erstellt ein Toolbar
//
//
//	Copyright (C) Anton Zechner 2017
//
//	ToolbarWnd is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
//	Sourcecode which use ToolbarWnd must be published. Commercial users
//	must published thier code too, or make an licence agreement with me.
//
//
//	ToolbarWnd wird unter GNU GENERAL PUBLIC LICENSE (GPL) vertreiben.
//	Sourcecode welcher ToolbarWnd verwendet muss verffentlicht werden.
//	Komerzielle Nutzer mssen ihren Code ebenfalls verffentlichen, oder
//	eine Nutzungsvereinbarung mit mir treffen.
//
//	az_software@inode.at
//
//	Version:	1.03
//
#if		   _MSC_VER > 1299
#ifndef		WINVER
#define		WINVER 	0x0501
#endif
#endif

#ifdef		UNICODE
#ifndef	   _UNICODE
#define    _UNICODE
#endif
#endif

#include	<stdio.h>

#include	<windows.h>
#include	<winuser.h>
#include	<commctrl.h>
#include	<string.h>
#include	<malloc.h>
#include	<tchar.h>

#include	"WinExtension.h"
#include	"ToolbarExWnd.h"

#ifdef	   _MSC_VER
#pragma		warning( disable :4996 )
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define 	U(h)				((unsigned)(h))
#define		SEPARATOR_WIDTH		8
#define 	DEFPAD_CX			7
#define 	DEFPAD_CY			7
#define 	DEFGAP_CX			2
#define 	DEFGAP_CY			4
#define 	TEXTOFF_X			2
#define 	TEXTOFF_Y			1
#define 	TEXTADD_X			2
#define 	TEXTADD_Y			2
#define 	THEMEIMGLIST		((HIMAGELIST)1)
#define 	MHWORD(w)			(((w)>>16)&0xFF80)
#define 	MLWORD(w)			(((w)>>16)&0x007F)
#define 	GetHandle(h)		((ToolbarData*)GetWindowLongPtr(h,0))
#define 	SYNCH_TYPE			CRITICAL_SECTION
#define 	SYNCH_CREATE(d)		InitializeCriticalSection(&d->sSynch);
#define 	SYNCH_DELETE(d)		DeleteCriticalSection(&d->sSynch);
#define 	BMI_ROW_WIDTH(b)	((((((b.biBitCount)==15)? 16:(b.biBitCount))*(b.biWidth)+31)/32)*4)
#define 	BMI_SIZE			(sizeof(BITMAPINFOHEADER)+256*sizeof(RGBQUAD))
#define 	BMI_SIZE_IDX		(sizeof(BITMAPINFOHEADER)+256*sizeof(WORD))
#define 	BMI_HEADER			BITMAPINFOHEADER
#define 	UNLOCK(d)			{													\
								if(d->sSynch.LockCount==0xFFFFFFFF)					\
									{												\
									char aLine[16];									\
									sprintf(aLine,"%i",__LINE__);					\
									MessageBoxA(0,aLine,"Unlock Error",MB_OK);		\
									}												\
								LeaveCriticalSection(&d->sSynch);					\
								}
#define 	LOCK(d)				EnterCriticalSection(&d->sSynch)
#ifdef		DrawText
#undef 		DrawText
#define 	DrawText(h,p,n,r,f)	DrawTextEx(h,(LPTSTR)(p),n,r,f,0)
#endif

#define 	MAX_COLORS			9
#define 	ID_UPDATE			0x197F
#define 	TBRF_SHIFT			0x00000001					// Unterbild beginnt bei der 2.ten X-Spalte
#define 	TBRF_FIXSIZE		0x00000002					// Alle Unterbilder sind gleich gro
#define 	TBRF_VARSIZE		0x00000004					// Das Bitmap hat eine vraiable Gre
#define 	TBRF_SCANNED		0x00000008					// Wurden alle Unterbilder eingelesen
#define 	TBRF_DELETE			0x00000010					// HABITMAP nach Ende lschen

#define 	TBIF_STRETCH		0x00000010					// Die Tastenbreite ber auf die Reihe strecken
#define 	TBIF_SIZE_OK		0x00000020					// Wurde die Tasten-Gre bereits ermittelt
#define 	TBIF_BITMAP_OK		0x00000040					// Die Gre und die Position des Bitmaps wurde ermittelt
#define 	TBIF_TEXT_BOTTOM	0x00000080					// Es ist ein Text unten vorhanden
#define 	TBIF_TEXT_GROUP		0x00000100					// Es ist ein Gruppentext unten vorhanden
#define 	TBIF_TEXT_RIGHT		0x00000200					// Es ist ein Text rechts vorhanden
#define 	TBIF_TEXT_EMPTY		0x00000400					// Es ist ein Leertext unten vorhanden
#define 	TBIF_REASSEMBLE		0x00000800					// Der Bilder mssen neu erstellt werden
#define 	TBIF_PACKED			0x00001000					// Die Taste ist ber eine andere gestellt
#define 	TBIF_BASE			0x00002000					// Die Taste ist ist unter anderen gestellt
#define 	TBIF_LAST			0x00004000					// Die Taste ist ist die letzte in einem Pack
#define 	TBIF_FULL_BUTTON	0x00008000					// Es ist eine Taste ber die ganze Hhe mit Text
#define 	TBIF_MASK			(TBIF_TEXT_BOTTOM|TBIF_TEXT_RIGHT)

#define 	TBFL_ZERO_FLAGS		0x00000001					// Die Flags in TBBUTTONEX auf 0 setzen

#define 	CHECK_BITCNT(b)		((0x01000111>>(b))&1)		// Wird der Bit-Count untersttzt


typedef struct
	{
	HINSTANCE		hInstance;								// Instance des Bildes
	DWORD_PTR		uBitmapId;								// Ist die ID des Bitmaps
	BITMAPINFO	   *pInfo;									// Header der Bilddaten
	LPBYTE			pPixels;								// Zeiger auf die Pixeldaten
	WORD			wFlags;									// Diverse Flags
	WORD			wBaseIndex;								// Index des ersten Bildes
	WORD			wRowWidth;								// Breite einer Zeile in Byte
	WORD			wCount;									// Anzahl der Bilder
	WORD			wScale;									// Wie soll die Recource skaliert werden (256=1:1)
	WORD			wLastPos;								// Nummer des zuletzten gesannten Bildes
	WORD			wLastX;									// Position des letzten Bildes (links unten)
	WORD			wLastY;									//			*
	WORD			wSizeX;									// Gre eines Einzelbildes
	WORD			wSizeY;									//			*
	}TBRESOURCE,*LPTBRESOURCE;

typedef struct
	{
	WORD			wInfoFlags;								// Diverse interne Info-Bits
	WORD			wSizeX;									// Reale Breite des Unterbildes
	WORD			wSizeY;									// Reale Hhe des Unterbildes
	WORD			wRealX;									// Reale Breite des Unterbildes
	WORD			wRealY;									// Reale Hhe des Unterbildes
	BYTE			bState;									// Tasten-Status (siehe TBSTATE_???)
	}ItemSave;

typedef struct TagDataStr
	{
	TagDataStr	   *pNext;									// Zeiger auf das 
	SHORT			sPosX;									// Position des TAG's
	SHORT			sPosY;									//			*
	TBTAGITEM		sItem;
	WCHAR			cText[1];								// Text fr das TAG
	}TBTAGDATA,*LPTBTAGDATA;

typedef struct
	{
	INT				iBitmap;								// Index des Bildes (-1==keines)
	INT       		idCommand;								// ID fr WM_COMMAND
	BYTE      		fsState;								// Tasten-Status (siehe TBSTATE_???)
	BYTE      		fsStyle;								// Tasten-Style  (siehe TBSTYLE_???)
	WORD			wFlags;									// Diverse Flags
	#ifdef		   _WIN64
	BYTE			bReserved[4];							// Unbenutzte Dummy-Bytes
	#endif
	DWORD_PTR		dwData;									// User-Daten fr die Taste
	INT_PTR			iString;								// Index des Textes

	WORD			wInfoFlags;								// Diverse interne Info-Bits
	WORD			wResource;								// Ist die Nummer der Resource
	WORD			wResPic;								// Ist die Nummer des Bildes in der Resource
	WORD			wRealX;									// Reale Breite des Unterbildes
	WORD			wRealY;									// Reale Hhe des Unterbildes
	WORD			wSizeX;									// Breite eines Unterbildes in Pixel
	WORD			wSizeY;									// Hhe eines Unterbildes in Pixel
	WORD			wShowX;									// Ist die Breite der Taste
	RECT			sRect;									// Position im Fenster
	LPBITMAPINFO	pInfo;									// Zeiger auf die Bilddaten
	LPBYTE			pPixels;								// Zeiger auf die Pixel-Daten des Einzelbildes
	HBITMAP			hBitmap;								// Ist das Bitmap-Handle des Einzelbildes
	}TBITEM,*LPTBITEM;


typedef struct
	{
	HWND			hWnd;									// Fenster Handle
	SYNCH_TYPE		sSynch;									// Zugriffsbeschgrnkung
	LPVOID			hTheme;									// Handle fr benutztes Thema (TOOLBAREX)
	LPVOID			hThemeBt;								// Handle fr benutztes Thema (BUTTON)
	HFONT			hFontN;									// Ist der normale Zeichensatz
	HFONT			hFontL;									// Der letzte zugewiesene Font
	UINT			uFontHeigh;								// Hhe des Fonts
	HWND			hParent;								// Fenster das die Nachrichten enpfngt
	HWND			hParentEx;								// Fenster das die Control-Nachrichten enpfngt
	HWND			hParentCtrl;							// Fenster das die Control-Nachrichten enpfngt (2.te)
	COLORREF		aColorChange [2];						// Farben fr ndern der Icons
	COLORREF		aColors      [MAX_COLORS];				// 0=Hintergrundfarbe  1=Frames  2=Textfarbe 3=Textfarbe wenn gesperrt
	char			aColorChanged[MAX_COLORS];				// Welche Farbe wurden verndert

	int				iFontHeight;							// Ist die Hhe des Fonts
	int				iFontLine;								// Ist die Position der Linie beim unterstreichen
	int				iFontOff;								// Ist die Position um der ein Text horizontal verschoben wird
	int				iIndent;								// Ist der X-Offset der ersten Taste
	int				iLastLine;								// Ist die Breite der letzten Zeile
	int				iMinWidthX;								// Minimale Tastenbreite bei Texten (-1=ungeprft)
	int				iMaxWidthX;								// Minimale Tastenbreite bei Texten	(-1=ungeprft)
	int				iButtonSizeX;							// Gre einer Taste (Bitmap samt Rahmen/Padding)
	int				iButtonSizeY;							//			*
	int				iBitmapSizeX;							// Gre eines einzelnen Bitmaps
	int				iBitmapSizeY;							//			*
	int				iPaddingX;								// Pixels zwischen Tastenrahmen und Bild
	int				iPaddingY;								//			*
	int				iGabX;									// Abstand zwischen den Tasten
	int				iGabY;									//			*

	LPTBRESOURCE	pResource;								// Liste mit den Bitmap Resourcen
	unsigned		uResourceMax;							// Array-Gre der Resourcen
	unsigned		uResourceCnt;							// Anzahl der Resourcen

	LPTBITEM	   *pItems;									// Liste mit Tasten
	unsigned		uItemsMax;								// Array-Gre der Tasten
	unsigned		uItemsCnt;								// Anzahl der Tasten

	LPWSTR		   *pStrings;								// Zeiger auf die Texte
	unsigned		uStringMax;								// Anzahl der Texte
	unsigned		uStringCnt;								// Anzahl der Texte

	TBTAGDATA	   *pTagChain;								// Liste mit Tags

	unsigned		uLines;									// Wievile Zeilen hat die Toolbar
	unsigned		uStructSize;							// Gre einer Tastenstruktur
	unsigned		uButtonZoom;							// Umwieviele 1/256 Steps werden die Bilder gezoomt
	unsigned		uDropState;								// Ist eine Dropdownd-Taste gedrckt
	unsigned		uMoveItem;								// Auf welcher Tatste war der Cursor zuletzt

	unsigned		uMarkItem;								// Ist die Taste mit der Marke
	unsigned		uMarkMode;								// Modus fr Marke (Bit0=nachher)
	RECT			sMarkRect;								// Ist das Rechteck mit der Marke

	unsigned		uMaxX;									// Aktuelle X-Gre	aller Tasten
	unsigned		uMaxY;									// Aktuelle Y-Gre aller Tasten
	unsigned		uSizeX;									// Aktuelle X-Fenster-Gre
	unsigned		uSizeY;									// Aktuelle Y-Fenster-Gre
	unsigned		uStyle;									// Ist der aktuelle Style des Fensters
	unsigned		uStyleEx;								// Erweiterte Sytle-Flags (siehe TVS_EX_???)

	unsigned	 	uButtonPos;								// Wo wurde eine Maustaste wurde zuletzt gedrckt
	unsigned	 	uButtonLast;							// Wann wurde eine Maustaste wurde zuletzt gedrckt
	char			cButtonFlag;							// Welche Maustaste wurde zuletzt gedrckt

	char			cBitPixels;								// Bits je Pixel
	char			cFullColor;								// Knnen 32-Bit Farben benutzt werden
	char			cClickFlag;								// Merker fr LBUTTON-DOWN bei Multiselect
	char			cIsEnabled;								// Ist das Fenster freigegeben
	char			cMoveFlags;								// Die alten Flags der Tasten
	char			cMoveDrop;								// Ist der Cursor auf eine Dropdown Taste
	char			cHasFocus;								// Hat das Fenster den Fokus
	char			cGlyphOk;								// Die Schaltflche ber Themen zeichnen
	char			cReCalc;								// Was muss neu berechnet werden (Bit0=Tastenpositionen Bit1=Tastenabmessungen Bit2=Alle Bilder Bit3=Gren neu berechnen)
	char			cIsHorz;								// Aktuelle Lage (1=Horizontal 0=Vertical)
	char			cAlign;									// 0=Horizontal 1=Vertical
	}ToolbarData;

typedef	HRESULT (WINAPI *SetWindowThemeT )(HWND,LPCWSTR,LPCWSTR);
typedef	HRESULT (WINAPI *EndBufferedPtT  )(HANDLE,BOOL);
typedef	HANDLE  (WINAPI *BeginBufferedPnT)(HDC,RECT*,DWORD,LPVOID,HDC*);
typedef	HRESULT (WINAPI *BufferedPtInitT )(VOID);
typedef	LPVOID  (WINAPI *OpenThemeDataT  )(HWND hwnd,LPCWSTR pszClassList);
typedef	HRESULT (WINAPI *CloseThemeDataT )(LPVOID);
typedef	HRESULT (WINAPI *DrawThemeBackgT )(LPVOID,HDC,int,int,const RECT*,const RECT*);
typedef	HRESULT (WINAPI *GetThemeBackgRcT)(LPVOID,HDC,int,int,LPCRECT,LPRECT);
typedef	HRESULT (WINAPI *GetThemeMetricT )(LPVOID,HDC,int,int,int,int*);
typedef	HRESULT (WINAPI *GetThemePartSzT )(LPVOID,HDC,int,int,LPCRECT,int,LPSIZE);
typedef	DWORD   (__fastcall   *GetPixelT )(LPBYTE pRow,UINT uCol);

static HMODULE			hUxThemeDll		= NULL;
static SetWindowThemeT	pSetWindowTheme	= NULL;
static EndBufferedPtT	pEndBufferedPt  = NULL;
static BeginBufferedPnT	pBeginBufferedPt= NULL;
static BufferedPtInitT	pBufferedPtInit = NULL;
static BufferedPtInitT	pBufferedPtExit = NULL;
static OpenThemeDataT	pOpenThemeData	= NULL;
static CloseThemeDataT	pCloseThemeData	= NULL;
static DrawThemeBackgT	pDrawThemeBackg	= NULL;
static GetThemeBackgRcT	pGetThemeBackgRc= NULL;
static GetThemePartSzT	pGetThemePartSz = NULL;

static HDC				hDcMain			= 0;				// Handle vom Hauptfenster
static UINT				uColorBits		= 0;				// Anzahl der Bits fr den Gerte-DC
static UINT				uButtonBits		= 0;				// Anzahl der Bits fr die Buttons
static HFONT			hDefaultFontN	= NULL;
static HFONT			hDefaultFontB	= NULL;
static HCURSOR			hCursorNo		= NULL;
static HCURSOR			hCursorMove		= NULL;
static HCURSOR			hCursorArrow	= NULL;
static LONG				lWindowCount	= -1;
static void				ToolbarDraw(HWND hWnd,HDC hDc,RECT *pRect);
static LRESULT CALLBACK ToolbarProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);

static const BYTE		aFrame1[] =
	{
	0x28,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x01,0x00,0x20,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,
	0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x24,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x23,0x54,0x54,0x54,0x58,0xEB,0xEB,0xEB,0x76,
	0xFF,0xFF,0xFF,0x7C,0xEB,0xEB,0xEB,0x76,0x54,0x54,0x54,0x58,0x00,0x00,0x00,0x23,
	0x00,0x00,0x00,0x43,0xE7,0xE7,0xE7,0x77,0xFA,0xFA,0xFA,0x68,0xF0,0xF0,0xF0,0x12,
	0xFA,0xFA,0xFA,0x68,0xE7,0xE7,0xE7,0x77,0x00,0x00,0x00,0x43,0x00,0x00,0x00,0x4D,
	0xF8,0xF8,0xF8,0x7F,0xDA,0xDA,0xDA,0x15,0xDA,0xDA,0xDA,0x15,0xDA,0xDA,0xDA,0x15,
	0xF8,0xF8,0xF8,0x7F,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xF5,0xF5,0xF5,0x81,
	0xD4,0xD4,0xD4,0x18,0xD4,0xD4,0xD4,0x18,0xD4,0xD4,0xD4,0x18,0xF5,0xF5,0xF5,0x81,
	0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xEF,0xEF,0xEF,0x83,0xBC,0xBC,0xBC,0x1B,
	0xBC,0xBC,0xBC,0x1B,0xBC,0xBC,0xBC,0x1B,0xEF,0xEF,0xEF,0x83,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xE9,0xE9,0xE9,0x85,0xB2,0xB2,0xB2,0x1E,0xB2,0xB2,0xB2,0x1E,
	0xB2,0xB2,0xB2,0x1E,0xE9,0xE9,0xE9,0x85,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,
	0xE2,0xE2,0xE2,0x87,0x9A,0x9A,0x9A,0x21,0x9A,0x9A,0x9A,0x21,0x9A,0x9A,0x9A,0x21,
	0xE2,0xE2,0xE2,0x87,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xDF,0xDF,0xDF,0x88,
	0x8D,0x8D,0x8D,0x24,0x8D,0x8D,0x8D,0x24,0x8D,0x8D,0x8D,0x24,0xDF,0xDF,0xDF,0x88,
	0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xD8,0xD8,0xD8,0x8A,0x7F,0x7F,0x7F,0x26,
	0x7F,0x7F,0x7F,0x26,0x7F,0x7F,0x7F,0x26,0xD8,0xD8,0xD8,0x8A,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xD6,0xD6,0xD6,0x8A,0x72,0x72,0x72,0x28,0x72,0x72,0x72,0x28,
	0x72,0x72,0x72,0x28,0xD6,0xD6,0xD6,0x8A,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,
	0xD2,0xD2,0xD2,0x8B,0x6D,0x6D,0x6D,0x2A,0x6D,0x6D,0x6D,0x2A,0x6D,0x6D,0x6D,0x2A,
	0xD2,0xD2,0xD2,0x8B,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x7D,
	0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x7D,
	0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x82,0xC5,0xC5,0xC5,0x31,
	0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x82,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x87,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,
	0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x87,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,
	0xFF,0xFF,0xFF,0x8D,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,
	0xFF,0xFF,0xFF,0x8D,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x93,
	0xD3,0xD3,0xD3,0x3A,0xD3,0xD3,0xD3,0x3A,0xD3,0xD3,0xD3,0x3A,0xFF,0xFF,0xFF,0x93,
	0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x9A,0xDE,0xDE,0xDE,0x46,
	0xDE,0xDE,0xDE,0x46,0xDE,0xDE,0xDE,0x46,0xFF,0xFF,0xFF,0x9A,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xA2,0xE6,0xE6,0xE6,0x52,0xE6,0xE6,0xE6,0x52,
	0xE6,0xE6,0xE6,0x52,0xFF,0xFF,0xFF,0xA2,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,
	0xFF,0xFF,0xFF,0xA9,0xEF,0xEF,0xEF,0x60,0xEF,0xEF,0xEF,0x60,0xEF,0xEF,0xEF,0x60,
	0xFF,0xFF,0xFF,0xA9,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xB0,
	0xF5,0xF5,0xF5,0x6D,0xF5,0xF5,0xF5,0x6D,0xF5,0xF5,0xF5,0x6D,0xFF,0xFF,0xFF,0xB0,
	0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x43,0xEF,0xEF,0xEF,0xA8,0xFF,0xFF,0xFF,0xA9,
	0xFA,0xFA,0xFA,0x78,0xFF,0xFF,0xFF,0xA9,0xEF,0xEF,0xEF,0xA8,0x00,0x00,0x00,0x43,
	0x00,0x00,0x00,0x23,0x6C,0x6C,0x6C,0x67,0xEE,0xEE,0xEE,0xAA,0xFF,0xFF,0xFF,0xB9,
	0xEE,0xEE,0xEE,0xAA,0x6C,0x6C,0x6C,0x67,0x00,0x00,0x00,0x23,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x22,0x00,0x00,0x00,0x42,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x42,
	0x00,0x00,0x00,0x22,0x00,0x00,0x00,0x00,
	};

static const BYTE		aFrame2[] =
	{
	0x28,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x01,0x00,0x20,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4E,
	0x00,0x00,0x00,0x92,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x92,0x00,0x00,0x00,0x4E,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4C,0x0A,0x0A,0x0A,0x94,0x2D,0x2D,0x2D,0x3D,
	0x2D,0x2D,0x2D,0x11,0x00,0x00,0x00,0x20,0x01,0x01,0x01,0x84,0x00,0x00,0x00,0x4C,
	0x00,0x00,0x00,0x91,0x38,0x38,0x38,0x5E,0x38,0x38,0x38,0x24,0x00,0x00,0x00,0x0B,
	0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x91,0x00,0x00,0x00,0xA6,
	0x45,0x45,0x45,0x54,0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0E,
	0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,
	0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,
	0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,0x37,0x37,0x37,0x25,
	0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,
	0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,
	0x45,0x45,0x45,0x54,0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x44,0x44,0x44,0x56,
	0x34,0x34,0x34,0x27,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x13,0x00,0x00,0x00,0x13,
	0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x43,0x43,0x43,0x57,0x33,0x33,0x33,0x28,
	0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x41,0x41,0x41,0x59,0x30,0x30,0x30,0x2A,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,
	0x41,0x41,0x41,0x59,0x2F,0x2F,0x2F,0x2B,0x00,0x00,0x00,0x13,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x55,
	0x35,0x35,0x35,0x26,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,
	0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x55,0x35,0x35,0x35,0x26,
	0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x55,0x35,0x35,0x35,0x26,0x00,0x00,0x00,0x0D,
	0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,
	0x43,0x43,0x43,0x57,0x34,0x34,0x34,0x27,0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0x0F,
	0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x41,0x41,0x41,0x59,
	0x30,0x30,0x30,0x2A,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x3D,0x3D,0x3D,0x5B,0x2C,0x2C,0x2C,0x2E,
	0x00,0x00,0x00,0x16,0x00,0x00,0x00,0x16,0x00,0x00,0x00,0x16,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x3B,0x3B,0x3B,0x5F,0x23,0x23,0x23,0x32,0x00,0x00,0x00,0x1B,
	0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,
	0x38,0x38,0x38,0x63,0x24,0x24,0x24,0x38,0x00,0x00,0x00,0x21,0x00,0x00,0x00,0x1F,
	0x00,0x00,0x00,0x20,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x36,0x36,0x36,0x6C,
	0x27,0x27,0x27,0x47,0x0B,0x0B,0x0B,0x2C,0x00,0x00,0x00,0x26,0x0B,0x0B,0x0B,0x2C,
	0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x91,0x2C,0x2C,0x2C,0x7D,0x2E,0x2E,0x2E,0x62,
	0x25,0x25,0x25,0x4B,0x1D,0x1D,0x1D,0x45,0x2B,0x2B,0x2B,0x80,0x00,0x00,0x00,0x91,
	0x00,0x00,0x00,0x4C,0x09,0x09,0x09,0x9D,0x2B,0x2B,0x2B,0x80,0x31,0x31,0x31,0x71,
	0x2B,0x2B,0x2B,0x80,0x0D,0x0D,0x0D,0xAA,0x00,0x00,0x00,0x4C,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x4A,0x00,0x00,0x00,0x8F,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x8F,
	0x00,0x00,0x00,0x4A,0x00,0x00,0x00,0x00,
	};

static const BYTE		aFrame3[] =
	{
	0x28,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x01,0x00,0x20,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,
	0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x2D,
	0x00,0x00,0x00,0x23,0x54,0x54,0x54,0x58,0xEB,0xEB,0xEB,0x76,0xFF,0xFF,0xFF,0x7C,
	0xFF,0xFF,0xFF,0x7C,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x43,0xE7,0xE7,0xE7,0x77,
	0xFA,0xFA,0xFA,0x68,0xF0,0xF0,0xF0,0x12,0xF3,0xF3,0xF3,0x82,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xF8,0xF8,0xF8,0x7F,0xDA,0xDA,0xDA,0x15,0xDA,0xDA,0xDA,0x15,
	0xF3,0xF3,0xF3,0x82,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xF5,0xF5,0xF5,0x81,
	0xD4,0xD4,0xD4,0x18,0xD4,0xD4,0xD4,0x18,0xF3,0xF3,0xF3,0x82,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xEF,0xEF,0xEF,0x83,0xBC,0xBC,0xBC,0x1B,0xBC,0xBC,0xBC,0x1B,
	0xEF,0xEF,0xEF,0x83,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xE9,0xE9,0xE9,0x85,
	0xB2,0xB2,0xB2,0x1E,0xB2,0xB2,0xB2,0x1E,0xE9,0xE9,0xE9,0x85,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xE2,0xE2,0xE2,0x87,0x9A,0x9A,0x9A,0x21,0x9A,0x9A,0x9A,0x21,
	0xE2,0xE2,0xE2,0x87,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xDF,0xDF,0xDF,0x88,
	0x8D,0x8D,0x8D,0x24,0x8D,0x8D,0x8D,0x24,0xDF,0xDF,0xDF,0x88,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xD8,0xD8,0xD8,0x8A,0x7F,0x7F,0x7F,0x26,0x7F,0x7F,0x7F,0x26,
	0xD9,0xD9,0xD9,0x89,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xD6,0xD6,0xD6,0x8A,
	0x72,0x72,0x72,0x28,0x72,0x72,0x72,0x28,0xD6,0xD6,0xD6,0x8A,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xD2,0xD2,0xD2,0x8B,0x6D,0x6D,0x6D,0x2A,0x6D,0x6D,0x6D,0x2A,
	0xD2,0xD2,0xD2,0x8B,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x7D,
	0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x82,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x82,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,
	0xFF,0xFF,0xFF,0x87,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x87,
	0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x8C,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x8D,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,
	0xFF,0xFF,0xFF,0x92,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x93,
	0xD3,0xD3,0xD3,0x3A,0xD3,0xD3,0xD3,0x3A,0xFF,0xFF,0xFF,0x98,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x9A,0xDE,0xDE,0xDE,0x46,0xDE,0xDE,0xDE,0x46,
	0xFF,0xFF,0xFF,0x9F,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xA2,
	0xE6,0xE6,0xE6,0x52,0xE6,0xE6,0xE6,0x52,0xFF,0xFF,0xFF,0xA6,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xA9,0xEF,0xEF,0xEF,0x60,0xEF,0xEF,0xEF,0x60,
	0xFF,0xFF,0xFF,0xAC,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xB0,
	0xF5,0xF5,0xF5,0x6D,0xF5,0xF5,0xF5,0x6D,0xFF,0xFF,0xFF,0xB1,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x43,0xEF,0xEF,0xEF,0xA8,0xFF,0xFF,0xFF,0xA9,0xFA,0xFA,0xFA,0x78,
	0xFF,0xFF,0xFF,0xB6,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x23,0x6C,0x6C,0x6C,0x67,
	0xEE,0xEE,0xEE,0xAA,0xFF,0xFF,0xFF,0xB9,0xFF,0xFF,0xFF,0xB9,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x22,0x00,0x00,0x00,0x42,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x2D,
	};


static const BYTE		aFrame4[] =
	{
	0x28,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x01,0x00,0x20,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4E,
	0x00,0x00,0x00,0x92,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x62,
	0x00,0x00,0x00,0x4C,0x0A,0x0A,0x0A,0x94,0x2D,0x2D,0x2D,0x3D,0x2D,0x2D,0x2D,0x11,
	0x2D,0x2D,0x2D,0x11,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x91,0x38,0x38,0x38,0x5E,
	0x38,0x38,0x38,0x24,0x00,0x00,0x00,0x0B,0x00,0x00,0x00,0x0B,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,
	0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,
	0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,
	0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,
	0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x54,0x37,0x37,0x37,0x25,0x00,0x00,0x00,0x0C,
	0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x44,0x44,0x44,0x56,
	0x34,0x34,0x34,0x27,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x43,0x43,0x43,0x57,0x33,0x33,0x33,0x28,0x00,0x00,0x00,0x10,
	0x00,0x00,0x00,0x10,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x41,0x41,0x41,0x59,
	0x30,0x30,0x30,0x2A,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x41,0x41,0x41,0x59,0x2F,0x2F,0x2F,0x2B,0x00,0x00,0x00,0x13,
	0x00,0x00,0x00,0x13,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x55,
	0x35,0x35,0x35,0x26,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x55,0x35,0x35,0x35,0x26,0x00,0x00,0x00,0x0D,
	0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x45,0x45,0x45,0x55,
	0x35,0x35,0x35,0x26,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x43,0x43,0x43,0x57,0x34,0x34,0x34,0x27,0x00,0x00,0x00,0x0F,
	0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x41,0x41,0x41,0x59,
	0x30,0x30,0x30,0x2A,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x3D,0x3D,0x3D,0x5B,0x2C,0x2C,0x2C,0x2E,0x00,0x00,0x00,0x16,
	0x00,0x00,0x00,0x16,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x3B,0x3B,0x3B,0x5F,
	0x23,0x23,0x23,0x32,0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x38,0x38,0x38,0x63,0x24,0x24,0x24,0x38,0x00,0x00,0x00,0x21,
	0x00,0x00,0x00,0x21,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0xA6,0x36,0x36,0x36,0x6C,
	0x27,0x27,0x27,0x47,0x0B,0x0B,0x0B,0x2C,0x0B,0x0B,0x0B,0x2C,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0x91,0x2C,0x2C,0x2C,0x7D,0x2E,0x2E,0x2E,0x62,0x25,0x25,0x25,0x4B,
	0x25,0x25,0x25,0x4B,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x4C,0x09,0x09,0x09,0x9D,
	0x2B,0x2B,0x2B,0x80,0x31,0x31,0x31,0x71,0x31,0x31,0x31,0x71,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4A,0x00,0x00,0x00,0x8F,0x00,0x00,0x00,0xA6,
	0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x62,
	};


static const BYTE		aFrame5[] =
	{
	0x28,0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x01,0x00,0x20,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,
	0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x24,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0x7C,
	0xFF,0xFF,0xFF,0x7C,0xEB,0xEB,0xEB,0x76,0x58,0x58,0x58,0x59,0x00,0x00,0x00,0x23,
	0xFC,0xFC,0xFC,0x7D,0xF0,0xF0,0xF0,0x12,0xF9,0xF9,0xF9,0x65,0xE7,0xE7,0xE7,0x77,
	0x00,0x00,0x00,0x43,0xFA,0xFA,0xFA,0x7F,0xDA,0xDA,0xDA,0x15,0xDA,0xDA,0xDA,0x15,
	0xFA,0xFA,0xFA,0x7F,0x00,0x00,0x00,0x4D,0xF7,0xF7,0xF7,0x80,0xD4,0xD4,0xD4,0x18,
	0xD4,0xD4,0xD4,0x18,0xF7,0xF7,0xF7,0x80,0x00,0x00,0x00,0x4D,0xF3,0xF3,0xF3,0x82,
	0xBC,0xBC,0xBC,0x1B,0xBC,0xBC,0xBC,0x1B,0xF3,0xF3,0xF3,0x82,0x00,0x00,0x00,0x4D,
	0xEF,0xEF,0xEF,0x83,0xB2,0xB2,0xB2,0x1E,0xB2,0xB2,0xB2,0x1E,0xEF,0xEF,0xEF,0x83,
	0x00,0x00,0x00,0x4D,0xE9,0xE9,0xE9,0x85,0x9A,0x9A,0x9A,0x21,0x9A,0x9A,0x9A,0x21,
	0xE9,0xE9,0xE9,0x85,0x00,0x00,0x00,0x4D,0xE2,0xE2,0xE2,0x87,0x8D,0x8D,0x8D,0x24,
	0x8D,0x8D,0x8D,0x24,0xE2,0xE2,0xE2,0x87,0x00,0x00,0x00,0x4D,0xDF,0xDF,0xDF,0x88,
	0x7F,0x7F,0x7F,0x26,0x7F,0x7F,0x7F,0x26,0xDF,0xDF,0xDF,0x88,0x00,0x00,0x00,0x4D,
	0xD9,0xD9,0xD9,0x89,0x72,0x72,0x72,0x28,0x72,0x72,0x72,0x28,0xD9,0xD9,0xD9,0x89,
	0x00,0x00,0x00,0x4D,0xD6,0xD6,0xD6,0x8A,0x6D,0x6D,0x6D,0x2A,0x6D,0x6D,0x6D,0x2A,
	0xD6,0xD6,0xD6,0x8A,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x87,0xC5,0xC5,0xC5,0x31,
	0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x87,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x87,
	0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x87,0x00,0x00,0x00,0x4D,
	0xFF,0xFF,0xFF,0x8C,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,0xFF,0xFF,0xFF,0x8C,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x92,0xC5,0xC5,0xC5,0x31,0xC5,0xC5,0xC5,0x31,
	0xFF,0xFF,0xFF,0x92,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x98,0xD3,0xD3,0xD3,0x3A,
	0xD3,0xD3,0xD3,0x3A,0xFF,0xFF,0xFF,0x98,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0x9F,
	0xDE,0xDE,0xDE,0x46,0xDE,0xDE,0xDE,0x46,0xFF,0xFF,0xFF,0x9F,0x00,0x00,0x00,0x4D,
	0xFF,0xFF,0xFF,0xA6,0xE6,0xE6,0xE6,0x52,0xE6,0xE6,0xE6,0x52,0xFF,0xFF,0xFF,0xA6,
	0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xAC,0xEF,0xEF,0xEF,0x60,0xEF,0xEF,0xEF,0x60,
	0xFF,0xFF,0xFF,0xAC,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xB1,0xF5,0xF5,0xF5,0x6D,
	0xF5,0xF5,0xF5,0x6D,0xFF,0xFF,0xFF,0xB1,0x00,0x00,0x00,0x4D,0xFF,0xFF,0xFF,0xB6,
	0xFA,0xFA,0xFA,0x78,0xFF,0xFF,0xFF,0xAA,0xEF,0xEF,0xEF,0xA8,0x00,0x00,0x00,0x43,
	0xFF,0xFF,0xFF,0xB9,0xFF,0xFF,0xFF,0xB9,0xEE,0xEE,0xEE,0xAA,0x6C,0x6C,0x6C,0x67,
	0x00,0x00,0x00,0x23,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x42,
	0x00,0x00,0x00,0x22,0x00,0x00,0x00,0x00,
	};

static const BYTE		aFrame6[] =
	{
	0x28,0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x01,0x00,0x20,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x92,
	0x00,0x00,0x00,0x92,0x00,0x00,0x00,0x4E,0x00,0x00,0x00,0x00,0x47,0x47,0x47,0x68,
	0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x20,0x01,0x01,0x01,0x84,0x00,0x00,0x00,0x4C,
	0x46,0x46,0x46,0x69,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0C,
	0x00,0x00,0x00,0x91,0x45,0x45,0x45,0x6B,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,
	0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0xA6,0x44,0x44,0x44,0x6C,0x00,0x00,0x00,0x0E,
	0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0xA6,0x42,0x42,0x42,0x6C,
	0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0xA6,
	0x40,0x40,0x40,0x6E,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x11,
	0x00,0x00,0x00,0xA6,0x40,0x40,0x40,0x6F,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,0x3F,0x3F,0x3F,0x70,0x00,0x00,0x00,0x13,
	0x00,0x00,0x00,0x13,0x00,0x00,0x00,0x13,0x00,0x00,0x00,0xA6,0x3C,0x3C,0x3C,0x71,
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,
	0x3C,0x3C,0x3C,0x72,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0xA6,0x3B,0x3B,0x3B,0x73,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,0x3B,0x3B,0x3B,0x73,0x00,0x00,0x00,0x0D,
	0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA6,0x3B,0x3B,0x3B,0x73,
	0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA6,
	0x3A,0x3A,0x3A,0x76,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0D,
	0x00,0x00,0x00,0xA6,0x36,0x36,0x36,0x79,0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0x0F,
	0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0xA6,0x35,0x35,0x35,0x7B,0x00,0x00,0x00,0x12,
	0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0xA6,0x32,0x32,0x32,0x7F,
	0x00,0x00,0x00,0x16,0x00,0x00,0x00,0x16,0x00,0x00,0x00,0x16,0x00,0x00,0x00,0xA6,
	0x2F,0x2F,0x2F,0x82,0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0x1B,
	0x00,0x00,0x00,0xA6,0x2E,0x2E,0x2E,0x85,0x00,0x00,0x00,0x1F,0x00,0x00,0x00,0x1F,
	0x00,0x00,0x00,0x20,0x00,0x00,0x00,0xA6,0x2B,0x2B,0x2B,0x88,0x00,0x00,0x00,0x26,
	0x00,0x00,0x00,0x26,0x0B,0x0B,0x0B,0x2C,0x00,0x00,0x00,0xA6,0x2E,0x2E,0x2E,0x9B,
	0x1D,0x1D,0x1D,0x45,0x1D,0x1D,0x1D,0x45,0x2B,0x2B,0x2B,0x80,0x00,0x00,0x00,0x91,
	0x34,0x34,0x34,0xBF,0x2B,0x2B,0x2B,0x80,0x2B,0x2B,0x2B,0x80,0x0D,0x0D,0x0D,0xAA,
	0x00,0x00,0x00,0x4C,0x00,0x00,0x00,0xA6,0x00,0x00,0x00,0x8F,0x00,0x00,0x00,0x8F,
	0x00,0x00,0x00,0x4A,0x00,0x00,0x00,0x00,
	};





//*****************************************************************************
//*
//*		DibCreateFrame
//*
//*****************************************************************************
//	Erzeugt ein Rahmenbild aus einzelnen Teilen einer Vorlage.
//	pMemory		: Zeiger auf die Vorlage
//	uSizeX/Y	: Gre des zu erstellenden Bildes
//	pPixels		: Hier wird der Zeiger auf die Bilddaten des Rahmenbildes gespeichert
//	uColor		: Ist die Farbe fr den Hintergund (0xFFFFFFFF=keiner)
//	Ergibt den Zeiger aufs das Rahmenbild.
LPBITMAPINFO DibCreateFrame(LPCVOID pMemory,UINT uSizeX,UINT uSizeY,LPVOID pPixels,COLORREF uColor)
{
LPBITMAPINFO	pInfo;
LPBITMAPINFO	pTarget;
LPBYTE			pOutput;
LPBYTE			pInput;
unsigned		uAlpha1;
unsigned		uAlpha2;
unsigned		uBaseX;
unsigned		uBaseY;
unsigned		uFixX;
unsigned		uFixY;
unsigned		uSize;
unsigned		uColR;
unsigned		uColG;
unsigned		uColB;
unsigned		uFact;




	if(uColor==0xFFFFFFFF)								// Keine Hintergrundfarbe
		{
		pInfo	= GetDibOfMemory(NULL,(LPVOID)pMemory,0,&pInput);
		}
	else{												// Hintergrundfarbe einfgen
		pInfo	= GetDibOfMemory(NULL,(LPVOID)pMemory,DIB_CREATE_BITMAP,&pInput);
		uSize	= pInfo->bmiHeader.biWidth*pInfo->bmiHeader.biHeight*4;
		uColR	= (uColor>> 0)&0xFF;
		uColG	= (uColor>> 8)&0xFF;
		uColB	= (uColor>>16)&0xFF;

		while(uSize>0)
			{
			uSize  -= 4;
			uFact	= pInput[uSize+3];

			if(!uFact)continue;

			uAlpha1	= uFact+1;
			uAlpha2	= 257-uAlpha1;

			pInput[uSize  ]	= (pInput[uSize  ]*uAlpha1 + uColB*uAlpha2)/256;
			pInput[uSize+1]	= (pInput[uSize+1]*uAlpha1 + uColG*uAlpha2)/256;
			pInput[uSize+2]	= (pInput[uSize+2]*uAlpha1 + uColR*uAlpha2)/256;
			pInput[uSize+3]	= 0xFF;
			}
		}


		pTarget	= CreateDibIndirect(NULL,uSizeX,uSizeY,32,&pOutput);
	if(!pTarget)
		{
		DibFreeHandle(pInfo);
		return NULL;
		}

	if(pInfo->bmiHeader.biWidth<=6)
			uFixX = 2;
	else	uFixX = 3;

	if(pInfo->bmiHeader.biHeight<=6)
			uFixY = 2;
	else	uFixY = 3;

	if(uSizeX<uFixX*2)uFixX=uSizeX/2;
	if(uSizeY<uFixY*2)uFixY=uSizeY/2;



	uBaseX	= pInfo->bmiHeader.biWidth -uFixX*2;
	uBaseY	= pInfo->bmiHeader.biHeight-uFixY*2;


	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,0           ,0           ,uFixX ,uFixY ,0           ,0           ,uFixX         ,uFixY);
	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,uFixX       ,0           ,uBaseX,uFixY ,uFixX       ,0           ,uSizeX-uFixX*2,uFixY);
	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,uFixX+uBaseX,0           ,uFixX ,uFixY ,uSizeX-uFixX,0           ,uFixX         ,uFixY);

  	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,0           ,uFixY       ,uFixX ,uBaseY,0           ,uFixY       ,uFixX         ,uSizeY-uFixY*2);
	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,uFixX       ,uFixY       ,uBaseX,uBaseY,uFixX       ,uFixY       ,uSizeX-uFixX*2,uSizeY-uFixY*2);
	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,uFixX+uBaseX,uFixY       ,uFixX ,uBaseY,uSizeX-uFixX,uFixY       ,uFixX         ,uSizeY-uFixY*2);

	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,0           ,uFixY+uBaseY,uFixX ,uFixY ,0           ,uSizeY-uFixY,uFixX         ,uFixY);
	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,uFixX       ,uFixY+uBaseY,uBaseX,uFixY ,uFixX       ,uSizeY-uFixY,uSizeX-uFixX*2,uFixY);
	DibConvertSizeEx(pTarget,pOutput,pInfo,pInput,uFixX+uBaseX,uFixY+uBaseY,uFixX ,uFixY ,uSizeX-uFixX,uSizeY-uFixY,uFixX         ,uFixY);

	*(LPBYTE*)pPixels = pOutput;

	DibFreeHandle(pInfo);


return pTarget;
}

//*****************************************************************************
//*
//*		DrawFullButtonEx
//*
//*****************************************************************************
//	Zeichnet ein Rahmenbild
//	pInfo		: Zeiger aus das Zielbild worauf gezeichnet wird.
//	pPixels		: Pixeldaten des Zielbildes.
//	iPosX/Y		: Zeichenposition im Zielbild.
//	iSizeX/Y	: Gre des Rahmenbildes
//	uFlags		: Diverse Anzeigeoptionen
//					Bit0 = Taste gedrckt
//					Bit1 = hervorgehoben
//					Bit2 = ausgeraut
//					Bit3 = Dropdown links
//					Bit4 = Dropdown links
//	uColor		: Farbe fr den Hintergund wenn hervorgehoben.
void DrawFullButtonEx(LPBITMAPINFO pInfo,LPVOID pPixels,INT iPosX,INT iPosY,INT iSizeX,int iSizeY,UINT uFlags,COLORREF uHotColor)
{
static LPCVOID	aFrames[]={aFrame1,aFrame2,aFrame3,aFrame4,aFrame5,aFrame6};
LPBYTE			pInput;
LPBITMAPINFO	pTemp;
unsigned		uPos;


	uPos = (uFlags&1)|((uFlags>>2)&6);

		pTemp = DibCreateFrame(aFrames[uPos],iSizeX,iSizeY,&pInput,(uFlags&2)? uHotColor:-1);
	if(!pTemp)return;

  	DibPutAlpha(pInfo,pPixels,iPosX,iPosY,pTemp,pInput);

	DibFreeHandle(pTemp);


}

//*****************************************************************************
//*
//*		GetControlSize
//*
//*****************************************************************************
//	Holt die Gre eine Controls
static BOOL GetControlSize(HWND hWnd,int *pSizeX,int *pSizeY)
{
WCHAR	cName[32];
RECT	sRect;
RECT	sItem;
int		iRet;



	if(!GetWindowRect(hWnd,&sRect))						// Position des Controls holen
		{
		*pSizeX = 8;
		*pSizeY = 8;
		return  FALSE;
		}

	*pSizeX = sRect.right -sRect.left;
	*pSizeY = sRect.bottom-sRect.top;

	   iRet = (int)SendMessage(hWnd,CB_GETLOCALE,0,0);
	if(iRet<16)return TRUE;								// Ist es eine Combobox

	   iRet = GetClassNameW(hWnd,cName,32);
	if(iRet!=8 || memicmp(cName,L"ComboBox",8*sizeof(WCHAR)))
		{
		return TRUE;
		}

	if(!GetWindowRect(hWnd,&sItem))						// Abmessungen der CB holen
		{
		return TRUE;
		}

	iRet	= sItem.top-sRect.top;

	if(iRet>0 && iRet<8)								// Rahmen adieren
			iRet *= 2;
	else	iRet  = 0;


	*pSizeY = sItem.bottom-sItem.top+iRet;


return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarExRegister
//*
//*****************************************************************************
//	Registiert das Toolbar Fenster.
//	Ergibt 1 wenn das Fenster erfolgreich registiert wurde.
int ToolbarExRegister(HINSTANCE hInstance)
{
static int		iIsRegistered=FALSE;
WNDCLASSEX		sClass;


    if(iIsRegistered)return TRUE;


    memset(&sClass,0,sizeof(sClass));

    sClass.cbSize           = sizeof(sClass);
    sClass.style            = CS_DBLCLKS;
    sClass.lpfnWndProc      = ToolbarProc;
    sClass.cbClsExtra       = 0;
    sClass.cbWndExtra       = sizeof(ToolbarData*);
    sClass.hInstance        = hInstance;
    sClass.hIcon            = NULL;
    sClass.hCursor          = LoadCursor(NULL,IDC_ARROW);
    sClass.hbrBackground	= GetSysColorBrush(COLOR_3DFACE);
    sClass.lpszMenuName     = NULL;
    sClass.hIconSm          = NULL;
    sClass.lpszClassName	= _T("ToolbarExWindow32");

    if(!RegisterClassEx(&sClass))return FALSE;

    iIsRegistered = TRUE;


return TRUE;
}

//*****************************************************************************
//*
//*		GlobalInit
//*
//*****************************************************************************
static void GlobalInit()
{
LOGBRUSH	sLog;
long		lCount;


	   lCount = InterlockedIncrement(&lWindowCount);
	if(lCount>0)return;

	sLog.lbHatch = 0;
	sLog.lbStyle = PS_SOLID;
	sLog.lbColor = GetSysColor(COLOR_BTNSHADOW);
	hCursorArrow = LoadCursor(NULL,IDC_ARROW);
	hCursorMove	 = LoadCursor(NULL,IDC_HAND);
	hCursorNo	 = LoadCursor(NULL,IDC_NO);

	if(!hUxThemeDll)
		{
		   hUxThemeDll = LoadLibrary(_T("UxTheme.dll"));
		if(hUxThemeDll)
			{
			pSetWindowTheme  = (SetWindowThemeT )GetProcAddress(hUxThemeDll,"SetWindowTheme"     );
			pEndBufferedPt   = (EndBufferedPtT  )GetProcAddress(hUxThemeDll,"EndBufferedPaint"   );
			pBeginBufferedPt = (BeginBufferedPnT)GetProcAddress(hUxThemeDll,"BeginBufferedPaint" );
			pBufferedPtInit	 = (BufferedPtInitT	)GetProcAddress(hUxThemeDll,"BufferedPaintInit"  );
			pBufferedPtExit  = (BufferedPtInitT	)GetProcAddress(hUxThemeDll,"BufferedPaintUnInit");
			pOpenThemeData	 = (OpenThemeDataT	)GetProcAddress(hUxThemeDll,"OpenThemeData"		 );
			pCloseThemeData	 = (CloseThemeDataT	)GetProcAddress(hUxThemeDll,"CloseThemeData"	 );
			pDrawThemeBackg	 = (DrawThemeBackgT )GetProcAddress(hUxThemeDll,"DrawThemeBackground");
			pGetThemePartSz  = (GetThemePartSzT	)GetProcAddress(hUxThemeDll,"GetThemePartSize"   );
			pGetThemeBackgRc = (GetThemeBackgRcT)GetProcAddress(hUxThemeDll,"GetThemeBackgroundContentRect");
			}
		}

	if(pBufferedPtInit)
		{
		pBufferedPtInit();
		}

	if(!uColorBits)
		{
		hDcMain		= GetDC(NULL);
		uColorBits	= GetDeviceCaps(hDcMain,BITSPIXEL);
		uButtonBits = uColorBits;
		if(uButtonBits>=12)uButtonBits=32;
		}


}


//*****************************************************************************
//*
//*		GlobalDeinit
//*
//*****************************************************************************
static void GlobalDeinit()
{
int	lCount;


	   lCount=InterlockedDecrement(&lWindowCount);
	if(lCount>=0)return;

    if(hDefaultFontN)
        {
        DeleteObject(hDefaultFontN);
        hDefaultFontN = NULL;
        }

    if(hDefaultFontB)
        {
        DeleteObject(hDefaultFontB);
        hDefaultFontB = NULL;
        }

	if(pBufferedPtExit)
		{
		pBufferedPtExit();
		}

	if(uColorBits)
		{
		ReleaseDC(NULL,hDcMain);
		uColorBits	= 0;
		uButtonBits = 0;
		}


}

//*****************************************************************************
//*
//*		SendNotify
//*
//*****************************************************************************
//	Sendet eine WM_NOTIFY Nachricht an das Elternfenster
//	pData		: Zeiger auf die Fensterdaten
//	pNotify		: Zeiger auf die Notify-Daten
//	Ergibt den Rckgabewert der WM_NOTIFY Nachrich
static LRESULT SendNotify(ToolbarData *pData,NMHDR *pNotify)
{


	pNotify->hwndFrom	= pData->hWnd;
    pNotify->idFrom		= GetWindowLong(pNotify->hwndFrom,GWL_ID);


return SendMessage(GetParent(pNotify->hwndFrom),WM_NOTIFY,pNotify->idFrom,(LPARAM)pNotify);
}

//*****************************************************************************
//*
//*		UpdateColors
//*
//*****************************************************************************
//	Aktualisiert alle Farben
//	pData		: Zeiger auf die Fensterdaten
static void UpdateColors(ToolbarData *pData)
{

	if(!pData->aColorChanged[TBC_GROUP   ])pData->aColors[TBC_GROUP   ]  = GetSysColor(COLOR_MENUTEXT);
	if(!pData->aColorChanged[TBC_TEXT    ])pData->aColors[TBC_TEXT    ]  = GetSysColor(COLOR_MENUTEXT);
	if(!pData->aColorChanged[TBC_HOT     ])pData->aColors[TBC_HOT     ]  = GetSysColor(COLOR_3DLIGHT);
	if(!pData->aColorChanged[TBC_BK      ])pData->aColors[TBC_BK      ]  = GetSysColor(COLOR_3DFACE);
	if(!pData->aColorChanged[TBC_HOTEX   ])pData->aColors[TBC_HOTEX   ]  = RGB(255,192,128);
	if(!pData->aColorChanged[TBC_MARK    ])pData->aColors[TBC_MARK    ]  = RGB(0,0,0);
	if(!pData->aColorChanged[TBC_TAGBK   ])pData->aColors[TBC_TAGBK   ]  = GetSysColor(COLOR_INFOBK);
	if(!pData->aColorChanged[TBC_TAGTEXT ])pData->aColors[TBC_TAGTEXT ]  = GetSysColor(COLOR_INFOTEXT);
	if(!pData->aColorChanged[TBC_TAGFRAME])pData->aColors[TBC_TAGFRAME]  = ((pData->aColors[TBC_TAGTEXT]&0xFEFEFE)>>1)+((pData->aColors[TBC_TAGBK]&0xFEFEFE)>>1);

}

//*****************************************************************************
//*
//*		UpdateMark
//*
//*****************************************************************************
//	Setzt einen neue Marke
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist die Nummer der Taste mit der Marke
//	uMode		: Modus fr die Marke (Bit0=nachher)
//	Ergibt 1 wenn die Marke sichtbar ist
static int UpdateMark(ToolbarData *pData,unsigned uIndex,unsigned uMode)
{
LPTBITEM		pItem;
RECT			sRect;
BOOL			bFlag;


	if(uIndex>=pData->uItemsCnt)								// Make ausblenden
		{
		if(pData->uMarkItem!=0xFFFFFFFF)
			{
			InvalidateRect(pData->hWnd,&pData->sMarkRect,FALSE);
			}

		SetRectEmpty(&pData->sMarkRect);
		pData->uMarkItem = 0xFFFFFFFF;
		pData->uMarkMode = 0;

		return 0;
		}


	sRect = pData->sMarkRect;
    pItem = pData->pItems[uIndex];

	if(pItem->fsState&TBSTATE_HIDDEN)							// Ist die Taste versteckt
		{
		if(pData->uMarkItem!=0xFFFFFFFF)
			{
			InvalidateRect(pData->hWnd,&pData->sMarkRect,FALSE);
			}

		SetRectEmpty(&pData->sMarkRect);
		pData->uMarkItem = 0xFFFFFFFF;
		pData->uMarkMode = 0;

		return 0;
		}

	if(pData->uMarkMode&0x01)									// Marke nacher zeichnen
		{
		if(pItem->wInfoFlags&TBIF_BASE)
			{
			pData->sMarkRect.left	= pItem->sRect.left;
			pData->sMarkRect.right	= pItem->sRect.right;
			pData->sMarkRect.top	= pItem->sRect.top-3;
			pData->sMarkRect.bottom	= pItem->sRect.top+3;
			uMode				   |= 2;
			}
		else{
			pData->sMarkRect.left	= pItem->sRect.right-3;
			pData->sMarkRect.right	= pItem->sRect.right+3;
			pData->sMarkRect.top	= pItem->sRect.top;
			pData->sMarkRect.bottom	= pItem->sRect.bottom;
			uMode				   &= ~2;
			}
		}
	else{														// Marke vorher zeichnen
		if(pItem->wInfoFlags&TBIF_PACKED)
			{
			pData->sMarkRect.left	= pItem->sRect.left;
			pData->sMarkRect.right	= pItem->sRect.right;
			pData->sMarkRect.top	= pItem->sRect.bottom-3;
			pData->sMarkRect.bottom	= pItem->sRect.bottom+3;
			uMode				   |= 2;
			}
		else{
			pData->sMarkRect.left	= pItem->sRect.left-3;
			pData->sMarkRect.right	= pItem->sRect.left+3;
			pData->sMarkRect.top	= pItem->sRect.top;
			pData->sMarkRect.bottom	= pItem->sRect.bottom;
			uMode				   &= ~2;
			}
		}


	if(!(uMode&2))
		{
		while(pItem->wInfoFlags&TBIF_BASE)						// Taste ist unter eine andere gestellt
			{
			uIndex++;
			if(uIndex>=pData->uItemsCnt)break;
			pItem = pData->pItems[uIndex];
			if(pItem->fsState&TBSTATE_HIDDEN)continue;
			pData->sMarkRect.top	= pItem->sRect.top;
			}

		while(pItem->wInfoFlags&TBIF_PACKED)					// Taste ist ber eine andere gestellt
			{
			if(uIndex==0)break;
			uIndex--;
			pItem = pData->pItems[uIndex];
			if(pItem->fsState&TBSTATE_HIDDEN)continue;
			pData->sMarkRect.bottom	= pItem->sRect.bottom;
			}
																// Gibt es unten einen Text
		if(pItem->wInfoFlags&(TBIF_TEXT_BOTTOM|TBIF_TEXT_EMPTY|TBIF_TEXT_GROUP))
			{
			pData->sMarkRect.bottom	-= pData->iFontHeight;
			pData->sMarkRect.bottom	-= TEXTOFF_Y;
			}

		if(pData->sMarkRect.left<0)
			{
			pData->sMarkRect.left ++;
			pData->sMarkRect.right++;
			}
		}

	bFlag = EqualRect(&sRect,&pData->sMarkRect);

	if(pData->uMarkItem!=0xFFFFFFFF && bFlag)					// Hat sich das Rechteck verndert
		{
		InvalidateRect(pData->hWnd,&sRect,FALSE);
		}

	if(bFlag)
		{
		InvalidateRect(pData->hWnd,&pData->sMarkRect,FALSE);
		}

	pData->uMarkItem = uIndex;
	pData->uMarkMode = uMode;


return 1;
}

//*****************************************************************************
//*
//*		UpdateRect
//*
//*****************************************************************************
//	Zeichnet einen Eintrag neu
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist die Nummer der Taste
//	Ergibt 1 wenn der Eintrag sichtbar war
static int UpdateRect(ToolbarData *pData,unsigned uIndex)
{
LPTBITEM		pItem;
unsigned		uValue;
unsigned		uFlags;


	if(uIndex>=pData->uItemsCnt)
		{
		return 0;
		}

	pItem = pData->pItems[uIndex];

	if(pItem->fsState&TBSTATE_HIDDEN)
		{
		return 0;
		}

	pItem->wInfoFlags  &= ~0x07;
	pData->uDropState	=  0;
	uValue				= pItem->fsState&(TBSTATE_CHECKED|TBSTATE_PRESSED);


	if(pData->uMoveItem!=uIndex)
		{
		if((pItem->fsState&TBSTATE_INDETERMINATE) || !(pItem->fsState&TBSTATE_ENABLED))
			{
			pItem->wInfoFlags |= (uValue)? 5:4;				// Taste gesperrt
			}
		else{
			pItem->wInfoFlags |= (uValue)? 1:0;				// Normal
			}
		}
	else{
		if(pItem->wFlags&(BTFL_CHECKBOX|BTFL_RADIO))
				uFlags = 0;
		else if(pItem->wFlags&BTFL_RIGHT_CLICK)
				uFlags = pData->cMoveFlags&MK_LBUTTON|MK_RBUTTON;
		else	uFlags = pData->cMoveFlags&MK_LBUTTON;

		if((pItem->fsState&TBSTATE_INDETERMINATE) || !(pItem->fsState&TBSTATE_ENABLED))
			{
			pItem->wInfoFlags |= (uValue)? 5:4;				// Taste gesperrt
			}
		else if(pData->cMoveDrop)
			{
			pData->uDropState  = (uFlags&MK_LBUTTON);		// ber Dropdown-Taste
			pItem->wInfoFlags |= (uValue)? 7:6;
			}
		else{
			pItem->wInfoFlags |= (uValue || uFlags)? 3:2;	// Hervorgehoben
			}
		}


	InvalidateRect(pData->hWnd,&pItem->sRect,FALSE);



return 1;
}

//*****************************************************************************
//*
//*		UpdateFrom
//*
//*****************************************************************************
//	Zeichnet alles ab einem Eintrag neu
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist die Nummer der Taste
//	Ergibt 1 wenn der Eintrag sichtbar war
static int UpdateFrom(ToolbarData *pData,unsigned uIndex)
{
LPTBITEM		pItem;
RECT			sRect;



	if(uIndex>=pData->uItemsCnt)
		{
		return 0;
		}

	pItem = pData->pItems[uIndex];

	if(pItem->fsState&TBSTATE_HIDDEN)
		{
		return 0;
		}

	sRect.top		= pItem->sRect.top;
	sRect.left		= pItem->sRect.left;
	sRect.bottom	= pItem->sRect.bottom;
	sRect.right		= pData->uSizeX;

	InvalidateRect(pData->hWnd,&sRect,FALSE);

	if(U(sRect.bottom)<pData->uSizeY)
		{
		sRect.left		= 0;
		sRect.top		= sRect.bottom;
		sRect.bottom	= pData->uSizeY;

		InvalidateRect(pData->hWnd,&sRect,FALSE);
		}


return 1;
}

//*****************************************************************************
//*
//*		UpdateView
//*
//*****************************************************************************
//	Zeichnet das ganze Fenster neu
//	pData		: Zeiger auf die Fensterdaten
//	Ergibt 1 wenn der Eintrag sichtbar war
static void UpdateView(ToolbarData *pData)
{
RECT	sRect;


	GetClientRect (pData->hWnd,&sRect);
	InvalidateRect(pData->hWnd,&sRect,FALSE);

}


//*****************************************************************************
//*
//*		UpdateFont
//*
//*****************************************************************************
//	Erzeugt einen den fetten Font fr das Fenster
//	pData		: Zeiger auf die Fensterdaten
//	iRedraw		: Soll das Fenster neugezeichnet werden
//	Ergibt 1 wenn der Font verndert wurde
static int UpdateFont(ToolbarData *pData)
{
TEXTMETRIC	sMetrics;
LOGFONT		sLog;
int			iVal;
int			iRet;
HDC			hDc;


	if(!hDefaultFontN)										// Den Standard-Font erzeugen
		{
		SystemParametersInfo(SPI_GETICONTITLELOGFONT,sizeof(sLog),&sLog,0);
		sLog.lfHeight=(sLog.lfHeight*3)/4;
		if(sLog.lfHeight<0 && sLog.lfHeight>-12)sLog.lfHeight=-12;
		if(sLog.lfHeight>0 && sLog.lfHeight< 12)sLog.lfHeight= 12;
		sLog.lfWeight=FW_NORMAL;hDefaultFontN=CreateFontIndirect(&sLog);
		sLog.lfWeight=FW_BOLD  ;hDefaultFontB=CreateFontIndirect(&sLog);
		}


	if(pData->hFontN==NULL)									// Standard-Font benutzen
		{
		pData->hFontN=hDefaultFontN;
		}


	if(pData->hFontN!=pData->hFontL)						// Wurde ein neuer Font definiert
		{
		pData->hFontL=pData->hFontN;

		hDc = GetDC(NULL);
		SelectObject(hDc,pData->hFontN);
		GetTextMetrics(hDc,&sMetrics);
		pData->iFontHeight =  sMetrics.tmHeight;
		pData->iFontLine   =  sMetrics.tmAscent+1;
		pData->iFontOff    = (sMetrics.tmPitchAndFamily&TMPF_FIXED_PITCH)? 0:-1;
		if(pData->hFontN!=hDefaultFontN)GetTextFace(hDc,LF_FACESIZE,sLog.lfFaceName);
		ReleaseDC(NULL,hDc);

		iRet  = 1;
		}
	else{
		iRet  = 0;
		}


	iVal = CalcTextSizeW(NULL,L"H",1,DT_SINGLELINE|DT_RETURNY,pData->hFontN);
	pData->uFontHeigh=HIWORD(iVal);


return iRet;
}

//*****************************************************************************
//*
//*		ToolbarButtonIndex
//*
//*****************************************************************************
//	Such den Index zu einer Tasten-ID
//	uCommandId	: Ist die ID-Der Taste, oder ein spezelles HI-Wort codiert
//					HI-Wort 0x1234 = Das LO-Wort ist der Index
//					HI-Wort 0x1248 = Das LO-Wort ist die ID
//					HI-Wort 0x128? = Das LO-Wort ist die ID (Bit 16..20 Entry)
//	Ergibt den Index oder 0xFFFFFFFF wenn die ID nicht gefunden wurde
static unsigned ToolbarButtonIndex(ToolbarData *pData,unsigned uCommandId)
{
unsigned		uTurn=0;
unsigned		uIndex;
LPTBITEM		pItem;


	     if(HIWORD(uCommandId)==0x1234)return     LOWORD(uCommandId);
	else if(HIWORD(uCommandId)==0x1248)uCommandId=LOWORD(uCommandId);
	else if(MHWORD(uCommandId)==0x1280)
		{
		uTurn		= MLWORD(uCommandId);
		uCommandId	= LOWORD(uCommandId);

		if((uCommandId&0x0FFF)==0x0E7E)						// Suche nach Flags
			{
			uCommandId &= 0xF000;

			for(uIndex=0;uIndex<pData->uItemsCnt;uIndex++)
				{
				pItem	= pData->pItems[uIndex];

				if(pItem->wFlags&uCommandId)
					{
					if(!uTurn)return uIndex;
					uTurn--;
					}
				}

			return 0xFFFFFFFF;
			}
		}

	for(uIndex=0;uIndex<pData->uItemsCnt;uIndex++)			// Such nach einer ID
		{
		pItem	= pData->pItems[uIndex];

		if(U(pItem->idCommand)==uCommandId)
			{
			if(!uTurn)return uIndex;
			uTurn--;
			}
		}



return 0xFFFFFFFF;
}

//*****************************************************************************
//*
//*		ToolbarSetButtonSize
//*
//*****************************************************************************
//	Stellt die Gre einer Taste ein. Das ist minimal die Gre des Bitmaps
//	samt Rahmen (iPaddingX/Y)
static unsigned ToolbarSetButtonSize(ToolbarData *pData,int iSizeX,int iSizeY)
{
int		iValX;
int		iValY;


	if(iSizeX<0 || iSizeY<0)
		{
		return 0;
		}

    if(iSizeX==0)iSizeX=24;
    if(iSizeY==0)iSizeY=24;

	iValX = pData->iPaddingX+pData->iButtonSizeX;
	iValY = pData->iPaddingY+pData->iButtonSizeY;

	if(iSizeX<iValX)iSizeX=iValX;
	if(iSizeY<iValY)iSizeY=iValY;

	if(pData->iButtonSizeX!=iSizeX || pData->iButtonSizeY!=iSizeY)
		{
		pData->iButtonSizeX = iSizeX;
		pData->iButtonSizeY = iSizeY;

		pData->cReCalc = 0x07;
		UpdateView(pData);
		}


return 1;
}

//*****************************************************************************
//*
//*		ToolbarSetWidth
//*
//*****************************************************************************
//	Stellt das Padding rund um das Bild ein
static unsigned ToolbarSetWidth(ToolbarData *pData,int iMinX,int iMaxX)
{


	if(iMinX<0)iMinX=-1;
	if(iMaxX<0)iMaxX=-1;

	if(pData->iMinWidthX!=iMinX || pData->iMaxWidthX!=iMaxX)
		{
		pData->iMinWidthX = iMinX;
		pData->iMaxWidthX = iMaxX;

		pData->cReCalc = 0x07;
		UpdateView(pData);
		}


return 1;
}

//*****************************************************************************
//*
//*		ToolbarListGap
//*
//*****************************************************************************
//	Stellt den Abstand zwischen den Tasten ein
static unsigned ToolbarListGap(ToolbarData *pData,int iGabX,int iGabY)
{

	if(HIWORD(iGabY)==0x1248)
		{
		iGabY =	GET_X_LPARAM(iGabY);
		}
	else{
		iGabY =	pData->iGabY;
		}

	if(iGabX<0)iGabX=0;
	if(iGabY<0)iGabY=0;

	if(pData->iGabX!=iGabX || iGabY!=pData->iGabY)
		{
		pData->iGabX	= iGabX;
		pData->iGabY	= iGabY;
		pData->cReCalc  = 0x07;
		UpdateView(pData);
		}


return 0;
}

//*****************************************************************************
//*
//*		ToolbarSetBitmapSize
//*
//*****************************************************************************
//	Stellt die minimal Gre der Bilder ein
static unsigned ToolbarSetBitmapSize(ToolbarData *pData,int iSizeX,int iSizeY)
{


	if(iSizeX<2)iSizeX=2;
	if(iSizeY<2)iSizeY=2;

	if(pData->iBitmapSizeX!=iSizeX || pData->iBitmapSizeY!=iSizeY)
		{
		pData->iBitmapSizeX = iSizeX;
		pData->iBitmapSizeY = iSizeY;
		pData->cReCalc     |= 0x03;
		UpdateView(pData);
		}


return TRUE;
}


//*****************************************************************************
//*
//*		ToolbarSetPadding
//*
//*****************************************************************************
//	Stellt das Padding rund um das Bild ein
static unsigned ToolbarSetPadding(ToolbarData *pData,int iPaddingX,int iPaddingY)
{


	if(iPaddingX<2)iPaddingX=2;
	if(iPaddingY<2)iPaddingY=2;

	if(pData->iPaddingX!=iPaddingX || pData->iPaddingY!=iPaddingY)
		{
		pData->iPaddingX = iPaddingX;
		pData->iPaddingY = iPaddingY;
		pData->cReCalc   = 0x07;
		UpdateView(pData);
		}


return MAKELONG(pData->iPaddingX,pData->iPaddingY);
}

//*****************************************************************************
//*
//*		ToolbarGetPixel32
//*
//*****************************************************************************
static DWORD __fastcall ToolbarGetPixel32(LPBYTE pRow,UINT uCol)
{
return *(LPDWORD)(pRow+uCol*4);
}

//*****************************************************************************
//*
//*		ToolbarGetPixel24
//*
//*****************************************************************************
static DWORD __fastcall ToolbarGetPixel24(LPBYTE pRow,UINT uCol)
{
return *(LPDWORD)(pRow+uCol*3)&0x00FFFFFF;
}

//*****************************************************************************
//*
//*		ToolbarGetPixel8
//*
//*****************************************************************************
static DWORD __fastcall ToolbarGetPixel8(LPBYTE pRow,UINT uCol)
{
return pRow[uCol];
}

//*****************************************************************************
//*
//*		ToolbarGetPixel4
//*
//*****************************************************************************
static DWORD __fastcall ToolbarGetPixel4(LPBYTE pRow,UINT uCol)
{
unsigned	uBits;

	uBits = pRow[uCol/2];

return (uBits>>(4*(1-(uCol&1))))&0x0F;
}

//*****************************************************************************
//*
//*		ToolbarGetPicPos
//*
//*****************************************************************************
//	Sucht eine Bildposition in einer variablen Resource.
//	pResource		: Ist die Bild-Resource die gesucht wird.
//	uPicNum			: ist die Nummer des Bildes das gesucht wird.
//	Ergibt TRUE wenn das Unterbild gefunden wurde.
static BOOL ToolbarGetPicPos(LPTBRESOURCE pResource,unsigned uPicNum)
{
LPBYTE			pPixels;
DWORD			uColor;
LPBITMAPINFO	pInfo;
GetPixelT		pProc;
unsigned		uPosZ;
unsigned		uPosY;
unsigned		uPosX;
unsigned		uPos;
unsigned		uOff;



	pPixels	= pResource->pPixels;
	pInfo	= pResource->pInfo;

															// Links-Unten beginnen
	if(uPicNum>pResource->wLastPos || pResource->wLastPos>pResource->wCount)
		{
		pResource->wLastPos = (WORD)0;
		pResource->wSizeX	= (WORD)0;
		pResource->wSizeY	= (WORD)0;
		pResource->wLastX	= (WORD)0;
		pResource->wLastY	= (WORD)(pInfo->bmiHeader.biHeight-1);
		}



	switch(pInfo->bmiHeader.biBitCount)						// Pixel-Funktion bestimmen
		{
	case  4:pProc=ToolbarGetPixel4;		break;
	case  8:pProc=ToolbarGetPixel8;		break;
	case 24:pProc=ToolbarGetPixel24;	break;
	default:pProc=ToolbarGetPixel32;	break;
		}

	uColor	= pProc(pPixels,0);
	uPosX	= pResource->wLastX;
	uPosY	= pInfo->bmiHeader.biHeight-pResource->wLastY-1;
	uPos	= pResource->wLastPos;

	for(;;)
		{
		if(pResource->wSizeY)								// Vor auf das nchste Bild
			{
			if(uPos>=uPicNum)break;

			uPosY  += pResource->wSizeY;
			uOff	= pResource->wRowWidth*(uPosY+1);
															// Springe zur nchsten X-Spalte
			if(uPosY+3>U(pInfo->bmiHeader.biHeight) || pProc(pPixels+uOff,uPosX)==uColor)
				{
				pResource->wFlags &= ~TBRF_SHIFT;
				uPosX += pResource->wSizeX;
				uPosY  = 0;
				}

			uPos++;
			}

		if(uPosY==0)										// Breite in X-Richtung berechnen
			{
			if(uPosX>=U(pInfo->bmiHeader.biWidth))
				{
				pResource->wFlags  |= TBRF_SCANNED;
				pResource->wCount	= uPos;
				pResource->wLastPos	= uPos;
				return FALSE;
				}

			for(uPosZ=uPosX+1;uPosZ<U(pInfo->bmiHeader.biWidth);uPosZ++)
				{
				if(pProc(pPixels,uPosZ)==uColor)break;
				}

			pResource->wSizeX = (WORD)(uPosZ-uPosX);
			}

															// In Y-Richtung suchen
		if(uPosY || pProc(pPixels+pResource->wRowWidth,uPosX)==uColor)
			{
			if(uPosY==0 && U(pInfo->bmiHeader.biWidth-1)==uPosX)
				{
				pResource->wSizeY = 0;						// Dummy Spalte am Ende
				uPosX  += 1;
				continue;
				}

			uOff = pResource->wRowWidth*2;

			if(uPosY==0 && pProc(pPixels+uOff,uPosX)==uColor)
				{
				pResource->wFlags |= TBRF_SHIFT;			// Die erste Spalte gehrt nicht zum Bild
				}

			uPosZ = uPosY+2;
			uOff  = pResource->wRowWidth*uPosZ;

			for(;uPosZ<U(pInfo->bmiHeader.biHeight);uPosZ++)
				{
				if(pProc(pPixels+uOff,uPosX)==uColor)break;
				uOff += pResource->wRowWidth;
				}

			pResource->wSizeY = (WORD)(uPosZ-uPosY);
			}
		else{
			pResource->wSizeY = (WORD)(pInfo->bmiHeader.biHeight);
			}
		}

	pResource->wLastPos	= (WORD)uPos;						// Bild wurde gefunden
	pResource->wLastX	= (WORD)uPosX;
	pResource->wLastY	= (WORD)uPosY;

	if(pResource->wLastPos>pResource->wCount)
		{
		pResource->wCount = pResource->wLastPos;
		}



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarDelStrings
//*
//*****************************************************************************
//	Mehrere Texte aus der Textliste lschen.
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Zeiger auf die Textliste
//	uCount		: Ist es eine Unicode Liste
//	Ergibt die Anzahl der gelschten Texte
static int ToolbarDelStrings(ToolbarData *pData,UINT uIndex,UINT uCount)
{
LPTBITEM	pItem;
unsigned	uPos;
int			iEnd;


	if(uIndex>=pData->uStringCnt || uCount==0)				// Prfe Textnummer
		{
		return 0;
		}

	if(uIndex+uCount>pData->uStringCnt)						// Prfe die Anzahl
		{
		uCount = pData->uStringCnt-uIndex;
		}

	for(uPos=0;uPos<uCount;uPos++)							// Alle Texte freigeben
		{
		delete pData->pStrings[uIndex+uPos];
		}

	pData->uStringCnt -= uCount;

	for(uPos=uIndex+uCount;uPos<pData->uStringCnt;uPos++)	// Texte nachrcken
		{
		pData->pStrings[uPos] = pData->pStrings[uPos+uCount];
		}

	iEnd = uIndex+uCount;

	for(uPos=0;uPos<pData->uItemsCnt;uPos++)				// Verwenden Tasten die Texte
		{
		   pItem = pData->pItems[uPos];
		if(pItem->iString>=iEnd)
			{
			pItem->iString -= uCount;
			}
		else if(pItem->iString>=(INT)uIndex)
			{
			pItem->wInfoFlags |= TBIF_REASSEMBLE;
			pData->cReCalc    |= 0x03;
			UpdateView(pData);
			}
		}



return uCount;
}

//*****************************************************************************
//*
//*		ToolbarAddStrings
//*
//*****************************************************************************
//	Fgt Texte aus einer Textliste ein.
//	pData		: Zeiger auf die Fensterdaten
//	lParam		: Zeiger auf die Textliste
//	bUnicode	: Ist es eine Unicode Liste
//	iSeparator	: Ist das Trennzeichen fr die Texte
//					0xFFFE = Einzelnet Text
//					0xFFFF = Erstes Zeichen	(TB_FROMLIST)
//	Ergibt den erstern String-Index oder -1 bei einem Fehler
static int ToolbarAddStrings(ToolbarData *pData,LPARAM lParam,INT iSeparator,BOOL bUnicode)
{
LPCWSTR		pWide=(LPCWSTR)lParam;
LPCSTR		pAnsi=(LPCSTR )lParam;
INT			iRet=pData->uStringCnt;
LPWSTR     *pListNew;
LPWSTR      pTextNew;
INT			iLen;
INT			iNum;
INT			iMax;



	if(iSeparator==TB_FROMLIST)								// Separator ist der Erste STring
		{
		if(bUnicode)
			{
			iSeparator = *pWide;
			pWide++;
			}
		else{
			iSeparator = *pAnsi;
			pAnsi++;
			}
		}
	else if(iSeparator==TB_SINGLETEXT)						// Einzelner Text
		{
		iMax		= 0;
		iSeparator	= 0;
		}
	else{													// Text Liste
		iMax = 0x8000;
		}


	for(;iMax>=0;iMax--)									// Alle Texte in der Liste durchlaufen
		{
		if(bUnicode)
			{
			if(!*pWide && iMax)break;
			for(iLen=0;(WORD)pWide[iLen]!=(WORD)iSeparator;iLen++);
			}
		else{
			if(!*pAnsi && iMax)break;
			for(iLen=0;(BYTE)pAnsi[iLen]!=(BYTE)iSeparator;iLen++);
			}

		if(iLen>256)return -1;


		if(pData->uStringCnt>=pData->uStringMax)			// Die Liste vergrern
			{
			pData->uStringMax  += 32;
			pListNew			= new LPWSTR[pData->uStringMax];
			if(!pData->uStringMax)return -1;

			if(pData->pStrings)
				{
				memcpy(pListNew,pData->pStrings,sizeof(LPCWSTR)*pData->uStringCnt);
				delete pData->pStrings;
				}

			pData->pStrings = pListNew;
			}

			pTextNew = new WCHAR[iLen+4];
		if(!pTextNew)return -1;

		pTextNew[0] = iLen;

		if(bUnicode)										// Text kopieren
			{
			memcpy(pTextNew+1,pWide,iLen*sizeof(WCHAR));
			pTextNew[iLen]=0;
			pWide += iLen+1;
			}
		else{
			iNum   = MultiByteToWideChar(CP_ACP,0,pAnsi,iLen,pTextNew+1,iLen+3);
			pTextNew[iNum+1]=0;
			pAnsi += iLen+1;
			}

		pData->pStrings[pData->uStringCnt] = pTextNew;
		pData->uStringCnt++;
		}


return iRet;
}

//*****************************************************************************
//*
//*		ToolbarAddStringRes
//*
//*****************************************************************************
//	Fgt Texte aus einer Recource ein. Der erste Buschstabe der Recource
//	ist das Trennzeichen.
//	pData		: Zeiger auf die Fensterdaten
//	lParam		: Zeiger auf die Textliste
//	bUnicode	: Ist es eine Unicode Liste
//	Ergibt den erstern String-Index oder -1 bei einem Fehler
static int ToolbarAddStringRes(ToolbarData *pData,HINSTANCE hInst,UINT uRes)
{
LPWSTR	pText;
HRSRC	hRsrc;
WCHAR	wChar;
UINT	uSize;
UINT	uPos;
INT		iRet;


	uPos	= (uRes>>4)+1;
	hRsrc	= FindResourceW(hInst,MAKEINTRESOURCEW(uPos),(LPWSTR)RT_STRING);

	if(!hRsrc)return -1;

	uSize	= SizeofResource(hInst,hRsrc);
	pText	= new WCHAR[uSize+4];

	if(!pText)return -1;

	uSize	= LoadStringW(hInst,uRes,pText,uSize+2);
	if(!uSize){delete pText;return -1;}

	wChar			= pText[0];
	pText[uPos  ]	= 0;
	pText[uPos+1]	= 0;

	iRet	= ToolbarAddStrings(pData,(LPARAM)(pText+1),wChar,TRUE);

	delete pText;



return iRet;
}

//*****************************************************************************
//*
//*		ToolbarDelItem
//*
//*****************************************************************************
//	Lscht eine Taste aus der Item-Liste.
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist der Index der Taste
static BOOL ToolbarDelItem(ToolbarData *pData,unsigned uIndex)
{
unsigned	uCount;
LPTBITEM	pItem;
RECT		sRect;
HWND		hWnd;


	if(uIndex>=pData->uItemsCnt)						// Position prfen
		{
		return FALSE;
		}

	pData->uItemsCnt--;

	pItem	=	pData->pItems[uIndex];
	uCount	=	pData->uItemsCnt-uIndex;

	memmove(pData->pItems+uIndex,pData->pItems+uIndex+1,uCount*sizeof(LPTBITEM));

	if(pItem->pInfo)									// Bilddaten lschen
		{
		DibFreeHandle(pItem->pInfo);
		pItem->pInfo = NULL;
		}

	if(pItem->wFlags&BTFL_CONTROL)						// Control ausblenden
		{
		hWnd = (HWND)pItem->dwData;

		if(hWnd && GetParent(hWnd)==pData->hWnd)
			{
			ShowWindow(hWnd,SW_HIDE);
			}
		}

	if(pData->uMoveItem!=0xFFFFFFFF)					// Ausgewhlte Taste anpassen
		{
		if(pData->uMoveItem>uIndex)
			{
			pData->uMoveItem--;
			}
		else if(pData->uMoveItem==uIndex)
			{
			pData->uMoveItem = 0xFFFFFFFF;
			}
		}


	delete pItem;										// Liste mit Tasten


	if(pData->uLines>1 || uIndex==0)					// Fenster neu zeichnen
		{
		InvalidateRect(pData->hWnd,NULL,FALSE);
		}
	else{
		sRect		= pData->pItems[uIndex-1]->sRect;
		sRect.left	= sRect.right;
		sRect.right	= pData->uSizeX;
		InvalidateRect(pData->hWnd,&sRect,FALSE);
		}




return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarAddItem
//*
//*****************************************************************************
//	Fgt eine neu Taste aus in die Item-Liste ein.
//	pData		: Zeiger auf die Fensterdaten.
//	uIndex		: Ist der Index bei dem die Taste eingefgt werden soll.
//	pButton		: Zeiger auf die Tasten-Daten.
//	uFlags		: Diverse Flags
//					TBFL_ZERO_FLAGS		= Die Flags in TBBUTTONEX auf 0 setzen
//	Ergibt den Index bei dem die Taste eingefgt wurde, bzw. -1 bei einem Fehler.
static int ToolbarAddItem(ToolbarData *pData,unsigned uIndex,LPTBBUTTONEX pButton,unsigned uFlags)
{
unsigned	uOffset;
unsigned	uCount;
LPTBITEM	pItem;
LPTBITEM   *pList;
RECT		sRect;
UINT		uVal;



	if(uIndex>pData->uItemsCnt)							// Prfe den Index
		{
		uIndex = pData->uItemsCnt;
		}

	if(uIndex>=pData->uItemsMax)						// Die Liste vergrern
		{
		pData->uItemsMax += 16+pData->uItemsMax/2;

			pList = new LPTBITEM[pData->uItemsMax];
		if(!pList)return -1;

		memcpy(pList,pData->pItems,pData->uItemsCnt*sizeof(LPTBITEM));
		delete pData->pItems;
		pData->pItems = pList;
		}

		pItem =	new TBITEM;
	if(!pItem)											// Zu wenig Speicher
		{
		return -1;
		}

	uCount	=	pData->uItemsCnt-uIndex;
	memmove(pData->pItems+uIndex+1,pData->pItems+uIndex,uCount*sizeof(LPTBITEM));
	pData->pItems[uIndex]	= pItem;

														// Ausgewhlte Taste anpassen
	if(pData->uMoveItem!=0xFFFFFFFF && pData->uMoveItem>=uIndex)
		{
		pData->uMoveItem++;
		}

	memcpy(pItem,pButton , sizeof(TBBUTTONEX));
	memset(((LPBYTE)pItem)+sizeof(TBBUTTONEX),0,sizeof(TBITEM)-sizeof(TBBUTTONEX));

	if(uFlags&TBFL_ZERO_FLAGS)pItem->wFlags=0;

	pItem->wInfoFlags = TBIF_REASSEMBLE;
	pData->uItemsCnt += 1;


	if(pItem->fsState&TBSTATE_HIDDEN)					// Ist die Taste versteckt
		{
		return uIndex;
		}

	pItem	= pData->pItems[(WORD)uIndex];
	uVal	= (pItem->fsState&(TBSTATE_CHECKED|TBSTATE_PRESSED))? 1:0;



	if(pItem->fsStyle&BTNS_SEP)							// Anzeigemodus
		{

		}
	else if(pItem->fsState&TBSTATE_INDETERMINATE)
		{
		pItem->wInfoFlags |= uVal+4;
		}
	else if(!(pItem->fsState&TBSTATE_ENABLED))
		{
		pItem->wInfoFlags |= uVal+4;
		}
	else{
		pItem->wInfoFlags |= uVal;
		}


														// Gibt es einen Gruppentext
	if(uIndex>0 && pData->pItems[uIndex-1]->wInfoFlags&(TBIF_PACKED|TBIF_BASE))
		{
		uOffset = uIndex-1;

		while(uOffset && !(pData->pItems[uOffset]->wInfoFlags&TBIF_BASE))
			{
			uOffset--;
			}

		pData->pItems[uOffset]->wInfoFlags |= TBIF_REASSEMBLE;

		for(uOffset++;uOffset<pData->uItemsCnt;uOffset++)
			{
			if(!(pData->pItems[uOffset]->wInfoFlags&TBIF_PACKED))break;
			pData->pItems[uOffset]->wInfoFlags |= TBIF_REASSEMBLE;
			}
		}


	pData->cReCalc |= 0x01;


	if(pData->uLines>1 || uIndex==0)					// Fenster neu zeichnen
		{
		InvalidateRect(pData->hWnd,NULL,FALSE);
		}
	else{
		sRect		= pData->pItems[uIndex-1]->sRect;
		sRect.left	= sRect.right;
		sRect.right	= pData->uSizeX;
		InvalidateRect(pData->hWnd,&sRect,FALSE);
		}



return uIndex;
}


//*****************************************************************************
//*
//*		ToolbarDelRecource
//*
//*****************************************************************************
//	Lscht eine Bitmap-Resource aus der Resource-Liste.
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist der Index der Resource
//	Ergibt TRUE wenn die Resource gelscht wurde, bzw. FALSE bei einem Fehler
static BOOL ToolbarDelRecource(ToolbarData *pData,unsigned uIndex)
{
LPTBRESOURCE	pResource;


	if(uIndex>=pData->uResourceCnt)
		{
		return FALSE;
		}

	pResource	= pData->pResource+uIndex;

	if(pResource->pInfo)								// Bilddaten lschen
		{
		DibFreeHandle(pResource->pInfo);
		pResource->pInfo	= NULL;
		pResource->pPixels	= NULL;
		}

	if(pResource->wFlags&TBRF_DELETE)					// HBITMAP lschen
		{
		DeleteObject((HBITMAP)pResource->uBitmapId);
		pResource->wFlags    &= ~TBRF_DELETE;
		pResource->uBitmapId  = 0;
		}



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarGetBitmapInfo
//*
//*****************************************************************************
//	Infos ber Bitmap-Resourcen abfragen
//	pData		: Zeiger auf die Fensterdaten
//	uPos		: Index der ersten Bitmap-Resource
//	iCount		: Anzahl der abzufragenden Bitmap-Resourcen
//	pInfo		: Hier werden die Infos gespeichert
//	Ergibt die Anzahl der gelesenen Infos
static int ToolbarGetBitmapInfo(ToolbarData *pData,unsigned uPos,int iCount,LPTBBITMAPINFO pInfo)
{
LPTBRESOURCE	pResource;
unsigned		uRet;


	pResource = pData->pResource+uPos;

	for(uRet=0;uPos<pData->uResourceCnt;uRet++,pResource++,pInfo++)
		{
		if(iCount<=0)break;

		if( (pResource->wFlags&TBRF_VARSIZE))				// Anzahl der Eintrge zhlen
		if(!(pResource->wFlags&TBRF_SCANNED))
			{
			ToolbarGetPicPos(pResource,0xFFFF);
			}

		pInfo->hInstance	= pResource->hInstance;
		pInfo->uBitmapId	= pResource->uBitmapId;
		pInfo->wCount		= pResource->wCount;
		pInfo->wScale		= pResource->wScale;
		pInfo->wSizeX		=(pResource->wFlags&TBRF_VARSIZE)? 0:pResource->wSizeX;
		pInfo->wSizeY		=(WORD)(pResource->pInfo->bmiHeader.biHeight);
		}

return uRet;
}

//*****************************************************************************
//*
//*		ToolbarGetPicSize
//*
//*****************************************************************************
//	Die Gre eines Einzelnen Bildes-abfragen
//	pData		: Zeiger auf die Fensterdaten
//	uPos		: Ist die ID des Bildes
//	Ergibt die Gre des Bildes oder -1 wenn das Bild nicht gefunden wurde.
static int ToolbarGetPicSize(ToolbarData *pData,unsigned uPos)
{
LPTBRESOURCE	pResource;
unsigned		uRes;
unsigned		uRet;



	if(uPos>0xFFFF)return -1;


	pResource = pData->pResource;

	for(uRes=0;;uRes++,pResource++)							// Suche die Resource zum Bild
		{
		if(uRes>=pData->uResourceCnt)
			{
			return -1;
			}

		if(!(pResource->wFlags&TBRF_VARSIZE))				// Immer die gleiche Bildgre
			{
			if(uPos>=pResource->wCount)
				{
				uPos -= pResource->wCount;
				continue;
				}

			uRet = MAKELONG(pResource->wSizeX,pResource->wSizeY);
			break;
			}

		if(pResource->wFlags&TBRF_SCANNED)					// Alle Bilder schon gezhlt
			{
			if(uPos>=pResource->wCount)
				{
				uPos -= pResource->wCount;
				continue;
				}
			}

		if(uPos<pResource->wCount)							// Bildersuche in Resorce neu starten
			{
			pResource->wLastPos	= 0xFFFF;
			pResource->wCount	= 0;
			}

		if(ToolbarGetPicPos(pResource,uPos))				// Suche das Unterbild in Resorce
			{
			uRet = MAKELONG(pResource->wSizeX,pResource->wSizeY);
			break;
			}

		uPos -= pResource->wCount;
		}


return uRet;
}

//*****************************************************************************
//*
//*		ToolbarAddBitmap
//*
//*****************************************************************************
//	Fgt eine Bitmap-Resource in die Resource-Liste ein.
//	pData		: Zeiger auf die Fensterdaten
//	iCount		: Anzahl der Bilder in der Bitmap-Resource
//				  Eine Skalierung kann mit TB_SCALED(n,s) bermittelt werden
//	pBitmap		: Zeiger auf die Bitmap-Daten
//	Ergibt die Einfgeposition oder -1 bei einem Fehler.
static int ToolbarAddBitmap(ToolbarData *pData,int iCount,LPTBADDBITMAPEX pBitmap)
{
LPTBRESOURCE	pResource;
LPBITMAPINFO	pInfo;
LPBITMAPINFO	pOrig;
unsigned		uOffset;
unsigned		uDelete;
unsigned		uScale;
unsigned		uFlags;
unsigned		uWidth;
unsigned		uMode;
unsigned		uPos;



	if(!pBitmap)											// Prfe die Werte
		{
		return -1;
		}

	if((iCount&0xFFFF0000)==TB_EXTENDED)					// Erweiterte Daten
		{
		uFlags	= pBitmap->wFlags;
		uWidth	= pBitmap->wWidth;
		uScale	= pBitmap->wScale;
		if(!uScale)uScale=256;
		if(!(uFlags&0x02))iCount &= 0xFFFF;
		uDelete	= ((uFlags&0x04) && !pBitmap->hInst)? TBRF_DELETE:0;
		}
	else{													// Normale Daten
		uScale  = 256;
		uWidth	= 0;
		uFlags	= 0;
		uDelete	= 0;
		}

	if(iCount<=0)											// Prfe die Werte
		{
		if(uDelete)DeleteObject((HBITMAP)pBitmap->nID);
		return -1;
		}


	// REM AZ prfen ob es eine System-Resource ist
	// if (pBitmap->hInst == HINST_COMMCTRL)


	if(pData->uResourceMax<=pData->uResourceCnt)			// Resource-Liste vergrern
		{
		pData->uResourceMax	= (pData->uResourceMax)? 1:pData->uResourceCnt+4;
		pResource			= new TBRESOURCE[pData->uResourceMax];
		memcpy(pResource,pData->pResource,pData->uResourceCnt*sizeof(TBRESOURCE));
		if(pData->pResource)delete pData->pResource;
		pData->pResource	= pResource;
		}


	uOffset		= 0;
	pResource	= pData->pResource;


	for(uPos=0;uPos<pData->uResourceCnt;uPos++,pResource++)	// Ist die Resource bereits geladen
		{
		if(pResource->hInstance==pBitmap->hInst)
		if(pResource->uBitmapId==pBitmap->nID)
			{
			if(pResource->wScale!=uScale)					// Hat sich die Sklaierung verndert
				{
				pResource->wScale = uScale;
				pData->cReCalc    = 0x07;

				UpdateView(pData);
				}

			return uOffset;
			}

		if(!(pResource->wFlags&TBRF_SCANNED))				// Anzahl der Eintrge zhlen
			{
			ToolbarGetPicPos(pResource,0xFFFF);
			}

		uOffset += pResource->wCount;
		}


	if(!pBitmap->hInst)										// Es wird ein HBITMAP direkt bergeben
			pInfo = GetDibOfBitmap  (NULL,				 (HBITMAP)pBitmap->nID,            &pResource->pPixels);
	else	pInfo = GetDibOfResource(NULL,pBitmap->hInst,(LPCTSTR)pBitmap->nID,RT_BITMAP,0,&pResource->pPixels);

	if(!pInfo)
		{
		if(uDelete)DeleteObject((HBITMAP)pBitmap->nID);
		return -1;
		}

	if(uFlags&0x02)											// Bilderanzahl aus Breite ermitteln
		{
		if(uWidth==TB_VARSIZE)
			{
			iCount = TB_VARSIZE;
			}
		else{
				iCount = (uWidth)? U(pInfo->bmiHeader.biWidth)/uWidth:0;
			if(!iCount)
				{
				if(uDelete)DeleteObject((HBITMAP)pBitmap->nID);
				DibFreeHandle(pInfo);
				return -1;
				}
			}
		}

	if((uFlags&0x01) && pInfo->bmiHeader.biWidth%iCount)	// Typ des Bildes automatisch erkennen
		{
		iCount	= TB_VARSIZE;
		}


	if(!CHECK_BITCNT(pInfo->bmiHeader.biBitCount))			// Wird die Bitanzahl untersttzt
		{
		pOrig	= pInfo;
		uMode	= (pData->cFullColor)? DIB_FORCE_COLORS(32):DIB_FORCE_COLORS(pData->cBitPixels);
		pInfo	= GetDibOfMemory(NULL,pOrig,uMode|DIB_PIXEL_DATA,&pResource->pPixels);
		DibFreeHandle(pOrig);

		if(!pInfo)
			{
			if(uDelete)DeleteObject((HBITMAP)pBitmap->nID);
			return -1;
			}

		if(!CHECK_BITCNT(pInfo->bmiHeader.biBitCount))		// Passt das konvertierte Bild
			{
			if(uDelete)DeleteObject((HBITMAP)pBitmap->nID);
			DibFreeHandle(pInfo);
			return -1;
			}
		}

	pResource->hInstance	= pBitmap->hInst;
	pResource->uBitmapId	= pBitmap->nID;
	pResource->pInfo		= pInfo;
	pResource->wBaseIndex	= uOffset;
	pResource->wScale		= uScale;
	pResource->wFlags		= uDelete;
	pResource->wRowWidth	= (WORD)BMI_ROW_WIDTH(pInfo->bmiHeader);

	if(iCount==TB_VARSIZE)									// Haben die Unterbilder eine variable Gre
		{
		pResource->wFlags  |= TBRF_VARSIZE;
		pResource->wLastPos	= 0xFFFF;
		pResource->wCount	= 0;

		if(uFlags&0x08)ToolbarGetPicPos(pResource,0xFFFF);
		if(uFlags&0x10)pBitmap->wWidth = TB_VARSIZE;
		if(uFlags&0x20)pBitmap->wScale = (WORD)(pInfo->bmiHeader.biHeight);
		}
	else{													// Alle Unterbilder gleich gro
		pResource->wFlags  |= TBRF_SCANNED|TBRF_FIXSIZE;
		pResource->wCount	= (WORD)(iCount);
		pResource->wSizeX	= (WORD)(pInfo->bmiHeader.biWidth/iCount);
		pResource->wSizeY	= (WORD)(pInfo->bmiHeader.biHeight);
		pResource->wLastY	= (WORD)(pInfo->bmiHeader.biHeight-1);

		if(pResource->wSizeX==0)
			{
			if(uDelete)DeleteObject((HBITMAP)pBitmap->nID);
			DibFreeHandle(pResource->pInfo);
			return -1;
			}

		if(uFlags&0x10)pBitmap->wWidth = pResource->wSizeX;
		if(uFlags&0x20)pBitmap->wScale = pResource->wSizeY;
		}

	if(uFlags&0x08)												// Count ermitteln
		{
		pBitmap->wCount = pResource->wCount;
		}

	pData->uResourceCnt++;


return uOffset;
}

//*****************************************************************************
//*
//*		ToolbarReplaceBitmap
//*
//*****************************************************************************
//	Tauscht eine Bitmap-Resource in der Resource-Liste aus.
//	pData		: Zeiger auf die Fensterdaten
//	uScale		: Ist die neue Scalierung (0=alte behalten)
//	pReplace	: Zeiger auf die Replace-Daten
//	Ergibt die Einfgeposition oder -1 bei einem Fehler.
static int ToolbarReplaceBitmap(ToolbarData *pData,UINT uScale,LPTBREPLACEBITMAP pReplace)
{
LPTBRESOURCE	pResource;
TBRESOURCE		sResource;
LPBYTE			pPixels;
UINT			uCount;
LPBITMAPINFO	pInfo;
UINT			uRes;


	// REM AZ prfen ob es eine System-Resource ist
	// if(pReplace->hInstOld==HINST_COMMCTRL)

	if(!pReplace || pReplace->nButtons<=0)					// Prfe Parameter
		{
		return FALSE;
		}

	pResource= pData->pResource;

	for(uRes=0;uRes<pData->uResourceCnt;uRes++,pResource++)	// Ist die neue Resource schon vorhanden
		{
		if(pResource->hInstance!=pReplace->hInstNew)continue;
		if(pResource->uBitmapId!=pReplace->nIDNew  )continue;

		return FALSE;
		}

	pResource = pData->pResource;

	for(uRes=0;;uRes++,pResource++)							// Suche die alte Resource
		{
		if(uRes>=pData->uResourceCnt)return FALSE;

		if(pResource->hInstance!=pReplace->hInstOld)continue;
		if(pResource->uBitmapId!=pReplace->nIDOld  )continue;

		break;
		}


	if(!pReplace->hInstNew)									// Es wird ein HBITMAP direkt bergeben
			pInfo	= GetDibOfBitmap  (NULL,(HBITMAP)pReplace->nIDNew,&pPixels);
	else	pInfo	= GetDibOfResource(NULL,pReplace->hInstNew,(LPCTSTR)pReplace->nIDNew,RT_BITMAP,0,&pPixels);

	if(!pInfo)return FALSE;



	if(!CHECK_BITCNT(pInfo->bmiHeader.biBitCount))			// Wird die Bitanzahl untersttzt
		{
		DibFreeHandle(pInfo);
		return FALSE;
		}
															// Prfe die Anzahl der Tasten
	if(pReplace->nButtons!=TB_VARSIZE && pInfo->bmiHeader.biWidth/pReplace->nButtons<=0)
		{
		DibFreeHandle(pInfo);
		return FALSE;
		}


	if(uRes+1<pData->uResourceCnt)							// Prfe die Bilderanzahl
		{
		if(pReplace->nButtons==TB_VARSIZE)
			{
			memset(&sResource,0,sizeof(sResource));

			sResource.wLastPos	= 0;
			sResource.wCount	= 0;
			sResource.pInfo		= pInfo;
			sResource.pPixels	= pPixels;
			sResource.wRowWidth	= (WORD)BMI_ROW_WIDTH(pInfo->bmiHeader);

			ToolbarGetPicPos(&sResource,0xFFFF);

			uCount = sResource.wCount;
			}
		else{
			uCount = pReplace->nButtons;
			}

		if( (pResource->wFlags&TBRF_VARSIZE))				// Anzahl der Eintrge zhlen
		if(!(pResource->wFlags&TBRF_SCANNED))
			{
			ToolbarGetPicPos(pResource,0xFFFF);
			}

		if(pResource->wCount!=uCount)
			{
			DibFreeHandle(pInfo);
			return FALSE;
			}
		}


	ToolbarDelRecource(pData,uRes);

	pResource->hInstance	= pReplace->hInstNew;
	pResource->uBitmapId	= (DWORD)pReplace->nIDNew;
	pResource->pPixels		= pPixels;
	pResource->pInfo		= pInfo;
	pResource->wRowWidth	= (WORD)BMI_ROW_WIDTH(pInfo->bmiHeader);

	if(uScale)
		{
		pResource->wScale	= uScale;
		}

	if(uCount==TB_VARSIZE)									// Haben die Unterbilder eine variable Gre
		{
		pResource->wFlags	= TBRF_VARSIZE;
		pResource->wLastPos	= 0xFFFF;
		pResource->wCount	= 0;
		}
	else{													// Alle Unterbilder gleich gro
		pResource->wFlags	= TBRF_SCANNED|TBRF_FIXSIZE;
		pResource->wCount	= (WORD)(pReplace->nButtons);
		pResource->wSizeX	= (WORD)(pInfo->bmiHeader.biWidth/pReplace->nButtons);
		pResource->wSizeY	= (WORD)(pInfo->bmiHeader.biHeight);
		pResource->wLastY	= (WORD)(pInfo->bmiHeader.biHeight-1);
		}

	pData->cReCalc = 0x07;



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarFindBitmap
//*
//*****************************************************************************
//	Such ein Unterbild in allen Resourcen
//	pData		: Zeiger auf die Fensterdaten.
//	uIndex		: Ist der absoluete Index der Bitmaps ber alle Resourcen.
//	pResIdx		: Hier wird der Index der Resourcen gespeichert.
//	pPosIdx		: Hier wird relative Index des Unterbildes in der Resource gespeichert.
//	uDirect		: Das Unterbild direkt mit *pResIdx und *pPosIdx laden
//	Ergibt einen Zeiger auf die Recource bzw. NULL wenn das Unterbild nicht gefunden wurde.
static LPTBRESOURCE ToolbarFindBitmap(ToolbarData *pData,unsigned uIndex,LPWORD pResIdx,LPWORD pPosIdx,UINT uDirect)
{
LPTBRESOURCE	pResource;
unsigned		uStart;
unsigned		uPos;
unsigned		uEnd;
unsigned		uPic;


	pResource	= pData->pResource;


	if(uDirect)												// Das Bild direkt suchen
		{
		pResource += *pResIdx;
		uPic	   = *pPosIdx;

		if(pResource->wFlags&TBRF_VARSIZE)					// Unterbilder mit variabler Gre
			{
			if(ToolbarGetPicPos(pResource,uPic))			// Suche das Bild in der Resource
				{
				return pResource;
				}
			}
		else if(uPic<pResource->wCount)						// Unterbilder mit gleicher Gre
			{
			pResource->wLastX	= pResource->wSizeX*uPic;
			return pResource;
			}

		return NULL;
		}


	uStart = 0;

	for(uPos=0;uPos<pData->uResourceCnt;uPos++,pResource++)	// Alle Recourcen durchlaufen
		{
		if(pResource->wFlags&TBRF_SCANNED)					// Weis man wieveile Bilder in der Resource sind
			{
			   uEnd = uStart+pResource->wCount;
			if(uEnd<=uIndex)
				{
				uStart = uEnd;
				continue;
				}
			}

		uPic = uIndex-uStart;

		if(pResource->wFlags&TBRF_VARSIZE)					// Variable Unterbild gren
			{
			if(ToolbarGetPicPos(pResource,uPic))			// Suche das Bild in der Resource
				{
				*pResIdx = (WORD)uPos;
				*pPosIdx = (WORD)uPic;
				return pResource;
				}
			}
		else if(uPic<pResource->wCount)						// Unterbilder mit gleicher Gre
			{
			pResource->wLastX = pResource->wSizeX*uPic;
			*pResIdx = (WORD)uPos;
			*pPosIdx = (WORD)uPic;
			return pResource;
			}

		uStart += pResource->wCount;
		}


return NULL;
}

//*****************************************************************************
//*
//*		ToolbarUpdateImages
//*
//*****************************************************************************
//	Erzeugt die Unterbilter neu.
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist der Index bei der Taste.
static void ToolbarUpdateImages(ToolbarData *pData,unsigned uIndex)
{
LPTBRESOURCE	pResource;
LPBYTE			pPixels;
LPBYTE			pScale;
LPCWSTR			pText;
LPBITMAPINFO	pInfo;
LPBITMAPINFO	pBase;
LPBITMAPINFO	pTemp;
LPTBITEM		pItem;
LPTBITEM		pNext;
RECT			sArea;
SIZE			sSize;
UINT			uScale;
UINT			uFlags;
COLORREF		uUpLeft;
COLORREF		uColor;
int				iPlace;
int				iTextX;
int				iTextY;
int				iSizeA;
int				iSizeB;
int				iSizeX;
int				iSizeY;
int				iDrawX;
int				iDrawY;
int				iDropN;
int				iAlpha;
int				iBaseX;
int				iBaseY;
int				iPicX;
int				iPicY;
int				iOffX;
int				iOffY;
int				iSetX;
int				iSetY;
int				iAddX;
int				iAddY;
int				iBits;
int				iMode;
int				iStep;
int				iSub;
int				iPos;
int				iVal;
UINT			uPic;
HDC				hDc;


	if(uIndex>=pData->uItemsCnt)							// Prfe die Position
		{
		return;
		}

	pItem	= pData->pItems[uIndex];
	iSizeX	= pItem->wRealX;
	iSizeY	= pItem->wRealY;
	pInfo	= pItem->pInfo;
	iPos	= (int)pItem->iString;
	hDc		= CreateCompatibleDC(NULL);
	pText	= (U(iPos)<pData->uStringCnt)? pData->pStrings[iPos]:L"\0";
	iBits	= (pData->cFullColor)? 32:pData->cBitPixels;
	iVal	= (pItem->fsStyle&BTNS_SEP)? 1:6;

															// Neue Gre vom Images
	if(!pInfo || pInfo->bmiHeader.biWidth!=iSizeX || pInfo->bmiHeader.biHeight!=iSizeY*iVal)
		{
		if(pInfo)
			{
			DibFreeHandle(pInfo);
			pItem->pInfo = NULL;
			}

		if((pItem->wFlags&BTFL_CONTROL)	&& !*pText)			// Control ohne Text
			{
			DeleteDC(hDc);
			return;
			}

			pInfo = CreateDibIndirect(hDc,iSizeX,iSizeY*iVal,iBits,&pItem->pPixels);
		if(!pInfo)
			{
			if((!iSizeX || !iSizeY) && (pItem->fsStyle&BTNS_SEP))
				{
				pItem->wInfoFlags &= ~TBIF_REASSEMBLE;
				}

			DeleteDC(hDc);
			return;
			}

		pItem->pInfo	= pInfo;
		pItem->hBitmap	= DibGetBitmap(pInfo);
		}

	SelectObject(hDc,pItem->hBitmap);
	SetTextColor(hDc,pData->aColors[TBC_TEXT]);
	SetBkMode   (hDc,TRANSPARENT);

	pItem->wInfoFlags &= ~TBIF_REASSEMBLE;


	if(pItem->fsStyle&BTNS_SEP)								// Ist es ein Separator
		{
		SetBkColor(hDc,pData->aColors[TBC_BK]);
		DrawRectangle(hDc,0,0,iSizeX,iSizeY);

		if(iSizeX>iSizeY)
			{
			iOffY	= (iSizeY-1)/2;
			iOffX	= pData->iGabX/2;
			iSizeX -= pData->iGabX;

			SetBkColor(hDc,GetSysColor(COLOR_3DSHADOW));
			DrawRectangle(hDc,iOffX,iOffY,iSizeX,1);
			SetBkColor(hDc,RGB(255,255,255));
			DrawRectangle(hDc,iOffX,iOffY+1,iSizeX,1);
			}
		else{
			iOffX	= (iSizeX-1)/2;
			iOffY	= pData->iGabY/2;
			iSizeY -= pData->iGabY;

			SetBkColor(hDc,GetSysColor(COLOR_3DSHADOW));
			DrawRectangle(hDc,iOffX,iOffY,1,iSizeY);
			SetBkColor(hDc,RGB(255,255,255));
			DrawRectangle(hDc,iOffX+1,iOffY,1,iSizeY);
			}

		DeleteDC(hDc);
		return;
		}


	SetBkColor(hDc,pData->aColors[TBC_BK]);					// Hintergrund zeichnen
	DrawRectangle(hDc,0,0,iSizeX,iSizeY*6);

	iSizeA	= iSizeY;
	iOffX	= pData->iGabX/2;
	iOffY	= pData->iGabY/2;
	iSizeX -= pData->iGabX;
	iSizeY -= pData->iGabY;
	iSizeB	= iSizeX;


	if(pItem->wInfoFlags&TBIF_PACKED)						// Hhe ausgleichen
		{
		iSizeY += pData->iGabY/2;
		}



	if(pItem->wFlags&BTFL_CONTROL)							// Externes Control
		{
		iTextY	= iOffY+iSizeY-TEXTOFF_Y;
		iTextY -= pData->uFontHeigh;

		SelectObject(hDc,pData->hFontN);

		for(iPos=0;iPos<6;iPos++,iOffY+=iSizeA)				// Texte zeichnen
			{
			iStep = iPos;

			if(pItem->wFlags&BTFL_NOT_SUNKEN)iStep&=~1;
			if(pItem->wFlags&BTFL_NOT_GRAYED)iStep&= 3;
			if(pItem->wFlags&BTFL_NOT_HOT	)iStep&=~2;

			sArea.top		= iTextY+iSizeA*iPos;
			uFlags			= (pItem->fsStyle&BTNS_NOPREFIX)? DT_NOPREFIX|DT_SINGLELINE:DT_SINGLELINE;

			uFlags         |= DT_CENTER|DT_TOP;
			sArea.left		= iOffX;
			sArea.right		= iOffX+iSizeB;
			sArea.bottom	= sArea.top+iSizeY;

			if(iStep&4)
					DrawGrayTextW(hDc,pText+1,*pText,&sArea,uFlags);
			else	DrawTextW    (hDc,pText+1,*pText,&sArea,uFlags);
			}
		}
	else if(pItem->wFlags&(BTFL_CHECKBOX|BTFL_RADIO))		// Ist es eine Checkbox
		{
		GetDrawSize(0,&sSize);

		iDrawY			= iSizeY;
		pItem->wShowX	= (WORD)(iOffX+sSize.cx);
		SelectObject(hDc,pData->hFontN);

		if(pItem->wInfoFlags&TBIF_TEXT_EMPTY)				// Es ist ein Leertext unten vorhanden
		if((pItem->wInfoFlags&(TBIF_PACKED|TBIF_BASE))!=TBIF_PACKED)
			{
			iDrawY -= pData->uFontHeigh;
			iDrawY -= TEXTOFF_Y;
			}

		for(iPos=0;iPos<6;iPos++,iOffY+=iSizeA)				// Checkboxen zeichnen
			{
			iStep			= iPos;
			sArea.top		= iOffY+(iDrawY-sSize.cy)/2;
			sArea.left		= iOffX;
			sArea.right		= sArea.left+sSize.cx;
			sArea.bottom	= sArea.top +sSize.cy;

			if(pItem->wFlags&BTFL_NOT_SUNKEN)iStep&=~1;
			if(pItem->wFlags&BTFL_NOT_GRAYED)iStep&= 3;
			if(pItem->wFlags&BTFL_NOT_HOT	)iStep&=~2;

			     if(iStep/2==0)uFlags=CBS_UNCHECKEDNORMAL;
			else if(iStep/2==1)uFlags=CBS_UNCHECKEDHOT;
			else               uFlags=CBS_UNCHECKEDDISABLED;

			if(iStep&1)uFlags+=4;							// Checkbox gewhlt

			if(pItem->wFlags&BTFL_RADIO)
					DrawRadioW   (hDc,NULL,&sArea,uFlags,0);
			else	DrawCheckBoxW(hDc,NULL,&sArea,uFlags,0);

			sArea.top		= iOffY;
			sArea.left		= sArea.right+TEXTOFF_X;
			sArea.right		= iOffX+iSizeX;
			sArea.bottom	= iOffY+iDrawY;

			uFlags = (pItem->fsStyle&BTNS_NOPREFIX)? DT_NOPREFIX|DT_SINGLELINE|DT_LEFT|DT_VCENTER:DT_SINGLELINE|DT_LEFT|DT_VCENTER;

			if(iStep>=4)
					DrawGrayTextW(hDc,pText+1,*pText,&sArea,uFlags);
			else	DrawTextW    (hDc,pText+1,*pText,&sArea,uFlags);
			}
		}
	else if(pItem->wInfoFlags&TBIF_BITMAP_OK)				// Taste mit Bild
		{
		pResource	= pData->pResource;
		pResource  += pItem->wResource;
		uPic		= pItem->wResPic;
		iDrawX		= iSizeX;
		iDrawY		= iSizeY;

		if(pResource->wFlags&TBRF_VARSIZE)					// Unterbilder mit variabler Gre
			{
			if(!ToolbarGetPicPos(pResource,uPic))return;
			}
		else{						 						// Unterbilder mit gleicher Gre
			if(uPic>=pResource->wCount)return;
			pResource->wLastX = pResource->wSizeX*uPic;
			}

		iBaseX	= pResource->wSizeX;
		iBaseY	= pResource->wSizeY;
		iPlace	= pResource->wLastX;
		uScale	= pResource->wScale;
		iSub	= 0;

		if(pResource->wFlags&TBRF_SHIFT)					// Unterbild beginnt bei der 2.ten X-Spalte
			{
			iBaseX--;
			iPlace++;
			}

		if(pData->uButtonZoom)
				uScale *= pData->uButtonZoom;
		else	uScale *= 256;

		if(uScale==0x10000 || !pData->cFullColor)			// Ist das Bild skalliert
			{
			uScale	= 0;
			iPicX	= iBaseX;
			iPicY	= iBaseY;
			}
		else{
			iPicX	= (iBaseX*uScale+0x7FFF)/0x10000;
			iPicY	= (iBaseY*uScale+0x7FFF)/0x10000;
			}

		if(pItem->wInfoFlags&TBIF_TEXT_BOTTOM)				// Gibt es einen Text unten
			{
			iTextX	= iOffX+iSizeX/2;
			iTextY	= iOffY+iDrawY-TEXTOFF_Y;
			iTextY -= pData->uFontHeigh;

			if(pItem->wFlags&BTFL_TEXT_INNER)
				{
				iTextY += pData->iPaddingY/2;
				iTextY -= pData->iPaddingY;
				}
			else{
				iDrawY -= pData->uFontHeigh;
				iDrawY -= TEXTOFF_Y;
				}
			}
		else if(pItem->wInfoFlags&TBIF_TEXT_EMPTY)			// Ist ein Leertext unten vorhanden
			{
			iDrawY -= pData->uFontHeigh;
			iDrawY -= TEXTOFF_Y;
			}

		if(pItem->wInfoFlags&TBIF_TEXT_RIGHT)				// Gibt es einen Text rechts
			{
			iTextX  = iOffX+pData->iPaddingX+iPicX+TEXTOFF_X;
			iTextY	= iOffY+(iDrawY-pData->uFontHeigh)/2;

			if(pItem->wFlags&BTFL_TEXT_INNER)
				{
				iTextX -= pData->iPaddingX;
				iTextX += pData->iPaddingX/2;
				iSub	= iDrawX-iPicX-pData->iPaddingX;
				}
			else{
				iDrawX	= iPicX+pData->iPaddingX;
				}
			}

		pItem->wShowX = iOffX+iDrawX;

		if(pItem->fsStyle&BTNS_DROPDOWN)					// Dropdown Taste vorhanden
			{
			   iVal	= (iPicY+2)/4;
			if(iVal<4)iVal=4;
			if(iVal>6)iVal=6;

			iDropN  = (13*iVal)/4;

			if(!(pItem->wInfoFlags&TBIF_TEXT_RIGHT))
				{
				iDrawX -= iDropN;
				}
			else if(!(pItem->wFlags&BTFL_TEXT_INNER))
				{
				pItem->wShowX+=iDropN;
				iTextX += iDropN;
				}
			else{
				iDrawX -= iDropN;
				iSub   -= iDropN;
				}

			pItem->wShowX |= iDropN<<10;
			}



		if(pResource->pInfo->bmiHeader.biBitCount!=32)		// Gibt es eine Alpha-Maske
			{
			iAlpha = 0;
			}
		else{
			pPixels	= pResource->pPixels;
			iPos	= pResource->pInfo->bmiHeader.biWidth;
			iPos   *= pResource->pInfo->bmiHeader.biHeight*4;

			for(iPos--;iPos>0;iPos-=4)
				{
				if(pPixels[iPos])break;
				}

			iAlpha = (iPos>0);
			}


		if(uScale)											// Muss das Zielbild skaliert werden
			{
			pTemp  = CreateDibIndirect(hDc,iPicX ,iPicY ,iBits,&pPixels);
			pBase  = CreateDibIndirect(hDc,iBaseX,iBaseY,iBits,&pScale );
			}
		else{
			pTemp  = CreateDibIndirect(hDc,iPicX ,iPicY ,iBits,&pPixels);
			pBase  = pTemp;
			pScale = pPixels;
			}


		if(iAlpha==0)										// Der linke obere Punkt ist der Hintergrund
			{
			uUpLeft = DibGetUpperLeft(pResource->pInfo,pResource->pPixels);
			uUpLeft = DibColorDefault(uUpLeft);
			}


		for(iPos=0;iPos<6;iPos++,iOffY+=iSizeA)				// Tasten zeichnen
			{
			iStep = iPos;

			if(pItem->wFlags&BTFL_NOT_SUNKEN)iStep&=~1;
			if(pItem->wFlags&BTFL_NOT_GRAYED)iStep&= 3;
			if(pItem->wFlags&BTFL_NOT_HOT	)iStep&=~2;

			if(!(iPos&1))									// Neues Icon erstellen
				{
				iVal = iPos/2;
															// Das Icon aus den Recourcen kopieren
				DibCopyImageEx(pResource->pInfo,pResource->pPixels,0,0,iPlace,pResource->wLastY-iBaseY+1,iBaseX,iBaseY,pBase);

				if(iAlpha==0)								// Hintergund im Icon anpassen
					{
					DibConvertDefault(pBase,pScale);
					DibConvertColor(pBase,pScale,uUpLeft|0x40000000,((iVal==1)? pData->aColors[TBC_HOT]:pData->aColors[TBC_BK]));
					}

				if(pItem->wFlags&BTFL_COLOR_CHANGE)			// Eine Farbe ndern
					{
					if((pItem->wFlags&BTFL_COLOR_DUAL)==BTFL_COLOR_DUAL)
						{
						uColor = (WORD)(pItem->dwData);
						DibConvertColor(pBase,pScale,pData->aColorChange[0],TBC_C16_TO_RGB(uColor)|0xFF000000);
						uColor = (WORD)(pItem->dwData>>16);
						DibConvertColor(pBase,pScale,pData->aColorChange[1],TBC_C16_TO_RGB(uColor)|0xFF000000);
						}
					else{
 						DibConvertColor(pBase,pScale,pData->aColorChange[0],(DWORD)pItem->dwData|0xFF000000);
						}
					}

				if(iStep&4)									// In Graustufen umwandeln
					{
					DibConvertGrayScale(pBase,pScale,pData->aColors[TBC_BK],0x20000000);
					}

				if(uScale)									// Die Gre skalieren
					{
					DibConvertSize(pTemp,pPixels,pBase,pScale);
					}
				}


			iSetX	= iOffX+(iDrawX-iPicX-iSub)/2;
			iSetY	= iOffY+(iDrawY-iPicY     )/2;
			iMode	= (pItem->wFlags&BTFL_FORCE_LIFTED) || !((pData->uStyle&TBSTYLE_FLAT) || (pItem->wFlags&BTFL_FORCE_FLAT));

			if(pItem->fsStyle&BTNS_DROPDOWN)
				{
				iStep |= 8;
				}

			if(iStep&1)										// Getdrckt
				{
				if(pData->cFullColor)
						DrawFullButtonEx(pInfo,pItem->pPixels,iOffX,iOffY,iDrawX,iDrawY,iStep,pData->aColors[TBC_HOTEX]);
				else	DrawFlatButtonEx(hDc,iOffX,iOffY,iDrawX,iDrawY,(iStep&2)? PFB_PRESSED|PFB_FILLED|PFB_HOT:PFB_PRESSED|PFB_FILLED);
				if(pData->iPaddingX>4)iSetX++;
				if(pData->iPaddingY>4)iSetY++;
				}
			else if((iStep&2) || iMode)						// Erhoben
				{
				if(pData->cFullColor)
						DrawFullButtonEx(pInfo,pItem->pPixels,iOffX,iOffY,iDrawX,iDrawY,iStep,pData->aColors[TBC_HOTEX]);
				else	DrawFlatButtonEx(hDc,iOffX,iOffY,iDrawX,iDrawY,(iStep&2)? PFB_FILLED|PFB_HOT:PFB_FILLED);
				}


			if(pItem->fsStyle&BTNS_DROPDOWN)				// Dropdown Taste vorhanden
				{
				sArea.top		= iOffY;
				sArea.left		= iOffX+iDrawX;
				sArea.right		= iOffX+iDrawX+iDropN;
				sArea.bottom	= iOffY+iDrawY;

				if(pData->cFullColor)
					{
					switch(iPos)
						{
					case 0:		uFlags=0x10;	iVal=DG_DROPDOWN|TS_NORMAL;   break;
					case 1:		uFlags=0x10;	iVal=DG_DROPDOWN|TS_NORMAL;   break;
					case 2:		uFlags=0x12;	iVal=DG_DROPDOWN|TS_HOT;	  break;
					case 3:		uFlags=0x13;	iVal=DG_DROPDOWN|TS_PRESSED;  break;
					case 4:		uFlags=0x14;	iVal=DG_DROPDOWN|TS_DISABLED; break;
					case 5:		uFlags=0x15;	iVal=DG_DROPDOWN|TS_DISABLED; break;
						}

					if(iPos || iMode)
						{
						DrawFullButtonEx(pInfo,pItem->pPixels,iOffX+iDrawX,iOffY,iDropN,iDrawY,uFlags,pData->aColors[TBC_HOTEX]);
						}

					DrawGlyph(hDc,&sArea,iVal|DG_USENOTHEMA,1);
					}
				else{
					uFlags			= (iPos&1)? DFCS_FLAT|DFCS_PUSHED|DFCS_SCROLLDOWN:DFCS_FLAT|DFCS_SCROLLDOWN;

					switch(iPos)
						{
					case 0:		uFlags=DFCS_FLAT|DFCS_SCROLLDOWN;						iVal=(iMode)? 0:PFB_MONO;	break;
					case 1:		uFlags=DFCS_FLAT|DFCS_SCROLLDOWN;						iVal=0; 					break;
					case 2:		uFlags=DFCS_FLAT|DFCS_SCROLLDOWN|DFCS_HOT;				iVal=0; 					break;
					case 3:		uFlags=DFCS_FLAT|DFCS_SCROLLDOWN|DFCS_HOT|DFCS_PUSHED;	iVal=PFB_PRESSED;			break;
					case 4:		uFlags=DFCS_FLAT|DFCS_SCROLLDOWN|DFCS_INACTIVE;			iVal=0;						break;
					case 5:		uFlags=DFCS_FLAT|DFCS_SCROLLDOWN|DFCS_INACTIVE;			iVal=0; 					break;
						}

					DrawFrameControl(hDc,&sArea,DFC_SCROLL,uFlags);
					DrawFlatButtonEx(hDc,iOffX+iDrawX,iOffY,iDropN,iDrawY,iVal);
					}
				}


			if(pData->cFullColor)							// Das Icon zeichnen
					DibPutAlpha(pInfo,pItem->pPixels,iSetX,iSetY,pTemp,pPixels);
			else	SetDIBitsToDevice(hDc,iSetX,iSetY,iPicX,iPicY,0,0,0,iPicY,pPixels,pTemp,DIB_RGB_COLORS);

															// Smoth-Filter
			if(pResource->pInfo->bmiHeader.biBitCount==4 && iBits==32)
				{
				iAddX = (pData->iPaddingX>4)? 2:0;
				iAddY = (pData->iPaddingY>4)? 2:0;
				DibConvertSmoth(pInfo,pItem->pPixels,iSetX-iAddX/2,iSetY-iAddY/2,iPicX+iAddX,iPicY+iAddY);
				}

			if(*pText)										// Text ausgeben
				{
				SelectObject(hDc,pData->hFontN);

				sArea.top		= iTextY+iSizeA*iPos;
				uFlags			= (pItem->fsStyle&BTNS_NOPREFIX)? DT_NOPREFIX|DT_SINGLELINE:DT_SINGLELINE;

				if(pItem->wInfoFlags&TBIF_TEXT_BOTTOM)
					{
					uFlags     |= DT_CENTER|DT_TOP;			// Text unten
					sArea.left	= iOffX;
					sArea.right	= iOffX+iSizeB;
					sArea.bottom= sArea.top+iSizeY;

					if(iStep&4)
							DrawGrayTextW(hDc,pText+1,*pText,&sArea,uFlags);
					else	DrawTextW    (hDc,pText+1,*pText,&sArea,uFlags);
					}
				else if(pItem->wInfoFlags&TBIF_TEXT_RIGHT)	// Text rechts
					{
					uFlags     |= DT_LEFT|DT_VCENTER;
					sArea.left	= iTextX;
					sArea.right	= pItem->sRect.right-pData->iGabX;

					if((iStep&1) && (pItem->wFlags&BTFL_TEXT_INNER))
						{
						if(pData->iPaddingX>4)sArea.left+=2;
						if(pData->iPaddingY>4)sArea.top +=1;
						}

					sArea.bottom= sArea.top+pData->uFontHeigh;

					if(iStep&4)
							DrawGrayTextW(hDc,pText+1,*pText,&sArea,uFlags);
					else	DrawTextW    (hDc,pText+1,*pText,&sArea,uFlags);
					}
				}
			}

		if(uScale)DibFreeHandle(pBase);
		DibFreeHandle(pTemp);
		}
	else{													// Taste ohne Bild
		iDrawY			= iSizeY;
		pItem->wShowX	= iOffX+iSizeX;
		SelectObject(hDc,pData->hFontN);

		if(pItem->wInfoFlags&TBIF_TEXT_EMPTY)
			{
			iDrawY -= pData->uFontHeigh;
			iDrawY -= TEXTOFF_Y;
			}

		for(iPos=0;iPos<6;iPos++,iOffY+=iSizeA)				// Tastenrahmen zeichnen
			{
			iStep			= iPos;
			sArea.top		= iOffY;
			sArea.left		= iOffX;
			sArea.right		= iOffX+iSizeX;
			sArea.bottom	= iOffY+iDrawY;

			if(pItem->wFlags&BTFL_NOT_SUNKEN)iStep&=~1;
			if(pItem->wFlags&BTFL_NOT_GRAYED)iStep&= 3;
			if(pItem->wFlags&BTFL_NOT_HOT	)iStep&=~2;

			     if(iStep&1   )uFlags=PBS_PRESSED;
			else if(iStep/2==2)uFlags=PBS_DISABLED;
			else if(iStep/2==1)uFlags=PBS_HOT;
			else 			   uFlags=PBS_NORMAL;

			DrawButtonW(hDc,NULL,&sArea,uFlags,0);

			uFlags = (pItem->fsStyle&BTNS_NOPREFIX)? DT_NOPREFIX|DT_SINGLELINE|DT_CENTER|DT_VCENTER:DT_SINGLELINE|DT_CENTER|DT_VCENTER;

			if(iStep>=4)
					DrawGrayTextW(hDc,pText+1,*pText,&sArea,uFlags);
			else	DrawTextW    (hDc,pText+1,*pText,&sArea,uFlags);
			}
		}


	if(pItem->wInfoFlags&TBIF_TEXT_GROUP)					// Gruppentext unten
		{
		pText = NULL;

		for(iPos=uIndex-1,iVal=0;iPos>=0;iPos--)
			{
			   pNext = pData->pItems[iPos];
			if(pNext->fsStyle&TBSTYLE_SEP )break;
			if(pNext->fsState&TBSTATE_WRAP)break;
			if(pNext->fsState&TBSTATE_HIDDEN)continue;

			iVal += pNext->wRealX;
			}

		for(iPos=uIndex+1,iSub=0;U(iPos)<pData->uItemsCnt;iPos++)
			{
			   pNext = pData->pItems[iPos];
			if(pNext->fsStyle&TBSTYLE_SEP )
				{
				iPos	= (int)pNext->iString;
				pText	= (U(iPos)<pData->uStringCnt)? pData->pStrings[iPos]:L"\0";
				break;
				}

			if(pNext->fsState&TBSTATE_WRAP)break;
			if(pNext->fsState&TBSTATE_HIDDEN)continue;

			iSub += pNext->wRealX;
			}

		sArea.left		=-iVal;
		sArea.right		= pItem->wRealX+iSub;
		sArea.bottom	= pItem->wRealY-pData->iGabY+pData->iGabY/2;
		sArea.top		= sArea.bottom -pData->uFontHeigh;

		SelectObject(hDc,pData->hFontN);
		SetTextColor(hDc,pData->aColors[TBC_GROUP]);

		for(iPos=0;iPos<6;iPos++,sArea.top+=pItem->wRealY,sArea.bottom+=pItem->wRealY)
			{
			DrawTextW(hDc,pText+1,*pText,&sArea,DT_SINGLELINE|DT_CENTER|DT_VCENTER);
			}
		}


	DeleteDC(hDc);

}

//*****************************************************************************
//*
//*		ToolbarCalcButtonSize
//*
//*****************************************************************************
//	Berechnet die Gre einer Taste
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist der Index bei der Taste.
//	bHorz		: Die Toolbar ist horizontal
//	pFlags		: Ergebnis nicht speichern und Flags bergeben
//	Ergibt folgende Werte die die Gre definieren
//				0 = Fehler bei der Grenberechnung
//				1 = Gre ist gleich geblieben
//				2 = Gre ist gleich geblieben aber Flags sind anderst
//				3 = Gre hat sich verndert
static int ToolbarCalcButtonSize(ToolbarData *pData,LPTBITEM pItem,BOOL bHorz,LPWORD pFlags)
{
LPTBRESOURCE	pResource;
LPCWSTR			pText;
SIZE			sSize;
UINT	 		uText;
UINT	 		uScale;
WORD			wFlags;
int				iTextY;
int				iTextX;
int				iSizeY;
int				iSizeX;
int				iPicY;
int				iPicX;
int				iVal;




	wFlags	= 0;


//******************** Separator **********************************************


	if(pItem->fsStyle&BTNS_SEP)								// Ist es ein Separator
		{
		wFlags |=  TBIF_STRETCH;

		if(bHorz)
			{
			iSizeX	= SEPARATOR_WIDTH;
			iSizeY	= 0;
			}
		else{
			iSizeY	= SEPARATOR_WIDTH;
			iSizeX	= 0;
			}

		goto Exit;
		}

	if(pItem->wFlags&BTFL_NOT_PACKING)						// Nicht vertikal stabeln
		{
		wFlags |=  TBIF_STRETCH;
		}

	   uText =  (int)pItem->iString;
	if(uText>=pData->uStringCnt)							// Kein Text vorhanden
		{
		pText	= L"\0";
		iTextX	= 0;
		iTextY	= 0;
		}
	else{													// Ein Text ist vorhanden
		pText = pData->pStrings[uText];

		if(*pText)											// Textabmessungen ermitteln
			{
			iVal	= (pItem->fsStyle&BTNS_NOPREFIX)? DT_NOPREFIX|DT_SINGLELINE|DT_RETURNY:DT_SINGLELINE|DT_RETURNY;
			iTextX	= CalcTextSizeW(NULL,pText+1,*pText,iVal,pData->hFontN);
			iTextY	= HIWORD(iTextX);
			iTextX	= LOWORD(iTextX);
			}
		else{
			iTextX	= 0;
			iTextY	= 0;
			}

		if( (pItem->wFlags&(BTFL_TEXT_SIZE)))				// Minimal Textbreite
		if(!(pItem->wFlags&(BTFL_CONTROL|BTFL_COLOR_CHANGE)))
			{
			   iVal	= LOWORD(pItem->dwData);
			if(iTextX<iVal)iTextX=iVal;
			}
		}


//******************** Control ************************************************


	if(pItem->wFlags&BTFL_CONTROL)							// Ist es ein extrenes Control
		{
		GetControlSize((HWND)pItem->dwData,&iSizeX,&iSizeY);

		iSizeX += pData->iGabX+2;
		//iSizeY += pData->iGabY+2;	// REM AZ auf WIN7 Aussehen prfen

		if(*pText)											// Text unter Control vorhanden
			{
			iTextX += TEXTADD_X*2;

			if(pData->iMinWidthX>0)
				{
				iVal = pData->iMinWidthX-pData->iPaddingX;
				if(iTextX<iVal)iTextX=iVal;
				}

			if(pData->iMaxWidthX>0)
				{
				iVal = pData->iMaxWidthX-pData->iPaddingX;
				if(iTextX>iVal)iTextX=iVal;
				}

			if(iSizeX<iTextX)
				{
				iSizeX = iTextX;
				}

			iSizeY += iTextY+TEXTOFF_Y;
			wFlags |= TBIF_TEXT_BOTTOM;
			}

		goto Exit;
		}


//******************** CheckBox ***********************************************


	if(pItem->wFlags&(BTFL_CHECKBOX|BTFL_RADIO))			// Ist es eine Checkbox
		{
		GetDrawSize(0,&sSize);

		if(*pText)											// CheckBox mit Text
			{
			   iSizeY = sSize.cy;
			if(iSizeY<iTextY)iSizeY=iTextY;

			iSizeY += pData->iPaddingY;
			iSizeX  = sSize.cx+pData->iGabX;
			iSizeX += iTextX+TEXTOFF_X*2;
			iVal	= pData->iGabX-pData->iGabX/2;
			iVal   -= TEXTADD_X;
			if(iVal<0)iSizeX-=iVal;

			wFlags |= TBIF_TEXT_RIGHT;
			}
		else{												// CheckBox ohne Text
			iSizeX	= sSize.cx+pData->iGabX+2;
			iSizeY	= sSize.cy+pData->iGabY+2;
			}

		goto Exit;
		}


//******************** Taste **************************************************


	if(pItem->iBitmap>=0)									// Ist ein Bild vorhanden
		{
			pResource = ToolbarFindBitmap(pData,pItem->iBitmap,&pItem->wResource,&pItem->wResPic,pItem->wInfoFlags&TBIF_BITMAP_OK);
		if(!pResource)
			{
			goto OnlyText;
			}

		iPicX	= pResource->wSizeX;
		iPicY	= pResource->wSizeY;
		uScale	= pResource->wScale;

		if(pResource->wFlags&TBRF_SHIFT)					// Unterbild beginnt bei der 2.ten X-Spalte
			{
			iPicX--;
			}

		if(pData->uButtonZoom)
				uScale *= pData->uButtonZoom;
		else	uScale *= 256;

		if(uScale!=0x10000 && pData->cFullColor)			// Bilder Zoomen
			{
			iPicX = (iPicX*uScale+0x7FFF)/0x10000;
			iPicY = (iPicY*uScale+0x7FFF)/0x10000;
			}


		wFlags |= TBIF_BITMAP_OK;
		iSizeX	= iPicX;
		iSizeY	= iPicY;

		if(pItem->fsStyle&BTNS_DROPDOWN)					// Dropdown Taste vorhanden
			{
			   iVal	= (iPicY+2)/4;
			if(iVal<4)iVal=4;
			if(iVal>6)iVal=6;

			iSizeX += (13*iVal)/4;
			}


		if(*pText)											// Gibt es einen Text
			{
			if(((pData->uStyle&TBSTYLE_LIST) && !(pItem->wFlags&BTFL_TEXT_BOTTOM)) || (pItem->wFlags&BTFL_TEXT_RIGHT))
				{											// Text ist rechts
				iSizeY += TEXTADD_Y;
				if(iSizeY<iTextY)iSizeY=iTextY;
				iSizeX += TEXTOFF_X+iTextX;
				iVal	= pData->iGabX-pData->iGabX/2;
				iVal   -= TEXTADD_X;
				if(iVal<0)iSizeX-=iVal;
				wFlags |= TBIF_TEXT_RIGHT;
				}											// Text ist unten
			else{
				if(iSizeX<iTextX)iSizeX=iTextX;

				if(pData->iMinWidthX>0)
					{
					iVal = pData->iMinWidthX-pData->iPaddingX;
					if(iSizeX<iVal)iSizeX=iVal;
					}

				if(pData->iMaxWidthX>0)
					{
					iVal = pData->iMaxWidthX-pData->iPaddingX;
					if(iSizeX>iVal)iSizeX=iVal;
					}

				iSizeX += pData->iPaddingX;
				iSizeY += iTextY+TEXTOFF_Y;
				wFlags |= TBIF_TEXT_BOTTOM;
				}
			}

		iSizeX += pData->iGabX+pData->iPaddingX;
		iSizeY += pData->iGabY+pData->iPaddingY;

		goto Exit;
		}


//******************** Nur Text ***********************************************
OnlyText:

	if(((pData->uStyle&TBSTYLE_LIST) && !(pItem->wFlags&BTFL_TEXT_BOTTOM)) || (pItem->wFlags&BTFL_TEXT_RIGHT))
		{													// Text ist rechts
		   iSizeY=iTextY+TEXTADD_Y;
		if(iSizeY<pData->iBitmapSizeY)
			{
			iSizeY = pData->iBitmapSizeY;
			}

		   iSizeX=iTextX+TEXTADD_X*2;
		if(iSizeX<pData->iBitmapSizeX)
			{
			iSizeX = pData->iBitmapSizeX;
			}

		wFlags |= TBIF_TEXT_RIGHT;
		}
	else{													// Text unten
		if(iTextX<4)iTextX=4;

		iSizeX	= iTextX+TEXTADD_X*2+4;

		if(pData->iMinWidthX>0)
			{
			iVal = pData->iMinWidthX-pData->iPaddingX;
			if(iSizeX<iVal)iSizeX=iVal;
			}

		if(pData->iMaxWidthX>0)
			{
			iVal = pData->iMaxWidthX-pData->iPaddingX;
			if(iSizeX>iVal)iSizeX=iVal;
			}

		iSizeY	= pData->iBitmapSizeY;
		iSizeY += iTextY+TEXTOFF_Y;
		wFlags |= TBIF_TEXT_BOTTOM|TBIF_FULL_BUTTON;
		}

	iSizeX += pData->iGabX+pData->iPaddingY;
	iSizeY += pData->iGabY+pData->iPaddingY;


//******************** Berechnete Gre auswerten *****************************
Exit:


	if(iSizeX<0)iSizeX=0;
	if(iSizeY<0)iSizeY=0;

	if(pFlags)
		{
		*pFlags = wFlags;
		return MAKELONG(iSizeX,iSizeY);
		}

															// Ist die Gre gleich geblieben
	if(pItem->wInfoFlags&TBIF_SIZE_OK)
	if(pItem->wSizeX==iSizeX)
	if(pItem->wSizeY==iSizeY)
		{
		if(!((pItem->wInfoFlags^wFlags)&TBIF_MASK))
			{
			return 1;
			}

		pItem->wInfoFlags  &=~(TBIF_MASK|TBIF_STRETCH|TBIF_BITMAP_OK);
		pItem->wInfoFlags  |=  TBIF_REASSEMBLE;
		pItem->wInfoFlags  |=  wFlags;
		return 2;
		}

	pItem->wSizeX		= (WORD)iSizeX;
	pItem->wSizeY		= (WORD)iSizeY;
	pItem->wRealX		= (WORD)iSizeX;
	pItem->wRealY		= (WORD)iSizeY;
	pItem->wInfoFlags  &=~(TBIF_MASK|TBIF_STRETCH|TBIF_BITMAP_OK);
	pItem->wInfoFlags  |=  TBIF_SIZE_OK;
	pItem->wInfoFlags  |=  TBIF_REASSEMBLE;
	pItem->wInfoFlags  |=  wFlags;




return 3;
}

//*****************************************************************************
//*
//*		ToolbarCalcPacking
//*
//*****************************************************************************
//	Berechnet welche Taste bereinander gepackt werden sollen.
//	pData		: Zeiger auf die Fensterdaten
//	uStart		: Index der ersten Taste die geprft werden soll
//	uUntil		: Index bis zu dem geprft werden soll
static void ToolbarCalcPacking(ToolbarData *pData,unsigned uStart,unsigned uUntil,int iMax)
{
unsigned		uIndex;
unsigned		uBase;
unsigned		uItem;
LPTBITEM		pItem;
LPTBITEM		pNext;
int				iRest;
int				iSize;
int				iMaxX;
int				iFlag;
int				iStep;
int				iAdd;




	if(!pData->cIsHorz || !(pData->uStyle&TBSTYLE_PACKED))	// Vertikale Ausrichtung
		{
		for(uIndex=uStart;uIndex<uUntil;uIndex++)
			{
			pItem	           =   pData->pItems[uIndex];
			pItem->wInfoFlags &= ~(TBIF_PACKED|TBIF_BASE|TBIF_LAST);
			}

		return;
		}


	for(uIndex=uStart;uIndex+1<uUntil;uIndex++)				// Suche das unterste erste Element
		{
		pItem	= pData->pItems[uIndex];

		if(pItem->fsState   &TBSTATE_HIDDEN)continue;
		if(pItem->fsState   &TBSTATE_WRAP  )continue;
		if(pItem->wInfoFlags&TBIF_STRETCH  )continue;

		iSize = pItem->wSizeY;

		if(iSize+3>=iMax)
			{
			continue;
			}

		pNext = pData->pItems[uIndex+1];

		if(pNext->wInfoFlags&TBIF_STRETCH)continue;			// Geht das Element ber die ganze Hhe

		iAdd  = pNext->wSizeY;
		iAdd -= pData->iGabY/2;
		iFlag = 0;

		if(iSize+iAdd>iMax)continue;						// Passt eine weitere Taste in die Y-Richtung

		   iMaxX=pItem->wRealX;
		if(iMaxX<pNext->wRealX)
			{
			iMaxX = pNext->wRealX;
			}

		pItem->wInfoFlags |= TBIF_BASE|iFlag;
		pNext->wInfoFlags |= TBIF_PACKED;
		pNext->wRealY	   = iAdd;
		iSize			  += iAdd;
		uBase			   = uIndex;

		for(uIndex+=2;uIndex<uUntil;uIndex++)				// Weitere Elemente suchen
			{
			pNext = pData->pItems[uIndex];

			if(pNext->fsState   &TBSTATE_HIDDEN)continue;
			if(pNext->wInfoFlags&TBIF_STRETCH  )break;		// Geht das Element ber die ganze Hhe

			iAdd  = pItem->wSizeY;
			iAdd -= pData->iGabY/2;

			if(iSize+iAdd>iMax)break;

			pNext->wInfoFlags |= TBIF_PACKED;
			pNext->wRealY	   = iAdd;
			iSize			  += iAdd;

			if(pNext->fsState&TBSTATE_WRAP)
				{
				uIndex++;
				break;
				}
			}

		iAdd				= iMax-iSize;
		iRest				= iAdd%(uIndex-uBase);
		iStep				= iAdd/(uIndex-uBase);
		uIndex			   -= 1;

		pNext				= pData->pItems[uIndex];
		pNext->wInfoFlags  |= TBIF_LAST;
		

		for(uItem=uIndex;uItem>uBase;)						// Die Abstnde angleichen
			{
			uItem--;
			pNext->wRealY  += iStep+iRest;
			pNext->wRealX	= iMaxX;
			pNext			= pData->pItems[uItem];
			iRest			= 0;
			}

		pItem->wRealY	= pItem->wSizeY;
		pItem->wRealY  += iStep;
		pItem->wRealX	= iMaxX;
		}

}

//*****************************************************************************
//*
//*		ToolbarCalcLayout
//*
//*****************************************************************************
//	Berechnet die Position der einzelnen Taste im Fenster
//	pData		: Zeiger auf die Fensterdaten
//	uFlags		: Diverse Flags
//					Bit0 = Das Ergebnis nicht speichern
//					Bit1 = Immer horizontal
//					Bit2 = Immer vertical
//	Ergibt die Gre der Felder (Low=X Hi=Y)
static int ToolbarCalcLayout(ToolbarData *pData,unsigned uFlags=0)
{
unsigned		uSepPos;
unsigned		uIndex;
unsigned		uStart;
LPTBITEM		pItem;
LPTBTAGDATA		pTags;
RECT			sRect;
WORD			wMask;
WORD			wInfo;
WORD			wFlags;
int				iNoSet;
int				iLines;
int				iCtrlX;
int				iCtrlY;
int				iSizeX;
int				iSizeY;
int				iPosX;
int				iPosY;
int				iMaxX;
int				iCalc;
int				iWrap;
int				iMaxN;
int				iHorz;
int				iOff;
int				iMax;
int				iRet;
int				iSub;



	wMask			= (pData->cReCalc&0x08)? ~(TBIF_TEXT_GROUP|TBIF_PACKED|TBIF_BASE|TBIF_SIZE_OK):~(TBIF_TEXT_GROUP|TBIF_PACKED|TBIF_BASE);
	iCalc			= (pData->cReCalc&0x06);
	iNoSet			=  uFlags&1;
	iLines			=  1;
	uSepPos			=  0;
	wFlags			=  0;
	uStart			=  0;
	wInfo			=  0;
	iWrap			=  0;
	iSub			=  0;

	if(iNoSet==0)											// Ergebnis nicht speichern
		{
		pData->cReCalc = 0;
		}
	else{
		pTags		= pData->pTagChain;
		
		for(;pTags;pTags=pTags->pNext)						// Tags neu berechnen
			{
			pTags->sItem.ucFlags |= TBTF_RECALC;
			}
		}

	if(uFlags&0x06)											// Immer Horizontal
		{
		iHorz = uFlags&2;
		}
	else{													// Prfe ob hozizontale Toolbar
		iHorz = 1;

		for(uIndex=0,iOff=0;uIndex<pData->uItemsCnt;uIndex++)
			{
			pItem = pData->pItems[uIndex];

			if(pItem->fsState&TBSTATE_HIDDEN)				// Ist die Taste sichtbar
				{
				continue;
				}

			if(pItem->fsState&TBSTATE_WRAP)
				{
				if(iOff)break;
				continue;
				}

			if(uIndex+1>=pData->uItemsCnt)
				{
				if(iOff)break;
				continue;
				}

			if((pItem->fsStyle&BTNS_SEP) && iOff==0)
				{
				continue;
				}

			iOff++;

			if(iOff>1)break;
			}

		if(uIndex>=pData->uItemsCnt && pData->uItemsCnt>1)
			{
			iHorz = 0;
			}
		}

	if(!iHorz)												// Vertikale Ausrichtung
		{
		iMax  = pData->iBitmapSizeX;
		pData->cIsHorz = 0;
		}
	else{
		iMax  = pData->iBitmapSizeY;
		pData->cIsHorz = 1;
		}

	iMaxN = iMax;



	if(iHorz)												// Prfen ob es Gruppentexte oder untere Linien gibt	
		{
		for(uIndex=0;uIndex<pData->uItemsCnt;uIndex++)	
			{
			pItem = pData->pItems[uIndex];

			if(pItem->fsState&TBSTATE_HIDDEN)				// Ist die Taste unsichtbar
				{
				continue;
				}

			if(pItem->fsStyle&BTNS_SEP)						// Hat der Separator einen Gruppentext
				{
				if(!(pItem->wFlags&BTFL_GROUP_TEXT))continue;
				break;
				}

			if(pItem->wInfoFlags&TBIF_TEXT_GROUP)			// Gibt es eine Gruppentext
				{
				break;
				}
															// Gibt es eine Bodenzeile
			if((pItem->wInfoFlags&TBIF_TEXT_BOTTOM) && !(pItem->wFlags&BTFL_TEXT_INNER) && !(pItem->wInfoFlags&TBIF_FULL_BUTTON))
				{
				break;
				}
			}
		
		if(uIndex<pData->uItemsCnt)							// Die Leer-Marke fr alle setzen
			{
			for(uIndex=0;uIndex<pData->uItemsCnt;uIndex++)	
				{
				pItem = pData->pItems[uIndex];

				if(pItem->fsState&(TBSTATE_HIDDEN|BTNS_SEP))		
					{
					continue;
					}

				pItem->wInfoFlags |= TBIF_TEXT_EMPTY;
				}

			iSub  = pData->uFontHeigh;
			iSub += TEXTOFF_Y;
			}
		}




	for(uIndex=0;uIndex<pData->uItemsCnt;uIndex++)			// Suche die hchste Taste
		{
		pItem				= pData->pItems[uIndex];
		pItem->wInfoFlags  &= wMask;


		if(pItem->fsState&TBSTATE_HIDDEN)					// Ist die Taste unsichtbar
			{
		    SetRectEmpty(&pItem->sRect);
		    continue;
			}

		if(iCalc || !(pItem->wInfoFlags&TBIF_SIZE_OK))		// Abmessungen neu berechnen
			{
			   iRet = ToolbarCalcButtonSize(pData,pItem,iHorz,0);
			if(iRet>=2 || iCalc>2)pItem->wInfoFlags|=TBIF_REASSEMBLE;
			}

		if(pItem->fsStyle&BTNS_SEP)							// Ist es ein Separator
			{
			if(iHorz && (pItem->wSizeX==0 || pItem->wRealY==0))
				{
				   iRet = ToolbarCalcButtonSize(pData,pItem,TRUE,0);
				if(iRet>=2 && !iNoSet)
					{
					pItem->wInfoFlags|=TBIF_REASSEMBLE;
					pItem->sRect.right=pItem->sRect.left;
					}
				}
			else if(!iHorz && (pItem->wRealX==0 || pItem->wSizeY==0))
				{
				   iRet = ToolbarCalcButtonSize(pData,pItem,FALSE,0);
				if(iRet>=2 && !iNoSet)
					{
					pItem->wInfoFlags|=TBIF_REASSEMBLE;
					pItem->sRect.right=pItem->sRect.left;
					}
				}
			}

		if(pItem->fsStyle&BTNS_SEP)							// Hat der Separator einen Gruppentext
			{
			if(iHorz && (pItem->wFlags&BTFL_GROUP_TEXT) && !(wFlags&TBIF_TEXT_BOTTOM))
				{
				   iMaxN += pData->uFontHeigh+1;
				if(iMaxN>iMax)iMax=iMaxN;

				ToolbarCalcPacking(pData,uSepPos,uIndex,iMax-iSub);

				for(;uSepPos<uIndex;uSepPos++)				// Text-Flags in Abschnitt aktualisieren
					{
					pItem	= pData->pItems[uSepPos];
					if(pItem->fsState&TBSTATE_HIDDEN)continue;
					if(pItem->wInfoFlags&TBIF_PACKED)continue;
					pItem->wInfoFlags |= TBIF_TEXT_GROUP;
					pItem->wInfoFlags |= TBIF_TEXT_EMPTY;
					}
				}
			else{
				ToolbarCalcPacking(pData,uSepPos,uIndex,iMax-iSub);
				}

			iMaxN	= pData->iBitmapSizeX;
			uSepPos	= uIndex+1;
			wFlags	= 0;
			continue;
			}

		if(!iHorz)											// Vertikale Ausrichtung
			{
			if(iMax<pItem->wSizeX)
				{
				iMax	= pItem->wSizeX;
				}
			}
		else{												// Horizontale Ausrichtung
			if(iMax<pItem->wSizeY)
				{
				iMax	= pItem->wSizeY;
				}

			if(iMaxN<pItem->wSizeY)
				{
				iMaxN	= pItem->wSizeY;
				wFlags |= pItem->wInfoFlags;
				}

			wInfo |= pItem->wInfoFlags;
			}
		}


	ToolbarCalcPacking(pData,uSepPos,uIndex,iMax-iSub);


//******************** Tastenpositionen berechnen *****************************


	wInfo  &=  TBIF_TEXT_GROUP|TBIF_TEXT_BOTTOM;
	iPosX	=  ( iHorz)? pData->iIndent:0;
	iPosY	=  (!iHorz)? pData->iIndent:0;
	iMaxX	=  0;
	iSizeX	=  0;
	iSizeY	=  0;

	for(uIndex=0;;uIndex++)									// Berechne die Positionen
		{
		if(iWrap || uIndex>=pData->uItemsCnt)				// Neue Reihe begonnen
			{
			uStart	= uIndex;
			uSepPos	= uIndex;

			if(uIndex>=pData->uItemsCnt)break;

			iWrap	= 0;
			iPosX	= 0;
			}

		pItem = pData->pItems[uIndex];

		if(pItem->fsState&TBSTATE_HIDDEN)					// Ist die Taste sichtbar
			{
		    continue;
			}

		if(!iHorz)											// Vertikal ausgerichtet
			{
			if(pItem->fsStyle&BTNS_SEP)
				{
				iSizeY = SEPARATOR_WIDTH;
				}
			else{
				iSizeY = pItem->wSizeY;
				}

			iSizeX	= iMax;
			iOff	= 0;
			}												// Separatoren am Anfang verstecken
		else if((pItem->fsStyle&BTNS_SEP) && uIndex>0 && iPosX==0)
			{
			iSizeX	= 0;
			iSizeY	= 0;
			iOff	= 0;
			}												// Separatoren am Ende verstecken
		else if((pItem->fsStyle&BTNS_SEP) && uIndex>0 && (pItem->fsState&TBSTATE_WRAP))
			{
			iSizeX	= 0;
			iSizeY	= 0;
			iOff	= 0;
			}
		else if(pItem->fsStyle&BTNS_SEP)					// Sichtbarer Separator
			{
			iSizeX	= SEPARATOR_WIDTH;
			iSizeY	= iMax;
			}
		else if(pItem->wInfoFlags&TBIF_BASE)				// Die Taste ist ist unter anderen gestellt
			{
			iSizeX	= pItem->wRealX;
			iSizeY	= pItem->wRealY;
			iOff	= iMax-iSizeY;
			}
		else if(pItem->wInfoFlags&TBIF_PACKED)				// Die Taste ist ber eine andere gestellt
			{
			iSizeX	= pItem->wRealX;
			iSizeY	= pItem->wRealY;
			iOff   -= iSizeY;
			}
		else{												// Wenn Gruppentexte Icons ohne untere Zeile mit Leerzeile auffllen
			if(wInfo && !(pItem->wInfoFlags&(TBIF_TEXT_BOTTOM|TBIF_TEXT_GROUP)))
				{
				pItem->wInfoFlags |= TBIF_TEXT_EMPTY;
				}

			iSizeX	= pItem->wSizeX;
			iSizeY	= iMax;
			iOff	= 0;
			}
															// Gre um untere Leerzeile erhhen
		if((pItem->wInfoFlags&(TBIF_TEXT_EMPTY |TBIF_BASE))==(TBIF_TEXT_EMPTY|TBIF_BASE))
		if((pItem->wInfoFlags& TBIF_TEXT_BOTTOM)==0 || !(pItem->wFlags&BTFL_TEXT_INNER ))
			{
			iSub    = pData->uFontHeigh;
			iSub   += TEXTOFF_Y;
			iOff   -= iSub;
			iSizeY += iSub;
			}


		if(!iNoSet)											// Hat sich die Position verndert
			{
			if(pItem->sRect.left!=iPosX || pItem->sRect.top!=iPosY+iOff || pItem->sRect.right-pItem->sRect.left!=iSizeX || pItem->sRect.bottom-pItem->sRect.top!=iSizeY)
				{
				pItem->wRealX		= iSizeX;
				pItem->wRealY		= iSizeY;
				pItem->sRect.left	= iPosX;
				pItem->sRect.top	= iPosY+iOff;
				pItem->sRect.right	= iPosX+iSizeX;
				pItem->sRect.bottom	= iPosY+iSizeY+iOff;

				if((pItem->wFlags&BTFL_CONTROL) && GetControlSize((HWND)pItem->dwData,&iCtrlX,&iCtrlY))
					{
					HWND	hWnd;

					hWnd			= (HWND)pItem->dwData;
					sRect.left		= iPosX+(iSizeX-iCtrlX)/2;
					sRect.top		= iSizeY;
															// Gibt es unten einen Text
					if(pItem->wInfoFlags&(TBIF_TEXT_BOTTOM|TBIF_TEXT_EMPTY|TBIF_TEXT_GROUP))
						{
						sRect.top  -= pData->iFontHeight;
						sRect.top  -= TEXTOFF_Y;
						}

					sRect.top		= iPosY+(sRect.top-iCtrlY)/2+iOff;

					if(hWnd && GetParent(hWnd)==pData->hWnd)
						{
						SetWindowPos(hWnd,NULL,sRect.left,sRect.top,0,0,SWP_NOSIZE|SWP_NOZORDER);
						ShowWindow  (hWnd,SW_SHOW);
						}
					}

				if(pItem->fsState&TBSTATE_WRAP)				// Den Rest der Zeile neu zeichnen
					{
					sRect		= pItem->sRect;
					sRect.right = pData->uSizeX;
					InvalidateRect(pData->hWnd,&pItem->sRect,FALSE);
					}

				if(uIndex==pData->uMarkItem)
					{
					UpdateMark(pData,pData->uMarkItem,pData->uMarkMode);
					}
				}
			}

		if(!iHorz)											// Vertikale Ausrichtung
			{
			iPosY  += iSizeY;

			if(iPosX<iSizeX)iPosX=iSizeX;
			if(iPosX>iMaxX )iMaxX=iPosX;

			iLines += 1;
			iWrap	= 1;
			iSizeY	= 0;
			}
		else if((pItem->fsState&TBSTATE_WRAP))				// Eine neue Zeile beginnen
			{
			if((pItem->fsStyle&BTNS_SEP) && iPosX==pData->iIndent)
					iPosY  += iSizeY;
			else	iPosY  += iMax;

			   iPosX+=iSizeX;
			if(iPosX>iMaxX)iMaxX=iPosX;

			iPosX	= 0;
			iLines += 1;
			iWrap	= 1;
			iSizeY	= 0;

			}
		else if(!iHorz)										// Verikale Ausrichtung
			{
			iPosY  += iSizeY;
			iSizeY	= 0;
			}
		else if(iOff<=3 || (pItem->wInfoFlags&TBIF_LAST))	// Horizontale Ausrichtung
			{												
			iPosX  += iSizeX;
			}
		}

	if(iPosX>iMaxX)iMaxX=iPosX;
	if(iPosX==0 && iLines>1)iLines--;


	if(iNoSet)												// Ergebnisse nicht speichern
		{
		pData->iLastLine = iPosX;
		return MAKELONG(iMaxX,iPosY+iSizeY);
		}

	pData->uLines	= iLines;
	pData->uMaxX	= iMaxX;
	pData->uMaxY	= iPosY+iSizeY;



return 1;
}

//*****************************************************************************
//*
//*		ToolbarCalcFrame
//*
//*****************************************************************************
//	Berechnet eine neue Gre vom Fenster
//	pData		: Zeiger auf die Fensterdaten
//	iWidth		: Ist die Maximale Breite vom Fenster
//	uFlags		: Bit0 = WRAP-Flags setzen
//				  Bit1 = In Y-Richtung scannen
//	Ergibt die berechnete Gre der Toolbar
static int ToolbarCalcFrame(ToolbarData *pData,int iWidth,UINT uFlags,ItemSave *pTemp=0)
{
LPTBITEM	   	pItem;
LRESULT			lSize;
int				iMode;
int				iIndex;
int				iValue;
int				iAlign;
int				iStart;
int				iStop;
int				iHalf;
int				iDiff;
int				iBase;
int				iMin;
int				iAll;


	if(pData->uItemsCnt==0)									// Keine Eintrge
		{
		return 0;
		}

	iAll   = pData->uItemsCnt;
	iAlign = pData->cAlign;

	if(!pTemp)												// Speicher fr alte Werte allokieren
		{
		pTemp = (ItemSave*)alloca(sizeof(ItemSave)*pData->uItemsCnt);

		for(iIndex=0;iIndex<iAll;iIndex++,pItem++)
			{
			pItem					 = pData->pItems[iIndex];
			pTemp[iIndex].wInfoFlags = pItem->wInfoFlags;
			pTemp[iIndex].wSizeX	 = pItem->wSizeX;
			pTemp[iIndex].wSizeY	 = pItem->wSizeY;
			pTemp[iIndex].wRealX	 = pItem->wRealX;
			pTemp[iIndex].wRealY	 = pItem->wRealY;
			pTemp[iIndex].bState	 = pItem->fsState;
			pItem->fsState			&= ~TBSTATE_WRAP;
			}
		}


	if(uFlags&0x02)											// In Y-Richtung scannen
		{
		iMin	= ToolbarCalcLayout(pData,0x05)&0xFFFF;
		lSize	= ToolbarCalcLayout(pData,0x03);
		iStop	= (lSize&0xFFFF)+16;
		iStart	= iMin;
		iMode	= 0;

		if(uFlags&0x01)
			{
			for(iIndex=0;iIndex<iAll;iIndex++,pItem++)		// Wrap-Flags speichern
				{
				pItem					= pData->pItems[iIndex];
				pTemp[iIndex].bState   &= ~TBSTATE_WRAP;
				pTemp[iIndex].bState   &=  TBSTATE_WRAP|pItem->fsState;
				}
			}

		for(;;)												// Binr sich dem Umbruch nhern
			{
			iDiff  = iStop-iStart;
			iHalf  = iStart+iDiff/2;

			for(iIndex=0;iIndex<iAll;iIndex++,pItem++)		// Wrap-Flags lschen
				{
				pItem			 = pData->pItems[iIndex];
				pItem->fsState	&= ~TBSTATE_WRAP;
				}

			lSize  = ToolbarCalcFrame(pData,(iHalf<=iMin)? 0:iHalf,0x01,pTemp);
			iValue = (int)(lSize>>16);

			if(iDiff<=0)break;

			if(iValue<iWidth)								// Ist das Fenster zu hoch
				{
				iStop  = iHalf;
				}
			else{
				iStart = (iDiff==1)? iHalf+1:iHalf;

				if(!(uFlags&0x01))continue;

				for(iIndex=0;iIndex<iAll;iIndex++,pItem++)	// Wrap-Flags speichern
					{
					pItem					= pData->pItems[iIndex];
					pTemp[iIndex].bState   &= ~TBSTATE_WRAP;
					pTemp[iIndex].bState   &=  TBSTATE_WRAP|pItem->fsState;
					}
				}
			}

		uFlags &= ~0x01;

		goto Exit;
		}




	if(iWidth>=16384)										// Eine horizontale Zeile
		{
		pData->cAlign = 0;
		iBase  = iAll;
		iMode  = 0x03;
		}
	else if(iWidth<=4)										// Eine vertikale Zeile
		{
		iBase  = iAll;
		iMode  = 0x05;

		for(iIndex=0;iIndex<iAll;iIndex++,pItem++)			// Wrap-Flags setzen
			{
			pItem			= pData->pItems[iIndex];
			if(pItem->fsState&TBSTATE_HIDDEN)continue;
			pItem->fsState |= TBSTATE_WRAP;
			}

		pData->cAlign = 1;
		}
	else{
		pData->cAlign = 0;
		iMode  = 0x01;
		iBase  = 0;
		}


	for(;;)													// Die Breite jeder Zeile bestimmen
		{
		iStart	= iBase+1;
		iStop	= iAll;

		if(iStart>iStop)iStart=iStop;

		for(;;)												// Binr sich dem Umbruch nhern
			{
			iDiff  = iStop -iStart;
			iHalf  = iStart+iDiff/2;

			pData->uItemsCnt = iHalf;

			lSize  = ToolbarCalcLayout(pData,iMode);
			iValue = pData->iLastLine;

			if(iValue==iWidth)								// Passt die Breite
				{
				break;
				}

			if(iDiff<=1)
				{
				if(iDiff>0 && iValue<iWidth)
					{
					iStart++;
					continue;
					}

				if(iValue>iWidth )
					{
					iHalf--;
					}

				break;
				}

			if(iValue>iWidth)								// Neue Breite zu gro
				{
				if(iDiff<=1){iHalf=iStart;break;};
				iStop  = iHalf-1;
				}
			else{
				iStart = iHalf;
				}
			}

		if(iHalf>iBase)
				iBase =iHalf;
		else	iBase+=1;

		if(iBase>=iAll)break;

		pItem			= pData->pItems[iBase-1];
		pItem->fsState |= TBSTATE_WRAP;
		}

Exit:

	if(!(uFlags&0x01))										// Alle Tasten wieder herstellen
		{
		for(iIndex=0;iIndex<(int)pData->uItemsCnt;iIndex++,pItem++)
			{
			pItem				= pData->pItems[iIndex];
			pItem->wInfoFlags	= pTemp[iIndex].wInfoFlags;
			pItem->wSizeX		= pTemp[iIndex].wSizeX;
			pItem->wSizeY		= pTemp[iIndex].wSizeY;
			pItem->wRealX		= pTemp[iIndex].wRealX;
			pItem->wRealY		= pTemp[iIndex].wRealY;
			pItem->fsState		= pTemp[iIndex].bState;
			}
		}
	else{
		ToolbarCalcLayout(pData,iMode&0xFE);
		}


	pData->uItemsCnt	= iAll;
	pData->cAlign		= iAlign;



return (int)lSize;
}

//*****************************************************************************
//*
//*		ToolbarHitTest
//*
//*****************************************************************************
//	Prft ob eine Taste unter einer Position liegt
//	pData		: Zeiger auf die Fensterdaten
//	iPosX/Y		: Ist die Cursor-Position
//	iMode		: Dropdown extra markieren
//	Ergibt den Index der Taste bzw. mit Bit 30 wird die Dropdown-Taste angezeigt
static int ToolbarHitTest(ToolbarData *pData,int iPosX,int iPosY,int iMode)
{
POINT			sPoint;
unsigned		uIndex;
LPTBITEM		pItem;
RECT			sRect;
int				iBase;


	sPoint.x	= iPosX;
	sPoint.y	= iPosY;


	for(uIndex=0;uIndex<pData->uItemsCnt;uIndex++)			// Suche die hchste Taste
		{
		pItem	= pData->pItems[uIndex];

		if(pItem->fsState&TBSTATE_HIDDEN)					// Ist die Taste sichtbar
			{
		    continue;
			}

		if(pItem->fsStyle&BTNS_SEP)							// Ist es ein Separator
			{
		    continue;
			}

		if(!PtInRect(&pItem->sRect,sPoint))					// Cursor im Bereich
			{
			continue;
			}

		sRect.top	 = pItem->sRect.top   +pData->iGabY/2;
		sRect.left	 = pItem->sRect.left  +pData->iGabX/2;
		sRect.right	 = pItem->sRect.left +(pItem->wShowX&0x03FF);
		sRect.bottom = pItem->sRect.bottom+pData->iGabY/2-pData->iGabY;

		if(!(pItem->wInfoFlags&TBIF_FULL_BUTTON))
		if( (pItem->wInfoFlags&TBIF_TEXT_EMPTY) || ((pItem->wInfoFlags&TBIF_TEXT_BOTTOM) && !(pItem->wFlags&BTFL_TEXT_INNER)))
		if( (pItem->wInfoFlags&(TBIF_PACKED|TBIF_BASE))!=TBIF_PACKED)
			{
			sRect.bottom -= pData->uFontHeigh;
			sRect.bottom -= TEXTOFF_Y;
			}

		if(!PtInRect(&sRect,sPoint))						// Cursor auf der Taste
			{
			break;
			}

		if(iMode)											// Mit Dropdown-Taste prfen
			{
			iBase = sRect.right-(pItem->wShowX>>10);

			if(iPosX>=iBase)
				{
				uIndex |= 0x40000000;
				}
			}

		return uIndex;
		}



return -1;
}

//*****************************************************************************
//*
//*		ToolbarMouseMove
//*
//*****************************************************************************
//	Es gab eine Cursor-Bewegung
//	pData		: Zeiger auf die Fensterdaten
//	iPosX/Y		: Ist die neue Cursor-Position
//	uFlags		: WPARAM von WM_MOUSEMOVE
static void ToolbarMouseMove(ToolbarData *pData,int iPosX,int iPosY,unsigned uFlags)
{
unsigned		uIndex;
unsigned		uValue;
LPTBITEM		pItem;



	uValue  = pData->uMoveItem;

	if(pData->cMoveDrop)
		{
		uValue |= 0x40000000;
		}

	   uIndex = ToolbarHitTest(pData,iPosX,iPosY,1);
	if(uIndex==uValue && (char)uFlags==pData->cMoveFlags)
		{
		return;
		}
															// Alte Taste wieder
	if(pData->uMoveItem!=uIndex && pData->uMoveItem<pData->uItemsCnt)
		{
		pItem				= pData->pItems[(WORD)pData->uMoveItem];
		pItem->wInfoFlags  &= ~0x07;
		uValue				= pItem->fsState&(TBSTATE_CHECKED|TBSTATE_PRESSED);

		if((pItem->fsState&TBSTATE_INDETERMINATE) || !(pItem->fsState&TBSTATE_ENABLED))
			{
			pItem->wInfoFlags |= (uValue)? 5:4;				// Taste gesperrt
			}
		else{
			pItem->wInfoFlags |= (uValue)? 1:0;				// Normal
			}

		pData->uMoveItem	= 0xFFFFFFFF;
		pData->cMoveDrop	= 0x00000000;
		pData->uDropState	= 0x00000000;
		InvalidateRect(pData->hWnd,&pItem->sRect,FALSE);
		}


	if(uIndex!=0xFFFFFFFF)
		{
		pItem				= pData->pItems[(WORD)uIndex];
		pItem->wInfoFlags  &= ~0x07;
		pData->uDropState	= 0;
		pData->cMoveDrop	= 0;
		uValue				= pItem->fsState&(TBSTATE_CHECKED|TBSTATE_PRESSED);

		if(pItem->wFlags&(BTFL_CHECKBOX|BTFL_RADIO))
				uFlags  = 0;
		else if(pItem->wFlags&BTFL_RIGHT_CLICK)
				uFlags &= MK_LBUTTON|MK_RBUTTON;
		else	uFlags &= MK_LBUTTON;

		if((pItem->fsState&TBSTATE_INDETERMINATE) || !(pItem->fsState&TBSTATE_ENABLED))
			{
			pItem->wInfoFlags |= (uValue)? 5:4;				// Taste gesperrt
			}
		else if(uIndex&0x40000000)
			{
			pData->uDropState  = (uFlags&MK_LBUTTON);		// ber Dropdown-Taste
			pItem->wInfoFlags |= (uValue)? 7:6;
			pData->cMoveDrop   = 1;
			}
		else{
			pItem->wInfoFlags |= (uValue || uFlags)? 3:2;	// Hervorgehoben
			}

		pData->uMoveItem = (WORD)uIndex;
		InvalidateRect(pData->hWnd,&pItem->sRect,FALSE);
		}


	pData->cMoveFlags	= (char)uFlags;


}


//*****************************************************************************
//*
//*		ToolbarChangeBitmap
//*
//*****************************************************************************
//	ndert das Bitmap einer Taste
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	iBitmap		: Neue Bitmap-Nummer
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarChangeBitmap(ToolbarData *pData,unsigned uIndex,int iBitmap)
{
LPTBITEM		pItem;


	if(uIndex>=pData->uItemsCnt)return FALSE;

	   pItem = pData->pItems[uIndex];
	if(pItem->iBitmap==iBitmap)return TRUE;


	pItem->iBitmap		= iBitmap;
	pItem->wInfoFlags  |= TBIF_REASSEMBLE;
	pItem->wInfoFlags  &=~TBIF_SIZE_OK;
	pData->cReCalc	   |= 0x01;

	UpdateFrom(pData,uIndex);



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarTagDraw
//*
//*****************************************************************************
//	Sucht den ausgewhlten Eintrag einer Gruppe
//	pData		: Zeiger auf die Fensterdaten
//	hDc			: Device-Context zum Zeichnen
//	iPosX/Y		: Position des TAG's (vom Textfeld ohne Rand)
//	pText		: Ist der Text fr das TAG
//	Ergibt die Breite und Hhe (nur Text ohne Rand)
static UINT ToolbarTagDraw(ToolbarData *pData,HDC hDc,int iPosX,int iPosY,LPCWSTR pText)
{
unsigned	uColor;
unsigned	uSizeX;
unsigned	uSizeY;
unsigned	uSize;
size_t		uLen;


	uColor = pData->aColors[TBC_TAGBK];

	SetBkMode    (hDc,(uColor==TB_NOFRAME)? TRANSPARENT:OPAQUE);
	SetBkColor   (hDc, uColor);
	SetTextColor (hDc, pData->aColors[TBC_TAGTEXT]);

	uLen	= (int)wcslen(pText);								// Textfeld zeichnen
	uSize	= TabbedTextOutW(hDc,iPosX,iPosY,pText,(int)uLen,0,NULL,0);
	uSizeX	= LOWORD(uSize);
	uSizeY	= HIWORD(uSize);

	if(uColor!=TB_NOFRAME)
		{
		DrawRectangle(hDc,iPosX+uSizeX,iPosY,1,uSizeY);		// Rechts eine extra Pixelreihe
		}

	if(pData->aColors[TBC_TAGFRAME]!=TB_NOFRAME)			// Rahmen zeichnen
		{
		uSizeX += 1;
		SetBkColor   (hDc,pData->aColors[TBC_TAGFRAME]);
		DrawRectangle(hDc,iPosX,iPosY-1     ,uSizeX,1);
		DrawRectangle(hDc,iPosX,iPosY+uSizeY,uSizeX,1);
		DrawRectangle(hDc,iPosX-1     ,iPosY,1,uSizeY);
		DrawRectangle(hDc,iPosX+uSizeX,iPosY,1,uSizeY);
		}


return uSize+1;
}

//*****************************************************************************
//*
//*		ToolbarTagDelete
//*
//*****************************************************************************
//	Ein TAG lschen
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist die ID des Eintrages
//	Ergibt den TRUE wenn der TAG gelscht wurde.
static int ToolbarTagDelete(ToolbarData *pData,UINT uIndex)
{
LPTBTAGDATA pLast;
LPTBTAGDATA pTag;


		pTag = pData->pTagChain;
	if(!pTag)
		{
		return FALSE;
		}

	pLast = NULL;

	if(HIWORD(uIndex)==0x1222)								// Via ID lschen	
		{
		uIndex &= 0xFFFF;

		for(;;)
			{
			if(!pTag)return FALSE;

			if(pTag->sItem.ucItemId==uIndex)break;
			pLast = pTag;
			pTag  = pTag->pNext;
			}
		}
	else{													// Via Index lschen
		for(;;)
			{
			if(!pTag)return FALSE;

			if(pTag->sItem.uIndex==uIndex)break;
			pLast = pTag;
			pTag  = pTag->pNext;
			}
		}


	if(pLast)												// Aus der Kette entfernen
			pLast->pNext		= pTag->pNext;
	else	pData->pTagChain	= pTag->pNext;


	delete pTag;


	UpdateView(pData);



return TRUE;
}


//*****************************************************************************
//*
//*		ToolbarTagInsert
//*
//*****************************************************************************
//	Ein neues TAG einfgen 
//	pData		: Zeiger auf die Fensterdaten
//	pTagItem	: Daten ese Eintrages 
//	pText		: Text fr das TAG
//	bUnicode	: Ist der Text in Unicode
//	Ergibt den TRUE wenn der TAG eingefgt wurde.
static int ToolbarTagInsert(ToolbarData *pData,LPTBTAGITEM pTagItem,LPWSTR pText,BOOL bUnicode)
{
LPTBTAGDATA pTag;
int			iLen;

	
	if(!bUnicode)
			iLen = (int)strlen((char*)pText);
	else	iLen = (int)wcslen(pText);

		pTag = (LPTBTAGDATA)new char[sizeof(TBTAGDATA)+iLen*sizeof(WCHAR)];
	if(!pTag)return FALSE;


	if(bUnicode)											// Text kopieren
		{
		memcpy(pTag->cText,pText,(iLen+1)*sizeof(WCHAR));
		}
	else{
		MultiByteToWideChar(CP_ACP,0,(char*)pText,iLen+1,pTag->cText,iLen+3);
		}

	
	memcpy(&pTag->sItem,pTagItem,sizeof(TBTAGITEM));

	if(pTag->sItem.ucFlags&TBTF_USEID)						// Den Index in eine ID umwandeln
		{
		pTag->sItem.uIndex = TB_USE_INDEX(pTag->sItem.uIndex);
		}

	if(pTag->sItem.ucFlags&TBTF_DELEXIST)					// Einen vorhanden Eintrag lschen
		{
		ToolbarTagDelete(pData,pTag->sItem.uIndex);
		}

	pTag->sItem.ucFlags |= TBTF_RECALC;

	pTag->pNext		 = pData->pTagChain;
	pData->pTagChain = pTag;

	UpdateView(pData);


return TRUE;		
}

//*****************************************************************************
//*
//*		ToolbarGetGroupIndex
//*
//*****************************************************************************
//	Sucht den ausgewhlten Eintrag einer Gruppe
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	bCheck		: Check setzen oder lschen
//	Ergibt den Index des ausgewhlten Eintrages oder 0xFFFFFFFF
static UINT ToolbarGetGroupIndex(ToolbarData *pData,unsigned uIndex)
{
LPTBITEM		pItem;
unsigned		uNow;




	for(uNow=uIndex;uNow>0;uNow--)							// Suche vorher
		{
		     pItem = pData->pItems[uNow];
		if(!(pItem->fsStyle&BTNS_GROUP))
			{
			break;
			}

		if(pItem->fsState&TBSTATE_CHECKED)
			{
			return uNow;
			}

		if(pItem->fsStyle&BTNS_GROUPSTART)
			{
			break;
			}
		}


	for(uNow=uIndex+1;uNow<pData->uItemsCnt;uNow++)			// Suche nachher
		{
		     pItem = pData->pItems[uNow];
		if(!(pItem->fsStyle&BTNS_GROUP))
			{
			break;
			}

		if((pItem->fsStyle&BTNS_GROUPSTART) && uNow>uIndex)
			{
			break;
			}

		if(pItem->fsState&TBSTATE_CHECKED)
			{
			return uNow;
			}
		}


return 0xFFFFFFFF;
}

//*****************************************************************************
//*
//*		ToolbarCheckButton
//*
//*****************************************************************************
//	Ob eine Taste gedrckt ist
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	bCheck		: Check setzen oder lschen
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarCheckButton(ToolbarData *pData,unsigned uIndex,BOOL bCheck)
{
LPTBITEM		pItem;
UINT			uOld;
BOOL			bNow;


	if(uIndex>=pData->uItemsCnt)return FALSE;

	pItem = pData->pItems[uIndex];

	if((pItem->fsStyle&BTNS_GROUP) && bCheck)				// Eintrag in Gruppe setzen
		{
		   uOld = ToolbarGetGroupIndex(pData,uIndex);
		if(uOld==uIndex)return FALSE;

	    if(uOld<pData->uItemsCnt)
			{
			pData->pItems[uOld]->fsState &= ~TBSTATE_CHECKED;
			UpdateRect(pData,uOld);
			}
		}


	bNow = pItem->fsState&TBSTATE_CHECKED;

	if( bNow &&  bCheck)return TRUE;
	if(!bNow && !bCheck)return TRUE;


	if(bCheck)
		{
		pItem->fsState |= TBSTATE_CHECKED;
		}
	else{
		pItem->fsState &=~TBSTATE_CHECKED;
		}

	UpdateRect(pData,uIndex);



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarMarkeButton
//*
//*****************************************************************************
//	Eine Taste ausblenden
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	bMark		: Marke setzen oder lschen
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarMarkeButton(ToolbarData *pData,unsigned uIndex,BOOL bMark)
{
LPTBITEM		pItem;
BOOL			bNow;


	if(uIndex>=pData->uItemsCnt)return FALSE;

	pItem	= pData->pItems[uIndex];
	bNow	= pItem->fsState&TBSTATE_MARKED;

	if( bNow &&  bMark)return TRUE;
	if(!bNow && !bMark)return TRUE;

	if(bMark)
		{
		pItem->fsState |= TBSTATE_MARKED;
		}
	else{
		pItem->fsState &=~TBSTATE_MARKED;
		}


return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarHideButton
//*
//*****************************************************************************
//	Eine Taste ausblenden
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	bHide		: Hide setzen oder lschen
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarHideButton(ToolbarData *pData,unsigned uIndex,BOOL bHide)
{
LPTBITEM		pItem;
BOOL			bNow;


	if(uIndex>=pData->uItemsCnt)return FALSE;

	pItem	= pData->pItems[uIndex];
	bNow	= pItem->fsState&TBSTATE_HIDDEN;

	if( bNow &&  bHide)return TRUE;
	if(!bNow && !bHide)return TRUE;

	if(bHide)
		{
		pItem->fsState |= TBSTATE_HIDDEN;
		}
	else{
		pItem->fsState &=~TBSTATE_HIDDEN;
		}

	pData->cReCalc |= 0x01;

	if(!bHide)
		{
		UpdateRect(pData,uIndex);
		}
	else if(uIndex)
		{
		UpdateRect(pData,uIndex-1);
		}
	else{
		UpdateView(pData);
		}


return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarDisabaleButton
//*
//*****************************************************************************
//	Eine Taste sperren
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	bDisable	: setzen oder lschen
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarDisabaleButton(ToolbarData *pData,unsigned uIndex,BOOL bDisable)
{
LPTBITEM		pItem;
BOOL			bNow;


	if(uIndex>=pData->uItemsCnt)return FALSE;

	pItem	= pData->pItems[uIndex];
	bNow	= pItem->fsState&TBSTATE_INDETERMINATE;

	if( bNow &&  bDisable)return TRUE;
	if(!bNow && !bDisable)return TRUE;

	if(bDisable)
		{
		pItem->fsState |= TBSTATE_INDETERMINATE;
		}
	else{
		pItem->fsState &=~TBSTATE_INDETERMINATE;
		}

	UpdateRect(pData,uIndex);



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarPressButton
//*
//*****************************************************************************
//	Eine Taste drcken
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	bDisable	: setzen oder lschen
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarPressButton(ToolbarData *pData,unsigned uIndex,BOOL bPress)
{
LPTBITEM		pItem;
BOOL			bNow;


	if(uIndex>=pData->uItemsCnt)return FALSE;

	pItem	= pData->pItems[uIndex];
	bNow	= pItem->fsState&TBSTATE_PRESSED;

	if( bNow &&  bPress)return TRUE;
	if(!bNow && !bPress)return TRUE;

	if(bPress)
		{
		pItem->fsState |= TBSTATE_PRESSED;
		}
	else{
		pItem->fsState &=~TBSTATE_PRESSED;
		}

	UpdateRect(pData,uIndex);



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarEnableButton
//*
//*****************************************************************************
//	Eine Taste freigeben
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	bEnable		: Enable setzen oder lschen
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarEnableButton(ToolbarData *pData,unsigned uIndex,BOOL bEnable)
{
LPTBITEM		pItem;
BOOL			bNow;


	if(uIndex>=pData->uItemsCnt)return FALSE;

	pItem	= pData->pItems[uIndex];
	bNow	= pItem->fsState&TBSTATE_ENABLED;

	if( bNow &&  bEnable)return TRUE;
	if(!bNow && !bEnable)return TRUE;

	if(bEnable)
		{
		pItem->fsState |= TBSTATE_ENABLED;
		}
	else{
		pItem->fsState &=~TBSTATE_ENABLED;
		}

	UpdateRect(pData,uIndex);



return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarGetButtonText
//*
//*****************************************************************************
//	Text zu einer Taste abfragen
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	pBuffer		: Hier wird der Text gespeichert
//	bUnicode	: Den Text ist in Unicode
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarGetButtonText(ToolbarData *pData,unsigned uIndex,LPWSTR pBuffer,BOOL bUnicode)
{
LPTBITEM		pItem;
LPCWSTR			pText;
int				iPos;
int				iLen;


	if(uIndex>=pData->uItemsCnt || !pBuffer)
		{
		return -1;
		}

	pItem	= pData->pItems[uIndex];
	iPos	= (INT)pItem->iString;
	pText	= (U(iPos)<pData->uStringCnt)? pData->pStrings[iPos]:L"\0";
	iLen	= *pText;

	if(bUnicode)
		{
		wcscpy(pBuffer,pText+1);
		}
	else{
		iLen  = WideCharToMultiByte(CP_ACP,0,pText+1,iLen,(char*)pBuffer,0xFFFF,NULL,NULL);
		}


return iLen;
}



//*****************************************************************************
//*
//*		ToolbarFindString
//*
//*****************************************************************************
//	Such nach einem Text
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index des Textes bei dem die Suche anfngt
//	uCount		: Ist die Anzahl der Texte die durchsucht werden sollen
//	pText		: Ist der Text nach dem gesucht werden soll
//	bUnicode	: Den Text ist in Unicode
//	Ergibt den Index des Textes wenn erfolgreich, bzw. -1 wenn nicht.
static BOOL ToolbarFindString(ToolbarData *pData,UINT uIndex,UINT uCount,LPARAM lParam,BOOL bUnicode)
{
WCHAR		aTemp[256];
LPCWSTR		pText;
LPCWSTR		pWide;
LPBYTE		pAnsi;
int			iLen;



	if(uIndex>=pData->uStringCnt || lParam==0 || uCount==0)
		{
		return -1;
		}


	if(!bUnicode)											// Ist es ein Ansi-Text
		{
		pAnsi = (LPBYTE)lParam;

		for(iLen=0;;iLen++)
			{
			if(iLen>=255)return -1;
			aTemp[iLen] = pAnsi[iLen];
			if(!pAnsi[iLen])break;
			}

		pWide = aTemp;
		}
	else{													// Unicode Text
		pWide = (LPCWSTR)lParam;
		iLen  = (INT)wcslen(pWide);
		}


	for(;uIndex<pData->uStringCnt;uIndex++)					// Suche nach dem Text
		{
		pText = pData->pStrings[uIndex];

		if(*pText==iLen && !memcmp(pWide,pText+1,sizeof(WCHAR)*iLen))
			{
			return uIndex;
			}

			uCount--;
		if(!uCount)break;
		}


return -1;
}

//*****************************************************************************
//*
//*		ToolbarGetString
//*
//*****************************************************************************
//	Einen Text abfragen
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index des Textes
//	iSize		: Ist die Gre des Puffers
//	lParam		: Hier wird der Text gespeichert
//	bUnicode	: Den Text ist in Unicode
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarGetString(ToolbarData *pData,unsigned uIndex,int iSize,LPARAM lParam,BOOL bUnicode)
{
LPCWSTR			pText;
int				iLen;


	if(uIndex>=pData->uStringCnt || !lParam || iSize<=0)
		{
		return -1;
		}

	pText	= pData->pStrings[uIndex];
	iLen	= *pText;

	if(bUnicode)
		{
		if(iLen>=iSize)iLen=iSize-1;
		wcsncpy((LPWSTR)lParam,pText+1,iSize);
		((LPWSTR)lParam)[iSize] = 0;
		}
	else{
		iLen  = WideCharToMultiByte(CP_ACP,0,pText+1,iLen,(char*)lParam,iSize,NULL,NULL);
		}


return iLen;
}

//*****************************************************************************
//*
//*		ToolbarSetButtonText
//*
//*****************************************************************************
//	ndert den Text einer Taste
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	lParam		: Ist der Zeiger auf den Text
//	bUnicode	: Ist der Text Unicode
//	Ergibt die Sting-ID oder -1 bei einem Fehler
static int ToolbarSetButtonText(ToolbarData *pData,UINT uIndex,LPARAM lParam,BOOL bUnicode)
{
LPTBITEM		pItem;
int				iPos;


	if(uIndex>=pData->uItemsCnt)return -1;


	   iPos = ToolbarFindString(pData,0,0xFFFF,lParam,bUnicode);
	if(iPos<0)
		{
		iPos = ToolbarAddStrings(pData,lParam,TB_SINGLETEXT,bUnicode);
		}

	pItem = pData->pItems[uIndex];

	if(iPos!=pItem->iString)
		{
		pItem->wInfoFlags |= TBIF_REASSEMBLE;
		pData->cReCalc    |= 0x03;
		pItem->iString	   = iPos;
		UpdateView(pData);
		}


return iPos;
}



//*****************************************************************************
//*
//*		ToolbarSetButtonInfo
//*
//*****************************************************************************
//	ndert diverse Optionen einer Taste
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	wParam		: Index der Taste fr TBIF_BYINDEX
//	pInfo		: Zeigher auf die neuen Tasteninfos
//	bUnicode	: Ist der Text Unicode
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarSetButtonInfo(ToolbarData *pData,UINT wParam,LPTBBUTTONINFOEX pInfo,BOOL bUnicode)
{
unsigned		uIndex;
LPTBITEM		pItem;
UINT			uMask;
UINT			uVal;


	if(!pInfo)return FALSE;

	uMask	= pInfo->dwMask;

	if(pInfo->cbSize<sizeof(TBBUTTONINFO))
		{
		return FALSE;
		}

	if(pInfo->cbSize<sizeof(TBBUTTONINFOEX))
		{
		uMask &= ~TBIF_FLAGS;
		}

	if((uMask&TBIF_BYINDEX) || !TB_SPECIAL(wParam))			// Daten via Index holen
		{
		uIndex = wParam;
		}
	else{
		uIndex = ToolbarButtonIndex(pData,U(wParam));
		}

	if(uIndex>=pData->uItemsCnt)
		{
		return FALSE;
		}


	pItem = pData->pItems[uIndex];

	if(uMask&TBIF_COMMAND)
		{
		pItem->idCommand = pInfo->idCommand;
		}

	if(uMask&TBIF_LPARAM)
		{
		ToolbarProc(pData->hWnd,TB_SETPARAM,uIndex,pInfo->lParam);
		}

	if(uMask&TBIF_IMAGE)
		{
		ToolbarProc(pData->hWnd,TB_SETBITMAPID,uIndex,pInfo->iImage);
		}

	if(uMask&(TBIF_STATE|TBIF_STYLE))
		{
		uVal	= MAKELONG(pInfo->fsStyle,pInfo->fsState);

		if(uMask&TBIF_STYLE)uVal|=0x0000FF00;
		if(uMask&TBIF_STATE)uVal|=0xFF000000;

		ToolbarProc(pData->hWnd,TB_SETSTYLESTATE,uIndex,uVal);
		}

	if(uMask&TBIF_FLAGS)
		{
		ToolbarProc(pData->hWnd,TB_SETFLAGS,uIndex,MAKELONG(pInfo->wFlags,pInfo->wMask));
		}

	if(uMask&TBIF_SIZE)
		{
		ToolbarProc(pData->hWnd,TB_SETTEXTSIZE,uIndex,pInfo->cx);
		}

	if(uMask&TBIF_STRINGID)
		{
		ToolbarProc(pData->hWnd,TB_SETSTRINGID,uIndex,(int)pInfo->pszText);
		}
	else if(uMask&TBIF_TEXT)
		{
		ToolbarSetButtonText(pData,uIndex,(LPARAM)pInfo->pszText,bUnicode);
		}




return TRUE;
}

//*****************************************************************************
//*
//*		ToolbarGetButtonInfo
//*
//*****************************************************************************
//	Holt diverse Optionen einer Taste
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Index der Taste
//	wParam		: Index der Taste fr TBIF_BYINDEX
//	pInfo		: Zeigher auf die neuen Tasteninfos
//	bUnicode	: Ist der Text Unicode
//	Ergibt TRUE wenn erfolgreich.
static BOOL ToolbarGetButtonInfo(ToolbarData *pData,UINT wParam,LPTBBUTTONINFOEX pInfo,BOOL bUnicode)
{
unsigned		uIndex;
LPTBITEM		pItem;
UINT			uMask;


	if(!pInfo)return -1;

	uMask	= pInfo->dwMask;

	if(pInfo->cbSize<sizeof(TBBUTTONINFO))
		{
		return -1;
		}

	if(pInfo->cbSize<sizeof(TBBUTTONINFOEX))
		{
		uMask &= ~TBIF_FLAGS;
		}

	if((uMask&TBIF_BYINDEX) || !TB_SPECIAL(wParam))			// Daten via Index holen
		{
		uIndex = wParam;
		}
	else{
		uIndex = ToolbarButtonIndex(pData,U(wParam));
		}

	if(uIndex>=pData->uItemsCnt)
		{
		return FALSE;
		}


	pItem = pData->pItems[uIndex];

	if(uMask&TBIF_COMMAND)
		{
		pInfo->idCommand = pItem->idCommand;
		}

	if(uMask&TBIF_LPARAM)
		{
		pInfo->lParam = pItem->dwData;
		}

	if(uMask&TBIF_IMAGE)
		{
		pInfo->iImage = pItem->iBitmap;
		}

	if(uMask&TBIF_STATE)
		{
		pInfo->fsState = pItem->fsState;
		}

	if(uMask&TBIF_STYLE)
		{
		pInfo->fsStyle = pItem->fsStyle;
		}

	if(uMask&TBIF_FLAGS)
		{
		pInfo->wFlags = pItem->wFlags;
		}

	if(uMask&TBIF_SIZE)
		{
		pInfo->cx = (WORD)(pItem->sRect.right-pItem->sRect.left);
		}

	if(uMask&TBIF_STRINGID)
		{
		pInfo->pszText = (LPTSTR)pItem->iString;
		}
	else if((uMask&TBIF_TEXT) && pInfo->cchText>0)
		{
		ToolbarGetString(pData,(int)pItem->iString,pInfo->cchText,(LPARAM)pInfo->pszText,bUnicode);
		}


return uIndex;
}


//*****************************************************************************
//*
//*		ToolbarDropdown
//*
//*****************************************************************************
//	Ein Dropdown-Event wurde ausgelst
//	pData		: Zeiger auf die Fensterdaten
//	uIndex		: Ist der Index der Taste
static void ToolbarDropdown(ToolbarData *pData,unsigned uIndex)
{
NMTOOLBAR		sInfo;
LPTBITEM		pItem;


	pItem	= pData->pItems[uIndex];


	sInfo.hdr.code	= TBN_DROPDOWN;
	sInfo.iItem		= uIndex;
	sInfo.pszText	= NULL;
	sInfo.cchText	= 0;

	memcpy(&sInfo.tbButton,pItem,sizeof(sInfo.tbButton));

	UNLOCK(pData);
	SendNotify(pData,&sInfo.hdr);
	LOCK(pData);


}

//*****************************************************************************
//*
//*		ToolbarMarkHitTest
//*
//*****************************************************************************
//	Hit-Test fr einfge Marke
//	pData		: Zeiger auf die Fensterdaten
//	iPosX/Y		: Koordinate im Fenster
//	pMark		: Speicher fr das Ergebnis
//	Ergibt 1 wenn eine Taste bei der Koordinate ist
static int ToolbarMarkHitTest(ToolbarData *pData,int iPosX,int iPosY,TBINSERTMARK *pMark)
{
unsigned	uIndex;
LPTBITEM	pItem;
int			iSizeX;
int			iSizeY;



	uIndex = ToolbarHitTest(pData,iPosX,iPosY,0);

	if(uIndex>=pData->uItemsCnt)								// Auf keinem Eintrag
		{
		pMark->iButton	= -1;
		pMark->dwFlags	= TBIMHT_BACKGROUND;
		return 0;
		}

	pMark->iButton	= uIndex;
	pMark->dwFlags	= 0;

	pItem	= pData->pItems[uIndex];
	iSizeX	= pItem->sRect.right -pItem->sRect.left;
	iSizeY	= pItem->sRect.bottom-pItem->sRect.top;

	if(pItem->wInfoFlags&TBIF_PACKED)							// Eintrag liegt ber anderem
		{
		if(iPosY>pItem->sRect.top +iSizeY/2)
		if(iPosX>pItem->sRect.left+iSizeX/4)
		if(iPosX<pItem->sRect.left+iSizeX/4+iSizeX/2)
			{
			return 1;
			}
		}

	if(pItem->wInfoFlags&TBIF_BASE)								// Eintrag liegt unter anderem
		{
		if(iPosY<pItem->sRect.top +iSizeY/2)
		if(iPosX>pItem->sRect.left+iSizeX/4)
		if(iPosX<pItem->sRect.left+iSizeX/4+iSizeX/2)
			{
			pMark->dwFlags	= TBIMHT_AFTER;
			return 1;
			}
		}

	if(iPosX>pItem->sRect.left+iSizeX/2)
		{
		pMark->dwFlags	= TBIMHT_AFTER;
		}



return 1;
}

//*****************************************************************************
//*
//*		ToolbarProc
//*
//*****************************************************************************
//	Ist die Fensterfunktion fr das Toolbar Fenster
static LRESULT CALLBACK ToolbarProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
LPTBITEM		pItem;
ToolbarData    *pData;
TBINSERTMARK   *pMark;
POINT			sPoint;
HWND			hSend1;
HWND			hSend2;
unsigned		uChange;
unsigned		uIndex;
unsigned		uItem;
unsigned		uHigh;
unsigned		uFlag;
unsigned		uMask;
unsigned		uVal;
unsigned		uOld;
unsigned		uIdx;
LPARAM			lRet;
int				iPos;
HDC				hDc;




	switch(uMsg)
		{
	case WM_NCCREATE:											// Den NC-Teil erstellen

				pData= new ToolbarData;
			if(!pData)return -1;

			GlobalInit();

			memset(pData,0,sizeof(ToolbarData));

			SYNCH_CREATE(pData);
			SetWindowLongPtr(hWnd,0,(LONG_PTR)pData);

			return 1;

	case WM_CREATE:												// Das Fenster erzeugen

				pData = GetHandle(hWnd);
			if(!pData)return -1;

			pData->cIsEnabled		= (char)IsWindowEnabled(hWnd);
			pData->uStyle			= GetWindowLong(hWnd,GWL_STYLE);
			pData->hWnd				= hWnd;
			pData->cReCalc			= 0x07;
			pData->uButtonZoom		= 256;
			pData->iMinWidthX		= -1;
			pData->iMaxWidthX		= -1;
			pData->iButtonSizeX 	= 23;
			pData->iButtonSizeY 	= 22;
			pData->iBitmapSizeX 	= 16;
			pData->iBitmapSizeY 	= 16;
			pData->iPaddingX		= DEFPAD_CX;
			pData->iPaddingY		= DEFPAD_CY;
			pData->iGabX			= DEFGAP_CX;
			pData->iGabY			= DEFGAP_CY;
			pData->aColorChange[0]	= RGB(255,0,255);
			pData->aColorChange[1]	= RGB(128,0,128);
			pData->uMoveItem		= 0xFFFFFFFF;
			pData->uMarkItem		= 0xFFFFFFFF;
			pData->uStructSize		= sizeof(TBBUTTON);
			pData->hParent			= GetParent(hWnd);

			if(pOpenThemeData)									// Soll ein Thema angezeigt werden
					pData->hTheme = pOpenThemeData(hWnd,L"TREEVIEW");
			else	pData->hTheme = NULL;

			hDc					= GetDC(hWnd);
			pData->cBitPixels	= GetDeviceCaps(hDc,BITSPIXEL);
			pData->cFullColor	= pData->cBitPixels>8;
			pData->cAlign		=(pData->uStyle&CCS_VERT)? 1:0;

			SetTimer(hWnd,ID_UPDATE,1000,NULL);
			ReleaseDC(hWnd,hDc);
			UpdateColors(pData);
			UpdateFont  (pData);

			return 0;

	case WM_DESTROY:											// Das Fenster zerstren

			pData = GetHandle(hWnd);

			LOCK(pData);

			for(iPos=pData->uItemsCnt;iPos>0;)					// Tasten lschen
				{
				iPos--;
				ToolbarDelItem(pData,iPos);
				}

			for(iPos=pData->uResourceCnt;iPos>0;)				// Resourcen lschen
				{
				iPos--;
				ToolbarDelRecource(pData,iPos);
				}

			for(iPos=pData->uStringCnt;iPos>0;)					// Strings lschen
				{
				iPos--;
				delete pData->pStrings[iPos];
				}

			while(pData->uItemsCnt>0)							// Tasten lschen
				{
				ToolbarDelItem(pData,pData->uItemsCnt-1);
				}

			if(pData->hThemeBt    ){pCloseThemeData(pData->hThemeBt);pData->hThemeBt    =0;}
			if(pData->hTheme      ){pCloseThemeData(pData->hTheme  );pData->hTheme      =0;}
			if(pData->pItems      ){delete pData->pItems			;pData->pItems      =0;}
			if(pData->pStrings    ){delete pData->pStrings			;pData->pStrings    =0;}
			if(pData->pResource   ){delete pData->pResource         ;pData->pResource   =0;}

			UNLOCK(pData);

			return 0;

	case WM_NCDESTROY:											// Das Fenster zerstren

			pData = GetHandle(hWnd);

			if(pData->pTagChain)
				{
				TBTAGDATA *pNext;
				TBTAGDATA *pTag;

				pTag=pData->pTagChain;

				for(;pTag;pTag=pNext)
					{
					pNext = pTag->pNext;
					delete pTag;
					}
				}

			SetWindowLongPtr(hWnd,0,0);
			SYNCH_DELETE(pData);
			memset(pData,0,sizeof(ToolbarData));
			delete pData;

			GlobalDeinit();

			return 0;

	case WM_SIZE:												// Die Fenstergre wurde verndert

			pData = GetHandle(hWnd);
			uFlag = 0;

			LOCK(pData);

			   uVal = LOWORD(lParam);
			if(uVal && uVal!=pData->uSizeX)
				{
				if(uVal>pData->uSizeX)							// Hat sich die Hhe vergrert
					{
					RECT	sRect;

					GetClientRect(hWnd,&sRect);
					sRect.left  	= uVal;
					sRect.right 	= pData->uSizeX;
					sRect.top	 	= 0;
					sRect.bottom	= pData->uSizeY;
					InvalidateRect(hWnd,&sRect,FALSE);
					}

				pData->uSizeX = uVal;
				}

			   uVal = HIWORD(lParam);
			if(uVal && uVal!=pData->uSizeY)
				{
				if(uVal>pData->uSizeY)							// Hat sich die Hhe vergrert
					{
					RECT	sRect;

					GetClientRect (hWnd,&sRect);
					uFlag			 = 1;
					sRect.bottom	 = uVal;
					sRect.top		 = pData->uSizeY;
					InvalidateRect(hWnd,&sRect,FALSE);
					}

				pData->uSizeY =  uVal;
				}


			UNLOCK(pData);

			return 0;

	case WM_ENABLE:												// Das Fenster freischalten oder sperren

			   pData = GetHandle(hWnd);
			if(pData->cIsEnabled!=((U(wParam))? 1:0))
				{
				uVal  = GetWindowLong(hWnd,GWL_STYLE);
				uVal &= (0xFFFF0000&~WS_DISABLED);
				uVal |= (0x0000FFFF| WS_DISABLED)&(pData->uStyle^WS_DISABLED);
				SetWindowLong(hWnd,GWL_STYLE,uVal);
				}

			return 0;

	case WM_SETFOCUS:											// Das Fenster bekommt den Fokus

				pData = GetHandle(hWnd);
			if(!pData->cHasFocus)
				{
				NMHDR	sNotify;
				sNotify.code = NM_SETFOCUS;
				SendNotify(pData,&sNotify);
				}

			return 0;

	case WM_KILLFOCUS:											// Das Fenster bekommt den Fokus

			   pData = GetHandle(hWnd);
			if(pData->cHasFocus)
				{
				NMHDR	sNotify;
				sNotify.code = NM_KILLFOCUS;
				SendNotify(pData,&sNotify);
				}

			return 0;

	case WM_TIMER:												// Zyklischer Timer
	case TB_TIMER:

			if(U(wParam)==ID_UPDATE)break;

			pData = GetHandle(hWnd);

			if(pData->uMoveItem==0xFFFFFFFF)
				{
				return 0;
				}

			LOCK(pData);

			GetCursorPos(&sPoint);
			ScreenToClient(hWnd,&sPoint);

			uFlag = 0;

			if(GetAsyncKeyState(VK_LBUTTON)&0x8000)uFlag|=MK_LBUTTON;
			if(GetAsyncKeyState(VK_RBUTTON)&0x8000)uFlag|=MK_RBUTTON;

			ToolbarMouseMove(pData,sPoint.x,sPoint.y,uFlag);

			UNLOCK(pData);

			return 0;

	case WM_MOUSELEAVE:											// Die Maus hat das Fenster verlassen

			GetCursorPos(&sPoint);
			ScreenToClient(hWnd,&sPoint);
			lParam = MAKELONG(sPoint.x,sPoint.y);
			wParam = 0x00000000;

	case WM_MOUSEMOVE:											// Gab es eine Mausbewegung

			pData = GetHandle(hWnd);

			LOCK(pData);

			ToolbarMouseMove(pData,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam),U(wParam));

			if(pData->uMoveItem!=0xFFFFFFFF && uMsg!=WM_MOUSELEAVE)
				{
				TRACKMOUSEEVENT sEvent;

				sEvent.cbSize		= sizeof(TRACKMOUSEEVENT);
				sEvent.dwFlags		= TME_LEAVE;
				sEvent.hwndTrack	= pData->hWnd;

				_TrackMouseEvent(&sEvent);
				}

			UNLOCK(pData);

			return 0;

	case WM_LBUTTONUP:											// Linken Mausklick aufheben

			pData = GetHandle(hWnd);
			lRet  = -1;

			LOCK(pData);

			if(pData->uMoveItem!=0xFFFFFFFF && !pData->cMoveDrop)
				{
				pItem = pData->pItems[pData->uMoveItem];
				lRet  = MAKELONG(pItem->idCommand,BN_CLICKED);

				if(pItem->wFlags&BTFL_CONTROL)
					{
					UNLOCK(pData);
					return 0;
					}

				if(pItem->fsStyle&TBSTYLE_CHECK)				// Umschalten
					{
					if(pItem->fsStyle&BTNS_GROUP)				// Eintrag in Gruppe setzen
						{
						   uIndex = ToolbarGetGroupIndex(pData,pData->uMoveItem);
						if(uIndex!=pData->uMoveItem && uIndex<pData->uItemsCnt)
							{
							pData->pItems[uIndex]->fsState &= ~TBSTATE_CHECKED;
							UpdateRect(pData,uIndex);
							}

						pItem->fsState |= TBSTATE_CHECKED;
						}
					else{										// Ohne Gruppe
						pItem->fsState ^= TBSTATE_CHECKED;
						}

					pData->cMoveFlags = -1;
					}
				}

			ToolbarMouseMove(pData,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam),U(wParam));

			UNLOCK(pData);

			if(lRet>=0)PostMessageW(pData->hParent,WM_COMMAND,lRet,(LPARAM)hWnd);

			return 0;

	case WM_LBUTTONDOWN:										// Linker Mausklick
	case WM_LBUTTONDBLCLK:

			pData = GetHandle(hWnd);

			LOCK(pData);

			ToolbarMouseMove(pData,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam),U(wParam));

			if(pData->uMoveItem!=0xFFFFFFFF && pData->cMoveDrop)
				{
				   pItem = pData->pItems[pData->uMoveItem];
				if(pItem->wFlags&BTFL_RIGHT_CLICK)
					{
					ToolbarDropdown(pData,pData->uMoveItem);
					}
				}

			UNLOCK(pData);

			return 0;

	case WM_RBUTTONUP:											// Rechten Mausklick aufheben

			pData = GetHandle(hWnd);
			lRet  = -1;

			LOCK(pData);

			ToolbarMouseMove(pData,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam),U(wParam));

			if(pData->uMoveItem!=0xFFFFFFFF && !pData->cMoveDrop)
				{
				   pItem = pData->pItems[pData->uMoveItem];
				if(pItem->wFlags&BTFL_RIGHT_CLICK)
					{
					lRet = MAKELONG(pItem->idCommand+1,BN_CLICKED);
					}
				}

			UNLOCK(pData);

			if(lRet>=0)PostMessageW(pData->hParent,WM_COMMAND,lRet,(LPARAM)hWnd);
			if(pData->uMoveItem!=0xFFFFFFFF)return 0;

			break;

	case WM_RBUTTONDOWN:										// Rechtermausklick
	case WM_RBUTTONDBLCLK:

			pData = GetHandle(hWnd);

			LOCK(pData);

			ToolbarMouseMove(pData,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam),U(wParam));

			UNLOCK(pData);

			break;

	case WM_KEYDOWN:											// Tastendruck

			return 0;

	case WM_KEYUP:												// Tastendruck

			return 0;

	case WM_DRAWITEM:											// Weietleiten von OwnerDraw des Headers

			if(wParam==1)
				{
				pData = GetHandle(hWnd);
				return SendMessage(GetParent(hWnd),uMsg,GetWindowLong(hWnd,GWL_ID),lParam);
				}

			return 0;

	case WM_SYSCOLORCHANGE:										// Wurden die Systemfarben verndert

			pData	= GetHandle(hWnd);

			LOCK(pData);

			if(pData->hTheme)
				{
				pCloseThemeData(pData->hTheme);
				pData->hTheme   = NULL;
				}

			if(pData->hThemeBt)
				{
				pCloseThemeData(pData->hThemeBt);
				pData->hThemeBt = NULL;
				}

			if(pOpenThemeData)
				{
				pData->hTheme   = pOpenThemeData(hWnd,L"TREEVIEW");
				pData->cGlyphOk = (char)((pData->hTheme)? 1:0);
				}

			pData->cReCalc = 0x07;
			UpdateColors(pData);
			UpdateView(pData);
			UNLOCK(pData);

			return 0;

	case WM_GETFONT:											// Aktuellen Font abfragen

			pData	= GetHandle(hWnd);
			return (LRESULT)pData->hFontN;

	case WM_SETFONT:											// Einen neuen Font zuweisen

			pData	= GetHandle(hWnd);

			LOCK(pData);

			pData->hFontN=(HFONT)wParam;
			if(UpdateFont(pData))UpdateView(pData);

			UNLOCK(pData);

			return 0;

	case WM_STYLECHANGED:										// Hat sich der Fensterstil gendert

			if(wParam==GWL_STYLE)
				{
				pData	= GetHandle(hWnd);
				lParam	= ((STYLESTRUCT*)lParam)->styleNew;

				LOCK(pData);
				uChange		   = U(lParam)^pData->uStyle;
				pData->uStyle  = U(lParam);

				if(uChange&WS_DISABLED)
					{
					pData->cIsEnabled=(char)((pData->uStyle&WS_DISABLED)? 0:1);
					UpdateView(pData);
					}

				if(uChange&CCS_VERT)
					{
					pData->cAlign	= (lParam&CCS_VERT)? 1:0;
					pData->cReCalc |= 0x03;
					UpdateView(pData);
					}

				if(uChange&(TBSTYLE_FLAT|TBSTYLE_PACKED))
					{
					pData->cReCalc  = 0x07;
					UpdateView(pData);
					}

				UNLOCK(pData);
				}

			return 0;

	case WM_PAINT:												// Das Fenster zeichnen

			if(wParam)
				{
				ToolbarDraw(hWnd,(HDC)wParam,NULL);
				}
			else{
				HANDLE		hBufferedPaint;
				HDC			hDcBuffer;
				PAINTSTRUCT sPaint;

				hDc = BeginPaint(hWnd,&sPaint);

				if(pBeginBufferedPt && pEndBufferedPt && 0)		// Gepuffert zeichnen
					{
					hBufferedPaint = pBeginBufferedPt(hDc,&sPaint.rcPaint,BPBF_COMPATIBLEBITMAP,NULL,&hDcBuffer);

					if(hBufferedPaint)
						{
						SetViewportOrgEx(hDcBuffer,sPaint.rcPaint.left,sPaint.rcPaint.top,NULL);

						ToolbarDraw(hWnd,hDcBuffer,&sPaint.rcPaint);

						pEndBufferedPt(hBufferedPaint,TRUE);
						}
					else{
						ToolbarDraw(hWnd,hDc,&sPaint.rcPaint);
						}
					}
				else{
					ToolbarDraw(hWnd,hDc,&sPaint.rcPaint);
					}

				EndPaint(hWnd,&sPaint);
				}

			return 0;

	case WM_COMMAND:											// Kommando Nachrichten

			pData = GetHandle(hWnd);

			LOCK(pData);

			uItem = LOWORD(wParam);
			uHigh = HIWORD(wParam);

			for(uIndex=0;;uIndex++)								// Suche Control
				{
				if(uIndex>=pData->uItemsCnt)
					{
					UNLOCK(pData);
					return 0;
					}

					pItem = pData->pItems[uIndex];
				if((pItem->wFlags&BTFL_CONTROL) && lParam==(LPARAM)pItem->dwData)
					{
					break;
					}
				}

			hSend1	= (pData->hParentEx  )? pData->hParentEx:pData->hParent;
			hSend2	= (pData->hParentCtrl==TB_SENDCB)? (HWND)SendMessage(hWnd,TB_GETSENDWND,0,0):pData->hParentCtrl;
			lRet	= -1;

			if(uHigh==CBN_SELCHANGE)							// Auswahl in ComboBox verndert
				{
				uIdx = (int)SendMessage((HWND)lParam,CB_GETCURSEL,0,0);
				lRet = (int)SendMessage((HWND)lParam,CB_GETITEMDATA,uIdx,0);

				if(lRet<=0)lRet=GetWindowLong((HWND)lParam,GWL_ID)+uIdx;
				}

			UNLOCK(pData);

			if(hSend1)
				{
				           PostMessage(hSend1,WM_COMMAND,wParam,lParam);
				if(lRet>=0)PostMessage(hSend1,WM_COMMAND,lRet  ,lParam);
				}

			if(hSend2)
				{
				           PostMessage(hSend2,WM_COMMAND,wParam,lParam);
				if(lRet>=0)PostMessage(hSend2,WM_COMMAND,lRet  ,lParam);
				}

			return 0;

	case WM_NOTIFY:												// Notify Nachrichten

			pData = GetHandle(hWnd);

			LOCK(pData);

			uItem = LOWORD(wParam);

			for(uIndex=0;;uIndex++)								// Suche Control
				{
				if(uIndex>=pData->uItemsCnt)
					{
					UNLOCK(pData);
					return 0;
					}

					pItem = pData->pItems[uIndex];
				if((pItem->wFlags&BTFL_CONTROL) && lParam==(LPARAM)pItem->dwData)
					{
					break;
					}
				}

			hSend1	= (pData->hParentEx  )? pData->hParentEx:pData->hParent;
			hSend2	= (pData->hParentCtrl==TB_SENDCB)? (HWND)SendMessage(hWnd,TB_GETSENDWND,0,0):pData->hParentCtrl;

			if(hSend1)SendMessage(hSend1,WM_NOTIFY,wParam,lParam);
			if(hSend2)SendMessage(hSend2,WM_NOTIFY,wParam,lParam);

			UNLOCK(pData);

			return 0;

	case TB_GETSENDWND:											// Callback fr Zielfenster

			return 0;

	case TB_GETBITMAPINFO:										// Infos ber die Bitmaps abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarGetBitmapInfo(pData,LOWORD(wParam),HIWORD(wParam),(LPTBBITMAPINFO)lParam);

			UNLOCK(pData);

			return lRet;

	case TB_GETIDLIST:											// Eine Liste mit allen Tasten ID's holen

			pData = GetHandle(hWnd);

			LOCK(pData);

			for(uIndex=0;uIndex<U(wParam);uIndex++)
				{
				if(uIndex>=pData->uItemsCnt)break;
				pItem = pData->pItems[uIndex];
				((LPUINT)lParam)[uIndex] = pItem->idCommand;
				}

			UNLOCK(pData);

			return uIndex;

	case TB_SETCOLOR:											// Farben einstellen

			if(wParam<0 || wParam>=MAX_COLORS)return 0xFFFFFFFF;

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet=pData->aColors[U(wParam)];

			if((COLORREF)lParam==TB_NOCOLOR)					// Standartfarbe einstellen
				{
				if(pData->aColorChanged[U(wParam)])
					{
					pData->aColorChanged[U(wParam)]=0;
					UpdateColors(pData);
					UpdateView  (pData);
					}
				}
			else{
				pData->aColorChanged[U(wParam)]=1;
				pData->aColors      [U(wParam)]=(COLORREF)lParam;

				if(lRet!=lParam)
					{
					UpdateColors(pData);
					UpdateView  (pData);
					}
				}

			UNLOCK(pData);

			return lRet;

	case TB_DELBITMAPS:											// Alle Bitmap-Resourcen lschen

			pData = GetHandle(hWnd);
			lRet  = 0;

			LOCK(pData);

			while(pData->uResourceCnt>0)
				{
				ToolbarDelRecource(pData,pData->uResourceCnt-1);
				pData->uResourceCnt--;
				pData->cReCalc = 0x07;
				lRet++;
				}

			UNLOCK(pData);

			return lRet;

	case TB_ADDBITMAP:											// Eine Bitmap-Resource hinzufgen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet  = ToolbarAddBitmap(pData,U(wParam),(LPTBADDBITMAPEX)lParam);

			UNLOCK(pData);

			return lRet;

	case TB_REPLACEBITMAP:										// Ein Bitmap tauschen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet  = ToolbarReplaceBitmap(pData,U(wParam),(LPTBREPLACEBITMAP)lParam);

			UNLOCK(pData);

			return lRet;

	case TB_SETBITMAPSIZE:										// Gre der einzelnen Bitmaps einstellen

			pData = GetHandle(hWnd);

			LOCK(pData);
			lRet = ToolbarSetBitmapSize(pData,LOWORD(lParam),HIWORD(lParam));
			UNLOCK(pData);

			return lRet;

	case TB_GETBITMAPSIZE:										// Gre der einzelnen Bitmaps abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);
			lRet  = MAKELONG(pData->iBitmapSizeX,pData->iBitmapSizeY);
			UNLOCK(pData);

			return lRet;

	case TB_GETSETOPTION:										// Diverse Optionen einstellen

			pData	= GetHandle(hWnd);
			lRet	= 0;

			switch(U(wParam)&~TBOP_WRITEOPTION)
				{
			case TBOP_CANGECOLOOR1:								// Erste  Farbe fr Farb-Icons
			case TBOP_CANGECOLOOR2:								// Zweite Farbe fr Farb-Icons

				lRet = pData->aColorChange[U(wParam)];

				if((wParam&TBOP_WRITEOPTION) && lParam!=lRet)
					{
					LOCK(pData);

					pData->aColorChange[U(wParam)] = U(lParam);

					if(pData->uItemsCnt>0)
						{
						pData->cReCalc = 0x07;
						UpdateView(pData);
						}

					UNLOCK(pData);
					break;
					}
				}

			return lRet;

	case TB_FINDCONTROL:										// Suche ein Control mit dem Fenster-Handle

			pData = GetHandle(hWnd);

			LOCK(pData);

			for(uIndex=0;;uIndex++)								// Suche Control
				{
				if(uIndex>=pData->uItemsCnt)
					{
					lRet = -1;
					break;
					}

					pItem = pData->pItems[uIndex];
				if((pItem->wFlags&BTFL_CONTROL) && lParam==(LPARAM)pItem->dwData)
					{
					lRet = uIndex;
					break;
					}
				}

			UNLOCK(pData);

			return lRet;

	case TB_GETSTRINGCOUNT:										// Anzahl der Texte abfragen

			pData = GetHandle(hWnd);
			return pData->uStringCnt;

	case TB_GETTOOLTIPS:										// Das Handle des ToolTip-Fensters abfragen

			return 0;

	case TB_SETTOOLTIPS:										// Das Handle fr das ToolTip-Fensters setzen

			return 0;

	case TB_GETPADDING:											// Den Leerum rund um die Bilder abfragen

			pData = GetHandle(hWnd);
			lRet = MAKELONG(pData->iPaddingX,pData->iPaddingY);
			return lRet;

	case TB_SETPADDING:											// Den Leerum rund um die Bilder einstellen

			pData = GetHandle(hWnd);

			LOCK(pData);
			lRet = ToolbarSetPadding(pData,LOWORD(lParam),HIWORD(lParam));
			UNLOCK(pData);

			return lRet;

	case TB_SETLISTGAP:											// Abstand zwischen den Tasten

			pData = GetHandle(hWnd);

			LOCK(pData);
			lRet = ToolbarListGap(pData,U(wParam),U(lParam));
			UNLOCK(pData);

			return lRet;

	case TB_SETBUTTONSIZE:										// Gre der Tasten einstellen

			pData = GetHandle(hWnd);

			LOCK(pData);
			lRet = ToolbarSetButtonSize(pData,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			UNLOCK(pData);

			return lRet;

	case TB_GETBUTTONSIZE:										// Gre der Tasten abfragen

			pData = GetHandle(hWnd);
			lRet  = MAKELONG(pData->iButtonSizeX,pData->iButtonSizeY);

			return lRet;

	case TB_SETBUTTONWIDTH:										// Minimale maximale Breite einer Taste einstellen

			pData = GetHandle(hWnd);

			LOCK(pData);
			lRet = ToolbarSetWidth(pData,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			UNLOCK(pData);

			return lRet;

	case TB_HITTEST:											// Prfen ob eine Taste gerckt ist

			pData = GetHandle(hWnd);

			LOCK(pData);
			lRet = ToolbarHitTest(pData,((LPPOINT)lParam)->x,((LPPOINT)lParam)->y,U(wParam)==0x12345678);
			UNLOCK(pData);

			return lRet;

	case TB_BUTTONSTRUCTSIZE:									// Gre der Tastenstruktur einstellen

			pData = GetHandle(hWnd);
			pData->uStructSize = U(wParam);
			return 0;

	case TB_ADDBUTTONSEX:										// Tasten einfgen mit Flags
	case TB_ADDBUTTONSA:
	case TB_ADDBUTTONSW:

			uIndex	= HIWORD(wParam);
			uHigh	= LOWORD(wParam);

			if(uHigh>=0x0400 || uHigh==0)
				{
				return 0;
				}

			pData = GetHandle(hWnd);
			lRet  = 1;

			LOCK(pData);

			if((uIndex&0xFC00)==0x5000)							// Ist ein Index in wParam
				{
				uIndex &= 0x03FF;
				}
			else if(uIndex)
				{
				lRet   = 0;
				uHigh  = 0;
				}
			else{
				uIndex = pData->uItemsCnt;
				}

			uVal  = (uMsg==TB_ADDBUTTONSEX)? sizeof(TBBUTTONEX):pData->uStructSize;
			uFlag = (uMsg==TB_ADDBUTTONSEX)? 0:TBFL_ZERO_FLAGS;

			for(;uHigh>0;uHigh--,uIndex++,lParam+=sizeof(TBBUTTONEX))
				{
				if(ToolbarAddItem(pData,uIndex,(LPTBBUTTONEX)lParam,uFlag)<0)
					{
					lRet = 0;
					}
				}

			UNLOCK(pData);

			return lRet;

	case TB_INSERTBUTTONEX:										// Eine Taste einfgen
	case TB_INSERTBUTTONA:
	case TB_INSERTBUTTONW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(ToolbarAddItem(pData,U(wParam),(LPTBBUTTONEX)lParam,(uMsg==TB_INSERTBUTTONEX)? TBFL_ZERO_FLAGS:0)<0)
					lRet = 0;
			else	lRet = 1;

			UNLOCK(pData);

			return lRet;

	case TB_SETINDENT:											// Den X-Offset der ersten Taste ndern

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(pData->iIndent!=(int)wParam)
				{
				pData->cReCalc |= 0x01;
				pData->iIndent	= (int)wParam;
				UpdateView(pData);
				}

			UNLOCK(pData);

			return 1;

	case TB_ADDSTRINGA:											// Strings einfgen

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(lParam>>16)
					lRet = ToolbarAddStrings  (pData,lParam,U(wParam),FALSE);
			else	lRet = ToolbarAddStringRes(pData,(HINSTANCE)wParam,U(lParam));

			UNLOCK(pData);

			return 1;

	case TB_ADDSTRINGW:											// Strings einfgen

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(lParam>>16)
					lRet = ToolbarAddStrings  (pData,lParam,U(wParam),TRUE);
			else	lRet = ToolbarAddStringRes(pData,(HINSTANCE)wParam,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_DELSTRINGS:											// Mehere Texte lschen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarDelStrings(pData,U(wParam),U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_BUTTONCOUNT:										// Anzahl der Tasten abfragen

			pData = GetHandle(hWnd);
			return pData->uItemsCnt;

	case TB_CHANGEBITMAP:										// ndert das Bild einer Taste

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarChangeBitmap(pData,uIndex,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_GETBITMAP:											// Bitmap abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex>=pData->uItemsCnt)lRet=0;
			else lRet =pData->pItems[uIndex]->iBitmap;

			UNLOCK(pData);

			return lRet;

	case TB_GETBUTTON:											// Tastedaten abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex>=pData->uItemsCnt || !lParam)
				{
				lRet = 0;
				}
			else{
				memcpy((LPVOID)lParam,pData->pItems[uIndex],pData->uStructSize);
				lRet = 1;
				}

			UNLOCK(pData);

			return lRet;

	case TB_CHECKBUTTON:										// Eine Taste als gedrckt setzen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarCheckButton(pData,uIndex,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_COMMANDTOINDEX:										// Die Taste zu einem Commando abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarButtonIndex(pData,U(wParam));

			UNLOCK(pData);

			return lRet;

	case TB_INDEXTOCOMMAND:										// Die Commando-ID einer Taste abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(uIndex<pData->uItemsCnt)
				{
				lRet = pData->pItems[uIndex]->idCommand;
				}
			else{
				lRet = 0xFFFFFFFF;
				}

			UNLOCK(pData);

			return lRet;

	case TB_DELETEBUTTONS:										// Mehrere Tasten lschen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet   = 0;
			uIndex = U(lParam);

			if(uIndex>pData->uItemsCnt)
				{
				uIndex = pData->uItemsCnt;
				}

			while(uIndex>0)
				{
				uIndex--;
				if(ToolbarDelItem(pData,uIndex))lRet++;
				if(uIndex==U(wParam))break;
				}

			UNLOCK(pData);

			return lRet;

	case TB_DELETEBUTTON:										// Eine Taste lschen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			lRet = ToolbarDelItem(pData,uIndex);

			UNLOCK(pData);

			return lRet;

	case TB_ENABLEBUTTON:										// Eine Taste freigeben

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarEnableButton(pData,uIndex,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_GETBUTTONTEXTA:										// Den Text der Tasten abfragen
	case TB_GETBUTTONTEXTW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarGetButtonText(pData,uIndex,(LPWSTR)lParam,uMsg==TB_GETBUTTONTEXTW);

			UNLOCK(pData);

			return lRet;

	case TB_GETEXTENDEDSTYLE:									// Erweiterte Flags einstellen

			pData = GetHandle(hWnd);
			return pData->uStyleEx;

	case TB_GETITEMRECT:										// Das Rechteck einer Taste abfragen
	case TB_GETRECT:

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(uMsg==TB_GETRECT)
				{
				uIndex = ToolbarButtonIndex(pData,U(wParam));
				}
			else{
				uIndex = U(wParam);

				if(TB_SPECIAL(uIndex))
					{
					uIndex = ToolbarButtonIndex(pData,uIndex);
					}
				}

			if(uIndex>=pData->uItemsCnt)
				{
				lRet = FALSE;
				}
			else{
				if(pData->cReCalc)ToolbarCalcLayout(pData);
				CopyRect((LPRECT)lParam,&pData->pItems[uIndex]->sRect);
				lRet = TRUE;
				}

			UNLOCK(pData);

			return lRet;

	case TB_GETSTATE:											// Tastenstatus abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex<pData->uItemsCnt)
					lRet = pData->pItems[uIndex]->fsState;
			else	lRet = -1;

			UNLOCK(pData);

			return lRet;

	case TB_FINDSTRINGA:										// Einen Text suchen
	case TB_FINDSTRINGW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarFindString(pData,LOWORD(wParam),HIWORD(wParam),lParam,uMsg==TB_FINDSTRINGW);

			UNLOCK(pData);

			return lRet;

	case TB_GETSTRINGA:											// Einen Text abfragen
	case TB_GETSTRINGW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarGetString(pData,HIWORD(wParam),LOWORD(wParam),lParam,uMsg==TB_GETSTRINGW);

			UNLOCK(pData);

			return lRet;

	case TB_SETBUTTONSTRINGA:											// Einen Text abfragen
	case TB_SETBUTTONSTRINGW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			lRet = ToolbarSetButtonText(pData,uIndex,lParam,uMsg==TB_SETBUTTONSTRINGW);

			UNLOCK(pData);

			return lRet;

	case TB_GETSTYLE:											// Style abfragen

			pData = GetHandle(hWnd);
			return pData->uStyle;

	case TB_HIDEBUTTON:											// Eine Taste asublenden

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarHideButton(pData,uIndex,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_MARKBUTTON:											// Das Marked-Flag setzen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarMarkeButton(pData,uIndex,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_PRESSBUTTON:										// Das Pressed-Flag setzen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarPressButton(pData,uIndex,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_INDETERMINATE:										// Eine Tatse sperren

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));
			lRet	= ToolbarDisabaleButton(pData,uIndex,U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_ISBUTTONCHECKED:									// Ist eine Tatse gedrckt

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex<pData->uItemsCnt)
					lRet = (pData->pItems[uIndex]->fsState&TBSTATE_CHECKED)? TRUE:FALSE;
			else	lRet = -1;

			UNLOCK(pData);

			return lRet;

	case TB_ISBUTTONENABLED:									// Ist eine Tatse freigegben

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex<pData->uItemsCnt)
					lRet = (pData->pItems[uIndex]->fsState&TBSTATE_ENABLED)? TRUE:FALSE;
			else	lRet = -1;

			UNLOCK(pData);

			return lRet;

	case TB_ISBUTTONHIDDEN:										// Ist eine Tatse ausgeblendet

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex<pData->uItemsCnt)
					lRet = (pData->pItems[uIndex]->fsState&TBSTATE_HIDDEN)? TRUE:FALSE;
			else	lRet = -1;

			UNLOCK(pData);

			return lRet;

	case TB_ISBUTTONHIGHLIGHTED:								// Ist eine Tatse ausgeblendet

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex<pData->uItemsCnt)
					lRet = (pData->pItems[uIndex]->fsState&TBSTATE_MARKED)? TRUE:FALSE;
			else	lRet = -1;

			UNLOCK(pData);

			return lRet;

	case TB_ISBUTTONINDETERMINATE:								// Ist eine Tatse gesperrt

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex<pData->uItemsCnt)
					lRet = (pData->pItems[uIndex]->fsState&TBSTATE_INDETERMINATE)? TRUE:FALSE;
			else	lRet = -1;

			UNLOCK(pData);

			return lRet;

	case TB_ISBUTTONPRESSED:									// Ist eine Tatse gedrckt

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= ToolbarButtonIndex(pData,U(wParam));

			if(uIndex<pData->uItemsCnt)
					lRet = (pData->pItems[uIndex]->fsState&TBSTATE_PRESSED)? TRUE:FALSE;
			else	lRet = -1;

			UNLOCK(pData);

			return lRet;

	case TB_SETPARENT:											// Den Empfnger fr Nachrichten einstellen

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(HIWORD(lParam)==0x1234)
					uIndex = LOWORD(lParam);
			else	uIndex = 0;

			switch(uIndex)
				{
			case 0:	lRet				= (LRESULT)pData->hParent;
					pData->hParent		= (HWND)wParam;
					break;

			case 1:	lRet				= (LRESULT)pData->hParentEx;
					pData->hParentEx	= (HWND)wParam;
					break;

			case 2:	lRet				= (LRESULT)pData->hParentCtrl;
					pData->hParentCtrl	= (HWND)wParam;
					break;

			default:lRet				= 0;
					break;
				}

			UNLOCK(pData);

			return lRet;

	case TB_CALCFRAME:											// Gre der Tasten vorab berechnen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarCalcFrame(pData,U(wParam),U(lParam));

			UNLOCK(pData);

			return lRet;

	case TB_GETFRAME:											// Gre aller Tasten abfragen

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(pData->cReCalc)ToolbarCalcLayout(pData);			// Muss die Toolbar neu berechnet werden
			lRet = MAKELONG(pData->uMaxX,pData->uMaxY);

			UNLOCK(pData);

			return lRet;

	case TB_CHANGEFLAGS:										// Flags von mehreren Tatsen ndern

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= LOWORD(wParam);
			uHigh	= HIWORD(wParam);
			uFlag	= LOWORD(lParam);
			uMask	= HIWORD(lParam);
			uFlag  &= uMask;
			lRet	= 0;

			if(uHigh==0x1248)									// Via ID suchen
				{
				uHigh	= pData->uItemsCnt;
				uItem	= uIndex;
				uIndex 	= 0;
				}
			else{
				uItem	= 0;
				}

			if(uHigh>pData->uItemsCnt)
				{
				uHigh = pData->uItemsCnt;
				}

			for(;uIndex<uHigh;uIndex++)							// nder die Flags
				{
				if(uIndex>uHigh)break;

				pItem	= pData->pItems[uIndex];

				if(uItem)
					{
					if(pItem->idCommand^uItem)continue;
					}
				else{
					if(pItem->fsStyle&BTNS_SEP)continue;
					}

				uOld = pItem->wFlags;
				uVal =(pItem->wFlags&~uMask)|uFlag;

				if(uOld!=uVal)									// Haben sich die Flags verndert
					{
					pItem->wFlags  = uVal;
					uVal		  ^= uOld;

					if(uVal&~BTFL_RIGHT_CLICK)
						{
						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						pData->cReCalc    |= 0x03;
						}
					}

				lRet++;
				}

			if(pData->cReCalc)
				{
				UpdateView(pData);
				}

			UNLOCK(pData);

			return lRet;

	case TB_CHANGESTATE:										// State von mehreren Tatsen ndern

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= LOWORD(wParam);
			uHigh	= HIWORD(wParam);
			uFlag	= LOWORD(lParam);
			uMask	= HIWORD(lParam);
			uFlag  &= uMask;
			lRet	= 0;

			if(uHigh==0x1248)									// Via ID suchen
				{
				uHigh	= pData->uItemsCnt;
				uItem	= uIndex;
				uIndex 	= 0;
				}
			else{
				uItem	= 0;
				}

			if(uHigh>pData->uItemsCnt)
				{
				uHigh = pData->uItemsCnt;
				}

			for(;uIndex<uHigh;uIndex++)							// ndern der Flags
				{
				if(uIndex>uHigh)break;

				pItem	= pData->pItems[uIndex];

				if(uItem)
					{
					if(pItem->idCommand^uItem)continue;
					}
				else{
					if(pItem->fsStyle&BTNS_SEP)continue;
					}

				uOld = pItem->fsState;
				uVal =(pItem->fsState&~uMask)|uFlag;

				if(uOld!=uVal)									// Hat sich der State verndert
					{
					pItem->fsState  = uVal;
					uVal           ^= uOld;

					if(uVal&(TBSTATE_HIDDEN|TBSTATE_WRAP))
						{
						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						pData->cReCalc    |= 0x03;
						}
					}

				lRet++;
				}

			if(pData->cReCalc)
				{
				UpdateView(pData);
				}

			UNLOCK(pData);

			return lRet;

	case TB_CHANGESTYLE:										// Style von mehreren Tatsen ndern

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= LOWORD(wParam);
			uHigh	= HIWORD(wParam);
			uFlag	= LOWORD(lParam);
			uMask	= HIWORD(lParam);
			uFlag  &= uMask;
			lRet	= 0;

			if(uHigh==0x1248)									// Via ID suchen
				{
				uHigh	= pData->uItemsCnt;
				uItem	= uIndex;
				uIndex 	= 0;
				}
			else{
				uItem	= 0;
				}

			if(uHigh>pData->uItemsCnt)
				{
				uHigh = pData->uItemsCnt;
				}

			for(;uIndex<uHigh;uIndex++)							// ndern des Styles
				{
				if(uIndex>uHigh)break;

				pItem	= pData->pItems[uIndex];

				if(uItem)
					{
					if(pItem->idCommand^uItem)continue;
					}
				else{
					if(pItem->fsStyle&BTNS_SEP)continue;
					}

				uOld = pItem->fsStyle;
				uVal =(pItem->fsStyle&~uMask)|uFlag;

				if(uOld!=uVal)
					{											// Hat sich der Style verndert
					pItem->fsStyle  = uVal;
					uVal           ^= uOld;

					if(uVal&~(BTNS_GROUP|BTNS_GROUPSTART))
						{
						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						pData->cReCalc    |= 0x03;
						}
					}

				lRet++;
				}

			if(pData->cReCalc)
				{
				UpdateView(pData);
				}

			UNLOCK(pData);

			return lRet;

	case TB_SETTEXTSIZE:										// Die Minimale Textbreite einstellen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex<pData->uItemsCnt)
				{
				pItem = pData->pItems[uIndex];

				if(pItem->wFlags&(BTFL_COLOR_CHANGE|BTFL_CONTROL))
					{
					lRet = -1;
					}
				else{
					if(lParam<  0)lParam=0;
					if(lParam>512)lParam=512;

					if(pItem->wFlags&BTFL_TEXT_SIZE)
						{
						lRet = LOWORD(pItem->dwData);
						pItem->dwData = (DWORD)lParam;
						}
					else{
						lRet = 0;
						}

					if(pItem->dwData!=LOWORD(lParam))			// Hat sich die Breite verndert
						{
						if(U(lParam))
								pItem->wFlags|= BTFL_TEXT_SIZE;
						else	pItem->wFlags&=~BTFL_TEXT_SIZE;

						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						pItem->dwData      = LOWORD(lParam);
						pData->cReCalc    |= 0x03;

						UpdateView(pData);
						}
					}
				}
			else{
				lRet = -1;
				}

			UNLOCK(pData);

			return lRet;

	case TB_COUNTITEMS:											// Eintrge Zhlen

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex	= LOWORD(wParam);
			uHigh	= HIWORD(wParam);
			uFlag	= LOWORD(lParam);
			uMask	= HIWORD(lParam);
			uFlag  &= uMask;
			lRet	= 0;

			if(uHigh>pData->uItemsCnt)
				{
				uHigh = pData->uItemsCnt;
				}

			for(;uIndex<uHigh;uIndex++)							// Suche die Eintrge
				{
				if(uIndex>uHigh)break;

				pItem	= pData->pItems[uIndex];

				if(uMask==0)
					{
					if(pItem->idCommand^uFlag)continue;
					}
				else{
					if((pItem->wFlags&uMask)!=uFlag)continue;
					}

				lRet++;
				}

			UNLOCK(pData);

			return lRet;

	case TB_SETFLAGS:											// Die Flags einstellen oder abfragen
	case TB_GETFLAGS:

			pData = GetHandle(hWnd);
 
			LOCK(pData);
 
			uIndex = U(wParam);
 
			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}
 
			if(uIndex<pData->uItemsCnt)
				{
				pItem	= pData->pItems[uIndex];
				lRet	= pItem->wFlags;
 
				if(uMsg==TB_SETFLAGS)							// Die Flags ndern
					{
					uVal	= pItem->wFlags&~HIWORD(lParam);
					uVal   |= LOWORD(lParam)&HIWORD(lParam);
					uMask	= (DWORD)lRet^uVal;
 
					if(uMask)									// Gibt es nderungen
						{
						pItem->wFlags = uVal;
 
						if(uMask&~BTFL_RIGHT_CLICK)
							{
							pItem->wInfoFlags |= TBIF_REASSEMBLE;
							pData->cReCalc    |= 0x03;
							UpdateView(pData);
							}
						}
					}
				}
			else{
				lRet = 0xFFFFFFFF;
				}
 
			UNLOCK(pData);
 
			return lRet;

	case TB_GETSTRINGID:										// Die String-ID einer Taste ndern
	case TB_SETSTRINGID:

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex<pData->uItemsCnt)
				{
				pItem	= pData->pItems[uIndex];
				lRet	= pItem->iString;

				if(U(lRet)!=U(lParam) && uMsg==TB_SETSTRINGID)
					{
					pItem->wInfoFlags |= TBIF_REASSEMBLE;
					pItem->iString	   = U(lParam);
					pData->cReCalc    |= 0x03;
					UpdateView(pData);
					}
				}
			else{
				lRet = 0;
				}

			UNLOCK(pData);

			return lRet;

	case TB_GETBITMAPID:										// Die Bitmap-ID einer Taste ndern
	case TB_SETBITMAPID:

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex<pData->uItemsCnt)
				{
				pItem	= pData->pItems[uIndex];
				lRet	= pItem->iBitmap;

				if(U(lRet)!=U(lParam) && uMsg==TB_SETBITMAPID)	// ndert sich die Bildgre
					{
					pItem->wInfoFlags |= TBIF_REASSEMBLE;
					pItem->iBitmap	   = U(lParam);

					   uOld=ToolbarGetPicSize(pData,U(lRet));
					if(uOld^ToolbarGetPicSize(pData,U(lParam)))
						{
						pData->cReCalc |= 0x03;
						UpdateView(pData);
						}
					else{
						InvalidateRect(pData->hWnd,&pItem->sRect,FALSE);
						}
					}
				}
			else{
				lRet = 0;
				}

			UNLOCK(pData);

			return lRet;

	case TB_GETCOMMANDID:										// Die Commando-ID einer Taste ndern
	case TB_SETCOMMANDID:

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex<pData->uItemsCnt)
				{
				pItem	= pData->pItems[uIndex];
				lRet	= pItem->idCommand;

				if(U(lRet)!=U(lParam) && uMsg==TB_SETCOMMANDID)	// ndert sich die Bildgre
					{
					pItem->idCommand = U(lParam);
					}
				}
			else{
				lRet = 0;
				}

			UNLOCK(pData);

			return lRet;

	case TB_GETSTYLESTATE:										// Die Style/State-Flags einer Taste ndern
	case TB_SETSTYLESTATE:

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex<pData->uItemsCnt)
				{
				pItem	= pData->pItems[uIndex];
				uHigh	= 0;

				if(HIWORD(lParam)==TB_ONLYSTATE)
					{
					lParam<<= 16;
					uHigh	= 1;
					}

				lRet	= MAKELONG(pItem->fsStyle,pItem->fsState);
				uMask	= (U(lParam)>>8)&(0x00FF00FF&~TBSTYLE_SEP);
				uFlag   = uMask&U(lParam);
				uFlag  |=~uMask&U(lRet);

				if(U(lRet)!=uFlag && uMsg==TB_SETSTYLESTATE)	// ndern sich die Flags
					{
					pItem->fsStyle = (BYTE)(uFlag);
					pItem->fsState = (BYTE)(uFlag>>16);
					uFlag         ^= U(lRet);

					if(uFlag&(BTNS_AUTOSIZE|BTNS_DROPDOWN|BTNS_SHOWTEXT))
						{
						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						pData->cReCalc    |= 0x03;
						UpdateView(pData);
						}
					else if(uFlag&((TBSTATE_WRAP|TBSTATE_ELLIPSES|TBSTATE_HIDDEN)<<16))
						{
						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						pData->cReCalc    |= 0x03;
						UpdateView(pData);
						}
					else if(uFlag&(~TBSTATE_MARKED<<16))
						{
						UpdateRect(pData,uIndex);
						}
					}

				if(uHigh)lParam>>=16;
				}
			else{
				lRet = 0;
				}

			UNLOCK(pData);

			return lRet;

	case TB_GETPARAM:
	case TB_SETPARAM:											// User-Daten einer Taste ndern

			pData = GetHandle(hWnd);

			LOCK(pData);

			uIndex = U(wParam);

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex<pData->uItemsCnt)
				{
				pItem	= pData->pItems[uIndex];
				lRet	= pItem->dwData;

				if(lRet!=lParam && uMsg==TB_SETPARAM)
					{
					   pItem->dwData = (DWORD)lParam;
					if(pItem->wFlags&BTFL_CONTROL)				// Control gendert
						{
						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						pData->cReCalc    |= 0x03;
						UpdateView(pData);
						}										// Farben verndert
					else if((pItem->wFlags&BTFL_COLOR_CHANGE) && pItem->iBitmap>=0)
						{
						pItem->wInfoFlags |= TBIF_REASSEMBLE;
						InvalidateRect(pData->hWnd,&pItem->sRect,FALSE);
						}
					}
				}
			else{
				lRet = 0;
				}

			UNLOCK(pData);

			return lRet;

	case TB_SETBUTTONINFOA:										// Diverse Einstellungen einer Taste ndern
	case TB_SETBUTTONINFOW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarSetButtonInfo(pData,U(wParam),(LPTBBUTTONINFOEX)lParam,uMsg==TB_SETBUTTONINFOW);

			UNLOCK(pData);

			return lRet;

	case TB_GETBUTTONINFOA:										// Diverse Einstellungen einer Taste holen
	case TB_GETBUTTONINFOW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet = ToolbarGetButtonInfo(pData,U(wParam),(LPTBBUTTONINFOEX)lParam,uMsg==TB_GETBUTTONINFOW);

			UNLOCK(pData);

			return lRet;

	case TB_SETSCALING:											// Die Skalierung einstellen

			pData = GetHandle(hWnd);

			LOCK(pData);

			if(U(wParam)==0xFFFFFFFF)							// Zoom vom ganzen Fenster
				{
				lRet = pData->uButtonZoom;
				uVal = U(lParam);
				if(!uVal)uVal=256;

				if(uVal!=0xFFFFFFFF && pData->uButtonZoom!=uVal)
					{
					pData->uButtonZoom = uVal;
					pData->cReCalc     = 0x0F;
					UpdateView(pData);
					}
				}
			else if(U(wParam)<pData->uResourceCnt)				// Zoom einer Resource
				{
				iPos = U(wParam);
				lRet = pData->pResource[iPos].wScale;
				uVal = U(lParam);
				if(!uVal)uVal=256;

				if(uVal!=0xFFFFFFFF && pData->pResource[iPos].wScale!=uVal)
					{
					pData->pResource[iPos].wScale = uVal;
					pData->cReCalc = 0x07;
					UpdateView(pData);
					}
				}
			else{
				lRet = -1;
				}

			UNLOCK(pData);

			return lRet;

	case TB_SETINSERTMARK:										// Eine Marke setzen

			pData = GetHandle(hWnd);

			LOCK(pData);

			pMark	= (TBINSERTMARK*)lParam;
			uIndex	= pMark->iButton;
			if(TB_SPECIAL(uIndex))uIndex=ToolbarButtonIndex(pData,uIndex);
			lRet	= UpdateMark(pData,uIndex,pMark->dwFlags&0x01);

			UNLOCK(pData);

			return lRet;

	case TB_INSERTMARKHITTEST:									// Hit-Test fr Marke

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet  = ToolbarMarkHitTest(pData,((LPPOINT)lParam)->x,((LPPOINT)lParam)->y,(TBINSERTMARK*)lParam);

			UNLOCK(pData);

			return lRet;

	case TB_RECACLLAYOUT:										// Die Toolba neuberechen

			pData = GetHandle(hWnd);

			LOCK(pData);

			pData->cReCalc |= LOBYTE(wParam);

			if(LOWORD(lParam) && pData->cReCalc)				// Neu zeichnen
				{
				UpdateView(pData);
				}

			UNLOCK(pData);

			return 1;

	case TB_CALCPICCOUNT:										// Anzahl der Sub-Bilder berechnen

			lRet = 0;

			if(wParam)
				{
				TBRESOURCE		sResource;

				if(wParam==TB_ISHBITMAP)						// Aus HBITMAP
					{
					sResource.pInfo=GetDibOfBitmap(NULL,(HBITMAP)lParam,&sResource.pPixels);
					}
				else if(wParam<0xFFFF)							// Aus Rescource-ID
					{
					sResource.pInfo=GetDibOfResourceW(NULL,(HMODULE)lParam,(LPCWSTR)wParam,(LPCWSTR)RT_BITMAP,0,&sResource.pPixels);
					}
				else{
					sResource.pInfo	  = (LPBITMAPINFO)wParam;
					sResource.pPixels = (LPBYTE      )lParam;
					}

				if(!sResource.pInfo || !sResource.pPixels)
					{
					return 0;
					}

				sResource.wLastPos	= 0;
				sResource.wCount	= 0;
				sResource.wRowWidth	= (WORD)BMI_ROW_WIDTH(sResource.pInfo->bmiHeader);

				ToolbarGetPicPos(&sResource,0xFFFF);

				if(wParam<=0xFFFF)
					{
					DibFreeHandle(sResource.pInfo);
					}

				lRet = sResource.wCount;
				}

			return lRet;

	case TB_INSERTTAGA:											// Einen Tag einfgen
	case TB_INSERTTAGW:

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet  = ToolbarTagInsert(pData,(LPTBTAGITEM)wParam,(LPWSTR)lParam,(uMsg==TB_INSERTTAGW));

			UNLOCK(pData);

			return lRet;
		
	case TB_DELETETAG:											// Einen Tag lschen

			pData = GetHandle(hWnd);

			LOCK(pData);

			lRet  = ToolbarTagDelete(pData,U(wParam));

			UNLOCK(pData);

			return lRet;
		}


return DefWindowProc(hWnd,uMsg,wParam,lParam);
}


//*****************************************************************************
//*
//*		ToolbarDraw
//*
//*****************************************************************************
//	Zeichnet das Fenster
static void ToolbarDraw(HWND hWnd,HDC hDc,RECT *pRect)
{
TBTAGDATA	   *pTag;	
ToolbarData    *pData;
LPBITMAPINFO	pInfo;
LPTBITEM		pItem;
RECT			sRect;
RECT			sLast;
BYTE			bFlags;
UINT			uIndex;
UINT			uLine;
UINT			uBase;
UINT			uSize;
UINT			uSlot;
UINT			uOld;
HDC				hMem;


	pData = GetHandle(hWnd);

	LOCK(pData);

	if(!pRect)
		{
		pRect = &sRect;
		GetClientRect(hWnd,&sRect);
		}

	if(pData->cReCalc)											// Muss die Toolbar neu berechnet werden
		{
		ToolbarCalcLayout(pData);
		}


	hMem = CreateCompatibleDC(hDc);
	SetBkColor(hDc,GetSysColor(COLOR_3DFACE));


	if(pData->iIndent>0 && pRect->left<pData->iIndent)
		{
		DrawRectangle(hDc,0,0,pData->iIndent,pData->uSizeY);
		}

	sLast.top	 = 0;
	sLast.left	 = 0;
	sLast.right	 = 0;
	sLast.bottom = 0;


	for(uIndex=0;uIndex<pData->uItemsCnt;uIndex++)				// Alle Tasten abschreiten
		{
		   pItem = pData->pItems[uIndex];
		if(pItem->fsState&TBSTATE_HIDDEN)
			{
			continue;
			}

		if(!UnionRect(&sRect,&pItem->sRect,pRect))				// Muss diese Taste gezeichnet werden
			{
			continue;
			}

		if(pItem->wInfoFlags&TBIF_REASSEMBLE)					// Muss die Tatse neu gezeichnet werden
			{
			ToolbarUpdateImages(pData,uIndex);
			}

		pInfo = pItem->pInfo;
		uLine = pItem->wInfoFlags&7;

		if(!pInfo)												// Noch kein Bild
			{
			DrawRectanglePtr(hDc,&pItem->sRect);
			continue;
			}
																// Zeile rechts fertig ausmalen
		if(sLast.top<pItem->sRect.top && sLast.right<(int)pData->uSizeX)
			{
			DrawRectangle(hDc,sLast.right,sLast.top,pData->uSizeX,sLast.bottom);
			sLast.right = pData->uSizeX;
			}

		if(pItem->fsStyle&BTNS_SEP)								// Separator
			{
			SelectObject(hMem,pItem->hBitmap);
			BitBlt(hDc,pItem->sRect.left,pItem->sRect.top,pItem->wRealX,pItem->wRealY,hMem,0,0,SRCCOPY);
			}
		else if(pItem->fsStyle&BTNS_DROPDOWN)					// Mit Drop-Down
			{
			uBase	= pItem->wShowX&0x03FF;
			uBase  -= pItem->wShowX>>10;

			if(uLine>=6)
				{
				uSlot	= (pData->uDropState)? 3:2;
				uLine  -= 4;
				}
			else if(uLine>=4)
				{
				uSlot	= uLine;
				}
			else{
				uSlot	= (uLine>=2)? 1:0;
				}

			uLine  *= pItem->wRealY;
			SelectObject(hMem,pItem->hBitmap);
			BitBlt(hDc,pItem->sRect.left,pItem->sRect.top,uBase,pItem->wRealY,hMem,0,uLine,SRCCOPY);

			uSlot  *= pItem->wRealY;
			SelectObject(hMem,pItem->hBitmap);
			BitBlt(hDc,pItem->sRect.left+uBase,pItem->sRect.top,pItem->wRealX-uBase,pItem->wRealY,hMem,uBase,uSlot,SRCCOPY);
			}
		else{													// Normale Taste
			uLine  *= pItem->wRealY;
			SelectObject(hMem,pItem->hBitmap);
			BitBlt(hDc,pItem->sRect.left,pItem->sRect.top,pItem->wRealX,pItem->wRealY,hMem,0,uLine,SRCCOPY);
			}

		if(pItem->sRect.right >pItem->sRect.left)
		if(pItem->sRect.bottom>pItem->sRect.top)
			{
			CopyRect(&sLast,&pItem->sRect);
			}
		}

	DrawRectangle(hDc,sLast.right,sLast.top,pData->uSizeX,sLast.bottom);
	DrawRectangle(hDc,pData->iIndent,sLast.bottom,pData->uSizeX,pData->uSizeY);


	if(pData->uMarkItem<pData->uItemsCnt)						// Marke zeichnen
		{
		uOld = SetBkColor(hDc,pData->aColors[TBC_MARK]);

		if(pData->uMarkMode&2)
			{
			uSize = pData->sMarkRect.right-pData->sMarkRect.left;
			DrawRectangle(hDc,pData->sMarkRect.left   ,pData->sMarkRect.top+2,uSize,2);
			DrawRectangle(hDc,pData->sMarkRect.left   ,pData->sMarkRect.top  ,2,6);
			DrawRectangle(hDc,pData->sMarkRect.right-2,pData->sMarkRect.top  ,2,6);
			}
		else{
			uSize = pData->sMarkRect.bottom-pData->sMarkRect.top;
			DrawRectangle(hDc,pData->sMarkRect.left+2 ,pData->sMarkRect.top     ,2,uSize);
			DrawRectangle(hDc,pData->sMarkRect.left   ,pData->sMarkRect.top     ,6,2);
			DrawRectangle(hDc,pData->sMarkRect.left   ,pData->sMarkRect.bottom-2,6,2);
			}

		SetBkColor(hDc,uOld);
		}

	
	for(pTag=pData->pTagChain;pTag;pTag=pTag->pNext)			// Tags zeichnen
		{
		SelectObject(hDc,GetStockObject(ANSI_VAR_FONT));

		if(pTag->sItem.ucFlags&TBTF_RECALC)						// Position neu berechnen			
			{
			pTag->sItem.ucFlags &= ~TBTF_RECALC;
			bFlags = pTag->sItem.ucFlags;
			uIndex = pTag->sItem.uIndex;

			if(TB_SPECIAL(uIndex))
				{
				uIndex = ToolbarButtonIndex(pData,uIndex);
				}

			if(uIndex>=pData->uItemsCnt)						// Index prfen
				{
				bFlags |= 0x0F;
				pItem   = NULL;
				}
			else{
				DrawTextW(hDc,pTag->cText,-1,&sRect,DT_CALCRECT);
				pItem = pData->pItems[uIndex];
				}

			switch(bFlags&0x03)
				{
			case TBTF_LEFT:		pTag->sPosX =(SHORT) pItem->sRect.left;											break;
			case TBTF_CENTER:	pTag->sPosX =(SHORT)(pItem->sRect.left +pItem->sRect.right+sRect.right+2)/2;	break;								
			case TBTF_RIGHT:	pTag->sPosX =(SHORT)(pItem->sRect.right-sRect.right-2);							break;								
			default:			pTag->sPosX =(SHORT) 0;															break;
				}

			switch(bFlags&0x0C)
				{
			case TBTF_TOP:		pTag->sPosY =(SHORT) pItem->sRect.top;											break;
			case TBTF_VCENTER:	pTag->sPosY =(SHORT)(pItem->sRect.top   +pItem->sRect.bottom+sRect.bottom)/2;	break;								
			case TBTF_BOTTOM:	pTag->sPosY =(SHORT)(pItem->sRect.bottom-sRect.bottom);							break;								
			default:			pTag->sPosY =(SHORT) 0;															break;
				}

			pTag->sPosX += pTag->sItem.sDeltaX;
			pTag->sPosY += pTag->sItem.sDeltaY;
			}
		
		ToolbarTagDraw(pData,hDc,pTag->sPosX,pTag->sPosY,pTag->cText);
		}

	DeleteDC(hMem);

	UNLOCK(pData);

}




