//*****************************************************************************
//*
//*
//*		AnsiBase.cpp
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2003
//
//
//	Mit #define TO_ANSIWND wird auf CAnsiWnd um geschalten
//

#include	"../AfxUtility.h"
#include	"../../StringFunctions.h"
#include	"../../RegKeys.h"

#ifdef		TO_ANSIWND
	#define		CAnsiView	CAnsiWnd
	#define 	CView		CWnd
#else
	#ifdef		CAnsiView
	#undef 		CAnsiView
	#endif
	#ifdef		CView
	#undef 		CView
	#endif
#endif

#ifdef     _DEBUG
#define new DEBUG_NEW
#undef      THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#ifndef 	ANSI_FLAGS
#define 	ANSI_FLAGS			0
#endif

#ifdef		USE_VIRTUAL_LINES
#define 	ANSI_STEP(a,b)		TranslateLine(a,b)
#else
#define 	ANSI_STEP(a,b)		((a)+(b))
#endif

#define 	ENTRIES(a)			(sizeof(a)/sizeof((a)[0]))
#define 	SEPARATOR_SIZE		7



//*****************************************************************************
//*
//*		GetIdString
//*
//*****************************************************************************
#ifdef		USE_RC

static const char *GetIdString(unsigned uId)
	{
	static int		iNr=0;
	static CString 	sStr[4];

		iNr++;
		sStr[iNr&3].LoadString(uId);

	return sStr[iNr&3];
	}

	#define 	STRING(s,i)		GetIdString(i)

#else

	#define 	STRING(s,i)		_T(s)

#endif

//*****************************************************************************
//*
//*		Definitionen fr nicht Implementierte  Menfunktionen
//*
//*****************************************************************************

#ifndef		ID_VIEW_COLOR
#define		ID_VIEW_COLOR		0x9230
#endif
#ifndef		ID_VIEW_FONT
#define		ID_VIEW_FONT		0x9231
#endif

#define		ID_MOUSEMOVE		0x8856   					// Timer ID



//*****************************************************************************
//*
//*		Nachrichten Verarbeitung
//*
//*****************************************************************************
#ifndef 	TO_ANSIWND
IMPLEMENT_DYNCREATE(CAnsiView, CView)
#endif

BEGIN_MESSAGE_MAP  (CAnsiView, CView)
	//{{AFX_MSG_MAP(CAnsiView)
	ON_WM_SIZE()
	ON_WM_CHAR()
	ON_WM_KEYUP()
	ON_WM_TIMER()
	ON_WM_CREATE()
	ON_WM_KEYDOWN()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_SETFOCUS()
	ON_WM_SETCURSOR()
	ON_WM_RBUTTONUP()
	ON_WM_KILLFOCUS()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_MOUSEWHEEL()
	ON_WM_GETDLGCODE()
	ON_WM_MBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONDBLCLK()
	ON_WM_SYSCOLORCHANGE()
	ON_COMMAND(ID_VIEW_COLOR		 , OnColorView)
	ON_COMMAND(ID_VIEW_FONT			 , OnFontView)
	ON_MESSAGE(WM_ANSIINFO			 , OnAnsiInfo)
	ON_MESSAGE(WM_XBUTTONDOWN		 , OnButtonDownX)
	#ifndef TO_ANSIWND
	ON_COMMAND(ID_FILE_PRINT         , OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT  , OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW , OnFilePrintPreview)
	ON_COMMAND(ID_FILE_PAGE_SETUP	 , OnPageSetup)
	ON_WM_DROPFILES()
	#else
	ON_WM_PAINT()
	#endif
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//*****************************************************************************
//*
//*		Statische Variableln
//*
//*****************************************************************************

COLORREF	 CAnsiView::m_uScrollColor	= 0;				// Farbe fr recht untere Ecke bei ScrollBars
HCURSOR		 CAnsiView::m_hCursorBeam	= 0;				// Globales Handle fr den Beam-Mauszeiger
HCURSOR		 CAnsiView::m_hCursorHand	= 0;				// Globales Handle fr den Hand-Mauszeiger
int			 CAnsiView::m_iClassCount	= 0;				// Anzahl der Klassen
LPTSTR		 CAnsiView::m_pClassName	= 0;				// Klassenname
CAnsiView   *CAnsiView::m_pClassLast	= 0;				// Zeiger auf letzte Klasse
int			 CAnsiView::m_iGroupId		= 0;				// Gruppen ID Nummer

#ifdef 		TO_ANSISPLIT
HCURSOR		 CAnsiView::m_hCursorSizeNs	= 0;				// Globales Handle fr den SizeNs-Mauszeiger
HCURSOR		 CAnsiView::m_hCursorSizeWe	= 0;				// Globales Handle fr den SizeNs-Mauszeiger
HCURSOR		 CAnsiView::m_hCursorSizeAll= 0;				// Globales Handle fr den SizeAll-Mauszeiger
#endif 	

#ifndef 	 TO_ANSIWND
AnsiPageData CAnsiView::m_sPrinterData={60,20,10,20,10,9,0,0,0,0,0,0};
#endif

//*****************************************************************************
//*
//*		Konstruktor ... Grundeinstellung der Variabeln
//*
//*****************************************************************************
CAnsiView::CAnsiView(LPCTSTR pGroupName,int iWidth,int iLinesMax)
{
CAnsiView  *pClass;
LPCTSTR		pName;



	if(m_iClassCount)
		{
		m_pClassLast->m_pClassNext = this;
		m_pClassPrev =m_pClassLast;							// Zeiger auf vorherige Klasse
		}
	else{
	    m_pClassPrev	= 0;
		}

	if(!m_hCursorBeam)										// Globales Handle fr den Beam-Mauszeiger
		{
		m_hCursorBeam	= LoadCursorW(NULL,(LPCWSTR)IDC_IBEAM  );
		m_hCursorHand	= LoadCursorW(NULL,(LPCWSTR)IDC_HAND   );

		#ifdef	TO_ANSISPLIT

		m_hCursorSizeWe	= LoadCursorW(NULL,(LPCWSTR)IDC_SIZEWE );
		m_hCursorSizeNs	= LoadCursorW(NULL,(LPCWSTR)IDC_SIZENS );
		m_hCursorSizeAll= LoadCursorW(NULL,(LPCWSTR)IDC_SIZEALL);

		#endif 	
		}

	m_hCursorBase		= m_hCursorBeam;
	m_hCursorNow		= m_hCursorBeam;
	m_iClassCount	   += 1;
	m_pClassNext		= 0;
	m_pClassLast		= this;

	if(!m_pClassName)										// Neue Klasse erzeugen
		{
		pName = AfxRegisterWndClass(CS_DBLCLKS,m_hCursorBase,(HBRUSH)GetStockObject(NULL_BRUSH));
		m_pClassName=new TCHAR[_tcslen(pName)+1];
		_tcscpy(m_pClassName,pName);
		}

	m_iWindowMode		=  ANSI_FLAGS;
	m_iWindowModeOld	=  0;
	m_iRedrawFlags		=  0;
	m_iMarginWidth		=  0;
	m_uMouseTicks 		=  0;
	m_bMouseTimer 		=  0;
	m_cMouseOn   		=  0;

	m_iFontSizeX		= 10;
	m_iFontSizeY		= 13;

	m_iCursorX			=  0;								// Grundeinstellung
	m_iCursorY			=  0;
	m_iCursorOldX		=  0;
	m_iCursorOldY		=  0;
	m_iScreenX			=  0;
	m_iScreenY			=  0;
	m_iScreenOldX		=  0;
	m_iScreenOldY		=  0;
	m_iOriginPtX		=  0;
	m_iOriginPtY		=  0;
	m_iScrollX			=  0;
	m_iScrollY			=  0;
	m_iScrollOldX		=  0;
	m_iScrollOldY		=  0;
	m_iScrollAddX		=  0;
	m_iScrollAddY		=  0;
	m_iScreenFullX		=  0;
	m_iScreenFullY		=  0;
	m_iScreenSizeX		=  0;
	m_iScreenSizeY		=  0;
	m_iCursorMode		=  0;
	m_iCursorModeOld	=  0;
	m_cCaretFocus		=  1;
	m_cCaret			=  0;

	m_iLineCount		=  iLinesMax+100;					// Anzahl der Zeilen
	m_iLineWidthMax		=  iWidth;							// Maximale Breie in Zeichen
	m_iLineWidthMaxOld	=  0;								// Alte maximale Breie
	m_iLineCountOld		=  0;								// Alte Anzahl der Zeilen
	m_pLineSizes		=  0;								// Breite der einzelnen Zeilen
	m_pPageBegin		=  0;								// Anfnge der Seiten
	m_iColorNow			= -1;								// Aktuell eingestellte Farbe
	m_iKeyState			=  0;								// Shift/Cltr - Tastensatus
	m_iOnSizeCnt		=  0;								// OnSize-Rekursions Nummer
	m_pConfigPath		=  0;								// Konigurationspfad
	m_cColorDisable		=  0;								// Sperrt die Farbverwaltung
	m_cClickMode		=  0;								// Modus der rechten Maustaste (0=aus 1=an 2=Rahmen 3=move)
	m_cCapture			=  0;								// Ist der Maus-Capture aktiv
	m_cFocusOn  		=  0;								// Hat das Fenster den Fokus

	#ifdef	TO_ANSISPLIT

	m_iScrollOldXB		=  0;								// Alte Scrollposition fr inaktive Bars
	m_iScrollOldYB		=  0;								//			*
	m_iScreenOldXB		=  0;								// Alte Schirmkoordinaten fr inaktive Bars
	m_iScreenOldYB		=  0;								//			*
	m_iSplitActiv		=  0;								// Aktives Fenster (Bit0=X Bit1=Y)
	m_cSplitBeam		=  0;								// Welcher Mauszeiger ist aktiv (0=Beam 1=Y 2=Y 3=XY)
	m_aSplitNow[0]		=  0;								// Position des Teiles (set , 0=X 1=Y)
	m_aSplitNow[1]		=  0;								//			*
	m_aSplitOn [0]		=  0;								// Ist das Fenster geteillt	 (0=X 1=Y)
	m_aSplitOn [1]		=  0;								//			*

	#endif


//******************** Steuerung der Gruppendaten *****************************


	pClass	 = m_pClassPrev;
	m_pGroup = 0;

	while(pClass)											// Suche ob Gruppe schon vorhanden ist
		{
		if(!_tcscmp(pClass->m_pGroup->pName,pGroupName))
			{
			m_pGroup = pClass->m_pGroup;
			m_pGroup->iCount++;
			break;
			}

		pClass = pClass->m_pClassPrev;
		}

	if(!m_pGroup)											// Neue Gruppe erzeugen
		{
		m_pGroup = new AnsiGroupData;

		memset(m_pGroup,0,sizeof(AnsiGroupData));

		m_pGroup->iId				= m_iGroupId;			// ID Nummer setzen
		m_pGroup->iCount			= 1;					// Anzahl der Gruppenmitglieder
		m_pGroup->pName				= pGroupName;			// Zeiger auf Namen
		m_pGroup->sAnsiFont.iFlags	= ANSI_FIXED;			// Schriftart
		m_pGroup->pColorTable		= 0;					// Farbtabelle fr die Schnellsuche


		ColorAdd(0xFFFFFFFF,0xFFFFFFFF,STRING(ANSI_FONT_TEXT,ID_ANSI_FONT_TEXT),1);
		m_iGroupId++;
		}

	SetCursorMode(2);

}


//*****************************************************************************
//*
//*		Destruktor ... Grundeinstellung der Variabeln
//*
//*****************************************************************************
CAnsiView::~CAnsiView()
{
int		i;


	if(m_pLineSizes )delete m_pLineSizes;
	if(m_pPageBegin )delete m_pPageBegin;
	if(m_pConfigPath)delete m_pConfigPath;


		m_pGroup->iCount--;
	if(!m_pGroup->iCount)									// Gruppendaten lschen
		{
		ColorDelete(&m_pGroup->pColorList);

		for(i=0;i<8;i++)
		if(m_pGroup->hFont)FontDelete(m_pGroup->hFont);
		if(m_pGroup->hFontEx[i])
		if(m_pGroup->hFontEx[i]!=m_pGroup->hFont)
			{
			FontDelete(m_pGroup->hFontEx[i]);
			}

		if(    m_pGroup->pColorTable)delete m_pGroup->pColorTable;
		if(    m_pGroup->pConfig	)delete m_pGroup->pConfig;
		delete m_pGroup->pColorList;
		delete m_pGroup;
		}

		 													// Klasse aus Liste lschen
	if(m_pClassPrev)m_pClassPrev->m_pClassNext=m_pClassNext;
	if(m_pClassNext)m_pClassNext->m_pClassPrev=m_pClassPrev;
	if(m_pClassLast==this)        m_pClassLast=m_pClassPrev;

	m_iClassCount--;

	if(m_iClassCount<=0)
		{
		#ifndef 	TO_ANSIWND
		if(m_sPrinterData.pHeaderData){delete m_sPrinterData.pHeaderData;m_sPrinterData.pHeaderData=0;}
		if(m_sPrinterData.pFooterData){delete m_sPrinterData.pFooterData;m_sPrinterData.pFooterData=0;}
		#endif

		if(m_pClassName){delete m_pClassName;m_pClassName=0;}
		}

}

//*****************************************************************************
//*
//*		Create
//*
//*****************************************************************************
#ifdef 	TO_ANSIWND
BOOL CAnsiWnd::Create(DWORD dwStyle,const RECT &sRect,CWnd *pParent,UINT uId)
{
return CWnd::Create(m_pClassName,_T("AnsiWnd"),dwStyle,sRect,pParent,uId);
}
#endif

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
#ifdef 	TO_ANSIWND
BOOL CAnsiWnd::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect, CWnd *pParent,UINT uId)
{
return CWnd::CreateEx(dwExStyle,m_pClassName,_T("AnsiWnd"),dwStyle,sRect,pParent,uId);
}
#endif

//*****************************************************************************
//*
//*		PreCreateWindow
//*
//*****************************************************************************
// Aufruf bevor Fenster sichtbar wird
#ifndef 	TO_ANSIWND
BOOL CAnsiView::PreCreateWindow(CREATESTRUCT &sData)
{


	sData.lpszClass	 = m_pClassName;

	if(m_iWindowMode&ANSI_DROP)
		{
		sData.dwExStyle	|= WS_EX_ACCEPTFILES;
		}

	if(CView::PreCreateWindow(sData))
		{
		m_hCursorNow = m_hCursorBase;
		return TRUE;
		}


return FALSE;
}
#endif

//*****************************************************************************
//*
//*		ColorAdd
//*
//*****************************************************************************
//	Hngt an die aktuelle Farbliste eine Farbe an.
//	uTextColor	: uTextColor (0xFFFFFFFF==Standard)
//	uBkColor	: Hintergrundfarbe (0xFFFFFFFF==Standard)
//	pName		: Name der Schriftart
//	iIndex		: Index Nummer in Registry der Schriftart. Das ist nicht
//				  die Farbnummer mit der der Text gezeichnet wird.
//					ANSI_ONLYNEW = Nur einfgen wenn noch nicht vorhanden
//	iFont		: Bestimmt die Schriftart
//						Bit 0 = Fett
//						Bit 1 = Italic
//						Bit 2 = Unterstrichen
//	Ergibt die Nummer der Farbe in der Farbliste, das ist die Farbnummer mit
//	der der Text gezeichnet wird.
int CAnsiView::ColorAdd(COLORREF uTextColor,COLORREF uBkColor,LPCTSTR pName,int iIndex,int iFont)
{
AnsiColor  *pList;
int			iNum;


	if(iIndex&ANSI_ONLYNEW)									// Nur einfgen wenn nicht vorhanden
		{
		iIndex &= ~ANSI_ONLYNEW;
		pList   =  m_pGroup->pColorList;

		if(pList)
			{
			for(iNum=0;pList[iNum].pName;iNum++)
				{
				if(pList[iNum].iIndex==iIndex)
					{
					return iNum;							// Farbindex ist schon vorhanden
					}
				}
			}
		}

	iNum = ColorInsert(&m_pGroup->pColorList,uTextColor,uBkColor,pName,iIndex,iFont);
	m_pGroup->iColorCount = iNum;

	if(m_pGroup->pColorTable)
		{
		delete m_pGroup->pColorTable;
		m_pGroup->pColorTable = 0;
		}



return iNum-1;
}

//*****************************************************************************
//*
//*		ColorInfo
//*
//*****************************************************************************
//	Diehnt zum Abfragen von Farbinformationen
//	iNum		: Ist die Nummer der abzufragenden Farbe.
//	uTextColor	: Hier wird die Textfarbe gespeichert.
//	uBkColor	: Hier wird die Hintergrundfarbe gespeichert.
//	iFont		: Hier wird die Schriftart gespeichert:
//						Bit 0 = Fett
//						Bit 1 = Italic
//						Bit 2 = Unterstrichen
//	Ergibt die 1 wenn die Farbe gefunden wurde  sonst 0
int CAnsiView::ColorInfo(int iNum,COLORREF &uTextColor,COLORREF &uBkColor,int &iFont)
{
AnsiColor *pEntry;


	if(iNum<0 || iNum>=m_pGroup->iColorCount)return 0;

	pEntry		= m_pGroup->pColorList+iNum;
	uTextColor	= pEntry->uColorText;
	uBkColor	= pEntry->uColorBk;
	iFont		= pEntry->iFontMode;


return 1;
}

//*****************************************************************************
//*
//*		ColorView
//*
//*****************************************************************************
//	ffnet eine Dialog zum ndern einer Farbe
//	Ergibt IDOK wenn Farbe gendert wurde.
void CAnsiView::OnColorView(){ColorView();}
int  CAnsiView::ColorView()
{
CAnsiColorDlg	sDlg(this);
int				iRet;


	if(m_cColorDisable)return IDCANCEL;

															//	Dialog zum Farben verndern aufrufen
	iRet = sDlg.DoModal(m_pGroup->pColorList,m_pGroup->hFont);

	SetCursorMode();

	if(iRet==IDOK)
		{
		ColorSave();
		RedrawGroup();
		}

	if(m_iRedrawFlags)DrawView(1);



return iRet;
}


//*****************************************************************************
//*
//*		ColorConfig
//*
//*****************************************************************************
//	Ld Farben,Schrift und Seiteneinstellungen aus der Registry.
//	pPath		: Bestimmt den Registry-Pfad aus dem die Daten geladen werden.
//					m_pGroup->pName + "\\Colors"
//				  Der Pfad wird in m_pGroup->pConfig oder m_pConfigPath
//				  gespeichert. Pfad sollte mit dem Registry-Namen aus der
//				  CWinApp erzeugt werden:
//					"Software\<AfxGetApp()->m_pszRegistryKey>\<Gruppenname>"
//	iMode		: Bestimmt von wo die Daten geladen werden.
//					1 = HKEY_LOCAL_MACHINE
//					0 = HKEY_CURREN_USER
//	Ergibt 1 wenn der Pfad gefunden wurde
int CAnsiView::ColorConfig(LPCTSTR pPath,int iMode)
{
TCHAR		cBuffer[1024];
COLORREF	uColBk;
COLORREF	uColTx;
CRegKeys	sKey;
int			iRedraw;
int			iIndex;
int			iFlag;
int			iLen;
int			iNum;
int			iRet;


	if(!pPath)return 0;

	if(m_cColorDisable)return 0;

	if(m_pConfigPath)
		{
		pPath = m_pConfigPath;
		}
	else{
		if(m_pGroup->pConfig)delete m_pGroup->pConfig;		// Pfad speichern

		iLen = _tcslen(pPath);

		if(iLen>0 && pPath[iLen-1]=='\\')iLen--;

		iNum = _tcslen(m_pGroup->pName);

			    m_pGroup->pConfig=new TCHAR[iNum+iLen+2];
		_tcscpy(m_pGroup->pConfig,pPath);
		_tcscpy(m_pGroup->pConfig+iLen,_T("\\"));
		_tcscpy(m_pGroup->pConfig+iLen+1,m_pGroup->pName);
		pPath = m_pGroup->pConfig;
		}

	if(iMode)
			iRet=sKey.Open(HKEY_LOCAL_MACHINE,pPath,CRK_READ);
	else	iRet=sKey.Open(HKEY_CURRENT_USER ,pPath,CRK_READ);


	if(!iRet)return 0;


//****************** Schrift laden ********************************************
ANSI_FONT	sAnsiFont;

	if(sKey.Get(&sAnsiFont,sizeof(sAnsiFont),_T("Font")))
		{
		m_pGroup->sAnsiFont=sAnsiFont;
		FontSet(0);
		};


//****************** Seiteneinstellungen **************************************
#ifndef TO_ANSIWND
AnsiPageData	sPrint;


	sPrint = m_sPrinterData;

	sKey.Get(&sPrint,sizeof(cBuffer),_T("Print"));

	sPrint.pHeaderData = 0;
	sPrint.pFooterData = 0;

	if(sKey.Get(cBuffer,sizeof(cBuffer),_T("Footer")))
		{
		sPrint.pFooterData    = new TCHAR[sKey.Len()+1];
		memcpy(sPrint.pFooterData,cBuffer,sKey.Len()*sizeof(TCHAR));
		}
	else{
		sPrint.pFooterData    = new TCHAR[1];
		sPrint.pFooterData[0] = 0;
		}

	if(sKey.Get(cBuffer,sizeof(cBuffer),_T("Header")))
		{
		sPrint.pHeaderData    = new TCHAR[sKey.Len()];
		memcpy(sPrint.pHeaderData,cBuffer,sKey.Len()*sizeof(TCHAR));
		}
	else{
		sPrint.pHeaderData    = new TCHAR[1];
		sPrint.pHeaderData[0] = 0;
		}

	if(m_sPrinterData.pHeaderData)delete m_sPrinterData.pHeaderData;
	if(m_sPrinterData.pFooterData)delete m_sPrinterData.pFooterData;

	m_sPrinterData = sPrint;


#endif

//****************** Farben laden **********************************************

	if(!sKey.Up(_T("Colors")))return 0;


	for(iNum=0;iNum<256;iNum++)
		{
		if(!sKey.Up())break;

		   iIndex=strtoi(sKey.Name());						// Index laden
		if(iIndex>0)
			{
			iFlag  = 0;
			uColBk = 0xFFFFFFFF;
			uColTx = 0xFFFFFFFF;

			if(sKey.Get(cBuffer,sizeof(cBuffer),_T("Name")))
				{
				sKey.Get(uColBk,_T("BG"  ));
				sKey.Get(uColTx,_T("TX"  ));
				sKey.Get(iFlag ,_T("Font"));

				if(!ColorChange(&m_pGroup->pColorList,uColTx,uColBk,cBuffer,iIndex,iFlag))
					{
					ColorAdd(uColTx,uColBk,cBuffer,iIndex,iFlag);
					}
				}
			}

		if(!sKey.Down())break;
		}


	iRedraw = m_iRedrawFlags;								// Fenster neu zeichnen
	m_iRedrawFlags = NZ_ALL;

	if(!iRedraw)DrawView(1);




return 1;
}


//*****************************************************************************
//*
//*		ColorSave
//*
//*****************************************************************************
//	Speichert Farben und Schrift-Einstellungen in der Registry
//	Ist Pfad=0 wird der Pfad aus m_pGroup->pConfig gennommen,
//	wecher ber ColorConfig gespeichert wird
//	Ergibt 1 wenn alles gespeichert wurde
int CAnsiView::ColorSave(LPCTSTR pPath)
{
CRegKeys		sKey;
LPCTSTR			pName;
TCHAR			cBuffer[256];
ANSI_FONT		sAnsiFont;
AnsiColor      *pColors;
int				uColBk;
int				uColTx;
int				iMode;
int				iNum;
int				iVal;



	if(m_cColorDisable)return 0;
	if(!pPath)pPath=m_pConfigPath;
	if(!pPath)pPath=m_pGroup->pConfig;
	if(!pPath)return 0;
	if(!sKey.Open(pPath))return 0;

//****************** Schrift speichern ****************************************


	sAnsiFont = m_pGroup->sAnsiFont;

	#ifndef 	TO_ANSIWND
	LPTSTR		pHeader;
	LPCTSTR		pFooter;

	pHeader = (m_sPrinterData.pHeaderData)? m_sPrinterData.pHeaderData:_T("");
	pFooter = (m_sPrinterData.pFooterData)? m_sPrinterData.pFooterData:_T("");
	sKey.Set(&m_sPrinterData,sizeof(m_sPrinterData),_T("Print"));
	sKey.Set(pFooter,_T("Footer"));
	sKey.Set(pHeader,_T("Header"));
	#endif
	sKey.Set(&sAnsiFont,sizeof(sAnsiFont),_T("Font"));


//****************** Farben speichern ******************************************


	if(!sKey.Up(_T("Colors")))return 0;

	iNum	= 0;
	pColors	= m_pGroup->pColorList;

	while(pColors[iNum].pName)
		{
		   iVal = pColors[iNum].iIndex;
		if(iVal)
			{
			itostr(iVal,cBuffer);

			if(sKey.Up(cBuffer))
				{
				pName	= pColors[iNum].pName;
				uColBk  = pColors[iNum].uColorBk;
				uColTx  = pColors[iNum].uColorText;
				iMode	= pColors[iNum].iFontMode;

				sKey.Set(uColBk ,_T("BG"  ));
				sKey.Set(uColTx ,_T("TX"  ));
				sKey.Set(pName  ,_T("Name"));
				sKey.Set(iMode  ,_T("Font"));

				sKey.Down();
				}
			}

		iNum++;
		}


return 1;
}

//*****************************************************************************
//*
//*		ColorChose
//*
//*****************************************************************************
//	Whlt fr einen Gertekontext die Zeichenfarbe aus
//	hDc			: Ist der Gertekontext
//	uColor		: Ist die Farbnummer, wenn die Bits 8-15 ungleich Null sind so
//				  wird daraus die Hintergrundfarbe erzeugt.
//	iFlags		: Wenn das Bit0 gesetzt ist, wird die Farbe aus den
//				  Druckerfarben m_sPrinterData.pColorList geladen.
//	Ergibt die zuletzt eingestellte Farbe.
int CAnsiView::ColorChose(HDC hDc,unsigned uColor,int iFlags)
{
AnsiColor	   *pColors;
unsigned 		uColBk;
unsigned		uColTx;
unsigned		uSelTx;
unsigned		uSelBk;
int				iColNum;


	#ifndef 	TO_ANSIWND

	int		iColorIndex;

	if(iFlags&1)											// Farbwahl beim Druckerausgabe
		{
		if(uColor==(unsigned)m_iPrintNow)return uColor;
		if(uColor>=(unsigned)m_sPrinterData.iColorCount)uColor=0;

		iColNum		= m_iPrintNow;
		m_iPrintNow	= uColor;

		   uColBk=m_sPrinterData.pColorList[uColor].uColorBk;
		if(uColBk==0xFFFFFFFF)uColBk=RGB(255,255,255);
		   uColTx=m_sPrinterData.pColorList[uColor].uColorText;
		if(uColTx==0xFFFFFFFF)uColTx=RGB(0,0,0);

		SetBkColor  (hDc,uColBk);
		SetTextColor(hDc,uColTx);

		   iColorIndex=m_sPrinterData.pColorList[uColor].iFontMode;
		if(iColorIndex<0)SelectObject(hDc,m_sPrinterData.hFontPrint);
		else             SelectObject(hDc,m_sPrinterData.aFontEx[iColorIndex&7]);

		return iColNum;
		}

	#endif

															// Farbwahl bei Bildschirmausgabe

	if(!(uColor&0xFF00))uColor|=uColor<<8;

	uSelTx = (uColor   )&0xFF;
	uSelBk = (uColor>>8)&0xFF;

	if(uColor==(unsigned)m_iColorNow)return uColor;
	if(uSelTx>=(unsigned)m_pGroup->iColorCount)uSelTx=0;
	if(uSelBk>=(unsigned)m_pGroup->iColorCount)uSelBk=0;



	pColors = m_pGroup->pColorList+uSelTx;					// Vordergrund einstellen

	   uColTx = pColors->uColorText;
	if(uColTx==0xFFFFFFFF)uColTx=GetSysColor(COLOR_WINDOWTEXT);

	SetTextColor(hDc,uColTx);

	if(m_iFontNow!=pColors->iFontMode)
		{
		   m_iFontNow=pColors->iFontMode;
		if(m_iFontNow<=0)
				SelectObject(hDc,m_pGroup->hFont);
		else	SelectObject(hDc,m_pGroup->hFontEx[m_iFontNow&7]);
		}



	pColors = m_pGroup->pColorList+uSelBk;					// Hintergrund einstellen

	   uColBk =pColors->uColorBk;
	if(uColBk==0xFFFFFFFF)
		{
		uColBk=GetSysColor(COLOR_WINDOW);
		}

	SetBkColor(hDc,uColBk);

	iColNum		= m_iColorNow;
	m_iColorNow	= uColor;


return iColNum;
}

//*****************************************************************************
//*
//*		SetCursorMode
//*
//*****************************************************************************
//	Stellt die Art des Cursors ein.
//	Wenn m_iRedrawFlags==0 ist wird der Cursor neu gezeichent
//	iMode		: Ist der neue Cursor-Modus:
//					Bit 0 : 0 = berschreiben , 1 = Einfgen
//					Bit 1 : 0 = Unsichtbar    , 1 = Sichtbar
//					Bit 2 = Aktulle Einstellung beibehalten nur neu Zeichen
//	Ergibt den letzten Cursor-Zustand
int	 CAnsiView::SetCursorMode(int iMode)
{
int		iRedraw;
int		iOld;


	iRedraw	= NZ_CURSOR;
	iOld	= m_iCursorMode;


	if(!m_hWnd)
		{
		if(!(iMode&~0x03))m_iCursorMode=iMode;
		return iOld;
		}


	if(m_cCaret){HideCaret();m_cCaret=0;}
	if(!(iMode&~0x03))m_iCursorMode=iMode;

	CreateSolidCaret((m_iCursorMode&1)? 2:m_iFontSizeX+1,m_iFontSizeY);

	if(m_iCursorMode&2)
		{
		if(!(iOld&2))iRedraw=NZ_ALL;
		if(!m_cCaret)ShowCaret();
		m_cCaret=1;
		}

	m_cCaretFocus = 0;

	if(!m_iRedrawFlags)
		{
		m_iRedrawFlags |= iRedraw;
		DrawView(1);
		}
	else{
		m_iRedrawFlags |= iRedraw;
		}


return iOld;
}

//*****************************************************************************
//*
//*		DoScroll
//*
//*****************************************************************************
//	Bewirkt ein berprfen der Scrollposition.
void CAnsiView::DoScroll(int iMode)
{
	m_iCursorOldX = m_iCursorX-((iMode&1)? 1:0);
	m_iCursorOldY = m_iCursorY-((iMode&2)? 1:0);
}

//*****************************************************************************
//*
//*		NoScroll
//*
//*****************************************************************************
//	Bewirkt das nach einer Cursor-nderung die Scrollposition nicht nachgefhrt wird.
void CAnsiView::NoScroll()
{
	m_iCursorOldX = m_iCursorX;
	m_iCursorOldY = m_iCursorY;
}

//*****************************************************************************
//*
//*		PositionCheck
//*
//*****************************************************************************
//	Prft die aktuelle Cursor und Scroll-Position.
//	iMode		: Bestimmt die Art der Prfung
//					Bit 0 = Wenn 1 wird die Scrollposition so gesetzt das der
//							Cursor in der Bildmitte ist.
//					Bit	1 = Keine Notify-Nachricht senden.
//					Bit 2 = Nicht neu zeichnen
void CAnsiView::PositionCheck(int iMode)
{
SCROLLINFO  sInfo;
BOOL		bChange;
int			iRedraw;
int			iChange;
int			iMaxX;
int			iMaxY;
int			iPosX;
int			iPosY;
int			iBits;
int			iRet;
int			iVal;


	iRedraw			= NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;

	iPosX = (m_iScreenX>>1)-m_iScrollAddX;
	iPosY = (m_iScreenY>>1)-m_iScrollAddY;

	#ifdef USE_VIRTUAL_LINES
	m_iCursorY = ANSI_STEP(m_iCursorY,0);
	m_iScrollY = ANSI_STEP(m_iScrollY,0);
	#endif


//***************** Cursor-Position im erlaubten Bereich ? ********************


	if(m_iCursorY>  m_iLineCount   )m_iCursorY=m_iLineCount;
	if(m_iCursorX>= m_iLineWidthMax)m_iCursorX=m_iLineWidthMax-1;
	if(m_iCursorY<0)m_iCursorY=ANSI_STEP(0,0);
	if(m_iCursorX<0)m_iCursorX=0;


//***************** Wurde der Fenstermodus verndert ? ************************


	if(m_iWindowModeOld!=m_iWindowMode)						// Alte Mode des Fensters
		{
		iChange = m_iWindowModeOld^m_iWindowMode;

		#ifdef TO_ANSISPLIT

		if(iChange&ANSI_SPLIT_XY)
			{
			OnSplitChanged(m_iWindowMode&ANSI_SPLIT_XY,m_aSplitNow[0],m_aSplitNow[1]);
			SetActiveField(m_iSplitActiv,FALSE);
			m_iScreenOldX = -1;
			m_iScreenOldY = -1;
			}

		#endif

		if(iChange&(ANSI_FIX_X|ANSI_NO_SCRBAR_X|ANSI_NO_SCROLL_X))m_iScreenOldX=-1;
		if(iChange&(ANSI_FIX_Y|ANSI_NO_SCRBAR_Y|ANSI_NO_SCROLL_Y))m_iScreenOldY=-1;

		if(m_iWindowMode&ANSI_NO_SCROLL_X)m_iScrollX=0;
		if(m_iWindowMode&ANSI_NO_SCROLL_Y)m_iScrollY=ANSI_STEP(0,0);

		m_iWindowModeOld = m_iWindowMode;
		}

//***************** Wurde Cursor-Position verndert ? *************************


	if(m_iCursorOldX!=m_iCursorX)							// Wurde der Cursor in X-Richtung bewegt
		{
		m_iRedrawFlags |= NZ_CURSOR;
		m_iCursorOldX   = m_iCursorX;

		if(m_iCursorOldY==m_iCursorY)
			{
			m_iCursorOldY = m_iCursorY+10;
			}

		if(m_iScreenSizeX>0)
			{
			iMaxX = m_iScreenX-m_iScrollAddX;
			if(m_iCursorX>=m_iScrollX+iMaxX)m_iScrollX=m_iCursorX-iMaxX+1;
			if(m_iCursorX< m_iScrollX      )m_iScrollX=m_iCursorX;
			}
		}


	if(m_iCursorOldY!=m_iCursorY)							// Wurde der Cursor in Y-Richtung bewegt
		{
		m_iRedrawFlags |= NZ_CURSOR;
		iPosY			= m_iCursorOldY-m_iCursorY;
		m_iCursorOldY	= m_iCursorY;

		if(m_iScreenSizeY>0)
			{
			iMaxY = m_iScreenY-m_iScrollAddY;

			if(!(iMode&1))iPosY=0;

			if(iPosY==1          || iPosY==-1         )iPosY=0;
			if(iPosY==m_iScreenY || iPosY==-m_iScreenY)iPosY=0;
			if(iPosY)iPosY=m_iScreenY/2;

			if(m_iCursorY>=m_iScrollY+iMaxY)
				{
				m_iScrollY = ANSI_STEP(m_iCursorY,iPosY-iMaxY+1);
				}

			if(m_iCursorY<m_iScrollY)
				{
				m_iScrollY = ANSI_STEP(m_iCursorY,-iPosY);
				}
			}
		}


//***************** Scroll-Position im erlaubten Bereich ? ********************


	if(m_iScrollX+iPosX>m_iLineWidthMax)					// Zu weit nach rechts gescrollt ?
		{													
		m_iScrollX 	 = m_iLineWidthMax-iPosX;
		m_iOriginPtX = 0;

		#ifdef	TO_ANSISPLIT

		m_aScrollX[m_iSplitActiv&1] = m_iScrollX;

		#endif 	
		}

	if(m_iScrollX<0)										// Zu weit nach links gescrollt ?
		{
		m_iScrollX   = 0;
		m_iOriginPtX = 0;

		#ifdef	TO_ANSISPLIT

		m_aScrollX[m_iSplitActiv&1] = m_iScrollX;

		#endif 	
		}

	if(m_iScrollY+iPosY>m_iLineCount)						// Zu weit nach unten gescrollt ?		
		{
		m_iScrollY   = ANSI_STEP(m_iLineCount,-iPosY);
		m_iOriginPtY = 0;

		#ifdef	TO_ANSISPLIT

		m_aScrollY[m_iSplitActiv>>1] = m_iScrollY;

		#endif 	
		}

	if(m_iScrollY<0)										// Zu weit nach oben gescrollt ?
		{
		m_iScrollY   = ANSI_STEP(0,0);
		m_iOriginPtY = 0;

		#ifdef	TO_ANSISPLIT

		m_aScrollY[m_iSplitActiv>>1] = m_iScrollY;

		#endif 	
		}

	#ifdef	TO_ANSISPLIT

	for(iVal=0;iVal<2;iVal++)
		{
		if(m_aScrollX[iVal]<0)
			{
			m_aScrollX[iVal] = 0;
			}

		if(m_aScrollY[iVal]<0)
			{
			m_aScrollY[iVal] = 0;
			}
		}

	#endif

//***************** Wurde Scroll-Position verndert ? *************************


	bChange = FALSE;
	iBits   = 0;

	if(m_iScrollOldX!=m_iScrollX)
	do	{
		if(m_iWindowMode&ANSI_NO_SCROLL_X)
			{
				m_iScrollX	= 0;
			if(!m_iScrollOldX)break;
			}

		#ifdef	TO_ANSISPLIT

		m_aScrollX[m_iSplitActiv&1] = m_iScrollX;			// Felderdaten schreiben

		#endif

		SetScrollPos(SB_HORZ ,m_iScrollX,TRUE);
		m_iScrollOldX		= m_iScrollX;
		m_iRedrawFlags		= NZ_ALL;
		bChange				= TRUE;
		}while(0);


	if(m_iScrollOldY!=m_iScrollY)
	do	{
		if(m_iWindowMode&ANSI_NO_SCROLL_Y)
			{
			m_iScrollY = ANSI_STEP(0,0);
			if(!m_iScrollOldY)break;
			}

		#ifdef	TO_ANSISPLIT

		m_aScrollY[m_iSplitActiv>>1] = m_iScrollY;			// Felderdaten schreiben

		#endif

		SetScrollPos(SB_VERT ,m_iScrollY,TRUE);
		m_iScrollOldY		= m_iScrollY;
		m_iRedrawFlags		= NZ_ALL;
		bChange				= TRUE;
		}while(0);


	#ifdef	TO_ANSISPLIT

	iVal = (m_iSplitActiv&1)? 0:1;

	if(m_aSplitOn[0] && m_iScrollOldXB!=m_aScrollX[iVal])	// Wurde die inaktive X-Seite gescrollt
	do	{
		if(m_iWindowMode&ANSI_NO_SCROLL_X)
			{
			m_aScrollX[iVal] = ANSI_STEP(0,0);
			if(!m_iScrollOldXB)break;
			}

		SetScrollPos(SB_HORZ_X,m_aScrollX[iVal],TRUE);
		m_iScrollOldXB		= m_aScrollX[iVal];
		m_iRedrawFlags		= NZ_ALL;
		}while(0);


	iVal = (m_iSplitActiv&2)? 0:1;

	if(m_aSplitOn[1] && m_iScrollOldYB!=m_aScrollY[iVal])	// Wurde die inaktive Y-Seite gescrollt
	do	{
		if(m_iWindowMode&ANSI_NO_SCROLL_Y)
			{
			m_aScrollY[iVal] = ANSI_STEP(0,0);
			if(!m_iScrollOldYB)break;
			}

		SetScrollPos(SB_VERT_X,m_aScrollY[iVal],TRUE);
		m_iScrollOldYB		= m_aScrollY[iVal];
		m_iRedrawFlags		= NZ_ALL;
		}while(0);

	#endif


	if(bChange && !(iMode&2))								// Notify-Nachricht fr Scroll
		{
		OnScrollChange();
		SendNotify(ANSI_SCROLL,m_iScrollX,m_iScrollY);
		}


//***************** Wurde Cursor-Modus verndert ? ****************************


	if(m_iCursorMode!=m_iCursorModeOld)
		{
		SetCursorMode();
		m_iCursorModeOld	= m_iCursorMode;
		m_iRedrawFlags	   |= NZ_CURSOR;
		}


//***************** Wurde Bild/Bildschirmbreite verndert ? *******************


	#ifdef	TO_ANSISPLIT

	iVal = (m_iSplitActiv&1)? 0:1;

	if(m_aSplitOn[0])
	if(m_iLineWidthMaxOld!=m_iLineWidthMax || m_iScreenOldXB!=m_aScreenX[iVal])
		{
		sInfo.cbSize		= sizeof(SCROLLINFO);
		sInfo.fMask			= SIF_ALL;
		sInfo.nMin  		= 0;
		sInfo.nMax  		= m_iLineWidthMax;
		sInfo.nPos			= m_aScrollX[iVal];
		sInfo.nPage 		= m_aScreenX[iVal];
		sInfo.nTrackPos		= 0;

		if(m_iWindowMode&ANSI_SPLIT_X)						// Bei splitbaren Fenster Scrollbar immer anzeigen
			{
			sInfo.fMask |= SIF_DISABLENOSCROLL;
			sInfo.nMax  += m_aScreenX[iVal]/2-1;
			}
		else{
			if(!(m_iWindowMode&ANSI_FIX_X       ))sInfo.nMax +=m_aScreenX[iVal]/2-1;
			if(  m_iWindowMode&ANSI_NO_SCRBAR_X  )sInfo.nMax  =0;
			if(  m_iWindowMode&ANSI_SHOW_SCRBAR_X)sInfo.fMask|=SIF_DISABLENOSCROLL;
			}

		if(m_aScrollX[iVal]>sInfo.nMax-(int)sInfo.nPage)	// Scroll-Position prfen
			{
			   iVal = sInfo.nMax-sInfo.nPage;
			if(iVal<0)iVal=0;

			if(m_aScrollX[iVal]>iVal)
				{
				m_aScrollX[iVal] = iVal;
				DrawViewAll();
				}
			}

		iRet = SetScrollInfo(SB_HORZ_X,&sInfo,0x03);

		if(iRet<2 && m_aScreenX[iVal]>m_iScreenOldXB)		// Wurde das Fenster breiter
			{
			m_iRedrawFlags	= NZ_ALL;
			}

		m_iScreenOldXB = m_aScreenX[iVal];
		}

	#endif


	if(m_iLineWidthMaxOld!=m_iLineWidthMax || m_iScreenOldX!=m_iScreenX)
		{
		sInfo.cbSize		= sizeof(SCROLLINFO);
		sInfo.fMask			= SIF_ALL;
		sInfo.nMin  		= 0;
		sInfo.nMax  		= m_iLineWidthMax;
		sInfo.nPos			= m_iScrollX;
		sInfo.nPage 		= m_iScreenX;
		sInfo.nTrackPos		= 0;

		#ifdef	TO_ANSISPLIT

		if(m_iWindowMode&ANSI_SPLIT_X)						// Bei splitbaren Fenster Scrollbar immer anzeigen
			{
			sInfo.fMask |= SIF_DISABLENOSCROLL;
			sInfo.nMax  += m_iScreenX/2-1;
			}
		else

		#endif

		{
		if(!(m_iWindowMode&ANSI_FIX_X       ))sInfo.nMax +=m_iScreenX/2-1;
		if(  m_iWindowMode&ANSI_NO_SCRBAR_X  )sInfo.nMax  =0;
		if(  m_iWindowMode&ANSI_SHOW_SCRBAR_X)sInfo.fMask|=SIF_DISABLENOSCROLL;
		}

		if(m_iScrollX>sInfo.nMax-(int)sInfo.nPage)			// Scroll-Position prfen
			{
			   iVal = sInfo.nMax-sInfo.nPage;
			if(iVal<0)iVal=0;

			if(m_iScrollX>iVal)
				{
				m_iScrollX = iVal;
				DrawViewAll();
				}
			}


		iRet = SetScrollInfo(SB_HORZ,&sInfo,0x03);

		iBits			   |= (iRet&2)<<29;
		iBits			   |= m_iScreenOldX^m_iScreenX;
		m_iLineWidthMaxOld	= m_iLineWidthMax;

		if(iRet<2 && m_iScreenX>m_iScreenOldX)				// Wurde das Fenster breiter
			{
			m_iRedrawFlags	= NZ_ALL;
			}
		}


//***************** Wurde Bild/Bildschirmhhe verndert ? ********************


	#ifdef	TO_ANSISPLIT

	iVal = (m_iSplitActiv&2)? 0:1;

	if(m_aSplitOn[1])
	if(m_iLineCountOld!=m_iLineCount || m_iScreenOldYB!=m_aScreenY[iVal])
		{
		sInfo.cbSize		= sizeof(SCROLLINFO);
		sInfo.fMask			= SIF_ALL;
		sInfo.nMin  		= 0;
		sInfo.nMax  		= m_iLineCount;
		sInfo.nPos  		= m_aScrollY[iVal];
		sInfo.nPage 		= m_aScreenY[iVal];
		sInfo.nTrackPos		= 0;

		if(m_iWindowMode&ANSI_SPLIT_Y)						// Bei splitbaren Fenster Scrollbar immer anzeigen
			{
			sInfo.fMask |= SIF_DISABLENOSCROLL;
			sInfo.nMax  += m_aScreenY[iVal]/2-2;
			}
		else{
			if(!(m_iWindowMode&ANSI_FIX_Y       ))sInfo.nMax +=m_aScreenY[iVal]/2-2;
			if(  m_iWindowMode&ANSI_NO_SCRBAR_Y  )sInfo.nMax  =0;
			if(  m_iWindowMode&ANSI_SHOW_SCRBAR_Y)sInfo.fMask|=SIF_DISABLENOSCROLL;
			}

		if(m_aScrollY[iVal]>sInfo.nMax-(int)sInfo.nPage)	// Scroll-Position prfen
			{
			   iVal = ANSI_STEP(m_iLineCount,-m_iScreenX/2);
			if(iVal<0)iVal=0;

			if(m_aScrollY[iVal]>iVal)
				{
				m_aScrollY[iVal] = iVal;
				DrawViewAll();
				}
			}

		if(m_aScreenY[iVal]>m_iScreenOldYB)					// Wurde das Fenster hher
			{
			DrawViewFrom(m_iLineCountOld);
			}

		iRet = SetScrollInfo(SB_VERT_X,&sInfo,0x03);

		m_iScreenOldYB = m_aScreenY[iVal];
		}

	#endif


	if(m_iLineCountOld!=m_iLineCount || m_iScreenOldY!=m_iScreenY)
		{
		sInfo.cbSize		= sizeof(SCROLLINFO);
		sInfo.fMask			= SIF_ALL;
		sInfo.nMin  		= 0;
		sInfo.nMax  		= m_iLineCount;
		sInfo.nPos  		= m_iScrollY;
		sInfo.nPage 		= m_iScreenY;
		sInfo.nTrackPos		= 0;

		#ifdef	TO_ANSISPLIT

		if(m_iWindowMode&ANSI_SPLIT_Y)						// Bei splitbaren Fenster Scrollbar immer anzeigen
			{
			sInfo.fMask |= SIF_DISABLENOSCROLL;
			sInfo.nMax  += m_iScreenY/2-2;
			}
		else

		#endif

		{
		if(!(m_iWindowMode&ANSI_FIX_Y       ))sInfo.nMax +=m_iScreenY/2-2;
		if(  m_iWindowMode&ANSI_NO_SCRBAR_Y  )sInfo.nMax  =0;
		if(  m_iWindowMode&ANSI_SHOW_SCRBAR_Y)sInfo.fMask|=SIF_DISABLENOSCROLL;
		}

		if(m_iScrollY>sInfo.nMax-(int)sInfo.nPage)			// Scroll-Position prfen
			{
			   iVal = ANSI_STEP(m_iLineCount,-m_iScreenY/2);
			if(iVal<0)iVal=0;

			if(m_iScrollY>iVal)
				{
				m_iScrollY = iVal;
				DrawViewAll();
				}
			}

		if(m_iScreenY>m_iScreenOldY)						// Wurde das Fenster hher
			{
			DrawViewFrom(m_iLineCountOld);
			}

		iRet = SetScrollInfo(SB_VERT,&sInfo,0x03);

		iBits			   |= (iRet&2)<<29;
		iBits			   |= m_iScreenOldY^m_iScreenY;
		m_iLineCountOld		= m_iLineCount;
		}


	if(iBits)												// Vernderungen der Zeilen an abgeleitete Klassen melden
		{
		if(iBits&(2<<29))
			{
			OnSize(0,m_iScreenFullX,m_iScreenFullY);
			}

		if(iBits&0x0FFFFFFF)
			{
			OnSize(m_iScreenX,m_iScreenY,m_iScreenX-m_iScreenOldX,m_iScreenY-m_iScreenOldY);
			}

		m_iScreenOldX		= m_iScreenX;
		m_iScreenOldY		= m_iScreenY;
		}


	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;


	if((iRedraw&NZ_LOCK) || (iMode&4))						// Nicht neu zeichnen
		{
		return;
		}

	if(!iRedraw && m_iRedrawFlags)
		{
		DrawView(1);
		}

}


//*****************************************************************************
//*
//*		SetMode
//*
//*****************************************************************************
//	ndert die Modusbits des Fensters
//	iMode		: Sind die neuen Bits
//					ANSI_NO_SCROLL	 = Kein    Scrollen des Fensters
//					ANSI_NO_SCROLL_Y = Kein  X-Scrollen des Fensters
//					ANSI_NO_SCROLL_Y = Kein  Y-Scrollen des Fensters
//					ANSI_NO_SCRBAR	 = Keine   Scroll-Leisten
//					ANSI_NO_SCRBAR_X = Keine X-Scroll-Leiste
//					ANSI_NO_SCRBAR_Y = Keine Y-Scroll-Leiste
//					ANSI_FIX		 = Bei genauem Fensterma    die Scrollbars entfernen
//					ANSI_FIX_X		 = Bei genauer Fensterbreite die Scrollbar  entfernen
//					ANSI_FIX_Y		 = Bei genauer Fensterhhe   die Scrollbar  entfernen
//					ANSI_SPLIT		 = Das Fenster kann in X und Y Richtung geteilt werden
//					ANSI_SPLIT_X	 = Das Fenster kann in X Richtung geteilt werden
//					ANSI_SPLIT_Y	 = Das Fenster kann in Y Richtung geteilt werden
//	iMask		: Weche Bits sollen gendert werden
//	Ergibt die vorherigen Modusbits.
int CAnsiView::SetMode(int iMode,int iMask)
{
int		iOld;
int		iNew;

	iOld  = m_iWindowMode;
	iNew  = m_iWindowMode&~iMask;
	iNew |=         iMode& iMask;

	if(iNew==iOld)return iOld;

	m_iWindowMode = iNew;
	PositionCheck();


return iOld;
}


//*****************************************************************************
//*
//*		PointToPos
//*
//*****************************************************************************
//	Berechnet aus Koordinaten eine Textposition
//	sPoint		: Pixel-Koordinaten vom Fenster
//	iPosX/Y		: Hier wird die Textposition gespeichert
//	iMode		: Bestimmt die Art wie die Koordinaten umgerechnet werden
//					Bit0 = LO:direkt berechnen  HI:nheste Position
//					Bit1 = LO:Feld auflsen    HI:Feld nicht auflsen
//	Ergibt wo sich die Koordinaten befinden
//					0 = nicht im Fentser
//					1 = auf Text
//					2 = auf Rand
int CAnsiView::PointToPos(CPoint sPoint,int &iPosX,int &iPosY,int iMode)
{
int		iScrollX;
int		iScrollY;
int		iAdd;


	iScrollX = m_iScrollX;
	iScrollY = m_iScrollX;


	#ifdef TO_ANSISPLIT

	int		iField;
	CPoint	sBase;

	if(!(iMode&2))											// Die Koordinate vom Feld umrechen
		{
		   iField = GetFieldOfPoint(sPoint,&sBase);
		if(iField&4)
			{
			sPoint.x -= sBase.x;
			sPoint.y -= sBase.y;
			}
		
		iScrollX = m_aScrollX[(iField   )&1];
		iScrollY = m_aScrollY[(iField>>1)&1];
		}

	#endif 	


	iAdd  = (iMode&1)? m_iFontSizeX/2:0;
	iAdd -=  m_iMarginWidth;
	iPosX = (sPoint.x+iAdd)/m_iFontSizeX+iScrollX;
	iPosY = (sPoint.y     )/m_iFontSizeY+iScrollY;


	if(iPosY<0)												// Vor der ersten Zeile
		{
		iPosX = 0;
		iPosY = 0;
		return 0;
		}
	
	if(iPosY>=m_iLineCount)									// Hinter der letzten Zeile
		{
		iPosX = 0;
		iPosY = m_iLineCount;

		return 0;
		}
	
	if(iPosX<0)												// Links vor dem Begin
		{
		iPosX = 0;
		}

	if(sPoint.x<m_iMarginWidth && sPoint.x>=0)				// Druck auf Rand
		{
		return 2;
		}


return 1;
}

//*****************************************************************************
//*
//*		SendNotify
//*
//*****************************************************************************
//	Eine Notify-Nachricht senden
void CAnsiView::SendNotify(int iType,int iParam1,int iParam2)
{
ANSI_NMHDR	 sNotify;
CWnd		*pWnd;


	   pWnd = GetParent();
	if(pWnd)
		{
		sNotify.sHdr.code		= iType;
		sNotify.sHdr.hwndFrom	= m_hWnd;
		sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
		sNotify.iParam1			= iParam1;
		sNotify.iParam2			= iParam2;

		pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);
		}

}

//*****************************************************************************
//*
//*		FontCreate
//*
//*****************************************************************************
//	Erzeugt einen Font mit den Daten aus pFontData
HFONT CAnsiView::FontCreate(ANSI_FONT *pFontData)
{
LOGFONT		sFont;


	memset(&sFont,0,sizeof(sFont));

	sFont.lfHeight			=  pFontData->iHeight;
	sFont.lfWidth			=  pFontData->iWidth;
	sFont.lfUnderline		= (pFontData->iFlags&ANSI_UNDERLINE);
	sFont.lfItalic			= (pFontData->iFlags&ANSI_ITALIC   );
	sFont.lfWeight			= (pFontData->iFlags&ANSI_BOLD     )? FW_BOLD:FW_NORMAL;
	sFont.lfClipPrecision	= (pFontData->iFlags&ANSI_TRUETYPE )? OUT_OUTLINE_PRECIS|OUT_TT_ONLY_PRECIS:CLIP_STROKE_PRECIS;
	sFont.lfPitchAndFamily	= (pFontData->iFlags&ANSI_FIXED    )? FIXED_PITCH:VARIABLE_PITCH;
	sFont.lfOutPrecision	=  OUT_RASTER_PRECIS;
	sFont.lfQuality			=  DEFAULT_QUALITY;
	sFont.lfCharSet			=  ANSI_CHARSET;

	strlcpy(sFont.lfFaceName,pFontData->cFontName,sizeof(sFont.lfFaceName));


return CreateFontIndirect(&sFont);
}


//*****************************************************************************
//*
//*		FontMetrics
//*
//*****************************************************************************
//	Speichert die Abmessungen von einer Schrift in pMetrics
void CAnsiView::FontMetrics(HFONT hFont,ANSI_FONT_METRICS *pMetrics,HDC hDefDc)
{
TEXTMETRIC	sData;
HDC			hDc;



	memset(pMetrics,0,sizeof(ANSI_FONT_METRICS));

	if(hDefDc)hDc=hDefDc;
	else	  hDc=::GetDC(m_hWnd);

	SelectObject(hDc,hFont);

	if(GetTextMetrics(hDc,&sData))
		{
		pMetrics->iCharWidth  =  sData.tmAveCharWidth;
		pMetrics->iCharHeight =  sData.tmHeight;
		pMetrics->iIsVector	  = (sData.tmPitchAndFamily&TMPF_VECTOR)? 1:0;
		}

	if(!hDefDc)::ReleaseDC(m_hWnd,hDc);


}

//*****************************************************************************
//*
//*		FontSet
//*
//*****************************************************************************
//	Stellt den Zeichensatz ein.
//	Setzt fr die Ganze Gruppe eine neue Schriftart
//	Wird fr die Hhe Null bergeben werden die Parameter der
//	aktullen sAnsiFont Strukur zum Erzeugen benutzt
void CAnsiView::FontSet(int iHeight,LPCTSTR pName)
{
LPCTSTR				pFontName[]={_T(""),_T("Courier"),_T("Terminal"),_T("Fixedsys"),_T(""),0};
ANSI_FONT			sAnsiFont;
ANSI_FONT_METRICS	sData;
HFONT				hFont;
int					iRedraw;
int					i;




	sAnsiFont	= m_pGroup->sAnsiFont;
	hFont		= 0;

	if(iHeight==0)
		{
		pFontName[0]=sAnsiFont.cFontName;
		}
	else{
		pFontName[0]=pName;

		if(iHeight<ANSI_FONT_MIN)iHeight=ANSI_FONT_MIN;
		if(iHeight>ANSI_FONT_MAX)iHeight=ANSI_FONT_MAX;

		sAnsiFont.iHeight		= iHeight;
		sAnsiFont.iWidth		= 0;
		sAnsiFont.iFlags        = ANSI_FIXED;
		sAnsiFont.cFontName[0]	= 0;
		}


	for(i=0;i<10;i++)										// Erstellen eines Fonts mit den Standartnamen
		{
		if(!pFontName[i])break;

		strlcpy(sAnsiFont.cFontName,pFontName[i],sizeof(sAnsiFont.cFontName));

		   hFont=FontCreate(&sAnsiFont);
		if(hFont)break;
		}

	if(hFont)												// Font speichern
		{
		for(i=0;i<8;i++)
			{
			if(m_pGroup->hFontEx[i])
			if(m_pGroup->hFontEx[i]==m_pGroup->hFont)
				{
				m_pGroup->hFontEx[i]=0;
				}
			}

		if(m_pGroup->hFont)FontDelete(m_pGroup->hFont);
			{
			m_pGroup->sAnsiFont	= sAnsiFont;
			m_pGroup->hFont		= hFont;
			}
		}


	if(IsWindow(m_hWnd))									// Font-Breite abmessen
		{
		FontMetrics(m_pGroup->hFont,&sData);

															// Cursor-Marke ausblenden bei nderungen
		if(m_cCaret && m_iFontSizeX!=sData.iCharWidth || m_iFontSizeY!=sData.iCharHeight)
			{
			HideCaret();
			m_cCaret = 0;
			}

		m_pGroup->iFontSizeY = m_iFontSizeY;

		   m_iFontSizeX=sData.iCharWidth;
		   m_iFontSizeY=sData.iCharHeight;
		if(m_iFontSizeX<=0)m_iFontSizeX=1;
		if(m_iFontSizeY<=0)m_iFontSizeY=1;

		m_pGroup->iFontSizeX = m_iFontSizeX;
		m_pGroup->iFontSizeY = m_iFontSizeY;

		if(m_pGroup->hFont)									// Zustatz Font's speichern
			{
			for(i=0;i<8;i++)
				{
				if(m_pGroup->hFontEx[i])
				if(m_pGroup->hFontEx[i]!=m_pGroup->hFont)
					{
					FontDelete(m_pGroup->hFontEx[i]);
					}

				sAnsiFont.iHeight		 = m_iFontSizeY;
				sAnsiFont.iWidth		 = m_iFontSizeX;

					   sAnsiFont.iFlags  = ANSI_FIXED;
				if(i&1)sAnsiFont.iFlags |= ANSI_BOLD;
				if(i&2)sAnsiFont.iFlags |= ANSI_ITALIC;
				if(i&4)sAnsiFont.iFlags |= ANSI_UNDERLINE;

				hFont=FontCreate(&sAnsiFont);

				if(!hFont)
					{
					m_pGroup->hFontEx[i]=m_pGroup->hFont;
					continue;
					}

				FontMetrics(hFont,&sData);					// Prfe Gre

				if(m_iFontSizeX!=sData.iCharWidth || m_iFontSizeY!=sData.iCharHeight)
					{
					FontDelete(hFont);
					m_pGroup->hFontEx[i]=m_pGroup->hFont;
					continue;
					}

				m_pGroup->hFontEx[i]=hFont;
				}
			}
		}




															// Neue Bildschirmgren berechnen
	m_iScreenX = (m_iScreenSizeX+m_iFontSizeX-1-m_iMarginWidth)/m_iFontSizeX;
	m_iScreenY = (m_iScreenSizeY+m_iFontSizeY-1               )/m_iFontSizeY;

	if(m_iScreenX<=0)m_iScreenX=1;
	if(m_iScreenY<=0)m_iScreenY=1;


	iRedraw			= m_iRedrawFlags;						// Fenster neu zeichnen
	m_iRedrawFlags	= NZ_ALL;

	if(!iRedraw)DrawView(1);


	if(IsWindow(m_hWnd))
		{
		SetCursorMode();
		PositionCheck(2);
		RedrawGroup();
		}

}

//*****************************************************************************
//*
//*		FontView
//*
//*****************************************************************************
//	Erzeugt Dialog-Fenster zum auswhlen des Zeichensatzes
void CAnsiView::OnFontView(){FontView();}
int  CAnsiView::FontView()
{
CHOOSEFONT	sChose;
LOGFONT		sFont;
HDC			hDc;



	sFont.lfHeight			= m_pGroup->sAnsiFont.iHeight;
	sFont.lfWidth			= m_pGroup->sAnsiFont.iHeight;
	sFont.lfEscapement	 	= 0;
	sFont.lfOrientation		= 0;
	sFont.lfWeight		 	= (m_pGroup->sAnsiFont.iFlags&ANSI_BOLD)? FW_BOLD:FW_NORMAL;
	sFont.lfItalic		 	= (m_pGroup->sAnsiFont.iFlags&ANSI_ITALIC);
	sFont.lfUnderline		= (m_pGroup->sAnsiFont.iFlags&ANSI_UNDERLINE);
	sFont.lfStrikeOut		= FALSE;
	sFont.lfCharSet			= ANSI_CHARSET;
	sFont.lfOutPrecision	= OUT_RASTER_PRECIS;
	sFont.lfClipPrecision	= CLIP_STROKE_PRECIS;
	sFont.lfQuality			= DEFAULT_QUALITY;
	sFont.lfPitchAndFamily	= FIXED_PITCH;

	hDc=::GetDC(m_hWnd);

	sChose.lStructSize		= sizeof(CHOOSEFONT);
	sChose.hwndOwner		= m_hWnd;
	sChose.hDC				= hDc;
	sChose.lpLogFont		= &sFont;
	sChose.iPointSize		= 10;
	sChose.Flags			= CF_LIMITSIZE|CF_FIXEDPITCHONLY|CF_BOTH|CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT;
	sChose.rgbColors		= GetSysColor(COLOR_WINDOWTEXT);
	sChose.lCustData		= NULL;
	sChose.lpfnHook			= NULL;
	sChose.lpTemplateName	= NULL;
	sChose.hInstance		= NULL;
	sChose.lpszStyle		= NULL;
	sChose.nFontType		= 0;
	sChose.nSizeMin			= ANSI_FONT_MIN;
	sChose.nSizeMax			= ANSI_FONT_MAX;

	::ReleaseDC(m_hWnd,hDc);

	if(ChooseFont(&sChose))
		{
		m_pGroup->sAnsiFont.iHeight = sFont.lfHeight;
		m_pGroup->sAnsiFont.iWidth  = sFont.lfWidth;
		m_pGroup->sAnsiFont.iFlags  = ANSI_FIXED;
		strlcpy(m_pGroup->sAnsiFont.cFontName,sFont.lfFaceName,sizeof(m_pGroup->sAnsiFont.cFontName));

		FontSet  (0);
		ColorSave( );										// Kofiguration speichern


		m_iRedrawFlags=NZ_ALL;
		SetCursorMode();
		DrawView(1);

		return 1;
		}

	if(m_iRedrawFlags)
		{
		DrawView(1);
		}


return 0;
}

//*****************************************************************************
//*
//*		FontInfo
//*
//*****************************************************************************
//	Holt Informationen zu altuellen Font einer Fenstergruppe
//	pGroupName	: Name der Gruppe
//	iWidth		: Hier wird die Schriftbreite gespeichert
//	iHeight		: Hier wird die Schrifthhe gespeichert
//	Ergibt TRUE wenn die Daten geholt werden konnten
BOOL CAnsiView::FontInfo(LPCTSTR pGroupName,int &iWidth,int &iHeight)
{
CRegKeys			sKey;
ANSI_FONT			sAnsiFont;
TEXTMETRIC			sData;
HFONT				hFont;
HDC					hWndDc;
int					iRet;


	if(pGroupName)											// Font aus Registry holen
		{
				 iRet=sKey.Open(HKEY_CURRENT_USER ,pGroupName,CRK_READ);
		if(!iRet)iRet=sKey.Open(HKEY_LOCAL_MACHINE,pGroupName,CRK_READ);
		if(!iRet)return FALSE;

		if(!sKey.Get(&sAnsiFont,sizeof(sAnsiFont),_T("Font")))
			{
			return FALSE;
			};
		}
	else{													// Standart-Font benuzen
		_tcscpy(sAnsiFont.cFontName,_T("Courier"));
		sAnsiFont.iHeight		= ANSI_FONT_STD;
		sAnsiFont.iFlags        = ANSI_FIXED;
		sAnsiFont.iWidth		= 0;
		}

		hFont=FontCreate(&sAnsiFont);
	if(!hFont)return FALSE;

	hWndDc = ::GetDC(NULL);

	SelectObject(hWndDc,hFont);

	if(!GetTextMetricsA(hWndDc,&sData))
		{
		sData.tmAveCharWidth  = 0;
		}

	::ReleaseDC(NULL,hWndDc);
	DeleteObject(hFont);

	if(sData.tmAveCharWidth<=0)return FALSE;
	if(sData.tmHeight      <=0)return FALSE;

	iWidth	= sData.tmAveCharWidth;
	iHeight	= sData.tmHeight;


return TRUE;
}

//*****************************************************************************
//*
//*		RedrawGroup
//*
//*****************************************************************************
//	Alle Fenster einer Gruppe neuzeichen
void CAnsiView::RedrawGroup()
{
CAnsiView	*pClass;
int			 iId;



	pClass	= m_pClassLast;
	iId		= m_pGroup->iId;


	while(pClass)											// Alle Fenster durchsuchen
		{
		if(iId==m_pGroup->iId)
			{
			if(m_pGroup->iFontSizeX>0 && pClass->m_iFontSizeX!=m_pGroup->iFontSizeX)
				{
				pClass->m_iFontSizeX = m_pGroup->iFontSizeX;

				if(m_iScreenSizeX>=0 && m_iScreenSizeY>0)
					{
					pClass->m_iScreenSizeX += 1;
					pClass->OnSize(0,m_iScreenFullX-1,m_iScreenFullY);
					}
				}

			if(m_pGroup->iFontSizeY>0 && pClass->m_iFontSizeY!=m_pGroup->iFontSizeY)
				{
				pClass->m_iFontSizeY = m_pGroup->iFontSizeY;

				if(m_iScreenSizeX>=0 && m_iScreenSizeY>0)
					{
					pClass->m_iScreenSizeY += 1;
					pClass->OnSize(0,m_iScreenFullX,m_iScreenFullY-1);
					}
				}

			if(IsWindow(pClass->m_hWnd))
				{
				pClass->m_iRedrawFlags = NZ_ALL;
				pClass->DrawView(1);
				}
			}

		pClass = pClass->m_pClassPrev;
		}


}

//*****************************************************************************
//*
//*		UpdateGroup
//*
//*****************************************************************************
//	Alle Fenster auf den neuersten Stand bringen.
//	iMode	: An Update() weiter gegeben siehe CEditor::Update()
void CAnsiView::UpdateGroup(int iMode)
{
CAnsiView  *pClass;
int			iId;



	pClass	= m_pClassLast;
	iId		= m_pGroup->iId;

	while(pClass)											// Alle Fenster durchsuchen
		{
		if(iId==m_pGroup->iId)
			{
			pClass->m_iRedrawFlags = NZ_ALL;
			pClass->Update(iMode);
			}

		pClass=pClass->m_pClassPrev;
		}


}

//*****************************************************************************
//*
//*		Update
//*
//*****************************************************************************
//	Fenster auf den neuersten Stand bringen
//	Dient nur zum Ableiten fr andere Klassen
void CAnsiView::Update(int iMode)
{

}

//*****************************************************************************
//*
//*		PrintLine
//*
//*****************************************************************************
//	Eine Zeile auf den Schirm zeichnen.
//	hDc		: Handle vom Gertekontext.
//	pRect	: Pixelkoordinaten des Bereiches in dem gezeichnet werden soll.
//	iLine	: Zeilennummer des Textes.
//	iStart	: Startzeichen in der Zeile.
//	iSize	: Anzahl der auszugebenden Zeichen.
//	iFlags  : Diverse Bits fr die Zeile
//				Bit0	  = Ist fr die die Ausgabe auf einem Drucker bei einem Aufruf aus OnPrint()
//				Bit1	  = Ist es nicht das aktive Feld bei TO_ANSISPLIT
//				Bit8..9   = Ist die Feldnummer bei TO_ANSISPLIT
//				Bit12..31 = Ist der X-Ursprung fr Tabulatoren
//			  iFlags muss an ColorChose weiter gegeben werden !
//	Ergibt die letzte Koordinate in Zeichen der ausgegeben Zeile.
int CAnsiView::PrintLine(HDC hDc,RECT *pRect,int iLine,int iStart,int iSize,int iFlags)
{
char   *pText="CAnsiView::PrintLine() wurde nicht abgeleitet !";
int		iLen;


	if(iLine>=m_iLineCount)return 0;

	iLen = strlen(pText);

	if(iLen<=iStart)return 0;

	DrawTextA(hDc,pText+iStart,iLen-iStart,pRect,DT_LEFT|DT_TOP|DT_NOCLIP|DT_NOPREFIX);


return	iLen-iStart;
}

//*****************************************************************************
//*
//*		PrintText
//*
//*****************************************************************************
//	Eine Zeile auf den Schirm mit Farben zeichnen.
//	hDc		: Handle vom Gertekontext.
//	pRect	: Pixelkoordinaten des Bereiches in dem gezeichnet werden soll.
//	pText	: Zeiger auf den Text der ausgegeben weren muss
//	pColors	: Zeiger auf die Farben fr den Text
//	iSize	: Anzahl der auszugebenden Zeichen.
//	iFlags  : Diverse Bits fr die Zeile
//				Bit0	= Ist fr die die Ausgabe auf einem Drucker bei einem Aufruf aus OnPrint()
//				Bit1	= Ist es das aktive Feld bei TO_ANSISPLIT
//				Bit8..9 = Ist die Feldnummer bei TO_ANSISPLIT
//			  iFlags muss an ColorChose weiter gegeben werden !
//	Ergibt die letzte Koordinate in Zeichen der ausgegeben Zeile.
void CAnsiView::PrintText(HDC hDc,RECT *pRect,char *pText,char *pColors,int iSize,int iFlags)
{
int		iColorNew;
int		iOutColor;
int		iOutPos;
int		iOutCnt;
int		iPosX;
int		iPosY;
int		iRet;
int		iOff;



	if(iSize<=0)return;

	iPosX	    = pRect->left;
	iPosY	    = pRect->top;
	iOutPos		= 0;
	iOutColor	=(pColors)?	pColors[0]:m_iColorNow;
	iColorNew	= iOutColor;



	for(iOff=0;iOff<iSize;iOff++)							// Zeichenschleife
		{
		if(pColors)iColorNew=pColors[iOff];

		if(iColorNew!=iOutColor || iOff+1==iSize)			// Zeichen ausgeben
			{
			iOutCnt = iOff-iOutPos;

			ColorChose(hDc,iOutColor,iFlags);

			iRet	= TabbedTextOutA(hDc,iPosX,iPosY,pText,iOutCnt,0,0,0);
			iPosX  += iRet&0xFFFF;
			pText  += iOutCnt;
			iOutPos = iOff;
			}
		}


}

//*****************************************************************************
//*
//*		PrintMargin   Rand auf den Schirm zeichnen;
//*
//*****************************************************************************
//	Zeichnet den linken Rand einer Zeile.
//	hDdc		: Handle vom Gertekontext.
//	pRect		: Pixelkoordinaten des Bereiches.
//	iLine		: Zeilennummer des linken Randes.
void CAnsiView::PrintMargin(HDC hDc,RECT *pRect,int iLine)
{
COLORREF	uOld;


	uOld = SetBkColor(hDc,RGB(192,192,192));
	ExtTextOut(hDc,0,0,ETO_OPAQUE,pRect,NULL,0,NULL);
	SetBkColor(hDc,uOld);

}

//*****************************************************************************
//*
//*		DrawView
//*
//*****************************************************************************
//	Zeichnet das Fenster neu.
//	iMode	: Bestimmt die Art des Neuzeichnens.
//				0 = Es wird alles neugezeichnet.
//				1 = Es wird je nach dem Inhalt von m_iRedrawFlags gezeichnet.
void CAnsiView::DrawView(int iMode)
{

	if(!iMode)
		{
		m_iRedrawFlags = NZ_ALL;
		OnDraw(0);
		}
	else if(m_iRedrawFlags)
		{
		OnDraw(0);
		}

}

//*****************************************************************************
//*
//*		OnCreate  Fenstererzeugung
//*
//*****************************************************************************
//	Erzeugen eines Fensters.
int CAnsiView::OnCreate(LPCREATESTRUCT pCreateStruct)
{
CRect		sRect;
int			iBits;
int			iPos;


	sRect.SetRectEmpty();

	m_uScrollColor  = GetSysColor(COLOR_3DFACE);
	m_uScrollColor -= (m_uScrollColor>>3)&0x001F1F1F;


	for(iPos=0;iPos<ENTRIES(m_aScrollBars);iPos++)			// StrollBars erstellen
		{
		iBits = (iPos&1)? SBS_VERT|SBS_RIGHTALIGN:SBS_HORZ|SBS_BOTTOMALIGN;
		if(iPos<2)iBits|=WS_VISIBLE;
		m_aScrollBars  [iPos].Create(iBits|WS_CHILD,sRect,this,ANSI_SRCOLL_ID+iPos);
		m_aScrollBarsOn[iPos] = 0;
		}


	if(m_pGroup)											// Schrift einstellen
		{
		if(m_pGroup->hFont)FontSet(0);
		else			   FontSet( );
		}


	SetCursorMode();

	m_iRedrawFlags = NZ_ALL;



return CView::OnCreate(pCreateStruct);
}

//*****************************************************************************
//*
//*		GetFieldOfPoint
//*
//*****************************************************************************
//	Ergibt das aktuelle Feld von einem Punkt
//	sPoint		: Ist der Punkt des Mauszeigers
//	pBase		: Hier wird der Anfangspunkt des Feldes gespeichert
//	Ergibt die Nummer auf welchen Bereich der Punktliegt
//					   0 = Auf keinem Feld
//					   1 = Auf vertikaler Teilung
//					   2 = Auf horizontaler Teilung
//					   3 = Auf beiden Teilungen
//					4..7 = Feldnummer mit Bit 2
#ifdef	TO_ANSISPLIT
int CAnsiView::GetFieldOfPoint(CPoint sPoint,CPoint *pBase)
{
int		iFieldX;
int		iFieldY;
CPoint	sBase;



	if(!pBase)pBase=&sBase;


	if(m_cCapture)											// Ist der Capture auf ein Feld aktiv
		{
		if(!(m_cCapture&4))
			{
			pBase->x = 0;
			pBase->y = 0;
			return m_cCapture&7;
			}

		pBase->x = (m_cCapture&1)? m_aSplitPos[0]+SEPARATOR_SIZE:0;
		pBase->y = (m_cCapture&2)? m_aSplitPos[1]+SEPARATOR_SIZE:0;

		return m_cCapture;
		}


	if(sPoint.x<0 || sPoint.y<0)							// Links oder oben auerhalb des Bereiches
		{
		pBase->x = 0;
		pBase->y = 0;
		return     0;
		}



	if(m_aSplitOn[1])										// Teilung horizontal vorhanden
		{
		if(sPoint.y<m_aSplitPos[1])
			{
			pBase->y = 0;
			iFieldY	 = 4;
			}
		else if(sPoint.y<m_aSplitPos[1]+SEPARATOR_SIZE)
			{
			pBase->y = m_aSplitPos[1];
			iFieldY	 = 2;
			}
		else if(sPoint.y<m_iScreenSizeY)
			{
			pBase->y = m_aSplitPos[1]+SEPARATOR_SIZE;
			iFieldY	 = 6;
			}
		else if(sPoint.x<SEPARATOR_SIZE && !m_aSplitOn[0] && (m_iWindowMode&ANSI_SPLIT_Y))
			{
			pBase->x = 0;
			return     1;
			}
		else if(sPoint.y<m_iScreenFullY)
			{
			pBase->y = 0;
			iFieldY	 = 0;
			}
		else{
			pBase->x = 0;
			pBase->y = 0;
			return     0;
			}
		}
	else{													// Keine horizontale Teilung vorhanden
		pBase->y = 0;

		if(sPoint.y<SEPARATOR_SIZE)
			{
			if(sPoint.x>=m_iScreenSizeX && (m_iWindowMode&ANSI_SPLIT_Y))
				{
				pBase->x = 0;
				return     2;
				}

			iFieldY	 = 4;
			}
		else if(sPoint.y<m_iScreenSizeY)
			{
			iFieldY	 = 4;
			}
		else if(sPoint.y<m_iScreenFullY)
			{
			iFieldY	 = 0;
			}
		else{
			pBase->x = 0;
			return     0;
			}
		}


	if(m_aSplitOn[0])										// Teilung vertikal vorhanden
		{
		if(sPoint.x<m_aSplitPos[0])
			{
			pBase->x = 0;
			iFieldX	 = 4;
			}
		else if(sPoint.x<m_aSplitPos[0]+SEPARATOR_SIZE)
			{
			pBase->x = m_aSplitPos[0];
			iFieldX	 = 1;
			}
		else if(sPoint.x<m_iScreenSizeX)
			{
			pBase->x = m_aSplitPos[0]+SEPARATOR_SIZE;
			iFieldX	 = 5;
			}
		else if(sPoint.y<SEPARATOR_SIZE && !m_aSplitOn[1] && (m_iWindowMode&ANSI_SPLIT_X))
			{
			pBase->x = 0;
			return     2;
			}
		else if(sPoint.x<m_iScreenFullX)
			{
			pBase->x = 0;
			iFieldX	 = 0;
			}
		else{
			pBase->x = 0;
			pBase->y = 0;
			return     0;
			}
		}
	else{													// Keine vertikal Teilung vorhanden
		pBase->x = 0;

		if(sPoint.x<SEPARATOR_SIZE)
			{
			if(sPoint.y>=m_iScreenSizeY && (m_iWindowMode&ANSI_SPLIT_X))
				{
				pBase->y = 0;
				return     1;
				}

			iFieldX	 = 4;
			}
		else if(sPoint.x<m_iScreenSizeX)
			{
			iFieldX	 = 4;
			}
		else if(sPoint.x<m_iScreenFullX)
			{
			iFieldX	 = 0;
			}
		else{
			pBase->y = 0;
			return     0;
			}
		}

	if(iFieldX&iFieldY&4)									// Iss X und Y auf einem Feld
		{
		return iFieldX|iFieldY;
		}

	if(iFieldX&4)iFieldX=0;
	if(iFieldY&4)iFieldY=0;


return iFieldX|iFieldY;
}
#endif

//*****************************************************************************
//*
//*		SplitWindow
//*
//*****************************************************************************
//	Spalte die Fenster wenn ANSI_SPLIT definiert ist
//	iMode		: Was soll geteilt werden
//					Bit  ANSI_SPLIT_X	= vertikal teilen
//					Bit  ANSI_SPLIT_Y	= horizontal teilen
//					Bits ANSI_SPLIT_XY	= beide Richtungen teilen
//	iPos		: Bestimmt die Pixel bei denen geteilt werden soll
//				  Bei ANSI_SPLIT_XY wird die Y-Position im High-Wort angegeben.
//				  Mit ANSI_SPLIT_POS(n) wird in Prozent geteilt (n:0...256)
//	Ergibt ungleich 0 wenn erfolgreich.
int CAnsiView::SplitWindow(int iMode,int iPos)
{
int		iRet=0;


	#ifdef TO_ANSISPLIT

	int		iPixel;
	int		iOnBit;

	if((iMode&ANSI_SPLIT_X) && (m_iWindowMode&ANSI_SPLIT_X))
		{
		if((iPos&0xFFFF0000)==0x1230000)
			{
			iPixel   = (m_iScreenSizeX*(iPos&0xFFFF))/256;
			}
		else{
			iPixel   = (short)iPos;
			iPos   >>= 16;
			}

		if(iPixel<0 || iPixel>=m_iScreenSizeX)
			{
			iPixel   = 0;
			}

		iOnBit = (iPixel)? 1:0;

		if(iPixel!=m_aSplitNow[0] || m_aSplitOn[0]!=iOnBit)
			{
			if(!iOnBit)iPixel=0;
			m_aSplitNow[0] = iPixel;
			m_aSplitOn [0] = iOnBit;
			iRet		  |= 1;
			}
		}

	if((iMode&ANSI_SPLIT_Y) && (m_iWindowMode&ANSI_SPLIT_Y))
		{
		if((iPos&0xFFFF0000)==0x1230000)
			{
			iPixel   = (m_iScreenSizeY*(iPos&0xFFFF))/256;
			}
		else{
			iPixel   = (short)iPos;
			}

		if(iPixel<0 || iPixel>=m_iScreenSizeY)
			{
			iPixel   = 0;
			}

		iOnBit = (iPixel)? 1:0;

		if(iPixel!=m_aSplitNow[1] || m_aSplitOn[1]!=iOnBit)
			{
			if(!iOnBit)iPixel=0;
			m_aSplitNow[1] = iPixel;
			m_aSplitOn [1] = iOnBit;
			iRet		  |= 2;
			}
		}

	if(iRet)
		{
		m_iRedrawFlags = NZ_ALL;
		OnSize(0,m_iScreenFullX,m_iScreenFullY);
		OnSplitChanged(m_iWindowMode&ANSI_SPLIT_XY,m_aSplitNow[0],m_aSplitNow[1]);
		}

	#endif


return iRet;
}

//*****************************************************************************
//*
//*		NextField
//*
//*****************************************************************************
//	Zum nchsten Feld springen bei TO_ANSISPLIT
//	iMode		: Ist die Richtung (0=weiter TAB+CLTR, 1=zurck mit TAB+CLTR+SHIFT)
//	Ergib die Nummer des neuen Feldes
int CAnsiView::NextField(int iMode)
{
int		iPos=-1;


	#ifdef TO_ANSISPLIT

	iPos  = m_iSplitActiv&3;
	if(iMode)iPos|=4;

	if(!m_aSplitOn[0])
		{
		iPos = "\2\2\0\0\2\0\0\2"[iPos];
		}
	else if(m_aSplitOn[1]==0)
		{
		iPos = "\1\0\0\0\1\0\0\0"[iPos];
		}
	else{
		iPos = "\1\2\3\0\3\0\1\2"[iPos];
		}

	SetActiveField(iPos);
	iPos = m_iSplitActiv;

	#endif


return iPos;
}

//*****************************************************************************
//*
//*		SetActiveField
//*
//*****************************************************************************
//	Ein anderes Feld im Fenster whlen (bei TO_ANSISPLIT)
//	iField		: Ist die neue Feldnummer
//					0 = links oben
//					1 = rechts oben
//					2 = links unten
//					3 = rechts unten
//	Ergibt 1 wenn ein anderes Feld gewhlt wurde.
int CAnsiView::SetActiveField(int iField,BOOL bRedraw)
{

	#ifdef	TO_ANSISPLIT

	int		iValue;
	int		iIndex;

	iField &= 3;

	if(!(m_iWindowMode&ANSI_SPLIT_X) || !m_aSplitOn[0])		// Prfe die Parameter
		{
		iField         &= ~1;
		m_aSplitOn[0]	=  0;
		}

	if(!(m_iWindowMode&ANSI_SPLIT_Y) || !m_aSplitOn[1])
		{
		iField         &= ~2;
		m_aSplitOn[1]	=  0;
		}

	if(m_iSplitActiv==iField)
		{
		return 0;
		}

	if((m_iSplitActiv^iField)&1)							// Hat sich etwas in X-Richtung gendert
		{
		iIndex			= iField&1;

		iValue			= m_iScrollOldXB;
		m_iScrollOldXB	= m_iScrollOldX;
		m_iScrollOldX	= iValue;

		iValue			= m_iScreenOldXB;
		m_iScreenOldXB	= m_iScreenOldX;
		m_iScreenOldX	= iValue;

		m_iScrollAddX	= m_aScrollAddX[iIndex];
		m_iScrollX		= m_aScrollX   [iIndex];
		m_iScreenX		= m_aScreenX   [iIndex];
		}


	if((m_iSplitActiv^iField)&2)							// Hat sich etwas in Y-Richtung gendert
		{
		iIndex			= iField>>1;

		iValue			= m_iScrollOldYB;
		m_iScrollOldYB	= m_iScrollOldY;
		m_iScrollOldY	= iValue;

		iValue			= m_iScreenOldYB;
		m_iScreenOldYB	= m_iScreenOldY;
		m_iScreenOldY	= iValue;

		m_iScrollAddY	= m_aScrollAddY[iIndex];
		m_iScrollY		= m_aScrollY   [iIndex];
		m_iScreenY		= m_aScreenY   [iIndex];
		}


	m_iSplitActiv   = iField;
	m_iRedrawFlags |= NZ_CURSOR;
	m_iRedrawFlags |= NZ_LOCK;

	if(m_cCaret)
		{
		HideCaret();
		m_cCaret = 0;
		}

	PositionCheck();

	m_iRedrawFlags &= ~NZ_LOCK;


	if(bRedraw)												// Das Fenster neu zeichenen
		{
		DrawView(1);
		}

	return 1;

	#else

	return -1;

	#endif

}


//*****************************************************************************
//*
//*		OnSize    Fenstergre wurde gendert
//*
//*****************************************************************************
//	Grennderung des Fensters
//	nType		: Ist der Typ der Grennderung (z.B. SIZE_MAXIMIZED)
//	iSizeX/Y	: Ist die neue Fenstergre in Pixel.	
void CAnsiView::OnSize(UINT nType,int iSizeX,int iSizeY)
{
int		iRedraw;
int		iValueX;
int		iValueY;


	m_iOnSizeCnt++;

	iRedraw = m_iRedrawFlags&NZ_LOCK;

	m_iScreenFullX	= iSizeX;
	m_iScreenFullY	= iSizeY;

															// Abmessungen der Scrollbars
	iValueY	= (m_aScrollBarsOn[SB_HORZ])? GetSystemMetrics(SM_CYHSCROLL):0;
	iValueX	= (m_aScrollBarsOn[SB_VERT])? GetSystemMetrics(SM_CXVSCROLL):0;

	iSizeX -= iValueX;	if(iSizeX<0)iSizeX=0;
	iSizeY -= iValueY;	if(iSizeY<0)iSizeY=0;


	#ifdef	TO_ANSISPLIT

	int		iSplit;
	int		iChars;
	int		iPixel;


	m_iScreenSizeX =  iSizeX;
	m_iScreenSizeY =  iSizeY;


	if(m_aSplitOn[0])										// Teilung vertikal vorhanden
		{
		iSplit	= m_aSplitNow[0];

		if(iSplit>iSizeX-SEPARATOR_SIZE)iSplit=iSizeX-SEPARATOR_SIZE;
		if(iSplit<0)iSplit=0;

		if(m_aSplitPos[0]!=iSplit)
			{
			m_aSplitPos[0] = iSplit;
			m_iRedrawFlags = NZ_ALL;
			}

		   iChars = (iSplit-m_iMarginWidth+m_iFontSizeX-1)/m_iFontSizeX;
		if(iChars<=0)iChars=1;


		if(m_aScreenX[0]!=iChars)
			{
			m_aScreenX[0]  = iChars;
			m_iRedrawFlags = NZ_ALL;
			}

		iPixel = iSizeX-iSplit-SEPARATOR_SIZE;

		   iChars = (iPixel-m_iMarginWidth+m_iFontSizeX-1)/m_iFontSizeX;
		if(iChars<=0)iChars=1;

		if(m_aScreenX[1]!=iChars)
			{
			m_aScreenX[1]  = iChars;
			m_iRedrawFlags = NZ_ALL;
			}

		m_aSplitSizeX[0] = iSplit;
		m_aSplitSizeX[1] = iPixel;

		m_aScrollAddX[0] = (m_aScreenX[0]*m_iFontSizeX)>(iSplit-m_iMarginWidth+2);
		m_aScrollAddX[1] = (m_aScreenX[1]*m_iFontSizeX)>(iPixel-m_iMarginWidth+2);

															// Zwei Scroll-Bars horizontal plazieren
		m_aScrollBars[SB_HORZ  ].MoveWindow(0                    ,iSizeY,iSplit,iValueY,m_iOnSizeCnt<=1);
		m_aScrollBars[SB_HORZ+2].MoveWindow(iSplit+SEPARATOR_SIZE,iSizeY,iPixel,iValueY,m_iOnSizeCnt<=1);

		if(m_iOnSizeCnt<=1)m_aScrollBars[SB_HORZ+2].ShowWindow(SW_SHOW);
		}
	else{													// Keine vertikale Teilung vorhanden
		   iSplit = iSizeX;
		if(iSplit<0)iSplit=0;

		   iChars = (iSplit-m_iMarginWidth+m_iFontSizeX-1)/m_iFontSizeX;
		if(iChars<=0)iChars=1;

		if(m_aScreenX[0]!=iChars)
			{
			m_aScreenX[0]  = iChars;
			m_iRedrawFlags = NZ_ALL;
			}

		if(m_aScreenX[1]!=1)
			{
			m_aScreenX[1]  = 1;
			m_iRedrawFlags = NZ_ALL;
			}

		m_aSplitSizeX[0] = iSizeX;
		m_aSplitSizeX[1] = 0;
		m_aSplitPos  [0] = 0;
		m_aSplitNow  [0] = 0;

		m_aScrollAddX[0] = (m_aScreenX[0]*m_iFontSizeX)>(iSplit-m_iMarginWidth+2);
		m_aScrollAddX[1] =  m_aScrollAddX[0];

															// Eine Scroll-Bars horizontal plazieren
		iPixel = (m_iWindowMode&ANSI_SPLIT_X)? SEPARATOR_SIZE:0;
		m_aScrollBars[SB_HORZ].MoveWindow(iPixel,iSizeY,iSizeX-iPixel,iValueY,m_iOnSizeCnt<=1);

		if(m_iOnSizeCnt<=1)m_aScrollBars[SB_HORZ+2].ShowWindow(SW_HIDE);

		if(m_iSplitActiv&1)
			{
			SetActiveField(m_iSplitActiv&2,FALSE);			// Wurde das aktive Feld ausgebendet
			m_iRedrawFlags = NZ_ALL;
			}
		}


	if(m_aSplitOn[1])										// Teilung horizontal vorhanden
		{
		iSplit	= m_aSplitNow[1];

		if(iSplit>iSizeY-SEPARATOR_SIZE)iSplit=iSizeY-SEPARATOR_SIZE;
		if(iSplit<0)iSplit=0;

		if(m_aSplitPos[1]!=iSplit)
			{
			m_aSplitPos[1] = iSplit;
			m_iRedrawFlags = NZ_ALL;
			}

		m_iScreenMaxY = iSplit/m_iFontSizeY;

		   iChars = (iSplit+m_iFontSizeY-1)/m_iFontSizeY;
		if(iChars<=0)iChars=1;

		if(m_aScreenY[0]!=iChars)
			{
			m_aScreenY[0]  = iChars;
			m_iRedrawFlags = NZ_ALL;
			}

		iPixel	= iSizeY-iSplit-SEPARATOR_SIZE;

		   iChars = (iPixel+m_iFontSizeY-1)/m_iFontSizeY;
		if(iChars<=0)iChars=1;

		if(m_aScreenY[1]!=iChars)
			{
			m_aScreenY[1]  = iChars;
			m_iRedrawFlags = NZ_ALL;
			}

		m_aSplitSizeY[0] = iSplit;
		m_aSplitSizeY[1] = iPixel;

		m_aScrollAddY[0] = (m_aScreenY[0]*m_iFontSizeY)>(iSplit+2);
		m_aScrollAddY[1] = (m_aScreenY[1]*m_iFontSizeY)>(iPixel+2);

															// Zwei Scroll-Bars vertikal plazieren
		m_aScrollBars[SB_VERT  ].MoveWindow(iSizeX,0                    ,iValueX,iSplit,m_iOnSizeCnt<=1);
		m_aScrollBars[SB_VERT+2].MoveWindow(iSizeX,iSplit+SEPARATOR_SIZE,iValueX,iPixel,m_iOnSizeCnt<=1);

		if(m_iOnSizeCnt<=1)m_aScrollBars[SB_VERT+2].ShowWindow(SW_SHOW);
		}
	else{													// Keine horizontale Teilung vorhanden
		   iSplit = iSizeY;
		if(iSplit<0)iSplit=0;

		   iChars = (iSplit+m_iFontSizeY-1)/m_iFontSizeY;
		if(iChars<=0)iChars=1;

		if(m_aScreenY[0]!=iChars)
			{
			m_aScreenY[0]  = iChars;
			m_iRedrawFlags = NZ_ALL;
			}

		if(m_aScreenY[1]!=1)
			{
			m_aScreenY[1]  = 1;
			m_iRedrawFlags = NZ_ALL;
			}

		m_iScreenMaxY    = iChars;
		m_aSplitSizeY[0] = iSizeY;
		m_aSplitSizeY[1] = 0;
		m_aSplitPos  [1] = 0;
		m_aSplitNow  [1] = 0;

		m_aScrollAddY[0] = (m_aScreenY[0]*m_iFontSizeY)>(iSplit+2);
		m_aScrollAddY[1] =  m_aScrollAddY[0];

															// Eine Scroll-Bars vertikal plazieren
		iPixel = (m_iWindowMode&ANSI_SPLIT_Y)? SEPARATOR_SIZE:0;
		m_aScrollBars[SB_VERT].MoveWindow(iSizeX,iPixel,iValueX,iSizeY-iPixel,m_iOnSizeCnt<=1);

		if(m_iOnSizeCnt<=1)m_aScrollBars[SB_VERT+2].ShowWindow(SW_HIDE);

		if(m_iSplitActiv&2)
			{
			SetActiveField(m_iSplitActiv&1,FALSE);			// Wurde das aktive Feld ausgebendet
			m_iRedrawFlags = NZ_ALL;
			}
		}


	m_iScreenX		= m_aScreenX   [m_iSplitActiv &1];
	m_iScreenY		= m_aScreenY   [m_iSplitActiv>>1];
	m_iScrollAddX	= m_aScrollAddX[m_iSplitActiv &1];
	m_iScrollAddY	= m_aScrollAddY[m_iSplitActiv>>1];

	#else

	m_aScrollBars[SB_HORZ].MoveWindow(0,iSizeY,iSizeX,iValueY,m_iOnSizeCnt<=1);
	m_aScrollBars[SB_VERT].MoveWindow(iSizeX,0,iValueX,iSizeY,m_iOnSizeCnt<=1);



	if(m_iScreenSizeX!=iSizeX)								// X-Koordinatennderung
		{
		if(m_iScreenSizeX<iSizeX){m_iRedrawFlags=NZ_ALL;}
		   m_iScreenX     = (iSizeX-m_iMarginWidth+m_iFontSizeX-1)/m_iFontSizeX;
		if(m_iScreenX<=0)m_iScreenX=1;
		   m_iScreenSizeX =  iSizeX;
		   m_iScrollAddX  = (m_iScreenX*m_iFontSizeX)>(iSizeX+2-m_iMarginWidth);
		}

	if(m_iScreenSizeY!=iSizeY)								// Y-Koordinatennderung
		{
		   m_iScreenY=(iSizeY+m_iFontSizeY-1)/m_iFontSizeY;
		if(m_iScreenY<=0)m_iScreenY=1;
		   m_iScreenSizeY =  iSizeY;
		   m_iScrollAddY  = (m_iScreenY*m_iFontSizeY)>(m_iScreenSizeY+2);

		if(m_iScreenSizeY<iSizeY)
			{
			DrawViewFrom(m_iScrollY+m_iScreenY-1);
			}
		}

	#endif

															// Bei rekursiven Aufruf nur ganz unten Zeichen
	if(m_iOnSizeCnt<=1 && m_iScreenFullX>0 && m_iScreenFullY>0)										
		{
		m_iRedrawFlags |=  NZ_LOCK;

		PositionCheck();

		m_iRedrawFlags &= ~NZ_LOCK;
		m_iRedrawFlags |=  iRedraw;

		if(!iRedraw && m_iRedrawFlags)
			{
			DrawView(1);
			}
		}

	m_iOnSizeCnt--;


}

//*****************************************************************************
//*
//*		OnPaint
//*
//*****************************************************************************
//	Neuzeichnen des Fensters (fr CWnd und nicht fr CView)
#ifdef 	TO_ANSIWND
void CAnsiWnd::OnPaint()
{
CPaintDC	sDc(this);

	OnDraw(&sDc);

}
#endif

//*****************************************************************************
//*
//*		OnDraw
//*
//*****************************************************************************
//	Neuzeichnen des Fensters
//	pDC		: Zeiger auf den Gertekontext.
//			  Wenn pDC == 0 ist dann kommt der Aufruf von Zeichnen und das
//			  Fenster wird je nach 'm_iRedrawFlags' gezeichnet
//			  Wenn pDC != 0 ist dann wird alles neu gezeichnet
void CAnsiView::OnDraw(CDC *pDC)
{
HDC			hDc;
HRGN		hRgnA;
HRGN		hRgnB;
RECT		sArea;
RECT		sRect;
RECT		sFrame;
POINT		sPoint;
int			iOrigin;
int			iReLine;
int			iRedraw;
int			iScreenX;
int			iScreenY;
int			iScrollX;
int			iScrollY;
int			iCurPos;
int			iCursor;
int			iValue;
int			iFlags;
int			iPosX;
int			iPosY;
int			iPos;
int			iEnd;
int			iDif;



	if(!pDC)
		{
		if(!m_hWnd)return;
		hDc=::GetDC(m_hWnd);								// Alles neu zeichnen ?
		}
	else{
		hDc=pDC->m_hDC;
		m_iRedrawFlags=NZ_ALL;
		}


//*********************** Grundeinstellung ************************************


	m_iFontNow	 = -1;
	m_iColorNow	 = -1;

	ColorChose(hDc,0);
	SelectObject(hDc,m_pGroup->hFont);

	#ifdef	TO_ANSISPLIT

	int		iReBits;
	int		iMemBit;
	int		iMemPos;
	int		iMemScX;
	int		iMemScY;
	int		iFrameX;
	int		iFrameY;
	int		iField;


	iFrameX		=  0;
	iFrameY		=  0;
	iMemScX		= -1;
	iMemScY		= -1;
	iMemPos		= -1;
	iField		= -1;
	iReLine		= m_iRedrawFlags&NZ_LINE_XX;
	iReBits		= m_iRedrawFlags;							// Fertig fr neue Eingaben
	iMemBit 	= m_iRedrawFlags;
		    	  m_iRedrawFlags=0;

	hRgnA		= CreateRectRgn(0,0,0,0);
	hRgnB		= CreateRectRgn(0,0,0,0);


	if(iReBits&NZ_ALL)										// Teiler zeihnen
		{
		if(m_aSplitOn[0])									// Teilung vertikal vorhanden
			{
			DrawSeparator(hDc,m_aSplitPos[0],m_iScreenFullY,FALSE);

			if(m_aSplitOn[1])								// Teilung horizontal vorhanden
				{
				DrawSeparator(                                hDc,m_aSplitPos[1],               m_aSplitPos[0]+1               ,TRUE);
				DrawSeparator(m_aSplitPos[0]+SEPARATOR_SIZE-1,hDc,m_aSplitPos[1],m_iScreenFullX-m_aSplitPos[0]+1-SEPARATOR_SIZE,TRUE);
				SelectObject (hDc,GetStockObject  (NULL_PEN    ));
				SelectObject (hDc,GetSysColorBrush(COLOR_3DFACE));
				Rectangle    (hDc,m_aSplitPos[0]+1,m_aSplitPos[1]+2,m_aSplitPos[0]+SEPARATOR_SIZE,m_aSplitPos[1]+SEPARATOR_SIZE-1);
				}
			else if(m_iWindowMode&ANSI_SPLIT_Y)
				{
				DrawSeparator(m_iScreenSizeX,hDc,0,m_iScreenFullX-m_iScreenSizeX,TRUE);
				}
			}
		else{
			if(m_aSplitOn[1])								// Teilung horizontal vorhanden
				{
				DrawSeparator(hDc,m_aSplitPos[1],m_iScreenFullX,TRUE);
				}
			else if(m_iWindowMode&ANSI_SPLIT_Y)
				{
				DrawSeparator(m_iScreenSizeX,hDc,0,m_iScreenFullX-m_iScreenSizeX,TRUE);
				}

			if(m_iWindowMode&ANSI_SPLIT_X)
				{
				DrawSeparator(m_iScreenSizeY,hDc,0,m_iScreenFullY-m_iScreenSizeY,FALSE);
				}
			}
		}


Next:

	iField++;

	if(iField>=4)											// Alle Felder abgearbeitet ?
		{
		iScreenX	= iMemScX;
		iScreenY	= iMemScY;
		iCurPos		= iMemPos;
		iRedraw		= iMemBit;
		goto Done;
		}

	if(m_aSplitOn[0])										// Teilung vertikal vorhanden
		{
		sRect.left	= (iField&1)? m_aSplitPos[0]+SEPARATOR_SIZE:0;
		sRect.right	= (iField&1)? m_iScreenSizeX:m_aSplitPos[0];

		iScrollX	=  m_aScrollX[iField&1];
		iScreenX	=  m_aScreenX[iField&1];
		}
	else{
		if(iField&1)goto Next;

		sRect.left	= 0;
		sRect.right	= m_iScreenSizeX;

		iScrollX	= m_iScrollX;
		iScreenX	= m_iScreenX;
		}


	if(m_aSplitOn[1])										// Teilung horizontal vorhanden
		{
		sRect.top	= (iField&2)? m_aSplitPos[1]+SEPARATOR_SIZE:0;
		sRect.bottom= (iField&2)? m_iScreenSizeY:m_aSplitPos[1];

		iScrollY	=  m_aScrollY[iField>>1];
		iScreenY	=  m_aScreenY[iField>>1];
		}
	else{
		if(iField&2)goto Next;

		sRect.top	= 0;
		sRect.bottom= m_iScreenSizeY;

		iScrollY	= m_iScrollY;
		iScreenY	= m_iScreenY;
		}

	iValue			= sRect.left+m_iMarginWidth;
															// Zeichenbereiche fr Text und linken Rand
	SetRectRgn(hRgnA,iValue    ,sRect.top,sRect.right,sRect.bottom);
	SetRectRgn(hRgnB,sRect.left,sRect.top,iValue     ,sRect.bottom);


	if(iField!=m_iSplitActiv)								// Inaktive Felder markieren
		{
		iFlags		=(iField<<8)|2;
		}
	else{
		iFlags		=(iField<<8);
		iFrameX		= sRect.left;
		iFrameY		= sRect.top;
		}

	sArea.left		= sRect.left;
	sArea.right		= iValue;
	sRect.left      = iValue;
	iOrigin			= iValue;								// Startposition fr Tabstops berechnen
	iOrigin		   -= m_iFontSizeX*iScrollX;


	if(m_iOriginPtX && !((m_iSplitActiv^iField)&1))			// Sind ein paar Pixel verschoben
		{
		iOrigin		 -=	m_iOriginPtX;
		sRect.left   -= m_iOriginPtX;
		sRect.right  -= m_iOriginPtX;
		sRect.right  += m_iFontSizeX;
		iScreenX     += 1;
		}

	if(m_iOriginPtY && !((m_iSplitActiv^iField)&2))
		{
		sRect.top    -= m_iOriginPtY;
		iScreenY     += 1;
		}

	m_iFieldDraw =  iField;
	iRedraw		 =  iReBits;
	iFlags      |=  iOrigin<<12;

	sRect.bottom = sRect.top+m_iFontSizeY;
	sFrame.top	 = sRect.top;
	sFrame.left	 = sRect.left;

	SelectObject(hDc,hRgnA);

	#else

	sRect.top	 = 0;										// Koordinaten der ersten Zeile
	sRect.left   = m_iMarginWidth;
	sRect.right  = m_iScreenSizeX;

	iScrollX	 = m_iScrollX;
	iScrollY	 = m_iScrollY;
	iScreenX	 = m_iScreenX;
	iScreenY	 = m_iScreenY;

	iOrigin      = m_iMarginWidth;							// Startposition fr Tabstops berechnen
	iOrigin		-= m_iFontSizeX*m_iScrollX;

	if(m_iOriginPtX)										// Sind ein paar Pixel verschoben
		{
		iOrigin		 -=	m_iOriginPtX;
		sRect.left   -= m_iOriginPtX;
		sRect.right  -= m_iOriginPtX;
		sRect.right  += m_iFontSizeX;
		iScreenX     += 1;
		}

	if(m_iOriginPtY)
		{
		sRect.top    -= m_iOriginPtY;
		iScreenY     += 1;
		}

	sRect.bottom = sRect.top+m_iFontSizeY;
															// Zeichenbereiche fr Text und linken Rand
	hRgnA = CreateRectRgn(m_iMarginWidth  ,0,m_iScreenSizeX,m_iScreenSizeY);
	hRgnB = CreateRectRgn(0,0,m_iMarginWidth,m_iScreenSizeY);
	SelectObject(hDc,hRgnA);

	sArea.left	 = 0;
	sArea.right  = m_iMarginWidth;
	sFrame.top	 = sRect.top;
	sFrame.left	 = sRect.left;
	iFlags		 =(iOrigin<<12)|2;
	iReLine		 = m_iRedrawFlags&NZ_LINE_XX;
	iRedraw		 = m_iRedrawFlags;							// Fertig fr neue Eingaben
				   m_iRedrawFlags=0;


	#endif


//*********************** Zeichne vernderte Zeilen ***************************


	if((iRedraw&NZ_LINE_CURSOR) && m_iCursorX-2<=iScrollX)// Ist Cursor vor dem X-Scroll-Start
		{
		iRedraw &= ~NZ_LINE_CURSOR;
		iRedraw |=  NZ_LINE;
		}

	if(iRedraw&NZ_ALL)										// Alles neu zeichnen
		{
		iPos	 = 0;
		iEnd	 = iScreenY;
		iRedraw &=~NZ_LINE_CURSOR;
		iRedraw |= NZ_ALL;
		}
	else if(iRedraw&NZ_LINE_FROM)							// An einer Zeile neu zeichnen
		{
		iPos	 = iReLine-iScrollY;
		iEnd	 = iScreenY;
		if(iPos<0)iPos=0;
		}
	else if(iRedraw&NZ_LINE_AREA)							// Einen Bereich neu zeichnen
		{
		iDif	 = (iReLine>>NZ_LINE_SHIFT)&NZ_LINE_COUNT;
		iPos	 = (iReLine&NZ_LINE_WW)-iScrollY;
		iEnd	 =  iPos+iDif;
		if(iPos<0       )iPos=0;
		if(iEnd>iScreenY)iEnd=iScreenY;
		}
	else if(iRedraw&NZ_LINE_NR)								// Eine Zeile neu zeichnen
		{
		iPos	 = iReLine-iScrollY;
		iEnd	 = iPos;

		if(iPos<0 || iPos>iScreenY)
			{
			iPos = 0;
			iEnd =-1;
			}
		}
	else{
		iPos	 = 0;
		iEnd	 =-1;
		}

	if(iRedraw&NZ_LINE_NEXT)								// Die Zeile nach dem Cursor neu zeichnen
		{
		#ifdef  USE_VIRTUAL_LINES
		iDif	= ANSI_STEP(m_iCursorY,1);
		#else
		iDif	= m_iCursorY-iScrollY+1;
		#endif

		if(iDif<=iScreenY)									// Ist der Bereich nicht mehr sichtbar
			{
			if(iDif<0       )iDif=0;
			if(iPos>iDif    )iPos=iDif;
			if(iEnd<iScreenY)iEnd=iScreenY;
			}
		}


	if(iRedraw&NZ_LINE)										// Die Zeile mit dem Cursor neu neichnen
		{
		#ifdef  USE_VIRTUAL_LINES
		iCursor	= FindVirtualLine(m_iCursorY);
		iCurPos = iCursor;
		#else
		iCursor	= m_iCursorY-iScrollY;
		iCurPos = iCursor;
		#endif
		}
	else{
		iCursor	= -1;
		iCurPos = -2;
		}


	#ifdef  USE_VIRTUAL_LINES
	iPosY	= ANSI_STEP(iScrollY,iPos);
	#else
	iPosY	= iScrollY+iPos;
	#endif

	if(iPos)												// Es ist nicht die erste Zeile ganz oben
		{
		iDif		  = iPos*m_iFontSizeY;
		sRect.top    += iDif;
		sRect.bottom += iDif;
		}


	for(;;)													// Die Zeilen zeichnen
		{
		if(iPos>iEnd)
			{
			if(iCursor<0)break;								// Muss die Zeile mit dem Cursor gezeichnet werden
			iDif		  = (iCursor-iPos)*m_iFontSizeY;
			sRect.top    += iDif;
			sRect.bottom += iDif;
			iPosY		  = ANSI_STEP(m_iCursorY,0);
			iPos		  = iCursor;
			iEnd		  = -1;
			iCursor		  = -1;
			}

		if(iPosY==m_iCursorY)								// Sind wir in der Zeile mit dem Cursor
			{
			iRedraw &= ~NZ_LINE_CURSOR;
			iCurPos  = iPos;
			iCursor  = -1;
			}
															// Cursor-Marke ausblenden
		if(iPos==m_iCaretY && m_cCaret ANSI_SPL(&& iField==m_iSplitActiv))
			{
			HideCaret();
			m_cCaret = 0;
			}

		iPosX = PrintLine(hDc,&sRect,iPosY,iScrollX,iScreenX,iFlags);

		if(iPosX<iScreenX)									// Restliche Zeile rechts ausmalen
			{
			iValue	    = sRect.left;
			sRect.left += iPosX*m_iFontSizeX;
			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sRect,NULL,0,NULL);
			sRect.left	= iValue;
			}

		if(m_iMarginWidth)									// Den linken Rahmen zeichnen
			{
			SelectObject(hDc,hRgnB);

			sArea.top	 = sRect.top;
			sArea.bottom = sRect.bottom;

			PrintMargin(hDc,&sArea,iPosY);

			SelectObject(hDc,hRgnA);
			}

		sRect.bottom += m_iFontSizeY;						// Zur nchsten Zeile springen
		sRect.top    += m_iFontSizeY;
		iPosY         = ANSI_STEP(iPosY,1);
		iPos		 += 1;
		}


//**************** Text nach Cursor zeichnen **********************************


	if(iRedraw&NZ_LINE_CURSOR)								// Cursor Zeile zeichnen
	if(m_iCursorY>=iScrollY         )
	if(m_iCursorY< iScrollY+iScreenY)
		{
		   iDif=iScreenX+iScrollX-m_iCursorX+2;
		if(iDif>0)
			{
			if(m_cCaret ANSI_SPL(&& iField==m_iSplitActiv))	// Cursor-Marke ausblenden
				{
				HideCaret();
				m_cCaret = 0;
				}

			if(iCurPos==-2)
				{
				#ifdef    USE_VIRTUAL_LINES
 				iCurPos = FindVirtualLine(m_iCursorY);
				#else
				iCurPos = m_iCursorY-iScrollY;
				#endif
				}

			sFrame.top	   += m_iFontSizeY*iCurPos;
			sFrame.bottom	= m_iFontSizeY+sFrame.top;
			sFrame.left	   += m_iFontSizeX*(m_iCursorX-iScrollX-2);
			sFrame.right	= m_iFontSizeX*iDif+sFrame.left;

			iPosX = PrintLine(hDc,&sFrame,m_iCursorY,m_iCursorX-2,iDif,iFlags);

			if(iPosX<iDif)									// Den Rest rechts der Zeile ausmalen
				{
				sFrame.left += iPosX*m_iFontSizeX;
				ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sFrame,NULL,0,NULL);
				}
			}
		}

	#ifdef	TO_ANSISPLIT

	if(iField==m_iSplitActiv)								// Werte vom aktuellen Feld speichern
		{
		iMemScX		= iScreenX;
		iMemScY		= iScreenY;
		iMemPos		= iCurPos;
		iMemBit		= iRedraw;
		}

	goto Next;

Done:

	#endif

//**************** Cursor Position ********************************************


	if(m_cCaret && (iRedraw&NZ_CURSOR))						// Cursor-Marke ausblenden
		{
		HideCaret();
		m_cCaret = 0;
		}


	if(!m_cCaret && (m_iCursorMode&2))						// Cursor-Marke neu zeichnen
		{
		if(m_iCursorX>=m_iScrollX && m_iCursorX<=m_iScrollX+iScreenX && m_cFocusOn)
			{
			if(iCurPos==-2)
				{
				#ifdef		  USE_VIRTUAL_LINES
 				iCurPos		= FindVirtualLine(m_iCursorY);
				#else
				iCurPos		= m_iCursorY-m_iScrollY;
				#endif
				}

			#ifdef 		  TO_ANSISPLIT
			if(iCurPos<m_iScreenMaxY || (m_iSplitActiv&2))	// Ist der Cursor auf der Halbzeile
			#endif 	

															// Ist der Cursor sichtbar
			if(iCurPos>=0 && iCurPos<=iScreenY ANSI_SPL(&& m_cClickMode!=3))
				{
				m_iCaretX	= m_iCursorX-m_iScrollX;
				m_iCaretY	= iCurPos;
				sPoint.x	= m_iFontSizeX*m_iCaretX-m_iOriginPtX;
				sPoint.y	= m_iFontSizeY*m_iCaretY-m_iOriginPtY;

				#ifdef 		  TO_ANSISPLIT
				sPoint.x   += iFrameX;
				sPoint.y   += iFrameY;
				#endif

				sPoint.x   += m_iMarginWidth;
				SetCaretPos(sPoint);
				ShowCaret();
				m_cCaret=1;
				}
			}
		}


//**************** Ecke links unten bei Scrollbar *****************************


	if((iRedraw&NZ_ALL) && m_aScrollBarsOn[SB_HORZ] && m_aScrollBarsOn[SB_VERT])
		{
		SetRectRgn(hRgnB,m_iScreenSizeX,m_iScreenSizeY,m_iScreenFullX,m_iScreenFullY);
		SelectObject(hDc,hRgnB);

		sRect.top	 = m_iScreenSizeY;
		sRect.left	 = m_iScreenSizeX;
		sRect.right	 = m_iScreenFullX;
		sRect.bottom = m_iScreenFullY;

		SetBkColor (hDc,m_uScrollColor);
		ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sRect,NULL,0,NULL);
		}


	DeleteObject(hRgnA);
	DeleteObject(hRgnB);


	if(!pDC)::ReleaseDC(m_hWnd,hDc);						// Gertekontext freigeben

}

//*****************************************************************************
//*
//*		PositionOk
//*
//*****************************************************************************
//	Den Cursor verschieben.
//	iMode	: Bestimmt die Art der Positionierung
//					Bit 0 = 0 : Absolute Koordinaten
//					Bit 0 = 1 : Relative Koordinaten
//					Bit 1 = 1 : Ziehen mit Shift oder Maus
//					Bit 2 = 1 : CLTR ist gedrckt
//					Bit 3 = 1 : Eingabe ber Maus
//					Bit 4 = 1 : Maus Taste zum ersten mal gedrckt
//					Bit 7 = 1 : Links/Rechts/Auf/Ab Taste wurde gedrckt
// Ergibt 1 wenn Koordinaten verntert wurden ansondten 0.
int CAnsiView::PositionOk(int iPosX,int iPosY,int iMode)
{

	if(iMode&1)
		{												// Relative Position
		m_iCursorX += iPosX;
		m_iCursorY  = ANSI_STEP(m_iCursorX,iPosY);
		}
	else{
		m_iCursorX  = iPosX;							// Absolute Position
		m_iCursorY  = ANSI_STEP(iPosY,0);
		}


return TRUE;
}

//*****************************************************************************
//*
//*		OnSetFocus
//*
//*****************************************************************************
//	Tastatur-EingabeFocus erhalten
void CAnsiView::OnSetFocus(CWnd *pWnd)
{

	m_cFocusOn	= 1;
	m_iKeyState	= 0;

	if(GetAsyncKeyState(VK_SHIFT)&0x8000)				// Shift-Taste gedrckt
			m_iKeyState|= KEY_SHIFT;
	else	m_iKeyState&=~KEY_SHIFT;

	if(GetAsyncKeyState(VK_CONTROL)&0x8000)				// Cltr -Taste gedrckt
			m_iKeyState|= KEY_CLTR;
	else	m_iKeyState&=~KEY_CLTR;


	if(m_cCaretFocus)									// Cursor einblenden
		{
		if(m_iCursorMode&2)
			{
			ShowCaret();
			m_cCaret      = 1;
			m_cCaretFocus = 0;
			}
		}

	SetCursorMode();
	SendNotify(ANSI_SETFOCUS);

}

//*****************************************************************************
//*
//*		OnKillFocus
//*
//*****************************************************************************
//	Tastatur-Eingabe-Focus verloren
void CAnsiView::OnKillFocus(CWnd *pWnd)
{

	m_cFocusOn = 0;

	if(m_cCaret)
		{
		HideCaret();
		m_cCaretFocus = 1;
		m_cCaret      = 0;
		}
	else{
		m_cCaretFocus = 0;
		}

	SendNotify(ANSI_KILLFOCUS);

}


//*****************************************************************************
//*
//*		OnGetDlgCode
//*
//*****************************************************************************
//	Abfrage welche Tasten braucht das Fenster.
UINT CAnsiView::OnGetDlgCode( )
{
return DLGC_WANTARROWS|DLGC_WANTCHARS;
}

//*****************************************************************************
//*
//*		OnSysColorChange
//*
//*****************************************************************************
//	Es hat sich eine Systemfarbe verndert.
void CAnsiView::OnSysColorChange()
{
AnsiColor  *pColor;
int			i;


	if(m_pGroup && m_pGroup->pColorList)
		{
		pColor = m_pGroup->pColorList;

		for(i=0;i<m_pGroup->iColorCount;i++,pColor++)
			{
			if(pColor->uColorText!=0xFFFFFFFF)
			if(pColor->uColorBk  !=0xFFFFFFFF)continue;

			ColorChange(&m_pGroup->pColorList,pColor->uColorText,pColor->uColorBk,pColor->pName,pColor->iIndex,pColor->iFontMode);
			}
		}

	m_uScrollColor = GetSysColor(COLOR_3DFACE);

	CView::OnSysColorChange();
}


//*****************************************************************************
//*
//*		OnKeyUp
//*
//*****************************************************************************
//	Taste losgelassen
void CAnsiView::OnKeyUp(UINT nChar,UINT nRepCnt,UINT nFlags)
{

	if(GetAsyncKeyState(VK_SHIFT)&0x8000)					// Shift-Taste gedrckt
			m_iKeyState|= KEY_SHIFT;
	else	m_iKeyState&=~KEY_SHIFT;

	if(GetAsyncKeyState(VK_CONTROL)&0x8000)					// Cltr -Taste gedrckt
			m_iKeyState|= KEY_CLTR;
	else	m_iKeyState&=~KEY_CLTR;


	while(nRepCnt>0)										// Tasten-Nachricht senden
		{
		OnKeyUp(nChar,nFlags );
		nRepCnt--;
		}

}

//*****************************************************************************
//*
//*		OnKeyDown
//*
//*****************************************************************************
//	Eine Taste wurde gedrckt
void CAnsiView::OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags)
{
int		iRedraw;
int		iStart;
int		iStop;
int		iMode;


	iMode	= 0;
	iStart	= 1;
	iRedraw	= m_iRedrawFlags;


	if(GetAsyncKeyState(VK_SHIFT)&0x8000)					// Shift-Taste gedrckt
			m_iKeyState|= KEY_SHIFT;
	else	m_iKeyState&=~KEY_SHIFT;

	if(GetAsyncKeyState(VK_CONTROL)&0x8000)					// Cltr -Taste gedrckt
			m_iKeyState|= KEY_CLTR;
	else	m_iKeyState&=~KEY_CLTR;


	if(nRepCnt>=30)nRepCnt=30;								// Wie oft berspringen
	if(nRepCnt>=4 )iStart=nRepCnt;


	while(nRepCnt>0)										// Tasten-Nachricht senden
		{
		nRepCnt--;

		iMode = OnKeyDown(nChar,nFlags);

		if(!iMode)											// Abbrechen wenn Taste verarbeitet
		do	{
			if(nChar==VK_CLEAR && m_iKeyState==0)			// '5' auf der ZehnerTastatur ist gleich Doppelklick
				{
				OnLButtonDblClk(0,CPoint(0,0),m_iCursorX,m_iCursorY);
				iMode=1;
				break;
				}


			if(nChar<0x21 || nChar>0x28)break;				// Nur Cursor Tasten weiter bearbeiten

			iMode = 1;


			if(m_iKeyState&KEY_CLTR)						// Spezialflle CLTR/HOME und CLTR/END
				{
				iMode=(m_iKeyState&KEY_SHIFT)? 2:0;
				if(nChar==VK_END ){PositionOk(0,m_iLineCount,iMode);break;}
				if(nChar==VK_HOME){PositionOk(0,0           ,iMode);break;}
				}


			if(m_iKeyState==KEY_CLTR || !(m_iCursorMode&2))	// Mit Cursor-Tasten scrollen
				{
				switch(nChar)
					{
					case VK_LEFT:	m_iScrollX-= 1;										OnScroll(nChar);break;// Links
					case VK_RIGHT:	m_iScrollX+= 1;										OnScroll(nChar);break;// Rechts
					case VK_UP:		m_iScrollY = ANSI_STEP(m_iScrollY,-1           );	OnScroll(nChar);break;// Auf
					case VK_DOWN:	m_iScrollY = ANSI_STEP(m_iScrollY, 1           );	OnScroll(nChar);break;// Ab
					case VK_PRIOR:	m_iScrollY = ANSI_STEP(m_iScrollY, 1-m_iScreenY);	OnScroll(nChar);break;// Seite auf
					case VK_NEXT:	m_iScrollY = ANSI_STEP(m_iScrollY, m_iScreenY-1);	OnScroll(nChar);break;// Seite ab
					}

				continue;
				}


										iMode =1;			// Funktion je nach Tasten Status
			if(m_iKeyState & KEY_SHIFT )iMode|=2;
			if(m_iKeyState & KEY_CLTR  )iMode|=4;


			switch(nChar)									// Normale Cursorbewegung
				{
			case VK_LEFT:	PositionOk(-1, 0				    ,iMode|0x80);break;	// Links
			case VK_RIGHT:	PositionOk( 1, 0				    ,iMode|0x80);break;	// Rechts
			case VK_UP:		PositionOk( 0,-1				    ,iMode|0x80);break;	// Auf
			case VK_DOWN:	PositionOk( 0, 1				    ,iMode|0x80);break;	// Ab
			case VK_END:	PositionOk(-m_iCursorX+m_iLineWidthMax, 0,iMode);break;	// Ende
			case VK_HOME:	PositionOk(-m_iCursorX, 0			     ,iMode);break;	// Anfang
			case VK_PRIOR:	m_iScrollY = ANSI_STEP(m_iScrollY, 1-m_iScreenY);		// Seite auf
							PositionOk( 0,1-m_iScreenY,iMode);break;
			case VK_NEXT:	m_iScrollY = ANSI_STEP(m_iScrollY, m_iScreenY-1);		// Seite ab
							PositionOk( 0, m_iScreenY-1,iMode);break;
				}

			}while(0);


		m_iRedrawFlags|= NZ_LOCK;
		PositionCheck();
		m_iRedrawFlags&=~NZ_LOCK;


		iStop=((nRepCnt%iStart)&&(m_iRedrawFlags&NZ_ALL));

		if(m_iRedrawFlags && !iStop)DrawView(1);
		}


	if(!iMode)CView::OnKeyDown(nChar,nRepCnt,nFlags);

}


//*****************************************************************************
//*
//*		OnChar    Tastatureingabe
//*
//*****************************************************************************
//	ASCII Zeichen empfangen
void CAnsiView::OnChar(UINT nChar,UINT nRepCnt,UINT nFlags)
{

	m_iRedrawFlags |= NZ_LOCK;

	while(nRepCnt>0)										// Tastennachricht senden
		{
		OnChar(nChar,nFlags);
		nRepCnt--;
		}

	PositionCheck();

	m_iRedrawFlags &= ~NZ_LOCK;

	if(m_iRedrawFlags)DrawView(1);


}

//*****************************************************************************
//*
//*		OnLButtonDown
//*
//*****************************************************************************
//	Linke Maustaste getdckt
void CAnsiView::OnLButtonDown(UINT nFlags,CPoint sPoint)
{
int		iRedraw;
int		iPosX;
int		iPosY;



	#ifdef TO_ANSISPLIT

	int		aOld[4];
	int		iField;
	int		iDelta;
	CPoint	sBase;
	CRect	sRect;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4))
		{
		if(!iField)return;

		sRect.top		= 0;
		sRect.left		= 0;
		sRect.right		= m_iScreenFullX;
		sRect.bottom	= m_iScreenFullY;
		sBase			= sPoint;

		ClientToScreen(&sRect);

		switch(iField)										// Die Gre der Felder verndern
			{
		case 1:
				m_aSplitNow[0]	= m_aSplitPos[0];
				sBase.x		   -= m_aSplitPos[0];
				iDelta			= Resize(this,1,sRect,m_aSplitPos[0],sBase,SEPARATOR_SIZE,SEPARATOR_SIZE-2)&0xFFFF;
				break;
		case 2:
				m_aSplitNow[1]	= m_aSplitPos[1];
				sBase.y		   -= m_aSplitPos[1];
				iDelta			= Resize(this,0,sRect,m_aSplitPos[1],sBase,SEPARATOR_SIZE,SEPARATOR_SIZE-2)<<16;
				break;

		case 3:
				m_aSplitNow[0]	= m_aSplitPos[0];
				m_aSplitNow[1]	= m_aSplitPos[1];
				sBase.x		   -= m_aSplitPos[0];
				sBase.y		   -= m_aSplitPos[1];
				iDelta			= Resize(this,2,sRect,MAKELONG(m_aSplitPos[0],m_aSplitPos[1]),sBase,SEPARATOR_SIZE,SEPARATOR_SIZE-2);
				break;
			}

		if(!iDelta)return;

		aOld[0]			 = m_aSplitOn [0];
		aOld[1]			 = m_aSplitOn [1];
		aOld[2]			 = m_aSplitNow[0];
		aOld[3]			 = m_aSplitNow[1];

		m_aSplitNow[0]	+= (short)(iDelta);					// Die Vernderungen addieren
		m_aSplitNow[1]	+= (short)(iDelta>>16);

		if(m_aSplitNow[0]<=2 || m_aSplitNow[0]>=m_iScreenSizeX-SEPARATOR_SIZE-2 || !(m_iWindowMode&ANSI_SPLIT_X))
			{
			m_aSplitOn [0] = 0;
			m_aSplitNow[0] = 0;
			}
		else{
			m_aSplitOn [0] = 1;
			}

		if(m_aSplitNow[1]<=2 || m_aSplitNow[1]>=m_iScreenSizeY-SEPARATOR_SIZE-2 || !(m_iWindowMode&ANSI_SPLIT_Y))
			{
			m_aSplitOn [1] = 0;
			m_aSplitNow[1] = 0;
			}
		else{
			m_aSplitOn [1] = 1;
			}

		if(aOld[0]==m_aSplitOn [0])							// Ist alles gleich geblieben
		if(aOld[1]==m_aSplitOn [1])
		if(aOld[2]==m_aSplitNow[0])
		if(aOld[3]==m_aSplitNow[1])
			{
			return;
			}

		m_iRedrawFlags = NZ_ALL;
		OnSize(0,m_iScreenFullX,m_iScreenFullY);			// Die Vernderungen bernehmen
		OnSplitChanged(m_iWindowMode&ANSI_SPLIT_XY,m_aSplitNow[0],m_aSplitNow[1]);

		return;
		}

	if((iField^m_iSplitActiv)&3)							// Anderse aktives Feld
		{
		SetActiveField(iField);
		}

	sPoint.x   -= sBase.x;
	sPoint.y   -= sBase.y;

	#else

	const int iField=4;

	#endif


	if(GetFocus()!=this)SetFocus();

	iRedraw			= NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;

	m_iClickPosX    = sPoint.x-m_iMarginWidth;
	m_iClickPosY    = sPoint.y;

	iPosX = (m_iClickPosX+m_iFontSizeX/2)/m_iFontSizeX+m_iScrollX;
	iPosY = ANSI_STEP(m_iScrollY,m_iClickPosY/m_iFontSizeY);


	do	{
		if(!(nFlags&ANSI_NO_SUBCALL))
			{
			if(sPoint.x<m_iMarginWidth && sPoint.x>=0)		// Druck auf Rand
				{
				if(OnMarginClick((nFlags&0xFF)|MK_DOWN|MK_LEFT,sPoint,iPosY))break;
				}
			else{
				if(OnLButtonDown(nFlags,sPoint,iPosX,iPosY))break;
				}
			}

		if(m_bMouseTimer)									// Timer ausschalten
			{
			KillTimer(ID_MOUSEMOVE);
			m_bMouseTimer = 0;
			}

		if(iPosX<m_iScrollX)								// Auf Rand gedrckt ?
			{
			m_cMouseOn=2;
			}
		else{
			if(!m_cMouseOn)PositionOk(iPosX,iPosY,0x18);

			if(nFlags&MK_CONTROL)							// Doppelklick wenn CLTR-gedrckt ist
				{
				if(!OnLButtonDblClk(nFlags,sPoint,iPosX,iPosY))
					{
					m_cMouseOn = 1;
					}
				}
			else{
				m_cMouseOn = 1;
				}
			}

		m_cCapture = iField;
		SetCapture();
		}while(0);


	PositionCheck();

	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;

	if(!iRedraw && m_iRedrawFlags)DrawView(1);

}

//*****************************************************************************
//*
//*		OnRButtonDown
//*
//*****************************************************************************
//	Linke Maustaste getdckt
void CAnsiView::OnRButtonDown(UINT nFlags,CPoint sPoint)
{
int		iRedraw;


	#ifdef TO_ANSISPLIT

	int		iField;
	CPoint	sBase;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4))
		{
		return;
		}

	m_iMouseField = iField;

	sPoint.x     -= sBase.x;
	sPoint.y     -= sBase.y;

	#endif


	iRedraw         = NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;
	m_iContextX		= (sPoint.x-m_iMarginWidth+m_iFontSizeX/2)/m_iFontSizeX+m_iScrollX;
	m_iContextY		= ANSI_STEP(m_iScrollY,sPoint.y/m_iFontSizeY);
	m_iLastClickX	= sPoint.x;
	m_iLastClickY	= sPoint.y;

	do	{
		if(!(nFlags&ANSI_NO_SUBCALL))
			{
			if(sPoint.x<m_iMarginWidth && sPoint.x>=0)		// Druck auf Rand
				{
				if(OnMarginClick((nFlags&0xFF)|MK_DOWN|MK_RIGHT,sPoint,m_iContextY))break;
				m_iContextX		= sPoint.x-m_iMarginWidth;
				m_cClickMode	= 2;
				}
			else{
				if(OnRButtonDown(nFlags,sPoint,m_iContextX,m_iContextY))break;
				m_cClickMode	= 1;
				}
			}
		}while(0);


	if(m_cMouseOn)m_cClickMode=0;

	if(m_cClickMode && !m_bMouseTimer)						// Timer fr das Fenster verschieben
		{
		m_bMouseNum   = 0;
		m_bMouseTimer = 2;
		SetTimer(ID_MOUSEMOVE,165,0);
		}

	PositionCheck();

	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;

	if(!iRedraw && m_iRedrawFlags)DrawView(1);

}

//*****************************************************************************
//*
//*		OnMButtonDown
//*
//*****************************************************************************
//	Mittlere Maustaste getdckt
void CAnsiView::OnMButtonDown(UINT nFlags,CPoint sPoint)
{

	#ifdef TO_ANSISPLIT

	int		iField;
	CPoint	sBase;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4))
		{
		return;
		}

	sPoint.x   -= sBase.x;
	sPoint.y   -= sBase.y;

	#endif

	ScrollCursor(this,sPoint);

}

//*****************************************************************************
//*
//*		OnLButtonUp
//*
//*****************************************************************************
//	Linke Maustaste losgelassen
void CAnsiView::OnLButtonUp(UINT nFlags,CPoint sPoint)
{
int		iPosX,iPosY;



	#ifdef TO_ANSISPLIT

	int		iField;
	CPoint	sBase;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4) && !m_cCapture)
		{
		if(m_cCapture)
			{
			m_cCapture = 0;
			ReleaseCapture();
			}

		return;
		}

	sPoint.x   -= sBase.x;
	sPoint.y   -= sBase.y;

	#endif


	if(GetFocus()!=this)SetFocus();

	m_iClickPosX = sPoint.x-m_iMarginWidth;
	m_iClickPosY = sPoint.y;



	iPosX = (m_iClickPosX+m_iFontSizeX/2)/m_iFontSizeX+m_iScrollX;
	iPosY = ANSI_STEP(m_iScrollY,m_iClickPosY/m_iFontSizeY);


	if(!(nFlags&ANSI_NO_SUBCALL))
		{
		if(sPoint.x<m_iMarginWidth && sPoint.x>=0)			// Druck auf Rand
			{
			if(OnMarginClick((nFlags&0xFF)|MK_UP|MK_LEFT,sPoint,iPosY))return;
			}
		else{
			if(OnLButtonUp(nFlags,sPoint,iPosX,iPosY))return;
			}
		}

	m_cMouseOn = 0;

	if(m_bMouseTimer)
		{
		KillTimer(ID_MOUSEMOVE);
		m_bMouseTimer=0;
		}

	m_cCapture = 0;
	ReleaseCapture();

}

//*****************************************************************************
//*
//*		OnRButtonUp
//*
//*****************************************************************************
//	Rechte Maustaste losgelassen
void CAnsiView::OnRButtonUp(UINT nFlags,CPoint sPoint)
{
int		iPosX,iPosY;


	if(m_bMouseTimer==2)									// Timer ausschalten
		{
		KillTimer(ID_MOUSEMOVE);
		m_bMouseTimer=0;
		}


	#ifdef TO_ANSISPLIT

	int		iField;
	CPoint	sBase;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4))
		{
		return;
		}

	sPoint.x   -= sBase.x;
	sPoint.y   -= sBase.y;

	#endif


	iPosX = (sPoint.x-m_iMarginWidth+m_iFontSizeX/2)/m_iFontSizeX+m_iScrollX;
	iPosY = ANSI_STEP(m_iScrollY,sPoint.y/m_iFontSizeY);



	if(!(nFlags&ANSI_NO_SUBCALL))
		{
		if(sPoint.x<m_iMarginWidth && sPoint.x>=0)			// Druck auf Rand
			{
			if(OnMarginClick((nFlags&0xFF)|MK_UP|MK_RIGHT,sPoint,iPosY))goto Exit;
			}
		else{
			if(OnRButtonUp(nFlags,sPoint,iPosX,iPosY))goto Exit;
			}
		}


	if(m_cClickMode==1 || m_cClickMode==2)					// Kontextmen starten
		{
		#ifdef TO_ANSISPLIT
		sPoint.x   += sBase.x;
		sPoint.y   += sBase.y;
		#endif

		StartMenu(sPoint);
		}

Exit:

	if(m_cClickMode==3)										// Neuzeichnen wenn Verschub
		{
		m_cCapture = 0;
		ReleaseCapture();

		SetBeam();

		if(m_iOriginPtX || m_iOriginPtY)
			{
			iPosX = m_iScrollX;
			iPosY = m_iScrollY;

			if(m_iOriginPtX*-2>=m_iFontSizeX)iPosX++;
			if(m_iOriginPtY*-2>=m_iFontSizeY)iPosY++;

			if(iPosX!=m_iScrollX || iPosY!=m_iScrollY)
				{
				SetScroll(iPosX,iPosY);
				}

			m_iOriginPtX = 0;
			m_iOriginPtY = 0;
			DrawViewAll();
			DrawView(0);
			}

		#ifdef TO_ANSISPLIT
		m_cClickMode    = 0;
		m_iRedrawFlags |= NZ_CURSOR;
		SetActiveField(m_cSplitBack);						// Zurck auf ursprngliche Feld
		if(m_iRedrawFlags)DrawView(1);
		#endif
		}

	m_cClickMode = 0;

}

//*****************************************************************************
//*
//*		OnLButtonDblClk
//*
//*****************************************************************************
void CAnsiView::OnLButtonDblClk(UINT nFlags,CPoint sPoint)
{
int		iPosX,iPosY,iRedraw;


	#ifdef TO_ANSISPLIT

	int		iField;
	CPoint	sBase;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4))
		{
		if(iField==1 && !m_aSplitOn[0])SplitWindow(ANSI_SPLIT_X);
		if(iField==2 && !m_aSplitOn[1])SplitWindow(ANSI_SPLIT_Y);
		return;
		}

	sPoint.x   -= sBase.x;
	sPoint.y   -= sBase.y;

	#endif


	if(GetFocus()!=this)SetFocus();

	iRedraw         = NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;

	m_iClickPosX    = sPoint.x-m_iMarginWidth;
	m_iClickPosY    = sPoint.y;

	iPosX  = (m_iClickPosX+m_iFontSizeX/2)/m_iFontSizeX+m_iScrollX;
	iPosY  = ANSI_STEP(m_iScrollY,m_iClickPosY/m_iFontSizeY);


	do	{
		if(!(nFlags&ANSI_NO_SUBCALL))
			{
			if(sPoint.x<m_iMarginWidth && sPoint.x>=0)		// Druck auf Rand
				{
				if(OnMarginClick((nFlags&0xFF)|MK_DBLCLK|MK_LEFT,sPoint,iPosY))break;
				}
			else{
				if(OnLButtonDblClk(nFlags,sPoint,iPosX,iPosY))break;
				}
			}

		if(iPosX>=m_iScrollX)
			{
			PositionOk(iPosX,iPosY,0x08);					// Nicht auf Rand gedrckt ?
			PositionCheck();
			m_uMouseTicks=GetTickCount()+300;				// Maus kurz sperren
			}
		}while(0);


	PositionCheck();

	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;

	if(!iRedraw && m_iRedrawFlags)DrawView(1);

}

//*****************************************************************************
//*
//*		OnRButtonDblClk
//*
//*****************************************************************************
void CAnsiView::OnRButtonDblClk(UINT nFlags,CPoint sPoint)
{
int		iPosX,iPosY,iRedraw;


	#ifdef TO_ANSISPLIT

	int		iField;
	CPoint	sBase;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4))
		{
		return;
		}

	sPoint.x   -= sBase.x;
	sPoint.y   -= sBase.y;

	#endif


	iRedraw         = NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;

	iPosX = (sPoint.x-m_iMarginWidth+m_iFontSizeX/2)/m_iFontSizeX+m_iScrollX;
	iPosY = ANSI_STEP(m_iScrollY,sPoint.y/m_iFontSizeY);


	do	{
		if(!(nFlags&ANSI_NO_SUBCALL))
			{
			if(sPoint.x<m_iMarginWidth && sPoint.x>=0)		// Druck auf Rand
				{
				if(OnMarginClick((nFlags&0xFF)|MK_DBLCLK|MK_RIGHT,sPoint,iPosY))break;
				}
			else{
				if(OnRButtonDblClk(nFlags,sPoint,iPosX,iPosY))break;
				}
			}
		}while(0);

	PositionCheck();

	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;

	if(!iRedraw && m_iRedrawFlags)DrawView(1);


}

//*****************************************************************************
//*
//*		OnMouseMove
//*
//*****************************************************************************
//	Die Maus bewegt sich ber das Fenster
void CAnsiView::OnMouseMove(UINT nFlags,CPoint sPoint)
{
int		iRedraw;
int		iOrigX;
int		iOrigY;
int		iDiffX;
int		iDiffY;
int		iSizeX;
int		iSizeY;
int		iPosX1;
int		iPosY1;
int		iPosX;
int		iPosY;


	#ifdef TO_ANSISPLIT

	HCURSOR	hCursor;
	INT		iField;
	CPoint	sBase;

	iField = GetFieldOfPoint(sPoint,&sBase);				// Auf welchem Feld steht der Mauszeiger

	if(!(iField&4))
		{
		if(m_bMouseTimer==2)
			{
			KillTimer(ID_MOUSEMOVE);
			m_bMouseTimer=0;
			}

		if(m_cSplitBeam!=iField)							// Ist ein anderer Split-Cursor aktiv
			{
			m_cSplitBeam = iField;

			switch(iField)
				{
			default:hCursor = m_hCursorBase;	break;
			case 1:	hCursor = m_hCursorSizeWe;	break;
			case 2:	hCursor = m_hCursorSizeNs;	break;
			case 3:	hCursor = m_hCursorSizeAll;	break;
				}

			m_hCursorNow = hCursor;
			::SetCursor(hCursor);
			}

		return;
		}

	if(m_cSplitBeam)										// Ist ein Split-Cursor aktiv
		{
		m_cSplitBeam = 0;

		m_hCursorNow = m_hCursorBase;
		::SetCursor(m_hCursorBase);
		}

	sPoint.x   -= sBase.x;
	sPoint.y   -= sBase.y;

	#else

	int const	iField=4;

	#endif



	iRedraw         = NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;

	m_iClickPosX	= sPoint.x-m_iMarginWidth;
	m_iClickPosY	= sPoint.y;

	iPosX = (m_iClickPosX+m_iFontSizeX/2)/m_iFontSizeX+m_iScrollX;
	iPosY = ANSI_STEP(m_iScrollY,m_iClickPosY/m_iFontSizeY);


	if(m_iMoveOldX!=iPosX || m_iMoveOldY!=iPosY)			// Neue Position
		{
		m_iMoveOldX = iPosX;
		m_iMoveOldY = iPosY;
		}


	if(m_cMouseOn==2)										// Wo wurde gedrckt
		{
		if((nFlags&MK_LBUTTON) && m_cMouseOn)
			{
			if(OnMarginClick((nFlags&0xFF)|MK_MOVE,sPoint,iPosY))return;

			PositionCheck();

			m_iRedrawFlags &= ~NZ_LOCK;
			m_iRedrawFlags |=  iRedraw;

			if(!iRedraw && m_iRedrawFlags)DrawView(1);
			}

		return;
		}

	if(m_cClickMode)										// Rechte Maustaste gedrckt
		{
		if(m_cClickMode==3)									// Mit der rechten Maustaste Fenster verschieben
			{
			if(!(nFlags&MK_RBUTTON))
				{
				OnRButtonUp(nFlags,sPoint);
				return;
				}

			iDiffX	= m_iLastClickX-sPoint.x;				// Abstand zum letzten R-Click berechnen
			iDiffY	= m_iLastClickY-sPoint.y;
			iOrigX	= iDiffX%m_iFontSizeX;
			iOrigY	= iDiffY%m_iFontSizeY;
			iDiffX /= m_iFontSizeX;
			iDiffY /= m_iFontSizeY;
			iPosX	= m_iLastScrollX+iDiffX;
			iPosY	= m_iLastScrollY+iDiffY;

			if(iOrigX<0){iOrigX+=m_iFontSizeX;iPosX--;}
			if(iOrigY<0){iOrigY+=m_iFontSizeY;iPosY--;}
			if(iPosX <0){iPosX=0;iOrigX=0;}
			if(iPosY <0){iPosY=0;iOrigY=0;}

			if(iPosX!=m_iScrollX || iPosY!=m_iScrollY)		// Scrollen mit der rechten Maustaste
				{
				m_iOriginPtX = iOrigX;
				m_iOriginPtY = iOrigY;
				SetScroll(iPosX,iPosY);
				}											// Nur Verschub von Pixeln im Buchstaben
			else if(m_iOriginPtX!=iOrigX || m_iOriginPtY!=iOrigY)
				{
				m_iOriginPtX = iOrigX;
				m_iOriginPtY = iOrigY;
				DrawViewAll();
				DrawView(0);
				}

			return;
			}

															// Umschalten auf Fenster verschieben
		if(abs(sPoint.x-m_iLastClickX)>=4 || abs(sPoint.y-m_iLastClickY)>=4)
			{
			StartMove();
			}
		}


	if((nFlags&MK_LBUTTON) && m_cMouseOn)					// Linke Maustaste gedrckt
	do	{													// Innerhalb des sichtbaren Bereichs
		iPosX1 = 0;
		iPosY1 = 0;


		#ifdef	TO_ANSISPLIT

		iSizeX	= m_aSplitSizeX[m_iSplitActiv& 1];
		iSizeY	= m_aSplitSizeY[m_iSplitActiv>>1];

		#else

		iSizeX	= m_iScreenSizeX;
		iSizeY	= m_iScreenSizeY;

		#endif
															// Ist der Mauszeiger auerhalb des Anzeigebereiches
		if(sPoint.x< m_iMarginWidth     )iPosX1= (sPoint.x-m_iMarginWidth     )>>3   ;
		if(sPoint.x>=iSizeX-m_iFontSizeX)iPosX1=((sPoint.x-iSizeX+m_iFontSizeX)>>3)|1;
		if(sPoint.y< 0	  	            )iPosY1= (sPoint.y                    )>>3   ;
		if(sPoint.y>=iSizeY-m_iFontSizeY)iPosY1=((sPoint.y-iSizeY+m_iFontSizeY)>>3)|1;

		m_iMouseX = iPosX1;
		m_iMouseY = iPosY1;


		if(!m_iMouseX && !m_iMouseY)						// Inerhalb des Bildbereiches
			{
			if(m_uMouseTicks<=GetTickCount())
				{
				PositionOk(iPosX,iPosY,0x0A);
				PositionCheck();
				}

			if(m_bMouseTimer)								// Timer ausschalten
				{
				KillTimer(ID_MOUSEMOVE);
				m_bMouseTimer=0;
				}

			break;
			}

		if(m_cMouseOn==1 && !m_iMouseY)						// Nur X auserhalb vom Bild
			{
			if(m_uMouseTicks<=GetTickCount())
				{
				PositionOk(m_iCursorX,iPosY,0x0A);
				PositionCheck();
				}
			}

		if(!m_bMouseTimer)									// Timer einschalten
			{
			SetTimer(ID_MOUSEMOVE,165,0);
			m_bMouseTimer=1;
			}

		}while(0);
	else{
		OnMouseMove(nFlags,sPoint,iPosX,iPosY);
		PositionCheck();
		}

	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;

	if(!iRedraw && m_iRedrawFlags)DrawView(1);


}

//*****************************************************************************
//*
//*		OnTimer
//*
//*****************************************************************************
void CAnsiView::OnTimer(UINT_PTR uTimerId)
{
int		iRedraw;
CPoint	sPoint;
CPoint	sBase;



	if(!m_bMouseTimer || uTimerId!=ID_MOUSEMOVE)			// Ist das Verschieben mit der rechten Maustatste aktiv
		{
		CView::OnTimer(uTimerId);
		return;
		}


	if(m_bMouseTimer==2)									// Rechte Maustaste aktiv
		{
		m_bMouseNum++;

		if(m_bMouseNum<5)return;

		GetCursorPos  (&sPoint);
		ScreenToClient(&sPoint);
		KillTimer(ID_MOUSEMOVE);
		m_bMouseTimer = 0;

		#ifdef	TO_ANSISPLIT

		int iField;

		iField = GetFieldOfPoint(sPoint,&sBase);

		if(iField!=m_iMouseField)return;

		sPoint.x   -= sBase.x;
		sPoint.y   -= sBase.y;

		#endif
															// Wenn nicht bewegt dann Verschieben starten
		if(abs(sPoint.x-m_iLastClickX)<4 || abs(sPoint.y-m_iLastClickY)<4)
			{
			StartMove();
			}

		return;
		}


	if(m_uMouseTicks>GetTickCount())return;					// Scrollen wenn die Maus auerhalb ist

	iRedraw			= NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;

	PositionOk(m_iMouseX,m_iMouseY,0x0B);
	PositionCheck();

	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;

	if(!iRedraw && m_iRedrawFlags)DrawView(1);


}

//*****************************************************************************
//*
//*		OnHScroll
//*
//*****************************************************************************
//	Das Fenster soll horizontal gescrollt werden.
void CAnsiView::OnHScroll(UINT uCode,UINT uPos,CScrollBar *pScrollBar)
{

	#ifdef	TO_ANSISPLIT

	UINT	uIdx;
	UINT	uBar;


	   uIdx	= pScrollBar-m_aScrollBars;
	if(uIdx>=4)
		{
		if(pScrollBar)return;
		uIdx = (m_iSplitActiv<<1)&1;
		}

	uBar = uIdx>>1;

	switch(uCode)
		{
	case SB_LINERIGHT:		m_aScrollX[uBar] += 1;					break;
	case SB_LINELEFT:		m_aScrollX[uBar] -= 1;					break;
	case SB_PAGERIGHT:		m_aScrollX[uBar] += m_aScreenX[uBar]-1;	break;
	case SB_PAGELEFT:		m_aScrollX[uBar] -= m_aScreenX[uBar]-1;	break;
	case SB_THUMBPOSITION:	m_aScrollX[uBar]  = uPos;				break;
	case SB_THUMBTRACK:		m_aScrollX[uBar]  = uPos;				break;
		}

	if(!((uBar^m_iSplitActiv)&1))
		{
		m_iScrollX = m_aScrollX[uBar];
		}

	#else

	switch(uCode)
		{
	case SB_LINERIGHT:		m_iScrollX += 1;						break;
	case SB_LINELEFT:		m_iScrollX -= 1;						break;
	case SB_PAGERIGHT:		m_iScrollX += m_iScreenX-1;				break;
	case SB_PAGELEFT:		m_iScrollX -= m_iScreenX-1;				break;
	case SB_THUMBPOSITION:	m_iScrollX  = uPos;						break;
	case SB_THUMBTRACK:		m_iScrollX  = uPos;						break;
		}

	#endif

	PositionCheck();
}

//*****************************************************************************
//*
//*		OnVScroll
//*
//*****************************************************************************
//	Das Fenster soll vertikal gescrollt werden.
void CAnsiView::OnVScroll(UINT uCode,UINT uPos,CScrollBar *pScrollBar)
{
SCROLLINFO		sScroll;


	#ifdef	TO_ANSISPLIT

	UINT	uIdx;
	UINT	uBar;


	   uIdx	= pScrollBar-m_aScrollBars;
	if(uIdx>=4)
		{
		if(pScrollBar)return;
		uIdx = m_iSplitActiv|1;
		}

	uBar = uIdx>>1;

	switch(uCode)
		{
	case SB_LINEDOWN:		m_aScrollY[uBar] = ANSI_STEP(m_aScrollY[uBar],                  1);break;
	case SB_LINEUP:			m_aScrollY[uBar] = ANSI_STEP(m_aScrollY[uBar],                 -1);break;
	case SB_PAGEDOWN:		m_aScrollY[uBar] = ANSI_STEP(m_aScrollY[uBar], m_aScreenY[uBar]-1);break;
	case SB_PAGEUP:			m_aScrollY[uBar] = ANSI_STEP(m_aScrollY[uBar], 1-m_aScreenY[uBar]);break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:		if(m_iLineCount<0x7F00)			// Scroll-Bereich unter 16-Bit
								{
								m_aScrollY[uBar] = ANSI_STEP(uPos,0);
								}
							else{
								sScroll.cbSize	 = sizeof(SCROLLINFO);
								sScroll.fMask	 = SIF_TRACKPOS;
								GetScrollInfo(SB_DIRECT|uIdx,&sScroll);
								m_aScrollY[uBar] = ANSI_STEP(sScroll.nTrackPos,0);
								}
		}

	if(!((uIdx^m_iSplitActiv)&2))
		{
		m_iScrollY = m_aScrollY[uBar];
		}

	#else

	switch(uCode)
		{
	case SB_LINEDOWN:		m_iScrollY = ANSI_STEP(m_iScrollY,            1);break;
	case SB_LINEUP:			m_iScrollY = ANSI_STEP(m_iScrollY,           -1);break;
	case SB_PAGEDOWN:		m_iScrollY = ANSI_STEP(m_iScrollY, m_iScreenY-1);break;
	case SB_PAGEUP:			m_iScrollY = ANSI_STEP(m_iScrollY, 1-m_iScreenY);break;
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:		if(m_iLineCount<0x7F00)			// Scroll-Bereich unter 16-Bit
								{
								m_iScrollY	   = ANSI_STEP(uPos,0);
								}
							else{
								sScroll.cbSize = sizeof(SCROLLINFO);
								sScroll.fMask  = SIF_TRACKPOS;
								GetScrollInfo(SB_VERT,&sScroll);
								m_iScrollY     = ANSI_STEP(sScroll.nTrackPos,0);
								}
		}

	#endif

	PositionCheck();
}

//*****************************************************************************
//*
//*		OnMouseWheel
//*
//*****************************************************************************
//	Wird aufgerufen wenn das Mausrad bettigt wurde.
BOOL CAnsiView::OnMouseWheel(UINT nFlags,short iDelta,CPoint sPoint)
{
int		iCount,iRedraw,iOldY;


	iRedraw			= NZ_LOCK&m_iRedrawFlags;
	m_iRedrawFlags |= NZ_LOCK;
	iOldY			= m_iScrollY;


	if(OnMouseWheel(nFlags,iDelta))
		{
		m_iRedrawFlags &= ~NZ_LOCK;
		m_iRedrawFlags |=  iRedraw;

		if(!iRedraw && m_iRedrawFlags)DrawView(1);

		return TRUE;
		}

	if(nFlags&MK_CONTROL)
		{
		if(iDelta<0)OnVScroll(SB_PAGEDOWN,iDelta,GetScrollBarCtrl(SB_VERT));
		else		OnVScroll(SB_PAGEUP  ,iDelta,GetScrollBarCtrl(SB_VERT));
		}
	else{
		iCount = (nFlags&MK_SHIFT)? 9:3;

		for(;iCount>0;iCount--)
			{
			if(iDelta<0)OnVScroll(SB_LINEDOWN,iDelta,GetScrollBarCtrl(SB_VERT));
			else		OnVScroll(SB_LINEUP  ,iDelta,GetScrollBarCtrl(SB_VERT));
			}
		}


	if(iOldY!=m_iScrollY)									// Hat sich die Scroll-Position verndert
		{
		OnScroll(0);
		
		if(nFlags&MK_LBUTTON)
			{
			GetCursorPos  (&sPoint);
			ScreenToClient(&sPoint);
			OnMouseMove(nFlags,sPoint);						// Positionen nachziehen wenn Maustaste gedrckt
			}
		}

	m_iRedrawFlags &= ~NZ_LOCK;
	m_iRedrawFlags |=  iRedraw;

	if(!iRedraw && m_iRedrawFlags)DrawView(1);



return TRUE;
}

//*****************************************************************************
//*
//*		OnPreparePrinting
//*
//*****************************************************************************
//	Wird aufgerufen vor dem Drucken hier wird die Seitenanzahl berechent.
#ifndef 	TO_ANSIWND
BOOL CAnsiView::OnPreparePrinting(CPrintInfo *pInfo)
{
CDC			sDcPrint;
ANSI_FONT	sAnsiFont;
int			iSizeX,iSizeY;
int			iRealX,iRealY;
int			iResX ,iResY;
int			iLogX ,iLogY;
int			iPages;
int			iLine;
int			iNum;
int			x,y,z;
BOOL		bExit;



	iPages = 0;

	if(m_sPrinterData.uFlags&8)								// Drucken mit Zeilenumbruch
		{
		pInfo->SetMaxPage(1000);
		}
	else{													// Kein Zeilen-Umbruch
		iPages	= m_iLineCount+m_sPrinterData.uLines+1;
		iPages /= m_sPrinterData.uLines;

		if(m_iLineCount<=0)iPages=1;

		pInfo->m_nCurPage=m_iScrollY/m_sPrinterData.uLines+1;
		pInfo->SetMaxPage(iPages);
		}


		bExit=DoPreparePrinting(pInfo);						// Standardvorbereitung
	if(!bExit)return bExit;



	if(!pInfo->m_pPD->PrintSelection() || pInfo->m_bPreview)
		{
		m_sPrinterData.iMarkStartX = 0;						// Nicht die Auswahl drucken
		m_sPrinterData.iMarkStartY = 0;
		m_sPrinterData.iMarkStopX  = 0;
		m_sPrinterData.iMarkStopY  = m_iLineCount;
		}

//*****************************************************************************


	sDcPrint.Attach(pInfo->m_pPD->m_pd.hDC);				// Erzeuge Printer-DC
	sDcPrint.m_bPrinting = TRUE;

	iResX	= sDcPrint.GetDeviceCaps(HORZRES);				// Seiten Abmessungen
	iResY	= sDcPrint.GetDeviceCaps(VERTRES);
	iSizeX 	= sDcPrint.GetDeviceCaps(HORZSIZE);
	iSizeY 	= sDcPrint.GetDeviceCaps(VERTSIZE);
	iLogX 	= sDcPrint.GetDeviceCaps(LOGPIXELSX);
	iLogY 	= sDcPrint.GetDeviceCaps(LOGPIXELSY);
	iRealX 	= iSizeX-m_sPrinterData.uMarginLeft  -m_sPrinterData.uMarginRight;
	iRealY 	= iSizeY-m_sPrinterData.uMarginBottom-m_sPrinterData.uMarginTop;

	m_sPrinterData.iFontSizeY  = (iRealY*iResY)/iSizeY;		// Schrifthhe berechnen
	m_sPrinterData.iFontSizeY /=  m_sPrinterData.uLines;
	m_sPrinterData.iFontSizeX  =  m_sPrinterData.iFontSizeY*iLogX*27;
	m_sPrinterData.iFontSizeX /=                            iLogY*48;
	m_sPrinterData.iStartX	   = (m_sPrinterData.uMarginLeft       *iResX)/iSizeX;
	m_sPrinterData.iStartY	   = (m_sPrinterData.uMarginTop        *iResY)/iSizeY;
	m_sPrinterData.iFooterY	   = ((iSizeY-6)                       *iResY)/iSizeY;
	m_sPrinterData.iHeaderY    = (((m_sPrinterData.uMarginTop-4)/2)*iResY)/iSizeY;

	if(m_sPrinterData.iHeaderY<0)
		{
		m_sPrinterData.iHeaderY = 0;
		}

	sAnsiFont         = m_pGroup->sAnsiFont;				// Schrift erzeugen
	sAnsiFont.iWidth  = m_sPrinterData.iFontSizeX;
	sAnsiFont.iHeight = m_sPrinterData.iFontSizeY;
	sAnsiFont.iFlags  = ANSI_FIXED;

	if(m_sPrinterData.hFontPrint)
		{
		FontDelete(m_sPrinterData.hFontPrint);
		}

	m_sPrinterData.hFontPrint=FontCreate(&sAnsiFont);


	for(iNum=1;iNum<8;iNum++)								// Extra Schriften
		{
		if(m_sPrinterData.uFlags&0x10)						// Nur gleiche Schriftarten
			{
			m_sPrinterData.aFontEx[iNum]=m_sPrinterData.hFontPrint;
			continue;
			}
			      sAnsiFont.iFlags   = ANSI_FIXED;
		if(iNum&1)sAnsiFont.iFlags  |= ANSI_BOLD;
		if(iNum&2)sAnsiFont.iFlags  |= ANSI_ITALIC;
		if(iNum&4)sAnsiFont.iFlags  |= ANSI_UNDERLINE;

		m_sPrinterData.aFontEx[iNum] = FontCreate(&sAnsiFont);
		}

	SelectObject(sDcPrint.m_hDC,m_sPrinterData.hFontPrint); // Schriftabmessungen

	m_sPrinterData.iCharCount  = (iRealX*iResX)/iSizeX;
	m_sPrinterData.iCharCount /= m_sPrinterData.iFontSizeX;

	sDcPrint.Detach();


//*********************** Seitenanzahl bestimmen ******************************


	if(m_sPrinterData.uFlags&8)								// Drucken mit Zeilenumbruch
		{
		if(m_pLineSizes)
			{
			delete m_pLineSizes;
			m_pLineSizes=0;
			}

		m_pLineSizes = new int[m_iLineCount];
		iPages		 = 0;
		iNum		 = 0;
		iLine		 = m_sPrinterData.iMarkStartY;

		for(;iLine<m_iLineCount;iLine++)					// Zeilenbreiten berechnen
			{
			x=LineLength(iLine);
			m_pLineSizes[iLine]=x;

			if(iLine >m_sPrinterData.iMarkStopY)break;
			if(iLine==m_sPrinterData.iMarkStopY)
				{
				if(x>m_sPrinterData.iMarkStopX)
					{
					x = m_sPrinterData.iMarkStopX;
					}
				}

			if(iLine==m_sPrinterData.iMarkStartY)
				{
				x -= m_sPrinterData.iMarkStartX;
				}

			if(x<=0)iNum++;
			else	iNum+=(x+m_sPrinterData.iCharCount-1)/m_sPrinterData.iCharCount;

			if(iNum>=(int)m_sPrinterData.uLines)
				{
				iPages+=iNum/ m_sPrinterData.uLines;
						iNum%=m_sPrinterData.uLines;
				}
			}

		if(iNum || !iPages)iPages++;
		}


//*********************** Seiten Anfnge bestimmen ****************************


	if(m_sPrinterData.uFlags&8)								// Drucken mit Zeilenumbruch
		{
		if(m_pPageBegin)delete  m_pPageBegin;
		   m_pPageBegin=new int[iPages*2+4];

		x = 0;
		y = m_sPrinterData.iMarkStartY;
		z = m_sPrinterData.iMarkStartX;

		while(x<iPages)
			{
			m_pPageBegin[x*2  ] = y;
			m_pPageBegin[x*2+1] = z;
			iNum = 0;

			while(iNum<(int)m_sPrinterData.uLines)
				{
				if(m_iScrollY==y)pInfo->m_nCurPage=x+1;
				if(y>=m_iLineCount)break;
				if(y>m_sPrinterData.iMarkStopY)break;

				z += m_sPrinterData.iCharCount;

				if(z>=m_pLineSizes[y]){z=0;y++;}
				iNum++;
				}

			x++;
			}
		}

	m_sPrinterData.iPages=iPages;
	pInfo->SetMaxPage(iPages);


return bExit;
}
#endif

//*****************************************************************************
//*
//*		OnBeginPrinting
//*
//*****************************************************************************
//	Wird aufgerufen vor dem Druckenbegin.
//	Hier werden Schriften und Farben reserviert.
#ifndef 	TO_ANSIWND
void CAnsiView::OnBeginPrinting(CDC *pDC,CPrintInfo *pInfo)
{
AnsiColor		   *pColors;
ANSI_FONT			sAnsiFont;
ANSI_FONT_METRICS	sData;
HFONT				hFont;
int					iMode;
int					iNum;
int					x,y,z;
int					r,g,b;




//*************** TT-Schrift fr Vorschau *************************************


	if(!(m_sPrinterData.uFlags&0x10))						// Druckvorschau oder TT-Schrift
	do	{
		FontMetrics(m_sPrinterData.hFontPrint,&sData,pDC->m_hDC);

		if(sData.iIsVector)break;

		sAnsiFont			= m_pGroup->sAnsiFont;			// Schrift erzeugen
		sAnsiFont.iHeight	= m_sPrinterData.iFontSizeY;
		sAnsiFont.iWidth	= m_sPrinterData.iFontSizeX;
		sAnsiFont.iFlags	= ANSI_TRUETYPE|ANSI_FIXED;

		strlcpy(sAnsiFont.cFontName,_T("Courier New"),sizeof(sAnsiFont.cFontName));

			hFont=FontCreate(&sAnsiFont);
		if(!hFont)
			{
			strlcpy(sAnsiFont.cFontName,_T("Lucida Console"),sizeof(sAnsiFont.cFontName));
			hFont=FontCreate(&sAnsiFont);
			}

		if(!hFont)
			{
			strlcpy(sAnsiFont.cFontName,_T(""),sizeof(sAnsiFont.cFontName));
			hFont=FontCreate(&sAnsiFont);
			}

		if(hFont)
			{
			if(m_sPrinterData.hFontPrint)FontDelete(m_sPrinterData.hFontPrint);

			m_sPrinterData.hFontPrint = hFont;

			for(iNum=1;iNum<8;iNum++)						// Extra Schriften
				{
				if(m_sPrinterData.aFontEx[iNum])
				if(m_sPrinterData.aFontEx[iNum]!=m_sPrinterData.hFontPrint)
					{
					FontDelete(m_sPrinterData.aFontEx[iNum]);
					}

				if(m_sPrinterData.uFlags&0x10)				// Nur gleiche Schriftarten
					{
					m_sPrinterData.aFontEx[iNum]=m_sPrinterData.hFontPrint;
					continue;
					}

					      sAnsiFont.iFlags  = ANSI_TRUETYPE;
				if(iNum&1)sAnsiFont.iFlags |= ANSI_BOLD;
				if(iNum&2)sAnsiFont.iFlags |= ANSI_ITALIC;
				if(iNum&4)sAnsiFont.iFlags |= ANSI_UNDERLINE;

				m_sPrinterData.aFontEx[iNum]=hFont=FontCreate(&sAnsiFont);
				}
			}
		}while(0);

//*********** Kopf und Fuss uLines Schrift ************************************


	x = pDC->GetDeviceCaps(VERTRES);						// Seiten Abmessungen
	y = pDC->GetDeviceCaps(VERTSIZE);
	z = ((7*x)/y)/2;										// Schrifthhe berechnen

	sAnsiFont.iHeight	 = z;
	sAnsiFont.iWidth	 = 0;
	sAnsiFont.iFlags	 = 0;
	strlcpy(sAnsiFont.cFontName,_T("Arial"),sizeof(sAnsiFont.cFontName));

		m_sPrinterData.hFontLine = FontCreate(&sAnsiFont);
	if(!m_sPrinterData.hFontLine)
		{
		strlcpy(sAnsiFont.cFontName,_T("Courier New"),sizeof(sAnsiFont.cFontName));
		m_sPrinterData.hFontLine = FontCreate(&sAnsiFont);
		}

	if(!m_sPrinterData.hFontLine)
		{
		strlcpy(sAnsiFont.cFontName,_T(""),sizeof(sAnsiFont.cFontName));
		m_sPrinterData.hFontLine = FontCreate(&sAnsiFont);
		}

//*********************** Druckfarben erzeugen ********************************

	ColorDelete(&m_sPrinterData.pColorList);

	pColors=m_pGroup->pColorList;
	m_sPrinterData.iColorCount=0;

	if(pColors)												// Zhle Farben
		{
		while(pColors->pName)
			{
			z = pColors->uColorBk;
			y = pColors->uColorText;

			if(z==(int)0xFFFFFFFF)z=RGB(255,255,255);
			if(y==(int)0xFFFFFFFF)y=RGB(0,0,0);

			if(m_sPrinterData.uFlags&1)						// Schwarz/Weis
				{
				y = RGB(0,0,0);
				z = RGB(255,255,255);
				}

			if(m_sPrinterData.uFlags&2)						// Graustufen ?
				{
				r = (z)&0xFF;g=(z>>8)&0xFF;b=(z>>16)&0xFF;
				z = (r*77+g*151+b*28)>>8;
				r = (y)&0xFF;g=(y>>8)&0xFF;b=(y>>16)&0xFF;
				y = (r*77+g*151+b*28)>>8;

				if(z< y && y-z<200){z=y-200;if(z<0){y-=z;z=0;}}
				if(z>=y && y-z<200){y=z-200;if(y<0){z-=y;y=0;}}

				z = RGB(z,z,z);
				y = RGB(y,y,y);
				}

			   iMode=pColors->iFontMode;
			if(iMode!=-1)									// Extra Druckmodi
				{
				if(iMode&0x0100)iMode>>=4;
				}

			ColorInsert(&m_sPrinterData.pColorList,y,z,_T(""),0,iMode);
			m_sPrinterData.iColorCount++;					// Nchste Farbe
			pColors++;
			}
		}

}
#endif

//*****************************************************************************
//*
//*		OnEndPrinting
//*
//*****************************************************************************
//	Wird aufgerufen nach dem Drucken
//	Hier werden Schriften und Farben freigegeben
#ifndef 	TO_ANSIWND
void CAnsiView::OnEndPrinting(CDC *pDC, CPrintInfo* pInfo)
{
int iNum;


	if(m_pLineSizes){delete m_pLineSizes;m_pLineSizes=0;}
	if(m_pPageBegin){delete m_pPageBegin;m_pPageBegin=0;}


	for(iNum=1;iNum<8;iNum++)
		{
		if(!m_sPrinterData.aFontEx[iNum])continue;
		if( m_sPrinterData.aFontEx[iNum]!=m_sPrinterData.hFontPrint)
			{
			FontDelete(m_sPrinterData.aFontEx[iNum]);
			}

		m_sPrinterData.aFontEx[iNum]=0;
		}

	if(m_sPrinterData.hFontLine ){FontDelete(m_sPrinterData.hFontLine );m_sPrinterData.hFontLine =0;}
	if(m_sPrinterData.hFontPrint){FontDelete(m_sPrinterData.hFontPrint);m_sPrinterData.hFontPrint=0;}

	ColorDelete(&m_sPrinterData.pColorList);

	m_sPrinterData.iColorCount=0;

}
#endif

//*****************************************************************************
//*
//*		OnPrint
//*
//*****************************************************************************
//	Ausdrucken der einzelnen Seiten
#ifndef 	TO_ANSIWND
void CAnsiView::OnPrint(CDC *pDC,CPrintInfo *pInfo)
{
int			x,y;
TCHAR		cBuffer[512];
TCHAR		cTemp  [512];
LPCTSTR		pLine;
CDocument  *pDoc;
CTime		sTime;
RECT		sFrame;
RECT		sRect;
HDC			hDc;
DWORD	 	uCnt;
int			iPage;
int			iFlags;
int			iMode;
int			iNum;
int			iLen;
int			iPos;



	hDc		= pDC->m_hDC;
	iPage	= pInfo->m_nCurPage-1;

	if(iPage>m_sPrinterData.iPages)return;


//*********************** Grundeinstellung ****************************


	m_iPrintNow = -1;
	ColorChose  (hDc,0,1);
	SelectObject(hDc,m_sPrinterData.hFontPrint);

	sRect.left		= m_sPrinterData.iStartX;
	sRect.right		= m_sPrinterData.iStartX+m_sPrinterData.iFontSizeX*m_sPrinterData.iCharCount;
	sRect.top 		= m_sPrinterData.iStartY;
	sRect.bottom	= m_sPrinterData.iStartY+m_sPrinterData.iFontSizeY;

	ColorChose(hDc,0,1);

	iFlags			= m_sPrinterData.iStartX;
	iFlags			=(iFlags<<12)|1;


//*********************** Zeichne alle Zeilen *************************


	if(m_sPrinterData.uFlags&8)								// Drucken mit Zeilenumbruch
		{
		iNum = m_pPageBegin[iPage*2  ];
		iPos = m_pPageBegin[iPage*2+1];

		for(y=0;y<(int)m_sPrinterData.uLines;y++)
			{
		 		    x=m_sPrinterData.iCharCount;
			if(iNum  >m_sPrinterData.iMarkStopY)break;
			if(iNum ==m_sPrinterData.iMarkStopY)
			if(iPos+x>m_sPrinterData.iMarkStopX)x=m_sPrinterData.iMarkStopX-iPos;

			x     = PrintLine(hDc,&sRect,iNum,iPos,x,iFlags);
			iPos += x;

			if(iPos>=m_pLineSizes[iNum]){iNum++;iPos=0;}

			sRect.top    += m_sPrinterData.iFontSizeY;
			sRect.bottom += m_sPrinterData.iFontSizeY;
			}
		}
	else{													// Drucken ohne Zeilenumbruch
		x    = (iPage)? 0:m_sPrinterData.iMarkStartX;
		iNum =  iPage *   m_sPrinterData.uLines+m_sPrinterData.iMarkStartY;

		for(y=0;y<(int)m_sPrinterData.uLines;y++,iNum++)
			{
			   iPos  =m_sPrinterData.iCharCount;
			if(iNum  >m_sPrinterData.iMarkStopY)break;
			if(iNum ==m_sPrinterData.iMarkStopY)
			if(iPos  <m_sPrinterData.iMarkStopX)iPos=m_sPrinterData.iMarkStopX;

			iPos -= x;

			PrintLine(hDc,&sRect,iNum,x,iPos,iFlags);

			sRect.top    += m_sPrinterData.iFontSizeY;
			sRect.bottom += m_sPrinterData.iFontSizeY;

			x=0;
			}
		}

//******************** Rahmen zeichnen ****************************************

	if(m_sPrinterData.uFlags&0x20)
		{
		CPen	sPen(PS_SOLID,2,RGB(0,0,0));

		pDC->SelectStockObject(NULL_BRUSH);
			 SelectObject(pDC->m_hDC,(HPEN)sPen);

		y=pDC->GetDeviceCaps(VERTSIZE);if(y<1)y=1;
		y=pDC->GetDeviceCaps(VERTRES )/y;
		x=pDC->GetDeviceCaps(HORZSIZE);if(x<1)x=1;
		x=pDC->GetDeviceCaps(HORZRES )/x;

		sFrame.top 		= m_sPrinterData.iStartY -y*3;
		sFrame.bottom	= m_sPrinterData.iFooterY-y*3;
		sFrame.left		= m_sPrinterData.iStartX -x*3;
		sFrame.right	= m_sPrinterData.iStartX+m_sPrinterData.iFontSizeX*m_sPrinterData.iCharCount+x*2;

		pDC->Rectangle(&sFrame);

		if(m_sPrinterData.uFlags&0x40)						// Flchen ausmahlen
			{
			pDC->SelectStockObject(LTGRAY_BRUSH);
			}

		if(m_sPrinterData.pHeaderData && *m_sPrinterData.pHeaderData)
			{
			sFrame.top 		= m_sPrinterData.iHeaderY-y*3;
			sFrame.bottom	= m_sPrinterData.iStartY -y*3;
			pDC->Rectangle(&sFrame);
			}

		if(m_sPrinterData.pFooterData && *m_sPrinterData.pFooterData)
			{
			sFrame.top 		= m_sPrinterData.iFooterY-y*3;
			sFrame.bottom	= pDC->GetDeviceCaps(VERTRES)-1;
			pDC->Rectangle(&sFrame);
			}

		}

//*********************** Kopf und Fuzeilen **************************

	if(pInfo->m_pPD->PrintSelection() && !(pInfo->m_bPreview))return;

	pDC->SetBkMode   (TRANSPARENT);
	pDC->SetBkColor  (RGB(255,255,255));
	pDC->SetTextColor(RGB(  0,  0,  0));
	SelectObject(hDc,m_sPrinterData.hFontLine);


	for(iNum=0;iNum<2;iNum++)
		{
		    pLine=(iNum)? m_sPrinterData.pFooterData:m_sPrinterData.pHeaderData;
		if(!pLine)continue;

		   iLen=_tcslen(pLine);
		if(iLen<=0)continue;

		iMode = DT_CENTER|DT_TOP;

		for(x=0,iPos=0;x<iLen;x++)
			{
			if(iPos>sizeof(cBuffer)-64)break;

			cBuffer[iPos]=0;

			if(pLine[x]=='\\' && pLine[x+1]=='n')
				{
				cBuffer[iPos]='\n';
				pLine[x];iPos++;x++;
				continue;
				}

			if(pLine[x]!='&')
				{
				cBuffer[iPos]=pLine[x];iPos++;
				continue;
				}

			x++;

			switch(pLine[x])
					{
			case'L':cBuffer[iPos]='\b';cBuffer[iPos+1]=0;iPos++;continue;
			case'R':cBuffer[iPos]='\r';cBuffer[iPos+1]=0;iPos++;continue;
			case'C':cBuffer[iPos]='\v';cBuffer[iPos+1]=0;iPos++;continue;
			case'N':itostr(iPage+1,cBuffer+iPos);break;
			case'M':itostr(pInfo->GetMaxPage(),cBuffer+iPos);break;
			case'D':if(!m_pDocument)break;
					pDoc =GetDocument();
					y=strlcpy(cTemp,pDoc->GetPathName(),sizeof(cTemp));
					if(y>0)y--;
					for(;y>0;y--)if(cTemp[y]=='\\'){y++;break;}
					strlcat(cBuffer,cTemp+y,sizeof(cBuffer));
					break;
			case'P':if(!m_pDocument)break;
						pDoc =GetDocument();
					strlcpy(cTemp,pDoc->GetPathName(),sizeof(cTemp));
					strlcat(cBuffer,cTemp,sizeof(cBuffer));
					break;
			case'T':strtimeset(cTemp,time(0),STR_DATE|STR_WEEKDAY|STR_ABC|STR_LONG);
					strlcat(cBuffer,cTemp,255);
					break;
			case'Z':strtimeset(cTemp,time(0),STR_TIME);
					strlcat(cBuffer,cTemp,sizeof(cBuffer));
					break;
			case'U':y=_tcslen(cBuffer);
					uCnt=sizeof(cBuffer)-y;
					GetUserName(cBuffer+y,&uCnt);
					break;
			default:cBuffer[iPos  ]='&';
					cBuffer[iPos+1]=pLine[x];
					cBuffer[iPos+2]=0;
					iPos+=2;
					continue;
					}

			iPos = _tcslen(cBuffer);
			}


		cBuffer[iPos]	= 0;
		sRect.top 		= (iNum)? m_sPrinterData.iFooterY:m_sPrinterData.iHeaderY;
		sRect.bottom	= pDC->GetDeviceCaps(VERTRES)-1;
		iLen			= iPos;

		for(x=0,iPos=0;x<iLen;)
			{
			   y=_tcscspn(cBuffer+x,_T("\n\r\b\v"));
			if(y==0)
				{
				if(cBuffer[x+y]=='\r'){iMode=DT_RIGHT |DT_TOP;x++;continue;}
				if(cBuffer[x+y]=='\b'){iMode=DT_LEFT  |DT_TOP;x++;continue;}
				if(cBuffer[x+y]=='\v'){iMode=DT_CENTER|DT_TOP;x++;continue;}
				if(cBuffer[x+y]=='\n'){iPos++;				  x++;continue;}
				}

			memset(cBuffer+x-iPos,'\n',iPos*sizeof(TCHAR));
			pDC->DrawText(cBuffer+x-iPos,y+iPos,&sRect,iMode|DT_NOCLIP|DT_NOPREFIX);
			x+=y;
			}

		}
}
#endif

//*****************************************************************************
//*
//*		OnPageSetup
//*
//*****************************************************************************
//	Dialog fr Einstellungen der Druckerausgaben ffnen.
#ifndef TO_ANSIWND
void CAnsiView::OnPageSetup()
{
CAnsiPrintDlg	sDlg(this);
int				iRet;



	   iRet=sDlg.DoModal(&m_sPrinterData);
	if(iRet==IDOK)ColorSave();

	if(m_iRedrawFlags)DrawView(1);

}
#endif

//*****************************************************************************
//*
//*		Debug Funktionen
//*
//*****************************************************************************
#ifdef _DEBUG
void CAnsiView::AssertValid() const
{ 	     CView::AssertValid();}
void CAnsiView::Dump(CDumpContext &sDc)	const
{	     CView::Dump(sDc);}
#endif

//*****************************************************************************
//*
//*		SetScrollInfo
//*
//*****************************************************************************
//	Einstellen der Infos fr eine ScrollBar
//	iBar		: Ist die ID fr die ScrollBar (0=SB_HORZ 1=SB_VERT)
//	iFlags		: Diverse Flags
//					Bit0 = Neu zeichnen
//					Bit1 = Kein OnSize(...) aufrufen
//	Ergibt in Bit0 ob der Aufruf erfolgeich war und in Bit1 ob sich
//	die Schirmgre gendert hat.
int	CAnsiView::SetScrollInfo(int iBar,LPSCROLLINFO pInfo,INT iFlags)
{
int		iChange;
int		iOn;



	ANSI_BAR(iBar);											// Bei Split-Fenstern den Index anpassen

	ASSERT(iBar>=0 && iBar<ENTRIES(m_aScrollBars));
	ASSERT((pInfo->fMask&SIF_ALL)==SIF_ALL);
	ASSERT(pInfo->nMin==0);


	if((UINT)pInfo->nMax>pInfo->nPage)
		{
		iOn = 1;
		}
	else if(pInfo->fMask&SIF_DISABLENOSCROLL)
		{
		iOn	= 1;
		}
	else{
		iOn	= 0;
		}


	if(m_aScrollBarsOn[iBar]!=iOn)							// Hat sich die Sichtbarkeit verndert
		{
		m_aScrollBarsOn[iBar] = iOn;

		if(iOn==0)
			{
			m_aScrollBars[iBar].ShowScrollBar(iOn);
			}

		if(m_iScreenFullX>0 && m_iScreenFullY>0 && !(iFlags&2))
			{
			OnSize(0,m_iScreenFullX,m_iScreenFullY);
			}

		if(iOn!=0)
			{
			m_aScrollBars[iBar].ShowScrollBar(iOn);
			}
		
		iChange = 2;
		}
	else{
		iChange = 0;
		}


	m_aScrollBars[iBar].EnableWindow(TRUE);	



return m_aScrollBars[iBar].SetScrollInfo(pInfo,iFlags&1)|iChange;
}

//*****************************************************************************
//*
//*		SetMargin
//*
//*****************************************************************************
//	Erzeugt einen Rand in der angegebenen Breite
//	iWidth		: Ist die Breite des Randes in Pixel
//	Ergibt 1 wenn Rand gendert wurde
int	CAnsiView::SetMargin(int iWidth)
{
CAnsiView  *pClass;
int			x,iId;



	if(iWidth<   0)iWidth=0;
	if(iWidth>2000)iWidth=0;
	if(iWidth==m_iMarginWidth)return 0;


	pClass	= m_pClassLast;
	iId		= m_pGroup->iId;

	while(pClass)											// Alle Fenster durchsuchen
		{
		if(iId==m_pGroup->iId)
			{
			pClass->m_iRedrawFlags=NZ_ALL;
			pClass->m_iMarginWidth=iWidth;

			if(!pClass->m_hWnd)return 1;

			x = pClass->m_iScreenFullX;
			    pClass->m_iScreenFullX+=1;

			pClass->m_iScrollOldX=-1000;
			pClass->OnSize(0,x,m_iScreenFullY);
			pClass->PositionCheck();
			pClass->DrawView(1);
			}

		pClass=pClass->m_pClassPrev;
		}


return 1;
}

//*****************************************************************************
//*
//*		LineLength
//*
//*****************************************************************************
//	Die Lnge einer Zeile in Buchstaben abfragen.
//	iLine		: Ist die Nummer der Zeile.
//	Ergibt die Lnge einer Zeile.
int CAnsiView::LineLength(int iLine)
{
	return 0;
}

//*****************************************************************************
//*
//*		SetCursor
//*
//*****************************************************************************
//	Setzt den Cursor auf einen neue Position
//	sPos		: Sind die neuen Zeilenkoordinaten fr die Cursor-Position
//	iPosX/Y		: Sind die neuen Zeilenkoordinaten fr die Cursor-Position
//	iMode		: Soll die Marke mitgezogen werden
//	Ergibt die alte Cursor Position
CPoint CAnsiView::SetCursor(CPoint sPos,int iMode){return SetCursor(sPos.x,sPos.y,iMode);}
CPoint CAnsiView::SetCursor(int iPosX,int iPosY,int iMode)
{
CPoint	iOld;


	iOld = CPoint(m_iCursorX,m_iCursorY);

	PositionOk(iPosX,iPosY,(iMode)? 0x02:0x00);
	PositionCheck(2);
	DrawView(1);



return iOld;
}

//*****************************************************************************
//*
//*		GetContext
//*
//*****************************************************************************
//	Ergibt die Position des letzten Contex-Rechts-Mausklicks
CPoint CAnsiView::GetContext()
{
	return CPoint(m_iContextX,m_iContextY);
}

//*****************************************************************************
//*
//*		SetScroll
//*
//*****************************************************************************
//	Setzt die Scroll-Position
//	sPos		: Sind die neuen Zeilenkoordinaten fr die Scroll-Position
//	iPosX/Y		: Sind die neuen Zeilenkoordinaten fr die Scroll-Position
//	iRedraw		: Soll das Fenster neu gezeichnet werden.
//	Ergibt die alte Scroll Position
CPoint CAnsiView::SetScroll(CPoint sPos){return SetScroll(sPos.x,sPos.y);}
CPoint CAnsiView::SetScroll(int iPosX,int iPosY)
{
CPoint		sOld;


	sOld = CPoint(m_iScrollX,m_iScrollY);

	m_iScrollX = iPosX;
	m_iScrollY = ANSI_STEP(iPosY,0);

	PositionCheck(2);
	DrawView(1);


return sOld;
}

CPoint CAnsiView::SetScroll(CPoint sPos,int iRedraw){return SetScroll(sPos.x,sPos.y,iRedraw);}
CPoint CAnsiView::SetScroll(int iPosX,int iPosY,int iRedraw)
{
CPoint		sOld;


	sOld = CPoint(m_iScrollX,m_iScrollY);

	m_iScrollX = iPosX;
	m_iScrollY = ANSI_STEP(iPosY,0);


	if(iRedraw)
		{
		PositionCheck(2);
		DrawView(1);
		}
	else{
		PositionCheck(6);
		}


return sOld;
}

//*****************************************************************************
//*
//*		GetScroll
//*
//*****************************************************************************
//	Ergibt die Scroll Position
CPoint CAnsiView::GetScroll()
{
	return CPoint(m_iScrollX,m_iScrollY);
}

//*****************************************************************************
//*
//*		GetScroll
//*
//*****************************************************************************
//	Ergibt die Fenstergre in Buchstaben
CSize CAnsiView::GetScreen()
{
	return CSize(m_iScreenX,m_iScreenY);
}

//*****************************************************************************
//*
//*		SetBeam
//*
//*****************************************************************************
//	Stellt das Handle fr das Mauszeigerbild ein.
//	hCursor		: Ist das neue Handle des Cursors. (Bei 0 Beam benutzen)
//	Ergibt das vorherige Handle des Mauszeigerbildes.
HCURSOR	CAnsiView::SetBeamEx(LPCTSTR uCursor){return SetBeam(::LoadCursorW(NULL,(LPCWSTR)uCursor));}
HCURSOR CAnsiView::SetBeam  (HCURSOR hCursor)
{
HCURSOR		hOld;


	if(hCursor==NULL)										// Beam benutzen
		{
		hCursor = m_hCursorBeam;
		}

	if(m_hCursorBase==hCursor)								// Keine nderung
		{
		return hCursor;
		}

	hOld			= m_hCursorBase;
	m_hCursorBase	= hCursor;

	#ifdef	TO_ANSISPLIT

	if(m_cSplitBeam)										// Ist ein Split-Cursor aktiv
		{
		return hOld;
		}

	#endif

	m_hCursorNow = hCursor;
	::SetCursor(hCursor);


return hOld;
}

//*****************************************************************************
//*
//*		StartMenu
//*
//*****************************************************************************
//	Startet ein Kontextmen bei einer bestimmten Position.
//	sPos	: Sind die Fenserkoordinaten fr das Men. Sie werden automatisch
//			  auf Bildschirmkoortinaten umgerechnet.
//	pMenu	: Ist der Zeiger auf die Men-Klasse. Bei NULL wird sie neu erstellt.	
void CAnsiView::StartMenu(CPoint sPos,CMenu *pMenu)
{
CMenu	sMenu;
CRect	sRect;


	if(!pMenu)
		{
		pMenu=&sMenu;
		sMenu.CreatePopupMenu();
		}


	#ifndef TO_ANSIWND
	pMenu->AppendMenu(MF_STRING,ID_FILE_PAGE_SETUP , STRING("&Seite einrichten"    ,ID_ANSI_MENU1));
	#endif

	if(!m_cColorDisable)
	pMenu->AppendMenu(MF_STRING,ID_VIEW_COLOR	   , STRING("&Farben einstellen"   ,ID_ANSI_MENU2));
	pMenu->AppendMenu(MF_STRING,ID_VIEW_FONT	   , STRING("Schri&ften einstellen",ID_ANSI_MENU3));

	#ifdef  ANSI_ICONS
	AddMenuBitmap(pMenu,ID_VIEW_FONT		,ANSI_ICONS,0);
	AddMenuBitmap(pMenu,ID_VIEW_COLOR		,ANSI_ICONS,1);
	#ifndef TO_ANSIWND
	AddMenuBitmap(pMenu,ID_FILE_PAGE_SETUP  ,ANSI_ICONS,2);
	#endif 	
	#endif

	
	GetWindowRect(&sRect);
	sPos += sRect.TopLeft();
	pMenu->TrackPopupMenu(TPM_LEFTALIGN,sPos.x,sPos.y,this);
	if(pMenu==&sMenu)sMenu.DestroyMenu();

}

//*****************************************************************************
//*
//*		OnButtonDownX
//*
//*****************************************************************************
//	Es wurde Seite auf/ab seitlich auf der Maus gedrckt
LRESULT CAnsiView::OnButtonDownX(WPARAM wParam,LPARAM lParam)
{

	if(wParam&0x00020000)
		{
		OnKeyDown(VK_PRIOR,1,0x0054);
		OnKeyUp  (VK_PRIOR,1,0x4054);
		}

	if(wParam&0x00010000)
		{
		OnKeyDown(VK_NEXT ,1,0x0055);
		OnKeyUp  (VK_NEXT ,1,0x4055);
		}

return TRUE;
}

//*****************************************************************************
//*
//*		OnAnsiInfo
//*
//*****************************************************************************
//	Callback fr Nachrichtenabfrage Nachrichten.
LRESULT CAnsiView::OnAnsiInfo(WPARAM wParam,LPARAM lParam)
{
CPoint	sPoint;


	switch(wParam)
		{
	case ANSI_GETCURSOR:										// Cursor-Position lesen
	case ANSI_GETMARK:	sPoint=GetCursor();						// Marken-Position lesen
						return MAKELONG(sPoint.x,sPoint.y);
	case ANSI_GETSCROLL:										// Scroll-Position lesen
						sPoint=GetScroll();
						return MAKELONG(sPoint.x,sPoint.y);
	case ANSI_SETCURSOR:										// Cursor-Position setzen
						sPoint=SetCursor((CPoint)lParam,0);
						return MAKELONG(sPoint.x,sPoint.y);
	case ANSI_SETMARK:											// Marken-Position setzen
						sPoint=SetCursor((CPoint)lParam,1);
						return MAKELONG(sPoint.x,sPoint.y);
	case ANSI_SETSCROLL:										// Scroll-Position setzen
						sPoint=SetScroll((CPoint)lParam);
						return MAKELONG(sPoint.x,sPoint.y);
		}


return 0;
}

//*****************************************************************************
//*
//*		ConfigSetPath
//*
//*****************************************************************************
//	Sellt den Pfad fr das Laden der Konfiguration ein.
//	pPath		: Zeiger auf den neuen Pfad
void CAnsiView::ConfigSetPath(LPCTSTR pPath)
{
int		iLen;


	if(m_pConfigPath)
		{
		delete m_pConfigPath;
		m_pConfigPath=0;
		}

	if(pPath)
		{
		iLen = _tcslen(pPath)+1;
		m_pConfigPath = new TCHAR[iLen];
		memcpy(m_pConfigPath,pPath,iLen*sizeof(TCHAR));
		}

};

//*****************************************************************************
//*
//*     ConfigPath
//*
//*****************************************************************************
//	Speichert einen Registriy-Pfad in einem Puffer. "Software\<Vendor>\<AppName>\<Path>"
//	pBuffer		: Zeiger auf den Puffer
//	iSize		: Ist die Puffergre
//	pPath		: Ist der Pfad der angehngt wird.
//				  Ist der Pfad 0 so wird er aus m_pGroup->pConfig gennommen.
//				  Fngt der Pfad mit einem '#' Zeichen an so wird er als
//				  absoluer Pfad interpretiert.
//	Ergibt die Lnge des erstelllen Pfades.
int CAnsiView::ConfigPath(LPTSTR pBuffer,int iSize,LPCTSTR pPath)
{
LPCTSTR		pName=_T("Software");
LPCTSTR		pNameApp;
LPCTSTR		pNameVen;
int			iLen;
int			iPos;



	if(iSize<=0)return 0;

	pBuffer[0] = 0;

	if(pPath && *pPath=='#')								// Absoltuer Pfad
		{
		pPath++;
		iLen=_tcslen(pPath);
		if(iLen+1>iSize)return 0;
		memcpy(pBuffer,pPath,(iLen+1)*sizeof(TCHAR));
		return iLen;
		}


	pNameApp = AfxGetApp()->m_pszRegistryKey;
	pNameVen = AfxGetApp()->m_pszAppName;


	if(m_pConfigPath)
		{
		pName	 = m_pConfigPath;
		pNameApp = 0;
		pNameVen = 0;
		}

	pBuffer[0]	 = 0;
	iPos		 = 0;

	if(pName)												// Pfad erstellen
		{
		iLen = _tcslen(pName);
		if(iPos+iLen+2>iSize)return iPos;
		_tcscpy(pBuffer+iPos,pName);
		iPos+=iLen;
		}

	if(pNameApp && *pNameApp)
		{
		iLen = _tcslen(pNameApp);
		if(iPos+iLen+2>iSize)return iPos;
		pBuffer[iPos]='\\';iPos++;
		_tcscpy(pBuffer+iPos,pNameApp);
		iPos+=iLen;
		}

	if(pNameVen && *pNameVen)
		{
		iLen = _tcslen(pNameVen);
		if(iPos+iLen+2>iSize)return iPos;
		pBuffer[iPos]='\\';iPos++;
		_tcscpy(pBuffer+iPos,pNameVen);
		iPos+=iLen;
		}

	if(pPath && *pPath)
		{
		iLen = _tcslen(pPath);
		if(iPos+iLen+2>iSize)return iPos;
		pBuffer[iPos]='\\';iPos++;
		_tcscpy(pBuffer+iPos,pPath);
		iPos+=iLen;
		}


return iPos;
}

//*****************************************************************************
//*
//*		StartMove
//*
//*****************************************************************************
//	Startet das Verschieben des Fensterinhalts mit der rechten Maustatste.
void CAnsiView::StartMove()
{

	if(m_bMouseTimer==2)									// Timer ausschalten
		{
		KillTimer(ID_MOUSEMOVE);
		m_bMouseTimer=0;
		}

	if(!m_cClickMode)return;

	#ifdef TO_ANSISPLIT

	int		iField;

	if(m_cCaret){HideCaret();m_cCaret=0;}
	m_cSplitBack	= m_iSplitActiv;
	iField			= m_iMouseField;

	SetActiveField(iField);

	#else

	const int	iField=3;

	#endif

	SetBeam(m_hCursorHand);
	m_iLastScrollX	= m_iScrollX;
	m_iLastScrollY	= m_iScrollY;
	m_cCapture		= iField;
	m_cClickMode	= 3;
	SetCapture();

}

//*****************************************************************************
//*
//*		Virtuelle Nachrichten Funktionen
//*
//*****************************************************************************
#ifdef USE_VIRTUAL_LINES
int	 CAnsiView::TranslateLine  (unsigned uLine,int iDelta){return uLine+iDelta;}
#endif

void CAnsiView::OnScrollChange (){}
void CAnsiView::OnSize         (int iSizeX,int iSizeY,int iDeltaX,int iDeltaY){}
int  CAnsiView::OnScroll       (UINT nChar                                   ){return 0;}
int  CAnsiView::OnKeyUp        (UINT nChar, UINT nFlags                      ){return 0;}
int  CAnsiView::OnKeyDown      (UINT nChar, UINT nFlags                      ){return 0;}
int  CAnsiView::OnLButtonDblClk(UINT nFlags,CPoint sPoint,int iPosX,int iPosY){return 0;}
int  CAnsiView::OnRButtonDblClk(UINT nFlags,CPoint sPoint,int iPosX,int iPosY){return 0;}
int  CAnsiView::OnLButtonDown  (UINT nFlags,CPoint sPoint,int iPosX,int iPosY){return 0;}
int  CAnsiView::OnRButtonDown  (UINT nFlags,CPoint sPoint,int iPosX,int iPosY){return 0;}
int  CAnsiView::OnLButtonUp    (UINT nFlags,CPoint sPoint,int iPosX,int iPosY){return 0;}
int  CAnsiView::OnRButtonUp    (UINT nFlags,CPoint sPoint,int iPosX,int iPosY){return 0;}
int  CAnsiView::OnMouseMove    (UINT nFlags,CPoint sPoint,int iPosX,int iPosY){return 0;}
int  CAnsiView::OnMarginClick  (UINT nFlags,CPoint sPoint          ,int iPosY){return 0;}
int  CAnsiView::OnMouseWheel   (UINT nFlags,short  iDelta                    ){return 0;}
int  CAnsiView::OnSplitChanged (UINT nFlags,int    iSlplitX, int iSlplitY    ){return 0;}


//*****************************************************************************
//*
//*		OnSetCursor
//*
//*****************************************************************************
BOOL CAnsiView::OnSetCursor(CWnd *pWnd,UINT nHitTest,UINT uMsg)
{
HWND	hWnd;


	hWnd = (pWnd)? pWnd->m_hWnd:NULL;

	if(hWnd==m_hWnd)								// Cursor am Hauptfenster
		{
		::SetCursor(m_hCursorNow);
		return TRUE;
		}
	

return FALSE;
}

//*****************************************************************************
//*
//*		OnChar
//*
//*****************************************************************************
//	Ein Zeichen wurde eingegeben.
int CAnsiView::OnChar(UINT nChar,UINT nFlags)
{

	if(nChar==VK_TAB && (m_iKeyState&KEY_CLTR))
		{
		NextField((m_iKeyState&KEY_SHIFT)? 1:0);
		return 0;
		}

	return 0;
}

//*****************************************************************************
//*
//*		AllowDrop
//*
//*****************************************************************************
//	Schaltet das Fallenlassen von Datein im Fenster ein oder aus.
#ifndef TO_ANSIWND
void CAnsiView::AllowDrop(BOOL bEnable)
{
unsigned	uStyleEx;

	uStyleEx = GetWindowLong(m_hWnd,GWL_EXSTYLE);

	if(bEnable)
			uStyleEx |= WS_EX_ACCEPTFILES;
	else	uStyleEx &=~WS_EX_ACCEPTFILES;

	SetWindowLong(m_hWnd,GWL_EXSTYLE,uStyleEx);

}
#endif

//*****************************************************************************
//*
//*		OnDropFiles
//*
//*****************************************************************************
//	Callback wenn Dateien auf das Fenster fallen gelassen werden.
#ifndef TO_ANSIWND
void CAnsiView::OnDropFiles(HDROP hDropInfo)
{
TCHAR		cFile[MAX_PATH];
CDocument  *pDoc;
int			iNum;


		pDoc = GetDocument();
	if(!pDoc)
		{
		DragFinish(hDropInfo);
		return;
		}

	iNum = DragQueryFile(hDropInfo,0,cFile,MAX_PATH);
	DragFinish(hDropInfo);


	if(iNum>0)
		{
		if(pDoc->SaveModified())
			{
			pDoc->OnOpenDocument(cFile);
			pDoc->SetPathName(cFile);
			RedrawWindow();
			}
		}


}
#endif

//*****************************************************************************
//*
//*		FindVirtualLine
//*
//*****************************************************************************
//	Sucht nach einer Zeilennummer im sichtbaren Bereich der Fensters.
//	iLine		: Ist die globale Zeilennummer
//	Ergibt den Offest der gefundenen Zeile relativ zum Scroll-Y-Start oder -1
//	wenn die Zeile nicht gefunden wurde
#ifdef  USE_VIRTUAL_LINES
int CAnsiView::FindVirtualLine(int iLine)
{
int		iPos;
int		iOff;

	iPos = m_iScrollY;

	for(iOff=0;iOff<=iScreenY;iOff++)
		{
		if(iPos==iLine)return iOff;
		iPos = ANSI_STEP(iPos,1);
		}
	}

return -1;
}
#endif


//*****************************************************************************
//*
//*     TabbedTextOutEx
//*
//*****************************************************************************
//	Eine Zeile auf den Schirm zeichnen mit Farben und Lngenausgleich.
//	hDc			: Gertekontext
//	pRect		: Pixelkoordinaten des Bereiches in dem der Text gezeichnet wird.
//	pString		: Zeiger auf den Text (Wenn das Bit gesetzt is ist er UTF8 kodiert)
//	pColors 	: Zeiger auf die Farben zu den Buchstaben
//	uColor			Wenn Bits 8-15 ungleich Null sind so ist es die Hintergrundfarbe.
//	iCount		: Ist die Anzahl der Buchstaben. Bei UTF8 kann der pString mehr 
//				  Bytes enthalten als iCount.	
//	iFlags		: Diverse Bits fr die Zeile (iFlags muss an ColorChose weiter gegeben werden!)
//					Bit0	  = Ist fr die die Ausgabe auf einem Drucker bei einem Aufruf aus OnPrint()
//					Bit1	  = Ist es nicht das aktive Feld bei TO_ANSISPLIT
//					Bit8	  = Ein extra Leerzeichen anhngen
//					Bit9	  = Bis zum Ende das Rechteck ausmalen
//					Bit10	  = Es sind UTF-8 Zeichen im Text
//					Bit11	  = Unterstreichen ist erlaubt
//					Bit12..31 = Ist der X-Ursprung fr Tabulatoren
//	iTabs		: Ist die Breite eines Tabs in Zeichen.
//	Ergibt die Anzahl der ausgegbenen Pixel in X-Richtung.
LONG CAnsiView::TabbedTextOutEx(HDC hDc,LPRECT pRect,const LPBYTE pString,const LPWORD pColors,INT iCount,INT iFlags,INT iTabs)
{
#define MASK_TRANS		0x80000000	
#define MASK_NULL		0x00000000
#define MASK_UTF8		0x0F000000
#define MASK_MAX		0x40000000
#define MASK_MIN		0x20000000
#define LINE_MAX		1024		
INT		aArray[LINE_MAX+4];
wchar_t	aWide[4];
CHAR	aText[4];
SIZE	sSize;
RECT	sArea;
RECT	sPen;
UINT	uSign;
WORD	wMask;
WORD	wPrev;
WORD	wColor;
int		iOrigin;
int		iWidth;
int		iFontX;
int		iBase;
int		iMode;
int		iNext;
int		iBits;
int		iLast;
int		iDiff;
int		iChar;
int		iOut;
int		iPos;
int		iMax;
int		iIdx;
int		iOff;
int		iLen;
int		iEnd;
int		iNum;




	   iWidth = pRect->right-pRect->left;
	if(iWidth<=0)return 0;									// Ist nichts zu Zeichen

	#ifndef 	TO_ANSIWND
	if(iFlags&1)iFontX = m_sPrinterData.iFontSizeX;	else	
	#endif 	
	iFontX = m_iFontSizeX;	
		

	if(iTabs==0)iTabs=4;
	if(iCount>LINE_MAX)iCount=LINE_MAX;

	iMax	= iWidth/iFontX+1;
	iTabs  *= iFontX;

	if(iCount>iMax)											// Ist der Text lnger als das Rechteck
		{
		iFlags &= ~0x0300;
		iCount  =  iMax;
		}

	wMask	= (iFlags&0x0800)? 0x7F7F:0xFFFF;				// Ist Unterstreichen mglich
	iBase	= (WORD)pRect->left;
	iOrigin	= iFlags>>12;
	iBits	= MASK_NULL;
	iChar	= 0;


	for(iPos=0;iPos<iCount;)								// Die Buchstabenlngen einlesen
		{
		iMax = iCount-iPos;

		if(iFlags&0x0400)									// Ist der Text UTF8 kodiert
			{
			for(iNum=0;iNum<iMax;iNum++)
				{
				if(pString[iChar]>=0xC0)break;
				if(pString[iChar]=='\t')break;
				iChar++;
				}
			}
		else{
			for(iNum=0;iNum<iMax;iNum++)					// Suche nchstes TAB
				{
				if(pString[iChar]=='\t')break;
				iChar++;
				}
			}

								
		if(iNum>0)											// Lngen der ASCII-Buchstaben berechnen
			{
			GetTextExtentExPointA(hDc,(char*)pString+iChar-iNum,iNum,0x8000,NULL,aArray+iPos+1,&sSize);

			aArray[iPos] = 0;
			iMax = iPos+iNum;

			for(;iPos<iMax;iPos++)
				{
				iDiff		 = aArray[iPos+1];				// Breite eines Buchstabens
				iDiff       -= aArray[iPos];
				aArray[iPos] = iBase|iBits;
				iBase       += iFontX;

				if(iDiff==iFontX)
					{
					iBits	 = MASK_NULL;					// Tranparent Flag lschen
					}
				else if(iDiff<iFontX)						// Ist der Buchstabe zu schmall
					{
					aArray[iPos] |= MASK_MIN;				
					iBits		  = MASK_NULL;
					}
				else{										// Ist der Buchstabe zu breit
					aArray[iPos] |= MASK_TRANS|MASK_MAX;				
					iBits		  = MASK_TRANS;
					}
				}
			}

		
		while(iPos<iCount && pString[iChar]=='\t')			// Tabulator gefunden	
			{
			iEnd  = iBase+iTabs;
			iEnd -=(iEnd-iOrigin)%iTabs;

			aArray[iPos] = iBase|iBits;
			iBase        = iEnd;
			iBits		 = 0;
			iChar       += 1;
			iPos		+= 1;
			}

															// UTF-8 Zeichen gefunden	
		if(iPos<iCount && pString[iChar]>=0xC0 && (iFlags&0x0400))			
			{
			iLen		 = utf8tochar((char*)pString+iChar,uSign);
			iNum		 = chartoutf16(aWide,uSign);
															// Lnge des UTF8 Zeichnes holen
			GetTextExtentExPointW(hDc,aWide,iNum,0x8000,NULL,aArray+iPos,&sSize);

			iDiff		 = aArray[iPos];
			aArray[iPos] = iBase;
			aArray[iPos]|= iLen<<24;
			aArray[iPos]|= iBits;
			iBits		 = MASK_NULL;

			if(iDiff<iFontX)								// Ist der Buchstabe zu schmal
				{
				aArray[iPos] |= MASK_MIN;				
				}
			else if(iDiff>iFontX)							// Ist der Buchstabe zu breit
				{
				aArray[iPos] |= MASK_TRANS|MASK_MAX;				
				iBits		  = MASK_TRANS;
				}
			
			iBase       += iFontX;
			iChar       += iLen;
			iPos		+= 1;
			}
		}


	iMode		 = SetBkMode(hDc,OPAQUE);
	sArea.top	 = pRect->top;
	sArea.bottom = pRect->bottom;	
	aArray[iPos] = iBase;
	iLast		 = 0;
	iChar		 = 0;


	for(iPos=0;iPos<iCount;)								// Die Buchstaben zeichnen
		{
		iBits 	= aArray [iPos];
		wColor	= pColors[iPos];
		ColorChose(hDc,wColor&wMask,iFlags);

		
		if((iBits^iLast)&MASK_TRANS)						// Hat sich das Tranparent-Flag verndert
			{
			if(iBits&MASK_TRANS)
				{
				wPrev	 = wColor;
				iIdx	 = iPos;

				for(iOff=iIdx+1;iOff<=iCount;iOff++)		// Den tranparenten Hintergrund vorab ausmahlen
					{
					if(aArray[iOff]&MASK_TRANS)
					if(wPrev==pColors[iOff])
						{
						continue;					
						}

					sArea.left	= (short)aArray[iIdx];
					sArea.right	= (short)aArray[iOff];

					ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);

					wPrev		= pColors[iOff];
					ColorChose(hDc,wPrev&wMask,iFlags);
					}

				iMode = SetBkMode(hDc,TRANSPARENT);
				ColorChose(hDc,wColor&wMask,iFlags);
				}
			else{
				iMode = SetBkMode(hDc,OPAQUE);
				}
			}
	
		iLast		= iBits;
		iOut		= (short)aArray[iPos];
		iIdx		= iPos;


		if(iBits&(MASK_UTF8))								// Is es ein UTF8-Zeichen 	
			{
			iLen	= utf8tochar((char*)pString+iChar,uSign);
			iNum	= chartoutf16(aWide,uSign);

			if(iBits&MASK_MIN)
				{
				aWide[iNum] = '\t';
				iNum       += 1;
				}

			TabbedTextOutW(hDc,iOut,sArea.top,aWide,iNum,1,&iTabs,iOut-iTabs+iFontX);
			
			iNum = 1;
			}												
		else if(iBits&(MASK_MIN|MASK_MAX))					// ASCII-Zeichen mit faslscher breite		
			{
			aText[0]	= pString[iChar];

			if(iBits&MASK_MIN)
				{
				aText[1] = '\t';
				iNum     = 2;
				}
			else{
				iNum	 = 1;
				}

			TabbedTextOutA(hDc,iOut,sArea.top,aText,iNum,1,&iTabs,iOut-iTabs+iFontX);

			iLen = 1;			
			iNum = 1;
			}												// ASCII-Zeichen		
		else{												
			for(iOff=iIdx+1;iOff<iCount;iOff++)				// Zhle die Zeichen bis zum nchsten Wechsel
				{
				iNext = aArray[iOff];

				if(iNext&(MASK_MIN|MASK_MAX|MASK_UTF8))
					{
					break;
					}

				if((iNext^iBits)&(MASK_TRANS))
					{
					break;
					}

				if(wColor!=pColors[iOff])
					{
					break;					
					}
				}

			iNum = iOff-iIdx;				

			TabbedTextOutA(hDc,iOut,sArea.top,(char*)pString+iChar,iNum,1,&iTabs,iOrigin);
			
			iLen = iNum;
			}


		iChar   += iLen;
		iPos	+= iNum;

															// Den Text unterstreichen
		if((wColor&0x80) && (BYTE)wColor!=0xFF && wMask!=0xFFFF)			
			{
			sPen.left	= iOut;
			sPen.right	= (short)aArray[iPos];
			sPen.top	= pRect->top+((m_iFontSizeY*15)>>4);
			sPen.bottom	= sPen.top+1; 

			if(iBits&MASK_TRANS)SetBkMode(hDc,OPAQUE);

			uSign = SetBkColor(hDc,RGB(255,128,128));
			ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sPen,NULL,0,NULL);
			SetBkColor(hDc,uSign);

			if(iBits&MASK_TRANS)SetBkMode(hDc,TRANSPARENT);
			}
		}

	iBase = (short)aArray[iPos];


	if((iFlags&0x0100) && iBase<pRect->right)				// Ein extra Leerzeichen am Ende ausgeben
		{
		if(wColor!=pColors[iPos])
			{
			wColor = pColors[iPos];
			ColorChose(hDc,wColor&wMask,iFlags);
			}

		sArea.left	= iBase;
		sArea.right	= iBase+iFontX;

		ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);

		iBase += iFontX;
		iPos  += 1;
		}


	if((iFlags&0x0200) && iBase<pRect->right)				// Bis zum Ende das Rechteck ausmalen
		{
		if(wColor!=pColors[iPos])
			{
			wColor = pColors[iPos];
			ColorChose(hDc,wColor&wMask,iFlags);
			}
		
		sArea.left	= iBase;
		sArea.right	= pRect->right;
		ExtTextOutW(hDc,0,0,ETO_OPAQUE,&sArea,NULL,0,NULL);

		iBase = pRect->right;
		}


	SetBkMode(hDc,iMode);

	
	

return iBase-pRect->left;
}

