//*****************************************************************************
//*
//*
//*		DiskTools.cpp
//*
//*
//*****************************************************************************
#if	defined(_MFC_VER) || defined(AZ_LIB_ADDAFX)
	#if		_MSC_VER > 1299
	#ifndef		WINVER
	#define		WINVER 	0x0501
	#endif
	#endif
	#ifndef		__AFX_H__
	#include	<Afx.h>
	#endif
#else
#include	<windows.h>
#endif
#include	<string.h>

#include	"StringFunctions.h"
#include	"FileFunctions.h"
#include	"DiskTools.h"
#include	"DiskIO.h"


#define 	DPD_AUTO		0
#define 	DPD_ANSI		1
#define 	DPD_UTF8 		2
#define 	DPD_UTF16		3
#define 	DPD_MAX_LOAD	4

#define 	DPD_LOCK(p)
#define 	DPD_UNLOCK(p)

#define 	DPU_FIND_ONE							((DiskFind)0x12345678)
#define 	DPU_LIST_TO_HANDLE(t,s,i)				((DiskFile)((t)|((s)<<8)|((i)<<16)))
#define 	DPU_TYPE_OF_HANDLE(p)					((unsigned char)((unsigned)(p)    ))
#define 	DPU_SLOT_OF_HANDLE(p)					((unsigned char)((unsigned)(p)>> 8))
#define 	DPU_ITEM_OF_HANDLE(p)					((unsigned     )((unsigned)(p)>>16))

#define 	ENTRIES(a)								(sizeof(a)/sizeof((a)[0]))

#if			defined(DISK_USEUTF16)
#define 	DPD_BASE	DPD_UTF16
#elif		defined(DISK_USEUTF8)
#define 	DPD_BASE	DPD_UTF8
#else
#define 	DPD_BASE	DPD_ANSI
#endif

#ifndef		DISK_CACHE_CHECK
#define 	DISK_CACHE_CHECK	1
#elif		!defined(_DEBUG)
#undef		DISK_CACHE_CHECK
#define 	DISK_CACHE_CHECK	0
#endif


#pragma pack(1)

typedef struct
	{
	BYTE	bBootIndicator;       					// 0x80 for bootable, 0 otherwise
    BYTE	bStartHead;           					// 0 based
    BYTE	bStartSector;         					// 1 based, bits 0..5, bits 6,7 are bits 8,9 of StartTrack
    BYTE	bStartTrack;          					// 0 based, bits 0..7
    BYTE	bOSType;              					// FAT-12: 1, FAT-16: 4, 6, 14, FAT-32: 11, 12
    BYTE	bEndHead;             					// see StartHead
    BYTE	bEndSector;           					// see StartSector
    BYTE	bEndTrack;								// see StartTrack
	DWORD	dwRelativeSector;						// offset to first sector of partition data
    DWORD	dwSectors;								// number of sectors in partition
	}ParRec;

typedef struct
	{
	BYTE	bBootCode[440];
	DWORD	dwSerialNumber;
	WORD	wFree;
	ParRec	sTable[4];								// partition table
	WORD	wSignature;								// 0xAA55
	}MasterBootRecord;

typedef struct
	{
	BYTE	bNearJump[3];							// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;						// relative to partition start
	BYTE    bNumberOfFATs;
	WORD    wDirEntries;							// zero on FAT-32
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;
	WORD    wSectorsPerFAT;							// zero on FAT-32
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;				// OS specific, unreliable
	DWORD	dwTotalSectors;
	BYTE    bPhysicalDiskNumber;					// 0x80, 0x81, etc.
	BYTE    bCurrentHead;							// not used
	BYTE    bSignature;								// needed by NT
	DWORD   dwSerialNumber;
	char    bLabel[11];								// not used
	char    cSystemID[8];
	BYTE	bBootCode[448];
	WORD	wSignature;								// 0xAA55
	}BootRecord16;

typedef struct
	{
	BYTE	bNearJump[3];							// 0xEB=Jump8	0xE9=Jump16
	char    cOEMName[8];
	WORD    wBytesPerSector;
	BYTE    bSectorsPerCluster;
	WORD    wReservedSectors;						// relative to partition start
	BYTE    bNumberOfFATs;
	WORD    wDirEntries;							// zero on FAT-32
	WORD    wSectorsOnDisk;
	BYTE    bMediaDescriptor;
	WORD    wSectorsPerFAT;							// zero on FAT-32
	WORD    wSectorsPerTrack;
	WORD    wNumberOfHeads;
	DWORD	dwNumberOfHiddenSectors;				// OS specific, unreliable
	DWORD	dwTotalSectors;
	DWORD	dwSectorsPerFAT;
	WORD    wFlags;									// if (Flags & 0x0080) { SingleFAT; ActiveFat = (Flags & 0x000F); }
	WORD    wVersion;
	DWORD	dwRootDirCluster;
	WORD    wFSInfoSector;							// relative to start of partition, usually 1, 0xFFFF for none.
	WORD    wBackupBootSector;						// usually 6
	BYTE    bReserved[12];							// should be all zero
	BYTE    bPhysicalDiskNumber;					// 0x80, 0x81, etc.
	BYTE    bCurrentHead;							// not used
	BYTE    bSignature;								// needed by NT
	DWORD   dwSerialNumber;
	char    bLabel[11];								// not used
	char    cSystemID[8];
	BYTE	bBootCode[420];
	WORD	wSignature;								// 0xAA55
	}BootRecord32;

typedef struct
	{
	BYTE	aSignatur[8];							// "EFI PART"
	BYTE	aRevision[4];							// 00h 00h 01h 00h
	DWORD	dwHeaderSize;							// Header-Gre - Little Endian (5Ch 00h 00h 00h entspricht 92 bytes)
	DWORD	dwHeaderCrc;							// Header-CRC32-Prfsumme
	DWORD	dwReserved;								// Reservierter Bereich - muss Null (0) sein
	QWORD	qwLbaPosition;							// Position des eigenen LBA (dieses Headers)
	QWORD	qwBackupPosition;						// Position des Backup-LBA (des anderen Headers)
	QWORD	qwFirstLba;								// Erster benutzbarer LBA fr Partitionseintrge (normalerweise 34)
	QWORD	qwLastLba;								// Letzter benutzbarer LBA fr Partitionseintrge (normalerweise Datentrgergre - 34)
	GUID	sDiskGuid;								// Datentrger-GUID (als Referenz siehe auch UUID bei UNIXe)
	QWORD	qwPartitionTab;							// Start-LBA der Partitionstabelle (normalerweise 2)
	DWORD	dwUsedEntries;							// Anzahl der Partitionseintrge (Partitionen)
	DWORD	dwEntrySize;							// Gre eines Partitionseintrags (normalerweise 128)
	DWORD	dwPartitionCrc;							// Partitionstabellen-CRC32-Prfsumme
	BYTE	aUnused[420];							// Muss 0 sein
	}GuidPartitionTable;

typedef struct
	{
	GUID	sPartitionsTyp;							// Partitionstyp-GUID
	GUID	sPartitionsGuid;						// Einzigartige Partitions-GUID
	QWORD	qwStartLba;								// Beginn der Partition (erster LBA - Little-Endian)
	QWORD	qwEndLba;								// Ende der Partition (letzter LBA - inklusive)
	QWORD	qwAttributes;							// Attribute (siehe folgende Tabelle)
	WCHAR	aPartitionName[32];						// Partitionsname (UTF-16LE Code-Einheit)
	BYTE	aUnused[8];								// Unbenutzt
	}GuidPartitionEntry;

#pragma pack()

typedef struct
	{
	TCHAR	cFilter[8];								// Filter fr Laufwerke
	UINT	uSlot;									// Aktueller Slot
	}VolumeFind;

typedef struct
	{
	QWORD	qwDriveSize;							// Gesamtgre der Festplatte
	QWORD	qwStart[DPD_MAX_LOAD];					// Startsektor der Partition
	QWORD	qwSize [DPD_MAX_LOAD];					// Gre der Partition in Sektoren
	DWORD	dwEntry[DPD_MAX_LOAD];					// Position des MBR/GPT Eintrages
	}PartitionLoad;


#if		DPD_CACHE_SIZE>0

#ifndef	DPD_CACHE_LEVELS
#define DPD_CACHE_LEVELS	5
#endif

#if		DPD_CACHE_SIZE<0xFFFF
typedef	WORD	CPWORD;
#else
typedef	DWORD	CPWORD;
#endif

typedef struct
	{
	DWORD	uTurn;									// Aktuelle Zugriffsnummer
	DWORD	aHashStart[256];						// Starteintrag fr Hash (0=leer)
	QWORD	aSector   [DPD_CACHE_SIZE];				// Sektornummer
	DWORD	aCount    [DPD_CACHE_SIZE];				// Anzahl der Zugriffe (0=unbenutzt)
	DWORD	aLast     [DPD_CACHE_SIZE];				// Letzte Zugriffsnummer
	CPWORD	aNext     [DPD_CACHE_SIZE];				// Nchster Hash-Eintrag (0=Ende)
	CPWORD	aPrev	  [DPD_CACHE_SIZE];				// Vorheriger Hash-Eintrag (0=Ende)
	CPWORD	aLevelNext[DPD_CACHE_SIZE];				// Nchster Hash-Eintrag fr Levels (0=Ende)
	CPWORD	aLevelPrev[DPD_CACHE_SIZE];				// Vorheriger Hash-Eintrag fr Levels (0=Ende)
	CPWORD	aLevelLast[DPD_CACHE_LEVELS];			// Erster Hash-Eintrag fr Levels (0=Ende)
	CPWORD	aLevelFrom[DPD_CACHE_LEVELS];			// Letzter Hash-Eintrag fr Levels (0=Ende)
	BYTE	aData	  [DPD_CACHE_SIZE*512];			// Speicher fr Sektoren
	}DiskCache;

#endif

static const unsigned char aTypeToChar[] =
	{
	DPD_UTF16,										// DPD_TYPE_NONE		0
	DPD_UTF16,										// DPD_TYPE_PHYSICAL	1
	DPD_UTF8,										// DPD_TYPE_EMPTY		2
	DPD_UTF16,										// DPD_TYPE_FAT			3
	DPD_UTF8,										// DPD_TYPE_EXFAT		4
	DPD_UTF8,										// DPD_TYPE_EXT			5
	};

static int DiskFillName(const void *pSource,DCHAR *pBuffer,int iSize,int iMode,int *pSize=0);

//*****************************************************************************
//*
//*		ansitochar
//*
//*****************************************************************************
static int inline ansitochar(const char *pText,unsigned &uUnicode)
{

	uUnicode = *(unsigned char*)pText;

return 1;
}

//*****************************************************************************
//*
//*		ansitochar
//*
//*****************************************************************************
static int inline chartoansi(char *pText,unsigned &uUnicode)
{

	if(uUnicode<256)
			*pText=(char)uUnicode;
	else	*pText='\xFF';


return 1;
}

//*****************************************************************************
//*
//*		FAT Untersttzung
//*
//*****************************************************************************

#if		 	DPD_HAS_FAT
#define 	_VOLUMES			1
#define 	_USE_MKFS			1
#define 	_FS_USERPTR			1
#define     _FF_UNICODE			1
#define 	_FS_GETFNAME		1
#define 	_FS_USERDATA		1
#define 	_FS_USESETTIME		1
#define 	_FS_USECRTTIME		1
#define 	FS_STATIC			static
#define 	FS_API				extern "C"
#define 	FS_POINTER			->user_ptr
#define 	FS_PARAM(p,a)		((FATDATA*)p)->user_data
#define 	GET_SECTOR_COUNT	CTRL_SECTOR_COUNT
#define 	FAT_DOS13			((sizeof(((FILINFO*)0)->fname[0])==1)? DPD_ANSI:DPD_UTF16)
#include	"../FileSystem/Fat/ff.c"

#ifndef 	FAT_LOCAL_TIME
#define 	FAT_LOCAL_TIME		1
#endif
#ifndef 	FAT_MAX_FIND
#define 	FAT_MAX_FIND		DISK_MAX_FIND
#endif
#ifndef 	FAT_MAX_FILE
#define 	FAT_MAX_FILE		DISK_MAX_FILES
#endif
#ifndef 	FR_PARAM
#define 	FR_PARAM			((FRESULT)(FR_TOO_MANY_OPEN_FILES+1))
#endif

#define 	YEAR_DAYS			(365)					// days in   1 year
#define		YEAR_DAYS_4			(YEAR_DAYS*4+1)			// days in   4 years
#define		YEAR_DAYS_100		(YEAR_DAYS_4*25-1)		// days in 100 years
#define		YEAR_DAYS_400		(YEAR_DAYS_100*4+1)		// days in 400 years

static const unsigned short	aTpPDays [] = { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366, 366, 366,   0};
static const unsigned short	aTpDays  [] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365, 365, 365,   0};

typedef	struct
	{
	unsigned short			usYear;					// Year (ie. 2006)
	unsigned char			ucMonth;				// Mounth (1..12)
	unsigned char			ucDay;					// Day (1..31)
	unsigned char			ucHour;					// Hour (0..24)
	unsigned char			ucMinute;				// Minute (0..59)
	unsigned char			ucSecond;				// Secound (0..59)
	unsigned char			ucIsDst;				// Daylight saving in minutes
	}FatTimeStruct;

typedef struct
	{
	DIR		sData;
	wchar_t	aName[256];
	}DIRFIND;

typedef struct
	{
	FATDATA	sFatData;								// FAT-Laufwerksdaten
	FATFS	sFatFs;									// FAT lokale Daten
	UINT	uNextFile;								// Nchstes Datei-Handle
	UINT	uNextFind;								// Nchstes Find-Handle
	FIL    *aFileList[FAT_MAX_FILE];				// Liste mit den Datei-Handles
	DIRFIND*aFindList[FAT_MAX_FIND];				// Liste mit den Find-Handles
	}FatEntry;


#if 		FAT_LOCAL_TIME
#include 	"System/TimeUtility.h"
int			iTimeZone	= -3600;
#endif


static DiskFile FatFileAlloc(void *pLocal,unsigned uSlot)
	{
	FatEntry   *pEntry=(FatEntry*)pLocal;
	unsigned	uTurn;
	unsigned	uItem;

	uItem = pEntry->uNextFile;

	for(uTurn=0;;uTurn++)
		{
		if(uTurn>=FAT_MAX_FILE)return 0;
		if(uItem>=FAT_MAX_FILE)uItem =0;
		if(pEntry->aFileList[uItem]==0)break;
		}

	pEntry->uNextFile        = uItem+1;
	pEntry->aFileList[uItem] = new FIL;

	return DPU_LIST_TO_HANDLE(DPD_TYPE_FAT,uSlot,uItem);
	}

static void FatFileFree(void *pLocal,DiskFile pHandle)
	{
	FatEntry   *pEntry=(FatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle);
	if(uItem>=FAT_MAX_FILE)return;

	delete pEntry->aFileList[uItem];
	pEntry->aFileList[uItem] = 0;
	}

static inline FIL *FatFileGet(void *pLocal,void *pHandle)
	{
	FatEntry   *pEntry=(FatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle);
	if(uItem>=FAT_MAX_FILE)return 0;

	return pEntry->aFileList[uItem];
	}

static DiskFind FatFindAlloc(void *pLocal,unsigned uSlot)
	{
	FatEntry   *pEntry=(FatEntry*)pLocal;
	unsigned	uTurn;
	unsigned	uItem;

	uItem = pEntry->uNextFind;

	for(uTurn=0;;uTurn++)
		{
		if(uTurn>=FAT_MAX_FIND)return 0;
		if(uItem>=FAT_MAX_FIND)uItem =0;
		if(pEntry->aFindList[uItem]==0)break;
		}

	pEntry->uNextFind        = uItem+1;
	pEntry->aFindList[uItem] = new DIRFIND;

	return (DiskFind)DPU_LIST_TO_HANDLE(DPD_TYPE_FAT,uSlot,uItem+0x1000);
	}

static void FatFindFree(void *pLocal,void *pHandle)
	{
	FatEntry   *pEntry=(FatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle)-0x1000;
	if(uItem>=FAT_MAX_FIND)return;

	delete pEntry->aFindList[uItem];
	pEntry->aFindList[uItem] = 0;
	}

static inline DIRFIND *FatFindGet(void *pLocal,void *pHandle)
	{
	FatEntry   *pEntry=(FatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle)-0x1000;
	if(uItem>=FAT_MAX_FIND)return 0;

	return pEntry->aFindList[uItem];
	}

static int FatError(FRESULT	iError)
	{
	switch(iError)
		{
	case FR_NO_FILE:			return DPD_ERR_FILE_NOT_FOUND;
	case FR_NO_PATH:            return DPD_ERR_PATH_NOT_FOUND;
	case FR_INVALID_NAME:       return DPD_ERR_INVALID_FILENAME;
	case FR_DENIED:             return DPD_ERR_ACCESS_DENIED;
	case FR_EXIST:              return DPD_ERR_ALREADY_EXISTS;
	case FR_WRITE_PROTECTED:    return DPD_ERR_ACCESS_DENIED;
	case FR_INVALID_DRIVE:      return DPD_ERR_PATH_NOT_FOUND;
	case FR_TOO_MANY_OPEN_FILES:return DPD_ERR_NO_HANDLES;
	case FR_PARAM:				return DPD_ERR_PARAM;
		}

	return DPD_ERR_UNKNOWN;
	}

static int FatConvertAttr(BYTE ucAttr)
	{
	int		iAttrib;

	iAttrib = DPD_READ|DPD_WRITE;

	if(ucAttr&AM_RDO)iAttrib&=~DPD_WRITE;
	if(ucAttr&AM_DIR)iAttrib|= DPD_DIR;
	if(ucAttr&AM_HID)iAttrib|= DPD_HIDDEN;
	if(ucAttr&AM_ARC)iAttrib|= DPD_ARCHIVE;

	return iAttrib;
	}

static unsigned FatWeekday(unsigned uDay,unsigned uMonth,unsigned uYear)
	{
	static const unsigned char	ucYearDays[16] = {0%7,306%7,337%7,0%7,31%7,61%7,92%7,122%7,153%7,184%7,214%7,245%7,275%7,365%7,365%7,365%7};
	unsigned					uVal;

	uMonth   = uMonth&0x0F;
	uDay    &= 0x1F;

	if(uMonth<3)uYear--;

	uVal     =  1;										// 1.March.0000 = Mo
	uVal	+= (uYear% 4)*YEAR_DAYS;	uYear/=4;
	uVal	+= (uYear%25)*YEAR_DAYS_4;	uYear/=25;
	uVal	+= (uYear% 4)*YEAR_DAYS_100;uYear/=4;
	uVal	+= (uYear   )*YEAR_DAYS_400;
	uVal    += ucYearDays[uMonth];
	uVal	+= uDay;
	uVal	%= 7;


	return	uVal+1;
	}

static unsigned FatIsDaylightSaving(const FatTimeStruct *pData)
	{
	unsigned	uVal;


	if(pData->ucMonth<=3)								// check begin
        {
        if(pData->ucMonth<3 || pData->ucDay<25)
			{
			return  0;
			}

        uVal = FatWeekday(25,3,pData->usYear);
        uVal = (7-uVal)+25;

        if(pData->ucDay>uVal)return 3600;
        if(pData->ucDay<uVal)return 0;

		uVal = (pData->ucIsDst)? 1:2;

        if(pData->ucHour<uVal)return 0;

		return 3600;
        }

	if(pData->ucMonth<=10)								// check end
        {
        if(pData->ucMonth<10 || pData->ucDay<25)
			{
			return 3600;
			}

        uVal=FatWeekday(25,10,pData->usYear);
        uVal=(7-uVal)+25;

        if(pData->ucDay>uVal)return 0;
        if(pData->ucDay<uVal)return 3600;

		uVal = (pData->ucIsDst)? 1:2;

        if(pData->ucHour<uVal)return 3600;

		return 0;
        }


	return 0;
	}

static DWORD FatDosToTime(unsigned short wDate,unsigned short wTime)
	{
	unsigned 	uTime;
	unsigned 	uYear;
	unsigned 	uDays;


	   uYear = (wDate>>9)+80;
	if(uYear&3)uDays  = aTpDays [((wDate>>5)-1)&0x0F];
	else	   uDays  = aTpPDays[((wDate>>5)-1)&0x0F];
			   uDays += ((wDate)&0x1F)-1;
			   uDays += (uYear-70  )*365;
			   uDays += (uYear-70+1)>>2;

			   uTime  = uDays*24;
			   uTime += (wTime>>11);
			   uTime *= 60;
			   uTime += (wTime>> 5)&0x3F;
			   uTime *= 60;
			   uTime += (wTime<< 1)&0x3F;

	#if FAT_LOCAL_TIME
	{
	FatTimeStruct 	sData;                              // FAT uses the local time

	sData.usYear	=  uYear+1900;
	sData.ucMonth	= (wDate>> 5)&0x0F;
	sData.ucDay		= (wDate    )&0x1F;
	sData.ucHour	= (wTime>>11);
	sData.ucIsDst	= 0;

	uTime 		   -= FatIsDaylightSaving(&sData);		// local time to global
	uTime          += iTimeZone;
	}
	#endif

	return uTime;
	}

static DWORD FatTimeToDos(DiskUtc uSysTime)
	{
	struct tm      *pTime;
	WORD			wTime;
	WORD			wDate;

	#if FAT_LOCAL_TIME
	pTime = localtime((const time_t*)&uSysTime);
	#else
	pTime = gmtime((const time_t*)&uSysTime);
	#endif

	if(pTime==NULL || pTime->tm_year<80 || pTime->tm_year>143)
		{
		return 0;
		}

	wTime = (pTime->tm_sec>>1) | ( pTime->tm_min   <<5) | ( pTime->tm_hour    <<11);
	wDate = (pTime->tm_mday  ) | ((pTime->tm_mon+1)<<5) | ((pTime->tm_year-80)<< 9);

	return (wTime)|(wDate<<16);
	}


#if			DPD_SYNC		!=CTRL_SYNC
#error		"wrong define for DPD_SYNC"
#endif
#if			DPD_CTRL_SECTOR_COUNT!=CTRL_SECTOR_COUNT
#error		"wrong define for DPD_CTRL_SECTOR_COUNT"
#endif
#if			DPD_CTRL_SECTOR_SIZE !=CTRL_SECTOR_SIZE
#error		"wrong define for DPD_CTRL_SECTOR_SIZE"
#endif
#if			DPD_CTRL_BLOCK_SIZE	!=CTRL_BLOCK_SIZE
#error		"wrong define for DPD_CTRL_BLOCK_SIZE"
#endif
#if			DPD_CTRL_POWER		!=CTRL_POWER
#error		"wrong define for DPD_CTRL_POWER"
#endif
#if			DPD_CTRL_LOCK		!=CTRL_LOCK
#error		"wrong define for DPD_CTRL_LOCK"
#endif
#if			DPD_CTRL_EJECT		!=CTRL_EJECT
#error		"wrong define for DPD_CTRL_EJECT"
#endif
#if			DPD_CTRL_SECTOR_CNT64!=CTRL_SECTOR_CNT64
#error		"wrong define for DPD_CTRL_SECTOR_CNT64"
#endif
#if			DPD_NOINIT			!=STA_NOINIT
#error		"wrong define for DPD_NOINIT"
#endif
#if			DPD_NODISK			!=STA_NODISK
#error		"wrong define for DPD_NODISK"
#endif
#if			DPD_PROTECT 		!=STA_PROTECT
#error		"wrong define for DPD_PROTECT"
#endif


#endif

//*****************************************************************************
//*
//*		EXFAT Untersttzung
//*
//*****************************************************************************

#if		 	DPD_HAS_EXFAT
#ifndef 	EXFAT_LOCAL_TIME
#define 	EXFAT_LOCAL_TIME	1
#endif
#ifndef 	EXFAT_LOGGING
#define 	EXFAT_LOGGING		0
#endif
#ifndef 	EXFAT_MAX_FIND
#define 	EXFAT_MAX_FIND		DISK_MAX_FIND
#endif
#ifndef 	EXFAT_MAX_FILE
#define 	EXFAT_MAX_FILE		DISK_MAX_FILES
#endif
#define 	EXFAT_STATIC		static

#include	"../FileSystem/ExFat/exfat.h"
#include	<errno.h>
#include	<sys/stat.h>
#include	<sys/types.h>

extern "C"
{
#include	"../FileSystem/ExFat/exfat_cluster.c"
#include	"../FileSystem/ExFat/exfat_io.c"
#include	"../FileSystem/ExFat/exfat_log.c"
#include	"../FileSystem/ExFat/exfat_lookup.c"
#include	"../FileSystem/ExFat/exfat_mount.c"
#include	"../FileSystem/ExFat/exfat_node.c"
#include	"../FileSystem/ExFat/exfat_time.c"
#include	"../FileSystem/ExFat/exfat_utf.c"
#include	"../FileSystem/ExFat/exfat_utils.c"
}

struct exfat_dev
	{
	DiskPartionData	   *pData;						// Zeiger auf Paritionsdaten
	int					iSlot;						// Nummer des Laufwerks
	int					iWrite;						// Ist der Cache verndert
	uint64_t			uSect;						// Sektornummer vom Cache
	unsigned char		aBuffer[512];				// Cache Puffer
	};

typedef struct
	{
	exfat_node		   *pNode;						// Aktuelle Node der Suche
	exfat_iterator		sNext;						// Iterator fr Suche
	wchar_t				aName[256];					// Filter fr Suche
	}ExFatFind;

typedef struct
	{
	exfat				sExFat;						// EXFAT Daten
	UINT				uOpenCnt;					// Wieviele Dateien wurden geffnet
	UINT				uFolderCnt;					// Wieviele Verzeichnisse wurden geffnet
	UINT				uNextFile;					// Nchstes Datei-Handle
	UINT				uNextFind;					// Nchstes Find-Handle
	exfat_node		   *aFileList[EXFAT_MAX_FILE];	// Liste mit den Datei-Handles
	ExFatFind 		   *aFindList[EXFAT_MAX_FIND];	// Liste mit den Find-Handles
	int64_t				aFilePos [EXFAT_MAX_FILE];	// Liste mit Dateizeigern
	char				aFileFlag[EXFAT_MAX_FILE];	// Liste mit Zugriffsflags
	}ExFatEntry;

static int ExFatError(int iError)
	{
	switch(iError)
		{
	case -EISDIR:				return DPD_ERR_FILE_NOT_FOUND;
	case -ENOENT:				return DPD_ERR_FILE_NOT_FOUND;
	case -ENOTDIR:              return DPD_ERR_PATH_NOT_FOUND;
	case -ENAMETOOLONG:			return DPD_ERR_INVALID_FILENAME;
	case -EILSEQ:				return DPD_ERR_INVALID_FILENAME;
	case -EACCES:				return DPD_ERR_ACCESS_DENIED;
	case -EPERM:				return DPD_ERR_ACCESS_DENIED;
	case -EIO:					return DPD_ERR_ACCESS_DENIED;
	case -EEXIST:               return DPD_ERR_ALREADY_EXISTS;
	case -ENOSPC:				return DPD_ERR_DISKFULL;
		}

	return DPD_ERR_UNKNOWN;
	}

static int ExFatConvertAttrib(const exfat_node *pNode)
	{
	int		iFlags;
	int		iRet;

	iFlags = pNode->flags;
	iRet   = (iFlags&EXFAT_ATTRIB_RO)? DPD_READ:DPD_READ|DPD_WRITE;

	if(iFlags&EXFAT_ATTRIB_DIR   )iRet|=DPD_DIR;
	if(iFlags&EXFAT_ATTRIB_HIDDEN)iRet|=DPD_HIDDEN;
	if(iFlags&EXFAT_ATTRIB_SYSTEM)iRet|=DPD_SYSTEM;
	if(iFlags&EXFAT_ATTRIB_VOLUME)iRet|=DPD_VOLUME;
	if(iFlags&EXFAT_ATTRIB_ARCH  )iRet|=DPD_ARCHIVE;

	return iRet;
	}

static DiskFile ExFatFileAlloc(void *pLocal,unsigned uSlot,exfat_node *pNode,int iMode)
	{
	ExFatEntry *pEntry=(ExFatEntry*)pLocal;
	unsigned	uTurn;
	unsigned	uItem;

	uItem = pEntry->uNextFile;

	for(uTurn=0;;uTurn++)
		{
		if(uTurn>=EXFAT_MAX_FILE)return 0;
		if(uItem>=EXFAT_MAX_FILE)uItem =0;
		if(pEntry->aFileList[uItem]==0)break;
		}

	pEntry->uNextFile        = uItem+1;
	pEntry->aFileList[uItem] = pNode;
	pEntry->aFileFlag[uItem] = iMode;
	pEntry->aFilePos [uItem] = 0;

	return DPU_LIST_TO_HANDLE(DPD_TYPE_EXFAT,uSlot,uItem);
	}

static void ExFatFileFree(void *pLocal,DiskFile pHandle)
	{
	ExFatEntry *pEntry=(ExFatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle);
	if(uItem>=EXFAT_MAX_FILE)return;

	pEntry->aFileList[uItem] = 0;
	}

static inline exfat_node *ExFatFileGet(void *pLocal,void *pHandle)
	{
	ExFatEntry *pEntry=(ExFatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle);
	if(uItem>=EXFAT_MAX_FILE)return 0;

	return pEntry->aFileList[uItem];
	}

static DiskFind ExFatFindAlloc(void *pLocal,unsigned uSlot,exfat_node *pNode)
	{
	ExFatEntry *pEntry=(ExFatEntry*)pLocal;
	ExFatFind  *pItem;
	unsigned	uTurn;
	unsigned	uItem;

	uItem = pEntry->uNextFind;

	for(uTurn=0;;uTurn++)
		{
		if(uTurn>=EXFAT_MAX_FIND)return 0;
		if(uItem>=EXFAT_MAX_FIND)uItem =0;
		if(pEntry->aFindList[uItem]==0)break;
		}

	pItem					 = new ExFatFind;
	pItem->pNode			 = pNode;
	pEntry->uNextFind		 = uItem+1;
	pEntry->aFindList[uItem] = pItem;

	return (DiskFind)DPU_LIST_TO_HANDLE(DPD_TYPE_EXFAT,uSlot,uItem+0x1000);
	}

static void ExFatFindFree(void *pLocal,void *pHandle)
	{
	ExFatEntry *pEntry=(ExFatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle)-0x1000;
	if(uItem>=EXFAT_MAX_FIND)return;

	delete pEntry->aFindList[uItem];
	pEntry->aFindList[uItem] = 0;
	}

static inline ExFatFind *ExFatFindGet(void *pLocal,void *pHandle)
	{
	ExFatEntry *pEntry=(ExFatEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle)-0x1000;
	if(uItem>=EXFAT_MAX_FIND)return 0;

	return pEntry->aFindList[uItem];
	}

static void ExFatNoteToInfo(exfat_node *pNode,DiskFindStruct *pFind)
	{
	DiskFillName(pNode->name,pFind->cName,ENTRIES(pFind->cName),DPD_UTF16);

	pFind->dwTimeAccess = (DiskUtc)pNode->atime;
	pFind->dwTimeCreate = (DiskUtc)pNode->ctime;
	pFind->dwTimeWrite  = (DiskUtc)pNode->mtime;
	pFind->dwSize       = (DWORD  )pNode->size;
	pFind->dwAttr		= ExFatConvertAttrib(pNode);
	pFind->dwAttr	   |= (DWORD )(pNode->size>>(32-DPD_HIGHSHIFT))&DPD_HIGHMASK;

	#if DISK_HAS_SHORT_NAME
	#error not supported
	#endif
	}

extern "C" struct exfat_dev *exfat_open(const char *pSpec,enum exfat_mode iMode,int iSlot)
	{
	DiskPartionData    *pData=(DiskPartionData*)pSpec;
	struct exfat_dev   *pDevice;
	DWORD				dwRdOnly;

	if(!pData)return NULL;

	pDevice	= new exfat_dev;
	memset(pDevice,0,sizeof(exfat_dev));

	pDevice->iSlot	= iSlot;
	pDevice->pData	= pData;
	pDevice->uSect	= 0xFFFFFFFFFFFFFFFF;


	if(DiskIoctlProc(pDevice->pData,pDevice->iSlot,DPD_CTRL_READONLY,&dwRdOnly)!=DPD_OK)
		{
		return NULL;
		}


	switch(iMode)
		{
	case EXFAT_MODE_RW:

		if(dwRdOnly)return NULL;

	case EXFAT_MODE_RO:
	case EXFAT_MODE_ANY:

		break;
		}

	return pDevice;
	}

extern "C" int exfat_close(struct exfat_dev *pDevice)
	{
	if(pDevice==NULL)return -EXDEV;

	if(pDevice->iWrite)
		{
		if(DiskWriteProc(pDevice->pData,pDevice->iSlot,pDevice->aBuffer,pDevice->uSect,1)==DPD_OK)
			{
			pDevice->iWrite = 0;
			}
		}

	memset(pDevice,0,sizeof(exfat_dev));
	delete pDevice;

	return 0;
	}

extern "C" int exfat_fsync(struct exfat_dev *pDevice)
	{

	if(pDevice->iWrite)
		{
		if(DiskWriteProc(pDevice->pData,pDevice->iSlot,pDevice->aBuffer,pDevice->uSect,1)==DPD_OK)
			{
			pDevice->iWrite = 0;
			}
		}

	if(DiskIoctlProc(pDevice->pData,pDevice->iSlot,DPD_CTRL_SYNC,NULL)!=DPD_OK)
		{
		return -EIO;
		}

	return 0;
	}

extern "C" int exfat_clear(struct exfat_dev *pDevice)
	{
	if(!pDevice->iWrite)return 0;

	if(DiskWriteProc(pDevice->pData,pDevice->iSlot,pDevice->aBuffer,pDevice->uSect,1)==DPD_OK)
		{
		pDevice->iWrite = 0;
		return 0;
		}

	return -EIO;
	}

extern "C" enum exfat_mode exfat_get_mode(const struct exfat_dev *pDevice)
	{
	DWORD		dwRdOnly;

	if(DiskIoctlProc(pDevice->pData,pDevice->iSlot,DPD_CTRL_READONLY,&dwRdOnly)!=DPD_OK)
		{
		return EXFAT_MODE_RO;
		}

	return (dwRdOnly)? EXFAT_MODE_RO:EXFAT_MODE_RW;
	}

extern "C" exfat_off_t exfat_get_size(const struct exfat_dev *pDevice)
	{
	uint64_t	uSize;

	if(DiskIoctlProc(pDevice->pData,pDevice->iSlot,DPD_CTRL_SECTOR_CNT64,&uSize)!=DPD_OK)
		{
		return 0;
		}

	return uSize<<9;
	}

extern "C" ssize_t exfat_pread(struct exfat_dev *pDevice,void *pBuffer,size_t uSize,exfat_off_t uOffset)
	{
	unsigned		uBytes;
	unsigned		uCount;
	unsigned		uBase;
	unsigned		uDone;
	uint64_t		uSect;
	uint64_t		uEnd;

	if((uSize&0x1FF)==0 && (uOffset&0x1FF)==0)			// Ganze Sektoren lesen
		{
		uCount	= uSize  >>9;
		uSect	= uOffset>>9;
		uEnd	= uSect+(uint64_t)uCount;

		if(pDevice->uSect==uSect && uCount==1)
			{
			memcpy(pBuffer,pDevice->aBuffer,512);
			return 512;
			}

		if(pDevice->iWrite && pDevice->uSect>=uSect && pDevice->uSect<uEnd)
			{
			if(exfat_clear(pDevice))return -EIO;
			}

		if(DiskReadProc(pDevice->pData,pDevice->iSlot,(unsigned char*)pBuffer,uSect,uCount)!=DPD_OK)
			{
			return -EIO;
			}

		return uSize;
		}


	uBase	= (unsigned)uOffset&0x1FF;
	uSect	= uOffset>>9;
	uDone	= 0;

	if(uBase || uSize<=512)								// Erster Sektor nicht ganz
		{
		   uBytes = 512-uBase;
		if(uBytes>uSize)uBytes=uSize;


		if(pDevice->uSect!=uSect)
			{
			if(pDevice->iWrite)
				{
				if(exfat_clear(pDevice))return -EIO;
				}

			if(DiskReadProc(pDevice->pData,pDevice->iSlot,pDevice->aBuffer,uSect,1)!=DPD_OK)
				{
				return -EIO;
				}

			pDevice->uSect = uSect;
			}

		memcpy(pBuffer,pDevice->aBuffer+uBase,uBytes);

		((uint64_t*)&pBuffer)[0] += uBytes;
		uOffset					 += uBytes;
		uDone					 += uBytes;
		uSize					 -= uBytes;

		if(uSize==0)return uDone;
		}

	if(uSize>=512)										// Ganze Sektoren in der Mitte
		{
		uCount	= uSize  >>9;
		uBytes  = uCount <<9;
		uSect	= uOffset>>9;
		uEnd	= uSect+(uint64_t)uCount;

		if(pDevice->iWrite && pDevice->uSect>=uSect && pDevice->uSect<uEnd)
			{
			if(exfat_clear(pDevice))return -EIO;
			}

		if(DiskReadProc(pDevice->pData,pDevice->iSlot,(unsigned char*)pBuffer,uSect,uCount)!=DPD_OK)
			{
			return -EIO;
			}

		((uint64_t*)&pBuffer)[0] += uBytes;
		uOffset					 += uBytes;
		uDone					 += uBytes;
		uSize					 -= uBytes;
		}


	if(uSize)											// Letzter Sektor nicht ganz
		{
		uBytes	= uSize;
		uSect	= uOffset>>9;

		if(pDevice->uSect!=uSect)
			{
			if(pDevice->iWrite)
				{
				if(exfat_clear(pDevice))return -EIO;
				}

			if(DiskReadProc(pDevice->pData,pDevice->iSlot,pDevice->aBuffer,uSect,1)!=DPD_OK)
				{
				return -EIO;
				}

			pDevice->uSect = uSect;
			}

		memcpy(pBuffer,pDevice->aBuffer,uBytes);

		((uint64_t*)&pBuffer)[0] += uBytes;
		uOffset					 += uBytes;
		uDone					 += uBytes;
		uSize					 -= uBytes;
		}

	return uDone;
	}

extern "C" ssize_t exfat_pwrite(struct exfat_dev *pDevice,const void* pBuffer,size_t uSize,exfat_off_t uOffset)
	{
	unsigned		uBytes;
	unsigned		uCount;
	unsigned		uBase;
	unsigned		uDone;
	uint64_t		uSect;
	uint64_t		uDiff;
	uint64_t		uEnd;

	if((uSize&0x1FF)==0 && (uOffset&0x1FF)==0)			// Ganze Sektoren schreiben
		{
		uCount	= uSize  >>9;
		uSect	= uOffset>>9;
		uEnd	= uSect+(uint64_t)uCount;


		if(pDevice->uSect>=uSect && pDevice->uSect<uEnd)
			{
			uDiff = uSect-pDevice->uSect;
			memcpy(pDevice->aBuffer,(char*)pBuffer+uDiff,512);
			pDevice->iWrite = 0;
			}

		if(DiskWriteProc(pDevice->pData,pDevice->iSlot,(unsigned char*)pBuffer,uSect,uCount)!=DPD_OK)
			{
			return -EIO;
			}

		return uSize;
		}

	uBase	= (unsigned)uOffset&0x1FF;
	uSect	= uOffset>>9;
	uDone	= 0;

	if(uBase || uSize<=512)								// Erster Sektor nicht ganz
		{
		   uBytes = 512-uBase;
		if(uBytes>uSize)uBytes = uSize;

		if(pDevice->uSect!=uSect)
			{
			if(pDevice->iWrite)
				{
				if(exfat_clear(pDevice))return -EIO;
				}

			if(DiskReadProc(pDevice->pData,pDevice->iSlot,pDevice->aBuffer,uSect,1)!=DPD_OK)
				{
				return -EIO;
				}

			pDevice->uSect = uSect;
			}

		pDevice->iWrite = 1;
		memcpy(pDevice->aBuffer+uBase,pBuffer,uBytes);

		((uint64_t*)&pBuffer)[0] += uBytes;
		uOffset					 += uBytes;
		uDone					 += uBytes;
		uSize					 -= uBytes;

		if(uSize==0)return uDone;
		}

	if(uSize>=512)										// Ganze Sektoren in der Mitte
		{
		uCount	= uSize  >>9;
		uBytes  = uCount <<9;
		uSect	= uOffset>>9;
		uEnd	= uSect+(uint64_t)uCount;

		if(pDevice->uSect>=uSect && pDevice->uSect<uEnd)
			{
			uDiff = uSect-pDevice->uSect;
			memcpy(pDevice->aBuffer,(char*)pBuffer+uDiff,512);
			pDevice->iWrite = 0;
			}

		if(DiskWriteProc(pDevice->pData,pDevice->iSlot,(unsigned char*)pBuffer,uSect,uCount)!=DPD_OK)
			{
			return -EIO;
			}

		((uint64_t*)&pBuffer)[0] += uBytes;
		uOffset					 += uBytes;
		uDone					 += uBytes;
		uSize					 -= uBytes;
		}

	if(uSize)											// Letzter Sektor nicht ganz
		{
		uBytes	= uSize;
		uSect	= uOffset>>9;

		if(pDevice->uSect!=uSect)
			{
			if(pDevice->iWrite)
				{
				if(exfat_clear(pDevice))return -EIO;
				}

			if(DiskReadProc(pDevice->pData,pDevice->iSlot,pDevice->aBuffer,uSect,1)!=DPD_OK)
				{
				return -EIO;
				}

			pDevice->uSect = uSect;
			}

		pDevice->iWrite = 1;
		memcpy(pDevice->aBuffer,pBuffer,uBytes);

		((uint64_t*)&pBuffer)[0] += uBytes;
		uOffset					 += uBytes;
		uDone					 += uBytes;
		uSize					 -= uBytes;
		}

	return uDone;
	}

#endif

//*****************************************************************************
//*
//*		EXT2..4 Untersttzung
//*
//*****************************************************************************

#if		 	DPD_HAS_EXT

#define 	CONFIG_DEBUG_PRINTF				0
#define		CONFIG_EXT4_BLOCKDEVS_COUNT		0
#define		CONFIG_EXT4_MOUNTPOINTS_COUNT	0
#define 	CONFIG_EXT4_STATIC				extern "C"  static
#define 	CONFIG_EXT4_INLINE				EXT4_INLINE
#define		CONFIG_EXT4_CHECK_ACCESSS		1
#define 	CONFIG_EXT4_NO_RECURSIV			1
#define 	CONFIG_EXT4_HAS_NAMEBUF			1
#define 	CONFIG_EXT4_SET_TIMES			1
#define 	CONFIG_EXT4_HAS_FTYPE			1

#ifndef 	EXT_MAX_FIND
#define 	EXT_MAX_FIND		DISK_MAX_FIND
#endif
#ifndef 	EXT_MAX_FILE
#define 	EXT_MAX_FILE		DISK_MAX_FILES
#endif

#include	<errno.h>
#include	"../FileSystem/Ext4/ext4.h"
#include	"../FileSystem/Ext4/ext4_inode.c"
#include	"../FileSystem/Ext4/ext4_block_group.c"
#include	"../FileSystem/Ext4/ext4_blockdev.c"
#include	"../FileSystem/Ext4/ext4_crc32.c"
#include	"../FileSystem/Ext4/ext4_dir.c"
#include	"../FileSystem/Ext4/ext4_dir_idx.c"
#include	"../FileSystem/Ext4/ext4_extent.c"
#include	"../FileSystem/Ext4/ext4_fs.c"
#include	"../FileSystem/Ext4/ext4_hash.c"
#include	"../FileSystem/Ext4/ext4_ialloc.c"
#include	"../FileSystem/Ext4/ext4_journal.c"
#include	"../FileSystem/Ext4/ext4_super.c"
#include	"../FileSystem/Ext4/ext4_trans.c"
#include	"../FileSystem/Ext4/ext4_xattr.c"
#include	"../FileSystem/Ext4/ext4_balloc.c"
#include	"../FileSystem/Ext4/ext4_bcache.c"
#include	"../FileSystem/Ext4/ext4_bitmap.c"
#include	"../FileSystem/Ext4/ext4.c"


typedef struct
	{
	ext4_dir			sData;							// Verzeichnissdaten
	int					iPathLen;						// Lnge des Pfades
	char				cName[256];						// Filtername
	char				cPath[DPD_MAX_PATH];			// Basispfad
	}ext4_find;

typedef struct
	{
	LPVOID				hHandle;						// EXT4 Handle
	DiskPartionData	   *pData;							// Partitionsdaten
	UINT				uSlot;							// Slotnummer der Partition
	BYTE				aBuffer[512];					// Puffer fr Sektordaten
	ext4_blockdev_iface	sInterface;						// Disk-Block-Interface
	ext4_blockdev		sDevice;						// Disk-Block-Gert
	UINT				uNextFile;						// Nchstes Datei-Handle
	UINT				uNextFind;						// Nchstes Find-Handle
	ext4_file          *aFileList[EXT_MAX_FILE];		// Liste mit den Datei-Handles
	ext4_find          *aFindList[EXT_MAX_FILE];		// Liste mit den Find-Handles
	}ExtEntry;

static DiskFile ExtFileAlloc(void *pLocal,unsigned uSlot)
	{
	ExtEntry   *pEntry=(ExtEntry*)pLocal;
	unsigned	uTurn;
	unsigned	uItem;

	uItem = pEntry->uNextFile;

	for(uTurn=0;;uTurn++)
		{
		if(uTurn>=EXT_MAX_FILE)return 0;
		if(uItem>=EXT_MAX_FILE)uItem =0;
		if(pEntry->aFileList[uItem]==0)break;
		}

	pEntry->uNextFile        = uItem+1;
	pEntry->aFileList[uItem] = new ext4_file;

	return DPU_LIST_TO_HANDLE(DPD_TYPE_EXT,uSlot,uItem);
	}

static void ExtFileFree(void *pLocal,DiskFile pHandle)
	{
	ExtEntry   *pEntry=(ExtEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle);
	if(uItem>=EXT_MAX_FILE)return;

	delete pEntry->aFileList[uItem];
	pEntry->aFileList[uItem] = 0;
	}

static inline ext4_file *ExtFileGet(void *pLocal,void *pHandle)
	{
	ExtEntry   *pEntry=(ExtEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle);
	if(uItem>=EXT_MAX_FILE)return 0;

	return pEntry->aFileList[uItem];
	}

static DiskFind ExtFindAlloc(void *pLocal,unsigned uSlot)
	{
	ExtEntry   *pEntry=(ExtEntry*)pLocal;
	unsigned	uTurn;
	unsigned	uItem;

	uItem = pEntry->uNextFind;

	for(uTurn=0;;uTurn++)
		{
		if(uTurn>=EXT_MAX_FIND)return 0;
		if(uItem>=EXT_MAX_FIND)uItem =0;
		if(pEntry->aFindList[uItem]==0)break;
		}

	pEntry->uNextFind        = uItem+1;
	pEntry->aFindList[uItem] = new ext4_find;

	return (DiskFind)DPU_LIST_TO_HANDLE(DPD_TYPE_EXT,uSlot,uItem+0x1000);
	}

static void ExtFindFree(void *pLocal,void *pHandle)
	{
	ExtEntry   *pEntry=(ExtEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle)-0x1000;
	if(uItem>=EXT_MAX_FIND)return;

	delete pEntry->aFindList[uItem];
	pEntry->aFindList[uItem] = 0;
	}

static inline ext4_find *ExtFindGet(void *pLocal,void *pHandle)
	{
	ExtEntry   *pEntry=(ExtEntry*)pLocal;
	unsigned	uItem;

	   uItem = DPU_ITEM_OF_HANDLE(pHandle)-0x1000;
	if(uItem>=EXT_MAX_FIND)return 0;

	return pEntry->aFindList[uItem];
	}

static int ExtConvertAttrib(unsigned uFlags)
	{
	unsigned	uBits;
	unsigned	uType;

	uBits  = (uFlags<<16);
	uType  = (uFlags&0xF000);

	if(uType==0x4000)uBits|=DPD_DIR;				// 100h..40h=user 20h..08h=group 04h..01h=other
	if(uFlags&0x0100)uBits|=DPD_READ;
	if(uFlags&0x0080)uBits|=DPD_WRITE;
	if(uFlags&0x0040)uBits|=DPD_EXEC;
	if(uFlags&0x0001)uBits|=DPD_HIDDEN;				// Samba-Hack
	if(uFlags&0x0008)uBits|=DPD_SYSTEM;
	if(uFlags&0x0040)uBits|=DPD_ARCHIVE;

	return uBits;
	}

static int ExtError(int iError)
	{
	switch(iError)
		{
	case ENOENT:	return DPD_ERR_FILE_NOT_FOUND;
	case ENOTDIR:	return DPD_ERR_PATH_NOT_FOUND;
	case EEXIST:	return DPD_ERR_ALREADY_EXISTS;
	case EACCES:	return DPD_ERR_ACCESS_DENIED;
	case EPERM:		return DPD_ERR_ACCESS_DENIED;
	case EIO:		return DPD_ERR_ACCESS_DENIED;
	case EROFS:		return DPD_ERR_ACCESS_DENIED;
	case ENOMEM:	return DPD_ERR_NO_HANDLES;
	case ENOSPC:	return DPD_ERR_DISKFULL;
	case EINVAL:	return DPD_ERR_PARAM;
	case EOK:		return DPD_ERR_NONE;
		}

	return DPD_ERR_UNKNOWN;
	}

static int ExtResolveLink(LPVOID hHandle,const char *pPath,int iLen,const char *pName,char *pTarget,ext4_attributes *pAttrib=NULL)
	{
	char		cName[DPD_MAX_PATH];
	char		cLink[DPD_MAX_PATH];
	const char *pLink;
	size_t		iSize;
	int			iTurn;
	int			iPos;
	int			iRet;

	if(pPath[0]=='/')
		{
		pPath++;
		iLen--;
		}

	if(iLen<0)
		{
		iLen = strlen(pPath);
		}

	if(iLen && pPath[iLen-1]=='/')							// Slash am Ende entfernen
		{
		iLen--;
		}

	if(!pTarget)
		{
		pTarget = cName;
		}

	if(pName)												// Ist der Name extra angegeben
		{
		memcpy(cName,pPath ,iLen );
		strcpy(cName+iLen+1,pName);
		cName[iLen] = '/';
		pLink = cName;
		}
	else{
		pLink = pPath;
		}
															// Link einlesen
	   iRet = ext4_readlink(hHandle,pLink,cLink,sizeof(cLink),&iSize);
	if(iRet!=EOK)
		{
		return -iRet;
		}

	iTurn = 0;

Next:

	if(!pName)												// Zurck zu Pfadende
		{
		while(iLen>0 && pPath[iLen]!='/')
			{
			iLen--;
			}
		}

	if(cLink[0]=='/')
		{
		iPos = 1;
		iLen = 0;
		}
	else{
		iPos = 0;
		}

	for(;iPos<(int)iSize;)									// Zum Begin des Pfades im Link
		{
		if(cLink[iPos]!='.')
			{
			if(iPos==0)iLen=0;
			break;
			}

		if(cLink[iPos+1]=='/')								// "./" berspringen
			{
			iPos += 2;
			continue;
			}

		if(cLink[iPos+1]==0)								// "." am Ende
			{
			iPos += 1;
			break;
			}

		if(cLink[iPos+1]!='.')								// "./" berspringen
			{
			break;
			}

		if(cLink[iPos+2]=='/')								// "../" berspringen
			{
			iPos += 3;
			}
		else if(cLink[iPos+2]==0)							// ".." berspringen
			{
			iPos += 2;
			}
		else{
			break;
			}

		if(iLen<=0)											// Schon im Root-Verzeichnis
			{
			return -ENOENT;
			}

		for(iLen--;iLen>0;iLen--)
			{
			if(pPath[iLen]=='/')break;
			}
		}

	iSize -= iPos;

	if(iLen+iSize>DPD_MAX_PATH-3)
		{
		return -EFBIG;
		}

	memcpy(pTarget,pPath,iLen);

	if(iSize>0)												// Pfad zusammen bauen
		{
		if(iLen){pTarget[iLen] = '/'; iLen++;}
		memcpy(pTarget+iLen,cLink+iPos,iSize);
		iLen += iSize;
		}

	pTarget[iLen] = 0;

															// Weitere Links vorhanden
	   iRet = ext4_readlink_ex(hHandle,pTarget,cLink,sizeof(cLink),&iSize,pAttrib);
	if(iRet==EOK)
		{
		   iTurn++;
		if(iTurn>4)return -EMLINK;

		pPath	= pTarget;
		pName	= NULL;

		goto Next;
		}

	return iLen;
	}

static int ExtResolvePath(LPVOID hHandle,char *pBase,int iPos,ext4_attributes *pAttrib=NULL)
	{
	char	cTemp[DPD_MAX_PATH];
	char   *pName;
	int		iNum;
	int		iRet;

	pName	= pBase+iPos;

	for(iNum=0;;iNum++)
		{
		if(pName[iNum]=='/')break;
		if(pName[iNum]==0  )break;
		}

	strcpy(cTemp,pName+iNum);
	pName[iNum] = 0;


	   iRet = ExtResolveLink(hHandle,pBase,iPos,pName,pBase,pAttrib);
	if(iRet>=0)
		{
		strncpy(pBase+iRet,cTemp,DPD_MAX_PATH-iRet-1);
		return EOK;
		}

	return -iRet;
	}

static int ext_raw_open(ext4_blockdev *pDevice)
	{
	ExtEntry   *pEntry;
	QWORD		qwSize;

	pEntry	= (ExtEntry*)(((LPBYTE)pDevice)-offsetof(ExtEntry,sDevice));
	qwSize	= pEntry->pData->aCountSec[pEntry->uSlot];

	pEntry->sDevice.bdif->ph_bsize	= 512;
	pEntry->sDevice.bdif->ph_bcnt	= qwSize;
	pEntry->sDevice.part_size		= qwSize*512;
	pEntry->sDevice.part_offset		= 0;

	return EOK;
	}

static int ext_raw_bread(ext4_blockdev *pDevice,void *pBuffer,uint64_t uSecOffset,uint32_t qwSectors)
	{
	ExtEntry   *pEntry;

	pEntry	= (ExtEntry*)(((LPBYTE)pDevice)-offsetof(ExtEntry,sDevice));

	if(DiskReadProc(pEntry->pData,pEntry->uSlot,(LPBYTE)pBuffer,uSecOffset,qwSectors)!=DPD_OK)
		{
		return EIO;
		}

	return EOK;
	}

static int ext_raw_bwrite(ext4_blockdev *pDevice,const void *pBuffer,uint64_t uSecOffset,uint32_t qwSectors)
	{
	ExtEntry   *pEntry;

	pEntry	= (ExtEntry*)(((LPBYTE)pDevice)-offsetof(ExtEntry,sDevice));

	if(DiskWriteProc(pEntry->pData,pEntry->uSlot,(LPBYTE)pBuffer,uSecOffset,qwSectors)!=DPD_OK)
		{
		return EIO;
		}

	return EOK;
	}

static int ext_raw_close(ext4_blockdev *pDevice)
	{
	return EOK;
	}


#endif



#if	DPD_CACHE_SIZE>0

//*****************************************************************************
//*
//*		DiskCacheInit
//*
//*****************************************************************************
//	Alle Cache-Daten initialisieren.
//	pCache		: Zeiger auf die Cache-Daten
static void DiskCacheInit(DiskCache *pCache)
{
unsigned	uPos;


	memset(pCache,0,offsetof(DiskCache,aData));

	for(uPos=0;uPos<DPD_CACHE_SIZE;uPos++)
		{
		pCache->aLevelNext[uPos] = uPos+2;
		pCache->aLevelPrev[uPos] = uPos;
		}

	pCache->aLevelNext[DPD_CACHE_SIZE-1] = 0;
	pCache->aLevelPrev[0               ] = 0;
	pCache->aLevelFrom[0               ] = 1;
	pCache->aLevelLast[0               ] = DPD_CACHE_SIZE;

}

//*****************************************************************************
//*
//*		DiskCacheUnlink
//*
//*****************************************************************************
//	Cache-Puffer aus der Level Kette entfernen
//	pCache		: Zeiger auf die Cache-Daten
//	uIndex		: Nummer des Cache-Puffers
#if	DISK_CACHE_CHECK
static inline void DiskCacheCheck(DiskCache *pCache,unsigned uIndex)
{
QWORD		aList[DPD_CACHE_SIZE];
BYTE		aDone[DPD_CACHE_SIZE];
QWORD		uSector;
BYTE		bHash;
unsigned 	uPos;
unsigned 	uNum;
unsigned 	uEnd;
unsigned 	uVal;
unsigned 	uIdx;
unsigned 	uLvl;


	ASSERT(uIndex<DPD_CACHE_SIZE);

	uSector = pCache->aSector[uIndex];
	bHash	= (BYTE)uSector;

	memset(aList,0,sizeof(aList));
	memset(aDone,0,sizeof(aDone));

	uNum	= 0;
	uPos	= pCache->aHashStart[bHash];

	while(uPos)											// Prfe die Hash-Ketten
		{
		uPos--;

		ASSERT(uPos<DPD_CACHE_SIZE);
		ASSERT(aDone[uPos]==0);

		for(uIdx=0;uIdx<uNum;uIdx++)
			{
			ASSERT(aList[uIdx]!=pCache->aSector[uPos]);
			}

		aDone[uPos] = 1;
		aList[uNum] = pCache->aSector[uPos];

		ASSERT((BYTE)aList[uNum]==bHash);

		uNum++;

		uPos = pCache->aNext[uPos];
		}


	memset(aList,0,sizeof(aList));
	memset(aDone,0,sizeof(aDone));

	uNum	= 0;
	uEnd	= 0;
	uLvl    = pCache->aCount[uIndex];

	if(uLvl>=DPD_CACHE_LEVELS)uLvl=DPD_CACHE_LEVELS-1;

	uPos	= pCache->aLevelFrom[uLvl];

	while(uPos)											// Prfe die Level-Ketten
		{
		uEnd = uPos;
		uPos--;

		ASSERT(uPos<DPD_CACHE_SIZE);
		ASSERT(aDone[uPos]==0);

		for(uIdx=0;uIdx<uNum;uIdx++)
			{
			ASSERT(aList[uIdx]!=pCache->aSector[uPos]);
			}

		aDone[uPos] = 1;
		aList[uNum] = pCache->aSector[uPos];

		uVal = pCache->aCount[uPos];
		if(uVal>=DPD_CACHE_LEVELS)uVal=DPD_CACHE_LEVELS-1;

		ASSERT(uVal==uLvl);

		uNum++;

		uPos = pCache->aLevelNext[uPos];
		}


	ASSERT(pCache->aLevelLast[uLvl]==uEnd);


}
#else
#define 	DiskCacheCheck(pCache,uIndex)
#endif

//*****************************************************************************
//*
//*		DiskCacheUnlink
//*
//*****************************************************************************
//	Cache-Puffer aus der Level Kette entfernen
//	pCache		: Zeiger auf die Cache-Daten
//	uIndex		: Nummer des Cache-Puffers
static inline void DiskCacheUnlink(DiskCache *pCache,unsigned uIndex)
{
unsigned	uLevel;
unsigned	uNext;
unsigned	uPrev;


	   uLevel = pCache->aCount[uIndex];					// Aus Level-Kette entfernen
	if(uLevel>=DPD_CACHE_LEVELS)
		{
		uLevel = DPD_CACHE_LEVELS-1;
		}

	uNext = pCache->aLevelNext[uIndex];
	uPrev = pCache->aLevelPrev[uIndex];

	if(uNext)
		{
		pCache->aLevelPrev[uNext-1] = uPrev;
		}
	else{
		pCache->aLevelLast[uLevel ] = uPrev;
		}

	if(uPrev)
		{
		pCache->aLevelNext[uPrev-1] = uNext;
		}
	else{
		pCache->aLevelFrom[uLevel ] = uNext;
		}

}

//*****************************************************************************
//*
//*		DiskCacheLink
//*
//*****************************************************************************
//	Cache-Puffer in eine Kette einfgen
//	pCache		: Zeiger auf die Cache-Daten
//	uIndex		: Nummer des Cache-Puffers
static inline void DiskCacheLink(DiskCache *pCache,unsigned uIndex)
{
unsigned	uLevel;
unsigned	uNext;


	   uLevel = pCache->aCount[uIndex];
	if(uLevel>=DPD_CACHE_LEVELS)
		{
		uLevel = DPD_CACHE_LEVELS-1;
		}
														// In Level-Kette 0 einfgen
	   uNext = pCache->aLevelFrom[uLevel];
	if(uNext)
		{
		pCache->aLevelPrev[uNext-1]	= uIndex+1;
		}
	else{
		pCache->aLevelLast[uLevel ] = uIndex+1;
		}


	pCache->aLevelPrev[uIndex] = 0;
	pCache->aLevelNext[uIndex] = uNext;
	pCache->aLevelFrom[uLevel] = uIndex+1;



}

//*****************************************************************************
//*
//*		DiskCacheSet
//*
//*****************************************************************************
//	berschreibt einen vorhanden Sektor im Cache
//	pCache		: Zeiger auf die Cache-Daten
//	uSector		: Ist die Sektornummer
//	pBuffer		: Ist der Puffer fr den Sektor
//	Ergibt 1 wenn der Sektor gespeichert wurde bzw. 0 wenn nicht
static int DiskCacheSet(DiskCache *pCache,QWORD uSector,const void *pBuffer)
{
BYTE		bHash;
unsigned	uIndex;



	bHash	= (BYTE)uSector;
	uIndex	= pCache->aHashStart[bHash];

	while(uIndex)										// Sektor bereits vorhanden
		{
		uIndex--;

		if(pCache->aSector[uIndex]==uSector)
			{
			memcpy(pCache->aData+uIndex*512,pBuffer,512);
			return 1;
			}

		uIndex	= pCache->aNext[uIndex];
		}


return 0;
}


//*****************************************************************************
//*
//*		DiskCacheLen
//*
//*****************************************************************************
//	Holt die Anzahl an Sektoren die nicht im Cache sind
//	pCache		: Zeiger auf die Cache-Daten
//	uSector		: Ist die Sektornummer
//	pBuffer		: Ist der Puffer fr den Sektor
//	Ergibt die Anzahl der vortlaufent gespeicherten Sektoren im Cache.
static int DiskCacheLen(DiskCache *pCache,QWORD uSector,unsigned uCount)
{
BYTE		bHash;
unsigned	uIndex;
unsigned	uRet;


	bHash	= (BYTE)uSector;

	for(uRet=0;uCount>0;uRet++,uSector++,uCount--,bHash++)
		{
		uIndex	= pCache->aHashStart[bHash];

		while(uIndex)										// Suchen den Sektor
			{
			uIndex--;

			if(pCache->aSector[uIndex]==uSector)
				{
				return uRet;
				}

			uIndex	= pCache->aNext[uIndex];
			}
		}


return uRet;
}

//*****************************************************************************
//*
//*		DiskCacheGet
//*
//*****************************************************************************
//	Holt einen Sektor aus dem Cache
//	pCache		: Zeiger auf die Cache-Daten
//	uSector		: Ist die Sektornummer
//	pBuffer		: Ist der Puffer fr den Sektor
//	Ergibt 1 wenn der Sektor gefunden wurde bzw. 0 wenn nicht
static int DiskCacheGet(DiskCache *pCache,QWORD uSector,void *pBuffer)
{
BYTE		bHash;
unsigned	uIndex;


	bHash	= (BYTE)uSector;
	uIndex	= pCache->aHashStart[bHash];

	while(uIndex)										// Suchen den Sektor
		{
		uIndex--;

		if(pCache->aSector[uIndex]==uSector)
			{
			memcpy(pBuffer,pCache->aData+uIndex*512,512);

			DiskCacheUnlink(pCache,uIndex);				// In anderes Level heben

			pCache->uTurn          += 1;
			pCache->aCount[uIndex] += 1;
			pCache->aLast [uIndex]  = pCache->uTurn;

			DiskCacheLink(pCache,uIndex);

			DiskCacheCheck(pCache,uIndex);

			return 1;
			}

		uIndex	= pCache->aNext[uIndex];
		}


return 0;
}


//*****************************************************************************
//*
//*		DiskCachePut
//*
//*****************************************************************************
//	Speichert einen Sektor im Cache
//	pCache		: Zeiger auf die Cache-Daten
//	uSector		: Ist die Sektornummer
//	pBuffer		: Ist der Puffer fr den Sektor
//	Ergibt 1 wenn der Sektor gespeichert wurde bzw. 0 wenn nicht
static int DiskCachePut(DiskCache *pCache,QWORD uSector,const void *pBuffer)
{
BYTE		bHash;
BYTE		bBase;
unsigned	uIndex;
unsigned	uValue;
unsigned	uLevel;
unsigned	uDiff;
unsigned	uPrev;
unsigned	uNext;
unsigned	uMin;



	bHash	= (BYTE)uSector;
	uIndex	= pCache->aHashStart[bHash];

	while(uIndex)										// Sektor bereits vorhanden
		{
		uIndex--;

		if(pCache->aSector[uIndex]==uSector)
			{
			memcpy(pCache->aData+uIndex*512,pBuffer,512);

			DiskCacheUnlink(pCache,uIndex);				// In anderes Level heben

			pCache->uTurn          += 1;
			pCache->aCount[uIndex] += 1;
			pCache->aLast [uIndex]  = pCache->uTurn;

			DiskCacheLink(pCache,uIndex);

			DiskCacheCheck(pCache,uIndex);

			return 1;
			}

		uIndex	= pCache->aNext[uIndex];
		}


	uValue	= 0;
	uMin	= 0;

	if(pCache->aLevelFrom[0])							// Gibt es unbenutzte Puffer
		{
		uIndex = pCache->aLevelFrom[0]-1;
		}
	else{
		for(uLevel=1;;uLevel++)
			{
			if(uLevel>=DPD_CACHE_LEVELS)
				{
				uIndex = uMin;
				break;
				}

				uIndex = pCache->aLevelLast[uLevel];
			if(!uIndex)continue;

			uIndex = uIndex-1;
			uDiff  = pCache->uTurn-pCache->aLast[uIndex];
			uDiff += 1024<<(DPD_CACHE_LEVELS-1-uLevel);

			if(uDiff>uValue)
				{
				uValue	= uDiff;
				uMin	= uIndex;
				}
			}
		}


	DiskCacheUnlink(pCache,uIndex);


	if(pCache->aCount[uIndex])							// Aushngen aus der Hash-Kette
		{
		uNext =       pCache->aNext  [uIndex];
		uPrev =       pCache->aPrev  [uIndex];
		bBase = (BYTE)pCache->aSector[uIndex];

		if(uNext)
			{
			pCache->aPrev[uNext-1]	  = uPrev;
			}

		if(uPrev)
			{
			pCache->aNext[uPrev-1]	  = uNext;
			}
		else{
			pCache->aHashStart[bBase] = uNext;
			}

		DiskCacheUnlink(pCache,uIndex);
		}


	   uNext = pCache->aHashStart[bHash];
	if(uNext)
		{
		pCache->aPrev[uNext-1]	= uIndex+1;
		}


	memcpy(pCache->aData+uIndex*512,pBuffer,512);		// Neuen Sektor einfgen

	pCache->uTurn			   += 1;
	pCache->aCount [uIndex]		= 1;
	pCache->aPrev  [uIndex]		= 0;
	pCache->aNext  [uIndex]		= uNext;
	pCache->aLast  [uIndex]		= pCache->uTurn;
	pCache->aSector[uIndex]		= uSector;
	pCache->aHashStart[bHash]	= uIndex+1;


	DiskCacheLink(pCache,uIndex);


	DiskCacheCheck(pCache,uIndex);



return 1;
}

#endif

//*****************************************************************************
//*
//*		DiskInitProc
//*
//*****************************************************************************
//	Ein Laufwerk initialisieren
//	pPtr		: Zeiger auf das FATDATA Objekt
//	uDrive		: Ist die Laufwerksnummer (0...n)
//	Ergibt die Status-Flags des Laufwerkes
extern "C" unsigned char _cdecl DiskInitProc(void *pPtr,DWORD uDrive)
{
DiskPartionData *pData=(DiskPartionData*)pPtr;


														// Existiert das Laufwerk
	if(uDrive>=pData->uDriveCount || pData->aDriveType[uDrive]==DPD_TYPE_NONE)
		{
		return DPD_NOINIT;
		}

	switch(pData->aDriveMode[uDrive])					// Art des Laufwerkes
		{
	case DPD_MODE_FILE:		 return DPD_COMMON;
	case DPD_MODE_FILE_RO:	 return DPD_PROTECT;
	case DPD_MODE_DISK:		 return DPD_COMMON;
	case DPD_MODE_DISK_RO:	 return DPD_PROTECT;
		}


return DPD_NOINIT;
}

//*****************************************************************************
//*
//*		DiskStatusProc
//*
//*****************************************************************************
//	Den Status einer File abfragen
//	pPtr		: Zeiger auf das FATDATA Objekt
//	uDrive		: Ist die Laufwerksnummer (0...n)
//	Ergibt die Status-Flags des Laufwerkes
extern "C" unsigned char _cdecl DiskStatusProc(void *pPtr,DWORD uDrive)
{
DiskPartionData *pData=(DiskPartionData*)pPtr;


														// Existiert das Laufwerk
	if(uDrive>=pData->uDriveCount || pData->aDriveType[uDrive]==DPD_TYPE_NONE)
		{
		return DPD_NOINIT;
		}

	switch(pData->aDriveMode[uDrive])					// Art des Laufwerkes
		{
	case DPD_MODE_FILE:		 return DPD_COMMON;
	case DPD_MODE_FILE_RO:	 return DPD_PROTECT;
	case DPD_MODE_DISK:		 return DPD_COMMON;
	case DPD_MODE_DISK_RO:	 return DPD_PROTECT;
		}


return DPD_NOINIT;
}


//*****************************************************************************
//*
//*		DiskReadProc
//*
//*****************************************************************************
//	Sektoren von einen Laufwerk einlesen
//	pPtr		: Zeiger auf das FATDATA Objekt
//	uDrive		: Ist die Laufwerksnummer (0...n, 0xFFFFFFFF=MBR lesen)
//	pBuffer		: Zeiger auf den Puffer fr die Sektoren
//	uSecOffset	: Sektor-Offset im Laufwerk
//	dwSectors	: Anzahl der zu lesenden Sektoren
//	Ergibt die Status-Flags des Laufwerkes
extern "C" unsigned char _cdecl DiskReadProc(void *pPtr,DWORD uDrive,unsigned char *pBuffer,QWORD uSecOffset,DWORD dwSectors)
{
DiskPartionData    *pData=(DiskPartionData*)pPtr;
DWORD				dwBytes;
LONG				lHigh;


														// Existiert das Laufwerk
	if(uDrive>=pData->uDriveCount || pData->aDriveType[uDrive]==DPD_TYPE_NONE)
		{
		return DPD_ERROR;
		}

	if(uSecOffset+dwSectors>pData->aCountSec[uDrive])
		{
		return DPD_ERROR;
		}


	uSecOffset += pData->aStartSec[uDrive];


	switch(pData->aDriveMode[uDrive])					// Art des Laufwerkes
		{
	case DPD_MODE_FILE:
	case DPD_MODE_FILE_RO:

			dwSectors  *= 512;
			uSecOffset *= 512;
			lHigh		= (DWORD)(uSecOffset>>32);

			if(SetFilePointer(pData->aDrivesPtr[uDrive],(DWORD)uSecOffset,&lHigh,FILE_BEGIN)==0xFFFFFFFF)
				{
				return DPD_ERROR;
				}

			if(ReadFile(pData->aDrivesPtr[uDrive],pBuffer,dwSectors,&dwBytes,NULL) || dwBytes==dwSectors)
				{
				return DPD_OK;
				}

			break;

	case DPD_MODE_DISK:
	case DPD_MODE_DISK_RO:

			#if DPD_CACHE_SIZE>0

			while(dwSectors>1)							// Mehr als ein Sektor
				{
				unsigned	uLen;

					uLen = DiskCacheLen((DiskCache*)pData->aCacheData[uDrive],uSecOffset,dwSectors);
				if(!uLen)
					{
					if(DiskCacheGet((DiskCache*)pData->aCacheData[uDrive],uSecOffset,pBuffer))
						{
						pBuffer    += 512;
						uSecOffset += 1;
						dwSectors  -= 1;
						continue;
						}

					uLen = 1;
					}
														// Sektoren einlesen
				if(!ReadSector(pData->aDrivesPtr[uDrive],uSecOffset,uLen,pBuffer))
					{
					return DPD_OK;
					}

				if(uLen>16)								// Zu groe Blcke nicht in den Cache
					{
					pBuffer    += uLen*512;
					uSecOffset += uLen;
					dwSectors  -= uLen;
					uLen	   -= uLen;
					}
				else while(uLen)						// Geladene Sektoren in den Cache
					{
					DiskCachePut((DiskCache*)pData->aCacheData[uDrive],uSecOffset,pBuffer);
					pBuffer    += 512;
					uSecOffset += 1;
					dwSectors  -= 1;
					uLen	   -= 1;
					}
				}

			if(dwSectors==0)
				{
				return DPD_OK;
				}
														// Den geladenen Sektor in den Cache
			if(DiskCacheGet((DiskCache*)pData->aCacheData[uDrive],uSecOffset,pBuffer))
				{
				return DPD_OK;
				}

			if(((unsigned)uSecOffset&7)==0 && dwSectors==1)
				{										// 4k-Blcke in den Cache
				BYTE	aBuffer[4096];

				if(ReadSector(pData->aDrivesPtr[uDrive],uSecOffset,8,aBuffer))
					{
					unsigned	uLen;

					memcpy(pBuffer,aBuffer,512);

					for(uLen=0;uLen<8;uLen++)
						{
						DiskCachePut((DiskCache*)pData->aCacheData[uDrive],uSecOffset+uLen,aBuffer+uLen*512);
						}

					return DPD_OK;
					}
				}

			#endif

			if(ReadSector(pData->aDrivesPtr[uDrive],uSecOffset,dwSectors,pBuffer))
				{
				#if DPD_CACHE_SIZE>0

				DiskCachePut((DiskCache*)pData->aCacheData[uDrive],uSecOffset,pBuffer);

				#endif

				return DPD_OK;
				}

			break;
		}




return DPD_ERROR;
}

//*****************************************************************************
//*
//*		DiskWriteProc
//*
//*****************************************************************************
//	Sektoren auf einem Laufwerk schreiben
//	pPtr		: Zeiger auf das FATDATA Objekt
//	uDrive		: Ist die Laufwerksnummer (0...n)
//	pBuffer		: Zeiger auf den Puffer fr die Sektoren
//	uSecOffset	: Sektor-Offset im Laufwerk
//	dwSectors	: Anzahl der zu schreibenden Sektoren
//	Ergibt die Status-Flags des Laufwerkes
extern "C" unsigned char _cdecl DiskWriteProc(void *pPtr,DWORD uDrive,const unsigned char *pBuffer,QWORD uSecOffset,DWORD dwSectors)
{
DiskPartionData    *pData=(DiskPartionData*)pPtr;
DWORD				dwBytes;
LONG				lHigh;


														// Existiert das Laufwerk
	if(uDrive>=pData->uDriveCount || pData->aDriveType[uDrive]==DPD_TYPE_NONE)
		{
		return DPD_ERROR;
		}

	if(uSecOffset+dwSectors>pData->aCountSec[uDrive])
		{
		return DPD_ERROR;
		}


	uSecOffset += pData->aStartSec[uDrive];


	switch(pData->aDriveMode[uDrive])					// Art des Laufwerkes
		{
	case DPD_MODE_FILE_RO:

			return DPD_READONLY;

	case DPD_MODE_FILE:

			dwSectors  *= 512;
			uSecOffset *= 512;
			lHigh		= (DWORD)(uSecOffset>>32);

			if(SetFilePointer(pData->aDrivesPtr[uDrive],(DWORD)uSecOffset,&lHigh,FILE_BEGIN)==0xFFFFFFFF)
				{
				return DPD_ERROR;
				}

			if(WriteFile(pData->aDrivesPtr[uDrive],pBuffer,dwSectors,&dwBytes,NULL) || dwBytes==dwSectors)
				{
				return DPD_OK;
				}

			break;

	case DPD_MODE_DISK_RO:

			return DPD_READONLY;

	case DPD_MODE_DISK:

			#if DPD_CACHE_SIZE>0

			if(dwSectors>16)							// Groe Blcke nur updaten
				{
				for(dwBytes=0;dwBytes<dwSectors;dwBytes++)
					{
					DiskCacheSet((DiskCache*)pData->aCacheData[uDrive],uSecOffset+dwBytes,pBuffer+dwBytes*512);
					}
				}
			else{										// Sektoren in den Cache bernehmen
				for(dwBytes=0;dwBytes<dwSectors;dwBytes++)
					{
					DiskCachePut((DiskCache*)pData->aCacheData[uDrive],uSecOffset+dwBytes,pBuffer+dwBytes*512);
					}
				}

			#endif

			if(WriteSector(pData->aDrivesPtr[uDrive],uSecOffset,dwSectors,pBuffer))
				{
				return DPD_OK;
				}

			break;
		}




return DPD_ERROR;
}

//*****************************************************************************
//*
//*		DiskIoctlProc
//*
//*****************************************************************************
//	Diverse File-Funktionen
//	pPtr		: Zeiger auf das FATDATA Objekt
//	uDrive		: Ist die Laufwerksnummer (0...n)
//	pBuffer		: Zeiger auf den Puffer fr die Sektoren
//	uSecOffset	: Sektor-Offset im Laufwerk
//	qwSectors	: Anzahl der zu schreibenden Sektoren
//	Ergibt die Status-Flags des Laufwerkes
extern "C" unsigned char _cdecl DiskIoctlProc(void *pPtr,DWORD uDrive,unsigned char ucCommand,void *pBuffer)
{
DiskPartionData *pData=(DiskPartionData*)pPtr;


														// Existiert das Laufwerk
	if(uDrive>=pData->uDriveCount || pData->aDriveType[uDrive]==DPD_TYPE_NONE)
		{
		return DPD_ERROR;
		}


	switch(ucCommand)									// Welches Kommando
		{
	case DPD_CTRL_SECTOR_COUNT:	*(DWORD*)pBuffer = (DWORD)pData->aCountSec[uDrive];	return DPD_OK;
	case DPD_CTRL_SECTOR_CNT64:	*(QWORD*)pBuffer = pData->aCountSec[uDrive];		return DPD_OK;
	case DPD_CTRL_SECTOR_SIZE:	*(WORD *)pBuffer = 512;								return DPD_OK;
	case DPD_CTRL_BLOCK_SIZE:	*(DWORD*)pBuffer = 1;								return DPD_OK;
	case DPD_CTRL_POWER:															return DPD_OK;
	case DPD_CTRL_EJECT:															return DPD_OK;
	case DPD_CTRL_LOCK:																return DPD_OK;
	case DPD_CTRL_SYNC:

		switch(pData->aDriveMode[uDrive])				// Art des Laufwerkes
			{
		case DPD_MODE_FILE:

				FlushFileBuffers(pData->aDrivesPtr[uDrive]);
				break;
			}

		return DPD_OK;

	case DPD_CTRL_READONLY:

		*(DWORD*)pBuffer = (pData->aDriveMode[uDrive]&1) && (pData->aDriveType[uDrive]>1);

		return DPD_OK;
		}


return DPD_PARAMERR;
}

//*****************************************************************************
//*
//*		DiskGetFreeSlot
//*
//*****************************************************************************
//	Sucht nach einem freien Slot in den Laufwerksdaten
//	pData		: Zeiger auf die Laufwerksdaten
//	Ergibt die Slotnummer oder -1 bei einem Fehler
static int _cdecl DiskGetFreeSlot(DiskPartionData *pData)
{
int		iPos;


	if(pData->uDriveCount>=DPD_MAX_PARTIONS)
		{
		return -1;
		}

	for(iPos=0;iPos<DPD_MAX_PARTIONS;iPos++)
		{
		if(pData->aDriveMode[iPos]==DPD_MODE_NONE)
			{
			return iPos;
			}
		}


return -1;
}

//*****************************************************************************
//*
//*		DiskClearFindStruct
//*
//*****************************************************************************
//	Lscht eine Find-Struktur
static inline void DiskClearFindStruct(DiskFindStruct *pFind)
{
	pFind->dwSize   = 0;
	pFind->cName[0] = 0;

    #if DISK_HAS_SHORT_NAME
    pFind->cShortName[0] = 0;
    #endif
}


//*****************************************************************************
//*
//*		DiskMountPartition
//*
//*****************************************************************************
//	Mountet eine Partition
//	pData		: Zeiger auf die Laufwerksdaten
//	uPhysical	: Nummer des pysikalischen Laufwerks
//	uSlot		: Nummer des logischen Laufwerks
//	Ergibt TRUE wenn die Partition erfolgreich gemounted wurde.
static int DiskMountPartition(DiskPartionData *pData,unsigned uPhysical,unsigned uSlot)
{
unsigned	uType;



	if(uSlot    >=DPD_MAX_PARTIONS)return FALSE;
	if(uPhysical>=DPD_MAX_PARTIONS)return FALSE;

	uType = pData->aDriveType[uSlot];


	#if DPD_HAS_FAT

	if(uType==DPD_TYPE_FAT)								// Mounte eine FAT-Partition
		{
		FatEntry   *pEntry;

		time(0);
		iTimeZone = _timezone;
		pEntry	  = new FatEntry;
		pData->aLocalData[uSlot] = pEntry;
		memset(pEntry,0,sizeof(FatEntry));

		pEntry->sFatData.user_ptr			= pData;
		pEntry->sFatData.user_data			= uSlot;
		pEntry->sFatData.disk_init_proc		= DiskInitProc;
		pEntry->sFatData.disk_status_proc	= DiskStatusProc;
		pEntry->sFatData.disk_ioctl_proc	= DiskIoctlProc;
		pEntry->sFatData.disk_write_proc	= DiskWriteProc;
		pEntry->sFatData.disk_read_proc		= DiskReadProc;

	 	if(f_mount(&pEntry->sFatData,0,&pEntry->sFatFs)==FR_OK)
			{
			return TRUE;
			}

		pData->aLocalData[uSlot] = 0;
		delete pEntry;
		}

	#endif

	#if DPD_HAS_EXFAT

	if(uType==DPD_TYPE_EXFAT)								// Mounte eine EXFAT-Partition
		{
		char			cOptions[64]="noatime";
		ExFatEntry     *pEntry;

		pEntry	  = new ExFatEntry;
		pData->aLocalData[uSlot] = pEntry;
		memset(pEntry,0,sizeof(ExFatEntry));

		if(pData->aDriveMode[uPhysical]&DPD_FLAG_READONLY)
			{
			strcat(cOptions,",ro");
			}

	 	if(!exfat_mount(&pEntry->sExFat,(char*)pData,cOptions,uSlot))
			{
			return TRUE;
			}

		pData->aLocalData[uSlot] = 0;
		delete pEntry;
		}

	#endif

	#if DPD_HAS_EXT

	if(uType==DPD_TYPE_EXT)									// Mounte eine EXT?-Partition
		{
		ExtEntry       *pEntry;
		bool	 		bRdOnly;
		int				iDone;
		int				iRet;

		iDone   = 0;
		pEntry	= new ExtEntry;
		pData->aLocalData[uSlot] = pEntry;
		memset(pEntry,0,sizeof(ExtEntry));
		bRdOnly = (pData->aDriveMode[uPhysical]&DPD_FLAG_READONLY)? 1:0;

		pEntry->sInterface.open		=  ext_raw_open;
		pEntry->sInterface.bread	=  ext_raw_bread;
		pEntry->sInterface.bwrite	=  ext_raw_bwrite;
		pEntry->sInterface.close	=  ext_raw_close;
		pEntry->sInterface.ph_bsize	=  512;
		pEntry->sInterface.ph_bbuf	=  pEntry->aBuffer;
		pEntry->sDevice.bdif		= &pEntry->sInterface;
		pEntry->pData				=  pData;
		pEntry->uSlot				=  uSlot;

		   iRet = ext4_mount(&pEntry->sDevice,&pEntry->hHandle,bRdOnly);
		if(iRet==EOK)
			{
			iDone = 1;

			if(bRdOnly)return TRUE;

			   iRet=ext4_recover(pEntry->hHandle);
			if(iRet!=EOK && iRet!=ENOTSUP)iDone = 0;
			   iRet=ext4_journal_start(pEntry->hHandle);
			if(iRet!=EOK)iDone = 0;
			   iRet=ext4_cache_write_back(pEntry->hHandle,1);
			if(iRet!=EOK)iDone = 0;
			if(iDone)return TRUE;

			ext4_umount(pEntry->hHandle);
			}

		pData->aLocalData[uSlot] = 0;
		delete pEntry;
		}

	#endif


return FALSE;
}


//*****************************************************************************
//*
//*		DiskScanPartition
//*
//*****************************************************************************
//	Sucht nach einer einzelnen Partitionen
//	pData		: Zeiger auf die Laufwerksdaten
//	uPhysical	: Nummer des pysikalischen Laufwerks
//	uStartSec	: Startsektor der Partition
//	uMaxSec		: Gre der Partition in Sektoren
//	bNoEmpty	: Nur formatierte Laufwerke mounten
//	bHidden		: War die Partition versteckt
//	Ergibt TRUE wenn eine Partition eingehngt wurde erfolgreich.
static BOOL DiskScanPartition(DiskPartionData *pData,DWORD uPhysical,QWORD uStartSec,QWORD uMaxSec,BOOL bNoEmpty,BOOL bHidden=FALSE)
{
unsigned char	aData[2048];
BootRecord16   *pBoot16;
BootRecord32   *pBoot32;
unsigned		uDrive;
unsigned		uType;
unsigned		uDone;
unsigned		uSign;

														// Prfe Laufwerksnummer
	if(uPhysical>=DPD_MAX_PARTIONS || pData->aDriveType[uPhysical]!=DPD_TYPE_PHYSICAL)
		{
		return FALSE;
		}

	if(pData->uDriveCount>=DPD_MAX_PARTIONS)			// Sind Pltze fr logische Partitionen frei
		{
		return FALSE;
		}

	if(uStartSec<pData->aStartSec[uPhysical])			// Prfe die Position des Startsektors
		{
		return FALSE;
		}
														// Prfe die Gre der Partition
	if(uStartSec+uMaxSec>pData->aStartSec[uPhysical]+pData->aCountSec[uPhysical])
		{
		return FALSE;
		}
														// Boot-Sektor einlesen
	if(DiskReadProc(pData,uPhysical,aData,uStartSec,4)!=DPD_OK)
		{
		return FALSE;
		}



	pBoot16	= (BootRecord16*)aData;
	pBoot32	= (BootRecord32*)aData;
	uDrive	= DiskGetFreeSlot(pData);
	uType	= DPD_TYPE_EMPTY;
	uDone	= 0;

														// Ist es eine EXFAT Partition
	if(pBoot32->wSignature==0xAA55 && !memcmp(pBoot16->cOEMName,"EXFAT",5))
		{
		uType = DPD_TYPE_EXFAT;
		}												// Ist es eine FAT Partition
	else if(pBoot32->wSignature==0xAA55 && (!memcmp(pBoot16->cSystemID,"FAT",3) || !memcmp(pBoot32->cSystemID,"FAT",3)))
		{
		uType = DPD_TYPE_FAT;
		}												// Ist es ein EXT? Superblock
	else if(*(LPDWORD)(aData+0x0400) && *(LPWORD)(aData+0x0438)==0xEF53)
		{
		uType = DPD_TYPE_EXT;
		}


	pData->aDrivesPtr[uDrive]	= pData->aDrivesPtr[uPhysical];
	pData->aDriveMode[uDrive]	= pData->aDriveMode[uPhysical];
	pData->aDriveType[uDrive]	= uType;
	pData->aDriveHide[uDrive]	= bHidden;
	pData->aRefCount [uDrive]	= uPhysical;
	pData->aStartSec [uDrive]	= uStartSec;
	pData->aCountSec [uDrive]	= uMaxSec;
	pData->aLocalData[uDrive]	= NULL;

	pData->aRefCount[uPhysical]++;						// Referenzzhler erhhen
	pData->uPartitionCount++;
	pData->uDriveCount++;

	uSign  = pData->uPartitionCount-1;
	uSign += DPD_BASE_LETTER-'0';
	if(uSign>=10)uSign +='A'-'9'-1;

	pData->aLetterToSlot[uSign] = (unsigned char)(uDrive+1);
	pData->aDriveSign[uDrive]   = (unsigned char)(uSign +1);

	#if DPD_HAS_FAT
	if(uType==DPD_TYPE_FAT)uDone = 1;						// Mounte eine FAT-Partition
	#endif

	#if DPD_HAS_EXFAT
	if(uType==DPD_TYPE_EXFAT)uDone = 1;						// Mounte eine EXFAT-Partition
	#endif

	#if DPD_HAS_EXT
	if(uType==DPD_TYPE_EXT)uDone=1;							// Mounte eine EXT?-Partition
	#endif


	if(uDone)
		{
		pData->aDriveType[uDrive] = uType;
		}

	if(!uDone)
		{
		if(bNoEmpty)
			{
			pData->aDrivesPtr[uDrive]	= 0;
			pData->aDriveMode[uDrive]	= DPD_MODE_NONE;
			pData->aDriveType[uDrive]	= DPD_TYPE_NONE;
			pData->aDriveHide[uDrive]	= 0;
			pData->aRefCount [uDrive]	= 0;
			pData->aStartSec [uDrive]	= 0;
			pData->aCountSec [uDrive]	= 0;
			pData->aDriveSign[uDrive]   = 0;
			pData->aLetterToSlot[uSign] = 0;

			pData->aRefCount[uPhysical]--;					// Referenzzhler verringern
			pData->uPartitionCount--;
			pData->uDriveCount--;
			return FALSE;
			}

		pData->aDriveType[uDrive] = DPD_TYPE_EMPTY;
		}



return TRUE;
}


//*****************************************************************************
//*
//*		DiskScanMBR
//*
//*****************************************************************************
//	Such nach Partitionen auf einem MBR
//	pData		: Zeiger auf die Laufwerksdaten
//	uPhysical	: Nummer des pysikalischen Laufwerks
//	uPartition	: Ist die Partition die gemountet werden soll (0xFFFFFFFF=alle)
//	uNoEmpty	: Nur formatierte Laufwerke mounten
//	Ergibt die Zahl der gemonteten Partitionen
static int DiskScanMBR(DiskPartionData *pData,DWORD uPhysical,unsigned uPartition,unsigned uNoEmpty,PartitionLoad *pEntires=0)
{
unsigned char		aHide[DPD_MAX_LOAD];
unsigned char		aData[2048];
DWORD				aNext[32];
DWORD				aTemp[32];
GuidPartitionEntry	aList[4];
BootRecord16	   *pBoot16;
BootRecord32	   *pBoot32;
MasterBootRecord   *pMbr;
GuidPartitionTable	sGuid;
DWORD				dwEnd;
DWORD				dwPos;
DWORD				dwMax;
DWORD				dwOff;
DWORD				dwNull;
DWORD				dwIndex;
DWORD				dwNumber;
QWORD				qwSectors;
DWORD				dwOffset;
DWORD				dwCount;
QWORD				qwSize;
QWORD				qwLast;
int					iAdd=0;
int					iNum=1;
int					iIdx;
int					iPos;
int					iOff;
int					iOk;


	if(uPhysical>=pData->uDriveCount)					// Prfe die Laufwerksnummer
		{
		return 0;
		}

	if(DiskReadProc(pData,uPhysical,aData,0,4)!=DPD_OK)	// MBR einlesen
		{
		return 0;
		}

	aNext[0]	= 0;
	aTemp[0]	= 0;
	pMbr		= (MasterBootRecord*)aData;				// Prfe ob Boot-Sektor statt MBR
	pBoot16		= (BootRecord16    *)aData;
	pBoot32		= (BootRecord32    *)aData;
	qwSectors	= pData->aCountSec[uPhysical];



	if(uPartition==0 || uPartition==0xFFFFFFFF)
		{												// Ist es ein EXFAT Boot-Sektor
		if(pBoot16->wSignature!=0xAA55 && !memcmp(pBoot16->cOEMName,"EXFAT",5))
			{
			if(DiskScanPartition(pData,uPhysical,0,qwSectors,uNoEmpty))return 1;
			}
														// Ist es ein FAT16 Boot-Sektor
		if(pBoot16->wSignature!=0xAA55 && !memcmp(pBoot16->cSystemID,"FAT",3))
			{
			if(DiskScanPartition(pData,uPhysical,0,qwSectors,uNoEmpty))return 1;
			}
														// Ist es ein FAT32 Boot-Sektor
		if(pBoot16->wSignature!=0xAA55 && !memcmp(pBoot32->cSystemID,"FAT",3))
			{
			if(DiskScanPartition(pData,uPhysical,0,qwSectors,uNoEmpty))return 1;
			}
														// Ist es ein EXT? Superblock
		if(*(LPDWORD)(aData+0x0400) && *(LPWORD)(aData+0x0438)==0xEF53)
			{
			if(DiskScanPartition(pData,uPhysical,0,qwSectors,uNoEmpty))return 1;
			}
		}

	if(pBoot16->wSignature!=0xAA55)						// Ist es ein MBR
		{
		return 0;
		}

	dwNull	= 0;
	dwCount	= 0;
	qwLast  = pData->aStartSec[uPhysical];
	qwLast += pData->aCountSec[uPhysical];

	memset(aHide,0,sizeof(aHide));


//******************** Untersuche GPT *****************************************


	if(pMbr->sTable[0].bOSType==0xEE)					// Gibt es eine EFI Partition
		{
		dwOffset = pMbr->sTable[0].dwRelativeSector;
		iOk      = DiskReadProc(pData,uPhysical,(unsigned char*)&sGuid,dwOffset,1);

		if(iOk==DPD_OK && memcmp(sGuid.aSignatur,"EFI PART",8)==0)
			{
			dwOffset = (DWORD)sGuid.qwPartitionTab*512;

			for(dwNumber=0;dwNumber<sGuid.dwUsedEntries;dwNumber++,dwOffset+=128)
				{
				if(dwNumber%4==0)
					{
					   iOk = DiskReadProc(pData,uPhysical,(unsigned char*)aList,sGuid.qwPartitionTab+dwNumber/4,1);
					if(iOk!=DPD_OK)break;
					}

				iOff	= dwNumber&3;
														// Prtition aus Datei bernehmen
				if(aList[iOff].qwStartLba==0 && pEntires)
					{
					for(iIdx=0;iIdx<DPD_MAX_LOAD;iIdx++)
						{
						if(pEntires->dwEntry[iIdx]!=dwOffset)continue;
						if(pEntires->qwStart[iIdx]==0)break;
						if(pEntires->qwSize [iIdx]==0)break;

						aList[iOff].qwStartLba = pEntires->qwStart[iIdx];
						aList[iOff].qwEndLba   = pEntires->qwStart[iIdx];
						aList[iOff].qwEndLba  += pEntires->qwSize [iIdx]-1;
						aList[iOff].sPartitionsTyp.Data1 = 0xEBD0A0A2;
						aHide[iOff] = 1;

						break;
						}
					}

				if(aList[iOff].qwStartLba==0)continue;	// Prfe die Partition
				if(aList[iOff].qwEndLba  ==0)continue;
				if(aList[iOff].qwEndLba  <=aList[iOff].qwStartLba)continue;
				if(aList[iOff].qwEndLba  >=qwSectors)continue;
				if(aList[iOff].qwStartLba>=qwSectors)continue;
				if(aList[iOff].sPartitionsTyp.Data1!=0xEBD0A0A2)
				if(aList[iOff].sPartitionsTyp.Data1!=0x44479540)
				if(aList[iOff].sPartitionsTyp.Data1!=0x4F68BCE3)
				if(aList[iOff].sPartitionsTyp.Data1!=0xB921B045)
				if(aList[iOff].sPartitionsTyp.Data1!=0x3B8F8425)
				if(aList[iOff].sPartitionsTyp.Data1!=0x0FC63DAF)continue;

				qwSize	= aList[iOff].qwEndLba;
														// Prfe die Gre
				if(qwSize>qwLast && pData->aStartSec[uPhysical]<qwSize)
					{
					qwLast = qwSize;
					pData->aCountSec[uPhysical] = qwSize-pData->aStartSec[uPhysical];
					}

				qwSize -= aList[iOff].qwStartLba-1;

														// Mounte die Partition
				if(dwNumber==uPartition || uPartition==0xFFFFFFFF)
				if(DiskScanPartition(pData,uPhysical,aList[iOff].qwStartLba,qwSize,uNoEmpty,aHide[iOff] && dwNumber<DPD_MAX_LOAD))
					{
					iAdd++;
					}
				}
			}

		if(iAdd)return iAdd;
		}


//******************** Untersuche MBR *****************************************


	dwPos	= 0;
	dwIndex	= 0;

	for(iPos=0;iPos<iNum && iPos<32;iPos++)
		{
		dwPos	= aNext[iPos];

		if(DiskReadProc(pData,uPhysical,aData,dwPos,1))	// Boot-Sektor einlesen
			{
			return 0;
			}

		   pMbr = (MasterBootRecord*)aData;
		if(pMbr->wSignature!=0xAA55)					// Prfe die Signatur
			{
			if(aTemp[iPos]==0          )continue;
			if(aTemp[iPos]==aNext[iPos])continue;
			aNext[iPos] = aTemp[iPos];
			aTemp[iPos] = 0;
			dwIndex    += 4;
			iPos       -= 1;
			continue;
			}

		if(qwSectors<=0xFFFFFFFF)
				dwEnd = (DWORD)qwSectors;
		else	dwEnd = 0xFFFFFFFF;

		dwOffset = 0x1BE;

		for(iOff=0;iOff<4;iOff++,dwIndex++,dwOffset+=16)// Scanne Tabelleneintrge
			{
			dwCount++;
														// Prtition aus Datei bernehmen
			if(pMbr->sTable[iOff].dwSectors==0 && pEntires && dwIndex<DPD_MAX_LOAD)
				{
				for(iIdx=0;iIdx<DPD_MAX_LOAD;iIdx++)
					{
					if(pEntires->dwEntry[iIdx]!=dwOffset)continue;
					if(pEntires->qwStart[iIdx]==0)break;
					if(pEntires->qwSize [iIdx]==0)break;

					pMbr->sTable[iOff].dwRelativeSector = (DWORD)pEntires->qwStart[iIdx];
					pMbr->sTable[iOff].dwSectors	    = (DWORD)pEntires->qwSize [iIdx];
					pMbr->sTable[iOff].bOSType          = 0x0E;
					aHide       [iOff]                  = 1;

					break;
					}
				}

			if( pMbr->sTable[iOff].dwSectors==0xFFFFFFFF){dwNull++;continue;}
			if(!pMbr->sTable[iOff].dwSectors){dwNull++;continue;}
			if(!pMbr->sTable[iOff].dwRelativeSector)continue;

			qwSize  = dwPos;
			qwSize += pMbr->sTable[iOff].dwRelativeSector;

			if(qwSize<qwLast)							// Prfe die Gre
				{
				   qwSize += pMbr->sTable[iOff].dwSectors;
				if(qwSize>qwLast && qwLast>pData->aStartSec[uPhysical])
					{
					qwLast = qwSize;
					pData->aCountSec[uPhysical] = qwSize-pData->aStartSec[uPhysical];
					}
				}

			if(pMbr->sTable[iOff].bOSType==0x01 ||		// FAT/NTFS/EXT Partitionen
			   pMbr->sTable[iOff].bOSType==0x04 ||
			   pMbr->sTable[iOff].bOSType==0x06 ||
			   pMbr->sTable[iOff].bOSType==0x07 ||
			   pMbr->sTable[iOff].bOSType==0x0B ||
			   pMbr->sTable[iOff].bOSType==0x0C ||
			   pMbr->sTable[iOff].bOSType==0x0E ||
			   pMbr->sTable[iOff].bOSType==0x11 ||
			   pMbr->sTable[iOff].bOSType==0x14 ||
			   pMbr->sTable[iOff].bOSType==0x16 ||
			   pMbr->sTable[iOff].bOSType==0x1B ||
			   pMbr->sTable[iOff].bOSType==0x1C ||
			   pMbr->sTable[iOff].bOSType==0x1E ||
			   pMbr->sTable[iOff].bOSType==0x83)
				{
				   dwOff = pMbr->sTable[iOff].dwRelativeSector+dwPos;
				if(dwOff>=dwEnd)continue;

				   dwMax = pMbr->sTable[iOff].dwSectors;
				if(dwMax+dwOff>dwEnd)
					{
					dwMax = dwEnd-dwOff;
					}
														// Mounte die Partition
				if(dwIndex==uPartition || uPartition==0xFFFFFFFF)
				if(DiskScanPartition(pData,uPhysical,dwOff,dwMax,uNoEmpty,aHide[iOff] && dwIndex<DPD_MAX_LOAD))
					{
					iAdd++;
					}

				if(dwIndex==uPartition)					// Nur eine Partition
					{
					iNum=0;
					break;
					}

				continue;
				}

			if(iNum<32)									// Erweiterte Partitionen
			if(pMbr->sTable[iOff].bOSType==0x05 || pMbr->sTable[iOff].bOSType==0x0F)
				{
				aNext[iNum] = dwPos+pMbr->sTable[iOff].dwRelativeSector;
				aTemp[iNum] = 0;

				if(iOff>0)								// Alternative Position (BUG in Win ???)
					{
					aTemp[iNum]  = dwPos;
					aTemp[iNum] += pMbr->sTable[iOff-1].dwSectors;
					aTemp[iNum] += pMbr->sTable[iOff-1].dwRelativeSector;
					}

				iNum++;
				}
			}
		}



return iAdd;
}

//*****************************************************************************
//*
//*		DiskScanFile
//*
//*****************************************************************************
//	Sucht eine Partition in einer Partitionsdatei
//	iDrive		: Ist das Laufwerk mit der Partitionsdatei (0=A: 1=B: ... 26=Z:)
//	pEntires	: Zeiger auf die Laufwerksdaten
//	Ergibt 0 wenn ok oder eine Fehlernummer kleiner 0
//					 1 = Partition gefunden
//					 0 = Partition nicht gefunden
//					-1 = Falsches Laufwerk
//					-2 = Partitionsdatei nicht gefunden
//					-3 = Paritionen haben das falsche Format
static int DiskScanFile(int iDrive,PartitionLoad *pEntires)
{
#define 	CMP_DWORD(p,n)	(((*(int*)(p))           )^(*(int*)(n)))
#define 	CMP_DW_EX(p,n)	(((*(int*)(p))&0xFFF0FFFF)^(*(int*)(n)))
char		cLine[348]="A:\\Partitions.txt";
QWORD		uValue;
FILE	   *pFile;
unsigned 	uPart;
unsigned 	uFlag;
BYTE		uSign;
int		 	iType;
int			iPos;
int			iRet;



	memset(pEntires,0,sizeof(PartitionLoad));

	if(iDrive<0 || iDrive>=26)						// Prfe die Laufwerksnummer
		{
		return -1;
		}

	cLine[0] += iDrive;

		pFile = fopen(cLine,"rb");
	if(!pFile)return -2;

	cLine[sizeof(cLine)-1] = 0;
	uPart	= 0x8000;
	uFlag	= 0;
	iRet	= 0;

	while(iRet>=0)									// Partitionsdatei zeilen weise laden
		{
		if(!fgets(cLine,sizeof(cLine)-1,pFile))break;

		for(iPos=0;cLine[iPos];iPos++)
			{
			if(cLine[iPos]!=' ' )
			if(cLine[iPos]!='\t')break;
			}

		if(cLine[iPos]=='[')						// Neue Partition definiert
			{
			if(CMP_DWORD(cLine+iPos  ,"[DRI")==0)
			if(CMP_DWORD(cLine+iPos+4,"VES]")==0)
				{
				uPart = 0xFFFF;
				continue;
				}

			if(CMP_DWORD(cLine+iPos  ,"[PAR"))continue;
			if(CMP_DWORD(cLine+iPos+4,"TITI"))continue;
			if(CMP_DW_EX(cLine+iPos+8,"ON0]"))continue;

			uPart = cLine[iPos+10]-'0';

			continue;
			}

		iType = 0;

		if(!CMP_DWORD(cLine+iPos,"SECT"))iType=1;
		if(!CMP_DWORD(cLine+iPos,"SIZE"))iType=2;
		if(!CMP_DWORD(cLine+iPos,"SHOW"))iType=3;

		for(iPos+=4;;iPos++)
			{
			if(cLine[iPos]!=' ' )
			if(cLine[iPos]!='\t')break;
			if(cLine[iPos]=='=' )break;
			}

		if(cLine[iPos]!='=')continue;

		for(iPos++;cLine[iPos];iPos++)
			{
			if(cLine[iPos]!='\t')break;
			if(cLine[iPos]=='=' )break;
			}

		if(cLine[iPos]=='0' && (cLine[iPos+1]=='x' || cLine[iPos+1]=='X'))
			{
			uValue = 0;

			for(iPos+=2;cLine[iPos];iPos++)
				{
				uSign = cLine[iPos];

				if(uSign>='0' && uSign<='9')
					{
					uSign = cLine[iPos]-'0';
					}
				else if(uSign>='A' && uSign<='F')
					{
					uSign = cLine[iPos]-'A'+10;
					}
				else if(uSign>='a' && uSign<='f')
					{
					uSign = cLine[iPos]-'a'+10;
					}
				else{
					break;
					}

				uValue *= 16;
				uValue += uSign;
				}
			}
		else{
			uValue = 0;

			for(;cLine[iPos];iPos++)
				{
				   uSign = cLine[iPos]-'0';
				if(uSign>=10)break;

				uValue *= 10;
				uValue += uSign;
				}
			}

		if(uPart>=DPD_MAX_LOAD)
			{
			if(uPart==0xFFFF)pEntires->qwDriveSize=uValue;
			continue;
			}

		switch(iType)
			{
		case 1:	pEntires->qwStart[uPart] =        uValue;	break;
		case 2:	pEntires->qwSize [uPart] =        uValue;	break;
		case 3:	pEntires->dwEntry[uPart] = (DWORD)uValue;	break;
			}
		}

	fclose(pFile);


return iRet;
}


//*****************************************************************************
//*
//*		DiskMount
//*
//*****************************************************************************
//	Mountet ein Laufwerk mit einer oder mehreren Partitionen.
//	pData		: Zeiger auf die Laufwerksdaten
//	pPathName	: Ist der Pfad der Image-Datei bzw. ein physisches Laufwerk:
//					"PhysicalDrive0"
//					"Physical0"
//					"D:\Physical0"
//					"E:\PhysicalN"
//	uFlags		: Diverse Flags zum laden
//					DPD_FLAG_READONLY 	1	Laufwerk im Readolny-Modus ffnen
//					DPD_FLAG_NO_EMPTY	2	Nur formatierte Laufwerke mounten
//					DPD_FLAG_SINGLE		4	Keine Subpfade bei nur einer Partition
//					DPD_FLAG_INIT		8	Die Laufwerksdaten initialisieren
//	pSlot		: Hier wird der Slot gespeichert in dem das Laufwerk gemountet
//				  wird. Dieser Zeiger kann auch NULL sein. (1..N)
//	uPartition	: Ist die Partition die gemountet werden soll (0xFFFFFFFF=alle)
//	Ergibt die Zahl der gemonteten Partitionen.
int DiskMount(DiskPartionData *pData,LPCTSTR pPathName,DWORD dwFlags,unsigned *pSlot,unsigned uPartition)
{
BYTE			aList[DPD_MAX_PARTIONS];
PartitionLoad	sInfo;
LPCTSTR			pName;
HANDLE			hHandle;
DWORD			dwHigh;
QWORD			qwSize;
UINT			uLetter;
UINT			uDrive;
UINT			uDisk;
UINT			uSign;
BOOL			bFlag;
int				iAdd;
int				iPos;



	if(pSlot)*pSlot=0xFFFFFFFF;

	if(dwFlags&DPD_FLAG_INIT)							// Laufwerksdaten initialisieren
		{
		memset(pData,0,sizeof(DiskPartionData));
		pData->uCurrenrPathLen	= 1;
		pData->aCurrentPath[0]	= '/';
		}

	if(!pPathName || !*pPathName)
		{
		return 0;
		}

	uDisk	= 0xFFFF;
	uDrive	= DiskGetFreeSlot(pData);
	memset(&sInfo,0,sizeof(sInfo));

	if(GetFileAttributes(pPathName)!=0xFFFFFFFF)		// Eine Image-Datei laden
		{
		   hHandle = CreateFile(pPathName,(dwFlags&DPD_FLAG_READONLY)? GENERIC_READ:GENERIC_READ|GENERIC_WRITE,(dwFlags&DPD_FLAG_READONLY)? FILE_SHARE_READ:0,0,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
		if(hHandle==INVALID_HANDLE_VALUE)
			{
			return 0;
			}

		dwHigh	= 0;
		qwSize 	= GetFileSize(hHandle,&dwHigh);
		qwSize |= (QWORD)dwHigh<<32;
														// Datei gre prfen
		if(qwSize<1024 || (qwSize==0xFFFFFFFF && GetLastError()!=NO_ERROR))
			{
			CloseHandle(hHandle);
			return 0;
			}

		pData->aMountFlag[uDrive]   = dwFlags;
		pData->aDrivesPtr[uDrive]	= hHandle;
		pData->aDriveType[uDrive]	= DPD_TYPE_PHYSICAL;
		pData->aDriveMode[uDrive]	= (unsigned char)(DPD_MODE_FILE|(dwFlags&DPD_FLAG_READONLY));
		pData->aCountSec [uDrive]	= qwSize/512;
		pData->aStartSec [uDrive]	= 0;
		pData->aRefCount [uDrive]	= 0;
		pData->uDriveCount		   += 1;
		}
	else{												// Direkter Zugriff auf Laufwerk
		uLetter = 0;
		uDisk	= 0;
		pName	= GetFileName(pPathName);

														// Ist es ein physikalische Laufwerk
		if(!strnicmp(pName,"PhysicalDrive",13) && pName[13]!=0 && (pName[14]==0 || pName[15]==0))
			{
			if(strwchar(pName[13],0x01) || (strwchar(pName[14],0x01) && pName[15]==0))
				{
				uDisk	  = atoi(pPathName+13)|0x8000;
				pPathName = pName;
				}
			else if(strwchar(pName[13],0x06) && pName[14]==0)
				{
				uLetter	  = (pName[13]&~0x20)-'A';
				uDisk     = GetDriveOfVolume(uLetter);
				if(uDisk)pPathName=pName;
				}
			}											// Ist es ein physikalische Laufwerk
		else if(!strnicmp(pName,"Physical",8) && pName[8]!=0 && (pName[9]==0 || pName[10]==0))
			{
			if(strwchar(pName[8],0x01) || (strwchar(pName[9],0x01) && pName[10]==0))
				{
				uDisk	  = atoi(pPathName+8)|0x8000;
				pPathName = pName;
				}
			else if(strwchar(pName[8],0x06) && pName[9]==0)
				{
				uLetter	  = (pName[8]&~0x20)-'A';
				uDisk     = GetDriveOfVolume(uLetter);
				if(uDisk)pPathName=pName;
				}
			}
														// Logisches Laufwerk (z.B.: "D:\")
		if(pPathName[1]==':' && (pPathName[2]==0 || pPathName[3]==0) && (pPathName[2]==0 || pPathName[2]=='\\' || pPathName[2]=='/') && strwchar(pPathName[0],0x06))
			{
			uDisk = (pPathName[0]&~0x20)-'A';
			}
		else if(uDisk==0)								// Falsche Pfadangabe
			{
			return 0;
			}

		if(uLetter)
			{
			DiskScanFile(uLetter,&sInfo);
			}

			hHandle = OpenDevice(uDisk,1);
		if(!hHandle)
			{
			return 0;
			}

		   qwSize = GetSectorCount(hHandle,&dwHigh);
		if(qwSize==0 || dwHigh!=512)					// Prfe die Gre
			{
			CloseDevice(hHandle);
			return 0;
			}

		pData->aMountFlag[uDrive]   = dwFlags;
		pData->aDrivesPtr[uDrive]	= hHandle;
		pData->aDriveType[uDrive]	= DPD_TYPE_PHYSICAL;
		pData->aDriveMode[uDrive]	= (unsigned char)(DPD_MODE_DISK|(dwFlags&DPD_FLAG_READONLY));
		pData->aCountSec [uDrive]	= qwSize;
		pData->aStartSec [uDrive]	= 0;
		pData->aRefCount [uDrive]	= 0;
		pData->aCacheData[uDrive]	= 0;
		pData->uDriveCount		   += 1;

		#if DPD_CACHE_SIZE>0
		pData->aCacheData[uDrive]	= new DiskCache;
		DiskCacheInit((DiskCache*)pData->aCacheData[uDrive]);
		#endif
		}


	if(pSlot)*pSlot=uDrive+1;							// Slot speichern


	for(iPos=0;iPos<DPD_MAX_PARTIONS;iPos++)
		{
		aList[iPos] = pData->aDriveType[iPos];
		}


		iAdd = DiskScanMBR(pData,uDrive,uPartition,dwFlags&DPD_FLAG_NO_EMPTY,&sInfo)+1;
	if(!iAdd)
		{
		DiskUnmount(pData,uDrive);
		return 0;
		}


	if(pData->aDriveMode[uDrive]==DPD_MODE_DISK)		// ReadOnly von Laufwerke entfernen
		{
		CloseDevice(pData->aDrivesPtr[uDrive]);
		pData->aDrivesPtr[uDrive] = NULL;
		bFlag = TRUE;

		if(uPartition>=0 && pData->uPartitionCount<=1)
			{
			for(iPos=0;iPos<DPD_MAX_PARTIONS;iPos++)
				{
				if(pData->aDriveType[iPos]<=DPD_TYPE_EMPTY)continue;
				if(pData->aDriveHide[iPos]==0             )continue;
				bFlag = FALSE;
				break;
				}
			}

			hHandle = OpenDevice(uDisk,2,bFlag);
		if(!hHandle)
			{
			for(iPos=0;iPos<DPD_MAX_PARTIONS;iPos++)	// Mounte neue Partitionen
				{
				if(aList[iPos]            !=DPD_TYPE_NONE )continue;
				if(pData->aDriveType[iPos]<=DPD_TYPE_EMPTY)continue;
				   pData->aDriveType[iPos] =DPD_TYPE_EMPTY;
				}

			DiskUnmount(pData,uDrive);
			return 0;
			}

		pData->aDrivesPtr[uDrive] = hHandle;
		}



	for(iPos=0;iPos<DPD_MAX_PARTIONS;iPos++)			// Mounte neue Partitionen
		{
		if(aList[iPos]            !=DPD_TYPE_NONE )continue;
		if(pData->aDriveType[iPos]<=DPD_TYPE_EMPTY)continue;

		pData->aMountFlag[iPos] = dwFlags;
		pData->aCacheData[iPos]	= pData->aCacheData[uDrive];

		if(DiskMountPartition(pData,uDrive,iPos))
			{
			continue;
			}

		if(dwFlags&DPD_FLAG_NO_EMPTY)					// Partition ganz entfernen
			{
			pData->aCacheData[iPos] = 0;
			pData->aDrivesPtr[iPos]	= 0;
			pData->aDriveMode[iPos]	= DPD_MODE_NONE;
			pData->aDriveType[iPos]	= DPD_TYPE_NONE;
			pData->aDriveHide[iPos]	= 0;
			pData->aRefCount [iPos]	= 0;
			pData->aStartSec [iPos]	= 0;
			pData->aCountSec [iPos]	= 0;
			pData->aDriveSign[iPos] = 0;

			   uSign = pData->aDriveSign[iPos]-1;
			if(uSign<64)pData->aLetterToSlot[uSign] = 0;

			pData->aRefCount[uDrive]--;					// Referenzzhler verringern
			pData->uPartitionCount--;
			pData->uDriveCount--;
			iAdd--;
			}
		else{											// Partition ungltig machen
			pData->aDriveType[iPos] = DPD_TYPE_EMPTY;
			}
		}



	if(dwFlags&DPD_FLAG_SINGLE)							// Keine Subpfade bei nur einer Partition
	if(pData->uDriveCount<=2 && pData->uPartitionCount==1)
		{
		for(iPos=0;iPos<DPD_MAX_PARTIONS;iPos++)
			{
			if(pData->aDriveType[iPos]<=DPD_TYPE_EMPTY)continue;
			pData->uSingleSlot = iPos+1;
			break;
			}
		}


return iAdd;
}


//*****************************************************************************
//*
//*		DiskUnmount
//*
//*****************************************************************************
//	Unmountet ein Laufwerk mit einer oder mehreren Partitionen.
//	pData		: Zeiger auf die Laufwerksdaten
//	uSlot		: Ist die Slotnummer vom Laufwerk (0=alle)
//	Ergibt die Anzahl der nmounteten Partitionen
int DiskUnmount(DiskPartionData *pData,unsigned uSlot)
{
HANDLE		hHandle;
unsigned	uSign;
unsigned	uPos;
unsigned	uRet;


	if(uSlot>DPD_MAX_PARTIONS)return 0;


	uRet = 0;

	if(uSlot==0)										// Alles unmonten
		{
		for(uPos=0;uPos<DPD_MAX_PARTIONS;uPos++)
			{
			if(pData->aDriveType[uPos]!=DPD_TYPE_PHYSICAL)continue;
			uRet += DiskUnmount(pData,uPos+1);
			}

		return uPos;
		}


	uSlot--;

	if(pData->aDriveType[uSlot]!=DPD_TYPE_PHYSICAL)		// Ist es ein phsikalischen Laufwerk
		{
		return 0;
		}

	if(pData->aDriveMode[uSlot]==DPD_MODE_NONE)
		{
		return 0;
		}


	DPD_LOCK(pData);

	hHandle = pData->aDrivesPtr[uSlot];

	for(uPos=0;uPos<DPD_MAX_PARTIONS;uPos++)			// Suche alle logischen Laufwerke
		{
		if(uPos==uSlot)continue;

		if(hHandle!=pData->aDrivesPtr[uPos]          )continue;
		if(pData->aDriveType[uPos]==DPD_TYPE_PHYSICAL)continue;

		switch(pData->aDriveType[uPos])
			{
		#if DPD_HAS_FAT

		case DPD_TYPE_FAT:								// FAT Parition
			{
			FatEntry   *pEntry;
			FIL		   *pFile;
			DIRFIND    *pFind;
			unsigned	uItem;

			pEntry	= (FatEntry*)pData->aLocalData[uPos];

			for(uItem=0;uItem<FAT_MAX_FILE;uItem++)
				{
					pFile = pEntry->aFileList[uItem];
				if(!pFile)continue;
				f_close(&pEntry->sFatData,pFile);
				FatFileFree(pEntry,DPU_LIST_TO_HANDLE(DPD_TYPE_FAT,uPos,uItem));
				}

			for(uItem=0;uItem<FAT_MAX_FIND;uItem++)
				{
					pFind = pEntry->aFindList[uItem];
				if(!pFind)continue;
				//f_closedir(&pEntry->sFatData,&pFind->sData);
				FatFindFree(pEntry,DPU_LIST_TO_HANDLE(DPD_TYPE_FAT,uPos,uItem+0x1000));
				}

			memset(pEntry,0,sizeof(FatEntry));
			delete pEntry;
			break;
			}

		#endif

		#if DPD_HAS_EXFAT

		case DPD_TYPE_EXFAT:							// EXFAT Parition
			{
			ExFatEntry *pEntry;
			ExFatFind  *pItem;
			exfat_node *pNode;
			unsigned	uItem;

			pEntry	= (ExFatEntry*)pData->aLocalData[uPos];

			for(uItem=0;uItem<EXFAT_MAX_FILE;uItem++)
				{
					pNode = pEntry->aFileList[uItem];
				if(!pNode)continue;

				exfat_flush_node(&pEntry->sExFat,pNode);
				exfat_put_node  (&pEntry->sExFat,pNode);
				ExFatFileFree(pEntry,DPU_LIST_TO_HANDLE(DPD_TYPE_EXFAT,uPos,uItem));
				}

			for(uItem=0;uItem<EXFAT_MAX_FIND;uItem++)
				{
					pItem = pEntry->aFindList[uItem];
				if(!pItem)continue;

				exfat_closedir(&pEntry->sExFat,&pItem->sNext);
				exfat_put_node(&pEntry->sExFat, pItem->pNode);
				ExFatFindFree(pEntry,DPU_LIST_TO_HANDLE(DPD_TYPE_EXFAT,uPos,uItem+0x1000));
				}

			exfat_unmount(&pEntry->sExFat);

			memset(pEntry,0,sizeof(ExFatEntry));
			delete pEntry;
			break;
			}

		#endif

		#if DPD_HAS_EXT

		case DPD_TYPE_EXT:								// EXT?-Partition
			{
			ExtEntry   *pEntry;
			ext4_file  *pFile;
			ext4_find  *pFind;
			unsigned	uItem;

			pEntry	= (ExtEntry*)pData->aLocalData[uPos];

			for(uItem=0;uItem<EXT_MAX_FILE;uItem++)
				{
					pFile = pEntry->aFileList[uItem];
				if(!pFile)continue;
				ext4_fclose(pFile);
				ExtFileFree(pEntry,DPU_LIST_TO_HANDLE(DPD_TYPE_EXT,uPos,uItem));
				}

			for(uItem=0;uItem<EXT_MAX_FIND;uItem++)
				{
					pFind = pEntry->aFindList[uItem];
				if(!pFind)continue;
				ext4_dir_close(&pFind->sData);
				ExtFindFree(pEntry,DPU_LIST_TO_HANDLE(DPD_TYPE_EXT,uPos,uItem+0x1000));
				}

			ext4_cache_write_back(pEntry->hHandle,0);
			ext4_journal_stop    (pEntry->hHandle);
			ext4_umount          (pEntry->hHandle);

			memset(pEntry,0,sizeof(ExtEntry));
			delete pEntry;

			break;
			}

		#endif

		case DPD_TYPE_EMPTY:

			break;
			}

		uSign	= pData->aDriveSign[uPos]-1;

		pData->aLocalData[uPos]	= NULL;					// Logisches Laufwerk aushngen
		pData->aCacheData[uPos] = NULL;
		pData->aDrivesPtr[uPos]	= NULL;
		pData->aDriveType[uPos]	= DPD_TYPE_NONE;
		pData->aDriveMode[uPos]	= DPD_MODE_NONE;
		pData->aDriveHide[uPos]	= 0;
		pData->aDriveSign[uPos]	= 0;
		pData->aRefCount [uPos]	= 0;
		pData->aStartSec [uPos]	= 0;
		pData->aCountSec [uPos]	= 0;
		pData->uDriveCount	   -= 1;

		if(uPos+1==pData->uCurrenrSlot)					// Aktueller Pfad rcksetzen
			{
			pData->uCurrenrSlot		= 0;
			pData->uCurrenrPathLen	= 1;
			pData->aCurrentPath[0]	= '/';
			pData->aCurrentPath[1]	= 0;
			pData->aCurrentPath[2]	= 0;
			pData->aCurrentPath[3]	= 0;
			}

		pData->aRefCount[uSlot]--;
		pData->aLetterToSlot[uSign] = 0;

		uRet++;
		}


	switch(pData->aDriveMode[uSlot])					// Handles schlieen
		{
	case DPD_MODE_FILE:
	case DPD_MODE_FILE_RO:

		CloseHandle(pData->aDrivesPtr[uSlot]);
		break;

	case DPD_MODE_DISK:
	case DPD_MODE_DISK_RO:

		CloseDevice(pData->aDrivesPtr[uSlot]);
		break;
		}

	#if DPD_CACHE_SIZE>0

	if(pData->aCacheData[uSlot])						// Cache lschen
		{
		delete pData->aCacheData[uSlot];
		pData->aCacheData[uSlot] = NULL;
		}

	#endif


	pData->aLocalData[uSlot] = NULL;					// Phsikalischen Laufwerk aushngen
	pData->aDrivesPtr[uSlot] = NULL;
	pData->aCacheData[uSlot] = NULL;
	pData->aDriveType[uSlot] = DPD_TYPE_NONE;
	pData->aDriveMode[uSlot] = DPD_MODE_NONE;
	pData->aDriveHide[uSlot] = 0;
	pData->aDriveSign[uSlot] = 0;
	pData->aRefCount [uSlot] = 0;
	pData->aStartSec [uSlot] = 0;
	pData->aCountSec [uSlot] = 0;
	pData->uDriveCount	    -= 1;


	if(pData->uDriveCount==0)
		{
		pData->uSingleSlot = 0;
		}

	DPD_UNLOCK(pData);



return uRet+1;
}

//*****************************************************************************
//*
//*		DiskConvertPath
//*
//*****************************************************************************
//	Konvertiert einen Pfad unter Bercksichtigung des aktuellen Pfades
//	pData		: Zeiger auf die Laufwerksdaten
//	pPath		: Pfad der konvertiert werden soll
//	pTarget		: Zielpuffer fr den Pfad
//	iSize		: Gre des Zielpuffers in Zeichen
//	iMode		: In was soll konvertiert werden (0=Auto 1=ANSI 2=UTF8 3=UTF16)
//	pSize		: Hier wird die Lnge des Pfades sgespeichert. (Wenn nicht 0)
//	Ergibt den Laufwerks-Slot+1 zum Pfad bzw. eine Fehlernummer kleiner 0,oder 0 fr Root.
//					-X = Fehlernummer
//					 0 = Root-Pfad ohne Laufwerksangabe
//					 1 = Pfad mit Laufwerk ("/0" "0:..." )
static int DiskConvertPath(DiskPartionData *pData,const DCHAR *pPath,void *pTarget,int iSize,int iMode=DPD_AUTO,int *pSize=0)
{
const DCHAR	   *pNext;
wchar_t 	   *pWort;
char 		   *pByte;
unsigned		uType;
unsigned 		uUnicode;
int				iLetter;
int				iDrive;
int				iFlag;
int				iLen;
int				iPos;


	if(iSize<4)return DPD_ERR_PARAM;

	iFlag	= 0;

	if(pPath[0]=='/' || pPath[0]=='\\')					// Absoluter Pfad "/...."
		{
		iFlag = 1;
		pPath++;
		}
	else if(pPath[0] && pPath[1]==':')					// Absoluter Pfad "D:..."
		{
		iFlag = 1;
		}
	else if(pData->uCurrenrPathLen<=1)					// Kein aktuelle Pfad definiert
		{
		iFlag = 1;
		}


	if(iFlag)											// Absoluten Pfad auswerten
		{
		if(pData->uSingleSlot)
			{
			iDrive = pData->uSingleSlot-1;

			if((unsigned)iDrive>=DPD_MAX_PARTIONS)
				{
				return DPD_ERR_PATH_NOT_FOUND;
				}

			if(pData->aDriveType[iDrive]<DPD_TYPE_EMPTY)
				{
				return DPD_ERR_PATH_NOT_FOUND;
				}

			iDrive++;
			}
		else{
			   iLetter = pPath[0];
			if(iLetter>='a')iLetter&=~0x20;

			if(iLetter==0)								// Root Pfad "/"
				{
				memcpy(pTarget,"/\0\0\0\0\0\0",8);
				return 0;
				}

			iLetter -= '0';
														// Lauferks-Slot von Buchstaben
			if((unsigned)iLetter>sizeof(pData->aLetterToSlot))
				{
				return DPD_ERR_PATH_NOT_FOUND;
				}

			pPath++;

			if(pPath[0]==':')
				{
				pPath++;
				}

			if(pPath[0]=='/' || pPath[0]=='\\')
				{
				pPath++;
				}
			else if(pPath[0])
				{
				return DPD_ERR_PATH_NOT_FOUND;
				}

			iDrive = pData->aLetterToSlot[iLetter];
			}

		pNext  = pPath;
		pPath  = NULL;
		}
	else{												// Relativer Pfad
		iDrive = pData->uCurrenrSlot;
		pNext  = pData->aCurrentPath;

		if(pData->uSingleSlot==0)
				pNext+=3;
		else	pNext+=1;
		}

	if(iDrive<=0)
		{
		return DPD_ERR_PATH_NOT_FOUND;
		}


	if(iMode==DPD_AUTO)									// Automatischer Modus
		{
		   uType = pData->aDriveType[iDrive-1];
		if(uType>=sizeof(aTypeToChar))
			{
			return DPD_ERR_PATH_NOT_FOUND;
			}

		iMode = aTypeToChar[uType];
		}


	switch(iMode)										// Automatischer Modus
		{
	case DPD_UTF16:										// Pfad als UTF16 ausgeben

			pWort	= (wchar_t*)pTarget;
			iSize  -= 2;

			for(iPos=0;;iPos+=iLen)
				{
				if(iPos>=iSize)return DPD_ERR_INVALID_FILENAME;

				#if   defined(DISK_USEUTF16)
				pNext += utf16tochar(pNext,uUnicode);
				#elif defined(DISK_USEUTF8)
				pNext += utf8tochar(pNext,uUnicode);
				#else
				pNext += ansitochar(pNext,uUnicode);
				#endif

				if(!uUnicode)
					{
					if(!pPath)break;
					pNext = pPath;
					pPath = NULL;
					iLen  = 0;
					continue;
					}

				if(uUnicode=='\\')uUnicode='/';

				iLen = chartoutf16(pWort+iPos,uUnicode);
				}

			pWort[iPos] = 0;
			break;

	case DPD_UTF8:										// Pfad als UTF8 ausgeben

			pByte	= (char*)pTarget;
			iSize  -= 5;

			for(iPos=0;;iPos+=iLen)
				{
				if(iPos>=iSize)return DPD_ERR_INVALID_FILENAME;


				#if   defined(DISK_USEUTF16)
				pNext += utf16tochar(pNext,uUnicode);
				#elif defined(DISK_USEUTF8)
				pNext += utf8tochar(pNext,uUnicode);
				#else
				pNext += ansitochar(pNext,uUnicode);
				#endif

				if(!uUnicode)
					{
					if(!pPath)break;
					pNext = pPath;
					pPath = NULL;
					iLen  = 0;
					continue;
					}

				if(uUnicode=='\\')uUnicode='/';

				iLen = chartoutf8(pByte+iPos,uUnicode);
				}

			pByte[iPos] = 0;
			break;

	case DPD_ANSI:										// Pfad als ANSI ausgeben

			pByte	= (char*)pTarget;
			iSize  -= 2;

			for(iPos=0;;iPos++)
				{
				if(iPos>=iSize)return DPD_ERR_INVALID_FILENAME;

				#if   defined(DISK_USEUTF16)
				pNext += utf16tochar(pNext,uUnicode);
				#elif defined(DISK_USEUTF8)
				pNext += utf8tochar(pNext,uUnicode);
				#else
				pNext += ansitochar(pNext,uUnicode);
				#endif

				if(!uUnicode)
					{
					if(!pPath)break;
					pNext = pPath;
					pPath = NULL;
					iPos -= 1;
					continue;
					}

				if(uUnicode=='\\')
						pByte[iPos] = '/';
				else if(uUnicode<255)
						pByte[iPos] = uUnicode;
				else	pByte[iPos] = '\xFF';
				}

			pByte[iPos] = 0;
			break;

	default:return DPD_ERR_PARAM;
		}


	if(pSize)*pSize=iPos;



return iDrive;
}

//*****************************************************************************
//*
//*		DiskFillName
//*
//*****************************************************************************
//	Konvertiert einen Namen ins aktuelle Pfad-Format (DCHAR)
//	pData		: Zeiger auf die Laufwerksdaten
//	pSource		: Name der konvertiert werden soll
//	pBuffer		: Zielpuffer fr den Name
//	iSize		: Gre des Zielpuffers in Zeichen
//	iMode		: Welches Format hat die Quelle (0=Auto 1=ANSI 2=UTF8 3=UTF16)
//	pSize		: Hier wird die Lnge des Textes sgespeichert. (Wenn nicht 0)
//	Ergibt die Lnge des Namens bzw. -1 bei einem Fehler
static int DiskFillName(const void *pSource,DCHAR *pBuffer,int iSize,int iMode,int *pSize)
{
unsigned 		uUnicode;
const wchar_t  *pWort;
const char     *pByte;
int				iLen;
int				iPos;
int				iOut;


	if(iSize<=0)										// Prfe die Gre
		{
		if(iSize>0)pBuffer[0]=0;
		return -1;
		}


	#if   defined(DISK_USEUTF16)
	iSize  -= 2;
	#elif defined(DISK_USEUTF8)
	iSize  -= 4;
	#else
	iSize  -= 1;
	#endif


	iOut	= 0;

	switch(iMode)										// Automatischer Modus
		{
	case DPD_UTF16:										// Pfad als UTF16 eingeben

			pWort	= (wchar_t*)pSource;

			for(iPos=0;;iPos+=iLen)
				{
				if(iOut>=iSize)
					{
					pBuffer[0]=0;
					return -1;
					}

				iLen = utf16tochar(pWort+iPos,uUnicode);

				if(!uUnicode)break;

				#if   defined(DISK_USEUTF16)
				iOut += chartoutf16(pBuffer+iOut,uUnicode);
				#elif defined(DISK_USEUTF8)
				iOut += chartoutf8(pBuffer+iOut,uUnicode);
				#else
				iOut += chartoansi(pBuffer+iOut,uUnicode);
				#endif
				}

			break;

	case DPD_UTF8:										// Pfad als UTF8 eingeben

			pByte	= (char*)pSource;

			for(iPos=0;;iPos+=iLen)
				{
				if(iOut>=iSize)
					{
					pBuffer[0]=0;
					return -1;
					}

				iLen = utf8tochar(pByte+iPos,uUnicode);

				if(!uUnicode)break;

				#if   defined(DISK_USEUTF16)
				iOut += chartoutf16(pBuffer+iOut,uUnicode);
				#elif defined(DISK_USEUTF8)
				iOut += chartoutf8(pBuffer+iOut,uUnicode);
				#else
				iOut += chartoansi(pBuffer+iOut,uUnicode);
				#endif
				}

			break;

	case DPD_ANSI:										// Pfad als ANSI eingeben

			pByte	= (char*)pSource;

			for(iPos=0;;iPos+=iLen)
				{
				if(iOut>=iSize)
					{
					pBuffer[0]=0;
					return -1;
					}

				iLen = ansitochar(pByte+iPos,uUnicode);

				if(!uUnicode)break;

				#if   defined(DISK_USEUTF16)
				iOut += chartoutf16(pBuffer+iOut,uUnicode);
				#elif defined(DISK_USEUTF8)
				iOut += chartoutf8(pBuffer+iOut,uUnicode);
				#else
				iOut += chartoansi(pBuffer+iOut,uUnicode);
				#endif
				}

			break;

	default:return -1;
		}


	pBuffer[iPos] = 0;
	if(pSize)*pSize=iPos;


return iPos;
}

//*****************************************************************************
//*
//*		DiskSetCurrentPath
//*
//*****************************************************************************
//	Stellt den aktuellen Pfad ein
//	pFilename		: Ist der Dateiname fr den aktuellen Pfad.
//	Ergibt 0 wenn ok, bzw. einen Fehlercode kleiner 0.
int DiskSetCurrentPath(DiskPartionData *pData,const DCHAR *pFilename)
{
DCHAR	aBuffer[DPD_MAX_PATH];
int		iDrive;
int		iLen;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH-4,DPD_BASE,&iLen);
	if(iDrive<0)
		{
		return iDrive;
		}

	if(iDrive==0)											// Root Verzeichnis "/"
		{
		pData->uCurrenrSlot		= iDrive;
		pData->uCurrenrPathLen	= 1;
		pData->aCurrentPath[0]	= '/';
		pData->aCurrentPath[1]	= 0;
		pData->aCurrentPath[2]	= 0;
		pData->aCurrentPath[3]	= 0;
		}
	else{
		pData->uCurrenrSlot		= iDrive;
		pData->uCurrenrPathLen	= iLen+3;
		pData->aCurrentPath[0]	= '/';
		pData->aCurrentPath[1]	= pData->aDriveSign[iDrive-1]+'0'-1;
		pData->aCurrentPath[2]	= '/';

		memcpy(pData->aCurrentPath+3,aBuffer,(iLen+1)*sizeof(DCHAR));
		}


return DPD_ERR_NONE;
}

//*****************************************************************************
//*
//*		DiskGetCurrentPath
//*
//*****************************************************************************
//	Holt den aktuellen Pfad
//	pData		: Zeiger auf die Laufwerksdaten
//	pBuffer		: Puffer fr den Pfad
//	iSize		: Gre des Puffers in Zeichen
//	Ergibt die Lnge des Pfades bzw. eine Fehlernummer kleiner 0.
int DiskGetCurrentPath(DiskPartionData *pData,DCHAR *pBuffer,int iSize)
{
DCHAR	   *pPath;


	if(iSize<=(int)pData->uCurrenrPathLen)
		{
		if(iSize>0)pBuffer[0]=0;
		return DPD_ERR_PARAM;
		}

	iSize = pData->uCurrenrPathLen;
	pPath = pData->aCurrentPath;

	if(pData->uSingleSlot)
		{
		iSize -= 2;
		pPath += 2;

		if(iSize<0)iSize=0;
		}

	memcpy(pBuffer,pPath,(iSize*1)*sizeof(DCHAR));
	pBuffer[iSize] = 0;


return iSize;
}

//*****************************************************************************
//*
//*		DiskFullPath
//*
//*****************************************************************************
//	Holt den aktuellen Pfad
//	pData		: Zeiger auf die Laufwerksdaten
//	pBuffer		: Puffer fr den Pfad
//	iSize		: Gre des Puffers in Zeichen
//	Ergibt die Lnge des Pfades bzw. eine Fehlernummer kleiner 0.
int DiskFullPath(DiskPartionData *pData,const DCHAR *pFilename,DCHAR *pBuffer,int iSize)
{
int		iDrive;
int		iLen;


	if(iSize<=4)
		{
		if(iSize>0)pBuffer[0]=0;
		return DPD_ERR_PARAM;
		}

	   iDrive = DiskConvertPath(pData,pFilename,pBuffer+3,iSize-3,DPD_BASE,&iLen);
	if(iDrive<0)
		{
		return iDrive;
		}

	if(iDrive==0)											// Root Verzeichnis "/"
		{
		pBuffer[0]	= '/';
		pBuffer[1]	= 0;

		return 1;
		}
	else{
		pBuffer[0]	= '/';
		pBuffer[1]	= pData->aDriveSign[iDrive-1]+'0'-1;
		pBuffer[2]	= '/';

		if(iLen==0)
			{
			if(pFilename[0]==0 || pFilename[1]==0 || pFilename[2]==0)
				{
				pBuffer[2]	= 0;
				return 2;
				}
			}
		}


return iLen+3;
}


//*****************************************************************************
//*
//*		DiskFileOpenEx
//*
//*****************************************************************************
//	ffnet eine Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name der Datei.
//	iMode		: Bestimmt wie die Datei geffnet werden soll.
//						DPD_READ	 = Datei zum Lesen ffnen
//						DPD_WRITE	 = Datei zum Schreiben ffnen und nicht auf 0 krzen
//						DPD_CREATE	 = Datei zum Schreiben ffnen und auf 0 krzen
//						DPD_TRUNC	 = Datei auf 0 krzen
//						DPD_HIDDEN	 = Datei versteckt erstellen
//						DPD_ARCHIVE  = Archive Attribut setzen
//	pError		: Hier wird die Fehlernummer gespeichert wenn der Zeiger
//				  ungleich NULL ist.
//	Ergibt ein File-Handle bzw. 0 bei einem Fehler.
DiskFile DiskFileOpenEx(DiskPartionData *pData,const DCHAR *pFilename,int iMode,int *pError)
{
wchar_t		aBuffer[DPD_MAX_PATH];
DiskFile	pHandle;
void	   *pLocal;
int			iDrive;
int			iError;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH);
	if(iDrive<=0)
		{
		if(pError)
			{
			if(!iDrive)iDrive=DPD_ERR_PATH_NOT_FOUND;
			*pError = iDrive;
			}

		return NULL;
		}

	iDrive--;

	if(pData->aDriveMode[iDrive]&1)							// Partition ist ReadOnly
		{
		if(iMode&(DPD_CREATE|DPD_WRITE))
			{
			*pError = DPD_ERR_ACCESS_DENIED;
			return NULL;
			}
		}

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;


	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		BYTE		ucMode;
		FRESULT		iRet;

		DPD_LOCK(pData);

			pHandle = FatFileAlloc(pLocal,iDrive);			// Handle holen
		if(!pHandle)
			{
			DPD_UNLOCK(pData);
			iError  = DPD_ERR_NO_HANDLES;
			break;
			}

		if(iMode&DPD_WRITE)									// Schreibrechte
			{
			ucMode = (iMode&DPD_TRUNC)? FA_CREATE_ALWAYS|FA_WRITE:FA_OPEN_EXISTING|FA_WRITE;
			}
		else{
			ucMode = FA_OPEN_EXISTING;
			}

		if(iMode&DPD_CREATE)                                // Einen neue Datei erstellen
			{
			ucMode = FA_CREATE_ALWAYS|FA_WRITE;
			}

		if(iMode&DPD_READ)									// Leserechte
			{
			ucMode |= FA_READ;
			}

		pFilePtr = FatFileGet(pEntry,pHandle);
		iRet	 = f_open(&pEntry->sFatData,pFilePtr,aBuffer,ucMode);

		if(iRet!=FR_OK)										// Fehler beim ffnen
			{
			if(pError)iError=FatError(iRet);
			FatFileFree(pEntry,pHandle);
			pHandle = 0;
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iRet;

		DPD_LOCK(pData);

		if(pEntry->uOpenCnt>=256)							// Noted flushen
			{
			pEntry->uOpenCnt = 0;
			exfat_flush_nodes  (&pEntry->sExFat);
			exfat_cache_release(&pEntry->sExFat,12);
			}

		   iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
		if(iRet!=0)
			{
			if(iMode&DPD_CREATE)							// Neue Datei erstellen
				{
				iRet   = exfat_mknod(&pEntry->sExFat,(char*)aBuffer);
				iMode &= ~(DPD_TRUNC|DPD_CREATE);
				if(iRet==0)
					{
					iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
					}
				}

			if(iRet!=0)iError=ExFatError(iRet);
			}
		else if(pNode->flags&EXFAT_ATTRIB_DIR)				// Ist es ein Verzeichniss
			{
			exfat_put_node(&pEntry->sExFat,pNode);
			iError = DPD_ERR_FILE_NOT_FOUND;
			}
		else if((pNode->flags&EXFAT_ATTRIB_RO) && (iMode&DPD_WRITE))
			{
			iError = DPD_ERR_ACCESS_DENIED;
			exfat_put_node(&pEntry->sExFat,pNode);
			}

		if(!iError)											// Handle holen
			{
			if(!(iMode&DPD_WRITE))iMode&=~DPD_TRUNC;

			if(iMode&(DPD_TRUNC|DPD_CREATE|DPD_WRITE))
				{
				pEntry->uOpenCnt++;
				}

			if(iMode&(DPD_TRUNC|DPD_CREATE))				// Datei auf 0 krzen
				{
				exfat_truncate  (&pEntry->sExFat,pNode,0,true);
				exfat_flush_node(&pEntry->sExFat,pNode);
				exfat_put_node  (&pEntry->sExFat,pNode);
				}


				pHandle = ExFatFileAlloc(pLocal,iDrive,pNode,iMode);
			if(!pHandle)
				{
				iError=DPD_ERR_NO_HANDLES;
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT?-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pNode;
		int			iFlag;
		int			iTurn;
		int			iRet;

		DPD_LOCK(pData);

			pHandle = ExtFileAlloc(pLocal,iDrive);			// Handle holen
		if(!pHandle)
			{
			DPD_UNLOCK(pData);
			iError  = DPD_ERR_NO_HANDLES;
			break;
			}

		if(iMode&DPD_WRITE)									// Schreibrechte
			{
			iFlag = (iMode&DPD_TRUNC)? O_RDWR|O_TRUNC|O_CREAT:O_RDWR;
			}
		else{
			iFlag = O_RDONLY;
			}

		if(iMode&DPD_CREATE)                                // Einen neue Datei erstellen
			{
			iFlag = O_RDWR|O_TRUNC|O_CREAT;
			}

		iTurn = 0;
		pNode = ExtFileGet (pEntry,pHandle);
Retry:	iRet  = ext4_fopen2(pEntry->hHandle,pNode,(char*)aBuffer,iFlag);

		if(iRet==EMLINK)									// Links auflsen
			{
			   iRet = ExtResolvePath(pEntry->hHandle,(char*)aBuffer,pNode->inode);
			if(iRet==EOK)
				{
				iTurn++;
				if(iTurn<4)goto Retry;
				iRet = EMLINK;
				}
			}

		if(iRet!=EOK)										// Fehler beim ffnen
			{
			if(pError)iError=ExtError(iRet);
			ExtFileFree(pEntry,pHandle);
			pHandle = 0;
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PATH_NOT_FOUND;
		pHandle = NULL;
		break;
		}


	if(pError)*pError=iError;


return pHandle;
}


//*****************************************************************************
//*
//*		DiskFileClose
//*
//*****************************************************************************
//	Schliet eine geffnete Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	Ergibt 0 wenn die Datei geschlossen werden konnte, bzw. einen Fehlercode kleiner 0.
int DiskFileClose(DiskPartionData *pData,DiskFile hFile)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}



	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		FRESULT		iRet;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = f_close(&pEntry->sFatData,pFilePtr);
			FatFileFree(pEntry,hFile);

			if(iRet!=FR_OK)									// Fehler beim Schlieen
				{
				iError = FatError(iRet);
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			exfat_flush_node(&pEntry->sExFat,pNode);
			exfat_put_node  (&pEntry->sExFat,pNode);
			ExFatFileFree(pEntry,hFile);
															// Gecachte Nodes freigeben
			if(pEntry->sExFat.cache_dirs>8 && pEntry->sExFat.cache_dirs%8==0)
				{
				pEntry->uOpenCnt = 0;
				exfat_flush_nodes  (&pEntry->sExFat);
				exfat_cache_release(&pEntry->sExFat,12);
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pFilePtr;
		int 		iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = ext4_fclose(pFilePtr);
			ExtFileFree(pEntry,hFile);

			if(iRet!=EOK)									// Fehler beim Schlieen
				{
				iError = ExtError(iRet);
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



return iError;
}


//*****************************************************************************
//*
//*		DiskFileSeekEx
//*
//*****************************************************************************
//	Verschiet die Lese/Schreibposition in einer Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	iPos		: Neue Lese/Schreibposition
//	iMode		: Bestimmt wie die Position bestimmt werden soll.
//					SYS_SEEK_SET	0	Absolute Position
//					SYS_SEEK_CUR	1	Relativ zur aktuellen Position
//					SYS_SEEK_END	2	Relativ zum Datei Ende
//	Ergibt die neue Position in der Datei.
DiskPos DiskFileSeekEx(DiskPartionData *pData,DiskFile hFile,DiskPos iPos,int iMode)
{
void	   *pLocal;
unsigned	uSlot;
unsigned	uType;




	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}



	pLocal = pData->aLocalData[uSlot];


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		DWORD		dwPos;
		FRESULT		iRet;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else if(iPos<-0x7FFFFFFF || iPos>0x7FFFFFFF)		// Prfe die Position
			{
			iPos = DPD_ERR_PARAM;
			}
		else{												// Berechne die Zielposition
			switch(iMode)
				{
			case 1:	dwPos = f_tell(pFilePtr)+(int)iPos;	break;
			case 2:	dwPos = f_size(pFilePtr)+(int)iPos;	break;
			default:dwPos = (int)iPos;					break;
				}

			if(dwPos&0x80000000)							// Prfe den Bereich
					iRet = FR_PARAM;
			else	iRet = f_lseek(&pEntry->sFatData,pFilePtr,dwPos);

			if(iRet!=FR_OK)									// Fehler beim Verschieben
					iPos = FatError(iRet);
			else	iPos = dwPos;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		unsigned	uItem;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else{
			uItem = DPU_ITEM_OF_HANDLE(hFile);

			switch(iMode)
				{
			case 1:	iPos = pEntry->aFilePos[uItem]+iPos;	break;
			case 2:	iPos = pNode->size            +iPos;	break;
				}

			if(iPos<0)
				{
				iPos=DPD_ERR_PARAM;
				}
			else{
				pEntry->aFilePos[uItem]=iPos;
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pFilePtr;
		int 		iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else{												// Berechne die Zielposition
			iRet = ext4_fseek(pFilePtr,iPos,iMode);

			if(iRet!=EOK)									// Fehler beim Verschieben
					iPos = ExtError(iRet);
			else	iPos = ext4_ftell(pFilePtr);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iPos = DPD_ERR_PARAM;
		break;
		}



return iPos;
}

//*****************************************************************************
//*
//*		DiskFileRead
//*
//*****************************************************************************
//	Liest Daten aus einer Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	pData		: Zeiger auf den Lesepuffer.
//	iSize		: Ist die Anzahl der zuladenden Bytes
//	Ergibt die Anzahl der geladenen Bytes bzw. einen Fehlercode kleiner 0.
int DiskFileRead(DiskPartionData *pData,DiskFile hFile,void *pBuffer,int iSize)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}

	if(iSize<0)
		{
		return DPD_ERR_PARAM;
		}


	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		UINT		uByte;
		FRESULT		iRet;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = f_read(&pEntry->sFatData,pFilePtr,pBuffer,iSize,&uByte);

			if(iRet!=FR_OK)									// Fehler beim Lesen
					iError = FatError(iRet);
			else	iError = uByte;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		unsigned	uItem;
		int			iRet;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			uItem = DPU_ITEM_OF_HANDLE(hFile);
			iRet  = exfat_generic_pread(&pEntry->sExFat,pNode,pBuffer,iSize,pEntry->aFilePos[uItem]);

			if(iRet>=0)
				{
				pEntry->aFilePos[uItem]+=iRet;
				iError=iRet;
				}
			else{
				iError=ExFatError(iRet);
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pFilePtr;
		size_t		iByte;
		int 		iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = ext4_fread(pFilePtr,pBuffer,iSize,&iByte);

			if(iRet!=EOK)									// Fehler beim Lesen
					iError = ExtError(iRet);
			else	iError = (int)iByte;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



return iError;
}

//*****************************************************************************
//*
//*		DiskFileWrite
//*
//*****************************************************************************
//	Schreibt Daten in eine Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	pData		: Zeiger auf den Lesepuffer.
//	iSize		: Ist die Anzahl der zuladenden Bytes
//	Ergibt die Anzahl der geladenen Bytes bzw. einen Fehlercode kleiner 0.
int DiskFileWrite(DiskPartionData *pData,DiskFile hFile,const void *pBuffer,int iSize)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}

	if(iSize<0)
		{
		return DPD_ERR_PARAM;
		}


	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		UINT		uByte;
		FRESULT		iRet;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = f_write(&pEntry->sFatData,pFilePtr,pBuffer,iSize,&uByte);

			if(iRet!=FR_OK)									// Fehler beim Schlieen
					iError = FatError(iRet);
			else	iError = uByte;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		unsigned	uItem;
		int			iRet;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else if(pNode->flags&EXFAT_ATTRIB_RO)
			{
			iError = DPD_ERR_ACCESS_DENIED;
			}
		else{
			uItem = DPU_ITEM_OF_HANDLE(hFile);

			if(pEntry->aFileFlag[uItem]&DPD_WRITE)			// Schreiberlaubnis vor handen
				{
				iRet  = exfat_generic_pwrite(&pEntry->sExFat,pNode,pBuffer,iSize,pEntry->aFilePos[uItem]);

				if(iRet>=0)
					{
					pEntry->aFilePos[uItem]+=iRet;
					iError=iRet;
					}
				else{
					iError=ExFatError(iRet);
					}
				}
			else{
				iError = DPD_ERR_ACCESS_DENIED;
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pFilePtr;
		size_t		iByte;
		int 		iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = ext4_fwrite(pFilePtr,pBuffer,iSize,&iByte);

			if(iRet!=EOK)									// Fehler beim Schreiben
					iError = ExtError(iRet);
			else	iError = (int)iByte;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



return iError;
}


//*****************************************************************************
//*
//*		DiskFileFlush
//*
//*****************************************************************************
//	Leert alle Zwischenpuffer einer Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	Ergibt 0 wenn die Puffer geleert werden konnten, bzw. eine Fehlernummer kleiner 0.
int DiskFileFlush(DiskPartionData *pData,DiskFile hFile)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}


	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		FRESULT		iRet;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = f_sync(&pEntry->sFatData,pFilePtr);

			if(iRet!=FR_OK)									// Fehler beim Schlieen
				{
				iError = FatError(iRet);
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iRet;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			   iRet  = exfat_flush_node(&pEntry->sExFat,pNode);
			if(iRet<0)
					iError=ExFatError(iRet);
			else	exfat_fsync(pEntry->sExFat.dev);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pFilePtr;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			ext4_block_cache_write_back(&pEntry->sDevice,1);
			ext4_block_cache_write_back(&pEntry->sDevice,0);
			iError = DPD_ERR_NONE;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}


	if(!iError)DiskIoctlProc(pData,DPU_SLOT_OF_HANDLE(hFile),DPD_CTRL_SYNC,NULL);



return iError;
}

//*****************************************************************************
//*
//*		DiskFileTellEx
//*
//*****************************************************************************
//	Liet den aktuellen Dateizeiger aus.
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	Ergibt den aktuellen Dateizeiger, bzw. eine Fehlernummer kleiner 0.
DiskPos	DiskFileTellEx(DiskPartionData *pData,DiskFile hFile)
{
void	   *pLocal;
unsigned	uSlot;
unsigned	uType;
DiskPos		iPos;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}


	pLocal = pData->aLocalData[uSlot];


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else{
			iPos = f_tell(pFilePtr);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		unsigned	uItem;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iPos  = DPD_ERR_PARAM;
			}
		else{
			uItem = DPU_ITEM_OF_HANDLE(hFile);
			iPos  = pEntry->aFilePos[uItem];
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pFilePtr;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else{
			iPos = ext4_ftell(pFilePtr);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iPos	= DPD_ERR_PARAM;
		break;
		}



return iPos;
}


//*****************************************************************************
//*
//*		DiskGetFileTime
//*
//*****************************************************************************
//	Holt die Dateizeiten von einer geffneten Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	pWrite		: Hier wird die nderungszeit gespeichert (Wenn ungleich NULL)
//	pCreate		: Hier wird die nderungszeit gespeichert (Wenn ungleich NULL)
//	pAccess		: Hier wird die nderungszeit gespeichert (Wenn ungleich NULL)
//	Ergibt 0 wenn ok, bzw. eine Fehlernummer kleiner 0.
int	DiskGetFileTime(DiskPartionData *pData,DiskFile	hFile,DiskUtc *pWrite,DiskUtc *pCreate,DiskUtc *pAccess)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}

	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		WORD		wTime;
		WORD		wDate;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			if(pAccess)
				{
				*pAccess = 0;
				}

			if(pCreate)
				{
				wDate	 = LD_WORD(pFilePtr->dir_ptr+DIR_CrtDate);
				wTime	 = LD_WORD(pFilePtr->dir_ptr+DIR_CrtTime);
				*pCreate = FatDosToTime(wDate,wTime);
				}

			if(pWrite)
				{
				wDate	 = LD_WORD(pFilePtr->dir_ptr+DIR_WrtDate);
				wTime	 = LD_WORD(pFilePtr->dir_ptr+DIR_WrtTime);
				*pWrite  = FatDosToTime(wDate,wTime);
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			if(pAccess)*pAccess=(DiskUtc)pNode->atime;
			if(pCreate)*pCreate=(DiskUtc)pNode->ctime;
			if(pWrite )*pWrite =(DiskUtc)pNode->mtime;
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry		*pEntry=(ExtEntry*)pLocal;
		ext4_file		*pFilePtr;
		ext4_attributes	 sAttib;
		int				 iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			sAttib.nlen = 0;
			iRet = ext4_file_get_attributes(pFilePtr,&sAttib);

			if(iRet!=EOK)									// Fehler bei Abfrage
				{
				iError = ExtError(iRet);
				}
			else{
				if(pAccess)*pAccess = sAttib.atime;
				if(pCreate)*pCreate = sAttib.ctime;
				if(pWrite )*pWrite  = sAttib.mtime;
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



return iError;
}

//*****************************************************************************
//*
//*		DiskSetFileTime
//*
//*****************************************************************************
//	ndert die Dateizeiten von einer geffneten Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	uWrite		: Hier wird die nderungszeit gespeichert (Wenn ungleich 0)
//	uCreate		: Hier wird die nderungszeit gespeichert (Wenn ungleich 0)
//	uAccess		: Hier wird die nderungszeit gespeichert (Wenn ungleich 0)
//	Ergibt 0 wenn ok, bzw. eine Fehlernummer kleiner 0.
int	DiskSetFileTime(DiskPartionData *pData,DiskFile	hFile,DiskUtc uWrite,DiskUtc uCreate,DiskUtc uAccess)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}

	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else if(!(pFilePtr->flag&FA_WRITE))
			{
			iError = DPD_ERR_ACCESS_DENIED;
			}
		else{
			if(uCreate)
				{
				pFilePtr->newcrea = FatTimeToDos(uCreate);

				if(pFilePtr->newcrea)
					{
					pFilePtr->flag |= FA__WRITTEN;
					ST_DWORD(pFilePtr->dir_ptr+DIR_CrtTime,pFilePtr->newcrea);
					}
				}

			if(uWrite)
				{
				pFilePtr->newtime = FatTimeToDos(uWrite);

				if(pFilePtr->newtime)
					{
					pFilePtr->flag |= FA__WRITTEN;
					ST_DWORD(pFilePtr->dir_ptr+DIR_WrtTime,pFilePtr->newtime);
					}
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			if(uAccess){pNode->atime=uAccess;pNode->flags|=EXFAT_ATTRIB_DIRTY;}
			if(uCreate){pNode->ctime=uCreate;pNode->flags|=EXFAT_ATTRIB_DIRTY;}
			if(uWrite ){pNode->mtime=uWrite ;pNode->flags|=EXFAT_ATTRIB_DIRTY;}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry		*pEntry=(ExtEntry*)pLocal;
		ext4_file		*pFilePtr;
		ext4_attributes	 sAttib;
		int				 iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			sAttib.atime = uAccess;
			sAttib.ctime = uCreate;
			sAttib.mtime = uWrite;
			sAttib.fmask = 0;
			sAttib.nlen  = 0;

			iRet   = ext4_file_set_attributes(pFilePtr,&sAttib);
			iError = ExtError(iRet);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



return iError;
}


//*****************************************************************************
//*
//*		DiskGetFileSize
//*
//*****************************************************************************
//	Holt die Dateilnge von einer geffneten Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	Ergibt die Dateigre, bzw. eine Fehlernummer kleiner 0.
DiskPos DiskGetFileSizeEx(DiskPartionData *pData,DiskFile hFile)
{
void	   *pLocal;
unsigned	uSlot;
unsigned	uType;
DiskPos		iPos;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}

	pLocal = pData->aLocalData[uSlot];
	iPos   = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else{
			iPos = f_size(pFilePtr);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else{
			iPos = pNode->size;
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry		*pEntry=(ExtEntry*)pLocal;
		ext4_file		*pFilePtr;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iPos = DPD_ERR_PARAM;
			}
		else{
			iPos = ext4_fsize(pFilePtr);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iPos	= DPD_ERR_PARAM;
		break;
		}



return iPos;
}

//*****************************************************************************
//*
//*		DiskSetFileSize
//*
//*****************************************************************************
//	ndert die Dateilnge von einer geffneten Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	uSize		: Ist die neue Gre fr die Datei
//	Ergibt 0 wenn ok, bzw. eine Fehlernummer kleiner 0.
int DiskSetFileSize(DiskPartionData *pData,DiskFile	hFile,DWORD uSize)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}

	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FIL		   *pFilePtr;
		DWORD		dwNow;
		FRESULT		iRet;


		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else if(!(pFilePtr->flag&FA_WRITE))
			{
			iError = DPD_ERR_ACCESS_DENIED;
			}
		else{
			dwNow = f_tell(pFilePtr);
			iRet  = f_lseek(&pEntry->sFatData,pFilePtr,uSize);

			if(iRet==FR_OK)
				{
				iRet = f_truncate(&pEntry->sFatData,pFilePtr);
				f_lseek(&pEntry->sFatData,pFilePtr,dwNow);
				}

			if(iRet!=FR_OK)									// Fehler bei der Aktion
				{
				iError = FatError(iRet);
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		unsigned	uItem;
		int			iRet;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			uItem = DPU_ITEM_OF_HANDLE(hFile);
			iRet  = exfat_truncate(&pEntry->sExFat,pNode,uSize,true);
			if(iRet<0)iError=ExFatError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_file  *pFilePtr;
		int			iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet   = ext4_ftruncate(pFilePtr,uSize);
			iError = ExtError(iRet);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



return iError;
}


//*****************************************************************************
//*
//*		DiskGetFileInfo
//*
//*****************************************************************************
//	Holt die Infos zu einer offenen Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	hFile		: File-Handle der Datei
//	pFind		: Hier werden die Infos gespeichert
//	Ergibt 0 wenn ok, bzw. eine Fehlernummer kleiner 0.
int DiskGetFileInfo(DiskPartionData *pData,DiskFile	hFile,DiskFindStruct *pFind)
{
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;



	uType = DPU_TYPE_OF_HANDLE(hFile);
	uSlot = DPU_SLOT_OF_HANDLE(hFile);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}

	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		char		aShort[16];
		ZCHAR		aName[256];
		FIL		   *pFilePtr;
		FRESULT		iRet;

		DPD_LOCK(pData);

			pFilePtr = FatFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			   iRet = f_getname(&pEntry->sFatData,pFilePtr,aName,ENTRIES(aName));
			if(iRet!=FR_OK)
				{
				fat_copyshort(aShort,(char*)pFilePtr->dir_ptr+DIR_Attr);
				DiskFillName(aName,pFind->cName,ENTRIES(pFind->cName),DPD_ANSI);
				}
			else{
				DiskFillName(aName,pFind->cName,ENTRIES(pFind->cName),DPD_UTF16);
				}

			#if DISK_HAS_SHORT_NAME
			fat_copyshort(pFind->cShortName,(char*)pFilePtr->dir_ptr+DIR_Attr);
			#endif

			pFind->dwTimeAccess = 0;
			pFind->dwTimeCreate = FatDosToTime(LD_WORD(pFilePtr->dir_ptr+DIR_CrtDate),LD_WORD(pFilePtr->dir_ptr+DIR_CrtTime));
			pFind->dwTimeWrite  = FatDosToTime(LD_WORD(pFilePtr->dir_ptr+DIR_WrtDate),LD_WORD(pFilePtr->dir_ptr+DIR_WrtTime));
			pFind->dwAttr 		= FatConvertAttr(pFilePtr->dir_ptr[DIR_Attr]);
			pFind->dwSize       = f_size(pFilePtr);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;

		DPD_LOCK(pData);

			pNode = ExFatFileGet(pEntry,hFile);
		if(!pNode)											// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			ExFatNoteToInfo(pNode,pFind);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry		*pEntry=(ExtEntry*)pLocal;
		ext4_file		*pFilePtr;
		ext4_attributes  sAttrib;
		int				 iRet;

		DPD_LOCK(pData);

			pFilePtr = ExtFileGet(pEntry,hFile);
		if(!pFilePtr)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			sAttrib.nlen = 255;

			iRet   = ext4_file_get_attributes(pFilePtr,&sAttrib);

			if(iRet!=EOK)									// Fehler bei Abfrage
				{
				iError = ExtError(iRet);
				}
			else{
				pFind->dwAttr		= ExtConvertAttrib(sAttrib.fmode);
				pFind->dwTimeAccess	= sAttrib.atime;
				pFind->dwTimeCreate	= sAttrib.ctime;
				pFind->dwTimeWrite	= sAttrib.mtime;
				pFind->dwSize		= (DWORD) (sAttrib.fsize);
				pFind->dwAttr	   |= (DWORD)((sAttrib.fsize>>(32-DPD_HIGHMASK))&DPD_HIGHSHIFT);

				DiskFillName(sAttrib.nbuf,pFind->cName,ENTRIES(pFind->cName),DPD_UTF8);

				#if DISK_HAS_SHORT_NAME
				pFind->cShortName[0] = 0;
				#endif
				}
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



return iError;
}


//*****************************************************************************
//*
//*		DiskGetAttributes
//*
//*****************************************************************************
//	Holt die Attributte zu einer Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name der Datei.
//	Ergibt die Attributte oder eine Fehlernummer kleiner 0.
//					DPD_READ
//				  	DPD_WRITE
//				  	DPD_DIR
//				  	DPD_HIDDEN
//				  	DPD_ARCHIVE
int DiskGetAttributes(DiskPartionData *pData,const DCHAR *pFilename)
{
wchar_t		aBuffer[DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
int			iError;
int			iLen;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO,&iLen);
	if(iDrive<=0)
		{
		if(!iDrive)return DPD_DIR;
		return iDrive;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;



	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FILINFO		sInfo;
		FRESULT		iRet;
		int			iSub;

		if(iLen>0 && aBuffer[iLen-1]=='/')					// Ist ein Slash am Ende
			{
			aBuffer[iLen-1] = 0;
			iSub = 1;
			}
		else{
			iSub = 0;
			}

		if(aBuffer[0]==0)									// Nur Laufwerksangabe "/0"
			{
			iError = DPD_DIR;
			}
		else{
			DPD_LOCK(pData);

			sInfo.lfname = 0;

			   iRet = f_stat(&pEntry->sFatData,aBuffer,&sInfo);
			if(iRet!=FR_OK)									// Fehler bei Aktion
				{
				iError = FatError(iRet);
				}
			else if(iSub && !(sInfo.fattrib&AM_DIR))		// Wenn Slash am Ende sind nur Verzeichnisse erlaubt
				{
				iError = DPD_ERR_FILE_NOT_FOUND;
				}
			else{
				iError = FatConvertAttr(sInfo.fattrib);
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iRet;
		int			iSub;

		if(iLen>0 && ((char*)aBuffer)[iLen-1]=='/')			// Ist ein Slash am Ende
			{
			((char*)aBuffer)[iLen-1] = 0;
			iSub = 1;
			}
		else{
			iSub = 0;
			}

		if(((char*)aBuffer)[0]==0)							// Nur Laufwerksangabe "/0"
			{
			iError = DPD_DIR;
			}
		else{
			DPD_LOCK(pData);

			   iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
			if(iRet!= 0)
				{
				iError = ExFatError(iRet);
				}											// Wenn Slash am Ende sind nur Verzeichnisse erlaubt
			else if(iSub && !(pNode->flags&EXFAT_ATTRIB_DIR))
				{
				iError = DPD_ERR_FILE_NOT_FOUND;
				}
			else{
				iError = ExFatConvertAttrib(pNode);
				exfat_put_node(&pEntry->sExFat,pNode);
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry	   *pEntry=(ExtEntry*)pLocal;
		ext4_attributes	sAttib;
		ext4_file		sFile;
		int 			iRet;
		int				iSub;

		if(iLen>0 && ((char*)aBuffer)[iLen-1]=='/')			// Ist ein Slash am Ende
			{
			((char*)aBuffer)[iLen-1] = 0;
			iSub = 1;
			}
		else{
			iSub = 0;
			}

		if(((char*)aBuffer)[0]==0)							// Nur Laufwerksangabe "/0"
			{
			iError = DPD_DIR;
			}
		else{
			DPD_LOCK(pData);

			   iRet = ext4_fopen_ex(pEntry->hHandle,&sFile,(char*)aBuffer);
			if(iRet!=EOK)									// Fehler beim ffnen
				{
				iError = ExtError(iRet);
				}
			else{
				sAttib.nlen = 0;

				   iRet = ext4_file_get_attributes(&sFile,&sAttib);
				if(iRet!=EOK)
					{
					iError = ExtError(iRet);
					}										// Wenn Slash am Ende sind nur Verzeichnisse erlaubt
				else if(iSub && (sAttib.fmode&0xF000)!=0x4000)
					{
					iError = DPD_ERR_FILE_NOT_FOUND;
					}
				else{
					iError = ExtConvertAttrib(sAttib.fmode)&0x7FFFFFFF;
					}
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PATH_NOT_FOUND;
		break;
		}



return iError;
}


//*****************************************************************************
//*
//*		DiskSetAttributes
//*
//*****************************************************************************
//	ndert die Attributte einer Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name der Datei.
//	iAttr		: Sind die neuen Attribute fr die Datei
//					DPD_READ
//				  	DPD_WRITE
//				  	DPD_DIR
//				  	DPD_HIDDEN
//				  	DPD_ARCHIVE
//	Ergibt 0 wenn ok, oder eine Fehlernummer kleiner 0.
int DiskSetAttributes(DiskPartionData *pData,const DCHAR *pFilename,int iAttr)
{
wchar_t		aBuffer[DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
int			iError;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO);
	if(iDrive<=0)
		{
		if(!iDrive)return DPD_DIR;
		return iDrive;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;



	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		BYTE		ucAttr;
		FRESULT		iRet;

		DPD_LOCK(pData);

		if(iAttr&(DPD_DIR|DPD_VOLUME))						// Prfe die Attribute
			{
			iError = DPD_ERR_PARAM;
			}
		else{
								 ucAttr = 0;
			if(iAttr&DPD_WRITE  )ucAttr&=~AM_RDO;
			if(iAttr&DPD_HIDDEN )ucAttr|= AM_HID;
			if(iAttr&DPD_ARCHIVE)ucAttr|= AM_ARC;

			   iRet = f_chmod(&pEntry->sFatData,aBuffer,ucAttr,AM_RDO|AM_HID|AM_ARC);
			if(iRet!=FR_OK)									// Fehler bei Aktion
					iError = FatError(iRet);
			else	iError = DPD_ERR_NONE;
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iFlag;
		int			iRet;

		DPD_LOCK(pData);

		   iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
		if(iRet!= 0)
			{
			iError = ExFatError(iRet);
			}
		else if(pNode->flags&EXFAT_ATTRIB_DIR)				// Ist es ein Verzeichniss
			{
			exfat_put_node(&pEntry->sExFat,pNode);
			iError = DPD_ERR_ACCESS_DENIED;
			}
		else{
			iFlag  = (iAttr&DPD_WRITE)? 0:EXFAT_ATTRIB_RO;

			if(iAttr&DPD_ARCHIVE)iFlag|=EXFAT_ATTRIB_ARCH;
			if(iAttr&DPD_HIDDEN )iFlag|=EXFAT_ATTRIB_HIDDEN;
			if(iAttr&DPD_SYSTEM )iFlag|=EXFAT_ATTRIB_SYSTEM;
			if(iAttr&DPD_SYSTEM )iFlag|=EXFAT_ATTRIB_SYSTEM;

			if(pNode->flags!=iFlag)
				{
				pNode->flags  = iFlag;
				pNode->flags |= EXFAT_ATTRIB_DIRTY;
				}

			exfat_put_node(&pEntry->sExFat,pNode);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry	   *pEntry=(ExtEntry*)pLocal;
		ext4_attributes	sAttib;
		ext4_file		sFile;
		int				iTurn;
		int 			iRet;

		if(aBuffer[0]==0)									// Nur Laufwerksangabe "/0"
			{
			iError = DPD_DIR;
			}
		else{
			iTurn = 0;

			DPD_LOCK(pData);

Retry:		   iRet = ext4_fopen2(pEntry->hHandle,&sFile,(char*)aBuffer,O_RDWR|O_NOCHECK);
			if(iRet!=EOK)									// Fehler beim ffnen
				{
				if(iRet==EMLINK)								// Links auflsen
					{
					   iRet = ExtResolvePath(pEntry->hHandle,(char*)aBuffer,sFile.inode);
					if(iRet==EOK)
						{
						iTurn++;
						if(iTurn<4)goto Retry;
						iRet = EMLINK;
						}
					}

				iError = ExtError(iRet);
				}
			else{
				sAttib.nlen		= 0;
				sAttib.atime	= 0;
				sAttib.ctime	= 0;
				sAttib.atime	= 0;

				if(iAttr&0xFFFF0000)						// Linux Attribute
					{
					sAttib.fmask |= 0x03FF;
					sAttib.fmode |= (WORD)(iAttr>>16);
					}
				else{
					sAttib.fmask  = 0x01DB;
					sAttib.fmode  = 0x0000;

					if(iAttr&DPD_READ   ){sAttib.fmode|=0x0124;sAttib.fmask|=0x0124;}
					if(iAttr&DPD_WRITE  ){sAttib.fmask|=0x0124;}
					if(iAttr&DPD_EXEC   ){sAttib.fmode|=0x0049;}
					if(iAttr&DPD_HIDDEN ){sAttib.fmode|=0x0001;} // Samba-Hack
					if(iAttr&DPD_SYSTEM ){sAttib.fmode|=0x0008;}
					if(iAttr&DPD_ARCHIVE){sAttib.fmode|=0x0040;}
					}

				   iRet = ext4_file_set_attributes(&sFile,&sAttib);
				if(iRet!=EOK)
					{
					iError = ExtError(iRet);
					}
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PATH_NOT_FOUND;
		break;
		}



return iError;
}

//*****************************************************************************
//*
//*		DiskFileLengthEx
//*
//*****************************************************************************
//	Holt die Gre einer Datei
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name der Datei
//	Ergibt 0 wenn die Datei nicht existiert, 1 wenn ja, 2 wenn es ein Verzeichnist ist.
DiskPos DiskFileLengthEx(DiskPartionData *pData,const DCHAR *pFilename)
{
wchar_t		aBuffer[DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
DiskPos		iPos;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO);
	if(iDrive<=0)
		{
		return (iDrive)? 0:2;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];


	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FILINFO		sInfo;
		FRESULT		iRet;

		if(aBuffer[0]==0)									// Nur Laufwerksangabe "/0"
			{
			iPos = 0;
			}
		else{
			DPD_LOCK(pData);

			sInfo.lfname = 0;

			   iRet = f_stat(&pEntry->sFatData,aBuffer,&sInfo);
			if(iRet!=FR_OK)									// Fehler bei Aktion
					iPos = FatError(iRet);
			else	iPos = sInfo.fsize;

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iRet;

		DPD_LOCK(pData);

		   iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
		if(iRet!= 0)
			{
			iPos = ExFatError(iRet);
			}
		else{
			iPos = pNode->size;
			exfat_put_node(&pEntry->sExFat,pNode);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry	   *pEntry=(ExtEntry*)pLocal;
		ext4_file		sFile;
		int				iTurn;
		int 			iRet;

		if(aBuffer[0]==0)									// Nur Laufwerksangabe "/0"
			{
			iPos = DPD_DIR;
			}
		else{
			iTurn = 0;

			DPD_LOCK(pData);

Retry:		   iRet = ext4_fopen2(pEntry->hHandle,&sFile,(char*)aBuffer,O_RDONLY);
			if(iRet==EISDIR)
				{
				iPos = 0;
				}
			else if(iRet!=EOK)								// Fehler beim ffnen
				{
				if(iRet==EMLINK)							// Links auflsen
					{
					   iRet = ExtResolvePath(pEntry->hHandle,(char*)aBuffer,sFile.inode);
					if(iRet==EOK)
						{
						iTurn++;
						if(iTurn<4)goto Retry;
						iRet = EMLINK;
						}
					}

				iPos = ExtError(iRet);
				}
			else{
				iPos = ext4_fsize(&sFile);
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iPos = DPD_ERR_FILE_NOT_FOUND;
		break;
		}



return iPos;
}

//*****************************************************************************
//*
//*		DiskFileExist
//*
//*****************************************************************************
//	Prft ob eine Datei existiert
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name der Datei
//	Ergibt folgende Werte
//					0 = wenn die Datei nicht existiert
//					1 = wenn die Datei existiert
//					2 = wenn die Datei ein Verzeichnist ist
int DiskFileExist(DiskPartionData *pData,const DCHAR *pFilename)
{
wchar_t		aBuffer[DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
int 		iVal;
int			iLen;


	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO,&iLen);
	if(iDrive<=0)
		{
		return (iDrive)? 0:2;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];


	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FILINFO		sInfo;
		FRESULT		iRet;
		int			iSub;

		if(iLen>0 && aBuffer[iLen-1]=='/')					// Ist ein Slash am Ende
			{
			aBuffer[iLen-1] = 0;
			iSub = 1;
			}
		else{
			iSub = 0;
			}

		if(aBuffer[0]==0)									// Nur Laufwerksangabe "/0"
			{
			iVal = 2;
			}
		else{
			DPD_LOCK(pData);

			sInfo.lfname = 0;

			   iRet = f_stat(&pEntry->sFatData,aBuffer,&sInfo);
			if(iRet!=FR_OK)									// Fehler bei Aktion
				{
				iVal = 0;
				}
			else if(iSub && !(sInfo.fattrib&AM_DIR))		// Wenn Slash am Ende sind nur Verzeichnisse erlaubt
				{
				iVal = 0;
				}
			else{
				iVal = (sInfo.fattrib&AM_DIR)? 2:1;
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iRet;
		int			iSub;

		if(iLen>0 && ((char*)aBuffer)[iLen-1]=='/')			// Ist ein Slash am Ende
			{
			((char*)aBuffer)[iLen-1] = 0;
			iSub = 1;
			}
		else{
			iSub = 0;
			}

		if(((char*)aBuffer)[0]==0)							// Nur Laufwerksangabe "/0"
			{
			iVal = 2;
			}
		else{
			DPD_LOCK(pData);

			   iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
			if(iRet!= 0)
				{
				iVal = 2;
				}											// Wenn Slash am Ende sind nur Verzeichnisse erlaubt
			else if(iSub && !(pNode->flags&EXFAT_ATTRIB_DIR))
				{
				iVal = 0;
				exfat_put_node(&pEntry->sExFat,pNode);
				}
			else{
				iVal = (pNode->flags&EXFAT_ATTRIB_DIR)? 2:1;
				exfat_put_node(&pEntry->sExFat,pNode);
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry	   *pEntry=(ExtEntry*)pLocal;
		ext4_attributes	sAttib;
		ext4_file		sFile;
		int 			iRet;
		int				iSub;

		if(iLen>0 && ((char*)aBuffer)[iLen-1]=='/')			// Ist ein Slash am Ende
			{
			((char*)aBuffer)[iLen-1] = 0;
			iSub = 1;
			}
		else{
			iSub = 0;
			}

		if(((char*)aBuffer)[0]==0)							// Nur Laufwerksangabe "/0"
			{
			iVal = 2;
			}
		else{
			DPD_LOCK(pData);

			   iRet = ext4_fopen_ex(pEntry->hHandle,&sFile,(char*)aBuffer);
			if(iRet!=EOK)									// Fehler beim ffnen
				{
				iVal = 0;
				}
			else{
				sAttib.nlen = 0;

				   iRet = ext4_file_get_attributes(&sFile,&sAttib);
				if(iRet!=EOK)
					{
					iVal = 0;
					}										// Wenn Slash am Ende sind nur Verzeichnisse erlaubt
				else if(iSub && (sAttib.fmode&0xF000)!=0x4000)
					{
					iVal = 0;
					}
				else{
					iVal = ((sAttib.fmode&0xF000)!=0x4000)? 2:1;
					}
				}

			DPD_UNLOCK(pData);
			}

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iVal = 0;
		break;
		}



return iVal;
}


//*****************************************************************************
//*
//*		DiskMkdir
//*
//*****************************************************************************
//	Erstellt ein neues Verzeichnis
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name des neuen Verzeichnisses.
//	Ergibt 0 wenn ok, oder eine Fehlernummer kleiner 0.
int DiskMkdir(DiskPartionData *pData,const DCHAR *pFilename)
{
wchar_t		aBuffer[DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
int			iError;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO);
	if(iDrive<=0)
		{
		return (iDrive)? iDrive:DPD_ERR_PATH_NOT_FOUND;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;



	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FRESULT		iRet;

		DPD_LOCK(pData);

		   iRet = f_mkdir(&pEntry->sFatData,aBuffer);
		if(iRet!=FR_OK)										// Fehler bei Aktion
			{
			iError = FatError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		int			iRet;

		DPD_LOCK(pData);

		   iRet = exfat_mkdir(&pEntry->sExFat,(char*)aBuffer);
		if(iRet<0)iError=ExFatError(iRet);

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		int 		iRet;

		DPD_LOCK(pData);

		   iRet = ext4_dir_mk(pEntry->hHandle,(char*)aBuffer);
		if(iRet!=EOK)										// Fehler bei Aktion
			{
			iError = ExtError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PATH_NOT_FOUND;
		break;
		}



return iError;
}

//*****************************************************************************
//*
//*		DiskRmdir
//*
//*****************************************************************************
//	Lscht ein Verzeichnis
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name des Verzeichnisses.
//	Ergibt 0 wenn ok, oder eine Fehlernummer kleiner 0.
int DiskRmdir(DiskPartionData *pData,const DCHAR *pFilename)
{
wchar_t		aBuffer[DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
int			iError;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO);
	if(iDrive<=0)
		{
		return (iDrive)? iDrive:DPD_ERR_PATH_NOT_FOUND;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;



	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FILINFO		sInfo;
		FRESULT		iRet;


		DPD_LOCK(pData);

		sInfo.lfname = 0;

		   iRet = f_stat(&pEntry->sFatData,aBuffer,&sInfo);
		if(iRet!=FR_OK)										// Fehler bei Aktion
			{
			iError = FatError(iRet);
			}
		else if(!(sInfo.fattrib&AM_DIR))					// Nur Verzeichnisse lschen
			{
			iError = DPD_ERR_PATH_NOT_FOUND;
			}
		else{
			   iRet = f_unlink(&pEntry->sFatData,aBuffer);
			if(iRet!=FR_OK)									// Fehler bei Aktion
				{
				iError = FatError(iRet);
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iRet;

		DPD_LOCK(pData);

		   iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
		if(iRet<0)
			{
			iError=ExFatError(iRet);
			}
		else{
			   iRet = exfat_rmdir(&pEntry->sExFat,pNode);
			if(iRet<0)
				{
				iError = ExFatError(iRet);
				}

			     exfat_put_node    (&pEntry->sExFat,pNode);
			iRet=exfat_cleanup_node(&pEntry->sExFat,pNode);
			if(!iError && iRet<0)iError=ExFatError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		int 		iRet;

		DPD_LOCK(pData);

		   iRet = ext4_dir_rm(pEntry->hHandle,(char*)aBuffer,0);
		if(iRet!=EOK)										// Fehler bei Aktion
			{
			iError = ExtError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PATH_NOT_FOUND;
		break;
		}



return iError;
}

//*****************************************************************************
//*
//*		DiskDelete
//*
//*****************************************************************************
//	Lscht eine Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	pFilename	: Ist der Name der Datei.
//	Ergibt 0 wenn ok, oder eine Fehlernummer kleiner 0.
int DiskDelete(DiskPartionData *pData,const DCHAR *pFilename)
{
wchar_t		aBuffer[DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
int			iError;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO);
	if(iDrive<=0)
		{
		return (iDrive)? iDrive:DPD_ERR_FILE_NOT_FOUND;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;



	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FILINFO		sInfo;
		FRESULT		iRet;


		DPD_LOCK(pData);

		sInfo.lfname = 0;

		   iRet = f_stat(&pEntry->sFatData,aBuffer,&sInfo);
		if(iRet!=FR_OK)										// Fehler bei Aktion
			{
			iError = FatError(iRet);
			}
		else if(sInfo.fattrib&AM_DIR)						// Nur Dateien lschen
			{
			iError = DPD_ERR_FILE_NOT_FOUND;
			}
		else{
			   iRet = f_unlink(&pEntry->sFatData,aBuffer);
			if(iRet!=FR_OK)									// Fehler bei Aktion
				{
				iError = FatError(iRet);
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		exfat_node *pNode;
		int			iRet;

		DPD_LOCK(pData);

		   iRet = exfat_lookup(&pEntry->sExFat,&pNode,(char*)aBuffer);
		if(iRet<0)
			{
			iError=ExFatError(iRet);
			}
		else{
			   iRet = exfat_unlink(&pEntry->sExFat,pNode);
			if(iRet<0)
				{
				iError = ExFatError(iRet);
				}

			     exfat_put_node    (&pEntry->sExFat,pNode);
			iRet=exfat_cleanup_node(&pEntry->sExFat,pNode);
			if(!iError && iRet<0)iError=ExFatError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		int 		iRet;

		DPD_LOCK(pData);

		   iRet = ext4_fremove(pEntry->hHandle,(char*)aBuffer);
		if(iRet!=EOK)										// Fehler bei Aktion
			{
			iError = ExtError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PATH_NOT_FOUND;
		break;
		}



return iError;
}

//*****************************************************************************
//*
//*		DiskRename
//*
//*****************************************************************************
//	Benennt eine Datei um.
//	pData			: Zeiger auf die Laufwerksdaten
//	pFilename		: Ist der Name der Datei.
//	pFilenameNew	: Ist der neue Dateiname.
//	Ergibt 0 wenn ok, oder eine Fehlernummer kleiner 0.
int DiskRename(DiskPartionData *pData,const DCHAR *pFilename,const DCHAR *pFilenameNew)
{
wchar_t		aNewName[DPD_MAX_PATH];
wchar_t		aBuffer [DPD_MAX_PATH];
void	   *pLocal;
int			iDrive;
int			iValue;
int			iError;



	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO);
	if(iDrive<=0)
		{
		return (iDrive)? iDrive:DPD_ERR_PATH_NOT_FOUND;
		}

	   iValue = DiskConvertPath(pData,pFilenameNew,aNewName,DPD_MAX_PATH,DPD_AUTO);
	if(iValue!=iDrive)
		{
		return (iDrive<0)? iDrive:DPD_ERR_UNKNOWN;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;



	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		FRESULT		iRet;

		DPD_LOCK(pData);

		   iRet = f_rename(&pEntry->sFatData,aBuffer,aNewName);
		if(iRet!=FR_OK)										// Fehler bei Aktion
			{
			iError = FatError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		int			iRet;

		DPD_LOCK(pData);

		   iRet = exfat_rename(&pEntry->sExFat,(char*)aBuffer,(char*)aNewName);
		if(iRet<0)
			{
			iError = ExFatError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		int 		iRet;

		DPD_LOCK(pData);

		   iRet = ext4_frename(pEntry->hHandle,(char*)aBuffer,(char*)aNewName);
		if(iRet!=EOK)										// Fehler bei Aktion
			{
			iError = ExtError(iRet);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PATH_NOT_FOUND;
		break;
		}



return iError;
}

//*****************************************************************************
//*
//*		DiskFindFirstEx
//*
//*****************************************************************************
//	Beginnt eine Suche nach Dateien
//	pData			: Zeiger auf die Laufwerksdaten
//	pFilename		: Ist der Name eines Verzeichnisses mit Wildcars ('*' oder '?').
//	pFind			: Hier werden die gefundenen Dateien gespeichert
//	pError			: Hier wird die Fehlernummer gespeichert, wenn nicht NULL.
//	Ergibt ein Find-Handle oder 0 bei einem Fehler.
DiskFind DiskFindFirstEx(DiskPartionData *pData,const DCHAR *pFilename,DiskFindStruct *pFind,int *pError)
{
wchar_t		aBuffer[DPD_MAX_PATH];
DiskFind	hHandle;
VolumeFind *pMyFind;
char	   *pBuffer;
void	   *pLocal;
int			iDrive;
int			iError;
int			iFlag;
int			iItem;
int			iMode;
int			iPos;
int			iLen;


	if(!pFind)
		{
		if(pError)*pError=DPD_ERR_PARAM;
		DiskClearFindStruct(pFind);
		return NULL;
		}

	   iDrive = DiskConvertPath(pData,pFilename,aBuffer,DPD_MAX_PATH,DPD_AUTO,&iLen);
	if(iDrive<0)
		{
		if(strwchar(pFilename[0],0x00200000))pFilename++;

		   iPos = strwlen(pFilename,0x01800007);
		if(iPos>0 && iPos<8 && !pFilename[iPos])
		if(iPos>strwlen(pFilename,0x07))					// Suche fr Verzeichnisse
			{
			DPD_LOCK(pData);

			for(iItem=0;iItem<DISK_MAX_FIND;iItem++)
				{
				if(pData->aFindPtr[iItem])continue;

				pMyFind = new VolumeFind;
				pData->aFindPtr[iItem] = pMyFind;
				hHandle = (DiskFind)DPU_LIST_TO_HANDLE(DPD_TYPE_PHYSICAL,0,iItem);
				memcpy(pMyFind->cFilter,pFilename,(iPos+1)*sizeof(DCHAR));
				pMyFind->uSlot = 0;
				if(!pError)pError=&iDrive;

				DPD_UNLOCK(pData);

				   *pError = DiskFindNext(pData,hHandle,pFind);
				if(*pError)
					{
					pData->aFindPtr[iItem] = 0;
					delete pMyFind;
					hHandle = NULL;
					}

				return hHandle;
				}

			DPD_UNLOCK(pData);

			iDrive = DPD_ERR_NO_HANDLES;
			}

		DiskClearFindStruct(pFind);
		if(pError)*pError=iDrive;
		return NULL;
		}


	pBuffer	= (char*)aBuffer;
	iMode	= aTypeToChar[pData->aDriveType[iDrive-1]];		// Modus fr Pfad

	if(iMode==DPD_UTF16)
		{
		for(iPos=iLen-1,iFlag=0;iPos>=0;iPos--)				// Suche Wildchars
			{
			if(aBuffer[iPos]=='/')break;
			if(aBuffer[iPos]!='*')
			if(aBuffer[iPos]!='?')continue;
			iFlag=1;
			}
		}
	else{
		for(iPos=iLen-1,iFlag=0;iPos>=0;iPos--)				// Suche Wildchars
			{
			if(pBuffer[iPos]=='/')break;
			if(pBuffer[iPos]!='*')
			if(pBuffer[iPos]!='?')continue;
			iFlag=1;
			}
		}

	iPos++;

	if(iPos>=iLen)											// Slash am Ende
		{
		if(iLen==0 && pData->uCurrenrPathLen<=1)
			{
			if(strwchar(pFilename[0],0x00200000))pFilename++;
			if(strwchar(pFilename[0],0x0007) && !pFilename[1])
				{											// "/0" oder "0"
				pFind->dwTimeCreate = 1;
				pFind->dwTimeAccess = 1;
				pFind->dwTimeWrite  = 1;
				pFind->dwSize       = 0;
				pFind->dwAttr 		= DPD_DIR;
				pFind->cName[0]		= pData->aDriveSign[iDrive-1]+'0';
				pFind->cName[1]		= 0;

				return DPU_FIND_ONE;
				}
			}

		if(pError)*pError=DPD_ERR_INVALID_FILENAME;
		DiskClearFindStruct(pFind);
		return NULL;
		}

	if(iLen-iPos>=ENTRIES(pFind->cName))
		{
		if(pError)*pError=DPD_ERR_PARAM;
		DiskClearFindStruct(pFind);
		return 0;
		}

	iDrive--;

	pLocal = pData->aLocalData[iDrive];
	iError = DPD_ERR_NONE;


//******************** Ist es eine einzlen Datei ******************************


	if(!iFlag)												// Keine Wildchars
		{
		switch(pData->aDriveType[iDrive])					// Was fr eine Partion ist gemountet
			{
		#if	DPD_HAS_FAT

		case DPD_TYPE_FAT:									// Zugriff auf FAT-Partition
			{
			FatEntry   *pEntry=(FatEntry*)pLocal;
			wchar_t		aName[256];
			FILINFO		sInfo;
			FRESULT		iRet;

			DPD_LOCK(pData);

			sInfo.lfname = aName;
			sInfo.lfsize = ENTRIES(aName);

	   		iRet = f_stat(&pEntry->sFatData,aBuffer,&sInfo);

			DPD_UNLOCK(pData);

			if(iRet!=FR_OK)
				{
				if(pError)*pError=FatError(iRet);
				DiskClearFindStruct(pFind);
				return NULL;
				}

			if(aName[0]==0)									// Kein langer Dateiname
					DiskFillName(sInfo.fname,pFind->cName,ENTRIES(pFind->cName),FAT_DOS13);
			else	DiskFillName(aName      ,pFind->cName,ENTRIES(pFind->cName),DPD_UTF16);


			#if DISK_HAS_SHORT_NAME
			memcpy(pFind->cShortName,sInfo.fname,sizeof(pFind->cShortName));
			#endif

			pFind->dwTimeAccess = 0;
			pFind->dwTimeCreate = FatDosToTime(sInfo.cdate,sInfo.ctime);
			pFind->dwTimeWrite  = FatDosToTime(sInfo.fdate,sInfo.ftime);
			pFind->dwAttr 		= FatConvertAttr(sInfo.fattrib);
			pFind->dwSize       = sInfo.fsize;

			if(pError)*pError=DPD_ERR_NONE;

			return DPU_FIND_ONE;
			}
		#endif

		#if	DPD_HAS_EXFAT

		case DPD_TYPE_EXFAT:								// Zugriff auf EXFAT-Partition
			{
			ExFatEntry *pEntry=(ExFatEntry*)pLocal;
			exfat_node *pNode;
			int			iRet;

			DPD_LOCK(pData);

			   iRet = exfat_lookup(&pEntry->sExFat,&pNode,pBuffer);
			if(iRet!= 0)
				{
				if(pError)*pError=ExFatError(iRet);
				DiskClearFindStruct(pFind);
				return NULL;
				}

			ExFatNoteToInfo(pNode,pFind);
			exfat_put_node(&pEntry->sExFat,pNode);

			if(pError)*pError=DPD_ERR_NONE;

			DPD_UNLOCK(pData);

			return DPU_FIND_ONE;
			}

		#endif

		#if	DPD_HAS_EXT

		case DPD_TYPE_EXT:									// Zugriff auf EXT-Partition
			{
			ExtEntry	   *pEntry=(ExtEntry*)pLocal;
			ext4_attributes	sAttrib;
			ext4_file		sFile;
			int				iTurn;
			int 			iRet;


			iTurn = 0;

			DPD_LOCK(pData);

Retry:		   iRet = ext4_fopen_ex(pEntry->hHandle,&sFile,(char*)aBuffer);
			if(iRet==EMLINK || sFile.ftype==EXT4_DE_SYMLINK)
				{											// Links auflsen
				   iRet = ExtResolvePath(pEntry->hHandle,(char*)aBuffer,sFile.inode);
				if(iRet==EOK)
					{
					iTurn++;
					if(iTurn<4)goto Retry;
					iRet = EMLINK;
					}
				}

			if(iRet==EOK)
				{
				sAttrib.nlen = 255;

		   		iRet = ext4_file_get_attributes(&sFile,&sAttrib);
				}

			DPD_UNLOCK(pData);

			if(iRet!=EOK)
				{
				if(pError)*pError=ExtError(iRet);
				DiskClearFindStruct(pFind);
				return NULL;
				}

			pFind->dwAttr		= ExtConvertAttrib(sAttrib.fmode);
			pFind->dwTimeAccess	= sAttrib.atime;
			pFind->dwTimeCreate	= sAttrib.ctime;
			pFind->dwTimeWrite	= sAttrib.mtime;
			pFind->dwSize		= (DWORD) (sAttrib.fsize);
			pFind->dwAttr	   |= (DWORD)((sAttrib.fsize>>(32-DPD_HIGHMASK))&DPD_HIGHSHIFT);

			DiskFillName(sAttrib.nbuf,pFind->cName,ENTRIES(pFind->cName),DPD_UTF8);

			#if DISK_HAS_SHORT_NAME
			pFind->cShortName[0] = 0;
			#endif

			if(pError)*pError=DPD_ERR_NONE;

			return DPU_FIND_ONE;
			}

		#endif

		default:											// Unbekannter Partitionstyp

			if(pError)*pError=DPD_ERR_PARAM;
			return NULL;
			}
		}


//******************** Suche starten ******************************************



	switch(pData->aDriveType[iDrive])						// Was fr eine Partion ist gemountet
		{
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		DIRFIND	   *pItem;
		FRESULT		iRet;


			hHandle	= FatFindAlloc(pEntry,iDrive);
		if(!hHandle)
			{
			iError = DPD_ERR_NO_HANDLES;
			break;
			}

		pItem = FatFindGet(pEntry,hHandle);

		memcpy(pItem->aName,aBuffer+iPos,(iLen-iPos+1)*sizeof(wchar_t));

		if(iPos>0 && aBuffer[iPos-1]=='/')iPos--;
		aBuffer[iPos] = 0;

		DPD_LOCK(pData);

		iRet = f_opendir(&pEntry->sFatData,&pItem->sData,aBuffer);

		if(iRet!=FR_OK)										// Gab es einen Fehler bei der Suche
			{
			iError=DPD_ERR_PATH_NOT_FOUND;
			FatFindFree(pEntry,hHandle);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		ExFatFind  *pItem;
		exfat_node *pNode;
		char 		cOld;
		int			iRet;

		DPD_LOCK(pData);

		cOld = pBuffer[iPos];
		pBuffer[iPos] = 0;

		   iRet = exfat_lookup(&pEntry->sExFat,&pNode,pBuffer);
		if(iRet!= 0)
			{
			if(iRet==-ENOENT)
					iError = DPD_ERR_PATH_NOT_FOUND;
			else	iError = ExFatError(iRet);
			DPD_UNLOCK(pData);
			break;
			}

		pEntry->uFolderCnt++;
		pBuffer[iPos] = cOld;

		if(!(pNode->flags&EXFAT_ATTRIB_DIR))				// Ist es ein Verzeichniss
			{
			exfat_put_node(&pEntry->sExFat,pNode);
			iError = DPD_ERR_PATH_NOT_FOUND;
			DPD_UNLOCK(pData);
			break;
			}

			hHandle = ExFatFindAlloc(pEntry,iDrive,pNode);
		if(!hHandle)
			{
			exfat_put_node(&pEntry->sExFat,pNode);
			iError = DPD_ERR_NO_HANDLES;
			break;
			}

		pItem = ExFatFindGet(pEntry,hHandle);

		   iRet = exfat_opendir(&pEntry->sExFat,pNode,&pItem->sNext);
		if(iRet!= 0)
			{
			exfat_put_node(&pEntry->sExFat,pNode);
			iError = ExFatError(iRet);
			DPD_UNLOCK(pData);
			break;
			}

		strfromutf8(pItem->aName,pBuffer+iPos,ENTRIES(pItem->aName));

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_find  *pItem;
		int			iTurn;
		int 		iRet;

			hHandle	= ExtFindAlloc(pEntry,iDrive);
		if(!hHandle)
			{
			iError = DPD_ERR_NO_HANDLES;
			break;
			}

		pItem = ExtFindGet(pEntry,hHandle);

		memcpy(pItem->cName,pBuffer+iPos,iLen-iPos+1);
		memcpy(pItem->cPath,pBuffer,iPos);

		if(iPos>0 && pBuffer[iPos-1]=='/')iPos--;

		pItem->iPathLen		= iPos;
		pItem->cPath[iPos]	= 0;
		pBuffer     [iPos]	= 0;
		iTurn				= 0;

		DPD_LOCK(pData);

Relink:	   iRet = ext4_dir_open(pEntry->hHandle,&pItem->sData,pBuffer);
		if(iRet==EMLINK)									// Links auflsen
			{
			   iRet = ExtResolvePath(pEntry->hHandle,pBuffer,pItem->sData.f.inode);
			if(iRet==EOK)
				{
				iTurn++;
				if(iTurn<4)goto Relink;
				iRet = EMLINK;
				}
			}

		if(iRet!=EOK)										// Gab es einen Fehler bei der Suche
			{
			iError=DPD_ERR_PATH_NOT_FOUND;
			ExtFindFree(pEntry,hHandle);
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	default:												// Unbekannter Partitionstyp

		iError = DPD_ERR_PARAM;
		break;
		}



	if(iError)
		{
		if(pError)*pError=iError;
		DiskClearFindStruct(pFind);
		return NULL;
		}


	   iError = DiskFindNext(pData,hHandle,pFind);			// Die erste Datei finden
	if(iError)
		{
		DiskFindClose(pData,hHandle);
		if(pError)*pError=DPD_ERR_PARAM;
		return NULL;
		}



return hHandle;
}


//*****************************************************************************
//*
//*		DiskFindNext
//*
//*****************************************************************************
//	Sucht die nchste Datei.
//	pData		: Zeiger auf die Laufwerksdaten
//	hHandle		: Ist das Handle der Suche.
//	pFind		: Hier werden die gefundenen Dateien gespeichert
//	Ergibt 0 wenn eine Datei gefunden wurde oder ein Fehlercode kleiner 0.
int DiskFindNext(DiskPartionData *pData,DiskFind hHandle,DiskFindStruct *pFind)
{
VolumeFind *pMyFind;
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;
unsigned	uItem;



	if(hHandle==DPU_FIND_ONE)								// War es eine Einzelsuche
		{
		DiskClearFindStruct(pFind);
		return DPD_ERR_FILE_NOT_FOUND;
		}

	uType = DPU_TYPE_OF_HANDLE(hHandle);
	uSlot = DPU_SLOT_OF_HANDLE(hHandle);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		DiskClearFindStruct(pFind);
		return DPD_ERR_PARAM;
		}


	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		wchar_t		aName[256];
		DIRFIND	   *pSearch;
		FILINFO		sInfo;
		FRESULT		iRet;

		DPD_LOCK(pData);

		sInfo.lfname 	= aName;
		sInfo.lfsize 	= ENTRIES(aName);

			pSearch = FatFindGet(pEntry,hHandle);
		if(!pSearch)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else for(;;)
			{
			iRet = f_readdir(&pEntry->sFatData,&pSearch->sData,&sInfo);

			if(iRet!=FR_OK)									// Keine weitere Datei gefunden
				{
				iError = FatError(iRet);
				break;
				}

			if(aName[0]==0)									// Kein langer Dateiname
				{
				strawcpy(aName,sInfo.fname,ENTRIES(aName));
				}

			if(strwcmp(aName,pSearch->aName))				// Passt der Name zum Filter
				{
				continue;
				}

			DiskFillName(aName,pFind->cName,ENTRIES(pFind->cName),DPD_UTF16);

			#if DISK_HAS_SHORT_NAME
			memcpy(pFind->cShortName,sInfo.fname,sizeof(pFind->cShortName));
			#endif

			pFind->dwTimeAccess = 0;
			pFind->dwTimeCreate = FatDosToTime(sInfo.cdate,sInfo.ctime);
			pFind->dwTimeWrite  = FatDosToTime(sInfo.fdate,sInfo.ftime);
			pFind->dwAttr 		= FatConvertAttr(sInfo.fattrib);
			pFind->dwSize       = sInfo.fsize;

			break;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		ExFatFind  *pItem;
		exfat_node *pNode;

		DPD_LOCK(pData);

			pItem = ExFatFindGet(pEntry,hHandle);
		if(!pItem)
			{
			iError = DPD_ERR_PARAM;
			}
		else for(;;)										// Verzeichniss einlesen
			{
				pNode = exfat_readdir(&pEntry->sExFat,&pItem->sNext);
			if(!pNode)
				{
				iError = DPD_ERR_FILE_NOT_FOUND;
				break;
				}

			if(strwcmp((wchar_t*)pNode->name,pItem->aName))	// Passt der Name zum Filter
				{
				exfat_put_node(&pEntry->sExFat,pNode);
				continue;
				}

			ExFatNoteToInfo(pNode,pFind);
			exfat_put_node(&pEntry->sExFat,pNode);
			break;
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry	           *pEntry=(ExtEntry*)pLocal;
		ext4_find		       *pSearch;
		ext4_attributes			sAttrib;
		const ext4_direntry    *pInfo;
		int 					iRet;

		DPD_LOCK(pData);

			pSearch = ExtFindGet(pEntry,hHandle);
		if(!pSearch)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else for(;;)
			{
				pInfo = ext4_dir_entry_next(&pSearch->sData);
			if(!pInfo)										// Keine weitere Datei gefunden
				{
				iError = DPD_ERR_FILE_NOT_FOUND;
				break;
				}

			if(strwcmp((char*)pInfo->name,pSearch->cName))	// Passt der Name zum Filter
				{
				continue;
				}

			if(pInfo->inode_type==EXT4_DE_SYMLINK)			// Link auflsen
				{
				   iRet = ExtResolveLink(pEntry->hHandle,pSearch->cPath,pSearch->iPathLen,(char*)pInfo->name,NULL,&sAttrib);
				if(iRet>0 && sAttrib.fmask==0xFFFFFFFF)goto SetAttr;
				}

			sAttrib.nlen = 0;

			   iRet = ext4_dir_get_attributes(&pSearch->sData,&sAttrib);
			if(iRet!=EOK)
				{
				iError = ExtError(iRet);
				break;
				}

			if((sAttrib.fmode&0xF000)==0xA000)				// Ist es ein Link
				{
				sAttrib.fsize = 0;

				if(pInfo->inode_type==EXT4_DE_UNKNOWN)
					{
					ExtResolveLink(pEntry->hHandle,pSearch->cPath,pSearch->iPathLen,(char*)pInfo->name,NULL,&sAttrib);
					}
				}

SetAttr:	pFind->dwAttr		= ExtConvertAttrib(sAttrib.fmode);
			pFind->dwTimeAccess	= sAttrib.atime;
			pFind->dwTimeCreate	= sAttrib.ctime;
			pFind->dwTimeWrite	= sAttrib.mtime;
			pFind->dwSize		= (DWORD) (sAttrib.fsize);
			pFind->dwAttr	   |= (DWORD)((sAttrib.fsize>>(32-DPD_HIGHMASK))&DPD_HIGHSHIFT);

			DiskFillName(pInfo->name,pFind->cName,ENTRIES(pFind->cName),DPD_UTF8);

			#if DISK_HAS_SHORT_NAME
			pFind->cShortName[0] = 0;
			#endif

			break;
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	case DPD_TYPE_PHYSICAL:									// Zugriff auf Root-Verzeichniss

		   uItem = DPU_ITEM_OF_HANDLE(hHandle);
		if(uItem>=DISK_MAX_FIND)
			{
			iError	= DPD_ERR_PARAM;
			break;
			}

			pMyFind = (VolumeFind*)pData->aFindPtr[uItem];
		if(!pMyFind)
			{
			iError	= DPD_ERR_PARAM;
			break;
			}

		DPD_LOCK(pData);

		for(pFind->cName[1]=0;;pMyFind->uSlot++)			// Durchlaufe alle Verzeichnisse
			{
			if(pMyFind->uSlot>=DPD_MAX_PARTIONS)
				{
				iError	= DPD_ERR_FILE_NOT_FOUND;
				break;
				}

			if(pData->aDriveType[pMyFind->uSlot]<=DPD_TYPE_PHYSICAL)
				{
				continue;
				}

			pFind->cName[0] = pData->aDriveSign[pMyFind->uSlot]+'0'-1;

			if(strwcmp(pFind->cName,pMyFind->cFilter))		// Passt der Name zum Filter
				{
				continue;
				}

			pMyFind->uSlot     += 1;
			pFind->dwTimeCreate = 1;
			pFind->dwTimeAccess = 1;
			pFind->dwTimeWrite  = 1;
			pFind->dwSize       = 0;
			pFind->dwAttr 		= DPD_DIR;

			break;
			}

		DPD_UNLOCK(pData);

		break;

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}



	if(iError)
		{
		DiskClearFindStruct(pFind);
		}


return iError;
}


//*****************************************************************************
//*
//*		DiskFindClose
//*
//*****************************************************************************
//	Schliet eine Suchanfrage.
//	pData		: Zeiger auf die Laufwerksdaten
//	hHandle		: Ist das Handle der Suche.
//	Ergibt 0 wenn eine Datei gefunden wurde oder ein Fehlercode kleiner 0.
int DiskFindClose(DiskPartionData *pData,DiskFind hHandle)
{
VolumeFind *pMyFind;
void	   *pLocal;
int			iError;
unsigned	uSlot;
unsigned	uType;
unsigned	uItem;



	if(hHandle==DPU_FIND_ONE)								// War es eine Einzelsuche
		{
		return DPD_ERR_NONE;
		}

	uType = DPU_TYPE_OF_HANDLE(hHandle);
	uSlot = DPU_SLOT_OF_HANDLE(hHandle);

	if(uSlot>=DPD_MAX_PARTIONS)								// Prfe Partitionsnummer
		{
		return DPD_ERR_PARAM;
		}


	pLocal = pData->aLocalData[uSlot];
	iError = DPD_ERR_NONE;


	switch(uType)
		{													// Was fr eine Partion ist gemountet
	#if	DPD_HAS_FAT

	case DPD_TYPE_FAT:										// Zugriff auf FAT-Partition
		{
		FatEntry   *pEntry=(FatEntry*)pLocal;
		DIRFIND	   *pSearch;

		DPD_LOCK(pData);

			pSearch = FatFindGet(pEntry,hHandle);
		if(!pSearch)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			// FRESULT	iRet;								// f_closedir ist in dieser Version nicht vorhanden
			// iRet = f_closedir(&pEntry->sFatData,&pSearch->sData);
			//
			//if(iRet!=FR_OK)								// Fehler beim Schlieen
			// 	{
			// 	iError = FatError(iRet);
			// 	}

			FatFindFree(pEntry,hHandle);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	#if	DPD_HAS_EXFAT

	case DPD_TYPE_EXFAT:									// Zugriff auf EXFAT-Partition
		{
		ExFatEntry *pEntry=(ExFatEntry*)pLocal;
		ExFatFind  *pItem;

		DPD_LOCK(pData);

			pItem = ExFatFindGet(pEntry,hHandle);
		if(!pItem)
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			exfat_closedir(&pEntry->sExFat,&pItem->sNext);
			exfat_put_node(&pEntry->sExFat, pItem->pNode);
			ExFatFindFree(pEntry,hHandle);
															// Gecachte Nodes freigeben
			if(pEntry->sExFat.cache_dirs>8 && !(pEntry->uFolderCnt&7))
				{
				pEntry->uOpenCnt = 0;
				exfat_flush_nodes  (&pEntry->sExFat);
				exfat_cache_release(&pEntry->sExFat,12);
				}
			}

		DPD_UNLOCK(pData);

		break;
		}

	#endif

	#if	DPD_HAS_EXT

	case DPD_TYPE_EXT:										// Zugriff auf EXT-Partition
		{
		ExtEntry   *pEntry=(ExtEntry*)pLocal;
		ext4_find  *pSearch;
		int			iRet;

		DPD_LOCK(pData);

			pSearch = ExtFindGet(pEntry,hHandle);
		if(!pSearch)										// Ist das Handle OK
			{
			iError = DPD_ERR_PARAM;
			}
		else{
			iRet = ext4_dir_close(&pSearch->sData);

			if(iRet!=EOK)									// Fehler beim Schlieen
			 	{
			 	iError = ExtError(iRet);
			 	}

			ExtFindFree(pEntry,hHandle);
			}

		DPD_UNLOCK(pData);
		break;
		}

	#endif

	case DPD_TYPE_PHYSICAL:									// Zugriff auf Root-Verzeichniss

		   uItem = DPU_ITEM_OF_HANDLE(hHandle);
		if(uItem>=DISK_MAX_FIND)
			{
			iError	= DPD_ERR_PARAM;
			break;
			}

			pMyFind = (VolumeFind*)pData->aFindPtr[uItem];
		if(!pMyFind)
			{
			iError	= DPD_ERR_PARAM;
			break;
			}

		pData->aFindPtr[uItem] = 0;
		delete pMyFind;
		break;

	default:												// Unbekannte Partition

		iError	= DPD_ERR_PARAM;
		break;
		}




return iError;
}


