//****************************************************************************
//*
//*
//*		StringFunctions.cpp
//*
//*
//****************************************************************************
//
//	Diese Datei enthlt diverse Textfunktionen.
//	Alle Funktionen sind auch Unicode fhig.
//
//	Copyright Anton Zechner (C) 2003
//
//	int	  vtostr    (...)		Zahl in String umwandeln (mit Einheiten "123.45 k")
//	int	  itostr    (...)		Zahl in String umwandeln (mit 0x.. 0b..)
//	int	  qtostr    (...)		64-Bit-Zahl in String umwandeln (mit 0x.. 0b..)
//	int	  strtoi    (...)		String in Zahl umwandeln (mit 0x.. 0b..)
//	int64 strtoq    (...)		String in Zahl umwandeln (mit 0x.. 0b..) (fr 64 Bit)
//	int	  strtoh    (...)		Hex-String in Zahl umwandeln (mit "01Bh FF00 )
//	int	  strtof    (...)		String in Fliekommazahl umwandel
//	int	  strtab    (...)		Leerraum einfgen
//	int	  strflen   (...)		Lnge einer Fliekommazahl bestimmen
//	int	  strfind   (...)		Suche String in String
//	int	  strifind  (...)		Suche String in String ohne Gros/Klein Unterscheidung
//	int	  strlfind  (...)		Suche String in String mit Lngenbeschrnkung
//	int	  strlifind (...)		Suche String in String ohne Gros/Klein Unterscheidung
//	int	  strchar   (...)		Suche Position nach Zeichen in String
//	int	  strlchar  (...)		Suche Zeichen in String
//	int	  strlspn   (...)		Suche erstes Zeichen vom Ende weg bis -1
//	int	  strcspnr  (...)		Suche erstes Zeichen vom Ende weg bis -1
//	int	  strspnr   (...)		Suche erstes ungleiches Zeichen vom Ende weg bis -1
//	int	  strcspnrn (...)		Suche erstes Zeichen vom Ende weg bis 0
//	int	  strspnrn  (...)		Suche erstes ungleiches Zeichen vom Ende weg bis 0
//	int	  strnlen   (...)		String Lnge bestimmen mit Beschrnkung
//	int	  strlcat   (...)		String anhngen mit Lngen Beschrnkung
//	int	  strlcpy   (...)		String kopieren mit Lngen Beschrnkung
//	int	  strtimeset(...)		Mache aus time_t einen Zeit-String
//	int	  strbits   (...)		Suche erstes ungleiches  Zeichen mit Bit-Feld
//	int	  strcbits  (...)		Suche erstes gleiches    Zeichen mit Bit-Feld
//	int	  strlbits  (...)		Suche erstes Position nach gleichen Zeichen mit Bit-Feld
//	int	  strtobits (...)		Bit-Feld erzeugen
//	int   strencode (...)		Text verschlsseln
//	int   strdecode (...)		Text entschlsseln
//	int   strcut    (...)		Leerzeichen vorne und hinten entfernen
//	int   strlcut   (...)		Leerzeichen vorne und hinten entfernen mit Lngenbeschrnkung
//	int   strpre    (...)		String vor einen anderen String einfgen
//	int   strlpre   (...)		String vor einen anderen String einfgen mit Lngenbeschrnkung
//	int	  strcalc   (...)		Berechnet eine Formel aus einem Text
//	int   strlenstr (...)		Berechne die Lnge einer Stringangabe "...."
//	int   strgetstr (...)		Hole den String aus einer Stringangabe "...."
//	int	  strtomem  (...)		Mache String zu Speicher "/x12/x34" -> :12 34
//	int	  memtostr  (...)		Mache Speicher zu String :11 12 "/x11/x12"
//	int	  utf8tochar(...)		Einen Text in ein Unicode-Zeichen umwandeln
//	int	  chartoutf8(...);		Einen Unicode-Zeichen in einen Text umwandeln
//
#include	"StringFunctions.h"
#include	<malloc.h>
#include	<math.h>

#ifdef	_MSC_VER
#pragma warning( disable :4035 )
#pragma warning( disable :4996 )
#pragma warning( disable :4514 )
#pragma warning( disable :4127 )
#endif

#ifdef	 linux
#include <sys/time.h>
#endif

#if			defined(_DEBUG) && defined(_MSC_VER) && !defined(NO_DEBUG_NEW)
#include	"DebugMemLeaks.h"
#endif

#define  STR_SLOT_COUNT		8
#define  STR_SLOT_SIZE		64
static   char				cStrBufferA[STR_SLOT_SIZE*STR_SLOT_COUNT];
static   wchar_t			cStrBufferW[STR_SLOT_SIZE*STR_SLOT_COUNT];
static   unsigned			uStrSlotA = 0;
static   unsigned			uStrSlotW = 0;
static	 int				iTimeZone =-1;


static	 const unsigned char	cHexDigits[]="0123456789ABCDEF";
static	 const unsigned			aStrJump[4]={0x0000,0x0008,0x0400,0x0408};

#define  GET_STR_DWORD(ptr)	   (*((unsigned*)(ptr)))
#define  CLR_BIT_PTR(bits,ptr)	 ((char*)(bits))[(*(unsigned char*)(ptr))>>3] &=~(1<<(*(ptr)&0x07))
#define  SET_BIT_PTR(bits,ptr)	 ((char*)(bits))[(*(unsigned char*)(ptr))>>3] |=  1<<(*(ptr)&0x07)
#define  GET_BIT_PTR(bits,ptr)	(((char*)(bits))[(*(unsigned char*)(ptr))>>3] &  (1<<(*(ptr)&0x07)))

#define  STR_NO_CHECK		0x7549B86A
#define  CMP_LETTER(a,b)	(((a)&~0x20)==((b)&~0x20))

#define  YEAR_DAYS			(365)					// Tage in   1 Jahr
#define	 YEAR_DAYS_4		(YEAR_DAYS*4+1)			// Tage in   4 Jahre
#define	 YEAR_DAYS_100		(YEAR_DAYS_4*25-1)		// Tage in 100 Jahre
#define	 YEAR_DAYS_400		(YEAR_DAYS_100*4+1)		// Tage in 400 Jahre



//*****************************************************************************
//*
//*		Masken-Tabelle fr strwlen Funktionen
//*
//*****************************************************************************
//	Bit0 != 0	Zahlen
//	Bit1 != 0	Grobuchstaben
//	Bit2 != 0	Kleinbuchstaben
//	Bit3 != 0	Zeichen '_'
//	Bit4 != 0	Zeichen >= 128
//	Bit5 != 0	Tabulator     '\t'
//	Bit6 != 0	Zeilenzeichen '\n' '\r'
//	Bit7 != 0	Leerzeichen   ' '
//	Bit8 != 0	Anfhrungszeichen '\"'
//	Bit9 != 0	Punkt '.'
//	Bit10!= 0	Anfhrungszeichen einfach '\''
//	Bit11!= 0	Dollarzeichen '$'
//	Bit12!= 0	Trennzeichen  ','
//	Bit13!= 0	Trennzeichen  ';'
//	Bit14!= 0	Klammeraffe   '@'
//	Bit15!= 0	Undzeichen    '&'
//	Bit16!= 0	Binrzahlen	  '0' bis '1'
//	Bit17!= 0	Oktal Zahlen  '0' bis '7'
//	Bit18!= 0	Hexadezimal   'a' bis 'f' und 'A bis 'F'
//	Bit19!= 0	Raute '#'
//	Bit20!= 0	Pfadzeichen '\\'
//	Bit21!= 0	Pfadzeichen '/'
//	Bit22!= 0	Pfadzeichen ':'
//	Bit23!= 0	Wildcart '*'
//	Bit24!= 0	Wildcart '?'
//	Bit25!= 0	Minus  '-'
//	Bit26!= 0	Plus   '+'
//	Bit27!= 0	Modulo '%'
//	Bit28!= 0	ANSI spezial Grobuchstaben
//	Bit29!= 0	ANSI spezial Kleinbuchstaben ohne ''
//	Bit30!= 0	Alle Zeichen die zu keiner anderen Gruppe gehren
//	Bit31!= 0	Nullzeichen

const unsigned int aStrwlenTab[256] =
		{
          /*00*/     /*01*/     /*02*/     /*03*/     /*04*/     /*05*/     /*06*/     /*07*/     /*08*/     /*09*/     /*0A*/     /*0B*/     /*0C*/     /*0D*/     /*0E*/     /*0F*/
/*00*/	0x80000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x00000020,0x00000040,0x40000000,0x40000000,0x00000040,0x40000000,0x40000000,/*00*/
/*10*/	0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,/*10*/
/*20*/	0x00000080,0x40000000,0x00000100,0x00080000,0x00000800,0x08000000,0x00008000,0x00000400,0x40000000,0x40000000,0x00800000,0x04000000,0x00001000,0x02000000,0x00000200,0x00200000,/*20*/
/*30*/	0x00030001,0x00030001,0x00020001,0x00020001,0x00020001,0x00020001,0x00020001,0x00020001,0x00000001,0x00000001,0x00400000,0x00002000,0x40000000,0x40000000,0x40000000,0x01000000,/*30*/
/*40*/	0x00004000,0x00040002,0x00040002,0x00040002,0x00040002,0x00040002,0x00040002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,/*40*/
/*50*/	0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x00000002,0x40000000,0x00100000,0x40000000,0x40000000,0x00000008,/*50*/
/*60*/	0x40000000,0x00040004,0x00040004,0x00040004,0x00040004,0x00040004,0x00040004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,/*60*/
/*70*/	0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x00000004,0x40000000,0x40000000,0x40000000,0x40000000,0x40000000,/*70*/
/*80*/	0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,/*80*/
/*90*/	0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,/*90*/
/*A0*/	0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,/*A0*/
/*B0*/	0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,0x00000010,/*B0*/
/*C0*/	0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,/*C0*/
/*D0*/	0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x10000000,0x00000010,0x20000000,/*D0*/
/*E0*/	0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,/*E0*/
/*F0*/	0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x20000000,0x00000010,0x00000010	/*F0*/
		};

const unsigned char aXmlStopTab[32] =			// fr '<' '>' '"' '\''
		{
      	//00   01   02   03   04   05   06   07   08   09   0A   0B   0C   0D   0E   0F
/*00*/	0x00,0x00,0x00,0x00,0x84,0x00,0x00,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*10*/	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		};

const unsigned char aXmlSpaceTab[32] =			// fr '<' '>' '"' '\'' ' ' '\t' '\n' '\r'
		{
      	//00   01   02   03   04   05   06   07   08   09   0A   0B   0C   0D   0E   0F
/*00*/	0x00,0x26,0x00,0x00,0x85,0x00,0x00,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
/*10*/	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		};


const unsigned char aStrCharValTab[128] =		// Zahlenwerte von Buchstaben (['A']=10 0x7F=falsch)
		{
      	//00   01   02   03   04   05   06   07   08   09   0A   0B   0C   0D   0E   0F
/*00*/	0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,	/*00*/
/*10*/	0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,	/*10*/
/*20*/	0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,	/*20*/
/*30*/	0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,	/*30*/
/*40*/	0x7F,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,	/*40*/
/*50*/	0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,0x20,0x21,0x22,0x23,0x7F,0x7F,0x7F,0x7F,0x7F,	/*50*/
/*60*/	0x7F,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,	/*60*/
/*70*/	0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,0x20,0x21,0x22,0x23,0x7F,0x7F,0x7F,0x7F,0x7F,	/*70*/
		};

const unsigned char aStrEscapeTab[128] =		// fr '\t' '\a' '\n' '\x..." der Wert 0xFF fr '\x...' und 0xFE fr '\77'
		{
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x22,0x00,0x00,0x00,0x00,0x27,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3F,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0x00,0x00,0x00,0x5C,0x00,0x00,0x00,
		0x00,0x07,0x08,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0A,0x00,
		0x00,0x00,0x0D,0x00,0x09,0x00,0x0B,0x00,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		};

static const unsigned	aStrStrBits[8]={ 0xFFFFFFFF,(1<<(' '-32))|(1<<(','-32))|(1<<(';'-32)),0,0,0,0,0,0};

static const unsigned char	ucAnsiCmpV[128] =	// Umlautumwandlung fr strvcmp
		{
		0x80,	/*128*/	'u',	/*129*/	'e',	/*130*/	'a',	/*131*/
		'a',	/*132*/	'a',	/*133*/	'a',	/*134*/	0x87,	/*135*/
		'e',	/*136*/	'e',	/*137*/	'e',	/*138*/	'i',	/*139*/
		'i',	/*140*/	'i',	/*141*/	'A',	/*142*/	'A',	/*143*/
		'E',	/*144*/	0x91,	/*145*/	0x92,	/*146*/	0x93,	/*147*/
		'o',	/*148*/	'o',	/*149*/	'u',	/*150*/	'u',	/*151*/
		0x98,	/*152*/	'U',	/*153*/	0x9A,	/*154*/	0x9B,	/*155*/
		0x9C,	/*156*/	0x9D,	/*157*/	0x9E,	/*158*/	0x9F,	/*159*/
		'a',	/*160*/	'i',	/*161*/	'o',	/*162*/	'u',	/*163*/
		'n',	/*164*/	'N',	/*165*/	0xA6,	/*166*/	0xA7,	/*167*/
		0xA8,	/*168*/	0xA9,	/*169*/	0xAA,	/*170*/	0xAB,	/*171*/
		0xAC,	/*172*/	0xAD,	/*173*/	0xAE,	/*174*/	0xAF,	/*175*/
		0xB0,	/*176*/	0xB1,	/*177*/	0xB2,	/*178*/	0xB3,	/*179*/
		0xB4,	/*180*/	0xB5,	/*181*/	0xB6,	/*182*/	0xB7,	/*183*/
		0xB8,	/*184*/	0xB9,	/*185*/	0xBA,	/*186*/	0xBB,	/*187*/
		0xBC,	/*188*/	0xBD,	/*189*/	0xBE,	/*190*/	0xBF,	/*191*/
		'A',	/*192*/	'A',	/*193*/	'A',	/*194*/	'A',	/*195*/
		'A',	/*196*/	'A',	/*197*/	'A',	/*198*/	'C',	/*199*/
		'E',	/*200*/	'E',	/*201*/	'E',	/*202*/	'E',	/*203*/
		'I',	/*204*/	'I',	/*205*/	'I',	/*206*/	'I',	/*207*/
		'D',	/*208*/	'N',	/*209*/	'O',	/*210*/	'O',	/*211*/
		'O',	/*212*/	'O',	/*213*/	'O',	/*214*/	0xD7,	/*215*/
		0xD8,	/*216*/	'U',	/*217*/	'U',	/*218*/	'U',	/*219*/
		'U',	/*220*/	'Y',	/*221*/	0xDE,	/*222*/	's',	/*223*/
		'a',	/*224*/	'a',	/*225*/	'a',	/*226*/	'a',	/*227*/
		'a',	/*228*/	'a',	/*229*/	'a',	/*230*/	'c',	/*231*/
		'e',	/*232*/	'e',	/*233*/	'e',	/*234*/	'e',	/*235*/
		'i',	/*236*/	'i',	/*237*/	'i',	/*238*/	'i',	/*239*/
		'd',	/*240*/	'n',	/*241*/	'o',	/*242*/	'o',	/*243*/
		'o',	/*244*/	'o',	/*245*/	'o',	/*246*/	0xF7,	/*247*/
		0xF8,	/*248*/	'u',	/*249*/	'u',	/*250*/	'u',	/*251*/
		'u',	/*252*/	'y',	/*253*/	0xDE,	/*254*/	'y',	/*255*/
		};

const unsigned __int64	aVtsMax[] =
	{
	1,					// 00
	10,					// 01
	100,				// 02
	1000,				// 03
	10000,				// 04
	100000,				// 05
	1000000,			// 06
	10000000,			// 07
	100000000,			// 08
	1000000000,			// 09 <- max int
	10000000000,		// 10
	100000000000,		// 11
	1000000000000,		// 12
	10000000000000,		// 13
	100000000000000,	// 14
	1000000000000000,	// 15
	};

static const char aVtsPrefix[]="yzafpnm kMGTPET";


static const int  aStrGetListTab[8] =
	{
	0x00403200,									// mit allem Trennen
	0x00002000,									// nur mit Semikolon trennen ';'
	0x00000200,									// nur mit Punkt trennen '.'
	0x00400000,									// nur mit Doppelpunkt trennen ':'
	0x00001000,									// nur mit Beistrich trennen ','
	0x00000000,									// nur mit Leerbereich trennen ' ' oder '\t'
	0x02000000,									// nur mit Minus trennen '-'
	0x00000020,									// nur mit Tabulator trennen '\t'
	};

typedef struct
	{
	char	cSize;								// Lnge des Namens
	char	cName[6];							// Name der Zeitzone
	short	sDiff;								// Unterschied (230==2h30min)
	}TimeZoneEntry;


static TimeZoneEntry	aTimeZones[] =			// https://www.timeanddate.de/zeitzonen/weltweit/
	{
	2,"WT",		   0,
	3,"ADT",	 400,
	3,"AST",     300,
	3,"CET",	 100,
	3,"CST",	-600,
	3,"EAT",	 300,
	3,"EDT",	-400,
	3,"EST",	-500,
	3,"FET",	 300,
	3,"GMT",	   0,
	3,"HST",   -1000,
	3,"JST",	 900,
	3,"MDT",    -600,
	3,"MEZ",	 100,
	3,"MSD",	 400,
	3,"MSK",	 300,
	3,"MST",    -700,
	3,"OEZ",     200,
	3,"PDT",    -700,
	3,"PST",    -800,
	3,"UTC",	   0,
	3,"WAT",	 100,
	3,"WST",	 100,
	4,"ACDT",	1030,
	4,"ACTS",	 930,
	4,"AEDT",	1100,
	4,"AEST",	1000,
	4,"AKDT",	-800,
	4,"AKST",	-900,
	4,"AWDT",    900,
	4,"AWST",    800,
	4,"CAST",    800,
	4,"CEST",	 200,
	4,"EAST",   -600,
	4,"MESZ",	 200,
	4,"NZDT",   1300,
	4,"NZST",   1200,
	4,"OESZ",    300,
	4,"WAST",	 200,
	5,"ACWST",	 845,
	};


//*****************************************************************************
//*
//*		strtimezome
//*
//*****************************************************************************
//	Sucht die Zeitdifferenz einer Zeitzone.
//	pText		: Ist der Name der Zeitzone (z.B. "CET")
//	iLen		: Ist die Lnge des Namens 
//	iUnicode	: Ist pName eine Unicode Text (1=ANSI 2=UNICODE)
//	Ergibt die Zeitdifferenz (230==2h30min) oder -1 wenn die Zeitzone nicht existiert.
static int strtimezome(const char *pText,int iLen,int iUnicode)
{
TimeZoneEntry  *pZone;
int				iSize;
int				iMin;
int				iMax;
int				iNow;
int				iCmp;
int				iPos;
int				iOff;


	iSize	= sizeof(aTimeZones)/sizeof(aTimeZones[0]);
	iMax	= iSize-1;
	iMin	= 0;


	while(iMax>=iMin)										// Binary-Search Algorithmus
		{
		iNow	= (iMax+iMin)/2;
		pZone	= aTimeZones+iNow;
		iCmp	= iLen-pZone->cSize;

		for(iPos=0,iOff=0;iCmp==0 && iPos<iLen;iPos++,iOff+=iUnicode)
			{
			iCmp  = pText[iOff]&~0x20;
			iCmp -= pZone->cName[iPos];
			}

		if(iNow<     0)return -1;
		if(iNow>=iSize)return -1;

		if(!iCmp)											// Sind die Namen gleich
			{
			return pZone->sDiff;
			}

		if(iCmp>0)											// Neue Grenzen berechnen
				iMin = iNow+1;
		else	iMax = iNow-1;
		}


return -1;
}

//****************************************************************************
//*
//*		strwlen
//*
//****************************************************************************
//	Zhlt die Zeichen die am Anfang eines Strings die zu einer Gruppe gehren
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	Ergibt die Position des ersten Zeichens das nicht zur Gruppe gehrt.
int _cdecl strwlen(const char *pString,int iMode)
{
#ifdef 	NO_STR_ASM

const unsigned char    *pPtr;


	pPtr = (unsigned char*)pString;

	for(;;)
		{
		if(!(aStrwlenTab[*pPtr]&iMode))break;
		pPtr++;
		}

return (int)((char*)pPtr-pString);

#else
_asm	{
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cl,[eax]				;//	Zeichen laden
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jz		e_0						;//			*
		inc		eax						;// Nchste Position
		jmp		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		}

#endif
}

int _cdecl strwlen(const wchar_t *pString,int iMode)
{
#ifdef 	NO_STR_ASM
const wchar_t   *pPtr;
unsigned				uChar;
int						iBits;

	pPtr = pString;

	for(;;)
		{
		uChar = *pPtr;
		iBits = (uChar<256)? aStrwlenTab[uChar]:0x40000000;

		if(!(iBits&iMode))break;

		pPtr++;
		}

return (int)(pPtr-pString);

#else
_asm	{
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cx,[eax]				;//	Zeichen laden
		cmp		ch,0					;//			*
		jne		s_2						;//			*
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jz		e_0						;//			*
		add		eax,2					;// Nchste Position
		jmp		s_1						;//	Ende erreicht

s_2:	test	edx,0x40000000			;// Zeichen ber 255
		jz		e_0						;//			*
		add		eax,2					;// Nchste Position
		jmp		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		shr		eax,1					;//			*
	}

#endif
}

//****************************************************************************
//*
//*		strwlen
//*
//****************************************************************************
//	Zhlt die Zeichen die am Anfang eines Strings die zu einer Gruppe gehren
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	iLen	: Ist die Lnge bis zu der maximal gezhlt wird
//	Ergibt die Position des ersten Zeichens das nicht zur Gruppe gehrt.
int _cdecl strwlen(const char *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

const unsigned char    *pPtr;


	pPtr = (unsigned char*)pString;

	for(;iLen>0;iLen--)
		{
		if(!(aStrwlenTab[*pPtr]&iMode))break;
		pPtr++;
		}

return (int)((char*)pPtr-pString);

#else
_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cl,[eax]				;//	Zeichen laden
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jz		e_0						;//			*
		inc		eax						;// Nchste Position
		dec		ebx						;//			*
		jnz		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		jmp		e_2						;//			*
e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
		}

#endif
}

int _cdecl strwlen(const wchar_t *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

const wchar_t   *pPtr;
unsigned				uChar;
int						iBits;

	pPtr = pString;

	for(;iLen>0;iLen--)
		{
		uChar = *pPtr;
		iBits = (uChar<256)? aStrwlenTab[uChar]:0x40000000;

		if(!(iBits&iMode))break;

		pPtr++;
		}

return (int)(pPtr-pString);

#else
_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cx,[eax]				;//	Zeichen laden
		cmp		ch,0					;//			*
		jne		s_3						;//			*
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
s_2:	jz		e_0						;//			*
		add		eax,2					;// Nchste Position
		dec		ebx						;//			*
		jnz		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		shr		eax,1					;//			*
		jmp		e_2						;//			*

s_3:	test	edx,0x40000000			;// Zeichen ber 255
		jnz		s_2						;//			*
		jmp		e_0						;//			*

e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
		}

#endif
}

//****************************************************************************
//*
//*		strcwlen
//*
//****************************************************************************
//	Zhlt die Zeichen die am Anfang eines Strings die nicht zu einer Gruppe gehren
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	Ergibt die Position des ersten Zeichens das zur Gruppe gehrt.
int _cdecl strcwlen(const char *pString,int iMode)
{
#ifdef 	NO_STR_ASM

const unsigned char    *pPtr;

	iMode ^= 0x80000000;
	pPtr   = (unsigned char*)pString;

	for(;;)
		{
		if(aStrwlenTab[*pPtr]&iMode)break;
		pPtr++;
		}

return (int)((char*)pPtr-pString);

#else
_asm	{
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString
		xor		edx,80000000h			;// Nullzeichen invertieren

s_1:	mov		cl,[eax]				;//	Zeichen laden
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jnz		e_0						;//			*
		inc		eax						;// Nchste Position
		jmp		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		}

#endif
}

int _cdecl strcwlen(const wchar_t *pString,int iMode)
{
#ifdef 	NO_STR_ASM
const wchar_t   *pPtr;
unsigned				uChar;
int						iBits;

	iMode ^= 0x80000000;
	pPtr   = pString;

	for(;;)
		{
		uChar = *pPtr;
		iBits = (uChar<256)? aStrwlenTab[uChar]:0x40000000;

		if(iBits&iMode)break;

		pPtr++;
		}

return (int)(pPtr-pString);

#else
_asm	{
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString
		xor		edx,80000000h			;// Nullzeichen invertieren

s_1:	mov		cx,[eax]				;//	Zeichen laden
		cmp		ch,0					;//			*
		jne		s_2						;//			*
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jnz		e_0						;//			*
		add		eax,2					;// Nchste Position
		jmp		s_1						;//	Ende erreicht

s_2:	test	edx,0x40000000			;// Zeichen ber 255
		jnz		e_0						;//			*
		add		eax,2					;// Nchste Position
		jmp		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		shr		eax,1					;//			*
	}

#endif
}


//****************************************************************************
//*
//*		strcwlen
//*
//****************************************************************************
//	Zhlt die Zeichen die am Anfang eines Strings die nicht zu einer Gruppe gehren
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	iLen	: Ist die Lnge bis zu der maximal gezhlt wird
//	Ergibt die Position des ersten Zeichens das zur Gruppe gehrt.
int _cdecl strcwlen(const char *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

const unsigned char    *pPtr;


	iMode ^= 0x80000000;
	pPtr   = (unsigned char*)pString;

	for(;iLen>0;iLen--)
		{
		if(aStrwlenTab[*pPtr]&iMode)break;
		pPtr++;
		}

return (int)((char*)pPtr-pString);

#else
_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString
		xor		edx,80000000h			;// Nullzeichen invertieren

s_1:	mov		cl,[eax]				;//	Zeichen laden
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jnz		e_0						;//			*
		inc		eax						;// Nchste Position
		dec		ebx						;//			*
		jnz		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		jmp		e_2						;//			*
e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
	}

#endif
}

int _cdecl strcwlen(const wchar_t *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

const wchar_t   *pPtr;
unsigned				uChar;
int						iBits;

	iMode ^= 0x80000000;
	pPtr   = pString;

	for(;iLen>0;iLen--)
		{
		uChar = *pPtr;
		iBits = (uChar<256)? aStrwlenTab[uChar]:0x40000000;

		if(iBits&iMode)break;

		pPtr++;
		}

return (int)(pPtr-pString);

#else
_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString
		xor		edx,80000000h			;// Nullzeichen invertieren

s_1:	mov		cx,[eax]				;//	Zeichen laden
		cmp		ch,0					;//			*
		jne		s_3						;//			*
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
s_2:	jnz		e_0						;//			*
		add		eax,2						;// Nchste Position
		dec		ebx						;//			*
		jnz		s_1						;//	Ende erreicht

e_0:	sub		eax,pString				;// Position zurckgeben
		shr		eax,1					;//			*
		jmp		e_2						;//			*

s_3:	test	edx,0x40000000			;// Zeichen ber 255
		jz		s_2						;//			*
		jmp		e_0						;//			*

e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
		}

#endif
}


//*****************************************************************************
//*
//*		strwfind
//*
//*****************************************************************************
//	Zhlt die Zeichen die am Anfang eines Strings die zu einer Gruppe gehren
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	iLen	: Ist die Lnge bis zu der maximal gezhlt wird
//	Ergibt die Position des ersten Zeichens das nicht zur Gruppe gehrt,
//	bzw. -1 wenn kein zeichnen der Gruppe gefunden wurde.
#ifdef _DEBUG

int _cdecl strwfind(const char *pString,int iMode,int iLen)
{
int		iRet;

	   iRet = strwlen(pString,iMode,iLen);
	if(iRet>=iLen || !pString[iRet])iRet=-1;

return iRet;
}

int _cdecl strwfind(const char *pString,int iMode)
{
int		iRet;

	iRet = strwlen(pString,iMode);

	if(!pString[iRet])iRet=-1;

return iRet;
}

int _cdecl strwfind(const wchar_t *pString,int iMode,int iLen)
{
int		iRet;

	   iRet = strwlen(pString,iMode,iLen);
	if(iRet>=iLen || !pString[iRet])iRet=-1;

return iRet;
}

int _cdecl strwfind(const wchar_t *pString,int iMode)
{
int		iRet;

	iRet = strwlen(pString,iMode);

	if(!pString[iRet])iRet=-1;

return iRet;
}

#endif


//*****************************************************************************
//*
//*		strwfind
//*
//*****************************************************************************
//	Zhlt die Zeichen die am Anfang eines Strings die nicht zu einer Gruppe gehren
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	iLen	: Ist die Lnge bis zu der maximal gezhlt wird
//	Ergibt die Position des ersten Zeichens das zur Gruppe gehrt,
//	bzw. -1 wenn kein zeichnen der Gruppe gefunden wurde.
#ifdef _DEBUG

int _cdecl strcwfind(const char *pString,int iMode,int iLen)
{
int		iRet;

	   iRet = strcwlen(pString,iMode,iLen);
	if(iRet>=iLen || !pString[iRet])iRet=-1;

return iRet;
}

int _cdecl strcwfind(const char *pString,int iMode)
{
int		iRet;

	iRet = strcwlen(pString,iMode);
	if(!pString[iRet])iRet=-1;

return iRet;
}

int _cdecl strcwfind(const wchar_t *pString,int iMode,int iLen)
{
int		iRet;

	   iRet = strcwlen(pString,iMode,iLen);
	if(iRet>=iLen || !pString[iRet])iRet=-1;

return iRet;
}

int _cdecl strcwfind(const wchar_t *pString,int iMode)
{
int		iRet;

	iRet = strcwlen(pString,iMode);
	if(!pString[iRet])iRet=-1;

return iRet;
}

#endif


//****************************************************************************
//*
//*		strbwlen
//*
//****************************************************************************
//	Zhlt die Zeichen die eines Strings die zu einer Gruppe gehren.
//	Die Suche geht dabei rckwrts und beginnt beim Zeichen [-1]
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	iLen	: Ist die Lnge bis zu der maximal gezhlt wird
//	Ergibt die Position des ersten Zeichens das nicht zur Gruppe gehrt.
int _cdecl strbwlen(const char *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

unsigned char	*pPtr;


	pString--;
	pPtr=(unsigned char*)pString;


	while(iLen>0 && aStrwlenTab[*pPtr]&iMode)
		{
		pPtr--;
		iLen--;
		}

return (int)(pString-(char*)pPtr);

#else

_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		dec		pString					;//	Bei Position -1 beginnen
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cl,[eax]				;//	Zeichen laden
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jz		e_0						;//			*
		dec		eax						;// Nchste Position
		dec		ebx						;//	Ende erreicht
		jnz		s_1						;//			*

e_0:	sub		eax,pString				;// Position zurckgeben
		neg		eax						;//			*
		jmp		e_2						;//			*
e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
		}

#endif
}

int _cdecl strbwlen(const wchar_t *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

unsigned				uChar;
int						iBits;
const wchar_t   *pPtr;


	pString--;
	pPtr=pString;


	while(iLen>0)
		{
		uChar = *pPtr;
		iBits = (uChar<256)? aStrwlenTab[*pPtr]:0x40000000;

		if(!(iBits&iMode))break;

		pPtr--;
		iLen--;
		}

return (int)(pString-pPtr);

#else

_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		sub		dword ptr pString,2		;//	Bei Position -1 beginnen
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cx,[eax]				;//	Zeichen laden
		cmp		ch,0					;//			*
		jne		s_3						;//			*
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jz		e_0						;//			*
s_2:	sub		eax,2					;// Nchste Position
		dec		ebx						;//	Ende erreicht
		jnz		s_1						;//			*

e_0:	sub		eax,pString				;// Position zurckgeben
		neg		eax						;//			*
		shr		eax,1					;//			*
		jmp		e_2						;//			*

s_3:	test	edx,0x40000000			;// Zeichen ber 255
		jnz		s_2						;//			*
		jmp		e_0						;//	Ende erreicht

e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
		}

#endif
}


//****************************************************************************
//*
//*		strdwlen
//*
//****************************************************************************
//	Zhlt die Zeichen die eines Strings die zu keiner Gruppe gehren.
//	Die Suche geht dabei rckwrts und beginnt beim Zeichen [-1]
//	pString	: Zeiger auf den pString
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	iLen	: Ist die Lnge bis zu der maximal gezhlt wird
//	Ergibt die Position des ersten Zeichens das zur Gruppe gehrt.
int _cdecl strdwlen(const char *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

unsigned char	*pPtr;


	pString--;
	pPtr=(unsigned char*)pString;


	while(iLen>0 && !(aStrwlenTab[*pPtr]&iMode))
		{
		pPtr--;
		iLen--;
		}

return (int)(pString-(char*)pPtr);

#else

_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		dec		pString					;//	Bei Position -1 beginnen
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cl,[eax]				;//	Zeichen laden
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jnz		e_0						;//			*
		dec		eax						;// Nchste Position
		dec		ebx						;//	Ende erreicht
		jnz		s_1						;//			*

e_0:	sub		eax,pString				;// Position zurckgeben
		neg		eax						;//			*
		jmp		e_2						;//			*
e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
		}

#endif
}

int _cdecl strdwlen(const wchar_t *pString,int iMode,int iLen)
{
#ifdef 	NO_STR_ASM

unsigned		uChar;
int				iBits;
const wchar_t  *pPtr;


	pString--;
	pPtr=pString;


	while(iLen>0)
		{
		uChar = *pPtr;
		iBits = (uChar<256)? aStrwlenTab[*pPtr]:0x40000000;

		if(iBits&iMode)break;

		pPtr--;
		iLen--;
		}

return (int)(pString-pPtr);

#else

_asm	{
		mov		ebx,iLen				;// ECX = Restliche Lnge
		cmp		ebx,0					;//	Genug Bytes brig ?
		jle		e_1						;//			*

		sub		dword ptr pString,2		;//	Bei Position -1 beginnen
		mov		edx,iMode				;// EDX = iMode
		xor		ecx,ecx					;// EBX = 0
		mov		eax,pString				;//	EAX = Zeiger auf pString

s_1:	mov		cx,[eax]				;//	Zeichen laden
		cmp		ch,0					;//			*
		jne		s_3						;//			*
		test	edx,[aStrwlenTab+ecx*4]	;//	Prfe Bits
		jnz		e_0						;//			*
s_2:	sub		eax,2					;// Nchste Position
		dec		ebx						;//	Ende erreicht
		jnz		s_1						;//			*

e_0:	sub		eax,pString				;// Position zurckgeben
		neg		eax						;//			*
		shr		eax,1					;//			*
		jmp		e_2						;//			*

s_3:	test	edx,0x40000000			;// Zeichen ber 255
		jz		s_2						;//			*
		jmp		e_0						;//	Ende erreicht

e_1:	xor		eax,eax					;// Null zurckgeben
e_2:
		}

#endif
}

//*****************************************************************************
//*
//*		strcharval
//*
//*****************************************************************************
//	Ergibt den Zahlenwert eines Zeichens. ('7'-->7  'A'-->10  G-->'16' ...)
#ifdef 	_DEBUG

int _cdecl strcharval(int iChar)
{
	if(iChar&~0x7F)return 0x7F;

return aStrCharValTab[iChar];
}

#endif

//*****************************************************************************
//*
//*		strwchar?
//*
//*****************************************************************************
//	Ergibt ob ein Zeichen zu einer Gruppe gehrt
#ifdef 	_DEBUG

int _cdecl strwchara(unsigned char ucChar,unsigned uFormat)
{
return aStrwlenTab[ucChar]&uFormat;
}

int _cdecl strwcharw(wchar_t usChar,unsigned uFormat)
{
	if(usChar>=256)return 0x40000000;

return aStrwlenTab[usChar]&uFormat;
}

#endif

//*****************************************************************************
//*
//*		strbchar?
//*
//*****************************************************************************
//	Ergibt ob ein Zeichen zu einer Bitfeldgruppe gehrt
#ifdef 	_DEBUG

int _cdecl strbcharw(unsigned char usChar,const void *pBits)
{
	if(usChar>=256)usChar=255;
	unsigned *pField=(unsigned*)pBits;
	return    pField[usChar/32]&(1<<(usChar&0x1F));
}

int _cdecl strbchara(unsigned char ucChar,const void *pBits)
{
	unsigned *pField=(unsigned*)pBits;
	return    pField[ucChar/32]&(1<<(ucChar&0x1F));
}

#endif

//****************************************************************************
//*
//*		strcspnr
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens aus einer Zeichenliste in
//  einem Text. Der Text wird dabei von hinten zum Anfang hin durchsucht.
//	pString		: Ist der Zeiger auf den Text
//	pList		: Ist der Zeiger auf die Zeichenliste
//  iLen		: Ist die Lnge des des Textes in Zeichen bzw. der Start-Offset
//				  der Suche. (Die Suche beginnt bei pString[iLen-1])
//				  Ist iLen==-1 wird die Lnge des des Textes genommen
//  Ergibt die Position des gefundenen Zeichens
//  Wird kein Zeichen gefunden wird -1 zurckgegeben
int	_cdecl strcspnr(const char *pString,const char *pList,int iLen)
{
int		aBits[8];


	if(iLen==-1)iLen=(int)strlen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;

	while(*pList)
		{
		SET_BIT_PTR(aBits,pList);
		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>=0;iLen--,pString--)
		{
		if(GET_BIT_PTR(aBits,pString))break;
		}

	return iLen;

//********************* Mit Assembler ****************************** ************
#else
__asm	{
		xor  ecx,ecx					;// Bits Null setzen
		mov  aBits   ,ecx				;//		*
		mov  aBits+4 ,ecx				;//		*
		mov  aBits+8 ,ecx				;//		*
		mov  aBits+12,ecx				;//		*
		mov  aBits+16,ecx				;//		*
		mov  aBits+20,ecx				;//		*
		mov  aBits+24,ecx				;//		*
		mov  aBits+28,ecx				;//		*

		mov  ebx,pList					;// Bits setzen
		lea  edx,aBits					;//		*
sc1:	mov  cl,[ebx]					;//		*
		jcxz sc2						;//		*
		bts  [edx],ecx					;//		*
		inc  ebx						;// nchstes Zeichen
		jmp  sc1						;//		*

sc2:	mov  eax,iLen					;// Zeichen suchen
		mov  ebx,pString				;//		*
		add  ebx,eax					;//		*
		inc  eax						;// Korrigieren
sc3:	dec  eax						;//		*
		jz	 sc4						;// Textende ?
		dec  ebx						;//		*
		mov  cl,[ebx]					;//		*
		bt   [edx],ecx					;//	Zeichen gefunden
		jnc  sc3						;//		*
sc4:	dec  eax						;// EAX korrigieren
		}

#endif
}

int	_cdecl strcspnr(const wchar_t *pString,const wchar_t *pList,int iLen)
{
const wchar_t *pUpper;
int					  aBits[8];


	if(iLen==-1)iLen=(int)wcslen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM



	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;
	pUpper  =0;


	while(*pList)
		{
		if(*pList<256)
			{
			SET_BIT_PTR(aBits,pList);
			}
		else{
			if(!pUpper)pUpper=pList;
			}

		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>=0;iLen--,pString--)
		{
		wchar_t		uChar;
		unsigned	uPos;

		   uChar = *pString;
		if(uChar<256)
			{
			if(GET_BIT_PTR(aBits,&uChar))break;
			continue;
			}

		for(uPos=0;pUpper[uPos];uPos++)
			{
			if(uChar==pUpper[uPos])return iLen;
			}
		}

	return iLen;

//********************* Mit Assembler ****************************** ************
#else
__asm	{
		xor   ecx,ecx					;// Bits Null setzen
		mov   aBits   ,ecx				;//		*
		mov   aBits+4 ,ecx				;//		*
		mov   aBits+8 ,ecx				;//		*
		mov   aBits+12,ecx				;//		*
		mov   aBits+16,ecx				;//		*
		mov   aBits+20,ecx				;//		*
		mov   aBits+24,ecx				;//		*
		mov   aBits+28,ecx				;//		*
		mov	  esi,ecx					;// ESI = pUpper

		mov   ebx,pList					;// Bits setzen
		lea   edx,aBits					;//		*
sc1:	mov   cx,[ebx]					;//		*
		jcxz  sc2						;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jnz   cg1						;//
		bts   [edx],ecx					;//		*
		add   ebx,2						;// nchstes Zeichen
		jmp   sc1						;//		*

cg1:	cmp	  esi,0						;// Ist pUpper Null ?
		cmove esi,ebx					;//
		add   ebx,2						;// nchstes Zeichen
		mov   pUpper,ebx				;//		*
		jmp   sc1						;//		*

cm1:	cmp	  esi,0						;// Ist pUpper Null ?
		je    sc3						;//		*
		push  eax						;// EAX retten
		mov   eax,ecx					;// AX zusuchendes Zeichen
		mov   edi,esi					;//		*
		mov   ecx,pUpper				;// ECX = Listengre
		repnz scasw						;// Suche Zeichen
		pop   eax						;//		*
		mov	  ecx,0						;//		*
		jnz   sc3						;//		*
		jmp	  sc4						;//		*

sc2:	sub	  pUpper,esi				;// EDX = Gre pUpper
		shr   dword ptr pUpper,1		;//		*
		mov   eax,iLen					;// Zeichen suchen
		mov   ebx,pString				;//		*
		lea   ebx,[ebx+eax*2]			;//		*
		inc   eax						;// Korrigieren
sc3:	dec   eax						;//		*
		jz	  sc4						;// Textende ?
		sub   ebx,2						;//		*
		mov   cx,[ebx]					;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jne   cm1						;//		*
		bt    [edx],ecx					;//	Zeichen gefunden
		jnc   sc3						;//		*
sc4:	dec   eax						;// EAX korrigieren
		}

#endif
}

//****************************************************************************
//*
//*		strspnr
//*
//****************************************************************************
//	Sucht das erste nicht Auftreten eines Zeichens aus einer Zeichenliste in einem Text
//  Der Text wird dabei von hinten zum Anfang hin durchsucht.
//	pString		: Ist der Zeiger auf den Text
//	pList		: Ist der Zeiger auf die Zeichenliste
//  iLen		: Ist die Lnge des des Textes in Zeichen bzw. der Start-Offset
//				  der Suche. (Die Suche beginnt bei pString[iLen-1])
//				  Ist iLen==-1 wird die Lnge des des Strings genommen
//  Ergibt die Position des gefundenen Zeichens
//  Wird ein Zeichen gefunden wird -1 zurckgegeben
int	_cdecl strspnr(const char *pString,const char *pList,int iLen)
{
int		aBits[8];


	if(iLen==-1)iLen=(int)strlen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;

	while(*pList)
		{
		SET_BIT_PTR(aBits,pList);
		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>=0;iLen--,pString--)
		{
		if(!GET_BIT_PTR(aBits,pString))break;
		}

	return iLen;

//********************* Mit Assembler ****************************** ************
#else
__asm	{
		xor  ecx,ecx					;// Bits Null setzen
		mov  aBits   ,ecx				;//		*
		mov  aBits+4 ,ecx				;//		*
		mov  aBits+8 ,ecx				;//		*
		mov  aBits+12,ecx				;//		*
		mov  aBits+16,ecx				;//		*
		mov  aBits+20,ecx				;//		*
		mov  aBits+24,ecx				;//		*
		mov  aBits+28,ecx				;//		*

		mov  ebx,pList					;// Bits setzen
		lea  edx,aBits					;//		*
sc1:	mov  cl,[ebx]					;//		*
		jcxz sc2						;//		*
		bts  [edx],ecx					;//		*
		inc  ebx						;// nchstes Zeichen
		jmp  sc1						;//		*

sc2:	mov  eax,iLen					;// Zeichen suchen
		mov  ebx,pString				;//		*
		add  ebx,eax					;//		*
		inc  eax						;// Korrigieren
sc3:	dec  eax						;//		*
		jz	 sc4						;// Textende ?
		dec  ebx						;//		*
		mov  cl,[ebx]					;//		*
		bt   [edx],ecx					;//	Zeichen gefunden
		jc   sc3						;//		*
sc4:	dec  eax						;// EAX korrigieren
		}

#endif
}

int	_cdecl strspnr(const wchar_t *pString,const wchar_t *pList,int iLen)
{
const wchar_t *pUpper;
int					  aBits[8];


	if(iLen==-1)iLen=(int)wcslen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM



	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;
	pUpper  =0;


	while(*pList)
		{
		if(*pList<256)
			{
			SET_BIT_PTR(aBits,pList);
			}
		else{
			if(!pUpper)pUpper=pList;
			}

		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>=0;iLen--,pString--)
		{
		wchar_t		uChar;
		unsigned	uPos;

		   uChar = *pString;
		if(uChar<256)
			{
			if(!GET_BIT_PTR(aBits,&uChar))break;
			continue;
			}

		for(uPos=0;pUpper[uPos];uPos++)
			{
			if(uChar!=pUpper[uPos])return iLen;
			}
		}

	return iLen;

//********************* Mit Assembler *****************************************
#else
__asm	{
		xor   ecx,ecx					;// Bits Null setzen
		mov   aBits   ,ecx				;//		*
		mov   aBits+4 ,ecx				;//		*
		mov   aBits+8 ,ecx				;//		*
		mov   aBits+12,ecx				;//		*
		mov   aBits+16,ecx				;//		*
		mov   aBits+20,ecx				;//		*
		mov   aBits+24,ecx				;//		*
		mov   aBits+28,ecx				;//		*
		mov	  esi,ecx					;// ESI = pUpper

		mov   ebx,pList					;// Bits setzen
		lea   edx,aBits					;//		*
sc1:	mov   cx,[ebx]					;//		*
		jcxz  sc2						;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jnz   cg1						;//
		bts   [edx],ecx					;//		*
		add   ebx,2						;// nchstes Zeichen
		jmp   sc1						;//		*

cg1:	cmp	  esi,0						;// Ist pUpper Null ?
		cmove esi,ebx					;//
		add   ebx,2						;// nchstes Zeichen
		mov   pUpper,ebx				;//		*
		jmp   sc1						;//		*

cm1:	cmp	  esi,0						;// Ist pUpper Null ?
		je    sc3						;//		*
		push  eax						;// EAX retten
		mov   eax,ecx					;// AX zusuchendes Zeichen
		mov   edi,esi					;//		*
		mov   ecx,pUpper				;// ECX = Listengre
		repnz scasw						;// Suche Zeichen
		pop   eax						;//		*
		mov	  ecx,0						;//		*
		jz    sc3						;//		*
		jmp	  sc4						;//		*

sc2:	sub	  pUpper,esi				;// EDX = Gre pUpper
		shr   dword ptr pUpper,1		;//		*
		mov   eax,iLen					;// Zeichen suchen
		mov   ebx,pString				;//		*
		lea   ebx,[ebx+eax*2]			;//		*
		inc   eax						;// Korrigieren
sc3:	dec   eax						;//		*
		jz	  sc4						;// Textende ?
		sub   ebx,2						;//		*
		mov   cx,[ebx]					;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jne   cm1						;//		*
		bt    [edx],ecx					;//	Zeichen gefunden
		jc	  sc3						;//		*
sc4:	dec   eax						;// EAX korrigieren
		}

#endif
}

//****************************************************************************
//*
//*		strcspnrn
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens aus einer Zeichenliste in einem
//	Text. Der Text wird dabei von hinten zum Anfang hin durchsucht, bis
//	zum ersten Zeichen.
//	pString		: Ist der Zeiger auf den Text
//	pList		: Ist der Zeiger auf die Zeichenliste
//  iLen		: Ist die Lnge des des Textes in Zeichen bzw. der Start-Offset
//				  der Suche. (Die Suche beginnt bei pString[iLen-1])
//				  Ist iLen==-1 wird die Lnge des des Strings genommen
//  Ergibt die Position des gefundenen Zeichens
//  Wird kein Zeichen gefunden wird 0 zurckgegeben
int	_cdecl strcspnrn(const char *pString,const char *pList,int iLen)
{
int		aBits[8];


	if(iLen==-1)iLen=(int)strlen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	if(iLen<=1)return 0;

	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;

	while(*pList)
		{
		SET_BIT_PTR(aBits,pList);
		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>0;iLen--,pString--)
		{
		if(GET_BIT_PTR(aBits,pString))break;
		}

	return iLen;

//********************* Mit Assembler ****************************** ************
#else
__asm	{
		xor  ecx,ecx					;// Bits Null setzen
		mov  aBits   ,ecx				;//		*
		mov  aBits+4 ,ecx				;//		*
		mov  aBits+8 ,ecx				;//		*
		mov  aBits+12,ecx				;//		*
		mov  aBits+16,ecx				;//		*
		mov  aBits+20,ecx				;//		*
		mov  aBits+24,ecx				;//		*
		mov  aBits+28,ecx				;//		*

		mov  ebx,pList					;// Bits setzen
		lea  edx,aBits					;//		*
sc1:	mov  cl,[ebx]					;//		*
		jcxz sc2						;//		*
		bts  [edx],ecx					;//		*
		inc  ebx						;// nchstes Zeichen
		jmp  sc1						;//		*

sc2:	mov  eax,iLen					;// Zeichen suchen
		mov  ebx,pString				;//		*
		add  ebx,eax					;//		*
sc3:	dec  eax						;//		*
		jz	 sc4						;// Textende ?
		dec  ebx						;//		*
		mov  cl,[ebx]					;//		*
		bt   [edx],ecx					;//	Zeichen gefunden
		jnc  sc3						;//		*
sc4:	}

#endif
}

int	_cdecl strcspnrn(const wchar_t *pString,const wchar_t *pList,int iLen)
{
const wchar_t *pUpper;
int					  aBits[8];


	if(iLen==-1)iLen=(int)wcslen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	if(iLen<=1)return 0;

	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;
	pUpper  =0;


	while(*pList)
		{
		if(*pList<256)
			{
			SET_BIT_PTR(aBits,pList);
			}
		else{
			if(!pUpper)pUpper=pList;
			}

		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>0;iLen--,pString--)
		{
		wchar_t		uChar;
		unsigned	uPos;

		   uChar = *pString;
		if(uChar<256)
			{
			if(GET_BIT_PTR(aBits,&uChar))break;
			continue;
			}

		for(uPos=0;pUpper[uPos];uPos++)
			{
			if(uChar==pUpper[uPos])return iLen;
			}
		}

	return iLen;

//********************* Mit Assembler ****************************** ************
#else
__asm	{
		xor   ecx,ecx					;// Bits Null setzen
		mov   aBits   ,ecx				;//		*
		mov   aBits+4 ,ecx				;//		*
		mov   aBits+8 ,ecx				;//		*
		mov   aBits+12,ecx				;//		*
		mov   aBits+16,ecx				;//		*
		mov   aBits+20,ecx				;//		*
		mov   aBits+24,ecx				;//		*
		mov   aBits+28,ecx				;//		*
		mov	  esi,ecx					;// ESI = pUpper


		mov   ebx,pList					;// Bits setzen
		lea   edx,aBits					;//		*
sc1:	mov   cx,[ebx]					;//		*
		jcxz  sc2						;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jnz   cg1						;//
		bts   [edx],ecx					;//		*
		add   ebx,2						;// nchstes Zeichen
		jmp   sc1						;//		*

cg1:	cmp	  esi,0						;// Ist pUpper Null ?
		cmove esi,ebx					;//
		add   ebx,2						;// nchstes Zeichen
		mov   pUpper,ebx				;//		*
		jmp   sc1						;//		*

cm1:	cmp	  esi,0						;// Ist pUpper Null ?
		je    sc3						;//		*
		push  eax						;// EAX retten
		mov   eax,ecx					;// AX zusuchendes Zeichen
		mov   edi,esi					;//		*
		mov   ecx,pUpper				;// ECX = Listengre
		repnz scasw						;// Suche Zeichen
		pop   eax						;//		*
		mov	  ecx,0						;//		*
		jnz   sc3						;//		*
		jmp	  sc4						;//		*

sc2:	sub	  pUpper,esi				;// EDX = Gre pUpper
		shr   dword ptr pUpper,1		;//		*
		mov   eax,iLen					;// Zeichen suchen
		mov   ebx,pString				;//		*
		lea   ebx,[ebx+eax*2]			;//		*
sc3:	dec   eax						;//		*
		jz	  sc4						;// Textende ?
		sub   ebx,2						;//		*
		mov   cx,[ebx]					;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jne   cm1						;//		*
		bt    [edx],ecx					;//	Zeichen gefunden
		jnc   sc3						;//		*
sc4:	}

#endif
}


//****************************************************************************
//*
//*		strspnrn
//*
//****************************************************************************
//	Sucht das erste nicht Auftreten eines Zeichens aus einer Zeichenliste in
//	einem Text. Der Text wird dabei von hinten zum Anfang hin durchsucht,
//	bis zum ersten Zeichen.
//	pString		: Ist der Zeiger auf den Text der durchsucht werden soll.
//	pList		: Ist der Zeiger auf die Zeichenliste.
//  iLen		: Ist die Lnge des des Textes in Zeichen bzw. der Start-Offset
//				  der Suche. (Die Suche beginnt bei pString[iLen-1])
//				  Ist iLen==-1 wird die Lnge des des Strings genommen
//  Ergibt die Position des gefundenen Zeichens
//  Wird ein Zeichen gefunden wird 0 zurckgegeben
int	_cdecl strspnrn(const char *pString,const char *pList,int iLen)
{
int		aBits[8];


	if(iLen==-1)iLen=(int)strlen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	if(iLen<=1)return 0;

	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;

	while(*pList)
		{
		SET_BIT_PTR(aBits,pList);
		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>0;iLen--,pString--)
		{
		if(!GET_BIT_PTR(aBits,pString))break;
		}

	return iLen;

//********************* Mit Assembler ****************************** ************
#else
__asm	{
		xor  ecx,ecx					;// Bits Null setzen
		mov  aBits   ,ecx				;//		*
		mov  aBits+4 ,ecx				;//		*
		mov  aBits+8 ,ecx				;//		*
		mov  aBits+12,ecx				;//		*
		mov  aBits+16,ecx				;//		*
		mov  aBits+20,ecx				;//		*
		mov  aBits+24,ecx				;//		*
		mov  aBits+28,ecx				;//		*

		mov  ebx,pList					;// Bits setzen
		lea  edx,aBits					;//		*
sc1:	mov  cl,[ebx]					;//		*
		jcxz sc2						;//		*
		bts  [edx],ecx					;//		*
		inc  ebx						;// nchstes Zeichen
		jmp  sc1						;//		*

sc2:	mov  eax,iLen					;// Zeichen suchen
		mov  ebx,pString				;//		*
		add  ebx,eax					;//		*
sc3:	dec  eax						;//		*
		jz	 sc4						;// Textende ?
		dec  ebx						;//		*
		mov  cl,[ebx]					;//		*
		bt   [edx],ecx					;//	Zeichen gefunden
		jc   sc3						;//		*
sc4:	}

#endif
}

int	_cdecl strspnrn(const wchar_t *pString,const wchar_t *pList,int iLen)
{
const wchar_t *pUpper;
int					  aBits[8];


	if(iLen==-1)iLen=(int)wcslen(pString);

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	if(iLen<=1)return 0;

	aBits[0]=0;
	aBits[1]=0;
	aBits[2]=0;
	aBits[3]=0;
	aBits[4]=0;
	aBits[5]=0;
	aBits[6]=0;
	aBits[7]=0;
	pUpper  =0;


	while(*pList)
		{
		if(*pList<256)
			{
			SET_BIT_PTR(aBits,pList);
			}
		else{
			if(!pUpper)pUpper=pList;
			}

		pList++;
		}

	iLen--;
	pString+=iLen;

	for(;iLen>0;iLen--,pString--)
		{
		wchar_t		uChar;
		unsigned	uPos;

		   uChar = *pString;
		if(uChar<256)
			{
			if(!GET_BIT_PTR(aBits,&uChar))break;
			continue;
			}

		for(uPos=0;pUpper[uPos];uPos++)
			{
			if(uChar!=pUpper[uPos])return iLen;
			}
		}

	return iLen;

//********************* Mit Assembler ****************************************
#else
__asm	{
		xor   ecx,ecx					;// Bits Null setzen
		mov   aBits   ,ecx				;//		*
		mov   aBits+4 ,ecx				;//		*
		mov   aBits+8 ,ecx				;//		*
		mov   aBits+12,ecx				;//		*
		mov   aBits+16,ecx				;//		*
		mov   aBits+20,ecx				;//		*
		mov   aBits+24,ecx				;//		*
		mov   aBits+28,ecx				;//		*
		mov	  esi,ecx					;// ESI = pUpper

		mov   ebx,pList					;// Bits setzen
		lea   edx,aBits					;//		*
sc1:	mov   cx,[ebx]					;//		*
		jcxz  sc2						;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jnz   cg1						;//
		bts   [edx],ecx					;//		*
		add   ebx,2						;// nchstes Zeichen
		jmp   sc1						;//		*

cg1:	cmp	  esi,0						;// Ist pUpper Null ?
		cmove esi,ebx					;//
		add   ebx,2						;// nchstes Zeichen
		mov   pUpper,ebx				;//		*
		jmp   sc1						;//		*

cm1:	cmp	  esi,0						;// Ist pUpper Null ?
		je    sc3						;//		*
		push  eax						;// EAX retten
		mov   eax,ecx					;// AX zusuchendes Zeichen
		mov   edi,esi					;//		*
		mov   ecx,pUpper				;// ECX = Listengre
		repnz scasw						;// Suche Zeichen
		pop   eax						;//		*
		mov	  ecx,0						;//		*
		jz    sc3						;//		*
		jmp	  sc4						;//		*

sc2:	sub	  pUpper,esi				;// EDX = Gre pUpper
		shr   dword ptr pUpper,1		;//		*
		mov   eax,iLen					;// Zeichen suchen
		mov   ebx,pString				;//		*
		lea   ebx,[ebx+eax*2]			;//		*
sc3:	dec   eax						;//		*
		jz	  sc4						;// Textende ?
		sub   ebx,2						;//		*
		mov   cx,[ebx]					;//		*
		cmp   ch,0						;// Zeichen kleiner 256 ?
		jne   cm1						;//		*
		bt    [edx],ecx					;//	Zeichen gefunden
		jc    sc3						;//		*
sc4:	}

#endif
}


//****************************************************************************
//*
//*		strtimeset
//*
//****************************************************************************
//	Konvertiert einen Unix-Zeit Variabel(time_t) in einen String
//	pString	: Ist der Puffer in dem der Zeit-String gespeichert wird.
//	iTime	: Ist die Unix-Zeit in Sekunden seit 1.Jan.1970
//	iMode	: Flags 	STR_WEEKDAY		Wochentag angeben
//						STR_TIME		Zeit angeben
//						STR_DATE		Datum angeben
//						STR_ZONE		Die Zeitzone anhngen ( ...+10:00)
//						STR_ABC			Monat angeben
//						STR_SEC			Sekunden angeben
//						STR_LONG		Das Jahr 4 stellig ausgeben
//						STR_GLOBAL		Eine globale Zeit ausgeben
//						STR_NO_DST		Sommerzeitumschaltung nicht bercksichtigen
//						STR_NO_NULL		Kein Null-Byte am Ende anhngen
//						STR_NO_COLON	Keinen Doppelpunkt bei der Zeit einfgen
//						STR_NO_SPACE	Keinen Space am Ende des Datums
//						STR_USEUNTER	'_' statt ' ' verwenden
//						STR_USESLASH	'/' statt '-' und '.' verwenden
//						STR_UNTERLINE	Ein '_' statt ':' bei der Zeit einfgen
//						STR_WEEKCOMMA	Beistrich nach Wochentag einfgen
//						STR_ADD_CHAR	Ein Zeichen zwischen Datum und Zeit (Bit24..31)
//						STR_ADD_SPACE	Ein weiters Leerzeichen zwischen Datum und Zeit
//						STR_ADD_TCHAR	Ein 'T' Zeichen zwischen Datum und Zeit
//						STR_ADD_UNTER	Ein '_' Zeichen zwischen Datum und Zeit
// Ergibt die Lnge des Zeit-Strings in Zeichen
static const char *pWeekDaysA[]={"So " ,"Mo " ,"Di " ,"Mi " ,"Do " ,"Fr " ,"Sa "};
static const char *pMonthStrA[]={"Jan.","Feb.","Mr.","Apr.","Mai.","Jun.","Jul.","Aug.","Sep.","Okt.","Nov.","Dez."};
static const char *pMonthEngA[]={"Jan.","Feb.","Mar.","Apr.","May.","Jun.","Jul.","Aug.","Sep.","Oct.","Nov.","Dec."};

int _cdecl strtimeset(char *pString,time_t iTime,int iMode)
{
char		cSpace,cBreak,cColon;
int			iSize,iAdd=0;
struct tm	sTmTime;
struct tm  *pTimePtr;



	if(sizeof(iTime)==4 && (iTime&0x80000000))		// berlauf in 2039
		{
		iTime -= 3600*24*(365*65+16);				// minus 65 Jahre
		iAdd   = 65;
		}


	iSize    = 0;

	if((iMode&(STR_GLOBAL|STR_NO_DST))==STR_NO_DST)	// Lokalezeit ohne Sommerzeit
		{
		if(iTimeZone==-1)
			{
			#ifdef _MSC_VER
			time(0);
			iTimeZone = _timezone;

			#else

			struct timezone	sZone;
			struct timeval	sTime;
			gettimeofday(&sTime,&sZone);
			iTimeZone = sZone.tz_minuteswest*60;

			#endif
			}

		iTime += iTimeZone;
		iMode |= STR_GLOBAL;

		if(iTime<0)iTime=0;
		}

    if(iMode&STR_GLOBAL)
            pTimePtr = gmtime   (&iTime);
    else    pTimePtr = localtime(&iTime);

	if(pTimePtr)									// Zeit umwandeln
		{
		sTmTime=*pTimePtr;
		}
	else{
		memset(&sTmTime,0,sizeof(sTmTime));
		}

	sTmTime.tm_year+=iAdd;

	cSpace = (iMode&STR_USEUNTER)? '_':' ';


	if(iMode&STR_WEEKDAY)							// Wochentag ausgben
		{
		*(int*)pString=*(int*)pWeekDaysA[sTmTime.tm_wday];
		pString += 3;
		iSize    = 3;

		if(iMode&STR_WEEKCOMMA)
			{
			pString[-1]  = (iMode&STR_USESLASH)? '/':',';
			pString[ 0]  = cSpace;
			pString     += 1;
			iSize        = 1;
			}
		}

	if(iMode&STR_DATE)								// Datum augeben TT-MM-JJJJ
		{
		cBreak = (iMode&STR_USESLASH)? '/':'.';

		pString[0] = (char)((sTmTime.tm_mday/10)+'0');
		pString[1] = (char)((sTmTime.tm_mday%10)+'0');
		pString[2] = cBreak;
		pString   += 3;
		iSize     += 3;

		if(iMode&STR_ABC)							// Monat als Text
			{
			*(int*)pString=*(int*)pMonthStrA[sTmTime.tm_mon];
			pString+=4;iSize+=4;
			}
		else{										// Monat als Zahl
							     sTmTime.tm_mon++;
			pString[0] = (char)((sTmTime.tm_mon/10)+'0');
			pString[1] = (char)((sTmTime.tm_mon%10)+'0');
			pString[2] = cBreak;
			pString   += 3;
			iSize     += 3;
			}
													// Vierstelliges Datum
		if(iMode&STR_LONG)
			{
			*(int*)pString=(sTmTime.tm_year>=100)? *(int*)"20 ":*(int*)"19 ";
			pString += 2;
			iSize   += 2;
			}

		pString[0] = (char)((sTmTime.tm_year/10)%10+'0');
		pString[1] = (char)((sTmTime.tm_year%10)   +'0');

		if(iMode&STR_NO_SPACE)						// Kein Space am Datumsende
			{
			pString   += 2;
			iSize     += 2;
			}
		else{
			pString[2] = cSpace;
			pString   += 3;
			iSize     += 3;
			}
		}
	else if(iMode&STR_DATE_ISO)						// Datum im englischen Format JJJJ-MM-TT
		{
		cBreak = (iMode&STR_USESLASH)? '/':'-';

		if(iMode&STR_LONG)							// Vierstelliges Datum
			{
			*(int*)pString=(sTmTime.tm_year>=100)? *(int*)"20 ":*(int*)"19 ";
			pString+=2;iSize+=2;
			}

		pString[0] = (char)((sTmTime.tm_year/10)%10+'0');
		pString[1] = (char)((sTmTime.tm_year%10)   +'0');
		pString[2] = cBreak;
		pString   += 3;
		iSize     += 3;

		if(iMode&STR_ABC)							// Monat als Text
			{
			*(int*)pString=*(int*)pMonthStrA[sTmTime.tm_mon];
			pString[3] = cBreak;
			pString   += 4;
			iSize     += 4;
			}										// Monat als Zahl
		else{
					             sTmTime.tm_mon++;
			pString[0] = (char)((sTmTime.tm_mon/10)+'0');
			pString[1] = (char)((sTmTime.tm_mon%10)+'0');
			pString[2] = cBreak;
			pString   += 3;
			iSize     += 3;
			}

		pString[0] = (char)((sTmTime.tm_mday/10)+'0');
		pString[1] = (char)((sTmTime.tm_mday%10)+'0');

		if(iMode&STR_NO_SPACE)						// Kein Space am Datumsende
			{
			pString   += 2;
			iSize     += 2;
			}
		else{
			pString[2] = cSpace;
			pString   += 3;
			iSize     += 3;
			}
		}

	if(iMode&STR_TIME)								// Zeitaugeben HH:MM
		{
		if(iMode&STR_ADD_CHAR)
			{
			pString[0] = (char)(iMode>>24);
			pString   += 1;
			iSize     += 1;
			}

		pString[0] = (char)((sTmTime.tm_hour/10)+'0');
		pString[1] = (char)((sTmTime.tm_hour%10)+'0');

		if(iMode&STR_USEHYPHEN)
				cColon = '-';
		else	cColon = (iMode&STR_UNTERLINE)? '_':':';

		iAdd   = (iMode&STR_NO_COLON )? -1:0;

		pString[2] = cColon; pString+=iAdd;	iSize+=iAdd;
		pString[3] = (char)((sTmTime.tm_min /10)+'0');
		pString[4] = (char)((sTmTime.tm_min %10)+'0');
		pString   += 5;
		iSize     += 5;

		if(iMode&STR_SEC)							// Sekunden anhngen HH:MM:SS
			{
			pString[0] = cColon; pString+=iAdd;	iSize+=iAdd;
			pString[1] = (char)((sTmTime.tm_sec /10)+'0');
			pString[2] = (char)((sTmTime.tm_sec %10)+'0');
			pString   += 3;
			iSize     += 3;
			}

		if(iMode&STR_ZONE)							// Zeitzone anhngen
			{
			if(iMode&STR_GLOBAL)
				{
				iAdd = 0;
				}
			else{
				iAdd = _timezone;
				if(sTmTime.tm_isdst)iAdd-=3600;
				}

			if(iAdd<0)
				{
				iAdd = -iAdd;
				pString[0] = '+';
				}
			else{
				pString[0] = '-';
				}

			iAdd			 = iAdd/60;
			sTmTime.tm_hour	 = iAdd/60;
			sTmTime.tm_min	 = iAdd%60;

			pString[1] = (char)((sTmTime.tm_hour/10)+'0');
			pString[2] = (char)((sTmTime.tm_hour%10)+'0');
			pString[3] = cColon;
			pString[4] = (char)((sTmTime.tm_min/10)+'0');
			pString[5] = (char)((sTmTime.tm_min%10)+'0');
			pString   += 6;
			iSize     += 6;
			}
		}

	if(!(iMode&STR_NO_NULL))						// Null-Byte anhngen
		{
		*pString = 0;
		}


return iSize;
}

static const wchar_t *pWeekDaysW[]={L"So " ,L"Mo " ,L"Di " ,L"Mi " ,L"Do " ,L"Fr " ,L"Sa "};
static const wchar_t *pMonthStrW[]={L"Jan.",L"Feb.",L"Mar.",L"Apr.",L"Mai.",L"Jun.",L"Jul.",L"Aug.",L"Sep.",L"Okt.",L"Nov ",L"Dez."};
static const wchar_t *pMonthEngW[]={L"Jan.",L"Feb.",L"Mar.",L"Apr.",L"May.",L"Jun.",L"Jul.",L"Aug.",L"Seb.",L"Oct.",L"Nov.",L"Dec."};

int _cdecl strtimeset(wchar_t *pString,time_t iTime,int iMode)
{
wchar_t					cSpace,cBreak,cColon;
int						iSize,iAdd=0;
struct tm				sTmTime;
struct tm		       *pTimePtr;
const wchar_t   *pPtr;




	if(sizeof(iTime)==4 && (iTime&0x80000000))		// berlauf in 2039
		{
		iTime -= 3600*24*(365*65+16);				// minus 65 Jahre
		iAdd   = 65;
		}



	iSize    = 0;


	if((iMode&(STR_GLOBAL|STR_NO_DST))==STR_NO_DST)	// Lokalezeit ohne Sommerzeit
		{
		if(iTimeZone==-1)
			{
			#ifdef _MSC_VER
			time(0);
			iTimeZone = _timezone;

			#else

			struct timezone	sZone;
			struct timeval	sTime;
			gettimeofday(&sTime,&sZone);
			iTimeZone = sZone.tz_minuteswest*60;

			#endif
			}

		iTime += iTimeZone;
		iMode |= STR_GLOBAL;

		if(iTime<0)iTime=0;
		}

    if(iMode&STR_GLOBAL)
            pTimePtr = gmtime   (&iTime);
    else    pTimePtr = localtime(&iTime);

	if(pTimePtr)									// Zeit umwandeln
		{
		sTmTime=*pTimePtr;
		}
	else{
		memset(&sTmTime,0,sizeof(sTmTime));
		}

	sTmTime.tm_year+=iAdd;

	cSpace = (iMode&STR_USEUNTER)? '_':' ';


	if(iMode&STR_WEEKDAY)							// Wochentag ausgben
		{
		pPtr       = pWeekDaysW[sTmTime.tm_wday];
		pString[0] = pPtr[0];
		pString[1] = pPtr[1];
		pString[2] = pPtr[2];
		pString   += 3;
		iSize      = 3;

		if(iMode&STR_WEEKCOMMA)
			{
			pString[-1]  = (iMode&STR_USESLASH)? '/':',';
			pString[ 0]  = cSpace;
			pString     += 1;
			iSize        = 1;
			}
		}

	if(iMode&STR_DATE)								// Datum augeben TT.MM.JJJJ
		{
		cBreak = (iMode&STR_USESLASH)? '/':'.';

		pString[0] = (char)((sTmTime.tm_mday/10)+'0');
		pString[1] = (char)((sTmTime.tm_mday%10)+'0');
		pString[2] = cBreak;
		pString   += 3;
		iSize     += 3;

		if(iMode&STR_ABC)							// Monat als Text
			{
			pPtr       = pMonthStrW[sTmTime.tm_mon];
			pString[0] = pPtr[0];
			pString[1] = pPtr[1];
			pString[2] = pPtr[2];
			pString[3] = pPtr[3];
			pString   +=4;
			iSize     +=4;
			}
		else{										// Monat als Zahl
							     sTmTime.tm_mon++;
			pString[0] = (char)((sTmTime.tm_mon/10)+'0');
			pString[1] = (char)((sTmTime.tm_mon%10)+'0');
			pString[2] = cBreak;
			pString   += 3;
			iSize     += 3;
			}
													// Vierstelliges Datum
		if(iMode&STR_LONG)
			{
			*(int*)pString=(sTmTime.tm_year>=100)? *(int*)"20 ":*(int*)"19 ";
			pString += 2;
			iSize   += 2;
			}

		pString[0] = (char)((sTmTime.tm_year/10)%10+'0');
		pString[1] = (char)((sTmTime.tm_year%10)   +'0');

		if(iMode&STR_NO_SPACE)						// Kein Space am Datumsende
			{
			pString   += 2;
			iSize     += 2;
			}
		else{
			pString[2] = cSpace;
			pString   += 3;
			iSize     += 3;
			}
		}
	else if(iMode&STR_DATE_ISO)						// Datum im englischen Format JJJJ-MM-TT
		{
		cBreak = (iMode&STR_USESLASH)? '/':'-';

		if(iMode&STR_LONG)							// Vierstelliges Datum
			{
			*(int*)pString=(sTmTime.tm_year>=100)? *(int*)"20 ":*(int*)"19 ";
			pString+=2;iSize+=2;
			}
		pString[0] = (char)((sTmTime.tm_year/10)%10+'0');
		pString[1] = (char)((sTmTime.tm_year%10)   +'0');
		pString[2] = cBreak;
		pString   += 3;
		iSize     += 3;

		if(iMode&STR_ABC)							// Monat als Text
			{
			pPtr       = pMonthStrW[sTmTime.tm_mon];
			pString[0] = pPtr[0];
			pString[1] = pPtr[1];
			pString[2] = pPtr[2];
			pString[3] = cBreak;
			pString   +=4;
			iSize     +=4;
			}										// Monat als Zahl
		else{
					             sTmTime.tm_mon++;
			pString[0] = (char)((sTmTime.tm_mon/10)+'0');
			pString[1] = (char)((sTmTime.tm_mon%10)+'0');
			pString[2] = cBreak;
			pString   += 3;
			iSize     += 3;
			}

		pString[0] = (char)((sTmTime.tm_mday/10)+'0');
		pString[1] = (char)((sTmTime.tm_mday%10)+'0');

		if(iMode&STR_NO_SPACE)						// Kein Space am Datumsende
			{
			pString   += 2;
			iSize     += 2;
			}
		else{
			pString[2] = cSpace;
			pString   += 3;
			iSize     += 3;
			}
		}

	if(iMode&STR_TIME)								// Zeitaugeben HH:MM
		{
		if(iMode&STR_ADD_CHAR)
			{
			pString[0] = (unsigned char)(iMode>>24);
			pString   += 1;
			iSize     += 1;
			}

		pString[0] = (char)((sTmTime.tm_hour/10)+'0');
		pString[1] = (char)((sTmTime.tm_hour%10)+'0');

		if(iMode&STR_USEHYPHEN)
				cColon = '-';
		else	cColon = (iMode&STR_UNTERLINE)? '_':':';

		iAdd   = (iMode&STR_NO_COLON )? -1:0;

		pString[2] = cColon; pString+=iAdd;	iSize+=iAdd;
		pString[3] = (char)((sTmTime.tm_min /10)+'0');
		pString[4] = (char)((sTmTime.tm_min %10)+'0');
		pString   += 5;
		iSize     += 5;

		if(iMode&STR_SEC)							// Sekunden anhngen HH:MM:SS
			{
			pString[0] = cColon; pString+=iAdd;	iSize+=iAdd;
			pString[1] = (char)((sTmTime.tm_sec /10)+'0');
			pString[2] = (char)((sTmTime.tm_sec %10)+'0');
			pString   += 3;
			iSize     += 3;
			}

		if(iMode&STR_ZONE)							// Zeitzone anhngen
			{
			if(iMode&STR_GLOBAL)
				{
				iAdd = 0;
				}
			else{
				iAdd = _timezone;
				if(sTmTime.tm_isdst)iAdd-=3600;
				}

			if(iAdd<0)
				{
				iAdd = -iAdd;
				pString[0] = '+';
				}
			else{
				pString[0] = '-';
				}

			iAdd			 = iAdd/60;
			sTmTime.tm_hour	 = iAdd/60;
			sTmTime.tm_min	 = iAdd%60;

			pString[1] = (char)((sTmTime.tm_hour/10)+'0');
			pString[2] = (char)((sTmTime.tm_hour%10)+'0');
			pString[3] = cColon;
			pString[4] = (char)((sTmTime.tm_min/10)+'0');
			pString[5] = (char)((sTmTime.tm_min%10)+'0');
			pString   += 6;
			iSize     += 6;
			}
		}

	if(!(iMode&STR_NO_NULL))						// Null-Byte anhngen
		{
		*pString = 0;
		}


return iSize;
}

//****************************************************************************
//*
//*		strtimenames
//*
//****************************************************************************
//	Stellt die Texte fr strtimeset ein
//	pMonthStr	: 12 mal 4 Byte lange Monatsnamen mit einem '.' am Ende
//	pWeekDays	:  7 mal 3 Byte lange Wochentagsnamen mit einem ' ' am Ende

static char		cWeekBuffer [3* 7];
static char		cMonthBuffer[4*12];

void _cdecl strtimenames(const char *pMonthStr,const char *pWeekDays)
{
int		i;


	if(pMonthStr)
		{
		memcpy(cMonthBuffer,pMonthStr,12*4);
		for(i=0;i<12;i++)pMonthStrA[i]=cMonthBuffer+i*4;
		}

	if(pWeekDays)
		{
		memcpy(cWeekBuffer,pWeekDays,7*3);
		for(i=0;i<7;i++)pWeekDaysA[i]=cWeekBuffer+i*3;
		}

}



static wchar_t	uWeekBuffer [3* 7];
static wchar_t	uMonthBuffer[4*12];

void _cdecl strtimenames(const wchar_t *pMonthStr,const wchar_t *pWeekDays)
{
int		i;


	if(pMonthStr)
		{
		memcpy(uMonthBuffer,pMonthStr,12*4*2);
		for(i=0;i<12;i++)pMonthStrW[i]=uMonthBuffer+i*4;
		}

	if(pWeekDays)
		{
		memcpy(uWeekBuffer,pWeekDays,7*3*2);
		for(i=0;i<7;i++)pWeekDaysW[i]=uWeekBuffer+i*3;
		}

}

//*****************************************************************************
//*
//*		strmonthtoi
//*
//*****************************************************************************
//	Wantelet einen Text in einen Monatsnamen um
//	pText	: Ist der Zeiger auf den Monatsnamen
//	Ergibt den Monatsindex (1...12) oder 0 bei einem Fehler
static int strmonthtoi(const char *pText)
{
int		i,iNum,iBits,iCount;
char 	c,d,e;



	while(*pText==' ')pText++;

	iBits  = 0;
	iCount = 12;

	for(i=0;i<3;i++)
		{
		for(iNum=11;iNum>=0;iNum--)
			{
			if(iBits&(1<<iNum))continue;

			c=pText           [i];
			d=pMonthStrA[iNum][i];
			e=pMonthEngA[iNum][i];

			if(c>='A' && c<='Z')c+=32;
			if(d>='A' && d<='Z')d+=32;
			if(e>='A' && e<='Z')e+=32;

			if(c!=d && c!=e)
				{
				iBits|=1<<iNum;
				   iCount--;
				if(iCount<=1)break;
				}
			}

		if(iCount<=1)break;
		}

	for(iNum=11;iNum>=0;iNum--)
		{
		if(!(iBits&(1<<iNum)))break;
		}


return iNum+1;
}

static int strmonthtoi(const wchar_t *pText)
{
int		i,iNum,iBits,iCount;
wchar_t c,d,e;



	while(*pText==' ')pText++;

	iBits  = 0;
	iCount = 12;

	for(i=0;i<3;i++)
		{
		for(iNum=11;iNum>=0;iNum--)
			{
			if(iBits&(1<<iNum))continue;

			c=pText           [i];
			d=pMonthStrW[iNum][i];
			e=pMonthEngW[iNum][i];

			if(c>='A' && c<='Z')c+=32;
			if(d>='A' && d<='Z')d+=32;
			if(e>='A' && e<='Z')e+=32;

			if(c!=d && c!=e)
				{
				iBits|=1<<iNum;
				   iCount--;
				if(iCount<=1)break;
				}
			}

		if(iCount<=1)break;
		}

	for(iNum=11;iNum>=0;iNum--)
		{
		if(!(iBits&(1<<iNum)))break;
		}


return iNum+1;
}


//*****************************************************************************
//*
//*		SysWeekday
//*
//*****************************************************************************
//	Returns the Weekday of a Date
//	uDay	: day of month (1-31)
//	uMounth	: mounth of the year (1=Jan ... )
//	uYear	: year, values from 2000 to 2100 are allowed
//	Returns 0=So 1=Mo 2=Di 3=Mi 4=Do 5=Fr 6=So
static unsigned _cdecl strweekday(unsigned uDay,unsigned uMonth,unsigned uYear)
{
static const unsigned char	ucYearDays[16] = {0%7,306%7,337%7,0%7,31%7,61%7,92%7,122%7,153%7,184%7,214%7,245%7,275%7,365%7,365%7,365%7};
unsigned					uVal;


	uMonth   = uMonth&0x0F;
	uDay    &= 0x1F;

	if(uMonth<3)uYear--;

	uVal     =  2;										// 1.Mrz.0000 = Mo
	uVal	+= (uYear% 4)*YEAR_DAYS;	uYear/=4;
	uVal	+= (uYear%25)*YEAR_DAYS_4;	uYear/=25;
	uVal	+= (uYear% 4)*YEAR_DAYS_100;uYear/=4;
	uVal	+= (uYear   )*YEAR_DAYS_400;
	uVal    += ucYearDays[uMonth];
	uVal	+= uDay;
	uVal	%= 7;


return	uVal;
}


//*****************************************************************************
//*
//*		strtoutc
//*
//*****************************************************************************
//	Wandelt eine Zeichenfolge in eine tm Struktur um
//	Mgliche Formate	:	1.Aug.2000 	    11:22:59
//							Di 14.Mai.1999	00:12
//							Di 14.3.99		0:59
//							14.3.1999		0:59:1
//							31-12-2000		112259
//							2000-Dez-03		1122
//							2000-12-03		11:22:59PM
//							Di 2000-12-03	11:22:59
//							15. Mai			11:22:59
//							Do 15.Mai		11:22:59
//							Do Oct 07		11:22PM
//							Oct 07		    11:22:59
//							31-12-99		11:22 PM
//							1999 Okt
//							Okt 1999
//							1999
//							2010-05-13T13:14:41Z
//							Oct 29 23:59:59 2013
//							Thu	Oct 29 23:59:59 2013
//	Ist die Angabe nicht vollstndig so werden die aktuellen Daten oder 0 eingefgt.
//	Jahres Zahlen kleiner 30 werden auf 2000 erweitert.
//	Jahres Zahlen grer  30 und kleiner 100 werden auf 1900 erweitert.
//	Eine Wochentagsangabe wird bersprungen.
//	pText		: Ist der Zeiger der in eine UTZ Zeit umgewandelt wird.
//	pData		: Hier werden die Zeitdaten aus dem Text gespeichert
//	iFlags		: Bestimmt die Art der Umwandlung
//					STR_DATE		= Datum einlesen
//					STR_TIME		= Zeit  einlesen
//					STR_GLOBAL		= Die Zeitangabe ist eine globale Zeit
//					STR_NOW			= Nicht angegebene Werte mit aktueller Zeit fllen
//					STR_NOW_TIME	= Nicht angegebene Zeiten mit aktueller Zeit fllen
//					STR_NOW_DATE	= Nicht angegebenes Datum mit aktueller Zeit fllen
//					STR_NOWEEKDAY	= Keinen Wochentag ber springen
//					STR_FORCEZONE	= Zeitzonen bevorzugen (..Z ...+1000 ..+10 ..-10:00)
//					STR_SAVEZONE	= Zeitzone tm_wday speichern (Bit 8..31)
//					STR_ALLOW_MS	= Millisekunden in tm_yday holen
//					STR_ALLOW_US	= Mikosekunden in tm_yday holen
//					STR_ALLOW_NS	= Mikosekunden in tm_yday holen
//	Ergibt ein Bitfeld in dem fr jeden gefundenen Eintrag ein Bit gesetzt wird:
//					Bit 0	    	= Millisekunden
//					Bit 1	    	= Sekunden
//					Bit 2	    	= Minuten
//					Bit 3	    	= Stunden
//					Bit 4	    	= Tag
//					Bit 5	    	= Monat
//					Bit 6	    	= Jahr
//					Bit 7	    	= Zeitzone (in tm_wday HIWORD)
int _cdecl strtoutc(const char *pText,struct tm *pData,int iFlags)
{
struct tm  *pTime;
time_t	    iTime;
char		cChar;
char		aMode[10];
char		aSign[10];
char		aSize[10];
int			aVals[10];
int			iBits;
int			iMask;
int			iMax;
int			iLen;
int			iSeg;
int			iNum;
int			iAdd;
int			iVal;


	for(;;pText++)											// Leerzeichen berspringen
		{
		if(*pText!=' ' )
		if(*pText!='\t')break;
		}


	for(iSeg=0;iSeg<10;iSeg++)								// Suche die Subtexte
		{
		   cChar = *pText;
		if(cChar==0)break;
		if(cChar>='0' && cChar<='9')						// Zahl gefunden
			{
			aSign[iSeg] = 0;
			aMode[iSeg] = 0;
			aSize[iSeg] = 1;
			aVals[iSeg] = atoi(pText);

			for(pText++;*pText>='0' && *pText<='9';pText++)
				{
				   aSize[iSeg]++;
				if(aSize[iSeg]>20)break;
				}
			}
		else{												// Wort gefunden
			aMode[iSeg] = 1;
			aSize[iSeg] = 1;
			aVals[iSeg] = strmonthtoi(pText);

			if(CMP_LETTER(pText[0],'p'))					// PM abfragen
			if(CMP_LETTER(pText[1],'m'))
			if(strwchara (pText[2],0x30000007)==0)
				{
				aMode[iSeg] = 2;
				aSize[iSeg]++;
				pText++;
				}

			if(CMP_LETTER(pText[0],'Z'))					// Zeitzone mit 'Z'
			if(iSeg>=5 && !strwchara(pText[1],0x07))
			if(aMode[iSeg-1]==0 && aVals[iSeg-1]<60)
			if(aMode[iSeg-2]==0 && aVals[iSeg-2]<60)
				{
				aMode[iSeg  ] = 3;
				aVals[iSeg  ] = 0;
				aSize[iSeg  ] = 4;
				aSign[iSeg-1] = '+';
				}

			for(pText++;;pText++,aSize[iSeg]++)				// Buchstaben zhlen
				{
				cChar = *pText;
				if(cChar>'Z')cChar-=32;
				if(cChar< 0 )continue;
				if(cChar<'A')break;
				if(cChar>'Z')break;
				if(aSize[iSeg]>20)break;
				}

			if(iSeg>=5 && aSize[iSeg]>=2 && aSize[iSeg]<=5)	// Zeitzone mit Text
			if(aMode[iSeg-1]==0 && aVals[iSeg-1]<60)
			if(aMode[iSeg-2]==0 && aVals[iSeg-2]<60)
				{
				   iAdd = strtimezome(pText-aSize[iSeg],aSize[iSeg],sizeof(*pText));
				if(iAdd!=-1)
					{
					aMode[iSeg  ] = 3;
					aVals[iSeg  ] = iAdd;
					aSize[iSeg  ] = 4;
					aSign[iSeg-1] = '+';
					}
				}
			}

		aSign[iSeg] = ' ';

		for(;;pText++)										// Leerraum analysieren
			{
			   cChar = *pText;
			if(cChar==0)break;

			if(strwchara(cChar,0xA8))						// ' ' oder '\t' oder '_'
				{
				continue;
				}

			if(cChar=='\xA0')								// Spezielles Leerzeichen
				{
				continue;
				}

			if(strwchara(cChar,0x06603200))					// '.,:;-/'
				{
				aSign[iSeg] = cChar;
				continue;
				}

			if(CMP_LETTER(cChar,'t'))						// Format "2010-05-13T13:14:41Z"
				{
				if(iSeg==2 && aSign[0]=='-' && aSign[1]=='-')
				if(aMode[0]==0 && aVals[0]>=1000 && aVals[0]<=9999)
				if(aMode[1]==0 && aVals[1]<=12   && aVals[1]>=1)
				if(aMode[2]==0 && aVals[2]<=31   && aVals[2]>=1)
					{
					aSign[iSeg] = ' ';
					continue;
					}
															// Format "Mo 2010-05-13T13:14:41Z"
				if(iSeg==3 && aSign[1]=='-' && aSign[2]=='-' && aMode[0])
				if(aMode[1]==0 && aVals[1]>=1000 && aVals[1]<=9999)
				if(aMode[2]==0 && aVals[2]<=12   && aVals[2]>=1)
				if(aMode[3]==0 && aVals[3]<=31   && aVals[3]>=1)
					{
					aSign[iSeg] = ' ';
					continue;
					}
				}

			break;
			}
		}


	while(iSeg>0 && aMode[iSeg-1]==1)						// Unsinn am Ende entfernen
		{
		if(iFlags&STR_DATE)
		if(iSeg==2 || iSeg==3)
		if(aVals[iSeg-1]>=1 && aVals[iSeg-1]<=12)
		if(aMode[iSeg-2]==0 && (iSeg!=3 || aMode[0]==1))
			{
			break;											// Monat am Ende nicht krzen
			}

		iSeg--;
		}


//******************** Datum umrechen *****************************************


	iNum			= 0;
	iBits			= 0;
	pTime			= 0;
	pData->tm_isdst = 0;
	pData->tm_hour	= 0;
	pData->tm_min	= 0;
	pData->tm_sec	= 0;
	pData->tm_yday	= 0;
	pData->tm_wday	= 0;
	pData->tm_mday	= 1;
	pData->tm_mon	= 0;
	pData->tm_year	= 70;


	if(iFlags&STR_DATE)
		{
		if(iSeg>=3 && aMode[0] && !(iFlags&STR_NOWEEKDAY))	// Wochentag berspringen
		if(iSeg!=3 || aVals[0]==0 || aMode[1]!=0 || aMode[2]!=0)
		if(iSeg!=4 || aVals[0]==0 || aMode[1]!=0 || aMode[2]!=0 || aMode[3]!=0 ||(aSign[2]!=':' &&  aSize[3]!= 6 && !(aSize[3]!=4 || aVals[1]>31 || aVals[2]>31 && aVals[3]<1700)) || (aVals[1]>31 && aSize[3]<=2))
		if(iSeg< 5 || aVals[0]==0 || aMode[1]!=0 || aMode[2]!=0 || aMode[3]!=0 || aMode[4]!= 0 || !(aSign[3]==':'&&   aVals[3]<24))
			{
			memmove(aSign,aSign+1,8*sizeof(aSign[0]));
			memmove(aMode,aMode+1,8*sizeof(aMode[0]));
			memmove(aSize,aSize+1,8*sizeof(aSize[0]));
			memmove(aVals,aVals+1,8*sizeof(aVals[0]));
			iSeg--;
			}
															// Format Oct 29 23:59:59 2013
		if(iSeg>=6 && aVals[5]>=1700 && aVals[4]<=59 && aVals[3]<=59 && aVals[2]<=23 && aVals[1]<=31 && aVals[0]<=31 && aMode[4]==0 && aMode[3]==0 && aMode[2]==0)
			{
			memmove(aSign+3,aSign+2,6*sizeof(aSign[0]));
			memmove(aMode+3,aMode+2,6*sizeof(aMode[0]));
			memmove(aSize+3,aSize+2,6*sizeof(aSize[0]));
			memmove(aVals+3,aVals+2,6*sizeof(aVals[0]));
			aSign[2] = aSign[6];
			aMode[2] = aMode[6];
			aSize[2] = aSize[6];
			aVals[2] = aVals[6];
			}

		iMask = 1<<6;


		if(iFlags&STR_FORMAT_DMY)							// Vorgegebenes Datumsformat
			{
			iNum = 3;

			switch((iFlags>>12)&3)
				{
			case 3:											// Nur Datum und Jahr

				if(iSeg<=1)break;
				memmove(aSign+1,aSign,9*sizeof(aSign[0]));
				memmove(aMode+1,aMode,9*sizeof(aMode[0]));
				memmove(aSize+1,aSize,9*sizeof(aSize[0]));
				memmove(aVals+1,aVals,9*sizeof(aVals[0]));
				iSeg++;

				break;

			case 2:											// [0] mit [2] tauschen

				aVals[9] = aVals[0];
				aVals[0] = aVals[2];
				aVals[2] = aVals[9];
				break;

			case 1:											// [0] mit [1] tauschen

				aVals[9] = aVals[0];
				aVals[0] = aVals[1];
				aVals[1] = aVals[9];
				break;

				}

			if(iSeg<3)iSeg=0;
															// [1] mit [2] tauschen
			if(iFlags&0x4000)
				{
				aVals[9] = aVals[1];
				aVals[1] = aVals[2];
				aVals[2] = aVals[9];
				}


			if(iNum>iSeg)iNum=iSeg;

			aMode[0] = 0;
			aMode[1] = 0;
			aMode[2] = 0;

			goto Fixed;
			}


		if(iSeg==4 && aMode[3]==0)							// "Okt 07 12:44" als "Okt 07 12:44:00"  vortuschen
			{
			aSign[3] = ':';
			aMode[4] = 0;
			aVals[4] = 0;
			}


		if(iSeg==4 || iSeg==5)								// "Okt 07 12:44:55" in "2008 Okt 07 12:44:55"
		if(aSign[2]==':' && aSign[3]==':')
		if(aSign[0]!=':' && aSign[1]!=':')
		if(aVals[0]<= 31 && aVals[1]<= 31)
		if(aVals[2]<= 23 && aVals[3]<= 59 && aVals[4]<= 59)
			{
			memmove(aSign+1,aSign,9*sizeof(aSign[0]));
			memmove(aMode+1,aMode,9*sizeof(aMode[0]));
			memmove(aSize+1,aSize,9*sizeof(aSize[0]));
			memmove(aVals+1,aVals,9*sizeof(aVals[0]));

			aSign[0] = ' ';
			aMode[0] = 0;
			aVals[0] = 1970;
			iMask    = 0;

			if(iFlags&STR_NOW_DATE)
				{
				iTime			= time(0);
				pTime			= (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				aVals[0]		= pTime->tm_year+1900;
				}

			iSeg++;
			}
															// Suche Zeit HH:MM:SS
			 if(iSeg>=5 && aSign[3]==':' && aSign[4]==':' && aMode[3]==0 && aMode[4]==0)iNum=3;
		else if(iSeg>=4 && aSign[2]==':' && aSign[3]==':' && aMode[2]==0 && aMode[3]==0)iNum=2;
		else if(iSeg< 3)iNum=iSeg;
		else			iNum=3;

															// Nur eine Zeitangabe HH:MM:SS
		if((iSeg==3 || iSeg==2) && aSign[0]==':' && (aSign[1]==':' || iSeg==2) && (iFlags&STR_TIME))
			{
			if(iFlags&STR_NOW_DATE)
				{
				iTime			= time(0);
				pTime			= (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				pData->tm_year	= pTime->tm_year;
				pData->tm_mon	= pTime->tm_mon;
				pData->tm_mday	= pTime->tm_mday;
				}
			else{
				pData->tm_year	= 70;
				pData->tm_mon	= 0;
				pData->tm_mday	= 1;
				}

			iNum = 0;

			goto Fixed;
			}

		if(iNum<3)											// Aktuelles Datum benutzen
			{
			if(iFlags&STR_NOW_DATE)
				{
				iTime			= time(0);
				pTime			= (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				pData->tm_year	= pTime->tm_year;
				pData->tm_mon	= pTime->tm_mon;
				pData->tm_mday	= pTime->tm_mday;
				}
			}

		if(aMode[1]==0 && iNum>=2)							// MM-DD[-YYYY] auf DD-MM[-YYYY]
		if(aMode[0]!=0 || (aVals[0]>=1 && aVals[0]<=12 && aVals[1]>12 && aVals[1]<=31))
			{
			aVals[9] = aVals[0];
			aVals[0] = aVals[1];
			aVals[1] = aVals[9];
			aMode[1] = aMode[0];
			aMode[0] = 0;
			}

		if(aMode[1]==0 && iNum>=3)							// YYYY-DD-MM zu YYYY-MM-DD
		if(aMode[2]!=0 || (aVals[2]>=1 && aVals[2]<=12 && aVals[1]>12 && aVals[1]<=31))
			{
			aVals[9] = aVals[1];
			aVals[1] = aVals[2];
			aVals[2] = aVals[9];
			aMode[1] = aMode[2];
			aMode[2] = 0;
			}

		if(iNum    >=3 && aVals[0]>=32 && aMode[0]==0)		// YYYY-MM-DD zu DD-MM-YYYY
		if(aVals[2]>=1 && aVals[2]<=31 && aMode[2]==0)
			{
			aVals[9] = aVals[0];
			aVals[0] = aVals[2];
			aVals[2] = aVals[9];
			}

Fixed:

		if(iNum>=1 && aMode[0]==0)							// Tagesangabe auswerten
			{
			if(aVals[0]>=1 && aVals[0]<=31)
				{
				pData->tm_mday = aVals[0];
				iBits         |= 1<<4;
				}
			}

		if(iNum>=2)											// Monatsangabe auswerten
			{
			if(aVals[1]>=1 && aVals[1]<=12)
				{
				pData->tm_mon  = aVals[1]-1;
				iBits		  |= 1<<5;
				}

			if(iNum==2 && iMask)							// Format YYYY-MM
			if(aVals[0]>=1700 && iBits==1<<5)
				{
				pData->tm_year = aVals[0]-1900;
				iBits		  |= 1<<6;
				}
			}

		if(iNum>=3 && aMode[2]==0)							// Jahresangabe auswerten
			{
			     if(aVals[2]< 30)aVals[2]+=2000;
			else if(aVals[2]<100)aVals[2]+=1900;

			pData->tm_year = aVals[2]-1900;
			iBits         |= iMask;
			}

		if(iBits==0 && !(iFlags&STR_FORMAT_DMY))			// Spezial Formate
			{
			if(iNum==2 && iMask && aVals[0]>=1700)			// Format YYYY-MM
			if(aVals[1]>=1 && aVals[1]<=12)
				{
				pData->tm_year = aVals[0]-1900;
				pData->tm_mon  = aVals[1]-1;
				iBits		  |= 3<<5;
				}

			if(iNum==2 && iMask && aVals[1]>=1700)			// Format MM-YYYY
			if(aVals[0]>=1 && aVals[0]<=12)
				{
				pData->tm_year = aVals[1]-1900;
				pData->tm_mon  = aVals[0]-1;
				iBits		  |= 3<<5;
				}

			if(iNum==1 && iMask && aVals[0]>=1700)			// Format YYYY
				{
				pData->tm_year = aVals[0]-1900;
				iBits		  |= 1<<6;
				}
			}

		pData->tm_wday = strweekday(pData->tm_year+1900,pData->tm_mon+1,pData->tm_mday);
		}


//******************** Zeit auslesen ******************************************


	iSeg -= iNum;


	if(iFlags&STR_TIME)
		{
		if((iFlags&STR_FORCEZONE) && iSeg>2)				// Zeitzonen bevorzugen
			{
			if(aMode[iNum+2]==3 && iSeg+iNum<10)			// "..11:59Z" zu "..11:59:00+0000"
				{
				memmove(aSign+iNum+2,aSign+iNum+1,(iSeg-1)*sizeof(aSign[0]));
				memmove(aMode+iNum+3,aMode+iNum+2,(iSeg-2)*sizeof(aMode[0]));
				memmove(aSize+iNum+3,aSize+iNum+2,(iSeg-2)*sizeof(aSize[0]));
				memmove(aVals+iNum+3,aVals+iNum+2,(iSeg-2)*sizeof(aVals[0]));

				aSign[iNum+1] = ':';
				aMode[iNum+2] = 0;
				aVals[iNum+2] = 0;

				iSeg++;
				}

			if(aSign[iNum+1]=='+' || aSign[iNum+1]=='-')	// "..11:59+1000" zu "..11:59:00+1000"
			if(aMode[iNum+2]== 0  && aSign[iNum+2]!='-' && iSeg+iNum<10)
				{
				memmove(aSign+iNum+2,aSign+iNum+1,(iSeg-1)*sizeof(aSign[0]));
				memmove(aMode+iNum+3,aMode+iNum+2,(iSeg-2)*sizeof(aMode[0]));
				memmove(aSize+iNum+3,aSize+iNum+2,(iSeg-2)*sizeof(aSize[0]));
				memmove(aVals+iNum+3,aVals+iNum+2,(iSeg-2)*sizeof(aVals[0]));

				aSign[iNum+1] = ':';
				aMode[iNum+2] = 0;
				aVals[iNum+2] = 0;

				iSeg++;
				}
															// "..11:59:59+1000" zu "..11:59:59.000+1000"
			if(iFlags&(STR_ALLOW_MS|STR_ALLOW_NS|STR_ALLOW_US))
			if(iSeg>=4            && aMode[iNum+2]== 0 )
			if(aMode[iNum+3]== 0  || aMode[iNum+3]== 3 )
			if(aSign[iNum+2]=='+' || aSign[iNum+2]=='-')
				{
				memmove(aSign+iNum+3,aSign+iNum+2,(iSeg-2)*sizeof(aSign[0]));
				memmove(aMode+iNum+4,aMode+iNum+3,(iSeg-3)*sizeof(aMode[0]));
				memmove(aSize+iNum+4,aSize+iNum+3,(iSeg-3)*sizeof(aSize[0]));
				memmove(aVals+iNum+4,aVals+iNum+3,(iSeg-3)*sizeof(aVals[0]));

				aSign[iNum+2] = '.';
				aMode[iNum+3] = 0;
				aVals[iNum+3] = 0;
				aSize[iNum+3] = 3;

				iSeg++;
				}
			}



		if(iSeg==1 && aMode[iNum]==0)						// Speziele Zeitformate
			{
			if(aSize[iNum]==4)								// HHMM
				{
				aMode[iNum+1] = 0;
				aVals[iNum+1] = aVals[iNum]%100;
				aVals[iNum  ] = aVals[iNum]/100;
				iSeg++;
				}
			else if(aSize[iNum]==6)							// HHMMSS
				{
				aMode[iNum+2] = 0;
				aMode[iNum+1] = 0;
				aVals[iNum+2] = aVals[iNum]%100;
				aVals[iNum+1] =(aVals[iNum]/100)%100;
				aVals[iNum  ] = aVals[iNum]/10000;
				iSeg+=2;
				}
			}

															
		if(iSeg<3 && iFlags&STR_NOW_TIME)					// Aktuelle Zeit verwenden
			{						  				
			if(!pTime)
				{
				iTime = time(0);
				pTime = (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				}

			pData->tm_hour	= pTime->tm_hour;
			pData->tm_min	= pTime->tm_min;
			pData->tm_sec	= pTime->tm_sec;
			}

		iAdd = 0;

		if(iSeg>=1 && aMode[iNum+0]==0)						// Stundenangabe auswerten
			{
			pData->tm_hour = aVals[iNum+0]%24;
			iBits         |= 1<<3;
			iAdd          += 1;
			}

		if(iSeg>=2 && aMode[iNum+1]==0)						// Minutenangabe auswerten
			{
			pData->tm_min  = aVals[iNum+1]%60;
			iBits         |= 1<<2;
			iAdd          += 1;
			}

		if(iSeg>=3 && aMode[iNum+2]==0)						// Sekundenangabe auswerten
			{
			pData->tm_sec  = aVals[iNum+2]%60;
			iBits         |= 1<<1;
			iAdd          += 1;
			}

		if((iBits&0x0E) && aMode[iNum+iAdd]==2)				// Ist PM vorhanden
		if(aVals[iNum+0]<12)
			{
			pData->tm_hour += 12;
			}

		iAdd = 3;

		if(iSeg>=4 && strwchara(aSign[iNum+2],0x00001200))	// Millisekunden anhngen
		if(iFlags&(STR_ALLOW_MS|STR_ALLOW_NS|STR_ALLOW_US))
			{
			     if(iFlags&STR_ALLOW_NS){iLen=9;iMax=1000000000;}
			else if(iFlags&STR_ALLOW_US){iLen=6;iMax=1000000;}
			else                        {iLen=3;iMax=1000;}

			while(aSize[iNum+3]>iLen)
				{
				aVals[iNum+3] /= 10;
				aSize[iNum+3] -= 1;
				}

			while(aSize[iNum+3]<iLen)
				{
				aVals[iNum+3] *= 10;
				aSize[iNum+3] += 1;
				}

			pData->tm_yday = aVals[iNum+3]%iMax;
			iBits         |= 1<<0;
			iAdd		   = 4;
			}

		if(iSeg>iAdd && iAdd==3 && aSign[iNum+2]=='.')		// Millisekunden fr Zeitzone berspringen
		if(aSign[iNum+3]=='+' || aSign[iNum+3]=='-')
			{
			iAdd++;
			}


		if(iSeg>iAdd)										// Zeitzone prfen
		if(aMode[iNum+iAdd  ]== 0  || aMode[iNum+iAdd  ]== 3 )
		if(aSize[iNum+iAdd  ]<= 4  && aSize[iNum+iAdd  ]>= 1 )
		if(aSign[iNum+iAdd-1]=='+' || aSign[iNum+iAdd-1]=='-')
			{
			iVal = aVals[iNum+iAdd];

			if(aSize[iNum+iAdd]<=2)							// Zeitzone nur zweistellig "...+10"
				{
				iVal *= 100;

				if(iAdd+1<iSeg && aVals[iNum+iAdd+1]<100)	// Zeitzone im Format "...+10:00"
				if(aSize[iNum+iAdd]==2 && aSign[iNum+iAdd]==':')
					{
					iVal += aVals[iNum+iAdd+1];
					}
				}

			iAdd = iVal%100;

			if(iVal>-2400 && iVal<2400 && abs(iAdd)<60)
				{
				iBits  |= 1<<7;

				if(iFlags&STR_SAVEZONE)						// Zeitzone in Wochentag speichern
					{
					iVal  = (iVal/100)*60+iAdd;
					pData->tm_wday |= (iVal*60)<<8;
					}
				}
			}
		}



return iBits;
}


int _cdecl strtoutc(const wchar_t *pText,struct tm *pData,int iFlags)
{
struct tm  *pTime;
time_t	    iTime;
wchar_t		cChar;
wchar_t		aMode[10];
wchar_t		aSign[10];
char 		aSize[10];
int			aVals[10];
int			iBits;
int			iMask;
int			iMax;
int			iLen;
int			iSeg;
int			iNum;
int			iAdd;
int			iVal;



	for(;;pText++)											// Leerzeichen berspringen
		{
		if(*pText!=' ' )
		if(*pText!='\t')break;
		}


	for(iSeg=0;iSeg<10;iSeg++)								// Suche die Subtexte
		{
		   cChar = *pText;
		if(cChar==0)break;
		if(cChar>='0' && cChar<='9')						// Zahl gefunden
			{
			aSign[iSeg] = 0;
			aMode[iSeg] = 0;
			aSize[iSeg] = 1;
			aVals[iSeg] = _wtoi(pText);

			for(pText++;*pText>='0' && *pText<='9';pText++)
				{
				   aSize[iSeg]++;
				if(aSize[iSeg]>20)break;
				}
			}
		else{												// Wort gefunden
			aMode[iSeg] = 1;
			aSize[iSeg] = 1;
			aVals[iSeg] = strmonthtoi(pText);

			if(CMP_LETTER(pText[0],'p'))					// PM abfragen
			if(CMP_LETTER(pText[1],'m'))
			if(strwcharw (pText[2],0x30000007)==0)
				{
				aMode[iSeg] = 2;
				aSize[iSeg]++;
				pText++;
				}

			if(CMP_LETTER(pText[0],'Z'))					// Zeitzone mit 'Z'
			if(iSeg>=5 && !strwcharw(pText[1],0x07))
			if(aMode[iSeg-1]==0 && aVals[iSeg-1]<60)
			if(aMode[iSeg-2]==0 && aVals[iSeg-2]<60)
				{
				aMode[iSeg  ] = 3;
				aVals[iSeg  ] = 0;
				aSize[iSeg  ] = 4;
				aSign[iSeg-1] = '+';
				}

			for(pText++;;pText++,aSize[iSeg]++)				// Buchstaben zhlen
				{
				cChar = *pText;
				if(cChar>'Z')cChar-=32;
				if(cChar< 0 )continue;
				if(cChar<'A')break;
				if(cChar>'Z')break;
				if(aSize[iSeg]>20)break;
				}

			if(iSeg>=5 && aSize[iSeg]>=2 && aSize[iSeg]<=5)	// Zeitzone mit Text
			if(aMode[iSeg-1]==0 && aVals[iSeg-1]<60)
			if(aMode[iSeg-2]==0 && aVals[iSeg-2]<60)
				{
				   iAdd = strtimezome((char*)(pText-aSize[iSeg]),aSize[iSeg],sizeof(*pText));
				if(iAdd!=-1)
					{
					aMode[iSeg  ] = 3;
					aVals[iSeg  ] = iAdd;
					aSize[iSeg  ] = 4;
					aSign[iSeg-1] = '+';
					}
				}
			}

		aSign[iSeg] = ' ';

		for(;;pText++)										// Leerraum analysieren
			{
			   cChar = *pText;
			if(cChar==0)break;

			if(strwcharw(cChar,0xA8))						// ' ' oder '\t' oder '_'
				{
				continue;
				}

			if(cChar=='\xA0')								// Spezielles Leerzeichen
				{
				continue;
				}

			if(strwcharw(cChar,0x06603200))					// '.,:;-/'
				{
				aSign[iSeg] = cChar;
				continue;
				}

			if(CMP_LETTER(cChar,'t'))						// Format "2010-05-13T13:14:41Z"
				{
				if(iSeg==2 && aSign[0]=='-' && aSign[1]=='-')
				if(aMode[0]==0 && aVals[0]>=1000 && aVals[0]<=9999)
				if(aMode[1]==0 && aVals[1]<=12   && aVals[1]>=1)
				if(aMode[2]==0 && aVals[2]<=31   && aVals[2]>=1)
					{
					aSign[iSeg] = ' ';
					continue;
					}
															// Format "Mo 2010-05-13T13:14:41Z"
				if(iSeg==3 && aSign[1]=='-' && aSign[2]=='-' && aMode[0])
				if(aMode[1]==0 && aVals[1]>=1000 && aVals[1]<=9999)
				if(aMode[2]==0 && aVals[2]<=12   && aVals[2]>=1)
				if(aMode[3]==0 && aVals[3]<=31   && aVals[3]>=1)
					{
					aSign[iSeg] = ' ';
					continue;
					}
				}

			break;
			}
		}


	while(iSeg>0 && aMode[iSeg-1]==1)						// Unsinn am Ende entfernen
		{
		if(iFlags&STR_DATE)
		if(iSeg==2 || iSeg==3)
		if(aVals[iSeg-1]>=1 && aVals[iSeg-1]<=12)
		if(aMode[iSeg-2]==0 && (iSeg!=3 || aMode[0]==1))
			{
			break;											// Monat am Ende nicht krzen
			}

		iSeg--;
		}


//******************** Datum umrechen *****************************************


	iNum			= 0;
	iBits			= 0;
	pTime			= 0;
	pData->tm_isdst = 0;
	pData->tm_hour	= 0;
	pData->tm_min	= 0;
	pData->tm_sec	= 0;
	pData->tm_yday	= 0;
	pData->tm_wday	= 0;
	pData->tm_mday	= 1;
	pData->tm_mon	= 0;
	pData->tm_year	= 70;


	if(iFlags&STR_DATE)
		{
		if(iSeg>=3 && aMode[0] && !(iFlags&STR_NOWEEKDAY))	// Wochentag berspringen
		if(iSeg!=3 || aVals[0]==0 || aMode[1]!=0 || aMode[2]!=0)
		if(iSeg!=4 || aVals[0]==0 || aMode[1]!=0 || aMode[2]!=0 || aMode[3]!=0 ||(aSign[2]!=':' &&  aSize[3]!= 6 && !(aSize[3]!=4 || aVals[1]>31 || aVals[2]>31 && aVals[3]<1700)) || (aVals[1]>31 && aSize[3]<=2))
		if(iSeg< 5 || aVals[0]==0 || aMode[1]!=0 || aMode[2]!=0 || aMode[3]!=0 || aMode[4]!= 0 || !(aSign[3]==':'&&   aVals[3]<24))
			{
			memmove(aSign,aSign+1,8*sizeof(aSign[0]));
			memmove(aMode,aMode+1,8*sizeof(aMode[0]));
			memmove(aSize,aSize+1,8*sizeof(aSize[0]));
			memmove(aVals,aVals+1,8*sizeof(aVals[0]));
			iSeg--;
			}


		iMask = 1<<6;


		if(iFlags&STR_FORMAT_DMY)							// Vorgegebenes Datumsformat
			{
			iNum = 3;

			switch((iFlags>>12)&3)
				{
			case 3:											// Nur Datum und Jahr

				if(iSeg<=1)break;
				memmove(aSign+1,aSign,9*sizeof(aSign[0]));
				memmove(aMode+1,aMode,9*sizeof(aMode[0]));
				memmove(aSize+1,aSize,9*sizeof(aSize[0]));
				memmove(aVals+1,aVals,9*sizeof(aVals[0]));
				iSeg++;

				break;

			case 2:											// [0] mit [2] tauschen

				aVals[9] = aVals[0];
				aVals[0] = aVals[2];
				aVals[2] = aVals[9];
				break;

			case 1:											// [0] mit [1] tauschen

				aVals[9] = aVals[0];
				aVals[0] = aVals[1];
				aVals[1] = aVals[9];
				break;

				}

			if(iSeg<3)iSeg=0;
															// [1] mit [2] tauschen
			if(iFlags&0x4000)
				{
				aVals[9] = aVals[1];
				aVals[1] = aVals[2];
				aVals[2] = aVals[9];
				}


			if(iNum>iSeg)iNum=iSeg;

			aMode[0] = 0;
			aMode[1] = 0;
			aMode[2] = 0;

			goto Fixed;
			}


		if(iSeg==4 && aMode[3]==0)							// "Okt 07 12:44" als "Okt 07 12:44:00"  vortuschen
			{
			aSign[3] = ':';
			aMode[4] = 0;
			aVals[4] = 0;
			}


		if(iSeg==4 || iSeg==5)								// "Okt 07 12:44:55" in "2008 Okt 07 12:44:55"
		if(aSign[2]==':' && aSign[3]==':')
		if(aSign[0]!=':' && aSign[1]!=':')
		if(aVals[0]<= 31 && aVals[1]<= 31)
		if(aVals[2]<= 23 && aVals[3]<= 59 && aVals[4]<= 59)
			{
			memmove(aSign+1,aSign,9*sizeof(aSign[0]));
			memmove(aMode+1,aMode,9*sizeof(aMode[0]));
			memmove(aSize+1,aSize,9*sizeof(aSize[0]));
			memmove(aVals+1,aVals,9*sizeof(aVals[0]));

			aSign[0] = ' ';
			aMode[0] = 0;
			aVals[0] = 1970;
			iMask    = 0;

			if(iFlags&STR_NOW_DATE)
				{
				iTime			= time(0);
				pTime			= (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				aVals[0]		= pTime->tm_year+1900;
				}

			iSeg++;
			}
															// Suche Zeit HH:MM:SS
			 if(iSeg>=5 && aSign[3]==':' && aSign[4]==':' && aMode[3]==0 && aMode[4]==0)iNum=3;
		else if(iSeg>=4 && aSign[2]==':' && aSign[3]==':' && aMode[2]==0 && aMode[3]==0)iNum=2;
		else if(iSeg< 3)iNum=iSeg;
		else			iNum=3;

															// Nur eine Zeitangabe HH:MM:SS
		if((iSeg==3 || iSeg==2) && aSign[0]==':' && (aSign[1]==':' || iSeg==2) && (iFlags&STR_TIME))
			{
			if(iFlags&STR_NOW_DATE)
				{
				iTime			= time(0);
				pTime			= (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				pData->tm_year	= pTime->tm_year;
				pData->tm_mon	= pTime->tm_mon;
				pData->tm_mday	= pTime->tm_mday;
				}
			else{
				pData->tm_year	= 70;
				pData->tm_mon	= 0;
				pData->tm_mday	= 1;
				}

			iNum = 0;

			goto Fixed;
			}


		if(iNum<3)											// Aktuelles Datum benutzen
			{
			if(iFlags&STR_NOW_DATE)
				{
				iTime			= time(0);
				pTime			= (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				pData->tm_year	= pTime->tm_year;
				pData->tm_mon	= pTime->tm_mon;
				pData->tm_mday	= pTime->tm_mday;
				}
			}

		if(aMode[1]==0 && iNum>=2)							// MM-DD[-YYYY] auf DD-MM[-YYYY]
		if(aMode[0]!=0 || (aVals[0]>=1 && aVals[0]<=12 && aVals[1]>12 && aVals[1]<=31))
			{
			aVals[9] = aVals[0];
			aVals[0] = aVals[1];
			aVals[1] = aVals[9];
			aMode[1] = aMode[0];
			aMode[0] = 0;
			}

		if(aMode[1]==0 && iNum>=3)							// YYYY-DD-MM zu YYYY-MM-DD
		if(aMode[2]!=0 || (aVals[2]>=1 && aVals[2]<=12 && aVals[1]>12 && aVals[1]<=31))
			{
			aVals[9] = aVals[1];
			aVals[1] = aVals[2];
			aVals[2] = aVals[9];
			aMode[1] = aMode[2];
			aMode[2] = 0;
			}

		if(iNum    >=3 && aVals[0]>=32 && aMode[0]==0)		// YYYY-MM-DD zu DD-MM-YYYY
		if(aVals[2]>=1 && aVals[2]<=31 && aMode[2]==0)
			{
			aVals[9] = aVals[0];
			aVals[0] = aVals[2];
			aVals[2] = aVals[9];
			}

Fixed:

		if(iNum>=1 && aMode[0]==0)							// Tagesangabe auswerten
			{
			if(aVals[0]>=1 && aVals[0]<=31)
				{
				pData->tm_mday = aVals[0];
				iBits         |= 1<<4;
				}
			}

		if(iNum>=2)											// Monatsangabe auswerten
			{
			if(aVals[1]>=1 && aVals[1]<=12)
				{
				pData->tm_mon  = aVals[1]-1;
				iBits		  |= 1<<5;
				}

			if(iNum==2 && iMask)							// Format YYYY-MM
			if(aVals[0]>=1700 && iBits==1<<5)
				{
				pData->tm_year = aVals[0]-1900;
				iBits		  |= 1<<5;
				}
			}

		if(iNum>=3 && aMode[2]==0)							// Jahresangabe auswerten
			{
			     if(aVals[2]< 30)aVals[2]+=2000;
			else if(aVals[2]<100)aVals[2]+=1900;

			pData->tm_year = aVals[2]-1900;
			iBits         |= iMask;
			}

		if(iBits==0 && !(iFlags&STR_FORMAT_DMY))			// Spezial Formate
			{
			if(iNum==2 && iMask && aVals[0]>=1700)			// Format YYYY-MM
			if(aVals[1]>=1 && aVals[1]<=12)
				{
				pData->tm_year = aVals[0]-1900;
				pData->tm_mon  = aVals[1]-1;
				iBits		  |= 3<<5;
				}

			if(iNum==2 && iMask && aVals[1]>=1700)			// Format MM-YYYY
			if(aVals[0]>=1 && aVals[0]<=12)
				{
				pData->tm_year = aVals[1]-1900;
				pData->tm_mon  = aVals[0]-1;
				iBits		  |= 3<<5;
				}

			if(iNum==1 && iMask && aVals[0]>=1700)			// Format YYYY
				{
				pData->tm_year = aVals[0]-1900;
				iBits		  |= 1<<6;
				}
			}

		pData->tm_wday = strweekday(pData->tm_year+1900,pData->tm_mon+1,pData->tm_mday);
		}


//******************** Zeit auslesen ******************************************


	iSeg -= iNum;


	if(iFlags&STR_TIME)
		{
		if((iFlags&STR_FORCEZONE) && iSeg>2)				// Zeitzonen bevorzugen
			{
			if(aMode[iNum+2]==3 && iSeg+iNum<10)			// "..11:59Z" zu "..11:59:00+0000"
				{
				memmove(aSign+iNum+2,aSign+iNum+1,(iSeg-1)*sizeof(aSign[0]));
				memmove(aMode+iNum+3,aMode+iNum+2,(iSeg-2)*sizeof(aMode[0]));
				memmove(aSize+iNum+3,aSize+iNum+2,(iSeg-2)*sizeof(aSize[0]));
				memmove(aVals+iNum+3,aVals+iNum+2,(iSeg-2)*sizeof(aVals[0]));

				aSign[iNum+1] = ':';
				aMode[iNum+2] = 0;
				aVals[iNum+2] = 0;

				iSeg++;
				}

			if(aSign[iNum+1]=='+' || aSign[iNum+1]=='-')	// "..11:59+1000" zu "..11:59:00+1000"
			if(aMode[iNum+2]== 0  && aSign[iNum+2]!='-' && iSeg+iNum<10)
				{
				memmove(aSign+iNum+2,aSign+iNum+1,(iSeg-1)*sizeof(aSign[0]));
				memmove(aMode+iNum+3,aMode+iNum+2,(iSeg-2)*sizeof(aMode[0]));
				memmove(aSize+iNum+3,aSize+iNum+2,(iSeg-2)*sizeof(aSize[0]));
				memmove(aVals+iNum+3,aVals+iNum+2,(iSeg-2)*sizeof(aVals[0]));

				aSign[iNum+1] = ':';
				aMode[iNum+2] = 0;
				aVals[iNum+2] = 0;

				iSeg++;
				}
															// "..11:59:59+1000" zu "..11:59:59.000+1000"
			if(iFlags&(STR_ALLOW_MS|STR_ALLOW_NS|STR_ALLOW_US))
			if(iSeg>=4            && aMode[iNum+2]== 0 )
			if(aMode[iNum+3]== 0  || aMode[iNum+3]== 3 )
			if(aSign[iNum+2]=='+' || aSign[iNum+2]=='-')
				{
				memmove(aSign+iNum+3,aSign+iNum+2,(iSeg-2)*sizeof(aSign[0]));
				memmove(aMode+iNum+4,aMode+iNum+3,(iSeg-3)*sizeof(aMode[0]));
				memmove(aSize+iNum+4,aSize+iNum+3,(iSeg-3)*sizeof(aSize[0]));
				memmove(aVals+iNum+4,aVals+iNum+3,(iSeg-3)*sizeof(aVals[0]));

				aSign[iNum+2] = '.';
				aMode[iNum+3] = 0;
				aVals[iNum+3] = 0;
				aSize[iNum+3] = 3;

				iSeg++;
				}
			}

		if(iSeg==1 && aMode[iNum]==0)						// Speziele Zeitformate
			{
			if(aSize[iNum]==4)								// HHMM
				{
				aMode[iNum+1] = 0;
				aVals[iNum+1] = aVals[iNum]%100;
				aVals[iNum  ] = aVals[iNum]/100;
				iSeg++;
				}
			else if(aSize[iNum]==6)							// HHMMSS
				{
				aMode[iNum+2] = 0;
				aMode[iNum+1] = 0;
				aVals[iNum+2] = aVals[iNum]%100;
				aVals[iNum+1] =(aVals[iNum]/100)%100;
				aVals[iNum  ] = aVals[iNum]/10000;
				iSeg+=2;
				}
			}

															
		if(iSeg<3 && (iFlags&STR_NOW_TIME))					// Aktuelle Zeit verwenden	
			{							  
			if(!pTime)
				{
				iTime = time(0);
				pTime = (iFlags&STR_GLOBAL)? gmtime(&iTime):localtime(&iTime);
				}

			pData->tm_hour	= pTime->tm_hour;
			pData->tm_min	= pTime->tm_min;
			pData->tm_sec	= pTime->tm_sec;
			}

		iAdd = 0;

		if(iSeg>=1 && aMode[iNum+0]==0)						// Stundenangabe auswerten
			{
			pData->tm_hour = aVals[iNum+0]%24;
			iBits         |= 1<<3;
			iAdd          += 1;
			}

		if(iSeg>=2 && aMode[iNum+1]==0)						// Minutenangabe auswerten
			{
			pData->tm_min  = aVals[iNum+1]%60;
			iBits         |= 1<<2;
			iAdd          += 1;
			}

		if(iSeg>=3 && aMode[iNum+2]==0)						// Sekundenangabe auswerten
			{
			pData->tm_sec  = aVals[iNum+2]%60;
			iBits         |= 1<<1;
			iAdd          += 1;
			}

		if((iBits&0x0E) && aMode[iNum+iAdd]==2)				// Ist PM vorhanden
		if(aVals[iNum+0]<12)
			{
			pData->tm_hour += 12;
			}

		iAdd = 3;

		if(iSeg>=4 && strwcharw(aSign[iNum+2],0x00001200))	// Millisekunden anhngen
		if(iFlags&(STR_ALLOW_MS|STR_ALLOW_NS|STR_ALLOW_US))
			{
			     if(iFlags&STR_ALLOW_NS){iLen=9;iMax=1000000000;}
			else if(iFlags&STR_ALLOW_US){iLen=6;iMax=1000000;}
			else                        {iLen=3;iMax=1000;}

			while(aSize[iNum+3]>iLen)
				{
				aVals[iNum+3] /= 10;
				aSize[iNum+3] -= 1;
				}

			while(aSize[iNum+3]<iLen)
				{
				aVals[iNum+3] *= 10;
				aSize[iNum+3] += 1;
				}

			pData->tm_yday = aVals[iNum+3]%iMax;
			iBits         |= 1<<0;
			iAdd		   = 4;
			}

		if(iSeg>iAdd && iAdd==3 && aSign[iNum+2]=='.')		// Millisekunden fr Zeitzone berspringen
		if(aSign[iNum+3]=='+' || aSign[iNum+3]=='-')
			{
			iAdd++;
			}


		if(iSeg>iAdd)										// Zeitzone prfen
		if(aMode[iNum+iAdd  ]== 0  || aMode[iNum+iAdd  ]== 3 )
		if(aSize[iNum+iAdd  ]<= 4  && aSize[iNum+iAdd  ]>= 1 )
		if(aSign[iNum+iAdd-1]=='+' || aSign[iNum+iAdd-1]=='-')
			{
			iVal = aVals[iNum+iAdd];

			if(aSize[iNum+iAdd]<=2)							// Zeitzone nur zweistellig "...+10"
				{
				iVal *= 100;

				if(iAdd+1<iSeg && aVals[iNum+iAdd+1]<100)	// Zeitzone im Format "...+10:00"
				if(aSize[iNum+iAdd]==2 && aSign[iNum+iAdd]==':')
					{
					iVal += aVals[iNum+iAdd+1];
					}
				}

			iAdd = iVal%100;

			if(iVal>-2400 && iVal<2400 && abs(iAdd)<60)
				{
				iBits  |= 1<<7;

				if(iFlags&STR_SAVEZONE)						// Zeitzone in Wochentag speichern
					{
					iVal  = (iVal/100)*60+iAdd;
					pData->tm_wday |= (iVal*60)<<8;
					}
				}
			}
		}



return iBits;
}

int _cdecl strtoutc(const char *pText,struct tm *pData,int iFlags,int iMax)
{
char	cTemp[64];

	strlcpy(cTemp,pText,sizeof(cTemp),iMax);

return strtoutc(cTemp,pData,iFlags);
}

int _cdecl strtoutc(const wchar_t *pText,struct tm *pData,int iFlags,int iMax)
{
wchar_t	cTemp[64];

	strlcpy(cTemp,pText,sizeof(cTemp),iMax);

return strtoutc(cTemp,pData,iFlags);
}

//*****************************************************************************
//*
//*		strtoutc
//*
//*****************************************************************************
//	Wandelt eine Zeichenfolge in eine UTC Zeit um
//	Mgliche Formate	:	1.Aug.2000 	    11:22:59
//							Di 14.Mai.1999	00:12
//							Di 14.3.99		0:59
//							14.3.1999		0:59:1
//							31-12-2000		112259
//							2000-Dez-03		1122
//							2000-12-03		11:22:59PM
//							Di 2000-12-03	11:22:59
//							15. Mai			11:22:59
//							Do 15.Mai		11:22:59
//							Do Oct 07		11:22PM
//							Oct 07		    11:22:59
//							31-12-99		11:22 PM
//							1999 Okt
//							Okt 1999
//							1999
//	Ist die Angabe nicht vollstndig so werden die aktuellen Daten oder 0 eingefgt.
//	Jahres Zahlen kleiner 30 werden auf 2000 erweitert.
//	Jahres Zahlen grer  30 und kleiner 100 werden auf 1900 erweitert.
//	Eine Wochentagsangabe wird bersprungen.
//	pText		: Ist der Zeiger der in eine UTZ Zeit umgewandelt wird.
//	iFlags		: Bestimmt die Art der Umwandlung
//					STR_DATE		= Datum einlesen
//					STR_TIME		= Zeit  einlesen
//					STR_GLOBAL		= Die Zeitangabe ist eine globale Zeit
//					STR_NO_DST		= Keine Sommerzeit bercksichtigen
//					STR_NOW			= Nicht angegebene Werte mit aktueller Zeit fllen.
//					STR_NOW_TIME	= Nicht angegebene Zeiten mit aktueller Zeit fllen
//					STR_NOW_DATE	= Nicht angegebenes Datum mit aktueller Zeit fllen
//					STR_NOWEEKDAY	= Keinen Wochentag ber springen
//					STR_FORCEZONE	= Zeitzonen bevorzugen (..Z ...+1000 ..+10 ..-10:00)
//					STR_SAVEZONE	= Zeitzone tm_wday speichern
//					STR_ALLOW_MS	= Millisekunden in tm_yday holen
//					STR_ALLOW_US	= Mikosekunden in tm_yday holen
//					STR_ALLOW_NS	= Mikosekunden in tm_yday holen
//	iMax	: Ist die maximale Lnge des UTC-Textes.
//	pFound	: Wenn dieser Zeiger ungleich 0 ist, wird hier ein Bitfeld
//				  gespeichert, in dem fr jeden gefundenen Eintrag ein Bit
//				  gesetzt wird:
//					Bit 0			= Sekunden
//					Bit 1			= Minuten
//					Bit 2			= Stunden
//					Bit 3			= Tag
//					Bit 4			= Monat
//					Bit 5			= Jahr
//					Bit 6	    	= Jahr
//					Bit 7	    	= Zeitzone
//	Ergibt die UTC Zeit bzw. 0 bei einem Fehler.
time_t _cdecl strtoutc(const char *pText,int iFlags,int *pFound)
{
struct tm	sData;
time_t		iTime;
int			iBits;
int			iZone;
int			iAdd;


	iBits = strtoutc(pText,&sData,iFlags|STR_SAVEZONE);

	if(sData.tm_year==70 && sData.tm_mon==0 && sData.tm_mday==1)
		{											// BUG in mktime
		sData.tm_mday = 2;
		iAdd = -3600*24;
		}
	else{
		iAdd = 0;
		}

	if(iFlags&STR_GLOBAL)
		{
		sData.tm_isdst = 1;							// 1=immer
		iTime  = mktime(&sData);
		if(iTime<0)return 0;
		iTime -= _timezone;
		iTime -= _dstbias;
		}
	else if(iBits&0x80)								// Zeitzone gefunden
		{
		sData.tm_isdst = 1;							// 1=immer
		iZone  = sData.tm_wday>>8;
		iTime  = mktime(&sData);
		if(iTime<0)return 0;
		iTime -= iZone;
		iTime -= _timezone;
		iTime -= _dstbias;
		}
	else{
		sData.tm_isdst = (iFlags&STR_NO_DST)? 0:-1;	// -1=automatisch
		iTime  = mktime(&sData);
		if(iTime<0)return 0;
		}

	if(pFound)*pFound=iBits;


return iTime+iAdd;
}

time_t _cdecl strtoutc(const wchar_t *pText,int iFlags,int *pFound)
{
struct tm	sData;
time_t		iTime;
int			iBits;
int			iZone;
int			iAdd;


	iBits = strtoutc(pText,&sData,iFlags|STR_SAVEZONE);

	if(sData.tm_year==70 && sData.tm_mon==0 && sData.tm_mday==1)
		{											// BUG in mktime
		sData.tm_mday = 2;
		iAdd = -3600*24;
		}
	else{
		iAdd = 0;
		}

	if(iFlags&STR_GLOBAL)
		{
		sData.tm_isdst = 1;							// 1=immer
		iZone  = sData.tm_wday>>8;
		iTime  = mktime(&sData);
		if(iTime<0)return 0;
		iTime -= iZone;
		iTime += _timezone;
		iTime += _dstbias;
		}
	else if(iBits&0x80)								// Zeitzone gefunden
		{
		sData.tm_isdst = 1;							// 1=immer
		iTime  = mktime(&sData);
		if(iTime<0)return 0;
		iTime -= sData.tm_wday>>8;
		iTime -= _timezone;
		iTime -= _dstbias;
		}
	else{
		sData.tm_isdst = (iFlags&STR_NO_DST)? 0:-1;	// -1=automatisch
		iTime  = mktime(&sData);
		if(iTime<0)return 0;
		}

	if(pFound)*pFound=iBits;


return iTime+iAdd;
}

time_t _cdecl strtoutc(const char *pText,int iFlags,int iMax,int *pFound)
{
char	cTemp[64];

	strlcpy(cTemp,pText,sizeof(cTemp),iMax);

return strtoutc(cTemp,iFlags,pFound);
}

time_t _cdecl strtoutc(const wchar_t *pText,int iFlags,int iMax,int *pFound)
{
wchar_t	cTemp[64];

	strlcpy(cTemp,pText,sizeof(cTemp),iMax);

return strtoutc(cTemp,iFlags,pFound);
}

//****************************************************************************
//*
//*		strtoi
//*
//****************************************************************************
//	Wandelt einen Text in eine Fixkommazahl um. SPACE'es und TAB's am Anfang
//	werden bersprungen. Minus Zeichen werden bercksichtigt. Folgende
//	Formate knnen eingelesen werden:
//
//				Normal:		123.. zu Dez-Zahl
//						   -123.. zu Dez-Zahl
//				Cpp-Formate:0x... zu Hex-Zahl
//							0d... zu Dez-Zahl
//							0o... zu Okt-Zahl
//							0b... zu Bin-Zahl
//				Asm-Formate:0...h zu Hex-Zahl
//							0...  zu Dez-Zahl
//							0...o zu Okt-Zahl
//							0...b zu Bin-Zahl
//
//	pStr	: Ist der Zeiger auf den Text der umgewandelt werden soll.
//	iMode	: Die Bits 0 bis 7 geben das Format an
//				  0 dann werden alle Formate gescannt auer Asm-Zahlen
//				  1 dann werden alle Asm-Formate gescannt auer 0?... Zahlen
//			    >=2 dann werden Zahlen der Basis iMode&0xFF gescannt.
//				251 dann werden Zahlen automatisch erkannt (C,ASM,IEC 0777)
//				252 dann werden Zahlen automatisch erkannt (C,ASM,IEC)
//				253 dann werden Zahlen im IEC-Format gescannt (123 16#AB 2#111_0000).
//				254 dann werden Zahlen im C-Format gescannt (123 0xAB 0777 0b111).
//				255 dann werden Zahlen im C-Format gescannt (123 0xAB 0777).
//			  Wenn Bit 10 gesetzt ist werden '_' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 11 gesetzt ist werden '\'' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 12 gesetzt ist werden SPACE und TAB am Ende bersprungen.
//			  Wenn Bit 13 gesetzt ist werden SPACE und TAB nach dem Minus nicht bersprungen.
//			  Wenn Bit 14 gesetzt ist werden Minuszeichen nicht bercksichtigt.
//			  Wenn Bit 15 gesetzt ist werden SPACE und TAB nicht bersprungen.
//	iMax	: Ist die maximale Lnge des UTC-Textes.
//	pLen	: Hier wird die Anzahl  der gescannten Zeichen gespeichert.
//			  Wenn pLen Null ist wird die Anzahl nicht gespeichert.
//	Ergibt die eingelesene Zahl
int _cdecl strtoi(const char *pStr,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
int		iVal;
int		i,j;


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwchara(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;strwchara(pStr[iPos],0xA0);iPos++);
		}

	if(pStr[iPos]=='-' && !(iMode&0x4000))			// Minuszeichen am Anfang
		{
		iMinus = 1;
		iPos  += 1;

		if(strwchara(*pStr,0xA0) && !(iMode&0xA000))// Leerzeichen nach Minus berspringen
			{
			for(iPos++;strwchara(pStr[iPos],0xA0);iPos++);
			}
		}

	if(iMode&0xFF)
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;
				if(CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			}
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else if(pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(pStr[iPos]=='0')						// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}

	for(i=0;;) 										// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwchara(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(iMinus)										// Negative Zahl
		{
		iVal = -iVal;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0);
			}

		*pLen = iPos;
		}



return iVal;
}

int _cdecl strtoi(const wchar_t *pStr,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
int		iVal;
int		i,j;


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwcharw(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;strwcharw(pStr[iPos],0xA0);iPos++);
		}

	if(pStr[iPos]=='-' && !(iMode&0x4000))			// Minuszeichen am Anfang
		{
		iMinus = 1;
		iPos  += 1;

		if(strwcharw(*pStr,0xA0) && !(iMode&0xA000))// Leerzeichen nach Minus berspringen
			{
			for(iPos++;strwcharw(pStr[iPos],0xA0);iPos++);
			}
		}

	if(iMode&0xFF)
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;
				if(CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			 }
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else if(pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(pStr[iPos]=='0')						// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}

	for(i=0;;) 										// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwcharw(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(iMinus)										// Negative Zahl
		{
		iVal = -iVal;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0);
			}

		*pLen = iPos;
		}



return iVal;
}


//****************************************************************************
//*
//*		strtoni
//*
//****************************************************************************
//	Wandelt einen Text in eine Fixkommazahl um. SPACE'es und TAB's am Anfang
//	werden bersprungen. Minus Zeichen werden bercksichtigt. Folgende
//	Formate knnen eingelesen werden:
//
//				Normal:		123.. zu Dez-Zahl
//						   -123.. zu Dez-Zahl
//				Cpp-Formate:0x... zu Hex-Zahl
//							0d... zu Dez-Zahl
//							0o... zu Okt-Zahl
//							0b... zu Bin-Zahl
//				Asm-Formate:0...h zu Hex-Zahl
//							0...  zu Dez-Zahl
//							0...o zu Okt-Zahl
//							0...b zu Bin-Zahl
//
//	pStr	: Ist der Zeiger auf den Text der umgewandelt werden soll.
//	iMax	: Ist die maximale Textlnge die ausgewertet wird.
//	iMode	: Die Bits 0 bis 7 geben das Format an
//				  0 dann werden alle Formate gescannt auer Asm-Zahlen
//				  1 dann werden alle Asm-Formate gescannt auer 0?... Zahlen
//			    >=2 dann werden Zahlen der Basis iMode&0xFF gescannt.
//				251 dann werden Zahlen automatisch erkannt (C,ASM,IEC 0777)
//				252 dann werden Zahlen automatisch erkannt (C,ASM,IEC)
//				253 dann werden Zahlen im IEC-Format gescannt (123 16#AB 2#111_0000).
//				254 dann werden Zahlen im C-Format gescannt (123 0xAB 0777 0b111).
//				255 dann werden Zahlen im C-Format gescannt (123 0xAB 0777).
//			  Wenn Bit 10 gesetzt ist werden '_' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 11 gesetzt ist werden '\'' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 12 gesetzt ist werden SPACE und TAB am Ende bersprungen.
//			  Wenn Bit 13 gesetzt ist werden SPACE und TAB nach dem Minus nicht bersprungen.
//			  Wenn Bit 14 gesetzt ist werden Minuszeichen nicht bercksichtigt.
//			  Wenn Bit 15 gesetzt ist werden SPACE und TAB nicht bersprungen.
//	pLen	: Hier wird die Anzahl  der gescannten Zeichen gespeichert.
//			  Wenn pLen Null ist wird die Anzahl nicht gespeichert.
//	Ergibt die eingelesene Zahl
int _cdecl strtoni(const char *pStr,int iMax,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
int		iVal;
int		i,j;


	if(iMax<=0)
		{
		if(pLen)*pLen=0;
		return 0;
		}


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwchara(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;iPos<iMax && strwchara(pStr[iPos],0xA0);iPos++);
		}
													// Minuszeichen am Anfang
	if(iPos<iMax && pStr[iPos]=='-' && !(iMode&0x4000))
		{
		iMinus = 1;
		iPos  += 1;

		if(!(iMode&0xA000))
			{
			for(;iPos<iMax && strwchara(pStr[iPos],0xA0);iPos++);
			}
		}


	if(iPos<iMax && (iMode&0xFF))
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;

				if(iPos<iMax && CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(iPos<iMax && CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			}
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && iPos+3<=iMax && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(iPos+2<=iMax && pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(iPos<iMax-2 && pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07,iMax-iPos-2)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(iPos<iMax-2 && pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else if(iPos<iMax-3 && pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(iPos<iMax-2 && pStr[iPos]=='0')			// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}


	for(i=0;iPos<iMax;) 							// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwchara(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(iMinus)										// Negative Zahl
		{
		iVal = -iVal;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0,iMax-iPos);
			}

		*pLen = iPos;
		}



return iVal;
}

int _cdecl strtoni(const wchar_t *pStr,int iMax,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
int		iVal;
int		i,j;


	if(iMax<=0)
		{
		if(pLen)*pLen=0;
		return 0;
		}


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwcharw(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;iPos<iMax && strwcharw(pStr[iPos],0xA0);iPos++);
		}
													// Minuszeichen am Anfang
	if(iPos<iMax && pStr[iPos]=='-' && !(iMode&0x4000))
		{
		iMinus = 1;
		iPos  += 1;

		if(!(iMode&0xA000))
			{
			for(;iPos<iMax && strwcharw(pStr[iPos],0xA0);iPos++);
			}
		}


	if(iPos<iMax && (iMode&0xFF))
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;

				if(iPos<iMax && CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(iPos<iMax && CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			}
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && iPos+3<=iMax && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(iPos+2<=iMax && pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(iPos<iMax-2 && pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07,iMax-iPos-2)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(iPos<iMax-2 && pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else if(iPos<iMax-3 && pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(iPos<iMax-2 && pStr[iPos]=='0')			// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}


	for(i=0;iPos<iMax;) 							// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwcharw(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(iMinus)										// Negative Zahl
		{
		iVal = -iVal;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0,iMax-iPos);
			}

		*pLen = iPos;
		}



return iVal;
}

//****************************************************************************
//*
//*		strtoq
//*
//****************************************************************************
//	Wandelt einen Text in eine 64-Bit-Fixkommazahl um. SPACE'es und TAB's am Anfang
//	werden bersprungen. Minus Zeichen werden bercksichtigt. Folgende
//	Formate knnen eingelesen werden:
//
//				Normal:		123.. zu Dez-Zahl
//						   -123.. zu Dez-Zahl
//				Cpp-Formate:0x... zu Hex-Zahl
//							0d... zu Dez-Zahl
//							0o... zu Okt-Zahl
//							0b... zu Bin-Zahl
//				Asm-Formate:0...h zu Hex-Zahl
//							0...  zu Dez-Zahl
//							0...o zu Okt-Zahl
//							0...b zu Bin-Zahl
//
//	pStr	: Ist der Zeiger auf den Text der umgewandelt werden soll.
//	iMode	: Die Bits 0 bis 7 geben das Format an
//				  0 dann werden alle Formate gescannt auer Asm-Zahlen
//				  1 dann werden alle Asm-Formate gescannt auer 0?... Zahlen
//			    >=2 dann werden Zahlen der Basis iMode&0xFF gescannt.
//				251 dann werden Zahlen automatisch erkannt (C,ASM,IEC 0777)
//				252 dann werden Zahlen automatisch erkannt (C,ASM,IEC)
//				253 dann werden Zahlen im IEC-Format gescannt (123 16#AB 2#111_0000).
//				254 dann werden Zahlen im C-Format gescannt (123 0xAB 0777 0b111).
//				255 dann werden Zahlen im C-Format gescannt (123 0xAB 0777).
//			  Wenn Bit 10 gesetzt ist werden '_' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 11 gesetzt ist werden '\'' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 12 gesetzt ist werden SPACE und TAB am Ende bersprungen.
//			  Wenn Bit 13 gesetzt ist werden SPACE und TAB nach dem Minus nicht bersprungen.
//			  Wenn Bit 14 gesetzt ist werden Minuszeichen nicht bercksichtigt.
//			  Wenn Bit 15 gesetzt ist werden SPACE und TAB nicht bersprungen.
//	pLen	: Hier wird die Anzahl  der gescannten Zeichen gespeichert.
//			  Wenn pLen Null ist wird die Anzahl nicht gespeichert.
//	Ergibt die eingelesene Zahl.
intq _cdecl strtoq(const char *pStr,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
intq	iVal;
int		i,j;


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwchara(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;strwchara(pStr[iPos],0xA0);iPos++);
		}

	if(pStr[iPos]=='-' && !(iMode&0x4000))			// Minuszeichen am Anfang
		{
		iMinus = 1;
		iPos  += 1;

		if(strwchara(*pStr,0xA0) && !(iMode&0xA000))// Leerzeichen nach Minus berspringen
			{
			for(iPos++;strwchara(pStr[iPos],0xA0);iPos++);
			}
		}

	if(iMode&0xFF)
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;
				if(CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			}
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else if(pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(pStr[iPos]=='0')						// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}


	for(i=0;;) 										// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwchara(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(iMinus)										// Negative Zahl
		{
		iVal = -iVal;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0);
			}

		*pLen = iPos;
		}



return iVal;
}

intq _cdecl strtoq(const wchar_t *pStr,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
intq	iVal;
int		i,j;


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwcharw(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;strwcharw(pStr[iPos],0xA0);iPos++);
		}

	if(pStr[iPos]=='-' && !(iMode&0x4000))			// Minuszeichen am Anfang
		{
		iMinus = 1;
		iPos  += 1;

		if(strwcharw(*pStr,0xA0) && !(iMode&0xA000))// Leerzeichen nach Minus berspringen
			{
			for(iPos++;strwcharw(pStr[iPos],0xA0);iPos++);
			}
		}

	if(iMode&0xFF)
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;
				if(CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			}
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else if(pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(pStr[iPos]=='0')						// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}


	for(i=0;;) 										// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwcharw(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(iMinus)										// Negative Zahl
		{
		iVal = -iVal;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0);
			}

		*pLen = iPos;
		}



return iVal;
}

//****************************************************************************
//*
//*		strtonq
//*
//****************************************************************************
//	Wandelt einen Text in eine 64-Bit-Fixkommazahl um. SPACE'es und TAB's am
//	Anfang werden bersprungen. Es kann die maximale Verarbeitungslnge 
//	bestimmt werden. Minus Zeichen werden bercksichtigt. 
//	Folgende Formate knnen eingelesen werden:
//
//				Normal:		123.. zu Dez-Zahl
//						   -123.. zu Dez-Zahl
//				Cpp-Formate:0x... zu Hex-Zahl
//							0d... zu Dez-Zahl
//							0o... zu Okt-Zahl
//							0b... zu Bin-Zahl
//				Asm-Formate:0...h zu Hex-Zahl
//							0...  zu Dez-Zahl
//							0...o zu Okt-Zahl
//							0...b zu Bin-Zahl
//
//	pStr	: Ist der Zeiger auf den Text der umgewandelt werden soll.
//	iMax	: Ist die maximale Textlnge die ausgewertet wird.
//	iMode	: Die Bits 0 bis 7 geben das Format an
//				  0 dann werden alle Formate gescannt auer Asm-Zahlen
//				  1 dann werden alle Asm-Formate gescannt auer 0?... Zahlen
//			    >=2 dann werden Zahlen der Basis iMode&0xFF gescannt.
//				251 dann werden Zahlen automatisch erkannt (C,ASM,IEC 0777)
//				252 dann werden Zahlen automatisch erkannt (C,ASM,IEC)
//				253 dann werden Zahlen im IEC-Format gescannt (123 16#AB 2#111_0000).
//				254 dann werden Zahlen im C-Format gescannt (123 0xAB 0777 0b111).
//				255 dann werden Zahlen im C-Format gescannt (123 0xAB 0777).
//			  Wenn Bit 10 gesetzt ist werden '_' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 11 gesetzt ist werden '\'' Zeichen in der Zahl bersprungen.
//			  Wenn Bit 12 gesetzt ist werden SPACE und TAB am Ende bersprungen.
//			  Wenn Bit 13 gesetzt ist werden SPACE und TAB nach dem Minus nicht bersprungen.
//			  Wenn Bit 14 gesetzt ist werden Minuszeichen nicht bercksichtigt.
//			  Wenn Bit 15 gesetzt ist werden SPACE und TAB nicht bersprungen.
//	pLen	: Hier wird die Anzahl  der gescannten Zeichen gespeichert.
//			  Wenn pLen Null ist wird die Anzahl nicht gespeichert.
//	Ergibt die eingelesene Zahl
intq _cdecl strtonq(const char *pStr,int iMax,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
int		iVal;
int		i,j;


	if(iMax<=0)
		{
		if(pLen)*pLen=0;
		return 0;
		}


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwchara(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;iPos<iMax && strwchara(pStr[iPos],0xA0);iPos++);
		}
													// Minuszeichen am Anfang
	if(iPos<iMax && pStr[iPos]=='-' && !(iMode&0x4000))
		{
		iMinus = 1;
		iPos  += 1;

		if(!(iMode&0xA000))
			{
			for(;iPos<iMax && strwchara(pStr[iPos],0xA0);iPos++);
			}
		}


	if(iPos<iMax && (iMode&0xFF))
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;

				if(iPos<iMax && CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(iPos<iMax && CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			}
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && iPos+3<=iMax && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(iPos+2<=iMax && pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(iPos<iMax-2 && pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07,iMax-iPos-2)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(iPos<iMax-2 && pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else if(iPos<iMax-3 && pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(iPos<iMax-2 && pStr[iPos]=='0')			// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}


	for(i=0;iPos<iMax;) 							// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwchara(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0);
			}

		*pLen = iPos;
		}


return iVal;
}

intq _cdecl strtonq(const wchar_t *pStr,int iMax,int iMode,int *pLen)
{
int		iMinus;
int		iBase;
int		iJump;
int		iPos;
int		iAdd;
int		iVal;
int		i,j;


	if(iMax<=0)
		{
		if(pLen)*pLen=0;
		return 0;
		}


	iMinus	=  0;
	iAdd	=  0;
	iPos	=  0;
	iVal	=  0;
	iBase	= 10;
	iJump	= aStrJump[(iMode>>10)&3];


	if(strwcharw(*pStr,0xA0) && !(iMode&0x8000))	// Leerzeichen am Anfang berspringen
		{
		for(iPos++;iPos<iMax && strwcharw(pStr[iPos],0xA0);iPos++);
		}
													// Minuszeichen am Anfang
	if(iPos<iMax && pStr[iPos]=='-' && !(iMode&0x4000))
		{
		iMinus = 1;
		iPos  += 1;

		if(!(iMode&0xA000))
			{
			for(;iPos<iMax && strwcharw(pStr[iPos],0xA0);iPos++);
			}
		}


	if(iPos<iMax && (iMode&0xFF))
		{
		iBase = iMode&0xFF;

		if(iBase==1)								// Assembler Formate
			{
			iBase = 10;
Asm:		for(;;)
				{
				i = strwlen(pStr+iPos,0x00040001,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'h')){iBase=16;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00010000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'b')){iBase=2;iAdd=1;break;}
				i = strwlen(pStr+iPos,0x00020000,iMax-iPos-1)+iPos;
				if(CMP_LETTER(pStr[i],'o')){iBase=8;iAdd=1;break;}
				break;
				}
			 }
		else if(iBase>=254)							// C-Formate
			{
			if(pStr[iPos]!='0')
				{
				iBase = 10;
				}
			else{
				iPos++;

				if(iPos<iMax && CMP_LETTER(pStr[iPos],'x'))
						{iBase=16;iPos++;}
				else if(iPos<iMax && CMP_LETTER(pStr[iPos],'b') && iBase==254)
						{iBase= 2;iPos++;}
				else	{iBase= 8;       }
				}
			 }
		else if(iBase==253)							// IEC-Formate
			{
			iBase = 10;

			if(pStr[iPos]=='1' && iPos+3<=iMax && pStr[iPos+2]=='#')
				{
				if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				}
			else if(iPos+2<=iMax && pStr[iPos+1]=='#')
				{
				if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				}
			}
		else if(iBase>=251)							// Automatisch
			{
			iBase = 10;

			if(iPos<iMax-2 && pStr[iPos]=='0')
				{
				i = strchar("xodb",pStr[iPos+1]|0x20);

				if(i&2)								// Prfe 0b 0o 0BFFh oder 0DFFh
					{
					j=strwlen(pStr+iPos+1,0x07,iMax-iPos-2)+iPos+1;
					if(CMP_LETTER(pStr[j],'h'))goto Asm;
					if((i&1) && j==1)goto Asm;
					}

				if(i<4)								// 0x... 0o... 0d.. 0b...
					{
					iBase = "\x10\x08\x0D\x02"[i];
					iPos += 2;
					}
				else{
					if(iMode&1)iBase=8;				// 01234
					iPos += 1;
					goto Asm;
					}
				}
			else if(iPos<iMax-2 && pStr[iPos+1]=='#')
				{
				     if(pStr[iPos  ]=='8'){iBase= 8;iPos+=2;iJump|=0x0008;}
				else if(pStr[iPos  ]=='2'){iBase= 2;iPos+=2;iJump|=0x0008;}
				else goto Asm;
				}
			else if(iPos<iMax-3 && pStr[iPos]=='1' && pStr[iPos+2]=='#')
				{
				     if(pStr[iPos+1]=='0'){iBase=10;iPos+=3;iJump|=0x0008;}
				else if(pStr[iPos+1]=='6'){iBase=16;iPos+=3;iJump|=0x0008;}
				else goto Asm;
				}
			else{
				goto Asm;
				}
			}
		}
	else if(iPos<iMax-2 && pStr[iPos]=='0')			// Spezial Zahlen
		{
		for(;;)
			{
			j=pStr[iPos+1];
			if(CMP_LETTER(j,'x')){iBase=16;iPos+=2;break;}
			if(CMP_LETTER(j,'d')){iBase=10;iPos+=2;break;}
			if(CMP_LETTER(j,'o')){iBase= 8;iPos+=2;break;}
			if(CMP_LETTER(j,'b')){iBase= 2;iPos+=2;break;}
								 {iBase=10;iPos+=1;break;}
			break;
			}
		}


	for(i=0;iPos<iMax;) 							// Zahl einlesen
		{
		   j = strcharval(pStr[iPos]);
		if(j>=iBase)								// Falsches Zeichen
			{
			if(!strwcharw(pStr[iPos],iJump))break;
			iPos++;
			continue;								// '_' und '\'' berspringen
			}

		iVal *= iBase;								// Zahl berechnen
		iVal += j;
		iPos += 1;
		}

	if(pLen)										// Lnge zurckgeben
		{
		iPos += iAdd;

		if(iMode&0x1000)							// Leerzeichen am Ende berspringen
			{
			iPos += strwlen(pStr+iPos,0xE0);
			}

		*pLen = iPos;
		}


return iVal;
}

//****************************************************************************
//*
//*		strtoh
//*
//****************************************************************************
//	Wandelt einen Hex-String in eine Zahl um.
//	SPACE und TAB am Anfang werden bersprungen.
//	pStr	: Ist der Zeiger auf den Hex-String
//			  z.B.  "FFF"  "12345678" "aFb123"
//	iLen	: Ist die Lnge des Hex-Strings in Zeichen
//			  Wenn iLen	kleiner 0 ist wird bis zum letzten Hex-Zeichen eingelesen.
//	Ergibt die eingelesene Zahl
int _cdecl strtoh(const char *pStr,int iLen)
{
char	cChar;
int		i,iVal;


	iVal = 0;

	while(iLen)
		{
		cChar = *pStr;

		if(cChar!=' ' && cChar!='\t')break;

		pStr++;
		iLen--;
		}

	while(iLen)
		{
		   i = strcharval(*pStr);
		if(i>=16)break;					// Falsches Zeichen

		iVal <<= 4;						// Zahl berechnen
		iVal  += i;
		pStr  += 1;
		iLen  -= 1;
		}


return iVal;
}

int _cdecl strtoh(const wchar_t *pStr,int iLen)
{
wchar_t		uChar;
int			i,iVal;


	iVal = 0;

	while(iLen)
		{
		uChar = *pStr;

		if(uChar!=' ' && uChar!='\t')break;

		pStr++;
		iLen--;
		}

	while(iLen)
		{
		   i = strcharval(*pStr);
		if(i>=16)break;					// Falsches Zeichen

		iVal <<= 4;						// Zahl berechnen
		iVal  += i;
		pStr  += 1;
		iLen  -= 1;
		}


return iVal;
}

//****************************************************************************
//*
//*		itostr
//*
//****************************************************************************
//	Wandelt eine Zahl in einen Text um.
//	iValue	: Ist die Zahl die umgewandelt werden soll
//	pStr	: Ist der Zeiger auf den Textpuffer wo die Zahl gespeichert wird.
//	iMode	: Bestimmt das Format der Textausgabe
//				0 Dez-Zahl  12345
//				1 Hex-Zahl	0x123
//      		2 Bin-Zahl 	0b101
//      		3 Okt-Zahl	0o765
//      		4 Hex-Zahl	0FFFh
//      		5 Bin-Zahl	1111b
//      		6 Okt-Zahl	7777o
//      		7 Hex-Zahl	16#FF
//      		8 Bin-Zahl	2#111
//      		9 Okt-Zahl	8#777
//			   10 Dez-Zahl  10#12
//			   15 Dez-Zahl  12345 (ohne Minus)
//			Ist iMode & 0x00003F00 ist die mindest Breite der Zahl
//			Ist iMode |=0x00004000 dann ist die mindest Breite auch die maximale
//			Ist iMode |=0x00008000 dann ist iMode&0xFF die Basis
//			Ist iMode |=0x00010000 dann wird die Zahl mit Leerzeichen auf min gefllt
//	Ergibt die pString-Lnge
int _cdecl itostr(int iValue,char *pStr,int iMode)
{
char        cBuffer[72];
unsigned	uBase;
int			iSize;
int			iFill;
int			iAdd;
int			iMax;
int			iMin;
int			i,j;


	i	  =  0;
	iFill =  0;
	iSize =  0;
	iAdd  =  0;
	iMin  =  1;
	iMax  = 32;

	if(!(unsigned char)iMode && iValue<0)		// Negative Zahlen
		 {
		 iSize  += 1;
		 pStr[0] ='-';
		 iValue  = -iValue;
		 }

	if(iMode&0x3F00)							// Fixe Breite
		{
		iMin = (iMode>>8)&0x3F;
		if(iSize&&iMin>2)iMin--;
		if(iMode&0x04000)iMax=iMin;
		if(iMode&0x10000)						// Mit Leerzeichen auffllen
			{
			iFill = iMin;
			iMin  = 1;
			}
		}

	if(iMode&0x8000)							// Verschiedene Basis
		{
		uBase = iMode&0xFF;
		}
	else{
		switch(iMode&0xFF)
			{
		case  1:*(short*)(pStr+iSize)=*(short*)"0x"  ;iSize+=2;uBase=16;break;
		case  2:*(short*)(pStr+iSize)=*(short*)"0b"  ;iSize+=2;uBase= 2;break;
		case  3:*(short*)(pStr+iSize)=*(short*)"0o"  ;iSize+=2;uBase= 8;break;
		case  4: cBuffer[0]='h';i=1;iAdd=1;iMin++;iMax++;      uBase=16;break;
		case  5: cBuffer[0]='b';i=1;      ;iMin++;iMax++;      uBase= 2;break;
		case  6: cBuffer[0]='o';i=1;iAdd=1;iMin++;iMax++;      uBase= 8;break;
		case  7: *(int  *)(pStr+iSize)=*(int  *)"16#";iSize+=3;uBase=16;break;
		case  8: *(short*)(pStr+iSize)=*(short*)"2#" ;iSize+=2;uBase= 2;break;
		case  9: *(short*)(pStr+iSize)=*(short*)"8#" ;iSize+=2;uBase= 8;break;
		case 10: *(int  *)(pStr+iSize)=*(int  *)"10#";iSize+=3;uBase=10;break;
		default:										       uBase=10;break;
			}
		}

	for(;i<iMax;i++)							// Zeichen berechnen
		{
		if(i>=iMin && !iValue)break;

		j  = (unsigned)iValue%uBase;
		j += (j<10)? '0':'A'-10;

		cBuffer[i] = (char)j;
		iValue     = (unsigned)iValue/uBase;
		}

	if(iAdd && cBuffer[i-1]>'9' && i<iMax)		// Null anhngen bei Assembler-Formaten
		{
		pStr[iSize]='0';
		iSize++;
		}

  	if(iFill>i)									// Mittlere Zeichen fllen
		{
		iFill -= i;
		iSize += iFill;
		*(int*)(pStr+iFill) = *(int*)pStr;

		for(iFill--;iFill>=0;iFill--)
			{
			pStr[iFill] = ' ';
			}
		}

	for(i--;i>=0;i--)							// Reihenfolge umdrehen
		{
		pStr[iSize]=cBuffer[i];
		iSize++;
		}


	pStr[iSize]=0;								// Nullzeichen anhngen



return iSize;
}

int _cdecl itostr(int iValue,wchar_t *pStr,int iMode)
{
wchar_t		uBuffer[72];
unsigned	uBase;
int			iSize;
int			iFill;
int			iAdd;
int			iMax;
int			iMin;
int			i,j;


	i	  =  0;
	iSize =  0;
	iFill =  0;
	iAdd  =  0;
	iMin  =  1;
	iMax  = 32;

	if(!(unsigned char)iMode && iValue<0)		// Negative Zahlen
		 {
		 iSize  += 1;
		 pStr[0] ='-';
		 iValue  = -iValue;
		 }

	if(iMode&0x3F00)							// Fixe Breite
		{
		iMin = (iMode>>8)&0x3F;
		if(iSize&&iMin>2)iMin--;
		if(iMode&0x04000)iMax=iMin;
		if(iMode&0x10000)						// Mit Leerzeichen auffllen
			{
			iFill = iMin;
			iMin  = 1;
			}
		}

	if(iMode&0x8000)							// Verschiedene Basis
		{
		uBase = iMode&0xFF;
		}
	else{
		switch(iMode&0xFF)
			{
		case  1:*(int*)(pStr+iSize)=*(int*)L"0x" ;iSize+=2;uBase=16;break;
		case  2:*(int*)(pStr+iSize)=*(int*)L"0b" ;iSize+=2;uBase= 2;break;
		case  3:*(int*)(pStr+iSize)=*(int*)L"0o" ;iSize+=2;uBase= 8;break;
		case  4: uBuffer[0]='h';i=1;iAdd=1;iMin++;iMax++  ;uBase=16;break;
		case  5: uBuffer[0]='b';i=1;      ;iMin++;iMax++  ;uBase= 2;break;
		case  6: uBuffer[0]='o';i=1;iAdd=1;iMin++;iMax++  ;uBase= 8;break;
		case  7: *(int*)(pStr+iSize)=*(int*)L"16";iSize+=3;uBase=16;pStr[iSize-1]='#';break;
		case  8: *(int*)(pStr+iSize)=*(int*)L"2#";iSize+=2;uBase= 2;break;
		case  9: *(int*)(pStr+iSize)=*(int*)L"8#";iSize+=2;uBase= 8;break;
		case 10: *(int*)(pStr+iSize)=*(int*)L"10";iSize+=3;uBase=10;pStr[iSize-1]='#';break;
		default:										   uBase=10;break;
			}
		}

	for(;i<iMax;i++)							// Zeichen berechnen
		{
		if(i>=iMin && !iValue)break;

		j  = (unsigned)iValue%uBase;
		j += (j<10)? '0':'A'-10;

		uBuffer[i] = (unsigned short)j;
		iValue     = (unsigned)iValue/uBase;
		}

	if(iAdd && uBuffer[i-1]>'9' && i<iMax)		// Null anhngen bei Assembler-Formaten
		{
		pStr[iSize]='0';
		iSize++;
		}

  	if(iFill>i)									// Mittlere Zeichen fllen
		{
		iFill -= i;
		iSize += iFill;
		*(int*)(pStr+iFill) = *(int*)pStr;

		for(iFill--;iFill>=0;iFill--)
			{
			pStr[iFill] = ' ';
			}
		}

	for(i--;i>=0;i--)							// Reihenfolge umdrehen
		{
		pStr[iSize]=uBuffer[i];
		iSize++;
		}


	pStr[iSize]=0;								// Nullzeichen anhngen



return iSize;
}

//****************************************************************************
//*
//*		qtostr
//*
//****************************************************************************
//	Wandelt eine Zahl in einen Text um.
//	iValue	: Ist die Zahl die umgewandelt werden soll
//	pStr	: Ist der Zeiger auf den Textpuffer wo die Zahl gespeichert wird.
//	iMode	: Bestimmt das Format der Textausgabe
//				0 Dez-Zahl  12345
//				1 Hex-Zahl	0x123
//      		2 Bin-Zahl 	0b101
//      		3 Okt-Zahl	0o765
//      		4 Hex-Zahl	0FFFh
//      		5 Bin-Zahl	1111b
//      		6 Okt-Zahl	7777o
//      		7 Hex-Zahl	16#FF
//      		8 Bin-Zahl	2#111
//      		9 Okt-Zahl	8#777
//			   10 Dez-Zahl  10#12
//			   15 Dez-Zahl  12345 (ohne Minus)
//			Ist iMode & 0x00003F00 ist die mindest Breite der Zahl
//			Ist iMode |=0x00004000 dann ist die mindest Breite auch die maximale
//			Ist iMode |=0x00008000 dann ist iMode&0xFF die Basis
//			Ist iMode |=0x00010000 dann wird die Zahl mit Leerzeichen auf min gefllt
//	Ergibt die pString-Lnge
int _cdecl qtostr(intq iValue,char *pStr,int iMode)
{
char        cBuffer[72];
unsigned	uBase;
int			iSize;
int			iFill;
int			iAdd;
int			iMax;
int			iMin;
int			i,j;


	i	  =  0;
	iFill =  0;
	iSize =  0;
	iAdd  =  0;
	iMin  =  1;
	iMax  = 32;

	if(!(unsigned char)iMode && iValue<0)		// Negative Zahlen
		 {
		 iSize  += 1;
		 pStr[0] ='-';
		 iValue  = -iValue;
		 }

	if(iMode&0x3F00)							// Mindestbreite
		{
		iMin = (iMode>>8)&0x3F;
		if(iSize&&iMin>2)iMin--;
		if(iMode&0x04000)iMax=iMin;
		if(iMode&0x10000)						// Mit Leerzeichen auffllen
			{
			iFill = iMin;
			iMin  = 1;
			}
		}

	if(iMode&0x8000)							// Verschiedene Basis
		{
		uBase = iMode&0xFF;
		}
	else{
		switch(iMode&0xFF)
			{
		case  1:*(short*)(pStr+iSize)=*(short*)"0x"  ;iSize+=2;uBase=16;break;
		case  2:*(short*)(pStr+iSize)=*(short*)"0b"  ;iSize+=2;uBase= 2;break;
		case  3:*(short*)(pStr+iSize)=*(short*)"0o"  ;iSize+=2;uBase= 8;break;
		case  4: cBuffer[0]='h';i=1;iAdd=1;iMin++;iMax++;      uBase=16;break;
		case  5: cBuffer[0]='b';i=1;      ;iMin++;iMax++;      uBase= 2;break;
		case  6: cBuffer[0]='o';i=1;iAdd=1;iMin++;iMax++;      uBase= 8;break;
		case  7: *(int  *)(pStr+iSize)=*(int  *)"16#";iSize+=3;uBase=16;break;
		case  8: *(short*)(pStr+iSize)=*(short*)"2#" ;iSize+=2;uBase= 2;break;
		case  9: *(short*)(pStr+iSize)=*(short*)"8#" ;iSize+=2;uBase= 8;break;
		case 10: *(int  *)(pStr+iSize)=*(int  *)"10#";iSize+=3;uBase=10;break;
		default:										       uBase=10;break;
			}
		}

	for(;i<iMax;i++)							// Zeichen berechnen
		{
		if(i>=iMin && !iValue)break;

		j  = (int)((unsigned __int64)iValue%(unsigned __int64)uBase);
		j += (j<10)? '0':'A'-10;

		cBuffer[i] = (char)j;
		iValue     = (unsigned __int64)iValue/uBase;
		}

	if(iAdd && cBuffer[i-1]>'9' && i<iMax)		// Null anhngen bei Assembler-Formaten
		{
		pStr[iSize]='0';
		iSize++;
		}

  	if(iFill>i)									// Mittlere Zeichen fllen
		{
		iFill -= i;
		iSize += iFill;
		*(int*)(pStr+iFill) = *(int*)pStr;

		for(iFill--;iFill>=0;iFill--)
			{
			pStr[iFill] = ' ';
			}
		}

	for(i--;i>=0;i--)							// Reihenfolge umdrehen
		{
		pStr[iSize]=cBuffer[i];
		iSize++;
		}


	pStr[iSize]=0;								// Nullzeichen anhngen



return iSize;
}

int _cdecl qtostr(intq iValue,wchar_t *pStr,int iMode)
{
unsigned short	uBuffer[72];
unsigned		uBase;
int				iSize;
int				iFill;
int				iAdd;
int				iMax;
int				iMin;
int				i,j;


	i	  =  0;
	iFill =  0;
	iSize =  0;
	iAdd  =  0;
	iMin  =  1;
	iMax  = 32;

	if(!(unsigned char)iMode && iValue<0)		// Negative Zahlen
		 {
		 iSize  += 1;
		 pStr[0] ='-';
		 iValue  = -iValue;
		 }

	if(iMode&0x3F00)							// Mindestbreite
		{
		iMin = (iMode>>8)&0x3F;
		if(iSize&&iMin>2)iMin--;
		if(iMode&0x04000)iMax=iMin;
		if(iMode&0x10000)						// Mit Leerzeichen auffllen
			{
			iFill = iMin;
			iMin  = 1;
			}
		}

	if(iMode&0x8000)							// Verschiedene Basis
		{
		uBase = iMode&0xFF;
		}
	else{
		switch(iMode&0xFF)
			{
		case  1:*(int*)(pStr+iSize)=*(int*)L"0x" ;iSize+=2;uBase=16;break;
		case  2:*(int*)(pStr+iSize)=*(int*)L"0b" ;iSize+=2;uBase= 2;break;
		case  3:*(int*)(pStr+iSize)=*(int*)L"0o" ;iSize+=2;uBase= 8;break;
		case  4: uBuffer[0]='h';i=1;iAdd=1;iMin++;iMax++  ;uBase=16;break;
		case  5: uBuffer[0]='b';i=1;      ;iMin++;iMax++  ;uBase= 2;break;
		case  6: uBuffer[0]='o';i=1;iAdd=1;iMin++;iMax++  ;uBase= 8;break;
		case  7: *(int*)(pStr+iSize)=*(int*)L"16";iSize+=3;uBase=16;pStr[iSize-1]='#';break;
		case  8: *(int*)(pStr+iSize)=*(int*)L"2#";iSize+=2;uBase= 2;break;
		case  9: *(int*)(pStr+iSize)=*(int*)L"8#";iSize+=2;uBase= 8;break;
		case 10: *(int*)(pStr+iSize)=*(int*)L"10";iSize+=3;uBase=10;pStr[iSize-1]='#';break;
		default:										   uBase=10;break;
			}
		}

	for(;i<iMax;i++)							// Zeichen berechnen
		{
		if(i>=iMin && !iValue)break;

		j  = (int)((unsigned __int64)iValue%(unsigned __int64)uBase);
		j += (j<10)? '0':'A'-10;

		uBuffer[i] = (unsigned short)j;
		iValue     = (unsigned __int64)iValue/uBase;
		}

	if(iAdd && uBuffer[i-1]>'9' && i<iMax)		// Null anhngen bei Assembler-Formaten
		{
		pStr[iSize]='0';
		iSize++;
		}

  	if(iFill>i)									// Mittlere Zeichen fllen
		{
		iFill -= i;
		iSize += iFill;
		*(int*)(pStr+iFill) = *(int*)pStr;

		for(iFill--;iFill>=0;iFill--)
			{
			pStr[iFill] = ' ';
			}
		}

	for(i--;i>=0;i--)							// Reihenfolge umdrehen
		{
		pStr[iSize]=uBuffer[i];
		iSize++;
		}


	pStr[iSize]=0;								// Nullzeichen anhngen



return iSize;
}

//****************************************************************************
//*
//*		strfind
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird.
//	iSize	: Ist die Lnge von pText
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strfind(const char *pString,const char *pText)
{
const char	*pFind;


	   pFind = strstr(pString,pText);
	if(pFind)return (int)(pFind-pString);

return -1;
}

int _cdecl strfind(const wchar_t *pString,const wchar_t *pText)
{
const wchar_t	*pFind;


	   pFind = wcsstr(pString,pText);
	if(pFind)return (int)(pFind-pString);

return -1;
}

int _cdecl strfind(const char *pString,const char *pText,int iSize)
{
char	cChar1,cChar2;
int		iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	cChar2 = pText[0];

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			cChar1=pString[iPos];
		if(!cChar1)return -1;

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memcmp(pString+iPos,pText,iSize)==0)break;
		}


return iPos-1;
}

int _cdecl strfind(const wchar_t *pString,const wchar_t *pText,int iSize)
{
unsigned short	uChar1,uChar2;
int				iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	uChar2 = pText[0];

	pText++;
	iSize--;
	iSize*=2;

	for(iPos=0;;)
		{
			uChar1=pString[iPos];
		if(!uChar1)return -1;

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memcmp(pString+iPos,pText,iSize)==0)break;
		}


return iPos-1;
}


//****************************************************************************
//*
//*		strifind
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird zwischen Gro und Kleinbuchstaben nicht unterschieden
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	iSize	: Ist die Lnge von pText
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strifind(const char *pString,const char *pText)
{
int		iPos,iSize;
char	cChar1,cChar2;



	    iSize = (int)strlen(pText);
	if(!iSize)return 0;

	   cChar2 = pText[0];
	if(cChar2>='a' && cChar2<='z')cChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			cChar1=pString[iPos];
		if(!cChar1)return -1;

		if(cChar1>='a' && cChar1<='z')
			{
			cChar1-=32;
			}

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memicmp(pString+iPos,pText,iSize)==0)break;
		}

return iPos-1;
}

int _cdecl strifind(const wchar_t *pString,const wchar_t *pText)
{
int				iPos,iSize;
unsigned short	uChar1,uChar2;



		iSize=(int)wcslen(pText);
	if(!iSize)return 0;

	   uChar2 = pText[0];
	if(uChar2>='a' && uChar2<='z')uChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			uChar1=pString[iPos];
		if(!uChar1)return -1;

		if(uChar1>='a' && uChar1<='z')
			{
			uChar1-=32;
			}

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memicmp(pString+iPos,pText,iSize*sizeof(wchar_t))==0)break;
		}

return iPos-1;
}

int _cdecl strifind(const char *pString,const char *pText,int iSize)
{
char	cChar1,cChar2;
int		iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	   cChar2 = pText[0];
	if(cChar2>='a' && cChar2<='z')cChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			cChar1=pString[iPos];
		if(!cChar1)return -1;

		if(cChar1>='a' && cChar1<='z')
			{
			cChar1-=32;
			}

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memicmp(pString+iPos,pText,iSize)==0)break;
		}


return iPos-1;
}

int _cdecl strifind(const wchar_t *pString,const wchar_t *pText,int iSize)
{
unsigned short	uChar1,uChar2;
int				iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	   uChar2 = pText[0];
	if(uChar2>='a' && uChar2<='z')uChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			uChar1=pString[iPos];
		if(!uChar1)return -1;

		if(uChar1>='a' && uChar1<='z')
			{
			uChar1-=32;
			}

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memicmp(pString+iPos,pText,iSize*sizeof(wchar_t))==0)break;
		}


return iPos-1;
}

//****************************************************************************
//*
//*		strlifind
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird zwischen Gro und Kleinbuchstaben nicht unterschieden und
//	bei Nullzeichen nicht abgebrochen.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	iLen	: Ist die Lnge der Zeichenfolge in Zeichen
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strlifind(const char *pString,const char *pText,int iLen)
{
int		iPos,iSize;
char	cChar1,cChar2;


		iSize = (int)strlen(pText);
	if(!iSize)return (iLen<=0)? -1:0;

	iLen -= iSize;

	   cChar2 = pText[0];
	if(cChar2>='a' && cChar2<='z')cChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			cChar1=pString[iPos];
		if(!cChar1)return -1;

		if(cChar1>='a' && cChar1<='z')
			{
			cChar1-=32;
			}

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memicmp(pString+iPos,pText,iSize)==0)break;
		}


return iPos-1;
}

int _cdecl strlifind(const wchar_t *pString,const wchar_t *pText,int iLen)
{
int				iPos,iSize;
unsigned short	uChar1,uChar2;


		iSize = (int)wcslen(pText);
	if(!iSize)return (iLen<=0)? -1:0;

	iLen -= iSize;

	   uChar2 = pText[0];
	if(uChar2>='a' && uChar2<='z')uChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			uChar1=pString[iPos];
		if(!uChar1)return -1;

		if(uChar1>='a' && uChar1<='z')
			{
			uChar1-=32;
			}

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memicmp(pString+iPos,pText,iSize*sizeof(wchar_t))==0)break;
		}


return iPos-1;
}

//****************************************************************************
//*
//*		strlfind
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird bei Nullzeichen nicht abgebrochen.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	iLen	: Ist die Lnge der Zeichenfolge in Zeichen
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strlfind(const char *pString,const char *pText,int iLen)
{
int		iPos,iSize;
char	cChar1,cChar2;


		iSize = (int)strlen(pText);
	if(!iSize)return (iLen<=0)? -1:0;

	cChar2 = pText[0];
	iLen  -= iSize;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			cChar1=pString[iPos];
		if(!cChar1)return -1;

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memcmp(pString+iPos,pText,iSize)==0)break;
		}


return iPos-1;
}

int _cdecl strlfind(const wchar_t *pString,const wchar_t *pText,int iLen)
{
int		iPos,iSize;
unsigned short	uChar1,uChar2;


		iSize = (int)wcslen(pText);
	if(!iSize)return (iLen<=0)? -1:0;

	uChar2 = pText[0];
	iLen  -= iSize;

	pText++;
	iSize--;
	iSize*=2;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			uChar1=pString[iPos];
		if(!uChar1)return -1;

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memcmp(pString+iPos,pText,iSize)==0*sizeof(wchar_t))break;
		}


return iPos-1;
}

//****************************************************************************
//*
//*		strlifind
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird zwischen Gro und Kleinbuchstaben nicht unterschieden und
//	bei Nullzeichen nicht abgebrochen.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	iLen	: Ist die Lnge der Zeichenfolge in Zeichen
//	iSize	: Ist die Lnge des Suchtextes in Zeichen
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strlifind(const char *pString,const char *pText,int iLen,int iSize)
{
char	cChar1,cChar2;
int		iPos;


	if(iSize<=0)return (iLen<=0)? -1:0;

	iLen -= iSize;

	   cChar2 = pText[0];
	if(cChar2>='a' && cChar2<='z')cChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			cChar1=pString[iPos];
		if(!cChar1)return -1;

		if(cChar1>='a' && cChar1<='z')
			{
			cChar1-=32;
			}

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memicmp(pString+iPos,pText,iSize)==0)break;
		}


return iPos-1;
}

int _cdecl strlifind(const wchar_t *pString,const wchar_t *pText,int iLen,int iSize)
{
unsigned short	uChar1,uChar2;
int				iPos;


	if(iSize<=0)return (iLen<=0)? -1:0;

	iLen -= iSize;

	   uChar2 = pText[0];
	if(uChar2>='a' && uChar2<='z')uChar2-=32;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			uChar1=pString[iPos];
		if(!uChar1)return -1;

		if(uChar1>='a' && uChar1<='z')
			{
			uChar1-=32;
			}

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memicmp(pString+iPos,pText,iSize*sizeof(wchar_t))==0)break;
		}


return iPos-1;
}

//****************************************************************************
//*
//*		strlfind
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird bei Nullzeichen nicht abgebrochen.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	iLen	: Ist die Lnge der Zeichenfolge in Zeichen
//	iSize	: Ist die Lnge des Suchtextes in Zeichen
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strlfind(const char *pString,const char *pText,int iLen,int iSize)
{
char	cChar1,cChar2;
int		iPos;


	if(iSize<=0)return (iLen<=0)? -1:0;

	cChar2 = pText[0];
	iLen  -= iSize;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			cChar1=pString[iPos];
		if(!cChar1)return -1;

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memcmp(pString+iPos,pText,iSize)==0)break;
		}


return iPos-1;
}

int _cdecl strlfind(const wchar_t *pString,const wchar_t *pText,int iLen,int iSize)
{
unsigned short	uChar1,uChar2;
int				iPos;


	if(iSize<=0)return (iLen<=0)? -1:0;

	uChar2 = pText[0];
	iLen  -= iSize;

	pText++;
	iSize--;
	iSize*=2;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			uChar1=pString[iPos];
		if(!uChar1)return -1;

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memcmp(pString+iPos,pText,iSize*sizeof(wchar_t))==0)break;
		}


return iPos-1;
}

//****************************************************************************
//*
//*		strfindex
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	iMode	: Diverse Modusbits fr verschiedene Funktionen
//				Bit0 = Zeilensprnge nach Text zulassen 
//				Bit1 = Anfhrungszeichen zulassen
//				Bit2 = Text ist immer ein ganzes Wort
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird.
//	cSign	: Ist das Zeichen das nach dem Text folgen muss.
//			  Leerstellen werden bersprungen.  
//	iSize	: Ist die Lnge von pText
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strfindex(const char *pString,int iMode,const char *pText,char cSign,int iSize)
{
char	cChar1;
char	cChar2;
int 	iBits;
int		iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	cChar2 = pText[0];
	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;
	
	for(iPos=0;;)
		{
			cChar1=pString[iPos];
		if(!cChar1)return -1;

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memcmp(pString+iPos,pText,iSize))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(pString[iPos]=='"' && cChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwchara(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits);
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

int _cdecl strfindex(const wchar_t *pString,int iMode,const wchar_t *pText,wchar_t cSign,int iSize)
{
unsigned short	uChar1;
unsigned short	uChar2;
int 			iBits;
int				iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	uChar2 = pText[0];
	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			uChar1=pString[iPos];
		if(!uChar1)return -1;

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memcmp(pString+iPos,pText,iSize*2))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(pString[iPos]=='"' && uChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwcharw(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits);
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

#ifdef 	_DEBUG

int _cdecl  strfindex(const char *pString,int iMode,const char *pFind,char cSign)
		{
		return strfindex(pString,iMode,pFind,cSign,(int)strlen(pFind));
		}

int _cdecl  strfindex(const char *pString,const char *pFind,char cSign)
		{
		return strfindex(pString,0x07,pFind,cSign,(int)strlen(pFind));
		}

int _cdecl strfindex(const char *pString,const char *pFind,char cSign,int iSize)
		{
		return strfindex(pString,0x07,pFind,cSign,iSize);
		}

int _cdecl  strfindex(const wchar_t *pString,int iMode,const wchar_t *pFind,wchar_t cSign)
		{
		return strfindex(pString,iMode,pFind,cSign,(int)wcslen(pFind));
		}

int _cdecl  strfindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign)
		{
		return strfindex(pString,0x07,pFind,cSign,(int)wcslen(pFind));
		}

int _cdecl strfindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign,int iSize)
		{
		return strfindex(pString,0x07,pFind,cSign,iSize);
		}

#endif 	

//****************************************************************************
//*
//*		strifindex
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird zwischen Gro und Kleinbuchstaben nicht unterschieden
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	iMode	: Diverse Modusbits fr verschiedene Funktionen
//				Bit0 = Zeilensprnge nach Text zulassen 
//				Bit1 = Anfhrungszeichen zulassen
//				Bit2 = Text ist immer ein ganzes Wort
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	cSign	: Ist das Zeichen das nach dem Text folgen muss.
//			  Leerstellen werden bersprungen.  
//	iSize	: Ist die Lnge von pText
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strifindex(const char *pString,int iMode,const char *pText,char cSign,int iSize)
{
char	cChar1;
char	cChar2;
int		iBits;
int		iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	   cChar2 = pText[0];
	if(cChar2>='a' && cChar2<='z')cChar2-=32;

	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			cChar1=pString[iPos];
		if(!cChar1)return -1;

		if(cChar1>='a' && cChar1<='z')
			{
			cChar1-=32;
			}

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memicmp(pString+iPos,pText,iSize))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(pString[iPos]=='"' && cChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwchara(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits);
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

int _cdecl strifindex(const wchar_t *pString,int iMode,const wchar_t *pText,wchar_t cSign,int iSize)
{
unsigned short	uChar1;
unsigned short	uChar2;
int				iBits;
int				iPos;


	if(iSize<=0)return (*pString==0)? -1:0;

	   uChar2 = pText[0];
	if(uChar2>='a' && uChar2<='z')uChar2-=32;

	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
			uChar1=pString[iPos];
		if(!uChar1)return -1;

		if(uChar1>='a' && uChar1<='z')
			{
			uChar1-=32;
			}

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memicmp(pString+iPos,pText,iSize))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(pString[iPos]=='"' && uChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwcharw(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits);
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

#ifdef 	_DEBUG

int _cdecl  strifindex(const char *pString,int iMode,const char *pFind,char cSign)
		{
		return strifindex(pString,iMode,pFind,cSign,(int)strlen(pFind));
		}

int _cdecl  strifindex(const char *pString,const char *pFind,char cSign)
		{
		return strifindex(pString,0x07,pFind,cSign,(int)strlen(pFind));
		}

int _cdecl strifindex(const char *pString,const char *pFind,char cSign,int iSize)
		{
		return strifindex(pString,0x07,pFind,cSign,iSize);
		}

int _cdecl  strifindex(const wchar_t *pString,int iMode,const wchar_t *pFind,wchar_t cSign)
		{
		return strifindex(pString,iMode,pFind,cSign,(int)wcslen(pFind));
		}

int _cdecl  strifindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign)
		{
		return strifindex(pString,0x07,pFind,cSign,(int)wcslen(pFind));
		}

int _cdecl strifindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign,int iSize)
		{
		return strifindex(pString,0x07,pFind,cSign,iSize);
		}

#endif 	

//****************************************************************************
//*
//*		strlifindex
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird zwischen Gro und Kleinbuchstaben nicht unterschieden und
//	bei Nullzeichen nicht abgebrochen.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	iMode	: Diverse Modusbits fr verschiedene Funktionen
//				Bit0 = Zeilensprnge nach Text zulassen 
//				Bit1 = Anfhrungszeichen zulassen
//				Bit2 = Text ist immer ein ganzes Wort
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	cSign	: Ist das Zeichen das nach dem Text folgen muss.
//			  Leerstellen werden bersprungen.  
//	iLen	: Ist die Lnge der Zeichenfolge in Zeichen
//	iSize	: Ist die Lnge des Suchtextes in Zeichen
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strlifindex(const char *pString,int iMode,const char *pText,char cSign,int iLen,int iSize)
{
char	cChar1;
char	cChar2;
int		iBits;
int		iPos;
int		iEnd;


	if(iSize<=0)return (iLen<=0)? -1:0;

	iEnd  = iLen;
	iLen -= iSize;

	   cChar2 = pText[0];
	if(cChar2>='a' && cChar2<='z')cChar2-=32;

	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			cChar1=pString[iPos];
		if(!cChar1)return -1;

		if(cChar1>='a' && cChar1<='z')
			{
			cChar1-=32;
			}

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memicmp(pString+iPos,pText,iSize))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(iPos<iEnd && pString[iPos]=='"' && cChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwchara(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits,iEnd-iPos);
		
		if(iPos>=iEnd)return -1;
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

int _cdecl strlifindex(const wchar_t *pString,int iMode,const wchar_t *pText,wchar_t cSign,int iLen,int iSize)
{
unsigned short	uChar1;
unsigned short	uChar2;
int				iBits;
int				iPos;
int				iEnd;


	if(iSize<=0)return (iLen<=0)? -1:0;

	iEnd  = iLen;
	iLen  -= iSize;

	   uChar2 = pText[0];
	if(uChar2>='a' && uChar2<='z')uChar2-=32;

	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			uChar1=pString[iPos];
		if(!uChar1)return -1;

		if(uChar1>='a' && uChar1<='z')
			{
			uChar1-=32;
			}

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memicmp(pString+iPos,pText,iSize))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(iPos<iEnd && pString[iPos]=='"' && uChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwcharw(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits,iEnd-iPos);
		
		if(iPos>=iEnd)return -1;
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

#ifdef 	_DEBUG

int _cdecl  strlifindex(const char *pString,int iMode,const char *pFind,char cSign,int iLen)
		{
		return strlifindex(pString,iMode,pFind,cSign,iLen,(int)strlen(pFind));
		}

int _cdecl  strlifindex(const char *pString,const char *pFind,char cSign,int iLen)
		{
		return strlifindex(pString,0x07,pFind,cSign,iLen,(int)strlen(pFind));
		}

int _cdecl strlifindex(const char *pString,const char *pFind,char cSign,int iLen,int iSize)
		{
		return strlifindex(pString,0x07,pFind,cSign,iLen,iSize);
		}

int _cdecl  strlifindex(const wchar_t *pString,int iMode,const wchar_t *pFind,wchar_t cSign,int iLen)
		{
		return strlifindex(pString,iMode,pFind,cSign,iLen,(int)wcslen(pFind));
		}

int _cdecl  strlifindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign,int iLen)
		{
		return strlifindex(pString,0x07,pFind,cSign,iLen,(int)wcslen(pFind));
		}

int _cdecl strlifindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign,int iLen,int iSize)
		{
		return strlifindex(pString,0x07,pFind,cSign,iLen,iSize);
		}

#endif 	

//****************************************************************************
//*
//*		strlfindex
//*
//****************************************************************************
//	Sucht die Position an wo eine Text in einer Zeichenfolge enthalten ist.
//	Dabei wird bei Nullzeichen nicht abgebrochen.
//	pString	: Ist der Zeiger auf die Zeichenfolge in der gesucht wird.
//	iMode	: Diverse Modusbits fr verschiedene Funktionen
//				Bit0 = Zeilensprnge nach Text zulassen 
//				Bit1 = Anfhrungszeichen zulassen
//				Bit2 = Text ist immer ein ganzes Wort
//	pText	: Ist der Zeiger auf den Text nach dem gesucht wird
//	cSign	: Ist das Zeichen das nach dem Text folgen muss.
//			  Leerstellen werden bersprungen.  
//	iLen	: Ist die Lnge der Zeichenfolge in Zeichen
//	iSize	: Ist die Lnge des Suchtextes in Zeichen
//	Ergibt die Position an der der Text gefunden wurde bzw. -1 wenn nicht.
int _cdecl strlfindex(const char *pString,int iMode,const char *pText,char cSign,int iLen,int iSize)
{
char	cChar1;
char	cChar2;
int		iBits;
int		iPos;
int		iEnd;


	if(iSize<=0)return (iLen<=0)? -1:0;

	cChar2 = pText[0];
	iEnd   = iLen;
	iLen  -= iSize;
	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			cChar1=pString[iPos];
		if(!cChar1)return -1;

		iPos++;

		if(cChar1!=cChar2)continue;
		if(memcmp(pString+iPos,pText,iSize))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(iPos<iEnd && pString[iPos]=='"' && cChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwchara(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits,iEnd-iPos);
		
		if(iPos>=iEnd)return -1;
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

int _cdecl strlfindex(const wchar_t *pString,int iMode,const wchar_t *pText,wchar_t cSign,int iLen,int iSize)
{
unsigned short	uChar1;
unsigned short	uChar2;
int				iBits;
int				iPos;
int				iEnd;


	if(iSize<=0)return (iLen<=0)? -1:0;

	uChar2 = pText[0];
	iEnd   = iLen;
	iLen  -= iSize;
	iBits  = (iMode&0x01)? 0xE0:0xA0;

	pText++;
	iSize--;

	for(iPos=0;;)
		{
		if(iPos>iLen)return -1;

			uChar1=pString[iPos];
		if(!uChar1)return -1;

		iPos++;

		if(uChar1!=uChar2)continue;
		if(memcmp(pString+iPos,pText,iSize*2))
			{
			continue;
			}

		iPos +=	iSize;
		
		if(iPos<iEnd && pString[iPos]=='"' && uChar2!='"' && (iMode&2) && iPos>iSize+1 && pString[iPos-iSize-2]=='"') 
			{
			iPos++;
			}
		else if(iMode&4)
			{
			if(iPos>iSize+1 && strwcharw(pString[iPos-iSize-2],0x3000001F))
				{
				continue;
				}
			}
		
		iPos += strwlen(pString+iPos,iBits,iEnd-iPos);
		
		if(iPos>=iEnd)return -1;
		
		if(pString[iPos]==cSign)
			{
			break;
			}
		}


return iPos+1;
}

#ifdef 	_DEBUG

int _cdecl  strlfindex(const char *pString,int iMode,const char *pFind,char cSign,int iLen)
		{
		return strlfindex(pString,iMode,pFind,cSign,iLen,(int)strlen(pFind));
		}

int _cdecl  strlfindex(const char *pString,const char *pFind,char cSign,int iLen)
		{
		return strlfindex(pString,0x07,pFind,cSign,iLen,(int)strlen(pFind));
		}

int _cdecl strlfindex(const char *pString,const char *pFind,char cSign,int iLen,int iSize)
		{
		return strlfindex(pString,0x07,pFind,cSign,iLen,iSize);
		}

int _cdecl  strlfindex(const wchar_t *pString,int iMode,const wchar_t *pFind,wchar_t cSign,int iLen)
		{
		return strlfindex(pString,iMode,pFind,cSign,iLen,(int)wcslen(pFind));
		}

int _cdecl  strlfindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign,int iLen)
		{
		return strlfindex(pString,0x07,pFind,cSign,iLen,(int)wcslen(pFind));
		}

int _cdecl strlfindex(const wchar_t *pString,const wchar_t *pFind,wchar_t cSign,int iLen,int iSize)
		{
		return strlfindex(pString,0x07,pFind,cSign,iLen,iSize);
		}

#endif 	

//*****************************************************************************
//*
//*		strencode
//*
//*****************************************************************************
//	Verschlsselt einen Text
//	pText		: Ist der Zeiger auf den zu verschlsselnden Textpuffer
//	uKey		: Ist der Schlssel
//	iLen		: Ist die Lnge von pText, bei -1 wird die String-Lnge benutzt
//	iStart		: Ist die Nummer ab der Zeichen codiert werden sollen
//				  Wenn ' ' angegeben wird werden alle Zeichen von 0 bis
//				  inklusive Space nicht codiert
//	Ergibt die Anzahl der codierten Zeichen
int _cdecl strencode(char *pText,unsigned uKey,int iLen,int iStart)
{
unsigned char  *pPtr=(unsigned char*)pText;
unsigned		uAdd[4],uMax,uDiv,uVal;
int				i;



	if(iLen  <0)iLen=(int)strlen(pText);
	if(iStart<1)iStart=1;

	uMax = 256-iStart-1;
	uDiv = iStart;

	for(i=0;i<4;i++)						// Die Schlssel erzeugen
		{
		   uAdd[i] =uKey&0xFF;
		if(uAdd[i]>=uMax)uAdd[i]=uMax-1;

		uKey>>=8;
		}


	for(i=0;i<iLen;i++)						// Verschlsseln
		{
		   uVal=pPtr[i];
		if(uVal<uDiv)continue;
		   uVal+=uAdd[i&3];
		if(uVal>=256)uVal+=uDiv;

		pPtr[i]=(char)uVal;
		}


return iLen;
}

int _cdecl strencode(wchar_t *pText,unsigned uKey,int iLen,int iStart)
{
unsigned		uAdd[4],uMax,uDiv,uVal;
int				i;



	if(iLen  <0)iLen=(int)wcslen(pText);
	if(iStart<1)iStart=1;

	uMax = 256-iStart-1;
	uDiv = iStart;

	for(i=0;i<4;i++)						// Die Schlssel erzeugen
		{
		   uAdd[i] =uKey&0xFF;
		if(uAdd[i]>=uMax)uAdd[i]=uMax-1;

		uKey>>=8;
		}


	for(i=0;i<iLen;i++)						// Verschlsseln
		{
		   uVal=pText[i];
		if(uVal<uDiv)continue;
		   uVal+=uAdd[i&3];
		if(uVal>=256)uVal+=uDiv;

		pText[i]=(unsigned short)uVal;
		}


return iLen;
}

//*****************************************************************************
//*
//*		strdecode
//*
//*****************************************************************************
//	Entschlsselt den Text
//	pText		: Ist der Zeiger auf den zu entschlsselnden Textpuffer
//	uKey		: Ist der Schlssel
//	iLen		: Ist die Lnge von pText, bei -1 wird die String-Lnge benutzt
//	iStart		: Ist die Nummer ab der Zeichen decodiert werden sollen
//				  Wenn ' ' angegeben wird werden alle Zeichen von 0 bis
//				  inklusive Space nicht decodiert
//	Ergibt die Anzahl der decodierten Zeichen
int _cdecl strdecode(char *pText,unsigned uKey,int iLen,int iStart)
{
unsigned char  *pPtr=(unsigned char*)pText;
unsigned		uAdd[4],uMax;
int				uDiv,uVal;
int				i;



	if(iLen  <0)iLen=(int)strlen(pText);
	if(iStart<1)iStart=1;

	uMax = 256-iStart-1;
	uDiv = iStart+256;

	for(i=0;i<4;i++)						// Die Schlssel erzeugen
		{
		   uAdd[i]=uKey&0xFF;
		if(uAdd[i]>=uMax)uAdd[i]=uMax-1;

		uKey>>=8;
		}


	for(i=0;i<iLen;i++)						// Entschlsseln
		{
		   uVal=pPtr[i]+256;
		if(uVal<uDiv)continue;
		   uVal-=uAdd[i&3];
		if(uVal<uDiv)uVal-=uDiv;

		pPtr[i]=(char)uVal;
		}


return iLen;
}

int _cdecl strdecode(wchar_t *pText,unsigned uKey,int iLen,int iStart)
{
unsigned		uAdd[4],uMax;
int				uDiv,uVal;
int				i;



	if(iLen  <0)iLen=(int)wcslen(pText);
	if(iStart<1)iStart=1;

	uMax = 256-iStart-1;
	uDiv = iStart+256;

	for(i=0;i<4;i++)						// Die Schlssel erzeugen
		{
		   uAdd[i] =uKey&0xFF;
		if(uAdd[i]>=uMax)uAdd[i]=uMax-1;

		uKey>>=8;
		}


	for(i=0;i<iLen;i++)						// Entschlsseln
		{
		   uVal=pText[i]+256;
		if(uVal<uDiv)continue;
		   uVal-=uAdd[i&3];
		if(uVal<uDiv)uVal-=uDiv;

		pText[i]=(unsigned short)uVal;
		}


return iLen;
}

//****************************************************************************
//*
//*		strtomem
//*
//****************************************************************************
//	Wandelt einen Text in einen Speicherbereich um. Spezialzeichen
//	wie // /n /r /b /a /0 /i werden bercksichtigt. Fngt der Text mit '"'
//	an so werden nur die Zeichen zwischen den Anfhrungszeichen ausgewertet.
//	pMem	: Ist der Zeiger auf den Zielpuffer wo die Bytes gespeichert werden.
//	iLen	: Ist die maximale Gre des Zielpuffers.
//			  Folgende Flags knnen gesetzt werden:
//					STR_NOQUOTE  : Anfhrungszeichen am Anfang nicht berspringen.
//					STR_ANSI     : Der Speicherbereich wird in ANSI codiert.
//					STR_UTF8	 : Der Speicherbereich wird in UTF8 codiert.
//					STR_WIDE	 : Der Speicherbereich wird in UTF16 codiert.
//	pStr	: Ist det Zeiger auf den Text der umgewandelt werden soll.
//	pCount	: Hier wird die Anzahl der verarbeiteten Zeichen aus dem Text
//			  gespeichert. Wenn der Zeiger Null ist wird nichts gespeichert.
//	Ergibt die Anzahl der gespeicherten Bytes im Zielpuffer
int	_cdecl strtomem(void *pMem,int iLen,const char *pText,int *pCount)
{
const char *pStart=pText;
char	   *pPtr=(char*)pMem;
char	    cTemp[4];
unsigned	uSign;
int			iSize;
int			iStop;
int			iNum;



	iSize = 0;
	iStop = 0;

	if(*pText=='\"' && !(iLen&STR_NOQUOTE))			// Anfhrungszeichen berspringen
		{
		iStop=1;
		pText++;
		}

	if((iLen&(STR_UTF8|STR_WIDE))==0)				// ANSI codiert
		{
		iLen&=~(STR_NOQUOTE|STR_ANSI|STR_UTF8|STR_WIDE);

		while(iSize<iLen)
			{
			   uSign=*pText;
			if(uSign==0)break;
			if(uSign=='"' && iStop){pText++;break;}
			if(uSign=='\\')							// Escape Sequenzen	umwandeln
				{
				pPtr[iSize]=(char)strtoc(pText,&iNum);
				pText += iNum;
				iSize += 1;
				continue;
				}

			pPtr[iSize]=(char)uSign;				// Zeichen speichern
			iSize += 1;
			pText += 1;
			}
		}
	else if(iLen&STR_UTF8)							// UTF8 codiert
		{
		iLen&=~(STR_NOQUOTE|STR_ANSI|STR_UTF8|STR_WIDE);

		while(iSize<iLen)
			{
			   uSign=*pText;
			if(uSign==0)break;
			if(uSign=='"' && iStop){pText++;break;}
			if(uSign=='\\')							// Escape-Sequenzen umwandeln
				{
				uSign=strtoc(pText,&iNum);
				pText+=iNum;
				}
			else{
				pText++;
				}

			if(uSign>=128 && iLen-iSize<4)
				{
				   iNum = chartoutf8(cTemp,uSign);
				if(iNum>iLen-iSize)iNum=iLen-iSize;
				memcpy(pPtr+iSize,cTemp,iNum);
				iSize += iNum;
				pText += iNum;
				}
			else{									// Zeichen speichern
				iNum   = chartoutf8(pPtr+iSize,uSign);
				iSize += iNum;
				pText += iNum;
				}
			}
		}
	else{											// UTF16 codiert
		iLen&=~(STR_NOQUOTE|STR_ANSI|STR_UTF8|STR_WIDE);

		while(iSize<iLen)
			{
			   uSign=*pText;
			if(uSign==0)break;
			if(uSign=='"' && iStop){pText++;break;}
			if(uSign=='\\')							// Escape-Sequenzen umwandeln
				{
				*(short*)(pPtr+iSize)=(short)strtoc(pText,&iNum);
				pText += iNum;
				iSize += 2;
				continue;
				}

			*(short*)(pPtr+iSize)=(short)uSign;
			iSize += 2;
			pText += 1;
			}
		}
													// Lnge abspeichern
	if(pCount)
		{
		*pCount=(int)(pText-pStart);
		}

	if(iSize<iLen)									// Nullzeichen anhngen
		{
		pPtr[iSize]=0;
		}



return iSize;
}

int	_cdecl strtomem(void *pMem,int iLen,const wchar_t *pText,int *pCount)
{
const wchar_t		   *pStart=pText;
char				   *pPtr=(char*)pMem;
char					cTemp[4];
unsigned				uSign;
int						iSize;
int						iStop;
int						iNum;



	iSize = 0;
	iStop = 0;

	if(*pText=='\"' && !(iLen&STR_NOQUOTE))			// Anfhrungszeichen berspringen
		{
		iStop=1;
		pText++;
		}

	if((iLen&(STR_UTF8|STR_WIDE))==0)				// ANSI codiert
		{
		iLen&=~(STR_NOQUOTE|STR_ANSI|STR_UTF8|STR_WIDE);

		while(iSize<iLen)
			{
			   uSign=*pText;
			if(uSign==0)break;
			if(uSign=='"' && iStop){pText++;break;}
			if(uSign=='\\')							// Escape-Sequenzen	umwandeln
				{
				pPtr[iSize]=(char)strtoc(pText,&iNum);
				pText += iNum;
				iSize += 1;
				continue;
				}

			pPtr[iSize]=(char)uSign;				// Zeichen speichern
			iSize += 1;
			pText += 1;
			}
		}
	else if(iLen&STR_UTF8)							// UTF8 codiert
		{
		iLen&=~(STR_NOQUOTE|STR_ANSI|STR_UTF8|STR_WIDE);

		while(iSize<iLen)
			{
			   uSign=*pText;
			if(uSign==0)break;
			if(uSign=='"' && iStop){pText++;break;}
			if(uSign=='\\')							// Escape-Sequenzen	umwandeln
				{
				uSign  = strtoc(pText,&iNum);
				pText += iNum;
				}
			else{
				pText++;
				}

			if(uSign>=128 && iLen-iSize<4)
				{
				   iNum = chartoutf8(cTemp,uSign);
				if(iNum>iLen-iSize)iNum=iLen-iSize;
				memcpy(pPtr+iSize,cTemp,iNum);
				iSize += iNum;
				pText += iNum;
				}
			else{									// Zeichen speichern
				iNum   = chartoutf8(pPtr+iSize,uSign);
				iSize += iNum;
				pText += iNum;
				}
			}
		}
	else{											// UTF16 codiert
		iLen&=~(STR_NOQUOTE|STR_ANSI|STR_UTF8|STR_WIDE);

		while(iSize<iLen)
			{
			   uSign=*pText;
			if(uSign==0)break;
			if(uSign=='"' && iStop){pText++;break;}
			if(uSign=='\\')							// Escape Sequenzen	umwandeln
				{
				*(short*)(pPtr+iSize)=(short)strtoc(pText,&iNum);
				pText += iNum;
				iSize += 2;
				continue;
				}

			*(short*)(pPtr+iSize)=(short)uSign;
			iSize += 2;
			pText += 1;
			}
		}

	if(pCount)										// Lnge abspeichern
		{
		*pCount=(int)(pText-pStart);
		}

	if(iSize<iLen)									// Nullzeichen anhngen
		{
		pPtr[iSize]=0;
		}



return iSize;
}

//****************************************************************************
//*
//*		memtostr
//*
//****************************************************************************
//	Wandelt einen Speicherbereich in einen Text um.
//	Zeichen kleiner 32 werden dabei in /x??? umgewandelt.
//	pStr	: Ist der Zeiger auf den Textpuffer in dem der Text gespeichert wird.
//	iMax	: Ist die Gre des Textpuffers in Zeichen
//	pMem	: Ist der Zeiger auf Speicherbereich von dem gelesen wird.
//	iLen	: Ist die Gre des Speicherbereiches in Byte
//	iMode	: STR_NOQUOTE		Keine Anfhrungszeichen einfgen
//			  STR_REGMODE		Spezialzeichen sind : \" \\ \n \r
//			  STR_HEXMODE		Spezialzeichen nur als "\x??" ausugeben
//			  STR_WIDE			Der Speicher ist im UTF16-Format codieren
//			  STR_UTF8			Der Speicher ist im UTF8-Format codieren
//			  STR_ANSI			Der Speicher ist im ANSI-Format codieren
//			  STR_CCPSPLIT		Spezialzeichen mit nachfolgendem Hexdigit trennen
//	Ergibt die Lnge des erzeugten Textes ohne Nullzeichen am Ende.
int _cdecl memtostr(char *pStr,int iMax,const void *pMem,int iLen,int iMode)
{
unsigned char  *pPtr=(unsigned char*)pMem;
unsigned		uSign;
int				iSize;
int				iFull;


	iSize = 0;
	iFull = iMax;

	if(iMode&STR_NOQUOTE)					// Am Anfang kein Anfhrungszeichen einfgen
		{
		iMax += 2;
		}
	else{
		if(iMax>=2)							// Am Anfang ein Anfhrungszeichen
			{
			pStr[0]='\"';
			pStr++;
			iMax--;
			}
		}

	if(!(iMode&(STR_WIDE|STR_UTF8)))		// ANSI codiert
		{
		while(iSize<iLen)
			{
			if(iMax<=2)break;

			uSign = pPtr[iSize];
			iSize++;

			if(uSign>=32 && uSign!='\"' && uSign!='\\')
				{
				pStr[0] = (char)uSign;
				pStr++;
				iMax--;
				continue;
				}

			if(iMax<=4)break;				// Zuwenig freier Speicher ?

 			pStr[0]='\\';
			pStr++;
			iMax--;

			if(!(iMode&STR_HEXMODE))
				{
				if(uSign=='\\'){*pStr='\\';iMax--;pStr++;continue;}
				if(uSign=='\"'){*pStr='\"';iMax--;pStr++;continue;}
				if(uSign=='\r'){*pStr='r' ;iMax--;pStr++;continue;}
				if(uSign=='\n'){*pStr='n' ;iMax--;pStr++;continue;}
				}

			if(!(iMode&(STR_HEXMODE|STR_REGMODE)))
				{
				if(uSign=='\''){*pStr='\'';iMax--;pStr++;continue;}
				if(uSign=='\t'){*pStr='t' ;iMax--;pStr++;continue;}
				if(uSign=='\a'){*pStr='a' ;iMax--;pStr++;continue;}
				if(uSign=='\b'){*pStr='b' ;iMax--;pStr++;continue;}
				if(uSign=='\f'){*pStr='f' ;iMax--;pStr++;continue;}
				if(uSign=='\v'){*pStr='v' ;iMax--;pStr++;continue;}
				}

			if(iMax<=6)break;

			if(iMode&STR_CCPSPLIT)			// Codierung "\x1234""abc"
				{
				pStr[0]  = 'x' ;
				pStr[1]  = cHexDigits[uSign>>  4];
				pStr[2]  = cHexDigits[uSign&0x0F];
				pStr    += 3;
				iMax    -= 3;

				if(iSize<iLen && strwchara(pPtr[iSize],0x00040001))
					{
					if(iMax<=7)break;

					pStr[0]  = '"' ;		// "" anhngen wenn weiteres HexDigit
					pStr[1]  = '"' ;
					pStr    += 2;
					iMax    -= 2;
					}

				continue;
				}
			else{
				pStr[0]  = 'x' ;
				pStr[1]  = '0' ;
				pStr[2]  = cHexDigits[uSign>>  4];
				pStr[3]  = cHexDigits[uSign&0x0F];
				pStr    += 4;
				iMax    -= 4;
				}
			}
		}
	else if(iMode&STR_UTF8)					// UTF8 codiert
		{
		while(iSize<iLen)
			{
			if(iMax<=2)break;

			iSize += utf8tochar((char*)pPtr+iSize,uSign);

			if(uSign>=256)continue;

			if(uSign>=32 && uSign!='\"' && uSign!='\\')
				{
				pStr[0]=(char)uSign;
				pStr++;
				iMax--;
				continue;
				}

			if(iMax<=4)break;				// Zuwenig freier Speicher ?

 			pStr[0] ='\\';
			pStr++;
			iMax--;

			if(!(iMode&STR_HEXMODE))
				{
				if(uSign=='\\'){*pStr='\\';iMax--;pStr++;continue;}
				if(uSign=='\"'){*pStr='\"';iMax--;pStr++;continue;}
				if(uSign=='\r'){*pStr='r' ;iMax--;pStr++;continue;}
				if(uSign=='\n'){*pStr='n' ;iMax--;pStr++;continue;}
				}

			if(!(iMode&(STR_HEXMODE|STR_REGMODE)))
				{
				if(uSign=='\''){*pStr='\'';iMax--;pStr++;continue;}
				if(uSign=='\t'){*pStr='t' ;iMax--;pStr++;continue;}
				if(uSign=='\a'){*pStr='a' ;iMax--;pStr++;continue;}
				if(uSign=='\b'){*pStr='b' ;iMax--;pStr++;continue;}
				if(uSign=='\f'){*pStr='f' ;iMax--;pStr++;continue;}
				if(uSign=='\v'){*pStr='v' ;iMax--;pStr++;continue;}
				}

			if(iMax<=6)break;

			if(iMode&STR_CCPSPLIT)			// Codierung "\x1234""abc"
				{
				pStr[0]  = 'x' ;
				pStr[1]  = cHexDigits[uSign>>  4];
				pStr[2]  = cHexDigits[uSign&0x0F];
				pStr    += 3;
				iMax    -= 3;

				if(iSize<iLen && strwchara(pPtr[iSize],0x00040001))
					{
					if(iMax<=7)break;

					pStr[0]  = '"' ;		// "" anhngen wenn weiteres HexDigit
					pStr[1]  = '"' ;
					pStr    += 2;
					iMax    -= 2;
					}

				continue;
				}

			pStr[0]  = 'x' ;
			pStr[1]  = '0' ;
			pStr[2]  = cHexDigits[uSign>>  4];
			pStr[3]  = cHexDigits[uSign&0x0F];
			pStr    += 4;
			iMax    -= 4;
			}
		}
	else if(iMode&STR_WIDE)					// UTF16 codiert
		{
		iLen &= ~1;

		while(iSize<iLen)
			{
			if(iMax<=2)break;

			uSign  = *(unsigned short*)(pPtr+iSize);
			iSize += 2;

			if(uSign>=256)continue;

			if(uSign>=32 && uSign!='\"' && uSign!='\\')
				{
				pStr[0]=(char)uSign;
				pStr++;
				iMax--;
				continue;
				}

			if(iMax<=4)break;				// Zuwenig freier Speicher ?

 			pStr[0] ='\\';
			pStr++;
			iMax--;

			if(!(iMode&STR_HEXMODE))
				{
				if(uSign=='\\'){*pStr='\\';iMax--;pStr++;continue;}
				if(uSign=='\"'){*pStr='\"';iMax--;pStr++;continue;}
				if(uSign=='\r'){*pStr='r' ;iMax--;pStr++;continue;}
				if(uSign=='\n'){*pStr='n' ;iMax--;pStr++;continue;}
				}

			if(!(iMode&(STR_HEXMODE|STR_REGMODE)))
				{
				if(uSign=='\''){*pStr='\'';iMax--;pStr++;continue;}
				if(uSign=='\t'){*pStr='t' ;iMax--;pStr++;continue;}
				if(uSign=='\a'){*pStr='a' ;iMax--;pStr++;continue;}
				if(uSign=='\b'){*pStr='b' ;iMax--;pStr++;continue;}
				if(uSign=='\f'){*pStr='f' ;iMax--;pStr++;continue;}
				if(uSign=='\v'){*pStr='v' ;iMax--;pStr++;continue;}
				}

			if(iMax<=6)break;

			if(iMode&STR_CCPSPLIT)			// Codierung "\x1234""abc"
				{
				pStr[0]  = 'x' ;
				pStr[1]  = cHexDigits[uSign>>  4];
				pStr[2]  = cHexDigits[uSign&0x0F];
				pStr    += 3;
				iMax    -= 3;

				if(iSize<iLen && strwchara(pPtr[iSize],0x00040001))
					{
					if(iMax<=7)break;

					pStr[0]  = '"' ;		// "" anhngen wenn weiteres HexDigit
					pStr[1]  = '"' ;
					pStr    += 2;
					iMax    -= 2;
					}

				continue;
				}

			pStr[0]  = 'x' ;
			pStr[1]  = '0' ;
			pStr[2]  = cHexDigits[uSign>>  4];
			pStr[3]  = cHexDigits[uSign&0x0F];
			pStr    += 4;
			iMax    -= 4;
			}
		}

	if(iMode&STR_NOQUOTE)					// Am Ende kein Anfhrungszeichen anhngen
		{
		iMax-=2;
		}
	else{
		if(iMax>=2)							// Am Ende ein Anfhrungszeichen anhngen
			{
			pStr[0]='\"';
			pStr++;
			iMax--;
			}
		}


	if(iMax>0)*pStr=0;



return iFull-iMax;
}

int _cdecl memtostr(wchar_t *pStr,int iMax,const void *pMem,int iLen,int iMode)
{
unsigned char  *pPtr=(unsigned char*)pMem;
unsigned		uSign;
int				iSize;
int				iFull;


	iSize = 0;
	iFull = iMax;

	if(iMode&STR_NOQUOTE)					// Am Anfang kein Anfhrungszeichen einfgen
		{
		iMax+=2;
		}
	else{
		if(iMax>=2)							// Am Anfang ein Anfhrungszeichen
			{
			pStr[0]='\"';
			pStr++;
			iMax--;
			}
		}

	if(!(iMode&(STR_WIDE|STR_UTF8)))		// ANSI codiert
		{
		while(iSize<iLen)
			{
			if(iMax<=2)break;

			uSign = pPtr[iSize];
			iSize++;

			if(uSign>=32 && uSign!='\"' && uSign!='\\')
				{
				pStr[0] = (unsigned short)uSign;
				pStr++;
				iMax--;
				continue;
				}

			if(iMax<=4)break;				// Zuwenig freier Speicher ?

 			pStr[0]='\\';
			pStr++;
			iMax--;

			if(!(iMode&STR_HEXMODE))
				{
				if(uSign=='\\'){*pStr='\\';iMax--;pStr++;continue;}
				if(uSign=='\"'){*pStr='\"';iMax--;pStr++;continue;}
				if(uSign=='\r'){*pStr='r' ;iMax--;pStr++;continue;}
				if(uSign=='\n'){*pStr='n' ;iMax--;pStr++;continue;}
				}

			if(!(iMode&(STR_HEXMODE|STR_REGMODE)))
				{
				if(uSign=='\''){*pStr='\'';iMax--;pStr++;continue;}
				if(uSign=='\t'){*pStr='t' ;iMax--;pStr++;continue;}
				if(uSign=='\a'){*pStr='a' ;iMax--;pStr++;continue;}
				if(uSign=='\b'){*pStr='b' ;iMax--;pStr++;continue;}
				if(uSign=='\f'){*pStr='f' ;iMax--;pStr++;continue;}
				if(uSign=='\v'){*pStr='v' ;iMax--;pStr++;continue;}
				}

			if(iMax<=6)break;

			if(iMode&STR_CCPSPLIT)			// Codierung "\x1234""abc"
				{
				pStr[0]  = 'x' ;
				pStr[1]  = cHexDigits[uSign>>  4];
				pStr[2]  = cHexDigits[uSign&0x0F];
				pStr    += 3;
				iMax    -= 3;

				if(iSize<iLen && strwchara(pPtr[iSize],0x00040001))
					{
					if(iMax<=7)break;

					pStr[0]  = '"' ;		// "" anhngen wenn weiteres HexDigit
					pStr[1]  = '"' ;
					pStr    += 2;
					iMax    -= 2;
					}

				continue;
				}

			pStr[0]  = 'x' ;
			pStr[1]  = '0' ;
			pStr[2]  = cHexDigits[uSign>>  4];
			pStr[3]  = cHexDigits[uSign&0x0F];
			pStr    += 4;
			iMax    -= 4;
			}
		}
	else if(iMode&STR_UTF8)					// UTF8 codiert
		{
		while(iSize<iLen)
			{
			if(iMax<=2)break;

			iSize += utf8tochar((char*)pPtr+iSize,uSign);

			if(uSign>=32 && uSign!='\"' && uSign!='\\')
				{
				pStr[0]=(unsigned short)uSign;
				pStr++;
				iMax--;
				continue;
				}

			if(iMax<=4)break;				// Zuwenig freier Speicher ?

 			pStr[0] ='\\';
			pStr++;
			iMax--;

			if(!(iMode&STR_HEXMODE))
				{
				if(uSign=='\\'){*pStr='\\';iMax--;pStr++;continue;}
				if(uSign=='\"'){*pStr='\"';iMax--;pStr++;continue;}
				if(uSign=='\r'){*pStr='r' ;iMax--;pStr++;continue;}
				if(uSign=='\n'){*pStr='n' ;iMax--;pStr++;continue;}
				}

			if(!(iMode&(STR_HEXMODE|STR_REGMODE)))
				{
				if(uSign=='\''){*pStr='\'';iMax--;pStr++;continue;}
				if(uSign=='\t'){*pStr='t' ;iMax--;pStr++;continue;}
				if(uSign=='\a'){*pStr='a' ;iMax--;pStr++;continue;}
				if(uSign=='\b'){*pStr='b' ;iMax--;pStr++;continue;}
				if(uSign=='\f'){*pStr='f' ;iMax--;pStr++;continue;}
				if(uSign=='\v'){*pStr='v' ;iMax--;pStr++;continue;}
				}

			if(iMax<=6)break;

			if(iMode&STR_CCPSPLIT)			// Codierung "\x1234""abc"
				{
				pStr[0]  = 'x' ;
				pStr[1]  = cHexDigits[uSign>>  4];
				pStr[2]  = cHexDigits[uSign&0x0F];
				pStr    += 3;
				iMax    -= 3;

				if(iSize<iLen && strwchara(pPtr[iSize],0x00040001))
					{
					if(iMax<=7)break;

					pStr[0]  = '"' ;		// "" anhngen wenn weiteres HexDigit
					pStr[1]  = '"' ;
					pStr    += 2;
					iMax    -= 2;
					}

				continue;
				}

			pStr[0]  = 'x' ;
			pStr[1]  = '0' ;
			pStr[2]  = cHexDigits[uSign>>  4];
			pStr[3]  = cHexDigits[uSign&0x0F];
			pStr    +=4;
			iMax    -=4;
			}
		}
	else if(iMode&STR_WIDE)					// UTF16 codiert
		{
		iLen &= ~1;

		while(iSize<iLen)
			{
			if(iMax<=2)break;

			uSign  = *(unsigned short*)(pPtr+iSize);
			iSize += 2;

			if(uSign>=32 && uSign!='\"' && uSign!='\\')
				{
				pStr[0]=(unsigned short)uSign;
				pStr++;
				iMax--;
				continue;
				}

			if(iMax<=4)break;				// Zuwenig freier Speicher ?

 			pStr[0] ='\\';
			pStr++;
			iMax--;

			if(!(iMode&STR_HEXMODE))
				{
				if(uSign=='\\'){*pStr='\\';iMax--;pStr++;continue;}
				if(uSign=='\"'){*pStr='\"';iMax--;pStr++;continue;}
				if(uSign=='\r'){*pStr='r' ;iMax--;pStr++;continue;}
				if(uSign=='\n'){*pStr='n' ;iMax--;pStr++;continue;}
				}

			if(!(iMode&(STR_HEXMODE|STR_REGMODE)))
				{
				if(uSign=='\''){*pStr='\'';iMax--;pStr++;continue;}
				if(uSign=='\t'){*pStr='t' ;iMax--;pStr++;continue;}
				if(uSign=='\a'){*pStr='a' ;iMax--;pStr++;continue;}
				if(uSign=='\b'){*pStr='b' ;iMax--;pStr++;continue;}
				if(uSign=='\f'){*pStr='f' ;iMax--;pStr++;continue;}
				if(uSign=='\v'){*pStr='v' ;iMax--;pStr++;continue;}
				}

			if(iMax<=6)break;

			if(iMode&STR_CCPSPLIT)			// Codierung "\x1234""abc"
				{
				pStr[0]  = 'x' ;
				pStr[1]  = cHexDigits[uSign>>  4];
				pStr[2]  = cHexDigits[uSign&0x0F];
				pStr    += 3;
				iMax    -= 3;

				if(iSize<iLen && strwchara(pPtr[iSize],0x00040001))
					{
					if(iMax<=7)break;

					pStr[0]  = '"' ;		// "" anhngen wenn weiteres HexDigit
					pStr[1]  = '"' ;
					pStr    += 2;
					iMax    -= 2;
					}

				continue;
				}

			pStr[0]  = 'x' ;
			pStr[1]  = '0' ;
			pStr[2]  = cHexDigits[uSign>>  4];
			pStr[3]  = cHexDigits[uSign&0x0F];
			pStr    +=4;
			iMax    -=4;
			}
		}

	if(iMode&STR_NOQUOTE)					// Am Ende kein Anfhrungszeichen anhngen
		{
		iMax-=2;
		}
	else{
		if(iMax>=2)							// Am Ende ein Anfhrungszeichen anhngen
			{
			pStr[0]='\"';
			pStr++;
			iMax--;
			}
		}


	if(iMax>0)*pStr=0;



return iFull-iMax;
}

//*****************************************************************************
//*
//*		strlcat
//*
//*****************************************************************************
//	Hngt an eine Zeichenfolge einen Text an.
//	Ein Nullzeichen wird dabei immer angehngt.
//	pString		: Zeiger auf die Zeichenfolge an die der Text angehngt wird.
//	pText		: Zeiger auf den Text der angehngt werden soll.
//	iMax		: Ist die maximale Lnge des Zeichenfolgenpuffers mit Nullzeichen.
//	Ergibt die neue Lnge der Zeichenfolge ohne Nullzeichen. (in Zeichen)
int	_cdecl strlcat(char *pString,const char *pText,int iMax)
{
int		iSize;

		iSize=strnlen(pString,iMax);
	if(	iSize>=iMax)return iSize;


return iSize+=strlcpy(pString+iSize,pText,iMax-iSize);
}

int	_cdecl strlcat(wchar_t *pString,const wchar_t *pText ,int iMax)
{
int		iSize;

		iSize=strnlen(pString,iMax);
	if(	iSize>=iMax)return iSize;


return iSize+=strlcpy(pString+iSize,pText,iMax-iSize);
}

//*****************************************************************************
//*
//*		strlcat
//*
//*****************************************************************************
//	Hngt an eine Zeichenfolge einen Text an, mit der Angabe der Textlnge.
//	Ein Nullzeichen wird dabei immer angehngt.
//	pString		: Zeiger auf die Zeichenfolge an die der Text angehngt wird.
//	pText		: Zeiger auf den Text der angehngt werden soll.
//	iMax		: Ist die maximale Lnge der Zeichenfolge mit Nullzeichen.
//	iTextLen	: Ist die maximale Lnge der Textes der angehngt wird.
//	Ergibt die neue Lnge der Zeichenfolge ohne Nullzeichen. (in Zeichen)
int	_cdecl strlcat (char *pString,const char *pText,int iMax,int iTextLen)
{
int		iSize;

		iSize=strnlen(pString,iMax);
	if(	iSize>=iMax)return iSize;


return iSize+=strlcpy(pString+iSize,pText,iMax-iSize,iTextLen);
}

int	_cdecl strlcat (wchar_t *pString,const wchar_t *pText ,int iMax,int iTextLen)
{
int		iSize;

		iSize=strnlen(pString,iMax);
	if(	iSize>=iMax)return iSize;


return iSize+=strlcpy(pString+iSize,pText,iMax-iSize,iTextLen);
}

//*****************************************************************************
//*
//*		strnlen
//*
//*****************************************************************************
//	Berechnet die Lnge von einer Zeichenfolge bis zu einer maximalen Position.
//	pString		: Zeiger auf die Zeichenfolge die verwendet wird.
//	iMax		: Ist die maximale Position biz zu der gezhlt wird.
//	Ergibt die Lnge der Zeichenfolge ohne Nullzeichen.
int	_cdecl strnlen(const char *pString,int iMax)
{
#ifdef 	NO_STR_ASM

register const char *pPtr;
register int		 iLen;
register unsigned 	 u;


	if(iMax<=0)return 0;

	iLen = iMax;
	pPtr = pString;

	for(;(int)pPtr&3;pPtr++,iLen--)				// Auf gerade Addresse
		{
		if(! iLen)return iMax;
		if(!*pPtr)return (int)(pPtr-pString);
		}

	for(;iLen>0;pPtr+=4,iLen-=4)				// Suche Textende
		{
		u = *(unsigned*)pPtr;
		u = u^~(0x7EFEFEFF+u);

		if(!(u&0x81010100))continue;

		u=*(unsigned*)pPtr;
		if(!(u&0x000000FF))return                 (int)(pPtr-pString);
		if(!(u&0x0000FF00))return (iLen<=1)? iMax:(int)(pPtr-pString+1);
		if(!(u&0x00FF0000))return (iLen<=2)? iMax:(int)(pPtr-pString+2);
		if(!(u&0xFF000000))return (iLen<=3)? iMax:(int)(pPtr-pString+3);
		}

return iMax;

//*****************************************************************************
#else
_asm	{
		mov		ecx,iMax				;// ECX = Restlnge
		cmp		ecx,0;					;// Ist die Maximallnge <= 0 ?
		jle		e_0						;//		0 zurckgeben

		mov		ebx,pString				;// EBX = Zeiger auf pString

		test	ebx,3					;//	if((char)pPtr&3)	Auf gerade Addresse
		jz		n_1						;//			*
s_1:	cmp		byte ptr [ebx],0		;// if(!*pPtr)return pPtr-pString
		je		e_3						;//			*
		dec		ecx						;// iLen --
		jecxz	e_2						;// if(!iLen)return iMax
		inc		ebx						;// pPtr ++
		test	ebx,3					;//	while((char)pPtr&3)
		jnz		s_1						;//			*

n_1:	mov		esi,7EFEFEFFh			;// ESI ist eine Konstante
		mov		edi,81010100h			;// EDI ist eine Konstante

		jecxz	e_2						;// if(!iLen)return iMax
s_2:	mov		eax,[ebx]				;// u=*(unsigned*)pPtr;
		mov		edx,esi					;//	u= u^~(0x7EFEFEFF+u);
		add		edx,eax					;//			*
		not		edx						;//			*
		xor		eax,edx					;//			*
		and		eax,edi					;//	if(u & 0x81010100)
		jnz		e_1						;//			*
		add		ebx,4					;// pPtr += 4
		sub		ecx,4					;// iLen -= 4
		jg		s_2						;// while(iLen>0)

e_2:	mov		eax,iMax				;// iMax zurckgeben
		jmp		ende					;//			*

e_0:	xor		eax,eax					;// Null zurckgeben
		jmp		ende					;//			*

e_1:	mov		eax,[ebx]				;// u=*(unsigned*)pPtr;
		test	al,al					;// if(!(u&0x000000FF))return pPtr-pString;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iLen)return iMax
		inc		ebx						;//			*
		test	ah,ah					;// if(!(u&0x0000FF00))return pPtr-pString+1;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iLen)return iMax
		inc		ebx						;//			*
		test	eax,00FF0000h			;// if(!(u&0x00FF0000))return pPtr-pString+2;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iLen)return iMax
		inc		ebx						;//			*
		test	eax,0FF000000h			;// if(!(u&0xFF000000))return pPtr-pString+3;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iLen)return iMax
		inc		ebx						;//			*
		jmp		s_2						;// while(iLen>0)

e_3:	mov		eax,ebx					;// return pPtr-pString
		sub		eax,pString				;//			*

ende:
		}

#endif
}

int	_cdecl strnlen(const wchar_t *pString,int iMax)
{
#ifdef 	NO_STR_ASM

register const wchar_t  *pPtr;
register int					iLen;
register unsigned 				u;


	if(iMax<=0)return 0;

	iLen = iMax;
	pPtr = pString;

	if((int)pPtr&2)								// Auf gerade Addresse
		{
		if(! iLen)return iMax;
		if(!*pPtr)return (int)(pPtr-pString);

		pPtr++;
		iLen-- ;
		}

	for(;iLen>0;pPtr+=2,iLen-=2)				// Suche Textende
		{
		u=*(unsigned*)pPtr;
		if(!(u&0x0000FFFF))return                 (int)(pPtr-pString);
		if(!(u&0xFFFF0000))return (iLen<=1)? iMax:(int)(pPtr-pString+1);
		}

return iMax;

//*****************************************************************************
#else
_asm	{
		mov		ecx,iMax				;// ECX = Restlnge
		cmp		ecx,0;					;// Ist die Maximallnge <= 0 ?
		jle		e_0						;//		0 zurckgeben

		mov		ebx,pString				;// EBX = Zeiger auf pString

		test	ebx,2					;//	if((unsigned)pPtr&2)	Auf gerade Addresse
		jz		n_1						;//			*
		cmp		word ptr [ebx],0		;// if(!*pPtr)return pPtr-pString
		je		e_3						;//			*
		dec		ecx						;// iLen --
		jecxz	e_1						;// if(!iLen)return iMax
		add		ebx,2					;// pPtr ++

n_1:	mov     esi,0FFFF0000h;			;// Konstante

		jecxz	e_1						;// if(!iLen)return iMax
s_2:	mov		eax,[ebx]				;// u=*(unsigned*)pPtr;
		test    ax,ax					;// if(!(u&0x0000FFFF))return pPtr-pString;
		jz		e_3						;//			*
		test    eax,esi					;// if(!(u&0xFFFF0000))return pPtr-pString;
		jz		e_2						;//			*
		add		ebx,4					;// pPtr += 4
		sub		ecx,2					;// iLen -= 4
		jg		s_2						;// while(iLen>0)

e_1:	mov		eax,iMax				;// iMax zurckgeben
		jmp		ende					;//			*
e_0:	xor		eax,eax					;// Null zurckgeben
		jmp		ende					;//			*
e_2:	dec		ecx						;// iLen --
		jecxz	e_1						;// if(!iLen)return iMax
		add		ebx,2					;//	pPtr ++
e_3:	mov		eax,ebx					;// return pPtr-pString
		sub		eax,pString				;//			*
		shr     eax,1					;//			*
ende:
	}

#endif
}

//*****************************************************************************
//*
//*		strlcpy
//*
//*****************************************************************************
//	Kopiert einen Text mit Lngenbeschrnkung.
//	Ein Nullzeichen wird dabei immer angehngt.
//	pString		: Ist der Zielpuffer in dem der Text gespeichert wird.
//	pFrom		: Ist ein Zeiger auf den Text der kopiert werden soll.
//	iLen		: Ist die maximale Gre der Zielpuffers mit Nullzeichen.
//	Ergibt die Lnge des kopierten Textes im Zielpuffer ohne Nullzeichen.
int	_cdecl strlcpy(char *pString,const char *pFrom,int iMax)
{
#ifdef 	NO_STR_ASM

register char       *pCpy;
register const char *pPtr;
register int		 iLen;
register unsigned 	 u,v;


	if(iMax<=0)return 0;						// Kein Speicher frei

	iLen = iMax;
	pPtr = pFrom;
	pCpy = pString;


	while((int)pCpy&3)							// Auf gerade Zieladdresse
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])return (int)(pPtr-pFrom);

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}

	for(;iLen>4;pPtr+=4,pCpy+=4,iLen-=4)		// 4-Byte Blcke kopieren
		{
		u=GET_STR_DWORD(pPtr);
		*(unsigned*)pCpy=u;

		v= u^~(0x7EFEFEFF+u);
		if(v & 0x81010100)						// War ein Null-Byte im Buffer
			{
			if(!(u&0x000000FF))return                 (int)(pPtr-pFrom);
			if(!(u&0x0000FF00))return (iLen<=1)? iMax:(int)(pPtr-pFrom+1);
			if(!(u&0x00FF0000))return (iLen<=2)? iMax:(int)(pPtr-pFrom+2);
			if(!(u&0xFF000000))return (iLen<=3)? iMax:(int)(pPtr-pFrom+3);
			}
		}

	for(;;)										// 1-Byte Blcke kopieren
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])break;

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}


return (int)(pPtr-pFrom);

//*****************************************************************************
#else

_asm	{
		mov		ecx,iMax				;// ECX = Restlnge
		cmp		ecx,0;					;// Ist die Maximallnge <= 0 ?
		jle		e_0						;//		0 zurckgeben

		mov		ebx,pString				;// EBX = Zeiger auf pString
		mov		esi,pFrom				;// ESI = Zeiger auf pFrom

		test	bl,3					;// if((char)pCpy&3) Auf gerade Zieladdresse
		jz		n_1						;//			*
s_1:	mov		al,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],al				;//			*
		cmp		al,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		inc		ebx						;// pCpy++
		inc		esi						;// pPtr++
		test	bl,3					;// while((char)pCpy&3)
		jnz		s_1						;//			*

n_1:	mov		edi,81010100h			;// EDI ist eine Konstante
s_2:	cmp		ecx,4					;// while(iLen>4)
		jle		s_3						;//			*
		mov		eax,[esi]				;//	u=*(unsigned*)pPtr
		mov		[ebx],eax				;//   *(unsigned*)pCpy=u;
		lea		edx,[eax+7EFEFEFFh]		;//	u= u^~(0x7EFEFEFF+u);
		not		edx						;//			*
		xor		eax,edx					;//			*
		and		eax,edi					;//	if(u & 0x81010100)
		jnz		e_1						;//			*
		add		ebx,4					;// pCpy  += 4
		add		esi,4					;// pPtr  += 4
		sub		ecx,4					;// iLen  -= 4
		jmp		s_2						;//			*

s_3:	mov		al,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],al				;//			*
		cmp		al,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		inc		ebx						;// pCpy++
		inc		esi						;// pPtr++
		jmp		s_3						;//			*

e_0:	xor		eax,eax					;// Null zurckgeben
		jmp		ende					;//			*

e_2:	mov		byte ptr [ebx],0		;// iMax zurckgeben
		jmp		e_3

e_1:	mov		eax,[esi]				;// u=*(unsigned*)pPtr;
		test	al,al					;// if(!(u&0x000000FF))return pPtr-pString;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		test	ah,ah					;// if(!(u&0x0000FF00))return pPtr-pString+1;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		test	eax,00FF0000h			;// if(!(u&0x00FF0000))return pPtr-pString+2;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		test	eax,0FF000000h			;// if(!(u&0xFF000000))return pPtr-pString+3;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		add		ebx,4					;//	pCpy  += 4
		jmp		s_2						;// while(iMax>0)

e_3:	mov		eax,esi					;// return pPtr-pString
		sub		eax,pFrom				;//			*
ende:	}

#endif
}

int	_cdecl strlcpy(wchar_t *pString,const wchar_t *pFrom,int iMax)
{
#ifdef 	NO_STR_ASM

register unsigned short *pCpy;
register const wchar_t  *pPtr;
register int			 iLen;
register unsigned 	   	 u;


	if(iMax<=0)return 0;						// Kein Speicher frei

	iLen = iMax;
	pPtr = pFrom;
	pCpy = (unsigned short*)pString;


	if((int)pCpy&2)								// Auf gerade Zieladdresse
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])return (int)(pPtr-pFrom);

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}

	for(;iLen>2;pPtr+=2,pCpy+=2,iLen-=2)		// 4-Byte Blcke kopieren
		{
		u=*(unsigned*)pPtr;
		  *(unsigned*)pCpy=u;

		if(!(u&0x0000FFFF))return                 (int)(pPtr-pFrom);
		if(!(u&0xFFFF0000))return (iLen<=1)? iMax:(int)(pPtr-pFrom+1);
		}

	for(;;)										// 1-Byte Blcke kopieren
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])break;

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}


return (int)(pPtr-pFrom);

//*****************************************************************************
#else

_asm	{
		mov		ecx,iMax				;// ECX = Restlnge
		cmp		ecx,0;					;// Ist die Maximallnge <= 0 ?
		jle		e_0						;//		0 zurckgeben

		mov		ebx,pString				;// EBX = Zeiger auf pString
		mov		esi,pFrom				;// ESI = Zeiger auf pFrom

		test	bl,2					;// if((char)pCpy&2) Auf gerade Zieladdresse
		jz		n_1						;//			*
		mov		ax,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],ax				;//			*
		cmp		ax,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		add		ebx,2					;// pCpy++
		add		esi,2					;// pPtr++

n_1:	mov		edi,0FFFF0000h			;// EDI ist eine Konstante
		sub		ecx,2					;// while(iLen>2)
		jle		s_3						;//			*
s_2:	mov		eax,[esi]				;//	u=*(unsigned*)pPtr
		mov		[ebx],eax				;//   *(unsigned*)pCpy=u;

		test    ax,ax					;// if(!(u&0x0000FFFF))return pPtr-pString;
		jz		e_3						;//			*
		add		esi,2					;// pPtr += 1
		test    eax,edi					;// if(!(u&0xFFFF0000))return pPtr-pString;
		jz		e_3						;//			*
		add		ebx,4					;// pCpy += 2
		add		esi,2					;// pPtr += 1
		sub		ecx,2					;// iLen -= 2
		jg		s_2						;//	while(iLen>=2)

s_3:	add		ecx,2					;//			*
s_4:	mov		ax,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],ax				;//			*
		cmp		ax,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		add		ebx,2					;// pCpy++
		add		esi,2					;// pPtr++
		jmp		s_4						;//			*

e_0:	xor		eax,eax					;// Null zurckgeben
		jmp		ende					;//			*

e_2:	mov		word ptr [ebx],0		;// iMax zurckgeben
e_3:	mov		eax,esi					;// return pPtr-pString
		sub		eax,pFrom				;//			*
		shr		eax,1					;//			*
ende:	}

#endif
}

//*****************************************************************************
//*
//*		strlcpy
//*
//*****************************************************************************
//	Kopiert einen Text mit Lngenbeschrnkung fr Quelle und Ziel.
//	Ein Nullzeichen wird dabei immer angehngt.
//	pString		: Ist der Zielpuffer in dem der Text gespeichert wird.
//	pFrom		: Ist ein Zeiger auf den Text der kopiert werden soll.
//	iMax		: Ist die maximale Gre der Zielpuffers mit Nullzeichen.
//	iSize		: Ist die maximale Lnge des Textes. Ist der Text lnger
//				  als iSize wird er nur bis zu iSize kopiert.
//	Ergibt die Lnge des kopierten Textes im Zielpuffer ohne Nullzeichen.
int	_cdecl strlcpy(char *pString,const char *pFrom,int iMax,int iSize)
{
#ifdef 	NO_STR_ASM

register char       *pCpy;
register const char *pPtr;
register int		 iLen;
register unsigned 	 u,v;


	if(iMax<=0)return 0;						// Kein Speicher frei
	if(iMax>iSize+1)iMax=iSize+1;

	iLen = iMax;
	pPtr = pFrom;
	pCpy = pString;


	while((int)pCpy&3)							// Auf gerade Zieladdresse
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])return (int)(pPtr-pFrom);

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}

	for(;iLen>4;pPtr+=4,pCpy+=4,iLen-=4)		// 4-Byte Blcke kopieren
		{
		u=GET_STR_DWORD(pPtr);
		*(unsigned*)pCpy=u;

		v= u^~(0x7EFEFEFF+u);
		if(v & 0x81010100)						// War ein Null-Byte im Puffer
			{
			if(!(u&0x000000FF))return                 (int)(pPtr-pFrom);
			if(!(u&0x0000FF00))return (iLen<=1)? iMax:(int)(pPtr-pFrom+1);
			if(!(u&0x00FF0000))return (iLen<=2)? iMax:(int)(pPtr-pFrom+2);
			if(!(u&0xFF000000))return (iLen<=3)? iMax:(int)(pPtr-pFrom+3);
			}
		}

	for(;;)										// 1-Byte Blcke kopieren
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])return (int)(pPtr-pFrom);

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}


return (int)(pPtr-pString);

//*****************************************************************************
#else

_asm	{
		mov		ecx,iMax				;// ECX = Restlnge
		mov		eax,iSize				;// EAX = Quellenlnge
		cmp		ecx,0;					;// Ist die Maximallnge <= 0 ?
		jle		e_0						;//		0 zurckgeben
		inc		eax						;// Quellenlnge kleiner ?
		cmp		eax,ecx					;//			*
		jge		s_0						;//			*
		mov		ecx,eax					;//			*

s_0:	mov		ebx,pString				;// EBX = Zeiger auf pString
		mov		esi,pFrom				;// ESI = Zeiger auf pFrom

		test	bl,3					;// if((char)pCpy&3) Auf gerade Zieladdresse
		jz		n_1						;//			*
s_1:	mov		al,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],al				;//			*
		cmp		al,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		inc		ebx						;// pCpy++
		inc		esi						;// pPtr++
		test	bl,3					;// while((char)pCpy&3)
		jnz		s_1						;//			*

n_1:	mov		edi,81010100h			;// EDI ist eine Konstante
s_2:	cmp		ecx,4					;// while(iLen>4)
		jle		s_3						;//			*
		mov		eax,[esi]				;//	u=*(unsigned*)pPtr
		mov		[ebx],eax				;//   *(unsigned*)pCpy=u;
		lea		edx,[eax+7EFEFEFFh]		;//	u= u^~(0x7EFEFEFF+u);
		not		edx						;//			*
		xor		eax,edx					;//			*
		and		eax,edi					;//	if(u & 0x81010100)
		jnz		e_1						;//			*
		add		ebx,4					;// pCpy  += 4
		add		esi,4					;// pPtr  += 4
		sub		ecx,4					;// iLen  -= 4
		jmp		s_2						;//			*

s_3:	mov		al,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],al				;//			*
		cmp		al,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		inc		ebx						;// pCpy++
		inc		esi						;// pPtr++
		jmp		s_3						;//			*

e_0:	xor		eax,eax					;// Null zurckgeben
		jmp		ende					;//			*

e_2:	mov		byte ptr [ebx],0		;// iMax zurckgeben
		jmp		e_3

e_1:	mov		eax,[esi]				;// u=*(unsigned*)pPtr;
		test	al,al					;// if(!(u&0x000000FF))return pPtr-pString;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		test	ah,ah					;// if(!(u&0x0000FF00))return pPtr-pString+1;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		test	eax,00FF0000h			;// if(!(u&0x00FF0000))return pPtr-pString+2;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		test	eax,0FF000000h			;// if(!(u&0xFF000000))return pPtr-pString+3;
		jz		e_3						;//			*
		dec		ecx						;//			*
		jecxz	e_2						;// if(!iMax)return iMax
		inc		esi						;//			*
		add		ebx,4					;//	pCpy  += 4
		jmp		s_2						;// while(iMax>0)

e_3:	mov		eax,esi					;// return pPtr-pString
		sub		eax,pFrom				;//			*
ende:	}

#endif
}

int	_cdecl strlcpy(wchar_t *pString,const wchar_t *pFrom,int iMax,int iSize)
{
#ifdef 	NO_STR_ASM

register unsigned short        *pCpy;
register const wchar_t  *pPtr;
register int					iLen;
register unsigned 				u;


	if(iMax<=0)return 0;						// Kein Speicher frei
	if(iMax>iSize+1)iMax=iSize+1;

	iLen = iMax;
	pPtr = pFrom;
	pCpy = (unsigned short*)pString;


	if((int)pCpy&2)								// Auf gerade Zieladdresse
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])return (int)(pPtr-pFrom);

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}

	for(;iLen>2;pPtr+=2,pCpy+=2,iLen-=2)		// 4-Byte Blcke kopieren
		{
		u=*(unsigned*)pPtr;
		  *(unsigned*)pCpy=u;

		if(!(u&0x0000FFFF))return                 (int)(pPtr-pFrom);
		if(!(u&0xFFFF0000))return (iLen<=1)? iMax:(int)(pPtr-pFrom+1);
		}

	for(;;)										// 1-Byte Blcke kopieren
		{
			pCpy[0]=pPtr[0];
		if(!pPtr[0])return (int)(pPtr-pFrom);

			iLen--;
		if(!iLen)
			{
			pCpy[0]=0;
			return (int)(pPtr-pFrom);
			}

		pCpy++;
		pPtr++;
		}


return (int)(pPtr-pString);

//*****************************************************************************
#else

_asm	{
		mov		ecx,iMax				;// ECX = Restlnge
		mov		eax,iSize				;// EAX = Quellenlnge
		cmp		ecx,0;					;// Ist die Maximallnge <= 0 ?
		jle		e_0						;//		0 zurckgeben
		inc		eax						;// Quellenlnge kleiner ?
		cmp		eax,ecx					;//			*
		jge		s_0						;//			*
		mov		ecx,eax					;//			*

s_0:	mov		ebx,pString				;// EBX = Zeiger auf pString
		mov		esi,pFrom				;// ESI = Zeiger auf pFrom

		test	bl,2					;// if((char)pCpy&2) Auf gerade Zieladdresse
		jz		n_1						;//			*
		mov		ax,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],ax				;//			*
		cmp		ax,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		add		ebx,2					;// pCpy++
		add		esi,2					;// pPtr++

n_1:	mov		edi,0FFFF0000h			;// EDI ist eine Konstante
		sub		ecx,2					;// while(iLen>2)
		jle		s_3						;//			*
s_2:	mov		eax,[esi]				;//	u=*(unsigned*)pPtr
		mov		[ebx],eax				;//   *(unsigned*)pCpy=u;

		test    ax,ax					;// if(!(u&0x0000FFFF))return pPtr-pString;
		jz		e_3						;//			*
		add		esi,2					;// pPtr += 1
		test    eax,edi					;// if(!(u&0xFFFF0000))return pPtr-pString;
		jz		e_3						;//			*
		add		ebx,4					;// pCpy += 2
		add		esi,2					;// pPtr += 1
		sub		ecx,2					;// iLen -= 2
		jg		s_2						;//	while(iLen>=2)

s_3:	add		ecx,2					;//			*
s_4:	mov		ax,[esi]				;// pCpy[0] = pPtr[0]
		mov		[ebx],ax				;//			*
		cmp		ax,0					;// if(!pPtr[0])return pPtr-pFrom;
		je		e_3						;//			*
		dec		ecx						;// iLen--;
		jecxz	e_2						;//	if(iLen==0) ... {pCpy[0]=0;return pPtr-pFrom;}
		add		ebx,2					;// pCpy++
		add		esi,2					;// pPtr++
		jmp		s_4						;//			*

e_0:	xor		eax,eax					;// Null zurckgeben
		jmp		ende					;//			*

e_2:	mov		word ptr [ebx],0		;// iMax zurckgeben
e_3:	mov		eax,esi					;// return pPtr-pString
		sub		eax,pFrom				;//			*
		shr		eax,1					;//			*
ende:	}

#endif
}

//*****************************************************************************
//*
//*		strtoc
//*
//*****************************************************************************
//	Wandelt	eine Text in ein Zeichen um. Dabei werden auch Escape-Folgen
//	bercksichtigt.   z.B. "/t" wird zu 9
//	pStr	: Ist der Zeicher auf den Text
//	pCount	: Hier wird die Anzahl der verarbeiteten Zeichen aus dem Text
//			  gespeichert. Wenn der Zeiger Null ist wird nichts gespeichert.
//	Ergibt das den Wert des Zeichens, ohne negative Erweiterung.
int	_cdecl strtoc(const char *pStr,int *pCount)
{
unsigned char	ucValue;
unsigned char	ucChar;
int				i;


	ucChar=*pStr;

	if(ucChar=='\\')								// Escape-Folge vorhanden
		{
		ucChar = pStr[1];

		if(ucChar>=0x80 || aStrEscapeTab[ucChar]==0)
			{
			if(pCount)*pCount=1;
			return '\\';
			}

		   ucChar = aStrEscapeTab[ucChar];
		if(ucChar<0xFE)
			{
			if(pCount)*pCount=2;
			return ucChar;
			}

		if(ucChar==0xFF)							// Hex-Zahl
			{
			for(ucValue=0,i=2;i<5;i++)
				{
				   ucChar = (char)strcharval(pStr[i]);
				if(ucChar>=16)break;				// Falsches Zeichen

				ucValue  = (char)(ucValue*16+ucChar);
				}

			if(i>=2)
				{
				if(pCount)*pCount=i;
				return ucValue;
				}
			}
		else{										// Oktal Zahl
			for(ucValue=0,i=1;i<4;i++)
				{
				   ucChar=pStr[i];
				if(ucChar<'0')break;
				if(ucChar>'7')break;

				ucValue<<=3;						// Zahl berechnen
				ucValue=(char)(ucValue+ucChar-'0');
				}

			if(i>1)
				{
				if(pCount)*pCount=i;
				return ucValue;
				}
			}

		if(pCount)*pCount=1;

		return '\\';
		}

	if(pCount)*pCount=1;


return ucChar;
}

int	_cdecl strtoc(const wchar_t *pStr,int *pCount)
{
unsigned short	usValue;
unsigned short 	usChar;
int				i;



	usChar=*pStr;

	if(usChar=='\\')								// Escape-Folge vorhanden
		{
		usChar = pStr[1];

		if(usChar>=0x80 || aStrEscapeTab[usChar]==0)
			{
			if(pCount)*pCount=1;
			return '\\';
			}

		   usChar = aStrEscapeTab[usChar];
		if(usChar<0xFE)								// Escape Folge
			{
			if(pCount)*pCount=2;
			return usChar;
			}

		if(usChar==0xFF)							// Hex-Zahl
			{
			for(usValue=0,i=2;i<5;i++)
				{
				   usChar = (short)strcharval(pStr[i]);
				if(usChar>=16)break;				// Falsches Zeichen

				usValue = (short)(usValue*16+usChar);
				}

			if(i>=2)
				{
				if(pCount)*pCount=i;
				return usValue;
				}
			}
		else{										// Oktal Zahl
			for(usValue=0,i=1;i<4;i++)
				{
				   usChar=pStr[i];
				if(usChar<'0')break;
				if(usChar>'7')break;

				usValue<<=3;						// Zahl berechnen
				usValue=(char)(usValue+usChar-'0');
				}

			if(i>1)
				{
				if(pCount)*pCount=i;
				return usValue;
				}
			}

		if(pCount)*pCount=1;

		return '\\';
		}

	if(pCount)*pCount=1;


return usChar;
}

//****************************************************************************
//*
//*		strtoz
//*
//****************************************************************************
//	Wandelt einen Text in eine Zahl um.
//	pValue	: Hier wird die Zahl gespeichert
//	pStr	: Ist der Zeiger auf den Text
//	iBase	: Ist die Basis der Zahl
//	iLen	: Ist die maximale Lnge des Textes
//	iSize	: Ist die Anzahl der zu speichernden Bytes fr die Zahl (maximal 8)
//	Ergibt die Anzahl der gescannten Zeichen
int _cdecl strtoz(int *pValue,const char *pStr,int iBase,int iLen,int iSize)
{
unsigned __int64	uVal;
int					i,iPos;


	uVal = 0;

	for(iPos=0;iPos<iLen;iPos++)			// Zahl einlesen
		{
		   i = strcharval(pStr[iPos]);
		if(i>=iBase)break;					// Falsches Zeichen

		uVal *= iBase;						// Zahl berechnen
		uVal += i;
		}

	memcpy(pValue,&uVal,iSize);



return iPos;
}

int _cdecl strtoz(int *pValue,const wchar_t *pStr,int iBase,int iLen,int iSize)
{
unsigned __int64	uVal;
int					i,iPos;


	uVal = 0;

	for(iPos=0;iPos<iLen;iPos++)			// Zahl einlesen
		{
		   i = strcharval(pStr[iPos]);
		if(i>=iBase)break;					// Falsches Zeichen

		uVal *= iBase;						// Zahl berechnen
		uVal += i;
		}

	memcpy(pValue,&uVal,iSize);



return iPos;
}

//****************************************************************************
//*
//*		strzlen
//*
//****************************************************************************
//	Berechnet die Lnge einer Texteszahl mit mit einer variablen Basis.
//	pStr	: Ist der Zeiger auf den Text mit der Zahl
//	iBase	: Ist die Basis der Zahl. Wenn die Basis kleiner Null ist
//			  werden Minuszeichen am Anfang mitgezhlt.
//	iLen	: Ist die maximale Lnge des Textes
//	Ergibt die Anzahl an Textzeichen die zu der Zahl gehren
int _cdecl strzlen(const char *pStr,int iBase,int iLen)
{
int		iPos=0;
int		i;



	if(iBase<0)								// Minus Zeichen bercksichtigen
		{
		iBase=-iBase;
		if(pStr[iPos]=='-')iPos++;
		}

	while(iPos<iLen)						// Zahl einlesen
		{
		   i = strcharval(pStr[iPos]);
		if(i>=iBase)break;					// Falsches Zeichen

		iPos++;
		}

return iPos;
}

int _cdecl strzlen(const wchar_t *pStr,int iBase,int iLen)
{
int		iPos=0;
int		i;



	if(iBase<0)								// Minus Zeichen bercksichtigen
		{
		iBase=-iBase;
		if(pStr[iPos]=='-')iPos++;
		}

	while(iPos<iLen)						// Zahl einlesen
		{
		   i = strcharval(pStr[iPos]);
		if(i>=iBase)break;					// Falsches Zeichen

		iPos++;
		}

return iPos;
}

//*****************************************************************************
//*
//*		strflen
//*
//*****************************************************************************
//	Berechnet die Lnge einer Fliekommatexteszahl mit vorangehenden Leerzeichen.
//	pStr	: Ist der Zeiger auf den Text mit der Zahl
//	Ergibt die Anzahl an Textzeichen die zur Fliekommazahl gehren.
int _cdecl strflen(const char *pStr)
{
int		iSize;

	for(iSize=0;strwchara(*pStr,0x000000A0); pStr++)iSize++;
	if (        strwchara(*pStr,0x06000000)){pStr++;iSize++;}
	for(       ;strwchara(*pStr,0x00000001); pStr++)iSize++;
	if (        *pStr=='.'                 ){pStr++;iSize++;

		if(*pStr=='#')
			{
			if(!CMP_LETTER(pStr[2],'N'))return iSize;

			if(CMP_LETTER(pStr[1],'I') && (CMP_LETTER(pStr[3],'F') || CMP_LETTER(pStr[3],'D')))
				{
				pStr +=4;
				iSize+=4;
				}
			else if(CMP_LETTER(pStr[1],'Q') && CMP_LETTER(pStr[3],'A') && CMP_LETTER(pStr[4],'N'))
				{
				pStr +=5;
				iSize+=5;
				}
			else{
				return iSize;
				}
			}

	for(       ;strwchara(*pStr,0x00000001); pStr++)iSize++;}
	if (       CMP_LETTER(*pStr,'E')       ){pStr++;iSize++;
	if (        strwchara(*pStr,0x06000000)){pStr++;iSize++;}
	for(       ;strwchara(*pStr,0x00000001); pStr++)iSize++;}


return iSize;
}

int _cdecl strflen(const wchar_t *pStr)
{
int		iSize;

	for(iSize=0;strwcharw(*pStr,0x000000A0); pStr++)iSize++;
	if (        strwcharw(*pStr,0x06000000)){pStr++;iSize++;}
	for(       ;strwcharw(*pStr,0x00000001); pStr++)iSize++;
	if (        *pStr=='.'                 ){pStr++;iSize++;

		if(*pStr=='#')
			{
			if(!CMP_LETTER(pStr[2],'N'))
				{
				return iSize;
				}
			else if(CMP_LETTER(pStr[1],'I') && (CMP_LETTER(pStr[3],'F') || CMP_LETTER(pStr[3],'D')))
				{
				pStr +=4;
				iSize+=4;
				}
			else if(CMP_LETTER(pStr[1],'Q') && CMP_LETTER(pStr[3],'A') && CMP_LETTER(pStr[4],'N'))
				{
				pStr +=5;
				iSize+=5;
				}
			else{
				return iSize;
				}
			}

	for(       ;strwcharw(*pStr,0x00000001); pStr++)iSize++;}
	if (       CMP_LETTER(*pStr,'E')       ){pStr++;iSize++;
	if (        strwcharw(*pStr,0x06000000)){pStr++;iSize++;}
	for(       ;strwcharw(*pStr,0x00000001); pStr++)iSize++;}


return iSize;
}

//*****************************************************************************
//*
//*		strflen
//*
//*****************************************************************************
//	Berechnet die Lnge einer Fliekommazahl mit vorangehenden Leerzeichen.
//	pStr	: Ist der Zeiger auf den Text mit der Zahl
//	iMax	: Ist die maximale Lnge vom Text
//	Ergibt die Anzahl an Textzeichen die zur Fliekommazahl gehren.
int _cdecl strflen(const char *pStr,int iMax)
{
int		iSize;


	if(iMax<=0)return 0;

	for(iSize=0;strwchara(*pStr,0x000000A0); pStr++){iSize++;if(iSize>=iMax)return iSize;}
	if (        strwchara(*pStr,0x06000000)){pStr++; iSize++;if(iSize>=iMax)return iSize;}
	for(       ;strwchara(*pStr,0x00000001); pStr++){iSize++;if(iSize>=iMax)return iSize;}
	if (        *pStr=='.'                 ){pStr++; iSize++;if(iSize>=iMax)return iSize;

		if(*pStr=='#')
			{
			if(iSize+4>iMax || !CMP_LETTER(pStr[2],'N'))
				{
				return iSize;
				}
			else if(CMP_LETTER(pStr[1],'I') && (CMP_LETTER(pStr[3],'F') || CMP_LETTER(pStr[3],'D')))
				{
				pStr  += 4;
				iSize += 4;
				}
			else if(iSize+5<=iMax && CMP_LETTER(pStr[1],'Q') && CMP_LETTER(pStr[3],'A') && CMP_LETTER(pStr[4],'N'))
				{
				pStr  += 5;
				iSize += 5;
				}
			else{
				return iSize;
				}

			if(iSize>=iMax)return iMax;
			}

	for(       ;strwchara (*pStr,0x00000001); pStr++) iSize++;if(iSize>=iMax)return iSize;}
	if (        CMP_LETTER(*pStr,'E')      ) {pStr++; iSize++;if(iSize>=iMax)return iSize;
	if (        strwchara (*pStr,0x06000000)){pStr++; iSize++;if(iSize>=iMax)return iSize;}
	for(       ;strwchara (*pStr,0x00000001); pStr++){iSize++;if(iSize>=iMax)return iSize;}}


return iSize;
}

int _cdecl strflen(const wchar_t *pStr,int iMax)
{
int		iSize;


	if(iMax<=0)return 0;

	for(iSize=0;strwcharw(*pStr,0x000000A0); pStr++){iSize++;if(iSize>=iMax)return iSize;}
	if (        strwcharw(*pStr,0x06000000)){pStr++; iSize++;if(iSize>=iMax)return iSize;}
	for(       ;strwcharw(*pStr,0x00000001); pStr++){iSize++;if(iSize>=iMax)return iSize;}
	if (        *pStr=='.'                 ){pStr++; iSize++;if(iSize>=iMax)return iSize;

		if(*pStr=='#')
			{
			if(iSize+4>iMax || !CMP_LETTER(pStr[2],'N'))
				{
				return iSize;
				}
			else if(CMP_LETTER(pStr[1],'I') && (CMP_LETTER(pStr[3],'F') || CMP_LETTER(pStr[3],'D')))
				{
				pStr  += 4;
				iSize += 4;
				}
			else if(iSize+5<=iMax && CMP_LETTER(pStr[1],'Q') && CMP_LETTER(pStr[3],'A') && CMP_LETTER(pStr[4],'N'))
				{
				pStr  += 5;
				iSize += 5;
				}
			else{
				return iSize;
				}

			if(iSize>=iMax)return iMax;
			}

	for(       ;strwcharw (*pStr,0x00000001); pStr++) iSize++;if(iSize>=iMax)return iSize;}
	if (        CMP_LETTER(*pStr,'E')      ) {pStr++; iSize++;if(iSize>=iMax)return iSize;
	if (        strwcharw (*pStr,0x06000000)){pStr++; iSize++;if(iSize>=iMax)return iSize;}
	for(       ;strwcharw (*pStr,0x00000001); pStr++){iSize++;if(iSize>=iMax)return iSize;}}


return iSize;
}

//*****************************************************************************
//*
//*		strflen
//*
//*****************************************************************************
//	Berechnet die Lnge einer Fliekommazahl mit vorangehenden Leerzeichen.
//	pStr	: Ist der Zeiger auf den Text mit der Zahl
//	iMax	: Ist die maximale Lnge vom Text
//	iMode	: Bestimmt die Art der erlaubten Formate
//				Bit0	= sind Vorzeichen erlaubt
//				Bit1 	= ist die exponentielle Darstellung erlaubt (1.0e+345)
//				Bit2 	= ist ',' als Komma erlaubt
//				Bit3 	= sind spezial Zahlen erlaubt (.#INV .#IND .#QNAN)
//				Bit4 	= Leerzeichen nach Minus nicht erlaubt
//				Bit5 	= Leerzeichen als Tausendertrennzeichen erlaubt
//				Bit6 	= Punkt als Tausendertrennzeichen erlaubt (nur mit Bit2)
//				Bit7 	= Unterline als Tausendertrennzeichen erlaubt
//				Bit8 	= Leerzeichen am Ende auch mit zhlen
//				Bit9	= Keine Leerzeichen am Anfang erlaubt
//	pPlacePtr: Hier werden die Positionen der Abschnitte gespeichert (wenn!=NULL)
//				Byte0	= Position des Vorzeichens (0=keines 1=[0] 2=[1]... 255=?)
//				Byte1	= Position der Mantisse    (0=keines 1=[0] 2=[1]... 255=?)
//				Byte2	= Position nach Komma      (0=keines 1=[0] 2=[1]... 255=?)
//				Byte3	= Position des Exponenten  (0=keiner 1=[0] 2=[1]... 255=?)
//	Ergibt die Anzahl an Textzeichen die zur Fliekommazahl gehren.
int _cdecl strflen(const char *pStr,int iMax,int iMode,void *pPlacePtr)
{
unsigned char  *pPlace;
unsigned	 	uTemp;
int				iSize;


	if(!(iMode&4))iMode&=~0x40;							// Tausendertrennzeichen mit '.' nur bei Komma


	if(pPlacePtr)
			pPlace = (unsigned char*)pPlacePtr;
	else	pPlace = (unsigned char*)&uTemp;


	*(int*)pPlace=0;

	if(iMax<=0)return 0;

	iSize = 0;


	if(!(iMode&0x0200))									// Leerzeichen am Anfang berspringen
		{
		for(;;)
			{
			if(!strwchara(*pStr,0x00A0))break;

			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}
		}

	if(iMode&0x01)										// Vorzeichen berspringen
		{
		if(strwchara(*pStr,0x06000000))					// '+' oder '-'
			{
			iSize++;
			pStr++;

			pPlace[0] = (char)((iSize<255)? iSize:255);

			if(iSize>=iMax)return iSize;

			if(iMode&0x10)								// Leerzeichen nach Vorzeichen berspringen
				{
				for(;;)
					{
					if(!strwchara(*pStr,0x00A0))break;

					iSize++;
					pStr++;

					if(iSize>=iMax)return iSize;
					}
				}
			}
		}



	if(strwchara(*pStr,0x0001))							// Gibt es eine Zahlenmantisse (123.456)
		{
		pPlace[1] = (char)((iSize<254)? iSize+1:255);

		for(;;)											// Erste Zahl berspringen
			{
			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;

			if(strwchara(*pStr,0x0288) && (iMode&0xE0))
				{
				if(*pStr!=' ' || !(iMode&0x20))			// Tausendertrennzeichen
				if(*pStr!='.' || !(iMode&0x40))
				if(*pStr!='_' || !(iMode&0x80))
					{
					break;
					}

				iSize++;
				pStr++;

				if(iSize>=iMax)return iSize;
				}

			if(!strwchara(*pStr,0x0001))
				{
				break;
				}
			}
		}



	if(strwchara(*pStr,0x1200))							// Kommastellen vorhanden '.' oder ','
		{
		if(*pStr==',' && !(iMode&0x04))
			{
			if(!pPlace[1])
				{
				iSize	  = pPlace[0]-1;
				pPlace[0] = 0;
				}

			return iSize;
			}

		iSize++;
		pStr++;

		if(iSize>=iMax)
			{
			return iSize-1;
			}

		if((iMode&0x08) && *pStr=='#')					// Spezial Zahlen .#INV .#IND .#QNAN
			{
			pPlace[2] = (char)((iSize<254)? iSize+1:255);

			if(iSize+4>iMax || !CMP_LETTER(pStr[2],'N'))
				{
				return iSize;
				}
			else if(CMP_LETTER(pStr[1],'I') && (CMP_LETTER(pStr[3],'F') || CMP_LETTER(pStr[3],'D')))
				{
				iSize += 4;
				pStr  += 4;
				}
			else if(iSize+5<=iMax && CMP_LETTER(pStr[1],'Q') && CMP_LETTER(pStr[3],'A') && CMP_LETTER(pStr[4],'N'))
				{
				iSize += 5;
				pStr  += 5;
				}
			else{
				return iSize;
				}

			if(iSize>=iMax)return iMax;
			}

		if(strwchara(*pStr,0x0001))						// Gibt es Zahlen nach dem Komma
			{
			pPlace[2] = (char)((iSize<254)? iSize+1:255);

			for(;;)										// Kommastellen berspringen
				{
				iSize++;
				pStr++;

				if(iSize>=iMax)return iSize;
				if(!strwchara(*pStr,0x0001))break;
				}
			}
		else if(!pPlace[1])								// Komma ohne Zahlen
			{
			if(pPlace[0])
				{
				iSize     = pPlace[0];
				pPlace[0] = 0;
				}

			return iSize-1;
			}
		}
	else if(!pPlace[1])									// Nur ein Vorzeichen
		{
		iSize     = pPlace[0];
		pPlace[0] = 0;

		return iSize;
		}


	if(CMP_LETTER(*pStr,'E'))							// Exponent berspringen
		{
		if(!(iMode&0x02))return iSize;

		iSize++;
		pStr++;

		pPlace[3] = (char)((iSize<254)? iSize+1:255);

		if(iSize>=iMax)return iSize;

		if(strwchara(*pStr,0x06000000))					// '+' oder '-' beim Exponent
			{
			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}

		for(;;)											// Zahlen im Exponent berspringen
			{
			if(!strwchara(*pStr,0x0001))break;

			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}
		}


	if(iMode&0x0100)									// Leerzeichen am Ende berspringen
		{
		for(;;)
			{
			if(!strwchara(*pStr,0x00A0))break;

			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}
		}


return iSize;
}


int _cdecl strflen(const wchar_t *pStr,int iMax,int iMode,void *pPlacePtr)
{
unsigned char  *pPlace;
unsigned	 	uTemp;
int				iSize;


	if(!(iMode&4))iMode&=~0x40;							// Tausendertrennzeichen mit '.' nur bei Komma


	if(pPlacePtr)
			pPlace = (unsigned char*)pPlacePtr;
	else	pPlace = (unsigned char*)&uTemp;


	*(int*)pPlace=0;

	if(iMax<=0)return 0;

	iSize = 0;


	if(!(iMode&0x0200))									// Leerzeichen am Anfang berspringen
		{
		for(;;)
			{
			if(!strwcharw(*pStr,0x00A0))break;

			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}
		}

	if(iMode&0x01)										// Vorzeichen berspringen
		{
		if(strwcharw(*pStr,0x06000000))					// '+' oder '-'
			{
			iSize++;
			pStr++;

			pPlace[0] = (char)((iSize<255)? iSize:255);

			if(iSize>=iMax)return iSize;

			if(iMode&0x10)								// Leerzeichen nach Vorzeichen berspringen
				{
				for(;;)
					{
					if(!strwcharw(*pStr,0x00A0))break;

					iSize++;
					pStr++;

					if(iSize>=iMax)return iSize;
					}
				}
			}
		}



	if(strwcharw(*pStr,0x0001))							// Gibt es eine Zahlenmantisse (123.456)
		{
		pPlace[1] = (char)((iSize<254)? iSize+1:255);

		for(;;)											// Erste Zahl berspringen
			{
			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;

			if(strwcharw(*pStr,0x0288) && (iMode&0xE0))
				{
				if(*pStr!=' ' || !(iMode&0x20))			// Tausendertrennzeichen
				if(*pStr!='.' || !(iMode&0x40))
				if(*pStr!='_' || !(iMode&0x80))
					{
					break;
					}

				iSize++;
				pStr++;

				if(iSize>=iMax)return iSize;
				}

			if(!strwcharw(*pStr,0x0001))break;
			}
		}



	if(strwcharw(*pStr,0x1200))							// Kommastellen vorhanden '.' oder ','
		{
		if(*pStr==',' && !(iMode&0x04))
			{
			if(!pPlace[1])
				{
				iSize	  = pPlace[0]-1;
				pPlace[0] = 0;
				}

			return iSize;
			}

		iSize++;
		pStr++;

		if(iSize>=iMax)
			{
			return iSize-1;
			}

		if((iMode&0x08) && *pStr=='#')					// Spezial Zahlen .#INV .#IND .#QNAN
			{
			pPlace[2] = (char)((iSize<254)? iSize+1:255);

			if(iSize+4>iMax || !CMP_LETTER(pStr[2],'N'))
				{
				return iSize;
				}
			else if(CMP_LETTER(pStr[1],'I') && (CMP_LETTER(pStr[3],'F') || CMP_LETTER(pStr[3],'D')))
				{
				iSize += 4;
				pStr  += 4;
				}
			else if(iSize+5<=iMax && CMP_LETTER(pStr[1],'Q') && CMP_LETTER(pStr[3],'A') && CMP_LETTER(pStr[4],'N'))
				{
				iSize += 5;
				pStr  += 5;
				}
			else{
				return iSize;
				}

			if(iSize>=iMax)return iMax;
			}

		if(strwcharw(*pStr,0x0001))						// Gibt es Zahlen nach dem Komma
			{
			pPlace[2] = (char)((iSize<254)? iSize+1:255);

			for(;;)										// Kommastellen berspringen
				{
				iSize++;
				pStr++;

				if(iSize>=iMax)return iSize;
				if(!strwcharw(*pStr,0x0001))break;
				}
			}
		else if(!pPlace[1])								// Komma ohne Zahlen
			{
			if(pPlace[0])
				{
				iSize     = pPlace[0];
				pPlace[0] = 0;
				}

			return iSize-1;
			}
		}
	else if(!pPlace[1])									// Nur ein Vorzeichen
		{
		iSize     = pPlace[0];
		pPlace[0] = 0;

		return iSize;
		}


	if(CMP_LETTER(*pStr,'E'))							// Exponent berspringen
		{
		if(!(iMode&0x02))return iSize;

		iSize++;
		pStr++;

		pPlace[3] = (char)((iSize<254)? iSize+1:255);

		if(iSize>=iMax)return iSize;

		if(strwcharw(*pStr,0x06000000))					// '+' oder '-' beim Exponent
			{
			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}

		for(;;)											// Zahlen im Exponent berspringen
			{
			if(!strwcharw(*pStr,0x0001))break;

			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}
		}


	if(iMode&0x0100)									// Leerzeichen am Ende berspringen
		{
		for(;;)
			{
			if(!strwcharw(*pStr,0x00A0))break;

			iSize++;
			pStr++;

			if(iSize>=iMax)return iSize;
			}
		}


return iSize;
}


//****************************************************************************
//*
//*		ctostr
//*
//****************************************************************************
//	Wandelt eine Zeichen in einen Text um. Dabei werden Zeichen kleiner 32
//	oder Spezialzeichen zu Escape-Folgen umgewandelt ("/x???").
//	iSign		: Ist der Zahlenwert des Zeichens das umgewandelt werden soll.
//	pStr		: Ist der Zeiger auf den Textpuffer.
//	iMode		: Ist der Ausgabemodus, wenn gleich Null so werden '\'' vor 
//				  und nach den Text eingefgt.
//	Ergibt die Lnge des erzeugten Textes in Zeichen.
int _cdecl ctostr(int iSign,char *pStr,int iMode)
{
int		iSize=0;



	if(!iMode)									// Am Anfang ein Anfhrungszeichen
		{
		*pStr='\'';pStr++;iSize++;
		}

	if(iSign=='\\')
		{
		*pStr='\\';pStr++;iSize++;
		*pStr='\\';pStr++;iSize++;
		}
	else if((unsigned)iSign=='\'')
		{
		*pStr='\\';pStr++;iSize++;
		*pStr='\'';pStr++;iSize++;
		}
	else if((unsigned)iSign>=' ')
		{
		     if(iSign=='\\'){*pStr='\\';pStr++;iSize++;*pStr='\\';iSize++;pStr++;}
		else if(iSign=='\"'){*pStr='\\';pStr++;iSize++;*pStr='\"';iSize++;pStr++;}
		else if(iSign=='\''){*pStr='\\';pStr++;iSize++;*pStr='\'';iSize++;pStr++;}
		else{
			*pStr=(char)iSign;pStr++;iSize++;
			}
		}
	else for(;;)
		{
		*pStr='\\';pStr++;iSize++;

		if(iSign=='\t'){*pStr='t' ;iSize++;pStr++;break;}
		if(iSign=='\r'){*pStr='r' ;iSize++;pStr++;break;}
		if(iSign=='\n'){*pStr='n' ;iSize++;pStr++;break;}
		if(iSign=='\a'){*pStr='a' ;iSize++;pStr++;break;}
		if(iSign=='\b'){*pStr='b' ;iSize++;pStr++;break;}
		if(iSign=='\f'){*pStr='f' ;iSize++;pStr++;break;}
		if(iSign=='\v'){*pStr='v' ;iSize++;pStr++;break;}

		if(iMode)
			{
			pStr[0]='x' ;
			pStr[1]='0' ;
			pStr[2]=cHexDigits[(iSign>>4)&0x0F];
			pStr[3]=cHexDigits[ iSign    &0x0F];
			pStr +=4;
			iSize+=4;
			}
		else{
			pStr[0]='x' ;
			pStr[1]=cHexDigits[(iSign>>4)&0x0F];
			pStr[2]=cHexDigits[ iSign    &0x0F];
			pStr +=3;
			iSize+=3;
			}

		break;
		}

	if(!iMode)									// Am Ende ein Anfhrungszeichen
		{
		*pStr='\'';pStr++;iSize++;
		}

	*pStr=0;



return iSize;
}


int _cdecl ctostr(int iSign,wchar_t *pStr,int iMode)
{
int		iSize=0;



	if(!iMode)									// Am Anfang ein Anfhrungszeichen
		{
		*pStr='\'';pStr++;iSize++;
		}

	if(iSign=='\\')
		{
		*pStr='\\';pStr++;iSize++;
		*pStr='\\';pStr++;iSize++;
		}
	else if((unsigned)iSign=='\'')
		{
		*pStr='\\';pStr++;iSize++;
		*pStr='\'';pStr++;iSize++;
		}
	else if((unsigned)iSign>=' ')
		{
		*pStr=(char)iSign;pStr++;iSize++;
		}
	else for(;;)
		{
		*pStr='\\';pStr++;iSize++;

		if(iSign=='\\'){*pStr='\\';iSize++;pStr++;break;}
		if(iSign=='\"'){*pStr='\"';iSize++;pStr++;break;}
		if(iSign=='\''){*pStr='\'';iSize++;pStr++;break;}
		if(iSign=='\t'){*pStr='t' ;iSize++;pStr++;break;}
		if(iSign=='\r'){*pStr='r' ;iSize++;pStr++;break;}
		if(iSign=='\n'){*pStr='n' ;iSize++;pStr++;break;}
		if(iSign=='\a'){*pStr='a' ;iSize++;pStr++;break;}
		if(iSign=='\b'){*pStr='b' ;iSize++;pStr++;break;}
		if(iSign=='\f'){*pStr='f' ;iSize++;pStr++;break;}
		if(iSign=='\v'){*pStr='v' ;iSize++;pStr++;break;}

		if(iMode)
			{
			pStr[0]='x' ;
			pStr[1]='0' ;
			pStr[2]=cHexDigits[(iSign>>4)&0x0F];
			pStr[3]=cHexDigits[ iSign    &0x0F];
			pStr +=4;
			iSize+=4;
			}
		else{
			pStr[0]='x' ;
			pStr[1]=cHexDigits[(iSign>>4)&0x0F];
			pStr[2]=cHexDigits[ iSign    &0x0F];
			pStr +=3;
			iSize+=3;
			}

		break;
		}

	if(!iMode)									// Am Ende ein Anfhrungszeichen
		{
		*pStr='\'';pStr++;iSize++;
		}

	*pStr=0;



return iSize;
}

//*****************************************************************************
//*
//*		strlspn
//*
//*****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text
//	welches nicht in einer Liste vorkommt.
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pList		: Ist die Liste mit Textzeichen nach dehnen gesucht werden soll.
//	Ergibt die Position nach dem gefundenen Zeichen. Wird kein Zeichen gefunden
//	wird die Textlnge zurckgegeben.

#ifdef 	_DEBUG

int _cdecl strlspn(const char *pString,const char *pList)
{
int i;

	i=(int)strcspn(pString,pList);
	if(pString[i])i++;

return i;
}

int _cdecl strlspn(const wchar_t *pString,const wchar_t *pList)
{
int i;

	i=(int)wcscspn(pString,pList);
	if(pString[i])i++;

return i;
}

#endif

//*****************************************************************************
//*
//*		strtof
//*
//*****************************************************************************
//	Wandelt einen Text in eine Fliekommazahl um.
//	pStr	: Zeiger auf den Text
//	pCount	: Hier wird die Anzahl der verarbeiteten Zeichen aus dem Text
//			  gespeichert. Wenn der Zeiger Null ist wird nichts gespeichert.
//	iMode	: Bestimmt die Art der erlaubten Formate
//				Bit0 = sind Vorzeichen erlaubt
//				Bit1 = ist die exponentielle Darstellung erlaubt (1.0e+345)
//				Bit2 = ist ',' als Komma erlaubt
//				Bit3 = sind spezial Zahlen erlaubt (.#INV .#IND .#QNAN)
//				Bit4 = Leerzeichen nach Minus erlaubt
//				Bit5 = Leerzeichen als Tausendertrennzeichen erlaubt
//				Bit6 = Punkt als Tausendertrennzeichen erlaubt (nur mit Bit2)
//				Bit7 = Unterline als Tausendertrennzeichen erlaubt
//				Bit8 = Leerzeichen am Ende auch mit zhlen
//				Bit9 = Kein Vorzeichen erlaubt
//	Ergibt den Wert der Fliekommazahl.
#ifdef 	_DEBUG

double _cdecl strtof(const char *pStr)
{
return atof(pStr);
};

double _cdecl strtof(const char *pStr,int *pCount)
{
	if(pCount)*pCount=strflen(pStr);

return atof(pStr);
}

double _cdecl strtof(const wchar_t *pStr)
{
double dVaue=0.0;

	swscanf(pStr,L"%lf",&dVaue);

return dVaue;
};

double _cdecl strtof(const wchar_t *pStr,int *pCount)
{
double dVaue=0.0;

	if(pCount)*pCount=strflen(pStr);
	swscanf(pStr,L"%lf",&dVaue);

return dVaue;
}

#endif

double _cdecl strtof(const char *pStr,int *pCount,int iMode)
{
char	cTemp[64];
char	cSign;
int		iCount;
int		iOut;
int		iPos;


	iCount = strflen(pStr,63,iMode);
	if(pCount)*pCount=iCount;

	for(iPos=0,iOut=0;iPos<iCount;iPos++)
		{
		cSign = pStr[iPos];

		if(cSign==' ' || cSign=='_')
			{
			continue;
			}
		else if(cSign==',' && (iMode&0x04))
			{
			cSign = '.';
			}
		else if(cSign==' ' && (iMode&0x20))
			{
			continue;
			}
		else if(cSign=='.' && (iMode&0x44)==0x44)
			{
			continue;
			}

		cTemp[iOut]=cSign;
		iOut++;
		}


	cTemp[iOut] = 0;


return atof(cTemp);
}

double _cdecl strtof(const wchar_t *pStr,int *pCount,int iMode)
{
char	cTemp[64];
char	cSign;
int		iCount;
int		iOut;
int		iPos;


	iCount = strflen(pStr,63,iMode);
	if(pCount)*pCount=iCount;

	for(iPos=0,iOut=0;iPos<iCount;iPos++)
		{
		cSign = (char)pStr[iPos];

		if(cSign==' ' || cSign=='_')
			{
			continue;
			}
		else if(cSign==',' && (iMode&0x04))
			{
			cSign = '.';
			}
		else if(cSign==' ' && (iMode&0x20))
			{
			continue;
			}
		else if(cSign=='.' && (iMode&0x44)==0x44)
			{
			continue;
			}


		cTemp[iOut]=cSign;
		iOut++;
		}


	cTemp[iOut] = 0;


return atof(cTemp);
}

//****************************************************************************
//*
//*		strchar
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//	iMax		: Ist die maximale Suchlnge im Text.
//  Ergibt die erste Position des Zeichens im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strchar(const char *pString,int iSign,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(char)iSign)break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	ecx,iMax				;// Maximal Bytezahl
		mov  	esi,pString				;// String-Adresse
		cmp  	ecx,0					;// Ist Bereich zu klein ?
		jle	 	ende					;// 		*
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,3					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsb							;//	Zeichen laden
		cmp		al,bl					;//			*
		je	 	ende1					;// 		*
		test	al,al					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		dec		ecx						;//			*
		jz		ende					;// 		*
		test 	esi,3					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	sub		ecx,4					;// Sind noch 4 Byte vorhanden
		jl		last					;//			*

		lodsd							;//	4 Zeichen laden
		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende4					;// 		*
		test	al,al					;// Textende erreicht ?
		jz   	ende4					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		je   	ende3					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jz   	ende3					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende2					;// 		*
		test	al,al					;// Textende erreicht ?
		jz   	ende2					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		je   	ende1					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jnz   	sch						;// 		*
		sub		esi,1					;//	Addresse um 1 korrigieren
		jmp		ende					;//			*

last:	add		ecx,4					;// Zhler um 4 korrigieren
		jecxz	ende					;//			*
		jmp		next					;//			*

ende4:	sub		esi,4					;//	Addresse um 4 korrigieren
		jmp		ende					;//			*
ende3:	sub		esi,3					;//	Addresse um 3 korrigieren
		jmp		ende					;//			*
ende2:	sub		esi,2					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,1					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		}


#endif

}

int	_cdecl strchar(const wchar_t *pString,int iSign,int iMax)
{


//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(unsigned short)iSign)break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	ecx,iMax				;// Maximal Bytezahl
		mov  	esi,pString				;// String-Adresse
		cmp  	ecx,0					;// Ist Bereich zu klein ?
		jle	 	ende					;// 		*
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,2					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsw							;//	Zeichen laden
		cmp		ax,bx					;//			*
		je	 	ende1					;// 		*
		test	ax,ax					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		dec		ecx						;//			*
		jz		ende					;// 		*
		test 	esi,2					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	sub		ecx,2					;// Sind noch 2 Worte vorhanden
		jl		last					;//			*

		lodsd							;//	4 Zeichen laden
		cmp  	ax,bx					;// Zeichen gefunden ?
		je   	ende2					;// 		*
		test	ax,ax					;// Textende erreicht ?
		jz   	ende2					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		cmp  	ax,bx					;// Zeichen gefunden ?
		je   	ende1					;// 		*
		test	ax,ax					;// Textende erreicht ?
		jnz   	sch						;// 		*
		sub		esi,2					;//	Addresse um 1 korrigieren
		jmp		ende					;//			*

last:	add		ecx,2					;// Zhler um 2 korrigieren
		jecxz	ende					;//			*
		jmp		next					;//			*

ende2:	sub		esi,4					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,2					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		shr		eax,1					;//			*
		}


#endif

}

//****************************************************************************
//*
//*		strchar
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//  Ergibt die erste Position des Zeichens im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strchar(const char *pString,int iSign)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(char)iSign)break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	esi,pString				;// String-Adresse
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,3					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsb							;//	Zeichen laden
		cmp		al,bl					;//			*
		je	 	ende1					;// 		*
		test	al,al					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		test 	esi,3					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	lodsd							;//	4 Zeichen laden
		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende4					;// 		*
		test	al,al					;// Textende erreicht ?
		jz   	ende4					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		je   	ende3					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jz   	ende3					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende2					;// 		*
		test	al,al					;// Textende erreicht ?
		jz   	ende2					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		je   	ende1					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jnz   	sch						;// 		*
		sub		esi,1					;//	Addresse um 1 korrigieren
		jmp		ende					;//			*

ende4:	sub		esi,4					;//	Addresse um 4 korrigieren
		jmp		ende					;//			*
ende3:	sub		esi,3					;//	Addresse um 3 korrigieren
		jmp		ende					;//			*
ende2:	sub		esi,2					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,1					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		}


#endif

}

int	_cdecl strchar(const wchar_t *pString,int iSign)
{


//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(unsigned short)iSign)break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	esi,pString				;// String-Adresse
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,2					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsw							;//	Zeichen laden
		cmp		ax,bx					;//			*
		je	 	ende1					;// 		*
		test	ax,ax					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		test 	esi,2					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	lodsd							;//	4 Zeichen laden
		cmp  	ax,bx					;// Zeichen gefunden ?
		je   	ende2					;// 		*
		test	ax,ax					;// Textende erreicht ?
		jz   	ende2					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		cmp  	ax,bx					;// Zeichen gefunden ?
		je   	ende1					;// 		*
		test	ax,ax					;// Textende erreicht ?
		jnz   	sch						;// 		*
		sub		esi,2					;//	Addresse um 1 korrigieren
		jmp		ende					;//			*

ende2:	sub		esi,4					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,2					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		shr		eax,1					;//			*
		}


#endif

}

//****************************************************************************
//*
//*		strlchar
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//	iMax		: Ist die maximale Suchlnge im Text.
//  Ergibt die erste Position nach dem Zeichen im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strlchar(const char *pString,int iSign,int iMax)
{


//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	int iPos;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(char)iSign){iPos++;break;}
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	ecx,iMax				;// Maximal Bytezahl
		mov  	esi,pString				;// String-Adresse
		cmp  	ecx,0					;// Ist Bereich zu klein ?
		jle	 	ende					;// 		*
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,3					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsb							;//	Zeichen laden
		test	al,al					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		cmp		al,bl					;//	Zeichen gefunden ?
		je	 	ende					;// 		*
		dec		ecx						;//	Maximale Lnge erreicht
		jz		ende					;// 		*
		test 	esi,3					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	sub		ecx,4					;// Sind noch 4 Byte vorhanden
		jl		last					;//			*

		lodsd							;//	4 Zeichen laden
		test	al,al					;// Textende erreicht ?
		jz   	ende4					;// 		*
		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende3					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jz   	ende3					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		je   	ende2					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		test	al,al					;// Textende erreicht ?
		jz   	ende2					;// 		*
		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende1					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jz   	ende1					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		jne   	sch						;// 		*
		jmp		ende					;//			*

last:	add		ecx,4					;// Zhler um 4 korrigieren
		jecxz	ende					;//			*
		jmp		next					;//			*

ende4:	sub		esi,4					;//	Addresse um 4 korrigieren
		jmp		ende					;//			*
ende3:	sub		esi,3					;//	Addresse um 3 korrigieren
		jmp		ende					;//			*
ende2:	sub		esi,2					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,1					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		}


#endif

}

int	_cdecl strlchar(const wchar_t *pString,int iSign,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
		}


return iPos;


//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	ecx,iMax				;// Maximal Bytezahl
		mov  	esi,pString				;// String-Adresse
		cmp  	ecx,0					;// Ist Bereich zu klein ?
		jle	 	ende					;// 		*
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,2					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsw							;//	Zeichen laden
		test	ax,ax					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		cmp		ax,bx					;//	Zeichen gefunden ?
		je	 	ende					;// 		*
		dec		ecx						;//	Maximale Lnge erreicht
		jz		ende					;// 		*
		test 	esi,2					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	sub		ecx,2					;// Sind noch 2 Worte vorhanden
		jl		last					;//			*

		lodsd							;//	4 Zeichen laden
		test	ax,ax					;// Textende erreicht ?
		jz   	ende2					;// 		*
		cmp  	ax,bx					;// Zeichen gefunden ?
		je   	ende1					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		test	ax,ax					;// Textende erreicht ?
		jz   	ende1					;// 		*
		cmp  	ax,bx					;// Zeichen gefunden ?
		jne   	sch						;// 		*
		jmp		ende					;//			*

last:	add		ecx,2					;// Zhler um 2 korrigieren
		jecxz	ende					;//			*
		jmp		next					;//			*

ende2:	sub		esi,4					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,2					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		shr		eax,1					;//			*
		}

#endif

}

//****************************************************************************
//*
//*		strlchar
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//  Ergibt die erste Position nach dem Zeichen im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strlchar(const char *pString,int iSign)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

	int iPos;

	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(char)iSign){iPos++;break;}
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	esi,pString				;// String-Adresse
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,3					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsb							;//	Zeichen laden
		test	al,al					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		cmp		al,bl					;//	Zeichen gefunden ?
		je	 	ende					;// 		*
		test 	esi,3					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	lodsd							;//	4 Zeichen laden
		test	al,al					;// Textende erreicht ?
		jz   	ende4					;// 		*
		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende3					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jz   	ende3					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		je   	ende2					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		test	al,al					;// Textende erreicht ?
		jz   	ende2					;// 		*
		cmp  	al,bl					;// Zeichen gefunden ?
		je   	ende1					;// 		*
		test	ah,ah					;// Textende erreicht ?
		jz   	ende1					;// 		*
		cmp  	ah,bl					;// Zeichen gefunden ?
		jne   	sch						;// 		*
		jmp		ende					;//			*

ende4:	sub		esi,4					;//	Addresse um 4 korrigieren
		jmp		ende					;//			*
ende3:	sub		esi,3					;//	Addresse um 3 korrigieren
		jmp		ende					;//			*
ende2:	sub		esi,2					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,1					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		}


#endif

}

int	_cdecl strlchar(const wchar_t *pString,int iSign)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
		}


return iPos;


//********************* Mit Assembler ******************************************
#else
__asm	{
		mov  	esi,pString				;// String-Adresse
		mov	 	ebx,iSign				;// Zeichen laden

		test 	esi,2					;// Auf eine gerade Adresse
		jz		sch						;//			*
next:	lodsw							;//	Zeichen laden
		test	ax,ax					;// Textende erreicht ?
		jz	 	ende1					;// 		*
		cmp		ax,bx					;//	Zeichen gefunden ?
		je	 	ende					;// 		*
		test 	esi,2					;// Ist Adresse gerade ?
		jnz		next					;//			*


sch:	lodsd							;//	4 Zeichen laden
		test	ax,ax					;// Textende erreicht ?
		jz   	ende2					;// 		*
		cmp  	ax,bx					;// Zeichen gefunden ?
		je   	ende1					;// 		*

		shr		eax,16					;//	Die oberen Zeichen holen

		test	ax,ax					;// Textende erreicht ?
		jz   	ende1					;// 		*
		cmp  	ax,bx					;// Zeichen gefunden ?
		jne   	sch						;// 		*
		jmp		ende					;//			*

ende2:	sub		esi,4					;//	Addresse um 2 korrigieren
		jmp		ende					;//			*
ende1:	sub		esi,2					;//	Addresse um 1 korrigieren

ende:	sub  	esi,pString				;// Lnge berechnen
		mov  	eax,esi					;//			*
		shr		eax,1					;//			*
		}

#endif

}

//****************************************************************************
//*
//*		strrchar
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens in einem Text vonhinten.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//	iLen		: Ist die Lnge des Textes, wird mit strlen() bestimmt
//				  wenn iLen nicht angegeben ist.
//  Ergibt die erste Position des Zeichens im Text, bzw. -1 wenn kein Zeichen
//	gefunden wurde.
int	_cdecl strrchar(const char *pString,int iSign,int iLen)
{

	for(iLen--;iLen>=0;iLen--)
		{
		if(pString[iLen]==(char)iSign)return iLen;
		}

return -1;
}

int	_cdecl strrchar(const wchar_t *pString,int iSign,int iLen)
{

	for(iLen--;iLen>=0;iLen--)
		{
		if(pString[iLen]==(wchar_t)iSign)return iLen;
		}

return -1;
}

#ifdef 	_DEBUG

int	_cdecl strrchar(const char *pString,int iSign)
{
int		iLen;

	iLen = (int)strlen(pString);

	for(iLen--;iLen>=0;iLen--)
		{
		if(pString[iLen]==(char)iSign)return iLen;
		}

return -1;
}

int	_cdecl strrchar(const wchar_t *pString,int iSign)
{
int		iLen;

	iLen = (int)wcslen(pString);

	for(iLen--;iLen>=0;iLen--)
		{
		if(pString[iLen]==(wchar_t)iSign)return iLen;
		}

return -1;
}

#endif

//****************************************************************************
//*
//*		strchar
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//	iMax		: Ist die maximale Suchlnge im Text.
//	iJump		: Ist ein Jump-Zeichen bei dem das nchste bersprungen wird.
//  Ergibt die erste Position nach dem Zeichen im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strchar(const char *pString,int iSign,int iMax,int iJump)
{
int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
		if(pString[iPos]==(char)iSign){break;}
		}


return iPos;
}

int	_cdecl strchar(const wchar_t *pString,int iSign,int iMax,int iJump)
{
int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(unsigned short)iJump){iPos++;if(iPos>=iMax)break;continue;}
		if(pString[iPos]==(unsigned short)iSign){break;}
		}


return iPos;
}

//****************************************************************************
//*
//*		strlchar
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//	iMax		: Ist die maximale Suchlnge im Text.
//	iJump		: Ist ein Jump-Zeichen bei dem das nchste bersprungen wird.
//  Ergibt die erste Position nach dem Zeichen im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strlchar(const char *pString,int iSign,int iMax,int iJump)
{
int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
		if(pString[iPos]==(char)iSign){iPos++;break;}
		}


return iPos;
}

int	_cdecl strlchar(const wchar_t *pString,int iSign,int iMax,int iJump)
{
int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==(unsigned short)iJump){iPos++;if(iPos>=iMax)break;continue;}
		if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
		}


return iPos;
}


//****************************************************************************
//*
//*		strchar
//*
//****************************************************************************
//	Sucht das Auftreten eines Zeichens in einem Text, bis zu einer Stopgruppe.
//	iMode		: Bestimmt die Gruppe bei der abgebrochen wird.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//	iMax		: Ist die maximale Suchlnge im Text.
//	iJump		: Ist ein Jump-Zeichen bei dem das nchste bersprungen wird.
//  Ergibt die erste Position des Zeichens im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strchar(int iMode,const char *pString,int iSign,int iMax,int iJump)
{
int iPos;

	iMode ^= 0x80000000;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(strwchara(pString[iPos],iMode))break;
		if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
		if(pString[iPos]==(char)iSign)break;
		}


return iPos;
}

int	_cdecl strchar(int iMode,const wchar_t *pString,int iSign,int iMax,int iJump)
{
int iPos;

	iMode ^= 0x80000000;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(strwcharw(pString[iPos],iMode))break;
		if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
		if(pString[iPos]==(unsigned short)iSign)break;
		}


return iPos;
}

//****************************************************************************
//*
//*		strchar
//*
//****************************************************************************
//	Sucht das Auftreten eines Zeichens in einem Text, bis zu einer Stopgruppe.
//	iMode		: Bestimmt die Gruppe bei der abgebrochen wird.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//	iMax		: Ist die maximale Suchlnge im Text.
//  Ergibt die erste Position des Zeichens im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strchar(int iMode,const char *pString,int iSign,int iMax)
{
int iPos;

	iMode ^= 0x80000000;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(strwchara(pString[iPos],iMode))break;
		if(pString[iPos]==(char)iSign)break;
		}


return iPos;
}

int	_cdecl strchar(int iMode,const wchar_t *pString,int iSign,int iMax)
{
int iPos;

	iMode ^= 0x80000000;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(strwcharw(pString[iPos],iMode))break;
		if(pString[iPos]==(unsigned short)iSign)break;
		}


return iPos;
}

//****************************************************************************
//*
//*		strchar
//*
//****************************************************************************
//	Sucht das Auftreten eines Zeichens in einem Text, bis zu einer Stopgruppe.
//	iMode		: Bestimmt die Gruppe bei der abgebrochen wird
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//  Ergibt die erste Position des Zeichens im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strchar(int iMode,const char *pString,int iSign)
{
int iPos;

	iMode ^= 0x80000000;

	for(iPos=0;;iPos++)
		{
		if(strwchara(pString[iPos],iMode))break;
		if(pString[iPos]==(char)iSign)break;
		}


return iPos;
}

int	_cdecl strchar(int iMode,const wchar_t *pString,int iSign)
{
int iPos;

	iMode ^= 0x80000000;

	for(iPos=0;;iPos++)
		{
		if(strwcharw(pString[iPos],iMode))break;
		if(pString[iPos]==(unsigned short)iSign)break;
		}


return iPos;
}


//****************************************************************************
//*
//*		strlchar
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	iMode		: Bestimmt die Gruppe bei der abgebrochen wird.
//				  Mit STR_MUST(iSign) wird abgebrochen wenn ein Zeichen
//				  nicht zur Gruppe gehrt.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//				  STR_MUST(iSign) schalte die Art der Gruppe um.
//	iMax		: Ist die maximale Suchlnge im Text.
//	iJump		: Ist ein Jump-Zeichen bei dem das nchste bersprungen wird.
//  Ergibt die erste Position nach dem Zeichen im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strlchar(int iMode,const char *pString,int iSign,int iMax,int iJump)
{
int iPos;


	if((iSign&0xFF000000)==STR_MUST(0))
		{
		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwchara(pString[iPos],iMode)==0)break;
			if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
			if(pString[iPos]==(char)iSign){iPos++;break;}
			}
		}
	else{
		iMode ^= 0x80000000;

		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwchara(pString[iPos],iMode))break;
			if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
			if(pString[iPos]==(char)iSign){iPos++;break;}
			}
		}


return iPos;
}

int	_cdecl strlchar(int iMode,const wchar_t *pString,int iSign,int iMax,int iJump)
{
int iPos;


	if((iSign&0xFF000000)==STR_MUST(0))
		{
		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwcharw(pString[iPos],iMode)==0)break;
			if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
			if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
			}
		}
	else{
		iMode ^= 0x80000000;

		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwcharw(pString[iPos],iMode))break;
			if(pString[iPos]==(char)iJump){iPos++;if(iPos>=iMax)break;continue;}
			if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
			}
		}


return iPos;
}


//****************************************************************************
//*
//*		strlchar
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	iMode		: Bestimmt die Gruppe bei der abgebrochen wird.
//				  Mit STR_MUST(iSign) wird abgebrochen wenn ein Zeichen
//				  nicht zur Gruppe gehrt.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//				  STR_MUST(iSign) schalte die Art der Gruppe um.
//	iMax		: Ist die maximale Suchlnge im Text.
//  Ergibt die erste Position nach dem Zeichen im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strlchar(int iMode,const char *pString,int iSign,int iMax)
{
int iPos;


	if((iSign&0xFF000000)==STR_MUST(0))
		{
		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwchara(pString[iPos],iMode)==0)break;
			if(pString[iPos]==(char)iSign){iPos++;break;}
			}
		}
	else{
		iMode ^= 0x80000000;

		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwchara(pString[iPos],iMode))break;
			if(pString[iPos]==(char)iSign){iPos++;break;}
			}
		}


return iPos;
}

int	_cdecl strlchar(int iMode,const wchar_t *pString,int iSign,int iMax)
{
int iPos;


	if((iSign&0xFF000000)==STR_MUST(0))
		{
		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwcharw(pString[iPos],iMode)==0)break;
			if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
			}
		}
	else{
		iMode ^= 0x80000000;

		for(iPos=0;iPos<iMax;iPos++)
			{
			if(strwcharw(pString[iPos],iMode))break;
			if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
			}
		}


return iPos;
}

//****************************************************************************
//*
//*		strlchar
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	iMode		: Bestimmt die Gruppe bei der abgebrochen wird.
//				  Mit STR_MUST(iSign) wird abgebrochen wenn ein Zeichen
//				  nicht zur Gruppe gehrt.
//	pString		: Ist der Zeiger auf den Text in dem gesucht werden soll.
//	iSign		: Ist das Zeichen das im Text gesucht werden soll.
//				  STR_MUST(iSign) schalte die Art der Gruppe um.
//  Ergibt die erste Position nach dem Zeichen im Text, bzw. wenn kein Zeichen
//	gefunden wurde die Stringlnge.
int	_cdecl strlchar(int iMode,const char *pString,int iSign)
{
int iPos;



	if((iSign&0xFF000000)==STR_MUST(0))
		{
		for(iPos=0;;iPos++)
			{
			if(strwchara(pString[iPos],iMode)==0)break;
			if(pString[iPos]==(char)iSign){iPos++;break;}
			}
		}
	else{
		iMode ^= 0x80000000;

		for(iPos=0;;iPos++)
			{
			if(strwchara(pString[iPos],iMode))break;
			if(pString[iPos]==(char)iSign){iPos++;break;}
			}
		}


return iPos;
}

int	_cdecl strlchar(int iMode,const wchar_t *pString,int iSign)
{
int iPos;


	if((iSign&0xFF000000)==STR_MUST(0))
		{
		for(iPos=0;;iPos++)
			{
			if(strwcharw(pString[iPos],iMode)==0)break;
			if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
			}
		}
	else{
		iMode ^= 0x80000000;

		for(iPos=0;;iPos++)
			{
			if(strwcharw(pString[iPos],iMode))break;
			if(pString[iPos]==(unsigned short)iSign){iPos++;break;}
			}
		}


return iPos;
}

//*****************************************************************************
//*
//*		strtoken
//*
//*****************************************************************************
//	Holt das nchste Zahlen-Token aus einem Text.
//	Vorzeichen wie + - ~ ! '...' werden dabei auch bercksichtigt.
//	pText	: Zeiger auf die Formel
//	pValue	: Zeiger auf das Ergebnis
//	pOk		: Zeiger auf die Variabel die angibt ob Fehler vorhanden waren
//	pProc	: Callback fr User-Tokens (bei NULL kein Aufruf)
//	pParam	: Parameter fr User-Token-Callback
//	Ergibt die Anzahl der Zeichen die in Text verarbeitet wurden.
int _cdecl strtoken(const char *pText,int *pValue,int *pOk,TokenCbIA pProc,void *pParam)
{
int	iPos,iOk,iLen;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	*pValue=0;
	iPos=strwlen(pText,0xA0);

	switch(pText[iPos])
		{
	case '\t':
	case ' ':	iPos++;
				break;
	case '(':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strcalc (pText+iPos,pValue,pOk,-1,pProc,pParam);
				iPos+=strwlen (pText+iPos,0xA0);
				if(pText[iPos]!=')'){*pOk=0;break;}
				iPos++;
				break;
	case '+':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				break;
	case '-':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=-*pValue;
				break;
	case '~':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=~*pValue;
				break;
	case '!':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=!*pValue;
				break;
	case '\'':	iPos++;
				*pValue=strtoc(pText+iPos,&iLen)&0xFF;
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				if(pText[iPos]=='\'')
						 iPos++;
				else    *pOk=0;
				break;
	default:	if(pProc && !strwchara(pText[iPos],1))	// User-Token suchen
					{
					   iLen = pProc(pParam,pText+iPos,pValue);
					if(iLen>0){iPos+=iLen;break;}
					}

				*pValue=strtoi(pText+iPos,252,&iLen);
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
		}

	iPos+=strwlen(pText+iPos,0xA0);


return iPos;
}

int _cdecl strtoken(const wchar_t *pText,int *pValue,int *pOk,TokenCbIW pProc,void *pParam)
{
int	iPos,iOk,iLen;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	*pValue=0;
	iPos=strwlen(pText,0xA0);

	switch(pText[iPos])
		{
	case '\t':
	case ' ':	iPos++;
				break;
	case '(':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,-1,pProc,pParam);
				iPos+=strwlen(pText+iPos,0xA0);
				if(pText[iPos]!=')'){*pOk=0;break;}
				iPos++;
				break;
	case '+':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				break;
	case '-':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=-*pValue;
				break;
	case '~':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=~*pValue;
				break;
	case '!':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=!*pValue;
				break;
	case '\'':	iPos++;
				*pValue=strtoc(pText+iPos,&iLen)&0xFF;
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				if(pText[iPos]=='\'')
						iPos++;
				else   *pOk=0;
				break;
	default:	if(pProc && !strwcharw(pText[iPos],1))	// User-Token suchen
					{
					   iLen = pProc(pParam,pText+iPos,pValue);
					if(iLen>0){iPos+=iLen;break;}
					}

				*pValue=strtoi(pText+iPos,252,&iLen);
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
		}

	iPos+=strwlen(pText+iPos,0xA0);


return iPos;
}

int _cdecl strtoken(const char *pText,__int64 *pValue,int *pOk,TokenCbQA pProc,void *pParam)
{
int	iPos,iOk,iLen;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	*pValue=0;
	iPos=strwlen(pText,0xA0);

	switch(pText[iPos])
		{
	case '\t':
	case ' ':	iPos++;
				break;
	case '(':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,-1,pProc,pParam);
				iPos+=strwlen(pText+iPos,0xA0);
				if(pText[iPos]!=')'){*pOk=0;break;}
				iPos++;
				break;
	case '+':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				break;
	case '-':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=-*pValue;
				break;
	case '~':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=~*pValue;
				break;
	case '!':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=!*pValue;
				break;
	case '\'':	iPos++;
				*pValue=strtoc(pText+iPos,&iLen)&0xFF;
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				if(pText[iPos]=='\'')
						iPos++;
				else   *pOk=0;
				break;
	default:	if(pProc && !strwchara(pText[iPos],1))	// User-Token suchen
					{
					   iLen = pProc(pParam,pText+iPos,pValue);
					if(iLen>0){iPos+=iLen;break;}
					}

				*pValue=strtoq(pText+iPos,252,&iLen);
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
		}

	iPos+=strwlen(pText+iPos,0xA0);


return iPos;
}

int _cdecl strtoken(const wchar_t *pText,__int64 *pValue,int *pOk,TokenCbQW pProc,void *pParam)
{
int	iPos,iOk,iLen;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	*pValue=0;
	iPos=strwlen(pText,0xA0);

	switch(pText[iPos])
		{
	case '\t':
	case ' ':	iPos++;
				break;
	case '(':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,-1,pProc,pParam);
				iPos+=strwlen(pText+iPos,0xA0);
				if(pText[iPos]!=')'){*pOk=0;break;}
				iPos++;
				break;
	case '+':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				break;
	case '-':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=-*pValue;
				break;
	case '~':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=~*pValue;
				break;
	case '!':	iPos++;
				iPos+=strwlen (pText+iPos,0xA0);
				iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
				*pValue=!*pValue;
				break;
	case '\'':	iPos++;
				*pValue=strtoc(pText+iPos,&iLen)&0xFF;
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				if(pText[iPos]=='\'')
						iPos++;
				else   *pOk=0;
				break;
	default:	if(pProc && !strwcharw(pText[iPos],1))	// User-Token suchen
					{
					   iLen = pProc(pParam,pText+iPos,pValue);
					if(iLen>0){iPos+=iLen;break;}
					}

				*pValue=strtoq(pText+iPos,252,&iLen);
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
		}

	iPos+=strwlen(pText+iPos,0xA0);


return iPos;
}

int _cdecl strtoken(const char *pText,double *pValue,int *pOk,TokenCbFA pProc,void *pParam)
{
int	iPos,iOk,iLen,iTyp;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	*pValue=0;
	iPos=strwlen(pText,0xA0);

	switch(pText[iPos])
		{
	case '\t':
	case ' ':	iPos++;
				break;
	case '(':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,-1,pProc,pParam);
				iPos+=strwlen(pText+iPos,0xA0);
				if(pText[iPos]!=')'){*pOk=0;break;}
				iPos++;
				break;
	case '+':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,5,pProc,pParam);
				break;
	case '-':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,5,pProc,pParam);
				*pValue=-*pValue;
				break;
	case '~':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,3,pProc,pParam);
				*pValue=(double)(~(__int64)*pValue);
				break;
	case '!':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,1,pProc,pParam);
				*pValue=(*pValue<0 || *pValue>0)? 1:0;
				break;
	case '\'':	iPos++;
				*pValue=strtoc(pText+iPos,&iLen)&0xFF;
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				if(pText[iPos]=='\'')
						iPos++;
				else   *pOk=0;
				break;
	case 'F':
	case 'f':
				if(CMP_LETTER(pText[iPos+1],'l'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'o'))
				if(CMP_LETTER(pText[iPos+4],'r'))
				if(strwchara(pText[iPos+5],0x3000001F)==0)
					{
					iPos+=5;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=floor(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'R':
	case 'r':
				if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'t'))
				if(strwchara(pText[iPos+4],0x00000001))
					{
					double	dBase;
					iPos+=4;
					dBase=strtof(pText+iPos,&iLen,0);
					if(dBase==0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						dBase=1.0;
						}
					iPos+=iLen;
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue<0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=pow(*pValue,1.0/dBase);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'u'))
				if(CMP_LETTER(pText[iPos+3],'n'))
				if(CMP_LETTER(pText[iPos+4],'d'))
				if(strwchara(pText[iPos+5],0x00000001))
					{
					iPos+=5;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=floor(*pValue+0.5);
					break;
					}
				*pOk=0;
				break;
	case 'L':
	case 'l':	if(pText[iPos+1]=='O' || pText[iPos+1]=='o')
				if(pText[iPos+2]=='G' || pText[iPos+1]=='g')
					{
					if(strwchara(pText[iPos+3],0x00000001))
						{
						double	dBase;
						iPos+=3;
						dBase=strtof(pText+iPos,&iLen,0);
						if(dBase==0.0)
							{
							if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
							dBase=1.0;
							}
						iPos+=iLen;
						iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
						if(*pValue<=0.0)
							{
							if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
							*pValue=1.0;
							}
						*pValue=log(*pValue)/log(dBase);
						break;
						}
					if(strwchara(pText[iPos+3],0x3000001F)==0)
						{
						iPos+=3;
						iPos+=strwlen (pText+iPos,0xA0);
						iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
						if(*pValue<=0.0)
							{
							if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
							*pValue=1.0;
							}
						*pValue=log10(*pValue);
						break;
						}
					}
				if(CMP_LETTER(pText[iPos+1],'n'))
				if(strwchara(pText[iPos+2],0x3000001F)==0)
					{
					iPos+=2;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue<=0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=1.0;
						}
					*pValue=log(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'A':
	case 'a':	if(CMP_LETTER(pText[iPos+1],'t'))
				if(CMP_LETTER(pText[iPos+2],'a'))
				if(CMP_LETTER(pText[iPos+3],'n'))
				if(strwchara(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=atan(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'c'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'t'))
				if(strwchara(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=3.1415926535897932384626433832795*0.5-atan(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'c'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'s'))
				if(strwchara(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue>1.0 || *pValue<-1.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=acos(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'s'))
				if(CMP_LETTER(pText[iPos+2],'i'))
				if(CMP_LETTER(pText[iPos+3],'n'))
				if(strwchara(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue>1.0 || *pValue<-1.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=asin(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'b'))
				if(CMP_LETTER(pText[iPos+2],'s'))
				if(strwchara(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=fabs(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'T':
	case 't':	if(CMP_LETTER(pText[iPos+1],'a'))
				if(CMP_LETTER(pText[iPos+2],'n'))
				if(strwchara(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=tan(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'C':
	case 'c':	if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'s'))
				if(strwchara(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=cos(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'t'))
				if(strwchara(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=tan(3.1415926535897932384626433832795*0.5-*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'e'))
				if(CMP_LETTER(pText[iPos+2],'i'))
				if(CMP_LETTER(pText[iPos+3],'l'))
				if(strwchara(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=4;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=ceil(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'S':
	case 's':	if(CMP_LETTER(pText[iPos+1],'i'))
				if(CMP_LETTER(pText[iPos+2],'n'))
				if(strwchara(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=sin(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'q'))
				if(CMP_LETTER(pText[iPos+2],'r'))
				if(CMP_LETTER(pText[iPos+3],'t'))
				if(strwchara(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=4;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue<0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=sqrt(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'P':
	case 'p':	if(CMP_LETTER(pText[iPos+1],'i'))
				if(strwchara(pText[iPos+2],0x3000001F)==0)
					{
					*pValue = 3.1415926535897932384626433832795;
					iPos+=2;
					iPos+=strwlen(pText+iPos,0xA0);
					break;
					}
				*pOk=0;
				break;
	case 'E':
	case 'e':	if(strwchara(pText[iPos+1],0x3000001F)==0)
					{
					*pValue = 2.7182818284590452353602874713527;
					iPos+=2;
					iPos+=strwlen(pText+iPos,0xA0);
					break;
					}
				*pOk=0;
				break;

	default:	if(!strwchara(pText[iPos],0x06000201))
					{
					*pOk=0;
					break;
					}

				iLen = strflen(pText+iPos,256,0x03,&iTyp);

				if(iTyp&0xFFFF0000)
					{
					*pValue=strtof(pText+iPos,&iLen,0x07);
					iPos+=iLen;
					if(iLen<=0)*pOk=0;
					break;
					}

				*pValue=(double)strtoq(pText+iPos,252,&iLen);
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				break;
		}

	if(pProc && !*pOk)									// User-Token suchen
		{
		   iLen = pProc(pParam,pText+iPos,pValue);
		if(iLen>0){iPos+=iLen;*pOk=1;}
		}


	iPos+=strwlen(pText+iPos,0xA0);


return iPos;
}

int _cdecl strtoken(const wchar_t *pText,double *pValue,int *pOk,TokenCbFW pProc,void *pParam)
{
int	iPos,iOk,iLen,iTyp;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	*pValue=0;
	iPos=strwlen(pText,0xA0);

	switch(pText[iPos])
		{
	case '\t':
	case ' ':	iPos++;
				break;
	case '(':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,-1,pProc,pParam);
				iPos+=strwlen(pText+iPos,0xA0);
				if(pText[iPos]!=')'){*pOk=0;break;}
				iPos++;
				break;
	case '+':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,5,pProc,pParam);
				break;
	case '-':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,5,pProc,pParam);
				*pValue=-*pValue;
				break;
	case '~':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,3,pProc,pParam);
				*pValue=(double)(~(__int64)*pValue);
				break;
	case '!':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=strcalc(pText+iPos,pValue,pOk,1,pProc,pParam);
				*pValue=(*pValue<0 || *pValue>0)? 1:0;
				break;
	case '\'':	iPos++;
				*pValue=strtoc(pText+iPos,&iLen)&0xFF;
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				if(pText[iPos]=='\'')
						iPos++;
				else   *pOk=0;
				break;
	case 'F':
	case 'f':
				if(CMP_LETTER(pText[iPos+1],'l'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'o'))
				if(CMP_LETTER(pText[iPos+4],'r'))
				if(strwcharw(pText[iPos+5],0x3000001F)==0)
					{
					iPos+=5;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=floor(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'R':
	case 'r':
				if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'t'))
				if(strwcharw(pText[iPos+4],0x00000001))
					{
					double	dBase;
					iPos+=4;
					dBase=strtof(pText+iPos,&iLen,0);
					if(dBase==0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						dBase=1.0;
						}
					iPos+=iLen;
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue<0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=pow(*pValue,1.0/dBase);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'u'))
				if(CMP_LETTER(pText[iPos+3],'n'))
				if(CMP_LETTER(pText[iPos+4],'d'))
				if(strwcharw(pText[iPos+5],0x00000001))
					{
					iPos+=5;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=floor(*pValue+0.5);
					break;
					}
				*pOk=0;
				break;
	case 'L':
	case 'l':	if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'g'))
					{
					if(strwcharw(pText[iPos+3],0x00000001))
						{
						double	dBase;
						iPos+=3;
						dBase=strtof(pText+iPos,&iLen,0);
						if(dBase==0.0)
							{
							if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
							dBase=1.0;
							}
						iPos+=iLen;
						iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
						if(*pValue<=0.0)
							{
							if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
							*pValue=1.0;
							}
						*pValue=log(*pValue)/log(dBase);
						break;
						}
					if(strwcharw(pText[iPos+3],0x3000001F)==0)
						{
						iPos+=3;
						iPos+=strwlen (pText+iPos,0xA0);
						iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
						if(*pValue<=0.0)
							{
							if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
							*pValue=1.0;
							}
						*pValue=log10(*pValue);
						break;
						}
					}
				if(CMP_LETTER(pText[iPos+1],'n'))
				if(strwcharw(pText[iPos+2],0x3000001F)==0)
					{
					iPos+=2;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue<=0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=1.0;
						}
					*pValue=log(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'A':
	case 'a':	if(CMP_LETTER(pText[iPos+1],'t'))
				if(CMP_LETTER(pText[iPos+2],'a'))
				if(CMP_LETTER(pText[iPos+3],'n'))
				if(strwcharw(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=atan(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'c'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'t'))
				if(strwcharw(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=3.1415926535897932384626433832795*0.5-atan(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'c'))
				if(CMP_LETTER(pText[iPos+2],'o'))
				if(CMP_LETTER(pText[iPos+3],'s'))
				if(strwcharw(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue>1.0 || *pValue<-1.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=acos(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'s'))
				if(CMP_LETTER(pText[iPos+2],'i'))
				if(CMP_LETTER(pText[iPos+3],'n'))
				if(strwcharw(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue>1.0 || *pValue<-1.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=asin(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'b'))
				if(CMP_LETTER(pText[iPos+2],'s'))
				if(strwcharw(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=fabs(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'T':
	case 't':	if(CMP_LETTER(pText[iPos+1],'a'))
				if(CMP_LETTER(pText[iPos+2],'n'))
				if(strwcharw(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=tan(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'C':
	case 'c':	if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'s'))
				if(strwcharw(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=cos(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'o'))
				if(CMP_LETTER(pText[iPos+2],'t'))
				if(strwcharw(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=tan(3.1415926535897932384626433832795*0.5-*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'e'))
				if(CMP_LETTER(pText[iPos+2],'i'))
				if(CMP_LETTER(pText[iPos+3],'l'))
				if(strwcharw(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=4;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=ceil(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'S':
	case 's':	if(CMP_LETTER(pText[iPos+1],'i'))
				if(CMP_LETTER(pText[iPos+2],'n'))
				if(strwcharw(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					*pValue=sin(*pValue);
					break;
					}
				if(CMP_LETTER(pText[iPos+1],'q'))
				if(CMP_LETTER(pText[iPos+2],'r'))
				if(CMP_LETTER(pText[iPos+3],'t'))
				if(strwcharw(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=4;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=strtoken(pText+iPos,pValue,pOk,pProc,pParam);
					if(*pValue<0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;break;}
						*pValue=0.0;
						}
					*pValue=sqrt(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'P':
	case 'p':	if(CMP_LETTER(pText[iPos+1],'i'))
				if(strwcharw(pText[iPos+2],0x3000001F)==0)
					{
					*pValue = 3.1415926535897932384626433832795;
					iPos+=2;
					iPos+=strwlen (pText+iPos,0xA0);
					break;
					}
				*pOk=0;
				break;
	case 'E':
	case 'e':	if(strwcharw(pText[iPos+1],0x3000001F)==0)
					{
					*pValue = 2.7182818284590452353602874713527;
					iPos+=2;
					iPos+=strwlen (pText+iPos,0xA0);
					break;
					}
				*pOk=0;
				break;

	default:	if(!strwcharw(pText[iPos],0x06000201))
					{
					*pOk=0;
					break;
					}

				iLen = strflen(pText+iPos,256,0x03,&iTyp);

				if(iTyp&0xFFFF0000)
					{
					*pValue=strtof(pText+iPos,&iLen,0x07);
					iPos+=iLen;
					if(iLen<=0)*pOk=0;
					break;
					}

				*pValue=(double)strtoq(pText+iPos,252,&iLen);
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				break;
		}

	if(pProc && !*pOk)									// User-Token suchen
		{
		   iLen = pProc(pParam,pText+iPos,pValue);
		if(iLen>0){iPos+=iLen;*pOk=1;}
		}

	iPos+=strwlen(pText+iPos,0xA0);


return iPos;
}

//*****************************************************************************
//*
//*		strcalc
//*
//*****************************************************************************
//	Berechnet eine Formel die in einem Text enthalten ist
//	pText	: Zeiger auf die Formel
//	pValue	: Zeiger auf das Ergebnis
//	pOk		: Zeiger auf die Variabel die angibt ob Fehler vorhanden waren
//	iLevel	: Hhe des Berechnugs-Levels ( -3=Auswahl -2=Start  -1 Klammer 0=Boolsch
//											1=Vergleich 2=Logik 3=Schiebeoperation
//											4=Strichrechnung 5=Punktrechnung )
//	Ergibt die Anzahl der Zeichen die in Text verarbeitet wurden
#ifdef _DEBUG
int _cdecl strcalc(const char *pText,int *pValue,int *pOk,TokenCbIA pProc,void *pParam)
{
	return strcalc(pText,pValue,pOk,-2,pProc,pParam);
}

int _cdecl strcalc(const char *pText,__int64 *pValue,int *pOk,TokenCbQA pProc,void *pParam)
{
	return strcalc(pText,pValue,pOk,-2,pProc,pParam);
}

int _cdecl strcalc(const char *pText,double *pValue,int *pOk,TokenCbFA pProc,void *pParam)
{
	return strcalc(pText,pValue,pOk,-2,pProc,pParam);
}

int _cdecl strcalc(const char *pText,TokenCbIA pProc,void *pParam)
{
int iValue=0;

	strcalc(pText,&iValue,0,-2,pProc,pParam);

return iValue;
}

double _cdecl strcalcf(const char *pText,TokenCbFA pProc,void *pParam)
{
double dValue=0;

	strcalc(pText,&dValue,0,-2,pProc,pParam);

return dValue;
}

intq _cdecl	 strcalc64(const char *pText,TokenCbQA pProc,void *pParam)
{
intq iValue=0;

	strcalc(pText,&iValue,0,-2,pProc,pParam);

return iValue;
}

#endif

int _cdecl strcalc(const char *pText,int *pValue,int *pOk,int iLevel,TokenCbIA pProc,void *pParam)
{
int		iPos,iOk,iTemp,iValue2;
char	cChar;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	iPos=strtoken(pText,pValue,pOk,pProc,pParam);
	if(!*pOk)return iPos;

	while(pText[iPos])
		{
		switch(pText[iPos])
			{
		case '\t':
		case ' ':	iPos++;
					break;
		case ')':	if(iLevel<-1)*pOk=0;
					return iPos;
		case ':':	if(iLevel!=-3)*pOk=0;
					return iPos;
		case '!':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue != iValue2;
					if(!*pOk)return iPos;
					break;
		case '=':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue == iValue2;
					if(!*pOk)return iPos;
					break;
		case '<':	cChar=pText[iPos+1];
					if(cChar=='<')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue<<=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue <= iValue2;
					else		  *pValue=*pValue <  iValue2;
					if(!*pOk)return iPos;
					break;
		case '>':	cChar=pText[iPos+1];
					if(cChar=='>')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue>>=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue >= iValue2;
					else		  *pValue=*pValue >  iValue2;
					if(!*pOk)return iPos;
					break;
		case '|':	if(pText[iPos+1]=='|')
						{
						if(iLevel>0)return iPos;		 iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue || iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue|=iValue2;
					if(!*pOk)return iPos;
					break;
		case '&':	if(pText[iPos+1]=='&')
						{
						if(iLevel>0)return iPos;	     iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue && iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue&=iValue2;
					if(!*pOk)return iPos;
					break;
		case '^':	if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue^=iValue2;
					if(!*pOk)return iPos;
					break;
		case '+':	if(iLevel>4)return iPos;		 iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue+=iValue2;
					if(!*pOk)return iPos;
					break;
		case '-':	if(iLevel>4)return iPos;		 iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue-=iValue2;
					if(!*pOk)return iPos;
					break;
		case '*':	if(iLevel>5)return iPos;		 iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue*=iValue2;
					if(!*pOk)return iPos;
					break;
		case '/':	if(iLevel>5)return iPos;		 iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue/=iValue2;
					if(!*pOk)return iPos;
					break;
		case '%':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue%=iValue2;
					if(!*pOk)return iPos;
					break;
		case '?':	if(iLevel>7)return iPos;
					iPos++;
					iTemp=STR_NO_CHECK;
					if(*pValue)
						{
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						*pValue=iValue2;
						iPos+=strcalc(pText+iPos,&iValue2,&iTemp,-2,pProc,pParam);
						}
					else{
						iPos+=strcalc (pText+iPos,&iValue2,&iTemp,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-2,pProc,pParam);
						*pValue=iValue2;
						}
					if(!*pOk)return iPos;
					break;
		default:	*pOk=0;
					return iPos;
			}

		}


return iPos;
}

int _cdecl strcalc(const char *pText,__int64 *pValue,int *pOk,int iLevel,TokenCbQA pProc,void *pParam)
{
int		iTemp,iPos,iOk;
__int64	iValue2;
char	cChar;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	iPos=strtoken(pText,pValue,pOk,pProc,pParam);
	if(!*pOk)return iPos;

	while(pText[iPos])
		{
		switch(pText[iPos])
			{
		case '\t':
		case ' ':	iPos++;
					break;
		case ')':	if(iLevel<-1)*pOk=0;
					return iPos;
		case ':':	if(iLevel!=-3)*pOk=0;
					return iPos;
		case '!':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue != iValue2;
					if(!*pOk)return iPos;
					break;
		case '=':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue == iValue2;
					if(!*pOk)return iPos;
					break;
		case '<':	cChar=pText[iPos+1];
					if(cChar=='<')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue<<=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue <= iValue2;
					else		  *pValue=*pValue <  iValue2;
					if(!*pOk)return iPos;
					break;
		case '>':	cChar=pText[iPos+1];
					if(cChar=='>')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue>>=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue >= iValue2;
					else		  *pValue=*pValue >  iValue2;
					if(!*pOk)return iPos;
					break;
		case '|':	if(pText[iPos+1]=='|')
						{
						if(iLevel>0)return iPos;		 iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue || iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue|=iValue2;
					if(!*pOk)return iPos;
					break;
		case '&':	if(pText[iPos+1]=='&')
						{
						if(iLevel>0)return iPos;	     iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue && iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue&=iValue2;
					if(!*pOk)return iPos;
					break;
		case '^':	if(iLevel>2)return iPos;		    iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue^=iValue2;
					if(!*pOk)return iPos;
					break;
		case '+':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue+=iValue2;
					if(!*pOk)return iPos;
					break;
		case '-':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue-=iValue2;
					if(!*pOk)return iPos;
					break;
		case '*':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue*=iValue2;
					if(!*pOk)return iPos;
					break;
		case '/':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue/=iValue2;
					if(!*pOk)return iPos;
					break;
		case '%':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue%=iValue2;
					if(!*pOk)return iPos;
					break;
		case '?':	if(iLevel>7)return iPos;
					iPos++;
					iTemp=STR_NO_CHECK;
					if(*pValue)
						{
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						*pValue=iValue2;
						iPos+=strcalc(pText+iPos,&iValue2,&iTemp,-2,pProc,pParam);
						}
					else{
						iPos+=strcalc (pText+iPos,&iValue2,&iTemp,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-2,pProc,pParam);
						*pValue=iValue2;
						}
					if(!*pOk)return iPos;
					break;
		default:	*pOk=0;
					return iPos;
			}

		}


return iPos;
}

int _cdecl strcalc(const char *pText,double *pValue,int *pOk,int iLevel,TokenCbFA pProc,void *pParam)
{
int		iTemp,iPos,iOk;
double	dValue2;
char	cChar;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	iPos=strtoken(pText,pValue,pOk,pProc,pParam);
	if(!*pOk)return iPos;

	while(pText[iPos])
		{
		switch(pText[iPos])
			{
		case '\t':
		case ' ':	iPos++;
					break;
		case ')':	if(iLevel<-1)*pOk=0;
					return iPos;
		case ':':	if(iLevel!=-3)*pOk=0;
					return iPos;
		case '!':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue=*pValue != dValue2;
					if(!*pOk)return iPos;
					break;
		case '=':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue=*pValue == dValue2;
					if(!*pOk)return iPos;
					break;
		case '<':	cChar=pText[iPos+1];
					if(cChar=='<')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&dValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue = (double)((__int64)*pValue<<(int)dValue2);
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue <= dValue2;
					else		  *pValue=*pValue <  dValue2;
					if(!*pOk)return iPos;
					break;
		case '>':	cChar=pText[iPos+1];
					if(cChar=='>')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&dValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue = (double)((__int64)*pValue>>(int)dValue2);
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue >= dValue2;
					else		  *pValue=*pValue >  dValue2;
					if(!*pOk)return iPos;
					break;
		case '|':	if(pText[iPos+1]=='|')
						{
						if(iLevel>0)return iPos;		 iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&dValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue=*pValue || dValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&dValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = (double)((__int64)*pValue|(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '&':	if(pText[iPos+1]=='&')
						{
						if(iLevel>0)return iPos;	     iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&dValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue=*pValue && dValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&dValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = (double)((__int64)*pValue&(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '^':	if(iLevel>2)return iPos;		    iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&dValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = (double)((__int64)*pValue^(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '+':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&dValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue+=dValue2;
					if(!*pOk)return iPos;
					break;
		case '-':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&dValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue-=dValue2;
					if(!*pOk)return iPos;
					break;
		case '*':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&dValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue*=dValue2;
					if(!*pOk)return iPos;
					break;
		case '/':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&dValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(dValue2==0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						dValue2=1.0;
						}
					*pValue/=dValue2;
					if(!*pOk)return iPos;
					break;
		case '%':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&dValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(dValue2==0.0){*pOk=0;return iPos;}
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						dValue2=1.0;
						}
					*pValue = fmod(*pValue,dValue2);
					if(!*pOk)return iPos;
					break;
		case '\'':	if(iLevel>6)return iPos;		    iPos++;
					if(iLevel<6)iPos+=strcalc (pText+iPos,&dValue2,pOk,7,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = pow(*pValue,dValue2);
					break;
		case '':	if(iLevel>6)return iPos;		    iPos++;
					dValue2 = *pValue;
					*pValue = dValue2*dValue2;
					break;
		case '':	if(iLevel>6)return iPos;		    iPos++;
					dValue2 = *pValue;
					*pValue = dValue2*dValue2*dValue2;
					break;
		case '?':	if(iLevel>7)return iPos;
					iPos++;
					iTemp=STR_NO_CHECK;
					if(*pValue)
						{
						iPos+=strcalc(pText+iPos,&dValue2,pOk,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						*pValue=dValue2;
						iPos+=strcalc(pText+iPos,&dValue2,&iTemp,-2,pProc,pParam);
						}
					else{
						iPos+=strcalc (pText+iPos,&dValue2,&iTemp,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						iPos+=strcalc(pText+iPos,&dValue2,pOk,-2,pProc,pParam);
						*pValue=dValue2;
						}
					if(!*pOk)return iPos;
					break;
		default:	*pOk=0;
					return iPos;
			}

		}


return iPos;
}

#ifdef _DEBUG
int _cdecl strcalc(const wchar_t *pText,int *pValue,int *pOk,TokenCbIW pProc,void *pParam)
{
	return strcalc(pText,pValue,pOk,-2,pProc,pParam);
}

int _cdecl strcalc(const wchar_t *pText,__int64 *pValue,int *pOk,TokenCbQW pProc,void *pParam)
{
	return strcalc(pText,pValue,pOk,-2,pProc,pParam);
}

int _cdecl strcalc(const wchar_t *pText,double *pValue,int *pOk,TokenCbFW pProc,void *pParam)
{
	return strcalc(pText,pValue,pOk,-2,pProc,pParam);
}

int _cdecl strcalc(const wchar_t *pText,TokenCbIW pProc,void *pParam)
{
int iValue=0;

	strcalc(pText,&iValue,0,-2,pProc,pParam);

return iValue;
}

double _cdecl strcalcf(const wchar_t *pText,TokenCbFW pProc,void *pParam)
{
double dValue=0;

	strcalc(pText,&dValue,0,-2,pProc,pParam);

return dValue;
}

intq _cdecl	 strcalc64(const wchar_t *pText,TokenCbQW pProc,void *pParam)
{
intq iValue=0;

	strcalc(pText,&iValue,0,-2,pProc,pParam);

return iValue;
}
#endif

int _cdecl strcalc(const wchar_t  *pText,int *pValue,int *pOk,int iLevel,TokenCbIW pProc,void *pParam)
{
int				iTemp,iPos,iOk,iValue2;
unsigned short	uChar;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	iPos=strtoken(pText,pValue,pOk,pProc,pParam);
	if(!*pOk)return iPos;

	while(pText[iPos])
		{

		switch(pText[iPos])
			{
		case '\t':
		case ' ':	iPos++;
					break;
		case ')':	if(iLevel<-1)*pOk=0;
					return iPos;
		case ':':	if(iLevel!=-3)*pOk=0;
					return iPos;
		case '!':	uChar=pText[iPos+1];
					if(uChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue != iValue2;
					if(!*pOk)return iPos;
					break;
		case '=':	uChar=pText[iPos+1];
					if(uChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue == iValue2;
					if(!*pOk)return iPos;
					break;
		case '<':	uChar=pText[iPos+1];
					if(uChar=='<')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue<<=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(uChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(uChar=='=')*pValue=*pValue <= iValue2;
					else		  *pValue=*pValue <  iValue2;
					if(!*pOk)return iPos;
					break;
		case '>':	uChar=pText[iPos+1];
					if(uChar=='>')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue>>=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(uChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(uChar=='=')*pValue=*pValue >= iValue2;
					else		  *pValue=*pValue >  iValue2;
					if(!*pOk)return iPos;
					break;
		case '|':	if(pText[iPos+1]=='|')
						{
						if(iLevel>0)return iPos;		 iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue || iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue|=iValue2;
					if(!*pOk)return iPos;
					break;
		case '&':	if(pText[iPos+1]=='&')
						{
						if(iLevel>0)return iPos;	     iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue && iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue&=iValue2;
					if(!*pOk)return iPos;
					break;
		case '^':	if(iLevel>2)return iPos;		    iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue^=iValue2;
					if(!*pOk)return iPos;
					break;
		case '+':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue+=iValue2;
					if(!*pOk)return iPos;
					break;
		case '-':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue-=iValue2;
					if(!*pOk)return iPos;
					break;
		case '*':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue*=iValue2;
					if(!*pOk)return iPos;
					break;
		case '/':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue/=iValue2;
					if(!*pOk)return iPos;
					break;
		case '%':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue%=iValue2;
					if(!*pOk)return iPos;
					break;
		case '?':	if(iLevel>7)return iPos;
					iPos++;
					iTemp=STR_NO_CHECK;
					if(*pValue)
						{
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						*pValue=iValue2;
						iPos+=strcalc(pText+iPos,&iValue2,&iTemp,-2,pProc,pParam);
						}
					else{
						iPos+=strcalc (pText+iPos,&iValue2,&iTemp,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-2,pProc,pParam);
						*pValue=iValue2;
						}
					if(!*pOk)return iPos;
					break;
		default:	*pOk=0;
					return iPos;
			}

		}


return iPos;
}

int _cdecl strcalc(const wchar_t  *pText,__int64 *pValue,int *pOk,int iLevel,TokenCbQW pProc,void *pParam)
{
int			iTemp,iPos,iOk;
__int64		iValue2;
wchar_t		uChar;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	iPos=strtoken(pText,pValue,pOk,pProc,pParam);
	if(!*pOk)return iPos;

	while(pText[iPos])
		{
		switch(pText[iPos])
			{
		case '\t':
		case ' ':	iPos++;
					break;
		case ')':	if(iLevel<-1)*pOk=0;
					return iPos;
		case ':':	if(iLevel!=-3)*pOk=0;
					return iPos;
		case '!':	uChar=pText[iPos+1];
					if(uChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue != iValue2;
					if(!*pOk)return iPos;
					break;
		case '=':	uChar=pText[iPos+1];
					if(uChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue=*pValue == iValue2;
					if(!*pOk)return iPos;
					break;
		case '<':	uChar=pText[iPos+1];
					if(uChar=='<')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue<<=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(uChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(uChar=='=')*pValue=*pValue <= iValue2;
					else		  *pValue=*pValue <  iValue2;
					if(!*pOk)return iPos;
					break;
		case '>':	uChar=pText[iPos+1];
					if(uChar=='>')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&iValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue>>=iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(uChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&iValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(uChar=='=')*pValue=*pValue >= iValue2;
					else		  *pValue=*pValue >  iValue2;
					if(!*pOk)return iPos;
					break;
		case '|':	if(pText[iPos+1]=='|')
						{
						if(iLevel>0)return iPos;		 iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue || iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue|=iValue2;
					if(!*pOk)return iPos;
					break;
		case '&':	if(pText[iPos+1]=='&')
						{
						if(iLevel>0)return iPos;	     iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&iValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
						*pValue=*pValue && iValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue&=iValue2;
					if(!*pOk)return iPos;
					break;
		case '^':	if(iLevel>2)return iPos;		    iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&iValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue^=iValue2;
					if(!*pOk)return iPos;
					break;
		case '+':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue+=iValue2;
					if(!*pOk)return iPos;
					break;
		case '-':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&iValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue-=iValue2;
					if(!*pOk)return iPos;
					break;
		case '*':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					*pValue*=iValue2;
					if(!*pOk)return iPos;
					break;
		case '/':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue/=iValue2;
					if(!*pOk)return iPos;
					break;
		case '%':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&iValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&iValue2,pOk  ,pProc,pParam);
					if(iValue2==0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						iValue2=1;
						}
					*pValue%=iValue2;
					if(!*pOk)return iPos;
					break;
		case '?':	if(iLevel>7)return iPos;
					iPos++;
					iTemp=STR_NO_CHECK;
					if(*pValue)
						{
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						*pValue=iValue2;
						iPos+=strcalc(pText+iPos,&iValue2,&iTemp,-2,pProc,pParam);
						}
					else{
						iPos+=strcalc (pText+iPos,&iValue2,&iTemp,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						iPos+=strcalc(pText+iPos,&iValue2,pOk,-2,pProc,pParam);
						*pValue=iValue2;
						}
					if(!*pOk)return iPos;
					break;
		default:	*pOk=0;
					return iPos;
			}

		}


return iPos;
}

int _cdecl strcalc(const wchar_t *pText,double *pValue,int *pOk,int iLevel,TokenCbFW pProc,void *pParam)
{
int		iTemp,iPos,iOk;
double	dValue2;
wchar_t	cChar;


	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	iPos=strtoken(pText,pValue,pOk,pProc,pParam);
	if(!*pOk)return iPos;

	while(pText[iPos])
		{
		switch(pText[iPos])
			{
		case '\t':
		case ' ':	iPos++;
					break;
		case ')':	if(iLevel<-1)*pOk=0;
					return iPos;
		case ':':	if(iLevel!=-3)*pOk=0;
					return iPos;
		case '!':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue=*pValue != dValue2;
					if(!*pOk)return iPos;
					break;
		case '=':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					if(iLevel>1)return iPos;		 iPos+=2;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue=*pValue == dValue2;
					if(!*pOk)return iPos;
					break;
		case '<':	cChar=pText[iPos+1];
					if(cChar=='<')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&dValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue = (double)((__int64)*pValue<<(int)dValue2);
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue <= dValue2;
					else		  *pValue=*pValue <  dValue2;
					if(!*pOk)return iPos;
					break;
		case '>':	cChar=pText[iPos+1];
					if(cChar=='>')
						{
						if(iLevel>3)return iPos;		 iPos+=2;
						if(iLevel<3)iPos+=strcalc (pText+iPos,&dValue2,pOk,4,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue = (double)((__int64)*pValue>>(int)dValue2);
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		 iPos+=(cChar=='=')? 2:1;
					if(iLevel<1)iPos+=strcalc (pText+iPos,&dValue2,pOk,2,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(cChar=='=')*pValue=*pValue >= dValue2;
					else		  *pValue=*pValue >  dValue2;
					if(!*pOk)return iPos;
					break;
		case '|':	if(pText[iPos+1]=='|')
						{
						if(iLevel>0)return iPos;		 iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&dValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue=*pValue || dValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&dValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = (double)((__int64)*pValue|(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '&':	if(pText[iPos+1]=='&')
						{
						if(iLevel>0)return iPos;	     iPos+=2;
						if(iLevel<0)iPos+=strcalc (pText+iPos,&dValue2,pOk,1,pProc,pParam);
						else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
						*pValue=*pValue && dValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		 iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&dValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = (double)((__int64)*pValue&(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '^':	if(iLevel>2)return iPos;		    iPos++;
					if(iLevel<2)iPos+=strcalc (pText+iPos,&dValue2,pOk,3,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = (double)((__int64)*pValue^(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '+':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&dValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue+=dValue2;
					if(!*pOk)return iPos;
					break;
		case '-':	if(iLevel>4)return iPos;		    iPos++;
					if(iLevel<4)iPos+=strcalc (pText+iPos,&dValue2,pOk,5,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue-=dValue2;
					if(!*pOk)return iPos;
					break;
		case '*':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&dValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue*=dValue2;
					if(!*pOk)return iPos;
					break;
		case '/':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&dValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(dValue2==0.0){*pOk=0;return iPos;}
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						dValue2=1.0;
						}
					*pValue/=dValue2;
					if(!*pOk)return iPos;
					break;
		case '%':	if(iLevel>5)return iPos;		    iPos++;
					if(iLevel<5)iPos+=strcalc (pText+iPos,&dValue2,pOk,6,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					if(dValue2==0.0)
						{
						if(*pOk!=STR_NO_CHECK){*pOk=0;return iPos;}
						dValue2=1.0;
						}
					*pValue = fmod(*pValue,dValue2);
					if(!*pOk)return iPos;
					break;
		case '\'':	if(iLevel>6)return iPos;		    iPos++;
					if(iLevel<6)iPos+=strcalc (pText+iPos,&dValue2,pOk,7,pProc,pParam);
					else	    iPos+=strtoken(pText+iPos,&dValue2,pOk  ,pProc,pParam);
					*pValue = pow(*pValue,dValue2);
					break;
		case '':	if(iLevel>6)return iPos;		    iPos++;
					dValue2 = *pValue;
					*pValue = dValue2*dValue2;
					break;
		case '':	if(iLevel>6)return iPos;		    iPos++;
					dValue2 = *pValue;
					*pValue = dValue2*dValue2*dValue2;
					break;
		case '?':	if(iLevel>7)return iPos;		    iPos++;
					iPos++;
					iTemp=STR_NO_CHECK;
					if(*pValue)
						{
						iPos+=strcalc(pText+iPos,&dValue2,pOk,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						*pValue=dValue2;
						iPos+=strcalc(pText+iPos,&dValue2,&iTemp,-2,pProc,pParam);
						}
					else{
						iPos+=strcalc (pText+iPos,&dValue2,&iTemp,-3,pProc,pParam);
						if(pText[iPos]!=':'){*pOk=0;return iPos;}
						iPos++;
						iPos+=strcalc(pText+iPos,&dValue2,pOk,-2,pProc,pParam);
						*pValue=dValue2;
						}
					if(!*pOk)return iPos;
					break;
		default:	*pOk=0;
					return iPos;
			}

		}


return iPos;
}

//*****************************************************************************
//*
//*		strtobits
//*
//*****************************************************************************
//	Erzeugt ein Bitfeld in einem Speicherbereich (32 Byte / 256 Bits)
//	pBits		: Zeiger auf den Speicherbereich in dem das Bitfeld
//				  gespeichert wird.
//	pList		: Ist eine Liste aus Zeichen die in das Bitfeld eingetragen
//				  werden sollen.
//	iMode		: Bit0 -->  0=Bitfeld lschen      1=Bitfeld nicht lschen
//				  Bit1 -->	0=Zeichen setzen Bits  0=Zeichen lschen Bits
void _cdecl strtobits(void *pBits,const char *pList,int iMode)
{
#ifdef	NO_STR_ASM


	if(!(iMode&1))memset(pBits,0,32);

	if(iMode&2)
		{	while(*pList){CLR_BIT_PTR(pBits,pList);pList++;}		}
	else{	while(*pList){SET_BIT_PTR(pBits,pList);pList++;}		}


//********************* Mit Assembler ******************************************
#else

_asm	{
		mov		ebx,pList					;// EBX = Zeiger auf pList
		mov  	edx,pBits					;//	EDX = Zeiger auf Bitfeld
		xor  	ecx,ecx						;//		*

		test  	iMode,1						;// Bittfeld lschen
		jnz		noclear						;//		*

		mov  	[edx   ],ecx				;// pBits lschen
		mov  	[edx+ 4],ecx				;//		*
		mov  	[edx+ 8],ecx				;//		*
		mov  	[edx+12],ecx				;//		*
		mov  	[edx+16],ecx				;//		*
		mov  	[edx+20],ecx				;//		*
		mov  	[edx+24],ecx				;//		*
		mov  	[edx+28],ecx				;//		*

noclear:

		test  	iMode,2						;// Bits lschen ?
		jnz  	sc2							;//		*

sc1:	mov  	cl,[ebx]					;//	Hinzufgen
		jcxz 	ende						;//		*
		bts  	[edx],ecx					;//	Bit setzen
		inc  	ebx							;// Auf nchstes Zeichen
		jmp  	sc1							;//		*

sc2:	mov  	cl,[ebx]					;//	Lschen
		jcxz 	ende						;//		*
		btr  	[edx],ecx					;//	Bit lschen
		inc  	ebx							;// Auf nchstes Zeichen
		jmp  	sc2							;//		*

ende:
		}

#endif

}

void _cdecl strtobits(void *pBits,const wchar_t *pList,int iMode)
{
#ifdef	NO_STR_ASM

unsigned 	uVal;

	if(!(iMode&1))memset(pBits,0,32);

	if(iMode&2)
		{
		while(*pList)
			{
			   uVal = *pList;
			if(uVal>255)uVal=255;
			CLR_BIT_PTR(pBits,&uVal);
			pList++;
			}
		}
	else{
		while(*pList)
			{
			   uVal = *pList;
			if(uVal>255)uVal=255;
			SET_BIT_PTR(pBits,pList);
			pList++;
			}
		}


//********************* Mit Assembler ******************************************
#else

_asm	{
		mov		ebx,pList					;// EBX = Zeiger auf pList
		mov  	edx,pBits					;//	EDX = Zeiger auf Bitfeld
		xor  	ecx,ecx						;//		*
		xor  	eax,255						;//		*

		test  	iMode,1						;// Bittfeld lschen
		jnz		noclear						;//		*

		mov  	[edx   ],ecx				;// pBits lschen
		mov  	[edx+ 4],ecx				;//		*
		mov  	[edx+ 8],ecx				;//		*
		mov  	[edx+12],ecx				;//		*
		mov  	[edx+16],ecx				;//		*
		mov  	[edx+20],ecx				;//		*
		mov  	[edx+24],ecx				;//		*
		mov  	[edx+28],ecx				;//		*

noclear:

		test  	iMode,2						;// Bits lschen ?
		jnz  	sc2							;//		*

sc1:	mov  	cx,[ebx]					;//	Hinzufgen
		test    ch,ch						;//		*
		cmovnz	ecx,eax						;//		*
		jcxz 	ende						;//		*
		bts  	[edx],ecx					;//	Bit setzen
		add  	ebx,1						;// Auf nchstes Zeichen
		jmp  	sc1							;//		*

sc2:	mov  	cx,[ebx]					;//	Lschen
		test    ch,ch						;//		*
		cmovnz	ecx,eax						;//		*
		jcxz 	ende						;//		*
		btr  	[edx],ecx					;//	Bit lschen
		add  	ebx,1						;// Auf nchstes Zeichen
		jmp  	sc2							;//		*

ende:
		}

#endif

}

//****************************************************************************
//*
//*		strbits
//*
//****************************************************************************
//	Sucht das erste nicht Auftreten eines Zeichens aus dem Bitfeld in einem Text
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//  Ergibt die Position des ersten nicht gefundenen Zeichens im Text.
//  Wenn immer ein Zeichen gefunden wird, dann wird die Textlnge zurckgegeben.
int	_cdecl strbits(const char *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;

	for(iPos=0;pString[iPos];iPos++)
		{
		if(!GET_BIT_PTR(pBits,pString+iPos))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		}

#endif


}

int	_cdecl strbits(const wchar_t *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;

	for(iPos=0;pString[iPos];iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(!GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif


}

//****************************************************************************
//*
//*		strbits
//*
//****************************************************************************
//	Sucht das erste nicht Auftreten eines Zeichens aus dem Bitfeld in einem Text
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//	iMax		: Ist die maximale Suchlnge im Text
//  Ergibt die Position des ersten nicht gefundenen Zeichens im Text.
//  Wenn immer ein Zeichen gefunden wird, dann wird die Textlnge zurckgegeben.
int	_cdecl strbits(const char *pString,const void *pBits,int iMax)
{
//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;pString[iPos] && iPos<iMax;iPos++)
		{
		if(!GET_BIT_PTR(pBits,pString+iPos))break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*
		inc  	esi							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc   	sc2							;//		*
		dec	 	esi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		}

#endif
}

int	_cdecl strbits(const wchar_t *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;

	for(iPos=0;pString[iPos] && iPos<iMax;iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(!GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*
		inc  	esi							;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc   	sc2							;//		*
		dec	 	esi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif
}

//****************************************************************************
//*
//*		strcbits
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens aus dem Bitfeld in einem Text
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//  Ergibt die Position des ersten gefundenen Zeichens im Text.
//  Wird kein Zeichen gefunden wird die Textlnge zurckgegeben.
int	_cdecl strcbits(const char *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;

	for(iPos=0;pString[iPos];iPos++)
		{
		if(GET_BIT_PTR(pBits,pString+iPos))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		}

#endif


}

int	_cdecl strcbits(const wchar_t *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;

	for(iPos=0;pString[iPos];iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif


}

//****************************************************************************
//*
//*		strcbits
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens aus dem Bitfeld in einem Text
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//	iMax		: Ist die maximale Suchlnge im Text
//  Ergibt die Position des ersten gefundenen Zeichens im Text.
//  Wird kein Zeichen gefunden wird die Textlnge zurckgegeben.
int	_cdecl strcbits(const char *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;pString[iPos] && iPos<iMax;iPos++)
		{
		if(GET_BIT_PTR(pBits,pString+iPos))break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*
		inc  	esi							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc   	sc2							;//		*
		dec	 	esi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		}

#endif
}

int	_cdecl strcbits(const wchar_t *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;

	for(iPos=0;pString[iPos] && iPos<iMax;iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*
		inc  	esi							;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc   	sc2							;//		*
		dec	 	esi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif
}

//****************************************************************************
//*
//*		strlbits
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//  Ergibt die Position des nach dem ersten gefundenen Zeichen im Text.
//  Wird kein Zeichen gefunden wird die Textlnge zurckgegeben.
int	_cdecl strlbits(const char *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;pString[iPos];iPos++)
		{
		if(GET_BIT_PTR(pBits,pString+iPos)){iPos++;break;}
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

		inc	 	eax							;//	Position erhhen
sc2:	sub  	eax,pString					;// Position berechnen
		}

#endif


}

int	_cdecl strlbits(const wchar_t *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;


	for(iPos=0;pString[iPos];iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(GET_BIT_PTR(pBits,&uValue)){iPos++;break;}
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

		add	 	eax,2						;//	Position erhhen
sc2:	sub  	eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif


}

//****************************************************************************
//*
//*		strlbits
//*
//****************************************************************************
//	Sucht die Position nach dem ersten Auftreten eines Zeichens in einem Text.
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//	iMax		: Ist die maximale Suchlnge im Text
//  Ergibt die Position des nach dem ersten gefundenen Zeichen im Text.
//  Wird kein Zeichen gefunden wird die Textlnge zurckgegeben.
int	_cdecl strlbits(const char *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;pString[iPos] && iPos<iMax;iPos++)
		{
		if(GET_BIT_PTR(pBits,pString+iPos)){iPos++;break;}
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*
		inc  	esi							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		dec	 	esi							;// Maximal Lnge erreicht
		jz   	sc2							;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

		inc	 eax							;//	Position erhhen
sc2:	sub  eax,pString					;// Position berechnen
		}

#endif


}

int	_cdecl strlbits(const wchar_t *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;


	for(iPos=0;pString[iPos] && iPos<iMax;iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(GET_BIT_PTR(pBits,&uValue)){iPos++;break;}
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*
		inc  	esi							;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		jcxz 	sc2							;// Textende ?
		dec	 	esi							;// Maximal Lnge erreicht
		jz   	sc2							;//		*
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

		add		eax,2						;//	Position erhhen
sc2:	sub		eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif


}

//****************************************************************************
//*
//*		strnbits
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens aus dem Bitfeld in einem Text
//	Bei einem Nullzeichen wird dabei nicht abgebrochen.
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//	iMax		: Ist die maximale Suchlnge im Text
//  Ergibt die Position des ersten gefundenen Zeichens im Text.
//  Wird kein Zeichen gefunden wird die Textlnge zurckgegeben.
int	_cdecl strnbits(const char *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;


	for(iPos=0;iPos<iMax;iPos++)
		{
		if(GET_BIT_PTR(pBits,pString+iPos))break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*
		inc  	esi							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc   	sc2							;//		*
		dec	 	esi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		}

#endif
}

int	_cdecl strnbits(const wchar_t *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;

	for(iPos=0;iPos<iMax;iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		esi,iMax					;// Lnge laden
		mov  	eax,pString					;//	EAX = Zeiger
		cmp	 	esi,0						;//		*
		jle	 	sc2							;//		*

		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*
		inc  	esi							;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc   	sc2							;//		*
		dec	 	esi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	sub  	eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif
}

//****************************************************************************
//*
//*		strnbits
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens aus dem Bitfeld in einem Text
//	Bei einem Nullzeichen wird dabei nicht abgebrochen.
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//  Ergibt die Position des ersten gefundenen Zeichens im Text.
//  Wird kein Zeichen gefunden wird die Textlnge zurckgegeben.
int	_cdecl strnbits(const char *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;

	for(iPos=0;;iPos++)
		{
		if(GET_BIT_PTR(pBits,pString+iPos))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		dec  	eax							;//		*

sc1:	inc  	eax							;// EAX korrigieren
		mov  	cl,[eax]					;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

		sub  	eax,pString					;// Position berechnen
		}

#endif


}

int	_cdecl strnbits(const wchar_t *pString,const void *pBits)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;

	for(iPos=0;;iPos++)
		{
		   uValue = pString[iPos];
		if(uValue>255)uValue=255;
		if(GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		eax,pString					;//	EAX = Zeiger
		mov  	edx,pBits					;//	EAX = Zeiger
		xor  	ecx,ecx						;// Bits Null setzen
		sub  	eax,2						;//		*
		mov		ebx,255						;//		*

sc1:	add  	eax,2						;// EAX korrigieren
		mov  	cx,[eax]					;//		*
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc  	sc1							;//		*

		sub  	eax,pString					;// Position berechnen
		shr		eax,1						;//		*
		}

#endif


}

//****************************************************************************
//*
//*		strdbits
//*
//****************************************************************************
//	Sucht das erste Auftreten eines Zeichens aus dem Bitfeld in einem Text
//	Die Suche luft rckwrts beginnend bei pString[-1].
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//	iMax		: Ist die maximale Suchlnge im Text
//  Ergibt die Position des ersten gefundenen Zeichens im Text.
//  Wird kein Zeichen gefunden wird iMax zurckgegeben.
int	_cdecl strdbits(const char *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;

	pString--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(GET_BIT_PTR(pBits,pString-iPos))break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		edi,iMax					;// Lnge laden
		xor		eax,eax						;// Offset auf 0
		cmp	 	edi,0						;//		*
		jle	 	sc2							;//		*
		mov  	esi,pString					;//	ESI = Zeiger
		xor		ecx,ecx						;//		*
		mov  	edx,pBits					;//	EDI = Zeiger
		dec		esi							;// ESI auf [-1]

sc1:	mov  	cl,[esi+eax]				;//	Byte laden
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc   	sc2							;//		*
		dec		eax							;//		*
		dec	 	edi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	neg		eax							;// Ergebnis = -Offset
		}

#endif
}

int	_cdecl strdbits(const wchar_t *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;


	pString--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		   uValue = pString[-iPos];
		if(uValue>255)uValue=255;
		if(GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		edi,iMax					;// Lnge laden
		xor		eax,eax						;// Offset auf 0
		cmp	 	edi,0						;//		*
		jle	 	sc2							;//		*
		mov  	esi,pString					;//	ESI = Zeiger
		xor		ecx,ecx						;//		*
		mov  	edx,pBits					;//	EDI = Zeiger
		sub		esi,2						;// ESI auf [-1]
		mov		ebx,255						;//		*

sc1:	mov  	cx,[esi+eax*2]				;//	Word laden
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jc   	sc2							;//		*
		dec		eax							;//		*
		dec	 	edi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	neg		eax							;// Ergebnis = -Offset
		}


#endif
}

//****************************************************************************
//*
//*		strbbits
//*
//****************************************************************************
//	Sucht das erste nicht Auftreten eines Zeichens aus dem Bitfeld in einem Text.
//	Die Suche luft rckwrts beginnend bei pString[-1].
//	pString		: Ist der Zeiger auf den Text in dem gesucht wird.
//	pBits		: Ist der Zeiger auf ein 32 Byte Bitfeld
//	iMax		: Ist die maximale Suchlnge im Text
//  Ergibt die Position des ersten gefundenen Zeichens im Text.
//  Wird kein Zeichen gefunden wird iMax zurckgegeben.
int	_cdecl strbbits(const char *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int iPos;

	pString--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(!GET_BIT_PTR(pBits,pString-iPos))break;
		}


return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		edi,iMax					;// Lnge laden
		xor		eax,eax						;// Offset auf 0
		cmp	 	edi,0						;//		*
		jle	 	sc2							;//		*
		mov  	esi,pString					;//	ESI = Zeiger
		xor		ecx,ecx						;//		*
		mov  	edx,pBits					;//	EDI = Zeiger
		dec		esi							;// ESI auf [-1]

sc1:	mov  	cl,[esi+eax]				;//	Byte laden
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc   	sc2							;//		*
		dec		eax							;//		*
		dec	 	edi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	neg		eax							;// Ergebnis = -Offset
		}

#endif
}

int	_cdecl strbbits(const wchar_t *pString,const void *pBits,int iMax)
{

//********************* Ohne Assembler *****************************************
#ifdef	NO_STR_ASM

int			iPos;
unsigned 	uValue;


	pString--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		   uValue = pString[-iPos];
		if(uValue>255)uValue=255;
		if(!GET_BIT_PTR(pBits,&uValue))break;
		}

return iPos;

//********************* Mit Assembler ******************************************
#else
__asm	{
		mov		edi,iMax					;// Lnge laden
		xor		eax,eax						;// Offset auf 0
		cmp	 	edi,0						;//		*
		jle	 	sc2							;//		*
		mov  	esi,pString					;//	ESI = Zeiger
		xor		ecx,ecx						;//		*
		mov  	edx,pBits					;//	EDI = Zeiger
		sub		esi,2						;// ESI auf [-1]
		mov		ebx,255						;//		*

sc1:	mov  	cx,[esi+eax*2]				;//	Word laden
		test	ch,ch						;//		*
		cmovnz	ecx,ebx						;//		*
		bt   	[edx],ecx					;//	Bit gesetzt ?
		jnc   	sc2							;//		*
		dec		eax							;//		*
		dec	 	edi							;// Maximal Lnge erreicht
		jnz  	sc1							;//		*

sc2:	neg		eax							;// Ergebnis = -Offset
		}


#endif
}

//*****************************************************************************
//*
//*		strtab
//*
//*****************************************************************************
//	Fgt einen Leerraum ein der ber die Spalten iFrom/iTo geht.
//	Am Ende wird des Leeraums wird ein Nullzeichen angehngt.
//	pString		: Ist der Zeiger auf die Position wo der Leerraum eingefgt wird.
//	iFrom		: Ist die Spalte in der der Leerraum beginnt.
//	iTo			: Ist die Spalte in der der Leerraum endet.
//	iTab		: Ist die Spaltenbreite fr ein Tab-Zeichen
//				  Ist iTab <= 0 so werden nur SPACE eingefgt
//	Ergibt die Anzahl der Zeichen die in den Leerraum kopiert wurden.
int	_cdecl strtab(char *pString,int iFrom,int iTo,int iTab)
{
int		iCount,iColFrom,iColTo ;




	if(iTo<=iFrom)
		{
		pString[0]=0;
		return 0;
		}

	if(iTab<=0) 								// Nur Space anhngen
		{
		iTo-=iFrom;
		memset(pString,' ',iTo);
		return  iTo;
		}

	iColFrom = iFrom/iTab;
	iColTo   = iTo  /iTab;

	if(iColFrom==iColTo  )						// Kein Sprung in andere Tab-Spalte
		{
		iTo   -= iFrom;
		iCount = iTo;

		for(;iTo;iTo--)
			{
			pString[0]=' ';
			pString++;
			}

		pString[0]=0;

		return iCount;
		}

	iCount=1;			   						// Auf nchste Tabspalte
	if((iFrom+1)%iTab)
			pString[0]='\t';
	else	pString[0]=' ';
			pString++;


	iColTo -= iColFrom+1;						// Tabspalten berspringen
	iCount += iColTo;

	for(;iColTo;iColTo--)
		{
		pString[0]='\t';
		pString++;
		}

	iTo    %= iTab;								// Die letzten SPACE einfgen
	iCount += iTo;

	for(;iTo;iTo--)
		{
		pString[0]=' ';
		pString++;
		}

	pString[0]=0;


return iCount;
}

int	_cdecl strtab(wchar_t *pString,int iFrom,int iTo,int iTab)
{
int		iCount,iColFrom,iColTo ;




	if(iTo<=iFrom)
		{
		pString[0]=0;
		return 0;
		}

	if(iTab<=0) 								// Nur Space anhngen
		{
		iTo-=iFrom;
		memset(pString,' ',iTo);
		return  iTo;
		}

	iColFrom = iFrom/iTab;
	iColTo   = iTo  /iTab;

	if(iColFrom==iColTo  )						// Kein Sprung in andere Tab-Spalte
		{
		iTo   -= iFrom;
		iCount = iTo;

		for(;iTo;iTo--)
			{
			pString[0]=' ';
			pString++;
			}

		pString[0]=0;

		return iCount;
		}

	iCount=1;			   						// Auf nchste Tab-Spalte
	if((iFrom+1)%iTab)
			pString[0]='\t';
	else	pString[0]=' ';
			pString++;


	iColTo -= iColFrom+1;						// Tabspalten berspringen
	iCount += iColTo;

	for(;iColTo;iColTo --)
		{
		pString[0]='\t';
		pString++;
		}

	iTo    %= iTab;								// Die letzten SPACE einfgen
	iCount += iTo;

	for(;iTo;iTo--)
		{
		pString[0]=' ';
		pString++;
		}

	pString[0]=0;


return iCount;
}

//*****************************************************************************
//*
//*		strtablen / strtablenutf8 / strtablenutf16
//*
//*****************************************************************************
//	Berechnet die Spalte wo ein Text mit Tabs aufhrt.
//	iLen		: Ist die Lnge des Textes
//	pText		: Ist der Zeiger auf den Text
//	iTab		: Ist die Tabulatorbreite.
//	iRaw		: Ist die Spalte in der der Text beginnt
//	Ergibt die Spalte vom Textende.
int	_cdecl strtablen(const char *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;pText[iPos];iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw++;
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

int	_cdecl strtablen(const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;pText[iPos];iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw++;																									
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

int	_cdecl strtablenutf8(const char *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;pText[iPos];iPos+=iNum)
		{
		iNum = utf8length(pText+iPos);

		if(pText[iPos]!='\t')
			{
			iRaw++;
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

int	_cdecl strtablenutf16(const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;pText[iPos];iPos+=iNum)
		{
		iNum = utf16length(pText+iPos);

		if(pText[iPos]!='\t')
			{
			iRaw++;
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

int	_cdecl strtablen(int iLen,const char *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;iPos<iLen;iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw++;
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

int	_cdecl strtablen(int iLen,const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;iPos<iLen;iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw++;
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

int	_cdecl strtablenutf8(int iLen,const char *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;iPos<iLen;iPos+=iNum)
		{
		iNum = utf8length(pText+iPos,iLen-iPos);

		if(pText[iPos]!='\t')
			{
			iRaw++;
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

int	_cdecl strtablenutf16(int iLen,const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;iPos<iLen;iPos+=iNum)
		{
		iNum = utf16length(pText+iPos,iLen-iPos);

		if(pText[iPos]!='\t')
			{
			iRaw++;
			continue;
			}

		iRaw += iTab;
		iRaw -=	iRaw%iTab;
		}


return iRaw;
}

//*****************************************************************************
//*
//*		strtabpos / strtabposutf8 / strtabposutf16
//*
//*****************************************************************************
//	Berechnet die Position einer Spalte in einem Text mit Tabs.
//	iEnd		: Ist die Spalte die gesucht wird
//	iLen		: Ist die Lnge des Textes
//	pText		: Ist der Zeiger auf den Text
//	iTab		: Ist die Tabulatorbreite.
//	iRaw		: Ist die Spalte in der der Text beginnt
//	Ergibt die Spalte vom Textende.
int	_cdecl strtabpos(int iEnd,const char *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;pText[iPos];iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

int	_cdecl strtabpos(int iEnd,const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;pText[iPos];iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

int	_cdecl strtabposutf8(int iEnd,const char *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;pText[iPos];iPos+=iNum)
		{
		iNum = utf8length(pText+iPos);

		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

int	_cdecl strtabposutf16(int iEnd,const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;pText[iPos];iPos+=iNum)
		{
		iNum = utf16length(pText+iPos);

		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

int	_cdecl strtabpos(int iEnd,int iLen,const char *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;iPos<iLen;iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

int	_cdecl strtabpos(int iEnd,int iLen,const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;


	for(iPos=0;iPos<iLen;iPos++)
		{
		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

int	_cdecl strtabposutf8(int iEnd,int iLen,const char *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;iPos<iLen;iPos+=iNum)
		{
		iNum = utf8length(pText+iPos,iLen-iPos);

		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

int	_cdecl strtabposutf16(int iEnd,int iLen,const wchar_t *pText,int iTab,int iRaw)
{
int		iPos;
int		iNum;


	for(iPos=0;iPos<iLen;iPos+=iNum)
		{
		iNum = utf16length(pText+iPos,iLen-iPos);

		if(pText[iPos]!='\t')
			{
			iRaw += 1;
			}
		else{
			iRaw += iTab;
			iRaw -= iRaw%iTab;
			}

		if(iRaw>iEnd)break;
		}


return iPos;
}

//*****************************************************************************
//*
//*		strlcut
//*
//*****************************************************************************
//	Kopiert einen Text ohne Leerzeichen und Zeilenumbrchen am Anfang und Ende.
//	pString		: Ist der Zeiger auf den Zielpuffer in den der Text eingefgt wird.
//	pSource		: Ist der Zeiger auf den Text der kopiert werden soll
//	iMax		: Ist die Gre des Textpuffers samt Nullzeichen.
//	Ergibt die neue Lnge vom Text ohne Nullzeichen
int _cdecl strlcut(char *pString,const char *pSource,int iMax)
{
int			iSize;


	if(iMax<=0)return 0;							// Ist der Puffer zu klein ?


	pSource += strwlen (pSource,0xE0);				// Leerzeichen am Anfang berspringen
	iSize    = strnlen (pSource,iMax-1);
	iSize   -= strbwlen(pSource+iSize,0xE0,iSize);	// Leerzeichen am Ende berspringen

	memmove(pString,pSource,iSize);
	pString[iSize]=0;


return iSize;
}

int _cdecl strlcut(wchar_t *pString,const wchar_t *pSource,int iMax)
{
int			iSize;


	if(iMax<=0)return 0;							// Ist der Puffer zu klein ?


	pSource += strwlen (pSource,0xE0);				// Leerzeichen am Anfang berspringen
	iSize    = strnlen (pSource,iMax-1);
	iSize   -= strbwlen(pSource+iSize,0xE0,iSize);	// Leerzeichen am Ende berspringen

	memmove(pString,pSource,iSize*2);
	pString[iSize]=0;


return iSize;
}

//*****************************************************************************
//*
//*		strcut
//*
//*****************************************************************************
//	Kopiert einen Text ohne Leerzeichen und Zeilenumbrchen am Anfang und Ende.
//	pString		: Ist der Zeiger auf den Zielpuffer in den der Text eingefgt wird.
//	pSource		: Ist der Zeiger auf den Text der kopiert werden soll
//	iMax		: Ist die Gre des Textpuffers samt Nullzeichen.
//	Ergibt die neue Lnge vom Text ohne Nullzeichen
int _cdecl strcut(char *pString,const char *pSource)
{
int iSize;


	pSource += strwlen (pSource,0xE0);				// Leerzeichen am Anfang berspringen
	iSize    = (int)strlen(pSource);
	iSize   -= strbwlen(pSource+iSize,0xE0,iSize);	// Leerzeichen am Ende berspringen

	memmove(pString,pSource,iSize);
	pString[iSize]=0;



return iSize;
}

int _cdecl strcut(wchar_t *pString,const wchar_t *pSource)
{
int iSize;


	pSource += strwlen (pSource,0xE0);				// Leerzeichen am Anfang berspringen
	iSize    = (int)wcslen(pSource);
	iSize   -= strbwlen(pSource+iSize,0xE0,iSize);	// Leerzeichen am Ende berspringen

	memmove(pString,pSource,iSize*2);
	pString[iSize]=0;



return iSize;
}

#ifdef 	_DEBUG
int  _cdecl  strcut  (char           *pString         ){return strcut (pString,pString     );}
int  _cdecl  strlcut (char           *pString,int iLen){return strlcut(pString,pString,iLen);}
int  _cdecl  strcut  (wchar_t *pString         ){return strcut (pString,pString     );}
int  _cdecl  strlcut (wchar_t *pString,int iLen){return strlcut(pString,pString,iLen);}
#endif

//*****************************************************************************
//*
//*		strpre
//*
//*****************************************************************************
//	Fgt einen Text vor einer Zeichenfolge ein.
//	pString		: Ist der Zeiger auf die Zeichenfolge vor die der Text eingefgt wird.
//	pSource		: Ist der Zeiger auf den Text eingefgt werden soll.
//	Ergibt die Lnge der neuen Zeichenfolge ohne Nullzeichen.
int _cdecl strpre(char *pString,const char *pSource)
{
int iLen1,iLen2;


	iLen1=(int)strlen(pString)+1;
	iLen2=(int)strlen(pSource);

	memmove(pString+iLen2,pString,iLen1);
	memcpy (pString      ,pSource,iLen2);


return iLen1+iLen2;
}

int _cdecl strpre(wchar_t *pString,const wchar_t *pSource)
{
int iLen1,iLen2;


	iLen1=(int)wcslen(pString)+1;
	iLen2=(int)wcslen(pSource);

	memmove(pString+iLen2,pString,iLen1*2);
	memcpy (pString      ,pSource,iLen2*2);


return iLen1+iLen2;
}

//*****************************************************************************
//*
//*		strlpre
//*
//*****************************************************************************
//	Fgt einen Text vor einer Zeichenfolge ein.
//	pString		: Ist der Zeiger auf die Zeichenfolge vor die der Text eingefgt wird.
//	pSource		: Ist der Zeiger auf den Text eingefgt werden soll.
//	iMax		: Ist die maximale Lnge die die Zeichenfolge ereichen kann.
//	Ergibt die Lnge der neuen Zeichenfolge ohne Nullzeichen.

int _cdecl strlpre(char *pString,const char *pSource,int iMax)
{
int iLen1,iLen2;


	if(iMax<=0)return 0;

	iLen1=(int)strlen(pString);
	iLen2=(int)strlen(pSource);

	if(iLen2>=iMax)
		{
		iMax--;
		memcpy(pString,pSource,iMax);
		pString[iMax]=0;
		return  iMax;
		}

	if(iLen1+iLen2>=iMax)
		{
		iLen1=iMax-iLen2-1;
		}


	memmove(pString+iLen2,pString,iLen1);
	memcpy (pString      ,pSource,iLen2);

	pString[iLen1+iLen2]=0;


return iLen1+iLen2;
}


int _cdecl strlpre(wchar_t *pString,const wchar_t *pSource,int iMax)
{
int iLen1,iLen2;


	if(iMax<=0)return 0;

	iLen1=(int)wcslen(pString);
	iLen2=(int)wcslen(pSource);

	if(iLen2>=iMax)
		{
		iMax--;
		memcpy(pString,pSource,iMax*2);
		pString[iMax]=0;
		return  iMax;
		}

	if(iLen1+iLen2>=iMax)
		{
		iLen1=iMax-iLen2-1;
		}


	memmove(pString+iLen2,pString,iLen1*2);
	memcpy (pString      ,pSource,iLen2*2);

	pString[iLen1+iLen2]=0;


return iLen1+iLen2;
}

//*****************************************************************************
//*
//*		strfilter
//*
//*****************************************************************************
//	Dateinamen konvertieren
//	pTarget	: Ist der Puffer in den der konvertierte Dateiname gespeichert wird.
//	pSource	: Ist der Zeiger auf den Dateiname der konvertiert werden soll.
//			  Wenn pTarget nicht angegeben ist wird der Dateiname direkt aus
//			  dem Puffer pString geholt.
//	iMode	: Bestimmt die Art wie der Dateiname konvertieren wird:
//					SFT_BIG			  0x0000  Alles gros  schreiben
//					SFT_LITTLE		  0x0001  Alles klein schreiben
//					SFT_BEGIN		  0x0002  Anfangsbuchstaben gro schreiben
//					SFT_POINTLITTLE	  0x0200  Den letzten Punktkleinmachen
//					SFT_SCANPATH	  0x0400  Pfadeintrge prfen
//					SFT_SCANWORDS	  0x0800  Worte einzeln prfen
//					SFT_LETLITTLE	  0x1000  Kleingeschriebene Worte kleinlassen
//					SFT_ANSI		  0x2000  Auch deutsche Umlaute und ANSI
//					SFT_ONLYLITTLE	  0x4000  Nur wenn alles klein  ist filtern
//					SFT_ONLYBIG		  0x8000  Nur wenn alles gro   ist filtern
//					SFT_ONLYSAME	  0xC000  Nur wenn alles gleich ist filtern
void strfilter(char *pTarget,const char *pSource,int iMode)
{
int				i,j,k=0,iOk,iStart,iPos,iOnly,iLet,iPoint;
unsigned		uMaskU,uMaskL,uMaskI,uMaskV,uMaskE;
unsigned		u,uLast;



	if(iMode&SFT_ANSI)
		{
		uMaskU=0x10000002;					// Grobuchstaben
		uMaskL=0x20000004;					// Kleinbuchstaben ohne '?'
		uMaskI=0x30000004;					// Kleinbuchstaben mit  '?'
		uMaskV=0x30000007;					// Buchstaben und Zahlen
		}
	else{
		uMaskU=0x00000002;
		uMaskL=0x00000004;
		uMaskI=0x00000004;
		uMaskV=0x00000007;
		}

	iLet   =  iMode&SFT_LETLITTLE;
	iOnly  =  iMode&SFT_ONLYSAME;
	uMaskE =  0;
	iPoint = -1;

	iPos=0;


//*****************************************************************************

	if((iMode&(SFT_SCANWORDS|SFT_SCANPATH))==0)
		{
		switch(iOnly)
			{
		case SFT_ONLYBIG:					// Nur ndern wenn alles Grobuchstaben ?

			for(j=0;;j++)
				{
					u=(unsigned char)pSource[j];
				if(!u)break;
				if(aStrwlenTab[u]&uMaskI)
					{
					if(pTarget!=pSource)strcpy(pTarget+iPos,pSource+iPos);
					return;
					}
				}
			break;

		case SFT_ONLYLITTLE:				// Nur ndern wenn alles Kleinbuchstaben ?

			for(j=0;;j++)
				{
					u=(unsigned char)pSource[j];
				if(!u)break;
				if(aStrwlenTab[u]&uMaskU)
					{
					if(pTarget!=pSource)strcpy(pTarget+iPos,pSource+iPos);
					return;
					}
				}
			break;

		case SFT_ONLYSAME:					// Nur ndern wenn Alles gleich ?

			for(k=0,j=0;;j++)
				{
					u=(unsigned char)pSource[j];
				if(!u)break;
				if(aStrwlenTab[u]&uMaskU)k|=1;
				if(aStrwlenTab[u]&uMaskI)k|=2;
				if(k==3)break;
				}

			if(k==0 || k==3)
				{
				if(pTarget!=pSource)strcpy(pTarget+iPos,pSource+iPos);
				return;
				}

			break;
			}

		iOnly=0;
		}
	else{
		if(iMode&SFT_SCANWORDS)
				uMaskE=0xCFF8FFF8|(uMaskV&0x30000000);
		else	uMaskE=0x80300000;
		}


//*****************************************************************************

	for(;;)
		{
		iStart=iPos;

		for(;;iPos++)						// berspringe Sonderzeichen
			{
			   u =(unsigned char)pSource[iPos];
			if(u==0  )break;
			if(u=='.')iPoint=iPos;
			if(aStrwlenTab[u]&uMaskV)break;
			}

		if(pTarget!=pSource)memcpy(pTarget+iStart,pSource+iStart,iPos-iStart);
		if(u==0)break;

		iOk=1;
		j=iPos;

		switch(iOnly)
			{

		case SFT_ONLYBIG:					// Nur ndern wenn gro ?

			for(;;)
				{
				u=(unsigned char)pSource[j];
				i=aStrwlenTab[u];
				if(  i&uMaskE)break;
				if(!(i&uMaskI)){j++;continue;}
				for(j++;;j++)
					{
					u=(unsigned char)pSource[j];
					if(aStrwlenTab[u]&uMaskE)break;
					}
				iOk=0;
				break;
				}

			break;

		case SFT_ONLYLITTLE:				// Nur ndern wenn klein ?

			for(;;)
				{
				u=(unsigned char)pSource[j];
				i=aStrwlenTab[u];
				if(  i&uMaskE)break;
				if(!(i&uMaskU)){j++;continue;}
				for(j++;;j++)
					{
					u=(unsigned char)pSource[j];
					if(aStrwlenTab[u]&uMaskE)break;
					}
				iOk=0;
				break;
				}

			break;

		case SFT_ONLYSAME:					// Nur ndern wenn gleich ?

			for(;;)
				{
				u=(unsigned char)pSource[j];
				i=aStrwlenTab[u];
				if(i&uMaskE)break;
				if(i&uMaskU)k|=1;
				if(i&uMaskI)k|=2;
				if(k!=3){j++;continue;}
				for(j++;;j++)
					{
					u=(unsigned char)pSource[j];
					if(aStrwlenTab[u]&uMaskE)break;
					}
				break;
				}

			if(k==0 || k==3)iOk=0;
			break;

		default:j=(int)strlen(pSource+iPos)+iPos;
			}

		if(!iOk)
			{
			memcpy(pTarget+iPos,pSource+iPos,j-iPos);
			iPos=j;
			continue;
			}

//*****************************************************************************

		if(iMode&SFT_BIG)						// In Grobuchstaben umwandeln
			{
			for(;iPos<j;iPos++)
				{
				u=(unsigned char)pSource[iPos];
				if(u==223 && ((iMode&SFT_ANSI))){pTarget[iPos]='S';  continue;}
				if(aStrwlenTab[u]&uMaskL){pTarget[iPos]=(char)(u-32);continue;}

				pTarget[iPos]=(char)u;
				}
			continue;
			}

		if(iMode&SFT_LITTLE)					// In Grobuchstaben umwandeln
			{
			for(;iPos<j;iPos++)
				{
				u=(unsigned char)pSource[iPos];
				if(aStrwlenTab[u]&uMaskU){pTarget[iPos]=(char)(u+32);continue;}
				pTarget[iPos]=(char)u;
				}
			continue;
			}


		for(uLast=' ';iPos<j;iPos++)			// Anfansbuchstaben gro Rest klein
			{
			   u=(unsigned char)pSource[iPos];
			if(u=='.')iPoint=iPos;

			if(aStrwlenTab[uLast]&uMaskV)
				{
				if(aStrwlenTab[u]&uMaskU){pTarget[iPos]=(char)(u+32);uLast=u;continue;}
				}
			else if(iLet==0)
				{
				if(u==223 && ((iMode&SFT_ANSI))){pTarget[iPos]='S';  uLast=u;continue;}
				if(aStrwlenTab[u]&uMaskL){pTarget[iPos]=(char)(u-32);uLast=u;continue;}
				}

			pTarget[iPos]=(char)u;
			uLast=u;
			}

		}

//*****************************************************************************


	if((iMode&SFT_POINTLITTLE) && iPoint>=0)	// Zeichen nach letztdem Punkt klein schreiben
		{
		iPoint++;
		u=pTarget[iPoint];
		if(aStrwlenTab[u]&uMaskU)pTarget[iPoint]=(char)(u+32);
		}


	pTarget[iPos]=0;

}

void strfilter(wchar_t *pTarget,const wchar_t *pSource,int iMode)
{
#define 		MASK(u,m)  ((u<256)? aStrwlenTab[u]&(m):0)
int				i,j,k=0,iOk,iStart,iPos,iOnly,iLet,iPoint;
unsigned		uMaskU,uMaskL,uMaskI,uMaskV,uMaskE;
unsigned		u,uLast;



	if(iMode&SFT_ANSI)
		{
		uMaskU=0x10000002;					// Grobuchstaben
		uMaskL=0x20000004;					// Kleinbuchstaben ohne '?'
		uMaskI=0x20000204;					// Kleinbuchstaben mit  '?'
		uMaskV=0x30000207;					// Buchstaben und Zahlen
		}
	else{
		uMaskU=0x00000002;
		uMaskL=0x00000004;
		uMaskI=0x00000004;
		uMaskV=0x00000007;
		}

	iLet   =  iMode&SFT_LETLITTLE;
	iOnly  =  iMode&SFT_ONLYSAME;
	uMaskE =  0;
	iPoint = -1;

	iPos=0;

//*****************************************************************************

	if((iMode&(SFT_SCANWORDS|SFT_SCANPATH))==0)
		{
		switch(iOnly)
			{
		case SFT_ONLYBIG:					// Nur ndern wenn alles Grobuchstaben ?

			for(j=0;;j++)
				{
					u=pSource[j];
				if(!u)break;
				if(MASK(u,uMaskI))
					{
					if(pTarget!=pSource)wcscpy(pTarget+iPos,pSource+iPos);
					return;
					}
				}
			break;

		case SFT_ONLYLITTLE:				// Nur ndern wenn alles Kleinbuchstaben ?

			for(j=0;;j++)
				{
					u=pSource[j];
				if(!u)break;
				if(MASK(u,uMaskU))
					{
					if(pTarget!=pSource)wcscpy(pTarget+iPos,pSource+iPos);
					return;
					}
				}
			break;

		case SFT_ONLYSAME:					// Nur ndern wenn Alles gleich ?

			for(k=0,j=0;;j++)
				{
					u=pSource[j];
				if(!u)break;
				if(MASK(u,uMaskU))k|=1;
				if(MASK(u,uMaskI))k|=2;
				if(k==3)break;
				}

			if(k==0 || k==3)
				{
				if(pTarget!=pSource)wcscpy(pTarget+iPos,pSource+iPos);
				return;
				}

			break;
			}

		iOnly=0;
		}
	else{
		if(iMode&SFT_SCANWORDS)
				uMaskE=0xCFF8FFF8|(uMaskV&0x30000000);
		else	uMaskE=0x80300000;
		}


//*****************************************************************************

	for(;;)
		{
		iStart=iPos;

		for(;;iPos++)						// berspringe Sonderzeichen
			{
			   u =pSource[iPos];
			if(u==0  )break;
			if(u=='.')iPoint=iPos;
			if(MASK(u,uMaskV))break;
			}

		if(pTarget!=pSource)memcpy(pTarget+iStart,pSource+iStart,iPos-iStart);
		if(u==0)break;

		iOk=1;
		j=iPos;

		switch(iOnly)
			{

		case SFT_ONLYBIG:					// Nur ndern wenn gro ?

			for(;;)
				{
				u=pSource[j];
				i=(u<256)? aStrwlenTab[u]:0x40000000;
				if(  i&uMaskE)break;
				if(!(i&uMaskI)){j++;continue;}
				for(j++;;j++){u=pSource[j];if(MASK(u,uMaskE))break;}
				iOk=0;
				break;
				}

			break;

		case SFT_ONLYLITTLE:				// Nur ndern wenn klein ?

			for(;;)
				{
				u=pSource[j];
				i=(u<256)? aStrwlenTab[u]:0x40000000;
				if(  i&uMaskE)break;
				if(!(i&uMaskU)){j++;continue;}
				for(j++;;j++){u=pSource[j];if(MASK(u,uMaskE))break;}
				iOk=0;
				break;
				}

			break;

		case SFT_ONLYSAME:					// Nur ndern wenn gleich ?

			for(;;)
				{
				u=pSource[j];
				i=aStrwlenTab[u];
				if(i&uMaskE)break;
				if(i&uMaskU)k|=1;
				if(i&uMaskI)k|=2;
				if(k!=3){j++;continue;}
				for(j++;;j++){u=pSource[j];if(MASK(u,uMaskE))break;}
				break;
				}

			if(k==0 || k==3)iOk=0;
			break;

		default:j=(int)wcslen(pSource+iPos)+iPos;
			}

		if(!iOk)
			{
			memcpy(pTarget+iPos,pSource+iPos,j-iPos);
			iPos=j;
			continue;
			}

//*****************************************************************************

		if(iMode&SFT_BIG)						// In Grobuchstaben umwandeln
			{
			for(;iPos<j;iPos++)
				{
				u=pSource[iPos];
				if(u==223 && ((iMode&SFT_ANSI))){pTarget[iPos]='S';         continue;}
				if(MASK(u,uMaskL)              ){pTarget[iPos]=(char)(u-32);continue;}
				pTarget[iPos]=(char)u;
				}
			continue;
			}

		if(iMode&SFT_LITTLE)					// In Grobuchstaben umwandeln
			{
			for(;iPos<j;iPos++)
				{
				u=pSource[iPos];
				if(MASK(u,uMaskU)){pTarget[iPos]=(char)(u+32);continue;}
				pTarget[iPos]=(char)u;
				}
			continue;
			}


		for(uLast=' ';iPos<j;iPos++)			// Anfansbuchstaben gro Rest klein
			{
			   u=pSource[iPos];
			if(u=='.')iPoint=iPos;

			if(MASK(uLast,uMaskV))
				{
				if(MASK(u,uMaskU)){pTarget[iPos]=(char)(u+32);uLast=u;continue;}
				}
			else if(iLet==0)
				{
				if(u==223 && ((iMode&SFT_ANSI))){pTarget[iPos]='S';         uLast=u;continue;}
				if(MASK(u,uMaskL)              ){pTarget[iPos]=(char)(u-32);uLast=u;continue;}
				}

			pTarget[iPos]=(char)u;
			uLast=u;
			}

		}

//*****************************************************************************


	if((iMode&SFT_POINTLITTLE) && iPoint>=0)	// Zeichen nach letztdem Punkt gro
		{
		iPoint++;
		u=pTarget[iPoint];
		if(MASK(u,uMaskU))pTarget[iPoint]=(char)(u+32);
		}


	pTarget[iPos]=0;

}

#ifdef _DEBUG
void _cdecl strfilter(char    *pString,int iMode){strfilter(pString,pString,iMode);}
void _cdecl strfilter(wchar_t *pString,int iMode){strfilter(pString,pString,iMode);}
#endif

//*****************************************************************************
//*
//*		strlenstr
//*
//*****************************************************************************
//	Ergibt die Lnge eines Strings. Wenn am Anfang nicht das cSign Zeichen
//	steht dann wird nur das erste Wort gezhlt bis " ,;"
//	pStr	: Zeiger auf pString
//	cJump	: Jump-Zeichen z.B. \" (bei 0 keine Prfung
//	iMax	: Ist die maximale Lnge des Strings
//	cSign	: Ist die iMode der Klammern \' \" ( ) [ ] { } < >
//	Ergibt die Lnge eines Strings inklusive Abschusszeichen
int _cdecl strlenstr(const char *pStr,char cJump,char cSign,int iMax)
{
unsigned 	aNextBits[8];
char		cChar;
int			iLen;
int			iOff;


	if(iMax<=0)return 0;

	iOff  = strwlen(pStr,0xA0,iMax);
	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)
		{
		return strcbits(pStr,aStrStrBits,iMax)+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}

	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)								// Kein Jump-Zeichnen
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1)+1;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;
		return iOff+iLen;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1;;iLen++)
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iLen++;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


return iOff+iLen;
}

int _cdecl strlenstr(const wchar_t *pStr,wchar_t uJump,wchar_t uSign,int iMax)
{
unsigned 	aNextBits[8];
wchar_t		uChar;
int			iLen;
int			iOff;


	if(iMax<=0)return 0;

	iOff  = strwlen(pStr,0xA0,iMax);
	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=uSign)
		{
		return strcbits(pStr,aStrStrBits,iMax)+iOff;
		}


	if(uSign!='"' && uSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(uSign=='(')uSign=')';
		else if(uSign=='[')uSign=']';
		else if(uSign=='{')uSign='}';
		else if(uSign=='<')uSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)uSign)/32] |= 1<<(uSign&0x1F);


	if(uJump==0)								// Kein Jump-Zeichnen
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1)+1;
		if(iLen<iMax && pStr[iLen]==uSign)iLen++;
		return iOff+iLen;
		}


	aNextBits[((unsigned char)uJump)/32] |= 1<<(uJump&0x1F);

	for(iLen=1;;iLen++)
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		uChar = pStr[iLen];

		if(uChar==uSign){iLen++;break;}
		if(uChar!=uJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


return iOff+iLen;
}

int _cdecl strlenstr(int iMode,const char *pStr,char cJump,char cSign,int iMax)
{
unsigned 	aNextBits[8];
char		cChar;
int			iLen;
int			iOff;


	if(iMax<=0)return 0;

	iOff  = strwlen(pStr,0xA0,iMax);
	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)
		{
		return strwlen(pStr,iMode,iMax)+iOff;
		}



	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1)+1;
		return iOff+iLen;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1;;iLen++)
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iLen++;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


return iOff+iLen;
}

int _cdecl strlenstr(int iMode,const wchar_t *pStr,wchar_t uJump,wchar_t uSign,int iMax)
{
unsigned 	aNextBits[8];
wchar_t		uChar;
int			iLen;
int			iOff;


	if(iMax<=0)return 0;

	iOff  = strwlen(pStr,0xA0,iMax);
	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=uSign)
		{
		return strwlen(pStr,iMode,iMax)+iOff;
		}


	if(uSign!='"' && uSign!='\'')
		{
		     if(uSign=='(')uSign=')';
		else if(uSign=='[')uSign=']';
		else if(uSign=='{')uSign='}';
		else if(uSign=='<')uSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)uSign)/32] |= 1<<(uSign&0x1F);


	if(uJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1)+1;
		return iOff+iLen;
		}


	aNextBits[((unsigned char)uJump)/32] |= 1<<(uJump&0x1F);

	for(iLen=1;;iLen++)
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		uChar = pStr[iLen];

		if(uChar==uSign){iLen++;break;}
		if(uChar!=uJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


return iOff+iLen;
}

//*****************************************************************************
//*
//*		strgetstr
//*
//*****************************************************************************
//	Kopiert einen String in einen Puffer. Leerzeichen am Anfang werden bersprungen.
//	Wenn am Anfang nicht das cSign Zeichen steht dann wird nur das erste Wort
//	kopiert. strwlen(...,0x3000000F) ansonsten alle Zeichen zwischen den cSign
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	pBuffer	: Zeiger auf Zielpuffer
//	pStr	: Zeiger auf pString
//	iSize	: Gre des Zielpuffers
//	cJump	: Jump-Zeichen z.B. \" (bei 0 keine Prfung
//	iMax	: Ist die maximale Lnge des Strings
//	cSign	: Ist die iMode der Klammern \' \" ( ) [ ] { } < >
//	Ergibt die Lnge der bersprungenen Zeichen
int _cdecl strgetstr(char *pBuffer,const char *pStr,int iSize,char cJump,char cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
char		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Nur erstes Wort holen
		{
		iLen=strcbits(pStr,aStrStrBits,iMax);
		strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
		strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

int _cdecl strgetstr(wchar_t *pBuffer,const wchar_t *pStr,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
wchar_t		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Nur erstes Wort holen
		{
		iLen=strcbits(pStr,aStrStrBits,iMax);
		strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
		strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

int _cdecl strgetstr(int iMode,char *pBuffer,const char *pStr,int iSize,char cJump,char cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
char		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Nur erstes Wort holen
		{
		iLen=strwlen(pStr,iMode,iMax);
		strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
		strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

int _cdecl strgetstr(int iMode,wchar_t *pBuffer,const wchar_t *pStr,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
wchar_t		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Nur erstes Wort holen
		{
		iLen=strwlen(pStr,iMode,iMax);
		strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
		strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

//*****************************************************************************
//*
//*		strgetstr
//*
//*****************************************************************************
//	Kopiert einen String in einen Puffer. Leerzeichen am Anfang werden bersprungen.
//	Wenn am Anfang nicht das cSign Zeichen steht dann wird nur das erste Wort
//	kopiert. strwlen(...,0x3000000F) ansonsten alle Zeichen zwischen den cSign
//	iMode	: Bestimmt die Gruppe, jedes Bits steht fr ein paar Zeichen
//			  (siehe aStrwlenTab)
//	pBuffer	: Zeiger auf Zielpuffer
//	pStr	: Zeiger auf String
//	iSize	: Gre des Zielpuffers
//	pLen	: Hier wird die Lnge des kopierten Strings gespeichert
//	cJump	: Jump-Zeichen z.B. \" (bei 0 keine Prfung
//	iMax	: Ist die maximale Lnge des Strings
//	cSign	: Ist die iMode der Klammern \' \" ( ) [ ] { } < >
//	Ergibt die Lnge der bersprungenen Zeichen
int _cdecl strgetstr(char *pBuffer,const char *pStr,int *pLen,int iSize,char cJump,char cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
char		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Keine Gruppierung
		{
		iLen = strcbits(pStr,aStrStrBits,iMax);
	   *pLen = strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}

	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
	   *pLen = strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	*pLen = strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

int _cdecl strgetstr(wchar_t *pBuffer,const wchar_t *pStr,int *pLen,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
wchar_t		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Keine Gruppierung
		{
		iLen = strcbits(pStr,aStrStrBits,iMax);
	   *pLen = strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
	   *pLen = strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	*pLen = strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

int _cdecl strgetstr(int iMode,char *pBuffer,const char *pStr,int *pLen,int iSize,char cJump,char cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
char		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Keine Gruppierung
		{
		iLen = strwlen(pStr,iMode,iMax);
	   *pLen = strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}


	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
	   *pLen = strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	*pLen = strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

int _cdecl strgetstr(int iMode,wchar_t *pBuffer,const wchar_t *pStr,int *pLen,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
int			iLen,iNum,iAdd,iOff;
unsigned 	aNextBits[8];
wchar_t		cChar;


	if(iMax<=0)
		{
		if((iMax&0x7F800000)!=STR_GET_ID)		// Ist ein Prfix-Zeichen vorhanden
			{
			return 0;
			}

		iNum  = (iMax>>16)&0x7F;
		iMax &= 0xFFFF;
		iOff  = strwlen(pStr,0xA0,iMax);

		if(iOff>=iMax || pStr[iOff]!=iNum)
			{
			return 0;
			}

		iOff += 1;
		iOff += strwlen(pStr+iOff,0xA0,iMax-iOff);
		}
	else{
		iOff  = strwlen(pStr,0xA0,iMax);
		}

	pStr += iOff;
	iMax -= iOff;

	if(iMax<=0)return iOff;

	if(pStr[0]!=cSign)							// Keine Gruppierung
		{
		iLen = strwlen(pStr,iMode,iMax);
	   *pLen = strlcpy(pBuffer,pStr,iSize,iLen);
		return iLen+iOff;
		}


	if(cSign!='"' && cSign!='\'')				// Ist es ein Klammerzeichen
		{
		     if(cSign=='(')cSign=')';
		else if(cSign=='[')cSign=']';
		else if(cSign=='{')cSign='}';
		else if(cSign=='<')cSign='>';
		}

	aNextBits[0] = 1|(1<<'\n')|(1<<'\r');
	aNextBits[1] = 0;
	aNextBits[2] = 0;
	aNextBits[3] = 0;
	aNextBits[4] = 0;
	aNextBits[5] = 0;
	aNextBits[6] = 0;
	aNextBits[7] = 0;
	aNextBits[((unsigned char)cSign)/32] |= 1<<(cSign&0x1F);


	if(cJump==0)
		{
		iLen = strcbits(pStr+1,aNextBits,iMax-1);
	   *pLen = strlcpy(pBuffer,pStr+1,iSize,iLen);

		iLen++;
		if(iLen<iMax && pStr[iLen]==cSign)iLen++;

		return iLen+iOff;
		}


	aNextBits[((unsigned char)cJump)/32] |= 1<<(cJump&0x1F);

	for(iLen=1,iAdd=0;;iLen++)					// Suche Textende mit Jump-Zeichen
		{
		iLen += strcbits(pStr+iLen,aNextBits,iMax-iLen);

		if(iLen>=iMax)break;

		cChar = pStr[iLen];

		if(cChar==cSign){iAdd=1;break;}
		if(cChar!=cJump)break;

		   iLen++;
		if(iLen>=iMax)break;

		if(pStr[iLen]==0)break;
		}


	*pLen = strlcpy(pBuffer,pStr+1,iSize,iLen-1);


return iOff+iLen+iAdd;
}

#ifdef _DEBUG
int _cdecl strgetstr(char *pBuffer,char cPrefix,const char *pStr,int iSize,char cJump,char cSign,int iMax)
{
return strgetstr(pBuffer,pStr,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

int _cdecl strgetstr(wchar_t *pBuffer,wchar_t cPrefix,const wchar_t *pStr,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
return strgetstr(pBuffer,pStr,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

int _cdecl strgetstr(char *pBuffer,char cPrefix,const char *pStr,int *pLen,int iSize,char cJump,char cSign,int iMax)
{
return strgetstr(pBuffer,pStr,pLen,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

int _cdecl strgetstr(wchar_t *pBuffer,wchar_t cPrefix,const wchar_t *pStr,int *pLen,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
return strgetstr(pBuffer,pStr,pLen,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

int _cdecl strgetstr(int iMode,char *pBuffer,char cPrefix,const char *pStr,int iSize,char cJump,char cSign,int iMax)
{
return strgetstr(iMode,pBuffer,pStr,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

int _cdecl strgetstr(int iMode,wchar_t *pBuffer,wchar_t cPrefix,const wchar_t *pStr,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
return strgetstr(iMode,pBuffer,pStr,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

int _cdecl strgetstr(int iMode,char *pBuffer,char cPrefix,const char *pStr,int *pLen,int iSize,char cJump,char cSign,int iMax)
{
return strgetstr(iMode,pBuffer,pStr,pLen,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

int _cdecl strgetstr(int iMode,wchar_t *pBuffer,wchar_t cPrefix,const wchar_t *pStr,int *pLen,int iSize,wchar_t cJump,wchar_t cSign,int iMax)
{
return strgetstr(iMode,pBuffer,pStr,pLen,iSize,cJump,cSign,STR_GET_PREFIX(cPrefix,iMax));
}

#endif

//*****************************************************************************
//*
//*		strvcmp
//*
//*****************************************************************************
//	Vergleicht zwei Texte und bercksichtigt dabei Zahlen in den Texten.
//	Zwischen Gro- und Kleinbuchstaben wird nicht unterschieden.
//	pStr1		: Zeiger auf den ersten  Text
//	pStr2		: Zeiger auf den zweiten Text
//	Ergibt	 0 = Wenn beide Texte gleich sind
//			 1 = Wenn pStr1 grer  als pStr2 ist
//			-1 = Wenn pStr1 kleiner als pStr2 ist
int __cdecl strvcmp(const char *pStr1,const char *pStr2)
{
unsigned char 	uChar1;
unsigned char 	uChar2;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;


	for(;;)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(uChar1== 0  || uChar2== 0 )break;
		if(uChar1<='9' && uChar2<='9')					// Ist eine eingebettete Zahl vorhanden
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoi(pStr1,0,&iLen1);
			uVal2 = strtoi(pStr2,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return strcmp(pStr1,pStr2);
				}

			   iCmp = memcmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			uChar1 = ucAnsiCmpV[uChar1-128];
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			uChar2 = ucAnsiCmpV[uChar2-128];
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}

		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}





return uChar1-uChar2;
}

int __cdecl strvcmp(const wchar_t *pStr1,const wchar_t *pStr2)
{
unsigned short 	uChar1;
unsigned short 	uChar2;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;


	for(;;)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(uChar1== 0  || uChar2== 0 )break;
		if(uChar1<='9' && uChar2<='9')					// Ist eine eingebettete Zahl vorhanden
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoi(pStr1,0,&iLen1);
			uVal2 = strtoi(pStr2,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return wcscmp(pStr1,pStr2);
				}

			   iCmp = memcmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			uChar1 = ucAnsiCmpV[uChar1-128];
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			uChar2 = ucAnsiCmpV[uChar2-128];
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}


		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}


return uChar1-uChar2;
}

//*****************************************************************************
//*
//*		strnvcmp
//*
//*****************************************************************************
//	Vergleicht zwei Texte und bercksichtigt dabei Zahlen in den Texten.
//	Zwischen Gro- und Kleinbuchstaben wird nicht unterschieden.
//	pStr1		: Zeiger auf den ersten  Text
//	pStr2		: Zeiger auf den zweiten Text
//	iMax		: Ist die maximale Textlnge die verglichen wird.
//	Ergibt	 0 = Wenn beide Texte gleich sind
//			 1 = Wenn pStr1 grer  als pStr2 ist
//			-1 = Wenn pStr1 kleiner als pStr2 ist
int __cdecl strnvcmp(const char *pStr1,const char *pStr2,int iMax)
{
unsigned char 	uChar1=0;
unsigned char 	uChar2=0;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;


	for(;iMax>0;iMax--)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(uChar1== 0  || uChar2== 0 )break;
		if(uChar1<='9' && uChar2<='9')					// Ist eine eingebettete Zahl vorhanden
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoni(pStr1,iMax,0,&iLen1);
			uVal2 = strtoni(pStr2,iMax,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return strcmp(pStr1,pStr2);
				}

			   iCmp = memcmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			uChar1 = ucAnsiCmpV[uChar1-128];
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			uChar2 = ucAnsiCmpV[uChar2-128];
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}

		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}





return uChar1-uChar2;
}

int __cdecl strnvcmp(const wchar_t *pStr1,const wchar_t *pStr2,int iMax)
{
unsigned short 	uChar1=0;
unsigned short 	uChar2=0;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;



	for(;iMax>0;iMax--)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(uChar1== 0  || uChar2== 0 )break;			// Ist eine eingebettete Zahl vorhanden
		if(uChar1<='9' && uChar2<='9')
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoni(pStr1,iMax,0,&iLen1);
			uVal2 = strtoni(pStr2,iMax,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return wcscmp(pStr1,pStr2);
				}

			   iCmp = memcmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			uChar1 = ucAnsiCmpV[uChar1-128];
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			uChar2 = ucAnsiCmpV[uChar2-128];
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}

		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}


return uChar1-uChar2;
}

//*****************************************************************************
//*
//*		strvicmp
//*
//*****************************************************************************
//	Vergleicht zwei Texte und bercksichtigt dabei Zahlen in den Texten.
//	Zwischen Gro- und Kleinbuchstaben wird unterschieden.
//	pStr1		: Zeiger auf den ersten  Text
//	pStr2		: Zeiger auf den zweiten Text
//	Ergibt	 0 = Wenn beide Texte gleich sind
//			 1 = Wenn pStr1 grer  als pStr2 ist
//			-1 = Wenn pStr1 kleiner als pStr2 ist
int __cdecl strvicmp(const char *pStr1,const char *pStr2)
{
unsigned char 	uChar1;
unsigned char 	uChar2;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;


	for(;;)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(strwchara(uChar1,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar1 |= 0x20;
			}

		if(strwchara(uChar2,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar2 |= 0x20;
			}

		if(uChar1== 0  || uChar2== 0 )break;
		if(uChar1<='9' && uChar2<='9')
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoi(pStr1,0,&iLen1);
			uVal2 = strtoi(pStr2,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return stricmp(pStr1,pStr2);
				}

			   iCmp = memicmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			   uChar1 = ucAnsiCmpV[uChar1-128];
			if(uChar1<='Z')uChar1+=0x20;
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			   uChar2 = ucAnsiCmpV[uChar2-128];
			if(uChar2<='Z')uChar2+=0x20;
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}

		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}





return uChar1-uChar2;
}

int __cdecl strvicmp(const wchar_t *pStr1,const wchar_t *pStr2)
{
unsigned short 	uChar1;
unsigned short 	uChar2;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;


	for(;;)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(strwcharw(uChar1,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar1 |= 0x20;
			}

		if(strwcharw(uChar2,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar2 |= 0x20;
			}

		if(uChar1== 0  || uChar2== 0 )break;
		if(uChar1<='9' && uChar2<='9')
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoi(pStr1,0,&iLen1);
			uVal2 = strtoi(pStr2,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return wcsicmp(pStr1,pStr2);
				}

			   iCmp = memicmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			   uChar1 = ucAnsiCmpV[uChar1-128];
			if(uChar1<='Z')uChar1+=0x20;
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			   uChar2 = ucAnsiCmpV[uChar2-128];
			if(uChar2<='Z')uChar2+=0x20;
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}

		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}





return uChar1-uChar2;
}

//*****************************************************************************
//*
//*		strnvicmp
//*
//*****************************************************************************
//	Vergleicht zwei Texte und bercksichtigt dabei Zahlen in den Texten.
//	Zwischen Gro- und Kleinbuchstaben wird unterschieden.
//	pStr1		: Zeiger auf den ersten  Text
//	pStr2		: Zeiger auf den zweiten Text
//	iMax		: Ist die maximale Textlnge die verglichen wird.
//	Ergibt	 0 = Wenn beide Texte gleich sind
//			 1 = Wenn pStr1 grer  als pStr2 ist
//			-1 = Wenn pStr1 kleiner als pStr2 ist
int __cdecl strnvicmp(const char *pStr1,const char *pStr2,int iMax)
{
unsigned char 	uChar1=0;
unsigned char 	uChar2=0;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;



	for(;iMax>0;iMax--)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(strwchara(uChar1,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar1 |= 0x20;
			}

		if(strwchara(uChar2,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar2 |= 0x20;
			}

		if(uChar1== 0  || uChar2== 0 )break;
		if(uChar1<='9' && uChar2<='9')
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoni(pStr1,iMax,0,&iLen1);
			uVal2 = strtoni(pStr2,iMax,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return stricmp(pStr1,pStr2);
				}

			   iCmp = memicmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			   uChar1 = ucAnsiCmpV[uChar1-128];
			if(uChar1<='Z')uChar1+=0x20;
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			   uChar2 = ucAnsiCmpV[uChar2-128];
			if(uChar2<='Z')uChar2+=0x20;
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}

		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}





return uChar1-uChar2;
}

int __cdecl strnvicmp(const wchar_t *pStr1,const wchar_t *pStr2,int iMax)
{
unsigned short 	uChar1=0;
unsigned short 	uChar2=0;
unsigned 		uVal1;
unsigned		uVal2;
int				iLen1;
int				iLen2;
int				iCmp;



	for(;iMax>0;iMax--)
		{
		uChar1 = *pStr1;
		uChar2 = *pStr2;

		if(strwcharw(uChar1,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar1 |= 0x20;
			}

		if(strwcharw(uChar2,0x10000002))				// In Kleinbuchstaben umwandeln
			{
			uChar2 |= 0x20;
			}

		if(uChar1== 0  || uChar2== 0 )break;
		if(uChar1<='9' && uChar2<='9')
		if(uChar1>='0' && uChar2>='0')
			{
			uVal1 = strtoni(pStr1,iMax,0,&iLen1);
			uVal2 = strtoni(pStr2,iMax,0,&iLen2);

			if(uVal1!=uVal2)
				{
				return (uVal1<uVal2)? -1:1;
				}

			if(iLen1!=iLen2)
				{
				return wcsicmp(pStr1,pStr2);
				}

			   iCmp = memicmp(pStr1,pStr2,iLen1);
			if(iCmp)return iCmp;

			pStr1 += iLen1;
			pStr2 += iLen2;
			continue;
			}

		if(uChar1==uChar2)								// Sind die Zeichen gleich
			{
			pStr1++;
			pStr2++;
			continue;
			}

		if(uChar1>=128)									// Umlaute umwandeln
			{
			   uChar1 = ucAnsiCmpV[uChar1-128];
			if(uChar1<='Z')uChar1+=0x20;
			}
		else if(uChar1>=92)
			{
			if(uChar1=='_' )uChar1=' ';
			if(uChar1=='\\')uChar1=0;
			}
		else if(uChar1<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar1=='.' )uChar1=1;
			if(uChar1=='/' )uChar1=0;
			}

		if(uChar2>=128)									// Umlaute umwandeln
			{
			   uChar2 = ucAnsiCmpV[uChar2-128];
			if(uChar2<='Z')uChar2+=0x20;
			}
		else if(uChar2>=92)
			{
			if(uChar2=='_' )uChar2=' ';
			if(uChar2=='\\')uChar2=0;
			}
		else if(uChar2<=47)								// "/..." und ".." an den Anfang
			{
			if(uChar2=='.' )uChar2=1;
			if(uChar2=='/' )uChar2=0;
			}

		if(uChar1!=uChar2)break;

		pStr1++;
		pStr2++;
		}





return uChar1-uChar2;
}

//*****************************************************************************
//*
//*		utf8prechar
//*
//*****************************************************************************
//	Holt ein UTF8-Zeichen aus einem Text.
//	Der Text wird dabei von hintern durchsucht.
//	pText		: Zeiger auf das Byte nach dem UTF8-Zeichen.
//	uUnicode	: Hier wird das Zeichen gespeichert.
//	Ergibt die Lnge des Zeichens in Byte.
int _cdecl utf8prechar(const char *pText,unsigned &uUnicode)
{
unsigned	uVal1;
unsigned	uVal2;
unsigned	uVal3;
unsigned	uVal4;



	uVal1 = ((unsigned char*)pText)[-1];

	if((uVal1&0xC0)!=0x80)									// 1-Byte-Zeichen
		{
		uUnicode = uVal1;
		return 1;
		}


	uVal2 = ((unsigned char*)pText)[-2];

	if((uVal2&0xE0)==0xC0)									// 2-Byte-Zeichen
		{
		uUnicode = (uVal1&0x3F)|((uVal2&0x1F)<<6);
		return 2;
		}

	if((uVal2&0xC0)!=0x80)
		{
		uUnicode = uVal1;
		return 1;
		}


	uVal3 = ((unsigned char*)pText)[-3];

	if((uVal3&0xF0)==0xE0)									// 3-Byte-Zeichen
		{
		uUnicode = (uVal1&0x3F)|((uVal2&0x3F)<<6)|((uVal3&0x0F)<<12);
		return 3;
		}

	if((uVal3&0xC0)!=0x80)
		{
		uUnicode = uVal1;
		return 1;
		}


	uVal4 = ((unsigned char*)pText)[-4];

	if((uVal4&0xF8)==0xF0)									// 4-Byte-Zeichen
		{
		uUnicode = (uVal1&0x3F)|((uVal2&0x3F)<<6)|((uVal3&0x3F)<<12)|((uVal4&0x07)<<18);
		return 4;
		}


	uUnicode = uVal1;


return 1;
}

//*****************************************************************************
//*
//*		utf8tochar
//*
//*****************************************************************************
//	Wandelt das erste UTF8-Zeichen aus einem Text in ein Zeichen um
//	pText		: Zeiger auf UTF8 Text
//	uUnicode	: Hier wird das Zeichen gespeichert
//	Ergibt die Lnge des Zeichens
int _cdecl utf8tochar(const char *pText,unsigned &uUnicode)
{
char		cSign1;
char		cSign2;
char		cSign3;
char		cChar;


	cChar=*pText;

	if((cChar&0xC0)==0xC0)
		{
		if((cChar&0xE0)==0xC0)					// Zwei Zeichen ?
			{
			cSign1 = pText[1];

			if((cSign1&0xC0)==0x80)
				{
				uUnicode = ((cChar&0x1F)<<6)|(cSign1&0x3F);
				return 2;
				}
			}
		else if((cChar&0xF0)==0xE0)				// Drei Zeichen ?
			{
			cSign1 = pText[1];
			cSign2 = pText[2];

			if((cSign1&0xC0)==0x80)
			if((cSign2&0xC0)==0x80)
				{
				uUnicode = ((cChar&0x0F)<<12)|((cSign1&0x3F)<<6)|(cSign2&0x3F);
				return 3;
				}
			}
		else if((cChar&0xF8)==0xF0)				// Vier Zeichen ?
			{
			cSign1 = pText[1];
			cSign2 = pText[2];
			cSign3 = pText[3];

			if((cSign1&0xC0)==0x80)
			if((cSign2&0xC0)==0x80)
			if((cSign3&0xC0)==0x80)
				{
				uUnicode = ((cChar&0x07)<<18)|((cSign1&0x3F)<<12)|((cSign2&0x3F)<<6)|(cSign3&0x3F);
				return 4;
				}
			}
		}


	uUnicode = (unsigned char)cChar;


return 1;
}

//*****************************************************************************
//*
//*		utf8length
//*
//*****************************************************************************
//	Berechnet die Lnge eines UTF8-Zeichens.
//	uUnicode	: Ist das Zeichen das umgewandelt werden soll
//	Ergibt die Lnge des Zeichens
int _cdecl utf8length(unsigned uUnicode)
{

	if(uUnicode<0x80)							// Ein Zeichen
		{
		return 1;
		}

	if(uUnicode<0x800)							// Zwei Zeichen
		{
		return 2;
		}

	if(uUnicode<0x10000)						// Drei Zeichen
		{
		return 3;
		}


return 4;
}

//*****************************************************************************
//*
//*		utf8length
//*
//*****************************************************************************
//	Berechnet die Lnge eines UTF8-Zeichens.
//	pText	: Zeiger auf den Text eine UTF 8 Zeichen
//	iMax	: Ist die maximale Textlnge
//	Ergibt die Lnge des Zeichens
int _cdecl utf8length(const char *pText)
{
char		cSign1;
char		cSign2;
char		cSign3;
char		cChar;


	cChar = *pText;

	if((cChar&0xC0)==0xC0)
		{
		if((cChar&0xE0)==0xC0)					// Zwei Zeichen ?
			{
			cSign1 = pText[1];

			if((cSign1&0xC0)==0x80)
				{
				return 2;
				}
			}
		else if((cChar&0xF0)==0xE0)				// Drei Zeichen ?
			{
			cSign1 = pText[1];
			cSign2 = pText[2];

			if((cSign1&0xC0)==0x80)
			if((cSign2&0xC0)==0x80)
				{
				return 3;
				}
			}
		else if((cChar&0xF8)==0xF0)				// Vier Zeichen ?
			{
			cSign1 = pText[1];
			cSign2 = pText[2];
			cSign3 = pText[3];

			if((cSign1&0xC0)==0x80)
			if((cSign2&0xC0)==0x80)
			if((cSign3&0xC0)==0x80)
				{
				return 4;
				}
			}
		}


return 1;
}

int _cdecl utf8length(const char *pText,int iMax)
{
char		cSign1;
char		cSign2;
char		cSign3;
char		cChar;


	if(iMax<=0)return 0;

	cChar = *pText;

	if((cChar&0xC0)==0xC0)
		{
		if((cChar&0xE0)==0xC0)					// Zwei Zeichen ?
			{
			cSign1 = (char)((iMax>1)? pText[1]:0);

			if((cSign1&0xC0)==0x80)
				{
				return  2;
				}
			}
		else if((cChar&0xF0)==0xE0)				// Drei Zeichen ?
			{
			cSign1 = (char)((iMax>1)? pText[1]:0);
			cSign2 = (char)((iMax>2)? pText[2]:0);

			if((cSign1&0xC0)==0x80)
			if((cSign2&0xC0)==0x80)
				{
				return  3;
				}
			}
		else if((cChar&0xF8)==0xF0)				// Vier Zeichen ?
			{
			cSign1 = (char)((iMax>1)? pText[1]:0);
			cSign2 = (char)((iMax>2)? pText[2]:0);
			cSign3 = (char)((iMax>3)? pText[3]:0);

			if((cSign1&0xC0)==0x80)
			if((cSign2&0xC0)==0x80)
			if((cSign3&0xC0)==0x80)
				{
				return  4;
				}
			}
		}


return 1;
}


//*****************************************************************************
//*
//*		chartoutf8
//*
//*****************************************************************************
//	Wandelt ein Zeichen in eine UTF8/UTF16 Text um
//	pText		: Zeiger auf UTF8/UTF16 Text wo das Zeichen gespeichert  wird
//	uUnicode	: Ist das Zeichen das umgewandelt werden soll
//	Ergibt die Lnge des Zeichens
int _cdecl chartoutf8(char *pText,unsigned uUnicode)
{
#ifdef 	NO_STR_ASM

	if(uUnicode<0x80)							// Ein Zeichen
		{
		pText[0]=(char)uUnicode;
		return 1;
		}

	if(uUnicode<0x800)							// Zwei Zeichen
		{
		pText[0]=(char)((uUnicode>>  6)|0xC0);
		pText[1]=(char)((uUnicode&0x3F)|0x80);
		return 2;
		}

	if(uUnicode<0x10000)						// Drei Zeichen
		{
		pText[0]=(char)( (uUnicode>>12)      |0xE0);
		pText[1]=(char)(((uUnicode>> 6)&0x3F)|0x80);
		pText[2]=(char)( (uUnicode     &0x3F)|0x80);
		return 3;
		}

	pText[0]=(char)(((uUnicode>>18)&0x07)|0xF0);
	pText[1]=(char)(((uUnicode>>12)&0x3F)|0x80);
	pText[2]=(char)(((uUnicode>> 6)&0x3F)|0x80);
	pText[3]=(char)( (uUnicode     &0x3F)|0x80);


	return 4;

#else

_asm	{
		mov		eax,uUnicode					;// EAX = uUnicode
		mov		ebx,pText						;// EBX = pText
		test	eax,0xFFFFFF80					;//	Ein Zeichen ?
		jnz		Zwei							;//		*
		mov		[ebx],al						;//		*
		mov		eax,1							;//		*
		jmp		Ende							;//		*

Zwei:	test	eax,0xFFFFF800					;//	Zwei Zeichen ?
		jnz		Drei							;//		*

		rol		eax,2							;// pText[0]=(char)((uUnicode>>  6)|0xC0)
		xchg	ah,al							;// pText[1]=(char)((uUnicode&0x3F)|0x80)
		ror		ah,2							;//		*
		or		ax,0x80C0						;//		*
		mov		[ebx],ax						;//		*
		mov		eax,2							;//		*
		jmp		Ende							;//		*

Drei:	test	eax,0xFFFF0000					;//	Drei Zeichen ?
		jnz		Vier							;//		*

		ror		eax,12							;//	pText[0]=(char)( (uUnicode>>12)      |0xE0);
		rol		ax ,2							;//	pText[1]=(char)(((uUnicode>> 6)&0x3F)|0x80);
		rol		eax,6							;//		*
		xchg	ah,al							;//		*
		or		ax,0x80E0						;//		*
		mov		[ebx],ax						;//		*
		rol		eax,6							;//	pText[2]=(char)( (uUnicode     &0x3F)|0x80);
		and		al ,0x3F						;//		*
		or		al ,0x80						;//		*
		mov		[ebx+2],al						;//		*
		mov		eax,3							;//		*
		jmp		Ende							;//		*


Vier:	and		eax,0x001FFFFF					;// Bits beschrnken
		mov		ecx,eax;						;//		*
		ror		eax,10							;//	pText[0]=(char)(((uUnicode>>18)&0x07)|0xF0);
		and		ecx,0x0FFF						;//		*
		shr		al ,2							;//	pText[1]=(char)(((uUnicode>>12)&0x3F)|0x80);
		ror		ecx,6							;//		*
		xchg	ah ,al							;//		*
		rol		cx ,2							;//	pText[2]=(char)(((uUnicode>> 6)&0x3F)|0x80);
		or		ax ,0x80F0						;//		*
		rol		ecx,6							;//	pText[3]=(char)( (uUnicode     &0x3F)|0x80);
		mov		[ebx],ax						;//		*
		xchg	ch ,cl							;//		*
		or		cx ,0x8080						;//		*
		mov		eax,4							;//		*
		mov		[ebx+2],cx						;//		*

Ende:
		}

#endif
}




//*****************************************************************************
//*
//*		utf16prechar
//*
//*****************************************************************************
//	Holt ein UTF16-Zeichen aus einem Text.
//	Der Text wird dabei von hintern durchsucht.
//	pText		: Zeiger auf das Byte nach dem UTF16-Zeichen.
//	uUnicode	: Hier wird das Zeichen gespeichert.
//	Ergibt die Lnge des Zeichens in Zeichen.
int _cdecl utf16prechar(const wchar_t *pText,unsigned &uUnicode)
{
unsigned	uVal1;
unsigned	uVal2;



	uVal1 = pText[-1];

	if((uVal1&0xF800)!=0xD800)								// 1-fach Zeichen
		{
		uUnicode = uVal1;
		return 1;
		}

	if((uVal1&0xFC00)==0xDC00)								// 2-fach Zeichen
		{
		uVal2 = pText[-2];

		if((uVal1&0xFC00)==0xDC00)
			{
			uUnicode = ((uVal1&0x03FF)|((uVal2&0x03FF)<<10))+0x10000;
			return 2;
			}

		uUnicode = (uVal1&0x03FF)+0x10000;
		return 1;
		}


	uUnicode = ((uVal1&0x03FF)<<10)+0x10000;


return 1;
}

//*****************************************************************************
//*
//*		utf16tochar
//*
//*****************************************************************************
//	Wandelt das erste UTF16-Zeichen aus einem Text in ein Zeichen um.
//	pText		: Zeiger auf den UTF16 Text.
//	uUnicode	: Hier wird das Zeichen gespeichert.
//	Ergibt die Lnge des Zeichens in Zeichen.
int _cdecl utf16tochar(const wchar_t *pText,unsigned &uUnicode)
{
unsigned	uVal1;
unsigned	uVal2;



	uVal1 = pText[0];

	if((uVal1&0xF800)!=0xD800)								// 1-fach Zeichen
		{
		uUnicode = uVal1;
		return 1;
		}

	if((uVal1&0xFC00)==0xD800)								// 2-fach Zeichen
		{
		uVal2 = pText[1];

		if((uVal2&0xFC00)==0xDC00)
			{
			uUnicode = (uVal2&0x03FF)+((uVal1&0x03FF)<<10)+0x10000;
			return 2;
			}

		uUnicode = ((uVal1&0x03FF)<<10)+0x10000;
		return 1;
		}


	uUnicode = ((uVal1&0x03FF)<<10)+0x10000;


return 1;
}

//*****************************************************************************
//*
//*		utf16length
//*
//*****************************************************************************
//	Berechnet die Lnge eines UTF16-Zeichens.
//	uUnicode	: Ist das Zeichen das umgewandelt werden soll.
//	Ergibt die Lnge des Zeichens in Zeichen.
int _cdecl utf16length(unsigned uUnicode)
{

	if(uUnicode<0x1000)							// Ein Zeichen
		{
		return 1;
		}


return 2;
}

//*****************************************************************************
//*
//*		utf16tochar
//*
//*****************************************************************************
//	Berechnet die Lnge eines UTF16-Zeichens.
//	pText		: Zeiger auf das UTF16-Zeichen als Text.
//	uUnicode	: Hier wird das Zeichen gespeichert.
//	Ergibt die Lnge des Zeichens in Zeichen.
int _cdecl utf16length(const wchar_t *pText)
{
unsigned	uVal1;
unsigned	uVal2;



	uVal1 = pText[0];

	if((uVal1&0xF800)!=0xD800)								// 1-fach Zeichen
		{
		return 1;
		}

	if((uVal1&0xFC00)==0xD800)								// 2-fach Zeichen
		{
		uVal2 = pText[1];

		if((uVal2&0xFC00)==0xDC00)
			{
			return 2;
			}

		return 1;
		}



return 1;
}

int _cdecl utf16length(const wchar_t *pText,int iMax)
{
unsigned	uVal1;
unsigned	uVal2;


	if(iMax<=0)return 0;

	uVal1 = pText[0];

	if((uVal1&0xF800)!=0xD800)								// 1-fach Zeichen
		{
		if(iMax<1)return iMax;
		return  1;
		}

	if((uVal1&0xFC00)==0xD800)								// 2-fach Zeichen
		{
		uVal2 = pText[1];

		if((uVal2&0xFC00)==0xDC00)
			{
			if(iMax<2)return iMax;
			return  2;
			}
		}

	if(iMax<1)return iMax;


return 1;
}

//*****************************************************************************
//*
//*		chartoutf16
//*
//*****************************************************************************
//	Wandelt ein Zeichen in einen UTF16 Text um
//	pText		: Zeiger auf UTF8/UTF16 Text wo das Zeichen gespeichert  wird
//	uUnicode	: Ist das Zeichen das umgewandelt werden soll
//	Ergibt die Lnge des Zeichens
int _cdecl chartoutf16(wchar_t *pText,unsigned uUnicode)
{

	if(uUnicode<0x10000)									// Ein Zeichen
		{
		pText[0]=(wchar_t)uUnicode;
		return 1;
		}


	uUnicode -= 0x10000;

	pText[0] = (wchar_t)(0xD800|(uUnicode>>10   ));
	pText[1] = (wchar_t)(0xDC00|(uUnicode&0x03FF));


return 2;
}

//*****************************************************************************
//*
//*		strreplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus.
//	pString	: Ist der Zeiger auf den Puffer in dem der genderte Text gespeichert wird.
//	pText	: Ist der Zeiger auf den Augangstext.
//	pFrom	: Ist der Text der getauscht werden soll.
//	pTo		: Ist der neue Text gegen den pFrom getauscht wird.
//	iMax	: Ist die Gre des Textpuffers in Zeichen.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strreplace(char *pString,const char *pText,const char *pFrom,const char *pTo,int iMax)
{
int		iNum;
int		iAdd;
int		iPos;
int		iLen;


	if(iMax<=0)return 0;

	iMax--;
	iAdd	= (int)strlen(pTo  );
	iNum	= (int)strlen(pFrom);
	iPos	= 0;

	for(;;)
		{
		   iLen = strfind(pText,pFrom);
		if(iLen<0)break;

		if(iPos+iLen>=iMax)
			{
			memmove(pString+iPos,pText,iMax-iPos);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pText,iLen);
		pText += iLen;
		iPos  += iLen;

		if(iPos+iNum>=iMax)
			{
			memcpy(pString+iPos,pTo,iMax-iPos);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pTo,iAdd);
		pText += iNum;
		iPos  += iAdd;
		}


	   iLen  = (int)strlen(pText);
	if(iLen>iMax-iPos)iLen=iMax-iPos;

	memmove(pString+iPos,pText,iLen);
	iPos += iLen;
	pString[iPos]=0;



return iPos;
}

int _cdecl strreplace(wchar_t *pString,const wchar_t *pText,const wchar_t *pFrom,const wchar_t *pTo,int iMax)
{
int		iNum;
int		iAdd;
int		iPos;
int		iLen;


	if(iMax<=0)return 0;

	iMax--;
	iAdd	= (int)wcslen(pTo  );
	iNum	= (int)wcslen(pFrom);
	iPos	= 0;

	for(;;)
		{
		   iLen = strfind(pText,pFrom);
		if(iLen<0)break;

		if(iPos+iLen>=iMax)
			{
			memmove(pString+iPos,pText,(iMax-iPos)*2);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pText,iLen*2);
		pText += iLen;
		iPos  += iLen;

		if(iPos+iNum>=iMax)
			{
			memcpy(pString+iPos,pTo,(iMax-iPos)*2);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pTo,iAdd*2);
		pText += iNum;
		iPos  += iAdd;
		}


	   iLen  = (int)wcslen(pText);
	if(iLen>iMax-iPos)iLen=iMax-iPos;

	memmove(pString+iPos,pText,iLen*2);
	iPos += iLen;
	pString[iPos]=0;



return iPos;
}

//*****************************************************************************
//*
//*		strreplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus.
//	pString	: Ist der Zeiger auf den Text der gendert werden soll.
//	pFrom	: Ist der Text der getauscht werden soll.
//	pTo		: Ist der neue Text gegen den pFrom getauscht wird.
//	iMax	: Ist die Gre des Textpuffers in Zeichen.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strreplace(char *pString,const char *pFrom,const char *pTo,int iMax)
{
char   *pBuffer;
int		iNum;
int		iAdd;
int		iLen;


	if(iMax<=0)return 0;

	iAdd	= (int)strlen(pTo  );
	iNum	= (int)strlen(pFrom);

	if(iAdd<=iNum)
		{
		return strreplace(pString,pString,pFrom,pTo,iMax);
		}

	pBuffer = (char*)alloca(iMax);
	iLen	= strreplace(pBuffer,pString,pFrom,pTo,iMax);

	memcpy(pString,pBuffer,iLen+1);


return iLen;
}

int _cdecl strreplace(wchar_t *pString,const wchar_t *pFrom,const wchar_t *pTo,int iMax)
{
wchar_t    *pBuffer;
int			iNum;
int			iAdd;
int			iLen;


	if(iMax<=0)return 0;

	iAdd	= (int)wcslen(pTo  );
	iNum	= (int)wcslen(pFrom);

	if(iAdd<=iNum)
		{
		return strreplace(pString,pString,pFrom,pTo,iMax);
		}

	pBuffer = (wchar_t*)alloca(iMax*2);
	iLen	= strreplace(pBuffer,pString,pFrom,pTo,iMax);

	memcpy(pString,pBuffer,(iLen+1)*2);


return iLen;
}

//*****************************************************************************
//*
//*		strreplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus.
//	pString	: Ist der Zeiger auf den Puffer in dem der genderte Text gespeichert wird.
//	pText	: Ist der Zeiger auf den Augangstext.
//	cFrom	: Ist das Zeichen das getauscht werden soll.
//	cTo		: Ist das neue Zeichen gegen das cFrom getauscht wird.
//	iMax	: Ist die Gre des Textpuffers in Zeichen.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strreplace(char *pString,const char *pText,char cFrom,char cTo,int iMax)
{
int		iPos;


	if(iMax<=0)return 0;

	iMax--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pText[iPos]==0)break;
		if(pText[iPos]!=cFrom)
				pString[iPos] = pText[iPos];
		else	pString[iPos] = cTo;
		}


	pString[iPos] = 0;


return iPos;
}

int _cdecl strreplace(wchar_t *pString,const wchar_t *pText,wchar_t cFrom,wchar_t cTo,int iMax)
{
int		iPos;


	if(iMax<=0)return 0;

	iMax--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pText[iPos]==0)break;
		if(pText[iPos]!=cFrom)
				pString[iPos] = pText[iPos];
		else	pString[iPos] = cTo;
		}


	pString[iPos] = 0;


return iPos;
}

#ifdef 	_DEBUG

int _cdecl strreplace(char *pString,char cFrom,char cTo,int iMax)
		{
		return strreplace(pString,pString,cFrom,cTo,iMax);
		}

int _cdecl strreplace(wchar_t *pString,wchar_t cFrom,wchar_t cTo,int iMax)
		{
		return strreplace(pString,pString,cFrom,cTo,iMax);
		}

#endif


//*****************************************************************************
//*
//*		strreplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus.
//	pString	: Ist der Zeiger auf den Text der gendert werden soll.
//	cFrom	: Ist das Zeichen das getauscht werden soll.
//	cTo		: Ist das neue Zeichen gegen das cFrom getauscht wird.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strreplace(char *pString,char cFrom,char cTo)
{
int		iPos;


	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==cFrom)
			{
			pString[iPos] = cTo;
			}
		}


return iPos;
}

int _cdecl strreplace(wchar_t *pString,wchar_t cFrom,wchar_t cTo)
{
int		iPos;


	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==cFrom)
			{
			pString[iPos] = cTo;
			}
		}


return iPos;
}

//*****************************************************************************
//*
//*		strireplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus. Dabei wird bei
//	der Suche nicht zwischen Gro- und Kleinbuchstaben unterschieden.
//	pString	: Ist der Zeiger auf den Puffer in dem der genderte Text gespeichert wird.
//	pText	: Ist der Zeiger auf den Augangstext.
//	pFrom	: Ist der Text der getauscht werden soll.
//	pTo		: Ist der neue Text gegen den pFrom getauscht wird.
//	iMax	: Ist die Gre des Textpuffers in Zeichen.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strireplace(char *pString,const char *pText,const char *pFrom,const char *pTo,int iMax)
{
int		iNum;
int		iAdd;
int		iPos;
int		iLen;


	if(iMax<=0)return 0;

	iMax--;
	iAdd	= (int)strlen(pTo  );
	iNum	= (int)strlen(pFrom);
	iPos	= 0;

	for(;;)
		{
		   iLen = strifind(pText,pFrom);
		if(iLen<0)break;

		if(iPos+iLen>=iMax)
			{
			memmove(pString+iPos,pText,iMax-iPos);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pText,iLen);
		pText += iLen;
		iPos  += iLen;

		if(iPos+iNum>=iMax)
			{
			memcpy(pString+iPos,pTo,iMax-iPos);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pTo,iAdd);
		pText += iNum;
		iPos  += iAdd;
		}

	   iLen  = (int)strlen(pText);
	if(iLen>iMax-iPos)iLen=iMax-iPos;

	memmove(pString+iPos,pText,iLen);
	iPos += iLen;
	pString[iPos]=0;



return iPos;
}

int _cdecl strireplace(wchar_t *pString,const wchar_t *pText,const wchar_t *pFrom,const wchar_t *pTo,int iMax)
{
int		iNum;
int		iAdd;
int		iPos;
int		iLen;


	if(iMax<=0)return 0;

	iMax--;
	iAdd	= (int)wcslen(pTo  );
	iNum	= (int)wcslen(pFrom);
	iPos	= 0;

	for(;;)
		{
		   iLen = strifind(pText,pFrom);
		if(iLen<0)break;

		if(iPos+iLen>=iMax)
			{
			memmove(pString+iPos,pText,(iMax-iPos)*2);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pText,iLen*2);
		pText += iLen;
		iPos  += iLen;

		if(iPos+iNum>=iMax)
			{
			memcpy(pString+iPos,pTo,(iMax-iPos)*2);
			pString[iMax]=0;
			return  iMax;
			}

		memmove(pString+iPos,pTo,iAdd*2);
		pText += iNum;
		iPos  += iAdd;
		}


	   iLen  = (int)wcslen(pText);
	if(iLen>iMax-iPos)iLen=iMax-iPos;

	memmove(pString+iPos,pText,iLen*2);
	iPos += iLen;
	pString[iPos]=0;



return iPos;
}

//*****************************************************************************
//*
//*		strireplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus. Dabei wird bei
//	der Suche nicht zwischen Gro- und Kleinbuchstaben unterschieden.
//	pString	: Ist der Zeiger auf den Text der gendert werden soll.
//	pFrom	: Ist der Text der getauscht werden soll.
//	pTo		: Ist der neue Text gegen den pFrom getauscht wird.
//	iMax	: Ist die Gre des Textpuffers in Zeichen.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strireplace(char *pString,const char *pFrom,const char *pTo,int iMax)
{
char   *pBuffer;
int		iNum;
int		iAdd;
int		iLen;


	if(iMax<=0)return 0;

	iAdd	= (int)strlen(pTo  );
	iNum	= (int)strlen(pFrom);

	if(iAdd<=iNum)
		{
		return strireplace(pString,pString,pFrom,pTo,iMax);
		}

	pBuffer = (char*)alloca(iMax);
	iLen	= strireplace(pBuffer,pString,pFrom,pTo,iMax);

	memcpy(pString,pBuffer,iLen+1);


return iLen;
}

int _cdecl strireplace(wchar_t *pString,const wchar_t *pFrom,const wchar_t *pTo,int iMax)
{
wchar_t    *pBuffer;
int			iNum;
int			iAdd;
int			iLen;


	if(iMax<=0)return 0;

	iAdd	= (int)wcslen(pTo  );
	iNum	= (int)wcslen(pFrom);

	if(iAdd<=iNum)
		{
		return strireplace(pString,pString,pFrom,pTo,iMax);
		}

	pBuffer = (wchar_t*)alloca(iMax*2);
	iLen	= strireplace(pBuffer,pString,pFrom,pTo,iMax);

	memcpy(pString,pBuffer,(iLen+1)*2);


return iLen;
}

//*****************************************************************************
//*
//*		strireplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus. Dabei wird bei
//	der Suche nicht zwischen Gro- und Kleinbuchstaben unterschieden.
//	pString	: Ist der Zeiger auf den Puffer in dem der genderte Text gespeichert wird.
//	pText	: Ist der Zeiger auf den Augangstext.
//	cFrom	: Ist das Zeichen das getauscht werden soll.
//	cTo		: Ist das neue Zeichen gegen das cFrom getauscht wird.
//	iMax	: Ist die Gre des Textpuffers in Zeichen.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strireplace(char *pString,const char *pText,char cFrom,char cTo,int iMax)
{
char	cChar;
int		iPos;


	if(iMax<=0)return 0;

	if(!strwchara(cFrom,0x06))
		{
		return strreplace(pString,pText,cFrom,cTo,iMax);
		}

	cChar = (char)(cFrom^0x20);
	iMax--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pText[iPos]==0)break;
		if(pText[iPos]!=cFrom && pText[iPos]!=cChar)
				pString[iPos] = pText[iPos];
		else	pString[iPos] = cTo;
		}


	pString[iPos] = 0;


return iPos;
}

int _cdecl strireplace(wchar_t *pString,const wchar_t *pText,wchar_t cFrom,wchar_t cTo,int iMax)
{
wchar_t	cChar;
int		iPos;


	if(iMax<=0)return 0;

	if(!strwcharw(cFrom,0x06))
		{
		return strreplace(pString,pText,cFrom,cTo,iMax);
		}

	cChar = (wchar_t)(cFrom^0x20);
	iMax--;

	for(iPos=0;iPos<iMax;iPos++)
		{
		if(pText[iPos]==0)break;
		if(pText[iPos]!=cFrom && pText[iPos]!=cChar)
				pString[iPos] = pText[iPos];
		else	pString[iPos] = cTo;
		}


	pString[iPos] = 0;


return iPos;
}

#ifdef 	_DEBUG

int _cdecl strireplace(char *pString,char cFrom,char cTo,int iMax)
		{
		return strireplace(pString,pString,cFrom,cTo,iMax);
		}

int _cdecl strireplace(wchar_t *pString,wchar_t cFrom,wchar_t cTo,int iMax)
		{
		return strireplace(pString,pString,cFrom,cTo,iMax);
		}

#endif


//*****************************************************************************
//*
//*		strireplace
//*
//*****************************************************************************
//	Tauscht einzelne Texte ein einem String gegen andere aus. Dabei wird bei
//	der Suche nicht zwischen Gro- und Kleinbuchstaben unterschieden.
//	pString	: Ist der Zeiger auf den Text der gendert werden soll.
//	cFrom	: Ist das Zeichen das getauscht werden soll.
//	cTo		: Ist das neue Zeichen gegen das cFrom getauscht wird.
//	Ergibt die Lnge des neuen Textes ohne Null-Byte.
int _cdecl strireplace(char *pString,char cFrom,char cTo)
{
char	cChar;
int		iPos;


	if(!strwchara(cFrom,0x06))
		{
		return strreplace(pString,cFrom,cTo);
		}

	cChar = (char)(cFrom^0x20);

	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==cFrom || pString[iPos]==cChar)
			{
			pString[iPos] = cTo;
			}
		}


return iPos;
}

int _cdecl strireplace(wchar_t *pString,wchar_t cFrom,wchar_t cTo)
{
wchar_t	cChar;
int		iPos;


	if(!strwcharw(cFrom,0x06))
		{
		return strreplace(pString,cFrom,cTo);
		}

	cChar = (wchar_t)(cFrom^0x20);

	for(iPos=0;;iPos++)
		{
		if(pString[iPos]==0)break;
		if(pString[iPos]==cFrom || pString[iPos]==cChar)
			{
			pString[iPos] = cTo;
			}
		}


return iPos;
}

//*****************************************************************************
//*
//*		vtostr
//*
//*****************************************************************************
//	Wandelt eine Zahl in einen Einheitentext um. Dabei wird die optimale
//	Einheit gesucht. z.B.:  1234 ->  "1.23 kB"
//	uValue		: Ist die Zahl die umgewandelt werden soll
//	pBuffer		: In diesem Puffer wird der Einheitentext gespeichert
//	uMode		: Bestimmt die Umwandlung
//					VTS_SIGNED     	= Die Zahl ist Vorzeichenbehaftet
//					VTS_SHRINKCOMMA	= Nullen am Ende der Kommastellen entfernen
//					VTS_ADDSUFFIX	= Automatisch eine Einheitanhngen (z.B. 123 K)
//					VTS_ADDSPACE	= Ein Leerzeichen nach den Kommastellen einfgen
//					VTS_CONSTSIZE	= Fr alle Zahlenwerte immer die gleiche Textlnge erzeugen
//					VTS_BIN10		= Statt	mit 1000 wird mit 2^10 dividiert
//					Bit  4.. 7		= Sind die Nachkommastellen
//					Bit  8..11		= Sind die maximalen Vorkommastellen
//					Bit 12..15		= Ist die Ausgangseinheit 10=M 9=k 8=- 7=m 6= ..
//					Bit 16..22		= Ist das Kommazeichen
//					Bit 24..30		= Ist das Tausendertrennzeichen (0=keine Trennung)
//	pSufix		: Ist die Einheit die angehngt werden soll
//					VTS_BYTESUFFIX ist fr den Byte/kB Spezialfall
//	pLevel		: Hier wird der Level gespeichert den die Ausgabeeinheit hat.
//				  Wenn der Zeiger 0 ist wird nicht bergeben.
//					10=M 9=k 8=- 7=m 6= ..
//	Ergibt die Lnge des erzeugten Textes in Zeichen.
int _cdecl vtostr(unsigned __int64 uValue,char *pBuffer,unsigned uMode,const void *pSufixPtr,int *pLevel)
{
const char		   *pSufix=(char*)pSufixPtr;
unsigned __int64	uVal64;
unsigned __int64	uDiv64;
unsigned			uMinus;
unsigned			uLevel;
unsigned			uComma;
unsigned			uChar;
unsigned			uDiv;
unsigned			uPre;
unsigned			uMax;
unsigned			uVal;
unsigned			uLen;
unsigned			uPos;
unsigned			uAdd;
unsigned			uBit;



	if((uMode&VTS_SIGNED) && ((__int64)uValue)<0)		// Vorzeichen bestimmen
		{
		uValue = -(__int64)uValue;
		uMinus = 1;
		}
	else{
		uMinus = 0;
		}


	uPre = (uMode>>8)&0x0F;
	uBit = (uMode&VTS_CONSTSIZE)? 0x010000|(uPre<<8):0;
	uMax = (unsigned)aVtsMax[uPre];

	if(uValue>0xFFFFFFFF)
		{
		uVal64 =  uValue;

		if(uMode&VTS_BIN10)								// 2 Hoch 10 Suche
			{
			for(uLevel=0,uDiv64=1;uVal64>=uMax;uLevel++)
				{
				uVal64 /= 1024;
				uDiv64 *= 1024;
				}
			}
		else{
			for(uLevel=0,uDiv64=1;uVal64>=uMax;uLevel++)
				{
				uVal64 /= 1000;
				uDiv64 *= 1000;
				}
			}

		if(uMinus)
				uLen = qtostr(-(int)uVal64,pBuffer,uBit| 0);
		else	uLen = qtostr(      uVal64,pBuffer,uBit|15);

		uValue -= uVal64*uDiv64;
		}
	else{
		uVal =  (unsigned)uValue;

		if(uMode&VTS_BIN10)								// 2 Hoch 10 Suche
			{
			for(uLevel=0,uDiv=1;uVal>=uMax;uLevel++)
				{
				uVal /= 1024;
				uDiv *= 1024;
				}
			}
		else{
			for(uLevel=0,uDiv=1;uVal>=uMax;uLevel++)
				{
				uVal /= 1000;
				uDiv *= 1000;
				}
			}

		if(uMinus)
				uLen = itostr(-(int)uVal,pBuffer,uBit| 0);
		else	uLen = itostr(      uVal,pBuffer,uBit|15);

		uDiv64  = uDiv;
		uValue -= uVal*uDiv;
		}


	if(uMode&0x7F000000)								// Tausender Trennzeichen
		{
		uChar = (uMode>>24)&0x7F;
		uAdd  = 0;

		for(uPos=3;uPos<uLen;uPos+=3)
			{
			if(pBuffer[uLen-uPos]<'0')break;
			if(pBuffer[uLen-uPos]>'9')break;

			memmove(pBuffer+uLen-uPos+1,pBuffer+uLen-uPos,uPos+uAdd);
			pBuffer[uLen-uPos]=(char)uChar;
			uAdd++;
			}

		if(uMode&VTS_CONSTSIZE)
		for(;uPos<uLen;uPos+=3)
			{
			memmove(pBuffer+uLen-uPos+1,pBuffer+uLen-uPos,uPos+uAdd);
			pBuffer[uLen-uPos]=' ';
			uAdd++;
			}

		uLen += uAdd;
		}

	   uComma = (uMode>>4)&0x0F;
	if(uComma>0)										// Kommastellen anhngen
		{
		if(pSufix==(char*)VTS_BYTESUFFIX && !(uMode&VTS_CONSTSIZE) && uLevel+((uMode>>12)&0x0F)==8)
			{
			uMode |= VTS_SHRINKCOMMA;
			}

			uChar = (uMode>>16)&0x7F;
		if(!uChar)uChar='.';

		pBuffer[uLen] = (char)uChar;
		uLen++;

		uVal64  = aVtsMax[uComma];
		uVal64 *= uValue;
		uVal64 /= uDiv64;

		if(uComma>9)
				uAdd = qtostr(          uVal64,pBuffer+uLen,0x400F|(uComma<<8));
		else	uAdd = itostr((unsigned)uVal64,pBuffer+uLen,0x400F|(uComma<<8));

														// Nullen am Ende entfernen
		if((uMode&VTS_SHRINKCOMMA) && !(uMode&VTS_CONSTSIZE))
			{
			while(uAdd>0)
				{
				if(pBuffer[uLen+uAdd-1]!='0')break;
				uAdd--;
				}

			if(uAdd>0)uLen+=uAdd;
			else	  uLen--;
			}
		else{
			uLen+=uAdd;
			}
		}

	if(uMode&VTS_ADDSPACE)								// Ein Leerzeichen nach den Kommastellen einfgen
		{
		pBuffer[uLen]=' ';
		uLen++;
		}

	uLevel += (uMode>>12)&0x0F;
	if(pLevel)*pLevel=uLevel;

	if(uMode&VTS_ADDSUFFIX)								// Automatisch eine Einheitanhngen (z.B. 123 K)
		{
		if(uLevel!=8)
			{
			pBuffer[uLen]=aVtsPrefix[uLevel];
			uLen++;

			if(pSufix==(char*)VTS_BYTESUFFIX)
				{
				pSufix = (uMode&VTS_CONSTSIZE)? "B  ":"B";
				}

			if(pSufix)
				{
				while(*pSufix)
					{
					pBuffer[uLen]=*pSufix;
					pSufix++;
					uLen++;
					}
				}
			}
		else{
			if(pSufix==(char*)VTS_BYTESUFFIX)
				{
				memcpy(pBuffer+uLen,"Byte",4);
				uLen+=4;
				pSufix=0;
				}
			else if(pSufix)								// Einen Einheitentext anhngen
				{
				while(*pSufix)
					{
					pBuffer[uLen]=*pSufix;
					pSufix++;
					uLen++;
					}

				if(uMode&VTS_CONSTSIZE)
					{
					pBuffer[uLen]=' ';
					uLen++;
					}
				}
			else if(uMode&VTS_CONSTSIZE)
				{
				pBuffer[uLen]=' ';
				uLen++;
				}
			}
		}


	pBuffer[uLen]=0;


return uLen;
}

int _cdecl vtostr(unsigned __int64 uValue,wchar_t *pBuffer,unsigned uMode,const void *pSufixPtr,int *pLevel)
{
const wchar_t	   *pSufix=(wchar_t*)pSufixPtr;
unsigned __int64	uVal64;
unsigned __int64	uDiv64;
unsigned			uMinus;
unsigned			uLevel;
unsigned			uComma;
unsigned			uChar;
unsigned			uDiv;
unsigned			uPre;
unsigned			uMax;
unsigned			uVal;
unsigned			uLen;
unsigned			uPos;
unsigned			uAdd;
unsigned			uBit;



	if((uMode&VTS_SIGNED) && ((__int64)uValue)<0)		// Vorzeichen bestimmen
		{
		uValue = -(__int64)uValue;
		uMinus = 1;
		}
	else{
		uMinus = 0;
		}


	uPre = (uMode>>8)&0x0F;
	uBit = (uMode&VTS_CONSTSIZE)? 0x010000|(uPre<<8):0;
	uMax = (unsigned)aVtsMax[uPre];

	if(uValue>0xFFFFFFFF)
		{
		uVal64 =  uValue;

		if(uMode&VTS_BIN10)								// 2 Hoch 10 Suche
			{
			for(uLevel=0,uDiv64=1;uVal64>=uMax;uLevel++)
				{
				uVal64 /= 1024;
				uDiv64 *= 1024;
				}
			}
		else{
			for(uLevel=0,uDiv64=1;uVal64>=uMax;uLevel++)
				{
				uVal64 /= 1000;
				uDiv64 *= 1000;
				}
			}

		if(uMinus)
				uLen = qtostr(-(int)uVal64,pBuffer,uBit| 0);
		else	uLen = qtostr(      uVal64,pBuffer,uBit|15);

		uValue -= uVal64*uDiv64;
		}
	else{
		uVal =  (unsigned)uValue;

		if(uMode&VTS_BIN10)								// 2 Hoch 10 Suche
			{
			for(uLevel=0,uDiv=1;uVal>=uMax;uLevel++)
				{
				uVal /= 1024;
				uDiv *= 1024;
				}
			}
		else{
			for(uLevel=0,uDiv=1;uVal>=uMax;uLevel++)
				{
				uVal /= 1000;
				uDiv *= 1000;
				}
			}

		if(uMinus)
				uLen = itostr(-(int)uVal,pBuffer,uBit| 0);
		else	uLen = itostr(      uVal,pBuffer,uBit|15);

		uDiv64  = uDiv;
		uValue -= uDiv*uVal;
		}


	if(uMode&0x7F000000)								// Tausender Trennzeichen
		{
		uChar = (uMode>>24)&0x7F;
		uAdd  = 0;

		for(uPos=3;uPos<uLen;uPos+=3)
			{
			if(pBuffer[uLen-uPos]<'0')break;
			if(pBuffer[uLen-uPos]>'9')break;

			memmove(pBuffer+uLen-uPos+1,pBuffer+uLen-uPos,uPos+uAdd);
			pBuffer[uLen-uPos]=(wchar_t)uChar;
			uAdd++;
			}

		if(uMode&VTS_CONSTSIZE)
		for(;uPos<uLen;uPos+=3)
			{
			memmove(pBuffer+uLen-uPos+1,pBuffer+uLen-uPos,uPos+uAdd);
			pBuffer[uLen-uPos]=' ';
			uAdd++;
			}

		uLen += uAdd;
		}

	   uComma = (uMode>>4)&0x0F;
	if(uComma>0)										// Kommastellen anhngen
		{
		if(pSufix==(wchar_t*)VTS_BYTESUFFIX && !(uMode&VTS_CONSTSIZE) && uLevel+((uMode>>12)&0x0F)==8)
			{
			uMode |= VTS_SHRINKCOMMA;
			}

			uChar = (uMode>>16)&0x7F;
		if(!uChar)uChar='.';

		pBuffer[uLen] = (wchar_t)uChar;
		uLen++;

		uVal64  = aVtsMax[uComma];
		uVal64 *= uValue;
		uVal64 /= uDiv64;

		if(uComma>9)
				uAdd = qtostr(          uVal64,pBuffer+uLen,0x400F|(uComma<<8));
		else	uAdd = itostr((unsigned)uVal64,pBuffer+uLen,0x400F|(uComma<<8));

														// Nullen am Ende entfernen
		if((uMode&VTS_SHRINKCOMMA) && !(uMode&VTS_CONSTSIZE))
			{
			while(uAdd>0)
				{
				if(pBuffer[uLen+uAdd-1]!='0')break;
				uAdd--;
				}

			if(uAdd>0)uLen+=uAdd;
			else	  uLen--;
			}
		else{
			uLen+=uAdd;
			}
		}

	if(uMode&VTS_ADDSPACE)								// Ein Leerzeichen nach den Kommastellen einfgen
		{
		pBuffer[uLen]=' ';
		uLen++;
		}

	uLevel += (uMode>>12)&0x0F;
	if(pLevel)*pLevel=uLevel;

	if(uMode&VTS_ADDSUFFIX)								// Automatisch eine Einheitanhngen (z.B. 123 K)
		{
		if(uLevel!=8)
			{
			pBuffer[uLen]=aVtsPrefix[uLevel];
			uLen++;

			if(pSufix==(wchar_t*)VTS_BYTESUFFIX)
				{
				pSufix = (uMode&VTS_CONSTSIZE)? L"B  ":L"B";
				}

			if(pSufix)
				{
				while(*pSufix)
					{
					pBuffer[uLen]=*pSufix;
					pSufix++;
					uLen++;
					}
				}
			}
		else{
			if(pSufix==(wchar_t*)VTS_BYTESUFFIX)
				{
				memcpy(pBuffer+uLen,L"Byte",4*sizeof(wchar_t));
				uLen+=4;
				pSufix=0;
				}
			else if(pSufix)								// Einen Einheitentext anhngen
				{
				while(*pSufix)
					{
					pBuffer[uLen]=*pSufix;
					pSufix++;
					uLen++;
					}

				if(uMode&VTS_CONSTSIZE)
					{
					pBuffer[uLen]=' ';
					uLen++;
					}
				}
			else if(uMode&VTS_CONSTSIZE)
				{
				pBuffer[uLen]=' ';
				uLen++;
				}
			}
		}


	pBuffer[uLen]=0;


return uLen;
}

//*****************************************************************************
//*
//*		strparseurl
//*
//*****************************************************************************
//	Parst eine URL und holt die einzelnen Abschnitte aus dem Text.
//	Wenn ein Abschnitt nicht gefunden wurde so werden die bergebenen Puffer
//	fr diesen Abschnitt nicht verndert.
//	pUrl		: Zeiger auf den URL-Text.
//	pHost		: Zeiger auf einen Puffer fr den Hostnamen.
//				  Wenn 0 dann wird nichts kopiert.
//	iHostSize	: Gre des Host-Puffers in Zeichen.
//	pUser		: Zeiger auf einen Puffer fr den User-Namen.
//				  Wenn 0 dann wird nichts kopiert.
//	pPasswd		: Zeiger auf einen Puffer fr das Passwort.
//				  Wenn 0 dann wird nichts kopiert.
//	pPort		: Zeiger auf einen Puffer fr die Portnummer.
//				  Wenn 0 dann wird nichts kopiert.
//	pProtocol	: Zeiger auf einen Puffer fr die Protokollangabe.
//				  Wenn 0 dann wird nichts kopiert.
//	pPath		: Zeiger auf einen Puffer fr den Pfad.
//				  Wenn 0 dann wird nichts kopiert.
//				  Wenn kein Suffix-Puffer angegeben ist werden auch Suffixe
//				  hier eingefgt.
//	iPathSize	: Gre des Pfadpuffers in Zeichen.
//	pSuffix		: Zeiger auf einen Puffer fr das Suffix. ( "?..." oder "#..." )
//				  Wenn 0 dann wird nichts kopiert.
//	iSuffixSize	: Gre des Suffix-Puffers in Zeichen. Wenn das STR_NOTUTF8
//				  Flag gesetzt ist werden keine UTF8 Zeichern dekotiert.
//	Ergibt ein Bitmuster das bestimmt welche Abschnitte gefunden wurde:
//					STR_URLHOST		0x0001	Ein Hostname wurde in der URL gefunden
//					STR_URLUSER		0x0002	Ein Username wurde in der URL gefunden
//					STR_URLPASSWD	0x0004	Ein Passwort wurde in der URL gefunden
//					STR_URLPORT		0x0008	Eine Portnummer wurde in der URL gefunden
//					STR_URLPROTOCOL	0x0010	Eine Protokollangabe wurde in der URL gefunden
//					STR_URLPATH		0x0020	Ein Pfad wurde in der URL gefunden
//					STR_URLSUFFIX	0x0040	Ein Suffix wurde in der URL gefunden
int _cdecl strparseurl(const char *pUrl,char *pHost,int iHostSize,char *pUser,char *pPasswd,unsigned *pPort,char *pProtocol,char *pPath,int iPathSize,char *pSuffix,int iSuffixSize)
{
int		iStart;
int		iFlags;
int		iPos;
int		iLen;
int		iEnd;
int		iOff;
int		iRet;
int		iVal;




	iRet	= 0;

	if((iSuffixSize&0x7FF0000)==STR_NOTUTF8)				// Keine UTF8-Zeichen dekodieren
		{
		iSuffixSize &= ~0x7FF0000;
		iFlags	= 4;
		}
	else{
		iFlags	= 5;
		}

	for(iPos=0;pUrl[iPos];iPos++)							// Suche nchsten Slash
		{
		if(pUrl[iPos]=='/' )break;
		if(pUrl[iPos]=='\\')break;
		}
															// Gibt es eine Protokollangabe ?
	if(iPos>0 && pUrl[iPos-1]==':' && pUrl[iPos]=='/' && pUrl[iPos+1]=='/')
		{
		iRet |= STR_URLPROTOCOL;

		if(pProtocol)
			{
			iLen = iPos-1;
			strhtmldecode(pUrl,iLen,pProtocol,16,iFlags);
			}

		iPos   += 2;
		iStart  = iPos;

		if(strwchara(pUrl[iPos],0x06) && pUrl[iPos+1]==':')	// Ist es ein Pfad mit Laufwerk
			{
			goto Path;
			}

		for(;pUrl[iPos];iPos++)								// Suche nchsten Slash
			{
			if(pUrl[iPos]=='/' )break;
			if(pUrl[iPos]=='\\')break;
			}
		}													// Format "//server/share"
	else if(iPos==0 && pUrl[0] && (pUrl[1]=='/' || pUrl[1]=='\\'))
		{
		for(iPos=2;pUrl[iPos];iPos++)
			{
			if(pUrl[iPos]=='/' )break;
			if(pUrl[iPos]=='\\')break;
			}

		iStart = 2;
		}
	else{
		iStart = 0;
		}

	iEnd = iPos;


	if(strwchara(pUrl[iStart],0x06) && pUrl[iStart+1]==':')	// Ist es ein Pfad mit Laufwerk
		{
		iPos = iStart;
		goto Path;
		}


	for(iPos=iStart;iPos<iEnd;iPos++)						// Suche '@' Zeichen in Hostnamen
		{
		if(pUrl[iPos]!='@')continue;

		for(iOff=iPos+1;iOff<iEnd;iOff++)					// Suche gibt es weitere '@' Zeichen
			{
			if(pUrl[iOff]!='@')continue;
			iPos = iOff;
			}

		break;
		}


	if(iPos<iEnd)											// Wurde ein Usernamen gefunden
		{
		iRet |= STR_URLUSER;


		for(iOff=iStart;iOff<iPos;iOff++)					// Suche Passwort
			{
			if(pUrl[iOff]!=':')continue;

			iRet |= STR_URLPASSWD;

			if(pPasswd)
				{
				iLen = iPos-iOff-1;
				strhtmldecode(pUrl+iOff+1,iLen,pPasswd,64,iFlags);
				}

			break;
			}

		if(pUser)
			{
			iLen = iOff-iStart;
			strhtmldecode(pUrl+iStart,iLen,pUser,64,iFlags);
			}

		iStart = iPos;
		if(pUrl[iStart])iStart++;
		}


	if(pUrl[iStart]=='[')									// IP-V6
		{
		for(iPos=iStart+1;iPos<iEnd;iPos++)
			{
			if(pUrl[iPos]==']'){iPos++;break;}
			}
		}
	else{													// Host dekodieren
		for(iPos=iStart;iPos<iEnd;iPos++)
			{
			if(pUrl[iPos]==':')break;
			}
		}


	iLen = iPos-iStart;

	if(iLen>0)
		{
		iRet |= STR_URLHOST;

		if(pHost)
			{
			strhtmldecode(pUrl+iStart,iLen,pHost,iHostSize,iFlags);
			}
		}



	if(pUrl[iPos]==':')										// Portnummer suchen
		{
		iPos++;

		if(pUrl[iPos]<='9' && pUrl[iPos]>='0')
			{
			iVal  = strtoi(pUrl+iPos,0,&iLen);
			iPos += iLen;
			iRet |= STR_URLPORT;
			if(pPort)*pPort=iVal;
			}
		}



	if(pUrl[iPos])											// Ist noch ein Pfad angehngt ?
		{
		iPos += 1;
Path:	iRet |= STR_URLPATH;
		iLen  = strcwlen(pUrl+iPos,0x01080000);

		if(pPath)
			{
			if((iPathSize&0x7FF0000)==STR_NOROOTSIGN)		// Kein '/' am Pfadanfang
				{
				iPathSize &= ~0x7FF0000;
				}
			else{
				if(pUrl[iPos]!='.' ||						// Relativer Pfad "./..."
				  (pUrl[iPos+1]!=0 && pUrl[iPos+1]!='/' && pUrl[iPos+1]=='\\'))
					{
					if(iPathSize>1)
						{
						pPath[0]   = '/';
						pPath	  += 1;
						iPathSize -= 1;
						}
					}
				}

			if(pSuffix)
					strhtmldecode(pUrl+iPos,iLen,pPath,iPathSize,iFlags);
			else	strhtmldecode(pUrl+iPos     ,pPath,iPathSize,iFlags);
			}

		iPos += iLen;

		if(pUrl[iPos])										// Ist ein Suffix vorhanden
			{
			iRet |= STR_URLSUFFIX;
			strhtmldecode(pUrl+iPos,pSuffix,iSuffixSize,iFlags);
			}
		}




return iRet;
}

int _cdecl strparseurl(const wchar_t *pUrl,wchar_t *pHost,int iHostSize,wchar_t *pUser,wchar_t *pPasswd,unsigned *pPort,wchar_t *pProtocol,wchar_t *pPath,int iPathSize,wchar_t *pSuffix,int iSuffixSize)
{
int		iStart;
int		iFlags;
int		iPos;
int		iLen;
int		iEnd;
int		iOff;
int		iRet;
int		iVal;




	iRet	= 0;

	if((iSuffixSize&0x7FF0000)==STR_NOTUTF8)				// Keine UTF8-Zeichen dekodieren
		{
		iSuffixSize &= ~0x7FF0000;
		iFlags	= 4;
		}
	else{
		iFlags	= 5;
		}


	for(iPos=0;pUrl[iPos];iPos++)							// Suche nchsten Slash
		{
		if(pUrl[iPos]=='/' )break;
		if(pUrl[iPos]=='\\')break;
		}
															// Gibt es eine Protokollangabe ?
	if(iPos>0 && pUrl[iPos-1]==':' && pUrl[iPos]=='/' && pUrl[iPos+1]=='/')
		{
		iRet |= STR_URLPROTOCOL;

		if(pProtocol)
			{
			iLen = iPos-1;
			strhtmldecode(pUrl,iLen,pProtocol,16,iFlags);
			}

		iPos   += 2;
		iStart  = iPos;

		if(strwcharw(pUrl[iPos],0x06) && pUrl[iPos+1]==':')	// Ist es ein Pfad mit Laufwerk
			{
			goto Path;
			}

		for(;pUrl[iPos];iPos++)								// Suche nchsten Slash
			{
			if(pUrl[iPos]=='/' )break;
			if(pUrl[iPos]=='\\')break;
			}
		}
	else if(iPos==0 && pUrl[0] && (pUrl[1]=='/' || pUrl[1]=='\\'))
		{
		for(iPos=2;pUrl[iPos];iPos++)
			{
			if(pUrl[iPos]=='/' )break;
			if(pUrl[iPos]=='\\')break;
			}

		iStart = 2;
		}
	else{
		iStart = 0;
		}

	iEnd = iPos;


	if(strwcharw(pUrl[iStart],0x06) && pUrl[iStart+1]==':')	// Ist es ein Pfad mit Laufwerk
		{
		iPos = iStart;
		goto Path;
		}


	for(iPos=iStart;iPos<iEnd;iPos++)						// Suche '@' Zeichen in Hostnamen
		{
		if(pUrl[iPos]!='@')continue;

		for(iOff=iPos+1;iOff<iEnd;iOff++)					// Suche gibt es weitere '@' Zeichen
			{
			if(pUrl[iOff]!='@')continue;
			iPos = iOff;
			}

		break;
		}


	if(iPos<iEnd)											// Wurde ein Usernamen gefunden
		{
		iRet |= STR_URLUSER;


		for(iOff=iStart;iOff<iPos;iOff++)					// Suche Passwort
			{
			if(pUrl[iOff]!=':')continue;

			iRet |= STR_URLPASSWD;

			if(pPasswd)
				{
				iLen = iPos-iOff-1;
				strhtmldecode(pUrl+iOff+1,iLen,pPasswd,64,iFlags);
				}

			break;
			}

		if(pUser)
			{
			iLen = iOff-iStart;
			strhtmldecode(pUrl+iStart,iLen,pUser,64,iFlags);
			}

		iStart = iPos;
		if(pUrl[iStart])iStart++;
		}


	if(pUrl[iStart]=='[')									// IP-V6
		{
		for(iPos=iStart+1;iPos<iEnd;iPos++)
			{
			if(pUrl[iPos]==']'){iPos++;break;}
			}
		}
	else{													// Host dekodieren
		for(iPos=iStart;iPos<iEnd;iPos++)
			{
			if(pUrl[iPos]==':')break;
			}
		}


	iLen = iPos-iStart;

	if(iLen>0)
		{
		iRet |= STR_URLHOST;

		if(pHost)
			{
			strhtmldecode(pUrl+iStart,iLen,pHost,iHostSize,iFlags);
			pHost[iLen]=0;
			}
		}



	if(pUrl[iPos]==':')										// Portnummer suchen
		{
		iPos++;

		if(pUrl[iPos]<='9' && pUrl[iPos]>='0')
			{
			iVal  = strtoi(pUrl+iPos,0,&iLen);
			iPos += iLen;
			iRet |= STR_URLPORT;
			if(pPort)*pPort=iVal;
			}
		}


	if(pUrl[iPos])											// Ist noch ein Pfad angehngt ?
		{
		iPos += 1;
Path:	iRet |= STR_URLPATH;
		iLen  = strcwlen(pUrl+iPos,0x01080000);

		if(pPath)
			{
			if((iPathSize&0x7FF0000)==STR_NOROOTSIGN)		// Kein '/' am Pfadanfang
				{
				iPathSize &= ~0x7FF0000;
				}
			else{
				if(pUrl[iPos]!='.' ||						// Relativer Pfad "./..."
				  (pUrl[iPos+1]!=0 && pUrl[iPos+1]!='/' && pUrl[iPos+1]=='\\'))
					{
					if(iPathSize>1)
						{
						pPath[0]   = '/';
						pPath	  += 1;
						iPathSize -= 1;
						}
					}
				}

			if(pSuffix)
					strhtmldecode(pUrl+iPos,iLen,pPath,iPathSize,iFlags);
			else	strhtmldecode(pUrl+iPos     ,pPath,iPathSize,iFlags);
			}

		iPos += iLen;

		if(pUrl[iPos])										// Ist ein Suffix vorhanden
			{
			iRet |= STR_URLSUFFIX;
			strhtmldecode(pUrl+iPos,pSuffix,iSuffixSize,iFlags);
			}
		}




return iRet;
}


//*****************************************************************************
//*
//*		strtoutf8
//*
//*****************************************************************************
//	Kopiert einen Text in einen UTF8-String.
//	pUtf8		: Zeiger auf den Textpuffer fr den UTF8-String.
//	pText		: Zeiger auf den Quelltext.
//	iLen		: Gre des Zielpuffers in Byte.
//	pCount		: Hier wird die Anzahl der UTF8-Zeichnen abgelegt.
//				  Wenn der Zeiger NULL ist wird nichts gespeichert.
//	iTextLen	: Maximale Gre des Quelltextes in Zeichen.
//	Ergibt die Gre des UTF8-String in Byte.
int _cdecl strtoutf8(char *pUtf8,const char *pText,int iLen,int iTextLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;


	iLen -= 4;
	if(iLen<0)return 0;

	for(iOut=0,iPos=0;iOut<iLen;iPos++)
		{
		if(iPos>=iTextLen)break;
		uUnicode = (unsigned char)pText[iPos];
		if(uUnicode==0)break;
		iOut    += chartoutf8(pUtf8+iOut,uUnicode);
		}

	pUtf8[iOut] = 0;



return iOut;
}

int _cdecl strtoutf8(char *pUtf8,const wchar_t *pText,int iLen,int iTextLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;


	iLen -= 4;
	if(iLen<0)return 0;

	for(iOut=0,iPos=0;iOut<iLen;iPos++)
		{
		if(iPos>=iTextLen)break;
		uUnicode = pText[iPos];
		if(uUnicode==0)break;

		if((uUnicode   >>10)==(0xD800>>10) && iPos+1<iLen)
		if((pText[iPos]>>10)==(0xDC00>>10))
			{
			uUnicode   = (0x03FF&uUnicode   )<<10;
			uUnicode  |= (0x03FF&pText[iPos]);
			iPos      += 1;
			}

		iOut += chartoutf8(pUtf8+iOut,uUnicode);
		}

	pUtf8[iOut] = 0;



return iOut;
}

int _cdecl strtoutf8(char *pUtf8,const char *pText,int iLen,int *pCount,int iTextLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;
int			iNum;


	iLen -= 4;
	if(iLen<0)												// Ist der Zielpuffer zu klein
		{
		if(pCount)*pCount=0;
		return 0;
		}

	for(iOut=0,iNum=0,iPos=0;iOut<iLen;iPos++)
		{
		if(iPos>=iTextLen)break;
		uUnicode = (unsigned char)pText[iPos];
		if(uUnicode==0)break;
		iOut    += chartoutf8(pUtf8+iOut,uUnicode);
		iNum    += 1;
		}

	pUtf8[iOut] = 0;
	if(pCount)*pCount=iNum;



return iOut;
}

int _cdecl strtoutf8(char *pUtf8,const wchar_t *pText,int iLen,int *pCount,int iTextLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;
int			iNum;


	iLen -= 4;
	if(iLen<0)												// Ist der Zielpuffer zu klein
		{
		if(pCount)*pCount=0;
		return 0;
		}

	for(iOut=0,iNum=0,iPos=0;iOut<iLen;iPos++)
		{
		if(iPos>=iTextLen)break;
		uUnicode = pText[iPos];
		if(uUnicode==0)break;

		if((uUnicode   >>10)==(0xD800>>10) && iPos+1<iLen)
		if((pText[iPos]>>10)==(0xDC00>>10))
			{
			uUnicode   = (0x03FF&uUnicode   )<<10;
			uUnicode  |= (0x03FF&pText[iPos]);
			iPos      += 1;
			}

		iOut    += chartoutf8(pUtf8+iOut,uUnicode);
		iNum    += 1;
		}

	pUtf8[iOut] = 0;
	if(pCount)*pCount=iNum;


return iOut;
}

int _cdecl strtoutf8(char *pUtf8,const char *pText,int iLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;


	iLen -= 4;
	if(iLen<0)return 0;

	for(iOut=0,iPos=0;iOut<iLen;iPos++)
		{
		uUnicode = (unsigned char)pText[iPos];
		if(uUnicode==0)break;
		iOut += chartoutf8(pUtf8+iOut,uUnicode);
		}

	pUtf8[iOut] = 0;



return iOut;
}

int _cdecl strtoutf8(char *pUtf8,const wchar_t *pText,int iLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;


	iLen -= 4;
	if(iLen<0)return 0;

	for(iOut=0,iPos=0;iOut<iLen;iPos++)
		{
		uUnicode = pText[iPos];
		if(uUnicode==0)break;

		if((uUnicode   >>10)==(0xD800>>10) && iPos+1<iLen)
		if((pText[iPos]>>10)==(0xDC00>>10))
			{
			uUnicode   = (0x03FF&uUnicode   )<<10;
			uUnicode  |= (0x03FF&pText[iPos]);
			iPos      += 1;
			}

		iOut += chartoutf8(pUtf8+iOut,uUnicode);
		}

	pUtf8[iOut] = 0;



return iOut;
}

int _cdecl strtoutf8(char *pUtf8,const char *pText,int iLen,int *pCount)
{
unsigned	uUnicode;
int			iOut;
int			iPos;
int			iNum;


	iLen -= 4;
	if(iLen<0)
		{
		if(pCount)*pCount=0;
		return 0;
		}

	for(iOut=0,iNum=0,iPos=0;iOut<iLen;iPos++)
		{
		uUnicode = (unsigned char)pText[iPos];
		if(uUnicode==0)break;
		iOut    += chartoutf8(pUtf8+iOut,uUnicode);
		iNum    += 1;
		}

	pUtf8[iOut] = 0;
	if(pCount)*pCount=iNum;



return iOut;
}

int _cdecl strtoutf8(char *pUtf8,const wchar_t *pText,int iLen,int *pCount)
{
unsigned	uUnicode;
int			iOut;
int			iPos;
int			iNum;


	iLen -= 4;
	if(iLen<0)
		{
		if(pCount)*pCount=0;
		return 0;
		}

	for(iOut=0,iNum=0,iPos=0;iOut<iLen;iPos++)
		{
		uUnicode = pText[iPos];
		if(uUnicode==0)break;

		if((uUnicode   >>10)==(0xD800>>10) && iPos+1<iLen)
		if((pText[iPos]>>10)==(0xDC00>>10))
			{
			uUnicode   = (0x03FF&uUnicode   )<<10;
			uUnicode  |= (0x03FF&pText[iPos]);
			iPos      += 1;
			}

		iOut    += chartoutf8(pUtf8+iOut,uUnicode);
		iNum    += 1;
		}

	pUtf8[iOut] = 0;
	if(pCount)*pCount=iNum;


return iOut;
}

//*****************************************************************************
//*
//*		strfromutf8
//*
//*****************************************************************************
//	Kopiert einen UTF8-String in einen Textpuffer.
//	pText		: Zeiger auf den Textpuffer.
//	pUtf8		: Zeiger auf den UTF8-String.
//	iLen		: Gre des Zielpuffers in Zeichen.
//	iTextLen	: Maximale Gre des UTF8-Strings in Byte.
//	Ergibt die Gre des kopierten Textes in Zeichen.
int _cdecl strfromutf8(char *pText,const char *pUtf8,int iLen,int iTextLen)
{
unsigned	uUnicode;
char		cTemp[4];
int			iOut;
int			iPos;


	iLen--;
	if(iLen<0)return 0;


	for(iOut=0,iPos=0;;iOut++)
		{
		if(iOut>=iLen)goto Exit;
		if(iPos+4>iTextLen)break;
		iPos += utf8tochar(pUtf8+iPos,uUnicode);
		if(uUnicode==0)goto Exit;
		if(uUnicode>255)uUnicode=255;
		pText[iOut] = (char)uUnicode;
		}

	memcpy(cTemp,pUtf8+iPos,iTextLen-iPos);
	cTemp[iTextLen-iPos]=0;

	for(iPos=0;;iOut++)										// Die letzten 3 Byte kopieren
		{
		if(iOut>=iLen)break;
		iPos += utf8tochar(cTemp+iPos,uUnicode);
		if(uUnicode==0)break;
		if(uUnicode>255)uUnicode=255;
		pText[iOut] = (char)uUnicode;
		}

Exit:

	pText[iOut] = 0;



return iOut;
}

int _cdecl strfromutf8(wchar_t *pText,const char *pUtf8,int iLen,int iTextLen)
{
unsigned	uUnicode;
char		cTemp[4];
int			iOut;
int			iPos;


	iLen -= 2;

	if(iLen<0)												// Ist der Zielpuffer zu klein
		{
		if(iLen==-1)*pText=0;
		return 0;
		}

	for(iOut=0,iPos=0;;)
		{
		if(iOut>=iLen)goto Exit;
		if(iPos+4>iTextLen)break;
		iPos += utf8tochar(pUtf8+iPos,uUnicode);
		if(uUnicode==0)goto Exit;
		iOut += chartoutf16(pText+iOut,uUnicode);
		}

	memcpy(cTemp,pUtf8+iPos,iTextLen-iPos);
	cTemp[iTextLen-iPos]=0;

	for(iPos=0;;)											// Die letzten 3 Byte kopieren
		{
		if(iOut>=iLen)break;
		iPos += utf8tochar(cTemp+iPos,uUnicode);
		if(uUnicode==0)break;
		iOut += chartoutf16(pText+iOut,uUnicode);
		}

Exit:

	pText[iOut] = 0;



return iOut;
}

int _cdecl strfromutf8(char *pText,const char *pUtf8,int iLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;


	iLen--;
	if(iLen<0)return 0;


	for(iOut=0,iPos=0;;iOut++)
		{
		if(iOut>=iLen)break;
		iPos += utf8tochar(pUtf8+iPos,uUnicode);
		if(uUnicode==0)break;
		if(uUnicode>255)uUnicode=255;
		pText[iOut] = (char)uUnicode;
		}

	pText[iOut] = 0;



return iOut;
}

int _cdecl strfromutf8(wchar_t *pText,const char *pUtf8,int iLen)
{
unsigned	uUnicode;
int			iOut;
int			iPos;


	iLen -= 2;

	if(iLen<0)												// Ist der Zielpuffer zu klein
		{
		if(iLen==-1)*pText=0;
		return 0;
		}

	for(iOut=0,iPos=0;;)
		{
		if(iOut>=iLen)break;
		iPos += utf8tochar(pUtf8+iPos,uUnicode);
		if(uUnicode==0)break;
		iOut += chartoutf16(pText+iOut,uUnicode);
		}

	pText[iOut] = 0;



return iOut;
}

//*****************************************************************************
//*
//*		strawcpy
//*
//*****************************************************************************
//	Kopiert einen Text in einen Puffer.
//	Dabei werden CHAR/WCHAR Umwandlung durchgefhrt
//	pBuffer		: Ist der Zeiger auf den Textpuffer in dem der Text abgelegt wird.
//	pText		: Ist der Zeiger auf den Quelltext.
//	iLen		: Ist die Gre des Textpuffers in Zeichen.
//	iTextLen	: Ist die Gre des Quelltextes in Zeichen.
//	Ergibt die neue Lnge des Textes in Zeichen.
int _cdecl strawcpy(char *pBuffer,const wchar_t *pText,int iLen,int iTextLen)
{
unsigned  	uChar;
int			iPos;


	iLen--;
	if(iLen<=0)return 0;

	for(iPos=0;iPos<iLen;iPos++)
		{
		if(iPos>=iTextLen)break;
		uChar = pText[iPos];
		if(uChar ==0)break;
		if(uChar>255)uChar=255;
		pBuffer[iPos]= (char)uChar;
		}


	pBuffer[iPos] = 0;


return iPos;
}

int _cdecl strawcpy(wchar_t *pBuffer,const char *pText,int iLen,int iTextLen)
{
int		iPos;


	iLen--;
	if(iLen<=0)return 0;

	for(iPos=0;iPos<iLen;iPos++)
		{
		if(iPos>=iTextLen)break;
		if(pText[iPos]==0)break;
		pBuffer[iPos]=(unsigned char)pText[iPos];
		}


	pBuffer[iPos] = 0;


return iPos;
}

#ifdef _DEBUG
int _cdecl strawcpy(wchar_t *pBuffer,const wchar_t *pText,int iLen,int iTextLen)
{
return strlcpy(pBuffer,pText,iLen,iTextLen);
}

int _cdecl strawcpy(char *pBuffer,const char *pText,int iLen,int iTextLen)
{
return strlcpy(pBuffer,pText,iLen,iTextLen);
}
#endif


#ifdef	_MSC_VER
#pragma warning( default :4035 )
#endif


//*****************************************************************************
//*
//*		strsubtext
//*
//*****************************************************************************
//	Sucht einen Text in einer Textliste.
//	pText		: Ist der gesuchte Text
//	pStrings	: Ist die Textliste. (z.B. "abc|xy|123" oder "abc\0xyz\0123\0")
//	cSign		: Ist das Trennzeichen fr die Textliste.
//	Ergibt den Index in der Textliste vom Text, oder -1 wenn der Text
//	nicht gefunden wurde.
int _cdecl strsubtext(const char *pText,const char *pStrings,char cSign)
{
int		iIndex;
int		iLen;
int		iCnt;
int		iAdd;


	iLen = (int)strlen(pText);

	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memcmp(pText,pStrings,iCnt))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strsubtext(const wchar_t *pText,const wchar_t *pStrings,wchar_t cSign)
{
int		iIndex;
int		iLen;
int		iCnt;
int		iAdd;


	iLen = (int)wcslen(pText);

	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memcmp(pText,pStrings,iCnt*sizeof(wchar_t)))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strsubtext(const char *pText,const char **pStrings,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!strcmp(pText,*pStrings))
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

int _cdecl strsubtext(const wchar_t *pText,const wchar_t **pStrings,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!wcscmp(pText,*pStrings))
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

//*****************************************************************************
//*
//*		strisubtext
//*
//*****************************************************************************
//	Sucht einen Text in einer Textliste ohne zwischen Grokleinbuchstaben zu unterscheiden.
//	pText		: Ist der gesuchte Text
//	pStrings	: Ist die Textliste. (z.B. "abc|xy|123" oder "abc\0xyz\0123\0")
//	cSign		: Ist das Trennzeichen fr die Textliste.
//	Ergibt den Index in der Textliste vom Text, oder -1 wenn der Text
//	nicht gefunden wurde.
int _cdecl strisubtext(const char *pText,const char *pStrings,char cSign)
{
int		iIndex;
int		iLen;
int		iCnt;
int		iAdd;


	iLen = (int)strlen(pText);

	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memicmp(pText,pStrings,iCnt))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strisubtext(const wchar_t *pText,const wchar_t *pStrings,wchar_t cSign)
{
int		iIndex;
int		iLen;
int		iCnt;
int		iAdd;


	iLen = (int)wcslen(pText);

	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memicmp(pText,pStrings,iCnt*sizeof(wchar_t)))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strisubtext(const char *pText,const char **pStrings,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!stricmp(pText,*pStrings))
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

int _cdecl strisubtext(const wchar_t *pText,const wchar_t **pStrings,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!wcsicmp(pText,*pStrings))
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

//*****************************************************************************
//*
//*		strlsubtext
//*
//*****************************************************************************
//	Sucht einen Text in einer Textliste.
//	pText		: Ist der gesuchte Text
//	pStrings	: Ist die Textliste. (z.B. "abc|xy|123" oder "abc\0xyz\0123\0")
//	iLen		: Ist die Lnge des Textes
//	cSign		: Ist das Trennzeichen fr die Textliste.
//	Ergibt den Index in der Textliste vom Text, oder -1 wenn der Text
//	nicht gefunden wurde.
int _cdecl strlsubtext(const char *pText,const char *pStrings,int iLen,char cSign)
{
int		iIndex;
int		iCnt;
int		iAdd;


	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memcmp(pText,pStrings,iCnt))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strlsubtext(const wchar_t *pText,const wchar_t *pStrings,int iLen,wchar_t cSign)
{
int		iIndex;
int		iCnt;
int		iAdd;


	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memcmp(pText,pStrings,iCnt*sizeof(wchar_t)))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strlsubtext(const char *pText,const char **pStrings,int iLen,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!strncmp(pText,*pStrings,iLen) && !(*pStrings)[iLen])
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

int _cdecl strlsubtext(const wchar_t *pText,const wchar_t **pStrings,int iLen,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!wcsncmp(pText,*pStrings,iLen) && !(*pStrings)[iLen])
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

//*****************************************************************************
//*
//*		strlisubtext
//*
//*****************************************************************************
//	Sucht einen Text in einer Textliste ohne zwischen Grokleinbuchstaben zu unterscheiden.
//	pText		: Ist der gesuchte Text
//	pStrings	: Ist die Textliste. (z.B. "abc|xy|123" oder "abc\0xyz\0123\0")
//	iLen		: Ist die Lnge des Textes
//	cSign		: Ist das Trennzeichen fr die Textliste.
//	Ergibt den Index in der Textliste vom Text, oder -1 wenn der Text
//	nicht gefunden wurde.
int _cdecl strlisubtext(const char *pText,const char *pStrings,int iLen,char cSign)
{
int		iIndex;
int		iCnt;
int		iAdd;


	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memicmp(pText,pStrings,iCnt))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strlisubtext(const wchar_t *pText,const wchar_t *pStrings,int iLen,wchar_t cSign)
{
int		iIndex;
int		iCnt;
int		iAdd;


	for(iIndex=0,iAdd=0;*pStrings;iIndex++)
		{
		for(iCnt=0;;iCnt++)
			{
			if(pStrings[iCnt]==cSign)
				{
				iAdd=1;
				break;
				}

			if(pStrings[iCnt]==0)
				{
				iAdd=0;
				break;
				}
			}

		if(iCnt==iLen && !memicmp(pText,pStrings,iCnt*sizeof(wchar_t)))
			{
			return iIndex;
			}

		pStrings += iCnt+iAdd;
		}

	if(iLen==0 && iAdd && cSign)
		{
		return iIndex;
		}


return -1;
}

int _cdecl strlisubtext(const char *pText,const char **pStrings,int iLen,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!strnicmp(pText,*pStrings,iLen) && !(*pStrings)[iLen])
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

int _cdecl strlisubtext(const wchar_t *pText,const wchar_t **pStrings,int iLen,int iMax)
{
int		iIndex;


	for(iIndex=0;iIndex<iMax && *pStrings;iIndex++)
		{
		if(!wcsnicmp(pText,*pStrings,iLen) && !(*pStrings)[iLen])
			{
			return iIndex;
			}

		pStrings++;
		}


return -1;
}

//*****************************************************************************
//*
//*		strsubget
//*
//*****************************************************************************
//	Speichert einen Subtext aus einer Textliste in einen Textpuffer ab.
//	pString		: Zeiger auf den Textpuffer.
//	pList		: Ist die Textliste in der der Text gesucht wird.
//					z.B. "abc|xy|123" , "abc\0xyz\0123\0"
//	iLen		: Ist die Gre der Zielpuffers in Zeichen.
//	uIndex		: Ist die Nummer des Subtext der kopiert werden soll.
//				  Wenn das Bit 0x80000000 gesetzt ist werden die unteren 31 Bits
//				  als Maske fr die zu kopierenden Subtexte benutzt.
//	cSign		: Ist das Trennzeichen fr die Textliste.
//	Ergibt die neue Lnge des Textpuffers ohne Nullzeichen in Zeichen.
int _cdecl strsubget(char *pString,const char *pList,int iLen,unsigned uIndex,char cSign)
{
unsigned	uMask;
unsigned	uLen;
unsigned	uPos;
unsigned	uAdd;


	if(iLen<=0)return 0;
	iLen--;


	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uPos=0;*pList;pList+=uLen+uAdd)						// Suche Subtexte
		{
		for(uLen=0;;uLen++)
			{
			if(pList[uLen]==cSign)
				{
				uAdd=1;
				break;
				}

			if(pList[uLen]==0)
				{
				uAdd=0;
				break;
				}
			}

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		if((int)(uPos+uLen)>=iLen)							// Puffergre prfen
			{
			memcpy(pString+uPos,pList,iLen-uPos);
			uPos = iLen;
			break;
			}

		memcpy(pString+uPos,pList,uLen);					// Subtext kopieren
		uPos   += uLen;
		uMask >>= 1;

		if(!uMask)break;
		}


	pString[uPos] = 0;


return uPos;
}

int _cdecl strsubget(wchar_t *pString,const wchar_t *pList,int iLen,unsigned uIndex,wchar_t cSign)
{
unsigned	uMask;
unsigned	uLen;
unsigned	uPos;
unsigned	uAdd;


	if(iLen<=0)return 0;
	iLen--;


	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uPos=0;*pList;pList+=uLen+1)						// Suche Subtexte
		{
		for(uLen=0;;uLen++)
			{
			if(pList[uLen]==cSign)
				{
				uAdd=1;
				break;
				}

			if(pList[uLen]==0)
				{
				uAdd=0;
				break;
				}
			}

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		if((int)(uPos+uLen)>=iLen)							// Puffergre prfen
			{
			memcpy(pString+uPos,pList,(iLen-uPos)*sizeof(wchar_t));
			uPos = iLen;
			break;
			}

		memcpy(pString+uPos,pList,uLen*sizeof(wchar_t));	// Subtext kopieren
		uPos   += uLen;
		uMask >>= 1;

		if(!uMask)break;
		}


	pString[uPos] = 0;


return uPos;
}

//*****************************************************************************
//*
//*		strsubget
//*
//*****************************************************************************
//	Holt den Zeiger auf einen Subtext aus einer Textliste.
//	pList		: Ist die Textliste in der der Text gesucht wird.
//					z.B. "abc|xyz|123" , "abc\0xyz\0123\0"
//	uIndex		: Ist die Nummer des Subtext der kopiert werden soll.
//				  Wenn das Bit 0x80000000 gesetzt ist werden die unteren 31 Bits
//				  als Maske fr die zu kopierenden Subtexte benutzt.
//	cSign		: Ist das Trennzeichen fr die Textliste.
//	pLen		: Hier wird die Lnge des Subtextes gespeichert
//	pDefault	: Dieser Zeiger wird zurck gegeben wenn kein Subtext gefunden wurde.
//	Ergibt die neue Lnge des Textpuffers ohne Nullzeichen in Zeichen.
const char *_cdecl strsubget(const char *pList,unsigned uIndex,char cSign,int *pLen,const char *pDefault)
{
unsigned	uLen;
unsigned	uAdd;




	for(;*pList;pList+=uLen+uAdd)							// Suche Subtexte
		{
		if(uIndex==0 && !pLen)								// Subtext erreicht
			{
			return pList;
			}

		for(uLen=0;;uLen++)									// Lnge vom Subtext ermitteln
			{
			if(pList[uLen]==cSign)
				{
				uAdd=1;
				break;
				}

			if(pList[uLen]==0)
				{
				uAdd=0;
				break;
				}
			}

		if(uIndex==0)										// Subtext erreicht
			{
			*pLen = uLen;
			return pList;
			}

		uIndex--;
		}


	if(pLen)
		{
		if(pDefault)
				*pLen=(int)strlen(pDefault);
		else	*pLen=0;
		}


return pDefault;
}

const wchar_t *_cdecl strsubget(const wchar_t *pList,unsigned uIndex,wchar_t cSign,int *pLen,const wchar_t *pDefault)
{
unsigned	uLen;
unsigned	uAdd;




	for(;*pList;pList+=uLen+uAdd)							// Suche Subtexte
		{
		if(uIndex==0 && !pLen)								// Subtext erreicht
			{
			return pList;
			}

		for(uLen=0;;uLen++)									// Lnge vom Subtext ermitteln
			{
			if(pList[uLen]==cSign)
				{
				uAdd=1;
				break;
				}

			if(pList[uLen]==0)
				{
				uAdd=0;
				break;
				}
			}

		if(uIndex==0)										// Subtext erreicht
			{
			*pLen = uLen;
			return pList;
			}

		uIndex--;
		}


	if(pLen)
		{
		if(pDefault)
				*pLen=(int)wcslen(pDefault);
		else	*pLen=0;
		}


return pDefault;
}

//*****************************************************************************
//*
//*		strsubget
//*
//*****************************************************************************
//	Speichert einen Subtext aus einer Textliste in einen Textpuffer ab.
//	pString		: Zeiger auf den Textpuffer.
//	pList		: Ist die Textliste bestehend aus Zeigen.
//				  Das Ende wird mit einem Nullzeiger markiert.
//	iLen		: Ist die Gre der Zielpuffers in Zeichen.
//	uIndex		: Ist die Nummer des Subtext der kopiert werden soll.
//				  Wenn das Bit 0x80000000 gesetzt ist werden die unteren 31 Bits
//				  als Maske fr die zu kopierenden Subtexte benutzt.
//	iMax		: Ist die maximale Anzahl an Eintrgen in der Liste.
//	Ergibt die neue Lnge des Textpuffers ohne Nullzeichen in Zeichen.
int _cdecl strsubget(char *pString,const char **pList,int iLen,unsigned uIndex,int iMax)
{
unsigned	uMask;
unsigned	uLen;
unsigned	uPos;


	if(iLen<=0)return 0;
	iLen--;


	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uPos=0;iMax>0;pList++,iMax--)						// Suche Subtexte
		{
		if(!*pList)break;

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		uLen = (int)strlen(*pList);

		if((int)(uPos+uLen)>=iLen)							// Puffergre prfen
			{
			memcpy(pString+uPos,*pList,iLen-uPos);
			uPos = iLen;
			break;
			}

		memcpy(pString+uPos,*pList,uLen);					// Subtext kopieren
		uPos   += uLen;
		uMask >>= 1;

		if(!uMask)break;
		}


	pString[uPos] = 0;


return uPos;
}

int _cdecl strsubget(wchar_t *pString,const wchar_t **pList,int iLen,unsigned uIndex,int iMax)
{
unsigned	uMask;
unsigned	uLen;
unsigned	uPos;


	if(iLen<=0)return 0;
	iLen--;


	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uPos=0;iMax>0;pList++,iMax--)						// Suche Subtexte
		{
		if(!*pList)break;

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		uLen = (int)wcslen(*pList);

		if((int)(uPos+uLen)>=iLen)							// Puffergre prfen
			{
			memcpy(pString+uPos,*pList,(iLen-uPos)*sizeof(wchar_t));
			uPos = iLen;
			break;
			}

		memcpy(pString+uPos,*pList,uLen*sizeof(wchar_t));	// Subtext kopieren
		uPos   += uLen;
		uMask >>= 1;

		if(!uMask)break;
		}


	pString[uPos] = 0;


return uPos;
}

//*****************************************************************************
//*
//*		strsublen
//*
//*****************************************************************************
//	Holt die Lnge von einem Subtext aus einer Textliste.
//	pList		: Ist die Textliste in der der Text gesucht wird.
//					z.B. "abc|xy|123" , "abc\0xyz\0123\0"
//	uIndex		: Ist die Nummer des Subtext der kopiert werden soll.
//				  Wenn das Bit 0x80000000 gesetzt ist werden die unteren 31 Bits
//				  als Maske fr die zu kopierenden Subtexte benutzt.
//	cSign		: Ist das Trennzeichen fr die Textliste.
//	pPos		: Hier wird die Position des ersten gefunden Subtextes
//				  gespeichert, wenn der Zeiger ungleich 0 ist.
//	Ergibt die Lnge des Subtextes, bzw. der kombinierten Subtexte.
int _cdecl strsublen(const char *pList,unsigned uIndex,char cSign,int *pPos)
{
const char *pStart=pList;
unsigned	uMask;
unsigned	uSize;
unsigned	uLen;
unsigned	uAdd;



	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uSize=0;*pList;pList+=uLen+uAdd)					// Suche Subtexte
		{
		for(uLen=0;;uLen++)
			{
			if(pList[uLen]==cSign)
				{
				uAdd=1;
				break;
				}

			if(pList[uLen]==0)
				{
				uAdd=0;
				break;
				}
			}

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		if(pPos)											// Erste Position speichern
			{
		   *pPos = (int)(pList-pStart);
			pPos = 0;
			}

		uSize  += uLen;
		uMask >>= 1;

		if(!uMask)break;
		}


	if(pPos)												// Endposition speichern
		{
	   *pPos = (int)(pList-pStart);
		}


return uSize;
}

int _cdecl strsublen(const wchar_t *pList,unsigned uIndex,wchar_t cSign,int *pPos)
{
const wchar_t  *pStart=pList;
unsigned		uMask;
unsigned		uSize;
unsigned		uLen;
unsigned		uAdd;


	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uSize=0;*pList;pList+=uLen+1)						// Suche Subtexte
		{
		for(uLen=0;;uLen++)
			{
			if(pList[uLen]==cSign)
				{
				uAdd=1;
				break;
				}

			if(pList[uLen]==0)
				{
				uAdd=0;
				break;
				}
			}

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		if(pPos)											// Erste Position speichern
			{
		   *pPos = (int)(pList-pStart);
			pPos = 0;
			}

		uSize  += uLen;
		uMask >>= 1;

		if(!uMask)break;
		}


	if(pPos)												// Endposition speichern
		{
	   *pPos = (int)(pList-pStart);
		}


return uSize;
}

//*****************************************************************************
//*
//*		strsublen
//*
//*****************************************************************************
//	Holt die Lnge von einem Subtext aus einer Textliste.
//	pList		: Ist die Textliste bestehend aus Zeigen.
//				  Das Ende wird mit einem Nullzeiger markiert.
//	uIndex		: Ist die Nummer des Subtext der kopiert werden soll.
//				  Wenn das Bit 0x80000000 gesetzt ist werden die unteren 31 Bits
//				  als Maske fr die zu kopierenden Subtexte benutzt.
//	iMax		: Ist die maximale Anzahl an Eintrgen in der Liste.
//	pPos		: Hier wird die Position des ersten gefunden Subtextes
//				  gespeichert, wenn der Zeiger ungleich 0 ist.
//	Ergibt die Lnge des Subtextes, bzw. der kombinierten Subtexte.
int _cdecl strsublen(const char **pList,unsigned uIndex,int iMax,int *pPos)
{
const char    **pStart=pList;
unsigned		uMask;
unsigned		uSize;


	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uSize=0;iMax>0;pList++,iMax--)						// Suche Subtexte
		{
		if(!*pList)break;

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		if(pPos)											// Erste Position speichern
			{
		   *pPos = (int)(pList-pStart);
			pPos = 0;
			}

		uSize  += (int)strlen(*pList);
		uMask >>= 1;

		if(!uMask)break;
		}


	if(pPos)												// Endposition speichern
		{
	   *pPos = (int)(pList-pStart);
		}


return uSize;
}

int _cdecl strsublen(const wchar_t **pList,unsigned uIndex,int iMax,int *pPos)
{
const wchar_t **pStart=pList;
unsigned		uMask;
unsigned		uSize;


	if(uIndex&0x80000000)									// Maske benutzen
		{
		uMask	= uIndex&0x7FFFFFFF;
		uIndex	= 0;
		}
	else{													// Index benutzen
		uMask	= 1;
		}


	for(uSize=0;iMax>0;pList++,iMax--)						// Suche Subtexte
		{
		if(!*pList)break;

		if(uIndex)											// Noch nicht erreicht
			{
			uIndex--;
			continue;
			}

		if(!(uMask&1))										// Subtext nicht ausmaskiert
			{
			uMask >>= 1;
			continue;
			}

		if(pPos)											// Erste Position speichern
			{
		   *pPos = (int)(pList-pStart);
			pPos = 0;
			}

		uSize  += (int)wcslen(*pList);
		uMask >>= 1;

		if(!uMask)break;
		}


	if(pPos)												// Endposition speichern
		{
	   *pPos = (int)(pList-pStart);
		}


return uSize;
}

//*****************************************************************************
//*
//*		strpcmp
//*
//*****************************************************************************
//	Vergleicht zwei Pfad-Strings. Die Zeichen '/' und '\' werden dabei als
//	gleiche Zeichen betrachtet.
//	pStr1	: Zeiger auf den ersten Pfad-String.
//	pStr2	: Zeiger auf den zweiten Pfad-String.
//	iMax	: Maximale Vergleichslnge in Zeichen.
//	Ergibt Null wenn die Pfad-Strings gleich sind oder:
//				>0 wenn pStr1 grer pStr2 ist
//				<0 wenn pStr1 kleiner pStr2 ist
int _cdecl strpcmp(const char *pStr1,const char *pStr2,int iMax)
{
unsigned char 	uChar1;
unsigned char 	uChar2;
int				iPos;



	for(iPos=0;iPos<iMax;iPos++)
		{
		uChar1 = pStr1[iPos];
		uChar2 = pStr2[iPos];

		if(uChar1==uChar2)
			{
			if(uChar1)continue;
			break;
			}

		if(uChar1==0)return -1;
		if(uChar2==0)return  1;
		if(uChar1=='\\')uChar1=0;
		if(uChar2=='\\')uChar2=0;
		if(uChar1=='/' )uChar1=0;
		if(uChar2=='/' )uChar2=0;
		if(uChar1=='.' )uChar1=1;
		if(uChar2=='.' )uChar2=1;

		if(uChar1!=uChar2)
			{
			return uChar1-uChar2;
			}
		}


return 0;
}

int _cdecl strpcmp(const wchar_t *pStr1,const wchar_t *pStr2,int iMax)
{
unsigned short 	uChar1;
unsigned short 	uChar2;
int				iPos;



	for(iPos=0;iPos<iMax;iPos++)
		{
		uChar1 = pStr1[iPos];
		uChar2 = pStr2[iPos];

		if(uChar1==uChar2)
			{
			if(uChar1)continue;
			break;
			}

		if(uChar1==0)return -1;
		if(uChar2==0)return  1;
		if(uChar1=='\\')uChar1=0;
		if(uChar2=='\\')uChar2=0;
		if(uChar1=='/' )uChar1=0;
		if(uChar2=='/' )uChar2=0;
		if(uChar1=='.' )uChar1=1;
		if(uChar2=='.' )uChar2=1;

		if(uChar1!=uChar2)
			{
			return uChar1-uChar2;
			}
		}


return 0;
}

//*****************************************************************************
//*
//*		stripcmp
//*
//*****************************************************************************
//	Vergleicht zwei Pfad-Strings. Die Zeichen '/' und '\' werden dabei als
//	gleiche Zeichen betrachtet.
//	pStr1	: Zeiger auf den ersten Pfad-String.
//	pStr2	: Zeiger auf den zweiten Pfad-String.
//	iMax	: Maximale Vergleichslnge in Zeichen.
//	Ergibt Null wenn die Pfad-Strings gleich sind oder:
//				>0 wenn pStr1 grer pStr2 ist
//				<0 wenn pStr1 kleiner pStr2 ist
int _cdecl strpicmp(const char *pStr1,const char *pStr2,int iMax)
{
unsigned char 	uChar1;
unsigned char 	uChar2;
int				iPos;



	for(iPos=0;iPos<iMax;iPos++)
		{
		uChar1 = pStr1[iPos];
		uChar2 = pStr2[iPos];

		if(uChar1==uChar2)
			{
			if(uChar1)continue;
			break;
			}

		if(uChar1==0)return -1;
		if(uChar2==0)return  1;

		if(uChar1<='\\')
			{
			if(uChar1>='A')
				{
				     if(uChar1<='Z' )uChar1+='a'-'A';
				else if(uChar1=='\\')uChar1=0;
				}
			else{
				if(uChar1=='/')uChar1=0;
				if(uChar1=='.')uChar1=1;
				}
			}

		if(uChar2<='\\')
			{
			if(uChar2>='A')
				{
				     if(uChar2<='Z' )uChar2+='a'-'A';
				else if(uChar2=='\\')uChar2=0;
				}
			else{
				if(uChar2=='/')uChar2=0;
				if(uChar2=='.')uChar2=1;
				}
			}

		if(uChar1!=uChar2)
			{
			return uChar1-uChar2;
			}
		}


return 0;
}

int _cdecl strpicmp(const wchar_t *pStr1,const wchar_t *pStr2,int iMax)
{
unsigned short 	uChar1;
unsigned short 	uChar2;
int				iPos;



	for(iPos=0;iPos<iMax;iPos++)
		{
		uChar1 = pStr1[iPos];
		uChar2 = pStr2[iPos];

		if(uChar1==uChar2)
			{
			if(uChar1)continue;
			break;
			}

		if(uChar1==0)return -1;
		if(uChar2==0)return  1;

		if(uChar1<='\\')
			{
			if(uChar1>='A')
				{
				     if(uChar1<='Z' )uChar1+='a'-'A';
				else if(uChar1=='\\')uChar1=0;
				}
			else{
				if(uChar1=='/')uChar1=0;
				if(uChar1=='.')uChar1=1;
				}
			}

		if(uChar2<='\\')
			{
			if(uChar2>='A')
				{
				     if(uChar2<='Z' )uChar2+='a'-'A';
				else if(uChar2=='\\')uChar2=0;
				}
			else{
				if(uChar2=='/')uChar2=0;
				if(uChar2=='.')uChar2=1;
				}
			}

		if(uChar1!=uChar2)
			{
			return uChar1-uChar2;
			}
		}


return 0;
}



const HtmlEntity aHtmlEntity[STR_COUNT_HTML] =
	{
	 193, 6, "Aacute",
	 194, 5, "Acirc",
	 198, 5, "AElig",
	 192, 6, "Agrave",
	 197, 5, "Aring",
	 195, 6, "Atilde",
	 196, 4, "Auml",
	 199, 6, "Ccedil",
	 201, 6, "Eacute",
	 202, 5, "Ecirc",
	 200, 6, "Egrave",
	 208, 3, "ETH",
	 203, 4, "Euml",
	 205, 6, "Iacute",
	 206, 5, "Icirc",
	 207, 4, "Iuml",
	 209, 6, "Ntilde",
	 211, 6, "Oacute",
	 212, 5, "Ocirc",
	 210, 6, "Ograve",
	 216, 6, "Oslash",
	 213, 6, "Otilde",
	 214, 4, "Ouml",
	 222, 5, "THORN",
	 218, 6, "Uacute",
	 219, 5, "Ucirc",
	 217, 6, "Ugrave",
	 220, 4, "Uuml",
	 221, 6, "Yacute",
	 225, 6, "aacute",
	 226, 5, "acirc",
	 180, 5, "acute",
	 230, 5, "aelig",
	 224, 6, "agrave",
	  38, 3, "amp",
	  39, 4, "apos",
	 229, 5, "aring",
	 227, 6, "atilde",
	 228, 4, "auml",
	 166, 6, "brvbar",
	 231, 6, "ccedil",
	 184, 5, "cedil",
	 162, 4, "cent",
	 169, 4, "copy",
	 164, 6, "curren",
	 176, 3, "deg",
	 247, 6, "divide",
	 233, 6, "eacute",
	 234, 5, "ecirc",
	 232, 6, "egrave",
	 240, 3, "eth",
	 235, 4, "euml",
	8364, 4, "euro",
	 189, 6, "frac12",
	 188, 6, "frac14",
	 190, 6, "frac34",
	  62, 2, "gt",
	 237, 6, "iacute",
	 238, 5, "icirc",
	 161, 5, "iexcl",
	 236, 6, "igrave",
	 204, 6, "Igrave",
	 191, 6, "iquest",
	 239, 4, "iuml",
	 171, 5, "laquo",
	  60, 2, "lt",
	 175, 4, "macr",
	 181, 5, "micro",
	 183, 6, "middot",
	 160, 4, "nbsp",
	 172, 3, "not",
	 241, 6, "ntilde",
	 243, 6, "oacute",
	 244, 5, "ocirc",
	 242, 6, "ograve",
	 170, 4, "ordf",
	 186, 4, "ordm",
	 248, 6, "oslash",
	 245, 6, "otilde",
	 246, 4, "ouml",
	 182, 4, "para",
	 177, 6, "plusmn",
	 163, 5, "pound",
	  34, 4, "quot",
	 187, 5, "raquo",
	 174, 3, "reg",
	 167, 4, "sect",
	 173, 3, "shy",
	 185, 4, "sup1",
	 178, 4, "sup2",
	 179, 4, "sup3",
	 223, 5, "szlig",
	 254, 5, "thorn",
	 215, 5, "times",
	 250, 6, "uacute",
	 251, 5, "ucirc",
	 249, 6, "ugrave",
	 168, 3, "uml",
	 252, 4, "uuml",
	 253, 6, "yacute",
	 165, 3, "yen",
	 255, 4, "yuml",
	};

const HtmlEntity aXmlEntity[STR_COUNT_XML] =
	{
	  38, 3, "amp",
	  39, 4, "apos",
	  62, 2, "gt",
	  60, 2, "lt",
	  34, 4, "quot",
	};

//*****************************************************************************
//*
//*		strfindhtmla
//*
//*****************************************************************************
//	Sucht ein HTML-Entity
static int __cdecl strfindhtmla(const void *pTextPtr,const void *pEntryPtr)
{
const char *pName =((HtmlEntity*)pEntryPtr)->pName;
const char *pText =(char*)pTextPtr;
int			iPos;
int			iDif;


	for(iPos=0;;iPos++)
		{
		if(!pName[iPos])
			{
			if(pText[iPos]<'A')return 0;
			if(pText[iPos]>'z')return 0;
			if(pText[iPos]<'a')
			if(pText[iPos]>'z')return 0;
			}

		iDif  = pText[iPos];
		iDif -= pName[iPos];

		if(iDif)break;
		}



return iDif;
}

static int __cdecl strfindhtmlw(const void *pTextPtr,const void *pEntryPtr)
{
const char     *pName =((HtmlEntity*)pEntryPtr)->pName;
const wchar_t  *pText =(wchar_t*)pTextPtr;
int				iPos;
int				iDif;


	for(iPos=0;;iPos++)
		{
		if(!pName[iPos])
			{
			if(pText[iPos]<'A')return 0;
			if(pText[iPos]>'z')return 0;
			if(pText[iPos]<'a')
			if(pText[iPos]>'z')return 0;
			}

		iDif  = pText[iPos];
		iDif -= pName[iPos];

		if(iDif)break;
		}



return iDif;
}

//*****************************************************************************
//*
//*		strhtmlentity
//*
//*****************************************************************************
//	Einem HTML-Entity ("&....) in ein Zeichen umwandeln.
//	pText		: Zeiger auf den Entity-Text (beginnt nach dem '&' Zeichen)
//	uUnicode	: Hier wird das Zeichen gespeichert.
//	pList		: Zeiger auf die Liste mit den Entity's (STR_ITEMS_HTML/STR_ITEMS_XML)
//	iCount		: Anzahl der Entity's (STR_ITEMS_HTML/STR_ITEMS_XML)
//	Ergibt die Lnge des Entity's, bzw. 0 bei einem Fehler.
int	_cdecl strhtmlentity(const char *pText,unsigned &uUnicode,const HtmlEntity *pList,int iCount)
{
HtmlEntity *pPtr;
int			iLen;
char		cChar;		



	cChar = pText[0];
	
	if(strwchara(cChar,0x06))								// Buchstaben "...&gt;..."
		{
			pPtr = (HtmlEntity*)bsearch(pText,pList,iCount,sizeof(HtmlEntity),strfindhtmla);
		if(!pPtr)goto Exit;
		uUnicode = pPtr->uValue;
		iLen     = pPtr->uSize;
		}
	else if(pText[0]!='#')									// Keine Zahl vorhanden
		{
		goto Exit;
		}
	else if((pText[1]&~0x20)=='X')							// Hexadezimal "...&#x8F;..."
		{
		uUnicode = strtoi(pText+2,16,&iLen);
		if(!iLen)goto Exit;
		iLen += 2;
		}
	else{													// Dezimal "...&#123;..."
		uUnicode = strtoi(pText+1,10,&iLen);
		if(!iLen)goto Exit;
		iLen += 1;
		}


	if(pText[iLen]==';')									// Semikolon am Ende berspringen
		{
		iLen += 1;
		}

	return iLen;

Exit:

	uUnicode = '&';


return 0;
}

int	_cdecl strhtmlentity(const wchar_t *pText,unsigned &uUnicode,const HtmlEntity *pList,int iCount)
{
HtmlEntity *pPtr;
int			iLen;
wchar_t		cChar;		



	cChar = pText[0];
	
	if(strwcharw(cChar,0x06))								// Buchstaben "...&gt;..."
		{
			pPtr = (HtmlEntity*)bsearch(pText,pList,iCount,sizeof(HtmlEntity),strfindhtmlw);
		if(!pPtr)goto Exit;
		uUnicode = pPtr->uValue;
		iLen     = pPtr->uSize;
		}
	else if(pText[0]!='#')									// Keine Zahl vorhanden
		{
		goto Exit;
		}
	else if((pText[1]&~0x20)=='X')							// Hexadezimal "...&#x8F;..."
		{
		uUnicode = strtoi(pText+2,16,&iLen);
		if(!iLen)goto Exit;
		iLen += 2;
		}
	else{													// Dezimal "...&#123;..."
		uUnicode = strtoi(pText+1,10,&iLen);
		if(!iLen)goto Exit;
		iLen += 1;
		}


	if(pText[iLen]==';')									// Semikolon am Ende berspringen
		{
		iLen += 1;
		}

	return iLen;

Exit:

	uUnicode = '&';


return 0;
}

//*****************************************************************************
//*
//*		strhtmldecode
//*
//*****************************************************************************
//	Dekodiert einen HTML Text in einen normalen Text
//	pText		: Zeiger auf den HTML-String
//	iLen		: Lnge des HTML-Strings, bei -1 wird bei 0 abgebrochen
//	pBuffer		: Puffer fr normalen Text
//	iMax		: Gre des Puffers in Zeichen
//	iFlags		: Diverse Flags
//					Bit0 = UTF8 Eingabe vom HTML-String
//					Bit1 = HTML-Entities beachten (z.B.: "&gt;")
//					Bit2 = HEX-Zeichen beachten   (z.B.: "%20" )
//					Bit3 = Bei '<' aufhren
//					Bit4 = Tags berspringen
//					Bit5 = '+' in Space umwandeln
//					Bit6 = <br> in "\n" umwandeln
//					Bit7 = <br> in "\r\n" umwandeln
//					Bit8 = '\r' und '\n' entfernen
//					Bit9 = Doppelte Leerzeichen zu einem vereinen
//					Bit10= Doppelte Leerzeichen+Tabs zu einem vereinen
//					Bit11= XML statt HTML Entities verwenden
//	pLen		: Hier wird die Lnge der dekodierten Zeichen gespeichert,
//				  wenn der Zeiger ungleich Null ist.
//	Ergibt die Lnge des erzeugten Textes in Zeichen.
int _cdecl strhtmldecode(const char *pText,int iLen,char *pBuffer,int iMax,int iFlags,int *pLen)
{
char		cTemp[8];
unsigned 	aBits[8]={1,0,0,0,0,0,0,0};
unsigned 	uChar;
HtmlEntity *pList;
int			iCnt;
int			iPos;
int			iOut;
int			iAdd;
int			iNum;


	if(iMax<=1)												// Ist der Puffer zu klein
		{
		if(iMax==1)pBuffer[0]=0;
		if(pLen)*pLen=0;
		return 0;
		}

	if(iLen<0)												// Bei Null-Byte aufhren
		{
		iLen	  = 0x10000000;
		}

	if(iFlags&0x0001)										// UTF8 Konvertierung
		{
		aBits[6]  = 0xFFFFFFFF;
		aBits[7]  = 0xFFFFFFFF;
		}

	if(iFlags&0x0002)										// HTML-Entities beachten
		{
		aBits['&'/32] |= 1<<('&'&0x1F);
		}

	if(iFlags&0x0004)										// HEX-Zeichen
		{
		aBits['%'/32] |= 1<<('%'&0x1F);
		}

	if(iFlags&0x00D8)										// Bei '<' aufhren und Tags berspringen und <br>
		{
		aBits['<'/32] |= 1<<('<'&0x1F);
		}

	if(iFlags&0x0020)										// '+' in Space umwandeln
		{
		aBits['+'/32] |= 1<<('+'&0x1F);
		}

	if(iFlags&0x0100)										// '\r' und '\n' entfernen
		{
		aBits['\r'/32] |= 1<<('\r'&0x1F);
		aBits['\n'/32] |= 1<<('\n'&0x1F);
		}

	if(iFlags&0x0200)										// Doppelte Leerzeichen vereinen
		{
		aBits[' '/32]  |= 1<<(' '&0x1F);
		}

	if(iFlags&0x0400)										// Doppelte Leerzeichen+Tabs vereinen
		{
		aBits[' ' /32] |= 1<<(' ' &0x1F);
		aBits['\t'/32] |= 1<<('\t'&0x1F);
		}

	if(iFlags&0x0800)										// XML statt HTML Entities verwenden
		{
		pList = STR_ITEMS_XML;
		iCnt  = STR_COUNT_XML;
		}
	else{
		pList = STR_ITEMS_HTML;
		iCnt  = STR_COUNT_HTML;
		}


	iOut  = 0;
	iMax -= 1;


	for(iPos=0;iPos<iLen;)
		{
		iAdd = strnbits(pText+iPos,aBits,iLen-iPos);

		if(iOut+iAdd>iMax)
			{
			iAdd = iMax-iOut;
			memcpy(pBuffer+iOut,pText+iPos,iAdd);
			iOut = iMax;
			break;
			}

		memcpy(pBuffer+iOut,pText+iPos,iAdd);

		iOut += iAdd;
		iPos += iAdd;

		if(iPos>=iLen)break;


		switch(pText[iPos])									// Spezial Zeichen auswerten
			{
		case 0:												// Textende erreicht

				if(iLen>=0x10000000)
					{
					iLen = iPos;
					continue;
					}

				pBuffer[iOut] = ' ';						// NULL in SPACE umwandeln
				iOut++;
				iPos++;
				continue;

		case '<':											// Bei '<' aufhren

				if(iFlags&0x08)
					{
					iLen = iPos;
					continue;
					}

				if(iFlags&0xC0)								// <br> in "\n" umwandeln
					{
					if(iPos+4<iLen && !strwchara(pText[iPos+3],0x0F))
					if(pText[iPos+1]=='b' || pText[iPos+1]=='B')
					if(pText[iPos+2]=='r' || pText[iPos+2]=='R')
						{
						if((iFlags&0x80) && iOut<iMax)
							{
							pBuffer[iOut] = '\r';
							iOut++;
							}

						if(iOut<iMax)
							{
							pBuffer[iOut] = '\n';
							iOut++;
							}
						}
					}

				while(iPos<iLen)							// Tags berspringen
					{
					if(pText[iPos]=='>'){iPos++;break;}
					iPos += strchar(pText+iPos,'>',iLen-iPos);
					}

				continue;

		case '&':											// HTML-Entities (z.B.: "&gt;")

				iPos++;

				   iNum = iLen-iPos;
				if(iNum<0)
					{
					pBuffer[iOut] = '&';
					iOut++;
					continue;
					}

				 if(iPos+8>iLen)						// Suche Entity
					{
					strlcpy(cTemp,pText+iPos,sizeof(cTemp),iLen-iPos);
					iAdd = strhtmlentity(cTemp,uChar,pList,iCnt);
					}
				else{
					iAdd = strhtmlentity(pText+iPos,uChar,pList,iCnt);
					}

				iPos += iAdd;

				break;

		case ' ':												// Doppelte Leerzeichen vereinen
		case '\t':

				iPos += 1;

				if(iOut==0)continue;
				if(strwchara(pBuffer[iOut-1],0xE0))continue;
				uChar  = ' ';
				break;

		case '\r':												// '\r' und '\n' entfernen
		case '\n':

				iPos += 1;

				if(iOut>0    && strwchara(pBuffer[iOut-1],0x3000001F))
				if(iPos<iLen && strwchara(pText  [iPos  ],0x3000001F))
					{
					uChar = ' ';
					break;
					}

				continue;

		case '%':												// HEX-Zeichen (z.B.: "%20" )

				iPos  += 1;
				iNum   = iLen-iPos; if(iNum>2)iNum=2;
				uChar  = strtoni(pText+iPos,iNum,16,&iAdd);
				iPos  += iAdd;
				break;

		case '+':												// '+' in Space umwandeln

				iPos  += 1;
				uChar  = ' ';
				break;

		default:												// UTF8-Zeichen

				 if(iPos+4>iLen)
					{
					strlcpy(cTemp,pText+iPos,sizeof(cTemp),iLen-iPos);
					iPos += utf8tochar(cTemp,uChar);
					}
				else{
					iPos += utf8tochar(pText+iPos,uChar);
					}

				break;

			}


		if(uChar<255)
			{
			pBuffer[iOut] = (char)uChar;
			iOut++;
			continue;
			}

		if(uChar==8364)											// Euro-Zeichen in Text umwandeln
			{
			iOut += strlcpy(pBuffer+iOut,"Euro",iMax-iOut);
			continue;
			}


		pBuffer[iOut] = '\xFF';
		iOut++;
		}


	if(pLen)*pLen=iPos;

	pBuffer[iOut] = 0;



return iOut;
}

int _cdecl strhtmldecode(const char *pText,int iLen,wchar_t *pBuffer,int iMax,int iFlags,int *pLen)
{
char		cTemp[8];
unsigned 	aBits[8]={1,0,0,0,0,0,0,0};
unsigned 	uChar;
HtmlEntity *pList;
int			iCnt;
int			iPos;
int			iOut;
int			iAdd;
int			iNum;


	if(iMax<=2)												// Ist der Puffer zu klein
		{
		if(iMax>=1)pBuffer[0]=0;
		if(pLen)*pLen=0;
		return 0;
		}

	if(iLen<0)												// Bei Null-Byte aufhren
		{
		iLen	  = 0x10000000;
		}

	if(iFlags&0x01)											// UTF8 Konvertierung
		{
		aBits[6]  = 0xFFFFFFFF;
		aBits[7]  = 0xFFFFFFFF;
		}

	if(iFlags&0x02)											// HTML-Entities beachten
		{
		aBits['&'/32] |= 1<<('&'&0x1F);
		}

	if(iFlags&0x04)											// HEX-Zeichen
		{
		aBits['%'/32] |= 1<<('%'&0x1F);
		}

	if(iFlags&0xD8)											// Bei '<' aufhren und Tags berspringen und <br>
		{
		aBits['<'/32] |= 1<<('<'&0x1F);
		}

	if(iFlags&0x20)											// '+' in Space umwandeln
		{
		aBits['+'/32] |= 1<<('+'&0x1F);
		}

	if(iFlags&0x0100)										// '\r' und '\n' entfernen
		{
		aBits['\r'/32] |= 1<<('\r'&0x1F);
		aBits['\n'/32] |= 1<<('\n'&0x1F);
		}

	if(iFlags&0x0200)										// Doppelte Leerzeichen vereinen
		{
		aBits[' '/32]  |= 1<<(' '&0x1F);
		}

	if(iFlags&0x0400)										// Doppelte Leerzeichen+Tabs vereinen
		{
		aBits[' ' /32] |= 1<<(' ' &0x1F);
		aBits['\t'/32] |= 1<<('\t'&0x1F);
		}

	if(iFlags&0x0800)										// XML statt HTML Entities verwenden
		{
		pList = STR_ITEMS_XML;
		iCnt  = STR_COUNT_XML;
		}
	else{
		pList = STR_ITEMS_HTML;
		iCnt  = STR_COUNT_HTML;
		}

	iOut  = 0;
	iMax -= 2;


	for(iPos=0;iPos<iLen;)
		{
		iAdd = strnbits(pText+iPos,aBits,iLen-iPos);

		if(iOut+iAdd>iMax)									// Puffer zu klein
			{
			iAdd = iMax-iOut;

			while(iAdd>0)
				{
				pBuffer[iOut] = (unsigned char)pText[iPos];
				iOut ++;
				iPos ++;
				iAdd --;
				}

			break;
			}

		while(iAdd>0)
			{
			pBuffer[iOut] = (unsigned char)pText[iPos];
			iOut ++;
			iPos ++;
			iAdd --;
			}

		if(iPos>=iLen)break;


		switch(pText[iPos])									// Spezial Zeichen auswerten
			{
		case 0:												// Textende erreicht

				if(iLen>=0x10000000)
					{
					iLen = iPos;
					continue;
					}

				pBuffer[iOut] = ' ';						// NULL in SPACE umwandeln
				iOut++;
				iPos++;
				continue;

		case '<':											// Bei '<' aufhren

				if(iFlags&0x08)
					{
					iLen = iPos;
					continue;
					}

				if(iFlags&0xC0)								// <br> in "\n" umwandeln
					{
					if(iPos+4<iLen && !strwchara(pText[iPos+3],0x0F))
					if(pText[iPos+1]=='b' || pText[iPos+1]=='B')
					if(pText[iPos+2]=='r' || pText[iPos+2]=='R')
						{
						if((iFlags&0x80) && iOut<iMax)
							{
							pBuffer[iOut] = '\r';
							iOut++;
							}

						if(iOut<iMax)
							{
							pBuffer[iOut] = '\n';
							iOut++;
							}
						}
					}

				while(iPos<iLen)							// Tags berspringen
					{
					if(pText[iPos]=='>'){iPos++;break;}
					iPos += strchar(pText+iPos,'>',iLen-iPos);
					}

				continue;

		case '&':											// HTML-Entities (z.B.: "&gt;")

				iPos++;

				   iNum = iLen-iPos;
				if(iNum<0)
					{
					pBuffer[iOut] = '&';
					iOut++;
					continue;
					}

				 if(iPos+8>iLen)						// Suche Entity
					{
					strlcpy(cTemp,pText+iPos,sizeof(cTemp),iLen-iPos);
					iAdd = strhtmlentity(cTemp,uChar,pList,iCnt);
					}
				else{
					iAdd = strhtmlentity(pText+iPos,uChar,pList,iCnt);
					}

				iPos += iAdd;

				break;

		case ' ':												// Doppelte Leerzeichen vereinen
		case '\t':
				iPos += 1;

				if(iOut==0)continue;
				if(strwcharw(pBuffer[iOut-1],0xE0))continue;
				uChar  = ' ';
				break;

		case '\r':												// '\r' und '\n' entfernen
		case '\n':

				iPos += 1;

				if(iOut>0    && strwcharw(pBuffer[iOut-1],0x3000001F))
				if(iPos<iLen && strwchara(pText  [iPos  ],0x3000001F))
					{
					uChar = ' ';
					break;
					}

				continue;

		case '%':												// HEX-Zeichen (z.B.: "%20" )

				iPos  += 1;
				iNum   = iLen-iPos; if(iNum>2)iNum=2;
				uChar  = strtoni(pText+iPos,iNum,16,&iAdd);
				iPos  += iAdd;
				break;

		case '+':												// '+' in Space umwandeln

				iPos  += 1;
				uChar  = ' ';
				break;

		default:												// UTF8-Zeichen

				 if(iPos+4>iLen)
					{
					strlcpy(cTemp,pText+iPos,sizeof(cTemp),iLen-iPos);
					iPos += utf8tochar(cTemp,uChar);
					}
				else{
					iPos += utf8tochar(pText+iPos,uChar);
					}

				break;

			}

		iOut += chartoutf16(pBuffer+iOut,uChar);
		continue;
		}


	if(pLen)*pLen=iPos;

	pBuffer[iOut] = 0;



return iOut;
}

int _cdecl strhtmldecode(const wchar_t *pText,int iLen,wchar_t *pBuffer,int iMax,int iFlags,int *pLen)
{
char		cTemp[8];
unsigned 	aBits[8]={1,0,0,0,0,0,0,0};
unsigned 	uChar;
void	   *pPtr;
int			iPos;
int			iOut;
int			iAdd;
int			iNum;


	if(iMax<=2)												// Ist der Puffer zu klein
		{
		if(iMax>=1)pBuffer[0]=0;
		if(pLen)*pLen=0;
		return 0;
		}

	if(iLen<0)												// Bei Null-Byte aufhren
		{
		iLen	  = 0x10000000;
		}

	if(iFlags&0x01)											// UTF8 Konvertierung
		{
		aBits[6]  = 0xFFFFFFFF;
		aBits[7]  = 0xFFFFFFFF;
		}

	if(iFlags&0x02)											// HTML-Entities beachten
		{
		aBits['&'/32] |= 1<<('&'&0x1F);
		}

	if(iFlags&0x04)											// HEX-Zeichen
		{
		aBits['%'/32] |= 1<<('%'&0x1F);
		}

	if(iFlags&0xD8)											// Bei '<' aufhren und Tags berspringen und <br>
		{
		aBits['<'/32] |= 1<<('<'&0x1F);
		}

	if(iFlags&0x20)											// '+' in Space umwandeln
		{
		aBits['+'/32] |= 1<<('+'&0x1F);
		}

	if(iFlags&0x0100)										// '\r' und '\n' entfernen
		{
		aBits['\r'/32] |= 1<<('\r'&0x1F);
		aBits['\n'/32] |= 1<<('\n'&0x1F);
		}

	iOut  = 0;
	iMax -= 2;


	for(iPos=0;iPos<iLen;)
		{
		iAdd = strnbits(pText+iPos,aBits,iLen-iPos);

		if(iOut+iAdd>iMax)									// Puffer zu klein
			{
			iAdd = iMax-iOut;

			while(iAdd>0)
				{
				pBuffer[iOut] = (unsigned char)pText[iPos];
				iOut ++;
				iPos ++;
				iAdd --;
				}

			break;
			}

		while(iAdd>0)
			{
			pBuffer[iOut] = (unsigned char)pText[iPos];
			iOut ++;
			iPos ++;
			iAdd --;
			}

		if(iPos>=iLen)break;


		switch(pText[iPos])									// Spezial Zeichen auswerten
			{
		case 0:												// Textende erreicht

				if(iLen>=0x10000000)
					{
					iLen = iPos;
					continue;
					}

				pBuffer[iOut] = ' ';						// NULL in SPACE umwandeln
				iOut++;
				iPos++;
				continue;

		case '<':											// Bei '<' aufhren

				if(iFlags&0x08)
					{
					iLen = iPos;
					continue;
					}

				if(iFlags&0xC0)								// <br> in "\n" umwandeln
					{
					if(iPos+4<iLen && !strwcharw(pText[iPos+3],0x0F))
					if(pText[iPos+1]=='b' || pText[iPos+1]=='B')
					if(pText[iPos+2]=='r' || pText[iPos+2]=='R')
						{
						if((iFlags&0x80) && iOut<iMax)
							{
							pBuffer[iOut] = '\r';
							iOut++;
							}

						if(iOut<iMax)
							{
							pBuffer[iOut] = '\n';
							iOut++;
							}
						}
					}

				while(iPos<iLen)							// Tags berspringen
					{
					if(pText[iPos]=='>'){iPos++;break;}
					iPos += strchar(pText+iPos,'>',iLen-iPos);
					}

				continue;

		case '&':											// HTML-Entities (z.B.: "&gt;")

				iPos++;

				   iNum = iLen-iPos;
				if(iNum<0)
					{
					pBuffer[iOut] = '&';
					iOut++;
					iPos++;
					continue;
					}

				if(iNum>7)iNum=7;


				if(pText[iPos]=='#')						// "&#123;"
					{
					iPos += 1;
					uChar = strtoni(pText+iPos,iNum-1,10,&iAdd);
					}
				else{
						iAdd = strwlen(pText+iPos,0x07,iNum);
					if(!iAdd)
						{
						pBuffer[iOut] = '&';
						iOut++;
						iPos++;
						continue;
						}

					 if(iPos+8>iLen)						// Suche Entity
						{
						strawcpy(cTemp,pText+iPos,sizeof(cTemp),iLen-iPos);
						pPtr = bsearch(cTemp,aHtmlEntity,sizeof(aHtmlEntity)/sizeof(HtmlEntity),sizeof(HtmlEntity),strfindhtmla);
						}
					else{
						pPtr = bsearch(pText+iPos,aHtmlEntity,sizeof(aHtmlEntity)/sizeof(HtmlEntity),sizeof(HtmlEntity),strfindhtmla);
						}

					if(!pPtr)
						{
						pBuffer[iOut] = '&';
						iOut++;
						iPos++;
						continue;
						}

					uChar = ((HtmlEntity*)pPtr)->uValue;
					}

				   iPos += iAdd;
				if(iPos<iLen && pText[iPos]==';')
					{
					iPos++;
					}

				break;

		case '\r':												// '\r' und '\n' entfernen
		case '\n':

				iPos += 1;

				if(iOut>0    && strwcharw(pBuffer[iOut-1],0x3000001F))
				if(iPos<iLen && strwcharw(pText  [iPos  ],0x3000001F))
					{
					uChar = ' ';
					break;
					}

				continue;

		case '%':												// HEX-Zeichen (z.B.: "%20" )

				iPos  += 1;
				iNum   = iLen-iPos; if(iNum>2)iNum=2;
				uChar  = strtoni(pText+iPos,iNum,16,&iAdd);
				iPos  += iAdd;
				break;

		case '+':												// '+' in Space umwandeln

				iPos  += 1;
				uChar  = ' ';
				break;

		default:												// UTF8-Zeichen

				strawcpy(cTemp,pText+iPos,sizeof(cTemp),(iPos+4>iLen)? iLen-iPos:4);
				iPos += utf8tochar(cTemp,uChar);
				break;

			}

		iOut += chartoutf16(pBuffer+iOut,uChar);
		continue;
		}


	if(pLen)*pLen=iPos;

	pBuffer[iOut] = 0;



return iOut;
}


#ifdef _DEBUG

int _cdecl strhtmldecode(const char *pText,char *pBuffer,int iMax,int iFlags,int *pLen)
{
	return strhtmldecode(pText,-1,pBuffer,iMax,iFlags,pLen);
}

int _cdecl strhtmldecode(const char *pText,wchar_t *pBuffer,int iMax,int iFlags,int *pLen)
{
	return strhtmldecode(pText,-1,pBuffer,iMax,iFlags,pLen);
}

int _cdecl strhtmldecode(const wchar_t *pText,wchar_t *pBuffer,int iMax,int iFlags,int *pLen)
{
	return strhtmldecode(pText,-1,pBuffer,iMax,iFlags,pLen);
}

#endif


//*****************************************************************************
//*
//*		strhtmlencode
//*
//*****************************************************************************
//	Kodiert einen Text in einen HTML-String.
//	pText		: Zeiger auf den Text.
//	iLen		: Lnge des Textes, bei -1 wird bei 0 abgebrochen
//	pBuffer		: Puffer fr HTML-String, der Puffer sollte 8 grer sein.
//	iMax		: Gre des Puffers in Zeichen
//	iFlags		: Diverse Flags
//					Bit0 = UTF8-Ausgabe zu HTML-String
//					Bit1 = HTML-Entities ausgeben (z.B.: "&gt;")
//					Bit2 = HEX-Zeichen beachten   (z.B.: "%20" )
//					Bit3 = Pfadzeichen auch codieren (":/\")
//					Bit5 = Space in '+' umwandeln
//					Bit6 = "\n" in <br> umwandeln
//					Bit7 = "\r\n" in <br>  umwandeln
//	pLen		: Hier wird die Lnge der dekodierten Zeichen gespeichert,
//				  wenn der Zeiger ungleich Null ist.
//	Ergibt die Lnge des erzeugten HTML-Strings in Zeichen.
int _cdecl strhtmlencode(const char *pText,int iLen,char *pBuffer,int iMax,int iFlags,int *pLen)
{
unsigned 	aBits[8]={1,0x500000C4,0,0,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF};
unsigned 	uChar;
int			iPos;
int			iOut;
int			iAdd;



	if(iMax<=8)												// Ist der Puffer zu klein
		{
		if(iMax>=1)pBuffer[0]=0;
		if(pLen)*pLen=0;
		return 0;
		}

	if(iLen<0)												// Bei Null-Byte aufhren
		{
		iLen	  = 0x10000000;
		}

	if(iFlags&0x04)											// HEX-Zeichen
		{
		aBits[0]  = 0xFFFFFFFF;
		aBits[1]  = 0xFC009FFF;
		aBits[2]  = 0x10000001;
		}
	else if(iFlags&0x20)									// Space in '+' umwandeln
		{
		aBits[1] |= 0x00000001;
		}

	if(iFlags&0x08)											// Pfadzeichen auch codieren
		{
		aBits[1] |= 0x04008000;
		aBits[2] |= 0x10000000;
		}

	if(iFlags&0x40)											// "\n" in <br> umwandeln
		{
		aBits[0] |= 0x00000400;
		}

	if(iFlags&0x80)											// "\r\n" in <br> umwandeln
		{
		aBits[0] |= 0x00002000;
		}

	iOut  = 0;
	iMax -= 1;


	for(iPos=0;iPos<iLen;)
		{
		iAdd = strnbits(pText+iPos,aBits,iLen-iPos);

		if(iOut+iAdd>iMax)
			{
			iAdd = iMax-iOut;
			memcpy(pBuffer+iOut,pText+iPos,iAdd);
			iOut = iMax;
			break;
			}

		memcpy(pBuffer+iOut,pText+iPos,iAdd);

		iOut += iAdd;
		iPos += iAdd;

		if(iPos>=iLen)break;


		if(pText[iPos]==0)									// Textende erreicht
			{
			if(iLen>=0x10000000)
				{
				iLen = iPos;
				continue;
				}

			if(iFlags&0x04)									// HEX-Zeichen
				{
				memcpy(pBuffer+iOut,"%00",4);
				iOut += 3;
				iPos += 1;
				continue;
				}

			pBuffer[iOut] = ' ';							// NULL zu SPACE
			iOut += 1;
			iPos += 1;
			continue;
			}

		uChar  = (unsigned char)pText[iPos];
		iPos  += 1;

		if(uChar>=128 && (iFlags&0x01))						// UTF8 Kodierung
			{
			iOut += chartoutf8(pBuffer+iOut,uChar);
			continue;
			}

		if(uChar==32 && (iFlags&0x20))						// Space in '+' umwandeln
			{
			pBuffer[iOut  ] = '+';
			iOut += 1;
			continue;
			}

		if(uChar=='\r' && (iFlags&0x80))					// "\r\n" zu <br>
			{
			if(iPos>=iLen)continue;
			if(pText[iPos]!='\n')
				{
				pBuffer[iOut  ] = '\r';
				iOut += 1;
				}

			if(iOut+4<iMax)
				{
				pBuffer[iOut] = '<';	iOut++;
				pBuffer[iOut] = 'b';	iOut++;
				pBuffer[iOut] = 'r';	iOut++;
				pBuffer[iOut] = '>';	iOut++;
				}

			iPos += 1;

			continue;
			}

		if(uChar=='\n' && (iFlags&0x40))					// "\n" zu <br>
			{
			if(iOut+4<iMax)
				{
				pBuffer[iOut] = '<';	iOut++;
				pBuffer[iOut] = 'b';	iOut++;
				pBuffer[iOut] = 'r';	iOut++;
				pBuffer[iOut] = '>';	iOut++;
				}

			continue;
			}

		if(uChar<256 && (iFlags&0x04))						// HEX-Zeichen
			{
			pBuffer[iOut  ] = '%';
			pBuffer[iOut+1] = "0123456789ABCDEF"[uChar>>4  ];
			pBuffer[iOut+2] = "0123456789ABCDEF"[uChar&0x0F];
			iOut += 3;
			continue;
			}

		pBuffer[iOut] = '&';
		iOut += 1;
		pBuffer[iOut] = '#';
		iOut += 1;
		iOut += itostr(uChar,pBuffer+iOut,15);
		pBuffer[iOut] = ';';
		iOut += 1;
		}


	if(pLen)*pLen=iPos;

	pBuffer[iOut] = 0;



return iOut;
}

int _cdecl strhtmlencode(const wchar_t *pText,int iLen,char *pBuffer,int iMax,int iFlags,int *pLen)
{
unsigned 	aBits[8]={1,0x500000C4,0,0,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF};
unsigned 	uChar;
int			iPos;
int			iOut;
int			iAdd;



	if(iMax<=8)												// Ist der Puffer zu klein
		{
		if(iMax>=1)pBuffer[0]=0;
		if(pLen)*pLen=0;
		return 0;
		}

	if(iLen<0)												// Bei Null-Byte aufhren
		{
		iLen	  = 0x10000000;
		}

	if(iFlags&0x04)											// HEX-Zeichen
		{
		aBits[0]  = 0xFFFFFFFF;
		aBits[1]  = 0xFC009FFF;
		aBits[2]  = 0x10000001;
		}
	else if(iFlags&0x20)									// Space in '+' umwandeln
		{
		aBits[1] |= 0x00000001;
		}

	if(iFlags&0x08)											// Pfadzeichen auch codieren
		{
		aBits[1] |= 0x04008000;
		aBits[2] |= 0x10000000;
		}

	if(iFlags&0x40)											// "\n" in <br> umwandeln
		{
		aBits[0] |= 0x00000400;
		}

	if(iFlags&0x80)											// "\r\n" in <br> umwandeln
		{
		aBits[0] |= 0x00002000;
		}



	iOut  = 0;
	iMax -= 1;


	for(iPos=0;iPos<iLen;)
		{
		iAdd = strnbits(pText+iPos,aBits,iLen-iPos);

		if(iOut+iAdd>iMax)									// Puffer zu klein
			{
			iAdd = iMax-iOut;

			while(iAdd>0)
				{
				pBuffer[iOut] = (unsigned char)pText[iPos];
				iOut ++;
				iPos ++;
				iAdd --;
				}

			break;
			}

		while(iAdd>0)
			{
			pBuffer[iOut] = (char)pText[iPos];
			iOut ++;
			iPos ++;
			iAdd --;
			}

		if(iPos>=iLen)break;


		if(pText[iPos]==0)									// Textende erreicht
			{
			if(iLen>=0x10000000)
				{
				iLen = iPos;
				continue;
				}

			if(iFlags&0x04)									// HEX-Zeichen
				{
				memcpy(pBuffer+iOut,"%00",4);
				iOut += 3;
				iPos += 1;
				continue;
				}

			pBuffer[iOut] = ' ';							// NULL zu SPACE
			iOut += 1;
			iPos += 1;
			continue;
			}

		if(iPos+1<iLen)
			{
			uChar = pText[iPos];
			iPos += 1;
			}
		else{
			iPos += utf16tochar(pText+iPos,uChar);
			}

		if(uChar>=128 && (iFlags&0x01))						// UTF8 Kodierung
			{
			iOut += chartoutf8(pBuffer+iOut,uChar);
			continue;
			}

		if(uChar==32 && (iFlags&0x20))						// Space in '+' umwandeln
			{
			pBuffer[iOut  ] = '+';
			iOut += 1;
			continue;
			}

		if(uChar=='\r')										// "\r\n" zu <br>
			{
			if(iPos>=iLen)continue;
			if(pText[iPos]!='\n')
				{
				pBuffer[iOut] = '\r';
				iOut += 1;
				}

			if(iOut+4<iMax)
				{
				pBuffer[iOut] = '<';	iOut++;
				pBuffer[iOut] = 'b';	iOut++;
				pBuffer[iOut] = 'r';	iOut++;
				pBuffer[iOut] = '>';	iOut++;
				}

			iPos += 1;

			continue;
			}

		if(uChar=='\n' && (iFlags&0x40))					// "\n" zu <br>
			{
			if(iOut+4<iMax)
				{
				pBuffer[iOut] = '<';	iOut++;
				pBuffer[iOut] = 'b';	iOut++;
				pBuffer[iOut] = 'r';	iOut++;
				pBuffer[iOut] = '>';	iOut++;
				}

			continue;
			}

		if(uChar<256 && (iFlags&0x04))						// HEX-Zeichen
			{
			pBuffer[iOut  ] = '%';
			pBuffer[iOut+1] = "0123456789ABCDEF"[uChar>>4  ];
			pBuffer[iOut+2] = "0123456789ABCDEF"[uChar&0x0F];
			iOut += 3;
			continue;
			}

		pBuffer[iOut] = '&';
		iOut += 1;
		pBuffer[iOut] = '#';
		iOut += 1;
		iOut += itostr(uChar,pBuffer+iOut,15);
		pBuffer[iOut] = ';';
		iOut += 1;
		}


	if(pLen)*pLen=iPos;

	pBuffer[iOut] = 0;



return iOut;
}

#ifdef _DEBUG

int _cdecl strhtmlencode(const char *pText,char *pBuffer,int iMax,int iFlags,int *pLen)
{
	return strhtmlencode(pText,-1,pBuffer,iMax,iFlags,pLen);
}

int _cdecl strhtmlencode(const wchar_t *pText,char *pBuffer,int iMax,int iFlags,int *pLen)
{
	return strhtmlencode(pText,-1,pBuffer,iMax,iFlags,pLen);
}

#endif


//*****************************************************************************
//*
//*		strlinsert
//*
//*****************************************************************************
//	Fgt einen Text in einen bestehenden String ein.
//	pString		: Zeiger auf den Text in den der String eingefgt werden soll.
//	iOff		: Offset im String wo der Text eingefgt werden soll
//	iNum		: Anzahl der Zeichen die ab dem Offset berschrieben werden sollen.
//	iSize		: Ist die aktuelle Lnge des Strings in Zeichen.
//	pSource		: Zeiger auf den einzufgenden Text
//	iMax		: Maximale Puffergre vom String
//	iLen		: Ist die Lnge des einzufgenden Textes.
//	Ergibt die neue Lnge des String in Zeichen.
int _cdecl strlinsert(char *pString,int iOff,int iNum,int iSize,const char *pSource,int iMax,int iLen)
{
int		iPos;
int		iEnd;


	iMax--;

	if(iMax<=0)										// Puffergre prfen
		{
		if(!iMax)pString[0]=0;
		return 0;
		}

	if(iSize>iMax)iSize=iMax;

	iLen = strnlen(pSource,iLen );
	iEnd = strnlen(pString,iSize);


	if(iOff>iEnd)									// Offset im Text prfen
		{
		iOff = iEnd;
		iNum = 0;
		}
	else if(iOff<0)
		{
		iNum += iOff;
		iOff  = 0;
		}

	if(iNum<0)										// berschreibbereich prfen
		{
		iNum = 0;
		}
	else if(iOff+iNum>iEnd)
		{
		iNum = iEnd-iOff;
		}


	if(iLen==iNum)									// Nur berschreiben
		{
		memmove(pString+iOff,pSource,iLen);
		pString[iEnd] = 0;
		return iEnd;
		}


	iPos = iEnd-iNum+iLen;

	if(iPos>iMax)									// Wird das Ergebnis zu lang
		{
		if(iOff+iLen>iMax)
			{
			iLen = iMax-iOff;
			iEnd = iOff+iNum;
			}
		else{
			iEnd = iNum+iMax-iLen;
			}

		iPos = iMax;
		}

													// Daten kopieren
	memmove(pString+iOff+iLen,pString+iOff+iNum,iEnd-iOff-iNum);
	memmove(pString+iOff,pSource,iLen);


	pString[iPos] = 0;



return iPos;
}

int _cdecl strlinsert(wchar_t *pString,int iOff,int iNum,int iSize,const wchar_t *pSource,int iMax,int iLen)
{
int		iPos;
int		iEnd;


	iMax--;

	if(iMax<=0)										// Puffergre prfen
		{
		if(!iMax)pString[0]=0;
		return 0;
		}

	if(iSize>iMax)iSize=iMax;

	iLen = strnlen(pSource,iLen );
	iEnd = strnlen(pString,iSize);


	if(iOff>iEnd)									// Offset im Text prfen
		{
		iOff = iEnd;
		iNum = 0;
		}
	else if(iOff<0)
		{
		iNum += iOff;
		iOff  = 0;
		}

	if(iNum<0)										// berschreibbereich prfen
		{
		iNum = 0;
		}
	else if(iOff+iNum>iEnd)
		{
		iNum = iEnd-iOff;
		}


	if(iLen==iNum)									// Nur berschreiben
		{
		memmove(pString+iOff,pSource,iLen*sizeof(wchar_t));
		pString[iEnd] = 0;
		return iEnd;
		}


	iPos = iEnd-iNum+iLen;

	if(iPos>iMax)									// Wird das Ergebnis zu lang
		{
		if(iOff+iLen>iMax)
			{
			iLen = iMax-iOff;
			iEnd = iOff+iNum;
			}
		else{
			iEnd = iNum+iMax-iLen;
			}

		iPos = iMax;
		}

													// Daten kopieren
	memmove(pString+iOff+iLen,pString+iOff+iNum,(iEnd-iOff-iNum)*sizeof(wchar_t));
	memmove(pString+iOff,pSource,iLen*sizeof(wchar_t));


	pString[iPos] = 0;



return iPos;
}

#ifdef _DEBUG

int _cdecl strlinsert(char *pString,int iOff,int iSize,int iNum,const char *pSource,int iMax)
	{
	return strlinsert(pString,iOff,iNum,iSize,pSource,iMax,(int)strlen(pSource));
	}

int _cdecl strlinsert(char *pString,int iOff,int iSize,const char *pSource,int iMax)
	{
	return strlinsert(pString,iOff,0,iSize,pSource,iMax,(int)strlen(pSource));
	}

int _cdecl strlinsert(char *pString,int iOff,int iSize,const char *pSource,int iMax,int iLen)
	{
	return strlinsert(pString,iOff,0,iSize,pSource,iMax,iLen);
	}

int _cdecl strlinsert(wchar_t *pString,int iOff,int iSize,int iNum,const wchar_t *pSource,int iMax)
	{
	return strlinsert(pString,iOff,iNum,iSize,pSource,iMax,(int)wcslen(pSource));
	}

int _cdecl strlinsert(wchar_t *pString,int iOff,int iSize,const wchar_t *pSource,int iMax)
	{
	return strlinsert(pString,iOff,0,iSize,pSource,iMax,(int)wcslen(pSource));
	}

int _cdecl strlinsert(wchar_t *pString,int iOff,int iSize,const wchar_t *pSource,int iMax,int iLen)
	{
	return strlinsert(pString,iOff,0,iSize,pSource,iMax,iLen);
	}

#endif

//*****************************************************************************
//*
//*		strinsert
//*
//*****************************************************************************
//	Fgt einen Text in einen bestehenden String ein.
//	pString		: Zeiger auf den Text in den der String eingefgt werden soll.
//	iOff		: Offset im String wo der Text eingefgt werden soll
//	iNum		: Anzahl der Zeichen die ab dem Offset berschrieben werden sollen.
//	pSource		: Zeiger auf den einzufgenden Text
//	iMax		: Maximale Puffergre vom String
//	iLen		: Ist die Lnge des einzufgenden Textes.
//	Ergibt die neue Lnge des String in Zeichen.
int _cdecl strinsert(char *pString,int iOff,int iNum,const char *pSource,int iMax,int iLen)
{
int		iPos;
int		iEnd;


	iMax--;

	if(iMax<=0)										// Puffergre prfen
		{
		if(!iMax)pString[0]=0;
		return 0;
		}


	iLen = strnlen(pSource,iLen);
	iEnd = strnlen(pString,iMax);


	if(iOff>iEnd)									// Offset im Text prfen
		{
		iOff = iEnd;
		iNum = 0;
		}
	else if(iOff<0)
		{
		iNum += iOff;
		iOff  = 0;
		}

	if(iNum<0)										// berschreibbereich prfen
		{
		iNum = 0;
		}
	else if(iOff+iNum>iEnd)
		{
		iNum = iEnd-iOff;
		}


	if(iLen==iNum)									// Nur berschreiben
		{
		memmove(pString+iOff,pSource,iLen);
		pString[iEnd] = 0;
		return iEnd;
		}


	iPos = iEnd-iNum+iLen;

	if(iPos>iMax)									// Wird das Ergebnis zu lang
		{
		if(iOff+iLen>iMax)
			{
			iLen = iMax-iOff;
			iEnd = iOff+iNum;
			}
		else{
			iEnd = iNum+iMax-iLen;
			}

		iPos = iMax;
		}

													// Daten kopieren
	memmove(pString+iOff+iLen,pString+iOff+iNum,iEnd-iOff-iNum);
	memmove(pString+iOff,pSource,iLen);


	pString[iPos] = 0;



return iPos;
}

int _cdecl strinsert(wchar_t *pString,int iOff,int iNum,const wchar_t *pSource,int iMax,int iLen)
{
int		iPos;
int		iEnd;


	iMax--;

	if(iMax<=0)										// Puffergre prfen
		{
		if(!iMax)pString[0]=0;
		return 0;
		}


	iLen = strnlen(pSource,iLen);
	iEnd = strnlen(pString,iMax);


	if(iOff>iEnd)									// Offset im Text prfen
		{
		iOff = iEnd;
		iNum = 0;
		}
	else if(iOff<0)
		{
		iNum += iOff;
		iOff  = 0;
		}

	if(iNum<0)										// berschreibbereich prfen
		{
		iNum = 0;
		}
	else if(iOff+iNum>iEnd)
		{
		iNum = iEnd-iOff;
		}


	if(iLen==iNum)									// Nur berschreiben
		{
		memmove(pString+iOff,pSource,iLen*sizeof(wchar_t));
		pString[iEnd] = 0;
		return iEnd;
		}


	iPos = iEnd-iNum+iLen;

	if(iPos>iMax)									// Wird das Ergebnis zu lang
		{
		if(iOff+iLen>iMax)
			{
			iLen = iMax-iOff;
			iEnd = iOff+iNum;
			}
		else{
			iEnd = iNum+iMax-iLen;
			}

		iPos = iMax;
		}

													// Daten kopieren
	memmove(pString+iOff+iLen,pString+iOff+iNum,(iEnd-iOff-iNum)*sizeof(wchar_t));
	memmove(pString+iOff,pSource,iLen*sizeof(wchar_t));


	pString[iPos] = 0;



return iPos;
}

#ifdef _DEBUG

int _cdecl strinsert(char *pString,int iOff,int iNum,const char *pSource,int iMax)
	{
	return strinsert(pString,iOff,iNum,pSource,iMax,(int)strlen(pSource));
	}

int _cdecl  strinsert(char *pString,int iOff,const char *pSource,int iMax)
	{
	return strinsert(pString,iOff,0,pSource,iMax,(int)strlen(pSource));
	}

int _cdecl  strinsert(char *pString,int iOff,const char *pSource,int iMax,int iLen)
	{
	return strinsert(pString,iOff,0,pSource,iMax,iLen);
	}

int _cdecl strinsert(wchar_t *pString,int iOff,int iNum,const wchar_t *pSource,int iMax)
	{
	return strinsert(pString,iOff,iNum,pSource,iMax,(int)wcslen(pSource));
	}

int _cdecl  strinsert(wchar_t *pString,int iOff,const wchar_t *pSource,int iMax)
	{
	return strinsert(pString,iOff,0,pSource,iMax,(int)wcslen(pSource));
	}

int _cdecl  strinsert(wchar_t *pString,int iOff,const wchar_t *pSource,int iMax,int iLen)
	{
	return strinsert(pString,iOff,0,pSource,iMax,iLen);
	}

#endif

//*****************************************************************************
//*
//*		strawcmp
//*
//*****************************************************************************
//	Vergleicht einen Unicode-Text mit einem normalen Text.
//	pStr1		: Ist der Zeiger auf den Unicode-Text
//	pStr2		: Ist der Zeiger auf den normalen Text
//	iMax		: Ist die maximale Vergleichslnge in Zeichen
//	Ergibt	<0 wenn pStr1 kleiner pStr2 ist
//			>0 wenn pStr1 grer  pStr2 ist
//			=0 wenn pStr1 gleich  pStr2 ist
int strawcmp(const wchar_t *pStr1,const char *pStr2,int iMax)
{
unsigned char  *pPtr=(unsigned char*)pStr2;
int				iChar2;
int				iChar1;
int				iPos;
int				iDif;


	for(iPos=0;iPos<iMax;iPos++)
		{
		iChar2	= pPtr [iPos];
		iChar1	= pStr1[iPos];
		iDif	= iChar1-iChar2;

		if(iDif || !iChar1)return iDif;
		}


return 0;
}

#ifdef _DEBUG

int _cdecl strawcmp(const char *pStr1,const char *pStr2,int iMax)
{
return strncmp(pStr1,pStr2,iMax);
}

int _cdecl strawcmp(const char *pStr1,const wchar_t *pStr2,int iMax)
{
return -strawcmp(pStr2,pStr1,iMax);
}

int _cdecl strawcmp(const wchar_t *pStr1,const wchar_t *pStr2,int iMax)
{
return wcsncmp(pStr1,pStr2,iMax);
}

#endif

//*****************************************************************************
//*
//*		strawicmp
//*
//*****************************************************************************
//	Vergleicht einen Unicode-Text mit einem normalen Text ohne Gros-/Kleinunterscheidung.
//	pStr1		: Ist der Zeiger auf den Unicode-Text
//	pStr2		: Ist der Zeiger auf den normalen Text
//	iMax		: Ist die maximale Vergleichslnge in Zeichen
//	Ergibt	<0 wenn pStr1 kleiner pStr2 ist
//			>0 wenn pStr1 grer  pStr2 ist
//			=0 wenn pStr1 gleich  pStr2 ist
int strawicmp(const wchar_t *pStr1,const char *pStr2,int iMax)
{
unsigned char  *pPtr=(unsigned char*)pStr2;
int				iChar2;
int				iChar1;
int				iPos;
int				iDif;


	for(iPos=0;iPos<iMax;iPos++)
		{
		iChar2	= pPtr [iPos];
		iChar1	= pStr1[iPos];

		if(iChar2<='Z' && iChar2>='A')iChar2+='a'-'A';
		if(iChar1<='Z' && iChar1>='A')iChar1+='a'-'A';

		iDif	= iChar1-iChar2;

		if(iDif || !iChar1)return iDif;
		}


return 0;
}

#ifdef _DEBUG

int _cdecl strawicmp(const char *pStr1,const char *pStr2,int iMax)
{
return strnicmp(pStr1,pStr2,iMax);
}

int _cdecl strawicmp(const char *pStr1,const wchar_t *pStr2,int iMax)
{
return -strawicmp(pStr2,pStr1,iMax);
}

int _cdecl strawicmp(const wchar_t *pStr1,const wchar_t *pStr2,int iMax)
{
return wcsnicmp(pStr1,pStr2,iMax);
}

#endif


//*****************************************************************************
//*
//*		strgetlist
//*
//*****************************************************************************
//	Holt Zahlenwerte aus einer Liste, und speichert sie in einem Array.
//	pString		: Zeiger auf den Listentext.
//	pList		: Zeiger auf das Array fr die Listenwerte.
//	iSize		: Ist die Gre des Listen-Arrays in Eintrgen.
//	iFlags		: Diverse Flags
//					STR_GL_CHAR		= die Listenwerte sind 1 Byte gro
//					STR_GL_SHORT	= die Listenwerte sind 2 Byte gro
//					STR_GL_INT		= die Listenwerte sind 4 Byte gro
//					STR_GL_NEG		= das Minuszeichen ist in der Liste erlaubt
//					STR_GL_ZERO		= den restlichen Listenbereich mit 0x00 fllen
//					STR_GL_FILL		= den restlichen Listenbereich mit 0xFF fllen
//					STR_GL_BRACKES	= der Text kann mit '"' eingegrenzt sein
//					STR_GL_BIN		= nur Binrzahlen sind in der Liste
//					STR_GL_DEC		= nur Dezimalzahlen sind in der Liste
//					STR_GL_OCT		= nur Oktalzahlen sind in der Liste
//					STR_GL_HEX		= nur Hexadezimalzahlen sind in der Liste
//					STR_GL_SEMI		= nur mit Semikolon trennen ';'
//					STR_GL_POINT	= nur mit Punkt trennen '.'
//					STR_GL_DPOINT	= nur mit Doppelpunkt trennen ':'
//					STR_GL_CUTTER	= nur mit Beistrich trennen ','
//					STR_GL_SPACE	= nur mit Leerbereich trennen ' ' oder '\t'
//					STR_GL_MINUS	= nur mit Leerbereich trennen '-'
//					STR_GL_TAB		= nur mit Tabulator trennen '\t'
//					STR_GL_LINES	= es gibt mehrere Zeilen
//					STR_GL_LINEADD	= mit '\' am Zeilenende wird die Zeile verlngert
//					STR_GL_HEXDUMP  = bei Hexdumps Adress+Ascci berspringen
//									  Bit 0..7 = Breite des Dumps in Werten
//	pCount		: Wenn dieser Zeiger ungleich 0 ist wird hier die Anzahl der
//				  verarbeiteten Zeichen gespeichert.
//	Ergibt die Anzahl der gespeicherten Werte in der Liste
int _cdecl strgetlist(const char *pString,void *pList,int iSize,int iFlags,int *pCount)
{
const char *pStart=pString;
int			iCount;
int			iShift;
int			iBase;
int			iByte;
int			iMask;
int			iStop;
int			iLen;
int			iVal;
int			iNum;
int			iMax;



	iByte	= ((iFlags>>8)&3)+1;
	iBase	= "\x00\x02\x0A\x08\x10\x00\x00\x00"[(iFlags>>12)&0x07];
	iMask	= aStrGetListTab[(iFlags>>16)&0x07];
	iShift	= 0x000000A0;


	if(!(iFlags&0x00070000))
		{
		iShift |= aStrGetListTab[0];
		iMask   = 0;
		}
	else{
		iShift &= ~(iMask&0x00000020);
		}


		pString += strwlen(pString,iShift);
	if(*pString=='"' && (iFlags&STR_GL_BRACKES))		// Ist die Liste zwischen "...."
		{
		pString += 1;
		pString += strwlen(pString,iShift);
		iStop    = 0x80000080;
		}
	else{
		iFlags	&=~STR_GL_BRACKES;
		iStop    = 0x80000000;
		}

		iMax=iFlags&STR_GL_HEXDUMP;						// Anzahl der Werte bei Hex-Dump (0=keiner)
	if(!iMax)iMax=0x10000000;
	else	 iBase=16;

	iCount = 0;

	while(!strwchara(*pString,iStop))					// Zeilenschleife
		{
		if(iFlags&STR_GL_HEXDUMP)						// berspringe Adresse im Hex-Dump
			{
			if(*pString=='[')							// [1234]
				{
				pString += 1;
				pString += strwlen(pString,0x3F);

				if(*pString==']')
					{
					pString += 1;
					pString += strwlen(pString,iShift);
					}
				}
			else{										// 1234
				iLen = strwlen(pString,0x0F);

				if(iLen!=iByte*2)
					{
					pString += iLen;
					pString += strwlen(pString,iShift);
					}
				}
			}

		for(iNum=0;iNum<iMax;iNum++)					// Wertschleife
			{
			if(*pString=='-' && !(iFlags&STR_GL_MINUS))	// Negativer Wert
				{
				if(!(iFlags&STR_GL_NEG))break;
				pString++;
				if(!strwchara(*pString,0x07))break;
				iVal = -strtoi(pString,iBase,&iLen);
				}
			else{
				iVal =  strtoi(pString,iBase,&iLen);
				}

			pString += iLen;
			pString += strwlen(pString,iShift);

			if(iMask && *pString)						// Trennzeichen prfen
				{
				if(!strwchara(*pString,iMask))
					{
					iLen     = 0;
					}
				else{
					iLen     = 1;
					pString += 1;
					pString += strwlen(pString,iShift);
					}
				}

			if(iLen==0)									// Kein Wert gefunden ?
				{
				break;
				}

			if(iCount<iSize)							// Wert in die Liste einfgen
				{
				memcpy(pList,&iVal,iByte);
				pList = (char*)pList+iByte;
				iCount++;
				}
			}


		if(iFlags&(STR_GL_HEXDUMP))						// Bei Hexdump zum Zeilenende
		if(iFlags&(STR_GL_LINEADD|STR_GL_LINES))
			{
			pString += strwlen(pString,~(0x40|iStop));
			if(iFlags&STR_GL_LINEADD)pString--;
			}


		if(iFlags&STR_GL_LINEADD)						// Zeile mit '\' vorsetzen
			{
			if(*pString!='\\')break;
			pString++;
			goto CheckLine;
			}

		if(!(iFlags&STR_GL_LINES))						// Weitere Zeilen
			{
			break;
			}

CheckLine:

		if(*pString=='\r')
			{
				pString++;
			if(*pString=='\n')pString++;
			pString += strwlen(pString,0x30);
			continue;
			}

		if(*pString=='\n')
			{
			pString += 1;
			pString += strwlen(pString,0x30);
			continue;
			}

		break;
		}


	if(pCount)											// Verarbeitete Lnge speichern
		{
		*pCount = (int)(pString-pStart);
		}

	if(iFlags&STR_GL_ZERO)								// den restlichen Listenbereich mit 0x00 fllen
		{
		memset(pList,0x00,(iSize-iCount)*iByte);
		}
	else if(iFlags&STR_GL_FILL)							// den restlichen Listenbereich mit 0xFF fllen
		{
		memset(pList,0xFF,(iSize-iCount)*iByte);
		}


return iCount;
}

int _cdecl strgetlist(const wchar_t *pString,void *pList,int iSize,int iFlags,int *pCount)
{
const wchar_t  *pStart=pString;
int				iCount;
int				iShift;
int				iBase;
int				iByte;
int				iMask;
int				iStop;
int				iLen;
int				iVal;
int				iNum;
int				iMax;



	pString+= strwlen(pString,0x20);
	iByte	= ((iFlags>>8)&3)+1;
	iBase	= "\x00\x02\x0A\x08\x10\x00\x00\x00"[(iFlags>>12)&0x07];
	iMask	= aStrGetListTab[(iFlags>>16)&0x07];
	iShift	= 0x000000A0;

	if(!(iFlags&0x00070000))
		{
		iShift |= aStrGetListTab[0];
		iMask   = 0;
		}
	else{
		iShift &= ~(iMask&0x00000020);
		}


	if(*pString=='"' && (iFlags&STR_GL_BRACKES))		// Ist die Liste zwischen "...."
		{
		pString += 1;
		pString += strwlen(pString,0x20);
		iStop    = 0x80000080;
		}
	else{
		iFlags	&=~STR_GL_BRACKES;
		iStop    = 0x80000000;
		}

		iMax=iFlags&STR_GL_HEXDUMP;
	if(!iMax)iMax=0x10000000;
	else	 iBase=16;

	iCount = 0;

	while(!strwcharw(*pString,iStop))					// Zeilenschleife
		{
		if(iFlags&STR_GL_HEXDUMP)						// berspringe Adresse im Hex-Dump
			{
			if(*pString=='[')							// [1234]
				{
				pString += 1;
				pString += strwlen(pString,0x2F);

				if(*pString==']')
					{
					pString += 1;
					pString += strwlen(pString,0x20);
					}
				}
			else{										// 1234
				iLen = strwlen(pString,0x0F);

				if(iLen!=iByte*2)
					{
					pString += iLen;
					pString += strwlen(pString,0x20);
					}
				}
			}

		for(iNum=0;iNum<iMax;iNum++)					// Wertschleife
			{
			if(*pString=='-' && !(iFlags&STR_GL_MINUS))	// Negativer Wert
				{
				if(!(iFlags&STR_GL_NEG))break;
				pString++;
				if(!strwcharw(*pString,0x07))break;
				iVal = -strtoi(pString,iBase,&iLen);
				}
			else{
				iVal =  strtoi(pString,iBase,&iLen);
				}

			pString += iLen;
			pString += strwlen(pString,iShift);

			if(iMask && *pString)						// Trennzeichen prfen
				{
				if(!strwcharw(*pString,iMask))
					{
					iLen = 0;
					}
				else{
					pString += 1;
					pString += strwlen(pString,iShift);
					iLen = 1;
					}
				}

			if(iLen==0)									// Kein Wert gefunden ?
				{
				break;
				}

			if(iCount<iSize)							// Wert in die Liste einfgen
				{
				memcpy(pList,&iVal,iByte);
				pList = (char*)pList+iByte;
				iCount++;
				}
			}


		if(iFlags&STR_GL_HEXDUMP)						// Bei Hexdump zum Zeilenende
		if(iFlags&(STR_GL_LINEADD|STR_GL_LINES))
			{
			pString += strwlen(pString,~(0x40|iStop));
			if(iFlags&STR_GL_LINEADD)pString--;
			}


		if(iFlags&STR_GL_LINEADD)						// Zeile mit '\' vorsetzen
			{
			if(*pString!='\\')break;
			pString++;
			goto CheckLine;
			}

		if(!(iFlags&STR_GL_LINES))						// Weitere Zeilen
			{
			break;
			}

CheckLine:

		if(*pString=='\r')
			{
				pString++;
			if(*pString=='\n')pString++;
			pString += strwlen(pString,0x20);
			continue;
			}

		if(*pString=='\n')
			{
			pString += 1;
			pString += strwlen(pString,0x20);
			continue;
			}

		break;
		}


	if(pCount)											// Verarbeitete Lnge speichern
		{
		*pCount = (int)(pString-pStart);
		}

	if(iFlags&STR_GL_ZERO)								// den restlichen Listenbereich mit 0x00 fllen
		{
		memset(pList,0x00,(iSize-iCount)*iByte);
		}
	else if(iFlags&STR_GL_FILL)							// den restlichen Listenbereich mit 0xFF fllen
		{
		memset(pList,0xFF,(iSize-iCount)*iByte);
		}


return iCount;
}


//*****************************************************************************
//*
//*		strwcmp
//*
//*****************************************************************************
//	Vergleicht zwei Texte unter Beachtung der Platzhalter '*' und '?'.
//	pString		: Ist der Zeiger auf den Text ohne Platzhalter.
//	pCompare	: Ist der Zeiger auf den Text mit  Platzhalter.
//	Ergibt 0 wenn die Texte gleich sind, oder einen Wert ungleich Null wenn nicht.
int _cdecl strwcmp(const char *pString, const char *pCompare)
{
int		iDiff;


	while(*pCompare)
		{
		if(*pCompare=='?')								// berspringe einzelnes Zeichen
			{
			if(!*pString)
				{
				return 1;
				}
			}
		else if (*pCompare=='*')						// Rekursiver Vergleich
			{
			if(!strwcmp(pString,pCompare+1))
				{
				return 0;
				}

			if(*pString && !strwcmp(pString+1,pCompare))
				{
				return 0;
				}

			return 1;
			}
		else{											// Vergleiche einzelnes Zeichen
			iDiff = *pCompare-*pString;

			if(iDiff)
				{
				return iDiff;
				}
			}

		pCompare++;
		pString++;
		}


	if(*pString)										// War alles gleich
		{
		return 1;
		}


return 0;
}

int _cdecl strwcmp(const wchar_t *pString, const wchar_t *pCompare)
{
int		iDiff;


	while(*pCompare)
		{
		if(*pCompare=='?')								// berspringe einzelnes Zeichen
			{
			if(!*pString)
				{
				return 1;
				}
			}
		else if (*pCompare=='*')						// Rekursiver Vergleich
			{
			if(!strwcmp(pString,pCompare+1))
				{
				return 0;
				}

			if(*pString && !strwcmp(pString+1,pCompare))
				{
				return 0;
				}

			return 1;
			}
		else{											// Vergleiche einzelnes Zeichen
			iDiff = *pCompare-*pString;

			if(iDiff)
				{
				return iDiff;
				}
			}

		pCompare++;
		pString++;
		}


	if(*pString)										// War alles gleich
		{
		return 1;
		}


return 0;
}

//*****************************************************************************
//*
//*		strwicmp
//*
//*****************************************************************************
//	Vergleicht zwei Texte unter Beachtung der Platzhalter '*' und '?'.
//	pString		: Ist der Zeiger auf den Text ohne Platzhalter.
//	pCompare	: Ist der Zeiger auf den Text mit  Platzhalter.
//	Ergibt 0 wenn die Texte gleich sind, oder einen Wert ungleich Null wenn nicht.
int _cdecl strwicmp(const char *pString, const char *pCompare)
{
int		iChar2;
int		iChar1;
int		iDiff;


	while(*pCompare)
		{
		if(*pCompare=='?')								// berspringe einzelnes Zeichen
			{
			if(!*pString)
				{
				return 1;
				}
			}
		else if (*pCompare=='*')						// Rekursiver Vergleich
			{
			if(!strwicmp(pString,pCompare+1))
				{
				return 0;
				}

			if(*pString && !strwicmp(pString+1,pCompare))
				{
				return 0;
				}

			return 1;
			}
		else{											// Vergleiche einzelnes Zeichen
			iChar2	= *pCompare;
			iChar1	= *pString;

			if(strwchara((char)iChar2,0x02))iChar2+='a'-'A';
			if(strwchara((char)iChar1,0x02))iChar1+='a'-'A';

			iDiff = iChar2-iChar1;

			if(iDiff)
				{
				return iDiff;
				}
			}

		pCompare++;
		pString++;
		}


	if(*pString)										// War alles gleich
		{
		return 1;
		}


return 0;
}

int _cdecl strwicmp(const wchar_t *pString, const wchar_t *pCompare)
{
int		iChar2;
int		iChar1;
int		iDiff;


	while(*pCompare)
		{
		if(*pCompare=='?')								// berspringe einzelnes Zeichen
			{
			if(!*pString)
				{
				return 1;
				}
			}
		else if (*pCompare=='*')						// Rekursiver Vergleich
			{
			if(!strwicmp(pString,pCompare+1))
				{
				return 0;
				}

			if(*pString && !strwicmp(pString+1,pCompare))
				{
				return 0;
				}

			return 1;
			}
		else{											// Vergleiche einzelnes Zeichen
			iChar2	= *pCompare;
			iChar1	= *pString;

			if(strwcharw((wchar_t)iChar2,0x02))iChar2+='a'-'A';
			if(strwcharw((wchar_t)iChar1,0x02))iChar1+='a'-'A';

			iDiff = iChar2-iChar1;

			if(iDiff)
				{
				return iDiff;
				}
			}

		pCompare++;
		pString++;
		}


	if(*pString)										// War alles gleich
		{
		return 1;
		}


return 0;
}

//*****************************************************************************
//*
//*		strfromcsv
//*
//*****************************************************************************
//	Dekodiert einen Eintrag aus einer CSV-Spalte
//	pString	: Zeiger auf die CSV-Spalte
//	pLen	: Hier wird die Lnge des Textes im Zielpuffer gespeichert
//	pBuffer	: Zeiger auf den Zielpuffer
//	iSize	: Lnge des	Zielpuffers
//	iLen	: Lnge der CSV-Spalte
//	cSign	: Ist das Trennzeichen (';' oder '\t')
//	Ergibt die verarbeitetet Lnge in der CSV-Spalte inklusive Trennzeichen am Ende.
int _cdecl strfromcsv(const char *pString,int *pLen,char *pBuffer,int iSize,int iLen,char cSign)
{
int		iOff;
int		iNum;


	iSize--;

	if(iLen<=0)											// Ist die Spalte leer
		{
		if(iSize>=0)*pBuffer=0;
		if(pLen)*pLen=0;
		return 0;
		}

	iNum = 0;

	if(*pString=='"')									// Angabe mit Anfhrungszeichen
		{
		for(iOff=1;iOff<iLen;iOff++)
			{
			if(pString[iOff]==0)
				{
				break;
				}

			if(pString[iOff]!='"')
				{
				if(iNum>=iSize)continue;
				pBuffer[iNum] = pString[iOff];
				iNum++;
				continue;
				}

			iOff++;

			if(iOff<iLen && pString[iOff]=='"')
				{
				if(iNum>=iSize)continue;
				pBuffer[iNum] = '"';
				iNum++;
				continue;
				}

			break;
			}

		}
	else{												// Angabe ohne Anfhrungszeichen
		for(iOff=0;iOff<iLen;iOff++)
			{
			if(strwchara(pString[iOff],0x80000040))
				{
				break;
				}

			if(pString[iOff]==cSign)
				{
				break;
				}

			if(iNum>=iSize)continue;
			pBuffer[iNum] = pString[iOff];
			iNum++;
			}
		}


	for(;iOff<iLen;iOff++)								// Vor ans Ende der Spalte
		{
		if(strwchara(pString[iOff],0x80000040))
			{
			break;
			}

		if(pString[iOff]==cSign)
			{
			iOff++;
			break;
			}
		}

	if(iSize>=0)pBuffer[iNum]=0;
	if(pLen)*pLen=iNum;


return iOff;
}

int _cdecl strfromcsv(const wchar_t *pString,int *pLen,wchar_t *pBuffer,int iSize,int iLen,wchar_t cSign)
{
int		iOff;
int		iNum;


	iSize--;

	if(iLen<=0)											// Ist die Spalte leer
		{
		if(iSize>=0)*pBuffer=0;
		if(pLen)*pLen=0;
		return 0;
		}

	iNum = 0;

	if(*pString=='"')									// Angabe mit Anfhrungszeichen
		{
		for(iOff=1;iOff<iLen;iOff++)
			{
			if(pString[iOff]==0)
				{
				break;
				}

			if(pString[iOff]!='"')
				{
				if(iNum>=iSize)continue;
				pBuffer[iNum] = pString[iOff];
				iNum++;
				continue;
				}

			iOff++;

			if(iOff<iLen && pString[iOff]=='"')
				{
				if(iNum>=iSize)continue;
				pBuffer[iNum] = '"';
				iNum++;
				continue;
				}

			break;
			}

		}
	else{												// Angabe ohne Anfhrungszeichen
		for(iOff=0;iOff<iLen;iOff++)
			{
			if(strwcharw(pString[iOff],0x80000040))
				{
				break;
				}

			if(pString[iOff]==cSign)
				{
				break;
				}

			if(iNum>=iSize)continue;
			pBuffer[iNum] = pString[iOff];
			iNum++;
			}
		}


	for(;iOff<iLen;iOff++)								// Vor ans Ende der Spalte
		{
		if(strwcharw(pString[iOff],0x80000040))
			{
			break;
			}

		if(pString[iOff]==cSign)
			{
			iOff++;
			break;
			}
		}

	if(iSize>=0)pBuffer[iNum]=0;
	if(pLen)*pLen=iNum;


return iOff;
}

//*****************************************************************************
//*
//*		strtocsv
//*
//*****************************************************************************
//	Kodiert einen Text in eine CSV-Spalte mit Abschuzeichen
//	pString	: Zeiger auf den Text
//	pBuffer	: Zeiger auf den Zielpuffer fr die CSV-Spalte
//	pLen	: Hier wird die Lnge des Textes im Zielpuffer gespeichert
//	iSize	: Lnge des	Zielpuffers
//	iLen	: Lnge der CSV-Spalte
//	cSign	: Ist das Trennzeichen (';' oder '\t')
//	Ergibt die Lnge des Zielpuffers inklusive Trennzeichen am Ende.
int _cdecl strtocsv(const char *pString,char *pBuffer,int iSize,int iLen,char cSign)
{
int		iOff;
int		iNum;



	if(iSize<=1)										// Puffergre berprfen
		{
		if(iSize<=0)return 0;
		*pBuffer = 0;
		return 0;
		}


	for(iNum=0;iNum<iLen;iNum++)						// Suche Spezialzeichen
		{
		if(pString[iNum]==0){iLen=iNum;break;}
		if(pString[iNum]==cSign)break;
		if(strwchara(pString[iNum],0x0160))break;
		}


	if(iNum==iLen && !strwchara(pString[0],0xE0) && (iNum==0 || !strwchara(pString[iNum-1],0xE0)))
		{
		iOff = strlcpy(pBuffer,pString,iSize,iLen);

		if(iOff+1>=iSize)iOff--;

		pBuffer[iOff] = cSign;	iOff++;
		pBuffer[iOff] = 0;

		return iOff;
		}


	if(iSize<=3)
		{
		pBuffer[0] = cSign;
		pBuffer[1] = 0;
		return 1;
		}

	pBuffer[0] = '"';
	iSize -= 3;
	iOff   = 1;

	for(iNum=0;iNum<iLen;iNum++)						// Suche Spezialzeichen
		{
		if(iOff>=iSize)break;
		if(pString[iNum]==0)break;

		if(pString[iNum]=='"')
			{
			if(iOff+2>iSize)break;
			pBuffer[iOff] = '"';	iOff++;
			pBuffer[iOff] = '"';	iOff++;
			continue;
			}

		pBuffer[iOff] = pString[iNum];
		iOff++;
		}


	pBuffer[iOff] = '"';	iOff++;						// Anfhrungszeichen anhngen
	pBuffer[iOff] = cSign;	iOff++;						// Trennzeichen anhngen
	pBuffer[iOff] = 0;


return iOff;
}

int _cdecl strtocsv(const wchar_t *pString,wchar_t *pBuffer,int iSize,int iLen,wchar_t cSign)
{
int		iOff;
int		iNum;



	if(iSize<=1)										// Puffergre berprfen
		{
		if(iSize<=0)return 0;
		*pBuffer = 0;
		return 0;
		}


	for(iNum=0;iNum<iLen;iNum++)						// Suche Spezialzeichen
		{
		if(pString[iNum]==0){iLen=iNum;break;}
		if(pString[iNum]==cSign)break;
		if(strwcharw(pString[iNum],0x0160))break;
		}


	if(iNum==iLen && !strwcharw(pString[0],0xE0) && (iNum==0 || !strwcharw(pString[iNum-1],0xE0)))
		{
		iOff = strlcpy(pBuffer,pString,iSize,iLen);

		if(iOff+1>=iSize)iOff--;

		pBuffer[iOff] = cSign;	iOff++;
		pBuffer[iOff] = 0;

		return iOff;
		}


	if(iSize<=3)
		{
		pBuffer[0] = cSign;
		pBuffer[1] = 0;
		return 1;
		}

	pBuffer[0] = '"';
	iSize -= 3;
	iOff   = 1;

	for(iNum=0;iNum<iLen;iNum++)						// Suche Spezialzeichen
		{
		if(iOff>=iSize)break;
		if(pString[iNum]==0)break;

		if(pString[iNum]=='"')
			{
			if(iOff+2>iSize)break;
			pBuffer[iOff] = '"';	iOff++;
			pBuffer[iOff] = '"';	iOff++;
			continue;
			}

		pBuffer[iOff] = pString[iNum];
		iOff++;
		}


	pBuffer[iOff] = '"';	iOff++;						// Anfhrungszeichen anhngen
	pBuffer[iOff] = cSign;	iOff++;						// Trennzeichen anhngen
	pBuffer[iOff] = 0;


return iOff;
}

#ifdef _DEBUG

int _cdecl strfromcsv(const char *pString,char *pBuffer,int iSize,int iLen,char cSign)
{
return strfromcsv(pString,0,pBuffer,iSize,iLen,cSign);
}

int _cdecl strfromcsv(const wchar_t *pString,wchar_t *pBuffer,int iSize,int iLen,wchar_t cSign)
{
return strfromcsv(pString,0,pBuffer,iSize,iLen,cSign);
}

#endif

//*****************************************************************************
//*
//*		strtoipv4
//*
//*****************************************************************************
//	Wandelt eine Text in eine IP-Adresse samt Sub-Net-Maske um.
//	Leerzeichen am Anfang und am Ende werden bersprungen.
//	pString		: Ist der Zeiger auf den Text mit der IP-Adresse.
//					z.B.:	"127.0.0.1"  oder "192.168.0.11/8"
//	pIpAddr		: Zeiger auf einen 4-Byte-Speicher fr die IP-Adresse.
//				  Bei Fehlern wird 255.255.255.255 gespeichert.
//				  Wenn dieser Zeiger 0 ist wird die IP-Adresse nicht gespeichert.
//	pSubNet		: Zeiger auf einen 4-Byte-Speicher fr die Sub-Net-Maske.
//				  Wenn dieser Zeiger 0 ist wird keine Sub-Net-Maske geholt.
//	uMode		: Bestimmt die Art der Textverarbeitung:
//					Bit0 = Leerzeichen am Anfang berspringen
//					Bit1 = Leerzeichen am Ende berspringen
//					Bit2 = Sub-Net-Maske mit ".../?" dekodieren
//					Bit3 = Erstes Byte auf Offset 3 speichern
//					Bit5 = Subnetz als INT-Bitzahl speichern
//					Bit6 = Port mit "...:123" einlesen
//	Ergibt die Lnge der verarbeiten Zeichen.
int _cdecl strtoipv4(const char *pString,void *pIpAddr,void *pSubNet,unsigned uMode)
{
unsigned char	aAddr[4];
unsigned		uError;
unsigned		uMask;
unsigned 		uVal;
int				iPos;
int				iOff;
int				iNum;


	if(uMode&0x01)										// Leerzeichen am Anfang berspringen
			iOff=strwlen(pString,0xA0);
	else	iOff=0;

	uMask	= 0;
	uError	= 0;

	for(iPos=0;;iPos++)									// Zahlen der IP-Adresse einlesen
		{
		   uVal = strtoi(pString+iOff,0xE00A,&iNum);
		if(uVal>255)uError=0xFFFFFFFF;
		aAddr[iPos] = (unsigned char)uVal;
		iOff += iNum;

		if(iPos>=3)										// Ende erreicht
			{
			if(!(uMode&0x44))break;

			if(uMode&0x40)								// Port-Angabe ("192.168.0.11:1234")
				{
				if(pString[iOff]!=':')break;
				}
			else{										// Subnetz ("192.168.0.11/24")
				if(pString[iOff]!='/')break;
				}

			iOff += 1;
			uMask = strtoi(pString+iOff,0xE00A,&iNum);
			iOff += iNum;

			break;
			}

		if(pString[iOff]!='.')							// Punkt nach der Zahl
			{
			uError=0xFFFFFFFF;
			break;
			}

		iOff++;
		}

	if(uMode&0x02)										// Leerzeichen am Ende berspringen
		{
		iOff += strwlen(pString+iOff,0xA0);
		}


	if(pIpAddr)											// IP-Adresse speichern
		{
		if(uMode&0x08)
			{
			uVal = uError|*(int*)aAddr;
			((char*)pIpAddr)[0] = (char)(uVal>>24);
			((char*)pIpAddr)[1] = (char)(uVal>>16);
			((char*)pIpAddr)[2] = (char)(uVal>>8);
			((char*)pIpAddr)[3] = (char)(uVal);
			}
		else{
			*(int*)pIpAddr  = *(int*)aAddr;
			*(int*)pIpAddr |= uError;
			}
		}


	if(pSubNet)											// Subnet-Maske bzw. Port speichern
		{
		if(!(uMode&0x40) && uMask>31)
			{
			uError = 0xFFFFFFFF;
			}

		if(uMode&0x60)
			{
			*(int*)pSubNet  = uMask;
			}
		else{
		 	uMask  = 0xFFFFFFFF<<uMask;
			uMask |= uError;

			if(uMode&0x08)
				{
				*(int*)pSubNet  = uMask;
				*(int*)pSubNet |= uError;
				}
			else{
				((char*)pSubNet)[0] = (char)(uMask>>24);
				((char*)pSubNet)[1] = (char)(uMask>>16);
				((char*)pSubNet)[2] = (char)(uMask>>8);
				((char*)pSubNet)[3] = (char)(uMask);
				}
			}
		}



return iOff;
}

int _cdecl strtoipv4(const wchar_t *pString,void *pIpAddr,void *pSubNet,unsigned uMode)
{
unsigned char	aAddr[4];
unsigned		uError;
unsigned		uMask;
unsigned 		uVal;
int				iPos;
int				iOff;
int				iNum;


	if(uMode&0x01)										// Leerzeichen am Anfang berspringen
			iOff=strwlen(pString,0xA0);
	else	iOff=0;

	uMask	= 0;
	uError	= 0;

	for(iPos=0;;iPos++)									// Zahlen der IP-Adresse einlesen
		{
		   uVal = strtoi(pString+iOff,0xE00A,&iNum);
		if(uVal>255)uError=0xFFFFFFFF;
		aAddr[iPos] = (unsigned char)uVal;
		iOff += iNum;

		if(iPos>=3)										// Ende erreicht
			{
			if(!(uMode&0x44))break;

			if(uMode&0x40)								// Port-Angabe ("192.168.0.11:1234")
				{
				if(pString[iOff]!=':')break;
				}
			else{										// Subnetz ("192.168.0.11/24")
				if(pString[iOff]!='/')break;
				}

			iOff += 1;
			uMask = strtoi(pString+iOff,0xE00A,&iNum);
			iOff += iNum;

			break;
			}

		if(pString[iOff]!='.')							// Punkt nach der Zahl
			{
			uError=0xFFFFFFFF;
			break;
			}

		iOff++;
		}

	if(uMode&0x02)										// Leerzeichen am Ende berspringen
		{
		iOff += strwlen(pString+iOff,0xA0);
		}


	if(pIpAddr)											// IP-Adresse speichern
		{
		if(uMode&0x08)
			{
			uVal = uError|*(int*)aAddr;
			((char*)pIpAddr)[0] = (char)(uVal>>24);
			((char*)pIpAddr)[1] = (char)(uVal>>16);
			((char*)pIpAddr)[2] = (char)(uVal>>8);
			((char*)pIpAddr)[3] = (char)(uVal);
			}
		else{
			*(int*)pIpAddr  = *(int*)aAddr;
			*(int*)pIpAddr |= uError;
			}
		}


	if(pSubNet)											// Subnet-Maske bzw. Port speichern
		{
		if(!(uMode&0x40) && uMask>31)
			{
			uError = 0xFFFFFFFF;
			}

		if(uMode&0x60)
			{
			*(int*)pSubNet  = uMask;
			}
		else{
		 	uMask  = 0xFFFFFFFF<<uMask;
			uMask |= uError;

			if(uMode&0x08)
				{
				*(int*)pSubNet  = uMask;
				*(int*)pSubNet |= uError;
				}
			else{
				((char*)pSubNet)[0] = (char)(uMask>>24);
				((char*)pSubNet)[1] = (char)(uMask>>16);
				((char*)pSubNet)[2] = (char)(uMask>>8);
				((char*)pSubNet)[3] = (char)(uMask);
				}
			}
		}



return iOff;
}

//*****************************************************************************
//*
//*		strtoipv6
//*
//*****************************************************************************
//	Wandelt eine Text in eine IPV6-Adresse samt Sub-Net-Maske um.
//	Leerzeichen am Anfang und am Ende werden bersprungen.
//	pString		: Ist der Zeiger auf den Text mit der IP-Adresse.
//					z.B.:	"1234:AAAA:BB:1" oder "1234:AAAA:BB:1/12" oder "::"
//							"127.0.0.1"  oder "192.168.0.11/8"
//	pIpAddr		: Zeiger auf einen 4-Byte-Speicher fr die IP-Adresse.
//				  Bei Fehlern wird 255.255.255.255 gespeichert.
//				  Wenn dieser Zeiger 0 ist wird die IP-Adresse nicht gespeichert.
//	pSubNet		: Wenn dieser Zeiger nicht NULL ist werden hier die Sub-Net-Bits
//				  gespeichert.
//	uMode		: Bestimmt die Art der Textverarbeitung:
//					Bit0 = Leerzeichen am Anfang berspringen
//					Bit1 = Leerzeichen am Ende berspringen
//					Bit2 = Sub-Net-Maske mit ".../?" dekodieren
//					Bit3 = Byte-Order der Worte drehen
//					Bit4 = Einhllung in Klammern ist erlaubt "[....]"
//					Bit6 = Port mit "...:123" einlesen (braucht "[....]:1234")
//	Ergibt die Lnge der verarbeiten Zeichen.
int _cdecl strtoipv6(const char *pString,void *pIpAddr,int *pSubNet,unsigned uMode)
{
unsigned short 	aAddr[8];
unsigned char  *pV4Data;
unsigned		uError;
unsigned		uMask;
unsigned		uNull;
unsigned 		uXor;
unsigned 		uVal;
int				iSub;
int				iPos;
int				iOff;
int				iNum;


	if(uMode&0x01)										// Leerzeichen am Anfang berspringen
			iOff=strwlen(pString,0xA0);
	else	iOff=0;


	if(pString[iOff]!='[' || !(uMode&0x10))				// Einhllung in Klammern vorhanden
			uMode&=~0x10;
	else	iOff++;

	uMask	= 0;
	iPos	= strwlen(pString+iOff,0x00040001);

	if(pString[iOff+iPos]=='.')							// Ist es eine IPV4-Adresse
		{
		iOff += strtoipv4(pString+iOff,aAddr+6,0,0x00);

		if(*(int*)(aAddr+6)==0xFFFFFFFF)
			{
			uError          =0xFFFFFFFF;
			*(int*)(aAddr  )=0xFFFFFFFF;
			*(int*)(aAddr+2)=0xFFFFFFFF;
			*(int*)(aAddr+4)=0xFFFFFFFF;
			}
		else{
		    uError	        =0x00000000;
			*(int*)(aAddr  )=0x00000000;
			*(int*)(aAddr+2)=0x00000000;
			*(int*)(aAddr+4)=0xFFFF0000;
			}

		if((uMode&0x40) && pString[iOff]!=']')			// Wenn Port einlesen ']' simulieren
			{
			uMode |= 0x10;
			}

		if(uMode&0x08)									// Byte-Order drehen
			{
			aAddr[6] = (unsigned short)((aAddr[6]<<8)|(aAddr[6]>>8));
			aAddr[7] = (unsigned short)((aAddr[7]<<8)|(aAddr[7]>>8));
			}
		}
	else{												// IPV6-Adresse
		uNull	= 8;
		uError	= 0;

		for(iPos=0;;iPos++)								// Zahlen der IP-Adresse einlesen
			{
			   uVal = strtoi(pString+iOff,0xE010,&iNum);
			if(uVal>0xFFFF)uError=0xFFFFFFFF;

			if(iNum==0)
				{
				if(uNull>=8 && pString[iOff  ]==':')
				if(iPos >=1 || pString[iOff+1]==':')
					{
					iOff += (iPos>=1)? 1:2;
					uNull =  iPos;
					iPos -=  1;
					continue;
					}

				if(pString[iOff]==':')
					{
					uError = 0xFFFFFFFF;
					}

				break;
				}

			iOff += iNum;

			if(pString[iOff]=='.')						// Spezille IP "::FFFF:192.168.0.1"
				{
                if(iPos<1 || iPos>6 || (uNull==8 && iPos!=6))
					{
					uError=0xFFFFFFFF;
					break;
					}

				iOff   -= iNum;
				uXor	= (uMode>>3)&1;
				pV4Data	= (unsigned char*)(aAddr+iPos);

				for(iSub=0;;iSub++,iOff++)				// IP-V4 einlesen
					{
					uVal = strtoi(pString+iOff,0xE00A,&iNum);

					if(uVal>255)break;

					iOff += iNum;
					pV4Data[iSub^uXor] = (unsigned char)uVal;

					if(iSub>=3){iSub=4;break;}
					if(pString[iOff]!='.')break;
					}

				if(iSub<4)								// Alle IP-V4 Segmente
					{
					uError=0xFFFFFFFF;
					}
				else{
					iPos += 2;
					}

				break;
				}

			if(uMode&0x08)								// Byte-Order drehen
				{
				aAddr[iPos]  = (unsigned short)(uVal);
				}
			else{
				aAddr[iPos]  = (unsigned short)(uVal>>8);
				aAddr[iPos] |= (unsigned short)(uVal<<8);
				}

			if(iPos>=7)									// Ende erreicht
				{
				iPos += 1;
				if(uNull>=8)break;
				uError=0xFFFFFFFF;
				break;
				}

			if(pString[iOff]!=':')						// Doppelpunkt nach der Zahl
				{
				iPos += 1;
				if(uNull<8)break;
				uError=0xFFFFFFFF;
				break;
				}

			iOff++;
			}

		if(uNull<8)										// Gab es eine Leerstelle
			{
			uVal  = 8-iPos;
			iNum  = iPos-uNull;
			memmove(aAddr+uNull+uVal,aAddr+uNull,iNum*2);
			memset (aAddr+uNull,0,uVal*2);
			}
		}


	if((uMode&0x44)==0x04 && pString[iOff]=='/')		// Subnetz einlesen
		{
		iOff += 1;
		uMask = strtoi(pString+iOff,0xE00A,&iNum);
		iOff += iNum;

		if(uError)uMask=0;
		}

	if((uMode&0x10) && pString[iOff]==']')				// Einhllung in Klammern vorhanden
		{
		iOff += 1;
		}

	if((uMode&0x50)==0x50 && pString[iOff]==':')		// Port einlesen
		{
		iOff += 1;
		uMask = strtoi(pString+iOff,0xE00A,&iNum);
		iOff += iNum;

		if(uError)uMask=0;
		}

	if(uMode&0x02)										// Leerzeichen am Ende berspringen
		{
		iOff += strwlen(pString+iOff,0xA0);
		}

	if(pIpAddr)											// IP-Adresse speichern
		{
		if(uError)
				memset(pIpAddr,0xFF ,16);
		else	memcpy(pIpAddr,aAddr,16);
		}

	if(pSubNet)											// Subnet-Bits speichern
		{
		if((uMask>31 && !(uMode&0x40)) || uError)uMask=0;
		*pSubNet = uMask;
		}



return iOff;
}

int _cdecl strtoipv6(const wchar_t *pString,void *pIpAddr,int *pSubNet,unsigned uMode)
{
unsigned short 	aAddr[8];
unsigned char  *pV4Data;
unsigned		uError;
unsigned		uMask;
unsigned		uNull;
unsigned 		uXor;
unsigned 		uVal;
int				iSub;
int				iPos;
int				iOff;
int				iNum;


	if(uMode&0x01)										// Leerzeichen am Anfang berspringen
			iOff=strwlen(pString,0xA0);
	else	iOff=0;


	if(pString[iOff]!='[' || !(uMode&0x10))				// Einhllung in Klammern vorhanden
			uMode&=~0x10;
	else	iOff++;

	uMask	= 0;
	iPos	= strwlen(pString+iOff,0x00040001);

	if(pString[iOff+iPos]=='.')							// Ist es eine IPV4-Adresse
		{
		iOff += strtoipv4(pString+iOff,aAddr+6,0,0x00);

		if(*(int*)(aAddr+6)==0xFFFFFFFF)
			{
			uError          =0xFFFFFFFF;
			*(int*)(aAddr  )=0xFFFFFFFF;
			*(int*)(aAddr+2)=0xFFFFFFFF;
			*(int*)(aAddr+4)=0xFFFFFFFF;
			}
		else{
			uError	        =0x00000000;
			*(int*)(aAddr  )=0x00000000;
			*(int*)(aAddr+2)=0x00000000;
			*(int*)(aAddr+4)=0xFFFF0000;
			}

		if((uMode&0x40) && pString[iOff]!=']')			// Wenn Port einlesen ']' simulieren
			{
			uMode |= 0x10;
			}

		if(uMode&0x08)									// Byte-Order drehen
			{
			aAddr[6] = (unsigned short)((aAddr[6]<<8)|(aAddr[6]>>8));
			aAddr[7] = (unsigned short)((aAddr[7]<<8)|(aAddr[7]>>8));
			}
		}
	else{												// IPV6-Adresse
		uNull	= 8;
		uError	= 0;

		for(iPos=0;;iPos++)								// Zahlen der IP-Adresse einlesen
			{
			   uVal = strtoi(pString+iOff,0xE010,&iNum);
			if(uVal>0xFFFF)uError=0xFFFFFFFF;

			if(iNum==0)
				{
				if(uNull>=8 && pString[iOff  ]==':')
				if(iPos >=1 || pString[iOff+1]==':')
					{
					iOff += (iPos>=1)? 1:2;
					uNull =  iPos;
					iPos -=  1;
					continue;
					}

				if(pString[iOff]==':')
					{
					uError = 0xFFFFFFFF;
					}

				break;
				}

			iOff += iNum;

			if(pString[iOff]=='.')						// Spezille IP "::FFFF:192.168.0.1"
				{
                if(iPos<1 || iPos>6 || (uNull==8 && iPos!=6))
					{
					uError=0xFFFFFFFF;
					break;
					}

				iOff -= iNum;
				uXor  = (uMode>>3)&1;
				pV4Data	= (unsigned char*)(aAddr+iPos);

				for(iSub=0;;iSub++,iOff++)				// IP-V4 einlesen
					{
					uVal = strtoi(pString+iOff,0xE00A,&iNum);

					if(uVal>255)break;

					iOff += iNum;

					pV4Data[iSub^uXor] = (unsigned char)uVal;

					if(iSub>=3){iSub=4;break;}
					if(pString[iOff]!='.')break;
					}

				if(iSub<4)								// Alle IP-V4 Segmente
					{
					uError=0xFFFFFFFF;
					}
				else{
					iPos += 2;
					}

				break;
				}

			if(uMode&0x08)								// Byte-Order drehen
				{
				aAddr[iPos]  = (unsigned short)(uVal);
				}
			else{
				aAddr[iPos]  = (unsigned short)(uVal>>8);
				aAddr[iPos] |= (unsigned short)(uVal<<8);
				}

			if(iPos>=7)									// Ende erreicht
				{
				iPos += 1;
				if(uNull>=8)break;
				uError=0xFFFFFFFF;
				break;
				}

			if(pString[iOff]!=':')						// Doppelpunkt nach der Zahl
				{
				iPos += 1;
				if(uNull<8)break;
				uError=0xFFFFFFFF;
				break;
				}

			iOff++;
			}

		if(uNull<8)										// Gab es eine Leerstelle
			{
			uVal  = 8-iPos;
			iNum  = iPos-uNull;
			memmove(aAddr+uNull+uVal,aAddr+uNull,iNum*2);
			memset (aAddr+uNull,0,uVal*2);
			}
		}


	if((uMode&0x44)==0x04 && pString[iOff]=='/')		// Subnetz einlesen
		{
		iOff += 1;
		uMask = strtoi(pString+iOff,0xE00A,&iNum);
		iOff += iNum;

		if(uError)uMask=0;
		}

	if((uMode&0x10) && pString[iOff]==']')				// Einhllung in Klammern vorhanden
		{
		iOff += 1;
		}

	if((uMode&0x50)==0x50 && pString[iOff]==':')		// Port einlesen
		{
		iOff += 1;
		uMask = strtoi(pString+iOff,0xE00A,&iNum);
		iOff += iNum;

		if(uError)uMask=0;
		}

	if(uMode&0x02)										// Leerzeichen am Ende berspringen
		{
		iOff += strwlen(pString+iOff,0xA0);
		}

	if(pIpAddr)											// IP-Adresse speichern
		{
		if(uError)
				memset(pIpAddr,0xFF ,16);
		else	memcpy(pIpAddr,aAddr,16);
		}

	if(pSubNet)											// Subnet-Bits speichern
		{
		if((uMask>31 && !(uMode&0x40)) || uError)uMask=0;
		*pSubNet = uMask;
		}



return iOff;
}

//*****************************************************************************
//*
//*		strisip
//*
//*****************************************************************************
//	Prft ob ein Text eine IP-V4 oder eine IP-V6-Adresse ist
//	pString		: Zeiger auf den Text mit der IP-Adresse
//					"192.168.0.1"
//					"192.168.0.1:1000"
//					"192.168.0.1/8"
//					"::1"
//					"[::1]:1000"
//					"::1/8"
//					"[::1/8]"
//					"::FFFF:192.168.0.1"
//	iMode		: Bestimmt die Art der Auswertung
//					Bit0 = Leerzeichen am Anfang berspringen
//	  				Bit1 = Leerzeichen am Ende berspringen
//					Bit4 = Einhllung in Klammern erlauben
//	Ergibt ein Bit-Feld mit folgenter Bedeutung
//					Bit 0..15 =  Verarbeitete Zeichen
//					Bit 16    = IP-V4-Adresse
//					Bit 17    = IP-V6-Adresse
//					Bit 18    = Portangabe vorhanden
//					Bit 19    = Subnetztangabe vorhanden
//					Bit 20    = [...] Klammern um Adresse
int _cdecl strisip(const char *pString,unsigned uMode)
{
unsigned		uNull;
unsigned		uRet;
int				iSub;
int				iPos;
int				iOff;
int				iNum;


	if(uMode&0x01)										// Leerzeichen am Anfang berspringen
			iOff=strwlen(pString,0xA0);
	else	iOff=0;


	uRet = 0;

	if(pString[iOff]!='[' || !(uMode&0x10))				// Einhllung in Klammern vorhanden
		{
		uMode &= ~0x10;
		}
	else{
		uRet |= 0x100000;
		iOff += 1;
		}

	iPos = strwlen(pString+iOff,0x00040001);

	if(pString[iOff+iPos]=='.')							// Ist es eine IPV4-Adresse
		{
		for(iSub=0;;iSub++,iOff++)
			{
			iOff += strwlen(pString+iOff,0x01);

			if(strwchara(pString[iOff],0x07))
				{
				return iOff;
				}

			if(iSub>=3)break;

			if(pString[iOff]!='.')
				{
				return iOff;
				}
			}

		uRet   |= 0x10000;
		}
	else{												// IPV6-Adresse
		uNull	= 8;

		for(iPos=0;;iPos++)								// Zahlen der IP-Adresse einlesen
			{
			   iNum = strwlen(pString+iOff,0x00040001);
			if(iNum==0)
				{
				if(uNull>=8 && pString[iOff  ]==':')
				if(iPos >=1 || pString[iOff+1]==':')
					{
					iOff += (iPos>=1)? 1:2;
					uNull =  iPos;
					iPos -=  1;
					continue;
					}

				if(pString[iOff]!=':')
					{
					return iOff;
					}

				break;
				}

			iOff += iNum;

			if(pString[iOff]=='.')						// Spezille IP "::FFFF:192.168.0.1"
				{
                if(iPos<1 || iPos>6 || (uNull==8 && iPos!=6))
					{
					return iOff;
					}

				iOff   -= iNum;

				for(iSub=0;;iSub++,iOff++)				// IP-V4 einlesen
					{
					iOff += strwlen(pString+iOff,0x01);

					if(strwchara(pString[iOff],0x07))
						{
						return iOff;
						}

					if(iSub>=3)break;

					if(pString[iOff]!='.')
						{
						return iOff;
						}

					}

				iPos += 2;
				break;
				}

			if(iPos>=7)									// Ende erreicht
				{
				iPos += 1;
				if(uNull>=8)break;
				return iOff;
				}

			if(pString[iOff]!=':')						// Doppelpunkt nach der Zahl
				{
				iPos += 1;
				if(uNull<8)break;
				return iOff;
				}

			iOff++;
			}

		uRet   |= 0x20000;
		}


	if(pString[iOff]=='/')								// Subnetz einlesen
		{
		iOff += 1;
		uRet |= 0x80000;
		iOff += strwlen(pString+iOff,0x01);
		}

	if((uMode&0x10) && pString[iOff]==']')				// Einhllung in Klammern vorhanden
		{
		iOff += 1;
		}

	if(pString[iOff]==':')								// Port einlesen
		{
		iOff += 1;
		uRet |= 0x40000;
		iOff += strwlen(pString+iOff,0x01);
		}

	if(uMode&0x02)										// Leerzeichen am Ende berspringen
		{
		iOff += strwlen(pString+iOff,0xA0);
		}



return iOff|uRet;
}

int _cdecl strisip(const wchar_t *pString,unsigned uMode)
{
unsigned		uNull;
unsigned		uRet;
int				iSub;
int				iPos;
int				iOff;
int				iNum;


	if(uMode&0x01)										// Leerzeichen am Anfang berspringen
			iOff=strwlen(pString,0xA0);
	else	iOff=0;


	uRet = 0;

	if(pString[iOff]!='[' || !(uMode&0x10))				// Einhllung in Klammern vorhanden
		{
		uMode &= ~0x10;
		}
	else{
		uRet |= 0x100000;
		iOff += 1;
		}

	iPos = strwlen(pString+iOff,0x00040001);

	if(pString[iOff+iPos]=='.')							// Ist es eine IPV4-Adresse
		{
		for(iSub=0;;iSub++,iOff++)
			{
			iOff += strwlen(pString+iOff,0x01);

			if(strwcharw(pString[iOff],0x07))
				{
				return iOff;
				}

			if(iSub>=3)break;

			if(pString[iOff]!='.')
				{
				return iOff;
				}
			}

		uRet   |= 0x10000;
		}
	else{												// IPV6-Adresse
		uNull	= 8;

		for(iPos=0;;iPos++)								// Zahlen der IP-Adresse einlesen
			{
			   iNum = strwlen(pString+iOff,0x00040001);
			if(iNum==0)
				{
				if(uNull>=8 && pString[iOff  ]==':')
				if(iPos >=1 || pString[iOff+1]==':')
					{
					iOff += (iPos>=1)? 1:2;
					uNull =  iPos;
					iPos -=  1;
					continue;
					}

				if(pString[iOff]!=':')
					{
					return iOff;
					}

				break;
				}

			iOff += iNum;

			if(pString[iOff]=='.')						// Spezille IP "::FFFF:192.168.0.1"
				{
                if(iPos<1 || iPos>6 || (uNull==8 && iPos!=6))
					{
					return iOff;
					}

				iOff   -= iNum;

				for(iSub=0;;iSub++,iOff++)				// IP-V4 einlesen
					{
					iOff += strwlen(pString+iOff,0x01);

					if(strwcharw(pString[iOff],0x07))
						{
						return iOff;
						}

					if(iSub>=3)break;

					if(pString[iOff]!='.')
						{
						return iOff;
						}

					}

				iPos += 2;
				break;
				}

			if(iPos>=7)									// Ende erreicht
				{
				iPos += 1;
				if(uNull>=8)break;
				return iOff;
				}

			if(pString[iOff]!=':')						// Doppelpunkt nach der Zahl
				{
				iPos += 1;
				if(uNull<8)break;
				return iOff;
				}

			iOff++;
			}

		uRet   |= 0x20000;
		}


	if(pString[iOff]=='/')								// Subnetz einlesen
		{
		iOff += 1;
		uRet |= 0x80000;
		iOff += strwlen(pString+iOff,0x01);
		}

	if((uMode&0x10) && pString[iOff]==']')				// Einhllung in Klammern vorhanden
		{
		iOff += 1;
		}

	if(pString[iOff]==':')								// Port einlesen
		{
		iOff += 1;
		uRet |= 0x40000;
		iOff += strwlen(pString+iOff,0x01);
		}

	if(uMode&0x02)										// Leerzeichen am Ende berspringen
		{
		iOff += strwlen(pString+iOff,0xA0);
		}



return iOff|uRet;
}



//*****************************************************************************
//*
//*		ipv4tostr
//*
//*****************************************************************************
//	Wandelt eine IPV4-Adresse in eine Text um
//	pString		: Ist der Zeiger auf den Textpuffer fr die IP-Adresse.
//				  Der Puffer sollte mindestens 32 Byte gro sein.
//	pIpAddr		: Zeiger auf den 4-Byte-Speicher der IP-Adresse.
//	uSubNet		: Anzahl der Bits fr das Subnet
//	uMode		: Bestimmt die Art der Textverarbeitung:
//					Bit0 = Leerzeichen am Anfang einfgen
//					Bit1 = Leerzeichen am Ende einfgen
//					Bit2 = Sub-Net-Maske ".../?" anhngen.
//					Bit3 = Erstes Byte ist auf Offset 3
//					Bit5 = Port "...:?" anhngen.
//	Ergibt die Lnge des erstellten Textes.
int _cdecl ipv4tostr(char *pString,const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
unsigned char  *pAddr;
unsigned		uPos;
unsigned		uXor;
unsigned		uOff;


	pAddr = (unsigned char*)pIpAddr;
	uXor  = (uMode&0x08)? 0x03:0x00;

	if(uMode&0x01)										// Leerzeichen am Anfang einfgen
		{
		pString[0]	= ' ';
		uOff		= 1;
		}
	else{
		uOff		= 0;
		}

	for(uPos=0;;uPos++)									// Zahlen fr IP-Adresse ausgeben
		{
		uOff += itostr(pAddr[uPos^uXor],pString+uOff,15);

		if(uPos==3)break;

		pString[uOff] = '.';
		uOff++;
		}

	if(uSubNet && (uMode&0x20))							// Port anhngen "..:?"
		{
		pString[uOff] = ':';
		uOff += 1;
		uOff += itostr(uSubNet&0xFFFF,pString+uOff,15);
		}
	else if(uSubNet && (uMode&0x04))					// Subnet anhngen "../?"
		{
		pString[uOff] = '/';
		uOff += 1;
		uOff += itostr(uSubNet,pString+uOff,15);
		}

	if(uMode&0x02)										// Leerzeichen am Ende einfgen
		{
		pString[uOff] = ' '; uOff++;
		pString[uOff] = 0;
		}


return uOff;
}

int _cdecl ipv4tostr(wchar_t *pString,const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
unsigned char  *pAddr;
unsigned		uPos;
unsigned		uXor;
unsigned		uOff;


	pAddr = (unsigned char*)pIpAddr;
	uXor  = (uMode&0x08)? 0x03:0x00;

	if(uMode&0x01)										// Leerzeichen am Anfang einfgen
		{
		pString[0]	= ' ';
		uOff		= 1;
		}
	else{
		uOff		= 0;
		}

	for(uPos=0;;uPos++)									// Zahlen fr IP-Adresse ausgeben
		{
		uOff += itostr(pAddr[uPos^uXor],pString+uOff,15);

		if(uPos==3)break;

		pString[uOff] = '.';
		uOff++;
		}

	if(uSubNet && (uMode&0x20))							// Port anhngen "..:?"
		{
		pString[uOff] = ':';
		uOff += 1;
		uOff += itostr(uSubNet&0xFFFF,pString+uOff,15);
		}
	else if(uSubNet && (uMode&0x04))					// Subnet anhngen "../?"
		{
		pString[uOff] = '/';
		uOff += 1;
		uOff += itostr(uSubNet,pString+uOff,15);
		}

	if(uMode&0x02)										// Leerzeichen am Ende einfgen
		{
		pString[uOff] = ' '; uOff++;
		pString[uOff] = 0;
		}


return uOff;
}

//*****************************************************************************
//*
//*		ipv4tostrptr
//*
//*****************************************************************************
//	Wandelt eine IPV4-Adresse in eine Text um
//	pIpAddr		: Zeiger auf den 4-Byte-Speicher der IP-Adresse.
//	uSubNet		: Anzahl der Bits fr das Subnet
//	uMode		: Bestimmt die Art der Textverarbeitung:
//					Bit0 = Leerzeichen am Anfang einfgen
//					Bit1 = Leerzeichen am Ende einfgen
//					Bit2 = Sub-Net-Maske ".../?" anhngen.
//					Bit3 = Erstes Byte ist auf Offset 3
//					Bit5 = Port "...:?" anhngen.
//	Ergibt einen Zeiger auf den Text der IP-Adresse.
const char *_cdecl ipv4tostrptra(const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
char	  *pBuffer;

	uStrSlotA++;
	pBuffer	= cStrBufferA+STR_SLOT_SIZE*(uStrSlotA%STR_SLOT_COUNT);
	ipv4tostr(pBuffer,pIpAddr,uSubNet,uMode);


return pBuffer;
}

const wchar_t *_cdecl ipv4tostrptrw(const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
wchar_t	  *pBuffer;

	uStrSlotW++;
	pBuffer	= cStrBufferW+STR_SLOT_SIZE*(uStrSlotW%STR_SLOT_COUNT);
	ipv4tostr(pBuffer,pIpAddr,uSubNet,uMode);


return pBuffer;
}

//*****************************************************************************
//*
//*		ipv4tostrex
//*
//*****************************************************************************
//	Wandelt eine IPV4-Adresse in eine Text um und analysiert die Subnet-Maske.
//	pString		: Ist der Zeiger auf den Textpuffer fr die IP-Adresse.
//				  Der Puffer sollte mindestens 32 Byte gro sein.
//	pIpAddr		: Zeiger auf den 4-Byte-Speicher der IP-Adresse.
//	pSubNet		: Ist die Subnet-Maske als Speicher (z.B. "255.255.255.0")
//				  Hier werden die Null-Bits gezhlt.
//	uMode		: Bestimmt die Art der Textverarbeitung:
//					Bit0 = Leerzeichen am Anfang einfgen
//					Bit1 = Leerzeichen am Ende einfgen
//					Bit2 = Sub-Net-Maske ".../?" anhngen.
//					Bit3 = Erstes Byte ist auf Offset 3
//					Bit5 = Port "...:?" anhngen.
//	Ergibt die Lnge des erstellten Textes.
int _cdecl ipv4tostrex(char *pString,const void *pIpAddr,const void *pSubNet,unsigned uMode)
{
unsigned char  *pData;
unsigned	 	uBits;
unsigned		uMask;


	uMask = 0;

	if(pSubNet)
		{
		if(uMode&0x08)									// Erstes Byte ist auf Offset 3
			{
			uBits = *(int*)pSubNet;
			}
		else{
			pData  = (unsigned char*)pSubNet;
			uBits  = pData[0]<<24;
			uBits |= pData[1]<<16;
			uBits |= pData[2]<<8;
			uBits |= pData[3];
			}

		if(uBits)										// Zhle die Bits in der Sub-Net-Maske
			{
			while(!(uBits&1))
				{
				uBits>>=1;
				uMask++;
				}
			}
		}

return ipv4tostr(pString,pIpAddr,uMask,uMode);
}

int _cdecl ipv4tostrex(wchar_t *pString,const void *pIpAddr,const void *pSubNet,unsigned uMode)
{
unsigned char  *pData;
unsigned	 	uBits;
unsigned		uMask;


	uMask = 0;

	if(pSubNet)
		{
		if(uMode&0x08)									// Erstes Byte ist auf Offset 3
			{
			uBits = *(int*)pSubNet;
			}
		else{
			pData  = (unsigned char*)pSubNet;
			uBits  = pData[0]<<24;
			uBits |= pData[1]<<16;
			uBits |= pData[2]<<8;
			uBits |= pData[3];
			}

		if(uBits)										// Zhle die Bits in der Sub-Net-Maske
			{
			while(!(uBits&1))
				{
				uBits>>=1;
				uMask++;
				}
			}
		}

return ipv4tostr(pString,pIpAddr,uMask,uMode);
}

//*****************************************************************************
//*
//*		ipv6tostr
//*
//*****************************************************************************
//	Wandelt eine IPV6-Adresse in eine Text um
//	pString		: Ist der Zeiger auf den Textpuffer fr die IP-Adresse.
//				  Der Puffer sollte mindestens 64 Byte gro sein.
//	pIpAddr		: Zeiger auf den 16-Byte-Speicher der IP-Adresse.
//	uSubNet		: Anzahl der Bits fr das Subnet
//	uMode		: Bestimmt die Art der Textverarbeitung:
//					Bit0 = Leerzeichen am Anfang einfgen
//					Bit1 = Leerzeichen am Ende einfgen
//					Bit2 = Sub-Net-Maske ".../?" anhngen.
//					Bit4 = In Klammern einhllen "[....]"
//					Bit5 = Port "...:?" anhngen.
//	Ergibt die Lnge des erstellten Textes.
int _cdecl ipv6tostr(char *pString,const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
unsigned short *pWord=(unsigned short*)pIpAddr;
unsigned char  *pByte=(unsigned char *)pIpAddr;
unsigned	   *pLong=(unsigned      *)pIpAddr;
int				iPos;
int				iMax;
int				iOff;
int				iOut;
int				iLen;


	if(uMode&0x01)										// Leerzeichen am Anfang einfgen
		{
		pString[0]	= ' ';
		iOut		= 1;
		}
	else{
		iOut		= 0;
		}


	if(pLong[0]==0xD93B3BD9)							// Ist es eine IPV4
		{
		iOut  += sprintf(pString+iOut,"%i.%i.%i.%i",pByte[4],pByte[5],pByte[6],pByte[7]);
		uMode &= ~0x10;
		}
	else if(pLong[2]==0xFFFF0000 && !pLong[1] && !pLong[0])
		{
		iOut  += sprintf(pString+iOut,"%i.%i.%i.%i",pByte[12],pByte[13],pByte[14],pByte[15]);
		uMode &= ~0x10;
		}
	else{
		if(uMode&0x30)									// In Klammern einhllen "[....]"
			{
			uMode        |= 0x10;
			pString[iOut] = '[';
			iOut++;
			}

		iMax =  0;
		iLen =  0;
		iOff = -1;

		for(iPos=0;iPos<8;iPos++)						// Suche nach den gren Leerbereich
			{
			if(!pWord[iPos])
				{
				iLen++;
				continue;
				}

			if(iLen>iMax)
				{
				iOff=iPos-iLen;
				iMax=iLen;
				}

			iLen=0;
			}

		if(iLen>iMax)
			{
			iOff=iPos-iLen;
			iMax=iLen;
			iLen=0;
			};

		for(iPos=0;iPos<8;iPos++)						// Werte ausgeben
			{
			if(iPos==iOff)
				{
				if(iPos==0)
					{
					pString[iOut]=':';
					iOut++;
					}

				pString[iOut]=':';iOut++;
				iPos += iMax-1;
				continue;
				}

			if(pByte[iPos*2])
					iOut+=sprintf(pString+iOut,"%X%02X",pByte[iPos*2  ],pByte[iPos*2+1]);
			else	iOut+=sprintf(pString+iOut,"%X"    ,pByte[iPos*2+1]);

			if(iPos<7)
				{
				pString[iOut]=':';
				iOut++;
				}
			}

		pString[iOut]=0;
		}


	if(uSubNet && (uMode&0x24)==0x04)					// Subnet anhngen "../?"
		{
		pString[iOut] = '/';
		iOut += 1;
		iOut += itostr(uSubNet,pString+iOut,15);
		}

	if(uMode&0x10)										// In Klammern einhllen "[....]"
		{
		pString[iOut] = ']';
		iOut++;
		}

	if(uSubNet && (uMode&0x20))							// Port anhngen "..:?"
		{
		pString[iOut] = ':';
		iOut += 1;
		iOut += itostr(uSubNet&0xFFFF,pString+iOut,15);
		}

	if(uMode&0x02)										// Leerzeichen am Ende einfgen
		{
		pString[iOut] = ' '; iOut++;
		pString[iOut] = 0;
		}


return iOut;
}

int _cdecl ipv6tostr(wchar_t *pString,const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
unsigned short *pWord=(unsigned short*)pIpAddr;
unsigned char  *pByte=(unsigned char *)pIpAddr;
unsigned	   *pLong=(unsigned      *)pIpAddr;
int				iPos;
int				iMax;
int				iOff;
int				iOut;
int				iLen;


	if(uMode&0x01)										// Leerzeichen am Anfang einfgen
		{
		pString[0]	= ' ';
		iOut		= 1;
		}
	else{
		iOut		= 0;
		}


	if(pLong[0]==0xD93B3BD9)							// Ist es eine IPV4
		{
		iOut  += swprintf(pString+iOut,L"%i.%i.%i.%i",pByte[4],pByte[5],pByte[6],pByte[7]);
		uMode &= ~0x10;
		}
	else if(pLong[2]==0xFFFF0000 && !pLong[1] && !pLong[0])
		{
		iOut  += swprintf(pString+iOut,L"%i.%i.%i.%i",pByte[12],pByte[13],pByte[14],pByte[15]);
		uMode &= ~0x10;
		}
	else{
		if(uMode&0x30)									// In Klammern einhllen "[....]"
			{
			uMode        |= 0x10;
			pString[iOut] = '[';
			iOut++;
			}

		iMax =  0;
		iLen =  0;
		iOff = -1;

		for(iPos=0;iPos<8;iPos++)						// Suche nach den gren Leerbereich
			{
			if(!pWord[iPos])
				{
				iLen++;
				continue;
				}

			if(iLen>iMax)
				{
				iOff=iPos-iLen;
				iMax=iLen;
				}

			iLen=0;
			}

		if(iLen>iMax)
			{
			iOff=iPos-iLen;
			iMax=iLen;
			iLen=0;
			};

		for(iPos=0;iPos<8;iPos++)						// Werte ausgeben
			{
			if(iPos==iOff)
				{
				if(iPos==0)
					{
					pString[iOut]=':';
					iOut++;
					}

				pString[iOut]=':';iOut++;
				iPos += iMax-1;
				continue;
				}

			if(pByte[iPos*2])
					iOut+=swprintf(pString+iOut,L"%X%02X",pByte[iPos*2  ],pByte[iPos*2+1]);
			else	iOut+=swprintf(pString+iOut,L"%X"    ,pByte[iPos*2+1]);

			if(iPos<7)
				{
				pString[iOut]=':';
				iOut++;
				}
			}

		pString[iOut]=0;
		}


	if(uSubNet && (uMode&0x24)==0x04)					// Subnet anhngen "../?"
		{
		pString[iOut] = '/';
		iOut += 1;
		iOut += itostr(uSubNet,pString+iOut,15);
		}

	if(uMode&0x10)										// In Klammern einhllen "[....]"
		{
		pString[iOut] = ']';
		iOut++;
		}

	if(uSubNet && (uMode&0x20))							// Port anhngen "..:?"
		{
		pString[iOut] = ':';
		iOut += 1;
		iOut += itostr(uSubNet&0xFFFF,pString+iOut,15);
		}

	if(uMode&0x02)										// Leerzeichen am Ende einfgen
		{
		pString[iOut] = ' '; iOut++;
		pString[iOut] = 0;
		}


return iOut;
}

//*****************************************************************************
//*
//*		ipv6tostrptr
//*
//*****************************************************************************
//	Wandelt eine IPV6-Adresse in eine Text um
//	pIpAddr		: Zeiger auf den 16-Byte-Speicher der IP-Adresse.
//	uSubNet		: Anzahl der Bits fr das Subnet
//	uMode		: Bestimmt die Art der Textverarbeitung:
//					Bit0 = Leerzeichen am Anfang einfgen
//					Bit1 = Leerzeichen am Ende einfgen
//					Bit2 = Sub-Net-Maske ".../?" anhngen.
//					Bit4 = In Klammern einhllen "[....]"
//					Bit5 = Port "...:?" anhngen.
//	Ergibt einen Zeiger auf den Text der IP-Adresse.
const char *_cdecl ipv6tostrptra(const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
char	  *pBuffer;

	uStrSlotA++;
	pBuffer	= cStrBufferA+STR_SLOT_SIZE*(uStrSlotA%STR_SLOT_COUNT);
	ipv6tostr(pBuffer,pIpAddr,uSubNet,uMode);


return pBuffer;
}

const wchar_t *_cdecl ipv6tostrptrw(const void *pIpAddr,unsigned uSubNet,unsigned uMode)
{
wchar_t	  *pBuffer;

	uStrSlotW++;
	pBuffer	= cStrBufferW+STR_SLOT_SIZE*(uStrSlotW%STR_SLOT_COUNT);
	ipv6tostr(pBuffer,pIpAddr,uSubNet,uMode);


return pBuffer;
}


//*****************************************************************************
//*
//*		strfindattrib
//*
//*****************************************************************************
//	Sucht in einem HTML-Abschnitt nach einem Attribut (<.... <name>="xxxx">)
//	Es wird nur bis zum nchsten '>' Zeichen gesucht. Zwischen Gro- und
//	Kleinbuchstaben wird nicht unterschieden
//	pText		: Zeiger auf den HTML-Text beginnend bei "<..."
//	iMax		: Gre von HTML-Abschnitt in Zeichen (bis "...>" oder zum HTML Ende"
//	pName		: Name vom Attribut (bei <name>="...")
//	pLen		: Dort wird die Lnge des Attributs   gespeichert wenn !=0
//	pEnd		: Hier wird die Lnge des Abschnittes gespeichert wenn !=0
//	Ergibt die Position an der das Attribut gefunden wurde (nach <name>="... )
//			bzw. -1 wenn das Attribut nicht gefunden wurde.
int _cdecl strfindattrib(const char *pText,int iMax,const char *pName,int *pLen,int *pEnd)
{
char	cSign;
char	cChar;
int		iCnt;
int		iPos;
int		iNum;



	iCnt	= (int)strlen(pName);
	cSign	= pName[0];

	if(cSign>='A' && cSign<='Z' )
		{
		cSign += 0x20;
		}


	for(iPos=0;iPos<iMax;iPos++)						// Suche das Attribut
		{
		cChar = pText[iPos];

		if(cChar=='>')									// Ende des Abschnittes "...>"
			{
			iPos++;
			break;
			}

		if(strwchara(pText[iPos],0x0500))				// Text gefunden und bersperingen  (bei " oder ')
			{
			iPos += 1;
			iPos += strcbits(pText+iPos,aXmlStopTab,iMax-iPos);

			if(iPos<iMax && strwchara(pText[iPos],0x0500))
				{
				iPos++;
				}

			continue;
			}

		if(!strwchara(cChar,0x1F))						// Beginnt hier ein Attribut "<name>=..."
			{
			continue;
			}

		if(cChar>='A' && cChar<='Z' )
			{
			cChar+=0x20;
			}
														// Passt der Attributnamen
		if(cChar!=cSign || memicmp(pText+iPos,pName,iCnt) || iPos+iCnt>=iMax || pText[iPos+iCnt]!='=')
			{
			iPos += strwlen(pText+iPos,0x1F,iMax-iPos);
			if(iPos<iMax && pText[iPos]!='='){iPos--;continue;}
			iPos += strlenstr(pText+iPos+1,0,'"',iMax-iPos-1);
			continue;
			}

		iPos += iCnt+1;
		iNum  = 0;

		if(iPos>=iMax)									// Kein Platzt mehr
			{
			iNum = 0;
			}
		else if(!strwchara(pText[iPos],0x0500))			// Attribut ohne Anfhrungszeichen
			{
			iNum = strcbits(pText+iPos,aXmlSpaceTab,iMax-iPos);
			}
		else if(iPos+1>=iMax)							// Kein Platz fr "...
			{
			iPos++;
			iNum = 0;
			}
		else{											// Attribut mit Anfhrungszeichen
			cChar = pText[iPos];
			iPos++;
			iNum = strcbits(pText+iPos,aXmlStopTab,iMax-iPos);
			}

		if(pLen)
			{
			*pLen = iNum;
			}

		if(pEnd)										// Suche Ende des Abschnittes (bis ...>)
			{
			*pEnd = strlchar(pText+iPos+iNum,'>',iMax-iPos-iNum)+iNum+iPos;
			}

		return iPos;
		}


	if(pEnd)											// Ende des Abschnittes (bis ...>)
		{
		*pEnd = iPos;
		}

	if(pLen)
		{
		*pLen = 0;
		}


return -1;
}

int _cdecl strfindattrib(const wchar_t *pText,int iMax,const wchar_t *pName,int *pLen,int *pEnd)
{
wchar_t	cSign;
wchar_t	cChar;
int		iCnt;
int		iPos;
int		iNum;



	iCnt	= (int)wcslen(pName);
	cSign	= pName[0];

	if(cSign>='A' && cSign<='Z' )
		{
		cSign += 0x20;
		}


	for(iPos=0;iPos<iMax;iPos++)						// Suche das Attribut
		{
		cChar = pText[iPos];

		if(cChar=='>')									// Ende des Abschnittes "...>"
			{
			iPos++;
			break;
			}

		if(strwcharw(pText[iPos],0x0500))				// Text gefunden und bersperingen  (bei " oder ')
			{
			iPos += 1;
			iPos += strcbits(pText+iPos,aXmlStopTab,iMax-iPos);

			if(iPos<iMax && strwcharw(pText[iPos],0x0500))
				{
				iPos++;
				}

			continue;
			}

		if(!strwcharw(cChar,0x1F))						// Beginnt hier ein Attribut "<name>=..."
			{
			continue;
			}

		if(cChar>='A' && cChar<='Z' )
			{
			cChar+=0x20;
			}
														// Passt der Attributnamen
		if(cChar!=cSign || memicmp(pText+iPos,pName,iCnt) || iPos+iCnt>=iMax || pText[iPos+iCnt]!='=')
			{
			iPos += strwlen(pText+iPos,0x1F,iMax-iPos);
			if(iPos<iMax && pText[iPos]!='='){iPos--;continue;}
			iPos += strlenstr(pText+iPos+1,0,'"',iMax-iPos-1);
			continue;
			}

		iPos += iCnt+1;
		iNum  = 0;

		if(iPos>=iMax)									// Kein Platzt mehr
			{
			iNum = 0;
			}
		else if(!strwcharw(pText[iPos],0x0500))			// Attribut ohne Anfhrungszeichen
			{
			iNum = strcbits(pText+iPos,aXmlSpaceTab,iMax-iPos);
			}
		else if(iPos+1>=iMax)							// Kein Platz fr "...
			{
			iPos++;
			iNum = 0;
			}
		else{											// Attribut mit Anfhrungszeichen
			cChar = pText[iPos];
			iPos++;
			iNum = strcbits(pText+iPos,aXmlStopTab,iMax-iPos);
			}

		if(pLen)
			{
			*pLen = iNum;
			}

		if(pEnd)										// Suche Ende des Abschnittes (bis ...>)
			{
			*pEnd = strlchar(pText+iPos+iNum,'>',iMax-iPos-iNum)+iNum+iPos;
			}

		return iPos;
		}


	if(pEnd)											// Ende des Abschnittes (bis ...>)
		{
		*pEnd = iPos;
		}

	if(pLen)
		{
		*pLen = 0;
		}


return -1;
}

//*****************************************************************************
//*
//*		strgetattrib
//*
//*****************************************************************************
//	Holt aus einem Abschnitt ein Attribut (<.... <name>="xxxx">)
//	pText		: Zeiger auf den Begin eines Abschnittes ("<....")
//	iMax		: Gre vom Abschnitt
//	pBuffer		: Pufferspeicher fr das Attribut
//	iSize		: Die Gre vom Textpuffer fr das Attribut
//	pName		: Name vom Attribut (bei <name>="...")
//	Ergibt die Lnge des gespeicherten Attributs im Puffer, bzw. 0 wenn nichts gefunden wurde.
int	_cdecl strgetattrib(const char *pText,int iMax,char *pBuffer,int iSize,const char *pName)
{
int		iPos,iLen;


	   iPos = strfindattrib(pText,iMax,pName,&iLen);
	if(iPos<0)
		{
		if(iSize>0)*pBuffer=0;
		return 0;
		}


return strhtmldecode(pText+iPos,iLen,pBuffer,iSize,0x03);
}

int	_cdecl strgetattrib(const wchar_t *pText,int iMax,wchar_t *pBuffer,int iSize,const wchar_t *pName)
{
int		iPos,iLen;


	   iPos = strfindattrib(pText,iMax,pName,&iLen);
	if(iPos<0)
		{
		if(iSize>0)*pBuffer=0;
		return 0;
		}


return strhtmldecode(pText+iPos,iLen,pBuffer,iSize,0x03);
}

//*****************************************************************************
//*
//*		strfindtag
//*
//*****************************************************************************
//	Sucht einen HTML-Tag in einem Text. (<tag....> ... </tag>)
//	pText		: Zeiger auf den Beginn eines Abschnittes ("<....")
//	iMax		: Gre vom Abschnitt
//	pName		: Name vom Tag immer ohne Slash (z.B. "div" oder "a")
//	pLen		: Hier wird die Position nach dem ersten Tag-Abschnitt gespeichert.
//				  Dieser Zeiger kann auch NULL sein.
//	pEnd		: Hier wird die Position nach dem End-Tag-Abschnitt gespeichert.
//				  Dieser Zeiger kann auch NULL sein.
//	iFlags		: Bestimmt die Art der Suche
//					Bit  0..7	= Tag-Lnge (0=mit strlen() holen)
//					Bit  8		= Einen End-Tag suchen
//					Bit  9		= Bei Null abbrechen
//					Bit 10		= End-Tags rekursiv suchen
//					Bit 11		= Bei Level -1 beginnen (nur mit Bit 8)
//					Bit 12		= Den Beginn vom End-Tag in pEnd speichern
//					Bit 13		= Lnge vom End-Tag im untersten Byte von pEnd speichern
//	Ergibt die Position des Abschnittes bzw. -1 wenn nichts gefunden wurde
int _cdecl strfindtag(const char *pText,int iMax,const char *pName,int *pLen,int *pEnd,int iFlags)
{
int		iLevel;
int		iNull;
int		iType;
int		iMode;
int		iPos;
int		iOff;
int		iLen;
int		iEnd;



		iLen = iFlags&0xFF;
	if(!iLen)											// Lnge vom Tag holen
		{
			iLen = (int)strlen(pName);
		if(!iLen)return -1;
		}

	iLevel	=(iFlags&0x0800)? -1:0;
	iNull	= iFlags&0x0200;
	iMode	= iFlags&0x0500;
	iEnd	= iMax-iLen-1;


	for(iPos=0;iPos<iEnd;)								// Suche TAG mit Maske
		{
		if(pText[iPos]==0 && iNull)
			{
			iMax = iPos;
			break;
			}

		if(pText[iPos]!='<')							// Begin des Tags
			{
			iPos++;
			continue;
			}

		iOff	= iPos+1;
		iType	= 0x0000;

		if(pText[iOff]=='/')							// Ist es ein End-TAG
			{
			if(!iMode)
				{
				iPos++;
				continue;
				}

			iOff++;
			iType=0x0100;
			}

		if(iOff+iLen>=iMax)
			{
			break;
			}
														// TAG-Namen prfen
		if(strwchara(pText[iOff+iLen],0x07) || memicmp(pText+iOff,pName,iLen))
			{
			while(iPos<iEnd)							// Vor zum Ende des Tags
				{
				if(pText[iPos]==0 && iNull)
					{
					iMax = iPos;
					goto Exit;
					}

				if(pText[iPos]=='>')
					{
					iPos++;
					break;
					}

				if(strwchara(pText[iPos],0x0500))		// Textbeginn mit '"' oder '\''
					{
					iPos += 1;
					iPos += strcbits(pText+iPos,aXmlStopTab,iMax-iPos);

					if(iPos<iMax && strwchara(pText[iPos],0x0500))
						{
						iPos++;
						}
					
					continue;
					}

				iPos++;
				}

			continue;
			}

		if(!pLen && !pEnd)								// Suche ohne Extas gefunden
			{
			if(!((iType^iFlags)&0x0100))
				{
				return iPos;
				}
			}

		while(iOff<iMax)								// TAG-Ende suchen
			{
			if(pText[iOff]==0 && iNull)
				{
				if(pLen)*pLen=iOff;
				if(pEnd)*pEnd=(iFlags&0x2000)? iOff<<8:iOff;
				return iPos;
				}

			if(pText[iOff]=='>')
				{
				iOff++;
				break;
				}

			if(strwchara(pText[iPos],0x0500))			// Textbeginn mit '"' oder '\''
				{
				iPos += 1;
				iPos += strlbits(pText+iPos,aXmlStopTab,iMax-iPos);
				continue;
				}

			iOff++;
			}

		if(pLen)*pLen=iOff;


		if(!(iFlags&0x0100))							// Warte auf Begin-TAG
			{
			if(iType!=0)
				{
				iPos = iOff;
				continue;
				}
			}
		else if(!(iFlags&0x0400))						// Nicht Rekursiv suchen
			{
			if(iType==0)
				{
				iPos = iOff;
				continue;
				}
			}
		else{											// Rekursiv suchen
			if(iType==0)
				{
				if(pText[iOff-2]=='/')iLevel++;
				iPos = iOff;
				continue;
				}
			else if(iLevel>0)							// Bei End-TAG eine Ebene runter
				{
				iLevel--;
				iPos = iOff;
				continue;
				}
			}

		if(pEnd)										// Suche das End-TAG
			{
			if(iType==0 && pText[iOff-2]=='/')			// Endmarke ".../>" im Begin-TAG gefunden
				{
				*pEnd = (iFlags&0x2000)? iOff<<8:iOff;
				return iPos;
				}
			
			if(iFlags&0x0100)
				{
				*pEnd = iOff;
				}
			else{
				   iEnd = strfindtag(pText+iOff,iMax-iOff,pName,pEnd,NULL,iLen|(iFlags&0x0600)|0x0100);
				if(iEnd<0)iEnd=*pEnd;
				
				iLen = pEnd[0]-iEnd;
				
				if(iFlags&0x1000)						// Anfang vom End-TAG abspeichern
						pEnd[0]  = iEnd+iOff;
				else	pEnd[0] += iOff;
				
				if(iFlags&0x2000)						// Lnge vom End-TAG im ersten Byte
					{
					if(iLen>255)iLen=255;
					pEnd[0] <<= 8;
					pEnd[0]  |= iLen;	
					}
				}
			}

		return iPos;
		}

Exit:

	if(pLen)											// Lnge abspeichern
		{
		*pLen = iMax;
		}


return -1;
}

int _cdecl strfindtag(const wchar_t *pText,int iMax,const wchar_t *pName,int *pLen,int *pEnd,int iFlags)
{
int		iLevel;
int		iNull;
int		iType;
int		iMode;
int		iPos;
int		iOff;
int		iLen;
int		iEnd;



		iLen = iFlags&0xFF;
	if(!iLen)											// Lnge vom Tag holen
		{
			iLen = (int)wcslen(pName);
		if(!iLen)return -1;
		}

	iLevel	=(iFlags&0x0800)? -1:0;
	iNull	= iFlags&0x0200;
	iMode	= iFlags&0x0500;
	iEnd	= iMax-iLen-1;


	for(iPos=0;iPos<iEnd;)								// Suche TAG mit Maske
		{
		if(pText[iPos]==0 && iNull)
			{
			iMax = iPos;
			break;
			}

		if(pText[iPos]!='<')							// Begin des Tags
			{
			iPos++;
			continue;
			}

		iOff	= iPos+1;
		iType	= 0x0000;

		if(pText[iOff]=='/')							// Ist es ein End-TAG
			{
			if(!iMode)
				{
				iPos++;
				continue;
				}

			iOff++;
			iType=0x0100;
			}

		if(iOff+iLen>=iMax)
			{
			break;
			}
														// TAG-Namen prfen
		if(strwcharw(pText[iOff+iLen],0x07) || wcsnicmp(pText+iOff,pName,iLen))
			{
			while(iPos<iEnd)							// Vor zum Ende des Tags
				{
				if(pText[iPos]==0 && iNull)
					{
					iMax = iPos;
					goto Exit;
					}

				if(pText[iPos]=='>')
					{
					iPos++;
					break;
					}

				if(strwcharw(pText[iPos],0x0500))		// Textbeginn mit '"' oder '\''
					{
					iPos += 1;
					iPos += strcbits(pText+iPos,aXmlStopTab,iMax-iPos);

					if(iPos<iMax && strwcharw(pText[iPos],0x0500))
						{
						iPos++;
						}

					continue;
					}

				iPos++;
				}

			continue;
			}

		if(!pLen && !pEnd)								// Suche ohne Extas gefunden
			{
			if(!((iType^iFlags)&0x0100))
				{
				return iPos;
				}
			}

		while(iOff<iMax)								// TAG-Ende suchen
			{
			if(pText[iOff]==0 && iNull)
				{
				if(pLen)*pLen=iOff;
				if(pEnd)*pEnd=(iFlags&0x2000)? iOff<<8:iOff;
				return iPos;
				}

			if(pText[iOff]=='>')
				{
				iOff++;
				break;
				}

			if(strwcharw(pText[iPos],0x0500))			// Textbeginn mit '"' oder '\''
				{
				iPos += 1;
				iPos += strlbits(pText+iPos,aXmlStopTab,iMax-iPos);
				continue;
				}

			iOff++;
			}

		if(pLen)*pLen=iOff;


		if(!(iFlags&0x0100))							// Warte auf Begin-TAG
			{
			if(iType!=0)
				{
				iPos = iOff;
				continue;
				}
			}
		else if(!(iFlags&0x0400))						// Nicht Rekursiv suchen
			{
			if(iType==0)
				{
				iPos = iOff;
				continue;
				}
			}
		else{											// Rekursiv suchen
			if(iType==0)
				{
				if(pText[iOff-2]=='/')iLevel++;
				iPos = iOff;
				continue;
				}
			else if(iLevel>0)							// Bei End-TAG eine Ebene runter
				{
				iLevel--;
				iPos = iOff;
				continue;
				}
			}

		if(pEnd)										// Suche das End-TAG
			{
			if(iType==0 && pText[iOff-2]=='/')			// Endmarke ".../>" im Begin-TAG gefunden
				{
				*pEnd = (iFlags&0x2000)? iOff<<8:iOff;
				return iPos;
				}
			
			if(iFlags&0x0100)
				{
				*pEnd = iOff;
				}
			else{
				   iEnd=strfindtag(pText+iOff,iMax-iOff,pName,pEnd,NULL,iLen|(iFlags&0x0600)|0x0100);
				if(iEnd<0)iEnd=*pEnd;
				
				iLen = pEnd[0]-iEnd;
				
				if(iFlags&0x1000)						// Anfang vom End-TAG abspeichern
						pEnd[0]  = iEnd+iOff;
				else	pEnd[0] += iOff;
				
				if(iFlags&0x2000)						// Lnge vom End-TAG im ersten Byte
					{
					if(iLen>255)iLen=255;
					pEnd[0] <<= 8;
					pEnd[0]  |= iLen;	
					}
				}
			}

		return iPos;
		}

Exit:

	if(pLen)											// Lnge abspeichern
		{
		*pLen = iMax;
		}


return -1;
}

#ifdef 	_DEBUG

int _cdecl strfindtag(const char *pText,const char *pName,int *pLen,int *pEnd,int iFlags)
	{
	return strfindtag(pText,0x10000000,pName,pLen,pEnd,iFlags|0x0200);
	}

int _cdecl strfindtag(const wchar_t *pText,const wchar_t *pName,int *pLen,int *pEnd,int iFlags)
	{
	return strfindtag(pText,0x10000000,pName,pLen,pEnd,iFlags|0x0200);
	}

#endif



//*****************************************************************************
//*
//*		strtaglen
//*
//*****************************************************************************
//	Berechnet die Lnge eines HTML-Tags in einem Text. (<tag....>)
//	iMax		: Gre vom Abschnitt
//	pText		: Zeiger auf den Beginn eines Abschnittes ("<....")
//	iFlags		: Bestimmt die Art der Suche
//					Bit  0		= Das '<' Zeichen am Anfang prfen
//					Bit  9		= Bei Null abbrechen
//	Ergibt die Position des Abschnittes bzw. -1 wenn nichts gefunden wurde
int _cdecl strtaglen(int iMax,const char *pText,int iFlags)
{
int		iNull;
int		iPos;


	if(iFlags&0x0001)									// Das '<' Zeichen am Anfang prfen
		{
		if(iMax<=0      )return 0;
		if(pText[0]!='<')return 0;
		}


	iNull = iFlags&0x0200;


	for(iPos=0;iPos<iMax;iPos++)						// TAG-Ende suchen
		{
		if(pText[iPos]=='0' && iNull)
			{
			break;
			}

		if(pText[iPos]=='>')
			{
			iPos++;
			break;
			}

		if(pText[iPos]=='"')							// Text zwischen Anfhrungszeichen berspringen
			{
			iPos += 1;
			iPos += strlchar(pText+iPos,'"',iMax-iPos);
			continue;
			}

		iPos++;
		}



return iPos;
}

int _cdecl strtaglen(int iMax,const wchar_t *pText,int iFlags)
{
int		iNull;
int		iPos;


	if(iFlags&0x0001)									// Das '<' Zeichen am Anfang prfen
		{
		if(iMax<=0      )return 0;
		if(pText[0]!='<')return 0;
		}


	iNull = iFlags&0x0200;


	for(iPos=0;iPos<iMax;iPos++)						// TAG-Ende suchen
		{
		if(pText[iPos]=='0' && iNull)
			{
			break;
			}

		if(pText[iPos]=='>')
			{
			iPos++;
			break;
			}

		if(pText[iPos]=='"')							// Text zwischen Anfhrungszeichen berspringen
			{
			iPos += 1;
			iPos += strlchar(pText+iPos,'"',iMax-iPos);
			continue;
			}

		iPos++;
		}



return iPos;
}


#ifdef 	_DEBUG

int _cdecl strtaglen(const char *pText,int iFlags)
	{
	return strtaglen(0x10000000,pText,iFlags|0x0200);
	}

int _cdecl strtaglen(const wchar_t *pText,int iFlags)
	{
	return strtaglen(0x10000000,pText,iFlags|0x0200);
	}

#endif


